package com.yihu.jw.application.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.application.dao.BaseApplicationMenuDao;
import com.yihu.jw.application.dao.BaseApplicationRoleMenuDao;
import com.yihu.jw.contant.CommonContant;
import com.yihu.jw.entity.base.application.BaseApplicationMenuDO;
import com.yihu.jw.entity.base.user.UserDO;
import com.yihu.jw.entity.hospital.consult.WlyyHospitalSysDictDO;
import com.yihu.jw.hospital.dict.WlyyHospitalSysDictDao;
import com.yihu.jw.mysql.query.BaseJpaService;
import com.yihu.jw.restmodel.base.role.MenuVO;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.user.UserDao;
import com.yihu.jw.utils.hibernate.HibenateUtils;
import org.apache.commons.lang3.StringUtils;
import org.jsoup.Connection;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.criteria.CriteriaBuilder;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 应用菜单资源配置
 */
@Service
@Transactional
public class BaseApplicationMenuService extends BaseJpaService<BaseApplicationMenuDO, BaseApplicationMenuDao> {

    @Autowired
    private BaseApplicationMenuDao applicationMenuDao;
    @Autowired
    private HibenateUtils hibenateUtils;

    @Autowired
    private WlyyHospitalSysDictDao hospitalSysDictDao;

    @Autowired
    private UserDao userDao;
    @Value("${wechat.id}")
    private String wechatId;

    /**
     * 创建和修改应用菜单实体
     *
     * @param menuDO
     * @return
     */
    public BaseApplicationMenuDO insertAndUpdateMenu(BaseApplicationMenuDO menuDO,String uuid){
        UserDO userDO = userDao.findById(uuid).get();
        if (StringUtils.isNoneBlank(menuDO.getId())){
            menuDO.setUpdateTime(new Date());
            menuDO.setUpdateUser(uuid);
            menuDO.setUpdateUserName(userDO.getName());
        }else {
            menuDO.setCreateTime(new Date());
            menuDO.setCreateUser(uuid);
            menuDO.setCreateUserName(userDO.getName());
            menuDO.setUpdateUser(uuid);
            menuDO.setUpdateTime(new Date());
            menuDO.setUpdateUserName(userDO.getName());
        }
        return applicationMenuDao.save(menuDO);
    }

    /**
     * 根据id删除菜单实体
     * @param ids
     */
    public void deleteById(String ids){
        String str[]= ids.split(",");
        for (int i=0;i<str.length;i++){
            applicationMenuDao.deleteById(str[i]);
        }
    }


    /**
     * 根据id获取菜单资源详情
     * @param id
     * @return
     */
    public BaseApplicationMenuDO findById(String id){
        return applicationMenuDao.findById(id).get();
    }


    /**
     * 获取资源菜单树型结构
     * @param type
     * @return
     */
    public List<BaseApplicationMenuDO> getMenuTree(String type,String name){
        List<BaseApplicationMenuDO> baseApplicationMenuDOS = new ArrayList<>();
        if (StringUtils.isNoneBlank(name)){
            baseApplicationMenuDOS = applicationMenuDao.getTypeAndNameList(type,name);
        }else {
            baseApplicationMenuDOS = applicationMenuDao.getParentList(type);
        }
        Map<String,List<BaseApplicationMenuDO>> map = baseApplicationMenuDOS.stream().collect(Collectors.groupingBy(BaseApplicationMenuDO::getParentId));
        baseApplicationMenuDOS.forEach(menu->{
            List<BaseApplicationMenuDO> tmp = map.get(menu.getId());
            menu.setChildren(tmp);
        });
        List<BaseApplicationMenuDO> applicationMenuDOS = new ArrayList<>();
        for (BaseApplicationMenuDO menuDO:baseApplicationMenuDOS){
            if (menuDO.getParentId().equalsIgnoreCase("0")){
                applicationMenuDOS.add(menuDO);
            }
        }
        return applicationMenuDOS;
    }


    /**
     * 查询资源菜单列表
     * @param type
     * @param saasId
     * @param orgCode
     * @param name
     * @param status
     * @param page
     * @param size
     * @return
     */
    public MixEnvelop selectByCondition(String type,String saasId, String orgCode, String name, Integer status, Integer page, Integer size){
        MixEnvelop mixEnvelop = new MixEnvelop();
        String conditionSql = "";
        if ("xm_ykyy_wx".equals(wechatId)) {
            conditionSql = "to_char(m.update_time,'YYYY-MM-DD hh24:mi:ss')  AS \"updateTime\",";
        } else {
            conditionSql = "date_format(m.update_time,'%Y-%m-%d %H:%i:%S' )  AS \"updateTime\",";
        }
        String sql = "SELECT\n" +
                "\tm.id,\n" +
                "\tm. CODE AS \"code\",\n" +
                "\tm. NAME AS \"name\",\n" +
                "\tm.saas_id AS \"saasId\",\n" +
                "\ts. NAME AS \"saasName\",\n" +
                "\tm.org_code AS \"orgCode\",\n" +
                "\to.name AS \"orgName\",\n" +
                "\tm.identify AS \"identify\",\n" +
                "\tm.parent_id AS \"parentId\",\n" +
                "\tm.menu_sort AS \"menuSort\",\n" +
                "\tm.type AS \"type\",\n" +
                "\tm. STATUS AS \"status\",\n" +
                "\tm.icon AS \"icon\",\n" +
                "\tm.url AS \"url\",\n" +
                "\tm.url_param AS \"urlParam\",\n" +
                "\tm.description AS \"description\",\n" +
                conditionSql +
                "\tm.update_user_name AS \"updateUserName\"\n" +

                "FROM\n" +
                "\tbase_application_menu m\n" +
                "LEFT JOIN base_saas s ON s.id = m.saas_id\n" +
                "LEFT JOIN base_org o ON m.org_code = o. CODE ";

        String sqlTotal = " SELECT\n" +
                "\tCOUNT(1) as total\n" +
                "FROM\n" +
                "\tbase_application_menu m\n" +
                "LEFT JOIN base_saas s ON s.id = m.saas_id\n" +
                "LEFT JOIN base_org o ON m.org_code = o. CODE ";
        String condition = " where 1=1 ";
        if (StringUtils.isNoneBlank(type)){
            condition +=" and m.type ='"+type+"' ";
        }
        if (StringUtils.isNoneBlank(saasId)){
            condition +=" and m.saas_id ='"+saasId+"' ";
        }
        if (StringUtils.isNoneBlank(orgCode)){
            condition +=" and m.org_code ='"+orgCode+"' ";
        }
        if (StringUtils.isNoneBlank(name)){
            condition +=" and m.name like '%"+name+"%' ";
        }
        if (status!=null){
            condition +=" and m.status = '"+status+"' ";
        }
        String orderCondition = " order by m.create_time desc ";

        List<Map<String,Object>> list = hibenateUtils.createSQLQuery(sql+condition+orderCondition,page,size);
        WlyyHospitalSysDictDO hospitalSysDictDO = hospitalSysDictDao.findById("V_APPLICATION_ROLE_DICT").get();
        JSONObject object = new JSONObject();
        if (hospitalSysDictDO!=null){
            object = JSONObject.parseObject(hospitalSysDictDO.getDictValue());
        }
        WlyyHospitalSysDictDO hospitalSysDictDO1 = hospitalSysDictDao.findById("V_APPLICATION_IDENTIFY").get();
        JSONObject jsonObject = new JSONObject();
        if (hospitalSysDictDO1!=null){
            jsonObject = JSONObject.parseObject(hospitalSysDictDO1.getDictValue());
        }
        if (hospitalSysDictDO!=null)
            for (Map<String,Object> map:list){
                String typeId = map.get("type").toString();
                String typeName= object.getString(typeId);
                map.put("typeName",typeName);
                String identify = map.get("identify").toString();
                String identifyName = jsonObject.getString(identify);
                map.put("identifyName",identifyName);
            }
        Long count = 0L;
        List<Map<String,Object>> total = hibenateUtils.createSQLQuery(sqlTotal+condition);
        if(total!=null){
            //mysql 与 Oracle 聚合函数返回类型不一致，需要判断装换
            count = hibenateUtils.objTransformLong(total.get(0).get("total"));
        }
        mixEnvelop.setTotalCount(count.intValue());
        mixEnvelop.setDetailModelList(list);
        mixEnvelop.setPageSize(size);
        mixEnvelop.setCurrPage(page);
        return mixEnvelop;
    }
}
