package com.yihu.jw.dict.service;

import com.yihu.jw.dict.dao.BaseCopywritingDao;
import com.yihu.jw.entity.base.dict.BaseCopywritingDO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * Created by yeshijie on 2022/11/28.
 */
@Service
public class BaseCopywritingService {

    @Autowired
    private BaseCopywritingDao copywritingDao;
    @Autowired
    private StringRedisTemplate redisTemplate;

    public BaseCopywritingDO create(BaseCopywritingDO copywritingDO){
        copywritingDO.setCreateTime(new Date());
        return copywritingDao.save(copywritingDO);
    }

    public BaseCopywritingDO findByModularAndScene(String modular,String scene){
        return copywritingDao.findByModularAndScene(modular, scene);
    }

    /**
     * 获取文案-缓存
     * @return
     */
    public String findCopywriting(String modular,String scene){
        String key = "copywriting:"+modular+":"+scene;

        String content = redisTemplate.opsForValue().get(key);
        if(StringUtils.isBlank(content)){
            content = findCopywritingNoRedis(modular, scene);
            if(StringUtils.isNotBlank(content)){
                redisTemplate.opsForValue().set(key,content);
            }
        }
        return content;
    }

    /**
     * 获取文案-无缓存
     * @return
     */
    public String findCopywritingNoRedis(String modular,String scene){
        BaseCopywritingDO copywritingDO = copywritingDao.findByModularAndScene(modular, scene);
        if(copywritingDO!=null){
            return copywritingDO.getContent();
        }
        return null;
    }

    /**
     * 把文案设置到缓存
     */
    public String setCopywritingToRedis(BaseCopywritingDO copywritingDO,String modular,String scene){
        String content = null;
        if(copywritingDO==null){
            copywritingDO = copywritingDao.findByModularAndScene(modular, scene);
        }
        if(copywritingDO!=null){
            String key = "copywriting:"+copywritingDO.getModular()+":"+copywritingDO.getScene();
            content = copywritingDO.getContent();
            redisTemplate.opsForValue().set(key,content);
        }
        return content;
    }
}
