package com.yihu.jw.healthUpload.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.dict.dao.DictHospitalDeptDao;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.doctor.dao.BaseDoctorHospitalDao;
import com.yihu.jw.doctor.dao.BaseDoctorRoleDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.doctor.BaseDoctorHospitalDO;
import com.yihu.jw.entity.base.doctor.BaseDoctorRoleDO;
import com.yihu.jw.entity.healthUpload.BaseDoctorHealthUploadDO;
import com.yihu.jw.entity.healthUpload.BaseDoctorMemberDO;
import com.yihu.jw.entity.hospital.consult.WlyyHospitalSysDictDO;
import com.yihu.jw.healthUpload.dao.BaseDoctorHealthUploadDao;
import com.yihu.jw.healthUpload.dao.BaseDoctorMemberDao;
import com.yihu.jw.hospital.dict.WlyyHospitalSysDictDao;
import com.yihu.jw.message.service.WxMessageUtil;
import com.yihu.jw.mysql.query.BaseJpaService;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.util.common.PwdUtil;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.idcard.IdCardUtil;
import com.yihu.jw.utils.hibernate.HibenateUtils;
import jxl.write.*;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.transaction.Transactional;
import java.io.IOException;
import java.io.OutputStream;
import java.util.*;

@Service
@Transactional
public class BaseDoctorHealthUploadService extends BaseJpaService<BaseDoctorHealthUploadDO, BaseDoctorHealthUploadDao> {
    private static Logger logger = LoggerFactory.getLogger(BaseDoctorHealthUploadService.class);
    @Autowired
    private BaseDoctorMemberDao doctorMemberDao;
    @Autowired
    private BaseDoctorDao baseDoctorDao;
    @Autowired
    private BaseDoctorHealthUploadDao baseDoctorHealthUploadDao;
    @Autowired
    private BaseDoctorHospitalDao baseDoctorHospitalDao;
    @Autowired
    private DictHospitalDeptDao dictHospitalDeptDao;
    @Autowired
    private BaseDoctorRoleDao doctorRoleDao;
    @Autowired
    private HibenateUtils hibenateUtils;
    @Value("${wechat.id}")
    private String wechatId;
    @Autowired
    private WlyyHospitalSysDictDao hospitalSysDictDao;
    @Resource
    private WxMessageUtil wxMessageUtil;


    /**
     * 新增医生成员关系
     *
     * @param doctorId
     * @param ids      逗号隔开
     * @return
     */
    public List<BaseDoctorMemberDO> saveAndCreateMember(String doctorId, String ids) {
        String str[] = ids.split(",");
        List<BaseDoctorMemberDO> baseDoctorMemberDOList = new ArrayList<>();
        for (int i = 0; i < str.length; i++) {
            String memberId = str[i];
            BaseDoctorMemberDO memberDO = doctorMemberDao.findByDoctorIdAndMemberIdAndDel(doctorId, memberId);
            if (memberDO == null) {
                BaseDoctorMemberDO baseDoctorMemberDO = new BaseDoctorMemberDO();
                baseDoctorMemberDO.setDel(1);
                baseDoctorMemberDO.setDoctorId(doctorId);
                baseDoctorMemberDO.setMemberId(memberId);
                baseDoctorMemberDO.setCreateTime(new Date());
                baseDoctorMemberDO.setUpdateTime(new Date());
                baseDoctorMemberDO = doctorMemberDao.save(baseDoctorMemberDO);
                baseDoctorMemberDOList.add(baseDoctorMemberDO);
            }
        }
        return baseDoctorMemberDOList;
    }

    /**
     * 查询医生的成员列表
     *
     * @param doctorId
     * @return
     */
    public List<BaseDoctorMemberDO> selectByDoctorId(String doctorId) {
        List<BaseDoctorMemberDO> baseDoctorMemberDOList = doctorMemberDao.findByDoctorIdAndDel(doctorId);
        BaseDoctorMemberDO memberDO1 = doctorMemberDao.findByDoctorIdAndMemberIdAndDel(doctorId, doctorId);
        //添加本人
        if (memberDO1 == null) {
            BaseDoctorMemberDO baseDoctorMemberDO = new BaseDoctorMemberDO();
            baseDoctorMemberDO.setDel(1);
            baseDoctorMemberDO.setDoctorId(doctorId);
            baseDoctorMemberDO.setMemberId(doctorId);
            baseDoctorMemberDO.setCreateTime(new Date());
            baseDoctorMemberDO.setUpdateTime(new Date());
            baseDoctorMemberDO = doctorMemberDao.save(baseDoctorMemberDO);
        }
        for (BaseDoctorMemberDO memberDO : baseDoctorMemberDOList) {
            BaseDoctorDO baseDoctorDO = baseDoctorDao.findByIdAndDel(memberDO.getMemberId());
            if (baseDoctorDO != null) {
                memberDO.setBaseDoctorDO(baseDoctorDO);
            }
        }
        return baseDoctorMemberDOList;
    }

    /**
     * 查询医生的成员列表带有已上报次数
     *
     * @param doctorId
     * @return
     */
    public List<BaseDoctorMemberDO> selectCountByDoctorId(String doctorId) {
        List<BaseDoctorMemberDO> baseDoctorMemberDOList = doctorMemberDao.findByDoctorIdAndDel(doctorId);
        for (BaseDoctorMemberDO memberDO : baseDoctorMemberDOList) {
            BaseDoctorDO baseDoctorDO = baseDoctorDao.findByIdAndDel(memberDO.getMemberId());
            if (baseDoctorDO != null) {
                memberDO.setBaseDoctorDO(baseDoctorDO);
            }
            List<BaseDoctorHealthUploadDO> baseDoctorHealthUploadDOList = baseDoctorHealthUploadDao.findByDoctorIdAndConsumer(memberDO.getMemberId(), doctorId);
            memberDO.setCount(baseDoctorHealthUploadDOList.size() + "");
        }
        return baseDoctorMemberDOList;
    }


    /**
     * 查询某一个成员id
     *
     * @param id 成员id
     * @return
     */
    public BaseDoctorMemberDO selectById(String id) {
        BaseDoctorMemberDO baseDoctorMemberDO = doctorMemberDao.findByIdAndDel(id);
        BaseDoctorDO baseDoctorDO = baseDoctorDao.findByIdAndDel(baseDoctorMemberDO.getMemberId());
        if (baseDoctorDO != null) {
            baseDoctorMemberDO.setBaseDoctorDO(baseDoctorDO);
        }
        List<BaseDoctorHospitalDO> doctorHospitalDOS = baseDoctorHospitalDao.findByDoctorCode(baseDoctorMemberDO.getMemberId());
        baseDoctorMemberDO.setDoctorHospitalDOS(doctorHospitalDOS);
        return baseDoctorMemberDO;
    }

    /**
     * 删除某个成员
     * @param id 成员id
     * @return
     */
    public BaseDoctorMemberDO delDoctorMember(String id) {
        BaseDoctorMemberDO baseDoctorMemberDO = doctorMemberDao.findByIdAndDel(id);
        baseDoctorMemberDO.setDel(0);
        baseDoctorMemberDO = doctorMemberDao.save(baseDoctorMemberDO);
        return baseDoctorMemberDO;
    }

    /**
     * 提交健康信息
     * @param baseDoctorHealthUploadDO 健康信息实体类
     * @return
     */
    public BaseDoctorHealthUploadDO createHealthUploadInfo(BaseDoctorHealthUploadDO baseDoctorHealthUploadDO) {
        baseDoctorHealthUploadDO.setCreateTime(new Date());
        baseDoctorHealthUploadDO = baseDoctorHealthUploadDao.save(baseDoctorHealthUploadDO);

        List<BaseDoctorHospitalDO> doctorHospitalDOS = baseDoctorHospitalDao.findByDoctorCode(baseDoctorHealthUploadDO.getDoctorId());
        try {
            if (baseDoctorHealthUploadDO.getHealthCode() == 2 || baseDoctorHealthUploadDO.getHealthCode() == 3 || Double.parseDouble(baseDoctorHealthUploadDO.getTemperature()) >= 37.3
                    || baseDoctorHealthUploadDO.getIsSymptoms() == 1 || baseDoctorHealthUploadDO.getIsEpidemic() == 1) {
                List<BaseDoctorRoleDO> roleDOS = doctorRoleDao.findByRoleCode("admin");
                for (BaseDoctorRoleDO roleDO : roleDOS) {
                    BaseDoctorDO doctorDO = baseDoctorDao.findByIdAndDel(roleDO.getDoctorCode());
                    if (doctorDO != null) {
                        WlyyHospitalSysDictDO hospitalSysDictDO  = hospitalSysDictDao.findById("isYichangRemind").orElse(null);
                        if (hospitalSysDictDO!=null){
                            String value = hospitalSysDictDO.getDictValue();
                            if (StringUtils.isNoneBlank(value)){
                                JSONObject object = JSONObject.parseObject(value);
                                String close = object.getString("close");
                                String time = object.getString("time");
                                String count =object.getString("count");
                                if (!close.equalsIgnoreCase("1")){
                                    String startEnd = DateUtil.getPreTime(DateUtil.getStringDate(),time);
                                    String sql = "select count(1) as total from wx_push_log where create_time >='"+startEnd+"' ";
                                    Map<String,Object> map = jdbcTemplate.queryForMap(sql);
                                    Integer total = 0;

                                    if (map!=null){
                                        if (map.get("total")!=null){
                                            total = Integer.parseInt(map.get("total").toString());
                                        }
                                    }
                                    logger.info("startEnd:"+startEnd+total);
                                    if (StringUtils.isNoneBlank(count)){
                                        if (total<Integer.parseInt(count)){
                                            wxMessageUtil.sendHealthUploadWxTemplage(doctorDO,doctorHospitalDOS,baseDoctorHealthUploadDO);
                                        }
                                    }
                                }
                            }
                        }else {
                            wxMessageUtil.sendHealthUploadWxTemplage(doctorDO,doctorHospitalDOS,baseDoctorHealthUploadDO);
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return baseDoctorHealthUploadDO;
    }

    /**
     * 查询成员的健康上报记录
     *
     * @param doctorId 医生id
     * @return
     */
    public MixEnvelop selectHealthUploadInfoByDoctorId(String doctorId, String startDate, String endDate, Integer page, Integer pageSize) {
        PageRequest pageRequest = PageRequest.of(page - 1, pageSize);
        MixEnvelop mixEnvelop = new MixEnvelop();
        List<BaseDoctorHealthUploadDO> list = new ArrayList<>();
        if (StringUtils.isNoneBlank(startDate)) {
            Date startTime = DateUtil.strToDateLong(startDate);
            Date endTime = DateUtil.strToDateLong(endDate);
            list = baseDoctorHealthUploadDao.selectHealthUploadInfoByDoctorIdAndDate(doctorId, startTime, endTime, pageRequest);
        } else {
            list = baseDoctorHealthUploadDao.selectHealthUploadInfoByDoctorId(doctorId, pageRequest);
        }
        mixEnvelop.setCurrPage(page);
        mixEnvelop.setPageSize(pageSize);
        mixEnvelop.setDetailModelList(list);
        mixEnvelop.setTotalCount(list == null ? 0 : list.size());
        return mixEnvelop;
    }

    public BaseDoctorHealthUploadDO selectHealthInfoById(String id) {
        BaseDoctorHealthUploadDO baseDoctorHealthUploadDO = baseDoctorHealthUploadDao.findById(id).orElse(null);
        return baseDoctorHealthUploadDO;
    }

    /**
     * 新增加医生信息
     *
     * @param photo
     * @param name
     * @param idcard
     * @param sex
     * @param mobile
     * @param dept
     * @param deptName
     * @param idCardType
     * @param idType
     * @return
     */
    public BaseDoctorDO saveDoctorInfo(String id, String photo, String name, String idcard, String sex, String mobile, String dept, String deptName, String idCardType, String idType, Integer mark) throws Exception {
        BaseDoctorDO baseDoctorDO = new BaseDoctorDO();
        if (StringUtils.isNoneBlank(id)) {
            baseDoctorDO = baseDoctorDao.findByIdAndDel(id);
            baseDoctorDO.setIdCardType(idCardType);
            baseDoctorDO.setIdType(idType);
            baseDoctorDO.setPhoto(photo);
            baseDoctorDO.setName(name);
            baseDoctorDO.setIdcard(idcard);
            baseDoctorDO.setSex(Integer.parseInt(sex));
            baseDoctorDO.setMobile(mobile);
            baseDoctorDO.setMark(mark);
            String deptCode = dept;
            baseDoctorDO.setChargeType("1");
            baseDoctorDO = baseDoctorDao.save(baseDoctorDO);
            List<BaseDoctorHospitalDO> baseDoctorHospitalDOS = baseDoctorHospitalDao.findByDoctorCode(baseDoctorDO.getId());
            if (null != baseDoctorHospitalDOS && baseDoctorHospitalDOS.size() > 0) {
                baseDoctorHospitalDao.deleteAll(baseDoctorHospitalDOS);
                BaseDoctorHospitalDO baseDoctorHospitalDO = new BaseDoctorHospitalDO();
                baseDoctorHospitalDO.setOrgCode("350211A1002");
                baseDoctorHospitalDO.setOrgName("马銮湾医院");
                baseDoctorHospitalDO.setDoctorCode(baseDoctorDO.getId());
                baseDoctorHospitalDO.setDeptCode(deptCode);
                baseDoctorHospitalDO.setDeptName(deptName);
                baseDoctorHospitalDO.setDel("1");
                baseDoctorHospitalDO.setCreateTime(new Date());
                baseDoctorHospitalDao.save(baseDoctorHospitalDO);
            } else {
                BaseDoctorHospitalDO baseDoctorHospitalDO = new BaseDoctorHospitalDO();
                baseDoctorHospitalDO.setOrgCode("350211A1002");
                baseDoctorHospitalDO.setOrgName("马銮湾医院");
                baseDoctorHospitalDO.setDoctorCode(baseDoctorDO.getId());
                baseDoctorHospitalDO.setDeptCode(deptCode);
                baseDoctorHospitalDO.setDeptName(deptName);
                baseDoctorHospitalDO.setDel("1");
                baseDoctorHospitalDO.setCreateTime(new Date());
                baseDoctorHospitalDao.save(baseDoctorHospitalDO);
            }
        } else {
            baseDoctorDO.setIdcard(idcard);
            baseDoctorDO.setBirthday(IdCardUtil.getBirthdayForIdcard(idcard));
            baseDoctorDO.setSex(Integer.parseInt(sex));
            baseDoctorDO.setLocked(0);
            baseDoctorDO.setEnabled(1);
            baseDoctorDO.setMark(mark);
            String disableFlag = "0";
            //互联网医院：1停用，0使用  转 i健康：1正常，0作废
            baseDoctorDO.setDel("1".equals(disableFlag) ? "0" : "1");
            baseDoctorDO.setMobile("0");
            baseDoctorDO.setName(name);
            baseDoctorDO.setIsFamous(0);
            baseDoctorDO.setCreateTime(new Date());
            baseDoctorDO.setUpdateTime(new Date());
            baseDoctorDO.setIdCardType(idCardType);
            baseDoctorDO.setIdType(idType);
            baseDoctorDO.setPhoto(photo);
            baseDoctorDO.setMobile(mobile);
            baseDoctorDO.setChargeType("1");

            if (StringUtils.isNotBlank(mobile)) {
                baseDoctorDO.setSalt(PwdUtil.randomString(5));
                try {
                    baseDoctorDO.setPassword(com.yihu.jw.util.encrypt.MD5.md5Hex(baseDoctorDO.getMobile().substring(baseDoctorDO.getMobile().length() - 6) + "{" + baseDoctorDO.getSalt() + "}"));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else {
                baseDoctorDO.setPassword(com.yihu.jw.util.encrypt.MD5.md5Hex("123456" + "{" + baseDoctorDO.getSalt() + "}"));
            }
            // 用医生和机构id、部门判断数据是否存在，若不存在则保存医生机构关联关系
            String deptCode = dept;
            baseDoctorDO = baseDoctorDao.save(baseDoctorDO);
            List<BaseDoctorHospitalDO> baseDoctorHospitalDOS = baseDoctorHospitalDao.findByDoctorCode(baseDoctorDO.getId());
            if (null != baseDoctorHospitalDOS && baseDoctorHospitalDOS.size() > 0) {
                baseDoctorHospitalDao.deleteAll(baseDoctorHospitalDOS);
                BaseDoctorHospitalDO baseDoctorHospitalDO = new BaseDoctorHospitalDO();
                baseDoctorHospitalDO.setOrgCode("350211A1002");
                baseDoctorHospitalDO.setOrgName("马銮湾医院");
                baseDoctorHospitalDO.setDoctorCode(baseDoctorDO.getId());
                baseDoctorHospitalDO.setDeptCode(deptCode);
                baseDoctorHospitalDO.setDeptName(deptName);
                baseDoctorHospitalDO.setDel("1");
                baseDoctorHospitalDO.setCreateTime(new Date());
                baseDoctorHospitalDao.save(baseDoctorHospitalDO);
            } else {
                BaseDoctorHospitalDO baseDoctorHospitalDO = new BaseDoctorHospitalDO();
                baseDoctorHospitalDO.setOrgCode("350211A1002");
                baseDoctorHospitalDO.setOrgName("马銮湾医院");
                baseDoctorHospitalDO.setDoctorCode(baseDoctorDO.getId());
                baseDoctorHospitalDO.setDeptCode(deptCode);
                baseDoctorHospitalDO.setDeptName(deptName);
                baseDoctorHospitalDO.setDel("1");
                baseDoctorHospitalDO.setCreateTime(new Date());
                baseDoctorHospitalDao.save(baseDoctorHospitalDO);
            }

        }
        return baseDoctorDO;
    }

    /**
     * @param dept
     * @param level    1院级管理员，2科室管理员
     * @param doctorId
     * @return
     */
    public JSONObject selectHealthByRole(String dept, int level, String doctorId) {

        String nowDate = DateUtil.getStringDateShort();
        Date startDate = DateUtil.strToDateLong(nowDate + " 00:00:00'");
        Date endDate = DateUtil.strToDateLong(nowDate + " 23:59:59'");
        BaseDoctorHealthUploadDO healthUploadDO = new BaseDoctorHealthUploadDO();
        List<BaseDoctorHealthUploadDO> healthUploadDOList = baseDoctorHealthUploadDao.selectHealthUploadInfoByDoctorIdAndStartAndEnd(doctorId,startDate,endDate);
        if (healthUploadDOList!=null&&healthUploadDOList.size()!=0){
            healthUploadDO = healthUploadDOList.get(0);
        }
        //今日本人提交
        String benrenSql = "select count(1) as total from base_doctor_health_upload where consumer='" + doctorId + "' and consumer in(doctor_id) and  create_time >='" + nowDate + " 00:00:00' and create_time<='" + nowDate + " 23:59:59'";
        //今日提交
        String personSql = "select count(1) as total from base_doctor_health_upload where consumer='" + doctorId + "' and  create_time >='" + nowDate + " 00:00:00' and create_time<='" + nowDate + " 23:59:59'";
        //代上报
        String daishangbaoSql = "select count(1) as total from base_doctor_health_upload where consumer='" + doctorId + "' and consumer not in(doctor_id)";
        //本人上报
        String benrenshangbaoSql = "select count(1) as total from base_doctor_health_upload where consumer='" + doctorId + "' and consumer in(doctor_id)";
        String doctorSql = "";
        String sql = "";
        String yichangSql = "";
        if (level == 1) {
            //总人次
            doctorSql = " select COUNT(1) as \"total\" from base_doctor d where d.del=1 ";
            //今日上报人次
            sql = "select COUNT(DISTINCT d.doctor_id) as \"total\" from base_doctor_health_upload d where 1=1 and d.create_time >='" + nowDate + " 00:00:00' and d.create_time<='" + nowDate + " 23:59:59' ";
            //今日异常人次
            yichangSql = "select COUNT(DISTINCT d.doctor_id) as \"total\" from ( SELECT\n" +
                    "\td3.*\n" +
                    "FROM\n" +
                    "\t( SELECT *,MAX(d1.create_time) as createTime FROM base_doctor_health_upload d1 GROUP BY\n" +
                    "\td1.doctor_id ) d2\n" +
                    "LEFT JOIN base_doctor_health_upload d3 ON d2.doctor_id=d3.doctor_id AND  d2.createTime = d3.create_time  ) d where 1=1 and d.create_time >='" + nowDate + " 00:00:00' and d.create_time<='" + nowDate + " 23:59:59' and (d.health_code in(2,3) OR CAST(d.temperature as DECIMAL)>=37.3 OR  d.is_symptoms=1 OR d.is_epidemic=1)";
        } else if (level == 2) {
            //总人次
            doctorSql = " select COUNT(1) as \"total\" from base_doctor d where d.del=1 and id IN (select dh.doctor_code from base_doctor_hospital dh where dh.dept_code = '" + dept + "' and dh.del=1 )";
            //今日上报人次
            sql = "select COUNT(DISTINCT d.doctor_id) as \"total\" from base_doctor_health_upload d where 1=1 and doctor_id in (select dh.doctor_code from base_doctor_hospital dh where dh.dept_code = '" + dept + "' and dh.del=1 ) and d.create_time >='" + nowDate + " 00:00:00' and d.create_time<='" + nowDate + " 23:59:59'";
            //今日异常人次
            yichangSql = "select COUNT(DISTINCT d.doctor_id) as \"total\" from ( SELECT\n" +
                    "\td3.*\n" +
                    "FROM\n" +
                    "\t( SELECT *,MAX(d1.create_time) as createTime FROM base_doctor_health_upload d1 GROUP BY\n" +
                    "\td1.doctor_id ) d2\n" +
                    "LEFT JOIN base_doctor_health_upload d3 ON d2.doctor_id=d3.doctor_id AND  d2.createTime = d3.create_time  ) d where 1=1 and doctor_id in (select dh.doctor_code from base_doctor_hospital dh where dh.dept_code = '" + dept + "' and dh.del=1 ) and d.create_time >='" + nowDate + " 00:00:00' and d.create_time<='" + nowDate + " 23:59:59' and (d.health_code in(2,3) OR CAST(d.temperature as DECIMAL)>=37.3 OR  d.is_symptoms=1 OR d.is_epidemic=1)";
        }
        //今日提交
        int personTotal = 0;
        //代上报
        int daishangbaoTotal = 0;
        //本人上报
        int benrenshangbaoTotal = 0;
        //总人次
        int doctorTotal = 0;
        //今日上报人次
        int sqlTotal = 0;
        //今日未上报人次
        int notUploadTotal = 0;
        //
        int yichangTotal = 0;
        Map<String, Object> personMap = jdbcTemplate.queryForMap(personSql);
        if (personMap != null) {
            if (personMap.get("total") != null) {
                personTotal = Integer.parseInt(personMap.get("total").toString());
            }
        }

        Map<String, Object> daishangbaoMap = jdbcTemplate.queryForMap(daishangbaoSql);
        if (daishangbaoMap != null) {
            if (daishangbaoMap.get("total") != null) {
                daishangbaoTotal = Integer.parseInt(daishangbaoMap.get("total").toString());
            }
        }

        Map<String, Object> benrenshangbaoMap = jdbcTemplate.queryForMap(benrenshangbaoSql);
        if (benrenshangbaoMap != null) {
            if (benrenshangbaoMap.get("total") != null) {
                benrenshangbaoTotal = Integer.parseInt(benrenshangbaoMap.get("total").toString());
            }
        }

        Map<String, Object> doctorMap = jdbcTemplate.queryForMap(doctorSql);
        if (doctorMap != null) {
            if (doctorMap.get("total") != null) {
                doctorTotal = Integer.parseInt(doctorMap.get("total").toString());
            }
        }
        Map<String, Object> sqlMap = jdbcTemplate.queryForMap(sql);
        if (sqlMap != null) {
            if (sqlMap.get("total") != null) {
                sqlTotal = Integer.parseInt(sqlMap.get("total").toString());
            }
        }
        int benrenTotal = 0;
        Map<String, Object> benrenMap = jdbcTemplate.queryForMap(benrenSql);
        if (benrenMap != null) {
            if (benrenMap.get("total") != null) {
                benrenTotal = Integer.parseInt(benrenMap.get("total").toString());
            }
        }

        Map<String, Object> yichangMap = jdbcTemplate.queryForMap(yichangSql);
        if (yichangMap != null) {
            if (yichangMap.get("total") != null) {
                yichangTotal = Integer.parseInt(yichangMap.get("total").toString());
            }
        }


        JSONObject object = new JSONObject();
        object.put("personTotal", personTotal);//今日上报人数
        object.put("daishangbaoTotal", daishangbaoTotal);//代上报
        object.put("benrenshangbaoTotal", benrenshangbaoTotal);//已上报
        object.put("doctorTotal", doctorTotal);//总人次
        object.put("sqlTotal", sqlTotal);//今日上报人数
        object.put("notUploadTotal", doctorTotal - sqlTotal);//今日未上报人数
        object.put("benrenTotal", benrenTotal);//今日本人提交
        object.put("healthUploadDO",healthUploadDO);
        object.put("yichangTotal",yichangTotal);//今日异常
        return object;
    }



    /**
     * 首页统计
     *
     * @param dept
     * @param idType
     * @param status    上报状态
     * @param state     健康状态
     * @param startDate
     * @param endDate
     * @return
     */
    public JSONObject selectHeaderTotal(String dept, String idType, String status, String state, String startDate, String endDate, String name) {
        JSONObject object = new JSONObject();
        String day = DateUtil.getTwoDay(endDate, startDate);
        String doctorSql = " select COUNT(1) as \"total\" from base_doctor d where d.del=1 ";
        Integer total = 0;//总人次
        String sql = "select COUNT(DISTINCT d.doctor_id) as \"total\" from base_doctor_health_upload d where 1=1 ";
        String deptCondition = "";
        if (StringUtils.isNoneBlank(dept)) {
            deptCondition += " (select dh.doctor_code from base_doctor_hospital dh where dh.dept_code = '" + dept + "' and dh.del=1 )";
        }
        String idTypeCondition = " ";
        if (StringUtils.isNoneBlank(idType)) {
            idTypeCondition += "  (select d1.id from base_doctor d1 where 1=1 and d1.id_type = '" + idType + "')";
        }
        if (StringUtils.isNoneBlank(deptCondition)) {
            doctorSql += " and d.id IN " + deptCondition;
        }
        if (StringUtils.isNoneBlank(idTypeCondition)) {
            doctorSql += " and d.id IN " + idTypeCondition;
        }
        if (StringUtils.isNoneBlank(name)) {
            doctorSql += " and d.name like '%" + name + "%' ";
        }
        Map<String, Object> map = jdbcTemplate.queryForMap(doctorSql);
        String doctorTotal = "";
        if (map != null) {
            doctorTotal = map.get("total").toString();
        }
        if (StringUtils.isNoneBlank(doctorTotal)) {
            //总人次
            if (!day.equalsIgnoreCase("0")) {
                total = Integer.parseInt(doctorTotal) * Integer.parseInt(day);
            } else {
                total = Integer.parseInt(doctorTotal);
            }
        }
        Integer yishangbaoTotal = 0;
        Integer weishangbaoTotal = 0;
        Integer zhengchangTotal = 0;
        Integer yichangTotal = 0;
        if (StringUtils.isNoneBlank(startDate)) {
            sql += " and d.create_time >='" + startDate + " 00:00:00' and d.create_time<='" + endDate + " 23:59:59' ";
        }
        if (StringUtils.isNoneBlank(state)) {
            if (state.equalsIgnoreCase("1")) {
                sql += " and (d.health_code not in(2,3) and CAST(d.temperature as DECIMAL)<37.3 and  d.is_symptoms=2 and d.is_epidemic=2) ";
            } else if (state.equalsIgnoreCase("2")) {
                sql += " and (d.health_code in(2,3) OR CAST(d.temperature as DECIMAL)>=37.3 OR  d.is_symptoms=1 OR d.is_epidemic=1) ";
            }
        }
        if (StringUtils.isNoneBlank(deptCondition)) {
            sql += " and d.doctor_id IN " + deptCondition;
        }
        if (StringUtils.isNoneBlank(idTypeCondition)) {
            sql += " and d.doctor_id IN " + idTypeCondition;
        }
        if (StringUtils.isNoneBlank(name)) {
            sql += " and d.doctor_name like '%" + name + "%'";
        }
        //已上传
        Map<String, Object> yishangchaungMap = jdbcTemplate.queryForMap(sql);
        if (yishangchaungMap != null) {
            if (yishangchaungMap.get("total") != null) {
                yishangbaoTotal = Integer.parseInt(yishangchaungMap.get("total").toString());
            }
        }
        //未上报
        weishangbaoTotal = total - yishangbaoTotal;


        //上传总次数
        Integer shangchuanTotal = 0;

        String shangchuanTotalSql = "select COUNT(1) as \"total\" from base_doctor_health_upload d where 1=1  ";

        if (StringUtils.isNoneBlank(startDate)) {
            shangchuanTotalSql += " and d.create_time >='" + startDate + " 00:00:00' and d.create_time<='" + endDate + " 23:59:59' ";
        }
        if (StringUtils.isNoneBlank(deptCondition)) {
            shangchuanTotalSql += " and d.doctor_id IN " + deptCondition;
        }
        if (StringUtils.isNoneBlank(idTypeCondition)) {
            shangchuanTotalSql += " and d.doctor_id IN " + idTypeCondition;
        }
        if (StringUtils.isNoneBlank(name)) {
            shangchuanTotalSql += " and d.doctor_name like '%" + name + "%'";
        }
        Map<String, Object> shangchuanMap = jdbcTemplate.queryForMap(shangchuanTotalSql);
        if (shangchuanMap != null) {
            if (shangchuanMap.get("total") != null) {
                shangchuanTotal = Integer.parseInt(shangchuanMap.get("total").toString());
            }
        }


        //正常、异常
        String sqlZhengchang = "select COUNT(1) as \"total\" from base_doctor_health_upload d where 1=1 and (d.health_code not in(2,3) AND CAST(d.temperature as DECIMAL)<37.3 AND  d.is_symptoms=2 and  d.is_epidemic=2) ";

        if (StringUtils.isNoneBlank(startDate)) {
            sqlZhengchang += " and d.create_time >='" + startDate + " 00:00:00' and d.create_time<='" + endDate + " 23:59:59' ";
        }
        if (StringUtils.isNoneBlank(deptCondition)) {
            sqlZhengchang += " and d.doctor_id IN " + deptCondition;
        }
        if (StringUtils.isNoneBlank(idTypeCondition)) {
            sqlZhengchang += " and d.doctor_id IN " + idTypeCondition;
        }
        if (StringUtils.isNoneBlank(name)) {
            sqlZhengchang += " and d.doctor_name like '%" + name + "%'";
        }
        //正常
        Map<String, Object> zhengchangMap = jdbcTemplate.queryForMap(sqlZhengchang);
        if (zhengchangMap != null) {
            if (zhengchangMap.get("total") != null) {
                zhengchangTotal = Integer.parseInt(zhengchangMap.get("total").toString());
            }
        }

        String sqlYichang = "select COUNT(1) as \"total\" from base_doctor_health_upload d where 1=1 and (d.health_code in(2,3) OR CAST(d.temperature as DECIMAL)>=37.3 OR  d.is_symptoms=1 OR d.is_epidemic=1)  ";
        if (StringUtils.isNoneBlank(startDate)) {
            sqlYichang += " and d.create_time >='" + startDate + " 00:00:00' and d.create_time<='" + endDate + " 23:59:59' ";
        }
        if (StringUtils.isNoneBlank(deptCondition)) {
            sqlYichang += " and d.doctor_id IN " + deptCondition;
        }
        if (StringUtils.isNoneBlank(idTypeCondition)) {
            sqlYichang += " and d.doctor_id IN " + idTypeCondition;
        }
        if (StringUtils.isNoneBlank(name)) {
            sqlYichang += " and d.doctor_name like '%" + name + "%'";
        }
        //正常
        Map<String, Object> yichangMap = jdbcTemplate.queryForMap(sqlYichang);
        if (yichangMap != null) {
            if (yichangMap.get("total") != null) {
                yichangTotal = Integer.parseInt(yichangMap.get("total").toString());
            }
        }
        object.put("total", total);//总人次
        object.put("yishangbaoTotal", yishangbaoTotal);//已上报
        object.put("weishangbaoTotal", weishangbaoTotal);//未上报
        object.put("zhengchangTotal", zhengchangTotal);//上报正常
        object.put("yichangTotal", yichangTotal);//上报异常
        object.put("shangchuanTotal", shangchuanTotal);//上传总次数
        return object;
    }


    /**
     * 首页按照科室列表统计
     *
     * @param dept
     * @param idType
     * @param status    上报状态
     * @param state     健康状态
     * @param startDate
     * @param endDate
     * @return
     */
    public List<Map<String, Object>> selectHeaderList(String dept, String idType, String status, String state, String startDate, String endDate, String name) {
        String condition = "";
        if (StringUtils.isNoneBlank(startDate)) {
            condition += " and d.create_time >='" + startDate + " 00:00:00' and d.create_time<='" + endDate + " 23:59:59' ";
        }
        String doctorSql = "";
        if (StringUtils.isNoneBlank(idType)) {
            doctorSql += " and d.id_type = '" + idType + "' ";
        }
        if (StringUtils.isNoneBlank(name)) {
            doctorSql += " and d.name like '%" + name + "%' ";
        }
        String sql = "";
        if (StringUtils.isNoneBlank(state)) {
            if (state.equalsIgnoreCase("1")) {
                sql += " and (d.health_code not in(2,3) and CAST(d.temperature as DECIMAL)<37.3 and  d.is_symptoms=2 and d.is_epidemic=2) ";
            } else if (state.equalsIgnoreCase("2")) {
                sql += " and (d.health_code in(2,3) OR CAST(d.temperature as DECIMAL)>=37.3 OR  d.is_symptoms=1 OR d.is_epidemic=1) ";
            }
        }
        String idTypeCondition = " ";
        if (StringUtils.isNoneBlank(idType)) {
            idTypeCondition += "  (select d1.id from base_doctor d1 where 1=1 and d1.id_type = '" + idType + "')";
        }
        if (StringUtils.isNoneBlank(idTypeCondition)) {
            sql += " and d.doctor_id IN " + idTypeCondition;
        }
        if (StringUtils.isNoneBlank(name)) {
            sql += " and d.doctor_name like '%" + name + "%'";
        }
        String ycandzcSql = "";
        if (StringUtils.isNoneBlank(idTypeCondition)) {
            ycandzcSql += " and d.doctor_id IN " + idTypeCondition;
        }
        if (StringUtils.isNoneBlank(name)) {
            ycandzcSql += " and d.doctor_name like '%" + name + "%'";
        }
        String deptSql = "SELECT\n" +
                "\tcode,\n" +
                "\tname,\n" +
                "\tdh1.total,\n" +
                "\tdh2.total as yishangbaoTotal,\n" +
                "\tdh3.total as shangchuanTotal,\n" +
                "\tdh4.total as zhengchangTotal,\n" +
                "\tdh5.total as yichangTotal,\n" +
                "\tdh6.total as yichangPersonTotal\n" +
                "FROM\n" +
                "\tdict_hospital_dept hd\n" +
                "LEFT JOIN (\n" +
                "\tSELECT\n" +
                "\t\tdh.dept_code,\n" +
                "\t\tCOUNT(1) AS \"total\"\n" +
                "\tFROM\n" +
                "\t\tbase_doctor d\n" +
                "\tLEFT JOIN base_doctor_hospital dh ON dh.doctor_code = d.id\n" +
                "\tWHERE\n" +
                "\t\td.del = 1\n" +
                "\tAND dh.del = 1\n" +
                doctorSql+
                "\tGROUP BY\n" +
                "\t\tdh.dept_code\n" +
                ") dh1 ON hd. CODE = dh1.dept_code\n" +
                "LEFT JOIN (\n" +
                "\tSELECT\n" +
                "\t\tdh.dept_code,\n" +
                "\t\tCOUNT(DISTINCT d.doctor_id) AS total\n" +
                "\tFROM\n" +
                "\t\tbase_doctor_health_upload d\n" +
                "\tLEFT JOIN base_doctor_hospital dh ON dh.doctor_code = d.doctor_id\n" +
                "\tWHERE\n" +
                "\t\t1 = 1\n" +
              condition+ sql+
                "\tGROUP BY\n" +
                "\t\tdh.dept_code\n" +
                ")dh2 ON dh2.dept_code=hd.code\n" +
                "LEFT JOIN (\n" +
                "\tSELECT\n" +
                "\t\tdh.dept_code,\n" +
                "\t\tCOUNT(1) AS total\n" +
                "\tFROM\n" +
                "\t\tbase_doctor_health_upload d\n" +
                "\tLEFT JOIN base_doctor_hospital dh ON dh.doctor_code = d.doctor_id\n" +
                "\tWHERE\n" +
                "\t\t1 = 1\n" +
                condition+ sql+
                "\tGROUP BY\n" +
                "\t\tdh.dept_code\n" +
                ")dh3 ON dh3.dept_code=hd.code\n" +
                "LEFT JOIN (\n" +
                "\tSELECT\n" +
                "\t\tdh.dept_code,\n" +
                "\t\tCOUNT(1) AS total\n" +
                "\tFROM\n" +
                "\t\tbase_doctor_health_upload d\n" +
                "\tLEFT JOIN base_doctor_hospital dh ON dh.doctor_code = d.doctor_id\n" +
                "\tWHERE\n" +
                "\t\t1 = 1\n" +
                "\tand (d.health_code not in(2,3) AND CAST(d.temperature as DECIMAL)<37.3 AND  d.is_symptoms=2 and  d.is_epidemic=2)\n" +
                condition+ycandzcSql+
                "\tGROUP BY\n" +
                "\t\tdh.dept_code\n" +
                ")dh4 ON dh4.dept_code=hd.code\n" +
                "LEFT JOIN (\n" +
                "\tSELECT\n" +
                "\t\tdh.dept_code,\n" +
                "\t\tCOUNT(1) AS total\n" +
                "\tFROM\n" +
                "\t\tbase_doctor_health_upload d\n" +
                "\tLEFT JOIN base_doctor_hospital dh ON dh.doctor_code = d.doctor_id\n" +
                "\tWHERE\n" +
                "\t\t1 = 1\n" +
                "\tand (d.health_code in(2,3) OR CAST(d.temperature as DECIMAL)>=37.3 OR  d.is_symptoms=1 OR d.is_epidemic=1)\n" +
                condition+ycandzcSql+
                "\tGROUP BY\n" +
                "\t\tdh.dept_code\n" +
                ")dh5 ON dh5.dept_code=hd.code\n" +
                "LEFT JOIN (\n" +
                "\tSELECT\n" +
                "\t\tdh.dept_code,\n" +
                "\t\tCOUNT(DISTINCT d.doctor_id) AS total\n" +
                "\tFROM\n" +
                "\t\tbase_doctor_health_upload d\n" +
                "\tLEFT JOIN base_doctor_hospital dh ON dh.doctor_code = d.doctor_id\n" +
                "\tWHERE\n" +
                "\t\t1 = 1\n" +
                "\tand (d.health_code in(2,3) OR CAST(d.temperature as DECIMAL)>=37.3 OR  d.is_symptoms=1 OR d.is_epidemic=1)\n" +
                condition+ycandzcSql+
                "\tGROUP BY\n" +
                "\t\tdh.dept_code\n" +
                ")dh6 ON dh6.dept_code=hd.code\n" +
                "WHERE\n" +
                "\thd.consult_dept_flag = 1 ";
        if (StringUtils.isNoneBlank(name)) {
            deptSql += " and hd.code IN(SELECT dha.dept_code FROM base_doctor_hospital dha LEFT JOIN base_doctor da ON da.id=dha.doctor_code where da.name LIKE '%" + name + "%') ";
        }
        if (StringUtils.isNoneBlank(dept)) {
            deptSql += " and hd.code = '" + dept + "' ";
        }
        List<Map<String, Object>> mapList = jdbcTemplate.queryForList(deptSql);
        for (Map<String,Object> deptMap:mapList){
            String day = DateUtil.getTwoDay(endDate, startDate);
            Integer total = 0;
            Integer yishangbaoTotal = 0;
            Integer weishangbaoTotal = 0;
            Integer zhengchangTotal = 0;
            Integer yichangTotal = 0;
            Integer shangchuanTotal = 0;
            Integer yichangPersonTotal = 0;
            if (deptMap.get("total")!=null){
                String doctorTotal = deptMap.get("total").toString();
                //总人次
                if (!day.equalsIgnoreCase("0")) {
                    total = Integer.parseInt(doctorTotal) * Integer.parseInt(day);
                } else {
                    total = Integer.parseInt(doctorTotal);
                }
            }
            if (deptMap.get("yishangbaoTotal")!=null){
                yishangbaoTotal = Integer.parseInt(deptMap.get("yishangbaoTotal").toString());
            }
            //未上报
            weishangbaoTotal = total - yishangbaoTotal;
            if (deptMap.get("zhengchangTotal")!=null){
                zhengchangTotal = Integer.parseInt(deptMap.get("zhengchangTotal").toString());
            }
            if (deptMap.get("yichangTotal")!=null){
                yichangTotal = Integer.parseInt(deptMap.get("yichangTotal").toString());
            }

            if (deptMap.get("yichangPersonTotal")!=null){
                yichangPersonTotal = Integer.parseInt(deptMap.get("yichangPersonTotal").toString());
            }
            if (deptMap.get("shangchuanTotal")!=null){
                shangchuanTotal = Integer.parseInt(deptMap.get("shangchuanTotal").toString());
            }

            JSONObject object = new JSONObject();
            object.put("total", total);//总人次
            object.put("yishangbaoTotal", yishangbaoTotal);//已上报
            object.put("weishangbaoTotal", weishangbaoTotal);//未上报
            object.put("zhengchangTotal", zhengchangTotal);//上报正常
            object.put("yichangTotal", yichangTotal);//上报异常
            object.put("yichangPersonTotal", yichangPersonTotal);//上报人异常
            object.put("shangchuanTotal", shangchuanTotal);//上传总次数
            deptMap.put("tongyishuju", object);
        }

        /*  String deptSql = " select code,name from dict_hospital_dept hd where hd.consult_dept_flag=1 ";
        if (StringUtils.isNoneBlank(name)) {
            deptSql += " and hd.code IN(SELECT dha.dept_code FROM base_doctor_hospital dha LEFT JOIN base_doctor da ON da.id=dha.doctor_code where da.name LIKE '%" + name + "%') ";
        }
        if (StringUtils.isNoneBlank(dept)) {
            deptSql += " and hd.code = '" + dept + "' ";
        }*/
       /* List<Map<String, Object>> mapList = jdbcTemplate.queryForList(deptSql);
        for (Map<String, Object> deptmap : mapList) {
            String deptCode = deptmap.get("code").toString();
            JSONObject object = new JSONObject();
            String day = DateUtil.getTwoDay(endDate, startDate);
            String doctorSql = " select COUNT(1) as \"total\" from base_doctor d where d.del=1 ";
            Integer total = 0;//总人次
            String sql = "select COUNT(DISTINCT d.doctor_id) as \"total\" from base_doctor_health_upload d where 1=1 ";
            String deptCondition = "";
            if (StringUtils.isNoneBlank(deptCode)) {
                deptCondition += " (select dh.doctor_code from base_doctor_hospital dh where dh.dept_code = '" + deptCode + "' and dh.del=1 )";
            }
            String idTypeCondition = " ";
            if (StringUtils.isNoneBlank(idType)) {
                idTypeCondition += "  (select d1.id from base_doctor d1 where 1=1 and d1.id_type = '" + idType + "')";
            }
            if (StringUtils.isNoneBlank(deptCondition)) {
                doctorSql += " and d.id IN " + deptCondition;
            }
            if (StringUtils.isNoneBlank(idTypeCondition)) {
                doctorSql += " and d.id IN " + idTypeCondition;
            }
            if (StringUtils.isNoneBlank(name)) {
                doctorSql += " and d.name like '%" + name + "%' ";
            }
            Map<String, Object> map = jdbcTemplate.queryForMap(doctorSql);
            String doctorTotal = "";
            if (map != null) {
                doctorTotal = map.get("total").toString();
            }
            if (StringUtils.isNoneBlank(doctorTotal)) {
                //总人次
                if (!day.equalsIgnoreCase("0")) {
                    total = Integer.parseInt(doctorTotal) * Integer.parseInt(day);
                } else {
                    total = Integer.parseInt(doctorTotal);
                }
            }
            Integer yishangbaoTotal = 0;
            Integer weishangbaoTotal = 0;
            Integer zhengchangTotal = 0;
            Integer yichangTotal = 0;
            if (StringUtils.isNoneBlank(startDate)) {
                sql += " and d.create_time >='" + startDate + " 00:00:00' and d.create_time<='" + endDate + " 23:59:59' ";
            }
            if (StringUtils.isNoneBlank(state)) {
                if (state.equalsIgnoreCase("1")) {
                    sql += " and (d.health_code not in(2,3) and CAST(d.temperature as DECIMAL)<37.3 and  d.is_symptoms=2 and d.is_epidemic=2) ";
                } else if (state.equalsIgnoreCase("2")) {
                    sql += " and (d.health_code in(2,3) OR CAST(d.temperature as DECIMAL)>=37.3 OR  d.is_symptoms=1 OR d.is_epidemic=1) ";
                }
            }
            if (StringUtils.isNoneBlank(deptCondition)) {
                sql += " and d.doctor_id IN " + deptCondition;
            }
            if (StringUtils.isNoneBlank(idTypeCondition)) {
                sql += " and d.doctor_id IN " + idTypeCondition;
            }
            if (StringUtils.isNoneBlank(name)) {
                sql += " and d.doctor_name like '%" + name + "%'";
            }
            //已上传
            Map<String, Object> yishangchaungMap = jdbcTemplate.queryForMap(sql);
            if (yishangchaungMap != null) {
                if (yishangchaungMap.get("total") != null) {
                    yishangbaoTotal = Integer.parseInt(yishangchaungMap.get("total").toString());
                }
            }
            //未上报
            weishangbaoTotal = total - yishangbaoTotal;


            //上传总次数
            Integer shangchuanTotal = 0;

            String shangchuanTotalSql = "select COUNT(1) as \"total\" from base_doctor_health_upload d where 1=1  ";

            if (StringUtils.isNoneBlank(startDate)) {
                shangchuanTotalSql += " and d.create_time >='" + startDate + " 00:00:00' and d.create_time<='" + endDate + " 23:59:59' ";
            }
            if (StringUtils.isNoneBlank(deptCondition)) {
                shangchuanTotalSql += " and d.doctor_id IN " + deptCondition;
            }
            if (StringUtils.isNoneBlank(idTypeCondition)) {
                shangchuanTotalSql += " and d.doctor_id IN " + idTypeCondition;
            }
            if (StringUtils.isNoneBlank(name)) {
                shangchuanTotalSql += " and d.doctor_name like '%" + name + "%'";
            }
            Map<String, Object> shangchuanMap = jdbcTemplate.queryForMap(shangchuanTotalSql);
            if (shangchuanMap != null) {
                if (shangchuanMap.get("total") != null) {
                    shangchuanTotal = Integer.parseInt(shangchuanMap.get("total").toString());
                }
            }


            //正常、异常
            String sqlZhengchang = "select COUNT(1) as \"total\" from base_doctor_health_upload d where 1=1 and (d.health_code not in(2,3) AND CAST(d.temperature as DECIMAL)<37.3 AND  d.is_symptoms=2 and  d.is_epidemic=2) ";

            if (StringUtils.isNoneBlank(startDate)) {
                sqlZhengchang += " and d.create_time >='" + startDate + " 00:00:00' and d.create_time<='" + endDate + " 23:59:59' ";
            }
            if (StringUtils.isNoneBlank(deptCondition)) {
                sqlZhengchang += " and d.doctor_id IN " + deptCondition;
            }
            if (StringUtils.isNoneBlank(idTypeCondition)) {
                sqlZhengchang += " and d.doctor_id IN " + idTypeCondition;
            }
            if (StringUtils.isNoneBlank(name)) {
                sqlZhengchang += " and d.doctor_name like '%" + name + "%'";
            }
            //正常
            Map<String, Object> zhengchangMap = jdbcTemplate.queryForMap(sqlZhengchang);
            if (zhengchangMap != null) {
                if (zhengchangMap.get("total") != null) {
                    zhengchangTotal = Integer.parseInt(zhengchangMap.get("total").toString());
                }
            }

            String sqlYichang = "select COUNT(1) as \"total\" from base_doctor_health_upload d where 1=1 and (d.health_code in(2,3) OR CAST(d.temperature as DECIMAL)>=37.3 OR  d.is_symptoms=1 OR d.is_epidemic=1)  ";
            if (StringUtils.isNoneBlank(startDate)) {
                sqlYichang += " and d.create_time >='" + startDate + " 00:00:00' and d.create_time<='" + endDate + " 23:59:59' ";
            }
            if (StringUtils.isNoneBlank(deptCondition)) {
                sqlYichang += " and d.doctor_id IN " + deptCondition;
            }
            if (StringUtils.isNoneBlank(idTypeCondition)) {
                sqlYichang += " and d.doctor_id IN " + idTypeCondition;
            }
            if (StringUtils.isNoneBlank(name)) {
                sqlYichang += " and d.doctor_name like '%" + name + "%'";
            }
            //异常
            Map<String, Object> yichangMap = jdbcTemplate.queryForMap(sqlYichang);
            if (yichangMap != null) {
                if (yichangMap.get("total") != null) {
                    yichangTotal = Integer.parseInt(yichangMap.get("total").toString());
                }
            }

            String sqlDistintYichang = "select COUNT(DISTINCT doctor_id) as \"total\" from (SELECT\n" +
                    "\td3.*\n" +
                    "FROM\n" +
                    "\t( SELECT *,MAX(d1.create_time) as createTime FROM base_doctor_health_upload d1 GROUP BY\n" +
                    "\td1.doctor_id ) d2\n" +
                    "LEFT JOIN base_doctor_health_upload d3 ON d2.doctor_id=d3.doctor_id AND  d2.createTime = d3.create_time  ) d where 1=1 and (d.health_code in(2,3) OR CAST(d.temperature as DECIMAL)>=37.3 OR  d.is_symptoms=1 OR d.is_epidemic=1)  ";
            if (StringUtils.isNoneBlank(startDate)) {
                sqlDistintYichang += " and d.create_time >='" + startDate + " 00:00:00' and d.create_time<='" + endDate + " 23:59:59' ";
            }
            if (StringUtils.isNoneBlank(deptCondition)) {
                sqlDistintYichang += " and d.doctor_id IN " + deptCondition;
            }
            if (StringUtils.isNoneBlank(idTypeCondition)) {
                sqlDistintYichang += " and d.doctor_id IN " + idTypeCondition;
            }
            if (StringUtils.isNoneBlank(name)) {
                sqlDistintYichang += " and d.doctor_name like '%" + name + "%'";
            }
            Integer yichangPersonTotal =0;
            //异常
            Map<String, Object> sqlDistintYichangMap = jdbcTemplate.queryForMap(sqlDistintYichang);
            if (sqlDistintYichangMap != null) {
                if (sqlDistintYichangMap.get("total") != null) {
                    yichangPersonTotal = Integer.parseInt(sqlDistintYichangMap.get("total").toString());
                }
            }
            object.put("total", total);//总人次
            object.put("yishangbaoTotal", yishangbaoTotal);//已上报
            object.put("weishangbaoTotal", weishangbaoTotal);//未上报
            object.put("zhengchangTotal", zhengchangTotal);//上报正常
            object.put("yichangTotal", yichangTotal);//上报异常
            object.put("yichangPersonTotal", yichangPersonTotal);//上报人异常
            object.put("shangchuanTotal", shangchuanTotal);//上传总次数
            deptmap.put("tongyishuju", object);
        }*/

        return mapList;
    }

    /**
     * 具体每一个统计时间列表
     *
     * @param dept
     * @param idType
     * @param status
     * @param state
     * @param startDate
     * @param endDate
     * @return
     */
    public JSONArray selectListByTotal(String dept, String idType, String status, String state, String startDate, String endDate, String name) {
        long days = DateUtil.getDays(endDate, startDate);
        List<String> list = new ArrayList<>();
        Date startTime = DateUtil.strToDateShort(startDate);
        for (int i = 0; i <= days; i++) {
            Date date = DateUtil.getPreDays(startTime, i);
            String str = DateUtil.dateToStrShort(date);
            list.add(str);
        }
        JSONArray array = new JSONArray();
        for (int i = 0; i < list.size(); i++) {
            String date = list.get(i);
            String doctorSql = " select COUNT(1) as \"total\" from base_doctor d where d.del=1 ";
            String deptCondition = "";
            if (StringUtils.isNoneBlank(dept)) {
                deptCondition += " (select dh.doctor_code from base_doctor_hospital dh where dh.dept_code = '" + dept + "' and dh.del=1 )";
            }
            String idTypeCondition = " ";
            if (StringUtils.isNoneBlank(idType)) {
                idTypeCondition += "  (select d1.id from base_doctor d1 where 1=1 and d1.id_type = '" + idType + "')";
            }
            if (StringUtils.isNoneBlank(deptCondition)) {
                doctorSql += " and d.id IN " + deptCondition;
            }
            if (StringUtils.isNoneBlank(idTypeCondition)) {
                doctorSql += " and d.id IN " + idTypeCondition;
            }
            if (StringUtils.isNoneBlank(status)) {
                if (status.equalsIgnoreCase("1")) {
                    doctorSql += " and d.id in(select d1.doctor_id from base_doctor_health_upload d1 where 1=1 and d1.create_time >='" + date + " 00:00:00' and d1.create_time<='" + date + " 23:59:59' )";
                } else if (status.equalsIgnoreCase("2")) {
                    doctorSql += " and d.id not in(select d1.doctor_id from base_doctor_health_upload d1 where 1=1 and d1.create_time >='" + date + " 00:00:00' and d1.create_time<='" + date + " 23:59:59' )";
                }
            }
            if (StringUtils.isNoneBlank(state)) {
                if (state.equalsIgnoreCase("1")) {
                    doctorSql += " and d.id in(select d1.doctor_id from base_doctor_health_upload d1 where 1=1 and d1.create_time >='" + date + " 00:00:00' and d1.create_time<='" + date + " 23:59:59' and (d1.health_code not in(2,3) and CAST(d1.temperature as DECIMAL)<37.3 and  d1.is_symptoms=2 and d1.is_epidemic=2))  ";
                } else if (state.equalsIgnoreCase("2")) {
                    doctorSql += " and d.id in(select d1.doctor_id from base_doctor_health_upload d1 where 1=1 and d1.create_time >='" + date + " 00:00:00' and d1.create_time<='" + date + " 23:59:59' and (d1.health_code in(2,3) OR CAST(d1.temperature as DECIMAL)>=37.3 OR  d1.is_symptoms=1 OR d1.is_epidemic=1)) ";
                }
            }
            if (StringUtils.isNoneBlank(name)) {
                doctorSql += " and d.name LIKE  '%" + name + "%' ";
            }
            Map<String, Object> map = jdbcTemplate.queryForMap(doctorSql);
            Integer total = 0;
            if (map != null) {
                if (map.get("total") != null) {
                    total = Integer.parseInt(map.get("total").toString());
                }
            }
            JSONObject object = new JSONObject();
            object.put("time", date);
            object.put("total", total);
            array.add(object);
        }
        return array;
    }

    /**
     * 具体每一个统计时间的详情
     *
     * @param dept
     * @param idType
     * @param status
     * @param state
     * @param startDate
     * @param endDate
     * @param name      total
     * @return
     */
    public JSONArray selectListInfoByTotal(String dept, String idType, String status, String state, String startDate, String endDate, String name) {
        long days = DateUtil.getDays(endDate, startDate);
        List<String> list = new ArrayList<>();
        Date startTime = DateUtil.strToDateShort(startDate);
        for (int i = 0; i <= days; i++) {
            Date date = DateUtil.getPreDays(startTime, i);
            String str = DateUtil.dateToStrShort(date);
            list.add(str);
        }
        JSONArray array = new JSONArray();
        for (int i = 0; i < list.size(); i++) {
            String date = list.get(i);
            String doctorSql = " select d.name,d.id,d.job_title_name as jobTitleName,d.photo from base_doctor d where d.del=1 ";
            String deptCondition = "";
            if (StringUtils.isNoneBlank(dept)) {
                deptCondition += " (select dh.doctor_code from base_doctor_hospital dh where dh.dept_code = '" + dept + "' and dh.del=1 )";
            }
            String idTypeCondition = " ";
            if (StringUtils.isNoneBlank(idType)) {
                idTypeCondition += "  (select d1.id from base_doctor d1 where 1=1 and d1.id_type = '" + idType + "')";
            }
            if (StringUtils.isNoneBlank(deptCondition)) {
                doctorSql += " and d.id IN " + deptCondition;
            }
            if (StringUtils.isNoneBlank(idTypeCondition)) {
                doctorSql += " and d.id IN " + idTypeCondition;
            }
            if (StringUtils.isNoneBlank(status)) {
                if (status.equalsIgnoreCase("1")) {
                    doctorSql += " and d.id in (select d1.doctor_id from base_doctor_health_upload d1 where 1=1 and d1.create_time >='" + date + " 00:00:00' and d1.create_time<='" + date + " 23:59:59')";
                } else if (status.equalsIgnoreCase("2")) {
                    doctorSql += " and d.id not in(select d1.doctor_id from base_doctor_health_upload d1 where 1=1 and d1.create_time >='" + date + " 00:00:00' and d1.create_time<='" + date + " 23:59:59')";
                }
            }
            if (StringUtils.isNoneBlank(state)) {
                if (state.equalsIgnoreCase("1")) {
                    doctorSql += " and d.id in(select d1.doctor_id from base_doctor_health_upload d1 where 1=1 and d1.create_time >='" + date + " 00:00:00' and d1.create_time<='" + date + " 23:59:59' and (d1.health_code not in(2,3) and CAST(d1.temperature as DECIMAL)<37.3 and  d1.is_symptoms=2 and d1.is_epidemic=2) )  ";
                } else if (state.equalsIgnoreCase("2")) {
                    doctorSql += " and d.id in(select d1.doctor_id from base_doctor_health_upload d1 where 1=1 and d1.create_time >='" + date + " 00:00:00' and d1.create_time<='" + date + " 23:59:59' and (d1.health_code in(2,3) OR CAST(d1.temperature as DECIMAL)>=37.3 OR  d1.is_symptoms=1 OR d1.is_epidemic=1)) ";
                }
            }
            if (StringUtils.isNoneBlank(name)) {
                doctorSql += " and d.NAME  LIKE '%" + name + "%'";
            }
            List<Map<String, Object>> mapList = jdbcTemplate.queryForList(doctorSql);
            for (Map<String, Object> map : mapList) {
                String id = map.get("id").toString();
                List<BaseDoctorHospitalDO> hospitalDOList = baseDoctorHospitalDao.findByDoctorCode(id);
                map.put("deptInfo", hospitalDOList);
                String stateCondition = "";
                if (StringUtils.isNoneBlank(state)) {
                    if (state.equalsIgnoreCase("1")) {
                        stateCondition += " and (d1.health_code not in(2,3) and CAST(d1.temperature as DECIMAL)<37.3 and  d1.is_symptoms=2 and d1.is_epidemic=2)  ";
                    } else if (state.equalsIgnoreCase("2")) {
                        stateCondition += " and (d1.health_code in(2,3) OR CAST(d1.temperature as DECIMAL)>=37.3 OR  d1.is_symptoms=1 OR d1.is_epidemic=1) ";
                    }
                }
                String sql = "select d1.health_code,d1.temperature,d1.is_symptoms as isSymptoms,d1.is_epidemic as isEpidemic,d1.id,d1.consumer,d1.consumer_name as consumerName,date_format(d1.create_time ,'%Y-%m-%d %H:%i:%S' ) AS createTime from base_doctor_health_upload d1 where 1=1 " + stateCondition + " and d1.create_time >='" + date + " 00:00:00' and d1.create_time<='" + date + " 23:59:59' and d1.doctor_id = '" + id + "' order by d1.create_time desc ";
                List<Map<String, Object>> mapList1 = jdbcTemplate.queryForList(sql);
                if (mapList1 != null && mapList1.size() != 0) {
                    for (Map<String, Object> map1 : mapList1) {
                        String healthCode = map1.get("health_code").toString();
                        Double temperature = Double.parseDouble(map1.get("temperature").toString());
                        String isSymptoms = map1.get("isSymptoms").toString();
                        String isEpidemic = map1.get("isEpidemic").toString();
                        if (healthCode.equalsIgnoreCase("2") || healthCode.equalsIgnoreCase("3") ||
                                temperature >= 37.3 || isSymptoms.equalsIgnoreCase("1") || isEpidemic.equalsIgnoreCase("1")) {
                            map1.put("healthState", "异常");
                        } else {
                            map1.put("healthState", "正常");
                        }
                        map.put("uploadState", "已上报");
                        map.put("uploadId", map1.get("id").toString());
                    }
                    map.put("recordList", mapList1);
                } else {
                    map.put("healthState", "未知");
                    map.put("uploadState", "未上报");
                    map.put("uploadId", "");
                    map.put("consumer", "");
                    map.put("consumerName", "");
                    map.put("createTime", "");
                    map.put("recordList", mapList1);
                }
            }
            JSONObject object = new JSONObject();
            object.put("time", date);
            object.put("list", mapList);
            array.add(object);
        }
        return array;
    }


    /**
     * 报表导出
     *
     * @param startDate
     * @param endDate
     * @return
     */
    public JSONArray selectListInfoExcelByTotal(OutputStream os, String startDate, String endDate, String dept) throws Exception {
        long days = DateUtil.getDays(endDate, startDate);
        List<String> list = new ArrayList<>();
        Date startTime = DateUtil.strToDateShort(startDate);
        for (int i = 0; i <= days; i++) {
            Date date = DateUtil.getPreDays(startTime, i);
            String str = DateUtil.dateToStrShort(date);
            list.add(str);
        }
        JSONArray array = new JSONArray();
        for (int i = 0; i < list.size(); i++) {
            String date = list.get(i);
            String doctorSql = "SELECT\n" +
                    "\td.idcard as idcard,\n" +
                    "\td.name as name,\n" +
                    "\td.mobile as mobile,\n" +
                    "\td.id_type as idType,\n" +
                    "\td.id as id,\n" +
                    "\tdhu.province_name as provinceName,\n" +
                    "\tdhu.city_name as cityName,\n" +
                    "\tdhu.town_name as townName,\n" +
                    "\tdhu.street_name as streetName,\n" +
                    "\tdhu.address as address,\n" +
                    "\tdhu.temperature as temperature,\n" +
                    "\tdhu.health_code as healthCode,\n" +
                    "\tdhu.tour_code as tourCode,\n" +
                    "\tdhu.is_symptoms as isSymtoms,\n" +
                    "\tdhu.is_epidemic as isEpidemic,\n" +
                    "\tdhu.is_outside as isOutside,\n" +
                    "\tdhu.is_nucleate as isNucleate,\n" +
                    "\tdhu.is_outside_high as isOutsideHigh, \n" +
                    "date_format(dhu.create_time ,'%Y-%m-%d %H:%i:%S' ) AS createTime " +
                    " FROM\n" +
                    "\tbase_doctor d\n" +
                    "LEFT JOIN base_doctor_health_upload dhu ON dhu.doctor_id = d.id WHERE dhu.create_time >= '" + date + " 00:00:00' AND dhu.create_time <= '" + date + " 23:59:59'  ";
            if (StringUtils.isNoneBlank(dept)) {
                doctorSql += " and d.id in (select dh.doctor_code from base_doctor_hospital dh where dh.dept_code = '" + dept + "' and dh.del=1 )";
            }
            List<Map<String, Object>> mapList = jdbcTemplate.queryForList(doctorSql);
            for (Map<String, Object> map : mapList) {
                String id = map.get("id").toString();
                List<BaseDoctorHospitalDO> baseDoctorHospitalDOS = baseDoctorHospitalDao.findByDoctorCode(id);
                if (baseDoctorHospitalDOS != null && baseDoctorHospitalDOS.size() != 0) {
                    map.put("deptName", baseDoctorHospitalDOS.get(0).getDeptName());
                }
            }
            JSONObject object = new JSONObject();
            object.put("time", date);
            object.put("list", mapList);
            array.add(object);
        }
        pushListWritePriceTotal(os, array, startDate, endDate);
        return array;
    }

    /**
     * mergeCells(a,b,c,d)
     * a 单元格的列号
     * <p>
     * b 单元格的行号
     * <p>
     * c 从单元格[a,b]起，向左合并到c列
     * <p>
     * d 从单元格[a,b]起，向下合并到d行
     * 收入日报导出
     *
     * @param os
     * @param ls
     * @param startTime
     * @param endTime
     * @throws Exception
     */
    public void pushListWritePriceTotal(OutputStream os, JSONArray ls, String startTime, String endTime) throws Exception {
        WritableWorkbook wwb = jxl.Workbook.createWorkbook(os);
        try {
            for (int i = 0; i < ls.size(); i++) {
                JSONObject jsonObject = ls.getJSONObject(i);
                String time = jsonObject.getString("time");
                WritableSheet ws;
                ws = wwb.createSheet(time, i + 1);
                String[] header = {"互联网综合服务平台健康上报情况", "", "", "", "", "", "", "", "", "", "", "", "", "", ""};//
                int k = 0;
                for (String h : header) {
                    addCell(ws, 0, k, h);//表名，行，列，header
                    k++;
                }

                String[] header2 = {"科室", "姓名", "电话", "身份类别", "上报状态", "所在地", "体温", "健康码", "行程码", "异常症状", "涉疫人员", "14天内从外地返", "核酸检测", "14天内有同往人从中高风险返厦", "健康状态", "上报时间"};//
                int k2 = 0;
                for (String h : header2) {
                    addCell(ws, 1, k2, h);//表名，行，列，header
                    k2++;
                }
                int j = 1;
                JSONArray array = jsonObject.getJSONArray("list");
                for (int z = 0; z < array.size(); z++) {
                    j = j + 1;
                    JSONObject object = array.getJSONObject(z);
                    addCell(ws, j, 0, object.getString("deptName"));//表名，行，列，header
                    addCell(ws, j, 1, object.getString("name"));//表名，行，列，header
                    addCell(ws, j, 2, object.getString("mobile"));//表名，行，列，header
                    if (StringUtils.isNoneBlank(object.getString("idType"))) {
                        addCell(ws, j, 3, object.getString("idType"));//表名，行，列，header
                    } else {
                        addCell(ws, j, 3, "");//表名，行，列，header
                    }
                    if (StringUtils.isNoneBlank(object.getString("temperature"))) {
                        String address = "";
                        if (StringUtils.isNoneBlank(object.getString("provinceName"))) {
                            address += object.getString("provinceName");
                        }
                        if (StringUtils.isNoneBlank(object.getString("cityName"))) {
                            address += object.getString("cityName");
                        }
                        if (StringUtils.isNoneBlank(object.getString("townName"))) {
                            address += object.getString("townName");
                        }
                        addCell(ws, j, 4, "已上报");//表名，行，列，header
                        addCell(ws, j, 5, address);//表名，行，列，header
                        addCell(ws, j, 6, object.getString("temperature"));//表名，行，列，header
                        String healthCodeName = "";
                        String healthStateName = "";
                        if (object.getString("healthCode").equalsIgnoreCase("1")) {
                            healthCodeName = "绿码";
                        } else if (object.getString("healthCode").equalsIgnoreCase("2")) {
                            healthCodeName = "红码";
                        } else if (object.getString("healthCode").equalsIgnoreCase("3")) {
                            healthCodeName = "黄码";
                        }
                        if (object.getString("healthCode").equalsIgnoreCase("2") || object.getString("healthCode").equalsIgnoreCase("3") ||
                                object.getDouble("temperature") >= 37.3 || object.getString("isSymtoms").equalsIgnoreCase("1") || object.getString("isEpidemic").equalsIgnoreCase("1")) {
                            healthStateName = "异常";
                        } else {
                            healthStateName = "正常";
                        }
                        String tourCodeName = "";
                        if (object.getString("tourCode").equalsIgnoreCase("1")) {
                            tourCodeName = "带星";
                        } else if (object.getString("tourCode").equalsIgnoreCase("2")) {
                            tourCodeName = "不带星";
                        }
                        String isSymtomsName = "";
                        if (object.getString("isSymtoms").equalsIgnoreCase("1")) {
                            isSymtomsName = "是";
                        } else if (object.getString("isSymtoms").equalsIgnoreCase("2")) {
                            isSymtomsName = "否";
                        }
                        String isEpidemicName = "";
                        if (object.getString("isEpidemic").equalsIgnoreCase("1")) {
                            isEpidemicName = "是";
                        } else if (object.getString("isEpidemic").equalsIgnoreCase("2")) {
                            isEpidemicName = "否";
                        }
                        String isOutsideName = "";
                        if (object.getString("isOutside").equalsIgnoreCase("1")) {
                            isOutsideName = "是";
                        } else if (object.getString("isOutside").equalsIgnoreCase("2")) {
                            isOutsideName = "否";
                        }
                        String isNucleateName = "";
                        if (wechatId.equalsIgnoreCase("xm_zjxl_wx")){
                            if (object.getString("isNucleate").equalsIgnoreCase("1")) {
                                isNucleateName = "未做";
                            } else if (object.getString("isNucleate").equalsIgnoreCase("2")) {
                                isNucleateName = "已做";
                            }
                        }else {
                            if (object.getString("isNucleate").equalsIgnoreCase("1")) {
                                isNucleateName = "未做";
                            } else if (object.getString("isNucleate").equalsIgnoreCase("2")) {
                                isNucleateName = "已做（院内）";
                            } else if (object.getString("isNucleate").equalsIgnoreCase("3")) {
                                isNucleateName = "已做（院外）";
                            }
                        }
                        String isOutsideHighName = "";
                        if (object.getString("isOutsideHigh").equalsIgnoreCase("1")) {
                            isOutsideHighName = "是";
                        } else if (object.getString("isOutsideHigh").equalsIgnoreCase("2")) {
                            isOutsideHighName = "否";
                        }
                        String createTime = "";
                        if (object.get("createTime") != null) {
                            createTime = object.getString("createTime");
                        }


                        addCell(ws, j, 7, healthCodeName);//表名，行，列，header
                        addCell(ws, j, 8, tourCodeName);//表名，行，列，header
                        addCell(ws, j, 9, isSymtomsName);//表名，行，列，header
                        addCell(ws, j, 10, isEpidemicName);//表名，行，列，header
                        addCell(ws, j, 11, isOutsideName);//表名，行，列，header
                        addCell(ws, j, 12, isNucleateName);//表名，行，列，header
                        addCell(ws, j, 13, isOutsideHighName);//表名，行，列，header
                        addCell(ws, j, 14, healthStateName);//表名，行，列，header
                        addCell(ws, j, 15, createTime);//表名，行，列，header
                    } else {
                        addCell(ws, j, 4, "未上报");//表名，行，列，header
                        addCell(ws, j, 5, "");//表名，行，列，header
                        addCell(ws, j, 6, "");//表名，行，列，header
                        addCell(ws, j, 7, "");//表名，行，列，header
                        addCell(ws, j, 8, "");//表名，行，列，header
                        addCell(ws, j, 9, "");//表名，行，列，header
                        addCell(ws, j, 10, "");//表名，行，列，header
                        addCell(ws, j, 11, "");//表名，行，列，header
                        addCell(ws, j, 12, "");//表名，行，列，header
                        addCell(ws, j, 13, "");//表名，行，列，header
                        addCell(ws, j, 14, "");//表名，行，列，header
                        addCell(ws, j, 15, "");//表名，行，列，header
                    }
                }
                ws.mergeCells(0, 0, 15, 0);
            }
            wwb.write();
            wwb.close();
        } catch (IOException e) {
            e.printStackTrace();
            if (wwb != null) wwb.close();
            throw e;
        }
    }

    //添加单元格内容
    public void addCell(WritableSheet ws, int row, int column, String data) throws WriteException {
        Label label = new Label(column, row, data);
        WritableCellFormat cellFormat = new WritableCellFormat();
        cellFormat.setAlignment(jxl.format.Alignment.CENTRE);
        cellFormat.setVerticalAlignment(jxl.format.VerticalAlignment.CENTRE);
        label.setCellFormat(cellFormat);
        ws.addCell(label);
    }


    /**
     * 查下最新一次记录
     *
     * @param doctorId
     * @param consumer
     * @return
     */
    public BaseDoctorHealthUploadDO selectRecordByDoctorIdAndConsumer(String doctorId, String consumer) {
        BaseDoctorHealthUploadDO baseDoctorHealthUploadDO = null;
        List<BaseDoctorHealthUploadDO> doctorHealthUploadDOList = baseDoctorHealthUploadDao.findByDoctorIdAndConsumer(doctorId, consumer);
        if (doctorHealthUploadDOList != null && doctorHealthUploadDOList.size() != 0) {
            baseDoctorHealthUploadDO = doctorHealthUploadDOList.get(0);
        }
        return baseDoctorHealthUploadDO;
    }


    /**
     *
     * @param orgCode
     * @param dept
     * @param doctorCode
     * @param key
     * @param flag 是否异常1是0否
     * @return
     */
    public List<Map<String, Object>> findDoctorByHospitalAndDept(String orgCode, String dept, String doctorCode,String key, Integer flag) {
        String sql = "SELECT " +
                " d.id AS \"id\", " +
                " d.photo AS \"photo\", " +
                " d.name AS \"name\", " +
                " d.expertise AS \"expertise\"," +
                " d.introduce AS \"introduce\"," +
                " d.job_title_code AS \"jobTitleCode\", " +
                " d.job_title_name AS \"jobTitleName\"," +
                " d.charge_type AS \"chargeType\"," +
                " d.outpatient_type AS \"outpatientType\"," +
                " d.consult_status AS \"consultStatus\"," +
                " h.dept_code AS \"deptCode\","+
                " h.dept_name AS \"deptName\","+
                " h.org_code AS \"orgCode\","+
                " h.org_name AS \"orgName\","+
                " t.photo AS \"hosptialphoto\","+
                " d.id_type AS \"idType\","+
                " d.id_card_type AS \"idCardType\" "+
                " FROM " +
                " base_doctor d " +
                " JOIN base_doctor_hospital h ON h.doctor_code = d.id " +
                " left join base_org t on t.id = h.org_code" +
                " WHERE  " +
                " 1=1 and d.del=1 ";
        Map<String, Object> params = new HashedMap();
        if (StringUtils.isNotBlank(dept)) {
            sql += " AND h.dept_code = '" + dept + "'";
        }
        if (StringUtils.isNotBlank(orgCode)) {
            sql += " AND h.org_code = '" + orgCode + "'";
        }
        if (StringUtils.isNotBlank(key)) {
            sql += " AND (h.dept_name like '%" + key + "%' OR d.name like '%" + key + "%' OR d.expertise like '%" + key + "%') ";
        }
        List<Map<String, Object>> list = hibenateUtils.createSQLQuery(sql, params);
        for (Map<String,Object> map:list){
            String id = map.get("id").toString();
            Date startTime = DateUtil.strToDateLong(DateUtil.getStringDateShort()+" 00:00:00");
            Date endTime = DateUtil.strToDateLong(DateUtil.getStringDateShort()+" 23:59:59");
            List<BaseDoctorHealthUploadDO> healthUploadDOS = baseDoctorHealthUploadDao.selectHealthUploadInfoByDoctorIdAndStartAndEnd(id,startTime,endTime);
            if (healthUploadDOS!=null&&healthUploadDOS.size()!=0){
                BaseDoctorHealthUploadDO healthUploadDO = healthUploadDOS.get(0);
                String healthStateName = "";
                if (healthUploadDO.getHealthCode()==2 ||healthUploadDO.getHealthCode()==3||Double.parseDouble(healthUploadDO.getTemperature())>=37.3
                        || healthUploadDO.getIsSymptoms()==1 || healthUploadDO.getIsEpidemic()==1) {
                    healthStateName = "异常";
                } else {
                    healthStateName = "正常";
                }
                map.put("healthStateName",healthStateName);
                map.put("healthUpload",healthUploadDO);
            }else {
                map.put("healthUpload",null);
            }
        }
        if (list != null && list.size() > 0 && StringUtils.isNotBlank(doctorCode)) {
            //排序
            for (int i = 0; i < list.size(); i++) {
                String id = (String) list.get(i).get("id");
                if (doctorCode.equals(id)) {
                    Collections.swap(list, 0, i);
                }
            }
        }

        return list;
    }

    /**
     * 首页头部统计
     *
     * @param dept
     * @param level  1院级管理员，2科室管理员
     * @return
     */
    public JSONObject selectIndexHeader(String dept, int level) {
        JSONObject object = new JSONObject();
        String day = "0";
        String doctorSql = " select COUNT(1) as \"total\" from base_doctor d where d.del=1 ";
        Integer total = 0;//总人次
        if (level==2){
            doctorSql +=" and d.id IN (select dh.doctor_code from base_doctor_hospital dh where dh.dept_code = '" + dept + "' and dh.del=1 ) ";
        }
        Map<String, Object> map = jdbcTemplate.queryForMap(doctorSql);
        String doctorTotal = "";
        if (map != null) {
            doctorTotal = map.get("total").toString();
        }
        if (StringUtils.isNoneBlank(doctorTotal)) {
            //总人次
            if (!day.equalsIgnoreCase("0")) {
                total = Integer.parseInt(doctorTotal) * Integer.parseInt(day);
            } else {
                total = Integer.parseInt(doctorTotal);
            }
        }
        String sql = "SELECT\n" +
                "\tCOUNT(DISTINCT d.doctor_id) AS total\n" +
                "FROM\n" +
                "\t(\n" +
                "\t\tSELECT\n" +
                "\t\t\td3.*\n" +
                "\t\tFROM\n" +
                "\t\t\t(\n" +
                "\t\t\t\tSELECT\n" +
                "\t\t\t\t\t*, MAX(d1.create_time) AS createTime\n" +
                "\t\t\t\tFROM\n" +
                "\t\t\t\t\tbase_doctor_health_upload d1\n" +
                "\t\t\t\tGROUP BY\n" +
                "\t\t\t\t\td1.doctor_id\n" +
                "\t\t\t) d2\n" +
                "\t\tLEFT JOIN base_doctor_health_upload d3 ON d2.doctor_id = d3.doctor_id\n" +
                "\t\tAND d2.createTime = d3.create_time\n" +
                "\t) d ";
        String highSql = sql +" where d.is_high=1 ";
        String redSql = sql +" where d.health_code=2 ";
        String yellowSql = sql +" where d.health_code=3 ";
        if (level==2){
            highSql +=" and d.doctor_id IN (select dh.doctor_code from base_doctor_hospital dh where dh.dept_code = '" + dept + "' and dh.del=1 ) ";
            redSql +=" and d.doctor_id IN (select dh.doctor_code from base_doctor_hospital dh where dh.dept_code = '" + dept + "' and dh.del=1 ) ";
            yellowSql +=" and d.doctor_id IN (select dh.doctor_code from base_doctor_hospital dh where dh.dept_code = '" + dept + "' and dh.del=1 ) ";
        }
        Map<String, Object> highMap = jdbcTemplate.queryForMap(highSql);
        String highTotals = "";
        Integer highTotal = 0;//中高风险统计
        if (highMap != null) {
            highTotals = highMap.get("total").toString();
        }
        if (StringUtils.isNoneBlank(highTotals)) {
            //中高风险统计
            highTotal = Integer.parseInt(highTotals);
        }
        Map<String, Object> redMap = jdbcTemplate.queryForMap(redSql);
        String redTotals = "";
        Integer redTotal = 0;//红码统计
        if (redMap != null) {
            redTotals = redMap.get("total").toString();
        }
        if (StringUtils.isNoneBlank(redTotals)) {
            //红码统计
            redTotal = Integer.parseInt(redTotals);
        }
        Map<String, Object> yellowMap = jdbcTemplate.queryForMap(yellowSql);
        String yellowTotals = "";
        Integer yellowTotal = 0;//黄码统计
        if (yellowMap != null) {
            yellowTotals = yellowMap.get("total").toString();
        }
        if (StringUtils.isNoneBlank(yellowTotals)) {
            //黄码统计
            yellowTotal = Integer.parseInt(yellowTotals);
        }
        object.put("total", total);//总人次
        object.put("highTotal", highTotal);//中高风险统计
        object.put("redTotal", redTotal);//红码统计
        object.put("yellowTotal", yellowTotal);//黄码统计
        return object;
    }


    /**
     * 首页出差科室统计
     * @param dept
     * @param level  1院级管理员，2科室管理员
     * @return
     */
    public List<Map<String,Object>> selectIndexScheduleDept(String dept, int level) {
        String condition = "";
        if (level==2){
            condition +=" and dh.dept_code = '"+dept+"'";
        }
        String sql = "SELECT\n" +
                "\thd.name as deptName,\n" +
                "\tCOUNT(DISTINCT d.doctor_id) AS total\n" +
                "FROM\n" +
                "\t(\n" +
                "\t\tSELECT\n" +
                "\t\t\td3.*\n" +
                "\t\tFROM\n" +
                "\t\t\t(\n" +
                "\t\t\t\tSELECT\n" +
                "\t\t\t\t\t*, MAX(d1.create_time) AS createTime\n" +
                "\t\t\t\tFROM\n" +
                "\t\t\t\t\tbase_doctor_health_upload d1\n" +
                "\t\t\t\tGROUP BY\n" +
                "\t\t\t\t\td1.doctor_id\n" +
                "\t\t\t) d2\n" +
                "\t\tLEFT JOIN base_doctor_health_upload d3 ON d2.doctor_id = d3.doctor_id\n" +
                "\t\tAND d2.createTime = d3.create_time\n" +
                "\t) d\n" +
                "LEFT JOIN base_doctor_hospital dh ON dh.doctor_code = d.doctor_id and dh.del=1 \n" +
                " LEFT JOIN dict_hospital_dept hd ON dh.dept_code = hd.code "+
                "where d.schedule_status IN(2,3)  "+condition +
                "GROUP BY dh.dept_code \n" +
                "ORDER BY COUNT(DISTINCT d.doctor_id) DESC  ";
        List<Map<String,Object>> mapList = jdbcTemplate.queryForList(sql);
        return mapList;
    }


    /**
     * 首页出差分布统计
     *
     * @param dept
     * @param level  1院级管理员，2科室管理员
     * @return
     */
    public List<Map<String,Object>> selectIndexScheduleCityName(String dept, int level) {
        String condition = "";
        if (level==2){
            condition +=" and dh.dept_code = '"+dept+"'";
        }
        String sql = "SELECT\n" +
                "\td.city_name  as cityName,\n" +
                "\tCOUNT(DISTINCT d.doctor_id) AS total\n" +
                "FROM\n" +
                "\t(\n" +
                "\t\tSELECT\n" +
                "\t\t\td3.*\n" +
                "\t\tFROM\n" +
                "\t\t\t(\n" +
                "\t\t\t\tSELECT\n" +
                "\t\t\t\t\t*, MAX(d1.create_time) AS createTime\n" +
                "\t\t\t\tFROM\n" +
                "\t\t\t\t\tbase_doctor_health_upload d1\n" +
                "\t\t\t\tGROUP BY\n" +
                "\t\t\t\t\td1.doctor_id\n" +
                "\t\t\t) d2\n" +
                "\t\tLEFT JOIN base_doctor_health_upload d3 ON d2.doctor_id = d3.doctor_id\n" +
                "\t\tAND d2.createTime = d3.create_time\n" +
                "\t) d\n" +
                "LEFT JOIN base_doctor_hospital dh ON dh.doctor_code = d.doctor_id and dh.del=1 \n" +
                " LEFT JOIN dict_hospital_dept hd ON dh.dept_code = hd.code "+
                "where  d.schedule_status IN(2,3)\n" +condition+
                "GROUP BY d.city_name \n" +
                "ORDER BY COUNT(DISTINCT d.doctor_id) DESC ";
        List<Map<String,Object>> mapList = jdbcTemplate.queryForList(sql);
        return mapList;
    }

    /**
     * 首页本地分布统计
     * @param dept
     * @param level  1院级管理员，2科室管理员
     * @return
     */
    public List<Map<String,Object>> selectIndexScheduleTown(String dept, int level) {
        String condition = "";
        if (level==2){
            condition +=" and dh.dept_code = '"+dept+"'";
        }
        String sql = "SELECT\n" +
                "\td.town_name  as townName,\n" +
                "\tCOUNT(DISTINCT d.doctor_id) AS total\n" +
                "FROM\n" +
                "\t(\n" +
                "\t\tSELECT\n" +
                "\t\t\td3.*\n" +
                "\t\tFROM\n" +
                "\t\t\t(\n" +
                "\t\t\t\tSELECT\n" +
                "\t\t\t\t\t*, MAX(d1.create_time) AS createTime\n" +
                "\t\t\t\tFROM\n" +
                "\t\t\t\t\tbase_doctor_health_upload d1\n" +
                "\t\t\t\tGROUP BY\n" +
                "\t\t\t\t\td1.doctor_id\n" +
                "\t\t\t) d2\n" +
                "\t\tLEFT JOIN base_doctor_health_upload d3 ON d2.doctor_id = d3.doctor_id\n" +
                "\t\tAND d2.createTime = d3.create_time\n" +
                "\t) d\n" +
                "LEFT JOIN base_doctor_hospital dh ON dh.doctor_code = d.doctor_id and dh.del=1 \n" +
                " LEFT JOIN dict_hospital_dept hd ON dh.dept_code = hd.code "+
                "where  hd.name IS NOT NULL and d.schedule_status = 1 and d.town_name  !=''\n" + condition +
                "GROUP BY d.town_name \n" +
                "ORDER BY COUNT(DISTINCT d.doctor_id) DESC ";
        List<Map<String,Object>> mapList = jdbcTemplate.queryForList(sql);
        return mapList;
    }

    /**
     * 首页异常实时情况
     * @param dept
     * @param level  1院级管理员，2科室管理员
     *
     * @return
     */
    public List<Map<String,Object>> selectIndexYichangList(String dept, int level,String startDate,String endDate,String doctorName) {
        String condition = "";
        if (level==2){
            condition +=" and dh.dept_code = '"+dept+"'";
        }
        if (StringUtils.isNoneBlank(startDate)){
            condition +=" and d.create_time >= '"+startDate+" 00:00:00'";
            condition +=" and d.create_time <= '"+endDate+" 23:59:59'";
        }
        if (StringUtils.isNoneBlank(doctorName)){
            condition +=" and d.doctor_name like '%"+doctorName+"%'";
        }
        String sql = "SELECT\n" +
                "\tdh.dept_name AS deptName,\n" +
                "\td.doctor_name AS doctorName,\n" +
                "\tbd.photo AS photo,\n" +
                "\td.id AS id,\n" +
                "\td.doctor_id AS doctorId,\n" +
                "\td.schedule_status AS scheduleStatus,\n" +
                "\td.province_name AS provinceName,\n" +
                "\td.city_name AS cityName,\n" +
                "\td.town_name AS townName,\n" +
                "\td.tour_code AS tourCode,\n" +
                "\td.health_code AS healthCode,\n" +
                "\td.temperature AS temperature,\n" +
                "\td.is_symptoms AS isSymptoms,\n" +
                "\td.is_epidemic AS isEpidemic,\n" +
                "\td.abnormal_symptoms AS abnormalSymptoms,\n" +
                "\td.abnormal_symptoms_other AS abnormalSymptomsOther,\n" +
                "\td.epidemic_person AS epidemicPerson,\n" +
                "\tdate_format(\n" +
                "\t\td.create_time,\n" +
                "\t\t'%Y-%m-%d %H:%i:%S'\n" +
                "\t) AS createTime\n" +
                "FROM\n" +
                "\t(\n" +
                "\t\tSELECT\n" +
                "\t\t\td3.*\n" +
                "\t\tFROM\n" +
                "\t\t\t(\n" +
                "\t\t\t\tSELECT\n" +
                "\t\t\t\t\t*, MAX(d1.create_time) AS createTime\n" +
                "\t\t\t\tFROM\n" +
                "\t\t\t\t\tbase_doctor_health_upload d1\n" +
                "\t\t\t\tGROUP BY\n" +
                "\t\t\t\t\td1.doctor_id\n" +
                "\t\t\t) d2\n" +
                "\t\tLEFT JOIN base_doctor_health_upload d3 ON d2.doctor_id = d3.doctor_id\n" +
                "\t\tAND d2.createTime = d3.create_time\n" +
                "\t) d\n" +
                "LEFT JOIN base_doctor_hospital dh ON dh.doctor_code = d.doctor_id and dh.del=1 " +
                "LEFT JOIN base_doctor bd ON bd.id=dh.doctor_code "+
                "WHERE\n" +
                "\t(\n" +
                "\t\td.health_code IN (2, 3)\n" +
                "\t\tOR CAST(d.temperature AS DECIMAL) >= 37.3\n" +
                "\t\tOR d.is_symptoms = 1\n" +
                "\t\tOR d.is_epidemic = 1\n" +
                "\t)\n" +
                "AND dh.dept_code IS NOT NULL\n" + condition+
                "ORDER BY\n" +
                "\td.create_time DESC ";
        List<Map<String,Object>> mapList = jdbcTemplate.queryForList(sql);
        return mapList;
    }

    /**
     * 首页核酸检测情况
     * @param dept
     * @param level  1院级管理员，2科室管理员
     * @return
     */
    public JSONObject selectIndexNucleicTotal(String dept, int level) {
        String condition = "";
        if (level==2){
            condition +=" and dh.dept_code = '"+dept+"'";
        }
        //已做核酸
        String yizuohesuanSql = "SELECT\n" +
                "\tCOUNT(DISTINCT d.doctor_id) as total\n" +
                "FROM\n" +
                "\t(\n" +
                "\t\tSELECT\n" +
                "\t\t\td3.*\n" +
                "\t\tFROM\n" +
                "\t\t\t(\n" +
                "\t\t\t\tSELECT\n" +
                "\t\t\t\t\t*, MAX(d1.create_time) AS createTime\n" +
                "\t\t\t\tFROM\n" +
                "\t\t\t\t\tbase_doctor_health_upload d1\n" +
                "\t\t\t\tGROUP BY\n" +
                "\t\t\t\t\td1.doctor_id\n" +
                "\t\t\t) d2\n" +
                "\t\tLEFT JOIN base_doctor_health_upload d3 ON d2.doctor_id = d3.doctor_id\n" +
                "\t\tAND d2.createTime = d3.create_time\n" +
                "\t) d\n" +
                "LEFT JOIN base_doctor_hospital dh ON dh.doctor_code = d.doctor_id and dh.del=1 \n" + condition+
                "where d.is_nucleate !=1 and dh.dept_code is not null ";
        Map<String,Object> yizuoMap = jdbcTemplate.queryForMap(yizuohesuanSql);
        String yizuoTotals = "";
        Integer yizuoTotal = 0;//已做核酸统计
        if (yizuoMap != null) {
            yizuoTotals = yizuoMap.get("total").toString();
        }
        if (StringUtils.isNoneBlank(yizuoTotals)) {
            //已做核酸统计
            yizuoTotal = Integer.parseInt(yizuoTotals);
        }
        //未做核酸
        String weizuohesuanSql = "SELECT\n" +
                "\tCOUNT(DISTINCT d.doctor_id) as total\n" +
                "FROM\n" +
                "\t(\n" +
                "\t\tSELECT\n" +
                "\t\t\td3.*\n" +
                "\t\tFROM\n" +
                "\t\t\t(\n" +
                "\t\t\t\tSELECT\n" +
                "\t\t\t\t\t*, MAX(d1.create_time) AS createTime\n" +
                "\t\t\t\tFROM\n" +
                "\t\t\t\t\tbase_doctor_health_upload d1\n" +
                "\t\t\t\tGROUP BY\n" +
                "\t\t\t\t\td1.doctor_id\n" +
                "\t\t\t) d2\n" +
                "\t\tLEFT JOIN base_doctor_health_upload d3 ON d2.doctor_id = d3.doctor_id\n" +
                "\t\tAND d2.createTime = d3.create_time\n" +
                "\t) d\n" +
                "LEFT JOIN base_doctor_hospital dh ON dh.doctor_code = d.doctor_id and dh.del=1 \n" +condition+
                "where d.is_nucleate =1 and dh.dept_code is not null \n";
        Map<String,Object> weizuoMap = jdbcTemplate.queryForMap(weizuohesuanSql);
        String weizuoTotals = "";
        Integer weizuoTotal = 0;//未做核酸统计
        if (weizuoMap != null) {
            weizuoTotals = weizuoMap.get("total").toString();
        }
        if (StringUtils.isNoneBlank(weizuoTotals)) {
            //未做核酸统计
            weizuoTotal = Integer.parseInt(weizuoTotals);
        }
        JSONObject object = new JSONObject();
        object.put("weizuoTotal",weizuoTotal);//未做核酸统计
        object.put("yizuoTotal",yizuoTotal);//已做核酸统计
        return object;
    }


    /**
     * 首页当日新增统计
     * @param dept
     * @param level  1院级管理员，2科室管理员
     * @return
     */
    public JSONObject selectIndexNewAdd(String dept, int level) {
        String condition = "";
        if (level==2){
            condition +=" and dh.dept_code = '"+dept+"'";
        }
        String startDate = DateUtil.getStringDateShort()+" 00:00:00";
        String endDate = DateUtil.getStringDateShort()+" 23:59:59";
        JSONObject object = new JSONObject();
        String sql = "SELECT\n" +
                "\tCOUNT(DISTINCT d.doctor_id) AS total\n" +
                "FROM\n" +
                "\t(\n" +
                "\t\tSELECT\n" +
                "\t\t\td3.*\n" +
                "\t\tFROM\n" +
                "\t\t\t(\n" +
                "\t\t\t\tSELECT\n" +
                "\t\t\t\t\t*, MAX(d1.create_time) AS createTime\n" +
                "\t\t\t\tFROM\n" +
                "\t\t\t\t\tbase_doctor_health_upload d1\n" +
                "\t\t\t\tGROUP BY\n" +
                "\t\t\t\t\td1.doctor_id\n" +
                "\t\t\t) d2\n" +
                "\t\tLEFT JOIN base_doctor_health_upload d3 ON d2.doctor_id = d3.doctor_id \n" +
                "\t\tAND d2.createTime = d3.create_time\n" +
                "\t) d "+
                "LEFT JOIN base_doctor_hospital dh ON dh.doctor_code = d.doctor_id and dh.del=1 \n where 1=1 and dh.dept_code is not null  "+condition+" ";
        String highSql = sql +" AND d.is_high=1 and d.create_time >='"+startDate+"' and d.create_time <='"+endDate+"'";
        String redSql = sql +" AND d.health_code=2 and d.create_time >='"+startDate+"' and d.create_time <='"+endDate+"'";
        String yellowSql = sql +" AND d.health_code=3 and d.create_time >='"+startDate+"' and d.create_time <='"+endDate+"'";
        String greenSql = sql +" AND d.health_code=1 and d.create_time >='"+startDate+"' and d.create_time <='"+endDate+"'";
        Map<String, Object> highMap = jdbcTemplate.queryForMap(highSql);
        String highTotals = "";
        Integer highTotal = 0;//中高风险统计
        if (highMap != null) {
            highTotals = highMap.get("total").toString();
        }
        if (StringUtils.isNoneBlank(highTotals)) {
            //中高风险统计
            highTotal = Integer.parseInt(highTotals);
        }
        Map<String, Object> redMap = jdbcTemplate.queryForMap(redSql);
        String redTotals = "";
        Integer redTotal = 0;//红码统计
        if (redMap != null) {
            redTotals = redMap.get("total").toString();
        }
        if (StringUtils.isNoneBlank(redTotals)) {
            //红码统计
            redTotal = Integer.parseInt(redTotals);
        }
        Map<String, Object> yellowMap = jdbcTemplate.queryForMap(yellowSql);
        String yellowTotals = "";
        Integer yellowTotal = 0;//黄码统计
        if (yellowMap != null) {
            yellowTotals = yellowMap.get("total").toString();
        }
        if (StringUtils.isNoneBlank(yellowTotals)) {
            //黄码统计
            yellowTotal = Integer.parseInt(yellowTotals);
        }
        Map<String, Object> greenMap = jdbcTemplate.queryForMap(greenSql);
        String greenTotals = "";
        Integer greenTotal = 0;//中高风险统计
        if (greenMap != null) {
            greenTotals = greenMap.get("total").toString();
        }
        if (StringUtils.isNoneBlank(greenTotals)) {
            //中高风险统计
            greenTotal = Integer.parseInt(greenTotals);
        }
        object.put("highTotal", highTotal);//中高风险统计
        object.put("redTotal", redTotal);//红码统计
        object.put("yellowTotal", yellowTotal);//黄码统计
        object.put("greenTotal", greenTotal);//绿码统计
        return object;
    }


    /**
     * 首页上报分析
     *
     * @param flag 1周2月
     * @param dept
     * @param level  1院级管理员，2科室管理员
     * @return
     */
    public JSONObject selectIndexTotalCircle(Integer flag,String dept, int level) {
        JSONObject object = new JSONObject();
        String startDate="";
        String endDate="";
        if (flag==1){
            startDate = DateUtil.getMondayOfThisWeek();
            endDate = DateUtil.getSundayOfThisWeek();
        }else if (flag==2){
            startDate = DateUtil.getFristDayOfMonth();
            endDate = DateUtil.getLastDayOfMonth();
        }
        String day = DateUtil.getTwoDay(endDate, startDate);
        String doctorSql = " select COUNT(1) as \"total\" from base_doctor d where d.del=1 ";
        Integer total = 0;//总人次
        String sqlCondition = "";
        if (level==2){
            sqlCondition =" and d.doctor_id IN (select dh.doctor_code from base_doctor_hospital dh where dh.dept_code = '" + dept + "' and dh.del=1 ) ";
        }
        String sql = "SELECT SUM(d1.total) as total from (select COUNT(DISTINCT d.doctor_id) as \"total\",date_format(d.create_time ,'%Y-%m-%d' ) from base_doctor_health_upload d where 1=1 "+sqlCondition+" and  d.create_time >='" + startDate + " 00:00:00' and d.create_time<='" + endDate + " 23:59:59' GROUP BY date_format(d.create_time ,'%Y-%m-%d' ))d1 ";
        if (level==2){
            doctorSql +=" and d.id IN (select dh.doctor_code from base_doctor_hospital dh where dh.dept_code = '" + dept + "' and dh.del=1 ) ";
        }
        Map<String, Object> map = jdbcTemplate.queryForMap(doctorSql);
        String doctorTotal = "";
        if (map != null) {
            doctorTotal = map.get("total").toString();
        }
        if (StringUtils.isNoneBlank(doctorTotal)) {
            //总人次
            if (!day.equalsIgnoreCase("0")) {
                total = Integer.parseInt(doctorTotal) * Integer.parseInt(day);
            } else {
                total = Integer.parseInt(doctorTotal);
            }
        }
        Integer yishangbaoTotal = 0;
        Integer weishangbaoTotal = 0;
        Integer zhengchangTotal = 0;
        Integer yichangTotal = 0;
        //已上传
        Map<String, Object> yishangchaungMap = jdbcTemplate.queryForMap(sql);
        if (yishangchaungMap != null) {
            if (yishangchaungMap.get("total") != null) {
                yishangbaoTotal = Integer.parseInt(yishangchaungMap.get("total").toString());
            }
        }
        //未上报
        weishangbaoTotal = total - yishangbaoTotal;


        //上传总次数
        Integer shangchuanTotal = 0;

        String shangchuanTotalSql = "select COUNT(1) as \"total\" from base_doctor_health_upload d where 1=1  ";

        if (StringUtils.isNoneBlank(startDate)) {
            shangchuanTotalSql += " and d.create_time >='" + startDate + " 00:00:00' and d.create_time<='" + endDate + " 23:59:59' ";
        }
        if (level==2){
            shangchuanTotalSql +=" and d.doctor_id IN (select dh.doctor_code from base_doctor_hospital dh where dh.dept_code = '" + dept + "' and dh.del=1 ) ";
        }
        Map<String, Object> shangchuanMap = jdbcTemplate.queryForMap(shangchuanTotalSql);
        if (shangchuanMap != null) {
            if (shangchuanMap.get("total") != null) {
                shangchuanTotal = Integer.parseInt(shangchuanMap.get("total").toString());
            }
        }


        //正常、异常
        String sqlZhengchang = "select COUNT(1) as \"total\" from base_doctor_health_upload d where 1=1 and (d.health_code not in(2,3) AND CAST(d.temperature as DECIMAL)<37.3 AND  d.is_symptoms=2 and  d.is_epidemic=2) ";

        if (StringUtils.isNoneBlank(startDate)) {
            sqlZhengchang += " and d.create_time >='" + startDate + " 00:00:00' and d.create_time<='" + endDate + " 23:59:59' ";
        }
        if (level==2){
            sqlZhengchang +=" and d.doctor_id IN (select dh.doctor_code from base_doctor_hospital dh where dh.dept_code = '" + dept + "' and dh.del=1 ) ";
        }
        //正常
        Map<String, Object> zhengchangMap = jdbcTemplate.queryForMap(sqlZhengchang);
        if (zhengchangMap != null) {
            if (zhengchangMap.get("total") != null) {
                zhengchangTotal = Integer.parseInt(zhengchangMap.get("total").toString());
            }
        }

        String sqlYichang = "select COUNT(1) as \"total\" from base_doctor_health_upload d where 1=1 and (d.health_code in(2,3) OR CAST(d.temperature as DECIMAL)>=37.3 OR  d.is_symptoms=1 OR d.is_epidemic=1)  ";
        if (StringUtils.isNoneBlank(startDate)) {
            sqlYichang += " and d.create_time >='" + startDate + " 00:00:00' and d.create_time<='" + endDate + " 23:59:59' ";
        }
        if (level==2){
            sqlYichang +=" and d.doctor_id IN (select dh.doctor_code from base_doctor_hospital dh where dh.dept_code = '" + dept + "' and dh.del=1 ) ";
        }
        //正常
        Map<String, Object> yichangMap = jdbcTemplate.queryForMap(sqlYichang);
        if (yichangMap != null) {
            if (yichangMap.get("total") != null) {
                yichangTotal = Integer.parseInt(yichangMap.get("total").toString());
            }
        }
        object.put("total", total);//总人次
        object.put("yishangbaoTotal", yishangbaoTotal);//已上报
        object.put("weishangbaoTotal", weishangbaoTotal);//未上报
        object.put("zhengchangTotal", zhengchangTotal);//上报正常
        object.put("yichangTotal", yichangTotal);//上报异常
        object.put("shangchuanTotal", shangchuanTotal);//上传总次数
        return object;
    }


    /**
     * 首页未报科室统计
     * @param dept
     * @param level  1院级管理员，2科室管理员
     * @return
     */
    public List<Map<String, Object>> selectIndexDeptList(String dept, int level) {
        String startDate = DateUtil.getStringDateShort();
        String condition = "";
        if (StringUtils.isNoneBlank(startDate)) {
            condition = "  and d.create_time >='" + startDate + " 00:00:00' and d.create_time<='" + startDate + " 23:59:59'  ";
        }
        String deptSql = "SELECT\n" +
                "\tcode,\n" +
                "\tname,\n" +
                "\tdh1.total,\n" +
                "\tdh2.total as yishangbaoTotal,\n" +
                "\tdh3.total as shangchuanTotal,\n" +
                "\tdh4.total as zhengchangTotal,\n" +
                "\tdh5.total as yichangTotal,\n" +
                "\tdh6.total as yichangPersonTotal\n" +
                "FROM\n" +
                "\tdict_hospital_dept hd\n" +
                "LEFT JOIN (\n" +
                "\tSELECT\n" +
                "\t\tdh.dept_code,\n" +
                "\t\tCOUNT(1) AS \"total\"\n" +
                "\tFROM\n" +
                "\t\tbase_doctor d\n" +
                "\tLEFT JOIN base_doctor_hospital dh ON dh.doctor_code = d.id\n" +
                "\tWHERE\n" +
                "\t\td.del = 1\n" +
                "\tAND dh.del = 1\n" +
                "\tGROUP BY\n" +
                "\t\tdh.dept_code\n" +
                ") dh1 ON hd. CODE = dh1.dept_code\n" +
                "LEFT JOIN (\n" +
                "\tSELECT\n" +
                "\t\tdh.dept_code,\n" +
                "\t\tCOUNT(DISTINCT d.doctor_id) AS total\n" +
                "\tFROM\n" +
                "\t\tbase_doctor_health_upload d\n" +
                "\tLEFT JOIN base_doctor_hospital dh ON dh.doctor_code = d.doctor_id and dh.del=1 \n" +
                "\tWHERE\n" +
                "\t\t1 = 1\n" +
                condition+
                "\tGROUP BY\n" +
                "\t\tdh.dept_code\n" +
                ")dh2 ON dh2.dept_code=hd.code\n" +
                "LEFT JOIN (\n" +
                "\tSELECT\n" +
                "\t\tdh.dept_code,\n" +
                "\t\tCOUNT(1) AS total\n" +
                "\tFROM\n" +
                "\t\tbase_doctor_health_upload d\n" +
                "\tLEFT JOIN base_doctor_hospital dh ON dh.doctor_code = d.doctor_id and dh.del=1 \n" +
                "\tWHERE\n" +
                "\t\t1 = 1\n" +
                condition+
                "\tGROUP BY\n" +
                "\t\tdh.dept_code\n" +
                ")dh3 ON dh3.dept_code=hd.code\n" +
                "LEFT JOIN (\n" +
                "\tSELECT\n" +
                "\t\tdh.dept_code,\n" +
                "\t\tCOUNT(1) AS total\n" +
                "\tFROM\n" +
                "\t\tbase_doctor_health_upload d\n" +
                "\tLEFT JOIN base_doctor_hospital dh ON dh.doctor_code = d.doctor_id and dh.del=1 \n" +
                "\tWHERE\n" +
                "\t\t1 = 1\n" +
                condition+
                "\tand (d.health_code not in(2,3) AND CAST(d.temperature as DECIMAL)<37.3 AND  d.is_symptoms=2 and  d.is_epidemic=2)\n" +
                "\tGROUP BY\n" +
                "\t\tdh.dept_code\n" +
                ")dh4 ON dh4.dept_code=hd.code\n" +
                "LEFT JOIN (\n" +
                "\tSELECT\n" +
                "\t\tdh.dept_code,\n" +
                "\t\tCOUNT(1) AS total\n" +
                "\tFROM\n" +
                "\t\tbase_doctor_health_upload d\n" +
                "\tLEFT JOIN base_doctor_hospital dh ON dh.doctor_code = d.doctor_id and dh.del=1 \n" +
                "\tWHERE\n" +
                "\t\t1 = 1\n" +
                condition+
                "\tand (d.health_code in(2,3) OR CAST(d.temperature as DECIMAL)>=37.3 OR  d.is_symptoms=1 OR d.is_epidemic=1)\n" +
                "\tGROUP BY\n" +
                "\t\tdh.dept_code\n" +
                ")dh5 ON dh5.dept_code=hd.code\n" +
                "LEFT JOIN (\n" +
                "\tSELECT\n" +
                "\t\tdh.dept_code,\n" +
                "\t\tCOUNT(DISTINCT d.doctor_id) AS total\n" +
                "\tFROM\n" +
                "\t\tbase_doctor_health_upload d\n" +
                "\tLEFT JOIN base_doctor_hospital dh ON dh.doctor_code = d.doctor_id and dh.del=1 \n" +
                "\tWHERE\n" +
                "\t\t1 = 1\n" +
                condition+
                "\tand (d.health_code in(2,3) OR CAST(d.temperature as DECIMAL)>=37.3 OR  d.is_symptoms=1 OR d.is_epidemic=1)\n" +
                "\tGROUP BY\n" +
                "\t\tdh.dept_code\n" +
                ")dh6 ON dh6.dept_code=hd.code\n" +
                "WHERE\n" +
                "\thd.consult_dept_flag = 1 ";

        if (level==2) {
            deptSql += " and hd.code = '" + dept + "' ";
        }
        deptSql +=" ORDER BY (IF(ISNULL(dh1.total),0,dh1.total)-IF(ISNULL(dh2.total),0,dh2.total)) DESC ";
        List<Map<String, Object>> mapList = jdbcTemplate.queryForList(deptSql);
        for (Map<String,Object> deptMap:mapList){
            String day = DateUtil.getTwoDay(startDate, startDate);
            Integer total = 0;
            Integer yishangbaoTotal = 0;
            Integer weishangbaoTotal = 0;
            Integer zhengchangTotal = 0;
            Integer yichangTotal = 0;
            Integer shangchuanTotal = 0;
            Integer yichangPersonTotal = 0;
            if (deptMap.get("total")!=null){
                String doctorTotal = deptMap.get("total").toString();
                //总人次
                if (!day.equalsIgnoreCase("0")) {
                    total = Integer.parseInt(doctorTotal) * Integer.parseInt(day);
                } else {
                    total = Integer.parseInt(doctorTotal);
                }
            }
            if (deptMap.get("yishangbaoTotal")!=null){
                yishangbaoTotal = Integer.parseInt(deptMap.get("yishangbaoTotal").toString());
            }
            //未上报
            weishangbaoTotal = total - yishangbaoTotal;
            if (deptMap.get("zhengchangTotal")!=null){
                zhengchangTotal = Integer.parseInt(deptMap.get("zhengchangTotal").toString());
            }
            if (deptMap.get("yichangTotal")!=null){
                yichangTotal = Integer.parseInt(deptMap.get("yichangTotal").toString());
            }

            if (deptMap.get("yichangPersonTotal")!=null){
                yichangPersonTotal = Integer.parseInt(deptMap.get("yichangPersonTotal").toString());
            }
            if (deptMap.get("shangchuanTotal")!=null){
                shangchuanTotal = Integer.parseInt(deptMap.get("shangchuanTotal").toString());
            }

            JSONObject object = new JSONObject();
            object.put("total", total);//总人次
            object.put("yishangbaoTotal", yishangbaoTotal);//已上报
            object.put("weishangbaoTotal", weishangbaoTotal);//未上报
            object.put("zhengchangTotal", zhengchangTotal);//上报正常
            object.put("yichangTotal", yichangTotal);//上报异常
            object.put("yichangPersonTotal", yichangPersonTotal);//上报人异常
            object.put("shangchuanTotal", shangchuanTotal);//上传总次数
            deptMap.put("tongyishuju", object);
        }
        return mapList;
    }


}