package com.yihu.jw.hospital.booking.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.hospital.booking.BaseSignDO;
import com.yihu.jw.entity.hospital.booking.PatientReservation;
import com.yihu.jw.hospital.booking.dao.BaseSignDao;
import com.yihu.jw.hospital.booking.dao.PatientReservationDao;
import com.yihu.jw.mysql.query.BaseJpaService;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.entity.ServiceException;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.transaction.Transactional;
import java.util.*;

/**
 * 预约挂号业务处理类
 *
 * @author George
 */
@Component
@Transactional(rollbackOn = Exception.class)
public class PatientReservationService extends BaseJpaService<PatientReservation,PatientReservationDao> {

    @Resource
    private PatientReservationDao patientReservationDao;
    @Autowired
    private ObjectMapper objectMapper;
    @Resource
    private BasePatientDao patientDao;
    @Resource
    private BaseDoctorDao doctorDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Resource
    private BaseSignDao baseSignDao;

    /**
     * 获取居民单日的预约挂号记录
     */
    public List<PatientReservation> findList(String patient){
        String sql = "select * from wlyy_patient_reservation where patient='"+patient+"'";
        sql += " and start_time>='"+DateUtil.getStringDateShort()+"' and start_time<='"+DateUtil.getStringDateShort()+" 23:59:59'";
        return jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(PatientReservation.class));
    }

    /**
     * 签到
     * @param code
     */
    public void sign(String code,Integer type){
        synchronized (code.intern()){
            List<BaseSignDO> list = baseSignDao.findByRelationIdAndType(code,type);
            if(list.size()==0){
                PatientReservation reservation = findByCode(code);
                if(reservation!=null){
                    BaseSignDO signDO = new BaseSignDO();
                    signDO.setPatient(reservation.getPatient());
                    signDO.setPatientName(reservation.getName());
                    signDO.setRelationId(code);
                    signDO.setType(type);
                    signDO.setCreateTime(new Date());
                    signDO = baseSignDao.save(signDO);

                    reservation.setSignId(signDO.getId());
                    patientReservationDao.save(reservation);
                }
            }
        }
    }


    public PatientReservation findByCode(String code) {
        return patientReservationDao.findByCode(code);
    }

    public PatientReservation findById(Long id) throws Exception {
        PatientReservation re = patientReservationDao.findById(id).orElse(null);
        if (re == null) {
            throw new ServiceException("获取预约详情失败！");
        }

        return re;
    }


    /**
     * 实体转map
     */
    private Map<String, String> entityToMap(PatientReservation obj) throws Exception {
        String mapString = objectMapper.writeValueAsString(obj);
        return objectMapper.readValue(mapString, Map.class);
    }

    /**
     * 更新预约状态
     */
    @Transactional
    public void updateStatus(Long id, Integer status) {
        PatientReservation obj = patientReservationDao.findById(id).orElse(null);
        if (!obj.getStatus().equals(status)) {
            obj.setStatus(status);
            patientReservationDao.save(obj);
        }
    }

    /**
     * 医生取消预约
     */
    @Transactional
    public void doctorCancelOrder(Long id,String doctor) {
        PatientReservation obj = patientReservationDao.findById(id).orElse(null);
        if (obj.getStatus()!=0) {
            obj.setStatus(0);
            String patient = obj.getPatient();
            obj.setCanceler(doctor);
            patientReservationDao.save(obj);
        }
    }

    /**
     * 居民取消预约
     */
    @Transactional
    public void patientCancelOrder(String code,String patient) {
        PatientReservation obj = patientReservationDao.findByCode(code);
        if (obj.getStatus()!=0) {
            obj.setStatus(0);
            obj.setCanceler(patient);
            obj.setCancelerName(obj.getName());
            obj.setCancelerType(5);
            obj.setCancelerTime(DateUtil.dateToStrLong(new Date()));
            patientReservationDao.save(obj);
        }
    }

    /**
     * 保存预约挂号记录
     *
     * @param code         预约号/流水号
     * @param doctor       医生标识
     * @param patient      患者标识
     * @param idcard       患者身份证号
     * @param name         患者姓名
     * @param phone        患者手机号
     * @param ssc          患者社保卡号
     * @param section_type 预约时间段：AM (上午)或者PM (下午)
     * @param start_time   一次预约段的开始时间
     * @param org_code     医生所在医疗机构编码
     * @param org_name     医疗机构名称
     * @param dept_code    科室编码
     * @param dept_name    科室名称
     * @param doctor_code  专家ID/编码
     * @param doctor_name  专家姓名
     * @param doctor_photo 专家头像
     * @return
     */
    public void save(String code, BaseDoctorDO doctor, String patient, String idcard, String name, String phone, String ssc, String section_type, String start_time, String org_code, String org_name, String dept_code, String dept_name, String doctor_code, String doctor_name, String doctor_job, String doctor_photo) {
        try {
            PatientReservation reservation = new PatientReservation();
            reservation.setCode(code);
            reservation.setCzrq(new Date());
            reservation.setType("1");
            reservation.setDeptCode(dept_code);
            reservation.setDeptName(dept_name);
            if (doctor != null) {
                reservation.setDname(doctor.getName());
                reservation.setDoctor(doctor.getId());
            }
            reservation.setDoctorCode(doctor_code);
            reservation.setDoctorName(doctor_name);
            reservation.setDoctorJob(doctor_job);
            reservation.setDoctorPhoto(doctor_photo);
            reservation.setIdcard(idcard);
            reservation.setName(doctor_name);
            reservation.setOrgCode(org_code);
            reservation.setOrgName(org_name);
            reservation.setPatient(patient);
            reservation.setPhone(phone);
            reservation.setSectionType(section_type);
            reservation.setSsc(ssc);
            reservation.setStartTime(DateUtil.strToDate(start_time,DateUtil.YYYY_M_D_HH_MM_SS));
            reservation.setStatus(1);
            // 保存记录
            PatientReservation temp = patientReservationDao.save(reservation);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }



    /**
     * 分页获取患者预约记录，根据团队
     */
    @Transactional
    public JSONArray getReservationByPatient(String patient, Long teamCode, int page, int pagesize) throws Exception {
        JSONArray array = new JSONArray();
        // 排序
        Sort sort = Sort.by(Direction.DESC, "id");
        // 分页信息
        PageRequest pageRequest = PageRequest.of(page - 1, pagesize, sort);
        Page<Object> result = null;
        if (teamCode != null && teamCode > 0) {//传入医生查询医生帮此患者的预约记录
            result = patientReservationDao.findByPatientAndAdminTeamCode(patient, teamCode, pageRequest);
        } else {
            result = patientReservationDao.findByPatient(patient, pageRequest);
        }
        List<Object> list = result.getContent();
        if (list != null) {
            Map<String, BaseDoctorDO> doctors = new HashMap<>();
            for (Object one : list) {
                PatientReservation item = (PatientReservation)one;
                JSONObject obj = new JSONObject(item);
                if (!StringUtils.isEmpty(item.getDoctor())) {
                    if (doctors.get(item.getDoctor()) != null) {
                        obj.put("photo", doctors.get(item.getDoctor()).getPhoto());
                    } else {
                        BaseDoctorDO doc = doctorDao.findById(item.getDoctor()).orElse(null);

                        if (doc != null) {
                            doctors.put(doc.getId(), doc);
                            obj.put("photo", doc.getPhoto());
                        }
                    }
                }
                obj.put("total",result.getTotalElements());
                array.put(obj);
            }
        }

        return array;
    }


    /**
     * 分页获取患者预约记录（医生端）
     */
    public List<Map<String, String>> getReservationByDoctor(String doctor, int page, int pagesize) throws Exception {
        List<Map<String, String>> re = new ArrayList<>();
        // 排序
        Sort sort = Sort.by(Direction.DESC, "id");
        // 分页信息
        PageRequest pageRequest = PageRequest.of(page - 1, pagesize, sort);
        List<PatientReservation> list = patientReservationDao.findByDoctor(doctor, pageRequest);
        //返回患者头像
        for (PatientReservation item : list) {
            Map<String, String> map = entityToMap(item);

            BasePatientDO patient = patientDao.findById(item.getPatient()).orElse(null);
            if (patient != null) {
                map.put("photo", patient.getPhoto());
            }
            re.add(map);

        }
        return re;
    }


    public Long countReservationByDoctorForPatient(String doctor, String patient) {
        return patientReservationDao.countReservationByDoctorForPatient(doctor, patient);
    }

}
