package com.yihu.jw.hospital.disease.service;


import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.entity.specialist.BaseDiseaseHospitalDO;
import com.yihu.jw.entity.specialist.rehabilitation.RehabilitationPlanTemplateDO;
import com.yihu.jw.entity.specialist.rehabilitation.RehabilitationTemplateDetailDO;
import com.yihu.jw.hospital.disease.dao.BaseDiseaseHospitalDao;
import com.yihu.jw.hospital.disease.dao.RehabilitationPlanTemplateDao;
import com.yihu.jw.hospital.disease.dao.RehabilitationTemplateDetailDao;
import com.yihu.jw.mysql.query.BaseJpaService;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.utils.AES;
import com.yihu.jw.utils.hibernate.HibenateUtils;
import com.yihu.jw.wlyy.service.WlyyBusinessService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;


@Service
public class BaseDiseaseHospitalService extends BaseJpaService<BaseDiseaseHospitalDO, BaseDiseaseHospitalDao> {

    @Autowired
    private BaseDiseaseHospitalDao diseaseHospitalDao;
    @Autowired
    private HibenateUtils hibenateUtils;
    @Autowired
    private RehabilitationPlanTemplateDao templateDao;
    @Autowired
    private RehabilitationTemplateDetailDao templateDetailDao;
    @Autowired
    private WlyyBusinessService wlyyBusinessService;

    /**
     * 新增和修改专病配置
     * @param diseaseHospitalDO
     * @return
     */
    public BaseDiseaseHospitalDO insertAndUpdate(BaseDiseaseHospitalDO diseaseHospitalDO){
        if (StringUtils.isNoneBlank(diseaseHospitalDO.getId())) {
            diseaseHospitalDO.setUpdateTime(new Date());
            return diseaseHospitalDao.save(diseaseHospitalDO);
        }else {
            diseaseHospitalDO.setCreateTime(new Date());
            diseaseHospitalDO.setSort(maxSort("base_disease_hospital")+1);
            diseaseHospitalDO = diseaseHospitalDao.save(diseaseHospitalDO);
            //新增专病默认配置康复计划
            defaultRehabilitationPlan(diseaseHospitalDO.getId());
            return diseaseHospitalDO;
        }
    }

    public void defaultRehabilitationPlan(String diseaseId){
        String templateId = "system";
        RehabilitationPlanTemplateDO templateDO = templateDao.findById(templateId).orElse(null);
        if(templateDO==null){
            return;
        }
        List<RehabilitationTemplateDetailDO> details = templateDetailDao.findTemplateDetailByTemplateId(templateId);
        if(details.size()==0){
            return;
        }
        RehabilitationPlanTemplateDO templateDONew = new RehabilitationPlanTemplateDO();
        BeanUtils.copyProperties(templateDO,templateDONew);
        templateDONew.setCreateTime(new Date());
        templateDONew.setId(null);
        templateDONew.setDiseaseId(diseaseId);
        templateDONew.setType("2");
        templateDONew = templateDao.save(templateDONew);
        String templateDONewId = templateDONew.getId();
        List<RehabilitationTemplateDetailDO> detailsNew = new ArrayList<>();
        for (RehabilitationTemplateDetailDO detailDO:details){
            RehabilitationTemplateDetailDO detail = new RehabilitationTemplateDetailDO();
            BeanUtils.copyProperties(detailDO,detail);
            detail.setId(null);
            detail.setTemplateId(templateDONewId);
            detail.setCreateTime(new Date());
            detailsNew.add(detail);
        }
        if(detailsNew.size()>0){
            templateDetailDao.saveAll(detailsNew);
        }
    }

    /**
     * 获取排序的序号
     *
     * @param tableName
     * @return
     */
    public  Integer maxSort(String tableName){
        String sql = "SELECT MAX(sort) as sort from "+tableName+" ";
        List<Map<String, Object>> rstotal = hibenateUtils.createSQLQuery(sql);
        Integer count = 0;
        if (rstotal != null && rstotal.size() > 0) {
            if (rstotal.get(0).get("sort")!=null){
                count = Integer.parseInt(rstotal.get(0).get("sort").toString());
            }
        }
        return count;
    }


    /**
     * 根据id获取专病配置详情
     * @param id
     * @return
     */
    public BaseDiseaseHospitalDO findById(String id){
        BaseDiseaseHospitalDO baseDiseaseHospitalDO =diseaseHospitalDao.findById(id).get();
        if (StringUtils.isNoneBlank(baseDiseaseHospitalDO.getArticleRelation())){
            String sql = "select t.id as \"id\",t.title as \"title\",t.read_count as \"readCount\"," +
                    "t.collection as \"collection\",t.fabulous as \"fabulous\",t.is_share as \"share\"," +
                    " t.intro as \"intro\",t.category_first_name as \"categoryFirstName\"," +
                    " t.category_second_name as \"categorySecondName\" ," +
                    "t.content as \"content\",t.image as \"image\",t.create_user_name as \"createUserName\",c.job_title_name as \"jobTitleName\"," +
                    "b.dept_name as \"deptName\",b.org_name as \"hospitalName\"," +
                    "t.create_time as \"createTime\" " +
                    " from wlyy_knowledge_article t left join wlyy_knowledge_article_dept a " +
                    " on a.article_id = t.id left join base_doctor_hospital b on b.doctor_code = t.create_user" +
                    " left join base_doctor c on c.id = t.create_user where 1=1 and t.del = 1 and t.status = 1";
            if (StringUtils.isNoneBlank(baseDiseaseHospitalDO.getArticleRelation())){
                if (baseDiseaseHospitalDO.getArticleRelation().contains(",")){
                    String categorySecond = baseDiseaseHospitalDO.getArticleRelation().replace(",","','");
                    sql+=" and t.category_second in ('"+categorySecond+"') ";
                }
            }

            List<Map<String,Object>> result = hibenateUtils.createSQLQuery(sql);
            baseDiseaseHospitalDO.setArticleRelationList(result);
        }
        return baseDiseaseHospitalDO;
    }

    /**
     * 查询专病中心列表
     */
    public MixEnvelop selectByCondition(String name, String status, Integer page, Integer size){
        String orderBy = "  order by sort asc,create_time desc  ";
        String condition = " ";
        String sql = "SELECT\n" +
                "\tid,\n" +
                "\tNAME,\n" +
                "\tphoto,\n" +
                "\tbanner,\n" +
                "\tdescription,\n" +
                "\tSTATUS,\n" +
                "\tdept,\n" +
                "\tsort,\n" +
                "\tdept_name AS deptName,\n" +
                "\tdiagnosis,\n" +
                "\tdiagnosis_name AS diagnosisName,\n" +
                "\tis_disease AS isDisease,\n" +
                "\tplan_doctor AS planDoctor,\n" +
                "\tplan_doctor_name AS planDoctorName,\n" +
                "\tarticle_relation as articleRelation,\n" +
                "\tarticle_relation_name as articleRelationName,\n" +
                "\tis_family as isFamily,\n" +
                "\tfamily_relation as familyRelation,\n" +
                "\tfamily_relation_name as familyRelationName,\n" +
                "\tdate_format(create_time ,'%Y-%m-%d %H:%i:%S' ) as createTime,\n" +
                "\tdate_format(update_time ,'%Y-%m-%d %H:%i:%S' ) as updateTime ," +
                "\trehabilitation_type 'rehabilitationType'," +
                "\trehabilitation_down_type 'rehabilitationDownType'," +
                "\trehabilitation_template_id 'rehabilitationTemplateId'\n" +
                "FROM\n" +
                "\tbase_disease_hospital dh where 1=1 ";
        if (StringUtils.isNoneBlank(name)){
            condition +=" and name like '%"+name+"%' ";
        }
        if (StringUtils.isNoneBlank(status)){
            condition +=" and status ='"+status+"' ";
        }
        List<Map<String, Object>> list = hibenateUtils.createSQLQuery(sql+condition+orderBy, page, size);
        String sqlCount ="select COUNT(1) as total from base_disease_hospital where 1=1 ";
        List<Map<String, Object>> rstotal = hibenateUtils.createSQLQuery(sqlCount+condition+orderBy);
        Long count = 0L;
        if (rstotal != null && rstotal.size() > 0) {
            count = Long.parseLong(rstotal.get(0).get("total").toString());
        }
        return MixEnvelop.getSuccessListWithPage("success", list, page, size, count);
    }


    /**
     * 根据id删除专病详情
     * @param id
     */
    public void deleteById(String id){
        diseaseHospitalDao.deleteById(id);
    }


    /**
     * 更新启动状态或者上移下移置顶
     * @param id 专病配置id
     * @param status 1启动0禁止
     * @param flag  1上移2下移3置顶
     * @return
     */
    public BaseDiseaseHospitalDO updateStatusOrSort(String id,Integer status,Integer flag){
        BaseDiseaseHospitalDO diseaseHospitalDO = diseaseHospitalDao.findById(id).get();
        if (flag!=null){
            int sort = 0;
            if (flag==1){
                String sql = "select sort from base_disease_hospital where sort<'"+diseaseHospitalDO.getSort()+"' order by sort desc limit 1";
                List<Map<String,Object>> mapList = hibenateUtils.createSQLQuery(sql);
                Integer count = 0;
                if (mapList != null && mapList.size() > 0) {
                    count = Integer.parseInt(mapList.get(0).get("sort").toString());
                }
                if (count==0){
                    sort = 0;
                }else {
                    sort=count;
                }
                BaseDiseaseHospitalDO diseaseHospitalDO1= diseaseHospitalDao.selectBySort(sort);
                if (diseaseHospitalDO1!=null){
                    diseaseHospitalDO1.setSort(diseaseHospitalDO.getSort());
                    diseaseHospitalDao.save(diseaseHospitalDO1);
                }

            }else if (flag==2){
                String sql = "select sort from base_disease_hospital where sort>'"+diseaseHospitalDO.getSort()+"' order by sort asc limit 1";
                List<Map<String,Object>> mapList = hibenateUtils.createSQLQuery(sql);
                Integer count = 0;
                if (mapList != null && mapList.size() > 0) {
                    count = Integer.parseInt(mapList.get(0).get("sort").toString());
                }
                if (count==0){
                    sort = 0;
                }else {
                    sort=count;
                }
                BaseDiseaseHospitalDO diseaseHospitalDO1= diseaseHospitalDao.selectBySort(sort);
                if (diseaseHospitalDO1!=null){
                    diseaseHospitalDO1.setSort(diseaseHospitalDO.getSort());
                    diseaseHospitalDao.save(diseaseHospitalDO1);
                }

            }else if (flag==3){
                sort = 1;
                BaseDiseaseHospitalDO diseaseHospitalDO1= diseaseHospitalDao.selectBySort(sort);
                if (diseaseHospitalDO1!=null){
                    diseaseHospitalDO1.setSort(diseaseHospitalDO.getSort());
                    diseaseHospitalDao.save(diseaseHospitalDO1);
                }
            }
            if (sort!=0){
                diseaseHospitalDO.setSort(sort);
            }
            return diseaseHospitalDao.save(diseaseHospitalDO);
        }else {
            diseaseHospitalDO.setStatus(status);
            diseaseHospitalDO.setUpdateTime(new Date());
            return diseaseHospitalDao.save(diseaseHospitalDO);
        }
    }

    /**
     * 获取十大专病列表
     * @return
     */
    public JSONObject getSpecialList(){
        return wlyyBusinessService.getSpecialList();
    }


    /**
     * 获取诊断
     *
     * @param pyKey
     * @param page
     * @param size
     * @return
     * @throws Exception
     */
    public MixEnvelop getICD10(String pyKey,Integer page,Integer size) throws Exception {
        String sql = "select code,name,py_code,d_code from dict_icd10 where 1=1 ";
        if (StringUtils.isNoneBlank(pyKey)){
            sql += " and (py_code like '%"+pyKey+"%' or name like '%"+pyKey+"%') ";
        }
        List<Map<String,Object>> mapList = hibenateUtils.createSQLQuery (sql,page,size);
        String sqlCount ="select COUNT(1) as total from dict_icd10 where 1=1 ";
        List<Map<String, Object>> rstotal = hibenateUtils.createSQLQuery(sqlCount);
        Long count = 0L;
        if (rstotal != null && rstotal.size() > 0) {
            count = Long.parseLong(rstotal.get(0).get("total").toString());
        }
        return MixEnvelop.getSuccessListWithPage("success", mapList, page, size, count);
    }

    /**
     * i健康授权参数加密
     * @param json
     * @return
     * @throws Exception
     */
    public  JSONObject ascRevertData(String json) throws Exception {
        JSONObject jsonObject = new JSONObject();
        if (StringUtils.isNoneBlank(json)){
            JSONObject object = JSONObject.parseObject(json);

            for (Map.Entry<String,Object> entry:object.entrySet()){
                if (entry.getKey().equalsIgnoreCase("name")){
                    String name= URLEncoder.encode(entry.getValue().toString());
                    jsonObject.put(entry.getKey(), AES.AscRevertData2(name));
                }else {
                    jsonObject.put(entry.getKey(), AES.AscRevertData2(entry.getValue().toString()));
                }

            }
        }
        return jsonObject;
    }
}
