package com.yihu.jw.hospital.integrate.service;

import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.doctor.dao.BaseDoctorHospitalDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.doctor.BaseDoctorHospitalDO;
import com.yihu.jw.entity.hospital.integrate.BaseHospitalActivityDO;
import com.yihu.jw.entity.hospital.integrate.BaseHospitalGoodsDO;
import com.yihu.jw.entity.hospital.integrate.BaseHospitalIntegrateDO;
import com.yihu.jw.entity.hospital.integrate.BaseHospitalUserActivityDO;
import com.yihu.jw.hospital.integrate.dao.BaseHospitalActivityDao;
import com.yihu.jw.hospital.integrate.dao.BaseHospitalIntegrateDao;
import com.yihu.jw.hospital.integrate.dao.BaseHospitalUserActivityDao;
import com.yihu.jw.mysql.query.BaseJpaService;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.utils.hibernate.HibenateUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Service
@Transactional
public class BaseHospitalActivityService extends BaseJpaService<BaseHospitalActivityDO, BaseHospitalActivityDao> {
    @Autowired
    private BaseHospitalActivityDao hospitalActivityDao;
    @Autowired
    private HibenateUtils hibenateUtils;
    @Autowired
    private BaseHospitalIntegrateDao hospitalIntegrateDao;
    @Autowired
    private BaseHospitalUserActivityDao userActivityDao;
    @Autowired
    private BaseDoctorDao doctorDao;
    @Autowired
    private BaseDoctorHospitalDao doctorHospitalDao;

    /**
     * 新增或者修改活动
     * @param hospitalActivityDO
     * @return
     */
    @Transactional
    public BaseHospitalActivityDO insertAndUpdate(BaseHospitalActivityDO hospitalActivityDO,BaseHospitalIntegrateDO integrateDO){
        BaseHospitalActivityDO activityDO = new BaseHospitalActivityDO();
        if (StringUtils.isNoneBlank(hospitalActivityDO.getId())){
            activityDO = hospitalActivityDao.findById(hospitalActivityDO.getId()).get();
            hospitalActivityDO.setUpdateTime(new Date());
            hospitalActivityDO.setDel(1);
            hospitalIntegrateDao.deleteByActivityId(hospitalActivityDO.getId());
        }else {
            hospitalActivityDO.setCreateTime(new Date());
            hospitalActivityDO.setUpdateTime(new Date());
            hospitalActivityDO.setDel(1);
            hospitalActivityDO.setSort(maxSort("base_hospital_activity")+1);
            hospitalActivityDO.setStatus(0);
        }
        hospitalActivityDO= hospitalActivityDao.save(hospitalActivityDO);
        if (integrateDO!=null){
            integrateDO.setActivityId(hospitalActivityDO.getId());
            hospitalIntegrateDao.save(integrateDO);
        }
        if (hospitalActivityDO.getRegistrationType()==0){
            userActivityDao.deleteByRelationCode(hospitalActivityDO.getId());
            //删除自动参加
            if (hospitalActivityDO.getParticipantsType()==1){
                String conditionSql="";
                if (hospitalActivityDO.getParticipantsArea().equalsIgnoreCase("0")){
                    conditionSql = "";
                }else if (hospitalActivityDO.getParticipantsArea().equalsIgnoreCase("1")){
                    String str[] = hospitalActivityDO.getParticipantsCode().split(",");
                    String deptCode= "";
                    for (int i=0;i<str.length;i++){
                        deptCode +="'"+str[i]+"',";
                    }
                    if (StringUtils.isNoneBlank(deptCode)){
                        deptCode = deptCode.substring(0,deptCode.length()-1);
                    }
                    conditionSql = " and d.id IN(SELECT dh.doctor_code from base_doctor_hospital dh where dh.dept_code in("+deptCode+") and dh.del=1) ";
                }
                String doctorSql = " SELECT d.id,d.name FROM base_doctor d where 1=1 and d.del=1 and d.name is not null  ";
                List<Map<String,Object>> mapList = jdbcTemplate.queryForList(doctorSql+conditionSql);
                for (Map<String,Object> map:mapList){
                    String id= map.get("id").toString();
                    String name = map.get("name").toString();
                    BaseHospitalUserActivityDO userActivityDO = new BaseHospitalUserActivityDO();
                    userActivityDO.setType(1);
                    userActivityDO.setName(name);
                    userActivityDO.setUser(id);
                    userActivityDO.setStatus(1);
                    userActivityDO.setRelationCode(hospitalActivityDO.getId());
                    userActivityDO.setRelationName(hospitalActivityDO.getTitle());
                    userActivityDO.setCreateTime(new Date());
                    userActivityDO.setUpdateTime(new Date());
                    userActivityDao.save(userActivityDO);
                }

            }else if (hospitalActivityDO.getParticipantsType()==2){
                String patientSql = " SELECT id,name FROM base_patient where del=1  ";
                List<Map<String,Object>> mapList = jdbcTemplate.queryForList(patientSql);
                for (Map<String,Object> map:mapList){
                    String id= map.get("id").toString();
                    String name = map.get("name").toString();
                    BaseHospitalUserActivityDO userActivityDO = new BaseHospitalUserActivityDO();
                    userActivityDO.setType(2);
                    userActivityDO.setName(name);
                    userActivityDO.setUser(id);
                    userActivityDO.setStatus(1);
                    userActivityDO.setRelationCode(hospitalActivityDO.getId());
                    userActivityDO.setRelationName(hospitalActivityDO.getTitle());
                    userActivityDO.setCreateTime(new Date());
                    userActivityDO.setUpdateTime(new Date());
                    userActivityDao.save(userActivityDO);
                }
            }
        }
        return hospitalActivityDO;
    }


    /**
     * 根据id删除活动
     * @param id
     */
    public void deleteById(String id){
        String str[]= id.split(",");
        for (int i=0;i<str.length;i++){
            hospitalActivityDao.deleteById(str[i]);
        }
    }

    /**
     * 根据id获取活动信息
     *
     * @param id
     * @return
     */
    public BaseHospitalActivityDO findById(String id){
        BaseHospitalActivityDO hospitalActivityDO = hospitalActivityDao.findById(id).get();
        BaseHospitalIntegrateDO hospitalIntegrateDO = hospitalIntegrateDao.selectByActivityId(hospitalActivityDO.getId());
        hospitalActivityDO.setIntegrateDO(hospitalIntegrateDO);
        return hospitalActivityDO;
    }

    public JSONObject selectInfoByUserAndActivityId(String user,String activityId){
        JSONObject object = new JSONObject();
        BaseHospitalActivityDO activityDO = hospitalActivityDao.findById(activityId).get();
        BaseHospitalUserActivityDO userActivityDO = userActivityDao.selectByUserAndRelationCode(user,activityId);
        List<BaseHospitalUserActivityDO> userActivityDOS = userActivityDao.selectByRelationCode(activityId);
        for (BaseHospitalUserActivityDO userActivityDO1:userActivityDOS){
            if (userActivityDO1.getType()==1){
                BaseDoctorDO doctorDO = doctorDao.findByIdAndDel(userActivityDO1.getUser());
                userActivityDO1.setDoctorDO(doctorDO);
            }
        }
        object.put("activityDO",activityDO);//活动详情
        object.put("userActivityList",userActivityDOS);//活动参与人数列表
        object.put("userActivityDO",userActivityDO);//改用户是否参与
        return object;
    }


    /**
     * 查询活动列表
     */
    public MixEnvelop selectByCondition(String title, Integer page, Integer size,Integer flag,Integer status,String startTime,String endTime,String user,String userId){
        String orderBy = "  order by sort asc,create_time desc  ";
        String condition = " ";
        String sql = "SELECT\n" +
                "\tid,\n" +
                "\ttype,\n" +
                "\ttitle,\n" +
                "\tbanner,\n" +
                "\tcount,\n" +
                "\torganizer_org_code AS organizerOrgCode,\n" +
                "\torganizer_org_name AS organizerOrgName,\n" +
                "\torganizer_dept_code AS organizerDeptCode,\n" +
                "\torganizer_dept_name AS organizerDeptName,\n" +
                "\tlocation,\n" +
                "\tSTATUS,\n" +
                "\tflag,\n" +
                "\tdel,\n" +
                "\tinfo,\n" +
                "\tcontent,\n" +
                "\tparticipants_type AS participantsType,\n" +
                "\tparticipants_area AS participantsArea,\n" +
                "\tparticipants_code AS participantsCode,\n" +
                "\tsort,\n" +
                "\tmanager as manager,\n" +
                "\tmanager_name as managerName,\n" +
                "\tregistration_type as registrationType,\n" +
                "\tdate_format(\n" +
                "\t\tregistration_start_time,\n" +
                "\t\t'%Y-%m-%d %H:%i:%S'\n" +
                "\t) AS registrationStartTime,\n" +
                "\tdate_format(\n" +
                "\t\tregistration_end_time,\n" +
                "\t\t'%Y-%m-%d %H:%i:%S'\n" +
                "\t) AS registrationEndTime,\n" +
                "\tis_task as isTask,\n" +
                "\ttask_code as taskCode,\n" +
                "\ttask_name as taskName,\n" +
                "\tbusiness_code as businessCode,\n" +
                "\tbusiness_name as businessName,\n" +
                "\tis_integrate as isIntegrate,\n" +
                "\tdate_format(\n" +
                "\t\tstart_time,\n" +
                "\t\t'%Y-%m-%d %H:%i:%S'\n" +
                "\t) AS startTime,\n" +
                "\tdate_format(\n" +
                "\t\tend_time,\n" +
                "\t\t'%Y-%m-%d %H:%i:%S'\n" +
                "\t) AS endTime,\n" +
                "\tdate_format(\n" +
                "\t\tcreate_time,\n" +
                "\t\t'%Y-%m-%d %H:%i:%S'\n" +
                "\t) AS createTime\n" +
                "FROM\n" +
                "\tbase_hospital_activity where 1=1 ";
        if (StringUtils.isNoneBlank(title)){
            condition +=" and title like '%"+title+"%' ";
        }

        if (flag!=null){
            condition +=" and flag = '"+flag+"' ";
        }

        if (status!=null){
            condition +=" and status = '"+status+"' ";
        }

        if (StringUtils.isNoneBlank(startTime)){
            condition +=" and start_time >= '"+startTime+"' ";
        }

        if (StringUtils.isNoneBlank(endTime)){
            condition +=" and start_time <= '"+endTime+"' ";
        }

        if (StringUtils.isNoneBlank(user)){
            condition +=" and id IN(SELECT relation_code FROM `base_hospital_user_activity` where user ='"+user+"') ";
        }

        if (StringUtils.isNoneBlank(userId)){
            List<BaseDoctorHospitalDO> doctorHospitalDOS = doctorHospitalDao.findByDoctorCode(userId);
            String deptCondition = "";
            if (doctorHospitalDOS.size()==1){
                deptCondition +=" participants_code like '%"+doctorHospitalDOS.get(0).getDeptCode()+"%' ";
            }else {
                for (BaseDoctorHospitalDO doctorHospitalDO:doctorHospitalDOS){
                    deptCondition +=" participants_code like '%"+doctorHospitalDO.getDeptCode()+"%' or ";
                }
                deptCondition = deptCondition.substring(0,deptCondition.length()-3);
            }

            condition +=" and (("+deptCondition+") or participants_code is null  or participants_code ='' ) ";
        }

        List<Map<String, Object>> list = hibenateUtils.createSQLQuery(sql+condition+orderBy, page, size);
        for (Map<String,Object> map:list){
            String id = map.get("id").toString();
            updateActivityStatus(id);
            if (map.get("registrationType").toString().equalsIgnoreCase("1")){
                List<BaseHospitalUserActivityDO> userActivityDOS = userActivityDao.selectByRelationCode(id);
                map.put("shengyuTotal",Integer.parseInt(map.get("count").toString())-userActivityDOS.size());
            }
        }
        String sqlCount ="select COUNT(1) as total from base_hospital_activity where 1=1 ";
        List<Map<String, Object>> rstotal = hibenateUtils.createSQLQuery(sqlCount+condition+orderBy);
        Long count = 0L;
        if (rstotal != null && rstotal.size() > 0) {
            count = Long.parseLong(rstotal.get(0).get("total").toString());
        }
        return MixEnvelop.getSuccessListWithPage("success", list, page, size, count);
    }

    public void updateActivityStatus(String id){
        BaseHospitalActivityDO activityDO = hospitalActivityDao.findById(id).get();
        if (activityDO!=null){
            Date startTime = activityDO.getStartTime();
            Date endTime = activityDO.getEndTime();
            Date date = new Date();
            Boolean flag = false;
            if (date.compareTo(startTime)==0||date.compareTo(endTime)==0){
                activityDO.setStatus(1);
            }else if (date.compareTo(startTime)>0&&date.compareTo(endTime)<0){
                activityDO.setStatus(1);
            }else if (date.compareTo(startTime)<0){
                activityDO.setStatus(0);
            }else if (date.compareTo(endTime)>0){
                activityDO.setStatus(2);
            }
            hospitalActivityDao.save(activityDO);
        }
    }

    /**
     * 获取排序的序号
     *
     * @param tableName
     * @return
     */
    public  Integer maxSort(String tableName){
        String sql = "SELECT MAX(sort) as sort from "+tableName+" ";
        List<Map<String, Object>> rstotal = hibenateUtils.createSQLQuery(sql);
        Integer count = 0;
        if (rstotal != null && rstotal.size() > 0) {
            if (rstotal.get(0).get("sort")!=null){
                count = Integer.parseInt(rstotal.get(0).get("sort").toString());
            }

        }
        return count;
    }


    /**
     * 更新启动状态或者上移下移置顶
     * @param id 活动id
     * @param status 1启动0禁止
     * @param flag  1上移2下移3置顶
     * @return
     */
    public BaseHospitalActivityDO updateFlagOrSort(String id, Integer status, Integer flag){
        BaseHospitalActivityDO baseHospitalActivityDO = hospitalActivityDao.findById(id).get();
        if (flag!=null){
            int sort = 0;
            if (flag==1){
                String sql = "select sort from base_hospital_activity where sort<'"+baseHospitalActivityDO.getSort()+"' order by sort desc limit 1";
                List<Map<String,Object>> mapList = hibenateUtils.createSQLQuery(sql);
                Integer count = 0;
                if (mapList != null && mapList.size() > 0) {
                    count = Integer.parseInt(mapList.get(0).get("sort").toString());
                }
                if (count==0){
                    sort = 0;
                }else {
                    sort=count;
                }
                BaseHospitalActivityDO hospitalActivityDO1= hospitalActivityDao.selectBySort(sort);
                if (hospitalActivityDO1!=null){
                    hospitalActivityDO1.setSort(baseHospitalActivityDO.getSort());
                    hospitalActivityDao.save(hospitalActivityDO1);
                }

            }else if (flag==2){
                String sql = "select sort from base_hospital_activity where sort>'"+baseHospitalActivityDO.getSort()+"' order by sort asc limit 1";
                List<Map<String,Object>> mapList = hibenateUtils.createSQLQuery(sql);
                Integer count = 0;
                if (mapList != null && mapList.size() > 0) {
                    count = Integer.parseInt(mapList.get(0).get("sort").toString());
                }
                if (count==0){
                    sort = 0;
                }else {
                    sort=count;
                }
                BaseHospitalActivityDO baseHospitalActivityDO1= hospitalActivityDao.selectBySort(sort);
                if (baseHospitalActivityDO1!=null){
                    baseHospitalActivityDO1.setSort(baseHospitalActivityDO.getSort());
                    hospitalActivityDao.save(baseHospitalActivityDO1);
                }

            }else if (flag==3){
                sort = 1;
                BaseHospitalActivityDO baseHospitalActivityDO1= hospitalActivityDao.selectBySort(sort);
                if (baseHospitalActivityDO1!=null){
                    baseHospitalActivityDO1.setSort(baseHospitalActivityDO.getSort());
                    hospitalActivityDao.save(baseHospitalActivityDO1);
                }
            }
            if (sort!=0){
                baseHospitalActivityDO.setSort(sort);
            }
            return hospitalActivityDao.save(baseHospitalActivityDO);
        }else {
            baseHospitalActivityDO.setFlag(status);
            baseHospitalActivityDO.setUpdateTime(new Date());
            return hospitalActivityDao.save(baseHospitalActivityDO);
        }
    }


    /**
     * 更新活动状态（0未开始1开始2结束）
     * @param id
     * @param status
     * @return
     */
    public BaseHospitalActivityDO updateStatusById(String id ,Integer status){
        BaseHospitalActivityDO activityDO = hospitalActivityDao.findById(id).get();
        activityDO.setStatus(status);
        return hospitalActivityDao.save(activityDO);
    }
}
