package com.yihu.jw.hospital.integrate.service;

import com.yihu.jw.entity.hospital.integrate.*;
import com.yihu.jw.hospital.integrate.dao.BaseHospitalGoodsDao;
import com.yihu.jw.hospital.integrate.dao.BaseHospitalGoodsExchageDao;
import com.yihu.jw.hospital.integrate.dao.BaseHospitalUserIntegrateDao;
import com.yihu.jw.mysql.query.BaseJpaService;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.utils.hibernate.HibenateUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Service
public class BaseHospitalGoodsService extends BaseJpaService<BaseHospitalGoodsDO, BaseHospitalGoodsDao> {
    @Autowired
    private BaseHospitalGoodsDao hospitalGoodsDao;
    @Autowired
    private HibenateUtils hibenateUtils;
    @Autowired
    private BaseHospitalGoodsExchageDao goodsExchageDao;
    @Autowired
    private BaseHospitalUserIntegrateDao userIntegrateDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;

    /**
     * 新增或者修改商品库存
     * @param goodsDO
     * @return
     */
    public BaseHospitalGoodsDO insertAndUpdate(BaseHospitalGoodsDO goodsDO){
        if (StringUtils.isNoneBlank(goodsDO.getId())){
            BaseHospitalGoodsDO hospitalGoodsDO = hospitalGoodsDao.findById(goodsDO.getId()).get();
            List<BaseHospitalGoodsExchageDO> exchageDOList = goodsExchageDao.selectByGoodsCode(goodsDO.getId());
            goodsDO.setRemainingInventory(hospitalGoodsDO.getInventory()-exchageDOList.size());
            goodsDO.setUpdateTime(new Date());
        }else {
            goodsDO.setRemainingInventory(goodsDO.getInventory());
            goodsDO.setCreateTime(new Date());
            goodsDO.setUpdateTime(new Date());
        }
        return hospitalGoodsDao.save(goodsDO);
    }


    /**
     * 根据id删除商品
     * @param id
     */
    public void deleteById(String id){
        hospitalGoodsDao.deleteById(id);
    }

    /**
     * 根据id获取商品信息
     *
     * @param id
     * @return
     */
    public BaseHospitalGoodsDO findById(String id){
        return hospitalGoodsDao.findById(id).get();
    }


    /**
     * 查询商品列表
     */
    public MixEnvelop selectByCondition(String name,Integer type,Integer status,String user,Integer page, Integer size){
        String orderBy = "  order by create_time desc  ";
        String condition = " ";
        String sql = "SELECT\n" +
                "\tid,\n" +
                "\tname,\n" +
                "\tbanner,\n" +
                "\ttype,\n" +
                "\tinventory,\n" +
                "\tintegrate,\n" +
                "\tremaining_inventory as remainingInventory,\n" +
                "\tparticipants_type as participantsType,\n" +
                "\texchange_total as exchangeTotal,\n" +
                "\tdescription as description,\n" +
                "\tdate_format(\n" +
                "\t\tcreate_time,\n" +
                "\t\t'%Y-%m-%d %H:%i:%S'\n" +
                "\t) AS createTime,\n" +
                "\tstatus\n" +
                "FROM\n" +
                "\tbase_hospital_goods  where 1=1  ";
        if (StringUtils.isNoneBlank(name)){
            condition +=" and name like '%"+name+"%' ";
        }

        if (type!=null){
            if (type==0){
                condition += " and integrate <=(SELECT IFNULL(SUM(integrate),0) as total FROM base_hospital_user_integrate where user='"+user+"') ";
            }else {
                condition +=" and type= '"+type+"' ";
            }

        }

        if (status!=null){
            condition +=" and status = '"+status+"' ";
        }


        List<Map<String, Object>> list = hibenateUtils.createSQLQuery(sql+condition+orderBy, page, size);
        for (Map<String,Object> map:list){
            String exchageSql = "SELECT IFNULL(COUNT(1),0) as total FROM base_hospital_goods_exchage WHERE goods_code = '"+map.get("id")+"' and status=1";
            if (StringUtils.isNoneBlank(user)){
                exchageSql +=" and user='"+user+"' ";
            }
            Map<String,Object> map1 = jdbcTemplate.queryForMap(exchageSql);
            if (map1!=null){
                map.put("exchageQt",map1.get("total"));
            }
        }
        String sqlCount ="select COUNT(1) as total from base_hospital_goods where 1=1 ";
        List<Map<String, Object>> rstotal = hibenateUtils.createSQLQuery(sqlCount+condition+orderBy);
        Long count = 0L;
        if (rstotal != null && rstotal.size() > 0) {
            count = Long.parseLong(rstotal.get(0).get("total").toString());
        }
        return MixEnvelop.getSuccessListWithPage("success", list, page, size, count);
    }

    /**
     * 商品上架下架
     * @param id
     * @param status
     * @return
     */
    public BaseHospitalGoodsDO updateGoodsStatus(String id,Integer status){
        BaseHospitalGoodsDO goodsDO = hospitalGoodsDao.findById(id).get();
        goodsDO.setStatus(status);
        goodsDO.setUpdateTime(new Date());
        return hospitalGoodsDao.save(goodsDO);
    }

    /**
     * 兑换商品
     *
     * @param goodsId
     * @param user
     * @param type
     * @param name
     * @return
     * @throws Exception
     */
    public BaseHospitalGoodsExchageDO exchageGoods(String goodsId,String user,Integer type,String name,Integer quantity) throws Exception {
        BaseHospitalGoodsDO goodsDO = hospitalGoodsDao.findById(goodsId).get();
        List<BaseHospitalGoodsExchageDO> goodsExchageDOS = goodsExchageDao.selectByUserAndGoodsCode(user,goodsId);
        if (goodsDO.getExchangeTotal()>0&&goodsDO.getExchangeTotal()>=goodsExchageDOS.size()){
            throw new Exception("每人兑现次数"+goodsDO.getExchangeTotal());
        }
        String integrateSql = "SELECT IFNULL(SUM(integrate),0) as total FROM base_hospital_user_integrate where user ='"+user+"' ";
        int integrate = goodsDO.getIntegrate()*quantity;
        Map<String,Object> map = jdbcTemplate.queryForMap(integrateSql);
        if (map!=null){
            Integer total = Integer.parseInt(map.get("total").toString());
            if (total<integrate){
                throw new Exception("积分余额不足！");
            }
        }
        BaseHospitalGoodsExchageDO goodsExchageDO = new BaseHospitalGoodsExchageDO();
        goodsExchageDO.setUser(user);
        goodsExchageDO.setType(type);
        goodsExchageDO.setName(name);
        goodsExchageDO.setOrderNo("HLWYY"+System.currentTimeMillis()+(int)(Math.random()*900)+100);
        goodsExchageDO.setGoodsCode(goodsDO.getId());
        goodsExchageDO.setGoodsName(goodsDO.getName());
        goodsExchageDO.setGoodsType(goodsDO.getType());
        goodsExchageDO.setIntegrate(goodsDO.getIntegrate());
        goodsExchageDO.setQuantity(quantity);
        goodsExchageDO.setStatus(0);
        goodsExchageDO.setCreateTime(new Date());
        goodsExchageDO = goodsExchageDao.save(goodsExchageDO);
        BaseHospitalUserIntegrateDO userIntegrateDO = new BaseHospitalUserIntegrateDO();
        userIntegrateDO.setIntegrate(-integrate);
        userIntegrateDO.setUser(user);
        userIntegrateDO.setName(name);
        userIntegrateDO.setIntegrateType(2);
        userIntegrateDO.setType(type);
        userIntegrateDO.setStatus(1);
        userIntegrateDO.setCreateTime(new Date());
        userIntegrateDO.setUpdateTime(new Date());
        userIntegrateDO.setRelationCode(goodsId);
        userIntegrateDO.setRelationName(goodsDO.getName());
        userIntegrateDao.save(userIntegrateDO);
        goodsExchageDO.setStatus(1);
        goodsExchageDao.save(goodsExchageDO);
        goodsDO.setRemainingInventory(goodsDO.getInventory()-quantity);
        hospitalGoodsDao.save(goodsDO);
        return goodsExchageDO;
    }


    /**
     * 查询商品兑换记录
     */
    public MixEnvelop selectGoodsExchageByCondition(String name,String user,String startTime,String endTime,Integer page, Integer size){
        String orderBy = "  order by create_time desc  ";
        String condition = " ";
        String sql = "SELECT\n" +
                "\tid,\n" +
                "\tname,\n" +
                "\torder_no as orderNo,\n" +
                "\ttype,\n" +
                "\tuser,\n" +
                "\tintegrate,\n" +
                "\tgoods_code as goodsCode,\n" +
                "\tgoods_name as goodsName,\n" +
                "\tgoods_type as goodsType,\n" +
                "\tquantity as quantity,\n" +
                "\tdate_format(\n" +
                "\t\tcreate_time,\n" +
                "\t\t'%Y-%m-%d %H:%i:%S'\n" +
                "\t) AS createTime,\n" +
                "\tstatus\n" +
                "FROM\n" +
                "\tbase_hospital_goods_exchage  where 1=1  ";
        if (StringUtils.isNoneBlank(name)){
            condition +=" and goods_name like '%"+name+"%' ";
        }
        if (StringUtils.isNoneBlank(user)){
            condition +=" and user = '"+user+"' ";
        }
        if (StringUtils.isNoneBlank(startTime)){
            condition +=" and create_time >= '"+startTime+"' ";
        }
        if (StringUtils.isNoneBlank(endTime)){
            condition +=" and create_time <= '"+endTime+"' ";
        }
        List<Map<String, Object>> list = hibenateUtils.createSQLQuery(sql+condition+orderBy, page, size);
        for (Map<String,Object> map:list){
            String goodsId=map.get("goodsCode").toString();
            BaseHospitalGoodsDO goodsDO = hospitalGoodsDao.findById(goodsId).get();
            map.put("goods",goodsDO);
        }
        String sqlCount ="select COUNT(1) as total from base_hospital_goods_exchage where 1=1 ";
        List<Map<String, Object>> rstotal = hibenateUtils.createSQLQuery(sqlCount+condition+orderBy);
        Long count = 0L;
        if (rstotal != null && rstotal.size() > 0) {
            count = Long.parseLong(rstotal.get(0).get("total").toString());
        }
        return MixEnvelop.getSuccessListWithPage("success", list, page, size, count);
    }

    /**
     * 根据兑换记录id获取商品兑换详情
     *
     * @param goodsExchageId
     * @return
     */
    public BaseHospitalGoodsExchageDO findGoodsExchageById(String goodsExchageId){
        return goodsExchageDao.findById(goodsExchageId).get();
    }

}
