package com.yihu.jw.hospital.integrate.service;

import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.entity.hospital.integrate.BaseHospitalActivityDO;
import com.yihu.jw.entity.hospital.integrate.BaseHospitalMedalDO;
import com.yihu.jw.hospital.integrate.dao.BaseHospitalActivityDao;
import com.yihu.jw.hospital.integrate.dao.BaseHospitalMedalDao;
import com.yihu.jw.mysql.query.BaseJpaService;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.utils.hibernate.HibenateUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Service
public class BaseHospitalMedalService extends BaseJpaService<BaseHospitalMedalDO, BaseHospitalMedalDao> {
    @Autowired
    private BaseHospitalMedalDao hospitalMedalDao;
    @Autowired
    private HibenateUtils hibenateUtils;

    /**
     * 新增或者修改勋章
     * @param hospitalMedalDO
     * @return
     */
    public BaseHospitalMedalDO insertAndUpdate(BaseHospitalMedalDO hospitalMedalDO){
        if (StringUtils.isNoneBlank(hospitalMedalDO.getId())){
            hospitalMedalDO.setUpdateTime(new Date());
        }else {
            hospitalMedalDO.setCreateTime(new Date());
            hospitalMedalDO.setUpdateTime(new Date());
        }
        return hospitalMedalDao.save(hospitalMedalDO);
    }


    /**
     * 根据id删除勋章
     * @param id
     */
    public void deleteById(String id){
        hospitalMedalDao.deleteById(id);
    }

    /**
     * 根据id获取勋章信息
     *
     * @param id
     * @return
     */
    public BaseHospitalMedalDO findById(String id){
        return hospitalMedalDao.findById(id).get();
    }


    /**
     * 查询勋章列表
     */
    public MixEnvelop selectByCondition(String name, Integer page, Integer size){
        String orderBy = "  order by create_time desc  ";
        String condition = " ";
        String sql = "SELECT\n" +
                "\tid,\n" +
                "\tname,\n" +
                "\tbanner,\n" +
                "\tintegrate_type AS integrateType,\n" +
                "\tintegrate_type_name AS integrateTypeName,\n" +
                "\tintegrate,\n" +
                "\tbelong,\n" +
                "\tdate_format(\n" +
                "\t\tcreate_time,\n" +
                "\t\t'%Y-%m-%d %H:%i:%S'\n" +
                "\t) AS createTime,\n" +
                "\tstatus\n" +
                "FROM\n" +
                "\tbase_hospital_medal  where 1=1  ";
        if (StringUtils.isNoneBlank(name)){
            condition +=" and name like '%"+name+"%' ";
        }

        List<Map<String, Object>> list = hibenateUtils.createSQLQuery(sql+condition+orderBy, page, size);
        String sqlCount ="select COUNT(1) as total from base_hospital_medal where 1=1 ";
        List<Map<String, Object>> rstotal = hibenateUtils.createSQLQuery(sqlCount+condition+orderBy);
        Long count = 0L;
        if (rstotal != null && rstotal.size() > 0) {
            count = Long.parseLong(rstotal.get(0).get("total").toString());
        }
        return MixEnvelop.getSuccessListWithPage("success", list, page, size, count);
    }


    /**
     * 医生端勋章列表
     *
     * @param doctorId
     * @param page
     * @param size
     * @return
     */
    public JSONObject selectMedalByUserId(String doctorId, Integer page, Integer size){
        JSONObject object = new JSONObject();
        String integrateSql ="SELECT\n" +
                "\tIFNULL(SUM(integrate), 0) AS total\n" +
                "FROM\n" +
                "\tbase_hospital_user_integrate\n" +
                "WHERE\n" +
                "\tuser = '"+doctorId+"' and status=1 ";
        List<Map<String, Object>> rstotal1 = hibenateUtils.createSQLQuery(integrateSql);
        Double integrateTotal = 0.0;
        if (rstotal1 != null && rstotal1.size() > 0) {
            integrateTotal = Double.parseDouble(rstotal1.get(0).get("total").toString());
        }
        String medalCount ="select COUNT(1) as total from base_hospital_medal where 1=1 and integrate<='"+integrateTotal+"'";
        List<Map<String, Object>> rstotal2 = hibenateUtils.createSQLQuery(medalCount);
        Long medalTotal = 0L;
        if (rstotal2 != null && rstotal2.size() > 0) {
            medalTotal = Long.parseLong(rstotal2.get(0).get("total").toString());
        }
        object.put("medalTotal",medalTotal);//医生勋章数量
        String orderBy = "  order by create_time desc  ";
        String condition = " ";
        String sql = "SELECT\n" +
                "\tid,\n" +
                "\tname,\n" +
                "\tbanner,\n" +
                "\tintegrate_type AS integrateType,\n" +
                "\tintegrate_type_name AS integrateTypeName,\n" +
                "\tintegrate,\n" +
                "\tbelong,\n" +
                "\tdate_format(\n" +
                "\t\tcreate_time,\n" +
                "\t\t'%Y-%m-%d %H:%i:%S'\n" +
                "\t) AS createTime,\n" +
                "\tstatus\n" +
                "FROM\n" +
                "\tbase_hospital_medal  where 1=1  ";
        List<Map<String, Object>> list = hibenateUtils.createSQLQuery(sql+condition+orderBy, page, size);
        for (Map<String,Object> map:list){
            Double integrate = Double.parseDouble(map.get("integrate").toString());
            if (integrateTotal>=integrate){
                map.put("isShow",1);
            }else {
                map.put("isShow",0);
            }
        }
        String sqlCount ="select COUNT(1) as total from base_hospital_medal where 1=1 ";
        List<Map<String, Object>> rstotal = hibenateUtils.createSQLQuery(sqlCount+condition+orderBy);
        Long count = 0L;
        if (rstotal != null && rstotal.size() > 0) {
            count = Long.parseLong(rstotal.get(0).get("total").toString());
        }
        object.put("list",list);
        object.put("page",page);
        object.put("size",size);
        object.put("total",count);
        return object;
    }

}
