package com.yihu.jw.hospital.message.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.im.ConsultDo;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.door.SignFamily;
import com.yihu.jw.entity.hospital.message.MessageNoticeSetting;
import com.yihu.jw.entity.hospital.message.SpecialistDynamicMessages;
import com.yihu.jw.entity.hospital.message.SystemMessageDO;
import com.yihu.jw.entity.hospital.prescription.WlyyOutpatientDO;
import com.yihu.jw.hospital.message.dao.MessageNoticeSettingDao;
import com.yihu.jw.hospital.message.dao.SpecialistDynamicMessagesDao;
import com.yihu.jw.hospital.message.dao.SystemMessageDao;
import com.yihu.jw.hospital.prescription.dao.OutpatientDao;
import com.yihu.jw.mysql.query.BaseJpaService;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.utils.StringUtil;
import com.yihu.jw.utils.YkyySMSService;
import com.yihu.jw.utils.hibernate.HibenateUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author zmk
 * @vsrsion 1.0
 * Created at 2019/11/13
 */
@Service
@Transactional
public class SystemMessageService extends BaseJpaService<SystemMessageDO, SystemMessageDao> {

    @Autowired
    private SystemMessageDao systemMessageDao;
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private OutpatientDao outpatientDao;
    @Autowired
    private BaseDoctorDao baseDoctorDao;
    @Autowired
    private YkyySMSService ykyySMSService;
    @Value("${wechat.id}")
    private String wechatId;
    @Autowired
    private HibenateUtils hibenateUtils;

    private final String MESSAGE_NOTICESETTING = "wlyy:message:setting:";
    @Autowired
    private MessageNoticeSettingDao messageNoticeSettingDao;
    @Autowired
    private StringRedisTemplate redisTemplate;

    @Autowired
    SpecialistDynamicMessagesDao specialistDynamicMessagesDao;

    /**
     * 新增线上就诊消息
     *
     * @param systemMessageDO
     * @return
     * @throws Exception
     */
    public SystemMessageDO addMessage(SystemMessageDO systemMessageDO) throws Exception {
        systemMessageDO.setIsRead("0");
        systemMessageDO.setDel("1");
        systemMessageDO.setCreateTime((new Date()));
        systemMessageDO = systemMessageDao.save(systemMessageDO);
        return systemMessageDO;
    }


    /**
     * 更新线上就诊消息
     *
     * @param id
     * @param read
     * @param del
     * @return
     * @throws Exception
     */
    public SystemMessageDO updateMessage(String id, String read, String del) throws Exception {
        SystemMessageDO systemMessageDO = systemMessageDao.findById(id).orElse(null);
        if (null == systemMessageDO) {
            return null;
        }
        if (!StringUtil.isEmpty(read)) {
            systemMessageDO.setIsRead(read);
            systemMessageDO.setReadTime(new Date());
        }
        if (!StringUtil.isEmpty(del)) {
            systemMessageDO.setDel(del);
        }
        systemMessageDao.save(systemMessageDO);
        return systemMessageDO;
    }


    /**
     * 保存线上就诊消息（外部调用）
     *
     * @param systemMessageDO
     * @throws Exception
     */
    public void saveMessage(SystemMessageDO systemMessageDO) throws Exception {
        systemMessageDO.setIsRead("0");
        systemMessageDO.setDel("1");
        systemMessageDO.setCreateTime(new Date());
        systemMessageDao.save(systemMessageDO);
    }


    /**
     * 根据id查询消息详情
     *
     * @param id
     * @return
     * @throws Exception
     */
    public SystemMessageDO queryById(String id) throws Exception {
        SystemMessageDO systemMessageDO = systemMessageDao.queryById(id);
        return systemMessageDO;
    }

    /**
     * 发送系统消息通用接口
     */
    public SystemMessageDO commenSystemMessage(String jsonParam, String jsonContent) throws Exception {
        if (StringUtils.isNoneBlank(jsonParam)) {
            SystemMessageDO systemMessageDO = objectMapper.readValue(jsonParam, SystemMessageDO.class);
            systemMessageDO.setDel("1");
            systemMessageDO.setIsRead("0");
            if (StringUtils.isNoneBlank(jsonContent)) {
                JSONObject jsonObject = JSONObject.parseObject(jsonContent);
                systemMessageDO.setData(jsonObject.toJSONString());
            }
            systemMessageDao.save(systemMessageDO);
            return systemMessageDO;
        }
        return null;
    }

    public String sendDoctorRemindSms(ConsultDo consultDo) {
        String msg = "";
        if ("xm_ykyy_wx".equalsIgnoreCase(wechatId)) {
            if (consultDo != null) {
                WlyyOutpatientDO wlyyOutpatientDO = outpatientDao.findById(consultDo.getRelationCode()).orElse(null);
                if (wlyyOutpatientDO != null) {
                    BaseDoctorDO baseDoctorDO = baseDoctorDao.findByIdAndDel(wlyyOutpatientDO.getDoctor());
                    if (baseDoctorDO != null) {
                        ykyySMSService.sendSmsByTempcode("message_remind_paitent", wlyyOutpatientDO, null, baseDoctorDO.getMobile());
                        msg = "发送成功";
                    } else {
                        msg = "发送失败，医生不存在";
                    }
                } else {
                    msg = "发送失败，门诊不存在";
                }
            } else {
                msg = "发送失败，consult不存在";
            }
        }

        return msg;
    }
    /*private MixEnvelop findOutPatientMessage(String reciver ,Integer page ,Integer pageSize){
        String sql = "select b.id as \"id\"," +
                " b.content as \"content\"," +
                "b.saas_id as \"saasId\"," +
                "b.is_read as \"isRead\"," +
                "b.type as \"type\"," +
                "b.sender as \"sender\"," +
                "b.sender_name as \"senderName\"," +
                "b.receiver as \"receiver\"," +
                "b.receiver_name as \"receiverName\"," +
                "b.relation_type as \"relationType\"," +
                "b.relation_code as \"relationCode\"," +
                "b.relation_name as \"relationName\"," +
                "b.status as \"status\"," +
                "b.del as \"del\"," +
                "b.create_user as \"createUser\"," +
                "b.create_user_name as \"createUserName\"," +
                "b.create_time as \"createTime\"," +
                "b.update_user as \"updateUser\"," +
                "b.update_user_name as \"updateUserName\"," +
                "b.update_time as \"updateTime\" " +
                " from wlyy_outpatient t left join base_user_message b " +
                " on t.id = b.relation_code " +
                " where b.del=1 ";
        if (StringUtils.isNoneBlank(reciver)){
            sql+=" and b.reciver = '"+reciver+"'";
        }
        sql+=" and t.status = '0' order by b.create_time asc ";
        String sqlAppend = sql+" and t.status != '0' order by b.create_time desc";
        List<Map<String,Object>> listfront = hi
    }*/


    /**
     * 获取消息提示开关
     *
     * @param user
     * @param type
     * @param messageType
     * @return (1开 ， 0关)
     */
    public Boolean getMessageNoticeSettingByMessageType(String user, String type, String messageType) {
        Integer re = 0;
        com.alibaba.fastjson.JSONObject setting = getMessageNoticSetting(user, type);
        Integer masterSwitch = setting.getInteger("masterSwitch");
        if (masterSwitch == 0) {
            return false;
        }
        switch (messageType) {
            case "masterSwitch":
                //总开关
                re = masterSwitch;
                break;
            case "imSwitch":
                //im消息开关
                re = setting.getInteger("imSwitch");
                break;
            case "familyTopicSwitch":
                //健管师邀请后推送开关
                Integer imSwitch = setting.getInteger("imSwitch");
                if (imSwitch == 1) {
                    re = setting.getInteger("familyTopicSwitch");
                }
                break;
            case "signSwitch":
                //签约消息开关
                re = setting.getInteger("signSwitch");
                break;
            case "healthSignSwitch":
                //体征消息开关
                re = setting.getInteger("healthSignSwitch");
                break;
            case "systemSwitch":
                //系统消息开关
                re = setting.getInteger("systemSwitch");
                break;
            case "prescriptionSwitch":
                //续方消息开关
                re = setting.getInteger("prescriptionSwitch");
                break;
            case "soundSwitch":
                //铃声提醒开关
                re = setting.getInteger("soundSwitch");
                break;
            case "vibrationSwitch":
                //振动提醒开关
                re = setting.getInteger("vibrationSwitch");
                break;
            case "coordinationSwitch":
                //协同消息开关
                re = setting.getInteger("coordinationSwitch");
                break;
            default:
                break;
        }

        return re == 1;
    }


    /**
     * 获取消息提醒设置
     *
     * @param user
     * @param type
     * @return
     */
    public com.alibaba.fastjson.JSONObject getMessageNoticSetting(String user, String type) {
        com.alibaba.fastjson.JSONObject re = null;
        String key = MESSAGE_NOTICESETTING + user + ":" + type;
        String response = redisTemplate.opsForValue().get(key);
        if (StringUtils.isBlank(response)) {
            MessageNoticeSetting messageNoticeSetting = messageNoticeSettingDao.findByUserAndType(user, type);
            if (messageNoticeSetting == null) {
                messageNoticeSetting = new MessageNoticeSetting();
                messageNoticeSetting.setCreateTime(new Date());
                messageNoticeSetting.setFamilyTopicSwitch(0);
                messageNoticeSetting.setCoordinationSwitch(1);
                messageNoticeSetting.setHealthSignSwitch(1);
                messageNoticeSetting.setImSwitch(1);
                messageNoticeSetting.setMasterSwitch(1);
                messageNoticeSetting.setPrescriptionSwitch(1);
                messageNoticeSetting.setSignSwitch(1);
                messageNoticeSetting.setSoundSwitch(1);
                messageNoticeSetting.setSystemSwitch(1);
                messageNoticeSetting.setType(type);
                messageNoticeSetting.setUser(user);
                messageNoticeSetting.setVibrationSwitch(1);
                messageNoticeSettingDao.save(messageNoticeSetting);
            }
            response = com.alibaba.fastjson.JSONObject.toJSONString(messageNoticeSetting);
            redisTemplate.opsForValue().set(key, response);
        }
        re = com.alibaba.fastjson.JSONObject.parseObject(response);
        return re;
    }


    //添加专科动态消息
    public void addSpecialistDynamicMessages(BasePatientDO patient, BaseDoctorDO specialist, BaseDoctorDO doctor, String result, String disease, String type, SignFamily signFamily) {
        try {
            String patientName = patient.getName();
            String doctorName = "";
            String specialistName = specialist.getName();
            SpecialistDynamicMessages specialistDynamicMessages = new SpecialistDynamicMessages();
            specialistDynamicMessages.setAddress(patient.getAddress());
            specialistDynamicMessages.setResult(result);
            specialistDynamicMessages.setName(patientName);
            specialistDynamicMessages.setCode(patient.getId());
            //这边需要修改医生的信息-20231019
//            if (doctor != null) {
//                specialistDynamicMessages.setDoctor(doctor.getCode());
//                specialistDynamicMessages.setHospital(doctor.getHospital());
//                doctorName = doctor.getName();
//            }
//            else if (signFamily != null) {
//                specialistDynamicMessages.setDoctor(signFamily.getDoctor());
//                specialistDynamicMessages.setHospital(signFamily.getHospital());
//                doctorName = signFamily.getDoctorName();
//            }
            specialistDynamicMessages.setDiseaseType(disease);
            specialistDynamicMessages.setCodeType("2");
            specialistDynamicMessages.setCreateTime(DateUtil.getStringDate());
            specialistDynamicMessages.setType(type);
            //这边后面改成用sql查-20231019
//            Map<String, String> copywritingMap = systemDictService.getDictMap("DynamicMessages" + type);
            Map<String, String> copywritingMap = null;
            //获取文案模板
            String content = copywritingMap.get("doctor");
            content = content.replace("【医生】", specialistName).replace("【居民】", patientName);
            //【医生】专科医生给您下转了居民【居民】
            //您代居民【居民】预约了【医生】专科医生的门诊
            //您向【医生】专科医生的发起了专科协同，居民【居民】
            //您向【医生】专科医生的发起了咨询求助，居民【居民】
            String specialistContent = copywritingMap.get("specialist");
            specialistContent = specialistContent.replace("【医生】", doctorName).replace("【居民】", patientName);
            //您给社区医生【医生】下转了居民【居民】
            //【医生】社区医生代居民【居民】预约了您的门诊
            //【医生】社区医生向您发起了专科协同，居民【居民】
            //【医生】社区医生向您发起了咨询求组，居民【居民】
            specialistDynamicMessages.setContent(content);
            specialistDynamicMessages.setSpecialistContent(specialistContent);

            specialistDynamicMessagesDao.save(specialistDynamicMessages);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 获取待接单的接口
     *
     * @param message {"read":1,"over":0}
     * @param page    页码
     * @param size    分页大小
     * @return
     * @throws Exception
     */
    public org.json.JSONObject getWaitingMessages(
            SystemMessageDO message, String types, Integer page, Integer size
    ) throws Exception {
        String sqlList = " SELECT a.*, b.`status` AS appliStatus ";
        String sqlCount = " select count(*) as total ";
        String sql = " FROM wlyy_message a " +
                " LEFT JOIN wlyy_door_service_application b ON a.relation_code = b.code ";
        sql += " where 1=1 AND a.state = 1 AND a.over = 1 ";
        if (!StringUtils.isEmpty(message.getReceiver())) {
            sql += " AND a.receiver = '" + message.getReceiver() + "'";
        }
        if (!StringUtils.isEmpty(types)) {
            sql += " AND a.type IN (" + types + ")";
        }
        List<Map<String, Object>> rstotal = jdbcTemplate.queryForList(sqlCount + sql);
        sql += " ORDER BY a.create_time DESC LIMIT " + (page - 1) * size + "," + size;
        List<Map<String, Object>> mapList = jdbcTemplate.queryForList(sqlList + sql);
        Long count = 0L;
        if (rstotal != null && rstotal.size() > 0) {
            count = (Long) rstotal.get(0).get("total");
        }
        JSONArray jsonArray = new JSONArray();
        for (Map<String, Object> one : mapList) {
            org.json.JSONObject object = new org.json.JSONObject();
            object.put("over", one.get("over"));
            object.put("code", one.get("code"));
            object.put("receiver", one.get("receiver"));
            object.put("del", one.get("del"));
            object.put("title", one.get("title"));
            object.put("type", one.get("type"));
            object.put("content", one.get("content"));
            object.put("sender", one.get("sender"));
            object.put("id", one.get("id"));
            object.put("state", one.get("state"));
            object.put("relation_code", one.get("relation_code"));
            object.put("has_read", one.get("has_read"));
            object.put("appliStatus", one.get("appliStatus"));
            Date date = (Date) one.get("create_time");
            object.put("create_time", DateUtil.dateToStr(date, "yyyy-MM-dd HH:mm:ss"));
            Date date1 = (Date) one.get("czrq");
            object.put("czrq", DateUtil.dateToStr(date1, "yyyy-MM-dd HH:mm:ss"));
            jsonArray.add(object);
        }
        org.json.JSONObject object = new org.json.JSONObject();
        object.put("total", count);
        object.put("waitingMessages", jsonArray);
        object.put("currPage", page);
        object.put("pageSize", size);
        return object;
    }


}
