package com.yihu.jw.hospital.prescription.service;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.hospital.consult.WlyyHospitalSysDictDO;
import com.yihu.jw.entity.hospital.httplog.WlyyHttpLogDO;
import com.yihu.jw.entity.hospital.mapping.DoctorMappingDO;
import com.yihu.jw.entity.hospital.mapping.PatientMappingDO;
import com.yihu.jw.entity.hospital.prescription.*;
import com.yihu.jw.hospital.dict.WlyyHospitalSysDictDao;
import com.yihu.jw.hospital.httplog.dao.WlyyHttpLogDao;
import com.yihu.jw.hospital.mapping.dao.DoctorMappingDao;
import com.yihu.jw.hospital.mapping.dao.PatientMappingDao;
import com.yihu.jw.hospital.prescription.dao.*;
import com.yihu.jw.hospital.prescription.service.entrance.HcyyEntranceService;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.restmodel.hospital.prescription.*;
import com.yihu.jw.util.common.IdCardUtil;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.http.HttpClientUtil;
import com.yihu.jw.mysql.query.BaseJpaService;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.DecimalFormat;
import java.util.*;

/**
 * Created by 海沧医院三方对接业务层
 */
@Service
@Transactional
public class HcyyPrescriptionService extends BaseJpaService<WlyyPrescriptionDO, PrescriptionDao> {

    private static final Logger logger = LoggerFactory.getLogger(HcyyPrescriptionService.class);


    @Autowired
    private OutpatientDao outpatientDao;
    @Autowired
    private BaseDoctorDao baseDoctorDao;
    @Autowired
    private DoctorMappingDao doctorMappingDao;
    @Autowired
    private PatientRegisterDao patientRegisterDao;
    @Autowired
    private WlyyHttpLogDao wlyyHttpLogDao;
    @Autowired
    private PatientMappingDao patientMappingDao;
    @Autowired
    private PrescriptionExpressageDao prescriptionExpressageDao;

    @Autowired
    private HttpClientUtil httpClientUtil;
    @Autowired
    private OauthCaConfigDao oauthCaConfigDao;
    @Autowired
    private PrescriptionInfoDao infoDao;
    @Autowired
    private PrescriptionDiagnosisDao diagnosisDao;
    @Autowired
    private BasePatientDao patientDao;
    @Autowired
    private BaseDoctorDao doctorDao;
    @Autowired
    private PrescriptionEmrDao prescriptionEmrDao;
    @Autowired
    private WlyyHospitalSysDictDao wlyyHospitalSysDictDao;
    @Autowired
    private HcyyEntranceService hcyyEntranceService;
    @Autowired
    private PrescriptionDao prescriptionDao;



    /**
     * 获取卡信息 余额
     * @param cardNo 就诊卡
     * @return
     */
    public JSONObject selectCardInfo(String cardNo) throws Exception {
        if (!StringUtils.isNoneBlank(cardNo)){
            throw new Exception("卡号不能为空");
        }
        JSONObject jsonObject = new JSONObject();
        String response=hcyyEntranceService.getCardInfo(cardNo);
        if (StringUtils.isNoneBlank(response)){
            JSONArray jsonArray = JSONObject.parseArray(response);
            if (jsonArray!=null&&jsonArray.size()!=0){
                JSONArray array = jsonArray.getJSONArray(0);
                if (array!=null&&array.size()!=0){
                    for (int i =0;i<array.size();i++){
                        JSONObject object = array.getJSONObject(i);
                        if (object.getString("CARD_STAT").equalsIgnoreCase("正常")){
                            jsonObject = object;
                        }
                    }

                }
            }

        }
        return jsonObject;
    }

    /**
     * 保存患者hisid
     * @param patient
     * @return
     */
    public String savePatientMapping(String patient) throws Exception {
        BasePatientDO patientDO = patientDao.findById(patient).orElse(null);
        PatientMappingDO patientMappingDO = patientMappingDao.findByPatient(patient);
        String mappingCode="";
        String obj =hcyyEntranceService.findPatientCard(patient);
        if (StringUtils.isNoneBlank(obj)){
            JSONArray array = JSONArray.parseArray(obj).getJSONArray(0);
            if (array!=null&&array.size()!=0){
                JSONObject object1 = array.getJSONObject(0);
                if (patientMappingDO==null){
                    patientMappingDO = new PatientMappingDO();
                    patientMappingDO.setIdcard(patientDO.getIdcard());
                    patientMappingDO.setPatientName(patientDO.getName());
                    patientMappingDO.setPatient(patientDO.getId());
                    patientMappingDO.setCreateTime(new Date());
                    patientMappingDO.setSource("1");
                    patientMappingDO.setMappingCode(object1.getString("SICKID"));
                }else {
                    patientMappingDO.setMappingCode(object1.getString("SICKID"));
                }
                mappingCode=object1.getString("SICKID");
                patientMappingDO = patientMappingDao.save(patientMappingDO);
            }
        }
        return mappingCode;
    }

    /**
     * 更新患者hisid
     * @param patient
     * @param cardNo
     * @return
     * @throws Exception
     */
    public String updatePatientMapping(String patient,String cardNo) throws Exception {
        BasePatientDO patientDO = patientDao.findById(patient).orElse(null);
        PatientMappingDO patientMappingDO = patientMappingDao.findByPatient(patient);
        String obj =hcyyEntranceService.findPatientCard(patient);
        if (StringUtils.isNoneBlank(obj)){
            JSONArray array = JSONArray.parseArray(obj).getJSONArray(0);
            for (int i=0;i<array.size();i++){
                JSONObject object1 = array.getJSONObject(i);
                logger.info("判断卡号是否相同："+object1.getString("CARD_NO")+"cardNo:"+cardNo);
                if (object1.getString("CARD_NO").equalsIgnoreCase(cardNo)){
                    logger.info("卡号相同");
                    if (patientMappingDO==null){
                        patientMappingDO = new PatientMappingDO();
                        patientMappingDO.setIdcard(patientDO.getIdcard());
                        patientMappingDO.setPatientName(patientDO.getName());
                        patientMappingDO.setPatient(patientDO.getId());
                        patientMappingDO.setCreateTime(new Date());
                        patientMappingDO.setSource("1");
                        patientMappingDO.setMappingCode(object1.getString("SICKID"));
                    }else {
                        patientMappingDO.setMappingCode(object1.getString("SICKID"));
                    }
                    patientMappingDO = patientMappingDao.save(patientMappingDO);
                }

            }
        }
        return patientMappingDO.getMappingCode();
    }

    /**
     * 获取门诊就诊记录列表
     * @param patient 居民id
     * @return
     */
    public List<WlyyOutpatientVO> selectOutpatientList(String patient, String startTime, String endTime, String cardNo) throws Exception {
        List<WlyyOutpatientVO> wlyyOutpatientVOList = new ArrayList<>();
        JSONArray array = new JSONArray();
        String response="";
        patient = updatePatientMapping(patient,cardNo);
        String obj = hcyyEntranceService.findOutpatientInfo(patient,null,startTime,endTime);
        if (StringUtils.isNoneBlank(obj)){
            array = JSONArray.parseArray(obj).getJSONArray(0);
            for (int i=0;i<array.size();i++){
                JSONObject object = array.getJSONObject(i);
                WlyyOutpatientVO outpatientVO = new WlyyOutpatientVO();
                outpatientVO.setAdmNo(isEmty(object.getString("ADMNO")));
                outpatientVO.setOriginAdmNo(isEmty(object.getString("ORIGINADMNO")));
                outpatientVO.setRegisterNo(isEmty(object.getString("REGISTERNO")));
                outpatientVO.setOriginRegisterNo(isEmty(object.getString("ORIGINREGISTERNO")));
                outpatientVO.setDept(isEmty(object.getString("DEPT")));
                outpatientVO.setDeptName(isEmty(object.getString("DEPTNAME")));
                outpatientVO.setPatient(isEmty(object.getString("PATIENT")));
                outpatientVO.setPatientName(isEmty(object.getString("PATIENTNAME")));
                outpatientVO.setDoctor(isEmty(object.getString("DOCTOR")));
                outpatientVO.setDoctorName(isEmty(object.getString("DOCTORNAME")));
                outpatientVO.setIdcard(isEmty(object.getString("IDCARD")));
                outpatientVO.setMjz(isEmty(object.getString("MJZ")));
                outpatientVO.setIcd10(isEmty(object.getString("ICDCODE")));
                outpatientVO.setIcd10Name(isEmty(object.getString("ICDNAME")));
                outpatientVO.setAdmDate(DateUtil.strToDate(object.getString("ADMDATE")));
                outpatientVO.setConDate(DateUtil.strToDate(object.getString("CONDATE")));
                outpatientVO.setDescription(isEmty(object.getString("DESCRIPTION")));
                outpatientVO.setDiseaseImg(isEmty(object.getString("DISEASEIMG")));
                outpatientVO.setCreateTime(DateUtil.strToDate(object.getString("CREATETIME")));
                outpatientVO.setStatus(isEmty(object.getString("STATUS")));
                outpatientVO.setConNo(isEmty(object.getString("VISITNUM")));
                outpatientVO.setPatient(patient);
                wlyyOutpatientVOList.add(outpatientVO);
            }
        }
        return wlyyOutpatientVOList;
    }

    /**
     * 获取单条门诊就诊记录
     * @param patient 居民id
     * @param conNo 就诊次数
     * @return
     */
    public WlyyOutpatientVO selectOutpatientInfo(String patient,String conNo) throws Exception {
        WlyyOutpatientVO outpatientVO = new WlyyOutpatientVO();
        JSONArray array = new JSONArray();
        String response="";
        BasePatientDO patientDO = patientDao.findById(patient).orElse(null);
        PatientMappingDO patientMappingDO = patientMappingDao.findByPatient(patient);
        if (patientMappingDO!=null){
            patient = patientMappingDO.getMappingCode();
        }else {
            patient = savePatientMapping(patient);
        }
        if (StringUtils.isNoneBlank(patient)) {
            String obj = hcyyEntranceService.findOutpatientInfo(patient, conNo, null, null);
            if (StringUtils.isNoneBlank(obj)) {
                array = JSONArray.parseArray(obj).getJSONArray(0);
                if (array != null && array.size() != 0) {
                    JSONObject object = array.getJSONObject(0);
                    outpatientVO.setAdmNo(isEmty(object.getString("ADMNO")));
                    outpatientVO.setOriginAdmNo(isEmty(object.getString("ORIGINADMNO")));
                    outpatientVO.setRegisterNo(isEmty(object.getString("REGISTERNO")));
                    outpatientVO.setOriginRegisterNo(isEmty(object.getString("ORIGINREGISTERNO")));
                    outpatientVO.setDept(isEmty(object.getString("DEPT")));
                    outpatientVO.setDeptName(isEmty(object.getString("DEPTNAME")));
                    outpatientVO.setPatientName(isEmty(object.getString("PATIENTNAME")));
                    outpatientVO.setDoctor(isEmty(object.getString("DOCTOR")));
                    outpatientVO.setDoctorName(isEmty(object.getString("DOCTORNAME")));
                    outpatientVO.setIdcard(isEmty(object.getString("IDCARD")));
                    outpatientVO.setMjz(isEmty(object.getString("MJZ")));
                    outpatientVO.setIcd10(isEmty(object.getString("ICDCODE")));
                    outpatientVO.setIcd10Name(isEmty(object.getString("ICDNAME")));
                    outpatientVO.setAdmDate(DateUtil.strToDate(object.getString("ADMDATE")));
                    outpatientVO.setConDate(DateUtil.strToDate(object.getString("CONDATE")));
                    outpatientVO.setDescription(isEmty(object.getString("DESCRIPTION")));
                    outpatientVO.setDiseaseImg(isEmty(object.getString("DISEASEIMG")));
                    outpatientVO.setCreateTime(DateUtil.strToDate(object.getString("CREATETIME")));
                    outpatientVO.setStatus(isEmty(object.getString("STATUS")));
                    outpatientVO.setConNo(isEmty(object.getString("VISITNUM")));
                    outpatientVO.setHospitalName("海沧医院");
                    outpatientVO.setHospital("350211A1002");
                    outpatientVO.setSex(patientDO.getSex() + "");
                    outpatientVO.setPatient(patientDO.getId());
                    outpatientVO.setBirthday(patientDO.getBirthday());
                }
            }
        }
        return outpatientVO;
    }

    /**
     * 获取药品字典
     * @param spellCode 拼音码
     * @return
     */
    public JSONArray selectDrugDict(String spellCode,String drugNo) throws Exception {
        logger.info("spellCode"+spellCode);
        JSONArray jsonArray = new JSONArray();
        String obj = hcyyEntranceService.GetDrugDict(spellCode,drugNo);
        if (StringUtils.isNoneBlank(obj)){
            JSONArray array = JSONArray.parseArray(obj).getJSONArray(0);
            for (int i=0;i<array.size();i++){
                JSONObject jsonObject = array.getJSONObject(i);
                JSONObject object1 = new JSONObject();
                object1.put("@action","select");
                object1.put("charge_code",jsonObject.getString("PHYSIC_CODE"));
                object1.put("serial","");
                object1.put("group_no","");
                object1.put("drugname",jsonObject.getString("PHYSIC_NAME"));
                object1.put("dosage",jsonObject.getString("DOSE_UNIT"));
                object1.put("MIN_DOSE",jsonObject.getString("MIN_DOSE"));
                object1.put("mini_unit",jsonObject.getString("PHYSIC_UNIT"));
                object1.put("pack_size",jsonObject.getString("PACK_SPEC"));
                object1.put("retprice",jsonObject.getString("RETAIL_PRICE"));
                object1.put("specification",jsonObject.getString("PHYSIC_SPEC"));
                object1.put("pack_retprice",jsonObject.getString("RETAIL_PRICE"));
                object1.put("stock_amount",jsonObject.getString("QUANTITY"));
                object1.put("visible_flag",jsonObject.getString("VALID_FLAG"));
                object1.put("drug_flag",jsonObject.getString("OTC_FLAG"));
                object1.put("py_code",jsonObject.getString("SPELL_CODE"));
                object1.put("pack_unit",jsonObject.getString("PRICE_UNIT"));
                object1.put("manu_name",jsonObject.getString("SUPPLIER_NAME"));
                object1.put("weigh_unit",jsonObject.getString("DOSE_UNIT"));
                object1.put("vol_unit","");
                object1.put("mini_unit_name",jsonObject.getString("PHYSIC_UNIT"));
                DecimalFormat df =new DecimalFormat("0.00");
                Double packSize=Double.parseDouble(jsonObject.get("PACK_SPEC").toString().equals("[]")?"1":jsonObject.getString("PACK_SPEC"));
                Double retprice=Double.parseDouble(jsonObject.get("RETAIL_PRICE").toString().equals("[]")?"0":jsonObject.getString("RETAIL_PRICE"));
                Double miniUnitPrice=0.00;
                if (0.00!=packSize){
                    miniUnitPrice=retprice/packSize;
                }
                object1.put("mini_unit_price",df.format(miniUnitPrice));
                object1.put("pack_unit_name",jsonObject.getString("PRICE_UNIT"));
                object1.put("weigh_unit_name",jsonObject.getString("DOSE_UNIT"));
                object1.put("vol_unit_name","");
                object1.put("win_no","");
                object1.put("jbyw",jsonObject.getString("BASE_PHYSIC_FLAG"));
                object1.put("gwyp","");
                object1.put("bz_dosage",jsonObject.getString("DOSEAGE"));
                object1.put("bz_supply",jsonObject.getString("USAGE"));
                object1.put("bz_supply_name",jsonObject.getString("USAGE"));
                object1.put("bz_unit",jsonObject.getString("DOSEAGE_UNIT"));
                object1.put("bz_unit_name",jsonObject.getString("DOSEAGE_UNIT"));
                object1.put("bz_usage",jsonObject.getString("FREQUENCY"));
                object1.put("bz_usage_name",jsonObject.getString("FREQUENCY"));
                object1.put("QUANTITY",jsonObject.getString("QUANTITY"));
                object1.put("DAY_COUNT",jsonObject.getString("DAY_COUNT"));
                jsonArray.add(object1);
            }
        }
        return jsonArray;
    }


    /**
     * 获取icd10字典
     * @param spellCode 拼音码
     * @return
     */
    public JSONArray selectIcd10Dict(String spellCode) throws Exception {
        JSONArray jsonArray = new JSONArray();
        String obj = hcyyEntranceService.GetICD10Dict(spellCode);
        if (StringUtils.isNoneBlank(obj)){

        }
        JSONArray array = JSONArray.parseArray(obj).getJSONArray(0);
        for (int i=0;i<array.size();i++){
            JSONObject jsonObject = array.getJSONObject(i);
            JSONObject object1 = new JSONObject();
            object1.put("@action","select");
            object1.put("add_code","");
            object1.put("code",jsonObject.getString("ICD10_CODE"));
            object1.put("d_code",jsonObject.getString(""));
            object1.put("name",jsonObject.getString("ICD10_NAME"));
            object1.put("py_code",jsonObject.getString("SPELL_CODE"));
            object1.put("valid_flag",jsonObject.getString("VALID_FLAG"));
            object1.put("icd10_type",jsonObject.getString("ICD10_TYPE"));
            object1.put("icd10_class",jsonObject.getString("ICD10_CLASS"));
            jsonArray.add(object1);
        }
        return jsonArray;
    }


    /**
     * 查询药品用药方法
     * @param spellCode 拼音码
     * @return
     */
    public JSONArray selectMedicineUsed(String spellCode) throws Exception {
        JSONArray jsonArray = new JSONArray();
        String obj = hcyyEntranceService.GetMedicineUsed(spellCode);
        if (StringUtils.isNoneBlank(obj)){
            JSONArray array = JSONArray.parseArray(obj).getJSONArray(0);
            for (int i=0;i<array.size();i++){
                JSONObject jsonObject = array.getJSONObject(i);
                JSONObject object1 = new JSONObject();
                object1.put("@action","select");
                object1.put("d_code",jsonObject.getString("ITEM_CODE"));
                object1.put("supply_code",jsonObject.getString("TAKE_MEDICINE_WAYS_CODE"));
                object1.put("supply_name",jsonObject.getString("TAKE_MEDICINE_WAYS_NAME"));
                object1.put("py_code",jsonObject.getString("SPELL_CODE"));
                object1.put("del_flag","");
                object1.put("yb_code","");
                object1.put("yb_name",jsonObject.getString("MEMO"));
                object1.put("print_name",jsonObject.getString("TAKE_MEDICINE_WAYS_NAME"));
                jsonArray.add(object1);
            }
        }
        return jsonArray;
    }


    /**
     * 获取药品用药频次字典
     * @return
     */
    public JSONArray selectFrequencyDict() throws Exception {
        JSONArray jsonArray = new JSONArray();
        String obj = hcyyEntranceService.GetFrequencyDict();
        if (StringUtils.isNoneBlank(obj)){
            JSONArray array = JSONArray.parseArray(obj).getJSONArray(0);
            for (int i=0;i<array.size();i++){
                JSONObject jsonObject = array.getJSONObject(i);
                JSONObject object1 = new JSONObject();
                object1.put("@action","select");
                object1.put("DAILY_TIMES",jsonObject.getString("FREQ_COUNT"));
                object1.put("FREQ_CODE",jsonObject.getString("FREQ_DESCRIBE"));
                object1.put("FREQ_DESCRIBE_CODE",jsonObject.getString("FREQ_DESCRIBE_CODE"));
                object1.put("FREQ_DESCRIBE",jsonObject.getString("FREQ_DESCRIBE"));
                object1.put("PHYSIC_NOTUSE_FLAG",jsonObject.getString("PHYSIC_NOTUSE_FLAG"));
                object1.put("FREQ_NAME",jsonObject.getString("FREQ_MEMO"));
                object1.put("INSUR_FREQ_DESCRIBE",jsonObject.getString("INSUR_FREQ_DESCRIBE"));
                object1.put("FREQ_INTERVAL",jsonObject.getString("FREQ_INTERVAL"));
                object1.put("FREQ_INTERVAL_UNIT",jsonObject.getString("FREQ_INTERVAL_UNIT"));
                object1.put("DEFAULT_SCHEDULE",jsonObject.getString("DEFAULT_SCHEDULE"));
                object1.put("EXECUTE_TIME_FLAG",jsonObject.getString("EXECUTE_TIME_FLAG"));
                object1.put("DAY_FLAG",jsonObject.getString("DAY_FLAG"));
                object1.put("WEEK_FLAG",jsonObject.getString("WEEK_FLAG"));
                object1.put("WEEK_1",jsonObject.getString("WEEK_1"));
                object1.put("WEEK_2",jsonObject.getString("WEEK_2"));
                object1.put("WEEK_3",jsonObject.getString("WEEK_3"));
                object1.put("WEEK_4",jsonObject.getString("WEEK_4"));
                object1.put("WEEK_5",jsonObject.getString("WEEK_5"));
                object1.put("WEEK_6",jsonObject.getString("WEEK_6"));
                object1.put("WEEK_7",jsonObject.getString("WEEK_7"));
                object1.put("SPELL_CODE",jsonObject.getString("SPELL_CODE"));
                object1.put("VALID_FLAG",jsonObject.getString("VALID_FLAG"));
                object1.put("WEEKLY_TIMES","");
                object1.put("IS_OP_IP","");
                object1.put("HOSP_CODE","");
                object1.put("FREQ_SP",jsonObject.getString("MEMO"));
                jsonArray.add(object1);
            }
        }
        return jsonArray;
    }

    /**
     * 获取历史病历
     * @param patientId 病人id
     * @param eventNo 病人住院号
     * @return
     */
    public List<WlyyPrescriptionEmrDO> selectHistoryEmr(String patientId,String eventNo) throws Exception {
        List<WlyyPrescriptionEmrDO> wlyyPrescriptionEmrDOList = new ArrayList<>();
        String obj = hcyyEntranceService.SelectEmrInfo(patientId,patientId);
        if("[[]]".equalsIgnoreCase(obj)||"[]".equalsIgnoreCase(obj)||StringUtils.isBlank(obj)){
            return null;
        }else {
            logger.info("obj:" + obj);
            JSONArray array = JSONArray.parseArray(obj.replace("[]", "\"\"")).getJSONArray(0);
            logger.info("arrayResponse:" + array.toJSONString());
            for (int i = 0; i < array.size(); i++) {
                JSONObject jsonObject = array.getJSONObject(i);
                logger.info("jsonObject:" + jsonObject);
                WlyyPrescriptionEmrDO wlyyPrescriptionEmrDO = new WlyyPrescriptionEmrDO();
                wlyyPrescriptionEmrDO.setComplaint(null == jsonObject.get("CHIEF_COMPLAINT") ? "" : jsonObject.get("CHIEF_COMPLAINT").toString());
                wlyyPrescriptionEmrDO.setMedicalHistory(null == jsonObject.get("PRESENT_ILLNESS") ? "" : jsonObject.get("PRESENT_ILLNESS").toString());
                wlyyPrescriptionEmrDO.setPastHistory(null == jsonObject.get("PAST_HISTORY") ? "" : jsonObject.get("PAST_HISTORY").toString());
                wlyyPrescriptionEmrDO.setPhysicalExamination(null == jsonObject.get("PHYSICAL_EXAMINATION") ? "" : jsonObject.get("PHYSICAL_EXAMINATION").toString());
                wlyyPrescriptionEmrDO.setAssistExamination(null == jsonObject.get("OBSERVE_RESULT") ? "" : jsonObject.get("OBSERVE_RESULT").toString());
                wlyyPrescriptionEmrDO.setAllergicHistory(null == jsonObject.get("ALLERGY_HISTORY") ? "" : jsonObject.get("ALLERGY_HISTORY").toString());
                wlyyPrescriptionEmrDO.setPopularHistory(null == jsonObject.get("EPIDEMIOLOGICAL") ? "" : jsonObject.get("EPIDEMIOLOGICAL").toString());
                wlyyPrescriptionEmrDO.setSpecialHistory(null == jsonObject.get("SPECIAL_HISTORY") ? "" : jsonObject.get("SPECIAL_HISTORY").toString());
                wlyyPrescriptionEmrDOList.add(wlyyPrescriptionEmrDO);
            }
        }
        return wlyyPrescriptionEmrDOList;
    }



    /**
     * 获取门诊下所有处方信息---内网调下
     * @param registerSn 流水号
     * @param patNo 居民id
     * @param admNo 住院唯一号
     * @param realOrder 处方号
     * @return
     */
    public List<WlyyPrescriptionVO> selectOriginPrescriptionListLocal(String registerSn, String patNo, String admNo, String realOrder) throws Exception {
        List<WlyyPrescriptionVO> prescriptionVOList = new ArrayList<>();
        String response="";
        String params = "";
        if (StringUtils.isNoneBlank(realOrder)){
            params +="&realOrder="+realOrder;
        }
        logger.info("处方request:"+registerSn+"=="+patNo+"==="+admNo+"===="+realOrder);
        String obj = hcyyEntranceService.getOriginPrescriptionList(registerSn,patNo,admNo,realOrder,null,null);
        if (StringUtils.isNoneBlank(obj)){
            JSONArray array = new JSONArray();
            array = JSONArray.parseArray(obj).getJSONArray(0);
            for (int i=0;i<array.size();i++){
                WlyyPrescriptionVO wlyyPrescriptionVO = new WlyyPrescriptionVO();
                JSONObject jsonObject = array.getJSONObject(i);
                wlyyPrescriptionVO.setCreateTime(DateUtil.strToDate(isEmty(jsonObject.getString("CREATETIME"))));
                wlyyPrescriptionVO.setCreateUser(isEmty(jsonObject.getString("CREATEUSER")));
                wlyyPrescriptionVO.setCreateUserName(isEmty(jsonObject.getString("CREATENAME")));
                wlyyPrescriptionVO.setUpdateTime(DateUtil.strToDate(isEmty(jsonObject.getString("UPDATETIME"))));
                wlyyPrescriptionVO.setUpdateUser(isEmty(jsonObject.getString("UPDATEUSER")));
                wlyyPrescriptionVO.setUpdateUserName(isEmty(jsonObject.getString("UPDATEUSERNAME")));
                wlyyPrescriptionVO.setRealOrder(isEmty(jsonObject.getString("REALORDER")));
                wlyyPrescriptionVO.setOriginRealOrder(isEmty(jsonObject.getString("ORIGINREALORDER")));
                wlyyPrescriptionVO.setAdmNo(isEmty(jsonObject.getString("ADMNO")));
                wlyyPrescriptionVO.setOriginAdmNo(isEmty(jsonObject.getString("ORIGINADMNO")));
                wlyyPrescriptionVO.setSerialNo(isEmty(jsonObject.getString("SERIALNO")));
                wlyyPrescriptionVO.setOriginSerialNo(isEmty(jsonObject.getString("ORIGINSERIALNO")));
                wlyyPrescriptionVO.setType(jsonObject.getInteger("TYPE"));
                wlyyPrescriptionVO.setPatientCode(isEmty(jsonObject.getString("PATIENTCODE")));
                wlyyPrescriptionVO.setPatientName(isEmty(jsonObject.getString("PATEINTNAME")));
                wlyyPrescriptionVO.setIdcard(isEmty(jsonObject.getString("IDCARD")));
                wlyyPrescriptionVO.setCardNo(isEmty(jsonObject.getString("CARDNO")));
                wlyyPrescriptionVO.setSsc(isEmty(jsonObject.getString("SSC")));
                wlyyPrescriptionVO.setDoctor(isEmty(jsonObject.getString("DOCTOR")));
                wlyyPrescriptionVO.setDoctorName(isEmty(jsonObject.getString("DOCTORNAME")));
                wlyyPrescriptionVO.setStatus(jsonObject.getInteger("STATUS"));
                wlyyPrescriptionVO.setMkFailReason(isEmty(jsonObject.getString("MKFAILREASON")));
                wlyyPrescriptionVO.setMkTime(DateUtil.strToDate(isEmty(jsonObject.getString("MKTIME"))));
                wlyyPrescriptionVO.setPrescribeTime(DateUtil.strToDate(isEmty(jsonObject.getString("PRESCRIBETIME"))));
                wlyyPrescriptionVO.setPrescribeReason(isEmty(jsonObject.getString("PRESCRIBEREASON")));
                wlyyPrescriptionVO.setPayTime(DateUtil.strToDate(isEmty(jsonObject.getString("PAYTIME"))));
                wlyyPrescriptionVO.setDosageTime(DateUtil.strToDate(isEmty(jsonObject.getString("DOSAGETIME"))));
                wlyyPrescriptionVO.setFinishTime(DateUtil.strToDate(isEmty(jsonObject.getString("FINISHTIME"))));
                wlyyPrescriptionVO.setDept(isEmty(jsonObject.getString("DEPT")));
                wlyyPrescriptionVO.setDeptName(isEmty(jsonObject.getString("DEPTNAME")));
                wlyyPrescriptionVO.setHospital(isEmty(jsonObject.getString("HOSPITAL")));
                wlyyPrescriptionVO.setHospitalName(isEmty(jsonObject.getString("HOSPITALNAME")));
                wlyyPrescriptionVO.setConsult(isEmty(jsonObject.getString("CONSULT")));
                wlyyPrescriptionVO.setDispensaryType(isEmty(jsonObject.getString("DISPENSARYTYPE"))!=null?Integer.parseInt(isEmty(jsonObject.getString("DISPENSARYTYPE"))):null);
                wlyyPrescriptionVO.setReason(isEmty(jsonObject.getString("REASON")));
                wlyyPrescriptionVO.setDiseaseImg(isEmty(jsonObject.getString("DISEASEIMG")));
                wlyyPrescriptionVO.setRemark(isEmty(jsonObject.getString("REMARK")));
                wlyyPrescriptionVO.setCancelReason(isEmty(jsonObject.getString("CANCELREASON")));
                wlyyPrescriptionVO.setCaCertData(isEmty(jsonObject.getString("CACERTDATA")));
                wlyyPrescriptionVO.setCaMessage(isEmty(jsonObject.getString("CAMESSAGE")));
                wlyyPrescriptionVO.setDigitalSignNo(isEmty(jsonObject.getString("DIGITALSIGNNO")));
                wlyyPrescriptionVO.setOriginalDataAbstract(isEmty(jsonObject.getString("ORIGINALDATAABSTRACT")));
                wlyyPrescriptionVO.setStrOriginalData(isEmty(jsonObject.getString("STRORIGINALDATA")));
                wlyyPrescriptionVO.setHisDeptCode(isEmty(jsonObject.getString("HISDEPTCODE")));
                wlyyPrescriptionVO.setHisDoctorCode(isEmty(jsonObject.getString("HISDOCTORCODE")));
                wlyyPrescriptionVO.setHisGisterTypeCode(isEmty(jsonObject.getString("HISGISTERTYPECODE")));
                wlyyPrescriptionVO.setHisRateTypeCode(isEmty(jsonObject.getString("HISRATETYPECODE")));
                wlyyPrescriptionVO.setHisHospital(isEmty(jsonObject.getString("HISHOSPITAL")));
                wlyyPrescriptionVO.setHisRegisterFee(isEmty(jsonObject.getString("HISREGISTERFEE"))!=null?Integer.parseInt(isEmty(jsonObject.getString("HISREGISTERFEE"))):null);
                wlyyPrescriptionVO.setPayStatus(isEmty(jsonObject.getString("PAYSTATUS"))!=null?Integer.parseInt(isEmty(jsonObject.getString("PAYSTATUS"))):null);
                //TODO 快递数据
                wlyyPrescriptionVO.setDispUser(isEmty(jsonObject.getString("DISPUSER")));
                wlyyPrescriptionVO.setDispUserName(isEmty(jsonObject.getString("DISPUSERNAME")));
                wlyyPrescriptionVO.setDispDate(DateUtil.strToDate(isEmty(jsonObject.getString("DISPDATE"))));

                //诊断
                String disagnosisObj = hcyyEntranceService.getOutpatientDiagnosis(registerSn,patNo,admNo,realOrder,null,null);
                JSONArray jsonArray = JSONArray.parseArray(disagnosisObj).getJSONArray(0);
                logger.info("诊断"+jsonArray.toJSONString());
                List<WlyyPrescriptionDiagnosisVO> prescriptionDiagnosisVOS = new ArrayList<>();
                for (int j=0;j<jsonArray.size();j++){
                    JSONObject disagnosisJson = jsonArray.getJSONObject(j);
                    WlyyPrescriptionDiagnosisVO prescriptionDiagnosisVO = new WlyyPrescriptionDiagnosisVO();
                    prescriptionDiagnosisVO.setCode(isEmty(disagnosisJson.getString("DIAGNOSECODE")));
                    prescriptionDiagnosisVO.setName(isEmty(disagnosisJson.getString("DIAGNOSENAME")));
                    prescriptionDiagnosisVO.setType(isEmty(disagnosisJson.getString("DIAGNOSETYPE"))==null?null:Integer.parseInt(disagnosisJson.getString("DIAGNOSETYPE")));
                    prescriptionDiagnosisVOS.add(prescriptionDiagnosisVO);
                }
                wlyyPrescriptionVO.setDiagnosisVOs(prescriptionDiagnosisVOS);

                //药品
                String infoObj = hcyyEntranceService.getOutpatientDrugInfo(registerSn,patNo,admNo,realOrder,null,null);
                JSONArray infoArray = JSONArray.parseArray(infoObj).getJSONArray(0);
                //药品
                List<WlyyPrescriptionInfoVO> wlyyPrescriptionInfoVOList = new ArrayList<>();
                for (int j=0;j<infoArray.size();j++){
                    WlyyPrescriptionInfoVO prescriptionInfoVO = new WlyyPrescriptionInfoVO();
                    JSONObject infoJson = infoArray.getJSONObject(j);
                    prescriptionInfoVO.setDrugNo(isEmty(infoJson.getString("DRUGNO")));
                    prescriptionInfoVO.setDrugName(isEmty(infoJson.getString("DRUGNAME")));
                    prescriptionInfoVO.setDispDeposite(isEmty(infoJson.getString("DISPDEPOSITE")));
                    prescriptionInfoVO.setDosage(isEmty(infoJson.getString("DOSAGE")));
                    prescriptionInfoVO.setQuantity(isEmty(infoJson.getString("QUANTITY")));
                    prescriptionInfoVO.setUnit(isEmty(infoJson.getString("UNIT")));
                    prescriptionInfoVO.setUnitName(isEmty(infoJson.getString("UNITNAME")));
                    prescriptionInfoVO.setPackUnitName(isEmty(infoJson.getString("PACKUNITNAME")));
                    prescriptionInfoVO.setPackUnit(isEmty(infoJson.getString("PACKUNIT")));
                    prescriptionInfoVO.setUsageCode(isEmty(infoJson.getString("USAGECODE")));
                    prescriptionInfoVO.setUsageName(isEmty(infoJson.getString("USAGENAME")));
                    prescriptionInfoVO.setSupplyCode(isEmty(infoJson.getString("SUPPLYCODE")));
                    prescriptionInfoVO.setSupplyName(isEmty(infoJson.getString("SUPPLYNAME")));
                    prescriptionInfoVO.setDays(isEmty(infoJson.getString("DAYS")));
                    prescriptionInfoVO.setFrequency(isEmty(infoJson.getString("FREQUENCY")));
                    prescriptionInfoVO.setSerial(isEmty(infoJson.getString("SERIAL")));
                /*    prescriptionInfoVO.setGroupNo(isEmty(infoJson.getString("groupNo")));*/
                    prescriptionInfoVO.setSpecification(isEmty(infoJson.getString("SPECIFICATION")));
                    prescriptionInfoVO.setPackRetprice(isEmty(infoJson.getString("PACKRETPRICE"))!=null?Double.parseDouble(isEmty(infoJson.getString("PACKRETPRICE"))):null);
                    prescriptionInfoVO.setHerbalCount(isEmty(infoJson.getString("HERBALCOUNT")));
                    prescriptionInfoVO.setPostCount(isEmty(infoJson.getString("POSTCOUNT")));
                    prescriptionInfoVO.setDel(isEmty(infoJson.getString("DEL"))!=null?Integer.parseInt(infoJson.getString("DEL")):null);
                    prescriptionInfoVO.setComm(infoJson.getString("COMM"));
                    wlyyPrescriptionInfoVOList.add(prescriptionInfoVO);
                    wlyyPrescriptionVO.setInfoVOs(wlyyPrescriptionInfoVOList);
                }


                //检查检验内容
                String inspectObj = hcyyEntranceService.getOutpatientCheckTestInfo(registerSn,patNo,admNo,realOrder,null,null);
                logger.info("检查检验内容inspectionResponse:"+inspectObj);
                List<WlyyInspectionVO> wlyyInspectionVOList = new ArrayList<>();
                JSONArray inspectionArray = JSONArray.parseArray(inspectObj).getJSONArray(0);
                for (int j=0;j<inspectionArray.size();j++){
                    WlyyInspectionVO inspectionVO = new WlyyInspectionVO();
                    JSONObject inspectionJson = inspectionArray.getJSONObject(j);
                    inspectionVO.setGroupName(isEmty(inspectionJson.getString("GROUPNAME")));
                    inspectionVO.setQuantity(isEmty(inspectionJson.getString("QUANTITYS"))!=null?Integer.parseInt(isEmty(inspectionJson.getString("QUANTITYS"))):null);
                    inspectionVO.setFrequency(isEmty(inspectionJson.getString("FREQUENCYS")));
                    inspectionVO.setParentCode(isEmty(inspectionJson.getString("PARENTCODE")));
                    inspectionVO.setParentName(isEmty(inspectionJson.getString("PARENTCODENAME")));
                    inspectionVO.setCode(isEmty(inspectionJson.getString("CODES")));
                    inspectionVO.setName(isEmty(inspectionJson.getString("NAMES")));
                    inspectionVO.setInformation(isEmty(inspectionJson.getString("INFORMATION")));
                    inspectionVO.setCheckEquip(isEmty(inspectionJson.getString("CHECKEQUIP")));
                    inspectionVO.setCheckPartName(isEmty(inspectionJson.getString("CHECKPARTNAME")));
                    inspectionVO.setCheckPart(isEmty(inspectionJson.getString("CHECKPART")));
                    wlyyInspectionVOList.add(inspectionVO);
                }
                wlyyPrescriptionVO.setInspectionVOs(wlyyInspectionVOList);
                prescriptionVOList.add(wlyyPrescriptionVO);
            }
        }
        return prescriptionVOList;
    }

    /**
     * 查询病历文书html信息
     * @param bstrEmrID
     * @return
     */
    public String selectBrowseSingleEMR(String bstrEmrID) throws Exception {
      String obj = hcyyEntranceService.BrowseSingleEMR(bstrEmrID);
        if(StringUtils.isNoneBlank(obj)){
            return obj;
        }else {
            return "获取失败";
        }
    }

    /**
     * 获取历史病历
     * @param patientId
     * @param eventNo
     * @return
     * @throws Exception
     */
    public JSONArray selectHistoryEmrFromEntrance(String patientId,String eventNo) throws Exception {
        String response="";
        JSONArray jsonArray = new JSONArray();
        PatientMappingDO mappingDO = patientMappingDao.findByPatient(patientId);
        String patient = null;
        if (mappingDO!=null){
            patient = mappingDO.getMappingCode();
        }
        String obj = hcyyEntranceService.SelectEmrInfo(patient,eventNo);
        if("[[]]".equalsIgnoreCase(obj)||"[]".equalsIgnoreCase(obj)||StringUtils.isBlank(obj)){
            return null;
        }else {
            logger.info("obj:"+obj);
            JSONArray array = JSONArray.parseArray(obj.replace("[]","\"\"")).getJSONArray(0);
            logger.info("arrayResponse:"+array.toJSONString());
            for (int j=0;j<array.size();j++){
                JSONObject object1 = array.getJSONObject(j);
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("DEPT_NAME",object1.getString("DEPT_NAME"));
                jsonObject.put("TREAT_DATE_TIME",DateUtil.getYyyymmddhhmmss(DateUtil.strToDateLong(object1.getString("VISIT_TIME"))));
                jsonObject.put("OPID",object1.getString("EMR_ID"));
                String firstFlag = object1.getString("FIRST_FLAG");
                if (firstFlag.equalsIgnoreCase("复诊")){
                    jsonObject.put("TREAT_DOCTOR_NAME",object1.getString("RETURN_DOC_SIGN"));
                }else if (firstFlag.equalsIgnoreCase("初诊")){
                    jsonObject.put("TREAT_DOCTOR_NAME",object1.getString("DOCTOR_SIGNATURE"));
                }
                jsonArray.add(jsonObject);
            }
            return jsonArray;
        }
    }

    /**
     * 查询历次就诊门诊病历文书
     * @param templateId
     * @param patientId 病人id
     * @param eventNo 病人住院号
     * @return
     */
    public String selectHistoryEmrRecord(String templateId,String patientId,String eventNo) throws Exception {
        String obj = hcyyEntranceService.SelectEmrDispRecord(templateId,patientId,eventNo);
        if(StringUtils.isNoneBlank(obj)){
            return obj;
        }else {
            return "获取失败";
        }
    }

    /**
     * 保存电子病历
     * @param prescriptionId 处方id
     * @return
     * @throws Exception
     */
    public String  saveEmrToEntrance(String prescriptionId,String baseSign) throws Exception {
        Map<String,String> params = new HashedMap();
        WlyyPrescriptionEmrDO wlyyPrescriptionEmrDO = new WlyyPrescriptionEmrDO();
        WlyyPrescriptionDO wlyyPrescriptionDO = prescriptionDao.findById(prescriptionId).orElse(null);
        JSONObject paramRequest = new JSONObject();
        if (null!=wlyyPrescriptionDO){
            paramRequest.put("patientName",wlyyPrescriptionDO.getPatientName());
            PatientMappingDO patientMappingDO =  patientMappingDao.findByPatient(wlyyPrescriptionDO.getPatientCode());
            BasePatientDO basePatientDO = patientDao.findById(wlyyPrescriptionDO.getPatientCode()).orElse(null);
            DoctorMappingDO doctorMappingDO = doctorMappingDao.findByDoctor(wlyyPrescriptionDO.getDoctor());
            WlyyOutpatientDO wlyyOutpatientDO = outpatientDao.findById(wlyyPrescriptionDO.getOutpatientId()).orElse(null);
            if(null!=patientMappingDO){
                paramRequest.put("patientid",patientMappingDO.getMappingCode());
            }
            //获取医生编号
            BaseDoctorDO doctorDO = doctorDao.findByIdAndDel(wlyyOutpatientDO.getDoctor());
            String operatorId = null;
            if (doctorDO!=null){
                String obj = hcyyEntranceService.selectDoctorInfo(doctorDO.getIdcard());
                JSONArray array = JSONArray.parseArray(obj);
                if (array!=null&&array.size()!=0){
                    JSONObject jsonObject = array.getJSONObject(0);
                    operatorId = jsonObject.getString("staffNo");
                }
            }
            if(null!=doctorMappingDO){
                paramRequest.put("operatorid",operatorId);
                paramRequest.put("operatorname",wlyyPrescriptionDO.getDoctorName());
            }
            paramRequest.put("type","2");
            wlyyPrescriptionEmrDO =  prescriptionEmrDao.findEmrByPrescriptionId(prescriptionId);
            if (null!=wlyyPrescriptionEmrDO){
                paramRequest.put("id",wlyyPrescriptionEmrDO.getId());
                params.put("emrInfo", JSON.toJSONString(wlyyPrescriptionEmrDO));
            }
            if (null!=basePatientDO){
                paramRequest.put("patientAge",IdCardUtil.getAgeForIdcard(basePatientDO.getIdcard()));
                paramRequest.put("patientsex",IdCardUtil.getSexForIdcard(basePatientDO.getIdcard()));
                paramRequest.put("patientName",basePatientDO.getName());
                paramRequest.put("patientClan","0"); }
            if (null!=wlyyOutpatientDO){
                paramRequest.put("deptCode",wlyyOutpatientDO.getDeptName());
                paramRequest.put("outpatientCode","R"+wlyyOutpatientDO.getRegisterNo());
                paramRequest.put("eventno","R"+wlyyOutpatientDO.getRegisterNo());
            }
            paramRequest.put("templateid","1158");
            paramRequest.put("catalogid","50");
            paramRequest.put("emrtype","93");
            paramRequest.put("emrtitle","互联网医院");

            if (StringUtils.isNoneBlank(baseSign)){
                //UploadVO uploadVO=fileUploadService.uploadImagesBase64(baseSign,fastdfs_file_url);
                //paramRequest.put("baseSign","https://ih.xmheart.com/fastdfs/"+uploadVO.getFullUri());
                paramRequest.put("baseSign",baseSign);
            }
        }
        params.put("jsonInfo",paramRequest.toJSONString());
        JSONObject jsonObject = JSONObject.parseObject(paramRequest.toJSONString());
        String obj = hcyyEntranceService.returnEmrHtmlByHLW(jsonObject,wlyyPrescriptionEmrDO);
        logger.info("保存电子病历"+obj);
        if (obj.contains("error")||obj.contains("ERROR")){
            return "faild";
        }else {
            return "success";
        }
    }


    /**
     * 查询检查部位
     * @param spellCode 拼音码
     * @return
     */
    public JSONArray selectCheckBody(String spellCode) throws Exception {
        JSONArray jsonArray = new JSONArray();
        String obj = hcyyEntranceService.GetCheckBody(spellCode);
        if (StringUtils.isNoneBlank(obj)){
            JSONArray array = JSONArray.parseArray(obj).getJSONArray(0);
            for (int i=0;i<array.size();i++){
                JSONObject jsonObject = array.getJSONObject(i);
                JSONObject object1 = new JSONObject();
                object1.put("@action","select");
                object1.put("name",jsonObject.getString("NAME"));
                object1.put("code",jsonObject.getString("CODE"));
                object1.put("data",jsonObject.getString("DATA"));
                object1.put("py_code",jsonObject.getString("SPELL_CODE"));
                object1.put("memo",jsonObject.getString("MEMO"));
                object1.put("wb_code",jsonObject.getString("WBZX_CODE"));
                object1.put("sequence",jsonObject.getString("SEQUENCE"));
                object1.put("valid",jsonObject.getString("VALID"));
                jsonArray.add(object1);
            }
        }
        return jsonArray;
    }
    /**
     * 查询检查项目
     * @param spellCode 拼音码或药品名字
     * @return
     */
    public JSONArray selectExamApply(String spellCode) throws Exception {
        JSONArray jsonArray = new JSONArray();
        String obj = hcyyEntranceService.GetExamApply(spellCode);
        if (StringUtils.isNoneBlank(obj)){
            JSONArray array = JSONArray.parseArray(obj).getJSONArray(0);
            for (int i=0;i<array.size();i++){
                JSONObject jsonObject = array.getJSONObject(i);
                JSONObject object1 = new JSONObject();
                object1.put("@action","select");
                object1.put("name",jsonObject.getString("ITEM_NAME"));
                object1.put("code",jsonObject.getString("ITEM_CODE"));
                object1.put("type",jsonObject.getString("ITEM_CLASS"));
                object1.put("py_code",jsonObject.getString("SPELL_CODE"));
                object1.put("charge_amount",jsonObject.getString("FACT_PRICE"));
                object1.put("child_amount",jsonObject.getString("CHILD_PRICE"));
                object1.put("charge_unit",jsonObject.getString("UNIT"));
                object1.put("wb_code",jsonObject.getString("FIVE_PEN_CODE"));
                object1.put("memo",jsonObject.getString("MEMO"));
                jsonArray.add(object1);
            }
        }
        return jsonArray;
    }
    /**
     * 查询检验项目
     * @param spellCode 拼音码或药品名字
     * @return
     */
    public JSONArray selectLisApply(String spellCode) throws Exception {
        JSONArray jsonArray = new JSONArray();
        String obj = hcyyEntranceService.GetLisApply(spellCode);
        if (StringUtils.isNoneBlank(obj)){
            JSONArray array = JSONArray.parseArray(obj).getJSONArray(0);
            for (int i=0;i<array.size();i++){
                JSONObject jsonObject = array.getJSONObject(i);
                JSONObject object1 = new JSONObject();
                object1.put("@action","select");
                object1.put("name",jsonObject.getString("ITEM_NAME"));
                object1.put("code",jsonObject.getString("ITEM_CODE"));
                object1.put("type",jsonObject.getString("ITEM_CLASS"));
                object1.put("py_code",jsonObject.getString("SPELL_CODE"));
                object1.put("charge_amount",jsonObject.getString("FACT_PRICE"));
                object1.put("child_amount",jsonObject.getString("CHILD_PRICE"));
                object1.put("charge_unit",jsonObject.getString("UNIT"));
                object1.put("wb_code",jsonObject.getString("FIVE_PEN_CODE"));
                object1.put("memo",jsonObject.getString("MEMO"));
                jsonArray.add(object1);
            }
        }
        return jsonArray;
    }


    /**
     * 检查检验项目同步his
     * @param
     * @return
     */
    public Map checkSyncHis(String IoFlag,String PayCardNo,String NullahNumber,String ApplyDept,String ApplyDoctor,List<WlyyInspectionDO> wlyyInspectionS) throws Exception {
        Map resultMap=new HashMap();
        String obj = hcyyEntranceService.saveInspectToHospital(IoFlag,PayCardNo,NullahNumber,ApplyDept,ApplyDoctor,wlyyInspectionS);
        JSONObject rootObject = JSONObject.parseObject(obj);
        if (null!=rootObject&&"0".equalsIgnoreCase(rootObject.getString("code"))){
            resultMap.put("success",rootObject.getString("success"));
            resultMap.put("code",rootObject.getString("code"));
            resultMap.put("msg",rootObject.getString("msg"));
            JSONObject dataObj = JSONObject.parseObject(rootObject.getString("data"));
            if (null!=dataObj){
                JSONObject presDetailobj = JSONObject.parseObject(dataObj.getString("PresDetail"));
                resultMap.put("msg",presDetailobj.getString("PresNo"));
                logger.info("his医嘱号:"+presDetailobj.getString("PresNo"));
            }
        }else if(null!=rootObject&&!"0".equalsIgnoreCase(rootObject.getString("code"))){
            resultMap.put("success",rootObject.getString("success"));
            resultMap.put("code",rootObject.getString("code"));
            resultMap.put("msg",rootObject.getString("msg"));
        }else {
            resultMap.put("success","obj转化失败");
        }
        return resultMap;
    }


    /**
     * 保存处方到entrance
     * @param ioFlag 门诊住院标志
     * @param payCardNo 就诊卡号
     * @param nullahNumber 挂号流水号
     * @param applyDept 执行科室
     * @param applyDoctor 开单医生
     * @param executeDept 执行科室(药房)
     * @return
     * @throws Exception
     */
    public JSONObject savePrescriptionToEntrance(String ioFlag,String payCardNo,String nullahNumber,String applyDept,String applyDoctor,String executeDept,List<WlyyPrescriptionInfoDO> wlyyPrescriptionInfoDOS) throws Exception {
        JSONObject jsonObject = new JSONObject();
        ObjectMapper mapper = new ObjectMapper();
        String obj = hcyyEntranceService.savePrescriptionToHospital(ioFlag,payCardNo,nullahNumber,applyDept,applyDoctor,executeDept,wlyyPrescriptionInfoDOS);
        logger.info("开方成功"+obj);
        if (StringUtils.isNoneBlank(obj)){
            jsonObject = JSONObject.parseObject(obj);
        }

        return jsonObject;
    }

    /**
     * 保存诊断到entrance
     * @param applyNo 必填，流水号
     * @param diagnosisName 必填，诊断名称
     * @param diagnosisCode 诊断代码
     * @return
     * @throws Exception
     */
    public JSONObject saveDiagnosisToEntrance(String applyNo,String diagnosisName,String diagnosisCode) throws Exception {
        JSONObject jsonObject = new JSONObject();
        String obj = hcyyEntranceService.saveDiagnosis(applyNo,diagnosisName,diagnosisCode,null,null,null,null);
        if (StringUtils.isNoneBlank(obj)){
            jsonObject = JSONObject.parseObject(obj);
        }
        return jsonObject;
    }

    /**
     * 发送退号信息到entrance
     * @param applyNo 必填，流水号
     * @param payCardNo 必填，卡号
     * @return
     * @throws Exception
     */
    public JSONObject refundConsultationToEntrance(String applyNo,String payCardNo) throws Exception {
        JSONObject jsonObject = new JSONObject();
        String response = "";
        String obj = hcyyEntranceService.refundConsultation(applyNo,payCardNo);
        if (StringUtils.isNoneBlank(obj)){
            jsonObject = JSONObject.parseObject(obj);
        }
        return jsonObject;
    }

    /**
     * 退号
     * @param outpatientId
     * @return
     * @throws Exception
     */
    public JSONObject refundRegisterNo(String outpatientId) throws Exception {
        JSONObject object = new JSONObject();
        WlyyOutpatientDO outpatientDO = outpatientDao.findById(outpatientId).orElse(null);
        if(outpatientDO!=null){
            DoctorMappingDO doctorMappingDO = doctorMappingDao.findByDoctor(outpatientDO.getDoctor());
            if (StringUtils.isNoneBlank(outpatientDO.getRegisterNo())){
                String obj = hcyyEntranceService.refundConsultation(outpatientDO.getRegisterNo(), outpatientDO.getCardNo());
                if (StringUtils.isNoneBlank(obj)){
                    object = JSONObject.parseObject(obj);
                }
                logger.info("退号结果 res: " + object.toString());
                if (object!=null&&object.getString("code").equalsIgnoreCase("0")){
                    WlyyPatientRegisterDO wlyyPatientRegisterDO = patientRegisterDao.findByRegisterNo(outpatientDO.getRegisterNo());
                    wlyyPatientRegisterDO.setDel(0);
                    patientRegisterDao.save(wlyyPatientRegisterDO);
                    outpatientDO.setHisStatus(-1);
                    outpatientDao.save(outpatientDO);
                }

                //保存日志
                WlyyHttpLogDO log = new WlyyHttpLogDO();
                log.setCode("refundregisterOutPatient");
                log.setName("退号");
                log.setPatient(outpatientDO.getPatient());
                log.setDoctor(outpatientDO.getDoctor());
                log.setResponse(object.toString());
                log.setRequest("outPatientId=" + outpatientId + "&doctor=" + outpatientDO.getDoctor());
                log.setStatus(object.getString("code"));
                log.setCreateTime(new Date());
                wlyyHttpLogDao.save(log);
            }
        }
        return  object;
    }

    /**
     * 删除处方信息
     * @param realOrder his处方号
     * @return
     * @throws Exception
     */
    public JSONObject deletePrescriptionToEntrance(String realOrder) throws Exception {
        Map<String,String> params = new HashedMap();
        JSONObject jsonObject = new JSONObject();
        WlyyPrescriptionDO wlyyPrescriptionDO = prescriptionDao.findByRealOrder(realOrder);
        if (null!=wlyyPrescriptionDO){
            params.put("IoFlag","0");
            WlyyOutpatientDO wlyyOutpatientDO = outpatientDao.findById(wlyyPrescriptionDO.getOutpatientId()).orElse(null);
            String PayCardNo = "";
            String ApplyDept = "";
            String NullahNumber = "";
            if (null!=wlyyOutpatientDO){
                PayCardNo=wlyyOutpatientDO.getCardNo();
                ApplyDept=wlyyOutpatientDO.getDept();
                NullahNumber=wlyyOutpatientDO.getRegisterNo();
            }
            DoctorMappingDO doctorMappingDO = doctorMappingDao.findByDoctor(wlyyPrescriptionDO.getDoctor());
            String ApplyDoctor = "";
            if (null!=doctorMappingDO){
                ApplyDoctor=doctorMappingDO.getDoctorName()+"/"+doctorMappingDO.getMappingCode();
            }
            String ExecuteDept="50100";
            String PresNo=realOrder;
            String obj = hcyyEntranceService.delHisPrescriptionByHisNo("0",PayCardNo,NullahNumber,ApplyDept,ApplyDoctor,ExecuteDept,PresNo);
            logger.info("删除处方信息"+obj);
            jsonObject = JSONObject.parseObject(obj);
        }
        return jsonObject;
    }


    /**
     * 挂号接口
     * @param outPatientId
     * @param doctor
     * @return
     */
    public JSONObject registerOutPatient(String outPatientId, String doctor){
        WlyyOutpatientDO outpatientDO = outpatientDao.findById(outPatientId).orElse(null);
        List<WlyyPrescriptionExpressageDO> expressageDOS =prescriptionExpressageDao.findByOutpatientId(outPatientId);
        DoctorMappingDO doctorMappingDO = doctorMappingDao.findByDoctor(doctor);
        BaseDoctorDO doctorDO = baseDoctorDao.findByIdAndDel(doctor);
        if (doctorMappingDO == null) {
            throw new RuntimeException("未找到医生映射信息");
        }
        PatientMappingDO patientMappingDO = patientMappingDao.findByPatient(outpatientDO.getPatient());
        if (patientMappingDO==null){
            throw new RuntimeException("未找到居民映射信息");
        }

        //查找居民当天挂号记录
        String date = DateUtil.dateToStr(new Date(), "yyyy-MM-dd");
        List<WlyyPatientRegisterDO> patientRegisterDOs = patientRegisterDao.findByPatientAndDateAndDoctorAndDel(outpatientDO.getPatient(), date, doctor,1);
        if (patientRegisterDOs != null && patientRegisterDOs.size() > 0) {
            WlyyPatientRegisterDO registerDO = patientRegisterDOs.get(0);
            //判断是否已经挂号，如果已经挂号
            if (StringUtils.isNotBlank(registerDO.getRegisterNo())) {

                JSONObject res = new JSONObject();
                res.put("@RESULT", "0");
                logger.info("已经挂号 res: " + res.toString());

                //更新挂号流水号
                outpatientDO.setConNo(registerDO.getConNo());
                outpatientDO.setRegisterNo(registerDO.getRegisterNo());
                outpatientDO.setAdmDate(registerDO.getCreateTime());
                outpatientDO.setAdmNo(registerDO.getRegisterNo());
                outpatientDao.save(outpatientDO);
                return res;
            }
        }
        String doctorFlag =  doctorMappingDO.getDoctorName()+"/"+doctorMappingDO.getMappingCode();
        String courierflag = "N";
        if (expressageDOS!=null&&expressageDOS.size()!=0){
            WlyyPrescriptionExpressageDO expressageDO = expressageDOS.get(0);
            if (expressageDO.getOneselfPickupFlg()!=null&&expressageDO.getOneselfPickupFlg()==1){
                courierflag="N";
            }else if (expressageDO.getOneselfPickupFlg()!=null&&expressageDO.getOneselfPickupFlg()==0){
                courierflag="Y";
            }
        }
        String chargeType = "";
        if ("1".equalsIgnoreCase(outpatientDO.getOutpatientType())&&"1".equalsIgnoreCase(outpatientDO.getType())){
            chargeType = doctorDO.getTwfzChargeType();
        }else if ("1".equalsIgnoreCase(outpatientDO.getOutpatientType())&&"2".equalsIgnoreCase(outpatientDO.getType())){
            chargeType = doctorDO.getSpfzChargeType();
        }else if ("3".equalsIgnoreCase(outpatientDO.getOutpatientType())&&"1".equalsIgnoreCase(outpatientDO.getType())){
            chargeType = doctorDO.getTwzxChargeType();
        }else if ("3".equalsIgnoreCase(outpatientDO.getOutpatientType())&&"2".equalsIgnoreCase(outpatientDO.getType())){
            chargeType = doctorDO.getSpzxChargeType();
        }else if ("3".equalsIgnoreCase(outpatientDO.getOutpatientType())&&"3".equalsIgnoreCase(outpatientDO.getType())){
            chargeType = doctorDO.getJyzxChargeType();
        }else if ("2".equalsIgnoreCase(outpatientDO.getOutpatientType())){
            chargeType = doctorDO.getXtfzChargeType();
        }else {
            chargeType = doctorDO.getChargeType();
        }
        JSONObject res = null;
        try {
            String obj = hcyyEntranceService.registered(doctorFlag,outpatientDO.getDept(),patientMappingDO.getMappingCode(),chargeType,courierflag);
            if (StringUtils.isNoneBlank(obj)){
                res = JSONObject.parseObject(obj);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        JSONObject result = new JSONObject();
        logger.info("挂号结果 res: " + res.toString());
        String rsCode = "";
        String serialNo = "";
        if (res!=null){
            JSONObject object = res.getJSONObject("returnContents");
            serialNo = object.getString("registerNo");
            if (object!=null){
                result.put("@RESULT", "0");
                result.put("@MSG","挂号成功！");
                rsCode = object.getString("registerNo");
            }
        }else {
            result.put("@RESULT", "-1");
            result.put("@MSG","挂号失败");
        }


        if (StringUtils.isNoneBlank(rsCode)) {
            //存储挂号号

            outpatientDO.setRegisterNo(serialNo);
            outpatientDO.setAdmDate(new Date());
            outpatientDO.setAdmNo(serialNo);
            logger.info("挂号流水 registerNo: ");
            outpatientDao.save(outpatientDO);

            //存储就诊记录
            WlyyPatientRegisterDO registerDO = new WlyyPatientRegisterDO();
            // 存储挂号医生与挂号科室
            registerDO.setDoctor(outpatientDO.getDoctor());
            registerDO.setDoctorName(outpatientDO.getDoctorName());
            registerDO.setDept(outpatientDO.getDept());
            registerDO.setDeptName(outpatientDO.getDeptName());
            registerDO.setDel(1);

            registerDO.setPatient(outpatientDO.getPatient());
            registerDO.setPatientName(outpatientDO.getPatientName());
            //挂号流水号与挂号次数
            registerDO.setRegisterNo(serialNo);
            registerDO.setCreateTime(new Date());
            registerDO.setDate(DateUtil.dateToStr(new Date(), "yyyy-MM-dd"));
            patientRegisterDao.save(registerDO);

        } else if ("-2".equals(rsCode)) {

            outpatientDO.setRegisterNo(serialNo);
            outpatientDO.setAdmDate(new Date());
            outpatientDO.setAdmNo(serialNo);
            logger.info("挂号流水 registerNo: " + serialNo );
            outpatientDao.save(outpatientDO);

            //存储就诊记录
            WlyyPatientRegisterDO registerDO = new WlyyPatientRegisterDO();

            registerDO.setDoctor(outpatientDO.getDoctor());
            registerDO.setDoctorName(outpatientDO.getDoctorName());
            registerDO.setDept(outpatientDO.getDept());
            registerDO.setDeptName(outpatientDO.getDeptName());
            registerDO.setDel(1);

            registerDO.setPatient(outpatientDO.getPatient());
            registerDO.setPatientName(outpatientDO.getPatientName());
            registerDO.setRegisterNo(serialNo);
            registerDO.setCreateTime(new Date());
            registerDO.setDate(DateUtil.dateToStr(new Date(), "yyyy-MM-dd"));

            patientRegisterDao.save(registerDO);

        }
        //保存日志
        WlyyHttpLogDO log = new WlyyHttpLogDO();
        log.setCode("registerOutPatient");
        log.setName("挂号");
        log.setPatient(outpatientDO.getPatient());
        log.setDoctor(doctor);
        log.setResponse(res.toString());
        log.setRequest("outPatientId=" + outPatientId + "&doctor=" + doctor);
        log.setStatus("1");
        log.setCreateTime(new Date());
        wlyyHttpLogDao.save(log);
        return result;
    }

    public String isEmty(String str){
        if (str.equalsIgnoreCase("[]")){
            return null;
        }else {
            return str;
        }
    }

    //======================合理用药===========================
    public String getUseDrugUrl(){

        List<WlyyHospitalSysDictDO> list =  wlyyHospitalSysDictDao.findByDictName("useDrugUrl");
        String url = list.get(0).getDictCode();
        return url;
    }

    /**
     * 3.1	基础标记
     *
     * @param hospitalCode 医院/院区编码
     * @param eventNo 就诊流水号
     * @param patientId 患者id
     * @param source 请求数据的来源，传入固定值：门诊
     * @return
     */
    public String initBaseParam(String hospitalCode,String eventNo,String patientId,String source){
        String base = "<base> ";
        source = "门诊";
        if (StringUtils.isNoneBlank(hospitalCode)){
            base +=" <hospital_code>"+hospitalCode+"</hospital_code> ";
        }

        if (StringUtils.isNoneBlank(eventNo)){
            base +=" <event_no>"+eventNo+"</event_no> ";
        }
        if (StringUtils.isNoneBlank(patientId)){
            base +=" <patient_id>"+patientId+"</patient_id> ";
        }
        if (StringUtils.isNoneBlank(source)){
            base +=" <source>"+source+"</source> ";
        }
        base+="</base>";
        return base;
    }


    /**
     *3.2	患者就诊
     * @param sex 传入4种值：男/女/M/F
     * @param name
     * @param idType 患者在医院登记的证件类型，如：身份证、户口本等
     * @param idNo 患者在医院登记证件对应的号码
     * @param birthWeight 患者出生时的体重, 必须包含数字和单位，计量单位：g，如3500g
     * @param birthday 患者的出生日期，格式：yyyy-mm-dd
     * @param medCardNo 患者就诊时所使用的医疗卡号
     * @param eventTime 患者到医院的就诊时间，一般为挂号时间，格式：yyyy-mm-dd hh24:mi:ss
     * @param deptId 系统运行核心关联字段，患者挂号的科室id
     * @param deptName 患者挂号的科室名称
     * @param payType 患者付费方式，如：自费、市医保、省医保等
     * @param pregnancy 是否怀孕 1: 是0: 否
     * @param timeOfPreg 孕期 孕妇的妊娠时长，只传数字，默认计量单位：周
     * @param breastFeeding 是否哺乳1: 是0: 否
     * @param height 身高
     * @param weight 体重
     * @param address 联系地址
     * @param phoneNo 联系电话
     * @param dialysis 是否透析 1: 是0: 否
     * @param patientCondition 一些笼统的描述信息，如：普通病人、危重病人
     * @return
     */
    public String initOptPatientParam(String sex,String name,String idType,String idNo,String birthWeight,String birthday,String medCardNo,
                                      String eventTime,String deptId,String deptName,String payType,String pregnancy,String timeOfPreg,String breastFeeding,
                                      String height,String weight,String address,String phoneNo,String dialysis,String patientCondition){
        String optPatient = "<opt_patient> ";
        if (StringUtils.isNoneBlank(sex)){
            optPatient +=" <sex>"+sex+"</sex> ";
        }
        if (StringUtils.isNoneBlank(name)){
            optPatient +=" <name>"+name+"</name> ";
        }
        if (StringUtils.isNoneBlank(idType)){
            optPatient +=" <id_type>"+idType+"</id_type> ";
        }
        if (StringUtils.isNoneBlank(idNo)){
            optPatient +=" <id_no>"+idNo+"</id_no> ";
        }
        if (StringUtils.isNoneBlank(birthWeight)){
            optPatient +=" <birth_weight>"+birthWeight+"</birth_weight> ";
        }
        if (StringUtils.isNoneBlank(birthday)){
            optPatient +=" <birthday>"+birthday+"</birthday> ";
        }
        if (StringUtils.isNoneBlank(medCardNo)){
            optPatient +=" <med_card_no>"+medCardNo+"</med_card_no> ";
        }
        if (StringUtils.isNoneBlank(eventTime)){
            optPatient +=" <event_time>"+eventTime+"</event_time> ";
        }
        if (StringUtils.isNoneBlank(deptId)){
            optPatient +=" <dept_id>"+deptId+"</dept_id> ";
        }
        if (StringUtils.isNoneBlank(deptName)){
            optPatient +=" <dept_name>"+deptName+"</dept_name> ";
        }
        if (StringUtils.isNoneBlank(payType)){
            optPatient +=" <pay_type>"+payType+"</pay_type> ";
        }
        if (StringUtils.isNoneBlank(pregnancy)){
            optPatient +=" <pregnancy>"+pregnancy+"</pregnancy> ";
        }
        if (StringUtils.isNoneBlank(timeOfPreg)){
            optPatient +=" <time_of_preg>"+timeOfPreg+"</time_of_preg> ";
        }
        if (StringUtils.isNoneBlank(breastFeeding)){
            optPatient +=" <breast_feeding>"+breastFeeding+"</breast_feeding> ";
        }

        if (StringUtils.isNoneBlank(height)){
            optPatient +=" <height>"+height+"</height> ";
        }
        if (StringUtils.isNoneBlank(weight)){
            optPatient +=" <weight>"+weight+"</weight> ";
        }
        if (StringUtils.isNoneBlank(address)){
            optPatient +=" <address>"+address+"</address> ";
        }
        if (StringUtils.isNoneBlank(phoneNo)){
            optPatient +=" <phone_no>"+phoneNo+"</phone_no> ";
        }
        if (StringUtils.isNoneBlank(dialysis)){
            optPatient +=" <dialysis>"+dialysis+"</dialysis> ";
        }
        if (StringUtils.isNoneBlank(patientCondition)){
            optPatient +=" <patient_condition>"+patientCondition+"</patient_condition> ";
        }
        optPatient+="</opt_patient>";
        return optPatient;
    }


    /**3.3.1	处方单
     *
     * @param recipeId  系统运行核心关联字段，一张处方实体的唯一标记，院内唯一，主键
     * @param recipeNo 如不存在，和处方id传值一样即可
     * @param recipeSource 三种值：门诊，急诊
     * @param recipeType 三种值：西药方，中成药方，草药方
     * @param recipeDocId 系统运行核心关联字段，开方医生工号
     * @param recipeDocName 开方医生姓名
     * @param recipeDocTitle 开方医生的职称，如：主治医师、副主任医师、主任医师等。
     * @param deptId 系统运行核心关联字段，开方医师所属科室id
     * @param deptName 开方医师所属科室名称
     * @param recipeTime 系统运行核心字段，格式：yyyy-mm-dd hh24:mi:ss
     * @param herbUnitPrice 如果是中药饮片处方，此处必填
     * @param herbPacketCount 如果是中药饮片处方，此处必填
     * @param recipeFeeTotal 处方总金额
     * @param recipeStatus 0: 新开或修改处方。数据要求：传入的所有药品明细发药数量值均必须大于0
     * @param originalRecipeId 当Recipe_status为1时表示退药处方时，则必须传入当前退药处方所对应原始处方的处方id
     * @param urgentFlag 用于处方的优先审核 1: 是0: 否

     * @return
     */
    public String initPrescriptionInfoParam(String recipeId,String recipeNo,String recipeSource,String recipeType,String recipeDocId,String recipeDocName,String recipeDocTitle,
                                      String deptId,String deptName,String recipeTime,String herbUnitPrice,String herbPacketCount,String recipeFeeTotal,
                                      String recipeStatus,String originalRecipeId,String urgentFlag){
        String optPatient = "<opt_prescription_info>";
        if (StringUtils.isNoneBlank(recipeId)){
            optPatient +=" <recipe_id>"+recipeId+"</recipe_id> ";
        }
        if (StringUtils.isNoneBlank(recipeNo)){
            optPatient +=" <recipe_no>"+recipeNo+"</recipe_no> ";
        }
        if (StringUtils.isNoneBlank(recipeSource)){
            optPatient +=" <recipe_source>"+recipeSource+"</recipe_source> ";
        }
        if (StringUtils.isNoneBlank(recipeType)){
            optPatient +=" <recipe_type>"+recipeType+"</recipe_type> ";
        }
        if (StringUtils.isNoneBlank(recipeDocId)){
            optPatient +=" <recipe_doc_id>"+recipeDocId+"</recipe_doc_id> ";
        }
        if (StringUtils.isNoneBlank(recipeDocName)){
            optPatient +=" <recipe_doc_name>"+recipeDocName+"</recipe_doc_name> ";
        }
        if (StringUtils.isNoneBlank(recipeDocTitle)){
            optPatient +=" <recipe_doc_title>"+recipeDocTitle+"</recipe_doc_title> ";
        }
        if (StringUtils.isNoneBlank(deptId)){
            optPatient +=" <dept_id>"+deptId+"</dept_id> ";
        }
        if (StringUtils.isNoneBlank(deptName)){
            optPatient +=" <dept_name>"+deptName+"</dept_name> ";
        }
        if (StringUtils.isNoneBlank(recipeTime)){
            optPatient +=" <recipe_time>"+recipeTime+"</recipe_time> ";
        }
        if (StringUtils.isNoneBlank(herbUnitPrice)){
            optPatient +=" <herb_unit_price>"+herbUnitPrice+"</herb_unit_price> ";
        }
        if (StringUtils.isNoneBlank(herbPacketCount)){
            optPatient +=" <herb_packet_count>"+herbPacketCount+"</herb_packet_count> ";
        }
        if (StringUtils.isNoneBlank(recipeFeeTotal)){
            optPatient +=" <recipe_fee_total>"+recipeFeeTotal+"</recipe_fee_total> ";
        }

        if (StringUtils.isNoneBlank(recipeStatus)){
            optPatient +=" <recipe_status>"+recipeStatus+"</recipe_status> ";
        }
        if (StringUtils.isNoneBlank(originalRecipeId)){
            optPatient +=" <original_recipe_id>"+originalRecipeId+"</original_recipe_id> ";
        }
        if (StringUtils.isNoneBlank(urgentFlag)){
            optPatient +=" <urgent_flag>"+urgentFlag+"</urgent_flag> ";
        }

        optPatient+="</opt_prescription_info>";
        return optPatient;
    }


    /**
     * 3.3.2	处方明细
     *
     * @param recipeId 系统运行核心关联字段，一张处方实体的唯一标记，院内唯一，主键
     * @param recipeItemId 系统运行核心关联字段，唯一标记处方中的每一项药物医嘱
     * @param drugId 系统运行核心关联字段，医院药品id，传入约定的药品编码
     * @param drugName 药品的通用名称
     * @param specification 药品规格的描述，如0.5gx24
     * @param preparation 药品剂型名称
     * @param manufacturerName 药品产地名称
     * @param countUnit 如果一盒药品12片，发药单位为片时，包装规格数量为1
     * @param drugDose 医生开立的单次给药剂量，值必须是数字+单位，如：0.5g 、200ml等
     * @param despensingNum 当处方状态为-1(部分药品退费退药)时，这里填写负值，表示具体退多少。
    正常处方状态情况，请填写最终生效值，表示最终的发药数量。
     * @param packUnit  发药单位，和发药数量对应，例如：瓶、盒、片等
     * @param unitPrice 药品的费用单价
     * @param feeTotal 药品的单价*发药数量，计量单位为元　
     * @param groupNo 药物的分组号，同组药物组号一致。
     * @param drugAdminRouteName 传入约定的药品使用途径，如：静脉推注等
     * @param drugUsingFreq 传入约定的药品给药频次，如：qd、bid等，
     * @param drugUsingTimePoint 药品给药时机，如：饭前、饭后、空腹等
     * @param drugUsingAim 药品的用药目的，预防|治疗|其他
     * @param drugUsingArea 药品使用部位
     * @param infusionSpeed 静脉输液药品滴注时长或速度的描述，需同时包含数字和单位，如：1小时、20滴/分钟
     * @param duration 由医生指定的药品使用天数或次数，如：5天，10次等
     * @param specialPrompt 备注信息，如饮片的先煎、后下等
     * @param skinTestFlag 1: 皮试0: 非皮试
     * @param skinTestResult 皮试结果
     * @param ouvasFlag 门诊静配标志
     * @param medicareType 结算方式
     * @param startTime 开始使用时间
     * @param endTime 结束使用时间
     * @return
     */
    public String initPrescriptionItemParam(String recipeId,String recipeItemId,String drugId,String drugName,String specification,String preparation,String manufacturerName,
                                            String countUnit,String drugDose,String despensingNum,String packUnit,String unitPrice,String feeTotal,
                                            String groupNo,String drugAdminRouteName,String drugUsingFreq,String drugUsingTimePoint,String drugUsingAim,String drugUsingArea,
                                            String infusionSpeed,String duration,String specialPrompt,String skinTestFlag,String skinTestResult,String ouvasFlag,
                                            String medicareType,String startTime,String endTime){
        String optPatient = "<opt_prescription_item>";
        if (StringUtils.isNoneBlank(recipeId)){
            optPatient +=" <recipe_id>"+recipeId+"</recipe_id> ";
        }
        if (StringUtils.isNoneBlank(recipeItemId)){
            optPatient +=" <recipe_item_id>"+recipeItemId+"</recipe_item_id> ";
        }
        if (StringUtils.isNoneBlank(drugId)){
            optPatient +=" <drug_id>"+drugId+"</drug_id> ";
        }
        if (StringUtils.isNoneBlank(drugName)){
            optPatient +=" <drug_name>"+drugName+"</drug_name> ";
        }
        if (StringUtils.isNoneBlank(specification)){
            optPatient +=" <specification>"+specification+"</specification> ";
        }
        if (StringUtils.isNoneBlank(preparation)){
            optPatient +=" <preparation>"+preparation+"</preparation> ";
        }
        if (StringUtils.isNoneBlank(manufacturerName)){
            optPatient +=" <manufacturer_name>"+manufacturerName+"</manufacturer_name> ";
        }
        if (StringUtils.isNoneBlank(countUnit)){
            optPatient +=" <count_unit>"+countUnit+"</count_unit> ";
        }
        if (StringUtils.isNoneBlank(drugDose)){
            optPatient +=" <drug_dose>"+drugDose+"</drug_dose> ";
        }
        if (StringUtils.isNoneBlank(despensingNum)){
            optPatient +=" <despensing_num>"+despensingNum+"</despensing_num> ";
        }
        if (StringUtils.isNoneBlank(packUnit)){
            optPatient +=" <pack_unit>"+packUnit+"</pack_unit> ";
        }
        if (StringUtils.isNoneBlank(unitPrice)){
            optPatient +=" <unit_price>"+unitPrice+"</unit_price> ";
        }
        if (StringUtils.isNoneBlank(feeTotal)){
            optPatient +=" <fee_total>"+feeTotal+"</fee_total> ";
        }

        if (StringUtils.isNoneBlank(groupNo)){
            optPatient +=" <group_no>"+groupNo+"</group_no> ";
        }
        if (StringUtils.isNoneBlank(drugAdminRouteName)){
            optPatient +=" <drug_admin_route_name>"+drugAdminRouteName+"</drug_admin_route_name> ";
        }
        if (StringUtils.isNoneBlank(drugUsingFreq)){
            optPatient +=" <drug_using_freq>"+drugUsingFreq+"</drug_using_freq> ";
        }

        if (StringUtils.isNoneBlank(drugUsingTimePoint)){
            optPatient +=" <drug_using_time_point>"+drugUsingTimePoint+"</drug_using_time_point> ";
        }

        if (StringUtils.isNoneBlank(drugUsingAim)){
            optPatient +=" <drug_using_aim>"+drugUsingAim+"</drug_using_aim> ";
        }

        if (StringUtils.isNoneBlank(drugUsingArea)){
            optPatient +=" <drug_using_area>"+drugUsingArea+"</drug_using_area> ";
        }

        if (StringUtils.isNoneBlank(infusionSpeed)){
            optPatient +=" <infusion_speed>"+infusionSpeed+"</infusion_speed> ";
        }
        if (StringUtils.isNoneBlank(duration)){
            optPatient +=" <duration>"+duration+"</duration> ";
        }

        if (StringUtils.isNoneBlank(specialPrompt)){
            optPatient +=" <special_prompt>"+specialPrompt+"</special_prompt> ";
        }

        if (StringUtils.isNoneBlank(skinTestFlag)){
            optPatient +=" <skin_test_flag>"+skinTestFlag+"</skin_test_flag> ";
        }

        if (StringUtils.isNoneBlank(skinTestResult)){
            optPatient +=" <skin_test_result>"+skinTestResult+"</skin_test_result> ";
        }

        if (StringUtils.isNoneBlank(ouvasFlag)){
            optPatient +=" <ouvas_flag>"+ouvasFlag+"</ouvas_flag> ";
        }

        if (StringUtils.isNoneBlank(ouvasFlag)){
            optPatient +=" <ouvas_flag>"+ouvasFlag+"</ouvas_flag> ";
        }

        if (StringUtils.isNoneBlank(medicareType)){
            optPatient +=" <medicare_type>"+medicareType+"</medicare_type> ";
        }

        if (StringUtils.isNoneBlank(startTime)){
            optPatient +=" <start_time>"+startTime+"</start_time> ";
        }

        if (StringUtils.isNoneBlank(endTime)){
            optPatient +=" <end_time>"+endTime+"</end_time> ";
        }

        optPatient+="</opt_prescription_item>";
        return optPatient;
    }
}
