package com.yihu.jw.inpatient.service;

import com.yihu.jw.contant.DictContant;
import com.yihu.jw.entity.base.inpatient.BaseInpatientRegistrationDO;
import com.yihu.jw.hospital.dict.service.WlyyHospitalSysDictService;
import com.yihu.jw.inpatient.dao.BaseInpatientRegistrationDao;
import com.yihu.jw.mysql.query.BaseJpaService;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.util.date.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * Created by yeshijie on 2022/12/5.
 */
@Service
public class BaseInpatientRegistrationService extends BaseJpaService {

    @Autowired
    private BaseInpatientRegistrationDao inpatientRegistrationDao;
    @Autowired
    private WlyyHospitalSysDictService hospitalSysDictService;

    public int countByStatus(String status){
        return inpatientRegistrationDao.countByStatus(status);
    }

    /**
     * 查找住院中的患者
     * @param idcard
     * @param name
     * @return
     */
    public BaseInpatientRegistrationDO findRecently(String idcard,String name){
        List<BaseInpatientRegistrationDO> list = inpatientRegistrationDao.findByIdcardAndName(idcard,name);
        if(list.size()>0){
            return list.get(0);
        }
        return null;
    }

    /**
     * 获取住院登记list
     * @param patient
     * @return
     */
    public List<BaseInpatientRegistrationDO> list(String patient){
        String sql = "SELECT * from base_inpatient_registration" +
                " where patient='"+patient+"' and status=0 order by create_time desc ";
        List<BaseInpatientRegistrationDO> list = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(BaseInpatientRegistrationDO.class));
        for (BaseInpatientRegistrationDO registrationDO:list){
            registrationDO.setStatusName(hospitalSysDictService.getDictValue(DictContant.INPATIENTREGIST_STATUS,registrationDO.getStatus()));
        }
        return list;
    }

    /**
     * 入院登记
     * @return
     */
    public BaseInpatientRegistrationDO registration(String id,String inpatientTime,String bedNo,String attendingDoctor
            ,String attendingDoctorName,String inpatientAreaId,String inpatientAreaName){
        BaseInpatientRegistrationDO registrationDO = inpatientRegistrationDao.findById(id).orElse(null);
        if(registrationDO!=null&&("1".equals(registrationDO.getStatus())||"2".equals(registrationDO.getStatus()))){
            registrationDO.setInpatientTime(DateUtil.strToDate(inpatientTime));
            registrationDO.setBedNo(bedNo);
            registrationDO.setStatus("2");
            registrationDO.setAttendingDoctor(attendingDoctor);
            registrationDO.setAttendingDoctorName(attendingDoctorName);
            registrationDO.setInpatientAreaId(inpatientAreaId);
            registrationDO.setInpatientAreaName(inpatientAreaName);
            inpatientRegistrationDao.save(registrationDO);
        }
        return registrationDO;
    }

    /**
     * 提交住院登记
     * @param id
     * @return
     */
    public BaseInpatientRegistrationDO submitRegist(String id,String mobile,String address,String contacts,String contactsMobile){
        BaseInpatientRegistrationDO registrationDO = inpatientRegistrationDao.findById(id).orElse(null);
        if(registrationDO!=null&&"0".equals(registrationDO.getStatus())){
            registrationDO.setUpdateTime(new Date());
            registrationDO.setStatus("1");
            registrationDO.setMobile(mobile);
            registrationDO.setAddress(address);
            registrationDO.setContacts(contacts);
            registrationDO.setContactsMobile(contactsMobile);
            registrationDO = inpatientRegistrationDao.save(registrationDO);
        }
        return registrationDO;
    }

    public BaseInpatientRegistrationDO confirmAdmission(String id){
        BaseInpatientRegistrationDO registrationDO = inpatientRegistrationDao.findById(id).orElse(null);
        if(registrationDO!=null&&"2".equals(registrationDO.getStatus())){
            registrationDO.setStatus("3");
            registrationDO = inpatientRegistrationDao.save(registrationDO);
        }

        return registrationDO;
    }

    public BaseInpatientRegistrationDO findDetail(String id){
        BaseInpatientRegistrationDO registrationDO = inpatientRegistrationDao.findById(id).orElse(null);
        if(registrationDO!=null){
            registrationDO.setInpatientTypeName(hospitalSysDictService.getDictValue(DictContant.INPATIENT_NATURE,registrationDO.getInpatientType()));
            registrationDO.setStatusName(hospitalSysDictService.getDictValue(DictContant.INPATIENTREGIST_STATUS,registrationDO.getStatus()));
        }
        return registrationDO;
    }

    public PageEnvelop page(String patient,String name,String startTime,String endTime,Integer page,Integer size){
        String sql = "select * from base_inpatient_registration where 1=1";
        if(StringUtils.isNotBlank(patient)){
            sql += " and patient = '"+patient+"'";
        }else {
            sql += " and status>0";
        }
        if(StringUtils.isNotBlank(name)){
            sql += " and (name like '%"+name+"%' or idcard like '%"+name+"%')";
        }
        if(StringUtils.isNotBlank(startTime)){
            sql += " and create_time>='"+startTime+"'";
        }
        if(StringUtils.isNotBlank(endTime)){
            sql += " and create_time<='"+endTime+"'";
        }
        String countSql = sql.replace("select *","select count(*)");
        String limit = " order by create_time desc limit "+(page-1)*size+","+size;

        List<BaseInpatientRegistrationDO> list = jdbcTemplate.query(sql+limit,new BeanPropertyRowMapper<>(BaseInpatientRegistrationDO.class));
        for (BaseInpatientRegistrationDO registrationDO:list){
            registrationDO.setStatusName(hospitalSysDictService.getDictValue(DictContant.INPATIENTREGIST_STATUS,registrationDO.getStatus()));
        }
        Long count = jdbcTemplate.queryForObject(countSql,Long.class);

        return PageEnvelop.getSuccessListWithPage("获取成功",list,page,size,count);
    };


}
