package com.yihu.jw.knowledge.service;


import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.dict.dao.DictHospitalDeptDao;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.dict.DictHospitalDeptDO;
import com.yihu.jw.entity.hospital.appointment.WlyySpecialDiseaseDO;
import com.yihu.jw.entity.knowledge.BaseKnowledgeDict;
import com.yihu.jw.entity.knowledge.BaseKnowledgeDictRelation;
import com.yihu.jw.hospital.appointment.dao.SpecialDiseaseDao;
import com.yihu.jw.knowledge.dao.BaseKnowledgeDictDao;
import com.yihu.jw.knowledge.dao.BaseKnowledgeDictRelationDao;
import com.yihu.jw.mysql.query.BaseJpaService;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.utils.hibernate.HibenateUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;


/**
 * 字典中心
 *
 * @wangzhinan
 */
@Service
public class BaseKnowledgeDictService extends BaseJpaService<BaseKnowledgeDict, BaseKnowledgeDictDao> {

    private org.slf4j.Logger logger = LoggerFactory.getLogger(BaseKnowledgeDictService.class);


    @Autowired
    private BaseKnowledgeDictDao dictDao;
    @Autowired
    private BaseKnowledgeDictRelationDao dictRelationDao;
    @Autowired
    private HibenateUtils hibenateUtils;
    @Autowired
    private DictHospitalDeptDao hospitalDeptDao;
    @Autowired
    private BaseDoctorDao doctorDao;
    @Autowired
    private SpecialDiseaseDao specialDiseaseDao;

    /**
     * 新增及修改字典
     * @param knowledgeDict
     * @return
     */
    public BaseKnowledgeDict insertAndUpdateDict(BaseKnowledgeDict knowledgeDict){
        if (StringUtils.isNoneBlank(knowledgeDict.getId())){
            BaseKnowledgeDict dict = dictDao.findById(knowledgeDict.getId()).get();
            knowledgeDict.setDel(dict.getDel());
            knowledgeDict.setSort(dict.getSort());
            knowledgeDict.setStatus(dict.getStatus());
            knowledgeDict.setUpdateTime(new Date());
        }else {
            knowledgeDict.setDel(1);
            knowledgeDict.setSort(maxSort("base_knowledge_dict")+1);
            knowledgeDict.setStatus(0);
            knowledgeDict.setCreateTime(new Date());
            knowledgeDict.setUpdateTime(new Date());
        }
        return dictDao.save(knowledgeDict);
    }

    /**
     * 获取排序的序号
     *
     * @param tableName
     * @return
     */
    public  Integer maxSort(String tableName){
        String sql = "SELECT MAX(sort) as sort from "+tableName+" where del=1  ";
        List<Map<String, Object>> rstotal = hibenateUtils.createSQLQuery(sql);
        Integer count = 0;
        if (rstotal != null && rstotal.size() > 0) {
            if (rstotal.get(0).get("sort")!=null){
                count = Integer.parseInt(rstotal.get(0).get("sort").toString());
            }

        }
        return count;
    }

    /**
     * 删除字典
     * @param id
     * @return
     */
    public void deleteDict(String id){
        String str[] = id.split(",");
        for (int i=0;i<str.length;i++){
            dictDao.updateDelById(str[i], 0);
        }

    }


    /**
     * 查询字典列表
     * @param name
     * @param type
     * @param status
     */
    public MixEnvelop selectDictList(String name, String type, String status, Integer page, Integer size){
        String orderBy = " order by sort asc,create_time desc ";
        String condition = "";
        String sql = "SELECT\n" +
                "\tid,\n" +
                "\ttype,\n" +
                "\tname,\n" +
                "\tdescription,\n" +
                "\tcreate_user as createUser,\n" +
                "\tcreate_user_name as createUserName,\n" +
                "\tstatus,\n" +
                "\tsort\n" +
                "FROM\n" +
                "\tbase_knowledge_dict where 1=1 ";
        if (StringUtils.isNoneBlank(name)){
            condition +=" and NAME like '%"+name+"%' ";
        }
        if (StringUtils.isNoneBlank(type)){
            condition +=" and type ='"+type+"' ";
        }
        if (StringUtils.isNoneBlank(status)){
            condition +=" and status ='"+type+"' ";
        }
        condition += " and del = 1";
        List<Map<String, Object>> list = hibenateUtils.createSQLQuery(sql+condition+orderBy, page, size);
        String sqlCount ="select COUNT(1) as total from base_knowledge_dict where 1=1 ";
        List<Map<String, Object>> rstotal = hibenateUtils.createSQLQuery(sqlCount+condition);
        Long count = 0L;
        if (rstotal != null && rstotal.size() > 0) {
            count = Long.parseLong(rstotal.get(0).get("total").toString());
        }
        return MixEnvelop.getSuccessListWithPage("success", list, page, size, count);
    }

    /**
     * 根据id查询某一个字典详情(关联数据)
     * @param id
     * @return
     */
    public BaseKnowledgeDict selectById(String id){
        BaseKnowledgeDict dict = dictDao.findById(id).get();
        if (dict.getStatus()==1){
            List<BaseKnowledgeDictRelation> dictRelations = dictRelationDao.selectListByDictId(dict.getId());
            for (BaseKnowledgeDictRelation dictRelation:dictRelations){
                String dept = dictRelation.getDept();
                String doctor = dictRelation.getDoctor();
                String deptstr[]= dept.split(",");
                String doctorstr[]= doctor.split(",");
                List<DictHospitalDeptDO> hospitalDeptDOS = new ArrayList<>();
                List<WlyySpecialDiseaseDO> doctorDOS = new ArrayList<>();
                for (int i=0;i<deptstr.length-1;i++){
                    DictHospitalDeptDO hospitalDeptDO = hospitalDeptDao.findByCode(deptstr[i]);
                    hospitalDeptDOS.add(hospitalDeptDO);
                }
                for (int i=0;i<doctorstr.length-1;i++){
                    WlyySpecialDiseaseDO doctorDO = specialDiseaseDao.findByCode(doctorstr[i]);
                    doctorDOS.add(doctorDO);
                }
                dictRelation.setDeptDOS(hospitalDeptDOS);
                dictRelation.setDoctorDOS(doctorDOS);

            }
            dict.setDictRelations(dictRelations);
        }
        return dict;
    }

    //根据字段id获取关联科室
    public JSONArray findDeptById(String id){
        JSONArray jsonArray = new JSONArray();
        List<BaseKnowledgeDictRelation> dictRelations = dictRelationDao.selectListByDictId(id);
        for (BaseKnowledgeDictRelation dictRelation:dictRelations){
            String dept = dictRelation.getDept();
            String deptstr[]= dept.split(",");
            for (int i=0;i<deptstr.length;i++){
                DictHospitalDeptDO hospitalDeptDO = hospitalDeptDao.findByCode(deptstr[i]);
                if(hospitalDeptDO!=null){
                    JSONObject json = new JSONObject();
                    json.put("code",hospitalDeptDO.getCode());
                    json.put("name",hospitalDeptDO.getName());
                    json.put("img",hospitalDeptDO.getDeptImg());
                    jsonArray.add(json);
                }
            }
        }
        return jsonArray;
    }


    /**
     * 字典设置排序
     * @param id
     * @param flag 1上移2下移3置顶
     */
    public BaseKnowledgeDict setDictSortById(String id,Integer flag){
        BaseKnowledgeDict knowledgeDict = dictDao.findById(id).get();
        int sort = 0;
        if (flag==1){
            String sql = "select sort from base_knowledge_dict where sort<'"+knowledgeDict.getSort()+"' and del=1 order by sort desc limit 1";
            List<Map<String,Object>> mapList = hibenateUtils.createSQLQuery(sql);
            Integer count = 0;
            if (mapList != null && mapList.size() > 0) {
                count = Integer.parseInt(mapList.get(0).get("sort").toString());
            }
            if (count==0){
                sort = 0;
            }else {
                sort=count;
            }
            BaseKnowledgeDict knowledgeDict1= dictDao.selectBySort(sort);
            if (knowledgeDict1!=null){
                knowledgeDict1.setSort(knowledgeDict.getSort());
                dictDao.save(knowledgeDict1);
            }
        }else if (flag==2){
            String sql = "select sort from base_knowledge_dict where sort>'"+knowledgeDict.getSort()+"' and del=1 order by sort asc limit 1";
            List<Map<String,Object>> mapList = hibenateUtils.createSQLQuery(sql);
            Integer count = 0;
            if (mapList != null && mapList.size() > 0) {
                count = Integer.parseInt(mapList.get(0).get("sort").toString());
            }
            if (count==0){
                sort = 0;
            }else {
                sort=count;
            }
            BaseKnowledgeDict knowledgeDict1= dictDao.selectBySort(sort);
            if (knowledgeDict1!=null){
                knowledgeDict1.setSort(knowledgeDict.getSort());
                dictDao.save(knowledgeDict1);
            }

        }else if (flag==3){
            sort = 1;
            BaseKnowledgeDict knowledgeDict1= dictDao.selectBySort(sort);
            if (knowledgeDict1!=null){
                knowledgeDict1.setSort(knowledgeDict.getSort());
                dictDao.save(knowledgeDict1);
            }

        }
        if (sort!=0){
            knowledgeDict.setSort(sort);
        }
        return dictDao.save(knowledgeDict);
    }


    public void setDictRelation(BaseKnowledgeDictRelation baseKnowledgeDictRelation,Integer status){
        if (status==0){
            dictRelationDao.deleteByDictId(baseKnowledgeDictRelation.getDictId());
        }else {
            baseKnowledgeDictRelation.setUpdateTime(new Date());
            baseKnowledgeDictRelation.setCreateTime(new Date());
            dictRelationDao.save(baseKnowledgeDictRelation);
        }
        BaseKnowledgeDict dict = dictDao.findById(baseKnowledgeDictRelation.getDictId()).get();
        dict.setStatus(status);
        dictDao.save(dict);
    }

}
