package com.yihu.jw.knowledge.service;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.entity.hospital.consult.WlyyHospitalSysDictDO;
import com.yihu.jw.entity.knowledge.BaseKnowledgeQuestion;
import com.yihu.jw.entity.knowledge.BaseKnowledgeQuestions;
import com.yihu.jw.entity.knowledge.BaseSystemDialogSetting;
import com.yihu.jw.entity.vo.DictVO;
import com.yihu.jw.hospital.dict.WlyyHospitalSysDictDao;
import com.yihu.jw.knowledge.dao.BaseKnowledgeDictDao;
import com.yihu.jw.knowledge.dao.BaseKnowledgeQuestionDao;
import com.yihu.jw.knowledge.dao.BaseKnowledgeQuestionsDao;
import com.yihu.jw.knowledge.dao.BaseSystemDialogSettingDao;
import com.yihu.jw.mysql.query.BaseJpaService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;


/**
 * 系统会话设置
 *
 * @wangzhinan
 */
@Service
public class BaseSystemDialogSettingService extends BaseJpaService<BaseSystemDialogSetting, BaseSystemDialogSettingDao> {

    private org.slf4j.Logger logger = LoggerFactory.getLogger(BaseSystemDialogSettingService.class);

    @Autowired
    private BaseSystemDialogSettingDao dialogSettingDao;
    @Autowired
    private BaseKnowledgeQuestionDao questionDao;
    @Autowired
    private BaseKnowledgeQuestionsDao questionsDao;
    @Autowired
    private BaseKnowledgeDictDao dictDao;
    @Autowired
    private WlyyHospitalSysDictDao hospitalSysDictDao;

    /**
     * 新增和修改系统会话设置
     *
     * @param dialogSettings
     * @return
     */
    public List<BaseSystemDialogSetting> insterAndUpdateSystemDialogSetting(List<BaseSystemDialogSetting> dialogSettings){
        for (BaseSystemDialogSetting dialogSetting:dialogSettings){
            if (StringUtils.isNoneBlank(dialogSetting.getId())){
                dialogSetting.setUpdateTime(new Date());
            }else {
                dialogSetting.setCreateTime(new Date());
            }
            dialogSettingDao.save(dialogSetting);
        }
        return dialogSettings;
    }

    /**
     * 根据不同业务code查询相关的系统会话设置配置信息;
     *
     * @param systemType
     * @return
     */
    public List<BaseSystemDialogSetting> selectBySystemType(String systemType){
        List<BaseSystemDialogSetting> systemDialogSettings = dialogSettingDao.selectBySystemType(systemType);
        for (BaseSystemDialogSetting dialogSetting:systemDialogSettings){
            if (dialogSetting.getRelaitonCodeType()!=null){
                if (dialogSetting.getRelaitonCodeType()==1){
                    String question[] = dialogSetting.getRelationCode().split(",");
                    List<BaseKnowledgeQuestion> questionList = new ArrayList<>();
                    for (int i=0;i<=question.length-1;i++){
                        String questionId = question[i];
                        BaseKnowledgeQuestion knowledgeQuestion = questionDao.findById(questionId).get();
                        questionList.add(knowledgeQuestion);
                    }
                    dialogSetting.setQuestions(questionList);
                }else  if (dialogSetting.getRelaitonCodeType()==2){
                    String dict[] = dialogSetting.getRelationCode().split(",");
                    WlyyHospitalSysDictDO dictDO = hospitalSysDictDao.findOneByDictNameAndDictCode("dictTypeDict","dictTypeDict");
                    JSONObject json = JSON.parseObject(dictDO.getDictValue());
                    List<DictVO> dictList = new ArrayList<>();
                    for (int i=0;i<=dict.length-1;i++){
                        String dictId = dict[i];
                        DictVO dictVO = new DictVO();
                        dictVO.setCode(dictId);
                        dictVO.setName(json.getString(dictId));
                        dictList.add(dictVO);
                    }
                    dialogSetting.setDictList(dictList);
                }else if (dialogSetting.getRelaitonCodeType()==3){
                    String questions[] = dialogSetting.getRelationCode().split(",");
                    List<BaseKnowledgeQuestions> questionsList = new ArrayList<>();
                    for (int i=0;i<=questions.length-1;i++){
                        String questionId = questions[i];
                        BaseKnowledgeQuestions knowledgeQuestions = questionsDao.findById(questionId).get();
                        questionsList.add(knowledgeQuestions);
                    }
                    dialogSetting.setQuestionsList(questionsList);
                }
            }
        }
        return systemDialogSettings;
    }

}
