package com.yihu.jw.message.service;

import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.doctor.BaseDoctorHospitalDO;
import com.yihu.jw.entity.base.org.BaseOrgDO;
import com.yihu.jw.entity.base.wx.*;
import com.yihu.jw.entity.care.course.RecruitStudentsRecordDO;
import com.yihu.jw.entity.healthUpload.BaseDoctorHealthUploadDO;
import com.yihu.jw.org.dao.BaseOrgDao;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.wechat.WeixinMessagePushUtils;
import com.yihu.jw.wechat.dao.BasePatientWechatDao;
import com.yihu.jw.wechat.dao.WechatDao;
import com.yihu.jw.wechat.dao.WxPushLogDao;
import com.yihu.jw.wechat.dao.WxTemplateConfigDao;
import com.yihu.jw.wechat.service.WxAccessTokenService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * Created by yeshijie on 2022/11/18.
 */
@Service
public class WxMessageUtil {

    private static final Logger logger = LoggerFactory.getLogger(WxMessageUtil.class);

    @Resource
    private WxTemplateConfigDao wxTemplateConfigDao;
    @Resource
    private WxAccessTokenService wxAccessTokenService;
    @Resource
    private WxPushLogDao wxPushLogDao;
    @Resource
    private WeixinMessagePushUtils weixinMessagePushUtils;
    @Value("${wechat.id}")
    private String wechatId;
    @Resource
    private BasePatientWechatDao basePatientWechatDao;
    @Resource
    private BaseOrgDao orgDao;
    @Resource
    private WechatDao wechatDao;

    public WxWechatDO getWxWechat(){
        return wechatDao.findById(wechatId).orElse(null);
    }


    public String getWechatId(){
        return wechatId;
    }

    public void putWxMsg(String openId,WxTemplateConfigDO newConfig,Integer type,JSONObject json){
        putWxMsg(getWechatId(),openId,newConfig,type,json);
    }

    public void putWxMsg(String wechatId,String openId,WxTemplateConfigDO newConfig,Integer type,JSONObject json){
        try {
            if(StringUtils.isBlank(wechatId)){
                wechatId = getWechatId();
            }
            System.out.println(wechatId);
            WxAccessTokenDO wxAccessTokenDO = wxAccessTokenService.getWxAccessTokenById(wechatId);

            if (wxAccessTokenDO == null) {
                logger.info("wx_access_token表获取为空，wechatId" + wechatId);
                return;
            }
            newConfig = setTemPlateUrl(newConfig,type,openId,json);
            //发起微信消息模板推送
            weixinMessagePushUtils.putWxMsg(wxAccessTokenDO.getAccessToken(), openId, newConfig);
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    /**
     * 批量异步发送微信模板消息
     */
    @Async
    public void sengdWxTemplates(List<RecruitStudentsRecordDO> list){
        String orgMobile = "";
        String orgId = "";
        for (RecruitStudentsRecordDO recordDO:list){
            if("".equals(orgId)){
                BaseOrgDO orgDO = orgDao.findByCode(recordDO.getOrgCode());
                orgMobile = orgDO.getMobile();
                orgId = orgDO.getId();
            }
            /**
             * 学生被录取的时候推送一个录取成功通知微信模板消息
             dc--ecFzT5vrnUFTNdVB6Sg4wXBnv14IX5sFP3jCFfk
             您好，恭喜您已被【机构名】成功录取
             姓名：【报名人姓名】
             手机号：【报名人手机号】
             请及时前往托育机构完成报名，如有疑问请联系招生热线【机构联系方式】
             （点击进入对应报名的机构详情页）
             */
            String first = "您好，恭喜您已被"+recordDO.getOrgName()+"成功录取";
            String remark = "请及时前往托育机构完成报名，如有疑问请联系招生热线"+orgMobile;
            JSONObject json = new JSONObject();
            json.put("orgId",orgId);
            List<BasePatientWechatDo> basePatientWechatDos = basePatientWechatDao.findByWechatIdAndPatientId(wechatId,recordDO.getPatient());
            if(basePatientWechatDos.size()>0){
                String openId = basePatientWechatDos.get(0).getOpenid();
                putTemplateWxMessage(wechatId,"template_success_notice","cgrxtz",openId,first,null,remark,2,json,
                        recordDO.getPatientName(),recordDO.getMobile());
            }
        }
    }

    public WxTemplateConfigDO findByWechatIdAndTemplateNameAndSceneAndStatus(String wechatId, String templateName, String scene, Integer status){
        if(StringUtils.isBlank(wechatId)){
            wechatId = getWechatId();
        }
        return wxTemplateConfigDao.findByWechatIdAndTemplateNameAndSceneAndStatus(wechatId, templateName, scene, status);
    }


    /**
     *
     * @param wechatId
     * @param templateName
     * @param scene
     * @param openId
     * @param first
     * @param url 跳转链接
     * @param remark
     * @param type  模板通知类型
     * @param json 用于对跳转连接的修改，
     * @param keywords
     */
    public void putTemplateWxMessage(String wechatId, String templateName, String scene, String openId, String first
            ,String url, String remark, Integer type,JSONObject json,String ...keywords) {
        try {
            if(StringUtils.isBlank(wechatId)){
                wechatId = getWechatId();
            }
            System.out.println(wechatId);
            WxAccessTokenDO wxAccessTokenDO = wxAccessTokenService.getWxAccessTokenById(wechatId);

            if (wxAccessTokenDO == null) {
                logger.info("wx_access_token表获取为空，wechatId" + wechatId);
                return;
            }
            WxTemplateConfigDO templateConfig = wxTemplateConfigDao.findByWechatIdAndTemplateNameAndSceneAndStatus(wechatId, templateName, scene, 1);
            WxTemplateConfigDO newConfig = new WxTemplateConfigDO();
            BeanUtils.copyProperties(templateConfig,newConfig);
            if (newConfig == null) {
                logger.info("微信模板不存在！请确认wechatId:" + wechatId + ",templateName:" + templateName + ",scene:" + scene);
                return;
            }
            logger.info("微信模板推送前");
            if (StringUtils.isNoneBlank(first)){
                newConfig.setFirst(first);
            }
            if (StringUtils.isNoneBlank(url)){
                newConfig.setUrl(url);
            }
            if (StringUtils.isNoneBlank(remark)){
                newConfig.setRemark(remark);
            }
            newConfig = setTemPlateUrl(newConfig,type,openId,json);
            int keyLength = keywords.length;
            if (keyLength >= 1) {
                if(StringUtils.isNoneBlank(keywords[0])){
                    newConfig.setKeyword1(keywords[0]);
                }
            }
            if (keyLength >= 2) {
                if(StringUtils.isNoneBlank(keywords[1])){
                    newConfig.setKeyword2(keywords[1]);
                }
            }
            if (keyLength >= 3) {
                if(StringUtils.isNoneBlank(keywords[2])){
                    newConfig.setKeyword3(keywords[2]);
                }
            }
            if (keyLength >= 4) {
                if(StringUtils.isNoneBlank(keywords[3])){
                    newConfig.setKeyword4(keywords[3]);
                }
            }
            if (keyLength >= 5) {
                if(StringUtils.isNoneBlank(keywords[4])){
                    newConfig.setKeyword5(keywords[4]);
                }
            }
            if (keyLength >= 6) {
                if(StringUtils.isNoneBlank(keywords[5])){
                    newConfig.setKeyword6(keywords[5]);
                }
            }
            if (keyLength >= 7) {
                if(StringUtils.isNoneBlank(keywords[6])){
                    newConfig.setKeyword7(keywords[6]);
                }
            }
            //发起微信消息模板推送
            weixinMessagePushUtils.putWxMsg(wxAccessTokenDO.getAccessToken(), openId, newConfig);
            logger.info("微信模板消息推送后");
        } catch (Exception e) {
            logger.info("微信模板推送异常");
            e.printStackTrace();
        }
    }

    public WxTemplateConfigDO setTemPlateUrl(WxTemplateConfigDO wxTemplateConfigDO,Integer type,String openid,JSONObject json){
        String url =  "/";
        if (json==null){
            return wxTemplateConfigDO;
        }
        switch (type){
            case 1:
                //咨询回复
                wxTemplateConfigDO.setUrl(wxTemplateConfigDO.getUrl() + "?sessionId=" + json.getString("sessionId")
                        +"&type="+json.getString("consultType"));
                break;
            case 2:
                //入学成功通知
                wxTemplateConfigDO.setUrl(wxTemplateConfigDO.getUrl() + "?id=" + json.getString("orgId")
                        +"&openid="+openid);
                break;
            case 19:
                String urlStr= wxTemplateConfigDO.getUrl();
                boolean status = urlStr.contains("openid=");
                if(!status){
                    urlStr=json.getString("url")+"?openid=" + openid;
                }
                wxTemplateConfigDO.setUrl(url + urlStr );
                break;
            case 30:
                //反馈通知 测试 TPbq9m0SAiVfRhXtXq17SDmYIfrJ8Whp2NpSrq9wlfI
                if(json.containsKey("consult")) {
                    wxTemplateConfigDO.setUrl(url + wxTemplateConfigDO.getUrl() + "?openid=" + openid + "&consult=" + json.getString("consult"));
                }else if (json.containsKey("id")){
                    wxTemplateConfigDO.setUrl(url + wxTemplateConfigDO.getUrl() + "?openid=" + openid + "&id=" + json.getString("id"));
                }else if(json.containsKey("relationCode")){
                    wxTemplateConfigDO.setUrl(url + wxTemplateConfigDO.getUrl() + "?openid=" + openid );
                }else if(json.containsKey("resultCode")){
                    wxTemplateConfigDO.setUrl(url + wxTemplateConfigDO.getUrl() + "?resultCode=" + json.getString("resultCode"));
                }
                break;
            case 31:
                //上门服务医生评分
                wxTemplateConfigDO.setUrl(url + wxTemplateConfigDO.getUrl()+"?openid=" + openid + "&id=" + json.getString("id")+ "&finish=" + json.getString("finish"));
                break;
            case 32:
                String url1 = "wx/common/";
                wxTemplateConfigDO.setUrl(url1 + wxTemplateConfigDO.getUrl()+"?openid=" + openid + "&consult=" + json.getString("consult") + "&status=" + json.getInteger("status"));
                break;
            case 33:
                wxTemplateConfigDO.setUrl(url + json.getString("url") + "?openid=" + openid + "&id=" + json.get("id")+ "&deviceId=" + json.getString("deviceId"));
                break;
            case 34:
                wxTemplateConfigDO.setUrl(url + wxTemplateConfigDO.getUrl() + "?openid=" + openid + "&orderId=" + json.getString("orderId")+ "&authorizeImage=" + json.getString("authorizeImage"));
                break;
            case 35:
                wxTemplateConfigDO.setUrl(url + wxTemplateConfigDO.getUrl()+json.getString("orderId")+"&isMask=1");
                break;
            default:
                //22
                break;
        }
        return wxTemplateConfigDO;
    }

    /**
     * 发送健康上报微信模板
     */
    public void sendHealthUploadWxTemplage(BaseDoctorDO doctorDO, List<BaseDoctorHospitalDO> doctorHospitalDOS, BaseDoctorHealthUploadDO baseDoctorHealthUploadDO){
        if (!StringUtils.isNoneBlank(doctorDO.getOpenid())) {
            logger.info("该用户" + doctorDO.getName() + "没有openid,无法推送模版消息，用户ID：" + doctorDO.getId() + "wechatId:"+wechatId);
        } else {
            WxTemplateConfigDO config = wxTemplateConfigDao.findByWechatIdAndTemplateNameAndSceneAndStatus(wechatId, "template_unusual_notice", "jkzlyctxtz", 1);
            WxTemplateConfigDO newConfig = new WxTemplateConfigDO();
            BeanUtils.copyProperties(config, newConfig);
            if (doctorHospitalDOS != null && doctorHospitalDOS.size() != 0) {
                newConfig.setFirst(config.getFirst().replace("key1", doctorHospitalDOS.get(0).getDeptName()).replace("key2", baseDoctorHealthUploadDO.getDoctorName()));
            } else {
                newConfig.setFirst(config.getFirst().replace("key1的", "").replace("key2", baseDoctorHealthUploadDO.getDoctorName()));
            }
            newConfig.setKeyword2(baseDoctorHealthUploadDO.getDoctorName());
            newConfig.setKeyword3(DateUtil.getStringDate());
            newConfig.setUrl(config.getUrl() + baseDoctorHealthUploadDO.getId() + "&docCode=" + baseDoctorHealthUploadDO.getDoctorId() + "");
            WxAccessTokenDO wxAccessTokenDO = wxAccessTokenService.getWxAccessTokenById(wechatId);
            logger.info("=======setUrl========" + newConfig.getUrl());
            weixinMessagePushUtils.putWxMsg(wxAccessTokenDO.getAccessToken(), doctorDO.getOpenid(), newConfig);
        }

        //保存发送模板记录,
        WxPushLogDO wxPushLogDO = new WxPushLogDO();
        wxPushLogDO.setCreateTime(new Date());
        wxPushLogDO.setOpenid(doctorDO.getOpenid());
        wxPushLogDO.setReceiver(doctorDO.getId());
        wxPushLogDO.setWechatId(wechatId);
        wxPushLogDO.setReceiverName(doctorDO.getName());
        wxPushLogDO.setScene("jkzlyctxtz");
        wxPushLogDao.save(wxPushLogDO);
    }
}
