package com.yihu.jw.nurse.service;

import com.yihu.jw.area.dao.BaseInpatientAreaDao;
import com.yihu.jw.dict.service.BaseCopywritingService;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.area.BaseInpatientAreaDO;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.inpatient.BaseInpatientRegistrationDO;
import com.yihu.jw.entity.base.nurse.BaseNursingCardDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.inpatient.dao.BaseInpatientRegistrationDao;
import com.yihu.jw.mysql.query.BaseJpaService;
import com.yihu.jw.nurse.dao.BaseNursingCardDao;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.util.common.IdCardUtil;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.entity.ServiceException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * Created by yeshijie on 2022/11/16.
 */
@Service
public class BaseNursingCardService extends BaseJpaService<BaseNursingCardDO, BaseNursingCardDao> {

    @Resource
    private BaseNursingCardDao nursingCardDao;
    @Resource
    private BaseDoctorDao doctorDao;
    @Resource
    private BasePatientDao patientDao;
    @Autowired
    private BaseInpatientRegistrationDao inpatientRegistrationDao;
    @Autowired
    private BaseInpatientAreaDao inpatientAreaDao;
    @Resource
    private BaseCopywritingService copywritingService;

    /**
     * 是否可申请
     * @return
     */
    public String ifCanApply(String idcard,String name){
        List<BaseInpatientRegistrationDO> list = inpatientRegistrationDao.findByIdcardAndName(idcard,name);
        if(list.size()>0){
            return list.get(0).getId();
        }
        return null;
    }
    public BaseNursingCardDO ifCanApply(BaseNursingCardDO nursingCardDO) throws Exception{
        String inpatientRegistrationId = nursingCardDO.getInpatientRegistrationId();
        if(StringUtils.isBlank(inpatientRegistrationId)){
            throw new ServiceException(copywritingService.findCopywriting("nursingCard","ifCanApply_error1"));
        }
        BaseInpatientRegistrationDO registrationDO = inpatientRegistrationDao.findById(inpatientRegistrationId).orElse(null);
        if(registrationDO==null){
            throw new ServiceException(copywritingService.findCopywriting("nursingCard","ifCanApply_error1"));
        }
        String patient = registrationDO.getPatient();
        String areaId = registrationDO.getInpatientAreaId();

        BaseInpatientAreaDO areaDO = inpatientAreaDao.findById(areaId).orElse(null);
        if(areaDO!=null){
            //判断每日允许最大陪护数
            if(areaDO.getDayMaxNum()!=null){
                int count = nursingCardDao.countByInpatientAreaIdAndStatus(areaId);
                if(count>=areaDO.getDayMaxNum()){
                    throw new ServiceException(copywritingService.findCopywriting("nursingCard","ifCanApply_error2"));
                }
            }

            //住院病区陪护规则判断，若未设置则不判断 单患者同时允许陪护数
            int count = countByPatientAndStatus(patient);
            if(count>0){
                BasePatientDO patientDO = patientDao.findById(patient).orElse(null);
                int age = IdCardUtil.getAgeByBirthdayOrIdcard(patientDO.getIdcard(),patientDO.getBirthday());
                if(age<75){
                    if(areaDO.getYangNum()!=null&&areaDO.getYangNum()<=count){
                        throw new ServiceException(copywritingService.findCopywriting("nursingCard","ifCanApply_error3"));
                    }
                }else{
                    if(areaDO.getOlderNum()!=null&&count>=areaDO.getOlderNum()){
                        throw new ServiceException(copywritingService.findCopywriting("nursingCard","ifCanApply_error4"));
                    }
                }
            }
        }

        nursingCardDO.setPatient(patient);
        nursingCardDO.setInpatientAreaId(areaId);
        nursingCardDO.setBedNo(registrationDO.getBedNo());
        nursingCardDO.setInpatientAreaName(registrationDO.getInpatientAreaName());
        return nursingCardDO;
    }

    /**
     * 陪护详情
     * @param id
     * @return
     */
    public BaseNursingCardDO nursingDetail(String id){
        BaseNursingCardDO cardDO = nursingCardDao.findById(id).orElse(null);
        cardDO.setStatusName(BaseNursingCardDO.Status.getStatusName(cardDO.getStatus()));

        return cardDO;
    }

    /**
     * 医生审核
     * @param id
     * @param status
     * @param doctor
     */
    public void doctorCheck(String id,Integer status,String doctor,String checkReason){
        BaseNursingCardDO cardDO =  nursingCardDao.findById(id).orElse(null);
        cardDO.setStatus(status);
        if(status==1){
            //计算到期时间
            BaseInpatientAreaDO areaDO = inpatientAreaDao.findById(cardDO.getInpatientAreaId()).orElse(null);
            if(areaDO!=null&&StringUtils.isNotBlank(areaDO.getTermOfValidity())){
                cardDO.setEndTime(DateUtil.getPreHours(new Date(),Integer.parseInt(areaDO.getTermOfValidity())));
            }else{
                cardDO.setEndTime(DateUtil.getPreHours(new Date(),24));
            }
        }
        if(status!=-2){
            BaseDoctorDO doctorDO = doctorDao.findById(doctor).orElse(null);
            cardDO.setDoctor(doctor);
            cardDO.setDoctorName(doctorDO.getName());
            cardDO.setCheckTime(new Date());
            cardDO.setCheckReason(checkReason);
        }
        nursingCardDao.save(cardDO);
    }

    /**
     * 统计申请中，已生效的陪护数
     * @param patient
     * @return
     */
    public int countByPatientAndStatus(String patient){
        return nursingCardDao.countByPatientAndStatus(patient);
    }

    public int countByInpatientAreaIdAndStatus(String inpatientAreaId){
        return nursingCardDao.countByInpatientAreaIdAndStatus(inpatientAreaId);
    }

    public int countByStatus(Integer status){
        return nursingCardDao.countByStatus(status);
    }

}
