package com.yihu.jw.processGuidance.service;

import com.yihu.jw.entity.base.processGuidance.*;
import com.yihu.jw.mysql.query.BaseJpaService;
import com.yihu.jw.processGuidance.dao.*;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * Created by yeshijie on 2022/10/8.
 */
@Service
public class BaseProcessGuidanceService extends BaseJpaService<BaseProcessGuidanceDO, BaseProcessGuidanceDao> {

    @Resource
    private BaseProcessGuidanceDao processGuidanceDao;
    @Resource
    private BaseProcessGuidanceDeptDao processGuidanceDeptDao;
    @Resource
    private BaseProcessGuidancePresetDao processGuidancePresetDao;
    @Resource
    private BaseProcessGuidanceNodeSmsDao processGuidanceNodeSmsDao;
    @Resource
    private BaseProcessGuidanceNodeWxDao processGuidanceNodeWxDao;
    @Resource
    private BaseProcessGuidanceNodeAppDao processGuidanceNodeAppDao;
    @Resource
    private BaseProcessGuidanceNodeDao processGuidanceNodeDao;

    @Transactional(rollbackFor = Exception.class)
    public Envelop delProcessGuidanceNode(String id) {
        processGuidanceNodeAppDao.deleteByNodeId(id);
        processGuidanceNodeDao.deleteById(id);
        return Envelop.getSuccess("删除成功");
    }

    @Transactional(rollbackFor = Exception.class)
    public Envelop delProcessGuidance(String id) {
        processGuidanceDeptDao.deleteByProcessGuidanceId(id);
        processGuidancePresetDao.deleteByProcessGuidanceId(id);
        processGuidanceDao.deleteById(id);
        return Envelop.getSuccess("删除成功");
    }

    @Transactional(rollbackFor = Exception.class)
    public ObjEnvelop saveProcessGuidanceNode(BaseProcessGuidanceNodeDO nodeDO) {
        String nodeId = nodeDO.getId();
        List<BaseProcessGuidanceNodeAppDO> nodeAppDOList = nodeDO.getNodeAppDOList();
        if(StringUtils.isNotBlank(nodeId)){
            //更新的时候先删除关联应用
            processGuidanceNodeAppDao.deleteByNodeId(nodeId);

            BaseProcessGuidanceNodeDO oldNode = processGuidanceNodeDao.findById(nodeId).orElse(null);
            if(oldNode==null){
                return ObjEnvelop.getError("参数错误，id不存在");
            }
            oldNode.setStatus(nodeDO.getStatus());
            oldNode.setNodeName(nodeDO.getNodeName());
            oldNode.setNodeCode(nodeDO.getNodeCode());
            oldNode.setNodeDescribe(nodeDO.getNodeDescribe());
            oldNode.setNotice(nodeDO.getNotice());
            nodeDO = processGuidanceNodeDao.save(oldNode);

            if(nodeAppDOList!=null&&nodeAppDOList.size()>0){
                nodeAppDOList.forEach(one->{
                    one.setCreateTime(new Date());
                    one.setNodeId(nodeId);
                });
                nodeAppDOList = processGuidanceNodeAppDao.saveAll(nodeAppDOList);
                nodeDO.setNodeAppDOList(nodeAppDOList);
            }
        }else{
            nodeDO.setCreateTime(new Date());
            nodeDO = processGuidanceNodeDao.save(nodeDO);
            final String nodeId1 = nodeDO.getId();

            if(nodeAppDOList!=null&&nodeAppDOList.size()>0){
                nodeAppDOList.forEach(one->{
                    one.setCreateTime(new Date());
                    one.setNodeId(nodeId1);
                });
                nodeAppDOList = processGuidanceNodeAppDao.saveAll(nodeAppDOList);
                nodeDO.setNodeAppDOList(nodeAppDOList);
            }
        }
        return ObjEnvelop.getSuccess("保存成功",nodeDO);
    }

    @Transactional(rollbackFor = Exception.class)
    public ObjEnvelop saveProcessGuidanceNodeWx(BaseProcessGuidanceNodeWxDO nodeWxDO) {
        String nodeWxId = nodeWxDO.getId();
        if(StringUtils.isNotBlank(nodeWxId)){
            BaseProcessGuidanceNodeWxDO oldNodeWx = processGuidanceNodeWxDao.findById(nodeWxId).orElse(null);
            if(oldNodeWx==null){
                return ObjEnvelop.getError("参数错误，id不存在");
            }
            oldNodeWx.setIsPush(nodeWxDO.getIsPush());
            oldNodeWx.setPreview(nodeWxDO.getPreview());
            oldNodeWx.setTemplateId(nodeWxDO.getTemplateId());
            oldNodeWx.setTemplateName(nodeWxDO.getTemplateName());
            nodeWxDO = processGuidanceNodeWxDao.save(oldNodeWx);
        }else{
            nodeWxDO = processGuidanceNodeWxDao.save(nodeWxDO);
        }

        return ObjEnvelop.getSuccess("保存成功",nodeWxDO);
    }

    @Transactional(rollbackFor = Exception.class)
    public ObjEnvelop saveProcessGuidanceNodeSms(BaseProcessGuidanceNodeSmsDO nodeSmsDO) {
        if(StringUtils.isBlank(nodeSmsDO.getNodeId())){
            return ObjEnvelop.getError("参数错误，节点id不能为空");
        }
        String nodeSmsDOId = nodeSmsDO.getId();
        if(StringUtils.isNotBlank(nodeSmsDOId)){
            BaseProcessGuidanceNodeSmsDO oldNodeSms = processGuidanceNodeSmsDao.findById(nodeSmsDOId).orElse(null);
            if(oldNodeSms==null){
                return ObjEnvelop.getError("参数错误，id不存在");
            }
            oldNodeSms.setContent(nodeSmsDO.getContent());
            oldNodeSms.setIsPush(nodeSmsDO.getIsPush());
            oldNodeSms.setTemplateId(nodeSmsDO.getTemplateId());
            nodeSmsDO = processGuidanceNodeSmsDao.save(oldNodeSms);
        }else{
            nodeSmsDO = processGuidanceNodeSmsDao.save(nodeSmsDO);
        }

        return ObjEnvelop.getSuccess("保存成功",nodeSmsDO);
    }

    @Transactional(rollbackFor = Exception.class)
    public ObjEnvelop saveProcessGuidance(BaseProcessGuidanceDO guidanceDO) {
        String guidanceId = guidanceDO.getId();
        List<BaseProcessGuidanceDeptDO> deptDOList = guidanceDO.getDeptDOList();
        List<BaseProcessGuidancePresetDO> presetDOList = guidanceDO.getPresetDOList();
        if(StringUtils.isNotBlank(guidanceId)){

            BaseProcessGuidanceDO old = processGuidanceDao.findById(guidanceId).orElse(null);
            if(old==null){
                return ObjEnvelop.getError("参数错误，id不存在");
            }
            //更新的时候先删除关联
            processGuidanceDeptDao.deleteByProcessGuidanceId(guidanceId);
            processGuidancePresetDao.deleteByProcessGuidanceId(guidanceId);

            old.setStatus(guidanceDO.getStatus());
            old.setName(guidanceDO.getName());
            old.setIsAllDept(guidanceDO.getIsAllDept());
            guidanceDO = processGuidanceDao.save(old);

            if(deptDOList!=null&&deptDOList.size()>0){
                deptDOList.forEach(one->{
                    one.setProcessGuidanceId(guidanceId);
                });
                deptDOList = processGuidanceDeptDao.saveAll(deptDOList);
                guidanceDO.setDeptDOList(deptDOList);
            }
            if(presetDOList!=null&&presetDOList.size()>0){
                presetDOList.forEach(one->{
                    one.setProcessGuidanceId(guidanceId);
                    one.setCreateTime(new Date());
                });
                presetDOList = processGuidancePresetDao.saveAll(presetDOList);
                guidanceDO.setPresetDOList(presetDOList);
            }
        }else{
            guidanceDO.setCreateTime(new Date());
            guidanceDO = processGuidanceDao.save(guidanceDO);
            final String guidanceDOId = guidanceDO.getId();

            if(deptDOList!=null&&deptDOList.size()>0){
                deptDOList.forEach(one->{
                    one.setProcessGuidanceId(guidanceDOId);
                });
                deptDOList = processGuidanceDeptDao.saveAll(deptDOList);
                guidanceDO.setDeptDOList(deptDOList);
            }
            if(presetDOList!=null&&presetDOList.size()>0){
                presetDOList.forEach(one->{
                    one.setProcessGuidanceId(guidanceDOId);
                    one.setCreateTime(new Date());
                });
                presetDOList = processGuidancePresetDao.saveAll(presetDOList);
                guidanceDO.setPresetDOList(presetDOList);
            }
        }

        return ObjEnvelop.getSuccess("保存成功",guidanceDO);
    }

}
