package com.yihu.jw.processGuidance.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.base.processGuidance.*;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.processGuidance.dao.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Created by yeshijie on 2022/11/18.
 */
@Service
public class ProcessGuidanceService {
    private static final Logger logger = LoggerFactory.getLogger(ProcessGuidanceService.class);

    @Autowired
    private BasePatientDao patientDao;
    @Autowired
    private BaseProcessGuidanceBookingDao processGuidanceBookingDao;
    @Autowired
    private BasePatientProcessGuidanceLogDao patientProcessGuidanceLogDao;
    @Autowired
    private BaseProcessGuidanceDao processGuidanceDao;
    @Autowired
    private BaseProcessGuidancePresetDao presetDao;
    @Autowired
    private BaseProcessGuidanceDeptDao deptDao;
    @Autowired
    private BaseProcessGuidanceThirdPushLogDao thirdPushLogDao;
    @Autowired
    private BaseProcessGuidanceNodeAppDao nodeAppDao;
    @Autowired
    private BaseProcessGuidanceNodeDao nodeDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;

    public Integer findStep(String patient){
        String sql = "SELECT CAST(node_code AS UNSIGNED) step from base_process_guidance_third_push_log " +
                "WHERE idcard = '"+patient+"' and guidance_id=1  ORDER BY step desc limit 1 ";
        List<Map<String,Object>> list = jdbcTemplate.queryForList(sql);
        if(list.size()>0){
            return Integer.parseInt(list.get(0).get("step")+"");
        }
        return null;
    }

    public Integer nextStep(String patient,Integer setp){
        setp++;
        List<BaseProcessGuidanceThirdPushLogDO> logDOList = thirdPushLogDao.findByIdcardAndNodeCode(patient,setp+"");
        if(logDOList.size()>0){
            BaseProcessGuidanceThirdPushLogDO logDO = logDOList.get(0);
            logDO.setGuidanceId("1");
            thirdPushLogDao.save(logDO);
        }else {
            setp--;
            List<BaseProcessGuidanceThirdPushLogDO> logDOS = thirdPushLogDao.findByIdcard(patient);
            if(logDOS.size()>0){
                for (BaseProcessGuidanceThirdPushLogDO logDO:logDOS){
                    if(!"1".equals(logDO.getNodeCode())){
                        logDO.setGuidanceId("0");
                    }
                }
                thirdPushLogDao.saveAll(logDOS);
                setp = 1;
            }

        }
        return setp;
    }


    public JSONArray findProcessGuidanceList(String patient,Integer isApp){
        JSONArray jsonArray = new JSONArray();
        List<BasePatientProcessGuidanceLogDO> logDOList = patientProcessGuidanceLogDao.findByPatientAndTypeAndStatus(patient,1,1);
        for (BasePatientProcessGuidanceLogDO logDO:logDOList){
            JSONObject json = new JSONObject();
            json.put("patient",logDO.getPatient());
            json.put("nodeCode",logDO.getNodeCode());
            json.put("code",logDO.getCode());
            json.put("relationId",logDO.getRelationId());

            JSONArray presetArray = new JSONArray();
            List<BaseProcessGuidancePresetDO> presetDOList = presetDao.findByProcessGuidanceId(logDO.getGuidanceId());
            List<BasePatientProcessGuidanceLogDO> logDOS = patientProcessGuidanceLogDao.findByPatientAndCodeAndType(patient,logDO.getCode(),2);
            Map<String,BasePatientProcessGuidanceLogDO> logDOMap = logDOS.stream().collect(Collectors.toMap
                    (BasePatientProcessGuidanceLogDO::getNodeCode,log->log,(oldLog,newLog)->oldLog));
            for (BaseProcessGuidancePresetDO presetDO:presetDOList){
                JSONObject presetJson = new JSONObject();
                String processCode = presetDO.getProcessCode();
                presetJson.put("processCode", processCode);
                presetJson.put("processName", presetDO.getProcessName());
                presetJson.put("sortNo", presetDO.getSortNo());
                presetJson.put("isOptional", presetDO.getIsOptional());
                if(logDOMap.containsKey(processCode)){
                    presetJson.put("status",1);
                    BaseProcessGuidanceBookingDO bookingDO = processGuidanceBookingDao.findById(logDOMap.get(processCode).getRelationId()).orElse(null);
                    presetJson.put("obj", bookingDO);
                    //注意事项
                    BaseProcessGuidanceNodeDO nodeDO = nodeDao.findByNodeCode(processCode);
                    if(nodeDO!=null){
                        presetJson.put("notice",nodeDO.getNotice());
                    }
                }else {
                    presetJson.put("status",0);
                }
                if(isApp==1){
                    List<BaseProcessGuidanceNodeAppDO> appDOS = nodeAppDao.findByNodeCode(processCode);
                    presetJson.put("appDOS", appDOS);
                }

                if("gotoHospital".equals(processCode)){
                    JSONObject obj = new JSONObject();
                    obj.put("address","厦门市海沧区浮莲路120号");
                    obj.put("tips","请携带您的身份证、医保卡");
                    presetJson.put("obj", obj);
                }
                if("gotoConsultationRoom".equals(processCode)){
                    JSONObject obj = new JSONObject();
                    obj.put("address","厦门市海沧区浮莲路120号");
                    obj.put("tips","可查看平面图进行指引");
                    presetJson.put("obj", obj);
                }
                if("gotoDoctor".equals(processCode)){
                    JSONObject obj = new JSONObject();
                    obj.put("doctorName","林殊");
                    obj.put("doctor","123456");
                    obj.put("patientNum","30");
                    obj.put("num","5");
                    obj.put("queueNo","18");
                    obj.put("wait","33分钟");
//                    obj.put("tips","");
                    presetJson.put("obj", obj);
                }
                if("gotoCheck".equals(processCode)){
                    JSONObject obj = new JSONObject();
                    obj.put("dept","消化内科");
                    obj.put("deptCode","1010");
                    obj.put("address","门诊四楼A区");
                    obj.put("tips","检查完成后返回就诊科室");
                    presetJson.put("obj", obj);
                }
                if("gotoFurtherConsultation".equals(processCode)){
                    JSONObject obj = new JSONObject();
                    obj.put("address","急诊大厅右侧自助机");
                    obj.put("tips","您于11月25日全身磁共振检查结果已出，请您近日到医院门诊3楼305室进行复诊，祝您健康。");
                    presetJson.put("obj", obj);
                }
                if("gotoTakeMedicine".equals(processCode)){
                    JSONObject obj = new JSONObject();
                    obj.put("address","急诊大厅右侧自助机");
                    obj.put("tips","您已完成就诊，可到急诊大厅右侧自助机上缴费领取取药凭证并进行取药。");
                    presetJson.put("obj", obj);
                }
                presetArray.add(presetJson);
            }
            json.put("presetDOList",presetArray);
            jsonArray.add(json);
        }
        return jsonArray;
    }


    /**
     * 医院推送的全流程指引
     * type 全流程流程类型
     */
    @Async
    public void sendGuidanceInfo(String jsonData){

        BaseProcessGuidanceThirdPushLogDO thirdPushLogDO = JSONObject.parseObject(jsonData, BaseProcessGuidanceThirdPushLogDO.class);
        String api = thirdPushLogDO.getApi();
        String param = thirdPushLogDO.getParam();
        String nodeCode = thirdPushLogDO.getNodeCode();
        if(StringUtils.isNotBlank(param)&&param.length()>22000){
            thirdPushLogDO.setParam(param.substring(0,22000));
        }
        try {
            /**
             * 根据不同的api调用不同的方法
             */
            switch (api){
                case "gotoOrg":
                    gotoOrg(thirdPushLogDO);
                    break;
                default:
                    logger.info("未找到适配接口");
                    break;
            }
        }catch (Exception e){
            e.printStackTrace();
        }

        thirdPushLogDO.setCreateTime(new Date());
        thirdPushLogDao.save(thirdPushLogDO);
    }

    @Transactional(rollbackFor = Exception.class)
    public void gotoOrg(BaseProcessGuidanceThirdPushLogDO thirdPushLogDO){
        BaseProcessGuidanceBookingDO bookingDO = JSONObject.parseObject(thirdPushLogDO.getParam(), BaseProcessGuidanceBookingDO.class);
        BasePatientDO patientDO = patientDao.findByIdcard(bookingDO.getIdcard());
        if(patientDO==null){
            bookingDO.setPatient(patientDO.getId());
        }
        bookingDO.setCreateTime(new Date());

        List<BaseProcessGuidancePresetDO> presetDOList = presetDao.findByProcessCode(thirdPushLogDO.getNodeCode());
        if(presetDOList.size()>0){
            BaseProcessGuidancePresetDO presetDO = presetDOList.get(0);
            bookingDO.setGuidanceId(presetDO.getProcessGuidanceId());
            thirdPushLogDO.setGuidanceId(presetDO.getProcessGuidanceId());
        }
        processGuidanceBookingDao.save(bookingDO);

        //新增流程开始日志
        //判断是否唯一的流程标志 暂无
//            String code = UUID.randomUUID().toString().replace("-", "");
        String code = bookingDO.getAdmNo();
        addLog(bookingDO.getPatient(),code, bookingDO.getId(), 1,1,thirdPushLogDO.getNodeCode(),bookingDO.getGuidanceId());
        //新增流程过程日志
        addLog(bookingDO.getPatient(),code, bookingDO.getId(), 2,1,thirdPushLogDO.getNodeCode(),bookingDO.getGuidanceId());
    }

    public void addLog(String patient,String code,String relationId,Integer type,Integer status,String nodeCode,String guidanceId){
        BasePatientProcessGuidanceLogDO logDO = new BasePatientProcessGuidanceLogDO();
        logDO.setCode(code);
        logDO.setPatient(patient);
        logDO.setRelationId(relationId);
        logDO.setType(type);
        logDO.setStatus(status);
        logDO.setGuidanceId(guidanceId);
        logDO.setNodeCode(nodeCode);
        logDO.setCreateTime(new Date());
        patientProcessGuidanceLogDao.save(logDO);
    }

    public void sendGuidance(String patient,String type,String orderId) {

    }
}
