package com.yihu.jw.team.service;

import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.common.AsynchService;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.team.BaseTeamDO;
import com.yihu.jw.entity.base.team.BaseTeamMemberDO;
import com.yihu.jw.entity.base.team.BaseTeamRelationDO;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.team.BaseDoctorTeamDao;
import com.yihu.jw.team.BaseDoctorTeamMemberDao;
import com.yihu.jw.team.BaseTeamRelationDao;
import com.yihu.jw.wlyy.service.WlyyBusinessService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 团队管理
 * Created by yeshijie on 2023/4/21.
 */
@Service
public class TeamService {

    @Autowired
    private BaseDoctorTeamDao teamDao;
    @Autowired
    private BaseDoctorTeamMemberDao teamMemberDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private BaseDoctorDao doctorDao;
    @Autowired
    private BasePatientDao patientDao;
    @Autowired
    private BaseTeamRelationDao teamRelationDao;
    @Autowired
    private AsynchService asynchService;
    @Autowired
    private WlyyBusinessService wlyyBusinessService;
    private static final String TONGAN_TOWN = "350212";

    //居民是否签约(签约详情)
    public Envelop isSign(String idCard,String info){
        JSONObject re = wlyyBusinessService.isSign(idCard);
        if(re!=null){
            JSONObject data = re.getJSONObject("obj");
            JSONObject json = new JSONObject();
            Integer adminTeamId = 0;
            if(data==null){
                //0未签约 1已签约不是同安社区 2已签约同安社区未关联专科团队 3已签约同安社区已关联专科团队
                json.put("status","0");
            }else{
                String hospital = data.getString("hospital");
                adminTeamId = data.getInteger("adminTeamId");
                if(hospital.startsWith(TONGAN_TOWN)){
                    List<BaseTeamRelationDO> list =  teamRelationDao.findAdminTeamById(adminTeamId);
                    if(list.size()>0){
                        json.put("status","3");
                    }else {
                        json.put("status","2");
                    }
                }else {
                    json.put("status","1");
                }
                if(StringUtils.isNotBlank(info)){
                    //返回详情
                    json.put("hospital",hospital);
                    json.put("hospitalName",data.getString("hospitalName"));
                    json.put("doctorName",data.getString("doctorName"));
//                    json.put("doctorHealthName",data.getString("doctorHealthName"));
                    json.put("jobName",data.getString("jobName"));
                    json.put("photo",data.getString("photo"));
                }
            }
            if(StringUtils.isNotBlank(info)){
                String sql = " SELECT t.id,t.dept_name deptName,t.recommend_doctor recommendDoctor,t.recommend_doctor_name recommendDoctorName,d.photo,d.job_title_name jobTitleName" +
                        " from base_team t LEFT JOIN base_team_relation r on t.id = r.team_id and r.admin_team_id= " + adminTeamId+
                        " LEFT JOIN base_doctor d on t.recommend_doctor = d.id " +
                        " WHERE t.del=1 and t.recommend='1' ORDER BY r.id desc LIMIT 3";
                List<Map<String,Object>> mapList = jdbcTemplate.queryForList(sql);
                json.put("doctorList",mapList);
            }
            return ObjEnvelop.getSuccess("获取成功",json);
        }else {
            return Envelop.getError("获取签约信息失败",-1);
        }
    }


    //删除团队成员
    public void delTeamMember(Long id){
        teamMemberDao.deleteById(id);
    }

    //获取医生信息
    public BaseDoctorDO doctorInfo(String id){
        return doctorDao.findByIdAndDel(id);
    }

    //保存团队
    @Transactional
    public BaseTeamDO saveTeam(String jsonData){
        BaseTeamDO teamDO = JSONObject.parseObject(jsonData,BaseTeamDO.class);
        if(teamDO.getCreateTime()==null){
            teamDO.setCreateTime(new Date());
        }

        if(StringUtils.isBlank(teamDO.getId())){
            teamDO.setDel("1");
        }
        teamDO = teamDao.save(teamDO);
        //判断推荐医生是否在团队中
        BaseTeamMemberDO memberDO = teamMemberDao.findUseDoctorCodeAndTeamCode(teamDO.getRecommendDoctor(),teamDO.getId());
        if(memberDO==null){
            memberDO = new BaseTeamMemberDO();
            memberDO.setDoctorCode(teamDO.getRecommendDoctor());
            memberDO.setCreateTime(new Date());
            memberDO.setUpdateTime(new Date());
            memberDO.setDel("1");
            memberDO.setTeamCode(teamDO.getId());
            memberDO.setOrgCode(teamDO.getOrgCode());
            teamMemberDao.save(memberDO);
        }

        return teamDO;
    }

    //按id获取专科团队信息
    public BaseTeamDO findTeamById(String id){
        return teamDao.findById(id).orElse(null);
    }

    //获取团队详情
    public Envelop teamDetail(String id){
        JSONObject json = new JSONObject();
        BaseTeamDO teamDO = findTeamById(id);
        List<Map<String,Object>> doctorList = new ArrayList<>();
        if(teamDO!=null){
            String recommendDoctor = teamDO.getRecommendDoctor();
            String sql = "SELECT d.id,d.name,d.photo,d.introduce,d.job_title_name jobTitleName,'0' recommend ,GROUP_CONCAT(h.dept_name) deptName " +
                    "from base_team_member m,base_doctor d LEFT JOIN base_doctor_hospital h on d.id = h.doctor_code and h.del='1' " +
                    "WHERE m.team_code='"+id+"' and m.del='1' and m.doctor_code=d.id ";
            if(StringUtils.isNotBlank(recommendDoctor)){
                sql += " and d.id <> '"+recommendDoctor+"'";
                String recSql = " SELECT d.id,d.name,d.photo,d.introduce,d.job_title_name jobTitleName,'1' recommend ,GROUP_CONCAT(h.dept_name) deptName " +
                        "from base_doctor d LEFT JOIN base_doctor_hospital h on d.id = h.doctor_code and h.del='1' " +
                        "WHERE d.id='"+recommendDoctor+"'  GROUP BY d.id,d.name,d.photo,d.introduce,d.job_title_name";
                List<Map<String,Object>> recList = jdbcTemplate.queryForList(recSql);
                if(recList.size()>0){
                    doctorList.add(recList.get(0));
                }
            }
            sql += " GROUP BY d.id,d.name,d.photo,d.introduce,d.job_title_name";
            List<Map<String,Object>> docList = jdbcTemplate.queryForList(sql);
            if(docList.size()>0){
                doctorList.addAll(docList);
            }
        }
        json.put("teamDO",teamDO);
        json.put("doctorList",doctorList);
        return ObjEnvelop.getSuccess("获取成功",json);
    }

    //获取专科团队列表
    public Envelop getTeamList(String type,Integer adminTeamId,String teamName, String dept, Integer page, Integer pageSize, String hospital,String recommend, String recommendDoctorName) {
        String countSql = " select count(t.id) ";
        String sql = "SELECT t.id,t.`name`,t.org_code orgCode,t.org_name orgName,t.dept,t.dept_name deptName,t.recommend_doctor recommendDoctor,d.photo," +
                "t.recommend_doctor_name recommendDoctorName,DATE_FORMAT(t.create_time,'%Y-%m-%d %H:%i:%S') createTime,t.intro,t.recommend,d.job_title_name jobTitleName ";
        String filter = " from base_team t left join base_doctor d on t.recommend_doctor=d.id ";
        if(adminTeamId!=null){
            filter += " inner join base_team_relation r on r.team_id=t.id and r.admin_team_id="+adminTeamId+" ";
        }
        filter += " WHERE t.del ='1'";
        if(StringUtils.isNotEmpty(hospital)){
            filter += " and t.org_code = '"+hospital+"' ";
        }
        if(StringUtils.isNotEmpty(dept)){
            filter += " and t.dept = '"+dept+"' ";
        }
        if(StringUtils.isNotEmpty(recommend)){
            filter += " and t.recommend = '"+recommend+"' ";
        }
        if(StringUtils.isNotEmpty(teamName)){
            filter += " and t.name like '%"+teamName+"%' ";
        }
        if(StringUtils.isNotEmpty(recommendDoctorName)){
            filter += " and t.recommend_doctor_name like '%"+recommendDoctorName+"%' ";
        }
        if (pageSize==null||pageSize <= 0) {
            pageSize = 10;
        }
        if(page==null||page<=0){
            page = 1;
        }
        String orderBy = "";
        if("1".equals(type)){
            orderBy = " order by t.recommend desc limit "+(page-1)*pageSize+","+pageSize;
        }else {
            orderBy = " order by t.id limit "+(page-1)*pageSize+","+pageSize;
        }
        Long count = jdbcTemplate.queryForObject(countSql+filter,Long.class);
        List<Map<String,Object>> list = jdbcTemplate.queryForList(sql+filter+orderBy);
        for (Map<String,Object> map:list){
            String id = map.get("id")+"";
            //获取团队成员
            String doctorSql = "SELECT d.id,d.name,d.photo from base_team_member m,base_doctor d " +
                    " WHERE m.team_code='"+id+"' and m.del='1' and m.doctor_code=d.id and d.del='1' ";
            List<Map<String,Object>> mapList = jdbcTemplate.queryForList(doctorSql);
            map.put("doctorList",mapList);
        }

        return PageEnvelop.getSuccessListWithPage("获取成功",list,page,pageSize,count);
    }


    //获取团队成员列表
    public Envelop getTeamMemberList(String teamId, String doctorName, Integer page, Integer pageSize, String mobile) {
        String countSql = " select count(t.id) ";
        String sql = "SELECT t.id,d.id doctor,d.`name`,d.idcard,d.job_title_name jobTitleName,d.mobile," +
                " DATE_FORMAT(t.create_time,'%Y-%m-%d %H:%i:%S') createTime ";
        String filter = " from base_team_member t,base_doctor d WHERE t.del ='1' and t.doctor_code = d.id";
        if(StringUtils.isNotEmpty(teamId)){
            filter += " and t.team_code = '"+teamId+"' ";
        }
        if(StringUtils.isNotEmpty(doctorName)){
            filter += " and d.`name` like '%"+doctorName+"%' ";
        }
        if(StringUtils.isNotEmpty(mobile)){
            filter += " and d.mobile like '%"+mobile+"%' ";
        }
        if (pageSize==null||pageSize <= 0) {
            pageSize = 10;
        }
        if(page==null||page<=0){
            page = 1;
        }
        String orderBy = " order by t.id limit "+(page-1)*pageSize+","+pageSize;
        Long count = jdbcTemplate.queryForObject(countSql+filter,Long.class);
        List<Map<String,Object>> list = jdbcTemplate.queryForList(sql+filter+orderBy);

        return PageEnvelop.getSuccessListWithPage("获取成功",list,page,pageSize,count);
    }

    //添加团队成员时的医生列表
    public Envelop doctorListByTeamId(String teamId,String name,String dept,Integer page, Integer pageSize){
        String countSql = " select count(d.id) ";
        String sql = "SELECT d.id,d.`name`,d.job_title_name jobTitleName,h.dept_code deptCode,h.dept_name deptName ,IF(m.id is null,'0','1') checked " ;
        String filter = "from base_doctor d  LEFT JOIN base_team_member m on d.id=m.doctor_code and m.del=1 ";
        if(StringUtils.isNotEmpty(teamId)){
            filter += " and m.team_code = '"+teamId+"' ";
        }
        filter += ",base_doctor_hospital h WHERE d.id = h.doctor_code";
        if(StringUtils.isNotEmpty(dept)){
            filter += " and h.dept_code = '"+dept+"'";
        }
        if(StringUtils.isNotEmpty(name)){
            filter += " and d.name like '%"+name+"%'";
        }
        String orderBy = " order by d.id limit "+(page-1)*pageSize+","+pageSize;
        Long count = jdbcTemplate.queryForObject(countSql+filter,Long.class);
        List<Map<String,Object>> list = jdbcTemplate.queryForList(sql+filter+orderBy);

        return PageEnvelop.getSuccessListWithPage("获取成功",list,page,pageSize,count);
    }

    //保存团队成员
    public void saveTeamMember(String teamId,String doctors){
        if(StringUtils.isNotEmpty(doctors)){
            BaseTeamDO teamDO = teamDao.findById(teamId).orElse(null);
            if(teamDO!=null){
                String doc[] = doctors.split(",");
                List<BaseTeamMemberDO> list = new ArrayList<>();
                for (String d:doc){
                    BaseTeamMemberDO memberDO = teamMemberDao.findUseDoctorCodeAndTeamCode(d,teamId);
                    if(memberDO==null){
                        memberDO = new BaseTeamMemberDO();
                        memberDO.setDel("1");
                        memberDO.setOrgCode(teamDO.getOrgCode());
                        memberDO.setTeamCode(teamId);
                        memberDO.setCreateTime(new Date());
                        memberDO.setUpdateTime(new Date());
                        memberDO.setDoctorCode(d);
                        list.add(memberDO);
                    }
                }
                teamMemberDao.saveAll(list);
            }
        }
    }

    //设置团队长
//    public void setTeamLeader(String teamId,String leadCode){
//        BaseTeamDO teamDO = teamDao.findById(teamId).orElse(null);
//        if(teamDO!=null){
//            teamDO.setLeaderCode(leadCode);
//        }
//    }

    //专科团队保存关联社区
    public void saveIjkTeam(String teamId,String adminTeamIds){
        if(StringUtils.isNotEmpty(adminTeamIds)){
            BaseTeamDO teamDO = teamDao.findById(teamId).orElse(null);
            if(teamDO!=null){
                String doc[] = adminTeamIds.split(",");
                List<BaseTeamRelationDO> list = new ArrayList<>();
                for (String d:doc){
                    Integer adminTeamId = Integer.parseInt(d);
                    List<BaseTeamRelationDO> temp = teamRelationDao.findAdminTeamById(teamId,adminTeamId);
                    if(temp.size()==0){
                        BaseTeamRelationDO relationDO = new BaseTeamRelationDO();
                        relationDO.setAdminTeamId(adminTeamId);
                        relationDO.setTeamId(teamId);
                        relationDO.setCreateTime(new Date());
                        list.add(relationDO);
                    }
                }
                teamRelationDao.saveAll(list);
            }
        }
        //同步i健康
        asynchService.saveIjkTeam(teamId,adminTeamIds);
    }

    @Transactional
    public void delIjkTeam(String teamId,Integer adminTeamId){
        teamRelationDao.delTeam(teamId,adminTeamId);
        //同步i健康
        asynchService.delIjkTeam(teamId,adminTeamId);
    }

}
