package com.yihu.jw.wechat.enterprise;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.wx.EnterpriseJsApiTicket;
import com.yihu.jw.entity.base.wx.WxWechatDO;
import com.yihu.jw.entity.hospital.enterprise.WxEnterpriseDO;
import com.yihu.jw.entity.hospital.enterprise.WxEnterpriseTokenDO;
import com.yihu.jw.entity.hospital.enterprise.WxEnterpriseUserDO;
import com.yihu.jw.entity.hospital.httplog.WlyyHttpLogDO;
import com.yihu.jw.hospital.httplog.dao.WlyyHttpLogDao;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.wechat.wxhttp.HttpUtil;
import com.yihu.jw.wechat.enterprise.dao.WxEnterpriseDao;
import com.yihu.jw.wechat.enterprise.dao.WxEnterpriseTokenDao;
import com.yihu.jw.wechat.enterprise.dao.WxEnterpriseUserDao;
import com.yihu.jw.wechat.service.WxAccessTokenService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Date;
import java.util.List;
import java.util.Random;

/**
 * Created by Trick on 2020/2/8.
 */
@Service
@Transactional
public class EnterpriseService {

    private Logger logger = LoggerFactory.getLogger(EnterpriseService.class);


    private String corpsecret = "SKuRnq7cZqE5ax1YxwBwkCa-jdujkzbpMjWDCYU2raE";

    private String corpid = "wwd7335a5d80a9b14f";

    private String agentId = "1000003";

    @Value("${FileTempPath.upload_temp_path}")
    private String upload_temp_path;

    @Autowired
    private WxEnterpriseTokenDao wxEnterpriseTokenDao;
    @Autowired
    private WxEnterpriseUserDao wxEnterpriseUserDao;
    @Autowired
    private WxEnterpriseDao wxEnterpriseDao;
    @Autowired
    private BaseDoctorDao baseDoctorDao;
    @Autowired
    private WlyyHttpLogDao wlyyHttpLogDao;
    @Autowired
    private WxAccessTokenService wxAccessTokenService;


    public String getToken(String enterpriseId)throws Exception{

        logger.info("getToken");
        if (enterpriseId.equalsIgnoreCase("xm_zsyy_wx")){
            enterpriseId ="zsyy";
        }

        WxEnterpriseDO wxEnterpriseDO = wxEnterpriseDao.findById(enterpriseId).orElse(null);

        List<WxEnterpriseTokenDO> wxEnterpriseTokenDOs = wxEnterpriseTokenDao.findByCodeOrderByCreateTimeDesc(wxEnterpriseDO.getCorpid());

        if(wxEnterpriseTokenDOs!=null&&wxEnterpriseTokenDOs.size()>0) {
            logger.info("token size:"+wxEnterpriseTokenDOs.size());
            WxEnterpriseTokenDO tokenDO = wxEnterpriseTokenDOs.get(0);
            if (tokenDO.getExpiresTime().after(new Date())) {
                return tokenDO.getAccessToken();
            }else{
                wxEnterpriseTokenDao.deleteAll(wxEnterpriseTokenDOs);
            }
        }

        String result = HttpUtil.sendGet("https://qyapi.weixin.qq.com/cgi-bin/gettoken?corpid="+wxEnterpriseDO.getCorpid()+"&corpsecret="+wxEnterpriseDO.getCorpsecret());

        logger.info("getToken result:"+result);

        JSONObject json = JSONObject.parseObject(result);

        //保存当前token
        WxEnterpriseTokenDO token = new WxEnterpriseTokenDO();
        Date date = new Date();
        token.setAccessToken(json.getString("access_token"));
        token.setCreateTime(date);
        token.setExpiresTime(DateUtil.setDateHours(date,2));
        token.setCode(wxEnterpriseDO.getCorpid());
        wxEnterpriseTokenDao.save(token);

        return token.getAccessToken();
    }

    /**
     * 小程序token
     * @param enterpriseId
     * @return
     * @throws Exception
     */
    public String getXCXToken(String enterpriseId)throws Exception{

        logger.info("getXCXToken");

        WxEnterpriseDO wxEnterpriseDO = wxEnterpriseDao.findById(enterpriseId+"_xcx").orElse(null);

        List<WxEnterpriseTokenDO> wxEnterpriseTokenDOs = wxEnterpriseTokenDao.findByCodeOrderByCreateTimeDesc(enterpriseId+"_xcx");

        if(wxEnterpriseTokenDOs!=null&&wxEnterpriseTokenDOs.size()>0) {
            logger.info("token size:"+wxEnterpriseTokenDOs.size());
            WxEnterpriseTokenDO tokenDO = wxEnterpriseTokenDOs.get(0);
            if (tokenDO.getExpiresTime().after(new Date())) {
                return tokenDO.getAccessToken();
            }else{
                wxEnterpriseTokenDao.deleteAll(wxEnterpriseTokenDOs);
            }
        }

        String result = HttpUtil.sendGet("https://qyapi.weixin.qq.com/cgi-bin/gettoken?corpid="+wxEnterpriseDO.getCorpid()+"&corpsecret="+wxEnterpriseDO.getCorpsecret());

        logger.info("getToken result:"+result);

        JSONObject json = JSONObject.parseObject(result);

        //保存当前token
        WxEnterpriseTokenDO token = new WxEnterpriseTokenDO();
        Date date = new Date();
        token.setAccessToken(json.getString("access_token"));
        token.setCreateTime(date);
        token.setExpiresTime(DateUtil.setDateHours(date,2));
        token.setCode(enterpriseId+"_xcx");
        wxEnterpriseTokenDao.save(token);

        return token.getAccessToken();
    }

    public String sendTextMes(String enterpriseId,String userId,String content)throws Exception{

        String url = "https://qyapi.weixin.qq.com/cgi-bin/message/send?access_token="+getToken(enterpriseId);

        JSONObject param = new JSONObject();
        param.put("touser",userId);
        param.put("toparty","");
        param.put("totag","");
        param.put("msgtype","text");

        WxEnterpriseDO wxEnterpriseDO = wxEnterpriseDao.findById(enterpriseId).orElse(null);

        param.put("agentid",wxEnterpriseDO.getAgentid());

        JSONObject text = new JSONObject();
        text.put("content",content);
        param.put("text",text);
        param.put("safe",0);
        param.put("enable_id_trans",0);
        param.put("enable_duplicate_check",0);

        String rs = HttpUtil.sendPost(url,param.toJSONString());

        logger.info("Mes:"+rs);

        return rs;
    }

    public String sendMKMesByDoctor(String enterpriseId,String doctor,String content)throws Exception {
        BaseDoctorDO baseDoctorDO = baseDoctorDao.findByIdAndDel(doctor);
        String res="";
        if (baseDoctorDO != null && StringUtils.isNotBlank(baseDoctorDO.getMobile())) {
            WxEnterpriseUserDO user = wxEnterpriseUserDao.findByEnterpriseIdAndMobile(enterpriseId, baseDoctorDO.getMobile());
            if (user != null) {
                WlyyHttpLogDO wlyyHttpLogDO = new WlyyHttpLogDO();
                wlyyHttpLogDO.setDoctor(doctor);
                wlyyHttpLogDO.setRequest(content);
                wlyyHttpLogDO.setName("im消息推送");
                wlyyHttpLogDO.setCode(enterpriseId);
                wlyyHttpLogDO.setCreateTime(new Date());
                wlyyHttpLogDO.setRequest("enterpriseId:"+enterpriseId+",doctor:"+doctor+",url:"+content);
                res=sendTextMes(enterpriseId, user.getUserid(), content);
                wlyyHttpLogDO.setResponse(res);
                logger.info(res);
                wlyyHttpLogDao.save(wlyyHttpLogDO);
                return res;
            }
        }
        return "";
    }

    public String  sendMKMesByMobile(String enterpriseId,String mobile,String content)throws Exception{

        WxEnterpriseUserDO user = wxEnterpriseUserDao.findByEnterpriseIdAndMobile(enterpriseId,mobile);
        if(user!=null){
            return sendTextMes(enterpriseId,user.getUserid(),content);
        }
        return "";
    }
    public String sendTWMesByDoctor(String enterpriseId,String doctor,String title,String description,String url)throws Exception{

        BaseDoctorDO baseDoctorDO = baseDoctorDao.findByIdAndDel(doctor);
        String res = "";
        if(baseDoctorDO!=null&&StringUtils.isNotBlank(baseDoctorDO.getMobile())){
            WxEnterpriseUserDO user = wxEnterpriseUserDao.findByEnterpriseIdAndMobile(enterpriseId,baseDoctorDO.getMobile());
            if(user!=null){
                res =sendTWMes(enterpriseId,user.getUserid(),title,description,url);
                logger.info("开始保存日志");
                logger.info("url==="+url);
                try {
                    if (StringUtils.isNoneBlank(url)){
                        logger.info("进入保存接口");
                        String outPatientId = url.substring(url.lastIndexOf("=")+1,url.length());
                        WlyyHttpLogDO wlyyHttpLogDO = new WlyyHttpLogDO();
                        wlyyHttpLogDO.setDoctor(doctor);
                        wlyyHttpLogDO.setRequest(url);
                        wlyyHttpLogDO.setName(title);
                        wlyyHttpLogDO.setCode(outPatientId);
                        wlyyHttpLogDO.setCreateTime(new Date());
                        wlyyHttpLogDO.setRequest("enterpriseId:"+enterpriseId+",doctor:"+doctor+",url:"+url);
                            wlyyHttpLogDO.setResponse(res);
                        logger.info(res);
                        wlyyHttpLogDao.save(wlyyHttpLogDO);
                    }
                }catch (Exception e){
                    e.printStackTrace();
                }
                return res;
            }
        }
        return "";
    }

    public String sendTWMesBymobile(String enterpriseId,String mobile,String title,String description,String url)throws Exception{
        WxEnterpriseUserDO user = wxEnterpriseUserDao.findByEnterpriseIdAndMobile(enterpriseId,mobile);
        if(user!=null){
            return sendTWMes(enterpriseId,user.getUserid(),title,description,url);
        }
        return "";
    }
    
    /**
     *
     * @param enterpriseId
     * @param doctor
     * @param appid
     * @param page
     * @param title
     * @param description （不能超过12个字符）
     * @param emphasis_first_item
     * @param content_item
     * @return
     * @throws Exception
     */
    public String sendXCXMesByDoctor(String enterpriseId, String doctor, String appid,String page, String title, String description, Boolean emphasis_first_item, String content_item)throws Exception {
        BaseDoctorDO baseDoctorDO = baseDoctorDao.findByIdAndDel(doctor);

        if (baseDoctorDO != null && StringUtils.isNotBlank(baseDoctorDO.getMobile())) {
            WxEnterpriseUserDO user = wxEnterpriseUserDao.findByEnterpriseIdAndMobile(enterpriseId, baseDoctorDO.getMobile());
            if (user != null) {
                return sendXCXMes(enterpriseId,user.getUserid(),appid,page,title,description,emphasis_first_item,content_item);
            }
        }
        return "";
    }

    public String sendXCXMesByMobile(String enterpriseId, String mobile, String appid,String page, String title, String description, Boolean emphasis_first_item, String content_item)throws Exception{

        WxEnterpriseUserDO user = wxEnterpriseUserDao.findByEnterpriseIdAndMobile(enterpriseId,mobile);
        if(user!=null){
            return sendXCXMes(enterpriseId,user.getUserid(),appid,page,title,description,emphasis_first_item,content_item);
        }
        return "";
    }


    public String sendTWMes(String enterpriseId,String userId,String title,String description,String url)throws Exception{

        String mesurl = "https://qyapi.weixin.qq.com/cgi-bin/message/send?access_token="+getToken(enterpriseId);

        JSONObject param = new JSONObject();
        param.put("touser",userId);
        param.put("toparty","");
        param.put("totag","");
        param.put("msgtype","news");
        WxEnterpriseDO wxEnterpriseDO = wxEnterpriseDao.findById(enterpriseId).orElse(null);

        param.put("agentid",wxEnterpriseDO.getAgentid());

        JSONObject news = new JSONObject();

        JSONArray articles = new JSONArray();
        JSONObject article = new JSONObject();

        article.put("title",title);
        article.put("description",description);
        article.put("url",url);
        article.put("picurl","");
        articles.add(article);

        news.put("articles",articles);

        param.put("news",news);
        param.put("enable_id_trans",0);
        param.put("enable_duplicate_check",0);

        logger.info("TWparam:"+param.toJSONString());

        String rs = HttpUtil.sendPost(mesurl,param.toJSONString());

        logger.info("TWMes:"+rs);

        return rs;
    }

    public String getDeptsId(String enterpriseId)throws Exception{

        logger.info("getDeptsId");

        String url = "https://qyapi.weixin.qq.com/cgi-bin/department/list?access_token="+getToken(enterpriseId);

        String rs = HttpUtil.sendGet(url);

        logger.info("dept rs:"+rs);

        JSONObject r = JSONObject.parseObject(rs);

        JSONArray depts = r.getJSONArray("department");

        String deptStr ="";

        if(depts!=null&&depts.size()>0){
            for(int i=0;i<depts.size();i++){
                JSONObject dept = (JSONObject) depts.get(i);
                deptStr+=dept.getString("id")+",";
            }
            deptStr = deptStr.substring(0,deptStr.length()-1);
        }
        logger.info("deptStr"+deptStr);

        return deptStr;
    }

    public String sendMKMes(String enterpriseId,String userId,String content)throws Exception{

        String mesurl = "https://qyapi.weixin.qq.com/cgi-bin/message/send?access_token="+getToken(enterpriseId);

        JSONObject param = new JSONObject();
        param.put("touser",userId);
        param.put("toparty","");
        param.put("totag","");
        param.put("msgtype","markdown");
        WxEnterpriseDO wxEnterpriseDO = wxEnterpriseDao.findById(enterpriseId).orElse(null);

        param.put("agentid",wxEnterpriseDO.getAgentid());

        JSONObject markdown = new JSONObject();

        markdown.put("content",content);

        param.put("markdown",markdown);
        param.put("enable_duplicate_check",0);

        logger.info("MKparam:"+param.toJSONString());

        String rs = HttpUtil.sendPost(mesurl,param.toJSONString());

        logger.info("MKMes:"+rs);
        WlyyHttpLogDO wlyyHttpLogDO = new WlyyHttpLogDO();
        wlyyHttpLogDO.setDoctor(userId);
        wlyyHttpLogDO.setRequest(content);
        wlyyHttpLogDO.setName("im消息推送");
        wlyyHttpLogDO.setCode(enterpriseId);
        wlyyHttpLogDO.setCreateTime(new Date());
        wlyyHttpLogDO.setRequest("enterpriseId:"+enterpriseId+",doctor:"+userId+",url:"+content);
        wlyyHttpLogDO.setResponse(rs);
        logger.info(rs);
        wlyyHttpLogDao.save(wlyyHttpLogDO);

        return rs;
    }

    /**
     *
     touser	否	成员ID列表（消息接收者，多个接收者用‘|’分隔，最多支持1000个）
     toparty	否	部门ID列表，多个接收者用‘|’分隔，最多支持100个。
     totag	否	标签ID列表，多个接收者用‘|’分隔，最多支持100个。
     msgtype	是	消息类型，此时固定为：miniprogram_notice
     appid	是	小程序appid，必须是与当前小程序应用关联的小程序
     page	否	点击消息卡片后的小程序页面，仅限本小程序内的页面。该字段不填则消息点击后不跳转。
     title	是	消息标题，长度限制4-12个汉字（支持id转译）
     description	否	消息描述，长度限制4-12个汉字（支持id转译）
     emphasis_first_item	否	是否放大第一个content_item
     content_item	否	消息内容键值对，最多允许10个item
     key	是	长度10个汉字以内
     value	是	长度30个汉字以内（支持id转译）
     enable_id_trans	否	表示是否开启id转译，0表示否，1表示是，默认0
     enable_duplicate_check	否	表示是否开启重复消息检查，0表示否，1表示是，默认0
     duplicate_check_interval	否	表示是否重复消息检查的时间间隔，默认1800s，最大不超过4小时
     * @throws Exception
     */
    public String sendXCXMes(String enterpriseId, String userId, String appid,String page, String title, String description, Boolean emphasis_first_item, String content_item)throws Exception{

        String mesurl = "https://qyapi.weixin.qq.com/cgi-bin/message/send?access_token="+getXCXToken(enterpriseId);

        JSONObject param = new JSONObject();
        param.put("touser",userId);
        param.put("toparty","");
        param.put("totag","");
        param.put("msgtype","miniprogram_notice");

        JSONObject miniprogram_notice = new JSONObject();
        miniprogram_notice.put("appid",appid);
        miniprogram_notice.put("page",page);
        miniprogram_notice.put("title",title);
        miniprogram_notice.put("description",description);
        miniprogram_notice.put("emphasis_first_item",emphasis_first_item);
        JSONArray contents = JSONArray.parseArray(content_item);
        miniprogram_notice.put("content_item",contents);
        param.put("miniprogram_notice",miniprogram_notice);

        param.put("enable_id_trans",0);
        param.put("enable_duplicate_check",0);


        logger.info("XCXparam:"+param.toJSONString());

        String rs = HttpUtil.sendPost(mesurl,param.toJSONString());

        logger.info("XCXMes:"+rs);

        return rs;
    }


    public JSONArray getUserInfo(String enterpriseId,String dept)throws Exception{

        logger.info("getUserInfo dept:"+dept);

        if(StringUtils.isNotBlank(dept)){

            String userUrl ="https://qyapi.weixin.qq.com/cgi-bin/user/list?access_token="+getToken(enterpriseId)+"&department_id="+dept+"&fetch_child=FETCH_CHILD";

            String users = HttpUtil.sendGet(userUrl);

            logger.info("usrs:"+users);

            JSONObject us = JSONObject.parseObject(users);

            JSONArray list = us.getJSONArray("userlist");

            logger.info("list size:"+list.size());

            return list;
        }
        return null;
    }

    public String saveAllUser(String enterpriseId)throws Exception{
        logger.info("saveAllUser");
        String depts = getDeptsId(enterpriseId);
        if(StringUtils.isNotBlank(depts)){
            //多部门过滤
            String dept[] = depts.split(",");
            logger.info("dept size:"+dept.length);
            if(dept!=null&&dept.length>0){
                for(String dp : dept){
                    JSONArray usrs = getUserInfo(enterpriseId,dp);
                    if(usrs!=null&&usrs.size()>0){
                        for(int i=0;i<usrs.size();i++){
                            JSONObject user = (JSONObject) usrs.get(i);
                            String mobile =user.getString("mobile");
                            if(StringUtils.isNotBlank(mobile)){
                                WxEnterpriseUserDO wxEnterpriseUserDO = wxEnterpriseUserDao.findByEnterpriseIdAndMobile(enterpriseId,mobile);
                                if(wxEnterpriseUserDO!=null){
                                    wxEnterpriseUserDO.setName(user.getString("name"));
                                    wxEnterpriseUserDO.setUserid(user.getString("userid"));
                                    wxEnterpriseUserDao.save(wxEnterpriseUserDO);
                                }else{
                                    WxEnterpriseUserDO userDO = new WxEnterpriseUserDO();
                                    userDO.setEnterpriseId(enterpriseId);
                                    userDO.setMobile(mobile);
                                    userDO.setUserid(user.getString("userid"));
                                    userDO.setName(user.getString("name"));
                                    wxEnterpriseUserDao.save(userDO);
                                }
                            }
                        }
                    }
                }
            }
        }
        return "ok";
    }

    /**
     * 获取企业微信的jsapi_ticket
     *
     * @return
     */
    public String getEnterpriseJsapi_ticketByToken(String wxId) throws Exception {
        try {

            System.out.println("getEnterpriseJsapi_ticketByToken="+wxId);
            Iterable<EnterpriseJsApiTicket> jsapiTickets = null;
            WxWechatDO wxWechat = null;

            //中山医院互联网医院
            if("xm_zsyy_wx".equals(wxId) || "xm_xzzx_wx".equals(wxId) || "xm_ykyy_wx".equals(wxId)||"xm_hcyy_wx".equals(wxId)){
                jsapiTickets = wxAccessTokenService.findEnterpriseJsapiTicket(wxId);
            }else{
                wxWechat = wxAccessTokenService.getWxInfo(wxId);
                jsapiTickets = wxAccessTokenService.findEnterpriseJsapiTicket(wxWechat.getAppOriginId());
            }

            if (jsapiTickets != null) {
                for (EnterpriseJsApiTicket jsApiTicket : jsapiTickets) {
                    if ((System.currentTimeMillis() - jsApiTicket.getAdd_timestamp()) < (jsApiTicket.getExpires_in() * 1000)) {
                        return jsApiTicket.getJsapi_ticket();
                    } else {
                        wxAccessTokenService.delEnterpriseJsapiTicket(jsApiTicket);
                        break;
                    }
                }
            }
            String token = "";

            //中山医院互联网医院

            token = getToken(wxId);
            System.out.println("getticket_token:"+token);
            if (token != null) {
                String token_url = "https://qyapi.weixin.qq.com/cgi-bin/get_jsapi_ticket";
                String params = "access_token=" + token ;
                String result = HttpUtil.sendGet(token_url, params);
                System.out.println("result:"+result);
                org.json.JSONObject json = new org.json.JSONObject(result);
                System.out.println("getticket:"+result);
                if (json.has("ticket")) {
                    String ticket = json.get("ticket").toString();
                    String expires_in = json.get("expires_in").toString();
                    EnterpriseJsApiTicket newJsApiTicket = new EnterpriseJsApiTicket();
                    newJsApiTicket.setJsapi_ticket(ticket);
                    newJsApiTicket.setExpires_in(Long.parseLong(expires_in));
                    if("xm_zsyy_wx".equals(wxId) || "xm_xzzx_wx".equals(wxId) || "xm_ykyy_wx".equals(wxId)||"xm_hcyy_wx".equals(wxId)){
                        newJsApiTicket.setAcc_id(wxId);
                    }else{
                        newJsApiTicket.setAcc_id(wxWechat.getAppOriginId());
                    }

                    wxAccessTokenService.addEnterpriseJsapiTicket(newJsApiTicket);
                    return ticket;
                } else {
                    return null;
                }
            } else {
                return null;
            }
        } catch (Exception e) {
            throw new Exception(e.getMessage());
        }
    }

    /**
     * 下载多媒体文件（请注意，视频文件不支持下载，调用该接口需http协议）
     *
     * @return
     */
    public InputStream getInputStream(String mediaId, String wechat_appid) throws Exception {
        String accessToken = getToken(wechat_appid);
        InputStream is = null;
        String url = "https://qyapi.weixin.qq.com/cgi-bin/media/get?access_token=" + accessToken + "&media_id=" + mediaId;
//		String url = "http://file.api.weixin.qq.com/cgi-bin/media/get?access_token=30_OBJbMMPW9TnyGHKf32bcjkXtxWnzBrtqWlSMBnNk8MWpJ6qis2TnsecRYNZImgahdx30WrsI1bGco6K67-j4sT1QkDxYKBrVvjaYF6QgbY8dgBMzuoARKoaxtX3ehiYvdCLSUHvogrpF3n0GANIfCHAHUP&media_id=R_060hnyXAFyHD9HaOLl15xhDIzDvcA61-Wz34GkXmCakdrsFJqxrPEBHewmAEK9";
        logger.info("media/get" + url);
        try {
            URL urlGet = new URL(url);
            HttpURLConnection http = (HttpURLConnection) urlGet.openConnection();
            http.setRequestMethod("GET"); // 必须是get方式请求
            http.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
            http.setDoOutput(true);
            http.setDoInput(true);
            System.setProperty("sun.net.client.defaultConnectTimeout", "30000");// 连接超时30秒
            System.setProperty("sun.net.client.defaultReadTimeout", "30000"); // 读取超时30秒
            http.connect();

            // 获取文件转化为byte流
            is = http.getInputStream();
            if (is.available() <= 200) {
                logger.info("下载多媒体文件 accessToken 过期");
                accessToken=getToken(wechat_appid);
                is = getInputStreamReLoad(mediaId, wechat_appid, accessToken);
                return is;
            }


        } catch (Exception e) {
            logger.info(e.getMessage());
            e.printStackTrace();
            throw new Exception("下载失败，" + e.getMessage());
        }
        return is;
    }

    /**
     * accessToken 过期时重新下载多媒体文件
     * 下载多媒体文件（请注意，视频文件不支持下载，调用该接口需http协议）
     *
     * @return
     */
    public InputStream getInputStreamReLoad(String mediaId, String wechat_appid, String accessToken) throws Exception {
        logger.info("accessToken 过期时重新下载多媒体文件：accessToken：" + accessToken);
        logger.info("accessToken 过期时重新下载多媒体文件：wechat_appid：" + wechat_appid);
        logger.info("accessToken 过期时重新下载多媒体文件：mediaId：" + mediaId);
        InputStream is = null;
        String url = "https://qyapi.weixin.qq.com/cgi-bin/media/get?access_token=" + accessToken + "&media_id=" + mediaId;
        logger.info("media/get" + url);
        try {
            URL urlGet = new URL(url);
            HttpURLConnection http = (HttpURLConnection) urlGet.openConnection();
            http.setRequestMethod("GET"); // 必须是get方式请求
            http.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
            http.setDoOutput(true);
            http.setDoInput(true);
            System.setProperty("sun.net.client.defaultConnectTimeout", "30000");// 连接超时30秒
            System.setProperty("sun.net.client.defaultReadTimeout", "30000"); // 读取超时30秒
            http.connect();
            // 获取文件转化为byte流
            is = http.getInputStream();

        } catch (Exception e) {
            logger.info(e.getMessage());
            e.printStackTrace();
            throw new Exception("下载失败，" + e.getMessage());
        }
        return is;
    }

    /**
     * 获取下载语音信息（jpg）
     *
     * @param mediaId 文件的id
     * @throws Exception
     */
    public String saveVoiceToDisk(String mediaId, String wechat_appid) throws Exception {
        // 文件保存的临时路径
        String tempPath = upload_temp_path + File.separator;
        System.out.println("tempPath:" + tempPath);
        // 拼接年月日路径
        String datePath = DateUtil.getStringDate("yyyy") + File.separator + DateUtil.getStringDate("MM") + File.separator + DateUtil.getStringDate("dd") + File.separator;
        System.out.println("datePath:" + datePath);
        // 重命名文件
        String fileBase = DateUtil.dateToStr(new Date(), DateUtil.YYYYMMDDHHMMSS) + "_" + new Random().nextInt(1000);
        System.out.println("fileBase:" + fileBase);
        String newFileName = fileBase + ".amr";
        String mp3FileName = fileBase + ".mp3";
        System.out.println("newFileName:" + newFileName);
        System.out.println("mp3FileName:" + mp3FileName);
        // 保存路径
        File uploadFile = new File(tempPath + datePath + newFileName);

        InputStream inputStream = null;
        FileOutputStream fileOutputStream = null;
        try {
            if (!uploadFile.getParentFile().exists()) {
                uploadFile.getParentFile().mkdirs();
            }
            inputStream = getInputStream(mediaId, wechat_appid);
            byte[] data = new byte[1024];
            int len = 0;
            fileOutputStream = new FileOutputStream(uploadFile);
            while ((len = inputStream.read(data)) != -1) {
                fileOutputStream.write(data, 0, len);
            }
            String amrFilePath = tempPath + datePath + newFileName;
            String Mp3FilePath = tempPath + datePath + mp3FileName;
            System.out.println("amrFilePath:" + amrFilePath);
            System.out.println("Mp3FilePath:" + Mp3FilePath);
//			fileUtil.changeToMp3(amrFilePath,Mp3FilePath);
            // 返回保存路径
            return amrFilePath;
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (inputStream != null) {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (fileOutputStream != null) {
                try {
                    fileOutputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return null;
    }

    /**
     * 获取微信服务器语音
     *
     * @return
     */
    public String fetchWxVoices(String wecha_appid,HttpServletRequest request) {
        String voiceIds = "";
        try {
            String voices = request.getParameter("voices");
            System.out.println("voices" + voices);
            if (org.apache.commons.lang3.StringUtils.isEmpty(voices)) {
                return voices;
            }
            String[] mediaIds = voices.split(",");
            for (String mediaId : mediaIds) {
                if (org.apache.commons.lang3.StringUtils.isEmpty(mediaId)) {
                    continue;
                }
                String temp = saveVoiceToDisk(mediaId, wecha_appid);
                if (org.apache.commons.lang3.StringUtils.isNotEmpty(temp)) {
                    if (voiceIds.length() == 0) {
                        voiceIds = temp;
                    } else {
                        voiceIds += "," + temp;
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
//			error(e);
        }
        return voiceIds;
    }
//    public String send

//    public static void main(String args[])throws Exception{
//
//        String corpsecret = "SKuRnq7cZqE5ax1YxwBwkCa-jdujkzbpMjWDCYU2raE";
//
//        String corpid = "wwd7335a5d80a9b14f";
//
//        String agentId = "1000003";
//
//        String result = HttpUtil.sendGet("https://qyapi.weixin.qq.com/cgi-bin/gettoken?corpid="+corpid+"&corpsecret="+corpsecret);
//
//        JSONObject json = JSONObject.parseObject(result);
//
//        String token = json.getString("access_token");
//
//        String url = "https://qyapi.weixin.qq.com/cgi-bin/message/send?access_token="+token;
//
//        JSONObject param = new JSONObject();
//        param.put("touser","MuOu");
//        param.put("toparty","");
//        param.put("totag","");
//        param.put("msgtype","markdown");
//        param.put("agentid",agentId);
//        JSONObject markdown = new JSONObject();
//
//        markdown.put("content","dd[查看详情](www.baidu.com)");
//
//        param.put("markdown",markdown);
//        param.put("enable_duplicate_check",0);
//
//
//        String rs = HttpUtil.sendPost(url,param.toJSONString());
//
//        System.out.println(rs);
//    }


//    public static void main(String args[])throws Exception{
//
//        String corpsecret = "SKuRnq7cZqE5ax1YxwBwkCa-jdujkzbpMjWDCYU2raE";
//
//        String corpid = "wwd7335a5d80a9b14f";
//
//        String agentId = "1000003";
//
//        String result = HttpUtil.sendGet("https://qyapi.weixin.qq.com/cgi-bin/gettoken?corpid="+corpid+"&corpsecret="+corpsecret);
//
//        JSONObject json = JSONObject.parseObject(result);
//
//        String token = json.getString("access_token");
//
//        String url = "https://qyapi.weixin.qq.com/cgi-bin/message/send?access_token="+token;
//
//        JSONObject param = new JSONObject();
//        param.put("touser","ShenZaiXin");
//        param.put("toparty","");
//        param.put("totag","");
//        param.put("msgtype","news");
//        param.put("agentid",agentId);
//
//        JSONObject news = new JSONObject();
//
//        JSONArray articles = new JSONArray();
//        JSONObject article = new JSONObject();
//
//        article.put("title","标题");
//        article.put("description","图文描述");
//        article.put("url","www.baidu.com");
//        article.put("picurl","");
//        articles.add(article);
//
//        news.put("articles",articles);
//
//        param.put("news",news);
//        param.put("enable_id_trans",0);
//        param.put("enable_duplicate_check",0);
//
//        String rs = HttpUtil.sendPost(url,param.toJSONString());
//
//    }

//    public static void main(String args[])throws Exception{
//
//        String corpsecret = "Uo-QnbYthpUC0tZt--l71n3QEafT3g2oH_j4mLjCokU";
//
//        String corpid = "ww4bf28f7a26e67fee";
//
//        String result = HttpUtil.sendGet("https://qyapi.weixin.qq.com/cgi-bin/gettoken?corpid="+corpid+"&corpsecret="+corpsecret);
//
//        JSONObject json = JSONObject.parseObject(result);
//
//        String token = json.getString("access_token");
//
////        String url = "https://qyapi.weixin.qq.com/cgi-bin/department/list?access_token="+token;
////
////        String rs = HttpUtil.sendGet(url);
////
////        JSONObject r = JSONObject.parseObject(rs);
////
////        JSONArray depts = r.getJSONArray("department");
////
////        String deptStr ="";
////
////        if(depts!=null&&depts.size()>0){
////            for(int i=0;i<depts.size();i++){
////                JSONObject dept = (JSONObject) depts.get(i);
////                deptStr+=dept.getString("id")+"|";
////            }
////           deptStr = deptStr.substring(0,deptStr.length()-1);
////        }
////        System.out.println("deptStr"+deptStr);
//
//        String userUrl ="https://qyapi.weixin.qq.com/cgi-bin/user/list?access_token="+token+"&department_id=20&fetch_child=FETCH_CHILD";
//
//        String users = HttpUtil.sendGet(userUrl);
//
//        JSONObject us = JSONObject.parseObject(users);
//
//        JSONArray list = us.getJSONArray("userlist");
//
//        System.out.println("list"+list.size());
//
//    }



}
