package com.yihu.jw.im.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.entity.base.dict.DictHospitalDeptDO;
import com.yihu.jw.entity.hospital.consult.WlyyHospitalSysDictDO;
import com.yihu.jw.entity.knowledge.*;
import com.yihu.jw.im.util.ImUtil;
import com.yihu.jw.knowledge.dao.*;
import com.yihu.jw.knowledge.service.BaseKnowledgeDictService;
import com.yihu.jw.sms.dao.HospitalSysDictDao;
import com.yihu.jw.util.date.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 机器人
 * Created by yeshijie on 2023/4/12.
 */
@Service
public class RobotService {

    @Autowired
    private ImUtil imUtil;
    @Autowired
    private BaseSystemDialogSettingDao dialogSettingDao;
    @Autowired
    private BaseKnowledgeDictDao knowledgeDictDao;
    @Autowired
    private BaseKnowledgeDictRelationDao knowledgeDictRelationDao;
    @Autowired
    private BaseKnowledgeQuestionDao knowledgeQuestionDao;
    @Autowired
    private  BaseKnowledgeFlowConfigurationRelationDao knowledgeFlowConfigurationRelationDao;
    @Autowired
    private BaseKnowledgeQuestionsDao knowledgeQuestionsDao;
    @Autowired
    private BaseKnowledgeQuestionsRelationDao knowledgeQuestionsRelationDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private ImService imService;
    @Autowired
    private HospitalSysDictDao hospitalSysDictDao;
    @Autowired
    private BaseKnowledgeDictService baseKnowledgeDictService;


    /**
     * 发送卡片
     * @param session_id
     * @param type 会话类型
     * @param cardType 1 常见问题 2科室医生 3 症状描述
     */
    public void sendCard(String session_id,String type,String cardType){
        if("1".equals(cardType)){
            //1常见问题
            List<BaseKnowledgeQuestion> questionList = knowledgeQuestionDao.findTopList(DateUtil.getStringDate(),5);
            JSONArray jsonArray = new JSONArray();
            for (BaseKnowledgeQuestion question:questionList){
                JSONObject json = new JSONObject();
                json.put("id",question.getId());
                json.put("questionName",question.getQuestionName());
                jsonArray.add(jsonArray);
            }
            JSONObject result = new JSONObject();
            result.put("content",jsonArray);
            result.put("content_type",ImUtil.ContentType.KnowCommonQuestion);
            result.put("session_id",session_id);
            sendReply(result);
            return;
        }
    }

    //获取发送者
    public WlyyHospitalSysDictDO getSender(){
        WlyyHospitalSysDictDO dictDO = hospitalSysDictDao.findById("customer").orElse(null);
        if(dictDO==null){
            dictDO = new WlyyHospitalSysDictDO();
            dictDO.setDictCode("customer");
            dictDO.setDictValue("客服");
        }
        return dictDO;
    }

    //发送回复消息
    public void sendReply(JSONObject json){
        try {
            WlyyHospitalSysDictDO dictDO = getSender();
            imService.patientGuaidenceAppend(dictDO.getDictCode(),dictDO.getDictValue(),json.getString("session_id"),
                    json.getString("content_type"),json.getString("content"),"1",null);
        }catch (Exception e){
            e.printStackTrace();
        }
        json = null;
    }

    public void sendReplyMap(Map<String,JSONObject> map){
        try {
            WlyyHospitalSysDictDO dictDO = getSender();
            for (JSONObject json:map.values()){
                imService.patientGuaidenceAppend(dictDO.getDictCode(),dictDO.getDictValue(),json.getString("session_id"),
                        json.getString("content_type"),json.getString("content"),"1",null);
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        map = null;
    }

    //智能导致助手回复
    public Map<String,JSONObject> robotReply(String session_id,String type,String content){
        Map<String,JSONObject> map = new HashMap<>();
        try {
            List<BaseSystemDialogSetting> settingList = dialogSettingDao.selectBySystemTypeAndFlag(sessionTypeToType(type),1);
            if(settingList.size()==0){
                //未配置自动回复内容
                return map;
            }
            Map<String,BaseSystemDialogSetting> mapSetting = settingList.stream().collect(Collectors.toMap(BaseSystemDialogSetting::getFunctionType, v -> v,(o1,o2)->o1));
            //欢迎语
            welcome(session_id,mapSetting,map);
            //自动匹配回复文字内容
//            replyContent(session_id,type,content,mapSetting,map);
        }catch (Exception e){
            e.printStackTrace();
        }
        return map;
    }

    public void reply(String sessionType,String content,String consult,Integer contentType){
        if(contentType==1||contentType==3001){
            //常见问题
            replyContent(sessionType,content,consult,contentType);
            return;
        }
        if(contentType==3006){
            //回复疾病
            replyDisease(sessionType,content,null,consult,contentType+"");
            return;
        }
    }

    //回复疾病
    public void replyDisease(String sessionType,String content,String sessionId,String consult,String contentType){
        try {
            JSONObject jsonObject = JSON.parseObject(content);
            String id = jsonObject.getString("id");
            JSONArray jsonArray = baseKnowledgeDictService.findDeptById(id);
            WlyyHospitalSysDictDO dictDO = getSender();
            if(StringUtils.isNotEmpty(consult)){
                imUtil.sendTopicIM(dictDO.getDictCode(), dictDO.getDictName(), consult, ImUtil.ContentType.KnowSymptomsDiseaseA.getValue(), jsonArray.toJSONString(),null);
            }else {
                imService.patientGuaidenceAppend(dictDO.getDictCode(),dictDO.getDictValue(),sessionId,
                        ImUtil.ContentType.KnowSymptomsDiseaseA.getValue(),jsonArray.toJSONString(),"1",null);
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    /**
     *
     * @param sessionType 会话类型
     * @param content 消息内容
     * @param consult 咨询code
     * @param contentType 消息类型 文本1 问题集3002
     */
    //普通咨询常见问题匹配
    public void replyContent(String sessionType,String content,String consult,Integer contentType){
        try {
            WlyyHospitalSysDictDO dictDO = getSender();
            if (contentType==3003){
                String contentTypes = ImUtil.ContentType.KnowCommonDict+"";
                String contentMsg = selectByContentType(contentTypes,content).toJSONString();
                imUtil.sendTopicIM(dictDO.getDictCode(), dictDO.getDictName(), consult, contentTypes, contentMsg,null);
            }
            BaseSystemDialogSetting setting = dialogSettingDao.findBySystemTypeAndFunctionType(sessionTypeToType(sessionType),"7",1);
            if(setting==null){
                return;
            }
            //查找相似问题
            BaseKnowledgeQuestion question = new BaseKnowledgeQuestion();
            if (contentType==1){
                question = findQuestionLikeName(content);
            }else if ((contentType+"").equalsIgnoreCase(ImUtil.ContentType.KnowCommonQuestions+"")){
                question = knowledgeQuestionDao.findById(content).get();
            }
            if(question!=null&&StringUtils.isNotBlank(question.getAnswer())){
                //匹配到相关问题后先推送答案
                String answer = question.getAnswer();
                imUtil.sendTopicIM(dictDO.getDictCode(), dictDO.getDictName(), consult, "1", answer,null);
                //匹配问题是否关联其他配置；
                if (question.getFlag()==null){
                    List<BaseKnowledgeFlowConfigurationRelation> flowConfigurationRelations = findFlowConfigurationRelationLikeName(content);
                    if (flowConfigurationRelations!=null&&flowConfigurationRelations.size()!=0){
                        BaseKnowledgeFlowConfigurationRelation flowConfigurationRelation = flowConfigurationRelations.get(0);
                        String contentTypes = "1";
                        String contentMsg = "";
                        if (flowConfigurationRelation.getRelationType().equalsIgnoreCase("2")){
                            contentTypes = ImUtil.ContentType.KnowCommonDict+"";
                            contentMsg = selectByContentType(contentTypes,flowConfigurationRelation.getRelationCode()).toJSONString();
                        }else  if (flowConfigurationRelation.getRelationType().equalsIgnoreCase("3")){
                            contentTypes = ImUtil.ContentType.KnowCommonQuestions+"";
                            contentMsg = selectByContentType(contentTypes,flowConfigurationRelation.getRelationCode()).toJSONString();
                        }
                        imUtil.sendTopicIM(dictDO.getDictCode(), dictDO.getDictName(), consult, contentTypes, contentMsg,null);
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }


    /**
     *
     * @param sessionId 会话id
     * @param type 咨询类型
     * @param content 内容
     * @param contentType 消息类型
     * @param businessType  业务类型
     * @param extend 扩展
     * 导诊咨询常见问题匹配
     */
    public void replyGuaidenceContent(String sessionId,String type,String content,String contentType,String businessType, String extend){
        try {
            if(contentType.equals(ImUtil.ContentType.KnowSymptomsDiseaseQ.getValue())){
                //回复疾病
                replyDisease(type,content,sessionId,null,contentType);
                return;
            }
            WlyyHospitalSysDictDO dictDO = getSender();
            if (contentType.equalsIgnoreCase("3003")){
                String contentTypes = ImUtil.ContentType.KnowCommonDict.getValue();
                String contentMsg = selectByContentType(contentTypes,content).toJSONString();
                imService.patientGuaidenceAppend(dictDO.getDictCode(),dictDO.getDictValue(),sessionId,
                        contentTypes+"", contentMsg, "1",null);
            }
            String systemType = sessionTypeToType(type);
            BaseSystemDialogSetting setting = dialogSettingDao.findBySystemTypeAndFunctionType(systemType,"7",1);
            if(setting==null){
                return;
            }
            Boolean re = imUtil.sessionIsExist(sessionId);
            if(re){
                //查找相似问题
                BaseKnowledgeQuestion question = new BaseKnowledgeQuestion();
                if (contentType.equalsIgnoreCase("1")){
                    question = findQuestionLikeName(content);
                }else if ((contentType+"").equalsIgnoreCase(ImUtil.ContentType.KnowCommonQuestions.getValue()+"")){
                    question = knowledgeQuestionDao.findById(content).orElse(null);
                }
                if(question!=null&&StringUtils.isNotBlank(question.getAnswer())){
                    //匹配到相关问题后先推送答案
                    String answer = question.getAnswer();

                    imService.patientGuaidenceAppend(dictDO.getDictCode(),dictDO.getDictValue(),sessionId,
                            contentType+"", answer, "1",null);
                    //匹配问题是否关联其他配置；
                    if (question.getFlag()==null){
                        List<BaseKnowledgeFlowConfigurationRelation> flowConfigurationRelations = findFlowConfigurationRelationLikeName(content);
                        if (flowConfigurationRelations!=null&&flowConfigurationRelations.size()!=0){
                            BaseKnowledgeFlowConfigurationRelation flowConfigurationRelation = flowConfigurationRelations.get(0);
                            String contentTypes = "1";
                            String contentMsg = "";
                            if (flowConfigurationRelation.getRelationType().equalsIgnoreCase("2")){
                                contentTypes = ImUtil.ContentType.KnowCommonDict+"";
                                contentMsg = selectByContentType(contentTypes,flowConfigurationRelation.getRelationCode()).toJSONString();
                            }else  if (flowConfigurationRelation.getRelationType().equalsIgnoreCase("3")){
                                contentTypes = ImUtil.ContentType.KnowCommonQuestions+"";
                                contentMsg = selectByContentType(contentTypes,flowConfigurationRelation.getRelationCode()).toJSONString();
                            }

                            imService.patientGuaidenceAppend(dictDO.getDictCode(),dictDO.getDictValue(),sessionId,
                                    contentTypes+"", contentMsg, "1",null);
                        }
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }


    public JSONObject selectByContentType(String contentType,String id){
        JSONObject object = new JSONObject();
        if (contentType.equalsIgnoreCase(ImUtil.ContentType.KnowCommonDict+"")){
            BaseKnowledgeDict dict = knowledgeDictDao.findById(id).get();
            BaseKnowledgeDictRelation dictRelation  = knowledgeDictRelationDao.selectByDictId(id);
            if (dictRelation!=null){
                String dept = dictRelation.getDept();
            }
            List<DictHospitalDeptDO> deptDOList = findDeptList(id);
            object.put("content",deptDOList);
            object.put("contentType",contentType);

        }else if (contentType.equalsIgnoreCase(ImUtil.ContentType.KnowCommonQuestions+"")){
            BaseKnowledgeQuestions questions = knowledgeQuestionsDao.findById(id).get();
            List<BaseKnowledgeQuestionsRelation> questionsRelations = knowledgeQuestionsRelationDao.selectByQuestionsId(questions.getId());
            object.put("content",questionsRelations);
            object.put("contentType",contentType);

        }
        return object;
    }



    public void replyContent(String session_id,String type,String content,Map<String,BaseSystemDialogSetting> mapSetting,Map<String,JSONObject> map){
        try {
            BaseSystemDialogSetting setting = null;
            if(mapSetting==null){
                setting = dialogSettingDao.findBySystemTypeAndFunctionType(sessionTypeToType(type),"7",1);
            }else {
                setting = mapSetting.get("7");
            }
            if(setting==null){
                return;
            }
            //查找相似问题
            BaseKnowledgeQuestion question = findQuestionLikeName(content);
            if(question!=null){
                String answer = question.getAnswer();
                JSONObject result = new JSONObject();
                result.put("content",answer);
                result.put("content_type","1");
                result.put("session_id",session_id);
                map.put("7",result);
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    public void welcome(String session_id,Map<String,BaseSystemDialogSetting> mapSetting,Map<String,JSONObject> map){
        try {
            Integer sessionStatus = imUtil.sessionStatus(session_id);
            if(sessionStatus==null||sessionStatus==1){
                //系统客服tip
                JSONObject tip = new JSONObject();
                tip.put("content","1");
                tip.put("content_type",ImUtil.ContentType.KnowCommonCustomer.getValue());
                tip.put("session_id",session_id);
                map.put("0",tip);
                //欢迎语
                BaseSystemDialogSetting welcomeSetting = mapSetting.get("1");
                if(welcomeSetting!=null&&StringUtils.isNotBlank(welcomeSetting.getContent())){
                    String welcomeContent = welcomeSetting.getContent();
                    JSONObject result = new JSONObject();
                    result.put("content",welcomeContent);
                    result.put("content_type","1");
                    result.put("session_id",session_id);
                    map.put("1",result);
                }
                //欢迎消息关联问题
                BaseSystemDialogSetting welcomeQueSetting = mapSetting.get("2");
                if(welcomeQueSetting!=null){
                    Integer relaitonCodeType = welcomeQueSetting.getRelaitonCodeType();//1常见问题2字典中心3问题集
                    relaitonCodeType = relaitonCodeType==null?-1:relaitonCodeType;
                    String relationCode = welcomeQueSetting.getRelationCode();//relationType2对应业务code，多个逗号隔开
                    if(StringUtils.isNotBlank(relationCode)){
                        return;
                    }
                    relationCode = relationCode.replace(",","','");
                    if(relaitonCodeType==1){
                        //1常见问题
                        JSONObject result = new JSONObject();
                        result.put("content",relationCode);
                        result.put("content_type",ImUtil.ContentType.KnowCommonQuestion.getValue());
                        result.put("session_id",session_id);
                        map.put("2",result);
                    }else if(relaitonCodeType==2){
                        JSONObject result = new JSONObject();
                        result.put("content",relationCode);
                        result.put("content_type",ImUtil.ContentType.KnowCommonDict.getValue());
                        result.put("session_id",session_id);
                        map.put("2",result);
                    }else if(relaitonCodeType==3){
                        JSONObject result = new JSONObject();
                        result.put("content",relationCode);
                        result.put("content_type",ImUtil.ContentType.KnowCommonQuestions.getValue());
                        result.put("session_id",session_id);
                        map.put("2",result);
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    //查找常见问题
    public BaseKnowledgeQuestion findQuestionLikeName(String content){
        if(!StringUtils.isNotEmpty(content)){
            return null;
        }
        String sql = "select p.* from base_knowledge_question p where p.status = 1 and p.del=1 and  " +
                " (p.time=1 or (p.time=2 and p.date<'"+DateUtil.getStringDate()+"')) " +
                " and (p.question_name like '%"+content+"%' or p.similar_question_name like '"+content+"')  order by sort limit 1";
        List<BaseKnowledgeQuestion> questionList = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(BaseKnowledgeQuestion.class));
        if(questionList.size()>0){
            return questionList.get(0);
        }
        return null;
    }

    //查找常见问题关联配置
    public List<BaseKnowledgeFlowConfigurationRelation> findFlowConfigurationRelationLikeName(String content){
        if(!StringUtils.isNotEmpty(content)){
            return null;
        }
        String sql = "select p.* from base_knowledge_flow_configuration_relation p where " +
                "  p.relation_name like '%"+content+"%'  order by sort asc ";
        List<BaseKnowledgeFlowConfigurationRelation> flowConfigurationRelations = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(BaseKnowledgeFlowConfigurationRelation.class));
        return flowConfigurationRelations;
    }

    public List<BaseKnowledgeQuestions> findTopQuestionsList(Integer limit,String ids){
        String sql = "select p.* from base_knowledge_questions p where p.status = 1 and p.del=1 and p.id in ('"+ids+"')" +
                "(p.time=1 or (p.time=2 and p.date<'"+DateUtil.getStringDate()+"'))  order by sort limit "+limit;
        List<BaseKnowledgeQuestions> questionList = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(BaseKnowledgeQuestions.class));
        return questionList;
    }

    public List<BaseKnowledgeQuestion> findTopQuestionList(Integer limit,String ids){
        String sql = "select p.* from base_knowledge_question p where p.status = 1 and p.del=1 and p.id in ('"+ids+"')" +
                "(p.time=1 or (p.time=2 and p.date<'"+DateUtil.getStringDate()+"'))  order by sort limit "+limit;
        List<BaseKnowledgeQuestion> questionList = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(BaseKnowledgeQuestion.class));
        return questionList;
    }

    /**
     * 查询相关字典
     * @param limit
     * @param ids
     * @return
     */
    public List<BaseKnowledgeDict> findTopDictList(Integer limit,String ids){
        String sql = "select p.* from base_knowledge_dict p where p.del=1 and p.id in ('"+ids+"') order by sort limit "+limit;
        List<BaseKnowledgeDict> dictList = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(BaseKnowledgeDict.class));
        return dictList;
    }

    /**
     * 根据业务不一样转换相关的会话类型
     * @param type
     * @return
     */
    //会话类型转类型
    public String sessionTypeToType(String type){
        if(!StringUtils.isNotEmpty(type)){
            return type;
        }
        if("18".equalsIgnoreCase(type)){
            return "1";
        }
        if("26".equalsIgnoreCase(type)){
            return "2";
        }
        if("27".equalsIgnoreCase(type)){
            return "3";
        }
        if("1".equalsIgnoreCase(type)||"5".equalsIgnoreCase(type)||"9".equalsIgnoreCase(type)||"12".equalsIgnoreCase(type)||"15".equalsIgnoreCase(type)||"16".equalsIgnoreCase(type)){
            return "4";
        }
        return type;
    }


    public List<DictHospitalDeptDO> findDeptList(String ids){
        String sql = "select p.* from dict_hospital_dept p where p.consult_dept_flag=1 and p.code in ('"+ids+"') ";
        List<DictHospitalDeptDO> deptDOList = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(DictHospitalDeptDO.class));
        return deptDOList;
    }
}
