package com.yihu.jw.entity.base.nurse;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.yihu.jw.entity.UuidIdentityEntityWithCreateTime;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;
import javax.persistence.Transient;
import java.util.Date;

/**
 * 陪护证
 * Created by yeshijie on 2022/11/16.
 */
@Entity
@Table(name = "base_nursing_card")
public class BaseNursingCardDO extends UuidIdentityEntityWithCreateTime {

    private String inpatientRegistrationId;//住院登记id
    private String inpatientAreaId;//病区id
    private String inpatientAreaName;//病区
    private Integer type;//陪护类型 2住院
    private Integer status;//状态 -2已失效 -1未通过 0审核中 1生效中
    private String statusName;//状态 -2已失效 -1未通过 0审核中 1生效中
    private Date endTime;//到期时间
    private String patient;//居民id
    private String name;//居民姓名
    private String idcard;//居民身份证
    private String bedNo;//床位号
    private String qrCode;//二维码
    private String nursePatient;//陪护人id
    private String nurseName;//陪护人姓名
    private String nurseMobile;//陪护人手机
    private String nurseIdcard;//陪护人身份证
    private Integer nurseAge;//陪护人年龄
    private Integer nurseSex;//陪护人性别
    private String familyRelation;//家人关系 关系和原有系统的一致
    private String nurseReason;//陪护说明
    private String nucleicReport;//核酸报告
    private String doctor;//审核医生
    private String doctorName;
    private String checkReason;//审核说明/拒绝原因
    private Date checkTime;//审核时间


    private Long remainingTimes;//陪护证剩余时间（秒）
    private Integer nursingNum;//已存在陪护人数
    private Integer remainingNum;//剩余陪护名额
    private Boolean hasNucleicReport;//是否已上传核酸报告

    /**
     * 状态：
     */
    public enum Status {

        invalid(-2, "已失效"),
        unpass(-1, "未通过"),
        checking(0, "审核中"),
        inForce(1, "生效中");

        private Integer type;
        private String desc;

        Status(Integer type, String desc) {
            this.type = type;
            this.desc = desc;
        }

        public static String getStatusName(Integer type){
            String name = "";
            switch (type){
                case -2:
                    name = invalid.desc;
                    break;
                case -1:
                    name = unpass.desc;
                    break;
                case 0:
                    name = checking.desc;
                    break;
                case 1:
                    name = inForce.desc;
                    break;
            }
            return name;
        }

        public Integer getType() {
            return type;
        }

        public void setType(Integer type) {
            this.type = type;
        }
    }

    @Column(name = "inpatient_registration_id")
    public String getInpatientRegistrationId() {
        return inpatientRegistrationId;
    }

    public void setInpatientRegistrationId(String inpatientRegistrationId) {
        this.inpatientRegistrationId = inpatientRegistrationId;
    }

    @Column(name = "inpatient_area_id")
    public String getInpatientAreaId() {
        return inpatientAreaId;
    }

    public void setInpatientAreaId(String inpatientAreaId) {
        this.inpatientAreaId = inpatientAreaId;
    }

    @Column(name = "inpatient_area_name")
    public String getInpatientAreaName() {
        return inpatientAreaName;
    }

    public void setInpatientAreaName(String inpatientAreaName) {
        this.inpatientAreaName = inpatientAreaName;
    }

    @Column(name = "type")
    public Integer getType() {
        return type;
    }

    public void setType(Integer type) {
        this.type = type;
    }

    @Column(name = "status")
    public Integer getStatus() {
        return status;
    }

    public void setStatus(Integer status) {
        this.status = status;
    }

    @Column(name = "end_time")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+08:00")
    public Date getEndTime() {
        return endTime;
    }

    public void setEndTime(Date endTime) {
        this.endTime = endTime;
    }

    @Column(name = "patient")
    public String getPatient() {
        return patient;
    }

    public void setPatient(String patient) {
        this.patient = patient;
    }

    @Column(name = "name")
    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    @Column(name = "idcard")
    public String getIdcard() {
        return idcard;
    }

    public void setIdcard(String idcard) {
        this.idcard = idcard;
    }

    @Column(name = "bed_no")
    public String getBedNo() {
        return bedNo;
    }

    public void setBedNo(String bedNo) {
        this.bedNo = bedNo;
    }

    @Column(name = "qr_code")
    public String getQrCode() {
        return qrCode;
    }

    public void setQrCode(String qrCode) {
        this.qrCode = qrCode;
    }

    @Column(name = "nurse_name")
    public String getNurseName() {
        return nurseName;
    }

    public void setNurseName(String nurseName) {
        this.nurseName = nurseName;
    }

    @Column(name = "nurse_mobile")
    public String getNurseMobile() {
        return nurseMobile;
    }

    public void setNurseMobile(String nurseMobile) {
        this.nurseMobile = nurseMobile;
    }

    @Column(name = "nurse_idcard")
    public String getNurseIdcard() {
        return nurseIdcard;
    }

    public void setNurseIdcard(String nurseIdcard) {
        this.nurseIdcard = nurseIdcard;
    }

    @Column(name = "nurse_age")
    public Integer getNurseAge() {
        return nurseAge;
    }

    public void setNurseAge(Integer nurseAge) {
        this.nurseAge = nurseAge;
    }

    @Column(name = "nurse_sex")
    public Integer getNurseSex() {
        return nurseSex;
    }

    public void setNurseSex(Integer nurseSex) {
        this.nurseSex = nurseSex;
    }

    @Column(name = "nucleic_report")
    public String getNucleicReport() {
        return nucleicReport;
    }

    public void setNucleicReport(String nucleicReport) {
        this.nucleicReport = nucleicReport;
    }

    @Transient
    public String getStatusName() {
        return statusName;
    }

    public void setStatusName(String statusName) {
        this.statusName = statusName;
    }

    @Column(name = "doctor")
    public String getDoctor() {
        return doctor;
    }

    public void setDoctor(String doctor) {
        this.doctor = doctor;
    }

    @Column(name = "doctor_name")
    public String getDoctorName() {
        return doctorName;
    }

    public void setDoctorName(String doctorName) {
        this.doctorName = doctorName;
    }

    @Column(name = "check_time")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+08:00")
    public Date getCheckTime() {
        return checkTime;
    }

    public void setCheckTime(Date checkTime) {
        this.checkTime = checkTime;
    }

    @Column(name = "nurse_reason")
    public String getNurseReason() {
        return nurseReason;
    }

    public void setNurseReason(String nurseReason) {
        this.nurseReason = nurseReason;
    }

    @Column(name = "family_relation")
    public String getFamilyRelation() {
        return familyRelation;
    }

    public void setFamilyRelation(String familyRelation) {
        this.familyRelation = familyRelation;
    }

    @Column(name = "check_reason")
    public String getCheckReason() {
        return checkReason;
    }

    public void setCheckReason(String checkReason) {
        this.checkReason = checkReason;
    }

    @Column(name = "nurse_patient")
    public String getNursePatient() {
        return nursePatient;
    }

    public void setNursePatient(String nursePatient) {
        this.nursePatient = nursePatient;
    }

    @Transient
    public Integer getNursingNum() {
        return nursingNum;
    }

    public void setNursingNum(Integer nursingNum) {
        this.nursingNum = nursingNum;
    }

    @Transient
    public Long getRemainingTimes() {
        return remainingTimes;
    }

    public void setRemainingTimes(Long remainingTimes) {
        this.remainingTimes = remainingTimes;
    }

    @Transient
    public Integer getRemainingNum() {
        return remainingNum;
    }

    public void setRemainingNum(Integer remainingNum) {
        this.remainingNum = remainingNum;
    }

    @Transient
    public Boolean getHasNucleicReport() {
        return hasNucleicReport;
    }

    public void setHasNucleicReport(Boolean hasNucleicReport) {
        this.hasNucleicReport = hasNucleicReport;
    }
}
