package com.yihu.jw.entity.door;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.yihu.jw.entity.UuidIdentityEntityWithOperator;
import com.yihu.jw.entity.followup.Followup;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionDO;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;
import javax.persistence.Transient;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;


/**
 * 上门服务工单实体
 *
 * @author Administrator on  2019年03月19日
 *
 */
@Entity
@Table(name = "wlyy_door_service_order")
public class WlyyDoorServiceOrderDO extends UuidIdentityEntityWithOperator {



    /**
     * 工单状态
     */
    public enum Status {

        cancel(-1, "已取消"),
        waitForSend(1, "待（调度员）派单"),
        waitForAccept(2, "待（医生）接单"),
        accept(3, "已接单"),
        waitForServe(4, "待服务"),
        waitForCommnet(5, "待评价"),
        complete(6,"已完成");

        private Integer type;
        private String desc;

        Status(Integer type, String desc) {
            this.type = type;
            this.desc = desc;
        }

        public Integer getType() {
            return type;
        }

        public void setType(Integer type) {
            this.type = type;
        }

    }

    /**
     * 支付方式
     */
    public enum PayWay {

        wechat(1, "微信支付"),
        offLine(2,"线下支付");

        private Integer type;
        private String desc;

        PayWay(Integer type, String desc) {
            this.type = type;
            this.desc = desc;
        }

        public Integer getType() {
            return type;
        }

        public void setType(Integer type) {
            this.type = type;
        }

    }

    /**
     * 取消类型
     */
    public enum CancelType {

        dispatcher(1, "调度员取消"),
        patient(2,"居民取消"),
        doctor(3, "医生取消");

        private Integer type;
        private String desc;

        CancelType(Integer type, String desc) {
            this.type = type;
            this.desc = desc;
        }

        public Integer getType() {
            return type;
        }

        public void setType(Integer type) {
            this.type = type;
        }

    }


    /**
     * 化验检查报告补录方式
     */
    public enum ExamPaperUploadWay {

        photo(1, "拍照补录"),
        interfaceData(2,"接口数据");

        private Integer type;
        private String desc;

        ExamPaperUploadWay(Integer type, String desc) {
            this.type = type;
            this.desc = desc;
        }

        public Integer getType() {
            return type;
        }

        public void setType(Integer type) {
            this.type = type;
        }

    }

    /**
     * 医生签到方式
     */
    public enum DoctorSignWay {

        locate(1, "定位"),
        sacn(2,"扫码");

        private Integer type;
        private String desc;

        DoctorSignWay(Integer type, String desc) {
            this.type = type;
            this.desc = desc;
        }

        public Integer getType() {
            return type;
        }

        public void setType(Integer type) {
            this.type = type;
        }

    }

    /**
     * 居民对工单需要服务情况是否已确认
     */
    public enum IsPatientConfirm {

        no(0, "未确认"),
        yes(1,"已确认");

        private Integer type;
        private String desc;

        IsPatientConfirm(Integer type, String desc) {
            this.type = type;
            this.desc = desc;
        }

        public Integer getType() {
            return type;
        }

        public void setType(Integer type) {
            this.type = type;
        }

    }  /**
     * 居民对工单需要服务情况是否已确认
     */
    public enum IsTransOtherOrg {

        no(0, "不转"),
        yes(1,"已转");

        private Integer type;
        private String desc;

        IsTransOtherOrg(Integer type, String desc) {
            this.type = type;
            this.desc = desc;
        }

        public Integer getType() {
            return type;
        }

        public void setType(Integer type) {
            this.type = type;
        }

    }


    private String packageId;//服务包的id
    private String feeAfter;//服务后收费标识（1是0否）
    private String signId;//签约id


    /**
     * 服务编号
     */
    private String number;

    /**
     * 代理发起工单的居民code，替父母，孩子等发起工单
     */
    private String proxyPatient;

    /**
     * 代理发起工单的居民code，替父母，孩子等发起工单
     */
    private String proxyPatientName;

    /**
     * 代理发起工单的居民联系电话
     */
    private String proxyPatientPhone;

    /**
     * 被服务的居民code，发起工单的居民的亲属
     */
    private String patient;

    /**
     * 被服务的居民姓名，发起工单的居民的亲属
     */
    private String patientName;

    /**
     * 被服务的居民联系电话
     */
    private String patientPhone;

    /**
     * 发起人与被服务人的关系：自己，父亲，母亲，儿子等
     */
    private String patientRelation;

    /**
     * 调度员code
     */
    private String dispatcher;

    /**
     * 调度员name
     */
    private String dispatcherName;

    /**
     * 居民期望服务时间
     */
//	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+08:00")
    private String patientExpectedServeTime;

    /**
     * 居民自己服务描述
     */
    private String serveDesc;

    /**
     * 上门服务的区
     */
    private String serveTown;

    /**
     * 上门服务详细地址
     */
    private String serveAddress;

    /**
     * 上门服务地址纬度
     */
    private String serveLat;

    /**
     * 上门服务地址经度
     */
    private String serveLon;

    /**
     * 调度员备注
     */
    private String remark;

    /**
     * 居民对工单需要服务情况是否已确认，0-未确认，1-已确认, 2-不同意
     */
    private Integer isPatientConfirm;

    /**
     * 居民对工单需要服务情况确认时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+08:00")
    private Date patientConfirmTime;

    /**
     * 工单是否转给其他机构，0-不转，1-已转
     */
    private Integer isTransOtherOrg;

    /**
     * 最后转接的机构code
     */
    private String transedOrgCode;

    /**
     * 最后转接的机构调度员code
     */
    private String transedDispatcher;

    /**
     * 最后转接的机构调度员name
     */
    private String transedDispatcherName;

    /**
     * 服务总费用
     */
    private BigDecimal totalFee;

    /**
     * 居民期望服务的医生姓名
     */
    private String expectedDoctorName;

    /**
     * 接单的医生code
     */
    private String doctor;

    /**
     * 接单的医生name
     */
    private String doctorName;

    /**
     * 接单的医生类型：医生，健管师，护士等
     */
    private String doctorType;

    /**
     * 接单的护理人员code
     */
    private String nursingStaff;

    /**
     * 接单的护理人员姓名
     */
    private String nursingStaffName;

    /**
     * 接单的护理人员类型：医生，健管师，护士等
     */
    private String nursingStaffType;

    /**
     * 医生预计到达时间
     */
    private String doctorArrivingTime;

    /**
     * 医生签到时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+08:00")
    private Date doctorSignTime;

    /**
     * 医生签到方式：1-定位，2-扫码
     */
    private Integer doctorSignWay;

    /**
     * 医生签到位置，记录详细地址
     */
    private String doctorSignLocation;

    /**
     * 医生签到照片
     */
    private String doctorSignImg;

    /**
     * 居民确认结束服务方式：1-电子签名，2-手持身份证拍照
     */
    private Integer patientConfirmFinishWay;

    /**
     * 居民确认结束服务照片
     */
    private String patientConfirmFinishImg;

    /**
     * 居民确认医生结束服务时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+08:00")
    private Date patientConfirmFinishTime;

    /**
     * 医生诊疗现场照片，最多9张，逗号分隔
     */
    private String presentImgs;

    /**
     * 是否需要上传补录报告：0-不需要，1-需要，待补录；2-需要，已补录
     */
    private Integer examPaperStatus;

    /**
     * 医生上传居民的化验检查报告照片
     */
    private String examPaperImgs;

    /**
     * 化验检查报告补录时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+08:00")
    private Date examPaperUploadTime;

    /**
     * 化验检查报告补录方式，1-拍照补录，2-接口数据
     */
    private Integer examPaperUploadWay;

    /**
     * 工单状态：-1-已取消，1-待（调度员）派单，2-待（医生）接单，3-待服务，4-待付款，5-待评价，6-已完成
     */
    private Integer status;

    /**
     * 工单完成时间（对工单评价完即工单完成）
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+08:00")
    private Date completeTime;

    /**
     * 取消类型：1-调度员取消，2-居民取消
     */
    private Integer cancelType;

    /**
     * 取消理由
     */
    private String cancelReason;

    /**
     * 取消时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+08:00")
    private Date cancelTime;

    /**
     * 付款方式：1-微信支付，2-线下支付（居民自己向医院支付，具体怎么支付由医院来定）
     */
    private Integer payWay;

    /**
     * 支付流水号
     */
    private String payNumber;

    /**
     * 支付时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+08:00")
    private Date payTime;

    /**
     * 工单对应的服务项
     */
    private List<Map<String, Object>> doorFeeDetailList;

    /**
     * 工单对应的出诊费
     */
    private List<WlyyDoorDoctorDO> djDetailList;

    /**
     * 工单服务医生
     */
    private List<Map<String, Object>> doctors;

    /**
     * 服务小结
     */
    private WlyyDoorConclusionDO doorConclusion;


    /**
     * 调度员响应时间（派单时间或取消时间）
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+08:00")
    private Date dispatcherResponseTime;

    /**
     * 服务医生响应时间（第一条咨询或者接单时间）
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+08:00")
    private Date serviceResponseTime;

    /**
     * 出诊费用
     */
    private List<Map<String, Object>> visitCost;

    /**
     * 服务费用 服务包支付、待支付
     */
    private Map<String, Object> serviceCost;

    /**
     * 患者性别
     */
    private String sex;

    /**
     * 患者年龄
     */
    private Integer age;

    /**
     * 患者头像
     */
    private String photo;

    /**
     * 人群类型名称
     */
    private String typeValue;

    /**
     * 会话id
     */
    private String sessionId;

    /**
     * 服务机构
     */
    private String hospital;
    private Integer conclusionStatus;//服务小结登记状态：1待补录；2-已补录
    private Integer prescriptionStatus;//开方状态：1开方完成，0未开方
    private String prescriptionCode;//处方单号,多个用逗号隔开
    private String outpatientId;//复诊id
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+08:00")
    private Date prescriptionTime;//开方完成时间
    private Integer type;//发起工单类型（1本人发起 2家人待预约 3医生代预约）
    private String authorizeImage;//电子健康卡授权图片
    private String relationCode;//业务关联
    private String serviceStatus;//服务类型 1-预约项目 2-即时项目
    private String orderInfo;//工单详情 0-未推送 1-未确认 2-已确认

//    private List<WlyyPrescriptionDO> prescriptionDOList;//上门前开的方
//    private List<WlyyPrescriptionDO> afterPrescriptionList;//上门后开的方

    private List<WlyyDoorPrescriptionDO> prescriptionDOList;//上门前开的方
    private List<WlyyDoorPrescriptionDO> afterPrescriptionList;//上门后开的方

    private List<Followup> followupList;//上门随访
    private String followupDate;//随访时间

    /**
     * 快捷类型，1是快捷类型，其他值不是
     */
    private String shortcutType;

    @Transient
    public List<WlyyDoorPrescriptionDO> getPrescriptionDOList() {
        return prescriptionDOList;
    }

    public void setPrescriptionDOList(List<WlyyDoorPrescriptionDO> prescriptionDOList) {
        this.prescriptionDOList = prescriptionDOList;
    }

    @Transient
    public List<WlyyDoorPrescriptionDO> getAfterPrescriptionList() {
        return afterPrescriptionList;
    }

    public void setAfterPrescriptionList(List<WlyyDoorPrescriptionDO> afterPrescriptionList) {
        this.afterPrescriptionList = afterPrescriptionList;
    }

    @Column(name = "number")
    public String getNumber() {
        return number;
    }
    public void setNumber(String number) {
        this.number = number;
    }

    @Column(name = "proxy_patient")
    public String getProxyPatient() {
        return proxyPatient;
    }
    public void setProxyPatient(String proxyPatient) {
        this.proxyPatient = proxyPatient;
    }

    @Column(name = "proxy_patient_name")
    public String getProxyPatientName() {
        return proxyPatientName;
    }
    public void setProxyPatientName(String proxyPatientName) {
        this.proxyPatientName = proxyPatientName;
    }

    @Column(name = "proxy_patient_phone")
    public String getProxyPatientPhone() {
        return proxyPatientPhone;
    }
    public void setProxyPatientPhone(String proxyPatientPhone) {
        this.proxyPatientPhone = proxyPatientPhone;
    }

    @Column(name = "patient")
    public String getPatient() {
        return patient;
    }
    public void setPatient(String patient) {
        this.patient = patient;
    }

    @Column(name = "patient_name")
    public String getPatientName() {
        return patientName;
    }
    public void setPatientName(String patientName) {
        this.patientName = patientName;
    }

    @Column(name = "patient_phone")
    public String getPatientPhone() {
        return patientPhone;
    }
    public void setPatientPhone(String patientPhone) {
        this.patientPhone = patientPhone;
    }

    @Column(name = "patient_relation")
    public String getPatientRelation() {
        return patientRelation;
    }
    public void setPatientRelation(String patientRelation) {
        this.patientRelation = patientRelation;
    }

    @Column(name = "dispatcher")
    public String getDispatcher() {
        return dispatcher;
    }
    public void setDispatcher(String dispatcher) {
        this.dispatcher = dispatcher;
    }

    @Column(name = "dispatcher_name")
    public String getDispatcherName() {
        return dispatcherName;
    }
    public void setDispatcherName(String dispatcherName) {
        this.dispatcherName = dispatcherName;
    }

    @Column(name = "patient_expected_serve_time")
    public String getPatientExpectedServeTime() {
        return patientExpectedServeTime;
    }
    public void setPatientExpectedServeTime(String patientExpectedServeTime) {
        this.patientExpectedServeTime = patientExpectedServeTime;
    }

    @Column(name = "serve_desc")
    public String getServeDesc() {
        return serveDesc;
    }
    public void setServeDesc(String serveDesc) {
        this.serveDesc = serveDesc;
    }

    @Column(name = "serve_town")
    public String getServeTown() {
        return serveTown;
    }
    public void setServeTown(String serveTown) {
        this.serveTown = serveTown;
    }

    @Column(name = "serve_address")
    public String getServeAddress() {
        return serveAddress;
    }
    public void setServeAddress(String serveAddress) {
        this.serveAddress = serveAddress;
    }

    @Column(name = "serve_lat")
    public String getServeLat() {
        return serveLat;
    }
    public void setServeLat(String serveLat) {
        this.serveLat = serveLat;
    }

    @Column(name = "serve_lon")
    public String getServeLon() {
        return serveLon;
    }
    public void setServeLon(String serveLon) {
        this.serveLon = serveLon;
    }

    @Column(name = "remark")
    public String getRemark() {
        return remark;
    }
    public void setRemark(String remark) {
        this.remark = remark;
    }

    @Column(name = "is_patient_confirm")
    public Integer getIsPatientConfirm() {
        return isPatientConfirm;
    }
    public void setIsPatientConfirm(Integer isPatientConfirm) {
        this.isPatientConfirm = isPatientConfirm;
    }

    @Column(name = "patient_confirm_time")
    public Date getPatientConfirmTime() {
        return patientConfirmTime;
    }
    public void setPatientConfirmTime(Date patientConfirmTime) {
        this.patientConfirmTime = patientConfirmTime;
    }

    @Column(name = "is_trans_other_org")
    public Integer getIsTransOtherOrg() {
        return isTransOtherOrg;
    }
    public void setIsTransOtherOrg(Integer isTransOtherOrg) {
        this.isTransOtherOrg = isTransOtherOrg;
    }

    @Column(name = "transed_org_code")
    public String getTransedOrgCode() {
        return transedOrgCode;
    }

    @Column(name = "expected_doctor_name")
    public String getExpectedDoctorName() {
        return expectedDoctorName;
    }

    public void setExpectedDoctorName(String expectedDoctorName) {
        this.expectedDoctorName = expectedDoctorName;
    }

    public void setTransedOrgCode(String transedOrgCode) {
        this.transedOrgCode = transedOrgCode;
    }

    @Column(name = "transed_dispatcher")
    public String getTransedDispatcher() {
        return transedDispatcher;
    }
    public void setTransedDispatcher(String transedDispatcher) {
        this.transedDispatcher = transedDispatcher;
    }

    @Column(name = "transed_dispatcher_name")
    public String getTransedDispatcherName() {
        return transedDispatcherName;
    }
    public void setTransedDispatcherName(String transedDispatcherName) {
        this.transedDispatcherName = transedDispatcherName;
    }

    @Column(name = "total_fee")
    public BigDecimal getTotalFee() {
        return totalFee;
    }
    public void setTotalFee(BigDecimal totalFee) {
        this.totalFee = totalFee;
    }



    @Column(name = "doctor")
    public String getDoctor() {
        return doctor;
    }
    public void setDoctor(String doctor) {
        this.doctor = doctor;
    }

    @Column(name = "doctor_name")
    public String getDoctorName() {
        return doctorName;
    }
    public void setDoctorName(String doctorName) {
        this.doctorName = doctorName;
    }

    @Column(name = "doctor_type")
    public String getDoctorType() {
        return doctorType;
    }
    public void setDoctorType(String doctorType) {
        this.doctorType = doctorType;
    }

    @Column(name = "doctor_arriving_time")
    public String getDoctorArrivingTime() {
        return doctorArrivingTime;
    }
    public void setDoctorArrivingTime(String doctorArrivingTime) {
        this.doctorArrivingTime = doctorArrivingTime;
    }

    @Column(name = "doctor_sign_time")
    public Date getDoctorSignTime() {
        return doctorSignTime;
    }
    public void setDoctorSignTime(Date doctorSignTime) {
        this.doctorSignTime = doctorSignTime;
    }

    @Column(name = "doctor_sign_way")
    public Integer getDoctorSignWay() {
        return doctorSignWay;
    }
    public void setDoctorSignWay(Integer doctorSignWay) {
        this.doctorSignWay = doctorSignWay;
    }

    @Column(name = "doctor_sign_location")
    public String getDoctorSignLocation() {
        return doctorSignLocation;
    }
    public void setDoctorSignLocation(String doctorSignLocation) {
        this.doctorSignLocation = doctorSignLocation;
    }

    @Column(name = "doctor_sign_img")
    public String getDoctorSignImg() {
        return doctorSignImg;
    }
    public void setDoctorSignImg(String doctorSignImg) {
        this.doctorSignImg = doctorSignImg;
    }

    @Column(name = "patient_confirm_finish_way")
    public Integer getPatientConfirmFinishWay() {
        return patientConfirmFinishWay;
    }
    public void setPatientConfirmFinishWay(Integer patientConfirmFinishWay) {
        this.patientConfirmFinishWay = patientConfirmFinishWay;
    }

    @Column(name = "patient_confirm_finish_img")
    public String getPatientConfirmFinishImg() {
        return patientConfirmFinishImg;
    }
    public void setPatientConfirmFinishImg(String patientConfirmFinishImg) {
        this.patientConfirmFinishImg = patientConfirmFinishImg;
    }

    @Column(name = "patient_confirm_finish_time")
    public Date getPatientConfirmFinishTime() {
        return patientConfirmFinishTime;
    }
    public void setPatientConfirmFinishTime(Date patientConfirmFinishTime) {
        this.patientConfirmFinishTime = patientConfirmFinishTime;
    }

    @Column(name = "present_imgs")
    public String getPresentImgs() {
        return presentImgs;
    }
    public void setPresentImgs(String presentImgs) {
        this.presentImgs = presentImgs;
    }

    @Column(name = "exam_paper_status")
    public Integer getExamPaperStatus() {
        return examPaperStatus;
    }
    public void setExamPaperStatus(Integer examPaperStatus) {
        this.examPaperStatus = examPaperStatus;
    }

    @Column(name = "exam_paper_imgs")
    public String getExamPaperImgs() {
        return examPaperImgs;
    }
    public void setExamPaperImgs(String examPaperImgs) {
        this.examPaperImgs = examPaperImgs;
    }

    @Column(name = "exam_paper_upload_time")
    public Date getExamPaperUploadTime() {
        return examPaperUploadTime;
    }
    public void setExamPaperUploadTime(Date examPaperUploadTime) {
        this.examPaperUploadTime = examPaperUploadTime;
    }

    @Column(name = "exam_paper_upload_way")
    public Integer getExamPaperUploadWay() {
        return examPaperUploadWay;
    }
    public void setExamPaperUploadWay(Integer examPaperUploadWay) {
        this.examPaperUploadWay = examPaperUploadWay;
    }

    @Column(name = "status")
    public Integer getStatus() {
        return status;
    }
    public void setStatus(Integer status) {
        this.status = status;
    }

    @Column(name = "complete_time")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+08:00")
    public Date getCompleteTime() {
        return completeTime;
    }
    public void setCompleteTime(Date completeTime) {
        this.completeTime = completeTime;
    }

    @Column(name = "cancel_type")
    public Integer getCancelType() {
        return cancelType;
    }
    public void setCancelType(Integer cancelType) {
        this.cancelType = cancelType;
    }

    @Column(name = "cancel_reason")
    public String getCancelReason() {
        return cancelReason;
    }
    public void setCancelReason(String cancelReason) {
        this.cancelReason = cancelReason;
    }

    @Column(name = "cancel_time")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+08:00")
    public Date getCancelTime() {
        return cancelTime;
    }
    public void setCancelTime(Date cancelTime) {
        this.cancelTime = cancelTime;
    }

    @Column(name = "pay_way")
    public Integer getPayWay() {
        return payWay;
    }
    public void setPayWay(Integer payWay) {
        this.payWay = payWay;
    }

    @Column(name = "pay_number")
    public String getPayNumber() {
        return payNumber;
    }
    public void setPayNumber(String payNumber) {
        this.payNumber = payNumber;
    }

    @Column(name = "pay_time")
    public Date getPayTime() {
        return payTime;
    }
    public void setPayTime(Date payTime) {
        this.payTime = payTime;
    }

    @Column(name = "hospital")
    public String getHospital() {
        return hospital;
    }

    public void setHospital(String hospital) {
        this.hospital = hospital;
    }

    @Transient
    public List<Map<String, Object>> getDoorFeeDetailList() {
        return doorFeeDetailList;
    }

    public void setDoorFeeDetailList(List<Map<String, Object>> doorFeeDetailList) {
        this.doorFeeDetailList = doorFeeDetailList;
    }

    @Transient
    public List<WlyyDoorDoctorDO> getDjDetailList() {
        return djDetailList;
    }

    public void setDjDetailList(List<WlyyDoorDoctorDO> djDetailList) {
        this.djDetailList = djDetailList;
    }

    @Transient
    public List<Map<String, Object>> getDoctors() {
        return doctors;
    }

    public void setDoctors(List<Map<String, Object>> doctors) {
        this.doctors = doctors;
    }

    @Transient
    public WlyyDoorConclusionDO getDoorConclusion() {
        return doorConclusion;
    }

    public void setDoorConclusion(WlyyDoorConclusionDO doorConclusion) {
        this.doorConclusion = doorConclusion;
    }

    @Column(name = "dispatcher_response_time")
    public Date getDispatcherResponseTime() {
        return dispatcherResponseTime;
    }

    public void setDispatcherResponseTime(Date dispatcherResponseTime) {
        this.dispatcherResponseTime = dispatcherResponseTime;
    }

    @Column(name = "service_response_time")
    public Date getServiceResponseTime() {
        return serviceResponseTime;
    }

    public void setServiceResponseTime(Date serviceResponseTime) {
        this.serviceResponseTime = serviceResponseTime;
    }

    @Transient
    public List<Map<String, Object>> getVisitCost() {
        return visitCost;
    }

    public void setVisitCost(List<Map<String, Object>> visitCost) {
        this.visitCost = visitCost;
    }

    @Transient
    public Map<String, Object> getServiceCost() {
        return serviceCost;
    }

    public void setServiceCost(Map<String, Object> serviceCost) {
        this.serviceCost = serviceCost;
    }

    @Transient
    public String getSex() {
        return sex;
    }

    public void setSex(String sex) {
        this.sex = sex;
    }

    @Transient
    public Integer getAge() {
        return age;
    }

    public void setAge(Integer age) {
        this.age = age;
    }

    @Transient
    public String getPhoto() {
        return photo;
    }

    public void setPhoto(String photo) {
        this.photo = photo;
    }

    @Transient
    public String getTypeValue() {
        return typeValue;
    }

    public void setTypeValue(String typeValue) {
        this.typeValue = typeValue;
    }

    @Transient
    public String getSessionId() {
        return sessionId;
    }

    public void setSessionId(String sessionId) {
        this.sessionId = sessionId;
    }

    @Transient
    public List<Followup> getFollowupList() {
        return followupList;
    }

    public void setFollowupList(List<Followup> followupList) {
        this.followupList = followupList;
    }

    public Integer getConclusionStatus() {
        return conclusionStatus;
    }

    public void setConclusionStatus(Integer conclusionStatus) {
        this.conclusionStatus = conclusionStatus;
    }

    public Integer getPrescriptionStatus() {
        return prescriptionStatus;
    }

    public void setPrescriptionStatus(Integer prescriptionStatus) {
        this.prescriptionStatus = prescriptionStatus;
    }

    public String getPrescriptionCode() {
        return prescriptionCode;
    }

    public void setPrescriptionCode(String prescriptionCode) {
        this.prescriptionCode = prescriptionCode;
    }

    public Date getPrescriptionTime() {
        return prescriptionTime;
    }

    public void setPrescriptionTime(Date prescriptionTime) {
        this.prescriptionTime = prescriptionTime;
    }

    public Integer getType() {
        return type;
    }

    public void setType(Integer type) {
        this.type = type;
    }

    public String getAuthorizeImage() {
        return authorizeImage;
    }

    public void setAuthorizeImage(String authorizeImage) {
        this.authorizeImage = authorizeImage;
    }

    @Column(name = "relation_code")
    public String getRelationCode() {
        return relationCode;
    }

    public void setRelationCode(String relationCode) {
        this.relationCode = relationCode;
    }

    @Column(name = "service_status")
    public String getServiceStatus() {
        return serviceStatus;
    }

    public void setServiceStatus(String serviceStatus) {
        this.serviceStatus = serviceStatus;
    }

    @Column(name = "order_info")
    public String getOrderInfo() {
        return orderInfo;
    }

    public void setOrderInfo(String orderInfo) {
        this.orderInfo = orderInfo;
    }

    @Column(name = "shortcut_type")
    public String getShortcutType() {
        return shortcutType;
    }

    public void setShortcutType(String shortcutType) {
        this.shortcutType = shortcutType;
    }

    @Column(name = "outpatient_id")
    public String getOutpatientId() {
        return outpatientId;
    }

    public void setOutpatientId(String outpatientId) {
        this.outpatientId = outpatientId;
    }

    @Column(name = "followup_date")
    public String getFollowupDate() {
        return followupDate;
    }

    public void setFollowupDate(String followupDate) {
        this.followupDate = followupDate;
    }

    @Column(name = "nursing_staff")
    public String getNursingStaff() {
        return nursingStaff;
    }

    public void setNursingStaff(String nursingStaff) {
        this.nursingStaff = nursingStaff;
    }

    @Column(name = "nursing_staff_name")
    public String getNursingStaffName() {
        return nursingStaffName;
    }

    public void setNursingStaffName(String nursingStaffName) {
        this.nursingStaffName = nursingStaffName;
    }

    @Column(name = "nursing_staff_type")
    public String getNursingStaffType() {
        return nursingStaffType;
    }

    public void setNursingStaffType(String nursingStaffType) {
        this.nursingStaffType = nursingStaffType;
    }

    public String getPackageId() {
        return packageId;
    }

    public void setPackageId(String packageId) {
        this.packageId = packageId;
    }

    public String getFeeAfter() {
        return feeAfter;
    }

    public void setFeeAfter(String feeAfter) {
        this.feeAfter = feeAfter;
    }

    public String getSignId() {
        return signId;
    }

    public void setSignId(String signId) {
        this.signId = signId;
    }
}