package com.yihu.jw.util.date;

import org.apache.commons.lang3.StringUtils;

import java.text.ParseException;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.util.Date;

/**
 * @author Sand -> ysj
 * @version 1.0 -> 2.0
 * @created 2015.08.26 8:49 -> 2022-09-06
 */
public class DateTimeUtil {
    public static final String simpleDateTimePattern = "yyyy-MM-dd HH:mm:ss";
    public static final String utcDateTimePattern = "yyyy-MM-dd'T'HH:mm:ss'Z'";
    public static final String slashDateTimePattern = "yyyy/MM/dd HH:mm:ss";
    public static final String simpleDatePattern = "yyyy-MM-dd";
    public static final String simpleDateTimeShortPattern = "yyyy/MM/dd HH:mm";

    public static final String ISO8601Pattern = "yyyy-MM-dd'T'HH:mm:ss'Z'ZZ";


    public static DateTimeFormatter simpleDateTimeShortFormat = DateTimeFormatter.ofPattern(simpleDateTimeShortPattern);
    public static DateTimeFormatter simpleDateTimeFormat = DateTimeFormatter.ofPattern(simpleDateTimePattern);
    public static DateTimeFormatter utcDateTimeFormat = DateTimeFormatter.ofPattern(utcDateTimePattern);
    public static DateTimeFormatter slashDateTimeFormat = DateTimeFormatter.ofPattern(slashDateTimePattern);
    public static DateTimeFormatter simpleDateFormat = DateTimeFormatter.ofPattern(simpleDatePattern);
    public static DateTimeFormatter iso8601Format = DateTimeFormatter.ofPattern(ISO8601Pattern);

    public static String simpleDateTimeFormat(Date date){
        return simpleDateTimeFormat.format(DateUtil.dateToLocalDateTime(date));
    }

    public static Date simpleDateTimeParse(String date) throws ParseException {
        if(StringUtils.isEmpty(date)) return null;

        return DateUtil.strToDateLong(date);
    }

    public static String iso8601DateTimeFormat(Date date){
        ZonedDateTime zonedDateTime = date.toInstant()
                .atZone(ZoneId.ofOffset("UTC", ZoneOffset.ofHours(+8)));
        return iso8601Format.format(zonedDateTime);
    }

    public static Date strToDate(String date,DateTimeFormatter dateTimeFormatter){
        LocalDateTime localDateTime = LocalDateTime.parse(date,dateTimeFormatter);
        return DateUtil.localTimeToDate(localDateTime);
    }

    public static Date iso8601DateTimeParse(String date){
        if(StringUtils.isEmpty(date)) return null;
        ZonedDateTime zonedDateTime = ZonedDateTime.parse(date,iso8601Format);

        return Date.from(zonedDateTime.toInstant());
    }

    public static String utcDateTimeFormat(Date date){
        //为空判断
        if(date!=null) {
            return utcDateTimeFormat.format(DateUtil.dateToLocalDateTime(date));
        }
        else{
            return null;
        }
    }

    public static Date utcDateTimeParse(String date){
        if(StringUtils.isEmpty(date)) return null;

        return strToDate(date,utcDateTimeFormat);
    }

    public static String slashDateTimeFormat(Date date){
        return slashDateTimeFormat.format(DateUtil.dateToLocalDateTime(date));
    }

    public static Date slashDateTimeParse(String date){
        return strToDate(date,slashDateTimeFormat);
    }

    public static String simpleDateFormat(Date date){
        return simpleDateFormat.format(DateUtil.dateToLocalDateTime(date));
    }

    public static Date simpleDateParse(String date) throws ParseException {
        if(StringUtils.isEmpty(date)) return null;

        return strToDate(date,simpleDateFormat);
    }

    public static String simpleDateTimeShortFormat(Date date){
        return simpleDateTimeShortFormat.format(DateUtil.dateToLocalDateTime(date));
    }

    public static Date simpleDateTimeShortParse(String date) throws ParseException {
        return strToDate(date,simpleDateTimeShortFormat);
    }

}
