package com.yihu.jw.util.date;

import org.apache.commons.collections.map.HashedMap;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.sql.Time;
import java.sql.Timestamp;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalAdjusters;
import java.time.temporal.WeekFields;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * jdk1.8 时间工具类
 * LocalDate 重写tostring 格式 2022-09-05
 * LocalTime 重写tostring 格式 09:46:24.991
 * LocalDateTime 重写tostring 格式 2022-09-05T09:46:24.991
 *
 * Created by yeshijie on 2022/9/5.
 */
public class DateUtil {

    public static final String DEFAULT_DATE_YEAR_FORMAT = "yyyy";
    public static final String DEFAULT_DATE_MONTH_FORMAT = "MM";
    public static final String PRINT_DATE_YM_FORMAT = "MMM., yyyy";
    public static final String PRINT_DATE_YMD_FORMAT = "MMM. d, yyyy";
    public static final String DEFAULT_YEARS = "0.0";
    public static final String DEFAULT_DATE_YMD_FORMAT = "yyyy-MM-dd";
    public static final String DEFAULT_DATE_YM_FORMAT = "yyyyMM";
    public static final String DEFAULT_DATE_MD_FORMAT = "MMdd";
    public static final String DEFAULT_CHAR_DATE_YMD_FORMAT = "yyyyMMdd";
    public static final String DEFAULT_TIMESTAMP_FORMAT = "yyyy-MM-dd HH:mm:ss.SSS";
    public static final String DEFAULT_SIMPLEDATE_FORMAT = "yyyy-MM-dd HH:mm:ss SSS";
    public static final String DEFAULT_YMDHMSDATE_FORMAT = "yyyy-MM-dd HH:mm:ss";
    public static final String DEFAULT_TIME_FORMAT = "HH:mm:ss";
    public static final String DEFAULT_NOW_STRING_FORMAT = "yyyyMMddHHmmssSSS";
    public static final String DATE_MDY_FORMAT = "MMddyyyy";
    public static final String DATE_MY_FORMAT = "MMyyyy";
    public static final String utcDateTimePattern = "yyyy-MM-dd'T'HH:mm:ss'Z'";
    public static final String utcDateTimePatternTZ = " yyyy-MM-dd'T'HH:mm:ssZZZ";
    public static final String DATE_WORLD_FORMAT = "yyyy-MM-dd'T'HH:mm:ss'Z'";

    public static final String HH_MM = "HH:mm";
    public static final String HH_MM_SS = "HH:mm:ss";
    public static final String YY = "yy";
    public static final String YYYYMM = "yyyyMM";
    public static final String YYYYMMDD = "yyyyMMdd";
    public static final String YYYY_MM_DD = "yyyy-MM-dd";
    public static final String YYYYMMDDHHMMSS = "yyyyMMddHHmmss";
    public static final String YYYY_MM_DD_HH = "yyyy-MM-dd HH";
    public static final String YYYY_MM_DD_HH_MM = "yyyy-MM-dd HH:mm";
    public static final String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";
    public static final String YYYY_M_D_HH_MM_SS = "yyyy/M/d HH:mm:ss";
    public static final String YYYY_MM_DD_ = "yyyy/MM/dd";
    public static final String YYYY_MM_DD_HH_MM_SS_ = "yyyy/MM/dd HH:mm:ss";
    public static final String YYYYMMddHHmmssSSS  = "yyyyMMddHHmmssSSS";
    public static final String YYYY_MM ="yyyy-MM";
    public static final String YYYYMMDD_HH_MM_SS = "yyyyMMdd HH:mm:ss";

    public static DateTimeFormatter dateFormatter = DateTimeFormatter.ofPattern(YYYY_MM_DD);
    public static DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern(YYYY_MM_DD_HH_MM_SS);// 24小时
    public static Long eightHoursMillis = TimeUnit.HOURS.toMillis(8);//8小时的毫秒数

    public static ZoneId zone = ZoneId.systemDefault();

    //取当前日期
    public static LocalDate getDateNow(){
        return LocalDate.now();
    }

    //取当前日期+时间
    public static LocalDateTime getDateTimeNow(){
        return LocalDateTime.now();
    }

    //取当前日时间
    public static LocalTime getTimeNow(){
        return LocalTime.now();
    }

    public static Date localTimeToDate(LocalDateTime localDateTime){
        return Date.from(localDateTime.atZone(zone).toInstant());
    }
    public static String localTimeToLong(LocalDateTime localDateTime){
        return dateTimeFormatter.format(localDateTime);
    }

    /**
     * 时间格式转中文格式
     */
    public static String dateToChinese(Date date) {
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy年MM月dd日 EEEEa hh:mm");
        return dateTimeFormatter.format(dateToLocalDateTime(date));
    }

    public static String dateToChineseDate(Date date) {
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy年MM月dd日");
        return dateTimeFormatter.format(dateToLocalDate(date));
    }

    public static String dateToChineseTime(Date date) {
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy年MM月dd日 HH:mm:ss");
        return dateTimeFormatter.format(dateToLocalDateTime(date));
    }

    public static String dateToChineseTime2(Date date) {
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy年MM月dd日 hh:mm");
        return dateTimeFormatter.format(dateToLocalDateTime(date));
    }

    public static Date getNight23(){
        Instant instant = LocalDateTime.now().withHour(22).atZone(zone).toInstant();
        return Date.from(instant);
    }

    public static Date getToday(){
        Instant instant = LocalDate.now().atStartOfDay(zone).toInstant();
        return Date.from(instant);
    }

    public static String getYearFromYMD(Date ymd) {
        return getYearFromYMD(toString(ymd));
    }

    public static String getYearFromYMD(String ymd) {
        return ymd != null && ymd.length() == "yyyy-MM-dd".length() ? ymd.substring(0, 4) : null;
    }

    public static String getCurrentString(String pattern) {
        return DateTimeFormatter.ofPattern(pattern).format(dateToLocalDateTime(new Date()));
    }

    public static String utcToDate(Date date) {
        return DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss'Z'").format(dateToLocalDateTime(date));
    }

    /**
     * 字符串转时间格式
     */
    public static Date strToDate(String strDate) {
        if (StringUtils.isEmpty(strDate)) {
            return null;
        }
        else{
            int length = strDate.length();
            if(strDate.contains("/"))
            {
                strDate = strDate.replace("/","-");
            }

            if(strDate.contains("-"))
            {
                if(length == 10)
                {
                    return strToDate(strDate,YYYY_MM_DD);
                }
                else if(length == 19)
                {
                    return strToDateTime(strDate,YYYY_MM_DD_HH_MM_SS);
                }
            }
            else{
                if(length == 8)
                {
                    return strToDate(strDate,YYYYMMDD);
                }
                else if(length == 14)
                {
                    return strToDateTime(strDate,YYYYMMDDHHMMSS);
                }
            }
        }

        return null;
    }

    /**
     * 获取现在时间
     *
     * @return 返回时间类型 yyyy-MM-dd HH:mm:ss
     */
    public static Date getNowDate() {
        return new Date();
    }

    /**
     * 将短时间格式字符串转换为时间
     */
    public static Date strToDate(String strDate, String format) {
        if (StringUtils.isEmpty(strDate)) {
            return null;
        }
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern(format);
        LocalDate localDate = LocalDate.parse(strDate,dateTimeFormatter);
        return Date.from(localDate.atStartOfDay(zone).toInstant());
    }

    /**
     * 将短时间格式字符串转换为时间
     */
    public static Date strToDateTime(String strDate, String format) {
        if (StringUtils.isEmpty(strDate)) {
            return null;
        }
        strDate = strDate.replace(".0","");
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern(format);
        LocalDateTime localDateTime = LocalDateTime.parse(strDate,dateTimeFormatter);
        return Date.from(localDateTime.atZone(zone).toInstant());
    }

    /**
     * 获取现在时间
     *
     * @return返回短时间格式 yyyy-MM-dd
     */
    public static Date getNowDateShort() {
        Instant instant = LocalDate.now().atStartOfDay(zone).toInstant();
        return Date.from(instant);
    }

    /**
     * 获取现在时间
     *
     * @return返回字符串格式 yyyy-MM-dd HH:mm:ss
     */
    public static String getStringDate() {
        return dateToStringLong(new Date(),YYYY_MM_DD_HH_MM_SS);
    }

    /**
     * 获取现在时间
     *
     * @return返回字符串格式 yyyy-MM-dd HH:mm:ss
     */
    public static String getStringDateUpper() {
        return dateToStringLong(new Date(),YYYYMMDDHHMMSS);
    }

    public static String getYyyymmddhhmmss(Date date) {
        return dateToStringLong(date,YYYYMMDDHHMMSS);
    }

    public static LocalDate dateToLocalDate(Date date){
        return date.toInstant().atZone(zone).toLocalDate();
    }

    public static LocalDateTime dateToLocalDateTime(Date date){
        return date.toInstant().atZone(zone).toLocalDateTime();
    }

    public static String dateToStringLong(Date date,String format){
        if(date == null){
            return "";
        }
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern(format);
        return dateTimeFormatter.format(dateToLocalDateTime(date));
    }

    /**
     * 获取现在时间
     *
     * @return返回字符串格式 yyyy-MM-dd HH:mm:ss
     */
    public static String getStringDate(String format) {
        Date date = new Date();
        return dateToStringLong(date,format);
    }

    /**
     * 获取现在时间
     *
     * @return返回字符串格式 yyyy-MM-dd HH:mm:ss
     */
    public static String getStringDatePre(String format,String days) {
        Date currentTime = new Date();
        long preTimes = currentTime.getTime()-24*60*60*1000;
        if (!StringUtils.isEmpty(days)){
            preTimes = currentTime.getTime()-Long.parseLong(days)*24*60*60*1000;
        }
        Date preDate = new Date(preTimes);
        return dateToStringLong(preDate,format);
    }
    /**
     * 获取现在时间
     *
     * @return 返回短时间字符串格式yyyy-MM-dd
     */
    public static String getStringDateShort() {
        Date currentTime = new Date();
        return dateToStringLong(currentTime,YYYY_MM_DD);
    }

    /**
     * 获取时间 小时:分;秒 HH:mm:ss
     */
    public static String getTimeShort() {
        Date currentTime = new Date();
        return dateToStringLong(currentTime,HH_MM_SS);
    }

    /**
     * 将长时间格式字符串转换为时间 yyyy-MM-dd HH:mm:ss
     */
    public static Date strToDateLong(String strDate) {
        if (StringUtils.isEmpty(strDate)) {
            return null;
        }
        return strToDateTime(strDate, YYYY_MM_DD_HH_MM_SS);
    }


    /**
     * 将长时间格式字符串转换为时间 yyyyMMdd HH:mm:ss
     */
    public static Date strToYmdDateLong(String strDate) {
        if (StringUtils.isEmpty(strDate)) {
            return null;
        }
        return strToDateTime(strDate, YYYYMMDD_HH_MM_SS);
    }

    public static Date strToDateShort(String strDate) {
        if (StringUtils.isEmpty(strDate)) {
            return null;
        }
        return strToDate(strDate, YYYY_MM_DD);
    }

    /**
     * 将长时间格式时间转换为字符串 yyyy-MM-dd HH:mm:ss
     */
    public static String dateToStrLong(Date dateDate) {
        if (dateDate == null) {
            return "";
        }
        return dateToStringLong(dateDate,YYYY_MM_DD_HH_MM_SS);
    }

    /**
     * 将长时间格式时间转换为字符串 yyyy-MM-dd HH:mm:ss
     */
    public static String dateToStrFormatLong(java.util.Date dateDate) {
        if (dateDate == null) {
            return "";
        }
        return dateToStringLong(dateDate,YYYYMMDDHHMMSS);
    }
    public static String dateToStrNoSecond(java.util.Date dateDate) {
        if (dateDate == null) {
            return "";
        }
        return dateToStringLong(dateDate,YYYY_MM_DD_HH_MM);
    }
    /**
     * 将长时间格式时间转换为字符串 YYYYMMDD
     */
    public static String dateToStrFormatShort(java.util.Date dateDate) {
        if (dateDate == null) {
            return "";
        }
        return dateToStringLong(dateDate,YYYYMMDD);
    }

    /**
     * 将长时间格式时间转换为字符串 yyyyMMdd
     */
    public static String dateToStrShort(java.util.Date dateDate) {
        if (dateDate == null) {
            return "";
        }
        return dateToStringLong(dateDate,YYYY_MM_DD);
    }

    /**
     * 将短时间格式时间转换为字符串 yyyy-MM-dd
     */
    public static String dateToStr(java.util.Date dateDate, String format) {
        if (dateDate == null) {
            return "";
        }
        return dateToStringLong(dateDate,format);
    }


    public static Date strToDateAppendNowTime(String strDate, String format) {
        if (StringUtils.isEmpty(strDate)) {
            return null;
        }
        strDate += " " + getTimeShort();
        return strToDateTime(strDate,format);
    }

    /**
     * 得到现在时间
     */
    public static Date getNow() {
        Date currentTime = new Date();
        return currentTime;
    }

    /**
     * 提取一个月中的最后一天
     */
    public static Date getLastDate(long day) {
        Date date = new Date();
        long date_3_hm = date.getTime() - 3600000 * 34 * day;
        Date date_3_hm_date = new Date(date_3_hm);
        return date_3_hm_date;
    }

    /**
     * 得到现在时间
     *
     * @return 字符串 yyyyMMdd HHmmss
     */
    public static String getStringToday() {
        Date currentTime = new Date();
        return dateToStringLong(currentTime,"yyyyMMdd HHmmss");
    }

    /**
     * 得到现在小时
     */
    public static String getHour() {
        Date currentTime = new Date();
        return dateToStringLong(currentTime,"HH");
    }

    /**
     * 得到现在分钟
     */
    public static String getTime() {
        Date currentTime = new Date();
        return dateToStringLong(currentTime,"mm");
    }

    /**
     * 根据用户传入的时间表示格式，返回当前时间的格式 如果是yyyyMMdd，注意字母y不能大写。
     *
     * @param sformat
     *            yyyyMMddhhmmss
     */
    public static String getUserDate(String sformat) {
        Date currentTime = new Date();
        return dateToStringLong(currentTime,sformat);
    }

    /**
     * 得到二个日期间的间隔天数
     */
    public static String getTwoDay(String sj1, String sj2) {
        LocalDate startDate = LocalDate.parse(sj1,dateFormatter);
        LocalDate endtDate = LocalDate.parse(sj2,dateFormatter);
        return startDate.until(endtDate, ChronoUnit.DAYS)+"";
    }

    /**
     * 时间前推或后推分钟,其中JJ表示分钟.
     */
    public static String getPreTime(String sj1, String jj) {
        LocalDateTime localDateTime = LocalDateTime.parse(sj1,dateTimeFormatter).plusMinutes(Integer.parseInt(jj));
        return dateTimeFormatter.format(localDateTime);
    }

    /**
     * 时间前推或后推天数（负数前推正数后推）
     * date 基准时间
     */
    public static Date getPreDays(Date date, int days) {
        Date day = null;
        try {
            LocalDateTime localDateTime = dateToLocalDateTime(date).plusDays(days);
            day = Date.from(localDateTime.atZone(zone).toInstant());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return day;
    }

    public static Date getPreHours(Date date, int hours) {
        Date day = null;
        try {
            LocalDateTime localDateTime = dateToLocalDateTime(date).plusHours(hours);
            day = Date.from(localDateTime.atZone(zone).toInstant());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return day;
    }

    /**
     * 得到一个时间延后或前移几分钟的时间,nowdate为时间,delay为前移或后延的分钟数
     */
    public static Date getNextMin(Date date, int delay) {
        try {
            LocalDateTime localDateTime = dateToLocalDateTime(date).plusMinutes(delay);
            return Date.from(localDateTime.atZone(zone).toInstant());
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * 得到一个时间延后或前移几天的时间,nowdate为时间,delay为前移或后延的天数
     */
    public static String getNextMin(String nowdate, int delay) {
        try {
            LocalDateTime localDateTime = LocalDateTime.parse(nowdate,dateTimeFormatter).plusMinutes(delay);
            DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern(HH_MM);
            return dateTimeFormatter.format(localDateTime);
        } catch (Exception e) {
            return "";
        }
    }

    public static String getNextMinute(String nowdate, int delay) {
        try {
            LocalDateTime localDateTime = LocalDateTime.parse(nowdate,dateTimeFormatter).plusMinutes(delay);
            return dateTimeFormatter.format(localDateTime);
        } catch (Exception e) {
            return "";
        }
    }

    /**
     * 得到一个时间延后或前移几天的时间,nowdate为时间,delay为前移或后延的天数
     */
    public static String getNextDay(String nowdate, int delay) {
        try {
            LocalDate localDate = LocalDate.parse(nowdate,dateFormatter).plusDays(delay);
            return dateFormatter.format(localDate);
        } catch (Exception e) {
            return "";
        }
    }

    /*
     * 将时间戳转换为时间
     */
    public static String stampToString(String s){
        LocalDateTime localDateTime = Instant.ofEpochMilli(Long.valueOf(s)).atZone(zone).toLocalDateTime();
        return dateTimeFormatter.format(localDateTime);
    }

    /*
     * 将时间戳转换为时间
     */
    public static Date stampToDate(String s){
        return Date.from(Instant.ofEpochMilli(Long.valueOf(s)).plusMillis(eightHoursMillis));
    }

    public static String getNextMinute(int minutes) {
        LocalDateTime localDateTime = LocalDateTime.now().withMinute(minutes);
        return dateTimeFormatter.format(localDateTime);
    }

    public static String getNextDay(Date d, int days) {
        return dateFormatter.format(dateToLocalDate(d).plusDays(days));
    }

    public static Date getNextDay1(Date d, int days) {
        return Date.from(dateToLocalDateTime(d).plusDays(days).atZone(zone).toInstant());
    }

    public static String getNextMonth(Date d, int months) {
        return dateFormatter.format(dateToLocalDate(d).plusMonths(months));
    }

    public static Date getNextMonthReturnDate(Date d, int months) {
        return Date.from(dateToLocalDateTime(d).plusMonths(months).atZone(zone).toInstant());
    }

    public static String getNextYear(Date d, int year) {
        return dateFormatter.format(dateToLocalDate(d).plusYears(year));
    }

    /**
     * 获取本月第一天
     */
    public static String getCurMonthFirstDayShort(){
        return dateFormatter.format(LocalDate.now().withDayOfMonth(1));
    }


    /**
     * 判断是否润年
     */
    public static boolean isLeapYear(String ddate) {
        /**
         * 详细设计： 1.被400整除是闰年，否则： 2.不能被4整除则不是闰年 3.能被4整除同时不能被100整除则是闰年
         * 3.能被4整除同时能被100整除则不是闰年
         */
        LocalDate localDate = LocalDate.parse(ddate,dateFormatter);
        return localDate.isLeapYear();
    }

    /**
     * 返回美国时间格式 26 Apr 2006
     */
    public static String getEDate(String str) {
        Date strtodate = strToDate(str,YYYY_MM_DD);
        String j = strtodate.toString();
        String[] k = j.split(" ");
        return k[2] + k[1].toUpperCase() + k[5].substring(2, 4);
    }

    /**
     * 获取一个月的最后一天
     */
    public static String getEndDateOfMonth(String dat) {// yyyy-MM-dd
        String str = dat.substring(0, 8);
        String month = dat.substring(5, 7);
        int mon = Integer.parseInt(month);
        if (mon == 1 || mon == 3 || mon == 5 || mon == 7 || mon == 8 || mon == 10 || mon == 12) {
            str += "31";
        } else if (mon == 4 || mon == 6 || mon == 9 || mon == 11) {
            str += "30";
        } else {
            if (isLeapYear(dat)) {
                str += "29";
            } else {
                str += "28";
            }
        }
        return str;
    }


    /**
     * 产生周序列,即得到当前时间所在的年度是第几周
     */
    public static String getSeqWeek() {
        LocalDate now = LocalDate.now();
        String week = Integer.toString(now.get(WeekFields.ISO.weekOfYear()));
        if (week.length() == 1){
            week = "0" + week;
        }
        String year = Integer.toString(now.getYear());
        return year + week;
    }

    /**
     * 获得一个日期所在的周的星期几的日期，如要找出2002年2月3日所在周的星期一是几号
     */
    public static String getWeek(String sdate, String num) {
        // 再转换为时间
        LocalDate localDate = LocalDate.parse(sdate,dateFormatter);
        switch (num) {
            case "1":  // 返回星期一所在的日期
                localDate.with(WeekFields.ISO.dayOfWeek(), DayOfWeek.MONDAY.getValue());
                break;
            case "2":  // 返回星期二所在的日期
                localDate.with(WeekFields.ISO.dayOfWeek(), DayOfWeek.TUESDAY.getValue());
                break;
            case "3":  // 返回星期三所在的日期
                localDate.with(WeekFields.ISO.dayOfWeek(), DayOfWeek.WEDNESDAY.getValue());
                break;
            case "4":  // 返回星期四所在的日期
                localDate.with(WeekFields.ISO.dayOfWeek(), DayOfWeek.THURSDAY.getValue());
                break;
            case "5":  // 返回星期五所在的日期
                localDate.with(WeekFields.ISO.dayOfWeek(), DayOfWeek.FRIDAY.getValue());
                break;
            case "6":  // 返回星期六所在的日期
                localDate.with(WeekFields.ISO.dayOfWeek(), DayOfWeek.SATURDAY.getValue());
                break;
            case "0":  // 返回星期日所在的日期
                localDate.with(WeekFields.ISO.dayOfWeek(), DayOfWeek.SUNDAY.getValue());
                break;
        }
        return dateFormatter.format(localDate);
    }

    /**
     * 根据一个日期，返回是星期几的字符串
     */
    public static String getWeek(String sdate) {
        // 再转换为时间
        // hour中存的就是星期几了，其范围 1~7
        // 1=星期日 7=星期六，其他类推
        LocalDate localDate = LocalDate.parse(sdate,dateFormatter);
        return DateTimeFormatter.ofPattern("EEEE").format(localDate);
    }

    public static String getWeekStr(String sdate) {
        String str;
        str = getWeek(sdate);
        switch (str) {
            case "1":
                str = "星期日";
                break;
            case "2":
                str = "星期一";
                break;
            case "3":
                str = "星期二";
                break;
            case "4":
                str = "星期三";
                break;
            case "5":
                str = "星期四";
                break;
            case "6":
                str = "星期五";
                break;
            case "7":
                str = "星期六";
                break;
        }
        return str;
    }

    /**
     * 日期比较大小
     */
    public static long compareDate(Date s1, Date s2) {
        try {
            return compareDate(YYYY_MM_DD, dateToStrShort(s1), dateToStrShort(s2));
        } catch (Exception e) {
            return -1;
        }

    }

    /**
     * 字符串日期比较大小
     */
    public static long compareDate(String format, String s1, String s2) {
        Date s = strToDate(s1,YYYY_MM_DD);
        Date end = strToDate(s2,YYYY_MM_DD);
        try {
            return s.getTime() - end.getTime();
        } catch (Exception e) {
            return -1;
        }
    }


    /**
     * 两个时间之间的天数
     */
    public static long getDays(String date1, String date2) {
        if(date1 == null || date1.equals("")){
            return 0;
        }
        if(date2 == null || date2.equals("")){
            return 0;
        }
        // 转换为标准时间
        LocalDate date = LocalDate.parse(date1,dateFormatter);
        LocalDate mydate = LocalDate.parse(date2,dateFormatter);

        return date.until(mydate,ChronoUnit.DAYS);
    }

    /**
     * 返回两个日期相差的天数
     */
    public static long getDays(Date date1, Date date2) {
        if (date1 == null || date2 == null){
            return 0;
        }
        long day = (date1.getTime() - date2.getTime()) / (24 * 60 * 60 * 1000);
        return day;
    }

    /**
     * 返回两个日期相差的小时数(保留2位小数)
     */
    public static float getHours(Date date1, Date date2) {
        if (date1 == null || date2 == null){
            return 0;
        }
        float hours = (date1.getTime() - date2.getTime()) / (float)(60 * 60 * 1000);
        BigDecimal decimal = new BigDecimal(hours);
        float hour = decimal.setScale(2, BigDecimal.ROUND_HALF_UP).floatValue();
        return hour;
    }


    /**
     * 形成如下的日历 ， 根据传入的一个时间返回一个结构 星期日 星期一 星期二 星期三 星期四 星期五 星期六 下面是当月的各个时间
     * 此函数返回该日历第一行星期日所在的日期
     */
    public static String getNowMonth(String sdate) {
        // 取该时间所在月的一号
        sdate = sdate.substring(0, 8) + "01";
        // 得到这个月的1号是星期几
        LocalDate localDate = LocalDate.parse(sdate,dateFormatter).with(WeekFields.ISO.dayOfWeek(), 7L);
        return dateFormatter.format(localDate);
    }

    /**
     * 取得数据库主键 生成格式为yyyymmddhhmmss+k位随机数
     *
     * @param k 表示是取几位随机数，可以自己定
     */
    public static String getNo(int k) {
        return getUserDate("yyyyMMddhhmmss") + getRandom(k);
    }

    /**
     * 取得流水号 生成格式为yyyymmddhhmmss+k位随机UUID
     *
     * @param k 表示是取几位随机数，可以自己定
     */
    public static String getUidNo(int k) {
        return getUserDate("yyyyMMddhhmmss") + UUID.randomUUID().toString().replaceAll("-", "").substring(0, k);
    }

    /**
     * 返回一个随机数
     */
    public static String getRandom(int i) {
        Random jjj = new Random();
        if(i == 0){
            return "";
        }
        String jj = "";
        for (int k = 0; k < i; k++) {
            jj = jj + jjj.nextInt(9);
        }
        return jj;
    }

    /**
     * 根据用户生日计算年龄 周岁计算
     */
    public static int getAgeByBirthday(Date birthday) {
        try {
            int age = 0;

            if (birthday == null) {
                return age;
            }
            LocalDate localDate = dateToLocalDate(birthday);
            return localDate.until(LocalDate.now()).getYears();
        } catch (Exception e) {
            return 0;
        }
    }


    /**
     *  字符串转时间
     */
    public static Date stringToDate(String str, String eg) {
        return strToDate(str,eg);
    }

    public static int getNowMonth(){
        return LocalDate.now().getMonthValue();
    }

    public static int getNowYear(){
        return LocalDate.now().getYear();
    }

    /**
     * 获取周一
     */
    public static String getMondayOfThisWeek() {
        LocalDate localDate = LocalDate.now().with(WeekFields.ISO.getFirstDayOfWeek());
        return dateFormatter.format(localDate);
    }

    /**
     * 获取某个时间的周一
     */
    public static String getMondayOfThisWeek(Date date) {
        LocalDate localDate = dateToLocalDate(date).with(WeekFields.ISO.getFirstDayOfWeek());
        return dateFormatter.format(localDate);
    }

    /**
     * 得到本周周日
     *
     * @return yyyy-MM-dd
     */
    public static String getSundayOfThisWeek() {
        LocalDate localDate = LocalDate.now().with(WeekFields.ISO.dayOfWeek(),7L);
        return dateFormatter.format(localDate);
    }

    /**
     * 获取某个时间的周末
     *
     * @return yyyy-MM-dd
     */
    public static String getSundayOfThisWeek(Date date) {
        LocalDate localDate = dateToLocalDate(date).with(WeekFields.ISO.dayOfWeek(),7L);
        return dateFormatter.format(localDate);
    }

    /**
     * 获取某一时间端内的工作天数
     */
    public static long getWorkDays(String start, String end){
        long day = 0;
        LocalDate localDateStart =LocalDate.parse(start,dateFormatter);
        LocalDate localDateEnd =LocalDate.parse(end,dateFormatter);
        while (!localDateEnd.isBefore(localDateStart)) {
            //如果不是周六或者周日则工作日+1
            if (localDateStart.getDayOfWeek().getValue() <6) {
                day++;
            }
            localDateStart = localDateStart.plusDays(1);
        }
        return day;
    }

    /**
     * 获取当月第一天
     */
    public static String getFristDayOfMonth() {
        LocalDate localDate = LocalDate.now().with(TemporalAdjusters.firstDayOfMonth());
        return dateFormatter.format(localDate);
    }
    /**
     * 获取当月最后一天
     */
    public static String getLastDayOfMonth(){
        LocalDate localDate = LocalDate.now().with(TemporalAdjusters.lastDayOfMonth());
        return dateFormatter.format(localDate);
    }

    public static int getSignYear(){
        if(LocalDate.now().getMonthValue()>=7){
            return getNowYear();
        }
        return getNowYear()-1;
    }

    /**
     * 计算预产期
     * 末次月经开始日期(第一天)，月份+9，日期+7
     */
    public static Date getDueDate(Date date){
        LocalDateTime localDateTime = dateToLocalDateTime(date);
        localDateTime = localDateTime.plusMonths(9).plusDays(7);
        return Date.from(localDateTime.atZone(zone).toInstant());
    }

    /**
     * 计算产检时间
     */
    public static Date getPrenatalInspectorDate(Date date,Integer day){
        LocalDateTime localDateTime = dateToLocalDateTime(date);
        localDateTime = localDateTime.plusDays(day);
        return Date.from(localDateTime.atZone(zone).toInstant());
    }

    /**
     * 将HH:MM格式的字符串转TIME
     */
    public static Time hhmmStrToTime(String hhmm){
        Time time = null;
        try {
            String s[] = hhmm.split(":");
            time = new Time(Integer.parseInt(s[0]),Integer.parseInt(s[1]),0);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return time;

    }

    /**
     * 获取当前时间的Timestamp
     */
    public static Timestamp getNowTimestamp(){
        Date date = new Date();
        Timestamp nousedate = new Timestamp(date.getTime());
        return nousedate;
    }

    /**
     *时间字符串转为 时间戳
     */
    public static Timestamp toTimestamp(String str, String format) {

        if (str == null) {
            return null;
        }

        try {
            return new Timestamp(stringToDate(str, format).getTime());
        } catch (Exception e) {
            return null;
        }

    }
    /**
     *时间字符串转为 时间戳
     */
    public static Timestamp toTimestamp(String str) {

        if (str == null) {
            return null;
        }

        try {
            return Timestamp.valueOf(str.trim());
        } catch (IllegalArgumentException iae) {
            return null;
        }

    }

    public static long compareDateTime(Date s1, Date s2) {
        return s1.getTime() - s2.getTime();
    }

    /**
     *  日期加减天数
     */
    public static Date setDateTime(Date date,int days){
        LocalDateTime localDateTime = dateToLocalDateTime(date).plusDays(days);
        return Date.from(localDateTime.atZone(zone).toInstant());
    }

    public static java.util.Date setDateHours(Date date,int hours){
        LocalDateTime localDateTime = dateToLocalDateTime(date).plusHours(hours);
        return Date.from(localDateTime.atZone(zone).toInstant());
    }

    /**
     * 获取去年日期
     */
    public static String getLastYear(){
        LocalDateTime localDateTime = LocalDateTime.now().plusYears(-1);
        return dateTimeFormatter.format(localDateTime);
    }

    /**
     * 获取儿童的年龄
     */
    public static String getChildAge(Date date){
        LocalDate now = LocalDate.now();
        LocalDate localDate = dateToLocalDate(date);

        int year = localDate.until(now).getYears();
        int month = localDate.until(now).getMonths();
        int day = localDate.until(now).getDays();

        String y = year>0? year+"岁":"";
        String m = month>0? month+"月":"";
        String d = day>0? day+"天":"";
        return y+m+d;
    }

    public static int getWeekOfMonth(String dateString){
        Date date = strToDate(dateString);
        return getWeekOfMonth(date);
    }

    public static int getWeekOfMonth(Date date){
        LocalDate localDate = dateToLocalDate(date);
        return localDate.get(WeekFields.ISO.weekOfMonth());
    }

    //将时间维度查出来的quotaDate转成yyyy-MM
    public static String changeQuotaDate(Date quotaDate){
        LocalDate localDate = dateToLocalDate(quotaDate);
        return DateTimeFormatter.ofPattern(YYYY_MM).format(localDate);
    }

    public static String getYear(String strDate, String format){
        LocalDate localDate = LocalDate.parse(strDate,dateFormatter);
        return localDate.getYear()+"";
    }

    public static String getMonth(String dateString){
        LocalDate localDate = LocalDate.parse(dateString,dateFormatter);
        return DateTimeFormatter.ofPattern(YYYY_MM).format(localDate);
    }

    /**
     * 获取某个时间
     *
     * @return返回短时间格式 yyyy-MM-dd
     */
    public static Date getDateShort(Date date) {
        LocalDate localDate = dateToLocalDate(date);
        LocalTime localTime = LocalTime.of(0,0,0);
        LocalDateTime localDateTime = LocalDateTime.of(localDate,localTime);
        return Date.from(localDateTime.atZone(zone).toInstant());
    }

    /**
     * 获取当月第一天
     */
    public static String getFristDayOfMonthThisDate(Date date) {
        LocalDate localDate = dateToLocalDate(date).with(TemporalAdjusters.firstDayOfMonth());
        return dateFormatter.format(localDate);
    }
    /**
     * 获取当月最后一天
     */
    public static String getLastDayOfMonthThisDate(Date date){
        LocalDate localDate = dateToLocalDate(date).with(TemporalAdjusters.lastDayOfMonth());
        return dateFormatter.format(localDate);
    }

    /**
     * 本年第一天
     * @return
     */
    public static String getFirstDayOfYear(){
        LocalDate localDate = LocalDate.now().with(TemporalAdjusters.firstDayOfYear());
        return dateFormatter.format(localDate);
    }
    public static String getFirstDayOfYear(Date date){
        LocalDate localDate = dateToLocalDate(date).with(TemporalAdjusters.firstDayOfYear());
        return dateFormatter.format(localDate);
    }

    /**
     * 本年最后一天
     */
    public static String getLastDayOfYearThisDate(Date date){
        LocalDate localDate = dateToLocalDate(date).with(TemporalAdjusters.lastDayOfYear());
        return dateFormatter.format(localDate);
    }

    /**
     * 根据日期对象返回星期几
     */
    public static int getWeekByDate(Date date){
        LocalDate localDate = dateToLocalDate(date);
        return localDate.getDayOfWeek().getValue()+1;
    }

    /**
     * 获取当天0点
     */
    public static Date getDateStart(){
        LocalDate localDate = LocalDate.now();
        LocalTime localTime = LocalTime.of(0,0,0);
        LocalDateTime localDateTime = LocalDateTime.of(localDate,localTime);
        return Date.from(localDateTime.atZone(zone).toInstant());
    }

    /**
     * 获取当天23:59:59
     */
    public static Date getDateEnd(){
        LocalDate localDate = LocalDate.now();
        LocalTime localTime = LocalTime.of(23,59,59);
        LocalDateTime localDateTime = LocalDateTime.of(localDate,localTime);
        return Date.from(localDateTime.atZone(zone).toInstant());
    }


    /**
     * 根据身份证的号码算出当前身份证持有者的年龄
     */
    public static int getAgeForIdcard(String idcard) {
        try {
            int age = 0;

            if (StringUtils.isEmpty(idcard)) {
                return age;
            }

            String birth = "";

            if (idcard.length() == 18) {
                birth = idcard.substring(6, 14);
            } else if (idcard.length() == 15) {
                birth = "19" + idcard.substring(6, 12);
            }

            int year = Integer.parseInt(birth.substring(0, 4));
            int month = Integer.parseInt(birth.substring(4, 6));
            int day = Integer.parseInt(birth.substring(6));
            LocalDate localDate = LocalDate.of(year,month,day);
            return localDate.until(LocalDate.now()).getYears();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return -1;
    }

    public static List<Map<String,Object>> findDates(Date dBegin, Date dEnd) {
        List<Map<String,Object>> lDate = new ArrayList();
        Map<String,Object> st = new HashedMap();
        st.put("date",DateUtil.dateToStr(dBegin,"yyyy-MM-dd"));
        st.put("avg",0);
        st.put("count", 0);
        lDate.add(st);

        Calendar calBegin = Calendar.getInstance();
        // 使用给定的 Date 设置此 Calendar 的时间
        calBegin.setTime(dBegin);

        Calendar calEnd = Calendar.getInstance();
        // 使用给定的 Date 设置此 Calendar 的时间
        calEnd.setTime(dEnd);

        // 测试此日期是否在指定日期之后
        while (dEnd.after(calBegin.getTime())) {
            // 根据日历的规则，为给定的日历字段添加或减去指定的时间量
            calBegin.add(Calendar.DAY_OF_MONTH, 1);
            if(!dEnd.after(calBegin.getTime())){
                break;
            }
            Map<String,Object> stt = new HashedMap();
            stt.put("date",DateUtil.dateToStr(calBegin.getTime(),"yyyy-MM-dd"));
            stt.put("avg",0);
            stt.put("count", 0);
            lDate.add(stt);
        }
        return lDate;
    }

    public static String getDateFromDateTime(String dateTime) {

        if (dateTime == null || dateTime.length() < YYYY_MM_DD.length()) {
            return null;
        }

        return dateTime.substring(0, 10);
    }

    /**
     * 判断时间是否在某个区间内
     */
    public static boolean isInArea(Date time,Date begin,Date end) {
        Long beginTime = compareDateTime(time,begin);
        Long endTime = compareDateTime(time, end);
        return beginTime > 0 && endTime < 0;
    }

    /**
     *  返回 xx天XX小时XX分钟XX秒前
     * @param date1 当前时间
     * @param date2 过去时间
     * @param fliter 0不携带秒 1携带秒
     */
    public static String getDifferentTimeInfo(Date date1, Date date2,Integer fliter){
        if (date1 == null || date2 == null){
            return null;
        }
        StringBuilder result = new StringBuilder("");
        long millisecondsDiff = date1.getTime() - date2.getTime();
        long secondsDiff = millisecondsDiff / TimeUnit.SECONDS.toMillis(1L);
        long minutesDiff = millisecondsDiff / TimeUnit.MINUTES.toMillis(1L);
        long hoursDiff = millisecondsDiff / TimeUnit.HOURS.toMillis(1L);
        long daysDiff = millisecondsDiff / TimeUnit.DAYS.toMillis(1L);
        long hourFieldDiff = hoursDiff - TimeUnit.DAYS.toHours(daysDiff);
        long minuteFieldDiff = minutesDiff - TimeUnit.HOURS.toMinutes(hoursDiff);
        long secondFieldDiff = secondsDiff - TimeUnit.MINUTES.toSeconds(minutesDiff);
        if (daysDiff > 0L) {
            result.append(String.format("%d天", daysDiff));
            result.append("前");
            return result.toString();
        }

        if (hourFieldDiff > 0L) {
            result.append(String.format("%d小时", hourFieldDiff));
        }
        if (minuteFieldDiff > 0L) {
            result.append(String.format("%d分钟", minuteFieldDiff));
            if (result.indexOf("小时")>0){
                result.append("前");
                return result.toString();
            }
        }
        if (1==fliter){
            if (secondFieldDiff > 0L) {
                result.append(String.format("%d秒", secondFieldDiff));
            }
        }else {
            if (result.indexOf("分钟")==-1){
                result.append("1分钟前");
                return result.toString();
            }
        }
        result.append("前");
        return result.toString();
    }

    /**
     *  返回 两个时间间隔天数/小时数
     * @param date1 当前时间
     * @param date2 过去时间
     */
    public static String getDifferentTimeInfo1(Date date1, Date date2){
        if (date1 == null || date2 == null){
            return null;
        }
        String result = null;
        long millisecondsDiff = date1.getTime() - date2.getTime();
        long secondsDiff = millisecondsDiff / TimeUnit.SECONDS.toMillis(1L);
        long minutesDiff = millisecondsDiff / TimeUnit.MINUTES.toMillis(1L);
        long hoursDiff = millisecondsDiff / TimeUnit.HOURS.toMillis(1L);
        long daysDiff = millisecondsDiff / TimeUnit.DAYS.toMillis(1L);
        long hourFieldDiff = hoursDiff - TimeUnit.DAYS.toHours(daysDiff);
        long minuteFieldDiff = minutesDiff - TimeUnit.HOURS.toMinutes(hoursDiff);
        long secondFieldDiff = secondsDiff - TimeUnit.MINUTES.toSeconds(minutesDiff);
        if (daysDiff > 0L) {
            result = (String.format("%d天", daysDiff));
            return result;
        }
        else if (hourFieldDiff > 0L) {
            result=String.format("%d小时", hourFieldDiff);
        }
        return result;
    }


    /**
     * 1970年1月1日来的 秒转化成时间
     */
    public static Date secondTransfor(int seconds){
        LocalDateTime localDateTime = LocalDateTime.of(1970,1,1,0,0,0);
        return Date.from(localDateTime.plusSeconds(seconds).atZone(zone).toInstant());
    }

    /**
     * 获取指定时间和当前时间的距离
     * 几秒前、几分钟前、几小时前、几天前
     * @param l1 秒级时间戳
     */
    public static String getTimeAgeStr(Long l1){
        String timeAgoStr;
        BigDecimal b1 = new BigDecimal(l1);
        BigDecimal b2 = new BigDecimal(System.currentTimeMillis()/1000);
        BigDecimal subtract = b2.subtract(b1);//秒数
        if(subtract.compareTo(new BigDecimal(60))<0){//小于一分钟前
            timeAgoStr = subtract +"秒前";
        }else if(subtract.compareTo(new BigDecimal(3600))<0){//小于一小时
            timeAgoStr = subtract.divide(new BigDecimal(60),0,BigDecimal.ROUND_HALF_DOWN).toString()+"分钟前";
        }else if(subtract.compareTo(new BigDecimal(86400))<0){//小于一天
            timeAgoStr = subtract.divide(new BigDecimal(3600),0,BigDecimal.ROUND_HALF_DOWN).toString()+"小时前";
        }else {//超过一天
            timeAgoStr = subtract.divide(new BigDecimal(86400),0,BigDecimal.ROUND_HALF_DOWN).toString()+"天前";
        }
        return timeAgoStr;
    }

    /**
     * 获取指定时间的前后若干时间戳
     * list[0]前
     * list[1]后
     */
    public static List<String> getTimeByBeforeAndAfterTime(String time,long l){
        List<String> list = new ArrayList<>();
        Date date = strToDate(time);
        Date beforeTime = new Date(date.getTime()-l);
        Date afterTime = new Date(date.getTime()+l);
        list.add(dateToStrLong(beforeTime));
        list.add(dateToStrLong(afterTime));
        return list;
    }
    /**
     * 计算当前时间和传入时间的相差
     * 秒数
     */
    public static Long behindSubBefore(String date){
        Date startDate = strToDate(date);
        Date currentDate = getNowDate();
        return (startDate.getTime() - currentDate.getTime()) / 1000;
    }

    public static int getWeekByString(String date){
        LocalDate localDate = LocalDate.parse(date,dateFormatter);
        int w = localDate.getDayOfWeek().getValue(); // 指示一个星期中的某天。
        if (w < 0)
            w = 0;
        return w;
    }

    public static String dateToWeek(String date) {
        String[] weekDays = { "周日", "周一", "周二", "周三", "周四", "周五", "周六" };
        return weekDays[getWeekByString(date)];
    }

    public static String getNowDateTime() {
        return dateToStringLong(new Date(),YYYYMMDDHHMMSS);
    }

    public static Timestamp getSysDateTime() {
        return new Timestamp(new Date().getTime());
    }

    public static Time getSysTime() {
        return new Time(new Date().getTime());
    }


    /**
     * 时间转字符串 yyyy-MM-dd HH:mm:ss
     */
    public static String toStringLong(Date date)
    {
        return toString(date,YYYY_MM_DD_HH_MM_SS);
    }

    public static String toString(Date date) {
        return toString(date, YYYY_MM_DD);
    }

    public static String toString(Date date, String format) {

        if (date == null) {
            return null;
        }

        if (format == null) {
            throw new IllegalArgumentException("The value of an argument is inaccurate.");
        }

        return dateToStr(date,format);
    }

    public static Date parseDate(String value, String pattern) {
        return strToDate(value,pattern);
    }

    public static Timestamp formatYMDToYMDHMS(String str) {
        if (str == null || str.trim().length() == 0) {
            return null;
        }
        str += " 00:00:00";
        java.util.Date date = strToDateLong(str);
        if (date == null) {
            return null;
        }
        Timestamp ts = fromatDateToTimestamp(new Date(date.getTime()));
        return ts;
    }

    public static Timestamp fromatDateToTimestamp(Date date) {
        Timestamp ts = null;
        try {
            ts = new Timestamp(date.getTime());
        } catch (Exception e) {
            return null;
        }
        return ts;
    }

    public static Date formatCharDateYMD(String str) {
        return formatCharDateYMD(str, YYYY_MM_DD);
    }

    public static Date formatCharDateYMD(String str, String format) {

        if (str == null || str.trim().length() == 0) {
            return null;
        }

        if (format == null) {
            throw new IllegalArgumentException("The value of an argument is inaccurate.");
        }

        return strToDate(str,format);
    }

    public static Date formatCharDateYMD(String yy, String mm, String dd) {

        if (yy == null || mm == null || dd == null || yy.trim().length() == 0 ||
                mm.trim().length() == 0 || dd.trim().length() == 0) {
            return null;
        }
        return formatCharDateYMD(yy + "-" + (mm != null && mm.length() == 1 ? "0" + mm : mm) + "-" +
                (dd != null && dd.length() == 1 ? "0" + dd : dd));
    }

    public static int getDifferenceOfDays(Date dateFrom, Date dateTo) {

        return new Long((dateTo.getTime() - dateFrom.getTime()) / 1000 / 60 / 60 / 24).intValue();
    }

    public static long getDifferenceOfSeconds(Date dateFrom, Date dateTo) {
        if(dateFrom==null||dateTo==null){
            return 0L;
        }
        long seconds = (dateTo.getTime() - dateFrom.getTime()) / 1000;
        seconds = seconds>0?seconds:0;
        return seconds;
    }

    public static long getDifferenceOfMillis(Date dateFrom, Date dateTo) {
        if(dateFrom==null||dateTo==null){
            return 0L;
        }
        long seconds = dateTo.getTime() - dateFrom.getTime();
        seconds = seconds>0?seconds:0;
        return seconds;
    }

    public static int getDifferenceOfDays(String dateFromStr, String dateToStr, String dateFormat) {

        Date dateFrom = parseDate(dateFromStr, dateFormat);
        Date dateTo = parseDate(dateToStr, dateFormat);

        return getDifferenceOfDays(dateFrom, dateTo);
    }

    public static String formatDate(Date value, String pattern) {
        return dateToStr(value,pattern);
    }


    /**
     * 获取本月第一天
     * @return
     */
    public static String getFirstMonthDay()
    {
        Date date = new Date();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMinimum(Calendar.DAY_OF_MONTH));
        return dateToStrShort(calendar.getTime());
    }

    /**
     * 获取本月最后一天
     *
     * @param
     * @return
     */
    public static String getLastMonthDay()
    {
        Date date = new Date();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMaximum(Calendar.DAY_OF_MONTH));
        return  dateToStrShort(calendar.getTime());
    }

    /**
     * 当前年的开始时间，即2012-01-01 00:00:00
     *
     * @return
     */
    public static String getCurrentYearStartTime() {
        Calendar c = Calendar.getInstance();
        Date now = null;
        try {
            c.set(Calendar.MONTH, 0);
            c.set(Calendar.DATE, 1);
            now = c.getTime();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return dateToStrShort(now);
    }

    /**
     * 当前年的结束时间，即2012-12-31 23:59:59
     *
     * @return
     */
    public  static String getCurrentYearEndTime() {
        Calendar c = Calendar.getInstance();
        Date now = null;
        try {
            c.set(Calendar.MONTH, 11);
            c.set(Calendar.DATE, 31);
            now = c.getTime();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return dateToStrShort(now);
    }

    /**
     * 同比 某个时间去年得时间
     * @param date
     * @return
     */
    public static String getYearDate(Date date){
        Calendar cal = Calendar.getInstance();
        //设置当前时间
        cal.setTime(date);
        //在当前时间基础上减一年
        cal.add(Calendar.YEAR, -1);
        return dateToStrShort(cal.getTime());
    }

    /**
     * 环比 某个时间上一期时间
     * @param date
     * @param n -2 就是上期两个月
     * @return
     */
    public static String getMonthDate(Date date,Integer n){
        Calendar cal = Calendar.getInstance();
        //设置当前时间
        cal.setTime(date);
        //在当前时间基础上减n个月
        cal.add(Calendar.MONTH, n);
        return dateToStrShort(cal.getTime());
    }

    /**
     * 获取两个日期相差的月数
     */
    public static int getMonthDiff(Date d1, Date d2) {
        Calendar c1 = Calendar.getInstance();
        Calendar c2 = Calendar.getInstance();
        c1.setTime(d1);
        c2.setTime(d2);
        int year1 = c1.get(Calendar.YEAR);
        int year2 = c2.get(Calendar.YEAR);
        int month1 = c1.get(Calendar.MONTH);
        int month2 = c2.get(Calendar.MONTH);
        int day1 = c1.get(Calendar.DAY_OF_MONTH);
        int day2 = c2.get(Calendar.DAY_OF_MONTH);
        // 获取年的差值
        int yearInterval = year1 - year2;
        // 如果 d1的 月-日 小于 d2的 月-日 那么 yearInterval-- 这样就得到了相差的年数
        if (month1 < month2 || month1 == month2 && day1 < day2) {
            yearInterval--;
        }
        // 获取月数差值
        int monthInterval = (month1 + 12) - month2;
        if (day1 < day2) {
            monthInterval--;
        }
        monthInterval %= 12;
        int monthsDiff = Math.abs(yearInterval * 12 + monthInterval);
        return monthsDiff;
    }

    /**
     * 根据数组返回若干天后的时间
     * @return 返回yyyy-MM-dd格式集合
     */
    public static List<String> getDaysByNow(int[] dayArray){
        List<String> list = new ArrayList<>();
        Date date = new Date();
        for (int i : dayArray) {
            Date preDays = DateUtil.getPreDays(date, i);
            list.add(DateUtil.dateToStrShort(preDays));
        }
        return list;
    }
}
