package com.yihu.jw.util.fastdfs;


import org.csource.fastdfs.*;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

/**
 * Created by szx on 2015/9/19.
 */
public class FastDFSClientPool {
    private static volatile FastDFSClientPool pool;

    private Map<StorageClient, Boolean> map;
    private Map<Integer, StorageClient> pathIndexMap;
    private Map<Integer, StorageClient> pathIndexMapUsed;

    private int maxPoolSize = 20;
    private int waitTime = 500;
    private int initPoolSize = 5;

    static {
        try {
            
            String basePath = FastDFSClientPool.class.getResource("/").getPath();
           
            ClientGlobal.init(FastDFSUtil.class.getResource("/config/fdfs_client.conf").getPath());


        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private FastDFSClientPool() {

        init();
    }

    public static FastDFSClientPool getInstance() {
        if (pool == null) {
            synchronized (FastDFSClientPool.class) {
                if(pool == null) {
                    pool = new FastDFSClientPool();
                }
            }
        }
        System.out.println("FastDFSClientPool:"+pool.toString());
        return pool;
    }

    private void init() {
        try {

            map = new HashMap<StorageClient, Boolean>();
            pathIndexMap = new HashMap<>();
            pathIndexMapUsed = new HashMap<>();

            //不做初始缓存
//            for (int i = 0; i < initPoolSize; i++) {
//                map.put(getNewStorageClient(), true);
//            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    public TrackerServer getTrackerServer(){
        try {
            TrackerClient tracker = new TrackerClient();
            return tracker.getConnection();
        }catch (Exception e){
            e.printStackTrace();
        }
        return null;

    }

    private StorageClient getNewStorageClient(Integer storePath) {
        try {
            TrackerClient tracker = new TrackerClient();
            TrackerServer trackerServer = tracker.getConnection();
            StorageServer storageServer =null;
            if (storePath !=null ){
                storageServer = tracker.getStoreStorage(trackerServer);
                String storageIp = storageServer.getSocket().getInetAddress().getHostAddress();
                Integer port = storageServer.getSocket().getPort();
                storageServer = new StorageServer(storageIp, port, storePath);
            }

            StorageClient client = new StorageClient(trackerServer, storageServer);

            return client;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public void releaseStorageClient(StorageClient client) {
        if (client == null) {
            return;
        }
        try {
            if(map.containsKey(client)) {
                    map.put(client, true);
            } else {
                for (Entry entry : pathIndexMapUsed.entrySet()) {
                   if (entry.getValue()==client){
                       pathIndexMap.put((Integer) entry.getKey(),client);
                       pathIndexMapUsed.put((Integer) entry.getKey(),null);
                    }
                }
                client = null;
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public synchronized StorageClient getStorageClient() {
        StorageClient client = null;
        try {
            for (Entry<StorageClient, Boolean> entry : map.entrySet()) {
                if (entry.getValue()) {
                    client = entry.getKey();
                    System.out.println("从缓存中获得fastdfs client");
                    map.put(client, false);
                    break;
                }
            }
            if (client == null) {
                if(map.size()<maxPoolSize) {
                    System.out.println("创建新fastdfs client:");
                    client = getNewStorageClient(null);
                    map.put(client, false);
                }

            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return client;
    }

    /**
     * 上传指定上传路径
     * @return
     */
    public synchronized StorageClient getStorageClientByPath(Integer storePath) {
        StorageClient client = null;
        try {
            for (Entry entry : pathIndexMap.entrySet()) {
                if (entry.getKey()==storePath){
                    if (null!=entry.getValue()){
                        client = (StorageClient) entry.getValue();
                        System.out.println("从缓存中获得fastdfs client");
                        pathIndexMap.put(storePath, null);
                        pathIndexMapUsed.put(storePath, client);
                        break;
                    }
                }
            }
            if (client == null) {
                if(pathIndexMap.size()<maxPoolSize) {
                    System.out.println("创建新fastdfs client:");
                    client = getNewStorageClient(storePath);
                    pathIndexMap.put(storePath, client);
                }

            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return client;
    }
}
