package com.yihu.jw.solr;

import com.yihu.jw.solr.config.SolrConfig;
import org.apache.solr.client.solrj.SolrClient;
import org.apache.solr.client.solrj.impl.CloudSolrClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.data.solr.server.support.HttpSolrClientFactory;
import org.springframework.stereotype.Service;
import org.xml.sax.SAXException;

import javax.xml.parsers.ParserConfigurationException;
import java.io.IOException;
import java.util.Optional;

/**
 * Solr连接池
 * @author hzp
 * @version 1.0
 * @created 2016.04.26
 */
@Service
@Scope(ConfigurableBeanFactory.SCOPE_SINGLETON)
public class SolrPool {

    @Autowired
    private SolrConfig solrConfig;

    private static volatile HttpSolrClientFactory factory;

    protected HttpSolrClientFactory getFactory() throws ParserConfigurationException, IOException, SAXException {
        if (factory != null) {
            return factory;
        }
        synchronized (HttpSolrClientFactory.class) {
            if (null == factory) {
                Optional<String> zkChroot = Optional.of("/");
                CloudSolrClient client = new CloudSolrClient.Builder(solrConfig.getZkHosts(),zkChroot).build();
                factory = new HttpSolrClientFactory(client);
            }
        }
        return factory;
    }

    public SolrClient getConnection(String core) throws Exception{
        if (factory != null) {
            return factory.getSolrClient();
        }
        return getFactory().getSolrClient();
    }

}
