package com.yihu.jw.gateway.filter;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.netflix.zuul.ZuulFilter;
import com.netflix.zuul.context.RequestContext;
import com.netflix.zuul.http.HttpServletRequestWrapper;
import com.netflix.zuul.http.ServletInputStreamWrapper;
import com.yihu.jw.gateway.AesEncryptUtils;
import com.yihu.jw.gateway.methlog.BaseLoginLogService;
import com.yihu.jw.gateway.methlog.BaseMethodLogService;
import com.yihu.jw.gateway.useragent.UserAgent;
import com.yihu.jw.restmodel.ResultStatus;
import com.yihu.jw.restmodel.web.Envelop;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Primary;
import org.springframework.data.redis.connection.jedis.JedisConnectionFactory;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.token.TokenStore;
import org.springframework.security.oauth2.provider.token.store.redis.RedisTokenStore;
import org.springframework.stereotype.Component;
import org.springframework.util.StreamUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.multipart.MultipartResolver;
import org.springframework.web.multipart.commons.CommonsMultipartResolver;

import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.util.*;

/**
 * Created by progr1mmer on 2017/12/27.
 */
@Component
public class BasicZuulFilter extends ZuulFilter {

    private static final Logger logger = LoggerFactory.getLogger(BasicZuulFilter.class);
    private static final String ACCESS_TOKEN_PARAMETER = "token";

    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private TokenStore tokenStore;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private BaseMethodLogService baseMethodLogService;
    @Autowired
    private BaseLoginLogService baseLoginLogService;
    @Autowired
    private UserAgent userAgent;
    @Autowired
    private RedisTokenStore redisTokenStore;


    @Override
    public String filterType() {
        return "pre";
    }

    @Override
    public int filterOrder() {
        return 0;
    }

    @Override
    public boolean shouldFilter() {

        return true;
    }

    @Override
    public Object run() {
        RequestContext ctx = RequestContext.getCurrentContext();
        HttpServletRequest request = ctx.getRequest();
        String url = request.getRequestURI();

        //防止SQL注入过滤器
        if(doSqlFilter(request)){
            return this.forbidden(ctx, ResultStatus.ERROR_PARA, "Illegal parameter");
        }

        //文件类型过滤器
        if(doFileFilter(request)){
            return this.forbidden(ctx, ResultStatus.ERROR_PARA, "Illegal file");
        }

        logger.info("BasicZuulFilter:"+url);

        if (!(url.contains("/open/noLogin/notify")||
            url.contains("/open/noLogin/ylzReceiveNotify")||
            url.contains("/open/noLogin/receiveNotify")||
            url.contains("/open/noLogin/getSFExpressInfoNew")||
            url.contains("/open/noLogin/sfroutepushservice")||
            url.contains("/open/noLogin/ylzSettleRecord")||
                url.contains("/weixin")||
                url.contains("/excelControl")||
                url.contains("/open/gc")||
                url.contains("/exportExcel")||
                url.contains("/prescription/pushListWrite")||
                url.contains("/prescription/pushListDrug")||
                url.contains("/open/noLogin/pushYkCardCharge")||
                url.contains("/open/noLogin/pushListWriteHealthInfo")||
                url.contains("/prescription/searchRecordWrite")||
                url.contains("/Funds/reconciliationExcel")||
                url.contains("/Funds/deviceInfoExcel"))){
            logger.info("入参"+ctx.getRequestQueryParams());
            try {
                decrypt(ctx,request);
                logger.info("第一步");
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        //保存操作日志
        baseMethodLogService.saveMethodLog(request);

        //内部微服务有不需要认证的地址请在URL上追加/open/来进行过滤，如/api/v1.0/open/**，不要在此继续追加！！！
        if (url.contains("/auth/")//验证服务
                || url.contains("/authgw/")//验证服务
                || url.contains("/authygTest/")//验证服务
                || url.contains("/base/wechat")//微信
                || url.contains("/patient/wxBase/")//微信
                || url.contains("/patient/wx/")//微信页面不拦截
                || url.contains("/basePatient/regist")//注册
                || url.contains("/basePatient/forgetPwd")//忘记密码
                || url.contains("/basePatient/sendCaptcha")//发送短信
                || url.contains("/sms_gateway/send")//发送短信
                || url.contains("/accountAppeal/create")
                || url.contains("/open/")//开放接口
                || url.contains("/exportExcel/")//导出
                || url.contains("/cloudCare/noLogin/")//医疗云开放接口
                || url.contains("/image/")//获取默认图片
                ||url.contains("/weixin")||
                url.contains("/excelControl")) {
            return true;
        }
        System.out.println("进入token验证");
        return this.authenticate(ctx, request, url);
    }


    public void decrypt(RequestContext ctx,HttpServletRequest request) throws Exception {
        String url = request.getRequestURI();
        String method = request.getMethod();
        if (method.equalsIgnoreCase("GET")){
            Map<String, List<String>> map = ctx.getRequestQueryParams();
            String aes_decodedStr = AesEncryptUtils.decrypt(map.get("object").toString());
            JSONObject object = JSONObject.parseObject(aes_decodedStr);
            if(object!=null){
                for(String str:object.keySet()){
                    List<String> arrayList = new ArrayList<>();
                    String value = object.getString(str);
              /*  if (sqlValidate(value)){
                    throw new Exception("Illegal parameter");
                }*/
                    arrayList.add(value);
                    map.put(str,arrayList);
                }
            }
            map.remove("object");
            ctx.setRequestQueryParams(map);
            logger.info("解密后的参数"+ctx.getRequestQueryParams());
        }else if ("POST".equals(method) || "PUT".equals(method)) {
            InputStream in = request.getInputStream();
            String body = StreamUtils.copyToString(in, Charset.forName("UTF-8"));
            // 如果body为空初始化为空json
            if (StringUtils.isEmpty(body)) {
                body = "{}";
            }
            logger.info("body" + body);
            // 获取请求参数name
            String jsonString = request.getParameter("object");
            logger.info("canshu"+jsonString);
            if (jsonString != null) {
                body="";
                String jsonobject = AesEncryptUtils.decrypt(jsonString);
                logger.info("解密：" + jsonobject);
                JSONObject jsonObject = JSONObject.parseObject(jsonobject);
                for(String str:jsonObject.keySet()){
                    String value = jsonObject.getString(str);
                    /*if (sqlValidate(value)){
                        throw new Exception("Illegal parameter");
                    }*/
                    if (!StringUtils.isEmpty(value)){
                        value = URLEncoder.encode(value);
                    }
                    body+=str+"="+ value+"&";
                }
                String newBody = body.substring(0,body.length()-1);
                logger.info("newBody" + newBody);
                final byte[] reqBodyBytes = newBody.getBytes();
                // 重写上下文的HttpServletRequestWrapper
                ctx.setRequest(new HttpServletRequestWrapper(request) {
                    @Override
                    public ServletInputStream getInputStream() throws IOException {
                        return new ServletInputStreamWrapper(reqBodyBytes);
                    }

                    @Override
                    public int getContentLength() {
                        return reqBodyBytes.length;
                    }

                    @Override
                    public long getContentLengthLong() {
                        return reqBodyBytes.length;
                    }
                });

            }
        }

    }

    /**
     * 验证token 权限地址
     * @param ctx
     * @param request
     * @param path
     * @return
     */
    private Object authenticate(RequestContext ctx, HttpServletRequest request, String path) {
        System.out.println("获取token");
        String accessToken = this.extractToken(request);
        System.out.println("获取token"+accessToken);
        if (null == accessToken) {
            return this.forbidden(ctx, ResultStatus.NULL_TOKEN, "token can not be null");
        }
        OAuth2AccessToken oAuth2AccessToken = tokenStore.readAccessToken(accessToken);
        if (null == oAuth2AccessToken) {
            return this.forbidden(ctx, ResultStatus.INVALID_TOKEN, "invalid token");
        }
        if (oAuth2AccessToken.isExpired()) {
            return this.forbidden(ctx, ResultStatus.EXPIRE, "expired token");
        }
        //将token的认证信息附加到请求中，转发给下游微服务
        OAuth2Authentication auth = tokenStore.readAuthentication(accessToken);
        ctx.addZuulRequestHeader("x-auth-name", auth.getName());
        //以下代码取消注释可开启Oauth2应用资源授权验证
//        Set<String> resourceIds = auth.getOAuth2Request().getResourceIds();
        String urls = redisTemplate.opsForValue().get("wlyy2:auth:token:"+accessToken);
        if(StringUtils.isEmpty(urls)){
            return this.forbidden(ctx, ResultStatus.NO_PERMI, "invalid token does not contain request resource " + path);
        }
        //官网服务-网关数据库不同，不验证数据库的token过期时间
        if (path.contains("/basegw/")) {
            return true;
        }
        //获取所有token资源
        String resourceIds[] = urls.split(",");
        String platform = request.getHeader("platform");
        for (String resourceId : resourceIds) {
            if (resourceId.equals("/**")) {
                System.out.println("/**"+true);
                //判断用户操作间隔
                if (null!=platform&&"city-ihealth-admin-web".equalsIgnoreCase(platform)){
                    //判断用户操作间隔
                    String userId = userAgent.getUID();
                    System.out.println(userId);
                    Boolean checkTimeOut = baseLoginLogService.checkTime(userId);
                    System.out.println("即将进入验证操作时间接口");
                    if (!checkTimeOut){
                        return this.forbidden(ctx, ResultStatus.OPERATE_TIME, "expired token");
                    }
                }
                return true;
            }
            if (!resourceId.startsWith("/")) {
                resourceId = "/" + resourceId;
            }
            path = path.toLowerCase();
            if (path.startsWith(resourceId)
                    && (path.length() == resourceId.length() || path.charAt(resourceId.length()) == '/')) {
                System.out.println("158"+true);
                //判断用户操作间隔
                if (null!=platform&&"city-ihealth-admin-web".equalsIgnoreCase(platform)){
                    //判断用户操作间隔
                    String userId = userAgent.getUID();
                    System.out.println(userId);
                    Boolean checkTimeOut = baseLoginLogService.checkTime(userId);
                    System.out.println("即将进入验证操作时间接口");
                    if (!checkTimeOut){
                        return this.forbidden(ctx, ResultStatus.OPERATE_TIME, "expired token");
                    }
                }
                return true;
            }
        }

        if (null!=platform&&"city-ihealth-admin-web".equalsIgnoreCase(platform)){
            //判断用户操作间隔
            String userId = userAgent.getUID();
            System.out.println(userId);
            Boolean checkTimeOut = baseLoginLogService.checkTime(userId);
            System.out.println("即将进入验证操作时间接口");
            if (!checkTimeOut){
                return this.forbidden(ctx, ResultStatus.OPERATE_TIME, "expired token");
            }
        }
        return this.forbidden(ctx, ResultStatus.NO_PERMI, "invalid token does not contain request resource " + path);
    }

    private String extractToken(HttpServletRequest request) {
        String accessToken = request.getHeader(ACCESS_TOKEN_PARAMETER);
        if (null == accessToken) {
            accessToken = request.getParameter(ACCESS_TOKEN_PARAMETER);
        }
        return accessToken;
    }

    private Object forbidden(RequestContext requestContext, int status, String errorMsg) {
        requestContext.setSendZuulResponse(false);
        Envelop envelop = new Envelop();
        envelop.setMessage(errorMsg);
        envelop.setStatus(status);
        try {
            JSONObject object = new JSONObject();
            object.put("status","200");
            object.put("data", AesEncryptUtils.encrypt(JSONObject.toJSONString(envelop)));
            //requestContext.setResponseStatusCode(status);
            requestContext.getResponse().getWriter().write(objectMapper.writeValueAsString(object));
        } catch (Exception e) {
            requestContext.setResponseStatusCode(ResultStatus.FAILED_RESP);
            logger.error(e.getMessage());
        }
        return false;
    }

    @Bean
    @Primary
    public RedisTokenStore redisTokenStore(JedisConnectionFactory jedisConnectionFactory) {
        return new RedisTokenStore(jedisConnectionFactory);
    }

    public boolean doSqlFilter(HttpServletRequest request){
        Enumeration params = request.getParameterNames();
        String sql = "";
        while (params.hasMoreElements()) {
            // 得到参数名
            String name = params.nextElement().toString();
            // 得到参数对应值
            String[] value = request.getParameterValues(name);
            for (int i = 0; i < value.length; i++) {
                sql = sql + value[i];
            }
            if (sqlValidate(sql)) {
                return true;
            }
        }
        return false;
    }
    public boolean doSqlFilterParams(HttpServletRequest request){
        Enumeration params = request.getParameterNames();
        String sql = "";
        while (params.hasMoreElements()) {
            // 得到参数名
            String name = params.nextElement().toString();
            // 得到参数对应值
            String[] value = request.getParameterValues(name);
            for (int i = 0; i < value.length; i++) {
                sql = sql + value[i];
            }
            if (sqlValidate(sql)) {
                return true;
            }
        }
        return false;
    }

    public boolean doFileFilter(HttpServletRequest req){
        MultipartResolver resolver = new CommonsMultipartResolver(req.getSession().getServletContext());
        if(resolver.isMultipart(req)){
            MultipartHttpServletRequest multipartRequest = resolver.resolveMultipart(req);
//            MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) req;
            Iterator<String> it = multipartRequest.getFileNames();

            while (it.hasNext()){
                String fileName = (String)it.next();
                logger.info("doFileFilter :"+fileName);
                MultipartFile file = multipartRequest.getFile(fileName);
                if(checkFile(file.getOriginalFilename())){
                    return true;
                }
            }
        }
        return false;
    }

    private static boolean sqlValidate(String str) {
        if (org.apache.commons.lang3.StringUtils.isNoneBlank(str)){
            str = str.toLowerCase();//统一转为小写,比较简单的单词加入右边空格，避免单词中包含字段
            String badStr = "and |exec |execute |insert |select |delete |update |drop |chr |mid |master |truncate |" +
                    "declare | sitename |net user|xp_cmdshell|or |exec |execute |create |" +
                    "table |from |grant |use |group_concat|column_name|" +
                    "information_schema.columns|table_schema|union |where |select |update |order |by |like |" ;//过滤掉的sql关键字，可以手动添加
            String[] badStrs = badStr.split("\\|");
            for (int i = 0; i < badStrs.length; i++) {
                if (str.indexOf(badStrs[i]) >= 0) {
                    return true;
                }
            }
            return false;
        }else {
            return true;
        }


    }

//    public static void main(String ag[]){
//        String str ="<?xml version='1.0' encoding='UTF-8'?><Request service=\"RoutePushService\" lang=\"zh-CN\"><Body><WaybillRoute id=\"4771647\" mailno=\"444019056680\" orderid=\"808080eb6c26bcad016c2826b9d6f794\" acceptTime=\"2019-11-26 12:21:46\" acceptAddress=\"深圳市\" remark=\"备注\" opCode=\"50\"/></Body></Request>";
//        sqlValidate(str);
//    }

    /**
     * 文件类型过滤器
     * @param fileName
     * @return
     */
    private  boolean checkFile(String fileName){
        boolean flag=true;
        String suffixList="xls,xlsx,jpg,gif,png,ico,bmp,jpeg,rm,rmvb,mov,wmv,avi,mp3,wav,ogg,mp4";
        //获取文件后缀
        String suffix=fileName.substring(fileName.lastIndexOf(".")+1, fileName.length());

        if(suffixList.contains(suffix.trim().toLowerCase())){

        }flag=false;
        return flag;
    }

    public final static Map<String, String> FILE_TYPE_MAP = new HashMap<String, String>();
    static {
            getAllFileType(); // 初始化文件类型信息
       }
    /**
      * 常用文件格式,不能上传的文件类型
      */
     private static void getAllFileType() {
//         FILE_TYPE_MAP.put("ffd8ffe000104a464946", "jpg"); // JPEG (jpg)
//         FILE_TYPE_MAP.put("89504e470d0a1a0a0000", "png"); // PNG (png)
//         FILE_TYPE_MAP.put("47494638396126026f01", "gif"); // GIF (gif)
//         FILE_TYPE_MAP.put("49492a00227105008037", "tif"); // TIFF (tif)
//         FILE_TYPE_MAP.put("424d228c010000000000", "bmp"); // 16色位图(bmp)
//         FILE_TYPE_MAP.put("424d8240090000000000", "bmp"); // 24位位图(bmp)
//         FILE_TYPE_MAP.put("424d8e1b030000000000", "bmp"); // 256色位图(bmp)
         FILE_TYPE_MAP.put("41433130313500000000", "dwg"); // CAD (dwg)
         FILE_TYPE_MAP.put("3c21444f435459504520", "html"); // HTML (html)
         FILE_TYPE_MAP.put("3c21646f637479706520", "htm"); // HTM (htm)
         FILE_TYPE_MAP.put("48544d4c207b0d0a0942", "css"); // css
         FILE_TYPE_MAP.put("696b2e71623d696b2e71", "js"); // js
//         FILE_TYPE_MAP.put("7b5c727466315c616e73", "rtf"); // Rich Text Format (rtf)
//         FILE_TYPE_MAP.put("38425053000100000000", "psd"); // Photoshop (psd)
//         FILE_TYPE_MAP.put("46726f6d3a203d3f6762", "eml"); // Email [Outlook Express 6] (eml)
//         FILE_TYPE_MAP.put("d0cf11e0a1b11ae10000", "doc"); // MS Excel 注意：word、msi 和 excel的文件头一样
//         FILE_TYPE_MAP.put("d0cf11e0a1b11ae10000", "vsd"); // Visio 绘图
//         FILE_TYPE_MAP.put("5374616E64617264204A", "mdb"); // MS Access (mdb)
//         FILE_TYPE_MAP.put("252150532D41646F6265", "ps");
//         FILE_TYPE_MAP.put("255044462d312e350d0a", "pdf"); // Adobe Acrobat (pdf)
//         FILE_TYPE_MAP.put("2e524d46000000120001", "rmvb"); // rmvb/rm相同
//         FILE_TYPE_MAP.put("464c5601050000000900", "flv"); // flv与f4v相同
//         FILE_TYPE_MAP.put("00000020667479706d70", "mp4");
//         FILE_TYPE_MAP.put("49443303000000002176", "mp3");
//         FILE_TYPE_MAP.put("000001ba210001000180", "mpg"); //
//         FILE_TYPE_MAP.put("3026b2758e66cf11a6d9", "wmv"); // wmv与asf相同
//         FILE_TYPE_MAP.put("52494646e27807005741", "wav"); // Wave (wav)
//         FILE_TYPE_MAP.put("52494646d07d60074156", "avi");
//         FILE_TYPE_MAP.put("4d546864000000060001", "mid"); // MIDI (mid)
//         FILE_TYPE_MAP.put("504b0304140000000800", "zip");
//         FILE_TYPE_MAP.put("526172211a0700cf9073", "rar");
//         FILE_TYPE_MAP.put("235468697320636f6e66", "ini");
         FILE_TYPE_MAP.put("504b03040a0000000000", "jar");
         FILE_TYPE_MAP.put("4d5a9000030000000400", "exe");// 可执行文件
         FILE_TYPE_MAP.put("3c25402070616765206c", "jsp");// jsp文件
         FILE_TYPE_MAP.put("4d616e69666573742d56", "mf");// MF文件
         FILE_TYPE_MAP.put("3c3f786d6c2076657273", "xml");// xml文件
         FILE_TYPE_MAP.put("494e5345525420494e54", "sql");// xml文件
         FILE_TYPE_MAP.put("7061636b616765207765", "java");// java文件
         FILE_TYPE_MAP.put("406563686f206f66660d", "bat");// bat文件
         FILE_TYPE_MAP.put("1f8b0800000000000000", "gz");// gz文件
         FILE_TYPE_MAP.put("6c6f67346a2e726f6f74", "properties");// bat文件
         FILE_TYPE_MAP.put("cafebabe0000002e0041", "class");// bat文件
         FILE_TYPE_MAP.put("49545346030000006000", "chm");// bat文件
         FILE_TYPE_MAP.put("04000000010000001300", "mxp");// bat文件
//         FILE_TYPE_MAP.put("504b0304140006000800", "docx");// docx文件
         FILE_TYPE_MAP.put("d0cf11e0a1b11ae10000", "wps");// WPS文字wps、表格et、演示dps都是一样的
         FILE_TYPE_MAP.put("6431303a637265617465", "torrent");
//         FILE_TYPE_MAP.put("6D6F6F76", "mov"); // Quicktime (mov)
//         FILE_TYPE_MAP.put("FF575043", "wpd"); // WordPerfect (wpd)
//         FILE_TYPE_MAP.put("CFAD12FEC5FD746F", "dbx"); // Outlook Express (dbx)
//         FILE_TYPE_MAP.put("2142444E", "pst"); // Outlook (pst)
//         FILE_TYPE_MAP.put("AC9EBD8F", "qdf"); // Quicken (qdf)
//         FILE_TYPE_MAP.put("E3828596", "pwl"); // Windows Password (pwl)
//         FILE_TYPE_MAP.put("2E7261FD", "ram"); // Real Audio (ram)
//         FILE_TYPE_MAP.put("null", null); // null
     }

    /**
     * 文件头比较
     * @param filePaht
     * @return
     */
    public static String getFileType(String filePaht) {
        String res = null;
        try {
            FileInputStream is = new FileInputStream(filePaht);
            byte[] b = new byte[10];
            is.read(b, 0, b.length);
            String fileCode = bytesToHexString(b);

            Iterator<String> keyIter = FILE_TYPE_MAP.keySet().iterator();
            while (keyIter.hasNext()) {
            String key = keyIter.next();
            // 验证前5个字符比较
            if (key.toLowerCase().startsWith(fileCode.toLowerCase().substring(0, 5))
                    || fileCode.toLowerCase().substring(0, 5).startsWith(key.toLowerCase())) {
                    res = FILE_TYPE_MAP.get(key);
                    break;
                }
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return res;
    }

    /**
      * 得到上传文件的文件头
      * @param src
      * @return
     */
    public static String bytesToHexString(byte[] src) {
        StringBuilder stringBuilder = new StringBuilder();
        if (src == null || src.length <= 0) {
            return null;
        }
        for (int i = 0; i < src.length; i++) {
            int v = src[i] & 0xFF;
            String hv = Integer.toHexString(v);
            if (hv.length() < 2) {
                stringBuilder.append(0);
             }
            stringBuilder.append(hv);
        }
        return stringBuilder.toString();
    }
}
