package com.yihu.jw.security.core.userdetails.jdbc;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.entity.base.login.BaseLoginLogDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.base.patient.BasePatientFamilyMemberDO;
import com.yihu.jw.entity.base.wx.BasePatientWechatDo;
import com.yihu.jw.entity.care.common.GetuiClientDO;
import com.yihu.jw.entity.hospital.family.WlyyPatientFamilyMemberDO;
import com.yihu.jw.security.core.userdetails.SaltUser;
import com.yihu.jw.security.dao.doctor.BaseDoctorDao;
import com.yihu.jw.security.dao.patient.BaseGetuiClientDao;
import com.yihu.jw.security.dao.patient.BasePatientDao;
import com.yihu.jw.security.dao.patient.BasePatientWechatDao;
import com.yihu.jw.security.dao.patient.WlyyPatientFamilyMemberDao;
import com.yihu.jw.security.model.WlyyUserDetails;
import com.yihu.jw.security.model.WlyyUserSimple;
import com.yihu.jw.security.service.HcyyService;
import com.yihu.jw.security.utils.IdCardUtil;
import com.yihu.jw.security.utils.ImUtil;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.http.HttpClientUtil;
import com.yihu.jw.util.encrypt.MD5;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateUtils;
import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.support.JdbcDaoSupport;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.provider.ClientDetails;
import org.springframework.security.oauth2.provider.ClientDetailsService;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.sql.DataSource;
import java.io.IOException;
import java.security.KeyPair;
import java.sql.Timestamp;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * Service - 用户信息
 * Created by progr1mmer on 2018/8/29.
 */
public class WlyyUserDetailsService extends JdbcDaoSupport implements UserDetailsService {

    private static final String DEFAULT_USER_DETAILS_STATEMENT = "select * from base_user u where u.username = ? or u.mobile = ? or u.idcard = ?";

    private static final String DEFAULT_DOCTOR_DETAILS_STATEMENT = "SELECT * FROM base_doctor d WHERE d.mobile = ? OR d.idcard = ?";

    private static final String DEFAULT_PATIENT_DETAILS_STATEMENT = "SELECT * FROM base_patient p WHERE p.mobile = ? OR p.idcard = ?";
    private static final String DEFAULT_PATIENT_DETAILS_IDCARD_STATEMENT = "SELECT * FROM base_patient p WHERE p.idcard = ? ";

    private static final String DEFAULT_PATIENT_INSERT_STATEMENT =
            "INSERT into base_patient (id,photo,idcard,password,salt,name,birthday,sex,mobile," +
                    "province_code,province_name,city_code,city_name,town_code,town_name,street_code,street_name,address," +
                    "del,locked,enabled,login_failure_count,login_date,card_type,alipay_id) " +
                    " values(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";

    private static final String PATIENT_INSERT_WEHCAT = "insert into base_patient_wechat (id,wechat_id,patient_id,openid,create_time) values(?,?,?,?,?)";

    private static final String UPDATE_PATIENT_CARD_TYPE = "update base_patient set card_type = ?,alipay_id = ? where id = ?";

    private static final String DEFAULT_FAMILY_STATEMENT = "SELECT * FROM base_patient_family_member p WHERE p.patient = ? and p.family_member = ?";

    private static final String DEFAULT_PATIENT_STATEMENT = "SELECT * FROM base_patient p WHERE p.id = ? ";


    private static final String BespeakRegist = "bespeakRegist:";

    @Autowired
    private StringRedisTemplate redisTemplate;
    @Value("${user.autoUnlockTime}")
    private Integer autoUnlockTime;//自动解锁时间 以分钟计算
    @Value("${user.reflashFailedCountTime}")
    private Integer reflashFailedCountTime;//将失败次数重置为0 ,以分钟计算
    @Value("${user.tryLoginTimes}")
    private Integer tryLoginTimes;//失败重试次数
    @Value("${wechat.id}")
    private String wxId;//微信id

    @Value("${iHealth.user-info-uri}")
    private String iHealthUserInfoUrl;//失败重试次数

    @Autowired
    private BasePatientWechatDao basePatientWechatDao;

    @Autowired
    private ClientDetailsService clientDetailsService;

    @Autowired
    private HttpClientUtil httpClientUtil;
    @Autowired
    private BasePatientDao patientDao;
    @Autowired
    private BaseDoctorDao baseDoctorDao;

    @Autowired
    private WlyyPatientFamilyMemberDao familyMemberDao;
    @Autowired
    private BaseGetuiClientDao baseGetuiClientDao;
    @Autowired
    private HcyyService hcyyService;
    @Autowired
    private ImUtil imUtil;
    @Value("${kick.eachOther}")
    private String kickEachOther;

    public WlyyUserDetailsService(DataSource dataSource) {
        this.setDataSource(dataSource);
    }

    /**
     * 用户登录判读接口
     * 判断loginType查找用户信息
     * 用户类型 1或默认为user，2：医生登录，3：患者登录
     *
     * @param username
     * @return
     * @throws UsernameNotFoundException
     */
    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        List<WlyyUserDetails> users = getWlyyUserDetails(username);
        if (users == null || users.size() == 0) {
            throw new UsernameNotFoundException(username);
        }
        if (null != users.get(0).isLocked() && users.get(0).isLocked()) {
            Date date = users.get(0).getLockedDate();
            if (new Date().after(DateUtils.addMinutes(date, autoUnlockTime))) {
                //解除锁定
                unlocked(username);
                users.get(0).setLocked(false);
            }
        }
        boolean isEnabled = null == users.get(0).isEnabled() ? false : users.get(0).isEnabled();
        boolean isLocked = null == users.get(0).isLocked() ? false : users.get(0).isLocked();
        return new SaltUser(username, users.get(0).getPassword(), users.get(0).getSalt(), isEnabled, isLocked, getGrantedAuthorities(username));
    }

    //医生注销注销
    @Transactional(rollbackFor = Exception.class)
    public void logOff(String id){
        baseDoctorDao.logOff(id);
    }



    public void unlocked(String username) {
        String loginType = getLogintype();
        if (StringUtils.isBlank(loginType) || "1".equals(loginType)) { //1或默认查找user表，为平台管理员账号
            //解除锁定
            this.getJdbcTemplate().update("update base_user u set login_failure_count = 0, u.locked = 0 where u.username = ? or u.mobile = ? or u.idcard = ?", username, username, username);
        } else if ("2".equals(loginType)) {//2.为医生账号
            //解除锁定
            this.getJdbcTemplate().update("update base_doctor d set d.login_failure_count = 0, d.locked = 0 where d.mobile = ? or d.idcard = ?", username, username);
        } else if ("3".equals(loginType)) { //3.患者账号
            //解除锁定
            this.getJdbcTemplate().update("update base_patient p set p.login_failure_count = 5, p.locked = 0 where p.mobile = ? or p.idcard = ?", username, username);
        } else if("4".equals(loginType)){
            this.getJdbcTemplate().update("update base_patient p set p.login_failure_count = 5, p.locked = 0 where p.mobile = ? or p.idcard = ?", username, getIdcard());
        }//...
    }

    /**
     * 用户登录判读接口
     * 判断loginType查找用户信息
     * 用户类型 1或默认为user，2：医生登录，3：患者登录，4：i健康系统-患者登录（i健康患者可以不注册直接登录互联网系统，首次使用，根据i健康居民code去i健康查询居民信息，然后保存到base.base_patient）
     * 用户类型 5 中山医院支付宝-患者登录（可以不注册直接登录） 6医养项目家人登录
     * @param username
     * @return
     */
    @Transactional
    public List<WlyyUserDetails> getWlyyUserDetails(String username) {

        String loginType = getLogintype();

        List<WlyyUserDetails> users = null;
        //1或默认查找user表，为平台管理员账号
        if (StringUtils.isBlank(loginType) || "1".equals(loginType)) {
            users = this.getJdbcTemplate().query(DEFAULT_USER_DETAILS_STATEMENT, new BeanPropertyRowMapper(WlyyUserDetails.class), username, username, username);
            //2.为医生登录账号
        } else if ("2".equals(loginType)) {
            users = this.getJdbcTemplate().query(DEFAULT_DOCTOR_DETAILS_STATEMENT, new BeanPropertyRowMapper(WlyyUserDetails.class), username, username);
            //3.患者登录
        } else if ("3".equals(loginType)) {
            //查找是否存在预注册 用户
            String userName = redisTemplate.opsForValue().get(BespeakRegist + username);
            if (StringUtils.isNotBlank(userName)) {
                //保存到数据库
                this.getJdbcTemplate().execute("update base_patient set del=1 where mobile='" + username + "'");
                redisTemplate.opsForValue().get(BespeakRegist + username);
                redisTemplate.delete(BespeakRegist + username);
            }
            users = this.getJdbcTemplate().query(DEFAULT_PATIENT_DETAILS_STATEMENT, new BeanPropertyRowMapper(WlyyUserDetails.class), username, username);
            // 第三方同步居民，端查询用户
        }else if("4".equals(loginType)){
            users = this.getJdbcTemplate().query(DEFAULT_PATIENT_DETAILS_STATEMENT, new BeanPropertyRowMapper(WlyyUserDetails.class),username,getIdcard());

            if (null == users || users.size() == 0) {
                String idcard = null;
                //厦门I健康账号同步
                if("xm_ihealth_wx".equals(getWechatId())){
                    List<NameValuePair> params = new ArrayList<>();
                    params.add(new BasicNameValuePair("clientId", getClientId()));
                    ClientDetails clientDetails = clientDetailsService.loadClientByClientId(getClientId());
                    params.add(new BasicNameValuePair("clientSecret", clientDetails.getClientSecret()));
                    params.add(new BasicNameValuePair("code", username));
                    params.add(new BasicNameValuePair("openid", getOpenid()));
                    String res = null;
                    try{
                        res = httpClientUtil.post(getSynPath(getWechatId()), params, "UTF-8");
                    }catch (Exception e){
                        logger.error("远程请求i健康用户信息异常：" + e.getMessage());
                        return users;
                    }
                    if(null == res){
                        return users;
                    }
                    JSONObject patient = JSONObject.parseObject(res);
                    if(patient.getInteger("status") == -1){
                        logger.error("去i健康查询用户信息失败：" + patient.getString("error"));
                        return users;
                    }
                    try {

                        String salt = randomString(5);
                        idcard = patient.getString("idcard");
                        String pw = idcard.substring(idcard.length()-6);

                        this.getJdbcTemplate().update(DEFAULT_PATIENT_INSERT_STATEMENT,
                                new Object[]{patient.getString("code"),
                                        patient.getString("photo"),
                                        patient.getString("idcard"),
                                        MD5.md5Hex(pw + "{" + salt + "}"),
                                        salt,
                                        patient.getString("name"),
                                        IdCardUtil.getBirthdayForIdcard(patient.getString("idcard")),
                                        Integer.parseInt(IdCardUtil.getSexForIdcard_new(patient.getString("idcard"))),
                                        patient.getString("mobile"),
                                        patient.getString("province"),
                                        patient.getString("provinceName"),
                                        patient.getString("city"),
                                        patient.getString("cityName"),
                                        patient.getString("town"),
                                        patient.getString("townName"),
                                        patient.getString("street"),
                                        patient.getString("streetName"),
                                        patient.getString("address"),
                                        1,
                                        0,
                                        1,
                                        0,
                                        new Date(),
                                        null,
                                        null
                                }
                        );
                    }catch (Exception e){
                        logger.error("将i健康居民数据同步到互联网医院居民表失败：" + e.getMessage());
                        return users;
                    }
                    try {
                        this.getJdbcTemplate().update(PATIENT_INSERT_WEHCAT,
                                new Object[]{
                                        UUID.randomUUID().toString(),
                                        getWechatId(),
                                        patient.getString("code"),
                                        getOpenid(),
                                        new Date()
                                }
                        );
                    }catch (Exception e){
                        logger.error("将i健康居民微信openid数据同步到互联网医院居民微信关联表失败：" + e.getMessage());
                        return users;
                    }
                    if (wxId.equalsIgnoreCase("xm_zsyy_wx")){
                        logger.info("获取中山医院openid：=====");
                        try {
                            this.getJdbcTemplate().update(PATIENT_INSERT_WEHCAT,
                                    new Object[]{
                                            UUID.randomUUID().toString(),
                                            wxId,
                                            patient.getString("code"),
                                            getZsyyOpenid(idcard),
                                            new Date()
                                    }
                            );
                        }catch (Exception e){
                            logger.error("将i健康居民转为中山医院openid数据同步到互联网医院居民微信关联表失败：" + e.getMessage());
                            return users;
                        }
                    }
                    WlyyUserDetails user = new WlyyUserDetails();
                    user.setName(patient.getString("name"));
                    user.setPassword(patient.getString("password"));
                    if(patient.getInteger("sex") == 1){
                        user.setGender(WlyyUserDetails.Gender.male);
                    }else if(patient.getInteger("sex") == 2){
                        user.setGender(WlyyUserDetails.Gender.female);
                    }
                    user.setId(username);
                    user.setIdcard(patient.getString("idcard"));
                    user.setMobile(patient.getString("mobile"));
                    user.setEnabled(true);
                    user.setLocked(false);
                    user.setLockedDate(null);
                    users.add(user);
                    BasePatientDO patientDO = patientDao.findByIdcardAndDel(idcard,"1");
                    WlyyPatientFamilyMemberDO basePatientFamilyMemberDO = familyMemberDao.findFamilyMemberByPatientAndRelationCode(patientDO.getId(),"7");
                    if (basePatientFamilyMemberDO==null){
                        basePatientFamilyMemberDO = new WlyyPatientFamilyMemberDO();
                        basePatientFamilyMemberDO.setPatient(patientDO.getId());
                        basePatientFamilyMemberDO.setFamilyRelation("7");
                        basePatientFamilyMemberDO.setFamilyRelationName("自己");
                        basePatientFamilyMemberDO.setCardType("身份证");
                        basePatientFamilyMemberDO.setCardNo(patientDO.getIdcard());
                        basePatientFamilyMemberDO.setCreateTime(new Date());
                        basePatientFamilyMemberDO.setUpdateTime(new Date());
                        basePatientFamilyMemberDO.setIsAuthorize(1);
                        basePatientFamilyMemberDO.setIsDel(1);
                        basePatientFamilyMemberDO.setFamilyMember(patientDO.getId());
                        familyMemberDao.save(basePatientFamilyMemberDO);
                    }
                }else if("xm_zsyy_wx".equals(getWechatId())){

                    JSONObject jsonObject=new JSONObject();
                    jsonObject.put("openId",getOpenid());
                    String res = null;
                    try{
                        res = httpClientUtil.sendPost(getSynPath("xm_zsyy_wx"), jsonObject.toString());
                    }catch (Exception e){
                        logger.error("远程请求中山医院用户信息异常：" + e.getMessage());
                        return users;
                    }
                    if(null == res){
                        return users;
                    }

                    JSONObject object1 = JSONObject.parseObject(res);
                    logger.info("object1"+object1.toJSONString());
                    if (null != object1&&null!=object1.get("status") && "200".equals(object1.get("status").toString())) {
                        JSONObject patient = object1.getJSONObject("userinfo");
                        String salt = randomString(5);
                        idcard = patient.getString("idcard");
                        String idtype = patient.getString("idtype");
                        String mobile = patient.getString("mobile");
                        String pw = mobile.substring(mobile.length()-6);
                        try {
                            this.getJdbcTemplate().update(DEFAULT_PATIENT_INSERT_STATEMENT,
                                    new Object[]{patient.getString("id"),
                                            null,
                                            patient.getString("idcard"),
                                            MD5.md5Hex(pw + "{" + salt + "}"),
                                            salt,
                                            patient.getString("name"),
                                            IdCardUtil.getBirthdayForIdcard(patient.getString("idcard")),
                                            Integer.parseInt(IdCardUtil.getSexForIdcard_new(patient.getString("idcard"))),
                                            mobile,
                                            null,
                                            null,
                                            null,
                                            null,
                                            null,
                                            null,
                                            null,
                                            null,
                                            null,
                                            1,
                                            0,
                                            1,
                                            0,
                                            new Date(),
                                            idtype,
                                            null
                                    }
                            );
                        }catch (Exception e){
                            logger.error("将中山医院居民数据同步到互联网医院居民表失败：" + e.getMessage());
                            return users;
                        }

                        WlyyUserDetails user = new WlyyUserDetails();
                        user.setName(patient.getString("name"));
                        user.setPassword(MD5.md5Hex(pw + "{" + salt + "}"));
                        if(patient.getInteger("sex") == 1){
                            user.setGender(WlyyUserDetails.Gender.male);
                        }else if(patient.getInteger("sex") == 2){
                            user.setGender(WlyyUserDetails.Gender.female);
                        }
                        user.setId(patient.getString("id"));
                        user.setIdcard(patient.getString("idcard"));
                        user.setMobile(patient.getString("mobile"));
                        user.setEnabled(true);
                        user.setLocked(false);
                        user.setLockedDate(null);
                        user.setCardType(idcard);
                        users.add(user);

                    }else{
                        logger.error("远程请求中山医院用户信息异常：" + res.toString());
                        return users;
                    }

                }
                BasePatientDO patientDO = patientDao.findByIdcardAndDel(idcard,"1");
                WlyyPatientFamilyMemberDO basePatientFamilyMemberDO = familyMemberDao.findFamilyMemberByPatientAndRelationCode(patientDO.getId(),"7");
                if (basePatientFamilyMemberDO==null){
                    basePatientFamilyMemberDO = new WlyyPatientFamilyMemberDO();
                    basePatientFamilyMemberDO.setPatient(patientDO.getId());
                    basePatientFamilyMemberDO.setFamilyRelation("7");
                    basePatientFamilyMemberDO.setFamilyRelationName("自己");
                    basePatientFamilyMemberDO.setCardType("身份证");
                    basePatientFamilyMemberDO.setCardNo(patientDO.getIdcard());
                    basePatientFamilyMemberDO.setCreateTime(new Date());
                    basePatientFamilyMemberDO.setUpdateTime(new Date());
                    basePatientFamilyMemberDO.setIsAuthorize(1);
                    basePatientFamilyMemberDO.setIsDel(1);
                    basePatientFamilyMemberDO.setFamilyMember(patientDO.getId());
                    familyMemberDao.save(basePatientFamilyMemberDO);
                }
            }
        }else if("5".equals(loginType)){
            //中山医院支付宝-患者登录（可以不注册直接登录）
            // 姓名，电话，证件类型，证件号和⽀付宝open_id以冒号组合，如：
            //张三:13012341234:01:350000200112231234:123456
            String data[] = username.split(":");
            String name = data[0];
            username = data[1];
            String idcard = data[3];
            users = this.getJdbcTemplate().query(DEFAULT_PATIENT_DETAILS_STATEMENT, new BeanPropertyRowMapper(WlyyUserDetails.class),username,idcard);
            if (null == users || users.size() == 0) {
                String patientId = UUID.randomUUID().toString();
                String salt = randomString(5);
                String pw = idcard.substring(idcard.length()-6);
                String password = MD5.md5Hex(pw + "{" + salt + "}");
                int sex = 0;
                try {
                    sex = Integer.parseInt(IdCardUtil.getSexForIdcard_new(idcard));
                    this.getJdbcTemplate().update(DEFAULT_PATIENT_INSERT_STATEMENT,
                            new Object[]{patientId,
                                    null,
                                    idcard,
                                    password,
                                    salt,
                                    name,
                                    IdCardUtil.getBirthdayForIdcard(idcard),
                                    sex,
                                    username,
                                    null,
                                    null,
                                    null,
                                    null,
                                    null,
                                    null,
                                    null,
                                    null,
                                    null,
                                    1,
                                    0,
                                    1,
                                    0,
                                    new Date(),
                                    data[2],
                                    data[4]
                            }
                    );
                }catch (Exception e){
                    logger.error("将中山医院支付宝登录数据同步到互联网医院居民表失败：" + e.getMessage());
                    return users;
                }
                try {
                    this.getJdbcTemplate().update(PATIENT_INSERT_WEHCAT,
                            new Object[]{
                                    UUID.randomUUID().toString(),
                                    getWechatId(),
                                    patientId,
                                    getOpenid(),
                                    new Date()
                            }
                    );
                }catch (Exception e){
                    logger.error("将中山医院支付宝登录数据同步到互联网医院居民微信关联表失败：" + e.getMessage());
                    return users;
                }

                WlyyUserDetails user = new WlyyUserDetails();
                user.setName(name);
                user.setPassword(password);
                if(sex == 1){
                    user.setGender(WlyyUserDetails.Gender.male);
                }else if(sex == 2){
                    user.setGender(WlyyUserDetails.Gender.female);
                }
                user.setId(username);
                user.setIdcard(idcard);
                user.setMobile(username);
                user.setEnabled(true);
                user.setLocked(false);
                user.setLockedDate(null);
                users.add(user);
                BasePatientDO patientDO = patientDao.findByIdcardAndDel(idcard,"1");
                WlyyPatientFamilyMemberDO basePatientFamilyMemberDO = familyMemberDao.findFamilyMemberByPatientAndRelationCode(patientDO.getId(),"7");
                if (basePatientFamilyMemberDO==null){
                    basePatientFamilyMemberDO = new WlyyPatientFamilyMemberDO();
                    basePatientFamilyMemberDO.setPatient(patientDO.getId());
                    basePatientFamilyMemberDO.setFamilyRelation("7");
                    basePatientFamilyMemberDO.setFamilyRelationName("自己");
                    basePatientFamilyMemberDO.setCardType("身份证");
                    basePatientFamilyMemberDO.setCardNo(patientDO.getIdcard());
                    basePatientFamilyMemberDO.setCreateTime(new Date());
                    basePatientFamilyMemberDO.setUpdateTime(new Date());
                    basePatientFamilyMemberDO.setIsAuthorize(1);
                    basePatientFamilyMemberDO.setIsDel(1);
                    basePatientFamilyMemberDO.setFamilyMember(patientDO.getId());
                    familyMemberDao.save(basePatientFamilyMemberDO);
                }
            }else{
                this.getJdbcTemplate().update(UPDATE_PATIENT_CARD_TYPE,
                        new Object[]{
                                data[2],
                                data[4],
                                users.get(0).getId()
                        }
                );
            }
        }else if("6".equals(loginType)) {
            //第三方登陆
            users = this.getJdbcTemplate().query(DEFAULT_PATIENT_DETAILS_STATEMENT, new BeanPropertyRowMapper(WlyyUserDetails.class), username, username);
        }else if ("7".equals(loginType)) {
            String data[] = username.split(":");
            String patientId = data[0];
            String familyPatientId = data[2];
            List<BasePatientFamilyMemberDO> familys =this.getJdbcTemplate().query(DEFAULT_FAMILY_STATEMENT, new BeanPropertyRowMapper(BasePatientFamilyMemberDO.class), patientId, familyPatientId);
            if(familys==null||familys.size()==0){
                return users;
            }
            username = data[1];
            //查找是否存在预注册 用户
            String userName = redisTemplate.opsForValue().get(BespeakRegist + username);
            if (StringUtils.isNotBlank(userName)) {
                //保存到数据库
                this.getJdbcTemplate().execute("update base_patient set del=1 where mobile='" + username + "'");
                redisTemplate.opsForValue().get(BespeakRegist + username);
                redisTemplate.delete(BespeakRegist + username);
            }
            users = this.getJdbcTemplate().query(DEFAULT_PATIENT_DETAILS_STATEMENT, new BeanPropertyRowMapper(WlyyUserDetails.class), username, username);
        }
        return users;
    }

    /**
     * 设置用户登录时间，返回登录信息
     * 判断loginType，用户类型 1或默认为user，2：医生登录，3：患者登录
     *
     * @param username
     * @return
     * @throws UsernameNotFoundException
     */
    public WlyyUserSimple authSuccess(String username) throws UsernameNotFoundException {
        //加载用户简略信息
        List<WlyyUserSimple> users = getWlyyUserSimple(username);
        if (users == null || users.size() == 0) {
            throw new UsernameNotFoundException(username);
        }
        return users.get(0);
    }

    public String authFailure() throws UsernameNotFoundException {
        //获取失败次数
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String username = request.getParameter("username");
        Map<String, Object> map = getLoginFailureCount(username);
        Integer loginFailureCount = null == map.get("login_failure_count") ? 0 : Integer.valueOf(map.get("login_failure_count").toString());
        Object timeObj = map.get("last_login_failure_time");
        if (timeObj == null) {
            loginFailureCount = 0;
        } else {
            Timestamp timestamp = (Timestamp) timeObj;
            //判断是否得重置登陆次数
            Date date = new Date(timestamp.getTime());
            if (new Date().after(DateUtils.addMinutes(date, reflashFailedCountTime))) {
                //重置登陆次数
                loginFailureCount = 0;
            }
        }
        loginFailureCount++;
        if (loginFailureCount >= tryLoginTimes) {
            locked(username);
            return "账号已被锁定,请" + autoUnlockTime + "分钟后重试!";
        } else {
            addFailureCount(username, loginFailureCount);
            return "账号或密码错误,还可以再试" + (tryLoginTimes - loginFailureCount) + "次!";
        }
    }

    /**
     * 获取失败次数 返回 map  key1: login_failure_count   失败次数
     * key2: last_login_failure_time 上次登录失败时间 /可能为null
     *
     * @param username
     * @return
     */
    public Map<String, Object> getLoginFailureCount(String username) {
        Map<String, Object> result = new HashMap<>();
        String loginType = getLogintype();
        if (StringUtils.isBlank(loginType) || "1".equals(loginType)) { //1或默认查找user表，为平台管理员账号
            result = this.getJdbcTemplate().queryForMap("select u.login_failure_count,u.last_login_failure_time from base_user u where u.username = ? or u.mobile = ? or u.idcard = ?", username, username, username);
        } else if ("2".equals(loginType)) {//2.为医生账号
            result = this.getJdbcTemplate().queryForMap("select d.login_failure_count,d.last_login_failure_time from base_doctor d where d.mobile = ? or d.idcard = ?", username, username);
        } else if ("3".equals(loginType)) { //3.患者账号
            result = this.getJdbcTemplate().queryForMap("select p.login_failure_count,p.last_login_failure_time from base_patient p where p.mobile = ? or p.idcard = ? limit 1", username, username);
        } else if ("4".equals(loginType)) { //4.第三方患者账号
            result = this.getJdbcTemplate().queryForMap("select p.login_failure_count,p.last_login_failure_time from base_patient p where p.mobile = ? or p.idcard = ?", username, getIdcard());
        }
        return result;
    }

    /**
     * 锁定账号
     *
     * @param username
     */
    public void locked(String username) {
        String loginType = getLogintype();
        if (StringUtils.isBlank(loginType) || "1".equals(loginType)) { //1或默认查找user表，为平台管理员账号
            //账号锁定
            this.getJdbcTemplate().update("update base_user u set u.login_failure_count = 5, u.locked = 1, u.locked_date = ? where u.username = ? or u.mobile = ? or u.idcard = ?", new Date(), username, username, username);
        } else if ("2".equals(loginType)) {//2.为医生账号
            //账号锁定
            this.getJdbcTemplate().update("update base_doctor d set d.login_failure_count = 5, d.locked = 1, d.locked_date = ? where d.mobile = ? or d.idcard = ?", new Date(), username, username);
        } else if ("3".equals(loginType)) { //3.患者账号
            //账号锁定
            this.getJdbcTemplate().update("update base_patient p set p.login_failure_count = 5, p.locked = 1, p.locked_date = ?  where p.mobile = ? or p.idcard = ?", new Date(), username, username);
        }else if ("4".equals(loginType)) { //3.同步的患者账号
            //账号锁定
            this.getJdbcTemplate().update("update base_patient p set p.login_failure_count = 5, p.locked = 1, p.locked_date = ?  where p.mobile = ? or p.idcard = ?", new Date(), username, getIdcard());
        }  //...
    }

    /**
     * 更新登录失败次数
     *
     * @param username
     * @param loginFailureCount
     */
    public void addFailureCount(String username, Integer loginFailureCount) {
        String loginType = getLogintype();
        if (StringUtils.isBlank(loginType) || "1".equals(loginType)) { //1或默认查找user表，为平台管理员账号
            //更新失败次数
            this.getJdbcTemplate().update("update base_user u set u.login_failure_count = ? ,u.last_login_failure_time = ? where u.username = ? or u.mobile = ? or u.idcard = ?", loginFailureCount, new Date(), username, username, username);
        } else if ("2".equals(loginType)) {//2.为医生账号
            //更新失败次数
            this.getJdbcTemplate().update("update base_doctor d set d.login_failure_count = ?, d.last_login_failure_time = ?  where d.mobile = ? or d.idcard = ?", loginFailureCount, new Date(), username, username);
        } else if ("3".equals(loginType)) { //3.患者账号
            //更新失败次数
            this.getJdbcTemplate().update("update base_patient p set p.login_failure_count = ? ,p.last_login_failure_time = ? where p.mobile = ? or p.idcard = ?", loginFailureCount, new Date(), username, username);
        } else if ("4".equals(loginType)) { //4.第三方同步患者账号
            //更新失败次数
            this.getJdbcTemplate().update("update base_patient p set p.login_failure_count = ? ,p.last_login_failure_time = ? where p.mobile = ? or p.idcard = ?", loginFailureCount, new Date(), username, getIdcard());
        } //...
    }

    /**
     * 更新个推clientId 和老人平板登录信息
     * @param clientId
     * @param padImei
     * @param loginType
     * @param id
     */
    public void updateInfo(String clientId,String clientType,String padImei,String loginType,String id,BaseLoginLogDO baseLoginLogDO){
        int isPateint = isPatient(loginType);
        if(StringUtils.isNotBlank(padImei)){
            try {
                if(isPateint == 1){
                    this.getJdbcTemplate().update("update base_patient p set p.pad_imei = ? where p.id = ? or p.idcard = ?", padImei, id,id);
                }
            }catch (Exception e){
                e.printStackTrace();
            }
        }
        if(StringUtils.isNotBlank(clientId)&&StringUtils.isNotBlank(clientType)&&isPateint<3){
            try {
                //先删除在新增
                deleteCidByUser(id);
                //删除clientId,避免收到其他人的
                deleteCidByCid(clientId);

                GetuiClientDO getuiClientDO = new GetuiClientDO();
                getuiClientDO.setClientId(clientId);
                getuiClientDO.setType(isPateint);
                getuiClientDO.setClientType(clientType);
                getuiClientDO.setUser(id);
                getuiClientDO.setCreateTime(new Date());
                baseGetuiClientDao.save(getuiClientDO);
            }catch (Exception e){
                e.printStackTrace();
            }
        }
        if(StringUtils.isNotBlank(baseLoginLogDO.getClientType())&&"0".equals(kickEachOther)){
            try {
                //pc管理端端推送消息
                JSONObject message = new JSONObject();
                //陈XXX登录云照护平台   登录终端：多媒体平板   2021-7-7 16:38:25
                String content = baseLoginLogDO.getName() + "登录云照护平台 登录终端：" + getClientName(baseLoginLogDO.getClientType())
                        + " " + DateUtil.dateToStrLong(baseLoginLogDO.getCreateTime());
                message.put("content",content);
                message.put("content_type",41);
                imUtil.sendPcManageMessageToPc("cloudCare_pcManage",message.toString());
            }catch (Exception e){
                e.printStackTrace();
            }
        }

    }

    public void deleteCidByUser(String user){
        try {
            String sql = "delete from base_getui_client where user='"+user+"'";
            getJdbcTemplate().execute(sql);
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    public void deleteCidByCid(String cid){
        try {
            String sql = "delete from base_getui_client where client_id='"+cid+"'";
            getJdbcTemplate().execute(sql);
        }catch (Exception e){
            e.printStackTrace();
        }
    }


    /**
     * app,wx,pad,pc
     * @param clientType
     * @return
     */
    private String getClientName(String clientType){
        if(StringUtils.isBlank(clientType)){
            return "";
        }
        if("app".equals(clientType)){
            return "APP";
        }
        if("wx".equals(clientType)){
            return "微信公众号";
        }
        if("pad".equals(clientType)){
            return "多媒体平板";
        }
        if("pc".equals(clientType)){
            return "电脑脑端";
        }
        return "";
    }

    /**
     * 根据登录类型判断登录是医生2 ，居民1 还是后台管理员 3
     * @param loginType
     * @return
     */
    public int isPatient(String loginType){
        if(StringUtils.isBlank(loginType) || "1".equals(loginType)){
            return 3;
        }
        if("2".equals(loginType)){
            return 2;
        }
        return 1;
    }

    private Collection<? extends GrantedAuthority> getGrantedAuthorities(String username) {
        Collection<GrantedAuthority> authorities = new ArrayList<>(1);
        authorities.add(new SimpleGrantedAuthority("ROLE_USER"));
        return authorities;
    }

    /**
     * 设置用户登录时间，返回登录信息
     * 判断loginType，用户类型 1或默认为user，2：医生登录，3：患者登录
     *
     * @param username
     * @return
     */
    public List<WlyyUserSimple> getWlyyUserSimple(String username) {

        String loginType = getLogintype();

        List<WlyyUserSimple> users = null;
        logger.info("login:登录进入5");
        //1或默认查找user表，为平台管理员账号
        if (StringUtils.isBlank(loginType) || "1".equals(loginType)) {
            //更新登录时间
            this.getJdbcTemplate().update("update base_user u set u.login_failure_count = 0, u.login_date = ? where u.username = ? or u.mobile = ? or u.idcard = ?", new Date(), username, username, username);
            users = this.getJdbcTemplate().query(DEFAULT_USER_DETAILS_STATEMENT, new BeanPropertyRowMapper(WlyyUserSimple.class), username, username, username);
            //2.为医生登录账号
        } else if ("2".equals(loginType)) {
            //更新登录时间
            this.getJdbcTemplate().update("update base_doctor d set d.login_failure_count = 0, d.login_date = ? where d.mobile = ? or d.idcard = ?", new Date(), username, username);
            users = this.getJdbcTemplate().query(DEFAULT_DOCTOR_DETAILS_STATEMENT, new BeanPropertyRowMapper(WlyyUserSimple.class), username, username);
            //3.患者登录
        } else if ("3".equals(loginType)) {
            //更新登录时间
            this.getJdbcTemplate().update("update base_patient p set p.login_failure_count = 0, p.login_date = ? where p.mobile = ? or p.idcard = ?", new Date(), username, username);
            users = this.getJdbcTemplate().query(DEFAULT_PATIENT_DETAILS_STATEMENT, new BeanPropertyRowMapper(WlyyUserSimple.class), username, username);
        } else if("4".equals(loginType)){
            //更新登录时间
            this.getJdbcTemplate().update("update base_patient p set p.login_failure_count = 0, p.login_date = ? where p.mobile = ? or p.idcard = ?", new Date(), username, getIdcard());
            users = this.getJdbcTemplate().query(DEFAULT_PATIENT_DETAILS_IDCARD_STATEMENT, new BeanPropertyRowMapper(WlyyUserSimple.class), getIdcard());
        }else if("5".equals(loginType)){
            //更新登录时间
            this.getJdbcTemplate().update("update base_patient p set p.login_failure_count = 0, p.login_date = ? where p.mobile = ? or p.idcard = ?", new Date(), username, username);
            users = this.getJdbcTemplate().query(DEFAULT_PATIENT_DETAILS_STATEMENT, new BeanPropertyRowMapper(WlyyUserSimple.class), username,username);
        }else if("6".equals(loginType)) {
            //第三方登陆
            this.getJdbcTemplate().update("update base_patient p set p.login_failure_count = 0, p.login_date = ? where p.mobile = ? or p.idcard = ?", new Date(), username, username);
            users = this.getJdbcTemplate().query(DEFAULT_PATIENT_DETAILS_STATEMENT, new BeanPropertyRowMapper(WlyyUserSimple.class), username,username);
        } else if ("7".equals(loginType)) {
            //更新登录时间
            this.getJdbcTemplate().update("update base_patient p set p.login_failure_count = 0, p.login_date = ? where p.mobile = ? or p.idcard = ?", new Date(), username, username);
            users = this.getJdbcTemplate().query(DEFAULT_PATIENT_DETAILS_STATEMENT, new BeanPropertyRowMapper(WlyyUserSimple.class), username, username);
        }
        logger.info("login:登录进入6");

        return users;
    }

    /**
     * 获取用户登录类型
     *
     * @return
     */
    public String getLogintype() {

        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();

        String loginType = request.getParameter("login_type");

        return loginType;
    }

    /**
     * 获取i健康用户登录openid型
     *
     * @return
     */
    public String getOpenid() {

        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();

        String openid = request.getParameter("openid");

        return openid;
    }

    /**
     * 获取i健康用户登录openid型
     *
     * @return
     */
    public String getZsyyOpenid(String idcard) throws IOException {
        String openid = "";
        String url = "https://wx.xmzsh.com/interface/queryOpenIdByIdCard.htm";
        JSONObject object = new JSONObject();
        object.put("idCard",idcard);
        String response = httpClientUtil.sendPost(url,object.toJSONString());
        logger.info("获取中山医院openid："+response);
        if (StringUtils.isNotBlank(response)){
            JSONObject jsonObject = JSONObject.parseObject(response);
            if (jsonObject.getString("respCode").equalsIgnoreCase("00")){
                JSONObject respEntityJson = jsonObject.getJSONObject("respEntity");
                openid = respEntityJson.getString("openId");
            }
        }
        return openid;
    }

    /**
     * 获取用户登录的clientId
     *
     * @return
     */
    public String getClientId() {

        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();

        String clientId = request.getParameter("client_id");

        return clientId;
    }

    /**
     * 获取用户登录的wechatId
     *
     * @return
     */
    public String getWechatId() {

        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();

        String wechatId = request.getParameter("wechatId");

        return wechatId;
    }

  /**
     * 获取用户登录的idcard
     *
     * @return
     */
    public String getIdcard() {

        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        KeyPair keyPair = (KeyPair) request.getSession().getAttribute("privateKey");
        String idcard = com.yihu.jw.security.utils.RSAUtils.decryptBase64(request.getParameter("idcard"), keyPair);
        logger.info("idcard"+idcard);
        return idcard;
    }

    public static List<String> loginTypeList = new ArrayList<>();
    static {
        loginTypeList.add("1");//user 登录
        loginTypeList.add("2");//医生登录
        loginTypeList.add("3");//居民登录
        loginTypeList.add("4");//第三方同步患者账号
        loginTypeList.add("5");//支付宝登录
        loginTypeList.add("6");//第三方登录-医联康护
        loginTypeList.add("7");//家人登录
        //loginTypeList.add("8");//4k 大屏登入
    }

    public boolean setRolePhth(String loginType, OAuth2AccessToken token, String id, StringRedisTemplate redisTemplate) {

        if (org.apache.commons.lang.StringUtils.isBlank(loginType) || loginTypeList.contains(loginType)) { //1或默认查找user表，为平台管理员账号
            String key = "wlyy2:auth:token:" + token.getValue();
            redisTemplate.opsForValue().set(key, "/**");
            redisTemplate.expire(key, token.getExpiresIn(), TimeUnit.SECONDS);
            return true;
        }
        return false;
    }

    public void updateOpenId(String openid, String userId) {
        this.getJdbcTemplate().update("update base_patient p set p.openid = ? where p.id= ?", openid, userId);
    }

    public void updateDoctorOpenId(String openid, String userId) {
        this.getJdbcTemplate().update("update base_doctor p set p.openid = ? where p.id= ?", openid, userId);
    }

    public void updateOpenIdAndWechatId(String openid, String userId,String wechatId) {
        List<BasePatientWechatDo> patientWechatDos = basePatientWechatDao.findByWechatIdAndPatientId(wechatId,userId);
        if (patientWechatDos!=null&&patientWechatDos.size()!=0){
            BasePatientWechatDo basePatientWechatDo = patientWechatDos.get(0);
            basePatientWechatDo.setOpenid(openid);
            basePatientWechatDao.save(basePatientWechatDo);
        }else {
            BasePatientWechatDo basePatientWechatDo=new BasePatientWechatDo();
            basePatientWechatDo.setCreateTime(new Date());
            basePatientWechatDo.setOpenid(openid);
            basePatientWechatDo.setPatientId(userId);
            basePatientWechatDo.setWechatId(wechatId);
            basePatientWechatDao.save(basePatientWechatDo);
        }
    }

    public String getSynPath(String wxId){
        String sql = "SELECT w.syn_path AS sysPath FROM wx_wechat w WHERE w.id ='"+wxId+"'";

        List<Map<String,Object>> list = this.getJdbcTemplate().queryForList(sql);

        if(list!=null&&list.size()>0){
            return (String)list.get(0).get("sysPath");
        }else{
            logger.error("getSynPath error");
        }
        return "";
    }

    public String randomString(int length) {
        String str = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
        StringBuffer buffer = new StringBuffer();
        Random random = new Random();

        for(int i = 0; i < length; ++i) {
            int pos = random.nextInt(str.length());
            buffer.append(str.charAt(pos));
        }

        return buffer.toString();
    }

    public JSONObject getHLWyyUser(String openid){
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("openId",openid);
        try {
            String res = httpClientUtil.sendPost(getSynPath("xm_zsyy_wx"), jsonObject.toString());
            JSONObject object1 = JSONObject.parseObject(res);
            if (null != object1&&null!=object1.get("status") && "200".equals(object1.get("status").toString())) {
                JSONObject patient = object1.getJSONObject("userinfo");
                logger.info("开始同步患者数据");
                String idcard = patient.getString("idcard");
                String idtype = patient.getString("idtype");
                BasePatientDO patientDO2 = patientDao.findByIdcardAndDel(idcard,"1");
                BasePatientDO patient1 = new BasePatientDO();
                String salt = UUID.randomUUID().toString().substring(0,5);
                String mobile = patient.getString("mobile");
                String pw = null;
                if(org.apache.commons.lang3.StringUtils.isNotBlank(mobile)){
                    pw = mobile.substring(mobile.length()-6);
                }else{
                    pw = idcard.substring(idcard.length()-6);
                }
                if (patientDO2!=null){
                    patient1=patientDO2;
                }else {
                    patient1.setPassword(MD5.md5Hex(pw + "{" + salt + "}"));
                    patient1.setSalt(salt);
                }
                patient1.setIdcard(idcard);
                patient1.setCardType(idtype);
                patient1.setName(patient.getString("name"));
                patient1.setSex(StringUtils.isNotBlank(idcard)?Integer.parseInt(IdCardUtil.getSexForIdcard_new(idcard)):null);
                patient1.setMobile(mobile);
                patient1.setDel("1");
                patient1.setEnabled(1);
                patient1.setLocked(0);
                patient1.setCreateTime(new Date());
                patient1.setUpdateTime(new Date());
                patient1.setBirthday(IdCardUtil.getBirthdayForIdcard(idcard));
                patient1 = patientDao.save(patient1);
                logger.info("结束同步患者数据");
                return patient;
            }
        }catch (Exception e){
            logger.error(e);
        }
        return null;
    }

    public JSONObject getHcHLWyyUser(String openid,String memberId){
        JSONObject patient = new JSONObject();
        try {
            String result = hcyyService.queryMemberByOpenId(openid,memberId);
            JSONObject jsonObject = JSONObject.parseObject(result);
            String RespCode = jsonObject.getString("RespCode");
            BasePatientDO patient1 = new BasePatientDO();
            if (RespCode.equalsIgnoreCase("10000")){
                JSONArray array = jsonObject.getJSONArray("Data");
                if (array!=null&&array.size()!=0){
                    patient = array.getJSONObject(0);

                    String idcard = patient.getString("IdCardNo");
                    BasePatientDO patientDO2 = patientDao.findByIdcardAndDel(idcard,"1");

                    String salt = UUID.randomUUID().toString().substring(0,5);
                    String mobile = patient.getString("Mobile");
                    String pw = null;
                    if(org.apache.commons.lang3.StringUtils.isNotBlank(mobile)){
                        pw = mobile.substring(mobile.length()-6);
                    }else{
                        pw = idcard.substring(idcard.length()-6);
                    }
                    if (patientDO2!=null){
                        patient1=patientDO2;
                    }else {
                        patient1.setPassword(MD5.md5Hex(pw + "{" + salt + "}"));
                        patient1.setSalt(salt);
                    }
                    patient1.setIdcard(idcard);
                    patient1.setCardType(patient.getString("CardType"));
                    patient1.setName(patient.getString("MemberName"));
                    patient1.setSex(StringUtils.isNotBlank(idcard)?Integer.parseInt(IdCardUtil.getSexForIdcard_new(idcard)):null);
                    patient1.setMobile(mobile);
                    patient1.setDel("1");
                    patient1.setEnabled(1);
                    patient1.setLocked(0);
                    patient1.setCreateTime(new Date());
                    patient1.setUpdateTime(new Date());
                    patient1.setBirthday(IdCardUtil.getBirthdayForIdcard(idcard));
                    patient1 = patientDao.save(patient1);
                    List<BasePatientWechatDo> patientWechatDoList = basePatientWechatDao.findByWechatIdAndPatientId("xm_hcyy_wx",patient1.getId());
                    basePatientWechatDao.deleteAll(patientWechatDoList);
                    BasePatientWechatDo patientWechatDo = new BasePatientWechatDo();
                    patientWechatDo.setOpenid(openid);
                    patientWechatDo.setPatientId(patient1.getId());
                    patientWechatDo.setWechatId("xm_hcyy_wx");
                    patientWechatDo.setCreateTime(new Date());
                    basePatientWechatDao.save(patientWechatDo);
                    logger.info("结束同步患者数据");
                }
            }

                return patient;
        }catch (Exception e){
            logger.error(e);
        }
        return null;
    }

    public static void main(String arg[]){

        String ps = MD5.md5Hex("123456" + "{" + "wyIlL" + "}");
        System.out.println(ps);
    }
}
