package com.yihu.jw.security.service;

import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.base.patient.PatientMedicareCardDO;
import com.yihu.jw.entity.base.wx.BasePatientWechatDo;
import com.yihu.jw.entity.hospital.family.WlyyPatientFamilyMemberDO;
import com.yihu.jw.security.dao.doctor.BaseDoctorDao;
import com.yihu.jw.security.dao.patient.BasePatientDao;
import com.yihu.jw.security.dao.patient.BasePatientMedicareCardDao;
import com.yihu.jw.security.dao.patient.BasePatientWechatDao;
import com.yihu.jw.security.dao.patient.WlyyPatientFamilyMemberDao;
import com.yihu.jw.security.utils.IdCardUtil;
import com.yihu.jw.util.encrypt.MD5;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * Created by Trick on 2020/3/19.
 */
@Service
@Transactional
public class RegisterService {

    @Autowired
    private BasePatientDao basePatientDao;

    @Autowired
    private BasePatientMedicareCardDao basePatientMedicareCardDao;

    @Autowired
    private BasePatientWechatDao basePatientWechatDao;

    @Autowired
    private BaseDoctorDao baseDoctorDao;
    @Autowired
    private WlyyPatientFamilyMemberDao familyMemberDao;

    @Autowired
    private StringRedisTemplate redisTemplate;
    @Value("${wechat.id}")
    private String wechatId;
    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Autowired
    private YkyyService ykyyService;

    public Map<String,Object> registerPatient(String name, String mobile, String idcard, String ssc, String pw,String openid,String wxId,String captcha)throws Exception{

        Map<String,Object> rs = new HashedMap();
        try{
            BasePatientDO p1 = basePatientDao.findByIdcardAndDel(idcard,"1");
            if(p1!=null){
                rs.put("code","-1");
                rs.put("mes","身份证已经存在");
                return rs;
            }

            List<BasePatientDO> p2 = basePatientDao.findByMobileAndDel(mobile,"1");
            if(p2!=null&&p2.size()>0){
                rs.put("code","-2");
                rs.put("mes","手机号已经存在");
                return rs;
            }

            BasePatientDO patient = new BasePatientDO();

            String salt = UUID.randomUUID().toString().substring(0,5);

            patient.setIdcard(idcard);
            patient.setName(name);
            patient.setPassword(MD5.md5Hex(pw + "{" + salt + "}"));
            patient.setSalt(salt);
            patient.setMobile(mobile);
            patient.setDel("1");
            patient.setEnabled(1);
            patient.setLocked(0);
            patient.setCreateTime(new Date());
            patient.setUpdateTime(new Date());
            patient.setBirthday(IdCardUtil.getBirthdayForIdcard(idcard));
            patient.setSex(Integer.parseInt(IdCardUtil.getSexForIdcard_new(idcard)));

            BasePatientDO temp = basePatientDao.save(patient);

            PatientMedicareCardDO medicareCardDO = new PatientMedicareCardDO();
            medicareCardDO.setDel("1");
            medicareCardDO.setParentType("A");
            medicareCardDO.setType("A_01");
            medicareCardDO.setCode(ssc);
            medicareCardDO.setPatientCode(temp.getId());

            basePatientMedicareCardDao.save(medicareCardDO);

            //保存微信信息
            if(StringUtils.isNotBlank(openid)&&StringUtils.isNotBlank(wxId)){
                BasePatientWechatDo wechatDo = new BasePatientWechatDo();
                wechatDo.setWechatId(wxId);
                wechatDo.setOpenid(openid);
                wechatDo.setPatientId(temp.getId());
                basePatientWechatDao.save(wechatDo);
            }
            /*if (org.apache.commons.lang3.StringUtils.isNoneBlank(wxId)&&wxId.equalsIgnoreCase("xm_ykyy_wx")){
                ykyyService.registerYkt(temp.getId(),pw,captcha);
            }*/

            rs.put("code","1");
            rs.put("mes","ok");
            rs.put("patient",temp);
            return rs;
        }catch (Exception e){
            e.printStackTrace();
        }
        rs.put("code","-3");
        rs.put("mes","保存用户信息失败");
        return rs;
    }
    public Map<String,Object> registerPatient(String mobile,String pw,String name,String idcard,String ssc,String openid)throws Exception{

        Map<String,Object> rs = new HashedMap();
        try{
            List<BasePatientDO> p2 = basePatientDao.findByMobileAndDel(mobile,"1");
            if(p2!=null&&p2.size()>0){
                rs.put("code","-2");
                rs.put("mes","手机号已经存在");
                return rs;
            }
            BasePatientDO patient = new BasePatientDO();
            String salt = UUID.randomUUID().toString().substring(0,5);
            patient.setPassword(MD5.md5Hex(pw + "{" + salt + "}"));
            patient.setSalt(salt);
            patient.setMobile(mobile);
            patient.setDel("1");
            patient.setEnabled(1);
            patient.setLocked(0);
            patient.setSsc(ssc);
            patient.setCreateTime(new Date());
            patient.setUpdateTime(new Date());
            if(StringUtils.isNoneBlank(name)){
                patient.setName(name);
            }
            if (StringUtils.isNoneBlank(idcard)){
                patient.setIdcard(idcard);
            }
            if(StringUtils.isNoneBlank(openid)){
                patient.setOpenid(openid);
            }
            if("hz_yyyzh_wx".equalsIgnoreCase(wechatId)){//杭州医养头像初始化
                if(StringUtils.isBlank(patient.getPhoto())){
                    String sql = " select dict_value from wlyy_hospital_sys_dict where dict_name='hz_default_avatar' and dict_code='jiashunan' ";
                    String hzPhoto = jdbcTemplate.queryForObject(sql,String.class);
                    patient.setPhoto(hzPhoto);
                }
            }

            BasePatientDO temp = basePatientDao.save(patient);

            PatientMedicareCardDO medicareCardDO = new PatientMedicareCardDO();
            medicareCardDO.setDel("1");
            medicareCardDO.setParentType("A");
            medicareCardDO.setType("A_01");
            medicareCardDO.setPatientCode(temp.getId());

            basePatientMedicareCardDao.save(medicareCardDO);
            WlyyPatientFamilyMemberDO wlyyPatientFamilyMemberDO = new WlyyPatientFamilyMemberDO();
            wlyyPatientFamilyMemberDO.setIsDel(1);
            wlyyPatientFamilyMemberDO.setPatient(patient.getId());
            wlyyPatientFamilyMemberDO.setFamilyMember(patient.getId());
            wlyyPatientFamilyMemberDO.setFamilyRelation("7");
            wlyyPatientFamilyMemberDO.setFamilyRelationName("本人");
            if (StringUtils.isNoneBlank(idcard)){
                wlyyPatientFamilyMemberDO.setCardNo(idcard);
            }
            familyMemberDao.save(wlyyPatientFamilyMemberDO);
            /*if (org.apache.commons.lang3.StringUtils.isNoneBlank(wxId)&&wxId.equalsIgnoreCase("xm_ykyy_wx")){
                ykyyService.registerYkt(temp.getId(),pw,captcha);
            }*/

            rs.put("code","1");
            rs.put("mes","ok");
            rs.put("patient",temp);
            return rs;
        }catch (Exception e){
            e.printStackTrace();
        }
        rs.put("code","-3");
        rs.put("mes","保存用户信息失败");
        return rs;
    }

    public Integer checkeMobileAndIdcard(String mobile,String idcard){
        BasePatientDO patientDO =  basePatientDao.findByIdcardAndDel(idcard, "1");
        if(patientDO==null){
            return -1;
        }
        if(!mobile.equals(patientDO.getMobile())){
            return -2;
        }
        return 1;
    }

    public String updatePatientPw(String pw,String mobile){

        List<BasePatientDO> list = basePatientDao.findByMobileAndDel(mobile,"1");
        if(list!=null&&list.size()>0){
            for(BasePatientDO patient:list){
                String salt = UUID.randomUUID().toString().substring(0,5);
                patient.setPassword(MD5.md5Hex(pw + "{" + salt + "}"));
                patient.setSalt(salt);
            }
            basePatientDao.saveAll(list);
        }else{
            return "-1";
        }
        return "ok";
    }

    public boolean saveDoctorPwlimitDate(String doctor){
        String key = "hlwyy:authPw:doctor:" + doctor;
        com.alibaba.fastjson.JSONObject valueJson = new com.alibaba.fastjson.JSONObject();
        valueJson.put("updateTime",new Date().getTime());
        valueJson.put("isChanged",true);
        redisTemplate.opsForValue().set(key,valueJson.toJSONString());
        return true;
    }

    public String updateDoctorPw(String mobile,String pw,String wxId){
        List<BaseDoctorDO> list = baseDoctorDao.findByMobile(mobile);
        if(list!=null&&list.size()>0){
            for(BaseDoctorDO doctorDO:list){
                //认证信息设置
                String salt = randomString(5);
                doctorDO.setPassword(MD5.md5Hex(pw + "{" + salt + "}"));
                doctorDO.setSalt(salt);
                baseDoctorDao.save(doctorDO);
                saveDoctorPwlimitDate(doctorDO.getId());
            }
        }else{
            return "-1";
        }


        return "ok";
    }


    public String updateDoctorMoble(String id,String mobile){
        BaseDoctorDO doctorDO = baseDoctorDao.getOne(id);
        //认证信息设置
        doctorDO.setMobile(mobile);
        baseDoctorDao.save(doctorDO);
        return "ok";
    }

    public String randomString(int length) {
        String str = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
        StringBuffer buffer = new StringBuffer();
        Random random = new Random();

        for(int i = 0; i < length; ++i) {
            int pos = random.nextInt(str.length());
            buffer.append(str.charAt(pos));
        }

        return buffer.toString();
    }

//    public static void main(String ag[]){
//        String dd ="jkzl@2020";
//        String pw =MD5.md5Hex(dd + "{jkzl}");
//        System.out.println(pw);
//    }
}
