package com.yihu.jw.afterDiagnosis.endpoint.prescription;

import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.afterDiagnosis.endpoint.BaseController;
import com.yihu.jw.afterDiagnosis.service.prescription.PrescriptionDispensaryCodeService;
import com.yihu.jw.afterDiagnosis.service.prescription.PrescriptionInfoService;
import com.yihu.jw.afterDiagnosis.service.prescription.PrescriptionPayService;
import com.yihu.jw.afterDiagnosis.service.prescription.WlyyPrescriptionService;
import com.yihu.jw.entity.hospital.prescription.*;
import com.yihu.jw.hospital.prescription.dao.PrescriptionExpressageDao;
import com.yihu.jw.util.entity.ServiceException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.json.JSONArray;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * Created by Administrator on 2017/7/21.
 */

@RestController("patientPrescriptionController")
@RequestMapping(value = "/patient/prescription")
@Api(description = "患者端-长处方接口")
public class PatientPrescriptionController extends BaseController {

    @Autowired
    private PrescriptionDispensaryCodeService prescriptionDispensaryCodeService;
    @Autowired
    private WlyyPrescriptionService prescriptionService;
    @Autowired
    private PrescriptionInfoService prescriptionInfoService;
    @Autowired
    private PrescriptionPayService prescriptionPayService;
    @Autowired
    private PrescriptionExpressageDao prescriptionExpressageDao;


    /**
     * 生成取药码
     */
    @RequestMapping(value = "/dispensaryCode/saveQRCode", method = RequestMethod.POST)
    public String saveQRCode2(
            @ApiParam(required = true, name = "prescriptionCode", value = "处方code") @RequestParam(value = "prescriptionCode", required = true) String prescriptionCode) {
        try {

            prescriptionDispensaryCodeService.saveQRCode(prescriptionCode);
            return write(200, "生成二维码成功！");
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            error(e);
            return error(-1, "生成二维码失败！");
        }
    }

    /**
     * 生成居民取药码
     *
     * @return
     */
    @RequestMapping(value = "/dispensaryCode/savePatientQRCode", method = RequestMethod.GET)
    public String saveQRCode(
            @ApiParam(required = true, name = "prescriptionCode", value = "处方code") @RequestParam(value = "prescriptionCode", required = true) String prescriptionCode) {
        try {
            PrescriptionDispensaryCode prescriptionDispensaryCode = prescriptionDispensaryCodeService.savePatientQRCode(prescriptionCode);
            if (prescriptionDispensaryCode == null) {
                return error(-1, "生成居民取药码失败！");
            } else {
                return write(200, "生成居民取药码成功！", "code", prescriptionDispensaryCode.getCode());
            }
        } catch (Exception e) {
            error(e);
            return error(-1, "生成居民取药码失败！");
        }
    }

    /**
     * 生成配送员取药码
     *
     * @return
     */
    @RequestMapping(value = "/dispensaryCode/saveDoctorQRCode", method = RequestMethod.GET)
    public String saveDoctorQRCode(
            @RequestParam(value = "prescriptionCode", required = true) String prescriptionCode,
            @RequestParam(value = "code", required = true) String code) {

        try {
            PrescriptionDispensaryCode prescriptionDispensaryCode = prescriptionDispensaryCodeService.saveQRCode(code,prescriptionCode, 2);
            if (prescriptionDispensaryCode == null) {
                return error(-1, "生成配送员取药码失败！");
            } else {
                return write(200, "生成配送员取药码成功！", "code", prescriptionDispensaryCode.getCode());
            }
        } catch (Exception e) {
            error(e);
            return error(-1, "生成配送员取药码失败！");
        }
    }


    /**
     * 生成配送员配送码
     *
     * @return
     */
    @RequestMapping(value = "/dispensaryCode/saveDoctorQRDeliveryCode", method = RequestMethod.GET)
    public String saveDoctorQRDeliveryCode(
            @RequestParam(value = "prescriptionCode", required = true) String prescriptionCode,
            @RequestParam(value = "code", required = true) String code) {

        try {
            PrescriptionDispensaryCode prescriptionDispensaryCode = prescriptionDispensaryCodeService.saveQRCode(code,prescriptionCode, 3);
            if (prescriptionDispensaryCode == null) {
                return error(-1, "生成配送员配送码失败！");
            } else {
                return write(200, "生成配送员配送码成功！", "code", prescriptionDispensaryCode.getCode());
            }
        } catch (Exception e) {
            error(e);
            return error(-1, "生成配送员配送码失败！");
        }
    }


    /**
     * 居民端获取取药码
     *
     * @param prescriptionCode
     * @return
     */
    @RequestMapping(value = "/dispensaryCode/getPatientQrcode", method = RequestMethod.GET)
    public String getPatientQrcode(@RequestParam(value = "prescriptionCode", required = true) String prescriptionCode) {

        try {
            Map<String, Object> map = prescriptionDispensaryCodeService.getQrcode(prescriptionCode, getRepUID(), 1);
            return write(200, "获取居民取药码成功！", "data", map);
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            error(e);
            return error(-1, "获取取药码失败！");
        }
    }

    /**
     * 居民端获取智慧药柜取药码
     *
     * @param prescriptionCode
     * @return
     */
    @RequestMapping(value = "/dispensaryCode/getJumiQrcode", method = RequestMethod.GET)
    public String getJumiQrcode(@RequestParam(value = "prescriptionCode", required = true) String prescriptionCode) {
        try {
            Map<String, Object> map = prescriptionDispensaryCodeService.getJumiQrcode(prescriptionCode);
            return write(200, "获取居民取药码成功！", "data", map);
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            error(e);
            return error(-1, "获取取药码失败！");
        }
    }

    /**
     * 显示居民所有的取药码列表
     *
     * @param status
     * @param timeType
     * @param page
     * @param pagesize type 1、居民端扫码列表 2、配送员（健管师）扫码列表
     * @return
     */
    @RequestMapping(value = "/dispensaryCode/list", method = RequestMethod.GET)
    public String list(
            @RequestParam(value = "status", required = false) Integer status,
            @RequestParam(value = "timeType", required = false) Integer timeType,
            @RequestParam(value = "page", required = true) Integer page,
            @RequestParam(value = "pagesize", required = true) Integer pagesize) {
        try {
            JSONArray result = prescriptionDispensaryCodeService.findByStatusAndTime(timeType, getRepUID(), status, page, pagesize, 1);
            return write(200, "获取信息成功！", "result", result);
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败！");
        }
    }

    /**
     * 订单跟踪 包含处方药品信息 订单记录  支付记录
     * 根据处方code获取处方流程
     *
     * @param prescriptionCode
     * @return
     */
    @RequestMapping(value = "/prescriptionFollow", method = RequestMethod.GET)
    @ApiOperation(value = "获取处方订单详情")
    public String prescriptionFollow(
            @RequestParam(required = true) @ApiParam(value = "处方code", name = "prescriptionCode") String prescriptionCode) {
        try {
            JSONObject jo = new JSONObject();
            //获取处方信息
            WlyyPrescriptionDO prescription = prescriptionService.findByCode(prescriptionCode);
            //获取处方药品信息
            List<WlyyPrescriptionInfoDO> prescriptionInfos = prescriptionInfoService.getPrescriptionInfo(prescriptionCode);
            //获取付款信息
            PrescriptionPay prescriptionPay = prescriptionPayService.findByPrescriptionCode(prescriptionCode);
            //获取配送信息
            WlyyPrescriptionExpressageDO prescriptionExpressage = prescriptionExpressageDao.findByPrescriptionCode(prescriptionCode);
            jo.put("prescriptionExpressage", prescriptionExpressage);
            jo.put("prescriptionPay", prescriptionPay);
            jo.put("prescriptionInfos", prescriptionInfos);
            if (prescription != null) {
                jo.put("createTime", prescription.getCreateTime());//创建时间
                jo.put("doctorName", prescription.getDoctorName());//审核医生
                jo.put("prescriptionHospital", prescription.getHospitalName());//出药机构
                jo.put("dispensaryDispensaryType", prescription.getDispensaryType());//处方配送方式
                jo.put("prescriptionStatus", prescription.getStatus());//处方状态
                jo.put("ssc",prescription.getSsc());//医保卡号
                jo.put("payTime",prescription.getPayTime());//支付时间
            }
            return write(200, "查询成功", "data", jo);
        } catch (Exception e) {
            return error(-1, "获取失败");
        }
    }

    /**
     * 续方确认收药接口
     *
     * @param prescriptionCode 续方code原状态为60
     * @param type 1:确认收药 2：延长收药
     * @return
     */
    @RequestMapping(value = "/confirmReceipt", method = RequestMethod.POST)
    @ApiOperation(value = "续方确认收药接口")
    public String confirmReceipt(
            @RequestParam(required = true) @ApiParam(value = "处方code", name = "prescriptionCode") String prescriptionCode,
            @RequestParam(required = true) @ApiParam(value = "收药方式", name = "type") Integer type) {
        try {
            Map<String,Object> result = prescriptionService.confirmReceipt(prescriptionCode,type);
            int code = Integer.parseInt(result.get("code").toString());
            String msg = result.get("msg").toString();
            return write(code, msg);
        } catch (Exception e) {
            return error(-1, "确认失败");
        }
    }



    @RequestMapping(value = "/fadeRecipe", method = RequestMethod.POST)
    @ApiOperation(value = "挂号作废处方接口")
    public String fadeRecipe(
            @RequestParam(required = true) @ApiParam(value = "处方code", name = "prescriptionCode") String prescriptionCode){
        try {
//            String result = jwPrescriptionService.fadeRecipe(prescriptionCode);

            return write(200, "挂号作废成功！", "result", "");
        } catch (ServiceException se){
            return error(-1,se.getMessage());
        } catch (Exception e) {
            error(e);
            return error(-1, "挂号作废失败+"+e.getMessage());
        }
    }

    /**
     *
     * @param prescriptionCode
     * @return
     */
    @RequestMapping(value = "/checkRecipeNoPay",method = RequestMethod.POST)
    @ApiOperation(value = "查询待结算订单是否作废，如果作废更新本地数据为线下取消")
    public String checkRecipeNoPay(@ApiParam(value = "处方业务主键",name = "prescriptionCode")@RequestParam(value = "prescriptionCode")String prescriptionCode){
        try{
            JSONObject object = prescriptionService.checkRecipeNoPay(2,prescriptionCode);
            return write(200,"操作成功！","result",object);
        }catch (Exception e){
            error(e);
            return error(-1,"操作失误！"+e.getMessage());
        }
    }

}
