package com.yihu.jw.afterDiagnosis.endpoint.prescription;


import com.yihu.jw.afterDiagnosis.endpoint.BaseController;
import com.yihu.jw.afterDiagnosis.service.prescription.PrescriptionDispensaryCodeService;
import com.yihu.jw.afterDiagnosis.service.prescription.PrescriptionInfoService;
import com.yihu.jw.entity.hospital.prescription.PrescriptionDispensaryCode;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.entity.ServiceException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * Created by Trick on 2017/7/25.
 */
@RestController
@RequestMapping(value = "/patient/prescriptionInfo")
@Api(description = "患者端-长处方接口")
public class PatientPrescriptionInfoController extends BaseController {

    @Autowired
    private PrescriptionInfoService prescriptionInfoService;

    @Autowired
    private PrescriptionDispensaryCodeService prescriptionDispensaryCodeService;

    /**
     * 获取处方列表
     * @param type
     * @return
     */
    @RequestMapping(value = "/getPrescriptionInfos" , method = RequestMethod.GET)
    @ApiOperation(value = "获取长处方信息列表")
    public String getPrescriptionInfos(@RequestParam(required = true)@ApiParam(name="type",value="1:查询处方;2我的续方;3:续方记录")String type,
                                       @RequestParam(required = false)@ApiParam(name="isRenewal",value="处方是否可续方:1.是;2.无过滤")String isRenewal,
                                       @RequestParam(required = false)@ApiParam(name="recipeNo",value="医嘱号，第一次获取上一方时传入0，第二次获取上一方时传入第一次返回的医嘱号，以此类推")String recipeNo,
                                       @RequestParam(required = false)@ApiParam(name="startDate",value="开始时间")String startDate,
                                       @RequestParam(required = false)@ApiParam(name="endDate",value="结束时间")String endDate,
                                       @RequestParam(required = false,defaultValue = "0")@ApiParam(name="diagnosisCode",value="所有诊断（0 全部 1高血压 2糖尿病）")String diagnosisCode,
                                       @RequestParam(required = false)@ApiParam(name="page",value="起始页")Integer page,
                                       @RequestParam(required = false)@ApiParam(name="size",value="每页记录数")Integer size,
                                       @RequestParam(required = true)@ApiParam(name="hospital",value="社区CODE")String hospital){
        try {
            return write(200, "查询成功！", "data", prescriptionInfoService.getPrescriptionInfos(type,recipeNo,diagnosisCode,startDate,endDate,getRepUID(),isRenewal,page,size,hospital));
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败！");
        }
    }

    @RequestMapping(value = "getRecipeMasterList" , method = RequestMethod.GET)
    @ApiOperation(value = "获取智业长处方信息列表")
    public String getRecipeMasterList(@RequestParam(required = false)@ApiParam(name="isRenewal",value="处方是否可续方:1.是;2.无过滤")String isRenewal,
                                      @RequestParam(required = false)@ApiParam(name="startDate",value="开始时间")String startDate,
                                      @RequestParam(required = false)@ApiParam(name="endDate",value="结束时间")String endDate,
                                      @RequestParam(required = false,defaultValue = "0")@ApiParam(name="diagnosisCode",value="所有诊断（0 全部 1高血压 2糖尿病）")String diagnosisCode,
                                      @RequestParam(required = false)@ApiParam(name="page",value="起始页")Integer page,
                                      @RequestParam(required = false)@ApiParam(name="size",value="每页记录数")Integer size,
                                      @RequestParam(required = false)@ApiParam(name="size",value="社区CODE")String hospital
    ){
        try {
            com.alibaba.fastjson.JSONObject json = prescriptionInfoService.getRecipeMasterList(getRepUID(),startDate,endDate,isRenewal,diagnosisCode,page,size,hospital);
            return write(200, "查询成功！", "data",json);
        }catch (Exception e){
            error(e);
            return error(-1, "查询失败！");
        }
    }

    @RequestMapping(value = "/getPrescription" ,method = RequestMethod.GET)
    @ApiOperation(value = "获取长处方详细信息")
    public String getPrescription(@RequestParam(required = true)@ApiParam(name="code",value="处方CODE")String code,
                                  @RequestParam(required = false)@ApiParam(name="preCode",value="续方CODE")String preCode,
                                  @RequestParam(required = false)@ApiParam(name="hospital",value="社区CODE")String hospital){
        try {
            return write(200, "查询成功！", "data", prescriptionInfoService.getPrescription(code,getRepUID(),preCode,hospital));
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败！");
        }
    }

    @RequestMapping(value = "/getContinuedPrescription", method = RequestMethod.GET)
    @ApiOperation(value = "获取续方详细信息")
    public String getContinuedPrescription(@RequestParam(required = true)@ApiParam(name="code",value="续方CODE")String code){
        try {
            return write(200, "查询成功！", "data", prescriptionInfoService.getContinuedPrescription(code));
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败！");
        }
    }

    @RequestMapping(value = "/getPrescriptionProcess" ,method = RequestMethod.GET)
    @ApiOperation(value = "获取续方记录详细信息")
    public String getPrescriptionProcess(@RequestParam(required = true)@ApiParam(name="code",value="续方CODE")String code){
        try {
            return write(200, "查询成功！", "data", prescriptionInfoService.getPrescriptionProcess(code));
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败！");
        }
    }

    @RequestMapping(value = "/cancelPrescriotion" , method = RequestMethod.POST)
    @ApiOperation(value = "取消续方")
    public String cancelPrescriotion(@RequestParam(required = true)@ApiParam(name="code",value="续方CODE")String code,
                                     @RequestParam(required = true)@ApiParam(name="reason",value="原因")String reason){
        try {
            return write(200, "查询成功！", "data", prescriptionInfoService.cancelPrescriotion(code,reason));
        } catch (ServiceException se){
            return error(-1,se.getMessage());
        } catch (Exception e) {
            error(e);
            return error(-1, "取消失败！");
        }
    }

    @ApiOperation("判断患者是否有续方记录")
    @RequestMapping(value = "checkPatientWithPrescription", method = RequestMethod.GET)
    public String checkPatientWithPrescription(){
        try {
            return write(200, "获取信息成功！", "data",prescriptionInfoService.checkPatientWithPrescription(getUID()));
        }catch (Exception e) {
            error(e);
            return error(-1, e.getMessage());
        }
    }
    
    @RequestMapping(value = "/getPatientLastPrescriptionProcess" ,method = RequestMethod.GET)
    @ApiOperation(value = "获取当前居民的上一条续方详情")
    public String getPatientLastPrescriptionProcess(){
        try {
            return write(200, "查询成功！", "data", prescriptionInfoService.getPatientLastPrescriptionProcess(getRepUID()));
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败！");
        }
    }
    
    
    @RequestMapping(value = "/getPatientLastPrescriptionExpressInfo" ,method = RequestMethod.GET)
    @ApiOperation(value = "获取当前居民的上一条续方的派送地址")
    public String getPatientLastPrescriptionExpressInfo(@RequestParam(required = true)@ApiParam(name="dispensaryType",value="派送方式")Integer dispensaryType){
        try {
            return write(200, "查询成功！", "data", prescriptionInfoService.getPatientLastPrescriptionExpressInfo(getRepUID(),dispensaryType));
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败！");
        }
    }
    
    /**
     * 扫码
     * @param code
     * @return
     */
    @RequestMapping(value = "/dispensaryCode/scanQrcode",method = RequestMethod.GET)
    public String scanQrcode(
            @RequestParam(value = "code",required = true) String code,
            @RequestParam(value = "patientCode",required = true) String patientCode
    ){
        try {
            PrescriptionDispensaryCode prescriptionDispensaryCode = prescriptionDispensaryCodeService.scanQrcode(code,getUID());
            return write(200, "扫码成功！", "data", prescriptionDispensaryCode);
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            error(e);
            return error(-1, e.getMessage());
        }
    }
    
    
    @RequestMapping(value = "/getTodayPrescriptionCount", method = RequestMethod.GET)
    @ApiOperation(value = "获取居民今天已发起的有效续方数量")
    public String getTodayPrescriptionCount(@RequestParam(required = true)
                                            @ApiParam(value = "居民CODE", name = "patientCode") String patientCode,
                                            @RequestParam(required = false)
                                            @ApiParam(value = "开始时间", name = "start_date") String start_date) {
        try {
            if(StringUtils.isBlank(start_date)){
                start_date = DateUtil.getStringDateShort();
            }
            return write(200, "查询成功！", "data", prescriptionInfoService.getTodayPrescriptionCount(patientCode,start_date));
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败！");
        }
    }

}

