package com.yihu.jw.afterDiagnosis.endpoint.prescription;

import com.yihu.jw.afterDiagnosis.endpoint.BaseController;
import com.yihu.jw.afterDiagnosis.service.prescription.PatientPrescriptionPayService;
import com.yihu.jw.afterDiagnosis.service.prescription.PrescriptionAdressService;
import com.yihu.jw.afterDiagnosis.service.prescription.PrescriptionNoticesService;
import com.yihu.jw.entity.order.PayLog;
import com.yihu.jw.hospital.prescription.service.PrescriptionService;
import com.yihu.jw.util.entity.ServiceException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;

/**
 * Created by Reece on 2017/7/28.
 */

@RestController("patientPrescriptionPayController")
@RequestMapping(value = "/patient/prescription/pay")
@Api(description = "患者端-长处方支付接口")
public class PatientPrescriptionPayController extends BaseController {

    @Autowired
    private PatientPrescriptionPayService payService;

    @Autowired
    private PrescriptionService prescriptionService;
    @Autowired
    private PrescriptionNoticesService noticesService;
    @Autowired
    private PrescriptionAdressService prescriptionAdressService;



    /**
     * 患者长处方支付接口 可配送方式查询
     *
     * @param orgCode          居民签约社区code
     * @param prescriptionCode 续方code
     * @return
     * @
     */
    @RequestMapping(value = "/dispatch", method = RequestMethod.POST)
    @ApiOperation(value = "配送方式查询接口")
    public String dispatch(
            @RequestParam(required = true) @ApiParam(value = "续方code", name = "prescriptionCode") String prescriptionCode,
            @RequestParam(required = true) @ApiParam(value = "居民签约社区code", name = "orgCode") String orgCode)  {
        try {
            com.alibaba.fastjson.JSONObject result = payService.dispatch(prescriptionCode, orgCode);
            return write(200, "获取成功!", "data", result);
        } catch (Exception e) {
            return error(-1, "获取失败!");
        }
    }


    /**
     * 患者长处方支付接口
     *
     * @param orgCode          患者签约医院code
     * @param prescriptionCode 续方code
     * @param type             配送方式 1:自取 2：快递 3：健管师配送
     * @param addressJson      配送信息json
     * @param totalAmount      订单总金额
     * @return
     * @
     */
    @RequestMapping(value = "/recipeCharge", method = RequestMethod.POST)
    @ApiOperation(value = "患者长处方支付接口")
    public String recipeCharge(
            @RequestParam(required = true) @ApiParam(value = "居民社区code", name = "orgCode") String orgCode,
            @RequestParam(required = true) @ApiParam(value = "续方code", name = "prescriptionCode") String prescriptionCode,
            @RequestParam(required = true) @ApiParam(value = "配送方式", name = "type") int type,
            @RequestParam(required = true) @ApiParam(value = "配送地址json", name = "addressJson", defaultValue = "{\"townName\":\"海沧区\",\"code\":\"3502050100\",\"address\":" +
                    "\"冰岛\",\"cityName\":\"厦门市\",\"townCode\":\"350205\",\"provinceCode\":\"350000\",\"cityCode\":\"350200\",\"name\":\"海沧区嵩屿街道社区卫生服务中心\"," +
                    "\"provinceName\":\"福建省\",\"streeCode\":\"35020501\",\"streeName\":\"皇后大道东\",\"phone\":\"13253541190\"}") String addressJson,
            @RequestParam(required = true) @ApiParam(value = "页面回调地址", name = "returnUrl") String returnUrl,
            @RequestParam(required = true) @ApiParam(value = "订单总金额", name = "totalAmount") int totalAmount,
            @RequestParam(required = false) @ApiParam(value = "送达时间", name = "deliveryTime") String deliveryTime)  {
        try {
            String patient = getUID();
            String repUid = getRepUID();

            if(!patient.equals(repUid)){
                return error(-1, "目前不支持在线共济支付，请使用居民本人账号登陆支付，或者到社区进行线下支付！");
            }
            
            Map result = payService.charge(type, addressJson, orgCode, prescriptionCode, totalAmount, patient, repUid, null, returnUrl, deliveryTime);
            if ("-1".equals(result.get("status").toString())) {
                return error(-1, "续方居民信息与微信支付身份信息不一致！");
            } else if ("-2".equals(result.get("status").toString())) {
                return error(-1, "续方居民信息挂号号或处方号不存在！");
            } else if ("-3".equals(result.get("status").toString())) {
                return error(-1, "您没有需要待结算的信息！");
            } else if ("-4".equals(result.get("status").toString())) {
                return error(-1, "附属卡未绑定！");
            } else {
                return write(200, "支付成功！", "data", result);
            }
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            return error(-1, "支付失败！");
        }
    }

    /**
     * 患者家签及长处方支付结果查询接口
     *
     * @param outChargeNo
     * @return
     */
    @RequestMapping(value = "/recipeChargeQuery", method = RequestMethod.GET)
    @ApiOperation(value = "患者支付结果查询接口")
    public String chargeQuery(@RequestParam(required = true) @ApiParam(value = "支付应用流水号", name = "outChargeNo") String outChargeNo)  {
        try {
            String result = payService.chargeQuery(outChargeNo, null);
            return write(200, "获取成功！", "data", result);
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            return error(-1, "获取失败！");
        }
    }

    /**
     *易联众支付结果查询
     * @param outChargeNo
     * @returnc
     */
    @RequestMapping(value = "/getChargeQuery", method = RequestMethod.GET)
    @ApiOperation(value = "患者支付结果查询接口")
    public String getChargeQuery(@RequestParam(required = true) @ApiParam(value = "支付应用流水号", name = "outChargeNo") String outChargeNo)  {
        try {
            String result = payService.getChargeQuery(outChargeNo, null);
            return write(200, "获取成功！", "data", result);
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            return error(-1, "获取失败！");
        }
    }

    /**
     * 查询家庭签约支付结果
     *
     * @return
     */
    @RequestMapping(value = "/getSignFamilyPayResult", method = RequestMethod.GET)
    @ApiOperation(value = "查询家庭签约支付结果")
    public String getSignFamilyPayResult(
            @RequestParam(required = true)
            @ApiParam(value = "支付应用流水号", name = "outChargeNo") String outChargeNo,
            @RequestParam(required = true)
            @ApiParam(value = "易联众appid", name = "appId") String appId,
            @RequestParam(required = true)
            @ApiParam(value = "易联众app secret", name = "appSecret") String appSecret) {
        try {
            String result = payService.getSignFamilyPayResult(outChargeNo, null, appId, appSecret);
            return write(200, "获取成功！", "data", result);
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            error(e);
            return error(-1, "获取失败！");
        }
    }


    /**
     * 长处方支付成功前端页面回调接口
     *
     * @param
     * @return
     */
    @RequestMapping(value = "/recipeReturnUrl", method = RequestMethod.POST)
    @ApiOperation(value = "支付成功前端页面回调接口")
    public String recipeReturnUrl(HttpServletRequest request, HttpServletResponse response)  {
        try {
            Map result = payService.recipeReturnUrl(request, response, null);
            return write(200, "页面回调成功!", "data", result);
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            return error(-1, "页面回调失败!");
        }
    }

    /**
     * 测试长处方相关模板通知消息
     *
     * @param patient   要发送对象居民code
     * @param doctName  相关医生姓名（type=1审核医生、type=2 null、type=3配送健管师、type=4审核医生）
     * @param ordenTime 订单完成时间（type=1 null、type=2 null、type=3 null、type=4 订单完成时间）
     * @param type      1:续方审核通知 2：取药代办通知 3：配送中信息变更 4：已完成服务结果
     * @param status    续方通知状态 0不通过 1通过 2调整处方  其余状态均为0
     * @param url       带参数的模板跳转链接
     */
    @RequestMapping(value = "/testSendMassages", method = RequestMethod.POST)
    @ApiOperation(value = "测试长处方相关模板通知消息")
    public String testSendMassages(
            @ApiParam(name = "patient", value = "要发送对象居民code")
            @RequestParam(value = "patient", required = true) String patient,
            @ApiParam(name = "doctName", value = "相关医生姓名（type=1审核医生、type=2 null、type=3配送健管师、type=4审核医生）")
            @RequestParam(value = "doctName", required = false) String doctName,
            @ApiParam(name = "ordenTime", value = "订单完成时间（type=1 null、type=2 null、type=3 null、type=4 订单完成时间）")
            @RequestParam(value = "ordenTime", required = false) String ordenTime,
            @ApiParam(name = "type", value = "1:续方审核通知 2：取药代办通知 3：配送中信息变更 4：已完成服务结果")
            @RequestParam(value = "type", required = true) int type,
            @ApiParam(name = "status", value = "续方通知状态 0不通过 1通过 2调整处方  其余状态均为0")
            @RequestParam(value = "status", required = true) int status,
            @ApiParam(name = "url", value = "带参数的模板跳转链接")
            @RequestParam(value = "url", required = true) String url) {
        try {
            noticesService.sendMessages(patient, doctName, ordenTime, type, status, url);
            return write(200, "发送成功!");
        } catch (Exception e) {
            return error(-1, "发送失败!");
        }
    }

    //===========================v1.3.9======================================================================
    @RequestMapping(value = "/getAddressList", method = RequestMethod.POST)
    @ApiOperation(value = "获取患者所有地址")
    public String getAdressList(@ApiParam(name = "patient", value = "居民code")
                                @RequestParam(value = "patient", required = true) String patient) {
        try {

            return write(200, "获取成功!", "data", prescriptionAdressService.getAdressList(patient));
        } catch (Exception e) {
            return error(-1, "获取失败!");
        }
    }

    @RequestMapping(value = "/getDefAddress", method = RequestMethod.POST)
    @ApiOperation(value = "获取患者默认地址")
    public String getDefAdress(String patient) {
        try {

            return write(200, "获取成功!", "data", prescriptionAdressService.getDefAdress(patient));
        } catch (Exception e) {
            error(e);
            return error(-1, "发送失败!");
        }
    }

    @RequestMapping(value = "/saveAddress", method = RequestMethod.POST)
    @ApiOperation(value = "保存地址")
    public String saveAdress(@ApiParam(name = "addressJson", value = "实体json串")
                             @RequestParam(value = "addressJson", required = true) String addressJson) {
        try {

            return write(200, "获取成功!", "data", prescriptionAdressService.saveAdress(addressJson));
        } catch (Exception e) {
            error(e);
            return error(-1, "获取失败!");
        }
    }

    @RequestMapping(value = "/delAddress", method = RequestMethod.POST)
    @ApiOperation(value = "删除地址")
    public String delAdress(@ApiParam(name = "id", value = "主键")
                            @RequestParam(value = "id", required = true) String id) {
        try {

            return write(200, "获取成功!", "data", prescriptionAdressService.delAdress(id));
        } catch (Exception e) {
            error(e);
            return error(-1, "获取失败!");
        }
    }

    @RequestMapping(value = "/getProvince", method = RequestMethod.POST)
    @ApiOperation(value = "获取省会")
    public String getProvince() {
        try {

            return write(200, "获取成功!", "data", prescriptionAdressService.getProvince());
        } catch (Exception e) {
            error(e);
            return error(-1, "获取失败!");
        }
    }

    @RequestMapping(value = "/getCityByProvince", method = RequestMethod.POST)
    @ApiOperation(value = "根据省会获取城市")
    public String getCityByProvince(@ApiParam(name = "province", value = "省会code")
                                    @RequestParam(value = "province", required = true) String province) {
        try {

            return write(200, "获取成功!", "data", prescriptionAdressService.getCityByProvince(province));
        } catch (Exception e) {
            error(e);
            return error(-1, "获取失败!");
        }
    }

    @RequestMapping(value = "/getTowmByCity", method = RequestMethod.POST)
    @ApiOperation(value = "根据城市获取区")
    public String getTowmByCity(@ApiParam(name = "city", value = "城市code")
                                @RequestParam(value = "city", required = true) String city) {
        try {

            return write(200, "获取成功!", "data", prescriptionAdressService.getTowmByCity(city));
        } catch (Exception e) {
            error(e);
            return error(-1, "获取失败!");
        }
    }

    /**
     * 查询缴费居民手机绑定情况
     *
     * @return
     */
    @RequestMapping(value = "/getPatientMobile", method = RequestMethod.GET)
    @ApiOperation(value = "查询缴费居民手机绑定情况")
    public String getPatientMobile() {
        try {
            String status = "1";
            String mobile = payService.getPatientMobile(getRepUID());
            if (StringUtils.isEmpty(mobile)){
                status = "0";
            }
            return write(200, "查询成功!", "data", status);
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败!");
        }
    }
    
    /**
     * 根据长处方代码查询支付日志
     * @param prescriptionCode
     * @return
     */
    @RequestMapping(value = "/getPayLogsByPrescriptionCode", method = RequestMethod.GET)
    @ApiOperation(value = "根据长处方代码查询支付日志")
    public String getPayLogsByPrescriptionCode(@ApiParam(name = "prescriptionCode", value = "处方CODE")
                                               @RequestParam(value = "prescriptionCode", required = true) String prescriptionCode){
        try {
            List<PayLog> result = payService.getPayLogsByPrescriptionCode(prescriptionCode);
            return write(200, "查询成功!", "data", result);
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败!");
        }
        
    }

}
