package com.yihu.jw.afterDiagnosis.endpoint.prescription;

import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.afterDiagnosis.endpoint.BaseController;
import com.yihu.jw.afterDiagnosis.service.prescription.PrescriptionAdjustService;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionDO;
import com.yihu.jw.hospital.prescription.dao.PrescriptionDao;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * Created by yeshijie on 2017/8/17.
 */
@RestController
@RequestMapping(value = "/doctor/prescriptionAdjust")
@Api(description = "医生端-长处方线下调整相关接口接口")
public class PrescriptionAdjustController extends BaseController {

	@Autowired
	private PrescriptionDao prescriptionDao;
	@Autowired
	private PrescriptionAdjustService prescriptionAdjustService;
	
	@RequestMapping(value = "adjustPrescription",method = RequestMethod.POST)
	@ApiOperation("线下调整处方")
	public String adjustPrescription(@ApiParam(name = "code", value = "续方code", defaultValue = "10")
	                                 @RequestParam(value = "code", required = true) String code,
	                                 @ApiParam(value = "科室", name = "dept") @RequestParam(required = true) String dept,
	                                 @ApiParam(value = "诊金", name = "registerFee") @RequestParam(required = true) String registerFee,
	                                 @ApiParam(value = "诊金类型", name = "rateTypeCode") @RequestParam(required = true) String rateTypeCode,
									 @RequestParam(required = false) @ApiParam(value = "科室code(智业校正减库存使用)", name = "deptCode") String deptCode,
									 @RequestParam(required = false) @ApiParam(value = "药房地址(顺丰取药使用)", name = "orgAddress") String orgAddress){
		try{
			//1、保存科室和挂号信息
			prescriptionAdjustService.saveDept(code,dept,rateTypeCode,registerFee,deptCode);
			//2、调整处方
			JSONObject json = prescriptionAdjustService.adjustPrescription(code,deptCode,orgAddress);
			if(json.getInteger("status")==-1){
				return error(-1,json.getString("msg"));
			}
			return write(200,"请求调整成功");
		}catch (Exception e){
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "uploadCaDigital",method = RequestMethod.POST)
	@ApiOperation("上传基位ca认证信息")
	public String uploadCaDigital(@ApiParam(name = "code", value = "续方code", defaultValue = "10")
	                              @RequestParam(value = "code", required = true) String code){
		try{
			JSONObject json = prescriptionAdjustService.uploadCaDigital(code);
			if(json.getInteger("status")==-1){
				return error(-1,json.getString("msg"));
			}
			return write(200,"上传ca认证信息成功");
		}catch (Exception e){
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "getPrescriptionStatus",method = RequestMethod.GET)
	@ApiOperation("获取续方状态")
	public String getPrescriptionStatus(@ApiParam(name = "code", value = "续方code", defaultValue = "10")
	                                    @RequestParam(value = "code", required = true) String code){
		try {
			WlyyPrescriptionDO prescription = prescriptionDao.findById(code).orElse(null);
			
			return write(200,"success","data",prescription.getStatus());
		}catch (Exception e){
			return errorResult(e);
		}
	}
}
