package com.yihu.jw.afterDiagnosis.endpoint.prescription;


import com.yihu.jw.afterDiagnosis.endpoint.BaseController;
import com.yihu.jw.afterDiagnosis.service.prescription.PrescriptionDispensaryCodeService;
import com.yihu.jw.afterDiagnosis.service.prescription.WlyyPrescriptionExpressageService;
import com.yihu.jw.entity.hospital.prescription.PrescriptionDispensaryCode;
import com.yihu.jw.util.entity.ServiceException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;

/**
 * Created by Trick on 2017/7/25.
 */
@RestController
@RequestMapping(value = "/doctor/prescriptionCode")
@Api(description = "医生端-长处方接口")
public class PrescriptionCodeController extends BaseController {
	
	
	@Autowired
	private PrescriptionDispensaryCodeService prescriptionDispensaryCodeService;
	
	@Autowired
	private WlyyPrescriptionExpressageService prescriptionExpressageService;

	
	/**
	 * 确认配送 二维码方式
	 * 根据处方code获取处方流程
	 *
	 * @param code wlyy_prescription_dispensary_code的code
	 * @return
	 */
	@RequestMapping(value = "/expressage", method = RequestMethod.POST)
	@ApiOperation(value = "确认配送")
	public String expressage(
			@ApiParam(required = true, name = "code", value = " 二维码code") @RequestParam(value = "code", required = true) String code) {
		try {
			Integer status = prescriptionExpressageService.expressage(code, getUID());
			if (status == -1) {
				return error(-1, "取药码错误");
			} else if (status == -2) {
				return error(-1, "编码已使用");
			} else if (status == -3) {
				return error(-1, "编码类型错误");
			}else if (status == -6) {
				throw new ServiceException("发药错误");
			}else if (status == -7) {
				throw new ServiceException("已无待取药品");
			}
			return write(200, "配送成功");
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	/**
	 * 医生端获取取药码
	 *
	 * @param prescriptionCode
	 * @return
	 */
	@RequestMapping(value = "/dispensaryCode/getDoctorDispensaryQrcode", method = RequestMethod.GET)
	@ApiOperation("医生端获取取药码")
	public String getDoctorDispensaryQrcode(
			@ApiParam(required = true, name = "prescriptionCode", value = "处方code") @RequestParam(value = "prescriptionCode", required = true) String prescriptionCode) {
		
		try{
			Map<String, Object> map = prescriptionDispensaryCodeService.getQrcode(prescriptionCode,getUID(),2);//xh1D2017031503333
			return write(200, "获取配送员取药码成功！", "data", map);
		} catch (ServiceException se) {
			return write(-1,se.getMessage());
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	/**
	 * 医生端获取配送码
	 *
	 * @param prescriptionCode
	 * @return
	 */
	@RequestMapping(value = "/dispensaryCode/getDoctorDeliveryQrcode", method = RequestMethod.GET)
	@ApiOperation("医生端获取配送码")
	public String getDoctorDeliveryQrcode(
			@ApiParam(required = true, name = "prescriptionCode", value = "处方code") @RequestParam(value = "prescriptionCode", required = true) String prescriptionCode) {
		
		try{
			Map<String, Object> map = prescriptionDispensaryCodeService.getQrcode(prescriptionCode,getUID(),3);
			if(map!=null&&map.size()>0){
				
				return write(200, "获取配送员配送码成功！", "data", map);
			}else{
				return error(-1, "获取配送员配送码失败！");
			}
		} catch (ServiceException se) {
			return write(-1,se.getMessage());
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	/**
	 * 扫码
	 * @param code
	 * @return
	 */
	@RequestMapping(value = "/dispensaryCode/scanQrcode",method = RequestMethod.GET)
	public String scanQrcode(
			@RequestParam(value = "code",required = true) String code){
		try {
			PrescriptionDispensaryCode prescriptionDispensaryCode = prescriptionDispensaryCodeService.scanQrcode(code,getUID());
			return write(200, "扫码成功！", "data", prescriptionDispensaryCode);
		} catch (ServiceException se) {
			return write(-1,se.getMessage());
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
}
