package com.yihu.jw.afterDiagnosis.endpoint.prescription;

import com.yihu.jw.afterDiagnosis.endpoint.BaseController;
import com.yihu.jw.afterDiagnosis.service.prescription.WlyyPrescriptionExpressageService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * Created by Reece on 2017/7/28.
 */

@RestController
@RequestMapping(value = "/doctor/prescription/expressage")
@Api(description = "医生端-长处配送相关接口")
public class PrescriptionExpressageController extends BaseController {
	
	@Autowired
	private WlyyPrescriptionExpressageService expressageService;

	/**
	 * 确认接收 药品
	 *
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/expressage2Hospital", method = RequestMethod.POST)
	@ApiOperation(value = "确认药品被服务站接收")
	public String expressage2Hospital(
			@ApiParam(required = true, name = "hospital", value = "机构code") @RequestParam(value = "hospital", required = true) String hospital,
			@ApiParam(required = true, name = "hospitalName", value = "机构名称") @RequestParam(value = "hospitalName", required = true) String hospitalName,
			@ApiParam(required = true, name = "hospitalAddress", value = "机构地址") @RequestParam(value = "hospitalAddress", required = true) String hospitalAddress,
			@ApiParam(required = true, name = "userCode", value = "接收人code") @RequestParam(value = "userCode", required = true) String userCode,
			@ApiParam(required = true, name = "userName", value = "接收人Name") @RequestParam(value = "userName", required = true) String userName,
			@ApiParam(required = true, name = "mobile", value = "电话号码") @RequestParam(value = "mobile", required = true) String mobile,
			@ApiParam(required = true, name = "prescriptionCode", value = "处方code") @RequestParam(value = "prescriptionCode", required = true) String prescriptionCode) throws Exception {
		try {
			expressageService.expressage2Hospital(hospital, hospitalName, hospitalAddress, userCode, userName, mobile, prescriptionCode);
			return success("接收成功");
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getDrugDeliveryState", method = RequestMethod.GET)
	@ApiOperation(value = "获取异常原因列表")
	public String getDrugDeliveryState(){
		try {
			return write(200, "获取成功", "data", expressageService.getDrugDeliveryState());
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getEsListByIDCardOrSSC", method = RequestMethod.GET)
	@ApiOperation(value = "根据医保号和身份证查询订单")
	public String getEsListByIDCardOrSSC(@ApiParam(required = true, name = "keyWord", value = "医保号或身份证") @RequestParam(required = true)String keyWord,
	                                     @ApiParam(required = true, name = "page", value = "起始页，重1开始") @RequestParam(required = true)Integer page,
	                                     @ApiParam(required = true, name = "pageSize", value = "每页大小") @RequestParam(required = true)Integer pageSize){
		try {
			return write(200, "获取成功", "data", expressageService.getEsListByIDCardOrSSC( keyWord, page, pageSize));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/saveDrugDelivery", method = RequestMethod.POST)
	@ApiOperation(value = "异常出药操作")
	public String saveDrugDelivery(@ApiParam(required = true, name = "prescriptionCode", value = "续方code") @RequestParam(required = true)String prescriptionCode,
	                               @ApiParam(required = true, name = "reasonCode", value = "原因code") @RequestParam(required = true)String reasonCode,
	                               @ApiParam(required = true, name = "reasonName", value = "原因名称") @RequestParam(required = true)String reasonName,
	                               @ApiParam(required = true, name = "reason", value = "详细原因") @RequestParam(required = true)String reason){
		try {
			return write(200, "获取成功", "data", expressageService.saveDrugDelivery( getUID(), prescriptionCode, reasonCode, reasonName, reason));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getPresStatesList", method = RequestMethod.GET)
	@ApiOperation(value = "获取续方状态原因列表")
	public String getPresStatesList(){
		try {
			return write(200, "获取成功", "data", expressageService.getPresStatesList());
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getPresEsPayList", method = RequestMethod.GET)
	@ApiOperation(value = "获取订单列表")
	public String getPresEsPayList(@ApiParam(name = "keyWord", value = "订单号，接收人,医保号或身份证") @RequestParam(required = false)String keyWord,
	                               @ApiParam(name = "state", value = "续方状态") @RequestParam(required = false)String state,
	                               @ApiParam(name = "type", value = "1 自取 2快递配送 3健管师配送 4网格员派送") @RequestParam(required = false)String type,
	                               @ApiParam(name = "startDate", value = "开始时间yyyy-MM--dd") @RequestParam(required = false)String startDate,
	                               @ApiParam(name = "endDate", value = "结束时间yyyy-MM-dd") @RequestParam(required = false)String endDate,
	                               @ApiParam(name = "operator", value = "操作人") @RequestParam(required = false)String operator,
	                               @ApiParam(name = "flag", value = "1为异常出药,0为正常") @RequestParam(required = false)String flag,
	                               @ApiParam(name = "page", value = "页面") @RequestParam(required = true)Integer page,
	                               @ApiParam(name = "pageSize", value = "页面大小") @RequestParam(required = true)Integer pageSize){
		try {
			return write(200, "获取成功", "data", expressageService.getPresEsPayList(getUID(),keyWord,state,type,startDate,endDate,operator,flag,page,pageSize));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getPresEsList", method = RequestMethod.GET)
	@ApiOperation(value = "获取出药记录列表")
	public String getPresEsList(@ApiParam(name = "keyWord", value = "订单号，接收人,医保号或身份证") @RequestParam(required = false)String keyWord,
	                            @ApiParam(name = "state", value = "续方状态") @RequestParam(required = false)String state,
	                            @ApiParam(name = "type", value = "1 自取 2快递配送 3健管师配送") @RequestParam(required = false)String type,
	                            @ApiParam(name = "startDate", value = "开始时间yyyy-MM--dd") @RequestParam(required = false)String startDate,
	                            @ApiParam(name = "endDate", value = "结束时间yyyy-MM-dd") @RequestParam(required = false)String endDate,
	                            @ApiParam(name = "operator", value = "操作人") @RequestParam(required = false)String operator,
	                            @ApiParam(name = "flag", value = "1为异常出药,0为正常") @RequestParam(required = false)String flag,
	                            @ApiParam(name = "page", value = "页面") @RequestParam(required = true)Integer page,
	                            @ApiParam(name = "pageSize", value = "页面大小") @RequestParam(required = true)Integer pageSize){
		try {return write(200, "获取成功", "data", expressageService.getPresEsList(getUID(),keyWord,state,type,startDate,endDate,operator,flag,page,pageSize));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getHospitalOperator", method = RequestMethod.GET)
	@ApiOperation(value = "获取操作人员下拉列表")
	public String getHospitalOperator(){
		try {
			return write(200, "获取成功", "data", expressageService.getHospitalOperator(getUID()));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	
	
	@RequestMapping(value = "/getPrescriptionExpressInfo" ,method = RequestMethod.GET)
	@ApiOperation(value = "获取处方的配送地址")
	public String getPrescriptionExpressInfo(@RequestParam(required = true)@ApiParam(name="code",value="续方CODE")String code){
		try {
			return write(200, "查询成功！", "data", expressageService.findByPrescriptionCode(code));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
}
