package com.yihu.jw.afterDiagnosis.endpoint.prescription;

import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.afterDiagnosis.endpoint.BaseController;
import com.yihu.jw.afterDiagnosis.service.prescription.PrescriptionInfoService;
import com.yihu.jw.afterDiagnosis.service.prescription.PrescriptionPayService;
import com.yihu.jw.afterDiagnosis.service.prescription.WlyyPrescriptionService;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.hospital.prescription.PrescriptionPay;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionDO;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionExpressageDO;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionInfoDO;
import com.yihu.jw.hospital.prescription.dao.PrescriptionExpressageDao;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.entity.ServiceException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * Created by Trick on 2017/7/25
 */
@RestController
@RequestMapping(value = "/doctor/prescriptionInfo")
@Api(description = "医生端-长处方接口")
public class PrescriptionInfoController extends BaseController {
	
	@Autowired
	private PrescriptionInfoService prescriptionInfoService;
	@Autowired
	private PrescriptionExpressageDao prescriptionExpressageDao;
	@Autowired
	private WlyyPrescriptionService prescriptionService;
	@Autowired
	private PrescriptionPayService prescriptionPayService;
	@Autowired
	private BasePatientDao patientDao;
	
	@RequestMapping(value = "getRecipeMasterList" , method = RequestMethod.GET)
	@ApiOperation(value = "获取智业长处方信息列表")
	public String getRecipeMasterList(@RequestParam(required = true)@ApiParam(name="patient",value="居民code",defaultValue = "ec7572875d27446cb4f067b13a85d72a")String patient,
	                                  @RequestParam(required = false)@ApiParam(name="isRenewal",value="处方是否可续方:1.是;2.无过滤")String isRenewal,
	                                  @RequestParam(required = false)@ApiParam(name="startDate",value="开始时间")String startDate,
	                                  @RequestParam(required = false)@ApiParam(name="endDate",value="结束时间")String endDate,
	                                  @RequestParam(required = false,defaultValue = "0")@ApiParam(name="diagnosisCode",value="所有诊断（0 全部 1高血压 2糖尿病 3高血压和糖尿病）")String diagnosisCode,
	                                  @RequestParam(required = false)@ApiParam(name="page",value="起始页")Integer page,
	                                  @RequestParam(required = false)@ApiParam(name="size",value="每页记录数")Integer size,
	                                  @RequestParam(required = false)@ApiParam(name="hospital",value="社区CODE")String hospital){
		try {
			JSONObject json = prescriptionInfoService.getRecipeMasterList(patient,startDate,endDate,isRenewal,diagnosisCode,page,size,hospital);
			return write(200, "查询成功！", "data",json);
		}catch (Exception e){
			return errorResult(e);
		}
	}
	
	/**
	 * 获取长处方信息列表
	 * @param type
	 * @return
	 */
	@RequestMapping(value = "/getPrescriptionInfos" , method = RequestMethod.GET)
	@ApiOperation(value = "获取长处方信息列表")
	public String getPrescriptionInfos(@RequestParam(required = true)@ApiParam(name="type",value="1:查询处方;2我的续方;3:续方记录")String type,
	                                   @RequestParam(required = false)@ApiParam(name="isRenewal",value="处方是否可续方:1.是;2.无过滤")String isRenewal,
	                                   @RequestParam(required = false)@ApiParam(name="recipeNo",value="医嘱号，第一次获取上一方时传入0，第二次获取上一方时传入第一次返回的医嘱号，以此类推")String recipeNo,
	                                   @RequestParam(required = false)@ApiParam(name="startDate",value="开始时间")String startDate,
	                                   @RequestParam(required = false)@ApiParam(name="endDate",value="结束时间")String endDate,
	                                   @RequestParam(required = false,defaultValue = "0")@ApiParam(name="diagnosisCode",value="所有诊断（0 全部 1高血压 2糖尿病 3高血压或糖尿病）")String diagnosisCode,
	                                   @RequestParam(required = false)@ApiParam(name="page",value="起始页")Integer page,
	                                   @RequestParam(required = false)@ApiParam(name="size",value="每页记录数")Integer size,
	                                   @RequestParam(required = true)@ApiParam(name="patientCode",value="居民CODE")String patientCode,
	                                   @RequestParam(required = false)@ApiParam(name="hospital",value="社区CODE")String hospital){
		try {
			return write(200, "查询成功！", "data", prescriptionInfoService.getPrescriptionInfos(type,recipeNo,diagnosisCode,startDate,endDate,patientCode,isRenewal,page,size,hospital));
		} catch (ServiceException se) {
			return write(-1,se.getMessage());
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getPrescriptionFilter", method = RequestMethod.GET)
	@ApiOperation(value = "获取过滤规则信息列表")
	public String getPrescriptionFilter(@RequestParam(required = false) @ApiParam(value = "团队code", name = "teamCode")Integer teamCode,
										@RequestParam(required = false) @ApiParam(value = "医生code", name = "doctorCode") String doctorCode,
	                                    @RequestParam(required = false) @ApiParam(value = "居民code", name = "patient")String patient) {
		try {
			return write(200, "查询成功！", "data", prescriptionInfoService.getPrescriptionFilter(teamCode,patient,doctorCode));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getDoctorPrescription", method = RequestMethod.GET)
	@ApiOperation(value = "获取全科医生/建管师续方列表/搜索")
	public String getDoctorPrescription(@RequestParam(required = false) @ApiParam(value = "团队code", name = "teamCode") Integer teamCode,
										@RequestParam(required = false) @ApiParam(value = "医生code", name = "doctorCode") String doctorCode,
	                                    @RequestParam(required = false) @ApiParam(value = "续方状态", name = "state") String state,
	                                    @RequestParam(required = false) @ApiParam(value = "所有诊断:1 糖尿病 2高血压，3其他", name = "diseases") String diseases,
	                                    @RequestParam(required = false) @ApiParam(name = "startDate", value = "开始时间") String startDate,
	                                    @RequestParam(required = false) @ApiParam(name = "endDate", value = "结束时间") String endDate,
	                                    @RequestParam(required = false) @ApiParam(name = "nameKeyword", value = "患者姓名模糊匹配") String nameKeyword,
	                                    @RequestParam(required = false) @ApiParam(name = "patient", value = "患者Code") String patient,
	                                    @RequestParam(required = false) @ApiParam(name = "page", value = "起始页") Integer page,
	                                    @RequestParam(required = false) @ApiParam(name = "size", value = "每页条数") Integer size) {
		try {
			return write(200, "查询成功！", "data", prescriptionInfoService.getDoctorPrescription(teamCode,doctorCode , state, diseases, startDate, endDate, nameKeyword, patient, page, size));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	@RequestMapping(value = "/getDoctorPrescriptionTotal", method = RequestMethod.GET)
	@ApiOperation(value = "获取全科医生数目/建管师续方列表数目/搜索数目")
	public String getDoctorPrescriptionTotal(@RequestParam(required = false) @ApiParam(value = "团队code", name = "teamCode") Integer teamCode,
	                                         @RequestParam(required = false) @ApiParam(value = "续方状态", name = "state") String state,
	                                         @RequestParam(required = false) @ApiParam(value = "所有诊断:1 糖尿病 2高血压,3其他", name = "dispensary") String diseases,
	                                         @RequestParam(required = false) @ApiParam(name = "startDate", value = "开始时间") String startDate,
	                                         @RequestParam(required = false) @ApiParam(name = "endDate", value = "结束时间") String endDate,
	                                         @RequestParam(required = false) @ApiParam(name = "nameKeyword", value = "患者姓名模糊匹配") String nameKeyword,
	                                         @RequestParam(required = false) @ApiParam(name = "patient", value = "患者Code") String patient) {
		try {
			return write(200, "查询成功！", "data", prescriptionInfoService.getDoctorPrescriptionTotal(teamCode, state, diseases, startDate, endDate, nameKeyword, patient));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	
	@RequestMapping(value = "/getContinuedPrescriptionAsDoctor", method = RequestMethod.GET)
	@ApiOperation(value = "续方详情")
	public String getContinuedPrescriptionAsDoctor(@RequestParam(required = true) @ApiParam(value = "续方CODE", name = "code") String code,
	                                               @RequestParam(required = true) @ApiParam(value = "团队长标识:1:是;2否", name = "type") String type) {
		try {
			return write(200, "查询成功！", "data", prescriptionInfoService.getContinuedPrescriptionAsDoctor(code));
		} catch (Exception e) {
			return errorResult(e);
		}
	}

    @RequestMapping(value = "/getOrgAddress", method = RequestMethod.GET)
    @ApiOperation(value = "获取医生所属机构药房")
    public String getOrgAddress(@RequestParam(required = true) @ApiParam(value = "所属机构CODE", name = "code") String code) {
        try {
            return write(200, "查询成功！","data",prescriptionInfoService.getOrgAddress(code));
        } catch (Exception e) {
			return errorResult(e);
        }
    }

	@RequestMapping(value = "/reviewPrescription", method = RequestMethod.POST)
	@ApiOperation(value = "续方审核")
	public String reviewPrescription(@RequestParam(required = true) @ApiParam(value = "续方CODE", name = "code") String code,
	                                 @RequestParam(required = false) @ApiParam(value = "不同意原因", name = "reason") String reason,
	                                 @RequestParam(required = false) @ApiParam(value = "不同意原因CODE", name = "reasonCode") String reasonCode,
	                                 @RequestParam(required = false) @ApiParam(value = "1同意,2不同意", name = "state") String state,
	                                 @RequestParam(required = false) @ApiParam(value = "科室", name = "dept") String dept,
	                                 @RequestParam(required = false) @ApiParam(value = "诊金", name = "registerFee") String registerFee,
	                                 @RequestParam(required = false) @ApiParam(value = "诊金类型", name = "rateTypeCode") String rateTypeCode,
									 @RequestParam(required = false) @ApiParam(value = "备注", name = "remark") String remark,
									 @RequestParam(required = false) @ApiParam(value = "科室code(智业校正减库存使用)", name = "deptCode") String deptCode,
									 @RequestParam(required = false) @ApiParam(value = "药房地址(顺丰取药使用)", name = "orgAddress") String orgAddress) {
		try {
			int rs = prescriptionInfoService.reviewPrescription(code, reason,reasonCode, state, dept, registerFee, rateTypeCode,remark,deptCode,orgAddress);
			if (rs == 0) {
				return write(200, "开方失败！", "data", rs);
			} else {
				return write(200, "成功提交开方", "data", rs);
			}
		} catch (ServiceException se) {
			return write(-1,se.getMessage());
		} catch (Exception e) {
			return errorResult(e);
		}
	}

	@RequestMapping(value = "/updatePresInfo", method = RequestMethod.POST)
	@ApiOperation(value = "调整处方")
	public String updatePresInfo(@RequestParam(required = true) @ApiParam(value = "续方CODE", name = "code") String code,
	                             @RequestParam(required = true) @ApiParam(value = "续方药品JSON", name = "infos") String infos,
	                             @RequestParam(required = false) @ApiParam(value = "疾病JSON", name = "diagnosis") String diagnosis,
	                             @RequestParam(required = false) @ApiParam(value = "调整原因", name = "reason") String reason,
	                             @RequestParam(required = false) @ApiParam(value = "调整原因标签Code,用,分割", name = "reasonCode") String reasonCode) {
		try {
			return write(200, "操作成功！", "data", prescriptionInfoService.updatePresInfo(code, infos, diagnosis, reason,reasonCode));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getInfoTitle", method = RequestMethod.POST)
	@ApiOperation(value = "调整处方-药品分类及种类数目")
	public String getInfoTitle() {
		try {
			return write(200, "操作成功！", "data", "");
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getInfoListByParentCode", method = RequestMethod.POST)
	@ApiOperation(value = "药品类别获取药品")
	private String getInfoListByParentCode(@RequestParam(required = true) @ApiParam(value = "药品类别Code", name = "code") String code) {
		try {
			return write(200, "操作成功！", "data", "");
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	/**
	 * 订单跟踪 包含处方药品信息 订单记录  支付记录
	 * 根据处方code获取处方流程
	 *
	 * @param prescriptionCode
	 * @return
	 */
	@RequestMapping(value = "/prescriptionFollow", method = RequestMethod.GET)
	@ApiOperation(value = "获取处方订单详情")
	public String prescriptionFollow(
			@RequestParam(required = true) @ApiParam(value = "处方code", name = "prescriptionCode") String prescriptionCode) {
		try {
			JSONObject jo = new JSONObject();
			//获取处方信息
			WlyyPrescriptionDO prescription = prescriptionService.findByCode(prescriptionCode);
			
			//获取处方药品信息
			List<WlyyPrescriptionInfoDO> prescriptionInfos = prescriptionInfoService.getPrescriptionInfo(prescriptionCode);
			//获取付款信息
			PrescriptionPay prescriptionPay = prescriptionPayService.findByPrescriptionCode(prescriptionCode);
			//获取配送信息
			WlyyPrescriptionExpressageDO prescriptionExpressage = prescriptionExpressageDao.findByPrescriptionCode(prescriptionCode);
			
			jo.put("prescriptionInfos", prescriptionInfos);
			jo.put("prescriptionPay", prescriptionPay);
			jo.put("prescriptionExpressage", prescriptionExpressage);
			if (prescription != null) {
				jo.put("createTime", prescription.getCreateTime());//创建时间
				jo.put("doctorName", prescription.getDoctorName());//审核医生
				jo.put("prescriptionHospital", prescription.getHospitalName());//出药机构
				jo.put("dispensaryDispensaryType", prescription.getDispensaryType());//处方配送方式
//				jo.put("dispensaryDispensaryTypeName", prescription.getDispensaryTypeName());//处方配送方式
				jo.put("prescriptionStatus", prescription.getStatus());//处方状态
//				jo.put("prescriptionStatusName", prescription.getStatusName(prescription.getStatus(), "",""));//处方状态名称
				BasePatientDO p =  patientDao.findById(prescription.getPatientCode()).orElse(null);
				jo.put("patient",p.getId());
				jo.put("patientName",p.getName());
				jo.put("IdCard",p.getIdcard());
				jo.put("ssc",p.getSsc());
//				jo.put("orgAddress",prescription.getOrgAddress());
//				jo.put("deptCode",prescription.getDeptCode());
//				jo.put("deptCodeName",prescription.getDeptCodeName());
			}
			return write(200, "查询成功", "data", jo);
		} catch (Exception e) {
			return errorResult(e);
		}
	}

	
	@RequestMapping(value = "/getHDoctorPrescriptionExpressage", method = RequestMethod.GET)
	@ApiOperation(value = "获取健康管理师续方订单列表")
	public String getHDoctorPrescriptionExpressage
			(@RequestParam(required = false) @ApiParam(value = "团队Code", name = "teamCode") String teamCode,
			 @RequestParam(required = false) @ApiParam(value = "1:需跟踪;2:已接收", name = "type") String type,
			 @RequestParam(required = false) @ApiParam(value = "患者姓名关键字", name = "nameKey") String nameKey,
			 @RequestParam(required = false) @ApiParam(value = "开始时间", name = "startDate") String startDate,
			 @RequestParam(required = false) @ApiParam(value = "结束时间", name = "endDate") String endDate,
			 @RequestParam(required = false) @ApiParam(value = "服务站", name = "hospital") String hospital,
			 @RequestParam(required = false) @ApiParam(value = "配送状态", name = "state") String state,
			 @RequestParam(required = false) @ApiParam(value = "页数", name = "page") Integer page,
			 @RequestParam(required = false) @ApiParam(value = "每页大小", name = "size") Integer size) {
		try {
			return write(200, "操作成功！", "data", prescriptionInfoService.getHDoctorPrescriptionExpressage(teamCode, type, getRepUID(), nameKey, startDate, endDate, hospital, state, page, size));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getHDoctorPrescriptionExpressageTotal", method = RequestMethod.GET)
	@ApiOperation(value = "获取健康管理师续方订单列表数目")
	public String getHDoctorPrescriptionExpressageTotal
			(@RequestParam(required = false) @ApiParam(value = "团队Code", name = "teamCode") String teamCode,
			 @RequestParam(required = false) @ApiParam(value = "1:需跟踪;2:已接收", name = "type") String type,
			 @RequestParam(required = false) @ApiParam(value = "患者姓名关键字", name = "nameKey") String nameKey,
			 @RequestParam(required = false) @ApiParam(value = "开始时间", name = "startDate") String startDate,
			 @RequestParam(required = false) @ApiParam(value = "结束时间", name = "endDate") String endDate,
			 @RequestParam(required = false) @ApiParam(value = "服务站", name = "hospital") String hospital,
			 @RequestParam(required = false) @ApiParam(value = "配送状态", name = "state") String state) {
		try {
			return write(200, "操作成功！", "data", prescriptionInfoService.getHDoctorPrescriptionExpressageTotal(teamCode, type, getUID(), nameKey, startDate, endDate, hospital, state));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getPrescriptionExpressageFilter", method = RequestMethod.GET)
	@ApiOperation(value = "获取健康管理师续方订单列表过滤条件")
	public String getPrescriptionExpressageFilter() {
		try {
			return write(200, "操作成功！", "data", prescriptionInfoService.getPrescriptionExpressageFilter(getUID()));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getPrescriptionExpressageAsdoctorFilter", method = RequestMethod.GET)
	@ApiOperation(value = "获取全科医生续方订单列表过滤条件")
	public String getPrescriptionExpressageAsdoctorFilter() {
		try {
			return write(200, "操作成功！", "data", prescriptionInfoService.getPrescriptionExpressageAsdoctorFilter());
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getDoctorPrescriptionExpressage", method = RequestMethod.GET)
	@ApiOperation(value = "获取全科医生续方订单列表")
	public String getDoctorPrescriptionExpressage
			(@RequestParam(required = false) @ApiParam(value = "团队Code", name = "teamCode") Integer teamCode,
			 @RequestParam(required = false) @ApiParam(value = "居民搜素关键字", name = "nameKey") String nameKey,
			 @RequestParam(required = false) @ApiParam(value = "开始时间", name = "startDate") String startDate,
			 @RequestParam(required = false) @ApiParam(value = "结束时间", name = "endDate") String endDate,
			 @RequestParam(required = false) @ApiParam(value = "机构", name = "hospital") String hospital,
			 @RequestParam(required = false) @ApiParam(value = "配送状态", name = "state") String state,
			 @RequestParam(required = false) @ApiParam(value = "配送方式:1 自取 2快递配送 3健管师配送", name = "dispensaryType") String
					 dispensaryType,
			 @RequestParam(required = false) @ApiParam(value = "传1为未分配健管师", name = "AllocationType") String
					 allocationType,
			 @RequestParam(required = false) @ApiParam(value = "起始页", name = "page") Integer page,
			 @RequestParam(required = false) @ApiParam(value = "每页大小", name = "size") Integer size) {
		try {
			return write(200, "操作成功！", "data",
					prescriptionInfoService.
							getDoctorPrescriptionExpressage(teamCode, nameKey, startDate, endDate, hospital, state, dispensaryType, allocationType, page, size));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getDoctorPrescriptionExpressageTotal", method = RequestMethod.GET)
	@ApiOperation(value = "获取全科医生续方订单列表数目")
	public String getDoctorPrescriptionExpressageTotal
			(@RequestParam(required = false) @ApiParam(value = "团队Code", name = "teamCode") Integer teamCode,
			 @RequestParam(required = false) @ApiParam(value = "居民搜素关键字", name = "nameKey") String nameKey,
			 @RequestParam(required = false) @ApiParam(value = "开始时间", name = "startDate") String startDate,
			 @RequestParam(required = false) @ApiParam(value = "结束时间", name = "endDate") String endDate,
			 @RequestParam(required = false) @ApiParam(value = "机构", name = "hospital") String hospital,
			 @RequestParam(required = false) @ApiParam(value = "配送状态", name = "state") String state,
			 @RequestParam(required = false) @ApiParam(value = "配送方式:1 自取 2快递配送 3健管师配送", name = "dispensaryType") String
					 dispensaryType,
			 @RequestParam(required = false) @ApiParam(value = "传1为未分配健管师", name = "AllocationType") String
					 allocationType) {
		try {
			return write(200, "操作成功！", "data",
					prescriptionInfoService.
							getDoctorPrescriptionExpressageTotal(teamCode, nameKey, startDate, endDate, hospital, state, dispensaryType, allocationType));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getCountExpressage", method = RequestMethod.GET)
	@ApiOperation(value = "获取指定团队未分配建管师订单数目")
	public String getCountExpressage
			(@RequestParam(required = true) @ApiParam(value = "团队Code", name = "teamCode") String teamCode) {
		try {
			return write(200, "操作成功！", "data", prescriptionInfoService.getCountExpressage(teamCode));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getHospitalListTitle", method = RequestMethod.GET)
	@ApiOperation(value = "获取服务站列表")
	public String getHospitalListTitle
			(@RequestParam(required = true) @ApiParam(value = "团队Code", name = "teamCode") String teamCode) {
		try {
			return write(200, "操作成功！", "data", prescriptionInfoService.getHospitalListTitle(teamCode));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/distributionHealthDoctor", method = RequestMethod.POST)
	@ApiOperation(value = "配置建管师")
	public String distributionHealthDoctor
			(@RequestParam(required = true) @ApiParam(value = "续方Code字符串用','分割", name = "codes") String codes,
			 @RequestParam(required = true) @ApiParam(value = "健康管理师Code", name = "healthDoctor") String
					 healthDoctor) {
		try {
			return write(200, "操作成功！", "data", 1);
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getIcd10Info", method = RequestMethod.GET)
	@ApiOperation(value = "获取诊断结果")
	public String getIcd10Info(@RequestParam(required = true) @ApiParam(value = "诊断结果关键字", name = "nameKey") String
			                           nameKey) {
		try {
			return write(200, "操作成功！", "data", prescriptionInfoService.getIcd10Info(nameKey));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getPrescriptionTemp", method = RequestMethod.GET)
	@ApiOperation(value = "获取调整处方模板")
	public String getPrescriptionTemp() {
		try {
			return write(200, "操作成功！", "data", prescriptionInfoService.getPrescriptionTemp(getUID()));
		} catch (ServiceException se) {
			return write(-1,se.getMessage());
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/initDrugData", method = RequestMethod.GET)
	@ApiOperation(value = "刷入redis平次数据")
	public String initDrugData() {
		try {
			return write(200, "操作成功！", "data", prescriptionInfoService.initDrugData());
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/drugToRate", method = RequestMethod.GET)
	@ApiOperation(value = "频次翻译")
	public String drugToRate(@RequestParam(required = true) @ApiParam(name = "code", value = "频次CODE") String code) {
		try {
			return write(200, "操作成功！", "data", prescriptionInfoService.drugToRate(code));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getPrescription", method = RequestMethod.GET)
	@ApiOperation(value = "获取长处方详细信息")
	public String getPrescription(@RequestParam(required = true) @ApiParam(name = "code", value = "处方CODE") String code
			,@RequestParam(required = true) @ApiParam(name = "patient", value = "居民CODE") String  patient,
			                      @RequestParam(required = false) @ApiParam(name = "preCode", value = "续方CODE")String preCode,
			                      @RequestParam(required = false) @ApiParam(name = "hospital", value = "社区CODE")String hospital) {
		try {
			return write(200, "查询成功！", "data", prescriptionInfoService.getPrescription(code,patient,preCode,hospital));
		} catch (ServiceException se) {
			return write(-1,se.getMessage());
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getRateList", method = RequestMethod.GET)
	@ApiOperation(value = "查询频次列表")
	public String getRateList() {
		try {
			return write(200, "查询成功！", "data", prescriptionInfoService.getRateList());
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getRegisterRee", method = RequestMethod.GET)
	@ApiOperation(value = "获取挂号费+诊察费下拉列表")
	public String getRegisterRee
			(@RequestParam(required = true) @ApiParam(name = "jwHospital", value = "开方机构") String jwHospital,
			 @RequestParam(required = true) @ApiParam(name = "jwDoctorCode", value = "开方医生Code") String jwDoctorCode) {
		try {
			return write(200, "查询成功！", "data", prescriptionInfoService.getRegisterRee(jwHospital, jwDoctorCode));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getDeptList", method = RequestMethod.GET)
	@ApiOperation(value = "获取医生科室列表")
	public String getDeptList(@RequestParam(required = true) @ApiParam(name = "jwHospital", value = "开方机构") String
			                          jwHospital,
	                          @RequestParam(required = true) @ApiParam(name = "jwDoctorCode", value = "开方医生Code") String jwDoctorCode) {
		try {
			return write(200, "查询成功！", "data", prescriptionInfoService.getDeptList(jwHospital, jwDoctorCode));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getUsageDict", method = RequestMethod.GET)
	@ApiOperation(value = "获取药品用法列表")
	public String getUsageDict() {
		try {
			return write(200, "查询成功！", "data", prescriptionInfoService.getUsageDict());
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getPhysicDays", method = RequestMethod.GET)
	@ApiOperation(value = "获取药品用法天数")
	public String getPhysicDays() {
		try {
			return write(200, "查询成功！", "data", prescriptionInfoService.getPhysicDays());
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getPhysicInfo", method = RequestMethod.GET)
	@ApiOperation(value = "获取药品信息")
	public String getPhysicInfo(@RequestParam(required = true) @ApiParam(name = "code", value = "药品code") String code) {
		try {
			return write(200, "查询成功！", "data", prescriptionInfoService.getPhysicInfo(code,getUID()));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	@RequestMapping(value = "/getReasonByType", method = RequestMethod.GET)
	@ApiOperation(value = "获取原因")
	public String getReasonByType(@RequestParam(required = true) @ApiParam(name = "code", value = "类型：0 审核原因标签；1 调整原因标签") Integer type) {
		try {
			return write(200, "查询成功！", "data", prescriptionInfoService.getReasonByType(type));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	/**
	 * 长处方统计--长处方记录筛选
	 * @Author zhangdan
	 * @param level 地区级别
	 * @param area 地区编号
	 * @param disease 疾病 高血压1 糖尿病2
	 * @param status 状态 1.总量，2.已完成，3.居民取消，4.审核不通过，5.进行中，6.其他原因取消(没有选择的情况传1)
	 * @param type 取药方式 1.自取，2快递配送，3健管师配送
	 * @param startTime 开始日期
	 * @param endTime 结束日期
	 * @return
	 */
	@RequestMapping(value = "/getPrescriptionByCondition", method = RequestMethod.GET)
	public String getPrescriptionByCondition(
			@RequestParam(required = true)String level,
			@RequestParam(required = true)String area,
			@RequestParam(required = false)String disease,
			@RequestParam(required = false,defaultValue = "1")String status,
			@RequestParam(required = false)String type,
			@RequestParam(required = false)String startTime,
			@RequestParam(required = false)String endTime,
			@RequestParam(required = true,defaultValue = "1")String pageNo,
			@RequestParam(required = false)String pageSize){
		try{
			if (StringUtils.isNotBlank(startTime)){
				if (StringUtils.isNotBlank(endTime)){
					Date startDate = DateUtil.strToDate(startTime,"yyyy-MM-dd");
					Date endDate = DateUtil.strToDate(endTime,"yyyy-MM-dd");
					if (startDate.after(endDate)){
						return  error(-2,"参数错误");
					}
				}
			}
			if (StringUtils.isBlank(pageSize)){
				pageSize="10";
			}
			return write(200, "查询成功", "data", prescriptionService.getPrescriptionByCondition(level,area,disease,status,type,startTime,endTime,Integer.valueOf(pageNo),Integer.valueOf(pageSize)));
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	/**
	 * 长处方统计--上方模糊搜索
	 * @Author zhangdan
	 * @param keyWord 模糊搜索关键字
	 * @return
	 */
	@RequestMapping(value = "/getPrescriptionByKeyWord", method = RequestMethod.GET)
	public String getPrescriptionByKeyWord(
			@RequestParam(required = true)String keyWord,
			@RequestParam(required = true,defaultValue = "1")String pageNo,
			@RequestParam(required = false)String pageSize){
		try{
			if (StringUtils.isBlank(pageSize)){
				pageSize="10";
			}
			return write(200, "查询成功", "data", prescriptionService.getPrescriptionByKeyWord(keyWord,Integer.valueOf(pageNo),Integer.valueOf(pageSize)));
		} catch (Exception e) {
			return errorResult(e);
		}
	}

	
	/**
	 * 长处方-修改审核检查步骤
	 * @param viewTag 模糊搜索关键字
	 * @param status 模糊搜索关键字
	 * @param prescriptionCode 处方CODE
	 * @return
	 */
	@RequestMapping(value = "/updatePrescriptionViewStatus", method = RequestMethod.POST)
	public String updatePrescriptionViewStatus(@RequestParam(required = true) @ApiParam(name = "viewTag", value = "审核步骤")String viewTag,
	                                           @RequestParam(required = true) @ApiParam(name = "status", value = "0未完成，1已完成")Integer status,
	                                           @RequestParam(required = true) @ApiParam(name = "prescriptionCode", value = "处方CODE")String prescriptionCode
	                                           ){
		try{
//			prescriptionService.updatePrescriptionViewStatus(viewTag,status,prescriptionCode);
			return write(200, "操作成功");
		} catch (ServiceException se) {
			return write(-1,se.getMessage());
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	/**
	 * 续方确认收药接口
	 *
	 * @param prescriptionCode 续方code原状态为60
	 * @param type 1:确认收药 2：延长收药
	 * @return
	 */
	@RequestMapping(value = "/confirmReceipt", method = RequestMethod.POST)
	@ApiOperation(value = "续方确认收药接口")
	public String confirmReceipt(
			@RequestParam(required = true) @ApiParam(value = "处方code", name = "prescriptionCode") String prescriptionCode,
			@RequestParam(required = true) @ApiParam(value = "收药方式", name = "type") Integer type) {
		try {
			Map<String,Object> result = prescriptionService.confirmReceipt(prescriptionCode,type);
			int code = Integer.parseInt(result.get("code").toString());
			String msg = result.get("msg").toString();
			return write(code, msg);
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	/**
	 * 修改处方收货地址
	 *
	 * @param prescriptionCode 续方code原状态为60
	 * @param addressJson
	 * @return
	 */
	@RequestMapping(value = "/updateAddress", method = RequestMethod.POST)
	@ApiOperation(value = "修改处方收货地址")
	public String updateAddress(
			@RequestParam(required = true) @ApiParam(value = "处方code", name = "prescriptionCode") String prescriptionCode,
			@RequestParam(required = true) @ApiParam(value = "派送方式", name = "dispensaryType") Integer dispensaryType,
			@RequestParam(required = true) @ApiParam(value = "配送地址json", name = "addressJson", defaultValue = "{\"townName\":\"海沧区\",\"code\":\"3502050100\",\"address\":" +
					"\"冰岛\",\"cityName\":\"厦门市\",\"townCode\":\"350205\",\"provinceCode\":\"350000\",\"cityCode\":\"350200\",\"name\":\"海沧区嵩屿街道社区卫生服务中心\"," +
					"\"provinceName\":\"福建省\",\"streeCode\":\"35020501\",\"streeName\":\"皇后大道东\",\"phone\":\"13253541190\"}") String addressJson) {
		try {
			prescriptionService.updateAddress(prescriptionCode,addressJson,dispensaryType);
			return write(200, "操作成功");
		} catch (Exception e) {
			return errorResult(e);
		}
	}
}
