package com.yihu.jw.afterDiagnosis.endpoint.prescription;


import com.yihu.jw.afterDiagnosis.endpoint.BaseController;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionExpressageLogDO;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionLogDO;
import com.yihu.jw.hospital.prescription.dao.PrescriptionDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionExpressageLogDao;
import com.yihu.jw.hospital.prescription.service.PrescriptionLogService;
import com.yihu.jw.util.date.DateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * Created by chenweida on 2017/7/27.
 */
@RestController("doctorPrescriptionLogController")
@RequestMapping(value = "/doctor/prescriptionlog")
@Api(description = "医生端-处方日志")
public class PrescriptionLogController extends BaseController {
	@Autowired
	private PrescriptionLogService prescriptionLogService;
	@Autowired
	private PrescriptionExpressageLogDao prescriptionExpressageLogDao;
	@Autowired
	private PrescriptionDao prescriptionDao;
	
	@RequestMapping(value = "/getPrescriptionLogs", method = RequestMethod.GET)
	@ApiOperation(value = "获取处方流程")
	public String addReply(
			@RequestParam(required = true, name = "prescriptionCode") @ApiParam(value = "处方code", name = "prescriptionCode") String prescriptionCode) {
		try {
			// (-3 支付过期 -2 患者自己取消 -1 审核不通过 , 0 待审核, 2调整中 10 审核通过/开方中  , 20开方完成/待支付, 21 支付失败 , 30 支付成功/待配药 ,
			//  40配药成功/待配送  41配送失败  42配送中   43配送到服务站  100配送到患者手中/已完成)
			Integer[] status = new Integer[]{
					WlyyPrescriptionLogDO.PrescriptionLogStatus.revieweding.getValue(),
					WlyyPrescriptionLogDO.PrescriptionLogStatus.changeing.getValue(),
					WlyyPrescriptionLogDO.PrescriptionLogStatus.reviewed_success.getValue(),
					WlyyPrescriptionLogDO.PrescriptionLogStatus.wait_pay.getValue(),
					WlyyPrescriptionLogDO.PrescriptionLogStatus.pay_success.getValue(),
					WlyyPrescriptionLogDO.PrescriptionLogStatus.wait_expressage.getValue(),
					WlyyPrescriptionLogDO.PrescriptionLogStatus.expressageing.getValue(),
					WlyyPrescriptionLogDO.PrescriptionLogStatus.finish.getValue()};
			List<WlyyPrescriptionLogDO> prescriptionLogs = prescriptionLogService.findPrescriptionLogsByPrescriptionCode(prescriptionCode, status);
			if (prescriptionLogs != null) {
				JSONArray ja = new JSONArray();
				
				prescriptionLogs.stream().forEach(one -> {
					JSONObject jo = new JSONObject();
					jo.put("createTime", DateUtil.dateToStrLong(one.getCreateTime()));
//					jo.put("statusName", one.getStatusName());
					ja.put(jo);
				});
				return write(200, "获取处方流程成功", "data", ja);
			} else {
				return error(-1, "获取失败");
			}
		} catch (Exception e) {
			return errorResult(e);
		}
	}
	
	/**
	 * 根据处方code获取处方流程
	 *
	 * @param prescriptionCode
	 * @return
	 */
	@RequestMapping(value = "/orderFollow", method = RequestMethod.GET)
	@ApiOperation(value = "订单状态跟踪")
	public String orderFollow(
			@ApiParam(value = "处方code", name = "prescriptionCode") @RequestParam(required = true, name = "prescriptionCode") String prescriptionCode) {
		try {
			
			// (-3 支付过期 -2 患者自己取消 -1 审核不通过 , 0 待审核, 2调整中 10 审核通过/开方中  , 20开方完成/待支付, 21 支付失败 , 30 支付成功/待配药 ,
			//  40配药成功/待配送  41配送失败  42配送中   43配送到服务站  100配送到患者手中/已完成)
			Integer[] status = new Integer[]{
					WlyyPrescriptionLogDO.PrescriptionLogStatus.reviewed_success.getValue(),
//                    WlyyPrescriptionLogDO.PrescriptionLogStatus.adding.getValue(), //暂无此状态
					WlyyPrescriptionLogDO.PrescriptionLogStatus.pay_success.getValue(),
					WlyyPrescriptionLogDO.PrescriptionLogStatus.wait_expressage.getValue(),
					WlyyPrescriptionLogDO.PrescriptionLogStatus.expressageing.getValue(),
					WlyyPrescriptionLogDO.PrescriptionLogStatus.finish.getValue()};
			List<WlyyPrescriptionLogDO> prescriptionLogs = prescriptionLogService.findPrescriptionLogsByPrescriptionCode(prescriptionCode, status);
			
			//加上快递的log
			List<WlyyPrescriptionExpressageLogDO> expressageLogs = prescriptionExpressageLogDao.findByPrescriptionCode(prescriptionCode);
			for (int i = 0; i < expressageLogs.size(); i++) {
				WlyyPrescriptionExpressageLogDO prescriptionExpressageLog = expressageLogs.get(i);
				WlyyPrescriptionLogDO prescriptionLog = new WlyyPrescriptionLogDO();
				prescriptionLog.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.expressage2hospital.getValue());
				prescriptionLog.setCreateTime(prescriptionExpressageLog.getCreateTime());
//				prescriptionLog.setHospital(prescriptionExpressageLog.getAcceptHospital());
//				prescriptionLog.setHospitalName(prescriptionExpressageLog.getAcceptHospitalName());
				prescriptionLog.setUserCode(prescriptionExpressageLog.getAcceptUser());
				prescriptionLog.setUserName(prescriptionExpressageLog.getAcceptUserName());
				prescriptionLogs.add(prescriptionLog);
			}
			//排序
			prescriptionLogs.sort((o1, o2) -> {
				long map1value = o1.getCreateTime().getTime();
				long map2value = o2.getCreateTime().getTime();
				
				if (map1value - map2value > 0) {
					return -1;
				} else if (map1value - map2value < 0) {
					return 1;
				} else {
					return 0;
				}
			});
			if (prescriptionLogs != null && prescriptionLogs.size() > 0) {
				JSONArray ja = new JSONArray();
				for (int i = 0; i < prescriptionLogs.size(); i++) {
					WlyyPrescriptionLogDO one = prescriptionLogs.get(i);
					JSONObject jo = new JSONObject();
					jo.put("createTime", DateUtil.dateToStrLong(one.getCreateTime()));
//					jo.put("statusName", one.getStatusName());
					jo.put("drugDeliveryflag", "0");
					ja.put(jo);
				}
				return write(200, "获取处方流程成功", "data", ja);
			} else {
				return error(-1, "获取失败");
			}
		} catch (Exception e){
			return errorResult(e);
		}
	}
}
