package com.yihu.jw.afterDiagnosis.endpoint.rehabilitation;


import com.yihu.jw.afterDiagnosis.endpoint.BaseController;
import com.yihu.jw.afterDiagnosis.service.rehabilitation.DiagnosisInformationService;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.util.entity.ServiceException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * Created by humingfen on 2019/2/11.
 */
@RestController
@RequestMapping(value = "/doctor/specialist/diagnosisInformation")
@Api(description = "医生端-诊疗信息管理")
public class DoctorDiagnosisInformationController extends BaseController {

    @Autowired
    private BaseDoctorDao doctorDao;
    @Autowired
    private DiagnosisInformationService diagnosisInformationService;

    @RequestMapping(value = "/createDiagnosisInformation", method = RequestMethod.POST)
    @ApiOperation(value = "创建诊疗信息")
    public String createDiagnosisInformation(@ApiParam(name = "patient", value = "居民code")
                                 @RequestParam String patient,
                                 @ApiParam(name = "patientName", value = "居民姓名")
                                 @RequestParam String patientName,
                                 @ApiParam(name = "planId", value = "康复计划id")
                                     @RequestParam String planId,
                                 @ApiParam(name = "hospital", value = "诊疗医院code")
                                     @RequestParam(required = false) String hospital,
                                 @ApiParam(name = "hospitalName", value = "诊疗医院名称")
                                     @RequestParam String hospitalName,
                                 @ApiParam(name = "department", value = "入院科室")
                                     @RequestParam String department,
                                 @ApiParam(name = "attendingDoctorName", value = "主治医生名称")
                                     @RequestParam String attendingDoctorName,
                                 @ApiParam(name = "admittingDiagnosis", value = "入院诊断")
                                     @RequestParam String admittingDiagnosis,
                                 @ApiParam(name = "dischargeDiagnosis", value = "出院诊断")
                                     @RequestParam String dischargeDiagnosis,
                                 @ApiParam(name = "advice", value = "医嘱小结")
                                     @RequestParam String advice,
                                 @ApiParam(name = "dischargeTime", value = "出院时间")
                                     @RequestParam String dischargeTime){
        try {
            BaseDoctorDO d = doctorDao.findByIdAndDel(getUID());
            return write(200, "保存成功", "data", diagnosisInformationService.createDiagnosisInformation(patient, patientName, planId, hospital, hospitalName, department,
                    attendingDoctorName, admittingDiagnosis, dischargeDiagnosis, advice, dischargeTime, d));
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "/findDiagnosisInformationByPlanId", method = RequestMethod.GET)
    @ApiOperation(value = "根据康复计划id获取诊疗信息")
    public ObjEnvelop findDiagnosisInformationByPlanId(@ApiParam(name = "planId", value = "康复计划id")
                                                   @RequestParam String planId){
        try {
            return diagnosisInformationService.findDiagnosisInformationByPlanId(planId);
        } catch (ServiceException se) {
            return ObjEnvelop.getError(se.getMessage());
        } catch (Exception e) {
            return ObjEnvelop.getError("获取失败",-1);
        }
    }

    @RequestMapping(value = "/findDiagnosisInformationByPatient", method = RequestMethod.GET)
    @ApiOperation(value = "根据居民获取诊疗信息")
    public PageEnvelop findDiagnosisInformationByPatient(@ApiParam(name = "patient", value = "居民code")
                                                        @RequestParam(value = "patient") String patient,
                                                         @ApiParam(name = "page", value = "第几页，page从0开始")
                                                        @RequestParam(value = "page") Integer page,
                                                         @ApiParam(name = "size", value = "行数")
                                                        @RequestParam(value = "size") Integer size){
        try {
            return diagnosisInformationService.findDiagnosisInformationByPatient(patient, page, size);
        } catch (Exception e) {
            return PageEnvelop.getError("获取失败",-1);
        }
    }
}
