package com.yihu.jw.afterDiagnosis.endpoint.rehabilitation;

import com.yihu.jw.afterDiagnosis.endpoint.BaseController;
import com.yihu.jw.afterDiagnosis.service.rehabilitation.*;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.rehabilitation.RehabilitationguidancePatientDO;
import com.yihu.jw.entity.specialist.rehabilitation.PatientMedicalRecordsDO;
import com.yihu.jw.entity.specialist.rehabilitation.RehabilitationOperateRecordsDO;
import com.yihu.jw.im.util.ImUtil;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.entity.ServiceException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Created by 刘文彬 on 2018/8/30.
 */
@RestController
@RequestMapping(value = "/doctor/specialist/rehabilitation", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(description = "医生端-康复管理")
public class DoctorRehabilitationManageController extends BaseController {

    @Autowired
    private RehabilitationManageService rehabilitationManageService;
    @Autowired
    private WlyyRehabilitationManageService wlyyRehabilitationManageService;
    @Autowired
    private WlyyRehabilitationPlanService wlyyRehabilitationPlanService;
    @Autowired
    private ImUtil imUtill;
    @Autowired
    private RehabilitationPlanService rehabilitationPlanService;
    @Autowired
    private RehabilitationGuidanceService rehabilitationGuidanceService;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private BaseDoctorDao doctorDao;

    @RequestMapping(value = "findRehabilitationPlanList", method = RequestMethod.GET)
    @ApiOperation("康复管理-康复计划列表")
    public MixEnvelop findRehabilitationPlanList(@ApiParam(name = "doctorCode", value = "医生code", required = false)
                                             @RequestParam(value = "doctorCode", required = false)String doctorCode,
                                                 @ApiParam(name = "patientCondition", value = "居民条件，可以按身份证或者居民名称模糊匹配", required = false)
                                             @RequestParam(value = "patientCondition", required = false)String patientCondition,
                                                 @ApiParam(name = "diseaseCode", value = "疾病类型code", required = false)
                                             @RequestParam(value = "diseaseCode", required = false)String diseaseCode,
                                                 @ApiParam(name = "planType", value = "安排类型（1康复计划，2转社区医院，3转家庭病床）", required = false)
                                             @RequestParam(value = "planType", required = false)Integer planType,
                                                 @ApiParam(name = "todaybacklog", value = "今日待办（1、今日待办，2、全部）", required = false)
                                             @RequestParam(value = "todaybacklog", required = false,defaultValue = "1")Integer todaybacklog,
                                                 @ApiParam(name = "page", value = "第几页，从1开始", required = true)
                                             @RequestParam(value = "page", required = false,defaultValue = "1")Integer page,
                                                 @ApiParam(name = "pageSize", value = "每页分页大小", required = true)
                                             @RequestParam(value = "pageSize", required = false,defaultValue = "10")Integer pageSize) {
        try {
            if(!StringUtils.isNotEmpty(doctorCode)){
                doctorCode = getRepUID();
            }
            return rehabilitationManageService.findRehabilitationPlan(doctorCode,diseaseCode,planType,todaybacklog,patientCondition,page,pageSize);
        } catch (ServiceException se) {
            return MixEnvelop.getError(se.getMessage(),-1);
        } catch (Exception e) {
            return MixEnvelop.getError("获取失败",-1);
        }
    }

    @RequestMapping(value = "selectBySpecialistDoctor", method = RequestMethod.GET)
    @ApiOperation("康复管理-获取签约专科居民")
    public String selectBySpecialistDoctor(@ApiParam(name = "doctorCode", value = "医生code", required = false)
                                             @RequestParam(value = "doctorCode", required = false)String doctorCode,
                                           @ApiParam(name = "doctorType", value = "医生类型（1专科2家医）", required = false)
                                           @RequestParam(value = "doctorType", required = false)String doctorType,
                                           @ApiParam(name = "patientInfo", value = "身份证或者姓名", required = false)
                                               @RequestParam(value = "patientInfo", required = false)String patientInfo,
                                           @ApiParam(name = "diseaseCode", value = "疾病类型code", required = false)
                                             @RequestParam(value = "diseaseCode", required = false)String diseaseCode,
                                           @ApiParam(name = "level", value = "1市2区3社区", required = false)
                                             @RequestParam(value = "level", required = false)String level,
                                           @ApiParam(name = "area", value = "区域", required = false)
                                             @RequestParam(value = "area", required = false)String area,
                                           @ApiParam(name = "page", value = "第几页，从1开始", required = true)
                                             @RequestParam(value = "page", required = false,defaultValue = "1")Integer page,
                                           @ApiParam(name = "pageSize", value = "每页分页大小", required = true)
                                             @RequestParam(value = "pageSize", required = false,defaultValue = "10")Integer pageSize,
                                           @ApiParam(name = "filterPlan", value = "是否过滤康复计划，1过滤，0或者放空不过滤", required = false)
                                               @RequestParam(value = "filterPlan", required = false)String filterPlan,
                                           @ApiParam(name = "turnDownState", value = "下转状态 0全部;1已下转;2已接受;3未下转", required = false)
                                           @RequestParam(value="turnDownState", required = false,defaultValue = "0")String turnDownState
                                           ) {
        try {
            if(!StringUtils.isNotEmpty(doctorCode)){
                doctorCode = getRepUID();
            }
            com.alibaba.fastjson.JSONObject result = wlyyRehabilitationManageService.selectBySpecialistDoctor(level,area,doctorCode,diseaseCode,page,pageSize,doctorType,patientInfo,filterPlan,turnDownState);
            com.alibaba.fastjson.JSONArray showList = result.getJSONArray("showList");
            com.alibaba.fastjson.JSONObject turnDownStateObj = result.getJSONObject("turnDownState");
            return write(200, "获取成功", "data", showList,"data2",turnDownStateObj);
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "findRehabilitationPlanDetailList", method = RequestMethod.GET)
    @ApiOperation("康复管理-康复计划详情列表")
    public ObjEnvelop findRehabilitationPlanDetailList(@ApiParam(name = "doctorCode", value = "医生code", required = false)
                                                   @RequestParam(value = "doctorCode", required = false)String doctorCode,
                                                       @ApiParam(name = "patientCode", value = "居民code", required = true)
                                                   @RequestParam(value = "patientCode", required = true)String patientCode){

        try {
            if(!StringUtils.isNotEmpty(doctorCode)){
                doctorCode = getRepUID();
            }
            return rehabilitationManageService.findRehabilitationPlanDetailList(doctorCode,patientCode);
        } catch (ServiceException se) {
            return ObjEnvelop.getError(se.getMessage(),-1);
        } catch (Exception e) {
            return ObjEnvelop.getError("获取失败",-1);
        }
    }


    @RequestMapping(value = "getPrescriptionFulfil",method = RequestMethod.GET)
    @ApiOperation("获取已完成处方续方")
    public String getPrescriptionFulfil(@ApiParam(name = "patient", value = "居民code", required = false)
                                        @RequestParam(value = "patient", required = false)String patient,
                                        @ApiParam(name = "doctor", value = "医生code", required = false)
                                        @RequestParam(value = "doctor", required = false)String doctor
    ){

        try {
            return write(200, "获取成功！", "data", wlyyRehabilitationManageService.getPrescriptionFulfil(patient,doctor));
        } catch (Exception e) {
            error(e);
            return error(-1, "请求失败");
        }
    }


    @RequestMapping(value = "getRehabilitationConsultationList",method = RequestMethod.GET)
    @ApiOperation("康复咨询")
    public String getRehabilitationConsultationList(@ApiParam(name = "patient",value = "居民code",required = false)
                                                    @RequestParam(value = "patient",required = false)String patient,
                                                    @ApiParam(name = "doctor",value = "医生code",required = false)
                                                    @RequestParam(value = "doctor",required = false)String doctor,
                                                    @ApiParam(name = "brief ",value = "简介 (1综合 0列表 2详细)",required = false)
                                                    @RequestParam(value = "brief",required = false)String brief
    ){
        try {
            return write(200,"获取成功！","data",wlyyRehabilitationManageService.getRehabilitationConsultationList(patient,doctor,brief));
        } catch (Exception e) {
            error(e);
            return error(-1,"请求失败");
        }
    }

    @RequestMapping(value = "getFollowupPlan",method = RequestMethod.GET)
    @ApiOperation("随访计划")
    public String getFollowupPlan(@ApiParam(name = "patient",value = "居民code",required = true)
                                  @RequestParam(value = "patient",required = true)String patient,
                                  @ApiParam(name="doctor",value = "医生code",required = true)
                                  @RequestParam(value = "doctor",required = true)String doctor,
                                  @ApiParam(name = "id",value = "随访id",required = false)
                                  @RequestParam(value = "id",required = false)String id,
                                  @ApiParam(name = "biref",value = "简介",required = false)
                                  @RequestParam(value = "biref",required = false)String biref){

        try {
            return write(200,"获取成功","data",wlyyRehabilitationManageService.getFollowupPlan(doctor,patient,id,biref));
        } catch (Exception e) {
            error(e);
            return error(-1,"请求失败");
        }
    }



    @ApiOperation("获取居民最近的一次住院病历")//wlyy_patient_medical_records_rehabilitation
    @RequestMapping(value = "getNewRehabilitation", method = RequestMethod.GET)
    public String getNewRehabilitation(
                                           @ApiParam(name = "patientCode", value = "居民code", required = true)
                                           @RequestParam(value = "patientCode", required = true)String patientCode){
        try {
            List<PatientMedicalRecordsDO> result = wlyyRehabilitationManageService.getNewRehabilitation(patientCode,getRepUID());
            return write(200, "获取成功", "data", result);
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "selectByGuidance", method = RequestMethod.GET)
    @ApiOperation("康复管理-根据计划查询指导留言")
    public String selectByGuidance(@ApiParam(name = "planId", value = "计划id", required = false)
                                                   @RequestParam(value = "planId", required = true)String planId,
                                                   @ApiParam(name = "itemCode", value = "服务项目code", required = true)
                                                   @RequestParam(value = "itemCode", required = true)String itemCode){

        try {
            List<Map<String,Object>> mapList = wlyyRehabilitationManageService.selectByGuidance(planId,itemCode);
            return write(200, "获取成功", "data", mapList);
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "calendarPlanDetail", method = RequestMethod.GET)
    @ApiOperation("康复管理-康复计划按日历展示")
    public ObjEnvelop calendarPlanDetail(@ApiParam(name = "executeStartTime", value = "日历开始时间（格式：yyyy-MM-dd HH:mm:ss）")
                                     @RequestParam(value = "executeStartTime", required = false)String executeStartTime,
                                     @ApiParam(name = "executeEndTime", value = "日历结束时间（格式：yyyy-MM-dd HH:mm:ss）")
                                     @RequestParam(value = "executeEndTime", required = false)String executeEndTime,
                                     @ApiParam(name = "planId", value = "计划id", required = true)
                                     @RequestParam(value = "planId")String planId,
                                     @ApiParam(name = "searchTask", value = "快速查找任务：（1电话/短信关怀,2康复咨询,3健康监测,4上门服务,5康复指导,6康复随访,7康复复诊）", required = false)
                                     @RequestParam(value = "searchTask", required = false)Integer searchTask,
                                     @ApiParam(name = "doctorCode", value = "医生code（专科医生、家庭医生）", required = false)
                                     @RequestParam(value = "doctorCode", required = false)String doctorCode,
                                     @ApiParam(name = "status", value = "任务状态（0未完成，1已完成，2已预约）", required = false)
                                     @RequestParam(value = "status", required = false)Integer status,
                                     @ApiParam(name="taskExecutor",value = "任务执行人：0全部；1我的任务：2他人任务",required = false)
                                     @RequestParam(value = "taskExecutor",required = false,defaultValue = "0")String taskExecutor){
        try {
            if(!StringUtils.isNotEmpty(doctorCode)){
                doctorCode = getRepUID();
            }
            return rehabilitationManageService.calendarPlanDetail(executeStartTime,executeEndTime,planId,searchTask,status,doctorCode,taskExecutor);
        } catch (ServiceException se) {
            return ObjEnvelop.getError(se.getMessage(),-1);
        } catch (Exception e) {
            return ObjEnvelop.getError("获取失败",-1);
        }
    }

    @RequestMapping(value = "calendarPlanDetailBypatient", method = RequestMethod.GET)
    @ApiOperation("康复管理--某患者的康复计划按日历展示")
    public ObjEnvelop calendarPlanDetailBypatient(@ApiParam(name = "executeStartTime", value = "日历开始时间（格式：yyyy-MM-dd HH:mm:ss）", required = false)
                                                  @RequestParam(value = "executeStartTime", required = false)String executeStartTime,
                                                  @ApiParam(name = "executeEndTime", value = "日历结束时间（格式：yyyy-MM-dd HH:mm:ss）", required = false)
                                                  @RequestParam(value = "executeEndTime", required = false)String executeEndTime,
                                                  @ApiParam(name = "patient", value = "患者code", required = true)
                                                  @RequestParam(value = "patient", required = true)String patient,
                                                  @ApiParam(name = "searchTask", value = "快速查找任务：（1电话/短信关怀,2康复咨询,3健康监测,4上门服务,5康复指导,6康复随访,7康复复诊）", required = false)
                                                  @RequestParam(value = "searchTask", required = false)Integer searchTask,
                                                  @ApiParam(name = "doctorCode", value = "医生code（专科医生、家庭医生）", required = false)
                                                  @RequestParam(value = "doctorCode", required = false)String doctorCode,
                                                  @ApiParam(name = "status", value = "任务状态（0未完成，1已完成，2已预约）", required = false)
                                                  @RequestParam(value = "status", required = false)Integer status,
                                                  @ApiParam(name="taskExecutor",value = "任务执行人：0全部；1我的任务：2他人任务",required = false)
                                                  @RequestParam(value = "taskExecutor",required = false,defaultValue = "0")String taskExecutor){
        try {
            if(!StringUtils.isNotEmpty(doctorCode)){
                doctorCode = getRepUID();
            }
            //居民康复计划
            List<Map<String,Object>> planList = wlyyRehabilitationManageService.getPatientAllRehabilitationPlan(patient,doctorCode);
            StringBuilder planIds = new StringBuilder();
            planIds.append(planList.get(0).get("id").toString());
            for (int i =1;i<planList.size();i++){
                planIds.append(","+planList.get(i).get("id").toString());
            }
            return rehabilitationManageService.calendarPlanDetail(executeStartTime,executeEndTime,planIds.toString(),searchTask,status,doctorCode,taskExecutor);
        } catch (ServiceException se) {
            return ObjEnvelop.getError(se.getMessage(),-1);
        } catch (Exception e) {
            return ObjEnvelop.getError("获取失败",-1);
        }
    }

    @RequestMapping(value = "calendarPlanDetailList", method = RequestMethod.GET)
    @ApiOperation("康复管理-康复计划按列表展示-限定频次")
    public ObjEnvelop calendarPlanDetailList(@ApiParam(name = "executeStartTime", value = "日历开始时间（格式：yyyy-MM-dd HH:mm:ss）", required = true)
                                     @RequestParam(value = "executeStartTime", required = false)String executeStartTime,
                                     @ApiParam(name = "executeEndTime", value = "日历结束时间（格式：yyyy-MM-dd HH:mm:ss）", required = true)
                                     @RequestParam(value = "executeEndTime", required = false)String executeEndTime,
                                     @ApiParam(name = "planId", value = "计划id", required = true)
                                     @RequestParam(value = "planId", required = true)String planId,
                                     @ApiParam(name = "searchTask", value = "快速查找任务：（1电话/短信关怀,2康复咨询,3健康监测,4上门服务,5康复指导,6康复随访,7康复复诊）", required = false)
                                     @RequestParam(value = "searchTask", required = false)Integer searchTask,
                                     @ApiParam(name = "doctorCode", value = "医生code（专科医生、家庭医生）", required = false)
                                     @RequestParam(value = "doctorCode", required = false)String doctorCode,
                                     @ApiParam(name = "status", value = "任务状态（0未完成，1已完成，2已预约）", required = false)
                                     @RequestParam(value = "status", required = false)Integer status,
                                     @ApiParam(name="taskExecutor",value = "任务执行人：0全部；1我的任务：2他人任务",required = false)
                                     @RequestParam(value = "taskExecutor",required = false,defaultValue = "0")String taskExecutor){
        try {
            if(!StringUtils.isNotEmpty(doctorCode)){
                doctorCode = getRepUID();
            }
            return rehabilitationManageService.calendarPlanDetailList(planId,searchTask,status,doctorCode,executeStartTime,executeEndTime,taskExecutor);
        } catch (ServiceException se) {
            return ObjEnvelop.getError(se.getMessage(),-1);
        } catch (Exception e) {
            return ObjEnvelop.getError("获取失败",-1);
        }
    }

    @RequestMapping(value = "calendarPlanDetailListBypatient", method = RequestMethod.GET)
    @ApiOperation("康复管理--某患者的康复计划按列表展示")
    public ObjEnvelop calendarPlanDetailListBypatient(@ApiParam(name = "executeStartTime", value = "日历开始时间（格式：yyyy-MM-dd HH:mm:ss）", required = false)
                                         @RequestParam(value = "executeStartTime", required = false)String executeStartTime,
                                         @ApiParam(name = "executeEndTime", value = "日历结束时间（格式：yyyy-MM-dd HH:mm:ss）", required = false)
                                         @RequestParam(value = "executeEndTime", required = false)String executeEndTime,
                                         @ApiParam(name = "patient", value = "患者code", required = true)
                                         @RequestParam(value = "patient", required = true)String patient,
                                         @ApiParam(name = "searchTask", value = "快速查找任务：（1电话/短信关怀,2康复咨询,3健康监测,4上门服务,5康复指导,6康复随访,7康复复诊）", required = false)
                                         @RequestParam(value = "searchTask", required = false)Integer searchTask,
                                         @ApiParam(name = "doctorCode", value = "医生code（专科医生、家庭医生）", required = false)
                                         @RequestParam(value = "doctorCode", required = false)String doctorCode,
                                         @ApiParam(name = "status", value = "任务状态（0未完成，1已完成，2已预约）", required = false)
                                         @RequestParam(value = "status", required = false)Integer status,
                                         @ApiParam(name="taskExecutor",value = "任务执行人：0全部；1我的任务：2他人任务",required = false)
                                         @RequestParam(value = "taskExecutor",required = false,defaultValue = "0")String taskExecutor){
        try {
            if(!StringUtils.isNotEmpty(doctorCode)){
                doctorCode = getRepUID();
            }
            List<Map<String,Object>> planList = wlyyRehabilitationManageService.getPatientAllRehabilitationPlan(patient,doctorCode);
            StringBuilder planIds = new StringBuilder();
            planIds.append(planList.get(0).get("id").toString());
            for (int i =1;i<planList.size();i++){
                planIds.append(","+planList.get(i).get("id").toString());
            }
            return rehabilitationManageService.calendarPlanDetailList(planIds.toString(),searchTask,status,doctorCode,executeStartTime,executeEndTime,taskExecutor);
        } catch (ServiceException se) {
            return ObjEnvelop.getError(se.getMessage(),-1);
        } catch (Exception e) {
            return ObjEnvelop.getError("获取失败",-1);
        }
    }



    @RequestMapping(value = "calendarPlanDetailItems", method = RequestMethod.GET)
    @ApiOperation("康复管理-康复计划服务项目按列表展示")
    public String calendarPlanDetailItems(@ApiParam(name = "executeStartTime", value = "日历开始时间（格式：yyyy-MM-dd HH:mm:ss）", required = false)
                                          @RequestParam(value = "executeStartTime", required = false)String executeStartTime,
                                          @ApiParam(name = "executeEndTime", value = "日历结束时间（格式：yyyy-MM-dd HH:mm:ss）", required = false)
                                          @RequestParam(value = "executeEndTime", required = false)String executeEndTime,
                                          @ApiParam(name = "planId", value = "计划id", required = true)
                                          @RequestParam(value = "planId", required = true)String planId,
                                          @ApiParam(name = "searchTask", value = "快速查找任务：（1、我的任务，2、健康教育，3、健康指导，4、随访，5、复诊）", required = false)
                                          @RequestParam(value = "searchTask", required = false)Integer searchTask,
                                          @ApiParam(name = "doctorCode", value = "医生code（专科医生、家庭医生）", required = false)
                                          @RequestParam(value = "doctorCode", required = false)String doctorCode,
                                          @ApiParam(name = "status", value = "任务状态（0未完成，1已完成，2已预约）", required = false)
                                          @RequestParam(value = "status", required = false)Integer status){
        try {
            if(!StringUtils.isNotEmpty(doctorCode)){
                doctorCode = getRepUID();
            }
            List<Map<String,Object>> result = wlyyRehabilitationManageService.calendarPlanDetailItems(executeStartTime,executeEndTime,planId,searchTask,status,doctorCode);
            return write(200, "获取成功", "data", result);
        } catch (Exception e) {
            return errorResult(e);
        }
    }


    @RequestMapping(value = "selectByPlanDetaiId", method = RequestMethod.GET)
    @ApiOperation("康复管理-获取操作记录-不限定频次")
    public String selectByPlanDetaiId(
            @ApiParam(name = "patient", value = "居民code", required = false)
            @RequestParam(value = "patient", required = false) String patient,
            @ApiParam(name = "planId", value = "计划id", required = true)
            @RequestParam(value = "planId", required = true) String planId,
            @ApiParam(name = "code", value = "服务项目code", required = true)
            @RequestParam(value = "code", required = true) String code,
            @ApiParam(name = "page", value = "", required = false)
            @RequestParam(value = "page", required = false) Integer page,
            @ApiParam(name = "pagesize", value = "", required = false)
            @RequestParam(value = "pagesize", required = false) Integer pagesize) {
        try {
            return write(200, "获取成功", "data", wlyyRehabilitationManageService.selectByPlanDetaiId(patient,planId,code,page,pagesize));
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "selectPlanByPatient", method = RequestMethod.GET)
    @ApiOperation("康复管理-根据居民code查找康复计划")
    public String calendarPlanDetailItems(@ApiParam(name = "patient", value = "居民code", required = false)
                                              @RequestParam(value = "patient", required = false)String patient,
                                          @ApiParam(name = "status", value = "任务状态（0未完成，1已完成，2已预约）", required = false)
                                          @RequestParam(value = "status", required = false)Integer status,
                                          @ApiParam(name = "planId", value = "计划id", required = false)
                                              @RequestParam(value = "planId", required = false)String planId){
        try {
            List<Map<String,Object>> result = wlyyRehabilitationManageService.selectPlanByPatient(patient,status,planId,getUID());
            return write(200, "获取成功", "data", result);
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "selectAllPlaneByPatient",method = RequestMethod.GET)
    @ApiOperation("获取居民所有康复计划")
    public String getPatientAllRehabilitationPlan(@ApiParam(name = "patient",value = "居民Code", required = true)
                                                  @RequestParam(value = "patient",required = true)String patient){
        try {
            return write(200, "获取成功", "data", wlyyRehabilitationManageService.getPatientAllRehabilitationPlan(patient,getUID()));
        }catch (Exception e){
            return errorResult(e);
        }
    }

    @RequestMapping(value = "selectAllCompleteServiceItemsByPatient",method = RequestMethod.GET)
    @ApiOperation("康复计划主页动态")
    public String selectAllCompleteServiceItemsByPatient(@ApiParam(name = "patient",value = "居民Code", required = true)
                                                  @RequestParam(value = "patient",required = true)String patient){
        try {
            return write(200, "获取成功", "data", wlyyRehabilitationManageService.selectAllCompleteServiceItemsByPatient(patient));
        }catch (Exception e){
            return errorResult(e);
        }
    }

    @RequestMapping(value = "manageAllPlanByPatient",method = RequestMethod.POST)
    @ApiOperation("一键确认管理居民所有的康复计划")
    public String manageAllPlanByPatient(@ApiParam(name="patient",value = "患者code",required = true)
                                         @RequestParam(value = "patient",required = true)String patient){
        try {
            return write(200,"操作成功","data",wlyyRehabilitationManageService.manageAllPlanByPatient(patient,getRepUID()));
        } catch (ServiceException se) {
            return error(-1,se.getMessage());
        } catch (Exception e){
            return errorResult(e);
        }
    }

    @RequestMapping(value = "serviceItemList", method = RequestMethod.GET)
    @ApiOperation("康复管理-多个康复计划服务项目内容列表")
    public ObjEnvelop serviceItemList(@ApiParam(name = "planDetailIds", value = "康复计划多个服务项目id（多个‘，’分隔）", required = true)
                                                   @RequestParam(value = "planDetailIds", required = true)String planDetailIds,
                                  @ApiParam(name = "doctorCode", value = "医生code", required = false)
                                  @RequestParam(value = "doctorCode", required = false)String doctorCode){

        try {
            if(!StringUtils.isNotEmpty(doctorCode)){
                doctorCode = getRepUID();
            }
            return rehabilitationManageService.serviceItemList(planDetailIds,doctorCode);
        } catch (ServiceException se) {
            return ObjEnvelop.getError(se.getMessage(),-1);
        } catch (Exception e) {
            return ObjEnvelop.getError("获取失败",-1);
        }
    }

    @RequestMapping(value = "serviceItem", method = RequestMethod.GET)
    @ApiOperation("康复管理-康复计划服务项目确认详情页")
    public ObjEnvelop serviceItem(@ApiParam(name = "planDetailId", value = "康复计划服务项目id", required = true)
                              @RequestParam(value = "planDetailId", required = true)String planDetailId,
                              @ApiParam(name = "doctorCode", value = "医生code", required = false)
                              @RequestParam(value = "doctorCode", required = false)String doctorCode,
                              @ApiParam(name = "recordId", value = "记录id", required = false)
                              @RequestParam(value = "recordId", required = false)String recordId){

        try {
            if(!StringUtils.isNotEmpty(doctorCode)){
                doctorCode = getRepUID();
            }
            return rehabilitationManageService.serviceItem(planDetailId,doctorCode,recordId);
        } catch (ServiceException se) {
            return ObjEnvelop.getError(se.getMessage(),-1);
        } catch (Exception e) {
            return ObjEnvelop.getError("获取失败",-1);
        }
    }

    @RequestMapping(value = "saveGuidanceMessage", method = RequestMethod.POST)
    @ApiOperation("康复管理-保存指导留言")
    public String saveGuidanceMessage(@ApiParam(name = "messageId", value = "消息id", required = true)
                                      @RequestParam(value = "messageId", required = true)String messageId,
                                      @ApiParam(name = "doctorCode", value = "医生code", required = false)
                                      @RequestParam(value = "doctorCode", required = false)String doctorCode,
                                      @ApiParam(name = "content", value = "聊天内容", required = true)
                                      @RequestParam(value = "content", required = true)String content,
                                      @ApiParam(name = "planDetailId", value = "服务项目id", required = true)
                                      @RequestParam(value = "planDetailId", required = true)String planDetailId,
                                      @ApiParam(name = "contentType", value = "内容类型(1,6,8 - 文本， 2，9- 图片， 3 - 语音， 4-文章， 5,7系统消息。12-语音 18-居民名片， 19-聊天记录)", required = true)
                                      @RequestParam(value = "contentType", required = true)Integer contentType){

        try {
            if(!StringUtils.isNotEmpty(doctorCode)){
                doctorCode = getRepUID();
            }
            BaseDoctorDO doctor = doctorDao.findByIdAndDel(doctorCode);
            Integer doctorType = doctor.getLevel();
            rehabilitationManageService.saveGuidanceMessage(messageId,doctorCode,doctorType,content,planDetailId,contentType);
            return write(200, "保存成功！");
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "updateStatusRehabilitationOperate", method = RequestMethod.POST)
    @ApiOperation("康复管理-更新康复计划操作完成日志状态")
    public String updateStatusRehabilitationOperate(@ApiParam(name = "planDetailId", value = "服务项目id", required = true)
                                                    @RequestParam(value = "planDetailId", required = true)String planDetailId,
                                                    @ApiParam(name = "status", value = "状态", required = true)
                                                    @RequestParam(value = "status", required = true)Integer status){

        try {
            rehabilitationManageService.updateStatusRehabilitationOperate(status,planDetailId);
            return write(200, "更新成功！");
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "planDetailAfterCall", method = RequestMethod.POST)
    @ApiOperation("康复管理-电话关怀拨打后生成记录对应怪坏记录")
    public String planDetailAfterCall(@ApiParam(name = "planDetailId", value = "服务项目id", required = true)
                                      @RequestParam(value = "planDetailId", required = true)String planDetailId,
                                      @ApiParam(name = "doctorCode", value = "执行任务的doctorCode", required = true)
                                      @RequestParam(value = "doctorCode", required = false)String doctorCode){

        try {
            if (StringUtils.isBlank(doctorCode)){
                doctorCode = getRepUID();
            }

            RehabilitationOperateRecordsDO recordsDO = rehabilitationManageService.planDetailAfterCall(planDetailId,doctorCode);
            return write(200, "电话关怀记录生成成功！", "data", recordsDO);
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "patientRehabilitationDetail", method = RequestMethod.GET)
    @ApiOperation("康复管理-居民详情页")
    public ObjEnvelop patientRehabilitationDetail(@ApiParam(name = "patientCode", value = "居民code", required = true)
                                              @RequestParam(value = "patientCode", required = true)String patientCode){

        try {
            return rehabilitationManageService.patientRehabilitationDetail(patientCode);
        } catch (ServiceException se) {
            return ObjEnvelop.getError(se.getMessage(),-1);
        } catch (Exception e) {
            return ObjEnvelop.getError("获取失败",-1);
        }
    }

    @RequestMapping(value = "recentPlanDetailRecord", method = RequestMethod.GET)
    @ApiOperation("康复管理-近期康复相关记录")
    public ObjEnvelop recentPlanDetailRecord(@ApiParam(name = "patientCode", value = "居民code", required = true)
                                         @RequestParam(value = "patientCode", required = true)String patientCode,
                                         @ApiParam(name = "startTime", value = "开始时间（格式：yyyy-MM-dd HH:mm:ss）", required = false)
                                         @RequestParam(value = "startTime", required = false)String startTime,
                                         @ApiParam(name = "endTime", value = "结束时间（格式：yyyy-MM-dd HH:mm:ss）", required = false)
                                         @RequestParam(value = "endTime", required = false)String endTime,
                                         @ApiParam(name = "page", value = "第几页，从1开始", required = true)
                                         @RequestParam(value = "page", required = false,defaultValue = "1")Integer page,
                                         @ApiParam(name = "pageSize", value = "每页分页大小", required = true)
                                        @RequestParam(value = "pageSize", required = false,defaultValue = "10")Integer pageSize){
        try {
            return rehabilitationManageService.recentPlanDetailRecord(patientCode,startTime,endTime,page,pageSize);
        } catch (ServiceException se) {
            return ObjEnvelop.getError(se.getMessage(),-1);
        } catch (Exception e) {
            return ObjEnvelop.getError("获取失败",-1);
        }
    }

    @RequestMapping(value = "createServiceQrCode", method = RequestMethod.POST)
    @ApiOperation("康复管理-根据康复计划明细id和医生code生成服务码")
    public String createServiceQrCode(@ApiParam(name = "planDetailId", value = "康复计划项目明细ID")@RequestParam(value = "planDetailId", required = true)String planDetailId,
                                      @ApiParam(name = "doctorCode", value = "医生code")@RequestParam(value = "doctorCode", required = true)String doctorCode){

        try {
            return write(200, "获取二维码成功！", "data", wlyyRehabilitationManageService.createServiceQrCode(planDetailId,doctorCode));
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            return errorResult(e);
        }
    }
    
    @RequestMapping(value = "createSpecialDoctorQrCode", method = RequestMethod.POST)
    @ApiOperation("康复管理-生成康复下转专科医生二维码")
    public String createSpecialDoctorQrCode(
            @ApiParam(name = "doctorCode", value = "医生code")@RequestParam(value = "doctorCode", required = true)String doctorCode,
            @ApiParam(name = "teamCode", value = "团队CODE")@RequestParam(value = "teamCode", required = true)String teamCode,
            @ApiParam(name = "teamName", value = "团队名称")@RequestParam(value = "teamName", required = true)String teamName
                                             ){
        
        try {
            return write(200, "获取二维码成功！", "data", wlyyRehabilitationManageService.createSpecialDoctorQrCode(doctorCode,teamCode,teamName));
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "checkAfterQrCode", method = RequestMethod.POST)
    @ApiOperation("康复管理-居民扫码后验证是否是关联的居民扫码")
    public String checkAfterQrCode(@ApiParam(name = "planDetailId", value = "康复计划项目明细ID")@RequestParam(value = "planDetailId", required = true)String planDetailId,
                                      @ApiParam(name = "patientCode", value = "居民端登录的居民code")@RequestParam(value = "patientCode", required = true)String patientCode){

        try {
            Map<String,Object> result = rehabilitationPlanService.checkAfterQrCode(planDetailId,patientCode);
            return write(200, "获取成功", "data", result);
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "serviceDoctorList", method = RequestMethod.GET)
    @ApiOperation("康复管理-医生端居民详情服务医生列表")
    public ObjEnvelop serviceDoctorList(@ApiParam(name = "patientCode", value = "居民code", required = true)
                                    @RequestParam(value = "patientCode", required = true)String patientCode){

        try {
            return rehabilitationManageService.serviceDoctorList(patientCode);
        } catch (ServiceException se) {
            return ObjEnvelop.getError(se.getMessage(),-1);
        } catch (Exception e) {
            return ObjEnvelop.getError("获取失败",-1);
        }
    }

    @RequestMapping(value = "appCalendarPlanDetailList", method = RequestMethod.GET)
    @ApiOperation("康复管理-app端、微信端计划的服务项目列表")
    public ObjEnvelop appCalendarPlanDetailList(@ApiParam(name = "executeStartTime", value = "日历开始时间（格式：yyyy-MM-dd HH:mm:ss）", required = true)
                                            @RequestParam(value = "executeStartTime", required = false)String executeStartTime,
                                            @ApiParam(name = "executeEndTime", value = "日历结束时间（格式：yyyy-MM-dd HH:mm:ss）", required = true)
                                            @RequestParam(value = "executeEndTime", required = false)String executeEndTime,
                                            @ApiParam(name = "planId", value = "计划id", required = true)
                                            @RequestParam(value = "planId", required = true)String planId,
                                            @ApiParam(name = "searchTask", value = "快速查找任务：（1、我的任务，2、随访，3、复诊，4、健康教育）", required = false)
                                            @RequestParam(value = "searchTask", required = false)Integer searchTask,
                                            @ApiParam(name = "status", value = "任务状态（0未完成，1已完成，2已预约）", required = false)
                                            @RequestParam(value = "status", required = false)Integer status){
        try {
            return rehabilitationManageService.appCalendarPlanDetailList(planId,searchTask,status,executeStartTime,executeEndTime);
        } catch (ServiceException se) {
            return ObjEnvelop.getError(se.getMessage(),-1);
        } catch (Exception e) {
            return ObjEnvelop.getError("获取失败",-1);
        }
    }

    @RequestMapping(value = "saveRehabilitationOperateRecord", method = RequestMethod.POST)
    @ApiOperation("康复管理-新增operateRecord")
    public String saveRehabilitationOperateRecord(@ApiParam(name = "dataJson", value = "实体json",defaultValue = "{\"rehabilitationDetailId\":\"402803f6657f195301657f4fbd3c0001\",\"patientCode\":\"00\",\"patientName\":\"11\",\"doctorCode\":\"22\",\"doctorName\":\"33\",\"node\":\"jjjjjj\",\"relationRecordType\":\"4\",\"relation_record_code\":\"55\",\"relationRecordImg\":\"666666\",\"status\":\"0\"}")@RequestParam(value = "dataJson", required = true)String dataJson){
        try {
            return write(200, "新增康复计划日志成功！", "data", rehabilitationPlanService.saveRehabilitationOperateRecord(dataJson));
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "updateNoteAndImageRehabilitationOperate", method = RequestMethod.POST)
    @ApiOperation("康复管理-康复计划完成时更新服务完成笔记和图片接口并且确认完成")
    public String updateNoteAndImageRehabilitationOperate(@ApiParam(name = "planDetailId", value = "康复计划项目明细ID")@RequestParam(value = "planDetailId", required = true)String planDetailId,
                                                          @ApiParam(name = "node", value = "服务完成笔记", required = false)@RequestParam(value = "node", required = false)String node,
                                                          @ApiParam(name = "image", value = "相关记录图片，json格式", required = false)@RequestParam(value = "image", required = false)String image,
                                                          @ApiParam(name = "relationCode", value = "业务关联code", required = false)@RequestParam(value = "relationCode", required = false)String relationCode,
                                                          @ApiParam(name = "status", value = "是否确认完成 0未确认 1确认", required = false)@RequestParam(value = "status", required = false)String status){

        try {
            if (StringUtils.isNotBlank(image)){
                try{
                    com.alibaba.fastjson.JSONArray jsonImg = com.alibaba.fastjson.JSONArray.parseArray(image);//json格式
                    image = jsonImg.toJSONString();
                }catch (Exception e){
                    com.alibaba.fastjson.JSONArray jsonImg = new com.alibaba.fastjson.JSONArray();
                    String[] images = image.split(",");
                    for (String tmp:images){
                        jsonImg.add(tmp);
                    }
                    image = jsonImg.toJSONString();
                }
            }
            rehabilitationManageService.updateNodeAndRelationRecordImg(node, image,status,planDetailId, relationCode);
            return write(200, "更新日志成功！");
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "updateRelationCodeByDetailId", method = RequestMethod.POST)
    @ApiOperation("康复管理-服务项目关联关系")
    public String updateRelationCodeByDetailId(@ApiParam(name = "planDetailId", value = "康复计划项目明细ID")@RequestParam(value = "planDetailId", required = true)String planDetailId,
                                                          @ApiParam(name = "relationCode", value = "业务关联code", required = false)@RequestParam(value = "relationCode", required = false)String relationCode){

        try {
            Integer result = wlyyRehabilitationManageService.updateRelationCodeByDetailId(planDetailId,relationCode);
            if (result==1){
                return write(200, "更新日志成功！");
            }else {
                return error(-1, "更新日志失败");
            }
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "selectPlanServerDoctor", method = RequestMethod.POST)
    @ApiOperation("康复管理-康复计划的服务医生")
    public String selectPlanServerDoctor(@ApiParam(name = "planId", value = "康复计划ID")@RequestParam(value = "planId", required = true)String planId){
        try {
            String doctorCode = getRepUID();
            return write(200, "查询成功！","data",wlyyRehabilitationManageService.selectPlanServerDoctor(planId,doctorCode));
        } catch (ServiceException se) {
            return error(-1,se.getMessage());
        } catch (Exception e) {
            return errorResult(e);
        }
    }


    @RequestMapping(value = "updatePlanDetailStatusById", method = RequestMethod.POST)
    @ApiOperation("康复管理-更新康复计划服务项目状态")
    public String updatePlanDetailStatusById(@ApiParam(name = "planDetailId", value = "服务项目id", required = true)
                                             @RequestParam(value = "planDetailId", required = true)String planDetailId,
                                             @ApiParam(name = "status", value = "状态（0未完成，1已完成，2已预约）", required = true)
                                             @RequestParam(value = "status", required = true)Integer status){
        try {
            rehabilitationManageService.updatePlanDetailStatusById(status,planDetailId);
            return write(200, "更新成功！");
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "updatePlanDetailExecuteTimeById", method = RequestMethod.POST)
    @ApiOperation("康复管理-修改康复计划服务项目执行时间")
    public String updatePlanDetailExecuteTimeById(@ApiParam(name = "planDetailId", value = "服务项目id", required = true)
                                             @RequestParam(value = "planDetailId", required = true)String planDetailId,
                                             @ApiParam(name = "executeTime", required = true)
                                             @RequestParam(value = "executeTime", required = true)String executeTime){

        try {
            rehabilitationManageService.updatePlanDetailExecuteTimeById(DateUtil.strToDateLong(executeTime),planDetailId);
            return write(200, "更新成功！");
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "updatePlanStatusById", method = RequestMethod.POST)
    @ApiOperation("康复管理-更新计划的状态")
    public String updatePlanStatusById(@ApiParam(name = "planId", value = "服务项目id", required = true)
                                             @RequestParam(value = "planId", required = true)String planId,
                                             @ApiParam(name = "status", value = "状态（0已中止，1进行中，2已完成）", required = true)
                                             @RequestParam(value = "status", required = true)Integer status,
                                             @ApiParam(name = "abortReason", value = "中止原因")
                                             @RequestParam(value = "abortReason",defaultValue = "",required = false)String abortReason){

        try {
            wlyyRehabilitationPlanService.updatePlanStatusById(planId,status,abortReason);
            return write(200, "更新成功！");
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "planSchedule", method = RequestMethod.GET)
    @ApiOperation("康复管理-计划总进度")
    public ObjEnvelop planSchedule(@ApiParam(name = "planId", value = "计划id", required = true)
                               @RequestParam(value = "planId", required = true)String planId,
                               @ApiParam(name = "patientCode", value = "居民code", required = true)
                               @RequestParam(value = "patientCode", required = true)String patientCode){

        try {
            return rehabilitationManageService.planSchedule(planId,patientCode,1);
        } catch (ServiceException se) {
            return ObjEnvelop.getError(se.getMessage(),-1);
        } catch (Exception e) {
            return ObjEnvelop.getError("获取失败",-1);
        }
    }

    @RequestMapping(value = "selectByMedicalCode", method = RequestMethod.GET)
    @ApiOperation("获取医院病历")
    public String selectByMedicalCode(@ApiParam(name = "medical", value = "住院code", required = true)
                               @RequestParam(value = "medical", required = true)String medical){

        try {
            List<PatientMedicalRecordsDO> result = wlyyRehabilitationManageService.selectByMedicalCode(medical);
            try {
                String sql = "UPDATE `wlyy`.`wlyy_patient_medical_records_rehabilitation` SET `doctor_read_status`='1' WHERE (`code`='"+medical+"') ";
                jdbcTemplate.update(sql);
            }catch (Exception e){
                e.printStackTrace();
            }
            return write(200, "获取成功！", "data", result);
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "patientCount", method = RequestMethod.GET)
    @ApiOperation("医生已计划数，已完成计划数（居民数）")
    public ObjEnvelop patientCount(@ApiParam(name = "doctorCode", value = "医生code", required = false)
                                                   @RequestParam(value = "doctorCode", required = false)String doctorCode){

        try {
            if(!StringUtils.isNotEmpty(doctorCode)){
                doctorCode = getRepUID();
            }
            return rehabilitationManageService.patientCount(doctorCode);
        } catch (Exception e) {
            return ObjEnvelop.getError("获取失败",-1);
        }
    }

    @RequestMapping(value = "sendToFamilyDoctor", method = RequestMethod.POST)
    @ApiOperation(value = "给家医发送居民的康复计划.")
    public String sendBusinessCard(@ApiParam(name = "doctorCode", value = "专医code")
                                   @RequestParam(value = "doctorCode", required = true) String doctorCode,
                                   @ApiParam(name = "doctorName", value = "专医名字")
                                   @RequestParam(value = "doctorName", required = true) String doctorName,
                                   @ApiParam(name = "sessionId", value = "会话id", defaultValue = "")
                                   @RequestParam(value = "sessionId", required = true) String sessionId,
                                   @ApiParam(name = "businessType", value = "businessType", defaultValue = "1")
                                   @RequestParam(value = "businessType", required = true) String businessType,
                                   @ApiParam(name = "contentJsonStr", value = "contentJsonStr")
                                   @RequestParam(value = "contentJsonStr", required = true) String contentJsonStr){
        try {
            imUtill.sendImMsg(doctorCode,doctorName,sessionId,ImUtil.ContentType.Rehabilitation.getValue(),contentJsonStr,businessType,null);
            return write(200,"发送消息成功！");
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e){
            return errorResult(e);
        }
    }

    @RequestMapping(value = "getServiceItemsAfterStop", method = RequestMethod.GET)
    @ApiOperation("康复管理-已执行项目清单")
    public ObjEnvelop getServiceItemsAfterStop(@ApiParam(name = "planId", value = "康复计划id", required = true)
                                @RequestParam(value = "planId", required = true)String planId){
        try {
            return rehabilitationManageService.getServiceItemsAfterStop(planId);
        } catch (ServiceException se) {
            return ObjEnvelop.getError(se.getMessage(),-1);
        } catch (Exception e) {
            return ObjEnvelop.getError("获取失败",-1);
        }
    }


    /**
     * 康复管理-频次列表
     *
     * @return
     */
    @RequestMapping(value = "selectFrequencys", method = RequestMethod.GET)
    @ApiOperation("康复管理-频次列表")
    public String selectFrequencys(){
        try {
            return write(200, "操作成功！","data",wlyyRehabilitationManageService.selectFrequences());
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "getServiceItemList", method = RequestMethod.GET)
    @ApiOperation("康复管理-获取服务项目列表")
    public String getServiceItemList(@ApiParam(name = "name", value = "服务项目名称", required = false)
                                     @RequestParam(value = "name", required = false) String name) {
        try {
            return write(200, "请求成功！", "data", wlyyRehabilitationManageService.getServiceItemList(name));
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "selectGuidances", method = RequestMethod.GET)
    @ApiOperation("康复管理-查看指导列表")
    public String selectType(@ApiParam(name = "type", value = "康复指导类型", required = false)
                                 @RequestParam(value = "type", required = false) Integer type,
                             @ApiParam(name = "id", value = "康复指导模板id", required = false)
                            @RequestParam(value = "id", required = false) String id,
                             @ApiParam(name = "title", value = "康复指导名称", required = false)
                                 @RequestParam(value = "title", required = false) String title,
                             @ApiParam(name = "planId", value = "计划id", required = false)
                                 @RequestParam(value = "planId", required = false) String planId,
                             @ApiParam(name = "page", value = "页码", required = false)
                                 @RequestParam(value = "page", required = false, defaultValue ="1") Integer page,
                             @ApiParam(name = "pagesize", value = "每页大小", required = false)
                                 @RequestParam(value = "pagesize", required = false,defaultValue = "15") Integer pagesize) {
        try {
            return write(200, "请求成功！", "data", rehabilitationGuidanceService.select(title,type,id,planId,page,pagesize));
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "selectPatientGuidances", method = RequestMethod.GET)
    @ApiOperation("康复管理-查看指导列表")
    public String selectPatientGuidances(@ApiParam(name = "patient", value = "居民code", required = false)
                             @RequestParam(value = "patient", required = false) String patient,
                             @ApiParam(name = "code", value = "康复指导模板code", required = false)
                             @RequestParam(value = "code", required = false) String code,
                             @ApiParam(name = "doctor", value = "医生code", required = false)
                             @RequestParam(value = "doctor", required = false) String doctor,
                             @ApiParam(name = "page", value = "页码", required = false)
                             @RequestParam(value = "page", required = false) Integer page,
                             @ApiParam(name = "pagesize", value = "每页大小", required = false)
                             @RequestParam(value = "pagesize", required = false) Integer pagesize) {
        try {
            return write(200, "请求成功！", "data", rehabilitationGuidanceService.selectPatientGuidances(patient,doctor,code,page,pagesize));
        } catch (Exception e) {
            return errorResult(e);
        }
    }



    @RequestMapping(value = "selectType", method = RequestMethod.GET)
    @ApiOperation("康复管理-查看指导列表标题")
    public String selectType(@ApiParam(name="planId",value = "根据计划id取该计划对应疾病的指导",required = false)
                             @RequestParam(value = "planId",required = false)String planId) {
        try {
            return write(200, "请求成功！", "data", rehabilitationGuidanceService.selectType(planId));
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "selectByRelationCode", method = RequestMethod.GET)
    @ApiOperation("康复管理-根据业务code查找康复指导")
    public String selectByRelationCode(@ApiParam(name = "relationCode", value = "业务关联code", required = false)
                             @RequestParam(value = "relationCode", required = false) String relationCode,
                             @ApiParam(name = "patient", value = "居民code", required = false)
                            @RequestParam(value = "patient", required = false) String patient,
                             @ApiParam(name = "doctor", value = "医生code", required = false)
                                 @RequestParam(value = "doctor", required = false) String doctor) {

        try {
            return write(200, "请求成功！", "data", rehabilitationGuidanceService.selectByRelationCode(patient,doctor,relationCode));
        } catch (Exception e) {
            return errorResult(e);
        }
    }


    @RequestMapping(value = "sendGuidance", method = RequestMethod.GET)
    @ApiOperation("康复管理-发送康复指导")
    public String selectType(@ApiParam(name = "json", value = "康复指导json", required = true)
                             @RequestParam(value = "json", required = true) String json,
                             @ApiParam(name = "planId", value = "计划ID", required = true)
                             @RequestParam(value = "planId", required = true) String planId) {

        try {
            com.alibaba.fastjson.JSONArray array = com.alibaba.fastjson.JSONArray.parseArray(json);
            List<RehabilitationguidancePatientDO> rehabilitationguidancePatientDOS = new ArrayList<>();
            for (int i=0;i<array.size();i++){
                com.alibaba.fastjson.JSONObject object = array.getJSONObject(i);
                RehabilitationguidancePatientDO rehabilitationguidancePatientDO = objectMapper.readValue(object.toJSONString(),RehabilitationguidancePatientDO.class);
                rehabilitationguidancePatientDOS.add(rehabilitationguidancePatientDO);
            }

            return write(200, "请求成功！", "data", rehabilitationGuidanceService.sendGuidanceToPatient(rehabilitationguidancePatientDOS,planId));
        } catch (Exception e) {
            return errorResult(e);
        }
    }


    @RequestMapping(value = "confirmManage", method = RequestMethod.GET)
    @ApiOperation("康复管理-确认管理")
    public String confirmManage(@ApiParam(name = "doctor", value = "签约医生code", required = true)
                             @RequestParam(value = "doctor", required = true) String signDoctor,
                             @ApiParam(name = "patient", value = "居民code", required = true)
                             @RequestParam(value = "patient", required = true) String patient,
                             @ApiParam(name = "specialist", value = "专医code", required = true)
                             @RequestParam(value = "specialist", required = true) String specialist,
                             @ApiParam(name = "relationId", value = "签约关系id", required = true)
                             @RequestParam(value = "relationId", required = false) String relationId,
                             @ApiParam(name = "planId", value = "计划id", required = true)
                             @RequestParam(value = "planId", required = false) String planId) {

        try {
            wlyyRehabilitationManageService.confirmManage(patient,signDoctor,specialist,relationId,planId);
            return write(200, "请求成功！");
        } catch (ServiceException se) {
            return error(-1,se.getMessage());
        } catch (Exception e) {
            return errorResult(e);
        }
    }


    @RequestMapping(value = "selectPlanDetailByPlanId", method = RequestMethod.GET)
    @ApiOperation("康复管理-查看其它服务计划")
    public String selectPlanDetailByPlanId(@ApiParam(name = "planId", value = "计划id", required = true)
                                @RequestParam(value = "planId", required = true) String planId,
                                @ApiParam(name = "planDetailId", value = "计划详情id", required = true)
                                    @RequestParam(value = "planDetailId", required = true) String planDetailId,
                                @ApiParam(name = "itemId", value = "服务项目code", required = true)
                                    @RequestParam(value = "itemId", required = true) String itemId) {

        try {
            return write(200, "请求成功！","data",wlyyRehabilitationManageService.selectPlanDetailByPlan(planId,planDetailId,itemId));
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "selectReservationRecord", method = RequestMethod.GET)
    @ApiOperation("康复管理-查看复诊记录")
    public String selectReservationRecord(@ApiParam(name = "patient", value = "居民code", required = true)
                                           @RequestParam(value = "patient", required = true) String patient,
                                          @ApiParam(name = "doctor", value = "医生code", required = false)
                                          @RequestParam(value = "doctor", required = false) String doctor,
                                          @ApiParam(name = "page", value = "页码", required = false)
                                              @RequestParam(value = "page", required = false) Integer page,
                                          @ApiParam(name = "pageSize", value = "条数", required = false)
                                              @RequestParam(value = "pageSize", required = false) Integer pageSize) {

        try {
            if (doctor==null||doctor==""){
                   doctor=getUID();
            }
            return write(200, "请求成功！","data",wlyyRehabilitationManageService.selectReservationRecord(patient,doctor,page,pageSize));
        } catch (Exception e) {
            return errorResult(e);
        }
    }


    /**
     *
     * @param patient 居民code
     * @param doctor 专科医生code
     * @param type 1康复指导2康复下转3复诊提醒4上门服务提醒5家签提醒
     * @param relationCode 业务关联code
     * @param planId 计划id
     * @return
     */
    @RequestMapping(value = "sendSpecialistWeixinMessage", method = RequestMethod.GET)
    @ApiOperation("康复管理-发送微信模板消息")
    public String sendSpecialistWeixinMessage(@ApiParam(name = "patient", value = "居民code", required = true)
                                           @RequestParam(value = "patient", required = true) String patient,
                                           @ApiParam(name = "doctor", value = "医生code", required = true)
                                           @RequestParam(value = "doctor", required = true) String doctor,
                                           @ApiParam(name = "type", value = "发送类型", required = true)
                                           @RequestParam(value = "type", required = true) Integer type,
                                           @ApiParam(name = "relationCode", value = "业务code", required = false)
                                           @RequestParam(value = "relationCode", required = true) String relationCode,
                                           @ApiParam(name = "planId", value = "计划id", required = false)
                                           @RequestParam(value = "planId", required = true) String planId,
                                           @ApiParam(name = "reservationType", value = "复诊类型:1线上，2线下，3远程", required = false)
                                           @RequestParam(value = "reservationType", required = false) String reservationType) {
        try {
            rehabilitationGuidanceService.sendSpecialistWeixinMessage(patient,doctor,type,relationCode,planId,reservationType);
            return write(200, "请求成功！");
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @GetMapping(value = "appointmentConsultation")
    @ApiOperation("预约协诊")
    public String appointmentConsultation(@ApiParam(name = "patient", value = "居民code", required = true)
                                                @RequestParam(value = "patient", required = true) String patient,
                                            @ApiParam(name = "doctor", value = "医生code-家医", required = true)
                                                @RequestParam(value = "doctor", required = true) String doctor,
                                            @ApiParam(name = "doctorName", value = "医生姓名-家医", required = true)
                                                @RequestParam(value = "doctorName", required = true) String doctorName,
                                          @ApiParam(name = "appointmentDoctor", value = "医生code-专科", required = true)
                                              @RequestParam(value = "appointmentDoctor", required = true) String appointmentDoctor,
                                          @ApiParam(name = "appointmentDoctorName", value = "医生姓名-专科", required = true)
                                              @RequestParam(value = "appointmentDoctorName", required = true) String appointmentDoctorName,
                                            @ApiParam(name = "sendTime", value = "发送时间", required = true)
                                                @RequestParam(value = "sendTime", required = true) String sendTime,
                                            @ApiParam(name = "planDetailId", value = "计划明细id", required = true)
                                                @RequestParam(value = "planDetailId", required = true) String planDetailId){
        try {
            rehabilitationManageService.appointmentConsultation(patient,doctor,doctorName,appointmentDoctor,appointmentDoctorName,sendTime,planDetailId,getUID());
            return write(200, "请求成功！","data","");
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    /**
     * 获取为某患者创建康复计划的专科医生列表,家庭医生，
     * @param patientCode 患者code
     * @return
     */
    @RequestMapping(value = "getPatientSpecialAndFamilyDoctor",method = RequestMethod.GET)
    @ApiOperation("获取为某患者创建康复计划的医生专科列表、家庭医生")
    public String getConsultDoctorForConsulting(@RequestParam("patientCode")String patientCode){
        try {
            return write(200,"查询成功","data",wlyyRehabilitationManageService.getPatientSpecialAndFamilyDoctor(patientCode));
        }catch (Exception e){
            return errorResult(e);
        }
    }

    @RequestMapping(value = "getPlanDetailForModifing",method = RequestMethod.GET)
    @ApiOperation(value = "根据planId、模板id获取计划列表")
    public String getPlanDetailForModifing(@ApiParam(name = "planId",value = "康复计划id",required = true)
                                           @RequestParam(value = "planId",required = true) String planId,
                                           @ApiParam(name = "templateId",value = "模板Id,不传默认为系统模板",required = false)
                                           @RequestParam(value = "templateId",required = false) String templateId){
        try {
            return write(200,"查询成功","data",wlyyRehabilitationManageService.getPlanDetailForModifing(planId, templateId));
        }catch (Exception e){
            return errorResult(e);
        }
    }

}
