package com.yihu.jw.afterDiagnosis.endpoint.rehabilitation;


import com.yihu.jw.afterDiagnosis.endpoint.BaseController;
import com.yihu.jw.afterDiagnosis.service.rehabilitation.RehabilitationManageService;
import com.yihu.jw.afterDiagnosis.service.rehabilitation.WlyyRehabilitationPlanService;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.util.entity.ServiceException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashSet;
import java.util.Set;

/**
 * Created by humingfen on 2018/8/22.
 */
@RestController
@RequestMapping(value = "/doctor/specialist/rehabilitationPlan")
@Api(description = "医生端-康复服务套餐管理")
public class DoctorRehabilitationPlanController extends BaseController {

    @Autowired
    private WlyyRehabilitationPlanService rehabilitationPlanService;
    @Autowired
    private RehabilitationManageService rehabilitationManageService;
    @Autowired
    private BaseDoctorDao doctorDao;

    @RequestMapping(value = "/findTemplateList", method = RequestMethod.GET)
    @ApiOperation(value = "获取康复服务套餐模板列表")
    public String templateList(@ApiParam(name = "doctor", value = "医生标识")
                               @RequestParam(required = false)String doctor,
                               @ApiParam(name = "patient", value = "居民标识")
                               @RequestParam(required = false)String patient,
                               @ApiParam(name = "adminTeamCode", value = "居民签约的专科行政团队id")
                                   @RequestParam(value = "adminTeamCode", required = false)Long adminTeamCode){
        try {
            if(!StringUtils.isNotBlank(doctor)){
                doctor = getUID();
            }
            return write(200, "获取成功", "data", rehabilitationPlanService.findTemplateInfo(doctor, patient, adminTeamCode));
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "/findTemplateDetail", method = RequestMethod.GET)
    @ApiOperation(value = "获取康复服务套餐模板明细")
    public String findTemplateDetail(@ApiParam(name = "templateId", value = "模板id")
                                     @RequestParam(required = true)String templateId){
        try {
            return write(200, "获取成功", "data", rehabilitationPlanService.findTemplateDetailInfo(templateId));
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "/createTemplate", method = RequestMethod.POST)
    @ApiOperation(value = "创建康复服务套餐模板")
    public String createTemplate(@ApiParam(name = "doctor", value = "医生标识")
                                 @RequestParam(required = false)String doctor,
                                 @ApiParam(name = "adminTeamCode", value = "团队id")
                                 @RequestParam(required = false)Long adminTeamCode,
                                 @ApiParam(name = "title", value = "康复服务模板名称")
                                 @RequestParam(required = true) String title){
        try {
            if(!StringUtils.isNotBlank(doctor)){
                doctor = getUID();
            }
            BaseDoctorDO d = doctorDao.findByIdAndDel(doctor);
            return write(200, "获取成功", "data", rehabilitationPlanService.createTemplate(title,d,adminTeamCode));
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "/createTemplateDetail", method = RequestMethod.POST)
    @ApiOperation(value = "创建康复服务套餐模板明细")
    public String createTemplateDetail(@ApiParam(name = "doctor", value = "医生标识")
                                       @RequestParam(required = false)String doctor,
                                       @ApiParam(name = "json", value = "康复服务模板明细")
                                       @RequestParam(required = true) String json,
                                       @ApiParam(name = "type", value = "create或者edit")
                                       @RequestParam(required = true) String type){
        try {
            if(!StringUtils.isNotBlank(doctor)){
                doctor = getUID();
            }
            BaseDoctorDO d = doctorDao.findByIdAndDel(doctor);
            return write(200, "获取成功", "data", rehabilitationPlanService.createTemplateDetail(json, type, d));
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "/deleteTemplate", method = RequestMethod.POST)
    @ApiOperation(value = "删除康复服务套餐模板")
    public String deleteTemplate(@ApiParam(name = "templateId", value = "模板id")
                                 @RequestParam(required = true)String templateId){
        try {
            return write(200, "获取成功", "data", rehabilitationPlanService.deleteTemplate(templateId));
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "/createRehabilitationPlan", method = RequestMethod.POST)
    @ApiOperation(value = "创建居民康复计划")
    public String createRehabilitationPlan(@ApiParam(name = "doctor", value = "医生标识")
                                           @RequestParam(required = false)String doctor,
                                           @ApiParam(name = "json", value = "康复计划json")
                                           @RequestParam(required = true) String json){
        try {
            if(!StringUtils.isNotBlank(doctor)){
                doctor = getUID();
            }
            BaseDoctorDO d = doctorDao.findByIdAndDel(doctor);
            JSONObject object = new JSONObject(json);
            JSONArray array = object.getJSONArray("detail");
            Set<String> list = new HashSet<>();
            for(int i=0;i<array.length();i++){
                JSONObject jsonObject = array.getJSONObject(i);
                String hospitalServiceItemId = jsonObject.getString("hospitalServiceItemId");
                String frequencyCode = jsonObject.getString("frequencyCode");
                Long timeType = jsonObject.getLong("timeType");
                String executeTimes =jsonObject.getString("executeTimes");
                String str[] = executeTimes.split(",");
                for (int j=i+1;j<array.length();j++){
                    JSONObject jsonObject1 = array.getJSONObject(j);
                    String hospitalServiceItemId1 = jsonObject1.getString("hospitalServiceItemId");
                    String frequencyCode1 = jsonObject1.getString("frequencyCode");
                    Long timeType1 = jsonObject1.getLong("timeType");
                    String executeTimes1 =jsonObject1.getString("executeTimes");
                    if (hospitalServiceItemId.equalsIgnoreCase(hospitalServiceItemId1)&&frequencyCode.equalsIgnoreCase(frequencyCode1)&&timeType.equals(timeType1)){
                        for (int z=0;z<str.length;z++){
                            if (executeTimes1.contains(str[z])){
                                list.add(hospitalServiceItemId);
                            }
                        }
                    }
                }
            }
            if (list != null && list.size()!=0){
                return write(-1, "计划创建失败！您设置了重复项目时间，请修改后生成!", "data", list);
            }
            return write(200, "获取成功", "data", rehabilitationPlanService.createRehabilitationPlan(json,d));
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "updatePatientImgById", method = RequestMethod.POST)
    @ApiOperation("更新康复计划居民签名照/证件照")
    public String updatePatientImgById(@ApiParam(name = "planId", value = "康复计划id", required = true)
                                       @RequestParam(value = "planId")String planId,
                                       @ApiParam(name = "patientImg", value = "居民签名/证件照地址")
                                       @RequestParam(value = "patientImg")String patientImg){

        try {
            rehabilitationPlanService.updatePatientImgById(planId, patientImg);
            return write(200, "更新成功！");
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "sendWxMsg", method = RequestMethod.POST)
    @ApiOperation("发送微信模板消息")
    public String sendWxTemplate(@ApiParam(name = "planId", value = "康复计划id", required = true)
                                       @RequestParam(value = "planId")String planId){

        try {
            ObjEnvelop jsonObject = rehabilitationManageService.getServiceItemsAfterStop(planId);
            rehabilitationPlanService.sendWxTemplate(planId, getUID(), jsonObject);
            return write(200, "发送成功！");
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "/updateRehabilitationPlan", method = RequestMethod.POST)
    @ApiOperation(value = "编辑居民康复计划")
    public String updateRehabilitationPlan(@ApiParam(name = "doctor", value = "医生标识")
                                           @RequestParam(required = false)String doctor,
                                           @ApiParam(name = "json", value = "康复计划json")
                                           @RequestParam(required = true) String json,
                                           @ApiParam(name = "updateStatus", value = "修改类型：0单个服务的修改(默认)，1整个计划修改")
                                           @RequestParam(required = false,defaultValue = "0") String updateStatus){
        try {
            if(!StringUtils.isNotBlank(doctor)){
                doctor = getUID();
            }
            BaseDoctorDO d = doctorDao.findByIdAndDel(doctor);
            JSONObject object = new JSONObject(json);
            JSONArray array = object.getJSONArray("detail");//获取服务内容
            Set<String> list = new HashSet<>();
            for(int i=0;i<array.length();i++){
                JSONObject jsonObject = array.getJSONObject(i);
                String hospitalServiceItemId = jsonObject.getString("hospitalServiceItemId");//服务id
                String frequencyCode = jsonObject.getString("frequencyCode");
                Long timeType = jsonObject.getLong("timeType");
                String executeTimes =jsonObject.getString("executeTimes");
                String str[] = executeTimes.split(",");
                for (int j=i+1;j<array.length();j++){
                    JSONObject jsonObject1 = array.getJSONObject(j);
                    String hospitalServiceItemId1 = jsonObject1.getString("hospitalServiceItemId");
                    String frequencyCode1 = jsonObject1.getString("frequencyCode");
                    Long timeType1 = jsonObject1.getLong("timeType");
                    String executeTimes1 =jsonObject1.getString("executeTimes");
                    if (hospitalServiceItemId.equalsIgnoreCase(hospitalServiceItemId1)&&frequencyCode.equalsIgnoreCase(frequencyCode1)&&timeType.equals(timeType1)){
                        for (int z=0;z<str.length;z++){
                            if (executeTimes1.contains(str[z])){//时间与之前项目时间相同
                                list.add(hospitalServiceItemId);
                            }
                        }
                    }
                }
            }
            if (list != null && list.size()!=0){
                return write(-1, "计划创建失败！您设置了重复项目时间，请修改后生成!", "data", list);
            }
            return write(200, "获取成功", "data", rehabilitationPlanService.updateRehabilitationPlan(json,d,updateStatus));
        } catch (ServiceException se) {
            return write(-1,se.getMessage());
        } catch (Exception e) {
            return errorResult(e);
        }
    }

    @RequestMapping(value = "getRehabilitationPlanLog",method = RequestMethod.GET)
    @ApiOperation(value = "获取康复计划修改记录")
    public String getRehabilitationPlanLog(@ApiParam(name = "planId",value = "康复计划Id",required = true)
                                           @RequestParam(value = "planId",required = true) String planId){

        try {
            return write(200,"获取成功","data",rehabilitationPlanService.getRehabilitationPlanLog(planId));
        }catch (Exception e){
            return errorResult(e);
        }
    }

}
