package com.yihu.jw.afterDiagnosis.service.followup;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.afterDiagnosis.dao.followup.FollowUpDao;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.elasticsearch.ElasticSearch7Helper;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.followup.Followup;
import com.yihu.jw.followup.*;
import com.yihu.jw.followup.followupDisease.*;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.entity.ServiceException;
import io.searchbox.client.JestClient;
import org.apache.commons.lang3.StringUtils;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.transaction.Transactional;
import java.beans.PropertyDescriptor;
import java.io.IOException;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Component
@Transactional(rollbackOn = Exception.class)
public class VisitDetailService {

    @Value("${es.index.PostpartumVisitRecord}")
    private String esIndexPostpartum;
    @Value("${es.index.FollowupSevereMentalDisorder}")
    private String esIndexSevereMentalDisorder;
    @Value("${es.index.OlderPeopleHealthCheckRecord}")
    private String esIndexOlderPeopleHealthCheck;
    @Value("${es.index.NewbornFamilyVisitRecord}")
    private String esIndexNewbornFamilyVisit;
    @Value("${es.index.phthisis_n_times_followup_index}")
    private String esIndexPhthisisNTimes;
    @Value("${es.index.GynecologicalInflammationDaily}")
    private String esIndexGynecologicalInflammationDaily;
    @Value("${es.index.GynecologicalInflammationFollowupArchives}")
    private String esIndexGynecologicalInflammationFollowupArchives;
    @Value("${es.index.COPDFirstTimeQuestionnaire}")
    private String esIndexCOPDFirstTimeQuestionnaire;
    @Value("${es.index.COPDDailyFollowup}")
    private String esIndexCOPDDailyFollowup;
    //慢性乳腺疾病日常随访
    @Value("${es.index.BreastFollowup}")
    private String esIndexBreastFollowup;//慢性乳腺疾病日常随访
    @Value("${es.index.CerebralApoplexyFollowup}")
    private String esIndexCerebralApoplexyFollowup;
    @Value("${es.index.CoronaryFollowup}")
    private String esIndexCoronaryFollowup;
    @Value("${es.index.FattyLiverFollowUp}")
    private String esIndexFattyLiverFollowUp;
    @Value("${es.index.NephropathyFollowup}")
    private String esIndexNephropathyFollowup;
    @Value("${es.index.ProstateFollowup}")
    private String esIndexProstateFollowup;
    @Autowired
    private ElasticSearch7Helper elasticSearch7Helper;
    @Autowired
    private FollowUpDao followupDao;
    @Autowired
    private BaseDoctorDao doctorDao;
    @Autowired
    private BasePatientDao patientDao;

    /**
     * @Title: getNullPropertyNames
     * @Description: 获取值为空的属性名称
     * @createdBy:byrc
     */
    public static String[] getNullPropertyNames(Object source) {
        BeanWrapper src = new BeanWrapperImpl(source);
        PropertyDescriptor[] pds = src.getPropertyDescriptors();

        Set<String> emptyNames = new HashSet<String>();
        for (PropertyDescriptor pd : pds) {
            Object srcValue = src.getPropertyValue(pd.getName());
            if (srcValue == null)
                emptyNames.add(pd.getName());
        }
        String[] result = new String[emptyNames.size()];
        return emptyNames.toArray(result);
    }

    /**
     * 保存孕产妇访视详情记录
     * @param followupId
     * @param followupProject
     * @param postpartumVisitDetails
     * @param doctorCode
     * @throws Exception
     */
    public void savePostpartumVisitDetails(String followupId, String followupProject, String postpartumVisitDetails, String doctorCode) throws Exception {
        BaseDoctorDO d = null;
        if(StringUtils.isNotBlank(doctorCode)){
            d = doctorDao.findByIdAndDel(doctorCode);
        }else {
            throw new ServiceException("该医生不存在！");
        }
        //先根据条件查找出来
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.query(
                new BoolQueryBuilder()
                        .must(QueryBuilders.matchQuery("followup_id", followupId))
//                        .must(QueryBuilders.matchQuery("followup_project", followupProject))
        );
        PostpartumVisitRecordES postpartumVisitRecordES = elasticSearch7Helper.searchOne(esIndexPostpartum,searchSourceBuilder,PostpartumVisitRecordES.class);

        Followup followup = followupDao.findById(Long.parseLong(followupId)).orElse(null);
        //设置当前随访状态
        if(followup.getStatus().equals("2")){
            followup.setStatus("3");
            followupDao.save(followup);
        }

        //保存新的随访详情信息,如果是编辑的id不变
        String id = null;
        if(postpartumVisitRecordES != null && StringUtils.isNotBlank(postpartumVisitRecordES.getId())) {
            id = postpartumVisitRecordES.getId();
        }
        postpartumVisitRecordES = JSON.parseObject(postpartumVisitDetails,PostpartumVisitRecordES.class);
        postpartumVisitRecordES.setId(id);
        postpartumVisitRecordES.setFollowup_id(followupId);
        postpartumVisitRecordES.setFollowup_project(followupProject);
        postpartumVisitRecordES.setCreate_time(new Date());
        postpartumVisitRecordES.setCreate_user(doctorCode);
        postpartumVisitRecordES.setCreate_user_name(d.getName());
        postpartumVisitRecordES.setVisiting_doctor_code(doctorCode);
        elasticSearch7Helper.saveOne(esIndexPostpartum, postpartumVisitRecordES);
    }

    /**
     * 根据随访id获取孕产妇访视详情
     * @param followupId
     * @param followupProject
     * @return
     * @throws Exception
     */
    public PostpartumVisitRecordES getPostpartumVisitRecord(String followupId, String followupProject) throws Exception{
        PostpartumVisitRecordES postpartumVisitRecordES = null;
        try {
            //根据随访ID、分类ID获取随访记录详情
            //先根据条件查找出来
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            searchSourceBuilder.query(
                    new BoolQueryBuilder()
                            .must(QueryBuilders.matchQuery("followup_id", followupId))
//                            .must(QueryBuilders.matchQuery("followup_project", followupProject))
            );

            postpartumVisitRecordES = elasticSearch7Helper.searchOne(esIndexPostpartum,searchSourceBuilder,PostpartumVisitRecordES.class);
        } catch (Exception e){
            e.printStackTrace();
        }
        return postpartumVisitRecordES;
    }

    /**
     * 保存严重精神障碍患者随访记录
     * @param followupId
     * @param followupProject
     * @param severeMentalDisorderData
     * @param doctorCode
     * @throws Exception
     */
    public void saveFollowupSevereMentalDisorder(String followupId, String followupProject, String severeMentalDisorderData, String doctorCode) throws Exception{
        BaseDoctorDO d = null;
        if(StringUtils.isNotBlank(doctorCode)){
            d = doctorDao.findByIdAndDel(doctorCode);
        }else {
            throw new ServiceException("该医生不存在！");
        }

        //先根据条件查找出来
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.query(
                new BoolQueryBuilder()
                        .must(QueryBuilders.matchQuery("followup_id", followupId))
//                        .must(QueryBuilders.matchQuery("followup_project", followupProject))
        );

        FollowupSevereMentalDisorderES severeMentalDisorderES = elasticSearch7Helper.searchOne(esIndexSevereMentalDisorder,searchSourceBuilder,FollowupSevereMentalDisorderES.class);
        Followup followup = followupDao.findById(Long.parseLong(followupId)).orElse(null);
        //设置当前随访状态
        if(followup.getStatus().equals("2")){
            followup.setStatus("3");
            followupDao.save(followup);
        }
        //保存新的随访详情信息
        String id = null;
        if(severeMentalDisorderES != null && StringUtils.isNotBlank(severeMentalDisorderES.getId())) {
            id = severeMentalDisorderES.getId();
        }
        severeMentalDisorderES = JSON.parseObject(severeMentalDisorderData,FollowupSevereMentalDisorderES.class);
        severeMentalDisorderES.setId(id);
        severeMentalDisorderES.setFollowup_id(followupId);
        severeMentalDisorderES.setFollowup_project(followupProject);
        severeMentalDisorderES.setCreate_time(new Date());
        severeMentalDisorderES.setCreate_user(doctorCode);
        severeMentalDisorderES.setCreate_user_name(d.getName());
        elasticSearch7Helper.saveOne(esIndexSevereMentalDisorder, severeMentalDisorderES);
    }

    /**
     * 获取严重精神障碍患者随访记录
     * @param followupId
     * @param followupProject
     * @return
     * @throws IOException
     */
    public FollowupSevereMentalDisorderES getFollowupSevereMentalDisorder(String followupId, String followupProject) throws IOException {
        FollowupSevereMentalDisorderES severeMentalDisorderES = null;
        try {
            //根据随访ID获取随访记录详情
            //先根据条件查找出来
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            searchSourceBuilder.query(
                    new BoolQueryBuilder()
                            .must(QueryBuilders.matchQuery("followup_id", followupId))
//                            .must(QueryBuilders.matchQuery("followup_project", followupProject))
            );
            severeMentalDisorderES = elasticSearch7Helper.searchOne(esIndexSevereMentalDisorder,searchSourceBuilder,FollowupSevereMentalDisorderES.class);
        } catch (Exception e){
            e.printStackTrace();
        }
        return severeMentalDisorderES;
    }

    /**
     * 保存老人健康体检随访记录
     * @param followupId
     * @param followupProject
     * @param olderPeopleHealthCheckData
     * @param doctorCode
     * @throws Exception
     */
    public void saveOlderPeopleHealthCheckRecord(String followupId, String followupProject, String olderPeopleHealthCheckData, String doctorCode,Integer bdStatus,Integer pgStatus) throws Exception{
        BaseDoctorDO d = null;
        if(StringUtils.isNotBlank(doctorCode)){
            d = doctorDao.findByIdAndDel(doctorCode);
        }else {
            throw new ServiceException("该医生不存在！");
        }
        //先根据条件查找出来
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.query(
                new BoolQueryBuilder()
                        .must(QueryBuilders.matchQuery("followup_id", followupId))
//                        .must(QueryBuilders.matchQuery("followup_project", followupProject))
        );

        OlderPeopleHealthCheckRecordES olderPeopleHealthCheckRecord = elasticSearch7Helper.searchOne(esIndexOlderPeopleHealthCheck,searchSourceBuilder,OlderPeopleHealthCheckRecordES.class);
        Followup followup = followupDao.findById(Long.parseLong(followupId)).orElse(null);
        if (followup.getFollowupClass().equals("8")){
            if (bdStatus != null && bdStatus == 1){
                followup.setBdStatus(1);
            }else {
                followup.setPgStatus(1);
            }
        }
        followupDao.save(followup);
        //设置当前随访状态
        if(followup.getStatus().equals("2")){
            followup.setStatus("3");
            followupDao.save(followup);
        }
        //保存新的随访详情信息
        String id = null;
        if(olderPeopleHealthCheckRecord != null && StringUtils.isNotBlank(olderPeopleHealthCheckRecord.getId())) {
            id = olderPeopleHealthCheckRecord.getId();
        }
        olderPeopleHealthCheckRecord = JSON.parseObject(olderPeopleHealthCheckData,OlderPeopleHealthCheckRecordES.class);
        olderPeopleHealthCheckRecord.setId(id);
        olderPeopleHealthCheckRecord.setFollowup_id(followupId);
        olderPeopleHealthCheckRecord.setFollowup_project(followupProject);
        olderPeopleHealthCheckRecord.setCreate_time(new Date());
        olderPeopleHealthCheckRecord.setCreate_user(doctorCode);
        olderPeopleHealthCheckRecord.setCreate_user_name(d.getName());
        elasticSearch7Helper.saveOne(esIndexOlderPeopleHealthCheck, olderPeopleHealthCheckRecord);
    }

    /**
     * 获取老人健康体检随访记录
     * @param followupId
     * @param followupProject
     * @return
     * @throws Exception
     */
    public OlderPeopleHealthCheckRecordES getOlderPeopleHealthCheckRecord(String followupId, String followupProject) throws Exception{
        OlderPeopleHealthCheckRecordES olderPeopleHealthCheckRecord = null;
        try {
            //根据随访ID获取随访记录详情
            //先根据条件查找出来
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            searchSourceBuilder.query(
                    new BoolQueryBuilder()
                            .must(QueryBuilders.matchQuery("followup_id", followupId))
//                            .must(QueryBuilders.matchQuery("followup_project", followupProject))
            );
            olderPeopleHealthCheckRecord = elasticSearch7Helper.searchOne(esIndexOlderPeopleHealthCheck,searchSourceBuilder,OlderPeopleHealthCheckRecordES.class);
        } catch (Exception e){
            e.printStackTrace();
        }
        return olderPeopleHealthCheckRecord;
    }

    /**
     * 保存新生儿家庭访视记录
     * @param followupId
     * @param followupProject
     * @param newbornFamilyVisitRecordData
     * @param doctorCode
     * @throws Exception
     */
    public void saveNewbornFamilyVisitRecord(String followupId, String followupProject, String newbornFamilyVisitRecordData, String doctorCode) throws Exception{
        BaseDoctorDO d = null;
        if(StringUtils.isNotBlank(doctorCode)){
            d = doctorDao.findByIdAndDel(doctorCode);
        }else {
            throw new ServiceException("该医生不存在！");
        }
        //先根据条件查找出来
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.query(
                new BoolQueryBuilder()
                        .must(QueryBuilders.matchQuery("followup_id", followupId))
//                        .must(QueryBuilders.matchQuery("followup_project", followupProject))
        );

        NewbornFamilyVisitRecordES newbornFamilyVisitRecordES = elasticSearch7Helper.searchOne(esIndexNewbornFamilyVisit,searchSourceBuilder,NewbornFamilyVisitRecordES.class);
        Followup followup = followupDao.findById(Long.parseLong(followupId)).orElse(null);
        //设置当前随访状态
        if(followup.getStatus().equals("2")){
            followup.setStatus("3");
            followupDao.save(followup);
        }
        //保存新的随访详情信息
        String id = null;
        if(newbornFamilyVisitRecordES != null && StringUtils.isNotBlank(newbornFamilyVisitRecordES.getId())) {
            id = newbornFamilyVisitRecordES.getId();
        }
        newbornFamilyVisitRecordES = JSON.parseObject(newbornFamilyVisitRecordData,NewbornFamilyVisitRecordES.class);
        newbornFamilyVisitRecordES.setId(id);
        newbornFamilyVisitRecordES.setFollowup_id(followupId);
        newbornFamilyVisitRecordES.setFollowup_project(followupProject);
        newbornFamilyVisitRecordES.setCreate_time(new Date());
        newbornFamilyVisitRecordES.setCreate_user(doctorCode);
        newbornFamilyVisitRecordES.setCreate_user_name(d.getName());
        elasticSearch7Helper.saveOne(esIndexNewbornFamilyVisit, newbornFamilyVisitRecordES);
    }

    /**
     * 获取新生儿家庭访视记录
     * @param followupId
     * @param followupProject
     * @return
     * @throws Exception
     */
    public NewbornFamilyVisitRecordES getNewbornFamilyVisitRecord(String followupId, String followupProject) throws Exception{
        JestClient jestClient = null;
        NewbornFamilyVisitRecordES newbornFamilyVisitRecordES = null;
        try {
            //根据随访ID获取随访记录详情
            //先根据条件查找出来
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            searchSourceBuilder.query(
                    new BoolQueryBuilder()
                            .must(QueryBuilders.matchQuery("followup_id", followupId))
//                            .must(QueryBuilders.matchQuery("followup_project", followupProject))
            );

            newbornFamilyVisitRecordES = elasticSearch7Helper.searchOne(esIndexNewbornFamilyVisit,searchSourceBuilder,NewbornFamilyVisitRecordES.class);
        } catch (Exception e){
            e.printStackTrace();
        }
        return newbornFamilyVisitRecordES;
    }

    /**
     * 根据居民code和访视类别获取上次随访记录
     * @param patientCode
     * @param followupClass 5.产后 6.新生儿 7.严重精神病 8.80岁以上老人 9.肺结核
     * @return
     */
    public JSONObject getLastVisitRecord(String patientCode, String followupClass) throws Exception {
        JSONObject res = new JSONObject();
        List<Followup> followupList = followupDao.findAllByPatientCodeAndClass(patientCode, followupClass);
        if (followupList.size() > 0) {
            String sql = "";
            res.put("followup", followupList.get(0));
            //产后记录
            if(followupClass.equals("5")) {
                PostpartumVisitRecordES es = this.getPostpartumVisitRecord(followupList.get(0).getId() + "", null);
                res.put("detail", es);
            }else  if(followupClass.equals("6")) {
                NewbornFamilyVisitRecordES es = this.getNewbornFamilyVisitRecord(followupList.get(0).getId() + "", null);
                res.put("detail", es);
            }else  if(followupClass.equals("7")) {
                FollowupSevereMentalDisorderES es = this.getFollowupSevereMentalDisorder(followupList.get(0).getId() + "", null);
                res.put("detail", es);
            }else  if(followupClass.equals("8")) {
                OlderPeopleHealthCheckRecordES es = this.getOlderPeopleHealthCheckRecord(followupList.get(0).getId() + "", null);
                res.put("detail", es);
            }else  if(followupClass.equals("9") && followupList.size() >= 2) {  //非第一次肺结核访视
//                List<JSONObject> returnList = phthisisFollowupService.queryNTimesByFollowupId(followupList.get(0).getId() + "");
//                res.put("detail", returnList);
            }
        }
        return res;
    }

    /**
     * 保存慢性妇科炎症日常随访
     * @param followupId
     * @param jsonData
     * @param doctorCode
     * @return
     * @throws Exception
     */
    public GynecologicalInflammationDailyFollowupES saveGynecologicalInflammationDaily(String followupId, String jsonData, String doctorCode) throws Exception {
        JSONObject jsonObject = JSONObject.parseObject(jsonData);
        Followup followup = followupDao.findById(Long.parseLong(followupId)).orElse(null);
        Followup followupTemp = JSON.parseObject(jsonObject.getString("followup"), Followup.class);
        BeanUtils.copyProperties(followupTemp, followup, getNullPropertyNames(followupTemp));
        //设置当前随访状态
        if(followup.getStatus().equals("2")){
            followup.setStatus("3");
            followupDao.save(followup);
        }

        BaseDoctorDO d = null;
        if(StringUtils.isNotBlank(doctorCode)){
            d = doctorDao.findByIdAndDel(doctorCode);
        }else {
            throw new ServiceException("该医生不存在！");
        }
        //先根据条件查找出来
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.query(
                new BoolQueryBuilder()
                        .must(QueryBuilders.matchQuery("followup_id", followupId))
//                        .must(QueryBuilders.matchQuery("followup_project", followupProject))
        );
        GynecologicalInflammationDailyFollowupES dailyFollowupES = elasticSearch7Helper.searchOne(esIndexGynecologicalInflammationDaily,searchSourceBuilder,GynecologicalInflammationDailyFollowupES.class);
        GynecologicalInflammationDailyFollowupES dailyFollowupNew = JSON.parseObject(jsonObject.getString("content"), GynecologicalInflammationDailyFollowupES.class);
        if(dailyFollowupES != null) {
            BeanUtils.copyProperties(dailyFollowupNew, dailyFollowupES, getNullPropertyNames(dailyFollowupNew));
        }else {
            dailyFollowupES = dailyFollowupNew;
        }
        //保存新的随访详情信息
        if(StringUtils.isBlank(dailyFollowupES.getId())) {
            dailyFollowupES.setFollowup_id(followupId);
            dailyFollowupES.setCreateTime(new Date());
            dailyFollowupES.setCreateOperator(doctorCode);
            dailyFollowupES.setCreateOperatorName(d.getName());
            dailyFollowupES.setValidFlag("1");
        }
        dailyFollowupES.setModifyTime(new Date());
        dailyFollowupES.setModifyOperator(doctorCode);
        dailyFollowupES.setModifyOperatorName(d.getName());
        elasticSearch7Helper.saveOne(esIndexGynecologicalInflammationDaily, dailyFollowupES);
        return dailyFollowupES;
    }

    /**
     * 获取慢性妇科炎症日常随访
     * @param followupId
     * @return
     * @throws Exception
     */
    public JSONObject getGynecologicalInflammationDaily(String followupId) throws Exception {
        JSONObject jsonObject = new JSONObject();
        Followup followup = followupDao.findById(Long.parseLong(followupId)).orElse(null);
        jsonObject.put("followup", followup);
        GynecologicalInflammationDailyFollowupES dailyFollowupES = null;
        try {
            //根据随访ID获取随访记录详情
            //先根据条件查找出来
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            searchSourceBuilder.query(
                    new BoolQueryBuilder()
                            .must(QueryBuilders.matchQuery("followup_id", followupId))
            );
            dailyFollowupES = elasticSearch7Helper.searchOne(esIndexGynecologicalInflammationDaily,searchSourceBuilder,GynecologicalInflammationDailyFollowupES.class);
            jsonObject.put("content", dailyFollowupES);
        } catch (Exception e){
            e.printStackTrace();
        }
        return jsonObject;
    }


    /**
     * 保存妇科炎症随访档案
     * @param followupId
     * @param jsonData
     * @param doctorCode
     * @return
     * @throws Exception
     */
    public GynecologicalInflammationFollowupArchivesES saveGynecologicalInflammationFollowupArchives(String followupId, String jsonData, String doctorCode) throws Exception {
        JSONObject jsonObject = JSONObject.parseObject(jsonData);
        Followup followup = followupDao.findById(Long.parseLong(followupId)).orElse(null);
        Followup followupTemp = JSON.parseObject(jsonObject.getString("followup"), Followup.class);
        BeanUtils.copyProperties(followupTemp, followup, getNullPropertyNames(followupTemp));
        //设置当前随访状态
        if(followup.getStatus().equals("2")){
            followup.setStatus("3");
            followupDao.save(followup);
        }
        BaseDoctorDO d = null;
        if(StringUtils.isNotBlank(doctorCode)){
            d = doctorDao.findByIdAndDel(doctorCode);
        }else {
            throw new ServiceException("该医生不存在！");
        }
        //先根据条件查找出来
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.query(
                new BoolQueryBuilder()
                        .must(QueryBuilders.matchQuery("followup_id", followupId))
//                        .must(QueryBuilders.matchQuery("followup_project", followupProject))
        );

        GynecologicalInflammationFollowupArchivesES followupArchives = elasticSearch7Helper.searchOne(esIndexGynecologicalInflammationFollowupArchives,searchSourceBuilder,GynecologicalInflammationFollowupArchivesES.class);
        GynecologicalInflammationFollowupArchivesES followupArchivesNew = JSON.parseObject(jsonObject.getString("content"), GynecologicalInflammationFollowupArchivesES.class);
        if(followupArchives != null) {
            BeanUtils.copyProperties(followupArchivesNew, followupArchives, getNullPropertyNames(followupArchivesNew));
        }else {
            followupArchives = followupArchivesNew;
        }
        //保存新的随访详情信息
        if(StringUtils.isBlank(followupArchives.getId())) {
            followupArchives.setFollowup_id(followupId);
            followupArchives.setCreateTime(new Date());
            followupArchives.setCreateOperator(doctorCode);
            followupArchives.setCreateOperatorName(d.getName());
            followupArchives.setValidFlag("1");
        }
        followupArchives.setModifyTime(new Date());
        followupArchives.setModifyOperator(doctorCode);
        followupArchives.setModifyOperatorName(d.getName());
        elasticSearch7Helper.saveOne(esIndexGynecologicalInflammationFollowupArchives, followupArchives);
        return followupArchives;
    }


    /**
     * 获取各类病种单日常随访
     * type:
     * BreastFollowup //4.慢性乳腺疾病日常随访
     * CerebralApoplexyFollowup //7.脑卒中日常随访
     * CoronaryFollowup//1.冠心病日常随访
     * FattyLiverFollowUp//8.脂肪肝日常随访
     * NephropathyFollowup//5.慢性肾病日常随访
     * ProstateFollowup//3.慢性前列腺疾病日常随访
     * @param followupId
     * @return
     * @throws Exception
     */
    public JSONObject selectFollowupDiseaseById(String followupId,String type) throws Exception {
        JSONObject jsonObject = new JSONObject();
        Followup followup = followupDao.findById(Long.parseLong(followupId)).orElse(null);
        jsonObject.put("followup", followup);
        BreastFollowupES breastFollowupES = null;
        CerebralApoplexyFollowupES cerebralApoplexyFollowupES = null;
        CoronaryFollowupES coronaryFollowupES = null;
        FattyLiverFollowUpES fattyLiverFollowUpES = null;
        NephropathyFollowupES nephropathyFollowupES = null;
        ProstateFollowupES prostateFollowupES = null;
        try {
            //根据随访ID获取随访记录详情
            //先根据条件查找出来
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            searchSourceBuilder.query(
                    new BoolQueryBuilder()
                            .must(QueryBuilders.matchQuery("followup_id", followupId))
            );
            if (type.equalsIgnoreCase("BreastFollowup")) {
                breastFollowupES = elasticSearch7Helper.searchOne(esIndexBreastFollowup,searchSourceBuilder,BreastFollowupES.class);
                jsonObject.put("content", breastFollowupES);
            } else if (type.equalsIgnoreCase("CerebralApoplexyFollowup")) {
                cerebralApoplexyFollowupES = elasticSearch7Helper.searchOne(esIndexCerebralApoplexyFollowup,searchSourceBuilder,CerebralApoplexyFollowupES.class);
                jsonObject.put("content", cerebralApoplexyFollowupES);
            } else if (type.equalsIgnoreCase("CoronaryFollowup")) {
                coronaryFollowupES = elasticSearch7Helper.searchOne(esIndexCoronaryFollowup,searchSourceBuilder,CoronaryFollowupES.class);
                jsonObject.put("content", coronaryFollowupES);
            } else if (type.equalsIgnoreCase("FattyLiverFollowUp")) {
                fattyLiverFollowUpES = elasticSearch7Helper.searchOne(esIndexFattyLiverFollowUp,searchSourceBuilder,FattyLiverFollowUpES.class);
                jsonObject.put("content", fattyLiverFollowUpES);
            } else if (type.equalsIgnoreCase("NephropathyFollowup")) {
                nephropathyFollowupES = elasticSearch7Helper.searchOne(esIndexNephropathyFollowup,searchSourceBuilder,NephropathyFollowupES.class);
                jsonObject.put("content", nephropathyFollowupES);
            } else if (type.equalsIgnoreCase("ProstateFollowup")) {
                prostateFollowupES = elasticSearch7Helper.searchOne(esIndexProstateFollowup,searchSourceBuilder,ProstateFollowupES.class);
                jsonObject.put("content", prostateFollowupES);
            }
        }catch (Exception e){
            e.printStackTrace();
            jsonObject.put("content", null);
        }
        return jsonObject;
    }

    /**
     *
     * @param followupId
     * @param jsonData
     * @param doctorCode
     * @param type
     * @return
     * @throws Exception
     * BreastFollowup: breast_followup
    CerebralApoplexyFollowup: cerebral_apoplexy_followup
    CoronaryFollowup: coronary_followup
    FattyLiverFollowUp: fatty_liverFollowUp
    NephropathyFollowup: nephropathy_followup
    ProstateFollowup: prostate_followup
     */
    public void saveFollowupDiseaseArchives(String followupId, String jsonData, String doctorCode,String type) throws Exception {
        JSONObject jsonObject = JSONObject.parseObject(jsonData);
        Followup followup = followupDao.findById(Long.parseLong(followupId)).orElse(null);
        Followup followupTemp = JSON.parseObject(jsonObject.getString("followup"), Followup.class);
        BeanUtils.copyProperties(followupTemp, followup, getNullPropertyNames(followupTemp));
        //设置当前随访状态
        if(followup.getStatus().equals("2")){
            followup.setStatus("3");
            followupDao.save(followup);
        }
        BaseDoctorDO d = null;
        if(StringUtils.isNotBlank(doctorCode)){
            d = doctorDao.findByIdAndDel(doctorCode);
        }else {
            throw new ServiceException("该医生不存在！");
        }
        //先根据条件查找出来
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.query(
                new BoolQueryBuilder()
                        .must(QueryBuilders.matchQuery("followup_id", followupId))
//                        .must(QueryBuilders.matchQuery("followup_project", followupProject))
        );
        BasePatientDO patient1 = patientDao.findById(followup.getPatientCode()).orElse(null);
        String idcard = "";
        if (patient1!=null){
            idcard = patient1.getIdcard();
        }
        String followDoctor = followup.getJwdoctorCode();//随访医生

        String followTime= DateUtil.dateToStrLong(followup.getFollowupDate());//随访时间
        //判断随访方式【1.门诊 2.电话 3.入户】,待确定
        String followupType = followup.getFollowupType();
        String followType = "";
        if(followupType.equals("1") || followupType.equals("22")) {
            followType="1";
        }else if(followupType.equals("3")) {
            followType="2";
        }
        BaseDoctorDO doctor = doctorDao.findByIdAndDel(followup.getDoctorCode());
        String followDoctorType=doctor.getLevel()+"";
        String nextFollowTime=DateUtil.dateToStrLong(followup.getFollowupNextDate());//下次随访时间
        String assessSource="4";//评估来源【1.前台三合一 2. 前台独立 3.后台自动 4.i健康】
        //判断是新增还是编辑，如果是编辑传回随访ID,新增则不传
        if (type.equalsIgnoreCase("BreastFollowup")){
            //4.慢性乳腺疾病日常随访
            BreastFollowupES breastFollowup = elasticSearch7Helper.searchOne(esIndexBreastFollowup,searchSourceBuilder,BreastFollowupES.class);
            BreastFollowupES breastFollowupNew = JSON.parseObject(jsonObject.getString("content"), BreastFollowupES.class);
            if(breastFollowup != null) {
                BeanUtils.copyProperties(breastFollowupNew, breastFollowup, getNullPropertyNames(breastFollowupNew));
            }else {
                breastFollowup = breastFollowupNew;
            }
            //保存新的随访详情信息
            if(StringUtils.isBlank(breastFollowup.getId())) {
                breastFollowup.setFollowup_id(followupId);
                breastFollowup.setCreateTime(DateUtil.getStringDate());
                breastFollowup.setCreateOperator(doctorCode);
                breastFollowup.setCreateOperatorName(d.getName());
                breastFollowup.setValidFlag("1");
                breastFollowup.setIdentityCardNo(idcard);
//                breastFollowup.setOrgId(orgId);
//                breastFollowup.setFollowOrgId(followOrgId);
                breastFollowup.setFollowDoctor(followDoctor);
                breastFollowup.setFollowTime(followTime);
                breastFollowup.setFollowType(followType);
                breastFollowup.setFollowDoctorType(followDoctorType);
                breastFollowup.setAssessSource(assessSource);
                breastFollowup.setNextFollowTime(nextFollowTime);
            }
            breastFollowup.setModifyTime(DateUtil.getStringDate());
            breastFollowup.setModifyOperator(doctorCode);
            breastFollowup.setModifyOperatorName(d.getName());
            elasticSearch7Helper.saveOne(esIndexBreastFollowup, breastFollowup);
        }else if (type.equalsIgnoreCase("CoronaryFollowup")){
            //1.冠心病日常随访
            CoronaryFollowupES coronaryFollowup = elasticSearch7Helper.searchOne(esIndexCoronaryFollowup,searchSourceBuilder,CoronaryFollowupES.class);
            CoronaryFollowupES coronaryFollowupNew = JSON.parseObject(jsonObject.getString("content"), CoronaryFollowupES.class);
            if(coronaryFollowup != null) {
                BeanUtils.copyProperties(coronaryFollowupNew, coronaryFollowup, getNullPropertyNames(coronaryFollowupNew));
            }else {
                coronaryFollowup = coronaryFollowupNew;
            }
            //保存新的随访详情信息
            if(StringUtils.isBlank(coronaryFollowup.getId())) {
                coronaryFollowup.setFollowup_id(followupId);
                coronaryFollowup.setCreateTime(DateUtil.getStringDate());
                coronaryFollowup.setCreateOperator(doctorCode);
                coronaryFollowup.setCreateOperatorName(d.getName());
                coronaryFollowup.setValidFlag("1");
                coronaryFollowup.setIdentityCardNo(idcard);
//                coronaryFollowup.setOrgId(orgId);
//                coronaryFollowup.setFollowOrgId(followOrgId);
                coronaryFollowup.setFollowDoctor(followDoctor);
                coronaryFollowup.setFollowTime(followTime);
                coronaryFollowup.setFollowType(followType);
                coronaryFollowup.setFollowDoctorType(followDoctorType);
                coronaryFollowup.setAssessSource(assessSource);
                coronaryFollowup.setNextFollowTime(nextFollowTime);
            }
            coronaryFollowup.setModifyTime(DateUtil.getStringDate());
            coronaryFollowup.setModifyOperator(doctorCode);
            coronaryFollowup.setModifyOperatorName(d.getName());
            elasticSearch7Helper.saveOne(esIndexCoronaryFollowup, coronaryFollowup);
        }else if (type.equalsIgnoreCase("CerebralApoplexyFollowup")){
            //7.脑卒中日常随访
            CerebralApoplexyFollowupES cerebralApoplexyFollowup = elasticSearch7Helper.searchOne(esIndexCerebralApoplexyFollowup,searchSourceBuilder,CerebralApoplexyFollowupES.class);
            CerebralApoplexyFollowupES cerebralApoplexyFollowupNew = JSON.parseObject(jsonObject.getString("content"), CerebralApoplexyFollowupES.class);
            if(cerebralApoplexyFollowup != null) {
                BeanUtils.copyProperties(cerebralApoplexyFollowupNew, cerebralApoplexyFollowup, getNullPropertyNames(cerebralApoplexyFollowupNew));
            }else {
                cerebralApoplexyFollowup = cerebralApoplexyFollowupNew;
            }
            //保存新的随访详情信息
            if(StringUtils.isBlank(cerebralApoplexyFollowup.getId())) {
                cerebralApoplexyFollowup.setFollowup_id(followupId);
                cerebralApoplexyFollowup.setCreateTime(DateUtil.getStringDate());
                cerebralApoplexyFollowup.setCreateOperator(doctorCode);
                cerebralApoplexyFollowup.setCreateOperatorName(d.getName());
                cerebralApoplexyFollowup.setValidFlag("1");
                cerebralApoplexyFollowup.setIdentityCardNo(idcard);
//                cerebralApoplexyFollowup.setOrgId(orgId);
//                cerebralApoplexyFollowup.setFollowOrgId(followOrgId);
                cerebralApoplexyFollowup.setFollowDoctor(followDoctor);
                cerebralApoplexyFollowup.setFollowTime(followTime);
                cerebralApoplexyFollowup.setFollowType(followType);
                cerebralApoplexyFollowup.setFollowDoctorType(followDoctorType);
                cerebralApoplexyFollowup.setAssessSource(assessSource);
                cerebralApoplexyFollowup.setNextFollowTime(nextFollowTime);
            }
            cerebralApoplexyFollowup.setModifyTime(DateUtil.getStringDate());
            cerebralApoplexyFollowup.setModifyOperator(doctorCode);
            cerebralApoplexyFollowup.setModifyOperatorName(d.getName());
            elasticSearch7Helper.saveOne(esIndexCerebralApoplexyFollowup , cerebralApoplexyFollowup);
        }else if (type.equalsIgnoreCase("FattyLiverFollowUp")){
            //8.脂肪肝日常随访
            FattyLiverFollowUpES fattyLiverFollowUp = elasticSearch7Helper.searchOne(esIndexFattyLiverFollowUp,searchSourceBuilder,FattyLiverFollowUpES.class);
            FattyLiverFollowUpES fattyLiverFollowUpNew = JSON.parseObject(jsonObject.getString("content"), FattyLiverFollowUpES.class);
            if(fattyLiverFollowUp != null) {
                BeanUtils.copyProperties(fattyLiverFollowUpNew, fattyLiverFollowUp, getNullPropertyNames(fattyLiverFollowUpNew));
            }else {
                fattyLiverFollowUp = fattyLiverFollowUpNew;
            }
            //保存新的随访详情信息
            if(StringUtils.isBlank(fattyLiverFollowUp.getId())) {
                fattyLiverFollowUp.setFollowup_id(followupId);
                fattyLiverFollowUp.setCreateTime(DateUtil.getStringDate());
                fattyLiverFollowUp.setCreateOperator(doctorCode);
                fattyLiverFollowUp.setCreateOperatorName(d.getName());
                fattyLiverFollowUp.setValidFlag("1");
                fattyLiverFollowUp.setIdentityCardNo(idcard);
//                fattyLiverFollowUp.setOrgId(orgId);
//                fattyLiverFollowUp.setFollowOrgId(followOrgId);
                fattyLiverFollowUp.setFollowDoctor(followDoctor);
                fattyLiverFollowUp.setFollowTime(followTime);
                fattyLiverFollowUp.setFollowType(followType);
                fattyLiverFollowUp.setFollowDoctorType(followDoctorType);
                fattyLiverFollowUp.setAssessSource(assessSource);
                fattyLiverFollowUp.setNextFollowTime(nextFollowTime);
            }
            fattyLiverFollowUp.setModifyTime(DateUtil.getStringDate());
            fattyLiverFollowUp.setModifyOperator(doctorCode);
            fattyLiverFollowUp.setModifyOperatorName(d.getName());
            elasticSearch7Helper.saveOne(esIndexFattyLiverFollowUp , fattyLiverFollowUp);
        }else if (type.equalsIgnoreCase("NephropathyFollowup")){
            //8.脂肪肝日常随访
            NephropathyFollowupES nephropathyFollowup = elasticSearch7Helper.searchOne(esIndexNephropathyFollowup,searchSourceBuilder,NephropathyFollowupES.class);
            NephropathyFollowupES nephropathyFollowupNew = JSON.parseObject(jsonObject.getString("content"), NephropathyFollowupES.class);
            if(nephropathyFollowup != null) {
                BeanUtils.copyProperties(nephropathyFollowupNew, nephropathyFollowup, getNullPropertyNames(nephropathyFollowupNew));
            }else {
                nephropathyFollowup = nephropathyFollowupNew;
            }
            //保存新的随访详情信息
            if(StringUtils.isBlank(nephropathyFollowup.getId())) {
                nephropathyFollowup.setFollowup_id(followupId);
                nephropathyFollowup.setCreateTime(DateUtil.getStringDate());
                nephropathyFollowup.setCreateOperator(doctorCode);
                nephropathyFollowup.setCreateOperatorName(d.getName());
                nephropathyFollowup.setValidFlag("1");
                nephropathyFollowup.setIdentityCardNo(idcard);
//                nephropathyFollowup.setOrgId(orgId);
//                nephropathyFollowup.setFollowOrgId(followOrgId);
                nephropathyFollowup.setFollowDoctor(followDoctor);
                nephropathyFollowup.setFollowTime(followTime);
                nephropathyFollowup.setFollowType(followType);
                nephropathyFollowup.setFollowDoctorType(followDoctorType);
                nephropathyFollowup.setAssessSource(assessSource);
                nephropathyFollowup.setNextFollowTime(nextFollowTime);
            }
            nephropathyFollowup.setModifyTime(DateUtil.getStringDate());
            nephropathyFollowup.setModifyOperator(doctorCode);
            nephropathyFollowup.setModifyOperatorName(d.getName());
            elasticSearch7Helper.saveOne(esIndexNephropathyFollowup , nephropathyFollowup);
        }else if (type.equalsIgnoreCase("ProstateFollowup")){
            //8.脂肪肝日常随访
            ProstateFollowupES prostateFollowup = elasticSearch7Helper.searchOne(esIndexProstateFollowup,searchSourceBuilder,ProstateFollowupES.class);
            ProstateFollowupES prostateFollowupNew = JSON.parseObject(jsonObject.getString("content"), ProstateFollowupES.class);
            if(prostateFollowup != null) {
                BeanUtils.copyProperties(prostateFollowupNew, prostateFollowup, getNullPropertyNames(prostateFollowupNew));
            }else {
                prostateFollowup = prostateFollowupNew;
            }
            //保存新的随访详情信息
            if(StringUtils.isBlank(prostateFollowup.getId())) {
                prostateFollowup.setFollowup_id(followupId);
                prostateFollowup.setCreateTime(DateUtil.getStringDate());
                prostateFollowup.setCreateOperator(doctorCode);
                prostateFollowup.setCreateOperatorName(d.getName());
                prostateFollowup.setValidFlag("1");
                prostateFollowup.setIdentityCardNo(idcard);
//                prostateFollowup.setOrgId(orgId);
//                prostateFollowup.setFollowOrgId(followOrgId);
                prostateFollowup.setFollowDoctor(followDoctor);
                prostateFollowup.setFollowTime(followTime);
                prostateFollowup.setFollowType(followType);
                prostateFollowup.setFollowDoctorType(followDoctorType);
                prostateFollowup.setAssessSource(assessSource);
                prostateFollowup.setNextFollowTime(nextFollowTime);

            }
            prostateFollowup.setModifyTime(DateUtil.getStringDate());
            prostateFollowup.setModifyOperator(doctorCode);
            prostateFollowup.setModifyOperatorName(d.getName());
            elasticSearch7Helper.saveOne(esIndexProstateFollowup , prostateFollowup);
        }
    }

    /**
     * 获取妇科炎症随访档案
     * @param followupId
     * @return
     * @throws Exception
     */
    public JSONObject getGynecologicalInflammationFollowupArchives(String followupId) throws Exception {
        JSONObject jsonObject = new JSONObject();
        Followup followup = followupDao.findById(Long.parseLong(followupId)).orElse(null);
        jsonObject.put("followup", followup);
        GynecologicalInflammationFollowupArchivesES followupArchivesES = null;
        try {
            //根据随访ID获取随访记录详情
            //先根据条件查找出来
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            searchSourceBuilder.query(
                    new BoolQueryBuilder()
                            .must(QueryBuilders.matchQuery("followup_id", followupId))
            );
            followupArchivesES = elasticSearch7Helper.searchOne(esIndexGynecologicalInflammationFollowupArchives,searchSourceBuilder,GynecologicalInflammationFollowupArchivesES.class);
            jsonObject.put("content", followupArchivesES);
        } catch (Exception e){
            e.printStackTrace();
        }
        return jsonObject;
    }

    /**
     * 保存慢性阻塞性肺病--首次问卷
     * @param followupId
     * @param jsonData
     * @param doctorCode
     * @return
     * @throws Exception
     */
    public COPDFirstTimeQuestionnaireES saveCOPDFirstTimeQuestionnaire(String followupId, String jsonData, String doctorCode) throws Exception {
        JSONObject jsonObject = JSONObject.parseObject(jsonData);
        Followup followup = followupDao.findById(Long.parseLong(followupId)).orElse(null);
        Followup followupTemp = JSON.parseObject(jsonObject.getString("followup"), Followup.class);
        BeanUtils.copyProperties(followupTemp, followup, getNullPropertyNames(followupTemp));
        //设置当前随访状态
        if(followup.getStatus().equals("2")){
            followup.setStatus("3");
            followupDao.save(followup);
        }
        BaseDoctorDO d = null;
        if(StringUtils.isNotBlank(doctorCode)){
            d = doctorDao.findByIdAndDel(doctorCode);
        }else {
            throw new ServiceException("该医生不存在！");
        }
        //先根据条件查找出来
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.query(
                new BoolQueryBuilder()
                        .must(QueryBuilders.matchQuery("followup_id", followupId))
        );
        COPDFirstTimeQuestionnaireES COPD = elasticSearch7Helper.searchOne(esIndexCOPDFirstTimeQuestionnaire,searchSourceBuilder,COPDFirstTimeQuestionnaireES.class);
        COPDFirstTimeQuestionnaireES COPDNew = JSON.parseObject(jsonObject.getString("content"), COPDFirstTimeQuestionnaireES.class);
        if(COPD != null) {
            BeanUtils.copyProperties(COPDNew, COPD, getNullPropertyNames(COPDNew));
        }else {
            COPD = COPDNew;
        }
        //保存新的随访详情信息
        if(StringUtils.isBlank(COPD.getId())) {
            COPD.setFollowup_id(followupId);
            COPD.setValidFlag("1");
            COPD.setCreateTime(new Date());
            COPD.setCreateOperator(doctorCode);
            COPD.setCreateOperatorName(d.getName());
        }
        COPD.setModifyTime(new Date());
        COPD.setModifyOperator(doctorCode);
        COPD.setModifyOperatorName(d.getName());
        elasticSearch7Helper.saveOne(esIndexCOPDFirstTimeQuestionnaire, COPD);
        return COPD;
    }

    /**
     * 获取慢性阻塞性肺病--首次问卷
     * @param followupId
     * @return
     * @throws IOException
     */
    public JSONObject getCOPDFirstTimeQuestionnaire(String followupId) throws IOException {
        JSONObject jsonObject = new JSONObject();
        Followup followup = followupDao.findById(Long.parseLong(followupId)).orElse(null);
        jsonObject.put("followup", followup);
        JestClient jestClient = null;
        COPDFirstTimeQuestionnaireES COPD = null;
        try {
            //根据随访ID获取随访记录详情
            //先根据条件查找出来
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            searchSourceBuilder.query(
                    new BoolQueryBuilder()
                            .must(QueryBuilders.matchQuery("followup_id", followupId))
            );

            COPD = elasticSearch7Helper.searchOne(esIndexCOPDFirstTimeQuestionnaire,searchSourceBuilder,COPDFirstTimeQuestionnaireES.class);
            jsonObject.put("content", COPD);
        } catch (Exception e){
            e.printStackTrace();
        }
        return jsonObject;
    }

    /**
     * 保存慢性阻塞性肺病--日常随访
     * @param followupId
     * @param jsonData
     * @param doctorCode
     * @return
     * @throws Exception
     */
    public COPDDailyFollowupES saveCOPDDailyFollowup(String followupId, String jsonData, String doctorCode) throws Exception {
        JSONObject jsonObject = JSONObject.parseObject(jsonData);
        Followup followup = followupDao.findById(Long.parseLong(followupId)).orElse(null);
        Followup followupTemp = JSON.parseObject(jsonObject.getString("followup"), Followup.class);
        BeanUtils.copyProperties(followupTemp, followup, getNullPropertyNames(followupTemp));
        //设置当前随访状态
        if(followup.getStatus().equals("2")){
            followup.setStatus("3");
            followupDao.save(followup);
        }

        BaseDoctorDO d = null;
        if(StringUtils.isNotBlank(doctorCode)){
            d = doctorDao.findByIdAndDel(doctorCode);
        }else {
            throw new ServiceException("该医生不存在！");
        }
        //先根据条件查找出来
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.query(
                new BoolQueryBuilder()
                        .must(QueryBuilders.matchQuery("followup_id", followupId))
        );

        COPDDailyFollowupES COPD = elasticSearch7Helper.searchOne(esIndexCOPDDailyFollowup,searchSourceBuilder,COPDDailyFollowupES.class);
        COPDDailyFollowupES COPDNew = JSON.parseObject(jsonObject.getString("content"), COPDDailyFollowupES.class);
        if(COPD != null) {
            BeanUtils.copyProperties(COPDNew, COPD, getNullPropertyNames(COPDNew));
        }else {
            COPD = COPDNew;
        }
        //保存新的随访详情信息
        if(StringUtils.isBlank(COPD.getId())) {
            COPD.setFollowup_id(followupId);
            COPD.setValidFlag("1");
            COPD.setCreateTime(new Date());
            COPD.setCreateOperator(doctorCode);
            COPD.setCreateOperatorName(d.getName());
        }
        COPD.setModifyTime(new Date());
        COPD.setModifyOperator(doctorCode);
        COPD.setModifyOperatorName(d.getName());
        elasticSearch7Helper.saveOne(esIndexCOPDDailyFollowup, COPD);
        return COPD;
    }

    /**
     * 获取慢性阻塞性肺病--日常随访
     * @param followupId
     * @return
     * @throws IOException
     */
    public JSONObject getCOPDDailyFollowup(String followupId) throws IOException {
        JSONObject jsonObject = new JSONObject();
        Followup followup = followupDao.findById(Long.parseLong(followupId)).orElse(null);
        jsonObject.put("followup", followup);
        COPDDailyFollowupES COPD = null;
        try {
            //根据随访ID获取随访记录详情
            //先根据条件查找出来
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            searchSourceBuilder.query(
                    new BoolQueryBuilder()
                            .must(QueryBuilders.matchQuery("followup_id", followupId))
            );
            COPD = elasticSearch7Helper.searchOne(esIndexCOPDDailyFollowup,searchSourceBuilder,COPDDailyFollowupES.class);
            jsonObject.put("content", COPD);
        } catch (Exception e){
            e.printStackTrace();
        }
        return jsonObject;
    }
}
