package com.yihu.jw.afterDiagnosis.service.prescription;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.yihu.fastdfs.FastDFSUtil;
import com.yihu.jw.afterDiagnosis.dao.prescription.PrescriptionDispensaryCodeDao;
import com.yihu.jw.entity.hospital.prescription.PrescriptionDispensaryCode;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionDO;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionInfoDO;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionLogDO;
import com.yihu.jw.hospital.message.dao.SystemMessageDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionExpressageLogDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionInfoDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionLogDao;
import com.yihu.jw.util.common.QrcodeUtil;
import com.yihu.jw.util.entity.ServiceException;
import org.apache.commons.lang.StringUtils;
import org.json.JSONArray;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by Administrator on 2017/7/21.
 *长处方取码
 */

@Service
public class PrescriptionDispensaryCodeService {

    @Autowired
    private PrescriptionDispensaryCodeDao prescriptionDispensaryCodeDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private PrescriptionInfoDao prescriptionInfoDao;
    @Autowired
    private PrescriptionDao prescriptionDao;
    @Autowired
    private SystemMessageDao messageDao;
    @Autowired
    private PrescriptionLogDao prescriptionLogDao;
    @Autowired
    private PrescriptionExpressageLogDao prescriptionExpressageLogDao;
    @Autowired
    private WlyyPrescriptionExpressageService prescriptionExpressageService;
    @Value("${fastDFS.fastdfs_file_url}")
    private String fastdfs_file_url;
    @Autowired
    private FastDFSUtil fastDFSUtil;


    private static Object obj = new Object();
    private static Object obj2 = new Object();

    //获取居民端取药码列表
    public  JSONArray findByStatusAndTime(Integer timeType, String patientCode, Integer status, Integer page, Integer pagesize,Integer type) {
        // 分页信息
        long timeLimit = 0;
        if(timeType!=null){
            switch (timeType){
                case 1:timeLimit = 7*24*60*60;break;//近一周
                case 2:timeLimit = 7*24*60*60*31;break;//近一月
                case 3:timeLimit = 7*24*60*60*31*6;break;//近半年
                case 4:timeLimit = 7*24*60*60*31*12;break;//近一年
            }
        }
        List<Object> params = new ArrayList<>();//p2.*,p1.diagnosis as diagnosis ,p3.address as address
        String precriptionSql = "select p1.status , p1.diagnosis ,p3.address as address,p2.prescription_code as prescriptionCode,p2.code as dispensaryCode,p2.type,p1.create_time as createTime  from wlyy_prescription_dispensary_code p2 " +
                " left join  wlyy_prescription p1 on p2.prescription_code=p1.code LEFT JOIN wlyy_prescription_expressage p3 on p1.code=p3.prescription_code where p1.patient=? and p2.type=?  ";
        params.add(patientCode);
        params.add(type);
        if(status!=null){
            precriptionSql += " and p1.status = ? ";
            params.add(status);
        }else {
            precriptionSql += " and p1.status in (?,?,?,?,?,?)" ;
            params.add(WlyyPrescriptionLogDO.PrescriptionLogStatus.pay_success.getValue());
            params.add(WlyyPrescriptionLogDO.PrescriptionLogStatus.wait_expressage.getValue());
            params.add(WlyyPrescriptionLogDO.PrescriptionLogStatus.expressageing_error.getValue());
            params.add(WlyyPrescriptionLogDO.PrescriptionLogStatus.expressageJGS.getValue());
            params.add(WlyyPrescriptionLogDO.PrescriptionLogStatus.expressageing.getValue());
            params.add(WlyyPrescriptionLogDO.PrescriptionLogStatus.expressage2hospital.getValue());
//            params.add(PrescriptionLog.PrescriptionLogStatus.finish.getValue());
        }
        if(timeType!=null){
            precriptionSql +=  "AND TIMESTAMPDIFF(SECOND, p2.create_Time, CURRENT_TIME) < ?  ";
            params.add(timeLimit);
        }
        precriptionSql += " ORDER BY p1.status asc ,p2.create_Time desc ";
        precriptionSql += " limit "+(page-1)*pagesize+","+pagesize;
        List<Map<String, Object>> result = jdbcTemplate.queryForList(precriptionSql,params.toArray());
        List<Map<String, Object>> result2 = new ArrayList<>();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        for (Map<String, Object> m:result) {
            List<WlyyPrescriptionInfoDO> list = prescriptionInfoDao.findByPrescriptionId(m.get("prescriptionCode").toString(),1);
            m.put("prescriptionInfo",list);
            m.put("createTime",m.get("createTime")!=null?simpleDateFormat.format(m.get("createTime")):"");
            result2.add(m);
        }
        JSONArray array = new JSONArray(result2);
        return array;
    }

    //判断居民是否有（21等待领药、20配送中、100已完成）取药码
    public int dispensaryCodeCount(String patientCode,Integer type){
        String sql = "select  count(1) from wlyy_prescription_dispensary_code p1 left join wlyy_prescription p2 on p1.prescription_code=p2.code where p1.is_use=0 and p1.type=? and p2.patient =? ";// where p2.status in(21,20,100)  and p2.patient =?1
        int count = jdbcTemplate.queryForObject(sql,Integer.class,type,patientCode);
        return  count;
    }

    //生成取药码时，判断是否已经有重复的取药码
    public boolean getResidentQRCodeCount(String code){
        int count = prescriptionDispensaryCodeDao.getResidentQRCodeCount(code);
        if(count>0){
            return true;
        }
        return false;
    }

    @Transactional
    public void saveQRCode(String prescriptionCode) throws Exception {
        WlyyPrescriptionDO prescription = prescriptionDao.findById(prescriptionCode).orElse(null);
        //取药类型:1 自取 2快递配送 3健管师配送 4网格员派送
        Integer type = prescription.getDispensaryType();

        PrescriptionDispensaryCode p1 = prescriptionDispensaryCodeDao.findByPrescriptionCodeAndIsUseAndType(prescriptionCode,0,1);
        if(p1==null){

            p1 = savePatientQRCode(prescriptionCode);
        }
        if(p1==null){
            throw new ServiceException("二维码生成失败！");
        }
        if(type==3){
            PrescriptionDispensaryCode p2 = prescriptionDispensaryCodeDao.findByPrescriptionCodeAndIsUseAndType(prescriptionCode,0,2);
            if(p2==null){

                p2 = saveQRCode(p1.getCode(),prescriptionCode,2);
            }
            if(p2==null){
                throw new ServiceException("二维码生成失败！");
            }
            PrescriptionDispensaryCode p3 = prescriptionDispensaryCodeDao.findByPrescriptionCodeAndIsUseAndType(prescriptionCode,0,3);
            if(p3==null){

                p3 = saveQRCode(p1.getCode(),prescriptionCode,3);
            }
            if(p3==null){
                throw new ServiceException("二维码生成失败！");
            }
        }
    }

    //生成居民端取药码
    @Transactional
    public PrescriptionDispensaryCode savePatientQRCode(String prescriptionCode) throws Exception {
        //获取年月日8位数
        // 二维码内容 改成用处方号当取药码
        WlyyPrescriptionDO prescription = prescriptionDao.findById(prescriptionCode).orElse(null);
        String code = prescription.getAdmNo();
        // 生成二维码
        InputStream ipt = QrcodeUtil.createQrcode(code,300,"png");

//        Prescription类  取药类型:1 自取 2快递配送 3健管师配送
    
        String fileUrl = "";
        // 圖片列表
        List<String> tempPaths = new ArrayList<String>();
        try {
            ObjectNode imgNode = fastDFSUtil.upload(ipt,"png","QRCode_"+System.currentTimeMillis());
            com.alibaba.fastjson.JSONObject json = com.alibaba.fastjson.JSONObject.parseObject(imgNode.toString());
            tempPaths.add(json.getString("fid"));
            String urls = "";
            for (String image : tempPaths) {
                if (urls.length() == 0) {
                    urls = image;
                } else {
                    urls += "," + image;
                }
            }
            fileUrl = fastdfs_file_url + urls;
        } catch (Exception e) {
            e.printStackTrace();
        }

        //新增居民处方取码记录
        PrescriptionDispensaryCode prescriptionDispensaryCode = new PrescriptionDispensaryCode();
        prescriptionDispensaryCode.setCode(code);
        prescriptionDispensaryCode.setCreateTime(new Date());
        prescriptionDispensaryCode.setImgUrl(fileUrl);
        prescriptionDispensaryCode.setPrescriptionCode(prescriptionCode);
        prescriptionDispensaryCode.setType(1);
        prescriptionDispensaryCode.setIsUse(0);
        return prescriptionDispensaryCodeDao.save(prescriptionDispensaryCode);
    }

    //生成配送员（健管师）（2、取药码  2、配送码）
    @Transactional
    public PrescriptionDispensaryCode saveQRCode(String code,String prescriptionCode,Integer type) throws Exception {
        String jgsCode= "";
        synchronized (obj2){
            boolean bl = true;
            while (bl){
                String randomNum = randomString("0123456789",2);
                jgsCode = code +randomNum;
                bl = getResidentQRCodeCount(jgsCode);
            }
        }
        // 生成二维码
        InputStream ipt = QrcodeUtil.createQrcode(jgsCode,300,"png");
//        Prescription类  取药类型:1 自取 2快递配送 3健管师配送
        String fileUrl = "";
        // 圖片列表
        List<String> tempPaths = new ArrayList<String>();
        try {
            ObjectNode imgNode = fastDFSUtil.upload(ipt,"png","QRCode_"+System.currentTimeMillis());
            com.alibaba.fastjson.JSONObject json = com.alibaba.fastjson.JSONObject.parseObject(imgNode.toString());
            tempPaths.add(json.getString("fid"));
            String urls = "";
            for (String image : tempPaths) {
                if (urls.length() == 0) {
                    urls = image;
                } else {
                    urls += "," + image;
                }
            }
            fileUrl = fastdfs_file_url + urls;
        } catch (Exception e) {
            e.printStackTrace();
        }

        //新增配送员处方取码记录
        PrescriptionDispensaryCode prescriptionDispensaryCode = new PrescriptionDispensaryCode();
        prescriptionDispensaryCode.setCode(jgsCode);
        prescriptionDispensaryCode.setCreateTime(new Date());
        prescriptionDispensaryCode.setImgUrl(fileUrl);
        prescriptionDispensaryCode.setPrescriptionCode(prescriptionCode);
        prescriptionDispensaryCode.setType(type);
        prescriptionDispensaryCode.setIsUse(0);
        return prescriptionDispensaryCodeDao.save(prescriptionDispensaryCode);

    }


    //随机生成匹配数字
    public  String randomString(String base, int length) {
        Random random = new Random();
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < length; i++) {
            int number = random.nextInt(base.length());
            sb.append(base.charAt(number));
        }
        return sb.toString();
    }

    public Map<String, Object> getQrcode(String prescriptionCode ,String patientCode ,Integer type) throws Exception {
        String sql = "select  p2.is_use as isUse,p2.code as dispensaryCode,p2.img_url,p3.hospital_name as hospitalName  from wlyy_prescription_dispensary_code p2 " +
                " left join  wlyy_prescription p1 on p2.prescription_code=p1.code LEFT JOIN wlyy_prescription_expressage p3 on p1.code=p3.prescription_code " +
                " where p2.prescription_code=?  and p2.type=? ";
        if(type==2||type==3){
            sql +=" and p3.expressage_code=? ";
        }else{
            sql +=" and p1.patient=? ";
        }
        List<Map<String, Object>> result = jdbcTemplate.queryForList(sql,prescriptionCode,type,patientCode);
        if(result.size()>0){
            return result.get(0);
        }
        throw new ServiceException("二维码错误或者已过期！");
    }


    public Map<String, Object> getJumiQrcode(String prescriptionCode) throws Exception {
        String sql = "SELECT jpc.del isUse,pick_code dispensaryCode ,jpc.qr_code img_url ,p.hospital_name hospitalName FROM wlyy_jumi_pick_code jpc , wlyy_prescription p\n" +
                "WHERE jpc.prescription_code = '"+prescriptionCode+"' AND jpc.prescription_code = p.id AND jpc.del = 0 ORDER BY jpc.id DESC";
        List<Map<String, Object>> result = jdbcTemplate.queryForList(sql);
        if(result.size()>0){
            return result.get(0);
        }
        throw new ServiceException("二维码错误或者已过期！");
    }


    @Transactional
    public PrescriptionDispensaryCode scanQrcode(String code,String userCode) throws Exception{

        PrescriptionDispensaryCode prescriptionDispensaryCode = prescriptionDispensaryCodeDao.findByCodeAndIsUse(code,0);

        if(prescriptionDispensaryCode!=null){
            WlyyPrescriptionDO prescription = prescriptionDao.findById(prescriptionDispensaryCode.getPrescriptionCode()).orElse(null);
            if(prescription.getDispensaryType()==3){
                //健管师配送
                //修改系统的续方消息的审核状态
                messageDao.updatePreScriptionMessage(prescription.getId(), "1", 7);
            }
        }else{
            throw new ServiceException("取药码错误！");
        }
        Integer status =  prescriptionExpressageService.expressage(code, userCode);
        if (status == -1) {
            throw new ServiceException("取药码错误！");
        } else if (status == -2) {
            throw new ServiceException("编码已使用！");
        } else if (status == -3) {
            throw new ServiceException("编码类型错误！");
        } else if (status == -6) {
            throw new ServiceException("发药错误");
        } else if (status == -7) {
            throw new ServiceException("已无待取药品");
        }
        return prescriptionDispensaryCode;
    }

    @Transactional
    public PrescriptionDispensaryCode scanQrcodeHlw(String code,String equ_num) throws Exception{

        WlyyPrescriptionDO prescription = prescriptionDao.findByRealOrder(code);

        if (1==prescription.getDispensaryType()){
            if (StringUtils.isNotBlank(equ_num)){
                //判断线上续方开单科室是否与设备关联的可是相同
                    String sql = "  select dept_code from wlyy_org_address_doctor  where hospital_code='"+prescription.getHospital()+"' and equ_nums like '%"+equ_num+"%' " ;
                    List<String> equDeptCodes = jdbcTemplate.queryForList(sql,String.class);
                    if (equDeptCodes.size()==0){
                        throw new ServiceException("该自助机未找到发药人呀！可能未投入使用");
                    }
                    if (!equ_num.equals(equDeptCodes.get(0))){
                        throw new ServiceException("出药设备与续方订单开药设备不符，无法取药");
                    }
            }
        }

        PrescriptionDispensaryCode prescriptionDispensaryCode = prescriptionDispensaryCodeDao.findByPrescriptionCodeAndIsUseAndType(prescription.getId(),0,1);

        if(prescriptionDispensaryCode!=null){
            if(prescription.getDispensaryType()==3){
                //健管师配送
                //修改系统的续方消息的审核状态
                messageDao.updatePreScriptionMessage(prescription.getId(), "1", 7);
            }
            if (prescription.getStatus()<WlyyPrescriptionLogDO.PrescriptionLogStatus.pay_success.getValue()){
                throw new ServiceException("续方未支付！");
            }
            if (prescription.getStatus()==WlyyPrescriptionLogDO.PrescriptionLogStatus.expressageing_error.getValue()){
                prescriptionDispensaryCode.setIsUse(1);
                prescriptionDispensaryCodeDao.save(prescriptionDispensaryCode);
                throw new ServiceException("订单已线下退费，无法取药！");
            }

            Integer status =  prescriptionExpressageService.expressage(prescriptionDispensaryCode.getCode(), prescription.getPatientCode());
            if (status == -1) {
                throw new ServiceException("取药码错误！");
            } else if (status == -2) {
                throw new ServiceException("编码已使用！");
            } else if (status == -3) {
                throw new ServiceException("编码类型错误！");
            } else if (status == -6) {
                throw new ServiceException("发药错误");
            }else if (status == -7) {
                throw new ServiceException("已无待取药品");
            }
        }else{
            throw new ServiceException("取药码错误或已被使用！");
        }

        return prescriptionDispensaryCode;
    }

}
