package com.yihu.jw.afterDiagnosis.service.prescription;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.afterDiagnosis.dao.device.DevicePatientHealthIndexDao;
import com.yihu.jw.afterDiagnosis.dao.followup.FollowUpDao;
import com.yihu.jw.afterDiagnosis.dao.followup.FollowupDrugsDao;
import com.yihu.jw.afterDiagnosis.dao.prescription.PrescriptionFollowupContentDao;
import com.yihu.jw.afterDiagnosis.service.device.PatientHealthIndexService;
import com.yihu.jw.afterDiagnosis.service.followup.FollowUpService;
import com.yihu.jw.entity.care.device.DevicePatientHealthIndex;
import com.yihu.jw.entity.followup.Followup;
import com.yihu.jw.entity.followup.FollowupDrugs;
import com.yihu.jw.entity.hospital.prescription.PrescriptionFollowupContent;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionDO;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionInfoDO;
import com.yihu.jw.followup.FollowupContentESDO;
import com.yihu.jw.hospital.prescription.dao.PrescriptionDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionInfoDao;
import com.yihu.jw.im.util.ImUtil;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.entity.ServiceException;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * 续方咨询问卷相关方法
 * @author huangwenjie
 * @date 2017/11/2 09:06
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class PrescriptionFollowupContentService {
	private static Logger logger = LoggerFactory.getLogger(PrescriptionFollowupContentService.class);
	
	@Autowired
	private PrescriptionFollowupContentDao prescriptionFollowupContentDao;
	@Autowired
	private PrescriptionInfoDao prescriptionInfoDao;
	@Autowired
	private FollowupDrugsDao followupDrugsDao;
	@Autowired
	private PrescriptionDao prescriptionDao;
	@Autowired
	private PatientHealthIndexService healthIndexService;
	@Autowired
	private FollowUpService followUpService;
	@Autowired
	private FollowUpDao followUpDao;
	@Autowired
	private ImUtil imUtil;
	@Autowired
	private JdbcTemplate jdbcTemplate;
	@Autowired
	private DevicePatientHealthIndexDao devicePatientHealthIndexDao;
	
	/**
	 * 根据续方CODE，文件类型获取数据
	 * @param prescriptionCode
	 * @param followupProject
	 * @return
	 */
	public List<PrescriptionFollowupContent> getByPrescriptionCodeAndFollowupProject(String prescriptionCode, String followupProject) throws Exception{
		
		//如果传多个类型，则返回多组数据
		if(followupProject.contains(",")){
			List<PrescriptionFollowupContent> datalist = new ArrayList<>();
			String[] projectArray = followupProject.split(",");
			
			for (String project: projectArray) {
				List<PrescriptionFollowupContent>  list = prescriptionFollowupContentDao.findByPrescriptionCodeAndFollowupProject(prescriptionCode,project);
				datalist.addAll(list);
			}
			
			return datalist;
		}else{
			return prescriptionFollowupContentDao.findByPrescriptionCodeAndFollowupProject(prescriptionCode,followupProject);
		}
	}
	
	/**
	 * 保存随访调查分类数据
	 * @param prescriptioncode
	 * @param followupProjectData
	 * @param sendIM
	 * @throws Exception
	 */
	public void saveInfo(String prescriptioncode, String followupProjectData,String patientCode,boolean sendIM) throws Exception{
		JSONArray objects = JSON.parseArray(followupProjectData);
		
		if(objects!= null & objects.size() >0){
			boolean isSendTZSHMsg = false;//是否已发送过体征和生活方式消息
			for (int i = 0; i < objects.size(); i++) {
				List<PrescriptionFollowupContent> savedatas = new ArrayList<>();
				JSONObject jsonObject = objects.getJSONObject(i);
				String followupProject = jsonObject.getString("followupProject");
				String projectValueStr = jsonObject.getString("projectData");
				if(StringUtils.isNotBlank(projectValueStr)){
					Map<String,String> projectValueMap = (Map)JSON.parse(projectValueStr);
					if(!projectValueMap.isEmpty()){
						projectValueMap.forEach( (key,value)->{
							PrescriptionFollowupContent prescriptionFollowupContent = new PrescriptionFollowupContent();
							prescriptionFollowupContent.setCode(UUID.randomUUID().toString());
							prescriptionFollowupContent.setPrescriptionCode(prescriptioncode);
							prescriptionFollowupContent.setFollowupProject(followupProject);
							prescriptionFollowupContent.setFollowupKey(key);
							prescriptionFollowupContent.setFollowupValue(value);
							prescriptionFollowupContent.setPatientCode(patientCode);
							prescriptionFollowupContent.setCreateTime(new Date());
							savedatas.add(prescriptionFollowupContent);
						});
					}
				}
				if(!savedatas.isEmpty()){
					prescriptionFollowupContentDao.deleteByPrescriptioncodeAndFollowupProject(prescriptioncode,followupProject);
					prescriptionFollowupContentDao.saveAll(savedatas);
					
					if(sendIM){
						org.json.JSONObject content = new org.json.JSONObject();
						content.put("title", "【本消息为系统提醒】：");
						if("1".equals(followupProject)){
							content.put("text", "居民已填写《身体异常症状问卷》");
							content.put("msgType", 1);
						}else{
							if(isSendTZSHMsg){//判断是否已发送过，避免重复发送
								continue;
							}
							content.put("text", "居民已填写《体征及生活方式调查问卷》");
							isSendTZSHMsg = true;
							content.put("msgType", 2);
						}
						content.put("sysMsg","填写完成并提交后，医生将可以查看您的问卷内容");
						WlyyPrescriptionDO p = prescriptionDao.findById(prescriptioncode).orElse(null);
						imUtil.sendTopicIM(p.getPatientCode(), p.getPatientName(), p.getConsult(), "17", content.toString(), null);
					}
				}
			}
			
//			if(org.apache.commons.lang3.StringUtils.isNoneBlank(prescriptioncode)){
//				try {
//					WlyyPrescriptionDO prescription = prescriptionDao.findById(prescriptioncode).orElse(null);
//					prescription.setViewWenjuan(1);
//					prescriptionDao.save(prescription);
//				}catch (Exception e){
//					logger.info(e.getMessage());
//				}
//			}
			
		}else{
			throw new ServiceException("保存的分类数据不能为空！");
		}
	}
	
	/**
	 *根据续方CODE获取随访调查问卷列表
	 * @param prescriptioncode
	 * @return
	 */
	public JSONArray getList(String prescriptioncode) throws Exception{
		PrescriptionFollowupContent prescriptionFollowupContent1 = null;
		PrescriptionFollowupContent prescriptionFollowupContent2 = null;
		
		List<PrescriptionFollowupContent>  list = prescriptionFollowupContentDao.findByPrescriptionCode(prescriptioncode);
		
		for (PrescriptionFollowupContent prescriptionFollowupContent: list) {
			if("1".equals(prescriptionFollowupContent.getFollowupProject())){
				if(prescriptionFollowupContent1 == null){
					prescriptionFollowupContent1 = prescriptionFollowupContent;
				}else{
					continue;
				}
			}
			
			if("2".equals(prescriptionFollowupContent.getFollowupProject())){
				if(prescriptionFollowupContent2 == null){
					prescriptionFollowupContent2 = prescriptionFollowupContent;
				}else{
					continue;
				}
			}
			
			if("4".equals(prescriptionFollowupContent.getFollowupProject())){
				if(prescriptionFollowupContent2 == null){
					prescriptionFollowupContent2 = prescriptionFollowupContent;
				}else{
					continue;
				}
			}
		}
		
		JSONArray result = new JSONArray();
		JSONObject jsonObject1 = new JSONObject();
		if(prescriptionFollowupContent1 != null){
			jsonObject1.put("type","1");
			jsonObject1.put("name","身体异常症状问卷");
			jsonObject1.put("statue","1");//1已填写，2未填写
			jsonObject1.put("createtime", DateUtil.dateToStrLong(prescriptionFollowupContent1.getCreateTime()));
		}else{
			jsonObject1.put("type","1");
			jsonObject1.put("name","身体异常症状问卷");
			jsonObject1.put("statue","2");//1已填写，2未填写
			jsonObject1.put("createtime","");
		}
		jsonObject1.put("typevalue","1");
		result.add(jsonObject1);
		
		JSONObject jsonObject2 = new JSONObject();
		if(prescriptionFollowupContent2 != null){
			jsonObject2.put("type","2");
			jsonObject2.put("name","体征及生活方式调查问卷");
			jsonObject2.put("statue","1");//1已填写，2未填写
			jsonObject2.put("createtime",DateUtil.dateToStrLong(prescriptionFollowupContent2.getCreateTime()));
		}else{
			jsonObject2.put("type","2");
			jsonObject2.put("name","体征及生活方式调查问卷");
			jsonObject2.put("statue","2");//1已填写，2未填写
			jsonObject2.put("createtime","");
		}
		jsonObject2.put("typevalue","2,4");
		result.add(jsonObject2);
		
		return result;
	}
	
	/**
	 * 续方药品导入随访记录
	 * @param prescriptioncode
	 * @param followup_id
	 */
	public void importPrescriptionDrugsToFollowup(String prescriptioncode, String followup_id) throws Exception {
		Long followupId = Long.valueOf(followup_id);
		Followup followup =followUpDao.findById(followupId).orElse(null);
		
		if(followup == null){
			throw new ServiceException("未找到随访记录！");
		}
		//删除原有用药记录
		followupDrugsDao.deleteByFollowupId(Long.valueOf(followupId));
		
		List<WlyyPrescriptionInfoDO> list = prescriptionInfoDao.findByPrescriptionId(prescriptioncode,1);
		
		if(list!=null && list.size()>0)
		{
			List<FollowupDrugs> newList = new ArrayList<>();
			for(WlyyPrescriptionInfoDO prescriptionInfo:list)
			{
				FollowupDrugs drug = new FollowupDrugs();
				drug.setFollowupId(followupId);
				
//				if(prescriptionInfo.getDrugName().contains("胰岛素") &&prescriptionInfo.getSubjectClass().contains("010")){
//					drug.setDrugsGroup("insulin");
//				}else{
//					drug.setDrugsGroup("normal");
//				}
				drug.setDrugsCode(prescriptionInfo.getDrugNo());
				drug.setDrugsName(prescriptionInfo.getDrugName());
				drug.setDose(prescriptionInfo.getDosage());
				drug.setUnit(prescriptionInfo.getUnit());
				drug.setFrequency(prescriptionInfo.getUsageCode());
				drug.setCreateTime(new Date());
				newList.add(drug);
			}
			followupDrugsDao.saveAll(newList);
		}
	}
	
	/**
	 * 续方患者的血糖导入随访记录
	 * @param prescriptioncode
	 * @param followup_id
	 */
	public void importPatientBloodSugarToFollowup(String prescriptioncode, String followup_id,boolean sendIM,String healthindexid) throws Exception {
		Long followupId = Long.valueOf(followup_id);
		
		Followup followup =followUpDao.findById(followupId).orElse(null);
		
		if(followup == null){
			throw new ServiceException("未找到随访记录！");
		}
		
		WlyyPrescriptionDO prescription = prescriptionDao.findById(prescriptioncode).orElse(null);
		
		String patientCode = prescription.getPatientCode();
		
		String stardate = DateUtil.getNextMonth(new Date(),-1)+" 00:00:00";
		String enddate = DateUtil.getStringDateShort()+" 23:59:59";
		
		List<Object> list = healthIndexService.findIndexByPatient2(patientCode, 1, stardate, enddate, 0, 2);
		
		String followupProjectData = "";
		
		org.json.JSONObject contentDoctor = new org.json.JSONObject();
		org.json.JSONObject contentPatient = new org.json.JSONObject();
		String imBloodSugarValue = "";
		Date imBloodSugarDate = new Date();
		
		int total = 0;
		String sql = "SELECT count(id) amount FROM wlyy_patient_device where  user='"+prescription.getPatientCode()+"' AND category_code = 1";
		List<Map<String,Object>> result =  jdbcTemplate.queryForList(sql);
		if (result != null && result.size() > 0) {
			if(result.get(0).get("amount") != null){
				Double amount = new Double(result.get(0).get("amount").toString());
				total = (result.get(0).get("amount") != null ? amount.intValue() : 0);
			}
		}
		if(total == 0){
			contentDoctor.put("deviceMsg","您未绑定血糖智能设备，请手动上传体征数据，绑定设备后系统将会自动将您的体征数据上报给医生");
		}
		
		if(StringUtils.isNotBlank(healthindexid) || !list.isEmpty()){
			
			JSONObject obj = new JSONObject();
			Integer valueType = 0;
			
			//如果有传体征ID，则查体征数据
			if(StringUtils.isNotBlank(healthindexid)){
				DevicePatientHealthIndex devicePatientHealthIndex = devicePatientHealthIndexDao.findById(new Long(healthindexid)).orElse(null);
				obj = (JSONObject)JSON.toJSON(devicePatientHealthIndex);
				valueType = obj.getInteger("value2");
				imBloodSugarValue = obj.getString("value1");
			}else{
				//如果没传体征ID，则获取最近的一条
				obj = (JSONObject)list.get(0);
				valueType = obj.getInteger("dataType");
				imBloodSugarValue = obj.getString("valuedata");
			}
			
			imBloodSugarDate = obj.getDate("recordDate");
			
			//餐前
			if(StringUtils.isNotBlank(imBloodSugarValue)){
				
				//餐前
				if(1 == valueType || 3 == valueType || 5 == valueType){
					followupProjectData = "{'BS_FPG':'"+imBloodSugarValue+"'}";
				}
				
				//参后
				if(2 == valueType || 4 == valueType || 6 == valueType){
					followupProjectData = "{'NO_BS_FPG':'"+imBloodSugarValue+"'}";
				}
				
				//睡前-随机血糖
				if(7 == valueType ){
					followupProjectData = "{'RANDOM_BLOOD_SUGAR':'"+imBloodSugarValue+"'}";
				}
			}
			
			FollowupContentESDO followupContentESDO = followUpService.esGetFollowupProjectData(followup_id,"3");
			
			if(followupContentESDO == null){
				followUpService.esSaveFollowupProjectData(followup_id, "3", followupProjectData);
			}else{
				FollowupContentESDO newfollowupContentESDO = JSON.parseObject(followupProjectData, FollowupContentESDO.class);
				if(StringUtils.isNotBlank(newfollowupContentESDO.getRANDOM_BLOOD_SUGAR())){
					followupContentESDO.setRANDOM_BLOOD_SUGAR(newfollowupContentESDO.getRANDOM_BLOOD_SUGAR());
				}
				
				if(StringUtils.isNotBlank(newfollowupContentESDO.getNO_BS_FPG())){
					followupContentESDO.setNO_BS_FPG(newfollowupContentESDO.getNO_BS_FPG());
				}
				
				if(StringUtils.isNotBlank(newfollowupContentESDO.getBS_FPG())){
					followupContentESDO.setBS_FPG(newfollowupContentESDO.getBS_FPG());
				}
				followUpService.esSaveFollowupContentESDAO(followupContentESDO);
			}
			
			//是否需要发送IM消息
			if(sendIM){
				//医生发送的消息
				contentDoctor.put("text", "请提供您最近一次的血糖检测数值及检测时间");
				contentDoctor.put("needUpload","false");
				contentDoctor.put("isSendWxTemplate",false);//是否发送咨询回复的微信模板
				contentDoctor.put("type",2);
				
				//医生发送Im消息
				imUtil.sendTopicIM(prescription.getDoctor(), prescription.getDoctorName(), prescription.getConsult(), "16", contentDoctor.toString(), null);
				
				//患者发送的消息
				contentPatient.put("title", "【本消息为系统提醒】：");
				contentPatient.put("tzMsg", "体征记录：血糖 "+imBloodSugarValue+"mmol/L");
				contentPatient.put("tzTime", "检测时间："+DateUtil.dateToStr(imBloodSugarDate,"yyyy-MM-dd HH:mm:ss"));
				contentPatient.put("tzAlert","温馨提示：本条体征为居民上传的健康体征数据，您可以");
				contentPatient.put("isSendWxTemplate",false);//是否发送咨询回复的微信模板
				contentPatient.put("needUpload","false");
				contentPatient.put("type",2);
				contentPatient.put("healthindexid",obj.getString("healthindexid"));
				
				
				//患者发送Im消息
				imUtil.sendTopicIM(prescription.getPatientCode(), prescription.getPatientName(), prescription.getConsult(), "16", contentPatient.toString(), null);
			}
		}else{
			//是否需要发送IM消息
			if(sendIM){
				//医生发送的消息
				contentDoctor.put("text", "请提供您最近一次的血糖检测数值及检测时间");
				contentDoctor.put("needUpload","true");
				contentDoctor.put("isSendWxTemplate",true);//是否发送咨询回复的微信模板
				contentDoctor.put("type",2);
				//医生发送Im消息
				imUtil.sendTopicIM(prescription.getDoctor(), prescription.getDoctorName(), prescription.getConsult(), "16", contentDoctor.toString(), null);
				
				//患者发送的消息
				contentPatient.put("title", "【本消息为系统提醒】：");
				contentPatient.put("tzMsg", "体征记录：无");
				contentPatient.put("tzTime", "检测时间：");
				contentPatient.put("tzAlert","温馨提示：居民近30天未上传血糖体征数据，已通知居民上传数据，请耐心等待");
				contentPatient.put("patientAlert","温馨提示：您近30天未上传血糖体征数据，您可以");
				contentPatient.put("needUpload","true");
				contentPatient.put("isSendWxTemplate",false);//是否发送咨询回复的微信模板
				contentPatient.put("type",2);
				
				//患者发送Im消息
				imUtil.sendTopicIM(prescription.getPatientCode(), prescription.getPatientName(), prescription.getConsult(), "16", contentPatient.toString(), null);
			}else{
				throw new ServiceException("未找到30天内的血糖记录！");
			}
		}

	}
	
	/**
	 * 续方患者的血压导入随访记录
	 * @param prescriptioncode
	 * @param followup_id
	 * @param sendIM 是否发送IM消息
	 */
	public void importPatientBloodPressureToFollowup(String prescriptioncode, String followup_id,boolean sendIM,String healthindexid) throws Exception {
		
		Long followupId = Long.valueOf(followup_id);
		
		Followup followup =followUpDao.findById(followupId).orElse(null);
		
		if(followup == null){
			throw new ServiceException("未找到随访记录！");
		}
		
		WlyyPrescriptionDO prescription = prescriptionDao.findById(prescriptioncode).orElse(null);
		
		String patientCode = prescription.getPatientCode();
		
		String stardate = DateUtil.getNextMonth(new Date(),-1)+" 00:00:00";
		String enddate = DateUtil.getStringDateShort()+" 23:59:59";
		org.json.JSONArray jsonArray = new org.json.JSONArray();
		
		if(StringUtils.isBlank(healthindexid)){
			
			jsonArray = healthIndexService.findIndexByPatient(patientCode, 2, stardate, enddate, 0, 2);
		}else{
			DevicePatientHealthIndex devicePatientHealthIndex = devicePatientHealthIndexDao.findById(new Long(healthindexid)).orElse(null);
			
			if(devicePatientHealthIndex != null){
				org.json.JSONObject json = new org.json.JSONObject();
				json.put("healthindexid", devicePatientHealthIndex.getId());
				json.put("patient",devicePatientHealthIndex.getUser());
				json.put("value1", devicePatientHealthIndex.getValue1());
				json.put("value2", devicePatientHealthIndex.getValue2());
				json.put("value3", devicePatientHealthIndex.getValue3());
				json.put("value4", devicePatientHealthIndex.getValue4());
				json.put("value5", devicePatientHealthIndex.getValue5());
				json.put("value6", devicePatientHealthIndex.getValue6());
				json.put("value7", devicePatientHealthIndex.getValue7());
				json.put("deviceSn",devicePatientHealthIndex.getDeviceSn());
				json.put("type", devicePatientHealthIndex.getType());
				
				Date date = devicePatientHealthIndex.getRecordDate();
				if (devicePatientHealthIndex.getType() == 2) {
					json.put("date", DateUtil.dateToStr(date, DateUtil.YYYY_MM_DD_HH_MM_SS));
				} else {
					json.put("date", DateUtil.dateToStr(date, DateUtil.YYYY_MM_DD));
				}
				json.put("sortDate",devicePatientHealthIndex.getSortDate());
				json.put("czrq", devicePatientHealthIndex.getCzrq());
				
				jsonArray.put(json);
			}
		}

		
		String followupProjectData = "";
		
		org.json.JSONObject contentDoctor = new org.json.JSONObject();
		org.json.JSONObject contentPatient = new org.json.JSONObject();
		
		int total = 0;
		String sql = "SELECT count(id) amount FROM wlyy_patient_device where  user='"+prescription.getPatientCode()+"' AND category_code = 2";
		List<Map<String,Object>> result =  jdbcTemplate.queryForList(sql);
		if (result != null && result.size() > 0) {
			if(result.get(0).get("amount") != null){
				Double amount = new Double(result.get(0).get("amount").toString());
				total = (result.get(0).get("amount") != null ? amount.intValue() : 0);
			}
		}
		if(total == 0){
			contentDoctor.put("deviceMsg","您未绑定血压智能设备，请手动上传体征数据，绑定设备后系统将会自动将您的体征数据上报给医生");
		}
		
		if(jsonArray.length() > 0){
			
			org.json.JSONObject jsonObject = jsonArray.getJSONObject(0);
			
			followupProjectData = "{'BP_U':'"+jsonObject.getString("value1")+"','BP_D':'"+jsonObject.getString("value2")+"'}";
			
			
			FollowupContentESDO followupContentESDO = followUpService.esGetFollowupProjectData(followup_id,"2");
			
			if(followupContentESDO == null){
				followUpService.esSaveFollowupProjectData(followup_id, "2", followupProjectData);
			}else{
				followupContentESDO.setBP_U(jsonObject.getString("value1"));
				followupContentESDO.setBP_D(jsonObject.getString("value2"));
				followUpService.esSaveFollowupContentESDAO(followupContentESDO);
			}
			
			
			//是否需要发送IM消息
			if(sendIM){
				//医生发送的消息
				contentDoctor.put("text", "请提供您最近一次的血压检测数值及检测时间");
				contentDoctor.put("needUpload","false");
				contentDoctor.put("isSendWxTemplate",false);//是否发送咨询回复的微信模板
				contentDoctor.put("type",1);
				
				//医生发送Im消息
				imUtil.sendTopicIM(prescription.getDoctor(), prescription.getDoctorName(), prescription.getConsult(), "16", contentDoctor.toString(), null);
				
				//患者发送的消息
				contentPatient.put("title", "【本消息为系统提醒】：");
				contentPatient.put("tzMsg", "体征记录：血压"+jsonObject.getString("value1")+"/"+jsonObject.getString("value2")+" mmHg");
				contentPatient.put("tzTime", "检测时间："+jsonObject.getString("date"));
				contentPatient.put("tzAlert","温馨提示：本条体征为居民上传的健康体征数据，您可以");
				contentPatient.put("needUpload","false");
				contentPatient.put("isSendWxTemplate",false);//是否发送咨询回复的微信模板
				contentPatient.put("type",1);
				contentPatient.put("healthindexid",jsonObject.get("healthindexid"));
				
				//患者发送Im消息
				imUtil.sendTopicIM(prescription.getPatientCode(), prescription.getPatientName(), prescription.getConsult(), "16", contentPatient.toString(), null);
			}
		}else{
			//是否需要发送IM消息
			if(sendIM){
				
				//医生发送的消息
				contentDoctor.put("text", "请提供您最近一次的血压检测数值及检测时间");
				contentDoctor.put("needUpload","true");
				contentDoctor.put("isSendWxTemplate",true);//是否发送咨询回复的微信模板
				contentDoctor.put("type",1);
				//医生发送Im消息
				imUtil.sendTopicIM(prescription.getDoctor(), prescription.getDoctorName(), prescription.getConsult(), "16", contentDoctor.toString(), null);
				
				//患者发送Im消息
				contentPatient.put("title", "【本消息为系统提醒】：");
				contentPatient.put("tzMsg", "体征记录：无");
				contentPatient.put("tzTime", "检测时间：");
				contentPatient.put("tzAlert","温馨提示：居民近30天未上传血压体征数据，已通知居民上传数据，请耐心等待");
				contentPatient.put("patientAlert","温馨提示：您近30天未上传血压体征数据，您可以");
				contentPatient.put("needUpload","true");
				contentPatient.put("isSendWxTemplate",false);//是否发送咨询回复的微信模板
				contentPatient.put("type",1);
				
				//患者发送Im消息
				imUtil.sendTopicIM(prescription.getPatientCode(), prescription.getPatientName(), prescription.getConsult(), "16", contentPatient.toString(), null);
			}else{
				throw new ServiceException("未找到30天内的血压记录！");
			}
		}
	
	}
	
	/**
	 * 续方问卷导入随访记录
	 * @param prescriptioncode
	 * @param followup_id
	 * @param type
	 */
	public void importfollowupcontent(String prescriptioncode, String followup_id, String type) throws Exception {
		
		Long followupId = Long.valueOf(followup_id);
		Followup followup =followUpDao.findById(followupId).orElse(null);
		
		if(followup == null){
			throw new ServiceException("未找到随访记录！");
		}
		WlyyPrescriptionDO prescription = prescriptionDao.findById(prescriptioncode).orElse(null);
		String patientCode = prescription.getPatientCode();

		if("1".equals(type)){
		//1身份异常症状问卷
			List<PrescriptionFollowupContent>  list = prescriptionFollowupContentDao.findByPrescriptionCodeAndFollowupProject(prescriptioncode,type);
			Map<String, String> datamap = new HashMap<>();
			if(!list.isEmpty()){
				for (PrescriptionFollowupContent prescriptionFollowupContent: list) {
					datamap.put(prescriptionFollowupContent.getFollowupKey(),prescriptionFollowupContent.getFollowupValue());
				}
			}
			
			followUpService.esSaveFollowupProjectData(followup_id, type, JSON.toJSONString(datamap));
		}else if("2".equals(type)){
			//2体征
			Map<String, String> datamap2 = new HashMap<>();
			
			List<PrescriptionFollowupContent>  list2 = prescriptionFollowupContentDao.findByPrescriptionCodeAndFollowupProject(prescriptioncode,"2");
			
			if(!list2.isEmpty()){
				for (PrescriptionFollowupContent prescriptionFollowupContent: list2) {
					if(StringUtils.isNotBlank(prescriptionFollowupContent.getFollowupValue())){
						datamap2.put(prescriptionFollowupContent.getFollowupKey(),prescriptionFollowupContent.getFollowupValue());
					}
				}
			}
			
			//判断随访中是否已经有体征的相关数据
			FollowupContentESDO followupContentESDO = followUpService.esGetFollowupProjectData(followup_id,"2");
			if(followupContentESDO == null){
				//如果没有,则自动获取近30天的血压
				//近30天血压
				String stardate = DateUtil.getNextMonth(new Date(),-1)+" 00:00:00";
				String enddate = DateUtil.getStringDateShort()+" 23:59:59";
				org.json.JSONArray jsonArray = healthIndexService.findIndexByPatient(patientCode, 2, stardate, enddate, 0, 2);
				if(jsonArray.length() > 0){
					org.json.JSONObject jsonObject = jsonArray.getJSONObject(0);
					datamap2.put("BP_D",jsonObject.getString("value1"));
					datamap2.put("BP_U",jsonObject.getString("value2"));
				}
			}else{
				datamap2.put("BP_D",followupContentESDO.getBP_D());
				datamap2.put("BP_U",followupContentESDO.getBP_U());
			}
			//保存随访体征数据
			if(!datamap2.isEmpty()){
				followUpService.esSaveFollowupProjectData(followup_id, "2", JSON.toJSONString(datamap2));
			}
			
			Map<String, String> datamap3 = new HashMap<>();
			List<PrescriptionFollowupContent>  list3 = prescriptionFollowupContentDao.findByPrescriptionCodeAndFollowupProject(prescriptioncode,"4");
			
			if(!list3.isEmpty()){
				for (PrescriptionFollowupContent prescriptionFollowupContent: list3) {
					if(StringUtils.isNotBlank(prescriptionFollowupContent.getFollowupValue())){
						datamap3.put(prescriptionFollowupContent.getFollowupKey(),prescriptionFollowupContent.getFollowupValue());
					}
				}
			}
			
			//保证生活方式
			if(!datamap3.isEmpty()){
				followUpService.esSaveFollowupProjectData(followup_id, "4", JSON.toJSONString(datamap3));
			}
		}else{}

	}
}
