package com.yihu.jw.afterDiagnosis.service.prescription;


import com.yihu.jw.afterDiagnosis.dao.followup.FollowUpDao;
import com.yihu.jw.afterDiagnosis.dao.prescription.PrescriptionAdjustDao;
import com.yihu.jw.afterDiagnosis.dao.prescription.PrescriptionAdjustReasonDao;
import com.yihu.jw.afterDiagnosis.dao.prescription.PrescriptionReviewedDao;
import com.yihu.jw.dict.dao.BaseDictIcd10Dao;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.org.OrgAddress;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.followup.Followup;
import com.yihu.jw.entity.hospital.consult.WlyyHospitalSysDictDO;
import com.yihu.jw.entity.hospital.prescription.*;
import com.yihu.jw.hospital.dict.WlyyHospitalSysDictDao;
import com.yihu.jw.hospital.message.dao.SystemMessageDao;
import com.yihu.jw.hospital.prescription.dao.*;
import com.yihu.jw.im.util.ImUtil;
import com.yihu.jw.org.dao.OrgAddressDao;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.util.common.CommonUtil;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.entity.ServiceException;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONArray;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created  Trick on 2017/7/25..
 */
@Service
@Transactional
public class PrescriptionInfoService {
    @Autowired
    private PrescriptionInfoDao prescriptionInfoDao;
    @Autowired
    private WlyyHospitalSysDictDao hospitalSysDictDao;
    @Autowired
    private PrescriptionDao prescriptionDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private BaseDoctorDao doctorDao;
    @Autowired
    private BasePatientDao patientDao;
    @Autowired
    private PrescriptionLogDao prescriptionLogDao;
    @Autowired
    private PrescriptionDiagnosisService prescriptionDiagnosisService;
    @Autowired
    private PrescriptionAdjustDao prescriptionAdjustDao;
    @Autowired
    private PrescriptionReviewedDao prescriptionReviewedDao;
    @Autowired
    private PrescriptionDiagnosisDao prescriptionDiagnosisDao;
    @Autowired
    private PrescriptionNoticesService prescriptionNoticesService;
    @Autowired
    private ImUtil imUtill;
    @Autowired
    private SystemMessageDao messageDao;
    @Autowired
    private PrescriptionExpressageDao prescriptionExpressageDao;
    @Autowired
    private BaseDictIcd10Dao dictIcd10Dao;
    @Autowired
    private StringRedisTemplate redisTemplate;
    //健康问题 高血压
    private static final String gxy = "HP0093";
    //健康问题 糖尿病
    private static final String tnb = "HP0047";
    //健康问题 糖尿病
    private static final String other = "other";
    @Autowired
    private WlyyPrescriptionExpressageService expressageService;
    @Autowired
    private WlyyPrescriptionService prescriptionService;
    @Autowired
    private PrescriptionAdjustReasonDao prescriptionAdjustReasonDao;
    @Autowired
    private FollowUpDao followUpDao;
    @Autowired
    private OrgAddressDao orgAddressDao;
    @Autowired
    private PrescriptionAdjustService prescriptionAdjustService;

    private static final Logger logger = LoggerFactory.getLogger(PrescriptionInfoService.class);

    /**
     * 获取处方下的详细药品列表
     *
     * @param prescriptionCode
     * @return
     */
    public List<WlyyPrescriptionInfoDO> getPrescriptionInfo(String prescriptionCode) {
        return prescriptionInfoDao.findByPrescriptionId(prescriptionCode,1);
    }


    public JSONObject getPrescriptionFilter(Integer teamCode,String patient,String doctorCode) {
        JSONObject rs = new JSONObject();
        List<WlyyHospitalSysDictDO> sdict = hospitalSysDictDao.findByDictName("PRESCRIPTION_STATE");
        List<Map<String,Object>> states = new ArrayList<>();
        for(WlyyHospitalSysDictDO sd : sdict){
            Map<String,Object> m = new HashedMap();
            m.put("code",sd.getDictCode());
            m.put("value",sd.getDictValue());
            m.put("count",getPresCountByState(teamCode,sd.getDictCode(),patient,doctorCode));
            states.add(m);
        }
        rs.put("states", states);
        List<Map<String, Object>> diseases = new ArrayList<>();
        Map<String, Object> dis1 = new HashMap<>();
        dis1.put("code", gxy);
        dis1.put("name", "高血压");
        Map<String, Object> dis2 = new HashMap<>();
        dis2.put("code", tnb);
        dis2.put("name", "糖尿病");
        Map<String, Object> dis3 = new HashMap<>();
        dis3.put("code", other);
        dis3.put("name", "其他");
        diseases.add(dis1);
        diseases.add(dis2);
        diseases.add(dis3);
        rs.put("diseases", diseases);

        return rs;
    }

    public int getPresCountByState(Integer teamCode,String states,String patient,String doctorCode){
        String sql = "SELECT " +
                " COUNT(1) AS count " +
                " FROM " +
                " wlyy_prescription p " +
                " WHERE " +
                " p.`status` in ("+states+")";
        if(StringUtils.isNotBlank(patient)){
            sql = sql+ " AND p.patient = '"+patient+"'";
        }
        if(StringUtils.isNotBlank(doctorCode)){
            sql = sql+ " AND p.doctor = '"+doctorCode+"'";
        }
        if(teamCode!=null){
            sql = sql+ " AND p.admin_team_id = "+teamCode;
        }
        List<Map<String,Object>> rs = jdbcTemplate.queryForList(sql);
        if(rs!=null&&rs.size()>0){
            Long r = (Long) rs.get(0).get("count");
            return r.intValue();
        }
        return 0;
    }

    /**
     * 1:查询处方;2我的续方;3:续方记录
     *
     * @param type
     * @return
     */
    public com.alibaba.fastjson.JSONArray getPrescriptionInfos(String type,String recipeNo, String diagnosisCode, String startDate, String endDate, String patient, String isRenewal, Integer page, Integer size,String hospital) throws Exception {

        if (StringUtils.isNotBlank(type)) {
            if ("1".equals(type)) {
                BasePatientDO p = patientDao.findById(patient).orElse(null);

//                String rp = jwPrescriptionService.getLastRecipe(p.getSsc(), recipeNo, startDate, endDate,hospital);
//                com.alibaba.fastjson.JSONArray pres = presModeAdapter.modeToPrescription(rp);
                com.alibaba.fastjson.JSONArray pres = new com.alibaba.fastjson.JSONArray();
                if(pres==null||pres.size()==0){
                    return pres;
                }

                for (int i = 0; i < pres.size(); i++) {
                    com.alibaba.fastjson.JSONObject r = (com.alibaba.fastjson.JSONObject) pres.get(i);
                    //判断是否在审核中
                    String rState = presCheckStateObj(r.getString("code"));
                    if ("1".equals(rState)) {
                        r.put("reviewedState", 1);
                        r.put("patientName", p.getName());
                        r.put("patient", p.getId());
                        r.put("prescriptionCode", "");
                    } else {
                        r.put("reviewedState", 0);
                        r.put("prescriptionCode", rState);
                        r.put("patientName", p.getName());
                        r.put("patient", p.getId());
                    }

                }
                //过滤可续签
                if (StringUtils.isNotBlank(isRenewal) && "1".equals(isRenewal)) {
                    Iterator iterator = pres.iterator();
                    while (iterator.hasNext()) {
                        com.alibaba.fastjson.JSONObject r = (com.alibaba.fastjson.JSONObject) iterator.next();
                        //判断状态是否符合
                        String reviewedState = r.getString("reviewedState");
                        if ("0".equals(reviewedState)) {
                            iterator.remove();
                            continue;
                        }

                        com.alibaba.fastjson.JSONArray des =  r.getJSONArray("prescriptionDt");

                        for (int i = 0; i < des.size(); i++) {
                            WlyyPrescriptionDiagnosisDO ds = des.getObject(i, WlyyPrescriptionDiagnosisDO.class);
                            if (gxy.equals(ds.getCode()) || tnb.equals(ds.getCode())) {
                                //为糖尿病高血压
                            } else {
                                //如果含有非糖尿病或高血压
                                iterator.remove();
                                continue;
                            }
                        }
                    }
                }

                //过滤病症
                if(StringUtils.isNotBlank(diagnosisCode)&&!"0".equals(diagnosisCode)){
                    Iterator iterator = pres.iterator();
                    while (iterator.hasNext()) {
                        com.alibaba.fastjson.JSONObject r = (com.alibaba.fastjson.JSONObject) iterator.next();
                        if("3".equals(diagnosisCode)){
                            com.alibaba.fastjson.JSONArray des = r.getJSONArray("prescriptionDt");
                            boolean flag = false;
                            for (int i = 0; i < des.size(); i++) {
                                WlyyPrescriptionDiagnosisDO ds = des.getObject(i, WlyyPrescriptionDiagnosisDO.class);
                                //如果含有选择病症，标记为true
                                if (gxy.equals(ds.getCode()) || tnb.equals(ds.getCode())) {
                                    //含有选择的病症
                                    flag = true;
                                }
                            }
                            //如果未含有选择的病症，则删除
                            if (!flag){
                                iterator.remove();
                            }
                        }else{
                            String dis = gxy;
                            if("1".equals(diagnosisCode)){
                                dis = gxy;
                            }else if("2".equals(diagnosisCode)){
                                dis = tnb;
                            }
    
                            com.alibaba.fastjson.JSONArray des = r.getJSONArray("prescriptionDt");
                            boolean flag = false;
                            for (int i = 0; i < des.size(); i++) {
                                WlyyPrescriptionDiagnosisDO ds = des.getObject(i, WlyyPrescriptionDiagnosisDO.class);
                                //如果含有选择病症，标记为true
                                if (dis.equals(ds.getCode())) {
                                    //含有选择的病症
                                    flag = true;
                                }
                            }
                            //如果未含有选择的病症，则删除
                            if (!flag){
                                iterator.remove();
                            }
                        }
                    }
                }

                return pres;
            } else {
                BasePatientDO p = patientDao.findById(patient).orElse(null);
                StringBuffer sqlBuffer = new StringBuffer("SELECT p.id,LEFT(p.create_time,19) AS createTime,p.status,p.hospital_name As hospitalName FROM wlyy_prescription p " +
                        "WHERE p.patient=?");
                List<Object> params = new ArrayList<>();
                params.add(patient);
                //查询已经审核通过的,否则查询所有
                if ("2".equals(type)) {
                    sqlBuffer.append(" AND p.status >="+ WlyyPrescriptionLogDO.PrescriptionLogStatus.wait_pay.getValue());
                }
                if (StringUtils.isNotBlank(startDate)) {
                    sqlBuffer.append(" AND p.create_time >= ?");
                    params.add(startDate + " 00:00:00");
                }
                if (StringUtils.isNotBlank(endDate)) {
                    sqlBuffer.append(" AND p.create_time <= ?");
                    params.add(endDate + " 23:59:59");
                }
                sqlBuffer.append(" ORDER BY p.create_time DESC ");
                if (page != null && page > 0 && size != null && size > 0) {
                    sqlBuffer.append(" LIMIT " + (page - 1) * size + "," + size);
                }
                //本地库
                List<Map<String, Object>> rs = jdbcTemplate.queryForList(sqlBuffer.toString(), params.toArray());
                //通过缓存查找药品和疾病
                for (Map<String, Object> map : rs) {
                    String code = (String) map.get("code");
                    map.put("patient", p.getId());
                    map.put("patientName", p.getName());

                    //判断是否已经审核
                    if(isAdjustPres(code)){
                        map.put("isAdjust", "1");
                    }else{
                        map.put("isAdjust", "0");
                    }
                    map.put("prescriptionInfo", prescriptionDiagnosisService.getPrescriptionInfo(code));
                    map.put("prescriptionDt", prescriptionDiagnosisService.getPrescriptionDiagnosis(code));
                }
                return com.alibaba.fastjson.JSONArray.parseArray(new JSONArray(rs).toString());
            }
        }
        return new com.alibaba.fastjson.JSONArray();
    }

    /**
     * 获取处方列表
     * @param patientCode
     * @param applyTimeFrom
     * @param applyTimeEnd
     * @param pageNo
     * @param pageSize
     * @return
     * @throws Exception
     */
    public com.alibaba.fastjson.JSONObject getRecipeMasterList(String patientCode,String applyTimeFrom,String applyTimeEnd,
               String isRenewal,String diagnosisCode,Integer pageNo,Integer pageSize,String hospital) throws Exception{
        BasePatientDO patient = patientDao.findById(patientCode).orElse(null);
        if(pageNo==null||pageNo<=0){
            pageNo = 0;
        }
        pageNo++;
        if(pageSize==null||pageSize<=0){
            pageSize = 5;
        }
        Integer start = (pageNo-1)*pageSize;
        Integer end = pageNo*pageSize;
        Integer totalCount = 0;
        Integer newtotalCount = 0;
        com.alibaba.fastjson.JSONObject returnJson = new com.alibaba.fastjson.JSONObject();
        com.alibaba.fastjson.JSONArray filterJsonArray = new com.alibaba.fastjson.JSONArray();
    
        if(StringUtils.isEmpty(applyTimeFrom)){
    
            Date now = new Date();
            //如果是2月29号则往前推1天, 2月28号往前推两天，
            SimpleDateFormat formatter = new SimpleDateFormat("MM-dd");
            String dateString = formatter.format(now);
            applyTimeEnd = DateUtil.getStringDateShort();
    
            Calendar cend = Calendar.getInstance();
            cend.add(Calendar.DAY_OF_YEAR, -84);
            applyTimeFrom = DateUtil.dateToStrShort(cend.getTime());
        }
        
        //判断时间是否小于 2019-12-27, 如果小于，则默认2019-12-27
        try {
            if(StringUtils.isNoneBlank(applyTimeFrom)){
                String days = DateUtil.getTwoDay("2019-12-27",applyTimeFrom);
                Integer day = Integer.parseInt(days);
                if(day>0){
                    applyTimeFrom = "2019-12-27";
                }
            }
        }catch (Exception e){
            logger.info(e.getMessage());
        }

        
//        com.alibaba.fastjson.JSONObject re = jwPrescriptionService.getRecipeMasterList(patientCode,applyTimeFrom,applyTimeEnd,hospital);
        com.alibaba.fastjson.JSONObject re = new com.alibaba.fastjson.JSONObject();
        totalCount = re.getInteger("totalCount");
        if(totalCount>start){
            com.alibaba.fastjson.JSONArray pres = re.getJSONArray("list");
            for (int i = 0; i < pres.size(); i++) {
                com.alibaba.fastjson.JSONObject r = (com.alibaba.fastjson.JSONObject) pres.get(i);
                String rState = presCheckStateObj(r.getString("code"));
                r.put("patientName", patient.getName());
                r.put("patient", patient.getId());
                if ("1".equals(rState)) {
                    r.put("reviewedState", 1);
                    r.put("prescriptionCode", "");
                } else {
                    r.put("reviewedState", 0);
                    r.put("prescriptionCode", rState);
                }
                com.alibaba.fastjson.JSONArray des = r.getJSONArray("prescriptionDt");
                //过滤可续签
                if (StringUtils.isNotBlank(isRenewal) && "1".equals(isRenewal)) {
                    //判断状态是否符合
                    String reviewedState = r.getString("reviewedState");
                    if ("0".equals(reviewedState)) {
                        continue;
                    }
                    boolean flag = false;
                    for (int j = 0; j < des.size(); j++) {
                        WlyyPrescriptionDiagnosisDO ds = des.getObject(j, WlyyPrescriptionDiagnosisDO.class);
                        if (gxy.equals(ds.getCode()) || tnb.equals(ds.getCode())) {
                            //为糖尿病高血压
                        } else {
                            //如果含有非糖尿病或高血压
                            flag = true;
                        }
                    }
                    //如果未含有选择的病症，则删除
                    if (flag){
                        continue;
                    }
                }
                
                //过滤病症
                if(StringUtils.isNotBlank(diagnosisCode)&&!"0".equals(diagnosisCode)){
                    boolean flag = false;
                    //高血压或者糖尿病
                    if("3".equals(diagnosisCode)){
                        for (int j = 0; j < des.size(); j++) {
                            WlyyPrescriptionDiagnosisDO ds = des.getObject(j, WlyyPrescriptionDiagnosisDO.class);
                            //如果含有选择病症，标记为true
                            if (gxy.equals(ds.getCode()) || tnb.equals(ds.getCode())) {
                                //含有选择的病症
                                flag = true;
                            }
                        }
                        //如果未含有选择的病症，则删除
                        if (!flag){
                            continue;
                        }
                    }else{
                        String dis = gxy;
                        if("1".equals(diagnosisCode)){
                            //高血压
                            dis = gxy;
                        }else if("2".equals(diagnosisCode)){
                            //糖尿病
                            dis = tnb;
                        }

                        for (int j = 0; j < des.size(); j++) {
                            WlyyPrescriptionDiagnosisDO ds = des.getObject(j, WlyyPrescriptionDiagnosisDO.class);
                            //如果含有选择病症，标记为true
                            if (dis.equals(ds.getCode())) {
                                //含有选择的病症
                                flag = true;
                            }
                        }
                        //如果未含有选择的病症，则删除
                        if (!flag){
                            continue;
                        }
                    }
                }
                newtotalCount++;
                filterJsonArray.add(r);
            }
        }
        int limit = newtotalCount>end?end:newtotalCount;
        //分页处理
        com.alibaba.fastjson.JSONArray returnJsonArray = new com.alibaba.fastjson.JSONArray();
        if(newtotalCount>start){
            for (int i = start; i < limit; i++) {
                com.alibaba.fastjson.JSONObject json = filterJsonArray.getJSONObject(i);
                returnJsonArray.add(json);
            }
        }
        returnJson.put("total",newtotalCount);
        returnJson.put("list",returnJsonArray);
        return returnJson;
    }

    /**
     *
     * @param code
     * @return
     */
    public boolean isAdjustPres(String code){
        StringBuffer isAdjustSql = new StringBuffer("SELECT a.id FROM wlyy_prescription_adjust a WHERE a.prescription_code = ? ");
        List<Map<String,Object>> adjusts = jdbcTemplate.queryForList(isAdjustSql.toString(),new Object[]{code});
        if(adjusts!=null&&adjusts.size()>0){
            return true ;
        }
        return false;
    }

    /**
     * 获取续方信息
     *
     * @param code
     * @return
     */
    public com.alibaba.fastjson.JSONObject getContinuedPrescription(String code) {
        WlyyPrescriptionDO prescription = prescriptionDao.findById(code).orElse(null);
        List<WlyyPrescriptionInfoDO> prescriptionInfos = prescriptionInfoDao.findByPrescriptionId(code,1);
        WlyyPrescriptionReviewedDO reviewed = prescriptionReviewedDao.findByPrescriptionCode(code);
        List<WlyyPrescriptionDiagnosisDO> diagnosises = prescriptionDiagnosisDao.findByPrescriptionId(code,1);
        com.alibaba.fastjson.JSONObject rs = new com.alibaba.fastjson.JSONObject();
        rs.put("prescription", prescription);
        rs.put("prescriptionInfo", prescriptionInfos);
        rs.put("reviewed", reviewed);
        rs.put("prescriptionDt", diagnosises);
        if (prescription != null && StringUtils.isNotBlank(prescription.getDoctor())) {
            BaseDoctorDO doctor = doctorDao.findByIdAndDel(prescription.getDoctor());
            if (doctor.getCertifiedOvertime() != null && (new Date().before(doctor.getCertifiedOvertime()))) {
                rs.put("isCertified", true);
                rs.put("doctorName", doctor.getName());
            } else {
                rs.put("isCertified", false);
                rs.put("doctorName", doctor.getName());
            }
        } else {
            rs.put("isCertified", false);
            rs.put("doctorName", "");
        }
        return rs;
    }

    /**
     * 获取处方详情
     *
     * @param code
     * @return
     */
    public com.alibaba.fastjson.JSONObject getPrescription(String code,String patient,String preCode,String hospital) {
        BasePatientDO p = patientDao.findById(patient).orElse(null);
//        String rs = jwPrescriptionService.getRecipe(code, p.getSsc(),preCode,hospital);
//        com.alibaba.fastjson.JSONObject r = presModeAdapter.modelToSinglePrescription(rs);
        com.alibaba.fastjson.JSONObject r = new com.alibaba.fastjson.JSONObject();

        String rState = presCheckStateObj(code);
        if ("1".equals(rState)) {
            r.put("reviewedState", 1);
            r.put("prescriptionCode", "");
        } else {
            r.put("reviewedState", 0);
            r.put("prescriptionCode", rState);
            
            Followup followup = followUpDao.getFollowupByPrescriptionCode(rState);
            r.put("followup",followup);
        }
        

        return r;
    }

    public com.alibaba.fastjson.JSONObject getPrescriptionProcess(String code) throws Exception{
        WlyyPrescriptionDO prescription = prescriptionDao.findById(code).orElse(null);
        List<WlyyPrescriptionInfoDO> prescriptionInfos = prescriptionInfoDao.findByPrescriptionId(code,1);
        WlyyPrescriptionReviewedDO reviewed = prescriptionReviewedDao.findByPrescriptionCode(code);
        List<PrescriptionAdjust> prescriptionAdjusts = prescriptionAdjustDao.findByPrescriptionCode(code);
        List<WlyyPrescriptionDiagnosisDO> diagnosises = prescriptionDiagnosisDao.findByPrescriptionId(code,1);

        com.alibaba.fastjson.JSONObject rs = new com.alibaba.fastjson.JSONObject();
        if (prescription != null && StringUtils.isNotBlank(prescription.getPatientCode())) {
            BasePatientDO p = patientDao.findById(prescription.getPatientCode()).orElse(null);
            Map<String, Object> pp = new HashedMap();

            pp.put("code", p.getId());
            pp.put("name", p.getName());
            pp.put("idcard", p.getIdcard());
            pp.put("ssc", p.getSsc());
            pp.put("mobile", p.getMobile());

            rs.put("patient", pp);
//            Map<String, Object> sf = new HashedMap();
//            SignFamily signFamily = signFamilyDao.findByIdcard(p.getIdcard());
//            if(signFamily!=null){
//                sf.put("doctor", signFamily.getDoctor());
//                sf.put("hospital", signFamily.getHospital());
//                sf.put("doctorName", signFamily.getDoctorName());
//                sf.put("hospitalName", signFamily.getHospitalName());
//            }else{
//                sf.put("doctor", prescription.getDoctor());
//                sf.put("hospital", prescription.getHospital());
//                sf.put("doctorName", prescription.getDoctorName());
//                sf.put("hospitalName", prescription.getHospitalName());
//            }
//            rs.put("signFamily", sf);
        } else {
            rs.put("patient", "");
            rs.put("signFamily", "");
        }


        if (prescription!=null&&reviewed != null && prescription.getStatus() == WlyyPrescriptionLogDO.PrescriptionLogStatus.wait_pay.getValue()&&reviewed.getReviewedTime()!=null) {
            Long s = (new Date().getTime()-reviewed.getReviewedTime().getTime()) / 1000;
            //rs.put("time",s);
            if (s > 86400) {
                prescription.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.pay_outtime.getValue());
                prescriptionDao.save(prescription);
            }
        } else {
            //rs.put("time","");
        }

        if(prescription.getStatus()==WlyyPrescriptionLogDO.PrescriptionLogStatus.wait_pay.getValue()){
            com.alibaba.fastjson.JSONObject object = new com.alibaba.fastjson.JSONObject();
            object = prescriptionService.checkRecipeNoPay(2,code);
            String status = object.getString("status");
            if("200".equals(status)){
                //-1操作失败 0新开 1待执行 5已执行 9作废 404没有找到对应的数据.
                String jwCode = object.getString("code");
                if("9".equals(jwCode)){
                    prescription.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.pay_cancle.getValue());
                }
                rs.put("jwStatus",object.getString("code"));
            }else{
            }
        }

        if (prescriptionAdjusts != null && prescriptionAdjusts.size() > 0) {
            rs.put("isAdjust", true);
        } else {
            rs.put("isAdjust", false);
        }
        rs.put("reviewed", reviewed);
        rs.put("prescription", prescription);
        rs.put("prescriptionInfo", prescriptionInfos);
        rs.put("prescriptionDt", diagnosises);
        WlyyPrescriptionExpressageDO prescriptionExpressage = prescriptionExpressageDao.findByPrescriptionCode(prescription.getId());
        rs.put("prescriptionExpress", prescriptionExpressage);
        return rs;
    }

    /**
     * 取消续方
     *
     * @param code
     * @return
     */
    public int cancelPrescriotion(String code, String reason) throws Exception {
        WlyyPrescriptionDO p = prescriptionDao.findById(code).orElse(null);
        p.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.patient_canel.getValue());
        p.setCancelReason(reason);
        prescriptionDao.save(p);
        //患者操作日志
        WlyyPrescriptionLogDO log = new WlyyPrescriptionLogDO();
        log.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.patient_canel.getValue());
        log.setCreateTime(new Date());
        log.setPrescriptionCode(p.getId());
        log.setUserCode(p.getPatientCode());
        log.setUserName(p.getPatientName());
        log.setUserType(1);
        log.setDatajson("患者取消");
        prescriptionLogDao.save(log);

        //修改系统的续方消息的审核状态
        messageDao.updatePreScriptionMessage(p.getConsult(), "3", 6);

        //同步智业接口
//        jwPrescriptionService.fadeRecipe(code);
    
        //取消长处方关联的随访记录
        canclePrescriptionFollowup(code);

        return 1;
    }

    public int upLoadPrescriotionState(String state, WlyyPrescriptionDO p, WlyyPrescriptionReviewedDO reviewed, String deptCode) throws Exception {
        //医生审核通过，发起开方及预结算
        if ("1".equals(state)) {
//            String rs = jwPrescriptionService.saveRecipe(p.getCode(),p.getJwGisterTypeCode(),p.getJwDeptCode(),deptCode);
 			String rs = "{\"msg\":\"请求成功！\",\"status\":200,\"data\":{\"CODE\":\"1\",\"returnData\":[[{\"VISIT_NO\":\"3277875\",\"RECIPE_NO\":\"5477158\"}]],\"byRefParaData\":[{\"errorMsg\":\"\"}]}}";
            com.alibaba.fastjson.JSONObject respone = com.alibaba.fastjson.JSONObject.parseObject(rs);
            int st = respone.getInteger("status");
//            int st = 200;
            com.alibaba.fastjson.JSONObject data = respone.getJSONObject("data");
            String code = data.getString("CODE");
//            String code = respone.getString("CODE");
            com.alibaba.fastjson.JSONArray byRefParaData = data.getJSONArray("byRefParaData");
            if (st == 200 && "1".equals(code) && data.getJSONArray("returnData").size() > 0) {
//            if (st == 200 && "1".equals(code) && respone.getJSONArray("returnData").size() > 0) {
                //开方成功返回遗嘱号
                com.alibaba.fastjson.JSONArray returnDatas = data.getJSONArray("returnData");
                com.alibaba.fastjson.JSONArray reData = (com.alibaba.fastjson.JSONArray) returnDatas.get(0);
                com.alibaba.fastjson.JSONObject returnData = (com.alibaba.fastjson.JSONObject) reData.get(0);
                p.setAdmNo(returnData.getString("VISIT_NO"));
                p.setSerialNo(returnData.getString("RECIPE_NO"));
                //设置状态为待支付
                p.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.wait_pay.getValue());
                //设置审核通过
                reviewed.setStatus(1);
                reviewed.setReviewedTime(new Date());
                prescriptionDao.save(p);

                //审核通过模板消息
                if(!isAdjustPres(p.getId())){
                    sendRMess(p.getId(), 1);
                }else{
                    //发送调整后处方模板并审核通过消息
                    sendRMess(p.getId(), 2);
                }


                //修改系统的续方消息的审核状态
                messageDao.updatePreScriptionMessage(p.getConsult(), "1", 6);

                //获取智业待结算接口，更新药品金额
                prescriptionService.getPerscriptionInfoCostFromPayInfo(p.getId());

                //发送Im消息
                JSONObject content = new JSONObject();
                content.put("title", "我已经审核您" + DateUtil.dateToStr(p.getCreateTime(), "yyyy-MM-dd") + "发起的续方申请");
                content.put("result", "审核通过");
                content.put("reason", "");
                content.put("url", "");
                imUtill.sendTopicIM(p.getDoctor(), p.getDoctorName(), p.getConsult(), "15", content.toString(), null);

                return 1;
            } else {
                p.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.add_error.getValue());
                p.setPrescribeReason(byRefParaData.getJSONObject(0).getString("errorMsg"));
                p.setPrescribeTime(new Date());
                prescriptionDao.save(p);


                return 0;
            }
        }
        return 1;
    }

    public List<Map<String, Object>> getDoctorPrescription(Integer teamCode,String doctorCode, String state, String diseases, String startDate, String endDate, String nameKeyword, String patient, Integer page, Integer size) {
        StringBuffer pre_sql = new StringBuffer("SELECT " +
                " p.`name`, " +
                " TIMESTAMPDIFF(YEAR,p.birthday,SYSDATE()) age, " +
                " p.sex, " +
                " p.photo, " +
                " pr.`status`, " +
                " pr.`code`, " +
                " LEFT(pr.create_time,19) AS createTime, " +
                " pr.doctor, " +
                " pr.min_drug_day AS minDrugDay," +
                " LEFT(pr.pres_create_time,19) AS presCreateTime " +
                " FROM " +
                " wlyy_prescription pr " +
                " LEFT JOIN base_patient p ON pr.patient = p.id ");
        List<Object> params = new ArrayList<>();
        //先判断疾病类型是否为空
        if (StringUtils.isNotBlank(diseases)) {
            if("other".equals(diseases)){
                //疾病类型不为空,关联查询疾病类型表
                pre_sql.append(" ,wlyy_prescription_diagnosis s " +
                        " WHERE s.prescription_code = pr.code " +
                        " AND s.health_problem !='HP0093' and s.health_problem !='HP0047' ");
                setSQL(pre_sql, params, teamCode,doctorCode, state, startDate, endDate, nameKeyword, patient, page, size);
            }else{
                //疾病类型不为空,关联查询疾病类型表
                pre_sql.append(" ,wlyy_prescription_diagnosis s " +
                        " WHERE s.prescription_code = pr.code " +
                        " AND s.health_problem = ?");
                params.add(diseases);
                setSQL(pre_sql, params, teamCode,doctorCode, state, startDate, endDate, nameKeyword, patient, page, size);
            }

        } else {
            //查询所有疾病类型表
            pre_sql.append(" WHERE 1=1 ");
            setSQL(pre_sql, params, teamCode,doctorCode, state, startDate, endDate, nameKeyword, patient, page, size);
        }
        List<Map<String, Object>> rs = jdbcTemplate.queryForList(pre_sql.toString(), params.toArray());

        //通过缓存查找药品和疾病
        for (Map<String, Object> map : rs) {
            String code = (String) map.get("code");
            map.put("prescriptionInfo", prescriptionDiagnosisService.getPrescriptionInfo(code));
            map.put("prescriptionDt", prescriptionDiagnosisService.getPrescriptionDiagnosis(code));
        }
        return rs;
    }

    public void setSQL(StringBuffer pre_sql, List<Object> params, Integer teamCode,String doctorCode, String state, String startDate, String endDate, String nameKeyword, String patient, Integer page, Integer size) {
        if (teamCode != null && teamCode > 0) {
            pre_sql.append(" AND pr.admin_team_id =?");
            params.add(teamCode);
        }
        if (StringUtils.isNotBlank(doctorCode)) {
            pre_sql.append(" AND pr.doctor =?");
            params.add(doctorCode);
        }
        if (StringUtils.isNotBlank(state)) {
            pre_sql.append(" AND pr.status IN (" + state + ") ");
        }
        if (StringUtils.isNotBlank(startDate)) {
            pre_sql.append(" AND pr.create_time >= ? ");
            params.add(startDate+" 00:00:00");
        }
        if (StringUtils.isNotBlank(endDate)) {
            pre_sql.append(" AND pr.create_time <= ? ");
            params.add(endDate+" 23:59:59");
        }
        if (StringUtils.isNotBlank(nameKeyword)) {
            pre_sql.append(" AND pr.patient_name like ? ");
            params.add("%" + nameKeyword + "%");
        }
        if (StringUtils.isNotBlank(patient)) {
            pre_sql.append(" AND pr.patient = ?");
            params.add(patient);
        }
        //排序
        if(StringUtils.isNotBlank(state)&&"0,2,3,4,10".equals(state)){
            pre_sql.append(" GROUP BY pr.code ORDER BY pr.min_drug_day-TIMESTAMPDIFF(DAY, pr.pres_create_time, NOW()) ASC,pr.create_time DESC");
        }else{
            pre_sql.append(" GROUP BY pr.code ORDER BY pr.create_time DESC");
        }

        if (page != null && page > 0 && size != null && size > 0) {
            pre_sql.append(" LIMIT " + (page - 1) * size + "," + size);
        }
    }

    public void setTotalSQL(StringBuffer pre_sql, List<Object> params, Integer teamCode, String state, String startDate, String endDate, String nameKeyword, String patient) {
        if (teamCode != null && teamCode > 0) {
            pre_sql.append(" AND pr.admin_team_id =?");
        }
        params.add(teamCode);

        if (StringUtils.isNotBlank(state)) {
            pre_sql.append(" AND pr.status IN (" + state + ") ");
        }
        if (StringUtils.isNotBlank(startDate)) {
            pre_sql.append(" AND pr.create_time >= ? ");
            params.add(startDate+" 00:00:00");
        }
        if (StringUtils.isNotBlank(endDate)) {
            pre_sql.append(" AND pr.create_time <= ? ");
            params.add(endDate+" 23:59:59");
        }
        if (StringUtils.isNotBlank(nameKeyword)) {
            pre_sql.append(" AND pr.patient_name like ? ");
            params.add("%" + nameKeyword + "%");
        }
        if (StringUtils.isNotBlank(patient)) {
            pre_sql.append(" AND pr.patient = ?");
            params.add(patient);
        }
    }

    /**
     * 获取处方详情总数
     * @param teamCode
     * @param state
     * @param diseases
     * @param startDate
     * @param endDate
     * @param nameKeyword
     * @param patient
     * @return
     */
    public JSONObject getDoctorPrescriptionTotal(Integer teamCode, String state, String diseases, String startDate, String endDate, String nameKeyword, String patient) {
        StringBuffer pre_sql = new StringBuffer("SELECT count(1) AS total" +
                " FROM " +
                " wlyy_prescription pr " +
                " LEFT JOIN base_patient p ON pr.patient = p.id ");
        List<Object> params = new ArrayList<>();
        //先判断疾病类型是否为空
        if (StringUtils.isNotBlank(diseases)) {
    
            if("other".equals(diseases)){
                //疾病类型不为空,关联查询疾病类型表
                pre_sql.append(" ,wlyy_prescription_diagnosis s " +
                        " WHERE s.prescription_code = pr.code " +
                        " AND s.health_problem !='HP0093' AND s.health_problem !='HP0047' ");
                setTotalSQL(pre_sql, params, teamCode, state, startDate, endDate, nameKeyword, patient);
            }else{
                //疾病类型不为空,关联查询疾病类型表
                pre_sql.append(" ,wlyy_prescription_diagnosis s " +
                        " WHERE s.prescription_code = pr.code " +
                        " AND s.health_problem = ?");
                params.add(diseases);
                setTotalSQL(pre_sql, params, teamCode, state, startDate, endDate, nameKeyword, patient);
            }
        } else {
            //查询所有疾病类型表
            pre_sql.append(" WHERE 1=1 ");
            setTotalSQL(pre_sql, params, teamCode, state, startDate, endDate, nameKeyword, patient);
        }
        List<Map<String, Object>> rs = jdbcTemplate.queryForList(pre_sql.toString(), params.toArray());
        JSONObject jsonRs = new JSONObject();
        if(rs!=null && rs.size()>0){
            jsonRs.put("total",rs.get(0).get("total"));
        }else{
            jsonRs.put("total",0);
        }
        return jsonRs;
    }


    /**
     * 获取续方信息
     *
     * @param code
     * @return
     */
    public com.alibaba.fastjson.JSONObject getContinuedPrescriptionAsDoctor(String code) {
        WlyyPrescriptionDO prescription = prescriptionDao.findById(code).orElse(null);
        List<WlyyPrescriptionInfoDO> prescriptionInfos = prescriptionInfoDao.findByPrescriptionId(code,1);
        WlyyPrescriptionReviewedDO reviewed = prescriptionReviewedDao.findByPrescriptionCode(code);
        List<PrescriptionAdjust> prescriptionAdjusts = prescriptionAdjustDao.findByPrescriptionCodeOrderByIdDesc(code);
        com.alibaba.fastjson.JSONObject rs = new com.alibaba.fastjson.JSONObject();
        rs.put("prescription", prescription);
        rs.put("prescriptionInfo", prescriptionInfos);
        rs.put("reviewed", reviewed);

        if (prescription != null && StringUtils.isNotBlank(prescription.getPatientCode())) {
            rs.put("patient", patientDao.findById(prescription.getPatientCode()).orElse(null));
        } else {
            rs.put("patient", "");
        }
        rs.put("diagnosis", prescriptionDiagnosisDao.findByPrescriptionId(code,1));

        if (prescriptionAdjusts != null && prescriptionAdjusts.size() > 0) {
            rs.put("prescriptionAdjust", prescriptionAdjusts.get(0));
        } else {
            rs.put("prescriptionAdjust", "");
        }
    
        Followup followup = followUpDao.getFollowupByPrescriptionCode(code);
        rs.put("followup",followup);
        
        WlyyPrescriptionExpressageDO prescriptionExpressage = prescriptionExpressageDao.findByPrescriptionCode(code);
        rs.put("prescriptionExpressage",prescriptionExpressage);
        return rs;
    }

    public List<OrgAddress> getOrgAddress(String hospitalCode){
        List<OrgAddress> orgAddressList = orgAddressDao.findByHospitalCode(hospitalCode);
        return orgAddressList;
    }

    public int reviewPrescription(String code, String reason,String reasonCode, String state, String dept, String registerFee, String rateTypeCode,String remark,String deptCode,String orgAddress) throws Exception {
        synchronized (code.intern()){
            if(StringUtils.isNotBlank(state)&&"2".equals(state)){
                registerFee = null ;
            }
            if (StringUtils.isNotBlank(state)) {
                WlyyPrescriptionDO p = prescriptionDao.findById(code).orElse(null);
                //状态0 或31 才能审核
                if(p.getStatus().equals(WlyyPrescriptionLogDO.PrescriptionLogStatus.revieweding.getValue())||
                        p.getStatus().equals(WlyyPrescriptionLogDO.PrescriptionLogStatus.add_error.getValue())){
                    //更新审核记录
                    prescriptionAdjustService.saveDept(code,dept,rateTypeCode,registerFee,deptCode);

                    WlyyPrescriptionReviewedDO reviewed = prescriptionReviewedDao.findByPrescriptionCode(code);
                    reviewed.setRemark(remark);
                    if ("1".equals(state)) {
                        if(StringUtils.isNotBlank(dept)){
                            String sql = "SELECT t.dept_name AS deptName FROM zy_iv_dept_dict t WHERE t.dept_code = '"+dept+"'" ;
                            List<Map<String,Object>> list = jdbcTemplate.queryForList(sql);
                            if(list!=null&&list.size()>0){
                                String name = (String)list.get(0).get("deptName");
                                p.setDeptName(name);
                                p.setDept(dept);
                            }else{
                                throw new ServiceException("科室不能为空");
                            }
                        }else{
                            throw new ServiceException("科室不能为空");
                        }

                        //审核通过,前往智业开方
                        p.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.adding.getValue());
//                        p.setReviewedTime(new Date());
//                        p.setJwDeptCode(dept);
//                        p.setJwRegisterFee(registerFee);
//                        p.setJwGisterTypeCode(rateTypeCode);//挂号类型

                    } else {
                        //审核不通过
                        p.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.no_reviewed.getValue());
//                        p.setReviewedTime(new Date());
                        reviewed.setReason(reason);
                        reviewed.setReviewedTime(new Date());
                        reviewed.setStatus(WlyyPrescriptionReviewedDO.PrescriptionReviewedStatus.reviewed_fail.getValue());

                        //修改系统的续方消息的审核状态
                        messageDao.updatePreScriptionMessage(p.getConsult(), "2", 6);
                        //审核不用过发送消息
                        //发送Im消息
                        JSONObject content = new JSONObject();
                        content.put("title", "我已经审核您" + DateUtil.dateToStr(p.getCreateTime(), "yyyy-MM-dd") + "发起的续方申请");
                        content.put("result", "审核不通过");
                        content.put("reason", reason);
                        content.put("url", "");
                        imUtill.sendTopicIM(p.getDoctor(), p.getDoctorName(), p.getConsult(), "15", content.toString(), null);

                        //审核不通过模板消息
                        sendRMess(p.getId(), 0);

                        //取消长处方关联的随访记录
                        canclePrescriptionFollowup(code);
                    }
                    prescriptionReviewedDao.save(reviewed);

                    //药房地址以及科室ocde
//                    p.setOrgAddress(orgAddress);
//                    p.setDeptCode(deptCode);
//                    OrgAddress address = orgAddressDao.findByDeptCode(deptCode);
//                    p.setOrgAddress(address.getOrgAddress());
//                    p.setDeptCodeName(address.getDeptName());
                    prescriptionDao.save(p);

                    //医生操作日志
                    WlyyPrescriptionLogDO log = new WlyyPrescriptionLogDO();
                    log.setStatus(p.getStatus());
                    log.setCreateTime(new Date());
                    log.setPrescriptionCode(p.getId());
                    log.setUserCode(p.getDoctor());
                    log.setUserName(p.getDoctorName());
                    log.setUserType(2);
                    log.setDatajson("医生审核/开方");
                    prescriptionLogDao.save(log);

                    //同步智业接口,前往开方
                    return upLoadPrescriotionState(state, p, reviewed, deptCode);
                }
            }
        }
        return 0;
    }

    public int updatePresInfo(String code, String infos, String diagnosis, String reason,String reasonCode) {
        if (StringUtils.isNotBlank(code) && StringUtils.isNotBlank(infos)) {
            WlyyPrescriptionDO prescription = prescriptionDao.findById(code).orElse(null);
            if (null==prescription){
                return 0;
            }
            String hospital = prescription.getHospital();
            String sql1 = " select mapping_code from wlyy_hospital_mapping where code='"+hospital+"' ";
            String jworgCode = jdbcTemplate.queryForObject(sql1,String.class);
            //同步智业接口
//            upload();
            //删除所有药品
            StringBuffer sql = new StringBuffer("UPDATE wlyy_prescription_info SET del = 0 WHERE prescription_code = '" + code + "'");
            jdbcTemplate.execute(sql.toString());
            net.sf.json.JSONArray jsonArray = net.sf.json.JSONArray.fromObject(infos);
            Iterator<Object> it = jsonArray.iterator();
            while (it.hasNext()) {
                net.sf.json.JSONObject info = (net.sf.json.JSONObject) it.next();
                WlyyPrescriptionInfoDO p = (WlyyPrescriptionInfoDO) net.sf.json.JSONObject.toBean(info, WlyyPrescriptionInfoDO.class);
                p.setDel(1);
                p.setPrescriptionId(code);
                prescriptionInfoDao.save(p);
            }
            prescriptionDiagnosisService.setPrescriptionInfo(code);

            //设置处方疾病类型
            if (StringUtils.isNotBlank(diagnosis)) {

                StringBuffer sql2 = new StringBuffer("DELETE FROM wlyy_prescription_diagnosis WHERE prescription_code = '" + code + "'");
                jdbcTemplate.execute(sql2.toString());

                JSONArray array = new JSONArray(diagnosis);
                Iterator<Object> itds = array.iterator();
                while (itds.hasNext()) {
                    JSONObject info = (JSONObject) itds.next();
                    WlyyPrescriptionDiagnosisDO p = new WlyyPrescriptionDiagnosisDO();
                    p.setCode(CommonUtil.getCode());
                    p.setCreateTime(new Date());
                    p.setName(info.getString("name"));
                    p.setCode(info.getString("code"));
//                    String json = dictIcd10Dao.getHealthProblemsByIcd10Code(info.getString("code"));
//                    if(StringUtils.isNoneBlank(json)){
//                        JSONObject healthProblem = new JSONObject(json);
//                        p.setCode(healthProblem.getString("key"));
//                        p.setName(healthProblem.getString("value"));
//                    }else{
//                        p.setCode("other");
//                        p.setName(info.getString("name"));
//                    }

                    p.setPrescriptionId(code);
                    prescriptionDiagnosisDao.save(p);
                }
                prescriptionDiagnosisService.setPrescriptionDiagnosis(code);
            }

            WlyyPrescriptionDO p = prescriptionDao.findById(code).orElse(null);
            //记录Log
            WlyyPrescriptionLogDO log = new WlyyPrescriptionLogDO();
            log.setStatus(p.getStatus());
            log.setCreateTime(new Date());
            log.setPrescriptionCode(p.getId());
            log.setUserCode(p.getDoctor());
            log.setUserName(p.getDoctorName());
            log.setUserType(2);
            log.setDatajson("调整处方");
            prescriptionLogDao.save(log);

            //记录流水
            PrescriptionAdjust prescriptionAdjust = new PrescriptionAdjust();
            prescriptionAdjust.setCode(CommonUtil.getCode());
            prescriptionAdjust.setCreateTime(new Date());
            prescriptionAdjust.setReason(reason);
            prescriptionAdjust.setUserCode(p.getDoctor());
            prescriptionAdjust.setUserName(p.getDoctorName());
            prescriptionAdjust.setUserType("1");
            prescriptionAdjust.setPrescriptionCode(code);
            prescriptionAdjustDao.save(prescriptionAdjust);

            //保存原因
            saveAjustReason(reasonCode,prescriptionAdjust,code);

            return 1;
        }
        return 0;
    }

    public void saveAjustReason(String reasonCode,PrescriptionAdjust prescriptionAdjust,String code){
        if(StringUtils.isNotBlank(reasonCode)){
            String reasoncodes[] = reasonCode.split(",");
            for(int i=0;i<reasoncodes.length;i++){
                WlyyHospitalSysDictDO dict = hospitalSysDictDao.findOneByDictNameAndDictCode("PrescriptionReasonDict0",reasoncodes[i]);
                PrescriptionAdjustReason prescriptionAdjustReason = new PrescriptionAdjustReason();
                prescriptionAdjustReason.setCode(CommonUtil.getCode());
                prescriptionAdjustReason.setPrescriptionCode(code);
                prescriptionAdjustReason.setAdjustCode(prescriptionAdjust.getCode());
                prescriptionAdjustReason.setReasonCode(reasoncodes[i]);
                prescriptionAdjustReason.setReasonValue(dict.getDictValue());
                prescriptionAdjustReason.setCreateTime(new Date());
                prescriptionAdjustReasonDao.save(prescriptionAdjustReason);
            }
        }
    }

    public String saveReviewedReason(String reasonCode,String reviewedCode ,String code){
        String reason = "";
        if(StringUtils.isNotBlank(reasonCode)){
            String reasoncodes[] = reasonCode.split(",");
            for(int i=0;i<reasoncodes.length;i++){
                WlyyHospitalSysDictDO dict = hospitalSysDictDao.findOneByDictNameAndDictCode("PrescriptionReasonDict0",reasoncodes[i]);
                reason += dict.getDictValue();
            }
        }
        return reason;
    }


    /**
     * 获取订单记录
     *
     * @param type      1.需跟踪订单 2.已经接受订单
     * @param doctor    医生code
     * @param nameKey   用户姓名关键字
     * @param startDate 开始时间
     * @param endDate   结束时间
     * @param hospital  服务站
     * @param state     状态
     * @param page      起始页
     * @param size      分页大小
     * @return
     */
    public JSONArray getHDoctorPrescriptionExpressage(String teamCode, String type, String doctor, String nameKey, String startDate, String endDate, String hospital, String state, Integer page, Integer size) {

        StringBuffer pre_sql = new StringBuffer("SELECT " +
                " p.`name`, " +
                " TIMESTAMPDIFF(YEAR,p.birthday,SYSDATE()) age, " +
                " p.sex, " +
                " p.photo, " +
                " pr.`status`, " +
                " pr.patient," +
                " pr.admin_team_id," +
                " pr.`code`, " +
                " LEFT(pr.create_time,19) AS createTime, " +
                " pr.doctor, " +
                " pr.dispensary_type AS dispensaryType ," +
                " e.expressage_code AS expressageCode, " +
                " e.expressage_name AS expressageName, " +
                " e.expressage_hospital_address AS expressageHospitalAddress, " +
                " e.expressage_hospital_name AS expressageHospitalName, " +
                " e.expressage_hospital_code AS expressageHospitalCode, " +
                " e.patient_hospital_address AS patientHospitalAddress, " +
                " e.patient_hospital_name AS patientHospitalName, " +
                " e.patient_hospital_code AS patientHospitalCode " +
                " FROM " +
                " wlyy_prescription pr " +
                " LEFT JOIN base_patient p ON pr.patient = p.id " +
                " JOIN wlyy_prescription_expressage e ON pr.code = e.prescription_code " +
                " WHERE 1=1 AND pr.status >="+WlyyPrescriptionLogDO.PrescriptionLogStatus.pay_success.getValue()+" ");
        List<Object> params = new ArrayList<>();
        setExpressageSql(pre_sql, params, teamCode, type, doctor, nameKey, startDate, endDate, hospital, state, page, size);
        List<Map<String, Object>> rs = jdbcTemplate.queryForList(pre_sql.toString(), params.toArray());
        //通过缓存查找药品和疾病
        for (Map<String, Object> map : rs) {
            String code = (String) map.get("code");
            map.put("prescriptionInfo", prescriptionDiagnosisService.getPrescriptionInfo(code));
            map.put("prescriptionDt", prescriptionDiagnosisService.getPrescriptionDiagnosis(code));
        }
        return new JSONArray(rs);
    }

    public JSONObject getHDoctorPrescriptionExpressageTotal(String teamCode, String type, String doctor, String nameKey, String startDate, String endDate, String hospital, String state) {

        StringBuffer pre_sql = new StringBuffer("SELECT count(1) AS total " +
                " FROM " +
                " wlyy_prescription pr " +
                " LEFT JOIN base_patient p ON pr.patient = p.id " +
                " JOIN wlyy_prescription_expressage e ON pr.code = e.prescription_code " +
                " WHERE 1=1 AND pr.status >="+WlyyPrescriptionLogDO.PrescriptionLogStatus.pay_success.getValue()+" ");
        List<Object> params = new ArrayList<>();
        setExpressageTotalSql(pre_sql, params, teamCode, type, doctor, nameKey, startDate, endDate, hospital, state);
        List<Map<String, Object>> rs = jdbcTemplate.queryForList(pre_sql.toString(), params.toArray());
        JSONObject jsonRs=new JSONObject();
        if(rs!=null&&rs.size()>0){
            jsonRs.put("total",rs.get(0).get("total"));
        }else{
            jsonRs.put("total",0);
        }
        return jsonRs;
    }

    public void setExpressageSql(StringBuffer pre_sql, List<Object> param, String teamCode, String type, String doctor, String nameKey, String startDate, String endDate, String hospital, String state, Integer page, Integer size) {

        if (StringUtils.isNotBlank(type)) {
            if ("1".equals(type)) {
                pre_sql.append(" AND e.expressage_Code = ?");
                param.add(doctor);
            } else {
                pre_sql.append(" AND e.accept_Code = ?");
                param.add(doctor);
            }
        }
        if (StringUtils.isNotBlank(teamCode)) {
            pre_sql.append(" AND pr.admin_team_id = ? ");
            param.add(teamCode);
        }
        if (StringUtils.isNotBlank(nameKey)) {
            pre_sql.append(" AND p.name LIKE ? ");
            param.add("%" + nameKey + "%");
        }
        if (StringUtils.isNotBlank(startDate)) {
            pre_sql.append(" AND pr.create_time >= ? ");
            param.add(startDate + " 00:00:00");
        }
        if (StringUtils.isNotBlank(endDate)) {
            pre_sql.append(" AND pr.create_time <= ? ");
            param.add(endDate + " 23:59:59");
        }
        if (StringUtils.isNotBlank(hospital)) {
            if("other".equals(hospital)){
                pre_sql.append(" AND (e.patient_hospital_code IS NULL OR e.patient_hospital_code ='') ");
            }else{
                pre_sql.append(" AND e.patient_hospital_code = ? ");
                param.add(hospital);
            }
        }
        if (StringUtils.isNotBlank(state)) {
            pre_sql.append(" AND pr.status = ? ");
            param.add(state);
        }
        pre_sql.append(" ORDER BY pr.create_time DESC ");
        if (page != null && size != null) {
            pre_sql.append(" LIMIT " + (page - 1) * size + "," + size);
        }
    }

    public void setExpressageTotalSql(StringBuffer pre_sql, List<Object> param, String teamCode, String type, String doctor, String nameKey, String startDate, String endDate, String hospital, String state) {

        if (StringUtils.isNotBlank(type)) {
            if ("1".equals(type)) {
                pre_sql.append(" AND e.expressage_Code = ?");
                param.add(doctor);
            } else {
                pre_sql.append(" AND e.accept_Code = ?");
                param.add(doctor);
            }
        }
        if (StringUtils.isNotBlank(teamCode)) {
            pre_sql.append(" AND pr.admin_team_id = ? ");
            param.add(teamCode);
        }
        if (StringUtils.isNotBlank(nameKey)) {
            pre_sql.append(" AND p.name LIKE ? ");
            param.add("%" + nameKey + "%");
        }
        if (StringUtils.isNotBlank(startDate)) {
            pre_sql.append(" AND pr.create_time >= ? ");
            param.add(startDate + " 00:00:00");
        }
        if (StringUtils.isNotBlank(endDate)) {
            pre_sql.append(" AND pr.create_time <= ? ");
            param.add(endDate + " 23:59:59");
        }
        if (StringUtils.isNotBlank(hospital)) {
            if("other".equals(hospital)){
                pre_sql.append(" AND (e.patient_hospital_code IS NULL OR e.patient_hospital_code ='')");
            }else{
                pre_sql.append(" AND e.patient_hospital_code = ? ");
                param.add(hospital);
            }
        }
        if (StringUtils.isNotBlank(state)) {
            pre_sql.append(" AND pr.status = ? ");
            param.add(state);
        }
    }

    /**
     * 获取订单跟踪下拉列表
     * @param doctor
     * @return
     */
    public JSONObject getPrescriptionExpressageFilter(String doctor) {
        JSONObject rs = new JSONObject();

        List<Map<String, Object>> states = new ArrayList<>();

        Map<String, Object> map1 = new HashMap<>();
        map1.put("name", "订单已支付");
        map1.put("code", WlyyPrescriptionLogDO.PrescriptionLogStatus.pay_success.getValue());
        states.add(map1);

        Map<String, Object> map2 = new HashMap<>();
        map2.put("name", "配药完成");
        map2.put("code", WlyyPrescriptionLogDO.PrescriptionLogStatus.wait_expressage.getValue());
        states.add(map2);

        Map<String, Object> map3 = new HashMap<>();
        map3.put("name", "配送中");
        map3.put("code", WlyyPrescriptionLogDO.PrescriptionLogStatus.expressageing.getValue());
        states.add(map3);

        Map<String, Object> map4 = new HashMap<>();
        map4.put("name", "药品到达服务站");
        map4.put("code", WlyyPrescriptionLogDO.PrescriptionLogStatus.expressage2hospital.getValue());
        states.add(map4);

        Map<String, Object> map5 = new HashMap<>();
        map5.put("name", "已完成");
        map5.put("code", WlyyPrescriptionLogDO.PrescriptionLogStatus.finish.getValue());
        states.add(map5);
        //设置配送状态下拉框
        rs.put("states", states);
        BaseDoctorDO d = doctorDao.findByIdAndDel(doctor);
        String sql = " SELECT " +
                " e.patient_hospital_name AS name, " +
                " e.patient_hospital_code AS code " +
                " FROM " +
                " wlyy_prescription_expressage e " +
                " WHERE " +
                " ( " +
                "  e.accept_code = '"+doctor+"' " +
                "  OR e.expressage_code = '"+doctor+"' " +
                " ) " +
                " AND e.patient_hospital_code IS NOT NULL " +
                " AND e.patient_hospital_code <> '' " +
                " GROUP BY " +
                " e.patient_hospital_code ";
        List<Map<String, Object>> hs = jdbcTemplate.queryForList(sql);
        rs.put("hospitals", hs);

        return rs;
    }

    //
    public int onMesSquareState(String mes) {
        try {
            JSONObject json = new JSONObject(mes);
            Integer state = json.getInt("state");
            String code = json.getString("prescription");
            return sendRMess(code, state);
        } catch (Exception e) {
            logger.error("报文解析失败!", e);
        }
        return 0;
    }

    /**
     * 发送审核模板消息
     * @param code
     * @param state 1.审核通过，0，审核不通过，2.处方调整并审核通过
     * @return
     */
    public int sendRMess(String code, Integer state) {
        try {
            WlyyPrescriptionDO prescription = prescriptionDao.findById(code).orElse(null);
            BasePatientDO p = patientDao.findById(prescription.getPatientCode()).orElse(null);
            if (state != null && state == 1) {
                prescriptionNoticesService.sendMessages(prescription.getPatientCode(), prescription.getDoctorName(), null, 1, 1, "prescription/html/re-prescription_info.html?openid=" + p.getOpenid() + "&toUser=" + p.getId() + "&toName=" + p.getName() + "&code=" + prescription.getId());
            } else if(state != null && state == 0){
                prescriptionNoticesService.sendMessages(prescription.getPatientCode(), prescription.getDoctorName(), null, 1, 0, "prescription/html/re-prescription_info.html?openid=" + p.getOpenid() + "&toUser=" + p.getId() + "&toName=" + p.getName() + "&code=" + prescription.getId());
            }else{
                prescriptionNoticesService.sendMessages(prescription.getPatientCode(), prescription.getDoctorName(), null, 1, 2, "prescription/html/re-prescription_info.html?openid=" + p.getOpenid() + "&toUser=" + p.getId() + "&toName=" + p.getName() + "&code=" + prescription.getId());
            }
            return 1;
        } catch (Exception e) {
            logger.error("续方消息发送失败!", e);
        }
        return 0;
    }

    public JSONObject getPrescriptionExpressageAsdoctorFilter() {
        JSONObject rs = new JSONObject();

        List<Map<String, Object>> states = new ArrayList<>();

        Map<String, Object> map1 = new HashMap<>();
        map1.put("name", "订单已支付");
        map1.put("code", WlyyPrescriptionLogDO.PrescriptionLogStatus.pay_success.getValue());
        states.add(map1);

        Map<String, Object> map2 = new HashMap<>();
        map2.put("name", "配药完成");
        map2.put("code", WlyyPrescriptionLogDO.PrescriptionLogStatus.wait_expressage.getValue());
        states.add(map2);

        Map<String, Object> map3 = new HashMap<>();
        map3.put("name", "配送中");
        map3.put("code", WlyyPrescriptionLogDO.PrescriptionLogStatus.expressageing.getValue());
        states.add(map3);

        Map<String, Object> map4 = new HashMap<>();
        map4.put("name", "药品到达服务站");
        map4.put("code", WlyyPrescriptionLogDO.PrescriptionLogStatus.expressage2hospital.getValue());
        states.add(map4);

        Map<String, Object> map5 = new HashMap<>();
        map5.put("name", "配送完成");
        map5.put("code", WlyyPrescriptionLogDO.PrescriptionLogStatus.finish.getValue());
        states.add(map5);
        //设置配送状态下拉框
        rs.put("states", states);


        //1 自取 2快递配送 3健管师配送
        List<Map<String, Object>> dispensaryTypes = new ArrayList<>();

        Map<String, Object> type1 = new HashMap<>();
        type1.put("name", "自取");
        type1.put("code", "1");
        dispensaryTypes.add(type1);

        Map<String, Object> type2 = new HashMap<>();
        type2.put("name", "快递配送");
        type2.put("code", "2");
        dispensaryTypes.add(type2);

        Map<String, Object> type3 = new HashMap<>();
        type3.put("name", "健管师配送");
        type3.put("code", "3");
        dispensaryTypes.add(type3);
        //设置配送状态下拉框
        rs.put("dispensaryTypes", dispensaryTypes);

        return rs;
    }

    public List<Map<String,Object>> getDoctorPrescriptionExpressage(Integer teamCode, String nameKey, String startDate, String endDate, String hospital, String state, String dispensaryType, String AllocationType, Integer page, Integer size) {

        StringBuffer pre_sql = new StringBuffer("SELECT " +
                " p.`name`, " +
                " TIMESTAMPDIFF(YEAR,p.birthday,SYSDATE()) age, " +
                " p.sex, " +
                " p.photo, " +
                " pr.`status`, " +
                " pr.`code`, " +
                " LEFT(pr.create_time,19) AS createTime, " +
                " pr.doctor, " +
                " pr.dispensary_type AS dispensaryType," +
                " e.address, " +
                " e.province_name AS provinceName, " +
                " e.city_name AS cityName," +
                " e.town_name AS townName," +
                " e.street_name AS streetName," +
                " e.hospital_name AS hospitalName," +
                " e.hospital_address AS hospitalAddress," +
                " e.expressage_code AS expressageCode, " +
                " e.expressage_name AS expressageName, " +
                " e.expressage_hospital_address AS expressageHospitalAddress, " +
                " e.expressage_hospital_name AS expressageHospitalName, " +
                " e.expressage_hospital_code AS expressageHospitalCode, " +
                " e.patient_hospital_address AS patientHospitalAddress, " +
                " e.patient_hospital_name AS patientHospitalName, " +
                " e.patient_hospital_code AS patientHospitalCode " +
                " FROM " +
                " wlyy_prescription pr " +
                " LEFT JOIN base_patient p ON pr.patient = p.id " +
                " JOIN wlyy_prescription_expressage e ON pr.code = e.prescription_code " +
                " JOIN wlyy_prescription_reviewed r ON pr.code = r.prescription_code " +
                " WHERE r.status=1 AND pr.status >="+WlyyPrescriptionLogDO.PrescriptionLogStatus.pay_success.getValue()+" ");
        List<Object> params = new ArrayList<>();
        setDoctorExpressageSql(pre_sql, params, teamCode, nameKey, startDate, endDate, hospital, state, dispensaryType, AllocationType, page, size);
        List<Map<String, Object>> rs = jdbcTemplate.queryForList(pre_sql.toString(), params.toArray());
        //通过缓存查找药品和疾病
        for (Map<String, Object> map : rs) {
            String code = (String) map.get("code");
            map.put("prescriptionInfo", prescriptionDiagnosisService.getPrescriptionInfo(code));
            map.put("prescriptionDt", prescriptionDiagnosisService.getPrescriptionDiagnosis(code));
        }
        return rs;
    }

    public JSONObject getDoctorPrescriptionExpressageTotal(Integer teamCode, String nameKey, String startDate, String endDate, String hospital, String state, String dispensaryType, String AllocationType) {

        StringBuffer pre_sql = new StringBuffer("SELECT count(1) AS total " +
                " FROM " +
                " wlyy_prescription pr " +
                " LEFT JOIN base_patient p ON pr.patient = p.id " +
                " JOIN wlyy_prescription_expressage e ON pr.code = e.prescription_code " +
                " JOIN wlyy_prescription_reviewed r ON pr.code = r.prescription_code " +
                " WHERE r.status=1 AND pr.status >="+WlyyPrescriptionLogDO.PrescriptionLogStatus.pay_success.getValue()+" ");
        List<Object> params = new ArrayList<>();
        setDoctorExpressageTotalSql(pre_sql, params, teamCode, nameKey, startDate, endDate, hospital, state, dispensaryType, AllocationType);
        List<Map<String, Object>> rs = jdbcTemplate.queryForList(pre_sql.toString(), params.toArray());
        JSONObject jsonRs = new JSONObject();
        if(rs!=null && rs.size()>0){
            jsonRs.put("total",rs.get(0).get("total"));
        }else{
            jsonRs.put("total",0);
        }
        return jsonRs;
    }

    public void setDoctorExpressageSql(StringBuffer pre_sql, List<Object> param, Integer teamCode, String nameKey, String startDate, String endDate, String hospital, String state, String dispensaryType, String AllocationType, Integer page, Integer size) {
        if (StringUtils.isNotBlank(nameKey)) {
            pre_sql.append(" AND p.name LIKE ? ");
            param.add("%" + nameKey + "%");
        }
        if (teamCode != null && teamCode > 0) {
            pre_sql.append(" AND pr.admin_team_id = ? ");
            param.add(teamCode);
        }
        if (StringUtils.isNotBlank(startDate)) {
            pre_sql.append(" AND pr.create_time >= ? ");
            param.add(startDate + " 00:00:00");
        }
        if (StringUtils.isNotBlank(endDate)) {
            pre_sql.append(" AND pr.create_time <= ? ");
            param.add(endDate + " 23:59:59");
        }
        if (StringUtils.isNotBlank(hospital)) {
            if("other".equals(hospital)){
                pre_sql.append(" AND (e.patient_hospital_code IS NULL OR e.patient_hospital_code ='')");
            }else{
                pre_sql.append(" AND e.patient_hospital_code = ? ");
                param.add(hospital);
            }
        }
        if (StringUtils.isNotBlank(state)) {
            pre_sql.append(" AND pr.status = ? ");
            param.add(state);
        }
        if (StringUtils.isNotBlank(dispensaryType)) {
            pre_sql.append(" AND pr.dispensary_type = ? ");
            param.add(dispensaryType);
        }
        if (StringUtils.isNotBlank(AllocationType)) {
            //1为未分配建管师列表
            if ("1".equals(AllocationType)) {
                pre_sql.append(" AND e.expressage_code IS NULL ");
            }
        }

        pre_sql.append(" ORDER BY pr.dispensary_type DESC,e.expressage_code ASC,r.reviewed_time DESC ");
        if (page != null && size != null) {
            pre_sql.append(" LIMIT " + (page - 1) * size + "," + size);
        }
    }

    public void setDoctorExpressageTotalSql(StringBuffer pre_sql, List<Object> param, Integer teamCode, String nameKey, String startDate, String endDate, String hospital, String state, String dispensaryType, String AllocationType) {
        if (StringUtils.isNotBlank(nameKey)) {
            pre_sql.append(" AND p.name LIKE ? ");
            param.add("%" + nameKey + "%");
        }
        if (teamCode != null && teamCode > 0) {
            pre_sql.append(" AND pr.admin_team_id = ? ");
            param.add(teamCode);
        }
        if (StringUtils.isNotBlank(startDate)) {
            pre_sql.append(" AND pr.create_time >= ? ");
            param.add(startDate + " 00:00:00");
        }
        if (StringUtils.isNotBlank(endDate)) {
            pre_sql.append(" AND pr.create_time <= ? ");
            param.add(endDate + " 23:59:59");
        }
        if (StringUtils.isNotBlank(hospital)) {
            if("other".equals(hospital)){
                pre_sql.append(" AND (e.patient_hospital_code IS NULL OR e.patient_hospital_code ='')");
            }else{
                pre_sql.append(" AND e.patient_hospital_code = ? ");
                param.add(hospital);
            }
        }
        if (StringUtils.isNotBlank(state)) {
            pre_sql.append(" AND pr.status = ? ");
            param.add(state);
        }
        if (StringUtils.isNotBlank(dispensaryType)) {
            pre_sql.append(" AND pr.dispensary_type = ? ");
            param.add(dispensaryType);
        }
        if (StringUtils.isNotBlank(AllocationType)) {
            //1为未分配建管师列表
            if ("1".equals(AllocationType)) {
                pre_sql.append(" AND e.expressage_code IS NULL ");
            }
        }
    }

    public JSONObject getCountExpressage(String teamCode) {
        StringBuffer pre_sql = new StringBuffer("SELECT " +
                " count(1) AS count  " +
                " FROM " +
                " wlyy_prescription pr " +
                " JOIN wlyy_prescription_expressage e ON pr. CODE = e.prescription_code  " +
                " JOIN wlyy_prescription_reviewed r ON pr.code = r.prescription_code " +
                " WHERE " +
                " pr.admin_team_id = ? " +
                " AND pr.dispensary_type = 3 " +
                " AND e.expressage_code IS NULL " +
                " AND r.status=1 ");
        List<Map<String, Object>> list = jdbcTemplate.queryForList(pre_sql.toString(), new Object[]{teamCode});

        if (list != null && list.size() > 0) {
            Map<String, Object> map = list.get(0);
            return new JSONObject(map);
        } else {
            JSONObject rs = new JSONObject();
            rs.put("count", 0);
            return rs;
        }
    }

    public JSONArray getHospitalListTitle(String teamCode) {
        StringBuffer pre_sql = new StringBuffer("SELECT e.patient_hospital_code AS hospitalCode ,e.patient_hospital_name AS hospitalName FROM " +
                "  wlyy_prescription pr " +
                "  JOIN wlyy_prescription_expressage e ON pr.code = e.prescription_code " +
                "  WHERE pr.admin_team_id = ? " +
                "  AND (e.expressage_code IS NULL OR e.expressage_code ='') " +
                "  AND pr.dispensary_type =3 " +
                "  GROUP BY e.patient_hospital_code ");
        List<Map<String, Object>> list = jdbcTemplate.queryForList(pre_sql.toString(), new Object[]{teamCode});
        return new JSONArray(list);
    }


    public JSONArray getIcd10Info(String nameKey) {
        StringBuffer stringBuffer = new StringBuffer(" SELECT t.code,t.name FROM icd10_dict t WHERE (t.name LIKE ? or t.phonetic_code like ?) and t.chronic_flag='1' ORDER BY LENGTH(t.name)");
        String tmp = "%" + nameKey + "%";
        List<Map<String, Object>> rs = jdbcTemplate.queryForList(stringBuffer.toString(), new Object[]{tmp,tmp});
        return new JSONArray(rs);
    }

    public int initDrugData() {
        String s = "1,q.d.,每日1次,1@2,b.i.d.,每日2次,2@3,t.i.d.,每日3次,3@4,q.i.d.,每日1次,1@5,q.n.,每日1次,1@6,q.d(11pm),每日1次,1@7,q.m,每日1次,1@8,s.t.,每日1次,1@9,p.r.n.,每日1次,1@10,q1/2h,每日48次,48@11,持续用,每日1次,1@12,q2h,每日12次,12@13,q3h,每日8次,8@14,q4h,每日6次,6@15,q6h,每日4次,4@16,q8h,每日3次,3@17,q12h,每日2次,2@18,q.o.d,每日0.5次,0.5@19,q2h×3次,每日6次,6@20,q3h×3次,每日9次,9@21,q3h×6次,每日18次,18@22,一天五次,每日5次,5@23,q72h,每日1/3次,0.33@24,早餐前,每日1次,1@25,午餐前,每日1次,1@26,晚餐前,每日1次,1@27,早餐后,每日1次,1@28,午餐后,每日1次,1@29,晚餐后,每日1次,1@30,q.d.(早餐),每日1次,1@31,q.d.(午餐),每日1次,1@32,q.d.(晚餐),每日1次,1@33,q1,每日24次,24@34,q2,每日12次,12@35,q3,每日8次,8@36,q1/4h,每日96次,96@37,qh,每日24次,24@38,q.o.d(单日),每日0.5次,0.5@39,q.o.d(双日),每日0.5次,0.5@40,q.w.1,每日1/7次,0.14@41,2/周(1-4),每日2/7次,0.29@42,2/周(2-5),每日2/7次,0.29@43,2/周(3-6),每日2/7次,0.29@44,3/周(1-3-5),每日3/7次,0.43@45,3/周(2-4-6),每日3/7次,0.43@46,q.w.2,每日2/7次,0.29@47,q.w.3,每日3/7次,0.43@48,q.w.4,每日4/7次,0.57@49,q.w.5,每日5/7次,0.71@50,q.w.6,每日6/7次,0.86@51,q.w.7,每日1次,1@52,S.0.S,每日1次,1@";
        String data[] = s.split("@");
        for (int i = 0; i < data.length; i++) {
            try {
                String d[] = data[i].split(",");
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("code", d[0]);
                jsonObject.put("name", d[1]);
                jsonObject.put("remark", d[2]);
                jsonObject.put("rate", d[3]);
                redisTemplate.opsForValue().set("wlyy:dict:recipeFrequency:" + d[0], jsonObject.toString());
            } catch (Exception e) {
                System.out.println(i + "-----------------------------------------------" + data[i]);
            }
        }
        return 0;
    }

    public JSONObject drugToRate(String code) {
        String val = redisTemplate.opsForValue().get("wlyy:dict:recipeFrequency:" + code);
        if(StringUtils.isBlank(val)){
            initDrugData();
            val = redisTemplate.opsForValue().get("wlyy:dict:recipeFrequency:" + code);
        }
        return new JSONObject(val);
    }

    /**
     * 获取处方对应续方审核状态
     *
     * @param code
     * @return 0 审核中，1为处理完成
     */
    public String presCheckState(String code) {
        StringBuffer sql = new StringBuffer(" SELECT pr.code FROM wlyy_prescription pr JOIN wlyy_prescription_reviewed r ON pr.code = r.prescription_code " +
                "WHERE pr.parent_code = ? AND pr.status >= " + WlyyPrescriptionLogDO.PrescriptionLogStatus.revieweding.getValue() + "" +
                " AND pr.status < " + WlyyPrescriptionLogDO.PrescriptionLogStatus.wait_pay.getValue());
        List<Map<String, Object>> mp = jdbcTemplate.queryForList(sql.toString(), code);
        if (mp != null && mp.size() > 0) {
            return "0";
        } else {
            return "1";
        }
    }

    /**
     * 判断丹田
     * @param patient
     * @param currentDay
     * @return
     */
    public String checkPre(String patient,String currentDay){
        StringBuffer sql = new StringBuffer("SELECT code from wlyy_prescription WHERE patient = ? and " +
                "`status`> "+WlyyPrescriptionLogDO.PrescriptionLogStatus.revieweding.getValue()+" and create_time>? and jw_pay_status = '0'");
        List<Map<String, Object>> mp = jdbcTemplate.queryForList(sql.toString(), patient,currentDay);
        if (mp != null && mp.size() > 0) {
            return "0";
        } else {
            return "1";
        }
    }

    public String presCheckStateObj(String code) {
        StringBuffer sql = new StringBuffer(" SELECT pr.code FROM wlyy_prescription pr JOIN wlyy_prescription_reviewed r ON pr.code = r.prescription_code " +
                "WHERE pr.parent_code = ? AND pr.status >= " + WlyyPrescriptionLogDO.PrescriptionLogStatus.revieweding.getValue() + " " +
                "AND pr.status < " + WlyyPrescriptionLogDO.PrescriptionLogStatus.wait_pay.getValue());
        List<Map<String, Object>> mp = jdbcTemplate.queryForList(sql.toString(), code);
        if (mp != null && mp.size() > 0) {
            return (String) ((mp.get(0)).get("code"));
        } else {
            return "1";
        }
    }


    public JSONArray getRateList() {
        StringBuffer sql = new StringBuffer("SELECT * from zy_common_dict WHERE dict_name = 'IV_RECIPE_FREQUENCY_DICT'");
        List<Map<String, Object>> rs = jdbcTemplate.queryForList(sql.toString());
        return new JSONArray(rs);
    }

    /**
     * @param doctor
     * @return

     * @throws Exception
     */
    public com.alibaba.fastjson.JSONArray getPrescriptionTemp(String doctor) throws Exception {
//        String rp = jwPrescriptionService.getRecipeTemplate(doctor);
//        com.alibaba.fastjson.JSONArray rs = presModeAdapter.modeTopresInfo(rp,doctor);
        com.alibaba.fastjson.JSONArray rs = new com.alibaba.fastjson.JSONArray();
        return rs;
    }

    /**
     * 获取医生挂号费用
     *
     * @param jwHospital
     * @param jwDoctorCode
     * @return
     */
    public JSONArray getRegisterRee(String jwHospital, String jwDoctorCode) {
        StringBuffer sql = new StringBuffer("SELECT " +
                " t.register_fee AS registerFee," +
                " t.register_type AS registerType," +
                " t.register_type_name AS registerTypeName " +
                " FROM " +
                " zy_iv_staff_reg_type_allot_dict t" +
                " WHERE" +
                " t.org_code = '" + jwHospital + "'" +
                " AND t.staff_code = '" + jwDoctorCode + "'" +
                " AND t.register_type_name NOT LIKE '%*%'");
        List<Map<String, Object>> rs = jdbcTemplate.queryForList(sql.toString());
        return new JSONArray(rs);
    }

    /**
     * 获取医生科室
     *
     * @param jwHospital
     * @param jwDoctorCode
     * @return
     */
    public JSONArray getDeptList(String jwHospital, String jwDoctorCode) {
        StringBuffer sql = new StringBuffer(" SELECT " +
                " d.dept_code AS deptCode, " +
                " d.dept_name AS deptName " +
                " FROM " +
                " zy_iv_dept_staff_allot_dict t " +
                " JOIN zy_iv_dept_dict d ON t.dept_code = d.dept_code " +
                " WHERE  " +
                " t.org_code = ?  " +
                " AND t.staff_code = ?");
        List<Map<String, Object>> rs = jdbcTemplate.queryForList(sql.toString(), new Object[]{jwHospital, jwDoctorCode});
        return new JSONArray(rs);
    }

    public JSONArray getUsageDict() {
        StringBuffer sql = new StringBuffer("SELECT t.code,t.`name` FROM zy_common_dict t WHERE t.dict_name='IV_RECIPE_USAGE_DICT' order by find_in_set(`code`,'4') DESC");
        List<Map<String, Object>> rs = jdbcTemplate.queryForList(sql.toString());
        return new JSONArray(rs);
    }
    
    public List<Map<String,Object>> getPhysicDays() {
        String sql = "SELECT t.code,t.`value` FROM system_dict t WHERE t.dict_name='PRESCRIPTION_DAYS'";
        List<Map<String,Object>> rs = jdbcTemplate.queryForList(sql);
        return rs;
    }

    public List<Map<String, Object>> getPhysicInfo(String code,String doctor) {
        StringBuffer sql = new StringBuffer("SELECT " +
                " t.min_dose AS minDose, " +
                " t.physic_code AS physicCode, " +
                " t.storage_conditions AS storageConditions, " +
                " t.dose_unit AS doseUnit, " +
                " com1.`name` AS doseUnitName, " +
                " t.quantity_unit AS quantityUnit, " +
                " com2.`name` AS quantityUnitName, " +
                " t.pack_unit As packUnit, " +
                " com3.`name` As packUnitName, " +
                " pack_spec AS packSpec, " +
                " subject_class AS subjectClass," +
                " d.dispQuantityUntuckFlag," +
                " d.dispPackUntuckFlag" +
                " FROM " +
                " zy_iv_physic_dict t, " +
                " (SELECT  o.disp_quantity_untuck_flag dispQuantityUntuckFlag, " +
                "          o.disp_pack_untuck_flag dispPackUntuckFlag " +
                "   FROM " +
                "        wlyy_doctor_mapping h, " +
                "        zy_iv_org_physic_allot_dict o " +
                "   WHERE o.org_code = h.jw_doctor_hospital " +
                "         AND h.doctor_code = ? " +
                "         AND o.physic_code = ?) d, " +
                " (SELECT code,name FROM zy_common_dict WHERE dict_name = 'IV_MEASURE_UNIT_DICT') com1, " +
                " (SELECT code,name FROM zy_common_dict WHERE dict_name = 'IV_MEASURE_UNIT_DICT') com2, " +
                " (SELECT code,name FROM zy_common_dict WHERE dict_name = 'IV_MEASURE_UNIT_DICT') com3 " +
                " WHERE " +
                " com1.`code` = t.dose_unit   " +
                " AND com2.`code` = t.quantity_unit   " +
                " AND com3.`code` = t.pack_unit   " +
                " AND t.physic_code = ?");
        List<Map<String, Object>> rs = jdbcTemplate.queryForList(sql.toString(),new Object[]{doctor,code,code});
        return rs;
    }


    public JSONArray getReasonByType(Integer type){

        String sql = "SELECT t.value,t.code FROM wlyy_prescription_reason_dict t WHERE t.type = "+type+" ORDER BY t.sort ASC ";
        List<Map<String,Object>> list =  jdbcTemplate.queryForList(sql);
        return  new JSONArray(list);
    }

	
	/**
	 * 根据长处方CODE，取消关联的随访记录
	 * @param prescriptionCode
	 */
	private void canclePrescriptionFollowup(String prescriptionCode){
		try {
			//续方取消，随访记录也取消,这里如果出错则直接捕获，记录日志，不往外抛，避免影响长处方逻辑
			Followup followup = followUpDao.getFollowupByPrescriptionCode(prescriptionCode);
			if(followup !=null){
				followup.setStatus("0");
				followUpDao.save(followup);
			}
		}catch (Exception e){
			logger.info("续方取消，随访记录同步取消失败："+e.getMessage());
		}
	}

    public Boolean checkPatientWithPrescription(String patient){
        List<WlyyPrescriptionDO> list =  prescriptionDao.findByPatientCode(patient);
        if(list!=null&&list.size()>0){
            return true;
        }else{
            return false;
        }
    }
    
    /**
     * 获取当前居民的上一条续方详情
     * @param repUID
     * @return
     */
    public com.alibaba.fastjson.JSONObject getPatientLastPrescriptionProcess(String repUID) throws Exception {
        com.alibaba.fastjson.JSONObject result = new com.alibaba.fastjson.JSONObject();
        List<WlyyPrescriptionDO> list =  prescriptionDao.findByPatientCode(repUID);
        if(list!=null&&list.size()>0){
            result =  this.getPrescriptionProcess(list.get(list.size() -1).getId());
        }
        return result;
    }
    
    /**
     * 获取当前居民和处方派送方式获取上一条续方的物流详情
     * @param repUID
     * @return
     */
    public WlyyPrescriptionExpressageDO getPatientLastPrescriptionExpressInfo(String repUID,Integer dispensaryType) throws Exception {
        com.alibaba.fastjson.JSONObject result = new com.alibaba.fastjson.JSONObject();
        List<WlyyPrescriptionDO> list =  prescriptionDao.findByPatientCodeAndDispensaryType(repUID,dispensaryType);
        WlyyPrescriptionExpressageDO prescriptionExpressage = null;
        if(list!=null&&list.size()>0){
            prescriptionExpressage = prescriptionExpressageDao.findByPrescriptionCode(list.get(list.size() -1).getId());
        }
        return prescriptionExpressage;
    }
    
    /**
     *
     * @param patientCode
     * @param start_date
     * @return
     */
    public Integer getTodayPrescriptionCount(String patientCode,String start_date) {
        Integer result = 0;
        String sqlCount = "SELECT id FROM wlyy_prescription WHERE patient='"+patientCode+"' AND `status`>=0 AND create_time>='"+start_date+" 00:00:00' AND create_time<='"+start_date+" 23:59:59'";
        List<Map<String,Object>> rstotal = jdbcTemplate.queryForList(sqlCount);
        Long count = 0L;
        if(rstotal!=null&&rstotal.size()>0){
            result = rstotal.size();
        }
        return result;
    }
}
