package com.yihu.jw.afterDiagnosis.service.prescription;


import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.base.wx.WxTemplateConfigDO;
import com.yihu.jw.message.service.WxMessageUtil;
import com.yihu.jw.patient.dao.BasePatientDao;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Created by Reece on 2017/8/3.
 */
@Service
public class PrescriptionNoticesService{
    @Autowired
    private WxMessageUtil wxMessageUtil;
    @Autowired
    private BasePatientDao patientDao;

    private static final Logger LOGGER = LoggerFactory.getLogger(PrescriptionNoticesService.class);


    /**
     * 发送续方相关模板消息(增加代理模式)
     *
     * @param patient   要发送对象居民code
     * @param doctName  相关医生姓名（type=1审核医生、type=2 null、type=3配送健管师、type=4审核医生）
     * @param ordenTime 订单完成时间（type=1 null、type=2 null、type=3 null、type=4 订单完成时间）
     * @param type      1:续方审核通知 2：取药代办通知 3：配送中信息变更 4：已完成服务结果
     * @param status    续方通知状态 0不通过 1通过 2调整处方  其余状态均为0
     * @param url       带参数的模板跳转链接
     */
    public void sendMessages(String patient, String doctName, String ordenTime, int type, int status, String url) throws Exception {
        LOGGER.info("=======================> sendMessages start!!");
        LOGGER.debug("patient "+patient);
        LOGGER.debug("doctName "+doctName);
        LOGGER.debug("ordenTime "+ordenTime);
        LOGGER.debug("type "+type);
        LOGGER.debug("status "+status);
        LOGGER.debug("url "+ url);
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        BasePatientDO people = patientDao.findById(patient).orElse(null);
        String openId = people.getOpenid();
        String name = people.getName();
        LOGGER.info("openid:"+openId+"and patient:"+patient+""+"and name:"+name+"and doctorName"+doctName+"and idcard:"+people.getIdcard()+"and ssc:"+people.getSsc()+"and type:"+type);

        JSONObject sendJson = new JSONObject();
        String first = "";
        String remark = "";
        WxTemplateConfigDO templateConfig = null;
        WxTemplateConfigDO newConfig = null;
        if (type == 1) {
            switch (status) {
                case 0:
                    templateConfig = wxMessageUtil.findByWechatIdAndTemplateNameAndSceneAndStatus(null,"template_doctor_audit","xfbtg",1);
                    newConfig = new WxTemplateConfigDO();
                    BeanUtils.copyProperties(templateConfig,newConfig);
                    first = newConfig.getFirst();
                    first = first.replace("key1",(name==null?"":name));
                    newConfig.setFirst(first);
                    newConfig.setKeyword2(doctName);
                    wxMessageUtil.putWxMsg(openId,newConfig,19,new JSONObject());
                    break;
                case 1:
                    templateConfig = wxMessageUtil.findByWechatIdAndTemplateNameAndSceneAndStatus(null,"template_doctor_audit","xftg",1);
                    newConfig = new WxTemplateConfigDO();
                    BeanUtils.copyProperties(templateConfig,newConfig);
                    first = newConfig.getFirst();
                    first = first.replace("key1",(name==null?"":name));
                    newConfig.setFirst(first);
                    newConfig.setKeyword2(doctName);
                    wxMessageUtil.putWxMsg(openId,newConfig,19,new JSONObject());
                    break;
                case 2:
                    templateConfig = wxMessageUtil.findByWechatIdAndTemplateNameAndSceneAndStatus(null,"template_doctor_audit","tztg",1);
                    newConfig = new WxTemplateConfigDO();
                    BeanUtils.copyProperties(templateConfig,newConfig);
                    first = newConfig.getFirst();
                    first = first.replace("key1",(name==null?"":name));
                    newConfig.setFirst(first);
                    newConfig.setKeyword2(doctName);
                    wxMessageUtil.putWxMsg(openId,newConfig,19,new JSONObject());
                    break;
            }
        } else if (type == 2) {
            templateConfig = wxMessageUtil.findByWechatIdAndTemplateNameAndSceneAndStatus(null,"template_doctor_survey","xfqy",1);
            newConfig = new WxTemplateConfigDO();
            BeanUtils.copyProperties(templateConfig,newConfig);
            first = newConfig.getFirst();
            first = first.replace("key1",(name==null?"":name));
            newConfig.setFirst(first);
            newConfig.setKeyword2(dateFormat.format(new Date()));
            wxMessageUtil.putWxMsg(openId,newConfig,11,new JSONObject());
        }
        if (type == 3) {
            templateConfig = wxMessageUtil.findByWechatIdAndTemplateNameAndSceneAndStatus(null,"template_doctor_change","xfqy",1);
            newConfig = new WxTemplateConfigDO();
            BeanUtils.copyProperties(templateConfig,newConfig);
            first = newConfig.getFirst();
            first = first.replace("key1",(name==null?"":name)).replace("key2",(doctName==null?"":doctName));
            newConfig.setFirst(first);
            newConfig.setKeyword1(doctName);
            newConfig.setKeyword2(dateFormat.format(new Date()));
            wxMessageUtil.putWxMsg(openId,newConfig,20,new JSONObject());
        }
        if (type == 4) {
            templateConfig = wxMessageUtil.findByWechatIdAndTemplateNameAndSceneAndStatus(null,"template_doctor_service","xfwc",1);
            newConfig = new WxTemplateConfigDO();
            BeanUtils.copyProperties(templateConfig,newConfig);
            first = newConfig.getFirst();
            first = first.replace("key1",(name==null?"":name));
            newConfig.setFirst(first);
            newConfig.setKeyword2(doctName);
            newConfig.setKeyword3(ordenTime);
            wxMessageUtil.putWxMsg(openId,newConfig,20,new JSONObject());
        }
    }
}
