package com.yihu.jw.afterDiagnosis.service.prescription;

import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.afterDiagnosis.dao.prescription.PrescriptionDispensaryCodeDao;
import com.yihu.jw.afterDiagnosis.dao.prescription.PrescriptionReviewedDao;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.hospital.prescription.*;
import com.yihu.jw.hospital.prescription.dao.PrescriptionDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionExpressageDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionExpressageLogDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionLogDao;
import com.yihu.jw.org.dao.BaseOrgDao;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.util.date.DateUtil;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * Created by chenweida on 2017/7/27.
 * 订单物流相关。
 */
@Service
@Transactional
public class WlyyPrescriptionExpressageService {

    private Logger logger = LoggerFactory.getLogger(WlyyPrescriptionExpressageService.class);
    @Autowired
    private PrescriptionExpressageDao prescriptionExpressageDao;
    @Autowired
    private PrescriptionDispensaryCodeDao prescriptionDispensaryCodeDao;
    @Autowired
    private PrescriptionReviewedDao prescriptionReviewedDao;
    @Autowired
    private PrescriptionLogDao prescriptionLogDao;
    @Autowired
    private PrescriptionDao prescriptionDao;
    @Autowired
    private PrescriptionNoticesService prescriptionNoticesService;
    @Autowired
    private PrescriptionExpressageLogDao prescriptionExpressageLogDao;
    @Autowired
    private WlyyPrescriptionService preBaseptionService;
    @Autowired
    private BaseDoctorDao doctorDao;
    @Autowired
    private BaseOrgDao hospitalDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private BasePatientDao patientDao;
    @Value("${testPattern.sign}")
    private String isClose;


    /**
     * 获取处方配送信息信息
     *
     * @param prescriptionCode 处方code
     * @return
     */
    public WlyyPrescriptionExpressageDO findByPrescriptionCode(String prescriptionCode) {
        return prescriptionExpressageDao.findByPrescriptionCode(prescriptionCode);
    }


    /**
     * 确认配送成功
     *
     * @param code     编码code
     * @param userCode
     * @return 返回取药码或者配送码的类型  1 居民取药码 2 配送员（健管师）取药码 3 配送员（健管师）配送码
     * 前端根据这个类型去跳转
     */
    @Transactional
    public Integer expressage(String code, String userCode) throws Exception {
        Integer returnStatus = -1;
        //获取根据wlyy_prescription_dispensary_code的code处方编码
        PrescriptionDispensaryCode prescriptionDispensaryCode = prescriptionDispensaryCodeDao.finByCode(code);

        if (prescriptionDispensaryCode == null) {
            //保存配送失败的日志
            WlyyPrescriptionLogDO prescriptionLog = new WlyyPrescriptionLogDO();
            prescriptionLog.setPrescriptionCode(prescriptionDispensaryCode.getPrescriptionCode());
            prescriptionLog.setCreateTime(new Date());
            prescriptionLog.setUserCode(userCode);
            prescriptionLog.setUserType(2);
            prescriptionLog.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.expressageing_error.getValue());
            prescriptionLog.setDatajson("编码不存在");
            prescriptionLogDao.save(prescriptionLog);
            return returnStatus;
        }
        if (prescriptionDispensaryCode.getIsUse() == 1) {
            //判断编码是否使用过
            WlyyPrescriptionLogDO prescriptionLog = new WlyyPrescriptionLogDO();
            prescriptionLog.setPrescriptionCode(prescriptionDispensaryCode.getPrescriptionCode());
            prescriptionLog.setCreateTime(new Date());
            prescriptionLog.setUserCode(userCode);
            prescriptionLog.setUserType(2);
            prescriptionLog.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.expressageing_error.getValue());
            prescriptionLog.setDatajson("编码重复扫描");
            prescriptionLogDao.save(prescriptionLog);
            returnStatus = -2;
            return returnStatus;
        }

        switch (prescriptionDispensaryCode.getType()) {
            case 1: {
                //判断是1 居民取药码
                returnStatus = dispensaryCode_1(userCode, prescriptionDispensaryCode);
                break;
            }
            case 2: {
                //判断是2 配送员（健管师）取药码
                returnStatus = -3;
                break;
            }
            case 3: {
                //判断是3 配送员（健管师）配送码
                returnStatus = 0;
                break;
            }
            default: {
                WlyyPrescriptionLogDO prescriptionLog = new WlyyPrescriptionLogDO();
                prescriptionLog.setPrescriptionCode(prescriptionDispensaryCode.getPrescriptionCode());
                prescriptionLog.setCreateTime(new Date());
                prescriptionLog.setUserCode(userCode);
                prescriptionLog.setUserType(2);
                prescriptionLog.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.expressageing_error.getValue());
                prescriptionLog.setDatajson("编码类型错误");
                prescriptionLogDao.save(prescriptionLog);
                returnStatus = -3;
                return returnStatus;
            }
        }
        if (returnStatus>=0){
            WlyyPrescriptionDO prescription = prescriptionDao.findById(prescriptionDispensaryCode.getPrescriptionCode()).orElse(null);
            if (null==prescription){
                returnStatus = -6;
                return returnStatus;
            }
            else {
                String response="";
                String applyNo="";//待发药单据号
                String execOperator = "";//药房发药人编码
                String sql = "  select jw_doctor from base_doctor doc INNER JOIN wlyy_doctor_mapping dom on doc.`code` = dom.doctor_code  " +
                        " INNER JOIN wlyy_org_address_doctor orgDoc on doc.code = orgDoc.doctor " +
                        " where orgdoc.hospital_code='"+prescription.getHospital()+"' and orgDoc.dept_code='"+prescription.getDept()+"' ";
                List<String> execOperatorMappings = jdbcTemplate.queryForList(sql,String.class);
                if (execOperatorMappings.size()>0){
                    execOperator = execOperatorMappings.get(0);
                    // 获取待发药单据号
                    if (isClose.equalsIgnoreCase("0")){
//                        String url = baseUrl + "/third/prescription/getPhysicApplyWaitExecList";
//                        List<NameValuePair> params = new ArrayList<>();
//                        params.add(new BasicNameValuePair("orgCode",prescription.getHospital()));
//                        params.add(new BasicNameValuePair("cardNo",prescription.getSsc()));
//                        params.add(new BasicNameValuePair("execOperator",execOperator));
//                        response = httpClientUtil.post(url, params, "UTF-8");
                    }else {
                        response = "{\"status\":200,\"msg\":\"查询待发药单据列表接口！\",\"data\":\"{\\\"CODE\\\":\\\"1\\\",\\\"returnData\\\":[[{\\\"APPLY_NO\\\":\\\"单据号\\\",\\\"RECIPE_NO\\\":\\\"医嘱号\\\",\\\"SUBJECT_CLASS_NAME\\\":\\\"科目类别名称\\\",\\\"APPLY_TIME\\\":\\\"开单时间\\\",\\\"APPLY_OPERATOR\\\":\\\"开单人编码\\\",\\\"APPLY_OPERATOR_NAME\\\":\\\"开单人姓名\\\",\\\"DIAGNOSE_NAME\\\":\\\"诊断名称\\\",\\\"EXEC_DEPT\\\":\\\"执行科室编码\\\",\\\"EXEC_DEPT_NAME\\\":\\\"执行科室名称\\\",\\\"APPLY_DETAIL\\\":[{\\\"APPLY_SUB_NO\\\":\\\"单据子序号\\\",\\\"ITEM_CODE\\\":\\\"项目代码\\\",\\\"ITEM_NAME\\\":\\\"项目名称\\\",\\\"ITEM_SPEC\\\":\\\"项目规格\\\",\\\"ITEM_PRICE\\\":\\\"项目单价\\\",\\\"ITEM_QUANTITY\\\":\\\"项目数量\\\",\\\"ITEM_UNIT_NAME\\\":\\\"单位\\\",\\\"ITEM_QUANTITY_C\\\":\\\"合计\\\",\\\"COST\\\":\\\"金额\\\",\\\"BIND_FLAG_NAME\\\":\\\"成组\\\",\\\"PHYSIC_DOSE\\\":\\\"剂量\\\",\\\"PHYSIC_DOSE_UNIT_NAME\\\":\\\"剂量单位\\\",\\\"FREQUENCY_NAME\\\":\\\"频次\\\",\\\"DAY_COUNT\\\":\\\"天数\\\",\\\"USAGE_NAME\\\":\\\"用药方法\\\",\\\"REMARK\\\":\\\"备注\\\"}]},{\\\"APPLY_NO\\\":12368727,\\\"RECIPE_NO\\\":5267456,\\\"SUBJECT_CLASS_NAME\\\":\\\"西药\\\",\\\"APPLY_TIME\\\":\\\"2022-01-12\\\",\\\"APPLY_OPERATOR\\\":10971,\\\"APPLY_OPERATOR_NAME\\\":\\\"林金城\\\",\\\"DIAGNOSE_NAME\\\":\\\"待定\\\",\\\"EXEC_DEPT\\\":\\\"14\\\",\\\"EXEC_DEPT_NAME\\\":\\\"药房\\\",\\\"APPLY_DETAIL\\\":[{\\\"APPLY_SUB_NO\\\":1,\\\"ITEM_CODE\\\":74638,\\\"ITEM_NAME\\\":\\\"阿莫西林克拉维酸钾分散片\\\",\\\"ITEM_SPEC\\\":\\\"125mg/62.5mg*12片/盒\\\",\\\"ITEM_PRICE\\\":28,\\\"ITEM_QUANTITY\\\":12,\\\"ITEM_UNIT_NAME\\\":\\\"片\\\",\\\"ITEM_QUANTITY_C\\\":\\\"1盒\\\",\\\"COST\\\":28,\\\"BIND_FLAG_NAME\\\":\\\"┓\\\",\\\"PHYSIC_DOSE\\\":2,\\\"PHYSIC_DOSE_UNIT_NAME\\\":\\\"片\\\",\\\"FREQUENCY_NAME\\\":\\\"b.i.d.\\\",\\\"DAY_COUNT\\\":2,\\\"USAGE_NAME\\\":\\\"口服\\\",\\\"REMARK\\\":null},{\\\"APPLY_SUB_NO\\\":2,\\\"ITEM_CODE\\\":42235,\\\"ITEM_NAME\\\":\\\"维拉帕米片(异博定)\\\",\\\"ITEM_SPEC\\\":\\\"40mg*30片/瓶\\\",\\\"ITEM_PRICE\\\":3.91,\\\"ITEM_QUANTITY\\\":4,\\\"ITEM_UNIT_NAME\\\":\\\"片\\\",\\\"ITEM_QUANTITY_C\\\":\\\"4片\\\",\\\"COST\\\":0.52,\\\"BIND_FLAG_NAME\\\":\\\"┛\\\",\\\"PHYSIC_DOSE\\\":1,\\\"PHYSIC_DOSE_UNIT_NAME\\\":\\\"mg\\\",\\\"FREQUENCY_NAME\\\":\\\"b.i.d.\\\",\\\"DAY_COUNT\\\":2,\\\"USAGE_NAME\\\":\\\"口服\\\",\\\"REMARK\\\":null}]}]],\\\"byRefParaData\\\":[{\\\"totalCount\\\":\\\"2\\\",\\\"errorMsg\\\":\\\"\\\"}]}\"}";
                    }
                    JSONObject res = JSONObject.parseObject(response);
                    if (res.getInteger("status")==200){
                        JSONObject data = new JSONObject();
                        if (res.get("data") instanceof String){
                            String dataInfo = res.getString("data");
                            data = JSONObject.parseObject(dataInfo);
                        }else {
                            data = res.getJSONObject("data");
                        }
                        if (!"1".equals(data.getString("CODE"))){
                            returnStatus = -6;
                            return returnStatus;
                        }else {
                            String waitForCount = data.getJSONArray("byRefParaData").getJSONObject(0).getString("totalCount");
                            if (Integer.parseInt(waitForCount)>0){
                                applyNo = data.getJSONArray("returnData").getJSONArray(0).getJSONObject(0).getString("APPLY_NO");
                            }
                            else {
                                returnStatus = -7;
                                return returnStatus;
                            }
                        }
                    }else {
                        returnStatus = -6;
                        return returnStatus;
                    }
                    if (isClose.equalsIgnoreCase("0")){
//                        String url = baseUrl + "/third/prescription/sickPhysicDosage";
//                        List<NameValuePair> params = new ArrayList<>();
//                        params.add(new BasicNameValuePair("orgCode",prescription.getHospital()));
//                        params.add(new BasicNameValuePair("applyNo", applyNo));
//                        params.add(new BasicNameValuePair("execOperator", execOperator));
//                        response = httpClientUtil.post(url, params, "UTF-8");
                    }else {
                        response = "{\"status\":200,\"msg\":\"发药成功！\",\"data\":\"{\\\"CODE\\\":\\\"1\\\",\\\"returnData\\\":[{\\\"result\\\":\\\"True\\\"}],\\\"byRefParaData\\\":[{\\\"layNo\\\":\\\"摆药单号\\\",\\\"errorMsg\\\":\\\"\\\"}]}\"}";
                    }
                    res = JSONObject.parseObject(response);
                    if (res.getInteger("status")==200) {
                        JSONObject data = new JSONObject();
                        if (res.get("data") instanceof String){
                            String dataInfo = res.getString("data");
                            data = JSONObject.parseObject(dataInfo);
                        }else {
                            data = res.getJSONObject("data");
                        }
                        if (!"1".equals(data.getString("CODE"))) {
                            returnStatus = -6;
                            return returnStatus;
                        }
                    }else {
                        returnStatus = -6;
                        return returnStatus;
                    }
                }
            }
        }
        return returnStatus;
    }

    /**
     * 1 居民取药码 业务
     *
     * @param userCode
     * @param prescriptionDispensaryCode
     * @return
     */
    private Integer dispensaryCode_1(String userCode, PrescriptionDispensaryCode prescriptionDispensaryCode) throws Exception {

        //修改处方状态为完成
        WlyyPrescriptionDO prescription = prescriptionDao.findById(prescriptionDispensaryCode.getPrescriptionCode()).orElse(null);
        Date finshTime = new Date();
        prescription.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.finish.getValue());
        prescription.setFinishTime(finshTime);

        //保存配送成功的日志
        WlyyPrescriptionLogDO prescriptionLog = new WlyyPrescriptionLogDO();
        prescriptionLog.setPrescriptionCode(prescriptionDispensaryCode.getPrescriptionCode());
        prescriptionLog.setCreateTime(finshTime);
        prescriptionLog.setUserCode(userCode);
        prescriptionLog.setUserType(2);
        prescriptionLog.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.finish.getValue());
        prescriptionLogDao.save(prescriptionLog);

        //修改取药码code为已经使用
        prescriptionDispensaryCode.setIsUse(1);

        //发送模板消息
        WlyyPrescriptionReviewedDO reviewed = prescriptionReviewedDao.findByPrescriptionCode(prescription.getId());
        BasePatientDO patient = patientDao.findById(prescription.getId()).orElse(null);
        String url = "prescription/html/order_tracking.html?openid=" + patient.getOpenid() + "" +
                "&code=" + prescriptionDispensaryCode.getPrescriptionCode() + "&toUser=" + patient.getId() + "&toName=" + patient.getName();
        prescriptionNoticesService.sendMessages(prescription.getPatientCode(), reviewed.getDoctorName(), DateUtil.dateToStrLong(finshTime), 4, 0, url);
        return prescriptionDispensaryCode.getType();
    }

    /**
     * 1 居民取药码 业务
     * 巨米智能药柜取药
     * @param userCode
     * @return
     */
    public void dispensaryCode_jumi(String userCode, WlyyPrescriptionDO prescription) throws Exception {
        //修改处方状态为完成
        Date finshTime = new Date();
        prescription.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.finish.getValue());
        prescription.setFinishTime(finshTime);

        //保存配送成功的日志
        WlyyPrescriptionLogDO prescriptionLog = new WlyyPrescriptionLogDO();
        prescriptionLog.setPrescriptionCode(prescription.getId());
        prescriptionLog.setCreateTime(finshTime);
        prescriptionLog.setUserCode(userCode);
        prescriptionLog.setUserType(2);
        prescriptionLog.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.finish.getValue());
        prescriptionLogDao.save(prescriptionLog);

        //发送模板消息
        WlyyPrescriptionReviewedDO reviewed = prescriptionReviewedDao.findByPrescriptionCode(prescription.getId());
        BasePatientDO patient = patientDao.findById(prescription.getPatientCode()).orElse(null);
        String url = "prescription/html/order_tracking.html?openid=" + patient.getOpenid() + "" +
                "&code=" + prescription.getId() + "&toUser=" + patient.getId() + "&toName=" + patient.getName();
        prescriptionNoticesService.sendMessages(prescription.getPatientCode(), reviewed.getDoctorName(), DateUtil.dateToStrLong(finshTime), 4, 0, url);
    }

    /**
     * 配送员（健管师）取药码 业务
     *
     * @param userCode
     * @param prescriptionDispensaryCode
     * @return
     */
    private Integer dispensaryCode_2(String userCode, PrescriptionDispensaryCode prescriptionDispensaryCode) throws Exception {
        //获取配送的医生信息
        BaseDoctorDO doctor = doctorDao.findByIdAndDel(userCode);


        WlyyPrescriptionDO prescription = prescriptionDao.findById(prescriptionDispensaryCode.getPrescriptionCode()).orElse(null);

        //修改处方状态为配送中
        Date expressageing = new Date();
        prescription.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.expressageing.getValue());

        //回填配送表的配送时间
        WlyyPrescriptionExpressageDO prescriptionExpressage = prescriptionExpressageDao.findByPrescriptionCode(prescriptionDispensaryCode.getPrescriptionCode());
//        prescriptionExpressage.setExpressageTime(expressageing);

        //得到patient
        BasePatientDO patient = patientDao.findById(prescription.getPatientCode()).orElse(null);

        //保存配送成功的日志
        WlyyPrescriptionLogDO prescriptionLog = new WlyyPrescriptionLogDO();
        prescriptionLog.setPrescriptionCode(prescriptionDispensaryCode.getPrescriptionCode());
        prescriptionLog.setCreateTime(new Date());
        prescriptionLog.setUserCode(userCode);
        prescriptionLog.setUserName(doctor.getName());
        prescriptionLog.setUserType(2);
        prescriptionLog.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.expressageing.getValue());
        prescriptionLogDao.save(prescriptionLog);

        //修改取药码code为已经使用
        prescriptionDispensaryCode.setIsUse(1);

        //发送模板消息通知患者药品开始配送
        String url = "prescription/html/order_tracking.html?openid=" + patient.getOpenid() + "" +
                "&code=" + prescriptionDispensaryCode.getPrescriptionCode() + "&toUser=" + patient.getId() + "&toName=" + patient.getName();
        prescriptionNoticesService.sendMessages(patient.getId(), doctor.getName(), DateUtil.dateToStrLong(expressageing), 3, 0, url);

        return prescriptionDispensaryCode.getType();
    }

    /**
     * 配送员（健管师）配送码
     * 没用的代码
     *
     * @param userCode
     * @param prescriptionDispensaryCode
     * @return
     */
    @Deprecated
    private Integer dispensaryCode_3(String userCode, PrescriptionDispensaryCode prescriptionDispensaryCode) throws Exception {
        //获取配送的医生信息
        BaseDoctorDO doctor = doctorDao.findByIdAndDel(userCode);

        //判断是不是最终的服务站
        //修改处方状态为配送到服务站
        WlyyPrescriptionDO prescription = prescriptionDao.findById(prescriptionDispensaryCode.getPrescriptionCode()).orElse(null);
        Date expressage2hospitalDate = new Date();
//        prescription.setExpressageTime(expressage2hospitalDate);
        prescription.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.expressage2hospital.getValue());

        //得到patient
        BasePatientDO patient = patientDao.findById(prescription.getPatientCode()).orElse(null);

        //保存配送成功的日志
        WlyyPrescriptionLogDO prescriptionLog = new WlyyPrescriptionLogDO();
        prescriptionLog.setPrescriptionCode(prescriptionDispensaryCode.getPrescriptionCode());
        prescriptionLog.setCreateTime(new Date());
        prescriptionLog.setUserCode(userCode);
        prescriptionLog.setUserName(doctor.getName());
        prescriptionLog.setUserType(2);
        prescriptionLog.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.expressage2hospital.getValue());
        prescriptionLogDao.save(prescriptionLog);

        //修改取药码code为已经使用
        prescriptionDispensaryCode.setIsUse(1);


        //发送模板消息通知患者取药
        //发送模板消息通知患者药品开始配送
        String url = "prescription/html/order_tracking.html?openid=" + patient.getOpenid() + "" +
                "&code=" + prescriptionDispensaryCode.getPrescriptionCode() + "&toUser=" + patient.getId() + "&toName=" + patient.getName();


        prescriptionNoticesService.sendMessages(prescription.getPatientCode(), doctor.getName(), DateUtil.dateToStrLong(expressage2hospitalDate), 4, 0, url);

        return prescriptionDispensaryCode.getType();

    }

    /**
     * 1，保存快递单号增
     * 2，加处方物流记录为配送
     * 3，修改处方状态为配送配送中
     *
     * @param prescriptionExpressage
     * @throws Exception
     */
    @Transactional
    public void updatePrescriptionExpressage(WlyyPrescriptionExpressageDO prescriptionExpressage) throws Exception {

        //修改处方状态为配送配送中
        prescriptionDao.updateStatus(prescriptionExpressage.getOutpatientId(), 65);

        //保存处方物流记录
        prescriptionExpressageDao.save(prescriptionExpressage);

        //保存配送日志
        WlyyPrescriptionLogDO prescriptionLog = new WlyyPrescriptionLogDO();
        prescriptionLog.setPrescriptionCode(prescriptionExpressage.getOutpatientId());
        prescriptionLog.setCreateTime(new Date());
        prescriptionLog.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.expressageing.getValue());
        prescriptionLogDao.save(prescriptionLog);
    }



    @Transactional
    public void expressage2Hospital(String hospital, String hospitalName, String hospitalAddress, String userCode, String userName, String mobile, String prescriptionCode) {
        WlyyPrescriptionExpressageDO prescriptionExpressage = prescriptionExpressageDao.findByPrescriptionCode(prescriptionCode);
        //添加配送日志
        WlyyPrescriptionExpressageLogDO prescriptionExpressageLog = new WlyyPrescriptionExpressageLogDO();
        prescriptionExpressageLog.setCreateTime(new Date());
        prescriptionExpressageLog.setPrescriptionId(prescriptionCode);

        //设置接收人
        prescriptionExpressageLog.setAcceptAddress(hospitalAddress);
        prescriptionExpressageLog.setAcceptHospital(hospital);
        prescriptionExpressageLog.setAcceptHospitalName(hospitalName);
        prescriptionExpressageLog.setAcceptTime(new Date());
        prescriptionExpressageLog.setAcceptUser(userCode);
        prescriptionExpressageLog.setAcceptUserName(userName);
        prescriptionExpressageLog.setAcceptUserMobile(mobile);
        //设置配送人
        prescriptionExpressageLog.setExpressageId(prescriptionExpressage.getId());
//        prescriptionExpressageLog.setExpressageHospitalAddress(prescriptionExpressage.getExpressageHospitalAddress());
//        prescriptionExpressageLog.setExpressageHospitalCode(prescriptionExpressage.getExpressageHospitalCode());
//        prescriptionExpressageLog.setExpressageHospitalName(prescriptionExpressage.getExpressageHospitalName());
//        prescriptionExpressageLog.setExpressageTime(prescriptionExpressage.getExpressageTime());
//        prescriptionExpressageLog.setExpressageUserCode(prescriptionExpressage.getExpressageCode());
//        prescriptionExpressageLog.setExpressageUserMobile(prescriptionExpressage.getExpressageMobile());
//        prescriptionExpressageLog.setExpressageUserName(prescriptionExpressage.getExpressageName());

        prescriptionExpressageLogDao.save(prescriptionExpressageLog);

        //判断如果服务站和患者填写的服务站一致
//        if (hospital.equals(prescriptionExpressage.getPatientHospitalCode())) {
//            // 如果是最后服务站
//            //设置配送码失效
//            prescriptionDispensaryCodeDao.updateToUse(prescriptionCode, 3);
//            //修改处方是配送完成
//            Prescription prescription = prescriptionDao.findByCode(prescriptionCode);
//            prescription.setStatus(PrescriptionLog.PrescriptionLogStatus.expressage2hospital.getValue());
//            prescription.setExpressageTime(new Date());
//        } else {
//            //如果不是重点服务站,更新配送表，更新配送信息是当前登陆人
//            prescriptionExpressage.setExpressageCode(userCode);
//            prescriptionExpressage.setExpressageName(userName);
//            prescriptionExpressage.setExpressageMobile(mobile);
//            prescriptionExpressage.setExpressageHospitalCode(hospital);
//            prescriptionExpressage.setExpressageHospitalName(hospitalName);
//            prescriptionExpressage.setExpressageHospitalAddress(hospitalAddress);
//        }
    }

    public List<Map<String, Object>> getDrugDeliveryState() {
        String sql = "SELECT * FROM system_dict t WHERE t.dict_name = 'DRUGDELIVERY_STATE' ORDER BY t.sort DESC";
        List<Map<String, Object>> rs = jdbcTemplate.queryForList(sql);
        return rs;
    }

    public Map<String, Object> getEsListByIDCardOrSSC(String keyWord, Integer page, Integer pageSize) {

        BasePatientDO p = null;
        Map<String,Object> map = new HashedMap();
        if (StringUtils.isNotBlank(keyWord)) {
            p = patientDao.findByIdcard(keyWord);
            if (p == null) {
                p = patientDao.findBySsc(keyWord);
            }
            if (p != null) {
                int start = (page - 1) * pageSize;

                String sqlTotal ="SELECT count(1) AS total" +
                        " FROM " +
                        " wlyy_prescription p " +
                        " JOIN wlyy_prescription_pay py ON py.prescription_code = p.id " +
                        " WHERE " +
                        " p.patient = '" + p.getId() + "' " +
                        " AND py.trade_status ='1' " +
                        " AND p.`status`>=50 " +
                        " ORDER BY p.`status` ASC ";
                List<Map<String, Object>> totallist = jdbcTemplate.queryForList(sqlTotal);
                Long total = (Long)totallist.get(0).get("total");

                if(total==0){
                    map.put("list",null);
                    map.put("page",page);
                    map.put("total",0);
                    map.put("records",0);
                    return map;
                }

                String sql = "SELECT " +
                        "  py.charge_no AS chargeNo, " +
                        "  p.patient_name AS patientName, " +
                        "  p.patient AS patientCode, " +
                        "  py.charge_time AS chargeTime, " +
                        "  p.dispensary_type AS dispensaryType, " +
                        "  p.`status` ," +
                        "  p.id AS prescriptionCode " +
                        " FROM " +
                        " wlyy_prescription p " +
                        " JOIN wlyy_prescription_pay py ON py.prescription_code = p.id " +
                        " WHERE " +
                        " p.patient = '" + p.getId() + "'" +
                        " AND py.trade_status ='1' " +
                        " AND p.`status`>=50 " +
                        " ORDER BY p.`status` ASC " +
                        " LIMIT " + start + "," + pageSize;
                List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);

                map.put("page",page);
                map.put("records",total);
                map.put("list",list);
                double t = (double)total;
                map.put("total",Math.ceil(t/pageSize));

                return map;
            }
        }
        return map;
    }

    public String saveDrugDelivery(String doctor, String prescriptionCode, String reasonCode, String reasonName, String reason) {
        WlyyPrescriptionDO p = prescriptionDao.findById(prescriptionCode).orElse(null);
        BaseDoctorDO d = doctorDao.findByIdAndDel(doctor);
        if (p != null) {
            p.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.finish.getValue());
//            p.setDrugDeliveryFlag(1);
//            p.setDrugDeliveryOperator(doctor);
//            p.setDrugDeliveryOperatorName(d.getName());
//            p.setDrugDeliveryReason(reason);
//            p.setDrugDeliveryReasonCode(reasonCode);
//            p.setDrugDeliveryReasonValue(reasonName);
//            p.setDrugDeliveryTime(new Date());
            p.setFinishTime(new Date());
            prescriptionDao.save(p);

            //更新日志
            WlyyPrescriptionLogDO prescriptionLog = new WlyyPrescriptionLogDO();
            prescriptionLog.setPrescriptionCode(p.getId());
            prescriptionLog.setCreateTime(new Date());
            prescriptionLog.setUserCode(doctor);
            prescriptionLog.setUserName(d.getName());
            prescriptionLog.setUserType(2);
            prescriptionLog.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.finish.getValue());
            prescriptionLogDao.save(prescriptionLog);

            try {
                //更新二维码（isUse）
                List<PrescriptionDispensaryCode> list = prescriptionDispensaryCodeDao.findByPrescriptionCode(prescriptionCode);
                if(list!=null&&list.size()>0){
                    //修改取药码code为已经使用
                    for(PrescriptionDispensaryCode prescriptionDispensaryCode:list){
                        prescriptionDispensaryCode.setIsUse(1);
                        prescriptionDispensaryCodeDao.save(prescriptionDispensaryCode);
                    }
                }
                //更新配送信息
//                WlyyPrescriptionExpressageDO prescriptionExpressage = prescriptionExpressageDao.findByPrescriptionCode(prescriptionCode);
//                if(prescriptionExpressage!=null){
//                    prescriptionExpressage.setHospitalDoctor(d.getName());
//                    prescriptionExpressage.setHospitalDoctorCode(doctor);
//                    prescriptionExpressage.setFetchingMedicineTime(new Date());
//                    prescriptionExpressageDao.save(prescriptionExpressage);
//                }
            }catch (Exception e){
                logger.info("更新二维码（isUse）,更新配送信息 异常:"+e.getMessage());
            }

            return "1";
        }
        return "0";
    }

    public List<Map<String, Object>> getPresStatesList() {
        List<Map<String, Object>> rs = new ArrayList<>();
        Map<String, Object> map1 = new HashedMap();
        map1.put("code", WlyyPrescriptionLogDO.PrescriptionLogStatus.pay_success.getValue());
        map1.put("name", WlyyPrescriptionLogDO.PrescriptionLogStatus.pay_success.getName());
        rs.add(map1);

        Map<String, Object> map2 = new HashedMap();
        map2.put("code", WlyyPrescriptionLogDO.PrescriptionLogStatus.wait_expressage.getValue());
        map2.put("name", WlyyPrescriptionLogDO.PrescriptionLogStatus.wait_expressage.getName());
        rs.add(map2);

        Map<String, Object> map3 = new HashedMap();
        map3.put("code", WlyyPrescriptionLogDO.PrescriptionLogStatus.expressageing.getValue());
        map3.put("name", WlyyPrescriptionLogDO.PrescriptionLogStatus.expressageing.getName());
        rs.add(map3);

        Map<String, Object> map4 = new HashedMap();
        map4.put("code", WlyyPrescriptionLogDO.PrescriptionLogStatus.expressage2hospital.getValue());
        map4.put("name", WlyyPrescriptionLogDO.PrescriptionLogStatus.expressage2hospital.getName());
        rs.add(map4);

        Map<String, Object> map5 = new HashedMap();
        map5.put("code", WlyyPrescriptionLogDO.PrescriptionLogStatus.finish.getValue());
        map5.put("name", WlyyPrescriptionLogDO.PrescriptionLogStatus.finish.getName());
        rs.add(map5);

        return rs;
    }

    public Map<String, Object> getPresEsPayList(String doctor,String keyWord, String state, String type, String startDate, String endDate, String operator,String flag,Integer page, Integer pageSize) {

        Map<String, Object> map = new HashedMap();
        BaseDoctorDO d = doctorDao.findByIdAndDel(doctor);
        String totalSql = "SELECT count(1) AS total " +
                " FROM " +
                " wlyy_prescription p " +
                " JOIN wlyy_prescription_pay py ON py.prescription_code = p.id " +
                " LEFT JOIN wlyy_prescription_expressage e ON p.id = e.prescription_code" +
                " WHERE" +
//                " p.hospital = '"+d.getHospital()+"' AND" +
                "  py.trade_status = '1' ";
        totalSql = setSQL( totalSql,keyWord,state,type,startDate,endDate,operator,flag);
        totalSql +=  " AND p.`status`>=50 ";
        List<Map<String, Object>> totallist = jdbcTemplate.queryForList(totalSql);
        Long total = (Long)totallist.get(0).get("total");

        if(total==0){
            map.put("list",null);
            map.put("page",page);
            map.put("total",0);
            map.put("records",0);
            return map;
        }

        int start = (page - 1) * pageSize;
        String sql = "SELECT " +
                "  py.charge_no AS chargeNo, " +
                "  p.patient_name AS patientName, " +
                "  p.patient AS patientCode, " +
                "  py.charge_time AS chargeTime, " +
                "  p.dispensary_type AS dispensaryType, " +
                "  e.hospital_doctor AS operatorName," +
                "  p.drug_delivery_flag AS drugDeliveryFlag, " +
                "  e.fetching_medicine_time AS drugDeliveryTime," +
                "  p.id AS prescriptionCode," +
                "  p.`status`, " +
                "  qrcode.img_url as img_url, " +
                "  e.province_name as provinceName, " +
                "  e.city_name as cityName, " +
                "  e.town_name as townName, " +
                "  e.street_name as streetName, " +
                "  e.mobile as expressMobile, " +
                "  e.address as address " +
                " FROM " +
                " wlyy_prescription p " +
                " JOIN wlyy_prescription_pay py ON py.prescription_code = p.id " +
                " LEFT JOIN wlyy_prescription_expressage e ON p.id = e.prescription_code" +
                " LEFT JOIN wlyy_prescription_dispensary_code qrcode on p.id = qrcode.prescription_code and qrcode.is_use=0 and qrcode.type=1" +
                " WHERE" +
//                " p.hospital = '"+d.getHospital()+"' AND" +
                "  py.trade_status = '1' ";
        sql = setSQL( sql,keyWord,state,type,startDate,endDate,operator,flag);
        sql +=  " AND p.`status`>=50 " +
                " ORDER BY py.charge_time DESC " +
                " LIMIT " + start + "," + pageSize;
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);

        map.put("page",page);
        map.put("records",total);
        map.put("list",list);
        double t = (double)total;
        map.put("total",Math.ceil(t/pageSize));

        return map;
    }

    public String setSQL(String sql,String keyWord, String state, String type, String startDate, String endDate,String operator,String flag){
        //关键字搜索
        if(StringUtils.isNotBlank(keyWord)){
            BasePatientDO p = patientDao.findByIdcard(keyWord);
            if(p!=null){
                sql += " AND p.patient = '"+p.getId()+"' ";
            }else{
                p = patientDao.findBySsc(keyWord);
                if(p!=null){
                    sql += " AND p.patient = '"+p.getId()+"' ";
                }else{
                    sql += " AND ( py.charge_no = '"+keyWord+"' OR p.patient_name ='"+keyWord+"' )";
                }
            }
        }

        if(StringUtils.isNotBlank(state)){
            sql += " AND p.status = "+state ;
        }
        if(StringUtils.isNotBlank(type)){
            sql +=" AND p.dispensary_type =" +type;
        }
        if(StringUtils.isNotBlank(startDate)){
            startDate +=" 00:00:00";
            sql += " AND py.charge_time >='"+startDate+"' ";
        }
        if(StringUtils.isNotBlank(endDate)){
            endDate +=" 23:59:59";
            sql += " AND py.charge_time <='"+endDate+"' ";
        }
        if(StringUtils.isNotBlank(operator)){
            sql += " AND e.hospital_doctor_code ='"+operator+"' ";
        }
        if(StringUtils.isNotBlank(flag)){
            sql += " AND p.drug_delivery_flag ="+flag+" ";
        }
        return sql;
    }

    public List<Map<String,Object>> getHospitalOperator(String doctor){
        BaseDoctorDO d = doctorDao.findByIdAndDel(doctor);
        String sql = "SELECT " +
                " p.hospital_doctor_code AS operator, " +
                " p.hospital_doctor AS operatorName " +
                " FROM " +
                " wlyy_prescription_expressage p " +
                " WHERE " +
//                " p.hospital_code = '"+d.getHospital()+"' AND" +
                "  p.hospital_doctor_code IS NOT NULL" +
                " AND p.hospital_doctor_code <>'' " +
                " GROUP BY " +
                " p.hospital_doctor_code ";
        List<Map<String,Object>> rs = jdbcTemplate.queryForList(sql);
        return rs;
    }

    public Map<String, Object> getPresEsList(String doctor,String keyWord, String state, String type, String startDate, String endDate, String operator,String flag,Integer page, Integer pageSize) {

        Map<String, Object> map = new HashedMap();
        BaseDoctorDO d = doctorDao.findByIdAndDel(doctor);
        String totalSql = "SELECT count(1) AS total " +
                " FROM " +
                " wlyy_prescription p " +
                " JOIN wlyy_prescription_pay py ON py.prescription_code = p.id " +
                " LEFT JOIN wlyy_prescription_expressage e ON p.id = e.prescription_code" +
                " WHERE" +
//                " p.hospital = '"+d.getHospital()+"' AND" +
                "  py.trade_status = '1' ";
        totalSql = setDrugSQL( totalSql,keyWord,state,type,startDate,endDate,operator,flag);
        totalSql +=  " AND p.`status`>=50 ";
        List<Map<String, Object>> totallist = jdbcTemplate.queryForList(totalSql);
        Long total = (Long)totallist.get(0).get("total");

        if(total==0){
            map.put("list",null);
            map.put("page",page);
            map.put("total",0);
            map.put("records",0);
            return map;
        }

        int start = (page - 1) * pageSize;
        String sql = "SELECT " +
                "  py.charge_no AS chargeNo, " +
                "  p.patient_name AS patientName, " +
                "  p.patient AS patientCode, " +
                "  py.charge_time AS chargeTime, " +
                "  p.dispensary_type AS dispensaryType, " +
                "  e.hospital_doctor AS operatorName," +
                "  p.drug_delivery_flag AS drugDeliveryFlag, " +
                "  e.fetching_medicine_time AS drugDeliveryTime," +
                "  p.id AS prescriptionCode," +
                "  p.`status` " +
                " FROM " +
                " wlyy_prescription p " +
                " JOIN wlyy_prescription_pay py ON py.prescription_code = p.id " +
                " LEFT JOIN wlyy_prescription_expressage e ON p.id = e.prescription_code" +
                " WHERE" +
//                " p.hospital = '"+d.getHospital()+"' AND" +
                "  py.trade_status = '1' ";
        sql = setDrugSQL( sql,keyWord,state,type,startDate,endDate,operator,flag);
        sql +=  " AND p.`status`>=50 " +
                " ORDER BY e.fetching_medicine_time DESC " +
                " LIMIT " + start + "," + pageSize;
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);

        map.put("page",page);
        map.put("records",total);
        map.put("list",list);
        double t = (double)total;
        map.put("total",Math.ceil(t/pageSize));

        return map;
    }

    public String setDrugSQL(String sql,String keyWord, String state, String type, String startDate, String endDate,String operator,String flag){
        //关键字搜索
        if(StringUtils.isNotBlank(keyWord)){
            BasePatientDO p = patientDao.findByIdcard(keyWord);
            if(p!=null){
                sql += " AND p.patient = '"+p.getId()+"' ";
            }else{
                p = patientDao.findBySsc(keyWord);
                if(p!=null){
                    sql += " AND p.patient = '"+p.getId()+"' ";
                }else{
                    sql += " AND ( py.charge_no = '"+keyWord+"' OR p.patient_name ='"+keyWord+"' )";
                }
            }
        }

        if(StringUtils.isNotBlank(state)){
            sql += " AND p.status = "+state ;
        }
        if(StringUtils.isNotBlank(type)){
            sql +=" AND p.dispensary_type =" +type;
        }
        if(StringUtils.isNotBlank(startDate)){
            startDate +=" 00:00:00";
            sql += " AND e.fetching_medicine_time >='"+startDate+"' ";
        }
        if(StringUtils.isNotBlank(endDate)){
            endDate +=" 23:59:59";
            sql += " AND e.fetching_medicine_time <='"+endDate+"' ";
        }
        if(StringUtils.isNotBlank(operator)){
            sql += " AND e.hospital_doctor_code ='"+operator+"' ";
        }
        if(StringUtils.isNotBlank(flag)){
            sql += " AND p.drug_delivery_flag ="+flag+" ";
        }
        return sql;
    }
}
