package com.yihu.jw.afterDiagnosis.service.prescription;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.afterDiagnosis.dao.followup.FollowUpDao;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.followup.Followup;
import com.yihu.jw.entity.hospital.mapping.PatientMappingDO;
import com.yihu.jw.entity.hospital.message.SystemMessageDO;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionDO;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionDiagnosisDO;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionExpressageDO;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionLogDO;
import com.yihu.jw.hospital.mapping.dao.PatientMappingDao;
import com.yihu.jw.hospital.message.dao.SystemMessageDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionDiagnosisDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionExpressageDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionInfoDao;
import com.yihu.jw.hospital.prescription.service.PrescriptionLogService;
import com.yihu.jw.message.service.MessageService;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.util.common.CommonUtil;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.entity.ServiceException;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.orm.jpa.JpaTransactionManager;
import org.springframework.stereotype.Service;

import java.text.DecimalFormat;
import java.util.*;

/**
 * Created by chenweida on 2017/7/27.
 */
@Service
public class WlyyPrescriptionService {

    private static Logger logger = LoggerFactory.getLogger(WlyyPrescriptionService.class);

    @Autowired
    private PrescriptionDao prescriptionDao;
    @Autowired
    private SystemMessageDao systemMessageDao;
    @Autowired
    private PrescriptionExpressageDao prescriptionExpressageDao;
    @Autowired
    private BasePatientDao patientDao;
    @Autowired
    private PrescriptionDiagnosisDao prescriptionDiagnosisDao;
    @Autowired
    private PrescriptionInfoDao prescriptionInfoDao;
    @Autowired
    private JpaTransactionManager transactionManager;
    @Autowired
    private FollowUpDao followUpDao;
    @Autowired
    private MessageService messageService;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private PrescriptionLogService payLogService;
    //健康问题 高血压
    private static final String gxy = "HP0093";
    //健康问题 糖尿病
    private static final String tnb = "HP0047";

    private static final String ot = "other";
    @Autowired
    private BaseDoctorDao doctorDao;
    @Autowired
    private PatientMappingDao patientMappingDao;

    /**
     *  获取处方信息
     * @param prescriptionCode 处方code
     * @return
     */
    public WlyyPrescriptionDO findByCode(String prescriptionCode) {
        return prescriptionDao.findById(prescriptionCode).orElse(null);
    }

    /**
     * 续方确认收药接口
     *
     * @param prescriptionCode 续方code
     * @param type 1:确认收药 2：延长收药
     * @return
     */
    public Map<String,Object> confirmReceipt(String prescriptionCode, Integer type) throws Exception{

        Map<String,Object> result = new HashedMap();
        try {
            result.put("code",200);
            result.put("msg","确认成功");

            WlyyPrescriptionDO prescription = prescriptionDao.findById(prescriptionCode).orElse(null);
            if (type == 1){
                int status = prescription.getStatus();
                if (!(status>=60 && status <100)){
                    result.put("code",-1);
                    result.put("msg","该订单未发药，不能确认收药");
                    return  result;
                }
                //直接更改状态为已完成
                prescription.setStatus(WlyyPrescriptionLogDO.PrescriptionLogStatus.finish.getValue());
                payLogService.addLog(prescription, WlyyPrescriptionLogDO.PrescriptionLogType.patientConfirm.getValue(), 1, 1);
                prescription.setFinishTime(new Date());
            }else if (type == 2){
                //不更改状态只添加延长收药时间
//                Integer extendCount = prescription.getExtendCount();
//                if (extendCount !=null && extendCount>=1){
//                    result.put("code",-1);
//                    result.put("msg","每笔订单只能延长一次");
//                   return  result;
//                }
//                if (extendCount ==null){
//                    prescription.setExtendCount(1);
//                }else {
//                    prescription.setExtendCount(prescription.getExtendCount()+1);
//                }
//                prescription.setExtendTime(new Date());
            }
            prescriptionDao.save(prescription);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }



    /**
     * 提醒健管师待取药
     * @param code
     * @param healthDoctor
     */
    public void waitExpressageMessage(String code,String healthDoctor){
        try{
            WlyyPrescriptionDO prescription = prescriptionDao.findById(code).orElse(null);
            if(WlyyPrescriptionLogDO.PrescriptionLogStatus.wait_expressage.getValue()==prescription.getStatus()){
                //配药完成之后才分配健管师，分配健管师后，提示健管师待取药
                List<SystemMessageDO> list = systemMessageDao.findByReceiverAndRelationCode(healthDoctor,code,7);
                if(list.size()==0){
                    WlyyPrescriptionExpressageDO expressage = prescriptionExpressageDao.findByPrescriptionCode(code);
                    if(StringUtils.isNotBlank(expressage.getId())){
                        String content = "您有一条新的续方订单待取药！";
                        SystemMessageDO message = new SystemMessageDO();
                        message.setCreateTime(new Date());
                        message.setIsRead("0");//设置未读
                        message.setOver("1");
                        message.setReceiver(healthDoctor);
                        message.setSender("system");
                        message.setCode(CommonUtil.getCode());
                        message.setSenderName("系统");
                        message.setTitle("居民"+prescription.getPatientName()+"的续方订单待取药");
                        message.setContent(content);
                        message.setType("7");//续方订单待取药
                        message.setReadonly(1);//是否只读消息
                        message.setDel("1");
                        message.setRelationCode(prescription.getId());
                        message.setPrescriptionStatus("0");//待取药

                        systemMessageDao.save(message);
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.info("提醒健管师待取药,消息新增失败");
        }
    }

    /**
     * 返回支付信息
     * @param cardNo 医保卡号
     * @param visitNo 挂号号
     * @return
     */
    public JSONObject getPayInfo(String cardNo,String visitNo,String recipeNo) throws Exception {

        //根据医保卡号和挂号号从基卫获取待结算明细
//        String result = jwPrescriptionService.getDispUnSettleFeeInfoList(cardNo,visitNo,recipeNo);

        String result ="{\"status\":200,\"msg\":\"查询处方结算结果列表\",\"data\":{\"CODE\":\"1\",\"returnData\":[[{\"ORG_CODE\":\"350211A1025\",\"SICK_ID\":\"5578625\",\"VISIT_NO\":\"26165592\",\"RATE_TYPE\":\"2\",\"VISIT_DEPT\":\"2\",\"VISIT_OPERATOR\":\"22\",\"INSUR_DISP_REGISTER_TYPE\":\"101\",\"INSUR_DOCTOR_RANK\":\"112\",\"VISIT_IDENTITY_CARD_NO\":\"422128195407201715\",\"SETTLE_DETAIL\":[{\"RN\":\"1\",\"APPLY_NO\":\"83222903\",\"APPLY_DEPT\":\"2\",\"APPLY_DEPT_NAME\":\"内科\",\"APPLY_OPERATOR\":\"22\",\"APPLY_OPERATOR_NAME\":\"吕一林\",\"APPLY_TIME\":\"2017/8/17 20:41:45\",\"INSUR_DISP_REGISTER_TYPE\":null,\"INSUR_DOCTOR_RANK\":null,\"APPLY_SUB_NO\":\"2\",\"SUBJECT_CLASS\":\"0101\",\"SUBJECT_CLASS_NAME\":\"西药\",\"ITEM_CODE\":\"245928\",\"ITEM_NAME\":\"缬沙坦胶囊(佳菲）\",\"ITEM_SPEC\":\"80mgx28粒/盒\",\"ITEM_PRICE\":\"17.15\",\"ITEM_QUANTITY\":\"56\",\"ITEM_UNIT\":\"213\",\"ITEM_UNIT_NAME\":\"粒\",\"INSUR_ITEM_CODE\":\"11504113234649110101\",\"INSUR_FLAG\":\"Y\",\"INSUR_INVOICE_TYPE_CODE\":\"01\",\"INSUR_INVOICE_TYPE_NAME\":\"西药费\",\"INSUR_ITEM_NAME\":\"缬沙坦\",\"INSUR_ITEM_SPEC\":\"80毫克/28粒\",\"INSUR_ITEM_UNIT_NAME\":\"粒\",\"INSUR_ITEM_PRICE\":\"0.6125\",\"FREQUENCY\":\"bid\",\"PHYSIC_DOSE\":\"80.00\",\"PHYSIC_DAY_COUNT\":\"21\",\"RECIPE_NO\":\"40886884\",\"PHYSIC_FORM\":\"Z1201\",\"PHYSIC_DOSE_UNIT\":\"mg\",\"PHYSIC_AMOUNT\":\"358400.00\",\"PHYSIC_AMOUNT_UNIT\":\"mg\",\"PHYSIC_USAGE\":\"01\",\"DAY_COUNT\":\"21\",\"COST\":\"34.30\",\"CHARGE\":\"34.30\",\"sumCost\":\"75.76\",\"feeWestCost\":\"63.76\",\"feeNostrumCost\":null,\"feeChineseCost\":null,\"feeInspectCost\":null,\"feeCheckCost\":null,\"feeOperateCost\":null,\"feeCureCost\":null},{\"RN\":\"2\",\"APPLY_NO\":\"83222903\",\"APPLY_DEPT\":\"2\",\"APPLY_DEPT_NAME\":\"内科\",\"APPLY_OPERATOR\":\"22\",\"APPLY_OPERATOR_NAME\":\"吕一林\",\"APPLY_TIME\":\"2017/8/17 20:41:45\",\"INSUR_DISP_REGISTER_TYPE\":null,\"INSUR_DOCTOR_RANK\":null,\"APPLY_SUB_NO\":\"1\",\"SUBJECT_CLASS\":\"0101\",\"SUBJECT_CLASS_NAME\":\"西药\",\"ITEM_CODE\":\"251043\",\"ITEM_NAME\":\"拉西地平分散片（贝苹）\",\"ITEM_SPEC\":\"4mg*28片/盒\",\"ITEM_PRICE\":\"39.28\",\"ITEM_QUANTITY\":\"21\",\"ITEM_UNIT\":\"216\",\"ITEM_UNIT_NAME\":\"片\",\"INSUR_ITEM_CODE\":\"11504108038084610102\",\"INSUR_FLAG\":\"Y\",\"INSUR_INVOICE_TYPE_CODE\":\"01\",\"INSUR_INVOICE_TYPE_NAME\":\"西药费\",\"INSUR_ITEM_NAME\":\"拉西地平\",\"INSUR_ITEM_SPEC\":\"4毫克/7片/4板\",\"INSUR_ITEM_UNIT_NAME\":\"片\",\"INSUR_ITEM_PRICE\":\"1.4029\",\"FREQUENCY\":\"qd\",\"PHYSIC_DOSE\":\"1.00\",\"PHYSIC_DAY_COUNT\":\"21\",\"RECIPE_NO\":\"40886884\",\"PHYSIC_FORM\":\"H0101\",\"PHYSIC_DOSE_UNIT\":\"mg\",\"PHYSIC_AMOUNT\":\"84.00\",\"PHYSIC_AMOUNT_UNIT\":\"mg\",\"PHYSIC_USAGE\":\"01\",\"DAY_COUNT\":\"21\",\"COST\":\"29.46\",\"CHARGE\":\"29.46\",\"sumCost\":null,\"feeWestCost\":null,\"feeNostrumCost\":null,\"feeChineseCost\":null,\"feeInspectCost\":null,\"feeCheckCost\":null,\"feeOperateCost\":null,\"feeCureCost\":null},{\"RN\":\"3\",\"APPLY_NO\":\"83222894\",\"APPLY_DEPT\":\"2\",\"APPLY_DEPT_NAME\":\"内科\",\"APPLY_OPERATOR\":\"22\",\"APPLY_OPERATOR_NAME\":\"吕一林\",\"APPLY_TIME\":\"2017/8/17 20:40:42\",\"INSUR_DISP_REGISTER_TYPE\":null,\"INSUR_DOCTOR_RANK\":null,\"APPLY_SUB_NO\":\"1\",\"SUBJECT_CLASS\":\"02041102\",\"SUBJECT_CLASS_NAME\":\"2.诊察费\",\"ITEM_CODE\":\"189819\",\"ITEM_NAME\":\"副主任医师（西医）诊察费\",\"ITEM_SPEC\":null,\"ITEM_PRICE\":\"12\",\"ITEM_QUANTITY\":\"1\",\"ITEM_UNIT\":\"1\",\"ITEM_UNIT_NAME\":\"次\",\"INSUR_ITEM_CODE\":\"011020000703\",\"INSUR_FLAG\":\"Y\",\"INSUR_INVOICE_TYPE_CODE\":\"06\",\"INSUR_INVOICE_TYPE_NAME\":\"诊察费\",\"INSUR_ITEM_NAME\":\"诊察费3\",\"INSUR_ITEM_SPEC\":\"无\",\"INSUR_ITEM_UNIT_NAME\":\"无\",\"INSUR_ITEM_PRICE\":\"3.0000\",\"FREQUENCY\":\"9999\",\"PHYSIC_DOSE\":\"无\",\"PHYSIC_DAY_COUNT\":\"1\",\"RECIPE_NO\":\"\",\"PHYSIC_FORM\":\"H2501\",\"PHYSIC_DOSE_UNIT\":\"*\",\"PHYSIC_AMOUNT\":\"1\",\"PHYSIC_AMOUNT_UNIT\":\"#\",\"PHYSIC_USAGE\":\"12\",\"DAY_COUNT\":\"1\",\"COST\":\"3\",\"CHARGE\":\"12\",\"sumCost\":null,\"feeWestCost\":null,\"feeNostrumCost\":null,\"feeChineseCost\":null,\"feeInspectCost\":null,\"feeCheckCost\":null,\"feeOperateCost\":null,\"feeCureCost\":null},{\"RN\":\"4\",\"APPLY_NO\":\"83222894\",\"APPLY_DEPT\":\"2\",\"APPLY_DEPT_NAME\":\"内科\",\"APPLY_OPERATOR\":\"22\",\"APPLY_OPERATOR_NAME\":\"吕一林\",\"APPLY_TIME\":\"2017/8/17 20:40:42\",\"INSUR_DISP_REGISTER_TYPE\":null,\"INSUR_DOCTOR_RANK\":null,\"APPLY_SUB_NO\":\"1\",\"SUBJECT_CLASS\":\"02041102\",\"SUBJECT_CLASS_NAME\":\"2.诊察费\",\"ITEM_CODE\":\"189819\",\"ITEM_NAME\":\"副主任医师（西医）诊察费\",\"ITEM_SPEC\":null,\"ITEM_PRICE\":\"12\",\"ITEM_QUANTITY\":\"1\",\"ITEM_UNIT\":\"1\",\"ITEM_UNIT_NAME\":\"次\",\"INSUR_ITEM_CODE\":\"011010000200\",\"INSUR_FLAG\":\"N\",\"INSUR_INVOICE_TYPE_CODE\":\"06\",\"INSUR_INVOICE_TYPE_NAME\":\"诊察费\",\"INSUR_ITEM_NAME\":\"诊察费1\",\"INSUR_ITEM_SPEC\":\"无\",\"INSUR_ITEM_UNIT_NAME\":\"无\",\"INSUR_ITEM_PRICE\":\"6.0000\",\"FREQUENCY\":\"9999\",\"PHYSIC_DOSE\":\"无\",\"PHYSIC_DAY_COUNT\":\"1\",\"RECIPE_NO\":\"\",\"PHYSIC_FORM\":\"H2501\",\"PHYSIC_DOSE_UNIT\":\"*\",\"PHYSIC_AMOUNT\":\"1\",\"PHYSIC_AMOUNT_UNIT\":\"#\",\"PHYSIC_USAGE\":\"12\",\"DAY_COUNT\":\"1\",\"COST\":\"6\",\"CHARGE\":\"12\",\"sumCost\":null,\"feeWestCost\":null,\"feeNostrumCost\":null,\"feeChineseCost\":null,\"feeInspectCost\":null,\"feeCheckCost\":null,\"feeOperateCost\":null,\"feeCureCost\":null},{\"RN\":\"5\",\"APPLY_NO\":\"83222894\",\"APPLY_DEPT\":\"2\",\"APPLY_DEPT_NAME\":\"内科\",\"APPLY_OPERATOR\":\"22\",\"APPLY_OPERATOR_NAME\":\"吕一林\",\"APPLY_TIME\":\"2017/8/17 20:40:42\",\"INSUR_DISP_REGISTER_TYPE\":null,\"INSUR_DOCTOR_RANK\":null,\"APPLY_SUB_NO\":\"1\",\"SUBJECT_CLASS\":\"02041102\",\"SUBJECT_CLASS_NAME\":\"2.诊察费\",\"ITEM_CODE\":\"189819\",\"ITEM_NAME\":\"副主任医师（西医）诊察费\",\"ITEM_SPEC\":null,\"ITEM_PRICE\":\"12\",\"ITEM_QUANTITY\":\"1\",\"ITEM_UNIT\":\"1\",\"ITEM_UNIT_NAME\":\"次\",\"INSUR_ITEM_CODE\":\"011020000100\",\"INSUR_FLAG\":\"Y\",\"INSUR_INVOICE_TYPE_CODE\":\"06\",\"INSUR_INVOICE_TYPE_NAME\":\"诊察费\",\"INSUR_ITEM_NAME\":\"普通门诊诊查费\",\"INSUR_ITEM_SPEC\":\"无\",\"INSUR_ITEM_UNIT_NAME\":\"无\",\"INSUR_ITEM_PRICE\":\"3.0000\",\"FREQUENCY\":\"9999\",\"PHYSIC_DOSE\":\"无\",\"PHYSIC_DAY_COUNT\":\"1\",\"RECIPE_NO\":\"\",\"PHYSIC_FORM\":\"H2501\",\"PHYSIC_DOSE_UNIT\":\"*\",\"PHYSIC_AMOUNT\":\"1\",\"PHYSIC_AMOUNT_UNIT\":\"#\",\"PHYSIC_USAGE\":\"12\",\"DAY_COUNT\":\"1\",\"COST\":\"3\",\"CHARGE\":\"12\",\"sumCost\":null,\"feeWestCost\":null,\"feeNostrumCost\":null,\"feeChineseCost\":null,\"feeInspectCost\":null,\"feeCheckCost\":null,\"feeOperateCost\":null,\"feeCureCost\":null}]}]],\"byRefParaData\":[{\"totalCount\":\"5\",\"errorMsg\":\"\"}]}}";

        JSONObject json = new JSONObject();
        if(StringUtils.isNotBlank(result)){

            JSONObject jwobj =  JSON.parseObject(result);
            JSONObject zyCfinfo = jwobj.getJSONObject("data");
            Integer code = zyCfinfo.getInteger("CODE");
            Integer status = jwobj.getInteger("status");

            if(200 == status && 1 == code){

                JSONArray zyCfinfoListReturnData = zyCfinfo.getJSONArray("returnData");
                
                if(zyCfinfoListReturnData.size()!=0 && !zyCfinfoListReturnData.isEmpty()){
    
                    JSONArray zyCfinfoList = zyCfinfoListReturnData.getJSONArray(0);
                    JSONArray cfInfo = new JSONArray();
                    for (int i = 0; i < zyCfinfoList.size(); i++) {
        
                        JSONObject zyCfinfoObj = zyCfinfoList.getJSONObject(i);
                        //支付明细
                        JSONArray zySettleDetailList = zyCfinfoList.getJSONObject(i).getJSONArray("SETTLE_DETAIL");
                        //根据挂号号获取处方明细
                        WlyyPrescriptionDO prescription = prescriptionDao.findByAdmNoAndRealOrder(zyCfinfoObj.getString("VISIT_NO"),recipeNo);

                        if (null==patientMappingDao.findByPatient(prescription.getPatientCode())){
                            BasePatientDO patientTmp = patientDao.findById(prescription.getPatientCode()).orElse(null);
                            if (null!=patientTmp){
                                PatientMappingDO patientMappingDO = new PatientMappingDO();
                                patientMappingDO.setPatient(patientTmp.getId());
                                patientMappingDO.setPatientName(patientTmp.getName());
                                patientMappingDO.setIdcard(patientTmp.getIdcard());
                                patientMappingDO.setMappingCode(zyCfinfoObj.getString("SICK_ID"));
                                patientMappingDO.setSource("1");
                                patientMappingDO.setCreateTime(new Date());
                                patientMappingDao.save(patientMappingDO);
                            }
                        }

                        List<WlyyPrescriptionDiagnosisDO> pds = prescriptionDiagnosisDao.findByPrescriptionId(prescription.getId(),1);
                        //诊断明细        {"zdms00":"诊断或症状描述","zdbh00":"诊断或症状编码"}
                        if (!pds.isEmpty()){
                            JSONArray zdInfo = new JSONArray();
                            for (WlyyPrescriptionDiagnosisDO pd : pds) {
                                JSONObject j1 = new JSONObject();
                                j1.put("zdms00",pd.getName());//诊断或症状描述"1型糖尿病性酮症"
                                j1.put("zdbh00",pd.getCode());//诊断或症状编码"E10.100"
                                zdInfo.add(j1);
                            }
                            json.put("zdlist",zdInfo);
                        }
        
        
                        //收费明细       {"xmgg00":"项目规格","cfhao0":"处方号","sfybxm":"是否医保项目","dcyydw":"单次用药单位","xmje00":"项目金额","xmbh00":"项目编号",
                        // "ysxm00":"医生姓名","yppl00":"药品频率","gytj00":"给药途径","ypts00":"药品天数","xmdj00":"项目单价","qyzl00":"取药总量","ylts00":"药量天数",
                        // "sptxbm":"商品条形编码","xmmc00":"项目名称","fpxmbh":"发票项目编号","xmsl00":"项目数量","ypyl00":"药品用量","qyzldw":"取药单位","xmdw00":"项目单位",
                        // "jx0000":"剂型"}
        
                        Integer cfxms0 = 0;
                        for (int j= 0; j < zySettleDetailList.size(); j++) {
                            JSONObject zySettleDetailObj = zySettleDetailList.getJSONObject(j);

                            cfxms0++;
                            JSONObject j2 = new JSONObject();
                            j2.put("xmbh00",zySettleDetailObj.getString("INSUR_ITEM_CODE"));//项目编号
                            j2.put("xmmc00",zySettleDetailObj.getString("INSUR_ITEM_NAME"));//项目名称
                            j2.put("fpxmbh",zySettleDetailObj.getString("INSUR_INVOICE_TYPE_CODE"));//发票项目编号
                            j2.put("sfybxm",zySettleDetailObj.getString("INSUR_FLAG"));//是否医保项目
            
                            Double _price = Double.parseDouble(null==zySettleDetailObj.getString("INSUR_ITEM_PRICE")?"0":zySettleDetailObj.getString("INSUR_ITEM_PRICE"));
                            DecimalFormat df = new DecimalFormat("######0.00");
                            _price = _price*100;
                            if(0.0==_price){
                                j2.put("xmdj00","");//项目单价
                            }else{
                                j2.put("xmdj00",df.format(_price));//项目单价
                            }
                            j2.put("xmdw00",zySettleDetailObj.getString("INSUR_ITEM_UNIT_NAME"));//项目单位
                            j2.put("xmgg00",zySettleDetailObj.getString("INSUR_ITEM_SPEC"));//项目规格(医保)
                            j2.put("xmsl00",zySettleDetailObj.getString("ITEM_QUANTITY"));//项目数量
            
                            Double _cost = Double.parseDouble(zySettleDetailObj.getString("COST"));
                            Integer infocost = CommonUtil.doubleToInt(_cost);
                            j2.put("xmje00",String.valueOf(infocost));//项目金额
            
                            j2.put("ysxm00",zySettleDetailObj.getString("APPLY_OPERATOR_NAME"));//医生姓名
                            j2.put("sptxbm","");//商品条形编码
                            j2.put("yppl00",zySettleDetailObj.getString("FREQUENCY"));//药品频率
                            j2.put("ypyl00",zySettleDetailObj.getString("PHYSIC_DOSE"));//药品用量
                            j2.put("cfhao0",zySettleDetailObj.getString("RECIPE_NO"));//处方号
                            j2.put("jx0000",zySettleDetailObj.getString("PHYSIC_FORM"));//剂型
                            j2.put("dcyydw",zySettleDetailObj.getString("PHYSIC_DOSE_UNIT"));//单次用药单位
                            j2.put("qyzl00",zySettleDetailObj.getString("PHYSIC_AMOUNT"));//取药总量
                            j2.put("qyzldw",zySettleDetailObj.getString("PHYSIC_AMOUNT_UNIT"));//取药单位
                            j2.put("gytj00",zySettleDetailObj.getString("PHYSIC_USAGE"));//给药途径
                            j2.put("ypts00",zySettleDetailObj.getString("DAY_COUNT"));//药品天数

                            //易联众新增字段
                            j2.put("xmrq00", zyCfinfoObj.getString("bkc014"));//实际就诊日期
                            j2.put("xmsj00", zyCfinfoObj.getString("bkc171"));//实际就诊时间

                            //新增字段 限制范围类别
                            String xzfwlb = "Y";
                            j2.put("xzfwlb",xzfwlb);
                            cfInfo.add(j2);
//                        }
                        }
        
                        //处方主信息     {"yyksbm":"医生科室编号","bcsfcs":"本次收费次数","yszjhm":"医生证件号码","ysxm00":"医生姓名","mzlb00":"门诊类别","sfydgh":"是否药店挂号",
                        // "bcghcs":"本次挂号次数","ghksmc":"挂号科室名称","cfxms0":"明细上传数量","mzlsh0":"医保挂号流水号 如果为空，将自动医保挂号并收费","ghfy00":"挂号费用",
                        // "yszjdj":"医师诊金级别","bqbm00":"病情编码","sfygwd":"是否医改网点"}
                        JSONObject jsonObject = new JSONObject();
                        jsonObject.put("yyksbm",zyCfinfoObj.getString("VISIT_DEPT"));//医生科室编号
                        jsonObject.put("bcsfcs","1");//本次收费次数
                        jsonObject.put("yszjhm",zyCfinfoObj.getString("VISIT_IDENTITY_CARD_NO"));//医生证件号码
                        jsonObject.put("ysxm00",prescription.getDoctorName());//医生姓名
                        jsonObject.put("mzlb00",zyCfinfoObj.getString("INSUR_DISP_REGISTER_TYPE"));//门诊类别---普通门诊默认为101
                        jsonObject.put("sfydgh","N");//是否药店挂号，默认为N，不是药店挂号
                        jsonObject.put("bcghcs","1");//本次挂号次数
                        jsonObject.put("ghksmc",prescription.getDeptName());//挂号科室名称
                        jsonObject.put("mzlsh0","");//医保挂号流水号
                        jsonObject.put("ghfy00","0");//挂号费用
                        jsonObject.put("bqbm00","");//病情编码---可为空，不传递
                        jsonObject.put("sfygwd","N");//是否医改网点--默认为否
                        jsonObject.put("yszjdj",zyCfinfoObj.getString("INSUR_DOCTOR_RANK"));//医师诊金级别
                        jsonObject.put("cfxms0",""+cfxms0);//明细上传数量
                        json.put("recipeContent",jsonObject);
                    }
                    json.put("mxlist",cfInfo);
                }
            }else{
                String errormessage = jwobj.getString("MESSAGE");
                throw new ServiceException(errormessage);
            }
        }
        return json;
    }

    /**
     *从待结算列表获取药品明细和总金额，更新到处方药品明细表
     *@author huangwenjie
     *@date 2017/8/16 19:15
     */
    public void getPerscriptionInfoCostFromPayInfo(String prescriptionCode){
        try {
            //根据挂号号获取处方明细
            WlyyPrescriptionDO prescription = prescriptionDao.findById(prescriptionCode).orElse(null);

            Map<String,String> zyInfoCodeAndCost = new HashMap<>();

            //根据医保卡号和挂号号从基卫获取待结算明细
//            String result = jwPrescriptionService.getDispUnSettleFeeInfoList(prescription.getSsc(),prescription.getVisitNo(),prescription.getRecipeNo());
            String result = "";
            if(StringUtils.isNotBlank(result)){

                JSONObject jwobj =  JSON.parseObject(result);
                JSONObject zyCfinfo = jwobj.getJSONObject("data");
                Integer code = zyCfinfo.getInteger("CODE");
                Integer status = jwobj.getInteger("status");

                if(200 == status && 1 == code){

                    JSONArray zyCfinfoListReturnData = zyCfinfo.getJSONArray("returnData");
                    JSONArray zyCfinfoList = zyCfinfoListReturnData.getJSONArray(0);
                    for (int i = 0; i < zyCfinfoList.size(); i++) {
                        //支付明细
                        JSONArray zySettleDetailList = zyCfinfoList.getJSONObject(i).getJSONArray("SETTLE_DETAIL");
                        for (int j= 0; j < zySettleDetailList.size(); j++) {
                            JSONObject zySettleDetailObj = zySettleDetailList.getJSONObject(j);

                            //药品编码
                            String zyinfoKey = zySettleDetailObj.getString("ITEM_CODE");
                            //总金额
                            String cost = zySettleDetailObj.getString("COST");

                            if(StringUtils.isNotBlank(zyinfoKey)){
                                zyInfoCodeAndCost.put(zyinfoKey,cost);
                            }
                        }
                    }
                }else{
                    String errormessage = jwobj.getString("MESSAGE");
                    throw new ServiceException(errormessage);
                }
            }else {
                throw new ServiceException("请求基卫接口报错，无数据返回！");
            }
        }catch (Exception e){
            logger.info("从待结算列表获取药品明细和总金额失败："+e.getMessage());
        }
    }


    public JSONObject checkRecipeNoPay(int type,String prescriptionCode) throws Exception{
//        String result = jwPrescriptionService.getDataStatus(type,prescriptionCode);
        String result = "";
        JSONObject statusObj = new JSONObject();

        if(StringUtils.isNotBlank(result)){
            JSONObject jsonobj = JSON.parseObject(result);
            JSONObject statusInfon = jsonobj.getJSONObject("data");
            Integer status = jsonobj.getInteger("status");

            if(200 == status){
                JSONArray ReturnData = statusInfon.getJSONArray("returnData");
                if(ReturnData.size()!=0 && !ReturnData.isEmpty()) {
                    JSONObject data = ReturnData.getJSONArray(0).getJSONObject(0);
                    String code = data.getString("RECIPE_STATUS_CODE");
                    //如果基卫返回回来的处方状态为“作废”，code为9的时候，修改本地数据库
                    //处方状态为“线下取消”
                    if("9".equals(code)){
                        prescriptionDao.updateStatus(prescriptionCode,WlyyPrescriptionLogDO.PrescriptionLogStatus.pay_cancle.getValue());
                        statusObj.put("status",200);
                        statusObj.put("code",9);
                        statusObj.put("message","基卫处方为作废，已修改本地数据库该处方为线下取消");
    
                        try {
                            //续方取消，随访记录也取消,这里如果出错则直接捕获，记录日志，不往外抛，避免影响长处方逻辑
                            Followup followup = followUpDao.getFollowupByPrescriptionCode(prescriptionCode);
                            if(followup !=null){
                                followup.setStatus("0");
                                followUpDao.save(followup);
                            }
                        }catch (Exception e){
                            logger.info("续方取消，随访记录同步取消失败："+e.getMessage());
                        }
                    }else{
                        statusObj.put("status",200);
                        statusObj.put("code",data.getString("RECIPE_STATUS_CODE"));
                        statusObj.put("message",data.getString("RECIPE_STATUS_NAME"));
                    }
                }
            }else if (404 == status){
                statusObj.put("status",404);
                statusObj.put("message","没有找到相应的结果");
            }else{
                statusObj.put("status",-1);
                statusObj.put("message","操作失败！");
            }
        }
        return statusObj;
    }



    /**
     * 长处方统计--长处方记录筛选
     * @param level
     * @param area
     * @param disease
     * @param status
     * @param type
     * @param startTime
     * @param endTime
     * @return
     */
    public JSONObject getPrescriptionByCondition(String level, String area, String disease, String status, String type, String startTime,String endTime,int pageNo, int pageSize){
       JSONObject jsonObject = new JSONObject();
        String sql ="SELECT" +
                " p. CODE," +
                " p.patient_name," +
                " p.doctor_name," +
                " p.hospital_name," +
                " p.dispensary_type," +
                " p. STATUS," +
                " p.create_time," +
                " GROUP_CONCAT(d.health_problem) AS health_problem," +
                " GROUP_CONCAT(d.health_problem_name) AS health_problem_name," +
                " REPLACE(FORMAT(a.total_amount / 100, 1),',','')  AS total_amount" +
                " FROM" +
                " wlyy_prescription p" +
                " LEFT JOIN wlyy_prescription_diagnosis d ON p. CODE = d.prescription_code" +
                " LEFT JOIN wlyy_prescription_pay a ON p. CODE = a.prescription_code" +
                " WHERE" +
                " IF (" +
                " p. STATUS > 50 AND a.trade_status IS NOT NULL, " +
                " a.trade_status = 1," +
                " 1 = 1" +
                ")";
        //判断疾病
        if(StringUtils.isNotBlank(disease)){
            if ("1".equals(disease)){
                disease = gxy;
            }else if ("2".equals(disease)){
                disease = tnb;
            }else if ("3".equals(disease)){
                disease = ot;
            }
            sql += " and d.health_problem ='"+disease+"' ";
        }
        //判断状态
        if ("2".equals(status)){//已完成
            sql += " AND p.`status` = "+WlyyPrescriptionLogDO.PrescriptionLogStatus.finish.getValue() ;
        }else if ("3".equals(status)){//居民取消
            sql += " AND p.`status` = "+WlyyPrescriptionLogDO.PrescriptionLogStatus.patient_canel.getValue() ;
        }else if ("4".equals(status)){//审核不通过
            sql += " AND p.`status` = "+WlyyPrescriptionLogDO.PrescriptionLogStatus.no_reviewed.getValue() ;
        }else if ("5".equals(status)){//进行中
            sql += " AND  p.`status` < " + WlyyPrescriptionLogDO.PrescriptionLogStatus.finish.getValue() +
                    " AND p.`status`>= " + WlyyPrescriptionLogDO.PrescriptionLogStatus.revieweding.getValue() ;
        }else if ("6".equals(status)){//其他原因取消
            sql += " AND p.`status` <= "+WlyyPrescriptionLogDO.PrescriptionLogStatus.pay_outtime.getValue() ;
        }
        //判断取药方式
        if (StringUtils.isNotBlank(type)){
            //类型1.自取，2.快递配送，3.健管师配送
            if("1".equals(type)){
                sql +=" AND p.dispensary_type ="+type;
            }else if("2".equals(type)){
                sql +=" AND p.dispensary_type ="+type;
            }else if("3".equals(type)){
                sql +=" AND p.dispensary_type ="+type;
            }
        }

        //市区
        if("4".equals(level)){
            //市区无过滤
            //区级
        }else if("3".equals(level)){
            sql += " AND LEFT(p.hospital, 6) = '"+area+"' ";
            //机构
        }else if("2".equals(level)){
            sql += " AND p.hospital = '"+area+"' ";
        }
        List<Map<String,Object>> resultList = new ArrayList<>();
        int start = (pageNo-1)*pageSize;
        if (StringUtils.isNotBlank(startTime) && StringUtils.isNotBlank(endTime)){
            startTime += " 00:00:00";
            endTime += " 23:59:59";
            sql += " AND p.create_time <= '" +endTime+"'"+
                    " AND p.create_time >= '"+startTime+"'";
        }

        sql +=" GROUP BY p.code ORDER  BY p.create_time DESC ";
        resultList = jdbcTemplate.queryForList(sql);
        jsonObject.put("count",resultList.size());
        double totalAmount = 0.0;
        for (Map<String,Object> map : resultList){
            if (map.get("total_amount")!=null){
                totalAmount += Double.valueOf(String.valueOf(map.get("total_amount")));
            }
        }
        DecimalFormat df = new DecimalFormat("#.0");
        jsonObject.put("totalAmount",totalAmount>0?df.format(totalAmount):0.0);
        sql += " limit ?,?";
        resultList = jdbcTemplate.queryForList(sql,new Object[]{start,pageSize});
        //针对疾病去重，left join pay表之后会出现很多条数据--zd
        for (Map<String,Object> map: resultList) {
            //疾病代码和疾病名称
            String healthProblems = String.valueOf(map.get("health_problem"));
            String resultValue = "";
            String resultNameValue = "";
            if (StringUtils.isNotBlank(healthProblems) && healthProblems.contains(",")){
                String[] healthProbleamStr = healthProblems.split(",");
                for (String healthProblemValue : healthProbleamStr) {
                    if (!resultValue.contains(healthProblemValue)){
                        resultValue +=healthProblemValue+",";
                        if (gxy.equals(healthProblemValue)){
                            resultNameValue += "高血压病,";
                        }
                        if (tnb.equals(healthProblemValue)){
                            resultNameValue += "糖尿病,";
                        }
                        if (ot.equals(healthProblemValue)){
                            resultNameValue += "其他,";
                        }
                    }
                }
                if (!StringUtils.isEmpty(resultValue) && resultValue.length()>1){
                    resultValue = resultValue.substring(0,resultValue.length()-1);
                }
                if (!StringUtils.isEmpty(resultNameValue) && resultNameValue.length()>1){
                    resultNameValue = resultNameValue.substring(0,resultNameValue.length()-1);
                }
                map.put("health_problem",resultValue);
                map.put("health_problem_name",resultNameValue);
            }
        }
        jsonObject.put("list",resultList);
        return jsonObject;
    }

    /**
     * 长处方统计--上方模糊搜索
     * @param keyWord
     * @return
     */
    public JSONObject getPrescriptionByKeyWord(String keyWord,int pageNo,int pageSize){
        JSONObject jsonObject = new JSONObject();
        String sql = "SELECT" +
                " p. CODE," +
                " p.patient_name," +
                " p.doctor_name," +
                " p.hospital_name," +
                " p.dispensary_type," +
                " p. STATUS," +
                " p.create_time," +
                " GROUP_CONCAT(d.health_problem) AS health_problem," +
                " GROUP_CONCAT(d.health_problem_name) AS health_problem_name,"+
                " TRUNCATE(a.total_amount/100,1) AS total_amount"+
                " FROM" +
                " wlyy_prescription p" +
                " LEFT JOIN wlyy_prescription_diagnosis d ON p. CODE = d.prescription_code" +
                " LEFT JOIN wlyy_prescription_pay a ON p.code = a.prescription_code" +
                " WHERE" +
                " IF(p.status>50 AND a.trade_status IS NOT NULL,a.trade_status=1,1=1) " ;
        //判断关键字
        if (StringUtils.isNotBlank(keyWord)){
            sql += " AND (p.patient_name like '%"+keyWord+"%' or p.doctor_name like '%"+keyWord+"%')";
        }
        sql += " GROUP BY p.code ORDER  BY p.create_time DESC ";
        List<Map<String,Object>> resultList = jdbcTemplate.queryForList(sql);
        jsonObject.put("count",resultList.size());
        double totalAmount = 0L;
        for (Map<String,Object> map : resultList){
            if (map.get("total_amount")!=null){
                totalAmount += Double.parseDouble(map.get("total_amount").toString());
            }
        }
        DecimalFormat df = new DecimalFormat("#.0");
        jsonObject.put("totalAmount",totalAmount>0?df.format(totalAmount):0.0);
        sql+=" limit ?,?";
        int start = (pageNo-1)*pageSize;
        resultList = jdbcTemplate.queryForList(sql,new Object[]{start,pageSize});
        jsonObject.put("list",resultList);
        return jsonObject;
    }

    /**
     * 长处方统计--点击查看详情
     * @param code
     * @return
     */
    public Map<String,Object> getPrescriptionByCode(String code){
        Map<String,Object> rsMap = new HashedMap();
        String sql ="SELECT p.code,p.patient_name,p.doctor_name,p.hospital_name,p.dispensary_type,d.health_problem " +
                "FROM wlyy_prescription p LEFT JOIN wlyy_prescription_diagnosis d ON  d.prescription_code = p.code " +
                "WHERE p.code=? ";
        List<Map<String,Object>> resultList = jdbcTemplate.queryForList(sql,new Object[]{code});
        if (resultList!=null && resultList.size()>0){
            rsMap = resultList.get(0);
        }
        return rsMap;
    }

    /**
     * 根据居民CODE，时间轴，就诊类型查询用药记录
     * @param patientCode 居民CODE
     * @param timeline 时间轴（根据传入的时间搜索前25天后25天）YYYY-MM-DD
     * @param type 1糖尿病，2高血压
     * @return
     */
    public String findPatientMedicationRecords(String patientCode, String timeline, String type,String hospital) throws Exception{

        Integer totalCount = 0;

        String RECIPE_NO = "";//符合规则基卫处方医嘱号

        BasePatientDO patient = patientDao.findById(patientCode).orElse(null);

        if(patient == null){
            throw new ServiceException("未找到居民！");
        }

        String medicationRecords = "";

        Date timeLineDate = DateUtil.strToDateShort(timeline);
        Date startTime = DateUtil.setDateTime(timeLineDate,-25);
        Date endTime = DateUtil.setDateTime(timeLineDate,25);

//        JSONObject re = jwPrescriptionService.getRecipeMasterList(patientCode,DateUtil.dateToStrShort(startTime),DateUtil.dateToStrShort(endTime),"");
        JSONObject re = new JSONObject();

        totalCount = re.getInteger("totalCount");
        if(totalCount>0){
            JSONArray pres = re.getJSONArray("list");
            for (int i = 0; i < pres.size(); i++) {
                JSONObject r = (JSONObject) pres.get(i);
                JSONArray des = r.getJSONArray("prescriptionDt");
                boolean flag = false;
                for (int j = 0; j < des.size(); j++) {
                    WlyyPrescriptionDiagnosisDO ds = des.getObject(j, WlyyPrescriptionDiagnosisDO.class);
                    if(type==null){
                        RECIPE_NO = r.getString("code");
                        break;
                    }
                    if ("1".equals(type)) {
                        //主诊断为糖尿病
                        if(tnb.equals(ds.getCode())){
                            RECIPE_NO = r.getString("code");
                            break;
                        }else{
                            continue;
                        }
                    } else  if("2".equals(type)){
                        //主诊断为高血压
                        if(gxy.equals(ds.getCode())){
                            RECIPE_NO = r.getString("code");
                            break;
                        }else{
                            continue;
                        }
                    }else{
                        continue;
                    }
                }
            }
        }

        if(org.apache.commons.lang.StringUtils.isNotBlank(RECIPE_NO)){
//            String response = jwPrescriptionService.getRecipe(RECIPE_NO,patient.getSsc(),null,hospital);
//            JSONObject jsonObject = presModeAdapter.modelToSinglePrescription(response);
            JSONObject jsonObject = new JSONObject();
            //药品
            JSONArray infos = jsonObject.getJSONArray("prescriptionInfo");

            for(int i=0;i<infos.size();i++){
                JSONObject info = infos.getJSONObject(i);
                String drugname =  info.getString("drugName");
                if(org.apache.commons.lang.StringUtils.isNotBlank(drugname)){
                    medicationRecords = medicationRecords + drugname;
                    if(i != (infos.size() -1)){
                        medicationRecords = medicationRecords + ",";
                    }
                }
            }

        }


        return medicationRecords;
    }

    
    /**
     * 修改处方收货地址
     *
     * @param code 续方code原状态为60
     * @param addressJson
     * @param addressJson
     * @return
     */
    public void updateAddress(String code, String addressJson, Integer dispensaryType) throws Exception{
        WlyyPrescriptionDO prescription = prescriptionDao.findById(code).orElse(null);
        prescription.setDispensaryType(dispensaryType);
        WlyyPrescriptionExpressageDO expressage = prescriptionExpressageDao.findByPrescriptionCode(prescription.getId());
        if (expressage == null) {
            expressage = new WlyyPrescriptionExpressageDO();
        }
//        expressage.setPrescriptionCode(prescription.getCode());
        // 续方居民信息.
        String payPatientCode = prescription.getPatientCode();

        JSONObject addressInfo = JSON.parseObject(addressJson);
    
        String phone = addressInfo.getString("phone");
        expressage.setMobile(phone);//收货人手机号码
//        配药机构信息(签约机构)
        expressage.setHospitalCode(prescription.getHospital());
        expressage.setHospitalName(prescription.getHospitalName());

        String name = addressInfo.getString("receiver");
        String proviceCode = addressInfo.getString("provinceCode");
        String proviceName = addressInfo.getString("provinceName");
        String cityCode = addressInfo.getString("cityCode");
        String cityName = addressInfo.getString("cityName");
        String townCode = addressInfo.getString("townCode");
        String townName = addressInfo.getString("townName");
        String streeCode = addressInfo.getString("streeCode");
        String streeName = addressInfo.getString("streeName");
        String address = addressInfo.getString("address");
        
        expressage.setName(name);//居民姓名
        expressage.setProvinceCode(proviceCode);//省代码
        expressage.setProvinceName(proviceName);//省名称
        expressage.setCityCode(cityCode);//市代码
        expressage.setCityName(cityName);//市名称
        expressage.setTownCode(townCode);//区code
        expressage.setTownName(townName);//区名称
        expressage.setStreetCode(streeCode);//街道code
        expressage.setStreetName(streeName);//街道名称
        expressage.setAddress(address);//居民详细地址
    
        expressage.setCreateTime(new Date());//创建时间
        expressage.setDel(1);//有效
        prescriptionExpressageDao.save(expressage);
        prescriptionDao.save(prescription);
    }
}
