package com.yihu.jw.afterDiagnosis.service.rehabilitation;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.afterDiagnosis.dao.rehabilitation.PatientDiagnosisInformationDao;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.specialist.rehabilitation.PatientDiagnosisInformationDO;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.rm.specialist.SpecialistMapping;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.mysql.query.BaseJpaService;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * Created by humingfen on 2019/1/11.
 */
@Service
@Transactional
public class DiagnosisInformationService extends BaseJpaService<PatientDiagnosisInformationDO, PatientDiagnosisInformationDao> {

    @Autowired
    private PatientDiagnosisInformationDao diagnosisInformationDao;
    @Autowired
    private BaseDoctorDao doctorDao;

    /**
     * 创建诊疗信息
     */
    public String createDiagnosisInformation(String patient, String patientName, String planId, String hospital, String hospitalName,
                                             String department, String attendingDoctorName, String admittingDiagnosis, String dischargeDiagnosis, String advice, String dischargeTime, BaseDoctorDO doctor) {
        String attendingDoctorCode = "";
        List<BaseDoctorDO> d = doctorDao.findDoctorByName(attendingDoctorName);
        if(d != null && d.size() == 1){
            attendingDoctorCode = d.get(0).getId();
        }

        PatientDiagnosisInformationDO informationDO = new PatientDiagnosisInformationDO();
        informationDO.setCreateTime(new Date());
        informationDO.setPatient(patient);
        informationDO.setPatientName(patientName);
        informationDO.setHospital(hospital);
        informationDO.setHospitalName(hospitalName);
        informationDO.setPlanId(planId);
        informationDO.setDepartment(department);
        informationDO.setAdmittingDiagnosis(attendingDoctorCode);
        informationDO.setAdmittingDiagnosis(admittingDiagnosis);
        informationDO.setDischargeDiagnosis(dischargeDiagnosis);
        informationDO.setAdvice(advice);
        informationDO.setCreateUser(doctor.getId());
        informationDO.setCreateUserName(doctor.getName());
        informationDO.setDischargeTime(DateUtil.strToDate(dischargeTime));

        diagnosisInformationDao.save(informationDO);
        return "success";
    }

    /**
     * 根据康复计划查找诊疗信息
     * @param planId
     * @return
     */
    public ObjEnvelop findDiagnosisInformationByPlanId(String planId) {
        return ObjEnvelop.getSuccess(SpecialistMapping.api_success, diagnosisInformationDao.findByPlanId(planId));
    }

    /**
     * 获取居民诊疗信息记录
     * @param patient
     * @param page
     * @param size
     * @return
     */
    public PageEnvelop findDiagnosisInformationByPatient(String patient, Integer page, Integer size) {
        PageRequest pageRequest = PageRequest.of(page, size);
        Page<PatientDiagnosisInformationDO> list = diagnosisInformationDao.findByPatient(patient,pageRequest);
        return PageEnvelop.getSuccessListWithPage(SpecialistMapping.api_success, list.getContent(), page, size,list.getTotalElements());
    }
}
