package com.yihu.jw.afterDiagnosis.service.rehabilitation;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.yihu.fastdfs.FastDFSUtil;
import com.yihu.jw.afterDiagnosis.dao.rehabilitation.FrequencyDao;
import com.yihu.jw.afterDiagnosis.dao.rehabilitation.PatientDiseaseServerDao;
import com.yihu.jw.afterDiagnosis.dao.rehabilitation.PatientMedicalRecordsRehabilitationDao;
import com.yihu.jw.afterDiagnosis.service.prescription.PrescriptionInfoService;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.doctor.dao.BaseDoctorPatientDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.doctor.BaseDoctorPatientDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.base.wx.WxWechatDO;
import com.yihu.jw.entity.followup.Followup;
import com.yihu.jw.entity.hospital.consult.WlyyHospitalSysDictDO;
import com.yihu.jw.entity.hospital.doctor.Frequency;
import com.yihu.jw.entity.hospital.message.SystemMessageDO;
import com.yihu.jw.entity.rehabilitation.RehabilitationguidancePatientDO;
import com.yihu.jw.entity.specialist.rehabilitation.PatientDiseaseServer;
import com.yihu.jw.entity.specialist.rehabilitation.PatientMedicalRecordsDO;
import com.yihu.jw.entity.specialist.rehabilitation.PatientRehabilitationPlanDO;
import com.yihu.jw.hospital.appointment.dao.SpecialDiseaseDao;
import com.yihu.jw.hospital.dict.WlyyHospitalSysDictDao;
import com.yihu.jw.hospital.message.dao.SystemMessageDao;
import com.yihu.jw.im.util.ImUtil;
import com.yihu.jw.message.service.WxMessageUtil;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.util.common.CommonUtil;
import com.yihu.jw.util.common.IdCardUtil;
import com.yihu.jw.util.common.QrcodeUtil;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.entity.ServiceException;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.io.InputStream;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by 刘文彬 on 2018/8/29.
 */
@Service
@Transactional
public class WlyyRehabilitationManageService{

    private Logger logger = LoggerFactory.getLogger(RehabilitationManageService.class);
    @Value("${demo.flag}")
    private Boolean demoFlag;
    @Autowired
    private ObjectMapper objectMapper;
    @Value("${im.data_base_name}")
    private String imDBName;
    @Autowired
    private RehabilitationInfoService rehabilitationInfoService;
    @Autowired
    private FastDFSUtil fastDFSUtil;
    @Value("${fastDFS.fastdfs_file_url}")
    private String fastdfs_file_url;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private BaseDoctorDao doctorDao;
    @Autowired
    private BasePatientDao patientDao;
    @Autowired
    private WxMessageUtil wxMessageUtil;
    @Autowired
    private BaseDoctorPatientDao doctorPatientDao;
    @Autowired
    private FrequencyDao frequencyDao;
    @Autowired
    private SpecialDiseaseDao specialDiseaseDao;
    @Autowired
    private PatientMedicalRecordsRehabilitationDao medicalRecordsRehabilitationDao;
    @Autowired
    private RehabilitationGuidanceService guidanceService;
    @Autowired
    private PatientDiseaseServerDao patientDiseaseServerDao;
    @Autowired
    private SystemMessageDao messageDao;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private WlyyHospitalSysDictDao hospitalSysDictDao;
    @Autowired
    private ImUtil imUtil;
    @Autowired
    private PrescriptionInfoService prescriptionInfoService;

    /************************************************************* start ************************************************************************/
    private String calendarPlanDetailList = "/svr-specialist/calendarPlanDetailList";//康复管理-康复计划按列表展示
    private String checkAfterQrCode="/svr-specialist/checkAfterQrCode";//居民扫码后验证是否是关联的居民扫码
    private String dailyJob = "/svr-specialist/dailyJob";//每日康复服务通知
    private String tomorrowJob ="/svr-specialist/tomorrowJob";//服务定时提醒
    private String dailyByJob ="/svr-specialist/dailyByJob";//每日康复服务提醒
    private String dailyJobReserve = "/svr-specialist/dailyJobReserve";//10天、7天、5天、当天康复服务预约复诊通知
    private String planListByPatient = "/svr-specialist/planListByPatient";//康复管理-根据居民获取康复计划
    /************************************************************* end ************************************************************************/


    /**
     *  @param doctorType 1专科2家医
     * @param level 1市2区3社区
     * @param turnDownState 下转状态 0全部;1已下转;2已接受;3未下转
     * @param area
     * @param doctor
     * @return
     */
    public JSONObject selectBySpecialistDoctor(String level, String area, String doctor, String disease, Integer page, Integer pageSize, String doctorType, String patientInfo, String filterPlan, String turnDownState){
        Integer turnDownStateTotal_0=0;//全部
        Integer turnDownStateTotal_1=0;//已下转
        Integer turnDownStateTotal_2=0;//已接受
        Integer turnDownStateTotal_3=0;//未下转
        JSONObject resultObj = new JSONObject();
//        BaseDoctorDO doctorObj = doctorDao.findByIdAndDel(doctor);
        List<Map<String, Object>> result = new ArrayList<>();
        String sqlCount = "SELECT count(distinct pr.id) total,count(distinct pr.id,IF(pr.is_manage=1,pr.is_manage,null) ) 'manageTotal' FROM" +
                " base_doctor doc,`wlyy_specialist_patient_relation` pr" +
                " LEFT JOIN base_patient p ON p.id = pr.patient ";
        String sql = "SELECT DISTINCT pr.id,p.`name`, p.id, p.idcard,p.photo, YEAR (now()) - YEAR (substring(p.idcard, 7, 8)) as age," +
                "p.sex,pr.doctor AS specialist,pr.doctor_name AS specialistName,doc.hospital AS specialHospital,doc.hospital_name AS specialHospitalName,pr.disease_code,sf.doctor_name ," +
                "sf.doctor, sf.hospital, sf.hospital_name,pr.is_manage,pr.manage_user,pr.manage_time FROM" +
                " base_doctor doc,`wlyy_specialist_patient_relation` pr" +
                " LEFT JOIN base_patient p ON p.id = pr.patient ";
        //关联居民计划表
        if("1".equals(filterPlan)){
            sql = sql + " INNER JOIN `wlyy_patient_rehabilitation_plan` plan on plan.patient = pr.patient ";
            sqlCount = sqlCount + " INNER JOIN `wlyy_patient_rehabilitation_plan` plan on plan.patient = pr.patient ";
        }
        sql =sql+  " LEFT JOIN wlyy_sign_family sf ON sf.patient=pr.patient AND sf.`status`=1 AND sf.type =2"  +
                " WHERE  doc.id = pr.doctor AND pr.sign_status > 0 AND pr.`status` = 1 ";
        sqlCount = sqlCount+  " LEFT JOIN wlyy_sign_family sf ON sf.patient=pr.patient AND sf.`status`=1 AND sf.type =2"  +
                " WHERE  doc.id = pr.doctor AND pr.sign_status > 0 AND pr.`status` = 1 ";

        String sqlCondition = "";
        if (doctorType.equalsIgnoreCase("1")){
            sqlCondition +=" AND pr.doctor = '"+doctor+"'";
        }else {
            sqlCondition +=" AND sf.doctor = '"+doctor+"' ";
        }

        if(StringUtils.isNoneBlank(patientInfo)){
            sqlCondition +=" AND (p.idcard like '%"+patientInfo+"%' OR p.`name`like '%"+patientInfo+"%') ";
        }
        if (level.equalsIgnoreCase("1")){
//            sqlCondition +=" and sf.hospital IN ( SELECT dh.`code` FROM `dm_hospital` dh where dh.city='"+area+"' and dh.del=1 )";
        }else if (level.equalsIgnoreCase("2")){
            sqlCondition +=" and sf.hospital IN ( SELECT dh.`code` FROM `dm_hospital` dh where dh.town='"+area+"' and dh.del=1 )";
        }else if (level.equalsIgnoreCase("3")){
            sqlCondition +=" and sf.hospital = '"+area+"' ";
        }
        if (StringUtils.isNoneBlank(disease)){
            sqlCondition +=" and pr.id IN (SELECT pds.specialist_relation_code FROM wlyy_patient_disease_server pds WHERE  pds.del=1 and pds.disease ='"+disease+"')";
        }
        //LIMIT+(page-1)*pageSize+","+pageSize

        sqlCount +=sqlCondition;
        List<Map<String,Object>> list = jdbcTemplate.queryForList(sqlCount);
        Map<String,Object> tmp = list.get(0);
        turnDownStateTotal_0=Integer.valueOf(tmp.get("total").toString());
        turnDownStateTotal_2+=Integer.valueOf(tmp.get("manageTotal").toString());
        turnDownStateTotal_1=turnDownStateTotal_0-turnDownStateTotal_2;

        if("1".equals(filterPlan)){
            sqlCondition += " ORDER BY pr.create_time DESC,plan.create_time desc limit "+(page-1)*pageSize+","+pageSize+" ";
        }else {
            sqlCondition += " ORDER BY pr.create_time DESC limit "+(page-1)*pageSize+","+pageSize+" ";
        }
        sql += sqlCondition;


        Map<String,Integer> countMap = new HashMap<>();
        List<Map<String,Object>> mapList = jdbcTemplate.queryForList(sql);
        for (Map<String,Object> map:mapList){
            String patient = map.get("code").toString();
            BasePatientDO patientDo = patientDao.findById(patient).orElse(null);
            Integer age = IdCardUtil.getAgeByIdcardOrBirthday(patientDo.getIdcard(),patientDo.getBirthday());
            map.put("age",age);
            String specialistRelationCode = map.get("id").toString();
            String relationManageState = map.get("is_manage")==null?"0":map.get("is_manage").toString();//管理状态
            List<PatientDiseaseServer> diseaseServers = patientDiseaseServerDao.findBySpecialistRelationCodeAndDel(specialistRelationCode,"1");
            StringBuffer diseaseName = new StringBuffer();
            for (int i=0;i<diseaseServers.size();i++){
                diseaseName.append(diseaseServers.get(i).getDiseaseName()+",");
            }
            if(StringUtils.isNoneBlank(diseaseName)){
                diseaseName.deleteCharAt(diseaseName.length()-1);
            }
            map.put("disease_name",diseaseName);

            if(countMap.containsKey(patient)){
                map.put("unCompleteCount",countMap.get(patient));
            }else {
                //计算待完成任务数
                String unCompleteCountSql = "SELECT COUNT(*) from ( SELECT COUNT(pd.id) AS total,pd.hospital_service_item_id,p.id  , COUNT( IF(pd.`status` = 1,1,null)) AS compeletTotal" +
                        " FROM wlyy_patient_rehabilitation_plan p,wlyy_rehabilitation_plan_detail pd WHERE" +
                        " p.patient = '"+patient+"' and pd.plan_id = p.id  AND  pd.frequency_code <> '' and pd.doctor='"+doctor+"' and pd.execute_time<'"+ DateUtil.getStringDateShort()+" 23:59:59' " +
                        "  GROUP BY pd.hospital_service_item_id,p.id HAVING total>compeletTotal) a";
                Integer unCompleteCount = jdbcTemplate.queryForObject(unCompleteCountSql,Integer.class);
                map.put("unCompleteCount",unCompleteCount);
                countMap.put(patient,unCompleteCount);
            }

            if(StringUtils.isBlank(relationManageState)||!relationManageState.equals("1")){//未进行管理
                map.put("turnDownState", 1);//已下转
                if(turnDownState.equals("1")||turnDownState.equals("0")){
                    result.add(map);
                }
            }
            else{
                map.put("turnDownState", 2);//已管理
                if(turnDownState.equals("2")||turnDownState.equals("0")){
                    result.add(map);
                }
            }
        }

        //家医端未下转（签约居民，但是还未下转的居民） 专科医生端未下转（与未下转列表同-去除未家签）
        if (doctorType.equalsIgnoreCase("1")){//专科
            sql = "select null as id,p.`name`,p.id,p.idcard,p.photo,YEAR (now()) - YEAR (substring(p.idcard, 7, 8)) AS age, p.sex,d.create_user AS specialist,d.create_user_name AS specialistName,\n" +
                    "doc.hospital AS specialHospital,doc.hospital_name AS specialHospitalName,s.doctor_name,s.doctor, s.hospital,s.hospital_name,null as is_manage,null as manage_user,null as manage_time \n" +
                    "from wlyy_patient_discharge d LEFT JOIN ( SELECT distinct i.patient FROM wlyy_patient_rehabilitation_plan i,wlyy_sign_family s,base_patient p  " +
                    "WHERE i.create_user='"+doctor+"' AND i. STATUS != 0 AND i.patient = s.patient AND p. CODE = i.patient AND s. STATUS = 1 AND s.type =2 AND s.expenses_status = 1 )A on d.patient =A.patient " +
                    " ,wlyy_sign_family s,base_patient p ,base_doctor doc where doc.id = d.create_user AND d.create_user='"+doctor+"' ";
            sqlCount = "select count(distinct d.patient) " +
                    "from wlyy_patient_discharge d LEFT JOIN ( SELECT distinct i.patient FROM wlyy_patient_rehabilitation_plan i,wlyy_sign_family s,base_patient p  " +
                    "WHERE i.create_user='"+doctor+"' AND i. STATUS != 0 AND i.patient = s.patient AND p. CODE = i.patient AND s. STATUS = 1 AND s.type =2 AND s.expenses_status = 1 )A on d.patient =A.patient " +
                    " ,wlyy_sign_family s,base_patient p ,base_doctor doc where doc.id = d.create_user AND d.create_user='"+doctor+"' ";
        }else{//家庭
            sql = "select null as id,p.`name`,p.id,p.idcard,p.photo,YEAR (now()) - YEAR (substring(p.idcard, 7, 8)) AS age, p.sex,d.create_user AS specialist,d.create_user_name AS specialistName,\n" +
                    "doc.hospital AS specialHospital,doc.hospital_name AS specialHospitalName,s.doctor_name,s.doctor, s.hospital,s.hospital_name,null as is_manage,null as manage_user,null as manage_time \n" +
                    "from wlyy_patient_discharge d LEFT JOIN ( SELECT distinct i.patient FROM wlyy_patient_rehabilitation_plan i,wlyy_sign_family s,base_patient p " +
                    "WHERE s.doctor='"+doctor+"' AND i. STATUS != 0 AND i.patient = s.patient AND p. CODE = i.patient AND s. STATUS = 1 AND s.type =2 AND s.expenses_status = 1 )A on d.patient =A.patient " +
                    ",wlyy_sign_family s,base_patient p ,base_doctor doc where doc.id = d.create_user AND s.doctor='"+doctor+"' ";
            sqlCount = "select count(distinct d.patient) " +
                    "from wlyy_patient_discharge d LEFT JOIN ( SELECT distinct i.patient FROM wlyy_patient_rehabilitation_plan i,wlyy_sign_family s,base_patient p " +
                    "WHERE s.doctor='"+doctor+"' AND i. STATUS != 0 AND i.patient = s.patient AND p. CODE = i.patient AND s. STATUS = 1 AND s.type =2 AND s.expenses_status = 1 )A on d.patient =A.patient " +
                    ",wlyy_sign_family s,base_patient p ,base_doctor doc where doc.id = d.create_user AND s.doctor='"+doctor+"' ";
        }
        sqlCondition="";

        if(StringUtils.isNoneBlank(patientInfo)){
            sqlCondition +=" AND (p.idcard like '%"+patientInfo+"%' OR p.`name`like '%"+patientInfo+"%') ";
        }
        if (level.equalsIgnoreCase("1")){

        }else if (level.equalsIgnoreCase("2")){
            sqlCondition +=" and s.hospital IN ( SELECT dh.`code` FROM `dm_hospital` dh where dh.town='"+area+"' and dh.del=1 )";
        }else if (level.equalsIgnoreCase("3")){
            sqlCondition +=" and s.hospital = '"+area+"' ";
        }

        if (StringUtils.isNoneBlank(disease)){
            sqlCondition +=" AND s.patient in (select patient from wlyy_patient_disease_server pds where pds.del=1 and pds.disease ='"+disease+"') ";
        }

        sql  = sql + sqlCondition ;
        sql = sql+ "and ( ISNULL(d.type) or d.type=2)  \n" +
                "and d.sign_status=1 AND d.patient = s.patient AND p. CODE = d.patient AND s. STATUS = 1 AND s.expenses_status = 1\n" +
                "AND A.patient is null GROUP BY d.patient ORDER BY d.create_time DESC limit "+(page-1)*pageSize+","+pageSize+" ";
        sqlCount = sqlCount+sqlCondition;
        sqlCount = sqlCount+ "and ( ISNULL(d.type) or d.type=2)  \n" +
                "and d.sign_status=1 AND d.patient = s.patient AND p. CODE = d.patient AND s. STATUS = 1 AND s.expenses_status = 1\n" +
                "AND A.patient is null ";

        turnDownStateTotal_3 = jdbcTemplate.queryForObject(sqlCount,Integer.class);
        turnDownStateTotal_0+=turnDownStateTotal_3;
        List<Map<String, Object>> mapList12 = jdbcTemplate.queryForList(sql);
        for (Map<String, Object> map : mapList12) {
            BasePatientDO patientDo = patientDao.findById(map.get("code").toString()).orElse(null);
            Integer age = IdCardUtil.getAgeByIdcardOrBirthday(patientDo.getIdcard(),patientDo.getBirthday());
            map.put("age",age);
            map.put("disease_name", "暂无");
            map.put("turnDownState",3);//未下
            if(turnDownState.equals("3")||turnDownState.equals("0")){
                result.add(map);
            }
        }

        //计算各数量

        JSONObject turnDownStateObj = new JSONObject();
        turnDownStateObj.put("stateAll",turnDownStateTotal_0);//全部
        turnDownStateObj.put("stateDown",turnDownStateTotal_1);//已下转
        turnDownStateObj.put("stateReceive",turnDownStateTotal_2);//已接受
        turnDownStateObj.put("stateNotDown",turnDownStateTotal_3);//未下转
        resultObj.put("showList",result);
        resultObj.put("turnDownState",turnDownStateObj);
        return resultObj;
    }


    @Transactional(propagation= Propagation.NOT_SUPPORTED)
    public List<PatientMedicalRecordsDO> getNewRehabilitation(String patient, String doctorCode) throws Exception {
        //智业中山同步病历
        String result = "";
        List<PatientMedicalRecordsDO> query = new ArrayList<>();
        BasePatientDO patient1 = patientDao.findById(patient).orElse(null);
        BaseDoctorDO doctor = doctorDao.findByIdAndDel(doctorCode);

        JSONObject jsonEvent = new JSONObject();
//        List<Map<String, String>> resultMap = patientRecordService.getAllEvent_new(patient, "2", "1", "1", "2010-10-10 00:00:00",doctor.getHospital());
        List<Map<String, String>> resultMap = new ArrayList<>();
        jsonEvent.put("data", JSON.parseArray(JSON.toJSONString(resultMap)));
        result = jsonEvent.toString();
        com.alibaba.fastjson.JSONObject rejson = com.alibaba.fastjson.JSONObject.parseObject(result);
        com.alibaba.fastjson.JSONArray temlist = rejson.getJSONArray("data");
        if (temlist.size() != 0) {//存在病历信息
            com.alibaba.fastjson.JSONObject tem = temlist.getJSONObject(0);
            /**
             * {
             "orgName": "厦门大学附属心血管病医院",
             "createTime": "2013-09-26 10:13:38",
             "patient": "386a35b8d17a47cbab6174ae7952b9b6",
             "orgCode": "182",
             "dataFrom": "1",
             "id": "9f35ff1f-f683-4f3c-a0a5-7ff8ef3df89e",
             "eventType": "2",
             "dianosis": "高血压2级",
             "eventDate": "2013-09-26 10:13:38"
             }
             */
            PatientMedicalRecordsDO pmr = new PatientMedicalRecordsDO();
            pmr.setHospitalName(tem.getString("orgName"));
            pmr.setHospital(tem.getString("orgCode"));
            if (demoFlag) {//测试获取为病历模板 hospitalcode获取为自己的code
                pmr.setHospital("ac02a9b87a3611e69f7c005056850d66");//
            }
        pmr.setDischargeDiagnosisName(tem.getString("dianosis"));
            pmr.setJwpatient(tem.getString("patient"));
        pmr.setAdmittingDiagnosisName(tem.getString("dianosis"));
            pmr.setEvent(tem.getString("id"));
            pmr.setDataFrom(tem.getString("dataFrom"));
            pmr.setAdvice("出院继续观察");
            if (tem.getString("dataFrom").equals("4"))//3中山 4智业
            {
                try {
                    String html = rehabilitationInfoService.getEmrContentByRID(tem.getString("patient"),tem.getString("id"),tem.getString("orgCode"));
                    if (StringUtils.isNoneBlank(html)){
                        List<String> tmp = CommonUtil.getTagContent(html,"xid=\"diagnosis.in\" (.*?)</TD>");
                        String diagnosisIn =tmp.size()>0?tmp.get(0):"";
                        if (StringUtils.isNotBlank(diagnosisIn)){
                            try {
                                diagnosisIn =diagnosisIn.substring(diagnosisIn.indexOf('>')+1);
                            }catch (Exception e){
                                diagnosisIn="";
                            }
                        }
                        tmp = CommonUtil.getTagContent(html,"xid=\"diagnosis.out\" (.*?)</TD>");
                        String diagnosisOut =tmp.size()>0?tmp.get(0):"";
                        if (StringUtils.isNotBlank(diagnosisOut)){
                            try {
                                diagnosisOut =diagnosisOut.substring(diagnosisOut.indexOf('>')+1);
                            }catch (Exception e){
                                diagnosisOut="";
                            }
                        }
                        if (StringUtils.isNoneBlank(diagnosisIn)){//入院诊断
                            pmr.setAdmittingDiagnosisName(diagnosisIn);
                        }
                        if (StringUtils.isNoneBlank(diagnosisOut)){//出院诊断
                            pmr.setDischargeDiagnosisName(diagnosisOut);
                        }
                    }
                }catch (Exception e){
                    e.printStackTrace();
                }
            }
//        pmr.setDiseaseCode(diseaseObj.getString("disease"));//疾病code
//        pmr.setDiseaseName(diseaseObj.getString("diseaseName"));//疾病名称
            String jsonData = JSON.toJSONString(pmr);
            //创建居民住院病历
            objectMapper.setDateFormat(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"));
            PatientMedicalRecordsDO patientMedicalRecordsDO = objectMapper.readValue(jsonData, PatientMedicalRecordsDO.class);
            PatientMedicalRecordsDO patientMedicalRecordsDO1 = rehabilitationInfoService.createMedicalRecords(doctorCode, patient, patient1.getName(), patientMedicalRecordsDO, null);
            if (patientMedicalRecordsDO1 != null) { //创建病历成功
                query.add(patientMedicalRecordsDO1);
                return query;
            }
        }
        String sql = "SELECT pmrr.* FROM wlyy_patient_medical_records_rehabilitation pmrr WHERE pmrr.patient = '"+patient+"' and (data_from=3 or  data_from=4) GROUP BY id DESC LIMIT 0,1";//find_in_set('"+diseaseCode+"',disease)
        query = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(PatientMedicalRecordsDO.class));
        return query;
    }

    public List<Map<String,Object>> selectByGuidance(String planId,String itemCode){
        String sql ="SELECT gm.* FROM wlyy_guidance_message_log gm LEFT JOIN wlyy_rehabilitation_plan_detail pd ON pd.id = gm.plan_detail_id " +
                "WHERE pd.plan_id = '"+planId+"' AND pd.hospital_service_item_id = '"+itemCode+"' ORDER BY gm.create_time DESC ";
        List<Map<String,Object>> mapList = jdbcTemplate.queryForList(sql);
        return mapList;
    }


    /**
     * 日历列表
     * @param planId 计划id
     * @param searchTask 快速查找任务（1、我的任务，2、随访，3、复诊，4、健康教育）
     * @param status 任务状态（0未完成，1已完成，2已预约）
     * @param doctorCode 登陆医生
     * @param taskExecutor 任务执行者 0全部；1我的任务 2他人任务
     */
    public ObjEnvelop calendarPlanDetailList(String patient, String planId, Integer searchTask, Integer status, String doctorCode, String executeStartTime, String executeEndTime, String taskExecutor){

        String[] planIdlist = planId.split(",");
        StringBuilder planCondition = new StringBuilder();
        planCondition .append("'"+planIdlist[0]+"'");
        for (int i=1;i<planIdlist.length;i++){
            planCondition .append(",'"+planIdlist[i]+"'");
        }
        String sql = " select d.*,DATE_FORMAT(d.execute_time,'%Y/%m/%d %H:%i') as executeTime ,i.code,i.name from wlyy_rehabilitation_plan_detail d " +
                " LEFT JOIN wlyy_rehabilitation_service_item i on i.code = d.hospital_service_item_id  " +
                " where  d.plan_id in ("+planCondition+" )AND  d.execute_time IS NOT NULL  and d.execute_time <>'' " ;
        if(searchTask!=null){
            if(searchTask==8){//
                sql+="and d.doctor='"+doctorCode+"' ";
            }else {
                sql+=" and i.code="+searchTask+" " ;
            }
        }
        if(taskExecutor.equals("1")){ //我的任务
            sql+="and d.doctor='"+doctorCode+"' ";
        }
        if(taskExecutor.equals("2")){//他人任务
            sql+="and d.doctor <>'"+doctorCode+"' ";
        }
        if (StringUtils.isNoneBlank(executeEndTime)&&StringUtils.isNoneBlank(executeStartTime)){
            sql+="and d.execute_time>='"+executeStartTime+"' and d.execute_time<='"+executeEndTime+"' ";
        }
        if(status!=null){
            sql+= "and d.status="+status;
        }
        sql +="  order by d.execute_time desc ";
        List<Map<String,Object>> rehabilitationDetailList = jdbcTemplate.queryForList(sql);

        String allPlanSql = "select  wf.id as id,wf.followup_no as followupNo,wf.followup_date as followupDate,wf.followup_plan_date as followupPlanDate,wf.followup_next_date as followupNextDate, " +
                "wf.followup_type as followupType,wf.followup_class as followupClass,wf.followup_manager_status as followupManagerStatus,wf.doctor_code as doctorCode,wf.doctor_name as doctorName, " +
                "wf.jwdoctor_code as jwdoctorCode,wf.org_code as orgCode,wf.org_name as orgName,wf.jworg_code as jworgCode,wf.patient_code as patientCode,wf.patient_name as patientName,wf.idcard, " +
                "wf.data_from as dataFrom,wf.status,wf.followup_content_phone as followupContentPhone,wf.creater,wf.sign_type as signType,wf.admin_team_code as adminTeamCode,wf.sign_code as signCode, " +
                "wf.prescription_code as prescriptionCode,wf.type from wlyy_followup wf where 1=1 and wf.status !=0 " +
                "and wf.patient_code = '"+patient+"' order by wf.followup_plan_date desc";
        List<Map<String , Object>> allPlanList = jdbcTemplate.queryForList(allPlanSql);

        return null;
    }



    public List<Map<String,Object>> calendarPlanDetailItems(String executeStartTime, String executeEndTime, String planId, Integer searchTask, Integer status, String doctorCode) throws Exception{
      String sqlCondition = "";
      if (searchTask!=null){
          if(searchTask==8){
              sqlCondition+="and d.doctor='"+doctorCode+"' ";
          }else {
              sqlCondition+=" and i.code="+searchTask+" " ;
          }
      }
      if(status!=null){
          sqlCondition  += "and d.status="+status;
      }//服务项目
      String sql = " select d.id,d.plan_id as planId,i.code,i.name,p.patient,d.frequency_code,DATE_FORMAT(d.create_time,'%Y-%m-%d %H:%i:%s') as createTime from wlyy_rehabilitation_plan_detail d " +
              " LEFT JOIN wlyy_patient_rehabilitation_plan p ON p.id=d.plan_id " +
              " LEFT JOIN wlyy_rehabilitation_service_item i on i.code = d.hospital_service_item_id  " +
              " where   d.plan_id='"+planId+"' "+sqlCondition+"  group by i.code";

      List<Map<String,Object>> mapList = jdbcTemplate.queryForList(sql);

      for (Map<String,Object> map:mapList){
          String sql1 = "SELECT d.frequency_code,d.plan_id as planId ,d.id,DATE_FORMAT(d.create_time,'%Y-%m-%d %H:%i:%s') AS createTime " +
                  "FROM wlyy_rehabilitation_plan_detail d " +
                  "WHERE d.plan_id = '"+planId+"' and d.hospital_service_item_id='"+map.get("code")+"'ORDER BY d.create_time DESC";
          List<Map<String,Object>> mapList1 = jdbcTemplate.queryForList(sql1);
          if (mapList1!=null&&mapList1.size()!=0){
              Map<String,Object> map1 = mapList1.get(0);
              map.put("frequency_code",map1.get("frequency_code"));
              map.put("planId",map1.get("planId"));
              map.put("id",map1.get("id"));
              map.put("createTime",map1.get("createTime"));
          }
      }

      logger.info("mapList:"+mapList.toString()+"sql::"+sql);
      for (Map<String,Object> map:mapList){
          if (!StringUtils.isNoneBlank(map.get("frequency_code").toString())){//
              String sqlCount = "SELECT ror.id,MAX(ror.complete_time) as date,count(*) as total FROM `wlyy_rehabilitation_operate_records` ror, " +
                      "wlyy_rehabilitation_plan_detail pd WHERE pd.id = ror.rehabilitation_detail_id " +
                      "AND pd.plan_id = '"+planId+"' AND pd.hospital_service_item_id='"+map.get("code")+"' AND ror.`status`=1 ";
              List<Map<String,Object>> mapList1 = jdbcTemplate.queryForList(sqlCount);
              int count = 0;
              if (map.get("code").toString().equalsIgnoreCase("6")){
                  String allPlanSql = "select  wf.id as id,wf.followup_no as followupNo,wf.followup_date as followupDate,wf.followup_plan_date as followupPlanDate,wf.followup_next_date as followupNextDate,\n" +
                          "wf.followup_type as followupType,wf.followup_class as followupClass,wf.followup_manager_status as followupManagerStatus,wf.doctor_code as doctorCode,wf.doctor_name as doctorName,\n" +
                          "wf.jwdoctor_code as jwdoctorCode,wf.org_code as orgCode,wf.org_name as orgName,wf.jworg_code as jworgCode,wf.patient_code as patientCode,wf.patient_name as patientName,wf.idcard,\n" +
                          "wf.data_from as dataFrom,wf.status,wf.followup_content_phone as followupContentPhone,wf.creater,wf.sign_type as signType,wf.admin_team_code as adminTeamCode,wf.sign_code as signCode,\n" +
                          "wf.prescription_code as prescriptionCode,wf.type from wlyy_followup wf where 1=1 and wf.status !=0\n" +
                          "and wf.patient_code = '"+map.get("patient")+"' order by wf.followup_plan_date desc";
                  List<Map<String , Object>> allPlanList = jdbcTemplate.queryForList(allPlanSql);
                  count = allPlanList.size();
              }

              if (map.get("code").toString().equalsIgnoreCase("7")){
                  String allPrescrSql = "SELECT p.`name`, TIMESTAMPDIFF(YEAR,p.birthday,SYSDATE()) age, p.sex, p.photo, pr.`status`, pr.`code`, \n" +
                          "LEFT(pr.create_time,19) AS createTime, pr.doctor, pr.min_drug_day AS minDrugDay, LEFT(pr.pres_create_time,19) AS presCreateTime \n" +
                          "FROM wlyy_prescription pr LEFT JOIN base_patient p ON pr.patient = p.id WHERE 1=1 AND pr.patient = '"+map.get("patient")+"' AND pr.status = 100 GROUP BY pr.`code`;";
                  List<Map<String , Object>> allPrescrList = jdbcTemplate.queryForList(allPrescrSql);
                  count = allPrescrList.size();
              }

              if (mapList1!=null && mapList1.size()!=0){
                  Map<String,Object> map1 = mapList1.get(0);
                  map.put("compeletTotal",new Integer(map1.get("total").toString()) + count );
                  map.put("firstDate",map1.get("date"));
              }

          }else {
              int codeCount = 0 ;

              if (map.get("code").toString().equalsIgnoreCase("6")){ //康复随访
                  //该居民的所有随访计划，需要同步展示到康复计划-康复随访内
                  String allPlanSql = "select  wf.id as id,wf.followup_no as followupNo,wf.followup_date as followupDate,wf.followup_plan_date as followupPlanDate,wf.followup_next_date as followupNextDate,\n" +
                          "wf.followup_type as followupType,wf.followup_class as followupClass,wf.followup_manager_status as followupManagerStatus,wf.doctor_code as doctorCode,wf.doctor_name as doctorName,\n" +
                          "wf.jwdoctor_code as jwdoctorCode,wf.org_code as orgCode,wf.org_name as orgName,wf.jworg_code as jworgCode,wf.patient_code as patientCode,wf.patient_name as patientName,wf.idcard,\n" +
                          "wf.data_from as dataFrom,wf.status,wf.followup_content_phone as followupContentPhone,wf.creater,wf.sign_type as signType,wf.admin_team_code as adminTeamCode,wf.sign_code as signCode,\n" +
                          "wf.prescription_code as prescriptionCode,wf.type from wlyy_followup wf where 1=1 and wf.status !=0\n" +
                          "and wf.patient_code = '"+map.get("patient")+"' order by wf.followup_plan_date desc";
                  List<Map<String , Object>> allPlanList = jdbcTemplate.queryForList(allPlanSql);
                  codeCount = allPlanList.size();
                  map.put("allPlanList",allPlanList);
              }

              if (map.get("code").toString().equalsIgnoreCase("7")){ //康复复诊
                  //居民的处方续方，需要同步展示到康复计划-康复复诊内
                  String allPrescrSql = "SELECT p.`name`, TIMESTAMPDIFF(YEAR,p.birthday,SYSDATE()) age, p.sex, p.photo, pr.`status`, pr.`code`, \n" +
                          "LEFT(pr.create_time,19) AS createTime, pr.doctor, pr.min_drug_day AS minDrugDay, LEFT(pr.pres_create_time,19) AS presCreateTime \n" +
                          "FROM wlyy_prescription pr LEFT JOIN base_patient p ON pr.patient = p.id WHERE 1=1 AND pr.patient = '"+map.get("patient")+"' AND pr.status = 100 GROUP BY pr.`code`;";
                  List<Map<String , Object>> allPrescrList = jdbcTemplate.queryForList(allPrescrSql);
                  codeCount = allPrescrList.size();
                  map.put("allPrescrList",allPrescrList);
              }

              String sqlCount="SELECT\n" +
                      "\tCOUNT(pd.id) AS total ,\n" +
                      "\tCOUNT(CASE WHEN pd.`status`=1 THEN 1 ELSE NULL END) AS compeletTotal\n" +
                      "FROM\n" +
                      "\twlyy_rehabilitation_plan_detail pd\n" +
                      "WHERE\n" +
                      "\t pd.plan_id = '"+planId+"' " +
                      "AND pd.hospital_service_item_id='"+map.get("code")+"' and (pd.frequency_code IS NOT NULL AND pd.frequency_code<>'')\n";

              List<Map<String,Object>> mapList1 = jdbcTemplate.queryForList(sqlCount);
              if (mapList1!=null && mapList1.size()!=0){
                  Map<String,Object> map1 = mapList1.get(0);
                  map.put("total",new Integer(map1.get("total").toString()) + codeCount);
                  map.put("compeletTotal",new Integer(map1.get("compeletTotal").toString()) + codeCount);
              }
              String dateSql = "SELECT  MAX(CASE WHEN pd.`status`=1 THEN pd.execute_time ELSE NULL END) AS firstDate FROM wlyy_rehabilitation_plan_detail pd " +
                      "WHERE  pd.plan_id = '"+planId+"' AND pd.hospital_service_item_id='"+map.get("code")+"'";
              List<Map<String,Object>> mapList2 = jdbcTemplate.queryForList(dateSql);
              if (mapList2!=null && mapList2.size()!=0){
                  Map<String,Object> map1 = mapList2.get(0);
                  map.put("firstDate",map1.get("firstDate"));
                  String condition = "";
                  if (map1.get("firstDate")!=null&&StringUtils.isNoneBlank(map1.get("firstDate").toString())){
                      condition = "and pd.execute_time >='"+map1.get("firstDate").toString()+"'";
                  }
                  String dateSql1 = "SELECT pd.execute_time as lastDate  FROM wlyy_rehabilitation_plan_detail pd " +
                          "WHERE  pd.plan_id = '"+planId+"' AND pd.hospital_service_item_id='"+map.get("code")+"' and pd.status=0  "+condition+" order by pd.execute_time ASC ";
                  List<Map<String,Object>> mapList3 = jdbcTemplate.queryForList(dateSql1);
                  if (mapList3!=null&&mapList3.size()!=0){
                      Map<String,Object> map2 = mapList3.get(0);
                      map.put("lastDate",map2.get("lastDate"));
                  }else {
                      map.put("lastDate",null);
                  }

              }

          }
          if (map.get("code").toString().equalsIgnoreCase("2"))
          {//康复咨询
              String typeSql = " and a.type in (2,18)";
              //总康复咨询数量
              String sqlTotal = "select count(*)\n" +
                      "from wlyy_consult a,wlyy_consult_team b where a.code = b.consult and a.patient = '"+map.get("patient")+"' and a.del = '1' "+typeSql+" ";
              Integer consultTotal = jdbcTemplate.queryForObject(sqlTotal,Integer.class);
              String healthCount = "SELECT * FROM wlyy_patient_reservation WHERE incidental_msg_type = 0 AND source = 4 AND patient = '"+map.get("patient")+"' ORDER BY start_time DESC";
              List<Map<String , Object>> healthList = jdbcTemplate.queryForList(healthCount);
              map.put("healthList",healthList);
              map.put("total",consultTotal + healthList.size());
              //完成康复咨询数量
              sqlTotal = "select count(*)\n" +
                      "from wlyy_consult a,wlyy_consult_team b where a.code = b.consult and a.patient = '"+map.get("patient")+"' and a.del = '1' "+typeSql+" and b.`status`=1";
              consultTotal = jdbcTemplate.queryForObject(sqlTotal,Integer.class);
              map.put("compeletTotal",consultTotal + healthList.size());
              //上次发起时间
              String sqlTemp ="select a.czrq " +
                      "from wlyy_consult a,wlyy_consult_team b where a.code = b.consult and a.patient = '"+map.get("patient")+"' and a.del = '1' "+typeSql+" order by a.czrq desc limit 1";
              List<Map<String,Object>> czrqMap = jdbcTemplate.queryForList(sqlTemp);
              if(czrqMap.size()!=0){
                  if(czrqMap.get(0).get("czrq")!=null){
                      map.put("lastDate",czrqMap.get(0).get("czrq").toString());
                  }
              }
              else{
                  map.put("lastDate",null);
              }
              //未回复康复咨询数量
              sqlTotal="SELECT count(*) FROM wlyy_consult_team a WHERE a.type = 18\n" +
                      "AND EXISTS ( SELECT DISTINCT t.id consultId FROM "+imDBName+".topics t, "+imDBName+".participants p, "+imDBName+".doctors d, "+imDBName+".sessions s \n" +
                      "WHERE p.participant_id = d.id AND t.session_id = p.session_id AND t.session_id = s.id\n" +
                      "AND s.type =18  AND t.`reply` = 0  AND a.consult = t.id\n" +
                      "and a.id in( select b.id from wlyy_consult a,wlyy_consult_team b \n" +
                      "where a.code = b.consult and a.patient = '"+map.get("patient")+"' and a.del = '1' "+typeSql+" )\n" +
                      ")";
              Integer noReplay = jdbcTemplate.queryForObject(sqlTotal,Integer.class);
              map.put("noReplay",noReplay);
          }
          if (map.get("code").toString().equalsIgnoreCase("3")){//健康检测，总的体征检测
              String createTime = map.get("createTime").toString();
              String sqlTotal = "select count(*) as total from device.wlyy_patient_health_index p where p.user ='"+map.get("patient")+"' and p.type IN(1,2) ";//and p.record_date >= '"+createTime+"'
              String sqlTemp = "select p.value1,p.value2,p.value3,p.value4,p.value5,p.value6,p.value7,p.type from device.wlyy_patient_health_index p where p.user ='"+map.get("patient")+"' and p.type IN(1,2) order by p.record_date desc limit 0,1 ";
              List<Map<String,Object>> maps = jdbcTemplate.queryForList(sqlTemp);
              List<Map<String,Object>> mapTotal = jdbcTemplate.queryForList(sqlTotal);
              if (mapTotal!=null&&mapTotal.size()!=0){
                  map.put("compeletTotal",mapTotal.get(0).get("total"));
              }
              if (maps!=null&&maps.size()!=0){
                  Map<String,Object> map1 = maps.get(0);
                  Map<String,Object> map2 = null;
                  if (maps.size()!=1){
                      map2= maps.get(1);
                  }
                  map.put("value1",map1.get("value1"));
                  map.put("value2",map1.get("value2"));
                  if (map2!=null){
                      if (Integer.parseInt(map1.get("value1").toString())>Integer.parseInt(map2.get("value1").toString())){
                          map.put("value1Flag",1);
                      }else if (Integer.parseInt(map1.get("value1").toString())==Integer.parseInt(map2.get("value1").toString())) {
                          map.put("value1Flag",0);
                      }else if  (Integer.parseInt(map1.get("value1").toString())<Integer.parseInt(map2.get("value1").toString())){
                          map.put("value1Flag",-1);
                      }
                      if (map1.get("type").toString().equalsIgnoreCase("2")){
                          if (Integer.parseInt(map1.get("value2").toString())>Integer.parseInt(map2.get("value2").toString())){
                              map.put("value2Flag",1);
                          }else if (Integer.parseInt(map1.get("value2").toString())==Integer.parseInt(map2.get("value2").toString())) {
                              map.put("value2Flag",0);
                          }else if  (Integer.parseInt(map1.get("value2").toString())<Integer.parseInt(map2.get("value2").toString())){
                              map.put("value2Flag",-1);
                          }
                      }else {
                          map.put("value2Flag",0);
                      }

                  }else {
                      map.put("value1Flag",0);
                      map.put("value2Flag",0);
                  }
                  map.put("value3",map1.get("value3"));
                  map.put("value4",map1.get("value4"));
                  map.put("value5",map1.get("value5"));
                  map.put("value6",map1.get("value6"));
                  map.put("value7",map1.get("value7"));
                  map.put("type",map1.get("type"));
              }else {
                  map.put("value1",null);
                  map.put("value2",null);
                  map.put("type",null);
              }

          }

      }

        return mapList;
    }


    public JSONObject selectByPlanDetaiId(String patient,String planId,String code,Integer page,Integer pagesize){

        String numSql = "";
        JSONObject object = new JSONObject();
        String sql ="SELECT\n" +
                " DATE_FORMAT(r.complete_time,'%Y-%m-%d %H:%i:%s') AS completeTime," +
                " '' AS followupPlanDate, " +
                " r.doctor_code AS doctorCode," +
                " r.doctor_name AS doctorName," +
                " r.`status`," +
                " d.hospital_name AS hosiptalName," +
                " d.hospital," +
                " r.id as recordId," +
                " si.`name`," +
                " si.`code`," +
                " pd.plan_id," +
                " pd.id," +
                " r.relation_code as relationCode," +
                " r.flag,1 AS flagType " +
                "FROM " +
                " wlyy_rehabilitation_operate_records r " +
                "LEFT JOIN wlyy_rehabilitation_plan_detail pd ON r.rehabilitation_detail_id = pd.id " +
                "LEFT JOIN wlyy_rehabilitation_service_item si ON si.code= pd.hospital_service_item_id " +
                "LEFT JOIN base_doctor d ON d.`code` = r.doctor_code " +
                "WHERE " +
                " pd.plan_id = '"+planId+"' " +
                " AND pd.hospital_service_item_id = '"+code+"' ";
        String sqlCount ="SELECT\n" +
                "\tCOUNT(*) as total\n" +
                "FROM\n" +
                "\twlyy_rehabilitation_operate_records r\n" +
                "LEFT JOIN wlyy_rehabilitation_plan_detail pd ON r.rehabilitation_detail_id = pd.id\n" +
                "LEFT JOIN wlyy_rehabilitation_service_item si ON si.code= pd.hospital_service_item_id\n" +
                "LEFT JOIN base_doctor d ON d.`code` = r.doctor_code\n" +
                "WHERE\n" +
                "\t pd.plan_id = '"+planId+"'\n" +
                "\t\tAND pd.hospital_service_item_id = '"+code+"'\n";

        //1.7.0.5
        if(StringUtils.isNotBlank(patient)){
            //1b9fa6e86c5e468cbd0f2aabade30c35
            //f4541d33728b48629fe75ec2139fcabb
            numSql = "select sum(total) total from (";
            if ("6".equals(code)){  //新增随访计划列表
//                sql += " UNION " +
                sql = " " +
                        " SELECT DISTINCT IFNULL(DATE_FORMAT(r.complete_time,'%Y-%m-%d %H:%i:%s'),wf.followup_date) completeTime," +
                        " wf.followup_plan_date followupPlanDate," +
                        " IFNULL(r.doctor_code,wf.doctor_code) doctorCode," +
                        " IFNULL(r.doctor_name,wf.doctor_name) doctorName," +
                        " IFNULL(r.`status`,wf.`status`) AS `status`,wf.org_name hosiptalName,wf.org_code hospital," +
                        " IFNULL(r.id,'') AS recordId,IF(r.id is null,'随访计划',IF(r.followup_detail_type=1,'社区随访（厦心）','康复随访')) AS name,wf.patient_code  code," +
                        " IFNULL(r.plan_id,'') AS plan_id,IFNULL(r.pid,wf.id) AS id, IFNULL(r.relation_code,'') AS relationCode," +
                        " IFNULL(r.flag,'') AS flag,IF(r.id is null,2,1) AS flagType " +
                        " FROM  wlyy_followup wf " +
                        " LEFT JOIN (SELECT r.*,pd.id pid,pd.plan_id,pd.followup_detail_type from wlyy_rehabilitation_operate_records r " +
                        "        LEFT JOIN wlyy_rehabilitation_plan_detail pd ON r.rehabilitation_detail_id = pd.id " +
                        "       where  pd.plan_id = '"+planId+"' " +
                        " AND pd.hospital_service_item_id = '"+code+"') r on r.relation_code=wf.id "+
                        " WHERE " +
                        "  wf.status !=0 AND wf.patient_code = '"+patient+"' ";
//                sqlCount +=" UNION SELECT " +
                sqlCount =" SELECT " +
                        " count(DISTINCT wf.id)total " +
                        " FROM " +
                        " wlyy_followup wf " +
                        " LEFT JOIN system_dict sd ON wf.followup_type = sd.CODE  " +
                        " AND sd.dict_name = 'FOLLOWUP_WAY_DICT'  " +
                        " WHERE " +
                        "  wf.patient_code = '"+patient+"'  " +
                        " AND wf.status !=0  ) total ";
            }else if ("7".equals(code)){//续方处方
                sql +=" UNION SELECT LEFT ( pr.create_time, 19 ) AS completeTime,LEFT ( pr.pres_create_time, 19 ) AS followupPlanDate ," +
                        " pr.doctor AS doctorCode, pr.doctor_name AS doctorNmae,pr.`status`," +
                        " pr.hospital_name, pr.hospital,'' AS recordId," +
                        " p.`name`, '' AS code,'' AS plan_id,pr.`code` AS id ,'' AS relationCode," +
                        " '' AS flag,3 AS flagType " +
                        "FROM  wlyy_prescription pr" +
                        " LEFT JOIN base_patient p ON pr.patient = p.id WHERE" +
                        " 1 = 1 AND pr.patient = '"+patient+"' AND pr.STATUS = '100'GROUP BY  pr.CODE ";

                sqlCount +="UNION SELECT " +
                        "count(DISTINCT pr.id)total FROM wlyy_prescription pr" +
                        " LEFT JOIN base_patient p ON pr.patient = p.id WHERE 1 = 1 " +
                        "AND pr.patient = '"+patient+"' AND pr.STATUS = '100' ) total";
            }else {
                sqlCount +=" ) total";
            }

            sqlCount = numSql.concat(sqlCount);

        }
        sql +="ORDER BY completeTime DESC limit "+(page-1)*pagesize+","+pagesize+" ";
        List<Map<String,Object>> mapList = jdbcTemplate.queryForList(sql);

        List<Map<String,Object>> maps = jdbcTemplate.queryForList(sqlCount);
        object.put("list",mapList);
        object.put("total",maps);
        object.put("page",page);
        object.put("pagesize",pagesize);
        return  object;
    }

    /**
     * 开始分页
     * @param list
     * @param pageNum 页码
     * @param pageSize 每页多少条数据
     * @return
     */
    public static List startPage(List list, Integer pageNum,
                                 Integer pageSize) {
        if (pageNum == null){
            pageNum = 1;
        }
        if (pageSize == null){
            pageSize = 10;
        }
        if (list == null) {
            return null;
        }
        if (list.size() == 0) {
            return null;
        }
        Integer count = list.size(); // 记录总数
        Integer pageCount = 0; // 页数
        if (count % pageSize == 0) {
            pageCount = count / pageSize;
        } else {
            pageCount = count / pageSize + 1;
        }
        int fromIndex = 0; // 开始索引
        int toIndex = 0; // 结束索引
        if (pageNum != pageCount) {
            fromIndex = (pageNum - 1) * pageSize;
            toIndex = fromIndex + pageSize;
        } else {
            fromIndex = (pageNum - 1) * pageSize;
            toIndex = count;
        }
        List pageList = list.subList(fromIndex, toIndex);
        return pageList;
    }

    public List<Map<String,Object>> selectPlanByPatient(String patient, Integer status, String planId,String doctor){
        if (StringUtils.isNoneBlank(planId)){
            String planSql = "select p.patient " +
                    " FROM wlyy_patient_rehabilitation_plan p where p.id='"+planId+"' ";
            List<Map<String,Object>> maps = jdbcTemplate.queryForList(planSql);
            if (maps!=null&&maps.size()!=0){
                Map<String,Object> map = maps.get(0);
                patient = map.get("patient").toString();
            }
        }

        String sql = "select d.code as doctor,p.create_time as createTime,p.id,d.photo,p.title,d.idcard,YEAR (now()) - YEAR (substring(d.idcard, 7, 8)) AS age, d.sex,p.name,p.patient,d.name as doctorName,d.dept_name as deptName,hospital_name as hospitalName,p.advice_content as adviceContent,p.medical_records_code as medicalRecordsCode,p.create_time as create_time " +
                " FROM wlyy_patient_rehabilitation_plan p " +
                " LEFT JOIN base_doctor d ON d.code = p.create_user " +
                "WHERE p.patient='"+patient+"'";
        if (status!=null){
            sql +=" and p.status ="+status;
        }
        if (StringUtils.isNoneBlank(doctor)){
            BaseDoctorDO doctor1 = doctorDao.findByIdAndDel(doctor);
            if (doctor1!=null&&doctor1.getLevel()==1){
                sql +=" and p.create_user ='"+doctor+"'";
            }
        }
    
        sql+=" union ";
    
        sql+=" SELECT d.CODE AS doctor," +
                "p.create_time AS createTime," +
                "p.id," +
                "d.photo," +
                "p.title," +
                "d.idcard,YEAR (now())-YEAR (substring(d.idcard,7,8)) AS age," +
                "d.sex,p.name," +
                "p.patient," +
                "d.NAME AS doctorName," +
                "d.dept_name AS deptName," +
                "hospital_name AS hospitalName," +
                "p.advice_content AS adviceContent," +
                "p.medical_records_code AS medicalRecordsCode," +
                "p.create_time AS create_time " +
                "FROM " +
                "wlyy_patient_rehabilitation_plan p " +
                "LEFT JOIN base_doctor d ON d.CODE=p.create_user" +
                " LEFT JOIN (" +
                "SELECT sr.patient AS patient,sr.team_code AS team_code " +
                "FROM wlyy_specialist_patient_relation sr " +
                "JOIN wlyy_doctor_special_share se ON se.special_code=sr.id " +
                "WHERE se.doctor='"+doctor+"' " +
                "AND sr.patient='"+patient+"') d " +
                "ON d.patient=p.patient WHERE " +
                "p.patient='"+patient+"' " +
                "AND d.team_code=p.team_code ";
        
        sql+=" ORDER BY create_time DESC ";
        List<Map<String,Object>> mapList = jdbcTemplate.queryForList(sql);
        for (Map<String,Object> map:mapList){

            String sqlSpecilist = "select * from wlyy_specialist_patient_relation where doctor = '"+map.get("code")+"' and patient ='"+patient+"' and sign_status=1 and status=1";
            List<Map<String,Object>> maps = jdbcTemplate.queryForList(sqlSpecilist);
            map.put("specialist",maps);
        }
        return mapList;
    }

    public List<Map<String,Object>> getPatientAllRehabilitationPlan(String patient,String doctor){
        String sql = "select p.team_code as teamCode,d.code as doctor,p.create_time as createTime,p.id,d.photo,p.title,d.idcard,YEAR (now()) - YEAR (substring(d.idcard, 7, 8)) AS age, d.sex,p.name,p.patient,p.disease,p.disease_name,d.name as doctorName,d.dept_name as deptName,hospital_name as hospitalName,p.advice_content as adviceContent,p.medical_records_code as medicalRecordsCode,p.create_time as create_time " +
                " FROM wlyy_patient_rehabilitation_plan p " +
                " LEFT JOIN base_doctor d ON d.code = p.create_user " +
                "WHERE p.patient='"+patient+"'";
        sql = sql +" union "+
                "SELECT p.team_code as teamCode,d.CODE AS doctor," +
                "p.create_time AS createTime," +
                "p.id," +
                "d.photo," +
                "p.title," +
                "d.idcard,YEAR (now())-YEAR (substring(d.idcard,7,8)) AS age," +
                "d.sex,p.name," +
                "p.patient,p.disease, " +
                "p.disease_name,"+
                "d.NAME AS doctorName," +
                "d.dept_name AS deptName," +
                "hospital_name AS hospitalName," +
                "p.advice_content AS adviceContent," +
                "p.medical_records_code AS medicalRecordsCode," +
                "p.create_time AS create_time " +
                "FROM " +
                "wlyy_patient_rehabilitation_plan p " +
                "LEFT JOIN base_doctor d ON d.CODE=p.create_user" +
                " LEFT JOIN (" +
                "SELECT sr.patient AS patient,sr.team_code AS team_code " +
                "FROM wlyy_specialist_patient_relation sr " +
                "JOIN wlyy_doctor_special_share se ON se.special_code=sr.id " +
                "WHERE sr.patient='"+patient+"') d " +
                "ON d.patient=p.patient WHERE " +
                "p.patient='"+patient+"' " +
                "AND d.team_code=p.team_code "+
                "ORDER BY create_time DESC";
        List<Map<String,Object>> mapList = jdbcTemplate.queryForList(sql);
        List<Map<String,Object>> result = new ArrayList<>();
        List<Map<String,Object>> hadManage = new ArrayList<>();
        for (Map<String,Object> mp:mapList){//获下转状态
            Date createTimeDate = DateUtil.strToDateLong(mp.get("createTime").toString());
            mp.put("createTimeDate",DateUtil.dateToStrShort(createTimeDate));
            String mpDoctor = mp.get("doctor").toString();
            String manageStatus;
            //获取最近完成服务
            sql = "select si.`code`,si.name,opr.complete_time update_time " +
                    "from wlyy_rehabilitation_plan_detail pd \n" +
                    "INNER  JOIN wlyy_rehabilitation_service_item si on si.`code` = pd.hospital_service_item_id \n" +
                    "INNER JOIN wlyy_rehabilitation_operate_records opr on opr.rehabilitation_detail_id = pd.id\n" +
                    "where plan_id='"+mp.get("id").toString()+"' and opr.`status` =1 ORDER BY opr.complete_time DESC ";
            List<Map<String,Object>> serverItems = jdbcTemplate.queryForList(sql);
            if(serverItems!=null&&serverItems.size()!=0){
                mp.put("completeServer",serverItems.get(0));
            }
            else{
                mp.put("completeServer","");
            }
            if (mp.get("teamCode")!=null){
                sql = "select is_manage from wlyy_specialist_patient_relation where  patient ='"+patient+"' and team_code='"+mp.get("teamCode").toString()+"' and sign_status>0 and status=1 ORDER BY sign_date desc limit 1";
            }
            else{
                sql = "select is_manage from wlyy_specialist_patient_relation where doctor = '"+mpDoctor+"' and patient ='"+patient+"' and sign_status>0 and status=1 ORDER BY sign_date desc limit 1";
            }
            try {
                manageStatus = jdbcTemplate.queryForObject(sql,String.class);
            }catch (Exception e){
                manageStatus="0";
            }
            if (StringUtils.isBlank(manageStatus)||manageStatus.equals("0")){//已下转,即未接收
                mp.put("turnDownState",1);
                result.add(mp);
            }
            else{//已接受
                mp.put("turnDownState",2);
                hadManage.add(mp);
            }
            //计算待完成任务数
            String unCompleteCountSql = "SELECT COUNT(*) from ( SELECT COUNT(pd.id) AS total,pd.hospital_service_item_id , COUNT( IF(pd.`status` = 1,1,null)) AS compeletTotal" +
                    " FROM wlyy_rehabilitation_plan_detail pd WHERE" +
                    " pd.plan_id = '"+mp.get("id")+"' AND  pd.doctor='"+doctor+"' and pd.execute_time<'"+DateUtil.getStringDateShort()+" 23:59:59' " +
                    "GROUP BY pd.hospital_service_item_id HAVING total>compeletTotal) a";
            Integer unCompleteCount = jdbcTemplate.queryForObject(unCompleteCountSql,Integer.class);
            mp.put("unCompleteCount",unCompleteCount);
        }
        result.addAll(hadManage);
        return  result;
    }

    public List<Map<String,Object>> selectAllCompleteServiceItemsByPatient(String patient){
        List<Map<String,Object>> resutList = new ArrayList<>();
        String sql ="select A.id 'planId',pdl.id,pdl.frequency_code,A.patient,A.`name`,si.id 'code',si.`name` 'serviceName',opr.complete_time 'czrq' from  \n" +
                "(select p.team_code as teamCode,d.code as doctor,p.create_time as createTime,p.id,p.patient,p.name " +
                "from wlyy_patient_rehabilitation_plan p\n" +
                "LEFT JOIN base_doctor d ON d.code = p.create_user\n" +
                "WHERE p.patient='"+patient+"'\n" +
                "union\n" +
                "SELECT p.team_code as teamCode,d.CODE AS doctor,p.create_time AS createTime,p.id,p.patient,p.name " +
                "from \n" +
                "wlyy_patient_rehabilitation_plan p\n" +
                "LEFT JOIN base_doctor d ON d.CODE=p.create_user\n" +
                "LEFT JOIN (\n" +
                "SELECT sr.patient AS patient,sr.team_code AS team_code \n" +
                "FROM wlyy_specialist_patient_relation sr \n" +
                "JOIN wlyy_doctor_special_share se ON se.special_code=sr.id \n" +
                "WHERE sr.patient='"+patient+"') d \n" +
                "ON d.patient=p.patient WHERE \n" +
                "p.patient='"+patient+"' \n" +
                "AND d.team_code=p.team_code \n" +
                "ORDER BY createTime DESC)A \n" +
                "INNER  JOIN wlyy_rehabilitation_plan_detail pdl ON A.id = pdl.plan_id\n" +
                "INNER JOIN wlyy_rehabilitation_service_item si on pdl.hospital_service_item_id = si.id\n" +
                "INNER JOIN wlyy_rehabilitation_operate_records opr on opr.rehabilitation_detail_id = pdl.id\n" +
                "where pdl.hospital_service_item_id <>2 and  opr.`status`=1 order by opr.complete_time desc ";
        List<Map<String,Object>> planDetailList = jdbcTemplate.queryForList(sql);
        resutList.addAll(planDetailList);
        //康复咨询
        sql= "select ct.consult,ct.type,ct.patient,ct.name,ct.czrq,ct.symptoms,top.`status`,top.end_time,top.evaluate,top.evaluate_time,top.reply,top.reply_time, '2' as 'serviceId','康复咨询' as 'serviceName'\n" +
                "from wlyy_consult_team ct\n" +
                "INNER JOIN "+imDBName+".topics top on top.id = ct.consult \n" +
                "where ct.patient='"+patient+"' and ct.type=18 and ct.del=1";
        List<Map<String,Object>> kangFuList = jdbcTemplate.queryForList(sql);
        for (Map<String,Object> map:kangFuList){
            if (map.get("status").toString().equals("0")&&map.get("reply").toString().equals("0")){
                map.put("serviceStatus","未回复");
            }
            if (map.get("status").toString().equals("0")&&map.get("reply").toString().equals("1")){
                map.put("serviceStatus","未结束");
            }
            if (map.get("status").toString().equals("10")){//&&map.get("evaluate").toString().equals("0")
                map.put("serviceStatus","已结束");
            }
        }
        resutList.addAll(kangFuList);
        Collections.sort(resutList, new Comparator<Map<String, Object>>() {
            public int compare(Map<String, Object> o1, Map<String, Object> o2) {
                Timestamp map1value = (Timestamp) o1.get("czrq");
                Timestamp  map2value = (Timestamp) o2.get("czrq");
                int sort = 1;
                if (map1value.getTime() - map2value.getTime() > 0) {
                    return sort == 1 ? -1 : 1;
                } else if (map1value.getTime() - map2value.getTime() < 0) {
                    return sort == 1 ? 1 : -1;
                } else {
                    return 0;
                }
            }
        });

        return resutList;
    }

    @Transactional
    public String manageAllPlanByPatient(String patient,String doctor){
        String sql = "SELECT re.id,re.doctor,re.doctor_name as doctor_name,pl.id planId,pl.name as patientName,pl.create_user,pl.create_user_name specialDoctorName  \n" +
                "from  wlyy_patient_rehabilitation_plan pl \n" +
                "INNER JOIN  wlyy_specialist_patient_relation re\n" +
                "on pl.patient = re.patient and pl.team_code = re.team_code\n" +
                "where   re.`status`>=0 and re.sign_status=1 \n" +
                "and  pl.patient ='"+patient+"' and (re.is_manage<>1 or re.is_manage is NULL) ";

        List<Map<String,Object>> planIds = jdbcTemplate.queryForList(sql);
        List<Map<String,Object>> planByCreate = jdbcTemplate.queryForList(sql+" GROUP BY pl.create_user");
        for (Map<String,Object> tmp:planIds){
            sql ="update wlyy_specialist_patient_relation set is_manage=1,manage_user ='"+doctor+"', manage_time ='"+DateUtil.getStringDate()+"' where id='"+tmp.get("id").toString()+"' ";
            jdbcTemplate.execute(sql);
        }
        BaseDoctorDO doctorObject = doctorDao.findByIdAndDel(doctor);//家庭医生
        //接收患者时，给服务医生群组发送消息
        List<String> patientRelationDoctor = new ArrayList<>();
        for (Map<String,Object> tmp:planByCreate){
            try {
                if(!patientRelationDoctor.contains(tmp.get("doctor").toString())){
                    guidanceService.sendSpecialistWeixinMessage(patient,tmp.get("doctor").toString(),2,null,null,null);
                    patientRelationDoctor.add(tmp.get("doctor").toString());
                }
                //患者服务医生群组发送Im消息
                JSONArray serverDoctorList =selectPlanServerDoctor(tmp.get("planId").toString(),doctor);
                int length = serverDoctorList.size();
                for (int i=0;i<length;i++){
                    JSONObject serverJson = serverDoctorList.getJSONObject(i);
                    if (serverJson.getString("type").equals("4")){
                        String sessionId = serverJson.getString("sessionId");
                        String message= tmp.get("specialDoctorName").toString()+"医生您好，我已确认接收管理患者"+tmp.get("patientName").toString()+"，谢谢。";
                        imUtil.sendImMsg(doctor,doctorObject.getName(),sessionId,"1",message,"1");
                    }
                }
            } catch (ServiceException se) {
                se.printStackTrace();
            }catch(Exception e){
                e.printStackTrace();
            }
        }
        return null;
    }

    public List<Map<String,Object>> getPatientInfo(String planId,String patient){
        String sql = "SELECT pr.id,d.code, d. NAME AS doctorName, d.hospital_name AS hospitalName,pr.health_status_code,pr.disease_code FROM wlyy_patient_rehabilitation_plan p " +
                " LEFT JOIN wlyy_specialist_patient_relation pr ON  pr.doctor = p.create_user and pr.patient = p.patient" +
                " LEFT JOIN base_doctor d ON d.code = p.create_user WHERE p.id='"+planId+"' and p.patient='"+patient+"' ";
        List<Map<String,Object>> mapList = jdbcTemplate.queryForList(sql);
        for(Map<String,Object> map:mapList){
            String doctor = map.get("code").toString();
//            SignFamily signFamily = signFamilyDao.findByPatient(patient);
//            map.put("signFamily",signFamily);
            BasePatientDO patient1 = patientDao.findById(patient).orElse(null);
            Integer age = IdCardUtil.getAgeByIdcardOrBirthday(patient1.getIdcard(),patient1.getBirthday());
            String sex = IdCardUtil.getSexForIdcard_new(patient1.getIdcard());
            map.put("patientCode", patient);
            map.put("patientName", patient1.getName());
            map.put("age",age);
            map.put("sex",sex);
            map.put("photo",patient1.getPhoto());
            map.put("mobile",patient1.getMobile());
            StringBuffer diseaseName = new StringBuffer();
//            if (map.get("health_status_code")!=null){
//                String healthStatusCode = map.get("health_status_code").toString();
//                SignPatientLabel label = signPatientLabelDao.findByLabelCodeAndLabelTypeAndStatus(healthStatusCode,"8",1);
//                if (label!=null){
//                    diseaseName.append(label.getLabelName()+",");
//                }
//            }
            String specialistRelationCode = map.get("id").toString();
            List<PatientDiseaseServer> diseaseServers = patientDiseaseServerDao.findBySpecialistRelationCodeAndDel(specialistRelationCode,"1");

            for (int i=0;i<diseaseServers.size();i++){
                diseaseName.append(diseaseServers.get(i).getDiseaseName()+",");
            }
            if(StringUtils.isNoneBlank(diseaseName)){
                diseaseName.deleteCharAt(diseaseName.length()-1);
            }
            map.put("type",diseaseName);

        }
        return mapList;
    }

    
    /**
     * 康复管理-生成康复下转专科医生二维码
     * @param doctorCode
     * @param teamCode
     * @param teamName
     * @return
     * @throws Exception
     */
    public String createSpecialDoctorQrCode(String doctorCode,String teamCode,String teamName) throws Exception{
        Map<String, Object> param = new HashedMap();
        WxWechatDO wxWechatDO = wxMessageUtil.getWxWechat();
        String fileUrl = "";
        //生成二维码图片
        String contentJsonStr="https://open.weixin.qq.com/connect/oauth2/authorize?appid="+wxWechatDO.getAppId()+"&redirect_uri="+wxWechatDO.getBaseUrl()+"%2fwx%2fhtml%2fkfgl%2fhtml%2frecover_manage.html%3FdoctorCode%3D"+doctorCode+"%26teamCode%3D"+teamCode+"%26teamName%3D"+teamName+"&response_type=code&scope=snsapi_base&state=STATE#wechat_redirect";
        InputStream ipt = QrcodeUtil.createQrcode(contentJsonStr, 300, "png");
            // 圖片列表
        List<String> tempPaths = new ArrayList<String>();
        try {
            ObjectNode imgNode = fastDFSUtil.upload(ipt, "png", "plan_service_qrcode" + System.currentTimeMillis());
            com.alibaba.fastjson.JSONObject json = com.alibaba.fastjson.JSONObject.parseObject(imgNode.toString());
            tempPaths.add(json.getString("fid"));
            String urls = "";
            for (String image : tempPaths) {
                if (urls.length() == 0) {
                    urls = image;
                } else {
                    urls += "," + image;
                }
            }
            fileUrl = fastdfs_file_url + urls;
        } catch (Exception e) {
            e.printStackTrace();
        }
        System.out.println("服务码-----------1"+fileUrl);
        param.put("doctorCode", doctorCode);
        param.put("imageUrl",fileUrl);
        return fileUrl;
    }

    public String createServiceQrCode(String planDetailId,String doctorCode) throws Exception{
        Map<String, Object> param = new HashedMap();
        String fileUrl = "";
        String sql ="SELECT service_qr_code FROM wlyy_rehabilitation_plan_detail where id='"+planDetailId+"'";
        List<Map<String,Object>> list = jdbcTemplate.queryForList(sql);
        if(list!=null && list.size()>0){
            fileUrl = String.valueOf(list.get(0).get("service_qr_code"));
        }
        if (StringUtils.isEmpty(fileUrl) || "null".equals(fileUrl)){
            //生成二维码图片
            WxWechatDO wxWechatDO = wxMessageUtil.getWxWechat();
            String contentJsonStr="https://open.weixin.qq.com/connect/oauth2/authorize?appid="+wxWechatDO.getAppId()+"&redirect_uri="+wxWechatDO.getBaseUrl()+"/wx/html/kfgl/html/confirm-service.html"+"?paramStr="+planDetailId+","+doctorCode+"&response_type=code&scope=snsapi_base&state=STATE#wechat_redirect";
            InputStream ipt = QrcodeUtil.createQrcode(contentJsonStr, 300, "png");
            // 圖片列表
            List<String> tempPaths = new ArrayList<String>();
            try {
                ObjectNode imgNode = fastDFSUtil.upload(ipt, "png", "plan_service_qrcode" + System.currentTimeMillis());
                com.alibaba.fastjson.JSONObject json = com.alibaba.fastjson.JSONObject.parseObject(imgNode.toString());
                tempPaths.add(json.getString("fid"));
                String urls = "";
                for (String image : tempPaths) {
                    if (urls.length() == 0) {
                        urls = image;
                    } else {
                        urls += "," + image;
                    }
                }
                fileUrl = fastdfs_file_url + urls;
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        //更新到康复计划居民关系表中
        String sqlupd = "update wlyy_rehabilitation_plan_detail set service_qr_code='" + fileUrl + "' where id='" + planDetailId + "'";
        jdbcTemplate.update(sqlupd);

        return fileUrl;
    }

    public Integer updateRelationCodeByDetailId(String planDetailId,String relationCode) throws Exception{
        String sql ="update wlyy_rehabilitation_plan_detail t set t.relation_code="+relationCode+" where t.id='"+planDetailId+"'";
        return  jdbcTemplate.update(sql);
    }

    @Transactional(propagation= Propagation.NOT_SUPPORTED)
    public JSONArray selectPlanServerDoctor(String planId,String doctorCode) throws Exception{
        String sql ="SELECT t.patient,t.create_user FROM wlyy_patient_rehabilitation_plan t  where t.id='"+planId+"'";//专科
        List<Map<String,Object>> mapList = jdbcTemplate.queryForList(sql);
        JSONArray array = new JSONArray();
        String patient = "";
        for (Map<String,Object> map:mapList){
            String doctor =map.get("create_user").toString();
            patient = map.get("patient").toString();
            JSONObject object = new JSONObject();
            BaseDoctorDO doctor1 = doctorDao.findByIdAndDel(doctor);
            object.put("code",doctor1.getId());
            object.put("name",doctor1.getName());
            object.put("level",doctor1.getLevel());
//            object.put("hospital",doctor1.getHospital());
//            object.put("hospitalName",doctor1.getHospitalName());
            if (doctor1.getLevel()==1){
                object.put("levelName","专科医生");
            }else if (doctor1.getLevel()==2){
                object.put("levelName","全科医生");
            }

            object.put("photo",doctor1.getPhoto());
            object.put("idcard",doctor1.getIdcard());
            array.add(object);
        }
        BaseDoctorPatientDO signFamily = doctorPatientDao.findOneByPatientCode(1,patient);
        if (signFamily!=null){
            JSONObject object = new JSONObject();
            BaseDoctorDO doctor = doctorDao.findByIdAndDel(signFamily.getDoctorCode());
            object.put("code",doctor.getId());
            object.put("name",doctor.getName());
            object.put("level",doctor.getLevel());
            if (doctor.getLevel()==1){
                object.put("levelName","专科医生");
            }else if (doctor.getLevel()==2){
                object.put("levelName","全科医生");
            }
            object.put("photo",doctor.getPhoto());
            object.put("idcard",doctor.getIdcard());
//            object.put("hospital",doctor.getHospital());
//            object.put("hospitalName",doctor.getHospitalName());
            array.add(object);
        }
        StringBuilder serverGroupSessionId =new StringBuilder();//群聊session_id
        StringBuilder serverGroupSessionName =new StringBuilder();//群聊名称
        JSONObject participants = new JSONObject();//群聊成员
        for(Object object:array){//获取服务医生P2P 未读消息以及session_id。
            JSONObject jsonObject = (JSONObject)object;
            serverGroupSessionId.append(jsonObject.getString("code")+"_");
            serverGroupSessionName.append(jsonObject.getString("name")+"、");
            participants.put(jsonObject.getString("code"),0);
            if(jsonObject.getString("code").equals(doctorCode)){//
                jsonObject.put("unRead",0);
                jsonObject.put("sessionId","");
                jsonObject.put("type","");
                jsonObject.put("business_type","");
                continue;
            }
            //服务医生间的消息获取
            com.alibaba.fastjson.JSONObject participantsP2P = new com.alibaba.fastjson.JSONObject();
            participantsP2P.put(doctorCode,0);
            participantsP2P.put(jsonObject.getString("code"),0);
            //获取登录医生与其他服务医生的session_id
            //{"data":{"business_type":1,"name":"余研","id":"3438fdc6c1082c7b3c3e93dc27da244b59b694dc","type":"2","create_date":1603436112458},"status":200}
            com.alibaba.fastjson.JSONObject result = imUtil.createSession(participantsP2P,"2",jsonObject.getString("name"),null);
            jsonObject.put("sessionId",result.getJSONObject("data").getString("id"));
            jsonObject.put("type",result.getJSONObject("data").getString("type"));
            jsonObject.put("business_type",result.getJSONObject("data").getInteger("business_type"));
            //获取未读消息
            result  = imUtil.getSessionUnreadMessageCount(jsonObject.getString("sessionId"),doctorCode);
            jsonObject.put("unRead",result.getInteger("count"));
        }
        //服务医生群聊数据获取
        JSONObject groupObject = new JSONObject();
        if(StringUtils.isNoneBlank(serverGroupSessionId)&&StringUtils.isNoneBlank(serverGroupSessionName)){
            serverGroupSessionId.append("server_group");
            serverGroupSessionName=serverGroupSessionName.deleteCharAt(serverGroupSessionName.length()-1);
            if(imUtil.sessionIsExist(serverGroupSessionId+"")){//会话存在
                JSONObject result  = imUtil.getSessionUnreadMessageCount(serverGroupSessionId+"",doctorCode);
                groupObject.put("unRead",result.getInteger("count"));
            }
            else{
                JSONObject result = imUtil.createSession(participants,"4",serverGroupSessionName+"",serverGroupSessionId+"");
                groupObject.put("unRead",0);
            }
            groupObject.put("name",serverGroupSessionName);
            groupObject.put("sessionId",serverGroupSessionId.toString());
            groupObject.put("business_type",1);
            groupObject.put("type","4");
        }
        array.add(groupObject);
        return  array;
    }


    public List<PatientMedicalRecordsDO> selectByMedicalCode(String code){
        String str[] = code.split(",");
        List<PatientMedicalRecordsDO> medicalRecordsDOS= new ArrayList<>();
        for (int i=0;i<str.length;i++){
            PatientMedicalRecordsDO medicalRecordsDO = medicalRecordsRehabilitationDao.findByCode(str[i]);
            medicalRecordsDOS.add(medicalRecordsDO);
        }

        return medicalRecordsDOS;

    }
    
    public List<PatientMedicalRecordsDO> findByPatient(String patient){
        return medicalRecordsRehabilitationDao.findByPatient(patient);
    }

    public List<PatientMedicalRecordsDO> findByPatientPlan(String patient , String diseaseCode, Integer pageIndex,Integer pageSize){
        Integer page = (pageIndex-1) * pageSize;
        String sql = "SELECT pmrr.* FROM wlyy_patient_medical_records_rehabilitation pmrr,wlyy_patient_rehabilitation_plan rp\n" +
                "WHERE pmrr.patient = '"+patient+"' AND find_in_set('"+diseaseCode+"',rp.disease) AND pmrr.code = rp.medical_records_code ORDER BY pmrr.create_time DESC LIMIT "+page+","+pageSize+" ";//find_in_set('"+diseaseCode+"',disease)
        List<PatientMedicalRecordsDO> query = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(PatientMedicalRecordsDO.class));
        return query;
    }

    public List<PatientRehabilitationPlanDO> getHealthPlan(String medicalRecordsCode){
        String sql = "SELECT * FROM wlyy_patient_rehabilitation_plan WHERE medical_records_code ='"+medicalRecordsCode+"'";
        List<PatientRehabilitationPlanDO> query = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(PatientRehabilitationPlanDO.class));
        return query;
    }


    public List<HashMap<String,Object>> getDiseaseList(String patient){
       String sql = "SELECT rp.*\n" +
               "FROM wlyy_patient_medical_records_rehabilitation pmrr,\n" +
               "wlyy_patient_rehabilitation_plan rp  \n" +
               "WHERE pmrr.patient = '"+patient+"' \n" +
               "AND pmrr.code = rp.medical_records_code \n" +
               "AND rp.disease IS NOT NULL " +
               "order by rp.create_time desc";
        List<PatientRehabilitationPlanDO> list = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(PatientRehabilitationPlanDO.class));
        List<HashMap<String,Object>> mapList = new ArrayList<>();
        if(list.size()>0){
            Integer size = 0;
            for (int i=0;i<list.size();i++){
                String[] disease = list.get(i).getDisease().split(",");
                String[] diseaseName = list.get(i).getDiseaseName().split(",");
                for (int j=0;j<disease.length;j++){
                    HashMap<String,Object> map = new HashMap<>();
                    map.put("disease",disease[j]);
                    if (disease[j].equals("14")){
                        map.put("diseaseName","胃肠病");
                    }else if (disease[j].equals("10")){
                        map.put("diseaseName","前列腺病");
                    }else if (disease[j].equals("13")){
                        map.put("diseaseName","乳腺疾病");
                    }else if (disease[j].equals("17")){
                        map.put("diseaseName","妇科病");
                    }else if (disease[j].equals("15")){
                        map.put("diseaseName","慢阻肺");
                    }else {
                        map.put("diseaseName",diseaseName[j]);
                    }

                    mapList.add(map);
                }
            }
        }
       return mapList;
    }



    public List<PatientRehabilitationPlanDO> getRehabilitationPlanList(String patient, String diseaseCode){
        String sql = "SELECT rp.*,d.hospital,d.hospital_name FROM wlyy_patient_rehabilitation_plan rp,base_doctor d  WHERE patient = '"+patient+"' \n" +
                "AND find_in_set('"+diseaseCode+"',disease) AND rp.create_user=d.`code` ORDER BY create_time DESC ";//DESC LIMIT 0,10
        return  jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(PatientRehabilitationPlanDO.class));
    }

    public List<PatientRehabilitationPlanDO> getRehabilitationPlanDetail(String planId){
        String sql = "SELECT * FROM wlyy_patient_rehabilitation_plan WHERE id = '"+planId+"'";
        return  jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(PatientRehabilitationPlanDO.class));
    }

    public List<RehabilitationguidancePatientDO> getRehabilitationGuidanceList(Long teamCode, String patient, String diseaseCode, Integer pageIndex, Integer pageSize){
        String sql = null;
        Integer page = (pageIndex-1) * pageSize;
        if (teamCode != null){
            sql = "SELECT rgp.*  FROM wlyy_rehabilitation_guidance_patient rgp , wlyy_rehabilitation_plan_detail rpd ,wlyy_patient_rehabilitation_plan prp\n" +
                    "WHERE rgp.relation_code = rpd.id AND rpd.plan_id = prp.id AND prp.disease = '"+diseaseCode+"' AND rgp.patient = '"+patient+"' AND prp.team_code = '"+teamCode+"' ORDER BY czrq DESC LIMIT "+page+","+pageSize+"";
        }else {
            sql = "SELECT rgp.*  FROM wlyy_rehabilitation_guidance_patient rgp , wlyy_rehabilitation_plan_detail rpd ,wlyy_patient_rehabilitation_plan prp\n" +
                    "WHERE rgp.relation_code = rpd.id AND rpd.plan_id = prp.id AND prp.disease = '"+diseaseCode+"' AND rgp.patient = '"+patient+"' ORDER BY czrq DESC LIMIT "+page+","+pageSize+"";
        }
        List<RehabilitationguidancePatientDO> list = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(RehabilitationguidancePatientDO.class));
        return list;
    }

    public List<Followup> getHouseholdFollowupList(Long teamCode, String patient, String diseaseCode, Integer pageIndex, Integer pageSize){
        Integer page = (pageIndex-1) * pageSize;
//        3	冠心病
//        9	脑卒中
//        10	慢性前列腺
//        11	肺癌
//        12	慢性肾病
//        13	慢性乳腺疾病
//        14	慢性胃肠病
//        15	慢性阻塞性肺病
//        16	慢性肝病
//        17	慢性妇科炎症
//随访类别，多类别“，”分割【1.高血压 2.糖尿病 3.肿瘤 4.精神分裂症 5.产后 6.新生儿 7.严重精神病 8.80岁以上老人 9.肺结核 10异常体温随访
// 11.冠心病日常随访 12.慢性妇科炎症13.慢性前列腺疾病日常随访 14.慢性乳腺疾病日常随访 15慢性肾病日常随访 16慢性阻塞性肺病 17脑卒中日常随访 18脂肪肝日常随访
        String disease = "";
        if (diseaseCode.equals("3")) { disease = "11" ; }else if (diseaseCode.equals("9")){disease = "17" ;}
        else if (diseaseCode.equals("10")){disease = "13" ;}else if (diseaseCode.equals("11")){disease = "9" ;}
        else if (diseaseCode.equals("12")){disease = "15" ;}else if (diseaseCode.equals("13")){disease = "14" ;}
        else if (diseaseCode.equals("14")){disease = "" ;}else if (diseaseCode.equals("15")){disease = "16" ;}
        else if (diseaseCode.equals("16")){disease = "18" ;}else if (diseaseCode.equals("17")){disease = "12" ;}
        String sql = null;
        if (teamCode!=null){
            sql = "SELECT * FROM wlyy_followup WHERE patient_code = '"+patient+"' " +
                    "AND find_in_set('"+disease+"',followup_class) AND admin_team_code = '"+teamCode+"' ORDER BY create_time DESC LIMIT "+page+","+pageSize+""; //find_in_set('"+disease+"',followup_class)
        }else {
            sql = "SELECT * FROM wlyy_followup WHERE patient_code = '"+patient+"' " +
                    "AND find_in_set('"+disease+"',followup_class) ORDER BY create_time DESC LIMIT "+page+","+pageSize+"";
        }
        List<Followup> list = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(Followup.class));
        BasePatientDO patient1 = new BasePatientDO();
        List<WlyyHospitalSysDictDO> dicts = hospitalSysDictDao.findByDictName("FOLLOWUP_WAY_DICT");
        if (list.size()>0){
            for (int i =0; i <list.size(); i++){
                Followup followup = list.get(i);
                patient1 = patientDao.findById(list.get(i).getPatientCode()).orElse(null);
                list.get(i).setPhoto(patient1.getPhoto());
//【1.高血压 2.糖尿病 3.肿瘤 4.精神分裂症 5.产后 6.新生儿 7.严重精神病 8.80岁以上老人 9.肺结核 11.冠心病日常随访 12.慢性妇科炎症
                if (followup.getFollowupClass().equals("1")) { followup.setFollowupClass("高血压"); }
                else if (followup.getFollowupClass().equals("2")) { followup.setFollowupClass("糖尿病"); }
                else if (followup.getFollowupClass().equals("1,2")) { followup.setFollowupClass("高血压，糖尿病"); }
                else if (followup.getFollowupClass().equals("3")) { followup.setFollowupClass("肿瘤"); }
                else if (followup.getFollowupClass().equals("4")) { followup.setFollowupClass("精神分裂症"); }
                else if (followup.getFollowupClass().equals("5")) { followup.setFollowupClass("产后"); }
                else if (followup.getFollowupClass().equals("6")) { followup.setFollowupClass("新生儿"); }
                else if (followup.getFollowupClass().equals("7")) { followup.setFollowupClass("严重精神病"); }
                else if (followup.getFollowupClass().equals("8")) { followup.setFollowupClass("80岁以上老人"); }
                else if (followup.getFollowupClass().equals("9")) { followup.setFollowupClass("肺结核"); }
                else if (followup.getFollowupClass().equals("11")) { followup.setFollowupClass("冠心病"); }
                else if (followup.getFollowupClass().equals("12")) { followup.setFollowupClass("慢性妇科炎症"); }
                // 13.慢性前列腺疾病日常随访 14.慢性乳腺疾病日常随访 15慢性肾病日常随访 16慢性阻塞性肺病17脑卒中日常随访 18脂肪肝日常随访
                else if (followup.getFollowupClass().equals("13")) { followup.setFollowupClass("慢性前列腺疾病"); }
                else if (followup.getFollowupClass().equals("14")) { followup.setFollowupClass("慢性乳腺疾病"); }
                else if (followup.getFollowupClass().equals("15")) { followup.setFollowupClass("慢性肾病"); }
                else if (followup.getFollowupClass().equals("16")) { followup.setFollowupClass("慢性阻塞性肺病"); }
                else if (followup.getFollowupClass().equals("17")) { followup.setFollowupClass("脑卒中"); }
                else if (followup.getFollowupClass().equals("18")) { followup.setFollowupClass("脂肪肝"); }
                else {}
                for (int a=0;a<dicts.size();a++){
                    if (followup.getFollowupType().equals(dicts.get(a).getDictCode())){
                        followup.setFollowupType(dicts.get(a).getDictValue());
                    }
                }
            }
        }
        return list;
    }

    public List<Followup> getHouseholdFollowupPatientList(Long teamCode, String patient, String diseaseCode, Integer pageIndex,Integer pageSize){
        Integer page = (pageIndex-1) * pageSize;
//        3	冠心病
//        9	脑卒中
//        10	慢性前列腺
//        11	肺癌
//        12	慢性肾病
//        13	慢性乳腺疾病
//        14	慢性胃肠病
//        15	慢性阻塞性肺病
//        16	慢性肝病
//        17	慢性妇科炎症
//随访类别，多类别“，”分割【1.高血压 2.糖尿病 3.肿瘤 4.精神分裂症 5.产后 6.新生儿 7.严重精神病 8.80岁以上老人 9.肺结核 10异常体温随访
// 11.冠心病日常随访 12.慢性妇科炎症13.慢性前列腺疾病日常随访 14.慢性乳腺疾病日常随访 15慢性肾病日常随访 16慢性阻塞性肺病 17脑卒中日常随访 18脂肪肝日常随访
        String disease = "";
        if (diseaseCode.equals("3")) { disease = "11" ; }else if (diseaseCode.equals("9")){disease = "17" ;}
        else if (diseaseCode.equals("10")){disease = "13" ;}else if (diseaseCode.equals("11")){disease = "9" ;}
        else if (diseaseCode.equals("12")){disease = "15" ;}else if (diseaseCode.equals("13")){disease = "14" ;}
        else if (diseaseCode.equals("14")){disease = "" ;}else if (diseaseCode.equals("15")){disease = "16" ;}
        else if (diseaseCode.equals("16")){disease = "18" ;}else if (diseaseCode.equals("17")){disease = "12" ;}
        String sql = null;
        if (teamCode!=null){
            sql = "SELECT * FROM wlyy_followup WHERE patient_code = '"+patient+"' " +
                    "AND find_in_set('"+disease+"',followup_class) AND admin_team_code = '"+teamCode+"' ORDER BY create_time DESC LIMIT "+page+","+pageSize+""; //find_in_set('"+disease+"',followup_class)
        }else {
            sql = "SELECT * FROM wlyy_followup WHERE patient_code = '"+patient+"' " +
                    "AND find_in_set('"+disease+"',followup_class) ORDER BY create_time DESC LIMIT "+page+","+pageSize+"";
        }
        List<Followup> list = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(Followup.class));
        BasePatientDO patient1 = new BasePatientDO();
        List<WlyyHospitalSysDictDO> dicts = hospitalSysDictDao.findByDictName("FOLLOWUP_WAY_DICT");
        if (list.size()>0){
            for (int i =0; i <list.size(); i++){
                Followup followup = list.get(i);
                patient1 = patientDao.findById(followup.getPatientCode()).orElse(null);
                followup.setPhoto(patient1.getPhoto());
//【1.高血压 2.糖尿病 3.肿瘤 4.精神分裂症 5.产后 6.新生儿 7.严重精神病 8.80岁以上老人 9.肺结核 11.冠心病日常随访 12.慢性妇科炎症
                if (followup.getFollowupClass().equals("1")) { followup.setFollowupClass("高血压"); }
                else if (followup.getFollowupClass().equals("2")) { followup.setFollowupClass("糖尿病"); }
                else if (followup.getFollowupClass().equals("1,2")) { followup.setFollowupClass("高血压，糖尿病"); }
                else if (followup.getFollowupClass().equals("3")) { followup.setFollowupClass("肿瘤"); }
                else if (followup.getFollowupClass().equals("4")) { followup.setFollowupClass("精神分裂症"); }
                else if (followup.getFollowupClass().equals("5")) { followup.setFollowupClass("产后"); }
                else if (followup.getFollowupClass().equals("6")) { followup.setFollowupClass("新生儿"); }
                else if (followup.getFollowupClass().equals("7")) { followup.setFollowupClass("严重精神病"); }
                else if (followup.getFollowupClass().equals("8")) { followup.setFollowupClass("80岁以上老人"); }
                else if (followup.getFollowupClass().equals("9")) { followup.setFollowupClass("肺结核"); }
                else if (followup.getFollowupClass().equals("11")) { followup.setFollowupClass("冠心病"); }
                else if (followup.getFollowupClass().equals("12")) { followup.setFollowupClass("慢性妇科炎症"); }
                // 13.慢性前列腺疾病日常随访 14.慢性乳腺疾病日常随访 15慢性肾病日常随访 16慢性阻塞性肺病17脑卒中日常随访 18脂肪肝日常随访
                else if (followup.getFollowupClass().equals("13")) { followup.setFollowupClass("慢性前列腺疾病"); }
                else if (followup.getFollowupClass().equals("14")) { followup.setFollowupClass("慢性乳腺疾病"); }
                else if (followup.getFollowupClass().equals("15")) { followup.setFollowupClass("慢性肾病"); }
                else if (followup.getFollowupClass().equals("16")) { followup.setFollowupClass("慢性阻塞性肺病"); }
                else if (followup.getFollowupClass().equals("17")) { followup.setFollowupClass("脑卒中"); }
                else if (followup.getFollowupClass().equals("18")) { followup.setFollowupClass("脂肪肝"); }
                else {}

                for (int a=0;a<dicts.size();a++){
                    if (followup.getFollowupType().equals(dicts.get(a).getDictCode())){
                        followup.setFollowupType(dicts.get(a).getDictValue());
                    }
                }
            }
        }
        return list;
    }

    /**
     * 查询频次列表
     *
     * @return
     */
    public List<Frequency> selectFrequences(){
        List<Frequency> frequencieList = frequencyDao.findByDel();
        return frequencieList;
    }

    /**
     * 获取康复服务项目列表
     * @return
     * @param name
     */
    public List<Map<String, Object>> getServiceItemList(String name) {
        String sql = "SELECT code,`name` FROM `wlyy_rehabilitation_service_item` ";
        if(StringUtils.isNotBlank(name)){
            sql += "where name like '%" + name + "%' ";
        }
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        return list;
    }


    public Integer confirmManage(String patient,String signDoctor,String specialist,String relationId,String planId) throws Exception {
        String sqlCondition;
        if (StringUtils.isNoneBlank(relationId)){
            sqlCondition=" and pr.id='"+relationId+"' ";
        }
        else{
            sqlCondition=" and pr.doctor = '"+specialist+ "'";
        }
        String sql = "update wlyy_specialist_patient_relation pr set pr.is_manage=1,pr.manage_user ='"+signDoctor+"', pr.manage_time ='"+DateUtil.getStringDate()+"' where pr.patient='"+patient+"' "+sqlCondition;
        Integer manage = jdbcTemplate.update(sql);
        if (manage==1){
            String specialistSql = "select pr.doctor,pr.patient from wlyy_specialist_patient_relation pr where pr.patient='"+patient+"' "+sqlCondition;
            List<Map<String,Object>> mapList = jdbcTemplate.queryForList(specialistSql);
            if (mapList!=null&&mapList.size()!=0){
                Map<String,Object> map = mapList.get(0);
                String doctor = (String)map.get("doctor");
                String patient1 = (String)map.get("patient");
                guidanceService.sendSpecialistWeixinMessage(patient1,doctor,2,null,null,null);
           }

            //确认管理给管理医生群组发送消息
            if (StringUtils.isNoneBlank(planId)){
                try {
                    JSONArray serverDoctorList =selectPlanServerDoctor(planId,signDoctor);
                    int length = serverDoctorList.size();
                    BaseDoctorDO doctor = doctorDao.findByIdAndDel(signDoctor);
                    BasePatientDO p = patientDao.findById(patient).orElse(null);
                    BaseDoctorDO specialistDoctor = doctorDao.findByIdAndDel(specialist);
                    for (int i=0;i<length;i++){
                        JSONObject serverJson = serverDoctorList.getJSONObject(i);
                        if (serverJson.getString("type").equals("4")){
                            String sessionId = serverJson.getString("sessionId");
                            String message= specialistDoctor.getName()+"医生您好，我已确认接收管理患者"+p.getName()+"，谢谢。";
                            imUtil.sendImMsg(signDoctor,doctor.getName(),sessionId,"1",message,"1");
                        }
                    }
                } catch (ServiceException se) {
                    se.printStackTrace();
                } catch(Exception e){
                    e.printStackTrace();
                }
            }
        }
        return manage;
    }


    /**
     *
     * @param planId 计划id
     * @param planDetailId 计划详情id
     * @param itemId 服务项目code
     * @return
     */
    public List<Map<String,Object>> selectPlanDetailByPlan(String planId,String planDetailId,String itemId){
        String sql ="SELECT * FROM wlyy_rehabilitation_plan_detail pd WHERE " +
                "pd.plan_id = '"+planId+"' " +
                "AND pd.id <>'"+planDetailId+"' " +
                "and pd.hospital_service_item_id='"+itemId+"' and status=0  order by pd.execute_time asc";
        List<Map<String,Object>> mapList = jdbcTemplate.queryForList(sql);
        for (Map<String,Object> map:mapList){
            String doctorCode = map.get("doctor")+"";
//            BaseDoctorDO doctor = doctorDao.findByIdAndDel(doctorCode);
//            map.put("hospital",doctor.getHospital());
//            map.put("hospitalName",doctor.getHospitalName());
//            map.put("dept",doctor.getDept());
//            map.put("deptName",doctor.getDeptName());
        }
        return mapList;
    }


    /**
     * 获取这个居民的所有康复计划对应的复诊记录
     * @param patient
     * @return
     */
    public JSONObject selectReservationRecord(String patient,String doctorCode,Integer page,Integer pageSize){
        JSONObject object = new JSONObject();
        BaseDoctorDO doctor = doctorDao.findByIdAndDel(doctorCode);
        String condition="";
        if (doctor!=null){
//            HospitalMapping hospitalMapping = hospitalMappingDao.findByCode(doctor.getCode());
//            if (hospitalMapping!=null){
//               condition = " and a.org_code='"+doctor.getHospital()+"' ";
//            }
        }
        String sql = "SELECT a.* FROM  wlyy_patient_reservation a  WHERE a.patient='"+patient+"'" +
                " AND (a.relation_code IS NOT NULL OR a.relation_code<>'') "+condition+"  ORDER BY a.czrq DESC limit "+(page-1)*pageSize+","+pageSize;
        String sqlCount = "SELECT a.* FROM  wlyy_patient_reservation a  WHERE a.patient='"+patient+"'" +
                " AND (a.relation_code IS NOT NULL OR a.relation_code<>'') "+condition+"  ORDER BY a.czrq DESC ";
        List<Map<String,Object>> mapList = jdbcTemplate.queryForList(sql);
        List<Map<String,Object>> mapList1 = jdbcTemplate.queryForList(sqlCount);
        object.put("data",mapList);
        object.put("total",mapList1.size());
        object.put("page",page);
        object.put("pageSize",pageSize);
        return object;
    }
    
    public String getPlantCodeByMedicalRecordsCode(String code) {

        String sql ="SELECT id as code FROM wlyy_patient_rehabilitation_plan pd WHERE " +
                "pd.medical_records_code = '"+code+"' ";
        List<Map<String,Object>> mapList = jdbcTemplate.queryForList(sql);
        String result = "";
        for (Map<String,Object> map:mapList){
            result = map.get("code")+"";
        }
        return result;
    }

    /**
     * 获取入住疾病的医院列表
     * @param diseaseType
     * @return
     */
    public List<Map<String,Object>> getDiseaseHospital(String diseaseType,String name,Integer page,Integer size){
        String sql = "select a.hospital,a.hospital_name hospitalName,b.id,b.code,b.name,b.province,b.province_name provinceName," +
                "b.city,b.city_name cityName,b.town,b.town_name townName,b.level,b.address,b.intro,b.photo," +
                "case b.`level` when 1 then '医院' WHEN 2 THEN '社区医院'  END levelName,c.mapping_code jwHospital " +
                "from wlyy_hospital_specialist a,dm_hospital b,wlyy_hospital_mapping c where a.code = '"+diseaseType+"' and a.hospital = b.code and a.hospital = c.`code` ";
        if(StringUtils.isNoneBlank(name)){
            sql += " and b.name like '%"+name+"%'";
        }
        sql+= " limit "+(page-1)*size+","+size;
        List<Map<String,Object>> hospitalCountList = jdbcTemplate.queryForList(sql);
        return  hospitalCountList;
    }
    
    /**
     * 获取居民疾病专题首页相关数据
     * @param patientCode
     * @param diseaseType
     * @return
     */
	public com.alibaba.fastjson.JSONObject getDiseaseTopicInfo(com.alibaba.fastjson.JSONObject object, String patientCode, String diseaseType) throws Exception {
        com.alibaba.fastjson.JSONObject result = new com.alibaba.fastjson.JSONObject();
	    //获取医院数量
        String hospitalCountSql = "SELECT count(id) AS total FROM wlyy_hospital_specialist WHERE code = '"+diseaseType+"'";
        List<Map<String,Object>> hospitalCountList = jdbcTemplate.queryForList(hospitalCountSql);
        if (hospitalCountList!=null && hospitalCountList.size()!=0){
            result.put("hospitalCount",hospitalCountList.get(0).get("total"));
        }else{
            result.put("hospitalCount",0);
        }
        
        //获取医院医生数量
        String hospitalDoctorCountSql = "SELECT count(d.id) AS total FROM wlyy_hospital_specialist h,base_doctor d WHERE h.hospital=d.hospital AND h.code = '"+diseaseType+"'";
        List<Map<String,Object>> hospitalDoctorList = jdbcTemplate.queryForList(hospitalDoctorCountSql);
        if (hospitalDoctorList!=null && hospitalDoctorList.size()!=0){
            result.put("doctorCount",hospitalDoctorList.get(0).get("total"));
        }else{
            result.put("doctorCount",0);
        }

        //获取中心医院名称
        String hospitalNameSql = "SELECT a.name,a.address FROM dm_hospital a,wlyy_hospital_specialist b WHERE a.code=b.hospital AND b.code = '"+diseaseType+"'";
        List<Map<String,Object>> hospitalName = jdbcTemplate.queryForList(hospitalNameSql);
        if (hospitalName!=null && hospitalName.size()!=0){
            result.put("hospitalName",hospitalName.get(0).get("name"));
            result.put("hospitaladdress",hospitalName.get(0).get("address"));
        }else{
            result.put("hospitalName",null);
        }

        //住院病历
        String medicalRecordsSql ="SELECT * from wlyy_patient_medical_records_rehabilitation where patient = '"+patientCode+"' order by create_time DESC limit 0,1";
        List<Map<String,Object>> medicalRecords = jdbcTemplate.queryForList(medicalRecordsSql);
        if (medicalRecords!=null && medicalRecords.size()!=0){
            result.put("medicalRecords",medicalRecords);
        }else{
            result.put("medicalRecords",null);
        }
        
        //体征数据
        String healthIndexSql ="SELECT * from device.wlyy_patient_health_index i where user = '"+patientCode+"' order by czrq desc limit 0,1;";

        List<Map<String,Object>> healthIndexs = jdbcTemplate.queryForList(healthIndexSql);
        if (healthIndexs!=null && healthIndexs.size()!=0){
            result.put("healthIndexs",healthIndexs);
        }else{
            result.put("healthIndexs",null);
        }
        
        //康复指导数量
        String guidanceSql ="SELECT count(id) AS total from wlyy_rehabilitation_guidance_patient where patient = '"+patientCode+"' order by czrq limit 0,1";
        List<Map<String,Object>> guidanceRecords = jdbcTemplate.queryForList(guidanceSql);
        if (guidanceRecords!=null && guidanceRecords.size()!=0){
            result.put("guidanceCount",guidanceRecords.get(0).get("total"));
        }else{
            result.put("guidanceCount",0);
        }
        
        Integer diseaseCode=0;
        
            switch (diseaseType){
            case "fa"://肺癌
                diseaseCode = 11;
                break;
            case "gxb"://冠心病
                diseaseCode = 3;
                break;
            case "mxfkyz"://慢性妇科炎症
                diseaseCode = 17;
                break;
            case "mxqlx"://慢性前列腺
                diseaseCode = 10;
                break;
            case "mxrxjb"://慢性乳腺疾病
                diseaseCode = 13;
                break;
            case "mxsb"://慢性肾病
                diseaseCode = 12;
                break;
            case "mxcwb"://慢性肠胃病
                diseaseCode = 14;
                break;
            case "mxzsxfb"://慢性阻塞性肺病
                diseaseCode = 15;
                break;
            case "ncz"://脑卒中
                diseaseCode = 9;
                break;
                case "zfg"://脂肪肝
                diseaseCode = 16;
                break;
            default:
                break;
        }

        //查看康复计划
        String planSql = "SELECT COUNT(*) from wlyy_patient_rehabilitation_plan WHERE  patient = '"+patientCode+"' and FIND_IN_SET('"+diseaseCode+"',disease) ";
        Integer planNum = jdbcTemplate.queryForObject(planSql,Integer.class);
        result.put("planNum",planNum);

        //健康评估
        String screenResultSql ="SELECT screen_result from wlyy_survey_screen_result where patient_code='"+patientCode+"' and disease ="+diseaseCode+" order by id desc limit 0,1;";
        List<Map<String,Object>> screenResultList = jdbcTemplate.queryForList(screenResultSql);
        if (screenResultList!=null && screenResultList.size()!=0){
            result.put("screen_result",screenResultList.get(0).get("screen_result"));
        }else{
            result.put("screen_result",null);
        }

        BasePatientDO patient = new BasePatientDO();
        if (object.getString("openId") != null){
            patient = patientDao.findByOpenid(object.getString("openId"));
        }else if (object.getString("patientId") != null){
            patient = patientDao.findById(object.getString("patientId")).orElse(null);
        }

        String healthPlanCount = "SELECT count(*) FROM wlyy_health_bank.wlyy_health_bank_activity WHERE id IN ( SELECT bt.transaction_id FROM wlyy_health_bank.wlyy_health_bank_task bt WHERE id IN (SELECT task_id FROM wlyy_health_bank.wlyy_health_bank_task_patient_detail WHERE patient_idcard = '"+patient.getIdcard()+"' )) AND del_flag =1 " ;
        int count = jdbcTemplate.queryForObject(healthPlanCount , Integer.class);
        result.put("healthPlanCountList" , count);
        return result;
        
	}


    public void sendMessage(String sendId, String receiverId, String content) {
	    BasePatientDO patient = patientDao.findById(sendId).orElse(null);
	    BaseDoctorDO specialist = doctorDao.findByIdAndDel(receiverId);
        SystemMessageDO message = new SystemMessageDO();
        message.setCode(CommonUtil.getCode());
        message.setSender(patient.getId());
        message.setSenderName(patient.getName());
        message.setType("24");
        message.setReceiver(receiverId);
        message.setReceiverName(specialist.getName());
        message.setTitle("下转邀请");
        message.setContent("居民"+ patient.getName() + "于" + DateUtil.getStringDate() + "邀请您参与康复计划，请及时处理");
        message.setData(content);
        //查询居民康复计划数
//        String sql ="SELECT count(1) FROM `wlyy_patient_rehabilitation_plan` where patient='"+sendId + "' and `status`= 1 ";
//        message.setValue1(jdbcTemplate.queryForObject(sql, Double.class));
        message.setIsRead("0");
        message.setOver("1");
        message.setDel("1");
        message.setState(1);
        message.setCreateTime(new Date());
        messageDao.save(message);
    }

    public JSONObject getPatientSpecialAndFamilyDoctor(String patientCode){
        JSONObject result = new JSONObject();
        //专科列表
        String sql = "SELECT DISTINCT p.patient, p.name AS patient_name, d.`code` as special_code, d.`name` as special_name, d.photo,d.dept_name,d.hospital_name \n" +
                "FROM wlyy_patient_rehabilitation_plan p, base_doctor d\n" +
                "WHERE p.patient = '"+patientCode+"' AND d.`code`= p.create_user GROUP BY p.patient, p.create_user";
        List<Map<String,Object>> specialList = jdbcTemplate.queryForList(sql);
        result.put("specialList",specialList);
        //全科 健康管理师列表
        sql  ="select s.patient,s.`name`,s.doctor,doc.photo as doctor_photo,s.doctor_name " +
                "\t\tfrom wlyy_sign_family s ,base_doctor doc\n" +
                "\t\twhere s.patient ='"+patientCode+"' and doc.`code` =s.doctor and s.type=2 and s.`status`=1;";
        Map<String,Object> doctor = jdbcTemplate.queryForMap(sql);
        result.put("doctor",doctor);
        return  result;
    }

    public JSONArray getPlanDetailForModifing(String planId,String templateId){
        String sql;
        JSONArray result = new JSONArray();
        if(StringUtils.isBlank(templateId)){
            sql ="select id from wlyy_rehabilitation_plan_template where title='系统模板'";
            templateId = jdbcTemplate.queryForObject(sql,String.class);
        }
        sql = "select d.hospital_service_item_id as code,i.`name`\n" +
                "from wlyy_rehabilitation_template_detail d LEFT JOIN wlyy_rehabilitation_service_item i\n" +
                "ON i.`code` = d.hospital_service_item_id\n" +
                "where d.template_id = '"+templateId+"'";
        List<Map<String,Object>> templateDetailList = jdbcTemplate.queryForList(sql);
        for (Map<String,Object> map:templateDetailList){
            sql ="select d.*,DATE_FORMAT(d.execute_time,'%Y/%m/%d %H:%i') as executeTime ,i.code,i.name \n" +
                    "from wlyy_rehabilitation_plan_detail d \n" +
                    "LEFT JOIN wlyy_rehabilitation_service_item i on i.code = d.hospital_service_item_id  \n" +
                    "where  d.plan_id ='"+planId+"' AND  d.execute_time IS NOT NULL  and d.execute_time <>''\n" +
                    "AND d.`status`<> 1\n" +
                    "and d.hospital_service_item_id ='"+map.get("code").toString()+"'\n" +
                    "ORDER BY hospital_service_item_id ASC,execute_time ASC;";
            List<Map<String,Object>> planDetailList = jdbcTemplate.queryForList(sql);
            JSONObject temp = new JSONObject();
            temp.put("counts",planDetailList.size());
            temp.put("code",map.get("code").toString());
            temp.put("name",map.get("name").toString());
            temp.put("list",planDetailList);
            result.add(temp);
        }
        return result;
    }


    /*完成续方处方*/
    public List<Map<String, Object>> getPrescriptionFulfil(String patient, String doctor) {
        return prescriptionInfoService.getDoctorPrescription(null, doctor, "100", null, null, null, null, patient, 1, 10);
    }

    /*康复咨询+健康咨询 列表*/
    public JSONArray getRehabilitationConsultationList(String patient,String doctor,String brief) {
        JSONArray jsonArray = new  JSONArray();
        //判断brief是否为1  1查询统计数量   其他查询详细
        if (StringUtils.isNotBlank(brief)&&"1".equals(brief)){
            String totalSql="SELECT COUNT(*) count," +
                    "( SELECT ct.czrq FROM wlyy_consult_team ct,im.topics it " +
                    "WHERE ct.consult = it.id AND ct.type IN (2, 18) AND ct.doctor = '"+doctor+"' AND ct.patient = '"+patient+"'" +
                    " ORDER BY ct.czrq DESC LIMIT 0,1) czrq " +
                    "FROM wlyy_consult_team c, im.topics t " +
                    "WHERE c.consult = t.id AND c.type IN (2, 18) AND c.doctor = '"+doctor+"' " +
                    "AND c.patient = '"+patient+"'";

            String notTotalSql ="SELECT COUNT(*) notCount FROM wlyy_consult_team c,im.topics t WHERE c.consult = t.id " +
                    "AND c.type IN (2,18) AND t.reply = 0 AND c.doctor = '"+doctor+"' " +
                    "AND c.patient = '"+patient+"'";

            List<Map<String, Object>> totalList = jdbcTemplate.queryForList(totalSql);
            List<Map<String, Object>> notTotalList = jdbcTemplate.queryForList(notTotalSql);
            for (Map<String, Object> map : totalList) {
                jsonArray.add(map);
            }
            for (Map<String, Object> map : notTotalList) {
                jsonArray.add(map);
            }
        }else{
                //针对单个病人
            String sql = "SELECT DISTINCT c.name,c.symptoms,c.czrq,c.status,c.type,c.photo " +
                    "FROM wlyy_consult_team c, im.topics t " +
                    "WHERE c.consult = t.id AND c.type IN (2,18) " +
                    " AND c.doctor = '"+doctor+"' " +
                    " AND c.patient = '"+patient+"'";

            List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
            for (Map<String, Object> map : list) {
                if (!map.containsKey("photo")||"".equals(map.get("photo"))){
                    map.put("photo","");
                }
                jsonArray.add(map);
            }
        }
        return jsonArray;
    }

    /*随访计划*/
    public JSONArray getFollowupPlan(String doctor, String patient, String id, String biref) {
        String sql = "";
        //康复 kf 随访 sf  续方xf
        JSONArray jsonArray = new JSONArray();
        if (StringUtils.isNotBlank(biref)&&"1".equals(biref)){
            String kfSql = "SELECT DISTINCT " +
                    "COUNT(*) kfTotal " +
                    "FROM " +
                    "wlyy_rehabilitation_plan_detail wrpd," +
                    "wlyy_patient_rehabilitation_plan wprp " +
                    "WHERE " +
                    "wrpd.plan_id = wprp.id " +
                    "AND wrpd.doctor = '"+doctor+"' " +
                    "AND wprp.patient = '"+patient+"' " +
                    "AND wrpd.`status` = 1";

            String sfsql = "SELECT " +
                    "count(*) sfTotal "+
                    "FROM " +
                    "wlyy_followup wf " +
                    "LEFT JOIN system_dict sd ON wf.followup_type = sd. CODE " +
                    "WHERE " +
                    "sd.dict_name = 'FOLLOWUP_WAY_DICT' " +
                    "AND wf.doctor_code = '"+doctor+"' " +
                    "AND wf.patient_code = '"+patient+"' " +
                    "AND wf.type IN (1, 2)"+
                    "AND wf.status = 1";

            String xfsql="select count(*) xfTotal from wlyy_prescription where status = 100 and patient = '"+patient+"' and doctor ='"+doctor+"'";

            List<Map<String, Object>> kflist = jdbcTemplate.queryForList(kfSql);
            List<Map<String, Object>> sflist = jdbcTemplate.queryForList(sfsql);
            List<Map<String, Object>> xflist = jdbcTemplate.queryForList(xfsql);

            int xfTotal = 0;
            int kfTotal = 0;
            int sfTotal = 0;

            if (xflist.size()==0){
                xfTotal = 0;
            }else {
                for (Map<String, Object> map : xflist) {
                    xfTotal = Integer.parseInt(map.get("xfTotal").toString());
                }
            }

            if (kflist.size()==0){
                kfTotal = 0;
            }else {
                for (Map<String, Object> map : kflist) {
                    kfTotal = Integer.parseInt(map.get("kfTotal").toString());
                }
            }

            if (sflist.size()==0){
                sfTotal = 0;
            }else {
                for (Map<String, Object> map : sflist) {
                    sfTotal = Integer.parseInt(map.get("sfTotal").toString());
                }
            }

            Map totalMap =new HashMap<>();
            totalMap.put("total",xfTotal+sfTotal+kfTotal);
            jsonArray.add(totalMap);

        }else {
            if (!StringUtils.isNotBlank(id)||"".equals(id)){
                sql = "SELECT " +
                        "wf.id," +
                        "wf.create_time," +
                        "wf.doctor_code doctorCode," +
                        "wf.doctor_name doctorName," +
                        "wf. STATUS STATUS " +
                        "FROM " +
                        "wlyy_followup wf " +
                        "LEFT JOIN system_dict sd ON wf.followup_type = sd. CODE " +
                        "WHERE " +
                        "sd.dict_name = 'FOLLOWUP_WAY_DICT' " +
                        "AND wf.doctor_code = '"+doctor+"' " +
                        "AND wf.patient_code = '"+patient+"' " +
                        "AND wf.type IN (1, 2)";
            }else {
                sql = "SELECT " +
                        "wf.followup_date followupDate," +
                        "wf.followup_type followupType," +
                        "wf.followup_class followupClass," +
                        "  sd.value `value`," +
                        "wf.followup_plan_date followupPlanDate," +
                        "wf.doctor_code doctorCode," +
                        "wf.doctor_name doctorName," +
                        "wf.org_name orgName," +
                        "wf.patient_code patientCode," +
                        "wf.patient_name patientName," +
                        "wf.status status " +
                        "FROM " +
                        "wlyy_followup wf " +
                        "LEFT JOIN system_dict sd ON wf.followup_type = sd.CODE " +
                        "WHERE " +
                        "sd.dict_name = 'FOLLOWUP_WAY_DICT' " +
                        " AND wf.doctor_code = '"+doctor+"' " +
                        " AND wf.patient_code = '"+patient+"' AND wf.id = '"+id+"' AND wf.type IN (1, 2)";
            }

            jsonArray.add(jdbcTemplate.queryForList(sql));
        }
        return jsonArray;
    }
}
