package com.yihu.jw.afterDiagnosis.service.rehabilitation;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.afterDiagnosis.dao.rehabilitation.*;
import com.yihu.jw.afterDiagnosis.vo.TemplateDetailVo;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.base.wx.WxTemplateConfigDO;
import com.yihu.jw.entity.hospital.doctor.Frequency;
import com.yihu.jw.entity.hospital.message.SystemMessageDO;
import com.yihu.jw.entity.specialist.RehabilitationServiceItemDO;
import com.yihu.jw.entity.specialist.rehabilitation.PatientRehabilitationPlanDO;
import com.yihu.jw.entity.specialist.rehabilitation.RehabilitationDetailDO;
import com.yihu.jw.entity.specialist.rehabilitation.RehabilitationPlanTemplateDO;
import com.yihu.jw.entity.specialist.rehabilitation.RehabilitationTemplateDetailDO;
import com.yihu.jw.hospital.message.dao.SystemMessageDao;
import com.yihu.jw.message.service.WxMessageUtil;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.restmodel.specialist.PatientSignInfoVO;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.util.common.CommonUtil;
import com.yihu.jw.util.date.DateUtil;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONArray;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Created by humingfen on 2018/8/22.
 */
@Service
@Transactional
public class WlyyRehabilitationPlanService {

    private Logger logger = LoggerFactory.getLogger(RehabilitationPlanService.class);

    @Autowired
    private SystemMessageDao systemMessageDao;
    @Autowired
    private BasePatientDao patientDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private FrequencyDao frequencyDao;
    @Autowired
    private BaseDoctorDao doctorDao;
    @Autowired
    private RehabilitationPlanService rehabilitationPlanService;
    @Autowired
    private PatientRehabilitationPlanDao patientRehabilitationPlanDao;
    @Autowired
    private RehabilitationDetailDao rehabilitationDetailDao;
    @Autowired
    private SpecialistService specialistService;
    @Autowired
    private RehabilitationPlanTemplateDao templateDao;
    @Autowired
    private RehabilitationTemplateDetailDao templateDetailDao;
    @Autowired
    private RehabilitationServiceItemDao serviceItemDao;
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private WxMessageUtil wxMessageUtil;

    public List<RehabilitationPlanTemplateDO> findTemplateInfo(String doctor, String patient, Long adminTeamId) throws Exception {
        if(adminTeamId == null && StringUtils.isNotBlank(doctor) && StringUtils.isNotBlank(patient)){
            PatientSignInfoVO patientSignSpecialistInfo = (PatientSignInfoVO) specialistService.findPatientSignSpecialistInfo(patient, doctor).getObj();
            adminTeamId = patientSignSpecialistInfo.getTeamCode();
        }
        List<RehabilitationPlanTemplateDO> list = templateDao.findByAdminTeamCode(adminTeamId);

        return list;
    }

    public List<TemplateDetailVo> findTemplateDetailInfo(String templateId) throws Exception {
        List<TemplateDetailVo> resultList = new ArrayList<>();
        List<RehabilitationTemplateDetailDO>  list = templateDetailDao.findTemplateDetailByTemplateId(templateId);
        List<RehabilitationServiceItemDO> itemDOList = serviceItemDao.findList();
        Map<String, List<RehabilitationTemplateDetailDO>> map = list.stream().collect(Collectors.groupingBy(RehabilitationTemplateDetailDO::getHospitalServiceItemId));
        for (String code : map.keySet()){
            TemplateDetailVo detailVo = new TemplateDetailVo();
            detailVo.setCode(code);
            detailVo.setId(code);
            detailVo.setList(map.get(code));
            for (RehabilitationServiceItemDO item:itemDOList){
                if(code.equals(item.getCode())){
                    detailVo.setName(item.getName());
                    break;
                }
            }
            resultList.add(detailVo);
        }
        return resultList;
    }

    public String createTemplate(String title, BaseDoctorDO doctor, Long teamId) throws Exception {
        RehabilitationPlanTemplateDO templateDO = new RehabilitationPlanTemplateDO();
        templateDO.setTitle(title);
        templateDO.setAdminTeamCode(teamId);
        templateDO.setCreateUser(doctor.getId());
        templateDO.setCreateUserName(doctor.getName());
        templateDO.setCreateTime(new Date());
        templateDO.setDel(1);
        templateDO.setUpdateTime(new Date());
        templateDO = templateDao.save(templateDO);
        String templateId = templateDO.getId();
        String user = templateDO.getCreateUser();
        String userName = templateDO.getCreateUserName();
        //默认新增服务项
        String sql = "SELECT code from system_dict WHERE `dict_name` = 'default_service_item_id'";
        List<String> list = jdbcTemplate.queryForList(sql,String.class);
        if(list!=null&&list.size()>0){
            list.stream().forEach(code->{
                RehabilitationTemplateDetailDO detailDO = new RehabilitationTemplateDetailDO();
                detailDO.setTemplateId(templateId);
                detailDO.setHospitalServiceItemId(code);
                detailDO.setCreateUser(user);
                detailDO.setCreateUserName(userName);
                detailDO.setCreateTime(new Date());
                detailDO.setUpdateTime(new Date());
                templateDetailDao.save(detailDO);
            });
        }

        return templateId;
    }

    public String createTemplateDetail(String json, String type, BaseDoctorDO doctor) throws Exception{
        JSONArray details = new JSONArray(json);
        for(int i =0;i<details.length();i++){
            JSONObject j = details.getJSONObject(i);
            j.put("createUser", doctor.getId());
            j.put("createUserName", doctor.getName());
        }
        if(type.equals("create")) {
            List<RehabilitationTemplateDetailDO> detailDOS = objectMapper.readValue(details.toString(), new TypeReference<List<RehabilitationTemplateDetailDO>>(){});

            for(RehabilitationTemplateDetailDO detail : detailDOS){
                detail.setCreateTime(new Date());
                detail.setUpdateTime(new Date());
                templateDetailDao.save(detail);
            }
        }else if(type.equals("edit")) {
            List<RehabilitationTemplateDetailDO> detailDOS = objectMapper.readValue(details.toString(), new TypeReference<List<RehabilitationTemplateDetailDO>>(){});
            String templateId = detailDOS.get(0).getTemplateId();
            if(templateId != null && templateId.length() > 0){
                templateDetailDao.deleteByTemplateId(templateId);
            }
            for(RehabilitationTemplateDetailDO detail : detailDOS){

                detail.setCreateTime(new Date());
                detail.setUpdateTime(new Date());

                templateDetailDao.save(detail);
            }
        }
        return "success";
    }


    @Transactional(propagation= Propagation.NOT_SUPPORTED)
    public String createRehabilitationPlan(String json, BaseDoctorDO doctor) throws Exception {
        JSONObject object = new JSONObject(json);
        String patient = object.getString("patient");
        BasePatientDO patientDo = patientDao.findById(patient).orElse(null);
        JSONArray array = object.getJSONArray("detail");
        for (int z=0;z<array.length();z++){
            JSONObject jsonObject = array.getJSONObject(z);
            Long timeType = jsonObject.getLong("timeType");
            Frequency frequency = frequencyDao.findByCode(jsonObject.getString("frequencyCode"));
            BaseDoctorDO doctor1 = doctorDao.findByIdAndDel(jsonObject.getString("doctor"));
//            jsonObject.put("hospital",doctor1.getHospital());
//            jsonObject.put("hospitalName",doctor1.getHospitalName());
            String executeTimes = jsonObject.getString("executeTimes");
            if(StringUtils.isNotBlank(executeTimes)) {
                List<String> list = new ArrayList<>();
                String[] executeTime = executeTimes.split(",");
                for (int i = 0; i < executeTime.length; i++) {
                    List<String> times = getFrequencyDate(frequency.getCode(), executeTime[i], frequency.getUnit(), frequency.getTime(), timeType);
                    for (int j = 0; j < times.size(); j++) {
                        if (list != null && list.size() != 0) {
                            if (!list.contains(times.get(j))) {
                                list.add(times.get(j));
                            }
                        } else {
                            list.add(times.get(j));
                        }
                    }
                }
                StringBuffer buffer = new StringBuffer();
                if (list != null && list.size() != 0) {
                    for (int i = 0; i < list.size(); i++) {
                        buffer.append(list.get(i) + ",");
                    }
                    if (StringUtils.isNoneBlank(buffer)) {
                        buffer.deleteCharAt(buffer.length() - 1);
                    }
                }
                jsonObject.put("executeTime", buffer);
            }
            jsonObject.remove("executeTimes");
        }
        object.put("createUser", doctor.getId());
        object.put("createUserName", doctor.getName());
        String planId = "";
        try {
            for (int z=0;z<array.length();z++){
                JSONObject j = array.getJSONObject(z);
                if(j.has("executeTime")) {
                    String executeTime = j.get("executeTime").toString();
                    String[] result = null;
                    if (executeTime.contains(",")) {
                        result = executeTime.split(",");
                    } else {
                        result = new String[1];
                        result[0] = executeTime;
                    }
                    int len = result.length;
                    while (len > 0) {
                        len--;
                        JSONObject temp = new JSONObject(j.toString());
                        temp.put("executeTime", result[len]);
                        temp.put("createUser", object.get("createUser"));
                        temp.put("createUserName", object.get("createUserName"));
                        array.put(temp);
                    }
                }else {
                    j.put("createUser", object.get("createUser"));
                    j.put("createUserName", object.get("createUserName"));
                    array.put(j);
                }
            }
            String planDetails = array.toString();
            objectMapper.setDateFormat(new SimpleDateFormat("yyyy-MM-dd HH:mm"));
            List<RehabilitationDetailDO> details = objectMapper.readValue(planDetails, new TypeReference<List<RehabilitationDetailDO>>(){});
            PatientRehabilitationPlanDO planDO = objectMapper.readValue(json.toString(), PatientRehabilitationPlanDO.class);
            planDO = rehabilitationPlanService.createPatientRehabilitationPlan(planDO);
            details = rehabilitationPlanService.createRehabilitationDetail(details, planDO.getId());
            planId = planDO.getId();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return planId;
    }

    public List<String> getFrequencyDate(String code,String date,String unit,String time,Long timeType) throws ParseException {
        List<String> list = new ArrayList<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String nowDate = date+":00";
        Date date6 = DateUtil.strToDateLong(nowDate);
        String date9 = sdf.format(date6)+" 09:00:00";//白天
        String date21 = sdf.format(date6)+" 21:00:00";//白天
        String date9s = DateUtil.getNextDay(date6,1)+" 09:00:00";//晚上
        Calendar calendar = Calendar.getInstance();
        if (unit.equalsIgnoreCase("D")&&!code.equalsIgnoreCase("qod")){
            String[] str = time.split(",");
            for (int i=0;i<str.length;i++){
                Date date1 = sdf.parse(date);
                calendar.setTime(date1);
                String lastDay = sdf.format(calendar.getTime()) + " "+str[i];
                list.add(lastDay);
            }
        }else if (unit.equalsIgnoreCase("D")&&code.equalsIgnoreCase("qod")){
            Date date1 = sdf.parse(date);
            calendar.setTime(date1);
            calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) +1);
            String lastDay = sdf.format(calendar.getTime()) + " "+time;
            list.add(lastDay);
        }else if (unit.equalsIgnoreCase("H")&&code.equalsIgnoreCase("q0.5h")){
            Date date1 = sdf.parse(date);
            calendar.setTime(date1);
            String endTime = null;
            String lastDay = null;
            if (timeType == 0){
                lastDay = sdf.format(calendar.getTime()) + " 00:00:00";
                endTime = sdf.format(calendar.getTime()) + " 23:59:59";
            }else if ( timeType == 1){
                lastDay = date9;
                endTime = date21;
            }else if (timeType == 2){
                lastDay = date21;
                endTime = date9s;
            }
            Date date2 = sdf1.parse(lastDay);
            Date date3 = sdf1.parse(endTime);
            Long time1 = date2.getTime();
            for (int i=0;i<47;i++){
                time1 += 30*60*1000;
                String day = sdf1.format(time1);
                Date date4 = sdf1.parse(day);
                if (date4.compareTo(date3)==-1 || date4.compareTo(date3) == 0){
                    list.add(day);
                }
            }
        }else if (unit.equalsIgnoreCase("H")&& code.equalsIgnoreCase("q2h")){
            Date date1 = sdf.parse(date);
            calendar.setTime(date1);
      /*      String lastDay = sdf.format(calendar.getTime()) + " 00:00:00";*/
            String endTime = null;
            String lastDay = null;
            if (timeType == 0){
                lastDay = sdf.format(calendar.getTime()) + " 00:00:00";
                endTime = sdf.format(calendar.getTime()) + " 23:59:59";
            }else if ( timeType == 1){
                lastDay = date9;
                endTime = date21;
            }else if (timeType == 2){
                lastDay = date21;
                endTime = date9s;
            }
            Date date2 = sdf1.parse(lastDay);
            Date date3 = sdf1.parse(endTime);
            calendar.setTime(date2);
            for (int i=0;i<11;i++){
                calendar.set(Calendar.HOUR,calendar.get(Calendar.HOUR)+2);
                Date date4 = sdf1.parse(sdf1.format(calendar.getTime()));
                System.out.print(sdf1.format(calendar.getTime()));
                if (date4.compareTo(date3)==-1||date4.compareTo(date3) == 0){
                    list.add(sdf1.format(calendar.getTime()));
                }
            }

        }else if (unit.equalsIgnoreCase("H")&&code.equalsIgnoreCase("q3h")){
            Date date1 = sdf.parse(date);
            calendar.setTime(date1);
            String endTime = null;
            String lastDay = null;
            if (timeType == 0){
                lastDay = sdf.format(calendar.getTime()) + " 00:00:00";
                endTime = sdf.format(calendar.getTime()) + " 23:59:59";
            }else if ( timeType == 1){
                lastDay = date9;
                endTime = date21;
            }else if (timeType == 2){
                lastDay = date21;
                endTime = date9s;
            }
            Date date2 = sdf1.parse(lastDay);
            Date date3 = sdf1.parse(endTime);
            calendar.setTime(date2);
            for (int i=0;i<7;i++){
                calendar.set(Calendar.HOUR,calendar.get(Calendar.HOUR)+3);
                Date date4 = sdf1.parse(sdf1.format(calendar.getTime()));
                if (date4.compareTo(date3)==-1||date4.compareTo(date3) == 0){
                    list.add(sdf1.format(calendar.getTime()));
                }

            }
        }else if (unit.equalsIgnoreCase("H")&&code.equalsIgnoreCase("q4h")){
            Date date1 = sdf.parse(date);
            calendar.setTime(date1);
            String endTime = null;
            String lastDay = null;
            if (timeType == 0){
                lastDay = sdf.format(calendar.getTime()) + " 00:00:00";
                endTime = sdf.format(calendar.getTime()) + " 23:59:59";
            }else if ( timeType == 1){
                lastDay = date9;
                endTime = date21;
            }else if (timeType == 2){
                lastDay = date21;
                endTime = date9s;
            }
            Date date2 = sdf1.parse(lastDay);
            Date date3 = sdf1.parse(endTime);
            calendar.setTime(date2);
            for (int i=0;i<5;i++){
                calendar.set(Calendar.HOUR,calendar.get(Calendar.HOUR)+4);
                Date date4 = sdf1.parse(sdf1.format(calendar.getTime()));
                if (date4.compareTo(date3)==-1||date4.compareTo(date3) == 0){
                    list.add(sdf1.format(calendar.getTime()));
                }

            }
        }else if (unit.equalsIgnoreCase("H")&&code.equalsIgnoreCase("qh")){
            Date date1 = sdf.parse(date);
            calendar.setTime(date1);
            String endTime = null;
            String lastDay = null;
            if (timeType == 0){
                lastDay = sdf.format(calendar.getTime()) + " 00:00:00";
                endTime = sdf.format(calendar.getTime()) + " 23:59:59";
            }else if ( timeType == 1){
                lastDay = date9;
                endTime = date21;
            }else if (timeType == 2){
                lastDay = date21;
                endTime = date9s;
            }
            Date date2 = sdf1.parse(lastDay);
            Date date3 = sdf1.parse(endTime);
            calendar.setTime(date2);
            for (int i=0;i<23;i++){
                calendar.set(Calendar.HOUR,calendar.get(Calendar.HOUR)+1);
                Date date4 = sdf1.parse(sdf1.format(calendar.getTime()));
                if (date4.compareTo(date3)==-1||date4.compareTo(date3) == 0){
                    list.add(sdf1.format(calendar.getTime()));
                }
            }
        }else if (unit.equalsIgnoreCase("W")&&code.equalsIgnoreCase("qw_1")){
            String date2 = DateUtil.getWeek(date,"1")+" 08:00:00";
            Date date1 = new Date();
            String date3 = sdf1.format(date1);
            if (date2.compareTo(date3)==1){
                list.add(date2);
            }
        }else if (unit.equalsIgnoreCase("W")&&code.equalsIgnoreCase("qw_2")){
            String date2 = DateUtil.getWeek(date,"2")+" 08:00:00";
            Date date1 = new Date();
            String date3 = sdf1.format(date1);
            if (date2.compareTo(date3)==1){
                list.add(date2);
            }
        }else if (unit.equalsIgnoreCase("W")&&code.equalsIgnoreCase("qw_3")){
            String date2 = DateUtil.getWeek(date,"3")+" 08:00:00";
            Date date1 = new Date();
            String date3 = sdf1.format(date1);
            if (date2.compareTo(date3)==1){
                list.add(date2);
            }
        }else if (unit.equalsIgnoreCase("W")&&code.equalsIgnoreCase("qw_4")){
            String date2 = DateUtil.getWeek(date,"4")+" 08:00:00";
            Date date1 = new Date();
            String date3 = sdf1.format(date1);
            if (date2.compareTo(date3)==1){
                list.add(date2);
            }
        }else if (unit.equalsIgnoreCase("W")&&code.equalsIgnoreCase("qw_5")){
            String date2 = DateUtil.getWeek(date,"5")+" 08:00:00";
            Date date1 = new Date();
            String date3 = sdf1.format(date1);
            if (date2.compareTo(date3)==1){
                list.add(date2);
            }
        }else if (unit.equalsIgnoreCase("W")&&code.equalsIgnoreCase("qw_6")){
            String date2 = DateUtil.getWeek(date,"6")+" 08:00:00";
            Date date1 = new Date();
            String date3 = sdf1.format(date1);
            if (date2.compareTo(date3)==1){
                list.add(date2);
            }
        }else if (unit.equalsIgnoreCase("W")&&code.equalsIgnoreCase("qw_7")){
            String date2 = DateUtil.getWeek(date,"7")+" 08:00:00";
            Date date1 = new Date();
            String date3 = sdf1.format(date1);
            if (date2.compareTo(date3)==1){
                list.add(date2);
            }
        }else if (unit.equalsIgnoreCase("W")&&code.equalsIgnoreCase("tiw135")){
            String date2 = DateUtil.getWeek(date,"1")+" 08:00:00";
            String date4 = DateUtil.getWeek(date,"3")+" 08:00:00";
            String date5 = DateUtil.getWeek(date,"5")+" 08:00:00";
            Date date1 = new Date();
            String date3 = sdf1.format(date1);
            if (date2.compareTo(date3)==1){
                list.add(date2);
            }
            if (date4.compareTo(date3)==1){
                list.add(date2);
            }
            if (date5.compareTo(date3) == 1){
                list.add(date2);
            }
        }else if (unit.equalsIgnoreCase("W")&&code.equalsIgnoreCase("tiw246")){
            String date2 = DateUtil.getWeek(date,"2")+" 08:00:00";
            String date4 = DateUtil.getWeek(date,"4")+" 08:00:00";
            String date5 = DateUtil.getWeek(date,"6")+" 08:00:00";
            Date date1 = new Date();
            String date3 = sdf1.format(date1);
            if (date2.compareTo(date3)==1){
                list.add(date2);
            }
            if (date4.compareTo(date3)==1){
                list.add(date2);
            }
            if (date5.compareTo(date3) == 1){
                list.add(date2);
            }
        }
        return list;
    }

    public String deleteTemplate(String templateId) {
        templateDao.updateDelById(templateId);
        return "success";
    }

    /**
     * 更新计划的状态
     * @param planId
     * @param status
     * @throws Exception
     */
    public void updatePlanStatusById(String planId, Integer status,String abortReason) throws Exception{
        Map<String, Object> param = new HashedMap();
        param.put("planId", planId);
        param.put("status", status);
        param.put("abortReason",abortReason);
        if(status!=0){
            rehabilitationPlanService.updatePlanStatusById(status,planId);
        }else {
            PatientRehabilitationPlanDO patientRehabilitationPlanDO=patientRehabilitationPlanDao.findById(planId).orElse(null);
            patientRehabilitationPlanDO.setAbortReason(abortReason);
            patientRehabilitationPlanDO.setAbortTime(new Date());
            patientRehabilitationPlanDO.setStatus(0);
            patientRehabilitationPlanDO.setUpdateTime(new Date());
            patientRehabilitationPlanDao.save(patientRehabilitationPlanDO);
            List<RehabilitationDetailDO> detailDOList = rehabilitationDetailDao.findByPlanIdAndStatus(planId,0);

            List<SystemMessageDO> messageList = new ArrayList<>();
            Map<String,String> recieveMap = new HashMap<>();
            String patientName = patientRehabilitationPlanDO.getName();
            String diseaseName = patientRehabilitationPlanDO.getDiseaseName();
            String createDoctorCode = patientRehabilitationPlanDO.getCreateUser();
            for (RehabilitationDetailDO obj : detailDOList) {
                String doctor = obj.getDoctor();
                if (!createDoctorCode.equals(doctor)){
                    if (!recieveMap.containsKey(doctor)){
                        recieveMap.put(doctor,obj.getDoctorName());
                    }
                }
            }
            for (Map.Entry<String,String> entry : recieveMap.entrySet()){
                //推送消息到IM
//                    pushMsgTask.put(entry.getKey(),"20", "康复计划中止消息", "您的居民"+patientName+"，已中止"+diseaseName+"康复计划，点击查看原因", "");
                SystemMessageDO message = initMessage(patientName,entry.getKey(),entry.getValue(),"system","system",
                        "康复计划-中止提醒","您的居民"+patientName+"，已中止"+diseaseName+"康复计划，点击查看原因。",planId,"29");
                messageList.add(message);
            }

            systemMessageDao.saveAll(messageList);
        }
    }

    private SystemMessageDO initMessage(String patientName,String doctor,String doctorName,String sender,String senderName,String title,String content,String relatioenCode,String type){
        SystemMessageDO message = new SystemMessageDO();
        message.setCreateTime(new Date());
        message.setContent(content);
        message.setIsRead("0");//设置未读
        message.setReceiver(doctor);//设置接受医生的code
        message.setReceiverName(doctorName);
        message.setSender(sender);//设置发送的用户
        message.setSenderName(senderName);
        message.setCode(UUID.randomUUID().toString().replaceAll("-", ""));
        message.setSenderPhoto("");
        message.setTitle(title);
        message.setType(type);//康复计划中止消息提醒
        message.setReadonly(1);//是否只读消息
        message.setRelationCode(relatioenCode);
        message.setOver("0");//未处理
        message.setState(1);
        message.setData("");
        message.setDel("1");
        return message;
    }

    /**
     * 更新居民签名照/证件照
     * @param planId
     * @param patientImg
     * @throws Exception
     */
    public void updatePatientImgById(String planId, String patientImg) throws Exception {
        patientRehabilitationPlanDao.updatePatientImgById(planId,patientImg);
    }

    public void sendWxTemplate(String planId, String doctorCode, ObjEnvelop objEnvelop) {
        JSONObject jsonObject  = (JSONObject) objEnvelop.getObj();
        if (!jsonObject.isNull("openid")) {
            BaseDoctorDO d = doctorDao.findByIdAndDel(doctorCode);
            String patient = jsonObject.getString("patientCode");
            String patientName = jsonObject.get("patientName") + "";

            WxTemplateConfigDO newConfig = new WxTemplateConfigDO();
            newConfig.setFirst(patientName + "您好，您有一个康复计划已完成，请确认接收。");
            newConfig.setKeyword1(jsonObject.get("disease_name") + "康复计划");
            newConfig.setKeyword2(d.getName());
            newConfig.setKeyword3(DateUtil.dateToStr(new Date ((Long) jsonObject.get("create_time")), "yyyy/MM/dd") + "-" + DateUtil.dateToStr(new Date(), "yyyy/MM/dd"));
            newConfig.setUrl("");
            wxMessageUtil.putWxMsg(jsonObject.get("openid")+"",newConfig,27,new com.alibaba.fastjson.JSONObject());
        }
    }

    public List<RehabilitationDetailDO>  updateRehabilitationPlan(String json, BaseDoctorDO doctor,String updateStatus) throws Exception {
        JSONObject object = new JSONObject(json);
        JSONArray array = object.getJSONArray("detail");
        for (int z=0;z<array.length();z++){
            JSONObject jsonObject = array.getJSONObject(z);
            Long timeType = jsonObject.getLong("timeType");
            Frequency frequency = frequencyDao.findByCode(jsonObject.getString("frequencyCode"));
            String executeTimes = jsonObject.getString("executeTimes");
            BaseDoctorDO doctor1 = doctorDao.findByIdAndDel(jsonObject.getString("doctor"));
//                jsonObject.put("hospital",doctor1.getHospital());
//                jsonObject.put("hospitalName",doctor1.getHospitalName());
            if(StringUtils.isNotBlank(executeTimes)) {
                List<String> list = new ArrayList<>();
                String[] executeTime = executeTimes.split(",");
                for (int i = 0; i < executeTime.length; i++) {
                    List<String> times = getFrequencyDate(frequency.getCode(), executeTime[i], frequency.getUnit(), frequency.getTime(), timeType);
                    for (int j = 0; j < times.size(); j++) {
                        if (list != null && list.size() != 0) {
                            if (!list.contains(times.get(j))) {
                                list.add(times.get(j));
                            }
                        } else {
                            list.add(times.get(j));
                        }
                    }
                }
                StringBuffer buffer = new StringBuffer();
                if (list != null && list.size() != 0) {
                    for (int i = 0; i < list.size(); i++) {
                        buffer.append(list.get(i) + ",");
                    }
                    if (StringUtils.isNoneBlank(buffer)) {
                        buffer.deleteCharAt(buffer.length() - 1);
                    }
                }
                jsonObject.put("executeTime", buffer);
            }
            jsonObject.remove("executeTimes");
        }

        String planId = object.getString("planId");
        String hospitalServiceItemId = object.getString("hospitalServiceItemId");
        //删除该服务项目今日之后未完成的计划
        if(StringUtils.isBlank(updateStatus)||updateStatus.equals("0")){//单个服务的修改
            rehabilitationPlanService.deleteDetailByPlanIdAndHospitalServiceItemId(planId, hospitalServiceItemId, DateUtil.getDateShort(new Date()));
        }
        else{
            rehabilitationPlanService.deleteDetailByPlanId(planId, DateUtil.getDateShort(new Date()));
        }
        for(Object planDetail : array) {
            String expense="";
            JSONObject j = (JSONObject)planDetail;
            if (j.get("expense") instanceof Integer){
                expense= String.valueOf(j.getInt("expense"));
            }
            else{
                expense = j.getString("expense");
                if(expense.contains(".")){
                    int pointIndex = expense.indexOf(".");
                    expense = expense.substring(0,pointIndex);
                }
            }
            j.put("expense",expense);
            if(j.has("executeTime")) {
                String executeTime = j.get("executeTime").toString();
                String[] result = null;
                if (executeTime.contains(",")) {
                    result = executeTime.split(",");
                } else {
                    result = new String[1];
                    result[0] = executeTime;
                }
                int len = result.length;
                while (len > 0) {
                    len--;
                    JSONObject temp = new JSONObject(j.toString());
                    temp.put("executeTime", result[len]);
                    temp.put("createUser", object.get("createUser"));
                    temp.put("createUserName", object.get("createUserName"));
                    array.put(temp);
                }
            }else {
                j.put("createUser", object.get("createUser"));
                j.put("createUserName", object.get("createUserName"));
                array.put(j);
            }
        }
        String planDetails = array.toString();
        ObjectMapper objectMapper = new ObjectMapper();
        objectMapper.setDateFormat(new SimpleDateFormat("yyyy-MM-dd HH:mm"));
        List<RehabilitationDetailDO> details = objectMapper.readValue(planDetails, new TypeReference<List<RehabilitationDetailDO>>(){});
        details = rehabilitationPlanService.createRehabilitationDetail(details, planId);


        String updateTime = DateUtil.dateToStrLong(details.get(0).getCreateTime());
        //康复计划修改 日志添加
        String sql ="INSERT INTO wlyy_patient_rehabilitation_plan_log(id,plan_id,create_user,create_user_name,create_time) " +
                "VALUES('"+ CommonUtil.getCode()+"','"+planId+"','"+doctor.getId()+"','"+ doctor.getName()+"','"+updateTime+"')";
        jdbcTemplate.execute(sql);

        return details;
    }

    /**
     * 获取康复计划修改记录
     * @param planId
     * @return
     */
    public List<Map<String,Object>> getRehabilitationPlanLog(String planId){
        String sql ="select * from wlyy_patient_rehabilitation_plan_log where plan_id = '"+planId+"' ORDER BY create_time DESC";
        List<Map<String,Object>> planLogList = jdbcTemplate.queryForList(sql);
        return planLogList;
    }
}
