package com.yihu.jw.base.endpoint.ManageMessagePush;

import com.yihu.jw.base.service.ManageMessagePush.ManageSystemPushMessageService;
import com.yihu.jw.contant.CommonContant;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * Created by wsl on 2022/7/4
 */


@RestController
@RequestMapping(value = "/ManageSystemPushMessage", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(value = "消息推送管理", description = "消息推送管理接口", tags = {"wlyy消息推送 - 消息推送管理接口"})
public class ManageSystemPushMessageController extends EnvelopRestEndpoint {


    @Autowired
    ManageSystemPushMessageService mspmservice;

    @ApiOperation("获取所有签约老人")
    @GetMapping(value = "/signOlds")
    public ListEnvelop signOlds(@ApiParam(name = "name", value = "居民名字", required = false)
                                @RequestParam(value = "name", required = false) String name) {
        try {
            List<BasePatientDO> basePatientDOS = mspmservice.signOlds(name);
            return success(basePatientDOS);
        } catch (Exception e) {
            e.printStackTrace();
            return failedListEnvelopException2(e);
        }
    }

    @ApiOperation("开关")
    @PostMapping(value = "/pushOnOff")
    public Envelop pushOnOff(@ApiParam(name = "pyCode",value = "开关状态",required = true)
                             @RequestParam(value = "pyCode",required = true)String pyCode,
                             @ApiParam(name = "dictName",value = "字典名字",required = true)
                             @RequestParam(value = "dictName",required = true)String dictName,
                             @ApiParam(name = "id",value = "字典id",required = true)
                             @RequestParam(value = "id",required = true)String id){

        try {
            mspmservice.pushOnOff(id,dictName,pyCode);
            return success("修改成功！");
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }

    @ApiOperation("获取推送记录列表")
    @PostMapping(value = "/pushMessageList")
    public ObjEnvelop pushMessageList(@ApiParam(name = "messageType",value = "消息类型",required = false)
                                       @RequestParam(value = "messageType",required = false)Integer messageType,
                                      @ApiParam(name = "status",value = "状态",required = false)
                                       @RequestParam(value = "status",required = false)Integer status,
                                      @ApiParam(name = "name",value = "名字查找",required = false)
                                       @RequestParam(value = "name",required = false)String name,
                                      @RequestParam(value = "page",required = true)Integer page,
                                      @RequestParam(value = "pageSize",required = true)Integer pageSize){
        try {
            return ObjEnvelop.getSuccess(CommonContant.Message.GET_SUCCESS,mspmservice.pushMessageList(messageType,status,name,page,pageSize),200);
        } catch (Exception e) {
            e.printStackTrace();
            return failedObjEnvelopException2(e);
        }
    }

    @ApiOperation("删除列表")
    @PostMapping(value = "/delPushMessageLog")
    public Envelop delPushMessageLog(@RequestParam(value = "id",required = true)String id){
        try {
            mspmservice.delPushMessageLog(id);
            return success("删除成功");
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }

    @ApiOperation("添加推送")
    @PostMapping(value = "/addPushMessage")
    public Envelop addPushMessage(@ApiParam(name = "content", value = "推送消息", required = true)
                                  @RequestParam(value = "content", required = true) String content,
                                  @ApiParam(name = "pushObjectType", value = "推送对象类型 1签约老人，2部分签约老人", required = true)
                                  @RequestParam(value = "pushObjectType", required = true) Integer pushObjectType,
                                  @ApiParam(name = "pushObjectJson", value = "推送对象", required = false)
                                  @RequestParam(value = "pushObjectJson", required = false) String pushObjectJson,
                                  @ApiParam(name = "pushDateType", value = "推送类型 1立即发送 2 定时发送", required = true)
                                  @RequestParam(value = "pushDateType", required = true) Integer pushDateType,
                                  @ApiParam(name = "pushDate", value = "推送时间 定时发送才传", required = false)
                                  @RequestParam(value = "pushDate", required = false) String pushDate,
                                  @ApiParam(name = "addType",value = "类型 1新增 2修改",required = false)
                                  @RequestParam(value = "addType",required = false)Integer addType,
                                  @ApiParam(name = "pushId",value = "推送id",required = false)
                                  @RequestParam(value = "pushId",required = false)String pushId) {
        try {
            mspmservice.addPushMessage(content,pushObjectType,pushObjectJson,pushDateType,pushDate,addType,pushId);
            return success("添加成功");
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }


    @ApiOperation("初始化居民推送开关")
    @PostMapping(value = "/patientPushOnOff")
    public Envelop patientPushOnOff(){
        try {
            mspmservice.patientPushOnOff();
            return success("成功!");
        } catch (Exception e) {
            e.printStackTrace();
            return failed("初始化失败!",500);
        }
    }



}
