package com.yihu.jw.base.endpoint.application;

import com.yihu.jw.application.service.BaseApplicationMenuService;
import com.yihu.jw.area.service.BaseCityService;
import com.yihu.jw.entity.base.application.BaseApplicationMenuDO;
import com.yihu.jw.entity.base.area.BaseCityDO;
import com.yihu.jw.restmodel.base.area.BaseCityVO;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.base.BaseRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
*
* @since 1.
*/
@RestController
@RequestMapping(value = BaseRequestMapping.ApplicationMenu.PREFIX)
@Api(value = "应用资源菜单管理", description = "应用资源菜单管理", tags = {"应用资源菜单管理"})
public class BaseApplicationMenuEndpoint extends EnvelopRestEndpoint {

@Autowired
private BaseApplicationMenuService applicationMenuService;

    @PostMapping(value = BaseRequestMapping.ApplicationMenu.insertAndUpdateMenu)
    @ApiOperation(value = "创建和修改应用菜单实体")
    public ObjEnvelop insertAndUpdateMenu(
        @ApiParam(name = "jsonData", value = "Json数据", required = true)
        @RequestParam(value = "jsonData") String jsonData) throws Exception {
            BaseApplicationMenuDO menuDO = toEntity(jsonData, BaseApplicationMenuDO.class);
            menuDO = applicationMenuService.insertAndUpdateMenu(menuDO,getUID());
            return success(menuDO);
    }

    @PostMapping(value = BaseRequestMapping.ApplicationMenu.DELETE)
    @ApiOperation(value = "根据id删除菜单实体")
    public Envelop delete(@ApiParam(name = "ids", value = "id串，中间用,分隔", required = true)
        @RequestParam(value = "ids") String ids) {
        applicationMenuService.deleteById(ids);
        return success("删除成功");
    }

    @GetMapping(value = BaseRequestMapping.ApplicationMenu.findById)
    @ApiOperation(value = "根据id获取菜单资源详情")
    public ObjEnvelop findById (
        @ApiParam(name = "id", value = "id", required = true)
        @RequestParam(value = "id") String id) throws Exception {
        return success(applicationMenuService.findById(id));
    }


    @GetMapping(value = BaseRequestMapping.ApplicationMenu.getMenuTree)
    @ApiOperation(value = "获取资源菜单树型结构")
    public Envelop getMenuTree (
            @ApiParam(name = "type", value = "type", required = true)
            @RequestParam(value = "type") String type,
            @ApiParam(name = "name", value = "name", required = false)
            @RequestParam(value = "name",required = false) String name) throws Exception {
        return success(applicationMenuService.getMenuTree(type,name));
    }

    @GetMapping(value = BaseRequestMapping.ApplicationMenu.selectByCondition)
    @ApiOperation(value = "查询资源菜单列表")
    public Envelop selectByCondition (
            @ApiParam(name = "type", value = "角色类型（字典V_APPLICATION_ROLE_DICT）", required = false)
            @RequestParam(value = "type",required = false) String type,
            @ApiParam(name = "saasId", value = "租户id", required = false)
            @RequestParam(value = "saasId",required = false) String saasId,
            @ApiParam(name = "orgCode", value = "机构id", required = false)
            @RequestParam(value = "orgCode",required = false) String orgCode,
            @ApiParam(name = "name", value = "资源菜单名称", required = false)
            @RequestParam(value = "name",required = false) String name,
            @ApiParam(name = "status", value = "是否生效(1生效0失效)", required = false)
            @RequestParam(value = "status",required = false) Integer status,
            @ApiParam(name = "page", value = "页码", required = false)
            @RequestParam(value = "page",required = false) Integer page,
            @ApiParam(name = "size", value = "大小", required = false)
            @RequestParam(value = "size",required = false) Integer size) throws Exception {
        return success(applicationMenuService.selectByCondition(type,saasId,orgCode,name,status,page,size));
    }

 }
