package com.yihu.jw.base.endpoint.course;

import com.yihu.jw.base.service.course.CourseCatalogueService;
import com.yihu.jw.entity.care.course.CourseCatalogueDO;
import com.yihu.jw.hospital.dict.WlyyHospitalSysDictDao;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * Created with IntelliJ IDEA.
 *
 * @Author: yeshijie
 * @Date: 2021/5/12
 * @Description:
 */
@RestController
@RequestMapping(value = "courseCatalogue", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(value = "课程目录管理", description = "课程目录管理服务接口", tags = {"care基础服务 - 课程目录管理服务接口"})
public class CourseCatalogueEndpoint extends EnvelopRestEndpoint {

    @Autowired
    private CourseCatalogueService courseCatalogueService;
    @Autowired
    private WlyyHospitalSysDictDao wlyyHospitalSysDictDao;

    @PostMapping(value = "create")
    @ApiOperation(value = "创建")
    public ObjEnvelop<CourseCatalogueDO> create (
            @ApiParam(name = "jsonData", value = "Json数据", required = true)
            @RequestParam String jsonData) throws Exception {
        CourseCatalogueDO catalogueDO = toEntity(jsonData, CourseCatalogueDO.class);
        catalogueDO = courseCatalogueService.save(catalogueDO);
        return success(catalogueDO);
    }

    @PostMapping(value = "delete")
    @ApiOperation(value = "删除")
    public Envelop delete(
            @ApiParam(name = "ids", value = "id串，中间用,分隔", required = true)
            @RequestParam(value = "ids") String ids) {
        courseCatalogueService.delete(ids.split(","));
        return success("删除成功");
    }

    @PostMapping(value = "update")
    @ApiOperation(value = "更新")
    public ObjEnvelop<CourseCatalogueDO> update (
            @ApiParam(name = "jsonData", value = "Json数据", required = true)
            @RequestParam String jsonData) throws Exception {
        CourseCatalogueDO catalogueDO = toEntity(jsonData, CourseCatalogueDO.class);
        if (null == catalogueDO.getId()) {
            return failed("ID不能为空", ObjEnvelop.class);
        }
        catalogueDO = courseCatalogueService.save(catalogueDO);
        return success(catalogueDO);
    }

    @GetMapping(value = "page")
    @ApiOperation(value = "获取分页")
    public PageEnvelop<CourseCatalogueDO> page (
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1")
            @RequestParam(value = "page") int page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15")
            @RequestParam(value = "size") int size) throws Exception {
        List<CourseCatalogueDO> catalogueDOs = courseCatalogueService.search(fields, filters, sorts, page, size);
        int count = (int)courseCatalogueService.getCount(filters);
        return success(catalogueDOs, count, page, size, CourseCatalogueDO.class);
    }

    @GetMapping(value = "list")
    @ApiOperation(value = "获取列表")
    public ListEnvelop<CourseCatalogueDO> list (
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts) throws Exception {
        List<CourseCatalogueDO> catalogueDOs = courseCatalogueService.search(fields, filters, sorts);
        return success(catalogueDOs, CourseCatalogueDO.class);
    }
}
