package com.yihu.jw.base.endpoint.course;

import com.yihu.jw.base.dao.course.CustomerServiceDao;
import com.yihu.jw.base.service.course.CustomerServiceSerivce;
import com.yihu.jw.entity.care.course.CustomerServiceDO;
import com.yihu.jw.hospital.dict.WlyyHospitalSysDictDao;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * Created with IntelliJ IDEA.
 *
 * @Author: yeshijie
 * @Date: 2021/5/12
 * @Description:
 */
@RestController
@RequestMapping(value = "customerService", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(value = "客服管理", description = "客服管理服务接口", tags = {"care基础服务 - 客服管理服务接口"})
public class CustomerServiceEndpoint extends EnvelopRestEndpoint {

    @Autowired
    private CustomerServiceSerivce customerServiceSerivce;
    @Autowired
    private WlyyHospitalSysDictDao wlyyHospitalSysDictDao;
    @Autowired
    private CustomerServiceDao customerServiceDao;

    @PostMapping(value = "create")
    @ApiOperation(value = "创建")
    public ObjEnvelop<CustomerServiceDO> create (
            @ApiParam(name = "jsonData", value = "Json数据", required = true)
            @RequestParam String jsonData) {
        try {
            CustomerServiceDO customerServiceDO = toEntity(jsonData, CustomerServiceDO.class);
            List<CustomerServiceDO> customerServiceDOList = customerServiceDao.findByOrgCode(customerServiceDO.getOrgCode());
            if(customerServiceDOList.size()>0){
                return ObjEnvelop.getError("该机构已存在客服团队",-1);
            }
            customerServiceDO = customerServiceSerivce.save(customerServiceDO);
            return success(customerServiceDO);
        }catch (Exception e){
            e.printStackTrace();
            return ObjEnvelop.getError("创建失败",-1);
        }

    }

    @PostMapping(value = "delete")
    @ApiOperation(value = "删除")
    public Envelop delete(
            @ApiParam(name = "ids", value = "id串，中间用,分隔", required = true)
            @RequestParam(value = "ids") String ids) {
        customerServiceSerivce.delete(ids.split(","));
        return success("删除成功");
    }

    @PostMapping(value = "update")
    @ApiOperation(value = "更新")
    public ObjEnvelop<CustomerServiceDO> update (
            @ApiParam(name = "jsonData", value = "Json数据", required = true)
            @RequestParam String jsonData) throws Exception {
        CustomerServiceDO customerServiceDO = toEntity(jsonData, CustomerServiceDO.class);
        if (null == customerServiceDO.getId()) {
            return failed("ID不能为空", ObjEnvelop.class);
        }
        customerServiceDO = customerServiceSerivce.save(customerServiceDO);
        return success(customerServiceDO);
    }

    @GetMapping(value = "page")
    @ApiOperation(value = "获取分页")
    public PageEnvelop<CustomerServiceDO> page (
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1")
            @RequestParam(value = "page") int page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15")
            @RequestParam(value = "size") int size) throws Exception {
        List<CustomerServiceDO> customerServiceDOs = customerServiceSerivce.search(fields, filters, sorts, page, size);
        int count = (int)customerServiceSerivce.getCount(filters);
        return success(customerServiceDOs, count, page, size, CustomerServiceDO.class);
    }

    @GetMapping(value = "list")
    @ApiOperation(value = "获取列表")
    public ListEnvelop<CustomerServiceDO> list (
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts) throws Exception {
        List<CustomerServiceDO> customerServiceDOs = customerServiceSerivce.search(fields, filters, sorts);
        return success(customerServiceDOs, CustomerServiceDO.class);
    }
}
