package com.yihu.jw.base.endpoint.course;

import com.yihu.jw.base.service.course.RecruitStudentsRecordService;
import com.yihu.jw.base.service.course.RecruitStudentsService;
import com.yihu.jw.entity.care.course.RecruitStudentsDO;
import com.yihu.jw.entity.care.course.RecruitStudentsRecordDO;
import com.yihu.jw.hospital.dict.WlyyHospitalSysDictDao;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.utils.StringUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * Created with IntelliJ IDEA.
 *
 * @Author: yeshijie
 * @Date: 2021/5/12
 * @Description:
 */
@RestController
@RequestMapping(value = "recruitStudents", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(value = "招生管理", description = "招生管理服务接口", tags = {"care基础服务 - 招生管理服务接口"})
public class RecruitStudentsEndpoint extends EnvelopRestEndpoint {

    @Autowired
    private RecruitStudentsService recruitStudentsService;
    @Autowired
    private WlyyHospitalSysDictDao wlyyHospitalSysDictDao;
    @Autowired
    private RecruitStudentsRecordService recruitStudentsRecordService;

    @PostMapping(value = "create")
    @ApiOperation(value = "创建")
    public ObjEnvelop<RecruitStudentsDO> create (
            @ApiParam(name = "jsonData", value = "Json数据", required = true)
            @RequestParam String jsonData) throws Exception {
        RecruitStudentsDO recruitStudentsDO = toEntity(jsonData, RecruitStudentsDO.class);
        recruitStudentsDO.setDel(1);
        recruitStudentsDO = recruitStudentsService.save(recruitStudentsDO);
        return success(recruitStudentsDO);
    }

    @PostMapping(value = "delete")
    @ApiOperation(value = "删除")
    public Envelop delete(
            @ApiParam(name = "ids", value = "id串，中间用,分隔", required = true)
            @RequestParam(value = "ids") String ids) {
        recruitStudentsService.deletes(ids);
        return success("删除成功");
    }

    @PostMapping(value = "update")
    @ApiOperation(value = "更新")
    public ObjEnvelop<RecruitStudentsDO> update (
            @ApiParam(name = "jsonData", value = "Json数据", required = true)
            @RequestParam String jsonData) throws Exception {
        RecruitStudentsDO recruitStudentsDO = toEntity(jsonData, RecruitStudentsDO.class);
        if (null == recruitStudentsDO.getId()) {
            return failed("ID不能为空", ObjEnvelop.class);
        }
        recruitStudentsDO = recruitStudentsService.save(recruitStudentsDO);
        return success(recruitStudentsDO);
    }

    @GetMapping(value = "page")
    @ApiOperation(value = "获取分页")
    public PageEnvelop<RecruitStudentsDO> page (
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1")
            @RequestParam(value = "page") int page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15")
            @RequestParam(value = "size") int size) throws Exception {
        if(StringUtil.isBlank(filters)){
            filters = "del=1";
        }else{
            filters += ";del=1";
        }
        List<RecruitStudentsDO> recruitStudentsDOs = recruitStudentsService.search(fields, filters, sorts, page, size);
        for (RecruitStudentsDO recruitStudentsDO:recruitStudentsDOs){
            List<RecruitStudentsRecordDO> list = recruitStudentsRecordService.findListById(recruitStudentsDO.getId());
            recruitStudentsDO.setAdmittedNum(list.size());
        }
        int count = (int)recruitStudentsService.getCount(filters);
        return success(recruitStudentsDOs, count, page, size, RecruitStudentsDO.class);
    }

    @GetMapping(value = "list")
    @ApiOperation(value = "获取列表")
    public ListEnvelop<RecruitStudentsDO> list (
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts) throws Exception {
        if(StringUtil.isBlank(filters)){
            filters = "del=1";
        }else{
            filters += ";del=1";
        }
        List<RecruitStudentsDO> recruitStudentsDOs = recruitStudentsService.search(fields, filters, sorts);
        return success(recruitStudentsDOs, RecruitStudentsDO.class);
    }


}
