package com.yihu.jw.base.endpoint.dict;

import com.yihu.jw.base.service.dict.DictDoctorDutyService;
import com.yihu.jw.base.service.dict.DictJobTitleService;
import com.yihu.jw.entity.base.dict.DictDoctorDutyDO;
import com.yihu.jw.restmodel.base.dict.DictJobTitleVO;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.base.BaseRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import com.yihu.jw.entity.base.dict.DictJobTitleDO;

/**
 * 职称字典控制器
 *
 * @version <pre>
 * Author	Version		Date		Changes
 * litaohong 	1.0  		2018年08月31日 	Created
 *
 * </pre>
 * @since 1.
 */
@RestController
@RequestMapping(value = BaseRequestMapping.DictJobTitle.PREFIX)
@Api(value = "职称字典管理", description = "职称字典管理服务接口", tags = {"wlyy基础服务 - 职称字典管理服务接口"})
public class DictJobTitleEndpoint extends EnvelopRestEndpoint {

    @Autowired
    private DictJobTitleService dictJobTitleService;
    @Autowired
    private DictDoctorDutyService dictDoctorDutyService;

    @PostMapping(value = BaseRequestMapping.DictJobTitle.CREATE)
    @ApiOperation(value = "创建")
    public ObjEnvelop<DictJobTitleVO> create(
            @ApiParam(name = "json_data", value = "Json数据", required = true)
            @RequestBody String jsonData) throws Exception {
        DictJobTitleDO dictJobTitle = toEntity(jsonData, DictJobTitleDO.class);
        dictJobTitle = dictJobTitleService.save(dictJobTitle);
        return success(dictJobTitle, DictJobTitleVO.class);
    }

    @PostMapping(value = BaseRequestMapping.DictJobTitle.DELETE)
    @ApiOperation(value = "删除")
    public Envelop delete(
            @ApiParam(name = "ids", value = "id串，中间用,分隔", required = true)
            @RequestParam(value = "ids") String ids) {
        dictJobTitleService.delete(ids.split(","));
        return success("删除成功");
    }

    @PostMapping(value = BaseRequestMapping.DictJobTitle.UPDATE)
    @ApiOperation(value = "更新")
    public ObjEnvelop<DictJobTitleVO> update(
            @ApiParam(name = "json_data", value = "Json数据", required = true)
            @RequestBody String jsonData) throws Exception {
        DictJobTitleDO dictJobTitle = toEntity(jsonData, DictJobTitleDO.class);
        if (null == dictJobTitle.getId()) {
            return failed("ID不能为空", ObjEnvelop.class);
        }
        dictJobTitle = dictJobTitleService.save(dictJobTitle);
        return success(dictJobTitle, DictJobTitleVO.class);
    }

    @GetMapping(value = BaseRequestMapping.DictJobTitle.PAGE)
    @ApiOperation(value = "获取分页")
    public PageEnvelop<DictJobTitleVO> page(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1")
            @RequestParam(value = "page") int page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15")
            @RequestParam(value = "size") int size) throws Exception {
        List<DictJobTitleDO> dictJobTitles = dictJobTitleService.search(fields, filters, sorts, page, size);
        int count = (int) dictJobTitleService.getCount(filters);
        return success(dictJobTitles, count, page, size, DictJobTitleVO.class);
    }

    @GetMapping(value = BaseRequestMapping.DictJobTitle.LIST)
    @ApiOperation(value = "获取列表")
    public ListEnvelop<DictDoctorDutyDO> list(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts) throws Exception {
        List<DictDoctorDutyDO> dictJobTitles = dictDoctorDutyService.search(fields, filters, sorts);
        return success(dictJobTitles, DictDoctorDutyDO.class);
    }

}
