package com.yihu.jw.base.endpoint.gw;


import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.article.service.KnowledgeArticleDictService;
import com.yihu.jw.base.service.gw.BaseHomePageService;
import com.yihu.jw.base.service.gw.BaseRulesService;
import com.yihu.jw.base.service.gw.BaseWebInfoService;
import com.yihu.jw.base.service.gw.ZjxlGwService;
import com.yihu.jw.entity.base.gw.BaseHomePageDO;
import com.yihu.jw.entity.base.gw.BaseRulesDO;
import com.yihu.jw.entity.base.gw.BaseWebInfoDO;
import com.yihu.jw.entity.hospital.article.KnowledgeArticleDictDO;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * Created by yeshijie on 2022/2/21.
 */
@RestController
@RequestMapping(value = "/zjxlGw")
@Api(value = "公司官网", tags = {"公司官网"})
public class ZjxlGwEndpoint extends EnvelopRestEndpoint {

    @Autowired
    private ZjxlGwService zjxlGwService;
    @Autowired
    private BaseHomePageService baseHomePageService;
    @Autowired
    private BaseWebInfoService baseWebInfoService;
    @Autowired
    private BaseRulesService baseRulesService;
    @Autowired
    private KnowledgeArticleDictService knowledgeArticleService;

    @GetMapping(value = "searchKnowledgePageList")
    @ApiOperation(value = "获取文章列表")
    public Envelop searchKnowledgePageList(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "page", value = "第几页，从1开始", required = true, defaultValue = "1")
            @RequestParam(value = "page", required = true,defaultValue = "1")Integer page,
            @ApiParam(name = "pageSize", value = "每页分页大小", required = true, defaultValue = "10")
            @RequestParam(value = "pageSize", required = true,defaultValue = "10")Integer pageSize){

        try {
            List<KnowledgeArticleDictDO> list  = knowledgeArticleService.search(fields,filters,sorts,page,pageSize);
            List<KnowledgeArticleDictDO> totals  = knowledgeArticleService.search(fields,filters,sorts);
            JSONObject result = new JSONObject();
            result.put("total",totals.size());
            result.put("detailModelList",list);
            return success(result);
        }catch (Exception e){
            return failedException(e);
        }
    }

    @PostMapping(value = "addOrUpdRules")
    @ApiOperation(value = "创建或修改制度")
    public Envelop addOrUpdRules(
            @ApiParam(name = "jsonData", value = "Json数据", required = true)
            @RequestParam String jsonData){
        try {
            BaseRulesDO baseRulesDO = toEntity(jsonData, BaseRulesDO.class);
            if(baseRulesDO.getCreateTime()==null){
                baseRulesDO.setCreateTime(new Date());
            }
            baseRulesService.save(baseRulesDO);
            return success("操作成功");
        }catch (Exception e){
            e.printStackTrace();
            return failed("操作失败");
        }
    }

    @PostMapping(value = "deleteRules")
    @ApiOperation(value = "删除制度")
    public Envelop deleteRules(
            @ApiParam(name = "ids", value = "id串，中间用,分隔", required = true)
            @RequestParam(value = "ids") String ids) {
        try {
            baseRulesService.delete(ids.split(","));
            return success("删除成功");
        }catch (Exception e){
            e.printStackTrace();
            return failed("删除失败");
        }
    }

    @GetMapping(value = "rulesPage")
    @ApiOperation(value = "获取制度分页")
    public PageEnvelop<BaseRulesDO> rulesPage(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1")
            @RequestParam(value = "page") int page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15")
            @RequestParam(value = "size") int size) {
        try {
            List<BaseRulesDO> baseOrgs = baseRulesService.search(fields, filters, sorts, page, size);
            int count = (int) baseRulesService.getCount(filters);
            return success(baseOrgs, count, page, size, BaseRulesDO.class);
        }catch (Exception e){
            e.printStackTrace();
            return PageEnvelop.getError("查询失败");
        }
    }

    @GetMapping(value = "findRulesById")
    @ApiOperation(value = "根据id查询制度")
    public ObjEnvelop findRulesById (
            @ApiParam(name = "id", value = "id")
            @RequestParam(value = "id", required = true) String id){
        try {
            BaseRulesDO baseRulesDO = zjxlGwService.findRulesById(id);
            return ObjEnvelop.getSuccess("查询成功",baseRulesDO);
        }catch (Exception e){
            e.printStackTrace();
            return ObjEnvelop.getError("查询失败");
        }
    }

    @GetMapping(value = "findWebInfo")
    @ApiOperation(value = "网页基本信息")
    public ObjEnvelop findWebInfo(){
        try {
            BaseWebInfoDO webInfoDO = zjxlGwService.findWebInfo();
            return ObjEnvelop.getSuccess("查询成功",webInfoDO);
        }catch (Exception e){
            e.printStackTrace();
            return ObjEnvelop.getError("查询失败");
        }
    }

    @PostMapping(value = "addOrUpdWebInfo")
    @ApiOperation(value = "创建或修改网页基本信息")
    public Envelop addOrUpdWebInfo(
            @ApiParam(name = "jsonData", value = "Json数据", required = true)
            @RequestParam String jsonData){
        try {
            BaseWebInfoDO webInfoDO = toEntity(jsonData, BaseWebInfoDO.class);
            if(webInfoDO.getCreateTime()==null){
                webInfoDO.setCreateTime(new Date());
            }
            baseWebInfoService.save(webInfoDO);
            return success("操作成功");
        }catch (Exception e){
            e.printStackTrace();
            return failed("操作失败");
        }
    }

    @PostMapping(value = "addOrUpdHomePage")
    @ApiOperation(value = "创建或修改首页")
    public Envelop addOrUpdHomePage(
            @ApiParam(name = "jsonData", value = "Json数据", required = true)
            @RequestParam String jsonData){
        try {
            BaseHomePageDO baseHomePageDO = toEntity(jsonData, BaseHomePageDO.class);
            if(baseHomePageDO.getCreateTime()==null){
                baseHomePageDO.setCreateTime(new Date());
            }
            baseHomePageService.save(baseHomePageDO);
            return success("操作成功");
        }catch (Exception e){
            e.printStackTrace();
            return failed("操作失败");
        }
    }

    @PostMapping(value = "deleteHomePage")
    @ApiOperation(value = "删除首页")
    public Envelop deleteHomePage(
            @ApiParam(name = "ids", value = "id串，中间用,分隔", required = true)
            @RequestParam(value = "ids") String ids) {
        try {
            baseHomePageService.delete(ids.split(","));
            return success("删除成功");
        }catch (Exception e){
            e.printStackTrace();
            return failed("删除失败");
        }
    }

    @GetMapping(value = "homePagePage")
    @ApiOperation(value = "获取制度分页")
    public PageEnvelop<BaseHomePageDO> homePagePage(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1")
            @RequestParam(value = "page") int page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15")
            @RequestParam(value = "size") int size) {
        try {
            List<BaseHomePageDO> homePageDOS = baseHomePageService.search(fields, filters, sorts, page, size);
            for (BaseHomePageDO homePageDO:homePageDOS){
                if(StringUtils.isNotBlank(homePageDO.getArticleId())){
                    KnowledgeArticleDictDO articleDictDO = knowledgeArticleService.findById(homePageDO.getArticleId());
                    if(articleDictDO!=null){
                        homePageDO.setArticleTitle(articleDictDO.getTitle());
                    }
                }
            }
            int count = (int) baseHomePageService.getCount(filters);
            return success(homePageDOS, count, page, size, BaseHomePageDO.class);
        }catch (Exception e){
            e.printStackTrace();
            return PageEnvelop.getError("查询失败");
        }
    }

    @GetMapping(value = "findHomePageById")
    @ApiOperation(value = "根据id查询首页")
    public ObjEnvelop findHomePageById (
            @ApiParam(name = "id", value = "id")
            @RequestParam(value = "id", required = true) String id){
        try {
            BaseHomePageDO homePageDO = zjxlGwService.findHomePageById(id);
            if(StringUtils.isNotBlank(homePageDO.getArticleId())){
                KnowledgeArticleDictDO articleDictDO = knowledgeArticleService.findById(homePageDO.getArticleId());
                if(articleDictDO!=null){
                    homePageDO.setArticleTitle(articleDictDO.getTitle());
                }
            }
            return ObjEnvelop.getSuccess("查询成功",homePageDO);
        }catch (Exception e){
            e.printStackTrace();
            return ObjEnvelop.getError("查询失败");
        }
    }
}
