package com.yihu.jw.base.endpoint.gw;


import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.article.service.KnowledgeArticleDictService;
import com.yihu.jw.base.service.gw.BaseHomePageService;
import com.yihu.jw.base.service.gw.BaseRulesService;
import com.yihu.jw.base.service.gw.ZjxlGwService;
import com.yihu.jw.entity.base.gw.BaseHomePageDO;
import com.yihu.jw.entity.base.gw.BaseRulesDO;
import com.yihu.jw.entity.base.gw.BaseWebInfoDO;
import com.yihu.jw.entity.hospital.article.KnowledgeArticleDictDO;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * Created by yeshijie on 2022/2/21.
 */
@RestController
@RequestMapping(value = "open/zjxlGw")
@Api(value = "公司官网-前端", tags = {"公司官网-前端"})
public class ZjxlGwOpenEndpoint extends EnvelopRestEndpoint {

    @Autowired
    private ZjxlGwService zjxlGwService;
    @Autowired
    private BaseHomePageService baseHomePageService;
    @Autowired
    private BaseRulesService baseRulesService;
    @Autowired
    private KnowledgeArticleDictService knowledgeArticleService;

    @GetMapping(value = "searchKnowledgePageList")
    @ApiOperation(value = "获取文章列表")
    public Envelop searchKnowledgePageList(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "page", value = "第几页，从1开始", required = true, defaultValue = "1")
            @RequestParam(value = "page", required = true,defaultValue = "1")Integer page,
            @ApiParam(name = "pageSize", value = "每页分页大小", required = true, defaultValue = "10")
            @RequestParam(value = "pageSize", required = true,defaultValue = "10")Integer pageSize){

        try {
            List<KnowledgeArticleDictDO> list  = knowledgeArticleService.search(fields,filters,sorts,page,pageSize);
            List<KnowledgeArticleDictDO> totals  = knowledgeArticleService.search(fields,filters,sorts);
            JSONObject result = new JSONObject();
            result.put("total",totals.size());
            result.put("detailModelList",list);
            return success(result);
        }catch (Exception e){
            return failedException(e);
        }
    }

    @GetMapping(value = "rulesPage")
    @ApiOperation(value = "获取制度分页")
    public PageEnvelop<BaseRulesDO> rulesPage(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1")
            @RequestParam(value = "page") int page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15")
            @RequestParam(value = "size") int size) {
        try {
            List<BaseRulesDO> baseOrgs = baseRulesService.search(fields, filters, sorts, page, size);
            int count = (int) baseRulesService.getCount(filters);
            return success(baseOrgs, count, page, size, BaseRulesDO.class);
        }catch (Exception e){
            e.printStackTrace();
            return PageEnvelop.getError("查询失败");
        }
    }

    @GetMapping(value = "findRulesById")
    @ApiOperation(value = "根据id查询制度")
    public ObjEnvelop findRulesById (
            @ApiParam(name = "id", value = "id")
            @RequestParam(value = "id", required = true) String id){
        try {
            BaseRulesDO baseRulesDO = zjxlGwService.findRulesById(id);
            return ObjEnvelop.getSuccess("查询成功",baseRulesDO);
        }catch (Exception e){
            e.printStackTrace();
            return ObjEnvelop.getError("查询失败");
        }
    }

    @GetMapping(value = "findWebInfo")
    @ApiOperation(value = "网页基本信息")
    public ObjEnvelop findWebInfo(){
        try {
            BaseWebInfoDO webInfoDO = zjxlGwService.findWebInfo();
            return ObjEnvelop.getSuccess("查询成功",webInfoDO);
        }catch (Exception e){
            e.printStackTrace();
            return ObjEnvelop.getError("查询失败");
        }
    }

    @GetMapping(value = "homePagePage")
    @ApiOperation(value = "获取制度分页")
    public PageEnvelop<BaseHomePageDO> homePagePage(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1")
            @RequestParam(value = "page") int page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15")
            @RequestParam(value = "size") int size) {
        try {
            List<BaseHomePageDO> homePageDOS = baseHomePageService.search(fields, filters, sorts, page, size);
            int count = (int) baseHomePageService.getCount(filters);
            return success(homePageDOS, count, page, size, BaseHomePageDO.class);
        }catch (Exception e){
            e.printStackTrace();
            return PageEnvelop.getError("查询失败");
        }
    }

    @GetMapping(value = "findHomePageById")
    @ApiOperation(value = "根据id查询首页")
    public ObjEnvelop findHomePageById (
            @ApiParam(name = "id", value = "id")
            @RequestParam(value = "id", required = true) String id){
        try {
            BaseHomePageDO baseHomePageDO = zjxlGwService.findHomePageById(id);
            return ObjEnvelop.getSuccess("查询成功",baseHomePageDO);
        }catch (Exception e){
            e.printStackTrace();
            return ObjEnvelop.getError("查询失败");
        }
    }
}
