package com.yihu.jw.base.endpoint.interfacemg;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.base.service.interfacemg.BaseUserInterfaceService;
import com.yihu.jw.entity.third.interfacemg.BaseThirdUserDO;
import com.yihu.jw.entity.third.interfacemg.BaseThirdUserInterfaceDO;
import com.yihu.jw.entity.third.interfacemg.BaseThirdUserInterfaceLogDO;
import com.yihu.jw.entity.third.interfacemg.BaseThirdUserInterfaceStandardDO;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.base.BaseRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * 第三方接口配置管理
 */
@RestController
@RequestMapping(value = BaseRequestMapping.UserInterface.PREFIX)
@Api(value = "第三方接口配置管理", description = "第三方接口配置管理", tags = {"第三方接口配置管理"})
public class BaseUserInterfaceEndpoint extends EnvelopRestEndpoint {
    @Autowired
    private  BaseUserInterfaceService baseUserInterfaceService;

    @PostMapping(value = BaseRequestMapping.UserInterface.userSaveAndUpdate)
    @ApiOperation(value = "第三方接口用户信息配置")
    public Envelop userSaveAndUpdate (
            @ApiParam(name = "jsonData", value = "第三方接口信息配置实体", required = false)
            @RequestParam(value = "jsonData", required = false) String jsonData) throws Exception {
        BaseThirdUserDO baseThirdUserDO = toEntity(jsonData, BaseThirdUserDO.class);
        return success(baseUserInterfaceService.userSaveAndUpdate(baseThirdUserDO));
    }

    @PostMapping(value = BaseRequestMapping.UserInterface.userInterfaceSaveAndUpdate)
    @ApiOperation(value = "第三方接口信息配置")
    public Envelop userInterfaceSaveAndUpdate (
            @ApiParam(name = "jsonData", value = "第三方接口信息配置实体", required = false)
            @RequestParam(value = "jsonData", required = false) String jsonData) throws Exception {
        BaseThirdUserInterfaceDO baseThirdUserInterfaceDO = toEntity(jsonData, BaseThirdUserInterfaceDO.class);
        return success(baseUserInterfaceService.userInterfaceSaveAndUpdate(baseThirdUserInterfaceDO));
    }

    @PostMapping(value = BaseRequestMapping.UserInterface.userInterfaceStandardsaveAndUpdate)
    @ApiOperation(value = "第三方接口标准信息配置")
    public Envelop userInterfaceStandardSaveAndUpdate (
            @ApiParam(name = "jsonData", value = "第三方接口标准信息配置实体", required = false)
            @RequestParam(value = "jsonData", required = false) String jsonData) throws Exception {
        JSONArray array = JSONArray.parseArray(jsonData);
        List<BaseThirdUserInterfaceStandardDO> interfaceStandardDOList = new ArrayList<>();

        for (int i=0;i<array.size();i++){
            JSONObject object = array.getJSONObject(i);
            BaseThirdUserInterfaceStandardDO baseThirdUserInterfaceStandardDO = toEntity(object.toJSONString(), BaseThirdUserInterfaceStandardDO.class);
            interfaceStandardDOList.add(baseThirdUserInterfaceStandardDO);
        }
        return success(baseUserInterfaceService.userInterfaceStandardsaveAndUpdate(interfaceStandardDOList));
    }

    @PostMapping(value = BaseRequestMapping.UserInterface.userInterfaceLogsaveAndUpdate)
    @ApiOperation(value = "第三方接口调用日志")
    public Envelop userInterfaceLogsaveAndUpdate (
            @ApiParam(name = "jsonData", value = "第三方接口调用日志实体", required = false)
            @RequestParam(value = "jsonData", required = false) String jsonData) throws Exception {
        BaseThirdUserInterfaceLogDO baseThirdUserInterfaceStandardDO = toEntity(jsonData, BaseThirdUserInterfaceLogDO.class);
        return success(baseUserInterfaceService.userInterfaceLogsaveAndUpdate(baseThirdUserInterfaceStandardDO));
    }

    @GetMapping(value = BaseRequestMapping.UserInterface.selectUserByCondition)
    @ApiOperation(value = "查询第三方接口用户信息")
    public Envelop selectUserByCondition (
            @ApiParam(name = "condition", value = "条件(名字)", required = false)
            @RequestParam(value = "condition", required = false) String condition,
            @ApiParam(name = "page", value = "页数", required = false)
            @RequestParam(value = "page", required = false) Integer page,
            @ApiParam(name = "size", value = "页码", required = false)
            @RequestParam(value = "size", required = false) Integer size) throws Exception {
        JSONObject jsonObject = baseUserInterfaceService.selectUserByCondition(condition,page,size);
        return success(jsonObject);
    }

    @GetMapping(value = BaseRequestMapping.UserInterface.selectUserInterfaceByCondition)
    @ApiOperation(value = "查询第三方接口信息")
    public Envelop selectUserInterfaceByCondition (
            @ApiParam(name = "condition", value = "条件(名字)", required = false)
            @RequestParam(value = "condition", required = false) String condition,
            @ApiParam(name = "userId", value = "用户id", required = false)
            @RequestParam(value = "userId", required = false) String userId,
            @ApiParam(name = "page", value = "页数", required = false)
            @RequestParam(value = "page", required = false) Integer page,
            @ApiParam(name = "size", value = "页码", required = false)
            @RequestParam(value = "size", required = false) Integer size) throws Exception {
        return success(baseUserInterfaceService.selectUserInterfaceByCondition(condition,userId,page,size));
    }

    @GetMapping(value = BaseRequestMapping.UserInterface.selectUserInterfaceStandardByCondition)
    @ApiOperation(value = "查询第三方接口标准信息")
    public Envelop selectUserInterfaceStandardByCondition (
            @ApiParam(name = "condition", value = "条件(名字)", required = false)
            @RequestParam(value = "condition", required = false) String condition,
            @ApiParam(name = "userInterfaceId", value = "接口id", required = false)
            @RequestParam(value = "userInterfaceId", required = false) String userInterfaceId,
            @ApiParam(name = "stdDatasetId", value = "数据集id", required = false)
            @RequestParam(value = "stdDatasetId", required = false) String stdDatasetId,
            @ApiParam(name = "page", value = "页数", required = false)
            @RequestParam(value = "page", required = false) Integer page,
            @ApiParam(name = "size", value = "页码", required = false)
            @RequestParam(value = "size", required = false) Integer size) throws Exception {
        return success(baseUserInterfaceService.selectUserInterfaceStandardByCondition(condition,userInterfaceId,stdDatasetId,page,size));
    }

    @GetMapping(value = BaseRequestMapping.UserInterface.selectUserInterfaceLogByCondition)
    @ApiOperation(value = "查询第三方接口调用日志信息")
    public Envelop selectUserInterfaceLogByCondition (
            @ApiParam(name = "condition", value = "条件(名字)", required = false)
            @RequestParam(value = "condition", required = false) String condition,
            @ApiParam(name = "userInterfaceId", value = "接口id", required = false)
            @RequestParam(value = "userInterfaceId", required = false) String userInterfaceId,
            @ApiParam(name = "page", value = "页数", required = false)
            @RequestParam(value = "page", required = false) Integer page,
            @ApiParam(name = "size", value = "页码", required = false)
            @RequestParam(value = "size", required = false) Integer size) throws Exception {
        return success(baseUserInterfaceService.selectUserInterfaceLogByCondition(condition,userInterfaceId,page,size));
    }

    @GetMapping(value = BaseRequestMapping.UserInterface.findUserById)
    @ApiOperation(value = "根据某个用户配置id查询详情")
    public ObjEnvelop findUserById (
            @ApiParam(name = "id", value = "详情id", required = false)
            @RequestParam(value = "id", required = false) String id) throws Exception {
        return success(baseUserInterfaceService.findUserById(id));
    }

    @GetMapping(value = BaseRequestMapping.UserInterface.findUserInterfaceById)
    @ApiOperation(value = "根据某个接口配置id查询详情")
    public ObjEnvelop findUserInterfaceById (
            @ApiParam(name = "id", value = "详情id", required = false)
            @RequestParam(value = "id", required = false) String id) throws Exception {
        return success(baseUserInterfaceService.findUserInterfaceById(id));
    }

    @GetMapping(value = BaseRequestMapping.UserInterface.findUserInterfaceStandardById)
    @ApiOperation(value = "根据某个接口标准配置id查询详情")
    public ObjEnvelop findUserInterfaceStandardById (
            @ApiParam(name = "id", value = "详情id", required = false)
            @RequestParam(value = "id", required = false) String id) throws Exception {
        return success(baseUserInterfaceService.findUserInterfaceStandardById(id));
    }

    @GetMapping(value = BaseRequestMapping.UserInterface.findUserInterfaceLogById)
    @ApiOperation(value = "根据某个接口调用日志id查询详情")
    public ObjEnvelop findUserInterfaceLogById (
            @ApiParam(name = "id", value = "详情id", required = false)
            @RequestParam(value = "id", required = false) String id) throws Exception {
        return success(baseUserInterfaceService.findUserInterfaceLogById(id));
    }
}
