package com.yihu.jw.base.endpoint.login;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.yihu.jw.contant.CommonContant;
import com.yihu.jw.entity.base.login.BaseLoginAppDO;
import com.yihu.jw.entity.base.login.BaseLoginComponentDO;
import com.yihu.jw.login.service.BaseLoginAppService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by yeshijie on 2022/11/28.
 */
@RestController
@RequestMapping(value = "loginApp")
@Api(value = "统一登录组件", tags = {"统一登录组件"})
public class LoginAppEndpoint extends EnvelopRestEndpoint {

    @Autowired
    private BaseLoginAppService loginAppService;


    @PostMapping(value = "createLoginApp")
    @ApiOperation(value = "新增/修改登录应用")
    public ObjEnvelop createLoginApp (
            @ApiParam(name = "jsonData", value = "Json数据", required = true)
            @RequestParam String jsonData){
        try {
            BaseLoginAppDO loginAppDO = toEntity(jsonData, BaseLoginAppDO.class);
            return ObjEnvelop.getSuccess(CommonContant.Message.SAVE_SUCCESS,loginAppService.createLoginApp(loginAppDO));
        }catch (Exception e){
            e.printStackTrace();
        }
        return ObjEnvelop.getError(CommonContant.Message.SAVE_ERROR);
    }

    @PostMapping(value = "loginAppStatus")
    @ApiOperation(value = "生效/失效")
    public Envelop loginAppStatus(
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id") String id,
            @ApiParam(name = "del", value = "1生效，0失效", required = true)
            @RequestParam(value = "del") Integer del) {
        try {
            return ObjEnvelop.getSuccess("修改成功",loginAppService.loginAppStatus(id,del));
        }catch (Exception e){
            e.printStackTrace();
        }
        return Envelop.getError("修改失败");
    }


    @PostMapping(value = "loginAppDelete")
    @ApiOperation(value = "删除登录应用")
    public Envelop loginAppDelete(
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id") String id) {
        try {
            loginAppService.loginAppDelete(id);
            return Envelop.getSuccess("删除成功");
        }catch (Exception e){
            e.printStackTrace();
        }
        return Envelop.getError("删除失败");
    }


    @GetMapping(value = "loginAppPage")
    @ApiOperation(value = "获取分页")
    public Envelop loginAppPage (
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1")
            @RequestParam(value = "page") int page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15")
            @RequestParam(value = "size") int size){
        try {
            List<BaseLoginAppDO> loginAppDOS = loginAppService.search(fields, filters, sorts, page, size);
            long count = loginAppService.getCount(filters);
            return PageEnvelop.getSuccessListWithPage(CommonContant.Message.GET_SUCCESS,loginAppDOS,page,size,count);
        }catch (Exception e){
            e.printStackTrace();
        }
        return Envelop.getError(CommonContant.Message.GET_ERROR);
    }

    @GetMapping(value = "findLoginCompoentList")
    @ApiOperation(value = "获取登录终端列表")
    public Envelop findLoginCompoentList (
            @ApiParam(name = "loginAppId", value = "登录应用id")
            @RequestParam(value = "loginAppId", required = true) String loginAppId){
        try {
            String filters = "loginAppId="+loginAppId;
            List<BaseLoginComponentDO> componentDOList =loginAppService.findByLoginAppId(loginAppId);
            return ListEnvelop.getSuccess(CommonContant.Message.GET_SUCCESS, componentDOList);
        }catch (Exception e){
            e.printStackTrace();
        }
        return Envelop.getError(CommonContant.Message.GET_ERROR);
    }


    @PostMapping(value = "createLoginCompoent")
    @ApiOperation(value = "新增/修改登录终端")
    public Envelop createLoginCompoent (
            @ApiParam(name = "jsonData", value = "Json数据", required = true)
            @RequestParam String jsonData){
        try {
            List<BaseLoginComponentDO> list = new ArrayList<>();
            JSONArray jsonArray = JSON.parseArray(jsonData);
            for (int i=0;i<jsonArray.size();i++){
                String json = jsonArray.getString(i);
                BaseLoginComponentDO loginAppDO = toEntity(json, BaseLoginComponentDO.class);
                loginAppDO = loginAppService.createLoginCompoent(loginAppDO);
                list.add(loginAppDO);
            }
            return ListEnvelop.getSuccess(CommonContant.Message.SAVE_SUCCESS,list);
        }catch (Exception e){
            e.printStackTrace();
        }
        return Envelop.getError(CommonContant.Message.SAVE_ERROR);
    }

    @PostMapping(value = "loginComponentDelete")
    @ApiOperation(value = "删除登录终端")
    public Envelop loginComponentDelete(
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id") String id) {
        try {
            loginAppService.loginComponentDelete(id);
            return Envelop.getSuccess("删除成功");
        }catch (Exception e){
            e.printStackTrace();
        }
        return Envelop.getError("删除失败");
    }

}
