package com.yihu.jw.base.endpoint.radio;

import com.yihu.jw.base.service.radio.WlyyRadioService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @author xyq
 * @create 2022-07-04 13:39
 * @slogan 他化自在，我自逍遥
 */
@RestController
@RequestMapping(value = "/raido/manager")
@Api(value = "收音机音频", description = "收音机音频管理接口", tags = {"wlyy基础服务 - 收音机音频管理接口"})
public class WlyyRadioEndpoint extends EnvelopRestEndpoint {

    @Autowired
    private WlyyRadioService radioService;

    @ApiOperation(value = "分页获取收音机列表")
    @GetMapping( "/getRadioPage")
    public PageEnvelop getRadioPage(@ApiParam(name = "name",value = "名字搜索") @RequestParam(required = false) String name,
                                    @ApiParam(name = "frequency",value = "频率搜索") @RequestParam(required = false) String frequency,
                                    @ApiParam(name = "page",value = "页码") @RequestParam(required = false,defaultValue = "1")Integer page,
                                    @ApiParam(name = "pageSize",value = "每页大小") @RequestParam(required = false,defaultValue = "20")Integer pageSize){

        try {
            return radioService.getRadioPage(name, frequency, page, pageSize);
        }catch (Exception e){
            return failedPageEnvelopException2(e);
        }
    }

    @ApiOperation(value = "修改收音机音频")
    @PostMapping("/updateRadio")
    public Envelop updateRadio(@ApiParam(name = "id",value = "id")@RequestParam(required = false) Long id,
                               @ApiParam(name = "name",value = "频道名称")@RequestParam String name,
                               @ApiParam(name = "frequency",value = "频率")@RequestParam String frequency,
                               @ApiParam(name = "introduction",value = "简介")@RequestParam(required = false) String introduction,
                               @ApiParam(name = "url",value = "播放地址")@RequestParam String url,
                               @ApiParam(name = "sortNum",value = "排序")@RequestParam(required = false,defaultValue = "1") Integer sortNum,
                               @ApiParam(name = "status",value = "1可用，0禁用")@RequestParam(required = false,defaultValue = "1") Integer status){
        try {
            return radioService.updateRadio(id,name, frequency, introduction, url, sortNum, status);
        }catch (Exception e){
            return failedException2(e);
        }
    }


    @ApiOperation(value = "新增收音机音频")
    @PostMapping("/addRadio")
    public Envelop addRadio(@ApiParam(name = "name",value = "频道名称")@RequestParam String name,
                           @ApiParam(name = "frequency",value = "频率")@RequestParam String frequency,
                           @ApiParam(name = "introduction",value = "简介")@RequestParam(required = false) String introduction,
                           @ApiParam(name = "url",value = "播放地址")@RequestParam String url,
                           @ApiParam(name = "sortNum",value = "排序")@RequestParam(required = false,defaultValue = "1") Integer sortNum,
                           @ApiParam(name = "status",value = "1可用，0禁用")@RequestParam(required = false,defaultValue = "1") Integer status){
        try {
            return radioService.updateRadio(null,name, frequency, introduction, url, sortNum, status);
        }catch (Exception e){
            return failedException2(e);
        }
    }

    @ApiOperation(value = "删除收音机音频")
    @PostMapping("/deleteRadio")
    public Envelop deleteRadio(@ApiParam(name = "id",value = "id")@RequestParam Long id){
        try {
            return radioService.deleteRadio(id);
        }catch (Exception e){
            return failedException2(e);
        }
    }

}
