package com.yihu.jw.base.endpoint.servicePackage;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.base.service.servicePackage.ServicePackageItemService;
import com.yihu.jw.base.util.ConstantUtils;
import com.yihu.jw.entity.base.servicePackage.ServicePackageItemDO;
import com.yihu.jw.entity.hospital.consult.WlyyHospitalSysDictDO;
import com.yihu.jw.hospital.dict.WlyyHospitalSysDictDao;
import com.yihu.jw.restmodel.qvo.ParamQvo;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.net.URLDecoder;
import java.util.List;
import java.util.Map;

/**
 * @author yeshijie on 2021/2/23.
 */
@RestController
@RequestMapping(value = "servicePackageItem", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(value = "服务项管理", description = "服务项管理服务接口", tags = {"wlyy基础服务 - 服务项管理服务接口"})
public class ServicePackageItemEndpoint extends EnvelopRestEndpoint {

    @Autowired
    private ServicePackageItemService servicePackageItemService;
    @Autowired
    private WlyyHospitalSysDictDao wlyyHospitalSysDictDao;


    /**
     * 查询所有服务项
     */
    @GetMapping(value = "getItemList")
    @ApiOperation(value = "查询所有服务项")
    public PageEnvelop<Map<String, Object>> getItemList(
            @ApiParam(name = "jsonStr", value = "Json数据", required = false) @RequestParam String jsonStr
    ) throws Exception {
        jsonStr = URLDecoder.decode(jsonStr, "UTF-8");
        System.out.println("入参:" + JSON.toJSONString(jsonStr));
        ParamQvo qvo = JSON.parseObject(jsonStr, ParamQvo.class);
        Map<String, Object> map = servicePackageItemService.getItemList(qvo);
        List<Map<String, Object>> list = (List<Map<String, Object>>) map.get("list");
        Integer count = (Integer) map.get("count");
        return success(list, count, qvo.getPage(), qvo.getPageSize());
    }


    @PostMapping(value = "create")
    @ApiOperation(value = "创建")
    public Envelop create(
            @ApiParam(name = "jsonData", value = "Json数据", required = true)
            @RequestParam String jsonData) throws Exception {
        ServicePackageItemDO servicePackageItemDO = toEntity(jsonData, ServicePackageItemDO.class);
        JSONObject result = servicePackageItemService.create(servicePackageItemDO);
        if (StringUtils.endsWithIgnoreCase(ConstantUtils.FAIL, result.getString("response"))) {
            return failed(result.getString("msg"));
        }
        return success(result.get("msg"));
    }


    /**
     * 改变状态 del改成0
     */
    @PostMapping(value = "changeState")
    @ApiOperation(value = "改变状态")
    public ObjEnvelop changeState(
            @ApiParam(name = "jsonStr", value = "jsonStr", required = true) @RequestParam String jsonStr) throws Exception {
        jsonStr = URLDecoder.decode(jsonStr, "UTF-8");
        ParamQvo qvo = JSON.parseObject(jsonStr, ParamQvo.class);
        if (org.apache.commons.lang3.StringUtils.isBlank(qvo.getId())) {
            return failed("ID不能为空", ObjEnvelop.class);
        }
        System.out.println("参数:" + JSON.toJSONString(qvo));
        ServicePackageItemDO entity = servicePackageItemService.changeState(qvo);
        return success(entity);
    }


    @PostMapping(value = "delete")
    @ApiOperation(value = "删除")
    public Envelop delete(
            @ApiParam(name = "ids", value = "id串，中间用,分隔", required = true) @RequestParam(value = "ids") String ids) throws Exception {
        servicePackageItemService.deleteById(ids);
        return success("删除成功");
    }



    //----------------------------------------------------------------------------------------------
    //
    //----------------------------------------------------------------------------------------------

    @GetMapping(value = "list")
    @ApiOperation(value = "获取列表")
    public ListEnvelop<ServicePackageItemDO> list(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段") @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件") @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档") @RequestParam(value = "sorts", required = false) String sorts) throws Exception {
        List<ServicePackageItemDO> servicePackageItemDOS = servicePackageItemService.search(fields, filters, sorts);
        return success(servicePackageItemDOS);
    }


    @GetMapping(value = "page")
    @ApiOperation(value = "获取分页")
    public PageEnvelop<ServicePackageItemDO> page(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1")
            @RequestParam(value = "page") int page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15")
            @RequestParam(value = "size") int size) throws Exception {
        List<ServicePackageItemDO> servicePackageItemDOS = servicePackageItemService.search(fields, filters, sorts, page, size);
        for (ServicePackageItemDO servicePackageItemDO : servicePackageItemDOS) {
            List<WlyyHospitalSysDictDO> wlyyHospitalSysDictDO = wlyyHospitalSysDictDao.findByDictName("isEffect");
            for (WlyyHospitalSysDictDO wlyyHospitalSysDictDO1 : wlyyHospitalSysDictDO) {
                if (servicePackageItemDO.getDel().equalsIgnoreCase(wlyyHospitalSysDictDO1.getDictCode())) {
                    servicePackageItemDO.setDelName(wlyyHospitalSysDictDO1.getDictValue());
                }
            }
        }
        int count = (int) servicePackageItemService.getCount(filters);
        return success(servicePackageItemDOS, count, page, size);
    }


}
