package com.yihu.jw.base.endpoint.servicePackage;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.support.ExcelTypeEnum;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.base.service.servicePackage.ServicePackageSubItemService;
import com.yihu.jw.base.util.ConstantUtils;
import com.yihu.jw.base.vo.excel.SubItemExcelVo;
import com.yihu.jw.entity.base.servicePackage.ServicePackageSubItemDO;
import com.yihu.jw.restmodel.qvo.ParamQvo;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.List;
import java.util.Map;


@RestController
@RequestMapping(value = "servicePackageSubItem", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(value = "服务子项管理", description = "服务子项管理", tags = {"wlyy基础服务 - 服务项管理服务接口"})
public class ServicePackageSubItemEndpoint extends EnvelopRestEndpoint {

    @Autowired
    private ServicePackageSubItemService subItemService;


    /**
     * 查询所有服务子项
     */
    @GetMapping(value = "getItemList")
    @ApiOperation(value = "查询所有服务项")
    public PageEnvelop<Map<String, Object>> getSubItemList(
            @ApiParam(name = "jsonStr", value = "Json数据", required = true) @RequestParam String jsonStr
    ) throws Exception {
        jsonStr = URLDecoder.decode(jsonStr, "UTF-8");
        ParamQvo qvo = JSON.parseObject(jsonStr, ParamQvo.class);
        Map<String, Object> map = subItemService.getSubItemList(qvo);
        List<Map<String, Object>> list = (List<Map<String, Object>>) map.get("list");
        Integer count = (Integer) map.get("count");
        return success(list, count, qvo.getPage(), qvo.getPageSize());
    }


    @PostMapping(value = "createSubItem")
    @ApiOperation(value = "创建服务子项")
    public Envelop createSubItem(
            @ApiParam(name = "jsonStr", value = "Json数据", required = true) @RequestParam String jsonStr) throws Exception {
        jsonStr = URLDecoder.decode(jsonStr, "UTF-8");
        ServicePackageSubItemDO subItemDO = toEntity(jsonStr, ServicePackageSubItemDO.class);
        JSONObject result = subItemService.createSubItem(subItemDO);
        if (StringUtils.endsWithIgnoreCase(ConstantUtils.FAIL, result.getString("response"))) {
            return failed(result.getString("msg"));
        }
        return success(result.get("msg"));
    }


    /**
     * 改变状态
     */
    @PostMapping(value = "changeState")
    @ApiOperation(value = "改变状态")
    public ObjEnvelop changeState(
            @ApiParam(name = "jsonStr", value = "jsonStr", required = true) @RequestParam String jsonStr) throws Exception {
        try {
            jsonStr = URLDecoder.decode(jsonStr, "UTF-8");
            ParamQvo qvo = JSON.parseObject(jsonStr, ParamQvo.class);
            if (org.apache.commons.lang3.StringUtils.isBlank(qvo.getId())) {
                return failed("ID不能为空", ObjEnvelop.class);
            }
            System.out.println("参数:" + JSON.toJSONString(qvo));
            ServicePackageSubItemDO entity = subItemService.changeState(qvo);
            return success(entity);
        } catch (Exception e) {
            e.printStackTrace();
            return failedObjEnvelopException(e);
        }
    }


    @PostMapping(value = "deleteById")
    @ApiOperation(value = "删除")
    public Envelop delete(
            @ApiParam(name = "ids", value = "id串，中间用,分隔", required = true) @RequestParam(value = "ids") String ids) throws Exception {
        subItemService.deleteById(ids);
        return success("删除成功");
    }


    /**
     * 导入数据
     */
    @PostMapping(value = "/importSubItemExcel")
    public Envelop importSubItemExcel(
            @ApiParam(value = "文件", required = true)
            @RequestParam(value = "file", required = true) MultipartFile file) {
        try {
            //获取文件的输入流
            InputStream inputStream = file.getInputStream();
            List<SubItemExcelVo> list = EasyExcel.read(inputStream)
                    .head(SubItemExcelVo.class)
                    .sheet(0)
                    .headRowNumber(1)
                    .doReadSync();

            subItemService.importSubItemExcel(list);
            return success("导入成功!");
        } catch (Exception exception) {
            System.out.println(exception);
//            throw new RuntimeException(exception);
            return failed("导入失败!");
        }
    }


    /**
     * 导出数据
     */
    @GetMapping("/exportSubItemExcel")
    public void exportSubItemExcel(
            HttpServletResponse response,
            @ApiParam(name = "jsonStr", value = "jsonStr", required = false) @RequestParam(value = "jsonStr") String jsonStr
    ) throws Exception {
        OutputStream outputStream = response.getOutputStream();
        try {
            jsonStr = URLDecoder.decode(jsonStr, "UTF-8");
            System.out.println("入参:" + JSON.toJSONString(jsonStr));
            ParamQvo qvo = JSON.parseObject(jsonStr, ParamQvo.class);
            this.setExcelResponseProp(response, "服务子项");
            List<SubItemExcelVo> resultList = subItemService.getSubItemVoList(qvo);

            EasyExcel.write(outputStream, SubItemExcelVo.class)
                    .excelType(ExcelTypeEnum.XLSX)
                    .sheet("服务子项")
                    .doWrite(resultList);
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            outputStream.flush();
            outputStream.close();
        }
    }

    /**
     * 设置响应结果
     */
    private void setExcelResponseProp(HttpServletResponse response, String rawFileName) throws Exception {
        //设置内容类型
        response.setContentType("application/vnd.vnd.ms-excel");
        //设置编码格式
        response.setCharacterEncoding("utf-8");
        //设置导出文件名称（避免乱码）
        String fileName = URLEncoder.encode(rawFileName.concat(".xlsx"), "UTF-8");
        // 设置响应头
        response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName);
    }


}
