package com.yihu.jw.base.endpoint.user;

import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.base.service.user.UserService;
import com.yihu.jw.base.util.ConstantUtils;
import com.yihu.jw.entity.base.user.UserDO;
import com.yihu.jw.restmodel.base.user.UserVO;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.base.BaseRequestMapping;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * Endpoint - 后台管理员
 * Created by progr1mmer on 2018/8/16.
 */
@RestController
@RequestMapping(value = BaseRequestMapping.User.PREFIX)
@Api(value = "用户管理", description = "用户管理服务接口", tags = {"wlyy基础服务 - 用户管理服务接口"})
public class UserEndpoint extends EnvelopRestEndpoint {

    @Autowired
    private UserService userService;

    @Value("${wechat.id}")
    private String wechatId;

    @PostMapping(value = BaseRequestMapping.User.CREATE)
    @ApiOperation(value = "创建")
    public Envelop create (
            @ApiParam(name = "jsonData", value = "Json数据", required = true)
            @RequestParam String jsonData){
        try {
            JSONObject jsonObject =  userService.createUser(jsonData);
            if (jsonObject.getString("response").equalsIgnoreCase(ConstantUtils.FAIL)) {
                return failed(jsonObject.getString("msg"));
            }
            return success(jsonObject.getString("msg"));
        }catch (Exception e){
            e.printStackTrace();
            return Envelop.getError("操作错误");
        }
    }

    @PostMapping(value = BaseRequestMapping.User.DELETE)
    @ApiOperation(value = "删除")
    public Envelop delete(
            @ApiParam(name = "ids", value = "id串，中间用,分隔", required = true)
            @RequestParam(value = "ids") String ids) {
        try {
            userService.delete(ids.split(","));
            return success("删除成功");
        }catch (Exception e){
            e.printStackTrace();
            return Envelop.getError("操作错误");
        }
    }

    @PostMapping(value = BaseRequestMapping.User.UPDATE)
    @ApiOperation(value = "更新")
    public Envelop update (
                @ApiParam(name = "jsonData", value = "Json数据")
                @RequestParam(value = "jsonData", required = false) String jsonData) {
        try {
            JSONObject jsonObject =  userService.updateUser(jsonData);
            if (jsonObject.getString("response").equalsIgnoreCase(ConstantUtils.FAIL)) {
                return failed(jsonObject.getString("msg"));
            }
            return success(jsonObject.getString("msg"));
        }catch (Exception e){
            e.printStackTrace();
            return Envelop.getError("操作错误");
        }
    }

    @GetMapping(value = BaseRequestMapping.User.PAGE)
    @ApiOperation(value = "获取分页")
    public PageEnvelop<UserVO> page (
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1")
            @RequestParam(value = "page") int page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15")
            @RequestParam(value = "size") int size){
        try {
            List<UserDO> userDOS = userService.search(fields, filters, sorts, page, size);
            int count = (int)userService.getCount(filters);
            return success(userDOS, count, page, size, UserVO.class);
        }catch (Exception e){
            e.printStackTrace();
            return PageEnvelop.getError("操作错误");
        }
    }

    @GetMapping(value = BaseRequestMapping.User.LIST)
    @ApiOperation(value = "获取列表")
    public ListEnvelop<UserVO> list (
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts) {
        try {
            List<UserDO> userDOS = userService.search(fields, filters, sorts);
            return success(userDOS, UserVO.class);
        }catch (Exception e){
            e.printStackTrace();
            return ListEnvelop.getError("操作错误");
        }
    }

    @GetMapping(value = BaseRequestMapping.User.CHECK_USERNAME)
    @ApiOperation(value = "检查用户名是否可用(message=available代表可用，message=inavailable代表不可用)")
    public Envelop checkName (
            @ApiParam(name = "username", value = "用户名", required = true)
            @RequestParam(value = "username", required = false) String username){
        try {
            if (userService.existUsername(username)) {
                return success("inavailable");
            } else {
                return success("available");
            }
        }catch (Exception e){
            e.printStackTrace();
            return Envelop.getError("操作错误");
        }
    }

    @GetMapping(value = BaseRequestMapping.User.CHECK_MOBILE)
    @ApiOperation(value = "检查手机号是否可用(message=available代表可用，message=inavailable代表不可用)")
    public Envelop checkMobile (
            @ApiParam(name = "mobile", value = "手机号", required = true)
            @RequestParam(value = "mobile", required = false) String mobile){

        try {
            if (userService.existMobile(mobile)) {
                return success("inavailable");
            } else {
                return success("available");
            }
        }catch (Exception e){
            e.printStackTrace();
            return Envelop.getError("操作错误");
        }
    }

    @GetMapping(value = BaseRequestMapping.User.findUserBaseInfo)
    @ApiOperation(value = "获取登录基本信息")
    public ObjEnvelop<Map<String,Object>> findUserBaseInfo(@ApiParam(name = "id", value = "用户id", required = true)
                                                               @RequestParam(value = "id", required = false)String id){
        try {
            return success(BaseRequestMapping.User.api_success,userService.findUserBaseInfo(id));
        }catch (Exception e){
            e.printStackTrace();
            return ObjEnvelop.getError("操作错误");
        }
    }

    /**
     * 获取单个用户信息
     *
     * @param id
     * @return
     * @throws Exception
     */
    @GetMapping(value = BaseRequestMapping.User.queryOne)
    @ApiOperation(value = "获取单个用户信息")
    public Envelop doctorHosplist(
            @ApiParam(name = "id", value = "获取单个用户信息")
            @RequestParam(value = "id", required = true) String id){
        try {
            JSONObject jsonObject = userService.getOneUserInfo(id);
            if (jsonObject.getString("response").equalsIgnoreCase(ConstantUtils.FAIL)) {
                return failed(jsonObject.getString("msg"));
            }
            return success(jsonObject.getJSONObject("msg"));
        }catch (Exception e){
            e.printStackTrace();
            return Envelop.getError("操作错误");
        }
    }

    /**
     * 获取用户管理机构树 省/市/区/机构 树形结构数据
     *
     * @param saasid
     * @return
     * @throws Exception
     */
    @GetMapping(value = BaseRequestMapping.User.orgTreeInfo)
    @ApiOperation(value = "获取用户管理机构树 省/市/区/机构 树形结构数据")
    public Envelop getOrgDutyTree(
            @ApiParam(name = "saasid", value = "saasid")
            @RequestParam(value = "saasid", required = false) String saasid,
            @ApiParam(name = "userId", value = "userId")
            @RequestParam(value = "userId", required = false) String userId,
            @ApiParam(name = "roleId", value = "roleId")
            @RequestParam(value = "roleId", required = false) String roleId){
        try {
            JSONObject jsonObject = userService.generateUserOrgTree(saasid,userId,roleId);
            if (jsonObject.getString("response").equalsIgnoreCase(ConstantUtils.FAIL)) {
                return failed(jsonObject.getString("msg"));
            }
            return success(jsonObject.get("msg"));
        }catch (Exception e){
            e.printStackTrace();
            return Envelop.getError("操作错误");
        }
    }

    /**
     * 获取用户管理机构树 省/市/区/机构 树形结构数据
     * 新增社区树
     * @param saasid
     * @return
     * @throws Exception
     */
    @GetMapping(value = BaseRequestMapping.User.orgChoiceInfo)
    @ApiOperation(value = "获取用户管理机构树 省/市/区/机构 树形结构数据(新增社区树)")
    public Envelop orgChoiceInfo(
            @ApiParam(name = "saasid", value = "saasid")
            @RequestParam(value = "saasid", required = false) String saasid,
            @ApiParam(name = "userId", value = "userId")
            @RequestParam(value = "userId", required = false) String userId,
            @ApiParam(name = "roleId", value = "roleId")
            @RequestParam(value = "roleId", required = false) String roleId){
        try {
            JSONObject jsonObject = userService.orgChoiceInfo(saasid,userId,roleId);
            if (jsonObject.getString("response").equalsIgnoreCase(ConstantUtils.FAIL)) {
                return failed(jsonObject.getString("msg"));
            }
            return success(jsonObject.get("msg"));
        }catch (Exception e){
            e.printStackTrace();
            return Envelop.getError("操作错误");
        }
    }

    /**
     * 获取用户信息列表
     *
     * @param saasid
     * @return
     * @throws Exception
     */
    @GetMapping(value = BaseRequestMapping.User.userInfoList)
    @ApiOperation(value = "获取用户信息列表")
    public PageEnvelop userInfoList(
            @ApiParam(name = "realName", value = "realName")
            @RequestParam(value = "realName", required = false) String realName,
            @ApiParam(name = "saasid", value = "saasid")
            @RequestParam(value = "saasid", required = false) String saasid,
            @ApiParam(name = "roleId", value = "roleId")
            @RequestParam(value = "roleId", required = false) String roleId,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1")
            @RequestParam(value = "page") int page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15")
            @RequestParam(value = "size") int size){
        try {
            JSONObject result = userService.userInfoList(realName,saasid, roleId,page,size,wechatId);
            return success(result.getJSONArray("msg"),result.getInteger("count"),page,size);
        }catch (Exception e){
            e.printStackTrace();
            return PageEnvelop.getError("操作错误");
        }
    }

    /**
     * 生效或失效某个用户
     *
     * @param id
     * @param status
     * @return
     * @throws Exception
     */
    @PostMapping(value = BaseRequestMapping.User.lockOrUnlock)
    @ApiOperation(value = "生效或失效某个用户")
    public Envelop enableOrDisableOrg(
            @ApiParam(name = "id", value = "用户标识")
            @RequestParam(value = "id", required = true) String id,
            @ApiParam(name = "status", value = "生效或失效标识")
            @RequestParam(value = "status", required = true) boolean status){
        try {
            JSONObject jsonObject =  userService.lockOrUnlocakUser(id, status);
            if (jsonObject.getString("response").equalsIgnoreCase(ConstantUtils.FAIL)) {
                return failed(jsonObject.getString("msg"));
            }
            return success(jsonObject.getString("response"));
        }catch (Exception e){
            e.printStackTrace();
            return Envelop.getError("操作错误");
        }
    }

    @PostMapping(value = "enableOrDisable")
    @ApiOperation(value = "生效失效某个用户(新写接口-按钮)")
    public Envelop enableOrDisable(
            @ApiParam(name = "id", value = "用户标识")
            @RequestParam(value = "id", required = true) String id,
            @ApiParam(name = "status", value = "生效或失效标识")
            @RequestParam(value = "status", required = true) boolean status) {
        try {
            JSONObject jsonObject =  userService.enableOrDisable(id, status);
            if (jsonObject.getString("response").equalsIgnoreCase(ConstantUtils.FAIL)) {
                return failed(jsonObject.getString("msg"));
            }
            return success(jsonObject.getString("response"));
        }catch (Exception e){
            e.printStackTrace();
            return Envelop.getError("操作错误");
        }
    }


    /**
     * 重置密码
     *
     * @param id
     * @return
     * @throws Exception
     */
    @PostMapping(value = BaseRequestMapping.User.resetPwd)
    @ApiOperation(value = "重置密码")
    public Envelop resetPwd(
            @ApiParam(name = "id", value = "用户标识")
            @RequestParam(value = "id", required = true) String id){
        try {
            JSONObject jsonObject =  userService.resetPassword(id);
            if (jsonObject.getString("response").equalsIgnoreCase(ConstantUtils.FAIL)) {
                return failed(jsonObject.getString("msg"));
            }
            return success(jsonObject.getString("msg"));
        }catch (Exception e){
            e.printStackTrace();
            return Envelop.getError("操作错误");
        }
    }
    
    @PostMapping(value = BaseRequestMapping.User.updatePwd)
    @ApiOperation(value = "修账号密码")
    public Envelop updatePwd(@ApiParam(name = "id", value = "用户ID")
                                  @RequestParam(value = "id", required = true)String id,
                                  @ApiParam(name = "pw", value = "密码")
                                  @RequestParam(value = "pw", required = true)String pw,
                                  @ApiParam(name = "orgPw", value = "原密码")
                                  @RequestParam(value = "orgPw", required = true)String orgPw){
        try {
            return success(userService.updatePwd(id,pw,orgPw));
        }catch (Exception e){
            e.printStackTrace();
            return Envelop.getError("操作错误");
        }
    }
}
