package com.yihu.jw.base.service.doctor;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.base.dao.doctor.BaseDoctorRoleDictDao;
import com.yihu.jw.base.util.ConstantUtils;
import com.yihu.jw.doctor.dao.BaseDoctorRoleDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorRoleDictDO;
import com.yihu.jw.mysql.query.BaseJpaService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.io.IOException;
import java.util.Date;
import java.util.List;

@Service
public class BaseDoctorRoleDictService extends BaseJpaService<BaseDoctorRoleDictDO, BaseDoctorRoleDao> {
    @Autowired
    private BaseDoctorRoleDictDao baseDoctorRoleDictDao;
    @Autowired
    private ObjectMapper objectMapper;

    /**
     * 创建角色信息
     *
     * @param jsonData
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public JSONObject createRoleInfo(String jsonData) {
        JSONObject result = new JSONObject();
        if (StringUtils.isEmpty(jsonData)) {
            result.put("message", "参数错误");
            result.put("status", -1);
            return result;
        }
        JSONObject role = JSONObject.parseObject(jsonData);
        if(StringUtils.isEmpty(role.getString("name")) || role.getString("name").length() > 50){
            result.put("msg", "角色名称必填，但是请不要超过50个字符！");
            result.put("status", -1);
            return result;
        }
        //组装角色信息
        BaseDoctorRoleDictDO roleInfoDO = null;
        try {
            roleInfoDO = objectMapper.readValue(role.toJSONString(), BaseDoctorRoleDictDO.class);
            roleInfoDO.setCreateTime(new Date());
        } catch (IOException e) {
            e.printStackTrace();
            result.put("msg", "类型转换失败");
            result.put("status", -1);
            return result;
        }
        baseDoctorRoleDictDao.save(roleInfoDO);
        result.put("status", 200);
        result.put("obj", roleInfoDO);
        result.put("msg", "sucess");
        return result;
    }


    /**
     * 修改角色
     *
     * @param jsonData
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public JSONObject updateRoleInfo(String jsonData) {
        JSONObject result = new JSONObject();
        if (StringUtils.isEmpty(jsonData)) {
            result.put("msg", "参数错误");
            result.put("status", -1);
            return result;
        }
        JSONObject role = JSONObject.parseObject(jsonData);
        //修改角色信息
        BaseDoctorRoleDictDO roleInfoDO = null;
        BaseDoctorRoleDictDO oldRoleInfo = null;
        try {
            roleInfoDO = objectMapper.readValue(role.toJSONString(), BaseDoctorRoleDictDO.class);

        } catch (IOException e) {
            e.printStackTrace();
            result.put("msg", "类型转换失败");
            result.put("status", -1);
            return result;
        }
        try {
            oldRoleInfo = baseDoctorRoleDictDao.findById(roleInfoDO.getId()).orElse(null);

        } catch (Exception e) {
            result.put("msg", "角色id不存在");
            result.put("status", -1);
            return result;
        }
        oldRoleInfo.setCode(roleInfoDO.getCode());
        oldRoleInfo.setName(roleInfoDO.getName());
        oldRoleInfo.setDel(roleInfoDO.getDel());
        baseDoctorRoleDictDao.save(oldRoleInfo);

        result.put("status", 200);
        result.put("msg", roleInfoDO);
        return result;
    }



    /**
     * 获取单个角色信息
     * @param roleId 角色id
     * @return
     */
    public JSONObject getOneRoleInfo(String roleId){
        JSONObject result = new JSONObject();
        if(StringUtils.isEmpty(roleId)){
            result.put("msg","参数错误");
            result.put("status", -1);
            return result;
        }

        //角色基本信息
        BaseDoctorRoleDictDO roleInfoDO = baseDoctorRoleDictDao.findById(roleId).orElse(null);;
        if(null == roleInfoDO){
            result.put("msg", "角色id不存在");
            result.put("status", -1);
            return result;
        }
        result.put("status", 200);
        result.put("obj",roleInfoDO);
        result.put("msg","success");
        return result;
    }



    /**
     * 生效或失效单个角色
     * @param roleId
     * @param del
     * @return
     */
    public JSONObject enableOrDisable(String roleId,String del){
        JSONObject result = new JSONObject();
        if(StringUtils.isEmpty(roleId) || StringUtils.isEmpty(del)){
            result.put("msg","参数错误");
            result.put("status", -1);
            return result;
        }
        BaseDoctorRoleDictDO roleInfoDO = baseDoctorRoleDictDao.findById(roleId).orElse(null);;
        if( null == roleInfoDO ){
            result.put("msg", "角色id不存在");
            result.put("status", -1);
            return result;
        }
        roleInfoDO.setDel(del);
        baseDoctorRoleDictDao.save(roleInfoDO);
        result.put("status", 200);
        return result;
    }

    /**
     * 判断某一租户下角色名称是否存在
     * @param saasId
     * @param name
     * @return
     */
    public JSONObject existNameOfSaas(String saasId,String name){
        JSONObject result = new JSONObject();
        if(StringUtils.isEmpty(saasId) || StringUtils.isEmpty(name)){
            result.put("msg","parameter saasId or name is null");
            result.put("response",ConstantUtils.FAIL);
            return result;
        }
        List<BaseDoctorRoleDictDO> roleInfoDO = baseDoctorRoleDictDao.getByname(name);
        result.put("response",200);
        result.put("msg",null == roleInfoDO);
        return result;
    }

}
