package com.yihu.jw.base.service.interfacemg;


import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.base.dao.interfacemg.BaseThirdUserDao;
import com.yihu.jw.base.dao.interfacemg.BaseThirdUserInterfaceDao;
import com.yihu.jw.base.dao.interfacemg.BaseThirdUserInterfaceLogDao;
import com.yihu.jw.base.dao.interfacemg.BaseThirdUserInterfaceStandardDao;
import com.yihu.jw.entity.third.interfacemg.BaseThirdUserDO;
import com.yihu.jw.entity.third.interfacemg.BaseThirdUserInterfaceDO;
import com.yihu.jw.entity.third.interfacemg.BaseThirdUserInterfaceLogDO;
import com.yihu.jw.entity.third.interfacemg.BaseThirdUserInterfaceStandardDO;
import com.yihu.jw.mysql.query.BaseJpaService;
import com.yihu.jw.utils.hibernate.HibenateUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 第三方接口配置
 *
 */
@Service
public class BaseUserInterfaceService extends BaseJpaService<BaseThirdUserInterfaceDO, BaseThirdUserInterfaceDao> {


    @Autowired
    private BaseThirdUserDao baseThirdUserDao;
    @Autowired
    private BaseThirdUserInterfaceDao baseThirdUserInterfaceDao;
    @Autowired
    private BaseThirdUserInterfaceStandardDao baseThirdUserInterfaceStandardDao;
    @Autowired
    private BaseThirdUserInterfaceLogDao baseThirdUserInterfaceLogDao;
    @Autowired
    private HibenateUtils hibenateUtils;

    /**
     * 新增/更新第三方用户信息
     * @param userDO
     * @return
     */
    public BaseThirdUserDO userSaveAndUpdate(BaseThirdUserDO userDO){
        userDO.setCreateTime(new Date());
        userDO.setUpdateTime(new Date());
        return baseThirdUserDao.save(userDO);
    }

    /**
     * 获取第三方某个用户配置详情
     * @param id
     * @return
     */
    public BaseThirdUserDO findUserById(String id){
        return baseThirdUserDao.findById(id).orElse(null);
    }

    /**
     * 获取第三方接口某个接口详情
     * @param id
     * @return
     */
    public BaseThirdUserInterfaceDO findUserInterfaceById(String id){
        return baseThirdUserInterfaceDao.findById(id).orElse(null);
    }

    /**
     * 获取第三方接口某个接口标准详情
     * @param id
     * @return
     */
    public BaseThirdUserInterfaceStandardDO findUserInterfaceStandardById(String id){
        return baseThirdUserInterfaceStandardDao.findById(id).orElse(null);
    }

    /**
     * 获取第三方接口某个接口日志详情
     * @param id
     * @return
     */
    public BaseThirdUserInterfaceLogDO findUserInterfaceLogById(String id){
        return baseThirdUserInterfaceLogDao.findById(id).orElse(null);
    }


    /**
     * 获取第三方用户配置秘钥
     * @param condition
     * @param page
     * @param size
     * @return
     */
    public JSONObject selectUserByCondition(String condition,Integer page,Integer size){
        String sql = "select id as \"id\",app_id as \"appId\",app_key as \"appKey\",url as \"url\",name as \"name\"," +
                "status as \"status\",del as \"del\" from base_third_user where 1=1 ";
        String conditionSql = "";
        if (StringUtils.isNoneBlank(condition)){
            conditionSql = " and name like '%"+condition+"%' ";
        }
        String sqlCount = "select count(1) as \"total\" from base_third_user where 1=1  ";

        List<Map<String,Object>> mapList = hibenateUtils.createSQLQuery(sql+conditionSql,page,size);
        List<Map<String,Object>> totalMapList = hibenateUtils.createSQLQuery(sqlCount+conditionSql);
        Integer total = 0;
        if (totalMapList!=null&&totalMapList.size()!=0){
            total = Integer.parseInt(totalMapList.get(0).get("total").toString());
        }
        JSONObject object = new JSONObject();
        object.put("total",total);
        object.put("list",mapList);
        object.put("page",page);
        object.put("size",size);
        return object;
    }


    /**
     * 新增/更新第三方接口信息
     * @param userInterfaceDO
     * @return
     */
    public BaseThirdUserInterfaceDO userInterfaceSaveAndUpdate(BaseThirdUserInterfaceDO userInterfaceDO){
        userInterfaceDO.setCreateTime(new Date());
        userInterfaceDO.setUpdateTime(new Date());
        return baseThirdUserInterfaceDao.save(userInterfaceDO);
    }


    /**
     * 获取第三方用户配置秘钥
     * @param condition
     * @param page
     * @param size
     * @return
     */
    public JSONObject selectUserInterfaceByCondition(String condition,String userId,Integer page,Integer size){
        String sql = "SELECT\n" +
                "\tuser_id AS \"userId\",\n" +
                "\tid AS \"id\",\n" +
                "\tCODE AS \"code\",\n" +
                "\tNAME AS \"name\",\n" +
                "\turi AS \"uri\",\n" +
                "\tremark as \"remark\",\n" +
                "\trequest_mode AS \"requestMode\",\n" +
                "\tresponse_type AS \"responseType\",\n" +
                "\tSTATUS AS \"status\",\n" +
                "\tadapter_version AS \"adapterVersion\",\n" +
                "\tdel as \"del\"\n" +
                "FROM\n" +
                "\tbase_third_user_interface where 1=1 ";
        String conditionSql = "";
        if (StringUtils.isNoneBlank(condition)){
            conditionSql += " and name like '%"+condition+"%' ";
        }
        if (StringUtils.isNoneBlank(userId)){
            conditionSql += " and user_id = '"+userId+"' ";
        }
        String sqlCount = "select count(1) as \"total\" from base_third_user_interface where 1=1  ";

        List<Map<String,Object>> mapList = hibenateUtils.createSQLQuery(sql+conditionSql,page,size);
        List<Map<String,Object>> totalMapList = hibenateUtils.createSQLQuery(sqlCount+conditionSql);
        Integer total = 0;
        if (totalMapList!=null&&totalMapList.size()!=0){
            total = Integer.parseInt(totalMapList.get(0).get("total").toString());
        }
        JSONObject object = new JSONObject();
        object.put("total",total);
        object.put("list",mapList);
        object.put("page",page);
        object.put("size",size);
        return object;
    }


    /**
     * 新增/更新第三方接口标准信息
     * @param userInterfaceStandardDOList
     * @return
     */
    public BaseThirdUserInterfaceStandardDO userInterfaceStandardsaveAndUpdate(List<BaseThirdUserInterfaceStandardDO> userInterfaceStandardDOList){
        BaseThirdUserInterfaceStandardDO userInterfaceStandardDO = new BaseThirdUserInterfaceStandardDO();
        for (BaseThirdUserInterfaceStandardDO interfaceStandardDO:userInterfaceStandardDOList){
            interfaceStandardDO.setCreateTime(new Date());
            interfaceStandardDO.setUpdateTime(new Date());
            userInterfaceStandardDO = baseThirdUserInterfaceStandardDao.save(interfaceStandardDO);

        }
        return userInterfaceStandardDO;
    }


    /**
     * 获取第三方接口标准信息
     * @param condition
     * @param page
     * @param size
     * @return
     */
    public JSONObject selectUserInterfaceStandardByCondition(String condition,String userInterfaceId,String stdDatasetId,Integer page,Integer size){
        String sql = "SELECT\n" +
                "\tid as \"id\",\n" +
                "\tuser_interface_id AS \"userInterfaceId\",\n" +
                "\tstd_dataset_id AS \"stdDatasetId\",\n" +
                "\tstd_metadata_id AS \"stdMetadataId\",\n" +
                "\tstd_metadata_code AS \"stdMetadataCode\",\n" +
                "\tstd_metadata_name as \"stdMetadataName\",\n" +
                "\tadapter_metadata_code AS \"adapterMetadataCode\",\n" +
                "\tadapter_metadata_name AS \"adapterMetadataName\",\n" +
                "\tstatus AS \"status\",\n" +
                "\tdel as \"del\"\n" +
                "FROM\n" +
                "\tbase_third_user_interface_standard where 1=1 ";
        String conditionSql = "";
        if (StringUtils.isNoneBlank(condition)){
            conditionSql += " and (std_metadata_name like '%"+condition+"%' or std_metadata_name like '%\"+condition+\"%') ";
        }
        if (StringUtils.isNoneBlank(stdDatasetId)){
            conditionSql += " and std_dataset_id = '"+stdDatasetId+"' ";
        }
        if (StringUtils.isNoneBlank(userInterfaceId)){
            conditionSql += " and user_interface_id = '"+userInterfaceId+"' ";
        }
        String sqlCount = "select count(1) as \"total\" from base_third_user_interface_standard where 1=1  ";

        List<Map<String,Object>> mapList = hibenateUtils.createSQLQuery(sql+conditionSql,page,size);
        List<Map<String,Object>> totalMapList = hibenateUtils.createSQLQuery(sqlCount+conditionSql);
        Integer total = 0;
        if (totalMapList!=null&&totalMapList.size()!=0){
            total = Integer.parseInt(totalMapList.get(0).get("total").toString());
        }
        JSONObject object = new JSONObject();
        object.put("total",total);
        object.put("list",mapList);
        object.put("page",page);
        object.put("size",size);
        return object;
    }


    /**
     * 新增/更新第三方接口请求日志
     * @param baseThirdUserInterfaceLogDO
     * @return
     */
    public BaseThirdUserInterfaceLogDO userInterfaceLogsaveAndUpdate(BaseThirdUserInterfaceLogDO baseThirdUserInterfaceLogDO){
        baseThirdUserInterfaceLogDO.setCreateTime(new Date());
        baseThirdUserInterfaceLogDO.setUpdateTime(new Date());
        return baseThirdUserInterfaceLogDao.save(baseThirdUserInterfaceLogDO);
    }


    /**
     * 获取第三方接口请求日志
     * @param condition
     * @param page
     * @param size
     * @return
     */
    public JSONObject selectUserInterfaceLogByCondition(String condition,String userInterfaceId,Integer page,Integer size){
        String sql = "SELECT\n" +
                "\tid as \"id\",\n" +
                "\tuser_interface_id as \"userInterfaceId\",\n" +
                "\turl as \"url\",\n" +
                "\trequest as \"request\",\n" +
                "\tresponse as \"response\",\n" +
                "\tstatus as \"status\",\n" +
                "\tremark as \"remark\"\n" +
                "FROM\n" +
                "\tbase_third_user_interface_log where 1=1 ";
        String conditionSql = "";
        if (StringUtils.isNoneBlank(condition)){
        }
        if (StringUtils.isNoneBlank(userInterfaceId)){
            conditionSql += " and user_interface_id = '"+userInterfaceId+"' ";
        }
        String sqlCount = "select count(1) as \"total\" from base_third_user_interface_log where 1=1  ";

        List<Map<String,Object>> mapList = hibenateUtils.createSQLQuery(sql+conditionSql,page,size);
        List<Map<String,Object>> totalMapList = hibenateUtils.createSQLQuery(sqlCount+conditionSql);
        Integer total = 0;
        if (totalMapList!=null&&totalMapList.size()!=0){
            total = Integer.parseInt(totalMapList.get(0).get("total").toString());
        }
        JSONObject object = new JSONObject();
        object.put("total",total);
        object.put("list",mapList);
        object.put("page",page);
        object.put("size",size);
        return object;
    }
}
