package com.yihu.jw.base.service.lifeCare;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.base.dao.lifeCare.LifeCareItemDictDao;
import com.yihu.jw.entity.care.lifeCare.LifeCareItemDictDO;
import com.yihu.jw.restmodel.web.PageEnvelop;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

/**
 * Created by Bing on 2022/3/9.
 */
@Service
public class LifeCareService {
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private LifeCareItemDictDao lifeCareItemDictDao;


    /**
     * 1.7.6
     * 一个服务项会对应多个机构
     * 获取服务项目列表
     */
    public PageEnvelop<List<Map<String,Object>>> getServeDictList(String orgName, String itemName, Integer page, Integer size){
        String sql = "SELECT dict.id,dict.`name`,dict.sort,dict.`code`,dict.life_care_img as lifeCareImg,dict.pad_icon_img as padIconImg,GROUP_CONCAT( DISTINCT dict.org_code SEPARATOR ',' ) AS orgCode" +
                ",GROUP_CONCAT( DISTINCT dict.org_name SEPARATOR ',' ) AS orgName FROM " +
                "base_life_care_item_dict dict INNER JOIN base_org org ON dict.org_code = org.CODE WHERE  org.del = 1" ;

        String sqlCount = " select count(dict.id) AS size  from base_life_care_item_dict dict INNER JOIN base_org org on dict.org_code = org.code  where  org.del=1  GROUP BY dict.`code`  ";
        String filter = " ";
        if (StringUtils.isNotBlank(orgName)){
            filter +=" and org.name like '%"+orgName+"%' ";
        }
        if (StringUtils.isNotBlank(itemName)){
            filter +=" and dict.name like '%"+itemName+"%' ";
        }
        List<Map<String, Object>> conuntList = jdbcTemplate.queryForList(sqlCount + filter);

        filter +=" GROUP BY dict.`code` ";

        filter += " order by dict.sort,dict.org_code ASC ";
        filter += " limit "+(page-1)*size+","+size;

        List<Map<String,Object>> list = jdbcTemplate.queryForList(sql+filter);

        return PageEnvelop.getSuccessListWithPage("success",list,page,size,Long.parseLong(conuntList.size()+""));
    }

    @Transactional(rollbackFor = Exception.class)
    public String saveLifeItem(String json) {
        List<LifeCareItemDictDO> lifeCareItemDictDO = JSONArray.parseArray(json, LifeCareItemDictDO.class);
        if (lifeCareItemDictDO.size() > 0 && StringUtils.isBlank(lifeCareItemDictDO.get(0).getId())) {
            String code = "";
            Boolean notExist = true;
            while (notExist) {
                code = RandomStringUtils.randomAlphanumeric(10);
                notExist = findServeItemByCode(code);
            }
            for (LifeCareItemDictDO careItemDictDO : lifeCareItemDictDO) {
                careItemDictDO.setCode(code);
            }
        }

        String lifeItemCode = lifeCareItemDictDO.get(0).getCode();
        String delOrgLifeItemSql = "DELETE FROM base_life_care_item_dict where code ='"+lifeItemCode+"' ";
        jdbcTemplate.update(delOrgLifeItemSql);

        lifeCareItemDictDao.saveAll(lifeCareItemDictDO);

        return "success";
    }

    public String updDelLifeItem(String code, Integer status, String orgCode) {

        String sql = "select * from base_life_care_item_dict  where code = '" + code + "' ";

        if (StringUtils.isNotBlank(orgCode)) {
            sql += " and orgCode = '" + orgCode + "'";
        }

        List<LifeCareItemDictDO> lifeCareItemDictDOList = jdbcTemplate.queryForList(sql, LifeCareItemDictDO.class);
        if (lifeCareItemDictDOList.size() < 1) {
            return " 未找到改服务项目 ";
        }
        lifeCareItemDictDOList.stream().forEach(lifeCareItemDictObj -> lifeCareItemDictObj.setDel(status));
        lifeCareItemDictDao.saveAll(lifeCareItemDictDOList);
        return "success";
    }

    @Transactional(rollbackFor = Exception.class)
    public String delLifeItem(String code, String orgCode){
        String sql = "DELETE FROM base_life_care_item_dict where code = '"+code+"'";

        if (StringUtils.isNotBlank(orgCode)) {
            sql+= " and orgCode ='"+orgCode+"'";
        }

        jdbcTemplate.update(sql);

        return "success";
    }

    public JSONObject getLifeItemDetail(String code){
        JSONObject result = new JSONObject();
        String lifeSql = "SELECT id,code,name,sort,del*1 as del,life_care_img AS lifeCareImg,pad_icon_img as padIconImg,org_code AS orgCode,org_name AS orgName,charge_standard AS chargeStandard," +
                "serve_standard AS serveStandard,remark FROM base_life_care_item_dict WHERE `code` = '"+code+"' ";

        List<Map<String, Object>> lifeList = jdbcTemplate.queryForList(lifeSql);

        if (lifeList.size()>0) {
            JSONObject lifeItemObj = new JSONObject();
            lifeItemObj.put("name",lifeList.get(0).get("name"));
            lifeItemObj.put("code",lifeList.get(0).get("code"));
            lifeItemObj.put("sort",lifeList.get(0).get("sort"));
            lifeItemObj.put("del",lifeList.get(0).get("del"));
            lifeItemObj.put("lifeCareImg",lifeList.get(0).get("lifeCareImg"));
            lifeItemObj.put("padIconImg",lifeList.get(0).get("padIconImg"));
            result.put("lifeItem",lifeItemObj);
        }
        result.put("data",lifeList);
        return result;
    }

    public boolean findServeItemByCode(String code){
        String sql = " select count(id) from base_life_care_item_dict where code='"+code+"' " ;
        Integer exist = jdbcTemplate.queryForObject(sql,Integer.class);
        return exist>0?true:false;
    }

}
