package com.yihu.jw.base.service.menu;

import com.yihu.jw.base.dao.menu.BaseAppMenuDao;
import com.yihu.jw.entity.base.menu.BaseAppMenu;
import com.yihu.jw.mysql.query.BaseJpaService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.ParseException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Created by yeshijie on 2022/9/27.
 */
@Service
public class BaseAppMenuService extends BaseJpaService<BaseAppMenu,BaseAppMenuDao> {

    @Resource
    private BaseAppMenuDao baseAppMenuDao;

    /**
     * 验证资源报表分类编码是否唯一
     *
     * @param id   主键
     * @param code 编码
     * @return true：唯一，false：不唯一
     */
    public Boolean isUniqueCode(String id, String code) {
        BaseAppMenu rsReportCategory = baseAppMenuDao.isUniqueCode(id, code);
        return rsReportCategory == null;
    }

    /**
     * 根据父级ID，获取其子节点；父级ID为 -1 时，返回顶级节点。
     * @param pid 父级ID。
     * @return 子节点集合
     */
    public List<BaseAppMenu> getChildrenByPid(String pid,Integer type) {
        List<BaseAppMenu> children;
        if (BaseAppMenu.topPid.equals(pid)) {
            children = baseAppMenuDao.getTopParents(type);
        } else {
            children = baseAppMenuDao.getChildrenByPid(pid,type);
        }
        return children;
    }

    /**
     * 根据父级集合，递归获取父级及其自子级集合，形成树形结构
     * @param parentList 父级集合
     * @return 父级及其子集的树形结构数据
     */
    public List<BaseAppMenu> getTreeByParents(List<BaseAppMenu> parentList,Integer type) {
        List<BaseAppMenu> resultList = new ArrayList<>();
        for (int i = 0; i < parentList.size(); i++) {
            BaseAppMenu parent = parentList.get(i);
            List<BaseAppMenu> childList = this.getChildrenByPid(parent.getId(),type);
            List<BaseAppMenu> childTreeList = getTreeByParents(childList,type);
            parent.setChildren(childTreeList);
            resultList.add(parent);
        }

        return resultList;
    }

    public List<BaseAppMenu> getAllTree(Integer type){
        //获取所有菜单
        List<BaseAppMenu> menus = baseAppMenuDao.findAllByType(type);
        //菜单按pid分组
        Map<String,List<BaseAppMenu>> pidMap = menus.stream().collect(Collectors.groupingBy(BaseAppMenu::getPid));
        //排序
        for (List<BaseAppMenu> list:pidMap.values()){
            list.sort(Comparator.comparingInt(BaseAppMenu::getSort));
        }
        //设置自己的子菜单
        menus.forEach(menu->menu.setChildren(pidMap.get(menu.getId())));
        //过滤 保留顶级菜单
        return menus.stream().filter(m-> BaseAppMenu.topPid.equals(m.getPid())).collect(Collectors.toList());
    }

    /**
     * 递归查询子节点
     * @param root  根节点
     * @param all   所有节点
     * @return 根节点信息
     */
    private List<BaseAppMenu> getChildrens(BaseAppMenu root, List<BaseAppMenu> all) {
        List<BaseAppMenu> children = all.stream().filter(m -> {
            return Objects.equals(m.getPid(), root.getId());
        }).map((m) -> {
                    m.setChildren(getChildrens(m, all));
                    return m;
                }
        ).collect(Collectors.toList());
        return children;
    }

    /**
     * 递归不满足条件的父级集合，获取其满足条件的子集，并返回子集及其父级的树形结构
     *
     * @param parents  不满足条件的父级集合
     * @param codeName 资源报表分类编码或名称
     * @return 满足条件的子集及其父级的树形结构
     */
    public List<BaseAppMenu> getTreeByParentsAndCodeName(List<BaseAppMenu> parents, String codeName,Integer type) throws ParseException {
        List<BaseAppMenu> treeData = new ArrayList<>();
        for (BaseAppMenu parent : parents) {
            String parentId = parent.getId();
            List<BaseAppMenu> childrenTree = new ArrayList<>();

            List<BaseAppMenu> children = this.getChildrenByPid(parentId,type);
            if (children.size() == 0) continue;

            // 获取满足条件的子节点
            String filters = "pid=" + parentId + ";code?" + codeName + " g1;name?" + codeName + " g1;";
            List<BaseAppMenu> childrenIin = (List<BaseAppMenu>) this.search(filters);
            if (childrenIin.size() != 0) {
                childrenTree.addAll(getTreeByParents(childrenIin,type));
            }
            // 递归不满足条件的子节点
            children.removeAll(childrenIin);
            if (children.size() != 0) {
                childrenTree.addAll(getTreeByParentsAndCodeName(children, codeName,type));
            }

            if (childrenTree.size() != 0) {
                parent.setChildren(childrenTree);
                treeData.add(parent);
            }
        }
        return treeData;
    }
}
