package com.yihu.jw.base.service.video;

import com.yihu.jw.base.dao.video.BaseVideoDao;
import com.yihu.jw.base.dao.video.BaseVideoGroupDao;
import com.yihu.jw.contant.CommonContant;
import com.yihu.jw.entity.care.video.BaseVideoDO;
import com.yihu.jw.entity.care.video.BaseVideoGroupDO;
import com.yihu.jw.mysql.query.BaseJpaService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author xyq
 * @create 2022-07-04 16:16
 * @slogan 他化自在，我自逍遥
 */
@Service
public class BaseVideoGroupService extends BaseJpaService<BaseVideoGroupDO, BaseVideoGroupDao> {

    @Autowired
    private BaseVideoDao videoDao;
    @Autowired
    private BaseVideoGroupDao videoGroupDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;


    /**
     * 分页获取视频栏目列表
     * @param name
     * @param page
     * @param pageSize
     * @return
     */
    public PageEnvelop getVideoGroupPage(String name,Integer page,Integer pageSize){
        page = page>0?page-1:0;
        String colSql = "SELECT id,name,sort_num sortNum,CAST(status AS UNSIGNED) status,CAST(DATE_FORMAT(create_time,'%Y-%m-%d %H:%i:%S') as char) createTime ";
        String countSql = "select count(*) ";
        String sql = " from base_video_group where 1 = 1 ";
        if(StringUtils.isNotBlank(name)){
            sql += " and name like '%"+name+"%' ";
        }
        String orderSql = " ORDER BY sort_num desc limit "+page*pageSize+","+pageSize;
        List<Map<String, Object>> list = jdbcTemplate.queryForList(colSql + sql + orderSql);
        Long count = jdbcTemplate.queryForObject(countSql + sql, Long.class);
        return PageEnvelop.getSuccessListWithPage("查询成功",list,page,pageSize,count);
    }

    /**
     * 修改视频栏目
     * @param id
     * @param name
     * @param sortNum
     * @param status
     * @return
     */
    public Envelop updateVideoGroup(Long id,String name,Integer sortNum,Integer status){
        if(StringUtils.isBlank(name)){
            return Envelop.getError("栏目名称不能为空");
        }
        BaseVideoGroupDO videoGroupDO = new BaseVideoGroupDO();
        if(id != null){
            videoGroupDO.setId(id);
        }
        videoGroupDO.setName(name);
        videoGroupDO.setSortNum(sortNum);
        videoGroupDO.setStatus(status);
        videoGroupDO.setCreateTime(new Date());
        videoGroupDao.save(videoGroupDO);
        return Envelop.getSuccess(CommonContant.Message.SAVE_SUCCESS);
    }

    /**
     * 删除视频栏目
     * @param id
     * @return
     */
    public Envelop deleteVideoGroup(Long id){
        String sql = "SELECT count(v.id) count FROM base_video v INNER JOIN base_video_group g ON v.type = g.id WHERE g.id = '"+id+"' ";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        if (Integer.parseInt(list.get(0).get("count").toString())>0){
        return Envelop.getError("该栏目下面存在视频,请勿删除！",500);
        }
        BaseVideoGroupDO byId = videoGroupDao.findById(id);
        if(byId != null){
            videoGroupDao.delete(byId);
        }
        return Envelop.getSuccess("删除成功");
    }


    /**
     * 分页获取视频列表
     * @param title
     * @param type
     * @param page
     * @param pageSize
     * @return
     */
    public PageEnvelop getVideoPage(String title,Integer type,Integer page,Integer pageSize){
        page = page>0?page-1:0;
        String colSql = "SELECT v.id,v.type,v.title,v.url,v.img,CAST(DATE_FORMAT(v.create_time,'%Y-%m-%d %H:%i:%S') as char) createTime, " +
                " v.introduction,v.sort_num sortNum,CAST(v.status as UNSIGNED) status,g.name ";
        String countSql = "SELECT count(*) ";
        String fromSql = " from base_video v left join base_video_group g on v.type = g.id where 1 = 1 ";
        if(StringUtils.isNotBlank(title)){
            fromSql += " and v.title like '%"+title+"%' ";
        }
        if(type != null && type > 0){
            fromSql += " and v.type = "+type;
        }
        Long count = jdbcTemplate.queryForObject(countSql + fromSql, Long.class);
        fromSql += " order by v.sort_num,v.create_time desc limit "+page*pageSize+","+pageSize;
        List<Map<String, Object>> list = jdbcTemplate.queryForList(colSql + fromSql);
        return PageEnvelop.getSuccessListWithPage("查询成功",list,page,pageSize,count);
    }

    /**
     * 修改视频信息
     * @param id
     * @param type
     * @param title
     * @param url
     * @param img
     * @param introduction
     * @param sortNum
     * @param status
     * @return
     */
    public Envelop updateVideo(String id,Integer type,String title,String url,String img,String introduction,Integer sortNum,Integer status){
        if(StringUtils.isBlank(title) || type == null || StringUtils.isBlank(url)){
            return Envelop.getError("必填参数不能为空");
        }
        BaseVideoDO videoDO = new BaseVideoDO();
        if(StringUtils.isNotBlank(id)){
            videoDO.setId(id);
        }
        videoDO.setTitle(title);
        videoDO.setType(type.toString());
        videoDO.setUrl(url);
        videoDO.setImg(img);
        videoDO.setIntroduction(introduction);
        videoDO.setSortNum(sortNum);
        videoDO.setStatus(status);
        videoDO.setCreateTime(new Date());
        videoDao.save(videoDO);
        return Envelop.getSuccess(CommonContant.Message.SAVE_SUCCESS);
    }

    /**
     * 删除视频
     * @param id
     * @return
     */
    public Envelop deleteVideo(String id){
        BaseVideoDO video = videoDao.findById(id).orElse(null);
        if(video != null){
            videoDao.delete(video);
        }
        return Envelop.getSuccess("删除成功");
    }
}
