package com.yihu.jw.basic.agadmin.controller.quota;

import com.yihu.jw.constants.ErrorCode;
import com.yihu.jw.basic.quota.service.TjQuotaCategoryService;
import com.yihu.jw.basic.quota.service.TjQuotaService;
import com.yihu.jw.entity.ehr.quota.TjQuotaCategory;
import com.yihu.jw.restmodel.ehr.tj.MQuotaCategory;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by wxw on 2017/8/31.
 */
@RestController
@RequestMapping("/quota")
@Api( tags = {"网关-统计查询"})
public class QuotaCategoryController extends EnvelopRestEndpoint {

    @Autowired
    private TjQuotaCategoryService quotaCategoryService;
    @Autowired
    private TjQuotaService tjQuotaService;

    @GetMapping("/getQuotaCategoryList")
    public Object getQuotaCategoryList(String name, String searchParm,Integer id, int page, int rows){
        String url = "/quotaCategory/pageList";
        StringBuffer stringBuffer = new StringBuffer();
        if (!StringUtils.isEmpty(name)) {
            stringBuffer.append("name?" + name + " g1;code?" + name + " g1;");
        }
        if (!StringUtils.isEmpty(searchParm)) {
            stringBuffer.append("name?" + searchParm + ";");
        }
        if (!StringUtils.isEmpty(id)) {
            stringBuffer.append("id<>" + id + ";");
        }
        String filters = stringBuffer.toString();

        try {
            PageEnvelop pageEnvelop = new PageEnvelop();
            List<TjQuotaCategory> quotaCategoryList = quotaCategoryService.search(null, filters, null, page, rows);
            if (quotaCategoryList != null){
                pageEnvelop.setDetailModelList(quotaCategoryList);
                pageEnvelop.setTotalCount((int)quotaCategoryService.getCount(filters));
                pageEnvelop.setStatus(200);
                pageEnvelop.setCurrPage(page);
                pageEnvelop.setPageSize(rows);
            } else{
                pageEnvelop.setStatus(200);
                pageEnvelop.setMessage("查询无数据");
                pageEnvelop.setTotalCount(0);
            }
            return pageEnvelop;
        } catch (Exception ex) {
            ex.printStackTrace();
            return failed(ERR_SYSTEM_DES);
        }
    }

    @RequestMapping("deleteQuotaCategory")
    public Object deleteQuotaCategory(Integer id) {
        String url = "/basic/api/v1.0/quotaCategory/delete";
        Map<String, Object> params = new HashMap<>();
        params.put("id", id);
        try {
            TjQuotaCategory tjQuotaCategory = quotaCategoryService.getById(id);
            if (null == tjQuotaCategory) {
                return failed( "操作对象不存在", ErrorCode.OBJECT_NOT_FOUND.value());
            }
            if (quotaCategoryService.findByField("parentId", id).size() > 0) {
                return failed("含有子分类，不能删除");
            }
            if (tjQuotaService.findByField("quotaType", id).size() > 0) {
                return failed("含有子类，不能删除");
            }
            quotaCategoryService.deleteQuotaCategory(id);
            return success(true);
        } catch (Exception e) {
            e.printStackTrace();
            return failed(e.getMessage());
        }
    }

    @GetMapping("detailById")
    public Envelop getTjQuotaById(Integer id ) {
        String url ="/quotaCategory/detailById";
        Map<String, Object> params = new HashMap<>();
        params.put("id", id);
        MQuotaCategory detailModel = null;
        try {
            TjQuotaCategory quotaCategory = quotaCategoryService.getById(id);
            return ObjEnvelop.getSuccess("查询成功",quotaCategory);
        } catch (Exception e) {
            e.printStackTrace();
            return failed("查询失败");
        }
    }

    @RequestMapping("/addOrUpdateQuotaCategory")
    public Envelop addOrUpdateQuotaCategory(String mode, String jsonDate){
        try{
            TjQuotaCategory quotaCategory = toEntity(jsonDate, TjQuotaCategory.class);
            if("new".equals(mode)){
                if (quotaCategoryService.getCountByName(quotaCategory.getName())>0){
                    return failed("名称不唯一");
                }
                if (quotaCategoryService.getCountByCode(quotaCategory.getCode())>0){
                    return failed("编码不唯一");
                }

                quotaCategory = quotaCategoryService.saveQuotaCategory(quotaCategory);
                return ObjEnvelop.getSuccess("保存成功",quotaCategory);
            } else if("modify".equals(mode)){
                quotaCategory = quotaCategoryService.updateQuotaCategory(quotaCategory);
                return ObjEnvelop.getSuccess("保存成功",quotaCategory);
            }
        }catch (Exception ex){
            ex.printStackTrace();
            return failed(ERR_SYSTEM_DES);
        }
        return failed("查询失败");
    }

    @GetMapping("hasExistsName")
    public Envelop hasExistsName(String name) {
        try {
            return ObjEnvelop.getSuccess("查询成功",quotaCategoryService.getCountByName(name)>0);
        } catch (Exception e) {
            e.getMessage();
        }
        return failed("查询失败");
    }

    @GetMapping("hasExistsCode")
    public Envelop hasExistsCode(String code) {
        try {
            return ObjEnvelop.getSuccess("查询成功",quotaCategoryService.getCountByCode(code)>0);
        } catch (Exception e) {
            e.getMessage();
        }
        return failed("查询失败");
    }

    @GetMapping("/getAllQuotaCategoryList")
    public Object getAllQuotaCategoryList(){
        String url = "/quotaCategory/getQuotaCategoryChild";
        try {
            List<TjQuotaCategory> quotaCategories = quotaCategoryService.getQuotaCategoryChild();
            return ListEnvelop.getSuccess("查询成功",quotaCategories);
        } catch (Exception ex) {
            ex.printStackTrace();
            return failed(ERR_SYSTEM_DES);
        }
    }

    @GetMapping("/getQuotaCategoryListTree")
    public Object getQuotaCategoryListTree(){
        String url = "/quotaCategory/list";
        try {
            List<TjQuotaCategory> quotaCategories = quotaCategoryService.getAllQuotaCategory();
            return ListEnvelop.getSuccess("查询成功",quotaCategories);
        } catch (Exception ex) {
            ex.printStackTrace();
            return failed(ERR_SYSTEM_DES);
        }
    }
}
