package com.yihu.jw.basic.agadmin.controller.quota;

import com.yihu.jw.basic.quota.service.TjDataSaveService;
import com.yihu.jw.entity.ehr.quota.TjDataSave;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.io.IOException;
import java.net.URLDecoder;
import java.util.Date;
import java.util.List;

/**
 * Created by llh on 2017/5/9.
 */
@RestController
@RequestMapping("/tjDataSave")
@Api( tags = {"网关-统计数据存储"})
public class TjDataSaveController extends EnvelopRestEndpoint {

    @Resource
    TjDataSaveService tjDataSaveService;

    //查询统计主维度
    @GetMapping("/getTjDataSave")
    public Envelop searchTjDataSave(String name, String searchParm,int page, int rows){
        String url = "/tj/getTjDataSaveList";
        StringBuffer stringBuffer = new StringBuffer();
        if (!StringUtils.isEmpty(name)) {
            stringBuffer.append("name?" + name );
        }
        if (!StringUtils.isEmpty(searchParm)) {
            stringBuffer.append("name?" + searchParm );
        }
        String filters = stringBuffer.toString();
        try {
            PageEnvelop PageEnvelop = new PageEnvelop();
            List<TjDataSave> tjDataSaveList = tjDataSaveService.search(null, filters, null, page, rows);
            if(tjDataSaveList != null){
                PageEnvelop.setDetailModelList(tjDataSaveList);
                PageEnvelop.setTotalCount((int)tjDataSaveService.getCount(filters));
                PageEnvelop.setStatus(200);
                PageEnvelop.setCurrPage(page);
                PageEnvelop.setPageSize(rows);
            }else{
                PageEnvelop.setStatus(200);
                PageEnvelop.setMessage("查询无数据");
                PageEnvelop.setTotalCount(0);
            }
            return PageEnvelop;
        } catch (Exception ex) {
            ex.printStackTrace();
            return failed(ERR_SYSTEM_DES);
        }
    }



    /**
     * 新增修改
     */
    @RequestMapping(value = "updateTjDataSave")
    public Envelop updateTjDataSave(String tjDataSaveModelJsonData) throws IOException {
        String url = "/tj/addTjDataSave";
        String[] strings = URLDecoder.decode(tjDataSaveModelJsonData, "UTF-8").split(";");
        try {
            TjDataSave detailModel = objectMapper.readValue(strings[0], TjDataSave.class);

            if (!StringUtils.isEmpty(detailModel.getId())) {
                Long tjDataSaveId = detailModel.getId();
                TjDataSave updateTjDataSave = tjDataSaveService.getById(tjDataSaveId);
                if (updateTjDataSave !=null ) {
                    updateTjDataSave.setCode(detailModel.getCode());
                    updateTjDataSave.setName(detailModel.getName());
                    updateTjDataSave.setType(detailModel.getType());
                    updateTjDataSave.setStatus(detailModel.getStatus());
                    updateTjDataSave.setRemark(detailModel.getRemark());

                    updateTjDataSave.setUpdateTime(new Date());
                    tjDataSaveService.save(updateTjDataSave);
                    return ObjEnvelop.getSuccess("保存成功！", updateTjDataSave);
                } else {
                    return failed("参数错误");
                }
            } else {
                detailModel.setCreateTime(new Date());
                tjDataSaveService.save(detailModel);
                return ObjEnvelop.getSuccess("保存成功",detailModel);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return failed(ERR_SYSTEM_DES);
        }
    }

    /**
     * 删除消息
     */
    @RequestMapping("deleteTjDataSave")
    public Envelop deleteTjDataSave(Long tjDataSaveId) {
        String url = "/tj/deleteTjDataSave";
        try {
            TjDataSave tjDataSave = tjDataSaveService.getById(tjDataSaveId);
            tjDataSave.setStatus(-1);
            tjDataSaveService.save(tjDataSave);
            return success("删除成功");
        } catch (Exception e) {
            e.printStackTrace();
            return failed(ERR_SYSTEM_DES);
        }
    }

    /**
     * 根据id获取消息
     */
    @GetMapping("getTjDataSaveById")
    public Envelop getTjQuotaById(Long id ) {
        String url ="/tj/getTjDataSaveById/" +id;
        try {
            TjDataSave updateTjDataSave = tjDataSaveService.getById(id);
            return ObjEnvelop.getSuccess("获取成功",updateTjDataSave);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return failed("查询失败");

    }

    /**
     * 校验name是否唯一,true已存在
     */
    @GetMapping("hasExistsName")
    public Envelop hasExistsName(String name) {
        String url = "/tj/dataSaveExistsName/"+ name ;
        try {
            String filter = "name=" + name;
            List<TjDataSave> list = tjDataSaveService.search(filter);
            return ObjEnvelop.getSuccess("查询成功",list.size() > 0);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return failed("查询失败");
    }

    /**
     * 校验code是否唯一
     */
    @GetMapping("hasExistsCode")
    public Envelop hasExistsCode(String code) {
        String url = "/tj/dataSaveExistsCode/" + code ;
        try {
            String filter = "code=" + code;
            List<TjDataSave> list = tjDataSaveService.search(filter);
            return ObjEnvelop.getSuccess("查询成功",list.size() > 0);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return  failed("查询失败");
    }
}
