package com.yihu.jw.basic.agadmin.controller.resource;

import com.yihu.jw.basic.apps.service.AppService;
import com.yihu.jw.basic.resource.dao.RsReportCategoryDao;
import com.yihu.jw.basic.resource.service.RsReportCategoryAppService;
import com.yihu.jw.basic.resource.service.RsReportCategoryService;
import com.yihu.jw.entity.ehr.apps.App;
import com.yihu.jw.entity.ehr.resource.RsReportCategory;
import com.yihu.jw.restmodel.ehr.resource.MRsReportCategory;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * 资源报表分类 controller
 *
 * @author 张进军
 * @date 2017/8/8 13:53
 */
@RestController
@RequestMapping("/resource/reportCategory")
@Api(tags = {"网关-报表分类"})
public class ReportCategoryController extends EnvelopRestEndpoint {

    @Autowired
    private RsReportCategoryAppService rsReportCategoryAppService;
    @Autowired
    private RsReportCategoryService rsReportCategoryService;
    @Autowired
    private RsReportCategoryDao rsReportCategoryDao;
    @Autowired
    private AppService appService;

    @RequestMapping("/configDialog")
    public Envelop configDialog(String id) {
        try {
            String appIds = rsReportCategoryAppService.getAppIdByCategory(id);
            return ObjEnvelop.getSuccess("获取成功",appIds);
        } catch (Exception e) {
            e.printStackTrace();
            return failed("获取成功");
        }
    }

    @RequestMapping("/appConfig")
    public Envelop appConfig(String id, String dialogType) {
        try {
            String appIds = rsReportCategoryAppService.getAppIdByCategory(id);
            return ObjEnvelop.getSuccess("获取成功",appIds);
        } catch (Exception e) {
            e.printStackTrace();
            return failed("获取成功");
        }
    }

    @RequestMapping(value = "detail")
    public Envelop detail(Integer id) {
        try {
            if (id != null) {
                return ObjEnvelop.getSuccess("查询成功",rsReportCategoryService.getById(id));
            }
            return success("查询成功");
        } catch (Exception e) {
            e.printStackTrace();
            return failed("查询失败");
        }
    }

    /**
     * 根据条件，获取资源报表分类（树形结构）
     */
    @RequestMapping("/getTreeData")
    public Envelop getTreeData(String codeName) {
        try {
            List<MRsReportCategory> resultList = new ArrayList<>();

            // 获取最顶层的资源报表分类集合
            List<RsReportCategory> topNodeList = rsReportCategoryService.getChildrenByPid(0);
            if (topNodeList.size() == 0) {
                return success("查询成功");
            }

            // 暂存最顶层资源报表分类中，满足条件的集合
            List<RsReportCategory> topNodeListIn = new ArrayList<>();
            // 暂存最顶层资源报表分类中，不满足条件的集合
            List<RsReportCategory> topNodeListOut = new ArrayList<>();

            if (StringUtils.isEmpty(codeName)) {
                List<RsReportCategory> treeList = rsReportCategoryService.getTreeByParents(topNodeList);
                return ListEnvelop.getSuccess("查询成功",convertToModels(treeList, resultList, MRsReportCategory.class, ""));
            }

            for (RsReportCategory reportCategory : topNodeList) {
                if (reportCategory.getCode().contains(codeName) || reportCategory.getName().contains(codeName)) {
                    topNodeListIn.add(reportCategory);
                    continue;
                }
                topNodeListOut.add(reportCategory);
            }
            if (topNodeListIn.size() != 0) {
                List<RsReportCategory> inList = rsReportCategoryService.getTreeByParents(topNodeListIn);
                resultList.addAll(convertToModels(inList, new ArrayList<>(), MRsReportCategory.class, ""));
            }
            List<RsReportCategory> outList = rsReportCategoryService.getTreeByParentsAndCodeName(topNodeListOut, codeName);
            resultList.addAll(convertToModels(outList, new ArrayList<>(), MRsReportCategory.class, ""));

            return ListEnvelop.getSuccess("查询成功",resultList);
        } catch (Exception ex) {
            ex.printStackTrace();
            return failed("查询失败");
        }
    }

    /**
     * 获取资源报表分类下拉框数据
     */
    @RequestMapping("/getComboTreeData")
    public Envelop getComboTreeData() {
        try {
            List<RsReportCategory> list = rsReportCategoryService.getAllTreeData();
            return ListEnvelop.getSuccess ("查询成功",convertToModels(list, new ArrayList<>(list.size()), MRsReportCategory.class, ""));
        } catch (Exception e) {
            e.printStackTrace();
            return failed("查询失败");
        }
    }

    /**
     * 保存
     */
    @RequestMapping("/save")
    public Envelop save(String data) {
        Envelop envelop = new Envelop();
        Map<String, Object> params = new HashMap<>();

        try {
            String rsReportCategory = data;
            RsReportCategory newRsReportCategory = toEntity(rsReportCategory, RsReportCategory.class);
            if (StringUtils.isEmpty(newRsReportCategory.getCode())) {
                return failed("编码不能为空！");
            }
            if (StringUtils.isEmpty(newRsReportCategory.getName())) {
                return failed("名称不能为空！");
            }

            if (newRsReportCategory.getId() == null) {
                // 新增
                if (null == newRsReportCategory.getPid()) {
                    newRsReportCategory.setPid(0);
                }
                if (null == newRsReportCategory.getSortNo()) {
                    newRsReportCategory.setSortNo(99);
                }

                if(newRsReportCategory.getId()==null){
                    //新增判断名称和code 是否唯一
                    if(!rsReportCategoryService.isUniqueName(0, newRsReportCategory.getName())){
                        return ObjEnvelop.getError("名称已存在",-1);
                    }
                    if(!rsReportCategoryService.isUniqueCode(0, newRsReportCategory.getCode())){
                        return ObjEnvelop.getError("编码已存在",-1);
                    }
                }else{
                    //修改 如果有修改name和code要判断是否唯一
                    RsReportCategory oldRsReportCategory = rsReportCategoryService.getById(newRsReportCategory.getId());
                    if(!oldRsReportCategory.getName().equals(newRsReportCategory.getName())){
                        if(!rsReportCategoryService.isUniqueName(newRsReportCategory.getId(), newRsReportCategory.getName())){
                            return ObjEnvelop.getError("名称已存在",-1);
                        }
                    }
                    if(!oldRsReportCategory.getCode().equals(newRsReportCategory.getCode())){
                        if(!rsReportCategoryService.isUniqueCode(newRsReportCategory.getId(), newRsReportCategory.getCode())){
                            return ObjEnvelop.getError("编码已存在",-1);
                        }
                    }
                }

                newRsReportCategory = rsReportCategoryService.save(newRsReportCategory);
                MRsReportCategory mRsReportCategory = convertToModel(newRsReportCategory, MRsReportCategory.class);
                return ObjEnvelop.getSuccess("操作成功",mRsReportCategory);
            } else {
                // 修改
                RsReportCategory updateModel = rsReportCategoryService.getById(newRsReportCategory.getId());

                updateModel.setCode(newRsReportCategory.getCode());
                updateModel.setName(newRsReportCategory.getName());
                updateModel.setPid(newRsReportCategory.getPid());
                updateModel.setSortNo(null == newRsReportCategory.getSortNo() ? 99 : newRsReportCategory.getSortNo());
                updateModel.setRemark(newRsReportCategory.getRemark());

                params.put("rsReportCategory", objectMapper.writeValueAsString(updateModel));

                newRsReportCategory = rsReportCategoryService.save(updateModel);
                MRsReportCategory mRsReportCategory = convertToModel(newRsReportCategory, MRsReportCategory.class);
                return ObjEnvelop.getSuccess("操作成功",mRsReportCategory);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return failed("操作失败");
        }
    }

    /**
     * 删除
     */
    @RequestMapping("/delete")
    public Envelop delete(Integer id) {
        try {
            rsReportCategoryDao.deleteById(id);
            return success("删除成功");
        } catch (Exception e) {
            e.printStackTrace();
            return failed("删除失败");
        }
    }

    /**
     * 验证资源报表分类编码是否唯一
     */
    @RequestMapping("/isUniqueCode")
    public Envelop isUniqueCode(@RequestParam Integer id, @RequestParam String code) {
        try {
            return ObjEnvelop.getSuccess("查询成功",rsReportCategoryService.isUniqueCode(id, code));
        } catch (Exception e) {
            e.printStackTrace();
            return failed("查询失败");
        }
    }

    /**
     * 验证资源报表分类名称是否唯一
     */
    @RequestMapping("/isUniqueName")
    public Envelop isUniqueName(@RequestParam Integer id, @RequestParam String name) {
        try {
            return ObjEnvelop.getSuccess("查询成功",rsReportCategoryService.isUniqueName(id, name));
        } catch (Exception e) {
            e.printStackTrace();
            return failed("查询失败");
        }
    }

    @RequestMapping("/selectAppList")
    public Envelop getSelectAppList(String searchNm,int page,int rows){
        if(StringUtils.isEmpty(searchNm)){
            return failed("id不能为空");
        }
        try {
            String filters = "id="+searchNm;
            List<App> appList = appService.search(null, filters, null, page, rows);
            return PageEnvelop.getSuccessListWithPage("",appList,page,rows,appService.getCount(filters));
        } catch (Exception e){
            e.printStackTrace();
            return failed("查询失败");
        }
    }
}
