package com.yihu.jw.basic.agadmin.controller.resource;


import com.fasterxml.jackson.core.type.TypeReference;

import com.yihu.jw.basic.resource.dao.RsResourceDao;
import com.yihu.jw.basic.resource.model.ResourceQuotaJson;
import com.yihu.jw.basic.resource.service.*;
import com.yihu.jw.entity.ehr.id.BizObject;
import com.yihu.jw.entity.ehr.quota.TjQuota;
import com.yihu.jw.entity.ehr.resource.*;
import com.yihu.jw.restmodel.ehr.resource.MRsResources;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.util.common.LogService;
import io.swagger.annotations.Api;
import org.springframework.data.domain.Page;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestClientException;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Controller - 视图管理控制器
 * Created by yww on 2016/5/27.
 */
@RestController
@RequestMapping("/resource/resourceManage")
@Api(tags = {"网关-视图管理控制器"})
public class ResourceManageController extends EnvelopRestEndpoint {

    @Resource
    private RsResourceCategoryService rsCategoryService;
    @Resource
    private RsResourceService rsResourceService;
    @Resource
    private RsAppResourceService rsAppResourceService;
    @Resource
    private RsResourceDao rsResourceDao;
    @Resource
    private RsResourceQuotaService resourceQuotaService;
    @Resource
    private RsResourceGrantService rsGrantService;
    @Resource
    private RsReportViewService rsReportViewService;

    @GetMapping("/infoInitial")
    public Envelop resourceInterfaceInfoInitial(String id,String mode,String categoryId, String name, String dataSource){
        String categoryName = "";
        try{
            if(!StringUtils.isEmpty(categoryId)) {
                RsResourceCategory rsResourceCategory = rsCategoryService.getRsCategoryById(categoryId);
                if(rsResourceCategory!=null){
                    categoryName = rsResourceCategory.getName();
                }
            }
            RsResource rsResource;
            if (!StringUtils.isEmpty(id)) {
                rsResource = rsResourceService.getResourceById(id);
            } else {
                rsResource = new RsResource();
            }
            rsResource.setCategoryName(categoryName);
            return ObjEnvelop.getSuccess("查询成功",rsResource);
        }catch (Exception e){
            e.printStackTrace();
            return failed("查询失败");
        }
    }

    @GetMapping(value = "/resourceInfo")
    public Envelop getResourceInfo(String id) throws Exception {
        RsResource  rsResource = rsResourceService.getResourceById(id);
        return ObjEnvelop.getSuccess("查询成功",rsResource);
    }


    /**
     * 配置授权浏览页面跳转
     */
    @GetMapping("/switch")
    public Envelop switchToPage(String resourceId){
        try{
            if (!StringUtils.isEmpty(resourceId)) {
                RsResource  rsResource = rsResourceService.getResourceById(resourceId);
                return ObjEnvelop.getSuccess("查询成功",rsResource);
            }
            return success();
        }catch (Exception e){
            e.printStackTrace();
            return failed("查询失败");
        }
    }

    /**
     * 资源分页查询 -- 弃用
     */
    @GetMapping("/resources")
    public Envelop searchResources(String searchNm, String categoryId, String rolesId, String appId, Integer dataSource, int page, int rows){
        String url = "/resources";
        Envelop envelop = new Envelop();
        Map<String, Object> params = new HashMap<>();
        StringBuffer stringBuffer = new StringBuffer();
        if (!StringUtils.isEmpty(searchNm)) {
            stringBuffer.append("code?" + searchNm + " g1;name?" + searchNm + " g1;");
        }
        if(dataSource != null && dataSource != 0) {
            stringBuffer.append("dataSource=" + dataSource + ";");
        }
        if(!StringUtils.isEmpty(categoryId)){
            stringBuffer.append("categoryId=" + categoryId);
        }else {
            return envelop;
        }
        params.put("filters", "");
        String filters = stringBuffer.toString();
        if (!StringUtils.isEmpty(filters)) {
            params.put("filters", filters);
        }
        if (!StringUtils.isEmpty(rolesId)) {
            params.put("rolesId", rolesId);
        }
        if (!StringUtils.isEmpty(appId)) {
            params.put("appId", appId);
        }
        params.put("page", page);
        params.put("size", rows);
        try {
            long total = 0;
            List<MRsResources> rsList;
            //过滤条件为空
            if(StringUtils.isEmpty(filters)) {
                Page<RsResource> resources = rsResourceService.getResources(null,reducePage(page),rows);
                total = resources.getTotalElements();
                return PageEnvelop.getSuccessListWithPage("查询成功",resources.toList(),page,rows,total);
            } else {
                if (!StringUtils.isEmpty(rolesId) && !StringUtils.isEmpty(appId)) {
                    String resourceIds = rsAppResourceService.getResourceIdByAppId(appId);
                    if (!StringUtils.isEmpty(resourceIds)) {
                        filters += ";id=" + resourceIds + ";" ;
                    } else {
                        filters += ";id=0000;" ;
                    }
                }
                List<RsResource> resources = rsResourceService.search(null,filters,null,page,rows);
                total = rsResourceService.getCount(filters);
                return PageEnvelop.getSuccessListWithPage("查询成功",resources,page,rows,total);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return failed(ERR_SYSTEM_DES);
        }
    }

    /**
     * 视图资源列表树
     * @param filters 过滤查询条件
     * @param dataSource 1 档案视图 2 指标视图
     */
    @GetMapping("/resources/tree")
    public Envelop getResourceTree(String filters, Integer dataSource, HttpServletRequest request) throws Exception {
        String url = "/resource/api/v1.0/resources/tree";
        //从Session中获取用户的角色和和授权视图列表作为查询参数
        Map<String, Object> params = new HashMap<>();
        if (!StringUtils.isEmpty(filters)) {
            params.put("filters", filters);
        }
        if(dataSource != null && dataSource != 0) {
            params.put("dataSource", dataSource);
        }
        List<Map<String, Object>> resultList = rsResourceService.getResourceTree(dataSource, "*", filters);
        return ListEnvelop.getSuccess("查询成功",resultList);
    }

    /**
     * 创建或更新资源
     */
    @PostMapping("/update")
    public Envelop updateResource(String dataJson, String mode){
        String url = "/resources";
        try{
            //RsResourcesModel model = objectMapper.readValue(dataJson, RsResourcesModel.class);
            Map<String, Object> resourceMap = objectMapper.readValue(dataJson, Map.class);
            if(resourceMap != null) {
                if (StringUtils.isEmpty(resourceMap.get("code"))) {
                    return failed("资源编码不能为空");
                }
                if (StringUtils.isEmpty(resourceMap.get("name"))) {
                    return failed("资源名称不能为空");
                }
                // 新增
                if ("new".equals(mode)) {
                    RsResource rs = toEntity(objectMapper.writeValueAsString(resourceMap), RsResource.class);
                    rs.setId(getObjectId(BizObject.Resources));
                    RsResource newRsResource =  rsResourceService.saveResource(rs);
                    return ObjEnvelop.getSuccess("新增成功",newRsResource);
                } else if ("modify".equals(mode)) {
                    String urlGet = "/resources/" + resourceMap.get("id");
                    RsResource rsResource = rsResourceService.getResourceById(resourceMap.get("id")+"");
                    if (rsResource==null) {
                        return failed("原资源信息获取失败");
                    }
                    resourceMap.put("creator", rsResource.getCreator());
                    Map<String, Object> params = new HashMap<>();
                    params.put("resource", objectMapper.writeValueAsString(resourceMap));
                    RsResource rs = toEntity(objectMapper.writeValueAsString(resourceMap),RsResource.class);
                    RsResource newRsResource =  rsResourceService.saveResource(rs);
                    return ObjEnvelop.getSuccess("更新成功",newRsResource);
                }
            }else {
                return failed("参数有误");
            }
        }catch (Exception e){
            LogService.getLogger(ResourceManageController.class).error(e.getMessage());
        }
        return failed("操作失败");
    }

    /**
     * 删除资源
     */
    @PostMapping("/delete")
    public Envelop deleteResource(String id) {
        String url = "/resources/" + id;
        Map<String, Object> params = new HashMap<>();
        params.put("id", id);
        try {
            if (isRsInUse(id)){
                return failed("已授权资源不能删除");
            }
            if(isRsReportInUse(id)){
                return failed("已关联报表资源不能删除");
            }
            if (isSetReport(id)){
                return failed("已配置报表不能删除");
            }
            rsResourceService.deleteResource(id);
            return success("删除成功");
        } catch (Exception e) {
            e.printStackTrace();
            return failed("删除失败");
        }
    }

    /**
     * 资源编码唯一性验证
     */
    @GetMapping("/isExistCode")
    public Envelop isExistCode(String code){
        String url = "/resources/isExistCode/"+code;
        try{
            RsResource rsResource = rsResourceDao.findByCode(code);

            return ObjEnvelop.getSuccess("查询成功",rsResource!=null);
        } catch (Exception ex){
            ex.printStackTrace();
            return failed(ERR_SYSTEM_DES);
        }
    }

    /**
     * 资源名称唯一性验证
     */
    @GetMapping("/isExistName")
    public Envelop isExistName(String name){
        String url = "/resources/isExistName";
        try{
            RsResource rsResource = rsResourceDao.findByName(name);
            return ObjEnvelop.getSuccess("查询成功",rsResource!=null);
        } catch (Exception ex){
            ex.printStackTrace();
            return failed(ERR_SYSTEM_DES);
        }
    }

    /**
     * 资源分类树-页面初始化时
     */
    @GetMapping("/categories")
    public Envelop getCategories(){
        try{
            String filters = "";
            String url = "/resources/categories/all";
            List<RsResourceCategory> resources = rsCategoryService.search(filters);
            return ListEnvelop.getSuccess("查询成功",resources);
        } catch (Exception e){
            e.printStackTrace();
        }
        return failed("查询失败");
    }

    /**
     * 带检索分页的查找资源分类方法,新增资源时
     */
    @GetMapping("/rsCategory")
    public Envelop searchRsCategory(String searchParm, HttpServletRequest request, int page, int rows){
        String url = "/resources/categories/search";
        try {
            Map<String, Object> params = new HashMap<>();
            StringBuffer stringBuffer = new StringBuffer();
            if (!StringUtils.isEmpty(searchParm)) {
                stringBuffer.append("name?" + searchParm +";");
            }
            params.put("filters", "");
            String filters = stringBuffer.toString();
            if (!StringUtils.isEmpty(filters)) {
                params.put("filters", filters);
            }

            long total = 0;
            if (StringUtils.isEmpty(filters)) {
                Page<RsResourceCategory> rsCatePage;
                rsCatePage = rsCategoryService.getRsCategories(null, reducePage(page), rows);
                total = rsCatePage.getTotalElements();
                return PageEnvelop.getSuccessListWithPage("查询成功",rsCatePage.toList(),page,rows,total);
            } else {
                List<RsResourceCategory> rsCateList;
                    rsCateList = rsCategoryService.search(null, filters, null, page, rows);
                total = rsCateList.size();
                return PageEnvelop.getSuccessListWithPage("查询成功",rsCateList,page,rows,total);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return failed("查询成功");
        }
    }

//    @RequestMapping("/getResourceQuotaInfo")
//    public Envelop getResourceQuotaInfo(String resourceId, String nameOrCode, int page, int rows){
//        String url = "/resources/getQuotaList";
//        String resultStr = "";
//        Envelop envelop = new Envelop();
//        Map<String, Object> params = new HashMap<>();
//        if (!StringUtils.isEmpty(resourceId)) {
//            params.put("filters", "resourceId=" + resourceId);
//        }
//        if (!StringUtils.isEmpty(nameOrCode)) {
//            params.put("quotaNameOrCode", nameOrCode);
//        }
//        params.put("page", page);
//        params.put("pageSize", rows);
//        try {
//            resultStr = HttpClientUtil.doGet(comUrl + url, params, username, password);
//            return resultStr;
//        } catch (Exception ex) {
//            ex.printStackTrace();
//            return failed(ERR_SYSTEM_DES);
//        }
//    }

    @PostMapping(value = "/addResourceQuota")
    public Envelop addResourceQuota(String resourceId, String jsonModel) throws IOException {
        String url = "/resourceQuota/batchAddResourceQuota";
        if (!StringUtils.isEmpty(resourceId)) {
            resourceQuotaService.deleteByResourceId(resourceId);
            return success("删除成功");
        }
        try {
            List<RsResourceQuota> list = objectMapper.readValue(jsonModel, new TypeReference<List<RsResourceQuota>>(){});
            //先删除表中resourceId=XX的数据
            if (list != null && list.size() > 0) {
                resourceQuotaService.deleteByResourceId(list.get(0).getResourceId());
            }
            for (RsResourceQuota rsResourceQuota : list) {
                resourceQuotaService.save(rsResourceQuota);
            }
            return ListEnvelop.getSuccess("操作成功",list);
        } catch (RestClientException e) {
            e.printStackTrace();
            return failed(ERR_SYSTEM_DES);
        }
    }

    /**
     * 指标上卷下钻预览
     * @param id
     * @param model
     * @param dimension  维度
     * @param quotaFilter 过滤条件 多个;拼接 如：org=123;city=001
     * @return
     */
//    @RequestMapping("/resourceShow")
//    public Envelop resourceShow(String id ,Model model ,String dimension,String quotaFilter,HttpServletRequest request) throws JsonProcessingException{
//        MChartInfoModel chartInfoModel = new MChartInfoModel();
//        String resultStr = "";
//        try {
//            Map<String, Object> params = new HashMap<>();
//            params.put("resourceId", id);
//            params.put("dimension", dimension);
//            Map<String, Object> quotaFilterMap = new HashMap<>();
//           if( !StringUtils.isEmpty(quotaFilter) ){
//               params.put("quotaFilter", quotaFilter);
//           }
//            resultStr = HttpClientUtil.doGet(comUrl + ServiceApi.Resources.GetRsQuotaPreview, params, username, password);
//            Envelop envelop = objectMapper.readValue(resultStr, Envelop.class);
//            String s = objectMapper.writeValueAsString((HashMap<String,String>)envelop.getObj());
//            chartInfoModel = objectMapper.readValue(s,MChartInfoModel.class);
//        } catch (Exception e) {
//            e.printStackTrace();
//         }
//        model.addAttribute("id", id);
//        model.addAttribute("chartInfoModel", objectMapper.writeValueAsString(chartInfoModel));
//        model.addAttribute("contentPage","/resource/resourcemanage/resoureShowCharts");
//        return "generalView";
//    }

    /**
     * 指标预览 包含上卷 下钻
     * @param id
     * @param dimension  维度
     * @param quotaFilter 过滤条件 多个;拼接 如：org=123;city=001
     * @return
     */
//    @RequestMapping("/resourceUpDown")
//    public MChartInfoModel getResourceUpDown(String id, String dimension,String quotaFilter ,HttpServletRequest request){
//        MChartInfoModel chartInfoModel = new MChartInfoModel();
//        try {
//            Envelop result = new Envelop();
//            String resultStr = "";
//            Map<String, Object> params = new HashMap<>();
//            params.put("resourceId", id);
//            params.put("dimension", dimension);
//            List<String> userOrgList  = getUserOrgSaasListRedis(request);
//            params.put("userOrgList", userOrgList);
//            Map<String, Object> quotaFilterMap = new HashMap<>();
//            if( !StringUtils.isEmpty(quotaFilter) ){
//                params.put("quotaFilter", quotaFilter);
//            }
//            resultStr = HttpClientUtil.doGet(comUrl + ServiceApi.Resources.GetRsQuotaPreview, params, username, password);
//            Envelop envelop = objectMapper.readValue(resultStr, Envelop.class);
//            String s = objectMapper.writeValueAsString((HashMap<String,String>)envelop.getObj());
//            chartInfoModel = objectMapper.readValue(s,MChartInfoModel.class);
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//        return chartInfoModel;
//    }

    /**
     * 判断资源是否已被授权
     */
    public boolean isRsInUse(String resourceId) throws Exception{
        String url = "/resources/grants/no_paging";
        Map<String,Object> params = new HashMap<>();
        params.put("filters","resourceId=" + resourceId);
        List<RsAppResource> rsGrant = rsGrantService.search("resourceId=" + resourceId);
        return rsGrant != null && rsGrant.size() > 0;
    }

    /**
     * 判断资源报表是否关联相关资源
     */
    public boolean isRsReportInUse(String resourceId) throws Exception {
        String url = "/resources/reportView/existByResourceId";
        List<RsReportView> rsReportViewList = rsReportViewService.findByResourceId(resourceId);
        return rsReportViewList != null && rsReportViewList.size() > 0;
    }

    public Boolean isSetReport(String resourceId) throws Exception{
        List<RsReportView> list = rsReportViewService.findByResourceId(resourceId);
        return list != null && list.size() > 0;
    }

    @GetMapping(value = "/getQuotaByResourceId")
    public Envelop getQuotaByResourceId(String resourceId){
        String url = "/resourceQuota/getQuotaByResourceId";
        Map<String,Object> params = new HashMap<>();
        params.put("resourceId",resourceId);
        try{
            List<TjQuota> quotaList = resourceQuotaService.getQuotaByResourceId(resourceId);
            return ListEnvelop.getSuccess("查询成功",quotaList);
        }catch (Exception ex){
            ex.printStackTrace();
            return failed(ERR_SYSTEM_DES);
        }
    }

    @PostMapping(value = "/updateResourceQuota")
    public Envelop updateResourceQuota(String jsonModel){
        String url = "/resourceQuota/updateResourceQuota";
        try{
            List<ResourceQuotaJson> list = objectMapper.readValue(jsonModel, new TypeReference<List<ResourceQuotaJson>>() {
            });
            resourceQuotaService.updateResourceQuota(list);
            return ListEnvelop.getSuccess("查询成功",list);
        }catch (Exception ex){
            ex.printStackTrace();
            return failed(ERR_SYSTEM_DES);
        }
    }
}
