package com.yihu.jw.basic.agadmin.controller.resource;

import com.yihu.jw.basic.resource.model.RsCategoryTypeTreeModel;
import com.yihu.jw.basic.resource.service.RsResourceCategoryService;
import com.yihu.jw.entity.ehr.id.BizObject;
import com.yihu.jw.entity.ehr.resource.RsResourceCategory;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.util.entity.ServiceException;
import io.swagger.annotations.Api;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

/**
 * 视图分类控制器
 * Created by AndyCai on 2015/12/14.
 */
@RequestMapping("/rscategory")
@RestController
@Api( tags = {"网关-资源分类"})
public class RsResourceCategoryController extends EnvelopRestEndpoint {

    @Autowired
    private RsResourceCategoryService rsCategoryService;

    //获取TreeData 用于初始页面显示嵌套model
    @RequestMapping("/getTreeGridData")
    public Envelop getTreeGridData(String codeName) throws Exception{
        ListEnvelop envelop = new ListEnvelop();
        //获取到的顶级cda类别集合
        String userResource = "*";
        String name = codeName;
        List<RsResourceCategory> categoryList;
        if(userResource.equals("*")) {
            categoryList = rsCategoryService.getRsCategoryByPid("");
        }else {
            categoryList = rsCategoryService.findByCodeAndPid("derived", "");
        }
        //顶级类别中符合条件的类别集合
        List<RsResourceCategory> mRsCategoriesSome = new ArrayList<>();
        //顶级类别中不符合条件的类别集合
        List<RsResourceCategory> mRsCategoriesOthers = new ArrayList<>();
        if (categoryList == null || categoryList.size() == 0){
            envelop.setStatus(-1);
            envelop.setMessage("没有匹配条件的资源类别！");
            return envelop;
        }
        List<RsCategoryTypeTreeModel> treeList = new ArrayList<>();
        if(StringUtils.isEmpty(name)){
            treeList = getRsCategoryTreeModelChild(categoryList);
            envelop.setDetailModelList(treeList);
            return envelop;
        }
        for(RsResourceCategory mRsCategory : categoryList){
            if(mRsCategory.getName().contains(name)){
                mRsCategoriesSome.add(mRsCategory);
                continue;
            }
            mRsCategoriesOthers.add(mRsCategory);
        }
        if (mRsCategoriesSome.size() != 0){
            treeList.addAll(getRsCategoryTreeModelChild(mRsCategoriesSome));
        }
        treeList .addAll(getRsCategoryTreeModelByName(mRsCategoriesOthers, name));
        envelop.setDetailModelList(treeList);
        return envelop;
    }

    /**
     *
     * 根据父级信息获取全部的子级信息（树形model）
     * @param info 父级信息
     * @return 全部子级信息
     */
    private List<RsCategoryTypeTreeModel> getRsCategoryTreeModelChild(List<RsResourceCategory> info) {
        List<RsCategoryTypeTreeModel> treeInfo = new ArrayList<>();
        for (RsResourceCategory typeInfo : info) {
            RsCategoryTypeTreeModel tree = convertToModel(typeInfo, RsCategoryTypeTreeModel.class);
            List<RsResourceCategory> categoryListChild = rsCategoryService.getRsCategoryByPid(typeInfo.getId());
            List<RsCategoryTypeTreeModel> listChildTree = getRsCategoryTreeModelChild(categoryListChild);
            tree.setChildren(listChildTree);
            treeInfo.add(tree);
        }
        return treeInfo;
    }

    /**
     * 递归不满足的父级类别集合的子集中满足条件TreeModel集合的方法
     * @param mRsCategories 不符合的父级类别的集合
     */
    private List<RsCategoryTypeTreeModel> getRsCategoryTreeModelByName(List<RsResourceCategory> mRsCategories, String name) throws ParseException {
        //结构：treeList 包含treeModel，treeModel包含listOfParent
        List<RsCategoryTypeTreeModel> treeList = new ArrayList<>();
        for(RsResourceCategory mRsCategory:mRsCategories){
            List<RsCategoryTypeTreeModel> childList = new ArrayList<>();
            RsCategoryTypeTreeModel treeModel = convertToModel(mRsCategory,RsCategoryTypeTreeModel.class);
            String pid = mRsCategory.getId();
            //获取所有下一级cda类别
            List<RsResourceCategory> listAll = rsCategoryService.getRsCategoryByPid(pid);
            if(listAll.size() == 0){
                continue;
            }
            //获取所有下一级符合要求的资源类别
            String filters ="pid="+pid+";name?"+name;

            //modify by cws
            List<RsResourceCategory> listSome = rsCategoryService.search(filters);
            //List<MRsCategory> listSome = (List<MRsCategory>)responseEntity.getBody();
            if(listSome.size()!=0){
                childList.addAll(getRsCategoryTreeModelChild(listSome));
            }
            //取剩下不符合要求的进行递归
            listAll.removeAll(listSome);
            if(listAll.size() != 0){
                childList.addAll(getRsCategoryTreeModelByName(listAll, name));
            }
            if(childList.size() != 0){
                treeModel.setChildren(childList);
                treeList.add(treeModel);
            }
        }
        return treeList;
    }

    /**
     * 新增或者更新
     */
    @RequestMapping("/saveCateType")
    public Envelop SaveCdaType(String dataJson) {
        String url = "/resources/category/update";
        try {
            RsResourceCategory rsCategory = toEntity(dataJson, RsResourceCategory.class);
            if(StringUtils.isEmpty(rsCategory.getName())){
                return failed("类别名称不能为空！");
            }
            if(StringUtils.isNotBlank(rsCategory.getId())) {
                rsCategoryService.createOrUpdRsCategory(rsCategory);
            }else{
                rsCategory.setId(getObjectId(BizObject.ResourceCategory));
                rsCategoryService.createOrUpdRsCategory(rsCategory);
            }
            return ObjEnvelop.getSuccess("新增成功",rsCategory);
        } catch (Exception ex){
            ex.printStackTrace();
            return failed("操作失败");
        }
    }

    /**
     * 删除
     */
    @RequestMapping("/delteCateTypeInfo")
    public Envelop delteCdaTypeInfo(String id) {
        if (StringUtils.isEmpty(id)){
            return failed("请选择要删除的数据");
        }
        try{
            String url = "/resources/category/" + id;
            rsCategoryService.deleteRsCategory(id);
            return success("删除成功");
        }catch (ServiceException se){
            return failed(se.getMessage());
        }catch (Exception ex){
            ex.printStackTrace();
            return failed(ERR_SYSTEM_DES);
        }
    }

    /**
     * 删除辅助接口，如有下级分类，前端给出不可删除警告
     */
    @RequestMapping("/getCateTypeByPid")
    public Envelop getCateTypeByPid(String id) {
        String url = "/resources/categories/pid/";
        try {
            List<RsResourceCategory> categoryList = rsCategoryService.getRsCategoryByPid(id);
            return ListEnvelop.getSuccess("查询成功",categoryList);
        } catch (Exception ex) {
            ex.printStackTrace();
            return failed(ERR_SYSTEM_DES);
        }
    }

    /**
     * 根据ID查找分类信息
     */
    @RequestMapping("/getCateTypeById")
    public Envelop getCdaTypeById(String strIds) {
        String url = "/resources/category/" + strIds;
        try{
            RsResourceCategory rsResourceCategory = rsCategoryService.getRsCategoryById(strIds);
            return ObjEnvelop.getSuccess("查询成功",rsResourceCategory);
        } catch (Exception e){
            e.printStackTrace();
            return ObjEnvelop.getError(ERR_SYSTEM_DES);
        }
    }

    /**
     * 获取可以作为父类别的cate类别列表
     */
    @RequestMapping("/getCateTypeExcludeSelfAndChildren")
    public Envelop getCdaTypeExcludeSelfAndChildren(String strId) {
        //页面新增修改访问的是同个接口
        try {
            String urlGetAll =  "/resources/categories/all";
            List<RsResourceCategory> resources;
            if(!StringUtils.isEmpty(strId)){
                List<RsResourceCategory> parentTypes = rsCategoryService.getRsCategoryByPid(strId);
                String childrenIds = getChildIncludeSelfByParentsAndChildrenIds(parentTypes,strId+",");   //递归获取
                resources = rsCategoryService.getCateTypeExcludeSelfAndChildren(childrenIds);
            }else {
                resources = rsCategoryService.search("");
            }
            return ListEnvelop.getSuccess("查询成功",resources);
        } catch (Exception ex){
            ex.printStackTrace();
            return failed(ERR_SYSTEM_DES);
        }

    }

    private String getChildIncludeSelfByParentsAndChildrenIds(List<RsResourceCategory> parentTypes,String childrenIds) {
        for (RsResourceCategory typeInfo : parentTypes) {
            childrenIds += typeInfo.getId() + ",";
            List<RsResourceCategory> listChild = rsCategoryService.getRsCategoryByPid(typeInfo.getId());
            if (listChild.size() > 0) {
                childrenIds = getChildIncludeSelfByParentsAndChildrenIds(listChild, childrenIds);
            }
        }
        return childrenIds;
    }

}
