package com.yihu.jw.basic.org.controller;

import com.yihu.jw.basic.org.dao.OrgHealthCategoryDao;
import com.yihu.jw.entity.ehr.org.OrgHealthCategory;
import com.yihu.jw.basic.org.service.OrgHealthCategoryService;
import com.yihu.jw.constants.ApiVersion;
import com.yihu.jw.constants.ServiceApi;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.restmodel.ehr.org.MOrgHealthCategory;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * 卫生机构类别 接口
 *
 * @author 张进军
 * @date 2017/12/21 12:00
 */
@RestController
@RequestMapping(value = ApiVersion.Version1_0)
@Api(description = "卫生机构类别接口", tags = {"机构管理--卫生机构类别接口"})
public class OrgHealthCategoryEndPoint extends EnvelopRestEndpoint {

    @Autowired
    private OrgHealthCategoryService orgHealthCategoryService;
    @Autowired
    OrgHealthCategoryDao orgHealthCategoryDao;

    @ApiOperation("根据ID获取卫生机构类别")
    @RequestMapping(value = ServiceApi.Org.HealthCategory.GetById, method = RequestMethod.GET)
    public ObjEnvelop getById(
            @ApiParam(name = "id", value = "主键", required = true)
            @PathVariable(value = "id") Long id) {
        ObjEnvelop envelop = new ObjEnvelop();
        envelop.setStatus(-1);
        try {
            MOrgHealthCategory mOrgHealthCategory = convertToModel(orgHealthCategoryService.getById(id), MOrgHealthCategory.class);
            envelop.setObj(mOrgHealthCategory);
            envelop.setStatus(200);
            envelop.setMessage("成功获取卫生机构类别。");
        } catch (Exception e) {
            e.printStackTrace();
            envelop.setMessage("获取卫生机构类别发生异常：" + e.getMessage());
        }
        return envelop;
    }

    @ApiOperation("获取所有的卫生机构类别")
    @RequestMapping(value = ServiceApi.Org.HealthCategory.FindAll, method = RequestMethod.GET)
    public ListEnvelop findAll() {
        ListEnvelop envelop = new ListEnvelop();
        envelop.setStatus(-1);
        try {
            List<OrgHealthCategory> list = orgHealthCategoryService.findAll();
            envelop.setDetailModelList(list);
            envelop.setStatus(200);
            envelop.setMessage("成功获取所有的卫生机构类别。");
        } catch (Exception e) {
            e.printStackTrace();
            envelop.setMessage("获取所有的卫生机构类别发生异常：" + e.getMessage());
        }
        return envelop;
    }

    @ApiOperation(value = "根据条件获取卫生机构类别")
    @RequestMapping(value = ServiceApi.Org.HealthCategory.Search, method = RequestMethod.GET)
    public ListEnvelop search(
            @ApiParam(name = "codeName", value = "卫生机构类别编码或名称")
            @RequestParam(value = "codeName", required = false) String codeName) {
        ListEnvelop envelop = new ListEnvelop();
        envelop.setStatus(-1);
        try {
            List<OrgHealthCategory> resultList = new ArrayList<>();

            // 获取最顶层的卫生机构类别集合
            List<OrgHealthCategory> topNodeList = orgHealthCategoryService.getChildrenByPid(-1L);
            if (topNodeList.size() == 0) {
                envelop.setDetailModelList(resultList);
                return envelop;
            }

            // 暂存最顶层卫生机构类别中，满足条件的集合
            List<OrgHealthCategory> topNodeListIn = new ArrayList<>();
            // 暂存最顶层卫生机构类别中，不满足条件的集合
            List<OrgHealthCategory> topNodeListOut = new ArrayList<>();

            if (StringUtils.isEmpty(codeName)) {
                resultList = orgHealthCategoryService.getTreeByParents(topNodeList);
                envelop.setDetailModelList(resultList);
                return envelop;
            }

            for (OrgHealthCategory reportCategory : topNodeList) {
                if (reportCategory.getCode().contains(codeName) || reportCategory.getName().contains(codeName)) {
                    topNodeListIn.add(reportCategory);
                    continue;
                }
                topNodeListOut.add(reportCategory);
            }
            if (topNodeListIn.size() != 0) {
                List<OrgHealthCategory> inList = orgHealthCategoryService.getTreeByParents(topNodeListIn);
                resultList.addAll(inList);
            }
            List<OrgHealthCategory> outList = orgHealthCategoryService.getTreeByParentsAndCodeName(topNodeListOut, codeName);
            resultList.addAll(outList);

            envelop.setDetailModelList(resultList);
            envelop.setStatus(200);
            envelop.setMessage("成功获取卫生机构类别。");
        } catch (Exception e) {
            e.printStackTrace();
            envelop.setMessage("获取卫生机构类别发生异常：" + e.getMessage());
        }
        return envelop;
    }

    @ApiOperation("新增卫生机构类别")
    @RequestMapping(value = ServiceApi.Org.HealthCategory.Save, method = RequestMethod.POST)
    public ObjEnvelop add(
            @ApiParam(value = "卫生机构类别JSON", required = true)
            @RequestBody String entityJson) {
        ObjEnvelop envelop = new ObjEnvelop();
        envelop.setStatus(-1);
        try {
            OrgHealthCategory newEntity = objectMapper.readValue(entityJson, OrgHealthCategory.class);
            newEntity.setTopPid(orgHealthCategoryService.getTopPidByPid(newEntity.getPid()));
            newEntity = orgHealthCategoryService.save(newEntity);

            MOrgHealthCategory mOrgHealthCategory = convertToModel(newEntity, MOrgHealthCategory.class);
            envelop.setObj(mOrgHealthCategory);
            envelop.setStatus(200);
            envelop.setMessage("成功新增卫生机构类别。");
        } catch (Exception e) {
            e.printStackTrace();
            envelop.setMessage("新增卫生机构类别发生异常：" + e.getMessage());
        }
        return envelop;
    }

    @ApiOperation("更新卫生机构类别")
    @RequestMapping(value = ServiceApi.Org.HealthCategory.Save, method = RequestMethod.PUT)
    public ObjEnvelop update(
            @ApiParam(value = "卫生机构类别JSON", required = true)
            @RequestBody String entityJson) {
        ObjEnvelop envelop = new ObjEnvelop();
        envelop.setStatus(-1);
        try {
            OrgHealthCategory updateEntity = objectMapper.readValue(entityJson, OrgHealthCategory.class);
            updateEntity = orgHealthCategoryService.save(updateEntity);

            MOrgHealthCategory mOrgHealthCategory = convertToModel(updateEntity, MOrgHealthCategory.class);
            envelop.setObj(mOrgHealthCategory);
            envelop.setStatus(200);
            envelop.setMessage("成功更新卫生机构类别。");
        } catch (Exception e) {
            e.printStackTrace();
            envelop.setMessage("更新卫生机构类别发生异常：" + e.getMessage());
        }
        return envelop;
    }

    @ApiOperation("删除卫生机构类别")
    @RequestMapping(value = ServiceApi.Org.HealthCategory.Delete, method = RequestMethod.DELETE)
    public Envelop delete(
            @ApiParam(name = "id", value = "卫生机构类别ID", required = true)
            @RequestParam(value = "id") Long id) {
        Envelop envelop = new Envelop();
        envelop.setStatus(-1);
        try {
            orgHealthCategoryDao.deleteById(id);

            envelop.setStatus(200);
            envelop.setMessage("成功删除卫生机构类别。");
        } catch (Exception e) {
            e.printStackTrace();
            envelop.setMessage("删除卫生机构类别发生异常：" + e.getMessage());
        }
        return envelop;
    }

    @ApiOperation("验证卫生机构类别编码是否唯一")
    @RequestMapping(value = ServiceApi.Org.HealthCategory.IsUniqueCode, method = RequestMethod.GET)
    public Envelop isUniqueCode(
            @ApiParam(name = "id", value = "卫生机构类别ID", required = true)
            @RequestParam(value = "id") Integer id,
            @ApiParam(name = "code", value = "卫生机构类别编码", required = true)
            @RequestParam(value = "code") String code) {
        Envelop envelop = new Envelop();
        envelop.setStatus(-1);
        try {
            Integer result = orgHealthCategoryService.isUniqueCode(id, code);
            envelop.setStatus(result);
            if (result==-1) {
                envelop.setMessage("该卫生机构类别编码已被使用，请重新填写！");
            }
        } catch (Exception e) {
            e.printStackTrace();
            envelop.setMessage("发生异常：" + e.getMessage());
        }
        return envelop;
    }

    @ApiOperation("验证卫生机构类别名称是否唯一")
    @RequestMapping(value = ServiceApi.Org.HealthCategory.IsUniqueName, method = RequestMethod.GET)
    public Envelop isUniqueName(
            @ApiParam(name = "id", value = "卫生机构类别ID", required = true)
            @RequestParam(value = "id") Integer id,
            @ApiParam(name = "code", value = "卫生机构类别名称", required = true)
            @RequestParam(value = "name") String name) {
        Envelop envelop = new Envelop();
        envelop.setStatus(-1);
        try {
            Integer result = orgHealthCategoryService.isUniqueName(id, name);
            envelop.setStatus(result);
            if (result==-1) {
                envelop.setMessage("该卫生机构类别名称已被使用，请重新填写！");
            }
        } catch (Exception e) {
            e.printStackTrace();
            envelop.setMessage("发生异常：" + e.getMessage());
        }
        return envelop;
    }

}
