package com.yihu.jw.basic.quota.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.basic.quota.service.TjDimensionSlaveService;
import com.yihu.jw.basic.quota.service.TjQuotaDimensionSlaveService;
import com.yihu.jw.basic.quota.service.TjQuotaLogService;
import com.yihu.jw.entity.ehr.quota.TjDimensionSlave;
import com.yihu.jw.entity.ehr.quota.TjQuotaDimensionSlave;
import com.yihu.jw.entity.ehr.quota.TjQuotaLog;
import com.yihu.jw.restmodel.ehr.tj.MTjQuotaLog;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.constants.ApiVersion;
import com.yihu.jw.constants.ServiceApi;
import com.yihu.jw.util.date.DateTimeUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.*;

/**
 * Created by janseny on 2017/5/8.
 */
@RestController
@RequestMapping(ApiVersion.Version1_0)
@Api(value = "TjQuotaLogEndPoint", description = "统计日志", tags = {"统计日志查询"})
public class TjQuotaLogEndPoint extends EnvelopRestEndpoint {

    @Autowired
    ObjectMapper objectMapper;
    @Autowired
    TjDimensionSlaveService tjDimensionSlaveService;
    @Autowired
    TjQuotaLogService tjQuotaLogService;
    @Autowired
    TjQuotaDimensionSlaveService tjQuotaDimensionSlaveService;
    @Autowired
    JdbcTemplate jdbcTemplate;
    @PersistenceContext
    protected EntityManager entityManager;

    @RequestMapping(value = ServiceApi.TJ.GetTjQuotaLogList, method = RequestMethod.GET)
    @ApiOperation(value = "根据查询条件查询从纬度")
    public List<MTjQuotaLog> getTjQuotaLogList(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段", defaultValue = "")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "quotaCode", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "quotaCode", required = false) String quotaCode,
            @ApiParam(name = "startTime", value = "过滤器，为空检索所有条件", defaultValue = "")
            @RequestParam(value = "startTime", required = false) String startTime,
            @ApiParam(name = "endTime", value = "过滤器，为空检索所有条件", defaultValue = "")
            @RequestParam(value = "endTime", required = false) String endTime,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档", defaultValue = "")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size", required = false) int size,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception {

        Map<String, Object> conditionMap = new HashMap<>();
        conditionMap.put("quotaCode", quotaCode);
        conditionMap.put("page", page);
        conditionMap.put("pageSize", size);

        Date startDate = DateTimeUtil.simpleDateTimeParse(startTime);
        Date endDate = DateTimeUtil.simpleDateTimeParse(endTime);
        if(null!=endDate){
            Calendar calendar   =   new GregorianCalendar();
            calendar.setTime(endDate);
            calendar.add(calendar.DATE,1);//把日期往后增加一天.整数往后推,负数往前移动
            endDate=calendar.getTime();   //日期往后推一天
        }
        conditionMap.put("startDate", startDate);
        conditionMap.put("endDate", endDate);
/*
        List<TjQuotaLog> tjQuotaLogs = tjQuotaLogService.searchQuotaLogByParams(conditionMap);
*/
        Long totalCount =Long.parseLong(tjQuotaLogService.searchQuotaLogByParamsTotalCount(conditionMap).toString());
        pagedResponse(request, response, totalCount, page, size);
        return (List<MTjQuotaLog>)convertToModels(null,new ArrayList<MTjQuotaLog>(0), MTjQuotaLog.class, null);
    }


    @RequestMapping(value = ServiceApi.TJ.GetTjQuotaLogRecentRecord, method = RequestMethod.GET)
    @ApiOperation(value = "获取最近日志列表")
    MTjQuotaLog getRecentRecord(
            @ApiParam(name = "quotaCode", value = "指标code", defaultValue = "")
            @RequestParam(value = "quotaCode", required = false) String quotaCode,
            @ApiParam(name = "endTime", value = "完成时间", defaultValue = "")
            @RequestParam(value = "endTime", required = false) String endTime
    ){
        MTjQuotaLog mTjQuotaLog = null;
        List<TjQuotaLog> tjQuotaLogs =  tjQuotaLogService.getRecentRecord(quotaCode,endTime);
        if(tjQuotaLogs != null && tjQuotaLogs.size() > 0){
            mTjQuotaLog = objectMapper.convertValue(tjQuotaLogs.get(0),MTjQuotaLog.class);
        }
        return mTjQuotaLog;
    }

    @RequestMapping(name = "/tj/initialResult", method = RequestMethod.GET)
    @ApiOperation(value = "指标结果页")
    public ListEnvelop initialResult(@ApiParam(name = "quotaCode", value = "指标code", required = true)
                                         @RequestParam(value = "quotaCode") String quotaCode){
        try {
            List<TjQuotaDimensionSlave> getDimensionSlaveByQuotaCodeList =  tjQuotaDimensionSlaveService.getTjQuotaDimensionSlaveByCode(quotaCode);
            int num = 1;
            JSONArray jsonArray = new JSONArray();
            for (TjQuotaDimensionSlave quotaDimensionSlaveModel : getDimensionSlaveByQuotaCodeList) {
                JSONObject jsonObject = (JSONObject)JSONObject.toJSON(quotaDimensionSlaveModel);
                String filter = "code=" + quotaDimensionSlaveModel.getSlaveCode();
                List<TjDimensionSlave> tjDimensionSlaves = tjDimensionSlaveService.search(filter);
                TjDimensionSlave tjDimensionSlave = null;
                if(tjDimensionSlaves != null && tjDimensionSlaves.size() >0){
                    tjDimensionSlave =  tjDimensionSlaves.get(0);
                    jsonObject.put("slaveKey" + num + "Name",tjDimensionSlave.getName());
                    num++;
                }
                jsonArray.add(jsonObject);
            }
            return ListEnvelop.getSuccess("查询成功",jsonArray);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ListEnvelop.getError("查询失败");
    }
}
