package com.yihu.jw.basic.redis.controler;

import com.yihu.jw.constants.ApiVersion;
import com.yihu.jw.constants.ServiceApi;
import com.yihu.jw.basic.redis.dao.RedisMqChannelDao;
import com.yihu.jw.basic.redis.pubsub.CustomMessageListenerAdapter;
import com.yihu.jw.basic.redis.pubsub.MessageCommonBiz;
import com.yihu.jw.basic.redis.service.RedisMqChannelService;
import com.yihu.jw.basic.redis.service.RedisMqMessageLogService;
import com.yihu.jw.basic.redis.service.RedisMqPublisherService;
import com.yihu.jw.basic.redis.service.RedisMqSubscriberService;
import com.yihu.jw.entity.ehr.redis.RedisMqChannel;
import com.yihu.jw.entity.ehr.redis.RedisMqMessageLog;
import com.yihu.jw.entity.ehr.redis.RedisMqPublisher;
import com.yihu.jw.entity.ehr.redis.RedisMqSubscriber;
import com.yihu.jw.restmodel.ehr.redis.MRedisCacheCategory;
import com.yihu.jw.restmodel.ehr.redis.MRedisMqChannel;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.listener.ChannelTopic;
import org.springframework.data.redis.listener.RedisMessageListenerContainer;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * Redis消息队列 接口
 *
 * @author 张进军
 * @date 2017/11/10 11:45
 */
@RestController
@RequestMapping(value = ApiVersion.Version1_0)
@Api(description = "消息队列接口", tags = {"Redis消息发布订阅--消息队列接口"})
public class RedisMqChannelEndPoint extends EnvelopRestEndpoint {

    @Autowired
    private RedisMqChannelService redisMqChannelService;
    @Autowired
    RedisMqChannelDao redisMqChannelDao;
    @Autowired
    private RedisMqMessageLogService redisMqMessageLogService;
    @Autowired
    private RedisMqSubscriberService redisMqSubscriberService;
    @Autowired
    private RedisMqPublisherService redisMqPublisherService;
    @Resource
    private RedisTemplate<String, Object> redisTemplate;
    @Autowired
    private RedisMessageListenerContainer redisMessageListenerContainer;

    @ApiOperation("根据ID获取消息队列")
    @RequestMapping(value = ServiceApi.Redis.MqChannel.GetById, method = RequestMethod.GET)
    public Envelop getById(
            @ApiParam(name = "id", value = "主键", required = true)
            @PathVariable(value = "id") Integer id) {
        try {
            MRedisMqChannel mRedisMqChannel = convertToModel(redisMqChannelService.getById(id), MRedisMqChannel.class);
            return ObjEnvelop.getSuccess("成功获取消息队列",mRedisMqChannel);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return failed("获取消息队列发生异常");
    }

    @ApiOperation(value = "根据条件获取消息队列")
    @RequestMapping(value = ServiceApi.Redis.MqChannel.Search, method = RequestMethod.GET)
    public Envelop search(
            @ApiParam(name = "fields", value = "返回的字段，为空则返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "筛选条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size", required = false) int size) {
        try {
            List<RedisMqChannel> redisMqChannelList = redisMqChannelService.search(fields, filters, sorts, page, size);
            long count = redisMqChannelService.getCount(filters);
            List<MRedisMqChannel> mRedisMqChannelList = (List<MRedisMqChannel>) convertToModels(redisMqChannelList, new ArrayList<MRedisMqChannel>(), MRedisMqChannel.class, fields);
            return PageEnvelop.getSuccessListWithPage("成功获取消息队列列表",mRedisMqChannelList,page,size,count);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return failed("获取消息队列发生异常");
    }

    @ApiOperation("新增消息队列")
    @RequestMapping(value = ServiceApi.Redis.MqChannel.Save, method = RequestMethod.POST)
    public Envelop add(
            @ApiParam(value = "消息队列JSON", required = true)
            @RequestBody String entityJson) {
        try {
            RedisMqChannel newEntity = objectMapper.readValue(entityJson, RedisMqChannel.class);
            newEntity.setDequeuedNum(0);
            newEntity.setEnqueuedNum(0);
            newEntity.setPublisherNum(0);
            newEntity.setSubscriberNum(0);
            newEntity = redisMqChannelService.save(newEntity);

            // 开启该订阅者的消息队列的消息监听
            String channel = newEntity.getChannel();
            CustomMessageListenerAdapter messageListener = MessageCommonBiz.newCustomMessageListenerAdapter(channel);
            redisMessageListenerContainer.addMessageListener(messageListener, new ChannelTopic(channel));

            MRedisCacheCategory mRedisCacheCategory = convertToModel(newEntity, MRedisCacheCategory.class);
            return ObjEnvelop.getSuccess("成功新增消息队列",mRedisCacheCategory);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return failed("新增消息队列发生异常");
    }

    @ApiOperation("更新消息队列")
    @RequestMapping(value = ServiceApi.Redis.MqChannel.Save, method = RequestMethod.PUT)
    public Envelop update(
            @ApiParam(value = "消息队列JSON", required = true)
            @RequestBody String entityJson) {
        try {
            RedisMqChannel updateEntity = objectMapper.readValue(entityJson, RedisMqChannel.class);
            updateEntity = redisMqChannelService.save(updateEntity);

            MRedisMqChannel mRedisMqChannel = convertToModel(updateEntity, MRedisMqChannel.class);
            return ObjEnvelop.getSuccess("成功更新消息队列",mRedisMqChannel);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return failed("更新消息队列发生异常");
    }

    @ApiOperation("删除消息队列")
    @RequestMapping(value = ServiceApi.Redis.MqChannel.Delete, method = RequestMethod.DELETE)
    public Envelop delete(
            @ApiParam(name = "id", value = "消息队列ID", required = true)
            @RequestParam(value = "id") Integer id) {
        RedisMqChannel redisMqChannel = redisMqChannelService.getById(id);
        String channel = redisMqChannel.getChannel();

        List<RedisMqMessageLog> messageLogList = redisMqMessageLogService.findByChannel(channel);
        if (messageLogList.size() != 0) {
            return failed("该消息队列存在订阅失败的消息，不能删除。");
        }
        List<RedisMqSubscriber> subscriberList = redisMqSubscriberService.findByChannel(channel);
        if (subscriberList.size() != 0) {
            return failed("该消息队列存在订阅者，不能删除。");
        }
        List<RedisMqPublisher> publisherList = redisMqPublisherService.findByChannel(channel);
        if (publisherList.size() != 0) {
            return failed("该消息队列存在发布者，不能删除。");
        }

        redisMqChannelDao.deleteById(id);

        // 删除该消息队列的消息监听
        CustomMessageListenerAdapter messageListener = MessageCommonBiz.newCustomMessageListenerAdapter(channel);
        redisMessageListenerContainer.removeMessageListener(messageListener, new ChannelTopic(channel));

        return Envelop.getSuccess("删除成功");
    }

    @ApiOperation("验证消息队列编码是否唯一")
    @RequestMapping(value = ServiceApi.Redis.MqChannel.IsUniqueChannel, method = RequestMethod.GET)
    public Envelop isUniqueChannel(
            @ApiParam(name = "id", value = "消息队列ID", required = true)
            @RequestParam(value = "id") Integer id,
            @ApiParam(name = "channel", value = "消息队列编码", required = true)
            @RequestParam(value = "channel") String channel) {
        try {
            boolean result = redisMqChannelService.isUniqueChannel(id, channel);
            if (!result) {
                return failed("该消息队列编码已被使用，请重新填写！");
            }
            return Envelop.getSuccess("编码是唯一");
        } catch (Exception e) {
            e.printStackTrace();
        }
        return failed("发生异常");
    }

    @ApiOperation("验证消息队列名称是否唯一")
    @RequestMapping(value = ServiceApi.Redis.MqChannel.IsUniqueChannelName, method = RequestMethod.GET)
    public Envelop isUniqueChannelName(
            @ApiParam(name = "id", value = "消息队列ID", required = true)
            @RequestParam(value = "id") Integer id,
            @ApiParam(name = "channelName", value = "消息队列名称", required = true)
            @RequestParam(value = "channelName") String channelName) {
        try {
            boolean result = redisMqChannelService.isUniqueChannelName(id, channelName);
            if (!result) {
                return failed("该消息队列名称已被使用，请重新填写！");
            }
            return Envelop.getSuccess("队列名称是唯一");
        } catch (Exception e) {
            e.printStackTrace();
        }
        return failed("发生异常");
    }

    @ApiOperation("验证消息队列是否已存在")
    @RequestMapping(value = ServiceApi.Redis.MqChannel.IsExist, method = RequestMethod.GET)
    public Envelop isExist(
            @ApiParam(name = "channel", value = "消息队列", required = true)
            @RequestParam(value = "channel") String channel) {
        return ObjEnvelop.getSuccess("验证成功",redisMqChannelService.isExist(channel));
    }

}
