package com.yihu.jw.basic.redis.controler;

import com.yihu.jw.constants.ApiVersion;
import com.yihu.jw.constants.ServiceApi;
import com.yihu.jw.basic.redis.dao.RedisMqPublisherDao;
import com.yihu.jw.basic.redis.service.RedisMqChannelService;
import com.yihu.jw.basic.redis.service.RedisMqPublisherService;
import com.yihu.jw.entity.ehr.redis.RedisMqChannel;
import com.yihu.jw.entity.ehr.redis.RedisMqPublisher;
import com.yihu.jw.restmodel.ehr.redis.MRedisMqPublisher;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * Redis消息发布者 接口
 *
 * @author 张进军
 * @date 2017/11/20 09:35
 */
@RestController
@RequestMapping(value = ApiVersion.Version1_0)
@Api(description = "消息发布者接口", tags = {"Redis消息发布订阅--消息发布者接口"})
public class RedisMqPublisherEndPoint extends EnvelopRestEndpoint {

    @Autowired
    private RedisMqPublisherService redisMqPublisherService;
    @Autowired
    RedisMqPublisherDao redisMqPublisherDao;
    @Autowired
    private RedisMqChannelService redisMqChannelService;

    @ApiOperation("根据ID获取消息发布者")
    @RequestMapping(value = ServiceApi.Redis.MqPublisher.GetById, method = RequestMethod.GET)
    public Envelop getById(
            @ApiParam(name = "id", value = "主键", required = true)
            @PathVariable(value = "id") Integer id) {
        try {
            MRedisMqPublisher mRedisMqPublisher = convertToModel(redisMqPublisherService.getById(id), MRedisMqPublisher.class);
            return ObjEnvelop.getSuccess("成功获取消息发布者",mRedisMqPublisher);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return failed("获取消息发布者发生异常");
    }

    @ApiOperation(value = "根据条件获取消息发布者")
    @RequestMapping(value = ServiceApi.Redis.MqPublisher.Search, method = RequestMethod.GET)
    public Envelop search(
            @ApiParam(name = "fields", value = "返回的字段，为空则返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "筛选条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size", required = false) int size) {
        try {
            List<RedisMqPublisher> list = redisMqPublisherService.search(fields, filters, sorts, page, size);
            long count = redisMqPublisherService.getCount(filters);
            List<MRedisMqPublisher> mList = (List<MRedisMqPublisher>) convertToModels(list, new ArrayList<MRedisMqPublisher>(), MRedisMqPublisher.class, fields);
            return PageEnvelop.getSuccessListWithPage("成功获取消息发布者列表",mList,page,size,count);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return failed("获取消息发布者发生异常");
    }

    @ApiOperation("新增消息发布者")
    @RequestMapping(value = ServiceApi.Redis.MqPublisher.Save, method = RequestMethod.POST)
    public Envelop add(
            @ApiParam(value = "消息发布者JSON", required = true)
            @RequestBody String entityJson) {
        try {
            RedisMqPublisher newEntity = objectMapper.readValue(entityJson, RedisMqPublisher.class);
            newEntity = redisMqPublisherService.save(newEntity);

            // 累计发布者数
            RedisMqChannel channel = redisMqChannelService.findByChannel(newEntity.getChannel());
            channel.setPublisherNum(channel.getPublisherNum() + 1);
            redisMqChannelService.save(channel);

            MRedisMqPublisher mRedisMqPublisher = convertToModel(newEntity, MRedisMqPublisher.class);
            return ObjEnvelop.getSuccess("成功新增消息发布者",mRedisMqPublisher);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return failed("新增消息发布者发生异常");
    }

    @ApiOperation("更新消息发布者")
    @RequestMapping(value = ServiceApi.Redis.MqPublisher.Save, method = RequestMethod.PUT)
    public Envelop update(
            @ApiParam(value = "消息发布者JSON", required = true)
            @RequestBody String entityJson) {
        try {
            RedisMqPublisher updateEntity = objectMapper.readValue(entityJson, RedisMqPublisher.class);
            updateEntity = redisMqPublisherService.save(updateEntity);

            MRedisMqPublisher mRedisMqPublisher = convertToModel(updateEntity, MRedisMqPublisher.class);
            return ObjEnvelop.getSuccess("成功更新消息发布者",mRedisMqPublisher);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return failed("更新消息发布者发生异常");
    }

    @ApiOperation("删除消息发布者")
    @RequestMapping(value = ServiceApi.Redis.MqPublisher.Delete, method = RequestMethod.DELETE)
    public Envelop delete(
            @ApiParam(name = "id", value = "消息发布者ID", required = true)
            @RequestParam(value = "id") Integer id) {
        try {
            RedisMqPublisher publisher = redisMqPublisherService.getById(id);
            redisMqPublisherDao.deleteById(id);

            // 扣减发布者数
            RedisMqChannel channel = redisMqChannelService.findByChannel(publisher.getChannel());
            channel.setPublisherNum(channel.getPublisherNum() - 1);
            redisMqChannelService.save(channel);

            return Envelop.getSuccess("成功删除消息发布者");
        } catch (Exception e) {
            e.printStackTrace();
        }
        return failed("删除消息发布者发生异常");
    }

    @ApiOperation("验证指定队列中发布者应用ID是否唯一")
    @RequestMapping(value = ServiceApi.Redis.MqPublisher.IsUniqueAppId, method = RequestMethod.GET)
    public Envelop isUniqueAppId(
            @ApiParam(name = "id", value = "消息订阅者ID", required = true)
            @RequestParam(value = "id") Integer id,
            @ApiParam(name = "channel", value = "消息队列编码", required = true)
            @RequestParam(value = "channel") String channel,
            @ApiParam(name = "appId", value = "发布者应用ID", required = true)
            @RequestParam(value = "appId") String appId) {
        try {
            boolean result = redisMqPublisherService.isUniqueAppId(id, channel, appId);
            if (!result) {
                return failed("该指定消息队列中订阅者应用ID已被使用，请重新填写！");
            }
            return Envelop.getSuccess("应用ID是唯一");
        } catch (Exception e) {
            e.printStackTrace();
        }
        return failed("发生异常");
    }

}
