package com.yihu.jw.basic.resource.controller;

import com.yihu.jw.constants.ServiceApi;
import com.yihu.jw.constants.ApiVersion;
import com.yihu.jw.basic.resource.dao.RsAdapterDictionaryDao;
import com.yihu.jw.entity.ehr.resource.RsAdapterDictionary;
import com.yihu.jw.basic.resource.service.RsAdapterDictionaryService;
import com.yihu.jw.restmodel.ehr.resource.MRsAdapterDictionary;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.entity.ehr.id.BizObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * @author linaz
 * @created 2016.05.17 16:33
 */
@RestController
@RequestMapping(value = ApiVersion.Version1_0 )
@Api(value = "RsAdapterDictionaryEndPoint", description = "适配字典", tags = {"资源服务-适配字典"})
public class RsAdapterDictionaryEndPoint extends EnvelopRestEndpoint {

    @Autowired
    private RsAdapterDictionaryService adapterDictionaryService;
    @Autowired
    private RsAdapterDictionaryDao adapterDictionaryDao;

    @RequestMapping(value = ServiceApi.Resources.AdapterDicts, method = RequestMethod.GET)
    @ApiOperation(value = "根据查询条件获取适配字典列表")
    public Envelop searchRsAdapterDictionaries(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段", defaultValue = "id,name,secret,url,createTime")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件", defaultValue = "")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档", defaultValue = "+name,+createTime")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size", required = false) int size) throws Exception {
        List<RsAdapterDictionary> adapterDictionaries = adapterDictionaryService.search(fields, filters, sorts, page, size);
        List<MRsAdapterDictionary> list = convertToModels(adapterDictionaries, new ArrayList<>(adapterDictionaries.size()), MRsAdapterDictionary.class, fields);
        return PageEnvelop.getSuccessListWithPage("查询成功",list,page,size,adapterDictionaryService.getCount(filters));
    }

    /**
     * 貌似重复的方法
     */
    @RequestMapping(value = ServiceApi.Adaptions.RsAdapterDictionaries, method = RequestMethod.GET)
    @ApiOperation("查询适配字典")
    public Envelop getDictionaries(
            @ApiParam(name="fields",value="返回字段",defaultValue = "")
            @RequestParam(value="fields",required = false)String fields,
            @ApiParam(name="filters",value="过滤",defaultValue = "")
            @RequestParam(value="filters",required = false)String filters,
            @ApiParam(name="sorts",value="排序",defaultValue = "")
            @RequestParam(value="sorts",required = false)String sorts,
            @ApiParam(name="page",value="页码",defaultValue = "1")
            @RequestParam(value="page",required = false)int page,
            @ApiParam(name="size",value="分页大小",defaultValue = "15")
            @RequestParam(value="size",required = false)int size) throws Exception {
        List<RsAdapterDictionary> adapterDictionaries = adapterDictionaryService.search(fields, filters, sorts, page, size);
        List<MRsAdapterDictionary> list = convertToModels(adapterDictionaries, new ArrayList<>(adapterDictionaries.size()), MRsAdapterDictionary.class, fields);
        return PageEnvelop.getSuccessListWithPage("查询成功",list,page,size,adapterDictionaryService.getCount(filters));
    }


    @RequestMapping(value = ServiceApi.Resources.AdapterDicts, method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "创建适配字典")
    public Envelop createRsAdapterDictionary(
            @ApiParam(name = "json_data", value = "", defaultValue = "")
            @RequestBody String jsonData) throws Exception {
        RsAdapterDictionary adapterDictionary = toEntity(jsonData, RsAdapterDictionary.class);
        adapterDictionary.setId(getObjectId(BizObject.RsAdapterDictionary));
        adapterDictionaryService.save(adapterDictionary);
        MRsAdapterDictionary mRsAdapterDictionary = convertToModel(adapterDictionary, MRsAdapterDictionary.class, null);
        return ObjEnvelop.getSuccess("创建成功",mRsAdapterDictionary);

    }

    /**
     * 貌似重复的方法
     * @param jsonData
     * @return
     * @throws Exception
     */
    @RequestMapping(value = ServiceApi.Adaptions.RsAdapterDictionaries, method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation("创建适配字典")
    public Envelop createDictionaries(
            @ApiParam(name = "json_data", value = "适配字典JSON", defaultValue = "")
            @RequestBody String jsonData) throws Exception {
        RsAdapterDictionary adapterDictionary = toEntity(jsonData, RsAdapterDictionary.class);
        adapterDictionary.setId(getObjectId(BizObject.RsAdapterDictionary));
        adapterDictionary = adapterDictionaryService.save(adapterDictionary);
        MRsAdapterDictionary mRsAdapterDictionary = convertToModel(adapterDictionary, MRsAdapterDictionary.class, null);
        return ObjEnvelop.getSuccess("创建成功",mRsAdapterDictionary);
    }

    @RequestMapping(value = ServiceApi.Adaptions.RsAdapterDictionaries, method = RequestMethod.PUT, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation("更新适配字典")
    public Envelop updateDictionary(
            @ApiParam(name = "json_data", value = "适配字典JSON", defaultValue = "")
            @RequestBody String jsonData) throws Exception {
        RsAdapterDictionary adapterDictionary = toEntity(jsonData, RsAdapterDictionary.class);
        adapterDictionary = adapterDictionaryService.save(adapterDictionary);
        MRsAdapterDictionary mRsAdapterDictionary = convertToModel(adapterDictionary, MRsAdapterDictionary.class, null);
        return ObjEnvelop.getSuccess("更新成功",mRsAdapterDictionary);
    }

    /**
     * 貌似重复的方法
     * @param jsonData
     * @return
     * @throws Exception
     */
    @RequestMapping(value = ServiceApi.Resources.AdapterDicts, method = RequestMethod.PUT, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "修改适配字典")
    public Envelop updateRsAdapterDictionary(
            @ApiParam(name = "json_data", value = "")
            @RequestBody String jsonData) throws Exception {
        RsAdapterDictionary adapterDictionary = toEntity(jsonData, RsAdapterDictionary.class);
        adapterDictionaryService.save(adapterDictionary);
        MRsAdapterDictionary mRsAdapterDictionary = convertToModel(adapterDictionary, MRsAdapterDictionary.class, null);
        return ObjEnvelop.getSuccess("更新成功",mRsAdapterDictionary);
    }


    @RequestMapping(value = ServiceApi.Resources.AdapterDict, method = RequestMethod.DELETE)
    @ApiOperation(value = "删除适配字典")
    public Envelop deleteRsAdapterDictionary(
            @ApiParam(name = "id", value = "id", defaultValue = "")
            @PathVariable(value = "id") String id) throws Exception {
        adapterDictionaryDao.deleteById(id);
        return success("删除成功");
    }

    /**
     * 貌似重复的方法
     */
    @RequestMapping(value = ServiceApi.Adaptions.RsAdapterDictionary, method = RequestMethod.DELETE)
    @ApiOperation("删除适配字典")
    public Envelop deleteDictionary(
            @ApiParam(name = "id", value = "适配字典ID", defaultValue = "")
            @PathVariable(value = "id") String id) throws Exception {
        adapterDictionaryDao.deleteById(id);
        return success("删除成功");
    }

    @RequestMapping(value = ServiceApi.Resources.AdapterDict, method = RequestMethod.GET)
    @ApiOperation(value = "根据id获取获取适配字典")
    public Envelop getRsAdapterDictionaryById(
            @ApiParam(name = "id", value = "", defaultValue = "")
            @RequestParam(value = "id") String id) {
        RsAdapterDictionary adapterDictionary = adapterDictionaryService.findById(id);
        MRsAdapterDictionary mRsAdapterDictionary = convertToModel(adapterDictionary, MRsAdapterDictionary.class, null);
        return ObjEnvelop.getSuccess("获取成功",mRsAdapterDictionary);
    }

    /**
     * 貌似重复的方法
     */
    @RequestMapping(value = ServiceApi.Adaptions.RsAdapterDictionary, method = RequestMethod.GET)
    @ApiOperation("根据ID获取适配字典")
    public Envelop getDictionaryById(
            @ApiParam(name = "id", value = "id", defaultValue = "")
            @PathVariable(value = "id") String id) throws Exception {
        RsAdapterDictionary adapterDictionary = adapterDictionaryService.findById(id);
        MRsAdapterDictionary mRsAdapterDictionary = convertToModel(adapterDictionary, MRsAdapterDictionary.class, null);
        return ObjEnvelop.getSuccess("获取成功",mRsAdapterDictionary);
    }

    @RequestMapping(value = ServiceApi.Resources.AdapterDictsBatch, method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "批量创建适配字典")
    public Envelop createRsAdapterDictionaries(
            @ApiParam(name = "json_data", value = "", defaultValue = "")
            @RequestBody String jsonData) throws Exception {
        List<RsAdapterDictionary> dictionaries = toEntity(jsonData,List.class);
        adapterDictionaryService.batchInsert(dictionaries);
        return success("创建成功");
    }


}