package com.yihu.jw.basic.resource.controller;

import com.yihu.jw.basic.resource.dao.RsDictionaryEntryDao;
import com.yihu.jw.basic.resource.service.RsDictionaryEntryService;
import com.yihu.jw.constants.ServiceApi;
import com.yihu.jw.entity.ehr.resource.RsDictionaryEntry;
import com.yihu.jw.restmodel.ehr.resource.MRsDictionaryEntry;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;

/**
 * @author linaz
 * @created 2016.05.17 16:33
 */
@RestController
@RequestMapping(value = "")
@Api(value = "RsDictionaryEntry", description = "标准字典项", tags = {"资源服务-标准字典项"})
public class RsDictionaryEntryEndPoint extends EnvelopRestEndpoint {
    @Autowired
    private RsDictionaryEntryService rsDictionaryEntryService;
    @Autowired
    private RsDictionaryEntryDao dictionaryEntryDao;

    @RequestMapping(value = ServiceApi.Resources.DictEntries, method = RequestMethod.GET)
    @ApiOperation(value = "根据查询条件获取标准字典项列表", notes = "根据查询条件获取标准字典项列表")
    public PageEnvelop searchRsDictionaryEntries(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段", defaultValue = "id,name,secret,url,createTime")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件", defaultValue = "")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档", defaultValue = "+name,+createTime")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size", required = false) int size,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception {
        List<RsDictionaryEntry> dictionaryEntries = rsDictionaryEntryService.search(fields, filters, sorts, page, size);
        PageEnvelop pageEnvelop = new PageEnvelop();
        pageEnvelop.setTotalCount(Integer.parseInt(rsDictionaryEntryService.getCount(filters)+""));
        pageEnvelop.setCurrPage(page);
        pageEnvelop.setPageSize(size);
        pageEnvelop.setDetailModelList((List<MRsDictionaryEntry>) convertToModels(dictionaryEntries, new ArrayList<MRsDictionaryEntry>(dictionaryEntries.size()), MRsDictionaryEntry.class, fields));
        return pageEnvelop;
    }



    @RequestMapping(value = ServiceApi.Resources.DictEntries, method = RequestMethod.POST)
    @ApiOperation(value = "创建标准字典项", notes = "创建标准字典项")
    public Envelop createRsDictionaryEntry(
            @ApiParam(name = "jsonData", value = "", defaultValue = "")
            @RequestParam(value = "jsonData", required = true)String jsonData) throws Exception {

        RsDictionaryEntry dictionaryEntry = toEntity(jsonData, RsDictionaryEntry.class);
        String code = dictionaryEntry.getCode();
        String dictCode = dictionaryEntry.getDictCode();
        if(isExistence(dictCode,code)){
            throw new Exception("字典项代码不能重复");
        }
        rsDictionaryEntryService.insert(dictionaryEntry);
        return success(convertToModel(dictionaryEntry, MRsDictionaryEntry.class, null));

    }

    @RequestMapping(value =ServiceApi.Resources.DictEntries, method = RequestMethod.PUT)
    @ApiOperation(value = "修改标准字典项", notes = "修改标准字典项")
    public Envelop updateRsDictionaryEntry(
            @ApiParam(name = "jsonData", value = "")
            @RequestParam(value = "jsonData", required = true)String jsonData) throws Exception {

        RsDictionaryEntry dictionaryEntry = toEntity(jsonData, RsDictionaryEntry.class);
        RsDictionaryEntry d = rsDictionaryEntryService.findById(dictionaryEntry.getId());
        String code = dictionaryEntry.getCode();
        String dictCode = dictionaryEntry.getDictCode();
        if(!d.getCode().equals(code) && isExistence(dictCode,code)){
            throw new Exception("字典代码不可修改");
        }
        rsDictionaryEntryService.save(dictionaryEntry);
        return success(convertToModel(dictionaryEntry, MRsDictionaryEntry.class, null));
    }

    @RequestMapping(value = ServiceApi.Resources.DictEntry, method = RequestMethod.DELETE)
    @ApiOperation(value = "删除标准字典项", notes = "删除标准字典项")
    public Envelop deleteRsDictionaryEntry(
            @ApiParam(name = "id", value = "id", defaultValue = "")
            @PathVariable(value = "id") int id) throws Exception {
        dictionaryEntryDao.deleteById(id);
        return success(true);
    }

    @RequestMapping(value = ServiceApi.Resources.DictEntry, method = RequestMethod.GET)
    @ApiOperation(value = "根据id获取获取标准字典")
    public Envelop getRsDictionaryEntryById(
            @ApiParam(name = "id", value = "", defaultValue = "")
            @PathVariable(value = "id") int id) {
        RsDictionaryEntry dictionaryEntry = rsDictionaryEntryService.findById(id);
        return success(convertToModel(dictionaryEntry, MRsDictionaryEntry.class));
    }

    @RequestMapping(value = ServiceApi.Resources.DictEntriesByDictCode, method = RequestMethod.GET)
    @ApiOperation(value = "根据dict_code获取获取标准字典")
    public ListEnvelop  getRsDictionaryEntryByDictCode(
            @ApiParam(name = "dict_code", value = "", defaultValue = "")
            @RequestParam(value = "dict_code") String dict_code)
    {
        List<RsDictionaryEntry> dictionaryEntries = rsDictionaryEntryService.findByDictCode(dict_code);
        return success((List<MRsDictionaryEntry>) convertToModels(dictionaryEntries, new ArrayList<>(dictionaryEntries.size()), MRsDictionaryEntry.class,null));
    }

    @RequestMapping(value = ServiceApi.Resources.DictEntriesExistence,method = RequestMethod.GET)
    @ApiOperation("根据过滤条件判断是否存在")
    public Envelop isExistence(
            @ApiParam(name="filters",value="filters",defaultValue = "")
            @RequestParam(value="filters") String filters) throws Exception {

        List ls = rsDictionaryEntryService.search("",filters,"", 1, 1);
        return success(ls!=null && ls.size()>0);
    }

    public boolean isExistence(String dictCode,String code) {
        return rsDictionaryEntryService.findByFields(new String[]{"dictCode","code"},new String[]{dictCode,code}).size() != 0;
    }


    @RequestMapping(value = ServiceApi.Resources.NoPageDictEntries, method = RequestMethod.GET)
    @ApiOperation(value = "根据查询条件获取标准字典项列表_不分页", notes = "根据查询条件获取标准字典项列表_不分页")
    public ListEnvelop searchNoPageRsDictEntries(
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件", defaultValue = "")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "dictId", value = "字典code")
            @RequestParam(value = "dictId", required = false) String dictId) throws Exception {
        if(StringUtils.isNotBlank(dictId)){
            filters = "dictCode="+dictId;
        }
        List<RsDictionaryEntry> dictionaryEntries = rsDictionaryEntryService.search(filters);
        return success((List<MRsDictionaryEntry>) convertToModels(dictionaryEntries, new ArrayList<>(dictionaryEntries.size()), MRsDictionaryEntry.class, ""));
    }
}