package com.yihu.jw.basic.resource.controller;

import com.yihu.jw.constants.ApiVersion;
import com.yihu.jw.constants.ServiceApi;
import com.yihu.jw.basic.resource.dao.RsReportCategoryDao;
import com.yihu.jw.basic.resource.model.RsCategoryTypeTreeModel;
import com.yihu.jw.basic.resource.service.RsResourceCategoryService;
import com.yihu.jw.entity.ehr.resource.RsResourceCategory;
import com.yihu.jw.restmodel.ehr.resource.MRsReportCategory;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.entity.ehr.resource.RsReportCategory;
import com.yihu.jw.basic.resource.service.RsReportCategoryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

/**
 * 资源报表分类 服务接口
 *
 * @author 张进军
 * @created 2017.8.8 20:32
 */
@RestController
@RequestMapping(value = ApiVersion.Version1_0)
@Api(value = "RsReportCategoryEndPoint", description = "资源报表分类", tags = {"资源服务-资源报表分类"})
public class RsReportCategoryEndPoint extends EnvelopRestEndpoint {

    @Autowired
    private RsReportCategoryService rsReportCategoryService;
    @Autowired
    private RsReportCategoryDao rsReportCategoryDao;
    @Autowired
    private RsResourceCategoryService rsCategoryService;

    @ApiOperation("根据ID获取资源报表分类")
    @RequestMapping(value = ServiceApi.Resources.RsReportCategory, method = RequestMethod.GET)
    public MRsReportCategory getById(
            @ApiParam(name = "id", value = "id", required = true)
            @PathVariable(value = "id") Integer id) throws Exception {
        return convertToModel(rsReportCategoryService.getById(id), MRsReportCategory.class);
    }

    @ApiOperation(value = "根据条件获取资源报表分类")
    @RequestMapping(value = ServiceApi.Resources.RsReportCategories, method = RequestMethod.GET)
    public List<MRsReportCategory> search(
            @ApiParam(name = "codeName", value = "资源分类编码或名称")
            @RequestParam(value = "codeName", required = false) String codeName) throws ParseException {
        List<MRsReportCategory> resultList = new ArrayList<>();

        // 获取最顶层的资源报表分类集合
        List<RsReportCategory> topNodeList = rsReportCategoryService.getChildrenByPid(0);
        if (topNodeList.size() == 0) {
            return resultList;
        }

        // 暂存最顶层资源报表分类中，满足条件的集合
        List<RsReportCategory> topNodeListIn = new ArrayList<>();
        // 暂存最顶层资源报表分类中，不满足条件的集合
        List<RsReportCategory> topNodeListOut = new ArrayList<>();

        if (StringUtils.isEmpty(codeName)) {
            List<RsReportCategory> treeList = rsReportCategoryService.getTreeByParents(topNodeList);
            return (List<MRsReportCategory>) convertToModels(treeList, resultList, MRsReportCategory.class, "");
        }

        for (RsReportCategory reportCategory : topNodeList) {
            if (reportCategory.getCode().contains(codeName) || reportCategory.getName().contains(codeName)) {
                topNodeListIn.add(reportCategory);
                continue;
            }
            topNodeListOut.add(reportCategory);
        }
        if (topNodeListIn.size() != 0) {
            List<RsReportCategory> inList = rsReportCategoryService.getTreeByParents(topNodeListIn);
            resultList.addAll(convertToModels(inList, new ArrayList<>(), MRsReportCategory.class, ""));
        }
        List<RsReportCategory> outList = rsReportCategoryService.getTreeByParentsAndCodeName(topNodeListOut, codeName);
        resultList.addAll(convertToModels(outList, new ArrayList<>(), MRsReportCategory.class, ""));

        return resultList;
    }

    @ApiOperation(value = "获取资源报表分类的树形下拉框数据")
    @RequestMapping(value = ServiceApi.Resources.RsReportCategoryComboTree, method = RequestMethod.GET)
    public List<MRsReportCategory> getComboTreeData() throws Exception {
        List<RsReportCategory> list = rsReportCategoryService.getAllTreeData();
        return (List<MRsReportCategory>) convertToModels(list, new ArrayList<>(list.size()), MRsReportCategory.class, "");
    }

    @ApiOperation("新增资源报表分类")
    @RequestMapping(value = ServiceApi.Resources.RsReportCategorySave, method = RequestMethod.POST)
    public ObjEnvelop add(
            @ApiParam(name = "rsReportCategory", value = "资源报表分类JSON", required = true)
            @RequestBody String rsReportCategory) {
        try {
            RsReportCategory newRsReportCategory = toEntity(rsReportCategory, RsReportCategory.class);
            if (null == newRsReportCategory.getPid()) {
                newRsReportCategory.setPid(0);
            }
            if (null == newRsReportCategory.getSortNo()) {
                newRsReportCategory.setSortNo(99);
            }

            if(newRsReportCategory.getId()==null){
                //新增判断名称和code 是否唯一
                if(!rsReportCategoryService.isUniqueName(0, newRsReportCategory.getName())){
                    return ObjEnvelop.getError("名称已存在",-1);
                }
                if(!rsReportCategoryService.isUniqueCode(0, newRsReportCategory.getCode())){
                    return ObjEnvelop.getError("编码已存在",-1);
                }
            }else{
                //修改 如果有修改name和code要判断是否唯一
                RsReportCategory oldRsReportCategory = rsReportCategoryService.getById(newRsReportCategory.getId());
                if(!oldRsReportCategory.getName().equals(newRsReportCategory.getName())){
                    if(!rsReportCategoryService.isUniqueName(newRsReportCategory.getId(), newRsReportCategory.getName())){
                        return ObjEnvelop.getError("名称已存在",-1);
                    }
                }
                if(!oldRsReportCategory.getCode().equals(newRsReportCategory.getCode())){
                    if(!rsReportCategoryService.isUniqueCode(newRsReportCategory.getId(), newRsReportCategory.getCode())){
                        return ObjEnvelop.getError("编码已存在",-1);
                    }
                }
            }

            newRsReportCategory = rsReportCategoryService.save(newRsReportCategory);
            MRsReportCategory mRsReportCategory = convertToModel(newRsReportCategory, MRsReportCategory.class);
            return ObjEnvelop.getSuccess("操作成功",mRsReportCategory);
        }catch (Exception e){
            e.printStackTrace();
        }
        return ObjEnvelop.getError("操作失败",-1);
    }

    @ApiOperation("更新资源报表分类")
    @RequestMapping(value = ServiceApi.Resources.RsReportCategorySave, method = RequestMethod.PUT)
    public MRsReportCategory update(
            @ApiParam(name = "rsReportCategory", value = "资源报表分类JSON", required = true)
            @RequestBody String rsReportCategory) throws Exception {
        RsReportCategory newRsReportCategory = toEntity(rsReportCategory, RsReportCategory.class);
        newRsReportCategory = rsReportCategoryService.save(newRsReportCategory);
        return convertToModel(newRsReportCategory, MRsReportCategory.class);
    }

    @ApiOperation("删除资源报表分类")
    @RequestMapping(value = ServiceApi.Resources.RsReportCategoryDelete, method = RequestMethod.DELETE)
    public void delete(
            @ApiParam(name = "id", value = "资源报表分类ID", required = true)
            @RequestParam(value = "id") Integer id) throws Exception {
        rsReportCategoryDao.deleteById(id);
    }

    @ApiOperation("验证资源报表分类编码是否唯一")
    @RequestMapping(value = ServiceApi.Resources.RsReportCategoryIsUniqueCode, method = RequestMethod.GET)
    public boolean isUniqueCode(
            @ApiParam(name = "id", value = "资源报表分类ID", required = true)
            @RequestParam(value = "id") Integer id,
            @ApiParam(name = "code", value = "资源报表分类编码", required = true)
            @RequestParam(value = "code") String code) throws Exception {
        return rsReportCategoryService.isUniqueCode(id, code);
    }

    @ApiOperation("验证资源报表分类名称是否唯一")
    @RequestMapping(value = ServiceApi.Resources.RsReportCategoryIsUniqueName, method = RequestMethod.GET)
    public boolean isUniqueName(
            @ApiParam(name = "id", value = "资源报表分类ID", required = true)
            @RequestParam(value = "id") Integer id,
            @ApiParam(name = "name", value = "资源报表分类名称", required = true)
            @RequestParam(value = "name") String name) throws Exception {
        return rsReportCategoryService.isUniqueName(id, name);
    }

    @RequestMapping(value = ServiceApi.Resources.RsReportCategoryNoPageCategories, method = RequestMethod.GET)
    @ApiOperation("获取资源报表类别")
    public List<MRsReportCategory> getAllCategories(
            @ApiParam(name = "filters", value = "过滤", defaultValue = "")
            @RequestParam(value = "filters", required = false) String filters) throws Exception {
        List<RsReportCategory> list = rsReportCategoryService.search(filters);
        return (List<MRsReportCategory>) convertToModels(list, new ArrayList<>(list.size()), MRsReportCategory.class, null);
    }

    @RequestMapping(value = ServiceApi.Resources.RsReportCategoryByApp, method = RequestMethod.GET)
    @ApiOperation("获取平台应用对应的报表分类")
    public List<RsReportCategory> getCategoryByApp(
            @ApiParam(name = "appId", value = "应用Id")
            @RequestParam(value = "appId") String appId) {
        List<RsReportCategory> reportCategoryList = rsReportCategoryService.getCategoryByApp(appId);
        return reportCategoryList;
    }

    @RequestMapping(value = ServiceApi.Resources.RsReportCategoryByIds, method = RequestMethod.GET)
    @ApiOperation("根据Id获取平台应用对应的报表分类")
    public List<RsReportCategory> getCategoryByIds(
            @ApiParam(name = "ids", value = "id")
            @RequestParam(value = "ids") List<Integer> ids) {
        List<RsReportCategory> categoryList = rsReportCategoryService.getCategoryByIds(ids);
        return categoryList;
    }

    @RequestMapping(value = ServiceApi.Resources.RsReportCategoryIdsByCode, method = RequestMethod.GET)
    @ApiOperation("根据code获取平台应用对应的报表分类子类")
    public List<Integer> getCategoryIdsByCode(
            @ApiParam(name = "code", value = "分类编码")
            @RequestParam(value = "code") String code) {
        List<Integer> categoryIds = rsReportCategoryService.getCategoryIds(code);
        return categoryIds;
    }

    @RequestMapping(value = ServiceApi.Resources.CategoryTree, method = RequestMethod.GET)
    @ApiOperation(value = "获取所有资源类别转成的RsCategoryTypeTreeModel列表，初始页面显示")
    public ListEnvelop getRsCategoryTreeModels(
            @ApiParam(name = "userResource", value = "授权资源")
            @RequestParam(value = "userResource") String userResource,
            @ApiParam(name = "name", value = "资源类别的名称")
            @RequestParam(value = "name",required = false) String name) throws Exception {
        ListEnvelop envelop = new ListEnvelop();
        //获取到的顶级cda类别集合
        List<RsResourceCategory> categoryList;
        if(userResource.equals("*")) {
            categoryList = rsCategoryService.getRsCategoryByPid("");
        }else {
            categoryList = rsCategoryService.findByCodeAndPid("derived", "");
        }
        //顶级类别中符合条件的类别集合
        List<RsResourceCategory> mRsCategoriesSome = new ArrayList<>();
        //顶级类别中不符合条件的类别集合
        List<RsResourceCategory> mRsCategoriesOthers = new ArrayList<>();
        if (categoryList == null || categoryList.size() == 0){
            envelop.setStatus(-1);
            envelop.setMessage("没有匹配条件的资源类别！");
            return envelop;
        }
        List<RsCategoryTypeTreeModel> treeList = new ArrayList<>();
        if(StringUtils.isEmpty(name)){
            treeList = getRsCategoryTreeModelChild(categoryList);
            envelop.setDetailModelList(treeList);
            return envelop;
        }
        for(RsResourceCategory mRsCategory : categoryList){
            if(mRsCategory.getName().contains(name)){
                mRsCategoriesSome.add(mRsCategory);
                continue;
            }
            mRsCategoriesOthers.add(mRsCategory);
        }
        if (mRsCategoriesSome.size() != 0){
            treeList.addAll(getRsCategoryTreeModelChild(mRsCategoriesSome));
        }
        treeList .addAll(getRsCategoryTreeModelByName(mRsCategoriesOthers, name));
        envelop.setDetailModelList(treeList);
        return envelop;
    }

    /**
     *
     * 根据父级信息获取全部的子级信息（树形model）
     * @param info 父级信息
     * @return 全部子级信息
     */
    private List<RsCategoryTypeTreeModel> getRsCategoryTreeModelChild(List<RsResourceCategory> info) {
        List<RsCategoryTypeTreeModel> treeInfo = new ArrayList<>();
        for (int i = 0; i < info.size(); i++) {
            RsResourceCategory typeInfo = info.get(i);
            RsCategoryTypeTreeModel tree = convertToModel(typeInfo, RsCategoryTypeTreeModel.class);
            List<RsResourceCategory> categoryListChild = rsCategoryService.getRsCategoryByPid(typeInfo.getId());
            List<RsCategoryTypeTreeModel> listChildTree = getRsCategoryTreeModelChild(categoryListChild);
            tree.setChildren(listChildTree);
            treeInfo.add(tree);
        }
        return treeInfo;
    }

    /**
     * 递归不满足的父级类别集合的子集中满足条件TreeModel集合的方法
     * @param mRsCategories 不符合的父级类别的集合
     * @param name
     * @return
     */
    private List<RsCategoryTypeTreeModel> getRsCategoryTreeModelByName(List<RsResourceCategory> mRsCategories, String name) throws ParseException {
        //结构：treeList 包含treeModel，treeModel包含listOfParent
        List<RsCategoryTypeTreeModel> treeList = new ArrayList<>();
        for(RsResourceCategory mRsCategory:mRsCategories){
            List<RsCategoryTypeTreeModel> childList = new ArrayList<>();
            RsCategoryTypeTreeModel treeModel = convertToModel(mRsCategory,RsCategoryTypeTreeModel.class);
            String pid = mRsCategory.getId();
            //获取所有下一级cda类别
            List<RsResourceCategory> listAll = rsCategoryService.getRsCategoryByPid(pid);
            if(listAll.size() == 0){
                continue;
            }
            //获取所有下一级符合要求的资源类别
            String filters ="pid="+pid+";name?"+name;

            //modify by cws
            List<RsResourceCategory> listSome = rsCategoryService.search(filters);
            //List<MRsCategory> listSome = (List<MRsCategory>)responseEntity.getBody();
            if(listSome.size()!=0){
                childList.addAll(getRsCategoryTreeModelChild(listSome));
            }
            //取剩下不符合要求的进行递归
            listAll.removeAll(listSome);
            if(listAll.size() != 0){
                childList.addAll(getRsCategoryTreeModelByName(listAll, name));
            }
            if(childList.size() != 0){
                treeModel.setChildren(childList);
                treeList.add(treeModel);
            }
        }
        return treeList;
    }
}
