package com.yihu.jw.basic.resource.dao;


import com.yihu.jw.entity.ehr.resource.DtoResourceMetadata;
import com.yihu.jw.entity.ehr.resource.RsResourceMetadata;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;


/**
 * Created by hzp on 2016/5/14.
 */
@Repository
public class ResourceBrowseMetadataDao {

    @Autowired
    private JdbcTemplate jdbcTemplate;

    /**
     * 获取某资源所有数据元
     */
    public List<RsResourceMetadata> findResourceMetadata(String resourcesCode) throws Exception {
        String sql = "SELECT m.* from rs_resource_metadata m,rs_resource r WHERE m.RESOURCES_ID=r.id and r.`code` = '"+resourcesCode+"'";
        RowMapper rowMapper = BeanPropertyRowMapper.newInstance(RsResourceMetadata.class);
        return jdbcTemplate.query(sql, rowMapper);
    }

    /**
     * 获取某资源所有数据元
     */
    public List<Map<String,Object>> getAllMetadata(String resourcesCode) throws Exception {
        String sql = "SELECT m.EHR_ID,m.name,m.DICT_CODE ,d.name dictName,m.COLUMN_TYPE columnType  " +
                "FROM rs_resource_metadata a, rs_resource b, rs_metadata m LEFT JOIN rs_dictionary d on m.dict_code=d.code " +
                "WHERE a.resources_id = b.id " +
                "AND a.metadata_id = m.ehr_id " +
                "AND b.code = '" + resourcesCode + "'";
        return jdbcTemplate.queryForList(sql);
    }

    /**
     * 获取某资源所有数据元
     */
    public List<DtoResourceMetadata> getAllResourceMetadata(String resourcesCode) throws Exception {
        String sql = "SELECT m.id,m.ehr_id, m.domain, m.name, m.std_code, m.display_code, m.column_type, m.null_able, m.dict_code, m.description, m.valid, a.group_type, a.group_data " +
                "FROM rs_resource_metadata a, rs_resource b, rs_metadata m " +
                "WHERE a.resources_id = b.id " +
                "AND a.metadata_id = m.ehr_id " +
                "AND b.code = '" + resourcesCode + "' ORDER BY a.sort";
        RowMapper rowMapper = BeanPropertyRowMapper.newInstance(DtoResourceMetadata.class);
        return jdbcTemplate.query(sql, rowMapper);
    }

    /**
     * 获取某资源有字典的数据元
     */
    public List<DtoResourceMetadata> getResourceMetadatasHasDict(String resourcesCodes) throws Exception {
        String sql = "SELECT m.id,m.ehr_id, m.domain, m.name, m.std_code, m.display_code, m.column_type, m.null_able, m.dict_code, m.description, m.valid, a.group_type, a.group_data " +
                "FROM rs_resource_metadata a, rs_resource b, rs_metadata m " +
                "WHERE a.resources_id = b.id " +
                "AND a.metadata_id = m.ehr_id " +
                "AND b.code in ('" + resourcesCodes + "') and m.DICT_CODE is not null ";
        RowMapper rowMapper = BeanPropertyRowMapper.newInstance(DtoResourceMetadata.class);
        return jdbcTemplate.query(sql, rowMapper);
    }

    /**
     * 获取某资源
     */
    public List<Map<String,Object>> getMetadatas(String resourcesCodes) throws Exception {
        String sql = "SELECT m.ehr_id, m.name " +
                "      FROM rs_resource_metadata a, rs_resource b, rs_metadata m " +
                "      WHERE a.resources_id = b.id " +
                "      AND a.metadata_id = m.ehr_id " +
                "AND b.code in ('" + resourcesCodes + "')  ";
        return jdbcTemplate.queryForList(sql);
    }

    /**
     * 获取某资源有字典的数据元
     */
    public List<DtoResourceMetadata> getResourceMetadataHasDict(String resourcesCode) throws Exception {
        String sql = "SELECT m.id,m.ehr_id, m.domain, m.name, m.std_code, m.display_code, m.column_type, m.null_able, m.dict_code, m.description, m.valid, a.group_type, a.group_data " +
                "FROM rs_resource_metadata a, rs_resource b, rs_metadata m " +
                "WHERE a.resources_id = b.id " +
                "AND a.metadata_id = m.ehr_id " +
                "AND b.code = '" + resourcesCode + "' and m.DICT_CODE is not null ";
        RowMapper rowMapper = BeanPropertyRowMapper.newInstance(DtoResourceMetadata.class);
        return jdbcTemplate.query(sql, rowMapper);
    }

    /**
     * 获取某资源数据元(根据资源数据元id过滤)
     */
    public List<DtoResourceMetadata> getAuthResourceMetadata(String rsMetadataIds) throws Exception {
        String sql = "SELECT m.id, m.domain, m.name, m.std_code, m.display_code, m.column_type, m.null_able, m.dict_code, m.description, m.valid, a.group_type, a.group_data " +
                "FROM rs_resource_metadata a, rs_metadata m " +
                "WHERE a.id IN (" + rsMetadataIds + ") " +
                "AND a.metadata_id = m.id ";
        RowMapper rowMapper = BeanPropertyRowMapper.newInstance(DtoResourceMetadata.class);
        return jdbcTemplate.query(sql, rowMapper);
    }

    /**
     * 为其他平台标准数据元转换新增的方法
     * @param rsMetadataIds
     * @return
     * @throws Exception
     */
    public List<DtoResourceMetadata> getRsMetadataByIds(String rsMetadataIds) throws Exception {
        String sql = "SELECT m.id, m.std_code, m.column_type, m.dict_code " +
                "FROM rs_metadata m " +
                "WHERE m.id IN (" + rsMetadataIds + ")";
        RowMapper rowMapper = BeanPropertyRowMapper.newInstance(DtoResourceMetadata.class);
        return jdbcTemplate.query(sql, rowMapper);
    }

    public List<Map<String, Object>> getMetaData(List<String> idsList){
        StringBuilder builder = new StringBuilder();
        for (String id : idsList){
            builder.append("'");
            builder.append(id);
            builder.append("',");
        }
        String ids = "";
        if (builder.length() > 1){
            ids = builder.substring(0, builder.length() - 1);
        }
        String sql ="SELECT id, name FROM rs_metadata WHERE id in(" + ids + ")";
        List<Map<String,Object>> list = jdbcTemplate.queryForList(sql);
        return list;
    }

}
