package com.yihu.jw.basic.resource.service;


import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.basic.resource.dao.ResourceBrowseDao;
import com.yihu.jw.basic.resource.dao.ResourceBrowseMetadataDao;
import com.yihu.jw.basic.resource.dao.RsResourceDao;
import com.yihu.jw.basic.resource.dao.RsResourceDefaultParamDao;
import com.yihu.jw.constants.ErrorCode;
import com.yihu.jw.entity.ehr.resource.DtoResourceMetadata;
import com.yihu.jw.entity.ehr.resource.RsResource;
import com.yihu.jw.entity.ehr.resource.RsResourceDefaultParam;
import com.yihu.jw.entity.ehr.resource.RsResourceMetadata;
import com.yihu.jw.exception.ApiException;
import com.yihu.jw.mysql.enums.Operation;
import com.yihu.jw.mysql.model.QueryCondition;
import com.yihu.jw.mysql.query.BaseJpaService;
import com.yihu.jw.profile.ProfileType;
import com.yihu.jw.profile.family.ResourceCells;
import com.yihu.jw.restmodel.ehr.resource.MRsColumnsModel;
import com.yihu.jw.restmodel.web.Envelop;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.lang.reflect.Method;
import java.util.*;


/**
 * Created by hzp on 2016/4/13.
 */
@Service
public class ResourceBrowseService extends BaseJpaService {


    @Autowired
    private RsResourceDao rsResourceDao;
    @Autowired
    private ResourceBrowseMetadataDao resourceBrowseMetadataDao;
    @Autowired
    private ResourceBrowseDao resourceBrowseDao;
    /*    @Autowired
        private StdTransformClient stdTransformClient;*/
    @Autowired
    private RedisService redisService;
    //    @Autowired
//    private ElasticsearchUtil elasticSearchUtil;
    @Autowired
    private RsResourceService rsResourceService;
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private RsResourceDefaultParamDao resourceDefaultParamDao;



    /**
     * 资源浏览 -- 资源数据元结构
     *
     * @param resourcesCode
     * @param roleId
     * @return
     * @throws Exception
     */
    public List<MRsColumnsModel> getResourceMetadata(String resourcesCode, String roleId) throws Exception {
        RsResource rsResource = rsResourceService.getResourceByCategory(resourcesCode,"standard");
        if (rsResource == null) {
            return null;
        }
        Map<String, String> correspondMap = new HashMap<>();
        List<DtoResourceMetadata> metadataList = resourceBrowseDao.getAccessMetadata(rsResource, roleId, correspondMap);
        //资源结构
        List<MRsColumnsModel> mRsColumnsModels = new ArrayList<>();
        if (metadataList != null) {
            for (DtoResourceMetadata r : metadataList) {
                MRsColumnsModel mRsColumnsModel = new MRsColumnsModel();
                mRsColumnsModel.setValue(r.getName());
                if (!StringUtils.isEmpty(r.getDictCode())) {
                    mRsColumnsModel.setCode(r.getStdCode());
                } else {
                    mRsColumnsModel.setCode(r.getStdCode());
                }
                mRsColumnsModel.setType(r.getColumnType());
                mRsColumnsModel.setDict(r.getDictCode());
                mRsColumnsModels.add(mRsColumnsModel);
            }
        }
        return mRsColumnsModels;
    }

    /**
     * 资源浏览 - 资源数据
     *
     * @param resourcesCode
     * @param roleId
     * @param orgCode
     * @param areaCode
     * @param queryCondition
     * @param page
     * @param size
     * @return
     * @throws Exception
     */
    public Object getResourceData(String resourcesCode, String roleId, String orgCode, String areaCode, String queryCondition, Integer page, Integer size) throws Exception {
        String queryParams = "";
        //获取资源信息
        List<QueryCondition> ql = new ArrayList<>();
        //设置参数
        if (!StringUtils.isEmpty(queryCondition)) {
            ql = parseCondition(queryCondition);
        }
        queryParams = addParams(queryParams, "q", conditionToString(ql));
        return getResultData(resourcesCode, roleId, orgCode, areaCode, queryParams, page, size);
    }

    public String conditionToString(List<QueryCondition> conditions) {
        String re = "";
        String NOT = "";
        if (conditions != null && conditions.size() > 0) {
            Iterator var4 = conditions.iterator();

            while(true) {
                while(var4.hasNext()) {
                    QueryCondition condition = (QueryCondition)var4.next();
                    if (re.equals("") && NOT.equals("")) {
                        re = re + this.conditionToString(condition);
                        if (this.conditionToString(condition).indexOf("NOT") == 0) {
                            NOT = "NOT(" + re.substring(3);
                            re = "";
                        }
                    } else {
                        String var6 = condition.getLogical();
                        byte var7 = -1;
                        switch(var6.hashCode()) {
                            case 2531:
                                if (var6.equals("OR")) {
                                    var7 = 1;
                                }
                                break;
                            case 64951:
                                if (var6.equals("AND")) {
                                    var7 = 0;
                                }
                                break;
                            case 77491:
                                if (var6.equals("NOT")) {
                                    var7 = 2;
                                }
                        }

                        switch(var7) {
                            case 0:
                                re = re + " AND ";
                                break;
                            case 1:
                                re = re + " OR  ";
                                break;
                            case 2:
                                re = re + " NOT ";
                        }

                        if (this.conditionToString(condition).indexOf("NOT") == 0) {
                            if (NOT.equals("")) {
                                NOT = " NOT(" + this.conditionToString(condition).substring(3);
                                re = re.substring(0, re.length() - 5);
                            } else {
                                NOT = NOT + re.substring(re.length() - 5) + this.conditionToString(condition).substring(3, this.conditionToString(condition).length());
                                re = re.substring(0, re.length() - 5);
                            }
                        } else {
                            re = re + this.conditionToString(condition);
                            re = "(" + re + ")";
                        }
                    }
                }

                return NOT.equals("") ? re : re + NOT + ")";
            }
        } else {
            re = "*:* ";
            return NOT.equals("") ? re : re + NOT + ")";
        }
    }

    /**
     * solr语法转换
     */
    public String conditionToString(QueryCondition condition) {
        String s = "";
        String operation = condition.getOperation();
        String field = condition.getField();
        Object keyword = condition.getKeyword();
        Object[] keywords = condition.getKeywords();
        switch (operation) {
            case Operation.LIKE:
                s = field + ":*" + keyword + "*";
                break;
            case Operation.LEFTLIKE:
                s = field + ":*" + keyword + "";
                break;
            case Operation.RIGHTLIKE:
                s = field + ":" + keyword + "*";
                break;
            case Operation.RANGE: {
                if (keywords.length >= 2) {
                    s = field + ":[" + keywords[0] + " TO " + keywords[1] + "]";
                }
                break;
            }
            case Operation.NOTRANGE: {
                if (keywords.length >= 2) {
                    s = "NOT " + field + ":[" + keywords[0] + " TO " + keywords[1] + "]";
                }
                break;
            }
            case Operation.NE: {
                s = "NOT(" + field + ":" + keyword + ")";
                break;
            }
            case Operation.IN: {
                String in = "";
                if (keywords != null && keywords.length > 0) {
                    for (Object key : keywords) {
                        if (in != null && in.length() > 0) {
                            in += " OR " + field + ":" + key;
                        } else {
                            in = field + ":" + key;
                        }
                    }
                } else if (keyword != null) {
                    in = field + ":" + keyword;
                }
                s = "( " + in + " )";
                break;
            }
            case Operation.NIN: {
                String in = "";
                if (keywords != null && keywords.length > 0) {
                    for (Object key : keywords) {
                        if (in != null && in.length() > 0) {
                            in += " OR " + field + ":" + key;
                        } else {
                            in = field + ":" + key;
                        }
                    }
                } else if (keyword != null) {
                    in = field + ":" + keyword;
                }
                s = "NOT (" + in + ")";
                break;
            }
            case Operation.GT: {
                s = field + ":{" + keyword + " TO *}";
                break;
            }
            case Operation.GTE: {
                s = field + ":[" + keyword + " TO * ]";
                break;
            }
            case Operation.LT: {
                s = field + ":" + "{* TO " + keyword + "}";
                break;
            }
            case Operation.LTE: {
                s = field + ":" + "[* TO " + keyword + "]";
                break;
            }
            case Operation.EQ: {
                s = field + ":" + keyword;
                break;
            }
            default:
                s = "unknown operation";
        }

        return s;
    }

    /**
     * 获取结果集
     *
     * @param resourcesCode
     * @param roleId
     * @param orgCode
     * @param areaCode
     * @param queryParams
     * @param page
     * @param size
     * @return
     * @throws Exception
     */
    public Object getResultData(String resourcesCode, String roleId, String orgCode, String areaCode, String queryParams, Integer page, Integer size) throws Exception {
        RsResource rsResources = rsResourceService.getResourceByCategory(resourcesCode,"standard");
        if (rsResources != null) {
            StringBuilder saas = new StringBuilder();
            if ("*".equals(orgCode) && "*".equals(areaCode)) {
                saas.append("*");
            } else {
                List<String> orgCodeList = objectMapper.readValue(orgCode, List.class);
                List<String> areaCodeList = objectMapper.readValue(areaCode, List.class);
                if (orgCodeList != null && orgCodeList.size() > 0) {
                    orgCodeList.forEach(item -> {
                        if (saas.length() <= 0) {
                            saas.append("(org_code:" + item);
                        } else {
                            saas.append(" OR org_code:" + item);
                        }
                    });
                }
                if (areaCodeList != null && areaCodeList.size() > 0) {
                    areaCodeList.forEach(item -> {
                        if (saas.length() <= 0) {
                            saas.append("(org_area:" + item);
                        } else {
                            saas.append(" OR org_area:" + item);
                        }
                    });
                }
                if (saas.length() > 0) {
                    saas.append(")");
                } else {
                    throw new ApiException( "无SAAS权限访问资源",ErrorCode.FORBIDDEN.value());
                }
            }
            String method = rsResources.getRsInterface();
            Class clazz = resourceBrowseDao.getClass();

            if(method.equalsIgnoreCase("getMysqlData")){
                Method _method = clazz.getMethod(method, new Class[]{ String.class, Integer.class, Integer.class});
                _method.setAccessible(true);
                return _method.invoke(resourceBrowseDao, rsResources.getCode(), page, size);
            }else if(method.equalsIgnoreCase("getMysqlStatistics")){
                Method _method = clazz.getMethod(method, new Class[]{ String.class, Integer.class, Integer.class});
                _method.setAccessible(true);
                return _method.invoke(resourceBrowseDao, rsResources.getCode(), page, size);
            }else {
                Method _method = clazz.getMethod(method, new Class[]{String.class, String.class, String.class, String.class, Integer.class, Integer.class});
                _method.setAccessible(true);
                return _method.invoke(resourceBrowseDao, resourcesCode, roleId, saas.toString(), queryParams, page, size);
            }

        }
        throw new ApiException( "无相关资源",ErrorCode.OBJECT_NOT_FOUND.value());
    }

    /**
     * 获取hbase细表数据
     */
//    public List<Object> getSubDateByRowkey (String rowKey, String version) throws Exception {
//        //查询出所有细表的rowKey
//        List<Object> resultList = new ArrayList<>();
//        String q = "{\"q\":\"profile_id:" + rowKey + "\"}";
//        Envelop page = resourceBrowseDao.getEhrCenterSub(null, "*", "*", q, 1, 1000);
//        List<Map<String, Object>> pageContent = page.getDetailModelList();
//        Map<String, Object> resultMap = new HashMap<>();
//        for (Map<String, Object> temp : pageContent) {
//            String subRowKey = temp.get("rowkey").toString();
//            String dataSetCode = subRowKey.split("\\$")[1];
//            if (!resultMap.containsKey(dataSetCode)) {
//                Map<String, Object> dataMap = new HashMap<>();
//                dataMap.put("name", redisService.getDataSetName(version, dataSetCode));
//                List<Map<String, Object>> dataList = new ArrayList<>();
//                Map<String, Object> tempMap = new HashMap<>(temp.size());
//                List<String> idsList = new ArrayList<>(temp.size());
//                for (String id : temp.keySet()) {
//                    if (id.startsWith("EHR")) {
//                        idsList.add(id);
//                    }
//                }
//                List<Map<String, Object>> metaList = resourceBrowseMetadataDao.getMetaData(idsList);
//                metaList.stream().forEach(one -> {
//                    Object obj = temp.get(String.valueOf(one.get("id")));
//                    tempMap.put(String.valueOf(one.get("name")), obj);
//                });
//                dataList.add(tempMap);
//                dataMap.put("data", dataList);
//                resultMap.put(dataSetCode, dataMap);
//            } else {
//                Map<String, Object> dataMap = (Map<String, Object>) resultMap.get(dataSetCode);
//                List<Map<String, Object>> dataList = (List<Map<String, Object>>) dataMap.get("data");
//                Map<String, Object> tempMap = new HashMap<>(temp.size());
//                List<String> idsList = new ArrayList<>(temp.size());
//                for (String id : temp.keySet()) {
//                    if (id.startsWith("EHR")) {
//                        idsList.add(id);
//                    }
//                }
//                List<Map<String, Object>> metaList = resourceBrowseMetadataDao.getMetaData(idsList);
//                metaList.stream().forEach(one -> {
//                    Object obj = temp.get(String.valueOf(one.get("id")));
//                    tempMap.put(String.valueOf(one.get("name")), obj);
//                });
//                dataList.add(tempMap);
//                dataMap.put("data", dataList);
//                resultMap.put(dataSetCode, dataMap);
//            }
//        }
//        resultList.addAll(resultMap.entrySet());
//        return resultList;
//    }

    /**
     * 综合查询档案数据检索
     *
     * @param resourcesCodes
     * @param metaData
     * @param orgCode
     * @param areaCode
     * @param queryCondition
     * @param page
     * @param size
     * @return
     * @throws Exception
     */
    public Envelop getCustomizeData(String resourcesCodes, String metaData, String orgCode, String areaCode, String queryCondition, Integer page, Integer size) throws Exception {
        //获取资源编码列表
        List<String> codeList = (List<String>) objectMapper.readValue(resourcesCodes, List.class);
        //资源判空检查
        for (String code : codeList) {
            RsResource rsResources = rsResourceService.getResourceByCategory(code,"standard");

            if (rsResources == null) {
                throw new ApiException( "无效的资源编码" + code,ErrorCode.BAD_REQUEST.value());
            }
        }
        StringBuilder saas = new StringBuilder();
        if ("*".equals(orgCode) && "*".equals(areaCode)) {
            saas.append("*");
        } else {
            List<String> orgCodeList = objectMapper.readValue(orgCode, List.class);
            List<String> areaCodeList = objectMapper.readValue(areaCode, List.class);
            if (orgCodeList != null && orgCodeList.size() > 0) {
                orgCodeList.forEach(item -> {
                    if (saas.length() <= 0) {
                        saas.append("(org_code:" + item);
                    } else {
                        saas.append(" OR org_code:" + item);
                    }
                });
            }
            if (areaCodeList != null && areaCodeList.size() > 0) {
                areaCodeList.forEach(item -> {
                    if (saas.length() <= 0) {
                        saas.append("(org_area:" + item);
                    } else {
                        saas.append(" OR org_area:" + item);
                    }
                });
            }
            if (saas.length() > 0) {
                saas.append(")");
            } else {
                throw new ApiException("无SAAS权限访问资源");
            }
        }
        List<QueryCondition> ql = new ArrayList<>();
        if (!StringUtils.isEmpty(queryCondition)) {
            ql = parseCondition(queryCondition);
        }
        String queryParams = "";
        if (ql.size() > 0) {
           /* if (solrQuery.conditionToString(ql).contains(":")) {
                queryParams = addParams(queryParams, "q", solrQuery.conditionToString(ql));
            } else {
                queryParams = addParams(queryParams, "q", "*:*");
            }*/
        } else {
            queryParams = addParams(queryParams, "q", "*:*");
        }
        queryParams = addParams(queryParams, "sort", "{\"create_date\":\"desc\"}");
        //基础数据字段
        queryParams = addParams(queryParams, "basicFl", org.apache.commons.lang3.StringUtils.join(ResourceCells.getMasterBasicCell(ProfileType.Standard), ","));
        //数据元信息字段
        List<String> customizeList = (List<String>) objectMapper.readValue(metaData, List.class);
        //参数集合
        List<String> paramList = new ArrayList<>(customizeList.size() * 2);
        for (String id : customizeList) {
            paramList.add(id);
            String dictCode = redisService.getRsMetadataDict(id);
            if (!StringUtils.isEmpty(dictCode)) {
                paramList.add(id + "_VALUE");
            }
        }
        String queryString = org.apache.commons.lang3.StringUtils.join(paramList, ",");
        queryParams = addParams(queryParams, "dFl", org.apache.commons.lang3.StringUtils.join(paramList, ","));
        return resourceBrowseDao.getEhrCenter(null, "*", saas.toString(), queryParams, page, size);
    }


    /**
     * 综合查询档案数据检索
     *
     * @param resourcesCodes
     * @param metaData
     * @param queryCondition
     * @param page
     * @param size
     * @return
     * @throws Exception
     */
    public Page<Map<String, Object>> getCustomizeDataMysql(String resourcesCodes, String metaData, String queryCondition, Integer page, Integer size) throws Exception {
        //获取资源编码列表
        List<String> codeList = (List<String>) objectMapper.readValue(resourcesCodes, List.class);
        String tableName = "";
        String paramSql = "";
        String where = "";
        Map<String,Integer> relationMap = new HashMap<>();//多视图关联
        Map<String,String> filedMap = new HashMap<>();//参数加别名
        Map<String,String> dataMap = new HashMap<>();
        if (!StringUtils.isEmpty(queryCondition)) {
            dataMap = parseConditionMap(queryCondition);
        }
        int dataSource = 0;
        //资源判空检查
        for (String code : codeList) {
            RsResource rsResources = rsResourceService.getResourceByCategory(code,"standard");
            if (rsResources == null) {
                throw new ApiException( "无效的资源编码" + code,ErrorCode.BAD_REQUEST.value());
            }
            if(dataSource==0){
                dataSource = rsResources.getDataSource();
            }
            if(rsResources.getDataSource()==2&&codeList.size()>1){
                throw new ApiException("指标统计，只能选择单个指标");
            }
            List<RsResourceMetadata> resourceMetadataList = resourceBrowseMetadataDao.findResourceMetadata(code);
            for (RsResourceMetadata metadata:resourceMetadataList){
                filedMap.put(metadata.getMetadataId(),code);
            }
            String sortOrder = "";
            List<RsResourceDefaultParam> paramsList = resourceDefaultParamDao.findByResourcesCode(code);
            for (RsResourceDefaultParam param : paramsList) {
                String paramKey = param.getParamKey();
                String paramValue = param.getParamValue();
                if ("sort".equals(paramKey)) {
                    Map<String, String> temp = objectMapper.readValue(paramValue, Map.class);
                    for (Map.Entry<String, String> entry:temp.entrySet()){
                        String key = entry.getKey();
                        String value = entry.getValue();
                        sortOrder ="T."+key+" "+ value+",";
                    }
                }else if ("table".equals(paramKey)){
                    paramValue = resourceBrowseDao.transforParamValue(paramValue,dataMap);
                    tableName += " ("+paramValue+") "+ code +",";
                }else if("relation".equals(paramKey)){
                    String relate[] = paramValue.split(",");
                    for (String re:relate){
                        if(relationMap.containsKey(re)){
                            relationMap.put(re,relationMap.get(re)+1);
                        }else {
                            relationMap.put(re,1);
                        }
                    }
                }else if (param.getParamKey().equals("param")){
                    paramSql = param.getParamValue();
                }
            }
        }
        String code1 = codeList.get(0);
        int codeLength = codeList.size();
        for (int i=1;i<codeLength ;i++) {
            String codeTmp = codeList.get(i);
            for (Map.Entry<String, Integer> entry : relationMap.entrySet()) {
                String key = entry.getKey();
                int value = entry.getValue();
                if(value == codeLength){
                    where += " and " + code1+"."+key+"="+codeTmp+"."+key;
                }
            }
        }

        tableName = tableName.substring(0,tableName.length()-1);

        if(!StringUtils.isEmpty(paramSql)){
            tableName = resourceBrowseDao.transforParam(paramSql,tableName,dataMap);
        }
        StringBuffer stringBuffer = new StringBuffer();
        if (!StringUtils.isEmpty(queryCondition)) {
            stringBuffer = parseCondition1(queryCondition);
        }
        stringBuffer.append(where);
        //数据元信息字段
        if(!StringUtils.isEmpty(metaData)){
            metaData = metaData.replaceAll(",","\",\"");
        }
        List<String> customizeList = (List<String>) objectMapper.readValue(metaData, List.class);
        //参数集合
        List<String> paramList = new ArrayList<>(customizeList.size() * 2);
        Map<String,Integer> fieldMap = new HashMap<>();//去除重复的列
        for (String id : customizeList) {
            if(fieldMap.containsKey(id)){
                continue;
            }
            fieldMap.put(id,1);
            String codeTmp = filedMap.get(id);
            if(id.contains("-")){
                String tmp = id.substring(id.lastIndexOf("-")+1);
                id = tmp + " as '" +id+"'";
            }
            if(codeTmp!=null){
                id = codeTmp+"."+id;
            }
            paramList.add(id);
        }
        String queryString = org.apache.commons.lang3.StringUtils.join(paramList, ",");
        String sql = " select "+queryString+" from "+tableName;
        String sortParam = "";
        return resourceBrowseDao.getMysqlDataAndCondition(sql, stringBuffer.toString(),sortParam, page, size,codeList,dataSource);
    }

    /**
     * 获取主表数据
     *
     * @param resourcesCode
     * @param roleId
     * @param saas
     * @param queryParams
     * @param page
     * @param size
     * @return
     * @throws Exception
     */
//    public Envelop getEhrCenter(String resourcesCode, String roleId, String saas, String queryParams, Integer page, Integer size) throws Exception {
//        return resourceBrowseDao.getEhrCenter(resourcesCode, roleId, saas, queryParams, page, size);
//    }

    /**
     * 获取细表数据
     *
     * @param resourcesCode
     * @param roleId
     * @param saas
     * @param queryParams
     * @param page
     * @param size
     * @return
     * @throws Exception
     */
//    public Envelop getEhrCenterSub(String resourcesCode, String roleId, String saas, String queryParams, Integer page, Integer size) throws Exception {
//        return resourceBrowseDao.getEhrCenterSub(resourcesCode, roleId, saas, queryParams, page, size);
//    }

    /**
     * 主表统计
     *
     * @param queryParams
     * @param page
     * @param size
     * @return
     * @throws Exception
     */
//    public Page<Map<String, Object>> countEhrCenter(String queryParams, Integer page, Integer size) throws Exception {
//        return resourceBrowseDao.countEhrCenter(queryParams, page, size);
//    }

    /**
     * 细表统计
     *
     * @param queryParams
     * @param page
     * @param size
     * @return
     * @throws Exception
     */
//    public Page<Map<String, Object>> countEhrCenterSub(String queryParams, Integer page, Integer size) throws Exception {
//        return resourceBrowseDao.countEhrCenterSub(queryParams, page, size);
//    }

    /**
     * 获取非结构化资源
     *
     * @param filters
     * @param sorts
     * @param page
     * @param size
     * @return
     * @throws Exception
     */
//    public Envelop getEhrFile(String filters, String sorts, Integer page, Integer size) throws Exception {
//        return resourceBrowseDao.getEhrFile(filters, sorts, page, size);
//    }

    /**
     * Mysql数据
     *
     * @param queryParams
     * @param page
     * @param size
     * @return
     * @throws Exception
     */
    public Page<Map<String, Object>> getMysqlData(String queryParams, Integer page, Integer size) throws Exception {
        return resourceBrowseDao.getMysqlData1(queryParams, page, size);
    }

    /**
     * @param queryParams
     * @param page
     * @param size
     * @return
     * @throws Exception
     */
//    public Page<String> getSolrIndexs(String queryParams, Integer page, Integer size) throws Exception {
//        return resourceBrowseDao.getSolrIndexs(queryParams, page, size);
//    }

    /**
     * 查询条件转换
     *
     * @param queryCondition
     * @return
     * @throws Exception
     */
    private List<QueryCondition> parseCondition(String queryCondition) throws Exception {
        List<QueryCondition> ql = new ArrayList<QueryCondition>();
        ObjectMapper mapper = new ObjectMapper();
        JavaType javaType = mapper.getTypeFactory().constructParametricType(List.class, Map.class);
        List<Map<String, Object>> list = objectMapper.readValue(queryCondition, javaType);
        if (list != null && list.size() > 0) {
            for (Map<String, Object> item : list) {
                String andOr = String.valueOf(item.get("andOr")).trim();
                String field = String.valueOf(item.get("field")).trim();
                String cond = String.valueOf(item.get("condition")).trim();
                String value = String.valueOf(item.get("value"));
                if (value.indexOf(",") > 0) {
                    ql.add(new QueryCondition(andOr, cond, field, value.split(",")));
                } else {
                    ql.add(new QueryCondition(andOr, cond, field, value));
                }
            }
        }
        return ql;
    }


    /**
     * 查询条件转换
     *
     * @param queryCondition
     * @return
     * @throws Exception
     */
    private StringBuffer parseCondition1(String queryCondition) throws Exception {
        StringBuffer stringBuffer = new StringBuffer();
        ObjectMapper mapper = new ObjectMapper();
        JavaType javaType = mapper.getTypeFactory().constructParametricType(List.class, Map.class);
        List<Map<String, Object>> list = objectMapper.readValue(queryCondition, javaType);
        if (list != null && list.size() > 0) {
            for (Map<String, Object> item : list) {
                String andOr = String.valueOf(item.get("andOr")).trim();
                String field = String.valueOf(item.get("field")).trim();
                if(field.contains("-")){
                    field = field.substring(field.lastIndexOf("-")+1);
//                    id = tmp + " as '" +id+"'";
                }
                String cond = String.valueOf(item.get("condition")).trim();
                String value = String.valueOf(item.get("value"));
                stringBuffer.append(" "+andOr+" "+field+cond+"'"+value+"' ");
            }
        }
        return stringBuffer;
    }

    /**
     * 查询条件转换
     *
     * @param queryCondition
     * @return
     * @throws Exception
     */
    private Map<String,String> parseConditionMap(String queryCondition) throws Exception {
        Map<String,String> map = new HashMap<>();
        ObjectMapper mapper = new ObjectMapper();
        JavaType javaType = mapper.getTypeFactory().constructParametricType(List.class, Map.class);
        List<Map<String, Object>> list = objectMapper.readValue(queryCondition, javaType);
        if (list != null && list.size() > 0) {
            for (Map<String, Object> item : list) {
                String andOr = String.valueOf(item.get("andOr")).trim();
                String field = String.valueOf(item.get("field")).trim();
                if(field.contains("-")){
                    field = field.substring(field.lastIndexOf("-")+1);
                }
                String cond = String.valueOf(item.get("condition")).trim();
                String value = String.valueOf(item.get("value"));
                map.put(field,value);
            }
        }
        return map;
    }

    /**
     * 新增参数
     *
     * @param oldParams
     * @param key
     * @param value
     * @return
     */
    private String addParams(String oldParams, String key, String value) {
        String newParam;
        if (value.startsWith("[") && value.endsWith("]")) {
            newParam = "\"" + key + "\":" + value;
        } else {
            newParam = "\"" + key + "\":\"" + value.replace("\"", "\\\"") + "\"";
        }
        if (oldParams != null && oldParams.length() > 3 && oldParams.startsWith("{") && oldParams.endsWith("}")) {
            return oldParams.substring(0, oldParams.length() - 1) + "," + newParam + "}";
        } else {
            return "{" + newParam + "}";
        }
    }


    /**
     * 获取档案包中数据集列表
     *
     * @param packId
     * @param version
     * @return
     * @throws Exception
     */
    public List<String> dataSetList(String packId, String version) throws Exception {
        Envelop envelop = new Envelop();
        List<String> dataSetCodes = new ArrayList<>();
        StringBuilder stringBuilder = new StringBuilder();
        if (org.apache.commons.lang.StringUtils.isNotEmpty(packId) && !"null".equals(packId)) {
            stringBuilder.append("pack_id=" + packId).append(";");
        }
        if (org.apache.commons.lang.StringUtils.isNotEmpty(version) && !"null".equals(version)) {
            stringBuilder.append("version=" + version);
        }

//        List<Map<String, Object>> list = elasticSearchUtil.list("json_archives_qc", "qc_dataset_info", stringBuilder.toString());
//        for (Map<String, Object> map : list) {
//            List<Map<String, Object>> dataSets = objectMapper.readValue(map.get("details").toString(), List.class);
//            dataSets.stream().forEach(dataSet -> {
//                String code = (String) dataSet.keySet().toArray()[0];
//                dataSetCodes.add(code);
//            });
//        }
        return dataSetCodes;
    }


    /**
     * 获取结果集
     *
     * @param dataSets 数据集编码列表
     * @param roleId   角色ID
     * @param orgCode  机构编码
     * @param areaCode 区域编码
     * @param rowKey   查询条件
     * @return
     * @throws Exception
     */
//    public Envelop getResultDataList(String version, List<String> dataSets, String roleId, String orgCode, String areaCode, String rowKey) throws Exception {
//        Map<String, Object> resultMap = new HashMap<>();
//        List<Map<String, Object>> resultData = null;
//        for (String resourcesCode : dataSets) {
//            Boolean isMultiRecord = redisService.getDataSetMultiRecord(version, resourcesCode);
//            RsResource rsResources = rsResourceService.getResourceByCategory(resourcesCode,"standard");
//            if (rsResources != null) {
//                StringBuilder saas = new StringBuilder();
//                if ("*".equals(orgCode) && "*".equals(areaCode)) {
//                    saas.append("*");
//                } else {
//                    List<String> orgCodeList = objectMapper.readValue(orgCode, List.class);
//                    List<String> areaCodeList = objectMapper.readValue(areaCode, List.class);
//                    if (orgCodeList != null && orgCodeList.size() > 0) {
//                        orgCodeList.forEach(item -> {
//                            if (saas.length() <= 0) {
//                                saas.append("(org_code:" + item);
//                            } else {
//                                saas.append(" OR org_code:" + item);
//                            }
//                        });
//                    }
//                    if (areaCodeList != null && areaCodeList.size() > 0) {
//                        areaCodeList.forEach(item -> {
//                            if (saas.length() <= 0) {
//                                saas.append("(org_area:" + item);
//                            } else {
//                                saas.append(" OR org_area:" + item);
//                            }
//                        });
//                    }
//                    if (saas.length() > 0) {
//                        saas.append(")");
//                    } else {
//                        throw new ApiException( "无SAAS权限访问资源",ErrorCode.FORBIDDEN.value());
//                    }
//                }
//                if (isMultiRecord) {
//                    //细表
//                    resultData = resourceBrowseDao.getEhrCenterSubByScan(resourcesCode, roleId, saas.toString(), rowKey);
//                } else {
//                    //主表
//                    resultData = resourceBrowseDao.getEhrCenterByScan(resourcesCode, roleId, saas.toString(), rowKey);
//                }
//                if (resultData != null && resultData.size() > 0) {
//                    resultMap.put(resourcesCode, resultData);
//                }
//            } else {
////                throw new ApiException(ErrorCode.OBJECT_NOT_FOUND, "无相关资源");
//                continue;
//            }
//
//        }
//
//        ObjEnvelop envelop = new ObjEnvelop();
//        envelop.setStatus(200);
//        envelop.setObj(resultMap);
//        return envelop;
//    }

    /**
     * 获取结果集 (通过数据集编码)
     *
     * @param dataSets 数据集编码列表
     * @param roleId   角色ID
     * @param orgCode  机构编码
     * @param areaCode 区域编码
     * @param rowKey   查询条件
     * @return
     * @throws Exception
     */
//    public Envelop getResultByDataSets(String version, List<String> dataSets, String roleId, String orgCode, String areaCode, String rowKey) throws Exception {
//        Map<String, Object> resultMap = new HashMap<>();
//        List<Map<String, Object>> resultData = null;
//        List<String> masterDatas = new ArrayList<>();
//        List<String> subDatas = new ArrayList<>();
//
//        StringBuilder saas = new StringBuilder();
//        if ("*".equals(orgCode) && "*".equals(areaCode)) {
//            saas.append("*");
//        } else {
//            List<String> orgCodeList = objectMapper.readValue(orgCode, List.class);
//            List<String> areaCodeList = objectMapper.readValue(areaCode, List.class);
//            if (orgCodeList != null && orgCodeList.size() > 0) {
//                orgCodeList.forEach(item -> {
//                    if (saas.length() <= 0) {
//                        saas.append("(org_code:" + item);
//                    } else {
//                        saas.append(" OR org_code:" + item);
//                    }
//                });
//            }
//            if (areaCodeList != null && areaCodeList.size() > 0) {
//                areaCodeList.forEach(item -> {
//                    if (saas.length() <= 0) {
//                        saas.append("(org_area:" + item);
//                    } else {
//                        saas.append(" OR org_area:" + item);
//                    }
//                });
//            }
//            if (saas.length() > 0) {
//                saas.append(")");
//            } else {
//                throw new ApiException( "无SAAS权限访问资源",ErrorCode.FORBIDDEN.value());
//            }
//        }
//
//        //分离查询，主细表分开查询
//        for (String resourcesCode : dataSets) {
//            Boolean isMultiRecord = redisService.getDataSetMultiRecord(version, resourcesCode);
//            RsResource rsResources = rsResourceService.getResourceByCategory(resourcesCode,"standard");
//            if (rsResources != null) {
//                if (isMultiRecord) {
//                    subDatas.add(resourcesCode);
//                } else {
//                    masterDatas.add(resourcesCode);
//                }
//            }
//        }
//
//        //主表数据
//        for (String resourcesCode : masterDatas) {
//            //主表
//            resultData = resourceBrowseDao.getEhrCenterByScan(resourcesCode, roleId, saas.toString(), rowKey);
//            if (resultData != null && resultData.size() > 0) {
//                resultMap.put(resourcesCode, resultData);
//            }
//        }
//
//        //细表查询
//        Map<String, Object> subMap = resourceBrowseDao.getEhrCenterSubMapByScan(subDatas, roleId, saas.toString(), rowKey);
//        resultMap.putAll(subMap);
//
//        ObjEnvelop envelop = new ObjEnvelop();
//        envelop.setStatus(200);
//        envelop.setObj(resultMap);
//        return envelop;
//    }

}


