package com.yihu.jw.basic.standard.controller;

import com.yihu.jw.basic.standard.service.adapter.AdapterDatasetService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.io.IOException;

/**
 * Created by LLH on 2016/1/6.
 */
@RestController("AdapterDatasetController")
@RequestMapping("/adapterCenter")
@Api(protocols = "https", value = "AdapterDatasetController",  tags = {"适配数据集"})
public class AdapterDatasetController extends EnvelopRestEndpoint {

    @Resource(name = AdapterDatasetService.BEAN_ID)
    private AdapterDatasetService datasetService;

    @GetMapping(value = "/searchDatasetList")
    @ApiOperation(value = "获取数据集列表", produces = "application/json", notes = "获取数据集列表")
    public Envelop getDatasetList(
            @ApiParam(name = "adapterVersion", value = "适配方案版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "condition", value = "查询条件：编码或名称")
            @RequestParam(value = "condition", required = false) String condition,
            @ApiParam(name = "order", value = "Order param,Must be json.ascending:asc,descending:desc")
            @RequestParam(value = "order", required = false) String order,
            @ApiParam(name = "rows", value = "Limit the size of result set. Must be an integer")
            @RequestParam(value = "rows", required = false) Integer rows,
            @ApiParam(name = "page", value = "Start position of result set. Must be an integer")
            @RequestParam(value = "page", required = false) Integer page) {

        return datasetService.getDetailModelResult(adapterVersion, condition, order, rows, page);
    }

    @GetMapping("/getForDatasetId")
    @ApiOperation(value = "获取适配数据集", produces = "application/json", notes = "获取适配数据集")
    public Envelop getForDatasetId(
            @ApiParam(name = "adapterVersion", value = "适配版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "datasetId", value = "适配数据集ID")
            @RequestParam(value = "datasetId") Integer datasetId) {

        return datasetService.get(adapterVersion, datasetId);
    }

    @PostMapping(value = "/delDataset")
    @ApiOperation(value = "删除数据集",  produces = "application/json", notes = "删除适配数据集")
    public Envelop delDataset(
            @ApiParam(name = "adapterVersion", value = "适配版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "datasetId", value = "适配数据集ID")
            @RequestParam(value = "datasetId") Integer datasetId) {
       return datasetService.delete(adapterVersion, datasetId);
    }

    @PostMapping(value = "/updateDataset")
    @ApiOperation(value = "修改适配数据集", produces = "application/json", notes = "修改适配数据集")
    public Envelop modify(
            @ApiParam(name = "adapterVersion", value = "适配版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "dataset", value = "数据集,JSON")
            @RequestParam(value = "dataset") String dataset) {
        return datasetService.modify(adapterVersion, dataset);
    }

    @PostMapping(value = "/addDataset")
    @ApiOperation(value = "保存适配数据集", produces = "application/json", notes = "保存数据集")
    public Envelop add(
            @ApiParam(name = "adapterVersion", value = "适配版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "dataset", value = "数据集信息")
            @RequestParam(value = "dataset") String dataset) {
        return datasetService.add(adapterVersion, dataset);
    }

    @GetMapping(value = "/getAdapterDataset")
    @ApiOperation(value = "获取定制数据集列表",  responseContainer = "List",produces = "application/json", notes = "获取数据集列表")
    public Envelop getDatasetCustomize(
            @ApiParam(name = "adapterVersion", value = "适配版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "stdVersion", value = "标准版本号")
            @RequestParam(value = "stdVersion") String stdVersion) {
        return datasetService.getDatasetCustomize(adapterVersion, stdVersion);
    }

    @RequestMapping(value = "/saveAdapterDataset", method = RequestMethod.POST)
    @ApiOperation(value = "定制适配数据集",  responseContainer = "List",produces = "application/json", notes = "获取数据集列表")
    public Envelop adapterDataSet(
            @ApiParam(name = "schemeId", value = "适配方案ID")
            @RequestParam(value = "schemeId") Long schemeId,
            @ApiParam(name = "adapterVersion", value = "适配版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "stdVersion", value = "标准版本号")
            @RequestParam(value = "stdVersion") String stdVersion,
            @ApiParam(name = "datasetIdStr", value = "适配数据集信息")
            @RequestParam(value = "datasetIdStr") String datasetIdStr,
            @ApiParam(name = "metaDataIdStr", value = "适配数据元信息")
            @RequestParam(value = "metaDataIdStr") String metaDataIdStr){
        return datasetService.updateCustomize(schemeId, adapterVersion, stdVersion, datasetIdStr,metaDataIdStr);
    }

    @RequestMapping(value = "/cloneDataSet", method = RequestMethod.GET)
    @ApiOperation(value = "克隆一个数据集", response = Envelop.class,produces = "application/json")
    public Envelop cloneDataSet(@ApiParam(name = "adapterVersion", value = "适配版本号")
                                @RequestParam(value = "adapterVersion") String adapterVersion,
                                @ApiParam(name = "code", value = "adapter_dataset_* 的std_dataset_code")
                                @RequestParam(value = "code") String code){
        try {
            return datasetService.cloneDataSet(adapterVersion, code);
        } catch (IOException e) {
            e.printStackTrace();
            return Envelop.getError("克隆数据集失败");
        }
    }

    @RequestMapping(value = "/deleteCloneDataSet", method = RequestMethod.DELETE)
    @ApiOperation(value = "删除克隆的数据集", response = Envelop.class,produces = "application/json")
    public Envelop deleteCloneDataSet(@ApiParam(name = "adapterVersion", value = "适配版本号")
                                      @RequestParam(value = "adapterVersion") String adapterVersion,
                                      @ApiParam(name = "id", value = "adapter_dataset_* 的id")
                                      @RequestParam(value = "id") Long id){
        return datasetService.deleteCloneDataSet(adapterVersion, id);
    }

    @RequestMapping(value = "/isNeedRelation", method = RequestMethod.GET,produces="application/json;charset=UTF-8")
    @ApiOperation(value = "查询是否需要关联关系", response = Envelop.class,produces = "application/json", notes = "查询是否需要关联关系")
    public Envelop isNeedRelation(@ApiParam(name = "adapterVersion", value = "适配版本号")
                                  @RequestParam(value = "adapterVersion") String adapterVersion,
                                  @ApiParam(name = "id", value = "adapter_dataset_* 的id")
                                  @RequestParam(value = "id") Long id){
        return datasetService.isNeedRelation(adapterVersion, id);
    }

    @RequestMapping(value = "/getDataSets", method = RequestMethod.GET,produces="application/json;charset=UTF-8")
    @ApiOperation(value = "查询关联关系的dataset列表", response = Envelop.class,produces = "application/json", notes = "查询是否需要关联关系")
    public Envelop getDataSets(@ApiParam(name = "adapterVersion", value = "适配版本号")
                               @RequestParam(value = "adapterVersion") String adapterVersion,
                               @ApiParam(name = "id", value = "adapter_dataset_* 的id")
                               @RequestParam(value = "id") Long id,
                               @ApiParam(name = "sourceVersion", value = "源标准version")
                               @RequestParam(value = "sourceVersion") String sourceVersion,
                               @ApiParam(name = "page", value = "页码")
                               @RequestParam(value = "page",required = false) Integer page,
                               @ApiParam(name = "rows", value = "每页条数")
                               @RequestParam(value = "rows",required = false) Integer rows,
                               @ApiParam(name = "filterCodeOrName", value = "根据code或者name条件查询")
                               @RequestParam(value = "filterCodeOrName",required = false) String filterCodeOrName) throws IOException {
        return datasetService.getDataSets(adapterVersion, id,sourceVersion,page,rows,filterCodeOrName);
    }

    /**
     * 单个数据集智能匹配
     */
    @RequestMapping(value = "/smartMatchMedata", method = RequestMethod.GET,produces = "application/json")
    @ApiOperation(value = "单个字典智能匹配", response = Envelop.class, produces = "application/json", notes = "单个字典智能匹配")
    public Envelop smartMatchMedata(
            @ApiParam(value = "源version", required = true) @RequestParam("std_version") String std_version,
            @ApiParam(value = "适配方案版本号", required = true) @RequestParam("adapter_version") String adapter_version,
            @ApiParam(value = "适配表的id(目标的id)", required = true) @RequestParam("id") Long id,
            @ApiParam(value = "适配表的adapterDataSetId(源的stdDatasetId)", required = true) @RequestParam("adapterDataSetId") Long adapterDataSetId) throws Exception {
        return datasetService.smartMatchMedata(std_version,adapter_version,id,adapterDataSetId);
    }
}
