package com.yihu.jw.basic.standard.controller;

import com.yihu.jw.basic.standard.match.dictionary.DictItemStrategyExecute;
import com.yihu.jw.basic.standard.service.adapter.AdapterDictService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.util.thread.ContextAttributes;
import com.yihu.jw.util.thread.LocalContext;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

/**
 * Created by LLH on 2016/1/6.
 */
@RestController("AdapterDictController")
@RequestMapping("/adapterCenter")
@Api(protocols = "https", value = "AdapterDictController", tags = {"适配字典"})
public class AdapterDictController extends EnvelopRestEndpoint {

    @Resource(name = AdapterDictService.BEAN_ID)
    private AdapterDictService dictService;

    @GetMapping(value = "/searchDictList")
    @ApiOperation(value = "获取字典列表", produces = "application/json", notes = "获取字典列表")
    public Envelop searchDictList(
            @ApiParam(name = "adapterVersion", value = "适配标准版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "condition", value = "查询条件：编码或名称")
            @RequestParam(value = "condition", required = false) String condition,
            @ApiParam(name = "order", value = "Order param,Must be json.ascending:asc,descending:desc")
            @RequestParam(value = "order", required = false) String order,
            @ApiParam(name = "rows", value = "Limit the size of result set. Must be an integer")
            @RequestParam(value = "rows", required = false) Integer rows,
            @ApiParam(name = "page", value = "Start position of result set. Must be an integer")
            @RequestParam(value = "page", required = false) Integer page) {
        return dictService.getDetailResult(adapterVersion, condition, order, rows, page);
    }

    @GetMapping(value = "/getForDictId")
    @ApiOperation(value = "获取字典", produces = "application/json", notes = "获取字典")
    public Envelop get(
            @ApiParam(name = "adapterVersion", value = "标准版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "dictId", value = "字典ID")
            @RequestParam(value = "dictId") Integer dictId) {
        return dictService.get(adapterVersion, dictId);
    }

    @PostMapping(value = "/delDict")
    @ApiOperation(value = "删除字典", produces = "application/json", notes = "删除字典")
    public Envelop delDict(
            @ApiParam(name = "adapterVersion", value = "标准版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "dictId", value = "字典ID")
            @RequestParam(value = "dictId") Integer dictId) {
        return dictService.delete(adapterVersion, dictId);
    }
    
    @PostMapping(value = "/updateDict")
    @ApiOperation(value = "修改字典", produces = "application/json", notes = "修改字典")
    public Envelop updateDict(
            @ApiParam(name = "adapterVersion", value = "标准版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "dict", value = "字典")
            @RequestParam(value = "dict") String dict) {
        return dictService.modify(adapterVersion, dict);
    }

    @PostMapping(value = "/addDict")
    @ApiOperation(value = "保存字典", produces = "application/json", notes = "保存字典")
    public Envelop addDict(
            @ApiParam(name = "adapterVersion", value = "标准版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "dict", value = "字典")
            @RequestParam(value = "dict") String dict) {
        return dictService.add(adapterVersion, dict);
    }
    @RequestMapping(value = "/isStrategyDic", method = RequestMethod.POST , produces = "application/json")
    @ApiOperation(value = "是否智能匹配",  produces = "application/json", notes = "是否智能匹配")
    public Envelop isStrategy(
            @ApiParam(value = "匹配version", required = true) @RequestParam("version") String version
    ) {
        try {
            return ObjEnvelop.getSuccess("查询成功","{\"isAdapt\":\"" + DictItemStrategyExecute.isAdapt(version) + "\"}");
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError("查询失败",-1);
        }
    }


    @RequestMapping(value = "/strategyDic", method = RequestMethod.POST)
    @ApiOperation(value = "智能匹配", produces = "application/json", notes = "智能匹配")
    public Envelop strategyDic(
            @ApiParam(value = "匹配version", required = true) @RequestParam("version") String version,
            @ApiParam(value = "平台标准version", required = true) @RequestParam("std_version") String std_version,
            @ApiParam(value = "适配方案版本号", required = true) @RequestParam("adapter_std_version") String adapter_std_version
    ) {
        try {
            synchronized (DictItemStrategyExecute.class){
                if (!DictItemStrategyExecute.isAdapt(version)) {
                    //判断之前是否已经有匹配有的话不进行匹配
                    DictItemStrategyExecute.setVersionYes(version);
                }else{
                    return Envelop.getSuccess("操作成功");
                }
            }
            String schemaName = LocalContext.getContext().getAttachment(ContextAttributes.SCHEMA);
            dictService.strategy(schemaName,version,std_version,adapter_std_version);
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("操作失败",-1);
        }
        return Envelop.getSuccess("操作成功");
    }


    @RequestMapping(value = "/getStrategyDicSize", method = RequestMethod.POST)
    @ApiOperation(value = "智能匹配进行",  produces = "application/json", notes = "智能匹配进行")
    public Envelop getStrategyDicSize(
            @ApiParam(value = "匹配version", required = true) @RequestParam("version") String version
    ){
        try {
            return ObjEnvelop.getSuccess("操作成功",DictItemStrategyExecute.getAllNumAndAdaptNum(version));
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("操作失败",-1);
        }
    }

    @RequestMapping(value = "/endStrategyDic", method = RequestMethod.POST)
    @ApiOperation(value = "智能匹配結束", produces = "application/json", notes = "智能匹配結束")
    public Envelop endStrategyDic(
            @ApiParam(value = "匹配version", required = true) @RequestParam("version") String version
    ){
        try {
            DictItemStrategyExecute.removeAdapt(version);
            return Envelop.getSuccess("操作成功");
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("操作失败",-1);
        }
    }

    /**
     * 单个字典智能匹配
     */
    @ResponseBody
    @RequestMapping(value = "/smartMatch", method = RequestMethod.GET,produces = "application/json")
    @ApiOperation(value = "单个字典智能匹配", response = Envelop.class, produces = "application/json", notes = "单个字典智能匹配")
    public Envelop smartMatch(
            @ApiParam(value = "源version", required = true) @RequestParam("std_version") String std_version,
            @ApiParam(value = "适配方案版本号", required = true) @RequestParam("adapter_version") String adapter_version,
            @ApiParam(value = "适配表的stdDictId(目标的stdDictId)", required = true) @RequestParam("stdDictId") Long stdDictId,
            @ApiParam(value = "适配表的adapterDictId(源的stdDictId)", required = true) @RequestParam("adapterDictId") Long adapterDictId) throws Exception {
        return dictService.smartMatch(std_version,adapter_version,stdDictId,adapterDictId);
    }

}
