package com.yihu.jw.basic.standard.controller;

import com.yihu.jw.basic.standard.service.adapter.AdapterDictEntryService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.io.IOException;

/**
 * Created by LLH on 2016/1/6.
 */
@RestController("AdapterDictEntryController")
@RequestMapping("/adapterCenter")
@Api(protocols = "https", value = "AdapterDictEntryController",  tags = {"适配字典项"})
public class AdapterDictEntryController extends EnvelopRestEndpoint {

    @Resource(name = AdapterDictEntryService.BEAN_ID)
    private AdapterDictEntryService dictentryService;

    @GetMapping(value = "/searchDictEntryList")
    @ApiOperation(value = "获取字典项列表",  produces = "application/json", notes = "获取字典项列表")
    public Envelop getDictEntryList(
            @ApiParam(name = "adapterVersion", value = "适配版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "condition", value = "查询条件")
            @RequestParam(value = "condition", required = false) String condition,
            @ApiParam(name = "order", value = "Order param,Must be json.ascending:asc,descending:desc")
            @RequestParam(value = "order", required = false) String order,
            @ApiParam(name = "rows", value = "Limit the size of result set. Must be an integer")
            @RequestParam(value = "rows", required = false) Integer rows,
            @ApiParam(name = "page", value = "Start position of result set. Must be an integer")
            @RequestParam(value = "page", required = false) Integer page){
        return dictentryService.getDetailResult(adapterVersion, condition, order, rows, page);
    }

    @GetMapping(value = "/getForDictEntryId")
    @ApiOperation(value = "获取字典项",  produces = "application/json", notes = "获取字典项")
    public Envelop get(
            @ApiParam(name = "adapterVersion", value = "标准版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "dictEntryId", value = "字典项ID")
            @RequestParam(value = "dictEntryId") Integer dictentryId) {
        return dictentryService.get(adapterVersion, dictentryId);
    }

    @PostMapping(value = "/updateDictEntry")
    @ApiOperation(value = "修改字典项", produces = "application/json", notes = "修改字典项")
    public Envelop updateDictEntry(
            @ApiParam(name = "adapterVersion", value = "标准版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "dictEntry", value = "字典项")
            @RequestParam(value = "dictEntry") String dictEntry) {
        return dictentryService.modify(adapterVersion, dictEntry);
    }

    @PostMapping(value = "/addDictEntry")
    @ApiOperation(value = "保存字典项", produces = "application/json", notes = "保存字典项")
    public Envelop add(
            @ApiParam(name = "adapterVersion", value = "标准版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "dictEntry", value = "字典项")
            @RequestParam(value = "dictEntry") String dictEntry) {
        return dictentryService.add(adapterVersion, dictEntry);
    }

    @PostMapping("/updateDictEntrys")
    @ApiOperation(value = "保存字典项",  produces = "application/json", notes = "保存数据元")
    public Envelop updateDictEntrys(
            @ApiParam(name = "adapterVersion", value = "标准版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "list", value = "字典项列表")
            @RequestParam(value = "list") String list) {
        return dictentryService.updateDictEntrys(adapterVersion, list);
    }

    @PostMapping("/delDictEntry")
    @ApiOperation(value = "删除字典项",  produces = "application/json", notes = "删除字典项,若字典项表为空管理字典清掉")
    public Envelop delete(
            @ApiParam(name = "adapterVersion", value = "标准版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "dictEntryId", value = "字典项ID")
            @RequestParam(value = "dictEntryId") Integer dictEntryId,
            @ApiParam(name = "stdDictId", value = "标准字典ID")
            @RequestParam(value = "stdDictId") Integer stdDictId,
            @ApiParam(name = "adapterDictId", value = "适配字典ID")
            @RequestParam(value = "adapterDictId") Integer adapterDictId) {
        return dictentryService.deleteMultiple(adapterVersion, dictEntryId, stdDictId, adapterDictId);
    }

    @RequestMapping(value = "/clearAdapt", method = RequestMethod.GET,produces="application/json;charset=UTF-8")
    @ApiOperation(value = "清除字典项适配", response = Envelop.class, produces = "application/json", notes = "清除字典项适配")
    public Envelop clearAdapt(
            @ApiParam(name = "adapterVersion", value = "标准版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "stdDictId", value = "标准字典ID")
            @RequestParam(value = "stdDictId") Long stdDictId) {
        return dictentryService.clearAdapt(adapterVersion, stdDictId);
    }

    @RequestMapping(value = "/cloneDictEntry", method = RequestMethod.GET,produces="application/json;charset=UTF-8")
    @ApiOperation(value = "克隆字典项", response = Envelop.class, produces = "application/json", notes = "克隆字典项")
    public Envelop cloneDictEntry(
            @ApiParam(name = "version", value = "版本号")
            @RequestParam(value = "version") String version,
            @ApiParam(name = "id", value = "adapter_dictentry_*的id")
            @RequestParam(value = "id") Long id) throws IOException {
        return dictentryService.cloneDictEntry(version, id);
    }

    @RequestMapping(value = "/deletecloneDictEntry", method = RequestMethod.DELETE,produces="application/json;charset=UTF-8")
    @ApiOperation(value = "克隆字典项", response = Envelop.class, produces = "application/json", notes = "克隆字典项")
    public Envelop deletecloneDictEntry(
            @ApiParam(name = "version", value = "版本号")
            @RequestParam(value = "version") String version,
            @ApiParam(name = "id", value = "adapter_dictentry_*的id")
            @RequestParam(value = "id") Long id) throws IOException {
        return dictentryService.deletecloneDictEntry(version, id);
    }
}
