package com.yihu.jw.basic.standard.controller;

import com.yihu.jw.basic.standard.match.metadata.MetadataStrategyExecute;
import com.yihu.jw.basic.standard.service.adapter.AdapterDatasetService;
import com.yihu.jw.basic.standard.service.adapter.AdapterMetadataService;
import com.yihu.jw.basic.standard.service.bo.AdapterVersion;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.util.thread.ContextAttributes;
import com.yihu.jw.util.thread.LocalContext;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

/**
 * Created by LLH on 2016/1/6.
 */
@RestController("AdapterMetadataController")
@RequestMapping("/adapterCenter")
@Api(protocols = "https", value = "AdapterMetadataController",  tags = {"适配字典"})
public class AdapterMetadataController extends EnvelopRestEndpoint {
    @Resource(name = AdapterMetadataService.BEAN_ID)
    private AdapterMetadataService metadataService;
    @Resource(name = AdapterDatasetService.BEAN_ID)
    private AdapterDatasetService adapterDatasetService;

    @GetMapping("/searchMetadataList")
    @ApiOperation(value = "查询数据元列表信息",  produces = "application/json", notes = "获取数据元列表")
    public Envelop getMetadataList(
            @ApiParam(name = "adapterVersion", value = "标准版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "condition", value = "查询条件")
            @RequestParam(value = "condition", required = false) String condition,
            @ApiParam(name = "order", value = "Order param,Must be json.ascending:asc,descending:desc")
            @RequestParam(value = "order", required = false) String order,
            @ApiParam(name = "rows", value = "Limit the size of result set. Must be an integer")
            @RequestParam(value = "rows", required = false) Integer rows,
            @ApiParam(name = "page", value = "Start position of result set. Must be an integer")
            @RequestParam(value = "page", required = false) Integer page)  throws Exception{
        return  metadataService.getDetailResult(adapterVersion, condition, order, rows, page);
    }

    @GetMapping("/getForMetadataId")
    @ApiOperation(value = "获取数据元", produces = "application/json", notes = "获取数据元")
    public Envelop get(
            @ApiParam(name = "adapterVersion", value = "标准版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "metadataId", value = "数据元ID")
            @RequestParam(value = "metadataId") Integer metadataId) {
        AdapterVersion version = new AdapterVersion(adapterVersion);
        return metadataService.get(version.getMetaDataTableName(), metadataId);
    }

    @PostMapping("/updateMetadata")
    @ApiOperation(value = "修改数据元",  produces = "application/json", notes = "修改数据元")
    public Envelop updateMetadata(
            @ApiParam(name = "adapterVersion", value = "标准版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "metadata", value = "数据元")
            @RequestParam(value = "metadata") String metadata) throws Exception {
        return metadataService.modify(adapterVersion, metadata);
    }

    @PostMapping("/addMetadata")
    @ApiOperation(value = "保存数据元",  produces = "application/json", notes = "保存数据元")
    public Envelop addMetadata(
            @ApiParam(name = "adapterVersion", value = "标准版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "metadata", value = "数据元")
            @RequestParam(value = "metadata") String metadata) throws Exception {
        return metadataService.add(adapterVersion, metadata);
    }

    @PostMapping("/updateMetadatas")
    @ApiOperation(value = "修改数据元",  produces = "application/json", notes = "保存数据元")
    public Envelop updateMetadatas(
            @ApiParam(name = "adapterVersion", value = "标准版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "list", value = "数据元列表")
            @RequestParam(value = "list") String list,String stdVersion) {
        return metadataService.updateMetadatas(adapterVersion,stdVersion, list);
    }

    @PostMapping("/delMetadata")
    @ApiOperation(value = "删除数据元",  produces = "application/json", notes = "删除数据元")
    public Envelop delete(
            @ApiParam(name = "adapterVersion", value = "标准版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "metadataId", value = "数据元ID")
            @RequestParam(value = "metadataId") Integer metadataId,
            @ApiParam(name = "stdDatasetId", value = "标准数据集ID")
            @RequestParam(value = "stdDatasetId") Integer stdDatasetId,
            @ApiParam(name = "adapterDatasetId", value = "适配数据集ID")
            @RequestParam(value = "adapterDatasetId") Integer adapterDatasetId) {
        return metadataService.deleteMultiple(adapterVersion, metadataId, stdDatasetId, adapterDatasetId);
    }

    @RequestMapping(value = "/isStrategy", method = RequestMethod.POST , produces = "application/json")
    @ApiOperation(value = "是否智能匹配", produces = "application/json", notes = "是否智能匹配")
    public Envelop isStrategy(
            @ApiParam(value = "匹配version", required = true) @RequestParam("version") String version
    ){
        try {
            return  ObjEnvelop.getSuccess("操作成功","{\"isAdapt\":\"" + MetadataStrategyExecute.isAdapt(version) + "\"}");
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("操作失败",-1);
        }
    }

    @RequestMapping(value = "/strategy", method = RequestMethod.POST)
    @ApiOperation(value = "智能匹配", response = Object.class, produces = "application/json", notes = "智能匹配")
    public Envelop strategy(
            @ApiParam(value = "匹配version", required = true) @RequestParam("version") String version,
            @ApiParam(value = "平台标准version", required = true) @RequestParam("std_version") String std_version,
            @ApiParam(value = "适配方案版本号", required = true) @RequestParam("adapter_std_version") String adapter_std_version
    ) throws Exception {
        try {
            //判断之前是否已经在匹配,有的话不进行匹配
            synchronized (MetadataStrategyExecute.class) {
                if (!MetadataStrategyExecute.isAdapt(version)) {
                    //判断之前是否已经有匹配有的话不进行匹配
                    MetadataStrategyExecute.setVersionYes(version);
                } else {
                    return Envelop.getSuccess("操作成功");
                }
            }

            String schemaName = LocalContext.getContext().getAttachment(ContextAttributes.SCHEMA);
            adapterDatasetService.strategy(schemaName,version, std_version, adapter_std_version);
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("操作失败",-1);
        }
        return Envelop.getSuccess("操作成功");
    }

    @RequestMapping(value = "/getStrategySize", method = RequestMethod.POST)
    @ApiOperation(value = "智能匹配进行", produces = "application/json", notes = "智能匹配进行")
    public Envelop getStrategySize(
            @ApiParam(value = "匹配version", required = true) @RequestParam("version") String version
    ) {
        try {
            return  ObjEnvelop.getSuccess("操作成功",MetadataStrategyExecute.getAllNumAndAdaptNum(version));
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("操作失败",-1);
        }
    }

    @RequestMapping(value = "/endStrategy", method = RequestMethod.POST)
    @ApiOperation(value = "智能匹配結束",  produces = "application/json", notes = "智能匹配結束")
    public Envelop endStrategy(
            @ApiParam(value = "匹配version", required = true) @RequestParam("version") String version
    ) {
        try {
            MetadataStrategyExecute.removeAdapt(version);
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("操作失败",-1);
        }
        return Envelop.getSuccess("操作成功");
    }

    @RequestMapping(value = "/clearMetadaAdapt", method = RequestMethod.GET,produces="application/json;charset=UTF-8")
    @ApiOperation(value = "清除数据集数据元适配", response = Envelop.class, produces = "application/json")
    public Envelop clearAdapt(
            @ApiParam(name = "adapterVersion", value = "标准版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "stdDataSetId", value = "标准字典ID")
            @RequestParam(value = "stdDataSetId") Long stdDataSetId,
            @ApiParam(name = "belongAdapterId", value = "所属id")
            @RequestParam(value = "belongAdapterId") Long belongAdapterId) {
        return metadataService.clearAdapt(adapterVersion, stdDataSetId,belongAdapterId);
    }
}

