package com.yihu.jw.basic.standard.controller;

import com.google.common.base.Splitter;

import com.yihu.jw.basic.standard.model.qc.RuleModel;
import com.yihu.jw.basic.standard.service.qc.RuleService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.beanutils.BeanUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * 质控规则管理
 *
 * @author CD
 * Created at 2017/05/05.
 */
@RequestMapping("/rule")
@RestController
@Api( tags = {"质控规则管理--质控规则接口"})
public class RuleController extends EnvelopRestEndpoint {

    @Resource(name = RuleService.BEAN_ID)
    private RuleService ruleService;


    @ApiOperation(value = "新增质控规则信息")
    @RequestMapping("addRule")
    public Envelop addRule(HttpServletRequest request) {
        try {
            RuleModel obj = new RuleModel();
            BeanUtils.populate(obj, request.getParameterMap());
            return ruleService.addRule(obj);
        } catch (Exception ex) {
            ex.printStackTrace();
            return failed(ex.getMessage());
        }
    }

    /**
     * 删除质控规则信息
     */
    @RequestMapping("/deleteRule")
    public Envelop deleteRule(HttpServletRequest request) {

        try {
            String id = request.getParameter("id");

            ruleService.deleteRule(id);
            return success("删除成功！");
        } catch (Exception e) {
            e.printStackTrace();
            return failed("删除失败！");
        }
    }

    /**
     * 批量删除质控规则信息
     */
    @RequestMapping("/deleteRuleBatch")
    public Envelop deleteRuleBatch(HttpServletRequest request) {
        try {
            String idList = request.getParameter("idList");
            List<String> ids = Splitter.on(",").trimResults().splitToList(idList);
            if (!CollectionUtils.isEmpty(ids)) {
                for (String id :
                        ids) {
                    ruleService.deleteRule(id);
                }
            }
            return success("删除成功！");
        } catch (Exception e) {
            e.printStackTrace();
            return failed("删除失败！");
        }
    }

    /**
     * 修改质控规则信息
     */
    @RequestMapping("updateRule")
    public Envelop updateRule(HttpServletRequest request) {
        try {
            RuleModel obj = new RuleModel();
            BeanUtils.populate(obj, request.getParameterMap());

            return ruleService.updateRule(obj);
        } catch (Exception ex) {
            ex.printStackTrace();
            return failed(ex.getMessage());
        }
    }

    /**
     * 绑定质控规则
     */
    @ApiOperation(" 根据数集ID&数据元编码绑定质控规则")
    @RequestMapping(value = "bindQcRule", method = RequestMethod.POST)
    public Envelop bindQcRule(
            @RequestParam(name = "stdVersion") String stdVersion,
            @RequestParam(name = "stdDataSetId") Long stdDataSetId,
            @RequestParam(name = "dataElement") String dataElement,
            @RequestParam(name = "ruleList") String ruleList) {
        Envelop envelop = new Envelop();
        try {
            ruleService.bindQcRule(stdVersion,stdDataSetId,dataElement, ruleList);//绑定质控规则反射方法
            envelop.setStatus(200);
        } catch (Exception ex) {
            ex.printStackTrace();
            envelop.setStatus(-1);
            envelop.setMessage("失败");
        }
        return envelop;
    }

    /**
     * 绑定质控规则
     */
    @ApiOperation("根据数据元编码批量绑定质控规则")
    @RequestMapping(value = "bindQcRuleBatch", method = RequestMethod.POST)
    public Envelop bindQcRuleBatch(
            @RequestParam(name = "stdVersion") String stdVersion,
            @RequestParam(name = "dataElement") String dataElement,
            @RequestParam(name = "ruleList") String ruleList) {
        Envelop envelop = new Envelop();
        try {
            ruleService.bindQcRuleBatch(stdVersion,dataElement, ruleList);//绑定质控规则反射方法
            envelop.setStatus(200);
        } catch (Exception ex) {
            ex.printStackTrace();
            envelop.setStatus(-1);
            envelop.setMessage("失败");
        }
        return envelop;
    }


    /**
     * 绑定质控规则
     */
    @ApiOperation(" 根据数集编码&数据元编码绑定质控规则")
    @RequestMapping(value = "bindQcRuleByCode", method = RequestMethod.POST)
    public Envelop bindQcRule(
            @RequestParam(name = "stdVersion") String stdVersion,
            @RequestParam(name = "stdDataSetCode") String stdDataSetCode,
            @RequestParam(name = "dataElement") String dataElement,
            @RequestParam(name = "ruleList") String ruleList) {
        Envelop envelop = new Envelop();
        try {

            ruleService.bindQcRule(stdVersion,stdDataSetCode,dataElement, ruleList);
            envelop.setStatus(200);
        } catch (Exception ex) {
            ex.printStackTrace();
            envelop.setMessage("失败");
        }

        return envelop;
    }


    /**
     * 获取绑定质控规则
     */
    @RequestMapping(value = "bindQcRule", method = RequestMethod.GET)
    public Envelop getBindQcRule(
            @RequestParam(name = "stdVersion") String stdVersion,
            @RequestParam(name = "stdDataSetId") Long stdDataSetId,
            @RequestParam(name = "dataElement") String dataElement) {
        Envelop envelop = new Envelop();
        try {
            List<RuleModel> bindQcRule = ruleService.getBindQcRule(stdVersion,stdDataSetId,dataElement);
            return ListEnvelop.getSuccess("获取成功",bindQcRule);
        } catch (Exception ex) {
            ex.printStackTrace();
            envelop.setStatus(-1);
            envelop.setMessage("绑定制控规则");
        }
        return envelop;
    }


    /**
     * 获取绑定质控规则
     */
    @RequestMapping(value = "bindQcRuleByCode", method = RequestMethod.GET)
    public Envelop bindQcRuleByCode(
            @RequestParam(name = "stdVersion") String stdVersion,
            @RequestParam(name = "stdDataSetCode") String stdDataSetCode,
            @RequestParam(name = "dataElement") String dataElement) {
        Envelop envelop = new Envelop();
        try {
            List<RuleModel> bindQcRule = ruleService.getBindQcRule(stdVersion,stdDataSetCode,dataElement);
            return ListEnvelop.getSuccess("获取成功",bindQcRule);
        } catch (Exception ex) {
            ex.printStackTrace();
            envelop.setStatus(-1);
            envelop.setMessage("绑定制控规则");
        }
        return envelop;
    }


    @ApiOperation("根据数据集，绑定其所有数据元的质控规则")
    @RequestMapping(value = "bindQcRuleByDataSet", method = RequestMethod.POST)
    public Envelop bindQcRuleByDataSet(
            @ApiParam(name = "stdVersion", value = "标准版本号", required = true)
            @RequestParam(name = "stdVersion") String stdVersion,
            @ApiParam(name = "dataSetId", value = "数据集ID", required = true)
            @RequestParam(name = "dataSetId") Long dataSetId) {
        Envelop envelop = new Envelop();
        try {
            ruleService.bindQcRuleByDataSet(stdVersion, dataSetId);
            envelop.setStatus(200);
        } catch (Exception e) {
            e.printStackTrace();
            envelop.setStatus(-1);
            envelop.setMessage(e.getMessage());
        }
        return envelop;
    }

    @ApiOperation("根据标准版本，绑定其所有数据元的质控规则")
    @RequestMapping(value = "bindQcRuleByStdVersion", method = RequestMethod.POST)
    public Envelop bindQcRuleByStdVersion(
            @ApiParam(name = "stdVersion", value = "标准版本", required = true)
            @RequestParam(name = "stdVersion") String stdVersion) {
        Envelop envelop = new Envelop();
        try {
            ruleService.batchBindRuleByStdVersion(stdVersion);
            envelop.setStatus(200);
            envelop.setMessage("标准版本："+stdVersion +"绑定规则成功");
        } catch (Exception e) {
            e.printStackTrace();
            envelop.setStatus(-1);
            envelop.setMessage(e.getMessage());
        }
        return envelop;
    }

    @ApiOperation("根据标准版本，删除其所有数据元的质控规则")
    @RequestMapping(value = "delQcRuleByStdVersion", method = RequestMethod.DELETE)
    public Envelop delQcRuleByStdVersion(
            @ApiParam(name = "stdVersion", value = "标准版本", required = true)
            @RequestParam(name = "stdVersion") String stdVersion) {
        Envelop envelop = new Envelop();
        try {
            ruleService.deleteQcMetaData(stdVersion);
            envelop.setStatus(200);
        } catch (Exception e) {
            e.printStackTrace();
            envelop.setStatus(-1);
            envelop.setMessage("删除失败");
        }
        return envelop;
    }


    @ApiOperation("获取数据元绑定质控规则(返回单个数据元）")
    @RequestMapping(value = "metaDataRule", method = RequestMethod.GET)
    public Envelop metaDataRule(
            @RequestParam(name = "stdVersion") String stdVersion,
            @RequestParam(name = "stdDataSetCode") String stdDataSetCode,
            @RequestParam(name = "metaDataCode") String metaDataCode) {
        Envelop envelop = new Envelop();
        try {
            String bindQcRule = ruleService.getQcRule(stdVersion,stdDataSetCode,metaDataCode);
            return ObjEnvelop.getSuccess("获取成功",bindQcRule);
        } catch (Exception ex) {
            ex.printStackTrace();
            envelop.setStatus(-1);
            envelop.setMessage("获取数据元质控绑定规则异常");
            return envelop;
        }

    }


    @ApiOperation("获取数据集下有绑定质控规则的数据元")
    @RequestMapping(value = "metaDatas", method = RequestMethod.GET)
    public Envelop getMetaDatasByDataSet(
            @RequestParam(name = "stdVersion") String stdVersion,
            @RequestParam(name = "stdDataSetCode") String stdDataSetCode) {
        Envelop envelop = new Envelop();
        try {
            List<String> metaDatas = ruleService.getMetaDataByDataSet(stdVersion,stdDataSetCode);
            return ListEnvelop.getSuccess("获取成功",metaDatas);
        } catch (Exception ex) {
            ex.printStackTrace();
            envelop.setStatus(-1);
            envelop.setMessage("获取数据元质控绑定规则异常");
            return envelop;
        }
    }
}
