package com.yihu.jw.basic.standard.controller;

import com.yihu.jw.basic.standard.service.adapter.AdapterSchemeService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

@RestController("SchemeController")
@RequestMapping("/adapterCenter")
@Api(protocols = "https", value = "SchemeController",  tags = {"适配方案"})
public class SchemeController extends EnvelopRestEndpoint {
    @Resource(name = AdapterSchemeService.BEAN_ID)
    private AdapterSchemeService adapterSchemeService;

    @GetMapping(value = "/searchScheme")
    @ApiOperation(value = "获取方案列表",  responseContainer = "List", notes = "获取方案列表")
    public ListEnvelop searchScheme(
            @ApiParam(name = "condition", value = "查询条件：适配方案名称、适配方案版本名称")
            @RequestParam(value = "condition", required = false) String condition) {
        return adapterSchemeService.getSchemeList(condition);
    }

    /**
     * 根据适配ID获取适配方案信息（get）
     * @param schemeId
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/getForSchemeId")
    @ApiOperation(value = "获取适配方案", produces = "application/json", notes = "获取适配方案")
    public ObjEnvelop get(
            @ApiParam(name = "schemeId", value = "标准版本ID")
            @RequestParam(value = "schemeId") Long schemeId) {
        return adapterSchemeService.get(schemeId);
    }


    @PostMapping("/updateScheme")
    @ApiOperation(value = "修改适配方案",  produces = "application/json", notes = "修改适配方案")
    public ObjEnvelop modify(
            @ApiParam(name = "scheme", value = "适配方案")
            @RequestParam(value = "scheme") String scheme) {
        return adapterSchemeService.update(scheme);
    }


    @PostMapping(value = "/addScheme")
    @ApiOperation(value = "保存适配方案",  produces = "application/json", notes = "保存适配方案")
    public ObjEnvelop add(
            @ApiParam(name = "scheme", value = "适配方案信息")
            @RequestParam(value = "scheme") String scheme) {
        return adapterSchemeService.add(scheme);
    }


    @PostMapping(value = "/deleteScheme")
    @ApiOperation(value = "删除适配方案",  produces = "application/json", notes = "删除适配方案")
    public Envelop delete(
            @ApiParam(name = "schemeId", value = "适配方案ID")
            @RequestParam(value = "schemeId") Long schemeId) {
        return adapterSchemeService.delete(schemeId);
    }
}
