package com.yihu.jw.basic.standard.controller;

import com.yihu.jw.basic.standard.service.adapter.AdapterSchemeVersionService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

@RestController("SchemeVersionController")
@RequestMapping("/adapterCenter")
@Api(protocols = "https", value = "SchemeVersionController", tags = {"适配版本"})
public class SchemeVersionController extends EnvelopRestEndpoint {
    @Resource(name = AdapterSchemeVersionService.BEAN_ID)
    private AdapterSchemeVersionService adapterSchemeVersion;

    @GetMapping("/getForVersionId")
    @ApiOperation(value = "根据版本id获取适配方案信息",  produces = "application/json", notes = "获取适配版本")
    public ObjEnvelop get(
            @ApiParam(name = "versionId", value = "适配版本ID")
            @RequestParam(value = "versionId") Long versionId) {
        return adapterSchemeVersion.get(versionId);
    }


    @PostMapping("/addVersion")
    @ApiOperation(value = "保存适配版本", produces = "application/json", notes = "保存适配版本")
    public ObjEnvelop add(
            @ApiParam(name = "version", value = "适配版本信息")
            @RequestParam(value = "version") String version) {
        return adapterSchemeVersion.add(version);
    }


    @PostMapping("/updateVersion")
    @ApiOperation(value = "修改适配版本",  produces = "application/json", notes = "修改适配版本")
    public ObjEnvelop modify(
            @ApiParam(name = "version", value = "标准版本")
            @RequestParam(value = "version") String version) {
        return adapterSchemeVersion.update(version);
    }


    @PostMapping("/publishVersion")
    @ApiOperation(value = "发布适配版本",  produces = "application/json", notes = "发布适配版本")
    public ObjEnvelop publish(
            @ApiParam(name = "versionId", value = "适配版本ID")
            @RequestParam(value = "versionId") Long versionId,
            @ApiParam(name = "publisher", value = "发布者")
            @RequestParam(value = "publisher", required = false) String publisher) {
        return adapterSchemeVersion.publish(versionId, publisher);
    }


    @PostMapping("/deleteVersion")
    @ApiOperation(value = "删除版本",  produces = "application/json", notes = "删除版本")
    public Envelop delete(
            @ApiParam(name = "versionId", value = "标准版本ID")
            @RequestParam(value = "versionId") Long versionId) {
        return adapterSchemeVersion.delete(versionId);
    }

    @GetMapping("getVersion")
    public PageEnvelop getVersion() {
        try {
            return adapterSchemeVersion.getVersion();
        } catch (Exception e) {
            e.printStackTrace();
            return PageEnvelop.getError("获取失败");
        }
    }

    @GetMapping("getVersionTree")
    public ListEnvelop getVersionTree() {
        try {
            return ListEnvelop.getSuccess("获取成功",adapterSchemeVersion.getVersionTree());
        } catch (Exception e) {
            e.printStackTrace();
            return ListEnvelop.getError("获取失败");
        }
    }


    @GetMapping(value = "/getVersions")
    @ApiOperation(value = "获取适配方案版本列表", produces = "application/json", notes = "获取标准版本列表")
    public ListEnvelop getList(
            @ApiParam(name = "schemeId", value = "适配方案ID")
            @RequestParam(value = "schemeId") Long schemeId,
            @ApiParam(name = "condition", value = "Search param,Must be json")
            @RequestParam(value = "condition", required = false) String condition,
            @ApiParam(name = "order", value = "Order param,Must be json.ascending:asc,descending:desc")
            @RequestParam(value = "order", required = false) String order,
            @ApiParam(name = "rows", value = "Limit the size of result set. Must be an integer")
            @RequestParam(value = "rows", required = false) Integer rows,
            @ApiParam(name = "page", value = "Start position of result set. Must be an integer")
            @RequestParam(value = "page", required = false) Integer page) {
        return adapterSchemeVersion.getDictionaryResult(schemeId, condition, order, rows, page);
    }
}
