package com.yihu.jw.basic.standard.controller;

import com.yihu.jw.basic.standard.model.standard.resultModel.StandardParamResultModel;
import com.yihu.jw.basic.standard.service.standard.StandardService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.util.sql.SqlConstants;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

@RestController("StandardController")
@RequestMapping("/standardCenter")
@Api(protocols = "https", value = "StandardController", tags = {"标准管理"})
public class StandardController extends EnvelopRestEndpoint {
    @Resource(name = StandardService.BEAN_ID)
    private StandardService standardService;


    @GetMapping("/searchStandard")
    @ApiOperation(value = "获取标准树列表", responseContainer = "List", notes = "获取标准列表")
    public ListEnvelop searchStandard(
            @ApiParam(name = "condition", value = "查询条件name：厂商、标准、标准版本名称，标准类别type：0为集成标准，1为应用标准")
            @RequestParam(value = "condition", required = false) String condition) {
        return standardService.getDetailModelResult(condition);
    }


    @GetMapping("/getSysParamList")
    @ApiOperation(value = "获取集成标准系统参数", notes = "获取集成标准系统参数")
    public ObjEnvelop getSysParamList() {
        StandardParamResultModel standardParamResultModel = standardService.getStandardParam(SqlConstants.STANDARD);
        return ObjEnvelop.getSuccess("获取成功",standardParamResultModel);
    }


    @GetMapping("/getForStandardId")
    @ApiOperation(value = "根据标准ID获取集成标准信息",  produces = "application/json", notes = "获取标准")
    public ObjEnvelop getForStandardId(
            @ApiParam(name = "standardId", value = "标准ID")
            @RequestParam(value = "standardId") Long standardId){
        return standardService.get(standardId);
    }


    @PostMapping("/updateStandard")
    @ApiOperation(value = "修改标准",  produces = "application/json", notes = "修改标准")
    public ObjEnvelop modify(
            @ApiParam(name = "standard", value = "标准基本信息")
            @RequestParam(value = "standard") String standard) {
        return standardService.update(standard);
    }


    @PostMapping("/addStandard")
    @ApiOperation(value = "保存标准",  produces = "application/json", notes = "保存标准")
    public ObjEnvelop add(
            @ApiParam(name = "standard", value = "标准基本信息")
            @RequestParam(value = "standard") String standard) {
        return standardService.add(standard);
    }


    @PostMapping("/deleteStandard")
    @ApiOperation(value = "删除标准",  produces = "application/json", notes = "删除标准")
    public Envelop delete(
            @ApiParam(name = "standardId", value = "标准ID")
            @RequestParam(value = "standardId") Long standardId) {
        return standardService.delete(standardId);
    }


    @GetMapping(value = "/getStandards")
    @ApiOperation(value = "获取平台标准名称/应用标准名称列表",  produces = "application/json", notes = "获取标准列表")
    public ListEnvelop getStandardList(
            @ApiParam(name = "condition", value = "Must be Json,其中type为区分集成与应用标准")
            @RequestParam(value = "condition", required = false) String condition,
            @ApiParam(name = "order", value = "Must be Json")
            @RequestParam(value = "order", required = false) String order,
            @ApiParam(name = "rows", value = "Limit the size of result set. Must be an integer")
            @RequestParam(value = "rows", required = false) Integer rows,
            @ApiParam(name = "page", value = "Start position of result set. Must be an integer")
            @RequestParam(value = "page", required = false) Integer page) {
        return standardService.getDictionaryResult(condition, order, rows, page);
    }
}
