package com.yihu.jw.basic.standard.controller;

import com.yihu.jw.basic.standard.service.standard.StandardVersionService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;

/**
 * Created by LLH on 2016/1/6.
 */
@RestController("StandardVersionController")
@RequestMapping("/standardCenter")
@Api(protocols = "https", value = "StandardVersionController",  tags = {"标准版本管理"})
public class StandardVersionController extends EnvelopRestEndpoint {
    @Resource(name = StandardVersionService.BEAN_ID)
    private StandardVersionService standardVersionService;

    @GetMapping("/getForVersionId")
    @ApiOperation(value = "获取标准版本", produces = "application/json", notes = "获取标准版本")
    public ObjEnvelop getForVersionId(
            @ApiParam(name = "versionId", value = "标准版本ID")
            @RequestParam(value = "versionId") Long versionId) {
        return standardVersionService.get(versionId);
    }


    @PostMapping("/addVersion")
    @ApiOperation(value = "保存标准版本",  produces = "application/json", notes = "保存标准版本")
    public ObjEnvelop addVersion(
            @ApiParam(name = "version", value = "标准版本信息")
            @RequestParam(value = "version") String version) throws Exception {
        return standardVersionService.add(version);
    }


    @PostMapping("/updateVersion")
    @ApiOperation(value = "修改版本", produces = "application/json", notes = "修改版本")
    public ObjEnvelop updateVersion(
            @ApiParam(name = "version", value = "标准版本")
            @RequestParam(value = "version") String version) throws Exception {
        return standardVersionService.update(version);
    }


    @PostMapping("/publishVersion")
    @ApiOperation(value = "发布版本", produces = "application/json", notes = "发布版本")
    public ObjEnvelop publish(
            @ApiParam(name = "versionId", value = "标准版本ID")
            @RequestParam(value = "versionId") Long versionId,
            @ApiParam(name = "publisher", value = "发布者")
            @RequestParam(value = "publisher") String publisher) throws Exception {
        return standardVersionService.publish(versionId, publisher);
    }


    @PostMapping("/deleteVersion")
    @ApiOperation(value = "删除版本", produces = "application/json", notes = "删除版本")
    public Envelop delete(
            @ApiParam(name = "versionId", value = "标准版本ID")
            @RequestParam(value = "versionId") Long versionId,
            @ApiParam(name = "version", value = "标准版本值")
            @RequestParam(value = "version") String version) throws Exception {
        return standardVersionService.delete(versionId,version);
    }


    @GetMapping("/getVersions")
    @ApiOperation(value = "获取标准版本列表",  produces = "application/json", notes = "获取标准版本列表")
    public ListEnvelop getVersions(
            @ApiParam(name = "standardId", value = "标准ID")
            @RequestParam(value = "standardId") Long standardId,
            @ApiParam(name = "condition", value = "Must be Json")
            @RequestParam(value = "condition", required = false) String condition,
            @ApiParam(name = "order", value = "Must be Json")
            @RequestParam(value = "order", required = false) String order,
            @ApiParam(name = "rows", value = "Limit the size of result set. Must be an integer")
            @RequestParam(value = "rows", required = false) Integer rows,
            @ApiParam(name = "page", value = "Start position of result set. Must be an integer")
            @RequestParam(value = "page", required = false) Integer page) {
        return standardVersionService.getDetailResultModelList(standardId, condition, order, rows, page);
    }

    @RequestMapping(value = "/importFile", method = RequestMethod.POST)
    @ApiOperation(value = "根据type导入数据接口", response = Object.class, produces = "application/json", notes = "导入数据接口")
    public Envelop importFile(@ApiParam(value = "类型（1:数据集 2:字典 3:机构数据 4：机构字典）", required = true) @RequestParam("type") String type,
                             @ApiParam(value = "项目版本", required = true) @RequestParam("version") String version,
                             @ApiParam(value = "项目版本", required = true) @RequestParam("standardId") Long standardId,
                             @ApiParam(value = "文件", required = true, allowMultiple = true) @RequestParam("file") MultipartFile file){
        try {
            //根據类别判断上传的文件1:数据集 2:字典
            if("1".equals(type)){
                version = standardVersionService.saveDatasetExcel(file, version,standardId);
            }else {
                version = standardVersionService.saveDictExcel(file, version,standardId);
            }
            //解析excel文件 保存
            return ObjEnvelop.getSuccess("操作成功","{\"version\":\"" + version + "\"}");
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("操作失败",-1);
        }
    }

    @RequestMapping(value = "/isUpdateDatasetAndDict", method = RequestMethod.POST)
    @ApiOperation(value = "项目列表",  produces = "application/json", notes = "项目列表")
    public Envelop isUpdateDatasetAndDict(
            @RequestParam(required = true) String version) {
        try {
            return ObjEnvelop.getSuccess("操作成功",standardVersionService.isUpdateDatasetAndDict(version));
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("操作失败",-1);
        }
    }


    @RequestMapping(value = "/deleteByType", method = RequestMethod.POST)
    @ApiOperation(value = "文件上传成功后的删除",  produces = "application/json", notes = "文件上传成功后的删除")
    public Envelop deleteByType(
            @ApiParam(value = "类型（1:数据集 2:字典 ）", required = true) @RequestParam("type") String type,
            @ApiParam(value = "主键(上传成功后返回的ID)", required = true) @RequestParam("version") String version) {
        try {
            //根據类别判断上传的文件1:数据集 2:字典 删除数据
            switch (type) {
                case "1": {
                    standardVersionService.deleteDataset(version);
                    break;
                }
                case "2": {
                    standardVersionService.deleteDict(version);
                    break;
                }
            }
        } catch (Exception e) {
            return Envelop.getError("操作失败",-1);
        }
        return Envelop.getSuccess("操作成功");
    }

}

