package com.yihu.jw.basic.standard.controller;


import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.fastdfs.FastDFSUtil;
import com.yihu.jw.basic.standard.model.BaseCDADataSetRelationship;
import com.yihu.jw.basic.standard.model.BaseCDADocument;
import com.yihu.jw.basic.standard.model.CDADocument;
import com.yihu.jw.basic.standard.service.bo.StandardVersion;
import com.yihu.jw.basic.standard.service.document.CDADataSetRelationshipManager;
import com.yihu.jw.basic.standard.service.document.CDADocumentService;
import com.yihu.jw.entity.ehr.id.BizObject;
import com.yihu.jw.restmodel.ehr.standard.CDAModel;
import com.yihu.jw.restmodel.ehr.standard.CdaDataSetRelationshipModel;
import com.yihu.jw.restmodel.ehr.standard.MCDADocument;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.util.common.LogService;
import com.yihu.jw.util.sql.ErrorCode;
import com.yihu.jw.util.date.DateUtil;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;


@RequestMapping(value = "/cda", produces = "application/json;charset=utf-8")
@RestController
@Api(value = "StdCdaController", tags = {"cda"})
public class StdCdaController extends EnvelopRestEndpoint {

    @Resource(name = CDADocumentService.BEAN_ID)
    private CDADocumentService cdaDocumentService;
    @Autowired
    private CDADataSetRelationshipManager cdaDatasetRelationshipManager;
    @Autowired
    private FastDFSUtil fastDFSUtil;
    @Autowired
    private ObjectMapper objectMapper;

    @RequestMapping("GetCdaListByKey")
    public Envelop getCdaListByKey(String strKey, String strVersion, String strType, Integer page, Integer rows) {

        String filters = "type="+strType;

        Envelop result = new Envelop();

        if (!StringUtils.isEmpty(strKey)){
            filters += " g1;code?"+strKey+" g2;name?"+strKey+" g2";
        }

        if (StringUtils.isEmpty(strVersion)) {
            result.setStatus(-1);
            result.setMessage(ErrorCode.VersionCodeIsNull.toString());
            return result;
        }

        try {
            List<BaseCDADocument> ls = cdaDocumentService.searchPage(BaseCDADocument.class, StandardVersion.CDADocumentTable, strVersion, filters, "", page, rows,true);
            Long totalCount = cdaDocumentService.searchCount(BaseCDADocument.class, strVersion, StandardVersion.CDADocumentTable, filters);
            Collection<MCDADocument> cdaDocumentList = convertToModels(ls, new ArrayList<>(ls.size()), MCDADocument.class, "");

            ResponseEntity<Collection<MCDADocument>> responseEntity = ResponseEntity.ok().body(cdaDocumentList);
            cdaDocumentList = responseEntity.getBody();
            List<CDAModel> cdaModelList = new ArrayList<>();
            for (MCDADocument mcdaDocument : cdaDocumentList) {
                mcdaDocument.setVersionCode(strVersion);
                cdaModelList.add(convertToCDAModel(mcdaDocument));
            }

            return PageEnvelop.getSuccessListWithPage("查询成功",cdaModelList,  page, rows, totalCount);
        } catch (Exception ex) {
            LogService.getLogger(StdCdaController.class).error(ex.getMessage());
            return Envelop.getError("查询失败");
        }

    }

    @RequestMapping("getCDAInfoById")
    public Envelop getCDAInfoById(String strId, String strVersion) {
        String strErrorMsg = "";
        Envelop result = new Envelop();
        if (StringUtils.isEmpty(strVersion)) {
            strErrorMsg += "请选择标准版本!";
        }
        if (StringUtils.isEmpty(strId)) {
            strErrorMsg += "请选择将要编辑的CDA!";
        }
        if (!StringUtils.isEmpty(strErrorMsg)) {
            result.setStatus(-1);
            result.setMessage(strErrorMsg);
            return result;
        }
        try {
            List<CDADocument> list = cdaDocumentService.searchPageData(BaseCDADocument.class, StandardVersion.CDADocumentTable, strVersion, "id=" + strId, null, null, null);
            Collection<MCDADocument> mcdaDocumentList = convertToModels(list, new ArrayList<>(list.size()), MCDADocument.class, "");

            if (mcdaDocumentList == null) {
                return failed("数据获取失败!");
            }
            List<CDAModel> cdaModelList = new ArrayList<>();
            for (MCDADocument mcdaDocument : mcdaDocumentList) {
                mcdaDocument.setVersionCode(strVersion);
                cdaModelList.add(convertToCDAModel(mcdaDocument));
            }
            return ListEnvelop.getSuccess("查询成功",cdaModelList);
        } catch (Exception ex) {
            LogService.getLogger(StdCdaController.class).error(ex.getMessage());
            return Envelop.getError("查询失败");
        }

    }

    @RequestMapping("getALLRelationByCdaId")
    public Envelop getALLRelationByCdaId(String cdaId, String strVersionCode) {
        try {
            Collection<BaseCDADataSetRelationship> mcdaDocumentList = cdaDatasetRelationshipManager.searchPageData
                    (BaseCDADataSetRelationship.class, StandardVersion.CDADatasetTable, strVersionCode, "cdaId=" + cdaId, null, null, null);
            List<CdaDataSetRelationshipModel> cdaDataSetRelationshipModels = new ArrayList<>();

            for (BaseCDADataSetRelationship mCdaDataSetRelationship : mcdaDocumentList) {
                CdaDataSetRelationshipModel cdaDataSetRelationshipModel = new CdaDataSetRelationshipModel();
                cdaDataSetRelationshipModel.setId(mCdaDataSetRelationship.getDataSetId());
                cdaDataSetRelationshipModel.setCdaId(mCdaDataSetRelationship.getCdaId());
                cdaDataSetRelationshipModel.setDataSetId(mCdaDataSetRelationship.getId());
                cdaDataSetRelationshipModels.add(cdaDataSetRelationshipModel);
            }

            if (cdaDataSetRelationshipModels.size() > 0) {
                return ListEnvelop.getSuccess("查询成功",cdaDataSetRelationshipModels);
            } else {
                return failed("查询失败");
            }
        } catch (Exception ex) {
            LogService.getLogger(StdCdaController.class).error(ex.getMessage());
            return failed("查询失败");
        }
    }

    @RequestMapping("SaveCdaInfo")
    public Envelop SaveCdaInfo(String cdaJson,String version) throws IOException {
        CDADocument cdaModel = objectMapper.readValue(cdaJson, CDADocument.class);
        try {
            if(StringUtils.isEmpty(cdaModel.getId())) {
                cdaModel.setStdVersion(version);
                BaseCDADocument model = (BaseCDADocument) cdaModel;

                model.setId(getObjectId(BizObject.STANDARD));
                model.setCreateDate(new Date());
                cdaDocumentService.add(version, model);
                model.setVersionCode(cdaModel.getVersionCode());
                return success(model);
            }
            else {
                List<CDADocument> list = cdaDocumentService.searchPageData(BaseCDADocument.class, StandardVersion.CDADocumentTable, cdaModel.getVersionCode(), "id=" + cdaModel.getId(), null, null, null);


                BaseCDADocument updateCdaModel = list.get(0);

                updateCdaModel.setCode(cdaModel.getCode());
                updateCdaModel.setName(cdaModel.getName());
                updateCdaModel.setSourceId(cdaModel.getSourceId());
                updateCdaModel.setType(cdaModel.getType());
                updateCdaModel.setDescription(cdaModel.getDescription());

                updateCdaModel.setUpdateDate(new Date());
                cdaDocumentService.modify(version, updateCdaModel);
                updateCdaModel.setVersionCode(cdaModel.getVersionCode());
                return success(updateCdaModel);
            }
        } catch (Exception ex) {
            LogService.getLogger(StdCdaController.class).error(ex.getMessage());
            return failed("修改失败");
        }
    }

    @RequestMapping("deleteCdaInfo")
    public Envelop deleteCdaInfo(String ids, String strVersionCode) {
        Envelop envelop = new Envelop();
        String strErrorMsg = "";
        if (StringUtils.isEmpty(strVersionCode)) {
            strErrorMsg += "标准版本不能为空!";
        }
        if (StringUtils.isEmpty(ids)) {
            strErrorMsg += "请先选择将要删除的CDA";
        }
        if (!StringUtils.isEmpty(strErrorMsg)) {
            envelop.setStatus(-1);
            envelop.setMessage(strErrorMsg);
            return envelop;
        }
        try {
            String[] cdaIds = ids.split(",");
            boolean bo = cdaDocumentService.deleteByField("id", cdaIds, StandardVersion.CDADocumentTable, strVersionCode);
            if(bo){
                return success("删除成功");
            }else {
                return failed("删除失败");
            }

        } catch (Exception ex) {
            LogService.getLogger(StdCdaController.class).error(ex.getMessage());
            return failed("删除失败");
        }
    }

    /**
     * 保存CDA信息
     * 1.先删除CDA数据集关联关系信息与cda文档XML文件，在新增信息
     * @param strDatasetIds 关联的数据集
     * @param strCdaId  cda文档 ID
     * @param strVersionCode 版本号
     * @param xmlInfo xml 文件内容
     * @return 操作结果
     */
    @RequestMapping("SaveRelationship")
    public Envelop SaveRelationship(String strDatasetIds, String strCdaId, String strVersionCode, String xmlInfo) {

        Envelop envelop = new Envelop();
        String strErrorMsg = "";
        if (StringUtils.isEmpty(strVersionCode)) {
            strErrorMsg += "标准版本不能为空!";
        }
        if (StringUtils.isEmpty(strCdaId)) {
            strErrorMsg += "请先选择CDA!";
        }
        if (!StringUtils.isEmpty(strErrorMsg)) {
            envelop.setStatus(-1);
            envelop.setMessage(strErrorMsg);
            return envelop;
        }
        try {
            boolean bo = cdaDatasetRelationshipManager.SaveDataSetRelationship(strDatasetIds, strCdaId, strVersionCode, xmlInfo);
            if(bo){
                return success("保存成功");
            }else {
                return failed("保存失败");
            }
        }catch(Exception ex){
            LogService.getLogger(StdCdaController.class).error(ex.getMessage());
            return failed("保存失败");
        }
    }

    /*
     * 根据CDA ID 获取数据集信息
     * @param strVersionCode 版本号
     * @param strCdaId CDAID
     * @return Result
     * */
    @RequestMapping("/getDatasetByCdaId")
    public Envelop getDatasetByCdaId(String strVersionCode, String strCdaId) {
        try {
            Collection<BaseCDADataSetRelationship> mcdaDocumentList = cdaDatasetRelationshipManager.searchPageData
                    (BaseCDADataSetRelationship.class, StandardVersion.CDADatasetTable, strVersionCode, "cdaId=" + strCdaId, null, null, null);
            List<CdaDataSetRelationshipModel> cdaDataSetRelationshipModels = new ArrayList<>();

            for (BaseCDADataSetRelationship mCdaDataSetRelationship : mcdaDocumentList) {
                CdaDataSetRelationshipModel cdaDataSetRelationshipModel = new CdaDataSetRelationshipModel();
                cdaDataSetRelationshipModel.setId(mCdaDataSetRelationship.getDataSetId());
                cdaDataSetRelationshipModel.setCdaId(mCdaDataSetRelationship.getCdaId());
                cdaDataSetRelationshipModel.setDataSetId(mCdaDataSetRelationship.getId());
                cdaDataSetRelationshipModels.add(cdaDataSetRelationshipModel);
            }

            if (cdaDataSetRelationshipModels.size() > 0) {
                return ListEnvelop.getSuccess("查询成功",cdaDataSetRelationshipModels);
            } else {
                return failed("查询失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return failed("查询失败");
        }
    }

    @RequestMapping("/validatorCda")
    public Object validatorCda(String code, String versionCode) {
        try {
            return success("查询失败");
        } catch (Exception ex) {
            LogService.getLogger(StdCdaController.class).error(ex.getMessage());
            return failed("查询失败");
        }
    }

    /**
     * 获取cda文档的XML文件信息。
     * <p>
     * 从服务器的临时文件路径中读取配置文件，并以XML形式返回。
     *
     * @param cdaId
     * @param versionCode
     * @return XML信息
     */
    @RequestMapping("/getCdaXmlFileInfo")
    public Envelop getCdaXmlFileInfo(String cdaId, String versionCode) {
        try {
            String strXmlInfo = "";
            String strFileGroup = "";
            String strSchemePath = "";

            String strPath = System.getProperty("java.io.tmpdir");
            strPath += "StandardFiles";

            String splitMark = System.getProperty("file.separator");
            String strXMLFilePath = strPath + splitMark + "xml" + splitMark + versionCode + splitMark + "downfile" + splitMark;

            List<String> listIds = new ArrayList<>();
            listIds.add(cdaId);

            BaseCDADocument cdaDocument = (BaseCDADocument) cdaDocumentService.get(BaseCDADocument.class, versionCode, StandardVersion.CDADocumentTable, cdaId);

            if (cdaDocument != null) {
                strFileGroup = cdaDocument.getFileGroup();
                strSchemePath = cdaDocument.getSchema();
            }

            File files = new File(strXMLFilePath);
            if (!files.exists()) {
                files.mkdirs();
            }

            String strLocalFileName = strXMLFilePath + "\\" + strSchemePath.replaceAll("/", "_");
            File localFile = new File(strLocalFileName);
            if (localFile.exists() && localFile.isFile()) {
                localFile.delete();
            }
            if (null != strFileGroup && !strFileGroup.equals("") && !strSchemePath.equals("")) {
                strLocalFileName = fastDFSUtil.download(strFileGroup, strSchemePath, strXMLFilePath);

                File file = new File(strLocalFileName);
                FileReader fr = new FileReader(file);
                BufferedReader bReader = new BufferedReader(fr);
                strXmlInfo = bReader.readLine();
            } else {
                strXmlInfo = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?><root></root>";
            }

            if (strXmlInfo != null) {
                return ObjEnvelop.getSuccess("获取成功",strXmlInfo);
            } else {
                return Envelop.getError("获取cda文档XML文件信息失败");
            }
        }catch (Exception e){
            e.printStackTrace();
            return Envelop.getError("获取cda文档XML文件信息失败");
        }
    }

    public CDAModel convertToCDAModel(MCDADocument mcdaDocument) {
        CDAModel cdaModel = convertToModel(mcdaDocument, CDAModel.class);
        cdaModel.setCreateDate(DateUtil.dateToStr(mcdaDocument.getCreateDate(), DateUtil.YYYY_MM_DD_HH_MM));
        cdaModel.setUpdateDate(DateUtil.dateToStr(mcdaDocument.getUpdateDate(), DateUtil.YYYY_MM_DD_HH_MM));
        return cdaModel;
    }
}
