package com.yihu.jw.basic.standard.controller;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.basic.standard.service.standard.StdDatasetService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.util.common.StringUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by LLH on 2016/1/6.
 */
@RestController("StdDatasetController")
@RequestMapping("/standardCenter")
@Api(protocols = "https", value = "StdDatasetController", tags = {"标准数据集管理"})
public class StdDatasetController extends EnvelopRestEndpoint {
    @Resource(name = StdDatasetService.BEAN_ID)
    private StdDatasetService stdDatasetService;


    @GetMapping("/searchDatasetList")
    @ApiOperation(value = "获取数据集列表",  produces = "application/json", notes = "获取数据集列表")
    public PageEnvelop searchDatasetList(
            @ApiParam(name = "stdVersion", value = "标准版本号")
            @RequestParam(value = "stdVersion") String stdVersion,
            @ApiParam(name = "condition", value = "查询条件：编码或名称")
            @RequestParam(value = "condition", required = false) String condition,
            @ApiParam(name = "q", value = "前端控件传过来的查询条件,此值会覆盖condition")
            @RequestParam(value = "q", required = false) String q,
            @ApiParam(name = "order", value = "Order param,Must be json.ascending:asc,descending:desc")
            @RequestParam(value = "order", required = false) String order,
            @ApiParam(name = "rows", value = "Limit the size of result set. Must be an integer")
            @RequestParam(value = "rows", required = false) Integer rows,
            @ApiParam(name = "page", value = "Start position of result set. Must be an integer")
            @RequestParam(value = "page", required = false) Integer page) {
        if(StringUtils.isNotBlank(q)){
            condition="{\"name\":\""+q+"\"}";
        }
        return stdDatasetService.getDetailModelList(stdVersion, condition, order, rows, page);
    }

    @GetMapping("/getForDatasetId")
    @ApiOperation(value = "获取数据集",  produces = "application/json", notes = "获取数据集")
    public ObjEnvelop getForDatasetId(
            @ApiParam(name = "stdVersion", value = "标准版本号")
            @RequestParam(value = "stdVersion") String stdVersion,
            @ApiParam(name = "datasetId", value = "数据集ID")
            @RequestParam(value = "datasetId") Long datasetId) {
        return stdDatasetService.get(stdVersion, datasetId);
    }

    @PostMapping("/delDataset")
    @ApiOperation(value = "删除数据集",  produces = "application/json", notes = "删除数据集")
    public Envelop delDataset(
            @ApiParam(name = "stdVersion", value = "标准版本号")
            @RequestParam(value = "stdVersion") String stdVersion,
            @ApiParam(name = "datasetId", value = "数据集ID")
            @RequestParam(value = "datasetId") Long datasetId) {
        return stdDatasetService.delete(stdVersion, datasetId);
    }

    @PostMapping("/updateDataset")
    @ApiOperation(value = "修改数据集",  produces = "application/json", notes = "修改数据集")
    public ObjEnvelop updateDataset(
            @ApiParam(name = "stdVersion", value = "标准版本号")
            @RequestParam(value = "stdVersion") String stdVersion,
            @ApiParam(name = "dataset", value = "数据集")
            @RequestParam(value = "dataset") String dataset) {
        return stdDatasetService.update(stdVersion, dataset);
    }

    @PostMapping("/addDataset")
    @ApiOperation(value = "保存数据集",  produces = "application/json", notes = "保存数据集")
    public ObjEnvelop add(
            @ApiParam(name = "stdVersion", value = "标准版本号")
            @RequestParam(value = "stdVersion") String stdVersion,
            @ApiParam(name = "dataset", value = "数据集")
            @RequestParam(value = "dataset") String dataset) {
        return stdDatasetService.add(stdVersion, dataset);
    }

    @GetMapping(value="/getDatasetCodeValues")
    @ApiOperation(value = "获取数据集下拉框信息",  produces = "application/json", notes = "获取机构数据元下拉框信息")
    public PageEnvelop getSelectList(
            @ApiParam(name = "stdVersion", value = "标准版本号")
            @RequestParam(value = "stdVersion") String stdVersion,
            @ApiParam(name = "q", value = "数据集名称或编码")
            @RequestParam(value = "q", required = false) String q,
            @ApiParam(name = "initVal", value = "初始化数据集名称或编码")
            @RequestParam(value = "initVal", required = false) String initVal,
            @ApiParam(name = "order", value = "Order param,Must be json.ascending:asc,descending:desc")
            @RequestParam(value = "order", required = false) String order,
            @ApiParam(name = "rows", value = "Limit the size of result set. Must be an integer")
            @RequestParam(value = "rows", required = false) Integer rows,
            @ApiParam(name = "page", value = "Start position of result set. Must be an integer")
            @RequestParam(value = "page", required = false) Integer page) {
        Map<String,String> condition = new HashMap<>();
        ObjectMapper mapper = new ObjectMapper();
        if(!StringUtil.isEmpty(q)){
            condition.put("name", q);
            rows = 10;
        }
        else if(!StringUtil.isEmpty(initVal)){
            condition.put("initVal", initVal);//默认值
        }
        String conditionStr = null;
        try {
             conditionStr = mapper.writeValueAsString(condition);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }
        return stdDatasetService.getDatasetSelectList(stdVersion, conditionStr, order, rows, 0);
    }
}
