package com.yihu.jw.basic.standard.dao;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.JsonNodeType;
import com.yihu.jw.basic.standard.service.bo.AdapterVersion;
import com.yihu.jw.basic.standard.service.bo.StandardVersion;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.util.common.NumberUtil;
import com.yihu.jw.util.common.StringUtil;
import com.yihu.jw.util.sql.SqlCreator;
import org.apache.commons.lang3.StringUtils;
import org.hibernate.query.NativeQuery;
import org.hibernate.query.Query;
import org.hibernate.Session;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Table;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.io.IOException;
import java.io.Serializable;
import java.math.BigInteger;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.*;


/**
 * @author HZY
 * @vsrsion 1.0
 * Created at 2016/8/8.
 */
@Transactional
@Repository("sqlGeneralDAO")
public class SQLGeneralDAO implements XSQLGeneralDAO {
    static private final Logger logger = LoggerFactory.getLogger(SQLGeneralDAO.class);

    @Autowired
    protected ObjectMapper objectMapper;
    @Autowired
    protected JdbcTemplate jdbcTemplate;
    @PersistenceContext
    protected EntityManager entityManager;

    public SQLGeneralDAO() {
    }

    public JdbcTemplate getJdbcTemplate() {
        return this.jdbcTemplate;
    }

    public Session currentSession() {
        return entityManager.unwrap(Session.class);
    }

    public Envelop getDataGridResult(String hql, Integer page, Integer rows) {
        Query query = currentSession().createQuery(hql);
        if (page != null && rows != null) {
            long count = query.list().size();
            query.setMaxResults(rows);
            query.setFirstResult((page - 1) * rows);
            return PageEnvelop.getSuccessListWithPage("查询成功",query.list(),page,rows,count);
        } else {
            List list = query.list();
            return PageEnvelop.getSuccessListWithPage("查询成功",list,page,rows,(long)list.size());
        }
    }

    public Envelop getDataGridResult(Query query, Integer page, Integer rows) {
        if (page != null && rows != null) {
            long count = query.list().size();
            query.setMaxResults(rows);
            query.setFirstResult((page - 1) * rows);
            return PageEnvelop.getSuccessListWithPage("查询成功",query.list(),page,rows,count);
        } else {
            List list = query.list();
            return PageEnvelop.getSuccessListWithPage("查询成功",list,page,rows,(long)list.size());
        }
    }

    public boolean delete(Class tclass, Object[] ids, String dataType, String version) {
        if (ids == null || ids.length == 0)
            return false;

        StandardVersion standardVersion = new StandardVersion(version);
        String tableName = standardVersion.getTableName(dataType);
        StringBuffer sb = new StringBuffer("'");
        for ( int i= 0; i < ids.length; i++){
            sb.append(ids[i]).append("','");
        }
        String st = sb.toString();
        st = st.substring(1,st.length()-1);
        String sql ="DELETE FROM " + tableName +" WHERE id in ("+st+") ";
        return jdbcTemplate.update(sql) > 0;

    }

    public void beginTransaction() throws Exception {
        currentSession().getTransaction().begin();
    }

    public void commitTransaction() throws Exception {
        currentSession().getTransaction().commit();
    }

    public void saveEntity(Object entity) throws Exception {
        if (entity != null) {
            currentSession().save(entity);
        }
    }

    public void updateEntity(Object entity) throws Exception {
        if (entity != null) {
            currentSession().update(entity);
        }
    }

    public void saveOrUpdateEntity(Object entity) throws Exception {
        if (entity != null) {
            currentSession().saveOrUpdate(entity);
        }
    }

    public void mergeEntity(Object entity) throws Exception {
        if (entity != null) {
            currentSession().merge(entity);
        }
    }

    public void deleteEntity(Object entity) throws Exception {
        if (entity != null) {
            currentSession().delete(entity);
        }
    }

    public <T> void deleteEntity(Class<T> cls, Serializable id) throws Exception {
        Object obj = currentSession().load(cls, id);
        currentSession().delete(obj);
    }

    public <T> T getEntity(Class<T> cls, Serializable id) throws Exception {
        return currentSession().get(cls, id);
    }

    public <T> T retrieve(Class<T> tclass,String tableName,Object id){
        return (T) get(tclass,tableName,id);
    }

    public List<Map<String, Object>> queryListBySql(String sql) throws Exception {
        List result = this.jdbcTemplate.queryForList(sql);
        return result;
    }

    public List<Map<String, Object>> queryListBySql(String sql, List params) throws Exception {
        List result = this.jdbcTemplate.queryForList(sql, params.toArray());
        return result;
    }

    public <T> List<T> queryListBySql(String sql, Class obj) throws Exception {
        RowMapper<T> rowMapper = BeanPropertyRowMapper.newInstance(obj);
        List result = this.jdbcTemplate.query(sql, rowMapper);
        return result;
    }

    public <T> List<T> queryListBySql(String sql, List params, Class obj) throws Exception {
        RowMapper<T> rowMapper = BeanPropertyRowMapper.newInstance(obj);
        List result = this.jdbcTemplate.query(sql, params.toArray(), rowMapper);
        return result;
    }

    public <T> T queryObjBySql(String sql, List params, Class obj) throws Exception {
        RowMapper<T> rowMapper = BeanPropertyRowMapper.newInstance(obj);
        T result = this.jdbcTemplate.queryForObject(sql, params.toArray(), rowMapper);
        return result;
    }

    public <T> T queryObjBySql(String sql, Class obj) throws Exception {
        RowMapper<T> rowMapper = BeanPropertyRowMapper.newInstance(obj);
        T result = this.jdbcTemplate.queryForObject(sql, rowMapper);
        return result;
    }

    public <T> T scalarBySql(String sql, Class cls) throws Exception {
        T result = (T) this.jdbcTemplate.queryForObject(sql, cls);
        return result;
    }

    public void execute(String sql) throws Exception {
        this.jdbcTemplate.execute(sql);
    }

    public <T> List getEntityList(Class<T> cls, String hql) {
        Query query = this.getCurrentSession().createQuery(hql);
        return query.list();
    }

    public <T> List getEntityList(Class<T> cls, String condition, String order, Integer limit, Integer offset) throws IOException {
        CriteriaBuilder criteriaBuilder = getCurrentSession().getCriteriaBuilder();
        CriteriaQuery<T> criteriaQuery = criteriaBuilder.createQuery(cls);
        Root<T> root = criteriaQuery.from(cls);
        List<Predicate> predicateList = new ArrayList<>();
        JsonNode jsonNode;
        Iterator fieldNames;
        String fieldName;
        if (!StringUtil.isEmpty(condition)) {
            jsonNode = objectMapper.readTree(condition);
            fieldNames = jsonNode.fieldNames();

            while (fieldNames.hasNext()) {
                fieldName = (String) fieldNames.next();
                if (jsonNode.get(fieldName).isInt()) {
                    Predicate predicate1 = criteriaBuilder.equal(root.get(fieldName), jsonNode.get(fieldName).asInt());
                    predicateList.add(predicate1);
                } else {
                    Predicate predicate1 = criteriaBuilder.equal(root.get(fieldName), jsonNode.get(fieldName).asText());
                    predicateList.add(predicate1);
                }
            }
        }

        if (!StringUtil.isEmpty(order)) {
            jsonNode = objectMapper.readTree(order);
            fieldNames = jsonNode.fieldNames();

            List<javax.persistence.criteria.Order> orderList = new ArrayList<>();
            while (fieldNames.hasNext()) {
                fieldName = (String) fieldNames.next();
                String value = jsonNode.get(fieldName).asText().toUpperCase();
                if (value.equals("ASC")) {
                    orderList.add(criteriaBuilder.asc(root.get(fieldName)));
                } else if (value.equals("DESC")) {
                    orderList.add(criteriaBuilder.desc(root.get(fieldName)));
                }
            }
            criteriaQuery.orderBy(orderList);
        }

        if(predicateList.size()>0){
            Predicate[] predicates = new Predicate[predicateList.size()];
            for (int i = 0; i < predicates.length; i++) {
                predicates[i] = predicateList.get(i);
            }
            criteriaQuery.where(predicates);
        }
        TypedQuery<T> typedQuery = getCurrentSession().createQuery(criteriaQuery);
        if (limit != null) {
            typedQuery.setMaxResults(limit);
            if (offset != null &&offset>0) {
                typedQuery.setFirstResult((offset - 1) * limit);
            }
        }
        return typedQuery.getResultList();
    }

    public <T> List<T> getEntityListByParentId(Class<T> cls, String parentFiledName, Long patentId, String condition, String order, Integer limit, Integer offset) throws IOException {
        Session session = this.getCurrentSession();
        CriteriaBuilder criteriaBuilder = session.getCriteriaBuilder();
        CriteriaQuery<T> criteriaQuery = criteriaBuilder.createQuery(cls);
        Root<T> root = criteriaQuery.from(cls);
        List<Predicate> predicateList = new ArrayList<>();
        Predicate predicate = criteriaBuilder.equal(root.get(parentFiledName), patentId);
        predicateList.add(predicate);

        ObjectMapper objectMapper = new ObjectMapper();
        JsonNode jsonNode;
        Iterator fieldNames;
        String fieldName;
        if (!StringUtil.isEmpty(condition)) {
            jsonNode = objectMapper.readTree(condition);
            fieldNames = jsonNode.fieldNames();

            while (fieldNames.hasNext()) {
                fieldName = (String) fieldNames.next();
                if (jsonNode.get(fieldName).isInt()) {
                    Predicate predicate1 = criteriaBuilder.equal(root.get(fieldName), jsonNode.get(fieldName).asInt());
                    predicateList.add(predicate1);
                } else {
                    Predicate predicate1 = criteriaBuilder.equal(root.get(fieldName), jsonNode.get(fieldName).asText());
                    predicateList.add(predicate1);
                }
            }
        }

        if (!StringUtil.isEmpty(order)) {
            jsonNode = objectMapper.readTree(order);
            fieldNames = jsonNode.fieldNames();
            List<javax.persistence.criteria.Order> orderList = new ArrayList<>();
            while (fieldNames.hasNext()) {
                fieldName = (String) fieldNames.next();
                String value = jsonNode.get(fieldName).asText().toUpperCase();
                if (value.equals("ASC")) {
                    orderList.add(criteriaBuilder.asc(root.get(fieldName)));
                } else if (value.equals("DESC")) {
                    orderList.add(criteriaBuilder.desc(root.get(fieldName)));
                }
            }
            criteriaQuery.orderBy(orderList);
        }

        Predicate[] predicates = new Predicate[predicateList.size()];
        for (int i = 0; i < predicates.length; i++) {
            predicates[i] = predicateList.get(i);
        }
        criteriaQuery.where(predicates);

        TypedQuery<T> typedQuery = session.createQuery(criteriaQuery);

        if (limit != null) {
            typedQuery.setMaxResults(limit);
            if (offset != null &&offset>0) {
                typedQuery.setFirstResult((offset - 1) * limit);
            }
        }
        return typedQuery.getResultList();
    }

    public List getList(Class tClass, String tableName, String condition, String order, Integer limit, Integer offset) {
        SqlCreator sqlCreator = new SqlCreator(tClass);
        ObjectMapper objectMapper = new ObjectMapper();

        try {
            JsonNode sql;
            Iterator query;
            String fieldName;
            String value;
            if (!StringUtil.isEmpty(condition)) {
                sql = objectMapper.readTree(condition);
                query = sql.fieldNames();

                while (query.hasNext()) {
                    fieldName = (String) query.next();
                    value = sql.get(fieldName).asText();
                    if (sql.get(fieldName).getNodeType().equals(JsonNodeType.NUMBER)) {
                        sqlCreator.equalCondition(fieldName, Integer.parseInt(value));
                    } else {
                        sqlCreator.equalCondition(fieldName, value);
                    }
                }
            }

            if (!StringUtil.isEmpty(order)) {
                sql = objectMapper.readTree(order);
                query = sql.fieldNames();

                while (query.hasNext()) {
                    fieldName = (String) query.next();
                    value = sql.get(fieldName).asText().toUpperCase();
                    sqlCreator.order(fieldName, value);
                }
            }
        } catch (Exception var13) {
            var13.printStackTrace();
        }

        String sql = sqlCreator.selectData(tableName);
        if (limit != null) {
            if (offset != null &&offset>0) {
                sql += " limit "+(offset - 1) * limit+","+limit;
            }else {
                sql += " limit "+limit;
            }

        }
        return jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(tClass));
    }

    public List getListByIdList(Class tClass, String tableName, String fieldName, List<Integer> idList) {
        SqlCreator sqlCreator = new SqlCreator(tClass);

        try {
            sqlCreator.inCondition(fieldName, StringUtils.join(idList,","));
        } catch (Exception var8) {
            var8.printStackTrace();
            return null;
        }

        String sql = sqlCreator.selectData(tableName);
        return jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(tClass));
    }

    public Integer getDataSetInt(Class tClass, String tableName, String condition) {
        SqlCreator sqlCreator = new SqlCreator(tClass);
        ObjectMapper objectMapper = new ObjectMapper();

        try {
            if (!StringUtil.isEmpty(condition)) {
                JsonNode sql = objectMapper.readTree(condition);
                Iterator query = sql.fieldNames();

                while (query.hasNext()) {
                    String count = (String) query.next();
                    String value = sql.get(count).asText();
                    if (NumberUtil.isInteger(value)) {
                        sqlCreator.equalCondition(count, Integer.parseInt(value));
                    } else {
                        sqlCreator.equalCondition(count, value);
                    }
                }
            }
        } catch (Exception var10) {
            var10.printStackTrace();
        }

        String sql1 = sqlCreator.countData(tableName);
        Integer count1 = jdbcTemplate.queryForObject(sql1,Integer.class);
        return count1;
    }

    public Object get(Class tClass, String tableName, Object id) {
        SqlCreator sqlCreator = new SqlCreator(tClass);
        sqlCreator.equalCondition("id", id);
        String sql = sqlCreator.selectData(tableName);
        return jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(tClass)).get(0);
    }

    public Object get(Class tClass, String stdVersion, String dataType, String id) {
        try {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            if (id != null) {
                String tableName = standardVersion.getTableName(dataType);
                SqlCreator sqlCreator = new SqlCreator(tClass);
                sqlCreator.equalCondition("id", id);
                String sql = sqlCreator.selectData(tableName);
                return jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(tClass)).get(0);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Envelop.getError("获取标准数据元失败");
    }

    public void insertBatch(final List<String> insertSqlList) {
        String[] sqlStr = new String[insertSqlList.size()];
        String[] sqlArr = insertSqlList.toArray(sqlStr);
        this.jdbcTemplate.batchUpdate(sqlArr);
    }

    public Session getCurrentSession() {
        return currentSession();
    }

    public Long getMaxId(String tableName) {
        String sql = "select max(id) from " + tableName;
        NativeQuery query = this.getCurrentSession().createSQLQuery(sql);
        Object object = query.uniqueResult();
        Long maxId = object == null ? 1L : (Long.parseLong(object.toString()) + 1);
        return maxId;
    }

    public NativeQuery getExeuteQuery(SqlCreator sqlCreator, String sql) {
        NativeQuery query = this.getCurrentSession().createSQLQuery(sql);
        Iterator var4 = sqlCreator.getKeyValueMap().keySet().iterator();

        while (var4.hasNext()) {
            String key = (String) var4.next();
            Object value = sqlCreator.getKeyValueMap().get(key);
            if (value instanceof Collection) {
                query.setParameterList(key, (Collection) value);
            } else {
                query.setParameter(key, value);
            }
        }

        return query;
    }

    public Boolean tableIsExist(String sql) {
        BigInteger tableNum = (BigInteger) this.getCurrentSession().createSQLQuery(sql).uniqueResult();
        if (tableNum.intValue() > 0) {
            return true;
        }
        return false;
    }

    protected void doClose(Session session, Statement stmt, ResultSet rs) {
        if (rs != null) {
            try {
                rs.close();
                rs = null;
            } catch (Exception var6) {
                rs = null;
                logger.error("close-ResultSet ", var6);
                var6.printStackTrace();
            }
        }

        if (stmt != null) {
            try {
                stmt.close();
                stmt = null;
            } catch (Exception var5) {
                stmt = null;
                logger.error("close-ResultSet ", var5);
                var5.printStackTrace();
            }
        }
    }

    public String getTableName(Class<?> clazz) {
        Table table = (Table) clazz.getAnnotation(Table.class);
        return table.name();
    }

    public List searchVersionData(Class tClass, String stdVersion,String dataType, String condition) throws IOException {
        return searchVersionData(tClass,stdVersion,dataType,condition,null,null,null);
    }

    public List searchVersionData(Class tClass, String stdVersion,String dataType, String condition, String order, Integer page, Integer size) throws IOException {
        String tableName = null;
        if (!StringUtil.isEmpty(stdVersion)) {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            tableName = standardVersion.getTableName(dataType);
        } else {
            tableName = getTableName(tClass);
        }

        SqlCreator sqlCreator = new SqlCreator(tClass);
        //where
        if (!StringUtil.isEmpty(condition)) {
            String[] filterArray = condition.split(";");
            for (int i = 0; i < filterArray.length; ++i) {
                String[] tokens = filterArray[i].split("=");
                if (tokens.length > 2) {
                    for (int j = 1; j < tokens.length; j++) {
                        if (j == tokens.length - 1)
                            tokens[1] = tokens[j];
                        else
                            tokens[0] += " " + tokens[j];
                    }
                }
                String group = null;
                if (tokens.length >= 2) group = tokens[1];

                if (group == null)
                    group = Integer.toString(i);

                sqlCreator.likeCondition(tokens[0], tokens[1]);
            }
        }

        //order
        if (StringUtils.isNotEmpty(order)) {
            String[] orderArray = order.split(",");
            for (String elem : orderArray) {
                String value = elem.startsWith("+") ? "ASC" : "DESC";
                sqlCreator.order(elem.substring(1), value);
            }
        }
        String sql = sqlCreator.selectData(tableName);
        return jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(tClass));
    }

    public Long searchVersionCount(Class tClass, String stdVersion, String dataType, String condition) throws IOException {
        String tableName = null;
        if (!StringUtil.isEmpty(stdVersion)) {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            tableName = standardVersion.getTableName(dataType);
        } else {
            tableName = getTableName(tClass);
        }

        SqlCreator sqlCreator = new SqlCreator(tClass);
        //where
        if (!StringUtil.isEmpty(condition)) {
            String[] filterArray = condition.split(";");
            for (int i = 0; i < filterArray.length; ++i) {
                String[] tokens = filterArray[i].split("=");
                if (tokens.length > 2) {
                    for (int j = 1; j < tokens.length; j++) {
                        if (j == tokens.length - 1)
                            tokens[1] = tokens[j];
                        else
                            tokens[0] += " " + tokens[j];
                    }
                }
                String group = null;
                if (tokens.length >= 2) group = tokens[1];

                if (group == null)
                    group = Integer.toString(i);

                sqlCreator.likeCondition(tokens[0], tokens[1]);
            }
        }

        String sql = sqlCreator.countData(tableName);
        Long count = jdbcTemplate.queryForObject(sql,Long.class);
        return count;
    }

    public Long searchCount(Class tClass, String stdVersion, String dataType, String condition) throws IOException {
        String tableName = null;
        if (!StringUtil.isEmpty(stdVersion)) {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            tableName = standardVersion.getTableName(dataType);
        } else {
            tableName = getTableName(tClass);
        }

        SqlCreator sqlCreator = new SqlCreator(tClass);
        //where
        if (!StringUtil.isEmpty(condition)) {
            Map<String, List<String>> whereMap = new HashMap<>();

            String[] filterArray = condition.split(";");
            for (int i = 0; i < filterArray.length; ++i) {
                String[] tokens = filterArray[i].split(" ");
                if (tokens.length > 2) {
                    for (int j = 1; j < tokens.length; j++) {
                        if (j == tokens.length - 1)
                            tokens[1] = tokens[j];
                        else
                            tokens[0] += " " + tokens[j];
                    }
                }

                String group = null;
                if (tokens.length >= 2)
                    group = tokens[1];
                if (group == null)
                    group = Integer.toString(i);

                List<String> list = whereMap.get(group);
                if(list==null || list.size()==0){
                    list = new ArrayList<String>();
                    list.add(tokens[0]);
                }else{
                    list.add(tokens[0]);
                }
                whereMap.put(group,list);
            }

            Collection<List<String>> lists = whereMap.values();

            for(List<String> whereCondition :lists){
                sqlCreator.makeWhere(whereCondition);
            }
        }

        String sql = sqlCreator.countData(tableName);
        Long count = jdbcTemplate.queryForObject(sql,Long.class);
        return count;
    }

    //这边不支持分组条件查询,如需要分组查询
    //因为调用的地方太多,未做大改造,新增了  searchPage 方法
    public List searchPageData(Class tClass,String dataType, String stdVersion, String condition, String order, Integer offset, Integer limit) throws IOException {
        StandardVersion standardVersion = new StandardVersion(stdVersion);
        String tableName = standardVersion.getTableName(dataType);
        SqlCreator sqlCreator = new SqlCreator(tClass);
        //where
        if (!StringUtil.isEmpty(condition)) {
            String[] filterArray = condition.split(";");
            for (int i = 0; i < filterArray.length; ++i) {
                String[] tokens = filterArray[i].split("=");
                if (tokens.length > 2){
                    for(int j=1; j<tokens.length; j++){
                        if(j==tokens.length-1)
                            tokens[1] = tokens[j];
                        else
                            tokens[0] += " " + tokens[j] ;
                    }
                }
                String group = null;
                if (tokens.length >= 2) group = tokens[1];

                if (group == null)
                    group = Integer.toString(i);

                if(tokens[1].contains(",")){
                    sqlCreator.inCondition(tokens[0],tokens[1]);
                }else{
                    sqlCreator.equalConditionQue(tokens[0],tokens[1]);
                }
            }
        }

        //order
        String sql = sqlCreator.selectData(tableName);
        if (StringUtils.isNotEmpty(order)) {
            sql += " order by "+order;
        }
        if(limit != null){
            if (offset != null &&offset>0) {
                sql += " limit "+ (offset - 1) * limit+","+limit;
            }else {
                sql += " limit " +limit;
            }
        }
        return jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(tClass));
    }



    public List searchPage(Class tClass,String dataType, String version, String condition, String order, Integer offset, Integer limit,boolean isStandard) throws IOException {
        String tableName;
        if(isStandard){
            StandardVersion standardVersion = new StandardVersion(version);
            tableName = standardVersion.getTableName(dataType);
        }else{
            AdapterVersion adapterVersion = new AdapterVersion(version);
            tableName = adapterVersion.getTableName(dataType);
        }

        SqlCreator sqlCreator = new SqlCreator(tClass);
        //where
        if (!StringUtil.isEmpty(condition)) {
            Map<String, List<String>> whereMap = new HashMap<>();

            String[] filterArray = condition.split(";");
            for (int i = 0; i < filterArray.length; ++i) {
                String[] tokens = filterArray[i].split(" ");
                if (tokens.length > 2) {
                    for (int j = 1; j < tokens.length; j++) {
                        if (j == tokens.length - 1)
                            tokens[1] = tokens[j];
                        else
                            tokens[0] += " " + tokens[j];
                    }
                }

                String group = null;
                if (tokens.length >= 2)
                    group = tokens[1];
                if (group == null)
                    group = Integer.toString(i);

                List<String> list = whereMap.get(group);
                if(list==null || list.size()==0){
                    list = new ArrayList<String>();
                    list.add(tokens[0]);
                }else{
                    list.add(tokens[0]);
                }
                whereMap.put(group,list);
            }

            Collection<List<String>> lists = whereMap.values();

            for(List<String> whereCondition :lists){
                sqlCreator.makeWhere(whereCondition);
            }
        }

        //order
        if (StringUtils.isNotEmpty(order)) {
            String[] orderArray = order.split(",");
            for(String elem : orderArray){
                String value = elem.startsWith("-") ? "DESC" : "ASC";
                sqlCreator.order(elem.substring(1), value);
            }
        }
        String sql = sqlCreator.selectData(tableName);
        return jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(tClass));
    }

    public boolean deleteByField(String field, Object[] ids, String dataType, String version) {
        if (ids == null || ids.length == 0)
            return false;

        StandardVersion standardVersion = new StandardVersion(version);
        String tableName = standardVersion.getTableName(dataType);
        String sql =
                "DELETE FROM " + tableName +
                        " WHERE " + field + " in (:ids) ";
        Query query = this.getCurrentSession().createSQLQuery(sql);
        query.setParameterList("ids", ids);
        return query.executeUpdate() > 0;

    }
}
