package com.yihu.jw.basic.standard.service.adapter;


import com.fasterxml.jackson.databind.JsonNode;
import com.yihu.jw.basic.standard.dao.IStdDataSetModelDao;
import com.yihu.jw.basic.standard.dao.IStdMetaDataModelDao;
import com.yihu.jw.basic.standard.match.matchModel.MetadataMatchVO;
import com.yihu.jw.basic.standard.match.metadata.MetadataStrategyExecute;
import com.yihu.jw.basic.standard.match.metadata.MetatdataStandardNameStrategy;
import com.yihu.jw.basic.standard.model.TreeView;
import com.yihu.jw.basic.standard.model.adapter.AdapterDatasetModel;
import com.yihu.jw.basic.standard.model.adapter.AdapterMetadataModel;
import com.yihu.jw.basic.standard.model.adapter.AdapterSchemeVersionModel;
import com.yihu.jw.basic.standard.model.standard.StdDataSetModel;
import com.yihu.jw.basic.standard.model.standard.StdMetaDataModel;
import com.yihu.jw.basic.standard.service.bo.AdapterVersion;
import com.yihu.jw.basic.standard.service.bo.StandardVersion;
import com.yihu.jw.basic.standard.service.standard.StdMetadataService;
import com.yihu.jw.basic.util.GetChineseFirst;
import com.yihu.jw.basic.util.LikeHashMap;
import com.yihu.jw.lang.SpringContext;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.util.common.StringUtil;
import com.yihu.jw.util.sql.ErrorCode;
import com.yihu.jw.basic.standard.dao.SQLGeneralDAO;
import com.yihu.jw.exception.ApiException;
import com.yihu.jw.util.sql.BeanTransformer;
import com.yihu.jw.util.sql.SqlConstants;
import com.yihu.jw.util.sql.SqlCreator;
import com.yihu.jw.util.thread.ContextAttributes;
import com.yihu.jw.util.thread.LocalContext;
import org.apache.commons.collections.CollectionUtils;
import org.hibernate.query.Query;
import org.hibernate.Session;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.*;

@Transactional
@Service("AdapterDatasetService")
public class AdapterDatasetService extends SQLGeneralDAO {

    public static final String BEAN_ID = "AdapterDatasetService";

    @Autowired
    private AdapterMetadataService adapterMetadataService;

    @Autowired
    private AdapterSchemeVersionService adapterSchemeVersionService;

    @Resource(name = "wlyyExecutor")
    private ThreadPoolTaskExecutor threadPoolTaskExecutor;

    @Autowired
    private IStdMetaDataModelDao stdMetaDataModelDao;

    @Autowired
    private IStdDataSetModelDao stdDataSetModelDao;

    @Autowired
    private StdMetadataService stdMetadataService;

    private AdapterDatasetService adapterDatasetService;

    public AdapterDatasetService() {
    }

    public void finalize() throws Throwable {
        super.finalize();
    }

    public ObjEnvelop add(String adapterVersion, AdapterDatasetModel adapterDatasetModel) {
        try {
            AdapterVersion version = new AdapterVersion(adapterVersion);
            long maxId = getMaxId(version.getDataSetTableName());
            adapterDatasetModel.setId( Long.valueOf(maxId));
            JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(adapterDatasetModel));
            SqlCreator sqlCreator = new SqlCreator(AdapterDatasetModel.class);
            String sql = sqlCreator.insertData(version.getDataSetTableName(), jsonNode);
            Query query = getExeuteQuery(sqlCreator, sql);
            query.executeUpdate();
            return ObjEnvelop.getSuccess("保存适配数据集成功",adapterDatasetModel);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError("保存适配数据集失败",-1);
        }
    }

    public ObjEnvelop add(String adapterVersion, String dataset) {
        try {
            AdapterVersion version = new AdapterVersion(adapterVersion);
            AdapterDatasetModel adapterDatasetModel = objectMapper.readValue(dataset, AdapterDatasetModel.class);
            adapterDatasetModel.setId(getMaxId(version.getDataSetTableName()));
            JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(adapterDatasetModel));
            SqlCreator sqlCreator = new SqlCreator(AdapterDatasetModel.class);
            String sql = sqlCreator.insertData(version.getDataSetTableName(), jsonNode);
            Query query = getExeuteQuery(sqlCreator, sql);
            query.executeUpdate();
            return ObjEnvelop.getSuccess("保存适配数据集成功",adapterDatasetModel);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError("保存适配数据集失败",-1);
        }
    }

    public Envelop delete(String adapterVersion, Integer datasetId) {
        try {
            if (adapterVersion == null || adapterVersion.length() == 0) {
                return Envelop.getSuccess("无效适配版本");
            }
            AdapterVersion version = new AdapterVersion(adapterVersion);
            Session session = getCurrentSession();
            String sql = "delete from " + version.getDataSetTableName() + " where id = :id";
            Query query = session.createSQLQuery(sql);
            query.setInteger("id", datasetId);
            query.executeUpdate();
            return Envelop.getSuccess("删除适配数据集成功");
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("删除适配数据集失败",-1);
        }
    }

    public ObjEnvelop modify(String adapterVersion, String dataset) {
        try {
            AdapterVersion version = new AdapterVersion(adapterVersion);
            AdapterDatasetModel adapterDatasetModel = objectMapper.readValue(dataset, AdapterDatasetModel.class);
            JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(adapterDatasetModel));
            SqlCreator sqlCreator = new SqlCreator(AdapterDatasetModel.class);
            String sql = sqlCreator.updateDataByTableKey(version.getDataSetTableName(), jsonNode);
            Query query = getExeuteQuery(sqlCreator, sql);
            query.executeUpdate();
            return ObjEnvelop.getSuccess("修改适配数据集成功",adapterDatasetModel);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError("修改适配数据集失败",-1);
        }
    }

    public ObjEnvelop get(String version, Integer datasetId) {
        try {
            AdapterVersion adapterVersion = new AdapterVersion(version);
            AdapterDatasetModel adapterDatasetModel = (AdapterDatasetModel) get(AdapterDatasetModel.class, adapterVersion.getDataSetTableName(), datasetId);
            return ObjEnvelop.getSuccess("获取适配数据集成功",adapterDatasetModel);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError("获取适配数据集失败",-1);
        }
    }

    public PageEnvelop getDetailModelResult(String version, String condition, String order, Integer rows, Integer page) {
        try {
            AdapterVersion adapterVersion = new AdapterVersion(version);
            List<AdapterDatasetModel> dataSetModelList = getDatasetList(AdapterDatasetModel.class, version, condition, order, rows, page);
            Integer count = getDatasetInt(AdapterDatasetModel.class, adapterVersion, condition);
            PageEnvelop detailModelResult = PageEnvelop.getSuccess("获取适配数据集成功");
            detailModelResult.setDetailModelList(dataSetModelList);
            detailModelResult.setTotalCount(count);
            return detailModelResult;
        } catch (Exception e) {
            e.printStackTrace();
            return PageEnvelop.getError("获取适配数据集失败",-1);
        }
    }

    public List getDatasetList(Class tClass, String version, String condition, String order, Integer limit, Integer offset) throws Exception {
        AdapterVersion adapterVersion = new AdapterVersion(version);
        String tableName = adapterVersion.getDataSetTableName();
        SqlCreator sqlCreator = new SqlCreator(tClass);
        if (!StringUtil.isEmpty(condition)) {
            JsonNode jsonNode = objectMapper.readTree(condition);
            String name = jsonNode.get("name").asText();
            sqlCreator.likeOrCondition("stdDatasetName", "stdDatasetCode", name);
        }
        if (!StringUtil.isEmpty(order)) {
            JsonNode jsonNode = objectMapper.readTree(order);
            Iterator<String> fieldNames = jsonNode.fieldNames();
            while (fieldNames.hasNext()) {
                String fieldName = fieldNames.next();
                String value = jsonNode.get(fieldName).asText().toUpperCase();
                sqlCreator.order(fieldName, value);
            }
        }
        String sql = sqlCreator.selectData(tableName);
        return jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(tClass));
    }

    public Integer getDatasetInt(Class tClass, AdapterVersion adapterVersion, String condition) {
        try {
            String tableName = adapterVersion.getDataSetTableName();
            SqlCreator sqlCreator = new SqlCreator(tClass);
            if (!StringUtil.isEmpty(condition)) {
                JsonNode jsonNode = objectMapper.readTree(condition);
                String name = jsonNode.get("name").asText();
                sqlCreator.likeOrCondition("stdDatasetName", "stdDatasetCode", name);
            }
            String sql = sqlCreator.countData(tableName);
            Integer count = jdbcTemplate.queryForObject(sql,Integer.class);
            return count;
        } catch (Exception e) {
            e.printStackTrace();
            throw new ApiException(ErrorCode.GetDataSetListFailed.getErrorCode());
        }
    }

    public List<TreeView> getAdapterCustomize(String adapterVersion) throws IOException {

        List<AdapterDatasetModel> adapterDataSetList = searchPage(AdapterDatasetModel.class, AdapterVersion.DataSetTablePrefix,adapterVersion, "", null, null, null,false);
        List<TreeView> adapterCustomizeList = new ArrayList<>();
        int i = 0;//用于判断左侧根节点是否需要全选

        for (AdapterDatasetModel adapterDataset : adapterDataSetList) {
            TreeView parent = new TreeView();
            parent.setId("SD" + adapterDataset.getId());
            parent.setPid("S0");
            parent.setText(adapterDataset.getStdDatasetName());
            if(adapterDataset.getNeedCrawer()!=null && adapterDataset.getNeedCrawer()==1){
                i++;
                parent.setIschecked(true);
                //添加到右侧树形列表
                TreeView rightParent = new TreeView();
                rightParent.setId("AD" + adapterDataset.getId());
                rightParent.setPid("A0");
                rightParent.setText(adapterDataset.getStdDatasetName());
                rightParent.setIschecked(true);
                adapterCustomizeList.add(rightParent);
            }else{
                parent.setIschecked(false);
            }
            adapterCustomizeList.add(parent);
            Long belongDatasetId = adapterDataset.getId();
            List<AdapterMetadataModel> adapterMetaDataList = adapterMetadataService.searchPage(AdapterMetadataModel.class, AdapterVersion.MetaDataTablePrefix,adapterVersion, "belongAdapterId="+belongDatasetId, null, null, null,false) ;
            //数据元
            for (AdapterMetadataModel adapterMetadat : adapterMetaDataList) {
                TreeView child = new TreeView();
                child.setId("SM" + adapterMetadat.getId());
                child.setPid("SD" + belongDatasetId);
                child.setText(adapterMetadat.getStdMetadataName());
                if(adapterMetadat.getNeedCrawer()!= null && adapterMetadat.getNeedCrawer()==1){
                    child.setIschecked(true);

                    //添加到右侧树形列表
                    TreeView rightChild = new TreeView();
                    rightChild.setId("AM" + adapterMetadat.getId());
                    rightChild.setPid("AD" + adapterDataset.getId());
                    rightChild.setText(adapterMetadat.getStdMetadataName());
                    rightChild.setIschecked(true);
                    adapterCustomizeList.add(rightChild);
                } else {
                    child.setIschecked(false);
                }
                adapterCustomizeList.add(child);
            }
        }

        //添加左侧根节点
        TreeView leftParent = new TreeView();
        leftParent.setId("S0");
        leftParent.setPid("-1");
        leftParent.setText("数据集");
        if(i >= 1){
            //首先添加右侧根节点
            TreeView adapterRoot = new TreeView();
            adapterRoot.setId("A0");
            adapterRoot.setPid("-1");
            adapterRoot.setText("数据集");
            adapterRoot.setIschecked(true);
            adapterCustomizeList.add(adapterRoot);
        }
        if (i == adapterDataSetList.size()) {
            leftParent.setIschecked(true);
        } else {
            leftParent.setIschecked(false);
        }

        adapterCustomizeList.add(leftParent);

        return adapterCustomizeList;
    }

    public void getAdapterCustomize(String adapterVersion, List<TreeView> treeViewList, List<Long> hasCheckDatasetIdList, List<Long> hasCheckMetadataIdList) {
        boolean adapter = false;  //定制是否添加根节点
        //获取所有定制数据集
        AdapterVersion aVersion = new AdapterVersion(adapterVersion);
        List<AdapterDatasetModel> adapterDataSetList = getList(AdapterDatasetModel.class, aVersion.getDataSetTableName(), "", "", null, null);
        List<AdapterMetadataModel> adapterMetaDataList = getList(AdapterMetadataModel.class, aVersion.getMetaDataTableName(), "", "", null, null);
        List<TreeView> adapterCustomizeList = new ArrayList<>();
        //数据集
        for (AdapterDatasetModel adapterDataset : adapterDataSetList) {
            TreeView parent = new TreeView();
            parent.setId("AD" + adapterDataset.getStdDatasetId());
            parent.setPid("A0");
            parent.setText(adapterDataset.getStdDatasetName());
            parent.setIschecked(true);
            adapterCustomizeList.add(parent);
            adapter = true;
            hasCheckDatasetIdList.add(adapterDataset.getStdDatasetId());
        }
        //数据元
        for (AdapterMetadataModel adapterMetadat : adapterMetaDataList) {
            TreeView child = new TreeView();
            child.setId("AM" + adapterMetadat.getStdMetadataId());
            child.setPid("AD" + adapterMetadat.getStdDatasetId());
            child.setText(adapterMetadat.getStdMetadataName());
            child.setIschecked(true);
            adapterCustomizeList.add(child);
            adapter = true;
            hasCheckMetadataIdList.add(adapterMetadat.getStdMetadataId());
        }
        //根节点
        if (adapter) {
            TreeView adapterRoot = new TreeView();
            adapterRoot.setId("A0");
            adapterRoot.setPid("-1");
            adapterRoot.setText("数据集");
            adapterRoot.setIschecked(true);
            adapterCustomizeList.add(adapterRoot);
        }
        treeViewList.addAll(adapterCustomizeList);
    }

    public void getStdCustomize(String stdVersion, List<TreeView> treeViewList, List<Long> hasCheckDatasetIdList, List<Long> hasCheckMetadataIdList) {
        String id;
        boolean check;     //是否勾选
        boolean std = false;      //标准是否添加根节点
        long childCheckCount;
        long datasetCount = 0;

        //获取所有标准数据集
        StandardVersion sVersion = new StandardVersion(stdVersion);
        List<StdDataSetModel> dataSetList = getList(StdDataSetModel.class, sVersion.getDataSetTableName(), "", "", null, null);
        String idListStr = SqlConstants.EMPTY;
        for (StdDataSetModel dataSet : dataSetList) {
            idListStr += dataSet.getId() + SqlConstants.COMMA;
        }
        Map<Long, List<StdMetaDataModel>> stdDataSetModelMap = new HashMap<>();
        if (!StringUtil.isEmpty(idListStr)) {
            String tableName = sVersion.getMetaDataTableName();
            SqlCreator sqlCreator = new SqlCreator(StdMetaDataModel.class);
            String sql = sqlCreator.selectData(tableName);
            List<StdMetaDataModel> stdMetaDataModelList = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(StdMetaDataModel.class));
            for (StdMetaDataModel stdMetaDataModel : stdMetaDataModelList) {
                List<StdMetaDataModel> stdMetaDataModels = stdDataSetModelMap.get(stdMetaDataModel.getDataSetId());
                if (stdMetaDataModels == null) {
                    stdMetaDataModels = new ArrayList<>();
                }
                stdMetaDataModels.add(stdMetaDataModel);
                stdDataSetModelMap.put(stdMetaDataModel.getDataSetId(), stdMetaDataModels);
            }
        }

        List<TreeView> stdCustomizeList = new ArrayList<>();

        for (StdDataSetModel dataSet : dataSetList) {
            TreeView parent = new TreeView();
            parent.setId("SD" + dataSet.getId());
            parent.setPid("S0");
            parent.setText(dataSet.getName());
            parent.setIschecked(false);
            std = true;
            childCheckCount = 0;
            List<StdMetaDataModel> metaDataList = stdDataSetModelMap.get(dataSet.getId());
            if (!CollectionUtils.isEmpty(metaDataList)) {
                for (StdMetaDataModel metaData : metaDataList) {
                    id = String.valueOf(metaData.getId());
                    if (hasCheckMetadataIdList.contains(metaData.getId())) {
                        check = true;
                        childCheckCount++;
                    } else {
                        check = false;
                    }
                    TreeView child = new TreeView();
                    child.setId("SM" + id);
                    child.setPid("SD" + dataSet.getId());
                    child.setText(metaData.getName());
                    child.setIschecked(check);
                    stdCustomizeList.add(child);
                }
                std = true;
            }
            if (!CollectionUtils.isEmpty(metaDataList) && metaDataList.size() == childCheckCount && childCheckCount > 0) {
                parent.setIschecked(true);//子节点全选
                datasetCount++;
            } else if (CollectionUtils.isEmpty(metaDataList) && hasCheckDatasetIdList.contains(dataSet.getId())) {
                parent.setIschecked(true);
                datasetCount++;
            }
            stdCustomizeList.add(parent);
        }
        //根节点
        if (std) {
            TreeView stdRoot = new TreeView();
            stdRoot.setId("S0");
            stdRoot.setPid("-1");
            stdRoot.setText("数据集");
            if (datasetCount == dataSetList.size()) {
                stdRoot.setIschecked(true);
            } else {
                stdRoot.setIschecked(false);
            }

            stdCustomizeList.add(stdRoot);
        }
        treeViewList.addAll(stdCustomizeList);
    }

    public ListEnvelop getDatasetCustomize(String adapterVersion, String stdVersion) {
        try {
            List<TreeView> treeViewList = getAdapterCustomize(adapterVersion);
            ListEnvelop detailModelResult = ListEnvelop.getSuccess("获取定制数据集列表成功",treeViewList);
            return detailModelResult;
        } catch (Exception e) {
            e.printStackTrace();
            return ListEnvelop.getError("获取定制数据集列表失败");
        }
    }

    public Envelop cloneDataSet(String adapterVersion,String code) throws IOException {
        Envelop envelop = new Envelop();
        AdapterDatasetModel adapterDataset = null;
        List<AdapterDatasetModel> adapterDatasets = getAdapterDatasetByCode(adapterVersion, code);
        if (adapterDatasets != null && adapterDatasets.size()>0) {
            adapterDataset = adapterDatasets.get(0);
        }else{
            envelop.setStatus(-1);
            envelop.setMessage("数据集不存在");
            return envelop;
        }
        if(adapterDataset==null){
            envelop.setStatus(-1);
            envelop.setMessage("数据集不存在");
            return envelop;
        }
        AdapterDatasetModel clone_adapter_dataset = new AdapterDatasetModel();
        clone_adapter_dataset.setSchemeId(adapterDataset.getSchemeId());
        clone_adapter_dataset.setStdDatasetId(adapterDataset.getStdDatasetId());
        clone_adapter_dataset.setStdDatasetCode(adapterDataset.getStdDatasetCode());
        clone_adapter_dataset.setStdDatasetName(adapterDataset.getStdDatasetName());
        clone_adapter_dataset.setIsClone(1);
        clone_adapter_dataset.setNeedCrawer(1);
        ObjEnvelop add = add(adapterVersion, clone_adapter_dataset);
        if(add.getStatus()!=200){
            envelop.setStatus(-1);
            envelop.setMessage("数据集克隆失败");
            return envelop;
        }
        List<AdapterMetadataModel> adapterMetadatas = adapterMetadataService.searchPage(AdapterMetadataModel.class,"adapter_metadata",adapterVersion,"belongAdapterId="+adapterDataset.getStdDatasetId(),null,1,9999,false);
        if(adapterMetadatas==null){
            envelop.setStatus(-1);
            envelop.setMessage("数据元不存在");
            return envelop;
        }
        AdapterDatasetModel clone_dataset = (AdapterDatasetModel)add.getObj();
        List<String> sqls = new ArrayList<>();
        AdapterVersion version = new AdapterVersion(adapterVersion);
        String dictIds = "";
        for(AdapterMetadataModel medata:adapterMetadatas){
            AdapterMetadataModel adapterMetadataModel = new AdapterMetadataModel();
            adapterMetadataModel.setSchemeId(medata.getSchemeId());
            adapterMetadataModel.setStdDatasetId(medata.getStdDatasetId());
            adapterMetadataModel.setStdMetadataId(medata.getStdMetadataId());
            adapterMetadataModel.setStdMetadataCode(medata.getStdMetadataCode());
            adapterMetadataModel.setStdMetadataName(medata.getStdMetadataName());
            Long stdDictId = medata.getStdDictId();
            adapterMetadataModel.setStdDictId(stdDictId);
            if(stdDictId != null && stdDictId!=0){
                dictIds+=","+stdDictId;
            }
            adapterMetadataModel.setBelongAdapterId(clone_dataset.getId());
            adapterMetadataModel.setNeedCrawer(1);
            String sql = getInsertSql(version, adapterMetadataModel);
            sqls.add(sql);
        }
        if (sqls.size() > 0) {
            adapterMetadataService.insertBatch(sqls);
        }
        if(!StringUtils.isEmpty(dictIds)){
            jdbcTemplate.execute("update "+new AdapterVersion(adapterVersion).getDictTableName() +" set need_crawer=1 where id in ("+dictIds.substring(1,dictIds.length())+")");
        }
        return Envelop.getSuccess("克隆成功");
    }

    public String getInsertSql(AdapterVersion version, AdapterMetadataModel medata) throws IOException {
        SqlCreator sqlCreator = new SqlCreator(AdapterMetadataModel.class);
        JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(medata));
        String sql = sqlCreator.insertDataToSQL(version.getMetaDataTableName(), jsonNode);
        return sql;
    }

    public Envelop isNeedRelation(String adapterVersion, Long id) {
        Envelop envelop = new Envelop();
        AdapterVersion version = new AdapterVersion(adapterVersion);
        String sql = "select DISTINCT adapter_dataset_id from "+ version.getMetaDataTableName() +" where adapter_metadata_code is not null and adapter_dataset_id is not null and belong_adapter_id = ?";
        List<Long> adapterIds = jdbcTemplate.queryForList(sql,Long.class, id);
        if(adapterIds!=null&&adapterIds.size()>=2){
            envelop.setStatus(200);
        }else{
            envelop.setStatus(-1);
        }
        return envelop;

    }

    public Envelop updateCustomize(Long schemeId, String adapterVersion, String stdVersion, String datasetIdStr, String metaDataIdStr) {
        AdapterVersion aVersion = new AdapterVersion(adapterVersion);
        StandardVersion sVersion = new StandardVersion(stdVersion);
        try {
            String[] datasetIdList = datasetIdStr.split(",");
            String[] metadataIdList = metaDataIdStr.split(",");
            List<Long> newDatasetIdList = new ArrayList<>();
            List<Long> newMetadataIdList = new ArrayList<>();

            for (String datasetId : datasetIdList) {
                if (!StringUtil.isStrEmpty(datasetId)) {
                    Long newDatasetId = Long.parseLong(datasetId);
                    newDatasetIdList.add(newDatasetId);
                }
            }

            for (String metadataId : metadataIdList) {
                if (!StringUtil.isStrEmpty(metadataId)) {
                    Long newMetadataId = Long.parseLong(metadataId);
                    newMetadataIdList.add(newMetadataId);
                }
            }

            if (newDatasetIdList.size() == 0) {
                return Envelop.getError("定制数据集失败");
            }

            updateDatasetCustomizeData(schemeId, aVersion, sVersion, newDatasetIdList, newMetadataIdList);
            return Envelop.getSuccess("定制数据集成功");
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("定制数据集失败");
        }
    }

    public void updateDatasetCustomizeData(Long schemeId, AdapterVersion aVersion, StandardVersion sVersion, List<Long> newDatasetIdList, List<Long> newMetadataIdList) throws Exception {
        //首先把适配数据集和适配的数据元,字典 全部设置成不需要采集
        String dataSetTableName = aVersion.getDataSetTableName();
        String metaDataTableName = aVersion.getMetaDataTableName();
        String dictName = aVersion.getDictTableName();
        jdbcTemplate.execute("update "+dataSetTableName +" set need_crawer=0");
        jdbcTemplate.execute("update "+metaDataTableName +" set need_crawer=0");
        jdbcTemplate.execute("update "+dictName +" set need_crawer=0");

        //设置需要采集的数据集
        if(CollectionUtils.isNotEmpty(newDatasetIdList)){
            String ids="";
            for(Long id:newDatasetIdList){
                ids+=","+id;
            }
            jdbcTemplate.execute("update "+dataSetTableName +" set need_crawer=1 where id in ("+ids.substring(1,ids.length())+")");
        }

        String metadaIds = "";
        //设置需要采集的数据元
        if(CollectionUtils.isNotEmpty(newMetadataIdList)){
            for(Long id:newMetadataIdList){
                metadaIds+=","+id;
            }
            metadaIds = metadaIds.substring(1, metadaIds.length());
            jdbcTemplate.execute("update "+metaDataTableName +" set need_crawer=1 where id in ("+metadaIds+")");
        }

        if(!StringUtils.isEmpty(metadaIds)){
            //设置需要采集的字典
            List<Long> dictIds = jdbcTemplate.queryForList("select DISTINCT  std_dict_id from " + metaDataTableName + " where std_dict_id is not null and std_dict_id!='' and id in (" + metadaIds + ")",Long.class);
            if(CollectionUtils.isNotEmpty(dictIds)){
                String ids = "";
                for(Long id:dictIds){
                    ids+=","+id;
                }
                jdbcTemplate.execute("update "+dictName +" set need_crawer=1 where id in ("+ids.substring(1,ids.length())+")");
            }
        }

    }

    private String delData(String adapterTableName, String item, List<Long> idList) {
        StringBuffer sql = new StringBuffer();
        String idStr = SqlConstants.EMPTY;
        for (Long id : idList) {
            idStr += id + SqlConstants.COMMA;
        }
        if (!StringUtil.isEmpty(idStr)) {
            idStr = SqlConstants.LEFT_BRACKET + idStr.substring(0, idStr.length() - 1) + SqlConstants.RIGHT_BRACKET;
        }
        sql.append(SqlConstants.DELETE + SqlConstants.FROM).append(adapterTableName).append(SqlConstants.WHERE).append(item).append(SqlConstants.IN).append(idStr);
        return sql.toString();
    }

    /**
     * 将第一个数组放入map(id,1) 然后循环第二个数组，如果map中找到对应键值就+1  不存在设置 1
     *
     * @param firtList   new
     * @param secondList old
     */
    private Map<String, List<Long>> getDiffId(List<Long> firtList, List<Long> secondList) {
        Map<Long, Integer> map = new HashMap<>();
        for (Long id : firtList) {
            map.put(id, 1);
        }

        for (Long id1 : secondList) {
            Integer flag = map.get(id1);
            if (flag != null) {
                map.put(id1, ++flag);
            } else {
                map.put(id1, 1);
            }
        }


        List<Long> diffLs = new ArrayList<>();
        for (Map.Entry<Long, Integer> entry : map.entrySet()) {
            if (entry.getValue() == 1) {
                diffLs.add(entry.getKey());
            }
        }

        List<Long> delIDLs = new ArrayList<>();
        List<Long> addIDLs = new ArrayList<>();
        for (Long diffL : diffLs) {
            if (firtList.contains(diffL)) {
                addIDLs.add(diffL);
            } else if (secondList.contains(diffL)) {
                delIDLs.add(diffL);
            }
        }
        Map<String, List<Long>> listMap = new HashMap<>();
        listMap.put("del", delIDLs);
        listMap.put("add", addIDLs);
        return listMap;
    }

    private String insertData(String adapterTableName, String stdTableName, String[] item1, String[] item2, Long schemeId, String item, List<Long> idList) {
        StringBuffer sql = new StringBuffer();
        String idStr = SqlConstants.EMPTY;
        for (Long id : idList) {
            idStr += id + SqlConstants.COMMA;
        }
        if (!StringUtil.isEmpty(idStr)) {
            idStr = SqlConstants.LEFT_BRACKET + idStr.substring(0, idStr.length() - 1) + SqlConstants.RIGHT_BRACKET;
        }

        sql.append(SqlConstants.INSERT_INTO + adapterTableName + SqlConstants.LEFT_BRACKET);
        for (String it : item1) {
            sql.append(it + SqlConstants.COMMA);
        }
        sql.append("scheme_id" + SqlConstants.RIGHT_BRACKET + SqlConstants.LEFT_BRACKET + SqlConstants.SELECT);
        for (String it : item2) {
            sql.append(it + SqlConstants.COMMA);
        }
        sql.append(schemeId + SqlConstants.FROM + stdTableName + SqlConstants.WHERE + item + SqlConstants.IN + idStr + SqlConstants.RIGHT_BRACKET);
        return sql.toString();
    }

    public List selectData(String adapterTableName, Class clazz) throws Exception {
        StringBuffer sql = new StringBuffer();
        sql.append("SELECT * FROM ").append(adapterTableName);
        return queryListBySql(sql.toString(), clazz);
    }

    public List getAdapterDatasetNotNullList(AdapterVersion adapterVersion, String condition) {
        try {
            StringBuffer sql = new StringBuffer();
            sql.append("SELECT * FROM ").append(adapterVersion.getDataSetTableName());
            if (!StringUtil.isEmpty(condition)) {
                JsonNode jsonNode = objectMapper.readTree(condition);
                String column = jsonNode.get("column").asText();
                sql.append(" WHERE ").append(column).append(" IS NOT NULL");
            }
            Query query = getCurrentSession().createSQLQuery(sql.toString());
            query.setResultTransformer(new BeanTransformer(AdapterDatasetModel.class));
            return query.list();
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public List getListByAdapterDatasetIdList(AdapterVersion adapterVersion, List<Integer> datasetIdList) {
        try {
            if (CollectionUtils.isEmpty(datasetIdList)) {
                return null;
            }
            SqlCreator sqlCreator = new SqlCreator(AdapterDatasetModel.class);
            sqlCreator.inCondition("stdDatasetId", datasetIdList);
            String sql = sqlCreator.selectData(adapterVersion.getDataSetTableName());
            return jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(AdapterDatasetModel.class));
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public List<AdapterDatasetModel> getAdapterDatasetByCode(String version, String datasetCode) {
        if (StringUtil.isEmpty(datasetCode)) {
            return null;
        }
        SqlCreator sqlCreator = new SqlCreator(AdapterDatasetModel.class);
        sqlCreator.equalCondition("stdDatasetCode", datasetCode);
        String sql = sqlCreator.selectData("adapter_dataset_" + version);
        return jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(AdapterDatasetModel.class));
    }

    public AdapterDatasetModel getAdapterDatasetByCode(String schemaName, String version, String datasetCode) {
        try {
            if (StringUtil.isEmpty(datasetCode)) {
                return null;
            }
            SqlCreator sqlCreator = new SqlCreator(AdapterDatasetModel.class);
            sqlCreator.equalCondition("stdDatasetCode", datasetCode);
            String sql = sqlCreator.selectData("adapter_dataset_" + version);

            String myCatAnnotation = "/*#mycat:schema=" + schemaName + "*/ ";
            String completeSql = myCatAnnotation + sql;

            return jdbcTemplate.query(completeSql,new BeanPropertyRowMapper<>(AdapterDatasetModel.class)).get(0);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }


    public List<AdapterDatasetModel> getAdapterDatasetByAdapterIdList(AdapterVersion adapterVersion, List<Integer> datasetIdList) {
        try {
            if (CollectionUtils.isEmpty(datasetIdList)) {
                return null;
            }
            SqlCreator sqlCreator = new SqlCreator(AdapterDatasetModel.class);
            sqlCreator.inCondition("adapterDatasetId", datasetIdList);
            String sql = sqlCreator.selectData(adapterVersion.getDataSetTableName());
            return jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(AdapterDatasetModel.class));
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public void strategy(String schemaName,String version,String std_version,String adapter_std_version) throws Exception {
        List<AdapterMetadataModel> unAdapterMetadataModels =  adapterMetadataService.getAllUnAdaptMetadata(schemaName,version);//等待适配的标准数据元
        List<AdapterMetadataModel> adapterMetadataModel = new ArrayList<AdapterMetadataModel>();//之前已经适配好的适配方案
        //获取所有未匹配的
        //查找出  之前已经适配好的适配方案
        List<AdapterSchemeVersionModel> all = adapterSchemeVersionService.getAll(schemaName);
        if(all!=null){
            for(AdapterSchemeVersionModel adapterSchemeVersionModel:all){
                String vers = adapterSchemeVersionModel.getVersion();
                if(org.apache.commons.lang3.StringUtils.isNotBlank(vers)){
                    List<AdapterMetadataModel> adaptMetadata = adapterMetadataService.getAllAdaptMetadata(schemaName,adapterSchemeVersionModel.getVersion());
                    adapterMetadataModel.addAll(adaptMetadata);
                }
            }
        }
        MetadataMatchVO matchVO = new MetadataMatchVO();
        matchVO.setCodeAdapter(matchVO.geNameGroup(adapterMetadataModel));

        //設置匹配的机构数据緩存
        List<StdMetaDataModel> orgMetadatas = stdMetadataService.getList(schemaName,adapter_std_version, null, null, null, null, null);//查找出机构所有的metadatas
        for (StdMetaDataModel stdMetadata:orgMetadatas){
            StdDataSetModel stdDataset = stdDataSetModelDao.getDatasetName(schemaName,adapter_std_version, stdMetadata.getDataSetId());
            if(null != stdDataset){
                MetatdataStandardNameStrategy.getNameMap().put(stdMetadata.getName() + "_" + stdDataset.getName(),stdMetadata);
                //MetatdataStandardNameStrategy.getRemarkMap().put(stdMetadata.getDefinition() + "_" + stdDataset.getName(),stdMetadata);  //根据数据元定义去匹配,匹配度低,浪费性能, 注释掉
            }
        }

        //执行匹配
        MetadataStrategyExecute.setFirst(version, unAdapterMetadataModels.size());//设置第一次匹配
        Set<String> sqlList = new HashSet<>();

        for (AdapterMetadataModel unAdapterMetadataModel : unAdapterMetadataModels) {
            threadPoolTaskExecutor.execute(new MetadataStrategyExecute(schemaName,unAdapterMetadataModel, matchVO, std_version,adapter_std_version,version, stdMetaDataModelDao,stdDataSetModelDao,adapterMetadataService));

            String metadataNameFirstCode = GetChineseFirst.cn2py(unAdapterMetadataModel.getStdMetadataName());//待匹配
            List<AdapterMetadataModel> adapterMetadatas = matchVO.getCodeAdapter().get(metadataNameFirstCode);//已存在
            if (adapterMetadatas != null && adapterMetadatas.size() > 0) {
                //获取待适配的数据集名
                Long unAdaptStdId = unAdapterMetadataModel.getStdDatasetId();
                StdDataSetModel adapterDataset = stdDataSetModelDao.getDatasetName(schemaName,adapter_std_version, unAdaptStdId);
                if (adapterDataset != null) {
                    adapterDatasetService = SpringContext.getService(AdapterDatasetService.BEAN_ID);
                    AdapterDatasetModel entity = adapterDatasetService.getAdapterDatasetByCode(schemaName,version, adapterDataset.getCode());
                    String sqrSql = null;
                    if(null != entity){
                        sqrSql = adapterDatasetService.saveAdaptDatasetSql(schemaName,entity, adapterDataset, version);
                    }
                    if(null != sqrSql && !"".equals(sqrSql)){
                        sqlList.add(sqrSql);
                    }
                }
            }
        }

        if(CollectionUtils.isNotEmpty(sqlList)){
            List<String> list=new ArrayList<>();
            for (String s : sqlList) {
                list.add(s);
            }
            super.insertBatch(list);
        }
    }

    public void saveAdaptDataset(AdapterDatasetModel strategyDataset,StdDataSetModel orgDataset,String version) throws Exception {
        if(null != strategyDataset){
            strategyDataset.setAdapterDatasetId(orgDataset.getId()+"");
            strategyDataset.setAdapterDatasetCode(orgDataset.getCode());
            strategyDataset.setAdapterDatasetName(orgDataset.getName());
            JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(strategyDataset));
            SqlCreator sqlCreator = new SqlCreator(AdapterDatasetModel.class);
            String sql = sqlCreator.updateDataByTableKey(new AdapterVersion(version).getDataSetTableName(), jsonNode);
            Query query = getExeuteQuery(sqlCreator, sql);
            query.executeUpdate();
        }
    }

    @Transactional
    public String saveAdaptDatasetSql(String schemaName, AdapterDatasetModel strategyDataset, StdDataSetModel orgDataset, String version) throws Exception {
        StringBuilder sqlBuffer = new StringBuilder();
        sqlBuffer.append(SqlConstants.UPDATE + new AdapterVersion(version).getDataSetTableName() + SqlConstants.SET);
        sqlBuffer.append(" adapter_dataset_id = " + orgDataset.getId());
        sqlBuffer.append(",adapter_dataset_code = '" + orgDataset.getCode() + "'");
        sqlBuffer.append(",adapter_dataset_name = '" + orgDataset.getName() + "'");
        sqlBuffer.append(" where id=" + strategyDataset.getId());

        String myCatAnnotation = "/*#mycat:schema=" + schemaName + "*/ ";
        String completeSql = myCatAnnotation + sqlBuffer;

        return completeSql;
    }

    public Envelop deleteCloneDataSet(String adapterVersion, Long id) {
        Envelop envelop = new Envelop();
        AdapterVersion version = new AdapterVersion(adapterVersion);
        String dataSetTableName = version.getDataSetTableName();
        String metaDataTableName = version.getMetaDataTableName();
        String dictTableName = version.getDictTableName();
        //删除适配数据集
        jdbcTemplate.execute("delete from "+dataSetTableName +" where id="+id);

        //判断适配字典是否需要转成不需要采集状态
        List<Long> needCrawerIds = jdbcTemplate.queryForList("select DISTINCT  std_dict_id from " + metaDataTableName + " where std_dict_id is not null and std_dict_id!='' and need_crawer=1 and belong_adapter_id!="+id,Long.class);
        jdbcTemplate.execute("update "+ dictTableName +" set need_crawer=0");
        if(CollectionUtils.isNotEmpty(needCrawerIds)){
            String ids = "";
            for(Long _id:needCrawerIds){
                ids+=","+_id;
            }
            jdbcTemplate.execute("update "+dictTableName +" set need_crawer=1 where id in ("+ids.substring(1,ids.length())+")");
        }

        //删除适配的数据元
        jdbcTemplate.execute("delete from "+metaDataTableName +" where belong_adapter_id="+id);
        return Envelop.getSuccess("删除成功");
    }

    public Envelop getDataSets(String adapterVersion, Long id,String sourceVersion,Integer page,Integer rows ,String filterCodeOrName)throws IOException {
        AdapterVersion version = new AdapterVersion(adapterVersion);
        String sql = "select DISTINCT adapter_dataset_id from "+ version.getMetaDataTableName() +" where adapter_metadata_code is not null and adapter_dataset_id is not null and belong_adapter_id = ?";
        List<Long> adapterIds = jdbcTemplate.queryForList(sql,Long.class, id);
        String ids="";
        if(!CollectionUtils.isEmpty(adapterIds)){
            for(Long adapterId:adapterIds){
                ids+=","+adapterId;
            }
        }
        if(!StringUtil.isEmpty(ids)){
            ids=ids.substring(1,ids.length());
        }else{
            ListEnvelop.getSuccess("查询成功",null);
        }
        String condition = "id=" + ids+";";
        if(!StringUtils.isEmpty(filterCodeOrName)){
            condition+="code?"+filterCodeOrName+" g1;name?"+filterCodeOrName+" g1";
        }
        List std_datasets = searchPage(StdDataSetModel.class, "std_dataset", sourceVersion, condition, null, page, rows,true);
        return ListEnvelop.getSuccess("查询成功",std_datasets);
    }

    public Envelop smartMatchMedata(String std_version, String adapter_version, Long id, Long adapterDataSetId) throws Exception {
        //获取等待适配的数据元
        List<AdapterMetadataModel> unAdapterMedataModels= adapterMetadataService.getUnAdaptMetadataByid(adapter_version,id);

        //获取源数据元
        Map<String, String> order = new HashMap<>();
        Map<String, Object> query = new HashMap<>();
        query.put("dataSetId",adapterDataSetId);
        StandardVersion standardVersion = new StandardVersion(std_version);
        List<StdMetaDataModel> medataList = stdMetadataService.getList(standardVersion, null, query, order, null, null);

        if(medataList!=null && medataList.size()>0){
            LikeHashMap<String, StdMetaDataModel> nameMap = new LikeHashMap<String, StdMetaDataModel>();
            for(StdMetaDataModel medata:medataList){
                String code = medata.getCode();
                if(!StringUtil.isEmpty(code)){
                    code = code.trim();
                }
                String name = medata.getName();
                if(!StringUtil.isEmpty(name)){
                    name = name.trim();
                }
                nameMap.put(code+"@"+name,medata);
            }
            if(unAdapterMedataModels!=null && unAdapterMedataModels.size()>0){
                String schemaName = LocalContext.getContext().getAttachment(ContextAttributes.SCHEMA);
                //查询到的匹配字典项
                StdMetaDataModel stdMetadaModel = null;
                for(AdapterMetadataModel model:unAdapterMedataModels){
                    //查询到的匹配字典项
                    StdMetaDataModel stdMetaDataModel = null;

                    String code = model.getStdMetadataCode();
                    if(!StringUtil.isEmpty(code)){
                        code = code.trim();
                    }

                    String name = model.getStdMetadataName();
                    if(!StringUtil.isEmpty(name)){
                        name = name.trim();
                    }

                    //通过name做适配
                    stdMetadaModel = nameMap.get("@" + name, true);
                    if (stdMetadaModel != null) {
                        adapterMetadataService.saveAdapt(schemaName,model, stdMetadaModel, adapter_version,"1");//只匹配字典项code ,为疑似匹配
                    }else{ //通过code做匹配
                        stdMetadaModel = nameMap.get(code+"@", true);
                        if(stdMetadaModel!=null){
                            adapterMetadataService.saveAdapt(schemaName,model, stdMetadaModel, adapter_version,"1");//只匹配字典项code ,为疑似匹配
                        }
                    }
                }
            }
        }
        AdapterVersion adapterVersion = new AdapterVersion(adapter_version);
        adapterMetadataService.changeDatasetRelation(adapterVersion,id,std_version);
        return Envelop.getSuccess("匹配成功");

    }
}