package com.yihu.jw.basic.standard.service.adapter;


import com.fasterxml.jackson.databind.JsonNode;
import com.yihu.jw.basic.standard.dao.IStdDictionaryModelDao;
import com.yihu.jw.basic.standard.dao.StdDictionaryEntryModelDao;
import com.yihu.jw.basic.standard.match.dictionary.DictItemStrategyExecute;
import com.yihu.jw.basic.standard.match.dictionary.DictitemStandardNameStrategy;
import com.yihu.jw.basic.standard.match.matchModel.DictItemMatchVO;
import com.yihu.jw.basic.standard.model.adapter.AdapterDictEntryModel;
import com.yihu.jw.basic.standard.model.adapter.AdapterDictModel;
import com.yihu.jw.basic.standard.model.adapter.AdapterSchemeVersionModel;
import com.yihu.jw.basic.standard.model.standard.StdDictionaryEntryModel;
import com.yihu.jw.basic.standard.model.standard.StdDictionaryModel;
import com.yihu.jw.basic.standard.service.bo.AdapterVersion;
import com.yihu.jw.basic.standard.service.bo.StandardVersion;
import com.yihu.jw.basic.standard.service.standard.StdDictService;
import com.yihu.jw.basic.standard.dao.SQLGeneralDAO;
import com.yihu.jw.basic.util.LikeHashMap;
import com.yihu.jw.lang.SpringContext;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.util.common.StringUtil;
import com.yihu.jw.util.sql.SqlConstants;
import com.yihu.jw.util.sql.SqlCreator;
import com.yihu.jw.util.thread.ContextAttributes;
import com.yihu.jw.util.thread.LocalContext;
import org.hibernate.query.Query;
import org.hibernate.Session;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Transactional
@Service("AdapterDictService")
public class AdapterDictService extends SQLGeneralDAO {

    public static final String BEAN_ID = "AdapterDictService";

    @Autowired
    private AdapterDictEntryService adapterDictEntryService;

    @Autowired
    private AdapterSchemeVersionService adapterSchemeVersionService;

    @Autowired
    private ThreadPoolTaskExecutor threadPoolTaskExecutor;

    @Autowired
    private IStdDictionaryModelDao stdDictionaryDao;

    @Autowired
    private StdDictionaryEntryModelDao stdDictionaryEntryDao;

    @Autowired
    private StdDictService stdDictService;

    private AdapterDictService adapterDictService;

    public AdapterDictService() {
    }

    public void finalize() throws Throwable {
        super.finalize();
    }

    public ObjEnvelop add(String adapterVersion, String dict) {
        try {
            AdapterVersion version = new AdapterVersion(adapterVersion);
            AdapterDictModel adapterDictModel = objectMapper.readValue(dict, AdapterDictModel.class);
            Session session = getCurrentSession();
            String sql = "select max(id) from " + version.getMetaDataTableName();
            Query query = session.createSQLQuery(sql);
            Object object = query.uniqueResult();
            adapterDictModel.setId(object == null ? 1L : Long.parseLong(object.toString()) + 1);

            JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(adapterDictModel));
            SqlCreator sqlCreator = new SqlCreator(AdapterDictModel.class);
            sql = sqlCreator.insertData(version.getMetaDataTableName(), jsonNode);
            query = getExeuteQuery(sqlCreator, sql);
            query.executeUpdate();
            return ObjEnvelop.getSuccess("保存适配字典成功",adapterDictModel);
        } catch (Exception e) {
           return ObjEnvelop.getError("保存适配字典失败");
        }
    }

    public Envelop delete(String adapterVersion, Integer dictId) {
        try {
            if (adapterVersion == null || adapterVersion.length() == 0) {
                return Envelop.getError("无效适配版本");
            }
            AdapterVersion version = new AdapterVersion(adapterVersion);
            Session session = getCurrentSession();
            String sql = "delete from " + version.getMetaDataTableName() + " where id = :id";
            Query query = session.createSQLQuery(sql);
            query.setInteger("id", dictId);
            query.executeUpdate();
            return Envelop.getSuccess("删除适配字典成功");
        } catch (Exception e) {
            return Envelop.getError("删除适配字典失败");
        }
    }

    public ObjEnvelop modify(String adapterVersion, String dict) {
        try {
            AdapterVersion version = new AdapterVersion(adapterVersion);
            AdapterDictModel adapterDictModel = objectMapper.readValue(dict, AdapterDictModel.class);
            JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(adapterDictModel));

            SqlCreator sqlCreator = new SqlCreator(AdapterDictModel.class);
            String sql = sqlCreator.updateDataByTableKey(version.getMetaDataTableName(), jsonNode);
            Query query = getExeuteQuery(sqlCreator, sql);
            query.executeUpdate();
            return ObjEnvelop.getSuccess("修改适配字典成功",adapterDictModel);
        } catch (Exception e) {
            return ObjEnvelop.getError("修改适配字典失败");
        }
    }


    public PageEnvelop getDetailResult(String adapterVersion, String condition, String order, Integer rows, Integer page) {
        try {
            List dictList = getDictList(AdapterDictModel.class, adapterVersion, condition, order, rows, page);
            Integer count = getDictInt(AdapterDictModel.class, adapterVersion, condition);
            PageEnvelop result = PageEnvelop.getSuccess("获取适配字典列表成功");
            result.setDetailModelList(dictList);
            result.setTotalCount(count);
            return result;
        } catch (Exception e) {
            return PageEnvelop.getError("获取适配字典列表成功");
        }
    }

    public ObjEnvelop get(String version, Integer dictId) {
        try {
            AdapterVersion adapterVersion = new AdapterVersion(version);
            AdapterDictModel adapterDictModel = (AdapterDictModel) get(AdapterDictModel.class, adapterVersion.getDictTableName(), dictId);
            return ObjEnvelop.getSuccess("获取适配字典成功",adapterDictModel);
        } catch (Exception e) {
            return ObjEnvelop.getError("获取适配字典失败");
        }
    }

    public List getDictList(Class tClass, String adapterVersion, String condition, String order, Integer limit, Integer offset) {
        AdapterVersion version = new AdapterVersion(adapterVersion);
        String tableName = version.getDictTableName();
        SqlCreator sqlCreator = new SqlCreator(tClass);
        try {
            if (!StringUtil.isEmpty(condition)) {
                JsonNode jsonNode = objectMapper.readTree(condition);
                String name = jsonNode.get("name").asText();
                sqlCreator.likeOrCondition("stdDictCode", "stdDictName", name);
            }
            if (!StringUtil.isEmpty(order)) {
                JsonNode jsonNode = objectMapper.readTree(order);
                Iterator<String> fieldNames = jsonNode.fieldNames();
                while (fieldNames.hasNext()) {
                    String fieldName = fieldNames.next();
                    String value = jsonNode.get(fieldName).asText().toUpperCase();
                    sqlCreator.order(fieldName, value);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        String sql = sqlCreator.selectData(tableName);
        if (limit != null) {
            if (offset != null &&offset>0) {
                sql += " limit "+(offset - 1) * limit+","+limit;
            }else {
                sql += " limit "+limit;
            }

        }
        return jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(tClass));
    }

    public Integer getDictInt(Class tClass, String adapterVersion, String condition) {
        AdapterVersion version = new AdapterVersion(adapterVersion);
        String tableName = version.getDictTableName();
        SqlCreator sqlCreator = new SqlCreator(tClass);
        try {
            if (!StringUtil.isEmpty(condition)) {
                JsonNode jsonNode = objectMapper.readTree(condition);
                String name = jsonNode.get("name").asText();
                sqlCreator.likeOrCondition("stdDictCode", "stdDictName", name);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        String sql = sqlCreator.countData(tableName);
        Integer count = jdbcTemplate.queryForObject(sql,Integer.class);
        return count;
    }
    public void strategy(String schemaName, String version,String std_version, String adapter_std_version) throws Exception {
        List<AdapterDictEntryModel> unAdapterDictEntryModels= adapterDictEntryService.getAllUnAdaptDictEntry(schemaName,version);//等待适配的标准字典项
        List<AdapterDictEntryModel> adapterDictEntryModels = new ArrayList<AdapterDictEntryModel>();//之前已经适配好的适配方案
        //查找出  之前已经适配好的适配方案
        List<AdapterSchemeVersionModel> all = adapterSchemeVersionService.getAll(schemaName);//
        if(all!=null){
            for(AdapterSchemeVersionModel adapterSchemeVersionModel:all){
                String vers = adapterSchemeVersionModel.getVersion();
                if(org.apache.commons.lang3.StringUtils.isNotBlank(vers)){
                    List<AdapterDictEntryModel> adaptMetadata = adapterDictEntryService.getAllAdaptDictEntry(schemaName,adapterSchemeVersionModel.getVersion());
                    adapterDictEntryModels.addAll(adaptMetadata);
                }
            }
        }
        DictItemMatchVO matchVO= new DictItemMatchVO();
        matchVO.setCodeAdapter(matchVO.geNameGroup(adapterDictEntryModels));

        //設置匹配的机构数据緩存
        StandardVersion standardVersion = new StandardVersion(adapter_std_version);
        List<StdDictionaryEntryModel> dictItems = stdDictService.getDicEntryList(schemaName,standardVersion, null, null, null, null);//查找源机构字典子项

        for (StdDictionaryEntryModel stdDictItem:dictItems){
            StdDictionaryModel dictionary = stdDictionaryDao.getDictionaryName(schemaName,adapter_std_version, stdDictItem.getDictId());//查找源机构字典项
            if(dictionary != null){
                DictitemStandardNameStrategy.getNameMap().put(stdDictItem.getCode()+"_"+stdDictItem.getValue() + "_" + dictionary.getName(),stdDictItem);//字典子项名_字典名
            }
        }

        //执行匹配
        DictItemStrategyExecute.setFirst(version, unAdapterDictEntryModels.size());//设置第一次匹配

        Set<String> sqlList = new HashSet<String>();

        for (AdapterDictEntryModel unAdapterDictEntryModel : unAdapterDictEntryModels) {
            threadPoolTaskExecutor.execute(new DictItemStrategyExecute(schemaName,unAdapterDictEntryModel, matchVO,std_version,adapter_std_version,version, stdDictionaryDao, stdDictionaryEntryDao,adapterDictEntryService));

            Long unAdaptDicId = unAdapterDictEntryModel.getStdDictId();
            StdDictionaryModel adapterDictionaryModel = stdDictionaryDao.getDictionaryName(schemaName,adapter_std_version, unAdaptDicId);
            if (adapterDictionaryModel != null) {
                adapterDictService = SpringContext.getService(AdapterDictService.BEAN_ID);
                AdapterDictModel entity = adapterDictService.getAdapterDictByCode(schemaName,version, adapterDictionaryModel.getCode());
                String sqrSql = null;
                if (null != entity) {
                    sqrSql = adapterDictService.saveAdaptDictSql(schemaName,entity, adapterDictionaryModel, version);
                }
                if (null != sqrSql && !"".equals(sqrSql)) {
                    sqlList.add(sqrSql);
                }
            }
        }
    }

    public void saveAdaptDict(AdapterDictModel strategyDict,StdDictionaryModel orgDict,String version) throws Exception {
        strategyDict.setAdapterDictId(orgDict.getId());
        strategyDict.setAdapterDictCode(orgDict.getCode());
        strategyDict.setAdapterDictName(orgDict.getName());
        JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(strategyDict));
        SqlCreator sqlCreator = new SqlCreator(AdapterDictModel.class);
        String sql = sqlCreator.updateDataByTableKey(new AdapterVersion(version).getDictTableName(), jsonNode);
        Query query = getExeuteQuery(sqlCreator, sql);
        query.executeUpdate();
    }


    public String saveAdaptDictSql(String schemaName, AdapterDictModel strategyDict, StdDictionaryModel orgDict, String version) throws Exception {
        StringBuilder sqlBuffer = new StringBuilder();
        sqlBuffer.append(SqlConstants.UPDATE + new AdapterVersion(version).getDictTableName() + SqlConstants.SET);
        sqlBuffer.append(" adapter_dict_id = " + orgDict.getId());
        sqlBuffer.append(",adapter_dict_code = '" + orgDict.getCode() + "'");
        sqlBuffer.append(",adapter_dict_name = '" + orgDict.getName() + "'");
        sqlBuffer.append(" where id=" + strategyDict.getId());

        String myCatAnnotation = "/*#mycat:schema=" + schemaName + "*/ ";
        String completeSql = myCatAnnotation + sqlBuffer.toString();

        return completeSql;
    }

    public AdapterDictModel getAdapterDictByCode(String schemaName, String version, String dictCode) {
        try {
            if (StringUtil.isEmpty(dictCode)) {
                return null;
            }
            SqlCreator sqlCreator = new SqlCreator(AdapterDictModel.class);
            sqlCreator.equalCondition("stdDictCode", dictCode);
            String sql = sqlCreator.selectData("adapter_dict_" + version);

            String myCatAnnotation = "/*#mycat:schema=" + schemaName + "*/ ";
            String completeSql = myCatAnnotation + sql;

            return jdbcTemplate.query(completeSql,new BeanPropertyRowMapper<>(AdapterDictModel.class)).get(0);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public Envelop smartMatch(String std_version, String adapter_version, Long stdDictId, Long adapterDictId) throws Exception {
        //获取等待适配的字典项
        List<AdapterDictEntryModel> unAdapterDictEntryModels= adapterDictEntryService.getUnAdaptDictEntryByDictId(adapter_version,stdDictId);

        //获取源字典项
        Map<String, Object> query = new HashMap<>();
        query.put("dictId",adapterDictId);
        List<StdDictionaryEntryModel> dicEntryList = stdDictService.getDicEntryList(new StandardVersion(std_version), query, null, null, null);


        if(dicEntryList!=null && dicEntryList.size()>0){
            LikeHashMap<String, StdDictionaryEntryModel> nameMap = new LikeHashMap<String, StdDictionaryEntryModel>();
            for(StdDictionaryEntryModel stdDictionaryEntryModel:dicEntryList){
                String code = stdDictionaryEntryModel.getCode();
                if(!StringUtil.isEmpty(code)){
                    code = code.trim();
                }
                String value = stdDictionaryEntryModel.getValue();
                if(!StringUtil.isEmpty(value)){
                    value = value.trim();
                }
                nameMap.put(code+"@"+value,stdDictionaryEntryModel);
            }
            if(unAdapterDictEntryModels!=null && unAdapterDictEntryModels.size()>0){
                String schemaName = LocalContext.getContext().getAttachment(ContextAttributes.SCHEMA);

                //通过code查询,连续十次查询不到,就不再通过code查询
                int selectByCode=0;
                for(AdapterDictEntryModel model:unAdapterDictEntryModels){
                    //查询到的匹配字典项
                    StdDictionaryEntryModel stdDictionaryEntryModel = null;

                    String code = model.getStdEntryCode();
                    if(!StringUtil.isEmpty(code)){
                        code = code.trim();
                    }
                    //code长度极长,很大概率是国家标准,直接通过code查询匹配,匹配度更高
                    //通过code查询,连续十次查询不到,就不再通过code查询(以防不是国家标准,浪费性能)
                    //if(!StringUtil.isEmpty(code) && code.length()>=5){
                    if(selectByCode<=10){
                        query.put("code",code);
                        List<StdDictionaryEntryModel> dictionaries = stdDictService.getDicEntryList(new StandardVersion(std_version), query, null, null, null);//源字典
                        if(dictionaries!=null && dictionaries.size()>0){
                            selectByCode=0;
                            if(dictionaries.size()==1){
                                stdDictionaryEntryModel = dictionaries.get(0);
                                adapterDictEntryService.saveAdapt(schemaName,model,stdDictionaryEntryModel , adapter_version,"2");
                                nameMap.remove(stdDictionaryEntryModel.getCode()+"@"+stdDictionaryEntryModel.getValue());
                            }else{
                                adapterDictEntryService.saveAdapt(schemaName,model, dictionaries.get(0), adapter_version,"1");
                            }
                            continue;
                        }else{
                            selectByCode++;
                        }
                    }
                    //}

                    //通过name做适配
                    String name = model.getStdEntryValue();
                    if(!StringUtil.isEmpty(name)){
                        name = name.trim();
                    }
                    stdDictionaryEntryModel = nameMap.get("@" + name, true);
                    if (stdDictionaryEntryModel != null) {
                        adapterDictEntryService.saveAdapt(schemaName,model, stdDictionaryEntryModel, adapter_version,"1");//只匹配字典项code ,为疑似匹配
                    }else{
                        if(!StringUtil.isEmpty(code) && code.length()>=3){//如果code长度足够长,很可能是国家标准规范,也可根据code进行匹配, 这边取>=3 ,若匹配效果不满意,可增大该值
                            stdDictionaryEntryModel = nameMap.get(code+"@", true);
                            if(stdDictionaryEntryModel!=null){
                                adapterDictEntryService.saveAdapt(schemaName,model, stdDictionaryEntryModel, adapter_version,"1");//只匹配字典项code ,为疑似匹配
                            }
                        }
                    }
                }
            }
        }
        return Envelop.getSuccess("匹配成功");
    }

}