package com.yihu.jw.basic.standard.service.adapter;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;

import com.yihu.jw.basic.standard.dao.SQLGeneralDAO;
import com.yihu.jw.basic.standard.model.adapter.AdapterDatasetModel;
import com.yihu.jw.basic.standard.model.adapter.AdapterMetadataModel;
import com.yihu.jw.basic.standard.model.adapter.AdapterSchemeModel;
import com.yihu.jw.basic.standard.model.adapter.resultModel.AdapterMetadataResultDetailModel;
import com.yihu.jw.basic.standard.model.adapter.resultModel.ApiMetadataResultDetailModel;
import com.yihu.jw.basic.standard.model.standard.StdDataSetModel;
import com.yihu.jw.basic.standard.model.standard.StdMetaDataModel;
import com.yihu.jw.basic.standard.service.bo.AdapterVersion;
import com.yihu.jw.basic.standard.service.bo.StandardVersion;
import com.yihu.jw.exception.ApiException;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.util.common.StringUtil;
import com.yihu.jw.util.sql.BeanTransformer;
import com.yihu.jw.util.sql.ErrorCode;
import com.yihu.jw.util.sql.SqlConstants;
import com.yihu.jw.util.sql.SqlCreator;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.hibernate.query.Query;
import org.hibernate.Session;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.util.*;
@Transactional
@Service("AdapterMetadataService")
public class AdapterMetadataService extends SQLGeneralDAO {

    public static final String BEAN_ID = "AdapterMetadataService";

    public AdapterMetadataService() {
    }

    public void finalize() throws Throwable {
        super.finalize();
    }

    public Envelop add(String adapterVersion, String metadata) throws Exception {
        try {
            AdapterVersion version = new AdapterVersion(adapterVersion);
            AdapterMetadataModel adapterMetadataModel = objectMapper.readValue(metadata, AdapterMetadataModel.class);
            Session session = getCurrentSession();
            String sql = "select max(id) from " + version.getMetaDataTableName();
            Query query = session.createSQLQuery(sql);
            Object object = query.uniqueResult();
            adapterMetadataModel.setId(object == null ? 1 : Long.parseLong(object.toString()) + 1);

            JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(adapterMetadataModel));
            SqlCreator sqlCreator = new SqlCreator(AdapterMetadataModel.class);
            sql = sqlCreator.insertData(version.getMetaDataTableName(), jsonNode);
            query = getExeuteQuery(sqlCreator, sql);
            query.executeUpdate();
            return ObjEnvelop.getSuccess("保存适配数据元成功",adapterMetadataModel);
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("保存适配数据元失败");
        }
    }

    public void delete(String adapterVersion, Integer metadataId) throws Exception {
        try {
            if (adapterVersion == null || adapterVersion.length() == 0) {
                throw new ApiException(ErrorCode.InvalidStdVersion.getErrorCode());
            }
            AdapterVersion version = new AdapterVersion(adapterVersion);
            Session session = getCurrentSession();
            String sql = "delete from " + version.getMetaDataTableName() + " where id = :id";
            Query query = session.createSQLQuery(sql);
            query.setInteger("id", metadataId);
            query.executeUpdate();
        } catch (Exception e) {
            throw new ApiException(ErrorCode.SaveMetaDataFailed.getErrorCode());
        }
    }

    public Envelop modify(String adapterVersion, String metadata) throws Exception {
        try {
            AdapterVersion version = new AdapterVersion(adapterVersion);
            AdapterMetadataModel adapterMetadataModel = objectMapper.readValue(metadata, AdapterMetadataModel.class);
            JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(adapterMetadataModel));

            SqlCreator sqlCreator = new SqlCreator(AdapterMetadataModel.class);
            String sql = sqlCreator.updateDataByTableKey(version.getMetaDataTableName(), jsonNode);
            Query query = getExeuteQuery(sqlCreator, sql);
            query.executeUpdate();
            return ObjEnvelop.getSuccess("修改适配数据元成功",adapterMetadataModel);
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("修改适配数据元失败");
        }
    }

    public Envelop get(String version, Integer dictEntryId) {
        try {
            AdapterVersion adapterVersion = new AdapterVersion(version);
            AdapterMetadataModel adapterMetadataModel = (AdapterMetadataModel) get(AdapterMetadataModel.class, adapterVersion.getDictTableName(), dictEntryId);
            return ObjEnvelop.getSuccess("获取适配数据元成功",adapterMetadataModel);
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("获取适配数据元失败");
        }
    }

    public Envelop getDetailResult(String adapterVersion, String condition, String order, Integer rows, Integer page) {
        try {
            List dictEntryList = getMetadataList(AdapterMetadataModel.class, adapterVersion, condition, order, rows, page);
            long count = getMetadataCount(AdapterMetadataModel.class, adapterVersion, condition);
            return PageEnvelop.getSuccessListWithPage("",dictEntryList,page,rows,count);
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("获取适配数据元列表失败");
        }
    }

    public List getMetadataList(Class tClass, String adapterVersion, String condition, String order, Integer limit, Integer offset) throws Exception {
        AdapterVersion version = new AdapterVersion(adapterVersion);
        String tableName = version.getMetaDataTableName();
        SqlCreator sqlCreator = new SqlCreator(tClass);

        if (!StringUtil.isEmpty(condition)) {
            try {
                JsonNode jsonNode = objectMapper.readTree(condition);
                String name = jsonNode.get("name").asText();
                if(jsonNode.get("stdDatasetId") != null){
                    sqlCreator.equalCondition("stdDatasetId", jsonNode.get("stdDatasetId").asInt());
                }
                JsonNode belongAdapterId = jsonNode.get("belongAdapterId");
                if(belongAdapterId!=null){
                    sqlCreator.equalCondition("belongAdapterId",belongAdapterId.asLong());
                }
                JsonNode needCrawerNode = jsonNode.get("needCrawer");
                if(needCrawerNode!=null){
                    sqlCreator.equalCondition("needCrawer",needCrawerNode.asLong());
                }
                sqlCreator.likeOrCondition("stdMetadataCode", "stdMetadataName", name);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        try {
            if (!StringUtil.isEmpty(order)) {
                JsonNode jsonNode = objectMapper.readTree(order);
                Iterator<String> fieldNames = jsonNode.fieldNames();
                while (fieldNames.hasNext()) {
                    String fieldName = fieldNames.next();
                    String value = jsonNode.get(fieldName).asText().toUpperCase();
                    sqlCreator.order(fieldName, value);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        String sql = sqlCreator.selectData(tableName);
        if (limit != null) {
            if (offset != null &&offset>0) {
                sql += " limit "+(offset - 1) * limit+","+limit;
            }else {
                sql += " limit "+limit;
            }
        }
        List<AdapterMetadataModel> modelList = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(AdapterMetadataModel.class));
        String ids = "";
        Map<Long, StdDataSetModel> stdDatasetModelMap = new HashMap<>();
        Long schemeId = null;
        for (AdapterMetadataModel model : modelList) {
            if (model.getAdapterDatasetId() != null) {
                schemeId = model.getSchemeId();
                ids+=","+model.getAdapterDatasetId();
            }
        }

        if (!StringUtil.isEmpty(ids)) {
            AdapterSchemeModel scheme = getEntity(AdapterSchemeModel.class, schemeId);
            StandardVersion standardVersion = new StandardVersion(scheme.getAdapterStdVersion());
            tableName = standardVersion.getDataSetTableName();
            sqlCreator = new SqlCreator(StdDataSetModel.class);
            sqlCreator.inCondition("id", ids.substring(1, ids.length()));
            sql = sqlCreator.selectData(tableName);
            List<StdDataSetModel> stdDatasetModelList = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(StdDataSetModel.class));
            for (StdDataSetModel model : stdDatasetModelList) {
                stdDatasetModelMap.put(model.getId(), model);
            }
        }

        List<ApiMetadataResultDetailModel> detailModelList = new ArrayList<>();
        for (AdapterMetadataModel model : modelList) {
            ApiMetadataResultDetailModel detailModel = new ApiMetadataResultDetailModel();
            detailModel.setId(model.getId());
            detailModel.setSchemeId(model.getSchemeId());
            detailModel.setAdapterDatasetId(model.getAdapterDatasetId());
            detailModel.setAdapterDictId(model.getAdapterDictId());
            detailModel.setAdapterDataType(model.getAdapterDataType());
            detailModel.setAdapterMetadataId(model.getAdapterMetadataId());
            detailModel.setAdapterMetadataCode(model.getAdapterMetadataCode());
            detailModel.setAdapterMetadataName(model.getAdapterMetadataName());
            detailModel.setAdapterDictId(model.getAdapterDictId());
            detailModel.setStdDictId(model.getStdDictId());
            detailModel.setStdDatasetId(model.getStdDatasetId());
            detailModel.setStdMetadataId(model.getStdMetadataId());
            detailModel.setStdMetadataCode(model.getStdMetadataCode());
            detailModel.setStdMetadataName(model.getStdMetadataName());
            detailModel.setAdapterInfo(model.getAdapterInfo());
            if (stdDatasetModelMap.get(model.getAdapterDatasetId()) != null) {
                detailModel.setAdapterDatasetCode(stdDatasetModelMap.get(model.getAdapterDatasetId()).getCode());
                detailModel.setAdapterDatasetName(stdDatasetModelMap.get(model.getAdapterDatasetId()).getName());
            }
            detailModel.setBelongAdapterId(model.getBelongAdapterId());
            detailModel.setAdapterDefault(model.getAdapterDefault());
            detailModelList.add(detailModel);
        }
        return detailModelList;
    }

    public Integer getMetadataCount(Class tClass, String adapterVersion, String condition) throws Exception {
        AdapterVersion version = new AdapterVersion(adapterVersion);
        String tableName = version.getMetaDataTableName();
        SqlCreator sqlCreator = new SqlCreator(tClass);
        if (!StringUtil.isEmpty(condition)) {
            try {
                JsonNode jsonNode = objectMapper.readTree(condition);
                String name = jsonNode.get("name").asText();
                sqlCreator.equalCondition("stdDatasetId", jsonNode.get("stdDatasetId").asInt());
                JsonNode belongAdapterId = jsonNode.get("belongAdapterId");
                if(belongAdapterId!=null){
                    sqlCreator.equalCondition("belongAdapterId",belongAdapterId .asLong());
                }
                JsonNode needCrawerNode = jsonNode.get("needCrawer");
                if(needCrawerNode!=null){
                    sqlCreator.equalCondition("needCrawer",needCrawerNode.asLong());
                }
                sqlCreator.likeOrCondition("stdMetadataCode", "stdMetadataName", name);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        String sql = sqlCreator.countData(tableName);
        return jdbcTemplate.queryForObject(sql,Integer.class);
    }

    public List getInfoList(String adapterVersion, String condition, String order, Integer limit, Integer offset) throws Exception{
        AdapterVersion version = new AdapterVersion(adapterVersion);
        List<AdapterMetadataModel> metadataModelList = getList(AdapterMetadataModel.class, version.getMetaDataTableName(), condition, order, limit, offset);
        List<Long> idList = new ArrayList<>();
        Map<Long, StdMetaDataModel> stdMetaDataModelMap = new HashMap<>();
        Long schemeId = null;
        for (AdapterMetadataModel model : metadataModelList) {
            if (model.getAdapterMetadataId() != null) {
                idList.add(model.getAdapterMetadataId());
                schemeId = model.getSchemeId();
            }
        }

        if (!CollectionUtils.isEmpty(idList)) {
            AdapterSchemeModel scheme = getEntity(AdapterSchemeModel.class, schemeId);
            StandardVersion standardVersion = new StandardVersion(scheme.getAdapterStdVersion());
            String tableName = standardVersion.getMetaDataTableName();
            SqlCreator sqlCreator = new SqlCreator(StdMetaDataModel.class);
            sqlCreator.inCondition("id", idList);
            String sql = sqlCreator.selectData(tableName);
            List<StdMetaDataModel> stdMetadataModelList = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(StdMetaDataModel.class));
            for (StdMetaDataModel model : stdMetadataModelList) {
                stdMetaDataModelMap.put(model.getId(), model);
            }
        }

        List<AdapterMetadataResultDetailModel> detailModelList = new ArrayList<>();
        for (AdapterMetadataModel model : metadataModelList) {
            if (StringUtils.isBlank(model.getAdapterMetadataCode()) || StringUtils.isBlank(model.getAdapterMetadataName())) {
                continue;
            }
            AdapterMetadataResultDetailModel detailModel = new AdapterMetadataResultDetailModel();
            detailModel.setId(model.getId());
            detailModel.setSchemeId(model.getSchemeId());
            detailModel.setAdapterDatasetId(model.getAdapterDatasetId());
            detailModel.setAdapterDictId(model.getAdapterDictId());
            detailModel.setAdapterDataType(model.getAdapterDataType());
            detailModel.setAdapterMetadataId(model.getAdapterMetadataId());
            detailModel.setAdapterMetadataCode(model.getAdapterMetadataCode());
            detailModel.setAdapterMetadataName(model.getAdapterMetadataName());
            detailModel.setAdapterDictId(model.getAdapterDictId());
            detailModel.setStdDictId(model.getStdDictId());
            detailModel.setStdDatasetId(model.getStdDatasetId());
            detailModel.setStdMetadataCode(model.getStdMetadataCode());
            detailModel.setStdMetadataName(model.getStdMetadataName());
            if (stdMetaDataModelMap.get(model.getAdapterMetadataId()) != null) {
                detailModel.setAdapterMetadataType(stdMetaDataModelMap.get(model.getAdapterMetadataId()).getColumnType());
            }
            detailModelList.add(detailModel);
        }
        return detailModelList;
    }

    public Envelop updateMetadatas(String adapterVersion,String stdVersion, String list) {
        try {
            AdapterVersion version = new AdapterVersion(adapterVersion);
            List<ApiMetadataResultDetailModel> adapterMetadataList = Arrays.asList(objectMapper.readValue(list, ApiMetadataResultDetailModel[].class));
            List<String> sqlList = new ArrayList<>();
            List<Long> idList = new ArrayList<>();
            Map<Long, ApiMetadataResultDetailModel> metadataResultMap = new HashMap<>();
            AdapterMetadataModel adapterMetadataModel = null;
            for (ApiMetadataResultDetailModel metadataModel : adapterMetadataList) {
                adapterMetadataModel = new AdapterMetadataModel();
                adapterMetadataModel.setId(metadataModel.getId());
                adapterMetadataModel.setSchemeId(metadataModel.getSchemeId());
                adapterMetadataModel.setAdapterDatasetId(metadataModel.getAdapterDatasetId());
                adapterMetadataModel.setAdapterDictId(metadataModel.getAdapterDictId());
                adapterMetadataModel.setAdapterDataType(metadataModel.getAdapterDataType());
                adapterMetadataModel.setAdapterMetadataId(metadataModel.getAdapterMetadataId());
                adapterMetadataModel.setAdapterMetadataCode(metadataModel.getAdapterMetadataCode());
                adapterMetadataModel.setAdapterMetadataName(metadataModel.getAdapterMetadataName());
                adapterMetadataModel.setAdapterDictId(metadataModel.getAdapterDictId());
                adapterMetadataModel.setStdDictId(metadataModel.getStdDictId());
                adapterMetadataModel.setStdMetadataId(metadataModel.getStdMetadataId());
                adapterMetadataModel.setStdDatasetId(metadataModel.getStdDatasetId());
                adapterMetadataModel.setStdMetadataCode(metadataModel.getStdMetadataCode());
                adapterMetadataModel.setStdMetadataName(metadataModel.getStdMetadataName());
                adapterMetadataModel.setAdapterInfo(metadataModel.getAdapterInfo());
                adapterMetadataModel.setBelongAdapterId(metadataModel.getBelongAdapterId());
                adapterMetadataModel.setAdapterDefault(metadataModel.getAdapterDefault());
                adapterMetadataModel.setNeedCrawer(1);
                idList.add(metadataModel.getStdDatasetId());
                if (!StringUtil.isEmpty(metadataModel.getAdapterDatasetCode())){
                    metadataResultMap.put(metadataModel.getStdDatasetId(), metadataModel);
                }
                JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(adapterMetadataModel));
                SqlCreator sqlCreator = new SqlCreator(AdapterMetadataModel.class);
                String sql = sqlCreator.updateDataByTableKey(version.getMetaDataTableName(), jsonNode);
                sqlList.add(sql);
                Query query = getExeuteQuery(sqlCreator, sql);
                query.executeUpdate();
            }
            if(adapterMetadataModel!=null){
                //获取belongAdapterId
                Long belongAdapterId = adapterMetadataModel.getBelongAdapterId();

                changeDatasetRelation(version,belongAdapterId,stdVersion);
            }

            return ListEnvelop.getSuccess("修改适配数据元成功",adapterMetadataList);
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("修改适配数据元失败");
        }
    }

    public void changeDatasetRelation(AdapterVersion version,Long belongAdapterId,String stdVersion){
        //获取匹配到的标准数据集id
        String sql = "select DISTINCT adapter_dataset_id from "+version.getMetaDataTableName() +" where belong_adapter_id=? and adapter_dataset_id is not null";
        List<Long> adapterDatasetIdList = jdbcTemplate.queryForList(sql,Long.class ,belongAdapterId);
        String adapterDatasetIds = "";
        String adapterDatasetCodes = "";
        String adapterDatasetNames = "";
        StandardVersion standardVersion = new StandardVersion(stdVersion);

        if(!CollectionUtils.isEmpty(adapterDatasetIdList)){
            String tableName = standardVersion.getDataSetTableName();
            SqlCreator sqlCreator = new SqlCreator(StdDataSetModel.class);
            String ids="";
            for(Long adapterDataset: adapterDatasetIdList){
                ids+=","+adapterDataset;
            }
            sqlCreator.inCondition("id", ids.substring(1,ids.length()));
            sql = sqlCreator.selectData(tableName);
            List<StdDataSetModel> stdDataSetModelList = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(StdDataSetModel.class));
            if(!CollectionUtils.isEmpty(stdDataSetModelList)){
                for(StdDataSetModel stdDataSetModel:stdDataSetModelList){
                    Long id = stdDataSetModel.getId();
                    String code = stdDataSetModel.getCode();
                    String name = stdDataSetModel.getName();
                    if(!"".equals(adapterDatasetIds)){
                        adapterDatasetIds+=","+id;
                    }else{
                        adapterDatasetIds=id+"";
                    }
                    if(!"".equals(adapterDatasetCodes)){
                        adapterDatasetCodes+=","+code;
                    }else{
                        adapterDatasetCodes=code;
                    }
                    if(!"".equals(adapterDatasetNames)){
                        adapterDatasetNames+=","+name;
                    }else{
                        adapterDatasetNames=name;
                    }
                }
            }
        }
        sql = SqlConstants.UPDATE + version.getDataSetTableName() + SqlConstants.SET+" " +
                "adapter_dataset_id='"+adapterDatasetIds+"',adapter_dataset_code='"+adapterDatasetCodes+"',adapter_dataset_name='"+adapterDatasetNames+
                "' where id="+belongAdapterId;
        jdbcTemplate.execute(sql);
    }

    public Envelop deleteMultiple(String adapterVersion, Integer metadataId, Integer stdDatasetId, Integer adapterDatasetId) {
        try {
            if (adapterVersion == null || adapterVersion.length() == 0) {
                return Envelop.getError("无效适配版本");
            }
            AdapterVersion version = new AdapterVersion(adapterVersion);
            Session session = getCurrentSession();

            String sql = "delete from " + version.getMetaDataTableName() + " where id = :id";
            Query query = session.createSQLQuery(sql);
            query.setInteger("id", metadataId);
            query.executeUpdate();

            SqlCreator sqlCreator = new SqlCreator(AdapterMetadataModel.class);
            sqlCreator.equalCondition("stdDatasetId", stdDatasetId);
            sqlCreator.equalCondition("adapterDatasetId", adapterDatasetId);
            sql = sqlCreator.countData(version.getMetaDataTableName());
            Integer count = jdbcTemplate.queryForObject(sql,Integer.class);
            if (count == 0) {
                sql = "delete from " + version.getDataSetTableName() + " where std_dataset_id = :stdDatasetId and adapter_dataset_id = :adapterDatasetId";
                query = session.createSQLQuery(sql);
                query.setInteger("stdDatasetId", stdDatasetId);
                query.setInteger("adapterDatasetId", adapterDatasetId);
                query.executeUpdate();
            }
            return Envelop.getSuccess("删除适配数据元成功");
        } catch (Exception e) {
            return Envelop.getError("删除适配数据元失败");
        }
    }

    public List getAdapterMetadataByDataset(String adapterVersion,Long datasetId){
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("stdDatasetId", datasetId);
        String tableName="adapter_metadata_"+adapterVersion;
        try {
            String condition =objectMapper.writeValueAsString(map);
            List<AdapterMetadataModel> metadataModelList = getList(AdapterMetadataModel.class,tableName, condition, null, null, null);
            return metadataModelList;
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }
        return null;

    }

    public AdapterDatasetModel getAdapterDataset(String adapterVersion, Long datasetId){
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("stdDatasetId", datasetId);
        String tableName="adapter_dataset_"+adapterVersion;
        String condition = null;
        try {
            condition = objectMapper.writeValueAsString(map);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }
        List<AdapterDatasetModel> datasetModel = getList(AdapterDatasetModel.class,tableName, condition, null, null, null);
        return datasetModel.get(0);
    }
    public AdapterMetadataModel getAdapterMetadata(String version, Integer adapterMetadataId) {
        try {
            SqlCreator sqlCreator = new SqlCreator(AdapterMetadataModel.class);
            sqlCreator.equalCondition("id", adapterMetadataId);
            String sql = sqlCreator.selectData("adapter_metadata_"+version);
            return jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(AdapterMetadataModel.class)).get(0);
        } catch (Exception e) {
            return null;
        }
    }

    public List getAdapterMetadataNotNullList(AdapterVersion adapterVersion, Long stdDatasetId, String condition) {
        try {
            StringBuffer sql = new StringBuffer();
            sql.append("SELECT * FROM ").append(adapterVersion.getMetaDataTableName());
            if (!StringUtil.isEmpty(condition)) {
                JsonNode jsonNode = objectMapper.readTree(condition);
                String column = jsonNode.get("column").asText();
                sql.append(" WHERE std_dataset_id = " + stdDatasetId + SqlConstants.AND);
                sql.append(column).append(" IS NOT NULL");
            }
            Query query = getCurrentSession().createSQLQuery(sql.toString());
            query.setResultTransformer(new BeanTransformer(AdapterMetadataModel.class));
            return query.list();
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public List<AdapterMetadataModel> getAllAdaptMetadata(String adapter_version){
        AdapterVersion adapterVersion = new AdapterVersion(adapter_version);
        StringBuffer sql = new StringBuffer();
        sql.append("SELECT m.*").append(" FROM "+adapterVersion.getMetaDataTableName()).append(" m where m.adapter_metadata_id is not null");
        Query query = getCurrentSession().createSQLQuery(sql.toString());
        query.setResultTransformer(new BeanTransformer(AdapterMetadataModel.class));
        List<AdapterMetadataModel> list = query.list();
        return list;
    }

    public List<AdapterMetadataModel> getAllAdaptMetadata(String schemaName, String adapter_version){
        AdapterVersion adapterVersion = new AdapterVersion(adapter_version);
        StringBuffer sql = new StringBuffer();
        sql.append("SELECT m.*").append(" FROM "+adapterVersion.getMetaDataTableName()).append(" m where m.adapter_metadata_id is not null");

        String myCatAnnotation = "/*#mycat:schema=" + schemaName + "*/ ";
        String completeSql = myCatAnnotation + sql;

        Query query = getCurrentSession().createSQLQuery(completeSql);
        query.setResultTransformer(new BeanTransformer(AdapterMetadataModel.class));
        List<AdapterMetadataModel> list = query.list();
        return list;
    }

    public List<AdapterMetadataModel> getAllUnAdaptMetadata(String version){
        AdapterVersion adapterVersion = new AdapterVersion(version);
        StringBuffer sql = new StringBuffer();
        sql.append("SELECT m.*").append(" FROM "+adapterVersion.getMetaDataTableName()).append(" m where m.adapter_metadata_id is null");
        Query query = getCurrentSession().createSQLQuery(sql.toString());
        query.setResultTransformer(new BeanTransformer(AdapterMetadataModel.class));
        List<AdapterMetadataModel> list = query.list();
        return list;
    }

    public List<AdapterMetadataModel> getAllUnAdaptMetadata(String schemaName, String version){
        AdapterVersion adapterVersion = new AdapterVersion(version);
        StringBuffer sql = new StringBuffer();
        sql.append("SELECT m.*").append(" FROM "+adapterVersion.getMetaDataTableName()).append(" m where m.adapter_metadata_id is null");

        String myCatAnnotation = "/*#mycat:schema=" + schemaName + "*/ ";
        String completeSql = myCatAnnotation + sql;


        Query query1 = getCurrentSession().createSQLQuery(sql.toString());
        query1.setResultTransformer(new BeanTransformer(AdapterMetadataModel.class));
        List<AdapterMetadataModel> list1 = query1.list();


        Query query = getCurrentSession().createSQLQuery(completeSql);
        query.setResultTransformer(new BeanTransformer(AdapterMetadataModel.class));
        List<AdapterMetadataModel> list = query.list();
        return list;
    }

    public List<AdapterMetadataModel> getUnAdaptMetadataByid(String version, Long id){
        AdapterVersion adapterVersion = new AdapterVersion(version);
        StringBuffer sql = new StringBuffer();
        sql.append("SELECT m.*").append(" FROM "+adapterVersion.getMetaDataTableName()).append(" m where m.adapter_info =0 and m.belong_adapter_id="+id);
        Query query = getCurrentSession().createSQLQuery(sql.toString());
        query.setResultTransformer(new BeanTransformer(AdapterMetadataModel.class));
        List<AdapterMetadataModel> list = query.list();
        return list;
    }


    public void saveAdapt(AdapterMetadataModel strategyMetadata, StdMetaDataModel orgMetadata, String version, String adapterInfo) throws Exception {
        strategyMetadata.setAdapterDatasetId(orgMetadata.getDataSetId());
        strategyMetadata.setAdapterMetadataId(orgMetadata.getId());
        strategyMetadata.setAdapterMetadataCode(orgMetadata.getInnerCode());
        strategyMetadata.setAdapterMetadataName(orgMetadata.getName());
        strategyMetadata.setAdapterInfo(adapterInfo);
        JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(strategyMetadata));
        SqlCreator sqlCreator = new SqlCreator(AdapterMetadataModel.class);
        String sql = sqlCreator.updateDataByTableKey(new AdapterVersion(version).getMetaDataTableName(), jsonNode);
        Query query = getExeuteQuery(sqlCreator, sql);
        query.executeUpdate();
    }

    public void saveAdapt(String schemaName, AdapterMetadataModel strategyMetadata, StdMetaDataModel orgMetadata, String version, String adapterInfo) throws Exception {
        strategyMetadata.setAdapterDatasetId(orgMetadata.getDataSetId());
        strategyMetadata.setAdapterMetadataId(orgMetadata.getId());
        strategyMetadata.setAdapterMetadataCode(orgMetadata.getInnerCode());
        strategyMetadata.setAdapterMetadataName(orgMetadata.getName());
        strategyMetadata.setAdapterInfo(adapterInfo);
        JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(strategyMetadata));
        SqlCreator sqlCreator = new SqlCreator(AdapterMetadataModel.class);
        String sql = sqlCreator.updateDataByTableKey(new AdapterVersion(version).getMetaDataTableName(), jsonNode);

        String myCatAnnotation = "/*#mycat:schema=" + schemaName + "*/ ";
        String completeSql = myCatAnnotation + sql;

        Query query = getExeuteQuery(sqlCreator, completeSql);
        query.executeUpdate();
    }

    public String saveAdaptSql(AdapterMetadataModel strategyMetadata, StdMetaDataModel orgMetadata, String version, String adapterInfo) throws Exception {
        StringBuilder sqlBuffer = new StringBuilder();
        if(null != orgMetadata){
            sqlBuffer.append(SqlConstants.UPDATE + new AdapterVersion(version).getMetaDataTableName() + SqlConstants.SET);
            sqlBuffer.append(" adapter_dataset_id = " + orgMetadata.getDataSetId());
            sqlBuffer.append(",adapter_metadata_id = " + orgMetadata.getId());
            sqlBuffer.append(",adapter_metadata_code = '" + orgMetadata.getInnerCode() + "'");
            sqlBuffer.append(",adapter_metadata_name = '" + orgMetadata.getName() + "'");
            if(null != adapterInfo){
                sqlBuffer.append(",adapter_info = " + adapterInfo);
            }
            sqlBuffer.append(" where id=" + strategyMetadata.getId());
        }
        return sqlBuffer.toString();
    }

    public void insertBatch(final List<String> insertSqlList) {
        super.insertBatch(insertSqlList);
    }

    public Envelop clearAdapt(String adapterVersion, Long stdDatasetId, Long belongAdapterId) {
        Envelop envelop = new Envelop();
        //数据元可能数量比较多--->>使用原生sql提高清空数据元适配效率
        StringBuilder sqlBuffer = new StringBuilder();
        AdapterVersion version = new AdapterVersion(adapterVersion);
        String metaDataTableName = version.getMetaDataTableName();

        //清空数据元适配
        sqlBuffer.append(SqlConstants.UPDATE + metaDataTableName + SqlConstants.SET );
        sqlBuffer.append(" adapter_dataset_id = null , adapter_metadata_id = null , adapter_metadata_code = null ," +
                "adapter_metadata_name = null, adapter_data_type=0 ,adapter_dict_id=null ,adapter_info=0 ,adapter_default=null " +
                "where belong_adapter_id="+belongAdapterId+" and std_dataset_id="+stdDatasetId);
        jdbcTemplate.execute(sqlBuffer.toString());

        //清空数据集适配
        String dataSetTable = version.getDataSetTableName();
        String sql = SqlConstants.UPDATE + dataSetTable + SqlConstants.SET+" " +
                "adapter_dataset_id =null ,adapter_dataset_code =null,adapter_dataset_name=null " +
                "where id="+ belongAdapterId;
        jdbcTemplate.execute(sql);

        envelop.setStatus(200);
        return envelop;
    }
}