package com.yihu.jw.basic.standard.service.standard;

import com.fasterxml.jackson.databind.JsonNode;
import com.yihu.jw.basic.standard.dao.SQLGeneralDAO;
import com.yihu.jw.basic.standard.model.*;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.util.common.StringUtil;
import com.yihu.jw.util.sql.ErrorCode;
import com.yihu.jw.exception.ApiException;
import com.yihu.jw.util.sql.RequestParamTransformer;
import com.yihu.jw.basic.standard.model.standard.StandardModel;
import com.yihu.jw.basic.standard.model.standard.StandardVersionModel;
import com.yihu.jw.basic.standard.model.standard.resultModel.StandardParamResultModel;
import com.yihu.jw.basic.standard.model.standard.resultModel.StandardResultDetailModel;
import com.yihu.jw.util.sql.SqlConstants;
import org.apache.commons.lang3.StringUtils;
import org.hibernate.Criteria;
import org.hibernate.query.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @created Airhead 2015/12/23.
 */
@Service("StandardService")
public class StandardService extends SQLGeneralDAO {

    public static final String BEAN_ID = "StandardService";

    public ListEnvelop getDictionaryResult(String condition,String order, Integer limit, Integer offset) {
        try {
            List<DictItem> detailModelList = new ArrayList<DictItem>();
            List<StandardModel> standardModelList = getList(condition, order, limit, offset);
            for (StandardModel standardModel : standardModelList) {
                DictItem items = new DictItem();
                Long id = standardModel.getId();
                items.setCode(id.toString());
                items.setValue(standardModel.getName());
                detailModelList.add(items);
            }
            return ListEnvelop.getSuccess("获取标准列表成功",detailModelList);
        } catch (Exception e) {
            return ListEnvelop.getError("获取适配列表失败");
        }

    }

    public List<StandardModel> getList(String condition, String order, Integer limit, Integer offset) {
        Map queryMap = RequestParamTransformer.parseJsonToMap(condition);
        Map orderMap = RequestParamTransformer.parseJsonToMap(order);
        return this.getList(queryMap, orderMap, limit, offset);
    }

    public ObjEnvelop add(String standard) {
        Boolean existCodeFlg = false;
        Boolean existNameFlg = false;
        try {
            StandardModel standardModel = objectMapper.readValue(standard, StandardModel.class);
            existCodeFlg = standardCodeValidate(standardModel.getCode());
            existNameFlg = standardNameValidate(standardModel.getName());
            if (!existCodeFlg && !existNameFlg) {
                this.add(standardModel);
                return ObjEnvelop.getSuccess("保存标准成功",standardModel);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (existCodeFlg) {
            return ObjEnvelop.getError("已存在标准编码");
        } else if (existNameFlg) {
            return ObjEnvelop.getError("已存在标准名称");
        } else {
            return ObjEnvelop.getError("保存标准失败");
        }
    }

    public Boolean standardCodeValidate(String code) {
        Map<String, Object> query = new HashMap<>();
        Map<String, String> order = new HashMap<>();
        query.put("code", code);

        List<StandardModel> list = getList(query, order, 1, 0);
        return !CollectionUtils.isEmpty(list);
    }

    public Boolean standardNameValidate(String name) {
        Map<String, Object> query = new HashMap<>();
        Map<String, String> order = new HashMap<>();
        query.put("name", name);

        List<StandardModel> list = getList(query, order, 1, 0);
        return !CollectionUtils.isEmpty(list);
    }

    @Transactional
    public Envelop delete(Long standardId) {
        Boolean publishedFlg = false;
        Boolean existFlg = false;
        try {
            ObjEnvelop objEnvelop = get(standardId);
            if (objEnvelop.getStatus()!=200) {
                return Envelop.getError("删除标准失败");
            }
            StandardModel standardModel = (StandardModel) objEnvelop.getObj();
            if (standardModel.getVersionStatus().equals(1)) {
                publishedFlg = true;
            }
            existFlg = existStandardVersionValid(standardModel.getId());
            if (!publishedFlg && !existFlg) {
                deleteEntity(StandardModel.class, standardId);
                return Envelop.getSuccess("删除标准成功");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (publishedFlg) {
            return Envelop.getError("该标准下已有标准版本");
        } else if (existFlg) {
            return Envelop.getError("已存在标准版本");
        } else {
            return Envelop.getError("删除标准失败");
        }
    }

    public Boolean standardNameValidate(Long standardId, String name) {
        String sql = "FROM StandardModel WHERE name = :name and id != :standardId";
        Query query = getCurrentSession().createQuery(sql);
        query.setParameter("standardId", standardId);
        query.setParameter("name", name);
        List<StandardModel> list = query.list();
        return !CollectionUtils.isEmpty(list);
    }

    public Boolean standardCodeValidate(Long standardId, String code) {
        String sql = "FROM StandardModel WHERE code = :code and id != :standardId";
        Query query = getCurrentSession().createQuery(sql);
        query.setParameter("standardId", standardId);
        query.setParameter("code", code);
        List<StandardModel> list = query.list();
        return !CollectionUtils.isEmpty(list);
    }

    @Transactional
    public ObjEnvelop update(String standard) {
        Boolean existCodeFlg = false;
        Boolean existNameFlg = false;
        try {
            StandardModel standardModel = objectMapper.readValue(standard, StandardModel.class);
            existCodeFlg = standardCodeValidate(standardModel.getId(), standardModel.getCode());
            existNameFlg = standardNameValidate(standardModel.getId(), standardModel.getName());
            if (!existCodeFlg && !existNameFlg) {
                this.update(standardModel);
                return ObjEnvelop.getSuccess("修改标准成功",standardModel);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (existCodeFlg) {
            return ObjEnvelop.getError("已存在标准编码");
        } else if (existNameFlg) {
            return ObjEnvelop.getError("已存在标准名称");
        } else {
            return ObjEnvelop.getError("修改标准失败");
        }
    }

    public ObjEnvelop get(Long standardId) {
        try {
            StandardModel standardModel = getEntity(StandardModel.class, standardId);
            return ObjEnvelop.getSuccess("获取标准成功",standardModel);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ObjEnvelop.getError("获取标准失败");
    }

    public StandardParamResultModel getStandardParam(String paramKey) {
        Criteria criteria = getCurrentSession().createCriteria(SystemParam.class);
        criteria.add(Restrictions.eq("paramKey", paramKey));
        SystemParam systemParam = (SystemParam) criteria.uniqueResult();

        Criteria criteria1 = getCurrentSession().createCriteria(SystemParam.class);
        criteria1.add(Restrictions.eq("paramKey", SqlConstants.STANDARD_ORG));
        SystemParam systemParam1 = (SystemParam) criteria1.uniqueResult();

        if (systemParam != null) {
            StandardParamResultModel standardParamResultModel = new StandardParamResultModel();
//            standardParamResultModel.setSuccessFlg(true);
//            standardParamResultModel.setMessage("获取参数成功");
            standardParamResultModel.setCode(systemParam.getParamValue());
            standardParamResultModel.setOrgCode(systemParam1.getParamValue());
            return standardParamResultModel;
        } else {
            StandardParamResultModel standardParamResultModel = new StandardParamResultModel();
//            standardParamResultModel.setSuccessFlg(false);
//            standardParamResultModel.setMessage("获取参数失败");
            return standardParamResultModel;
        }
    }

    public List<StandardResultDetailModel> getStandardListFormat(List<SystemOrganization> publisherModelList,
                                                                 List<StandardModel> standardModelList,
                                                                 List<StandardVersionModel> versionModelList,
                                                                 Map<String, String> codeToIdMap,
                                                                 String name) {
        List<StandardResultDetailModel> resultDetailModelList = new ArrayList<>();
        Map<String, StandardResultDetailModel> resultDetailModelMap = new HashMap<>();
        Map<String, StandardResultDetailModel> resultDetailModelCheckMap = new HashMap<>();
        Map<String, StandardResultDetailModel> resultDetailModelAddMap = new HashMap<>();
        for (SystemOrganization publisherModel : publisherModelList) {
            StandardResultDetailModel resultDetailModel = new StandardResultDetailModel();
            String id = "publisher" + publisherModel.getId();
            resultDetailModel.setId(id);
            resultDetailModel.setName(publisherModel.getFullName());
            resultDetailModel.setType(0);
            resultDetailModel.setCode(publisherModel.getCode());
            codeToIdMap.put(publisherModel.getCode(), id);
            if (StringUtil.isEmpty(name) || publisherModel.getFullName().contains(name)) {
                resultDetailModelCheckMap.put(id, resultDetailModel);
                resultDetailModelAddMap.put(id, resultDetailModel);
                resultDetailModelList.add(resultDetailModel);
            }
            resultDetailModelMap.put(id, resultDetailModel);
        }

        for (StandardModel standardModel : standardModelList) {
            String pid = codeToIdMap.get(standardModel.getPublisherOrgCode());
            if (StringUtil.isEmpty(pid)) {
                continue;
            }
            String id = "standard" + standardModel.getId();
            StandardResultDetailModel resultDetailModel = new StandardResultDetailModel();
            resultDetailModel.setId(id);
            resultDetailModel.setPid(codeToIdMap.get(standardModel.getPublisherOrgCode()));
            resultDetailModel.setType(1);
            resultDetailModel.setName(standardModel.getName());
            resultDetailModel.setCode(standardModel.getCode());
            if (resultDetailModelCheckMap.containsKey(pid)) {
                resultDetailModelCheckMap.put(id, resultDetailModel);
                resultDetailModelList.add(resultDetailModel);
                resultDetailModelAddMap.put(id, resultDetailModel);
            } else if (StringUtil.isEmpty(name) || standardModel.getName().contains(name)) {
                resultDetailModelCheckMap.put(id, resultDetailModel);
                resultDetailModelList.add(resultDetailModel);
                if (!resultDetailModelAddMap.containsKey(pid)) {
                    resultDetailModelAddMap.put(pid, resultDetailModel);
                    resultDetailModelList.add(resultDetailModelMap.get(pid));
                }
            }
            resultDetailModelMap.put(id, resultDetailModel);
        }

        for (StandardVersionModel versionModel : versionModelList) {
            String pid = "standard" + versionModel.getStandardId();
            String id = "version" + versionModel.getId();
            StandardResultDetailModel resultDetailModel = new StandardResultDetailModel();
            resultDetailModel.setId(id);
            resultDetailModel.setPid(pid);
            resultDetailModel.setPublisher(versionModel.getPublishUser());
            resultDetailModel.setStatus(versionModel.getPublishStatus());
            resultDetailModel.setName(versionModel.getVersionName());
            resultDetailModel.setCode(versionModel.getVersion());
            resultDetailModel.setDate(StringUtil.toString(versionModel.getPublishTime()));
            resultDetailModel.setType(2);
            if (resultDetailModelCheckMap.containsKey(pid)) {
                resultDetailModelList.add(resultDetailModel);
            } else if (StringUtil.isEmpty(name) || versionModel.getVersionName().contains(name)) {
                resultDetailModelList.add(resultDetailModel);
                if (!resultDetailModelAddMap.containsKey(pid)) {
                    resultDetailModelAddMap.put(resultDetailModel.getPid(), resultDetailModelMap.get(resultDetailModel.getPid()));
                    if (resultDetailModelMap.get(pid) != null && resultDetailModelMap.get(resultDetailModel.getPid()) != null) {
                        resultDetailModelAddMap.put(resultDetailModelMap.get(pid).getPid(), resultDetailModelMap.get(resultDetailModelMap.get(resultDetailModel.getPid()).getPid()));
                        resultDetailModelList.add(resultDetailModelMap.get(resultDetailModel.getPid()));
                        resultDetailModelList.add(resultDetailModelMap.get(resultDetailModelMap.get(resultDetailModel.getPid()).getPid()));
                    }
                }
            }
        }
        return resultDetailModelList;
    }

    public ListEnvelop getDetailModelResult(String condition) {
        try {
            List<StandardResultDetailModel> standardResultDetailModelList = getStandardList(condition);
            return ListEnvelop.getSuccess("获取标准列表成功",standardResultDetailModelList);
        } catch (Exception e) {
            e.printStackTrace();
            return ListEnvelop.getError("获取标准列表失败");
        }
    }

    public List<StandardResultDetailModel> getStandardList(String condition) {
        String name = SqlConstants.EMPTY;
        Integer type = null;
        if (!StringUtil.isEmpty(condition)) {
            try {
                JsonNode jsonNode = objectMapper.readTree(condition);
                if (jsonNode.get("type") != null) {
                    type = jsonNode.get("type").asInt();
                }
                if (jsonNode.get("name") != null) {
                    name = jsonNode.get("name").asText();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        StandardParamResultModel standardParam = getStandardParam(SqlConstants.STANDARD);
        StandardParamResultModel standardParam1 = getStandardParam(SqlConstants.STANDARD_ORG);
        String code = standardParam.getCode();
        String orgCode = standardParam1.getCode();
        List<SystemOrganization> publisherModelList = new ArrayList<>();

        Criteria criteria = getCurrentSession().createCriteria(StandardModel.class);
        if (type == 0) {
            criteria.add(Restrictions.eq("code", code));
            String publisherHql = "FROM SystemOrganization WHERE code =:orgCode";
            Query query = getCurrentSession().createQuery(publisherHql);
            query.setParameter("orgCode", orgCode);
            publisherModelList = query.list();
        } else if (type == 1) {
            criteria.add(Restrictions.ne("code", code));
            String publisherHql = "FROM SystemOrganization";
            Query query = getCurrentSession().createQuery(publisherHql);
            publisherModelList = query.list();
        }

        List<StandardModel> standardModelList = criteria.list();
        Map<String, String> codeToIdMap = new HashMap<>();
        List<Long> idList = new ArrayList<>();
        for (StandardModel standardModel : standardModelList) {
            idList.add(standardModel.getId());
        }
        List<StandardVersionModel> versionModelList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(idList)) {
            String versionHql = "select * from std_version  WHERE standard_id in ("+ StringUtils.join(idList,",") +")";
            versionModelList = jdbcTemplate.query(versionHql,new BeanPropertyRowMapper<>(StandardVersionModel.class));
        }

        List<StandardResultDetailModel> resultDetailModelList = getStandardListFormat(publisherModelList,
                standardModelList,
                versionModelList,
                codeToIdMap,
                name);
        return resultDetailModelList;
    }

    //TODO:TEST
    public StandardModel fork(Long standardId) {
        try {
            ObjEnvelop objEnvelop = get(standardId);
            StandardModel baseStandard = (StandardModel) objEnvelop.getObj();

            StandardModel newStandard = new StandardModel(baseStandard);
            newStandard.setId(null);
            newStandard.setName(baseStandard.getName() + "-副本");
            newStandard.setCode(baseStandard.getCode() + "-副本");
            newStandard.setVersionStatus(0);

            saveEntity(newStandard);

            return newStandard;
        } catch (Exception e) {
            e.printStackTrace();
        }

        throw new ApiException(ErrorCode.StandardForkFailed.getErrorCode());
    }

    public boolean existStandardVersionValid(Long standardId) {
        String hql = "FROM StandardVersionModel WHERE standardId = :standardId";
        Query query = getCurrentSession().createQuery(hql);
        query.setParameter("standardId", standardId);
        List<StandardVersionModel> standardModelList = query.list();
        return !CollectionUtils.isEmpty(standardModelList);
    }

    private void update(StandardModel standardModel) throws Exception {
        updateEntity(standardModel);
    }

    private void add(StandardModel standardModel) throws Exception {
        saveEntity(standardModel);
    }

    private List<StandardModel> getList(Map<String, Object> query, Map<String, String> order, Integer limit, Integer offset) {

        Session session = getCurrentSession();
        Criteria criteria = session.createCriteria(StandardModel.class);
        if (query!=null) {
            for (String key : query.keySet()) {
                if (key.equals("type")) {
                    Integer type = (Integer) query.get(key);
                    String code = getStandardParam(SqlConstants.STANDARD).getCode();
                    if (type == 0) {
                        criteria.add(Restrictions.eq("code", code));
                    } else if (type == 1) {
                        criteria.add(Restrictions.ne("code", code));
                    }
                } else {
                    criteria.add(Restrictions.eq(key, query.get(key)));
                }
            }
        }
        if (limit != null) {
            criteria.setMaxResults(limit);
            if (offset != null &&offset>0) {
                int tmp = 0;
                if((offset - 1) * limit>0){
                    tmp = (offset - 1) * limit;
                }
                criteria.setFirstResult(tmp);
            }
        }
        if (order != null) {
            for (String key : order.keySet()) {
                if (order.get(key).equals(SqlConstants.ASC)) {
                    criteria.addOrder(Order.asc(key));
                } else if (order.get(key).equals(SqlConstants.DESC)) {
                    criteria.addOrder(Order.desc(key));
                }
            }
        }
        return (List<StandardModel>) criteria.list();
    }
}
