package com.yihu.jw.basic.standard.service.standard;

import com.fasterxml.jackson.databind.JsonNode;
import com.yihu.jw.basic.standard.model.Select2;
import com.yihu.jw.basic.standard.model.standard.StdDictionaryEntryModel;
import com.yihu.jw.basic.standard.service.bo.StandardVersion;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.util.common.StringUtil;
import com.yihu.jw.util.sql.ErrorCode;
import com.yihu.jw.basic.standard.dao.SQLGeneralDAO;
import com.yihu.jw.exception.ApiException;
import com.yihu.jw.util.sql.RequestParamTransformer;
import com.yihu.jw.util.sql.SqlConstants;
import com.yihu.jw.util.sql.SqlCreator;
import org.hibernate.query.Query;
import org.hibernate.Session;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.util.*;

/**
 * @author Sand
 * @version 1.0
 * @created 16-7月-2015 20:57:06
 */
@Transactional
@Service("StdDictEntryService")
public class StdDictEntryService extends SQLGeneralDAO {

    public static final String BEAN_ID = "StdDictEntryService";

    public StdDictEntryService() {
    }

    public void finalize() throws Throwable {
    }

    public List<StdDictionaryEntryModel> getList(String stdVersion, Long dictId, String condition, String order, Integer limit, Integer offset) {
        try {
            Map queryMap = RequestParamTransformer.parseJsonToMap(condition);
            Map orderMap = RequestParamTransformer.parseJsonToMap(order);
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            return this.getList(standardVersion, dictId, queryMap, orderMap, limit, offset);
        } catch (Exception e) {
            e.printStackTrace();
        }

        throw new ApiException(ErrorCode.StandardDictEntryGetListFailed.getErrorCode());
    }

    public ObjEnvelop add(String stdVersion, String dictEntry) {
        Boolean existFlg = false;
        try {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            StdDictionaryEntryModel stdDictionaryEntryModel = objectMapper.readValue(dictEntry, StdDictionaryEntryModel.class);
            existFlg = dictEntryCodeValidate(standardVersion, stdDictionaryEntryModel.getDictId(), stdDictionaryEntryModel.getCode());
            if (!existFlg) {
                this.add(standardVersion, stdDictionaryEntryModel);
                return ObjEnvelop.getSuccess("保存标准字典项成功",stdDictionaryEntryModel);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (existFlg) {
            return ObjEnvelop.getError("已存在标准字典项编码");
        } else {
            return ObjEnvelop.getError("保存标准字典项失败");
        }
    }

    public Boolean dictEntryCodeValidate(StandardVersion standardVersion, Long dictId, String code) {
        Map<String, Object> query = new HashMap<>();
        Map<String, String> order = new HashMap<>();
        query.put("code", code);

        List<StdDictionaryEntryModel> list = this.getList(standardVersion, dictId, query, order, 1, 1);
        if (!CollectionUtils.isEmpty(list)) {
            return true;
        }
        return false;
    }

    public Envelop delete(String stdVersion, Long dictEntryId) {
        try {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            if (dictEntryId != null) {
                this.delete(standardVersion, dictEntryId);
                return Envelop.getSuccess("删除标准字典项成功");
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new ApiException(ErrorCode.StandardDictEntryDeleteFailed.getErrorCode());
        }
        return Envelop.getError("删除标准字典项失败");
    }

    public Boolean dictEntryCodeValidate(StandardVersion standardVersion, Long dictId, Long dictEntryId, String code) {
        SqlCreator sqlCreator = new SqlCreator(StdDictionaryEntryModel.class);
        sqlCreator.equalCondition("dictId", dictId);
        sqlCreator.equalCondition("code", code);
        sqlCreator.neCondition("id", dictEntryId);
        String sql = sqlCreator.selectData(standardVersion.getDictEntryTableName());
        List<StdDictionaryEntryModel> list = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(StdDictionaryEntryModel.class));
        if (!CollectionUtils.isEmpty(list)) {
            return true;
        }
        return false;
    }

    public ObjEnvelop update(String stdVersion, String dictEntry) {
        Boolean existFlg = false;
        try {
            StandardVersion standardVersion = new StandardVersion(stdVersion);

            StdDictionaryEntryModel stdDictionaryEntryModel = objectMapper.readValue(dictEntry, StdDictionaryEntryModel.class);
            existFlg = dictEntryCodeValidate(standardVersion, stdDictionaryEntryModel.getDictId(), stdDictionaryEntryModel.getId(), stdDictionaryEntryModel.getCode());
            if (!existFlg) {
                this.modify(standardVersion, stdDictionaryEntryModel);
                return ObjEnvelop.getSuccess("修改标准字典项成功",stdDictionaryEntryModel);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (existFlg) {
            return ObjEnvelop.getError("已存在标准字典项编码");
        } else {
            return ObjEnvelop.getError("修改标准字典项失败");
        }
    }

    public ObjEnvelop get(String stdVersion, Integer dictEntryId) {
        try {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            if (dictEntryId != null) {
                StdDictionaryEntryModel stdDictionaryEntryModel = this.get(standardVersion, dictEntryId);
                return ObjEnvelop.getSuccess("获取标准字典项成功",stdDictionaryEntryModel);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return ObjEnvelop.getError("获取标准字典项失败");
    }

    public void add(StandardVersion standardVersion, StdDictionaryEntryModel stdDictionaryEntryModel) throws IOException {
        Session session = getCurrentSession();

        String dictEntryTableName = standardVersion.getDictEntryTableName();
        String sql = "select max(id) from " + dictEntryTableName;
        Query query = session.createSQLQuery(sql);
        Object object = query.uniqueResult();

        Long id = object == null ? 1L : Long.parseLong(object.toString()) + 1;
        stdDictionaryEntryModel.setId(id);

        JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(stdDictionaryEntryModel));
        SqlCreator sqlCreator = new SqlCreator(StdDictionaryEntryModel.class);
        sql = sqlCreator.insertData(dictEntryTableName, jsonNode);
        query = getExeuteQuery(sqlCreator, sql);
        query.executeUpdate();
    }

    public PageEnvelop getDetailModelList(String stdVersion, Long dictId, String condition, String order, Integer rows, Integer page) {
        try {
            List<StdDictionaryEntryModel> detailModelList = getDictEntryList(StdDictionaryEntryModel.class, stdVersion, dictId, condition, order, rows, page);
            Integer count = getDictEntryInt(StdDictionaryEntryModel.class, stdVersion, dictId, condition);
            PageEnvelop result = PageEnvelop.getSuccess("获取标准字典项列表成功");
            result.setDetailModelList(detailModelList);
            result.setTotalCount(count);
            return result;
        } catch (Exception e) {
            e.printStackTrace();
            return PageEnvelop.getError("获取标准字典项列表失败");
        }
    }

    public List getDictEntryList(Class tClass, String stdVersion, Long dictId, String condition, String order, Integer limit, Integer offset) {
        StandardVersion standardVersion = new StandardVersion(stdVersion);
        String tableName = standardVersion.getDictEntryTableName();
        SqlCreator sqlCreator = new SqlCreator(tClass);
        sqlCreator.equalCondition("dictId", dictId);
        if (!StringUtil.isEmpty(condition)) {
            try {
                JsonNode jsonNode = objectMapper.readTree(condition);
                String name = jsonNode.get("name").asText();
                sqlCreator.likeOrCondition("code", "value", name);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        try {
            if (!StringUtil.isEmpty(order)) {
                JsonNode jsonNode = objectMapper.readTree(order);
                Iterator<String> fieldNames = jsonNode.fieldNames();
                while (fieldNames.hasNext()) {
                    String fieldName = fieldNames.next();
                    String value = jsonNode.get(fieldName).asText().toUpperCase();
                    sqlCreator.order(fieldName, value);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        String sql = sqlCreator.selectData(tableName);
        if (limit != null) {
            if (offset != null &&offset>0) {
                sql += " limit "+(offset - 1) * limit+","+limit;
            }else {
                sql += " limit "+limit;
            }

        }
        return jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(tClass));
    }

    public Integer getDictEntryInt(Class tClass, String stdVersion, Long dictId, String condition) {
        StandardVersion standardVersion = new StandardVersion(stdVersion);
        String tableName = standardVersion.getDictEntryTableName();
        SqlCreator sqlCreator = new SqlCreator(tClass);
        sqlCreator.equalCondition("dictId", dictId);
        if (!StringUtil.isEmpty(condition)) {
            try {
                JsonNode jsonNode = objectMapper.readTree(condition);
                String name = jsonNode.get("name").asText();
                sqlCreator.likeOrCondition("code", "value", name);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        String sql = sqlCreator.countData(tableName);
        Integer count = jdbcTemplate.queryForObject(sql,Integer.class);
        return count;
    }

    public PageEnvelop getDictEntrySelectList(String stdVersion, Long dictId, String condition, String order, Integer rows, Integer page) {
        try {
            List<StdDictionaryEntryModel> dictEntryModelList = getDictEntryList(StdDictionaryEntryModel.class, stdVersion, dictId, condition, order, rows, page);
            List<Select2> detailModelList = new ArrayList<>();
            for (StdDictionaryEntryModel dictEntry : dictEntryModelList) {
                Select2 select2 = new Select2();
                select2.setId(dictEntry.getId() + SqlConstants.COMMA + dictEntry.getCode());
                select2.setText(dictEntry.getValue());
                detailModelList.add(select2);
            }
            PageEnvelop result = PageEnvelop.getSuccess("获取标准字典项下拉列表成功");
            result.setDetailModelList(detailModelList);
            result.setTotalCount(detailModelList.size());
            return result;
        } catch (Exception e) {
            e.printStackTrace();
            return PageEnvelop.getError("获取标准字典项下拉列表失败");
        }
    }

    public StdDictionaryEntryModel getStEntryValueByCode(String version, Integer dictId, String entryCode) {
        try {
            SqlCreator sqlCreator = new SqlCreator(StdDictionaryEntryModel.class);
            sqlCreator.equalCondition("dictId", dictId);
            sqlCreator.equalCondition("code", entryCode);
            String sql = sqlCreator.selectData("std_dictionary_entry_" + version);
            List<StdDictionaryEntryModel> list = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(StdDictionaryEntryModel.class));
            if (list == null || list.size() == 0) {
                return null;
            } else {
                return list.get(0);
            }
        } catch (Exception e) {
            return null;
        }
    }

    private StdDictionaryEntryModel get(StandardVersion standardVersion, Integer dictEntryId) {
        String dictEntryTableName = standardVersion.getDictEntryTableName();

        SqlCreator sqlCreator = new SqlCreator(StdDictionaryEntryModel.class);
        sqlCreator.equalCondition("id", dictEntryId);
        String sql = sqlCreator.selectData(dictEntryTableName);
        return jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(StdDictionaryEntryModel.class)).get(0);
    }

    private void modify(StandardVersion standardVersion, StdDictionaryEntryModel stdDictionaryEntryModel) throws IOException {
        JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(stdDictionaryEntryModel));
        SqlCreator sqlCreator = new SqlCreator(StdDictionaryEntryModel.class);
        String dictEntryTableName = standardVersion.getDictEntryTableName();
        String sql = sqlCreator.updateDataByTableKey(dictEntryTableName, jsonNode);
        Query query = getExeuteQuery(sqlCreator, sql);
        query.executeUpdate();
    }

    private void delete(StandardVersion standardVersion, Long dictEntryId) {
        Session session = getCurrentSession();
        String sql = "delete from " + standardVersion.getDictEntryTableName() + " where id = :id";
        Query query = session.createSQLQuery(sql);
        query.setLong("id", dictEntryId);
        query.executeUpdate();
    }

    private List<StdDictionaryEntryModel> getList(StandardVersion standardVersion, Long dictId, Map<String, Object> query, Map<String, String> order, Integer limit, Integer offset) {
        SqlCreator sqlCreator = new SqlCreator(StdDictionaryEntryModel.class);
        sqlCreator.equalCondition("dictId", dictId);
        for (String key : query.keySet()) {
            sqlCreator.equalCondition(key, query.get(key));
        }
        for (String key : order.keySet()) {
            sqlCreator.order(key, order.get(key));
        }

        String sql = sqlCreator.selectData(standardVersion.getDictEntryTableName());
        if (limit != null) {
            if (offset != null &&offset>0) {
                sql += " limit "+(offset - 1) * limit+","+limit;
            }else {
                sql += " limit "+limit;
            }

        }
        return jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(StdDictionaryEntryModel.class));
    }

}//end DictEntryManager