package com.yihu.jw.basic.standard.service.standard;

import com.fasterxml.jackson.databind.JsonNode;
import com.yihu.jw.basic.standard.dao.SQLGeneralDAO;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.util.common.StringUtil;
import com.yihu.jw.util.sql.*;
import com.yihu.jw.exception.ApiException;
import com.yihu.jw.basic.standard.model.Select2;
import com.yihu.jw.basic.standard.model.standard.*;
import com.yihu.jw.basic.standard.model.standard.resultModel.StandardParamResultModel;
import com.yihu.jw.basic.standard.service.bo.StandardVersion;
import org.hibernate.Criteria;
import org.hibernate.query.NativeQuery;
import org.hibernate.query.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 数据元管理接口实现。
 *
 * @author Sand
 * @version 1.0
 * @created 2015.07.10 15:22
 */
@Transactional
@Service("StdMetadataService")
public class StdMetadataService extends SQLGeneralDAO {

    public static final String BEAN_ID = "StdMetadataService";
    @Autowired
    StandardService standardService;

    public StdMetadataService() {
    }

    public List<StdMetaDataModel> getList(String schemaName, String stdVersion, Long datasetId, String condition, String order, Integer limit, Integer offset) {

        try {
            Map<String, Object> queryMap = RequestParamTransformer.parseJsonToMap(condition);
            Map<String, String> orderMap = RequestParamTransformer.parseJsonToMap(order);
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            return this.getList(schemaName,standardVersion, datasetId, queryMap, orderMap, limit, offset);
        } catch (Exception e) {
            e.printStackTrace();
        }

        throw new ApiException(ErrorCode.StandardMetaDataGetListFailed.getErrorCode());
    }

    private List<StdMetaDataModel> getList(String schemaName,StandardVersion standardVersion, Long datasetId, Map<String, Object> query, Map<String, String> order, Integer limit, Integer offset) {
        SqlCreator sqlCreator = new SqlCreator(StdMetaDataModel.class);
        if(null!=datasetId){
            sqlCreator.equalCondition("datasetId", datasetId);
        }
        for (String key : query.keySet()) {
            sqlCreator.equalCondition(key, query.get(key));
        }
        for (String key : order.keySet()) {
            sqlCreator.order(key, order.get(key));
        }

        String sql = sqlCreator.selectData(standardVersion.getMetaDataTableName());

        String myCatAnnotation = "/*#mycat:schema=" + schemaName + "*/ ";
        String completeSql = myCatAnnotation + sql;

        if (limit != null) {
            if (offset != null &&offset>0) {
                completeSql += " limit "+(offset - 1) * limit+","+limit;
            }else {
                completeSql += " limit "+limit;
            }
        }
        return jdbcTemplate.query(completeSql,new BeanPropertyRowMapper<>(StdMetaDataModel.class));
    }

    public List<StdMetaDataModel> getList(String stdVersion, Long datasetId, String condition, String order, Integer limit, Integer offset) {

        try {
            Map<String, Object> queryMap = RequestParamTransformer.parseJsonToMap(condition);
            Map<String, String> orderMap = RequestParamTransformer.parseJsonToMap(order);
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            return this.getList(standardVersion, datasetId, queryMap, orderMap, limit, offset);
        } catch (Exception e) {
            e.printStackTrace();
        }

        throw new ApiException(ErrorCode.StandardMetaDataGetListFailed.getErrorCode());
    }

    public ObjEnvelop add(String stdVersion, String metadata) {

        Boolean existFlg = false;
        try {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            StdMetaDataModel stdMetaDataModel = objectMapper.readValue(metadata, StdMetaDataModel.class);
            existFlg = metadataCodeValidate(standardVersion, stdMetaDataModel.getDataSetId(), stdMetaDataModel.getCode());
            if (!existFlg) {
                this.add(standardVersion, stdMetaDataModel);
                return ObjEnvelop.getSuccess("保存数据元成功",stdMetaDataModel);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (existFlg) {
            return ObjEnvelop.getError("已存在标准数据元编码");
        } else {
            return ObjEnvelop.getError("保存数据元失败");
        }
    }

    public Boolean metadataCodeValidate(StandardVersion standardVersion, Long datasetId, String code) {
        Map<String, Object> query = new HashMap<>();
        Map<String, String> order = new HashMap<>();
        query.put("code", code);

        List<StdMetaDataModel> list = this.getList(standardVersion, datasetId, query, order, 1, 1);
        return !CollectionUtils.isEmpty(list);
    }

    public Envelop delete(String stdVersion, Long metadataId) {
        try {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            if (metadataId != null) {
                this.delete(standardVersion, metadataId);
                return Envelop.getSuccess("删除数据元成功");
            }
        } catch (Exception e) {
            throw new ApiException(ErrorCode.StandardMetaDataDeleteFailed.getErrorCode());
        }
        return Envelop.getError("删除数据元失败");
    }

    public Boolean metadataCodeValidate(StandardVersion standardVersion, Long datasetId, Long metadataId, String code) {
        SqlCreator sqlCreator = new SqlCreator(StdMetaDataModel.class);
        sqlCreator.equalCondition("dataSetId", datasetId);
        sqlCreator.equalCondition("code", code);
        sqlCreator.neCondition("id", metadataId);
        String sql = sqlCreator.selectData(standardVersion.getMetaDataTableName());
        List<StdMetaDataModel> list = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(StdMetaDataModel.class));
        return !CollectionUtils.isEmpty(list);
    }

    public ObjEnvelop update(String stdVersion, String metadata) {
        Boolean existFlg = false;
        try {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            StdMetaDataModel stdMetaDataModel = objectMapper.readValue(metadata, StdMetaDataModel.class);
            existFlg = metadataCodeValidate(standardVersion, stdMetaDataModel.getDataSetId(), stdMetaDataModel.getId(), stdMetaDataModel.getCode());
            if (!existFlg) {
                this.modify(standardVersion, stdMetaDataModel);
                return ObjEnvelop.getSuccess("修改数据元成功",stdMetaDataModel);
            } else {
                return ObjEnvelop.getError("已存在标准数据元编码");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ObjEnvelop.getError("修改数据元失败");
    }

    public ObjEnvelop get(String stdVersion, Integer metadataId) {
        try {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            if (metadataId != null) {
                String metaDataTableName = standardVersion.getMetaDataTableName();
                SqlCreator sqlCreator = new SqlCreator(StdMetaDataModel.class);
                sqlCreator.equalCondition("id", metadataId);
                String sql = sqlCreator.selectData(metaDataTableName);
                StdMetaDataModel stdMetaDataModel = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(StdMetaDataModel.class)).get(0);
                if (stdMetaDataModel != null) {
                    stdMetaDataModel.checkValue();
                }
                return ObjEnvelop.getSuccess("获取标准数据元成功",stdMetaDataModel);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ObjEnvelop.getError("获取标准数据元失败");
    }


    private void modify(StandardVersion standardVersion, StdMetaDataModel stdMetaDataModel) throws IOException {
        JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(stdMetaDataModel));
        SqlCreator sqlCreator = new SqlCreator(StdMetaDataModel.class);
        String metaDataTableName = standardVersion.getMetaDataTableName();
        String sql = sqlCreator.updateDataByTableKey(metaDataTableName, jsonNode);
        Query query = getExeuteQuery(sqlCreator, sql);
        query.executeUpdate();
    }

    private void delete(StandardVersion standardVersion, Long metadataId) {
        Session session = getCurrentSession();
        String sql = "delete from " + standardVersion.getMetaDataTableName() + " where id = :id";
        Query query = session.createSQLQuery(sql);
        query.setParameter("id", metadataId);
        query.executeUpdate();
    }

    public void add(StandardVersion standardVersion, StdMetaDataModel stdMetaDataModel) throws IOException {

        Session session = getCurrentSession();
        String metaDataTableName = standardVersion.getMetaDataTableName();
        String sql = "select max(id) from " + metaDataTableName;
        Query query = session.createSQLQuery(sql);
        Object object = query.uniqueResult();
        stdMetaDataModel.setId(object == null ? 1L : Long.parseLong(object.toString()) + 1);

        JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(stdMetaDataModel));
        SqlCreator sqlCreator = new SqlCreator(StdMetaDataModel.class);
        sql = sqlCreator.insertData(metaDataTableName, jsonNode);
        query = getExeuteQuery(sqlCreator, sql);
        query.executeUpdate();
    }

    public List<StdMetaDataModel> getList(StandardVersion standardVersion, Long datasetId, Map<String, Object> query, Map<String, String> order, Integer limit, Integer offset) {
        SqlCreator sqlCreator = new SqlCreator(StdMetaDataModel.class);
        if(null!=datasetId){
            sqlCreator.equalCondition("dataSetId", datasetId);
        }
        for (String key : query.keySet()) {
            sqlCreator.equalCondition(key, query.get(key));
        }
        for (String key : order.keySet()) {
            sqlCreator.order(key, order.get(key));
        }

        String sql = sqlCreator.selectData(standardVersion.getMetaDataTableName());
        if (limit != null) {
            if (offset != null &&offset>0) {
                sql += " limit "+(offset - 1) * limit+","+limit;
            }else {
                sql += " limit "+limit;
            }

        }
        return jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(StdMetaDataModel.class));
    }

    public PageEnvelop getDetailModelList(String stdVersion, Long dictId, String condition, String order, Integer rows, Integer page) {
        try {
            List<StdDictionaryEntryModel> detailModelList = getMetadataList(StdMetaDataModel.class, stdVersion, dictId, condition, order, rows, page);
            Integer count = getMetadataCount(StdMetaDataModel.class, stdVersion, dictId, condition);
            PageEnvelop result = PageEnvelop.getSuccess("获取标准字数据元列表成功");
            result.setDetailModelList(detailModelList);
            result.setTotalCount(count);
            return result;
        } catch (Exception e) {
            e.printStackTrace();
            return PageEnvelop.getError("获取标准数据元列表失败");
        }
    }

    public List getMetadataList(Class tClass, String stdVersion, Long datasetId, String condition, String order, Integer limit, Integer offset) {
        StandardVersion standardVersion = new StandardVersion(stdVersion);
        String tableName = standardVersion.getMetaDataTableName();
        String sql = "select * from "+tableName;

        sql += " where dataset_id = "+ datasetId;
        if (!StringUtil.isEmpty(condition)) {
            try {
                JsonNode jsonNode = objectMapper.readTree(condition);
                String name = jsonNode.get("name").asText();
                sql += " and (code like '%"+name+"%' or name like '%"+name+"%' )";
            } catch (IOException e) {
                e.printStackTrace();
            }
        }


        if (!StringUtil.isEmpty(order)) {
            sql += " order by "+order;
        }

        if(limit != null){
            if (offset != null &&offset>0) {
                sql += " limit "+ (offset - 1) * limit+","+limit;
            }else {
                sql += " limit " +limit;
            }
        }
        return jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(tClass));
    }

    public Integer getMetadataCount(Class tClass, String stdVersion, Long datasetId, String condition) {
        StandardVersion standardVersion = new StandardVersion(stdVersion);
        String tableName = standardVersion.getMetaDataTableName();
        SqlCreator sqlCreator = new SqlCreator(tClass);
        sqlCreator.equalCondition("dataSetId", datasetId);
        if (!StringUtil.isEmpty(condition)) {
            try {
                JsonNode jsonNode = objectMapper.readTree(condition);
                String name = jsonNode.get("name").asText();
                sqlCreator.likeOrCondition("code", "name", name);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        String sql = sqlCreator.countData(tableName);
        return jdbcTemplate.queryForObject(sql,Integer.class);
    }

    public PageEnvelop getMetadataSelectList(String stdVersion, Long dictId, String condition, String order, Integer rows, Integer page) {
        try {
            List<StdMetaDataModel> metadataList = getMetadataList(StdMetaDataModel.class, stdVersion, dictId, condition, order, rows, page);
            List<Select2> detailModelList = new ArrayList<>();
            for (StdMetaDataModel metaDataModel : metadataList) {
                Select2 select2 = new Select2();
                if (metaDataModel.getDictId() != null) {
                    select2.setId(metaDataModel.getId() + SqlConstants.COMMA + metaDataModel.getCode() + SqlConstants.COMMA + metaDataModel.getDictId());
                } else {
                    select2.setId(metaDataModel.getId() + SqlConstants.COMMA + metaDataModel.getCode());
                }
                select2.setText(metaDataModel.getName());
                detailModelList.add(select2);
            }
            PageEnvelop detailModelResult = PageEnvelop.getSuccess("获取标准数据元下拉列表成功");
            detailModelResult.setDetailModelList(detailModelList);
            detailModelResult.setTotalCount(detailModelList.size());
            return detailModelResult;
        } catch (Exception e) {
            e.printStackTrace();
            return PageEnvelop.getError("获取标准数据元下拉列表失败");
        }
    }

    public List getAllVersionMetadataList(String condition) {
        //数据元列表应该是所选择的数据集列表的数据元并集去重 @modify by Yingjie Chen
        try {
            Map<String,List<StdDataSetModel>> version2DataSetMap = new HashMap<>();
            String sql;
            String code = SqlConstants.EMPTY;
            if (!StringUtil.isEmpty(condition)) {
                JsonNode jsonNode = objectMapper.readTree(condition);
                code = jsonNode.get("code").asText();
            }

            Criteria criteria = getCurrentSession().createCriteria(StandardModel.class);
            StandardParamResultModel standardParam = standardService.getStandardParam(SqlConstants.STANDARD);
            String VersionCode = standardParam.getCode();
            criteria.add(Restrictions.eq("code", VersionCode));
            List<StandardModel> standardModelList = criteria.list();
            List<Long> idList = standardModelList.stream().map(StandardModel::getId).collect(Collectors.toList());
            List<StandardVersionModel> versionModelList;
            Session session = getCurrentSession();
            if (!CollectionUtils.isEmpty(idList)) {
                String hql = "FROM StandardVersionModel WHERE publishTime is not null AND standardId IN :idList order by publishTime";
                Query query = getCurrentSession().createQuery(hql);
                query.setParameterList("idList", idList);
                versionModelList = query.list();
                for (StandardVersionModel versionModel : versionModelList) {
                    SqlCreator sqlDatasetCreator = new SqlCreator(StdDataSetModel.class);
                    StandardVersion version = new StandardVersion(versionModel.getVersion());
                    sqlDatasetCreator.equalCondition("code", code);
                    sql = sqlDatasetCreator.selectData(version.getDataSetTableName());
                    NativeQuery sqlQuery = session.createNativeQuery(sql);
                    BeanTransformer datasetTransformer = new BeanTransformer(StdDataSetModel.class);
                    sqlQuery.setResultTransformer(datasetTransformer);
                    sqlQuery.setParameter("code", code);
                    List<StdDataSetModel> dataList = sqlQuery.list();
                    if (dataList.size() > 0) {
                        version2DataSetMap.put(versionModel.getVersion(),dataList);
                    }
                }
            }

            if (!version2DataSetMap.isEmpty()) {
                sql = SqlConstants.EMPTY;
                int count = 1;
                Map<Integer, String> datasetIdMap = new HashMap<>();

                Iterator<Map.Entry<String, List<StdDataSetModel>>> iter = version2DataSetMap.entrySet().iterator();
                while (iter.hasNext()) {
                    Map.Entry<String, List<StdDataSetModel>> entry = iter.next();
                    SqlCreator sqlMetadataCreator = new SqlCreator(StdMetaDataModel.class);
                    StandardVersion version = new StandardVersion(entry.getKey());
                    List<StdDataSetModel> dataList = entry.getValue();
                    datasetIdMap.put(count++, StringUtil.toString(dataList.get(0).getId()));
                    sqlMetadataCreator.equalConditionQue("dataSetId", dataList.get(0).getId());
                    sql = sql + sqlMetadataCreator.selectData(version.getMetaDataTableName()) + SqlConstants.UNION;

                }

                BeanTransformer metadataTransformer = new BeanTransformer(StdMetaDataModel.class);
                List<StdMetaDataModel> distinctModelList = new ArrayList<>();
                if (!StringUtil.isEmpty(sql)) {
                    sql = sql.substring(0, sql.length() - SqlConstants.UNION.length());
                    NativeQuery sqlQuery;
                    sqlQuery = getCurrentSession().createSQLQuery(sql);
                    sqlQuery.setResultTransformer(metadataTransformer);
                    for (int i = 1; i < count; i++) {
                        sqlQuery.setParameter(i - 1, datasetIdMap.get(i));
                    }
                    List<StdMetaDataModel> metadataList = sqlQuery.list();
                    Map<String, StdMetaDataModel> meatadataModelMap = new HashMap<>();
                    for (StdMetaDataModel model : metadataList) {
                        meatadataModelMap.put(model.getCode(), model);
                    }
                    distinctModelList.addAll(meatadataModelMap.keySet().stream().map(meatadataModelMap::get).collect(Collectors.toList()));
                }

                return distinctModelList;
            } else {
                return new ArrayList<>();
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new ApiException(ErrorCode.GetDataSetListFailed.getErrorCode());
        }
    }


}