package com.yihu.jw.basic.standard.service.standard;

import com.yihu.jw.basic.standard.dao.SQLGeneralDAO;
import com.yihu.jw.basic.standard.model.SystemOrganization;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.util.sql.ErrorCode;
import com.yihu.jw.exception.ApiException;
import com.yihu.jw.basic.standard.model.DictItem;
import com.yihu.jw.util.sql.RequestParamTransformer;
import com.yihu.jw.basic.standard.model.standard.resultModel.PublisherResultModel;
import com.yihu.jw.util.sql.SqlConstants;
import org.hibernate.Criteria;
import org.hibernate.query.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @created Airhead 2015/12/23.
 */
@Transactional
@Service("StdPublisherService")
public class StdPublisherService extends SQLGeneralDAO {

    public static final String BEAN_ID = "StdPublisherService";

    public ListEnvelop getDictionaryResult(String condition, String order, Integer limit, Integer offset) {
        try {
            List<DictItem> detailModelList = new ArrayList<DictItem>();
            List<SystemOrganization> stdPublisherModelList = getList(condition, order, limit, offset);
            for(SystemOrganization systemOrganization : stdPublisherModelList){
                DictItem items = new DictItem();
                String id = systemOrganization.getCode();
                String name = systemOrganization.getFullName();
                items.setCode(id.toString());
                items.setValue(name);
                detailModelList.add(items);
            }
            return ListEnvelop.getSuccess("获取发布机构列表成功",detailModelList);
        } catch (Exception e) {
            return ListEnvelop.getError("获取发布机构列表失败");
        }
    }

    public List<SystemOrganization> getList(String condition, String order, Integer limit, Integer offset) {
        try{
            Map<String, Object> queryMap = RequestParamTransformer.parseJsonToMap(condition);
            Map<String, String> orderMap = RequestParamTransformer.parseJsonToMap(order);
            return this.getList(queryMap, orderMap, limit, offset);
        } catch (Exception e) {
            e.printStackTrace();
        }

        throw new ApiException(ErrorCode.GetPublisherListFaield.getErrorCode());
    }

    private List<SystemOrganization> getList(Map<String, Object> query, Map<String, String> order, Integer limit, Integer offset) {
        Session session = getCurrentSession();
        Criteria criteria = session.createCriteria(SystemOrganization.class);
        if (query != null) {
            for (String key : query.keySet()) {
                criteria.add(Restrictions.eq(key, query.get(key)));
            }
        }
        if (limit != null) {
            criteria.setMaxResults(limit);
            if (offset != null &&offset>0) {
                criteria.setFirstResult((offset - 1) * limit);
            }
        }
        if (order != null) {
            for (String key : order.keySet()) {
                if (order.get(key).equals(SqlConstants.ASC)) {
                    criteria.addOrder(Order.asc(key));
                } else if (order.get(key).equals(SqlConstants.DESC)) {
                    criteria.addOrder(Order.desc(key));
                }
            }
        }
        List<SystemOrganization> systemOrganizationList = criteria.list();

        return systemOrganizationList;
    }

    public ObjEnvelop get(String stdPublisherId) {
        try {
            Session session = getCurrentSession();
            Criteria criteria = session.createCriteria(SystemOrganization.class);
            criteria.add(Restrictions.eq("id", stdPublisherId));
            SystemOrganization systemOrganization = (SystemOrganization) criteria.uniqueResult();
            PublisherResultModel publisherResultModel = new PublisherResultModel();
            publisherResultModel.setCode(systemOrganization.getCode());
            publisherResultModel.setName(systemOrganization.getFullName());
            return ObjEnvelop.getSuccess("获取机构信息成功",publisherResultModel);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError("获取机构信息失败");
        }
    }

    public ObjEnvelop getStandardPublisher() {
        try {
            String paramSql = "select s.param_value from system_param s where s.param_key = '"+ SqlConstants.STANDARD_ORG+"'";
            Query paramQuery = getCurrentSession().createSQLQuery(paramSql);
            String publisherCode = (String) paramQuery.uniqueResult();
            Criteria criteria = getCurrentSession().createCriteria(SystemOrganization.class);
            criteria.add(Restrictions.eq("code", publisherCode));
            SystemOrganization systemOrganization = (SystemOrganization) criteria.uniqueResult();
            PublisherResultModel publisherResultModel = new PublisherResultModel();
            publisherResultModel.setCode(systemOrganization.getCode());
            publisherResultModel.setName(systemOrganization.getFullName());
            return ObjEnvelop.getSuccess("获取机构信息成功",publisherResultModel);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError(ErrorCode.GetPublisherFailed.getErrorCode());
        }
    }
}
