package com.yihu.jw.basic.system.service;


import com.yihu.jw.basic.system.dao.DictDao;
import com.yihu.jw.basic.system.dao.DictListDao;
import com.yihu.jw.basic.system.model.SystemDict;
import com.yihu.jw.basic.system.model.SystemDictList;
import com.yihu.jw.restmodel.web.Envelop;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * Created by chenweida on 2016/1/20.
 */
@Service("dictManager")
public class DictManager  {

    public static final String BEAN_ID = "dictManager";

    @Resource(name = DictDao.BEAN_ID )
    private DictDao dictItemDao;//字典项
    @Resource(name = DictListDao.BEAN_ID)
    private DictListDao dictDao;//字典


    /**
     * 通过字典名字,
     * 字典项的code查找字典项值
     * @param dictName
     * @param code
     * @return
     */
    public String getDictIetmValue(String dictName,String code){
        SystemDict dictItem = dictItemDao.getDictIetmValue(dictName, code);
        if(dictItem!=null){
            return dictItem.getValue();
        }
        return "";
    }

    @Transactional
    public Envelop getDictList(Map<String, Object> params) throws Exception {
        return dictDao.getDictList(params);
    }

    @Transactional
    public Envelop getDictItemList(Map<String, Object> params) throws Exception {
        return dictItemDao.getDictList(params);
    }


    public SystemDict getSystemDictById(String id) throws Exception {
        return dictItemDao.getEntity(SystemDict.class, id);
    }


    public SystemDictList getSystemDictListById(String id) throws Exception {
        return dictDao.getEntity(SystemDictList.class, id);
    }

    @Transactional
    public Envelop deleteDict(String id) throws Exception {
        SystemDictList sd = dictDao.getEntity(SystemDictList.class, id);
        dictItemDao.delDictItemByDictName(sd.getDictName());
        dictDao.deleteEntity(sd);
        return Envelop.getSuccess("删除成功");
    }

    @Transactional
    public Envelop addDict(SystemDictList rr) throws Exception {
        SystemDictList systemDictListTemp = dictDao.getDitcByName(rr.getDictName());
        if (systemDictListTemp != null) {
            return Envelop.getError("字典代码重复");
        }
        dictDao.saveEntity(rr);
        return Envelop.getSuccess("保存成功");
    }

    @Transactional
    public Envelop updateDict(SystemDictList rr) throws Exception {
        SystemDictList systemDictListTemp = dictDao.getDitcByNameWithOutId(rr.getDictName(), rr.getId());
        if (systemDictListTemp != null) {
            return Envelop.getError("字典代码重复");
        }
        SystemDictList sd = dictDao.getEntity(SystemDictList.class, rr.getId());
        sd.setPid(rr.getPid());
        sd.setChineseName(rr.getChineseName());
        sd.setDictName(rr.getDictName());
        sd.setPyCode(rr.getPyCode());
        sd.setRelationColCode(rr.getRelationColCode());
        sd.setRelationColValue(rr.getRelationColValue());
        sd.setRelationTable(rr.getRelationTable());
        sd.setRemark(rr.getRemark());
        sd.setRelationColExtend(rr.getRelationColExtend());
        return Envelop.getSuccess("更新成功");
    }

    @Transactional
    public Envelop deleteDictItem(String id) throws Exception {
        SystemDict sd = dictItemDao.getEntity(SystemDict.class, id);
        dictItemDao.deleteEntity(sd);
        return Envelop.getSuccess("删除成功");
    }

    @Transactional
    public Envelop addDictItem(SystemDict rr) throws Exception {
       // SystemDict rrTemp = dictItemDao.getDictItemByCode(rr.getInnerCode());
       // if (rrTemp != null) {
       //     return Envelop.getError("字典项代码已经存在");
      //  }
        dictItemDao.saveEntity(rr);

        return Envelop.getSuccess("保存成功");
    }

    @Transactional
    public Envelop updateDictItem(SystemDict rr) throws Exception {
      //  SystemDict rrTemp = dictItemDao.getDictItemByCodeWithOutId(rr.getInnerCode(),rr.getId());
       // if (rrTemp != null) {
       //    return Envelop.getError("字典项代码已经存在");
      //  }
        SystemDict sd = dictItemDao.getEntity(SystemDict.class, rr.getId());
        sd.setPyCode(rr.getPyCode());
        sd.setDictName(rr.getDictName());
        sd.setCode(rr.getCode());
        sd.setSort(rr.getSort());
        sd.setValue(rr.getValue());
        return Envelop.getSuccess("更新成功");
    }

    /**
     * 通过字典名字,
     * 字典项的code查找字典项值
     * @param dictName
     * @param code
     * @return
     */
    public SystemDict getDictIetm(String dictName,String code){
        SystemDict dictItem = dictItemDao.getDictIetmValue(dictName, code);
        return dictItem;
    }

    /**
     * @param dictName
     * @param codes
     * @return
     */
    public List<SystemDict> getDictEntries(String dictName,String[] codes) {
       return dictItemDao.getDictEntries(dictName,codes);
    }
}
