package com.yihu.jw.door.controller.doctor;

import com.alibaba.fastjson.JSON;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.doctor.dao.BaseDoctorHospitalDao;
import com.yihu.jw.door.controller.BaseController;
import com.yihu.jw.door.dao.DoorServiceOrderDao;
import com.yihu.jw.door.service.DoorOrderService;
import com.yihu.jw.door.service.common.PatientService;
import com.yihu.jw.door.service.common.WlyyMessageService;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.org.BaseOrgDO;
import com.yihu.jw.entity.hospital.message.SystemMessageDO;
import com.yihu.jw.org.dao.BaseOrgDao;
import com.yihu.jw.patient.dao.BasePatientDao;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Created by yeshijie on 2020/12/30.
 */
@RestController
@RequestMapping(value = "/doctor", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(description = "医生端-基础信息")
public class DoctorController extends BaseController {

    @Autowired
    private BaseDoctorDao doctorDao;
    @Autowired
    private BaseDoctorHospitalDao doctorHospitalDao;
    @Autowired
    private BasePatientDao patientDao;
    @Autowired
    private PatientService patientService;
    @Autowired
    private BaseOrgDao orgDao;
    @Autowired
    private WlyyMessageService messageService;
    @Autowired
    private DoorOrderService doorOrderService;
    @Autowired
    DoorServiceOrderDao doorServiceOrderDao;


    @GetMapping(value = "/appointmentRevisitOnDoor")
    @ApiOperation("appointmentRevisitOnDoor")
    public String patientList(String orderId,String doctorId
    ) {
        try {
            doorOrderService.appointmentRevisitOnDoor(doorServiceOrderDao.findOne(orderId),doctorId);
            return write(200, "查询成功", "data", "");
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "查询失败");
        }
    }



    @RequestMapping(value = "message/getWaitingMessages",method = RequestMethod.GET)
    @ApiOperation("获取服务工单待接单列表")
    public String getWaitingMessages(
            @ApiParam(name = "message",value = "消息对象")
            @RequestParam(value = "message")String message,
            @ApiParam(name="types",value="消息类型")
            @RequestParam(value="types",required = true) String types,
            @ApiParam(name="page",value="第几页",defaultValue = "1")
            @RequestParam(value="page",required = true) String page,
            @ApiParam(name="pageSize",value="",defaultValue = "10")
            @RequestParam(value="pageSize",required = true) String pageSize){
        if (StringUtils.isBlank(pageSize)) {
            pageSize = "10";
        }
        if (page.equals("0")) {
            page = "1";
        }
        String[] split = StringUtils.split(types, ",");
        List<Integer> typeList = new ArrayList<>();
        for (String s : split) {
            typeList.add(Integer.valueOf(s));
        }
        SystemMessageDO message1 = new SystemMessageDO();
        com.alibaba.fastjson.JSONObject object = JSON.parseObject(message);
        message1.setOver(object.getString("over"));
        message1.setIsRead(object.get("read")+"");
        message1.setReceiver(getUID());
        try {
            JSONObject waitingMessages = messageService.getWaitingMessages(message1, types, Integer.valueOf(page), Integer.valueOf(pageSize));
            return write(200, "查询成功","data",waitingMessages);
        }catch (Exception e){
            error(e);
            return error( -1, "查询失败！");
        }
    }

    @GetMapping(value = "/patientList")
    @ApiOperation("查询居民列表信息")
    public String patientList(
        @ApiParam(name = "name", value = "居民姓名", required = false) @RequestParam(value = "name", required = false) String name,
        @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1") @RequestParam(value = "page") int page,
        @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15") @RequestParam(value = "size") int size
        ) {
        try {
            List<Map<String,Object>> p = patientService.patientList(name,page,size);
            return write(200, "查询成功", "data", p);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "查询失败");
        }
    }

    /**
     * 查询居民信息
     *
     * @param patient 患者
     * @return
     */
    @GetMapping(value = "/patient")
    @ApiOperation("查询居民信息")
    public String getPatient(String patient) {
        try {
            if (StringUtils.isEmpty(patient)) {
                return error(-1, "居民不能为空");
            }
            JSONObject p = patientService.getPatient(patient,getUID());
            return write(200, "查询成功", "data", p);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "查询失败");
        }
    }

    @GetMapping(value = "getHospitalInfoByCode")
    @ApiOperation("根据机构code获取机构信息")
    public String getHospitalInfoByCode(String code) {
        try {
            BaseOrgDO baseOrgDO = orgDao.findByCode(code);
            return write(200, "查询成功", "data", baseOrgDO);
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败");
        }
    }

    @GetMapping(value = "getHospitals")
    @ApiOperation("获取医院")
    public String getHospitals() {
        try {
            List<BaseOrgDO> orgDOs = orgDao.findByAll();
            return write(200, "查询成功", "data", orgDOs);
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败");
        }
    }


    /**
     * 医生基本信息查询接口
     *
     * @return
     */
    @ApiOperation("医生基本信息查询接口")
    @GetMapping(value = "baseinfo")
    public String baseinfo(
            @RequestParam(required = false) String code) {
        try {
            if (StringUtils.isEmpty(code)) {
                code = getUID();
            }
            BaseDoctorDO temp = doctorDao.findById(code);
//            Integer leader = doctorAdminTeamDao.findLeader(code);
//            Hospital hospital = hospitalDao.findByCode(temp.getHospital());
            if (temp != null) {
                JSONObject json = new JSONObject();
                // 设置区县code
//                json.put("town", temp.getTown());
                // 设置医生手机号码
                json.put("mobile", temp.getMobile());
                // 设置医生标识
                json.put("code", temp.getId());
                // 设置医生姓名
                json.put("name", temp.getName());
                // 设置医生头像
                json.put("photo", temp.getPhoto());
                // 设置医生性别
                json.put("sex", temp.getSex());
                // 设置省名称
                json.put("provinceName", temp.getProvinceName());
                // 设置城市名称
                json.put("cityName", temp.getCityName());
                // 设置医院名称
//                json.put("hospitalName", temp.getHospitalName());
                // 设置医院
//                json.put("hospital", temp.getHospital());
                // 设置医院等级1.三甲，2.二甲，3.1级以下
//                json.put("hosLevel", hospital.getHosLevel());
                // 设置部门名称
//                json.put("deptName", StringUtils.isNotEmpty(temp.getDeptName()) ? temp.getDeptName() : "");
                // 设置职称code
                json.put("jobCode", StringUtils.isNotEmpty(temp.getJobTitleCode()) ? temp.getJobTitleCode() : "");
                // 设置职称
                json.put("jobName", StringUtils.isNotEmpty(temp.getJobTitleName()) ? temp.getJobTitleName() : "");
                // 设置医生专长
                json.put("expertise", StringUtils.isNotEmpty(temp.getExpertise()) ? temp.getExpertise() : "");
                // 设置医生简介
                json.put("introduce", StringUtils.isNotEmpty(temp.getIntroduce()) ? temp.getIntroduce() : "");
                // 二维码地址
                json.put("qrcode", temp.getQrcode());
                // 咨询统计
//                Map<String, Long> consultCount = consultTeamService.getAllCount(code);
                // 咨询总数
//                json.put("consultAmount", consultCount.get("all"));
//                // 当天咨询数
//                json.put("consultAmountToday", consultCount.get("today"));
                //身份类型
                json.put("level", "1");
                //身份证号
                json.put("idcard", temp.getIdcard());
                //是否认证
                json.put("isCertified", temp.getIscertified());
                //是否名医
                json.put("isFamous", temp.getIsFamous());
                //是否医生提示 提示过是1 其他都是0
                json.put("isPasswordPrompt", StringUtils.isEmpty(temp.getIsPasswordPrompt()) ? "0" : temp.getIsPasswordPrompt());
                //获取医生角色和区域权限
//                List<Map<String, String>> roleMap = roleService.getUserRoleAndArea(temp.getCode());
//                json.put("userRole", roleMap);

                //获取医生后台管理员系统角色，用于判断是否是居民上门服务业务中的调度员
//                json.put("doctorRole",roleService.findManageUserRole(temp.getCode()));
//                是否团队长
//                json.put("isLeader", leader == null ? "0" : "1");
//                团队长所管辖的团队code 一个团队长只管辖一个团队
//                json.put("adminTeamCode", leader);


                return write(200, "医生信息查询成功！", "data", json);
            } else {
                return error(-1, "医生信息查询失败！");
            }
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "医生信息查询失败！");
        }
    }
}
