package com.yihu.jw.door.controller.doctor;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.door.controller.BaseController;
import com.yihu.jw.door.service.DoorServiceApplicationService;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.door.WlyyDoorServiceApplicationDo;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.util.common.IdCardUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * Created by zdm on 2019/3/13.
 */
@RestController
@RequestMapping(value = "/doctor/doorServiceAuditing", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(description = "医生端-上门服务资质申请审核")
public class DoctorDoorServiceAuditController extends BaseController {
    @Autowired
    private DoorServiceApplicationService doorServiceApplicationService;
    @Autowired
    private BasePatientDao patientService;

    @RequestMapping(value = "getDoorSericeApplicationStatus", method = RequestMethod.GET)
    @ApiOperation(value = "获取资质审核状态")
    public String getDoorSericeApplicationStatus(
            @ApiParam(name = "patient", value = "patient")
            @RequestParam(value = "patient", required = true) String patient) {
        try {
            return write(200, "获取成功", "data", doorServiceApplicationService.getDoorSericeApplicationStatus(patient));
        } catch (Exception ex) {
            ex.printStackTrace();
            return error(-1, "获取失败!"+ex.getMessage());
        }
    }


    @RequestMapping(value = "getPOnsiteServiceByDoctor", method = RequestMethod.GET)
    @ApiOperation(value = "获取待审核的居民上门服务资质申请")
    public String getPOnsiteServiceByDoctor(
            @ApiParam(name = "doctorCode", value = "当前医生code")
            @RequestParam(value = "doctorCode", required = false) String doctorCode,
            @ApiParam(name = "auditorType", value = "医生角色：1调度员、2家医",required = true)
            @RequestParam(value = "auditorType", required = true) String auditorType,
            @ApiParam(name = "patientName", value = "居民姓名")
            @RequestParam(value = "patientName", required = false) String patientName,
            @ApiParam(name = "mobile", value = "居民联系方式")
            @RequestParam(value = "mobile", required = false) String mobile,
            @ApiParam(name = "crowdType", value = "人群类型：1失能老人，2半失能老人，3残疾人，4计生特殊家庭，5其他")
            @RequestParam(value = "crowdType", required = false) String crowdType,
            @ApiParam(name = "status", value = "资质状态：为空默认查找审核过的申请，0审核中，1未通过，2审核通过，3已过期")
            @RequestParam(value = "status", required = false) String status,
            @RequestParam(value = "page", required = false, defaultValue = "1") Integer page,
            @RequestParam(value = "pageSize", required = false, defaultValue = "15") Integer pageSize) {
        try {
            if(StringUtils.isBlank(doctorCode)){
                doctorCode=getRepUID();
            }
            JSONObject jsonObject = doorServiceApplicationService.getPOnsiteServiceByDoctor( doctorCode, auditorType, patientName, mobile, crowdType, status, page, pageSize);
            return write(200, "获取成功", "data", jsonObject);
        } catch (Exception ex) {
            ex.printStackTrace();
            return error(-1, "获取失败!"+ex.getMessage());
        }
    }


    @RequestMapping(value = "auditing", method = RequestMethod.POST)
    @ApiOperation(value = "获取居民上门服务资质审核及调度")
    public String auditing(
            @ApiParam(name = "id", value = "资质申请id")
            @RequestParam(value = "id", required = false) Long id,
            @ApiParam(name = "operationType", value = "操作类型：1审核，2调度",required = true)
            @RequestParam(value = "operationType", required = true) String operationType,
            @ApiParam(name = "doctorCode", value = "当前医生code,可不传")
            @RequestParam(value = "doctorCode", required = false) String doctorCode,
            @ApiParam(name = "status", value = "审核状态：1未通过，2审核通过")
            @RequestParam(value = "status", required = false) String status,
            @ApiParam(name = "startDate", value = "开始时间", required = false)
            @RequestParam(value = "startDate",required = false)String startDate,
            @ApiParam(name = "endDate", value = "结束时间", required = false)
            @RequestParam(value = "endDate",required = false)String endDate,
            @ApiParam(name = "failureReasons", value = "拒绝的原因")
            @RequestParam(value = "failureReasons", required = false) String failureReasons) {
        try {
            if(StringUtils.isBlank(doctorCode)){
                doctorCode=getRepUID();
            }
            WlyyDoorServiceApplicationDo wlyyDoorServiceApplicationDo = doorServiceApplicationService.auditing( id,doctorCode,status, startDate,endDate,operationType,failureReasons);
            return write(200, "审核成功", "data", wlyyDoorServiceApplicationDo);
        } catch (Exception ex) {
            ex.printStackTrace();
            return error(-1, "审核失败!"+ex.getMessage());
        }
    }

    @RequestMapping(value = "getPOnsiteServiceById", method = RequestMethod.GET)
    @ApiOperation(value = "获取居民上门服务资质申请详情")
    public String getPOnsiteServiceById(
            @ApiParam(name = "id", value = "资质申请id")
            @RequestParam(value = "id", required = false) Long id) {
        try {
            JSONObject jsonObject = doorServiceApplicationService.getPOnsiteServiceById(id);
            return write(200, "获取成功", "data", jsonObject);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "获取失败!"+ex.getMessage());
        }
    }

    @RequestMapping(value = "getPOnsiteServiceByPatient", method = RequestMethod.GET)
    @ApiOperation(value = "医生根据查找的居民code获取居民的资质申请信息")
    public String getPOnsiteServiceByPatient(
            @ApiParam(name = "patientCode", value = "居民code")
            @RequestParam(value = "patientCode", required = true) String patientCode) {
        try {
            ObjectMapper objectMapper=new ObjectMapper();
            JSONObject jsonObject=new JSONObject();
            BasePatientDO patientDo=patientService.findById(patientCode);
            WlyyDoorServiceApplicationDo wlyyDoorServiceApplicationDo = doorServiceApplicationService.findOnsiteServiceByPatient(patientCode);
            if (null == wlyyDoorServiceApplicationDo.getId()) {
                JSONObject patient = doorServiceApplicationService.findPatientByCode(patientCode);
                jsonObject.put("applicationStatus",0);
                jsonObject.put("patient",patient);
                jsonObject.put("photo",patientDo.getPhoto());
                return write(200, "获取成功", "data", jsonObject);
            }else{
                jsonObject.put("applicationStatus","1");
                jsonObject.put("age", IdCardUtil.getAgeForIdcard(wlyyDoorServiceApplicationDo.getPatientIdcard()));
                jsonObject.put("sex",IdCardUtil.getSexForIdcard_new(wlyyDoorServiceApplicationDo.getPatientIdcard()));
                jsonObject.put("photo",patientDo.getPhoto());
                jsonObject.put("patient",com.alibaba.fastjson.JSONObject.toJSON(patientDo));
                jsonObject.put("wlyyDoorServiceApplicationDo", objectMapper.writeValueAsString(wlyyDoorServiceApplicationDo));
            }
            return write(200, "获取成功", "data", jsonObject);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "获取失败!"+ex.getMessage());
        }
    }

    @RequestMapping(value = "create", method = RequestMethod.POST)
    @ApiOperation(value = "医生为居民开通服务资格")
    public String create(
            @ApiParam(name = "jsonData", value = "开通服务资格内容jsonData")
            @RequestParam(value = "jsonData",required = true)String jsonData,
            @ApiParam(name = "doctorCode", value = "医生doctorCode")
            @RequestParam(value = "doctorCode",required = false)String doctorCode){
        try {
            if(StringUtils.isBlank(doctorCode)){
                doctorCode=getRepUID();
            }
            WlyyDoorServiceApplicationDo wlyyDoorServiceApplicationDo = doorServiceApplicationService.create("2",jsonData,doctorCode);
            return write(200, "获取成功", "data", wlyyDoorServiceApplicationDo);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "获取失败!"+e.getMessage());
        }
    }

    @RequestMapping(value = "myMyTest",method = RequestMethod.POST)
    @ApiOperation("测试")
    public String myMyTest(){
        try {
            return write(200,"请求成功","data",doorServiceApplicationService.myMyTest());
        }catch (Exception e){
            e.printStackTrace();
            return error(-1,"请求失败"+e.getMessage());
        }
    }
}
