package com.yihu.jw.door.controller.doctor.followup;

import com.alibaba.fastjson.JSON;
import com.yihu.jw.door.controller.BaseController;
import com.yihu.jw.door.service.DoorOrderService;
import com.yihu.jw.door.service.followup.FollowUpService;
import com.yihu.jw.followup.FollowupContentESDO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * Created by yeshijie on 2021/1/9.
 */
@RestController
@RequestMapping(value = "/doctor/followup", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(description = "医生端-随访接口")
public class DoctorFollowUpController extends BaseController {

    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private FollowUpService followUpService;
    @Autowired
    private DoorOrderService doorOrderService;



    @ApiOperation("暂不需要新增随访")
    @PostMapping(value = "/notAddFollowup")
    public String notAddFollowup(@ApiParam(name = "orderId", value = "订单id", defaultValue = "4")
                              @RequestParam(value = "orderId", required = true) String orderId) {
        try {
            doorOrderService.notAddFollowup(orderId);
            return write(200, "操作成功！");
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }

    @ApiOperation("新增临时随访记录（返回ID）")
    @PostMapping(value = "/addFollowup")
    public String addFollowup(@ApiParam(name = "patient", value = "患者代码", defaultValue = "443a196ef8744536a531260eb26c05d7")
                              @RequestParam(value = "patient", required = true) String patient,
                              @ApiParam(name = "date", value = "下次随访时间", defaultValue = "2016-12-15 20:00:00")
                              @RequestParam(value = "date", required = false) String date,
                              @ApiParam(name = "followupType", value = "随访方式【字典FOLLOWUP_WAY_DICT】", defaultValue = "25")
                              @RequestParam(value = "followupType", required = true) String followupType,
                              @ApiParam(name = "followupClass", value = "随访类别【1.高血压 2.糖尿病 3高糖】", defaultValue = "1")
                              @RequestParam(value = "followupClass", required = true) String followupClass,
                              @ApiParam(name = "followupManagerStatus", value = "随访管理状态【字典FOLLOWUP_MANAGER_STATUS】", defaultValue = "1")
                              @RequestParam(value = "followupManagerStatus", required = false) String followupManagerStatus,
                              @ApiParam(name = "plandate", value = "下次随访时间", defaultValue = "2016-12-14 20:00:00")
                              @RequestParam(value = "plandate", required = false) String plandate,
                              @ApiParam(name = "relateCode", value = "关联code", defaultValue = "关联code")
                              @RequestParam(value = "relateCode", required = false) String relateCode,
                              @ApiParam(name = "type", value = "随访类型", defaultValue = "0")
                              @RequestParam(value = "type", required = false) Integer type) {
        try {
            followUpService.addFollowup(getUID(), patient, date, followupType, followupClass, followupManagerStatus,plandate,relateCode,type);
            return write(200, "新增临时随访记录成功！");
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "新增临时随访记录失败！" + e.getMessage());
        }
    }

    @ApiOperation("获取随访信息")
    @GetMapping(value = "/getFollowup")
    public String getFollowup(@ApiParam(name = "id", value = "随访记录ID", defaultValue = "4")
                              @RequestParam(value = "id", required = true) String id) {
        try {
            Map<String, String> response = followUpService.getFollowup(id);

            try {
                String sql = "UPDATE`wlyy_followup` SET `doctor_read_status`='1' WHERE (`id`='"+id+"') ";
                jdbcTemplate.update(sql);
            }catch (Exception e){
                e.printStackTrace();
            }

            return write(200, "获取随访信息成功！", "data", response);
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "获取随访信息失败！" + e.getMessage());
        }
    }

    @ApiOperation("获取随访项目列表")
    @GetMapping(value = "/getFollowupProject")
    public String getFollowupProject(@ApiParam(name = "id", value = "随访记录ID", defaultValue = "4")
                                     @RequestParam(value = "id", required = true) String id) {
        try {
            List<Map<String, String>> list = followUpService.getFollowupProject(id);

            return write(200, "获取随访项目列表成功！", "data", list);
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "获取随访项目列表失败！" + e.getMessage());
        }
    }

    @ApiOperation("获取随访项目数据")
    @GetMapping(value = "/getFollowupProjectData")
    public String getFollowupProjectData(@ApiParam(name = "id", value = "随访记录ID", defaultValue = "4")
                                         @RequestParam(value = "id", required = true) String id,
                                         @ApiParam(name = "followupProject", value = "随访项目")
                                         @RequestParam(value = "followupProject", required = false) String followupProject) {
        try {
            FollowupContentESDO followupContentESDO = followUpService.esGetFollowupProjectDataList(id, followupProject);
            String jsonString = JSON.toJSONString(followupContentESDO);
            Map maps = (Map)JSON.parse(jsonString);
            return write(200, "获取随访项目数据成功！", "data", maps);
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "获取随访项目数据失败！" + e.getMessage());
        }
    }

    @ApiOperation("保存随访项目数据")
    @PostMapping(value = "/saveFollowupProjectData")
    public String saveFollowupProjectData(@ApiParam(name = "id", value = "随访记录ID", defaultValue = "4")
                                          @RequestParam(value = "id", required = true) String id,
                                          @ApiParam(name = "followupProject", value = "随访项目", defaultValue = "2")
                                          @RequestParam(value = "followupProject", required = false) String followupProject,
                                          @ApiParam(name = "followupProjectData", value = "随访项目数据", defaultValue = "{\"BLOOD_SUGAR\":\"33.3\",\"BLOOD_SUGAR_TYPE\":\"1\",\"WEIGHT\":\"76\",\"WEIGHT_EXP\":\"60\",\"BMI\":\"11\"}")
                                          @RequestParam(value = "followupProjectData", required = true) String followupProjectData) {
        try {
            followUpService.esSaveFollowupProjectData(id, followupProject, followupProjectData);
            return write(200, "保存随访项目数据成功！");
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "保存随访项目数据失败！" + e.getMessage());
        }
    }

    @ApiOperation("完成随访记录")
    @PostMapping(value = "/finishFollowup")
    public String finishFollowup(@ApiParam(name = "id", value = "随访记录ID", defaultValue = "")
                                 @RequestParam(value = "id", required = true) String id) {
        try {
            followUpService.finishFollowup(id);

            return write(200, "完成随访记录成功！");
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "完成随访记录失败！" + e.getMessage());
        }
    }

}
