package com.yihu.jw.door.controller.patient;

import com.yihu.jw.door.controller.BaseController;
import com.yihu.jw.door.dao.common.SignFamilyDao;
import com.yihu.jw.door.service.common.ServerPackageService;
import com.yihu.jw.entity.base.org.BaseOrgDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.door.SignFamily;
import com.yihu.jw.org.dao.BaseOrgDao;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.patient.dao.BasePatientMedicareCardDao;
import com.yihu.jw.util.common.IdCardUtil;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.encrypt.MD5;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * Created by yeshijie on 2020/12/28.
 */
@RestController
@RequestMapping(value = "/patient", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(description = "居民端-居民信息")
public class PatientController extends BaseController {

    @Autowired
    private BasePatientDao patientDao;
    @Autowired
    private SignFamilyDao signFamilyDao;
    @Autowired
    private BaseOrgDao baseOrgDao;
    @Autowired
    private BasePatientMedicareCardDao patientMedicareCardDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private ServerPackageService serverPackageService;

    @RequestMapping(value = "serviceOrder/selectServiceTypes", method = RequestMethod.POST)
    @ApiOperation(value = "获取服务项目类型-上门服务")
    public String selectServiceTypes(){
        try {
            return write(200, "查询成功", "data",serverPackageService.selectPatientTypes());
        } catch (Exception ex) {
            error(ex);
            return error(-1, "查询失败");
        }
    }

    @GetMapping(value = "getProfessionalDict")
    @ApiOperation("职业状态字典")
    public String getProfessionalDict(){
        try {
            String sql = "SELECT code,`value` from base_system_dict_entry WHERE dict_code = 'PROFESSIONAL_STATE' ORDER BY sort";
            List<Map<String,Object>> rs = jdbcTemplate.queryForList(sql);
            return write(200, "获取成功！", "data",rs);
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "获取字典信息失败！");
        }
    }

    @GetMapping(value = "getNationDict")
    @ApiOperation("获取民族信息列表")
    public String getNationDict(){
        try {
            String sql = "SELECT t.NATION_CODE,t.NATION_NAME FROM zy_nation_dict t";
            List<Map<String,Object>> rs = jdbcTemplate.queryForList(sql);
            return write(200, "获取成功！", "data",rs);
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "获取字典信息失败！");
        }
    }

    public static void main(String[] args) {

        System.out.println(MD5.md5Hex("011633{40b56bcdf86f4c03ae230bf2104da9bb}"));
    }

    /**
     * 患者基本信息查询接口
     *
     * @return
     */
    @GetMapping(value = "baseinfo")
    @ApiOperation("患者基本信息查询接口")
    public String baseinfo() {
        try {
            BasePatientDO temp = patientDao.findById(getUID());
            SignFamily jiating;
            synchronized (temp.getId().intern()){
                jiating = signFamilyDao.findByPatientAndType(getUID(), 2);
                if(jiating == null){
                    //泰安业务，如果没有签约，默认签约一个医生
                    jiating = new SignFamily();
                    jiating.setStatus(1);
                    jiating.setCzrq(new Date());
                    jiating.setCode(getUID());
                    String sql = "SELECT" +
                            "        d.id," +
                            "                d.name," +
                            "                dh.org_code," +
                            "                dh.org_name" +
                            "        FROM" +
                            "        base_doctor d" +
                            "        LEFT JOIN base_doctor_hospital dh ON dh.doctor_code = d.id" +
                            "        WHERE" +
                            "        d.enabled = 1" +
                            "        AND dh.org_code = '350211A1002'" +
                            "        AND d.del = 1 limit 1";
                    List<Map<String,Object>> list =  jdbcTemplate.queryForList(sql);
                    jiating.setDoctor(list.get(0).get("id").toString());
                    jiating.setDoctorName(list.get(0).get("name").toString());
                    jiating.setExpensesStatus("1");
                    jiating.setHospital(list.get(0).get("org_code").toString());
                    jiating.setHospitalName(list.get(0).get("org_name").toString());
                    jiating.setSignYear("2020");
                    jiating.setIdcard(temp.getIdcard());
                    jiating.setMobile(temp.getMobile());
                    jiating.setPatient(temp.getId());
                    jiating.setName(temp.getName());
                    jiating.setType(2);
                    signFamilyDao.save(jiating);
                }
            }

            JSONObject json = new JSONObject();
            int prescription = 0;
            int sign = 0;
            String expensesStatus = "";

            if (jiating != null) {
                json.put("signCode",jiating.getCode());
                expensesStatus = jiating.getExpensesStatus();
                BaseOrgDO hospital = baseOrgDao.findByCode(jiating.getHospital());
                if(hospital!=null){
                    json.put("signArea", hospital.getTownCode());
                    json.put("signAreaName", hospital.getTownName());
                }
            }else{
                json.put("signArea", "");
                json.put("signAreaName", "");
            }

            if (temp != null) {
                // 设置患者标识
                json.put("code", temp.getId());
                // 设置患者姓名
                json.put("name", temp.getName());
                // 设置患者头像
                json.put("photo", temp.getPhoto());
                // 设置患者年龄
                json.put("age", IdCardUtil.getAgeByIdcardOrBirthday(temp.getIdcard(),temp.getBirthday()));
                // 设置患者生日
                json.put("birthday", DateUtil.dateToStrShort(temp.getBirthday()));
                // 设置患者性别
                json.put("sex", temp.getSex());
                // 设置手机号码
                json.put("mobile", temp.getMobile());
                // 设置身份证号
                json.put("idcard", IdCardUtil.getIdcardEncode(temp.getIdcard()));//加密过的完整身份证
                // 设置社保卡号
/*                json.put("ssc", temp.getSsc());
                //设置医疗包保险号
                json.put("medicareNumber", temp.getMedicareNumber());*/
                // 设置省
                json.put("province", temp.getProvinceCode());
                // 设置市
                json.put("city", temp.getCityCode());
                // 设置区县
                json.put("area", temp.getTownCode());
                // 设置省名称
                json.put("provinceName", temp.getProvinceName());
                // 设置市名称
                json.put("cityName", temp.getCityName());
                // 设置区县名称
                json.put("areaName", temp.getTownName());
                // 设置街道
                json.put("street", temp.getStreetCode());
                // 设置街道名称
                json.put("streetName", temp.getStreetName());
                // 设置地址
                json.put("address", temp.getAddress());
                // 设置医院
                if(jiating != null){
                    json.put("hospital", jiating.getHospital());
                    json.put("hospitalName",jiating.getHospitalName());
                }else{
                    json.put("hospital", "");
                    json.put("hospitalName","");
                }

                // 设置签约状态：0未签约，1三师，2家庭，3三师和家庭
                json.put("expensesStatus", expensesStatus);
                json.put("sign", sign);
                json.put("prescription", prescription);//是否可以续方咨询0不可以、1可以
/*                json.put("countryCode",temp.getSickVillage()==null?"":temp.getSickVillage());
                json.put("countryName",temp.getSickVillageName()==null?"":temp.getSickVillageName());*/
                json.put("idcardAll", temp.getIdcard());//完整身份证

                //添加居民历史登录记录
/*                List<Patient> otherPatientList = patientService.findByOpenid(temp.getCode(), temp.getOpenid());
                json.put("otherPatientList", otherPatientList);*/
                return write(200, "患者信息查询成功！", "data", json);
            } else {
                return error(-1, "患者信息查询失败！");
            }
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "患者信息查询失败！");
        }
    }
}
