package com.yihu.jw.door.controller.patient;

import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.door.service.DoorOrderService;
import com.yihu.jw.door.service.WlyyDoorPrescriptionService;
import com.yihu.jw.door.service.WlyyDoorServiceOrderService;
import com.yihu.jw.restmodel.ResponseContant;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 上门服务工单控制器
 *
 * @version <pre>
 * Author	Version		Date		Changes
 * Administrator 	1.0        2019年03月20日 	Created
 *
 * </pre>
 * @since 1.
 */
@RestController
@RequestMapping(value = "/patient/wlyyDoorServiceOrder")
@Api(value = "上门服务工单管理", description = "上门服务工单管理服务接口")
public class WlyyDoorServiceOrderController extends EnvelopRestEndpoint {

    @Autowired
    private WlyyDoorServiceOrderService wlyyDoorServiceOrderService;

    @Autowired
    private DoorOrderService doorOrderService;
    @Autowired
    private WlyyDoorPrescriptionService doorPrescriptionService;


    @PostMapping(value = "qucikSendIM")
    @ApiOperation(value = "通过IM向客户端推送消息")
    public Envelop qucikSendIM(
            @ApiParam(name = "orderId", value = "工单id", required = true) @RequestParam String orderId,
            @ApiParam(name = "sendId", value = "发送者code", required = false) @RequestParam String sendId,
            @ApiParam(name = "sendName", value = "发送者姓名", required = false) @RequestParam String sendName,
            @ApiParam(name = "contentType", value = "消息内容类型：1-文本信息，19-服务工单格式化消息", required = true) @RequestParam String contentType,
            @ApiParam(name = "content", value = "消息内容json", required = true) @RequestParam String content
    ) {

        int result = wlyyDoorServiceOrderService.qucikSendIM(orderId, sendId, sendName, contentType, content);
        if (result == -1) {
            return failed("发送失败！");
        }
        return success("发送成功");
    }

    @PostMapping(value = "create")
    @ApiOperation(value = "创建上门服务咨询")
    public Envelop create(@ApiParam(name = "jsonData", value = "Json数据", required = true) @RequestParam String jsonData) {
        JSONObject result = new JSONObject();
        try{
            result = wlyyDoorServiceOrderService.create(jsonData);
        }catch (Exception e){
            e.printStackTrace();
            return failed(e.getMessage());
        }
        if (result.getIntValue(ResponseContant.resultFlag) == ResponseContant.fail) {
            return failed(result.getString(ResponseContant.resultMsg));
        }
        return success(result);
    }

    @PostMapping(value = "delete")
    @ApiOperation(value = "删除")
    public Envelop delete(
            @ApiParam(name = "ids", value = "id串，中间用,分隔", required = true) @RequestParam(value = "ids") String ids) {
        wlyyDoorServiceOrderService.delete(ids.split(","));
        return success("删除成功");
    }

    @PostMapping(value = "update")
    @ApiOperation(value = "更新")
    public Envelop update(@ApiParam(name = "jsonData", value = "Json数据", required = true) @RequestParam String jsonData) {
        JSONObject result = wlyyDoorServiceOrderService.update(jsonData);
        if (result.getIntValue(ResponseContant.resultFlag) == ResponseContant.fail) {
            return failed(result.getString(ResponseContant.resultMsg));
        }
        return success(result.get(ResponseContant.resultMsg));
    }


    @GetMapping(value = "queryDispatcherInfoByPatient")
    @ApiOperation(value = "获取居民所签约的机构的调度员信息")
    public Envelop queryDispatcherInfoByPatient(
            @ApiParam(name = "patient", value = "居民code") @RequestParam(value = "patient", required = true) String patient) {
        JSONObject result = wlyyDoorServiceOrderService.queryDispatcherInfoByPatient(patient);
        if (result.getIntValue(ResponseContant.resultFlag) == ResponseContant.fail) {
            return failed(result.getString(ResponseContant.resultMsg));
        }
        int count = result.getIntValue(ResponseContant.count);
        return success(result.get(ResponseContant.resultMsg));
    }

    @GetMapping(value = "infoList")
    @ApiOperation(value = "查询预约咨询记录")
    public PageEnvelop page(
            @ApiParam(name = "patient", value = "居民code") @RequestParam(value = "patient", required = true) String patient,
            @ApiParam(name = "status", value = "工单状态，状态为全部时不传") @RequestParam(value = "status", required = false) Integer status,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1") @RequestParam(value = "page") int page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15") @RequestParam(value = "size") int size) {
        JSONObject result = wlyyDoorServiceOrderService.queryInfoList(patient, status, page, size);
        if (result.getIntValue(ResponseContant.resultFlag) == ResponseContant.fail) {
            return PageEnvelop.getError(result.getString(ResponseContant.resultMsg), -1);
        }
        int count = result.getIntValue(ResponseContant.count);
        return success((List) result.get(ResponseContant.resultMsg), count, page, size);
    }

    @GetMapping(value = "queryInfoStatusCountList")
    @ApiOperation(value = "统计上门服务工单各状态下的数量列表")
    public Envelop queryInfoStatusCountList(
            @ApiParam(name = "patient", value = "居民code") @RequestParam(value = "patient", required = false) String patient) {
        JSONObject result = wlyyDoorServiceOrderService.queryInfoStatusCountList(patient);
        if (result.getIntValue(ResponseContant.resultFlag) == ResponseContant.fail) {
            return failed(result.getString(ResponseContant.resultMsg));
        }
        return success(result.get(ResponseContant.resultMsg));
    }

    @GetMapping(value = "list")
    @ApiOperation(value = "获取列表")
    public Envelop list(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段") @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件") @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档") @RequestParam(value = "sorts", required = false) String sorts) {
        List  result = new ArrayList();
        try {
            result = wlyyDoorServiceOrderService.search(fields, filters, sorts);
        } catch (ParseException e) {
            e.printStackTrace();
            return failed("查询列表失败！",-1);
        }
        return success(result);
    }

    @GetMapping(value = "queryOne")
    @ApiOperation(value = "查询单个咨询工单详情")
    public ObjEnvelop queryOneDetail(
            @ApiParam(name = "id", value = "工单id") @RequestParam(value = "id", required = true) String id) {
        return success(wlyyDoorServiceOrderService.queryOneDetail(id));
    }

    @GetMapping(value = "twoDimensionalCode")
    @ApiOperation(value = "生成二维码接口")
    public Envelop twoDimensionalCode(
            @ApiParam(name = "id", value = "工单id") @RequestParam(value = "id", required = true) String id) {
        String url = wlyyDoorServiceOrderService.twoDimensionalCode(id);
       if (StringUtils.isNotEmpty(url)){
            return success("获取二维码成功",200,url);
        }else {
            return failed("获取二维码失败,请确认工单",-1);
        }
    }

    @PostMapping(value = "updateOrderInfo")
    @ApiOperation(value = "查看知情同意书，修改状态")
    public Envelop updateOrderInfo(
            @ApiParam(name = "id", value = "工单id") @RequestParam(value = "id", required = true) String id) {
        Integer orderInfo = wlyyDoorServiceOrderService.updateOrderInfo(id);
        if(orderInfo > 0){
            return success("状态修改成功",orderInfo);
        }
        return failed("状态修改失败",-1);
    }

    @GetMapping(value = "countPatientDoorTimes")
    @ApiOperation(value = "获取居民已经享受出诊服务的次数")
    public ObjEnvelop countPatientDoorTimes(
            @ApiParam(name = "patient", value = "居民code") @RequestParam(value = "patient", required = true) String patient) {
        return success(wlyyDoorServiceOrderService.countPatientDoorTimes(patient));
    }

    @PostMapping(value = "payOrder")
    @ApiOperation(value = "居民付款")
    public ObjEnvelop payOrder(
            @ApiParam(name = "orderId ", value = "工单id") @RequestParam(value = "orderId", required = true) String orderId,
            @ApiParam(name = "payWay", value = "支付方式：1-微信支付，2-线下支付") @RequestParam(value = "payWay", required = true) int payWay) {
        return success(wlyyDoorServiceOrderService.payOrder(orderId, payWay));
    }

    @PostMapping("updateDoctorInfo")
    @ApiOperation(value = "修改保存服务医生（更新上门医生）")
    public Envelop updateDoctorInfo(
            @ApiParam(value = "jsonData", name = "jsonData")
            @RequestParam(value = "jsonData", required = false) String jsonData) {
        try {
            wlyyDoorServiceOrderService.updateDoctorInfo(jsonData);
            return success("修改成功");
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    @PostMapping(value = "updateOrderCardInfo")
    @ApiOperation(value = "更新预约简要信息")
    public Envelop updateOrderCardInfo(
            @ApiParam(name = "jsonData", value = "json数据") @RequestParam(value = "jsonData", required = true) String jsonData) {
        JSONObject result = wlyyDoorServiceOrderService.updateOrderCardInfo(jsonData);
        if (result.getIntValue(ResponseContant.resultFlag) == ResponseContant.fail) {
            return failed(result.getString(ResponseContant.resultMsg));
        }
        return success(result.get(ResponseContant.resultMsg));
    }

    @GetMapping(value = "queryOneConclusion")
    @ApiOperation(value = "查询服务小结")
    public Envelop queryOneConclusion(
            @ApiParam(name = "id", value = "工单id") @RequestParam(value = "id", required = true) String id) {
        return success(doorOrderService.getDoorConclusion(id, true));
    }

    @PostMapping(value = "cancelOrder")
    @ApiOperation(value = "取消工单")
    public Envelop cancelOrder(
            @ApiParam(name = "orderId", value = "工单id") @RequestParam(value = "orderId", required = true) String orderId,
            @ApiParam(name = "reason", value = "取消理由") @RequestParam(value = "reason", required = true) String reason) {
        JSONObject result = new JSONObject();
        try{
            result = wlyyDoorServiceOrderService.cancelOrder(orderId, 2, reason,null,null);
            if (result.getIntValue(ResponseContant.resultFlag) == ResponseContant.fail) {
                return failed(result.getString(ResponseContant.resultMsg));
            }
        }catch (Exception e){
            e.printStackTrace();
            return failed(e.getMessage());
        }

        return success(result.get(ResponseContant.resultMsg));

    }

    @GetMapping(value = "queryDoctorList")
    @ApiOperation(value = "服务人员列表（医生列表）")
    public PageEnvelop queryDoctorList(
            @ApiParam(name = "hospital", value = "居民签约机构code") @RequestParam(value = "hospital", required = true) String hospital,
            @ApiParam(name = "doctorName", value = "医生姓名") @RequestParam(value = "doctorName") String doctorName,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1") @RequestParam(value = "page") int page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15") @RequestParam(value = "size") int size) {
        try {
            JSONObject result = wlyyDoorServiceOrderService.queryDoctorList(getUID(),hospital, doctorName, null, page, size);
            if (result.getIntValue(ResponseContant.resultFlag) == ResponseContant.fail) {
                return PageEnvelop.getError(result.getString(ResponseContant.resultMsg), -1);
            }
            int count = result.getIntValue(ResponseContant.count);
            return success((List) result.get(ResponseContant.resultMsg), count, page, size);
        }catch (Exception e){
            e.printStackTrace();
        }
        return PageEnvelop.getSuccess("查询失败");
    }

    @PostMapping("updateIsPatientConfirm")
    @ApiOperation(value = "更新居民确认情况状态")
    public Envelop updateIsPatientConfirm(
            @ApiParam(name = "orderId", value = "工单id")
            @RequestParam(value = "orderId") String orderId,
            @ApiParam(value = "isPatientConfirm", name = "isPatientConfirm")
            @RequestParam(value = "isPatientConfirm") Integer isPatientConfirm) {
        try {
            wlyyDoorServiceOrderService.updateIsPatientConfirm(orderId, isPatientConfirm);
            return success("修改成功");
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    @GetMapping(value = "getPrescriptionByCode")
    @ApiOperation("获取长处方详情")
    public Envelop getPrescriptionByCode(@ApiParam(name = "recipeNo", value = "挂号号",required = true)
                                         @RequestParam String recipeNo) {
        try {
            JSONObject json = doorPrescriptionService.getPrescriptionByCode(recipeNo, getUID());
            return success(json);
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    @GetMapping(value = "getProxyPatientList")
    @ApiOperation("获取代预约服务对象列表")
    public Envelop getProxyPatientList() {
        try {
            List<Map<String,Object>> list = wlyyDoorServiceOrderService.getProxyPatientList(getUID());
            return success(list);
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    @GetMapping(value = "getServiceItemFees")
    @ApiOperation("获取未付款的服务项")
    public Envelop getServiceItemFees(@ApiParam(name = "orderId", value = "订单id",required = true)
                                         @RequestParam String orderId) {
        try {
            JSONObject json = wlyyDoorServiceOrderService.getServiceItemFees(orderId);
            return success(json);
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    @PostMapping(value = "payServiceItemFees")
    @ApiOperation("付服务项费用")
    public Envelop payServiceItemFees(@ApiParam(name = "orderId", value = "订单id",required = true)
                                          @RequestParam String orderId,
                                      @ApiParam(name = "ids", value = "订单id",required = true)
                                      @RequestParam String[] ids) {
        try {
            JSONObject json = wlyyDoorServiceOrderService.payServiceItemFees(ids,orderId);
            return success(json);
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

}