package com.yihu.jw.door.service;


import com.yihu.jw.door.dao.DoctorRegistrationDao;
import com.yihu.jw.door.dao.DoctorRegistrationTempDao;
import com.yihu.jw.entity.door.DoctorRegistration;
import com.yihu.jw.entity.door.DoctorRegistrationTemp;
import com.yihu.mysql.query.BaseJpaService;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 医生登记服务模板
 * Created by wangpeiqiang on 2019/3/12.
 */
@Service
@Transactional
public class DoctorRegistrationTempService extends BaseJpaService<DoctorRegistrationTemp, DoctorRegistrationTempDao> {

    @Autowired
    DoctorRegistrationTempDao doctorRegistrationTempDao;
    @Autowired
    JdbcTemplate jdbcTemplate;
    @Autowired
    DoctorRegistrationDao doctorRegistrationDao;

    public  List<Map<String, Object>> getRegistrationTempData(String filter) {
        //List<Map> doctorRegistrationList =  doctorRegistrationDao.findAll();
        String sql = "select id,code,model_name,create_time,is_hide from wlyy_doctor_registration";
        List<Map<String, Object>> doctorRegistrationList = jdbcTemplate.queryForList(sql);
        List<DoctorRegistrationTemp> doctorRegistrationTempList = this.doctorRegistrationTempDao.findAllRegistrationTemp("%"+filter+"%");
        if(doctorRegistrationList!=null && doctorRegistrationList.size()>0){
            for(Map map:doctorRegistrationList){
                String code = map.get("code").toString();
                List<DoctorRegistrationTemp> secRegistrationTempList = this.getChildrenList(doctorRegistrationTempList,code);
                map.put("secRegistrationTempList",secRegistrationTempList);
            }
        }
        return doctorRegistrationList;
    }

    public  List<Map<String, Object>> getListByDoctorCode(String doctor,String filter) {
        //List<Map> doctorRegistrationList =  doctorRegistrationDao.findAll();
        String sql = "select id,code,model_name,create_time,is_hide from wlyy_doctor_registration";
        List<Map<String, Object>> doctorRegistrationList = jdbcTemplate.queryForList(sql);
//        List<DoctorRegistrationTemp> doctorRegistrationTempList = this.doctorRegistrationTempDao.findAllByDoctorCode(doctor,"%"+filter+"%");

        if(doctorRegistrationList!=null && doctorRegistrationList.size()>0){
            for(Map map:doctorRegistrationList){
                String code = map.get("code").toString();
                List<DoctorRegistrationTemp> secRegistrationTempList = doctorRegistrationTempDao.findRegistrationTempByRegistration(code);
//                List<DoctorRegistrationTemp> secRegistrationTempList = this.getChildrenList(doctorRegistrationTempList,code);
                map.put("secRegistrationTempList",secRegistrationTempList);
            }
        }
        return doctorRegistrationList;
    }

    private List<DoctorRegistrationTemp> getChildrenList(List<DoctorRegistrationTemp> doctorRegistrationTempList, String code) {
        List<DoctorRegistrationTemp> doctorRegistrationTempArrayList = new ArrayList<>();
        for (DoctorRegistrationTemp doctorRegistrationTemp: doctorRegistrationTempList) {
            if(doctorRegistrationTemp.getRegistration().equals(code)){
                doctorRegistrationTempArrayList.add(doctorRegistrationTemp);
            }
        }
        return doctorRegistrationTempArrayList;
    }


    public List<DoctorRegistration> findAllRegistrationList() throws Exception{
        List<DoctorRegistration> list =  doctorRegistrationDao.findAll();
        return list;
    }

    /**
     * 添加登记服务模板
     *
     * @param doctor  医生
     * @param content 登记服务内容
     * @return
     */
    public DoctorRegistrationTemp add(String doctor, String content, String registration) throws Exception{
        DoctorRegistrationTemp doctorRegistrationTemp = new DoctorRegistrationTemp();
        doctorRegistrationTemp.setCode(getCode());
        doctorRegistrationTemp.setDoctor(doctor);
        doctorRegistrationTemp.setContent(content);
        doctorRegistrationTemp.setSendTimes(0);
        doctorRegistrationTemp.setCreateTime(new Date());
        doctorRegistrationTemp.setRegistration(registration);
        return doctorRegistrationTempDao.save(doctorRegistrationTemp);
    }

    /**
     * 查询单个登记服务模板详情
     *
     * @param modelCode
     * @returnxiang
     * @throws Exception
     */
    public JSONObject listDetail(String modelCode) throws Exception {
        String sql =  "SELECT a.`model_name`,b.`content`,b.`create_time`,b.`send_times`" +
                "FROM wlyy_doctor_registration as a LEFT JOIN wlyy_doctor_registration_temp as b ON a.`code` =b.`registration`" +
                "WHERE b.`code`=?";

        Map temp = (Map) jdbcTemplate.queryForMap(sql, modelCode);
        String modelName = (String) temp.get("model_name");
        String content = (String) temp.get("content");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date createTime = (Date) temp.get("create_time");
        String dateTime = sdf.format(createTime);
        String sendTimes = (String) temp.get("send_times").toString();
        JSONObject json = new JSONObject();
        json.put("modelName", modelName);
        json.put("content", content);
        json.put("createTime", dateTime);
        json.put("sendTimes", sendTimes);
        return json;
    }

    /**
     * 修改登记服务模板
     *
     * @param code    模板编码
     * @param content 内容
     * @return
     */
    public DoctorRegistrationTemp modify(String code, String content) throws Exception {
        DoctorRegistrationTemp doctorRegistrationTemp = doctorRegistrationTempDao.findByCode(code);

        if (doctorRegistrationTemp == null) {
            throw new Exception("模板不存在");
        }

        doctorRegistrationTemp.setContent(content);

        return doctorRegistrationTempDao.save(doctorRegistrationTemp);
    }

    /**
     * 删除登记服务模板
     *
     * @param code 模板编码
     * @return
     */
    public int delete(String code) throws Exception {
        String[] codes = code.split(",");

        if (codes != null && codes.length > 0) {
            for (String temp : codes) {
                String sql = "SELECT t.doctor FROM wlyy_doctor_registration_temp t WHERE t.code=?";
                String doctor = jdbcTemplate.queryForObject(sql, String.class, temp);
                if (!"system".equals(doctor)) {
                    doctorRegistrationTempDao.deleteByCode(temp);
                }

            }
        }
        return 1;
    }
}
