package com.yihu.jw.door.service;

import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.door.dao.WlyyDoorFeeDetailDao;
import com.yihu.jw.entity.door.WlyyDoorFeeDetailDO;
import com.yihu.jw.restmodel.ResponseContant;
import com.yihu.jw.util.entity.EntityUtils;
import com.yihu.mysql.query.BaseJpaService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * 
 * 服务工单价格明细（服务项价格，医生出诊费用）服务service
 * 
 * @version 
 * <pre>
 * Author	Version		Date		Changes
 * Administrator    1.0  2019年03月20日 Created
 *
 * </pre>
 * @since 1.
 */
@Service
public class WlyyDoorFeeDetailService extends BaseJpaService<WlyyDoorFeeDetailDO, WlyyDoorFeeDetailDao> {

    private Logger logger = LoggerFactory.getLogger(WlyyDoorFeeDetailService.class);

    @Autowired
    private WlyyDoorFeeDetailDao wlyyDoorFeeDetailDao;

    @Autowired
    private JdbcTemplate jdbcTemplate;

      /**
     * 判断某个服务项数量是否大于2
     * @param id
     * @return
     */
    public boolean numberGrateThan2(String id){
        if(StringUtils.isEmpty(id)){
            return false;
        }
        WlyyDoorFeeDetailDO feeDetailDO = wlyyDoorFeeDetailDao.findOne(id);
        if(null == feeDetailDO){
            return false;
        }
        return feeDetailDO.getNumber() > 1 ? true : false;
    }



    //服务工单价格明细（服务项价格，医生出诊费用）服务基本信息
    private WlyyDoorFeeDetailDO jsonToWlyyDoorFeeDetail(JSONObject result, JSONObject jsonObjectParam, String jsonData) {
        WlyyDoorFeeDetailDO wlyyDoorFeeDetail;
        try {
            wlyyDoorFeeDetail = EntityUtils.jsonToEntity(jsonObjectParam.get("wlyyDoorFeeDetail").toString(),WlyyDoorFeeDetailDO.class);
        } catch (Exception e) {
            result.put(ResponseContant.resultFlag,ResponseContant.fail);
            String failMsg = "服务工单价格明细（服务项价格，医生出诊费用）服务基本信息："  + e.getMessage();
            result.put(ResponseContant.resultMsg,failMsg);
            logger.error(failMsg);
            return new WlyyDoorFeeDetailDO();
        }

        this.save(wlyyDoorFeeDetail);
        return wlyyDoorFeeDetail;
    }

     /**
     * 创建服务工单价格明细（服务项价格，医生出诊费用）
     * @param jsonData
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public JSONObject create(String jsonData){
        JSONObject result = new JSONObject();
        JSONObject jsonObjectParam ;

        try {
            jsonObjectParam = JSONObject.parseObject(jsonData);
        } catch (Exception e) {
            result.put(ResponseContant.resultFlag,ResponseContant.fail);
            String failMsg = "参数转换成JSON对象异常：" + e.getMessage();
            result.put(ResponseContant.resultMsg,failMsg);
            logger.error(failMsg);
            return result;
        }

        WlyyDoorFeeDetailDO wlyyDoorFeeDetail = new WlyyDoorFeeDetailDO();
        wlyyDoorFeeDetail = jsonToWlyyDoorFeeDetail(result, jsonObjectParam,jsonData);

        result.put(ResponseContant.resultFlag,ResponseContant.success);
        result.put(ResponseContant.resultMsg,wlyyDoorFeeDetail);
        return result;
    }

    /**
     * 服务工单价格明细（服务项价格，医生出诊费用）（包括角色分配的菜单，模块等）
     * @param jsonData
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public JSONObject update(String jsonData){
        JSONObject result = new JSONObject();
        JSONObject jsonObjectParam ;

        try {
            jsonObjectParam = JSONObject.parseObject(jsonData);
        } catch (Exception e) {
            result.put(ResponseContant.resultFlag,ResponseContant.fail);
            String failMsg = "参数转换成JSON对象异常：" + e.getMessage();
            result.put(ResponseContant.resultMsg,failMsg);
            logger.error(failMsg);
            return result;
        }

        WlyyDoorFeeDetailDO wlyyDoorFeeDetail = new WlyyDoorFeeDetailDO();
        wlyyDoorFeeDetail = jsonToWlyyDoorFeeDetail(result, jsonObjectParam,jsonData);

        result.put(ResponseContant.resultFlag,ResponseContant.success);
        result.put(ResponseContant.resultMsg,wlyyDoorFeeDetail);
        return result;
    }


   /**
     * 查询服务工单价格明细（服务项价格，医生出诊费用）列表
     *
     * @return
     */
    public JSONObject queryInfoList(String fields, String filters, String sorts, int page, int size) {
        JSONObject result = new JSONObject();

        int start = 0 == page ? page++ : (page - 1) * size;
        int end = 0 == size ? 15 : page * size;

        String sql = "";

        String finalSql = sql.replace("{fields}", fields)
                .replace("{start}", String.valueOf(start))
                .replace("{end}", String.valueOf(end));

        String countSql = "";

        String finqlCountSql = countSql.replace("{filters}", filters)
                .replace("{start}", String.valueOf(start));

        List<WlyyDoorFeeDetailDO> sqlResultlist = new ArrayList<>();
    try {
        sqlResultlist = jdbcTemplate.query(finalSql, new BeanPropertyRowMapper<>(WlyyDoorFeeDetailDO.class));
    } catch (Exception e) {
        result.put(ResponseContant.resultFlag, ResponseContant.fail);
        result.put(ResponseContant.resultMsg, "从数据库服务工单价格明细（服务项价格，医生出诊费用）列表信息失败：" + e.getMessage());
        return result;
    }

    Integer count = 0;
    try {
        count = jdbcTemplate.queryForObject(finqlCountSql, Integer.class);
    } catch (Exception e) {
        result.put(ResponseContant.resultFlag, ResponseContant.fail);
        result.put(ResponseContant.resultMsg, "从数据库统计服务工单价格明细（服务项价格，医生出诊费用）数量失败：" + e.getMessage());
        return result;
    }

    result.put(ResponseContant.resultFlag, ResponseContant.success);
    result.put(ResponseContant.resultMsg, sqlResultlist);
    JSONObject countItem = new JSONObject();
    countItem.put("count", count);
    result.putAll(countItem);

    return result;
    }
}
