package com.yihu.jw.door.service;

import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.door.dao.WlyyDoorProcessLogDao;
import com.yihu.jw.entity.door.WlyyDoorProcessLogDO;
import com.yihu.jw.restmodel.ResponseContant;
import com.yihu.jw.util.entity.EntityUtils;
import com.yihu.mysql.query.BaseJpaService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * 
 * 工单状态变更记录（调度员操作记录）服务service
 * 
 * @version 
 * <pre>
 * Author	Version		Date		Changes
 * Administrator    1.0  2019年04月09日 Created
 *
 * </pre>
 * @since 1.
 */
@Service
public class WlyyDoorProcessLogService extends BaseJpaService<WlyyDoorProcessLogDO, WlyyDoorProcessLogDao> {

    private Logger logger = LoggerFactory.getLogger(WlyyDoorProcessLogService.class);

    @Autowired
    private JdbcTemplate jdbcTemplate;


    //工单状态变更记录（调度员操作记录）服务基本信息
    private WlyyDoorProcessLogDO jsonToWlyyDoorProcessLog(JSONObject result, JSONObject jsonObjectParam,String jsonData) {
        WlyyDoorProcessLogDO wlyyDoorProcessLog;
        try {
            wlyyDoorProcessLog = EntityUtils.jsonToEntity(jsonObjectParam.get("wlyyDoorProcessLog").toString(),WlyyDoorProcessLogDO.class);
        } catch (Exception e) {
            result.put(ResponseContant.resultFlag,ResponseContant.fail);
            String failMsg = "工单状态变更记录（调度员操作记录）服务基本信息："  + e.getMessage();
            result.put(ResponseContant.resultMsg,failMsg);
            logger.error(failMsg);
            return new WlyyDoorProcessLogDO();
        }

        this.save(wlyyDoorProcessLog);
        return wlyyDoorProcessLog;
    }

     /**
     * 创建工单状态变更记录（调度员操作记录）
     * @param jsonData
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public JSONObject create(String jsonData){
        JSONObject result = new JSONObject();
        JSONObject jsonObjectParam ;

        try {
            jsonObjectParam = JSONObject.parseObject(jsonData);
        } catch (Exception e) {
            result.put(ResponseContant.resultFlag,ResponseContant.fail);
            String failMsg = "参数转换成JSON对象异常：" + e.getMessage();
            result.put(ResponseContant.resultMsg,failMsg);
            logger.error(failMsg);
            return result;
        }

        WlyyDoorProcessLogDO wlyyDoorProcessLog = new WlyyDoorProcessLogDO();
        wlyyDoorProcessLog = jsonToWlyyDoorProcessLog(result, jsonObjectParam,jsonData);
        result.put(ResponseContant.resultFlag,ResponseContant.success);
        result.put(ResponseContant.resultMsg,wlyyDoorProcessLog);
        return result;
    }

    /**
     * 工单状态变更记录（调度员操作记录）（包括角色分配的菜单，模块等）
     * @param jsonData
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public JSONObject update(String jsonData){
        JSONObject result = new JSONObject();
        JSONObject jsonObjectParam ;

        try {
            jsonObjectParam = JSONObject.parseObject(jsonData);
        } catch (Exception e) {
            result.put(ResponseContant.resultFlag,ResponseContant.fail);
            String failMsg = "参数转换成JSON对象异常：" + e.getMessage();
            result.put(ResponseContant.resultMsg,failMsg);
            logger.error(failMsg);
            return result;
        }

        WlyyDoorProcessLogDO wlyyDoorProcessLog = new WlyyDoorProcessLogDO();
        wlyyDoorProcessLog = jsonToWlyyDoorProcessLog(result, jsonObjectParam,jsonData);
        result.put(ResponseContant.resultFlag,ResponseContant.success);
        result.put(ResponseContant.resultMsg,wlyyDoorProcessLog);
        return result;
    }


   /**
     * 查询工单状态变更记录（调度员操作记录）列表
     *
     * @return
     */
    public JSONObject queryInfoList(String fields, String filters, String sorts, int page, int size) {
        JSONObject result = new JSONObject();

        int start = 0 == page ? page++ : (page - 1) * size;
        int end = 0 == size ? 15 : page * size;

        String sql = "";

        String finalSql = sql.replace("{fields}", fields)
                .replace("{start}", String.valueOf(start))
                .replace("{end}", String.valueOf(end));

        String countSql = "";

        String finqlCountSql = countSql.replace("{filters}", filters)
                .replace("{start}", String.valueOf(start));

        List<WlyyDoorProcessLogDO> sqlResultlist = new ArrayList<>();
    try {
        sqlResultlist = jdbcTemplate.query(finalSql, new BeanPropertyRowMapper<>(WlyyDoorProcessLogDO.class));
    } catch (Exception e) {
        result.put(ResponseContant.resultFlag, ResponseContant.fail);
        result.put(ResponseContant.resultMsg, "从数据库工单状态变更记录（调度员操作记录）列表信息失败：" + e.getMessage());
        return result;
    }

    Integer count = 0;
    try {
        count = jdbcTemplate.queryForObject(finqlCountSql, Integer.class);
    } catch (Exception e) {
        result.put(ResponseContant.resultFlag, ResponseContant.fail);
        result.put(ResponseContant.resultMsg, "从数据库统计工单状态变更记录（调度员操作记录）数量失败：" + e.getMessage());
        return result;
    }

    result.put(ResponseContant.resultFlag, ResponseContant.success);
    result.put(ResponseContant.resultMsg, sqlResultlist);
    JSONObject countItem = new JSONObject();
    countItem.put("count", count);
    result.putAll(countItem);

    return result;
    }
}
