package com.yihu.jw.door.service.common;

import com.yihu.jw.door.dao.common.SignFamilyDao;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.base.patient.PatientMedicareCardDO;
import com.yihu.jw.entity.door.SignFamily;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.patient.dao.BasePatientMedicareCardDao;
import com.yihu.jw.util.common.IdCardUtil;
import com.yihu.jw.util.date.DateUtil;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * Created by yeshijie on 2020/12/31.
 */
@Service
public class PatientService {
    @Autowired
    private BasePatientDao patientDao;
    @Autowired
    private SignFamilyDao signFamilyDao;
    @Autowired
    private BasePatientMedicareCardDao patientMedicareCardDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;


    public List<Map<String,Object>> patientList(String name, Integer page, Integer size){
        List<Map<String,Object>> resultList = new ArrayList<>(32);
        String sql = "select p.* from base_patient p where p.del = '1' and p.enabled = 1 ";
        if(StringUtils.isNoneBlank(name)){
            sql += " and p.name like '%"+name+"%'";
        }

        int start = 0 == page ? page++ : (page - 1) * size;
        int end = 0 == size ? 15 : size;
        sql += " limit "+start+","+end;

        List<BasePatientDO> list = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(BasePatientDO.class));
        for (BasePatientDO p:list){
            Map<String,Object> json = new HashedMap();
            json.put("id", p.getId());
            json.put("code", p.getId());
            // 设置患者姓名
            json.put("name", p.getName());
            // 设置患者手机号
            json.put("mobile", p.getMobile());
            // 设置患者微信openid
            json.put("openid", StringUtils.isNotEmpty(p.getOpenid()) ? p.getOpenid() : "");
            json.put("openidTime", DateUtil.dateToStr(p.getOpenidTime(), DateUtil.YYYY_MM_DD_HH_MM_SS));
            // 设置患者联系电话
            json.put("phone", p.getPhone());
            // 设置患者头像
            json.put("photo", p.getPhoto());
            // 设置患者年龄
            json.put("age", IdCardUtil.getAgeByIdcardOrBirthday(p.getIdcard(),p.getBirthday()));
            // 设置患者性别
            json.put("sex", p.getSex());
            // 身份证号
            json.put("idcard", p.getIdcard());
            //1.4.2新增medicareNumber
//        json.put("medicareNumber", p.getMedicareNumber());
            // 设置患者居住省份
            json.put("provinceName", p.getProvinceName());
            json.put("cityName", p.getCityName());
            json.put("townName", p.getTownName());
            json.put("streetName", p.getStreetName());
            // 设置患者地址
            json.put("address", p.getAddress());
            resultList.add(json);
        }

        return resultList;
    }

    /**
     * 查询单个居民信息
     *
     * @param patient
     * @return
     * @throws Exception
     */
    public JSONObject getPatient(String patient, String doctor) throws Exception {
        JSONObject json = new JSONObject();
        BasePatientDO p = patientDao.findById(patient);

        PatientMedicareCardDO medicareCardDO = patientMedicareCardDao.findByTypeAndPatientCodeAndDel("A_01",patient,"1");
        if (p == null) {
            throw new Exception("patient info can not find");
        }


        // 设置患者标识
        json.put("code", p.getId());
        // 设置患者姓名
        json.put("name", p.getName());
        // 设置患者手机号
        json.put("mobile", p.getMobile());
        // 设置患者微信openid
        json.put("openid", StringUtils.isNotEmpty(p.getOpenid()) ? p.getOpenid() : "");
        json.put("openidTime", DateUtil.dateToStr(p.getOpenidTime(), DateUtil.YYYY_MM_DD_HH_MM_SS));
        // 设置患者联系电话
        json.put("phone", p.getPhone());
        // 设置患者头像
        json.put("photo", p.getPhoto());
        // 设置患者年龄
        json.put("age", IdCardUtil.getAgeByIdcardOrBirthday(p.getIdcard(),p.getBirthday()));
        // 设置患者性别
        json.put("sex", p.getSex());
        // 身份证号
        json.put("idcard", p.getIdcard());
        //1.4.2新增medicareNumber
//        json.put("medicareNumber", p.getMedicareNumber());
        // 设置患者居住省份
        json.put("provinceName", p.getProvinceName());
        json.put("cityName", p.getCityName());
        json.put("townName", p.getTownName());
        json.put("streetName", p.getStreetName());
        // 设置患者地址
        json.put("address", p.getAddress());
        // 社保号/医保卡
        if(medicareCardDO!=null){
            json.put("ssc", medicareCardDO.getCode());
        }

        //病情类型：0健康，1高血压，2糖尿病，(1,2)高血压+糖尿病
/*        json.put("disease",p.getDisease());
        // 病情：0绿标，1黄标，2红标，
        json.put("diseaseCondition",p.getDiseaseCondition());
        //预警状态
        json.put("standardStatus",p.getStandardStatus());
        //设备状态：0未绑定，1血糖仪，2血压仪，3血糖仪+血压仪
        json.put("deviceType",p.getDeviceType()==null?"":p.getDeviceType());

        //20190719 增加返回字段：档案状态（-2冻结 1未管理 2死亡3 正常4 高危）
        json.put("archiveStatus",p.getArchiveStatus() == null? "":p.getArchiveStatus().toString());*/


        SignFamily familySign = signFamilyDao.findSignByPatient(patient, 2);

        if (familySign != null) {
            // 设置患者紧急联系人
            json.put("signCode",familySign.getCode());

            json.put("emerMobile", StringUtils.isEmpty(familySign.getEmerMobile()) ? "" : String.valueOf(familySign.getEmerMobile()));
            // 设置签约日期
            json.put("qyrq", familySign.getApplyDate() != null ? DateUtil.dateToStr((Date) familySign.getApplyDate(), DateUtil.YYYY_MM_DD) : "");
            // 设置签约日期
            json.put("patientApplyDate", familySign.getPatientApplyDate() != null ? DateUtil.dateToStr((Date) familySign.getPatientApplyDate(), DateUtil.YYYY_MM_DD) : "");
            // 缴费情况
            json.put("expensesStatus", StringUtils.isNotEmpty(familySign.getExpensesStatus()) ? String.valueOf(familySign.getExpensesStatus()) : "0");
            // 缴费时间
            json.put("expensesTime", familySign.getExpensesTime() != null ? DateUtil.dateToStr((Date) familySign.getExpensesTime(), DateUtil.YYYY_MM_DD_HH_MM) : "");
            // 缴费类型
            json.put("expensesType", StringUtils.isNotEmpty(familySign.getExpensesType()) ? String.valueOf(familySign.getExpensesType()) : "");
            // 设置签约状态
            json.put("familyStatus", familySign.getStatus());
            json.put("jtBeginDate", DateUtil.dateToStr(familySign.getBegin(), DateUtil.YYYY_MM_DD));
            json.put("jtEndDate", DateUtil.dateToStr(familySign.getEnd(), DateUtil.YYYY_MM_DD));


        }



/*        boolean epTime = false;
        try {
            epTime = redisTemplate.opsForSet().isMember("wechat:focus:remind:set", p.getCode());
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (!epTime) {
            json.put("wechatFocusRemind", 0);
        } else {
            json.put("wechatFocusRemind", 1);
        }*/

/*        json.put("countryCode",p.getSickVillage()==null?"":p.getSickVillage());
        json.put("countryName",p.getSickVillageName()==null?"":p.getSickVillageName());*/

/*        //1.4.2 是否追踪居民健康体征
        if(StringUtils.isNotBlank(doctor)){
            TrackPatient tp = trackPatientDao.findByDoctorCodeAndPatientCode(doctor,patient);
            if(tp!=null){
                json.put("isTrack",tp.getDel());
            }else{
                json.put("isTrack","0");
            }
        }
        //1.4.9  是否专科医生服务
        JSONArray jsonArray =specialistService.findPatientSignSpecialist(patient);
        if(jsonArray!=null&&jsonArray.length()>0){
            json.put("isSpecialist",1);
            json.put("specialistArray", jsonArray);
        }else{
            json.put("isSpecialist",0);
        }
        //1.5.0 是否有康复计划
        JSONArray planList = rehabilitationManageService.planListByPatient(patient);
        if(planList!=null&&planList.length()>0){
            json.put("havePlan",1);
        }else{
            json.put("havePlan",0);
        }*/
        return json;
    }
}
