package com.yihu.jw.door.service.followup;

import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.door.dao.followup.FollowupDrugsDao;
import com.yihu.jw.entity.followup.FollowupDrugs;
import com.yihu.jw.followup.FollowupContentESDO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.transaction.Transactional;
import java.util.*;

/**
 * Created by yeshijie on 2021/1/11.
 */
@Component
@Transactional(rollbackOn = Exception.class)
public class FollowupDrugsService {

    @Autowired
    private FollowupDrugsDao followupDrugsDao;
    @Autowired
    private FollowUpService followUpService;
    @Autowired
    private ObjectMapper objectMapper;

    /**
     * 获取面访用药数据
     */
    public Map<String,Object> getFollowupDrugs(String id) throws Exception
    {
        Map<String,Object> re = new HashMap<>();

        FollowupContentESDO followupContentESDO = followUpService.esGetFollowupProjectData(id, "6");
        if(followupContentESDO != null){
            re.put("DRUG_COMPLIANCE_CODE",followupContentESDO.getDRUG_COMPLIANCE_CODE());
        }

        //获取用药记录
        List<FollowupDrugs> drugsList = followupDrugsDao.findByFollowupId(Long.valueOf(id));
        re.put("DRUG_LIST",drugsList);

        return re;
    }


    /**
     *保存面访用药数据
     */
    @Transactional
    public void saveFollowupDrugs(String id,String drugsData) throws Exception {
        JavaType javaType = objectMapper.getTypeFactory().constructParametricType(List.class, Map.class);
        List<Map<String,String>> list = objectMapper.readValue(drugsData,javaType);

        Long followupId = Long.valueOf(id);
        //删除原有用药记录
        followupDrugsDao.deleteByFollowupId(Long.valueOf(followupId));

        if(list!=null && list.size()>0)
        {
            List<FollowupDrugs> newList = new ArrayList<>();
            for(Map<String,String> item:list)
            {
                FollowupDrugs drug = new FollowupDrugs();
                drug.setFollowupId(followupId);
                drug.setDrugsGroup(item.get("drugsGroup"));
                drug.setDrugsCode(item.get("drugsCode"));
                drug.setDrugsName(item.get("drugsName"));
                drug.setDose(item.get("dose"));
                drug.setUnit(item.get("unit"));
                drug.setFrequency(item.get("frequency"));
                drug.setCreateTime(new Date());

                newList.add(drug);
            }
            followupDrugsDao.save(newList);
        }
    }
}
