package com.yihu.jw.door.service.prescription;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.door.dao.WlyyDoorPrescriptionDao;
import com.yihu.jw.door.service.WlyyDoorPrescriptionService;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.door.WlyyDoorPrescriptionDO;
import com.yihu.jw.hospital.mapping.dao.DoctorMappingDao;
import com.yihu.jw.message.dao.MessageDao;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.util.http.HttpClientUtil;
import com.yihu.jw.util.security.MD5;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 基位思明上门服务开方接口
 * Created by hmf on 2019/9/19.
 */
@Service
public class JwDoorPrescriptionService {
    private static final Logger logger = LoggerFactory.getLogger(JwDoorPrescriptionService.class);
    //基卫服务地址
    private String jwUrl="";
    @Autowired
    private HttpClientUtil httpClientUtil;
    @Autowired
    private BasePatientDao patientDao;
    @Autowired
    private BaseDoctorDao doctorDao;
    @Autowired
    private DoctorMappingDao doctorMappingDao;
    @Autowired
    private WlyyDoorPrescriptionDao doorPrescriptionDao;
    @Autowired
    private MessageDao messageDao;
    @Autowired
    private WlyyDoorPrescriptionService doorPrescriptionService;

    /**
     * 获取历史就诊记录
     * @param doctor 开方医生
     * @param patient 居民
     * @param visitDept 开方科室
     * @param visitTimeFrom
     * @param visitTimeEnd
     * @return
     */
    public String getVisitRecordListUnion(String doctor, String patient, String visitDept, String visitTimeFrom, String visitTimeEnd) throws Exception {
        String re = "";
        //获取居民社保卡号和签约机构
        BasePatientDO p = patientDao.findById(patient);
//        DoctorMappingDO doctorMapping = doctorMappingDao.findListByDoctorCode(doctor).get(0);
        String url = jwUrl + "/third/doorPrescription/getVisitRecordListUnion";
        List<NameValuePair> params = new ArrayList<>();
//        params.add(new BasicNameValuePair("cardNo", p.getSsc()));
//        params.add(new BasicNameValuePair("visitOperator", doctorMapping.getMappingCode()));
        params.add(new BasicNameValuePair("visitDept", visitDept));
        params.add(new BasicNameValuePair("visitTimeFrom", visitTimeFrom));
        params.add(new BasicNameValuePair("visitTimeEnd", visitTimeEnd));

        String response = httpClientUtil.post(url, params, "UTF-8");
        if (!StringUtils.isEmpty(response)) {
            org.json.JSONObject jsonObject = new org.json.JSONObject(response);
            int status = jsonObject.getInt("status");
            if (status == 200) {
                String data = jsonObject.getString("data");
                if (!StringUtils.isEmpty(data) && (data.startsWith("error")||data.startsWith("System-Error"))) {
                    throw new Exception(data);
                } else {
                    re = data;
                }
            } else {
                throw new Exception(jsonObject.getString("msg"));
            }
        } else {
            throw new Exception("null response.");
        }
        return re;
    }

    /**
     * 获取处方明细
     * @param visitNo
     * @param hospital
     * @return
     */
    public String getVisitRecipeDetailUnion(String visitNo, String recipeNo, String hospital){
        String url = jwUrl + "/third/doorPrescription/getVisitRecipeDetailUnion";
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("visitNo", visitNo));
        params.add(new BasicNameValuePair("recipeNo", recipeNo));
        params.add(new BasicNameValuePair("hospital", hospital));
        String response = httpClientUtil.post(url, params, "UTF-8");

        return response;
    }

    /**
     * 处方开方接口
     * @param prescriptionCode 处方code
     * @return
     * @throws Exception
     */
    public String saveRecipeUnion(String prescriptionCode,String visitOperator,String visitDept){
        String response = null;
        try {
            String url = jwUrl + "/third/doorPrescription/saveRecipeUnion";
            List<NameValuePair> params = new ArrayList<>();
            params.add(new BasicNameValuePair("prescriptionCode", prescriptionCode));
            params.add(new BasicNameValuePair("visitDept", visitDept));
            params.add(new BasicNameValuePair("visitOperator", visitOperator));

            response = httpClientUtil.post(url, params, "UTF-8");
            return response;
        }catch (Exception e){
            logger.info("处方开方接口saveRecipeUnion:"+response);
            throw e;
        }
    }

    /**
     * 取消处方接口
     * @param prescriptionCode 处方code
     * @return
     * @throws Exception
     */
    public JSONObject fadeRecipeUnion(String prescriptionCode, String doctor){
        JSONObject result = new JSONObject();
        WlyyDoorPrescriptionDO prescription = doorPrescriptionDao.findByCode(prescriptionCode);
        BaseDoctorDO d = doctorDao.findById(doctor);
//        DoctorMapping doctorMapping = doctorMappingDao.findByDoctorCode(doctor);
        try {
            if (StringUtils.isNotBlank(prescription.getVisitNo())) {
                String url = jwUrl + "/third/doorPrescription/fadeRecipeUnion";
                List<NameValuePair> params = new ArrayList<>();
                params.add(new BasicNameValuePair("visitNo", prescription.getVisitNo()));//挂号号
                params.add(new BasicNameValuePair("fadeDept", prescription.getJwDeptCode()));//作废科室编码
//                params.add(new BasicNameValuePair("fadeOperator", doctorMapping.getJwDoctor()));//作废人员编码
//                params.add(new BasicNameValuePair("hospital", d.getHospital()));//机构编码
                String response = httpClientUtil.post(url, params, "UTF-8");
                if (StringUtils.isNotBlank(response)) {
                    JSONObject reobj = JSON.parseObject(response);
                    Integer status = reobj.getInteger("status");
                    String errmsg = reobj.getString("msg");
                    if (200 != status) {
                        result.put("status", -1);
                        result.put("msg", "基卫接口（挂号作废）请求失败," + errmsg);
                        return result;
                    }
                    result.put("status", 200);
                    result.put("msg", "基卫接口（挂号作废）请求成功！");
                    prescription.setStatus(-1);
                    prescription.setUpdateTime(new Date());
                    doorPrescriptionDao.save(prescription);
                } else {
                    result.put("status", -1);
                    result.put("msg", "基卫接口（挂号作废）请求失败，无数据返回！");
                    return result;
                }
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        return result;
    }

    /**
     * CA认证通过接口
     *
     * @return
     */
    public org.json.JSONObject requestRealNameSoftCertAndSign(String doctorCode, String strRealNameSoftCertCalledPasswd, String strOriginalData, String prescriptionCode) throws Exception {
        WlyyDoorPrescriptionDO prescription = doorPrescriptionDao.findByCode(prescriptionCode);
        //处方详情
        JSONObject res = new JSONObject();
        BaseDoctorDO doctor = doctorDao.findById(doctorCode);
        String url = jwUrl + "/third/prescription/RequestRealNameSoftCertAndSign";
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("strUserIdcardNum", doctor.getIdcard()));
        params.add(new BasicNameValuePair("strRealNameSoftCertCalledPasswd", strRealNameSoftCertCalledPasswd));
        if(StringUtils.isBlank(strOriginalData)){
            res = doorPrescriptionService.getPrescriptionByCode(prescription.getRecipeNo(), prescription.getPatient());
            strOriginalData = res.toString();
        }
        String originalDataAbstract = MD5.GetMD5Code(strOriginalData);
        params.add(new BasicNameValuePair("strOriginalData", originalDataAbstract));
        params.add(new BasicNameValuePair("prescriptionCode", prescriptionCode));
//        params.add(new BasicNameValuePair("srcBusinessStreamNO",srcBusinessStreamNO));
        String response = httpClientUtil.post(url, params, "UTF-8");
        if (!StringUtils.isEmpty(response)) {
            org.json.JSONObject responseObject = new org.json.JSONObject(response);
            int status = responseObject.getInt("status");
            String msg = responseObject.getString("msg");
            if (status == 200) {
                String data = responseObject.getString("data");
                if (StringUtils.isEmpty(data)) {
                    throw new Exception("null data");
                } else {
                    String[] str = data.split("\\|\\|");
//                    String strSignData = str[0];
//                    String strCertData = str[1];
                    org.json.JSONObject jsonObject = new org.json.JSONObject();
                    jsonObject.put("strSignData", str[0]);
                    jsonObject.put("strCertData", str[1]);
                    jsonObject.put("srcBusinessStreamNO", str[2]);
                    jsonObject.put("data", res);
                    prescription.setCaMessage(str[0]);
                    prescription.setCaCertData(str[1]);
                    prescription.setDigitalSignNo(str[2]);
                    prescription.setStrOriginalData(res.getJSONObject("prescriptionDrugList").toString());
                    prescription.setOriginalDataAbstract(originalDataAbstract);
                    doorPrescriptionDao.save(prescription);
                    //更新ca认证系统消息
//                    messageDao.updateMessageByRelationCode(prescriptionCode, 409);
                    this.uploadCaDigital(prescriptionCode);
                    return jsonObject;
                }
            } else {
                String error = msg.split("\\|\\|")[0].split(":")[2];
                throw new Exception(error);
            }
        } else {
            throw new Exception("null response.");
        }
    }

    //上传基位ca认证信息
    public JSONObject uploadCaDigital(String prescriptionCode) {
        JSONObject re = new JSONObject();
        WlyyDoorPrescriptionDO prescription = doorPrescriptionDao.findByCode(prescriptionCode);
        String url = jwUrl + "/third/doorPrescription/affirmCARecipe";
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("code", prescriptionCode));//续方code
        String response = httpClientUtil.post(url, params, "UTF-8");
        JSONObject json = JSONObject.parseObject(response);
        if (json.getInteger("status") == 200) {
            JSONObject data = JSONObject.parseObject(json.getString("data"));
            String zyCode = data.getString("CODE");
            if ("1".equals(zyCode)) {
                JSONArray returnData = data.getJSONArray("returnData");
                if (returnData.size() > 0) {
                    //{"status":200,"msg":"CA认证通过接口！","data":"{\"CODE\":\"1\",\"returnData\":[{\"result\":\"True\"}],\"byRefParaData\":[{ \"errorMsg\": \"\" }]}"}
                    JSONObject pre = returnData.getJSONObject(0);
                    if ("True".equals(pre.getString("result"))) {
                        //认证成功-审核完成
                        re.put("status", 200);
                    } else {
                        JSONArray byRefParaData = data.getJSONArray("byRefParaData");
                        re.put("status", -1);
                        re.put("msg", byRefParaData.getJSONObject(0).getString("errorMsg"));
                    }
                } else {
                    JSONArray byRefParaData = data.getJSONArray("byRefParaData");
                    re.put("status", -1);
                    re.put("msg", byRefParaData.getJSONObject(0).getString("errorMsg"));
                }
            } else {
                re.put("status", -1);
                re.put("msg", data.getString("MESSAGE"));
            }
        } else {
            re.put("status", -1);
            re.put("msg", "上传ca认证信息失败");
        }
        return re;
    }
}
