package com.yihu.jw.entrance.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.entity.hospital.prescription.*;
import com.yihu.jw.hospital.prescription.dao.PrescriptionDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionDiagnosisDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionInfoDao;
import com.yihu.jw.hospital.prescription.service.entrance.XzzxEntranceService;
import com.yihu.jw.im.service.ImService;
import com.yihu.jw.restmodel.hospital.prescription.WlyyPrescriptionDiagnosisVO;
import com.yihu.jw.restmodel.hospital.prescription.WlyyPrescriptionInfoVO;
import com.yihu.jw.restmodel.hospital.prescription.WlyyPrescriptionVO;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Created by Trick on 2020/3/18.
 */
@RestController
@RequestMapping(value ="/xzzx")
@Api(value = "心脏中心平台服务", description = "心脏中心平台服务", tags = {"心脏中心平台服务"})
public class XzzxCotroller extends EnvelopRestEndpoint {

    @Autowired
    private XzzxEntranceService xzzxEntranceService;
    @Autowired
    private PrescriptionDao prescriptionDao;
    @Autowired
    private PrescriptionDiagnosisDao prescriptionDiagnosisDao;
    @Autowired
    private PrescriptionInfoDao prescriptionInfoDao;
    @Autowired
    private ImService imService;

    @GetMapping(value = "/getDeptInfo")
    @ApiOperation(value = "获取部门信息", notes = "获取部门信息")
    public ObjEnvelop getDeptInfo(@ApiParam(name = "code", value = "code", required = false)
                                      @RequestParam(value = "code",required = false)String code)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.getDeptInfo(code));
    }


    @GetMapping(value = "/getDoctorInfo")
    @ApiOperation(value = "同步医生信息", notes = "同步医生信息")
    public ObjEnvelop getDoctorInfo(@ApiParam(name = "idCard", value = "身份证号", required = false)
                                        @RequestParam(value = "idCard",required = false)String idcard)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.getDoctorInfo(idcard));
    }

    @GetMapping(value = "/selectDoctorInfo")
    @ApiOperation(value = "获取医生信息", notes = "获取医生信息")
    public ObjEnvelop selectDoctorInfo(@ApiParam(name = "idCard", value = "身份证号", required = false)
                                    @RequestParam(value = "idCard",required = false)String idcard)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.selectDoctorInfo(idcard));
    }

    @GetMapping(value = "/getJobTitle")
    @ApiOperation(value = "同步职称", notes = "同步职称")
    public ObjEnvelop getJobTitle()throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.getJobTitle());
    }

    @GetMapping(value = "/getChargeDict")
    @ApiOperation(value = "同步号别", notes = "同步号别")
    public ObjEnvelop getChargeDict()throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.getChargeDict());
    }


    /**
     * 查询患者就诊卡
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/findPatientCard")
    @ApiOperation(value = "查询患者就诊卡", notes = "查询患者就诊卡")
    public ObjEnvelop findPatientCard(@ApiParam(name = "idCard", value = "身份证号", required = false)
                                          @RequestParam(value = "idCard",required = false)String idCard)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.findPatientCard(idCard));
    }


    /**
     * 获取卡信息
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/getCardInfo")
    @ApiOperation(value = "获取卡信息", notes = "获取卡信息")
    public ObjEnvelop getCardInfo(@ApiParam(name = "cardNo", value = "就诊卡", required = false)
                                      @RequestParam(value = "cardNo",required = false)String cardNo)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.getCardInfo(cardNo));
    }

    /**
     * 获取卡信息
     * @return
     * @throws Exception
     */
    @PostMapping(value = "/SendSms")
    @ApiOperation(value = "发送短信验证码", notes = "发送短信验证码")
    public ObjEnvelop SendSms(@ApiParam(name = "json", value = "验证码内容", required = false)
                                  @RequestParam(value = "json",required = false)String json)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.SendSms(json));
    }


    /**
     * 获取单条门诊就诊记录
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/findOutpatientInfo")
    @ApiOperation(value = "获取单条门诊就诊记录", notes = "获取单条门诊就诊记录")
    public ObjEnvelop findOutpatientInfo(@ApiParam(name = "patient", value = "居民id", required = false)
                                  @RequestParam(value = "patient",required = false)String patient,
                                         @ApiParam(name = "conNo", value = "就诊次数", required = false)
                                         @RequestParam(value = "conNo",required = false)String conNo,
                                         @ApiParam(name = "startTime", value = "开始时间", required = false)
                                             @RequestParam(value = "startTime",required = false)String startTime,
                                         @ApiParam(name = "endTime", value = "结束时间", required = false)
                                             @RequestParam(value = "endTime",required = false)String endTime)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.findOutpatientInfo(patient,conNo,startTime,endTime));
    }

    /**
     * 获取门诊下所有处方信息
     * @param registerSn 流水号(必传)
     * @param patNo 病人id(必传)
     * @param admNo 住院唯一号(必传)
     * @param realOrder 处方号(非必传，传相当于查单条处方)
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/getOriginPrescriptionList")
    @ApiOperation(value = "获取门诊下所有处方信息", notes = "获取门诊下所有处方信息")
    public ObjEnvelop getOriginPrescriptionList(@ApiParam(name = "registerSn", value = "流水号", required = false)
                                         @RequestParam(value = "registerSn",required = false)String registerSn,
                                         @ApiParam(name = "patNo", value = "病人id", required = false)
                                         @RequestParam(value = "patNo",required = false)String patNo,
                                         @ApiParam(name = "admNo", value = "住院唯一号", required = false)
                                          @RequestParam(value = "admNo",required = false)String admNo,
                                         @ApiParam(name = "realOrder", value = "处方号", required = false)
                                          @RequestParam(value = "realOrder",required = false)String realOrder,
                                         @ApiParam(name = "startTime", value = "开始时间", required = false)
                                         @RequestParam(value = "startTime",required = false)String startTime,
                                         @ApiParam(name = "endTime", value = "结束时间", required = false)
                                         @RequestParam(value = "endTime",required = false)String endTime)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.getOriginPrescriptionList(registerSn,patNo,admNo,realOrder,startTime,endTime));
    }


    /**
     * 获取门诊病人诊断
     * @param registerSn 流水号(必传)
     * @param patNo 病人id(必传)
     * @param admNo 住院唯一号(必传)
     * @param realOrder 处方号(非必传，传相当于查单条处方)
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/getOutpatientDiagnosis")
    @ApiOperation(value = "获取门诊病人诊断", notes = "获取门诊病人诊断")
    public ObjEnvelop getOutpatientDiagnosis(@ApiParam(name = "registerSn", value = "流水号", required = false)
                                             @RequestParam(value = "registerSn",required = false)String registerSn,
                                             @ApiParam(name = "patNo", value = "病人id", required = false)
                                             @RequestParam(value = "patNo",required = false)String patNo,
                                             @ApiParam(name = "admNo", value = "住院唯一号", required = false)
                                             @RequestParam(value = "admNo",required = false)String admNo,
                                             @ApiParam(name = "realOrder", value = "处方号", required = false)
                                             @RequestParam(value = "realOrder",required = false)String realOrder,
                                             @ApiParam(name = "startTime", value = "开始时间", required = false)
                                             @RequestParam(value = "startTime",required = false)String startTime,
                                             @ApiParam(name = "endTime", value = "结束时间", required = false)
                                             @RequestParam(value = "endTime",required = false)String endTime)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.getOutpatientDiagnosis(registerSn,patNo,admNo,realOrder,startTime,endTime));
    }



    /**
     * 获取门诊病人处方药品信息
     * @param registerSn 流水号(必传)
     * @param patNo 病人id(必传)
     * @param admNo 住院唯一号(必传)
     * @param realOrder 处方号(非必传，传相当于查单条处方)
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/getOutpatientDrugInfo")
    @ApiOperation(value = "获取门诊病人处方药品信息", notes = "获取门诊病人处方药品信息")
    public ObjEnvelop getOutpatientDrugInfo(@ApiParam(name = "registerSn", value = "流水号", required = false)
                                             @RequestParam(value = "registerSn",required = false)String registerSn,
                                             @ApiParam(name = "patNo", value = "病人id", required = false)
                                             @RequestParam(value = "patNo",required = false)String patNo,
                                             @ApiParam(name = "admNo", value = "住院唯一号", required = false)
                                             @RequestParam(value = "admNo",required = false)String admNo,
                                             @ApiParam(name = "realOrder", value = "处方号", required = false)
                                             @RequestParam(value = "realOrder",required = false)String realOrder,
                                             @ApiParam(name = "startTime", value = "开始时间", required = false)
                                             @RequestParam(value = "startTime",required = false)String startTime,
                                             @ApiParam(name = "endTime", value = "结束时间", required = false)
                                             @RequestParam(value = "endTime",required = false)String endTime)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.getOutpatientDrugInfo(registerSn,patNo,admNo,realOrder,startTime,endTime));
    }

    /**
     * 2.1.3	获取患者结算信息
     * @param icCardNo 社保卡号(必传)
     * @param his_serial his挂号号(必传)
     * @param org_code 医院编码(必传)
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/presUpload")
    @ApiOperation(value = "获取处方信息", notes = "获取处方信息")
    public ObjEnvelop presUpload(@ApiParam(name = "icCardNo", value = "社保卡号", required = true)
                                            @RequestParam(value = "icCardNo",required = true)String icCardNo,
                                            @ApiParam(name = "his_serial", value = "his挂号号", required = true)
                                            @RequestParam(value = "his_serial",required = true)String his_serial,
                                            @ApiParam(name = "org_code", value = "医院编码", required = true)
                                            @RequestParam(value = "org_code",required = true)String org_code)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.presUpload(icCardNo,his_serial,org_code));
    }

    /**
     * 获取门诊病人检查检验信息
     * @param registerSn 流水号(必传)
     * @param patNo 病人id(必传)
     * @param admNo 住院唯一号(必传)
     * @param realOrder 处方号(非必传，传相当于查单条处方)
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/getOutpatientCheckTestInfo")
    @ApiOperation(value = "获取门诊病人检查检验信息", notes = "获取门诊病人检查检验信息")
    public ObjEnvelop getOutpatientCheckTestInfo(@ApiParam(name = "registerSn", value = "流水号", required = false)
                                            @RequestParam(value = "registerSn",required = false)String registerSn,
                                            @ApiParam(name = "patNo", value = "病人id", required = false)
                                            @RequestParam(value = "patNo",required = false)String patNo,
                                            @ApiParam(name = "admNo", value = "住院唯一号", required = false)
                                            @RequestParam(value = "admNo",required = false)String admNo,
                                            @ApiParam(name = "realOrder", value = "处方号", required = false)
                                            @RequestParam(value = "realOrder",required = false)String realOrder,
                                            @ApiParam(name = "startTime", value = "开始时间", required = false)
                                            @RequestParam(value = "startTime",required = false)String startTime,
                                            @ApiParam(name = "endTime", value = "结束时间", required = false)
                                            @RequestParam(value = "endTime",required = false)String endTime)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.getOutpatientCheckTestInfo(registerSn,patNo,admNo,realOrder,startTime,endTime));
    }



    /**
     * 挂号（调his后台包）
     * @param doctor 医生ID(必填)
     * @param dept 科室编码(必填)
     * @param sickId 病人id
     * @param clinicclass 挂号类别(必填)
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/registered")
    @ApiOperation(value = "挂号（调his后台包）", notes = "挂号（调his后台包）")
    public ObjEnvelop registered(@ApiParam(name = "doctor", value = "医生ID", required = false)
                                         @RequestParam(value = "doctor",required = false)String doctor,
                                         @ApiParam(name = "dept", value = "科室编码", required = false)
                                         @RequestParam(value = "dept",required = false)String dept,
                                 @ApiParam(name = "sickId", value = "病人id", required = false)
                                     @RequestParam(value = "sickId",required = false)String sickId,
                                 @ApiParam(name = "clinicclass", value = "挂号类别", required = false)
                                     @RequestParam(value = "clinicclass",required = false)String clinicclass,
                                 @ApiParam(name = "courierflag", value = "快递发药标识（Y快递N为自取）", required = false)
                                     @RequestParam(value = "courierflag",required = false)String courierflag)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.registered(doctor,dept,sickId,clinicclass,courierflag));
    }

    /**
     * 处方状态变更
     * @param admNo 就诊唯一号(必填)
     * @param realOrder 处方号(必填)
     * @param status 处方状态：3结算完成(必填)
     * @return
             * @throws Exception
     */
    @PostMapping(value = "/updatePreStatus")
    @ApiOperation(value = "处方状态变更", notes = "处方状态变更")
    public ObjEnvelop updatePreStatus(@ApiParam(name = "admNo", value = "就诊唯一号", required = false)
                                      @RequestParam(value = "admNo",required = false)String admNo,
                                      @ApiParam(name = "realOrder", value = "处方号", required = false)
                                      @RequestParam(value = "realOrder",required = false)String realOrder,
                                      @ApiParam(name = "status", value = "处方状态：3结算完成", required = false)
                                      @RequestParam(value = "status",required = false)String status)throws Exception{
        Map<String,Object> map = xzzxEntranceService.updatePreStatusLocal(admNo,realOrder,status);
        if (status.equalsIgnoreCase("1")){
            List<WlyyPrescriptionDO> prescriptionDOList =  prescriptionDao.findByAdmNoAndRealOrderList(admNo,realOrder);
                    if (prescriptionDOList!=null&&prescriptionDOList.size()!=0){
                WlyyPrescriptionDO wlyyPrescriptionDO = prescriptionDOList.get(0);
                JSONObject object = (JSONObject) JSONObject.toJSON(wlyyPrescriptionDO);
                System.out.println("wlyyPrescriptionDO参数入参"+object.toJSONString());
                WlyyPrescriptionVO prescriptionVO = JSONObject.toJavaObject(object,WlyyPrescriptionVO.class);
                List<WlyyPrescriptionInfoDO> wlyyPrescriptionInfoDOS = prescriptionInfoDao.findByPrescriptionId(wlyyPrescriptionDO.getId(),1);
                List<WlyyPrescriptionInfoVO> wlyyPrescriptionInfoVOS = new ArrayList<>();
                for (WlyyPrescriptionInfoDO wlyyPrescriptionInfoDO:wlyyPrescriptionInfoDOS){
                    JSONObject jsonObject = (JSONObject) JSONObject.toJSON(wlyyPrescriptionInfoDO);
                    System.out.println("wlyyPrescriptionInfoDO参数入参"+jsonObject.toJSONString());
                    WlyyPrescriptionInfoVO prescriptionInfoVO = JSONObject.toJavaObject(jsonObject,WlyyPrescriptionInfoVO.class);
                    wlyyPrescriptionInfoVOS.add(prescriptionInfoVO);
                }
                prescriptionVO.setInfoVOs(wlyyPrescriptionInfoVOS);
                List<WlyyPrescriptionDiagnosisDO> wlyyPrescriptionDiagnosisDOS = prescriptionDiagnosisDao.findByPrescriptionId(wlyyPrescriptionDO.getId(),1);
                List<WlyyPrescriptionDiagnosisVO> wlyyPrescriptionDiagnosisVOS = new ArrayList<>();
                for (WlyyPrescriptionDiagnosisDO wlyyPrescriptionDiagnosisDO:wlyyPrescriptionDiagnosisDOS){
                    JSONObject jsonObject = (JSONObject) JSONObject.toJSON(wlyyPrescriptionDiagnosisDO);
                    System.out.println("wlyyPrescriptionDiagnosisDO参数入参"+jsonObject.toJSONString());
                    WlyyPrescriptionDiagnosisVO prescriptionDiagnosisVO = JSONObject.toJavaObject(jsonObject,WlyyPrescriptionDiagnosisVO.class);
                    wlyyPrescriptionDiagnosisVOS.add(prescriptionDiagnosisVO);
                }
                prescriptionVO.setDiagnosisVOs(wlyyPrescriptionDiagnosisVOS);
                imService.pushPrescriptionImMessage(prescriptionVO);
            }
        }
        return ObjEnvelop.getSuccess("ok",map);
    }


    /**
     * 获取icd10字典
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/getICD10Dict")
    @ApiOperation(value = "获取icd10字典", notes = "获取icd10字典")
    public ObjEnvelop getICD10Dict(@ApiParam(name = "spellCode", value = "拼音码", required = false)
                                  @RequestParam(value = "spellCode",required = false)String spellCode)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.GetICD10Dict(spellCode));
    }

    /**
     * 获取药品字典
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/getDrugDict")
    @ApiOperation(value = "获取药品字典", notes = "获取药品字典")
    public ObjEnvelop getDrugDict(@ApiParam(name = "spellCode", value = "拼音码", required = false)
                                   @RequestParam(value = "spellCode",required = false)String spellCode,
                                  @ApiParam(name = "drugNo", value = "药品编码", required = false)
                                  @RequestParam(value = "drugNo",required = false)String drugNo)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.GetDrugDict(spellCode,drugNo));
    }


    /**
     * 查询药品用药方法
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/getMedicineUsed")
    @ApiOperation(value = "查询药品用药方法", notes = "查询药品用药方法")
    public ObjEnvelop getMedicineUsed(@ApiParam(name = "spellCode", value = "拼音码", required = false)
                                  @RequestParam(value = "spellCode",required = false)String spellCode)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.GetMedicineUsed(spellCode));
    }

    /**
     * 获取药品用药频次字典
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/getFrequencyDict")
    @ApiOperation(value = "获取药品用药频次字典", notes = "获取药品用药频次字典")
    public ObjEnvelop getFrequencyDict()throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.GetFrequencyDict());
    }

    /**
     * 获取检查部位字典
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/getCheckBody")
    @ApiOperation(value = "获取检查部位字典", notes = "获取检查部位字典")
    public ObjEnvelop getCheckBody(@ApiParam(name = "spellCode", value = "拼音码", required = false)
                                       @RequestParam(value = "spellCode",required = false)String spellCode)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.GetCheckBody(spellCode));
    }


    /**
     * 获取检查单据信息
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/getExamApply")
    @ApiOperation(value = "获取检查单据信息", notes = "获取检查单据信息")
    public ObjEnvelop getExamApply(@ApiParam(name = "spellCode", value = "拼音码", required = false)
                                       @RequestParam(value = "spellCode",required = false)String spellCode)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.GetExamApply(spellCode));
    }

    /**
     * 获取检验单据信息
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/getLisApply")
    @ApiOperation(value = "获取检验单据信息", notes = "获取检验单据信息")
    public ObjEnvelop getLisApply(@ApiParam(name = "spellCode", value = "拼音码", required = false)
                                   @RequestParam(value = "spellCode",required = false)String spellCode)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.GetLisApply(spellCode));
    }

    /**
     * 互联网医院开处方信息更新到his
     *
     * @param IoFlag  门诊住院标志
     * @param PayCardNo 就诊卡号
     * @param NullahNumber 挂号流水号
     * @param ApplyDept 执行科室
     * @param ApplyDoctor 开单医生
     * @param ExecuteDept 执行科室(药房)
     * @param prescriptionInfoDOS
     */
    @PostMapping(value = "/savePrescriptionToHospital")
    @ApiOperation(value = "互联网医院开处方信息更新到his", notes = "互联网医院开处方信息更新到his")
    public ObjEnvelop savePrescriptionToHospital(@ApiParam(name = "IoFlag", value = "门诊住院标志", required = false)
                                                     @RequestParam(value = "IoFlag",required = false)String IoFlag,
                                                 @ApiParam(name = "PayCardNo", value = "就诊卡号", required = false)
                                                    @RequestParam(value = "PayCardNo",required = false)String PayCardNo,
                                                 @ApiParam(name = "NullahNumber", value = "挂号流水号", required = false)
                                                     @RequestParam(value = "NullahNumber",required = false)String NullahNumber,
                                                 @ApiParam(name = "ApplyDept", value = "执行科室", required = false)
                                                     @RequestParam(value = "ApplyDept",required = false)String ApplyDept,
                                                 @ApiParam(name = "ApplyDoctor", value = "开单医生", required = false)
                                                     @RequestParam(value = "ApplyDoctor",required = false)String ApplyDoctor,
                                                 @ApiParam(name = "ExecuteDept", value = "执行科室(药房)", required = false)
                                                     @RequestParam(value = "ExecuteDept",required = false)String ExecuteDept,
                                                 @ApiParam(name = "prescriptionInfoDOS", value = "药品信息", required = false)
                                                     @RequestParam(value = "prescriptionInfoDOS",required = false)String prescriptionInfoDOS)throws Exception{
        List<WlyyPrescriptionInfoDO> prescriptionInfoDOList = new ArrayList<>();
        if (prescriptionInfoDOS!=null){
            JSONArray jsonArray = JSONArray.parseArray(prescriptionInfoDOS);
            for (int i=0;i<jsonArray.size();i++){
                WlyyPrescriptionInfoDO wlyyPrescriptionInfoDO = toEntity(jsonArray.getJSONObject(i).toJSONString(),WlyyPrescriptionInfoDO.class);
                prescriptionInfoDOList.add(wlyyPrescriptionInfoDO);
            }
        }
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.savePrescriptionToHospital(IoFlag,PayCardNo,NullahNumber,ApplyDept,ApplyDoctor,ExecuteDept,prescriptionInfoDOList));
    }


    /**
     * 删除处方信息
     *
     * @param IoFlag  门诊住院标志
     * @param PayCardNo 就诊卡号
     * @param NullahNumber 挂号流水号
     * @param ApplyDept 执行科室
     * @param ApplyDoctor 开单医生
     * @param ExecuteDept 执行科室(药房)
     */
    @PostMapping(value = "/delHisPrescriptionByHisNo")
    @ApiOperation(value = "删除处方信息", notes = "删除处方信息")
    public ObjEnvelop delHisPrescriptionByHisNo(@ApiParam(name = "IoFlag", value = "门诊住院标志", required = false)
                                                 @RequestParam(value = "IoFlag",required = false)String IoFlag,
                                                 @ApiParam(name = "PayCardNo", value = "就诊卡号", required = false)
                                                 @RequestParam(value = "PayCardNo",required = false)String PayCardNo,
                                                 @ApiParam(name = "NullahNumber", value = "挂号流水号", required = false)
                                                 @RequestParam(value = "NullahNumber",required = false)String NullahNumber,
                                                 @ApiParam(name = "ApplyDept", value = "执行科室", required = false)
                                                 @RequestParam(value = "ApplyDept",required = false)String ApplyDept,
                                                 @ApiParam(name = "ApplyDoctor", value = "开单医生", required = false)
                                                 @RequestParam(value = "ApplyDoctor",required = false)String ApplyDoctor,
                                                 @ApiParam(name = "ExecuteDept", value = "执行科室(药房)", required = false)
                                                 @RequestParam(value = "ExecuteDept",required = false)String ExecuteDept,
                                                 @ApiParam(name = "PresNo", value = "必填，医嘱号", required = false)
                                                 @RequestParam(value = "PresNo",required = false)String PresNo)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.delHisPrescriptionByHisNo(IoFlag,PayCardNo,NullahNumber,ApplyDept,ApplyDoctor,ExecuteDept,PresNo));
    }

    /**
     * 保存入门诊断信息到his
     * @param ApplyNo
     * @param DiagnosisName
     * @param DiagnosisCode
     * @param ChnSymPtomsCode
     * @param ChnSymPtomsName
     * @param ChnDiagnosisCode
     * @param ChnDiagnosisName
     * @return
     * @throws Exception
     */
    @PostMapping(value = "/saveDiagnosis")
    @ApiOperation(value = "保存入门诊断信息到his", notes = "保存入门诊断信息到his")
    public ObjEnvelop saveDiagnosis(@ApiParam(name = "ApplyNo", value = "必填，流水号", required = false)
                                                 @RequestParam(value = "ApplyNo",required = false)String ApplyNo,
                                                 @ApiParam(name = "DiagnosisName", value = "必填，诊断名称", required = false)
                                                 @RequestParam(value = "DiagnosisName",required = false)String DiagnosisName,
                                                 @ApiParam(name = "DiagnosisCode", value = "诊断代码", required = false)
                                                 @RequestParam(value = "DiagnosisCode",required = false)String DiagnosisCode,
                                                 @ApiParam(name = "ChnSymPtomsCode", value = "中医证型代码", required = false)
                                                 @RequestParam(value = "ChnSymPtomsCode",required = false)String ChnSymPtomsCode,
                                                 @ApiParam(name = "ChnSymPtomsName", value = "中医证型", required = false)
                                                 @RequestParam(value = "ChnSymPtomsName",required = false)String ChnSymPtomsName,
                                                 @ApiParam(name = "ChnDiagnosisCode", value = "中医诊断代码", required = false)
                                                 @RequestParam(value = "ChnDiagnosisCode",required = false)String ChnDiagnosisCode,
                                                 @ApiParam(name = "ChnDiagnosisName", value = "中医诊断名称", required = false)
                                                 @RequestParam(value = "ChnDiagnosisName",required = false)String ChnDiagnosisName)throws Exception{

        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.saveDiagnosis(ApplyNo,DiagnosisName,DiagnosisCode,ChnSymPtomsCode,ChnSymPtomsName,ChnDiagnosisCode,ChnDiagnosisName));
    }


    /**
     * 发送退号信息到his
     * @param ApplyNo
     * @return
     * @throws Exception
     */
    @PostMapping(value = "/refundConsultation")
    @ApiOperation(value = "发送退号信息到his", notes = "发送退号信息到his")
    public ObjEnvelop refundConsultation(@ApiParam(name = "ApplyNo", value = "必填，流水号", required = false)
                                    @RequestParam(value = "ApplyNo",required = false)String ApplyNo,
                                    @ApiParam(name = "PayCardNo", value = "必填，就诊卡号", required = false)
                                    @RequestParam(value = "PayCardNo",required = false)String PayCardNo)throws Exception{

        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.refundConsultation(ApplyNo,PayCardNo));
    }
    /**
     * 获取检验单据信息
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/saveInspectToHospital")
    @ApiOperation(value = "互联网医院开检查检验信息更新到his", notes = "互联网医院开检查检验信息更新到his")
    public ObjEnvelop saveInspectToHospital(@ApiParam(name = "PayCardNo", value = "就诊卡号", required = false)
                                  @RequestParam(value = "PayCardNo",required = false)String PayCardNo,
                                            @ApiParam(name = "IoFlag", value = "门诊住院标志", required = false)
                                            @RequestParam(value = "IoFlag",required = false)String IoFlag,
                                            @ApiParam(name = "NullahNumber", value = "挂号流水号", required = false)
                                                @RequestParam(value = "NullahNumber",required = false)String NullahNumber,
                                            @ApiParam(name = "ApplyDept", value = "执行科室", required = false)
                                                @RequestParam(value = "ApplyDept",required = false)String ApplyDept,
                                            @ApiParam(name = "ApplyDoctor", value = "开单医生", required = false)
                                                @RequestParam(value = "ApplyDoctor",required = false)String ApplyDoctor,
                                            @ApiParam(name = "wlyyInspectionDOS", value = "检查json", required = false)
                                                @RequestParam(value = "wlyyInspectionDOS",required = false) String wlyyInspectionDOS)throws Exception{
        JSONArray jsonArray = JSONArray.parseArray(wlyyInspectionDOS);
        List<WlyyInspectionDO> wlyyInspectionS = new ArrayList<>();
        for (int i=0;i<jsonArray.size();i++){
            WlyyInspectionDO inspectionDO = toEntity(jsonArray.get(i).toString(),WlyyInspectionDO.class);
            wlyyInspectionS.add(inspectionDO);

        }
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.saveInspectToHospital(IoFlag,PayCardNo,NullahNumber,ApplyDept,ApplyDoctor,wlyyInspectionS));
    }
    /**
     * 合理用药审核接口
     * @return
     * @throws Exception
     */
    @PostMapping(value = "/checkPrescription")
    @ApiOperation(value = "合理用药审核接口", notes = "合理用药审核接口")
    public ObjEnvelop checkPrescription(@ApiParam(name = "infos", value = "药品信息", required = false)
                                            @RequestParam(value = "infos",required = false)String infos,
                                            @ApiParam(name = "patientAndDoctor", value = "医患信息", required = false)
                                            @RequestParam(value = "patientAndDoctor",required = false) String patientAndDoctor)throws Exception{
        JSONArray jsonArray = JSONArray.parseArray(infos);
        List<WlyyPrescriptionInfoDO> wlyyPrescriptionInfoDOS = new ArrayList<>();
        for (int i=0;i<jsonArray.size();i++){
            WlyyPrescriptionInfoDO wlyyPrescriptionInfo = toEntity(jsonArray.get(i).toString(),WlyyPrescriptionInfoDO.class);
            wlyyPrescriptionInfoDOS.add(wlyyPrescriptionInfo);
        }
        JSONObject jsonObject = JSONObject.parseObject(patientAndDoctor);
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.checkPrescription(jsonObject,wlyyPrescriptionInfoDOS));
    }
    /**
     * 合理用药保存接口
     * @return
     * @throws Exception
     */
    @PostMapping(value = "/saveCheckPrescription")
    @ApiOperation(value = "合理用药保存接口", notes = "合理用药保存接口")
    public ObjEnvelop saveCheckPrescription(@ApiParam(name = "infos", value = "药品信息", required = false)
                                        @RequestParam(value = "infos",required = false)String infos,
                                        @ApiParam(name = "patientAndDoctor", value = "医患信息", required = false)
                                        @RequestParam(value = "patientAndDoctor",required = false) String patientAndDoctor)throws Exception{
        JSONArray jsonArray = JSONArray.parseArray(infos);
        List<WlyyPrescriptionInfoDO> wlyyPrescriptionInfoDOS = new ArrayList<>();
        for (int i=0;i<jsonArray.size();i++){
            WlyyPrescriptionInfoDO wlyyPrescriptionInfo = toEntity(jsonArray.get(i).toString(),WlyyPrescriptionInfoDO.class);
            wlyyPrescriptionInfoDOS.add(wlyyPrescriptionInfo);
        }
        JSONObject jsonObject = JSONObject.parseObject(patientAndDoctor);
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.saveCheckPrescription(jsonObject,wlyyPrescriptionInfoDOS));
    }
    /**
     * 合理用药作废接口
     * @return
     * @throws Exception
     */
    @PostMapping(value = "/deleteCheckPrescription")
    @ApiOperation(value = "合理用药作废接口", notes = "合理用药作废接口")
    public ObjEnvelop deleteCheckPrescription(@ApiParam(name = "infos", value = "药品信息", required = false)
                                              @RequestParam(value = "infos",required = false)String infos,
                                              @ApiParam(name = "patientAndDoctor", value = "医患信息", required = false)
                                              @RequestParam(value = "patientAndDoctor",required = false) String patientAndDoctor)throws Exception{
        List<WlyyPrescriptionInfoDO> wlyyPrescriptionInfoDOS = new ArrayList<>();
        if (StringUtils.isNotBlank(infos)){
            JSONArray jsonArray = JSONArray.parseArray(infos);
            for (int i=0;i<jsonArray.size();i++){
                WlyyPrescriptionInfoDO wlyyPrescriptionInfo = toEntity(jsonArray.get(i).toString(),WlyyPrescriptionInfoDO.class);
                wlyyPrescriptionInfoDOS.add(wlyyPrescriptionInfo);
            }
        }
        JSONObject jsonObject = JSONObject.parseObject(patientAndDoctor);
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.deleteCheckPrescription(jsonObject,wlyyPrescriptionInfoDOS));
    }
    /**
     * 合理用药作废接口
     * @return
     * @throws Exception
     */
    @PostMapping(value = "/returnEmrHtmlByHLW")
    @ApiOperation(value = "保存电子病历接口", notes = "保存电子病历接口")
    public ObjEnvelop returnEmrHtmlByHLW(@ApiParam(name = "jsonInfo", value = "医生患者信息", required = false)
                                              @RequestParam(value = "jsonInfo",required = false)String jsonInfo,
                                              @ApiParam(name = "emrInfo", value = "病历信息", required = false)
                                              @RequestParam(value = "emrInfo",required = false) String emrInfo)throws Exception{

        JSONObject jsonObject = JSONObject.parseObject(jsonInfo);
        WlyyPrescriptionEmrDO wlyyPrescriptionEmrDO = toEntity(emrInfo,WlyyPrescriptionEmrDO.class);
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.returnEmrHtmlByHLW(jsonObject,wlyyPrescriptionEmrDO));
    }
    /**
     * 查询历史就诊病历
     * @param patientId 病人id(必填)
     * @param eventNo 住院号
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/selectEmrInfo")
    @ApiOperation(value = "查询历史就诊病历", notes = "查询历史就诊病历")
    public ObjEnvelop findHistoryEmr (@ApiParam(name = "patientId", value = "病人id", required = false)
                                      @RequestParam(value = "patientId",required = false)String patientId,
                                      @ApiParam(name = "eventNo", value = "住院号", required = false)
                                      @RequestParam(value = "eventNo",required = false)String eventNo)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.SelectEmrInfo(patientId,eventNo));
    }
    /**
     * 查询历次就诊门诊病历文书
     * @param patientId 病人id(必填)
     * @param eventNo 住院号
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/selectEmrDispRecord")
    @ApiOperation(value = "查询历次就诊门诊病历文书", notes = "查询历次就诊门诊病历文书")
    public ObjEnvelop SelectEmrDispRecord (@ApiParam(name = "templateId", value = "模板id", required = false)
                                               @RequestParam(value = "templateId",required = false)String templateId,
                                        @ApiParam(name = "patientId", value = "病人id", required = false)
                                      @RequestParam(value = "patientId",required = false)String patientId,
                                      @ApiParam(name = "eventNo", value = "住院号", required = false)
                                      @RequestParam(value = "eventNo",required = false)String eventNo)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.SelectEmrDispRecord(templateId,patientId,eventNo));
    }


    /**
     * 查询病历文书列表
     * @param patientId 病人id(必填)
     * @param eventNo 住院号
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/selectEmrList")
    @ApiOperation(value = "查询病历文书列表", notes = "查询病历文书列表")
    public ObjEnvelop SelectEmrList(@ApiParam(name = "patientId", value = "病人id", required = false)
                                           @RequestParam(value = "patientId",required = false)String patientId,
                                           @ApiParam(name = "eventNo", value = "住院号", required = false)
                                           @RequestParam(value = "eventNo",required = false)String eventNo)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.GetEMRList(patientId,eventNo));
    }

    /**
     * 查询病历文书html信息
     * @param bstrEmrID 文书id号
     *
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/browseSingleEMR")
    @ApiOperation(value = "查询病历文书html信息", notes = "查询病历文书html信息")
    public ObjEnvelop BrowseSingleEMR(@ApiParam(name = "bstrEmrID", value = "文书id号", required = false)
                                    @RequestParam(value = "bstrEmrID",required = false)String bstrEmrID)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.BrowseSingleEMR(bstrEmrID));
    }


    /**
     * 1获取患者结算信息
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/getSettleInfo")
    @ApiOperation(value = "2.1.1获取患者结算信息", notes = "2.1.1获取患者结算信息")
    public ObjEnvelop getSettleInfo(@ApiParam(name = "icCardNo", value = "社会保障卡号", required = false)
                                         @RequestParam(value = "icCardNo",required = false)String icCardNo,
                                    @ApiParam(name = "his_serial", value = "HIS挂号号", required = false)
                                        @RequestParam(value = "his_serial",required = false) String his_serial,
                                    @ApiParam(name = "org_code", value = "医院编码", required = false)
                                        @RequestParam(value = "org_code",required = false) String org_code)throws Exception{

        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.getSettleInfo(icCardNo,his_serial,org_code));
    }

    /**
     * 2结算确认
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/confirmSettle")
    @ApiOperation(value = "2.1.2结算确认", notes = "2.1.2结算确认")
    public ObjEnvelop confirmSettle(@ApiParam(name = "icCardNo", value = "社会保障卡号", required = false)
                                    @RequestParam(value = "icCardNo",required = false)String icCardNo,
                                    @ApiParam(name = "his_serial", value = "HIS挂号号", required = false)
                                    @RequestParam(value = "his_serial",required = false) String his_serial,
                                    @ApiParam(name = "org_code", value = "医院编码", required = false)
                                    @RequestParam(value = "org_code",required = false) String org_code,
                                    @ApiParam(name = "outpatientId", value = "patientId", required = false)
                                        @RequestParam(value = "outpatientId",required = false) String outpatientId)throws Exception{

        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.confirmSettle(his_serial,icCardNo,org_code,outpatientId));
    }

    /**
     * 2.1.4电子票开票结果确认
     *
     * @param sick_id 病人ID
     * @param settle_no HIS结算号
     * @param einvoice_code 电子票据代码
     * @param einvoice_number 电子票据号码
     * @param einvoice_check_coder 电子票据校验码
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/eleInvoiceConfirm")
    @ApiOperation(value = "2.1.4电子票开票结果确认", notes = "2.1.4电子票开票结果确认")
    public ObjEnvelop eleInvoiceConfirm(@ApiParam(name = "sick_id", value = "病人ID", required = false)
                                    @RequestParam(value = "sick_id",required = false)String sick_id,
                                    @ApiParam(name = "settle_no", value = "HIS结算号", required = false)
                                    @RequestParam(value = "settle_no",required = false) String settle_no,
                                    @ApiParam(name = "einvoice_code", value = "电子票据代码", required = false)
                                    @RequestParam(value = "einvoice_code",required = false) String einvoice_code,
                                    @ApiParam(name = "einvoice_number", value = "电子票据号码", required = false)
                                        @RequestParam(value = "einvoice_number",required = false) String einvoice_number,
                                    @ApiParam(name = "einvoice_check_coder", value = "电子票据校验码", required = false)
                                        @RequestParam(value = "einvoice_check_coder",required = false) String einvoice_check_coder)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.eleInvoiceConfirm(sick_id,settle_no,einvoice_code,einvoice_number,einvoice_check_coder));
    }

}
