package com.yihu.jw.entrance.controller.third;

import com.yihu.jw.entity.hospital.httplog.WlyyHttpLogDO;
import com.yihu.jw.entity.hospital.message.SystemMessageDO;
import com.yihu.jw.entrance.service.HospitalSystemMessageService;
import com.yihu.jw.hospital.httplog.dao.WlyyHttpLogDao;
import com.yihu.jw.hospital.prescription.service.PrescriptionService;
import com.yihu.jw.hospital.prescription.service.entrance.XzzxEntranceService;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.Map;

/**
 * Created by zdm on 2019/5/30.
 */
@RestController
@RequestMapping(value ="/prescriptionUpdate")
@Api(value = "处方状态变更", description = "处方状态变更", tags = {"hospital-entrance处方状态变更"})
public class PrescriptionUpdateController extends EnvelopRestEndpoint {

    @Autowired
    private PrescriptionService prescriptionService;
    @Autowired
    private WlyyHttpLogDao wlyyHttpLogDao;
    @Autowired
    private XzzxEntranceService xzzxEntranceService;

    @Value("${wechat.id}")
    private String wxId;

    @Autowired
    private HospitalSystemMessageService hospitalSystemMessageService;

    @PostMapping(value = "/preUpdate")
    @ApiOperation(value = "药房配药his提示处方变更")
    public ObjEnvelop prescriptionUpdate(
            @ApiParam(name = "admNo", value = "门诊就诊唯一号", required = true)
            @RequestParam(value = "admNo") String admNo,
            @ApiParam(name = "realOrder", value = "处方号", required = true)
            @RequestParam(value = "realOrder") String realOrder,
            @ApiParam(name = "status", value = "处方状态：1审方失败，2审核完成，3结算完成", required = true)
            @RequestParam(value = "status") String status) {
        try {
            WlyyHttpLogDO log = new WlyyHttpLogDO();

            log.setCode("prescriptionUpdate");
            log.setName("药房发药回调");
            log.setRequest("admNo="+admNo+"&realOrder="+realOrder+"&status="+status);
            log.setCreateTime(new Date());
            log.setStatus("0");
            log = wlyyHttpLogDao.save(log);
            Map<String,Object> obj= prescriptionService.updatePrescriptionByHisStatu(admNo,realOrder,status);
            log.setResponse("更新结果"+obj.get("flag"));

            wlyyHttpLogDao.save(log);

            //推送IM消息
            hospitalSystemMessageService.sendImMessage((SystemMessageDO)obj.get("mes"));

            return ObjEnvelop.getSuccess("更新成功",obj.get("flag"));
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError("更新失败"+e.getMessage());
        }
    }
    
    
    @PostMapping(value = "/getSFExpressInfo")
    @ApiOperation(value = "获取顺丰物流面单信息")
    public ObjEnvelop getSFExpressInfo(
            @ApiParam(name = "admNo", value = "门诊就诊唯一号", required = true)
            @RequestParam(value = "admNo") String admNo,
            @ApiParam(name = "realOrder", value = "处方号", required = true)
            @RequestParam(value = "realOrder") String realOrder) {
        try {
            WlyyHttpLogDO log = new WlyyHttpLogDO();
            log.setCode("getSFExpressInfo");
            log.setName("获取顺丰物流面单信息");
            log.setRequest("admNo="+admNo+"&realOrder="+realOrder);
            log.setCreateTime(new Date());
            wlyyHttpLogDao.save(log);
            System.out.println("获取顺丰物流面单信息:admNo="+admNo+"&realOrder="+realOrder);
            //String result = "{\"mailno\":\"444844980525\",\"mailtype\":\"标准快递\",\"destcode\":\"592\",\"j_name\":\"厦门大学附属中山医院\",\"j_provinceName\":\"福建省\",\"j_cityName\":\"厦门市\",\"j_townName\":\"思明区\",\"j_address\":\"湖滨南路201-209号\",\"j_phone\":\"4003008888\",\"d_name\":\"陈筑城\",\"d_provinceName\":\"福建省\",\"d_cityName\":\"厦门市\",\"d_townName\":\"思明区\",\"d_address\":\"软件园二期望海路55号\",\"d_phone\":\"18205968888\",\"express_type\":11,\"pay_method\":2,\"receive_time\":\"\",\"dispensaryType\":\"1\"}";
            Object result = prescriptionService.getSFExpressInfo(admNo, realOrder,wxId);
            if ("".equals(result)){
                return ObjEnvelop.getError("获取面单信息失败");
            }else {
                return ObjEnvelop.getSuccess("获取面单信息成功",result);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError("获取面单信息失败,"+e.getMessage());
        }
    }

    @PostMapping(value = "/getSFExpressInfoNewAndOld")
    @ApiOperation(value = "获取顺丰物流面单信息")
    public ObjEnvelop getSFExpressInfoNewAndOld(
            @ApiParam(name = "admNo", value = "门诊就诊唯一号", required = true)
            @RequestParam(value = "admNo") String admNo,
            @ApiParam(name = "realOrder", value = "处方号", required = true)
            @RequestParam(value = "realOrder") String realOrder) {
        try {
            WlyyHttpLogDO log = new WlyyHttpLogDO();
            log.setCode("getSFExpressInfo");
            log.setName("获取顺丰物流面单信息");
            log.setRequest("admNo="+admNo+"&realOrder="+realOrder);
            log.setCreateTime(new Date());
            wlyyHttpLogDao.save(log);
            System.out.println("获取顺丰物流面单信息:admNo="+admNo+"&realOrder="+realOrder);
            //String result = "{\"mailno\":\"444844980525\",\"mailtype\":\"标准快递\",\"destcode\":\"592\",\"j_name\":\"厦门大学附属中山医院\",\"j_provinceName\":\"福建省\",\"j_cityName\":\"厦门市\",\"j_townName\":\"思明区\",\"j_address\":\"湖滨南路201-209号\",\"j_phone\":\"4003008888\",\"d_name\":\"陈筑城\",\"d_provinceName\":\"福建省\",\"d_cityName\":\"厦门市\",\"d_townName\":\"思明区\",\"d_address\":\"软件园二期望海路55号\",\"d_phone\":\"18205968888\",\"express_type\":11,\"pay_method\":2,\"receive_time\":\"\",\"dispensaryType\":\"1\"}";
            Object result = prescriptionService.getSFExpressInfoNew(admNo, realOrder,wxId);
            if ("".equals(result)){
                return ObjEnvelop.getError("获取面单信息失败");
            }else {
                return ObjEnvelop.getSuccess("获取面单信息成功",result);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError("获取面单信息失败,"+e.getMessage());
        }
    }

    @PostMapping(value = "/getSFExpressInfoNew")
    @ApiOperation(value = "获取顺丰物流面单信息")
    public ObjEnvelop getSFExpressInfoNew(
            @ApiParam(name = "admNo", value = "门诊就诊唯一号", required = true)
            @RequestParam(value = "admNo") String admNo,
            @ApiParam(name = "realOrder", value = "处方号", required = false)
            @RequestParam(value = "realOrder") String realOrder) {
        try {
            WlyyHttpLogDO log = new WlyyHttpLogDO();
            log.setCode("getSFExpressInfo");
            log.setName("获取顺丰物流面单信息");
            log.setRequest("admNo="+admNo+"&realOrder="+realOrder);
            log.setCreateTime(new Date());
            wlyyHttpLogDao.save(log);
            System.out.println("获取顺丰物流面单信息:admNo="+admNo+"&realOrder="+realOrder);
            //String result = "{\"mailno\":\"444844980525\",\"mailtype\":\"标准快递\",\"destcode\":\"592\",\"j_name\":\"厦门大学附属中山医院\",\"j_provinceName\":\"福建省\",\"j_cityName\":\"厦门市\",\"j_townName\":\"思明区\",\"j_address\":\"湖滨南路201-209号\",\"j_phone\":\"4003008888\",\"d_name\":\"陈筑城\",\"d_provinceName\":\"福建省\",\"d_cityName\":\"厦门市\",\"d_townName\":\"思明区\",\"d_address\":\"软件园二期望海路55号\",\"d_phone\":\"18205968888\",\"express_type\":11,\"pay_method\":2,\"receive_time\":\"\",\"dispensaryType\":\"1\"}";
            Object result = xzzxEntranceService.getSFExpressInfoToHospital(admNo, realOrder);
            if ("".equals(result)){
                return ObjEnvelop.getError("获取面单信息失败");
            }else {
                return ObjEnvelop.getSuccess("获取面单信息成功",result);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError("获取面单信息失败,"+e.getMessage());
        }
    }


    @PostMapping(value = "/getSFExpressInfoNewYk")
    @ApiOperation(value = "获取顺丰物流面单信息")
    public ObjEnvelop getSFExpressInfoNewYk(
            @ApiParam(name = "admNo", value = "门诊就诊唯一号", required = true)
            @RequestParam(value = "admNo") String admNo,
            @ApiParam(name = "realOrder", value = "处方号", required = true)
            @RequestParam(value = "realOrder") String realOrder) {
        try {
            WlyyHttpLogDO log = new WlyyHttpLogDO();
            log.setCode("getSFExpressInfo");
            log.setName("获取顺丰物流面单信息");
            log.setRequest("admNo="+admNo+"&realOrder="+realOrder);
            log.setCreateTime(new Date());
            wlyyHttpLogDao.save(log);
            System.out.println("获取顺丰物流面单信息:admNo="+admNo+"&realOrder="+realOrder);
            //String result = "{\"mailno\":\"444844980525\",\"mailtype\":\"标准快递\",\"destcode\":\"592\",\"j_name\":\"厦门大学附属中山医院\",\"j_provinceName\":\"福建省\",\"j_cityName\":\"厦门市\",\"j_townName\":\"思明区\",\"j_address\":\"湖滨南路201-209号\",\"j_phone\":\"4003008888\",\"d_name\":\"陈筑城\",\"d_provinceName\":\"福建省\",\"d_cityName\":\"厦门市\",\"d_townName\":\"思明区\",\"d_address\":\"软件园二期望海路55号\",\"d_phone\":\"18205968888\",\"express_type\":11,\"pay_method\":2,\"receive_time\":\"\",\"dispensaryType\":\"1\"}";
            Object result = prescriptionService.getSFExpressInfoNew(admNo, realOrder,wxId);
            if ("".equals(result)){
                return ObjEnvelop.getError("获取面单信息失败");
            }else {
                return ObjEnvelop.getSuccess("获取面单信息成功",result);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError("获取面单信息失败,"+e.getMessage());
        }
    }
}
