package com.yihu.jw.entrance.util.mqsdk;

import com.ewell.mq.queue.QueueTools;
import com.ibm.mq.MQException;
import com.ibm.mq.MQQueueManager;
import net.sf.json.JSON;
import net.sf.json.JSONSerializer;
import net.sf.json.xml.XMLSerializer;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.io.*;

/**
 * Created by zdm on 2019/5/16.
 */
public class MqSdkUtil {
    private static final Logger log = LoggerFactory.getLogger(MqSdkUtil.class);
    //    @Value("${ftp.pwd}")
    private static String gateKeeper="QMGR.S60";
    private QueueTools queueTools;
    private MQQueueManager queueManager;


    /**
     * 通过工具类获取队列管理器实例。connect函数的参数为配置文件中队列管理器配置标签名。该实例如果不调用方法进行断开操作可长期保持连接
     * gateKeeper
     *
     * @return
     */
    public MQQueueManager getMQQueueManager(String gateKeeper) throws Exception {
        QueueTools queueTools = new QueueTools();
        MQQueueManager queueManager = null;
        queueManager = queueTools.connect(gateKeeper);
        return queueManager;
    }

    /**
     * 发送请求消息，保存MsgId
     *
     * @return
     * @throws Exception
     */
    public String putMsg(String reqMsg, String xmlNodeName) throws Exception {
        String msgId = queueTools.putMsg(queueManager, xmlNodeName, reqMsg);
        return msgId;
    }

    /**
     * 根据发送时的MsgId提取请求（使用get-wait短期监听提取方式，可自定义超时时间）
     *
     * @param xmlNodeName
     * @param msgId
     * @return
     * @throws Exception
     */
    public String getResp(String xmlNodeName, String msgId) throws Exception {
        //响应数据
        String respMsg = queueTools.getMsgById(queueManager, xmlNodeName, msgId, 3);
        return respMsg;
    }


    /**
     * 返回的响应数据中会包含此次请求的返回数据量和当前消息在返回消息中的顺序编号，进行判断是否还要继续获取
     * @param reqMsg 请求的数据
     * @param xmlNodeName 获取请求节点
     * @param parsingWay 解析方式：1为单个对象，2为数组
     */
    public static String putReqAndGetResp(String reqMsg,String xmlNodeName,String parsingWay) throws Exception{
        QueueTools queueTools = new QueueTools();
        // 队列管理器实例
        MQQueueManager queueManager = null;
        // 消息Id
        String msgId = null;
        // 响应数据
        String respMsg = null;
        try {
            // 连接MQ，获取队列管理器实例，该实例如果不调用方法进行断开操作可长期保持连接。连接函数会自动读取配置文件标签名为“QMGR.SXX”下的配置信息进行连接。，
            queueManager = queueTools.connect(gateKeeper);
            // 发送请求消息
            msgId = queueTools.putMsg(queueManager, xmlNodeName, reqMsg);
            // 获取响应消息
            respMsg = queueTools.getMsgById(queueManager, xmlNodeName, msgId, 3);
            if (StringUtils.isNotBlank(respMsg)) {
                if ("1".equals(parsingWay)) {
                    respMsg = xml2jsonObject(respMsg);
                } else {
                    respMsg = xml2jsonArray(respMsg);
                }
            }
            // 此处为获取响应数据后的业务处理
            log.info("respMsg==" + respMsg);
            return respMsg;
        } catch (MQException e) {
            // 2033表示队列中没有消息
            if (e.reasonCode == 2033) {
            } else {
                e.printStackTrace();
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            //断开MQ连接
            if (null != queueManager) {
                try {
                    queueManager.disconnect();
                } catch (MQException e) {
                    e.printStackTrace();
                }
            }
        }
        return "";
    }


    //=======================================================封装xml=============================================================================================

    /**
     * JSON(数组)字符串<STRONG>转换</STRONG>成XML字符串
     *
     * @param jsonString
     * @return
     */
    public static String json2xml(String jsonString) {
        XMLSerializer xmlSerializer = new XMLSerializer();
        //是否展示类型
        xmlSerializer.setObjectName("ESBEntry");
        xmlSerializer.setTypeHintsEnabled(false);
//{"Fid":"BS15017","Password":[],"UserName":[]},"MessageHeader":{"Fid":"BS15017","MsgDate":"2018-11-21 13:20:40","SourceSysCode":"S60","TargetSysCode":"S01"},"MsgInfo":{"endNum":"20000","Msg":[],"query":{"@compy":"=","@item":"CARD_NO","@splice":"and","@value":"D26818411"},"startNum":"1"}}
        return xmlSerializer.write(JSONSerializer.toJSON(jsonString));
        // return xmlSerializer.write(JSONArray.fromObject(jsonString));//这种方式只支持JSON数组
    }


    //=================================================================解析xml=======================================================================

    /**
     * 将xml字符串<STRONG>转换</STRONG>为JSON字符串
     * XML中Msg为数组
     *
     * @param xmlString xml字符串
     * @return JSON<STRONG>对象</STRONG>
     */
    public static String xml2jsonArray(String xmlString) throws Exception {
        XMLSerializer xmlSerializer = new XMLSerializer();
        JSON json = xmlSerializer.read(xmlString);
        Object jsonObject = ((net.sf.json.JSONObject) json).get("MsgInfo");
        if (null != jsonObject) {
            net.sf.json.JSONObject jsonObject1 = (net.sf.json.JSONObject) jsonObject;
            //获取到数组中的CDATA进行再解析
            net.sf.json.JSONArray jsonArray = (net.sf.json.JSONArray) jsonObject1.get("Msg");
            net.sf.json.JSONArray jsonArray1 = new net.sf.json.JSONArray();
            for (Object object : jsonArray) {
                JSON jsonsub = xmlSerializer.read(object.toString());
                jsonArray1.add(((net.sf.json.JSONObject) jsonsub).get("row"));
            }
            //重新封装到json中
            ((net.sf.json.JSONObject) json).element("MsgInfo", jsonArray1);
        }
        return getMsgInfo(json);
    }

    /**
     * 将xml字符串<STRONG>转换</STRONG>为JSON字符串
     * 其中Msg为一个对象
     *
     * @param xmlString xml字符串
     * @return JSON<STRONG>对象</STRONG>
     */
    public static String xml2jsonObject(String xmlString) throws Exception {
        XMLSerializer xmlSerializer = new XMLSerializer();
        JSON json = xmlSerializer.read(xmlString);
        Object jsonObject = ((net.sf.json.JSONObject) json).get("MsgInfo");
        if (null != jsonObject) {
            net.sf.json.JSONObject jsonObject1 = (net.sf.json.JSONObject) jsonObject;
            //获取到数组中的CDATA进行再解析
            JSON jsonsub = xmlSerializer.read((jsonObject1).get("Msg").toString());
            //重新封装到json中
            ((net.sf.json.JSONObject) jsonObject).element("Msg", jsonsub);
        }
        return getMsgInfo(json);
    }

    /**
     * 获取返参的主体信息
     * @param json
     * @return
     * @throws Exception
     */
    public static String getMsgInfo(JSON json)throws Exception{
        Object jsonObject= (( net.sf.json.JSONObject) json).get("RetInfo");
        if(null!=jsonObject){
            net.sf.json.JSONObject  jsonObject1=( net.sf.json.JSONObject)jsonObject;
            if(null!=jsonObject1.get("RetCode")&&"1".equals(jsonObject1.get("RetCode").toString())){
                return (( net.sf.json.JSONObject) json).get("MsgInfo").toString();
            }
        }
        return "";
    }

}
