package com.yihu.jw.job;

import com.yihu.jw.service.channel.PrescriptionStatusUpdateService;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 处方状态更新job
 * Created by zdm on 2019/2/20.
 * 互联网医院线上下处方之后保存到中山医院的临时表-
 * （中山医院正式下处方，审核成功之后保存，审核失败不保存）-审核成功之后药房发药，回调互联网医院。
 * 在正式下处方的时候，互联网医院每两分钟到中山医院获取处方详情：
 * 1、若是可以正常查询且状态为审方成功，则只需要变更状态，调用im消息通知居民
 * 2、若可以正常查询处方，状态为失败，则需要将状态改为失败，并根据处方号重新获取新的处方直到成功为止。
 * 3、若根据处方号查询不到处方，则需要根据门诊就诊记录获取处方号，再获取处方状态重复1、2
 */
public class PrescriptionStatusUpdateJob implements Job {
    private static final Logger logger = LoggerFactory.getLogger(PrescriptionStatusUpdateJob.class);
    @Autowired
    private PrescriptionStatusUpdateService prescriptionStatusUpdateService;
    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        logger.info("START========PrescriptionStatusUpdateJob========");
        try {
            prescriptionStatusUpdateService.autoPush();
            logger.info("END========PrescriptionStatusUpdateJob========");
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("END===ERROE===PrescriptionStatusUpdateJob,message:"+e.getMessage());
        }
    }
}
