package com.yihu.jw.service.channel;

import com.yihu.jw.dailyReport.dao.BaseDailyReportDetailDao;
import com.yihu.jw.dailyReport.dao.BaseDailyReportUploadDao;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.doctor.dao.BaseDoctorHospitalDao;
import com.yihu.jw.doctor.dao.BaseDoctorRoleDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.doctor.BaseDoctorHospitalDO;
import com.yihu.jw.entity.base.doctor.BaseDoctorRoleDO;
import com.yihu.jw.entity.dailyReport.BaseDailyReportUploadDO;
import com.yihu.jw.entity.hospital.enterprise.WxEnterpriseUserDO;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.wechat.WeixinMessagePushUtils;
import com.yihu.jw.wechat.dao.WxPushLogDao;
import com.yihu.jw.wechat.dao.WxTemplateConfigDao;
import com.yihu.jw.wechat.dao.WxTemplateDao;
import com.yihu.jw.wechat.enterprise.EnterpriseService;
import com.yihu.jw.wechat.enterprise.dao.WxEnterpriseUserDao;
import com.yihu.jw.wechat.service.WxAccessTokenService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * Created by Bing on 2022/7/26.
 */
@Service
public class DailyReportService {

    @Autowired
    private WxTemplateConfigDao wxTemplateConfigDao;
    @Autowired
    private WxTemplateDao wxTemplateDao;
    @Autowired
    private WxAccessTokenService wxAccessTokenService;
    @Autowired
    private WeixinMessagePushUtils weixinMessagePushUtils;
    @Autowired
    private BaseDailyReportDetailDao reportDetailDao;
    @Autowired
    private BaseDailyReportUploadDao reportUploadDao;
    @Value("${wechat.id}")
    private String wechatId;
    @Autowired
    private BaseDoctorDao doctorDao;
    @Autowired
    private WxPushLogDao wxPushLogDao;
    @Autowired
    private WxEnterpriseUserDao wxEnterpriseUserDao;
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private BaseDoctorRoleDao doctorRoleDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private BaseDoctorHospitalDao baseDoctorHospitalDao;

    private Logger logger= LoggerFactory.getLogger(DailyReportService.class);

    public void dailyReportRemind() throws Exception {
        List<BaseDoctorDO> doctorDOList = doctorDao.findByDel();
        String nowDate = DateUtil.getStringDateShort();
        for (BaseDoctorDO doctorDO:doctorDOList){
            if ("0592008".equals(doctorDO.getId())){
                continue;
            }
            BaseDailyReportUploadDO reportUploadDO = reportUploadDao.findByDoctorIdAndReportDate(doctorDO.getId(),nowDate);
            if (null==reportUploadDO){
                WxEnterpriseUserDO enterpriseUserDO = wxEnterpriseUserDao.findByEnterpriseIdAndMobile(wechatId,doctorDO.getMobile());
                if(enterpriseUserDO==null){
                    logger.info("该用户"+doctorDO.getName()+"没有企业微信手机号,无法推送模版消息，用户ID："+doctorDO.getId()+"wechatId:"+wechatId);
                }else{
                    String title = "待办事项";
                    String des = "您好，系统检测到您今日尚未提交日报信息，请及时提交！";
                    String url = "https://ehr.yihu.com/hlwyy/zjxl/dailyReport/#/workSummary/commit";
                    String res = enterpriseService.sendTWMesByDoctor(wechatId,doctorDO.getId(),title,des,url);
                }
            }
        }
    }

    /**
     * 日报未上传通知
     * @throws Exception
     */
    public void dailyReportWsbTotalRemind() throws Exception {
        String boosID = "0592008";//老板id
        String nowDate = DateUtil.getStringDateShort();
        List<BaseDoctorRoleDO> deptAdminRoleList = doctorRoleDao.findByRoleCode("deptAdmin");
        //管理员
        for (BaseDoctorRoleDO adminRole : deptAdminRoleList) {
            BaseDoctorDO doctorDO = doctorDao.findById(adminRole.getDoctorCode()).orElse(null);
            String dept_code = "";
            if (doctorDO != null) {
                List<BaseDoctorHospitalDO> doctorHospitalDOs = baseDoctorHospitalDao.findByDoctorCode(doctorDO.getId());
                if (doctorHospitalDOs.size() > 0) {
                    dept_code = doctorHospitalDOs.get(0).getDeptCode();
                }
            }
            WxEnterpriseUserDO enterpriseUserDO = wxEnterpriseUserDao.findByEnterpriseIdAndMobile(wechatId, doctorDO.getMobile());
            if (enterpriseUserDO == null) {
                logger.info("该用户" + doctorDO.getName() + "没有企业微信手机号,无法推送模版消息，用户ID：" + doctorDO.getId() + "wechatId:" + wechatId);
            } else {
                //获取部门人员
                if (StringUtils.isNotBlank(dept_code)) {
                    String doctorSql = " select COUNT(distinct d.id) as 'total' from base_doctor d  INNER JOIN base_doctor_hospital dh on d.id = dh.doctor_code " +
                            "where d.del=1 and dh.del=1 and dh.dept_code='" + dept_code + "'  ";
                    Integer doctorTotal = jdbcTemplate.queryForObject(doctorSql, Integer.class);
                    //查询出已经上传人数
                    String uploadSql = " select count(1) from base_doctor_daily_report_upload up " +
                            "where up.report_date>='" + nowDate + "' and up.report_date<='" + nowDate + "' and up.doctor_id in ( " +
                            " select d.id from base_doctor d  INNER JOIN base_doctor_hospital dh on d.id = dh.doctor_code " +
                            "where d.del=1 and dh.del=1 and dh.dept_code='" + dept_code + "' )  ";
                    Integer sqlTotal = jdbcTemplate.queryForObject(uploadSql, Integer.class);
                    Integer weishangbaoTotal = doctorTotal - sqlTotal;
                    String title = "每日日报未上报汇总";
                    String des = "今日还有" + weishangbaoTotal + "人未提交日报，点击查看。";
                    String url = "https://ehr.yihu.com/hlwyy/zjxl/dailyReport/#/home/index";
                    String res = enterpriseService.sendTWMesByDoctor(wechatId, doctorDO.getId(), title, des, url);
                }
            }
        }

        String adminDoctorSql = " select distinct doctor_code from base_doctor_role where doctor_code<>'0592008'  ";
        List<String> adminDoctorS = jdbcTemplate.queryForList(adminDoctorSql,String.class);
        if (adminDoctorS.size()>0){
            BaseDoctorDO doctorDO = doctorDao.findById(boosID).orElse(null);
            if (doctorDO!=null){
                WxEnterpriseUserDO enterpriseUserDO = wxEnterpriseUserDao.findByEnterpriseIdAndMobile(wechatId, doctorDO.getMobile());
                if (enterpriseUserDO == null) {
                    logger.info("该用户" + doctorDO.getName() + "没有企业微信手机号,无法推送模版消息，用户ID：" + doctorDO.getId() + "wechatId:" + wechatId);
                } else {
                    String doctorSql = " select COUNT(distinct d.id) as 'total' from base_doctor d and d.id in ( "+adminDoctorSql+" )  ";
                    Integer doctorTotal = jdbcTemplate.queryForObject(doctorSql, Integer.class);
                    //查询出已经上传人数
                    String uploadSql = " select count(1) from base_doctor_daily_report_upload up " +
                            "where up.report_date>='" + nowDate + "' and up.report_date<='" + nowDate + "' and up.doctor_id in ( "+adminDoctorSql+" )  ";
                    Integer sqlTotal = jdbcTemplate.queryForObject(uploadSql, Integer.class);
                    Integer weishangbaoTotal = doctorTotal - sqlTotal;
                    String title = "每日日报未上报汇总";
                    String des = "今日还有" + weishangbaoTotal + "人未提交日报，点击查看。";
                    String url = "https://ehr.yihu.com/hlwyy/zjxl/dailyReport/#/home/index";
                    String res = enterpriseService.sendTWMesByDoctor(wechatId, doctorDO.getId(), title, des, url);
                }
            }
        }
    }

    /**
     * 日报上传统计
     * @throws Exception
     */
    public void dailyReportTotalRemind() throws Exception {
        String boosID = "0592008";//老板id
        String nowDate = DateUtil.getStringDateShort();
        List<BaseDoctorRoleDO> deptAdminRoleList = doctorRoleDao.findByRoleCode("deptAdmin");
        //管理员
        for (BaseDoctorRoleDO adminRole : deptAdminRoleList) {
            BaseDoctorDO doctorDO = doctorDao.findById(adminRole.getDoctorCode()).orElse(null);
            String dept_code = "";
            if (doctorDO != null) {
                List<BaseDoctorHospitalDO> doctorHospitalDOs = baseDoctorHospitalDao.findByDoctorCode(doctorDO.getId());
                if (doctorHospitalDOs.size() > 0) {
                    dept_code = doctorHospitalDOs.get(0).getDeptCode();
                }
            }
            WxEnterpriseUserDO enterpriseUserDO = wxEnterpriseUserDao.findByEnterpriseIdAndMobile(wechatId, doctorDO.getMobile());
            if (enterpriseUserDO == null) {
                logger.info("该用户" + doctorDO.getName() + "没有企业微信手机号,无法推送模版消息，用户ID：" + doctorDO.getId() + "wechatId:" + wechatId);
            } else {
                //获取部门人员
                if (StringUtils.isNotBlank(dept_code)) {
                    String doctorSql = " select COUNT(distinct d.id) as 'total' from base_doctor d  INNER JOIN base_doctor_hospital dh on d.id = dh.doctor_code " +
                            "where d.del=1 and dh.del=1 and dh.dept_code='" + dept_code + "'  ";
                    Integer doctorTotal = jdbcTemplate.queryForObject(doctorSql, Integer.class);
                    //查询出已经上传人数
                    String uploadSql = " select count(1) from base_doctor_daily_report_upload up " +
                            "where up.report_date>='" + nowDate + "' and up.report_date<='" + nowDate + "' and up.doctor_id in ( " +
                            " select d.id from base_doctor d  INNER JOIN base_doctor_hospital dh on d.id = dh.doctor_code " +
                            "where d.del=1 and dh.del=1 and dh.dept_code='" + dept_code + "' )  ";
                    Integer yishangbaoTotal = jdbcTemplate.queryForObject(uploadSql, Integer.class);
                    Integer weishangbaoTotal = doctorTotal - yishangbaoTotal;
                    //已核实
                    String verifiedTotalSql = " select count(1) from base_doctor_daily_report_upload up " +
                            "where up.report_date>='" + nowDate + "' and up.report_date<='" + nowDate + "' and up.state=1 and up.doctor_id in ( " +
                            " select d.id from base_doctor d  INNER JOIN base_doctor_hospital dh on d.id = dh.doctor_code " +
                            "where d.del=1 and dh.del=1 and dh.dept_code='" + dept_code + "' )  ";
                    Integer verifiedTotal = jdbcTemplate.queryForObject(verifiedTotalSql,Integer.class);
                    //未核实
                    String notVerifiedTotalSql = " select count(1) from base_doctor_daily_report_upload up " +
                            "where up.report_date>='" + nowDate + "' and up.report_date<='" + nowDate + "' and up.state=0 and up.doctor_id in ( " +
                            " select d.id from base_doctor d  INNER JOIN base_doctor_hospital dh on d.id = dh.doctor_code " +
                            "where d.del=1 and dh.del=1 and dh.dept_code='" + dept_code + "' )  ";
                    Integer notVerifiedTotal = jdbcTemplate.queryForObject(notVerifiedTotalSql,Integer.class);
                    String title = "每日日报上报汇总";
                    String des = "今日上报情况：已上报："+yishangbaoTotal+" 未上报："+weishangbaoTotal+" 已核实："+verifiedTotal+" 未核实：" + notVerifiedTotal + ";点击查看；";
                    String url = "https://ehr.yihu.com/hlwyy/zjxl/dailyReport/#/home/index";
                    String res = enterpriseService.sendTWMesByDoctor(wechatId, doctorDO.getId(), title, des, url);
                }
            }
        }

        String adminDoctorSql = " select distinct doctor_code from base_doctor_role where doctor_code<>'0592008'  ";
        List<String> adminDoctorS = jdbcTemplate.queryForList(adminDoctorSql,String.class);
        if (adminDoctorS.size()>0){
            BaseDoctorDO doctorDO = doctorDao.findById(boosID).orElse(null);
            if (doctorDO!=null){
                WxEnterpriseUserDO enterpriseUserDO = wxEnterpriseUserDao.findByEnterpriseIdAndMobile(wechatId, doctorDO.getMobile());
                if (enterpriseUserDO == null) {
                    logger.info("该用户" + doctorDO.getName() + "没有企业微信手机号,无法推送模版消息，用户ID：" + doctorDO.getId() + "wechatId:" + wechatId);
                } else {
                    String doctorSql = " select COUNT(distinct d.id) as 'total' from base_doctor d where 1=1 and d.id in ( "+adminDoctorSql+" )  ";
                    Integer doctorTotal = jdbcTemplate.queryForObject(doctorSql, Integer.class);
                    //查询出已经上传人数
                    String uploadSql = " select count(1) from base_doctor_daily_report_upload up " +
                            "where up.report_date>='" + nowDate + "' and up.report_date<='" + nowDate + "' and up.doctor_id in ( "+adminDoctorSql+" )  ";
                    Integer yishangbaoTotal = jdbcTemplate.queryForObject(uploadSql, Integer.class);
                    Integer weishangbaoTotal = doctorTotal - yishangbaoTotal;
                    //已核实
                    String verifiedTotalSql = " select count(1) from base_doctor_daily_report_upload up " +
                            "where up.report_date>='" + nowDate + "' and up.report_date<='" + nowDate + "' and up.state=1 and up.doctor_id in ( "+adminDoctorSql+" )  ";
                    Integer verifiedTotal = jdbcTemplate.queryForObject(verifiedTotalSql,Integer.class);
                    //未核实
                    String notVerifiedTotalSql = " select count(1) from base_doctor_daily_report_upload up " +
                            "where up.report_date>='" + nowDate + "' and up.report_date<='" + nowDate + "' and up.state=0 and up.doctor_id in ( "+adminDoctorSql+" )  ";
                    Integer notVerifiedTotal = jdbcTemplate.queryForObject(notVerifiedTotalSql,Integer.class);
                    String title = "每日日报上报汇总";
                    String des = "今日上报情况：已上报："+yishangbaoTotal+" 未上报："+weishangbaoTotal+"已核实："+verifiedTotal+"未核实：" + notVerifiedTotal + ";点击查看；";
                    String url = "https://ehr.yihu.com/hlwyy/zjxl/dailyReport/#/home/index";
                    String res = enterpriseService.sendTWMesByDoctor(wechatId, doctorDO.getId(), title, des, url);
                }
            }
        }
    }

}
