package com.yihu.jw.service.channel;


import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.base.wx.WxPushLogDO;
import com.yihu.jw.entity.hospital.consult.WlyyHospitalSysDictDO;
import com.yihu.jw.entity.hospital.consult.WlyyHospitalWaitingRoomDO;
import com.yihu.jw.entity.hospital.doctor.WlyyPatientRegisterTimeDO;
import com.yihu.jw.entity.hospital.message.SystemMessageDO;
import com.yihu.jw.entity.hospital.prescription.WlyyOutpatientDO;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionDO;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionDiagnosisDO;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionInfoDO;
import com.yihu.jw.entity.order.BusinessOrderDO;
import com.yihu.jw.hospital.consult.dao.HospitalWaitingRoomDao;
import com.yihu.jw.hospital.doctor.dao.PatientRegisterTimeDao;
import com.yihu.jw.hospital.message.service.SystemMessageService;
import com.yihu.jw.hospital.prescription.dao.OutpatientDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionDiagnosisDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionInfoDao;
import com.yihu.jw.hospital.prescription.service.PrescriptionLogService;
import com.yihu.jw.hospital.prescription.service.PrescriptionService;
import com.yihu.jw.hospital.prescription.service.entrance.EntranceService;
import com.yihu.jw.hospital.prescription.service.entrance.YkyyEntranceService;
import com.yihu.jw.hospital.ykyy.service.YkyyService;
import com.yihu.jw.im.service.ImService;
import com.yihu.jw.order.BusinessOrderService;
import com.yihu.jw.order.dao.BusinessOrderDao;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.restmodel.hospital.prescription.WlyyPrescriptionDiagnosisVO;
import com.yihu.jw.restmodel.hospital.prescription.WlyyPrescriptionInfoVO;
import com.yihu.jw.restmodel.hospital.prescription.WlyyPrescriptionVO;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.status.EnvelopStatus;
import com.yihu.jw.sms.dao.HospitalSysDictDao;
import com.yihu.jw.util.common.IdCardUtil;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.http.HttpUtils;
import com.yihu.jw.utils.YkyySMSService;
import com.yihu.jw.utils.hibernate.HibenateUtils;
import com.yihu.jw.wechat.dao.WxPushLogDao;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * 处方状态更新job
 * Created by zdm on 2019/7/20.
 * 互联网医院线上下处方之后保存到中山医院的临时表-（中山医院正式下处方，审核成功之后保存，审核失败不保存）-审核成功之后药房发药，回调互联网医院。
 * 在正式下处方的时候，互联网医院每两分钟到中山医院获取处方详情：
 * 1、若是可以正常查询且状态为审方成功，则只需要变更状态，调用im消息通知居民
 * 2、若可以正常查询处方，状态为失败，则需要将状态改为失败，并根据处方号重新获取新的处方直到成功为止。
 * 3、若根据处方号查询不到处方，则需要根据门诊就诊记录(有就诊次数的数据)获取处方号，再获取处方状态重复1、2
 */
@Component
@Transactional
public class PrescriptionStatusUpdateService {
    private static Logger logger = LoggerFactory.getLogger(PrescriptionStatusUpdateService.class);
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Value("${wechat.ids}")
    private String wechatId;
    @Value("${hlwyyEntrance.url}")
    private String hlwyyEntranceUrl;
    @Autowired
    private PrescriptionDao prescriptionDao;
    @Autowired
    private PrescriptionDiagnosisDao prescriptionDiagnosisDao;
    @Autowired
    private PrescriptionInfoDao prescriptionInfoDao;
    @Autowired
    private OutpatientDao outpatientDao;
    @Autowired
    private ImService imService;
    @Autowired
    private BusinessOrderDao businessOrderDao;
    @Autowired
    private BusinessOrderService businessOrderService;
    @Autowired
    private PrescriptionLogService prescriptionLogService;
    @Autowired
    private HospitalWaitingRoomDao hospitalWaitingRoomDao;
    @Autowired
    private PatientRegisterTimeDao patientRegisterTimeDao;
    @Autowired
    private YkyyService ykyyService;
    @Autowired
    private BasePatientDao basePatientDao;
    @Autowired
    private PrescriptionService prescriptionService;
    @Autowired
    private SystemMessageService systemMessageService;
    @Autowired
    private YkyyEntranceService ykyyEntranceService;
    @Autowired
    private HibenateUtils hibenateUtils;
    @Autowired
    private HospitalSysDictDao hospitalSysDictDao;
    @Autowired
    private WxPushLogDao wxPushLogDao;
    @Autowired
    private YkyySMSService ykyySMSService;
    @Autowired
    private EntranceService entranceService;

    public void autoPush() throws Exception {

        logger.info("autoPush start");
        //获取所有就诊中已挂号、已下临时处方、并且未结束的处方
        String sql = "SELECT o.id,o.con_no,o.register_no,p.mapping_code,pre.adm_no,pre.real_order,pre.id preId,pre.dispensary_type,pre.status " +
                "FROM base.wlyy_outpatient o LEFT JOIN base.wlyy_patient_mapping p ON o.patient=p.patient " +
                "LEFT JOIN base.wlyy_prescription pre on o.id=pre.outpatient_id " +
                "WHERE o.status=1 AND o.con_no IS NOT NULL  AND pre.real_order IS NOT NULL  ";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        for (Map<String, Object> map : list) {
            //处方号
            String realOrder = String.valueOf(map.get("real_order"));
            //门诊就诊唯一号
            String admNo = String.valueOf(map.get("adm_no"));
            //第三方居民唯一号
            String patientMappingCode = String.valueOf(map.get("mapping_code"));
            //中山医院居民就诊次数
            String conNo = String.valueOf(map.get("con_no"));
            //处方原状态
            String status = String.valueOf(map.get("status"));
            //处方Id
            String preId = String.valueOf(map.get("preId"));
            //门诊记录Id
            String outPatientId = String.valueOf(map.get("id"));

            //根据处方号获取处方状态
            Map<String, Object> params = new HashMap<>();
            params.put("registerSn", null);
            params.put("patNo", patientMappingCode);
//            params.put("admNo", admNo);
            params.put("realOrder", realOrder);
            //判断处方是否存在
            boolean preExistFlag = true;
            List<WlyyPrescriptionVO> wlyyPrescriptionVOS = getWlyyPrescriptionVO(params);
            //若用处方号获取处方为空，则需要使用居民编号及门诊就诊唯一号查询
            if (null == wlyyPrescriptionVOS) {
                preExistFlag = false;
                params.remove("realOrder");
                wlyyPrescriptionVOS = getWlyyPrescriptionVO(params);
            }

            //根据处方id获取处方详情
            WlyyPrescriptionDO wlyyPrescriptionDO = prescriptionDao.findById(preId).orElse(null);
            //根据门诊id获取门诊记录:如果处方状态为已完成，需要将门诊状态改为已诊
            WlyyOutpatientDO wlyyOutpatientDO = outpatientDao.findById(outPatientId).orElse(null);

            if (null != wlyyPrescriptionVOS) {

                ObjectMapper objectMapper = new ObjectMapper();
                WlyyPrescriptionVO wlyyPrescriptionVO = objectMapper.convertValue(wlyyPrescriptionVOS.get(0), WlyyPrescriptionVO.class);

                //如果处方状态一致，则不做操作
                if (wlyyPrescriptionDO.getStatus() != wlyyPrescriptionVO.getStatus()) {
                    return;
                }
                //存储处方操作日志
                prescriptionLogService.addPrescriptionLog(wlyyPrescriptionDO.getId(),wlyyPrescriptionDO.getStatus(),2, wlyyPrescriptionDO.getDoctor(), wlyyPrescriptionDO.getDoctorName(),"", new Date());

                //调用im发送模板消息
                /**
                 * 续方取消：-3 支付过期取消，-2 患者自己取消 ，-1 医生取消
                 * 候诊中：0 候诊中
                 * 就诊中：10 诊断中，11 药师审核失败 / 调整中，12  药师审核完成，13  开方失败/调整中
                 * 待支付：20 诊断完成/开方成功/待支付，21 支付失败/待支付中
                 * 待取药：30 支付成功/等待配药，31 配药成功/等待取药，32 配送中
                 * 已完成：100 已完成/未评价 ，101 已完成已经评价
                 */
                try {
                    if (13 < wlyyPrescriptionVO.getStatus() && wlyyPrescriptionDO.getStatus() < 20) {
                        logger.info("autoPush imService send");
                        imService.pushPrescriptionImMessage(wlyyPrescriptionVO);
                    }
                } catch (Exception e) {
                    logger.error("im异常:" + e.getMessage());
                }


                //his处方变更时，会删除原本处方，重新更新处方到本地
                if (!preExistFlag) {
                    updatePrescriptionInfo(preId,wlyyPrescriptionVO,wlyyOutpatientDO);
                }
                /*//更改门诊状态为已经检查
                if (100 == wlyyPrescriptionVO.getStatus()) {
                    wlyyOutpatientDO.setStatus("3");
                }
                outpatientDao.save(wlyyOutpatientDO);*/

                //更新处方状态记录
                wlyyPrescriptionDO.setStatus(wlyyPrescriptionVO.getStatus());
                prescriptionDao.save(wlyyPrescriptionDO);
            }
        }
        logger.info("autoPush end");
    }

    public void updatePrescriptionInfo(String preId, WlyyPrescriptionVO wlyyPrescriptionVO, WlyyOutpatientDO wlyyOutpatientDO) {

        logger.info("autoPush updatePrescriptionInfo ");
        //删除原有药品信息
        List<WlyyPrescriptionInfoDO> oldInfos = prescriptionInfoDao.findByPrescriptionId(preId,1);
        if (oldInfos != null && oldInfos.size() > 0) {
            prescriptionInfoDao.deleteByPrescriptionId(preId);
        }
        //删除之前诊断
        List<WlyyPrescriptionDiagnosisDO> ds = prescriptionDiagnosisDao.findByPrescriptionId(preId,1);
        if (ds != null && ds.size() > 0) {
            prescriptionDiagnosisDao.deleteByPrescriptionId(preId);
        }
        String Icd10 = "";
        String Icd10Name = "";
        List<WlyyPrescriptionDiagnosisDO> wlyyPrescriptionDiagnosisDOS = new ArrayList<>();
        for (WlyyPrescriptionDiagnosisVO diagnosisVO : wlyyPrescriptionVO.getDiagnosisVOs()) {
            WlyyPrescriptionDiagnosisDO diagnosisDO = new WlyyPrescriptionDiagnosisDO();
            //将诊断拼接放到门诊记录中
            if ("1".equals(diagnosisVO.getType())) {
                Icd10 = diagnosisVO.getCode() + "," + Icd10;
                Icd10Name += diagnosisVO.getName() + "," + Icd10Name;
            } else {
                Icd10 += diagnosisVO.getCode() + ",";
                Icd10Name += diagnosisVO.getName() + ",";
            }
            diagnosisDO.setDel(1);
            diagnosisDO.setCode(diagnosisVO.getCode());
            diagnosisDO.setName(diagnosisVO.getName());
            diagnosisDO.setType(diagnosisVO.getType());
            diagnosisDO.setPrescriptionId(preId);
            diagnosisDO.setCreateTime(new Date());
            wlyyPrescriptionDiagnosisDOS.add(diagnosisDO);
        }
        //重新保存诊断
        prescriptionDiagnosisDao.saveAll(wlyyPrescriptionDiagnosisDOS);
        //药品更新
        List<WlyyPrescriptionInfoDO> wlyyPrescriptionInfoDOS = new ArrayList<>();
        for (WlyyPrescriptionInfoVO infoVO : wlyyPrescriptionVO.getInfoVOs()) {
            WlyyPrescriptionInfoDO infoDO = new WlyyPrescriptionInfoDO();
            //处方code 关联表wlyy_prescription code
            infoDO.setPrescriptionId(preId);
            //药品编号
            infoDO.setDrugNo(infoVO.getDrugNo());
            //药品名称
            infoDO.setDrugName(infoVO.getDrugName());
            //库房号
            infoDO.setDispDeposite(infoVO.getDispDeposite());
            // 药量
            infoDO.setDosage(infoVO.getDosage());
            //数量
            infoDO.setQuantity(infoVO.getQuantity());
            //单位
            infoDO.setUnit(infoVO.getUnit());
            //單位名稱
            infoDO.setUnitName(infoVO.getUnitName());
            //包装单位
            infoDO.setPackUnit(infoVO.getPackUnit());
            //包装单位名称
            infoDO.setPackUnitName(infoVO.getPackUnitName());
            // 用法
            infoDO.setSupplyCode(infoVO.getSupplyCode());
            //用法名稱
            infoDO.setSupplyName(infoVO.getSupplyName());
            //天数
            infoDO.setDays(infoVO.getDays());
            //组号
            infoDO.setFrequency(infoVO.getFrequency());
            //药品收费码
            infoDO.setSerial(infoVO.getSerial());
            //库房号
            infoDO.setGroupNo(infoVO.getGroupNo());
            //规格
            infoDO.setSpecification(infoVO.getSpecification());
            // 1可用 0删除
            infoDO.setDel(1);
            //频率
            infoDO.setUsageCode(infoVO.getUsageCode());
            //頻率名稱
            infoDO.setUsageName(infoVO.getUsageName());
            //包装价格
            infoDO.setPackRetprice(infoVO.getPackRetprice());
            wlyyPrescriptionInfoDOS.add(infoDO);
        }
        prescriptionInfoDao.saveAll(wlyyPrescriptionInfoDOS);

        Icd10 = Icd10.substring(0, Icd10.length() - 1);
        Icd10Name = Icd10Name.substring(0, Icd10Name.length() - 1);
        wlyyOutpatientDO.setIcd10(Icd10);
        wlyyOutpatientDO.setIcd10Name(Icd10Name);

    }

    /**
     * 获取处方详情
     *
     * @param params
     * @return
     * @throws Exception
     */
    public List<WlyyPrescriptionVO> getWlyyPrescriptionVO(Map<String, Object> params) throws Exception {
        //根据处方号获取处方状态
        String result = HttpUtils.doGet(hlwyyEntranceUrl + "/mqsdk/BS16017", params).getContent();
        ObjectMapper objectMapper = new ObjectMapper();
        ListEnvelop listEnvelop = objectMapper.readValue(result, ListEnvelop.class);
        if (null != listEnvelop && EnvelopStatus.success.code.equals(listEnvelop.getStatus()) && null != listEnvelop.getDetailModelList() && listEnvelop.getDetailModelList().size() > 0) {
//                logger.info("result:"+listEnvelop.getDetailModelList());
            List<WlyyPrescriptionVO> wlyyPrescriptionVOS = listEnvelop.getDetailModelList();
            return wlyyPrescriptionVOS;
        }
        return null;
    }


    /**
     * 设置门诊为已完成
     */
    public void setOutPatientOver(){
        logger.info("setOutPatientOver start ");
        //复诊记录状态状态安全锁，每天0点才允许触发，避免误调用
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        int i = calendar.get(Calendar.HOUR_OF_DAY);
        //将所有的已接诊的处方记录
        List<WlyyOutpatientDO> outpatientDOs = outpatientDao.findByStatus();
        if(outpatientDOs!=null&&outpatientDOs.size()>0){
            System.out.println(outpatientDOs.get(0).getId()+"id______-");
            for(WlyyOutpatientDO outpatientDO:outpatientDOs){
                long currentTime = new Date().getTime();
                Date temp = new Date();
                if (outpatientDO.getConDate()==null){
                    temp = outpatientDO.getCreateTime();
                }else {
                    temp = outpatientDO.getConDate();
                }
                long outpatientTime = outpatientDO.getAdmDate()==null?temp.getTime():outpatientDO.getAdmDate().getTime();

                if(currentTime-outpatientTime>24*60*60*1000){
                    logger.info("结束接诊时间满足条件");
                    //结束门诊
                    outpatientDO.setStatus("3");
                    outpatientDO.setEndTime(new Date());
                    String consultCode ="";
                    try {
                        consultCode = imService.getConsultCodeByOutpatientId(outpatientDO.getId());
                        logger.info("consultCode"+consultCode);
                        if(StringUtils.isNoneBlank(consultCode)){
                            imService.finish(consultCode,"admin",2);
                        }
                    }catch(Exception e)  {
                        e.printStackTrace();
                        logger.info("自动结束咨询报错 consultCode： "+consultCode);
                    }
                    logger.info(outpatientDO.getId()+"的处方记录设置为结束");
                }
            }
            outpatientDao.saveAll(outpatientDOs);
            /*//退费
            for (WlyyOutpatientDO outpatientDO:outpatientDOs){
                long currentTime = new Date().getTime();
                Date temp = new Date();
                if (outpatientDO.getConDate()==null){
                    temp = outpatientDO.getCreateTime();
                }else {
                    temp = outpatientDO.getConDate();
                }
                long outpatientTime = outpatientDO.getAdmDate()==null?temp.getTime():outpatientDO.getAdmDate().getTime();
                if(currentTime-outpatientTime>24*60*60*1000) {
                    logger.info("退费时间满足条件");
                    try {
                        BusinessOrderDO businessOrderDO = businessOrderDao.selectByRelationCode(outpatientDO.getId());
                        if (businessOrderDO!=null){
                            if (businessOrderDO.getPayType()==1){
                                //businessOrderService.orderRefund(wechatId,businessOrderDO.getPatient(),businessOrderDO.getOrderNo(),businessOrderDO.getPayPrice(),businessOrderDO.getDescription());
                            }else if (businessOrderDO.getPayType()==3){
    *//*
                        businessOrderService.ylzOrderRefund(wechatId,businessOrderDO.getPatient(),businessOrderDO.getOrderNo(),businessOrderDO.getPayPrice(),businessOrderDO.getDescription());
    *//*
                            }
                        } } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }*/
            logger.info("setOutPatientOver count :"+outpatientDOs.size());
        }else {
            logger.info("setOutPatientOver count :"+0);
        }
        logger.info("setOutPatientOver end ");

    }


    public void setOutPatientStatusOver(){
        logger.info("setOutPatientOver start ");
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        int i = calendar.get(Calendar.HOUR_OF_DAY);
        //复诊记录状态状态安全锁，每天0~2点才允许触发，避免误调用
        //将所有的已接诊的处方记录
        if(i==0||i==1){
            List<WlyyOutpatientDO> outpatientDOs = outpatientDao.findByStatus();
            if(outpatientDOs!=null&&outpatientDOs.size()>0){

                for(WlyyOutpatientDO outpatientDO:outpatientDOs){
                    //结束门诊
                    outpatientDO.setStatus("0");
                }
                outpatientDao.saveAll(outpatientDOs);
                //退费
                for (WlyyOutpatientDO outpatientDO:outpatientDOs){
                    BusinessOrderDO businessOrderDO = businessOrderDao.selectByRelationCode(outpatientDO.getId());
                    if (businessOrderDO!=null){
                        try {
                            if (businessOrderDO.getPayType()==1){
                                businessOrderService.orderRefund(wechatId,businessOrderDO.getPatient(),businessOrderDO.getOrderNo(),businessOrderDO.getPayPrice(),businessOrderDO.getDescription());
                            }else if (businessOrderDO.getPayType()==3){
/*
                                businessOrderService.ylzOrderRefund(wechatId,businessOrderDO.getPatient(),businessOrderDO.getOrderNo(),businessOrderDO.getPayPrice(),businessOrderDO.getDescription());
*/
                            }
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    }
                }


                logger.info("setOutPatientOver count :"+outpatientDOs.size());
            }else {
                logger.info("setOutPatientOver count :"+0);
            }
        }else{
            logger.info("setOutPatientOver Time is not up ");
        }
        logger.info("setOutPatientOver end ");
        Set set = new HashSet();
    }
    public  String  singelCancel(String outpatientId){
        logger.info("进入结束超时未接诊的门诊");
        WlyyOutpatientDO wlyyOutpatientDO = outpatientDao.findById(outpatientId).orElse(null);
        String msgReturn = "";
        if (null!=wlyyOutpatientDO){
            logger.info("开始结束超时未接诊的门诊");
            String cancelValue = "超时订单系统自动取消";
            String cancelRemark = "当天未接诊的门诊订单自动取消";
            String outPatientId = wlyyOutpatientDO.getId();
            long currentTime = new Date().getTime();
            long outpatientTime = wlyyOutpatientDO.getCreateTime().getTime();
            if (currentTime-outpatientTime>24*60*60*1000) {
                logger.info("时间满足条件");
                //判断医生是否接诊
                List<WlyyHospitalWaitingRoomDO> roomDOs = hospitalWaitingRoomDao.findByOutpatientId(outPatientId);
                if (roomDOs != null && roomDOs.size() > 0) {
                    for (WlyyHospitalWaitingRoomDO roomDO : roomDOs) {
                        if (roomDO.getVisitStatus() == 2) {
                            logger.info("id为：" + outPatientId + "的门诊已被医生接诊");
                        } else {
                            roomDO.setVisitStatus(-1);
                            logger.info("修改候诊室状态，将候诊病人移除候诊室");
                            hospitalWaitingRoomDao.save(roomDO);
                        }
                    }
                }
                //更改门诊状态
                wlyyOutpatientDO.setStatus("-1");
                String description = null;
                wlyyOutpatientDO.setDoctorCancelRemark(cancelRemark);
                wlyyOutpatientDO.setDoctorCancelType("6");
                wlyyOutpatientDO.setDoctorCancelValue(cancelValue);
                wlyyOutpatientDO.setEndTime(new Date());
                wlyyOutpatientDO.setOperator("system");
                description = cancelValue;
                //系统消息
                SystemMessageDO messageDO = new SystemMessageDO();
                messageDO.setTitle("超时订单系统自动取消");
                messageDO.setType("13");
                messageDO.setSender(wlyyOutpatientDO.getDoctor());
                messageDO.setSenderName(wlyyOutpatientDO.getDoctorName());
                messageDO.setRelationCode(wlyyOutpatientDO.getId());
                messageDO.setReceiver(wlyyOutpatientDO.getPatient());
                messageDO.setReceiverName(wlyyOutpatientDO.getPatientName());
                net.sf.json.JSONObject data = new net.sf.json.JSONObject();
                try {
                    data.put("name", wlyyOutpatientDO.getPatientName());
                    data.put("age", IdCardUtil.getAgeForIdcard(wlyyOutpatientDO.getIdcard()));
                    data.put("consumerName",wlyyOutpatientDO.getConsumerName());
                    data.put("gender", IdCardUtil.getSexForIdcard(wlyyOutpatientDO.getIdcard()));
                    data.put("question", wlyyOutpatientDO.getIcd10Name());
                    String msg = "";
                    if ("1".equalsIgnoreCase(wlyyOutpatientDO.getType())) {
                        data.put("type", "9");
                        msg += wlyyOutpatientDO.getPatientName() + ",您好！ 您有一个图文复诊已被系统取消，取消原因：" + cancelValue + "。取消说明：" + cancelRemark + "。";
                        prescriptionService.sendWxTemplateMsg(wechatId, wlyyOutpatientDO.getId(), null, "9", "systemCancelRemind", cancelRemark);
                    }
                    if ("2".equalsIgnoreCase(wlyyOutpatientDO.getType())) {
                        data.put("type", "16");
                        msg += wlyyOutpatientDO.getPatientName() + ",您好！ 您有一个视频复诊已被系统取消，取消原因：" + cancelValue + "。取消说明：" + cancelRemark + "。";
                        prescriptionService.sendWxTemplateMsg(wechatId, wlyyOutpatientDO.getId(), null, "16", "systemCancelRemind", cancelRemark);
                    }
                    data.put("msg", msg);
                    messageDO.setData(data.toString());
                    systemMessageService.saveMessage(messageDO);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                //推送消息到眼科通
                BasePatientDO patient = basePatientDao.findById(wlyyOutpatientDO.getPatient()).orElse(null);
                if ("xm_ykyy_wx".equalsIgnoreCase(wechatId)) {
                    logger.info("推送消息到眼科通");
                    ykyyService.pushNotificationToYktPatient(patient.getYktId(), messageDO.getTitle(), data.get("msg").toString());
                }

                outpatientDao.save(wlyyOutpatientDO);
                logger.info("修改保存的门诊记录:" + wlyyOutpatientDO.getId());

                //删除门诊号源
                List<WlyyPatientRegisterTimeDO> list = patientRegisterTimeDao.findByOutpatientId(wlyyOutpatientDO.getId());
                if (list != null && list.size() > 0) {
                    patientRegisterTimeDao.deleteAll(list);
                }
                BusinessOrderDO businessOrderDO = businessOrderDao.selectByRelationCode(outPatientId);
                try {
                    if (wechatId.equalsIgnoreCase("xm_ykyy_wx")) {
                        if (businessOrderDO.getStatus()==1) {
                            businessOrderService.orderRefund(wechatId, wlyyOutpatientDO.getConsumer(), businessOrderDO.getOrderNo(), businessOrderDO.getPayPrice(), description);
                        }
                        //眼科接诊时更新眼科通状态
                        if (businessOrderDO != null) {
                            ykyyService.updateYktOrderStatus(businessOrderDO.getOrderNo(), "2");
                        }
                    } else if (wechatId.equalsIgnoreCase("xm_zsyy_wx")) {
                        WlyyHospitalSysDictDO hospitalSysDictDO =  hospitalSysDictDao.findById("isCloseRefund").orElse(null);
                        if (hospitalSysDictDO!=null&&hospitalSysDictDO.getDictValue().equalsIgnoreCase("1")){
                            try {
                                if (businessOrderDO.getStatus()==1){
                                    entranceService.hisRefund(outPatientId);
                                }

                            }catch (Exception e){
                                e.printStackTrace();
                            }
                        }

/*
                    businessOrderService.ylzOrderRefund(wechatId, wlyyOutpatientDO.getPatient(), businessOrderDO.getOrderNo(), businessOrderDO.getPayPrice(), description);
*/

                    }
                    msgReturn = "success";
                } catch (Exception e) {
                    e.printStackTrace();
                    msgReturn = "failed";
                }
            }
            logger.info(wlyyOutpatientDO.getId()+"的门诊记录设置为取消");
        }
        return msgReturn;
    }
    public String  cancelAll(String day){
        logger.info("进入结束超时未接诊的门诊");
        List<WlyyOutpatientDO> timeout = outpatientDao.findAllWaitingOutpatient();
        long days = 1;
        if (StringUtils.isNotBlank(day)){
            days = Integer.parseInt(day);
        }
        String msgReurn = "";
        if (null!=timeout&&timeout.size()>0){
            logger.info("开始结束超时未接诊的门诊");
            String cancelValue = "超时订单系统自动取消";
            String cancelRemark = "当天未接诊的门诊订单自动取消";
            for(WlyyOutpatientDO wlyyOutpatientDO:timeout){
                try {
                    long currentTime = new Date().getTime();
                    long outpatientTime = wlyyOutpatientDO.getCreateTime().getTime();
                    if (currentTime-outpatientTime>24*60*60*1000*days){
                        logger.info("时间满足条件");
                        String outPatientId = wlyyOutpatientDO.getId();
                        //判断医生是否接诊
                        List<WlyyHospitalWaitingRoomDO> roomDOs = hospitalWaitingRoomDao.findByOutpatientId(outPatientId);
                        if (roomDOs != null && roomDOs.size() > 0) {
                            for (WlyyHospitalWaitingRoomDO roomDO : roomDOs) {
                                if (roomDO.getVisitStatus() == 2) {
                                    logger.info("id为："+outPatientId+"的门诊已被医生接诊");
                                } else {
                                    roomDO.setVisitStatus(-1);
                                    logger.info("修改候诊室状态，将候诊病人移除候诊室");
                                    hospitalWaitingRoomDao.save(roomDO);
                                }
                            }
                        }
                        //更改门诊状态
                        wlyyOutpatientDO.setStatus("-1");
                        String description = null;
                        wlyyOutpatientDO.setDoctorCancelRemark(cancelRemark);
                        wlyyOutpatientDO.setDoctorCancelType("6");
                        wlyyOutpatientDO.setDoctorCancelValue(cancelValue);
                        wlyyOutpatientDO.setEndTime(new Date());
                        wlyyOutpatientDO.setOperator("system");
                        description = cancelValue;
                        //系统消息
                        SystemMessageDO messageDO = new SystemMessageDO();
                        messageDO.setTitle("超时订单系统自动取消");
                        messageDO.setType("13");
                        messageDO.setSender(wlyyOutpatientDO.getDoctor());
                        messageDO.setSenderName(wlyyOutpatientDO.getDoctorName());
                        messageDO.setRelationCode(wlyyOutpatientDO.getId());
                        messageDO.setReceiver(wlyyOutpatientDO.getPatient());
                        messageDO.setReceiverName(wlyyOutpatientDO.getPatientName());
                        net.sf.json.JSONObject data = new net.sf.json.JSONObject();
                        data.put("name", wlyyOutpatientDO.getPatientName());
                        data.put("consumerName",wlyyOutpatientDO.getConsumerName());
                        data.put("age", IdCardUtil.getAgeForIdcard(wlyyOutpatientDO.getIdcard()));
                        data.put("gender", IdCardUtil.getSexForIdcard(wlyyOutpatientDO.getIdcard()));
                        data.put("question", wlyyOutpatientDO.getIcd10Name());
                        String msg = "";
                        if ("1".equalsIgnoreCase(wlyyOutpatientDO.getType())) {
                            data.put("type", "9");
                            msg += wlyyOutpatientDO.getPatientName() + ",您好！ 您有一个图文复诊已被系统取消，取消原因：" + cancelValue + "。取消说明：" + cancelRemark + "。";
                            prescriptionService.sendWxTemplateMsg(wechatId, wlyyOutpatientDO.getId(),null, "9", "systemCancelRemind", cancelRemark);
                        }
                        if ("2".equalsIgnoreCase(wlyyOutpatientDO.getType())) {
                            data.put("type", "16");
                            msg += wlyyOutpatientDO.getPatientName() + ",您好！ 您有一个视频复诊已被系统取消，取消原因：" + cancelValue + "。取消说明：" + cancelRemark + "。";
                            prescriptionService.sendWxTemplateMsg(wechatId, wlyyOutpatientDO.getId(),null, "16", "systemCancelRemind", cancelRemark);
                        }
                        data.put("msg", msg);
                        messageDO.setData(data.toString());
                        systemMessageService.saveMessage(messageDO);
                        //推送消息到眼科通
                        BasePatientDO patient = basePatientDao.findById(wlyyOutpatientDO.getPatient()).orElse(null);
                        if ("xm_ykyy_wx".equalsIgnoreCase(wechatId)) {
                            logger.info("推送消息到眼科通");
                            ykyyService.pushNotificationToYktPatient(patient.getYktId(), messageDO.getTitle(), data.get("msg").toString());
                        }
                        logger.info("修改保存的门诊记录:"+wlyyOutpatientDO.getId());
                        //删除门诊号源
                        List<WlyyPatientRegisterTimeDO> list = patientRegisterTimeDao.findByOutpatientId(wlyyOutpatientDO.getId());
                        if (list != null && list.size() > 0) {
                            patientRegisterTimeDao.deleteAll(list);
                        }
                        BusinessOrderDO businessOrderDO = businessOrderDao.selectByRelationCode(outPatientId);
                        if (wechatId.equalsIgnoreCase("xm_ykyy_wx")) {
                            if (businessOrderDO.getStatus()==1) {
                                businessOrderService.orderRefund(wechatId, wlyyOutpatientDO.getConsumer(), businessOrderDO.getOrderNo(), businessOrderDO.getPayPrice(), description);
                            }//眼科接诊时更新眼科通状态
                            if (businessOrderDO!=null){
                                ykyyService.updateYktOrderStatus(businessOrderDO.getOrderNo(),"2");
                            }
                        } else if (wechatId.equalsIgnoreCase("xm_zsyy_wx")) {
                            WlyyHospitalSysDictDO hospitalSysDictDO =  hospitalSysDictDao.findById("isCloseRefund").orElse(null);
                            if (hospitalSysDictDO!=null&&hospitalSysDictDO.getDictValue().equalsIgnoreCase("1")){
                                try {
                                    if (businessOrderDO.getStatus()==1){
                                        entranceService.hisRefund(outPatientId);
                                    }
                                }catch (Exception e){
                                    e.printStackTrace();
                                }
                            }
        /*
                            businessOrderService.ylzOrderRefund(wechatId, wlyyOutpatientDO.getPatient(), businessOrderDO.getOrderNo(), businessOrderDO.getPayPrice(), description);
        */
                        }
                        outpatientDao.save(wlyyOutpatientDO);
                        logger.info(wlyyOutpatientDO.getId()+"的门诊记录设置为取消");
                        msgReurn="success";
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    logger.info(wlyyOutpatientDO.getId()+"的门诊记录取消失败");
                    msgReurn="faild";
                }
            }
        }
        return msgReurn;
    }

    public void cancelRecieve(){
        //将所有的已接诊的处方记录
        List<WlyyOutpatientDO> outpatientDOs = outpatientDao.findByStatus();
        if(outpatientDOs!=null&&outpatientDOs.size()>0){
            System.out.println(outpatientDOs.get(0).getId()+"id______-");
            for(WlyyOutpatientDO outpatientDO:outpatientDOs){
                //结束门诊
                outpatientDO.setStatus("3");
                outpatientDO.setEndTime(new Date());
                String consultCode ="";
                try {
                    consultCode = imService.getConsultCodeByOutpatientId(outpatientDO.getId());
                    logger.info("consultCode"+consultCode);
                    if(StringUtils.isNoneBlank(consultCode)){
                        imService.finish(consultCode,"admin",2);
                    }
                }catch(Exception e)  {
                    e.printStackTrace();
                    logger.info("自动结束咨询报错 consultCode： "+consultCode);
                }
                logger.info(outpatientDO.getId()+"的处方记录设置为结束");
            }
            outpatientDao.saveAll(outpatientDOs);
            //退费
            for (WlyyOutpatientDO outpatientDO:outpatientDOs){
                try {
                    BusinessOrderDO businessOrderDO = businessOrderDao.selectByRelationCode(outpatientDO.getId());
                    if (businessOrderDO!=null){
                        if (businessOrderDO.getPayType()==1){
                            businessOrderService.orderRefund(wechatId,businessOrderDO.getPatient(),businessOrderDO.getOrderNo(),businessOrderDO.getPayPrice(),businessOrderDO.getDescription());
                        }else if (businessOrderDO.getPayType()==3){
        /*
                            businessOrderService.ylzOrderRefund(wechatId,businessOrderDO.getPatient(),businessOrderDO.getOrderNo(),businessOrderDO.getPayPrice(),businessOrderDO.getDescription());
        */
                        }
                    } } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            logger.info("setOutPatientOver count :"+outpatientDOs.size());
        }else {
            logger.info("setOutPatientOver count :"+0);
        }
        logger.info("setOutPatientOver end ");
    }
    public void cancelRecieveSingle(String outpatientId){
        //将所有的已接诊的处方记录
        WlyyOutpatientDO outpatientDO = outpatientDao.findById(outpatientId).orElse(null);
        if(outpatientDO!=null){
            System.out.println(outpatientDO.getId()+"id______-");
            //结束门诊
            outpatientDO.setStatus("3");
            outpatientDO.setEndTime(new Date());
            String consultCode ="";
            try {
                consultCode = imService.getConsultCodeByOutpatientId(outpatientDO.getId());
                logger.info("consultCode"+consultCode);
                if(StringUtils.isNoneBlank(consultCode)){
                    imService.finish(consultCode,"admin",2);
                }
            }catch(Exception e)  {
                e.printStackTrace();
                logger.info("自动结束咨询报错 consultCode： "+consultCode);
            }
            logger.info(outpatientDO.getId()+"的处方记录设置为结束");
            outpatientDao.save(outpatientDO);
            //退费
           /* try {
                BusinessOrderDO businessOrderDO = businessOrderDao.selectByRelationCode(outpatientDO.getId());
                if (businessOrderDO!=null){
                    if (businessOrderDO.getPayType()==1){
                        businessOrderService.orderRefund(wechatId,businessOrderDO.getPatient(),businessOrderDO.getOrderNo(),businessOrderDO.getPayPrice(),businessOrderDO.getDescription());
                    }else if (businessOrderDO.getPayType()==3){

                        businessOrderService.ylzOrderRefund(wechatId,businessOrderDO.getPatient(),businessOrderDO.getOrderNo(),businessOrderDO.getPayPrice(),businessOrderDO.getDescription());

                    }
                } } catch (Exception e) {
                e.printStackTrace();
            }*/
        }else {
            logger.info("setOutPatientOver count :"+0);
        }
        logger.info("setOutPatientOver end ");
    }



    public String  cancelAllandSendMessage(String day){
        logger.info("进入结束超时未接诊的门诊");
        List<WlyyOutpatientDO> timeout = outpatientDao.findAllWaitingOutpatient();
        long days = 1;
        if (StringUtils.isNotBlank(day)){
            days = Integer.parseInt(day);
        }
        String msgReurn = "";
        if (null!=timeout&&timeout.size()>0){
            logger.info("开始结束超时未接诊的门诊");
            String cancelValue = "超时订单系统自动取消";
            String cancelRemark = "当天未接诊的门诊订单自动取消";
            for(WlyyOutpatientDO wlyyOutpatientDO:timeout){
                try {
                    long currentTime = new Date().getTime();
                    long outpatientTime = wlyyOutpatientDO.getCreateTime().getTime();
                    if (currentTime-outpatientTime>24*60*60*1000*days){
                        logger.info("时间满足条件");
                        String outPatientId = wlyyOutpatientDO.getId();
                        //判断医生是否接诊
                        List<WlyyHospitalWaitingRoomDO> roomDOs = hospitalWaitingRoomDao.findByOutpatientId(outPatientId);
                        if (roomDOs != null && roomDOs.size() > 0) {
                            for (WlyyHospitalWaitingRoomDO roomDO : roomDOs) {
                                if (roomDO.getVisitStatus() == 2) {
                                    logger.info("id为："+outPatientId+"的门诊已被医生接诊");
                                } else {
                                    roomDO.setVisitStatus(-1);
                                    logger.info("修改候诊室状态，将候诊病人移除候诊室");
                                    hospitalWaitingRoomDao.save(roomDO);
                                }
                            }
                        }
                        //更改门诊状态
                        wlyyOutpatientDO.setStatus("-1");
                        String description = null;
                        wlyyOutpatientDO.setDoctorCancelRemark(cancelRemark);
                        wlyyOutpatientDO.setDoctorCancelType("6");
                        wlyyOutpatientDO.setDoctorCancelValue(cancelValue);
                        wlyyOutpatientDO.setEndTime(new Date());
                        wlyyOutpatientDO.setOperator("system");
                        description = cancelValue;
                        //系统消息
                        SystemMessageDO messageDO = new SystemMessageDO();
                        messageDO.setTitle("超时订单系统自动取消");
                        messageDO.setType("13");
                        messageDO.setSender(wlyyOutpatientDO.getDoctor());
                        messageDO.setSenderName(wlyyOutpatientDO.getDoctorName());
                        messageDO.setRelationCode(wlyyOutpatientDO.getId());
                        messageDO.setReceiver(wlyyOutpatientDO.getPatient());
                        messageDO.setReceiverName(wlyyOutpatientDO.getPatientName());
                        net.sf.json.JSONObject data = new net.sf.json.JSONObject();
                        data.put("name", wlyyOutpatientDO.getPatientName());
                        data.put("consumerName",wlyyOutpatientDO.getConsumerName());
                        data.put("age", IdCardUtil.getAgeForIdcard(wlyyOutpatientDO.getIdcard()));
                        data.put("gender", IdCardUtil.getSexForIdcard(wlyyOutpatientDO.getIdcard()));
                        data.put("question", wlyyOutpatientDO.getIcd10Name());
                        String msg = "";
                        if ("1".equalsIgnoreCase(wlyyOutpatientDO.getType())) {
                            data.put("type", "9");
                            msg += wlyyOutpatientDO.getPatientName() + ",您好！ 您有一个图文复诊已被系统取消，取消原因：" + cancelValue + "。取消说明：" + cancelRemark + "。";
                            //prescriptionService.sendWxTemplateMsg(wechatId, wlyyOutpatientDO.getId(),null, "9", "systemCancelRemind", cancelRemark);
                        }
                        if ("2".equalsIgnoreCase(wlyyOutpatientDO.getType())) {
                            data.put("type", "16");
                            msg += wlyyOutpatientDO.getPatientName() + ",您好！ 您有一个视频复诊已被系统取消，取消原因：" + cancelValue + "。取消说明：" + cancelRemark + "。";
                            // prescriptionService.sendWxTemplateMsg(wechatId, wlyyOutpatientDO.getId(),null, "16", "systemCancelRemind", cancelRemark);
                        }
                        data.put("msg", msg);
                        messageDO.setData(data.toString());
                        systemMessageService.saveMessage(messageDO);
                        //推送消息到眼科通
                        BasePatientDO patient = basePatientDao.findById(wlyyOutpatientDO.getPatient()).orElse(null);
                        if ("xm_ykyy_wx".equalsIgnoreCase(wechatId)) {
                            logger.info("推送消息到眼科通");
                            ykyyService.pushNotificationToYktPatient(patient.getYktId(), messageDO.getTitle(), data.get("msg").toString());
                        }
                        logger.info("修改保存的门诊记录:"+wlyyOutpatientDO.getId());
                        //删除门诊号源
                        List<WlyyPatientRegisterTimeDO> list = patientRegisterTimeDao.findByOutpatientId(wlyyOutpatientDO.getId());
                        if (list != null && list.size() > 0) {
                            patientRegisterTimeDao.deleteAll(list);
                        }
                        BusinessOrderDO businessOrderDO = businessOrderDao.selectByRelationCode(outPatientId);
                        if (wechatId.equalsIgnoreCase("xm_ykyy_wx")) {
                            if (businessOrderDO.getStatus()==1){
                                businessOrderService.orderRefund(wechatId, wlyyOutpatientDO.getConsumer(), businessOrderDO.getOrderNo(), businessOrderDO.getPayPrice(), description);
                            }
                            //眼科接诊时更新眼科通状态
                            if (businessOrderDO!=null){
                                ykyyService.updateYktOrderStatus(businessOrderDO.getOrderNo(),"2");
                            }
                        } else if (wechatId.equalsIgnoreCase("xm_zsyy_wx")) {
                            WlyyHospitalSysDictDO hospitalSysDictDO =  hospitalSysDictDao.findById("isCloseRefund").orElse(null);
                            if (hospitalSysDictDO!=null&&hospitalSysDictDO.getDictValue().equalsIgnoreCase("1")){
                                try {
                                    if (businessOrderDO.getStatus()==1){
                                        entranceService.hisRefund(outPatientId);
                                    }
                                }catch (Exception e){
                                    e.printStackTrace();
                                }

                            }
        /*
                            businessOrderService.ylzOrderRefund(wechatId, wlyyOutpatientDO.getPatient(), businessOrderDO.getOrderNo(), businessOrderDO.getPayPrice(), description);
        */
                        }
                        outpatientDao.save(wlyyOutpatientDO);
                        logger.info(wlyyOutpatientDO.getId()+"的门诊记录设置为取消");
                        msgReurn="success";
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    logger.info(wlyyOutpatientDO.getId()+"的门诊记录取消失败");
                    msgReurn="faild";
                }
            }
        }
        return msgReurn;
    }


    public void unCheckPrescription() throws Exception {
        String startTime= DateUtil.getStringDateShort() +" 00:00:00";
        String endTime= DateUtil.getStringDateShort() +" 23:59:59";
        JSONArray array = ykyyEntranceService.getUnCheckPrescription(startTime,endTime);
        String sendMesControl = "1";//1 开启发送短信 0 关闭
        WlyyHospitalSysDictDO wlyyHospitalSysDictDO = hospitalSysDictDao.findById("ykSmsControl").orElse(null);
        WlyyHospitalSysDictDO yunweiSysDict = hospitalSysDictDao.findById("yunweiMobile").orElse(null);
        String remind = yunweiSysDict.getDictValue();
        String mobiles = yunweiSysDict.getDictCode();
        if (wlyyHospitalSysDictDO!=null){
            sendMesControl = wlyyHospitalSysDictDO.getDictValue();
        }
        if(array!=null&&array.size()>0) {
            for (int i = 0; i < array.size(); i++) {
                JSONObject json = array.getJSONObject(i);
                String brid = json.getString("BRID");
                String cfsb = json.getString("CFSB");

                //查找居民信息
                BasePatientDO patientDO = ykyyEntranceService.findPatientByMapingCode(brid);
                if (patientDO != null) {
                    //查找对应的处方
                    String sql = "SELECT w.OUTPATIENT_ID,w.ID,w.DOCTOR_NAME,to_char(w.CREATE_TIME ,'yyyy/MM/dd hh24:mi:ss' ) as CREATE_TIME from WLYY_PRESCRIPTION w WHERE REAL_ORDER = '" + cfsb + "' ORDER BY w.CREATE_TIME desc";
                    List<Map<String, Object>> pre = hibenateUtils.createSQLQuery(sql);
                    if(pre!=null && pre.size()>0){
                        String outpatientId = pre.get(0).get("OUTPATIENT_ID").toString();
                        String createTime =pre.get(0).get("CREATE_TIME").toString();
                        Date createDate =  DateUtil.strToDateLong(createTime);
                        Calendar cal= Calendar.getInstance();
                        cal.setTime(createDate);
                        cal.add(Calendar.MINUTE,Integer.parseInt(remind));
                        Long now = new Date().getTime();
                        Long min = cal.getTime().getTime();
                        WlyyOutpatientDO wlyyOutpatientDO = outpatientDao.findById(outpatientId).orElse(null);
                        if (min<=now){
                            String mobileList[] = mobiles.split(",");
                            if ("1".equalsIgnoreCase(sendMesControl)){
                                for (int j=0;j<mobileList.length;j++){
                                    WxPushLogDO wxPushLogDO = new WxPushLogDO();
                                    wxPushLogDO.setTempName("未审核提醒");
                                    wxPushLogDO.setOpenid(wlyyOutpatientDO.getId());
                                    wxPushLogDO.setReceiver(wlyyOutpatientDO.getPatient());
                                    wxPushLogDO.setCreateTime(new Date());
                                    wxPushLogDao.save(wxPushLogDO);
                                    ykyySMSService.ykyySendSMS(mobileList[j],wlyyOutpatientDO.getDoctorName()+"已为患者:开具处方，请尽快通知相关人员处理。");
                                }

                            }
                        }

                    }
                }
            }
        }
    }

    public String finishOldConsult(String consult){

        int resutl = imService.finish(consult,"admin",2);
        //医生未回复可以退费
        try {
            businessOrderService.consultRefund(consult,wechatId);
        }catch (Exception e){
            e.printStackTrace();
            return  "failed";
        }
        return "success";
    }
}
