package com.yihu.jw.service.channel;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.entity.hospital.DmHospitalDO;
import com.yihu.jw.entity.hospital.consult.WlyyHospitalSysDictDO;
import com.yihu.jw.entity.hospital.team.WlyyAdminTeamDO;
import com.yihu.jw.entity.hospital.team.WlyyAdminTeamMemberDO;
import com.yihu.jw.hospital.HospitalDao;
import com.yihu.jw.hospital.dict.WlyyHospitalSysDictDao;
import com.yihu.jw.hospital.team.dao.WlyyAdminTeamDao;
import com.yihu.jw.hospital.team.dao.WlyyAdminTeamMemberDao;
import com.yihu.jw.wlyy.wlyyhttp.WlyyHttpService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

@Service
public class SyncWlyyHospitalService {
    private static final Logger logger = LoggerFactory.getLogger(SyncWlyyHospitalService.class);

    @Autowired
    private WlyyHttpService wlyyHttpService;
    @Autowired
    private HospitalDao hospitalDao;
    @Autowired
    private WlyyAdminTeamDao wlyyAdminTeamDao;
    @Autowired
    private WlyyAdminTeamMemberDao wlyyAdminTeamMemberDao;
    @Autowired
    private WlyyHospitalSysDictDao wlyyHospitalSysDictDao;

    public void syncHospital(String townCode){
        String param="";
        if (StringUtils.isNoneBlank(townCode)){
            param="?townCode="+townCode;
        }
        JSONObject re = wlyyHttpService.sendWlyyMesGet("findHospital",param);
        if(re!=null){
            JSONArray hospitals = JSONArray.parseArray(re.getString("data"));
            logger.info("hospitals size:"+hospitals.size());
            if(hospitals!=null&&hospitals.size()>0){
                Integer addCount = 0;
                Integer updateCount =0;
                for(int i=0;i<hospitals.size();i++){
                    JSONObject h = (JSONObject) hospitals.get(i);
                    String code = h.getString("code");
                    if(!hospitalDao.existsByCode(code)){
                        DmHospitalDO org = new DmHospitalDO();
                        org.setCode(code);
                        org.setName(h.getString("name"));
                        org.setProvince(h.getString("province"));
                        org.setProvinceName(h.getString("provinceName"));
                        org.setCity(h.getString("city"));
                        org.setCityName(h.getString("cityName"));
                        org.setTown(h.getString("town"));
                        org.setTownName(h.getString("townName"));
                        org.setLevel(h.getInteger("level"));
                        org.setDel("1");
                        hospitalDao.save(org);
                        addCount++;
                    }else{
                        DmHospitalDO org = hospitalDao.findByCode(code);
                        org.setName(h.getString("name"));
                        org.setProvince(h.getString("province"));
                        org.setProvinceName(h.getString("provinceName"));
                        org.setCity(h.getString("city"));
                        org.setCityName(h.getString("cityName"));
                        org.setTown(h.getString("town"));
                        org.setTownName(h.getString("townName"));
                        org.setLevel(h.getInteger("level"));
                        hospitalDao.save(org);
                        updateCount++;
                    }
                }
                logger.info("hospitals addCount:"+addCount+" updateCount:"+updateCount);
            }
        }
    }


    public void syncTeam(String orgCode){

        String param="";
        if(StringUtils.isNotBlank(orgCode)){
            param="?orgCode="+orgCode;
        }
        JSONObject re = wlyyHttpService.sendWlyyMesGet("findHospitalTeam",param);
        if(re!=null){
            JSONArray teamlisf = JSONArray.parseArray(re.getString("data"));
            logger.info("team size:"+teamlisf.size());
            if(teamlisf!=null&&teamlisf.size()>0){
                Integer addCount = 0;
                Integer updateCount =0;
                for(int i=0;i<teamlisf.size();i++){
                    JSONObject h = (JSONObject) teamlisf.get(i);
                    if ("true".equalsIgnoreCase(h.getString("available"))){
                        h.put("available",1);
                    }else {
                        h.put("available",0);
                    }
                    WlyyAdminTeamDO wlyyAdminTeam =JSON.toJavaObject(h,WlyyAdminTeamDO.class);
                    WlyyAdminTeamDO wlyyAdminTeamDO=wlyyAdminTeamDao.findById(Integer.parseInt(wlyyAdminTeam.getId()+"")).orElse(null);
                    if(wlyyAdminTeamDO==null){
                        addCount++;
                    }else{
                        updateCount++;
                    }
                    wlyyAdminTeamDao.save(wlyyAdminTeam);
                }
                logger.info("team addCount:"+addCount+" updateCount:"+updateCount);
            }
        }
    }


    public void syncTeamMember(String teamId){
        List<WlyyAdminTeamDO> list=wlyyAdminTeamDao.findTeamAndAvailable();
        String wlyyDomainUrl="";
        WlyyHospitalSysDictDO wlyyHospitalSysDictDO = wlyyHospitalSysDictDao.findById("wlyyDomainUrl").orElse(null);
        if (wlyyHospitalSysDictDO!=null){
            wlyyDomainUrl=wlyyHospitalSysDictDO.getDictValue();
        }
        for (WlyyAdminTeamDO wlyyAdminTeamDO:list){
            String param="";
            if (StringUtils.isNoneBlank(teamId)){
                param="?teamId="+teamId;
            }else if(null!=wlyyAdminTeamDO.getId()){
                param="?teamId="+wlyyAdminTeamDO.getId();
            }
            JSONObject re = wlyyHttpService.sendWlyyMesGet("findTeamMembers",param);
            if(re!=null){
                JSONArray teamlisf = JSONArray.parseArray(re.getString("data"));
                logger.info("member size:"+teamlisf.size());
                if(teamlisf!=null&&teamlisf.size()>0){
                    Integer addCount = 0;
                    Integer updateCount =0;
                    for(int i=0;i<teamlisf.size();i++){
                        JSONObject h = (JSONObject) teamlisf.get(i);
                        if ("true".equalsIgnoreCase(h.getString("available"))){
                            h.put("available",1);
                        }else {
                            h.put("available",0);
                        }
                        WlyyAdminTeamMemberDO wlyyAdminTeamMember =JSON.toJavaObject(h,WlyyAdminTeamMemberDO.class);
                        if (StringUtils.isNoneBlank(wlyyAdminTeamMember.getPhoto())&&wlyyAdminTeamMember.getPhoto().contains("group")){
                            wlyyAdminTeamMember.setPhoto(wlyyDomainUrl+wlyyAdminTeamMember.getPhoto());
                        }
                        WlyyAdminTeamMemberDO wlyyAdminTeamMemberDO=wlyyAdminTeamMemberDao.findById(wlyyAdminTeamMember.getId()).orElse(null);
                        if(wlyyAdminTeamMemberDO==null){
                            wlyyAdminTeamMember.setLastModified(new Date());
                            addCount++;
                        }else{
                            wlyyAdminTeamMember.setLastModified(new Date());
                            updateCount++;
                        }
                        wlyyAdminTeamMemberDao.save(wlyyAdminTeamMember);
                    }
                    logger.info("team addCount:"+addCount+" updateCount:"+updateCount);
                }
            }
            if (StringUtils.isNoneBlank(teamId)){
                break;
            }
        }

    }

    public void syncTeamMemberByTime(String nowTime){
        String param="?modifyTime="+nowTime;
        JSONObject re = wlyyHttpService.sendWlyyMesGet("findTeamMembersByModifyTime",param);
        String wlyyDomainUrl="";
        WlyyHospitalSysDictDO wlyyHospitalSysDictDO = wlyyHospitalSysDictDao.findById("wlyyDomainUrl").orElse(null);
        if (wlyyHospitalSysDictDO!=null){
            wlyyDomainUrl=wlyyHospitalSysDictDO.getDictValue();
        }
        if(re!=null){
            JSONArray teamlist = JSONArray.parseArray(re.getString("data"));
            logger.info("member size:"+teamlist.size());
            if(teamlist!=null&&teamlist.size()>0){
                Integer addCount = 0;
                Integer updateCount =0;
                for(int i=0;i<teamlist.size();i++){
                    JSONObject h = (JSONObject) teamlist.get(i);
                    if ("true".equalsIgnoreCase(h.getString("available"))){
                        h.put("available",1);
                    }else {
                        h.put("available",0);
                    }
                    WlyyAdminTeamMemberDO wlyyAdminTeamMember =JSON.toJavaObject(h,WlyyAdminTeamMemberDO.class);
                    if (StringUtils.isNoneBlank(wlyyAdminTeamMember.getPhoto())){
                        wlyyAdminTeamMember.setPhoto(wlyyDomainUrl+wlyyAdminTeamMember.getPhoto());
                    }
                    WlyyAdminTeamMemberDO wlyyAdminTeamMemberDO=wlyyAdminTeamMemberDao.findById(wlyyAdminTeamMember.getId()).orElse(null);
                    if(wlyyAdminTeamMemberDO==null){
                        addCount++;
                    }else{
                        updateCount++;
                    }
                    wlyyAdminTeamMemberDao.save(wlyyAdminTeamMember);
                }
                logger.info("team addCount:"+addCount+" updateCount:"+updateCount);
            }
        }

    }

}
