package com.yihu.jw.web.quota;

import com.yihu.jw.entity.hospital.prescription.WlyyOutpatientDO;
import com.yihu.jw.hospital.dict.WlyyHospitalSysDictDao;
import com.yihu.jw.hospital.prescription.service.YkyyPrescriptionService;
import com.yihu.jw.im.service.ImService;
import com.yihu.jw.internet.service.DataGeneratorService;
import com.yihu.jw.internet.service.InternetCommonService;
import com.yihu.jw.internet.service.InternetService;
import com.yihu.jw.internet.service.YkyyCommonService;
import com.yihu.jw.internet.service.ykyy.YkyyInternetService;
import com.yihu.jw.job.*;
import com.yihu.jw.job.dailyReport.DailyReportRemindJob;
import com.yihu.jw.job.dailyReport.DailyReportRemindSecondJob;
import com.yihu.jw.job.dailyReport.DailyReportTotalRemindJob;
import com.yihu.jw.job.dailyReport.DailyReportWsbTotalRemindJob;
import com.yihu.jw.job.yk.YKYYDataUploadJob;
import com.yihu.jw.job.ykyy.UnCheckPrescriptionJob;
import com.yihu.jw.job.ykyy.UnSettledHISPrescriptionJob;
import com.yihu.jw.job.ykyy.UpdateStatusByPayTimeJob;
import com.yihu.jw.job.ykyy.YkyyDataUploadJob;
import com.yihu.jw.repository.job.QuartzJobConfigDao;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.service.channel.*;
import com.yihu.jw.service.quota.JobService;
import com.yihu.jw.util.AutoTimeOutRemind;
import com.yihu.jw.util.OverdueJob;
import com.yihu.jw.util.SystemConf;
import com.yihu.jw.util.TimeoutOverDueJob;
import com.yihu.jw.web.BaseController;
import com.yihu.jw.wechat.service.WxTemplateService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;

/**
 * 任务启动
 *
 * @author chenweida
 */
@RestController
@RequestMapping(value = "/job", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(description = "后台-任务控制")
public class JobController extends BaseController {

    private org.slf4j.Logger logger = LoggerFactory.getLogger(JobController.class);

    private final JobService jobService;
    private final QuartzHelper quartzHelper;
    @Autowired
    private ImService imService;

    @Autowired
    private InternetService internetService;

    @Autowired
    private DataGeneratorService dataGeneratorService;

    @Autowired
    private YkyyInternetService ykyyInternetService;

    @Autowired
    private YkyyCommonService ykyyCommonService;

    @Autowired
    private InternetCommonService internetCommonService;
    @Autowired
    private WxTemplateService wxTemplateService;
    @Autowired
    private QuartzJobConfigDao wlyyJobConfigDao;
    @Autowired
    private WlyyHospitalSysDictDao wlyyHospitalSysDictDao;
    @Autowired
    private YkyyPrescriptionService ykyyPrescriptionService;
    @Autowired
    private PrescriptionStatusUpdateService prescriptionStatusUpdateService;
    @Autowired
    private RemindDoctorService remindDoctorService;
    @Autowired
    private SyncWlyyHospitalService syncWlyyHospitalService;
    @Autowired
    private HealthUploadService healthUploadService;
    @Autowired
    private TimeoutOverDueService timeoutOverDueService;

    @Autowired
    public JobController(JobService jobService, QuartzHelper quartzHelper) {
        this.jobService = jobService;
        this.quartzHelper = quartzHelper;
    }

    /**
     * 测试方法
     * @return
     */
    @RequestMapping(value = "unSettledHISPrescription", method = RequestMethod.GET)
    public String unSettledHISPrescription() {
        try {
            wxTemplateService.unSettledHISPrescription();
            return success("成功！");
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "失败:" + e.getMessage());
        }
    }


    @RequestMapping(value = "removeJob", method = RequestMethod.GET)
    public String removeJob(String taskId) {
        try {
            if(quartzHelper.isExistJob(taskId)){
                quartzHelper.removeJob(taskId);
                return success("删除成功！");
            }
            return success("删除成功！");
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "删除失败:" + e.getMessage());
        }
    }

    @RequestMapping(value = "getCalendar", method = RequestMethod.GET)
    public ObjEnvelop getCalendar() throws Exception{
       return success(quartzHelper.getCalendar());
    }



    @RequestMapping(value = "isExist", method = RequestMethod.GET)
    public String isExist(String taskId) {
        try {
            if(quartzHelper.isExistJob(taskId)){
                return success("job已经存在！");
            }
            return success("job不存在！");

        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "执行失败:" + e.getMessage());
        }
    }

    @RequestMapping(value = "reStartById", method = RequestMethod.GET)
    public String reStartById(String taskId) {
        try {

            if(quartzHelper.isExistJob(taskId)){
                quartzHelper.removeJob(taskId);
            }
            switch(taskId){
                case "prescription_overdue_job":
                    if (!quartzHelper.isExistJob("prescription_overdue_job")) {
                        String trigger = SystemConf.getInstance().getSystemProperties().getProperty("prescription_overdue_job");
                        quartzHelper.addJob(PrescriptionOverdueJob.class, trigger, "prescription_overdue_job", new HashMap<String, Object>());
                        logger.info("prescription_overdue_job  job success");
                    } else {
                        logger.info("prescription_overdue_job  job exist");
                    }
                    break;
                case "data_ykupload_job" :
                    //互联网医院 监管平台上报
                    if (!quartzHelper.isExistJob("data_ykupload_job")) {
                        String trigger = SystemConf.getInstance().getSystemProperties().getProperty("data_ykupload_job");
                        quartzHelper.addJob(YkyyDataUploadJob.class, trigger, "data_ykupload_job", new HashMap<String, Object>());
                        logger.info("data_ykupload_job  job success");
                    } else {
                        logger.info("data_ykupload_job  job exist");
                    }

                    break;
                case "data_common_upload_job" :
                    //互联网医院 监管平台上报
                    if (!quartzHelper.isExistJob("data_common_upload_job")) {
                        String trigger = SystemConf.getInstance().getSystemProperties().getProperty("data_common_upload_job");
                        quartzHelper.addJob(CommonDataUploadJob.class, trigger, "data_common_upload_job", new HashMap<String, Object>());
                        logger.info("data_common_upload_job  job success");
                    } else {
                        logger.info("data_common_upload_job  job exist");
                    }

                    break;
                case "unsettled_prescription_notice_job" :
                    //互联网医院 待结算消息推送
                    if (!quartzHelper.isExistJob("unsettled_prescription_notice_job")) {
                        String trigger = SystemConf.getInstance().getSystemProperties().getProperty("unsettled_prescription_notice_job");
                        quartzHelper.addJob(UnSettledHISPrescriptionJob.class, trigger, "unsettled_prescription_notice_job", new HashMap<String, Object>());
                        logger.info("unsettled_prescription_notice_job  job success");
                    } else {
                        logger.info("unsettled_prescription_notice_job  job exist");
                    }

                    break;
                case "uncheck_prescription_notice_job" :
                    //互联网医院 待结算消息推送
                    if (!quartzHelper.isExistJob("uncheck_prescription_notice_job")) {
                        String trigger = SystemConf.getInstance().getSystemProperties().getProperty("uncheck_prescription_notice_job");
                        quartzHelper.addJob(UnCheckPrescriptionJob.class, trigger, "uncheck_prescription_notice_job", new HashMap<String, Object>());
                        logger.info("uncheck_prescription_notice_job  job success");
                    } else {
                        logger.info("uncheck_prescription_notice_job  job exist");
                    }

                    break;
                case "pay_status_notice_job" :
                    //互联网医院 待结算消息推送
                    if (!quartzHelper.isExistJob("pay_status_notice_job")) {
                        String trigger = SystemConf.getInstance().getSystemProperties().getProperty("pay_status_notice_job");
                        quartzHelper.addJob(UpdateStatusByPayTimeJob.class, trigger, "pay_status_notice_job", new HashMap<String, Object>());
                        logger.info("pay_status_notice_job  job success");
                    } else {
                        logger.info("pay_status_notice_job  job exist");
                    }
                    break;
                case "send_health_upload_job" :
                    //马銮湾健康上报未填写推送
                    if (!quartzHelper.isExistJob("send_health_upload_job")) {
                        String trigger = SystemConf.getInstance().getSystemProperties().getProperty("send_health_upload_job");
                        quartzHelper.addJob(SendDoctorHealthUploadMessageJob.class, trigger, "send_health_upload_job", new HashMap<String, Object>());
                        logger.info("send_health_upload_job  job success");
                    } else {
                        logger.info("send_health_upload_job  job exist");
                    }
                    break;
                case "health_upload_remind_job" :
                    //健康上报未填写通知
                    if (!quartzHelper.isExistJob("health_upload_remind_job")) {
                        String trigger = SystemConf.getInstance().getSystemProperties().getProperty("health_upload_remind_job");
                        quartzHelper.addJob(HealthUploadRemindMessageJob.class, trigger, "health_upload_remind_job", new HashMap<String, Object>());
                        logger.info("health_upload_remind_job  job success");
                    } else {
                        logger.info("health_upload_remind_job  job exist");
                    }
                    break;
                case "second_health_upload_remind_job" :
                    //健康上报未填写二次通知
                    if (!quartzHelper.isExistJob("second_health_upload_remind_job")) {
                        String trigger = SystemConf.getInstance().getSystemProperties().getProperty("second_health_upload_remind_job");
                        quartzHelper.addJob(HealthUploadSecondRemindMessageJob.class, trigger, "second_health_upload_remind_job", new HashMap<String, Object>());
                        logger.info("second_health_upload_remind_job  job success");
                    } else {
                        logger.info("second_health_upload_remind_job  job exist");
                    }
                    break;
                case "health_upload_weishangbao_remind_job" :
                    //健康上报未上报汇总
                    if (!quartzHelper.isExistJob("health_upload_weishangbao_remind_job")) {
                        String trigger = SystemConf.getInstance().getSystemProperties().getProperty("health_upload_weishangbao_remind_job");
                        quartzHelper.addJob(HealthUploadWeishangbaoRemindMessageJob.class, trigger, "health_upload_weishangbao_remind_job", new HashMap<String, Object>());
                        logger.info("health_upload_weishangbao_remind_job  job success");
                    } else {
                        logger.info("health_upload_weishangbao_remind_job  job exist");
                    }
                    break;
                case "health_upload_total_remind_job" :
                    //健康上报上报汇总
                    if (!quartzHelper.isExistJob("health_upload_total_remind_job")) {
                        String trigger = SystemConf.getInstance().getSystemProperties().getProperty("health_upload_total_remind_job");
                        quartzHelper.addJob(HealthUploadTotalRemindMessageJob.class, trigger, "health_upload_total_remind_job", new HashMap<String, Object>());
                        logger.info("health_upload_total_remind_job  job success");
                    } else {
                        logger.info("health_upload_total_remind_job  job exist");
                    }
                    break;
                case "send_health_total_upload_job" :
                    //马銮湾健康上报上报汇总
                    if (!quartzHelper.isExistJob("send_health_total_upload_job")) {
                        String trigger = SystemConf.getInstance().getSystemProperties().getProperty("send_health_total_upload_job");
                        quartzHelper.addJob(HealthUploadTotalRemindMessageJob.class, trigger, "send_health_total_upload_job", new HashMap<String, Object>());
                        logger.info("send_health_total_upload_job  job success");
                    } else {
                        logger.info("send_health_total_upload_job  job exist");
                    }
                    break;
                case "syn_doctor_info_job" :
                    //互联网医院 医生信息同步
                    if (!quartzHelper.isExistJob("syn_doctor_info_job")) {
                        String trigger = SystemConf.getInstance().getSystemProperties().getProperty("syn_doctor_info_job");
                        quartzHelper.addJob(SynDoctorInfoJob.class, trigger, "syn_doctor_info_job", new HashMap<String, Object>());
                        logger.info("SynDoctorInfoJob  job success");
                    } else {
                        logger.info("SynDoctorInfoJob  job exist");
                    }
                    break;
                case "CSTXJOB" :
                    if (!quartzHelper.isExistJob("CSTXJOB")) {
                        System.out.println("id"+taskId);
                        String trigger = SystemConf.getInstance().getSystemProperties().getProperty("CSTXJOB");
                        System.out.println(trigger);
                        quartzHelper.addJob(DoctorTimeOutTipsJob.class, trigger, "CSTXJOB", new HashMap<String, Object>());
                        logger.info("CSTXJOB  job success");
                    } else {
                        logger.info("CSTXJOB  job exist");
                    }

                    break;
                case "prescription_auto_delete_job" :
                    //互联网医院 监管平台上报
                    if (!quartzHelper.isExistJob("prescription_auto_delete_job")) {
                        String trigger = SystemConf.getInstance().getSystemProperties().getProperty("prescription_auto_delete_job");
                        quartzHelper.addJob(PrescriptionAutoDeleteJob.class, trigger, "prescription_auto_delete_job", new HashMap<String, Object>());
                        logger.info("prescription_auto_delete_job  job success");
                    } else {
                        logger.info("prescription_auto_delete_job  job exist");
                    }

                    break;
                case "data_ykupload_job_2" :
                    //互联网医院 监管平台上报
                    if (!quartzHelper.isExistJob("data_ykupload_job_2")) {
                        String trigger = SystemConf.getInstance().getSystemProperties().getProperty("data_ykupload_job_2");
                        quartzHelper.addJob(YKYYDataUploadJob.class, trigger, "data_ykupload_job_2", new HashMap<String, Object>());
                        logger.info("data_ykupload_job_2  job success");
                    } else {
                        logger.info("data_ykupload_job_2  job exist");
                    }
                    break;

                case "DailyReportRemindJob" :
                    //日报待上报提醒
                    if (!quartzHelper.isExistJob("DailyReportRemindJob")) {
                        String trigger = SystemConf.getInstance().getSystemProperties().getProperty("DailyReportRemindJob");
                        quartzHelper.addJob(DailyReportRemindJob.class, trigger, "DailyReportRemindJob", new HashMap<String, Object>());
                        logger.info("DailyReportRemindJob  job success");
                    } else {
                        logger.info("DailyReportRemindJob  job exist");
                    }
                    break;
                case "DailyReportRemindSecondJob" :
                    //日报待上报提醒
                    if (!quartzHelper.isExistJob("DailyReportRemindSecondJob")) {
                        String trigger = SystemConf.getInstance().getSystemProperties().getProperty("DailyReportRemindSecondJob");
                        quartzHelper.addJob(DailyReportRemindSecondJob.class, trigger, "DailyReportRemindSecondJob", new HashMap<String, Object>());
                        logger.info("DailyReportRemindSecondJob  job success");
                    } else {
                        logger.info("DailyReportRemindSecondJob  job exist");
                    }
                    break;
                case "DailyReportTotalRemindJob" :
                    //日报上报统计
                    if (!quartzHelper.isExistJob("DailyReportTotalRemindJob")) {
                        String trigger = SystemConf.getInstance().getSystemProperties().getProperty("DailyReportTotalRemindJob");
                        quartzHelper.addJob(DailyReportTotalRemindJob.class, trigger, "DailyReportTotalRemindJob", new HashMap<String, Object>());
                        logger.info("DailyReportTotalRemindJob  job success");
                    } else {
                        logger.info("DailyReportTotalRemindJob  job exist");
                    }
                    break;
                case "DailyReportWsbTotalRemindJob" :
                    //日报未上报通知
                    if (!quartzHelper.isExistJob("DailyReportWsbTotalRemindJob")) {
                        String trigger = SystemConf.getInstance().getSystemProperties().getProperty("DailyReportWsbTotalRemindJob");
                        quartzHelper.addJob(DailyReportWsbTotalRemindJob.class, trigger, "DailyReportWsbTotalRemindJob", new HashMap<String, Object>());
                        logger.info("DailyReportWsbTotalRemindJob  job success");
                    } else {
                        logger.info("DailyReportWsbTotalRemindJob  job exist");
                    }
                    break;
                default :
            }
            return success("启动成功！");

        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "启动失败:" + e.getMessage());
        }
    }

    /**
     * 生成过去几天的数据
     *
     * @param day
     * @return
     */
    @RequestMapping(value = "productDataByDay", method = RequestMethod.GET)
    public String productDataByDay(Integer day) {
        try {
            jobService.productDataByDay(day);
            return success("启动成功！");
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "启动失败:" + e.getMessage());
        }
    }

    /**
     * 生成过去某一天的全部的数据
     *
     * @param day
     * @return
     */
    @RequestMapping(value = "productDataByOneDay", method = RequestMethod.GET)
    public String productDataByOneDay(String day) {
        try {
            jobService.productDataByOneDay(day);
            return success("启动成功！");
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "启动失败:" + e.getMessage());
        }
    }

    /**
     * 生成过去某一天的某一个指标的数据
     *
     * @param day
     * @return
     */
    @RequestMapping(value = "productDataByOneDayWithId", method = RequestMethod.GET)
    public String productDataByOneDayWithId(String day, String id) {
        try {
            jobService.productDataByOneDayWithId(day, id);
            return success("启动成功！");
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "启动失败:" + e.getMessage());
        }
    }

    /**
     * 生成过去某几天的某一个指标的数据
     *
     * @param day
     * @return
     */
    @RequestMapping(value = "productDataByDayAndId", method = RequestMethod.GET)
    public String productDataByDayAndId(Integer day, String id) {
        try {
            jobService.productDataByDayAndId(day, id);
            return success("启动成功！");
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "启动失败:" + e.getMessage());
        }
    }

    /**
     * 启动任务
     *
     * @param id id
     * @return
     */
    @RequestMapping(value = "startById", method = RequestMethod.GET)
    public String startById(String id) {
        try {

            jobService.startById(id);
            return success("启动成功！");
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "启动失败:" + e.getMessage());
        }
    }

    /**
     * 停止任务
     *
     * @param id id
     * @return
     */
    @RequestMapping(value = "stopById", method = RequestMethod.GET)
    public String stopById(String id) {
        try {
            jobService.stopById(id);
            return success("停止成功！");
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "启动失败:" + e.getMessage());
        }
    }

    /**
     * 停止所有任务
     *
     * @return
     */
    @RequestMapping(value = "stopAll", method = RequestMethod.GET)
    public String stopAll() {
        try {
            jobService.stopAll();
            return success("停止成功！");
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "启动失败:" + e.getMessage());
        }
    }

    /**
     * 启动所有任务
     *
     * @return
     */
    @RequestMapping(value = "startAll", method = RequestMethod.GET)
    public String startAll() {
        try {
            jobService.startAll();
            return success("启动成功！");
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "启动失败:" + e.getMessage());
        }
    }

    @RequestMapping(value = "/PrescriptionStatusUpdateJob", method = RequestMethod.POST)
    @ApiOperation("处方状态变更")
    public String PrescriptionStatusUpdateJob() {
        try {
                quartzHelper.startNow(PrescriptionStatusUpdateJob.class, UUID.randomUUID().toString(), null);
            return write(200, "启动成功");
        } catch (Exception e) {
            error(e);
            return error(-1, "启动失败");
        }
    }

    @RequestMapping(value = "/PrescriptionOverdueJob", method = RequestMethod.POST)
    @ApiOperation("门诊过期")
    public String PrescriptionOverdueJob() {
        try {
            quartzHelper.startNow(PrescriptionOverdueJob.class, UUID.randomUUID().toString(), null);
            return write(200, "启动成功");
        } catch (Exception e) {
            error(e);
            return error(-1, "启动失败");
        }
    }

    @RequestMapping(value = "/internetUpload", method = RequestMethod.GET)
    @ApiOperation("监管平台测试")
    public String internetUpload(String taskId,String endDate,String startDate,String keyId) {
        try {

            String res = "";
            switch(taskId){
                case "2.2":
                    logger.info("START========2.2 医院基本信息========");
                    res =  internetService.upNsOrganBas();
                    logger.info("END==========2.2 医院基本信息 ========" + res);
                    break;
                case "2.3" :
                    // 2、分步执行需要JOB执行的服务
                    logger.info("START========2.3 医师基本信息========");
                    res = internetService.upNsDoctorRecord(keyId);
                    logger.info("END==========2.3 医师基本信息 ========" + res);
                    break; 
                case "2.5" :
                    logger.info("START========2.5 网上预约挂号上传开始========");
                    res = internetService.upAppointmentOnline(startDate,endDate,keyId);
                    logger.info("END==========2.5 网上预约挂号上传结束========" + res);
                    break; 
                case "2.6" :
                    logger.info("START========2.6 网上医技预约上传开始========");
                    res = internetService.upMedicalOnline(startDate,endDate,keyId);
                    logger.info("END==========2.6 网上医技预约上传结束========" + res);
                    break; 
                case "2.7" :
                    logger.info("START========2.7 医生评价信息开始========");
                    res = internetService.upNsDoctorScore(startDate,endDate,keyId);
                    logger.info("END==========2.7 医生评价信息结束========" + res);
                    break; 
                case "2.8" :
                    logger.info("START========2.8 网络咨询服务信息开始========");
                    res = internetService.upNsOnlineAsk(startDate,endDate,keyId);
                    logger.info("END==========2.8 网络咨询服务信息结束========" + res);
                    break; 
                case "2.9" :
                    logger.info("START========2.9 网络咨询服务信息开始========");
                    res = internetService.upNsOnlineMed(startDate,endDate,keyId);
                    logger.info("END==========2.9 网络咨询服务信息结束========" + res);
                    break; 
                case "2.10" :
                    logger.info("START========2.10 电子处方表开始========");
                    res = internetService.upPrescription(startDate,endDate,keyId);
                    logger.info("END==========2.10 电子处方表结束========" + res);
                    break; 
                case "2.11" :
                    logger.info("START========2.11 电子处方_药品明细表开始 ========");
                    res = internetService.upPrescriptionDrug(startDate,endDate,keyId);
                    logger.info("END==========2.11 电子处方_药品明细表结束 ========" + res);
                    break;
                case "2.12" :
                    logger.info("START========2.12 在线诊疗服务信息-过程图片上传 ========");
                    res = internetService.upNsOnlineImg(startDate,endDate,keyId);
                    logger.info("END==========2.12 在线诊疗服务信息-过程图片上传 ========" + res);
                    break;
                case "2.13" :
                    logger.info("START========2.13 用户协议信息 ========");
                    res = internetService.upNsOrganProtocol(startDate,endDate,keyId);
                    logger.info("END==========2.13 用户协议信息 ========" + res);
                    break;
                default :
            }
            return success("success");
        } catch (Exception e) {
            error(e);
            return error(-1, "启动失败");
        }
    }

    @RequestMapping(value = "/setData", method = RequestMethod.POST)
    @ApiOperation("生成数据")
    public String setData(String doctor,String date) {
        try {
            //生成门诊记录
            List<WlyyOutpatientDO> list = dataGeneratorService.setData(doctor,date);
            //生成咨询数据
            imService.setJobData(list,date);

            return write(200, "数据已经生成");
        } catch (Exception e) {
            error(e);
            return error(-1, "数据已经生成");
        }
    }

    @RequestMapping(value = "/waitingItem", method = RequestMethod.POST)
    @ApiOperation("测试健康上报推送")
    public String waitingItem() {
        try {
            healthUploadService.waitingItem();
            return write(200, "测试推送成功");
        } catch (Exception e) {
            error(e);
            return error(-1, "测试推送失败");
        }
    }

    @RequestMapping(value = "/healthUploadRemind", method = RequestMethod.POST)
    @ApiOperation("测试健康上报推送")
    public String healthUploadRemind(@ApiParam(name = "flag", value = "1未上报提醒2未上报汇总3每日汇总")
                                  @RequestParam(value = "flag", required = false) Integer flag) {
        try {
            if (flag==1){
                healthUploadService.healthUploadRemind();
            }else if (flag==2){
                healthUploadService.healthUploadWsbTotalRemind();
            }else if (flag==3){
                healthUploadService.healthUploadTotalRemind();
            }
            return write(200, "测试推送成功");
        } catch (Exception e) {
            error(e);
            return error(-1, "测试推送失败");
        }
    }

    @RequestMapping(value = "/delData", method = RequestMethod.POST)
    @ApiOperation("删除数据")
    public String delData(String doctor,String date) {
        try {
            List<WlyyOutpatientDO> list = dataGeneratorService.delData(doctor,date);

            imService.deleteJobData(list,date);
            return write(200, "数据已经删除");
        } catch (Exception e) {
            error(e);
            return error(-1, "数据已经删除");
        }
    }

    @RequestMapping(value = "/ykyyUpload", method = RequestMethod.GET)
    @ApiOperation("上传")
    public String ykyyUpload(String id,String startDate,String endDate,String keyId) {
        try {
            switch(id){
                case "2.2" :
                    ykyyInternetService.upNsOrganBas();
                    break; //可选
                case "2.3" :
                    //语句
                    ykyyInternetService.upNsDoctorRecord(keyId);
                    break; //可选
                case "2.5" :
                    //语句
                    ykyyInternetService.upAppointmentOnline(startDate,endDate,keyId);
                    break; //可选
                case "2.6" :
                    //语句
                    ykyyInternetService.upMedicalOnline(startDate,endDate,keyId);
                    break; //可选
                case "2.7" :
                    //语句
                    ykyyInternetService.upNsDoctorScore(startDate,endDate,keyId);
                    break; //可选upNsDoctorScore
                case "2.8" :
                    //语句
                    ykyyInternetService.upNsOnlineAsk(startDate,endDate,keyId);
                    break; //可选
                case "2.9" :
                    //语句
                    ykyyInternetService.upNsOnlineMed(startDate,endDate,keyId);
                    break; //可选
                case "2.10" :
                    //语句
                    ykyyInternetService.upPrescription(startDate,endDate,keyId);
                    break; //可选
                case "2.11" :
                    //语句
                    ykyyInternetService.upPrescriptionDrug(startDate,endDate,keyId);
                    break; //可选
                case "2.12" :
                    //语句
                    ykyyInternetService.upNsOnlineImg(startDate,endDate,keyId);
                    break; //可选
                case "2.13" :
                    //语句
                    ykyyInternetService.upNsOrganProtocol(startDate,endDate,keyId);
                    break; //可选
                case "2.14" :
                    //语句
                    ykyyInternetService.upNsInvokeTotal(startDate,endDate);
                    break; //可选
                default : //可选
                    //语句
            }
            return write(200, "执行成功");
        } catch (Exception e) {
            error(e);
            return error(-1, "执行失败");
        }
    }




    @RequestMapping(value = "/inernetCommonUpload", method = RequestMethod.GET)
    @ApiOperation("监管平台通用业务数据上传")
    public String inernetCommonUpload(String id,String startDate,String endDate,String keyId) {
        try {
            switch(id){
                case "2.2" :
                    internetCommonService.upNsOrganBas();
                    break; //可选
                case "2.3" :
                    //语句
                    internetCommonService.upNsDoctorRecord(keyId);
                    break; //可选
                case "2.4" :
                    //语句
                    internetCommonService.achNsDoctorRecord(keyId);
                    break; //可选
                case "2.5" :
                    //语句
                    internetCommonService.upAppointmentOnline(startDate,endDate,keyId);
                    break; //可选
                case "2.6" :
                    //语句
                    internetCommonService.upMedicalOnline(startDate,endDate,keyId);
                    break; //可选
                case "2.7" :
                    //语句
                    internetCommonService.upNsDoctorScore(startDate,endDate,keyId);
                    break; //可选upNsDoctorScore
                case "2.8" :
                    //语句
                    internetCommonService.upNsOnlineAsk(startDate,endDate,keyId);
                    break; //可选
                case "2.9" :
                    //语句
                    internetCommonService.upNsOnlineMed(startDate,endDate,keyId);
                    break; //可选
                case "2.10" :
                    //语句
                    internetCommonService.upPrescription(startDate,endDate,keyId);
                    break; //可选
                case "2.11" :
                    //语句
                    internetCommonService.upPrescriptionDrug(startDate,endDate,keyId);
                    break; //可选
                case "2.12" :
                    //语句
                    internetCommonService.upNsOnlineImg(startDate,endDate,keyId);
                    break; //可选
                case "2.13" :
                    //语句
                    internetCommonService.upNsOrganProtocol(startDate,endDate,keyId);
                    break; //可选
                case "2.14" :
                    //语句
                    long days = getDaySub(startDate,endDate);
                    SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
                    for (int i = 0;i<days;i++){
                        endDate = format.format(com.yihu.jw.util.date.DateUtil.getPreDays(format.parse(startDate),1));
                        internetCommonService.upNsInvokeTotal(startDate,endDate);
                        startDate = endDate;
                    }

                    break; //可选
                default : //可选
                    //语句
            }
            return write(200, "执行成功");
        } catch (Exception e) {
            error(e);
            return error(-1, "执行失败");
        }
    }

    public static void main(String[] args) throws Exception {
        String startDate = "2020-08-19";
        String endDate = "2020-12-01";
        long days = getDaySub(startDate,endDate);
        System.out.println(days);
    }
    public static long getDaySub(String beginDateStr,String endDateStr) throws Exception {

        long day = 0;
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        Date beginDate;
        Date endDate;
        try {
            beginDate = format.parse(beginDateStr);
            endDate = format.parse(endDateStr);
            day = (endDate.getTime()-beginDate.getTime())/(24*60*60*1000);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        System.out.println("day:"+day);

        return day;
    }

    @RequestMapping(value = "/testImg", method = RequestMethod.GET)
    @ApiOperation("测试图片")
    public String testImg(String path) {
      return write(200, ykyyInternetService.testImg(path));
    }


    @RequestMapping(value = "changeDoctorRemindTimeOut", method = RequestMethod.GET)
    public String changeDoctorRemindTimeOut(String corn){
        AutoTimeOutRemind autoTimeOutRemind = new AutoTimeOutRemind();

        return autoTimeOutRemind.change(corn);

    }
    @RequestMapping(value = "changeOverDueTime", method = RequestMethod.GET)
    public String changeOverDueTime(String corn){
        OverdueJob autoTimeOutRemind = new OverdueJob();

        return autoTimeOutRemind.change(corn);

    }
    @RequestMapping(value = "changeCancelTime", method = RequestMethod.GET)
    public String changeCancelTime(String corn){
        TimeoutOverDueJob cancel = new TimeoutOverDueJob();

        return cancel.change(corn);

    }
    @RequestMapping(value = "cancelSinglePre", method = RequestMethod.GET)
    @ApiOperation("取消未接诊")
    public String cancelSinglePre(String outpatientId,String day){
        if (StringUtils.isNotBlank(outpatientId)){
            System.out.println(outpatientId);
            return prescriptionStatusUpdateService.singelCancel(outpatientId);
        }else {
            return prescriptionStatusUpdateService.cancelAll(day);
        }
    }
    @RequestMapping(value = "overRecivePre", method = RequestMethod.GET)
    @ApiOperation("结束接诊")
    public void cancelRecivePre(String outpatientId){
        if (StringUtils.isNotBlank(outpatientId)){
            System.out.println(outpatientId);
            prescriptionStatusUpdateService.cancelRecieveSingle(outpatientId);
        }else {
            prescriptionStatusUpdateService.cancelRecieve();
        }
    }
    @RequestMapping(value = "cancelSinglePreByday", method = RequestMethod.GET)
    @ApiOperation("按天取消未接诊")
    public String cancelSinglePreSendMessage(String day){

        return prescriptionStatusUpdateService.cancelAllandSendMessage(day);

    }

    @RequestMapping(value = "finishOldConsult", method = RequestMethod.GET)
    @ApiOperation("取消旧的咨询")
    public String finishOldConsult(String consult){

        return success(prescriptionStatusUpdateService.finishOldConsult(consult));

    }

    @RequestMapping(value = "saveVideoFile", method = RequestMethod.GET)
    @ApiOperation("下载云信录制是哦")
    public String saveVideoFile(String vip){
        return success(ykyyPrescriptionService.saveVideoFile(vip));
    }


    @RequestMapping(value = "saveVideoFileList", method = RequestMethod.GET)
    @ApiOperation("下载云信录制")
    public String saveVideoFile(@RequestParam(value = "startTime", required = false)String startTime,
                                @RequestParam(value = "endTime", required = false)String endTime, Integer status, Integer page , Integer pageSize){
        return success(ykyyPrescriptionService.saveVideoFileList(startTime,endTime,status,page,pageSize));
    }
    @RequestMapping(value = "/testRemindDoctor", method = RequestMethod.GET)
    @ApiOperation("测试发送消息")
    public String testRemindDoctor(){
        remindDoctorService.sendSystemMessage();
        return success("发送成功");
    }


    @RequestMapping(value = "/syncDmHospital", method = RequestMethod.GET)
    @ApiOperation("syncDmHospital")
    public String syncDmHospital(@RequestParam(value = "param", required = false)String param){
        syncWlyyHospitalService.syncHospital(param);
        return success("操作成功");
    }
    @RequestMapping(value = "/syncTeam", method = RequestMethod.GET)
    @ApiOperation("syncTeam")
    public String syncTeam(@RequestParam(value = "orgCode", required = false)String orgCode){
        syncWlyyHospitalService.syncTeam(orgCode);
        return success("操作成功");
    }
    @RequestMapping(value = "/syncTeamMember", method = RequestMethod.GET)
    @ApiOperation("syncTeamMember")
    public String syncTeamMember(@RequestParam(value = "teamId", required = false)String teamId){
        syncWlyyHospitalService.syncTeamMember(teamId);
        return success("操作成功");
    }

    @RequestMapping(value = "/syncTeamMemberByTime", method = RequestMethod.GET)
    @ApiOperation("syncTeamMemberByTime")
    public String syncTeamMemberByTime(@RequestParam(value = "nowTime", required = false)String nowTime){
        syncWlyyHospitalService.syncTeamMemberByTime(nowTime);
        return success("操作成功");
    }

    @RequestMapping(value = "/cancelOutPatientOver", method = RequestMethod.GET)
    @ApiOperation("cancelOutPatientOver")
    public String syncTeamMemberByTime(){
        timeoutOverDueService.cancelOutPatientOver();
        return success("操作成功");
    }

    @RequestMapping(value = "/ykinternetUpload", method = RequestMethod.GET)
    @ApiOperation("眼科监管平台测试")
    public String ykinternetUpload(String taskId,String endDate,String startDate,String keyId) {
        try {

            String res = "";
            switch(taskId){
                case "2.2":
                    logger.info("START========2.2 医院基本信息========");
                    res =  ykyyCommonService.upNsOrganBas();
                    logger.info("END==========2.2 医院基本信息 ========" + res);
                    break;
                case "2.3" :
                    // 2、分步执行需要JOB执行的服务
                    logger.info("START========2.3 医师基本信息========");
                    res = ykyyCommonService.upNsDoctorRecord(startDate,endDate,keyId);
                    logger.info("END==========2.3 医师基本信息 ========" + res);
                    break;
                case "2.5" :
                    logger.info("START========2.5 网上预约挂号上传开始========");
                    res = ykyyCommonService.upAppointmentOnline(startDate,endDate,keyId);
                    logger.info("END==========2.5 网上预约挂号上传结束========" + res);
                    break;
                case "2.6" :
                    logger.info("START========2.6 网上医技预约上传开始========");
                    res = ykyyCommonService.upMedicalOnline(startDate,endDate,keyId);
                    logger.info("END==========2.6 网上医技预约上传结束========" + res);
                    break;
                case "2.7" :
                    logger.info("START========2.7 医生评价信息开始========");
                    res = ykyyCommonService.upNsDoctorScore(startDate,endDate,keyId);
                    logger.info("END==========2.7 医生评价信息结束========" + res);
                    break;
                case "2.8" :
                    logger.info("START========2.8 网络咨询服务信息开始========");
                    res = ykyyCommonService.upNsOnlineAsk(startDate,endDate,keyId);
                    logger.info("END==========2.8 网络咨询服务信息结束========" + res);
                    break;
                case "2.9" :
                    logger.info("START========2.9 网络咨询服务信息开始========");
                    res = ykyyCommonService.upNsOnlineMed(startDate,endDate,keyId);
                    logger.info("END==========2.9 网络咨询服务信息结束========" + res);
                    break;
                case "2.10" :
                    logger.info("START========2.10 电子处方表开始========");
                    res = ykyyCommonService.upPrescription(startDate,endDate,keyId);
                    logger.info("END==========2.10 电子处方表结束========" + res);
                    break;
                case "2.11" :
                    logger.info("START========2.11 电子处方_药品明细表开始 ========");
                    res = ykyyCommonService.upPrescriptionDrug(startDate,endDate,keyId);
                    logger.info("END==========2.11 电子处方_药品明细表结束 ========" + res);
                    break;
                default :
            }
            return success("success");
        } catch (Exception e) {
            error(e);
            return error(-1, "启动失败");
        }
    }

    @RequestMapping(value = "/testNow", method = RequestMethod.POST)
    @ApiOperation("立即执行")
    public String executeSignFamilyPayResultJob(String className) {
        try {
            String uuid = getUID();
            Class c1 = Class.forName(className);//com.yihu.jw.care.job.consult.FinishConsultJob
            quartzHelper.startNow(c1, uuid, null);
            quartzHelper.removeJob(uuid);
            return write(200, "启动成功");
        } catch (Exception e) {
            error(e);
            return error(-1, e.getMessage());
        }
    }

}
