package com.yihu.jw.hospital.endpoint.Xzyy;

import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.hospital.prescription.service.XzyyPrescriptionService;
import com.yihu.jw.hospital.prescription.service.entrance.XzzxEntranceService;
import com.yihu.jw.restmodel.hospital.prescription.WlyyPrescriptionVO;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * Created by Trick on 2020/1/19.
 */
@RestController
@RequestMapping(value ="/Xzyy")
@Api(value = "心脏医院", description = "心脏医院", tags = {"心脏医院"})
public class XzyyController extends EnvelopRestEndpoint {


    @Autowired
    private XzzxEntranceService xzzxEntranceService;
    @Autowired
    private BaseDoctorDao baseDoctorDao;
    @Autowired
    private XzyyPrescriptionService xzyyPrescriptionService;

    /**
     * 查询患者就诊卡
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/findPatientCard")
    @ApiOperation(value = "查询患者就诊卡", notes = "查询患者就诊卡")
    public ListEnvelop findPatientCard(@ApiParam(name = "patient", value = "居民code", required = false)
                                      @RequestParam(value = "patient",required = false)String patient)throws Exception{
        try {
            return success(xzzxEntranceService.selectPateintCard(patient));
        }catch (Exception e){
            return null;
        }

    }


    /**
     * 获取卡信息
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/getCardInfo")
    @ApiOperation(value = "获取卡信息", notes = "获取卡信息")
    public ObjEnvelop getCardInfo(@ApiParam(name = "cardNo", value = "就诊卡", required = false)
                                  @RequestParam(value = "cardNo",required = false)String cardNo)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.selectCardInfo(cardNo));
    }

    /**
     * 获取门诊就诊记录列表
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/findOutpatientList")
    @ApiOperation(value = "获取门诊就诊记录列表", notes = "获取门诊就诊记录列表")
    public ListEnvelop findOutpatientInfo(@ApiParam(name = "patient", value = "居民code", required = true)
                                         @RequestParam(value = "patient",required = true)String patient,
                                         @ApiParam(name = "startTime", value = "开始时间", required = false)
                                         @RequestParam(value = "startTime",required = false)String startTime,
                                         @ApiParam(name = "endTime", value = "结束时间", required = false)
                                             @RequestParam(value = "endTime",required = false)String endTime,
                                          @ApiParam(name = "cardNo", value = "就诊卡", required = false)
                                              @RequestParam(value = "cardNo",required = false)String cardNo)throws Exception{
        return success(xzzxEntranceService.selectOutpatientList(patient,startTime,endTime,cardNo));
    }

    /**
     * 获取单条门诊就诊记录
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/findOutpatientInfo")
    @ApiOperation(value = "获取门诊就诊记录", notes = "获取门诊就诊记录")
    public ObjEnvelop findOutpatientInfo(@ApiParam(name = "patient", value = "居民code", required = true)
                                         @RequestParam(value = "patient",required = true)String patient,
                                         @ApiParam(name = "conNo", value = "就诊次数", required = false)
                                         @RequestParam(value = "conNo",required = false)String conNo)throws Exception{
        return success(xzzxEntranceService.selectOutpatientInfo(patient,conNo));
    }

    /**
     * 获取门诊下所有处方信息
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/findPrescriptionList")
    @ApiOperation(value = "获取门诊下所有处方信息", notes = "获取门诊下所有处方信息")
    public ListEnvelop findPrescriptionList(@ApiParam(name = "outPatientId", value = "门诊id", required = true)
                                           @RequestParam(value = "outPatientId",required = true)String outPatientId,
                                           @ApiParam(name = "realOrder", value = "处方code", required = false)
                                           @RequestParam(value = "realOrder",required = false)String realOrder)throws Exception{
            return success(xzzxEntranceService.findPrescriptionList(outPatientId,realOrder));
    }


    /**
     * 获取门诊下所有处方信息
     * @param registerSn 流水号(必传)
     * @param patNo 病人id(必传)
     * @param admNo 住院唯一号(必传)
     * @param realOrder 处方号(非必传，传相当于查单条处方)
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/getOriginPrescriptionList")
    @ApiOperation(value = "获取门诊下所有处方信息", notes = "获取门诊下所有处方信息")
    public MixEnvelop getOriginPrescriptionList(@ApiParam(name = "registerSn", value = "流水号", required = false)
                                                @RequestParam(value = "registerSn",required = false)String registerSn,
                                                @ApiParam(name = "patNo", value = "病人id", required = false)
                                                @RequestParam(value = "patNo",required = false)String patNo,
                                                @ApiParam(name = "admNo", value = "住院唯一号", required = false)
                                                @RequestParam(value = "admNo",required = false)String admNo,
                                                @ApiParam(name = "realOrder", value = "处方号", required = false)
                                                @RequestParam(value = "realOrder",required = false)String realOrder)throws Exception{
        MixEnvelop envelop = new MixEnvelop();
        List<WlyyPrescriptionVO> list = xzzxEntranceService.selectPrescriptionList(registerSn,patNo,admNo,realOrder,null,null);
        envelop.setDetailModelList(list);
        return envelop;
    }



    /**
     * 处方状态变更
     * @param admNo 就诊唯一号(必填)
     * @param realOrder 处方号(必填)
     * @param status 处方状态：3结算完成(必填)
     * @return
     * @throws Exception
     */
    @PostMapping(value = "/updatePreStatus")
    @ApiOperation(value = "处方状态变更", notes = "处方状态变更")
    public ObjEnvelop updatePreStatus(@ApiParam(name = "admNo", value = "就诊唯一号", required = false)
                                      @RequestParam(value = "admNo",required = false)String admNo,
                                      @ApiParam(name = "realOrder", value = "处方号", required = false)
                                      @RequestParam(value = "realOrder",required = false)String realOrder,
                                      @ApiParam(name = "status", value = "处方状态：3结算完成", required = false)
                                      @RequestParam(value = "status",required = false)String status)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.updatePreStatus(admNo,realOrder,status));
    }

    /**
     * 查询历史就诊病历
     * @param patientId 病人id(必填)
     * @param eventNo 病人住院号
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/findHistoryEmr")
    @ApiOperation(value = "查询历史就诊病历", notes = "查询历史就诊病历")
    public ObjEnvelop findHistoryEmr(@ApiParam(name = "patientId", value = "病人id", required = false)
                                      @RequestParam(value = "patientId",required = false)String patientId,
                                      @ApiParam(name = "eventNo", value = "病人住院号", required = false)
                                      @RequestParam(value = "eventNo",required = false)String eventNo)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.selectHistoryEmr(patientId,eventNo));
    }
    /**
     * 查询历史就诊病历
     * @param patientId 病人id(必填)
     * @param eventNo 病人住院号
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/findHistoryEmrRecord")
    @ApiOperation(value = "查询历史就诊病历", notes = "查询历史就诊病历")
    public ObjEnvelop findHistoryEmrRecord(@ApiParam(name = "templateId", value = "模板id", required = false)
                                               @RequestParam(value = "templateId",required = false)String templateId,
                                        @ApiParam(name = "patientId", value = "病人id", required = false)
                                     @RequestParam(value = "patientId",required = false)String patientId,
                                     @ApiParam(name = "eventNo", value = "病人住院号", required = false)
                                     @RequestParam(value = "eventNo",required = false)String eventNo)throws Exception{
        return ObjEnvelop.getSuccess("ok",xzzxEntranceService.selectHistoryEmrRecord(templateId,patientId,eventNo));
    }
    /**
     * 请求自动签名授权接口
     * @param sysTag
     * @param doctorId
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/selfSignRequest")
    @ApiOperation(value = "请求自动签名授权接口", notes = "请求自动签名授权接口")
    public ObjEnvelop selfSignRequest(@ApiParam(name = "sysTag", value = "默认his", required = false)
                                           @RequestParam(value = "sysTag",required = false)String sysTag,
                                           @ApiParam(name = "doctorId", value = "医生id", required = false)
                                           @RequestParam(value = "doctorId",required = false)String doctorId)throws Exception{
        BaseDoctorDO baseDoctorDO = baseDoctorDao.findByIdAndDel(doctorId);
        return ObjEnvelop.getSuccess("ok", xzyyPrescriptionService.selfSignRequest(baseDoctorDO.getOpenid(),sysTag));
    }
    /**
     * 请求自动签名授权接口
     * @param sysTag
     * @param doctorId
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/selfSignResult")
    @ApiOperation(value = "获取授权结果", notes = "获取授权结果")
    public ObjEnvelop selfSignResult(@ApiParam(name = "sysTag", value = "默认his", required = false)
                                      @RequestParam(value = "sysTag",required = false)String sysTag,
                                      @ApiParam(name = "doctorId", value = "医生id", required = false)
                                      @RequestParam(value = "doctorId",required = false)String doctorId)throws Exception{
        BaseDoctorDO baseDoctorDO = baseDoctorDao.findByIdAndDel(doctorId);
        return ObjEnvelop.getSuccess("ok", xzyyPrescriptionService.selfSignGetResult(baseDoctorDO,sysTag));
    }
    /**
     * 请求自动签名授权接口
     * @param sysTag
     * @param doctorId
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/selfSignQuit")
    @ApiOperation(value = "退出授权接口", notes = "退出授权接口")
    public ObjEnvelop selfSignQuit(@ApiParam(name = "sysTag", value = "默认his", required = false)
                                      @RequestParam(value = "sysTag",required = false)String sysTag,
                                      @ApiParam(name = "doctorId", value = "医生id", required = false)
                                      @RequestParam(value = "doctorId",required = false)String doctorId)throws Exception{
        BaseDoctorDO baseDoctorDO = baseDoctorDao.findByIdAndDel(doctorId);
        return ObjEnvelop.getSuccess("ok", xzyyPrescriptionService.selfSignQuit(baseDoctorDO.getOpenid(),sysTag));
    }

    /**
     * 请求自动签名授权接口
     * @param doctorId
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/checkIsNeedSign")
    @ApiOperation(value = "判断是否需要授权", notes = "判断是否需要授权")
    public ObjEnvelop checkIsNeedSign(
                                   @ApiParam(name = "doctorId", value = "医生id", required = false)
                                   @RequestParam(value = "doctorId",required = false)String doctorId,
                                   @ApiParam(name = "sysTag", value = "系统类型", required = false)
                                   @RequestParam(value = "sysTag",required = false)String sysTag)throws Exception{
        return ObjEnvelop.getSuccess("ok", xzyyPrescriptionService.checkDoctorNeedSign(doctorId,sysTag));
    }

}
