package com.yihu.jw.hospital.endpoint.account;

import com.yihu.jw.patient.service.BasePatientGroupDictService;
import com.yihu.jw.patient.service.BasePatientService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.base.BaseRequestMapping;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("patient" )
@Api(tags = "用户基础信息", description = "互联网医院")
public class PatientEndpoint extends EnvelopRestEndpoint {

    @Autowired
    private BasePatientService basePatientService;
    @Autowired
    private BasePatientGroupDictService basePatientGroupDictService;


    @GetMapping(value = BaseRequestMapping.BasePatient.getPatientById)
    @ApiOperation(value = "获取居民信息")
    public Envelop getPatientById(
            @ApiParam(name = "id", value = "居民id")
            @RequestParam(value = "id", required = true) String id) throws Exception {
        try {
            return success(basePatientService.getPatientById(id));
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }

    @PostMapping(value = BaseRequestMapping.BasePatient.updatePatientPw)
    @ApiOperation(value = "修改居民账号密码")
    public Envelop updateDoctorPw(@ApiParam(name = "id", value = "居民ID")
                                  @RequestParam(value = "id", required = true)String id,
                                  @ApiParam(name = "pw", value = "密码")
                                  @RequestParam(value = "pw", required = true)String pw,
                                  @ApiParam(name = "orgPw", value = "原密码")
                                  @RequestParam(value = "orgPw", required = false)String orgPw)throws Exception{

        try {
            Boolean isSuccess = basePatientService.updatePatientPw(id,pw,orgPw);
            if (isSuccess){
                return success(isSuccess);
            }else {
                return failed("修改失败，请检查原密码是否正确");
            }
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }

    }
    @GetMapping(value = BaseRequestMapping.BasePatient.getPatientCountByDoctor)
    @ApiOperation(value = "获取居民候诊数量信息")
    public Envelop getPatientCountByDoctor(
            @ApiParam(name = "doctor", value = "doctor")
            @RequestParam(value = "doctor", required = true) String doctor,
            @ApiParam(name = "patient", value = "patient")
            @RequestParam(value = "patient", required = false) String patient) throws Exception {

        try {
            return success(basePatientService.getPatientCountByDoctor(doctor,patient));
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }
    @GetMapping(value = BaseRequestMapping.BasePatient.getPatientEmrByDoctor)
    @ApiOperation(value = "获取居民电子病历信息")
    public MixEnvelop getPatientEmrByDoctor(@ApiParam(name = "patient", value = "patient")
                                             @RequestParam(value = "patient", required = false) String patient,
                                            @ApiParam(name = "doctor", value = "doctor")
                                        @RequestParam(value = "doctor", required = false) String doctor,
                                            @ApiParam(name = "page", value = "page")
                                             @RequestParam(value = "page", required = false) Integer page,
                                            @ApiParam(name = "pageSize", value = "pageSize")
                                             @RequestParam(value = "pageSize", required = false) Integer pageSize) throws Exception {

        try {
            return basePatientService.findEmrByPatientId(patient,doctor,page,pageSize);
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }

    @GetMapping(value = BaseRequestMapping.BasePatient.findPaitentPrescription)
    @ApiOperation(value = "查询患者所有处方信息", notes = "查询患者所有处方信息")
    public PageEnvelop findPaitentPrescription(@ApiParam(name = "keyName", value = "搜索关键字")
                                                  @RequestParam(value = "keyName", required = false)String keyName,
                                               @ApiParam(name = "patient", value = "患者id")
                                               @RequestParam(value = "patient", required = false)String patient,
                                               @ApiParam(name = "doctor", value = "医生id")
                                                   @RequestParam(value = "doctor", required = false)String doctor,
                                                  @ApiParam(name = "status[]", value = "状态")
                                                  @RequestParam(value = "status[]", required = false)String status,
                                                  @ApiParam(name = "isSelectDrug", value = "1查询处方药品1是0查询检查")
                                                  @RequestParam(value = "isSelectDrug", required = false)String isSelectDrug,
                                                  @ApiParam(name = "page", value = "第几页")
                                                  @RequestParam(value = "page",required = false,defaultValue = "1") Integer page,
                                                  @ApiParam(name = "size", value = "页面大小")
                                                  @RequestParam(value = "size",required = false,defaultValue = "10") Integer size)throws Exception {
        try {
            return basePatientService.findPatientAllPrescription(doctor,keyName,status,page,size,patient,isSelectDrug);
        } catch (Exception e) {
            return failedPageEnvelopException(e);
        }
    }
    @PostMapping(value = BaseRequestMapping.BasePatient.createPatientGroupDict)
    @ApiOperation(value = "创建分组")
    public Envelop createPatientGroupDict(@ApiParam(name = "groupId", value = "groupId")
                                            @RequestParam(value = "groupId", required = false) String groupId,
                                            @ApiParam(name = "doctor", value = "doctor")
                                            @RequestParam(value = "doctor", required = false) String doctor,
                                            @ApiParam(name = "groupCode", value = "groupCode")
                                            @RequestParam(value = "groupCode", required = false) String groupCode,
                                            @ApiParam(name = "groupName", value = "groupName")
                                            @RequestParam(value = "groupName", required = false) String groupName) throws Exception {

        try {
            return success(basePatientGroupDictService.createPatientGroupDict(groupId,doctor,groupCode,groupName));
        }catch (Exception e) {
            return failedException(e);
        }
    }

    @GetMapping(value = BaseRequestMapping.BasePatient.deletePatientGroupDict)
    @ApiOperation(value = "删除分组")
    public Envelop deletePatientGroupDict(@ApiParam(name = "groupId", value = "groupId")
                                          @RequestParam(value = "groupId", required = false) String groupId,
                                          @ApiParam(name = "del", value = "del")
                                          @RequestParam(value = "del", required = false) String del) throws Exception {

        try {
            return success(basePatientGroupDictService.delPatientGroupDict(groupId,del));
        }catch (Exception e) {
            return failedException(e);
        }
    }


    @GetMapping(value = BaseRequestMapping.BasePatient.findGroupByDoctor)
    @ApiOperation(value = "查询词典")
    public MixEnvelop findByDoctor(@ApiParam(name = "doctor", value = "doctor")
                                          @RequestParam(value = "doctor", required = false) String doctor,
                                          @ApiParam(name = "groupName", value = "groupName")
                                          @RequestParam(value = "groupName", required = false) String groupName,
                                @ApiParam(name = "page", value = "page")
                                    @RequestParam(value = "page", required = false) Integer page,
                                @ApiParam(name = "pageSize", value = "pageSize")
                                    @RequestParam(value = "pageSize", required = false) Integer pageSize) throws Exception {

        try {
            return basePatientGroupDictService.findByDoctor(doctor,groupName,page,pageSize);
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }

    @GetMapping(value = BaseRequestMapping.BasePatient.movePatientToGroup)
    @ApiOperation(value = "将患者移动到指定分组中")
    public Envelop movePatientToGroup(@ApiParam(name = "groupId", value = "groupId")
                                   @RequestParam(value = "groupId", required = false) String groupId,
                                   @ApiParam(name = "patient", value = "patient")
                                   @RequestParam(value = "patient", required = false) String patient,
                                   @ApiParam(name = "doctor", value = "doctor")
                                   @RequestParam(value = "doctor", required = false) String doctor)  {
        try {
            basePatientGroupDictService.movePatientToGroup(groupId,patient,doctor);
            return Envelop.getSuccess("操作成功");
        }catch (Exception e){
            return failedException(e);
        }
    }
    @GetMapping(value = BaseRequestMapping.BasePatient.delPatientGroup)
    @ApiOperation(value = "删除组中患者")
    public Envelop delPatientGroup(@ApiParam(name = "groupId", value = "groupId")
                                      @RequestParam(value = "groupId", required = false) String groupId)  {
        try {
            basePatientGroupDictService.delPatientGroup(groupId);
            return Envelop.getSuccess("操作成功");
        }catch (Exception e){
            return failedException(e);
        }
    }


    @GetMapping(value = BaseRequestMapping.BasePatient.findPatientGroup)
    @ApiOperation(value = "查询分组患者")
    public MixEnvelop findPatientGroup(@ApiParam(name = "patientName", value = "patientName")
                                   @RequestParam(value = "patientName", required = false) String patientName,
                                   @ApiParam(name = "doctor", value = "doctor")
                                   @RequestParam(value = "doctor", required = false) String doctor,
                                       @ApiParam(name = "groupId", value = "groupId")
                                           @RequestParam(value = "groupId", required = false) String groupId,
                                   @ApiParam(name = "page", value = "page")
                                   @RequestParam(value = "page", required = false) Integer page,
                                   @ApiParam(name = "pageSize", value = "pageSize")
                                   @RequestParam(value = "pageSize", required = false) Integer pageSize) throws Exception {

        try {
            return basePatientGroupDictService.findPatientGroup(patientName,doctor,groupId,page,pageSize);
        }catch (Exception e){
            return failedMixEnvelopException(e);
        }
    }

    @GetMapping(value = BaseRequestMapping.BasePatient.findPatientGroupNum)
    @ApiOperation(value = "查询分组患者数量")
    public Envelop findPatientGroupNum(@ApiParam(name = "doctor", value = "doctor")
                                          @RequestParam(value = "doctor", required = false) String doctor,
                                          @ApiParam(name = "groupIds", value = "多个用逗号隔开")
                                          @RequestParam(value = "groupIds", required = false) String groupIds) throws Exception {

        try {
            return success(basePatientGroupDictService.findPatientGroupNum(doctor,groupIds));
        }catch (Exception e){
            return failedMixEnvelopException(e);
        }
    }


    @GetMapping(value = BaseRequestMapping.BasePatient.findOnePatientInfo)
    @ApiOperation(value = "查询单个患者信息")
    public Envelop findOnePatientInfo(@ApiParam(name = "doctor", value = "doctor")
                                       @RequestParam(value = "doctor", required = false) String doctor,
                                       @ApiParam(name = "patient", value = "patient")
                                       @RequestParam(value = "patient", required = false) String patient) throws Exception {

        try {
            return success(basePatientService.findPatientInfoById(patient,doctor));
        }catch (Exception e){
            return failedMixEnvelopException(e);
        }
    }

}
