package com.yihu.jw.hospital.endpoint.appointment;

import com.yihu.jw.hospital.appointment.service.AppointmentService;
import com.yihu.jw.hospital.appointment.service.SpecialDiseaseService;
import com.yihu.jw.restmodel.hospital.prescription.WlyyOutpatientVO;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * Created by Trick on 2019/10/30.
 */
@RestController
@RequestMapping(value = BaseHospitalRequestMapping.Appointment.PREFIX)
@Api(value = "预约挂号", description = "预约挂号", tags = {"预约挂号接口"})
public class AppointmentEndPoint extends EnvelopRestEndpoint {

    @Autowired
    private AppointmentService appointmentService;
    @Autowired
    private SpecialDiseaseService specialDiseaseService;

    @GetMapping(value = BaseHospitalRequestMapping.Appointment.getDoctorSchedule)
    @ApiOperation(value = "获取医生号源")
    public ListEnvelop getDoctorSchedule(@ApiParam(name = "dept", value = "部门ID")
                                         @RequestParam(value = "dept",required = false) String dept,
                                         @ApiParam(name = "docMappingCode", value = "中山医院医生ID精确查询")
                                         @RequestParam(value = "docMappingCode",required = false)String docMappingCode,
                                         @ApiParam(name = "nameKey", value = "医生姓名模糊")
                                         @RequestParam(value = "nameKey",required = false)String nameKey,
                                         @ApiParam(name = "specialCode", value = "专科")
                                         @RequestParam(value = "specialCode",required = false)String specialCode,
                                         @ApiParam(name = "chargeType", value = "号别")
                                         @RequestParam(value = "chargeType",required = false)String chargeType,
                                         @ApiParam(name = "startDate", value = "开始时间")
                                         @RequestParam(value = "startDate",required = true)String startDate,
                                         @ApiParam(name = "endDate", value = "结束时间")
                                         @RequestParam(value = "endDate",required = true)String endDate) throws Exception {
        return success(appointmentService.getDoctorSchedule( dept,  docMappingCode, nameKey, specialCode, chargeType, startDate, endDate));
    }

    @GetMapping(value = BaseHospitalRequestMapping.Appointment.getDoctorScheduleTime)
    @ApiOperation(value = "获取医生具体号源时间")
    public ListEnvelop getDoctorScheduleTime(@ApiParam(name = "dept", value = "部门ID")
                                         @RequestParam(value = "dept",required = true) String dept,
                                         @ApiParam(name = "docMappingCode", value = "医生ID精确查询")
                                         @RequestParam(value = "docMappingCode",required = true)String docMappingCode,
                                         @ApiParam(name = "date", value = "日期")
                                         @RequestParam(value = "date",required = true)String date,
                                         @ApiParam(name = "ampm", value = "上午a,下午p")
                                         @RequestParam(value = "ampm",required = true)String ampm) throws Exception {
        return success(appointmentService.getDoctorScheduleTime(dept,docMappingCode,date,ampm));
    }

    @GetMapping(value = BaseHospitalRequestMapping.Appointment.getAppointmentList)
    @ApiOperation(value = "获取预约记录列表")
    public ListEnvelop getAppointmentList(@ApiParam(name = "idcard", value = "患者idcard")
                                          @RequestParam(value = "idcard",required = false) String idcard,
                                          @ApiParam(name = "doctor", value = "医生ID精确查询")
                                          @RequestParam(value = "doctor",required = false)String doctor,
                                          @ApiParam(name = "doctorName", value = "医生姓名模糊")
                                          @RequestParam(value = "doctorName",required = false)String doctorName,
                                          @ApiParam(name = "dept", value = "部门")
                                          @RequestParam(value = "dept",required = false)String dept,
                                          @ApiParam(name = "chargeType", value = "号别")
                                          @RequestParam(value = "chargeType",required = false)String chargeType,
                                          @ApiParam(name = "startDate", value = "开始时间")
                                          @RequestParam(value = "startDate",required = true)String startDate,
                                          @ApiParam(name = "endDate", value = "结束时间")
                                          @RequestParam(value = "endDate",required = true)String endDate) throws Exception {
        return success(appointmentService.getAppointmentList(idcard,doctor,doctorName,dept,startDate,endDate,chargeType));
    }

    @PostMapping(value = BaseHospitalRequestMapping.Appointment.makeAppointment)
    @ApiOperation(value = "预约")
    public ObjEnvelop makeAppointment(@ApiParam(name = "appointmentJson", value = "预约实体")
                                       @RequestParam(value = "appointmentJson",required = false) String appointmentJson) throws Exception {
        return success(appointmentService.makeAppointment(appointmentJson));
    }

    @PostMapping(value = BaseHospitalRequestMapping.Appointment.cancelAppointment)
    @ApiOperation(value = "取消预约")
    public ObjEnvelop cancelAppointment(@ApiParam(name = "appointmentJson", value = "预约实体")
                                        @RequestParam(value = "appointmentJson",required = false) String appointmentJson) throws Exception {
        return success(appointmentService.cancelAppointment(appointmentJson));
    }

    @PostMapping(value = BaseHospitalRequestMapping.Appointment.getDoctorByMappingCode)
    @ApiOperation(value = "转化医生映射code")
    public ObjEnvelop getDoctorByMappingCode(@ApiParam(name = "mappingCode", value = "患者ID")
                                        @RequestParam(value = "mappingCode",required = false) String mappingCode) throws Exception {
        return success(appointmentService.getDoctorByMappingCode(mappingCode));
    }

    @GetMapping(value = BaseHospitalRequestMapping.Appointment.findAllSpecialDisease)
    @ApiOperation(value = "获取专病字典")
    public ListEnvelop findAllSpecialDisease() throws Exception {
        return success(specialDiseaseService.findAllSpecialDisease());
    }
}
