package com.yihu.jw.hospital.endpoint.archives;

import com.yihu.jw.contant.CommonContant;
import com.yihu.jw.entity.base.patient.BaseIdcardRecordDO;
import com.yihu.jw.entity.hospital.record.HospitalPatientInpatientRecordDO;
import com.yihu.jw.hospital.record.service.HospitalPatientInpatientRecordService;
import com.yihu.jw.patient.dao.BaseIdcardRecordDao;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * Created by yeshijie on 2022/11/28.
 */
@RestController
@RequestMapping(value = "medicalRecordCopying")
@Api(value = "病案复印",  tags = {"病案复印"})
public class MedicalRecordCopyingEndpoint extends EnvelopRestEndpoint {

    @Autowired
    private HospitalPatientInpatientRecordService inpatientRecordService;
    @Autowired
    private BaseIdcardRecordDao idcardRecordDao;

    @GetMapping(value = "page")
    @ApiOperation(value = "获取病例分页")
    public PageEnvelop page (
            @ApiParam(name = "patient", value = "居民id")
            @RequestParam(value = "patient", required = true) String patient,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1")
            @RequestParam(value = "page") int page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15")
            @RequestParam(value = "size") int size) {
        try {
            String filters = "patient="+patient;

            List<HospitalPatientInpatientRecordDO> recordDOS = inpatientRecordService.search(null, filters, "-visitTime", page, size);
            long count = inpatientRecordService.getCount(filters);
            return PageEnvelop.getSuccessListWithPage(CommonContant.Message.GET_SUCCESS,recordDOS,page,size,count);
        }catch (Exception e){
            e.printStackTrace();
        }
        return PageEnvelop.getError(CommonContant.Message.GET_ERROR);
    }


    @GetMapping(value = "findHaveIdcardRecord")
    @ApiOperation(value = "查找居民有是否身份验证信息")
    public ObjEnvelop findHasIdcardRecord(@ApiParam(name = "patient",required = true, value = "居民id")
                                      @RequestParam(value = "patient") String patient) {
        try {
            BaseIdcardRecordDO idcardRecordDO = idcardRecordDao.findByPatient(patient);
            return ObjEnvelop.getSuccess("查询成功",idcardRecordDO!=null);
        }catch (Exception e){
            e.printStackTrace();
        }
        return ObjEnvelop.getError("查询失败");
    }

    @GetMapping(value = "findIdcardRecord")
    @ApiOperation(value = "查找居民身份验证信息")
    public ObjEnvelop findIdcardRecord(@ApiParam(name = "patient",required = true, value = "居民id")
                                          @RequestParam(value = "patient") String patient) {
        try {
            BaseIdcardRecordDO idcardRecordDO = idcardRecordDao.findByPatient(patient);
            return ObjEnvelop.getSuccess("查询成功",idcardRecordDO);
        }catch (Exception e){
            e.printStackTrace();
        }
        return ObjEnvelop.getError("查询失败");
    }

    @PostMapping(value = "createIdcardRecord")
    @ApiOperation(value = "创建")
    public ObjEnvelop create (
            @ApiParam(name = "jsonData", value = "Json数据", required = true)
            @RequestParam String jsonData) {
        try {
            BaseIdcardRecordDO idcardRecordDO = toEntity(jsonData, BaseIdcardRecordDO.class);
            String patient = idcardRecordDO.getPatient();
            synchronized (patient.intern()){
                BaseIdcardRecordDO old = idcardRecordDao.findByPatient(patient);
                if(old!=null){
                    return ObjEnvelop.getError("您已提交过身份验证，请勿重复提交");
                }
                if(StringUtils.isBlank(idcardRecordDO.getType())){
                    idcardRecordDO.setType("1");
                }
                idcardRecordDO.setCreateTime(new Date());
                idcardRecordDO.setStatus("0");
                idcardRecordDao.save(idcardRecordDO);
            }
            return ObjEnvelop.getSuccess("创建成功",idcardRecordDO);
        }catch (Exception e){
            e.printStackTrace();
        }
        return ObjEnvelop.getError("创建失败");
    }

}
