package com.yihu.jw.hospital.endpoint.archives;

import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.contant.CommonContant;
import com.yihu.jw.entity.base.inpatient.BaseInpatientRegistrationDO;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionDO;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionInfoDO;
import com.yihu.jw.entity.hospital.record.HospitalPatientInpatientRecordDO;
import com.yihu.jw.hospital.prescription.dao.PrescriptionDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionInfoDao;
import com.yihu.jw.hospital.record.service.HospitalPatientInpatientRecordService;
import com.yihu.jw.inpatient.service.BaseInpatientRegistrationService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


/**
 * Created by yeshijie on 2022/12/8.
 */
@RestController
@RequestMapping(value = "patient/archives")
@Api(value = "居民端-档案记录",  tags = {"居民端-档案记录"})
public class PatientArchivesEndpoint extends EnvelopRestEndpoint {

    @Autowired
    private HospitalPatientInpatientRecordService inpatientRecordService;
    @Autowired
    private BaseInpatientRegistrationService inpatientRegistrationService;
    @Autowired
    private PrescriptionDao prescriptionDao;
    @Autowired
    private PrescriptionInfoDao prescriptionInfoDao;

    @GetMapping(value = "findInpatientRecently")
    @ApiOperation(value = "获取最近一次住院记录")
    public Envelop findInpatientRecently (
            @ApiParam(name = "patient", value = "居民id")
            @RequestParam(value = "patient", required = true) String patient){
        try {
            HospitalPatientInpatientRecordDO recordDO = inpatientRecordService.findByPatientRecently(patient);
            if(recordDO.getStatus()==1){
                recordDO.setStatusName("住院中");
            }else {
                recordDO.setStatusName("已出院");
            }

            return ObjEnvelop.getSuccess(CommonContant.Message.GET_SUCCESS,recordDO);
        }catch (Exception e){
            e.printStackTrace();
        }
        return Envelop.getError(CommonContant.Message.GET_ERROR);
    }

    @GetMapping(value = "leaveHospitalDrugs")
    @ApiOperation(value = "出院带药")
    public Envelop leaveHospitalDrugs(@ApiParam(name = "patient", value = "patient", required = true)
                        @RequestParam(value = "patient", required = true)String patient,
                        @ApiParam(name = "startTime", value = "开始时间", required = true)
                        @RequestParam(value = "startTime", required = true)String startTime,
                        @ApiParam(name = "endTime", value = "结束时间", required = true)
                        @RequestParam(value = "endTime", required = true)String endTime,
                        @ApiParam(name = "page", value = "第几页，从1开始", required = true, defaultValue = "1")
                        @RequestParam(value = "page", required = true,defaultValue = "1")Integer page,
                        @ApiParam(name = "size", value = "每页分页大小", required = true, defaultValue = "10")
                        @RequestParam(value = "size", required = true,defaultValue = "10")Integer size){
        try {
            PageEnvelop pageEnvelop = inpatientRegistrationService.page(patient,null,startTime,endTime,page,size);
            for (Object o:pageEnvelop.getDetailModelList()){
                BaseInpatientRegistrationDO registrationDO = (BaseInpatientRegistrationDO)o;
                JSONObject data = new JSONObject();
                List<WlyyPrescriptionDO> pre = prescriptionDao.findByPatientCode(patient);
                if(pre.size()>0){
                    WlyyPrescriptionDO prescriptionDO = pre.get(0);
                    data.put("prescriptionId",prescriptionDO.getId());
                    List<WlyyPrescriptionInfoDO> infoDOS = prescriptionInfoDao.findByPrescriptionId(prescriptionDO.getId(),1);
                    data.put("infoDOS",infoDOS);
                }
            }
            return pageEnvelop;
        }catch (Exception e){
            return failedException2(e);
        }
    }
}
