package com.yihu.jw.hospital.endpoint.article;

import com.yihu.jw.article.service.BaseMenuManageService;
import com.yihu.jw.entity.base.menu.BaseMenuDictDO;
import com.yihu.jw.entity.hospital.article.KnowledgeArticleDoctorDO;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.base.BaseRequestMapping;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @author yeshijie on 2018/9/26.
 */
@RestController
@RequestMapping(value = BaseRequestMapping.MenuDict.PREFIX)
@Api(value = "app文章分类管理", tags = {"app文章分类管理"})
public class BaseMenuManageEndpoint extends EnvelopRestEndpoint {


    @Autowired
    private BaseMenuManageService menuService;


    @PostMapping(value = "setCollection")
    @ApiOperation(value = "设置文章状态")
    public ObjEnvelop setCollection(@ApiParam(name = "id", value = "文章id")
                                           @RequestParam(value = "id",required = true)String id,
                                           @ApiParam(name = "flag", value = "1收藏2阅读3点赞4分享")
                                           @RequestParam(value = "flag",required = true)Integer flag,
                                           @ApiParam(name = "status", value = "收藏（1收藏0未收藏）/阅读（1已读0未读）/点赞（1已赞0未赞）分享（1已分享0未分享）")
                                           @RequestParam(value = "status",required = true)Integer status,
                                           @ApiParam(name = "user", value = "用户code")
                                           @RequestParam(value = "user",required = true)String user,
                                           @ApiParam(name = "userType", value = "用户类型（1患者2医生）")
                                           @RequestParam(value = "userType",required = true)String userType)throws Exception {

        try {
            return success(menuService.setCollectionById(id,flag,status,user,userType));
        }catch (Exception e){
            return failedObjEnvelopException(e);
        }
    }

    @PostMapping(value = BaseRequestMapping.MenuDict.deleteMenuDict)
    @ApiOperation(value = "删除菜单")
    public Envelop deleteMenuDict(
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id") String id) {
        try {
            menuService.deletMenuDict(id);
            return success("删除成功");
        }catch (Exception e){
            return failedException(e);
        }
    }

    @PostMapping(value = BaseRequestMapping.MenuDict.createMenuDict)
    @ApiOperation(value = "新建或修改")
    public Envelop create (
            @ApiParam(name = "jsonData", value = "Json数据", required = true)
            @RequestParam String jsonData) {
        try {
            BaseMenuDictDO baseMenuDictDO = menuService.createOrUpdateMenu(jsonData,2);
            return success(baseMenuDictDO);
        } catch (Exception e) {
            return failedException(e);
        }
    }
    @PostMapping(value = BaseRequestMapping.MenuDict.findOneMenuDict)
    @ApiOperation(value = "根据id查询链接单条")
    public Envelop findOneMenuDict(
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id") String id,
            @ApiParam(name = "patient", value = "居民id", required = false)
            @RequestParam(value = "patient", required = false) String patient) {
        try {
            return success(menuService.findOneMenuDict(id,patient));
        }catch (Exception e){
            return failedException(e);
        }
    }
    @PostMapping(value = BaseRequestMapping.MenuDict.MOVE_DOWN)
    @ApiOperation(value = "下移")
    public Envelop moveDown(
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id") String id) {
        try {
            BaseMenuDictDO baseMenuDictDO = menuService.downMenu(id);
            return success(baseMenuDictDO);
        }catch (Exception e){
            return failedException(e);
        }

    }

    @PostMapping(value = BaseRequestMapping.MenuDict.MOVE_UP)
    @ApiOperation(value = "上移")
    public Envelop moveUp(
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id") String id) {
        try {
            BaseMenuDictDO baseMenuDictDO = menuService.upMenu(id);
            return success(baseMenuDictDO);
        }catch (Exception e){
            return failedException(e);
        }
    }

    @PostMapping(value = BaseRequestMapping.MenuDict.STATUS)
    @ApiOperation(value = "生效/失效")
    public Envelop status(
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id") String id,
            @ApiParam(name = "status", value = "1生效，0失效", required = true)
            @RequestParam(value = "status") Integer status) {
        try {
            menuService.updateStatus(id, status);
            return success("修改成功");
        }catch (Exception e){
            return failedException(e);
        }
    }
    @GetMapping(value = BaseRequestMapping.MenuDict.LIST)
    @ApiOperation(value = "查询菜单字典列表")
    public Envelop findMenuDictByKey(
            @ApiParam(name = "parentId", value = "parentId", required = false)
            @RequestParam(value = "parentId",required = false) String parentId,
            @ApiParam(name = "name", value = "name", required = false)
            @RequestParam(value = "name",required = false) String name,
            @ApiParam(name = "status", value = "status", required = false)
            @RequestParam(value = "status",required = false) Integer status) {
        try {
            return success(menuService.findMenuDictByKey(parentId, name,status,2));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @PostMapping(value = BaseHospitalRequestMapping.KnowledgeArticle.updateArticleUser)
    @ApiOperation(value = "患者更新文章状态")
    public Envelop updateArticleUser( @ApiParam(name = "flag", value = "1点赞2收藏3常用4分享5阅读")
                                      @RequestParam(value = "flag",required = false)Integer flag,
                                      @ApiParam(name = "articleId", value = "文章id")
                                      @RequestParam(value = "articleId",required = false)String articleId,
                                      @ApiParam(name = "userCode", value = "用户编码")
                                      @RequestParam(value = "userCode",required = false)String userCode,
                                      @ApiParam(name = "status", value = "对应的状态(0,1)")
                                      @RequestParam(value = "status",required = false)Integer status)throws Exception {

        try {
            return success(menuService.updateArticleUser(flag,articleId,userCode,status));
        }catch (Exception e){
            return failedException2(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.KnowledgeArticle.selectMenuList)
    @ApiOperation(value = "查询患者关注文章列表分类")
    public Envelop selectMenuList( @ApiParam(name = "flag", value = "1点赞2收藏3常用4分享5阅读")
                                      @RequestParam(value = "flag",required = false)Integer flag,
                                      @ApiParam(name = "userCode", value = "用户编码")
                                      @RequestParam(value = "userCode",required = false)String userCode,
                                      @ApiParam(name = "status", value = "对应的状态(0,1)")
                                      @RequestParam(value = "status",required = false)Integer status)throws Exception {

        try {
            return success(menuService.selectMenuList(userCode,flag,status));
        }catch (Exception e){
            return failedException2(e);
        }
    }


    @GetMapping(value = BaseHospitalRequestMapping.KnowledgeArticle.selectArticleListByCategory)
    @ApiOperation(value = "根据分类查找患者关注文章列表")
    public Envelop selectArticleListByCategory( @ApiParam(name = "categoryFirst", value = "一级分类")
                                       @RequestParam(value = "categoryFirst",required = false)String categoryFirst,
                                   @ApiParam(name = "categorySecond", value = "二级分类")
                                   @RequestParam(value = "categorySecond",required = false)String categorySecond,
                                   @ApiParam(name = "flag", value = "1点赞2收藏3常用4分享5阅读")
                                   @RequestParam(value = "flag",required = false)Integer flag,
                                   @ApiParam(name = "userCode", value = "用户编码")
                                   @RequestParam(value = "userCode",required = false)String userCode,
                                   @ApiParam(name = "status", value = "对应的状态(0,1)")
                                   @RequestParam(value = "status",required = false)Integer status,
                                   @ApiParam(name = "page", value = "页码")
                                       @RequestParam(value = "page",required = false)Integer page,
                                   @ApiParam(name = "size", value = "每一页大小")
                                       @RequestParam(value = "size",required = false)Integer size)throws Exception {

        try {
            return success(menuService.selectArticleListByCategory(categoryFirst,categorySecond,userCode,flag,status,page,size));
        }catch (Exception e){
            return failedException2(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.KnowledgeArticle.findArticleUserById)
    @ApiOperation(value = "查询患者关联的文章")
    public Envelop selectArticleListByCategory(@ApiParam(name = "userCode", value = "用户编码")
                                                @RequestParam(value = "userCode",required = false)String userCode,
                                                @ApiParam(name = "articleId", value = "文章id")
                                                @RequestParam(value = "articleId",required = false)String articleId)throws Exception {

        try {
            return success(menuService.findArticleUserById(userCode,articleId));
        }catch (Exception e){
            return failedException2(e);
        }
    }

    @PostMapping(value = BaseHospitalRequestMapping.KnowledgeArticle.insertArticleOperate)
    @ApiOperation(value = "新增医生文章/医生操作（评论、点赞、收藏）")
    public Envelop insert(@ApiParam(name = "json", value = "json实体")
                          @RequestParam(value = "json",required = false)String json)throws Exception {

        try {
            KnowledgeArticleDoctorDO knowledgeArticleDoctorDO = toEntity(json,KnowledgeArticleDoctorDO.class);
            return success(menuService.insert(knowledgeArticleDoctorDO));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @PostMapping(value = BaseHospitalRequestMapping.KnowledgeArticle.deleteArticleDoctorById)
    @ApiOperation(value = "取消点赞/收藏")
    public Envelop deleteArticleDoctorById(@ApiParam(name = "baseArticleDoctorId", value = "评论id")
                                            @RequestParam(value = "baseArticleDoctorId",required = false)String baseArticleDoctorId,
                                           @ApiParam(name = "relationCode", value = "文章id/医生id")
                                           @RequestParam(value = "relationCode",required = false)String relationCode,
                                           @ApiParam(name = "type", value = "类型2点赞3收藏")
                                               @RequestParam(value = "type",required = false)Integer type,
                                           @ApiParam(name = "user", value = "用户编码")
                                               @RequestParam(value = "user",required = false)String user)throws Exception {

        try {
            menuService.deleteArticleDoctorById(baseArticleDoctorId,relationCode,type,user);
            return success();
        }catch (Exception e){
            return failedException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.KnowledgeArticle.selectArticleDoctorById)
    @ApiOperation(value = "查询医生关联文章回复数据")
    public Envelop selectArticleDoctorById(@ApiParam(name = "user", value = "用户id")
                                               @RequestParam(value = "user",required = false)String user,
                                           @ApiParam(name = "id", value = "文章id")
                                           @RequestParam(value = "id",required = false)String id,
                                           @ApiParam(name = "page", value = "当前页")
                                           @RequestParam(value = "page",required = false)Integer page,
                                           @ApiParam(name = "size", value = "分页大小）")
                                           @RequestParam(value = "size",required = false)Integer size)throws Exception {

        try {
            return success(menuService.selectArticleDoctorById(user,id,1,page,size));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.KnowledgeArticle.selectArticleDoctorByArticleIdAndUser)
    @ApiOperation(value = "判断该文章当前医生是否点赞或者收藏")
    public Envelop selectArticleDoctorByArticleIdAndUser(@ApiParam(name = "id", value = "文章id/医生id")
                                                         @RequestParam(value = "id",required = false)String id,
                                                         @ApiParam(name = "user", value = "用户编码")
                                                         @RequestParam(value = "user",required = false)String user)throws Exception {

        try {
            return success(menuService.selectArticleDoctorByArticleIdAndUser(id,user));
        }catch (Exception e){
            return failedException(e);
        }
    }
}
