package com.yihu.jw.hospital.endpoint.article;

import com.yihu.jw.article.dao.KnowledgeArticleDictDao;
import com.yihu.jw.article.service.BaseMenuManageService;
import com.yihu.jw.entity.hospital.article.KnowledgeArticleBrowseDO;
import com.yihu.jw.entity.hospital.article.KnowledgeArticleDictDO;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.base.BaseRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping(value = BaseRequestMapping.MenuNologin.PREFIX)
@Api(value = "首页菜单管理免登录接口", description = "首页菜单管理免登录接口", tags = {"基础服务 - 首页菜单管理免登录接口"})
public class BaseMenuNoLoginEndpoint extends EnvelopRestEndpoint {
    @Autowired
    private BaseMenuManageService menuService;
    @Resource
    private KnowledgeArticleDictDao knowledgeArticleDictDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;

    @GetMapping(value = BaseRequestMapping.MenuNologin.findAllMenu)
    @ApiOperation(value = "查询首页列表")
    public Envelop findMenuShowByKey(
            @ApiParam(name = "parentId", value = "parentId", required = false)
            @RequestParam(value = "parentId",required = false) String parentId,
            @ApiParam(name = "name", value = "name", required = false)
            @RequestParam(value = "name",required = false) String name) {
        try {
            return success(menuService.findMenuShow());
        }catch (Exception e){
            return failedException(e);
        }
    }

    @GetMapping(value = BaseRequestMapping.MenuNologin.findMenuByParentId)
    @ApiOperation(value = "查询二级菜单")
    public Envelop findMenuByParentId(
            @ApiParam(name = "parentId", value = "parentId", required = false)
            @RequestParam(value = "parentId",required = false) String parentId,
            @ApiParam(name = "status", value = "status", required = false)
            @RequestParam(value = "status",required = false) Integer status) {
        try {
            return success(menuService.findByParentId(parentId,status));
        }catch (Exception e){
            return failedException(e);
        }
    }
    @GetMapping(value = BaseRequestMapping.MenuNologin.findArticleByMenuId)
    @ApiOperation(value = "查询文章")
    public Envelop findArticleByMenuid(
            @ApiParam(name = "menuId", value = "menuId", required = false)
            @RequestParam(value = "menuId",required = true) String menuId,
            @ApiParam(name = "title", value = "menuId", required = false)
            @RequestParam(value = "title",required = false) String title,
            @ApiParam(name = "page", value = "page", required = false)
            @RequestParam(value = "page",required = true) Integer page,
            @ApiParam(name = "pageSize", value = "pageSize", required = false)
            @RequestParam(value = "pageSize",required = true) Integer pageSize) {
        try {
            return success(menuService.findArticleByMenuId(menuId,title,page,pageSize,"0"));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @GetMapping(value = BaseRequestMapping.MenuNologin.queryById)
    @ApiOperation(value = "根据id查询文章")
    public Envelop queryByCode(
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id", required = true)String id) throws Exception{
        try {
            KnowledgeArticleDictDO articleDO = knowledgeArticleDictDao.findById(id).orElse(null);
            String recommendDoctor = articleDO.getRecommendDoctor();
            articleDO.setBrowseTotal(menuService.selectArticleBrowseByRelationCode(id,1));
            if (recommendDoctor!=null&&recommendDoctor!=""){
                recommendDoctor = recommendDoctor.replace(",","','");
                String sql = "select d.id,\n" +
                        "\td.NAME as name,\n" +
                        "\td.job_title_code AS jobTitleCode,\n" +
                        "\td.job_title_name AS jobTitleName,\n" +
                        "\tCONCAT(dh.dept_name) as deptName,\n" +
                        "\td.introduce as introduce,\n" +
                        "\td.expertise as expertise,\n" +
                        "\td.photo from base_doctor d LEFT JOIN base_doctor_hospital dh ON dh.doctor_code= d.id where d.id in ('"+recommendDoctor+"') GROUP BY d.id ";
                List<Map<String,Object>> mapList = jdbcTemplate.queryForList(sql);
                articleDO.setDoctorList(mapList);
            }
            return success(articleDO);
        }catch (Exception e){
            return failedException(e);
        }
    }


    @PostMapping(value = BaseRequestMapping.MenuNologin.insertBrowseDO)
    @ApiOperation(value = "新增用户浏览数据")
    public Envelop insertBrowseDO(
            @ApiParam(name = "json", value = "json浏览实体", required = true)
            @RequestParam(value = "json", required = true)String json) throws Exception{
        try {
            KnowledgeArticleBrowseDO browseDO = toEntity(json, KnowledgeArticleBrowseDO.class);
            return success(menuService.insertBrowseDO(browseDO));
        }catch (Exception e){
            return failedException(e);
        }
    }

}
