package com.yihu.jw.hospital.endpoint.base;

import com.yihu.jw.area.service.BaseCityService;
import com.yihu.jw.area.service.BaseStreetService;
import com.yihu.jw.dict.service.BaseDictJobTitleService;
import com.yihu.jw.dict.service.HospitalDeptService;
import com.yihu.jw.doctor.service.BaseDoctorHosService;
import com.yihu.jw.entity.base.area.BaseCityDO;
import com.yihu.jw.entity.base.dict.DictHospitalDeptDO;
import com.yihu.jw.entity.base.dict.DictJobTitleDO;
import com.yihu.jw.entity.base.doctor.BaseDoctorHospitalDO;
import com.yihu.jw.entity.base.org.BaseOrgDO;
import com.yihu.jw.hospital.prescription.service.entrance.YkyyEntranceService;
import com.yihu.jw.org.service.BaseOrgInfoService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.wlyy.service.WlyyBusinessService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * Created by cws on 2019/5/28.
 */
@RestController
@RequestMapping("internet/base" )
@Api(tags = "基础字典专用，用于各下拉框数据获取", description = "互联网医院")
public class BaseInfoEndpoint extends EnvelopRestEndpoint {


    @Value("${fastDFS.fastdfs_file_url}")
    private String fastdfs_file_url;
    @Autowired
    private BaseCityService cityService;

    @Autowired
    private HospitalDeptService hospitalDeptService;

    @Autowired
    private BaseDictJobTitleService baseDictJobTitleService;

    @Autowired
    private BaseDoctorHosService baseDoctorHosService;

    @Autowired
    private BaseOrgInfoService baseOrgInfoService;

    @Autowired
    private BaseStreetService baseStreetService;

    @Autowired
    private WlyyBusinessService wlyyBusinessService;

    @Autowired
    private YkyyEntranceService ykyyEntranceService;
    /**
     * 获取市列表信息
     * @return
     */
    @GetMapping(value = "/getCity")
    @ApiOperation(value = "获取市列表信息", notes = "基础数据获取")
    public Envelop getCity() throws Exception {
        try {
            List<BaseCityDO> res =  cityService.getAllCity();
            if(res != null && res.size() > 0 ){
                return  success(res);
            }else{
                return success("无法获取市列表信息！");
            }
        }catch (Exception e){
            return failedException(e);
        }
    }

    /**
     * 根据市编码，获取该市范围内所有的医院列表信息
     * @return
     */
    @GetMapping(value = "/getOrgByCity")
    @ApiOperation(value = "根据市编码，获取该市范围内所有的医院列表信息", notes = "基础数据获取")
    public Envelop getOrgByCity(
            @ApiParam(name = "cityCode", value = "市编码")
            @RequestParam(value = "cityCode", required = false) String cityCode,
            @ApiParam(name = "level", value = "等级医院，1等级医院，2.社区医院")
            @RequestParam(value = "level", required = false) Integer level) {

        try {
            List<BaseOrgDO> res =  baseOrgInfoService.getOrgByCity(cityCode,level);
            if(res != null && res.size() > 0 ){
                return  success(res);
            }else{
                return success("该市未维护医院信息！");
            }
        }catch (Exception e){
            return failedException(e);
        }
    }

    /**
     * 根据机构编码获取部门列表信息
     * @param orgCode
     * @return
     */
    @GetMapping(value = "/getDeptByOrgCode")
    @ApiOperation(value = "根据机构编码获取部门列表信息", notes = "基础数据获取")
    public Envelop getDeptByOrgCode(
            @ApiParam(name = "orgCode", value = "机构编码")
            @RequestParam(value = "orgCode", required = false) String orgCode,
            @ApiParam(name = "name", value = "科室名称")
            @RequestParam(value = "name", required = false) String name,
            @ApiParam(name = "consultDeptFlag", value = "是否是就诊科室,1为查询门诊，不传为全部")
            @RequestParam(value = "consultDeptFlag", required = false) String consultDeptFlag)  {

        try {
            List<DictHospitalDeptDO> res =  hospitalDeptService.getHosDeptByCode(orgCode,consultDeptFlag,name);
            if(res != null && res.size() > 0 ){
                return  success(res);
            }else{
                return success("无法获取该机构的部门列表信息！");
            }
        }catch (Exception e){
            return failedException(e);
        }
    }

    /**
     * 根据机构编码及科室信息获取科室内所有医生列表信息
     * @param orgCode
     * @return
     */
    @GetMapping(value = "/getDoctorByOrgAndDept")
    @ApiOperation(value = "根据机构编码及科室信息获取科室内所有医生列表信息", notes = "基础数据获取")
    public Envelop getDoctorByOrgAndDept(
            @ApiParam(name = "orgCode", value = "机构编码")
            @RequestParam(value = "orgCode", required = false) String orgCode,
            @ApiParam(name = "deptCode", value = "部门编码")
            @RequestParam(value = "deptCode", required = false) String deptCode) {

        try {
            List<BaseDoctorHospitalDO> res =  baseDoctorHosService.getDoctorByOrgAndDept(orgCode,deptCode);
            if(res != null && res.size() > 0 ){
                return  success(res);
            }else{
                return success("无法获取该机构科室下的医生信息！");
            }
        }catch (Exception e){
            return failedException(e);
        }
    }

    /**
     * 获取机构职称信息
     * @return
     */
    @GetMapping(value = "/getJobBySaasId")
    @ApiOperation(value = "获取机构职称信息", notes = "基础数据获取")
    public Envelop getJobBySaasId(
            @ApiParam(name = "saasId", value = "租户ID，当为空的情况下查询所有")
            @RequestParam(value = "saasId", required = false) String saasId)  {

        try {
            List<DictJobTitleDO> res= baseDictJobTitleService.getBySaasId(saasId);
            if(res != null && res.size() > 0 ){
                return  success(res);
            }else{
                return success("无法获取该机构的职称信息！");
            }
        }catch (Exception e){
            return failedException(e);
        }
    }

    //==================区域互联网======================================

    @GetMapping(value = "/findByTown")
    @ApiOperation(value = "获取街道信息", notes = "获取街道信息")
    public Envelop findByTown(@ApiParam(name = "town", value = "区级code")
                              @RequestParam(value = "town", required = true) String town)  {

        try {
            return success(baseStreetService.findByTown(town));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @GetMapping(value = "/findWlyyHospital")
    @ApiOperation(value = "同步医院信息", notes = "同步医院信息")
    public Envelop findWlyyHospital() throws Exception {
        return success(wlyyBusinessService.findWlyyHospital());
    }

    @GetMapping(value = "/findWlyyDept")
    @ApiOperation(value = "同步部门信息", notes = "同步部门信息")
    public Envelop findWlyyDept()  throws Exception{
        return success(wlyyBusinessService.findWlyyDept());
    }

    @GetMapping(value = "/findWlyyDoctor")
    @ApiOperation(value = "同步医生信息", notes = "同步医生信息")
    public Envelop findWlyyDoctor() throws Exception{
        return success(wlyyBusinessService.findWlyyDoctor());
    }

    //===============区域互联网===========================================

    //===============眼科中心信息同步======================================
    @GetMapping(value = "/updateYkyyDept")
    @ApiOperation(value = "同步科室数据", notes = "同步医生数据")
    public Envelop updateYkyyDept() throws Exception{
        return success(ykyyEntranceService.updateYkyyDept());
    }

    @GetMapping(value = "/updateYkyyDoctor")
    @ApiOperation(value = "同步医生信息", notes = "同步医生信息")
    public Envelop updateYkyyDoctor(@ApiParam(name = "idcard", value = "医生身份证")
                                        @RequestParam(value = "idcard", required = false)String idcard) throws Exception {
        return success(ykyyEntranceService.updateYkyyDoctor(idcard));
    }

    @GetMapping(value = "/getCFXX")
    @ApiOperation(value = "同步处方信息生成数据", notes = "同步处方信息生成数据")
    public Envelop getCFXX(@ApiParam(name = "brids", value = "病人id")
                           @RequestParam(value = "brids", required = true)String brids,
                           @ApiParam(name = "ysdm", value = "医生id")
                           @RequestParam(value = "ysdm", required = true)String ysdm,
                           @ApiParam(name = "startDate", value = "生成时间")
                           @RequestParam(value = "startDate", required = true)String startDate) throws Exception {
        return success(ykyyEntranceService.getCFXX(brids,ysdm,startDate));
    }

    @GetMapping(value = "/findPatientByMapingCode")
    @ApiOperation(value = "findPatientByMapingCode", notes = "findPatientByMapingCode")
    public Envelop findPatientByMapingCode(String code) throws Exception {
        return success(ykyyEntranceService.findPatientByMapingCode(code));
    }


    @GetMapping(value = "/updateYkTDoctor")
    @ApiOperation(value = "同步眼科通医生信息", notes = "同步眼科通医生信息")
    public Envelop updateYkTDoctor() throws Exception {
        return success(ykyyEntranceService.updateYkTDoctor(fastdfs_file_url));
    }
//    @GetMapping(value = "/test")
//    @ApiOperation(value = "test", notes = "test")
//    public Envelop test() throws Exception {
//        return success(ykyyEntranceService.test());
//    }

    //===============眼科中心信息同步======================================

}

