package com.yihu.jw.hospital.endpoint.booking;

import com.yihu.jw.contant.CommonContant;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.base.wx.WxTemplateConfigDO;
import com.yihu.jw.entity.hospital.booking.PatientReservation;
import com.yihu.jw.hospital.booking.service.PatientReservationService;
import com.yihu.jw.hospital.prescription.service.entrance.JwSmjkEntranceService;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.entity.ServiceException;
import com.yihu.jw.wechat.dao.WxTemplateConfigDao;
import com.yihu.jw.wechat.service.WechatUtilService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * Created by yeshijie on 2022/11/22.
 */
@RestController
@RequestMapping("patient/booking" )
@Api(tags = "居民预约挂号管理")
public class PatientBookingEndpoint extends EnvelopRestEndpoint {

    private static final Logger logger = LoggerFactory.getLogger(PatientBookingEndpoint.class);

    @Autowired
    private PatientReservationService patientReservationService;
    @Autowired
    private BasePatientDao patientDao;
    @Autowired
    private WechatUtilService wechatUtilService;
    @Autowired
    private JwSmjkEntranceService jwSmjkService;
    @Autowired
    private WxTemplateConfigDao templateConfigDao;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    protected HttpServletRequest request;
    @Value("${wechat.id}")
    private String wechatId;

    @PostMapping(value = "sign")
    @ApiOperation("签到")
    public Envelop sign(@ApiParam(name = "code", value = "预约code")
                        @RequestParam(value = "code", required = true) String code) {
        try {
            patientReservationService.sign(code,1);
           return success("签到成功");
        } catch (Exception e) {
            e.printStackTrace();
            return failedException2(e);
        }
    }

    @GetMapping(value = "findList")
    @ApiOperation("获取居民单日的预约挂号记录")
    public Envelop findList(@ApiParam(name = "patient", value = "居民id")
                            @RequestParam(value = "patient", required = true) String patient) {
        try {
            return ListEnvelop.getSuccess(CommonContant.Message.GET_SUCCESS, patientReservationService.findList(patient));
        } catch (Exception e) {
            e.printStackTrace();
            return failedException2(e);
        }
    }

    @PostMapping(value = "CancelOrder")
    @ApiOperation("取消挂号单")
    public Envelop CancelOrder(@ApiParam(name = "orderId", value = "订单id", defaultValue = "48")
                              @RequestParam(value = "orderId", required = true) String orderId,
                               @ApiParam(name = "ssc", value = "社保卡号", defaultValue = "48")
                              @RequestParam(value = "ssc", required = true) String ssc) {
        try {
            //获取订单信息
            boolean re = jwSmjkService.CancelOrder(orderId,ssc);

            if (re) {
                PatientReservation obj = patientReservationService.findByCode(orderId);
                if(obj==null){//不是在i健康预约
                    return success("取消挂号单成功！");
                }
                //更新状态
                patientReservationService.patientCancelOrder(orderId,obj.getPatient());//"9aa5c557e06a4324911487a035195545"
                //微信消息
                BasePatientDO p = patientDao.findById(obj.getPatient()).orElse(null);
                WxTemplateConfigDO templateConfig = templateConfigDao.findByWechatIdAndTemplateNameAndScene(wechatId,"template_appoint_failed","qxgh");
                String remark = templateConfig.getRemark();
                remark = remark.replace("key1",(obj.getName()==null?"":obj.getName())).replace("key2",(obj.getStartTime()==null?"":obj.getStartTime()+""))
                        .replace("br","\n");
                if (StringUtils.isNotEmpty(p.getOpenid())) {
                    WxTemplateConfigDO newConfig = new WxTemplateConfigDO();
                    BeanUtils.copyProperties(templateConfig, newConfig);
                    newConfig.setRemark(remark);
                    //判断是否判定openId,有没有发则查找家人发送
                    if(StringUtils.isNotBlank(p.getOpenid())){
                        wechatUtilService.putWxMsg(p.getOpenid(),newConfig);
                    }
                }
                return success("取消挂号单成功！");
            } else {
                return Envelop.getError("取消挂号单失败！");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return failedException2(e);
        }
    }

    /**
     * ======================预约挂号 start=====================
     */
    private void recordRequestLog(String methodName){
        String uri = request.getRequestURI();//返回请求行中的资源名称
        String url = request.getRequestURL().toString();//获得客户端发送请求的完整url
        String ip = request.getRemoteAddr();//返回发出请求的IP地址
        String params = "";
        if("GET".equals(request.getMethod())){
            params = request.getQueryString();//返回请求行中的参数部分
        }else if("POST".equals(request.getMethod())){
            Map<String,String> map = new HashMap<>();

            Enumeration paraNames = request.getParameterNames();
            while (paraNames.hasMoreElements()){
                String paraName = (String) paraNames.nextElement();
                String[] paraValues = request.getParameterValues(paraName);
                if(paraValues.length ==1 ){
                    if(paraValues[0].length() !=0){
                        map.put(paraName,paraValues[0]);
                    }
                }
            }

            Set<Map.Entry<String,String>> set = map.entrySet();
            for (Map.Entry entry:set){
                params =  params +entry.getKey()+":"+entry.getValue()+"--";
            }
        }

        String host=request.getRemoteHost();//返回发出请求的客户机的主机名
        int port =request.getRemotePort();//返回发出请求的客户机的端口号。
        logger.info("ip:"+ip+",port:"+port+",host:"+host+",api:"+methodName+",uri:"+uri+",url:"+url+",params:"+params);
    }


    @PostMapping(value = "GetOrgDepListToMysql")
    @ApiOperation("从数据库获取科室接口")
    public Envelop GetOrgDepListToMysql(
            @ApiParam(name = "filter", value = "过滤条件", defaultValue = "")
            @RequestParam(value = "filter", required = false) String filter) {
        try {
            return ListEnvelop.getSuccess("获取科室列表成功",jwSmjkService.GetOrgDepListToMysql(null,filter));
        } catch (Exception e) {
            return failedException2(e);
        }
    }

    /**
     * 获取转诊预约医生列表
     */
    @RequestMapping(value = "/RegDeptSpeDoctorList",method = RequestMethod.POST)
    @ApiOperation("获取转诊预约医生列表")
    public String getRegDeptSpeDoctorList(
            @ApiParam(name="hospitalId",value="医院编码") @RequestParam(name="hospitalId",required = false) String hospitalId,
            @ApiParam(name="hosDeptId",value="科室编码",defaultValue = "1040610") @RequestParam(name="hosDeptId",required = true) String hosDeptId,
            @ApiParam(name = "filter", value = "过滤条件")
            @RequestParam(value = "filter", required = false) String filter,
            @ApiParam(name = "page", value = "第几页")
            @RequestParam(value = "page", required = false) Integer page,
            @ApiParam(name = "size", value = "每页记录数")
            @RequestParam(value = "size", required = false) Integer size){

        try {
            //7天排班的医生列表
            String list = jwSmjkService.GetDoctorList(hospitalId,hosDeptId);
            return list;
        } catch (Exception ex) {
            ex.printStackTrace();
            return error(-1,"获取转诊预约医生列表列表失败");
        }
    }

    @RequestMapping(value = "GetDoctorArrangeSimple", method = RequestMethod.POST)
    @ApiOperation("获取医生排班接口（一级）")
    public Envelop GetDoctorArrangeSimple(@ApiParam(name = "hospitalId", value = "医院ID", defaultValue = "350211A1001")
                                         @RequestParam(value = "hospitalId", required = false) String hospitalId,
                                         @ApiParam(name = "hosDeptId", value = "科室ID", defaultValue = "1010210")
                                         @RequestParam(value = "hosDeptId", required = true) String hosDeptId,
                                         @ApiParam(name = "doctorId", value = "医生ID", defaultValue = "03101")
                                         @RequestParam(value = "doctorId", required = true) String doctorId) {
        try {
            recordRequestLog("GetDoctorArrangeSimple");
            String list = jwSmjkService.GetDoctorArrangeSimple(hospitalId,hosDeptId,doctorId);
            return ObjEnvelop.getSuccess("",list);
        } catch (Exception e) {
            return failedException2(e);
        }
    }

    @RequestMapping(value = "GetDoctorInfo", method = RequestMethod.POST)
    @ApiOperation("根据医生编码获取医生详细信息")
    public String GetDoctorInfo(@ApiParam(name = "doctorId", value = "医生id", defaultValue = "07101")
                                @RequestParam(value = "doctorId", required = true) String doctorId,
                                @ApiParam(name = "hospitalId", value = "医院id", defaultValue = "350211A1001")
                                @RequestParam(value = "hospitalId", required = false) String hospitalId,
                                @ApiParam(name = "hosDeptId", value = "科室id", defaultValue = "1040610")
                                @RequestParam(value = "hosDeptId", required = true) String hosDeptId) {
        try {
            recordRequestLog("GetDoctorInfo10");
            JSONObject result  = jwSmjkService.GetDoctorInfoTen(hospitalId,hosDeptId,doctorId);
            return result.toString();
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1,CommonContant.Message.GET_ERROR);
        }
    }

    @RequestMapping(value = "GetDoctorArrange", method = RequestMethod.POST)
    @ApiOperation("获取医生排班接口（包含排班详细）")
    public String GetDoctorArrange(@ApiParam(name = "hospitalId", value = "医院ID", defaultValue = "350211G1102")
                                   @RequestParam(value = "hospitalId", required = false) String hospitalId,
                                   @ApiParam(name = "hosDeptId", value = "科室ID", defaultValue = "3020001")
                                   @RequestParam(value = "hosDeptId", required = true) String hosDeptId,
                                   @ApiParam(name = "doctorId", value = "医生ID", defaultValue = "AA2")
                                   @RequestParam(value = "doctorId", required = true) String doctorId) {
        try {
            recordRequestLog("GetDoctorArrange");
            String list = jwSmjkService.GetDoctorArrange(hospitalId,hosDeptId,doctorId);
            return list;
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1,CommonContant.Message.GET_ERROR);
        }
    }

    @RequestMapping(value = "CreateOrder", method = RequestMethod.POST)
    @ApiOperation("创建挂号单")
    public Envelop CreateOrder(@ApiParam(name = "hospitalId", value = "医院ID", defaultValue = "350211A1002")
                              @RequestParam(value = "hospitalId", required = false) String hospitalId,
                              @ApiParam(name = "hospitalName", value = "医院名称", defaultValue = "马銮湾医院")
                              @RequestParam(value = "hospitalName", required = false) String hospitalName,
                              @ApiParam(name = "hosDeptId", value = "科室ID", defaultValue = "1040610")
                              @RequestParam(value = "hosDeptId", required = true) String hosDeptId,
                              @ApiParam(name = "hosDeptName", value = "医院科室名称", defaultValue = "儿二科")
                              @RequestParam(value = "hosDeptName", required = true) String hosDeptName,
                              @ApiParam(name = "doctorId", value = "医生ID", defaultValue = "07101")
                              @RequestParam(value = "doctorId", required = true) String doctorId,
                              @ApiParam(name = "doctorName", value = "医生姓名", defaultValue = "林素莲")
                              @RequestParam(value = "doctorName", required = true) String doctorName,
                              @ApiParam(name = "arrangeDate", value = "排班信息", defaultValue = "{\"sectionType\":\"a\",\"startTime\":\"2016-09-02 08:20:00\",\"endTime\":\"2016-09-02 08:30:00\"}")
                              @RequestParam(value = "arrangeDate", required = true) String arrangeDate,
                              @ApiParam(name = "patient", value = "患者代码", defaultValue = "00258f46c4fe11ec8771005056ab2351")
                              @RequestParam(value = "patient", required = true) String patient,
                              @ApiParam(name = "patientName", value = "患者姓名", defaultValue = "孙杨")
                              @RequestParam(value = "patientName", required = true) String patientName,
                              @ApiParam(name = "cardNo", value = "身份证号码", defaultValue = "35052419880511553X")
                              @RequestParam(value = "cardNo", required = true) String cardNo,
                              @ApiParam(name = "clinicCard", value = "市民卡号", defaultValue = "D57117706")
                              @RequestParam(value = "clinicCard", required = true) String clinicCard,
                              @ApiParam(name = "patientPhone", value = "患者手机", defaultValue = "13950116510")
                              @RequestParam(value = "patientPhone", required = true) String patientPhone,
                              @ApiParam(name = "key", value = "验证码key")
                              @RequestParam(value = "key", required = false) String key,
                              @ApiParam(name = "text", value = "text")
                              @RequestParam(value = "text", required = false) String text) {
//        try{
//            boolean pass = false;
//            key = key.replaceFirst(":",":"+patient+"_");
//
//            String captcha = redisTemplate.opsForValue().get(key);
//            if (captcha != null && captcha.equals(text.toLowerCase())){
//                pass = true;
//                redisTemplate.delete(key);
//            }
//
//            if(!pass){
//                return Envelop.getError("验证码校验失败");
//            }
//        } catch (Exception e){
//            return failedException2(e);
//        }

        try {
            recordRequestLog("CreateOrder");

            if (StringUtils.isEmpty(patientName)) {
                return Envelop.getError("未设置姓名！");
            }
            if (StringUtils.isEmpty(cardNo)) {
                return Envelop.getError("未设置身份证号！");
            }
            if (StringUtils.isEmpty(clinicCard)) {
                return Envelop.getError("未设置社保卡号！");
            }
            if (StringUtils.isEmpty(patientPhone)) {
                return Envelop.getError("未设置手机号码！");
            }
            String orderCode = jwSmjkService.CreateOrder(hospitalId,hospitalName,hosDeptId,hosDeptName,doctorId,doctorName,arrangeDate,patient,patientName,cardNo,clinicCard,patientPhone);
            //预约发送微信消息
            PatientReservation obj = patientReservationService.findByCode(orderCode);
            if (obj != null) {
                BasePatientDO p = patientDao.findById(obj.getPatient()).orElse(null);

//                if(StringUtils.isNotBlank(obj.getDoctor())){
//                    consultService.sendMucMessageBySingnType(obj.getDoctor(),obj.getDoctorName(),patient,"我已成功为您预约：" + DateUtil.dateToStrLong(obj.getStartTime()) + "，" + hospitalName + hosDeptName + doctorName + "医生的号源。您可直接前往医院就诊</br><a name='guahao' href='javascript:void(0)' data-id='" + obj.getId() + "'>点击查看详情</a>","1",p.getName());
//                }

                WxTemplateConfigDO templateConfig = templateConfigDao.findByWechatIdAndTemplateNameAndScene(wechatId,"template_appoint_success","yyghcg");
                String remark = templateConfig.getRemark();
                String first = templateConfig.getFirst();
                first = first.replace("key1",p.getName());
                // 推送消息给微信端
                if(StringUtils.isNotBlank(p.getOpenid())){
                    // 添加到发送队列
                    WxTemplateConfigDO newConfig = new WxTemplateConfigDO();
                    BeanUtils.copyProperties(templateConfig, newConfig);
                    newConfig.setRemark(remark);
                    newConfig.setFirst(first);
                    newConfig.setKeyword1(DateUtil.dateToStrLong(obj.getStartTime()));
                    newConfig.setKeyword2((obj.getOrgName() + " " + obj.getDeptName()+ " " + obj.getDoctorName()));
                    wechatUtilService.putWxMsg(p.getOpenid(),newConfig);
                }
                //发送短信消息


                return ObjEnvelop.getSuccess( "创建挂号单成功！",obj.getId());
            } else {
                return Envelop.getError( "创建挂号单失败！");
            }
        } catch (ServiceException se) {
            return Envelop.getError(se.getMessage());
        } catch (Exception e) {
            return failedException2(e);
        }
    }

    @RequestMapping(value = "GetRegList", method = RequestMethod.POST)
    @ApiOperation("获取患者预约信息列表接口--患者端")
    public String GetRegList(@ApiParam(name = "patient", value = "患者编号", defaultValue = "00258f46c4fe11ec8771005056ab2351")
                             @RequestParam(value = "patient", required = false) String patient) {
        try {
            recordRequestLog("GetRegList");

            String list = jwSmjkService.GetRegList(patient);
            return list;
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1,CommonContant.Message.GET_ERROR);
        }
    }

}
