package com.yihu.jw.hospital.endpoint.complaint;

import com.yihu.jw.complaint.BaseComplaintService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/complaint" )
@Api(tags = "投诉管理员管理", description = "互联网医院")
public class BaseComplaintEndPoint  extends EnvelopRestEndpoint {
    @Autowired
    private BaseComplaintService baseComplaintService;
    /**
     * 新增修改投诉类型字典
     * @return
     */
    @GetMapping(value = "/createOrUpdateDict")
    @ApiOperation(value = "新增修改投诉类型字典", notes = "新增修改投诉类型字典")
    public Envelop createOrUpdateDict(@ApiParam(name = "id", value = "id", required = false)
                                      @RequestParam(value = "id", required = false) String id,
                                      @ApiParam(name = "name", value = "name", required = false)
                                      @RequestParam(value = "name", required = false) String name) throws Exception {
        try {
            return success(baseComplaintService.createOrUpdateDict(id,name)) ;
        }catch (Exception e){
            return failedException(e);
        }
    }
    /**
     * 删除字典
     * @return
     */
    @GetMapping(value = "/deleteComplaintDict")
    @ApiOperation(value = "删除字典", notes = "删除字典")
    public Envelop deleteComplaintDict(@ApiParam(name = "id", value = "id", required = false)
                                      @RequestParam(value = "id", required = false) String id) throws Exception {
        try {
            return success(baseComplaintService.deleteDict(id)) ;
        }catch (Exception e){
            return failedException(e);
        }
    }
    /**
     * 查询单条字典
     * @return
     */
    @GetMapping(value = "/findOneComplaintDict")
    @ApiOperation(value = "查询单条字典", notes = "查询单条字典")
    public Envelop findOneComplaintDict(@ApiParam(name = "id", value = "id", required = false)
                                       @RequestParam(value = "id", required = false) String id) throws Exception {
        try {
            return success(baseComplaintService.findOneDict(id)) ;
        }catch (Exception e){
            return failedException(e);
        }
    }
    /**
     * 查询字典列表
     * @return
     */
    @GetMapping(value = "/findComplaintDictList")
    @ApiOperation(value = "查询字典列表", notes = "查询字典列表")
    public Envelop findComplaintDictList(@ApiParam(name = "name", value = "name", required = false)
                                        @RequestParam(value = "name", required = false) String name) throws Exception {
        try {
            return success(baseComplaintService.findAll(name)) ;
        }catch (Exception e){
            return failedException(e);
        }
    }
    /**
     * 新增修改投诉管理医生
     * @return
     */
    @GetMapping(value = "/createOrUpdateComplainDoctor")
    @ApiOperation(value = "新增修改投诉管理医生", notes = "新增修改投诉管理医生")
    public Envelop createOrUpdateComplainDoctor(@ApiParam(name = "id", value = "id", required = false)
                                      @RequestParam(value = "id", required = false) String id,
                                      @ApiParam(name = "complaintId", value = "complaintId", required = false)
                                      @RequestParam(value = "complaintId", required = false) String complaintId,
                                                @ApiParam(name = "doctorList", value = "doctorList", required = false)
                                                    @RequestParam(value = "doctorList", required = false) String doctorList,
                                                @ApiParam(name = "deptCode", value = "deptCode", required = false)
                                                    @RequestParam(value = "deptCode", required = false) String deptCode) throws Exception {
        try {
            baseComplaintService.createOrUpdateComplainDoctor( id, complaintId , doctorList);
            return success() ;
        }catch (Exception e){
            return failedException(e);
        }
    }




    /**
     * 删除医生
     * @return
     */
    @GetMapping(value = "/deleteComplaintDoctor")
    @ApiOperation(value = "删除医生", notes = "删除医生")
    public Envelop deleteComplaintDoctor(@ApiParam(name = "id", value = "id", required = false)
                                       @RequestParam(value = "id", required = false) String id) throws Exception {
        try {
            return success(baseComplaintService.deleteDoctorComplain(id)) ;
        }catch (Exception e){
            return failedException(e);
        }
    }
    /**
     * 查询管理医生单条
     * @return
     */
    @GetMapping(value = "/findOneComplaintDoctor")
    @ApiOperation(value = "查询管理医生单条", notes = "查询管理医生单条")
    public Envelop findOneComplaintDoctor(@ApiParam(name = "id", value = "id", required = false)
                                        @RequestParam(value = "id", required = false) String id) throws Exception {
        try {
            return success(baseComplaintService.findOneDoctorComplain(id)) ;
        }catch (Exception e){
            return failedException(e);
        }
    }
    /**
     * 查询医生列表
     * @return
     */
    @GetMapping(value = "/findComplaintDoctorList")
    @ApiOperation(value = "查询医生列表", notes = "查询医生列表")
    public MixEnvelop findComplaintDoctorList(@ApiParam(name = "doctorName", value = "doctorName", required = false)
                                         @RequestParam(value = "doctorName", required = false) String doctorName,
                                              @ApiParam(name = "doctor", value = "doctor", required = false)
                                              @RequestParam(value = "doctor", required = false) String doctor,
                                              @ApiParam(name = "deptName", value = "deptName", required = false)
                                           @RequestParam(value = "deptName", required = false) String deptName,
                                              @ApiParam(name = "complaintType", value = "complaintType", required = false)
                                               @RequestParam(value = "complaintType", required = false) String complaintType,
                                              @ApiParam(name = "page", value = "page", required = false)
                                               @RequestParam(value = "page", required = false) Integer page,
                                              @ApiParam(name = "pageSize", value = "pageSize", required = false)
                                               @RequestParam(value = "pageSize", required = false) Integer pageSize
                                           ) throws Exception {
        try {
            return baseComplaintService.findComplaintDoctor(doctor,doctorName,deptName,complaintType,page,pageSize) ;
        }catch (Exception e){
            return failedMixEnvelopException(e);
        }
    }

    /**
     * 患者投诉
     * @return
     */
    @PostMapping(value = "/patiemtComplaint")
    @ApiOperation(value = "患者投诉", notes = "患者投诉")
    public Envelop patiemtComplaint(@ApiParam(name = "json", value = "json", required = false)
                                          @RequestParam(value = "json", required = false) String json) throws Exception {
        try {
            return success(baseComplaintService.patiemtComplaint(json)) ;
        }catch (Exception e){
            return failedException(e);
        }
    }
    /**
     * 转交投诉
     * @return
     */
    @PostMapping(value = "/passTo")
    @ApiOperation(value = "转交投诉", notes = "转交投诉")
    public Envelop passTo(@ApiParam(name = "id", value = "id", required = false)
                                    @RequestParam(value = "id", required = false) String id,
                          @ApiParam(name = "complaintId", value = "complaintId", required = false)
                          @RequestParam(value = "complaintId", required = false) String complaintId,
                          @ApiParam(name = "doctor", value = "doctor", required = false)
                              @RequestParam(value = "doctor", required = false) String doctor,
                          @ApiParam(name = "reason", value = "reason", required = false)
                              @RequestParam(value = "reason", required = false) String reason,
                          @ApiParam(name = "operator", value = "operator", required = false)
                              @RequestParam(value = "operator", required = false) String operator) throws Exception {
        try {
            return success(baseComplaintService.passTo(id,complaintId,doctor,operator,reason)) ;
        }catch (Exception e){
            return failedException(e);
        }
    }

    /**
     * 接受
     * @return
     */
    @GetMapping(value = "/doctorAccept")
    @ApiOperation(value = "接受投诉", notes = "接受投诉")
    public Envelop doctorAccept(@ApiParam(name = "id", value = "id", required = false)
                          @RequestParam(value = "id", required = false) String id,
                          @ApiParam(name = "doctor", value = "doctor", required = false)
                          @RequestParam(value = "doctor", required = false) String doctor) throws Exception {
        try {
            return success(baseComplaintService.doctorAccept(id,doctor)) ;
        }catch (Exception e){
            return failedException(e);
        }
    }
    /**
     * 反馈投诉
     * @return
     */
    @GetMapping(value = "/dealComplaint")
    @ApiOperation(value = "反馈投诉", notes = "反馈投诉")
    public Envelop dealComplaint(@ApiParam(name = "id", value = "id", required = false)
                                @RequestParam(value = "id", required = false) String id,
                                 @ApiParam(name = "operator", value = "operator", required = false)
                                 @RequestParam(value = "operator", required = false) String operator,
                                 @ApiParam(name = "content", value = "content", required = false)
                                     @RequestParam(value = "content", required = false) String content,
                                 @ApiParam(name = "imgUrl", value = "imgUrl", required = false)
                                     @RequestParam(value = "imgUrl", required = false) String imgUrl,
                                 @ApiParam(name = "deptCode", value = "deptCode", required = false)
                                     @RequestParam(value = "deptCode", required = false) String deptCode,
                                @ApiParam(name = "doctor", value = "doctor", required = false)
                                @RequestParam(value = "doctor", required = false) String doctor) throws Exception {
        try {
            return success(baseComplaintService.dealComplaint(id,operator,content,imgUrl,deptCode)) ;
        }catch (Exception e){
            return failedException(e);
        }
    }

    /**
     * 反馈投诉
     * @return
     */
    @GetMapping(value = "/findComplaintList")
    @ApiOperation(value = "查询投诉建议列表", notes = "查询投诉建议列表")
    public Envelop findComplaintList(@ApiParam(name = "doctor", value = "doctor", required = false)
                                 @RequestParam(value = "doctor", required = false) String doctor,
                                 @ApiParam(name = "status", value = "status", required = false)
                                 @RequestParam(value = "status", required = false) String status,
                                     @ApiParam(name = "role", value = "role", required = false)
                                         @RequestParam(value = "role", required = false) String role,
                                 @ApiParam(name = "patientName", value = "patientName", required = false)
                                 @RequestParam(value = "patientName", required = false) String patientName,
                                 @ApiParam(name = "patientId", value = "patientId", required = false)
                                 @RequestParam(value = "patientId", required = false) String patientId,
                                 @ApiParam(name = "keyWord", value = "keyWord", required = false)
                                 @RequestParam(value = "keyWord", required = false) String keyWord,
                                     @ApiParam(name = "complaintId", value = "complaintId", required = false)
                                         @RequestParam(value = "complaintId", required = false) String complaintId,
                                     @ApiParam(name = "endTime", value = "endTime", required = false)
                                         @RequestParam(value = "endTime", required = false) String endTime,
                                     @ApiParam(name = "page", value = "page", required = false)
                                         @RequestParam(value = "page", required = false) Integer page,
                                     @ApiParam(name = "pageSize", value = "pageSize", required = false)
                                         @RequestParam(value = "pageSize", required = false) Integer pageSize,
                                 @ApiParam(name = "startTime", value = "startTime", required = false)
                                 @RequestParam(value = "startTime", required = false) String startTime) throws Exception {
        try {
            if (!StringUtils.isNotBlank(doctor)){
                doctor=getUID();
            }
            return success(baseComplaintService.findComplaintList(role, doctor, status, patientName, patientId, keyWord,complaintId,
                     startTime, endTime, page , pageSize)) ;
        }catch (Exception e){
            return failedException(e);
        }
    }
    /**
     * 查询投诉建议单挑
     * @return
     */
    @GetMapping(value = "/findOnePaitentComplaint")
    @ApiOperation(value = "查询投诉建议单条", notes = "查询投诉建议单条")
    public Envelop findOnePaitentComplaint(@ApiParam(name = "id", value = "id", required = false)
                                          @RequestParam(value = "id", required = false) String id) throws Exception {
        try {
            return success(baseComplaintService.findOnePaitentComplaint(id)) ;
        }catch (Exception e){
            return failedException(e);
        }
    }

    /**
     * 居民反馈查询
     * @return
     */
    @GetMapping(value = "/findPatientComplaintList")
    @ApiOperation(value = "查询投诉建议列表", notes = "查询投诉建议列表")
    public Envelop findPatientComplaintList(
                                     @ApiParam(name = "status", value = "status", required = false)
                                     @RequestParam(value = "status", required = false) String status,
                                     @ApiParam(name = "patientName", value = "patientName", required = false)
                                     @RequestParam(value = "patientName", required = false) String patientName,
                                     @ApiParam(name = "patientId", value = "patientId", required = false)
                                     @RequestParam(value = "patientId", required = false) String patientId,
                                     @ApiParam(name = "complaintType", value = "complaintType", required = false)
                                     @RequestParam(value = "complaintType", required = false) String complaintType,
                                     @ApiParam(name = "endTime", value = "endTime", required = false)
                                     @RequestParam(value = "endTime", required = false) String endTime,
                                     @ApiParam(name = "page", value = "page", required = false)
                                     @RequestParam(value = "page", required = false) Integer page,
                                     @ApiParam(name = "pageSize", value = "pageSize", required = false)
                                     @RequestParam(value = "pageSize", required = false) Integer pageSize,
                                     @ApiParam(name = "startTime", value = "startTime", required = false)
                                     @RequestParam(value = "startTime", required = false) String startTime) throws Exception {
        try {
            return success(baseComplaintService.findPatientComplaintList(  status, patientName, patientId,
                    startTime, endTime,complaintType, page , pageSize)) ;
        }catch (Exception e){
            return failedException(e);
        }
    }


    /**
     * 查询投诉建议单挑
     * @return
     */
    @GetMapping(value = "/findComplaintCount")
    @ApiOperation(value = "查询投诉建议数量", notes = "查询投诉建议单挑")
    public Envelop findComplaintCount(@ApiParam(name = "doctor", value = "doctor", required = false)
                                      @RequestParam(value = "doctor", required = false) String doctor,
                                      @ApiParam(name = "role", value = "role", required = false)
                                      @RequestParam(value = "role", required = false) String role) throws Exception {
        try {
            return success(baseComplaintService.findComplaintCount(doctor,role)) ;
        }catch (Exception e){
            return failedException(e);
        }
    }
    /**
     * 查询投诉建议单挑
     * @return
     */
    @GetMapping(value = "/findComplaintCountByPatient")
    @ApiOperation(value = "患者查询投诉建议数量", notes = "患者查询投诉建议数量")
    public Envelop findComplaintCountByPatient(@ApiParam(name = "patient", value = "patient", required = false)
                                      @RequestParam(value = "patient", required = false) String patient) throws Exception {
        try {
            return success(baseComplaintService.findComplaintCountPatient(patient)) ;
        }catch (Exception e){
            return failedException(e);
        }
    }

    /**
     * 居民撤销反馈
     * @return
     */
    @GetMapping(value = "/cancelPatientComplaint")
    @ApiOperation(value = "居民撤销反馈", notes = "居民撤销反馈")
    public Envelop cancelPatientComplaint(@ApiParam(name = "id", value = "id", required = false)
                                               @RequestParam(value = "id", required = false) String id,
                                          @ApiParam(name = "patient", value = "patient", required = false)
                                          @RequestParam(value = "patient", required = false) String patient) throws Exception {
        try {
            return success(baseComplaintService.cancelPaitentComplaint(id,patient)) ;
        }catch (Exception e){
            return failedException(e);
        }
    }

    /**
     * 查询进度
     * @return
     */
    @GetMapping(value = "/findComplaintOperateLog")
    @ApiOperation(value = "查询反馈进度", notes = "查询反馈进度")
    public Envelop findComplaintOperateLog(@ApiParam(name = "id", value = "id", required = false)
                                          @RequestParam(value = "id", required = false) String id) throws Exception {
        try {
            return success(baseComplaintService.findComplaintOperateLog(id)) ;
        }catch (Exception e){
            return failedException(e);
        }
    }
}
