package com.yihu.jw.hospital.endpoint.config;

import com.yihu.jw.contant.CommonContant;
import com.yihu.jw.doctor.service.BaseDoctorInfoService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @author zmk
 * @vsrsion 1.0
 * Created at 2019/12/9
 */
@RestController
@RequestMapping(value = BaseHospitalRequestMapping.DoctorSetting.PREFIX)
@Api(tags = "互联网医院后台医生服务配置", description = "互联网医院后台医生服务配置")
public class DoctorServiceEndPoint extends EnvelopRestEndpoint {

    @Autowired
    private BaseDoctorInfoService baseDoctorService;



    @GetMapping(value = BaseHospitalRequestMapping.DoctorSetting.queryList)
    @ApiOperation(value = "获取分页列表")
    public Envelop queryList(
            @ApiParam(name = "city", value = "市区")
            @RequestParam(value = "city", required = false) String city,
            @ApiParam(name = "hospital", value = "机构")
            @RequestParam(value = "hospital", required = false) String hospital,
            @ApiParam(name = "status", value = "状态")
            @RequestParam(value = "status", required = false) String status,
            @ApiParam(name = "name", value = "科室或医生名称")
            @RequestParam(value = "name", required = false) String name,
            @ApiParam(name = "type", value = "咨询类型")
            @RequestParam(value = "type", required = false) String type,
            @ApiParam(name = "typeCode", value = "分部，无分部不传")
            @RequestParam(value = "typeCode", required = false)String typeCode,
            @ApiParam(name = "dept", value = "部门")
            @RequestParam(value = "dept", required = false)String dept,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1")
            @RequestParam(value = "page") int page,
            @ApiParam(name = "pageSize", value = "页码", required = true, defaultValue = "10")
            @RequestParam(value = "pageSize") int pageSize) throws Exception {

        try{
            return baseDoctorService.queryList(city,hospital,status,name,type,typeCode,dept,page,pageSize);
        }catch (Exception e){
            return failedException(e);
        }

    }


    @GetMapping(value = BaseHospitalRequestMapping.DoctorSetting.queryById)
    @ApiOperation(value = "根据医生id查询详情")
    public Envelop queryById(
            @ApiParam(name = "doctorId", value = "医生id")
            @RequestParam(value = "doctorId", required = true) String doctorId,
            @ApiParam(name = "typeCode", value = "分部，无不传")
            @RequestParam(value = "typeCode", required = false)String typeCode)throws Exception{


        try {
            Object object = baseDoctorService.queryById(doctorId,typeCode);
            if (null != object){
                return success("查询成功",object);
            }
            return failed("查询失败");
        }catch (Exception e){
            return failedException(e);
        }
    }


    @PostMapping(value = BaseHospitalRequestMapping.DoctorSetting.update)
    @ApiOperation(value = "修改医生服务配置")
    public Envelop update(
            @ApiParam(name = "baseDoctorVo", value = "JSON数据", required = true)
            @RequestParam(value = "baseDoctorVo") String baseDoctorVo)throws Exception{

        try {
            JSONObject jsonObject = JSONObject.fromObject(baseDoctorVo);
            Boolean update = baseDoctorService.update(jsonObject);
            if (!update){
                return failed("保存失败，参数不可为空");
            }
            return success(CommonContant.Message.SAVE_ERROR);
        }catch (Exception e){
            return failedException(e);
        }
    }

    @PostMapping(value = BaseHospitalRequestMapping.DoctorSetting.updateList)
    @ApiOperation(value = "批量修改医生服务配置")
    public Envelop updateList(
            @ApiParam(name = "baseDoctorVos", value = "JSON数据", required = true)
            @RequestParam(value = "baseDoctorVos") String baseDoctorVos)throws Exception{

        try {
            JSONArray jsonArray = JSONArray.fromObject(baseDoctorVos);
            Boolean update = baseDoctorService.updateList(jsonArray);
            return success(CommonContant.Message.SAVE_ERROR);
        }catch (Exception e){
            return failedException(e);
        }
    }


    @GetMapping(value = BaseHospitalRequestMapping.DoctorSetting.updateStatus)
    @ApiOperation(value = "修改医生状态")
    public Envelop updateStatus(
            @ApiParam(name = "doctorId", value = "状态")
            @RequestParam(value = "doctorId", required = true) String doctorId,
            @ApiParam(name = "status", value = "状态")
            @RequestParam(value = "status", required = true) String status)throws Exception{

        try {
            baseDoctorService.updateStatus(doctorId,status);

            return success("修改成功");
        }catch (Exception e){
            return failedException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.DoctorSetting.updateSort)
    @ApiOperation(value = "修改医生排名")
    public Envelop updateSort(
            @ApiParam(name = "doctorId", value = "状态")
            @RequestParam(value = "doctorId", required = true) String doctorId,
            @ApiParam(name = "sort", value = "排名")
            @RequestParam(value = "sort", required = true) Integer sort)throws Exception{

        try {
            baseDoctorService.updateSort(doctorId,sort);

            return success("修改成功");
        }catch (Exception e){
            return failedException(e);
        }
    }

    @PostMapping(value = BaseHospitalRequestMapping.DoctorSetting.updateDoctorPw)
    @ApiOperation(value = "修改医生账号密码")
    public Envelop updateDoctorPw(@ApiParam(name = "id", value = "医生ID")
                                  @RequestParam(value = "id", required = true)String id,
                                  @ApiParam(name = "pw", value = "密码")
                                  @RequestParam(value = "pw", required = true)String pw,
                                  @ApiParam(name = "orgPw", value = "原密码")
                                  @RequestParam(value = "orgPw", required = true)String orgPw)throws Exception{

        try {
            return success(baseDoctorService.updateDoctorPw(id,pw,orgPw));
        }catch (Exception e){
            return failedException(e);
        }
    }
}
