package com.yihu.jw.hospital.endpoint.consult;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.doctor.dao.BaseDoctorHospitalDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.doctor.BaseDoctorHospitalDO;
import com.yihu.jw.entity.base.im.ConsultDo;
import com.yihu.jw.entity.base.im.ConsultTeamDo;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.hospital.mapping.PatientMappingDO;
import com.yihu.jw.entity.hospital.message.SystemMessageDO;
import com.yihu.jw.entity.hospital.prescription.WlyyOutpatientDO;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionDO;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionEmrDO;
import com.yihu.jw.hospital.mapping.dao.PatientMappingDao;
import com.yihu.jw.hospital.prescription.dao.OutpatientDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionDao;
import com.yihu.jw.hospital.prescription.service.HcyyPrescriptionService;
import com.yihu.jw.hospital.prescription.service.PrescriptionService;
import com.yihu.jw.hospital.prescription.service.XzyyPrescriptionService;
import com.yihu.jw.hospital.prescription.service.entrance.EntranceService;
import com.yihu.jw.hospital.prescription.service.entrance.YkyyEntranceService;
import com.yihu.jw.hospital.service.SystemMessage.HospitalSystemMessageService;
import com.yihu.jw.im.dao.ConsultDao;
import com.yihu.jw.im.service.ImService;
import com.yihu.jw.order.pay.ylz.YlzPayService;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import com.yihu.jw.sms.service.ZhongShanSMSService;
import com.yihu.jw.util.common.IdCardUtil;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.utils.StringUtil;
import com.yihu.jw.wechat.service.WxTemplateService;
import com.yihu.jw.wlyy.service.WlyyBusinessService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.TimeUnit;


/**
 * @author huangwenjie
 */
@RestController
@RequestMapping(value = BaseHospitalRequestMapping.DodtorIM.PREFIX)
@Api(value = "医生端-咨询IM&诊室&导诊台相关接口", description = "医生端-咨询IM&诊室&导诊台相关接口", tags = {"医生端-咨询IM&诊室&导诊台相关接口"})
public class DoctorConsultEndpoint extends EnvelopRestEndpoint {
	
	private static final Logger logger = LoggerFactory.getLogger(DoctorConsultEndpoint.class);
	
	@Autowired
	private ImService imService;
	
	@Autowired
	private ZhongShanSMSService smsService;
	
	@Autowired
	private PrescriptionService prescriptionService;
	
	@Autowired
	private BaseDoctorDao baseDoctorDao;
	
	@Autowired
	private OutpatientDao outpatientDao;
	
	@Autowired
	private WlyyBusinessService wlyyBusinessService;
	
	@Autowired
	private BaseDoctorHospitalDao baseDoctorHospitalDao;
	
	@Autowired
	private StringRedisTemplate redisTemplate;
	
	@Autowired
	private HospitalSystemMessageService hospitalSystemMessageService;
	@Autowired
	public BasePatientDao basePatientDao;
	@Autowired
	private WxTemplateService wxTemplateService;
	@Autowired
	private XzyyPrescriptionService xzyyPrescriptionService;
	@Autowired
	private ConsultDao consultDao;
	@Autowired
	private YkyyEntranceService ykyyEntranceService;
	@Autowired
	private YlzPayService ylzPayService;
	@Autowired
	private PatientMappingDao patientMappingDao;
	@Autowired
	private PrescriptionDao prescriptionDao;
	@Autowired
	private EntranceService entranceService;
	@Autowired
	private HcyyPrescriptionService hcyyPrescriptionService;
	
	@Value("${wechat.id}")
	private String wxId;

	@Value("${demo.flag}")
	private String demoFlag;

	
	
	@GetMapping (value = BaseHospitalRequestMapping.DodtorIM.findWaitingRoomOutpatientByDoctor)
	@ApiOperation(value = "医生可接单列表(图文复诊、视频复诊、协同门诊)", notes = "医生可接单列表(图文复诊、视频复诊、协同门诊)")
	public ListEnvelop findPrescriptionConsultByDoctor(
			@ApiParam(name = "doctor", value = "医生CODE",defaultValue = "1cd15ffe6b3a11e69f7c005056850d66")
			@RequestParam(value = "doctor",required = true) String doctor,
			@ApiParam(name = "type", value = "图文诊室:1、视频:2")
			@RequestParam(value = "type",required = false) Integer type,
			@ApiParam(name = "query_status", value = "0:图文复诊候诊 1:图文复诊抢单列表")
			@RequestParam(value = "",required = false) Integer query_status,
			@ApiParam(name = "dept", value = "部门")
			@RequestParam(value = "",required = false) String dept,
			@ApiParam(name = "sex", value = "性别1男2女")
			@RequestParam(value = "sex",required = false) String sex,
			@ApiParam(name = "keyWord", value = "姓名关键字")
			@RequestParam(value = "keyWord",required = false) String keyWord,
			@ApiParam(name = "patient", value = "患者id")
			@RequestParam(value = "patient",required = false) String patient,
			@ApiParam(name = "ageGroup", value = "年龄段")
			@RequestParam(value = "ageGroup",required = false) String ageGroup){
		try {
			return success("请求成功",prescriptionService.findWaitingRoomOutpatientByDoctor(doctor,type,query_status,dept,sex,keyWord,ageGroup,patient));
		}catch (Exception e){
			return failedListEnvelopException(e);
		}
	}
	
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.findWaitingRoomPatient)
	@ApiOperation(value = "导诊台居民列表", notes = "导诊台居民列表")
	public ListEnvelop findWaitingRoomPatient(
			@ApiParam(name = "dept", value = "部门CODE")
			@RequestParam(value = "dept",required = false) String dept,
			@ApiParam(name = "type", value = "图文复诊:1视频复诊:2")
			@RequestParam(value = "type",required = false) Integer type){
//		String result = "[{\"id\":\"XDF1231231LLX\",\"name\":\"张三\",\"sex\":1,\"age\":68,\"photo\":\"/group/3d45/111.jpg\",\"type\":1,\"consult_type\":1,\"date\":\"2019-06-20\",\"time\":\"12:12\",\"online_status\":0,\"clinic_room_name\":\"01诊室\"},{\"id\":\"XDF1231231LLL\",\"name\":\"李四\",\"sex\":2,\"age\":50,\"photo\":\"/group/3d45/112.jpg\",\"type\":1,\"consult_type\":1,\"date\":\"2019-06-21\",\"time\":\"12:12\",\"online_status\":1,\"clinic_room_name\":\"02诊室\"},{\"id\":\"XDF1231231L2L\",\"name\":\"王五\",\"sex\":1,\"age\":22,\"photo\":\"/group/3d45/112.jpg\",\"type\":1,\"consult_type\":1,\"date\":\"2019-07-23\",\"time\":\"08:08\",\"online_status\":1,\"clinic_room_name\":\"03诊室\"}]";
		try {
			return success("请求成功",prescriptionService.findWaitingRoomPatient(dept,type));
		}catch (Exception e){
			return failedListEnvelopException(e);
		}
	}
	
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.findWaitingRoomStatusCount)
	@ApiOperation(value = "导诊台顶部数据状态", notes = "导诊台顶部数据状态")
	public Envelop findWaitingRoomStatusCount(
			@ApiParam(name = "dept", value = "部门CODE")
			@RequestParam(value = "dept",required = false) String dept){
//		String result = "{\"waiting_count\":20,\"outpatient_count\":30,\"waiting_count_difference\":5,\"outpatient_count_difference\":-5,\"yesterday_waiting_count\":15,\"yesterday_outpatient_count\":25}";

		try {
			return success("请求成功",prescriptionService.findWaitingRoomStatusCount(dept,wxId));
		}catch (Exception e){
			return failedException(e);
		}
	}
	
	
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.findClinicRoomList)
	@ApiOperation(value = "导诊台诊室列表", notes = "导诊台诊室列表")
	public Envelop findClinicRoomList(
			@ApiParam(name = "dept", value = "部门CODE")
			@RequestParam(value = "dept",required = false) String dept,
			@ApiParam(name = "date", value = "日期")
			@RequestParam(value = "date",required = false) String date,
			@ApiParam(name = "consult_status", value = "0离线，1空闲，2接诊中")
			@RequestParam(value = "consult_status",required = false) String consult_status){
//		String result = "[{\"id\":\"XXD2019887711\",\"doctor_name\":\"蔡建春\",\"visit_status\":0,\"patient_name\":\"张三\",\"visit_time\":\"2019-06-21 08:30:00\",\"time_cost\":50,\"waiting_count\":26,\"room_name\":\"01诊室\"},{\"id\":\"XXD2019887712\",\"doctor_name\":\"蔡阿梅\",\"visit_status\":1,\"patient_name\":\"李四\",\"visit_time\":\"2019-06-21 09:30:00\",\"time_cost\":60,\"waiting_count\":33,\"room_name\":\"02诊室\"}]";

		try{
			return success("请求成功",prescriptionService.findClinicRoomList(dept,date,consult_status,wxId));
		}catch (Exception e){
			return failedException(e);
		}

	}
	
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.findClinicRoomStatus)
	@ApiOperation(value = "查询单个医生诊室状态", notes = "查询单个医生诊室状态")
	public Envelop findClinicRoomStatus(
			@ApiParam(name = "doctor_code", value = "医生CODE")
			@RequestParam(value = "doctor_code",required = true) String doctorCode,
			@ApiParam(name = "dept_code", value = "部门CODE")
			@RequestParam(value = "dept",required = false) String dept){
		String result = "{\"id\":\"402803816babc778016babca6d540008\"\"doctor_name\":\"梁敬兴\",\"visit_status\":0,\"patient_name\":\"白海灵\",\"visit_time\":\"2019-09-30 08:30:00\",\"time_cost\":50,\"waiting_count\":26,\"room_name\":\"01诊室\"，\"dept_code\":'1190007',\"dept_name\":'急诊外科',\"visit_count\":2,\"ending_count\":0,\"offline_count\":2,}";
//		JSONObject obj = JSON.parseObject(result);
		try {
			return success("请求成功",prescriptionService.findClinicRoomStatus(doctorCode));
		}catch (Exception e){
			return failedException(e);
		}
	}
	
	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.addPrescriptionConsult)
	@ApiOperation(value = "添加(复诊咨询,协同门诊)", notes = "添加(复诊咨询,协同门诊)")
	public Envelop addPrescriptionConsult(@ApiParam(name = "outpatientCode", value = "HIS就诊记录CODE", defaultValue = "1")
			@RequestParam(value = "outpatientCode", required = true) String outpatientCode,
			@ApiParam(name = "patientCode", value = "复诊居民", defaultValue = "9638fa184ad14a71ba7e4bf931670778")
			@RequestParam(value = "patientCode", required = true) String patientCode,
            @ApiParam(name = "doctorCode", value = "当前医生", defaultValue = "1cd15ffe6b3a11e69f7c005056850d66")
	        @RequestParam(value = "doctorCode", required = true) String doctorCode,
			@ApiParam(name = "type", value = "咨询类型（1、图文；2、视频,3、图文+视频）", defaultValue = "1")
			@RequestParam(value = "type", required = true) Integer type,
			@ApiParam(name = "reason", value = "复诊说明", defaultValue = "复诊说明")
			@RequestParam(value = "reason", required = false) String reason,
										  @ApiParam(name = "source", value = "来源")
											  @RequestParam(value = "source", required = false) String source)throws Exception{
			ConsultTeamDo consult = new ConsultTeamDo();
			WlyyOutpatientDO outpatientDO = outpatientDao.findById(outpatientCode).orElse(null);
			if (null!=outpatientDO){
                if(!outpatientDO.getDoctor().equalsIgnoreCase(doctorCode)){
                    return failed("当前患者已选择其他医生进行咨询或复诊。");
                }
				if ("1".equalsIgnoreCase(outpatientDO.getOutpatientType())){
					if("2".equalsIgnoreCase(outpatientDO.getType())){
						consult.setType(16);//视频复诊
					}else{
						consult.setType(9);//图文复诊
					}
					//咨询类型（1、图文；2、视频,3、图文+视频）
					consult.setHealthindexType(Integer.valueOf(StringUtils.isNotBlank(outpatientDO.getType())?outpatientDO.getType():"0"));
				}else if ("3".equalsIgnoreCase(outpatientDO.getOutpatientType())){
					if("2".equalsIgnoreCase(outpatientDO.getType())){
						consult.setType(17);//视频咨询
					}else if ("1".equalsIgnoreCase(outpatientDO.getType())){
						consult.setType(1);//图文咨询
					}
				}else if ("2".equalsIgnoreCase(outpatientDO.getOutpatientType())){
					consult.setType(12);//协同门诊
				}
			}

			if (outpatientDO.getStatus().equalsIgnoreCase("-1")){
				return failed("患者已取消！");
			}


//			int res = 0;
		
			JSONObject result = new JSONObject();
			synchronized (outpatientCode.intern()){
				if(!"xm_ykyy_wx".equals(wxId)&&!"sd_tnzyy_wx".equals(wxId)){//中山医院-心脏中心-海沧医院需要挂号
					//1.调用挂号接口
					logger.info("调用挂号接口====START");
					String rsCode = "";
					String mes = "";
					if("xm_xzzx_wx".equals(wxId)){ //心脏中心挂号
						if (!outpatientDO.getOutpatientType().equalsIgnoreCase("2")&&!outpatientDO.getOutpatientType().equalsIgnoreCase("3")){
							JSONObject res = xzyyPrescriptionService.registerOutPatient(outpatientCode,doctorCode);
							rsCode = res.getString("@RESULT");
							mes = res.getString("@MSG");
						}else  {
							rsCode="0";
							mes="已挂号";
						}
						/*if ("1".equalsIgnoreCase(outpatientDO.getOutpatientType())){
							if ("2".equalsIgnoreCase(outpatientDO.getType())){
								prescriptionService.sendWxTemplateMsg(wxId,outpatientCode,null,"16","outpatientMsgRemind","");
							}
						}
						if ("3".equalsIgnoreCase(outpatientDO.getOutpatientType())){
							if ("2".equalsIgnoreCase(outpatientDO.getType())){
								prescriptionService.sendWxTemplateMsg(wxId,outpatientCode,null,"17","outpatientMsgRemind","");
							}
						}*/

					}else if("xm_hcyy_wx".equals(wxId)){ //海沧医院挂号
						if (!outpatientDO.getOutpatientType().equalsIgnoreCase("2")&&!outpatientDO.getOutpatientType().equalsIgnoreCase("3")){
							JSONObject res = hcyyPrescriptionService.registerOutPatient(outpatientCode,doctorCode);
							rsCode = res.getString("@RESULT");
							mes = res.getString("@MSG");
						}else  {
							rsCode="0";
							mes="已挂号";
						}
						/*if ("1".equalsIgnoreCase(outpatientDO.getOutpatientType())){
							if ("2".equalsIgnoreCase(outpatientDO.getType())){
								prescriptionService.sendWxTemplateMsg(wxId,outpatientCode,null,"16","outpatientMsgRemind","");
							}
						}
						if ("3".equalsIgnoreCase(outpatientDO.getOutpatientType())){
							if ("2".equalsIgnoreCase(outpatientDO.getType())){
								prescriptionService.sendWxTemplateMsg(wxId,outpatientCode,null,"17","outpatientMsgRemind","");
							}
						}*/

					}else {
						if (demoFlag.equalsIgnoreCase("true")){
							rsCode="0";
							mes="已挂号";
						}else {
							if (!outpatientDO.getOutpatientType().equalsIgnoreCase("2")&&!outpatientDO.getOutpatientType().equalsIgnoreCase("3")){
								net.sf.json.JSONObject res = prescriptionService.registerOutPatient(outpatientCode,doctorCode);
								rsCode = (String)res.get("@RESULT");
								mes = (String)res.get("@MSG");
							}else  {
								rsCode="0";
								mes="已挂号";
							}

						}
					}

					if("0".equals(rsCode)||"-2".equals(rsCode)){
						//挂号成功，已经挂号
						//2.
						result = imService.addPrescriptionConsult(outpatientCode, patientCode,doctorCode,consult,reason,type,wxId,source);
					}else{
						return failed(mes);
					}
					logger.info("调用挂号接口====END");
				}else {
					result = imService.addPrescriptionConsult(outpatientCode, patientCode,doctorCode,consult,reason,type,wxId,source);
				}
				try {
					prescriptionService.sendWxTemplateMsg(wxId, outpatientCode,null, consult.getType().toString(), "doctorPick", "");
					if ("xm_zsyy_wx".equalsIgnoreCase(wxId)){
						String msgUrl="https://hlwyy.xmzsh.com/ims-wx/index.html#/returnVisit/record?outpatientId="+outpatientCode;
						prescriptionService.sendZsMsgFlow("OL_WZ_START",outpatientCode,"",msgUrl,"医生已接诊");
					}
				}catch (Exception e){
					e.printStackTrace();
				}
			}
			return success("操作成功",result);
	}

	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.refundRegisterNo)
	@ApiOperation(value = "退号", notes = "退号")
	public Envelop refundRegisterNo(@ApiParam(name = "outpatientCode", value = "就诊记录code", defaultValue = "1")
										  @RequestParam(value = "outpatientCode", required = true) String outpatientCode)throws Exception{

		JSONObject result = new JSONObject();
		synchronized (outpatientCode.intern()){

			if(!"xm_ykyy_wx".equals(wxId)){//中山医院-心脏中心需要挂号
				//1.调用退号接口
				logger.info("调用退号接口====START");
				String rsCode = "";
				String mes = "";
				if("xm_xzzx_wx".equals(wxId)){ //心脏中心挂号
					if (demoFlag.equalsIgnoreCase("true")){
						rsCode="0";
						mes="已退号";
					}else {
						JSONObject res = xzyyPrescriptionService.refundRegisterNo(outpatientCode);
						if (res!=null&&res.getString("code").equalsIgnoreCase("0")){
							rsCode = "0";
							mes="已退号";
						}else {
							rsCode = "-1";
							mes=res.getString("msg");
						}
					}

				}else if("xm_hcyy_wx".equals(wxId)){ //心脏中心挂号
					if (demoFlag.equalsIgnoreCase("true")){
						rsCode="0";
						mes="已退号";
					}else {
						JSONObject res = hcyyPrescriptionService.refundRegisterNo(outpatientCode);
						if (res!=null&&res.getString("code").equalsIgnoreCase("0")){
							rsCode = "0";
							mes="已退号";
						}else {
							rsCode = "-1";
							mes=res.getString("msg");
						}
					}

				}else {
					if (demoFlag.equalsIgnoreCase("true")){
						rsCode="0";
						mes="已退号";
					}else {
						net.sf.json.JSONObject res = prescriptionService.refundRegisterNo(outpatientCode);
						rsCode = (String)res.get("@RESULT");
						mes = (String)res.get("@MSG");
					}
				}

				if("-1".equals(rsCode)){
					return failed(mes);
				}
				logger.info("调用退号接口====END");
			}else {
			}

		}
		return success("操作成功",result);
	}


	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.addConsult)
	@ApiOperation(value = "添加(复诊咨询,协同门诊)", notes = "添加(复诊咨询,协同门诊)")
	public Envelop addConsult(@ApiParam(name = "outpatientCode", value = "HIS就诊记录CODE", defaultValue = "1")
										  @RequestParam(value = "outpatientCode", required = true) String outpatientCode,
										  @ApiParam(name = "patientCode", value = "复诊居民", defaultValue = "9638fa184ad14a71ba7e4bf931670778")
										  @RequestParam(value = "patientCode", required = true) String patientCode,
										  @ApiParam(name = "doctorCode", value = "当前医生", defaultValue = "1cd15ffe6b3a11e69f7c005056850d66")
										  @RequestParam(value = "doctorCode", required = true) String doctorCode,
										  @ApiParam(name = "type", value = "咨询类型（1、图文；2、视频,3、图文+视频）", defaultValue = "1")
										  @RequestParam(value = "type", required = true) Integer type,
										  @ApiParam(name = "reason", value = "复诊说明", defaultValue = "复诊说明")
										  @RequestParam(value = "reason", required = false) String reason,
							  @ApiParam(name = "source", value = "来源")
								  @RequestParam(value = "source", required = false) String source)throws Exception{
		ConsultTeamDo consult = new ConsultTeamDo();
		WlyyOutpatientDO outpatientDO = outpatientDao.findById(outpatientCode).orElse(null);
		if (null!=outpatientDO){
		    if(!outpatientDO.getDoctor().equalsIgnoreCase(doctorCode)){
                return failed("当前患者已选择其他医生进行咨询或复诊。");
            }
			if ("1".equalsIgnoreCase(outpatientDO.getOutpatientType())){
				if("2".equalsIgnoreCase(outpatientDO.getType())){
					consult.setType(16);//视频复诊
				}else{
					consult.setType(9);//图文复诊
				}
				//咨询类型（1、图文；2、视频,3、图文+视频）
				consult.setHealthindexType(Integer.valueOf(StringUtils.isNotBlank(outpatientDO.getType())?outpatientDO.getType():"0"));
			}else if ("3".equalsIgnoreCase(outpatientDO.getOutpatientType())){
				if("2".equalsIgnoreCase(outpatientDO.getType())){
					consult.setType(17);//视频咨询
				}else if ("1".equalsIgnoreCase(outpatientDO.getType())){
					consult.setType(1);//图文咨询
				}
			}
		}
//			int res = 0;
		if (outpatientDO.getStatus().equalsIgnoreCase("-1")){
			return failed("患者已取消！");
		}

		JSONObject result = new JSONObject();
		synchronized (outpatientCode.intern()){

			/*if(!"xm_ykyy_wx".equals(wxId)){//中山医院-心脏中心需要挂号
				//1.调用挂号接口
				logger.info("调用挂号接口====START");
				String rsCode = "";
				String mes = "";
				if("xm_xzzx_wx".equals(wxId)){ //心脏中心挂号
					JSONObject res = xzyyPrescriptionService.registerOutPatient(outpatientCode,doctorCode);
					rsCode = res.getString("@RESULT");
					mes = res.getString("@MSG");
					if (type==2){
						prescriptionService.sendWxTemplateMsg(wxId,outpatientCode,null,"16","outpatientMsgRemind","");
					}
				}else {
					if (demoFlag.equalsIgnoreCase("true")){
						rsCode="0";
						mes="已挂号";
					}else {
						net.sf.json.JSONObject res = prescriptionService.registerOutPatient(outpatientCode,doctorCode);
						rsCode = (String)res.get("@RESULT");
						mes = (String)res.get("@MSG");
					}
				}

				if("0".equals(rsCode)||"-2".equals(rsCode)){
					//挂号成功，已经挂号
				}else{
					failed(mes);
				}
				logger.info("调用挂号接口====END");
			}
*/

			//2.
			result = imService.addConsult(outpatientCode, patientCode,doctorCode,consult,reason,type,wxId,source);
		}
		return success("操作成功",result);
	}
	
	
	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.generalAddExpertConsult)
	@ApiOperation(value = "全科医生发起求助专科医生的专家咨询", notes = "全科医生发起求助专科医生的专家咨询")
	public Envelop generalAddExpertConsult(
			@ApiParam(name = "patient_code", value = "居民CODE")
			@RequestParam(value = "patient_code",required = true) String patient_code,
			@ApiParam(name = "general_doctor", value = "全科医生")
			@RequestParam(value = "general_doctor",required = true) String general_doctor,
			@ApiParam(name = "special_doctor", value = "专科医生")
			@RequestParam(value = "special_doctor",required = true) String special_doctor,
			@ApiParam(name = "when", value = "发病时间")
			@RequestParam(value = "when",required = false) String when,
			@ApiParam(name = "symptoms", value = "主述")
			@RequestParam(value = "symptoms",required = false) String symptoms,
			@ApiParam(name = "images", value = "图片")
			@RequestParam(value = "images",required = false) String images,
			@ApiParam(name = "voice", value = "语音")
			@RequestParam(value = "voice",required = false) String voice,
			@ApiParam(name = "source", value = "来源")
			@RequestParam(value = "source",required = false) String source)throws Exception{
		
		ConsultTeamDo consult = new ConsultTeamDo();
		consult.setType(15);//家医咨询求助互联网医院专科医生：全科医生-专科医生-咨询聊天
		
		String result = "";
		synchronized (patient_code.intern()){
			result = imService.generalAddExpertConsult(patient_code, general_doctor,special_doctor,when,symptoms,images,consult,source);
			
			BaseDoctorDO generalDoctor = baseDoctorDao.findByIdAndDel(general_doctor);
			BaseDoctorDO specialDoctor = baseDoctorDao.findByIdAndDel(special_doctor);
			// 查询患者信息
			BasePatientDO p = basePatientDao.findById(patient_code).orElse(null);
			//发送系统消息
			SystemMessageDO systemMessageDO = new SystemMessageDO();
			systemMessageDO.setTitle("新增专家咨询");
			systemMessageDO.setType("4");
			systemMessageDO.setReceiver(specialDoctor.getId());
			systemMessageDO.setReceiverName(specialDoctor.getName());
			systemMessageDO.setRelationCode(patient_code+"_"+general_doctor + "_" +special_doctor + "_15");
			systemMessageDO.setSender(generalDoctor.getId());
			systemMessageDO.setSenderName(generalDoctor.getName());
			
			net.sf.json.JSONObject data = new net.sf.json.JSONObject();
			data.put("name",p.getName());
			data.put("age", IdCardUtil.getAgeForIdcard(p.getIdcard()));
			data.put("gender",IdCardUtil.getSexForIdcard_new(p.getIdcard()));
			data.put("question",symptoms);
			data.put("type",15);
			systemMessageDO.setData(data.toString());
			hospitalSystemMessageService.saveMessage(systemMessageDO);
			
		}
		return success("操作成功",result);
	}
	
	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.finish)
	@ApiOperation(value = "医生结束咨询", notes = "医生结束咨询")
	public Envelop finish(
			@ApiParam(name = "consult", value = "咨询CODE")
			@RequestParam(value = "consult",required = false) String consult,
			@ApiParam(name = "doctorCode", value = "医生COEE")
			@RequestParam(value = "doctorCode",required = false) String doctorCode
	) throws Exception {
		int resutl = imService.finish(consult,doctorCode,2);
		BaseDoctorDO baseDoctorDO = baseDoctorDao.findByIdAndDel(doctorCode);
		JSONObject msgObj = new JSONObject();
		msgObj.put("msg",baseDoctorDO.getName()+"结束了咨询");
		msgObj.put("consultcode",consult);
		String jsonStr = "";
		if ("xm_zsyy_wx".equalsIgnoreCase(wxId)){
			//结束时医生未开处方则发送模板消息结算
			ConsultDo cons = consultDao.findById(consult).orElse(null);
			if (StringUtils.isNoneBlank(cons.getRelationCode())){
				WlyyOutpatientDO outpatientDO = outpatientDao.findById(cons.getRelationCode()).orElse(null);
				List<WlyyPrescriptionDO> wlyyPrescriptionDOS = prescriptionDao.findByOutpatientId(outpatientDO.getId());
				if (wlyyPrescriptionDOS==null||wlyyPrescriptionDOS.size()==0){
					logger.info("发送诊查费支付模板消息start");
					BasePatientDO patientDO = basePatientDao.findById(outpatientDO.getPatient()).orElse(null);
					String userName = null;
					String idcard = null;
					if (patientDO != null) {
						userName = patientDO.getName();
						idcard = patientDO.getIdcard();
					}
					PatientMappingDO patientMappingDO = patientMappingDao.findByPatient(outpatientDO.getConsumer());
					String userNo = null;
					if (patientMappingDO != null) {
						userNo = patientMappingDO.getMappingCode();
					}
					String cardNo = null;
					if (StringUtils.isNoneBlank(outpatientDO.getCardNo())) {
						cardNo = outpatientDO.getCardNo();
					}
					String date=DateUtil.dateToStr(outpatientDO.getRegisterDate(), "yyyyMMddHHmmss");
                    String free= 0+"";
                    if (StringUtils.isNoneBlank(outpatientDO.getRealOrder())){
                        net.sf.json.JSONObject object =entranceService.BS15054(patientMappingDO.getMappingCode(),outpatientDO.getRealOrder(),false);
                        if(object!=null){
							net.sf.json.JSONObject res = object.getJSONObject("resquest");
							String rsCode = res.getString("@RESULT");
							if ("0".equals(rsCode)) {
								if (res.getString("@settle_flag").equalsIgnoreCase("N")){
									free = res.getString("@total_charge");
								}
							}
							logger.info("获取待结算数据 res: " + res.toString());
						}

                    }
					if (!free.equalsIgnoreCase("0")){
						ylzPayService.msgPush("01", cardNo, "01", patientDO.getMobile(), "00", userNo, userName, idcard, outpatientDO.getXtgzh(), outpatientDO.getDeptName(), outpatientDO.getDoctorName(), date, free, "1","WX_PUB");
					}
					logger.info("发送诊查费支付模板消息end");
				}
				if (outpatientDO.getStatus().equalsIgnoreCase("0")){
					outpatientDO.setStatus("-1");
				}else {
					outpatientDO.setStatus("3");
				}
				outpatientDO.setEndTime(new Date());
				System.out.println(outpatientDO.getStatus()+"================");
				outpatientDao.save(outpatientDO);
				try {
				    ConsultDo consultDo = consultDao.findByRelationCode(outpatientDO.getId());
                    String msgUrl="";
                    if (consultDo!=null){
                        msgUrl="https://hlwyy.xmzsh.com/ims-wx/index.html#/fuwupingjia?consultCode="+consultDo.getId();
                    }
					prescriptionService.sendZsMsgFlow("OL_WZ_END",outpatientDO.getId(),"",msgUrl,"医生问诊结束");
				}catch (Exception e){
					e.printStackTrace();
				}
			}
		}
		if(1 == resutl){
			jsonStr = "{\"id\":\""+ UUID.randomUUID().toString()+"\",\"sender_id\":\""+doctorCode+"\",\"sender_name\":\"系统\",\"timestamp\":"+new Date().getTime()+",\"content_type\":7,\"content\":"+msgObj.toString()+",\"business_type\":1}";
		}

		return success(jsonStr);
	}
	
	
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.getConsultInfoAndPatientInfo)
	@ApiOperation(value = "获取咨询问题，图片，居民信息", notes = "获取咨询问题，图片，居民信息")
	public Envelop getConsultInfoAndPatientInfo(
			@ApiParam(name = "consult", value = "咨询CODE")
			@RequestParam(value = "consult",required = false) String consult,
			@ApiParam(name = "patientCode", value = "居民COEE")
			@RequestParam(value = "patientCode",required = false) String patientCode)throws Exception {
		try {
			return success("请求成功",imService.getConsultInfoAndPatientInfo(consult,patientCode));
		}catch (Exception e){
			return failedException(e);
		}
	}
	
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.getOutpatientidByConsoultCode)
	@ApiOperation(value = "根据咨询CODE获取就诊记录ID", notes = "根据咨询CODE获取就诊记录ID")
	public Envelop getOutpatientidByConsoultCode(
			@ApiParam(name = "consult", value = "咨询CODE")
			@RequestParam(value = "consult",required = false) String consult)throws Exception {
		try {
			return success("请求成功",imService.getOutpatientidByConsoultCode(consult));
		}catch (Exception e){
			return failedException(e);
		}
	}
	
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.getVideoPrescriptionByDoctor)
	@ApiOperation(value = "获取正在进行中的视频复诊/协同门诊", notes = "获取正在进行中的视频复诊/协同门诊")
	public ListEnvelop getVideoPrescriptionByDoctor(
			@ApiParam(name = "doctor", value = "医生CODE")
			@RequestParam(value = "doctor",required = false) String doctor,
			@ApiParam(name = "general_doctor", value = "全科医生CODE")
			@RequestParam(value = "general_doctor",required = false) String general_doctor
			)throws Exception{
		try {
			return success("请求成功",prescriptionService.getVideoPrescriptionByDoctor(doctor,general_doctor,wxId));
		}catch (Exception e){
			return failedListEnvelopException(e);
		}
	}
	
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.getWaitingForVisitVideoPrescriptionByDoctor)
	@ApiOperation(value = "获取等待就诊的视频复诊/协同门诊", notes = "获取等待就诊的视频复诊/协同门诊")
	public ListEnvelop getWaitingForVisitVideoPrescriptionByDoctor(
			@ApiParam(name = "doctor", value = "医生CODE")
			@RequestParam(value = "doctor",required = true) String doctor,
			@ApiParam(name = "type", value = "1在线复诊2协同3专家咨询")
			@RequestParam(value = "type",required = false) String type,
            @ApiParam(name = "keyWord", value = "病人姓名")
            @RequestParam(value = "keyWord",required = false) String keyWord,
			@ApiParam(name = "patient", value = "病人id")
			@RequestParam(value = "patient",required = false) String patient
            )throws Exception{
		try {
			return success("请求成功",prescriptionService.getWaitingForVisitVideoPrescriptionByDoctor(doctor,wxId,type,keyWord,patient));
		}catch (Exception e){
			return failedListEnvelopException(e);
		}
	}
	
	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.pickVideoPrescripitonWaitingPeople)
	@ApiOperation(value = "医生抢单(视频)", notes = "医生抢单(视频)")
	public Envelop pickVideoPrescripitonWaitingPeople(
			@ApiParam(name = "outpatientCode", value = "HIS就诊记录CODE", defaultValue = "1")
	        @RequestParam(value = "outpatientCode", required = true) String outpatientCode,
			@ApiParam(name = "doctor", value = "医生CODE")
			@RequestParam(value = "doctor",required = true) String doctor)throws Exception{

		try {
			prescriptionService.pickVideoPrescripitonWaitingPeople(outpatientCode,doctor);
			WlyyOutpatientDO wlyyOutpatientDO = outpatientDao.findById(outpatientCode).orElse(null);
			if (StringUtils.isNoneBlank(wlyyOutpatientDO.getType())){
				if (wlyyOutpatientDO.getType().equalsIgnoreCase("2")&&wlyyOutpatientDO.getOutpatientType().equalsIgnoreCase("3")){
					prescriptionService.sendWxTemplateMsg(wxId, wlyyOutpatientDO.getId(),null, "17", "videoOrderRemind", "");
				}else if (wlyyOutpatientDO.getType().equalsIgnoreCase("2")&&wlyyOutpatientDO.getOutpatientType().equalsIgnoreCase("1")){
					prescriptionService.sendWxTemplateMsg(wxId, wlyyOutpatientDO.getId(),null, "16", "videoOrderRemind", "");

				}
			}
			//发送外层SOCKET消息 在线复诊
			if("2".equals(wlyyOutpatientDO.getOutpatientType())){
				System.out.println("发送外层SOCKET消息：协同门诊医生抢单之后，通知全科医生修改状态");
				JSONObject object = new JSONObject();
				object.put("socket_sms_type",6);
				object.put("relation_code",wlyyOutpatientDO.getId());
				imService.sendWaiSocketMessage(wlyyOutpatientDO.getDoctor(),wlyyOutpatientDO.getGeneralDoctor(),object.toString(),"1");
			}else{
				System.out.println("发送外层SOCKET消息：协同门诊医生抢单之后，通知居民修改状态");
				JSONObject object = new JSONObject();
				object.put("socket_sms_type",6);
				object.put("relation_code",wlyyOutpatientDO.getId());
				imService.sendWaiSocketMessage(wlyyOutpatientDO.getDoctor(),wlyyOutpatientDO.getConsumer(),object.toString(),"1");
			}
			return success("请求成功");
		}catch (Exception e){
			return failedException(e);
		}
	}
	
	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.updateDoctorConsultStatus)
	@ApiOperation(value = "修改医生咨询在线状态", notes = "修改医生咨询在线状态")
	public Envelop updateDoctorConsultStatus(
			@ApiParam(name = "consultStatus", value = "1在线接诊，0下线", defaultValue = "1")
			@RequestParam(value = "consultStatus", required = true) String consultStatus,
			@ApiParam(name = "doctor", value = "医生CODE")
			@RequestParam(value = "doctor",required = true) String doctor){

		try {
			imService.updateDoctorConsultStatus(consultStatus,doctor);
			return success("请求成功");
		}catch (Exception e){
			return failedException(e);
		}
	}
	
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.records)
	@ApiOperation(value = "医生咨询记录查询")
	public Envelop records(
			@ApiParam(name = "doctor", value = "医生id")
			@RequestParam(value = "doctor",required = false) String doctor,
			@ApiParam(name = "title", value = "咨询标题关键字")
			@RequestParam(value = "title",required = false) String title,
			@ApiParam(name = "id", value = "咨询ID")
			@RequestParam(value = "id",required = false) String id,
			@ApiParam(name = "type", value = "咨询类型")
			@RequestParam(value = "type",required = true) String type,
			@ApiParam(name = "status", value = "咨询状态：0全部，1候诊中，2就诊中，3结束")
			@RequestParam(value = "status",required = true) Integer status,
			@ApiParam(name = "start_time", value = "开始时间 YYYY-MM-DD HH:MM:SS")
			@RequestParam(value = "start_time",required = false) String start_time,
			@ApiParam(name = "end_time", value = "结束时间 YYYY-MM-DD HH:MM:SS")
			@RequestParam(value = "end_time",required = false) String end_time,
			@ApiParam(name = "page", value = "第几页")
			@RequestParam(value = "page",required = false) int page,
			@ApiParam(name = "pagesize", value = "分页大小")
			@RequestParam(value = "pagesize",required = false) int pagesize
	)throws Exception{

		try {
			List<Map<String,Object>>  data = imService.findConsultRecordByDoctor(doctor, id,type,status, page,pagesize, title,start_time,end_time);

			if (data != null) {
				for(Map<String,Object> consult :data){
					//如果是协同门诊，多返回全科医生的详细信息
					if(12 == Integer.parseInt(consult.get("type").toString()) && StringUtils.isNoneBlank((String)consult.get("generalDoctor"))){
						BaseDoctorDO baseDoctorDO = baseDoctorDao.findById((String)consult.get("generalDoctor")).orElse(null);
						List<BaseDoctorHospitalDO> baseDoctorHospitalDOList = baseDoctorHospitalDao.findByDoctorCode((String)consult.get("generalDoctor"));
						if(baseDoctorDO!=null){
							consult.put("generalDoctorName", baseDoctorDO.getName());
						}
						if (baseDoctorHospitalDOList!=null&&baseDoctorHospitalDOList.size()!=0){
							BaseDoctorHospitalDO baseDoctorHospitalDO = baseDoctorHospitalDOList.get(0);
							consult.put("generalOrgName",baseDoctorHospitalDO.getOrgName());
							consult.put("generalDeptName",baseDoctorHospitalDO.getDeptName());
						}
					}

				}
			}

			Long total = imService.countConsultRecordByDoctor(doctor, id,type,status,title,start_time,end_time);

			JSONObject result = new JSONObject();
			result.put("total",total);
			result.put("list",data);
			return success(result);
		}catch (Exception e){
			return failedException(e);
		}
	}
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.records2)
	@ApiOperation(value = "医生咨询记录查询")
	public Envelop records2(
			@ApiParam(name = "doctor", value = "医生id")
			@RequestParam(value = "doctor",required = false) String doctor,
			@ApiParam(name = "title", value = "咨询标题关键字")
			@RequestParam(value = "title",required = false) String title,
			@ApiParam(name = "patient", value = "患者id")
			@RequestParam(value = "patient",required = false) String patient,
			@ApiParam(name = "id", value = "咨询ID")
			@RequestParam(value = "id",required = false) String id,
			@ApiParam(name = "type", value = "咨询类型")
			@RequestParam(value = "type",required = true) String type,
			@ApiParam(name = "status", value = "咨询状态：0全部，1候诊中，2就诊中，3结束")
			@RequestParam(value = "status",required = true) Integer status,
			@ApiParam(name = "start_time", value = "开始时间 YYYY-MM-DD HH:MM:SS")
			@RequestParam(value = "start_time",required = false) String start_time,
			@ApiParam(name = "end_time", value = "结束时间 YYYY-MM-DD HH:MM:SS")
			@RequestParam(value = "end_time",required = false) String end_time,
			@ApiParam(name = "page", value = "第几页")
			@RequestParam(value = "page",required = false) int page,
			@ApiParam(name = "pagesize", value = "分页大小")
			@RequestParam(value = "pagesize",required = false) int pagesize
	)throws Exception{
		try {
			List<Map<String,Object>>  data = imService.findConsultRecordByDoctorNew(doctor, id,type,status, page,pagesize, title,start_time,end_time,patient);

			if (data != null) {
				for(Map<String,Object> consult :data){
					//如果是协同门诊，多返回全科医生的详细信息
					if(12 == Integer.parseInt(consult.get("type").toString()) && StringUtils.isNoneBlank((String)consult.get("generalDoctor"))){
						BaseDoctorDO baseDoctorDO = baseDoctorDao.findByIdAndDel((String)consult.get("generalDoctor"));
						List<BaseDoctorHospitalDO> baseDoctorHospitalDOList = baseDoctorHospitalDao.findByDoctorCode((String)consult.get("generalDoctor"));
						if(baseDoctorDO!=null){
							consult.put("generalDoctorName", baseDoctorDO.getName());
						}
						if (baseDoctorHospitalDOList!=null&&baseDoctorHospitalDOList.size()!=0){
							BaseDoctorHospitalDO baseDoctorHospitalDO = baseDoctorHospitalDOList.get(0);
							consult.put("generalOrgName",baseDoctorHospitalDO.getOrgName());
							consult.put("generalDeptName",baseDoctorHospitalDO.getDeptName());
						}
					}
				}
			}

			Long total = imService.countConsultRecordByDoctor2(doctor, id,type,status,title,start_time,end_time);

			JSONObject result = new JSONObject();
			result.put("total",total);
			result.put("list",data);
			return success(result);
		} catch (Exception e){
			return failedException(e);
		}
	}
	
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.testSendPrescriptionIM)
	@ApiOperation(value = "测试发送处方消息", notes = "测试发送处方消息")
	public Envelop testSendPrescriptionIM()throws Exception{
		imService.pushPrescriptionImMessage(null);
		return success("请求成功");
	}
	
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.generalDoctorWaitingNumber)
	@ApiOperation(value = "全科医生等候协同门诊，排队数量", notes = "全科医生等候协同门诊，排队数量")
	public Envelop generalDoctorWaitingNumber(
			@ApiParam(name = "outpatientid", value = "就诊记录CODE")
			@RequestParam(value = "outpatientid",required = true) String outpatientid,
			@ApiParam(name = "doctor", value = "医生code")
			@RequestParam(value = "doctor",required = true) String doctor){

		try {
			JSONObject result = new JSONObject();
			result.put("waitingNumber",prescriptionService.generalDoctorWaitingNumber(doctor,wxId,outpatientid));
			return success("请求成功",result);
		}catch (Exception e){
			return failedException(e);
		}
	}
	
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.findWaitingRoomOutpatientNumberByDoctor)
	@ApiOperation(value = "查询医生所有可抢单的数量", notes = "查询医生所有可抢单的数量")
	public Envelop findWaitingRoomOutpatientNumberByDoctor(
			@ApiParam(name = "doctor", value = "医生CODE",defaultValue = "1cd15ffe6b3a11e69f7c005056850d66")
			@RequestParam(value = "doctor",required = true) String doctor,
			@ApiParam(name = "dept", value = "部门CODE",defaultValue = "")
			@RequestParam(value = "dept",required = true) String dept){
		try {
			return success("请求成功",prescriptionService.findWaitingRoomOutpatientNumberByDoctor(doctor,dept));
		}catch (Exception e){
			return failedException(e);
		}
	}
	
	
	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.refuseOutpatientByDoctor)
	@ApiOperation(value = "拒绝接诊", notes = "拒绝接诊")
	public ObjEnvelop refuseOutpatientByDoctor(@ApiParam(name = "outPatientId", value = "门诊记录")
	                                   @RequestParam(value = "outPatientId", required = false)String outPatientId,
	                                   @ApiParam(name = "cancelType", value = "取消原因类型")
	                                   @RequestParam(value = "cancelType", required = false)String cancelType,
	                                   @ApiParam(name = "cancelValue", value = "取消原因名称")
	                                   @RequestParam(value = "cancelValue", required = false)String cancelValue,
	                                   @ApiParam(name = "cancelRemark", value = "取消原因详细说明")
	                                   @RequestParam(value = "cancelRemark", required = false)String cancelRemark) {

		Map<String,Object> map = new HashedMap();
		try {
			map = prescriptionService.cancelOutPatient(outPatientId,cancelType,cancelValue,cancelRemark,2,wxId);
			if (map != null) {
				logger.info("进入外层");
				if (map.get("code").toString().equalsIgnoreCase("1")){
					String patientName = map.get("patientName").toString();
					String patientCode = map.get("patientCode").toString();
					String doctor = map.get("doctor").toString();
					ConsultDo consultDo = consultDao.findByRelationCode(outPatientId);
					if (consultDo!=null){
						int resutl = imService.finish(consultDo.getId(),doctor,2);
						if (resutl!=-1){
						logger.info("结束咨询成功！");
					}
				}
					String immsg = imService.sendOutPatientCancle(patientName,patientCode,doctor,cancelRemark,cancelValue,outPatientId);
					System.out.println("发送拒绝接诊消息成功："+immsg);
				}
			}
		}catch (Exception e){
			System.out.println("发送拒绝接诊消息失败："+e.getMessage());
		}
		return success(map);
	}
	
	
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.testSendWaiSocketMessage)
	@ApiOperation(value = "测试发送处方消息", notes = "测试发送处方消息")
	public Envelop testSendWaiSocketMessage(@ApiParam(name = "sender", value = "发送者")
	                                            @RequestParam(value = "sender", required = false)String sender,
	                                        @ApiParam(name = "reviver", value = "接受者")
	                                        @RequestParam(value = "reviver", required = false)String reviver
	                                        )throws Exception{
//		imService.testSendWaiSocketMessage(sender,reviver);
		smsService.ZhongShangSendSMS(sender,reviver);
		return success("请求成功");
	}
	
	
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.doctorIndexConsultCount)
	@ApiOperation(value = "复诊图文咨询人数，视频咨询人数、协同门诊人数，可抢单人数（已废弃）", notes = "复诊图文咨询人数，视频咨询人数、协同门诊人数，可抢单人数")
	public Envelop doctorIndexConsultCount(@ApiParam(name = "doctor", value = "医生CODE")
	                                       @RequestParam(value = "doctor",required = true) String doctor){
		JSONObject result = new JSONObject();
		
		
//		//专家咨询
//		Integer zjCount = imService.sessionCountByType(doctor,1,0);
//
//		//复诊咨询
//		Integer imgCount = imService.sessionCountByType(doctor,9,0);
//
//		//正在进行中的视频复诊
//		Long _videoOnlineCount = prescriptionService.doctorIndexConsultCount(doctor);
//		Integer videoCount = _videoOnlineCount.intValue();
//
//		//获取候诊居民数量（包含进行中的）
//		Long _waitVideoCount = prescriptionService.getWaitVideoCount(doctor);
//		Integer waitVideoCount = _waitVideoCount.intValue();
//
//
//		result.put("zjCount",zjCount);
//
//		imgCount = imgCount - videoCount;
//		if(imgCount <=0){
//			imgCount = 0;
//		}
//
//		result.put("zjCount",zjCount);//专家咨询数量
//		result.put("imgCount",imgCount);//图文复诊数量
//		result.put("videoCount",waitVideoCount+_videoOnlineCount);//视频复诊数量
//		result.put("xtCount",0);//协同门诊候诊数量
		
//		JSONObject resultPick = prescriptionService.findWaitingRoomOutpatientNumberByDoctor(doctor);
//		result.put("imgPickCount",resultPick.getIntValue("twCount"));
//		result.put("videoPickCount",resultPick.getIntValue("spCount"));
//		result.put("xtPickCount",resultPick.getIntValue("xtCount"));
		
		
		return success("请求成功",result);
	}
	
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.doctorSpecialistConsultCount)
	@ApiOperation(value = "医生端：进行中的专家咨询数量", notes = "医生端：进行中的专家咨询数量")
	public Envelop doctorSpecialistConsultCount(@ApiParam(name = "doctor", value = "医生CODE")
	                                       @RequestParam(value = "doctor",required = true) String doctor){

		try {
			//专家咨询
			Integer zjCount = imService.sessionCountByType(doctor,1,0);
			//全科医生求助专科
			Integer zjqzCount = imService.sessionCountByType(doctor,15,0);
			JSONObject result = new JSONObject();
			result.put("zjCount",zjCount+zjqzCount);//专家咨询数量
			return success("请求成功",result);
		}catch (Exception e){
			return failedException(e);
		}
	}

	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.doctorSpecialistConsultCountNew)
	@ApiOperation(value = "医生端：进行中的专家咨询数量", notes = "医生端：进行中的专家咨询数量")
	public Envelop doctorSpecialistConsultCountNew(@ApiParam(name = "doctor", value = "医生CODE")
												@RequestParam(value = "doctor",required = true) String doctor){

		try {
			//专家咨询
			JSONObject result = new JSONObject();
			result.put("zjCount",imService.doctorConsults(doctor));//专家咨询数量
			return success("请求成功",result);
		}catch (Exception e){
			return failedException(e);
		}
	}
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.doctorSpecialistConsultCountNew2)
	@ApiOperation(value = "医生端：进行中的专家咨询数量", notes = "医生端：进行中的专家咨询数量")
	public Envelop doctorSpecialistConsultCountNew2(@ApiParam(name = "doctor", value = "医生CODE")
												   @RequestParam(value = "doctor",required = true) String doctor){

		try {
			//专家咨询
			JSONObject result = new JSONObject();
			result.put("zjCount",imService.doctorConsultsNew(doctor));//专家咨询数量
			return success("请求成功",result);
		}catch (Exception e){
			return failedException(e);
		}
	}
	
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.doctorReviewConsultCount)
	@ApiOperation(value = "医生端：图文复诊、视频复诊咨询数量、协同门诊候诊数量", notes = "医生端：图文复诊、视频复诊咨询数量、协同门诊候诊数量")
	public Envelop doctorReviewConsultCount(@ApiParam(name = "doctor", value = "医生CODE")
	                                            @RequestParam(value = "doctor",required = true) String doctor){

		try {
			JSONObject result = new JSONObject();
			logger.info("action:doctorReviewConsultCount--start:"+DateUtil.dateToStrLong(new Date()));
			result.put("imgCount",prescriptionService.getWaitVideoCount(doctor,"1","1",wxId));//图文复诊数量
			result.put("videoCount",prescriptionService.getWaitVideoCount(doctor,"2","1",wxId));//视频复诊数量
			result.put("xtCount",prescriptionService.getWaitVideoCount(doctor,"","2",wxId));//协同门诊候诊数量
			result.put("imgConsultCount",prescriptionService.getWaitVideoCount(doctor,"1","3",wxId));//图文咨询数量
			result.put("videoConsultCount",prescriptionService.getWaitVideoCount(doctor,"2","3",wxId));//视频咨询数量
			result.put("homeConsultCount",prescriptionService.getWaitVideoCount(doctor,"3","3",wxId));//家医咨询数量
			if ("sd_tnzyy_wx".equalsIgnoreCase(wxId)){
				result.put("doorServiceCount",prescriptionService.waitUpdoorCount(doctor));//上门服务数量
			}
			logger.info("action:doctorReviewConsultCount--end:"+DateUtil.dateToStrLong(new Date()));
			return success("请求成功",result);
		}catch (Exception e){
			return failedException(e);
		}
	}
	
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.doctorPickCount)
	@ApiOperation(value = "医生端：可抢单数量", notes = "医生端：可抢单数量")
	public Envelop doctorPickCount(@ApiParam(name = "doctor", value = "医生CODE")
	                                        @RequestParam(value = "doctor",required = true) String doctor,
	                               @ApiParam(name = "dept", value = "部门")
	                               @RequestParam(value = "dept",required = true) String dept){
		

		try {
			JSONObject resultPick = prescriptionService.findWaitingRoomOutpatientNumberByDoctor(doctor,dept);
			JSONObject result = new JSONObject();
			result.put("imgPickCount",resultPick.getIntValue("twCount"));
			result.put("videoPickCount",resultPick.getIntValue("spCount"));
			result.put("xtPickCount",resultPick.getIntValue("xtCount"));
			return success("请求成功",result);
		}catch (Exception e){
			return failedException(e);
		}
	}
	
	
	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.alertPatientOnline)
	@ApiOperation(value = "医生提醒居民上线", notes = "医生提醒居民上线")
	public Envelop alertPatientOnline(@ApiParam(name = "patient", value = "居民CODE")
	                                         @RequestParam(value = "patient",required = true) String patient) {
		

		try {
			String isAlert = redisTemplate.opsForValue().get("patient_alert_"+patient);
			if(StringUtils.isBlank(isAlert)){
				redisTemplate.opsForValue().set("patient_alert_"+patient,patient, 1, TimeUnit.DAYS);
			}
			return success("操作成功");
		}catch (Exception e){
			return failedException(e);
		}
	}
	
	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.isExistsUnfinishedConsult)
	@ApiOperation(value = "查询全科医生与专科医生是否存在未结束的咨询", notes = "查询全科医生与专科医生是否存在未结束的咨询")
	public Envelop isExistsUnfinishedConsult(@ApiParam(name = "patient_code", value = "居民CODE")
	                                          @RequestParam(value = "patient_code", required = true)String patient_code,
	                                          @ApiParam(name = "general_doctor", value = "全科医生")
	                                          @RequestParam(value = "general_doctor", required = true)String general_doctor,
	                                          @ApiParam(name = "special_doctor", value = "专科医生")
	                                          @RequestParam(value = "special_doctor", required = false)String special_doctor
	                                          ) throws Exception {

		try {
			if(org.apache.axis.utils.StringUtils.isEmpty(patient_code)){
				patient_code= getUID();
			}
			return success("请求成功",imService.getUnfinishedConsultByActualSender(patient_code,general_doctor));
		}catch (Exception e){
			return failedException(e);
		}
	}
	
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.expertConsultRecords)
	@ApiOperation(value = "(健康咨询)医生咨询记录查询")
	public Envelop expertConsultRecords(
			@ApiParam(name = "patient", value = "居民CODE")
			@RequestParam(value = "patient",required = true) String patient,
			@ApiParam(name = "doctor", value = "医生id")
			@RequestParam(value = "doctor",required = true) String doctor,
			@ApiParam(name = "title", value = "咨询标题关键字")
			@RequestParam(value = "title",required = false) String title,
			@ApiParam(name = "id", value = "咨询ID")
			@RequestParam(value = "id",required = false) String id,
			@ApiParam(name = "type", value = "咨询类型")
			@RequestParam(value = "type",required = true) Integer type,
			@ApiParam(name = "status", value = "咨询状态：0全部，1候诊中，2就诊中，3结束")
			@RequestParam(value = "status",required = true) Integer status,
			@ApiParam(name = "start_time", value = "开始时间 YYYY-MM-DD HH:MM:SS")
			@RequestParam(value = "start_time",required = false) String start_time,
			@ApiParam(name = "end_time", value = "结束时间 YYYY-MM-DD HH:MM:SS")
			@RequestParam(value = "end_time",required = false) String end_time,
			@ApiParam(name = "page", value = "第几页")
			@RequestParam(value = "page",required = false) int page,
			@ApiParam(name = "pagesize", value = "分页大小")
			@RequestParam(value = "pagesize",required = false) int pagesize
	)throws Exception{

		try {
			List<Map<String,Object>>  data = imService.findexpertConsultRecordByDoctor(doctor, id,type,status, page,pagesize, title,start_time,end_time,patient);

			if (data != null) {
				for (Map<String,Object> consult : data) {


					//如果是协同门诊，多返回全科医生的详细信息
					if(13 == Integer.parseInt((String)consult.get("type")) && StringUtils.isNoneBlank((String)consult.get("generalDoctor"))){
						BaseDoctorDO baseDoctorDO = baseDoctorDao.findByIdAndDel((String)consult.get("generalDoctor"));
						consult.put("generalDoctorName", baseDoctorDO.getName());
					}

					List<BaseDoctorHospitalDO> hospitalDOs =  baseDoctorHospitalDao.findByDoctorCode(doctor);
					if(hospitalDOs!=null&&hospitalDOs.size()>0){
						consult.put("hospital",hospitalDOs.get(0));
					}else{
						consult.put("hospital",null);
					}
				}
			}

			Long total = imService.findexpertConsultRecordByDoctor(doctor, id,type,status,title,start_time,end_time,patient);

			JSONObject result = new JSONObject();
			result.put("total",total);
			result.put("list",data);
			return success(result);
		}catch (Exception e){
			return failedException(e);
		}
	}
	
	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.getPatientCodeByWlyyIdcard)
	@ApiOperation(value = "(健康咨询)根据i健康居民身份证号获取互联网医院居民CODE", notes = "(健康咨询)根据i健康居民身份证号获取互联网医院居民CODE")
	public Envelop getPatientCodeByWlyyIdcard(@ApiParam(name = "idcard", value = "i健康居民身份证")
	                                         @RequestParam(value = "idcard", required = true)String idcard
	) throws Exception {
		return success("请求成功",wlyyBusinessService.getPatientCodeByWlyyIdcard(idcard));
	}
	
	
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.getMessageForward)
	@ApiOperation(value = "(健康咨询)请求转发的消息内容", notes = "(健康咨询)请求转发的消息内容")
	public Envelop getMessageForward(
	                              @ApiParam(name = "messageids", value = "消息ID,英文逗号连接", defaultValue = "")
	                                  @RequestParam(value = "messageids", required = true) String messageids,
	                              @ApiParam(name = "title", value = "会话标题", defaultValue = "")
	                                  @RequestParam(value = "title", required = true) String title,
	                              @ApiParam(name = "sessionType", value = "原会话的会话类型", defaultValue = "")
	                                  @RequestParam(value = "sessionType", required = true) String sessionType
	) throws Exception {
		try {
			return success("请求成功",imService.getMessageById(sessionType,messageids,title));
		}catch (Exception e){
			return failedException(e);
		}
	}
	
	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.postMessageForward)
	@ApiOperation(value = "(健康咨询)转发消息", notes = "(健康咨询)转发消息")
	public Envelop messageForward(@ApiParam(name = "senderIdcard", value = "发送者身份证", defaultValue = "")
	                              @RequestParam(value = "senderIdcard", required = true) String senderIdcard,
	                              @ApiParam(name = "reciverIdcard", value = "发送者身份证", defaultValue = "")
	                              @RequestParam(value = "reciverIdcard", required = true) String reciverIdcard,
	                              @ApiParam(name = "messageids", value = "消息ID,英文逗号连接", defaultValue = "")
	                              @RequestParam(value = "messageids", required = true) String messageids,
	                              @ApiParam(name = "title", value = "会话标题", defaultValue = "")
	                              @RequestParam(value = "title", required = true) String title,
	                              @ApiParam(name = "sessionType", value = "原会话的会话类型", defaultValue = "")
	                              @RequestParam(value = "sessionType", required = true) String sessionType
	) throws Exception {
		return success("转发成功",wlyyBusinessService.wlyyMessageForward(senderIdcard,reciverIdcard,messageids,title,sessionType));
	}
	
	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.videoInviteStatus)
	@ApiOperation(value = "医生端：发起/断开 会话视频邀请", notes = "医生端：发起/断开 会话视频邀请")
	public Envelop videoInviteStatus(@ApiParam(name = "session_id", value = "会话ID")
	                                  @RequestParam(value = "session_id",required = true) String session_id,
	                                 @ApiParam(name = "status", value = "会话ID:1发起，0挂断")
	                                 @RequestParam(value = "status",required = true) Integer status)throws Exception  {
		imService.changeVideoInviteStatus(session_id,status);
		return success("操作成功");
	}
	
	@GetMapping(value = BaseHospitalRequestMapping.DodtorIM.getVideoInviteStatus)
	@ApiOperation(value = "医生端：查询会话视频邀请状态", notes = "医生端：查询会话视频邀请状态")
	public Envelop getVideoInviteStatus(@ApiParam(name = "session_id", value = "会话ID")
	                                 @RequestParam(value = "session_id",required = true) String session_id)throws Exception  {

		try{
			String status = redisTemplate.opsForValue().get("video_invite_"+session_id);
			if(StringUtil.isBlank(status)){
				status = "0";
			}
			return success("查询成功",status);
		}catch (Exception e){
			return failedException(e);
		}

		
	}

	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.videoInviteMiniProgram)
	@ApiOperation(value = "医生端：发起/断开 会话小程序视频邀请", notes = "医生端：发起/断开 会话小程序视频邀请")
	public Envelop videoInviteMiniProgram(@ApiParam(name = "sender_id", value = "发起人")
	                                 @RequestParam(value = "sender_id",required = true) String sender_id,
	                                 @ApiParam(name = "sender_name", value = "发起人名字")
	                                 @RequestParam(value = "sender_name",required = true) String sender_name,
                                      @ApiParam(name = "reciver_id", value = "接收人")
                                      @RequestParam(value = "reciver_id",required = true) String reciver_id,
                                      @ApiParam(name = "reciver_name", value = "接收人")
                                      @RequestParam(value = "reciver_name",required = true) String reciver_name,
                                      @ApiParam(name = "reciver_type", value = "接收人类型1居民 2医生")
                                      @RequestParam(value = "reciver_name",required = true) String reciver_type,
                                      @ApiParam(name = "session_id", value = "会话ID")
                                      @RequestParam(value = "session_id",required = true) String session_id,
                                      @ApiParam(name = "wechat_id", value = "来源")
                                      @RequestParam(value = "wechat_id",required = false) String wechat_id
	                                      )throws Exception  {
		wxTemplateService.sendWeTempMesMiniProgram(sender_id,sender_name,reciver_id,reciver_name,session_id);
		return success("操作成功");
	}
	
	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.sendOutPatientSuggest)
	@ApiOperation(value = "医生端：发送咨询建议消息", notes = "医生端：发送咨询建议消息")
	public Envelop sendOutPatientSuggest(@ApiParam(name = "adviceJson", value = "咨询建议JSON")
	                                     @RequestParam(value = "adviceJson", required = true)String adviceJson,
	                                     @ApiParam(name = "session_id", value = "咨询sessionid")
	                                     @RequestParam(value = "session_id", required = true)String session_id,
	                                     @ApiParam(name = "doctor", value = "医生CODE")
	                                         @RequestParam(value = "doctor", required = true)String doctor,
	                                     @ApiParam(name = "doctorName", value = "")
	                                         @RequestParam(value = "doctorName", required = true)String doctorName
                                      
	)throws Exception  {
		try {
			prescriptionService.sendOutPatientSuggest(adviceJson);
			String immsg = imService.sendOutPatientSuggestMsg(adviceJson,doctor,doctorName,session_id);
			System.out.println("发送咨询建议消息成功："+immsg);
			return success("操作成功");
		}catch (Exception e){
			System.out.println("发送咨询建议消息失败："+e.getMessage());
//            e.printStackTrace();
			return failedException(e);

		}

	}
	
	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.getConsultSuggest)
	@ApiOperation(value = "医生端：发送咨询建议消息", notes = "医生端：发送咨询建议消息")
	public Envelop getConsultSuggest(@ApiParam(name = "consultcode", value = "咨询CODE")
	                                     @RequestParam(value = "consultcode", required = true)String consultcode
	
	)throws Exception  {
		return success("操作成功",prescriptionService.getConsultSuggest(consultcode));
	}
	
/*	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.sendOutPatientCancle)
	@ApiOperation(value = "医生端：发送医生取消IM消息", notes = "医生端：发送医生取消IM消息")
	public Envelop sendOutPatientCancle(@ApiParam(name = "patientName", value = "patientName")
	                                     @RequestParam(value = "patientName", required = true)String patientName,
	                                     @ApiParam(name = "session_id", value = "咨询sessionid")
	                                     @RequestParam(value = "session_id", required = true)String session_id,
	                                     @ApiParam(name = "doctor", value = "医生CODE")
	                                     @RequestParam(value = "doctor", required = true)String doctor,
	                                     @ApiParam(name = "doctorName", value = "")
	                                     @RequestParam(value = "doctorName", required = true)String doctorName,
	                                    @ApiParam(name = "cancleReason", value = "取消原因")
	                                        @RequestParam(value = "cancleReason", required = true)String cancleReason,
	                                    @ApiParam(name = "cancleContent", value = "取消说明")
	                                        @RequestParam(value = "cancleContent", required = true)String cancleContent
	
	)throws Exception  {
		try {
			String immsg = imService.sendOutPatientCancle(patientName,doctor,doctorName,session_id,cancleReason,cancleContent);
			System.out.println("发送医生取消IM消息成功："+immsg);
		}catch (Exception e){
			System.out.println("发送医生取消IM消息失败："+e.getMessage());
//            e.printStackTrace();
		}
		return success("操作成功");
	}*/

	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.guidanceFinishConsult)
	@ApiOperation(value = "导诊助手结束导诊", notes = "导诊助手结束导诊")
	public ObjEnvelop guidanceFinishConsult(@ApiParam(name = "sessionId", value = "sessionId")
												@RequestParam(value = "sessionId", required = true)String sessionId,
											@ApiParam(name = "userId", value = "userId")
											@RequestParam(value = "userId", required = true)String userId) {

		Boolean flag =false;
		try {
			flag = imService.guidanceFinishConsult(sessionId,userId);
			return success(flag);
		}catch (Exception e){
			return failedObjEnvelopException(e);
		}

	}


	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.doctorReplyUpdateYktState)
	@ApiOperation(value = "医生回复修改眼科通状态", notes = "医生回复修改眼科通状态")
	public ObjEnvelop doctorReplyUpdateYktState(@ApiParam(name = "consult", value = "consult")
												@RequestParam(value = "consult", required = true)String consult,
												@ApiParam(name = "doctor", value = "doctor")
												@RequestParam(value = "doctor", required = true)String doctor) {

		Boolean flag =false;
		try {
			imService.doctorReplyUpdateYktState(consult,doctor,wxId);
			flag=true;
			return success(flag);
		}catch (Exception e){
			System.out.println("医生回复修改眼科通状态："+e.getMessage());
			return failedObjEnvelopException(e);
		}

	}
	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.outPatientOrder)
	@ApiOperation(value = "线上开检查单", notes = "线上开具检查单")
	public ObjEnvelop ykOlineOrder(@ApiParam(name = "jsondata", value = "jsondata")
												@RequestParam(value = "jsondata", required = true)String jsondata,
								   @ApiParam(name = "doctor", value = "doctor")
								   @RequestParam(value = "doctor", required = true)String doctor,
								   @ApiParam(name = "patient", value = "patient")
									   @RequestParam(value = "patient", required = true)String patient)throws Exception {

		String success= ykyyEntranceService.outPatientOrderYk(jsondata,false,doctor,patient);
		ObjEnvelop objEnvelop = new ObjEnvelop();
		objEnvelop.setObj(success);
		return objEnvelop;
	}
	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.findYzztByGslb)
	@ApiOperation(value = "组套项目查询", notes = "组套项目查询")
	public ObjEnvelop findYzztByGslb(@ApiParam(name = "gslb", value = "gslb")
								   @RequestParam(value = "gslb", required = false)String gslb)throws Exception {

		JSONArray array= ykyyEntranceService.findYzztByGslb(gslb);
		ObjEnvelop objEnvelop = new ObjEnvelop();
		objEnvelop.setObj(array);
		return objEnvelop;
	}
	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.findZtmxByid)
	@ApiOperation(value = "组套项目明细查询", notes = "组套项目明细查询")
	public ObjEnvelop findZtmxByid(@ApiParam(name = "ztId", value = "ztId")
									 @RequestParam(value = "ztId", required = true)String ztId)throws Exception {

		JSONArray array= ykyyEntranceService.findZtmxByid(ztId);
		ObjEnvelop objEnvelop = new ObjEnvelop();
		objEnvelop.setObj(array);
		return objEnvelop;
	}
	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.findZlxm)
	@ApiOperation(value = "诊疗项目查询", notes = "诊疗项目查询")
	public ObjEnvelop findZlxm(@ApiParam(name = "itemName", value = "itemName")
								   @RequestParam(value = "itemName", required = false)String itemName,
							   @ApiParam(name = "itemCode", value = "itemCode")
							   @RequestParam(value = "itemCode", required = false)String itemCode,
                               @ApiParam(name = "deptCode", value = "deptCode")
                                   @RequestParam(value = "deptCode", required = false)String deptCode,
                               @ApiParam(name = "yqdm", value = "yqdm")
                                   @RequestParam(value = "yqdm", required = false)String yqdm,
                               @ApiParam(name = "pyCode", value = "pyCode")
                                   @RequestParam(value = "pyCode", required = false)String pyCode)throws Exception {

		JSONArray array= ykyyEntranceService.findZlxm(itemName,itemCode,deptCode,yqdm,pyCode);
		ObjEnvelop objEnvelop = new ObjEnvelop();
		objEnvelop.setObj(array);
		return objEnvelop;
	}
	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.findZlxmMx)
	@ApiOperation(value = "诊疗项目明细查询", notes = "诊疗项目明细查询")
	public ObjEnvelop findZlxmMx(@ApiParam(name = "zlxmId", value = "zlxmId")
							   @RequestParam(value = "zlxmId", required = true)String zlxmId)throws Exception {

		JSONArray array= ykyyEntranceService.findZlxmMx(zlxmId);
		ObjEnvelop objEnvelop = new ObjEnvelop();
		objEnvelop.setObj(array);
		return objEnvelop;
	}
	@PostMapping(value = BaseHospitalRequestMapping.DodtorIM.getPrevious)
	@ApiOperation(value = "查询病人的电子病历", notes = "查询病人的电子病历")
	public ObjEnvelop findPatientEmr(@ApiParam(name = "patientId", value = "patientId")
									 @RequestParam(value = "patientId", required = true)String patientId)throws Exception {

		List<WlyyPrescriptionEmrDO> array= ykyyEntranceService.getPrevious(patientId);
		ObjEnvelop objEnvelop = new ObjEnvelop();
		objEnvelop.setObj(array);
		return objEnvelop;
	}

	@GetMapping (value = BaseHospitalRequestMapping.DodtorIM.findRecordByDoctor)
	@ApiOperation(value = "查询服务患者", notes = "查询服务患者")
	public ObjEnvelop findRecordByDoctor(@ApiParam(name = "doctor", value = "医生id")
											 @RequestParam(value = "doctor",required = false) String doctor,
										 @ApiParam(name = "type", value = "咨询类型")
											 @RequestParam(value = "type",required = false) String type,
										 @ApiParam(name = "patientName", value = "patientName")
											 @RequestParam(value = "patientName",required = false) String patientName,
										 @ApiParam(name = "idcard", value = "idcard")
											 @RequestParam(value = "idcard",required = false) String idcard,
										 @ApiParam(name = "page", value = "第几页")
											 @RequestParam(value = "page",required = false) int page,
										 @ApiParam(name = "pagesize", value = "分页大小")
											 @RequestParam(value = "pagesize",required = false) int pagesize)throws Exception {


		try {
			List<Map<String,Object>> list = imService.findRecordByDoctor(doctor,type,idcard,patientName,page,pagesize);
			return ObjEnvelop.getSuccess("操作成功",list);
		}catch (Exception e){
			return failedObjEnvelopException(e);
		}
	}

	@GetMapping (value = BaseHospitalRequestMapping.DodtorIM.findRecordCountByDoctor)
	@ApiOperation(value = "查询服务患者数量", notes = "查询服务患者数量")
	public ObjEnvelop findRecordCountByDoctor(@ApiParam(name = "doctor", value = "医生id")
										 @RequestParam(value = "doctor",required = false) String doctor,
										 @ApiParam(name = "type", value = "咨询类型多个用逗号隔开")
										 @RequestParam(value = "type",required = false) String type,
										 @ApiParam(name = "patientName", value = "patientName")
										 @RequestParam(value = "patientName",required = false) String patientName,
										 @ApiParam(name = "idcard", value = "idcard")
										 @RequestParam(value = "idcard",required = false) String idcard)throws Exception {


		try {
			return ObjEnvelop.getSuccess("操作成功",imService.findRecordCountByDoctor(doctor,type,idcard,patientName));
		}catch (Exception e){
			return failedObjEnvelopException(e);
		}
	}

}
