package com.yihu.jw.hospital.endpoint.dailyReport;


import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.dailyReport.service.DailyReportUploadService;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.dailyReport.BaseDailyReportDetailDO;
import com.yihu.jw.entity.dailyReport.BaseDailyReportItemDO;
import com.yihu.jw.entity.dailyReport.BaseDailyReportUploadDO;
import com.yihu.jw.entity.healthUpload.BaseDoctorMemberDO;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Created by Bing on 2022/7/25.
 */
@RestController
@RequestMapping(value = "dailyReport")
@Api(value = "公司日报上报", description = "公司日报上报")
public class DailyReportUploadPoint extends EnvelopRestEndpoint {

    @Autowired
    private DailyReportUploadService dailyReportUploadService;


    @PostMapping(value = "saveAndCreateMember")
    @ApiOperation(value = "新增医生成员关系")
    public ListEnvelop saveAndCreateMember(@ApiParam(name = "doctorId", value = "医生id")
                                           @RequestParam(value = "doctorId", required = true) String doctorId,
                                           @ApiParam(name = "ids", value = "成员id,逗号隔开")
                                           @RequestParam(value = "ids", required = true) String ids) {
        try {
            List<BaseDoctorMemberDO> doctorHealthUploadDOS = dailyReportUploadService.saveAndCreateMember(doctorId,ids);
            return success(doctorHealthUploadDOS);
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @PostMapping(value = "delDoctorMember")
    @ApiOperation(value = "删除某个成员")
    public ObjEnvelop delDoctorMember(@ApiParam(name = "id", value = "成员列表id")
                                      @RequestParam(value = "id", required = true) String id) {
        try {
            BaseDoctorMemberDO memberDO = dailyReportUploadService.delDoctorMember(id);
            return success(memberDO);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @GetMapping(value = "selectByDoctorId")
    @ApiOperation(value = "查询医生的成员列表")
    public ListEnvelop selectByDoctorId(@ApiParam(name = "doctorId", value = "医生id")
                                        @RequestParam(value = "doctorId", required = true) String doctorId) {
        try {
            List<BaseDoctorMemberDO> memberDOList = dailyReportUploadService.selectByDoctorId(doctorId);
            return success(memberDOList);
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "selectCountByDoctorId")
    @ApiOperation(value = "查询医生的成员列表带有已上报次数")
    public ListEnvelop selectCountByDoctorId(@ApiParam(name = "doctorId", value = "医生id")
                                             @RequestParam(value = "doctorId", required = true) String doctorId) {
        try {
            List<BaseDoctorMemberDO> memberDOList = dailyReportUploadService.selectCountByDoctorId(doctorId);
            return success(memberDOList);
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "selectById")
    @ApiOperation(value = "查询某一个成员信息")
    public ObjEnvelop selectById(@ApiParam(name = "id", value = "成员列表id")
                                 @RequestParam(value = "id", required = true) String id) {
        try {
            BaseDoctorMemberDO memberDO = dailyReportUploadService.selectById(id);
            return success(memberDO);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @PostMapping(value = "createDailyReport")
    @ApiOperation(value = "提交每日上报信息")
    public ObjEnvelop createDailyReport(@ApiParam(name = "doctorId", value = "doctorId")
                                        @RequestParam(value = "doctorId", required = true) String doctorId,
                                        @ApiParam(name = "report_date", value = "上报日期yyyy-MM-dd")
                                        @RequestParam(value = "report_date", required = true) String report_date,
                                        @ApiParam(name = "leaveState", value = "请假状态 0未请假1已请假 默认0")
                                        @RequestParam(value = "leaveState", required = false,defaultValue = "0") Integer leaveState,
                                        @ApiParam(name = "leaveHour", value = "请假时长默认0")
                                        @RequestParam(value = "leaveHour", required = false,defaultValue = "0") String leaveHour,
                                        @ApiParam(name = "week_name", value = "星期一...")
                                        @RequestParam(value = "week_name", required = true) String week_name,
                                        @ApiParam(name = "totalHour", value = "上报时长")
                                        @RequestParam(value = "totalHour", required = false) String totalHour,
                                        @ApiParam(name = "jsonData", value = "事项信息实体")
                                        @RequestParam(value = "jsonData", required = true) String jsonData) {
        try {
            List<BaseDailyReportDetailDO> healthUploadDOs = new ArrayList<>();
            if (StringUtils.isNoneBlank(jsonData)){
                healthUploadDOs = JSONArray.parseArray(jsonData,BaseDailyReportDetailDO.class);
            }
            BaseDailyReportUploadDO dailyReport = dailyReportUploadService.createDailyReport(doctorId,report_date,leaveState,leaveHour,week_name,totalHour,healthUploadDOs);
            return success(dailyReport);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @PostMapping(value = "verifiedItem")
    @ApiOperation(value = "核实每日报表")
    public ObjEnvelop verifiedItem(@ApiParam(name = "doctorId", value = "doctorId")
                                   @RequestParam(value = "doctorId", required = true) String doctorId,
                                   @ApiParam(name = "reportID", value = "reportID")
                                   @RequestParam(value = "reportID", required = true) String reportID,
                                   @ApiParam(name = "jsonData", value = "事项信息实体")
                                   @RequestParam(value = "jsonData", required = false) String jsonData) {
        try {
            List<BaseDailyReportDetailDO> healthUploadDOs = new ArrayList<>();
            if (StringUtils.isNoneBlank(jsonData)){
                healthUploadDOs = JSONArray.parseArray(jsonData,BaseDailyReportDetailDO.class);
            }
            BaseDailyReportUploadDO dailyReport = dailyReportUploadService.verifiedItem(doctorId,reportID,healthUploadDOs);
            return success(dailyReport);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @PostMapping(value = "verifiedDoubtItem")
    @ApiOperation(value = "核实每日报表疑虑")
    public ObjEnvelop verifiedDoubtItem(@ApiParam(name = "doctorId", value = "doctorId")
                                   @RequestParam(value = "doctorId", required = true) String doctorId,
                                   @ApiParam(name = "jsonData", value = "事项信息实体")
                                   @RequestParam(value = "jsonData", required = false) String jsonData) {
        try {
            List<BaseDailyReportDetailDO> healthUploadDOs = new ArrayList<>();
            if (StringUtils.isNoneBlank(jsonData)){
                healthUploadDOs = JSONArray.parseArray(jsonData,BaseDailyReportDetailDO.class);
            }
            BaseDailyReportUploadDO dailyReport = dailyReportUploadService.verifiedDoubtItem(doctorId,healthUploadDOs);
            return success(dailyReport);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @GetMapping(value = "selectReportByDoctorId2")
    @ApiOperation(value = "查询成员的当天上传记录")
    public ObjEnvelop selectReportByDoctorId2(@ApiParam(name = "doctorId", value = "医生id")
                                             @RequestParam(value = "doctorId", required = true) String doctorId) {
        try {
            BaseDailyReportUploadDO list = dailyReportUploadService.selectReportByDoctorId2(doctorId);
            return ObjEnvelop.getSuccess("success",list);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }


    @GetMapping(value = "selectReportByDoctorId")
    @ApiOperation(value = "查询成员的日报上传记录")
    public ObjEnvelop selectReportByDoctorId(@ApiParam(name = "doctorId", value = "医生id")
                                             @RequestParam(value = "doctorId", required = true) String doctorId,
                                             @ApiParam(name = "startDate", value = "开始日期yyyy-MM-dd")
                                             @RequestParam(value = "startDate", required = true) String startDate,
                                             @ApiParam(name = "endDate", value = "结束日期")
                                             @RequestParam(value = "endDate", required = true) String endDate) {
        try {
            List<Map<String,List<BaseDailyReportUploadDO>>> list = dailyReportUploadService.selectReportByDoctorId(doctorId,startDate,endDate);
            return ObjEnvelop.getSuccess("success",list);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @GetMapping(value = "selectReportById")
    @ApiOperation(value = "根据id查看日报记录")
    public ObjEnvelop selectReportById(@ApiParam(name = "id", value = "日报id")
                                       @RequestParam(value = "id", required = true) String id) {
        try {
            BaseDailyReportUploadDO reportUploadDO = dailyReportUploadService.selectReportById(id);
            return ObjEnvelop.getSuccess("success",reportUploadDO);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @GetMapping(value = "saveDoctorInfo")
    @ApiOperation(value = "新增加医生信息")
    public ObjEnvelop saveDoctorInfo(@ApiParam(name = "photo", value = "医生头像")
                                     @RequestParam(value = "photo", required = false) String photo,
                                     @ApiParam(name = "name", value = "姓名")
                                     @RequestParam(value = "name", required = false) String name,
                                     @ApiParam(name = "idcard", value = "证件号码")
                                     @RequestParam(value = "idcard", required = false) String idcard,
                                     @ApiParam(name = "sex", value = "性别")
                                     @RequestParam(value = "sex", required = false) String sex,
                                     @ApiParam(name = "mobile", value = "手机号")
                                     @RequestParam(value = "mobile", required = false) String mobile,
                                     @ApiParam(name = "dept", value = "科室编码")
                                     @RequestParam(value = "dept", required = false) String dept,
                                     @ApiParam(name = "deptName", value = "科室名称")
                                     @RequestParam(value = "deptName", required = false) String deptName,
                                     @ApiParam(name = "idCardType", value = "证件类别")
                                     @RequestParam(value = "idCardType", required = false) String idCardType,
                                     @ApiParam(name = "idType", value = "身份类别")
                                     @RequestParam(value = "idType", required = false) String idType,
                                     @ApiParam(name = "id", value = "医生id")
                                     @RequestParam(value = "id", required = false) String id,
                                     @ApiParam(name = "mark", value = "数据权限是否")
                                     @RequestParam(value = "mark", required = false) Integer mark) {
        try {
            BaseDoctorDO baseDoctorDO = dailyReportUploadService.saveDoctorInfo(id,photo,name,idcard,sex,mobile,dept,deptName,idCardType,idType,mark);
            return success(baseDoctorDO);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    /**
     *
     * @param dept
     * @param level
     * @param doctorId
     * @return
     */
    @GetMapping(value = "selectDailyReportByRole")
    @ApiOperation(value = "医生首页")
    public ObjEnvelop selectDailyReportByRole(@ApiParam(name = "dept", value = "院级管理员无需传科室code")
                                        @RequestParam(value = "dept", required = false) String dept,
                                        @ApiParam(name = "level", value = "1院级管理员，2科室管理员")
                                        @RequestParam(value = "level", required = false) Integer level,
                                        @ApiParam(name = "doctorId", value = "医生id")
                                        @RequestParam(value = "doctorId", required = false) String doctorId) {
        try {
            JSONObject jsonObject = dailyReportUploadService.selectDailyReportByRole(dept,level,doctorId);
            return success(jsonObject);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @GetMapping(value = "selectHeaderList")
    @ApiOperation(value = "首页按照科室统计")
    public ListEnvelop selectHeaderList(@ApiParam(name = "dept", value = "科室code")
                                        @RequestParam(value = "dept", required = false) String dept,
                                        @ApiParam(name = "idType", value = "身份类别")
                                        @RequestParam(value = "idType", required = false) String idType,
                                        @ApiParam(name = "state", value = "核实状态0未核实1已核实")
                                        @RequestParam(value = "state", required = false) String state,
                                        @ApiParam(name = "startDate", value = "开始时间")
                                        @RequestParam(value = "startDate", required = false) String startDate,
                                        @ApiParam(name = "endDate", value = "结束时间")
                                        @RequestParam(value = "endDate", required = false) String endDate,
                                        @ApiParam(name = "name", value = "姓名")
                                        @RequestParam(value = "name", required = false) String name) {
        try {
            return success(dailyReportUploadService.selectHeaderList(dept,idType,state,startDate,endDate,name));
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "selectHeaderListHeshi")
    @ApiOperation(value = "首页按照科室统计")
    public ListEnvelop selectHeaderListHeshi(@ApiParam(name = "dept", value = "科室code")
                                        @RequestParam(value = "dept", required = false) String dept,
                                             @ApiParam(name = "doctor", value = "当前用户")
                                             @RequestParam(value = "doctor", required = false) String doctor,
                                        @ApiParam(name = "idType", value = "身份类别")
                                        @RequestParam(value = "idType", required = false) String idType,
                                        @ApiParam(name = "state", value = "核实状态0未核实1已核实")
                                        @RequestParam(value = "state", required = false) String state,
                                        @ApiParam(name = "startDate", value = "开始时间")
                                        @RequestParam(value = "startDate", required = false) String startDate,
                                        @ApiParam(name = "endDate", value = "结束时间")
                                        @RequestParam(value = "endDate", required = false) String endDate,
                                        @ApiParam(name = "name", value = "姓名")
                                        @RequestParam(value = "name", required = false) String name) {
        try {
            return success(dailyReportUploadService.selectHeaderListHeshi(doctor,dept,idType,state,startDate,endDate,name));
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "selectListByTotal")
    @ApiOperation(value = "获取列表的时间及数量")
    public ListEnvelop selectListByTotal(@ApiParam(name = "dept", value = "科室code")
                                         @RequestParam(value = "dept", required = false) String dept,
                                         @ApiParam(name = "idType", value = "身份类别")
                                         @RequestParam(value = "idType", required = false) String idType,
                                         @ApiParam(name = "status", value = "上报状态1已上报 2未上报")
                                         @RequestParam(value = "status", required = false) String status,
                                         @ApiParam(name = "state", value = "核实状态 0未核实 1已核实")
                                         @RequestParam(value = "state", required = false) String state,
                                         @ApiParam(name = "doubtType", value = "存疑类型 存疑类型，1不饱和存疑 2效率存疑")
                                         @RequestParam(value = "doubtType", required = false) Integer doubtType,
                                         @ApiParam(name = "startDate", value = "开始时间")
                                         @RequestParam(value = "startDate", required = false) String startDate,
                                         @ApiParam(name = "endDate", value = "结束时间")
                                         @RequestParam(value = "endDate", required = false) String endDate,
                                         @ApiParam(name = "name", value = "名字搜索")
                                         @RequestParam(value = "name", required = false) String name) {
        try {
            JSONArray jsonArray = dailyReportUploadService.selectListByTotal(dept,idType,status,state,startDate,endDate,name,doubtType);
            return success(jsonArray);
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "selectListInfoByTotal")
    @ApiOperation(value = "获取列表的具体人员信息")
    public ListEnvelop selectListInfoByTotal(@ApiParam(name = "dept", value = "科室code")
                                             @RequestParam(value = "dept", required = false) String dept,
                                             @ApiParam(name = "idType", value = "身份类别")
                                             @RequestParam(value = "idType", required = false) String idType,
                                             @ApiParam(name = "status", value = "上报状态1已上报 2未上报")
                                             @RequestParam(value = "status", required = false) String status,
                                             @ApiParam(name = "state", value = "核实状态 0未核实 1已核实")
                                             @RequestParam(value = "state", required = false) String state,
                                             @ApiParam(name = "doubtType", value = "存疑类型 存疑类型，1不饱和存疑 2效率存疑")
                                             @RequestParam(value = "doubtType", required = false) Integer doubtType,
                                             @ApiParam(name = "startDate", value = "开始时间")
                                             @RequestParam(value = "startDate", required = false) String startDate,
                                             @ApiParam(name = "endDate", value = "结束时间")
                                             @RequestParam(value = "endDate", required = false) String endDate,
                                             @ApiParam(name = "name", value = "具体医生姓名")
                                             @RequestParam(value = "name", required = false) String name) {
        try {
            JSONArray jsonArray = dailyReportUploadService.selectListInfoByTotal(dept,idType,status,state,startDate,endDate,name,doubtType);
            return success(jsonArray);
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "selectListByTotalDoubt")
    @ApiOperation(value = "获取存疑列表的时间及数量")
    public ListEnvelop selectListByTotalDoubt(@ApiParam(name = "dept", value = "科室code")
                                         @RequestParam(value = "dept", required = false) String dept,
                                         @ApiParam(name = "idType", value = "身份类别")
                                         @RequestParam(value = "idType", required = false) String idType,
                                         @ApiParam(name = "doubtType", value = "存疑类型0全部 存疑类型，1不饱和存疑 2效率存疑")
                                         @RequestParam(value = "doubtType", required = false) Integer doubtType,
                                         @ApiParam(name = "startDate", value = "开始时间")
                                         @RequestParam(value = "startDate", required = false) String startDate,
                                         @ApiParam(name = "endDate", value = "结束时间")
                                         @RequestParam(value = "endDate", required = false) String endDate,
                                         @ApiParam(name = "name", value = "名字搜索")
                                         @RequestParam(value = "name", required = false) String name) {
        try {
            JSONArray jsonArray = dailyReportUploadService.selectListByTotalDoubt(dept,idType,startDate,endDate,name,doubtType);
            return success(jsonArray);
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "selectListInfoByTotalDoubt")
    @ApiOperation(value = "获取存疑列表的具体人员信息")
    public ListEnvelop selectListInfoByTotalDoubt(@ApiParam(name = "dept", value = "科室code")
                                             @RequestParam(value = "dept", required = false) String dept,
                                             @ApiParam(name = "idType", value = "身份类别")
                                             @RequestParam(value = "idType", required = false) String idType,
                                             @ApiParam(name = "doubtType", value = "存疑类型0全部 存疑类型，1不饱和存疑 2效率存疑")
                                             @RequestParam(value = "doubtType", required = false) Integer doubtType,
                                             @ApiParam(name = "startDate", value = "开始时间")
                                             @RequestParam(value = "startDate", required = false) String startDate,
                                             @ApiParam(name = "endDate", value = "结束时间")
                                             @RequestParam(value = "endDate", required = false) String endDate,
                                             @ApiParam(name = "name", value = "具体医生姓名")
                                             @RequestParam(value = "name", required = false) String name) {
        try {
            JSONArray jsonArray = dailyReportUploadService.selectListInfoByTotalDoubt(dept,idType,startDate,endDate,name,doubtType);
            return success(jsonArray);
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "selectRecordByDoctorIdAndConsumer")
    @ApiOperation(value = "查下上报最新一次记录", notes = "查下上报最新一次记录")
    public Envelop selectRecordByDoctorIdAndConsumer(@ApiParam(name = "doctorId", value = "医生id")
                                                     @RequestParam(value = "doctorId", required = false) String doctorId) {
        try{
            return success(dailyReportUploadService.selectRecordByDoctorIdAndConsumer(doctorId));
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }

    @GetMapping(value = "findDoctorByHospitalAndDept")
    @ApiOperation(value = "查询部门下医生", notes = "查询部门下医生")
    public ListEnvelop findDoctorByHospitalAndDept(@ApiParam(name = "orgCode", value = "机构code")
                                                   @RequestParam(value = "orgCode", required = false) String orgCode,
                                                   @ApiParam(name = "dept", value = "部门code")
                                                   @RequestParam(value = "dept", required = false) String dept,
                                                   @ApiParam(name = "doctorCode", value = "需要置顶医生")
                                                   @RequestParam(value = "doctorCode", required = false) String doctorCode,
                                                   @ApiParam(name = "key", value = "关键字搜索，医生名字/科室名称/专长")
                                                   @RequestParam(value = "key", required = false) String key,
                                                   @ApiParam(name = "flag", value = "是否核实0未核实1已核实")
                                                   @RequestParam(value = "flag", required = false) Integer flag) {
        try {
            return success(dailyReportUploadService.findDoctorByHospitalAndDept(orgCode, dept, doctorCode, key, flag));
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "findDoctorByHospitalAndDeptHeshi")
    @ApiOperation(value = "查询部门下医生", notes = "查询部门下医生")
    public ListEnvelop findDoctorByHospitalAndDeptHeshi(@ApiParam(name = "orgCode", value = "机构code")
                                                   @RequestParam(value = "orgCode", required = false) String orgCode,
                                                   @ApiParam(name = "doctor", value = "当前账户")
                                                   @RequestParam(value = "doctor", required = false) String doctor,
                                                   @ApiParam(name = "dept", value = "部门code")
                                                   @RequestParam(value = "dept", required = false) String dept,
                                                   @ApiParam(name = "doctorCode", value = "需要置顶医生")
                                                   @RequestParam(value = "doctorCode", required = false) String doctorCode,
                                                   @ApiParam(name = "key", value = "关键字搜索，医生名字/科室名称/专长")
                                                   @RequestParam(value = "key", required = false) String key,
                                                   @ApiParam(name = "flag", value = "是否核实0未核实1已核实")
                                                   @RequestParam(value = "flag", required = false) Integer flag) {
        try {
            return success(dailyReportUploadService.findDoctorByHospitalAndDeptHeshi(doctor,orgCode, dept, doctorCode, key, flag));
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "selectIndexTotalCircle")
    @ApiOperation(value = "首页上报分析", notes = "首页上报分析")
    public Envelop selectIndexTotalCircle(@ApiParam(name = "flag", value = "1周2月")
                                          @RequestParam(value = "flag", required = false) Integer flag,
                                          @ApiParam(name = "dept", value = "院级管理员无需传科室code")
                                          @RequestParam(value = "dept", required = false) String dept,
                                          @ApiParam(name = "level", value = "1院级管理员，2科室管理员")
                                          @RequestParam(value = "level", required = false) Integer level) throws Exception {
        try{
            return success(dailyReportUploadService.selectIndexTotalCircle(flag,dept,level));
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }

    @GetMapping(value = "selectHeaderTotal")
    @ApiOperation(value = "首页统计")
    public ObjEnvelop selectHeaderTotal(@ApiParam(name = "dept", value = "科室code")
                                        @RequestParam(value = "dept", required = false) String dept,
                                        @ApiParam(name = "idType", value = "身份类别")
                                        @RequestParam(value = "idType", required = false) String idType,
                                        @ApiParam(name = "status", value = "上报状态1已上报 2未上报")
                                        @RequestParam(value = "status", required = false) String status,
                                        @ApiParam(name = "state", value = "核实状态0未核实1已核实")
                                        @RequestParam(value = "state", required = false) String state,
                                        @ApiParam(name = "startDate", value = "开始时间")
                                        @RequestParam(value = "startDate", required = false) String startDate,
                                        @ApiParam(name = "endDate", value = "结束时间")
                                        @RequestParam(value = "endDate", required = false) String endDate,
                                        @ApiParam(name = "name", value = "姓名")
                                        @RequestParam(value = "name", required = false) String name) {
        try {
            JSONObject jsonObject = dailyReportUploadService.selectHeaderTotal(dept,idType,status,state,startDate,endDate,name);
            return success(jsonObject);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    /*************************************日报项目相关******************************************/
    @PostMapping(value = "newReportItem")
    @ApiOperation(value = "管理员新增日报项目")
    public ObjEnvelop newReportItem(@ApiParam(name = "user", value = "user")
                                    @RequestParam(value = "user", required = true) String user,
                                    @ApiParam(name = "projectCode", value = "项目Code")
                                    @RequestParam(value = "projectCode", required = true) String projectCode,
                                    @ApiParam(name = "projectName", value = "项目名称")
                                    @RequestParam(value = "projectName", required = true) String projectName,
                                    @ApiParam(name = "title", value = "项目标题")
                                    @RequestParam(value = "title", required = true) String title,
                                    @ApiParam(name = "content", value = "项目内容")
                                    @RequestParam(value = "content", required = true) String content,
                                    @ApiParam(name = "begin_time", value = "begin_time")
                                    @RequestParam(value = "begin_time", required = true) String begin_time,
                                    @ApiParam(name = "end_time", value = "end_time")
                                    @RequestParam(value = "end_time", required = true) String end_time,
                                    @ApiParam(name = "member_type", value = "成员类型，1指定成员，2某部门")
                                    @RequestParam(value = "member_type", required = true) Integer member_type,
                                    @ApiParam(name = "members", value = "成员or部门编码，多个逗号隔开")
                                    @RequestParam(value = "members", required = true) String members) {
        try {
            BaseDailyReportItemDO itemDO = dailyReportUploadService.newReportItem(user,projectCode,projectName,title,content,begin_time,end_time,member_type,members);
            return success(itemDO);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @PostMapping(value = "completeReportItem")
    @ApiOperation(value = "管理员完成日报项目")
    public ObjEnvelop completeReportItem(@ApiParam(name = "user", value = "user")
                                     @RequestParam(value = "user", required = true) String user,
                                     @ApiParam(name = "item_id", value = "item_id")
                                     @RequestParam(value = "item_id", required = true) String item_id,
                                     @ApiParam(name = "completionUser", value = "完成人")
                                     @RequestParam(value = "completionUser", required = false) String completionUser,
                                     @ApiParam(name = "completionHour", value = "耗时")
                                     @RequestParam(value = "completionHour", required = false) String completionHour,
                                     @ApiParam(name = "completionTime", value = "完成时间yyyy-MM-dd HH:mm:ss")
                                     @RequestParam(value = "completionTime", required = false) String completionTime) {
        try {
            BaseDailyReportItemDO itemDO =  dailyReportUploadService.completeReportItem(user,item_id,completionUser,completionHour,completionTime);
            return success(itemDO);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @PostMapping(value = "modifyReportItem")
    @ApiOperation(value = "管理员修改日报项目")
    public ObjEnvelop modifyReportItem(@ApiParam(name = "user", value = "user")
                                    @RequestParam(value = "user", required = true) String user,
                                    @ApiParam(name = "projectCode", value = "项目Code")
                                    @RequestParam(value = "projectCode", required = true) String projectCode,
                                    @ApiParam(name = "projectName", value = "项目名称")
                                    @RequestParam(value = "projectName", required = true) String projectName,
                                    @ApiParam(name = "item_id", value = "item_id")
                                    @RequestParam(value = "item_id", required = true) String item_id,
                                    @ApiParam(name = "title", value = "项目标题")
                                    @RequestParam(value = "title", required = true) String title,
                                    @ApiParam(name = "content", value = "项目内容")
                                    @RequestParam(value = "content", required = true) String content,
                                    @ApiParam(name = "state", value = "-1取消，0待办，1已完成")
                                    @RequestParam(value = "state", required = true) Integer state,
                                    @ApiParam(name = "begin_time", value = "begin_time")
                                    @RequestParam(value = "begin_time", required = true) String begin_time,
                                    @ApiParam(name = "end_time", value = "end_time")
                                    @RequestParam(value = "end_time", required = true) String end_time,
                                    @ApiParam(name = "members", value = "成员编码，多个逗号隔开")
                                    @RequestParam(value = "members", required = true) String members) {
        try {
            BaseDailyReportItemDO itemDO = dailyReportUploadService.modifyReportItem(user,item_id,projectCode,projectName,title,content,state,begin_time,end_time,members);
            return success(itemDO);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @PostMapping(value = "delReportItem")
    @ApiOperation(value = "管理员删除日报项目")
    public Envelop delReportItem(@ApiParam(name = "user", value = "user")
                                    @RequestParam(value = "user", required = true) String user,
                                    @ApiParam(name = "item_id", value = "item_id")
                                    @RequestParam(value = "item_id", required = true) String item_id) {
        try {
            dailyReportUploadService.delReportItem(user,item_id);
            return success();
        } catch (Exception e) {
            return failedException(e);
        }
    }

    @PostMapping(value = "findReportItemById")
    @ApiOperation(value = "查询单条日报项目详情")
    public ObjEnvelop findReportItemById(@ApiParam(name = "item_id", value = "item_id")
                                       @RequestParam(value = "item_id", required = true) String item_id) {
        try {
            BaseDailyReportItemDO itemDO = dailyReportUploadService.findReportItemById(item_id);
            return success(itemDO);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @GetMapping(value = "findReportItemListAdmin")
    @ApiOperation(value = "管理员获取所有日报项目")
    public ListEnvelop findReportItemListAdmin(@ApiParam(name = "user", value = "user")
                                      @RequestParam(value = "user", required = true) String user,
                                      @ApiParam(name = "title", value = "事项标题")
                                      @RequestParam(value = "title", required = false) String title,
                                      @ApiParam(name = "content", value = "项目内容")
                                      @RequestParam(value = "content", required = false) String content,
                                      @ApiParam(name = "state", value = "-1取消，0待办，1已完成")
                                      @RequestParam(value = "state", required = false) Integer state,
                                      @ApiParam(name = "begin_time", value = "begin_time")
                                      @RequestParam(value = "begin_time", required = false) String begin_time,
                                      @ApiParam(name = "end_time", value = "end_time")
                                      @RequestParam(value = "end_time", required = false) String end_time,
                                      @ApiParam(name = "createUser", value = "createUser")
                                      @RequestParam(value = "createUser", required = false) String createUser,
                                      @ApiParam(name = "createUserName", value = "createUserName")
                                      @RequestParam(value = "createUserName", required = false) String createUserName) {
        try {
            List<BaseDailyReportItemDO> list = dailyReportUploadService.findReportItemListAdmin(user,title,content,state,begin_time,end_time,createUser,createUserName);
            return ListEnvelop.getSuccess("success",list);
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "findReportItemList")
    @ApiOperation(value = "员工获取所有日报项目")
    public ListEnvelop findReportItemList(@ApiParam(name = "user", value = "user")
                                      @RequestParam(value = "user", required = true) String user,
                                      @ApiParam(name = "title", value = "事项标题")
                                      @RequestParam(value = "title", required = false) String title,
                                      @ApiParam(name = "content", value = "项目内容")
                                      @RequestParam(value = "content", required = false) String content,
                                      @ApiParam(name = "state", value = "-1取消，0待办，1已完成")
                                      @RequestParam(value = "state", required = false) Integer state,
                                      @ApiParam(name = "begin_time", value = "begin_time")
                                      @RequestParam(value = "begin_time", required = false) String begin_time,
                                      @ApiParam(name = "end_time", value = "end_time")
                                      @RequestParam(value = "end_time", required = false) String end_time,
                                      @ApiParam(name = "createUser", value = "createUser")
                                      @RequestParam(value = "createUser", required = false) String createUser,
                                      @ApiParam(name = "createUserName", value = "createUserName")
                                      @RequestParam(value = "createUserName", required = false) String createUserName) {
        try {
            List<BaseDailyReportItemDO> list =  dailyReportUploadService.findReportItemList(user,title,content,state,begin_time,end_time,createUser,createUserName);
            return ListEnvelop.getSuccess("success",list);
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "statisticReportProjectList")
    @ApiOperation(value = "管理员日报项目统计列表一级")
    public ListEnvelop statisticReportProjectList(@ApiParam(name = "user", value = "user")
                                               @RequestParam(value = "user", required = true) String user,
                                               @ApiParam(name = "projectCode", value = "项目Code")
                                               @RequestParam(value = "projectCode", required = false) String projectCode,
                                               @ApiParam(name = "startDate", value = "startDate")
                                               @RequestParam(value = "startDate", required = false) String startDate,
                                               @ApiParam(name = "endDate", value = "endDate")
                                               @RequestParam(value = "endDate", required = false) String endDate) {
        try {
            return ListEnvelop.getSuccess("success",dailyReportUploadService.statisticReportProjectList(user,projectCode,startDate,endDate));
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }


    @GetMapping(value = "statisticReportItemList")
    @ApiOperation(value = "管理员日报项目统计列表二级")
    public PageEnvelop statisticReportItemList(@ApiParam(name = "user", value = "user")
                                               @RequestParam(value = "user", required = true) String user,
                                               @ApiParam(name = "projectCode", value = "项目Code")
                                               @RequestParam(value = "projectCode", required = false) String projectCode,
                                               @ApiParam(name = "title", value = "事项标题")
                                               @RequestParam(value = "title", required = false) String title,
                                               @ApiParam(name = "content", value = "项目内容")
                                               @RequestParam(value = "content", required = false) String content,
                                               @ApiParam(name = "state", value = "-1取消，0待办，1已完成")
                                               @RequestParam(value = "state", required = false) Integer state,
                                               @ApiParam(name = "startDate", value = "startDate")
                                               @RequestParam(value = "startDate", required = false) String startDate,
                                               @ApiParam(name = "endDate", value = "endDate")
                                               @RequestParam(value = "endDate", required = false) String endDate,
                                               @ApiParam(name = "createUser", value = "createUser")
                                               @RequestParam(value = "createUser", required = false) String createUser,
                                               @ApiParam(name = "createUserName", value = "createUserName")
                                               @RequestParam(value = "createUserName", required = false) String createUserName,
                                               @ApiParam(name = "page", value = "page")
                                               @RequestParam(value = "page", required = true)Integer page,
                                               @ApiParam(name = "size", value = "size")
                                               @RequestParam(value = "size", required = true)Integer size) {
        try {
            return dailyReportUploadService.statisticReportItemList(user,projectCode,title,content,state,startDate,endDate,createUser,createUserName,page,size);
        } catch (Exception e) {
            return failedPageEnvelopException(e);
        }
    }

    @GetMapping(value = "statisticReportItemById")
    @ApiOperation(value = "管理某项目统计情况")
    public ListEnvelop statisticReportItemById(@ApiParam(name = "user", value = "user")
                                               @RequestParam(value = "user", required = true) String user,
                                               @ApiParam(name = "item_id", value = "item_id")
                                               @RequestParam(value = "item_id", required = true) String item_id,
                                               @ApiParam(name = "type", value = "1天、2周、3月")
                                               @RequestParam(value = "type",defaultValue = "1", required = false) Integer type) {
        try {
            return ListEnvelop.getSuccess("success",dailyReportUploadService.statisticReportItemById(user,item_id,type));
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "statisticReportItemMembersById")
    @ApiOperation(value = "管理某项目员工耗时统计情况")
    public ListEnvelop statisticReportItemMembersById(@ApiParam(name = "user", value = "user")
                                               @RequestParam(value = "user", required = true) String user,
                                               @ApiParam(name = "item_id", value = "item_id")
                                               @RequestParam(value = "item_id", required = true) String item_id,
                                               @ApiParam(name = "type", value = "0全部不按日期进行统计，1天、2周、3月")
                                               @RequestParam(value = "type",defaultValue = "1", required = false) Integer type,
                                               @ApiParam(name = "member", value = "指定查看某个成员的用时")
                                               @RequestParam(value = "member", required = false) String member) {
        try {
            return ListEnvelop.getSuccess("success",dailyReportUploadService.statisticReportItemMembersById(user,item_id,type,member));
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @RequestMapping(value = "excelControl/getProjectItemReport", method = RequestMethod.GET)
    @ApiOperation("导出项目计划周报月报用时情况")
    public Envelop getProjectItemReport(@ApiParam(name = "user", value = "user")
                                     @RequestParam(value = "user",defaultValue = "1", required = true) String user,
                                     @ApiParam(name = "type", value = "1周报2月报")
                                     @RequestParam(value = "type",defaultValue = "1", required = true) Integer type,
                                     @ApiParam(name = "date", value = "日期yyyy-MM-dd,周报")
                                     @RequestParam(value = "date", required = true) String date,
            HttpServletResponse response) {
        try {
            dailyReportUploadService.getProjectItemReport(response,user,type,date);
            return success("success");
        } catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }

    @RequestMapping(value = "excelControl/getUserReportList", method = RequestMethod.GET)
    @ApiOperation("导出员工周报月报提交情况")
    @ResponseBody
    public Envelop getUserReportList(@ApiParam(name = "user", value = "user")
                                  @RequestParam(value = "user",defaultValue = "1", required = true) String user,
                                  @ApiParam(name = "type", value = "1周报2月报")
                                  @RequestParam(value = "type",defaultValue = "1", required = true) Integer type,
                                  @ApiParam(name = "date", value = "日期yyyy-MM-dd,周报")
                                  @RequestParam(value = "date",defaultValue = "1", required = true) String date,
                                  HttpServletResponse response) {
        try {
            dailyReportUploadService.getUserReportList(response,user,type,date);
            return success("success");
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

}
