package com.yihu.jw.hospital.endpoint.dict;

import com.yihu.jw.dict.service.PrivateDictService;
import com.yihu.jw.entity.hospital.message.SystemMessageDO;
import com.yihu.jw.hospital.dict.service.WlyyHospitalSysDictService;
import com.yihu.jw.hospital.endpoint.message.SystemMessageController;
import com.yihu.jw.hospital.message.service.BaseBannerService;
import com.yihu.jw.hospital.message.service.SystemMessageService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.models.auth.In;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

@RestController
@RequestMapping(value = BaseHospitalRequestMapping.PatientNoLogin.PREFIX)
@Api(value = "隐私策略管理", description = "隐私策略管理", tags = {"互联网医院 - 隐私策略管理"})
public class PrivateDictEndpoint extends EnvelopRestEndpoint {
    private static Logger logger = LoggerFactory.getLogger(PrivateDictEndpoint.class);
    @Autowired
    private PrivateDictService privateDictService;
    @Autowired
    private WlyyHospitalSysDictService wlyyHospitalSysDictService;
    @Autowired
    private SystemMessageService systemMessageService;



    @ApiOperation("查询隐私策略")
    @PostMapping(value= BaseHospitalRequestMapping.PatientNoLogin.findPrivateDict)
    public Envelop findPrivateDict(@ApiParam(name = "dictCode", value = "隐私code")
                                    @RequestParam(value = "dictCode", required = false) String dictCode,
                                   @ApiParam(name = "hospitalCode", value = "")
                                   @RequestParam(value = "hospitalCode", required = false) String hospitalCode){
        try {
            return success(privateDictService.findByHospitalCodeAndDictCode(dictCode));
        }catch (Exception e){
            return failedException(e);
        }


    }
    @PostMapping(value= BaseHospitalRequestMapping.PatientNoLogin.savePrivateDict)
    @ApiOperation("更新/新增隐私策略")
    public MixEnvelop savePrivateDict(
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id", required = true)String id,
            @ApiParam(name = "dictCode", value = "dictCode", required = false)
            @RequestParam(value = "dictCode", required = false)String dictCode,
            @ApiParam(name = "dictName", value = "dictName", required = false)
            @RequestParam(value = "dictName", required = false)String dictName,
            @ApiParam(name = "dictValue", value = "dictValue", required = false)
            @RequestParam(value = "dictValue", required = false)String dictValue,
            @ApiParam(name = "hospitalCode", value = "hospitalCode", required = false)
            @RequestParam(value = "hospitalCode", required = false)String hospitalCode,
            @ApiParam(name = "hospitalName", value = "hospitalName", required = false)
            @RequestParam(value = "hospitalName", required = false)String hospitalName,
            @ApiParam(name = "imgUrl", value = "imgUrl", required = false)
            @RequestParam(value = "imgUrl", required = false)String imgUrl)throws Exception {

        try {
            MixEnvelop mixEnvelop =privateDictService.addPrivateDict(id,dictCode,dictName,dictValue,hospitalCode,hospitalName,imgUrl);
            return mixEnvelop;
        }catch (Exception e){
            return failedMixEnvelopException(e);
        }
    }
    @ApiOperation("删除隐私策略")
    @GetMapping(value= BaseHospitalRequestMapping.PatientNoLogin.delPrivateDict)
    public MixEnvelop delPrivateDict(@ApiParam(name = "id", value = "id", required = true)
                                         @RequestParam(value = "id", required = true)String id){

        try {
            MixEnvelop mixEnvelop =privateDictService.deleteById(id);
            return mixEnvelop;
        }catch (Exception e){
            return failedMixEnvelopException(e);
        }
    }
    @ApiOperation("查询字典")
    @GetMapping(value= BaseHospitalRequestMapping.PatientNoLogin.findDict)
    public MixEnvelop findDict(@ApiParam(name = "name", value = "name", required = false)
                                     @RequestParam(value = "name", required = false)String name,
                                @ApiParam(name = "modelName", value = "modelName", required = false)
                                @RequestParam(value = "modelName", required = false)String modelName,
                                @ApiParam(name = "code", value = "code", required = false)
                                @RequestParam(value = "code", required = false)String code,
                                @ApiParam(name = "value", value = "value", required = false)
                                    @RequestParam(value = "value", required = false)String value1,
                                @ApiParam(name = "page", value = "page", required = false)
                                    @RequestParam(value = "page", required = false) Integer page,
                                @ApiParam(name = "pageSize", value = "pageSize", required = false)
                                    @RequestParam(value = "pageSize", required = false)Integer pageSize){

        try {
            return wlyyHospitalSysDictService.findDictsByNameCode(modelName,name,code,value1,page,pageSize);
        }catch (Exception e){
            return failedMixEnvelopException(e);
        }
    }
    @ApiOperation("删除字典")
    @GetMapping(value= BaseHospitalRequestMapping.PatientNoLogin.delDict)
    public Envelop delDict(@ApiParam(name = "id", value = "id", required = true)
                                     @RequestParam(value = "id", required = true)String id){

        try {
            return success(wlyyHospitalSysDictService.deleteDictById(id));
        }catch (Exception e){
            return failedMixEnvelopException(e);
        }
    }
    @ApiOperation("新增/修改字典")
    @GetMapping(value= BaseHospitalRequestMapping.PatientNoLogin.createOrUpdateDict)
    public Envelop createOrUpdateDict(@ApiParam(name = "json", value = "id", required = true)
                           @RequestParam(value = "json", required = true)String json) throws Exception{

        try {
            return success(wlyyHospitalSysDictService.updateOrCreateDict(json));

        }catch (Exception e){
            return failedException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.SystemMessage.searchMessagePageList)
    @ApiOperation(value = "获取线上就诊消息列表")
    public Envelop searchMessagePageList(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "type", value = "类型")
            @RequestParam(value = "type", required = false) String type,
            @ApiParam(name = "page", value = "第几页，从1开始", required = true, defaultValue = "1")
            @RequestParam(value = "page", required = true,defaultValue = "1")Integer page,
            @ApiParam(name = "pageSize", value = "每页分页大小", required = true, defaultValue = "10")
            @RequestParam(value = "pageSize", required = true,defaultValue = "10")Integer pageSize)throws Exception{



        try {
            if(org.apache.commons.lang.StringUtils.isBlank(filters)){
                filters="del=1";
            }else{
                String doctorId = wlyyHospitalSysDictService.findByDoctorMappingCode(filters);
                filters="receiver="+doctorId+";del=1";
                if (StringUtils.isNotBlank(type)){
                    filters = filters+";type="+type;
                }
            }
            List<SystemMessageDO> list  = systemMessageService.search(fields,filters,sorts,page,pageSize);
            int count = (int) systemMessageService.getCount(filters);
            return success(list, count, page, pageSize, SystemMessageDO.class);
        }catch (Exception e){
            return failedException(e);
        }
    }



    @PostMapping(value = BaseHospitalRequestMapping.SystemMessage.commenSystemMessage)
    @ApiOperation(value = "发送系统消息通用接口")
    public Envelop commenSystemMessage(
            @ApiParam(name = "jsonParam", value = "实体参数")
            @RequestParam(value = "jsonParam", required = false) String jsonParam,
            @ApiParam(name = "jsonContent", value = "消息内容")
            @RequestParam(value = "jsonContent", required = false) String jsonContent)throws Exception{

        try {
            return success(systemMessageService.commenSystemMessage(jsonParam,jsonContent));
        }catch (Exception e){
            return failedException(e);
        }
    }

}
