package com.yihu.jw.hospital.endpoint.drugstore;

import com.yihu.jw.entity.base.area.BaseDrugStoreDO;
import com.yihu.jw.entity.base.area.BaseProvinceDO;
import com.yihu.jw.hospital.drugstore.service.BaseDrugStoreService;
import com.yihu.jw.restmodel.base.area.BaseProvinceVO;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.base.BaseRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.management.MXBean;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping(value = BaseRequestMapping.drugStoreManage.PREFIX)
@Api(value = "药店管理", description = "药店管理接口", tags = {"药店管理接口"})
public class BaseDrugStoreEndpoint extends EnvelopRestEndpoint {
    @Autowired
    private BaseDrugStoreService baseDrugStoreService;
    @GetMapping(value = BaseRequestMapping.drugStoreManage.findDrugstore)
    @ApiOperation(value = "药店查询")
    public MixEnvelop findDrugstore (
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1")
            @RequestParam(value = "page") int page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15")
            @RequestParam(value = "size") int size) throws Exception {

        try {
            List<BaseDrugStoreDO> baseProvinces = baseDrugStoreService.search(fields, filters, sorts, page, size);
            int count = (int)baseDrugStoreService.getCount(filters);
            MixEnvelop mixEnvelop = new MixEnvelop();
            mixEnvelop.setDetailModelList(baseProvinces);
            mixEnvelop.setCurrPage(page);
            mixEnvelop.setTotalCount(count);
            mixEnvelop.setPageSize(size);
            return mixEnvelop;
        }catch (Exception e){
            return failedMixEnvelopException(e);
        }
    }
    @GetMapping(value = BaseRequestMapping.drugStoreManage.findDrugstoreByDistance)
    @ApiOperation(value = "药店距离查询")
    public MixEnvelop findDrugstoreBydistance (
            @ApiParam(name = "latitude", value = "经度")
            @RequestParam(value = "latitude", required = false) String latitude,
            @ApiParam(name = "longitude", value = "纬度")
            @RequestParam(value = "longitude", required = false) String longitude,
            @ApiParam(name = "drugStoreName", value = "药店名称")
            @RequestParam(value = "drugStoreName", required = false) String drugStoreName,
            @ApiParam(name = "hospitalName", value = "医院名称")
            @RequestParam(value = "hospitalName", required = false) String hospitalName) throws Exception {

        try {
            List<BaseDrugStoreDO> list = baseDrugStoreService.countDistanceListIncode(latitude,longitude,drugStoreName,hospitalName);
            MixEnvelop mixEnvelop = new MixEnvelop();
            mixEnvelop.setDetailModelList(list);
            return mixEnvelop;
        }catch (Exception e){
            return failedMixEnvelopException(e);
        }
    }
    @GetMapping(value = BaseRequestMapping.drugStoreManage.findDrugstoreById)
    @ApiOperation(value = "药店id查询")
    public Envelop findDrugStoreById(@ApiParam(name = "latitude", value = "经度")
                                         @RequestParam(value = "latitude", required = false) String latitude,
                                     @ApiParam(name = "longitude", value = "纬度")
                                         @RequestParam(value = "longitude", required = false) String longitude,
                                        @ApiParam(name = "id", value = "药店id")
                                     @RequestParam(value = "id", required = false) String id){

        try {
            BaseDrugStoreDO baseDrugStoreDO = baseDrugStoreService.getById(id,latitude,longitude);
            if (null!=baseDrugStoreDO){
                return success(baseDrugStoreDO);
            }else {
                return failed("查询失败");
            }
        }catch (Exception e){
            return failedMixEnvelopException(e);
        }
    }

}
