package com.yihu.jw.hospital.endpoint.healthUpload;


import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.contant.CommonContant;
import com.yihu.jw.dict.dao.DictHospitalDeptDao;
import com.yihu.jw.entity.base.dict.DictHospitalDeptDO;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.healthUpload.BaseDoctorHealthUploadDO;
import com.yihu.jw.entity.healthUpload.BaseDoctorMemberDO;
import com.yihu.jw.entity.healthUpload.DeptVO;
import com.yihu.jw.entity.healthUpload.HealthReportingFrequencyDO;
import com.yihu.jw.healthUpload.service.BaseDoctorHealthUploadService;
import com.yihu.jw.healthUpload.service.HealthReportingFrequencyService;
import com.yihu.jw.restmodel.web.*;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author wangzhinan
 *
 * Created at 20211003
 */
@RestController
@RequestMapping(value = "healthUpload")
@Api(value = "马銮湾医院健康上报功能", tags = "马銮湾医院健康上报功能")
public class HealthUploadPoint extends EnvelopRestEndpoint {

    private static final Logger logger = LoggerFactory.getLogger(HealthUploadPoint.class);

    @Autowired
    private BaseDoctorHealthUploadService baseDoctorHealthUploadService;
    @Autowired
    private HealthReportingFrequencyService healthReportingFrequencyService;
    @Autowired
    private DictHospitalDeptDao hospitalDeptDao;

    @PostMapping(value = "delReportingFrequency")
    @ApiOperation(value = "新增上报频率")
    public Envelop delReportingFrequency (
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id") String id) {
        try {
            healthReportingFrequencyService.deleteById(id);
            return success("删除成功");
        }catch (Exception e){
            e.printStackTrace();
            return Envelop.getError(CommonContant.Message.SAVE_ERROR);
        }
    }

    @PostMapping(value = "saveReportingFrequency")
    @ApiOperation(value = "新增/修改上报频率")
    public Envelop saveReportingFrequency (
            @ApiParam(name = "jsonData", value = "jsonData", required = true)
            @RequestParam(value = "jsonData") String jsonData) {
        try {
            HealthReportingFrequencyDO frequencyDO = toEntity(jsonData, HealthReportingFrequencyDO.class);
            frequencyDO.setCreateTime(new Date());
            frequencyDO = healthReportingFrequencyService.save(frequencyDO);
            return success(CommonContant.Message.SAVE_ERROR,frequencyDO);
        }catch (Exception e){
            e.printStackTrace();
            return Envelop.getError(CommonContant.Message.SAVE_ERROR);
        }
    }

    @GetMapping(value = "reportingFrequencyById")
    @ApiOperation(value = "获取上报频率明细")
    public Envelop reportingFrequencyById (
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id") String id) {
        try {
            return ObjEnvelop.getSuccess(CommonContant.Message.GET_SUCCESS,healthReportingFrequencyService.findDetail(id));
        }catch (Exception e){
            e.printStackTrace();
            return Envelop.getError("查询失败");
        }
    }

    @GetMapping(value = "reportingFrequencyPage")
    @ApiOperation(value = "获取上报频率分页")
    public PageEnvelop reportingFrequencyPage (
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1")
            @RequestParam(value = "page") Integer page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15")
            @RequestParam(value = "size") Integer size) {
        try {
            List<HealthReportingFrequencyDO> list = healthReportingFrequencyService.search(null, null, null, page, size);
            for (HealthReportingFrequencyDO frequencyDO:list){
                frequencyDO.setTypeName(healthReportingFrequencyService.getTypeName(frequencyDO.getType()));
                if(StringUtils.isNotBlank(frequencyDO.getDeptJson())){
                    List<DeptVO> deptVOList = new ArrayList<>();
                    String depts[] = frequencyDO.getDeptJson().split(",");
                    for (String dep:depts){
                        DictHospitalDeptDO deptDO = hospitalDeptDao.findByCode(dep);
                        if(deptDO!=null){
                            DeptVO deptVO = new DeptVO();
                            deptVO.setCode(dep);
                            deptVO.setName(deptDO.getName());
                            deptVOList.add(deptVO);
                        }
                    }
                    frequencyDO.setDeptList(deptVOList);
                }
            }
            int count = (int)healthReportingFrequencyService.getCount(null);
            return success(list, count, page, size, HealthReportingFrequencyDO.class);
        }catch (Exception e){
            e.printStackTrace();
            return PageEnvelop.getError("查询失败");
        }
    }

    @PostMapping(value = "saveAndCreateMember")
    @ApiOperation(value = "新增医生成员关系")
    public ListEnvelop saveAndCreateMember(@ApiParam(name = "doctorId", value = "医生id")
                                              @RequestParam(value = "doctorId", required = true) String doctorId,
                                           @ApiParam(name = "ids", value = "成员id,逗号隔开")
                                          @RequestParam(value = "ids", required = true) String ids) {
        try {
            List<BaseDoctorMemberDO> doctorHealthUploadDOS = baseDoctorHealthUploadService.saveAndCreateMember(doctorId,ids);
            return success(doctorHealthUploadDOS);
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "selectByDoctorId")
    @ApiOperation(value = "查询医生的成员列表")
    public ListEnvelop selectByDoctorId(@ApiParam(name = "doctorId", value = "医生id")
                                           @RequestParam(value = "doctorId", required = true) String doctorId) {
        try {
            List<BaseDoctorMemberDO> memberDOList = baseDoctorHealthUploadService.selectByDoctorId(doctorId);
            return success(memberDOList);
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "selectCountByDoctorId")
    @ApiOperation(value = "查询医生的成员列表带有已上报次数")
    public ListEnvelop selectCountByDoctorId(@ApiParam(name = "doctorId", value = "医生id")
                                        @RequestParam(value = "doctorId", required = true) String doctorId) {
        try {
            List<BaseDoctorMemberDO> memberDOList = baseDoctorHealthUploadService.selectCountByDoctorId(doctorId);
            return success(memberDOList);
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "selectById")
    @ApiOperation(value = "查询某一个成员信息")
    public ObjEnvelop selectById(@ApiParam(name = "id", value = "成员列表id")
                                             @RequestParam(value = "id", required = true) String id) {
        try {
            BaseDoctorMemberDO memberDO = baseDoctorHealthUploadService.selectById(id);
            return success(memberDO);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @PostMapping(value = "delDoctorMember")
    @ApiOperation(value = "删除某个成员")
    public ObjEnvelop delDoctorMember(@ApiParam(name = "id", value = "成员列表id")
                                 @RequestParam(value = "id", required = true) String id) {
        try {
            BaseDoctorMemberDO memberDO = baseDoctorHealthUploadService.delDoctorMember(id);
            return success(memberDO);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @PostMapping(value = "createHealthUploadInfo")
    @ApiOperation(value = "提交健康信息")
    public ObjEnvelop createHealthUploadInfo(@ApiParam(name = "jsonData", value = "健康信息实体")
                                      @RequestParam(value = "jsonData", required = true) String jsonData) {
        try {
            BaseDoctorHealthUploadDO healthUploadDO = objectMapper.readValue(jsonData, BaseDoctorHealthUploadDO.class);
            healthUploadDO = baseDoctorHealthUploadService.createHealthUploadInfo(healthUploadDO);
            return success(healthUploadDO);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @GetMapping(value = "selectHealthUploadInfoByDoctorId")
    @ApiOperation(value = "查询成员的健康上报记录")
    public MixEnvelop selectHealthUploadInfoByDoctorId(@ApiParam(name = "doctorId", value = "医生id")
                                                        @RequestParam(value = "doctorId", required = true) String doctorId,
                                                       @ApiParam(name = "startDate", value = "开始时间")
                                                       @RequestParam(value = "startDate", required = false) String startDate,
                                                       @ApiParam(name = "endDate", value = "结束时间")
                                                           @RequestParam(value = "endDate", required = false) String endDate,
                                                       @ApiParam(name = "page", value = "页码")
                                                           @RequestParam(value = "page", required = false) Integer page,
                                                       @ApiParam(name = "pageSize", value = "大小")
                                                           @RequestParam(value = "pageSize", required = false) Integer pageSize) {
        try {
            MixEnvelop mixEnvelop = baseDoctorHealthUploadService.selectHealthUploadInfoByDoctorId(doctorId,startDate,endDate,page,pageSize);
            return mixEnvelop;
        } catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }
    @GetMapping(value = "selectHealthInfoById")
    @ApiOperation(value = "根据id查询健康上报情况")
    public ObjEnvelop selectHealthUploadInfoByDoctorId(@ApiParam(name = "id", value = "医生id")
                                                       @RequestParam(value = "id", required = true) String id) {
        try {
            BaseDoctorHealthUploadDO healthUploadDO = baseDoctorHealthUploadService.selectHealthInfoById(id);
            return success(healthUploadDO);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @GetMapping(value = "saveDoctorInfo")
    @ApiOperation(value = "新增加医生信息")
    public ObjEnvelop saveDoctorInfo(@ApiParam(name = "photo", value = "医生头像")
                                                       @RequestParam(value = "photo", required = false) String photo,
                                                       @ApiParam(name = "name", value = "姓名")
                                                       @RequestParam(value = "name", required = false) String name,
                                                       @ApiParam(name = "idcard", value = "证件号码")
                                                       @RequestParam(value = "idcard", required = false) String idcard,
                                                       @ApiParam(name = "sex", value = "性别")
                                                       @RequestParam(value = "sex", required = false) String sex,
                                                       @ApiParam(name = "mobile", value = "手机号")
                                                       @RequestParam(value = "mobile", required = false) String mobile,
                                                        @ApiParam(name = "dept", value = "科室编码")
                                                         @RequestParam(value = "dept", required = false) String dept,
                                                         @ApiParam(name = "deptName", value = "科室名称")
                                                             @RequestParam(value = "deptName", required = false) String deptName,
                                                         @ApiParam(name = "idCardType", value = "证件类别")
                                                             @RequestParam(value = "idCardType", required = false) String idCardType,
                                                         @ApiParam(name = "idType", value = "身份类别")
                                                             @RequestParam(value = "idType", required = false) String idType,
                                     @ApiParam(name = "id", value = "医生id")
                                         @RequestParam(value = "id", required = false) String id,
                                     @ApiParam(name = "mark", value = "数据权限是否")
                                         @RequestParam(value = "mark", required = false) Integer mark) {
        try {
             BaseDoctorDO baseDoctorDO = baseDoctorHealthUploadService.saveDoctorInfo(id,photo,name,idcard,sex,mobile,dept,deptName,idCardType,idType,mark);
            return success(baseDoctorDO);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    /**
     *
     * @param dept
     * @param level
     * @param doctorId
     * @return
     */
    @GetMapping(value = "selectHealthByRole")
    @ApiOperation(value = "医生首页")
    public ObjEnvelop selectHeaderTotal(@ApiParam(name = "dept", value = "院级管理员无需传科室code")
                                        @RequestParam(value = "dept", required = false) String dept,
                                        @ApiParam(name = "level", value = "1院级管理员，2科室管理员")
                                        @RequestParam(value = "level", required = false) Integer level,
                                        @ApiParam(name = "doctorId", value = "上报状态")
                                        @RequestParam(value = "doctorId", required = false) String doctorId) {
        try {
            JSONObject jsonObject = baseDoctorHealthUploadService.selectHealthByRole(dept,level,doctorId);
            return success(jsonObject);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @GetMapping(value = "selectHeaderTotal")
    @ApiOperation(value = "首页统计")
    public ObjEnvelop selectHeaderTotal(@ApiParam(name = "dept", value = "科室code")
                                                       @RequestParam(value = "dept", required = false) String dept,
                                                       @ApiParam(name = "idType", value = "身份类别")
                                                       @RequestParam(value = "idType", required = false) String idType,
                                                       @ApiParam(name = "status", value = "上报状态")
                                                       @RequestParam(value = "status", required = false) String status,
                                                       @ApiParam(name = "state", value = "健康状态")
                                                       @RequestParam(value = "state", required = false) String state,
                                                       @ApiParam(name = "startDate", value = "开始时间")
                                                       @RequestParam(value = "startDate", required = false) String startDate,
                                        @ApiParam(name = "endDate", value = "结束时间")
                                            @RequestParam(value = "endDate", required = false) String endDate,
                                        @ApiParam(name = "name", value = "姓名")
                                            @RequestParam(value = "name", required = false) String name) {
        try {
            JSONObject jsonObject = baseDoctorHealthUploadService.selectHeaderTotal(dept,idType,status,state,startDate,endDate,name);
            return success(jsonObject);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @GetMapping(value = "selectHeaderList")
    @ApiOperation(value = "首页按照科室统计")
    public ListEnvelop selectHeaderList(@ApiParam(name = "dept", value = "科室code")
                                        @RequestParam(value = "dept", required = false) String dept,
                                        @ApiParam(name = "idType", value = "身份类别")
                                        @RequestParam(value = "idType", required = false) String idType,
                                        @ApiParam(name = "status", value = "上报状态")
                                        @RequestParam(value = "status", required = false) String status,
                                        @ApiParam(name = "state", value = "健康状态")
                                        @RequestParam(value = "state", required = false) String state,
                                        @ApiParam(name = "startDate", value = "开始时间")
                                        @RequestParam(value = "startDate", required = false) String startDate,
                                        @ApiParam(name = "endDate", value = "结束时间")
                                        @RequestParam(value = "endDate", required = false) String endDate,
                                        @ApiParam(name = "name", value = "姓名")
                                        @RequestParam(value = "name", required = false) String name) {
        try {
            return success(baseDoctorHealthUploadService.selectHeaderList(dept,idType,status,state,startDate,endDate,name));
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "selectListByTotal")
    @ApiOperation(value = "获取列表的时间及数量")
    public ListEnvelop selectListByTotal(@ApiParam(name = "dept", value = "科室code")
                                        @RequestParam(value = "dept", required = false) String dept,
                                        @ApiParam(name = "idType", value = "身份类别")
                                        @RequestParam(value = "idType", required = false) String idType,
                                        @ApiParam(name = "status", value = "上报状态")
                                        @RequestParam(value = "status", required = false) String status,
                                        @ApiParam(name = "state", value = "健康状态")
                                        @RequestParam(value = "state", required = false) String state,
                                        @ApiParam(name = "startDate", value = "开始时间")
                                        @RequestParam(value = "startDate", required = false) String startDate,
                                         @ApiParam(name = "endDate", value = "结束时间")
                                             @RequestParam(value = "endDate", required = false) String endDate,
                                         @ApiParam(name = "name", value = "名字搜索")
                                             @RequestParam(value = "name", required = false) String name) {
        try {
            JSONArray jsonArray = baseDoctorHealthUploadService.selectListByTotal(dept,idType,status,state,startDate,endDate,name);
            return success(jsonArray);
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "selectListInfoByTotal")
    @ApiOperation(value = "获取列表的具体人员信息")
    public ListEnvelop selectListInfoByTotal(@ApiParam(name = "dept", value = "科室code")
                                         @RequestParam(value = "dept", required = false) String dept,
                                         @ApiParam(name = "idType", value = "身份类别")
                                         @RequestParam(value = "idType", required = false) String idType,
                                         @ApiParam(name = "status", value = "上报状态")
                                         @RequestParam(value = "status", required = false) String status,
                                         @ApiParam(name = "state", value = "健康状态")
                                         @RequestParam(value = "state", required = false) String state,
                                         @ApiParam(name = "startDate", value = "开始时间")
                                         @RequestParam(value = "startDate", required = false) String startDate,
                                         @ApiParam(name = "endDate", value = "结束时间")
                                         @RequestParam(value = "endDate", required = false) String endDate,
                                             @ApiParam(name = "name", value = "具体医生姓名")
                                                 @RequestParam(value = "name", required = false) String name) {
        try {
            JSONArray jsonArray = baseDoctorHealthUploadService.selectListInfoByTotal(dept,idType,status,state,startDate,endDate,name);
            return success(jsonArray);
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "pushListWriteHealthInfo")
    @ApiOperation(value = "健康上报数据导出", notes = "健康上报数据导出")
    public Envelop pushListWrite(@ApiParam(name = "startDate", value = "开始时间")
                                 @RequestParam(value = "startDate", required = false) String startDate,
                                 @ApiParam(name = "endDate", value = "结束时间")
                                 @RequestParam(value = "endDate", required = false) String endDate,
                                 @ApiParam(name = "dept", value = "科室id")
                                     @RequestParam(value = "dept", required = false) String dept,
                                 HttpServletResponse response) throws Exception {
        try{
            response.setContentType("octets/stream");
            response.setHeader("Content-Disposition", "attachment; filename=" + new String(URLEncoder.encode("健康上报报表")+".xls"));
            OutputStream os = response.getOutputStream();
            return success(baseDoctorHealthUploadService.selectListInfoExcelByTotal(os, startDate,endDate,dept));
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }

    @GetMapping(value = "selectRecordByDoctorIdAndConsumer")
    @ApiOperation(value = "查下上报最新一次记录", notes = "查下上报最新一次记录")
    public Envelop selectRecordByDoctorIdAndConsumer(@ApiParam(name = "doctdorId", value = "医生id")
                                 @RequestParam(value = "doctorId", required = false) String doctdorId,
                                 @ApiParam(name = "consumer", value = "代上报人")
                                 @RequestParam(value = "consumer", required = false) String consumer) throws Exception {
        try{
            return success(baseDoctorHealthUploadService.selectRecordByDoctorIdAndConsumer(doctdorId,consumer));
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.Prescription.findDoctorByHospitalAndDept)
    @ApiOperation(value = "查询部门下医生", notes = "查询部门下医生")
    public ListEnvelop findDoctorByHospitalAndDept(@ApiParam(name = "orgCode", value = "机构code")
                                                   @RequestParam(value = "orgCode", required = false) String orgCode,
                                                   @ApiParam(name = "dept", value = "部门code")
                                                   @RequestParam(value = "dept", required = false) String dept,
                                                   @ApiParam(name = "doctorCode", value = "需要置顶医生")
                                                   @RequestParam(value = "doctorCode", required = false) String doctorCode,
                                                   @ApiParam(name = "key", value = "关键字搜索，医生名字/科室名称/专长")
                                                   @RequestParam(value = "key", required = false) String key,
                                                   @ApiParam(name = "flag", value = "是否异常1是0否")
                                                   @RequestParam(value = "flag", required = false) Integer flag) {
        try {
            return success(baseDoctorHealthUploadService.findDoctorByHospitalAndDept(orgCode, dept, doctorCode, key, flag));
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "selectIndexHeader")
    @ApiOperation(value = "首页头部统计", notes = "首页头部统计")
    public Envelop selectIndexHeader(@ApiParam(name = "dept", value = "院级管理员无需传科室code")
                                         @RequestParam(value = "dept", required = false) String dept,
                                     @ApiParam(name = "level", value = "1院级管理员，2科室管理员")
                                         @RequestParam(value = "level", required = false) Integer level) throws Exception {
        try{
            return success(baseDoctorHealthUploadService.selectIndexHeader(dept,level));
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }

    @GetMapping(value = "selectIndexScheduleDept")
    @ApiOperation(value = "首页出差科室统计", notes = "首页出差科室统计")
    public Envelop selectIndexScheduleDept(@ApiParam(name = "dept", value = "院级管理员无需传科室code")
                                               @RequestParam(value = "dept", required = false) String dept,
                                           @ApiParam(name = "level", value = "1院级管理员，2科室管理员")
                                               @RequestParam(value = "level", required = false) Integer level) throws Exception {
        try{
            return success(baseDoctorHealthUploadService.selectIndexScheduleDept(dept,level));
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }

    @GetMapping(value = "selectIndexScheduleCityName")
    @ApiOperation(value = "首页出差分布统计", notes = "首页出差分布统计")
    public Envelop selectIndexScheduleCityName(@ApiParam(name = "dept", value = "院级管理员无需传科室code")
                                                   @RequestParam(value = "dept", required = false) String dept,
                                               @ApiParam(name = "level", value = "1院级管理员，2科室管理员")
                                                   @RequestParam(value = "level", required = false) Integer level) throws Exception {
        try{
            return success(baseDoctorHealthUploadService.selectIndexScheduleCityName(dept,level));
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }

    @GetMapping(value = "selectIndexScheduleTown")
    @ApiOperation(value = "首页本地分布统计", notes = "首页本地分布统计")
    public Envelop selectIndexScheduleTown(@ApiParam(name = "dept", value = "院级管理员无需传科室code")
                                               @RequestParam(value = "dept", required = false) String dept,
                                           @ApiParam(name = "level", value = "1院级管理员，2科室管理员")
                                               @RequestParam(value = "level", required = false) Integer level) throws Exception {
        try{
            return success(baseDoctorHealthUploadService.selectIndexScheduleTown(dept,level));
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }

    @GetMapping(value = "selectIndexYichangList")
    @ApiOperation(value = "首页异常实时情况", notes = "首页异常实时情况")
    public Envelop selectIndexYichangList(@ApiParam(name = "dept", value = "院级管理员无需传科室code")
                                              @RequestParam(value = "dept", required = false) String dept,
                                          @ApiParam(name = "level", value = "1院级管理员，2科室管理员")
                                              @RequestParam(value = "level", required = false) Integer level,
                                          @ApiParam(name = "startDate", value = "开始时间")
                                              @RequestParam(value = "startDate", required = false) String startDate,
                                          @ApiParam(name = "endDate", value = "结束时间")
                                              @RequestParam(value = "endDate", required = false) String endDate,
                                          @ApiParam(name = "condition", value = "条件（医生名称）")
                                              @RequestParam(value = "condition", required = false) String condition) throws Exception {
        try{
            return success(baseDoctorHealthUploadService.selectIndexYichangList(dept,level,startDate,endDate,condition));
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }

    @GetMapping(value = "selectIndexTotalCircle")
    @ApiOperation(value = "首页上报分析", notes = "首页上报分析")
    public Envelop selectIndexTotalCircle(@ApiParam(name = "flag", value = "1周2月")
                                              @RequestParam(value = "flag", required = false) Integer flag,
                                          @ApiParam(name = "dept", value = "院级管理员无需传科室code")
                                          @RequestParam(value = "dept", required = false) String dept,
                                          @ApiParam(name = "level", value = "1院级管理员，2科室管理员")
                                              @RequestParam(value = "level", required = false) Integer level) throws Exception {
        try{
            return success(baseDoctorHealthUploadService.selectIndexTotalCircle(flag,dept,level));
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }

    @GetMapping(value = "selectIndexNucleicTotal")
    @ApiOperation(value = "首页核酸检测情况", notes = "首页核酸检测情况")
    public Envelop selectIndexNucleicTotal(@ApiParam(name = "dept", value = "院级管理员无需传科室code")
                                               @RequestParam(value = "dept", required = false) String dept,
                                           @ApiParam(name = "level", value = "1院级管理员，2科室管理员")
                                               @RequestParam(value = "level", required = false) Integer level) throws Exception {
        try{
            return success(baseDoctorHealthUploadService.selectIndexNucleicTotal(dept,level));
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }

    @GetMapping(value = "selectIndexNewAdd")
    @ApiOperation(value = "首页当日新增统计", notes = "首页当日新增统计")
    public Envelop selectIndexNewAdd(@ApiParam(name = "dept", value = "院级管理员无需传科室code")
                                         @RequestParam(value = "dept", required = false) String dept,
                                     @ApiParam(name = "level", value = "1院级管理员，2科室管理员")
                                         @RequestParam(value = "level", required = false) Integer level) throws Exception {
        try{
            return success(baseDoctorHealthUploadService.selectIndexNewAdd(dept,level));
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }

    @GetMapping(value = "selectIndexDeptList")
    @ApiOperation(value = "首页未报科室统计", notes = "首页未报科室统计")
    public Envelop selectIndexDeptList(@ApiParam(name = "dept", value = "院级管理员无需传科室code")
                                           @RequestParam(value = "dept", required = false) String dept,
                                       @ApiParam(name = "level", value = "1院级管理员，2科室管理员")
                                           @RequestParam(value = "level", required = false) Integer level) throws Exception {
        try{
            return success(baseDoctorHealthUploadService.selectIndexDeptList(dept,level));
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }


}
