package com.yihu.jw.hospital.endpoint.hospital;


import com.yihu.jw.entity.base.doctor.BaseDoctorBackgroundDO;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.doctor.BaseDoctorMessageDO;
import com.yihu.jw.hospital.doctor.dao.service.BaseDoctorBGService;
import com.yihu.jw.hospital.prescription.service.PayInfoNoticeService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping(value = BaseHospitalRequestMapping.BaseBackground.backgroundManage)
@Api(value = "背景管理", description = "背景管理", tags = {"互联网医院 - 医生背景管理"})
public class BaseDoctorBGController extends EnvelopRestEndpoint {
    @Autowired
    private BaseDoctorBGService baseDoctorBGService;
    @Autowired
    private PayInfoNoticeService payInfoNoticeService;
    @ApiOperation("获取背景板")
    @PostMapping(value= BaseHospitalRequestMapping.BaseBackground.getAllBackground)
    public Envelop getAllBG( @ApiParam(name = "page", value = "第几页，从1开始", required = true, defaultValue = "1")
                                 @RequestParam(value = "page", required = true,defaultValue = "1")Integer page,
                             @ApiParam(name = "pageSize", value = "每页分页大小", required = true, defaultValue = "10")
                                 @RequestParam(value = "pageSize", required = true,defaultValue = "10")Integer pageSize){

        try {
            Integer curPage = (page-1)*pageSize;
            List<BaseDoctorBackgroundDO> list = baseDoctorBGService.getBackground(curPage,pageSize);
            if (list.size()>0){
                return success(list);
            }else {
                return failed("查询失败");
            }
        }catch (Exception e){
            System.out.println(e.getClass());
            return failedException(e);
        }
    }
    @ApiOperation("保存医生选择的背景")
    @PostMapping(value= BaseHospitalRequestMapping.BaseBackground.saveDoctorBG)
    public Envelop saveBG(@ApiParam(name = "doctor", value = "doctor", required = false)
                              @RequestParam(value = "doctor", required = false)String doctor,
                          @ApiParam(name = "id", value = "id", required = false)
                          @RequestParam(value = "id", required = false)String id){

        try {
            BaseDoctorDO baseDoctorDO = baseDoctorBGService.setDoctorBackground(doctor,id);
            if (null!=baseDoctorDO){
                return success("操作成功");
            }else {
                return failed("操作失败");
            }
        }catch (Exception e){
            System.out.println(e.getClass());
            return failedException(e);
        }
    }
    @ApiOperation("新增背景板")
    @PostMapping(value= BaseHospitalRequestMapping.BaseBackground.createDoctorBG)
    public Envelop createBg(@ApiParam(name = "bgName", value = "bgName", required = false)
                          @RequestParam(value = "bgName", required = false)String bgName,
                          @ApiParam(name = "bgUrl", value = "bgUrl", required = false)
                          @RequestParam(value = "bgUrl", required = false)String bgUrl){

        try {
            BaseDoctorBackgroundDO baseDoctorBackgroundDO = baseDoctorBGService.createBackground(bgName,bgUrl);
            if (null!=baseDoctorBackgroundDO){
                return success("操作成功");
            }else {
                return failed("操作失败");
            }
        }catch (Exception e){
            System.out.println(e.getClass());
            return failedException(e);
        }
    }
    @ApiOperation("删除背景板")
    @PostMapping(value= BaseHospitalRequestMapping.BaseBackground.removeDoctorBG)
    public Envelop removeBg(@ApiParam(name = "id", value = "id", required = false)
                            @RequestParam(value = "id", required = false)String id){

        try {
            baseDoctorBGService.deleteBg(id);
            return success("操作成功");
        }catch (Exception e){
            System.out.println(e.getClass());
            return failedException(e);
        }
    }

    @ApiOperation("修改背景板")
    @PostMapping(value= BaseHospitalRequestMapping.BaseBackground.updateDoctorBG)
    public Envelop updateBg(@ApiParam(name = "id", value = "id", required = false)
                                @RequestParam(value = "id", required = false)String id,
            @ApiParam(name = "bgName", value = "bgName", required = false)
                            @RequestParam(value = "bgName", required = false)String bgName,
                            @ApiParam(name = "baUrl", value = "baUrl", required = false)
                            @RequestParam(value = "baUrl", required = false)String baUrl){

        try {
            BaseDoctorBackgroundDO baseDoctorBackgroundDO = baseDoctorBGService.updateBackground(id,baUrl,bgName);
            if (null!=baseDoctorBackgroundDO){
                return success("操作成功");
            }else {
                return failed("操作失败");
            }
        }catch (Exception e){
            System.out.println(e.getClass());
            return failedException(e);
        }
    }

    @ApiOperation("查询某个医生快捷回复列表")
    @GetMapping(value= BaseHospitalRequestMapping.BaseBackground.selectDoctorMessage)
    public Envelop selectDoctorMessage(@ApiParam(name = "doctor", value = "doctor", required = true)
                            @RequestParam(value = "doctor", required = true)String doctor,
                            @ApiParam(name = "contentType", value = "contentType", required = false)
                            @RequestParam(value = "contentType", required = false)Integer contentType,
                            @ApiParam(name = "content", value = "content", required = false)
                            @RequestParam(value = "content", required = false)String content,
                            @ApiParam(name = "page", value = "page", required = false)
                                @RequestParam(value = "page", required = false)Integer page,
                            @ApiParam(name = "size", value = "size", required = false)
                                @RequestParam(value = "size", required = false)Integer size){

        try {
            List<Map<String,Object>> mapList = baseDoctorBGService.selectByDoctor(doctor,contentType,content,page,size);
            return success(mapList);
        }catch (Exception e){
            System.out.println(e.getClass());
            return failedException(e);
        }
    }

    @ApiOperation("保存医生快捷回复")
    @PostMapping(value= BaseHospitalRequestMapping.BaseBackground.saveDoctorMessage)
    public Envelop saveDoctorMessage(@ApiParam(name = "json", value = "json", required = true)
                                       @RequestParam(value = "json", required = true)String json) throws Exception{

        try {
            BaseDoctorMessageDO doctorMessageDO = objectMapper.readValue(json,BaseDoctorMessageDO.class);
            return success(baseDoctorBGService.saveDoctorMessage(doctorMessageDO));
        }catch (Exception e){
            System.out.println(e.getClass());
            return failedException(e);
        }
    }


    @ApiOperation("设置医生快捷回复置顶")
    @PostMapping(value= BaseHospitalRequestMapping.BaseBackground.setMessageHead)
    public Envelop setMessageHead(@ApiParam(name = "id", value = "id", required = true)
                                     @RequestParam(value = "id", required = true)String id) throws Exception{

        try {
            return success(baseDoctorBGService.setMessageHead(id));
        }catch (Exception e){
            System.out.println(e.getClass());
            return failedException(e);
        }
    }

    @ApiOperation("获取医生快捷回复消息")
    @PostMapping(value= BaseHospitalRequestMapping.BaseBackground.selectMessageById)
    public Envelop selectMessageById(@ApiParam(name = "id", value = "id", required = true)
                                  @RequestParam(value = "id", required = true)String id) throws Exception{

        try {
            return success(baseDoctorBGService.selectById(id));
        }catch (Exception e){
            System.out.println(e.getClass());
            return failedException(e);
        }
    }


    @ApiOperation("删除医生快捷回复")
    @PostMapping(value= BaseHospitalRequestMapping.BaseBackground.delDoctorMessage)
    public Envelop delDoctorMessage(@ApiParam(name = "id", value = "id", required = true)
                                  @RequestParam(value = "id", required = true)String id) throws Exception{

        try {
            return success(baseDoctorBGService.delDoctorMessage(id));
        }catch (Exception e){
            System.out.println(e.getClass());
            return failedException(e);
        }
    }
    @GetMapping(value = BaseHospitalRequestMapping.Prescription.pushListWrite)
    @ApiOperation(value = "订单导出", notes = "订单导出")
    public void pushListWrite(@ApiParam(name = "status", value = "流程状态，多状态用‘,’分割")
                              @RequestParam(value = "status", required = false)String status,
                              @ApiParam(name = "oneselfPickupFlg", value = "是否自取 1是 0否")
                              @RequestParam(value = "oneselfPickupFlg", required = false)String oneselfPickupFlg,
                              @ApiParam(name = "nameKey", value = "配送员名称")
                              @RequestParam(value = "nameKey", required = false)String nameKey,
                              @ApiParam(name = "startTime", value = "开始时间,yyyy-MM-dd")
                              @RequestParam(value = "startTime", required = false)String startTime,
                              @ApiParam(name = "endTime", value = "结束时间,yyyy-MM-dd")
                              @RequestParam(value = "endTime", required = false)String endTime,
                              HttpServletResponse response) throws Exception{
        List<Map<String,Object>> list = payInfoNoticeService.dealSqlQuery(startTime,endTime);
        List<Map<String,Object>> listDay = payInfoNoticeService.dealSqlQueryGroupDay(startTime,endTime);
        OutputStream os = response.getOutputStream();
        response.reset();
        response.setContentType("application/octet-stream; charset=utf-8");
        response.setCharacterEncoding("UTF-8");
        //response.setHeader("Content-Disposition", "attachment;filename="+new String("sss".getBytes(), "ISO8859-1"));qwe
        response.setHeader("content-disposition",
                "attachment;filename=" + URLEncoder.encode("在线诊疗统计情况.xls", "UTF-8"));
        payInfoNoticeService.exportEsPrescription(os,list,listDay);
    }
}
