package com.yihu.jw.hospital.endpoint.hospital;

import com.yihu.jw.entity.base.doctor.BaseDoctorBackgroundDO;
import com.yihu.jw.entity.hospital.message.BaseBannerDO;
import com.yihu.jw.entity.hospital.suggest.BaseSuggestionDO;
import com.yihu.jw.hospital.suggest.service.BaseSuggestionService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping(value = BaseHospitalRequestMapping.suggestion.suggestionManage)
@Api(value = "反馈建议", description = "反馈建议", tags = {"互联网医院 - 反馈建议"})
public class BaseSuggestionController extends EnvelopRestEndpoint {
    @Autowired
    private BaseSuggestionService baseSuggestionService;
    @ApiOperation("查询反馈建议")
    @PostMapping(value= BaseHospitalRequestMapping.suggestion.findSuggestionByType)
    public MixEnvelop findSuggestionByType(@ApiParam(name = "type", value = "类型id", required = false)
                                @RequestParam(value = "type", required = false)String type,
                                           @ApiParam(name = "userId", value = "发送者id", required = false)
                                           @RequestParam(value = "userId", required = false)String userId,
                               @ApiParam(name = "page", value = "第几页，从1开始", required = true, defaultValue = "1")
                            @RequestParam(value = "page", required = true,defaultValue = "1")Integer page,
                               @ApiParam(name = "pageSize", value = "每页分页大小", required = true, defaultValue = "10")
                            @RequestParam(value = "pageSize", required = true,defaultValue = "10")Integer pageSize){


        try {
            MixEnvelop mixEnvelop=baseSuggestionService.findSuggestionByType(type,userId,page,pageSize);
            return mixEnvelop;
        }catch (Exception e){
            System.out.println(e.getClass());
            return failedMixEnvelopException(e);
        }
    }
    @PostMapping(value= BaseHospitalRequestMapping.suggestion.saveSuggestion)
    @ApiOperation("新增反馈建议")
    public Envelop saveSuggestion(
            @ApiParam(name = "json", value = "Json数据", required = true)
            @RequestParam(value = "json", required = false)String json)throws Exception{

        try {
            BaseSuggestionDO baseSuggestionDO = toEntity(json, BaseSuggestionDO.class);
            BaseSuggestionDO result = baseSuggestionService.createSuggestion(baseSuggestionDO);
            if (null == result){
                return failed("新增失败");
            }
            return success("新增成功");
        }catch (Exception e){
            return failedException(e);
        }
    }


    @PostMapping(value= BaseHospitalRequestMapping.suggestion.responseSuggestion)
    @ApiOperation("回复反馈建议")
    public Envelop responseSuggestion(
            @ApiParam(name = "id", value = "建议id", required = true)
    @RequestParam(value = "id", required = true)String id,
            @ApiParam(name = "content", value = "回复内容", required = true)
            @RequestParam(value = "content", required = false)String content,
            @ApiParam(name = "userId", value = "回复人id", required = true)
            @RequestParam(value = "userId", required = false)String userId)throws Exception{

        try {
            if (!StringUtils.isNoneBlank(userId)){
                userId= getUID();
            }
            String userName = getUNAME();
            baseSuggestionService.reponseSuggestion(id,content,userId,userName);
            return success("回复成功");
        }catch (Exception e){
            return failedException(e);
        }

    }
    @PostMapping(value= BaseHospitalRequestMapping.suggestion.findSuggestionById)
    @ApiOperation("回复反馈建议")
    public Envelop findSuggestionById(
            @ApiParam(name = "id", value = "建议id", required = true)
            @RequestParam(value = "id", required = true)String id) {
        try {
            BaseSuggestionDO baseSuggestionDO = baseSuggestionService.findSuggestionById(id);
            return success(baseSuggestionDO);
        }catch (Exception e){
            return failedException(e);
        }
    }
}
