package com.yihu.jw.hospital.endpoint.hospital;


import com.yihu.jw.entity.hospital.suggest.BaseSuggestionDictDO;
import com.yihu.jw.hospital.suggest.service.BaseSuggestionDictService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping(value = BaseHospitalRequestMapping.suggestionDict.suggestionDictManage)
@Api(value = "反馈建议类型", description = "反馈建议类型", tags = {"互联网医院 - 反馈建议类型"})
public class BaseSuggestionDictController extends EnvelopRestEndpoint {
    @Autowired
    private BaseSuggestionDictService baseSuggestionDictService;
    @ApiOperation("查询反馈建议类型")
    @PostMapping(value= BaseHospitalRequestMapping.suggestionDict.findSuggestionType)
    public MixEnvelop findSuggestionByType(@ApiParam(name = "name", value = "类型名称", required = false)
                                           @RequestParam(value = "name", required = false)String name,
                                           @ApiParam(name = "des", value = "类型描述", required = false)
                                           @RequestParam(value = "des", required = false)String des,
                                           @ApiParam(name = "page", value = "第几页，从1开始", required = true, defaultValue = "1")
                                           @RequestParam(value = "page", required = true,defaultValue = "1")Integer page,
                                           @ApiParam(name = "pageSize", value = "每页分页大小", required = true, defaultValue = "10")
                                           @RequestParam(value = "pageSize", required = true,defaultValue = "10")Integer pageSize){


        try {
            MixEnvelop mixEnvelop=baseSuggestionDictService.findByNameAndDes(name,des,page,pageSize);
            return mixEnvelop;
        } catch (Exception e){
            return failedMixEnvelopException(e);
        }
    }
    @PostMapping(value= BaseHospitalRequestMapping.suggestionDict.saveSuggestionType)
    @ApiOperation("新增反馈建议类型")
    public Envelop saveSuggestionType(
            @ApiParam(name = "json", value = "Json数据", required = true)
            @RequestParam(value = "json", required = false)String json)throws Exception{

        try {
            BaseSuggestionDictDO baseSuggestionDO = toEntity(json, BaseSuggestionDictDO.class);
            BaseSuggestionDictDO result = baseSuggestionDictService.createSuggestionType(baseSuggestionDO);
            if (null == result){
                return failed("新增失败");
            }
            return success("新增成功");
        }catch (Exception e){
            return failedException(e);
        }
    }
    @GetMapping(value= BaseHospitalRequestMapping.suggestionDict.delSuggestionType  )
    @ApiOperation("删除反馈建议类型")
    public Envelop delSuggestionType(
            @ApiParam(name = "id", value = "建议id", required = true)
            @RequestParam(value = "id", required = true)String id)throws Exception{

        try {
            boolean flag = baseSuggestionDictService.deleteSuggest(id);
            if (true==flag){
                return success("删除成功");
            }else {
                return failed("删除失败");
            }
        }catch (Exception e){
            return failedException(e);
        }
    }
    @PostMapping(value= BaseHospitalRequestMapping.suggestionDict.updateSuggestionType)
    @ApiOperation("更新建议类型")
    public Envelop updateSuggestionType(
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id", required = true)String id,
            @ApiParam(name = "suggestName", value = "suggestName", required = false)
            @RequestParam(value = "suggestName", required = false)String suggestName,
            @ApiParam(name = "suggestDes", value = "suggestDes", required = false)
            @RequestParam(value = "suggestDes", required = false)String suggestDes,
            @ApiParam(name = "updateUser", value = "updateUser", required = false)
            @RequestParam(value = "updateUser", required = false)String updateUser)throws Exception {

        try {
            if (!StringUtils.isNoneBlank(updateUser)){
                updateUser = getUID();
            }
            String updateUserName = getUNAME();
            BaseSuggestionDictDO result = baseSuggestionDictService.updateSuggest(id,suggestName,suggestDes,updateUser,updateUserName);
            if (null == result) {
                return failed("更新失败,礼物不存在");
            }
            return success("更新成功");
        }catch (Exception e){
            return failedException(e);
        }
    }

    @GetMapping(value= BaseHospitalRequestMapping.suggestionDict.upSuggestionType)
    @ApiOperation("上移")
    public Envelop upBanner(@ApiParam(name = "id", value = "id", required = true)
                            @RequestParam(value = "id", required = true)String id){

        try {
            BaseSuggestionDictDO baseSuggestionDictDO = new BaseSuggestionDictDO();
            String erroMessage = null;
            try {
                baseSuggestionDictDO = baseSuggestionDictService.upSuggestion(id);
            } catch (Exception e) {
                erroMessage = e.getMessage();
            }
            if (org.apache.commons.lang.StringUtils.isNotEmpty(erroMessage)){
                return failed("上移失败"+erroMessage);
            }else {
                return success("上移成功");
            }
        }catch (Exception e){
            return failedException(e);
        }

    }
    @GetMapping(value= BaseHospitalRequestMapping.suggestionDict.downSuggestionType)
    @ApiOperation("下移")
    public Envelop downBanner(@ApiParam(name = "id", value = "id", required = true)
                              @RequestParam(value = "id", required = true)String id){

        try {
            BaseSuggestionDictDO baseSuggestionDictDO = new BaseSuggestionDictDO();
            String erroMessage = null;
            try {
                baseSuggestionDictDO = baseSuggestionDictService.downSuggestion(id);
            } catch (Exception e) {
                erroMessage = e.getMessage();
            }
            if (org.apache.commons.lang.StringUtils.isNotEmpty(erroMessage)){
                return failed("下移失败"+erroMessage);
            }else {
                return success("下移成功");
            }
        }catch (Exception e){
            return failedException(e);
        }
    }
}
