package com.yihu.jw.hospital.endpoint.integrate;


import com.yihu.jw.entity.hospital.integrate.BaseHospitalActivityDO;
import com.yihu.jw.entity.hospital.integrate.BaseHospitalIntegrateDO;
import com.yihu.jw.hospital.integrate.service.BaseHospitalActivityService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import com.yihu.jw.utils.StringUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;



@RestController
@RequestMapping(value = BaseHospitalRequestMapping.BaseHospitalActivity.PREFIX)
@Api(value = "活动管理", description = "活动管理", tags = {"活动管理"})
public class BaseHospitalActivityController extends EnvelopRestEndpoint {

    @Autowired
    private BaseHospitalActivityService hospitalActivityService;

    @ApiOperation("新增或者修改")
    @PostMapping(value = BaseHospitalRequestMapping.BaseHospitalActivity.insertAndUpdate)
    public Envelop insertAndUpdate(@ApiParam(name = "json", value = "json活动实体", required = true)
                                       @RequestParam(value = "json", required = true) String json,
                                   @ApiParam(name = "integrateJson", value = "积分配置json", required = true)
                                   @RequestParam(value = "integrateJson", required = false) String integrateJson) {
        try {
            BaseHospitalActivityDO hospitalActivityDO = toEntity(json,BaseHospitalActivityDO.class);
            BaseHospitalIntegrateDO hospitalIntegrateDO = new BaseHospitalIntegrateDO();
            if (StringUtils.isNoneBlank(integrateJson)){
                 hospitalIntegrateDO= toEntity(integrateJson,BaseHospitalIntegrateDO.class);
            }
            return success(hospitalActivityService.insertAndUpdate(hospitalActivityDO,hospitalIntegrateDO));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @ApiOperation("根据id删除活动")
    @PostMapping(value = BaseHospitalRequestMapping.BaseHospitalActivity.deleteById)
    public Envelop deleteById(@ApiParam(name = "id", value = "活动id", required = true)
                                   @RequestParam(value = "id", required = true) String id) {
        try {
            hospitalActivityService.deleteById(id);
            return success();
        }catch (Exception e){
            return failedException(e);
        }
    }


    @ApiOperation("根据id查询活动详情")
    @GetMapping(value = BaseHospitalRequestMapping.BaseHospitalActivity.findById)
    public Envelop findById(@ApiParam(name = "id", value = "活动id", required = true)
                              @RequestParam(value = "id", required = true) String id) {
        try {
            return success(hospitalActivityService.findById(id));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @ApiOperation("根据活动id和用户编码查询该用户相关的活动信息")
    @GetMapping(value = BaseHospitalRequestMapping.BaseHospitalActivity.selectInfoByUserAndActivityId)
    public Envelop selectInfoByUserAndActivityId(@ApiParam(name = "id", value = "活动id", required = true)
                            @RequestParam(value = "id", required = true) String id,
                            @ApiParam(name = "user", value = "用户编码", required = true)
                            @RequestParam(value = "user", required = true) String user) {
        try {
            return success(hospitalActivityService.selectInfoByUserAndActivityId(user,id));
        }catch (Exception e){
            return failedException(e);
        }
    }


    @ApiOperation("查询活动列表")
    @GetMapping(value = BaseHospitalRequestMapping.BaseHospitalActivity.selectByCondition)
    public Envelop selectByCondition(@ApiParam(name = "page", value = "页码", required = true)
                                     @RequestParam(value = "page", required = true) Integer page,
                                     @ApiParam(name = "size", value = "大小", required = true)
                                    @RequestParam(value = "size", required = true) Integer size,
                                     @ApiParam(name = "title", value = "活动标题", required = false)
                                         @RequestParam(value = "title", required = false) String title,
                                     @ApiParam(name = "flag", value = "0禁用1启用", required = false)
                                         @RequestParam(value = "flag", required = false) Integer flag,
                                     @ApiParam(name = "status", value = "活动状态（0未开始1开始2结束）", required = false)
                                         @RequestParam(value = "status", required = false) Integer status,
                                     @ApiParam(name = "startTime", value = "开始时间", required = false)
                                         @RequestParam(value = "startTime", required = false) String startTime,
                                     @ApiParam(name = "endTime", value = "结束时间", required = false)
                                         @RequestParam(value = "endTime", required = false) String endTime,
                                     @ApiParam(name = "user", value = "用户编码", required = false)
                                         @RequestParam(value = "user", required = false) String user,
                                     @ApiParam(name = "userId", value = "当前用户id（医生查看活动需要传）", required = false)
                                         @RequestParam(value = "userId", required = false) String userId) {
        try {
            return success(hospitalActivityService.selectByCondition(title,page,size,flag,status,startTime,endTime,user,userId));
        }catch (Exception e){
            return failedException(e);
        }
    }


    @ApiOperation("更新启动状态或者上移下移置顶")
    @PostMapping(value = BaseHospitalRequestMapping.BaseHospitalActivity.updateFlagOrSort)
    public Envelop updateFlagOrSort(@ApiParam(name = "id", value = "活动id", required = true)
                              @RequestParam(value = "id", required = true) String id,
                                    @ApiParam(name = "status", value = "1启动0禁止", required = false)
                                    @RequestParam(value = "status", required = false) Integer status,
                                    @ApiParam(name = "flag", value = "1上移2下移3置顶", required = false)
                                        @RequestParam(value = "flag", required = false) Integer flag) {
        try {
            return success(hospitalActivityService.updateFlagOrSort(id,status,flag));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @ApiOperation("更新活动状态（0未开始1开始2结束）")
    @PostMapping(value = BaseHospitalRequestMapping.BaseHospitalActivity.updateStatusById)
    public Envelop updateStatusById(@ApiParam(name = "id", value = "活动id", required = true)
                                    @RequestParam(value = "id", required = true) String id,
                                    @ApiParam(name = "status", value = "0未开始1开始2结束", required = false)
                                    @RequestParam(value = "status", required = false) Integer status) {
        try {
            return success(hospitalActivityService.updateStatusById(id,status));
        }catch (Exception e){
            return failedException(e);
        }
    }
}