package com.yihu.jw.hospital.endpoint.integrate;


import com.yihu.jw.entity.hospital.integrate.BaseHospitalGoodsDO;
import com.yihu.jw.entity.hospital.integrate.BaseHospitalMedalDO;
import com.yihu.jw.hospital.integrate.service.BaseHospitalGoodsService;
import com.yihu.jw.hospital.integrate.service.BaseHospitalMedalService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;


@RestController
@RequestMapping(value = BaseHospitalRequestMapping.BaseHospitalGoods.PREFIX)
@Api(value = "兑换商品配置", description = "兑换商品配置", tags = {"兑换商品配置"})
public class BaseHospitalGoodsController extends EnvelopRestEndpoint {

    @Autowired
    private BaseHospitalGoodsService hospitalGoodsService;

    @ApiOperation("新增或者修改商品")
    @PostMapping(value = BaseHospitalRequestMapping.BaseHospitalGoods.insertAndUpdate)
    public Envelop insertAndUpdate(@ApiParam(name = "json", value = "json积分实体", required = true)
                                       @RequestParam(value = "json", required = true) String json) {
        try {
            BaseHospitalGoodsDO hospitalGoodsDO = toEntity(json, BaseHospitalGoodsDO.class);
            return success(hospitalGoodsService.insertAndUpdate(hospitalGoodsDO));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @ApiOperation("商品上架下架")
    @PostMapping(value = BaseHospitalRequestMapping.BaseHospitalGoods.updateGoodsStatus)
    public Envelop updateGoodsStatus(@ApiParam(name = "id", value = "商品id", required = true)
                                   @RequestParam(value = "id", required = true) String id,
                                   @ApiParam(name = "status", value = "0下架1上架", required = true)
                                   @RequestParam(value = "status", required = true) Integer status) {
        try {
            return success(hospitalGoodsService.updateGoodsStatus(id,status));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @ApiOperation("根据id删除商品")
    @PostMapping(value = BaseHospitalRequestMapping.BaseHospitalGoods.deleteById)
    public Envelop deleteById(@ApiParam(name = "id", value = "商品id", required = true)
                                   @RequestParam(value = "id", required = true) String id) {
        try {
            hospitalGoodsService.deleteById(id);
            return success();
        }catch (Exception e){
            return failedException(e);
        }
    }


    @ApiOperation("根据id查询商品详情")
    @GetMapping(value = BaseHospitalRequestMapping.BaseHospitalGoods.findById)
    public Envelop findById(@ApiParam(name = "id", value = "商品id", required = true)
                              @RequestParam(value = "id", required = true) String id) {
        try {
            return success(hospitalGoodsService.findById(id));
        }catch (Exception e){
            return failedException(e);
        }
    }


    @ApiOperation("查询商品管理列表")
    @GetMapping(value = BaseHospitalRequestMapping.BaseHospitalGoods.selectByCondition)
    public Envelop selectByCondition(@ApiParam(name = "page", value = "页码", required = true)
                                     @RequestParam(value = "page", required = true) Integer page,
                                     @ApiParam(name = "size", value = "大小", required = true)
                                    @RequestParam(value = "size", required = true) Integer size,
                                     @ApiParam(name = "name", value = "商品名称", required = false)
                                         @RequestParam(value = "name", required = false) String name,
                                     @ApiParam(name = "type", value = "商品类型（1成长勋章2实物商品）", required = false)
                                         @RequestParam(value = "type", required = false) Integer type,
                                     @ApiParam(name = "status", value = "上架状态（0下架1上架）", required = false)
                                         @RequestParam(value = "status", required = false) Integer status,
                                     @ApiParam(name = "user", value = "用户编码", required = false)
                                         @RequestParam(value = "user", required = false) String user) {
        try {
            return success(hospitalGoodsService.selectByCondition(name,type,status,user,page,size));
        }catch (Exception e){
            return failedException(e);
        }
    }


    @ApiOperation("兑换商品")
    @PostMapping(value = BaseHospitalRequestMapping.BaseHospitalGoods.exchageGoods)
    public Envelop selectMedalByUserId(@ApiParam(name = "goodsId", value = "商品id", required = true)
                                     @RequestParam(value = "goodsId", required = true) String goodsId,
                                     @ApiParam(name = "user", value = "用户编码", required = true)
                                     @RequestParam(value = "user", required = true) String user,
                                     @ApiParam(name = "type", value = "用户类型（1医生2患者）", required = true)
                                     @RequestParam(value = "type", required = false) Integer type,
                                       @ApiParam(name = "name", value = "用户姓名", required = true)
                                           @RequestParam(value = "name", required = true) String name,
                                     @ApiParam(name = "quantity", value = "数量", required = true)
                                           @RequestParam(value = "quantity", required = false) Integer quantity) {
        try {
            return success(hospitalGoodsService.exchageGoods(goodsId,user,type,name,quantity));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @ApiOperation("查询商品兑换记录")
    @GetMapping(value = BaseHospitalRequestMapping.BaseHospitalGoods.selectGoodsExchageByCondition)
    public Envelop selectGoodsExchageByCondition(@ApiParam(name = "page", value = "页码", required = true)
                                     @RequestParam(value = "page", required = true) Integer page,
                                     @ApiParam(name = "size", value = "大小", required = true)
                                     @RequestParam(value = "size", required = true) Integer size,
                                     @ApiParam(name = "name", value = "商品名称", required = false)
                                     @RequestParam(value = "name", required = false) String name,
                                     @ApiParam(name = "user", value = "用户编码", required = false)
                                     @RequestParam(value = "user", required = false) String user,
                                     @ApiParam(name = "startTime", value = "开始时间", required = false)
                                     @RequestParam(value = "startTime", required = false) String startTime,
                                     @ApiParam(name = "endTime", value = "结束时间", required = false)
                                     @RequestParam(value = "endTime", required = false) String endTime) {
        try {
            return success(hospitalGoodsService.selectGoodsExchageByCondition(name,user,startTime,endTime,page,size));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @ApiOperation("根据兑换记录id获取商品兑换记录详情")
    @GetMapping(value = BaseHospitalRequestMapping.BaseHospitalGoods.findGoodsExchageById)
    public Envelop findGoodsExchageById(@ApiParam(name = "id", value = "商品兑换id", required = true)
                                                 @RequestParam(value = "id", required = true) String id) {
        try {
            return success(hospitalGoodsService.findGoodsExchageById(id));
        }catch (Exception e){
            return failedException(e);
        }
    }

}