package com.yihu.jw.hospital.endpoint.integrate;


import com.yihu.jw.entity.hospital.integrate.BaseHospitalMedalDO;
import com.yihu.jw.hospital.integrate.service.BaseHospitalMedalService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;


@RestController
@RequestMapping(value = BaseHospitalRequestMapping.BaseHospitalMedal.PREFIX)
@Api(value = "勋章配置", description = "勋章配置", tags = {"勋章配置"})
public class BaseHospitalMedalController extends EnvelopRestEndpoint {

    @Autowired
    private BaseHospitalMedalService hospitalMedalService;

    @ApiOperation("新增或者修改")
    @PostMapping(value = BaseHospitalRequestMapping.BaseHospitalMedal.insertAndUpdate)
    public Envelop insertAndUpdate(@ApiParam(name = "json", value = "json积分实体", required = true)
                                       @RequestParam(value = "json", required = true) String json) {
        try {
            BaseHospitalMedalDO hospitalMedalDO = toEntity(json, BaseHospitalMedalDO.class);
            return success(hospitalMedalService.insertAndUpdate(hospitalMedalDO));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @ApiOperation("根据id删除勋章配置")
    @PostMapping(value = BaseHospitalRequestMapping.BaseHospitalMedal.deleteById)
    public Envelop deleteById(@ApiParam(name = "id", value = "勋章配置id", required = true)
                                   @RequestParam(value = "id", required = true) String id) {
        try {
            hospitalMedalService.deleteById(id);
            return success();
        }catch (Exception e){
            return failedException(e);
        }
    }


    @ApiOperation("根据id查询勋章配置详情")
    @GetMapping(value = BaseHospitalRequestMapping.BaseHospitalMedal.findById)
    public Envelop findById(@ApiParam(name = "id", value = "勋章配置id", required = true)
                              @RequestParam(value = "id", required = true) String id) {
        try {
            return success(hospitalMedalService.findById(id));
        }catch (Exception e){
            return failedException(e);
        }
    }


    @ApiOperation("查询勋章配置列表")
    @GetMapping(value = BaseHospitalRequestMapping.BaseHospitalMedal.selectByCondition)
    public Envelop selectByCondition(@ApiParam(name = "page", value = "页码", required = true)
                                     @RequestParam(value = "page", required = true) Integer page,
                                     @ApiParam(name = "size", value = "大小", required = true)
                                    @RequestParam(value = "size", required = true) Integer size,
                                     @ApiParam(name = "name", value = "勋章名称", required = false)
                                         @RequestParam(value = "name", required = false) String name) {
        try {
            return success(hospitalMedalService.selectByCondition(name,page,size));
        }catch (Exception e){
            return failedException(e);
        }
    }


    @ApiOperation("医生端勋章列表")
    @GetMapping(value = BaseHospitalRequestMapping.BaseHospitalMedal.selectMedalByUserId)
    public Envelop selectMedalByUserId(@ApiParam(name = "page", value = "页码", required = true)
                                     @RequestParam(value = "page", required = true) Integer page,
                                     @ApiParam(name = "size", value = "大小", required = true)
                                     @RequestParam(value = "size", required = true) Integer size,
                                     @ApiParam(name = "doctorId", value = "医生id", required = false)
                                     @RequestParam(value = "doctorId", required = false) String doctorId) {
        try {
            return success(hospitalMedalService.selectMedalByUserId(doctorId,page,size));
        }catch (Exception e){
            return failedException(e);
        }
    }

}