package com.yihu.jw.hospital.endpoint.integrate;

import com.yihu.jw.hospital.integrate.service.BaseHospitalUserActivityService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;


@RestController
@RequestMapping(value = BaseHospitalRequestMapping.BaseHospitalUserActivity.PREFIX)
@Api(value = "用户参与活动", description = "用户参与活动", tags = {"用户参与活动"})
public class BaseHospitalUserActivityController extends EnvelopRestEndpoint {

    @Autowired
    private BaseHospitalUserActivityService userActivityService;

    /**
     * 查询活动报名记录
     *
     * @param page
     * @param size
     * @param title
     * @return
     */
    @GetMapping(value= BaseHospitalRequestMapping.BaseHospitalUserActivity.selectUserActivityByCondition)
    @ApiOperation("查询活动报名记录")
    public Envelop selectUserActivityByCondition(@ApiParam(name = "page", value = "页码", required = true)
                                          @RequestParam(value = "page", required = true) Integer page,
                                      @ApiParam(name = "size", value = "大小", required = true)
                                          @RequestParam(value = "size", required = true) Integer size,
                                      @ApiParam(name = "title", value = "活动标题", required = false)
                                          @RequestParam(value = "title", required = false) String title) {
        return success(userActivityService.selectUserActivityByCondition(title,page,size));
    }

    @GetMapping(value= BaseHospitalRequestMapping.BaseHospitalUserActivity.findUserActivityById)
    @ApiOperation("根据活动报名记录id获取活动报名情况详情")
    public Envelop findUserActivityById(@ApiParam(name = "userActivityId", value = "活动报名记录id", required = true)
                                                 @RequestParam(value = "userActivityId", required = true) String userActivityId) {
        return success(userActivityService.findUserActivityById(userActivityId));
    }

    /**
     * 查询某个活动下某个用户获取的积分详情
     *
     * @param page
     * @param size
     * @param activityId
     * @return
     */
    @GetMapping(value= BaseHospitalRequestMapping.BaseHospitalUserActivity.selectUserIntegrateByCondition)
    @ApiOperation("查询某个活动下某个用户获取的积分详情")
    public Envelop selectUserIntegrateByCondition(@ApiParam(name = "page", value = "页码", required = true)
                                                 @RequestParam(value = "page", required = true) Integer page,
                                                 @ApiParam(name = "size", value = "大小", required = true)
                                                 @RequestParam(value = "size", required = true) Integer size,
                                                 @ApiParam(name = "user", value = "用户id", required = true)
                                                 @RequestParam(value = "user", required = true) String user,
                                                 @ApiParam(name = "activityId", value = "活动id", required = true)
                                                 @RequestParam(value = "activityId", required = true) String activityId) {
        return success(userActivityService.selectUserIntegrateByCondition(user,activityId,page,size));
    }

    @GetMapping(value= BaseHospitalRequestMapping.BaseHospitalUserActivity.selectUserActivityByUser)
    @ApiOperation("查询某个用户参与的活动情况")
    public Envelop selectUserActivityByUser(@ApiParam(name = "page", value = "页码", required = true)
                                                  @RequestParam(value = "page", required = true) Integer page,
                                                  @ApiParam(name = "size", value = "大小", required = true)
                                                  @RequestParam(value = "size", required = true) Integer size,
                                                  @ApiParam(name = "user", value = "用户id", required = true)
                                                  @RequestParam(value = "user", required = true) String user) {
        return success(userActivityService.selectUserActivityByUser(user,page,size));
    }


    @PostMapping(value= BaseHospitalRequestMapping.BaseHospitalUserActivity.receiveIntegrate)
    @ApiOperation("用户领取积分")
    public Envelop receiveIntegrate(@ApiParam(name = "activityId", value = "活动id", required = true)
                                            @RequestParam(value = "activityId", required = true) String activityId,
                                            @ApiParam(name = "user", value = "用户id", required = true)
                                            @RequestParam(value = "user", required = true) String user) {
        return success(userActivityService.receiveIntegrate(user,activityId));
    }

    @PostMapping(value= BaseHospitalRequestMapping.BaseHospitalUserActivity.takeActivity)
    @ApiOperation("用户报名")
    public Envelop takeActivity(@ApiParam(name = "activityId", value = "活动id", required = true)
                                    @RequestParam(value = "activityId", required = true) String activityId,
                                    @ApiParam(name = "user", value = "用户id", required = true)
                                    @RequestParam(value = "user", required = true) String user,
                                    @ApiParam(name = "name", value = "用户名", required = true)
                                    @RequestParam(value = "name", required = true) String name,
                                    @ApiParam(name = "type", value = "类型（1医生2患者）", required = true)
                                    @RequestParam(value = "type", required = true) Integer type) throws Exception {
        return success(userActivityService.takeActivity(user,name,type,activityId));
    }


    @GetMapping(value= BaseHospitalRequestMapping.BaseHospitalUserActivity.selectUserIntegrateListByCondition)
    @ApiOperation("医生-个人积分榜单/接诊榜单/单个活动榜单")
    public Envelop selectUserIntegrateListByCondition(@ApiParam(name = "page", value = "页码", required = true)
                                @RequestParam(value = "page", required = true) Integer page,
                                @ApiParam(name = "size", value = "大小", required = true)
                                @RequestParam(value = "size", required = true) Integer size,
                                @ApiParam(name = "user", value = "用户id", required = true)
                                @RequestParam(value = "user", required = true) String user,
                                @ApiParam(name = "businessCode", value = "业务code", required = false)
                                @RequestParam(value = "businessCode", required = false) String businessCode,
                                @ApiParam(name = "activityId", value = "活动id", required = false)
                                @RequestParam(value = "activityId", required = false) String activityId) throws Exception {
        return success(userActivityService.selectUserIntegrateListByCondition(user,businessCode,activityId,page,size));
    }

    @GetMapping(value= BaseHospitalRequestMapping.BaseHospitalUserActivity.selectDeptRomByCondition)
    @ApiOperation("医生-科室热门榜")
    public Envelop selectDeptRomByCondition(@ApiParam(name = "page", value = "页码", required = true)
                                                      @RequestParam(value = "page", required = true) Integer page,
                                                      @ApiParam(name = "size", value = "大小", required = true)
                                                      @RequestParam(value = "size", required = true) Integer size,
                                                      @ApiParam(name = "businessCode", value = "业务code", required = false)
                                                      @RequestParam(value = "businessCode", required = false) String businessCode) throws Exception {
        return success(userActivityService.selectDeptRomByCondition(businessCode,page,size));
    }


    @GetMapping(value= BaseHospitalRequestMapping.BaseHospitalUserActivity.selectIntegrateInfoByUser)
    @ApiOperation("根据用户编码获取积分账户信息及明细")
    public Envelop selectIntegrateInfoByUser(@ApiParam(name = "page", value = "页码", required = true)
                                            @RequestParam(value = "page", required = true) Integer page,
                                            @ApiParam(name = "size", value = "大小", required = true)
                                            @RequestParam(value = "size", required = true) Integer size,
                                            @ApiParam(name = "user", value = "用户编码", required = false)
                                            @RequestParam(value = "user", required = false) String user,
                                            @ApiParam(name = "startTime", value = "开始时间", required = false)
                                            @RequestParam(value = "startTime", required = false) String startTime,
                                            @ApiParam(name = "endTime", value = "结束时间", required = false)
                                            @RequestParam(value = "endTime", required = false) String endTime) throws Exception {
        return success(userActivityService.selectIntegrateInfoByUser(user,startTime,endTime,page,size));
    }

}