package com.yihu.jw.hospital.endpoint.knowledge;


import com.yihu.jw.entity.knowledge.BaseKnowledgeDict;
import com.yihu.jw.entity.knowledge.BaseKnowledgeDictRelation;
import com.yihu.jw.knowledge.dao.BaseKnowledgeDictDao;
import com.yihu.jw.knowledge.service.BaseKnowledgeDictService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;


@RestController
@RequestMapping(value = BaseHospitalRequestMapping.BaseKnowledgeDict.PREFIX)
@Api(value = "知识库-字典配置", description = "知识库-字典配置", tags = {"知识库-字典配置"})
public class BaseKnowledgeDictController extends EnvelopRestEndpoint {

    @Autowired
    private BaseKnowledgeDictService dictService;

    @ApiOperation("新增/更新字典")
    @PostMapping(value= BaseHospitalRequestMapping.BaseKnowledgeDict.insertAndUpdateDict)
    public Envelop insertAndUpdateDict(@ApiParam(name = "json", value = "字典实体json", required = true)
                                   @RequestParam(value = "json", required = true)String json){

        try {
            BaseKnowledgeDict dict = toEntity(json,BaseKnowledgeDict.class);
            return success(dictService.insertAndUpdateDict(dict));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @ApiOperation("删除字典")
    @PostMapping(value= BaseHospitalRequestMapping.BaseKnowledgeDict.deleteDict)
    public Envelop deleteDict(@ApiParam(name = "id", value = "字典id", required = true)
                                   @RequestParam(value = "id", required = true)String id){

        try {
            dictService.deleteDict(id);
            return success();
        }catch (Exception e){
            return failedException(e);
        }
    }
    @ApiOperation("查询字典列表")
    @GetMapping(value= BaseHospitalRequestMapping.BaseKnowledgeDict.selectDictList)
    public Envelop selectDictList(@ApiParam(name = "name", value = "字典名称", required = false)
                              @RequestParam(value = "name", required = false)String name,
                              @ApiParam(name = "type", value = "字典类型", required = false)
                              @RequestParam(value = "type", required = false)String type,
                              @ApiParam(name = "status", value = "1已设置0未设置", required = false)
                                  @RequestParam(value = "status", required = false)String status,
                              @ApiParam(name = "page", value = "页码", required = false)
                                  @RequestParam(value = "page", required = false)Integer page,
                              @ApiParam(name = "size", value = "大小", required = false)
                                  @RequestParam(value = "size", required = false)Integer size){

        try {
            return success(dictService.selectDictList(name,type,status,page,size));
        }catch (Exception e){
            return failedException(e);
        }
    }


    @ApiOperation("根据id查询某一个字典详情(关联数据)")
    @GetMapping(value= BaseHospitalRequestMapping.BaseKnowledgeDict.selectById)
    public Envelop selectById(@ApiParam(name = "id", value = "字典id", required = true)
                              @RequestParam(value = "id", required = true)String id){

        try {
            return success(dictService.selectById(id));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @ApiOperation("字典设置排序")
    @GetMapping(value= BaseHospitalRequestMapping.BaseKnowledgeDict.setDictSortById)
    public Envelop setDictSortById(@ApiParam(name = "id", value = "字典id", required = true)
                              @RequestParam(value = "id", required = true)String id,
                                   @ApiParam(name = "flag", value = "1上移2下移", required = true)
                                   @RequestParam(value = "flag", required = true)Integer flag){

        try {
            return success(dictService.setDictSortById(id,flag));
        }catch (Exception e){
            return failedException(e);
        }
    }


    @ApiOperation("设置关联关系及取消关联关系")
    @GetMapping(value= BaseHospitalRequestMapping.BaseKnowledgeDict.setDictRelation)
    public Envelop setDictRelation(@ApiParam(name = "json", value = "json关联实体", required = true)
                                   @RequestParam(value = "json", required = true)String json,
                                   @ApiParam(name = "status", value = "1已设置0未设置", required = true)
                                   @RequestParam(value = "status", required = true)Integer status){

        try {
            BaseKnowledgeDictRelation dictRelation = toEntity(json,BaseKnowledgeDictRelation.class);
            dictService.setDictRelation(dictRelation,status);
            return success();
        }catch (Exception e){
            return failedException(e);
        }
    }
}