package com.yihu.jw.hospital.endpoint.knowledge;


import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.entity.knowledge.*;
import com.yihu.jw.knowledge.service.BaseKnowledgeQuestionService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;


@RestController
@RequestMapping(value = BaseHospitalRequestMapping.BaseKnowledgeQuestion.PREFIX)
@Api(value = "知识库-问题配置", description = "知识库-问题配置", tags = {"知识库-问题配置"})
public class BaseKnowledgeQuestionController extends EnvelopRestEndpoint {

    @Autowired
    private BaseKnowledgeQuestionService questionService;

    @ApiOperation("新增/更新常见问题")
    @PostMapping(value= BaseHospitalRequestMapping.BaseKnowledgeQuestion.insertAndUpdateQuestion)
    public Envelop insertAndUpdateQuestion(@ApiParam(name = "json", value = "常见问题实体json", required = true)
                                       @RequestParam(value = "json", required = true)String json){

        try {
            BaseKnowledgeQuestion question = toEntity(json,BaseKnowledgeQuestion.class);
            return success(questionService.insertAndUpdateQuestion(question));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @ApiOperation("删除常见问题")
    @PostMapping(value= BaseHospitalRequestMapping.BaseKnowledgeQuestion.deleteQuestion)
    public Envelop deleteQuestion(@ApiParam(name = "id", value = "常见问题id", required = true)
                              @RequestParam(value = "id", required = true)String id){

        try {
            questionService.deleteQuestion(id);
            return success();
        }catch (Exception e){
            return failedException(e);
        }
    }
    @ApiOperation("查询常见问题列表")
    @GetMapping(value= BaseHospitalRequestMapping.BaseKnowledgeQuestion.selectQuestionList)
    public Envelop selectQuestionList(@ApiParam(name = "name", value = "问题名称", required = false)
                                  @RequestParam(value = "name", required = false)String name,
                                  @ApiParam(name = "type", value = "字典类型", required = false)
                                  @RequestParam(value = "type", required = false)String type,
                                  @ApiParam(name = "status", value = "1开启0关闭", required = false)
                                  @RequestParam(value = "status", required = false)String status,
                                  @ApiParam(name = "page", value = "页码", required = false)
                                  @RequestParam(value = "page", required = false)Integer page,
                                  @ApiParam(name = "size", value = "大小", required = false)
                                  @RequestParam(value = "size", required = false)Integer size){

        try {
            return success(questionService.selectQuestionList(name,type,status,page,size));
        }catch (Exception e){
            return failedException(e);
        }
    }


    @ApiOperation("根据id查询某一个常见问题详情")
    @GetMapping(value= BaseHospitalRequestMapping.BaseKnowledgeQuestion.selectById)
    public Envelop selectById(@ApiParam(name = "id", value = "常见问题id", required = true)
                              @RequestParam(value = "id", required = true)String id){

        try {
            return success(questionService.selectById(id));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @ApiOperation("常见问题设置排序")
    @GetMapping(value= BaseHospitalRequestMapping.BaseKnowledgeQuestion.setQuestionSortById)
    public Envelop setQuestionSortById(@ApiParam(name = "id", value = "字典id", required = true)
                                   @RequestParam(value = "id", required = true)String id,
                                   @ApiParam(name = "flag", value = "1上移2下移", required = true)
                                   @RequestParam(value = "flag", required = true)Integer flag){

        try {
            return success(questionService.setQuestionSortById(id,flag));
        }catch (Exception e){
            return failedException(e);
        }
    }

    //======================问题集===================================

    @ApiOperation("新增/更新问题集合")
    @PostMapping(value= BaseHospitalRequestMapping.BaseKnowledgeQuestion.insertAndUpdateQuestions)
    public Envelop insertAndUpdateQuestions(@ApiParam(name = "json", value = "问题集实体json", required = true)
                                           @RequestParam(value = "json", required = true)String json,
                                            @ApiParam(name = "relationJson", value = "问题集关联实体json", required = true)
                                            @RequestParam(value = "relationJson", required = true)String relationJson){

        try {
            BaseKnowledgeQuestions questions = toEntity(json,BaseKnowledgeQuestions.class);
            JSONArray array = JSONArray.parseArray(relationJson);
            List<BaseKnowledgeQuestionsRelation> questionsRelationList = new ArrayList<>();
            for (int i=0;i<array.size();i++){
                JSONObject object = array.getJSONObject(i);
                BaseKnowledgeQuestionsRelation questionsRelation = toEntity(object.toJSONString(),BaseKnowledgeQuestionsRelation.class);
                questionsRelationList.add(questionsRelation);
            }
            return success(questionService.insertAndUpdateQuestions(questions,questionsRelationList));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @ApiOperation("删除问题集合")
    @PostMapping(value= BaseHospitalRequestMapping.BaseKnowledgeQuestion.deleteQuestions)
    public Envelop deleteQuestions(@ApiParam(name = "id", value = "问题集id", required = true)
                                  @RequestParam(value = "id", required = true)String id){

        try {
            questionService.deleteQuestions(id);
            return success();
        }catch (Exception e){
            return failedException(e);
        }
    }
    @ApiOperation("查询问题集合列表")
    @GetMapping(value= BaseHospitalRequestMapping.BaseKnowledgeQuestion.selectQuestionsList)
    public Envelop selectQuestionsList(@ApiParam(name = "name", value = "字典名称", required = false)
                                      @RequestParam(value = "name", required = false)String name,
                                      @ApiParam(name = "type", value = "字典类型", required = false)
                                      @RequestParam(value = "type", required = false)String type,
                                      @ApiParam(name = "status", value = "1开启0关闭", required = false)
                                      @RequestParam(value = "status", required = false)String status,
                                      @ApiParam(name = "page", value = "页码", required = false)
                                      @RequestParam(value = "page", required = false)Integer page,
                                      @ApiParam(name = "size", value = "大小", required = false)
                                      @RequestParam(value = "size", required = false)Integer size){

        try {
            return success(questionService.selectQuestionsList(name,type,status,page,size));
        }catch (Exception e){
            return failedException(e);
        }
    }


    @ApiOperation("根据id查询某一个问题集合详情")
    @GetMapping(value= BaseHospitalRequestMapping.BaseKnowledgeQuestion.selectQuestionsById)
    public Envelop selectQuestionsById(@ApiParam(name = "id", value = "问题集合id", required = true)
                              @RequestParam(value = "id", required = true)String id){

        try {
            return success(questionService.selectQuestionsById(id));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @ApiOperation("问题集合设置排序")
    @GetMapping(value= BaseHospitalRequestMapping.BaseKnowledgeQuestion.setQuestionsSortById)
    public Envelop setQuestionsSortById(@ApiParam(name = "id", value = "字典id", required = true)
                                       @RequestParam(value = "id", required = true)String id,
                                       @ApiParam(name = "flag", value = "1上移2下移3置顶", required = true)
                                       @RequestParam(value = "flag", required = true)Integer flag){

        try {
            return success(questionService.setQuestionsSortById(id,flag));
        }catch (Exception e){
            return failedException(e);
        }
    }
    //--------------问题关联配置--------------------------

    @ApiOperation("问题流程关联配置")
    @PostMapping(value= BaseHospitalRequestMapping.BaseKnowledgeQuestion.setQuestionConfigurationRelation)
    public Envelop setQuestionConfigurationRelation(@ApiParam(name = "json", value = "问题关联配置JSON", required = true)
                                                    @RequestParam(value = "json", required = true)String json,
                                                    @ApiParam(name = "questionId", value = "问题id", required = true)
                                                    @RequestParam(value = "questionId", required = true)String questionId,
                                                    @ApiParam(name = "status", value = "问题id", required = true)
                                                    @RequestParam(value = "status", required = true)Integer status){
        try {
            JSONArray array = JSONArray.parseArray(json);
            List<BaseKnowledgeFlowConfigurationRelation> flowConfigurationRelations = new ArrayList<>();
            for (int i=0;i<array.size();i++){
                BaseKnowledgeFlowConfigurationRelation flowConfigurationRelation = toEntity(array.getJSONObject(i).toJSONString(),BaseKnowledgeFlowConfigurationRelation.class);
                flowConfigurationRelations.add(flowConfigurationRelation);
            }
            return success(questionService.setQuestionConfigurationRelation(flowConfigurationRelations,questionId,status));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @ApiOperation("根据问题id获取问题关联配置")
    @GetMapping(value= BaseHospitalRequestMapping.BaseKnowledgeQuestion.selectFlowConfigurationRelations)
    public Envelop selectFlowConfigurationRelations(@ApiParam(name = "questionId", value = "问题id", required = true)
                                                    @RequestParam(value = "questionId", required = true)String questionId){

        try {
            return success(questionService.selectFlowConfigurationRelations(questionId));
        }catch (Exception e){
            return failedException(e);
        }
    }
}