package com.yihu.jw.hospital.endpoint.message;

import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.entity.hospital.message.BaseBannerDoctorDO;
import com.yihu.jw.hospital.message.dao.BaseBannerDoctorDao;
import com.yihu.jw.hospital.message.service.BaseBannerDoctorService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.apache.commons.lang.StringUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@RestController
@RequestMapping(value = BaseHospitalRequestMapping.DoctorBanner.doctorBanner)
@Api(value = "患者赠送锦旗", description = "患者锦旗管理", tags = {"互联网医院 - 患者锦旗管理接口"})
public class BannerDoctorController extends EnvelopRestEndpoint {
    @Autowired
    private BaseBannerDoctorService baseBannerDoctorService;
    @ApiOperation("批量赠送锦旗")
    @PostMapping(value= BaseHospitalRequestMapping.DoctorBanner.giveBanner)
    public ObjEnvelop giveBanner(@ApiParam(name = "bannerIds", value = "bannerIds", required = true)
                                  @RequestParam(value = "bannerIds", required = true)String bannerIds,
                                 @ApiParam(name = "doctor", value = "doctor", required = false)
                                  @RequestParam(value = "doctor", required = false)String doctor,
                                 @ApiParam(name = "doctorName", value = "doctorName", required = false)
                                  @RequestParam(value = "doctorName", required = false)String doctorName,
                                 @ApiParam(name = "content", value = "content", required = false)
                                  @RequestParam(value = "content", required = false)String content,
                                 @ApiParam(name = "type", value = "type", required = false)
                                  @RequestParam(value = "type", required = false)Integer type,
                                 @ApiParam(name = "key", value = "key", required = false)
                                  @RequestParam(value = "key", required = false)String key,
                                 @ApiParam(name = "value", value = "value", required = false)
                                  @RequestParam(value = "value", required = false)String value) throws Exception{

        try {
            ObjEnvelop objEnvelop = new ObjEnvelop();
            List<String> list = new ArrayList<>();
            if(StringUtils.isNotEmpty(bannerIds)){
                list =  Arrays.asList(bannerIds.split(","));
            }
            String patientId= getUID();
            objEnvelop = baseBannerDoctorService.bannerGive(patientId,list,content,doctor,doctorName,type,key,value);
            return objEnvelop;
        }catch (Exception e){
            return failedObjEnvelopException(e);
        }
    }
    @ApiOperation("查询赠送锦旗")
    @PostMapping(value= BaseHospitalRequestMapping.DoctorBanner.getBanner)
    public Envelop getBanners(@ApiParam(name = "doctor", value = "doctor", required = false)
                                     @RequestParam(value = "doctor", required = false)String doctor){

        try {
            JSONObject jsonObject = baseBannerDoctorService.getDoctorBanners(doctor);
            return success(jsonObject);
        }catch (Exception e){
            return failedObjEnvelopException(e);
        }
    }
    @ApiOperation("查询未读消息")
    @PostMapping(value= BaseHospitalRequestMapping.DoctorBanner.getMessageNotRead)
    public Envelop getNotRead(@ApiParam(name = "doctor", value = "doctor", required = false)
                              @RequestParam(value = "doctor", required = false)String doctor,
                              @ApiParam(name = "page", value = "page", required = false)
                              @RequestParam(value = "page", required = false,defaultValue = "1")Integer page,
                              @ApiParam(name = "pageSize", value = "pageSize", required = false)
                                  @RequestParam(value = "pageSize", required = false,defaultValue = "10")Integer pageSize){

        try {
            JSONObject jsonObject = baseBannerDoctorService.getMessageNotRead(doctor,page,pageSize);
            return success(jsonObject);
        }catch (Exception e){
            return failedObjEnvelopException(e);
        }
    }
    @ApiOperation("修改消息已读状态")
    @PostMapping(value= BaseHospitalRequestMapping.DoctorBanner.changeIsRead)
    public Envelop changeIsRead(@ApiParam(name = "id", value = "id", required = false)
                              @RequestParam(value = "id", required = false)String id,
                                @ApiParam(name = "doctor", value = "doctor", required = false)
                                @RequestParam(value = "doctor", required = false)String doctor){

        try {
            baseBannerDoctorService.isRead(id,doctor);
            return success("success");
        }catch (Exception e){
            return failedObjEnvelopException(e);
        }
    }
}
