package com.yihu.jw.hospital.endpoint.message;

import com.yihu.jw.entity.hospital.message.BaseBannerDO;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.hospital.message.service.BaseBannerService;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping(value = BaseHospitalRequestMapping.BaseBannerssage.baseBanner)
@Api(value = "医生锦旗管理", description = "医生锦旗管理", tags = {"互联网医院 - 医生锦旗管理接口"})
public class BaseBannerController extends EnvelopRestEndpoint{

    private static Logger logger = LoggerFactory.getLogger(SystemMessageController.class);
    @Autowired
    private BaseBannerService baseBannerService;

    @ApiOperation("查询锦旗")
    @PostMapping(value= BaseHospitalRequestMapping.BaseBannerssage.queryByAndStatud)
    public Envelop queryByAndStatud(@ApiParam(name = "name", value = "锦旗名称")
                                                   @RequestParam(value = "name", required = false) String name,
                                        @ApiParam(name = "status", value = "锦旗状态")
                                                   @RequestParam(value = "status", required = false) String status,
                                        @ApiParam(name = "page", value = "第几页，从1开始", required = true, defaultValue = "1")
                                                   @RequestParam(value = "page", required = true,defaultValue = "1")Integer page,
                                        @ApiParam(name = "pageSize", value = "每页分页大小", required = true, defaultValue = "10")
                                                   @RequestParam(value = "pageSize", required = true,defaultValue = "10")Integer pageSize){


            try {
                return baseBannerService.queryBannerByNameAndStatus(name,status,page,pageSize);
            }catch (Exception e){
                return failedException(e);
            }

    }
    @PostMapping(value= BaseHospitalRequestMapping.BaseBannerssage.createBanner)
    @ApiOperation("新增医生锦旗")
    public Envelop createBanner(
            @ApiParam(name = "json", value = "Json数据", required = true)
            @RequestParam(value = "json", required = false)String json)throws Exception{

        try {
            BaseBannerDO bannerDO = toEntity(json,BaseBannerDO.class);
            BaseBannerDO result = baseBannerService.creatBanner(bannerDO);
            if (null == result){
                return failed("新增失败");
            }
            return success("新增成功");
        }catch (Exception e){
            return failedException(e);
        }

    }

    @PostMapping(value= BaseHospitalRequestMapping.BaseBannerssage.updateBanner)
    @ApiOperation("更新锦旗信息")
    public Envelop updateMessage(
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id", required = true)String id,
            @ApiParam(name = "name", value = "name", required = false)
            @RequestParam(value = "name", required = false)String name,
            @ApiParam(name = "img", value = "img", required = false)
            @RequestParam(value = "img", required = false)String img,
            @ApiParam(name = "status", value = "status", required = false)
            @RequestParam(value = "status", required = false)String status,
            @ApiParam(name = "description", value = "description", required = false)
            @RequestParam(value = "description", required = false)String description)throws Exception {

        try {
            String updateUser = getUID();
            String updateUserName = getUNAME();
            BaseBannerDO result = baseBannerService.updateBanner(id,name,img,status,updateUser,updateUserName,description);
            if (null == result) {
                return failed("更新失败,礼物不存在");
            }
            return success("更新成功");
        }catch (Exception e){
            return failedException(e);
        }
    }
    @GetMapping(value= BaseHospitalRequestMapping.BaseBannerssage.deleteBanner)
    @ApiOperation("删除礼物信息")
    public Envelop deleteBanner(@ApiParam(name = "id", value = "id", required = true)
                                    @RequestParam(value = "id", required = true)String id){

        try {
            boolean flag = baseBannerService.deleteBanner(id);
            if (true==flag){
                return success("删除成功");
            }else {
                return failed("删除失败");
            }
        }catch (Exception e){
            return failedException(e);
        }

    }
    @GetMapping(value= BaseHospitalRequestMapping.BaseBannerssage.upBanner)
    @ApiOperation("上移锦旗信息")
    public Envelop upBanner(@ApiParam(name = "id", value = "id", required = true)
                                @RequestParam(value = "id", required = true)String id){
        try {
            BaseBannerDO bannerDO = baseBannerService.upBanner(id);
            return success("上移成功");
        }catch (Exception e){
            return failedException(e);
        }

    }
    @GetMapping(value= BaseHospitalRequestMapping.BaseBannerssage.downBanner)
    @ApiOperation("下移锦旗信息")
    public Envelop downBanner(@ApiParam(name = "id", value = "id", required = true)
                                 @RequestParam(value = "id", required = true)String id){

        try {
            BaseBannerDO bannerDO = baseBannerService.downBanner(id);
            return success("下移成功");
        }catch (Exception e){
            return failedException(e);
        }
    }
    @GetMapping(value= BaseHospitalRequestMapping.BaseBannerssage.onOff)
    @ApiOperation("锦旗开关")
    public Envelop onOff(@ApiParam(name = "id", value = "id", required = true)
                              @RequestParam(value = "id", required = true)String id,
                         @ApiParam(name = "status", value = "status", required = true)
                         @RequestParam(value = "status", required = true)String status){

       try {
           BaseBannerDO bannerDO = baseBannerService.onOff(id,status);
           if (null!=bannerDO.getId()){
               return success("开关切换成功");
           }else {
               return failed("开关失败");
           }
       }catch (Exception e){
           return failedException(e);
       }

    }



}
