package com.yihu.jw.hospital.endpoint.message;

import com.yihu.jw.contant.CommonContant;
import com.yihu.jw.entity.hospital.message.SystemMessageDO;
import com.yihu.jw.hospital.message.service.SystemMessageService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * @author zmk
 * @vsrsion 1.0
 * Created at 2019/11/18
 */
@RestController
@RequestMapping(value = BaseHospitalRequestMapping.SystemMessage.PREFIX)
@Api(value = "线上就诊消息管理器", description = "线上就诊消息管理器", tags = {"互联网医院 - 线上就诊消息管理器接口"})
public class SystemMessageController extends EnvelopRestEndpoint {

    private static Logger logger = LoggerFactory.getLogger(SystemMessageController.class);

    @Autowired
    private SystemMessageService systemMessageService;


    @PostMapping(value= BaseHospitalRequestMapping.SystemMessage.addMessage)
    @ApiOperation("新增线上就诊消息")
    public Envelop addMessage(
            @ApiParam(name = "jsonData", value = "Json数据", required = true)
            @RequestBody SystemMessageDO systemMessageDO)throws Exception{


        try {
            SystemMessageDO messageDO = systemMessageService.addMessage(systemMessageDO);
            if (null == messageDO){
                return failed("新增失败");
            }
            return success("新增成功");
        }catch (Exception e){
            return failedException(e);
        }
    }



    @PostMapping(value= BaseHospitalRequestMapping.SystemMessage.updateMessage)
    @ApiOperation("更新线上就诊消息")
    public Envelop updateMessage(
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id", required = true)String id,
            @ApiParam(name = "read", value = "read", required = false)
            @RequestParam(value = "read", required = false)String read,
            @ApiParam(name = "del", value = "del", required = false)
            @RequestParam(value = "del", required = false)String del)throws Exception{


        try {
            SystemMessageDO systemMessageDO = systemMessageService.updateMessage(id,read,del);
            if (null == systemMessageDO){
                return failed("更新失败,就诊消息不存在");
            }
            return success("更新成功");
        }catch (Exception e){
            return failedException(e);
        }

    }


    @GetMapping(value = BaseHospitalRequestMapping.SystemMessage.searchMessagePageList)
    @ApiOperation(value = "获取线上就诊消息列表")
    public Envelop searchMessagePageList(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "page", value = "第几页，从1开始", required = true, defaultValue = "1")
            @RequestParam(value = "page", required = true,defaultValue = "1")Integer page,
            @ApiParam(name = "pageSize", value = "每页分页大小", required = true, defaultValue = "10")
            @RequestParam(value = "pageSize", required = true,defaultValue = "10")Integer pageSize)throws Exception{


        try {
            if(StringUtils.isBlank(filters)){
                filters="del=1";
            }else{
                filters=filters+";del=1";
            }
            List<SystemMessageDO> list  = systemMessageService.search(fields,filters,sorts,page,pageSize);
            for (SystemMessageDO systemMessageDO:list){

            }
            int count = (int) systemMessageService.getCount(filters);
            return success(list, count, page, pageSize, SystemMessageDO.class);
        }catch (Exception e){
            return failedException(e);
        }
    }


    @GetMapping(value = BaseHospitalRequestMapping.SystemMessage.queryById)
    @ApiOperation(value = "获取线上就诊消息详情")
    public Envelop queryById(
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id", required = true)String id)throws Exception{


        try {
            SystemMessageDO systemMessageDO = systemMessageService.queryById(id);
            if (null == systemMessageDO){
                return failed("获取失败,线上就诊消息不存在");
            }
            return success(CommonContant.Message.GET_SUCCESS,200,systemMessageDO);
        }catch (Exception e){
            return failedException(e);
        }

    }


}
