package com.yihu.jw.hospital.endpoint.nurse;

import com.yihu.jw.contant.CommonContant;
import com.yihu.jw.entity.base.nurse.BaseNursingCardDO;
import com.yihu.jw.healthUpload.service.BasePatientHealthUploadService;
import com.yihu.jw.nurse.service.BaseNursingCardService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.entity.ServiceException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 陪护证
 * Created by yeshijie on 2022/11/16.
 */
@RestController
@RequestMapping(value = "nursing")
@Api(value = "陪护证", tags = "陪护证")
public class NursingCardEndpoint extends EnvelopRestEndpoint {

    @Resource
    private BaseNursingCardService nursingCardService;
    @Autowired
    private BasePatientHealthUploadService patientHealthUploadService;

    @PostMapping(value = "doctor/check")
    @ApiOperation(value = "审核")
    public Envelop doctorCheck(
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id") String id,
            @ApiParam(name = "doctor", value = "医生id", required = true)
            @RequestParam(value = "doctor") String doctor,
            @ApiParam(name = "checkReason", value = "审核说明/拒绝原因")
            @RequestParam(value = "checkReason", required = false) String checkReason,
            @ApiParam(name = "status", value = "状态 -2失效,-1拒绝 1通过", required = true)
            @RequestParam(value = "status") Integer status) {
        try {
            nursingCardService.doctorCheck(id, status, doctor, checkReason);
            return success("操作成功!");
        }catch (Exception e){
            return failedException(e);
        }
    }

    @GetMapping(value = "ifCanApply")
    @ApiOperation(value = "是否可申请")
    public Envelop ifCanApply(
            @ApiParam(name = "idcard", value = "患者身份证", required = true)
            @RequestParam(value = "idcard") String idcard,
            @ApiParam(name = "name", value = "患者姓名", required = true)
            @RequestParam(value = "name") String name) {
        try {
            //院内存在住院患者时，才可提交页面
            return success(CommonContant.Message.GET_SUCCESS, nursingCardService.ifCanApply(idcard,name));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @GetMapping(value = "nursingDetail")
    @ApiOperation(value = "陪护明细")
    public Envelop nursingDetail(
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id") String id) {
        try {
            BaseNursingCardDO cardDO = nursingCardService.nursingDetail(id);
            return success("申请成功!", cardDO);
        }catch (Exception e){
            return failedException(e);
        }
    }

    @GetMapping(value = "patient/page")
    @ApiOperation(value = "获取分页")
    public PageEnvelop patientPage (
            @ApiParam(name = "patient", required = true, value = "居民id")
            @RequestParam(value = "patient") String patient,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1")
            @RequestParam(value = "page") Integer page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15")
            @RequestParam(value = "size") Integer size) {
        try {
            String filters = "nursePatient="+patient;
            String sorts = "-status,-createTime";
            List<BaseNursingCardDO> list = nursingCardService.search(null, filters, sorts, page, size);
            boolean hasNucleicReport = false;
            if(list.size()>0){
                //判断是否上传健康上报
                hasNucleicReport = patientHealthUploadService.isUploadType3(patient);
            }
            for (BaseNursingCardDO cardDO:list){
                cardDO.setStatusName(BaseNursingCardDO.Status.getStatusName(cardDO.getStatus()));
                cardDO.setHasNucleicReport(hasNucleicReport);
                if(cardDO.getStatus()==1){
                    cardDO.setRemainingTimes(DateUtil.getDifferenceOfMillis(new Date(),cardDO.getEndTime()));
                }
            }
            int count = (int)nursingCardService.getCount(filters);
            return success(list, count, page, size, BaseNursingCardDO.class);
        }catch (Exception e){
            e.printStackTrace();
            return PageEnvelop.getError("查询失败");
        }
    }

    @GetMapping(value = "doctor/page")
    @ApiOperation(value = "获取分页")
    public PageEnvelop page (
            @ApiParam(name = "status", value = "状态 -2已失效 -1未通过 0审核中 1生效中")
            @RequestParam(value = "status", required = false) Integer status,
            @ApiParam(name = "name", value = "姓名或身份证号")
            @RequestParam(value = "name", required = false) String name,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1")
            @RequestParam(value = "page") Integer page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15")
            @RequestParam(value = "size") Integer size) {
        try {
            String filters = "";
            if (status != null){
                filters = "status="+status;
            }
            if(StringUtils.isNotBlank(name)){
                if(StringUtils.isNotBlank(filters)){
                    filters +=";";
                }
                filters+="name?"+name+" g1;idcard?"+name+" g1";
            }
            List<BaseNursingCardDO> list = nursingCardService.search(null, filters, null, page, size);
            Map<String,Integer> areaMaxNum = new HashMap<>();
            for (BaseNursingCardDO cardDO:list){
                cardDO.setStatusName(BaseNursingCardDO.Status.getStatusName(cardDO.getStatus()));
                String inpatientAreaId = cardDO.getInpatientAreaId();
                int remainingNum = 0;
                if(StringUtils.isNotBlank(inpatientAreaId)){
                    if(!areaMaxNum.containsKey(inpatientAreaId)){
                        areaMaxNum.put(inpatientAreaId,nursingCardService.countByInpatientAreaIdAndStatus(inpatientAreaId));
                    }
                    remainingNum = areaMaxNum.get(inpatientAreaId);
                }
                cardDO.setRemainingNum(remainingNum);
                cardDO.setNursingNum(nursingCardService.countByPatientAndStatus(cardDO.getPatient()));
            }
            int count = (int)nursingCardService.getCount(filters);
            return success(list, count, page, size, BaseNursingCardDO.class);
        }catch (Exception e){
            e.printStackTrace();
            return PageEnvelop.getError("查询失败");
        }
    }


    @PostMapping(value = "patient/create")
    @ApiOperation(value = "申请陪护证")
    public Envelop createNursing(
            @ApiParam(name = "jsonData", value = "JSON数据", required = true)
            @RequestParam(value = "jsonData") String jsonData ) {
        try {
            BaseNursingCardDO nursingCardDO = toEntity(jsonData, BaseNursingCardDO.class);

            //是否可申请，及病区字段获取
            nursingCardDO = nursingCardService.ifCanApply(nursingCardDO);
            nursingCardDO.setType(2);
            nursingCardDO.setCreateTime(new Date());
            nursingCardDO.setStatus(0);//审核中
            nursingCardDO = nursingCardService.save(nursingCardDO);
            return success("申请成功!", nursingCardDO);
        }catch (ServiceException se){
            return Envelop.getError(se.getMessage());
        }catch (Exception e){
            return failedException2(e);
        }
    }



}
