package com.yihu.jw.hospital.endpoint.prescription;

import com.yihu.jw.contant.CommonContant;
import com.yihu.jw.hospital.prescription.service.DrugsPlanService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * Created by yeshijie on 2022/11/24.
 */
@RestController
@RequestMapping(value = "patient/drugsPlan")
@Api(value = "用药指导", tags = {"用药指导"})
public class DrugsPlanEndpoint extends EnvelopRestEndpoint {

    @Autowired
    private DrugsPlanService drugsPlanService;

    @PostMapping(value = "createDrugsPlan")
    @ApiOperation(value = "手动生成用药计划")
    public Envelop createDrugsPlan(
            @ApiParam(name = "prescriptionId", value = "处方id")
            @RequestParam(value = "prescriptionId", required = true) String prescriptionId) {
        try {
            drugsPlanService.createDrugsPlan(prescriptionId);
            return success(CommonContant.Message.GET_SUCCESS);
        } catch (Exception e) {
            return failedException2(e);
        }
    }

    @GetMapping(value = "findHaveDrugPlan")
    @ApiOperation(value = "按月查询居民每天的用药计划数")
    public Envelop findHaveDrugPlan(
            @ApiParam(name = "patient", value = "居民id")
            @RequestParam(value = "patient", required = true) String patient,
            @ApiParam(name = "month", value = "日期，yyyy-MM")
            @RequestParam(value = "month", required = true) String month) {
        try {
            return ListEnvelop.getSuccess(CommonContant.Message.GET_SUCCESS,drugsPlanService.findHaveDrugPlan(patient,month));
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "findDrugsPlan")
    @ApiOperation(value = "获取居民某天的服药计划")
    public Envelop findDrugsPlan(
            @ApiParam(name = "patient", value = "居民id")
            @RequestParam(value = "patient", required = true) String patient,
            @ApiParam(name = "date", value = "日期，yyyy-MM-dd")
            @RequestParam(value = "date", required = true) String date) {
        try {
            return ListEnvelop.getSuccess(CommonContant.Message.GET_SUCCESS,drugsPlanService.findDrugsPlan(patient,date));
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "findDrugsPlanMessage")
    @ApiOperation(value = "获取居民的今日用药计划消息")
    public Envelop findDrugsPlanMessage(
            @ApiParam(name = "patient", value = "居民id")
            @RequestParam(value = "patient", required = true) String patient) {
        try {
            return ListEnvelop.getSuccess(CommonContant.Message.GET_SUCCESS,drugsPlanService.findDrugsPlanMessage(patient));
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "prescriptionList")
    @ApiOperation(value = "查询居民处方列表")
    public Envelop prescriptionList(
            @ApiParam(name = "patient", value = "居民id")
            @RequestParam(value = "patient", required = true) String patient,
            @ApiParam(name = "page", value = "第几页")
            @RequestParam(value = "page", required = true) Integer page,
            @ApiParam(name = "pagesize", value = "每页大小")
            @RequestParam(value = "pagesize", required = true) Integer pagesize) {
        try {
            return drugsPlanService.prescriptionList(patient,page,pagesize);
        } catch (Exception e) {
            return failedPageEnvelopException2(e);
        }
    }
    
}
