package com.yihu.jw.hospital.endpoint.prescription;

import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.hospital.message.SystemMessageDO;
import com.yihu.jw.entity.hospital.prescription.BaseAppointmentLogDO;
import com.yihu.jw.entity.hospital.prescription.WlyyOutpatientDO;
import com.yihu.jw.hospital.prescription.service.PrescriptionService;
import com.yihu.jw.hospital.prescription.service.YkyyPrescriptionService;
import com.yihu.jw.hospital.prescription.service.entrance.YkyyEntranceService;
import com.yihu.jw.hospital.service.SystemMessage.HospitalSystemMessageService;
import com.yihu.jw.order.BusinessOrderService;
import com.yihu.jw.restmodel.hospital.prescription.WlyyOutpatientVO;
import com.yihu.jw.restmodel.hospital.prescription.WlyyPrescriptionVO;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * Created by Trick on 2019/5/16
 */
@RestController
@RequestMapping(value = BaseHospitalRequestMapping.YkyyPrescription.PREFIX)
@Api(value = "眼科医院在线复诊", description = "眼科医院在线复诊接口", tags = {"眼科医院在线复诊接口"})
public class YkyyPrescriptionEndpoint extends EnvelopRestEndpoint {

    @Autowired
    private YkyyPrescriptionService prescriptionService;
    @Autowired
    private PrescriptionService sysPrescriptionService;
    @Autowired
    private YkyyEntranceService ykyyEntranceService;
    @Autowired
    private BusinessOrderService businessOrderService;
    @Autowired
    private PrescriptionService commenPrescriptionService;
    @Autowired
    private HospitalSystemMessageService hospitalSystemMessageService;
    @Autowired
    private BaseDoctorDao doctorDao;

    @Value("${demo.flag}")
    private boolean demoFlag;

    @Value("${wechat.id}")
    private String wxId;
    @Value("${pay.flag}")
    private boolean payFlag;

    @GetMapping(value = "vlisReportMasterlist")
    @ApiOperation(value = "检验信息列表")
    public ListEnvelop vlisReportMasterlist(@ApiParam(name = "patient", value = "居民id")
                                          @RequestParam(value = "patient",required = true) String patient,
                                          @ApiParam(name = "page", value = "第几页")
                                          @RequestParam(value = "page",required = false) Integer page,
                                          @ApiParam(name = "size", value = "页面大小")
                                          @RequestParam(value = "size",required = false) Integer size,
                                          @ApiParam(name = "startTime", value = "开始时间")
                                          @RequestParam(value = "startTime",required = false) String startTime,
                                          @ApiParam(name = "endTime", value = "结束时间")
                                          @RequestParam(value = "endTime",required = false) String endTime) throws Exception{

        try {
            if(page==null){
                page = 1;
            }
            if(size==null){
                size = 10;
            }
            return success(ykyyEntranceService.vlisReportMasterlist(patient, page, size,startTime,endTime));
        }catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "vlisReportMasterDetail")
    @ApiOperation(value = "检验信息明细")
    public ObjEnvelop vlisReportMasterDetail(@ApiParam(name = "reportId", value = "报告id")
                                         @RequestParam(value = "reportId",required = true) String reportId)throws Exception{

        try {
            return success(ykyyEntranceService.vlisReportMasterDetail(reportId));
        }catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @GetMapping(value = "vlisReportMasterlistNew")
    @ApiOperation(value = "检验信息列表新接口")
    public ListEnvelop vlisReportMasterlistNew(@ApiParam(name = "patient", value = "居民id")
                                            @RequestParam(value = "patient",required = true) String patient,
                                            @ApiParam(name = "startTime", value = "开始时间")
                                            @RequestParam(value = "startTime",required = false) String startTime,
                                            @ApiParam(name = "endTime", value = "结束时间")
                                            @RequestParam(value = "endTime",required = false) String endTime) throws Exception{

        try {
            return success(ykyyEntranceService.getSampleByYZ(patient,startTime,endTime));
        }catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "vlisReportMasterDetailNew")
    @ApiOperation(value = "检验信息明细新接口")
    public ObjEnvelop vlisReportMasterDetailNew(@ApiParam(name = "reportNo", value = "报告id")
                                             @RequestParam(value = "reportNo",required = true) String reportNo)throws Exception{

        try {
            return success(ykyyEntranceService.getSampleDetailByYZ(reportNo));
        }catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @GetMapping(value = "vpacsReportDetail")
    @ApiOperation(value = "检查信息明细")
    public ObjEnvelop vpacsReportDetail(@ApiParam(name = "reportId", value = "报告id")
                                             @RequestParam(value = "reportId",required = true) String reportId)throws Exception{
        try {
            return success(ykyyEntranceService.vpacsReportDetail(reportId));
        }catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @GetMapping(value = "vpacsReportList")
    @ApiOperation(value = "检查信息列表")
    public ListEnvelop vpacsReportList(@ApiParam(name = "patient", value = "居民id")
                                            @RequestParam(value = "patient",required = true) String patient,
                                            @ApiParam(name = "page", value = "第几页")
                                            @RequestParam(value = "page",required = false) Integer page,
                                            @ApiParam(name = "size", value = "页面大小")
                                            @RequestParam(value = "size",required = false) Integer size) throws Exception{

        try {
            if(page==null){
                page = 1;
            }
            if(size==null){
                size = 10;
            }
            return success(ykyyEntranceService.vpacsReportList(patient, page, size));
        }catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.Prescription.findOutpatientList)
    @ApiOperation(value = " 查询某个时间段的患者门诊就诊记录")
    public ListEnvelop findOutpatientList(@ApiParam(name = "patient", value = "居民id")
                                          @RequestParam(value = "patient",required = false) String patient,
                                          @ApiParam(name = "startTime", value = "开始时间")
                                          @RequestParam(value = "startTime",required = false) String startTime,
                                          @ApiParam(name = "endTime", value = "结束时间")
                                          @RequestParam(value = "endTime",required = false) String endTime,
                                          @ApiParam(name = "ksdm", value = "科室代码")
                                              @RequestParam(value = "ksdm",required = false) String ksdm,
                                                      @ApiParam(name = "cardNo", value = "就诊卡号")
                                                      @RequestParam(value = "cardNo",required = false) String cardNo) throws Exception {

        try {
            List<WlyyOutpatientVO> vos = prescriptionService.findOutpatientList(patient,cardNo, startTime, endTime, demoFlag,ksdm);
            return success(vos);
        }catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.Prescription.synDoctorInfo)
    @ApiOperation(value = " 同步医生")
    public ObjEnvelop synDoctorInfo(@ApiParam(name = "idcard", value = "身份证")
                                         @RequestParam(value = "idcard",required = false) String idcard)throws Exception{

        try {
            ObjEnvelop obj =  new ObjEnvelop();
            prescriptionService.synDoctorInfo(idcard);
            obj.setStatus(200);
            obj.setMessage("同步成功");
            return success(obj);
        }catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }



    @GetMapping(value = BaseHospitalRequestMapping.Prescription.findOutpatientInfo)
    @ApiOperation(value = " 查询单条门诊就诊记录")
    public ObjEnvelop findOutpatientInfo(@ApiParam(name = "patient", value = "居民id")
                                         @RequestParam(value = "patient",required = false) String patient,
                                         @ApiParam(name = "conNo", value = "就诊次数")
                                         @RequestParam(value = "conNo",required = false) String conNo,
                                         @ApiParam(name = "ksdm", value = "科室编号")
                                             @RequestParam(value = "ksdm",required = false) String ksdm)throws Exception{

        try {
            WlyyOutpatientVO obj = prescriptionService.findOutpatientInfo(patient,conNo,demoFlag,ksdm);
            return success(obj);
        }catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.Prescription.findOriginPrescriptionList)
    @ApiOperation(value = "门诊下所有处方信息")
    public ListEnvelop findOriginPrescriptionList(
            @ApiParam(name = "registerSn", value = "流水号")
            @RequestParam(value = "registerSn",required = false) String registerSn,
            @ApiParam(name = "patNo", value = "病人id")
            @RequestParam(value = "patNo",required = false) String patNo,
            @ApiParam(name = "admNo", value = "住院唯一号")
            @RequestParam(value = "admNo",required = false) String admNo,
            @ApiParam(name = "realOrder", value = "处方号")
            @RequestParam(value = "realOrder",required = false) String realOrder) throws Exception {

        try {
            List<WlyyPrescriptionVO> obj = prescriptionService.findOriginPrescriptionList(registerSn, patNo, realOrder, admNo,demoFlag);
            return success(obj);
        }catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }


    @PostMapping(value = BaseHospitalRequestMapping.YkyyPrescription.appointmentRevisit)
    @ApiOperation(value = "居民发起复诊", notes = "居民发起复诊")
    public ObjEnvelop appointmentRevisit(@ApiParam(name = "outpatientJson", value = "居民门诊json")
                                         @RequestParam(value = "outpatientJson", required = false)String outpatientJson,
                                         @ApiParam(name = "expressageJson", value = "居民物流json")
                                         @RequestParam(value = "expressageJson", required = false)String expressageJson,
                                         @ApiParam(name = "registerJson", value = "预约时间")
                                         @RequestParam(value = "registerJson", required = false)String registerJson,
                                         @ApiParam(name = "chargeType", value = "预约实体json")
                                         @RequestParam(value = "chargeType", required = false)String chargeType)throws Exception {


        try {
            WlyyOutpatientDO wlyyOutpatientDO = prescriptionService.appointmentRevisit(outpatientJson,expressageJson,registerJson,chargeType);
            if (wlyyOutpatientDO!=null&&null==wlyyOutpatientDO.getId()&&!wlyyOutpatientDO.getOutpatientType().equalsIgnoreCase("3")){
                ObjEnvelop objEnvelop =new ObjEnvelop();
                String msg = "";
                if (1==wlyyOutpatientDO.getRemindCount()){
                    msg = "自取";
                }else {
                    msg = "快递配送";
                }
                objEnvelop.setMessage("您已经选择过"+msg+"方式，请勿修改");
                objEnvelop.setStatus(300);
                return objEnvelop;
            }

            if (wlyyOutpatientDO.getPayStatus() != null && wlyyOutpatientDO.getPayStatus() == 1) {
                //发送系统消息
                SystemMessageDO systemMessageDO = sysPrescriptionService.sendOutPatientMes(wlyyOutpatientDO, payFlag);
                //发送IM消息
                hospitalSystemMessageService.sendImMessage(systemMessageDO);
            }


            //发送医生抢单消息
            if (StringUtils.isBlank(wlyyOutpatientDO.getDoctor())) {
                hospitalSystemMessageService.sendImPichCheckMessage(wlyyOutpatientDO);
            }

          /*SystemMessageDO systemMessageDO = commenPrescriptionService.sendOutPatientMes(wlyyOutpatientDO,payFlag);*/
       /* //发送系统消息
        SystemMessageDO systemMessageDO = prescriptionService.sendOutPatientMes(wlyyOutpatientDO);
        //发送IM消息
        hospitalSystemMessageService.sendImMessage(systemMessageDO);

        //发送医生抢单消息
        if(StringUtils.isBlank(wlyyOutpatientDO.getDoctor())){
            hospitalSystemMessageService.sendImPichCheckMessage(wlyyOutpatientDO);
        }*/

            return success(BaseHospitalRequestMapping.Prescription.api_success,wlyyOutpatientDO);
        }catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }


    @GetMapping(value = BaseHospitalRequestMapping.Prescription.getDrugDictionary)
    @ApiOperation(value = "获取药品字典", notes = "获取药品字典")
    public ListEnvelop getDrugDictionary(@ApiParam(name = "drugNo", value = "药品编码")
                                         @RequestParam(value = "drugNo", required = false)String drugNo,
                                         @ApiParam(name = "pyKey", value = "拼音关键字")
                                         @RequestParam(value = "pyKey", required = false)String pyKey,
                                         @ApiParam(name = "winNo", value = "分部编码")
                                         @RequestParam(value = "winNo", required = false)String winNo,
                                         @ApiParam(name = "groupNo", value = "分组")
                                         @RequestParam(value = "groupNo", required = false)String groupNo,
                                         @ApiParam(name = "oupatienId", value = "门诊Id")
                                             @RequestParam(value = "oupatienId", required = false)String oupatienId)throws Exception {

        try {
            return success(prescriptionService.getDrugDictionary(drugNo,pyKey,winNo,groupNo,oupatienId));
        }catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }


    @GetMapping(value = BaseHospitalRequestMapping.Prescription.getDrugUse)
    @ApiOperation(value = "获取用法", notes = "获取用法")
    public ListEnvelop getDrugUse(@ApiParam(name = "pyKey", value = "拼音关键字")
                                  @RequestParam(value = "pyKey", required = false)String pyKey,
                                  @ApiParam(name = "drugNo", value = "药品编码")
                                  @RequestParam(value = "drugNo", required = false)String drugNo)throws Exception {

        try {
            return success(prescriptionService.getDrugUse(drugNo,pyKey,demoFlag));
        }catch (Exception e) {
            return failedListEnvelopException(e);
        }

    }

    @GetMapping(value = BaseHospitalRequestMapping.Prescription.getDrugFrequency)
    @ApiOperation(value = "获取用药频次", notes = "获取用药频次")
    public ListEnvelop getDrugFrequency()throws Exception {

        try {
            return success(prescriptionService.getDrugFrequency(demoFlag));
        }catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.Prescription.getInspectionDictionary)
    @ApiOperation(value = "获取检查检验", notes = "获取检查检验")
    public ListEnvelop getInspectionDictionary(@ApiParam(name = "pyKey", value = "拼音码")
                                               @RequestParam(value = "pyKey", required = false)String pyKey,
                                               @ApiParam(name = "winNo", value = "6总部7金榜8夏禾")
                                               @RequestParam(value = "winNo", required = false)String winNo,
                                               @ApiParam(name = "codes", value = "检查检验收费码codes,用逗号分割")
                                               @RequestParam(value = "codes", required = false)String codes)throws Exception{
        try {
            return success(prescriptionService.getInspectionDictionary(pyKey,codes,winNo));
        }catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.Prescription.findPatientCard)
    @ApiOperation(value = "查询患者就诊卡", notes = "查询患者就诊卡")
    public ListEnvelop findPatientCard(@ApiParam(name = "patient", value = "居民Code")
                                       @RequestParam(value = "patient", required = true)String patient)throws Exception {

        try {
            return success(prescriptionService.findPatientCard(patient));
        }catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.Prescription.synYkIcd10)
    @ApiOperation(value = "同步眼科icd10", notes = "同步眼科icd10")
    public ListEnvelop findPatientCard()throws Exception {
        try {
            return success(prescriptionService.synYkICD10());
        }catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }


    @GetMapping(value = BaseHospitalRequestMapping.Prescription.prescriptionPay)
    @ApiOperation(value = "处方支付", notes = "处方支付")
    public ObjEnvelop findPatientCard(@ApiParam(name = "code", value = "处方code")
                                       @RequestParam(value = "code", required = true)String code,
                                       @ApiParam(name = "price", value = "支付费用")
                                       @RequestParam(value = "price", required = false)Double price,
                                      @ApiParam(name = "tradeType", value = "支付方式")
                                          @RequestParam(value = "tradeType", required = false)String tradeType,
                                      @ApiParam(name = "appletCode", value = "appletCode", required = false)
                                          @RequestParam(value = "appletCode",required = false)String appletCode)throws Exception {
        try {
            return success("ok",prescriptionService.savePrescriptionOrder(code,price,tradeType,wxId,appletCode));
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.Prescription.getICD10)
    @ApiOperation(value = "获取ICD10诊断编码", notes = "获取ICD10诊断编码")
    public ListEnvelop getICD10(@ApiParam(name = "pyKey", value = "拼音关键字")
                                @RequestParam(value = "pyKey", required = false)String pyKey)throws Exception {

        try {
            return success(prescriptionService.getICD10(pyKey,demoFlag));
        } catch (Exception e) {
            return failedListEnvelopException(e);
        }

    }


    @GetMapping(value = BaseHospitalRequestMapping.YkyyPrescription.yktPay)
    @ApiOperation(value = "眼科通app支付", notes = "眼科通app支付")
    public ObjEnvelop yktAppPayJson(@ApiParam(name = "relationCode", value = "关联code")
                                      @RequestParam(value = "relationCode", required = true)String relationCode)throws Exception {
        try {
            return success("ok",businessOrderService.yktAppPayJson(relationCode));
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }


    @GetMapping(value = BaseHospitalRequestMapping.YkyyPrescription.getSql)
    @ApiOperation(value = "通用sql", notes = "通用sql")
    public ObjEnvelop findInfoBysql(@ApiParam(name = "sql", value = "sql")
                                    @RequestParam(value = "sql", required = true)String sql)throws Exception {
        try {
            return success("ok",ykyyEntranceService.findInfoBysql(sql));
        } catch (Exception e) {
            return  failedObjEnvelopException(e);
        }
    }


    @GetMapping(value = BaseHospitalRequestMapping.YkyyPrescription.findDeptByOrgCode)
    @ApiOperation(value = "查询眼科科室信息", notes = "查询眼科科室信息")
    public ListEnvelop findDeptByOrgCode(@ApiParam(name = "orgCode", value = "orgCode")
                                    @RequestParam(value = "orgCode", required = false)String orgCode,
                                        @ApiParam(name = "deptCode", value = "deptCode")
                                        @RequestParam(value = "deptCode", required = false)String deptCode,
                                        @ApiParam(name = "deptName", value = "deptName")
                                            @RequestParam(value = "deptName", required = false)String deptName)throws Exception {


        try {
            return success("ok",ykyyEntranceService.findDeptByYq(orgCode,deptCode,deptName));
        }catch (Exception e) {
            return  failedListEnvelopException(e);
        }

    }

    @GetMapping(value = "GetFYQD")
    @ApiOperation(value = "门诊费用查询", notes = "门诊费用查询")
    public Envelop GetFYQD(@ApiParam(name = "familyId", value = "家庭关系id")
                                         @RequestParam(value = "familyId", required = false)String familyId,
                           @ApiParam(name = "loginId", value = "登录用户")
                           @RequestParam(value = "loginId", required = false)String loginId,
                           @ApiParam(name = "startDate", value = "开始时间")
                                         @RequestParam(value = "startDate", required = false)String startDate,
                           @ApiParam(name = "endDate", value = "结束时间")
                                         @RequestParam(value = "endDate", required = false)String endDate)throws Exception {


        try {
            return success("ok",prescriptionService.GetFYQD(familyId,startDate,endDate,loginId));
        }catch (Exception e) {
            return  failedListEnvelopException(e);
        }

    }

    @GetMapping(value = "saveYkAppointment")
    @ApiOperation(value = "保存预约记录", notes = "保存预约记录")
    public Envelop GetFYQD(@ApiParam(name = "jsonData", value = "json格式")
                           @RequestParam(value = "jsonData", required = false)String jsonData)throws Exception {
        try {
            BaseAppointmentLogDO baseAppointmentLogDO = toEntity(jsonData,BaseAppointmentLogDO.class);
            return success("ok",prescriptionService.saveYkAppointment(baseAppointmentLogDO));
        }catch (Exception e) {
            return  failedListEnvelopException(e);
        }
    }

    @GetMapping(value = "selectList")
    @ApiOperation(value = "查询预约记录", notes = "查询预约记录")
    public ListEnvelop selectList(@ApiParam(name = "condition", value = "姓名、科室、医生")
                           @RequestParam(value = "condition", required = false)String condition,
                              @ApiParam(name = "startTime", value = "开始时间")
                              @RequestParam(value = "startTime", required = false)String startTime,
                              @ApiParam(name = "endTime", value = "结束时间")
                                  @RequestParam(value = "endTime", required = false)String endTime,
                              @ApiParam(name = "page", value = "页码")
                                  @RequestParam(value = "page", required = false)Integer page,
                              @ApiParam(name = "pageSize", value = "页大小")
                                  @RequestParam(value = "pageSize", required = false)Integer pageSize)throws Exception {
        try {
            return success("ok",prescriptionService.selectList(condition,startTime,endTime,page,pageSize));
        }catch (Exception e) {
            return  failedListEnvelopException(e);
        }
    }
}
