package com.yihu.jw.hospital.endpoint.role;

import com.yihu.jw.area.dao.BaseInpatientAreaDao;
import com.yihu.jw.area.service.BaseInpatientAreaService;
import com.yihu.jw.contant.CommonContant;
import com.yihu.jw.entity.base.area.BaseInpatientAreaDO;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.PageEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
import java.util.Optional;

/**
 * Created by yeshijie on 2022/12/2.
 */
@RestController
@RequestMapping(value = "inpatientArea")
@Api(value = "住院病区管理",tags = {"住院病区管理"})
public class BaseInpatientAreaEndpoint extends EnvelopRestEndpoint {

    @Autowired
    private BaseInpatientAreaService inpatientAreaService;
    @Autowired
    private BaseInpatientAreaDao inpatientAreaDao;

    @PostMapping(value = "create")
    @ApiOperation(value = "新增或修改")
    public Envelop create (
            @ApiParam(name = "jsonData", value = "Json数据", required = true)
            @RequestParam String jsonData) {
        try {
            BaseInpatientAreaDO areaDO = toEntity(jsonData, BaseInpatientAreaDO.class);
            if(StringUtils.isNotBlank(areaDO.getId())){
                //修改
                Optional<BaseInpatientAreaDO>  optional = inpatientAreaDao.findById(areaDO.getId());
                if(!optional.isPresent()){
                    return Envelop.getError("id不存在");
                }
                BaseInpatientAreaDO old = optional.get();
                old.setDel(areaDO.getDel());
                old.setName(areaDO.getName());
                inpatientAreaDao.save(old);
            }else {
                //新增
                areaDO.setCreateTime(new Date());
                inpatientAreaDao.save(areaDO);
            }
            return ObjEnvelop.getSuccess(CommonContant.Message.SAVE_ERROR, areaDO);
        }catch (Exception e){
            e.printStackTrace();
        }
        return Envelop.getError(CommonContant.Message.SAVE_ERROR);
    }

    @PostMapping(value = "saveAreaRule")
    @ApiOperation(value = "保存陪护规则")
    public Envelop saveAreaRule (
            @ApiParam(name = "jsonData", value = "Json数据", required = true)
            @RequestParam String jsonData) {
        try {
            BaseInpatientAreaDO areaDO = toEntity(jsonData, BaseInpatientAreaDO.class);
            Optional<BaseInpatientAreaDO>  optional = inpatientAreaDao.findById(areaDO.getId());
            if(!optional.isPresent()){
                return Envelop.getError("id不存在");
            }
            BaseInpatientAreaDO old = optional.get();
            areaDO.setName(old.getName());
            areaDO.setDel(old.getDel());
            areaDO.setCreateTime(old.getCreateTime());
            inpatientAreaDao.save(areaDO);
            return ObjEnvelop.getSuccess(CommonContant.Message.SAVE_ERROR, areaDO);
        }catch (Exception e){
            e.printStackTrace();
        }
        return Envelop.getError(CommonContant.Message.SAVE_ERROR);
    }

    @PostMapping(value = "updDel")
    @ApiOperation(value = "生效/失效")
    public Envelop updStatus(
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id") String id,
            @ApiParam(name = "del", value = "1生效，0失效", required = true)
            @RequestParam(value = "del") Integer del) {
        try {
            Optional<BaseInpatientAreaDO>  optional = inpatientAreaDao.findById(id);
            if(!optional.isPresent()){
                return Envelop.getError("id不存在");
            }
            BaseInpatientAreaDO old = optional.get();
            old.setDel(del);
            inpatientAreaDao.save(old);
            return ObjEnvelop.getSuccess("修改成功",old);
        }catch (Exception e){
            e.printStackTrace();
        }
        return Envelop.getError("修改失败");
    }

    @GetMapping(value = "findById")
    @ApiOperation(value = "获取详情")
    public Envelop findById(
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id") String id) {
        try {
            Optional<BaseInpatientAreaDO>  optional = inpatientAreaDao.findById(id);
            if(!optional.isPresent()){
                return Envelop.getError("id不存在");
            }
            return ObjEnvelop.getSuccess(CommonContant.Message.GET_SUCCESS,optional.get());
        }catch (Exception e){
            e.printStackTrace();
        }
        return Envelop.getError(CommonContant.Message.GET_ERROR);
    }

    @PostMapping(value = "delete")
    @ApiOperation(value = "删除")
    public Envelop delete(
            @ApiParam(name = "id", value = "id", required = true)
            @RequestParam(value = "id") String id) {
        try {
            Optional<BaseInpatientAreaDO>  optional = inpatientAreaDao.findById(id);
            if(!optional.isPresent()){
                return Envelop.getError("id不存在");
            }
            inpatientAreaDao.deleteById(id);
            return Envelop.getSuccess("删除成功");
        }catch (Exception e){
            e.printStackTrace();
        }
        return Envelop.getError("删除失败");
    }

    @GetMapping(value = "page")
    @ApiOperation(value = "获取分页")
    public Envelop page (
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1")
            @RequestParam(value = "page") int page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15")
            @RequestParam(value = "size") int size){
        try {
            List<BaseInpatientAreaDO> list = inpatientAreaService.search(fields, filters, sorts, page, size);
            long count = inpatientAreaService.getCount(filters);
            return PageEnvelop.getSuccessListWithPage(CommonContant.Message.GET_SUCCESS,list,page,size,count);
        }catch (Exception e){
            e.printStackTrace();
        }
        return Envelop.getError(CommonContant.Message.GET_ERROR);
    }

    @GetMapping(value = "list")
    @ApiOperation(value = "获取列表")
    public Envelop list (
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档")
            @RequestParam(value = "sorts", required = false) String sorts) throws Exception {
        try {
            List<BaseInpatientAreaDO> list = inpatientAreaService.search(fields, filters, sorts);
            return ListEnvelop.getSuccess(CommonContant.Message.GET_SUCCESS,list);
        }catch (Exception e){
            e.printStackTrace();
        }
        return Envelop.getError(CommonContant.Message.GET_ERROR);
    }
}
