package com.yihu.jw.hospital.endpoint.specialist;



import com.yihu.jw.entity.specialist.BaseDiseaseHospitalDO;
import com.yihu.jw.hospital.disease.service.BaseDiseaseHospitalService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;



@RestController
@RequestMapping(value = BaseHospitalRequestMapping.BaseDiseaseHospital.PREFIX)
@Api(value = "专病中心-专病配置", description = "专病中心-专病配置", tags = {"专病中心-专病配置"})
public class BaseDiseaseHospitalController extends EnvelopRestEndpoint {

    @Autowired
    private BaseDiseaseHospitalService baseDiseaseHospitalService;

    @ApiOperation("新增/修改专病配置")
    @PostMapping(value= BaseHospitalRequestMapping.BaseDiseaseHospital.insertAndUpdate)
    public Envelop insertAndUpdateQuestion(@ApiParam(name = "json", value = "专病配置实体类json", required = true)
                                       @RequestParam(value = "json", required = true)String json){

        try {
            BaseDiseaseHospitalDO diseaseHospitalDO = toEntity(json,BaseDiseaseHospitalDO.class);
            return success(baseDiseaseHospitalService.insertAndUpdate(diseaseHospitalDO));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @ApiOperation("根据id获取专病配置详情")
    @GetMapping(value= BaseHospitalRequestMapping.BaseDiseaseHospital.findById)
    public Envelop findById(@ApiParam(name = "id", value = "专病配置id", required = true)
                              @RequestParam(value = "id", required = true)String id){

        try {
            return success(baseDiseaseHospitalService.findById(id));
        }catch (Exception e){
            return failedException(e);
        }
    }
    @ApiOperation("查询专病中心列表")
    @GetMapping(value= BaseHospitalRequestMapping.BaseDiseaseHospital.selectByCondition)
    public Envelop selectQuestionList(@ApiParam(name = "name", value = "专病名称", required = false)
                                  @RequestParam(value = "name", required = false)String name,
                                  @ApiParam(name = "status", value = "1开启0关闭", required = false)
                                  @RequestParam(value = "status", required = false)String status,
                                  @ApiParam(name = "page", value = "页码", required = false)
                                  @RequestParam(value = "page", required = false)Integer page,
                                  @ApiParam(name = "size", value = "大小", required = false)
                                  @RequestParam(value = "size", required = false)Integer size){

        try {
            return success(baseDiseaseHospitalService.selectByCondition(name,status,page,size));
        }catch (Exception e){
            return failedException(e);
        }
    }


    @ApiOperation("根据id删除专病详情")
    @PostMapping(value= BaseHospitalRequestMapping.BaseDiseaseHospital.deleteById)
    public Envelop deleteById(@ApiParam(name = "id", value = "专病配置id", required = true)
                                           @RequestParam(value = "id", required = true)String id){

        try {
            baseDiseaseHospitalService.deleteById(id);
            return success();
        }catch (Exception e){
            return failedException(e);
        }
    }

    @ApiOperation("更新启动状态或者上移下移置顶")
    @PostMapping(value= BaseHospitalRequestMapping.BaseDiseaseHospital.updateStatusOrSort)
    public Envelop deleteById(@ApiParam(name = "id", value = "专病配置id", required = true)
                              @RequestParam(value = "id", required = true)String id,
                              @ApiParam(name = "status", value = "1启动0禁止", required = false)
                              @RequestParam(value = "status", required = false)Integer status,
                              @ApiParam(name = "flag", value = "1上移2下移3置顶", required = false)
                                  @RequestParam(value = "flag", required = false)Integer flag){

        try {
            return success(baseDiseaseHospitalService.updateStatusOrSort(id,status,flag));
        }catch (Exception e){
            return failedException(e);
        }
    }


    @ApiOperation("获取诊断")
    @GetMapping(value= BaseHospitalRequestMapping.BaseDiseaseHospital.getICD10)
    public Envelop getICD10(@ApiParam(name = "pyKey", value = "拼音码或者诊断名称", required = false)
                                      @RequestParam(value = "pyKey", required = false)String pyKey,
                                      @ApiParam(name = "page", value = "页码", required = false)
                                      @RequestParam(value = "page", required = false)Integer page,
                                      @ApiParam(name = "size", value = "大小", required = false)
                                      @RequestParam(value = "size", required = false)Integer size){

        try {
            return success(baseDiseaseHospitalService.getICD10(pyKey,page,size));
        }catch (Exception e){
            return failedException(e);
        }
    }

    @ApiOperation("i健康授权参数加密")
    @PostMapping(value= BaseHospitalRequestMapping.BaseDiseaseHospital.ascRevertData)
    public Envelop ascRevertData(@ApiParam(name = "json", value = "加密的json值", required = false)
                            @RequestParam(value = "json", required = false)String json){

        try {
            return success(baseDiseaseHospitalService.ascRevertData(json));
        }catch (Exception e){
            return failedException(e);
        }
    }


    @ApiOperation("获取十大专病")
    @GetMapping(value= BaseHospitalRequestMapping.BaseDiseaseHospital.getSpecialList)
    public Envelop getSpecialList(){

        try {
            return success(baseDiseaseHospitalService.getSpecialList());
        }catch (Exception e){
            return failedException(e);
        }
    }

}