package com.yihu.jw.hospital.endpoint.statistics;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.es.service.StatisticsEsService;
import com.yihu.jw.restmodel.hospital.statistics.ScreenQvo;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.util.List;
import java.util.Map;


/**
 * Created by wangzhinan on 2019/12/3.
 */
@RestController
@RequestMapping(value = BaseHospitalRequestMapping.Statistics.PREFIX)
@Api(value = "统计分析", description = "统计分析", tags = {"统计分析"})
public class EsStatisticsEndpoint extends EnvelopRestEndpoint {

    Logger logger = LoggerFactory.getLogger(EsStatisticsEndpoint.class);


    @Autowired
    private StatisticsEsService statisticsEsService;


    /**
     * @param startDate
     * @param endDate
     * @param area
     * @param level       2 市  3区  4医院 5、科室 6医生
     * @param index
     * @param level2_type 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.outPatientLine)
    @ApiOperation(value = "门诊类别统计-曲线")
    public ObjEnvelop getOutPatientLine(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) int interval,
            @RequestParam(required = false) String level2_type) {
        //新版与旧版统计适配
        JSONObject result = new JSONObject();
        logger.info("index:" + index + " and level2_type:" + level2_type + " and level:" + level);
        try {
            if (index.equalsIgnoreCase("3,21") || index.equalsIgnoreCase("3") || index.equalsIgnoreCase("21")) {
                result = statisticsEsService.getOutPatientPriceLine(startDate, endDate, area, level, index, interval, level2_type);
            } else {
                result = statisticsEsService.getOutPatientLine(startDate, endDate, area, level, index, interval, level2_type);
            }
            return success(result);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }


    }


    /**
     * @param startDate
     * @param endDate
     * @param area
     * @param level       2 市  3区  4医院 5、科室 6医生
     * @param index
     * @param level2_type 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.prescriptionLine)
    @ApiOperation(value = "开方量周统计-曲线")
    public ObjEnvelop getPrescriptionLine(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) int interval,
            @RequestParam(required = false) String level2_type) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getPrescriptionLine(startDate, endDate, area, level, index, interval, level2_type);
            return success(result);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }

    }

    /**
     * @param startDate
     * @param endDate
     * @param area
     * @param level       2 市  3区  4医院 5、科室 6医生
     * @param index
     * @param level2_type 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.prescriptionTj)
    @ApiOperation(value = "开方量统计表格")
    public ObjEnvelop prescriptionTj(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) int interval,
            @RequestParam(required = false) String level2_type) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getPrescriptionLineBy(startDate, endDate, area, level, index, interval, level2_type);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }

    /**
     * @param startDate
     * @param endDate
     * @param area
     * @param level       2 市  3区  4医院 5、科室 6医生
     * @param index
     * @param level2_type 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.prescriptionByKd)
    @ApiOperation(value = "开方快递量统计表格")
    public ObjEnvelop prescriptionByKd(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) int interval,
            @RequestParam(required = false) String level2_type) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getPrescriptionLineByType(startDate, endDate, area, level, index, interval, level2_type);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }

    @GetMapping(value = BaseHospitalRequestMapping.Statistics.total)
    @ApiOperation(value = " 查询总数")
    public ObjEnvelop getIndexTotal(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) String level2_type) {
        String[] indexes = index.split(",");
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getTotalAmount(startDate, endDate, area, level, index, level2_type);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    @GetMapping(value = BaseHospitalRequestMapping.Statistics.outPatientCircular)
    @ApiOperation(value = "门诊类别统计-圆形")
    public ObjEnvelop getOutPatientCircular(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) String level2_type) {
        String[] indexes = index.split(",");
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getOutPatientCircular(startDate, endDate, area, level, index, level2_type);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }

    /**
     * 门诊统计-分级
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level
     * @param index
     * @param level2_type 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.outPatientGrade)
    @ApiOperation(value = "门诊统计-分级")
    public ObjEnvelop getOutPatient(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) String level2_type) {
        String[] indexes = index.split(",");
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getOutPatientGrade(startDate, endDate, area, level, index, level2_type);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    /**
     * 问诊量统计-类别曲线
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level       2 市  3区  4医院 5、科室 6医生
     * @param index
     * @param level2_type 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.outPatientTypeLine)
    @ApiOperation(value = "问诊量统计-类别曲线")
    public ObjEnvelop getOutPatientTypeline(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) String level2_type) {
        String[] indexes = index.split(",");
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getOutPatientTypeline(startDate, endDate, area, level, index, level2_type);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }

    @GetMapping(value = BaseHospitalRequestMapping.Statistics.doctorStatictis)
    @ApiOperation(value = "人员统计")
    public ObjEnvelop getDoctorStatistics(@RequestParam(required = true) String startDate,
                                          @RequestParam(required = true) String endDate,
                                          @RequestParam(required = true) String area,
                                          @RequestParam(required = true) int level,
                                          @RequestParam(required = true) String index,
                                          @RequestParam(required = false) String level2_type,
                                          @RequestParam(required = true) Integer flag) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getDoctorStatistics(startDate, endDate, area, level, index, level2_type, flag);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }

    @GetMapping(value = BaseHospitalRequestMapping.Statistics.doctorTotal)
    @ApiOperation(value = "各职称人数统计")
    public ObjEnvelop getDoctorTotal(@RequestParam(required = true) String startDate,
                                     @RequestParam(required = true) String endDate,
                                     @RequestParam(required = true) String area,
                                     @RequestParam(required = true) int level,
                                     @RequestParam(required = true) String index,
                                     @RequestParam(required = false) String level2_type,
                                     @RequestParam(required = true) Integer flag) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getDoctorTotal(startDate, endDate, area, level, index, level2_type, flag);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }

    @GetMapping(value = BaseHospitalRequestMapping.Statistics.doctorStatictisExcel)
    @ApiOperation(value = "人数统计-excel表")
    public ObjEnvelop getDoctorStatisticsExcel(@RequestParam(required = true) String startDate,
                                               @RequestParam(required = true) String endDate,
                                               @RequestParam(required = true) String area,
                                               @RequestParam(required = true) int level,
                                               @RequestParam(required = true) String index,
                                               @RequestParam(required = false) String level2_type,
                                               @RequestParam(required = true) Integer flag) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getDoctorStatisticsExcel(startDate, endDate, area, level, index, level2_type, flag);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    /**
     * 专科咨询列表
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level     2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.specialConsultHead)
    @ApiOperation(value = " 专科咨询分析-头部")
    public ObjEnvelop specialConsultHead(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getSpecialConsultHead(startDate, endDate, area, level, index);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    /**
     * 专科咨询列表
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level     2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.specialConsultLine)
    @ApiOperation(value = "专科咨询分析曲线")
    public ObjEnvelop specialConsultLine(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) int interval,
            @RequestParam(required = false) String level2_type) {

        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getSpecialConsultLine(startDate, endDate, area, level, index, interval, level2_type);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }


    /**
     * 专科咨询列表
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level     2 市  3区  4医院 5、科室 6医生
     * @param lowLevel  2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.specialConsultExcel)
    @ApiOperation(value = "专科咨询列表")
    public ObjEnvelop specialConsultExcel(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String lowLevel) {

        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getConsultingStatisticsList(startDate, endDate, level, area, lowLevel);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }


    /**
     * 服务评价总平均分
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level     2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.scoreHead)
    @ApiOperation(value = " 服务评价总平均分-头部")
    public ObjEnvelop getScoreHead(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index) {
        //新版与旧版统计适配
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getScoreHead(startDate, endDate, area, level, index);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    /**
     * 服务评价分析曲线
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level     2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.scoreLine)
    @ApiOperation(value = "服务评价分析曲线")
    public ObjEnvelop getScoreLine(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) int interval,
            @RequestParam(required = false) String level2_type) {

        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getScoreLine(startDate, endDate, area, level, index, interval, level2_type);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }


    /**
     * 服务评价分析列表
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level     2 市  3区  4医院 5、科室 6医生
     * @param lowLevel  2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.scoreList)
    @ApiOperation(value = "服务评价分析列表")
    public ObjEnvelop getScoreList(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String lowLevel) {

        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getScoreList(startDate, endDate, level, area, lowLevel);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }


    @GetMapping(value = BaseHospitalRequestMapping.Statistics.scheduleHead)
    @ApiOperation(value = "排班头部")
    public ObjEnvelop getScheduleTotal(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index) {

        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getScheduleTotal(startDate, endDate, index, level, area);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }


    @GetMapping(value = BaseHospitalRequestMapping.Statistics.scheduleLine)
    @ApiOperation(value = "排班曲线")
    public ObjEnvelop getScheduleLine(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = true) Integer interval) {

        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getScheduleLine(startDate, endDate, area, level, index, interval);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }

    @GetMapping(value = BaseHospitalRequestMapping.Statistics.scheduleList)
    @ApiOperation(value = "排班表格")
    public ObjEnvelop getScheduleList(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = true) String levelCode) {

        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getScheduleList(startDate, endDate, index, level, area, levelCode);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }


    /**
     * 咨询列表
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level     2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.consultHead)
    @ApiOperation(value = " 咨询分析-头部")
    public ObjEnvelop getConsultHead(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) String lowCode) {
        //新版与旧版统计适配
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getConsultHead(startDate, endDate, area, level, index, lowCode);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    /**
     * 咨询列表
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level     2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.consultLine)
    @ApiOperation(value = "咨询分析曲线")
    public ObjEnvelop getConsultLine(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) int interval,
            @RequestParam(required = false) String level2_type) {

        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getConsultLine(startDate, endDate, area, level, index, interval, level2_type);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }


    /**
     * 咨询列表
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level     2 市  3区  4医院 5、科室 6医生
     * @param lowLevel  2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.consultList)
    @ApiOperation(value = "咨询列表")
    public ObjEnvelop getConsultingList(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = true) String lowLevel) {

        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getConsultingList(startDate, endDate, level, area, lowLevel, index);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }


    /**
     * app端首页门诊量统计
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level
     * @param index     5咨询量，6专家咨询,11图文复诊，13专科协同，15视频复诊
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.consultTotalApp)
    @ApiOperation(value = "app端首页门诊量统计")
    public ObjEnvelop getConsultTotalApp(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getConsultTotalApp(startDate, endDate, area, level, index);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    /**
     * app端首页门诊量-圆形图
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.consultCircularApp)
    @ApiOperation(value = "app端首页门诊量-圆形图")
    public ObjEnvelop getConsultCircularApp(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getConsultCircularApp(startDate, endDate, area, level);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    /**
     * app端首页门诊量统计-曲线图
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level
     * @param index       5
     * @param interval
     * @param level2_type
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.consultLineApp)
    @ApiOperation(value = "app端首页门诊量统计-曲线图")
    public ObjEnvelop getConsultLineApp(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = true) Integer interval,
            @RequestParam(required = false) String level2_type) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getConsultLineApp(startDate, endDate, area, level, index, interval, level2_type);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }

    @GetMapping(value = BaseHospitalRequestMapping.Statistics.consultListApp)
    @ApiOperation(value = "app端首页门诊量统计-表格")
    public ObjEnvelop getConsultListApp(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = true) String lowLevel) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getConsultListApp(startDate, endDate, level, area, lowLevel, index);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    /**
     * @param startDate
     * @param endDate
     * @param area
     * @param level
     * @param index       专家咨询6，图文复诊11，专家协同13，视频复诊15
     * @param interval
     * @param level2_type
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.consultLinePC)
    @ApiOperation(value = "pc端首页门诊量统计-曲线图")
    public ObjEnvelop getConsultLinePC(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = true) Integer interval,
            @RequestParam(required = false) String lowCode,
            @RequestParam(required = false) String level2_type) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getConsultLinePC(startDate, endDate, area, level, index, interval, level2_type, lowCode);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }

    /**
     * pc端首页门诊量统计-表格
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level
     * @param index     专家咨询6，图文复诊11，专家协同13，视频复诊15
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.consultListPC)
    @ApiOperation(value = "pc端首页门诊量统计-表格")
    public ObjEnvelop getConsultLinePC(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) String lowCode,
            @RequestParam(required = true) String lowLevel) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getConsultingListPC(startDate, endDate, level, area, lowLevel, index, lowCode);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }

    /**
     * app端首页数据统计
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level
     * @param index     6专家，13协同，11、在线复诊
     * @param timeLevel
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.indexTotalApp)
    @ApiOperation(value = "app端首页数据统计")
    public ObjEnvelop getIndexTotalApp(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = true) String timeLevel) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getIndexTotalApp(startDate, endDate, area, level, index, timeLevel);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    /**
     * @param startDate
     * @param endDate
     * @param area
     * @param level     2 市  3区  4医院 5科室 6医生
     * @param index
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.appIndexHead)
    @ApiOperation(value = "app首页头部收入统计")
    public ObjEnvelop appIndexHead(
            @RequestParam(required = false) String startDate,
            @RequestParam(required = false) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.appIndexHead(startDate, endDate, area, level, index);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }

    /**
     * @param startDate
     * @param endDate
     * @param area
     * @param level     2 市  3区  4医院 5科室 6医生
     * @param index
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.appIndexCenter)
    @ApiOperation(value = "app首页中部门诊量统计")
    public ObjEnvelop appIndexCenter(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = false) String index) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.appIndexCenter(startDate, endDate, area, level, index);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }


    /**
     * app收入统计-头部
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level     2 市  3区  4医院 5科室 6医生
     * @param index
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.appPriceHead)
    @ApiOperation(value = "app收入统计-头部")
    public ObjEnvelop appPriceHead(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.appPriceHead(startDate, endDate, area, level, index);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }


    /**
     * app收入统计-头部
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level     2 市  3区  4医院 5科室 6医生
     * @param index
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.appPrescriptionHead)
    @ApiOperation(value = "app处方统计-头部")
    public ObjEnvelop appPrescriptionHead(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.appPrescriptionHead(startDate, endDate, area, level, index);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }

    /**
     * app中间饼状图-中部
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level     2 市  3区  4医院 5科室 6医生
     * @param index
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.appCenterCircular)
    @ApiOperation(value = "app中间饼状图-中部")
    public ObjEnvelop appCenter(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) String flag) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.appCenterCircular(startDate, endDate, area, level, index, flag);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    @GetMapping(value = BaseHospitalRequestMapping.Statistics.articleTotal)
    @ApiOperation(value = " 文章头部统计")
    public ObjEnvelop getArticleTotalHead(
            @ApiParam(name = "startDate", value = "开始时间", required = false)
            @RequestParam(value = "startDate", required = false) String startDate,
            @ApiParam(name = "endDate", value = "结束时间", required = false)
            @RequestParam(value = "endDate", required = false) String endDate,
            @ApiParam(name = "area", value = "区域编码 如 level2 area传市编码", required = true)
            @RequestParam(value = "area", required = true) String area,
            @ApiParam(name = "level", value = "2 市  3区  4医院 5、科室 6医生", required = true)
            @RequestParam(value = "level", required = true) int level,
            @ApiParam(name = "flag", value = "0自定义 1本月2近半年3本年", required = true)
            @RequestParam(value = "flag", required = true) Integer flag) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getArticleTotalHead(flag, startDate, endDate, area, level);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    @GetMapping(value = BaseHospitalRequestMapping.Statistics.getArticlePublishList)
    @ApiOperation(value = " 发布量柱状图")
    public ListEnvelop getArticleTotalHead(
            @ApiParam(name = "startDate", value = "开始时间", required = false)
            @RequestParam(value = "startDate", required = false) String startDate,
            @ApiParam(name = "endDate", value = "结束时间", required = false)
            @RequestParam(value = "endDate", required = false) String endDate,
            @ApiParam(name = "area", value = "区域编码 如 level2 area传市编码", required = true)
            @RequestParam(value = "area", required = true) String area,
            @ApiParam(name = "level", value = "2 市  3区  4医院 5、科室 6医生", required = true)
            @RequestParam(value = "level", required = true) int level,
            @ApiParam(name = "lowLevel", value = "下一级区域等级   2 市  3区  4医院 5、科室 6医生", required = true)
            @RequestParam(value = "lowLevel", required = true) String lowLevel,
            @ApiParam(name = "flag", value = "0自定义 1本月2近半年3本年", required = true)
            @RequestParam(value = "flag", required = true) Integer flag) {

        try {
            return success(statisticsEsService.getArticlePublishList(flag, startDate, endDate, level, area, lowLevel));
        } catch (Exception e) {
            e.printStackTrace();
            return failedListEnvelopException(e);
        }
    }


    @GetMapping(value = BaseHospitalRequestMapping.Statistics.getArticleOperateDateLine)
    @ApiOperation(value = " 文章互动情况")
    public ObjEnvelop getArticleOperateDateLine(
            @ApiParam(name = "startDate", value = "开始时间", required = false)
            @RequestParam(value = "startDate", required = false) String startDate,
            @ApiParam(name = "endDate", value = "结束时间", required = false)
            @RequestParam(value = "endDate", required = false) String endDate,
            @ApiParam(name = "area", value = "区域编码 如 level2 area传市编码", required = true)
            @RequestParam(value = "area", required = true) String area,
            @ApiParam(name = "level", value = "2 市  3区  4医院 5、科室 6医生", required = true)
            @RequestParam(value = "level", required = true) int level,
            @ApiParam(name = "flag", value = "0自定义 1本月2近半年3本年", required = true)
            @RequestParam(value = "flag", required = true) Integer flag) {

        try {
            return success(statisticsEsService.getArticleOperateDateLine(flag, startDate, endDate, level, area));
        } catch (Exception e) {
            e.printStackTrace();
            return failedObjEnvelopException(e);
        }
    }


    @GetMapping(value = BaseHospitalRequestMapping.Statistics.getArticleExcel)
    @ApiOperation(value = " 文章全院明细")
    public Envelop getArticleExcel(
            @ApiParam(name = "startDate", value = "开始时间", required = false)
            @RequestParam(value = "startDate", required = false) String startDate,
            @ApiParam(name = "endDate", value = "结束时间", required = false)
            @RequestParam(value = "endDate", required = false) String endDate,
            @ApiParam(name = "area", value = "区域编码 如 level2 area传市编码", required = true)
            @RequestParam(value = "area", required = true) String area,
            @ApiParam(name = "level", value = "2 市  3区  4医院 5、科室 6医生", required = true)
            @RequestParam(value = "level", required = true) int level,
            @ApiParam(name = "flag", value = "0自定义 1本月2近半年3本年", required = true)
            @RequestParam(value = "flag", required = true) Integer flag,
            @ApiParam(name = "lowLevel", value = "下一级区域等级   2 市  3区  4医院 5、科室 6医生", required = false)
            @RequestParam(value = "lowLevel", required = false) String lowLevel,
            @ApiParam(name = "lowCode", value = "", required = false)
            @RequestParam(value = "lowCode", required = false) String lowCode) {

        try {
            return success(statisticsEsService.getArticleExcel(flag, startDate, endDate, level, area, lowLevel, lowCode));
        } catch (Exception e) {
            e.printStackTrace();
            return failedObjEnvelopException(e);
        }
    }


    @GetMapping(value = BaseHospitalRequestMapping.Statistics.pushArticleCountExcel)
    @ApiOperation(value = " 导出文章全院明细")
    public ListEnvelop pushArticleCountExcel(
            @ApiParam(name = "startDate", value = "开始时间", required = false)
            @RequestParam(value = "startDate", required = false) String startDate,
            @ApiParam(name = "endDate", value = "结束时间", required = false)
            @RequestParam(value = "endDate", required = false) String endDate,
            @ApiParam(name = "area", value = "区域编码 如 level2 area传市编码", required = true)
            @RequestParam(value = "area", required = true) String area,
            @ApiParam(name = "level", value = "2 市  3区  4医院 5、科室 6医生", required = true)
            @RequestParam(value = "level", required = true) int level,
            @ApiParam(name = "flag", value = "0自定义 1本月2近半年3本年", required = true)
            @RequestParam(value = "flag", required = true) Integer flag,
            @ApiParam(name = "lowLevel", value = "下一级区域等级   2 市  3区  4医院 5、科室 6医生", required = false)
            @RequestParam(value = "lowLevel", required = false) String lowLevel,
            @ApiParam(name = "lowCode", value = "", required = false)
            @RequestParam(value = "lowCode", required = false) String lowCode,
            HttpServletResponse response) {

        try {
            List<Map<String, Object>> list = statisticsEsService.selectArticleCountExcel(flag, startDate, endDate, level, area, lowLevel, lowCode);
            response.setContentType("octets/stream");
            response.setHeader("Content-Disposition", "attachment; filename=" + new String("pushArticleTotalExcel.xls"));
            OutputStream os = response.getOutputStream();
            statisticsEsService.pushArticleTotalExcel(os, list);
            return success(list);
        } catch (Exception e) {
            e.printStackTrace();
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.Statistics.getArticleTotalList)
    @ApiOperation(value = " 文章发布量排行")
    public Envelop getArticleTotalList(
            @ApiParam(name = "startDate", value = "开始时间", required = false)
            @RequestParam(value = "startDate", required = false) String startDate,
            @ApiParam(name = "endDate", value = "结束时间", required = false)
            @RequestParam(value = "endDate", required = false) String endDate,
            @ApiParam(name = "area", value = "区域编码 如 level2 area传市编码", required = true)
            @RequestParam(value = "area", required = true) String area,
            @ApiParam(name = "level", value = "2 市  3区  4医院 5、科室 6医生", required = true)
            @RequestParam(value = "level", required = true) int level,
            @ApiParam(name = "flag", value = "0自定义 1本月2近半年3本年", required = true)
            @RequestParam(value = "flag", required = true) Integer flag,
            @ApiParam(name = "lowLevel", value = "下一级区域等级   2 市  3区  4医院 5、科室 6医生", required = false)
            @RequestParam(value = "lowLevel", required = false) String lowLevel,
            @ApiParam(name = "lowCode", value = "", required = false)
            @RequestParam(value = "lowCode", required = false) String lowCode) {

        try {
            return success(statisticsEsService.getArticleTotalList(flag, startDate, endDate, level, area, lowLevel, lowCode));
        } catch (Exception e) {
            e.printStackTrace();
            return failedObjEnvelopException(e);
        }
    }


    @GetMapping(value = BaseHospitalRequestMapping.Statistics.getArticleTypeCircle)
    @ApiOperation(value = " 文章类型统计")
    public Envelop getArticleTypeCircle(
            @ApiParam(name = "startDate", value = "开始时间", required = false)
            @RequestParam(value = "startDate", required = false) String startDate,
            @ApiParam(name = "endDate", value = "结束时间", required = false)
            @RequestParam(value = "endDate", required = false) String endDate,
            @ApiParam(name = "area", value = "区域编码 如 level2 area传市编码", required = true)
            @RequestParam(value = "area", required = true) String area,
            @ApiParam(name = "level", value = "2 市  3区  4医院 5、科室 6医生", required = true)
            @RequestParam(value = "level", required = true) int level,
            @ApiParam(name = "flag", value = "0自定义 1本月2近半年3本年", required = true)
            @RequestParam(value = "flag", required = true) Integer flag,
            @ApiParam(name = "lowLevel", value = "下一级区域等级   2 市  3区  4医院 5、科室 6医生", required = false)
            @RequestParam(value = "lowLevel", required = false) String lowLevel,
            @ApiParam(name = "lowCode", value = "", required = false)
            @RequestParam(value = "lowCode", required = false) String lowCode) {

        try {
            return success(statisticsEsService.getArticleTypeCircle(flag, startDate, endDate, level, area, lowLevel, lowCode));
        } catch (Exception e) {
            e.printStackTrace();
            return failedObjEnvelopException(e);
        }
    }


    /**
     * 查询收入，开方，问诊量
     * 参数：
     * strJson={"startDate":"2023-07-18","endDate":"2023-07-19","area":"350200","level":2,"flag":1}
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.getTotal3Data)
    @ApiOperation(value = "查询收入，开方，问诊量")
    public ObjEnvelop getTotal3Data(@RequestParam(required = true) String strJson) {
        JSONObject result = new JSONObject();
        ScreenQvo qvo = JSON.parseObject(strJson, ScreenQvo.class);
        try {
            result = statisticsEsService.getTotal3Data(qvo);
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("接口（getTotal3Data）：报错==>入参：" + JSON.toJSONString(qvo));
        }
        return success(result);
    }

    /**
     * 获取咨询各类型的数量
     * 参数：{"startDate":"2023-07-01","endDate":"2023-07-19","area":"350200","level":2,"flag":1}
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.getKindsConsultCount)
    @ApiOperation(value = "获取咨询各类型的数量")
    public ObjEnvelop getKindsConsultCount(@RequestParam(required = true) String strJson) {
        JSONObject result = new JSONObject();
        ScreenQvo qvo = JSON.parseObject(strJson, ScreenQvo.class);
        try {
            result = statisticsEsService.getKindsConsultCount(qvo);
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("接口（getKindsConsultCount）：报错==>入参：" + JSON.toJSONString(qvo));
        }
        return success(result);
    }

    /**
     * 【业务情况】 统计全院在相应时间内累计的：已接诊量、未接诊量、取消量的数据，三者之和等于总问诊量；
     * 获取 已接诊量、未接诊量、取消量、处方量数量
     * 参数：
     * timeType=1 按天分组
     * timeType=3 按月分组
     * strJson={"startDate":"2023-07-15","endDate":"2023-07-16","area":"350200","level":2,"flag":1}
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.getConsultAndPrescriptionList)
    @ApiOperation(value = "获取【已接诊量、未接诊量、取消量、处方量】数据")
    public ObjEnvelop getConsultAndPrescriptionList(@RequestParam(required = true) String strJson) {
        JSONObject result = new JSONObject();
        ScreenQvo qvo = JSON.parseObject(strJson, ScreenQvo.class);
        try {
            result = statisticsEsService.getConsultAndPrescriptionList(qvo);
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("接口（getConsultAndPrescriptionList）：报错==>入参：" + JSON.toJSONString(qvo));
        }
        return success(result);
    }


    /**
     * 费用情况
     * 【费用情况】分别展示每日药品费与诊查费用情况
     * 参数：
     * timeType=1 按天分组
     * timeType=3 按月分组
     * strJson={"startDate":"2023-07-15","endDate":"2023-07-16","area":"350200","level":2,"flag":1}
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.getDrugAndExaminationFeeList)
    @ApiOperation(value = "获取【药品费、诊查费】数据")
    public ObjEnvelop getDrugAndExaminationFeeList(@RequestParam(required = true) String strJson) {
        JSONObject result = new JSONObject();
        ScreenQvo qvo = JSON.parseObject(strJson, ScreenQvo.class);
        try {
            result = statisticsEsService.getDrugAndExaminationFeeList(qvo);
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("接口（getDrugAndExaminationFeeList）：报错==>入参：" + JSON.toJSONString(qvo));
        }
        return success(result);
    }

    /**
     * 获取【科室、医生】的接诊量接诊率排名
     * 参数：
     * 科室参数：strJson={"startDate":"2023-07-01","endDate":"2023-07-20","area":"350211A1002","level":4,"flag":"1"}
     * 医生参数：strJson={"startDate":"2023-07-17","endDate":"2023-07-20","area":"","level":5,"flag":"1"}
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.getDeptOrDoctorVolumeList)
    @ApiOperation(value = " 获取【科室、医生】的接诊量接诊率排名")
    public ObjEnvelop getDeptOrDoctorVolumeList(@RequestParam(required = true) String strJson) {
        JSONObject result = new JSONObject();
        ScreenQvo qvo = JSON.parseObject(strJson, ScreenQvo.class);
        try {
            result = statisticsEsService.getDeptOrDoctorVolumeList(qvo);
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("接口（getDeptOrDoctorVolumeList）：报错==>入参：" + JSON.toJSONString(qvo));
        }
        return success(result);
    }


    /**
     * 【全院明细】
     * 科室、医生、问诊量、接诊量、未接诊量、接诊率、处方量、检查费、药品费、总收入
     * 参数：
     * 科室：strJson={"startDate":"2023-07-01","endDate":"2023-07-20","area":"350211A1002","level":4,"lowLevel":5,"flag":"1"}
     * 医生：strJson={"startDate":"2023-07-17","endDate":"2023-07-20","area":"科室代码","level":5,"lowLevel":6,"flag":"1"}
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.getAllHospitalDetails)
    @ApiOperation(value = "【全院明细】")
    public ObjEnvelop getAllHospitalDetails(@RequestParam(required = true) String strJson) {
        JSONObject result = new JSONObject();
        ScreenQvo qvo = JSON.parseObject(strJson, ScreenQvo.class);
        try {
            result = statisticsEsService.getAllHospitalDetails(qvo);
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("接口（getAllHospitalDetails）：报错==>入参：" + JSON.toJSONString(qvo));
        }
        return success(result);
    }

    /**
     * 导出全院数据 exportHospitalDetails
     * {"startDate":"","endDate":"","area":"350211A1002","level":4,"lowLevel":5,"flag":1}
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.exportHospitalDetails)
    @ApiOperation(value = "【导出全院明细】")
    public void exportHospitalDetails(@RequestParam(required = true) String strJson, HttpServletResponse response) {
        ScreenQvo qvo = JSON.parseObject(strJson, ScreenQvo.class);
        try {
            response.setContentType("octets/stream");
            response.setHeader("Content-Disposition", "attachment; filename=" + new String("hospitalDetails.xls"));
            OutputStream os = response.getOutputStream();
            statisticsEsService.getAllDeptAndDoctorDetails(qvo, os); //导出
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("接口（exportHospitalDetails）：报错==>入参：" + JSON.toJSONString(qvo));
        }
    }


    /**
     * 查看科室排班数据
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.getDeptScheduling)
    @ApiOperation(value = "查看科室排班数据")
    public ObjEnvelop getDeptScheduling(@RequestParam(required = true) String strJson, HttpServletResponse response) {
        JSONObject result = new JSONObject();
        ScreenQvo qvo = JSON.parseObject(strJson, ScreenQvo.class);
        try {
            result = statisticsEsService.getDeptScheduling(qvo);
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("接口（getDeptScheduling）：报错==>入参：" + JSON.toJSONString(qvo));
        }
        return success(result);
    }

    /**
     * 获取功能全部的设置
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.getAllFunctionSetting)
    @ApiOperation(value = "获取功能全部的设置")
    public ObjEnvelop getAllFunctionSetting() {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getAllFunctionSetting();
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("接口（getAllFunctionSetting）：报错");
        }
        return success(result);
    }


    /**
     * 获取医生常用设置
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.getDoctorSetting)
    @ApiOperation(value = "获取医生常用设置")
    public Envelop getDoctorSetting(
            @ApiParam(name = "doctorCode", value = "医生code", required = true) @RequestParam(value = "doctorCode", required = false) String doctorCode) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getDoctorSetting(doctorCode);
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("接口（getDoctorSetting）：报错==>入参：" + JSON.toJSONString(doctorCode));
        }
        return success(result);
    }

    /**
     * 修改医生的常用设置
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.updateDoctorSetting)
    @ApiOperation(value = "修改医生的常用设置")
    public Envelop updateDoctorSetting(
            @ApiParam(name = "doctorCode", value = "医生code", required = true) @RequestParam(value = "doctorCode", required = false) String doctorCode,
            @ApiParam(name = "functionCodeList", value = "菜单id", required = true) @RequestParam(value = "functionCodeList", required = false) String functionCodeList) {
        JSONObject result = new JSONObject();
        try {
            int i = statisticsEsService.updateDoctorSetting(doctorCode, functionCodeList);
            if (i == 1) {
                return success("更新成功");
            } else {
                return success("更新失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("接口（updateDoctorSetting）：报错==>入参：" + doctorCode + "----" + functionCodeList);
        }
        return null;
    }

    /**
     * 新增功能的信息
     */
    @PostMapping(value = BaseHospitalRequestMapping.Statistics.insertFunctionDic)
    @ApiOperation(value = "新增功能的信息")
    public String insertFunctionDic(
            @ApiParam(name = "dicCode", value = "code", required = true) @RequestParam(value = "dicCode", required = true) String dicCode,
            @ApiParam(name = "dicName", value = "功能名称", required = true) @RequestParam(value = "dicName", required = false) String dicName,
            @ApiParam(name = "img", value = "图片", required = true) @RequestParam(value = "img", required = false) String img,
            @ApiParam(name = "modelName", value = "模块名称", required = true) @RequestParam(value = "modelName", required = false) String modelName
    ) {
        try {
            statisticsEsService.insertFunctionDic(dicCode, dicName, img, modelName);
            return write(200, "保存成功");
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, e.getMessage());
        }
    }
}