package com.yihu.jw.hospital.endpoint.survey;

import com.yihu.jw.hospital.service.consult.BasePatientBusinessService;
import com.yihu.jw.hospital.survey.service.SurveyService;
import com.yihu.jw.restmodel.hospital.consult.WlyyHospitalSysDictVO;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * Created by Trick on 2019/9/10.
 */
@RestController
@RequestMapping(value = BaseHospitalRequestMapping.WlyySurvey.PREFIX)
@Api(value = "问卷调查", description = "问卷调查", tags = {"问卷调查"})
public class SurveyEndpoint extends EnvelopRestEndpoint {

    @Autowired
    private SurveyService surveyService;

    @Autowired
    private BasePatientBusinessService businessService;

    @GetMapping(value = BaseHospitalRequestMapping.WlyySurvey.findSurveyLabel)
    @ApiOperation(value = "问题-查询字典")
    public ListEnvelop findSurveyLabel(@ApiParam(name = "name", value = "1.surveyLabel 标签；2.surveyScreenLabel 问卷类型")
                                       @RequestParam(value = "name",required = true) String name) throws Exception {

        try {
            List<WlyyHospitalSysDictVO> vos = surveyService.findSurveyLabel(name);
            return success(vos);
        }catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.WlyySurvey.findSurveyQuestion)
    @ApiOperation(value = "问题-获取问卷问题列表")
    public MixEnvelop findSurveyQuestion(@ApiParam(name = "title", value = "标题")
                                          @RequestParam(value = "title",required = false) String title,
                                         @ApiParam(name = "creater", value = "创建者")
                                        @RequestParam(value = "creater",required = false) String creater,
                                         @ApiParam(name = "questionType", value = "问题类型")
                                          @RequestParam(value = "questionType",required = false)Integer questionType,
                                         @ApiParam(name = "page", value = "第几页，1开始")
                                          @RequestParam(value = "page",required = true)Integer page,
                                         @ApiParam(name = "size", value = "每页大小")
                                          @RequestParam(value = "size",required = true)Integer size) throws Exception {
        try {
            return surveyService.findSurveyQuestion(title,creater,questionType,page,size);
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.WlyySurvey.findBySurveyId)
    @ApiOperation(value = "问题-获取问卷问题详情（单条）")
    public ObjEnvelop findBySurveyId(@ApiParam(name = "surveyId", value = "问卷Id")
                                     @RequestParam(value = "surveyId",required = true) String surveyId) throws Exception {
        try {
            return success(surveyService.findBySurveyId(surveyId));
        }catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @PostMapping(value = BaseHospitalRequestMapping.WlyySurvey.saveSurveyQuestion)
    @ApiOperation(value = "问题-批量保存问卷问题")
    public ObjEnvelop saveSurveyQuestion(@ApiParam(name = "sqjsons", value = "问题实体列表json")
                                         @RequestParam(value = "sqjsons",required = true)String sqjsons,
                                         @ApiParam(name = "sqOptionJson", value = "选项实体列表json")
                                         @RequestParam(value = "sqOptionJson",required = false)String sqOptionJson) throws Exception {
        try {
            return success(surveyService.saveSurveyQuestion(sqjsons,sqOptionJson));

        }catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @PostMapping(value = BaseHospitalRequestMapping.WlyySurvey.updataSurveyQuestion)
    @ApiOperation(value = "问题-更新问题（单条）")
    public ObjEnvelop updataSurveyQuestion(@ApiParam(name = "sqjson", value = "问题实体json")
                                           @RequestParam(value = "sqjson",required = true)String sqjson,
                                           @ApiParam(name = "sqOptionJson", value = "选项实体列表json")
                                           @RequestParam(value = "sqOptionJson",required = false)String sqOptionJson) throws Exception {
        try {
            return success(surveyService.updataSurveyQuestion(sqjson,sqOptionJson));
        }catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @PostMapping(value = BaseHospitalRequestMapping.WlyySurvey.delSurveyQuestion)
    @ApiOperation(value = "问题-批量删除")
    public ObjEnvelop delSurveyQuestion(@ApiParam(name = "ids", value = "问题实体ID,逗号分割")
                                        @RequestParam(value = "ids",required = true)String ids) throws Exception {
        try {
            return success(surveyService.delSurveyQuestion(ids));
        }catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }


    @GetMapping(value = BaseHospitalRequestMapping.WlyySurvey.findSurveyTemplate)
    @ApiOperation(value = "模板-查询模板列表")
    public MixEnvelop findSurveyTemplate(@ApiParam(name = "title", value = "标题")
                                              @RequestParam(value = "title",required = false) String title,
                                         @ApiParam(name = "creater", value = "创建者")
                                         @RequestParam(value = "creater",required = false) String creater,
                                          @ApiParam(name = "label", value = "标签code")
                                              @RequestParam(value = "label",required = false)String label,
                                          @ApiParam(name = "page", value = "第几页，1开始")
                                              @RequestParam(value = "page",required = true)Integer page,
                                          @ApiParam(name = "size", value = "每页大小")
                                              @RequestParam(value = "size",required = true)Integer size) throws Exception {

        try {
            return surveyService.findSurveyTemplate( title, creater,label, page, size);
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.WlyySurvey.findSurveyTemplateById)
    @ApiOperation(value = "模板-获取模板详情（单条）")
    public ObjEnvelop findSurveyTemplateById(@ApiParam(name = "tempId", value = "模板ID")
                                             @RequestParam(value = "tempId",required = true) String tempId,
                                             @ApiParam(name = "userCode", value = "当前用户id")
                                             @RequestParam(value = "userCode",required = false) String userCode ) throws Exception {
        try {
            return success(surveyService.findSurveyTemplateById(tempId,userCode));
        }catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @PostMapping(value = BaseHospitalRequestMapping.WlyySurvey.saveSurveyTemplate)
    @ApiOperation(value = "问题-批量保存问卷问题")
    public ObjEnvelop saveSurveyTemplate(@ApiParam(name = "tempJson", value = "模板实体json")
                                         @RequestParam(value = "tempJson",required = true)String tempJson,
                                         @ApiParam(name = "tempQJson", value = "模板问题列表json")
                                         @RequestParam(value = "tempQJson",required = false)String tempQJson,
                                         @ApiParam(name = "tempOpJson", value = "模板问题选项列表json")
                                         @RequestParam(value = "tempOpJson",required = false)String tempOpJson,
                                         @ApiParam(name = "labelJson", value = "模板标签json")
                                         @RequestParam(value = "labelJson",required = false)String labelJson,
                                         @ApiParam(name = "labelInspJson", value = "模板筛查标签json")
                                         @RequestParam(value = "labelInspJson",required = false)String labelInspJson) throws Exception {
        try {
            return success(surveyService.saveSurveyTemplate(tempJson,tempQJson,tempOpJson,labelJson,labelInspJson));
        }catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @PostMapping(value = BaseHospitalRequestMapping.WlyySurvey.delSurveyTemplate)
    @ApiOperation(value = "模板-删除模板")
    public ObjEnvelop delSurveyTemplate(@ApiParam(name = "tempId", value = "模板实体json")
                                        @RequestParam(value = "tempId",required = true)String tempId) throws Exception {
        try {
            return success(surveyService.delSurveyTemplate(tempId));
        }catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    //=================调查问卷居民查询\答题
    @PostMapping(value = BaseHospitalRequestMapping.WlyySurvey.saveSurveyDept)
    @ApiOperation(value = "模板-设置模板发送的科室")
    public ObjEnvelop saveSurveyDept(@ApiParam(name = "tempId", value = "模板ID")
                                     @RequestParam(value = "tempId",required = true)String tempId,
                                     @ApiParam(name = "sdJsons", value = "部门与科室关系json")
                                     @RequestParam(value = "sdJsons",required = false)String sdJsons)throws Exception {
        try {
            return success(surveyService.saveSurveyDept(tempId,sdJsons));
        }catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @PostMapping(value = BaseHospitalRequestMapping.WlyySurvey.saveDeptSurvey)
    @ApiOperation(value = "模板-设置科室发送的模板")
    public ObjEnvelop saveDeptSurvey(@ApiParam(name = "dept", value = "部门code")
                                     @RequestParam(value = "dept",required = true)String dept,
                                     @ApiParam(name = "sdJsons", value = "部门与科室关系json")
                                     @RequestParam(value = "sdJsons",required = false)String sdJsons)throws Exception {
        try {
            return success(surveyService.saveDeptSurvey(dept,sdJsons));
        }catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.WlyySurvey.findSurveyByDept)
    @ApiOperation(value = "模板-查询部门下发放的问卷")
    public ListEnvelop findSurveyByDept(@ApiParam(name = "dept", value = "科室")
                                        @RequestParam(value = "dept",required = true)String dept)throws Exception {

        try {
            return success(surveyService.findSurveyByDept(dept));
        }catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }


    @GetMapping(value = BaseHospitalRequestMapping.WlyySurvey.findSurveyByDeptAndPatient)
    @ApiOperation(value = "模板-查询部门下发放的问卷用户答题情况")
    public ListEnvelop findSurveyByDeptAndPatient(@ApiParam(name = "dept", value = "科室")
                                                  @RequestParam(value = "dept",required = true)String dept,
                                                  @ApiParam(name = "patient", value = "居民code")
                                                  @RequestParam(value = "patient",required = true)String patient)throws Exception {
        try {
            return success(surveyService.findSurveyByDeptAndPatient(dept,patient));
        }catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.WlyySurvey.findDeptBySurvey)
    @ApiOperation(value = "模板-查询模板下的科室")
    public ListEnvelop findDeptBySurvey(@ApiParam(name = "tempId", value = "模板ID")
                                        @RequestParam(value = "tempId",required = true)String tempId)throws Exception {
        try {
            return success(surveyService.findDeptBySurvey(tempId));
        }catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }


    @PostMapping(value = BaseHospitalRequestMapping.WlyySurvey.saveSurveyAnswer)
    @ApiOperation(value = "模板-保存问卷答案")
    public ObjEnvelop saveSurveyAnswer(@ApiParam(name = "suJson", value = "发送状态json")
                                     @RequestParam(value = "suJson",required = true)String suJson,
                                     @ApiParam(name = "suaJsons", value = "答案集合json")
                                     @RequestParam(value = "suaJsons",required = true)String suaJsons)throws Exception {
        try {
            return success(surveyService.saveSurveyAnswer(suJson, suaJsons));
        }catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.WlyySurvey.findAnswerBySurveyTempCode)
    @ApiOperation(value = "模板-查询用户问卷答案")
    public ListEnvelop findAnswerBySurveyTempCode(@ApiParam(name = "patient", value = "居民ID")
                                                  @RequestParam(value = "patient",required = true)String patient,
                                                  @ApiParam(name = "tempId", value = "模板id")
                                                  @RequestParam(value = "tempId",required = true)String tempId,
                                                  @ApiParam(name = "surverUserId", value = "答题批次号")
                                                  @RequestParam(value = "surverUserId",required = false)String surverUserId)throws Exception {
        try {
            return success(surveyService.findAnswerBySurveyTempCode(surverUserId,patient,tempId));
        }catch (Exception e) {
            return failedListEnvelopException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.WlyySurvey.findSurveyTemplateResult)
    @ApiOperation(value = "统计-查询问题情况")
    public ObjEnvelop findSurveyTemplateResult(@ApiParam(name = "tempId", value = "模板ID")
                                                @RequestParam(value = "tempId",required = true)String tempId)throws Exception {

        try {
            return success(surveyService.findSurveyTemplateResult(tempId));
        }catch (Exception e) {
            return failedObjEnvelopException(e);
        }
    }

    @GetMapping(value = BaseHospitalRequestMapping.WlyySurvey.findQuestionInfoList)
    @ApiOperation(value = "统计-查询问题情况")
    public MixEnvelop findQuestionInfoList(@ApiParam(name = "comment", value = "选项备注")
                                           @RequestParam(value = "comment",required = false) String comment,
                                           @ApiParam(name = "temId", value = "选项内容，问答题内容")
                                           @RequestParam(value = "temId",required = false)String temId,
                                           @ApiParam(name = "content", value = "选项内容，问答题内容")
                                           @RequestParam(value = "content",required = false)String content,
                                           @ApiParam(name = "tempQuestionCode", value = "问题code")
                                           @RequestParam(value = "tempQuestionCode",required = false)String tempQuestionCode,
                                           @ApiParam(name = "tempOptionCode", value = "选项code")
                                           @RequestParam(value = "tempOptionCode",required = false)String tempOptionCode,
                                           @ApiParam(name = "page", value = "第几页，1开始")
                                           @RequestParam(value = "page",required = true)Integer page,
                                           @ApiParam(name = "size", value = "每页大小")
                                           @RequestParam(value = "size",required = true)Integer size)throws Exception {

        try {
            return surveyService.findQuestionInfoList(comment,temId,content,tempQuestionCode,tempOptionCode,page, size);
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }
    @GetMapping(value = BaseHospitalRequestMapping.WlyySurvey.findAnswerList)
    @ApiOperation(value = "统计-综合查询问题答案")
    public MixEnvelop findAnswerList(@ApiParam(name = "tempId", value = "模板id")
                                     @RequestParam(value = "tempId",required = false) String tempId,
                                     @ApiParam(name = "patient", value = "标题")
                                     @RequestParam(value = "patient",required = false) String patient,
                                     @ApiParam(name = "patientName", value = "患者名字")
                                         @RequestParam(value = "patientName",required = false) String patientName,
                                     @ApiParam(name = "title", value = "标题")
                                     @RequestParam(value = "title",required = false) String title,
                                     @ApiParam(name = "page", value = "第几页，1开始")
                                     @RequestParam(value = "page",required = true)Integer page,
                                     @ApiParam(name = "size", value = "每页大小")
                                     @RequestParam(value = "size",required = true)Integer size)throws Exception {

        try {
            return surveyService.findAnswerList(tempId,patient,patientName,title,page,size);
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }
    }
    //=================

    @GetMapping(value = BaseHospitalRequestMapping.WlyySurvey.getSurveyList)
    @ApiOperation(value = "查询问卷模板")
    public MixEnvelop getUsedList(@ApiParam(name = "title", value = "模板名称")
                                     @RequestParam(value = "title",required = false) String title,
                               @ApiParam(name = "doctor", value = "医生id")
                                     @RequestParam(value = "doctor",required = false) String doctor,
                               @ApiParam(name = "used", value = "used")
                                     @RequestParam(value = "used",required = false) Integer used,
                                  @ApiParam(name = "labelCode", value = "labelCode")
                                      @RequestParam(value = "labelCode",required = false) String labelCode,
                               @ApiParam(name = "page", value = "第几页，1开始",defaultValue = "1")
                                     @RequestParam(value = "page",required = true,defaultValue = "1")Integer page,
                               @ApiParam(name = "pageSize", value = "每页大小",defaultValue = "10")
                                     @RequestParam(value = "pageSize",required = true,defaultValue = "10")Integer pageSize)throws Exception {
        try {
            return businessService.getUsedList(title,doctor,labelCode,used,page,pageSize);
        }catch (Exception e) {
            return failedMixEnvelopException(e);
        }

    }
    @GetMapping(value = BaseHospitalRequestMapping.WlyySurvey.setUsed)
    @ApiOperation(value = "设置模板为常用")
    public Envelop setUsed(@ApiParam(name = "tempId", value = "模板id")
                                     @RequestParam(value = "tempId",required = false) String tempId,
                                     @ApiParam(name = "used", value = "是否常用")
                                     @RequestParam(value = "used",required = false) Integer used,
                                     @ApiParam(name = "doctor", value = "医生id")
                                     @RequestParam(value = "doctor",required = false) String doctor)throws Exception {


        try {
            businessService.setUsed(tempId,used,doctor);
            return success("操作成功");
        }catch (Exception e) {
            return failedException(e);
        }
    }
}
