package com.yihu.jw.hospital.endpoint.team;

import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.team.service.TeamService;
import com.yihu.jw.wlyy.service.WlyyBusinessService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * Created by yeshijie on 2023/4/24.
 */
@RestController
@RequestMapping(value = "doctorTeam")
@Api(value = "医生团队",  tags = {"医生团队"})
public class DoctorTeamEndpoint extends EnvelopRestEndpoint {

    @Autowired
    private TeamService teamService;
    @Autowired
    private WlyyBusinessService wlyyBusinessService;
    @Autowired
    private BasePatientDao patientDao;
    private static final String TONGAN_TOWN = "350212";

    @GetMapping(value = "doctorInfo")
    @ApiOperation(value = "获取社区医生信息")
    public String doctorInfo(@RequestParam(required = true) String code) {
        try {
            return wlyyBusinessService.doctorInfo(code).toJSONString();
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1,"查询失败");
        }
    }

    @GetMapping(value = "patientInfo")
    @ApiOperation(value = "获取i健康签约居民信息")
    public String patientInfo(@RequestParam(required = true) String code) {
        try {
            return wlyyBusinessService.patientInfo(code).toJSONString();
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1,"查询失败");
        }
    }

    @GetMapping(value = "getAdminTeamList")
    @ApiOperation(value = "获取社区团队列表")
    public String getAdminTeamList(@RequestParam(required = false) String teamName,
                                    @RequestParam(required = false) String teamId,
                                    @RequestParam(required = false) String leadName,
                                   @RequestParam(required = false) String doctorName,
                                   @RequestParam(required = false) String mobile,
                                    @RequestParam(required = false) Integer page,
                                    @RequestParam(required = false) Integer pageSize,
                                    @RequestParam(required = false) String hospital) {
        try {
            return wlyyBusinessService.getAdminTeamList(teamName,TONGAN_TOWN,teamId,leadName,page,pageSize,hospital,doctorName,mobile).toJSONString();
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1,"查询失败");
        }
    }

    @GetMapping(value = "getAdminTeamMemberList")
    @ApiOperation(value = "获取社区团队成员列表")
    public String getAdminTeamMemberList(@RequestParam(required = false) String teamId,
                                          @RequestParam(required = false) String doctorName,
                                          @RequestParam(required = false) Integer page,
                                          @RequestParam(required = false) Integer pageSize,
                                          @RequestParam(required = false) String mobile) {
        try {
            return wlyyBusinessService.getAdminTeamMemberList(teamId,doctorName,page,pageSize,mobile).toJSONString();
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1,"查询失败");
        }
    }

    @GetMapping(value = "getHospitalList")
    @ApiOperation(value = "获取社区机构列表分页")
    public String getHospitalList(@RequestParam(required = false) String name,
                                   @RequestParam(required = false) Integer page,
                                   @RequestParam(required = false) Integer pageSize) {
        try {
            return wlyyBusinessService.getHospitalList(name,page,pageSize,TONGAN_TOWN).toJSONString();
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1,"查询失败");
        }
    }

    @GetMapping(value = "getSignList")
    @ApiOperation(value = "获取签约社区列表分页")
    public String getSignList(@RequestParam(required = false) String name,
                               @RequestParam(required = false) Integer page,
                               @RequestParam(required = false) Integer pageSize,
                               @RequestParam(required = false) String teamId,
                               @RequestParam(required = false) String hospital,
                               @RequestParam(required = false) String adminTeamCode,
                               @RequestParam(required = false) String doctor,
                               @RequestParam(required = false) String doctorName,
                               @RequestParam(required = false) String doctorHealthName,
                               @RequestParam(required = false) String hospitalName,
                               @RequestParam(required = false) String idcard,
                               @RequestParam(required = false) String address,
                               @RequestParam(required = false) String teamName,
                               @RequestParam(required = false) String leaderName) {
        try {
            return wlyyBusinessService.getSignList(name,page,pageSize,TONGAN_TOWN,hospital,adminTeamCode,doctor,doctorName,
                    doctorHealthName,hospitalName, idcard, address,teamName,leaderName,teamId).toJSONString();
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1,"查询失败");
        }
    }
}
