package com.yihu.jw.hospital.endpoint.team;

import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.team.service.TeamService;
import com.yihu.jw.wlyy.service.WlyyBusinessService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * Created by yeshijie on 2023/4/24.
 */
@RestController
@RequestMapping(value = "specialistTeam")
@Api(value = "专科医生团队",  tags = {"专科医生团队"})
public class SpecialistTeamEndpoint extends EnvelopRestEndpoint {

    @Autowired
    private TeamService teamService;
    @Autowired
    private WlyyBusinessService wlyyBusinessService;
    @Autowired
    private BasePatientDao patientDao;
    private static final String TONGAN_TOWN = "350212";

    @GetMapping(value = "isSign")
    @ApiOperation(value = "居民是否签约")
    public Envelop isSign(@RequestParam(required = true) String id) {
        try {
            BasePatientDO patientDO = patientDao.findById(id).orElse(null);
            if(patientDO==null){
                return Envelop.getError("居民不存在",-1);
            }
            return teamService.isSign(patientDO.getIdcard(),null);
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("查询失败",-1);
        }
    }

    @GetMapping(value = "signInfo")
    @ApiOperation(value = "居民签约详情")
    public Envelop signInfo(@RequestParam(required = true) String id) {
        try {
            BasePatientDO patientDO = patientDao.findById(id).orElse(null);
            if(patientDO==null){
                return Envelop.getError("居民不存在",-1);
            }
            return teamService.isSign(patientDO.getIdcard(),"1");
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("查询失败",-1);
        }
    }

    @GetMapping(value = "teamDetail")
    @ApiOperation(value = "获取团队详情")
    public Envelop teamDetail(@RequestParam(required = true) String id) {
        try {
            return teamService.teamDetail(id);
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("查询失败",-1);
        }
    }

    @GetMapping(value = "getTeamList")
    @ApiOperation(value = "获取专科团队列表")
    public Envelop getTeamList(@RequestParam(required = false) String teamName,
                               @RequestParam(required = false) String dept,
                               @RequestParam(required = false) String recommend,
                               @RequestParam(required = false) String type,
                               @RequestParam(required = false) Integer adminTeamId,
                               @RequestParam(required = false) String hospital,
                               @RequestParam(required = false) String recommendDoctorName,
                               @RequestParam(required = false) Integer page,
                               @RequestParam(required = false) Integer pageSize) {
        try {
            return teamService.getTeamList(type,adminTeamId,teamName,dept,page,pageSize,hospital,recommend,recommendDoctorName);
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("查询失败",-1);
        }
    }

    @GetMapping(value = "getTeamMemberList")
    @ApiOperation(value = "获取团队成员列表")
    public Envelop getTeamMemberList(@RequestParam(required = false) String teamId,
                               @RequestParam(required = false) String doctorName,
                               @RequestParam(required = false) String mobile,
                               @RequestParam(required = false) Integer page,
                               @RequestParam(required = false) Integer pageSize) {
        try {
            return teamService.getTeamMemberList(teamId,doctorName,page,pageSize,mobile);
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("查询失败",-1);
        }
    }

    @GetMapping(value = "doctorListByTeamId")
    @ApiOperation(value = "添加团队成员时的医生列表")
    public Envelop doctorListByTeamId(@RequestParam(required = false) String teamId,
                                     @RequestParam(required = false) String name,
                                     @RequestParam(required = false) String dept,
                                     @RequestParam(required = false) Integer page,
                                     @RequestParam(required = false) Integer pageSize) {
        try {
            return teamService.doctorListByTeamId(teamId,name,dept,page,pageSize);
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("查询失败",-1);
        }
    }

    @PostMapping(value = "saveTeamMember")
    @ApiOperation(value = "保存团队成员")
    public Envelop saveTeamMember(@RequestParam(required = true) String teamId,@RequestParam(required = true) String doctors) {
        try {
            teamService.saveTeamMember(teamId,doctors);
            return Envelop.getSuccess("保存成功");
        }  catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("保存失败",-1);
        }
    }


    @GetMapping(value = "doctorInfo")
    @ApiOperation(value = "获取医生信息")
    public Envelop doctorInfo(@RequestParam(required = true) String id) {
        try {
            return ObjEnvelop.getSuccess("查询成功",teamService.doctorInfo(id));
        }  catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("查询失败",-1);
        }
    }

    @GetMapping(value = "findTeamById")
    @ApiOperation(value = "按id获取专科团队信息")
    public Envelop findTeamById(@RequestParam(required = true) String id) {
        try {
            return ObjEnvelop.getSuccess("查询成功",teamService.findTeamById(id));
        }  catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("查询失败",-1);
        }
    }

    @PostMapping(value = "saveTeam")
    @ApiOperation(value = "保存团队")
    public Envelop saveTeam(@RequestParam(required = true) String jsonData) {
        try {
            return ObjEnvelop.getSuccess("保存成功",teamService.saveTeam(jsonData));
        }  catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("保存失败",-1);
        }
    }

    @PostMapping(value = "delTeamMember")
    @ApiOperation(value = "删除团队成员")
    public Envelop delTeamMember(@RequestParam(required = true) Long id) {
        try {
            teamService.delTeamMember(id);
            return Envelop.getSuccess("保存成功");
        }  catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("保存失败",-1);
        }
    }

    @PostMapping(value = "saveIjkTeam")
    @ApiOperation(value = "专科团队保存关联社区")
    public Envelop saveIjkTeam(@RequestParam(required = true) String teamId,
                               @RequestParam(required = true) String adminTeamIds) {
        try {
            teamService.saveIjkTeam(teamId,adminTeamIds);
            return Envelop.getSuccess("保存成功");
        }  catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("保存失败",-1);
        }
    }

    @PostMapping(value = "delIjkTeam")
    @ApiOperation(value = "删除关联社区")
    public Envelop delIjkTeam(@RequestParam(required = true) String teamId,
                              @RequestParam(required = true) Integer adminTeamId)  {
        try {
            teamService.delIjkTeam(teamId,adminTeamId);
            return Envelop.getSuccess("删除成功");
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("删除失败",-1);
        }
    }

    @GetMapping(value = "getTeamRelationList")
    @ApiOperation(value = "获取关联社区列表分页")
    public String getTeamRelationList(@RequestParam(required = false) String teamId,
                                       @RequestParam(required = false) Integer page,
                                       @RequestParam(required = false) Integer pageSize,
                                       @RequestParam(required = false) String doctorName,
                                       @RequestParam(required = false) String mobile,
                                       @RequestParam(required = false) String hospitalName,
                                       @RequestParam(required = false) String leaderName,
                                       @RequestParam(required = false) String teamName) {
        try {
            return wlyyBusinessService.getTeamRelationList(teamId,doctorName,page,pageSize,mobile,hospitalName,teamName,leaderName).toJSONString();
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1,"查询失败");
        }
    }


}
