package com.yihu.jw.hospital.endpoint.ykyy;

import com.yihu.jw.contant.CommonContant;
import com.yihu.jw.entity.order.BusinessOrderDO;
import com.yihu.jw.hospital.prescription.service.PrescriptionService;
import com.yihu.jw.hospital.prescription.service.entrance.YkyyEntranceService;
import com.yihu.jw.hospital.ykyy.service.YkyyService;
import com.yihu.jw.im.dao.ConsultTeamDao;
import com.yihu.jw.order.BusinessOrderService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import com.yihu.jw.wechat.enterprise.EnterpriseService;
import com.yihu.jw.wechat.service.WxTemplateService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Map;

/**
 * Created by Trick on 2020/1/19.
 */
@RestController
@RequestMapping(value ="/ykyy")
@Api(value = "眼科医院", description = "眼科医院", tags = {"眼科医院"})
public class YkyyController extends EnvelopRestEndpoint {


    @Autowired
    private YkyyService ykyyService;
    @Value("${wechat.id}")
    private String wxId;
    @Autowired
    private BusinessOrderService businessOrderService;
    @Autowired
    private YkyyEntranceService ykyyEntranceService;
    @Autowired
    private WxTemplateService wxTemplateService;
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private ConsultTeamDao consultTeamDao;
    @Autowired
    private PrescriptionService prescriptionService;

    @Value("${demo.flag}")
    private boolean demoFlag;
    @Value("${pay.flag}")
    private boolean payFlag;

    @GetMapping(value = "/selectCardList")
    @ApiOperation(value = "查询就诊卡")
    public ObjEnvelop selectCardList(@ApiParam(name = "patient", value = "sql语句", required = true)
                                      @RequestParam(value = "patient",required = true)String patient){
        try {
            return ObjEnvelop.getSuccess("ok",ykyyService.selectCards(patient));
        } catch (Exception e) {
            return ObjEnvelop.getError(e.getMessage());
        }
    }


    @GetMapping(value = "/addYkCard")
    @ApiOperation(value = "添加就诊卡")
    public ObjEnvelop addYkCard(@ApiParam(name = "patient", value = "sql语句", required = true)
                                     @RequestParam(value = "patient",required = true)String patient,
                                     @ApiParam(name = "card", value = "卡号", required = true)
                                     @RequestParam(value = "card",required = true)String card,
                                     @ApiParam(name = "type", value = "卡类型（0社保卡，1就诊卡）", required = true)
                                         @RequestParam(value = "type",required = true)String type,
                                    @ApiParam(name = "userId", value = "", required = false)
                                    @RequestParam(value = "userId",required = false)String userId){
        try {
            return ObjEnvelop.getSuccess("ok",ykyyService.addYkCard(patient,card,userId,type));
        } catch (Exception e) {
            return ObjEnvelop.getError(e.getMessage());
        }
    }


    /**
     * 眼科通登录接口
     *
     * @param tel
     * @param password
     * @return
     */
    @GetMapping(value = "/yktLogin")
    @ApiOperation(value = "眼科通登录接口")
    public ObjEnvelop yktLogin(@ApiParam(name = "tel", value = "电话号码", required = true)
                                @RequestParam(value = "tel",required = true)String tel,
                                @ApiParam(name = "password", value = "", required = true)
                                @RequestParam(value = "password",required = false)String password){
        try {
            return ObjEnvelop.getSuccess("ok",ykyyService.yktLogin(tel,password));
        } catch (Exception e) {
            return ObjEnvelop.getError(e.getMessage());
        }
    }

    @GetMapping(value = "/getShortMessage")
    @ApiOperation(value = "眼科通获取验证码")
    public ObjEnvelop getShortMessage(@ApiParam(name = "tel", value = "电话号码", required = true)
                               @RequestParam(value = "tel",required = true)String tel,
                               @ApiParam(name = "type", value = "1注册2登录", required = true)
                               @RequestParam(value = "type",required = false)String type){
        try {
            return ObjEnvelop.getSuccess("ok",ykyyService.getShortMessage(type,tel));
        } catch (Exception e) {
            return ObjEnvelop.getError(e.getMessage());
        }
    }


    /**
     * 眼科通忘记密码
     *
     * @param tel
     * @param password
     * @return
     */
    @GetMapping(value = "/forgetPassword")
    @ApiOperation(value = "眼科通忘记密码")
    public ObjEnvelop yktLogin(@ApiParam(name = "tel", value = "电话号码", required = true)
                               @RequestParam(value = "tel",required = true)String tel,
                               @ApiParam(name = "password", value = "", required = true)
                               @RequestParam(value = "password",required = false)String password,
                               @ApiParam(name = "vercode", value = "", required = true)
                               @RequestParam(value = "vercode",required = false)String vercode){
        try {
            return ObjEnvelop.getSuccess("ok",ykyyService.forgetPassword(tel,password,vercode));
        } catch (Exception e) {
            return ObjEnvelop.getError(e.getMessage());
        }
    }


    /**
     * 更新药品订单信息到眼科通
     *
     * @param orderNo
     * @param drugorder
     * @param cfsb
     * @return
     */
    @GetMapping(value = "/updateDrugorderInfo")
    @ApiOperation(value = "更新药品订单信息到眼科通")
    public ObjEnvelop updateDrugorderInfo(@ApiParam(name = "orderNo", value = "咨询订单号", required = true)
                               @RequestParam(value = "orderNo",required = true)String orderNo,
                               @ApiParam(name = "drugorder", value = "药品订单号", required = true)
                               @RequestParam(value = "drugorder",required = false)String drugorder,
                               @ApiParam(name = "cfsb", value = "处方识别号", required = true)
                               @RequestParam(value = "cfsb",required = false)String cfsb){
        try {
            return ObjEnvelop.getSuccess("ok",ykyyService.updateDrugorderInfo(orderNo,drugorder,cfsb));
        } catch (Exception e) {
            return ObjEnvelop.getError(e.getMessage());
        }
    }


    /**
     * 创建订单
     * @param json
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "createOrder", method = RequestMethod.POST)
    @ApiOperation("创建订单")
    public ObjEnvelop saveOrder(
            @ApiParam(name = "json", value = "json", required = true)
            @RequestParam(required = true)String json) throws Exception {
        BusinessOrderDO businessOrderDO = objectMapper.readValue(json,BusinessOrderDO.class);
        try {
            return ObjEnvelop.getSuccess("ok",businessOrderService.saveOrder(businessOrderDO));
        } catch (Exception e) {
            return ObjEnvelop.getError(e.getMessage());
        }
    }

    /**
     * 调用微信
     * @param body
     * @param totalFee
     * @param tradeType
     * @param openId
     * @param outTradeNo
     * @param notifyUrl
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "weixinPay", method = RequestMethod.POST)
    @ApiOperation("调用微信支付")
    public ObjEnvelop unifiedorder(
            @ApiParam(name = "body", value = "body", required = true)
            @RequestParam(required = true)String body,
            @ApiParam(name = "totalFee", value = "totalFee", required = true)
            @RequestParam(required = true)String totalFee,
            @ApiParam(name = "tradeType", value = "tradeType", required = true)
            @RequestParam(required = true)String tradeType,
            @ApiParam(name = "openId", value = "openId", required = true)
            @RequestParam(required = true)String openId,
            @ApiParam(name = "outTradeNo", value = "outTradeNo", required = true)
            @RequestParam(required = true)String outTradeNo,
            @ApiParam(name = "notifyUrl", value = "notifyUrl", required = true)
            @RequestParam(required = true)String notifyUrl,
            @ApiParam(name = "patient", value = "patient", required = false)
            @RequestParam(required = false)String patient) throws Exception {
        try {
            return ObjEnvelop.getSuccess("ok",businessOrderService.unifiedorder(wxId,body,totalFee,tradeType,openId,outTradeNo,notifyUrl,patient));
        } catch (Exception e) {
            return ObjEnvelop.getError(e.getMessage());
        }
    }



    /**
     * 创建订单并获取微信支付参数
     * @param json
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "saveOrderAndPay", method = RequestMethod.POST)
    @ApiOperation("创建订单并获取微信支付参数")
    public ObjEnvelop saveOrderAndPay(
            @ApiParam(name = "json", value = "json", required = true)
            @RequestParam(required = false)String json) throws Exception {
        BusinessOrderDO businessOrderDO = objectMapper.readValue(json,BusinessOrderDO.class);
        try {
            return ObjEnvelop.getSuccess("ok",businessOrderService.saveOrderAndPay(businessOrderDO,wxId));
        } catch (Exception e) {
            return ObjEnvelop.getError(e.getMessage());
        }
    }


    /**
     * 微信支付异步回调
     * @param request
     * @param response
     * @return
     * @throws IOException
     */
    @ResponseBody
    @ApiOperation("微信支付结果通知")
    @RequestMapping(value = BaseHospitalRequestMapping.WxPay.Notify, method = {RequestMethod.GET, RequestMethod.POST})
    public Map<String, String> wxPayNotify(HttpServletRequest request, HttpServletResponse response) throws IOException {
        String result = "";
        String inputLine;
        while ((inputLine = request.getReader().readLine()) != null) {
            result += inputLine;
        }
        request.getReader().close();
        Map<String, String> map = businessOrderService.getWxPayResultNotify(result);
        return map;

    }


    /**
     *
     * 微信退款
     * @param patient
     * @param orderNo
     * @param refundPrice
     * @param description
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "orderRefund", method = RequestMethod.POST)
    @ApiOperation("微信退款")
    public ObjEnvelop orderRefund(
            @ApiParam(name = "patient", value = "patient", required = true)
            @RequestParam(required = true)String patient,
            @ApiParam(name = "orderNo", value = "orderNo", required = true)
            @RequestParam(required = true)String orderNo,
            @ApiParam(name = "refundPrice", value = "refundPrice", required = true)
            @RequestParam(required = true)String refundPrice,
            @ApiParam(name = "description", value = "description", required = true)
            @RequestParam(required = true)String description) throws Exception {
        try {
            return ObjEnvelop.getSuccess("ok",businessOrderService.orderRefund(wxId,patient,orderNo,Double.parseDouble(refundPrice),description));
        } catch (Exception e) {
            return ObjEnvelop.getError(e.getMessage());
        }
    }



    @RequestMapping(value = "sendDataToYkt", method = RequestMethod.POST)
    @ApiOperation("手动同步订单到眼科通")
    public ObjEnvelop sendDataToYkt(
            @ApiParam(name = "orderNo", value = "orderNo", required = true)
            @RequestParam(required = true)String orderNo) throws Exception {
        try {
            return ObjEnvelop.getSuccess("ok",businessOrderService.sendDataToYkt(orderNo));
        } catch (Exception e) {
            return ObjEnvelop.getError(e.getMessage());
        }
    }


    @RequestMapping(value = "updateStatusToYk", method = RequestMethod.POST)
    @ApiOperation("手动同步订单状态到眼科通")
    public ObjEnvelop updateStatusToYk(
            @ApiParam(name = "orderNo", value = "orderNo", required = true)
            @RequestParam(required = true)String orderNo) throws Exception {
        try {
            return ObjEnvelop.getSuccess("ok",businessOrderService.updateStatusToYk(orderNo));
        } catch (Exception e) {
            return ObjEnvelop.getError(e.getMessage());
        }
    }


    /**
     * 获取微信支付参数
     *
     * @param relationCode 关联code
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "selectOrderByRelationCode", method = RequestMethod.POST)
    @ApiOperation("获取微信支付参数")
    public ObjEnvelop selectOrderByRelationCode(
            @ApiParam(name = "relationCode", value = "relationCode", required = true)
            @RequestParam(required = true)String relationCode,
            @ApiParam(name = "tradeType", value = "tradeType", required = false)
            @RequestParam(required = false)String tradeType,
            @ApiParam(name = "appletCode", value = "appletCode", required = false)
            @RequestParam(required = false)String appletCode) throws Exception {
        try {
            return ObjEnvelop.getSuccess("ok",businessOrderService.selectOrderByRelationCode(relationCode,tradeType,wxId,payFlag,appletCode));
        } catch (Exception e) {
            return ObjEnvelop.getError(e.getMessage());
        }
    }


    /**
     *
     * 获取眼科通数据
     * @param patient
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "registerYkt", method = RequestMethod.POST)
    @ApiOperation("获取眼科通数据")
    public ObjEnvelop registerYkt(
            @ApiParam(name = "patient", value = "patient", required = true)
            @RequestParam(required = true)String patient) throws Exception {
        try {
            ykyyService.registerYkt(patient);
            return ObjEnvelop.getSuccess("ok","");
        } catch (Exception e) {
            return ObjEnvelop.getError(e.getMessage());
        }
    }

    /**
     *
     * 根据电话获取医生信息接口
     * @param phone
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "getDoctorInfoByLoginId", method = RequestMethod.GET)
    @ApiOperation("根据电话获取医生信息接口")
    public ObjEnvelop getDoctorInfoByLoginId(
            @ApiParam(name = "phone", value = "phone", required = true)
            @RequestParam(required = true)String phone) throws Exception {
        try {
            return ObjEnvelop.getSuccess(CommonContant.Message.GET_SUCCESS,ykyyService.GetDoctorInfoByLoginId(phone));
        } catch (Exception e) {
            return ObjEnvelop.getError(e.getMessage());
        }
    }

    /**
     *
     * his同步数据
     * @param code
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "ceshi", method = RequestMethod.POST)
    @ApiOperation("his同步数据")
    public ObjEnvelop findByRealOrder(
            @ApiParam(name = "code", value = "code", required = true)
            @RequestParam(required = true)String code) throws Exception {
        try {
            ykyyEntranceService.findByRealOrder(code,demoFlag);
            return ObjEnvelop.getSuccess("ok","");
        } catch (Exception e) {
            return ObjEnvelop.getError(e.getMessage());
        }
    }
    
    
    @ResponseBody
    @ApiOperation("发送微信模版消息")
    @RequestMapping(value = BaseHospitalRequestMapping.WxPay.sendWxTemple, method = {RequestMethod.GET, RequestMethod.POST})
    public Envelop sendWxTemple(@ApiParam(name = "userName", value = "居民名称")
                                              @RequestParam(value = "userName", required = false,defaultValue = "") String userName,
                                          @ApiParam(name = "senderName", value = "发送人名称")
                                              @RequestParam(value = "senderName", required = false,defaultValue = "") String senderName,
                                          @ApiParam(name = "idCard", value = "身份证")
                                              @RequestParam(value = "idCard", required = false,defaultValue = "") String idCard,
                                          @ApiParam(name = "phone", value = "居民手机")
                                              @RequestParam(value = "phone", required = false,defaultValue = "") String phone,
                                          @ApiParam(name = "title", value = "标题")
                                              @RequestParam(value = "title", required = false,defaultValue = "") String title,
                                          @ApiParam(name = "content", value = "简介")
                                              @RequestParam(value = "content", required = false,defaultValue = "") String content,
                                          @ApiParam(name = "contentString", value = "回复内容")
                                              @RequestParam(value = "contentString", required = false,defaultValue = "") String contentString,
                                          @ApiParam(name = "url", value = "跳转URL")
                                              @RequestParam(value = "url", required = false,defaultValue = "") String url
    )throws Exception  {
        wxTemplateService.sendWxTemple(userName,senderName,idCard,phone,title,content,contentString,url);
        return success("操作成功");
    }
    
    
    @PostMapping(value = "/sendMesToYkt")
    @ApiOperation(value = "眼科通App消息推送接口")
    public Envelop sendMesToYkt(
            @ApiParam(name = "doctor", value = "医生code", required = true)
            @RequestParam(value = "doctor")String doctor,
            @ApiParam(name = "patient", value = "居民code", required = true)
            @RequestParam(value = "patient")String patient,
            @ApiParam(name = "orderType", value = "订单类型(1图文、2电话 3视频)", required = true)
            @RequestParam(value = "orderType")String orderType,
            @ApiParam(name = "enterpriseId", value = "企业号", required = true)
            @RequestParam(value = "enterpriseId")String enterpriseId,
            @ApiParam(name = "content", value = "内容", required = true)
            @RequestParam(value = "content")String content) throws Exception{
        System.out.println("content======"+content);
        content = content.replace("#","%23");
        System.out.println("转换后content"+content);
        /*try {
            if (StringUtils.isNoneBlank(content)){
                String url = content.substring(content.lastIndexOf("(")+1,content.lastIndexOf(")"));
                if (StringUtils.isNoneBlank(url)){
                    String msg = content.substring(0,content.lastIndexOf("["));
                    System.out.println(msg);
                    String sessionId = url.substring(url.lastIndexOf("=")+1);
                    String consult ="";
                    if (sessionId.contains("_")){
                        String[] outPatientId = sessionId.split("_");
                        if (outPatientId!=null&&outPatientId.length>=2){
                            consult = outPatientId[1];
                        }
                    }else {
                        consult = sessionId;
                    }
                    System.out.println(consult);
                    ConsultTeamDo consultTeamDo = consultTeamDao.queryByRelationCode(consult);
                    if (consultTeamDo!=null){
                        if (wxId.equalsIgnoreCase("xm_ykyy_wx")){
                            prescriptionService.wxTempalteJPush("consult_remind_doctor",null,consultTeamDo,msg,"","","");
                        }
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            System.out.println("极光消息发送失败");
        }*/
        //ConsultTeamDo consultModel = consultTeamDao.findByConsult(consult);
        String response = enterpriseService.sendMKMesByDoctor(enterpriseId,doctor,content);
        System.out.println("企业模板消息"+response);
        return success(ykyyEntranceService.sendMesToYkt(doctor,patient,orderType));
    }

    /**
     *
     * 获取患者密码
     * @param mobile
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "getPatientPhone", method = RequestMethod.POST)
    @ApiOperation("获取患者密码")
    public ObjEnvelop getPatientPhone(
            @ApiParam(name = "mobile", value = "mobile", required = true)
            @RequestParam(required = true)String mobile) throws Exception {
        try {
            ykyyService.getPatientPhone(mobile);
            return ObjEnvelop.getSuccess("ok","");
        } catch (Exception e) {
            return ObjEnvelop.getError(e.getMessage());
        }
    }

    /**
     *
     * 获取医生密码
     * @param mobile
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "getDoctorPhone", method = RequestMethod.POST)
    @ApiOperation("获取医生密码")
    public ObjEnvelop getDoctorPhone(
            @ApiParam(name = "mobile", value = "mobile", required = true)
            @RequestParam(required = true)String mobile) throws Exception {
        try {
            ykyyService.getDoctorPhone(mobile);
            return ObjEnvelop.getSuccess("ok","");
        } catch (Exception e) {
            return ObjEnvelop.getError(e.getMessage());
        }
    }
}
