/**
 * 软件著作权：易联众
 * 系统名称：支付平台
 */
package com.yihu.jw.hospital.endpoint.ylz;

import java.io.IOException;
import java.io.InputStream;
import java.net.URLDecoder;
import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.entity.order.BusinessOrderDO;
import com.yihu.jw.order.BusinessOrderService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.StreamUtils;
import org.springframework.web.bind.annotation.RequestMapping;

import com.alibaba.fastjson.JSON;
import com.ylzinfo.onepay.sdk.OnepayDefaultClient;
import com.ylzinfo.onepay.sdk.domain.ResponseParams;
import com.ylzinfo.onepay.sdk.exception.PayException;
import com.ylzinfo.onepay.sdk.utils.StringUtil;

/**
 * 医保移动支付商户端模拟测试
 * 
 * @date 2015年10月21日
 * @author sunzhiwei
 * @version 1.0.0
 * @param <T>
 */
@Controller
public class YlzNotifyController<T> {
	private static final Logger LOGGER = LoggerFactory.getLogger(YlzNotifyController.class);

	@Autowired
	private BusinessOrderService businessOrderService;

	/**
	 * 商户页面跳转(模拟测试)
	 * @param request
	 * @param response
	 * @throws IOException
	 * @throws PayException
	 */
	@RequestMapping(value = "/mmp/returnUrl")
	public void testReturnUrl(HttpServletRequest request, HttpServletResponse response) throws IOException, PayException {
		response.setContentType("text/html;charset=utf-8");
		LOGGER.info("跳转参数：{}", request.getQueryString());
		response.getWriter().write("返回结果:\n" + request.getQueryString() +"\n");

		String onepayUrl = request.getParameter("onepayUrl");
		String appId = "1BHEOI11C00J7B2CA8C0000071FA53E1";
		String appSecret = "1BHEOH8HB0006E0A0A0A00002DB28BC5";
		String signType = "MD5";
		String encryptType = "DES";

		try {
			OnepayDefaultClient client = new OnepayDefaultClient(onepayUrl, appId, appSecret, signType, encryptType);
			Map<String, String> returnParams = getRequestMap(request.getQueryString());
			boolean isVerify = client.verifyReturnSign(returnParams, returnParams.get("sign"));
			LOGGER.info("验签结果：{}", isVerify);
			if (isVerify) {
				response.getWriter().write("签名结果:\nSIGN SUCCESS.\n");
				// TODO 继续处理业务
				//		如：查询显示订单信息
			} else {
				response.getWriter().write("签名结果:\nSIGN ERROR !!!\n");
			}
		} catch (Exception e) {
			response.getWriter().write(e.getMessage());
		}
	}

	/**
	 * 转换URL参数中为Map
	 * @param reqQueryString
	 * @return
	 * @throws PayException
	 */
	private Map<String, String> getRequestMap(String reqQueryString) throws PayException {
		if (StringUtil.isEmpty(reqQueryString)) {
			throw new PayException("跳转参数为空");
		}

		Map<String, String> targetMap = null;

		try {
			reqQueryString = URLDecoder.decode(reqQueryString, "utf-8");
			targetMap = resolveFormContext(reqQueryString, "=", "&");
			LOGGER.info("targetMap=={}", targetMap);
		} catch (Exception e) {
			throw new PayException("跳转参数处理异常," + reqQueryString);
		}
		return targetMap;
	}

	/**
	 * 解析URL参数串
	 * @param formContext
	 * @param wordFirstsplitRegex
	 * @param wordSecondsplitRegex
	 * @return
	 */
	public static Map<String, String> resolveFormContext(String formContext, String wordFirstsplitRegex, String wordSecondsplitRegex) {
		if (StringUtil.isEmpty(formContext)) {
			return null;
		}
		Map<String, String> targetMap = new HashMap<String, String>();
		String[] wordSeconds = formContext.split(wordSecondsplitRegex);
		for (String wordSecond : wordSeconds) {
			if (StringUtil.isEmpty(wordSecond)) {
				continue;
			}

			int idx = wordSecond.indexOf(wordFirstsplitRegex);
			targetMap.put(wordSecond.substring(0, idx), wordSecond.substring(idx + 1, wordSecond.length()));
		}
		return targetMap;
	}
	
	/**
	 * 商户回调通知(模拟测试)
	 * @param request
	 * @param response
	 * @throws IOException
	 * @throws PayException
	 */
	@RequestMapping(value = "/mmp/receiveNotify")
	public void mmpReceiveNotify(HttpServletRequest request, HttpServletResponse response) throws IOException, PayException {
		response.setContentType("text/html;charset=utf-8");
		String appId = "1BHEOI11C00J7B2CA8C0000071FA53E1";
		String appSecret = "1BHEOH8HB0006E0A0A0A00002DB28BC5";
		InputStream inputStream = request.getInputStream();
		String params = StreamUtils.copyToString(inputStream, Charset.forName("UTF-8"));
		LOGGER.info("回调参数：{}", params);
		if (StringUtil.isEmpty(params)) {
			response.getWriter().write("empty");
			return;
		}
		// 参数转换
		ResponseParams<?> encryptRes = JSON.parseObject(params, ResponseParams.class);
		OnepayDefaultClient client = new OnepayDefaultClient("", appId, appSecret, encryptRes.getSignType(), encryptRes.getEncryptType());

		try {
			LOGGER.info("解密前：{}", JSON.toJSONString(encryptRes));
			ResponseParams<?> decryptRes = client.decryptNotifyResponse(encryptRes);
			LOGGER.info("解密后：{}", JSON.toJSONString(decryptRes));
			String param =JSON.toJSONString(decryptRes);
			boolean isDecrypt = ("-1".equals(decryptRes.getRespCode()) ? false : true); // 是否解密失败
			if (!isDecrypt) {
				response.getWriter().write("DECRYPT FAILURE");
			} else {
				boolean isVerify = client.verifyResponseSign(decryptRes);
				LOGGER.info("验签结果：{}", isVerify);
				if (!isVerify) {
					response.getWriter().write("FAILURE");
				} else {
					JSONObject object = JSONObject.parseObject(param);
					JSONObject jsonObject = object.getJSONObject("param");
					LOGGER.info("object"+object.toJSONString()+"jsonObject"+jsonObject.toJSONString());
					String orderNo = jsonObject.getString("outChargeNo");
					businessOrderService.updatePayStatus(orderNo);
					response.getWriter().write("SUCCESS");
				}
			}
		} catch (Exception e) {
			response.getWriter().write(e.getMessage());
		}
	}
}
