package com.yihu.jw.hospital.service.SystemMessage;

import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.hospital.message.SystemMessageDO;
import com.yihu.jw.entity.hospital.prescription.WlyyOutpatientDO;
import com.yihu.jw.hospital.message.service.SystemMessageService;
import com.yihu.jw.hospital.prescription.service.PrescriptionService;
import com.yihu.jw.im.service.ImService;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.util.idcard.IdCardUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;


/**
 * 互联网医院系统消息业务层
 * @author huangwenjie
 * @date 2019/11/22 13:53
 */
@Service
public class HospitalSystemMessageService  {

	private static Logger logger = LoggerFactory.getLogger(HospitalSystemMessageService.class);


	@Autowired
	private SystemMessageService systemMessageService;
	
	@Autowired
	private BasePatientDao basePatientDao;
	
	@Autowired
	private ImService imService;
	
	@Autowired
	private PrescriptionService prescriptionService;
	
	/**
	 * 保存系统消息&发送IM外层刷新事件的消息
	 * @param systemMessageDO
	 * @throws Exception
	 */
	public void saveMessage(SystemMessageDO systemMessageDO) throws Exception{
		//发送系统消息
		systemMessageService.saveMessage(systemMessageDO);
		//发型IM外层消息
		
		JSONObject object = new JSONObject();
		object.put("socket_sms_type",11);//系统消息刷新事件
		object.put("relation_code",systemMessageDO.getType());
		imService.sendWaiSocketMessage(systemMessageDO.getSender(),systemMessageDO.getReceiver(),object.toString(),"1");
	}

	/**
	 * 发送IM消息推送
	 * @param systemMessageDO
	 * @throws Exception
     */
	public void sendImMessage(SystemMessageDO systemMessageDO) throws Exception{
		try{
			//发型IM外层消息
			JSONObject object = new JSONObject();
			object.put("socket_sms_type",11);//系统消息刷新事件
			object.put("relation_code",systemMessageDO.getType());
			if (StringUtils.isNoneBlank(systemMessageDO.getType())&&"3".equalsIgnoreCase(systemMessageDO.getType())){
				object.put("outpatientId",systemMessageDO.getRelationCode());
			}
			imService.sendWaiSocketMessage(systemMessageDO.getSender(),systemMessageDO.getReceiver(),object.toString(),"1");
		}catch (Exception e){
			logger.error(e.toString());
		}

	}
	
	/**
	 * 通知医生抢单消息
	 * @param wlyyOutpatientDO
	 */
	@Async
	public void sendImPichCheckMessage(WlyyOutpatientDO wlyyOutpatientDO) throws Exception {
		BasePatientDO basePatientDO =basePatientDao.findById(wlyyOutpatientDO.getPatient()).orElse(null);
		String tagMsg = "";
		JSONObject object = new JSONObject();
		object.put("socket_sms_type",12);//抢单事件
		if("1".equals(wlyyOutpatientDO.getOutpatientType())&&"1".equals(wlyyOutpatientDO.getType())){//图文复诊
			object.put("relation_code","1:"+wlyyOutpatientDO.getId());
			tagMsg = "图文复诊";
		}else if ("1".equals(wlyyOutpatientDO.getOutpatientType())&&"2".equals(wlyyOutpatientDO.getType())){//视频复诊
			object.put("relation_code","3:"+wlyyOutpatientDO.getId());
			tagMsg = "视频复诊";
		}else if ("2".equals(wlyyOutpatientDO.getOutpatientType())){//协同门诊
			object.put("relation_code","3:"+wlyyOutpatientDO.getId());
			tagMsg = "协同门诊";
		}else{}
		
		object.put("msg",basePatientDO.getName()+"("+ IdCardUtil.getSexForIdcardReturnName(basePatientDO.getIdcard())+" "
				+ IdCardUtil.getAgeForIdcard(basePatientDO.getIdcard())+"岁)发起了随机邀请"+tagMsg+",问题："+wlyyOutpatientDO.getDescription());
		
//		List<Map<String,Object>> doctorlist = prescriptionService.findDoctorByHospitalAndDept();
		
//		imService.sendWaiSocketMessage(wlyyOutpatientDO.getPatient(),wlyyOutpatientDO.getDoctor(),object.toString(),"1");
	}

}
