package com.yihu.jw.hospital.service.consult;

import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.org.BaseOrgDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.base.wx.BasePatientWechatDo;
import com.yihu.jw.file_upload.FileUploadService;
import com.yihu.jw.hospital.prescription.service.entrance.EntranceService;
import com.yihu.jw.hospital.prescription.service.entrance.XzzxEntranceService;
import com.yihu.jw.org.dao.BaseOrgDao;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.restmodel.iot.common.UploadVO;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.util.http.HttpClientUtil;
import com.yihu.jw.util.wechat.wxhttp.HttpUtil;
import com.yihu.jw.wechat.dao.BasePatientWechatDao;
import com.yihu.jw.wechat.dao.WechatDao;
import com.yihu.jw.wechat.service.WechatInfoService;
import com.yihu.jw.wechat.service.WxAccessTokenService;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONObject;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.transaction.Transactional;
import java.io.InputStream;
import java.io.PrintWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.util.List;
import java.util.Map;
@Service
public class QrcodeService {
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(QrcodeService.class);
    @Autowired
    private WechatInfoService wechatInfoService;
    @Value("${wechat.id}")
    private String wxId;
    @Value("${fastDFS.fastdfs_file_url}")
    private String fastdfs_file_url;
    @Autowired
    private BaseDoctorDao baseDoctorDao;
    @Autowired
    private BasePatientDao basePatientDao;
    @Autowired
    private BasePatientWechatDao basePatientWechatDao;
    @Autowired
    private FileUploadService fileUploadService;
    @Value("${testPattern.sign}")
    private String isClose;
    @Value("${testPattern.remote_inner_url}")
    private String remote_inner_url;
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private WechatDao wechatDao;

    @Autowired
    private WxAccessTokenService wxAccessTokenService;
    @Autowired
    private EntranceService entranceService;
    @Autowired
    private XzzxEntranceService xzzxEntranceService;
    @Autowired
    private HttpClientUtil httpClientUtil;
    @Autowired
    private BaseOrgDao baseOrgDao;
    /**
     * 生成所有医生的二维码
     *
     * @return
     * @throws Exception
     */
    @Transactional
    public void makeAllSpecialistQrcode() throws Exception {
        String token=null;
        //中山医院互联网医院
        if("xm_zsyy_wx".equals(wxId)) {
            String response = entranceService.getAccessToken("100033");
            net.sf.json.JSONObject object1 = net.sf.json.JSONObject.fromObject(response);
            if (null != object1&&null!=object1.get("respCode") && "00".equals(object1.get("respCode"))) {
                token  = object1.getJSONObject("respEntity").getString("access_token");
            } else {
                throw new Exception("获取ACCESSTOKEN失败");
            }

        }else if("xm_xzzx_wx".equals(wxId)){
            token = xzzxEntranceService.getXzToken();
        }else{
            token = wxAccessTokenService.getWxAccessTokenById(wxId).getAccessToken();
        }
        System.out.println("token"+token);
        System.out.println("wxId"+wxId);
        Iterable<BaseDoctorDO> doctors = baseDoctorDao.findByDel();
        for (BaseDoctorDO doctorDO :doctors){
            if (null==doctorDO.getQrcode()||"".equals(doctorDO.getQrcode())){
                // 二维码内容
                String content = "doctor_" + doctorDO.getId() +"_"+doctorDO.getName();
                System.out.println("content"+content);
                // 二维码图片文件名
                String fileName = doctorDO.getId()+"_"+doctorDO.getMobile()+".png";
                /*String path = QrcodeService.class.getResource("/").getPath().replace("/WEB-INF/classes/", "")
                        + File.separator + "qrcode";*/
                System.out.println("fileName"+fileName);
                // 通过微信接口生成医生二维码
                String uri = makeQrcodeFromWeiXin(content, fileName.replaceAll("\r\n", ""), token);
                System.out.println("uri"+uri);
                doctorDO.setQrcode(uri);
                baseDoctorDao.save(doctorDO);
            }
        }
    }
    /**
     * 生成某个医生的二维码
     *
     * @param doctor
     * @return
     * @throws Exception
     */
    public String makeSpecialistQrcode(String doctor) throws Exception {
        System.out.println("doctorID"+doctor);

        String token=null;
        //中山医院互联网医院
        if("xm_zsyy_wx".equals(wxId)) {
            String response = entranceService.getAccessToken("100033");
            net.sf.json.JSONObject object1 = net.sf.json.JSONObject.fromObject(response);
            if (null != object1&&null!=object1.get("respCode") && "00".equals(object1.get("respCode"))) {
                token  = object1.getJSONObject("respEntity").getString("access_token");
            } else {
                throw new Exception("获取ACCESSTOKEN失败");
            }

        }else if("xm_xzzx_wx".equals(wxId)){
            token = xzzxEntranceService.getXzToken();
        }else{
            token = wxAccessTokenService.getWxAccessTokenById(wxId).getAccessToken();
        }
        System.out.println("token"+token);
        System.out.println("wxId"+wxId);
        BaseDoctorDO baseDoctorDO = baseDoctorDao.findByIdAndDel(doctor);
        if (null!=baseDoctorDO){
            if (null==baseDoctorDO.getQrcode()||"".equals(baseDoctorDO.getQrcode())){
                // 二维码内容
                String content = "doctor_" + baseDoctorDO.getId() +"_"+baseDoctorDO.getName();
                System.out.println("content"+content);
                // 二维码图片文件名
                String fileName = baseDoctorDO.getId()+".png";
                /*String path = QrcodeService.class.getResource("/").getPath().replace("/WEB-INF/classes/", "")
                        + File.separator + "qrcode";*/
                System.out.println("fileName"+fileName);
                // 通过微信接口生成医生二维码
                String uri = makeQrcodeFromWeiXin(content, fileName, token);
                System.out.println("uri"+uri);
                baseDoctorDO.setQrcode(uri);
                baseDoctorDao.save(baseDoctorDO);
                return uri;
            }else {
                return baseDoctorDO.getQrcode();
            }
        } else {
            throw new Exception("找不到对应医生信息!");
        }
    }

    /**
     * 生成某个医生的小程序二维码
     *
     * @param doctor
     * @return
     * @throws Exception
     */
    public String makeSpecialistXcxQrcode(String doctor) throws Exception {
        System.out.println("doctorID"+doctor);
        String token = wxAccessTokenService.getWxAppletsAccessTokenById(wxId).getAccessToken();
        System.out.println("token"+token);
        System.out.println("wxId"+wxId);
        if (StringUtils.isNoneBlank(doctor)){
            BaseDoctorDO baseDoctorDO = baseDoctorDao.findByIdAndDel(doctor);
            if (null!=baseDoctorDO){
                if (null==baseDoctorDO.getAppletQrCode()||"".equals(baseDoctorDO.getAppletQrCode())){
                    // 二维码图片文件名
                    String fileName = baseDoctorDO.getId()+".png";
                    System.out.println("fileName"+fileName);
                    // 通过微信接口生成医生二维码
                    String page = "pages/patient/index";
                    String scene = "_t=doc&_c="+baseDoctorDO.getIdcard()+"";
                    String uri = makeXcxQrcodeFromWx(fileName,page,scene,token);
                    System.out.println("uri"+uri);
                    baseDoctorDO.setAppletQrCode(uri);
                    baseDoctorDao.save(baseDoctorDO);
                    return uri;
                }else {
                    return baseDoctorDO.getAppletQrCode();
                }
            } else {
                throw new Exception("找不到对应医生信息!");
            }
        }else {
            Iterable<BaseDoctorDO> doctors = baseDoctorDao.findByDel();
            for (BaseDoctorDO doctorDO : doctors) {
                if (null!=doctorDO){
                    if (null==doctorDO.getAppletQrCode()||"".equals(doctorDO.getAppletQrCode())){

                        // 二维码图片文件名
                        String fileName = doctorDO.getId()+".png";
                /*String path = QrcodeService.class.getResource("/").getPath().replace("/WEB-INF/classes/", "")
                        + File.separator + "qrcode";*/
                        System.out.println("fileName"+fileName);
                        // 通过微信接口生成医生二维码
                        String page = "pages/patient/index";
                        String scene = "_t=doc&_c="+doctorDO.getIdcard()+"";
                        String uri = makeXcxQrcodeFromWx(fileName,page,scene,token);
                        System.out.println("uri"+uri);
                        doctorDO.setAppletQrCode(uri);
                        baseDoctorDao.save(doctorDO);
                        return uri;
                    }else {
                        return doctorDO.getAppletQrCode();
                    }
                } else {
                    throw new Exception("找不到对应医生信息!");
                }
            }
        }
        return null;
    }


    public String makeUrlQrcode(String flag) throws Exception {
        System.out.println("flag"+flag);

        String token=null;
        //中山医院互联网医院
        if("xm_zsyy_wx".equals(wxId)) {
            String response = entranceService.getAccessToken("100033");
            net.sf.json.JSONObject object1 = net.sf.json.JSONObject.fromObject(response);
            if (null != object1&&null!=object1.get("respCode") && "00".equals(object1.get("respCode"))) {
                token  = object1.getJSONObject("respEntity").getString("access_token");
            } else {
                throw new Exception("获取ACCESSTOKEN失败");
            }

        }else if("xm_xzzx_wx".equals(wxId)){
            token = xzzxEntranceService.getXzToken();
        }else{
            token = wxAccessTokenService.getWxAccessTokenById(wxId).getAccessToken();
        }
        System.out.println("token"+token);
        System.out.println("wxId"+wxId);

        // 二维码内容
        String content = "flag_" + flag ;
        System.out.println("content"+content);
        // 二维码图片文件名
        String fileName = flag+".png";
        /*String path = QrcodeService.class.getResource("/").getPath().replace("/WEB-INF/classes/", "")
                + File.separator + "qrcode";*/
        System.out.println("fileName"+fileName);
        // 通过微信接口生成医生二维码
        String uri = makeQrcodeFromWeiXin(content, fileName, token);
        System.out.println("uri"+uri);

        return uri;
    }
    /**
     * 从微信生成二维码并下载到本地
     *
     * @param content  场景值
     * @param fileName 二维码文件名
     * @param token
     * @throws
     */
    public String makeQrcodeFromWeiXin(String content, String fileName, String token)  {
        try {
            String token_url = "https://api.weixin.qq.com/cgi-bin/qrcode/create?access_token=" + token;
            String params = "{\"action_name\": \"QR_LIMIT_STR_SCENE\", \"action_info\": {\"scene\": {\"scene_str\": \"" + content + "\"}}}";
            LOGGER.info("参数:"+params);

            String result = HttpUtil.sendPost(token_url, params);
            LOGGER.info("步骤1:"+result);

            if (!StringUtils.isEmpty(result)) {
                JSONObject json = new JSONObject(result);
                LOGGER.info("步骤2:"+json.toString());
                if (json.has("ticket")) {
                    // 请求输入流
                    InputStream inputStream = null;
                    // 下载二维码图片
                    URL urlGet = new URL("https://mp.weixin.qq.com/cgi-bin/showqrcode?ticket="
                            + URLEncoder.encode(json.get("ticket").toString(), "UTF-8"));
                    HttpURLConnection connection = (HttpURLConnection) urlGet.openConnection();
                    connection.connect();
                    inputStream = connection.getInputStream();
                    UploadVO uploadVO = new UploadVO();
                    MultipartFile multipartFile = new MockMultipartFile(fileName,fileName,"", inputStream);
                    if (isClose.equalsIgnoreCase("1")){
                        System.out.println("isClose"+1);
                        Map<String, Object> map = fileUploadService.uploadImg(multipartFile);
                        uploadVO.setFullUri(map.get("accessory").toString());
                    }else if(isClose.equals("2")){
                        System.out.println("isClose"+2);
                        String originalFilename = multipartFile.getOriginalFilename();
                        String[] fs = originalFilename.split("\\.");
                        String type = fs[1];
                        //内网上传
                        String rs = fileUploadService.request(remote_inner_url,multipartFile,type);
                        System.out.println("rs"+rs);
                        com.alibaba.fastjson.JSONObject rsJson = JSON.parseObject(rs);
                        uploadVO = objectMapper.readValue(rsJson.getJSONObject("obj").toJSONString(),UploadVO.class);
                        inputStream.close();

                    }else {
                        // 上传服务器
                        uploadVO = fileUploadService.uploadStream(multipartFile.getInputStream(),fileName,fastdfs_file_url);
                    }

                    return uploadVO.getFullUri();
                } else {
                    System.out.println("请求微信生成二维码失败!");
                }
            } else {
                System.out.println("请求微信生成二维码失败!");
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println(e.getMessage());
        }
        return null;
    }



    /**
     * 从微信生成小程序二维码并下载到本地
     * @param fileName
     * @param page
     * @param scene
     * @param accessToken
     * @return
     */
     public String makeXcxQrcodeFromWx(String fileName, String page, String scene, String accessToken) {
           try {
           /*    String token_url = "https://api.weixin.qq.com/wxa/getwxacodeunlimit?access_token=" + accessToken;
               String params = "{\n" +
                       " \"page\": \""+page+"\",\n" +
                       " \"scene\": \""+scene+"\",\n" +
                       " \"check_path\": true,\n" +
                       " \"env_version\": \"release\"\n" +
                       "}";
               LOGGER.info("参数:"+params);

               String result = HttpUtil.sendPost(token_url, params);
               LOGGER.info("步骤1:"+result);*/
               //调用微信接口生成二维码
               URL url = new URL("https://api.weixin.qq.com/wxa/getwxacodeunlimit?access_token=" + accessToken);
               HttpURLConnection httpURLConnection = (HttpURLConnection) url.openConnection();
               httpURLConnection.setRequestMethod("POST");// 提交模式
               // conn.setConnectTimeout(10000);//连接超时 单位毫秒
               // conn.setReadTimeout(2000);//读取超时 单位毫秒
               // 发送POST请求必须设置如下两行
               httpURLConnection.setDoOutput(true);
               httpURLConnection.setDoInput(true);
               // 获取URLConnection对象对应的输出流
               PrintWriter printWriter = new PrintWriter(httpURLConnection.getOutputStream());
               // 发送请求参数
               JSONObject paramJson = new JSONObject();
               //这就是你二维码里携带的参数 String型  名称不可变
               paramJson.put("scene", scene);
               //注意该接口传入的是page而不是path
               paramJson.put("page", page);
               paramJson.put("width", 430);
               //这是设置扫描二维码后跳转的页面
           /*    paramJson.put("width", 430);*/
              /* paramJson.put("is_hyaline", false);*/
            /*   paramJson.put("auto_color", true);*/
               printWriter.write(paramJson.toString());
               // flush输出流的缓冲
               printWriter.flush();
               /*BufferedInputStream bis = new BufferedInputStream(httpURLConnection.getInputStream());
               OutputStream os = new FileOutputStream(new File("C:/Users/wangzhinan/Desktop/1.png"));
               int len;
               byte[] arr = new byte[1024];
               while ((len = bis.read(arr)) != -1) {
                    os.write(arr, 0, len);
                    os.flush();
               }
               os.close();*/
               //开始获取数据
               InputStream inputStream = httpURLConnection.getInputStream();
               UploadVO uploadVO = new UploadVO();
               MultipartFile multipartFile = new MockMultipartFile(fileName,fileName,"", inputStream);
               if (isClose.equalsIgnoreCase("1")){
                   System.out.println("isClose"+1);
                   Map<String, Object> map = fileUploadService.uploadImg(multipartFile);
                   uploadVO.setFullUri(map.get("accessory").toString());
               }else if(isClose.equals("2")){
                   System.out.println("isClose"+2);
                   String originalFilename = multipartFile.getOriginalFilename();
                   String[] fs = originalFilename.split("\\.");
                   String type = fs[1];
                   //内网上传
                   String rs = fileUploadService.request(remote_inner_url,multipartFile,type);
                   System.out.println("rs"+rs);
                   com.alibaba.fastjson.JSONObject rsJson = JSON.parseObject(rs);
                   uploadVO = objectMapper.readValue(rsJson.getJSONObject("obj").toJSONString(),UploadVO.class);
                   inputStream.close();

               }else {
                   // 上传服务器
                   uploadVO = fileUploadService.uploadStream(multipartFile.getInputStream(),fileName,fastdfs_file_url);
               }
               return uploadVO.getFullUri();
           }catch (Exception e){
               e.printStackTrace();
           }
           return null;
     }


    public ObjEnvelop getWXUserInfo(String patient) throws Exception{
        BasePatientDO basePatientDO = basePatientDao.findById(patient).orElse(null);
        ObjEnvelop objEnvelop = new ObjEnvelop();
        if (null!=basePatientDO){
            String openId=basePatientDO.getOpenid();
            if (StringUtils.isBlank(openId)){
                List<BasePatientWechatDo> basePatientWechatDos = basePatientWechatDao.findByWechatIdAndPatientId(wxId,patient);
                if (null!=basePatientWechatDos&&basePatientWechatDos.size()>0){
                    openId = basePatientWechatDos.get(0).getOpenid();
                }
            }
            String token=null;
            //中山医院互联网医院
            if("xm_zsyy_wx".equals(wxId)) {
                String response = entranceService.getAccessToken("100033");
                net.sf.json.JSONObject object1 = net.sf.json.JSONObject.fromObject(response);
                if (null != object1&&null!=object1.get("respCode") && "00".equals(object1.get("respCode"))) {
                    token  = object1.getJSONObject("respEntity").getString("access_token");
                } else {
                    throw new Exception("获取ACCESSTOKEN失败");
                }

            }else if("xm_xzzx_wx".equals(wxId)){
                objEnvelop.setStatus(200);
                com.alibaba.fastjson.JSONObject jsonObject = new com.alibaba.fastjson.JSONObject();
                jsonObject.put("nickname","nickname");
                objEnvelop.setMessage(jsonObject.toString());
                return objEnvelop;
            }else{
                token = wxAccessTokenService.getWxAccessTokenById(wxId).getAccessToken();
            }
            System.out.println("token"+token);
            System.out.println("wxId"+wxId);
            String url = "https://api.weixin.qq.com/cgi-bin/user/info?access_token="+token+"&openid="+openId+"&lang=zh_CN";
            String res = httpClientUtil.get(url,"utf-8");
            String userName = "";
            if (StringUtils.isNoneBlank(res)){
                com.alibaba.fastjson.JSONObject jsonObject = com.alibaba.fastjson.JSONObject.parseObject(res);
                if (null!=jsonObject.get("errcode")){
                    objEnvelop.setMessage(jsonObject.getString("errmsg"));
                    objEnvelop.setStatus(Integer.valueOf(jsonObject.getString("errcode")));
                }else {
                    userName = jsonObject.getString("nickname");
                    objEnvelop.setStatus(200);
                    objEnvelop.setMessage(jsonObject.toString());
                }
                LOGGER.info("usernme:"+userName);
            }

            System.out.println(res);
            return objEnvelop;
        }else {
            objEnvelop.setMessage("无用户信息");
            objEnvelop.setStatus(-1);
        }
        return objEnvelop;
    }
    //生成公众号二维码
    public String createPublicQrcode(String fileName) throws Exception{
        ObjEnvelop objEnvelop = new ObjEnvelop();
        String token=null;
        //中山医院互联网医院
        if("xm_zsyy_wx".equals(wxId)) {
            String response = entranceService.getAccessToken("100033");
            net.sf.json.JSONObject object1 = net.sf.json.JSONObject.fromObject(response);
            if (null != object1&&null!=object1.get("respCode") && "00".equals(object1.get("respCode"))) {
                token  = object1.getJSONObject("respEntity").getString("access_token");
            } else {
                throw new Exception("获取ACCESSTOKEN失败");
            }

        }else if("xm_xzzx_wx".equals(wxId)){
            token = xzzxEntranceService.getXzToken();
        }else{
            token = wxAccessTokenService.getWxAccessTokenById(wxId).getAccessToken();
        }
        System.out.println("token"+token);
        System.out.println("wxId"+wxId);
        String token_url = "https://api.weixin.qq.com/cgi-bin/qrcode/create?access_token=" + token;
        String params = "{\"action_name\": \"QR_LIMIT_STR_SCENE\", \"action_info\": {\"scene\": {\"scene_str\": \"1\"}}}";
        LOGGER.info("参数:"+params);
        String res = HttpUtil.sendPost(token_url, params);
        if (!StringUtils.isEmpty(res)) {
            JSONObject json = new JSONObject(res);
            LOGGER.info("步骤2:"+json.toString());
            if (json.has("ticket")) {
                // 请求输入流
                InputStream inputStream = null;
                // 下载二维码图片
                URL urlGet = new URL("https://mp.weixin.qq.com/cgi-bin/showqrcode?ticket="
                        + URLEncoder.encode(json.get("ticket").toString(), "UTF-8"));
                HttpURLConnection connection = (HttpURLConnection) urlGet.openConnection();
                connection.connect();
                inputStream = connection.getInputStream();
                UploadVO uploadVO = new UploadVO();
                MultipartFile multipartFile = new MockMultipartFile(fileName,fileName,"", inputStream);
                if (isClose.equalsIgnoreCase("1")){
                    System.out.println("isClose"+1);
                    Map<String, Object> map = fileUploadService.uploadImg(multipartFile);
                    uploadVO.setFullUri(map.get("accessory").toString());
                }else if(isClose.equals("2")){
                    System.out.println("isClose"+2);
                    String originalFilename = multipartFile.getOriginalFilename();
                    String[] fs = originalFilename.split("\\.");
                    String type = fs[1];
                    //内网上传
                    String rs = fileUploadService.request(remote_inner_url,multipartFile,type);
                    System.out.println("rs"+rs);
                    com.alibaba.fastjson.JSONObject rsJson = JSON.parseObject(rs);
                    uploadVO = objectMapper.readValue(rsJson.getJSONObject("obj").toJSONString(),UploadVO.class);
                    inputStream.close();

                }else {
                    // 上传服务器
                    uploadVO = fileUploadService.uploadStream(multipartFile.getInputStream(),fileName,fastdfs_file_url);
                }

                return uploadVO.getFullUri();
            } else {
                System.out.println("请求微信生成二维码失败!");
                return "";
            }
    }else {
            return "";
        }
    }

    /**
     * 从微信生成小程序二维码并下载到本地
     *
     * @param page  小程序地址
     * @param scene 小程序参数
     * @param token
     * @throws
     */
    public String makeQrcodeFromXiaochengxu(String page, String scene,String fileName, String token)  {
        try {
            String token_url = "https://api.weixin.qq.com/wxa/getwxacodeunlimit?access_token=" + token;
            com.alibaba.fastjson.JSONObject object = new com.alibaba.fastjson.JSONObject();
            object.put("page",page);
            object.put("scene",scene);
            object.put("check_path",true);
            object.put("env_version","release");
            LOGGER.info("参数:"+object.toString());

            String result = HttpUtil.sendPost(token_url, object.toJSONString());
            LOGGER.info("步骤1:"+result);

            if (!StringUtils.isEmpty(result)) {
                JSONObject json = new JSONObject(result);
                LOGGER.info("步骤2:"+json.toString());
                if (json.getString("errcode").equalsIgnoreCase("0")) {
               /*     String strBuffer = json.getString("buffer");
                    StringBuffer sb = new StringBuffer(strBuffer);
                    InputStream in=ByteArrayInputStream(s_tmp.getBytes());*/
                    // 请求输入流
                    InputStream inputStream = null;
                    // 下载二维码图片
                    URL urlGet = new URL("https://mp.weixin.qq.com/cgi-bin/showqrcode?ticket="
                            + URLEncoder.encode(json.get("ticket").toString(), "UTF-8"));
                    HttpURLConnection connection = (HttpURLConnection) urlGet.openConnection();
                    connection.connect();
                    inputStream = connection.getInputStream();
                    UploadVO uploadVO = new UploadVO();
                    MultipartFile multipartFile = new MockMultipartFile(fileName,fileName,"", inputStream);
                    if (isClose.equalsIgnoreCase("1")){
                        System.out.println("isClose"+1);
                        Map<String, Object> map = fileUploadService.uploadImg(multipartFile);
                        uploadVO.setFullUri(map.get("accessory").toString());
                    }else if(isClose.equals("2")){
                        System.out.println("isClose"+2);
                        String originalFilename = multipartFile.getOriginalFilename();
                        String[] fs = originalFilename.split("\\.");
                        String type = fs[1];
                        //内网上传
                        String rs = fileUploadService.request(remote_inner_url,multipartFile,type);
                        System.out.println("rs"+rs);
                        com.alibaba.fastjson.JSONObject rsJson = JSON.parseObject(rs);
                        uploadVO = objectMapper.readValue(rsJson.getJSONObject("obj").toJSONString(),UploadVO.class);
                        inputStream.close();

                    }else {
                        // 上传服务器
                        uploadVO = fileUploadService.uploadStream(multipartFile.getInputStream(),fileName,fastdfs_file_url);
                    }

                    return uploadVO.getFullUri();
                } else {
                    System.out.println("请求微信生成二维码失败!");
                }
            } else {
                System.out.println("请求微信生成二维码失败!");
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println(e.getMessage());
        }
        return null;
    }
    /*public static void main(String[] args) {
        String ss = "{\\\"subscribe\\\":1,\\\"openid\\\":\\\"okVzxwiQsS9avEsD0THVBinDcPhs\\\",\\\"nickname\\\":\\\"July\\\",\\\"sex\\\":1,\\\"language\\\":\\\"zh_CN\\\",\\\"city\\\":\\\"\\\",\\\"province\\\":\\\"\\\",\\\"country\\\":\\\"圣基茨和尼维斯\\\",\\\"headimgurl\\\":\\\"http:\\\\/\\\\/thirdwx.qlogo.cn\\\\/mmopen\\\\/PdibpV1sFDHfmNUvXBr0ia2hPlJ1X8ShEHiaXFgtILic6OibZC4DFibxK8AjPnPByicn1s1LFE08qnBo0bpnWORNm0rOribZFfnD6Kx7\\\\/132\\\",\\\"subscribe_time\\\":1596677396,\\\"remark\\\":\\\"\\\",\\\"groupid\\\":0,\\\"tagid_list\\\":[],\\\"subscribe_scene\\\":\\\"ADD_SCENE_PROFILE_CARD\\\",\\\"qr_scene\\\":0,\\\"qr_scene_str\\\":\\\"\\\"}";
        ss=ss.replace("\\","");
        com.alibaba.fastjson.JSONObject jsonObject = com.alibaba.fastjson.JSONObject.parseObject(ss);
        System.out.println(jsonObject.toString());
    }*/
    public MixEnvelop getWXGZHQrcode(String orgCode){
        BaseOrgDO baseOrgDO = baseOrgDao.findByCode(orgCode);
        MixEnvelop mixEnvelop= new MixEnvelop();
        if (null!=baseOrgDO){
            String qrCode = baseOrgDO.getQrcode();
            if (StringUtils.isBlank(qrCode)){
                try {
                    qrCode  = createPublicQrcode( "hlwyy.png");
                    baseOrgDO.setQrcode(qrCode);
                    baseOrgDao.save(baseOrgDO);
                } catch (Exception e) {
                    e.printStackTrace();
                    mixEnvelop.setMessage("生成公众号二维码失败");
                    mixEnvelop.setStatus(-1);
                    return mixEnvelop;
                }
            }
            mixEnvelop.setMessage("success");
            mixEnvelop.setObj(qrCode);
            mixEnvelop.setStatus(200);
            return mixEnvelop;
        }else {
            mixEnvelop.setMessage("获取公众号二维码失败");
            mixEnvelop.setStatus(-1);
            return mixEnvelop;
        }
    }
}
