package com.yihu.jw.hospital.service.consult;

import com.yihu.jw.entity.hospital.consult.WlyyHospitalSysDictDO;
import com.yihu.jw.hospital.dao.consult.SysDictDao;
import com.yihu.jw.utils.hibernate.HibenateUtils;
import net.sf.json.JSONObject;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

@Service
public class SysDictService {

    @Autowired
    private SysDictDao sysDictDao;
    @Autowired
    private HibenateUtils hibenateUtils;

    /**
     * 新增&更新社区医生邀请专家配置
     * @param jsonObject
     * @return
     */
    public WlyyHospitalSysDictDO saveConsultSetting(JSONObject jsonObject,String hospital,String dictName ){

        WlyyHospitalSysDictDO wlyyHospitalSysDictDO = new WlyyHospitalSysDictDO();
        //JSON数据解析
        Map dataDetail  = JSONObject.fromObject(jsonObject);
        String dictCode = dataDetail.get("dictCode") == "" ? "": dataDetail.get("dictCode") .toString();
        String dictValue = dataDetail.get("dictValue") == "" ? "": dataDetail.get("dictValue") .toString();
        String id = dataDetail.get("id") == "" ? "": dataDetail.get("id") .toString();
        String saasId = "";

        if("".equals(id)){
            wlyyHospitalSysDictDO.setSaasId(saasId);
            wlyyHospitalSysDictDO.setHospital(hospital);
            wlyyHospitalSysDictDO.setDictName(dictName);
            wlyyHospitalSysDictDO.setDictCode(dictCode);
            wlyyHospitalSysDictDO.setDictValue(dictValue);
            wlyyHospitalSysDictDO = sysDictDao.save(wlyyHospitalSysDictDO);
        }else{
            WlyyHospitalSysDictDO wlyyHospitalSysDictDOL = sysDictDao.findById(id).orElse(null);
            wlyyHospitalSysDictDOL.setDictValue(dictValue);
            wlyyHospitalSysDictDO = sysDictDao.save(wlyyHospitalSysDictDOL);
        }

        return wlyyHospitalSysDictDO;
    }

    /**
     * 根据医院编号及字典编码，获取配置界面中某一配置项的值信息
     * dictName( 候诊延时： WaitingDelay )  dictCode ( 等待时长：WaitingTime  过号延时保号： OverdueDelay   过号延时时长： OverdueTime  延时退费规则: DelayedRefundRule )
     * dictName( 邀请专家配置： InvitedConfig )   dictCode ( 支付配置：PayType  邀请确认：InvitedValid   退费规则：RefundRule )
     * @param hospital
     * @param dictCode
     * @return
     */
    public WlyyHospitalSysDictDO findByHospitalAndDictCode(String hospital, String dictCode){
        WlyyHospitalSysDictDO wlyyHospitalSysDictDO = sysDictDao.findByHospitalAndDictCode(hospital,dictCode);
        if(wlyyHospitalSysDictDO != null){
            return wlyyHospitalSysDictDO;
        }else{
            return null;
        }
    }

    /**
     * 根据医院编号及字典名称，获取配置界面所有的配置信息
     * dictName( 候诊延时： WaitingDelay )
     * dictName( 邀请专家配置： InvitedConfig )
     * @param hospital
     * @param dictName
     * @return
     */
    public List<WlyyHospitalSysDictDO> findByHospitalAndDictName(String hospital, String dictName){
        List<WlyyHospitalSysDictDO> wlyyHospitalSysDictDOs = sysDictDao.findByHospitalAndDictName(hospital,dictName);
        if(wlyyHospitalSysDictDOs != null & wlyyHospitalSysDictDOs.size() > 0){
            return wlyyHospitalSysDictDOs;
        }else{
            return null;
        }
    }



    public List<Map<String, Object>> findByHosAndDict(String hospital, String dictName){
        Map<String,Object> params = new HashedMap();
        String sql ="SELECT " +
                " a.id AS \"id\", " +
                " a.dict_name AS \"dictName\", " +
                " a.dict_code AS \"dictCode\", " +
                " a.dict_value AS \"dictValue\" " +
                " FROM " +
                " wlyy_hospital_sys_dict a " +
                " WHERE 1=1 ";
        if(StringUtils.isNotBlank(hospital)){
            sql+=" AND a.hospital =:hospital";
            params.put("hospital",hospital);
        }
        if(StringUtils.isNotBlank(dictName)){
            sql+=" AND a.dict_name =:dictName";
            params.put("dictName",dictName);
        }
        List<Map<String, Object>> list = hibenateUtils.createSQLQuery(sql, params);
        return  list;
    }


}
