package com.yihu.jw.hospital.service.consult;

import com.yihu.jw.entity.hospital.prescription.WlyyPatientExpressageAddrDO;
import com.yihu.jw.hospital.prescription.dao.WlyyPatientExpressageAddrDao;
import com.yihu.jw.mysql.query.BaseJpaService;
import com.yihu.jw.utils.EntityUtils;
import com.yihu.jw.utils.StringUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * 居民收获地址服务service
 *
 * @version <pre>
 * Author	Version		Date		Changes
 * Administrator    1.0  2019年06月12日 Created
 *
 * </pre>
 * @since 1.
 */
@Service
public class WlyyPatientExpressageAddrService extends BaseJpaService<WlyyPatientExpressageAddrDO, WlyyPatientExpressageAddrDao> {

    Logger logger = LoggerFactory.getLogger(WlyyPatientExpressageAddrService.class);

    @Autowired
    private WlyyPatientExpressageAddrDao patientExpressageAddrDao;

    /**
     * 新增居民收货地址
     *
     * @param jsonData
     * @return
     */
    public WlyyPatientExpressageAddrDO createAddr(String jsonData)throws Exception{
        WlyyPatientExpressageAddrDO addrDO = null;
        if (!StringUtil.isEmpty(jsonData)){
            addrDO = EntityUtils.jsonToEntity(jsonData, WlyyPatientExpressageAddrDO.class);
            if(1 == addrDO.getIsDefault()){
                List<WlyyPatientExpressageAddrDO> defaultAddrDOList = patientExpressageAddrDao.queryByPatientAndIsDefault(addrDO.getPatient(),1);
                if(defaultAddrDOList != null && defaultAddrDOList.size()>0){
                    WlyyPatientExpressageAddrDO olddefaultAddr = defaultAddrDOList.get(0);
                    olddefaultAddr.setIsDefault(0);
                    this.save(olddefaultAddr);
                }
            }
            this.save(addrDO);
        }
        return addrDO;
    }


     public boolean hasDefault(String patient){
        if(StringUtils.isEmpty(patient)){
            return false;
        }
        return patientExpressageAddrDao.existsByPatientAndIsDefault(patient,1);
     }

    /**
     * 将地址设置为默认地址
     * @param id
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public Object setAddrDefault(String patient,String id)throws Exception{
        StringBuilder error = new StringBuilder("OK");
        WlyyPatientExpressageAddrDO newDefaultAddr = patientExpressageAddrDao.findById(id).orElse(null);
        if(null == newDefaultAddr){
            error.replace(0,2,"当前地址不存在!");
            return error;
        }
        List<WlyyPatientExpressageAddrDO> defaultAddrDOList = patientExpressageAddrDao.queryByPatientAndIsDefault(patient,1);
        if(defaultAddrDOList != null && defaultAddrDOList.size()>0){
            WlyyPatientExpressageAddrDO oldDefaultAddr = defaultAddrDOList.get(0);
            oldDefaultAddr.setIsDefault(0);
            this.save(oldDefaultAddr);
        }
        newDefaultAddr.setIsDefault(1);
        this.save(newDefaultAddr);
        return error;
    }


    /**
     * 更新居民收货地址
     * @param addrDO
     * @return
     * @throws Exception
     */
    public WlyyPatientExpressageAddrDO updateAddr(WlyyPatientExpressageAddrDO addrDO)throws Exception{
        if(1 == addrDO.getIsDefault()){
            List<WlyyPatientExpressageAddrDO> defaultAddrDOList = patientExpressageAddrDao.queryByPatientAndIsDefault(addrDO.getPatient(),1);
            if(defaultAddrDOList != null && defaultAddrDOList.size()>0){
                WlyyPatientExpressageAddrDO olddefaultAddr = defaultAddrDOList.get(0);
                olddefaultAddr.setIsDefault(0);
                olddefaultAddr.setCreateTime(new Date());
                this.save(olddefaultAddr);
            }
        }
        if(addrDO.getCreateTime()==null){
            addrDO.setCreateTime(new Date());
        }
        this.save(addrDO);
        return addrDO;
    }

}
