package com.yihu.jw.hospital.service.wechat;


import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.hospital.prescription.service.PrescriptionService;
import com.yihu.jw.util.wechat.WeiXinMessageReplyUtils;
import com.yihu.jw.util.wechat.WeiXinMessageUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 微信事件处理
 * <p>
 * Created by lyr on 2016/08/11
 */
@Component
public class WeiXinEventProcess {

    private static Logger logger = LoggerFactory.getLogger(WeiXinEventProcess.class);

    @Value("${wechat.id}")
    private String wechatId;
    @Autowired
    private BaseDoctorDao baseDoctorDao;
    @Autowired
    private PrescriptionService prescriptionService;
    /**
     * 微信推送事件处理
     *
     * @param message
     * @return
     */
    public String eventProcess(Map<String, String> message) throws Exception {
        String result = "";
        String event = message.get("Event");
        switch (event) {
            case WeiXinMessageUtils.EVENT_TYPE_SCAN: // 扫描二维码事件
                result = scanEventProcess(message);
                break;
            case WeiXinMessageUtils.EVENT_TYPE_SUBSCRIBE: // 订阅事件
                if (message.containsKey("EventKey") && StringUtils.isNotEmpty(message.get("EventKey")) && message.containsKey("Ticket")
                        && StringUtils.isNotEmpty(message.get("Ticket"))) {
                    result = scanEventProcess(message);
                } /*else {
                    result = subscribeEventProcess(message);
                }*/
                break;
            /*case WeiXinMessageUtils.EVENT_TYPE_UNSUBSCRIBE: // 取消订阅事件
                 result = unsubscribeEventProcess(message);
                break;
            case WeiXinMessageUtils.EVENT_TYPE_CLICK: //菜单点击
                result = this.clickProcess(message);*/
            default:
                break;
        }

        return result;
    }

    /**
     * 微信扫码事件处理（封装图文消息体）
     *
     * @param message
     * @return
     * @throws Exception
     */
    private String scanEventProcess(Map<String, String> message) throws Exception {
        String result = "";
        // 场景值
        String baseUrl = prescriptionService.findDomainUrlInDict();
        String eventKey = message.get("EventKey");

        if (StringUtils.isNotEmpty(eventKey) && (eventKey.startsWith("doctor_"))) {

            // 图文信息
            List<Map<String, String>> articles = new ArrayList<>();
            Map<String, String> article = new HashMap<>();
            // 二维码参数
            String[] keys = eventKey.replace("doctor_", "").split("_");
            // 图片URL
            String picUrl = "";
            BaseDoctorDO baseDoctorDO = baseDoctorDao.findByIdAndDel(keys[0]);
            String url = null;
            if (null!=baseDoctorDO){
                picUrl = baseDoctorDO.getPhoto();
            }
            // 医生二维码跳转URL
            if ("xm_ykyy_wx".equalsIgnoreCase(wechatId)){
                url = baseUrl+"/ims-wx/#/doctorInfo?doctorCode="+keys[0];
                picUrl =baseUrl+ "/"+picUrl;
                article.put("Url", url);
                article.put("Title", keys[1] + "医生个人诊间");
                article.put("Description", "厦门眼科中心医院互联网医院");
                article.put("PicUrl", picUrl);
                articles.add(article);
            }else {
                article.put("Url", url);
                article.put("Title", keys[1] + "医生个人诊间");
                article.put("Description", "海沧医院互联网医院");
                article.put("PicUrl", picUrl);
                articles.add(article);
            }

            // 图文消息URL

            //图片地址
//            picUrl = picUrl.replace("{server}", serverUrl);



            // 消息XML
            result = WeiXinMessageReplyUtils.replyNewsMessage(message.get("FromUserName"), message.get("ToUserName"), articles);
        }else if (StringUtils.isNotEmpty(eventKey) && (eventKey.startsWith("flag_"))){

            // 图文信息
            List<Map<String, String>> articles = new ArrayList<>();
            Map<String, String> article = new HashMap<>();
            // 二维码参数

            // 图片URL
            String picUrl = "";

            String url = null;
            // 医生二维码跳转URL
            if ("xm_ykyy_wx".equalsIgnoreCase(wechatId)){
                url =baseUrl+"/ims-wx/redirect.html";
                picUrl = baseUrl+"/group1/M00/00/13/wKghxmAZD-iAEyHgAAA-zHpYNyg537.png";
                article.put("Url", url);
                article.put("Title", "核酸检测预约");
                article.put("Description", "厦门眼科中心医院互联网医院");
                article.put("PicUrl", picUrl);
                articles.add(article);
            }

            // 图文消息URL

            //图片地址
//            picUrl = picUrl.replace("{server}", serverUrl);



            // 消息XML
            result = WeiXinMessageReplyUtils.replyNewsMessage(message.get("FromUserName"), message.get("ToUserName"), articles);


        }

        return result;
    }


}
