package  com.yihu.customer.controller.common.district;

import  com.yihu.customer.controller.BaseController;
import com.yihu.customer.service.common.district.DistrictService;
import jxl.Workbook;
import jxl.read.biff.BiffException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.Map;

/**
 * 省市区三级地址控制类
 *
 * @author George
 */
@Controller
@RequestMapping(value = "common",produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
public class DistrictController extends BaseController {

    @Autowired
    private DistrictService districtService;

    /**
     * 省市一二三级查询接口
     *
     * @param type 1一级目录，2二级目录，3三级目录，4街道目录
     * @param code 省或市标识
     * @return
     */
    @RequestMapping(value = "district")
    @ResponseBody
    public String district(int type, String code) {
        try {
            List<?> list = districtService.findByType(type, code);
            return write(200, "查询成功！", "list", list);
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败！");
        }
    }

    /**
     * 省市一二三级查询接口(包含权限控制)
     *
     * @param type 1一级目录，2二级目录，3三级目录，4街道目录
     * @param code 省或市标识
     * @return
     */
    @RequestMapping(value = "districtAuthority")
    @ResponseBody
    public String districtAuthority(int type, String code) {
        try {
            List<Map<String, String>> roleMap = (List<Map<String, String>>) request.getSession().getAttribute("roleMap");
            List<?> list = districtService.findByType(type, code, roleMap);
            return write(200, "查询成功！", "list", list);
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败！");
        }
    }

    @RequestMapping(value = "/district/importData", produces = "application/json;charset=UTF-8")
    @ResponseBody
    public String importData(HttpServletRequest request) {
        try {
            request.setCharacterEncoding("UTF-8");
            InputStream inputStream = request.getInputStream();
            Workbook workbook = Workbook.getWorkbook(inputStream);

            districtService.importStreet(workbook);

            return write(200, "操作成功");
        } catch (IOException | BiffException e) {
            e.printStackTrace();
            error(e);
            return error(-1, "操作失败！");
        }
    }
}
