package  com.yihu.customer.controller.manager.appeal;

import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.feedback.Appeal;
import com.yihu.customer.service.manager.feedback.AppealService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import springfox.documentation.annotations.ApiIgnore;

import java.util.Map;

/**
 * Created by Reece on 2017/5/6.
 */
@Controller
@RequestMapping(value = "admin/appeal")
@Api(description = "管理系统账号申诉接口")
public class AppealController extends BaseController {
    @Autowired
    private AppealService appealService;

    //页面初始化数据
    @RequestMapping(value = "/initial", method = RequestMethod.GET)
    @ApiIgnore
    @ApiOperation(value = "意见反馈")
    public String initFeedback() {
        return "appeal/appeal_list";
    }

    /**
     * 初始化列表页
     * @param type 账号申诉的类别
     * @param identity 身份
     * @param status 状态
     * @param page
     * @param rows
     * @return
     */
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "列表")
    public String list(
            @RequestParam(required = false,defaultValue = "-1") int type,
            @RequestParam(required = false,defaultValue = "-1") int identity,
            @RequestParam(required = false,defaultValue = "-1") int status,
            @RequestParam(value = "page",defaultValue = "1") int page,
            @RequestParam(value = "rows",defaultValue = "15") int rows) {
        try {
            Page<Appeal> appeals = appealService.findAppeal(page, rows, type, identity, status);
            return write(200, "操作成功", page, rows, appeals);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败");
        }
    }

    @RequestMapping(value = "/delAppeal")
    @ResponseBody
    @ApiOperation(value="批量删除")
    public String delAppeal(
            @RequestParam(required = true) String ids){
        try{
            appealService.delappeal(ids);
            return write(200,"操作成功");
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败");
        }
    }


    //跳转到意见详情页
    @RequestMapping(value = "showDetail", method = RequestMethod.GET)
    @ApiIgnore
    @ApiOperation(value = "跳转到意见详情页")
    public String infoInit(Integer id ) {
        request.setAttribute("id",id);
        return "appeal/appeal_detail";
    }

    /**
     * 根据申诉ID查询申诉详情，并更改状态
     * @param id
     * @return
     */
    @RequestMapping(value = "getAppealById")
    @ResponseBody
    public String getAppealById(@RequestParam(value = "id") Long id) {
        try {
            Map appeal = appealService.getAppealById(id);
            return write(200, "操作成功！", "data", appeal);
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }

    ////跳转到处理页面
    //@RequestMapping(value = "dealInit", method = RequestMethod.GET)
    //@ApiIgnore
    //@ApiOperation(value = "跳转到处理页面")
    //public String dealInit(Integer id ) {
    //    request.setAttribute("id",id);
    //    return "appeal/appeal_deal";
    //}

    /**
     * 根据申诉id,回复内容result进行处理
     * @param id
     * @param result
     * @return
     */
    @RequestMapping(value="dealAppeal", method = RequestMethod.POST)
    @ResponseBody
    public String dealAppeal(@RequestParam(required = true,value="id") Long id,
                             @RequestParam(required = true,value="result") String result){
        try {
            appealService.dealAppeal(id,result);
            return write(200,"操作成功!");
        }catch (Exception e){
            error(e);
            return error(-1, "操作失败!");
        }
    }
}
