package  com.yihu.customer.controller.manager.basedata;


import  com.yihu.customer.controller.WeixinBaseController;
import com.yihu.customer.service.manager.basedata.BaseDataImportService;
import com.yihu.customer.service.wechat.WechatApiService;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 基础数据导入
 */
@Controller
@RequestMapping(value = "/admin/basedata",produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
public class BaseDataImportController extends WeixinBaseController {

    @Autowired
    private BaseDataImportService dataImportService;
    @Autowired
    private WechatApiService wechatApiService;

    //页面跳转（主页面）
    @RequestMapping(value = "initial")
    public String listInit() {
        return "basedata/basedata_list";
    }

    @RequestMapping(value = "labelImport")
    public String labelImport() {
        return "basedata/basedata_label";
    }

    @RequestMapping(value = "selectLabel")
    @ResponseBody
    public String selectLabel() {
        try {
            return write(200, "操作成功", "data", dataImportService.selectLabel());
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败！");
        }
    }

    /**
     * 同步标签至微信
     * serverTypeLables 服务类型 标签列表
     * diseaseTypes 服务类型 标签列表
     * healthTypeLables 服务类型 标签列表
     * @return
     */
    @RequestMapping(value = "syncLabelToWeixin")
    @ResponseBody
    public String syncLabelToWeixin(
            @RequestParam(name = "serverTypeLables",required = false) String serverTypeLables,
            @RequestParam(name = "diseaseTypes",required = false) String diseaseTypes,
            @RequestParam(name = "healthTypeLables",required = false) String healthTypeLables,
            @RequestParam(name = "townLables",required = false) String townLables,
            @RequestParam(name = "townCode",required = false) String townCode
    ) {
        try {
            List<String> weixinOpenIdList = wechatApiService.getOpenIdList(getAccessToken());
            List<String> lableOpenIds = new ArrayList<>();
            String result = "";
            if(StringUtils.isEmpty(townCode)){
                return error(-1, "参数区县不能为空！");
            }
            if(StringUtils.isNotEmpty(serverTypeLables)){
                lableOpenIds =  dataImportService.getLabelWeixinOpenId(1, serverTypeLables, weixinOpenIdList,townCode);
                result = syscTagsToWeixin(serverTypeLables,lableOpenIds);
            }
            if(StringUtils.isNotEmpty(diseaseTypes)){
                lableOpenIds =  dataImportService.getLabelWeixinOpenId(2, diseaseTypes, weixinOpenIdList,townCode);
                result = syscTagsToWeixin(serverTypeLables,lableOpenIds);
            }
            if( StringUtils.isNotEmpty(healthTypeLables)){
                lableOpenIds =   dataImportService.getLabelWeixinOpenId(2, healthTypeLables, weixinOpenIdList,townCode);
                result =syscTagsToWeixin(serverTypeLables,lableOpenIds);
            }
            if (StringUtils.isNoneEmpty(townLables)){
                lableOpenIds =   dataImportService.getLabelWeixinOpenId(3, townLables, weixinOpenIdList,townCode);
                result =syscTagsToWeixin(townLables,lableOpenIds);
            }

            return write(200, "操作成功", "data", result);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败！");
        }
    }

    /**
     * 医生同步微信
     *
     * @param townLables
     * @return
     */
    @RequestMapping(value = "syncLabelToWeixin1")
    @ResponseBody
    public String syncLabelToWeixin1(
            @RequestParam(name = "townLables",required = false) String townLables) {
        try {
            List<String> weixinOpenIdList = wechatApiService.getOpenIdList(getAccessToken());
            List<String> lableOpenIds = new ArrayList<>();
            String result = "";
            if (StringUtils.isNoneEmpty(townLables)){
                lableOpenIds =   dataImportService.getLabelWeixinOpenId1(townLables, weixinOpenIdList);
                result =syscTagsToWeixin(townLables,lableOpenIds);
            }

            return write(200, "操作成功", "data", result);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败！");
        }
    }

    public String  syscTagsToWeixin(String lables,List<String> openIds) throws UnsupportedEncodingException, JSONException {
        if(openIds ==null || openIds.size() ==0){
            return "success";
        }
        String result = "";
        String tagId = "";
        Map<String,String> weixinExistTags = wechatApiService.getWeiXinTaggs(getAccessToken());
        String [] lableArr = lables.split(",");
        for(int i=0 ;i< lableArr.length ;i++) {
            String tag = lableArr[i].trim();
            if (weixinExistTags.containsKey(tag)) {
                tagId = weixinExistTags.get(tag);
            } else {
                tagId = wechatApiService.createTagging(getAccessToken(), tag);
            }
            List<String> sysOpenIds = new ArrayList<>();
            if(openIds.size() > 50){
                int m = openIds.size()/50;
                int d = openIds.size()%50;
                if(d!=0){
                    m++;
                }
                for(int k=0;k < m ;k++){
                    int to = (k+1)*50;
                    if(to > openIds.size()){
                        to = openIds.size();
                    }
                    sysOpenIds = openIds.subList(k*50,to);
                    result = wechatApiService.batchtagging(getAccessToken(), sysOpenIds, tagId);
                }
            }else {
                result = wechatApiService.batchtagging(getAccessToken(), openIds, tagId);
            }
        }
        return result;
    }



}
