package  com.yihu.customer.controller.manager.datastatic;

import  com.yihu.customer.controller.BaseController;
import com.yihu.customer.service.statistic.DoorService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import jxl.Workbook;
import jxl.write.Label;
import jxl.write.WritableSheet;
import jxl.write.WritableWorkbook;
import jxl.write.WriteException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.util.List;
import java.util.Map;

/**
 * Created by humingfen on 2020/12/14.
 * 康复下转
 */

@Controller
@RequestMapping(value = "admin/door", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(description = "上门服务")
public class DoorController extends BaseController {

    @Autowired
    private DoorService doorService;

    @RequestMapping(value = "initial", method = RequestMethod.GET)
    public String initListPage() {
        return "static/door_list";
    }

    @RequestMapping(value = "list", method = RequestMethod.POST)
    @ApiOperation("上门服务列表")
    @ResponseBody
    public String searchList(
            @RequestParam(value = "number", required = false) String number,
            @RequestParam(value = "town", required = false) @ApiParam(value = "市区") String town,
            @RequestParam(value = "hospital", required = false) String hospital,
            @RequestParam(value = "startDate", required = false) String startDate,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "idcard", required = false) String idcard,
            @RequestParam(value = "endDate", required = false) String endDate,
            @RequestParam(value = "type", required = false) @ApiParam("上门资质人群类型，1残疾人，2肺结核患者，3计生特殊家庭，4重性精神病,5失能/半失能老人,6其他") String type,
            @RequestParam(value = "page", required = true) Integer page,
            @RequestParam(value = "rows", required = true) Integer rows) {
        try {
            Page list = doorService.searchList(number, town, hospital,name,idcard,startDate, endDate, type, page, rows);
            return write(200, "操作成功", page, rows, list);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败！");
        }
    }

    @RequestMapping(value = "deleteById", method = RequestMethod.POST)
    @ApiOperation("删除上门服务记录")
    @ResponseBody
    public String deleteById(
            @RequestParam(value = "id")
            @ApiParam(value = "上门服务id") String id) {
        try {
            doorService.deleteById(id);
            return write(200, "操作成功");
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败！");
        }
    }

    @RequestMapping(value = "exportExcel", method = RequestMethod.POST)
    @ApiOperation("导出上门列表")
    @ResponseBody
    public void searchList(
            @RequestParam(value = "town", required = false) @ApiParam(value = "市区") String town,
            @RequestParam(value = "hospital", required = false) String hospital,
            @RequestParam(value = "startDate", required = false) String startDate,
            @RequestParam(value = "endDate", required = false) String endDate,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "idcard", required = false) String idcard,
            @RequestParam(value = "type", required = false) @ApiParam("上门资质人群类型，1残疾人，2肺结核患者，3计生特殊家庭，4重性精神病,5失能/半失能老人,6其他") String type,
            HttpServletResponse response) {
        try {
            List<Map<String, Object>> list = doorService.exportList(town, hospital, name,idcard,startDate, endDate,type);
            response.setContentType("octets/stream");
            response.setHeader("Content-Disposition", "attachment; filename=" + new String("doorList.xls"));
            OutputStream os = response.getOutputStream();
            this.write(os, list);
        } catch (Exception ex) {
            error(ex);
        }
    }

    private void write(OutputStream os, List<Map<String, Object>> list) throws Exception {
        WritableWorkbook wwb = Workbook.createWorkbook(os);
        try {
            WritableSheet sheet = wwb.createSheet("sheet", 1);
            String header[] = {"服务对象", "身份证号", "居民联系方式", "服务类型", "服务医生","服务社区","调度员备注", "服务次数", "工单状态"};
            int i = 0;
            for (String h : header) {
                addCell(sheet, 0, i, h);
                i++;
            }
            int j = 1;
            for (Map<String, Object> map : list) {
                addCell(sheet, j, 0, map.get("patientName") + "");
                addCell(sheet, j, 1, map.get("idcard") + "");
                addCell(sheet, j, 2, map.get("mobile") != null ? map.get("mobile").toString() : "");
                addCell(sheet, j, 3, map.get("typeValue") + "");
                addCell(sheet, j, 4, map.get("doctorName") != null ? map.get("doctorName").toString() : "");
                addCell(sheet, j, 5, map.get("hospitalName") != null ? map.get("hospitalName").toString() : "");
                addCell(sheet, j, 6, map.get("remark") != null ? map.get("remark").toString() : "");
                addCell(sheet, j, 7, map.get("count") + "");
                if (map.get("status").toString().equals("-1")) {
                    addCell(sheet, j, 8, "已取消");
                } else  if (map.get("status").toString().equals("1")) {
                    addCell(sheet, j, 8, "待派单");
                }else  if (map.get("status").toString().equals("2")){
                    addCell(sheet, j, 8, "待接单");
                }else  if (map.get("status").toString().equals("3")){
                    addCell(sheet, j, 8, "已接单");
                }else  if (map.get("status").toString().equals("4")){
                    addCell(sheet, j, 8, "待服务");
                }else  if (map.get("status").toString().equals("5")){
                    addCell(sheet, j, 8, "待评价");
                }else  if (map.get("status").toString().equals("6")){
                    addCell(sheet, j, 8, "已完成");
                }
                j++;
            }
            wwb.write();
            wwb.close();
        } catch (Exception e) {
            e.printStackTrace();
            if (wwb != null) wwb.close();
        }
    }

    private void addCell(WritableSheet ws, int row, int column, String data) throws WriteException {
        Label label = new Label(column, row, data);
        ws.addCell(label);
    }
}
