package  com.yihu.customer.controller.manager.datastatic;

import  com.yihu.customer.controller.BaseController;
import com.yihu.customer.service.statistic.RehabilitationPatientService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import jxl.Workbook;
import jxl.write.Label;
import jxl.write.WritableSheet;
import jxl.write.WritableWorkbook;
import jxl.write.WriteException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.util.List;
import java.util.Map;

/**
 * Created by humingfen on 2020/12/14.
 * 康复下转
 */

@Controller
@RequestMapping(value = "admin/rehabilitation", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(description = "康复下转居民")
public class RehabilitationPatientController extends BaseController {

    @Autowired
    private RehabilitationPatientService rehabilitationPatientService;

    @RequestMapping(value = "initial", method = RequestMethod.GET)
    public String initListPage() {
        return "static/rehabilitation_patient_list";
    }

    @RequestMapping(value = "infoInit", method = RequestMethod.GET)
    public String initInfoPage(@RequestParam(value = "id") String id){
        request.setAttribute("id",id);
        return "static/rehabilitation_patient_detail";
    }

    @RequestMapping(value = "list", method = RequestMethod.POST)
    @ApiOperation("康复下转列表")
    @ResponseBody
    public String searchList(
            @RequestParam(value = "id", required = false) @ApiParam(value = "康复计划id") String id,
            @RequestParam(value = "town", required = false) @ApiParam(value = "市区") String town,
            @RequestParam(value = "hospital", required = false) String hospital,
            @RequestParam(value = "startDate", required = false) String startDate,
            @RequestParam(value = "endDate", required = false) String endDate,
            @RequestParam(value = "disease", required = false) @ApiParam("疾病类型code") String disease,
            @RequestParam(value = "isManage", required = false) @ApiParam(value = "是否管理，1是0否", name = "isManage") Integer isManage,
            @RequestParam(value = "page", required = true) Integer page,
            @RequestParam(value = "rows", required = true) Integer rows) {
        try {
            Page list = rehabilitationPatientService.searchList(id, town, hospital, startDate, endDate, disease, isManage, page, rows);
            return write(200, "操作成功", page, rows, list);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败！");
        }
    }

    @RequestMapping(value = "exportExcel", method = RequestMethod.POST)
    @ApiOperation("导出康复下转居民列表")
    @ResponseBody
    public void searchList(
            @RequestParam(value = "town", required = false) @ApiParam(value = "市区") String town,
            @RequestParam(value = "hospital", required = false) String hospital,
            @RequestParam(value = "startDate", required = false) String startDate,
            @RequestParam(value = "endDate", required = false) String endDate,
            @RequestParam(value = "disease", required = false) @ApiParam("疾病类型code") String disease,
            @RequestParam(value = "isManage", required = false) @ApiParam(value = "是否管理，1是0否", name = "isManage") Integer isManage,
            HttpServletResponse response) {
        try {
            List<Map<String, Object>> list = rehabilitationPatientService.exportList(town, hospital, startDate, endDate, disease, isManage);
            response.setContentType("octets/stream");
            response.setHeader("Content-Disposition", "attachment; filename=" + new String("rehabilitationPatientList.xls"));
            OutputStream os = response.getOutputStream();
            this.write(os, list);
        } catch (Exception ex) {
            error(ex);
        }
    }

    private void write(OutputStream os, List<Map<String, Object>> list) throws Exception {
        WritableWorkbook wwb = Workbook.createWorkbook(os);
        try {
            WritableSheet sheet = wwb.createSheet("sheet", 1);
            String header[] = {"姓名", "身份证号", "疾病类型", "专科医院", "专科医生", "签约社区", "签约医生", "是否接收","是否服务", "下转时间"};
            int i = 0;
            for (String h : header) {
                addCell(sheet, 0, i, h);
                i++;
            }
            int j = 1;
            for (Map<String, Object> map : list) {
                addCell(sheet, j, 0, map.get("name") + "");
                addCell(sheet, j, 1, map.get("idcard") + "");
                addCell(sheet, j, 2, map.get("diseaseName") + "");
                addCell(sheet, j, 3, map.get("sHospitalName") + "");
                addCell(sheet, j, 4, map.get("specialistName") + "");
                addCell(sheet, j, 5, map.get("hospitalName") + "");
                addCell(sheet, j, 6, map.get("doctorName") + "");
                if (map.get("isManage") == null || map.get("isManage").toString().equals("0")) {
                    addCell(sheet, j, 7, "否");
                } else {
                    addCell(sheet, j, 7, "是");
                }
                if (map.get("served") == null || map.get("served").toString().equals("0")) {
                    addCell(sheet, j, 8, "否");
                } else {
                    addCell(sheet, j, 8, "是");
                }
                addCell(sheet, j, 9, map.get("createTime").toString());
                j++;
            }
            wwb.write();
            wwb.close();
        } catch (Exception e) {
            e.printStackTrace();
            if (wwb != null) wwb.close();
        }
    }

    private void addCell(WritableSheet ws, int row, int column, String data) throws WriteException {
        Label label = new Label(column, row, data);
        ws.addCell(label);
    }


}
