package  com.yihu.customer.controller.manager.dict;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.ManageDict;
import  com.yihu.customer.entity.twoCancers.BreastCervicalAddress;
import com.yihu.customer.service.manager.dict.ManageDictService;
import com.yihu.customer.service.manager.twoCancers.TwoCancersService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * Created by Administrator on 2017/1/24.
 */
@Controller
@RequestMapping(value = "/admin/dict")
@Api(value = "dict", description = "系统全局字典管理", tags = {"系统字典"})
public class ManageDictController extends BaseController {

    @Autowired
    ManageDictService dictService;
    @Autowired
    TwoCancersService twoCancersService;
    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Autowired
    private ObjectMapper objectMapper;

    @RequestMapping(value = "initial", method = RequestMethod.GET)
    public String listPageInit() {
        return "dict/manageDict";
    }

    @ApiOperation(value = "获取字典列表")
    @RequestMapping(value = "/dictList")
    @ResponseBody
    public String getDictionaries(
            @RequestParam(value = "searchNm", required = false) String searchNm,
            @RequestParam(value = "page", required = false) int page,
            @RequestParam(value = "rows", required = false) int pageSize) {
        if (page <= 0) {
            page = 1;
        }
        if (pageSize <= 0) {
            page = 15;
        }
        PageRequest pageRequest = PageRequest.of(page - 1, pageSize);
        String filters = "";
        if (!StringUtils.isEmpty(searchNm)) {
            filters = "name?" + searchNm;
        }
        try {
            List list = dictService.search("", filters, "", page, pageSize);
            long count = dictService.getCount(filters);
            PageImpl<ManageDict> manageDicts = new PageImpl<ManageDict>(list, pageRequest, count);
            return write(200, "", page, pageSize, manageDicts);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "系统错误");
        }
    }

    @ApiOperation(value = "创建字典")
    @RequestMapping(value = "/create")
    @ResponseBody
    public String createDictionary(
            @ApiParam(name = "name", value = "新增字典名称")
            @RequestParam(value = "name") String name,
            @ApiParam(name = "reference", value = "字典简介", defaultValue = "")
            @RequestParam(value = "reference", required = false) String reference) throws Exception {
        try {
            if (StringUtils.isEmpty(name)) {
                return error(-1, "新增字段名称不能为空！");
            }
            ManageDict dict = new ManageDict();
            dict.setName(name);
            dict.setPhoneticCode(name);
            dict.setReference(reference);
            dict.setAuthorId("wwcs");
            dict.setCreateDate(new Date());
            ManageDict systemDict = dictService.createDict(dict);
            return write(200, "操作成功");
        } catch (Exception ex) {
            error(ex);
            return write(-1, "操作失败！");
        }
    }

    @ApiOperation(value = "获取字典")
    @RequestMapping(value = "/dict")
    @ResponseBody
    public String getDictionary(
            @ApiParam(name = "id", value = "字典ID", defaultValue = "")
            @PathVariable(value = "id") long id) {
        ManageDict dict = dictService.retrieve(id);
        if (dict == null) {
            return write(-1, "字典不存在！");
        }
        return write(200, "操作成功", "data", dict);
    }

    @ApiOperation(value = "更新字典")
    @RequestMapping(value = "/update")
    @ResponseBody
    public String updateDictionary(
            @ApiParam(name = "dictId", value = "字典id")
            @RequestParam(value = "dictId", required = true) long dictId,
            @ApiParam(name = "name", value = "新增字典名称")
            @RequestParam(value = "name", required = true) String name,
            @ApiParam(name = "reference", value = "字典简介", defaultValue = "")
            @RequestParam(value = "reference", required = false) String reference) {
        ManageDict dict = dictService.retrieve(dictId);
        if (dict == null) {
            return error(-1, "字典不存在！");
        }
        if (StringUtils.isEmpty(name)) {
            return error(-1, "字典名称不能为空");
        }
        try {
            dict.setName(name);
            dictService.updateDict(dict);
            return write(200, "操作成功！");
        } catch (Exception ex) {
            error(ex);
            return write(-1, "操作失败！");
        }
    }

    @ApiOperation(value = "删除字典")
    @RequestMapping(value = "/delete")
    @ResponseBody
    public String deleteDictionary(
            @ApiParam(name = "dictId", value = "字典ID", defaultValue = "")
            @RequestParam(value = "dictId") long id) {
        try {
            dictService.deleteDict(id);
            return write(200, "操场成功");
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败！");
        }
    }

    @RequestMapping(value = "/isNameExist")
    @ApiOperation(value = "判断提交的字典名称是否已经存在")
    @ResponseBody
    public String isDictNameExists(
            @ApiParam(name = "name", value = "name", defaultValue = "")
            @RequestParam(value = "name") String name) {
        boolean res = dictService.isDictNameExists(name);
        if (res) {
            return error(-1, "字典名称已存在！");
        }
        return write(200, "字典名称不存在！");
    }


    @RequestMapping(value = "/syncZyReservationOrgAndDept",method = RequestMethod.GET)
    @ApiOperation(value = "数据同步-同步医生科室")
    @ResponseBody
    public String syncZyReservationOrgAndDept() {
        boolean res = dictService.syncZyReservationOrgAndDept();
        if (res) {
            return error(200, "操作成功！");
        }
        return write(-1, "操作失败！");
    }

    @RequestMapping(value = "/syncZyReservationOrgAndDept/initial",method = RequestMethod.GET)
    public String syncZyReservationOrgAndDepInitialt() {
        return "dict/zyReservationDeptAndOrg";
    }


    @RequestMapping(value = "/delTwoCancersAddress",method = RequestMethod.POST)
    @ApiOperation(value = "两癌-删除筛查地址")
    @ResponseBody
    public String delTwoCancersAddress(@RequestParam(value = "id")Integer id){
        try {
            twoCancersService.delTwoCancersAddress(id);
            return write(200,"操作成功!");
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1,"操作失败!");
        }
    }

    @RequestMapping(value = "/saveTwoCancersAddress",method = RequestMethod.POST)
    @ApiOperation(value = "两癌-保存筛查地址")
    @ResponseBody
    public String saveTwoCancersAddress(String jsonData){
        try {
            Integer result = twoCancersService.saveTwoCancersAddress(jsonData);
            if (result>0){
               return write(200,"操作成功!");
            }else {
               return error(-2,"该机构已存在!");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1,"操作失败!");
        }
    }

    @RequestMapping(value = "/findHospitalList",method = RequestMethod.POST)
    @ApiOperation(value = "两癌-获取医院机构")
    @ResponseBody
    public String findHospitalList(){
        try {
            List<Map<String, Object>> result = twoCancersService.findHospitalList();
            return write(200,"操作成功!","data",result);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1,"操作失败!");
        }
    }

    @RequestMapping(value = "/findTwoCancersAddressList",method = RequestMethod.GET)
    @ApiOperation(value = "两癌-获取列表")
    @ResponseBody
    public String findTwoCancersAddressList(@RequestParam(value = "page",required = false)Integer page,
                      @RequestParam(value = "rows")Integer rows,
                      @RequestParam(value = "hospitalName",required = false)String hospitalName){
        try {

            Page<BreastCervicalAddress> list = twoCancersService.findTwoCancersAddressList(page,rows,hospitalName);
            return write(200, "查询成功", page, rows, list);
        } catch (Exception e) {
            e.printStackTrace();
            return  error(-1,"操作失败!");
        }
    }

    @RequestMapping(value = "/twoCancers/initial",method = RequestMethod.GET)
    public String listInit(){
        return "dict/twoCancersAddress_list";
    }

    //编辑页面
    @RequestMapping(value = "twoCancers/updateInit/{id}", method = RequestMethod.GET)
    public String update(@PathVariable("id") Long id) {
        try {
            request.setAttribute("id", id);
            request.setAttribute("type", request.getParameter("type"));
        } catch (Exception e) {
            error(e);
        }
        return "dict/twoCancersAddress_edit";
    }

    @RequestMapping(value = "twoCancers/getHospital", method = RequestMethod.GET)
    @ResponseBody
    public String getTwoCancerHospitalList(){
        try {
            return write(200,"操作成功","list",twoCancersService.getTwoCancerHospitalList());
        }catch (Exception e){
            e.printStackTrace();
            return error(-1,"查询机构失败");
        }
    }


    @RequestMapping(value = "twoCancers/getAddressOne", method = RequestMethod.GET)
    @ResponseBody
    public String getTwoCancerAddressOne(@RequestParam(value = "id")String id){
        try {
            List<BreastCervicalAddress> list = twoCancersService.getTwoCancerAddressOne(id);
            if (list.size()>0){
                return write(200,"操作成功","data",list.get(0));
            }else {
                return error(-1,"未获取到该条记录");
            }
        }catch (Exception e){
            e.printStackTrace();
            return error(-1,"查询机构失败");
        }
    }

    @RequestMapping(value = "twoCancersCount/initial", method = RequestMethod.GET)
    public String initListPage() {
        return "static/two_cancers_count";
    }

    @RequestMapping(value = "twoCancers/findTwoCancerCouCount", method = RequestMethod.GET)
    @ResponseBody
    public String findTwoCancerCouCount(@RequestParam(value = "startTime")String startTime,
                                        @RequestParam(value = "endTime")String endTime){
        try {
            JSONObject twoCancerCouCount = twoCancersService.findTwoCancerCouCount(startTime, endTime);
            return write(200,"成功","data",twoCancerCouCount);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1,"查询失败");
        }
    }

}
