package  com.yihu.customer.controller.manager.dict;

import com.fasterxml.jackson.databind.ObjectMapper;
import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.DictEntryKey;
import  com.yihu.customer.entity.ManageDictEntry;
import com.yihu.customer.service.manager.dict.ManageDictEntryService;
import com.yihu.customer.service.manager.dict.ManageDictService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * Created by Administrator on 2017/1/24.
 */
@Controller
@RequestMapping(value = "/admin/dictEntry")
@Api(value = "DictEntry", description = "系统全局字典项管理", tags = {"系统字典项"})
public class ManageDictEntryController extends BaseController {
    @Autowired
    ManageDictService dictService;

    @Autowired
    ManageDictEntryService dictEntryService;

    @Autowired
    ObjectMapper objectMapper;

    @ApiOperation(value = "获取字典项列表，分页")
    @RequestMapping(value = "/entryList")
    @ResponseBody
    public String getDictEntries(
            @RequestParam(value = "dictId", required = true) long dictId,
            @RequestParam(value = "page", required = false) int page,
            @RequestParam(value = "rows", required = false) int pageSize) {
        if (page <= 0) {
            page = 1;
        }
        if (pageSize <= 0) {
            page = 15;
        }
        PageRequest pageRequest = PageRequest.of(page - 1, pageSize);
        if (dictId <= 0) {
            return error(-1, "字典Id不能为空！");
        }
        String filters = "dictId=" + dictId;
        try {
            List list = dictEntryService.search("", filters, "", page, pageSize);
            long count = dictEntryService.getCount(filters);
            PageImpl<ManageDictEntry> manageDicts = new PageImpl<ManageDictEntry>(list, pageRequest, count);
            return write(200, "", page, pageSize, manageDicts);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "系统错误");
        }
    }

    @ApiOperation(value = "获取字典项列表，不分页")
    @RequestMapping(value = "/entryListForDDL")
    @ResponseBody
    public String searchDictEntriesNoPage(
            @RequestParam(value = "dictId", required = true) long dictId) {
        String filters = "dictId=" + dictId;
        try {
            List list = dictEntryService.search(filters);
            return write(200, "", "dictList", list);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "系统错误");
        }
    }

    @ApiOperation(value = "创建字典项")
    @RequestMapping(value = "/create")
    @ResponseBody
    public String createDictEntry(
            @RequestParam(value = "dictId", required = true) Long dictId,
            @RequestParam(value = "code", required = true) String code,
            @RequestParam(value = "value") String value,
            @RequestParam(value = "sort", required = false) Integer sort,
            @RequestParam(value = "catalog", required = false) String catalog) {

        if (dictId == null) {
            return error(-1, "字典Id不能为空");
        }
        if (StringUtils.isEmpty(code)) {
            return error(-1, "字典项编码不能为空");
        }
        if (StringUtils.isEmpty(value)) {
            return error(-1, "字典项值不能为空");
        }

        ManageDictEntry dictEntryModel = new ManageDictEntry();
        dictEntryModel.setDictId(dictId);
        dictEntryModel.setCode(code);
        dictEntryModel.setValue(value);
        dictEntryModel.setSort(sort);
        dictEntryModel.setCatalog(catalog);
//            if(Boolean.parseBoolean()){
//                return error(-1,"代码在该字典中已存在，请确认。");
//            }
        try {
            ManageDictEntry save = dictEntryService.save(dictEntryModel);
            return write(200, "操作成功！");
        } catch (Exception ex) {
            error(ex);
            return write(-1, "操作失败！");
        }
    }

    @ApiOperation(value = "获取字典项")
    @RequestMapping(value = "/dictEntry")
    @ResponseBody
    public String getDictEntry(
            @ApiParam(name = "dictId", value = "字典ID", defaultValue = "")
            @RequestParam(value = "dictId") long dictId,
            @ApiParam(name = "code", value = "字典项代码", defaultValue = "")
            @PathVariable(value = "code") String code) {
        ManageDictEntry manageDictEntry = dictEntryService.getDictEntry(dictId, code);
        return write(200, "操作成功！", "data", manageDictEntry);
    }

    @ApiOperation(value = "删除字典项")
    @RequestMapping(value = "/delete")
    @ResponseBody
    public Object deleteDictEntry(
            @ApiParam(name = "dictId", value = "字典ID", defaultValue = "")
            @RequestParam(value = "dictId") long dictId,
            @ApiParam(name = "code", value = "字典ID", defaultValue = "")
            @RequestParam(value = "code") String code) throws Exception {
        try {
            dictEntryService.deleteDictEntry(dictId, code);
            return write(200, "操场成功");
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败！");
        }
    }

    @ApiOperation(value = "修改字典项")
    @RequestMapping(value = "/update")
    @ResponseBody
    public String updateDictEntry(
            @RequestParam(value = "dictId", required = true) Long dictId,
            @RequestParam(value = "code", required = true) String code,
            @RequestParam(value = "value") String value,
            @RequestParam(value = "sort", required = false) Integer sort,
            @RequestParam(value = "catalog", required = false) String catalog) {
        if (dictId == null) {
            return error(-1, "字典id不能为空");
        }
        if (StringUtils.isEmpty(code)) {
            return error(-1, "字典项编码code不能为空");
        }
        ManageDictEntry dictEntryModel = dictEntryService.retrieve(new DictEntryKey(code, dictId));
        if (null == dictEntryModel) {
            return error(-1, "字典项不存在！");
        }
        dictEntryModel.setValue(value);
        dictEntryModel.setSort(sort);
        dictEntryModel.setCatalog(catalog);
//        if(Boolean.parseBoolean()){
//                return error(-1,"代码在该字典中已存在，请确认。");
//            }
        try {
            dictEntryService.saveDictEntry(dictEntryModel);
            return write(200, "操作成功！");
        } catch (Exception ex) {
            error(ex);
            return write(-1, "操作失败！");
        }
    }

    @RequestMapping(value = "/isCodeExist")
    @ApiOperation(value = "根基dictId和code判断提交的字典项名称是否已经存在")
    @ResponseBody
    public boolean isDictEntryCodeExists(
            @ApiParam(name = "dictId", value = "dictId", defaultValue = "")
            @RequestParam(value = "dictId") long dictId,
            @ApiParam(name = "code", value = "code", defaultValue = "")
            @RequestParam(value = "code") String code) {
        return dictEntryService.isDictContainEntry(dictId, code);
    }
}
