package  com.yihu.customer.controller.manager.feedback;

import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.feedback.Feedback;
import com.yihu.customer.service.manager.feedback.FeedbackService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import springfox.documentation.annotations.ApiIgnore;

import java.util.Map;

/**
 * Created by Reece on 2017/5/6.
 */
@Controller
@RequestMapping(value = "admin/feedback")
@Api(description = "管理系统意见反馈接口")
public class FeedbackController extends BaseController {

    @Autowired
    private FeedbackService feedbackService;

    //页面初始化数据
    @RequestMapping(value = "/initial", method = RequestMethod.GET)
    @ApiIgnore
    @ApiOperation(value = "意见反馈")
    public String initFeedback() {
        return "feedback/feedback_list";
    }

    /**
     * 初始化列表页
     * @param type 意见反馈的类别
     * @param identity 身份
     * @param status 状态
     * @param page
     * @param rows
     * @return
     */
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "列表")
    public String list(
            @RequestParam(required = false,defaultValue = "-1") int type,
            @RequestParam(required = false,defaultValue = "-1") int identity,
            @RequestParam(required = false,defaultValue = "-1") int status,
            @RequestParam(value = "page",defaultValue = "1") int page,
            @RequestParam(value = "rows",defaultValue = "15") int rows) {
        try {
            Page<Feedback> res = feedbackService.findFeedback(page,rows,type,identity,status);
            return write(200, "操作成功", page, rows, res);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败");
        }
    }

    @RequestMapping(value = "/delfeedback")
    @ResponseBody
    @ApiOperation(value="批量删除")
    public String delfeedback(
            @RequestParam(required = true) String ids){
        try{
            feedbackService.delfeedback(ids);
            return write(200,"操作成功");
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败");
        }
    }

    @RequestMapping(value = "/showDetail", method = RequestMethod.GET)
    @ApiOperation(value = "跳转到意见详情页,并更改状态")
    public String infoInit(@RequestParam long id) {
        try {
            Map map = feedbackService.getFeedbackById(id);
            request.setAttribute("id",id);
            return "feedback/feeback_detail";
        } catch (Exception e) {
            error(e);
            return invalidUserException(e, -1, "查询失败！");
        }
    }


    /**
     * 根据反馈ID查询反馈详情，并更改状态
     * @param id
     * @return
     */
    @RequestMapping(value = "getFeedbackById")
    @ResponseBody
    public String getFeedbackById(@RequestParam(value = "id") Long id) {
        try {
            Map feedback = feedbackService.getFeedbackById(id);
            return write(200, "操作成功！", "data", feedback);
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }

    /**
     * 根据反馈id,回复内容result进行处理
     * @param id
     * @param result
     * @return
     */
    @RequestMapping(value="dealFeedback", method = RequestMethod.POST)
    @ResponseBody
    public String dealAppeal(@RequestParam(required = true,value="id") Long id,
                             @RequestParam(required = true,value="result") String result){
        try {
            Feedback feedback = feedbackService.dealFeedback(id,result);
            feedbackService.sendMsg(feedback);
            return write(200,"操作成功!");
        }catch (Exception e){
            error(e);
            return error(-1, "操作失败!");
        }
    }

}
