package  com.yihu.customer.controller.manager.healthbank;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.node.ObjectNode;
import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.healthbank.ActivityDO;
import com.yihu.customer.service.manager.healthbank.ActivityService;
import com.yihu.customer.util.fastdfs.FastDFSUtil;
import io.swagger.annotations.ApiParam;
import net.sf.json.JSONArray;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import java.io.InputStream;
import java.util.List;
import java.util.Map;

/**
 * Created by humingfen on 2018/6/19.
 * 健康活动管理
 */
@Controller
@RequestMapping(value = "admin/activity", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
public class ActivityController extends BaseController {
    private Logger logger = LoggerFactory.getLogger(ActivityController.class);

    @Autowired
    private ActivityService activityService;
    @ApiParam
    FastDFSUtil fastDFSUtil;

    @RequestMapping(value = "initial", method = RequestMethod.GET)
    public String initListPage(){
        return "healthbank/activity_list";
    }

    @RequestMapping(value = "infoInit")
    public String initInfoPage(@RequestParam(value = "id") String id,String type){
        request.setAttribute("id",id);
        request.setAttribute("type",type);
        return "healthbank/activity_modify";
    }

    @RequestMapping(value = "init")
    public String init(String type){
        request.setAttribute("type",type);
        return "healthbank/activity_modify";
    }

    @RequestMapping(value ="list",method = RequestMethod.POST)
    @ResponseBody
    public String searchList(
            @RequestParam(name = "activity") String activity,
            @RequestParam(value = "page",required = false) Integer page,
            @RequestParam(value = "rows",required = false) Integer pageSize){
        try{
            JSONObject jsonObject = activityService.searchList(activity, page, pageSize);
            String totalCount = String.valueOf(jsonObject.get("totalCount"));
            Long total = Long.parseLong(totalCount);
            return write(200,"操作成功",page,pageSize,total,(List<ActivityDO>)jsonObject.get("detailModelList"));
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }

    @RequestMapping(value = "findById")
    @ResponseBody
    public String findById(@RequestParam(value = "id") String id){
        try {
            String activity = "{\"id\":\""+ id + "\"}";
            return  write(200,"操作成功","data",activityService.searchList(activity, 1, 1));
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }

    @RequestMapping(value = "update")
    @ResponseBody
    public String update(String jsonData,
                         @RequestParam(value = "value1") String value1,
                         @RequestParam(value = "value2") String value2,
                         @RequestParam(value = "value3") String value3,
                         @RequestParam(value = "registrationNumber")String registrationNumber) {
        try {
            activityService.update(jsonData,value1,value2,value3,registrationNumber);
            return write(200, "操作成功" );
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }

    @RequestMapping(value = "create")
    @ResponseBody
    public String create(@RequestParam(value = "jsonData") String jsonData,
                         @RequestParam(value = "value1") String value1,
                         @RequestParam(value = "value2") String value2,
                         @RequestParam(value = "value3") String value3,
                         @RequestParam(value = "registrationNumber")String registrationNumber) {
        try {
            activityService.create(jsonData,value1,value2,value3,registrationNumber);
            return write(200, "操作成功" );
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }

    @RequestMapping(value = "upload", produces = "application/json;charset=UTF-8")
    @ResponseBody
    public String upload(@RequestParam(value = "file") MultipartFile file, HttpServletRequest request) throws Exception {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        String tempPaths = null;
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile mf = entity.getValue();
            InputStream inputStream  = mf.getInputStream();
            String fileName = mf.getOriginalFilename();
            //文件后缀
            String fileExt = fileName.substring(fileName.lastIndexOf(".") + 1).toLowerCase();
            //文件名称
            String newFileName = mf.getOriginalFilename().substring(0,mf.getOriginalFilename().lastIndexOf("."));
            //上传到fastdfs
            ObjectNode imgNode = fastDFSUtil.upload(inputStream, fileExt, newFileName);
            com.alibaba.fastjson.JSONObject json = com.alibaba.fastjson.JSONObject.parseObject(imgNode.toString());
            tempPaths = json.getString("fid");
        }
        JSONObject json = new JSONObject();
        json.put("status", 200);
        json.put("msg", "上传成功");
        // 图片标识对象的HTTP链接
        json.put("urls", tempPaths);
        return json.toString();
    }

    @RequestMapping(value = "delete")
    @ResponseBody
    public String delete(@RequestParam(value = "id") String id) {
        try {
            String[] ids = {id};
            String s = JSONArray.fromObject(ids).toString();
            activityService.delete(s);
            return write(200, "操作成功" );
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }

    @RequestMapping(value = "deleteById")
    @ResponseBody
    public String deleteById(@RequestParam(value = "id") String id) {
        try {
            activityService.deleteById(id);
            return write(200, "操作成功" );
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }

    @RequestMapping(value = "editActivity")
    @ResponseBody
    public String editActivity(@RequestParam(value = "id") String id,
                               @ApiParam(name = "type",value = "操作类型：上线：0，下线：2，推荐：3")
                               @RequestParam(value = "type") String type) {
        try {
            return write(200, "操作成功","data",activityService.editActivity(id,type) );
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }

    @RequestMapping(value = "findActivityById")
    @ResponseBody
    public String findActivityById(@RequestParam(value = "id") String id) {
        try {
            return write(200, "操作成功" ,"data",activityService.findActivityById(id));
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }

    @RequestMapping(value ="page",method = RequestMethod.POST)
    @ResponseBody
    public String searchList(
            @ApiParam(name = "status", value = "活动状态：1上线，2下线")
            @RequestParam(value = "status", required = false) Integer status,
            @ApiParam(name = "crowdType", value = "人群类型：1:签约居民、2平台用户")
            @RequestParam(value = "crowdType", required = false) Integer crowdType,
            @ApiParam(name = "releaseTime", value = "上线时间")
            @RequestParam(value = "releaseTime", required = false) String releaseTime,
            @ApiParam(name = "activityOfflineTime", value = "下线时间")
            @RequestParam(value = "activityOfflineTime", required = false) String activityOfflineTime,
            @ApiParam(name = "filter", value = "活动名、发布机构、面向范围名称")
            @RequestParam(value = "filter", required = false) String filter,
            @ApiParam(name = "page", value = "第几页，从1开始")
            @RequestParam(value = "page", defaultValue = "1", required = false) Integer page,
            @ApiParam(name = "size", defaultValue = "10", value = "，每页分页大小")
            @RequestParam(value = "size", required = false) Integer size){
        try{
            JSONObject jsonObject = activityService.pageActivity(status, crowdType, releaseTime,activityOfflineTime,filter,page,size);
            return write(200,"操作成功","data",jsonObject);
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }
}
