package  com.yihu.customer.controller.manager.healthbank;

import com.alibaba.fastjson.JSONArray;
import com.fasterxml.jackson.databind.ObjectMapper;
import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.health.HealthPlan;
import com.yihu.customer.service.health.HealthPlanService;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

/**
 * Created by wzn54 on 2019/1/3.
 */
@Controller
@RequestMapping("admin/healthPlan")
@Api(value = "healthPlan", description = "健康计划", tags = {"健康计划"})
public class HealthPlanController extends BaseController {

    @Autowired
    private HealthPlanService planService;
    @Autowired
    private ObjectMapper objectMapper;


    /**
     * 创建计划
     * @param plan
     * @param planTask
     * @return
     */
    @RequestMapping(value ="create",method = RequestMethod.POST)
    @ResponseBody
    public String create(
            @RequestParam(name = "plan") String plan,
            @RequestParam(value = "planTask",required = true) String planTask){
        try{
            JSONArray array  = JSONArray.parseArray(planTask);
            HealthPlan healthPlan = objectMapper.readValue(plan,HealthPlan.class);
            planService.create(healthPlan,array);
            return write(200,"操作成功");
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }


    /**
     * 查询列表
     *
     * @param templateName
     * @param page
     * @param size
     * @return
     */
    @RequestMapping(value ="select",method = RequestMethod.POST)
    @ResponseBody
    public String select(
            @RequestParam(name = "templateName",required = false) String templateName,
            @RequestParam(value = "page", defaultValue = "1",required = false)Integer page,
            @RequestParam(value = "size", required = false)Integer size){
        try{
            return write(200,"查询成功","data",planService.select(templateName,page,size));
        }catch (Exception ex){
            error(ex);
            return error(-1,"查询失败！");
        }
    }


    /**
     * 删除计划任务
     *
     * @param code
     * @return
     */
    @RequestMapping(value ="deleteByCode",method = RequestMethod.POST)
    @ResponseBody
    public String deleteByCode(
            @RequestParam(name = "code",required = true) String code){
        try{
            planService.deleteByCode(code);
            return write(200,"查询成功");
        }catch (Exception ex){
            error(ex);
            return error(-1,"查询失败！");
        }
    }


    /**
     * 编辑计划状态
     *
     * @param code
     * @param status
     * @return
     */
    @RequestMapping(value ="updateStatusByCode",method = RequestMethod.POST)
    @ResponseBody
    public String updateStatusByCode(
            @RequestParam(name = "code",required = true) String code,
            @RequestParam(name = "status",required = true)Integer status){
        try{
            planService.updateStatusByCode(code,status);
            return write(200,"查询成功");
        }catch (Exception ex){
            error(ex);
            return error(-1,"查询失败！");
        }
    }


    /**
     * 编辑计划任务
     *
     * @param plan
     * @param planTask
     * @return
     */
    @RequestMapping(value ="update",method = RequestMethod.POST)
    @ResponseBody
    public String update(
            @RequestParam(name = "plan") String plan,
            @RequestParam(value = "planTask",required = true) String planTask){
        try{
            JSONArray array  = JSONArray.parseArray(planTask);
            HealthPlan healthPlan = objectMapper.readValue(plan,HealthPlan.class);
            planService.update(healthPlan,array);
            return write(200,"操作成功");
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }

    /**
     * 根据code获取计划任务详情
     *
     * @param code
     * @return
     */
    @RequestMapping(value ="selectByCode",method = RequestMethod.POST)
    @ResponseBody
    public String selectByCode(
            @RequestParam(name = "code",required = true) String code){
        try{

            return write(200,"查询成功","data",planService.selectByCode(code));
        }catch (Exception ex){
            error(ex);
            return error(-1,"查询失败！");
        }
    }


}
