package  com.yihu.customer.controller.manager.healthbank;

import com.alibaba.fastjson.JSONArray;
import com.fasterxml.jackson.databind.ObjectMapper;
import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.health.HealthTask;
import  com.yihu.customer.entity.health.HealthTaskQuestion;
import com.yihu.customer.service.health.HeathTaskService;
import io.swagger.annotations.Api;
import jxl.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import java.io.InputStream;
import java.util.List;

/**
 * Created by wzn54 on 2019/1/3.
 */
@Controller
@RequestMapping("admin/healthTask")
@Api(value = "healthTask", description = "健康任务", tags = {"健康任务"})
public class HealthtaskController extends BaseController {

    @Autowired
    private HeathTaskService taskService;
    @Autowired
    private ObjectMapper objectMapper;


    /**
     * 创建健康任务
     *
     * @param task
     * @param question
     * @return
     */
    @RequestMapping(value ="create",method = RequestMethod.POST)
    @ResponseBody
    public String create(
            @RequestParam(name = "task") String task,
            @RequestParam(value = "question",required = true) String question){
        try{
            JSONArray array  = JSONArray.parseArray(question);
            HealthTask healthTask = objectMapper.readValue(task,HealthTask.class);
            taskService.create(healthTask,array);
            return write(200,"操作成功");
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }


    /**
     * 删除健康任务
     *
     * @param code
     * @return
     */
    @RequestMapping(value ="delete",method = RequestMethod.POST)
    @ResponseBody
    public String delete(@RequestParam(name = "code") String code){
        try{
            taskService.delete(code);
            return write(200,"操作成功");
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }

    @RequestMapping(value ="selectByCode",method = RequestMethod.POST)
    @ResponseBody
    public String selectByCode(@RequestParam(name = "code") String code){
        try{
            return write(200,"操作成功","data",taskService.selectByCode(code));
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }


    /**
     * 根据code修改任务状态
     *
     * @param code
     * @return
     */
    @RequestMapping(value ="updateStatusByCode",method = RequestMethod.POST)
    @ResponseBody
    public String updateStatusByCode(@RequestParam(name = "code") String code,
                                     @RequestParam(name = "status") Integer status){
        try{
            taskService.updateStatusByCode(code,status);
            return write(200,"查询成功");
        }catch (Exception ex){
            error(ex);
            return error(-1,"查询失败！");
        }
    }

    /**
     * 获取健康任务列表
     *
     * @param name
     * @param type
     * @param status
     * @return
     */
    @RequestMapping(value ="select",method = RequestMethod.POST)
    @ResponseBody
    public String select(@RequestParam(name = "name",required = false) String name,
                         @RequestParam(name = "type",required = false) Integer type,
                         @RequestParam(name = "status",required = false) Integer status,
                         @RequestParam(value = "page", defaultValue = "1",required = false)Integer page,
                         @RequestParam(value = "size", required = false)Integer size){
        try{
            return write(200,"查询成功","data",taskService.select(type,status,name,page,size));
        }catch (Exception ex){
            error(ex);
            return error(-1,"查询失败！");
        }
    }


    /**
     * 编辑健康任务
     *
     * @param task
     * @param question
     * @return
     */
    @RequestMapping(value ="update",method = RequestMethod.POST)
    @ResponseBody
    public String update(
            @RequestParam(name = "task",required = true) String task,
            @RequestParam(value = "question",required = true) String question){
        try{
            JSONArray array  = JSONArray.parseArray(question);
            HealthTask healthTask = objectMapper.readValue(task,HealthTask.class);
            taskService.update(healthTask,array);
            return write(200,"操作成功");
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }

    //导入健康任务-问题
    @RequestMapping(value = "importData", produces = "application/json;charset=UTF-8")
    @ResponseBody
    public String importData(@RequestParam(name = "file") MultipartFile file) {
        try {
            request.setCharacterEncoding("UTF-8");
            InputStream inputStream = file.getInputStream();
            Workbook workbook = Workbook.getWorkbook(inputStream);
            List<HealthTaskQuestion> healthTaskQuestions= taskService.importData(workbook);
            return write(200, "操作成功","data",healthTaskQuestions);
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }


}
