package  com.yihu.customer.controller.manager.healthchannel;

import  com.yihu.customer.controller.WeixinBaseController;
import  com.yihu.customer.entity.healthchannel.HealthChannelDo;
import com.yihu.customer.service.manager.healthchannel.ChannelService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;

@Controller
@RequestMapping(value = "/admin/channel",produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(description = "健康频道")
public class ChannelController extends WeixinBaseController {

    @Autowired
    private ChannelService channelService;


    @RequestMapping(value = "list", method = RequestMethod.GET)
    @ApiOperation(value = "获取健康频道分类层级列表")
    @ResponseBody
    public String list(
            @ApiParam(name = "level", value = "分类层级:0频道，1推荐栏目分类，2置顶栏目分类", required = true)
            @RequestParam(value = "level", required = false) String level,
            @ApiParam(name = "classification", value = "上级分类id", required = false)
            @RequestParam(value = "classification", required = false) String classification) {
        try {
            List<Map<String,Object>> healthChannelDos = channelService.searchList(level,classification);
            return write(200, "获取成功", "data",healthChannelDos);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "获取失败！");
        }
    }

    @RequestMapping(value = "create", method = RequestMethod.POST)
    @ApiOperation(value = "创建频道分类")
    @ResponseBody
    public String create(
            @ApiParam(name = "jsonData", value = "频道分类：{\"name\":\"健康专栏\",\"code\":\"health_column\",\"classification\":\"4\",\"classificationName\":\"健康专栏\",\"level\":\"1\"}", required = true)
            @RequestParam(value = "jsonData", required = true) String jsonData) {
        try {
            HealthChannelDo healthChannelDo=channelService.create(jsonData, "system");
            return write(200, "创建成功","data",healthChannelDo);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "创建失败");
        }
    }

    @RequestMapping(value = "getChannelList", method = RequestMethod.GET)
    @ApiOperation(value = "获取健康频道列表")
    @ResponseBody
    public String getChannelList(@ApiParam(name = "page",value = "当前页",defaultValue = "1")
                                     @RequestParam(value = "page", required = false) Integer page,
                                 @ApiParam(name = "pageSize",value = "分页大小",defaultValue = "10")
                                     @RequestParam(value = "pageSize", required = false) Integer pageSize) {
        try {
            Map<String,Object> healthChannelDos = channelService.getChannelList(page, pageSize);
            return write(200, "获取成功", "data",healthChannelDos);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "获取失败！");
        }
    }

    @RequestMapping(value = "createChannel", method = RequestMethod.POST)
    @ApiOperation(value = "创建编辑频道")
    @ResponseBody
    public String createChannel(
            @ApiParam(name = "jsonData", value = "频道信息", required = true)
            @RequestParam(value = "jsonData", required = true) String jsonData,
            @ApiParam(name = "userCode", value = "用户code")
            @RequestParam(value = "userCode") String userCode,
            HttpServletRequest req) {
        try {
            if (StringUtils.isBlank(userCode)) {
                userCode = String.valueOf(req.getSession().getAttribute("code"));
            }
            return write(200, "创建成功", "data", channelService.createChannel(jsonData, userCode));
        } catch (Exception ex) {
            error(ex);
            return error(-1, "创建失败");
        }
    }

    @RequestMapping(value = "updateDelFlagById", method = RequestMethod.POST)
    @ApiOperation(value = "更新生效或者失效状态")
    @ResponseBody
    public String updateDelFlagById(
            @ApiParam(name = "channelId", value = "频道id", required = true)
            @RequestParam(value = "channelId") Long channelId,
            @ApiParam(name = "delFlag", value = "是否有效：0失效，1生效", required = true)
            @RequestParam(value = "delFlag") Integer delFlag) {
        try {
            channelService.updateDelFlagById(channelId, delFlag);
            return write(200, "更新成功");
        } catch (Exception ex) {
            error(ex);
            return error(-1, "更新失败");
        }
    }

    @RequestMapping(value = "getRecommendCategoryByChannelId", method = RequestMethod.GET)
    @ApiOperation(value = "根据频道获取推荐或置顶栏目及类别")
    @ResponseBody
    public String getRecommendCategory(
            @ApiParam(name = "channelId", value = "频道id", required = true)
            @RequestParam(value = "channelId") Long channelId,
            @ApiParam(name = "level", value = "分类层级:0频道，1推荐栏目，2文章置顶栏目", required = true)
            @RequestParam(value = "level") String level,
            @ApiParam(name = "code", value = "内容类型code，健康文章传health_article，精选问答是selected_questions_and_answers，" +
                    "平台咨讯platform_information，健康专栏health_column")
            @RequestParam(value = "code",required = false) String code) {
        try {
            return write(200, "获取成功", "data", channelService.getCategoryByCode(channelId + "", level, code));
        } catch (Exception ex) {
            error(ex);
            return error(-1, "获取失败");
        }
    }

    @RequestMapping(value = "getChannelById", method = RequestMethod.GET)
    @ApiOperation(value = "根据id获取频道信息")
    @ResponseBody
    public String getChannelById(
            @ApiParam(name = "channelId", value = "频道id", required = true)
            @RequestParam(value = "channelId") Long channelId) {
        try {
            return write(200, "获取成功", "data", channelService.getChannelById(channelId));
        } catch (Exception ex) {
            error(ex);
            return error(-1, "获取失败");
        }
    }

}
