package  com.yihu.customer.controller.manager.hos;

import com.fasterxml.jackson.databind.ObjectMapper;
import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.organization.HlwyyHospital;
import com.yihu.customer.service.manager.hos.HlwyyHospitalService;
import  com.yihu.customer.repository.organization.HlwyyHospitalDao;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.util.UUID;

/**
 * Created by yeshijie on 2023/2/1.
 */
@Controller
@RequestMapping(value = "/admin/hos/hlwyy")
public class HlwyyHospitalController extends BaseController {

    @Autowired
    private HlwyyHospitalService hlwyyHospitalService;
    @Autowired
    private HlwyyHospitalDao hlwyyHospitalDao;
    @Value("${fastDFS.fastdfs_file_url}")
    private String imgUrlDomain;

    //医院列表页面
    @RequestMapping(value = "initialHospitalList", method = RequestMethod.GET)
    public String listInit() {
        request.setAttribute("imgUrlDomain", imgUrlDomain);
        return "hos/hlwyy/hospital_list";
    }

    //编辑页面
    @RequestMapping(value = "infoInit/{id}", method = RequestMethod.GET)
    public String infoInit(@PathVariable("id") Long id, String mode) {
        if (id == 0) {
            request.setAttribute("UUID", UUID.randomUUID().toString().replace("-", ""));
        }
        request.setAttribute("imgUrlDomain", imgUrlDomain);
        request.setAttribute("id", id);
        request.setAttribute("mode", mode);
        return "hos/hlwyy/hospital_edit";
    }

    //新增 修改
    @RequestMapping(value = "createOrUpdate", method = RequestMethod.POST)
    @ResponseBody
    public String createOrUpdate(String jsonData) {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            HlwyyHospital hospital = objectMapper.readValue(jsonData, HlwyyHospital.class);
            hospital = hlwyyHospitalService.createHospital(hospital);
            return write(200,"操作成功","data",hospital);
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }


    //修改生生效
    @RequestMapping(value = "updDel", method = RequestMethod.POST)
    @ResponseBody
    public String updDel(
            @RequestParam(name = "id") Long id,
            @RequestParam(name = "del") Integer del) {
        try {
            HlwyyHospital hospital = hlwyyHospitalService.getHospital(id);
            if(hospital!=null){
                hospital.setDel(del);
                hlwyyHospitalDao.save(hospital);
            }
            return success("操作成功！");
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }

    //修改推荐
    @RequestMapping(value = "updRecommond", method = RequestMethod.POST)
    @ResponseBody
    public String updRecommond(
            @RequestParam(name = "id") Long id,
            @RequestParam(name = "recommend") Integer recommend) {
        try {
            HlwyyHospital hospital = hlwyyHospitalService.getHospital(id);
            if(hospital!=null){
                hospital.setRecommend(recommend);
                hlwyyHospitalDao.save(hospital);
            }
            return success("操作成功！");
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }

    //根据id获取单个
    @RequestMapping(value = "getHospital", method = RequestMethod.GET)
    @ResponseBody
    public String getHospital(@RequestParam(value = "id") Long id) {
        try {
            HlwyyHospital hospital = hlwyyHospitalService.getHospital(id);
            return write(200, "操作成功！", "hos", hospital);
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }

    //获取列表(分页)
    @RequestMapping(value = "hospitalList", method = RequestMethod.GET)
    @ResponseBody
    public String hospitalList(@ApiParam(name = "name", value = "医院名称", required = false)
                                  @RequestParam(value = "name",required = false) String name,
                                  @ApiParam(name = "page", value = "第几页", required = false)
                                  @RequestParam(value = "page",required = false) Integer page,
                                  @ApiParam(name = "rows", value = "分页大小", required = false)
                                  @RequestParam(value = "rows",required = false) Integer rows) {
        try {
            if(page==null||page<=0){
                page = 1;
            }
            if(rows==null||rows<=0){
                rows = 10;
            }
            Page<HlwyyHospital> hospitals = hlwyyHospitalService.searchHospitals(name, page, rows);
            return write(200, "操作成功！", page, rows, hospitals);
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }
}
