package  com.yihu.customer.controller.manager.hos;

import com.fasterxml.jackson.databind.ObjectMapper;
import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.Doctor;
import  com.yihu.customer.entity.Hospital;
import com.yihu.customer.service.manager.hos.HosService;
import  com.yihu.customer.repository.DoctorDao;
import  com.yihu.customer.repository.HospitalDao;
import jxl.Workbook;
import jxl.read.biff.BiffException;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * 医院信息管理
 * Created by yww on 2016/9/27.
 */
@Controller
@RequestMapping(value = "/admin/hos")
public class HosController extends BaseController {
    @Autowired
    private HosService hospitalService;
    @Autowired
    private HospitalDao hospitalDao;
    @Autowired
    private DoctorDao doctorDao;

    @RequestMapping(value = "resetPwdInit")
    public String resetPwdInit() {

        return "hos/reset_pwd";
    }

    //页面跳转（主页面）
    @RequestMapping(value = "initial")
    public String listInit() {
        return "hos/hospital_list";
    }


    //页面跳转（详情页面）
    @RequestMapping(value = "infoInit/{id}", method = RequestMethod.GET)
    public String infoInit(@PathVariable("id") Long id, String mode) {

        if (id == 0) {
            request.setAttribute("UUID", UUID.randomUUID().toString().replace("-", ""));
        }
        request.setAttribute("id", id);
        request.setAttribute("mode", mode);
        return "hos/hospital_edit";
    }

    //新增
    @RequestMapping(value = "create")
    @ResponseBody
    public String createHospital(String jsonData) {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            Hospital hospital = objectMapper.readValue(jsonData, Hospital.class);
            hospitalService.createHospital(hospital);
            return success("操作成功！");
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }

    }

    //修改
    @RequestMapping(value = "update")
    @ResponseBody
    public String updateHospital(String jsonData) {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            Hospital hospital = objectMapper.readValue(jsonData, Hospital.class);
            hospitalService.createHospital(hospital);
            return success("操作成功！");
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }


    }

    //修改
    @RequestMapping(value = "updateDel")
    @ResponseBody
    public String updateHospitalStatus(
            @RequestParam(name = "id") Long id,
            @RequestParam(name = "del") String del) {
        try {
            boolean res = hospitalService.updateDel(id, del);
            return res == true ? success("操作成功！") : error(-1, "操作失败！");
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }

    //根据id删除
    @RequestMapping(value = "delete")
    @ResponseBody
    public String deleteHospital(@RequestParam(value = "id") Long id) {
        try {
            hospitalService.deleteHospital(id);
            return success("操作成功！");
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }

    //根据id获取单个
    @RequestMapping(value = "hospital")
    @ResponseBody
    public String getHospital(@RequestParam(value = "id") Long id) {
        try {
            Hospital hospital = hospitalService.getHospital(id);
            return write(200, "操作成功！", "hos", hospital);
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }

    //获取列表(不分页)
    @RequestMapping(value = "hospitalsNoPage")
    @ResponseBody
    public String searchHospitalNoPage(@RequestParam(value = "name", required = false) String name,
                                       @RequestParam(value = "level",required = false)String level) {
        try {
            List<Hospital> hospitals = hospitalService.searchHospitalNoPage(name,level);
            return write(200, "操作成功！", "hoss", hospitals);
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }


    //获取列表type = 1 根据城市获取  type = 2 根据区获取
    @RequestMapping(value = "hospitalsByType")
    @ResponseBody
    public String getHospitalByType(@RequestParam(value = "type", required = true) Integer type,@RequestParam(value = "code", required = true) String code) {
        try {
            List<Hospital> hospitals = hospitalService.getHospitalByType(type,code);
            return write(200, "操作成功！", "list", hospitals);
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }

    //获取列表(分页)
    @RequestMapping(value = "hospitalList")
    @ResponseBody
    public String searchHospitals(String name, String doctorName, String doctorMobile, int page, int rows) {
        try {
            Page<Hospital> hospitals = hospitalService.searchHospitals(name, doctorName, doctorMobile, page, rows);
            return write(200, "操作成功！", page, rows, hospitals);
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }

    //医院名字是否存在
    @RequestMapping(value = "isNameExist")
    @ResponseBody
    public String isNameExist(String name) {
        try {
            return hospitalService.isNameExist(name);
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }

    }

    /**
     * 按城市查找社区
     *
     * @param city
     * @return
     */
    @RequestMapping(value = "community")
    @ResponseBody
    public String community(String city) {
        try {
            List<Map<String, String>> roleMap = (List<Map<String, String>>) request.getSession().getAttribute("roleMap");
            Map<String, JSONObject> map = new HashMap<String, JSONObject>();
            if (roleMap.size() > 0) {//管理员
                List<Hospital> list = hospitalDao.findByCity(city);
                if (list != null && list.size() > 0) {
                    String areaString = "";
                    String communityString = "";
                    Boolean cityFlag = false;
                    for (Map<String, String> rmap : roleMap) {
                        String code = rmap.get("code");
                        if ("350200".equals(code)) {
                            cityFlag = true;
                            break;
                        } else if (code.length() == 6) {
                            areaString += rmap.get("areas") + ",";
                        } else {
                            communityString += rmap.get("areas") + ",";
                        }
                    }
                    for (Hospital hos : list) {
                        String town = hos.getTown() + ":" + hos.getTownName();
                        JSONArray jsonArray = null;
                        JSONObject jsonObject = null;
                        String hosCode = hos.getCode();
                        String hosName = hos.getName();

                        if (!cityFlag) {
                            if (areaString.length() > 0) {
                                if (areaString.indexOf(hos.getTown() + ",") < 0) {
                                    continue;
                                }
                            } else {
                                if (communityString.indexOf(hosCode + ",") < 0) {
                                    continue;
                                }
                            }
                        }

                        JSONObject json = new JSONObject();
                        json.put("code", hosCode);
                        json.put("name", hosName);

                        if (map.containsKey(town)) {
                            jsonObject = map.get(town);
                            jsonArray = jsonObject.getJSONArray(town);
                            jsonArray.put(json);
                        } else {
                            jsonObject = new JSONObject();
                            jsonArray = new JSONArray();
                            jsonArray.put(json);
                            jsonObject.put(town, jsonArray);
                            map.put(town, jsonObject);
                        }
                    }
                }
            } else {
                //医生
                String mobile = (String) request.getSession().getAttribute("mobile");
                Doctor doctor = doctorDao.findByMobile(mobile);
                Hospital hos = hospitalDao.findByCode(doctor.getHospital());

                String town = hos.getTown() + ":" + hos.getTownName();
                JSONArray jsonArray = new JSONArray();
                ;
                JSONObject jsonObject = new JSONObject();
                ;
                JSONObject json = new JSONObject();
                json.put("code", hos.getCode());
                json.put("name", hos.getName());
                jsonArray.put(json);
                jsonObject.put(town, jsonArray);
                map.put(town, jsonObject);
            }

            JSONArray jsonArray = new JSONArray();
            for (String key : map.keySet()) {
                JSONObject json = new JSONObject();
                json.put("town", key);
                json.put("community", map.get(key).get(key));
                jsonArray.put(json);
            }
            return write(200, "查询成功！", "list", jsonArray);
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败！");
        }
    }

    //医院名字是否存在
    @RequestMapping(value = "isCcodeExist")
    @ResponseBody
    public String isCodeExist(String code) {
        try {

            return hospitalService.isCodeExist(code);
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }

    }

    @RequestMapping(value = "importData", produces = "application/json;charset=UTF-8")
    @ResponseBody
    public String importData(HttpServletRequest request) {
        try {
            request.setCharacterEncoding("UTF-8");
            InputStream inputStream = request.getInputStream();
            Workbook workbook = Workbook.getWorkbook(inputStream);

            hospitalService.importData(workbook);

            return write(200, "操作成功");
        } catch (IOException | BiffException e) {
            e.printStackTrace();
            error(e);
            return error(-1, "操作失败！");
        }
    }

}
