package  com.yihu.customer.controller.manager.hos;

import com.fasterxml.jackson.databind.ObjectMapper;
import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.WlyyOrgAddress;
import com.yihu.customer.service.manager.hos.HosPharmacyService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;


/**
 * 药房管理
 * Created by Bing on 2021/5/27.
 */
@Controller
@RequestMapping(value = "/admin/hos/pharmacy")
@Api(description = "社区机构药房管理")
public class HosPharmacyController extends BaseController {

    @Autowired
    private HosPharmacyService hosPharmacyService;

    //页面跳转（主页面）
    @RequestMapping(value = "initial",method = RequestMethod.GET)
    public String listInit(Model model, @RequestParam(required = true) String hospitalCode) {
        model.addAttribute("hospitalCode", hospitalCode);
        return "hos/hospital_pharmacy_list";
    }

    //机构列表药房页面
    @RequestMapping(value = "pharmacyList", method = {RequestMethod.GET,RequestMethod.POST})
    @ResponseBody
    public String pharmacyList(@RequestParam(value = "hospitalCode", required = true) String hospitalCode,
                               @RequestParam(value = "name", required = false) String name,
                               @RequestParam(value = "page", required = true) Integer page,
                               @RequestParam(value = "rows", required = true) Integer rows) {
        try {
            if (StringUtils.isBlank(hospitalCode)){
                return error(-1, "所属机构编码不能为空！");
            }
            Page<WlyyOrgAddress> list = hosPharmacyService.pharmacyList(hospitalCode,name,page,rows);
            return write(200, "查询成功", page, rows, list);
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败！");
        }
    }

    //编辑页面
    @RequestMapping(value = "updateInit/{id}", method = RequestMethod.GET)
    public String update(@PathVariable("id") Long id) {
        try {
            request.setAttribute("id", id);
            request.setAttribute("type", request.getParameter("type"));
        } catch (Exception e) {
            error(e);
        }
        return "hos/hospital_pharmacy_edit";
    }

    @RequestMapping(value = "update",method = RequestMethod.POST)
    @ResponseBody
    public String updateOrgAddress(String jsonData) {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            WlyyOrgAddress orgAddress = objectMapper.readValue(jsonData, WlyyOrgAddress.class);
            hosPharmacyService.updateOrgAddress(orgAddress);
            return success("操作成功！");
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败!" + e.getMessage());
        }
    }

    @RequestMapping(value = "updateStatus", method = RequestMethod.POST)
    @ResponseBody
    public String updateStatus(
            @RequestParam(value = "id", required = true) Long id,
            @RequestParam(value = "del", required = true) Integer del) {
        try {
            if (id == null) {
                return error(-1, "id不能为空！！");
            }

            if (del == null) {
                return error(-1, "状态不能为空！！");
            }

            boolean res = hosPharmacyService.updateDel(del, id);
            return res == true ? write(200, "操作成功！") : error(-1, "操作失败！");
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }

    @RequestMapping(value = "findOne",method = RequestMethod.GET)
    @ResponseBody
    public String findOne(
            @RequestParam(value = "id", required = true) Long id) {
        try {
            WlyyOrgAddress doctor = hosPharmacyService.findOne(id);
            return write(200, "操作成功！", "pharmacy", doctor);
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }

    @RequestMapping(value = "getHlwDevices",method = RequestMethod.GET)
    @ApiOperation("获取社区智能药柜设备列表")
    @ResponseBody
    public String getHlwDevices(
            @RequestParam(value = "orgCode", required = true) String orgCode) {
        try {

            return write(200, "操作成功！", "list", hosPharmacyService.getHlwDevices(orgCode));
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }

}
