package  com.yihu.customer.controller.manager.notification;

import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.wechat.WechatTemplateConfig;
import com.yihu.customer.service.manager.wechat.WechatTemplateService;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;
import java.util.Map;

/**
* @Description: 微信模板管理
* @Author: WuJunjie
* @Date: Created in 2018/3/20 9:07
*/
@Controller
@RequestMapping("/admin/wechatTemplate")
public class WechatTemplateController extends BaseController {
    @Autowired
    private WechatTemplateService wechatTemplateService;

    /**
     * 列表页面初始化数据
     * @param model
     * @return
     */
    @RequestMapping(value = "initial", method = RequestMethod.GET)
    @ApiIgnore
    public String listInitial(Model model) {
        return "wechat/wechat_list";
    }

    /**
     * 编辑、查看、新增页面初始化数据
     * @param
     * @return
     */
    @RequestMapping(value = "editInitial", method = RequestMethod.GET)
    @ApiIgnore
    public String editInitial(String templateName,String scene,String mode) {
        request.setAttribute("templateName",templateName);
        request.setAttribute("scene",scene);
        request.setAttribute("mode",mode);
        return "wechat/wechat_edit";
    }

    /**
     * 根据模板自定义名称和场景值查询模板消息
     * @param templateName
     * @param scene
     * @param page
     * @param pageSize
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "list", method = RequestMethod.POST)
    @ResponseBody
    public String searchNotificationList(
            @ApiParam(name = "templateName", value = "模板名称")
            @RequestParam(value = "templateName", required = false) String templateName,
            @ApiParam(name = "scene", value = "模板场景值")
            @RequestParam(value = "scene", required = false) String scene,
            @ApiParam(name = "page")
            @RequestParam(value = "page") int page,
            @ApiParam(name = "rows")
            @RequestParam(value = "rows") int pageSize) {
        try {
            List<WechatTemplateConfig> templetes = wechatTemplateService.searchList(templateName, scene, page, pageSize);
            int count = wechatTemplateService.getListCount(templateName, scene);
            return write(200, "操作成功", page, pageSize,count, templetes);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败");
        }
    }

    /**
     * 根据模板自定义名称和场景值查询模板消息详情
     * @param templateName
     * @param scene
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "getTemplate", method = RequestMethod.POST)
    @ResponseBody
    public String getTemplate(
            @ApiParam(name = "templateName", value = "模板名称")
            @RequestParam(value = "templateName", required = true) String templateName,
            @ApiParam(name = "scene", value = "模板场景值")
            @RequestParam(value = "scene", required = true) String scene) {
        try {
            List<Map<String,Object>> templete = wechatTemplateService.getTemplate(templateName, scene);
            return write(200, "操作成功", "data",templete);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败");
        }
    }

    /**
     * 修改模板详情
     * @param templateName
     * @param scene
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "updateTemplate", method = RequestMethod.POST)
    @ResponseBody
    public String updateTemplate(
            @ApiParam(name = "templateName", value = "模板名称")
            @RequestParam(value = "templateName", required = true) String templateName,
            @ApiParam(name = "scene", value = "模板场景值")
            @RequestParam(value = "scene", required = true) String scene,
            @ApiParam(name = "sceneDescription", value = "模板场景描述")
            @RequestParam(value = "sceneDescription", required = true) String sceneDescription,
            @ApiParam(name = "first", value = "first")
            @RequestParam(value = "first", required = false) String first,
            @ApiParam(name = "remark", value = "remark")
            @RequestParam(value = "remark", required = false) String remark,
            @ApiParam(name = "value1", value = "value1")
            @RequestParam(value = "value1", required = false) String value1,
            @ApiParam(name = "value2", value = "value2")
            @RequestParam(value = "value2", required = false) String value2,
            @ApiParam(name = "value3", value = "value3")
            @RequestParam(value = "value3", required = false) String value3,
            @ApiParam(name = "value4", value = "value4")
            @RequestParam(value = "value4", required = false) String value4,
            @ApiParam(name = "value5", value = "value5")
            @RequestParam(value = "value5", required = false) String value5,
            @ApiParam(name = "value6", value = "value6")
            @RequestParam(value = "value6", required = false) String value6,
            @ApiParam(name = "value7", value = "value7")
            @RequestParam(value = "value7", required = false) String value7) {
        try {
            wechatTemplateService.updateTemplate(templateName, scene,sceneDescription,first,remark,
                    value1,value2,value3,value4,value5,value6,value7);
            return write(200, "操作成功", "data","");
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败");
        }
    }

    /**
     * 新增模板详情
     * @param templateName
     * @param scene
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "saveTemplate", method = RequestMethod.POST)
    @ResponseBody
    public String saveTemplate(
            @ApiParam(name = "templateName", value = "模板名称")
            @RequestParam(value = "templateName", required = true) String templateName,
            @ApiParam(name = "scene", value = "模板场景值")
            @RequestParam(value = "scene", required = true) String scene,
            @ApiParam(name = "sceneDescription", value = "模板场景描述")
            @RequestParam(value = "sceneDescription", required = true) String sceneDescription,
            @ApiParam(name = "first", value = "first")
            @RequestParam(value = "first", required = false) String first,
            @ApiParam(name = "remark", value = "remark")
            @RequestParam(value = "remark", required = false) String remark,
            @ApiParam(name = "value1", value = "value1")
            @RequestParam(value = "value1", required = false) String value1,
            @ApiParam(name = "value2", value = "value2")
            @RequestParam(value = "value2", required = false) String value2,
            @ApiParam(name = "value3", value = "value3")
            @RequestParam(value = "value3", required = false) String value3,
            @ApiParam(name = "value4", value = "value4")
            @RequestParam(value = "value4", required = false) String value4,
            @ApiParam(name = "value5", value = "value5")
            @RequestParam(value = "value5", required = false) String value5,
            @ApiParam(name = "value6", value = "value6")
            @RequestParam(value = "value6", required = false) String value6,
            @ApiParam(name = "value7", value = "value7")
            @RequestParam(value = "value7", required = false) String value7) {
        try {
            Map<String,Object> result = wechatTemplateService.saveTemplate(templateName, scene,sceneDescription,first,remark,
                    value1,value2,value3,value4,value5,value6,value7);
            int code = Integer.parseInt(result.get("code").toString());
            String msg = result.get("msg").toString();
            return write(code, msg);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败");
        }
    }

}
