package  com.yihu.customer.controller.manager.patient;

import com.fasterxml.jackson.databind.ObjectMapper;
import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.Patient;
import com.yihu.customer.service.manager.patient.AdminPatientService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;

import java.util.UUID;

/**
 * Created by yww on 2016/10/8.
 * 后台管理 患者信息维护
 */
@Controller
@RequestMapping(value="admin/patient")
public class AdminPatientController extends BaseController {
    @Autowired
    private AdminPatientService adminPatientService;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private ObjectMapper objectMapper;

    //患者列表页跳转
    @RequestMapping(value = "initial", method = RequestMethod.GET)
    public String listInit(){
        return "patient/patient_list";
    }

    //患者详情页跳转（uuid)
    @RequestMapping(value="infoInit/{id}",method = RequestMethod.GET)
    public String InfoInit(Model model, @PathVariable("id") Long id, String mode) throws Exception {
        model.addAttribute("patient", "{}");
        if(id==0){
            request.setAttribute("UUID", UUID.randomUUID().toString().replace("-", ""));
            model.addAttribute("UUID", UUID.randomUUID().toString().replace("-", ""));
        }
        if (id > 0) {
            Patient patient = adminPatientService.getPatient(id);
            if (patient != null)
                model.addAttribute("patient", objectMapper.writeValueAsString(patient));
        }
        request.setAttribute("mode", mode);
        request.setAttribute("id",id);
        return "patient/patient_modify";
    }

    //获取患者列表
    @RequestMapping(value = "patientList")
    @ResponseBody
    public String searchPatientList(
            String name,String idcard,String ssc,String mobile,Integer openIdStatus,String openid, int page,int rows){
        try {
            Page<Patient> patients = adminPatientService.searchPatientList(name, idcard, ssc,mobile,openIdStatus, openid, page, rows);
            return write(200,"操作成功",page,rows,patients);
        }catch (Exception ex){
            error(ex);
            return error(-1, "操作失败！");
        }
    }

    //根据id获取
    @RequestMapping(value = "patient")
    @ResponseBody
    public String getPatient(Long id){
        try{
            Patient patient = adminPatientService.getPatient(id);
            return write(200,"操作成功","patient",patient);
        }catch (Exception ex){
            error(ex);
            return error(-1, "操作失败！");
        }
    }

    //新增
    @RequestMapping(value = "create")
    @ResponseBody
    public String createPatient(String jsonData){
        try{
            ObjectMapper objectMapper = new ObjectMapper();
            Patient patient = objectMapper.readValue(jsonData,Patient.class);
            String msg = "";
            if(StringUtils.isEmpty(patient.getName())){
                msg += "姓名不能为空；";
            }
            if(StringUtils.isEmpty(patient.getIdcard())){
                msg += "身份证不能为空；";
            }
            if(StringUtils.isEmpty(patient.getSsc())){
                msg += "社保卡不能为空；";
            }
            if(!StringUtils.isEmpty(msg)){
                return error(-1,msg);
            }
//            String idcard = patient.getIdcard();
//            if(idcard == null){
//                return error(-1,"身份证不能为空！");
//            }
//            //姓名+身份证号+社保卡号 检验
//            SocialSecurityInfo info = adminPatientService.findSocialSecurityInfo(idcard);
//            boolean flag = true;
//            if(StringUtils.equals(info.getXming0(),patient.getName())){
//                //名字不符
//                flag = false;
//            }
//            if(StringUtils.equals(info.getCardno(),patient.getSsc()) || StringUtils.equals(info.getCard16(),patient.getSsc())){
//                //社保卡不符（两种社保卡形式）
//                flag = false;
//            }
//            if(!flag){
//                return error(-1, "姓名，身份证，社保卡信息不对应，请核对！");
//            }
            adminPatientService.createPatient(patient);
            return write(200,"操作成功");
        }catch (Exception ex){
            error(ex);
            return error(-1, "操作失败！");
        }
    }

    //修改
    @RequestMapping(value = "update")
    @ResponseBody
    public String updatePatient(String jsonData){
        try{
            ObjectMapper objectMapper = new ObjectMapper();
            Patient patient = objectMapper.readValue(jsonData,Patient.class);
            String msg = "";
            if(patient.getId() ==0){
                msg += "id不能为空；";
            }
            if(StringUtils.isEmpty(patient.getName())){
                msg += "姓名不能为空；";
            }
            if(StringUtils.isEmpty(patient.getIdcard())){
                msg += "身份证不能为空；";
            }
            if(StringUtils.isEmpty(patient.getSsc())){
                msg += "社保卡不能为空；";
            }
            if(!StringUtils.isEmpty(msg)){
                return error(-1,msg);
            }
//            String idcard = patient.getIdcard();
//            if(idcard == null){
//                return error(-1,"身份证不能为空！");
//            }
//            //姓名+身份证号+社保卡号 检验
//            SocialSecurityInfo info = adminPatientService.findSocialSecurityInfo(idcard);
//            if(info == null){
//                return error(-1,"根据该身份证，没有找到相关的社保信息！");
//            }
//            boolean flag = true;//是否符合
//            if(!StringUtils.equals(info.getXming0().trim(),patient.getName().trim())){
//                //名字不符
//                flag = false;
//            }
//            if(!(StringUtils.equals(info.getCardno().trim(),patient.getSsc().trim()) || StringUtils.equals(info.getCard16().trim(),patient.getSsc().trim()))){
//                //社保卡不符（两种社保卡形式）
//                flag = false;
//            }
//            if(!flag){
//                return error(-1,"姓名，身份证，社保卡信息不对应，请核对！");
//            }
            adminPatientService.updatePatient(patient);
            return write(200,"操作成功");
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }

    @RequestMapping(value = "updatePatientBaseInfo2")
    @ResponseBody
    public String updatePatientBaseInfo2(@RequestParam(value = "id")String id,
                                         @RequestParam(value = "name",required = false)String name,
                                         @RequestParam(value = "idcard",required = false)String idcard,
                                         @RequestParam(value = "ssc",required = false)String ssc){
        try{
            if (StringUtils.isBlank(id)){
                return error(-1,"操作失败！");
            }
            adminPatientService.updatePatientBaseInfo2(id,name,idcard,ssc);
            return write(200,"操作成功");
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }

    //根据删除
    @RequestMapping(value = "delete")
    @ResponseBody
    public String deletePatient(Long id){
        try{
            adminPatientService.deletePatient(id);
            return write(200,"操作成功");
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }

    //患者死亡
    @RequestMapping(value = "death")
    @ResponseBody
    public String death(Long id){
        try{
            adminPatientService.death(id);
            return write(200,"操作成功");
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }


    //手机号唯一
    @RequestMapping(value = "isMoblieExist")
    @ResponseBody
    public String isMobileExist(String mobile){
        try{
            return adminPatientService.isMobileExist(mobile);
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }
    //身份证号唯一
    @RequestMapping(value = "isIdCardExist")
    @ResponseBody
    public String isIdCardExist(String idcard){
        try{
            Patient patient = adminPatientService.findByIdcard(idcard);
            if (patient==null){
                return "false";
            }else {
                return 1==patient.getStatus()?"true":"false";
            }
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }
    //社保卡号唯一
    @RequestMapping(value = "isSscExist")
    @ResponseBody
    public String isSscExist(String ssc){
        try{
            Patient patient = adminPatientService.findBySsc(ssc);
            if (patient==null){
                return "false";
            }else {
                return 1==patient.getStatus()?"true":"false";
            }
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }
    //初始化密码
    @RequestMapping(value = "initPassword")
    @ResponseBody
    public String initPassword(
            @RequestParam(value = "patientId", required = true) Long patientId,
            @RequestParam(value = "idCard", required = true) String idCard) {
        try {
            boolean res = adminPatientService.initPassword(patientId, idCard);
            return res == true?write(200, "操作成功！"):error(-1, "操作失败！");
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }

    @RequestMapping(value = "updateStatus",method = RequestMethod.POST)
    @ResponseBody
    public String updateStatus(
            @RequestParam(value = "patientId", required = true) Long patientId,
            @RequestParam(value = "status", required = true) Integer status) {
        try {
            boolean res = adminPatientService.updataStatus(status, patientId);
            return res == true?write(200, "操作成功！"):error(-1, "操作失败！");
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }

    /**
     * 解锁账户
     */
    @RequestMapping(value = "unlodckUser",method = RequestMethod.POST)
    @ResponseBody
    public String unlodckUser(
            @RequestParam(value = "userCode", required = true) String userCode,
            @RequestParam(value = "type", required = true) String type) {
        try {
            String sql = "delete from wlyy_user_login_log where user_code ='"+userCode+"' and login_type='"+type+"'";
            jdbcTemplate.execute(sql);
            return write(200, "操作成功！");
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }
}
