package  com.yihu.customer.controller.manager.patient;

import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.copd.CopdPatient;
import com.yihu.customer.service.manager.patient.CopdPatientService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import jxl.Workbook;
import jxl.write.Label;
import jxl.write.WritableSheet;
import jxl.write.WritableWorkbook;
import jxl.write.WriteException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.util.List;
import java.util.Map;

/**
 * Created by Bing on 2022/1/16.
 */
@Controller
@Api(description = "慢阻肺居民")
@RequestMapping(value = "admin/copd",produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
public class CopdPatientController extends BaseController {

    @Autowired
    private CopdPatientService copdPatientService;

    @RequestMapping(value = "initial", method = RequestMethod.GET)
    public String listInit(){
        return "patient/copd_patient_list";
    }

    @RequestMapping(value = "addInit", method = RequestMethod.GET)
    public String addInit(){
        return "patient/copd_patient_add";
    }

    //编辑页面
    @RequestMapping(value = "updateInit/{patient}", method = RequestMethod.GET)
    public String updateInit(@PathVariable("patient") String patient,
                             @RequestParam(value = "hospital",required = false)String hospital) {
        try {
            request.setAttribute("patient", patient);
            request.setAttribute("hospitalId", hospital);
            request.setAttribute("type", request.getParameter("type"));
        } catch (Exception e) {
            error(e);
        }
        return "patient/copd_patient_modify";
    }

    @RequestMapping(value = "getPatientList", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation("获取慢阻肺患者列表")
    public String getPatientList(@RequestParam(value = "name", required = false)String name,
                                 @RequestParam(value = "idcard", required = false)String idcard,
                                 @RequestParam(value = "mobile", required = false)String mobile,
                                 @RequestParam(value = "hospitalName", required = false)String hospitalName,
                                 @RequestParam(value = "doctorName", required = false)String doctorName,
                                 @RequestParam(value = "page", required = true) Integer page,
                                 @RequestParam(value = "rows", required = true) Integer rows) {
        try {
            Page<CopdPatient> result = copdPatientService.getPatientList(name,idcard,mobile,hospitalName,doctorName,page,rows);
            return write(200, "查询成功", page, rows, result);
        } catch (Exception e) {
            error(e);
            return error(-1,"查询失败");
        }
    }

    @RequestMapping(value = "updateCopdPatient", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation("保存慢阻肺患者修改")
    public String updateCopdPatient(@RequestParam(value = "json", required = false)String json) {
        try {
            return write(200, "保存成功", "data",copdPatientService.updateCopdPatient(json));
        } catch (Exception e) {
            error(e);
            return error(-1,e.getMessage());
        }
    }

    @RequestMapping(value = "getCopdPatientInfo", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation("获取慢阻肺患者列表")
    public String getCopdPatientInfo(@RequestParam(value = "patient", required = false)String patient) {
        try {
            CopdPatient copdPatient = copdPatientService.getCopdPatientInfo(patient);
            if (null==copdPatient){
                return error(-1,"未获取到慢阻肺居民信息");

            }
            return write(200, "查询成功", "data",copdPatient);
        } catch (Exception e) {
            error(e);
            return error(-1,"查询失败");
        }
    }

    @RequestMapping(value = "initCopdPatient", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation("慢阻肺居民还原（未管理状态）")
    public String initCopdPatient(@RequestParam(value = "patient", required = false)String patient) {
        try {
            return write(200, "操作成功", "data",copdPatientService.initCopdPatient(patient));
        } catch (Exception e) {
            error(e);
            return error(-1,"操作失败");
        }
    }

    @RequestMapping(value = "initSecondScreening", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation("慢阻肺居民二次排纳还原")
    public String initSecondScreening(@RequestParam(value = "patient", required = false)String patient) {
        try {
            return write(200, "操作成功", "data",copdPatientService.initSecondScreening(patient));
        } catch (Exception e) {
            error(e);
            return error(-1,"操作失败");
        }
    }

    @RequestMapping(value = "delCopdPatient", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation("慢阻肺居民删除")
    public String delCopdPatient(@RequestParam(value = "patient", required = false)String patient) {
        try {
            return write(200, "删除成功", "data",copdPatientService.delCopdPatient(patient));
        } catch (Exception e) {
            error(e);
            return error(-1,"删除失败");
        }
    }

    @RequestMapping(value = "addCopdPatient", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation("添加慢阻肺居民")
    public String delCopdPatient(@RequestParam(value = "idcard", required = false)String idcard,
                                 @RequestParam(value = "hospital", required = false)String hospital,
                                 @RequestParam(value = "doctor", required = false)String doctor) {
        try {
            return write(200, "添加成功", "data",copdPatientService.addCopdPatient(idcard,hospital,doctor));
        } catch (Exception e) {
            error(e);
            return error(-1,e.getMessage());
        }
    }

    @RequestMapping(value = "getFollowupList",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation("获取慢阻肺居民随访计划列表")
    public String getFollowupList(@RequestParam(value = "patient", required = false)String patient) {
        try {
            return write(200, "添加成功", "data",copdPatientService.getFollowupList(patient));
        } catch (Exception e) {
            error(e);
            return error(-1,e.getMessage());
        }
    }

    @RequestMapping(value = "modifyFollow")
    @ResponseBody
    @ApiOperation("修改居民慢阻肺计划")
    public String modifyFollow(@RequestParam(value = "json", required = false)String json) {
        try {
            return write(200, "添加成功", "data",copdPatientService.modifyFollow(json));
        } catch (Exception e) {
            error(e);
            return error(-1,e.getMessage());
        }
    }

    //编辑页面
    @RequestMapping(value = "followupListInit/{patient}", method = RequestMethod.GET)
    public String followupListInit(@PathVariable("patient") String patient,
                             @RequestParam(value = "hospital",required = false)String hospital) {
        try {
            request.setAttribute("patient", patient);
            request.setAttribute("type", request.getParameter("type"));
        } catch (Exception e) {
            error(e);
        }
        return "patient/copd_followup_modify";
    }

    @RequestMapping(value = "exportExcel", method = RequestMethod.POST)
    @ApiOperation("导出慢阻肺患者列表")
    @ResponseBody
    public void exportList(
            @RequestParam(value = "name", required = false)String name,
            @RequestParam(value = "idcard", required = false)String idcard,
            @RequestParam(value = "mobile", required = false)String mobile,
            @RequestParam(value = "hospitalName", required = false)String hospitalName,
            @RequestParam(value = "doctorName", required = false)String doctorName,
            HttpServletResponse response) {
        try {
            List<Map<String, Object>> list = copdPatientService.exportList(name,idcard,mobile,hospitalName,doctorName);
            response.setContentType("octets/stream");
            response.setHeader("Content-Disposition", "attachment; filename=" + new String("copdPatientList.xls"));
            OutputStream os = response.getOutputStream();
            this.write(os, list);
        } catch (Exception ex) {
            error(ex);
        }
    }

    private void write(OutputStream os, List<Map<String, Object>> list) throws Exception {
        WritableWorkbook wwb = Workbook.createWorkbook(os);
        try {
            WritableSheet sheet = wwb.createSheet("sheet", 1);
            String header[] = {"姓名", "性别", "身份证号", "手机号", "社保卡", "关联机构", "关联医生", "分组", "状态"};
            int i = 0;
            for (String h : header) {
                addCell(sheet, 0, i, h);
                i++;
            }
            int j = 1;
            for (Map<String, Object> map : list) {
                addCell(sheet, j, 0, map.get("name") + "");
                addCell(sheet, j, 1, null==map.get("sex_name")?"未知":"1".equals(map.get("sex_name").toString())?"男":"女");
                addCell(sheet, j, 2, map.get("idcard") + "");
                addCell(sheet, j, 3, map.get("phone") + "");
                addCell(sheet, j, 4, map.get("ssc") + "");
                addCell(sheet, j, 5, map.get("hospital_name") + "");
                addCell(sheet, j, 6, map.get("doctor_name") + "");
                addCell(sheet, j, 7, null==map.get("group_screening_status")?"未分组":"1".equals(map.get("group_screening_status").toString())?"随访观察组":"肺康复锻炼组");
                String status = map.get("status").toString();
                String statusName="";
                switch (status){
                    case "0": statusName="未管理";break;
                    case "1": statusName="已管理";break;
                    case "2": statusName="失访";break;
                    case "4": statusName="不纳入管理";break;
                    case "8": statusName="死亡";break;

                }
                addCell(sheet, j, 8, statusName);
                j++;
            }
            wwb.write();
            wwb.close();
        } catch (Exception e) {
            e.printStackTrace();
            if (wwb != null) wwb.close();
        }
    }

    private void addCell(WritableSheet ws, int row, int column, String data) throws WriteException {
        Label label = new Label(column, row, data);
        ws.addCell(label);
    }
}
