package  com.yihu.customer.controller.manager.pneumonia;

import com.fasterxml.jackson.databind.ObjectMapper;
import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.pneumonia.PneumoniaAttackAddress;
import  com.yihu.customer.entity.pneumonia.PneumoniaAttackArea;
import  com.yihu.customer.entity.pneumonia.PneumoniaDataNew;
import com.yihu.customer.service.pneumonia.PneumoniaAttackAreaService;
import com.yihu.customer.service.pneumonia.PneumoniaService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;


/**
 * Created by Bing on 2021/8/2.
 * 新冠肺炎相关
 */
@Controller
@RequestMapping("/admin/pneumonia")
public class PneumoniaController extends BaseController {

    @Autowired
    private PneumoniaService pneumoniaService;
    @Autowired
    private PneumoniaAttackAreaService pneumoniaAttackAreaService;

    //页面跳转（主页面）
    @RequestMapping(value = "data/initial",method = RequestMethod.GET)
    public String dataInit() {
        return "pneumonia/pneumonia_data_info";
    }

    //页面跳转（主页面）
    @RequestMapping(value = "address/initial",method = RequestMethod.GET)
    public String addressInit(Model model) {
        return "pneumonia/attack_address_list";
    }

    //页面跳转（主页面）
    @RequestMapping(value = "area/initial",method = RequestMethod.GET)
    public String areaInit(Model model) {
        return "pneumonia/attack_area_list";
    }

    //疫情数据获取
    @RequestMapping(value = "data/info",method = RequestMethod.GET)
    @ResponseBody
    public String dataInfo() {
        try {
            return write(200,"获取成功","data",pneumoniaService.dataInfo());
        }catch (Exception e){
            e.printStackTrace();
            return error(-1,"获取失败");
        }
    }
    //疫情数据修改
    @RequestMapping(value = "data/modify",method = RequestMethod.POST)
    @ResponseBody
    public String dataModify(String jsonData) {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            PneumoniaDataNew pneumoniaDataNew = objectMapper.readValue(jsonData, PneumoniaDataNew.class);
            pneumoniaService.updateDataInfo(pneumoniaDataNew);
            return write(200,"修改成功","data","");
        }catch (Exception e){
            e.printStackTrace();
            return error(-1,"修改失败");
        }
    }
    //确诊小区获取
    @RequestMapping(value = "address/list",method = RequestMethod.GET)
    @ResponseBody
    public String addressList(@RequestParam(value = "page", required = true) Integer page,
                              @RequestParam(value = "rows", required = true) Integer rows) {
        try {
            Page<PneumoniaAttackAddress> result = pneumoniaService.addressList(page,rows);
            return write(200, "查询成功", page, rows, result);
        }catch (Exception e){
            e.printStackTrace();
            return error(-1,"查询失败");
        }
    }

    //编辑页面
    @RequestMapping(value = "address/updateInit/{id}", method = RequestMethod.GET)
    public String update(@PathVariable("id") Long id) {
        try {
            request.setAttribute("id", id);
            request.setAttribute("type", request.getParameter("type"));
        } catch (Exception e) {
            error(e);
        }
        return "pneumonia/attack_address_edit";
    }

    //确诊小区编辑 新增
    @RequestMapping(value = "address/modify",method = RequestMethod.POST)
    @ResponseBody
    public String addressModify(String jsonData) {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            PneumoniaAttackAddress pneumoniaDataNew = objectMapper.readValue(jsonData, PneumoniaAttackAddress.class);
            pneumoniaService.addressModify(pneumoniaDataNew);
            return write(200,"修改成功","data","");
        }catch (Exception e){
            e.printStackTrace();
            return error(-1,"修改失败");
        }
    }

    //确诊小区新增
    @RequestMapping(value = "address/createInit",method = RequestMethod.GET)
    public String addressCreate() {
        request.setAttribute("id", 0);
        request.setAttribute("type", request.getParameter("type"));
        return "pneumonia/attack_address_edit";
    }

    //确诊小区删除
    @RequestMapping(value = "address/updateStatus",method = RequestMethod.POST)
    @ResponseBody
    public String updateStatus(
            @RequestParam(value = "id", required = true) Long id,
            @RequestParam(value = "del", required = true) Integer del) {
        try {
            if (id == null) {
                return error(-1, "id不能为空！！");
            }

            if (del == null) {
                return error(-1, "状态不能为空！！");
            }

            boolean res = pneumoniaService.addressUpdateDel(del, id);
            return res == true ? write(200, "操作成功！") : error(-1, "操作失败！");
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }

    @RequestMapping(value = "address/findOne",method = RequestMethod.GET)
    @ResponseBody
    public String addressInfo(@RequestParam(value = "id", required = true) Long id) {
        try {
            PneumoniaAttackAddress result = pneumoniaService.addressFindOne(id);
            return write(200,"查询成功","pneumoniaAddress",result);
        }catch (Exception e){
            e.printStackTrace();
            return error(-1,"查询失败");
        }
    }

    //管控新增
    @RequestMapping(value = "area/createInit",method = RequestMethod.GET)
    public String areaCreate() {
        request.setAttribute("id", 0);
        request.setAttribute("type", request.getParameter("type"));
        return "pneumonia/attack_area_edit";
    }

    //管控区域获取
    @RequestMapping(value = "area/list",method = RequestMethod.GET)
    @ResponseBody
    public String areaList(@RequestParam(value = "page", required = true) Integer page,
                              @RequestParam(value = "rows", required = true) Integer rows) {
        try {
            Page<PneumoniaAttackArea> result = pneumoniaAttackAreaService.areaList(page,rows);
            return write(200, "查询成功", page, rows, result);
        }catch (Exception e){
            e.printStackTrace();
            return error(-1,"查询失败");
        }
    }

    //管控区域删除
    @RequestMapping(value = "area/updateStatus",method = RequestMethod.POST)
    @ResponseBody
    public String updateAreaStatus(
            @RequestParam(value = "id", required = true) Long id,
            @RequestParam(value = "del", required = true) Integer del) {
        try {
            if (id == null) {
                return error(-1, "id不能为空！！");
            }

            if (del == null) {
                return error(-1, "状态不能为空！！");
            }

            boolean res = pneumoniaAttackAreaService.areaUpdateDel(del, id);
            return res == true ? write(200, "操作成功！") : error(-1, "操作失败！");
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }

    @RequestMapping(value = "area/findOne",method = RequestMethod.GET)
    @ResponseBody
    public String areaInfo(@RequestParam(value = "id", required = true) Long id) {
        try {
            PneumoniaAttackArea result = pneumoniaAttackAreaService.areaFindOne(id);
            return write(200,"查询成功","pneumoniaArea",result);
        }catch (Exception e){
            e.printStackTrace();
            return error(-1,"查询失败");
        }
    }

    //管控区域编辑 新增
    @RequestMapping(value = "area/modify",method = RequestMethod.POST)
    @ResponseBody
    public String areaModify(String jsonData) {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            PneumoniaAttackArea pneumoniaAttackAreaNew = objectMapper.readValue(jsonData, PneumoniaAttackArea.class);
            pneumoniaAttackAreaService.areaModify(pneumoniaAttackAreaNew);
            return write(200,"修改成功","data","");
        }catch (Exception e){
            e.printStackTrace();
            return error(-1,"修改失败");
        }
    }

    //编辑页面
    @RequestMapping(value = "area/updateInit/{id}", method = RequestMethod.GET)
    public String areaUpdate(@PathVariable("id") Long id) {
        try {
            request.setAttribute("id", id);
            request.setAttribute("model", request.getParameter("type"));
        } catch (Exception e) {
            error(e);
        }
        return "pneumonia/attack_area_edit";
    }

    //管控小区地区工具
    @RequestMapping(value = "area/map",method = RequestMethod.GET)
    public String areaMap() {
        return "getPoint";
    }
}
