package  com.yihu.customer.controller.manager.satisfaction;

import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.satisfaction.SatisfactionSurveyDetail;
import com.yihu.customer.service.satisfaction.SatisfactionService;
import com.yihu.customer.util.WorkbookUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import jxl.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 满意度调查计划和详情
 * @author xyq
 * @create 2022-07-18 10:30
 * @slogan 他化自在，我自逍遥
 */
@Controller
@RequestMapping(value = "/synergy/admin/satisfaction",produces = "application/json;charset=UTF-8")
@Api(description = "调查计划")
public class SatisfactionController extends BaseController {

    @Autowired
    private SatisfactionService satisfactionService;

    @ApiOperation("创建调查计划")
    @ResponseBody
    @RequestMapping(value = "/addPlan",method = RequestMethod.POST)
    public String addPlan(@ApiParam(value = "调查计划名字") @RequestParam String name,
                          @ApiParam(value = "调查范围，0集美区，1社区") @RequestParam Integer rangType,
                          @ApiParam(value = "调查社区code，逗号拼接") @RequestParam String rangCode,
                          @ApiParam(value = "筛选类型，0随机筛选，1按社区平均筛选，2按团队平均筛选")@RequestParam Integer filterType,
                          @ApiParam(value = "筛选数量") @RequestParam Integer filterNumber,
                          @ApiParam(value = "调查客服") @RequestParam String userCode,
                          @ApiParam(value = "调查开始时间") @RequestParam(required = false) String beginTime,
                          @ApiParam(value = "调查结束时间") @RequestParam(required = false) String endTime,
                          @ApiParam(value = "备注") @RequestParam(required = false) String remark){
        try {
            if(filterNumber <= 0){
                return error(-1,"筛选数量必须大于0");
            }
            return write(200,"创建成功","data",satisfactionService.addOrUpdatePlan(null,name, rangType, rangCode, filterType, filterNumber, userCode, beginTime, endTime, remark));
        }catch (Exception e){
            e.printStackTrace();
            return error(-1,"创建失败");
        }
    }

    @ApiOperation("获取筛选总数")
    @ResponseBody
    @RequestMapping(value = "/getSearchTotal",method = RequestMethod.GET)
    public String getSearchTotal(@ApiParam(value = "调查范围，0集美区，1社区") @RequestParam Integer rangType,
                          @ApiParam(value = "调查社区code，逗号拼接") @RequestParam(required = false) String rangCode,
                          @ApiParam(value = "筛选类型，0随机筛选，1按社区平均筛选，2按团队平均筛选")@RequestParam Integer filterType,
                          @ApiParam(value = "筛选数量") @RequestParam Integer filterNumber){
        try {
            if(filterNumber <= 0){
                return error(-1,"筛选数量必须大于0");
            }
            return write(200,"创建成功","data",satisfactionService.getSearchTotal(rangType, rangCode, filterType, filterNumber));
        }catch (Exception e){
            e.printStackTrace();
            return error(-1,"创建失败");
        }
    }

    @ApiOperation("修改调查计划")
    @ResponseBody
    @RequestMapping(value = "/updatePlan",method = RequestMethod.POST)
    public String updatePlan(@ApiParam(value = "计划id") @RequestParam Long id,
                             @ApiParam(value = "调查计划名字") @RequestParam String name,
                             @ApiParam(value = "调查范围，0集美区，1社区") @RequestParam Integer rangType,
                             @ApiParam(value = "调查社区code，逗号拼接") @RequestParam String rangCode,
                             @ApiParam(value = "筛选类型，0随机筛选，1按社区平均筛选，2按团队平均筛选")@RequestParam Integer filterType,
                             @ApiParam(value = "筛选数量") @RequestParam Integer filterNumber,
                             @ApiParam(value = "调查客服") @RequestParam String userCode,
                             @ApiParam(value = "调查开始时间") @RequestParam(required = false) String beginTime,
                             @ApiParam(value = "调查结束时间") @RequestParam(required = false) String endTime,
                             @ApiParam(value = "备注") @RequestParam(required = false) String remark){
        try {
            if(filterNumber <= 0){
                return error(-1,"筛选数量必须大于0");
            }
            satisfactionService.addOrUpdatePlan(id, name, rangType, rangCode, filterType, filterNumber, userCode, beginTime, endTime, remark);
            return write(200,"修改成功");
        }catch (Exception e){
            e.printStackTrace();
            return error(-1,"修改失败");
        }
    }

    @ApiOperation("获取社区列表")
    @ResponseBody
    @RequestMapping(value = "/getHospitalList",method = RequestMethod.GET)
    public String getHospitalList(){
        try {
            return write(200,"查询成功","list",satisfactionService.getHospitalList());
        }catch (Exception e){
            return error(-1,"查询失败");
        }
    }

    @ApiOperation("获取客服列表")
    @ResponseBody
    @RequestMapping(value = "/getSysUserList",method = RequestMethod.GET)
    public String getSysUserList(){
        try {
            return write(200,"查询成功","list",satisfactionService.getSysUserList());
        }catch (Exception e){
            return error(-1,"查询失败");
        }
    }

    @ApiOperation("满意度调查计划列表")
    @ResponseBody
    @RequestMapping(value = "/getPlanList",method = RequestMethod.GET)
    public String getPlanList(@ApiParam(value = "调查计划名字查询") @RequestParam(required = false)String name,
                              @ApiParam(value = "客服名字查询") @RequestParam(required = false)String userName,
                              @ApiParam(value = "社区code查询") @RequestParam(required = false)String rangCode,
                              @ApiParam(value = "调查开始时间") @RequestParam(required = false) String beginTime,
                              @ApiParam(value = "调查结束时间") @RequestParam(required = false) String endTime,
                              @ApiParam(value = "客服code") @RequestParam(required = false) String userCode,
                              @ApiParam @RequestParam(required = false,defaultValue = "1")Integer page,
                              @ApiParam @RequestParam(required = false,defaultValue = "20")Integer pageSize){
        try {
            return write(200,"查询成功","data",satisfactionService.getPlanList(name, userName, rangCode, beginTime, endTime, page, pageSize,userCode));
        }catch (Exception e){
            e.printStackTrace();
            return error(-1,"查询失败");
        }
    }

    @ApiOperation("根据计划id查计划详情列表")
    @ResponseBody
    @RequestMapping(value = "/getPlanDetailListByPlanId",method = RequestMethod.GET)
    public String getPlanDetailListByPlanId(@ApiParam(value = "计划id") @RequestParam Integer planId,
                                            @ApiParam(value = "签约机构") @RequestParam(required = false) String hospital,
                                            @ApiParam(value = "手机号") @RequestParam(required = false) String mobile,
                                            @ApiParam(value = "姓名") @RequestParam(required = false) String name,
                                            @ApiParam(value = "筛选重复号码 只查询出号码重复数据1是")
                                            @RequestParam(required = false,defaultValue = "0") Integer mobileRepeat,
                                            @ApiParam(value = "页码") @RequestParam(required = false,defaultValue = "1")Integer page,
                                            @ApiParam(value = "每页大小")@RequestParam(required = false,defaultValue = "20")Integer pageSize){
        try {
            return write(200,"查询成功","data",satisfactionService.getPlanDetailListByPlanId(planId,hospital,mobile,name,mobileRepeat,page,pageSize));
        }catch (Exception e){
            e.printStackTrace();
            return error(-1,"查询失败");
        }
    }

    @ApiOperation("根据id编辑一条调查度调查")
    @ResponseBody
    @RequestMapping(value = "/updatePlanDetailById",method = RequestMethod.POST)
    public String updatePlanDetailById(@ApiParam(value = "详情id") @RequestParam Long id,
                                       @ApiParam(value = "电话") @RequestParam(required = false) String mobile,
                                       @ApiParam(value = "住址")@RequestParam(required = false)String address,
                                       @ApiParam(value = "接通情况。电话有通但未接、挂断、停机标记0，本人或家属接听标记1") @RequestParam(required = false) Integer isOn,
                                       @ApiParam(value = "是否知道签约的家庭医生，0不知道，1知道")@RequestParam(required = false)Integer isKnowDoctor,
                                       @ApiParam(value = "是否介绍家庭医生签约的好处，0否，1是")@RequestParam(required = false)Integer isIntroduceDoctor,
                                       @ApiParam(value = "对家签医生是否满意？1满意2不满意")@RequestParam(required = false)Integer isSatisfyDoctor,
                                       @ApiParam(value = "备注") @RequestParam(required = false) String remark){
        try{
            SatisfactionSurveyDetail one = satisfactionService.updatePlanDetailById(id, mobile, address, isOn, isKnowDoctor, isIntroduceDoctor, isSatisfyDoctor, remark);
            if(one == null){
                return error(-1,"调查记录不存在");
            }else {
                return write(200,"操作成功");
            }
        }catch (Exception e){
            return error(-1,"操作失败");
        }
    }


    @RequestMapping(value = "exportPlanDetailList",method = RequestMethod.GET)
    @ApiOperation("导出调查列表")
    public void exportPlanDetailList(@ApiParam(value = "计划id")@RequestParam Long planId, HttpServletResponse response){
        try {
            List<Map<String, Object>> list = satisfactionService.getPlanDetailListByPlanIdAndOver(planId);
                String fileName = satisfactionService.getPlanName(planId);
                response.setContentType("octets/stream");
                response.setCharacterEncoding("utf-8");
                fileName = URLEncoder.encode(fileName, "utf-8");
                response.setHeader("Content-Disposition", "attachment; filename="+ fileName + new String(".xls"));
                OutputStream os = response.getOutputStream();
                WorkbookUtil workbookUtil = new WorkbookUtil();
                //表头，第一行
                String[] header = {"居民姓名","性别","联系电话","签约机构","家庭医生","现住址","接通情况（电话有通但未接、挂断、停机标记‘0’，本人或家属接听标记‘1’)","请问您是否知道签约的家庭医生？","是否介绍家庭医生签约的好处","对家签医生是否满意？1满意2不满意","回访客服","备注"};
                //每一列对应的键
                Map<Integer,String> columnMap = new HashMap<>();
                columnMap.put(0,"patient_name");
                columnMap.put(1,"sexStr");
                columnMap.put(2,"mobile");
                columnMap.put(3,"sign_org_name");
                columnMap.put(4,"doctor_name");
                columnMap.put(5,"address");
                columnMap.put(6,"is_on");
                columnMap.put(7,"isKnowDoctorStr");
                columnMap.put(8,"isIntroduceDoctorStr");
                columnMap.put(9,"is_satisfy_doctor");
                columnMap.put(10,"return_user_name");
                columnMap.put(11,"remark");
                workbookUtil.write(Workbook.createWorkbook(os), list, columnMap, header);
                os.flush();
                os.close();
        }catch (Exception e){
           e.printStackTrace();
        }
    }


}
