package  com.yihu.customer.controller.manager.survey;

import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.survey.SurveyQuestion;
import com.yihu.customer.service.manager.survey.SurveyQuestionOptionService;
import com.yihu.customer.service.manager.survey.SurveyQuestionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import springfox.documentation.annotations.ApiIgnore;

/**
 * Created by yeshijie on 2017/3/7.
 */
@Controller
@RequestMapping(value="admin/surveyQuestion")
//@Api(description = "问题题库数据接口")
@Api(value = "surveyQuestion", description = "问题题库数据接口", tags = {"问题题库"})
public class SurveyQuestionController extends BaseController{

    @Autowired
    private SurveyQuestionService surveyQuestionService;
    @Autowired
    private SurveyQuestionOptionService surveyQuestionOptionService;

    //---问题管理列表---
    @RequestMapping(value = "initial", method = RequestMethod.GET)
    @ApiIgnore
    @ApiOperation(value = "问题管理列表")
    public String initQuestionList(){
        return "questionnaire/question/question_list";
    }

    //页面跳转（新增问题页面）
    @RequestMapping(value ="infoInit",method = RequestMethod.GET)
    @ApiIgnore
    @ApiOperation(value = "页面跳转（新增问题页面）")
    public String infoInit(String id,String mode){
        request.setAttribute("id",id);
        request.setAttribute("mode",mode);
        return "questionnaire/question/question_add";
    }

    //页面跳转（编辑问题页面）
    @RequestMapping(value ="editQuestion",method = RequestMethod.GET)
    @ApiIgnore
    @ApiOperation(value = "页面跳转（编辑问题页面）")
    public String editQuestion(String id,String mode){
        request.setAttribute("id",id);
        request.setAttribute("mode",mode);
        return "questionnaire/question/question_edit";
    }

    //跳转到问题查看页
    @RequestMapping(value = "seeQuestion", method = RequestMethod.GET)
    @ApiIgnore
    @ApiOperation(value = "跳转到问题查看页")
    public String seeQuestion(String id,String type) {
        request.setAttribute("id",id);
        request.setAttribute("type",type);
        return "questionnaire/question/see_question";
    }

    @RequestMapping(value = "list", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "问题列表数据")
    public String searchQuestionList(
            @ApiParam(name = "title", value = "标题",defaultValue = "")
            @RequestParam(value = "title", required = false) String title,
            @ApiParam(name = "questionType", value = "问题类型（0单选 1多选 2问答）",defaultValue = "0")
            @RequestParam(value = "questionType", required = false) Integer questionType,
            @ApiParam(name = "page",value = "第几页",defaultValue = "1")
            @RequestParam(value = "page",required = true) int page,
            @ApiParam(name = "rows",value = "页面大小",defaultValue = "2")
            @RequestParam(value = "rows",required = true) int rows) {
        try {
            Page<SurveyQuestion> res = surveyQuestionService.findQuestion(page,rows,title,questionType);
            return write(200, "操作成功", page, rows, res);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败");
        }
    }

    /**
     * 问题新增页面
     * @param model
     * @return
     */
    @RequestMapping(value = "addQuestionInitial", method = RequestMethod.GET)
    @ApiIgnore
    @ApiOperation(value = "问题列表数据")
    public String addQuestionInitial(Model model) {
        return "survey/question/addQuestion";
    }

    @RequestMapping(value = "addQuestions", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "新增问题（可批量新增）")
    public String addQuestions(@ApiParam(name = "jsonData", value = "新增json",defaultValue = "")
                                @RequestParam(value = "jsonData", required = true) String jsonData) {
        try {
            System.out.println("jsonData:"+jsonData);
            surveyQuestionService.saveOrUpdateQuestion(jsonData);
            return success("新增成功！");
        } catch (Exception e) {
            error(e);
            return error(-1, "新增失败！");
        }
    }

    @RequestMapping(value = "getQuestion", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "根据id获取单个问题")
    public String getQuestion(@ApiParam(name = "id", value = "问题id",defaultValue = "1")
                               @RequestParam(value = "id", required = true) Long id) {
        try {
            SurveyQuestion question = surveyQuestionService.findById(id);
            if(question == null){
                return error(-1, "获取问题Id不存在");
            }else if("0".equals(question.getDel())){
                return error(-1, "该问题已被删除");
            }else{
                JSONObject json = surveyQuestionService.getQuestion(question);
                return write(200,"获取成功","data",json);
            }
        } catch (Exception e) {
            error(e);
            return error(-1, "获取失败！");
        }
    }


    @RequestMapping(value = "getQuestions", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "根据id获取单个问题")
    public String getQuestion(@ApiParam(name = "id", value = "问题id",defaultValue = "1")
                              @RequestParam(value = "ids", required = true) String  ids) {
        try {
            JSONArray questions = surveyQuestionService.findByIds(ids);
             return write(200,"获取成功","data",questions);
        } catch (Exception e) {
            error(e);
            return error(-1, "获取失败！");
        }
    }

    @RequestMapping(value = "delQuestion", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "根据id删除单个问题")
    public String delQuestion(@ApiParam(name = "id", value = "问题id",defaultValue = "1")
                              @RequestParam(value = "id", required = true) Long id) {
        try {
            surveyQuestionService.delQuestion(id);
            return success("删除成功");
        } catch (Exception e) {
            error(e);
            return error(-1, "删除失败！");
        }
    }

    @RequestMapping(value = "delQuestions", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "批量删除")
    public String delQuestions(@ApiParam(name = "ids", value = "问题ids",defaultValue = "1;2;3")
                              @RequestParam(value = "ids", required = true) String ids) {
        try {
            if(StringUtils.isEmpty(ids)){
                return error(-1, "删除问题ids不能为空！");
            }
            surveyQuestionService.delQuestions(ids);
            return success("删除成功");
        } catch (Exception e) {
            error(e);
            return error(-1, "删除失败！");
        }
    }

    @RequestMapping(value = "updQuestion", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "更新问题")
    public String updQuestion(@ApiParam(name = "questionData", value = "问题json",defaultValue = "{}")
                               @RequestParam(value = "questionData", required = true) String questionData) {
        try {
            System.out.println(questionData);
            return success("更新成功");
        } catch (Exception e) {
            error(e);
            return error(-1, "更新失败！");
        }
    }



}
