package  com.yihu.customer.controller.manager.survey;

import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.User;
import  com.yihu.customer.entity.survey.SurveyTemplate;
import com.yihu.customer.service.manager.survey.SurveyTemplateService;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;

/**
 * Created by yeshijie on 2017/3/7.
 */
@Controller
@RequestMapping(value="admin/surveyTemplate/")
public class SurveyTemplateController extends BaseController{
    @Autowired
    SurveyTemplateService surveyTemplateService;
    //---问卷模板管理列表---
    @RequestMapping(value = "initial", method = RequestMethod.GET)
    public String initTeamList(){
        return "questionnaire/template/template_list";
    }

    //跳转到问题查看页
    @RequestMapping(value = "importQuestion", method = RequestMethod.GET)
    @ApiIgnore
    @ApiOperation(value = "跳转到问题查看页")
    public String importQuestion() {
        return "questionnaire/template/import_question";
    }

    //跳转到问卷模板筛查结果列表---
    @RequestMapping(value = "initResult", method = RequestMethod.GET)
    public String initResult(String templateCode, Integer diseaseType, String mode){
        request.setAttribute("templateCode",templateCode);
        request.setAttribute("diseaseType",diseaseType);
        request.setAttribute("mode",mode);
        return "questionnaire/template/template_result";
    }

    @RequestMapping(value = "list", method = RequestMethod.GET)
    @ResponseBody
    public String list(
            @ApiParam(name = "title", value = "标题",defaultValue = "")
            @RequestParam(value = "title", required = false) String title,
            @ApiParam(name = "label", value = "标签信息",defaultValue = "0")
            @RequestParam(value = "label", required = false) Integer labelCode,
            @ApiParam(name = "page",value = "第几页",defaultValue = "1")
            @RequestParam(value = "page",required = true) int page,
            @ApiParam(name = "rows",value = "页面大小",defaultValue = "2")
            @RequestParam(value = "rows",required = true) int rows){
        try {
            Page<JSONObject> res = surveyTemplateService.queryList(page,rows,title,labelCode);
            return write(200, "操作成功", page, rows, res);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败");
        }
    }


    @RequestMapping(value = "list/options", method = RequestMethod.GET)
    @ResponseBody
    public String listOption(
            @ApiParam(name = "q", value = "标题",defaultValue = "")
            @RequestParam(value = "q", required = false) String title,
            @ApiParam(name = "label", value = "标签信息",defaultValue = "0")
            @RequestParam(value = "label", required = false) Integer labelCode,
            @ApiParam(name = "page",value = "第几页",defaultValue = "1")
            @RequestParam(value = "page",required = true) int page,
            @ApiParam(name = "rows",value = "页面大小",defaultValue = "2")
            @RequestParam(value = "rows",required = true) int rows){
        try {
            Page<JSONObject> res = surveyTemplateService.queryList(page,rows,title,labelCode);
            JSONArray array = new JSONArray();
            for(JSONObject p:res.getContent()){
                JSONObject optionObj = new JSONObject();
                optionObj.put("id",p.get("id"));
                optionObj.put("text",p.get("title"));
                array.put(optionObj);
            }
            return write(200,"获取成功","list",array);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败");
        }
    }



    //---新增问卷模板---
    @RequestMapping(value = "addTemplate", method = RequestMethod.GET)
    public String addTemplate(String id,String mode){
        request.setAttribute("templateId",id);
        request.setAttribute("mode",mode);
        return "questionnaire/template/add_template";
    }

    //---编辑问卷模板---
    @RequestMapping(value = "editTemplate", method = RequestMethod.GET)
    public String editTemplate(String id,String mode){
        request.setAttribute("id",id);
        request.setAttribute("mode",mode);
        return "questionnaire/template/edit_template";
    }

    //---保存标签---
    @RequestMapping(value = "editLabel", method = RequestMethod.POST)
    public String editLabel(@ApiParam(name = "模板id", value = "模板id",defaultValue = "0")
                                @RequestParam(value = "templateId", required = true) long templateId,
                            @ApiParam(name = "标签JSON", value = "标签JSON",defaultValue = "")
                            @RequestParam(value = "jsonData", required = true) String jsonData){
        try {
            surveyTemplateService.saveLabel(templateId,jsonData);
            return write(200,"保存成功");
        }catch (Exception e){
            return write(-1,"保存失败");
        }
    }

    @RequestMapping(value = "save", method = RequestMethod.POST)
    @ResponseBody
    public String saveTemplate(@ApiParam(name = "jsonData", value = "新增json",defaultValue = "")
                                   @RequestParam(value = "jsonData", required = true) String jsonData,
                               HttpServletRequest request){
        User loginUser = (User) request.getSession().getAttribute("userInfo");
        try {
            SurveyTemplate surveyTemplate = surveyTemplateService.saveOrUpdate(jsonData,loginUser.getCode());
            return write(200,"保存成功","data",surveyTemplate);
        }catch (Exception e){
            e.printStackTrace();
            return write(-1,"保存失败");
        }
    }


    @RequestMapping(value = "template", method = RequestMethod.GET)
    @ResponseBody
    public String getTemplate(@ApiParam(name = "id", value = "模板id",defaultValue = "0")
                              @RequestParam(value = "id", required = true) long id,
                              @ApiParam(name = "question", value = "是否加载问题",defaultValue = "0")
                              @RequestParam(value = "question", required = false) long question,
                              HttpServletRequest request){
        try {
            com.alibaba.fastjson.JSONObject surveyTemplate =  surveyTemplateService.getTemplate(id,question);
            return write(200,"获取成功","data",surveyTemplate);
        }catch (Exception e){
            return write(-1,"获取失败");
        }
    }

    @RequestMapping(value = "template/title", method = RequestMethod.GET)
    @ResponseBody
    public String getTemplate(@ApiParam(name = "title", value = "模板title",defaultValue = "")
                              @RequestParam(value = "title", required = true) String  title){
        try {
            SurveyTemplate surveyTemplate =  surveyTemplateService.findByTitle(title);
            return write(200,"获取成功","data",surveyTemplate);
        }catch (Exception e){
            return write(-1,"获取失败");
        }
    }

    @RequestMapping(value = "delete", method = RequestMethod.POST)
    @ResponseBody
    public String delete(@ApiParam(name = "模板ID", value = "模板ID",defaultValue = "0")
                              @RequestParam(value = "templateId", required = true) Long templateId){
        try {
           surveyTemplateService.deleteTemplate(templateId);
            return write(200,"删除成功");
        }catch (Exception e){
            return write(-1,"获取失败");
        }
    }
    
    @RequestMapping(value = "online", method = RequestMethod.POST)
    @ResponseBody
    public String online(@ApiParam(name = "模板ID", value = "模板ID",defaultValue = "0")
                         @RequestParam(value = "templateId", required = true) Long templateId){
        try {
            surveyTemplateService.online(templateId);
            return write(200,"操作成功");
        }catch (Exception e){
            return write(-1,"获取失败");
        }
    }
}
