package  com.yihu.customer.controller.manager.user;

import com.fasterxml.jackson.databind.ObjectMapper;
import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.ManageFeature;
import  com.yihu.customer.entity.ManageRoleFeature;
import  com.yihu.customer.filter.RoleCache;
import com.yihu.customer.service.manager.user.ManageFeatureService;
import  com.yihu.customer.model.FeatureTree;
import  com.yihu.customer.repository.ManageRoleFeatureDao;
import com.yihu.customer.util.Envelop;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import springfox.documentation.annotations.ApiIgnore;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by yww on 2016/12/29.
 */
@Controller
@RequestMapping("admin/feature")
@Api(description = "管理系统权限")
public class ManageFeatureController extends BaseController {
    @Autowired
    RoleCache roleCache;


    @Autowired
    private ManageFeatureService featureService;

    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private JdbcTemplate jdbcTemplate;

    //页面跳转
    @RequestMapping(value = "initial", method = RequestMethod.GET)
    @ApiIgnore
    public String featureListInit() {
        return "user/feature/featureGrid";
    }

    @RequestMapping(value = "/tree", produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object tree() {
        try {
            List<ManageFeature> features = featureService.search("type<>3;");
            Map<Long, FeatureTree> map = new HashMap<>();
            map.put(new Long(0), new FeatureTree(0, "三师后台管理系统", -1, "0", ""));
            for (ManageFeature model : features) {
                map.put(model.getId(), new FeatureTree(
                        model.getId(), model.getName(), model.getParentId(), model.getType(), model.getIconUrl()));
            }
            FeatureTree p;
            for (FeatureTree model : map.values()) {
                if ((p = map.get(model.getParentId())) != null)
                    p.addChild(model);
            }

            List rs = new ArrayList<>();
            rs.add(map.get(new Long(0)));
            Envelop envelop = new Envelop();
            envelop.setSuccessFlg(true);
            envelop.setDetailModelList(rs);
            return objectMapper.writeValueAsString(envelop);
        } catch (Exception ex) {
            ex.printStackTrace();
            return write(-1, "系统错误！");
        }
    }

    @RequestMapping(value = "/list", method = RequestMethod.POST)
    @ResponseBody
    public Object search(String fields, String filters, String sorts, int page, int rows, String extParms) {

        try {
            Map<String, Object> params = new HashMap<>();
            params.put("fields", nullToSpace(fields));
            params.put("filters", nullToSpace(filters));
            params.put("sorts", nullToSpace(sorts));
            params.put("page", page);
            params.put("size", rows);
            List<ManageFeature> features = featureService.search(nullToSpace(fields), nullToSpace(filters), nullToSpace(sorts), page, rows);
            Envelop envelop = new Envelop();
            envelop.setSuccessFlg(true);
            envelop.setDetailModelList(features);
            return objectMapper.writeValueAsString(envelop);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "系统错误！");
        }
    }

    @RequestMapping("/gotoModify")
    @ApiIgnore
    public String gotoModify(Model model, String id, String mode, String extParms) {
        try {
            if (StringUtils.isEmpty(id)) {
                return error(-1, "权限id不能为空！");
            }
            if ("new".equals(mode)) {
                model.addAttribute("model", "");
                model.addAttribute("parentId", id);
            } else {
                ManageFeature feature = featureService.retrieve(Long.parseLong(id));
                model.addAttribute("model", objectMapper.writeValueAsString(feature));
                model.addAttribute("parentId", feature.getParentId());
            }
            model.addAttribute("mode", mode);
            model.addAttribute("extParms", extParms);
            return "user/feature/featureInfoDialog";
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "系统错误");
        }
    }

    @RequestMapping("/existence")
    @ResponseBody
    public String isExistence(String filters) {
        try {
            List list = featureService.search(filters);
            if (list != null && list.size() > 0) {
                return write(200, "存在子项", "data", list.get(0));
            }
            return write(200, "不存在子项", "data", "");
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "系统错误！");
        }
    }

    @RequestMapping("/delete")
    @ResponseBody
    public Object delete(String ids, String idField, String extParms, String type) {
        try {
            if (StringUtils.isEmpty(ids)) {
                return error(-1, "权限id不能为空！");
            }
            boolean res;
            if ("unique".equals(type)) {
                Long id = Long.parseLong(ids);
                res = featureService.deleteUnique(id);
                if (res) {
                    return write(200, "删除成功");
                }
                return error(-1, "删除失败");
            } else {
                return error(-1, "只允许删除单个权限！");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "系统错误！");
        }
    }

    @Autowired
    private ManageRoleFeatureDao roleFeatureDao;

    @RequestMapping("/common/update1")
    @ResponseBody
    public Object update1() {
        try {
            String sql = "SELECT * from manage_feature WHERE id not in ( " +
                    "SELECT DISTINCT a.feature_id from manage_role_feature a " +
                    ")";
            List<ManageFeature> list = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(ManageFeature.class));
            List<ManageRoleFeature> roleFeatures = new ArrayList<>();
            for (ManageFeature manageFeature:list){
                ManageRoleFeature roleFeature = new ManageRoleFeature();
                roleFeature.setFeatureId(manageFeature.getId());
                roleFeature.setRoleId(new Long(1));//system的id固定1
                roleFeatures.add(roleFeature);
            }
            roleFeatureDao.saveAll(roleFeatures);

            return write(200, "新增成功！", "obj", "feature");
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "操作失败！");
        }
    }

    @RequestMapping("/update")
    @ResponseBody
    public Object update(String id, String model, String modelName, String extParms) {
        try {
            ManageFeature feature = objectMapper.readValue(model, ManageFeature.class);
            String errorMsg = "";
            if (StringUtils.isEmpty(feature.getCode())) {
                errorMsg += errorMsg + "编码不能为空！;";
            }
            if (StringUtils.isEmpty(feature.getName())) {
                errorMsg += errorMsg + "名称不能为空！;";
            }
            if (StringUtils.isEmpty(feature.getType())) {
                errorMsg += errorMsg + "类型不能为空！;";
            }
            if (!StringUtils.isEmpty(errorMsg)) {
                return error(-1, errorMsg);
            }
            if (StringUtils.isEmpty(id) || "0".equals(id)) {
                boolean res = featureService.createFeature(feature);
                if (res) {
                    return write(200, "新增成功！", "obj", feature);
                }
                return error(-1, "新增失败！");
            } else {
                boolean res = featureService.updateFeature(feature);
                if (res) {
                    return write(200, "修改成功！", "obj", feature);
                }
                return error(-1, "修改失败！");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "操作失败！");
        }
    }

    public String nullToSpace(String str) {
        if (str == null)
            return "";
        return str;
    }

    //    @Autowired
//    private ManageFeatureService featureService;
//
//    @Autowired
//    private ObjectMapper objectMapper;
//
//    //页面跳转
//    @RequestMapping(value = "list",method = RequestMethod.GET)
//    @ApiIgnore
//    public String featureListInit(){
//        return "user/feature/featureGrid";
//    }
//
//    @RequestMapping(value = "info",method = RequestMethod.GET)
//    @ApiIgnore
//    public String featureInfoInit(){
//        return "user/feature/featureInfoDialog";
//    }
//    //增删改查
//    @RequestMapping(value = "create",method = RequestMethod.POST)
//    @ResponseBody
//    @ApiOperation(value = "新增权限", notes = "*")
//    public String createFeature(
//            @ApiParam(name = "dataJson",value = "权限对象Json串")
//            @RequestParam(value = "dataJson")String dataJson){
//        try{
//            ManageFeature feature = objectMapper.readValue(dataJson, ManageFeature.class);
//            //数据验证操作
//            String errorMsg = "";
//            if(StringUtils.isEmpty(feature.getCode())){
//                errorMsg += errorMsg +"权限编码不能为空！；";
//            }else {
//                if(featureService.isCodeExist(feature.getCode())){
//                    errorMsg += errorMsg+"权限编码已存在！;";
//                }
//            }
//            if(StringUtils.isEmpty(feature.getName())){
//                errorMsg += errorMsg +"权限名称不能为空！；";
//            }else {
//                if(featureService.isNameExist(feature.getName())){
//                    errorMsg += errorMsg+ "权限名称已存在！;";
//                }
//            }
//            if(!StringUtils.isEmpty(errorMsg)){
//                return error(-1, errorMsg);
//            }
//            //新增操作(新增失败抛异常）
//            Envelop envelop = featureService.createFeature(feature);
//            if(!envelop.isSuccessFlg()){
//                return error(-1,envelop.getErrorMsg());
//            }
//            return write(200,"操作成功");
//        }catch (JsonProcessingException jex){
//            error(jex);
//            return error(-1,"JSON参数有误，请核对");
//        }catch (Exception ex){
//            error(ex);
//            return error(-1,"操作失败");
//        }
//    }
//
//    @RequestMapping(value = "update",method = RequestMethod.POST)
//    @ResponseBody
//    @ApiOperation(value = "修改权限", notes = "*")
//    public String updateFeature(
//            @ApiParam(name = "dataJson",value = "权限对象Json串")
//            @RequestParam(value = "dataJson")String dataJson){
//        try{
//            ManageFeature feature = objectMapper.readValue(dataJson, ManageFeature.class);
//            //数据验证操作
//            String errorMsg = "";
//            if(feature == null || feature.getId() == 0){
//                errorMsg += errorMsg+"权限id不能为空！；";
//            }
//            if(StringUtils.isEmpty(feature.getCode())){
//                errorMsg += errorMsg +"权限编码不能为空！；";
//            }else {
//                if(featureService.isCodeExist(feature.getCode())){
//                    errorMsg += errorMsg+"权限编码已存在！;";
//                }
//            }
//            if(StringUtils.isEmpty(feature.getName())){
//                errorMsg += errorMsg +"权限名称不能为空！；";
//            }else {
//                if(featureService.isNameExist(feature.getName())){
//                    errorMsg += errorMsg+ "权限名称已存在！;";
//                }
//            }
//            if(!StringUtils.isEmpty(errorMsg)){
//                return error(-1, errorMsg);
//            }
//            //新增操作(新增失败抛异常）
//            Envelop envelop = featureService.updateFeature(feature);
//            if(!envelop.isSuccessFlg()){
//                return error(-1,envelop.getErrorMsg());
//            }
//            return write(200,"操作成功");
//        }catch (JsonProcessingException jex){
//            error(jex);
//            return error(-1,"JSON参数有误，请核对");
//        }catch (Exception ex){
//            error(ex);
//            return error(-1,"操作失败");
//        }
//    }
//
//    @RequestMapping(value = "delete",method = RequestMethod.POST)
//    @ResponseBody
//    @ApiOperation(value = "删除权限", notes = "")
//    public String deleteFeature(
//            @ApiParam(name = "id",value = "权限id")
//            @RequestParam(name = "id")int id,
//            @ApiParam(name = "code",value = "权限编码")
//            @RequestParam(name = "code")String code){
//        try{
//            if(id<=0){
//                return error(-1,"权限id不能为空！");
//            }
//            if(StringUtils.isEmpty(code)){
//                return error(-1,"权限编码不能为空！");
//            }
//            featureService.deleteFeature(id, code);
//            return write(200,"操作成功");
//        }catch (Exception ex){
//            return error(-1,"操作失败！");
//        }
//    }
//
//    @RequestMapping(value = "deleteNoRole",method = RequestMethod.POST)
//    @ResponseBody
//    @ApiOperation(value = "删除权限", notes = "")
//    public String deleteFeatureNoUser(
//            @ApiParam(name = "id",value = "权限id")
//            @RequestParam(name = "id")int id,
//            @ApiParam(name = "code",value = "权限编码")
//            @RequestParam(name = "code") String code){
//        try{
//            if(id<=0){
//                return error(-1,"权限id不能为空！");
//            }
//            if(StringUtils.isEmpty(code)){
//                return error(-1,"权限编码不能为空！");
//            }
//            featureService.deleteFeatureNoRole(id, code);
//            return write(200,"操作成功");
//        }catch (Exception ex){
//            return error(-1,"操作失败！");
//        }
//    }
//
//    @RequestMapping(value = "role",method = RequestMethod.POST)
//    @ResponseBody
//    @ApiOperation(value = "获取权限", notes = "根据id获取权限")
//    public String getFeature(
//            @ApiParam(name = "id",value = "权限id")
//            @RequestParam(name = "id")int id){
//        ManageFeature feature = featureService.getFeature(id);
//        return write(200,"操作成功","data",feature);
//    }
//
//    @RequestMapping(value = "roles")
//    @ResponseBody
//    @ApiOperation(value = "获取权限列表-分页", notes = "*")
//    public String searchFeatures(
//            @ApiParam(name = "name",value = "权限名称",defaultValue = "")
//            @RequestParam(value = "name",required = false)String name,
//            @ApiParam(name = "page",value = "页码",defaultValue = "1")
//            @RequestParam(value = "page",required = false)Integer page,
//            @ApiParam(name = "rows",value = "每页数",defaultValue = "15")
//            @RequestParam(value = "rows",required = false)Integer pageSize){
//        try{
//            Page<ManageFeature> manageFeatures = featureService.searchFeatures(name, page, pageSize);
//            return write(200,"操作成功",page,pageSize,manageFeatures);
//        }catch (Exception ex){
//            error(ex);
//            return error(-1,"操作失败！");
//        }
//
//    }
//
    @RequestMapping(value = "featuresNoPage")
    @ResponseBody
    @ApiOperation(value = "获取权限列表-不分页", notes = "*")
    public Object searchFeaturesNoPage(String searchNm) {
        try {
            return featureService.search("nam?" + searchNm + ";");
        } catch (Exception ex) {
            error(ex);
            return "[]";
        }
    }

    @RequestMapping(value = "isCodeExist", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "判断权限编码是否存在,唯一", notes = "*")
    public Object isCodeExist(
            @ApiParam(name = "code", value = "权限编码", defaultValue = "")
            @RequestParam(name = "code") String code) {
        return featureService.isCodeExist(code);
    }

    @RequestMapping(value = "isNameExist", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "判断权限名称是否存在，同一级唯一", notes = "*")
    public Object isNameExist(
            @ApiParam(name = "name", value = "权限名称", defaultValue = "")
            @RequestParam(name = "name") String name,
            @ApiParam(name = "parentId", value = "上一级权限id")
            @RequestParam(value = "parentId") long parentId) {
        return featureService.isNameExist(name, parentId);
    }

    @RequestMapping(value = "isUrlExist", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "判断权限名称是否存在，唯一", notes = "*")
    public Object isUrlExist(
            @ApiParam(name = "url", value = "权限名称", defaultValue = "")
            @RequestParam(name = "url") String url) {
        return featureService.isUrlExist(url);
    }
}
