package  com.yihu.customer.controller.manager.user;

import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.ManageFeature;
import  com.yihu.customer.entity.ManageRoleFeature;
import com.yihu.customer.service.manager.user.ManageFeatureService;
import com.yihu.customer.service.manager.user.ManageRoleFeatureService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * Created by yww on 2016/12/29.
 */
@Controller
@RequestMapping("admin/roleFeature")
@Api(description = "管理系统角色-权限关系")
public class ManageRoleFeatureController extends BaseController {
    @Autowired
    private ManageRoleFeatureService roleFeatureService;

    @Autowired
    private ManageFeatureService featureService;

    @RequestMapping(value = "initial")
    public String initial(Model model, String obj) {
        model.addAttribute("obj", obj);
        return "user/roles/rolesFeature";
    }

    /**
     * 角色组权限列表查询接口（左侧查询出所有权限，不分页，页面检索）（右侧查询角色组已配置的权限，不分页）
     *
     * @param treeType（查询列表类型：所有权限；角色组已配置权限）
     * @param roleId
     * @return
     * @throws Exception
     */
    @RequestMapping("/searchFeatureTree")
    @ResponseBody
    public Object searchFeatureTree(String treeType, long roleId) throws Exception {
        if (treeType.equals("configFeature")) {
            return featureService.findByRoleId(roleId);
        } else {
            List<ManageFeature> features = featureService.search("");
            if (features.size() > 0) {
                for (ManageFeature m : features) {
                    ManageRoleFeature relation = roleFeatureService.findRelation(m.getId(), roleId);
                    m.setIschecked(false);
                    if (relation != null) {
                        m.setIschecked(true);
                    }
                }
            }
            return features;
        }
    }


    @RequestMapping(value = "updateFeatureConfig")
    @ApiOperation(value = "根据角色组、应用权限id批量修改,一对多")
    @ResponseBody
    public String batchUpdateRoleFeatureRelation(
            @ApiParam(name = "roleId", value = "角色组Id")
            @RequestParam(value = "roleId") long roleId,
            @ApiParam(name = "featureIds", value = "选择的应用权限ids", defaultValue = "")
            @RequestParam(name = "featureIds", required = false) String featureIds) throws Exception {
        boolean bo = false;
        //根据传入的参数获取新增、删除ids
        if (StringUtils.isEmpty(featureIds)) {
            //删除角色组下配置的所有应用权限
            bo = roleFeatureService.deleteRoleFeatureRelationByRoleId(roleId);
            if (bo)
                return write(200, "操作成功！");
            return error(-1, "操作成功！");
        }
        //获取新增和删除ids
        String[] featureIdsStr = featureIds.split(",");
        List<Long> newFeatureIds = new ArrayList<>();
        for (int i = 0; i < featureIdsStr.length; i++) {
            newFeatureIds.add(Long.parseLong(featureIdsStr[i]));
        }
        Collection<ManageRoleFeature> roleFeatures = roleFeatureService.findByField("roleId", roleId);
        if (roleFeatures == null || roleFeatures.size() == 0) {
            //原角色组不存在配置权限则直接新增
            Long[] ids = new Long[newFeatureIds.size()];
            bo = roleFeatureService.batchUpdateRoleFeatureRelation(roleId, newFeatureIds.toArray(ids), "");
            if (bo)
                return write(200, "操作成功！");
            return error(-1, "操作成功！");
        }

        String deleteFeatureIds = "";
        StringBuffer deleteBuffer = new StringBuffer();
        for (ManageRoleFeature m : roleFeatures) {
            Long featureId = m.getFeatureId();
            if (newFeatureIds.contains(featureId)) {
                newFeatureIds.remove(featureId);
                continue;
            }
            deleteBuffer.append(featureId);
            deleteBuffer.append(",");
        }
        if (deleteBuffer.length() > 0) {
            deleteFeatureIds = deleteBuffer.substring(0, deleteBuffer.length() - 1);
        }
        Long[] addFeatureIds = newFeatureIds.toArray(new Long[newFeatureIds.size()]);
        bo = roleFeatureService.batchUpdateRoleFeatureRelation(roleId, addFeatureIds, deleteFeatureIds);
        if (bo)
            return write(200, "操作成功！");
        return error(-1, "操作成功！");
    }

//    @RequestMapping(value = "create",method = RequestMethod.POST)
//    @ApiOperation(value = "为角色组配置功能权限，单个")
//    @ResponseBody
//    public String createRoleFeature(String roleId,String featureId){
//        if(StringUtils.isEmpty(featureId)){
//            return error(-1, "权限不能为空！");
//        }
//        if(StringUtils.isEmpty(roleId)){
//            return error(-1,"角色组id不能为空！");
//        }
//        try{
//            ManageRoleFeature model = new ManageRoleFeature();
//            model.setRoleId(Long.parseLong(roleId));
//            model.setFeatureId(Long.parseLong(featureId));
//            ManageRoleFeature roleFeatureRelationNew = roleFeatureService.save(model);
//            return write(200, "操作成功");
//        }catch (Exception ex){
//            return error(-1,"操作失败！");
//        }
//    }
//    @RequestMapping(value = "delete",method = RequestMethod.POST)
//    @ApiOperation(value = "根据角色组id,权限Id删除角色组-功能权限关系")
//    @ResponseBody
//    public boolean deleteRoleFeature(
//            @ApiParam(name = "featureId",value = "功能权限id")
//            @RequestParam(value = "featureId") String featureId,
//            @ApiParam(name = "roleId",value = "角色组id")
//            @RequestParam(value = "roleId") String roleId){
//        ManageRoleFeature relation = roleFeatureService.findRelation(Long.parseLong(featureId), Long.parseLong(roleId));
//        if(null != relation){
//            roleFeatureService.delete(relation.getId());
//        }
//        return true;
//    }
//
//    @RequestMapping(value = "delete/byRoleId",method = RequestMethod.POST)
//    @ApiOperation(value = "根据角色组id删除所配置的应用权限")
//    @ResponseBody
//    public boolean deleteRoleFeatureRelationByRoleId(
//            @ApiParam(name = "roleId",value = "角色组id")
//            @RequestParam(value = "roleId") Long roleId){
//        return  roleFeatureService.deleteRoleFeatureRelationByRoleId(roleId);
//    }
//
//    @RequestMapping(value = "update/batch",method = RequestMethod.POST)
//    @ApiOperation(value = "批量修改角色组-应用权限关系，一对多")
//    @ResponseBody
//    public boolean batchUpdateRoleFeatureRelation(
//            @ApiParam(name = "roleId",value = "角色组Id")
//            @RequestParam(value = "roleId") Long roleId,
//            @ApiParam(name = "addFeatureIds",value = "要新增的featureIds",defaultValue = "")
//            @RequestParam(name = "addFeatureIds",required = false) Long[] addFeatureIds,
//            @ApiParam(name = "deleteFeatureIds",value = "要删除的featureIds",defaultValue = "")
//            @RequestParam(value = "deleteFeatureIds",required = false) String deleteFeatureIds) throws Exception{
//        roleFeatureService.batchUpdateRoleFeatureRelation(roleId,addFeatureIds,deleteFeatureIds);
//        return true;
//    }
//
//    @RequestMapping(value = "roleFeatures",method = RequestMethod.GET)
//    @ApiOperation(value = "查询角色组-功能权限关系列表---分页")
//    @ResponseBody
//    public String searchRoleFeature(
//            @ApiParam(name = "roleId", value = "角色id", defaultValue = "1")
//            @RequestParam(value = "size", required = true) long roleId,
//            @ApiParam(name = "page", value = "页码", defaultValue = "1")
//            @RequestParam(value = "page", required = false) int page,
//            @ApiParam(name = "rows", value = "分页大小", defaultValue = "15")
//            @RequestParam(value = "rows", required = false) int pageSize){
//
//        try{
//            String filters = "roleId="+roleId+";";
//            if(page<=0){
//                page = 1;
//            }
//            if (pageSize<=0){
//                pageSize = 15;
//            }
//            List list = roleFeatureService.search("", filters, "", page, pageSize);
//            long total = roleFeatureService.getCount(filters);
//            return write(200,"操作成功",page,pageSize,total,list);
//        }catch (Exception ex){
//            error(ex);
//            return error(-1,"操作失败！");
//        }
//    }
//    @RequestMapping(value = "roleFeaturesNoPage",method = RequestMethod.GET)
//    @ApiOperation(value = "查询角色组-功能权限关系列表---不分页")
//    @ResponseBody
//    public String searchRoleFeatureNoPaging(
//            @ApiParam(name = "roleId", value = "", defaultValue = "1")
//            @RequestParam(value = "roleId", required = true) String roleId) {
//        try {
//            String filters = "roleId=" + roleId + ";";
//            List list = roleFeatureService.search(filters);
//            return write(200, "操作成功", "data", list);
//        } catch (Exception ex) {
//            error(ex);
//            return error(-1, "操作失败！");
//        }
//    }
//
//    @RequestMapping(value = "isExist", method = RequestMethod.GET)
//    @ApiOperation(value = "通用根据过滤条件，判断是否存在")
//    @ResponseBody
//    public boolean getAppFeaturesFilter(
//            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
//            @RequestParam(value = "filters", required = false) String filters) throws Exception {
//        Long count = roleFeatureService.getCount(filters);
//        return count>0?true:false;
//    }
}
