package  com.yihu.customer.controller.manager.user;


import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.ManageRoleUser;
import com.yihu.customer.service.manager.user.ManageRoleUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * Created by yww on 2016/12/29.
 */
@Controller
@RequestMapping("admin/roleUser")
@Api(description = "管理系统角色组人员配置")
public class ManageRoleUserController extends BaseController {
    @Autowired
    private ManageRoleUserService roleUserService;

    @RequestMapping("/initial")
    public String initial(Model model, String obj) {
        model.addAttribute("obj", obj);
        return "user/roles/rolesUsers";
    }

    @RequestMapping(value = "update", method = RequestMethod.POST)
    @ApiOperation(value = "单个角色组人员的增删")
    @ResponseBody
    public boolean updateRoleUser(
            @ApiParam(name = "userCode", value = "人员编码")
            @RequestParam(value = "userCode") String userCode,
            @ApiParam(name = "roleId", value = "角色组id")
            @RequestParam(value = "roleId") long roleId,
            @ApiParam(name = "mode", value = "增/删，create/delete")
            @RequestParam(value = "mode") String mode) {
        if (StringUtils.equals("create", mode)) {
            return roleUserService.create(userCode, roleId);
        }
        if (StringUtils.equals("delete", mode)) {
            ManageRoleUser roleUser = roleUserService.findRelation(userCode, roleId);
            if (null != roleUser) {
                roleUserService.delete(roleUser.getId());
            }
        }
        return true;
    }


    @RequestMapping(value = "create", method = RequestMethod.POST)
    @ApiOperation(value = "为角色组配置人员，单个")
    @ResponseBody
    public boolean createRoleUser(
            @ApiParam(name = "userCode", value = "人员编码")
            @RequestParam(value = "userCode") String userCode,
            @ApiParam(name = "roleId", value = "角色组id")
            @RequestParam(value = "roleId") long roleId) {
        return roleUserService.create(userCode, roleId);
    }

    @RequestMapping(value = "delete", method = RequestMethod.POST)
    @ApiOperation(value = "根据角色组编码,人员Id删除角色组人员,单个")
    @ResponseBody
    public boolean deleteRoleUser(
            @ApiParam(name = "userCode", value = "人员编码")
            @RequestParam(value = "userCode") String userCode,
            @ApiParam(name = "roleId", value = "角色组id")
            @RequestParam(value = "roleId") String roleId) {
        ManageRoleUser roleUser = roleUserService.findRelation(userCode, Long.parseLong(roleId));
        if (null != roleUser) {
            roleUserService.delete(roleUser.getId());
        }
        return true;
    }

    @RequestMapping(value = "delete/roleUsers", method = RequestMethod.POST)
    @ApiOperation(value = "根据人员编码，删除其与所有角色组关系")
    @ResponseBody
    public boolean deleteRoleUserBuUserId(
            @ApiParam(name = "userCode", value = "人员编码")
            @RequestParam(value = "userCode") String userCode) throws Exception {
        Collection<ManageRoleUser> roleUsers = roleUserService.search("userCode=" + userCode);
        List<Long> ids = new ArrayList<>();
        for (ManageRoleUser roleUser : roleUsers) {
            ids.add(roleUser.getId());
        }
        roleUserService.delete(ids);
        return true;
    }

//    @RequestMapping(value = "deleteBatch/roleUsers",method = RequestMethod.POST)
//    @ApiOperation(value = "人员id,角色组ids，批量删除人员-角色组关系")
//    @ResponseBody
//    public boolean batchDeleteRoleUserRelation(
//            @ApiParam(name = "userCode",value = "人员编码")
//            @RequestParam(value = "userCode") String userCode,
//            @ApiParam(name = "roleIds",value = "角色组ids")
//            @RequestParam(value = "roleIds") String roleIds) throws Exception{
//        return roleUserService.batchDeleteRoleUserRelation(userCode, roleIds);
//    }

//    @RequestMapping(value = "create/roleUsers",method = RequestMethod.POST)
//    @ApiOperation(value = "批量新增人员所属角色组，一对多")
//    @ResponseBody
//    public boolean batchCreateRolUsersRelation(
//            @ApiParam(name = "userCode",value = "人员编码")
//            @RequestParam(value = "userCode") String userCode,
//            @ApiParam(name = "roleIds",value = "角色组ids,多个用逗号隔开")
//            @RequestParam(value = "roleIds") String roleIds) throws Exception{
//        roleUserService.batchCreateRoleUsersRelation(userCode,roleIds);
//        return true;
//    }

//    @RequestMapping(value = "update/roleUsers",method = RequestMethod.POST)
//    @ApiOperation(value = "批量修改人员所属角色组关系，一对多")
//    @ResponseBody
//    public boolean batchUpdateRoleUsersRelation(
//            @ApiParam(name = "userCode",value = "人员编码")
//            @RequestParam(value = "userCode") String userCode,
//            @ApiParam(name = "roleIds",value = "角色组ids,多个用逗号隔开")
//            @RequestParam(value = "roleIds") String roleIds) throws Exception{
//        roleUserService.batchUpdateRoleUsersRelation(userCode,roleIds);
//        return true;
//    }

    @RequestMapping(value = "roleUsers", method = RequestMethod.POST)
    @ApiOperation(value = "查询角色组人员关系列表---分页")
    @ResponseBody
    public String searchRoleUser(
            @ApiParam(name = "roleId", value = "", defaultValue = "1")
            @RequestParam(value = "roleId", required = true) String roleId,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page,
            @ApiParam(name = "rows", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "rows", required = false) int pageSize) {
        try {
            Page<ManageRoleUser> manageRoleUsers = roleUserService.searchRoleUser(roleId, page, pageSize);
            return write(200, "操作成功", page, pageSize, manageRoleUsers);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败！");
        }
    }

    @RequestMapping(value = "roleUsersNoPage", method = RequestMethod.POST)
    @ApiOperation(value = "查询角色组人员关系列表---不分页")
    @ResponseBody
    public String searchRoleUserNoPaging(
            @ApiParam(name = "roleId", value = "", defaultValue = "1")
            @RequestParam(value = "roleId", required = true) String roleId) {
        try {
            String filters = "roleId=" + roleId + ";";
            List list = roleUserService.search(filters);
            return write(200, "操作成功", "detailModelList", list);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败！");
        }
    }
}
