package  com.yihu.customer.controller.manager.user;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.ManageRoles;
import com.yihu.customer.service.manager.user.ManageRoleUserService;
import com.yihu.customer.service.manager.user.ManageRolesService;
import com.yihu.customer.util.Envelop;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;

/**
 * Created by yww on 2016/12/29.
 */
@Controller
@RequestMapping("admin/roles")
@Api(description = "管理系统用户角色")
public class ManageRolesController extends BaseController {
    @Autowired
    private ManageRolesService rolesService;

    @Autowired
    private ManageRoleUserService roleUserService;

    @Autowired
    private ObjectMapper objectMapper;

    //页面跳转
    @RequestMapping(value = "initial", method = RequestMethod.GET)
    @ApiIgnore
    public String rolesListInit() {
        return "user/roles/roles";
    }

    @RequestMapping(value = "info")
    @ApiIgnore
    public String roleInfoInit(Model model, @RequestParam(value = "id", required = false) long id, String mode) throws Exception {
        model.addAttribute("mode", mode);
        if (id > 0 && (StringUtils.equals("modify", mode) || StringUtils.equals("view", mode))) {
            model.addAttribute("obj", objectMapper.writeValueAsString(rolesService.retrieve(id)));
        } else {
            model.addAttribute("obj", "{}");
        }
        return "user/roles/rolesInfoDialog";
    }

    @RequestMapping(value = "create", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "创建角色", notes = "*")
    public String createRole(
            @ApiParam(name = "dataJson", value = "角色对象Json串")
            @RequestParam(value = "dataJson") String dataJson) {
        try {
            ManageRoles role = objectMapper.readValue(dataJson, ManageRoles.class);
            //数据验证操作
            String errorMsg = "";
            if (StringUtils.isEmpty(role.getCode())) {
                errorMsg += errorMsg + "角色编码不能为空！；";
            } else {
                if (rolesService.isCodeExist(role.getCode())) {
                    errorMsg += errorMsg + "角色编码已存在！;";
                }
            }
            if (StringUtils.isEmpty(role.getName())) {
                errorMsg += errorMsg + "角色名称不能为空！；";
            } else {
                if (rolesService.isNameExist(role.getName())) {
                    errorMsg += errorMsg + "角色名称已存在！;";
                }
            }
            if (!StringUtils.isEmpty(errorMsg)) {
                return error(-1, errorMsg);
            }
            //新增操作(新增失败抛异常）
            Envelop envelop = rolesService.createRole(role);
            if (!envelop.isSuccessFlg()) {
                return error(-1, envelop.getErrorMsg());
            }
            return write(200, "操作成功");
        } catch (JsonProcessingException jex) {
            error(jex);
            return error(-1, "JSON参数有误，请核对");
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败");
        }
    }

    @RequestMapping(value = "update", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "修改角色", notes = "*")
    public String updateRole(
            @ApiParam(name = "dataJson", value = "角色对象Json串")
            @RequestParam(value = "dataJson") String dataJson) {
        try {
            ManageRoles role = objectMapper.readValue(dataJson, ManageRoles.class);
            //数据验证操作
            String errorMsg = "";
            if (role == null || role.getId() == 0) {
                errorMsg += errorMsg + "角色id不能为空！；";
            }
            //manage角色不允许修改
            if (role.getId() == 1 || StringUtils.equals(role.getCode(), "systemManager")) {
                return error(-1, "系统管理员角色不允许修改！");
            }
            ManageRoles old = rolesService.retrieve(role.getId());
            if (StringUtils.isEmpty(role.getCode())) {
                errorMsg += errorMsg + "角色编码不能为空！；";
            } else {
                if (rolesService.isCodeExist(role.getCode()) && !StringUtils.equals(role.getCode(), old.getCode())) {
                    errorMsg += errorMsg + "角色编码已存在！;";
                }
            }
            if (StringUtils.isEmpty(role.getName())) {
                errorMsg += errorMsg + "角色名称不能为空！；";
            } else {
                if (rolesService.isNameExist(role.getName()) && !StringUtils.equals(role.getName(), old.getName())) {
                    errorMsg += errorMsg + "角色名称已存在！;";
                }
            }
            if (!StringUtils.isEmpty(errorMsg)) {
                return error(-1, errorMsg);
            }
            //新增操作(新增失败抛异常）
            Envelop envelop = rolesService.updateRole(role);
            if (!envelop.isSuccessFlg()) {
                return error(-1, envelop.getErrorMsg());
            }
            return write(200, "操作成功");
        } catch (JsonProcessingException jex) {
            error(jex);
            return error(-1, "JSON参数有误，请核对");
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败");
        }
    }

    @RequestMapping(value = "delete", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "删除角色", notes = "删除未配置人员角色，并删除角色权限关系")
    public String deleteRoleNoUser(
            @ApiParam(name = "id", value = "角色id")
            @RequestParam(name = "id") long id,
            @ApiParam(name = "code", value = "角色编码")
            @RequestParam(name = "code") String code) {
        try {
            if (id <= 0) {
                return error(-1, "角色id不能为空！");
            }
            if (id == 1 || StringUtils.equals(code, "systemManager")) {
                return error(-1, "系统管理员角色不允许删除！");
            }
            if (StringUtils.isEmpty(code)) {
                return error(-1, "角色编码不能为空！");
            }
            //判断角色组是否有配置人员，有配置则不能删除
            boolean bo = roleUserService.isRoleExistUser(id);
            if (bo) {
                return error(-1, "角色存在人员配置，请先删除！");
            }
            rolesService.deleteRoleNoUser(id, code);
            return write(200, "操作成功");
        } catch (Exception ex) {
            return error(-1, "操作失败！");
        }
    }

    @RequestMapping(value = "role", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "获取角色", notes = "根据id获取角色")
    public String getRole(
            @ApiParam(name = "id", value = "角色id")
            @RequestParam(name = "id") int id) {
        ManageRoles role = rolesService.getRole(id);
        return write(200, "操作成功", "data", role);
    }

    @RequestMapping(value = "rolesList")
    @ResponseBody
    @ApiOperation(value = "获取角色列表-分页", notes = "*")
    public String searchRoles(
            @ApiParam(name = "searchNm", value = "角色编码或名称", defaultValue = "")
            @RequestParam(value = "searchNm", required = false) String searchNm,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) Integer page,
            @ApiParam(name = "rows", value = "每页数", defaultValue = "15")
            @RequestParam(value = "rows", required = false) Integer pageSize) {
        try {
            Page<ManageRoles> manageRoles = rolesService.searchRoles(searchNm, page, pageSize);
            return write(200, "操作成功", page, pageSize, manageRoles);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败！");
        }

    }

    @RequestMapping(value = "rolesNoPage", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "获取角色列表-不分页", notes = "*")
    public String searchRolesNoPage(
            @ApiParam(name = "name", value = "角色名称", defaultValue = "")
            @RequestParam(name = "name", required = false) String name) {
        try {
            List<ManageRoles> manageRoles = rolesService.searchRolesNoPage(name);
            return write(200, "操作成功", "data", manageRoles);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "操作失败！");
        }

    }

    @RequestMapping(value = "isCodeExist", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "判断角色编码是否存在", notes = "*")
    public boolean isCodeExist(
            @ApiParam(name = "code", value = "角色编码", defaultValue = "")
            @RequestParam(name = "code") String code) {
        return rolesService.isCodeExist(code.trim());
    }

    @RequestMapping(value = "isNameExist", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "判断角色名称是否存在", notes = "*")
    public boolean isNameExist(
            @ApiParam(name = "name", value = "角色名称", defaultValue = "")
            @RequestParam(name = "name") String name) {
        return rolesService.isNameExist(name.trim());
    }

    @RequestMapping(value = "isRoleExistUser", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "判断角色是否配置人员", notes = "*")
    public boolean isRoleExistUser(
            @ApiParam(name = "id", value = "角色id", defaultValue = "")
            @RequestParam(value = "id") long id) {
        return roleUserService.isRoleExistUser(id);
    }
}
