package  com.yihu.customer.controller.manager.user;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.User;
import com.yihu.customer.service.manager.user.UserService;
import com.yihu.customer.util.MD5;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * Created by yww on 2016/12/6.
 */
@Controller
@RequestMapping(value = "admin/user")
public class UserController extends BaseController {
    @Autowired
    private UserService userService;

    @Autowired
    private ObjectMapper objectMapper;

    @RequestMapping(value = "initial", method = RequestMethod.GET)
    public String listPageInit(){
        return "user/user_list";
    }
    @RequestMapping(value = "infoInit",method = RequestMethod.GET)
    public String infoPageInit(Long id,String mode){
        request.setAttribute("id",id);
        request.setAttribute("mode",mode);
        return "user/user_modify";
    }

    @RequestMapping(value = "userList")
    @ResponseBody
    public String searchList(
            @RequestParam(value = "code",required = false) String code,
            @RequestParam(value = "name",required = false) String name,
            @RequestParam(value = "mobile",required = false) String mobile,
            @RequestParam(value= "page",required = false) Integer page,
            @RequestParam(value = "rows",required = false) Integer pageSize){
        try{
            Page<User> userPage = userService.searchList(code, name, mobile, page, pageSize);
            return write(200,"操作成功",page,pageSize,userPage);
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }

    @RequestMapping(value = "user")
    @ResponseBody
    public String getUser(
            @ApiParam(name = "id", defaultValue = "2")
            @RequestParam(name = "id", required = true) Long id) {
        try{
            User user = userService.retrieve(id);
            return write(200,"操作成功","data",user);

        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }

    @RequestMapping(value = "isCodeExist")
    @ResponseBody
    public String isCodeExist(String code){
        List<User> users = userService.findByField("code", code);
        if(users != null && users.size()>0){
            return "true";
        }
        return "false";

    }
    @RequestMapping(value = "isMobileExist")
    @ResponseBody
    public String isMobileExist(String mobile){
        List<User> users = userService.findByField("mobile", mobile);
        if(users != null && users.size()>0){
            return "true";
        }
        return "false";

    }

    @RequestMapping(value = "delete")
    @ResponseBody
    public String deleteUser(Long id){
        try{
            if (id == null) {
                return error(-1, "id不能为空！");
            }
            User user = userService.retrieve(id);
            if (StringUtils.equals(user.getCode(), "admin")) {
                return error(-1, "超级管理员admin不允许删除！");
            }
            userService.deleteUser(id);
            return write(200,"操作成功！");
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }

    }

    @RequestMapping(value = "create")
    @ResponseBody
    public String createUser(String jsonData){
        try{
            User user = objectMapper.readValue(jsonData, User.class);
            String mobile = user.getMobile();
            if(mobile == null){
                return write(-1,"手机号不能为空");
            }
            user.setCzrq(new Date());
            String password=user.getMobile().substring(5);
            String salt= UUID.randomUUID().toString().replace("-", "");
            password = MD5.GetMD5Code("a"+password+"*"+salt);
            user.setPassword(password);
            user.setSalt(salt);
            User userNew = userService.save(user);
            return write(200,"操作成功！");
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }

    }
    @RequestMapping(value = "update")
    @ResponseBody
    public String updateUser(String jsonData){
        try{
    
            JSONObject data = JSONObject.parseObject(jsonData);
            
            String oldpsw = data.getString("oldpsw");
            
            User user = objectMapper.readValue(jsonData, User.class);
            String mobile = user.getMobile();
            if(mobile == null){
                return write(-1,"手机号不能为空");
            }
            User userNew = userService.retrieve(user.getId());
            userNew.setCode(user.getCode());
            userNew.setName(user.getName());
            userNew.setType(user.getType());
            userNew.setMobile(user.getMobile());
            userNew.setOrganizationId(user.getOrganizationId());
            userNew.setRoles(user.getRoles());
    
            String password=user.getPassword();
            
            //如果新密码不为空，则进入修改密码的分支
            if(StringUtils.isNotBlank(password)){
                
                if(StringUtils.isBlank(oldpsw)){
                    return error(-1,"旧密码不能为空");
                }
    
                oldpsw =  MD5.GetMD5Code(oldpsw+userNew.getSalt());
    
                if(!oldpsw.equals(userNew.getPassword())){
                    return error(-1,"旧密码错误，无法修改");
                }

                if(!userService.testPwd(password)){
                    return error(-1,"密码必须是包含大小写字母、数字、特殊符号的8位以上组合！");
                }
    
                String salt= UUID.randomUUID().toString().replace("-", "");
                password = MD5.GetMD5Code(password+salt);
                userNew.setPassword(password);
                userNew.setSalt(salt);
            }

//            //修改手机号对应初始化密码（因暂未提供修改密码操作）-手机号后六位
//            if(userNew.getMobile() != user.getMobile()){
//                String password=mobile.substring(5);
//                String salt= UUID.randomUUID().toString().replace("-", "");
//                password = MD5.GetMD5Code(password+salt);
//                userNew.setPassword(password);
//                userNew.setSalt(salt);
//            }
            userNew = userService.save(userNew);
            return write(200,"操作成功！");
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }

    }

    //初始化密码
    @RequestMapping(value = "initPwd")
    @ResponseBody
    public String initPwd(
            @RequestParam(value = "mobile",required = true)String mobile){
        try{
            boolean res = userService.initPwd(mobile);
            if(!res){
                return error(-1,"操作失败！");
            }
            return write(200,"操作成功！");
        }catch (Exception ex){
            error(ex);
            return error(-1,"操作失败！");
        }
    }


    //修改密码
    @RequestMapping(value = "changePwd",method = RequestMethod.POST)
    @ResponseBody
    public String changePwd(
            @RequestParam(value = "id")Long id,
            @RequestParam(value = "oldPwd")String oldPwd,
            @RequestParam(value = "newPwd1")String newPwd1,
            @RequestParam(value = "newPwd2")String newPwd2,
            HttpServletRequest req){
        try {

            User curUser = userService.retrieve(id);
            if(curUser==null){
                return error(-1,"用户不存在");
            }

            if(StringUtils.isBlank(oldPwd)){
                return error(-1,"旧密码不能为空");
            }
            if(StringUtils.isBlank(newPwd1)){
                return error(-1,"新密码不能为空");
            }
            if(!newPwd1.equals(newPwd2)){
                return error(-1,"两次输入的新密码不一致");
            }

            if(!userService.testPwd(newPwd1)){
                return error(-1,"密码必须是包含大小写字母、数字、特殊符号的8位以上组合");
            }

            oldPwd =  MD5.GetMD5Code(oldPwd+curUser.getSalt());

            if(!oldPwd.equals(curUser.getPassword())){
                return error(-1,"旧密码错误，无法修改");
            }
            newPwd1 = MD5.GetMD5Code(newPwd1+curUser.getSalt());
            curUser.setPassword(newPwd1);
            userService.save(curUser);

            //更新缓存
            req.getSession().setAttribute("authPw",userService.saveUserPwlimitDate(curUser.getCode()));
            return write(200,"操作成功");
        }catch (Exception ex){
            error(ex);
            return error(-1, "操作失败！");
        }
    }

    //根据姓名获取所有有效用户列表（用于分页下拉框）
    @RequestMapping("/searchUsers")
    @ResponseBody
    public Object searchUsers(String searchNm, int page, int rows) {
        try {
            Page<User> users = userService.searchUsers(searchNm, page, rows);
            return write(200, "操作成功", page, rows, users);
        } catch (Exception ex) {
            error(ex);
            return error(-1,"操作失败！");
        }
    }

}
