package  com.yihu.customer.controller.synergy.gdp;

import  com.yihu.customer.controller.BaseController;
import  com.yihu.customer.entity.gdp.GdpSurveyCheck;
import  com.yihu.customer.entity.gdp.GdpSurveyRecord;
import com.yihu.customer.service.gdp.HealthGdpService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * Created by yeshijie on 2021/11/8.
 */
@RestController
@RequestMapping(value = "/synergy/healthGdp",produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(description = "健康gdp相关")
public class HealthGdpSurveyController extends BaseController {

    @Autowired
    private HealthGdpService healthGdpService;

    @RequestMapping(value = "searchGdpSurveyRecords",method = RequestMethod.GET)
    public String searchGdpSurveyRecords(String companyType, String industryType, String companyName, Integer page, Integer rows) {
        try {
            Page<GdpSurveyRecord> records = healthGdpService.searchGdpSurveyRecords(companyType, industryType, companyName, page, rows);
            return write(200, "操作成功！", page, rows, records);
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }

    @RequestMapping(value = "searchGdpSurveyChecks",method = RequestMethod.GET)
    public String searchGdpSurveyChecks(String companyName, Integer page, Integer rows) {
        try {
            Page<GdpSurveyCheck> records = healthGdpService.searchGdpSurveyChecks(companyName, page, rows);
            return write(200, "操作成功！", page, rows, records);
        } catch (Exception e) {
            error(e);
            return error(-1, "操作失败！");
        }
    }

    @RequestMapping(value = "findCheckById",method = RequestMethod.GET)
    @ApiOperation("查询单法人单位核查")
    public String findCheckById(@ApiParam(name = "id",value = "id")
                                @RequestParam(name = "id",required =true) Long id){
        try {
            return write(200,"查询成功","data",healthGdpService.findCheckById(id));
        }catch (Exception e){
            error(e);
            return error(-1,e.getMessage());
        }
    }

    @RequestMapping(value = "findRecordById",method = RequestMethod.GET)
    @ApiOperation("查询单位经营情况")
    public String findRecordById(@ApiParam(name = "id",value = "id")
                                 @RequestParam(name = "id",required =true) Long id){
        try {
            return write(200,"查询成功","data",healthGdpService.findRecordById(id));
        }catch (Exception e){
            error(e);
            return error(-1,e.getMessage());
        }
    }

    @RequestMapping(value = "delRecordById",method = RequestMethod.GET)
    @ApiOperation("删除单位经营情况")
    public String delRecordById(@ApiParam(name = "id",value = "id")
                                @RequestParam(name = "id",required =true) Long id){
        try {
            healthGdpService.delRecordById(id);
            return success("操作成功");
        }catch (Exception e){
            error(e);
            return error(-1,e.getMessage());
        }
    }

    @RequestMapping(value = "delCheckById",method = RequestMethod.GET)
    @ApiOperation("删除单法人单位核查")
    public String delCheckById(@ApiParam(name = "id",value = "id")
                                 @RequestParam(name = "id",required =true) Long id){
        try {
            healthGdpService.delCheckById(id);
            return success("操作成功");
        }catch (Exception e){
            error(e);
            return error(-1,e.getMessage());
        }
    }

    @ApiOperation("导出word")
    @RequestMapping(value = "exportSurvey", method = RequestMethod.GET)
    public void writeWordFile(@ApiParam(name = "ids", value = "id串，多个逗号间隔", required = true)
                              @RequestParam(value = "ids", required = true) String ids,
                              @ApiParam(name = "type", value = "id串，多个逗号间隔", required = true)
                              @RequestParam(value = "type", required = true) Integer type,
                              HttpServletResponse response) {
       try {
           File file = healthGdpService.writeWordFile(ids,type,response);
           if (file != null) {
               response.setCharacterEncoding("utf-8");
               response.setContentType("multipart/form-data");
               response.setHeader("Content-Disposition", "attachment;fileName=service_summary.zip");

               InputStream inputStream = new FileInputStream(file);
               OutputStream outputStream = response.getOutputStream();
               byte[] b = new byte[2048];
               int length = 0;
               while ((length = inputStream.read(b)) > 0) {
                   outputStream.write(b, 0, length);
               }
               outputStream.flush();
               outputStream.close();
               inputStream.close();
           } else {
               response.setStatus(500);
           }
        } catch (Exception e) {
            e.printStackTrace();
           response.setStatus(500);
        }
    }

    @ApiOperation("导出EXCEL")
    @RequestMapping(value = "exportSurveyExcel", method = RequestMethod.GET)
    public void writeExcelFile(@ApiParam(name = "ids", value = "id串，多个逗号间隔", required = true)
                              @RequestParam(value = "ids", required = true) String ids,
                              @ApiParam(name = "type", value = "id串，多个逗号间隔", required = true)
                              @RequestParam(value = "type", required = true) Integer type,
                              HttpServletResponse response) {
        try {
            File file = healthGdpService.writeExcelFile(ids,type,response);
            if (file != null) {
                response.setCharacterEncoding("utf-8");
                response.setContentType("multipart/form-data");
                response.setHeader("Content-Disposition", "attachment;fileName=service_summary.zip");

                InputStream inputStream = new FileInputStream(file);
                OutputStream outputStream = response.getOutputStream();
                byte[] b = new byte[2048];
                int length = 0;
                while ((length = inputStream.read(b)) > 0) {
                    outputStream.write(b, 0, length);
                }
                outputStream.flush();
                outputStream.close();
                inputStream.close();
            } else {
                response.setStatus(500);
            }
        } catch (Exception e) {
            e.printStackTrace();
            response.setStatus(500);
        }
    }
}
