/*******************************************************************************
 * Copyright (c) 2005, 2014 springside.github.io
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 *******************************************************************************/
package  com.yihu.customer.repository;

import java.util.List;

import  com.yihu.customer.entity.Doctor;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.PagingAndSortingRepository;

public interface DoctorDao extends PagingAndSortingRepository<Doctor, Long>, JpaSpecificationExecutor<Doctor> {

	Doctor findByName(String name);
    @Query("select count(1) from Doctor d where d.idCard = ?1 and d.status = 1 ")
    int isExistIdCard(String idCard);

	// 按医生标识查询医生信息
	@Query("select p from Doctor p where p.code = ?1")
	Doctor findByCode(String code);

    @Query("select p from Doctor p where p.code in ?1 and p.level in ?2")
    List<Doctor> findByCodesAndLevel(String[] code, Integer[] level);


	// 手机号查询医生信息
	@Query("select p from Doctor p where p.mobile = ?1 and p.status = 1")
	Doctor findByMobile(String mobile);

    // 手机号唯一性判断
    @Query("select count(1) from Doctor d where d.mobile = ?1 and d.status = 1")
    int isExistMobile(String mobile);

	// 手机号查询医生信息
	@Query("select p from Doctor p where p.mobile = ?1 and p.id <> ?2")
	List<Doctor> findByMobileNotId(String mobile, long id);

	@Query("select p from Doctor p where p.hospital=?1  and p.level = ?2 and p.iscertified=1 and p.status=1 ")
	List<Doctor> getDoctorCombobxByHospital(String hospital, String type);

    @Modifying
    @Query(value = "update Doctor p set p.status=?1 where p.id=?2 ")
    int updateStatus(Integer status, Long id);

    @Query("select p from Doctor p where p.name = ?1 and p.hospital = ?2")
    List<Doctor> findByNameHospital(String mobile, String hospital);

    @Query("select p from Doctor p where p.name = ?1 and p.idCard = ?2")
    List<Doctor> findByNameIdcard(String mobile, String idcard);

    @Modifying
    @Query(value = "update Doctor p set p.password=?1,p.salt=?2 where p.id=?3 ")
    int initPassword(String password, String salt, Long id);

    // 身份证查询医生信息
    @Query("select p from Doctor p where p.idCard = ?1 and p.status = 1")
    Doctor findByIdCard(String idcard);

    //搜索及列表查询
    @Query("SELECT p  FROM Doctor p, WlyyUserRole r, WlyyRole ro " +
            " WHERE r.user = p.code AND r.role = ro.code AND p.status = 1 AND p.name = ?1 AND p.idCard = ?2")
    Page<Doctor> findByFilter(String name, String idcard, Pageable pageable);

    //搜索及列表查询
    @Query("SELECT p  FROM Doctor p, WlyyUserRole r, WlyyRole ro " +
            " WHERE r.user = p.code AND r.role = ro.code AND p.status = 1 AND p.name = ?1 ")
    Page<Doctor> findByName(String name, Pageable pageable);

    //搜索及列表查询
    @Query("SELECT p  FROM Doctor p, WlyyUserRole r, WlyyRole ro " +
            " WHERE r.user = p.code AND r.role = ro.code AND p.status = 1 AND p.idCard = ?1 ")
    Page<Doctor> findByIdcard(String idcard, Pageable pageable);

    //搜索及列表查询
    @Query("SELECT p  FROM Doctor p, WlyyUserRole r, WlyyRole ro " +
            " WHERE r.user = p.code AND r.role = ro.code AND p.status = 1 ")
    Page<Doctor> findByAllFilter(Pageable pageable);

    //搜索及列表查询
    @Query("SELECT p FROM Doctor p where p.status= 1 ")
    Page<Doctor> findByFilterAll(Pageable pageable);


    //搜索及列表查询不在wlyy_user_role表的医生
    @Query(value = "SELECT p FROM Doctor p where p.status= 1 AND p.name = ?1 AND p.idCard = ?2 ")
    Page<Doctor> findDoctorByFilter(String name, String idcard, Pageable pageable);

    //搜索及列表查询不在wlyy_user_role表的医生
    @Query(value = "SELECT p FROM Doctor p where p.status= 1 AND p.name = ?1 ")
    Page<Doctor> findDoctorByName(String name, Pageable pageable);

    //搜索及列表查询不在wlyy_user_role表的医生
    @Query(value = "SELECT p FROM Doctor p where p.status= 1 AND p.idCard = ?1 ")
    Page<Doctor> findDoctorByIdcard(String idcard, Pageable pageable);

    //搜索及列表查询
    @Query("SELECT p  FROM Doctor p, WlyyUserRole r, WlyyRole ro " +
            " WHERE r.user = p.code AND p.hospital = ro.code AND p.status = 1 AND p.code = ?1 ")
    Doctor findListByCode(String code);
}
