package com.yihu.customer.service;

import com.yihu.customer.util.DateUtil;
import com.yihu.customer.util.HttpManageClientUtil;
import com.yihu.customer.util.SystemConf;
import  com.yihu.customer.entity.SMS;
import  com.yihu.customer.entity.User;
import  com.yihu.customer.model.LoginVo;
import  com.yihu.customer.repository.SMSDao;
import  com.yihu.customer.repository.UserDao;
import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Component
@Transactional(rollbackFor = Exception.class)
public class SMSService extends BaseService {

    @Autowired
    private SMSDao smsDao;
    @Autowired
    private UserDao userDao;

    @Value("${spring.sms.yihu_sms_url}")
    private String yihu_sms_url;
    @Value("${spring.sms.yihu_sms_code}")
    private String yihu_sms_code;
    @Value("${spring.sms.yihu_sms_name}")
    private String yihu_sms_name;
    @Value("${spring.sms.yihu_sms_password}")
    private String yihu_sms_password;
    @Value(("${doctorAssistant.api}")+"/wlyygc/doctor/message")
    private String messageApi;

    public LoginVo check(String mobile, Integer type, String captcha) {
        // 根据手机号和验证码查询对应的短信信息
        // 排序
        Sort sort = Sort.by(Sort.Direction.DESC, "id");
        // 分页信息
        LoginVo loginVo = new LoginVo();
        if(type == 22){
            User user = userDao.findByMobile(mobile);
            mobile = user.getMobile();
            loginVo.setUser(user);
        }

        PageRequest pageRequest = PageRequest.of(0, 1, sort);
        Page<SMS> page = smsDao.findByCaptcha(mobile, captcha, type, pageRequest);
        SMS sms = null;
        for (SMS temp : page) {
            if (temp != null) {
                sms = temp;
                break;
            }
        }
        // 验证码校验
        String message = null;
        if (sms == null) {
            // 验证码错误
            message = "请输入正确的验证码";
        } else if (type != sms.getType()) {
            // 验证码无效，也视为错误
            message = "请输入正确的验证码";
        } else if (sms.getDeadline().before(new Date())) {
            // 验证码过期
            message = "验证码已过期" ;
        }
        loginVo.setMessage(message);
        return loginVo;
    }


    /**
     * 外部短信调用 发送短信验证码接口
     *
     * @param mobile 手机号
     * @param type   消息类型:22 管理系统登录
     * @return
     * @throws Exception
     */
    public String sendWithContent(String mobile, int type) throws Exception {
        // 同一ip一天不允许超过10条短信
        String today = DateUtil.getStringDateShort();
        Date begin = DateUtil.strToDate(today + " 00:00:00", DateUtil.YYYY_MM_DD_HH_MM_SS);
        Date end = DateUtil.strToDate(today + " 23:59:59", DateUtil.YYYY_MM_DD_HH_MM_SS);
        // }
        // 一天的短信数不允许超过10条
        int count = smsDao.countByMobile(mobile, begin, end);
        if (count >= SystemConf.MAX_SMS_MOBILE) {
            return "当前手机号：[" + mobile + "]今天最多发送" + SystemConf.MAX_SMS_MOBILE + "条短信！";
        }
        // 保存短信发送记录
        SMS sms = new SMS();
        // 生成6位随机验证码
        String captcha = String.valueOf(Math.random()).substring(2, 8);
        String content = "您的登录验证码为："+captcha;
        Date date = new Date();
        // 延后5分钟
        sms.setContent(content);
        sms.setDeadline(DateUtil.getNextMin(date, type == 10 ? 30 : 5));
        sms.setCzrq(date);
        sms.setCaptcha(captcha);
        sms.setMobile(mobile);
        sms.setIp("127.0.0.1");
        sms.setType(type);
        sms.setStatus(1);
        JSONObject json=null;
        try {
            List<NameValuePair> par = new ArrayList<NameValuePair>();
            par.add(new BasicNameValuePair("mobiles", mobile));
            par.add(new BasicNameValuePair("content", content));
            String result = HttpManageClientUtil.post(messageApi+"/sendMessage",par,"UTF-8");
            json = new JSONObject(result);
        }catch (Exception e){
            e.printStackTrace();
        }
        if (json == null) {
            // 发送失败
            throw new ServiceException("短信发送失败！");
        }else if(1!=json.getInt("successNum")){
            throw new ServiceException("短信发送失败！");
        } else {
            //发送成功，保存到数据库
            smsDao.save(sms);
        }
        return "ok";
    }

    /**
     * 发送短信
     *
     * @param mobile
     * @param content
     * @return
     */
    public JSONObject sendMsg(String mobile, String content) {
        // 调用总部发送信息的接口
        String result = HttpManageClientUtil.post(yihu_sms_url, buildSmsParams(content, mobile), "GBK");
        JSONObject json = toJson(result);
        System.out.println("json="+json);
        return json;
    }

    public List<NameValuePair> buildSmsParams(String content, String mobile) {
        List<NameValuePair> params = new ArrayList<NameValuePair>();
        params.add(new BasicNameValuePair("SpCode", yihu_sms_code));
        params.add(new BasicNameValuePair("LoginName", yihu_sms_name));
        params.add(new BasicNameValuePair("Password", yihu_sms_password));
        params.add(new BasicNameValuePair("MessageContent", content));
        params.add(new BasicNameValuePair("UserNumber", mobile));
        params.add(new BasicNameValuePair("SerialNumber", String.valueOf(System.currentTimeMillis())));
        params.add(new BasicNameValuePair("ScheduleTime", ""));
        params.add(new BasicNameValuePair("f", "1"));
        return params;
    }

    public static JSONObject toJson(String result) {
        JSONObject json = new JSONObject();
        try {
            String[] temps = result.split("&");
            for (String temp : temps) {
                if (temp.split("=").length != 2) {
                    continue;
                }
                String key = temp.split("=")[0];
                String value = temp.split("=")[1];
                json.put(key, value);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return json;
    }

//    public static void main(String[] args) {

        // JSONObject params = new JSONObject();
        // params.put("SpCode", SystemConf.SMS_SP_CODE);
        // params.put("LoginName", SystemConf.SMS_LOGIN_NAME);
        // params.put("Password", SystemConf.SMS_PASSWORD);
        // params.put("MessageContent", "您的找回密码验证码为：123456");
        // params.put("UserNumber", "18559687019");
        // params.put("SerialNumber", "");
        // params.put("ScheduleTime", "");
        // params.put("f", 1);
        //
        // // String result = HttpUtil.sendPost(SystemConf.SMS_URL, params.toString(), "GBK");
        // String result = HttpClientUtil.post(SystemConf.SMS_URL, buildSmsParams("您的找回密码验证码为：123456", "18559687019"), "GBK");
        // JSONObject json = toJson(result);


        // System.out.println(json.toString());
        // System.out.println(json.getInt("result"));
        // if (json.getInt("result") != 0) {
        // System.out.println("短信发送失败！");
        // }

//        System.out.println(toJson("result=0&description=发送短信成功&taskid=22624861237&faillist=&task_id=22624861237"));
//    }

}
