package com.yihu.customer.service.common.district;

 
 

import com.yihu.customer.service.BaseService;
import  com.yihu.customer.entity.*;
import  com.yihu.customer.repository.*;
import com.yihu.customer.service.common.excel.ExcelData;
import com.yihu.customer.util.ReadExcelUtil;
import jxl.Sheet;
import jxl.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 省市区三级业务处理类
 *
 * @author George
 */
@Service
public class DistrictService extends BaseService {

    @Autowired
    private ProvinceDao provinceDao;
    @Autowired
    private CityDao cityDao;
    @Autowired
    private TownDao townDao;
    @Autowired
    private StreetDao streetDao;
    @Autowired
    private HospitalDao hospitalDao;
    @Autowired
    private CountryDao countryDao;

    /**
     * 查询省市区三级目录
     *
     * @param type 1一级目录，2二级目录，3三级目录，4街道目录
     * @param code 目录标识
     * @return
     */
    public List<?> findByType(int type, String code) {
        switch (type) {
            case 1:
                return findProvince();
            case 2:
                return findCity(code);
            case 3:
                return findTown(code);
            case 4:
                return findStreet(code);
            case 5:
                return findCountry(code);
        }
        return null;
    }

    /**
     * 查询省市区三级目录
     *
     * @param type 1一级目录，2二级目录，3三级目录，4街道目录
     * @param code 目录标识
     * @return
     */
    public List<?> findByType(int type, String code, List<Map<String, String>> roleMap) {
        switch (type) {
            case 1:
                return findProvince();
            case 2:
                return findCity(code);
            case 3:
                return findTown(code, roleMap);
            case 4:
                return findStreet(code);
            case 5:
                return findHospital(code);
        }
        return null;
    }


    /**
     * 查询所有的省份信息
     *
     * @return
     */
    public List<Province> findProvince() {
        List<Province> list = new ArrayList<Province>();
        Sort sort = Sort.by(Sort.Direction.DESC, "id");
        Iterable<Province> iterable = provinceDao.findAll(sort);
        if (iterable != null) {
            Iterator<Province> it = iterable.iterator();
            while (it != null && it.hasNext()) {
                list.add(it.next());
            }
        }
        return list;
    }

    /**
     * 查询省份下的城市信息
     *
     * @param province 省编码
     * @return
     */
    public List<City> findCity(String province) {
        List<City> list = new ArrayList<City>();
        Iterable<City> iterable = cityDao.findByProvince(province);
        if (iterable != null) {
            Iterator<City> it = iterable.iterator();
            while (it != null && it.hasNext()) {
                list.add(it.next());
            }
        }
        return list;
    }

    /**
     * 查询城市下的区县信息
     *
     * @param city 城市编码
     * @return
     */
    public List<Town> findTown(String city) {
        List<Town> list = new ArrayList<Town>();
        Iterable<Town> iterable = townDao.findByCity(city);
        if (iterable != null) {
            Iterator<Town> it = iterable.iterator();
            while (it != null && it.hasNext()) {
                list.add(it.next());
            }
        }
        return list;
    }


    /**
     * 查询城市下的区县信息(包含权限控制)
     *
     * @param city 城市编码
     * @return
     */
    public List<Town> findTown(String city, List<Map<String, String>> roleMap) {
        List<Town> list = new ArrayList<Town>();
        Iterable<Town> iterable = townDao.findByCity(city);
        if (iterable != null) {
            String areaString = "";
            Boolean cityFlag = false;
            for (Map<String, String> map : roleMap) {
                String code = map.get("code");
                if ("350200".equals(code)) {
                    cityFlag = true;
                    break;
                } else if (code.length() == 6) {
                    areaString += map.get("areas") + ",";
                }
            }

            Iterator<Town> it = iterable.iterator();
            while (it != null && it.hasNext()) {
                Town town = it.next();
                String townCode = town.getCode();
                if (cityFlag) {
                    list.add(town);
                } else {
                    if (areaString.indexOf(townCode) >= 0) {
                        list.add(town);
                    }
                }
            }
        }
        return list;
    }


    /**
     * 查询城市下的街道信息
     *
     * @param town 区县编码
     * @return
     */
    public List<Street> findStreet(String town) {
        List<Street> list = new ArrayList<Street>();
        Iterable<Street> iterable = streetDao.findByTown(town);
        if (iterable != null) {
            Iterator<Street> it = iterable.iterator();
            while (it != null && it.hasNext()) {
                list.add(it.next());
            }
        }
        return list;
    }


    public List<Hospital> findHospital(String town) {
        List<Hospital> list = new ArrayList<Hospital>();
        Iterable<Hospital> iterable = hospitalDao.findByTownCode(town);
        if (iterable != null) {
            Iterator<Hospital> it = iterable.iterator();
            while (it != null && it.hasNext()) {
                list.add(it.next());
            }
        }
        return list;
    }

    /**
     * 查询城市下的街道信息
     *
     * @param street 街道编码
     * @return
     */
    public List<Country> findCountry(String street) {
        List<Country> list = new ArrayList<Country>();
        Iterable<Country> iterable = countryDao.findByStreet(street);
        if (iterable != null) {
            Iterator<Country> it = iterable.iterator();
            while (it != null && it.hasNext()) {
                list.add(it.next());
            }
        }
        return list;
    }

    public Province findProvinceByName(String name) {
        return provinceDao.findByName(name);
    }

    public City findCityByName(String name) {
        return cityDao.findByName(name);
    }

    public Town finTownByName(String name) {
        return townDao.findByName(name);
    }

    public Town finTownByNameAndCity(String name,String city) {
        return townDao.findByNameAndCity(name,city);
    }

    public Street findStreetByName(String name) {
        return streetDao.findByName(name);
    }

    public boolean importStreet(Workbook workbook) {
        Sheet[] sheets = workbook.getSheets();
        Sheet sheet = sheets[0];
        int rows = ReadExcelUtil.getRightRows(sheet);

        List<Street> streetList = new ArrayList<>();
        for (int row = 1; row < rows; row++) {  //索引从0开始，第一行为标题
            Street street = new Street();
            Map<Integer, ExcelData> mapping = mappingStreet(street);
            int finalRow = row;
            mapping.forEach((index, excelData) -> {
                String value = sheet.getCell(index, finalRow).getContents().trim();
                excelData.transform(value);
            });
            //增加查询街道是否存在,存在则不存入streetList
            Street findStreet = streetDao.findByCode(street.getCode());
            if(findStreet==null){
                streetList.add(street);
            }
        }
        streetDao.saveAll(streetList);
        return true;
    }

    private Map<Integer, ExcelData> mappingStreet(Street street) {
        Map<Integer, ExcelData> dataMap = new HashMap<>();
        //导入街道
        dataMap.put(0, new ExcelData() {
            @Override
            public void transform(String data) {
                street.setName(data);
            }
        });
        //导入街道代码
        dataMap.put(1, new ExcelData() {
            @Override
            public void transform(String data) {
                street.setCode(data);
            }
        });
        //导入升
        dataMap.put(2, new ExcelData() {
            @Override
            public void transform(String data) {
                street.setProvince(data);
            }
        });

        dataMap.put(3, new ExcelData() {
            @Override
            public void transform(String data) {
                street.setCity(data);
            }
        });

        dataMap.put(4, new ExcelData() {
            @Override
            public void transform(String data) {
                street.setTown(data);
            }
        });

        return dataMap;
    }
}
