package com.yihu.customer.service.health;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

import com.yihu.customer.service.BaseService;
import  com.yihu.customer.entity.health.*;
import  com.yihu.customer.repository.health.*;
import com.yihu.customer.util.DateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * Created by wangzhinan on 2019/1/3.
 */
@Service
@Transactional
public class HealthPlanService extends BaseService {

    @Autowired
    private HealthPlanDao planDao;
    @Autowired
    private HealthTaskDao taskDao;
    @Autowired
    private HealthTaskQuestionDao questionDao;
    @Autowired
    private HealthTaskQuestionOptionDao optionDao;
    @Autowired
    private HealthTaskRecordDao taskRecordDao;
    @Autowired
    private HealthTaskQuestionRecordDao questionRecordDao;
    @Autowired
    private HealthTaskQuestionOptionRecordDao optionRecordDao;
    @Autowired
    private HealthPlanTaskDao planTaskDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;

    /**
     * 创建计划任务
     *
     * @param plan
     * @param array
     * @return
     * @throws Exception
     */
    public Boolean create(HealthPlan plan, JSONArray array) throws Exception {
        plan.setCode(getCode());
        plan.setDel(1);
        plan.setStatus(1);
        plan.setCreateTime(new Date());
        plan.setUpdateTime(new Date());
        plan.setUsedCount(0L);
        HealthPlan healthPlan = planDao.save(plan);
        if (healthPlan == null){
            throw new Exception("健康计划保存失败!");
        }
        for (int i=0;i<array.size();i++){
            HealthPlanTask planTask = new HealthPlanTask();
            JSONObject object = array.getJSONObject(i);
            String taskCode = object.getString("taskCode");
            HealthTask task = taskDao.selectByCode(taskCode);
            HealthTaskRecord taskRecord = taskRecordDao.selectByCode(taskCode);
            if (taskRecord == null){
                taskRecord = new HealthTaskRecord();
                taskRecord.setCode(task.getCode());
                taskRecord.setCreateTime(new Date());
                taskRecord.setUpdateTime(new Date());
                taskRecord.setDel(task.getDel());
                taskRecord.setIntegrate(task.getIntegrate());
                taskRecord.setQuestionCount(task.getQuestionCount());
                taskRecord.setName(task.getName());
                taskRecord.setSendQuestionType(task.getSendQuestionType());
                taskRecord.setSource(task.getSource());
                taskRecord.setUseCount(task.getUseCount());
                taskRecord.setStatus(task.getStatus());
                taskRecord.setType(task.getType());
                taskRecordDao.save(taskRecord);
            }
            List<HealthTaskQuestion> questions = questionDao.selectByHealthTaskCode(taskCode);
            List<HealthTaskQuestionRecord> questionRecords = questionRecordDao.selectByHealthTaskCode(taskCode);
            if (questionRecords == null || questionRecords.size()==0){
                for (HealthTaskQuestion question:questions){
                    HealthTaskQuestionRecord questionRecord = new HealthTaskQuestionRecord();
                    questionRecord.setAnalysis(question.getAnalysis());
                    questionRecord.setCode(question.getCode());
                    questionRecord.setCreateTime(new Date());
                    questionRecord.setUpdateTime(new Date());
                    questionRecord.setDel(question.getDel());
                    questionRecord.setHealthTaskCode(question.getHealthTaskCode());
                    questionRecord.setQuestionType(question.getQuestionType());
                    questionRecord.setTitle(question.getTitle());
                    questionRecordDao.save(questionRecord);
                }
            }
            for (HealthTaskQuestion question:questions){
                List<HealthTaskQuestionOption> options = optionDao.selectByQuestionCode(question.getCode());
                List<HealthTaskQuestionOptionRecord> optionRecords = optionRecordDao.selectByQuestionCode(question.getCode());
                if (optionRecords == null || optionRecords.size() ==0){
                    for (HealthTaskQuestionOption option:options){
                        HealthTaskQuestionOptionRecord optionRecord = new HealthTaskQuestionOptionRecord();
                        optionRecord.setCode(option.getCode());
                        optionRecord.setContent(option.getContent());
                        optionRecord.setCorredAnswer(option.getCorredAnswer());
                        optionRecord.setDel(option.getDel());
                        optionRecord.setQuestionCode(option.getQuestionCode());
                        optionRecord.setCreateTime(new Date());
                        optionRecord.setUpdateTime(new Date());
                        optionRecordDao.save(optionRecord);
                    }
                }
                question.setOption(options);
            }
            planTask.setCode(getCode());
            planTask.setCreateTime(new Date());
            planTask.setUpdateTime(new Date());
            planTask.setDays(object.getInteger("days"));
            planTask.setDel(1);
            planTask.setFrequency(object.getInteger("frequency"));
            planTask.setStartTime(DateUtil.strToDateLong(object.getString("startTime")));
            planTask.setHealthPlanCode(healthPlan.getCode());
            planTask.setHealthTaskCode(taskCode);
            planTaskDao.save(planTask);
        }
        return true;
    }

    /**
     * 列表查询
     *
     * @param templateName
     * @param page
     * @param size
     * @return
     */
    public JSONObject select(String templateName,Integer page,Integer size){
        StringBuffer buffer = new StringBuffer();
        if (templateName == null || templateName ==""){
            buffer.append(" where del = 1");
        }else {
            buffer.append(" where del=1 and template_name like '%"+templateName+"%'");
        }
        String sql = "select * from wlyy_health_plan "+buffer+" LIMIT "+(page-1)*size+","+size;
        List<HealthPlan> plans = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(HealthPlan.class));
        String sqlCount = "select COUNT(1) AS total from wlyy_health_plan "+buffer;
        List<Map<String,Object>> rstotal = jdbcTemplate.queryForList(sqlCount);
        Long count = 0L;
        if(rstotal!=null&&rstotal.size()>0){
            count = (Long) rstotal.get(0).get("total");
        }
        JSONObject object = new JSONObject();
        object.put("currPage",page);
        object.put("pageSize",size);
        object.put("totalCount",count);
        Long  totalPage = 0L;
        if (count % size == 0) {
            totalPage = count / size;
        } else {
            totalPage = count / size + 1;
        }
        object.put("totalPage",totalPage);
        object.put("detailModelList",plans);
        return object;
    }

    /**
     * 删除计划
     *
     * @param code
     * @throws Exception
     */
    public void deleteByCode(String code) throws Exception {
        int i = planDao.deleteByCode(code);
        if (i == 0){
            throw new Exception("计划删除失败!");
        }
        List<HealthPlanTask> planTasks = planTaskDao.selectByHealthPlanCode(code);
        for (HealthPlanTask planTask:planTasks){
            int j = taskRecordDao.deleteByCode(planTask.getHealthTaskCode());
            if (j==0){
                throw new Exception("任务删除失败！");
            }
            List<HealthTaskQuestionRecord> questionRecords = questionRecordDao.selectByHealthTaskCode(planTask
            .getHealthTaskCode());
            for(HealthTaskQuestionRecord questionRecord:questionRecords){
                int z = optionRecordDao.deleteByQuestionCode(questionRecord.getCode());
                if (z==0){
                    throw new Exception("选项删除失败！");
                }
            }
            int k = questionRecordDao.deleteByHealthTaskCode(planTask.getHealthTaskCode());
            if (k==0){
                throw new Exception("问题删除失败！");
            }
        }
        int j = planTaskDao.deleteByHealthPlanCode(code);
        if (j==0){
            throw new Exception("计划任务删除失败!");
        }
    }


    /**
     * 编辑计划状态
     *
     * @param code
     * @param status
     * @throws Exception
     */
    public void updateStatusByCode(String code,Integer status) throws Exception {
        int i = planDao.updateStatusByCode(code,status);
        if (i==0){
            throw new Exception("编辑计划状态失败！");
        }
    }


    /**
     * 编辑计划任务
     *
     * @param plan
     * @param array
     * @throws Exception
     */
    public void update(HealthPlan plan,JSONArray array) throws Exception {
        plan.setDel(1);
        plan.setUpdateTime(new Date());
        HealthPlan healthPlan = planDao.save(plan);
        if (healthPlan == null){
            throw new Exception("健康计划保存失败!");
        }
        List<HealthPlanTask> planTasks = planTaskDao.selectByHealthPlanCode(plan.getCode());
        for (HealthPlanTask planTask:planTasks){
            HealthTaskRecord taskRecord = taskRecordDao.selectByCode(planTask.getHealthTaskCode());
            if (taskRecord != null){
                int i =taskRecordDao.delete(planTask.getHealthTaskCode());//物理删除
                if (i==0){
                    throw new Exception("任务物理删除失败！");
                }
            }
            List<HealthTaskQuestionRecord> questionRecords = questionRecordDao.selectByHealthTaskCode(planTask.getHealthTaskCode());
            for (HealthTaskQuestionRecord questionRecord:questionRecords){
                List<HealthTaskQuestionOptionRecord> optionRecords=optionRecordDao.selectByQuestionCode(questionRecord.getCode());
                if (optionRecords !=null && optionRecords.size()!=0){
                    int j = optionRecordDao.delete(questionRecord.getCode());
                    if (j==0){
                        throw new Exception("选项物理删除失败！");
                    }
                }
            }
            if (questionRecords!= null && questionRecords.size()!=0){
                int k = questionRecordDao.delete(taskRecord.getCode());
                if (k == 0){
                    throw new Exception("问题物理删除失败！");
                }
            }
        }
        if (planTasks != null && planTasks.size()!=0){
            int z = planTaskDao.delete(plan.getCode());
            if (z==0){
                throw new Exception("计划任务物理删除失败！");
            }
        }
        for (int i=0;i<array.size();i++){
            HealthPlanTask planTask = new HealthPlanTask();
            JSONObject object = array.getJSONObject(i);
            String taskCode = object.getString("taskCode");
            String code = object.getString("code");
            HealthTask task = taskDao.selectByCode(taskCode);
            HealthTaskRecord taskRecord = taskRecordDao.selectByCode(taskCode);
            if (taskRecord == null){
                taskRecord = new HealthTaskRecord();
                taskRecord.setCode(task.getCode());
                taskRecord.setCreateTime(new Date());
                taskRecord.setUpdateTime(new Date());
                taskRecord.setDel(task.getDel());
                taskRecord.setIntegrate(task.getIntegrate());
                taskRecord.setQuestionCount(task.getQuestionCount());
                taskRecord.setName(task.getName());
                taskRecord.setSendQuestionType(task.getSendQuestionType());
                taskRecord.setSource(task.getSource());
                taskRecord.setUseCount(task.getUseCount());
                taskRecord.setStatus(task.getStatus());
                taskRecord.setType(task.getType());
                taskRecordDao.save(taskRecord);
            }
            List<HealthTaskQuestion> questions = questionDao.selectByHealthTaskCode(taskCode);
            List<HealthTaskQuestionRecord> questionRecords = questionRecordDao.selectByHealthTaskCode(taskCode);
            if (questionRecords == null || questionRecords.size()==0){
                for (HealthTaskQuestion question:questions){
                    HealthTaskQuestionRecord questionRecord = new HealthTaskQuestionRecord();
                    questionRecord.setAnalysis(question.getAnalysis());
                    questionRecord.setCode(question.getCode());
                    questionRecord.setCreateTime(new Date());
                    questionRecord.setUpdateTime(new Date());
                    questionRecord.setDel(question.getDel());
                    questionRecord.setHealthTaskCode(question.getHealthTaskCode());
                    questionRecord.setQuestionType(question.getQuestionType());
                    questionRecord.setTitle(question.getTitle());
                    questionRecordDao.save(questionRecord);
                }
            }
            for (HealthTaskQuestion question:questions){
                List<HealthTaskQuestionOption> options = optionDao.selectByQuestionCode(question.getCode());
                List<HealthTaskQuestionOptionRecord> optionRecords = optionRecordDao.selectByQuestionCode(question.getCode());
                if (optionRecords == null || optionRecords.size() ==0){
                    for (HealthTaskQuestionOption option:options){
                        HealthTaskQuestionOptionRecord optionRecord = new HealthTaskQuestionOptionRecord();
                        optionRecord.setCode(option.getCode());
                        optionRecord.setContent(option.getContent());
                        optionRecord.setCorredAnswer(option.getCorredAnswer());
                        optionRecord.setDel(option.getDel());
                        optionRecord.setQuestionCode(option.getQuestionCode());
                        optionRecord.setCreateTime(new Date());
                        optionRecord.setUpdateTime(new Date());
                        optionRecordDao.save(optionRecord);
                    }
                }
            }
            if (code == null || code ==""){
                planTask.setCode(getCode());
            }else {
                planTask.setCode(code);
            }
            planTask.setCreateTime(new Date());
            planTask.setUpdateTime(new Date());
            planTask.setDays(object.getInteger("days"));
            planTask.setDel(1);
            planTask.setFrequency(object.getInteger("frequency"));
            planTask.setStartTime(DateUtil.strToDateLong(object.getString("startTime")));
            planTask.setHealthPlanCode(healthPlan.getCode());
            planTask.setHealthTaskCode(taskCode);
            planTaskDao.save(planTask);
        }
    }


    public HealthPlan selectByCode(String code){
        HealthPlan plan = planDao.selectByCode(code);
        String sql = "select * from wlyy.wlyy_health_task_record htr where htr.del=1 " +
                "and htr.code in(select hpt.health_task_code from wlyy.wlyy_health_plan_task hpt " +
                "where hpt.del=1 and hpt.health_plan_code = '"+code+"')";
        List<HealthTaskRecord> taskRecords = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(HealthTaskRecord.class));
        plan.setTaskRecords(taskRecords);

        return plan;
    }

}
