package com.yihu.customer.service.health;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.customer.service.BaseService;
import  com.yihu.customer.entity.health.HealthTask;
import  com.yihu.customer.entity.health.HealthTaskQuestion;
import  com.yihu.customer.entity.health.HealthTaskQuestionOption;
import  com.yihu.customer.repository.health.HealthTaskDao;
import  com.yihu.customer.repository.health.HealthTaskQuestionDao;
import  com.yihu.customer.repository.health.HealthTaskQuestionOptionDao;
import com.yihu.customer.service.common.excel.ExcelData;
import com.yihu.customer.util.ReadExcelUtil;
import jxl.Sheet;
import jxl.Workbook;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

import static com.sun.tools.javac.util.Assert.error;

/**
 * Created by wangzhinan on 2019/1/3.
 */
@Service
@Transactional
public class HeathTaskService extends BaseService {


    @Autowired
    private HealthTaskDao taskDao;
    @Autowired
    private HealthTaskQuestionDao questionDao;
    @Autowired
    private HealthTaskQuestionOptionDao optionDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;

    /**
     * 创建健康任务
     *
     * @param task
     * @param array
     * @return
     * @throws Exception
     */
    public Boolean create(HealthTask task, JSONArray array) throws Exception {
        task.setCreateTime(new Date());
        task.setUpdateTime(new Date());
        task.setDel(1);
        task.setCode(getCode());
        task.setStatus(1);
        HealthTask healthTask = taskDao.save(task);
        if (healthTask == null){
            throw new Exception("健康任务保存失败!");
        }
        for (int i=0;i<array.size();i++){
            JSONObject jsonObject = array.getJSONObject(i);
            HealthTaskQuestion question = new HealthTaskQuestion();
            question.setAnalysis(jsonObject.getString("analysis"));
            question.setCode(getCode());
            question.setCreateTime(new Date());
            question.setUpdateTime(new Date());
            question.setDel(1);
            question.setTitle(jsonObject.getString("title"));
            question.setHealthTaskCode(task.getCode());
            question.setQuestionType(jsonObject.getInteger("questionType"));
            HealthTaskQuestion healthTaskQuestion= questionDao.save(question);
            if (healthTaskQuestion == null){
                throw new Exception("健康任务问题保存失败!");
            }
            JSONArray jsonArray = JSONArray.parseArray(jsonObject.getString("options"));
            for (int j=0;j<jsonArray.size();j++){
                JSONObject object = jsonArray.getJSONObject(j);
                HealthTaskQuestionOption option = new HealthTaskQuestionOption();
                option.setCode(getCode());
                option.setCreateTime(new Date());
                option.setUpdateTime(new Date());
                option.setQuestionCode(healthTaskQuestion.getCode());
                option.setDel(1);
                option.setContent(object.getString("content"));
                option.setCorredAnswer(object.getInteger("corredAnswer"));
                HealthTaskQuestionOption taskQuestionOption =  optionDao.save(option);
                if (taskQuestionOption == null){
                    throw new Exception("健康问题选项保存失败!");
                }
            }
        }
        return true;
    }


    /**
     * 删除健康任务
     *
     * @param taskCode
     * @throws Exception
     */
    public void delete(String taskCode) throws Exception {
        int i = taskDao.deleteByCode(taskCode);
        if (i==0){
            throw new Exception("任务删除失败");
        }
        int j = questionDao.deleteByHealthTaskCode(taskCode);
        if (j==0){
            throw new Exception("问题删除失败");
        }
        List<HealthTaskQuestion> questions = questionDao.selectByHealthTaskCode(taskCode);
        for (HealthTaskQuestion question:questions){
            int z = optionDao.deleteByQuestionCode(question.getCode());
            if (z==0){
                throw new Exception("选项删除失败");
            }
        }
    }

    /**
     * 根据code获取任务信息
     *
     * @param code
     * @return
     */
    public HealthTask selectByCode(String code){
        HealthTask task = taskDao.selectByCode(code);
        List<HealthTaskQuestion> questions = questionDao.selectByHealthTaskCode(code);
        for (HealthTaskQuestion question:questions){
            List<HealthTaskQuestionOption> options = optionDao.selectByQuestionCode(question.getCode());
            question.setOption(options);
        }
        task.setQuestions(questions);
        return task;
    }


    /**
     * 修改任务状态
     *
     * @param code
     * @param status
     * @throws Exception
     */
    public void updateStatusByCode(String code,Integer status) throws Exception {
        int i = taskDao.updateStatusByCode(code,status);
        if (i==0){
            throw new Exception("更新失败!");
        }
    }

    /**
     * 获取列表
     *
     * @param type
     * @param status
     * @param name
     * @return
     */
    public JSONObject select(Integer type,Integer status,String name,Integer page,Integer size){
        StringBuffer buffer =new StringBuffer();
        if (type == null&&status == null&&(name == null|| name=="")){
            buffer.append(" where del = 1");
        }else if (type != null&&status ==null&&(name == null|| name=="")){
            buffer.append(" where del =1 and  type = "+type);
        }else if (type == null&&status !=null&&(name == null|| name=="")){
            buffer.append(" where del =1 and status = "+status);
        }else if (type == null&&status !=null&&(name != null&& name!="")){
            buffer.append(" where del =1 and name like '%"+name+"%'");
        }else if (type != null&&status !=null&&(name != null&& name!="")){
            buffer.append(" where del =1 and  type="+type+" and status = "+status+" and name like '%"+name+"%'");
        }
        String sql = "select * from wlyy_health_task "+buffer +" LIMIT "+(page-1)*size+","+size;
        List<HealthTask> tasks = jdbcTemplate.query(sql,new BeanPropertyRowMapper(HealthTask.class));
        String sqlCount = "select COUNT(1) AS total from wlyy_health_task "+buffer;
        List<Map<String,Object>> rstotal = jdbcTemplate.queryForList(sqlCount);
        Long count = 0L;
        if(rstotal!=null&&rstotal.size()>0){
            count = (Long) rstotal.get(0).get("total");
        }
        JSONObject object = new JSONObject();
        object.put("currPage",page);
        object.put("pageSize",size);
        object.put("totalCount",count);
        Long  totalPage = 0L;
        if (count % size == 0) {
            totalPage = count / size;
        } else {
            totalPage = count / size + 1;
        }
        object.put("totalPage",totalPage);
        object.put("detailModelList",tasks);
        return object;
    }


    /**
     * 编辑健康任务
     *
     * @param task
     * @param array
     * @throws Exception
     */
    public Boolean update(HealthTask task,JSONArray array) throws Exception {
        task.setUseCount(0L);
        task.setDel(1);
        task.setUpdateTime(new Date());
        HealthTask healthTask = taskDao.save(task);
        if (healthTask == null){
            throw new Exception("健康任务保存失败!");
        }
        List<HealthTaskQuestion> questions = questionDao.selectByHealthTaskCode(task.getCode());
        for (HealthTaskQuestion question:questions){
            List<HealthTaskQuestionOption> options =  optionDao.selectByQuestionCode(question.getCode());
            if (options != null&& options.size() != 0){
                int z = optionDao.delete(question.getCode());
                if (z==0){
                    throw new Exception("选项删除失败");
                }
            }
        }
        if (questions != null && questions.size() != 0) {
            int k = questionDao.delete(task.getCode());
            if (k == 0) {
                throw new Exception("问题删除失败");
            }
        }
        for (int i=0;i<array.size();i++){
            JSONObject jsonObject = array.getJSONObject(i);
            HealthTaskQuestion question = new HealthTaskQuestion();
            question.setAnalysis(jsonObject.getString("analysis"));
            if (jsonObject.getString("code") == null || jsonObject.getString("code") ==""){
                question.setCode(getCode());
            }else {
                question.setCode(jsonObject.getString("code"));
            }
            question.setCreateTime(new Date());
            question.setUpdateTime(new Date());
            question.setDel(1);
            question.setTitle(jsonObject.getString("title"));
            question.setHealthTaskCode(task.getCode());
            question.setQuestionType(jsonObject.getInteger("questionType"));
            HealthTaskQuestion healthTaskQuestion= questionDao.save(question);
            if (healthTaskQuestion == null){
                throw new Exception("健康任务问题保存失败!");
            }
            JSONArray jsonArray = JSONArray.parseArray(jsonObject.getString("options"));
            for (int j=0;j<jsonArray.size();j++){
                JSONObject object = jsonArray.getJSONObject(j);
                HealthTaskQuestionOption option = new HealthTaskQuestionOption();
                if (object.getString("code") == null || object.getString("code") ==""){
                    option.setCode(getCode());
                }else {
                    option.setCode(object.getString("code"));
                }
                option.setCreateTime(new Date());
                option.setUpdateTime(new Date());
                option.setQuestionCode(healthTaskQuestion.getCode());
                option.setDel(1);
                option.setContent(object.getString("content"));
                option.setCorredAnswer(object.getInteger("corredAnswer"));
                HealthTaskQuestionOption taskQuestionOption =  optionDao.save(option);
                if (taskQuestionOption == null){
                    throw new Exception("健康问题选项保存失败!");
                }
            }
        }
        return true;
    }

    @Transactional(rollbackFor = Exception.class)
    public  List<HealthTaskQuestion> importData(Workbook workbook) throws Exception {
        Sheet[] sheets = workbook.getSheets();
        Sheet sheet = sheets[0];
        int rows = ReadExcelUtil.getRightRows(sheet);
        List<HealthTaskQuestion> healthTaskQuestions = new ArrayList<>();

        HealthTaskQuestion healthTaskQuestion;
        for (int row = 1; row < rows; row++) {  //索引从0开始，第一行为标题

            healthTaskQuestion=new HealthTaskQuestion();
            healthTaskQuestion.setCode(getCode());
            Map<Integer, ExcelData> mapping = mappingHealthTaskQuestion(healthTaskQuestion);
            int finalRow = row;
            mapping.forEach((index, excelData) -> {
                String value = sheet.getCell(index, finalRow).getContents().trim();
                excelData.transform(value);
            });
            if (!(null != healthTaskQuestion.getOption() && healthTaskQuestion.getOption().size() > 0)) {
                throw new Exception("第" + row + "行，未录入答案！");
            } else {
                List<HealthTaskQuestionOption> healthTaskQuestionOptions=new ArrayList<>();
                healthTaskQuestionOptions = healthTaskQuestion.getOption().stream()
                        .filter(model -> StringUtils.isNotBlank(model.getContent())).collect(Collectors.toList());
                if (null != healthTaskQuestionOptions && healthTaskQuestionOptions.size() > 0) {
//                    optionDao.save(healthTaskQuestionOptions);
                    healthTaskQuestion.setOption(healthTaskQuestionOptions);
                }else{
                    throw new Exception("第" + row + "行，未录入答案！");
                }
            }
            healthTaskQuestions.add(healthTaskQuestion);
        }
//        questionDao.save(healthTaskQuestions);
        return healthTaskQuestions;
    }


    private Map<Integer, ExcelData> mappingHealthTaskQuestion(HealthTaskQuestion healthTaskQuestion) throws Exception {
        List<HealthTaskQuestionOption> healthTaskQuestionOptions = new ArrayList<>();
        Map<Integer, ExcelData> dataMap = new HashMap<>();
        //问题类型
        dataMap.put(1, new ExcelData() {
            @Override
            public void transform(String data) {
                //1、单选题，2、判断题，3、多选题
                if ("单选题".equals(data)) {
                    healthTaskQuestion.setQuestionType(1);
                } else if ("判断题".equals(data)) {
                    healthTaskQuestion.setQuestionType(2);
                } else if ("多选题".equals(data)) {
                    healthTaskQuestion.setQuestionType(3);
                } else {
                    error("问题类型不正确！");
                }
            }
        });
        //问题
        dataMap.put(2, new ExcelData() {
            @Override
            public void transform(String data) {
                healthTaskQuestion.setTitle(data);
            }
        });
        int j = 3;
        for (int i = 0; i < 5; i++) {
            //选项
            HealthTaskQuestionOption   healthTaskQuestionOption = initHealthTaskQuestionOption(healthTaskQuestion);
            dataMap.put(j, new ExcelData() {
                @Override
                public void transform(String data) {
                    healthTaskQuestionOption.setContent(data);
                }
            });
            j++;
            //是否正确 1、正确，0、错误
            dataMap.put(j , new ExcelData() {
                @Override
                public void transform(String data) {
                    if ("是".equals(data)) {
                        healthTaskQuestionOption.setCorredAnswer(1);
                    } else if ("否".equals(data)) {
                        healthTaskQuestionOption.setCorredAnswer(0);
                    }

                }
            });
            j++;
            healthTaskQuestionOptions.add(healthTaskQuestionOption);
        }
        //解析
        dataMap.put(13, new ExcelData() {
            @Override
            public void transform(String data) {
                healthTaskQuestion.setAnalysis(data);
            }
        });
        healthTaskQuestion.setOption(healthTaskQuestionOptions);
        return dataMap;
    }

    /**
     * 问题答案初始化
     * @param healthTaskQuestion 问题
     * @return
     */
    public HealthTaskQuestionOption initHealthTaskQuestionOption(HealthTaskQuestion healthTaskQuestion){
        HealthTaskQuestionOption healthTaskQuestionOption =new HealthTaskQuestionOption();
        healthTaskQuestionOption.setQuestionCode(healthTaskQuestion.getCode());
        healthTaskQuestionOption.setCode(getCode());
        healthTaskQuestionOption.setDel(1);
        healthTaskQuestionOption.setCreateTime(new Date());
        healthTaskQuestionOption.setUpdateTime(new Date());
        return healthTaskQuestionOption;
    }


}
