package com.yihu.customer.service.integratedquery;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.customer.util.DateUtil;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

/**
 * Created by janseny on 2019/2/18.
 */
@Service
@Transactional
public class IntegratedqueryService {

    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private ObjectMapper objectMapper;

    /**
     * 厦门I健康微信公众号 各团队关注数
     * @param startDate
     * @param endDate
     * @return
     */
    public List<Map<String, Object>> getTeamAttentionCount(String startDate, String endDate,String team){
        String sql = "SELECT count(*) count,fa.admin_team_code team from wlyy_patient wp,wlyy_sign_family fa" +
                " where wp.`code` = fa.patient and wp.openid is not null and wp.openid <> '' and fa.admin_team_code is not NULL";
        if(StringUtils.isNotEmpty(startDate) && StringUtils.isNotEmpty(endDate)){
            sql += " and wp.openid_time >= '" + startDate + "' and wp.openid_time <= '" + endDate + " 23:59:59' ";
        }
        if(StringUtils.isNotEmpty(team) ){
            sql += " and fa.admin_team_code =" + team;
        }
        sql += " GROUP BY fa.admin_team_code";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        if(list != null){
            return list;
        }else {
            return null;
        }
    }

    /**
     * 统计 已完成线上长处方 数
     * @param startDate
     * @param endDate
     * @return
     */
    public List<Map<String, Object>> getTeamPrescriptionCount(String startDate, String endDate,String team,String area){
        String sql = "SELECT count(p.id) count , p.admin_team_id team from wlyy_prescription p LEFT JOIN dm_hospital h ON p.hospital=h.code ";
        if(StringUtils.isNotEmpty(startDate) && StringUtils.isNotEmpty(endDate)){
            sql += " and p.create_time >= '" + startDate + "' and p.create_time <= '" + endDate + " 23:59:59' ";
        }
        if(StringUtils.isNotEmpty(team) ){
            sql += " and p.admin_team_id =" + team;
        }

        if(StringUtils.isNotEmpty(area) ){
            sql += " and h.town='" + area+"'";
        }
        sql += " GROUP BY p.admin_team_id";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        if(list != null){
            return list;
        }else {
            return null;
        }
    }

    /**
     * 设备24小时异常总数
     * @param startDate
     * @param endDate
     * @param team
     * @return
     */
    public List<Map<String, Object>> getDevice24ExceptionCount(String startDate, String endDate,String team,String area){
        String sql = "SELECT count(*) count,d.grant_admin_team team FROM device.wlyy_patient_health_index p " +
                " left join device.wlyy_devices d on d.device_code=p.device_sn LEFT JOIN wlyy.dm_hospital h ON d.grant_org_code=h.code " +
                " where p.del=1 and  p.status=1 and p.manage_result=2 and d.grant_admin_team is not NULL";
        if(StringUtils.isNotEmpty(startDate) && StringUtils.isNotEmpty(endDate)){
            sql += " and czrq >= '" + startDate + "' and czrq <= '" + endDate + " 23:59:59' ";
        }
        if(StringUtils.isNotEmpty(team) ){
            sql += " and d.grant_admin_team =" + team;
        }
        if(StringUtils.isNotEmpty(area) ){
            sql += " and h.town='" + area+"'";
        }
        sql += " GROUP BY d.grant_admin_team";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        if(list != null){
            return list;
        }else {
            return null;
        }
    }

    public List<Map<String, Object>> getPatientBirthdayCount(String startDate, String endDate,String team,String area){
        String sql = "select count(*) count,f.admin_team_code team from wlyy_patient p left join wlyy_sign_family f on p.code = f.patient  ";
        if (StringUtils.isNotEmpty(area)) {
            sql += "LEFT JOIN dm_hospital h ON f.hospital=h.code ";
        }
        sql += " where f.admin_team_code IS NOT NULL ";
        if(StringUtils.isNotEmpty(startDate) && StringUtils.isNotEmpty(endDate)){
            Integer compare = DateUtil.strToDate(startDate.substring(5), "MM-dd").compareTo(DateUtil.strToDate(endDate.substring(5), "MM-dd"));
            if(compare < 0) {
                sql += " and DATE_FORMAT(CASE LENGTH(p.idcard) WHEN 18 THEN SUBSTR(p.idcard, 9, 6) WHEN 15 THEN SUBSTR(p.idcard, 7, 6) END, '%m-%d') between DATE_FORMAT('" + startDate + "', '%m-%d') and DATE_FORMAT('" + endDate + "', '%m-%d') ";
            }else {
                sql += " and (DATE_FORMAT(CASE LENGTH(p.idcard) WHEN 18 THEN SUBSTR(p.idcard, 9, 6) WHEN 15 THEN SUBSTR(p.idcard, 7, 6) END, '%m-%d') between DATE_FORMAT('" + startDate + "', '%m-%d') and DATE_FORMAT('2018-12-31', '%m-%d') ) or "+
                        "(DATE_FORMAT(p.birthday, '%m-%d') between DATE_FORMAT('2019-01-01', '%m-%d') and DATE_FORMAT('" + endDate + "', '%m-%d') ) ";
            }
        }
        if(StringUtils.isNotEmpty(team) ){
            sql += " and admin_team_code =" + team;
        }
        if(StringUtils.isNotEmpty(area) ){
            sql += " and h.town='" + area+"'";
        }
        sql += " group by admin_team_code";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        if(list != null){
            return list;
        }else {
            return null;
        }
    }

    /**
     * 获取计免绑定量
     * @param startDate 开始时间
     * @param endDate 结束时间
     * @param team 团队
     * @param area 区域
     * @return
     */
    public List<Map<String, Object>> getChildFamilyImmuneCount(String startDate, String endDate,String team,String area){
        String sql = "SELECT  count(*) count,f.admin_team_code team FROM wlyy_child_family_immune c " +
                "LEFT JOIN wlyy_sign_family f ON c.family_code=f.patient LEFT JOIN dm_hospital h ON f.hospital=h.`code` WHERE c.del = 0 and f.admin_team_code is not null   ";
        if(StringUtils.isNotEmpty(startDate) && StringUtils.isNotEmpty(endDate)){
            sql += " and  DATE_FORMAT(c.create_time, '%y-%m-%d') between DATE_FORMAT('" + startDate + "', '%y-%m-%d') and DATE_FORMAT('" + endDate + "','%y-%m-%d') ";
        }
        if(StringUtils.isNotEmpty(team) ){
            sql += " and f.admin_team_code =" + team;
        }
        if(StringUtils.isNotEmpty(area) ){
            sql += " and h.town='" + area+"'";
        }
        sql += " group by f.admin_team_code";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        if(list != null){
            return list;
        }else {
            return null;
        }
    }







}
