package com.yihu.customer.service.jw;

import  com.yihu.customer.entity.Patient;
import  com.yihu.customer.entity.reservation.ReservationHospital;
import  com.yihu.customer.entity.reservation.ReservationHospitalDept;
import  com.yihu.customer.repository.PatientDao;
import com.yihu.customer.util.DateUtil;
import com.yihu.customer.util.HttpManageClientUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Created by hzp on 2016/11/14.
 * 基卫健康市民健康服务
 */
@Service
public class JwSmjkService {

    //基卫服务地址
    @Value("${sign.check_upload}")
    private String jwUrl;
    @Autowired
    private PatientDao patientDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;

//    private ObjectMapper objectMapper = new ObjectMapper();

//    @Autowired
//    private HttpClientUtil HttpClientUtil;

    /****************************************************************************************************************************/
    /**
     * （内网）获取转诊预约医生号源信息
     */
    public String getRegDeptSpeDoctorSectionList(String OrgCode,String DeptCode,String strStart,String strEnd,String DocCode)  throws Exception
    {
        String re = "";
        String url = jwUrl + "/third/smjk/RegDeptSpeDoctorSectionList";

        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("OrgCode", OrgCode));
        params.add(new BasicNameValuePair("DeptCode", DeptCode));
        params.add(new BasicNameValuePair("strStart", strStart));
        params.add(new BasicNameValuePair("strEnd", strEnd));
        params.add(new BasicNameValuePair("DocCode", DocCode));

        String response =  HttpManageClientUtil.post(url, params, "UTF-8");

        if (!StringUtils.isEmpty(response)) {
            JSONObject jsonObject = new JSONObject(response);
            int status = jsonObject.getInt("status");
            if (status == 200) {
                String data = jsonObject.getString("data");
                if (!StringUtils.isEmpty(data) && (data.startsWith("error")||data.startsWith("System-Error"))) {
                    throw new Exception(data);
                } else {
                    re = data;
                }
            } else {
                throw new Exception(jsonObject.getString("msg"));
            }
        } else {
            throw new Exception("null response.");
        }

        return re;
    }

    /**
     * （内网）预约挂号接口
     */
    public String webRegisterByFamily(String idcard,String patientName,String ssid,String sectionType,String startTime,String orgCode,String deptCode,String deptName,String doctorCode,String doctorName,String patientPhone)  throws Exception
    {
        String re = "";
        String url = jwUrl + "/third/smjk/WebRegisterByFamily";

        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("idcard", idcard));
        params.add(new BasicNameValuePair("patientName", patientName));
        params.add(new BasicNameValuePair("ssid", ssid));
        params.add(new BasicNameValuePair("sectionType", sectionType));
        params.add(new BasicNameValuePair("startTime", startTime));
        params.add(new BasicNameValuePair("orgCode", orgCode));
        params.add(new BasicNameValuePair("deptCode", deptCode));
        params.add(new BasicNameValuePair("deptName", deptName));
        params.add(new BasicNameValuePair("doctorCode", doctorCode));
        params.add(new BasicNameValuePair("doctorName", doctorName));
        params.add(new BasicNameValuePair("patientPhone", patientPhone));

        String response = HttpManageClientUtil.post(url, params, "UTF-8");

        if (!StringUtils.isEmpty(response)) {
            JSONObject jsonObject = new JSONObject(response);
            int status = jsonObject.getInt("status");
            if (status == 200) {
                String data = jsonObject.getString("data");
                if (!StringUtils.isEmpty(data) && (data.startsWith("error")||data.startsWith("System-Error"))) {
                    throw new Exception(data);
                } else {
                    re = data;
                }
            } else {
                throw new Exception(jsonObject.getString("msg"));
            }
        } else {
            throw new Exception("null response.");
        }

        //返回挂号单
        return re;
    }

    /**
     * 获取医院列表
     * @param type
     * @return
     * @throws Exception
     */
    public String GetOrgList(String type) throws Exception{
        String url = jwUrl + "/third/guahao/GetOrgList";
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("type", type));

        String response = HttpManageClientUtil.post(url, params, "UTF-8");

        return response;
    }

    /**
     * 获取科室接口
     * @param hospitalId
     * @return
     * @throws Exception
     */
    public String GetOrgDepList(String hospitalId) throws Exception{
        String url = jwUrl + "/third/guahao/GetOrgDepList";
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("hospitalId", hospitalId));

        String response = HttpManageClientUtil.post(url, params, "UTF-8");
        return response;
    }

    /**
     * 从数据库总获取医院列表
     *
     * @param type
     * @param name
     * @return
     * @throws Exception
     */
    public List<ReservationHospital> GetOrgListToMysql(String type, String name) throws Exception{

        StringBuffer buffer = new StringBuffer();
        if (StringUtils.isNoneBlank(name)){
            buffer.append(" and rh.name like '%"+name+"%'");
        }else {
            buffer.append("");
        }
        String sql = "select *  from wlyy.wlyy_reservation_hospital rh where 1=1 and rh.type= '"+type+"' "+buffer;
        List<ReservationHospital> reservationHospitals = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(ReservationHospital.class));
        return reservationHospitals;
    }

    /**
     * 从数据库总获取科室
     *
     * @param hospitalId
     * @param name
     * @return
     * @throws Exception
     */
    public List<ReservationHospitalDept> GetOrgDepListToMysql(String hospitalId, String name) throws Exception{
        StringBuffer buffer = new StringBuffer();
        if (StringUtils.isNoneBlank(name)){
            buffer.append(" and rh.name like '%"+name+"%'");
        }else {
            buffer.append("");
        }
        String sql = "select *  from wlyy.wlyy_reservation_hospital_dept rh where 1=1 AND rh.hospital_code = '"+hospitalId+"' "+buffer;
        List<ReservationHospitalDept> reservationHospitals = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(ReservationHospitalDept.class));
        return reservationHospitals;
    }

    /**
     * 获取医生列表接口
     * @param hospitalId
     * @return
     * @throws Exception
     */
    public String GetDoctorList(String hospitalId,String hosDeptId) throws Exception{
        String url = jwUrl + "/third/guahao/GetDoctorList";
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("hospitalId", hospitalId));
        params.add(new BasicNameValuePair("hosDeptId", hosDeptId));

        String response = HttpManageClientUtil.post(url, params, "UTF-8");
        return response;
    }

    /**
     * 获取医生排班接口(包含排班详细)
     * @param hospitalId
     * @return
     * @throws Exception
     */
    public String GetDoctorArrange(String hospitalId,String hosDeptId,String doctorId) throws Exception{
        String url = jwUrl + "/third/guahao/GetDoctorArrange";
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("hospitalId", hospitalId));
        params.add(new BasicNameValuePair("hosDeptId", hosDeptId));
        params.add(new BasicNameValuePair("doctorId", doctorId));

        String response = HttpManageClientUtil.post(url, params, "UTF-8");
        return response;
    }

    /**
     * 获取医生排班接口(一级)
     * @param hospitalId
     * @return
     * @throws Exception
     */
    public String GetDoctorArrangeSimple(String hospitalId,String hosDeptId,String doctorId) throws Exception{
        String url = jwUrl + "/third/guahao/GetDoctorArrangeSimple";
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("hospitalId", hospitalId));
        params.add(new BasicNameValuePair("hosDeptId", hosDeptId));
        params.add(new BasicNameValuePair("doctorId", doctorId));

        String response = HttpManageClientUtil.post(url, params, "UTF-8");
        return response;
    }


    /**
     * 创建挂号单
     * @param hospitalId
     * @return
     * @throws Exception
     */
    public String CreateOrder(String hospitalId,String hospitalName,String hosDeptId,String hosDeptName,
                               String doctorId,String doctorName,String arrangeDate,String patient,
                               String patientName,String cardNo,String clinicCard,String patientPhone) throws Exception{
        String re = "";
        String url = jwUrl + "/third/guahao/CreateOrder";
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("hospitalName", hospitalName));
        params.add(new BasicNameValuePair("hospitalId", hospitalId));
        params.add(new BasicNameValuePair("hosDeptId", hosDeptId));
        params.add(new BasicNameValuePair("hosDeptName", hosDeptName));
        params.add(new BasicNameValuePair("doctorId", doctorId));
        params.add(new BasicNameValuePair("doctorName", doctorName));
        params.add(new BasicNameValuePair("arrangeDate", arrangeDate));
        params.add(new BasicNameValuePair("patient", patient));
        params.add(new BasicNameValuePair("patientName", patientName));
        params.add(new BasicNameValuePair("cardNo", cardNo));
        params.add(new BasicNameValuePair("clinicCard", clinicCard));
        params.add(new BasicNameValuePair("patientPhone", patientPhone));

        String response = HttpManageClientUtil.post(url, params, "UTF-8");
        if (!StringUtils.isEmpty(response)) {
            JSONObject jsonObject = new JSONObject(response);
            int status = jsonObject.getInt("status");
            if (status == 200) {
                re = jsonObject.getString("data");
            } else {
                String msg = "处理失败，请联系管理员";
                if(!jsonObject.isNull("msg")){
                    msg = jsonObject.getString("msg");
                }
                throw new Exception(msg);
            }
        } else {
            throw new Exception("null response.");
        }
        return re;
    }


    /**
     * 取消挂号单
     * @param orderId
     * @return
     * @throws Exception
     */
    public Boolean CancelOrder(String orderId,String clinicCard) throws Exception{
        boolean re = false;
        String url = jwUrl + "/third/guahao/CancelOrder";
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("orderId", orderId));
        params.add(new BasicNameValuePair("clinicCard", clinicCard));

        String response = HttpManageClientUtil.post(url, params, "UTF-8");
        if (!StringUtils.isEmpty(response)) {
            JSONObject jsonObject = new JSONObject(response);
            int status = jsonObject.getInt("status");
            if (status == 200) {
                re = jsonObject.getBoolean("data");
            } else {
                String msg = "处理失败，请联系管理员";
                if(!jsonObject.isNull("msg")){
                    msg = jsonObject.getString("msg");
                }
                throw new Exception(msg);
            }
        } else {
            throw new Exception("null response.");
        }
        return re;
    }

    /**
     * 获取医生信息
     * @param hospitalId
     * @return
     * @throws Exception
     */
    public String GetDoctorInfo(String hospitalId,String hosDeptId,String doctorId) throws Exception{
        String re = "";
        String url = jwUrl + "/third/guahao/GetDoctorInfo";
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("hospitalId", hospitalId));
        params.add(new BasicNameValuePair("hosDeptId", hosDeptId));
        params.add(new BasicNameValuePair("doctorId", doctorId));

        String response = HttpManageClientUtil.post(url, params, "UTF-8");
        return response;
    }

    /**
     * 获取医生信息
     * @param hospitalId
     * @param hosDeptId
     * @param doctorId
     * @return
     * @throws Exception
     */
    public GuahaoDoctor getGuahaoDoctor(String hospitalId, String hosDeptId, String doctorId) throws Exception{
        String response = GetDoctorInfo(hospitalId,hosDeptId,doctorId);
        if (!StringUtils.isEmpty(response)) {
            JSONObject jsonObject = new JSONObject(response);
            int status = jsonObject.getInt("status");
            if (status == 200) {
                GuahaoDoctor doctor = new GuahaoDoctor();
                JSONObject json = jsonObject.getJSONObject("data");
                doctor.setId(json.isNull("id")?"":json.getString("id"));//医生编码
                doctor.setName(json.isNull("name")?"":json.getString("name"));//医生姓名
                doctor.setSex(json.isNull("sex")?"":json.getString("sex"));//医生性别
                doctor.setTitle(json.isNull("title")?"":json.getString("title"));//医生职称
                doctor.setEdu(json.isNull("edu")?"":json.getString("edu"));//医生学历
                doctor.setIntroduce(json.isNull("introduce")?"":json.getString("introduce"));//医生简介
                doctor.setPhoto(json.isNull("photo")?"":json.getString("photo"));//医生头像
                doctor.setFee(json.isNull("fee")?"":json.getString("fee"));//医生挂号费
                doctor.setHosDeptId(json.isNull("hosDeptId")?"":json.getString("hosDeptId"));//医生科室编码
                doctor.setHosDeptName(json.isNull("hosDeptName")?"":json.getString("hosDeptName"));//医生科室名称
                doctor.setHospitalId(json.isNull("hospitalId")?"":json.getString("hospitalId"));//医生医院编码
                doctor.setHospitalName(json.isNull("hospitalName")?"":json.getString("hospitalName"));//医生医院名称
                return doctor;
            } else {
                String msg = "处理失败，请联系管理员";
                if(!jsonObject.isNull("msg")){
                    msg = jsonObject.getString("msg");
                }
                throw new Exception(msg);
            }
        } else {
            throw new Exception("null response.");
        }
    }

    /**
     * 获取预约状态(0 撤销 1 确认 2 已诊  3停诊)
     * @param hospitalId
     * @return
     * @throws Exception
     */
    public Integer GetOrderStatus(String hospitalId,String hosDeptId,String doctorId) throws Exception{
        Integer re = null;
        String url = jwUrl + "/third/guahao/GetOrderStatus";
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("hospitalId", hospitalId));
        params.add(new BasicNameValuePair("hosDeptId", hosDeptId));
        params.add(new BasicNameValuePair("doctorId", doctorId));

        String response = HttpManageClientUtil.post(url, params, "UTF-8");
        if (!StringUtils.isEmpty(response)) {
            JSONObject jsonObject = new JSONObject(response);
            int status = jsonObject.getInt("status");
            if (status == 200) {
                re = jsonObject.getInt("data");
            } else {
                throw new Exception(jsonObject.getString("msg"));
            }
        } else {
            throw new Exception("null response.");
        }

        return re;
    }

    /**
     * 根据患者医保卡获取近三个月的预约记录
     * @param keyword
     * @return
     * @throws Exception
     */
    public String GetRegList(String keyword) throws Exception{
        Patient p = patientDao.findByIdcard(keyword);
        if(p==null){
            p = patientDao.findBySsc(keyword);
        }
        if(p==null){
            return null;
        }
        String url = jwUrl + "/third/guahao/GetRegList";
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("patient", p.getCode()));

        String response = HttpManageClientUtil.post(url, params, "UTF-8");
        return response;
    }

    /**
     * 根据患者,预约编号，机构编号获取单条记录预约记录
     * @param patient
     * @return
     * @throws Exception
     */
    public String GetRegDetail(String patient,String orgCode,String regCode) throws Exception{
        String re = "";
        String url = jwUrl + "/third/guahao/GetRegDetail";
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("patient", patient));
        params.add(new BasicNameValuePair("orgCode", orgCode));
        params.add(new BasicNameValuePair("regCode", regCode));

        String response = HttpManageClientUtil.post(url, params, "UTF-8");
        return response;
    }


    /*************************************** 旧版本函数 *************************************************************************/

    /**
     * 获取转诊预约医生列表
     * @param orgCode   机构编码
     * @param deptCode  科室编码
     * @return
     */
    public String getRegDeptSpeDoctorList(String orgCode, String deptCode) {
        String url = jwUrl + "/third/smjk/RegDeptSpeDoctorList";
        List<NameValuePair> params = new ArrayList<>();

        //当前时间;
        String strStart = DateUtil.getStringDateShort();
        //10天预约
        String strEnd = DateUtil.getNextDay(strStart, 11);

        params.add(new BasicNameValuePair("orgCode", orgCode));
        params.add(new BasicNameValuePair("deptCode", deptCode));
        params.add(new BasicNameValuePair("strStart", strStart));
        params.add(new BasicNameValuePair("strEnd", strEnd));

        String response = HttpManageClientUtil.post(url, params, "UTF-8");
        return response;
    }

    public JSONObject GetDoctorInfoTen(String hospitalId, String hosDeptId, String doctorId) throws JSONException {
        String url = jwUrl + "/third/smjk/RegDeptSpeDoctorList";
        List<NameValuePair> params = new ArrayList<>();

        //当前时间;
        String strStart = DateUtil.getStringDateShort();
        //10天预约
        String strEnd = DateUtil.getNextDay(strStart, 11);

        params.add(new BasicNameValuePair("orgCode", hospitalId));
        params.add(new BasicNameValuePair("deptCode", hosDeptId));
        params.add(new BasicNameValuePair("strStart", strStart));
        params.add(new BasicNameValuePair("strEnd", strEnd));

        String response = HttpManageClientUtil.post(url, params, "UTF-8");

//        String response = "{\"status\":200,\"msg\":\"获取转诊预约医生列表成功！\",\"data\":[{\"id\":\"02301\",\"name\":\"陈建国\",\"sex\":\"1\",\"title\":\"主任医师\",\"edu\":\"\",\"introduce\":\"青光眼、白内障。\",\"photo\":\"\",\"fee\":null,\"hosDeptId\":\"1070210\",\"hosDeptName\":\"眼科门诊\",\"hospitalId\":\"350211A1001\",\"hospitalName\":\"厦门大学附属第一医院\"},{\"id\":\"10003\",\"name\":\"庄静宜\",\"sex\":\"2\",\"title\":\"主任医师\",\"edu\":\"\",\"introduce\":\"白内障、眼底病、眼整形。\",\"photo\":\"\",\"fee\":null,\"hosDeptId\":\"1070210\",\"hosDeptName\":\"眼科门诊\",\"hospitalId\":\"350211A1001\",\"hospitalName\":\"厦门大学附属第一医院\"},{\"id\":\"10004\",\"name\":\"辛奕菁\",\"sex\":\"2\",\"title\":\"主任医师\",\"edu\":\"\",\"introduce\":\"眼视光（屈光不正、近视激光手术、OK镜验配、弱视）；眼表疾病及眼科常见疾病\",\"photo\":\"\",\"fee\":null,\"hosDeptId\":\"1070210\",\"hosDeptName\":\"眼科门诊\",\"hospitalId\":\"350211A1001\",\"hospitalName\":\"厦门大学附属第一医院\"},{\"id\":\"10005\",\"name\":\"胡艳霞\",\"sex\":\"2\",\"title\":\"主任医师\",\"edu\":\"\",\"introduce\":\"白内障、青光眼。\",\"photo\":\"\",\"fee\":null,\"hosDeptId\":\"1070210\",\"hosDeptName\":\"眼科门诊\",\"hospitalId\":\"350211A1001\",\"hospitalName\":\"厦门大学附属第一医院\"},{\"id\":\"10008\",\"name\":\"林延\",\"sex\":\"2\",\"title\":\"主任医师\",\"edu\":\"学士\",\"introduce\":\"眼底病、青光眼、眼内激光治疗。\",\"photo\":\"\",\"fee\":null,\"hosDeptId\":\"1070210\",\"hosDeptName\":\"眼科门诊\",\"hospitalId\":\"350211A1001\",\"hospitalName\":\"厦门大学附属第一医院\"},{\"id\":\"10010\",\"name\":\"丁丽远\",\"sex\":\"2\",\"title\":\"主任医师\",\"edu\":\"\",\"introduce\":\"眼底病、青光眼、眼外伤、眼表疾病。\",\"photo\":\"\",\"fee\":null,\"hosDeptId\":\"1070210\",\"hosDeptName\":\"眼科门诊\",\"hospitalId\":\"350211A1001\",\"hospitalName\":\"厦门大学附属第一医院\"},{\"id\":\"10011\",\"name\":\"罗陈川\",\"sex\":\"1\",\"title\":\"主任医师\",\"edu\":\"学士\",\"introduce\":\"眼底病、青光眼。\",\"photo\":\"\",\"fee\":null,\"hosDeptId\":\"1070210\",\"hosDeptName\":\"眼科门诊\",\"hospitalId\":\"350211A1001\",\"hospitalName\":\"厦门大学附属第一医院\"},{\"id\":\"10021\",\"name\":\"陈焓\",\"sex\":\"1\",\"title\":\"副主任医师\",\"edu\":\"学士\",\"introduce\":\"眼底病、眼外伤、白内障及眼科疾病。\",\"photo\":\"\",\"fee\":null,\"hosDeptId\":\"1070210\",\"hosDeptName\":\"眼科门诊\",\"hospitalId\":\"350211A1001\",\"hospitalName\":\"厦门大学附属第一医院\"},{\"id\":\"10054\",\"name\":\"刘显勇\",\"sex\":\"1\",\"title\":\"主任医师\",\"edu\":\"学士\",\"introduce\":\"眼科疾病。\",\"photo\":\"\",\"fee\":null,\"hosDeptId\":\"1070210\",\"hosDeptName\":\"眼科门诊\",\"hospitalId\":\"350211A1001\",\"hospitalName\":\"厦门大学附属第一医院\"},{\"id\":\"10082\",\"name\":\"邱永年\",\"sex\":\"1\",\"title\":\"主任医师\",\"edu\":\"\",\"introduce\":\"青光眼、白内障。\",\"photo\":\"\",\"fee\":null,\"hosDeptId\":\"1070210\",\"hosDeptName\":\"眼科门诊\",\"hospitalId\":\"350211A1001\",\"hospitalName\":\"厦门大学附属第一医院\"},{\"id\":\"1070210\",\"name\":\"普通号\",\"sex\":\"\",\"title\":\"\",\"edu\":\"\",\"introduce\":\"\",\"photo\":\"\",\"fee\":null,\"hosDeptId\":\"1070210\",\"hosDeptName\":\"眼科门诊\",\"hospitalId\":\"350211A1001\",\"hospitalName\":\"厦门大学附属第一医院\"}]}";

        JSONObject jsonData = new JSONObject(response);
        JSONArray  jsonArray = jsonData.getJSONArray("data");
        for (int i=0; i<jsonArray.length(); i++) {

            JSONObject jsonObject = jsonArray.getJSONObject(i);

            if(doctorId.equals(jsonObject.getString("id"))){
                return jsonObject;
            }
        }

        return null;

    }

    public String doPostReturn(String url,String jsonParam) throws JSONException {
        String response = HttpManageClientUtil.post(jwUrl+url, getJosonObjectParam(jsonParam), "UTF-8");
        return response;
    }

    public List<NameValuePair> getJosonObjectParam(String json) throws JSONException {
        List<NameValuePair> param = new ArrayList<>();
        JSONObject jsonObject = new JSONObject(json);
        Iterator<String> it =  jsonObject.keys();
        while(it.hasNext()){
            String key = it.next();
            param.add(new BasicNameValuePair(key,(String)jsonObject.get(key)));
        }
        return param;
    }
}
