package com.yihu.customer.service.manager.consult;

import com.yihu.customer.service.SMSService;
import com.yihu.customer.service.manager.message.ManageMessageService;
import com.yihu.customer.service.manager.team.ManageWlyyAdminTeamMemberService;
import com.yihu.customer.util.DateUtil;
import com.yihu.customer.util.MessageType;
import com.yihu.customer.util.PushMsgTask;
import com.yihu.customer.util.query.BaseJpaService;
import  com.yihu.customer.entity.Doctor;
import  com.yihu.customer.entity.Message;
import  com.yihu.customer.entity.MessageNoticeSetting;
import  com.yihu.customer.entity.WlyyAdminTeamMember;
import  com.yihu.customer.entity.consult.WlyyConsultRemindLog;
import  com.yihu.customer.entity.consult.WlyyReplyConsult;
import  com.yihu.customer.repository.DoctorDao;
import  com.yihu.customer.repository.MessageDao;
import  com.yihu.customer.repository.consult.WlyyConsultRemindLogDao;
import org.apache.commons.lang3.StringUtils;
import org.hibernate.Query;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * Created by 卓 on 2017/5/15.
 */
@Service
public class WlyyConsultRemindLogService extends BaseJpaService<WlyyConsultRemindLog,Long> {
    @Autowired
    private DoctorDao doctorDao;

    @Autowired
    private WlyyReplyConsultService wlyyReplyConsultService;

    @Autowired
    private WlyyConsultRemindLogDao wlyyConsultRemindLogDao;

    @Autowired
    private ManageWlyyAdminTeamMemberService manageWlyyAdminTeamMemberService;
    @Autowired
    private SMSService smsService;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private MessageDao messageDao;
    @Autowired
    private ManageMessageService manageMessageService;


     public PageImpl<WlyyConsultRemindLog> list(String city, String town, String hospital, String teamCode, String doctorName,String doctor, Date beginDate, Date endDate,String remindType, int page, int pageSize){
        if (page <= 0) {
            page = 1;
        }
        if (pageSize <= 0) {
            pageSize = 15;
        }
        // 分页信息
        PageRequest pageRequest = PageRequest.of(page - 1, pageSize);
        //过滤条件
        StringBuffer filters = new StringBuffer();
        if(StringUtils.isNotBlank(city)){
            filters.append("and city =:city ");
        }
        if(StringUtils.isNotBlank(town)){
            filters.append("and town =:town ");
        }
        if(StringUtils.isNotBlank(hospital)){
            filters.append("and hospital =:hospital ");
        }
        List<String> doctors = new ArrayList<>();
        if(StringUtils.isNotBlank(teamCode)){
            //根据团队查询需匹配团队内的医生信息
            List<WlyyAdminTeamMember> members =  manageWlyyAdminTeamMemberService.searchTeamMembers(Long.valueOf(teamCode),null);
            for(WlyyAdminTeamMember member:members){
                doctors.add(member.getDoctorCode());
            }
            filters.append("and doctor in (:doctors) ");
        }
        if(StringUtils.isNotBlank(doctorName)){
            filters.append("and doctorName  like :doctorName ");
        }
         if(StringUtils.isNotBlank(doctor)){
             filters.append("and doctor  = :doctor ");
         }
        if(beginDate!=null){
            filters.append("and remindTime>=:beginDate ");
        }
        if(endDate!=null){
            filters.append("and remindTime<=:endDate ");
        }
        if(StringUtils.isNotBlank(remindType)){
            filters.append("and remindType =:remindType ");
         }
         filters.append(" order by p.remindTime desc ");
        Query queryCount = currentSession().createQuery("select count(1) from WlyyConsultRemindLog p where 1=1 " +filters.toString());
        Query queryList = currentSession().createQuery("select p from WlyyConsultRemindLog p where 1=1  " +filters.toString());
        if (!org.springframework.util.StringUtils.isEmpty(city)) {
            queryCount.setString("city",city);
            queryList.setString("city",city);
        }
        if (!org.springframework.util.StringUtils.isEmpty(town)) {
            queryCount.setString("town",town);
            queryList.setString("town", town);        }
        if (!org.springframework.util.StringUtils.isEmpty(hospital)) {
            queryCount.setString("hospital",hospital);
            queryList.setString("hospital", hospital);
        }
        if (!org.springframework.util.StringUtils.isEmpty(teamCode)) {
            queryCount.setParameterList("doctors",doctors);
            queryList.setParameterList("doctors", doctors);
        }
        if (!org.springframework.util.StringUtils.isEmpty(doctorName)) {
            queryCount.setString("doctorName","%"+doctorName+"%");
            queryList.setString("doctorName","%"+doctorName+"%");
        }
         if (!org.springframework.util.StringUtils.isEmpty(doctor)) {
             queryCount.setString("doctor",doctor);
             queryList.setString("doctor",doctor);
         }
         if (!org.springframework.util.StringUtils.isEmpty(beginDate)) {
             queryCount.setString("beginDate", DateUtil.dateToStr(beginDate,DateUtil.YYYY_MM_DD_HH_MM_SS));
             queryList.setString("beginDate", DateUtil.dateToStr(beginDate,DateUtil.YYYY_MM_DD_HH_MM_SS));
         }
         if (!org.springframework.util.StringUtils.isEmpty(endDate)) {
             queryCount.setString("endDate",DateUtil.dateToStr(endDate,DateUtil.YYYY_MM_DD_HH_MM_SS));
             queryList.setString("endDate", DateUtil.dateToStr(endDate,DateUtil.YYYY_MM_DD_HH_MM_SS));
         }
         if (!org.springframework.util.StringUtils.isEmpty(remindType)) {
             queryCount.setString("remindType",remindType);
             queryList.setString("remindType",remindType);
         }
        //获取总条数
        int count = ((Number) queryCount.iterate().next()).intValue();
        queryList.setFirstResult(pageSize * (page - 1));
        queryList.setMaxResults(pageSize);
        List<WlyyConsultRemindLog> list = queryList.list();
        return new PageImpl<WlyyConsultRemindLog>(list,pageRequest,count);
    }

    /**
     * 提醒某个医生
     * @param doctor
     * @param remindType
     */
    public void remind(String doctor,Integer remindType,String content){
        Doctor d  = doctorDao.findByCode(doctor);
        List<WlyyReplyConsult> list = wlyyReplyConsultService.getListByFilter(d.getCity(),d.getTown(),d.getHospital(),d.getAdminTeamId(),d.getName(),d.getCode(),0,0,null,null);
        int count = list.size();
        WlyyConsultRemindLog wlyyConsultRemindLog =new WlyyConsultRemindLog();
        wlyyConsultRemindLog.setDoctor(doctor);
        wlyyConsultRemindLog.setHospital(d.getHospital());
        wlyyConsultRemindLog.setHospitalName(d.getHospitalName());
        wlyyConsultRemindLog.setCity(d.getCity());
        wlyyConsultRemindLog.setCityName(d.getCityName());
        wlyyConsultRemindLog.setRemindType(remindType);
        wlyyConsultRemindLog.setTown(d.getTown());
        wlyyConsultRemindLog.setTownName(d.getTownName());
        wlyyConsultRemindLog.setDoctorName(d.getName());
        wlyyConsultRemindLog.setRemindTime(new Date());

        wlyyConsultRemindLogDao.save(wlyyConsultRemindLog);
        if(remindType==WlyyConsultRemindLog.REMIND_SMS){
            //发送短信
            String msgContent = d.getName()+"医生，您有"+count+"条待回复咨询，请登录厦门i健康及时回复。";
            if(StringUtils.isNotBlank(content)){
                msgContent = content;
            }
            smsService.sendMsg(d.getMobile(),msgContent);
        }else if(remindType==WlyyConsultRemindLog.REMIND_SYSTEM){
            //推送系统消息
            String systemContent = "您有"+count+"条待回复咨询，请及时回复。";
            if(StringUtils.isNotBlank(content)){
                systemContent = content;
            }

            Message message = new Message();
            message.setCzrq(new Date());
            message.setCreateTime(new Date());
            message.setContent(systemContent);
            message.setRead(1);//设置未读
            message.setReceiver(doctor);//设置接受医生的code
            message.setSender("admin");//设置发送的用户
            message.setSenderName("系统");
            message.setCode(getCode());
            message.setSenderPhoto("");
            message.setConsultNum(count);
            message.setTitle("待回复咨询提醒");
            message.setType(5);//咨询提醒
            message.setReadonly(1);//是否只读消息
            message.setSignStatus("1");//新的签约
            message.setSex(1);
            message.setOver("1");//未处理
            message.setData("");
            message.setDel("1");
            if(count==1){
                String topicId = list.get(0).getCode();
                //如果是1条未回复的话，需要返回sessionId和sessionName
                String sql = "SELECT " +
                        "   ts.session_id sessionId" +
                        "   ,ts.name" +
                        " from im.topics ts " +
                        " WHERE ts.id = '"+topicId+"'";
                List<Map<String,Object>> session = jdbcTemplate.queryForList(sql);
                if(session!=null&&session.size()>0){
                    Map<String,Object> map = session.get(0);
                    message.setSessionId(map.get("sessionId").toString());
                    message.setSessionName(map.get("name").toString());
                }
            }
            messageDao.save(message);
            if(manageMessageService.getMessageNoticeSettingByMessageType(doctor,"1", MessageNoticeSetting.MessageTypeEnum.imSwitch.getValue())){
                // 发送消息给医生
                PushMsgTask.getInstance().put(doctor, MessageType.MESSAGE_TYPE_CONSULT_REPLY.D_CR_01.name(),MessageType.MESSAGE_TYPE_CONSULT_REPLY.咨询未回复.name(), systemContent, "admin");
            }
        }
    }

    public int countByFilter(String city, String town, String hospital, String teamCode, String doctorName,String doctor, Date beginDate, Date endDate) {
        //过滤条件
        StringBuffer filters = new StringBuffer();
        if (StringUtils.isNotBlank(city)) {
            filters.append("and city =:city ");
        }
        if (StringUtils.isNotBlank(town)) {
            filters.append("and town =:town ");
        }
        if (StringUtils.isNotBlank(hospital)) {
            filters.append("and hospital =:hospital ");
        }
        if (StringUtils.isNotBlank(teamCode)) {
            filters.append("and teamCode like :teamCode ");
        }
        if (StringUtils.isNotBlank(doctorName)) {
            filters.append("and (doctorName  like :doctorName or healthDoctorName like :healthDoctorName) ");
        }
        if (beginDate != null) {
            filters.append("and remindTime>=:beginDate ");
        }
        if (endDate != null) {
            filters.append("and remindTime<=:endDate ");
        }
        if (doctor != null) {
            filters.append("and (doctor  =:doctor) ");
        }
        Query queryCount = currentSession().createQuery("select count(1) from WlyyConsultRemindLog p where 1=1 " + filters.toString());
        if (!org.springframework.util.StringUtils.isEmpty(city)) {
            queryCount.setString("city", city);
        }
        if (!org.springframework.util.StringUtils.isEmpty(town)) {
            queryCount.setString("town", town);
        }
        if (!org.springframework.util.StringUtils.isEmpty(hospital)) {
            queryCount.setString("hospital", hospital);
        }
        if (!org.springframework.util.StringUtils.isEmpty(teamCode)) {
            queryCount.setString("teamCode", teamCode);
        }
        if (!org.springframework.util.StringUtils.isEmpty(doctorName)) {
            queryCount.setString("doctorName", "%" + doctorName + "%");
            queryCount.setString("healthDoctorName", "%" + doctorName + "%");
        }
        if (!org.springframework.util.StringUtils.isEmpty(beginDate)) {
            queryCount.setString("beginDate",DateUtil.dateToStr(beginDate,DateUtil.YYYY_MM_DD_HH_MM_SS));
        }
        if (!org.springframework.util.StringUtils.isEmpty(endDate)) {
            queryCount.setString("endDate",DateUtil.dateToStr(endDate,DateUtil.YYYY_MM_DD_HH_MM_SS));
        }
        if (!org.springframework.util.StringUtils.isEmpty(doctor)) {
            queryCount.setString("doctor", doctor);
        }
        //获取总条数
        int count = ((Number) queryCount.iterate().next()).intValue();
        return count;
    }
}
