package com.yihu.customer.service.manager.device;

import  com.yihu.customer.device.entity.DeviceManufacturer;
import  com.yihu.customer.device.repository.DeviceManufacturerDao;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springside.modules.persistence.DynamicSpecifications;
import org.springside.modules.persistence.SearchFilter;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by humingfen on 2018/6/6.
 * manufacturer数据库
 */
@Service
public class DeviceManufacturerService  {
    @Autowired
    private DeviceManufacturerDao deviceManufacturerDao;

    public Page<DeviceManufacturer> searchList(String manufacturerName, String manufacturerCode, String address, Integer page, Integer pageSize){
        if(page == null){
            page = 1;
        }
        if(pageSize == null){
            pageSize = 15;
        }
        PageRequest pageRequest = PageRequest.of(page - 1, pageSize);
        Map<String,SearchFilter> filter = new HashMap<>();

        if(!StringUtils.isEmpty(manufacturerName)){
            filter.put("manufacturerName",new SearchFilter("manufacturerName", SearchFilter.Operator.LIKE,manufacturerName));
        }
        if(!StringUtils.isEmpty(manufacturerCode)){
            filter.put("manufacturerCode",new SearchFilter("manufacturerCode",SearchFilter.Operator.LIKE,manufacturerCode));
        }
        if(!StringUtils.isEmpty(address)){
            filter.put("address",new SearchFilter("address",SearchFilter.Operator.LIKE,address));
        }
        Specification<DeviceManufacturer> spec = DynamicSpecifications.bySearchFilter(filter.values(), DeviceManufacturer.class);
        return deviceManufacturerDao.findAll(spec,pageRequest);
    }


    public DeviceManufacturer findById(Long id) {
        return deviceManufacturerDao.findById(id).get();
    }

    public void  updateManufacturer(DeviceManufacturer manufacturer) {
        deviceManufacturerDao.updateManufacturer(manufacturer.getManufacturerName(), manufacturer.getManufacturerCode(), manufacturer.getAddress(), manufacturer.getRepresentative(), manufacturer.getDel(), manufacturer.getId());
    }

    public void createManufacturer(DeviceManufacturer manufacturer) {
        manufacturer.setCzrq(new Date());
        deviceManufacturerDao.save(manufacturer);
    }

    public Boolean existCode(String code) {
        DeviceManufacturer manufacturer = deviceManufacturerDao.findByCode(code);
        if(manufacturer == null){
            return false;
        }
        return true;
    }

    public void deleteManufacturer(Long id) {
        deviceManufacturerDao.deleteById(id);
    }

    public List<DeviceManufacturer> findAll() {
        List<DeviceManufacturer> deviceManufacturer = (List<DeviceManufacturer>) deviceManufacturerDao.getAll();
        return deviceManufacturer;
    }
}
