package com.yihu.customer.service.manager.device;

import com.yihu.customer.service.common.excel.ExcelData;
import  com.yihu.customer.device.entity.DeviceManufacturer;
import  com.yihu.customer.device.entity.WlyyDevice;
import  com.yihu.customer.device.repository.DeviceManufacturerDao;
import  com.yihu.customer.device.repository.WlyyDeviceDao;
import  com.yihu.customer.entity.DmDevice;
import  com.yihu.customer.repository.DmDeviceDao;
import com.yihu.customer.util.ReadExcelUtil;
import jxl.Sheet;
import jxl.Workbook;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springside.modules.persistence.DynamicSpecifications;
import org.springside.modules.persistence.SearchFilter;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by yww on 2016/12/1.
 * device数据库
 */
@Service
public class WlyyDeviceService {
    @Autowired
    private WlyyDeviceDao deviceDao;
    @Autowired
    private DeviceManufacturerDao deviceManufacturerDao;
    @Autowired
    private DmDeviceDao dmDeviceDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;

    public WlyyDevice findById(Long id){
        return deviceDao.findById(id).get();
    }

    public Page<WlyyDevice> searchDeviceList(String deviceName,String deviceCode,String orgName,String linkman,Integer page,Integer pageSize){
        if(page == null){
            page = 1;
        }
        if(pageSize == null){
            pageSize = 15;
        }
        PageRequest pageRequest = PageRequest.of(page - 1, pageSize);

        Map<String,SearchFilter> filter = new HashMap<>();

        if(!StringUtils.isEmpty(orgName)){
            filter.put("orgName",new SearchFilter("orgName", SearchFilter.Operator.LIKE,orgName));
        }
        if(!StringUtils.isEmpty(deviceCode)){
            filter.put("deviceCode",new SearchFilter("deviceCode",SearchFilter.Operator.EQ,deviceCode));
        }
        if(!StringUtils.isEmpty(deviceName)){
            filter.put("deviceName",new SearchFilter("deviceName",SearchFilter.Operator.LIKE,deviceName));
        }
        if(!StringUtils.isEmpty(linkman)){
            filter.put("linkman",new SearchFilter("linkman",SearchFilter.Operator.EQ,linkman));
        }
        Specification<WlyyDevice> spec = DynamicSpecifications.bySearchFilter(filter.values(), WlyyDevice.class);
        return deviceDao.findAll(spec,pageRequest);
    }

    @Transactional
    public WlyyDevice createDevice(WlyyDevice device){
        device.setApplyDate(new Date());
        String manufacturerCode = deviceManufacturerDao.getManufacturerCodeByCode(device.getManufacturerId());
        device.setManufacturerCode(manufacturerCode);
        //根据型号设置绑定次数形式
        if(device.getDeviceModel() != null) {
            DmDevice dmDevice = dmDeviceDao.findByModel(device.getDeviceModel());
            if(dmDevice == null) {
                if(device.getDeviceName().contains("血糖")) {
                    device.setBindingCount("{\"1\":\"0\"}");
                }else if(device.getDeviceName().contains("血压")) {
                    device.setBindingCount("{\"1\":\"0\", \"2\":\"0\"}");
                }
            }else {
                if(dmDevice.getIsMultiUser().equals("1")) {
                    device.setBindingCount("{\"1\":\"0\", \"2\":\"0\"}");
                }else if(dmDevice.getIsMultiUser().equals("0")) {
                    device.setBindingCount("{\"1\":\"0\"}");
                }
            }
        }
        device.setIsGrant(0);
        device.setIsBinding(0);
        WlyyDevice deviceNew = deviceDao.save(device);
        return deviceNew;
    }

    @Transactional
    public WlyyDevice updateDevice(WlyyDevice device){
        WlyyDevice deviceNew = deviceDao.findById(device.getId()).get();
        if(deviceNew == null){
            return null;
        }
        String manufacturerCode = deviceManufacturerDao.getManufacturerCodeByCode(device.getManufacturerId());
        deviceNew.setManufacturerCode(manufacturerCode);
        deviceNew.setOrgName(device.getOrgName());
        deviceNew.setLinkman(device.getLinkman());
        deviceNew.setTel(device.getTel());
        deviceNew.setServerIp(device.getServerIp());
        deviceNew.setDeviceName(device.getDeviceName());
        deviceNew.setDeviceModel(device.getDeviceModel());
        deviceNew.setDeviceCode(device.getDeviceCode());
        deviceNew.setManufacturerId(device.getManufacturerId());
        deviceNew.setManufacturer(device.getManufacturer());
        deviceNew.setAddress(device.getAddress());
        deviceNew.setRepresentative(device.getRepresentative());
        deviceNew.setApplicantName(device.getApplicantName());
        deviceNew.setApplicantIdentity(device.getApplicantIdentity());
        deviceNew.setApplicantTel(device.getApplicantTel());
        deviceNew.setApplicantMail(device.getApplicantMail());
        deviceNew.setSim(device.getSim());
        return deviceDao.save(deviceNew);
    }

    @Transactional
    public boolean deleteDevice(Long id){
        deviceDao.deleteById(id);
        return true;
    }

    public String existDeviceCode(String deviceCode){
        WlyyDevice device = deviceDao.findByDeviceCode(deviceCode);
        if(device == null){return "false";}
        return "true";
    }
    
    
    public List<WlyyDevice> exporthDeviceList(String deviceName, String deviceCode, String orgName, String linkman){
        
        Map<String,SearchFilter> filter = new HashMap<>();
        
        if(!StringUtils.isEmpty(orgName)){
            filter.put("orgName",new SearchFilter("orgName", SearchFilter.Operator.LIKE,orgName));
        }
        if(!StringUtils.isEmpty(deviceCode)){
            filter.put("deviceCode",new SearchFilter("deviceCode",SearchFilter.Operator.EQ,deviceCode));
        }
        if(!StringUtils.isEmpty(deviceName)){
            filter.put("deviceName",new SearchFilter("deviceName",SearchFilter.Operator.LIKE,deviceName));
        }
        if(!StringUtils.isEmpty(linkman)){
            filter.put("linkman",new SearchFilter("linkman",SearchFilter.Operator.EQ,linkman));
        }
        Specification<WlyyDevice> spec = DynamicSpecifications.bySearchFilter(filter.values(), WlyyDevice.class);
        return deviceDao.findAll(spec);
    }

    public boolean importData(Workbook workbook) {
        Sheet[] sheets = workbook.getSheets();
        Sheet sheet = sheets[0];
        int rows = ReadExcelUtil.getRightRows(sheet);

        List<WlyyDevice> deviceList = new ArrayList<>();
        for (int row = 1; row < rows; row++) {  //索引从0开始，第一行为标题
            WlyyDevice device = new WlyyDevice();
            Map<Integer, ExcelData> mapping = mappingDevice(device);
            int finalRow = row;
            mapping.forEach((index, excelData) -> {
                String value = sheet.getCell(index, finalRow).getContents().trim();
                excelData.transform(value);
            });
            //判断设备厂商是否存在，不存在则存入
            if(StringUtils.isNotBlank(device.getManufacturerId())) {
                DeviceManufacturer manufacturer = deviceManufacturerDao.findByCode(device.getManufacturerId());
                if (manufacturer == null && StringUtils.isNotBlank(device.getManufacturerCode()) && StringUtils.isNotBlank(device.getManufacturer())) {
                    manufacturer = new DeviceManufacturer();
                    manufacturer.setCode(device.getManufacturerId());
                    manufacturer.setManufacturerCode(device.getManufacturerCode());
                    manufacturer.setManufacturerName(device.getManufacturer());
                    manufacturer.setCzrq(new Date());
                    deviceManufacturerDao.save(manufacturer);
                }else if(manufacturer != null){
                    device.setManufacturerCode(manufacturer.getManufacturerCode());
                    device.setManufacturer(manufacturer.getManufacturerName());
                }
            }
            device.setIsGrant(0);
            device.setIsBinding(0);
            //判断设备是否导入
            Integer count = deviceDao.existDeviceCode(device.getDeviceCode());
            if(count != 0) {
                WlyyDevice wlyyDevice = deviceDao.findByDeviceCode(device.getDeviceCode());
                device.setId(wlyyDevice.getId());
            }
            deviceList.add(device);
        }
        deviceDao.saveAll(deviceList);
        return true;
    }

    private Map<Integer, ExcelData> mappingDevice(WlyyDevice device) {
        Map<Integer, ExcelData> dataMap = new HashMap<>();
        //申请日期
        dataMap.put(1, new ExcelData() {
            @Override
            public void transform(String data) {
                if(StringUtils.isNotBlank(data)) {
                    SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    try {
                        device.setApplyDate(simpleDateFormat.parse(data));
                    } catch (ParseException e) {
                        e.printStackTrace();
                    }
                }
            }
        });
        //机构名称
        dataMap.put(2, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setOrgName(data);
            }
        });
        //联系人
        dataMap.put(3, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setLinkman(data);
            }
        });
        //联系电话
        dataMap.put(4, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setTel(data);
            }
        });
        //服务IP地址
        dataMap.put(5, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setServerIp(data);
            }
        });
        //设备名称
        dataMap.put(6, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setDeviceName(data);
            }
        });
        //设备型号
        dataMap.put(7, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setDeviceModel(data);
                DmDevice dmDevice = dmDeviceDao.findByModel(device.getDeviceModel());
                if(dmDevice != null) {
                    if(dmDevice.getIsMultiUser().equals("1")) {
                        device.setBindingCount("{\"1\":\"0\", \"2\":\"0\"}");
                    }else if(dmDevice.getIsMultiUser().equals("0")) {
                        device.setBindingCount("{\"1\":\"0\"}");
                    }
                }
            }
        });
        //设备唯一标识
        dataMap.put(8, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setDeviceCode(data);
            }
        });
        //厂商代码
        dataMap.put(9, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setManufacturerCode(data);
            }
        });
        //厂商名称
        dataMap.put(10, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setManufacturer(data);
            }
        });
        //厂商地址
        dataMap.put(11, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setAddress(data);
            }
        });
        //法人代表
        dataMap.put(12, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setRepresentative(data);
            }
        });
        //申请注册人
        dataMap.put(13, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setApplicantName(data);
            }
        });
        //申请人身份证号
        dataMap.put(14, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setApplicantIdentity(data);
            }
        });
        //申请人手机
        dataMap.put(15, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setApplicantTel(data);
            }
        });
        //申请人邮箱
        dataMap.put(16, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setApplicantMail(data);
            }
        });
        //SIM卡号
        dataMap.put(17, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setSim(data);
            }
        });
        //厂商表关联id
        dataMap.put(18, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setManufacturerId(data);
            }
        });
        //发放的患者所在团队
        dataMap.put(19, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setGrantAdminTeam(data);
            }
        });
        //发放所在机构
        dataMap.put(20, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setGrantOrgCode(data);
            }
        });
        //发放时间
        dataMap.put(21, new ExcelData() {
            @Override
            public void transform(String data) {
                if(StringUtils.isNotBlank(data)) {
                    SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    try {
                        device.setGrantTime(simpleDateFormat.parse(data));
                    } catch (ParseException e) {
                        e.printStackTrace();
                    }
                }
            }
        });
        //根据仪器名称设定绑定次数
        dataMap.put(22, new ExcelData() {
            @Override
            public void transform(String data) {
                if(device.getBindingCount() == null) {
                    if (data.contains("血压")) {
                        device.setBindingCount("{\"1\":\"0\", \"2\":\"0\"}");
                    } else if (data.contains("血糖")) {
                        device.setBindingCount("{\"1\":\"0\"}");
                    }
                }
            }
        });

        //居委会code
        dataMap.put(23, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setNeighborhoodCommitteeCode(data);
            }
        });
        //居委会名称
        dataMap.put(24, new ExcelData() {
            @Override
            public void transform(String data) {
                device.setNeighborhoodCommitteeName(data);
            }
        });

        return dataMap;
    }

    /**
     * 全部解绑后设备表更新为未绑定
     * @param deviceSn
     */
    public void unBindUpdateIsBinding(String deviceSn)throws Exception{
        String patientDeviceSql = "select count(*) num from wlyy.wlyy_patient_device pd where pd.device_sn ='"+deviceSn+"' and pd.del=0";
        Map<String,Object> map = jdbcTemplate.queryForMap(patientDeviceSql);
        int num = Integer.valueOf(String.valueOf(map.get("num")));
        String sql = "update device.wlyy_devices dd set dd.is_binding=? where dd.device_code=?";
        if (num==0){
            jdbcTemplate.update(sql,new Object[]{0,deviceSn});
            //deviceDetailDao.updateUnbindAll(0,deviceSn);
        }
        if (num==1){
            jdbcTemplate.update(sql,new Object[]{1,deviceSn});
            //deviceDetailDao.updateUnbindAll(1,deviceSn);
        }
        if (num>1){
            jdbcTemplate.update(sql,new Object[]{2,deviceSn});
            //deviceDetailDao.updateUnbindAll(2,deviceSn);
        }
    }
}
