package com.yihu.customer.service.manager.device;

import com.yihu.customer.util.query.BaseJpaService;
import  com.yihu.customer.device.entity.PatientDeviceExportVO;
import  com.yihu.customer.device.entity.WlyyPatientDeviceVO;
import  com.yihu.customer.entity.Patient;
import  com.yihu.customer.entity.WlyyPatientDevice;
import  com.yihu.customer.repository.PatientDao;
import  com.yihu.customer.repository.WlyyPatientDeviceDao;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by yww on 2016/12/7.
 * 居民设备表，wlyy数据库
 */
@Service
public class WlyyPatientDeviceService extends BaseJpaService<WlyyPatientDevice,WlyyPatientDeviceDao> {
    @Autowired
    private PatientDao patientDao;
    @Autowired
    JdbcTemplate jdbcTemplate;

    public Page<WlyyPatientDevice> searchList(String deviceName,String categoryCode,String deviceSn,String userName,Integer page,Integer pageSize) throws Exception{

        if(page <= 0){
            page = 1;
        }
        if (pageSize <= 0){
            pageSize = 15;
        }
        PageRequest pageRequest = PageRequest.of(page-1,pageSize);
        StringBuilder filters = new StringBuilder();
        if(!StringUtils.isEmpty(userName)){
            List<String> codes = patientDao.findByName(userName);
            if(codes != null && codes.size() >0){
                filters.append("user="+StringUtils.join(codes, ",")+";");
            }else {
                return new PageImpl<WlyyPatientDevice>(new ArrayList<>(), pageRequest, 0);
            }
        }
        if(!StringUtils.isEmpty(deviceSn)){
            filters.append("deviceSn?"+deviceSn+";");
        }
        if(!StringUtils.isEmpty(deviceName)){
            filters.append("deviceName?"+deviceName+";");
        }
        if(!StringUtils.isEmpty(categoryCode)){
            filters.append("categoryCode="+categoryCode+";");
        }
        List<WlyyPatientDevice> list = (List<WlyyPatientDevice>)search("", filters.toString(), "-czrq", page, pageSize);
        long count = 0;
        String regex = "(\\w{3})(\\w+)(\\w{3})";
        if(list !=null && list.size()>0){
            count = getCount(filters.toString());
            for (WlyyPatientDevice info : list){
                Patient patient = findPatient(info.getUser());
                if(StringUtils.isNotBlank(info.getUserIdcard())){
                    info.setUserIdcard(info.getUserIdcard().replaceAll(regex, "$1****$3"));
                }
                info.setUserName(patient== null?"":patient.getName());
            }
        }
        return new PageImpl<WlyyPatientDevice>(list,pageRequest,count);

    }
    //查询居民姓名
    public Patient findPatient(String patientCode){
        if(StringUtils.isEmpty(patientCode)){
            return null;
        }
        return patientDao.findByCode(patientCode);
    }
    
    /**
     * 根据社区查找居民设备绑定详情
     * @param town
     * @param hospital
     * @return
     */
	public List<PatientDeviceExportVO> exportPatientDeviceList(String town, String hospital) {
        List<PatientDeviceExportVO> list = new ArrayList<>();
        String sql  = "SELECT s.patient," +
                "s.`name`," +
                "s.mobile," +
                "CONCAT(LEFT (s.idcard,6),'**********',RIGHT (s.idcard,2)) idcard," +
                "d.device_sn," +
                "d.device_name," +
                "s.doctor_name," +
                "p.address," +
                "p.openid," +
                "CASE WHEN p.openid IS NULL THEN '未绑定' ELSE '已绑定' END isopenid " +
                "FROM wlyy_sign_family s,wlyy_patient_device d,wlyy_patient p " +
                "WHERE s.patient=d.`user` AND s.hospital='"+hospital+"' AND s.`status`> 0 AND s.patient=p.CODE ORDER BY patient";
        list =  jdbcTemplate.query(sql,new BeanPropertyRowMapper(PatientDeviceExportVO.class));
        return list ;
	}
    
    /*=
     * 根据社区或者医生名称查找居民设备绑定详情
     *
     * @param deviceName
     * @param categoryCode
     * @param deviceSn
     * @param userName
     * @param doctorName
     * @param hospital
     * @param page
     * @param pageSize
     * @return
     * @throws Exception
     */
	public List<WlyyPatientDeviceVO> searchListNew(String deviceName,
                                                   String categoryCode,
                                                   String deviceSn,
                                                   String userName,
                                                   String doctorName,
                                                   String hospitalName,
                                                   String hospitalCode,
                                                   Integer page, Integer pageSize) throws Exception{
        
        List<WlyyPatientDeviceVO> list = new ArrayList<>();
        /*String sql  = "SELECT s.patient," +
                "d.id as id," +
                "d.device_id as deviceId," +
                "d.user as user," +
                "p.name as userName," +
                "d.category_code as categoryCode," +
                "d.device_name as deviceName," +
                "d.device_sn as deviceSn," +
                "d.sim as sim," +
                "d.user_type as userType," +
                "s.mobile," +
                "CONCAT(LEFT (s.idcard,6),'**********',RIGHT (s.idcard,2)) userIdcard," +
                "s.doctor_name as doctorName," +
                "s.doctor as doctor," +
                "d.czrq as czrq," +
                "s.hospital_name as hospitalName," +
                "s.hospital as hospital," +
                "s.doctor_health_name as doctorHealthName,"+
                "s.doctor_health as doctorHealth "+
                "FROM " +
                "wlyy_sign_family s," +
                "wlyy_patient_device d," +
                "wlyy_patient p " +
                "WHERE s.patient=d.`user` ";*/
        String sql = "SELECT d.id as id,d.device_id as deviceId,d.user as user,p.name as userName,d.category_code as categoryCode,d.device_name as deviceName," +
                "d.device_sn as deviceSn,d.sim as sim,d.user_type as userType,p.mobile," +
                "CONCAT(LEFT (p.idcard,6),'**********',RIGHT (p.idcard,2)) userIdcard,d.czrq as czrq " +
                "FROM wlyy_patient_device d,wlyy_patient p " +
                "WHERE d.`user`=p.CODE ";
     /*   if(StringUtils.isNotBlank(hospitalName)){
            sql = sql + "AND s.hospital_name like '%"+hospitalName+"%' ";
        }

        if(StringUtils.isNotBlank(hospitalCode)){
            sql = sql + "AND s.hospital = '"+hospitalCode+"' ";
        }

        if(StringUtils.isNotBlank(doctorName)){
            sql = sql + "AND (s.doctor_name like '%"+doctorName+"%' or s.doctor_health_name like '%"+doctorName+"%') ";
        }*/
        
        if(StringUtils.isNotBlank(userName)){
            sql = sql + "AND p.name like '%"+userName+"%' ";
        }
        
        if(StringUtils.isNotBlank(deviceSn)){
            sql = sql + "AND d.device_sn like '%"+deviceSn+"%' ";
        }
        
        if(StringUtils.isNotBlank(categoryCode)){
            sql = sql + "AND d.category_code like '%"+categoryCode+"%' ";
        }
        
        if(StringUtils.isNotBlank(deviceName)){
            sql = sql + "AND d.device_name like '%"+deviceName+"%' ";
        }
        
       /* sql = sql +"AND s.`status`> 0 AND s.patient=p.CODE " +
                "ORDER BY czrq DESC ";*/
        
        sql = sql+"ORDER BY czrq DESC limit " + (page-1)*pageSize+", "+pageSize;
        System.out.println("list:"+sql);
        list =  jdbcTemplate.query(sql,new BeanPropertyRowMapper(WlyyPatientDeviceVO.class));
        return list ;
	}
    
    public Long getCountSearchListNew(String deviceName, String categoryCode, String deviceSn, String userName, String doctorName, String hospitalName,String hospitalCode) {
    
        String sql  = "SELECT count(d.id) " +
                "FROM " +
                " wlyy_sign_family s," +
                " wlyy_patient_device d," +
                " wlyy_patient p " +
                "WHERE " +
                " d.`user`=p.CODE ";
       /* if(StringUtils.isNotBlank(hospitalName)){
            sql = sql + "AND s.hospital_name like '%"+hospitalName+"%' ";
        }
    
        if(StringUtils.isNotBlank(hospitalCode)){
            sql = sql + "AND s.hospital = '"+hospitalCode+"' ";
        }
    
        if(StringUtils.isNotBlank(doctorName)){
            sql = sql + "AND s.doctor_name like '%"+userName+"%' ";
        }*/
    
        if(StringUtils.isNotBlank(userName)){
            sql = sql + "AND p.name like '%"+doctorName+"%' ";
        }
    
        if(StringUtils.isNotBlank(deviceSn)){
            sql = sql + "AND d.device_sn like '%"+deviceSn+"%' ";
        }
    
        if(StringUtils.isNotBlank(categoryCode)){
            sql = sql + "AND d.category_code like '%"+categoryCode+"%' ";
        }
    
        if(StringUtils.isNotBlank(deviceName)){
            sql = sql + "AND d.device_name like '%"+deviceName+"%' ";
        }
    
        sql = sql +"AND s.`status`> 0 AND s.patient=p.CODE ";
    
        System.out.println("count:"+sql);
    
        Long total = jdbcTemplate.queryForObject(sql, Long.class);
    
        return total ;
    }
}
