package com.yihu.customer.service.manager.dict;

import  com.yihu.customer.entity.DictEntryKey;
import  com.yihu.customer.entity.ManageDictEntry;
import  com.yihu.customer.repository.ManageDictEntryDao;
import com.yihu.customer.util.query.BaseJpaService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * Created by Administrator on 2017/1/24.
 */
@Service
public class ManageDictEntryService extends BaseJpaService<ManageDictEntry, ManageDictEntryDao> {
    @Autowired
    private ManageDictEntryDao dictEntryDao;

    /**
     * 下一字典项排序号。
     *
     * @param dictId
     * @return
     */
    public int getNextSN(long dictId) {
        ManageDictEntryDao repo = (ManageDictEntryDao) getJpaRepository();
        Integer nextSort = repo.getNextEntrySN(dictId);
        return null == nextSort ? 1 : nextSort + 1;
    }

    /**
     * 获取所有字典项。对于大字典，若不分页效率可能会很低。
     *
     * @param dictId 字典ID
     * @param page   分页，-1 表示查找全部
     * @param size   页大小, page 为 -1 时忽略此参数
     * @return
     */
    public Page<ManageDictEntry> getDictEntries(long dictId, int page, int size) {
        ManageDictEntryDao repo = (ManageDictEntryDao) getJpaRepository();

        return repo.findByDictId(dictId, page == -1 ? null : PageRequest.of(page, size));
    }

    /**
     * 获取简易字典项列表.
     *
     * @param dictId
     * @param codes  字典项代码列表,为空返回所有字典项. 但对于大字典效率会很低.
     * @return
     */
    public List<ManageDictEntry> getDictEntries(long dictId, String[] codes) {
        ManageDictEntryDao repo = (ManageDictEntryDao) getJpaRepository();

        if (codes == null) {
            return repo.findByDictId(dictId);
        } else {
            return repo.findByDictIdAndCodes(dictId, codes);
        }
    }

    /**
     * 获取字典项。
     *
     * @param dictId
     * @param code
     * @return
     */
    public ManageDictEntry getDictEntry(long dictId, String code) {
        ManageDictEntryDao repo = (ManageDictEntryDao) getJpaRepository();

        return repo.findById(new DictEntryKey(code, dictId)).get();
    }

    /**
     * 按字典ID与字典项值查找字典项.
     *
     * @param dictId
     * @param value
     * @param page
     * @param size
     * @return
     */
    public Page<ManageDictEntry> findByDictIdAndValueLike(long dictId, String value, int page, int size) {
        ManageDictEntryDao repo = (ManageDictEntryDao) getJpaRepository();

        return repo.findByDictIdAndValueLike(dictId, value, PageRequest.of(page, size));
    }

    /**
     * 按字典ID查找字典项.
     *
     * @param dictId
     * @param page
     * @param size
     * @return
     */
    public Page<ManageDictEntry> findByDictId(long dictId, int page, int size) {
        ManageDictEntryDao repo = (ManageDictEntryDao) getJpaRepository();

        return repo.findByDictId(dictId, PageRequest.of(page, size));
    }

    public boolean isDictContainEntry(long dictId, String code) {
        ManageDictEntryDao repo = (ManageDictEntryDao) getJpaRepository();

        ManageDictEntry manageDictEntry = repo.findById(new DictEntryKey(code, dictId)).get();

        return manageDictEntry != null;
    }

    public ManageDictEntry saveDictEntry(ManageDictEntry manageDictEntry) {
//        ManageDictEntryDao repo = (ManageDictEntryDao) getJpaRepository();

        return dictEntryDao.save(manageDictEntry);
    }

    public void createDictEntry(ManageDictEntry manageDictEntry) {
        ManageDictEntryDao repo = (ManageDictEntryDao) getJpaRepository();

        repo.save(manageDictEntry);
    }

    public void deleteDictEntry(long dictId, String code) {
//        ManageDictEntryDao repo = (ManageDictEntryDao) getJpaRepository();
//        repo.delete(new DictEntryKey(code, dictId));
        dictEntryDao.deleteById(new DictEntryKey(code, dictId));
    }
}
