package com.yihu.customer.service.manager.dmjob;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.transaction.Transactional;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springside.modules.persistence.DynamicSpecifications;
import org.springside.modules.persistence.SearchFilter;
import org.springside.modules.persistence.SearchFilter.Operator;

import  com.yihu.customer.entity.DmJob;
import  com.yihu.customer.repository.DmJobDao;

@Service
public class DmJobService {
	@Autowired
	private DmJobDao dmJobDao;

	@Transactional
	public void saveDmJob(DmJob dmJob) throws Exception{
		DmJob dmJobTemp=dmJobDao.getByName(dmJob.getName());
		if(dmJobTemp!=null){
			throw new Exception("名称重复");
		}
		dmJobDao.save(dmJob);
	}

	@Transactional
	public void updateDmJob(DmJob dmJobTemp) throws Exception {
		DmJob dmJob = dmJobDao.getByNameWithOutId(dmJobTemp.getName(),dmJobTemp.getId());
		if(dmJobTemp!=null){
			throw new Exception("名称重复");
		}
		
		dmJob=dmJobDao.findById(dmJobTemp.getId()).get();
		dmJob.setName(dmJobTemp.getName());
	}

	@Transactional
	public void deleteDmJob(Long id)  throws Exception{
		DmJob dmJob=dmJobDao.findById(id).get();
		dmJob.setDel("0");
	}

	public DmJob findOneDmJob(Long id)  throws Exception{
		return dmJobDao.findById(id).get();
	}

	public Page<DmJob> getDmJobList(int page,int pagesize,String name,Long organization) throws Exception{
		if (page <= 0) {
			page = 1;
		}
		if (pagesize <= 0) {
			pagesize = 10;
		}
		// 设置查询条件//
		Map<String, SearchFilter> filters = new HashMap<String, SearchFilter>();
		// 名称模糊查询
		filters.put("del", new SearchFilter("del", Operator.EQ, "1"));
		filters.put("organization", new SearchFilter("organization", Operator.EQ, organization));
		if(StringUtils.isNotBlank(name)){
			filters.put("name", new SearchFilter("name", Operator.LIKE, name));
		}
		// 分页信息
		PageRequest pageRequest = PageRequest.of(page - 1, pagesize);

		Specification<DmJob> spec = DynamicSpecifications.bySearchFilter(filters.values(), DmJob.class);
		return dmJobDao.findAll(spec, pageRequest);
	}

	public List<DmJob> getAllDmJobList() {
		return dmJobDao.getAllDmJobList();
	}
}
