package com.yihu.customer.service.manager.family;

import com.yihu.customer.service.BaseService;
import com.yihu.customer.service.manager.hos.HosDoctorService;
import com.yihu.customer.service.manager.sign.FamilyContractService;
import com.yihu.customer.util.CommonUtil;
import  com.yihu.customer.entity.Doctor;
import  com.yihu.customer.entity.Patient;
import  com.yihu.customer.entity.SignFamily;
import  com.yihu.customer.repository.PatientDao;
import  com.yihu.customer.repository.PatientFamilyMemberDao;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONArray;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 家庭成员服务
 * <p>
 * Created by lyr on 2016/10/17.
 */
@Service
public class FamilyMemberService extends BaseService {
    @Autowired
    PatientFamilyMemberDao memberDao;
    @Autowired
    PatientDao patientDao;

    @Autowired
    JdbcTemplate jdbcTemplate;

    @Autowired
    FamilyContractService contractService;

    @Autowired
    private HosDoctorService hosDoctorService;

    Map<Integer, String> relations = new HashMap<>();



    @PostConstruct
    public void init() {
        relations.put(0, "其他");
        relations.put(1, "父亲");
        relations.put(2, "母亲");
        relations.put(3, "老公");
        relations.put(4, "老婆");
        relations.put(5, "儿子");
        relations.put(6, "女儿");
        relations.put(7, "未知");
    }


    /**
     * 获取所有家庭成员，包括自己
     * @param patient
     * @return
     */
    public List<Map<String,Object>> getAllFamilyMembers(String patient) throws Exception {
        List<Map<String,Object>> list = new ArrayList<>();
        list.add(getPatient(patient));

        String sql = "select * " +
                " from " +
                "    wlyy_patient_family_member t1, " +
                "    wlyy_patient t2 " +
                " where " +
                "    t2.code in (select family_member from wlyy_patient_family_member where patient = ? ) " +
                "    and t1.patient = ? " +
                "    and t1.family_member = t2.code order by t1.family_relation ASC ";

        List<Map<String, Object>> result = jdbcTemplate.queryForList(sql, new Object[]{patient, patient});

        if (result != null && result.size() > 0) {
            JSONArray tem = new JSONArray();//存储其他
            for (Map<String, Object> map : result) {
                Map<String,Object> map1 = new HashMap();
                SignFamily sign = contractService.findSignInfo(map.get("code").toString());
                boolean ssSign = false;
                boolean jtSign = false;
                map1.put("status","-1");
                map1.put("statusName","未签约");
                if(sign!=null){
                    if (sign.getType() == 1 ) {
                        ssSign = true;
                    } else if (sign.getType() == 2 ) {
                        jtSign = true;
                    }
                    map1.put("status",sign.getStatus());;
                    map1.put("statusName","已签约");
                    map1.put("expensesStatusName",sign.getExpensesStatusName());
                    String doctorCode = sign.getDoctor();
                    map1.put("doctor",doctorCode);
                    map1.put("doctorName",sign.getDoctorName());
                    Doctor doctor = hosDoctorService.getDoctorByCode(doctorCode);
                    map1.put("doctorMobile",doctor.getMobile());
                }
                Integer family_relation = Integer.valueOf(map.get("family_relation").toString());
                map1.put("code", map.get("code"));
                map1.put("name", map.get("name"));
                map1.put("sex", map.get("sex"));
                map1.put("isAuthorize", map.get("is_authorize"));//0:未授权,1:已授权
                map1.put("birthday", map.get("birthday"));

                String idCard = StringUtils.isEmpty(String.valueOf(map.get("idcard"))) ? "" : String.valueOf(map.get("idcard"));
                map1.put("idcard", CommonUtil.getIdcardEncode(idCard));
                map1.put("idcardAll", idCard);
                map1.put("photo", map.get("photo"));
                map1.put("mobile", map.get("mobile"));
                map1.put("address", StringUtils.isEmpty(String.valueOf(map.get("address"))) ? "" : map.get("address"));
                map1.put("familyRelation", family_relation);
                map1.put("familyRelationName", relations.get(map.get("family_relation")));
                if(family_relation==0){
                    tem.put(map1);
                }else {
                    list.add(map1);
                }
            }
        }
        return list;
    }

    public Map getPatient(String patient) throws Exception {
        Patient p = patientDao.findByCode(patient);

        //查询签约信息
        Map map = new HashMap<String,Object>();
        SignFamily sign = contractService.findSignInfo(p.getCode());
        boolean ssSign = false;
        boolean jtSign = false;
        map.put("statusName","未签约");
        map.put("status",-1);
        if (sign != null) {
            map.put("statusName","已签约");
            map.put("status",sign.getStatus());
            String doctorCode = sign.getDoctor();
            map.put("doctor",doctorCode);
            map.put("doctorName",sign.getDoctorName());
            Doctor doctor = hosDoctorService.getDoctorByCode(doctorCode);
            map.put("doctorMobile",doctor.getMobile());
            map.put("expensesStatusName",sign.getExpensesStatusName());
        }
        map.put("code", p.getCode());
        map.put("name", p.getName());
        map.put("sex", p.getSex());
        map.put("birthday", p.getBirthday());
        map.put("isAuthorize", 1);
        String idCard = StringUtils.isEmpty(p.getIdcard()) ? "" : p.getIdcard();
        map.put("idcard", CommonUtil.getIdcardEncode(idCard));
        map.put("idcardAll", idCard);
        map.put("ssc", p.getSsc());
        map.put("photo", StringUtils.isEmpty(p.getPhoto()) ? "" : p.getPhoto());
        map.put("mobile", StringUtils.isEmpty(p.getMobile()) ? "" : p.getMobile());
        map.put("address", StringUtils.isEmpty(p.getAddress()) ? "" : p.getAddress());
        map.put("familyRelation", -1);
        map.put("familyRelationName", "自己");
        map.put("isAuthorize", 1);//0:未授权,1:已授权

        return map;
    }
}
