package com.yihu.customer.service.manager.feedback;

import com.yihu.customer.service.BaseService;
import com.yihu.customer.service.SMSService;
import com.yihu.customer.util.DateUtil;
import  com.yihu.customer.entity.Doctor;
import  com.yihu.customer.entity.Message;
import  com.yihu.customer.entity.Patient;
import  com.yihu.customer.entity.feedback.Appeal;
import  com.yihu.customer.repository.DoctorDao;
import  com.yihu.customer.repository.MessageDao;
import  com.yihu.customer.repository.PatientDao;
import  com.yihu.customer.repository.SMSDao;
import  com.yihu.customer.repository.feedback.AppealDao;
import com.yihu.customer.util.HttpManageClientUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springside.modules.persistence.DynamicSpecifications;
import org.springside.modules.persistence.SearchFilter;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by Reece on 2017/5/6.
 */
@Service
@Transactional
public class AppealService extends BaseService {

    private static Logger logger = LoggerFactory.getLogger(AppealService.class);

    @Autowired
    private AppealDao appealDao;
    @Autowired
    private PatientDao patientDao;
    @Autowired
    private DoctorDao doctorDao;
    @Autowired
    private SMSService smsService;
    @Autowired
    private SMSDao smsDao;
    @Value("${wlyyService.api}")
    private String wlyyService;
    @Autowired
    private MessageDao messageDao;
    @Value("${image.imgUrlDomain}")
    private String imgUrlDomain;
    @Value(("${doctorAssistant.api}")+"/wlyygc/doctor/message")
    private String messageApi;

    /**
     * 分页查找问题
     *
     * @param page
     * @param rows
     * @return
     */
    public Page<Appeal> findAppeal(int page, int rows, int type, int identity, int status) {
        if (page <= 0) {
            page = 1;
        }
        if (rows <= 0) {
            rows = 15;
        }
        // 排序
        Sort sort = Sort.by(Sort.Direction.DESC, "id");
        // 分页信息
        PageRequest pageRequest = PageRequest.of(page - 1, rows, sort);
        // 设置查询条件
        Map<String, SearchFilter> filters = new HashMap<>();
        if (type!=-1){
            filters.put("type", new SearchFilter("type", SearchFilter.Operator.EQ, type));
        }
        if (identity!=-1){
            filters.put("identity", new SearchFilter("identity", SearchFilter.Operator.EQ, identity));
        }
        if (status!=-1){
            filters.put("status", new SearchFilter("status", SearchFilter.Operator.EQ, status));
        }
        // 未删除
        filters.put("del", new SearchFilter("del", SearchFilter.Operator.EQ, "1"));
        Specification<Appeal> spec = DynamicSpecifications.bySearchFilter(filters.values(), Appeal.class);
        return appealDao.findAll(spec, pageRequest);
    }



    /**
     * 根据类型、身份、状态查询申诉列表
     *
     * @param type
     * @param identity
     * @param status
     * @param pageNo
     * @param pageSize
     * @return
     */
    public List list(int type, int identity, int status, int pageNo, int pageSize) throws Exception {
        PageRequest request = PageRequest.of(pageNo, pageSize);
        List<Appeal> appealList = appealDao.findAppealList(type, identity, status, request);
        List list = new ArrayList();
        for (Appeal appeal : appealList) {
            Map<String, Object> map = new HashMap();
            long id = appeal.getId();
            int feedbackTypee = appeal.getType();
            String creater = appeal.getCreater();
            int idetity = appeal.getIdentity();
            String phone = appeal.getPhone();
            map.put("id", id);
            map.put("type", feedbackTypee);
            map.put("creater", creater);
            map.put("identity", idetity);
            map.put("phone", phone);
            list.add(map);
        }
        return list;
    }

    /**
     * 根据申诉ID查询申诉详情，并更改状态
     *
     * @param id
     * @return
     */
    public Map getAppealById(long id) throws Exception {
        Map<String, Object> map = new HashMap<>();
        Appeal appeal = appealDao.findFeedbackById(id);
        int identity = appeal.getIdentity();
        int type = appeal.getType();
        String creater = appeal.getCreater();
        String phone = appeal.getPhone();
        String description = appeal.getDescription();
        String images = appeal.getImages();
        String result = appeal.getResult();
        int status = appeal.getStatus();
//        identity 1、患者 2、医生
        String name = null;
        if(StringUtils.isNotBlank(creater)){
            if (identity == 1) {
                name = patientDao.findNameByCode(creater);
            }else{
                Doctor doctor = doctorDao.findByCode(creater);
                name = doctor.getName();
            }
        }
        map.put("type", type);
        map.put("name", name);
        map.put("identity", identity);
        map.put("phone", phone);
        map.put("description", description);
        map.put("name", appeal.getName());
        map.put("createTime", DateUtil.dateToStrLong(appeal.getCreateTime()));
        map.put("idcard", appeal.getIdcard());
        map.put("images", images);
        map.put("result", result);
        map.put("imgUrlDomain", imgUrlDomain);
        if (status == 0){
            appealDao.modifyAppealStatus(id,1);
        }
        return map;
    }

    //根据申诉ID保存处理结果，并更改状态
    public void saveAppealResult(long id, String result) throws Exception {
        appealDao.modifyAppealResult(id,result);
        appealDao.modifyAppealStatus(id,2);
    }

    public void delappeal(String ids) {
        if(ids!=null){
            String[] _ids = ids.split(",");
            for(String id:_ids){
                Long _id = Long.valueOf(id);
                appealDao.delAppeal(_id);
            }
        }
    }

    /***
     * 根据申诉id保存处理结果
     * @param id
     * @param result
     * @throws Exception
     */
    @Transactional
    public void dealAppeal(Long id,String result) throws Exception {
        Appeal appeal = appealDao.findFeedbackById(id);
        appeal.setResult(result);
        appeal.setStatus(2);
        appealDao.save(appeal);

        try {
            //发送处理完成短信
            //您某年某月某日在i健康平台提交的账号申诉已处理，请使用i健康应用（微信公众号）查看处理结果，查看路径：账号登录—忘记密码—账号申诉—申诉记录
            //医生和居民用不同的内容，医生是i健康应用，居民是i健康微信公众号
            StringBuffer msgContent = new StringBuffer("您");
            msgContent.append(DateUtil.getYMD(appeal.getCreateTime()));
            msgContent.append("在i健康平台提交的账号申诉已处理，请使用");
            if (appeal.getIdentity() == 1) {
                msgContent.append("微信公众号");
            } else {
                msgContent.append("i健康应用");
            }
            msgContent.append("查看处理结果，查看路径：账号登录—忘记密码—账号申诉—申诉记录");

            List<NameValuePair> par = new ArrayList<NameValuePair>();
            par.add(new BasicNameValuePair("mobiles", appeal.getPhone()));
            par.add(new BasicNameValuePair("content", msgContent.toString()));
            try {
                logger.info("====================================================================");
                String smsresult = HttpManageClientUtil.post(messageApi+"/sendMessage",par,"UTF-8");
                logger.info("发送短信消息返回结果:"+smsresult);
                logger.info("======================================================================");
            }catch (Exception e){
                e.printStackTrace();
            }

            if (appeal.getIdentity() == 2) {
                String creater = appeal.getCreater();
                SimpleDateFormat formatDate = new SimpleDateFormat("yyyy年MM月dd日 HH:mm");
                Date createTime = appeal.getCreateTime();
                String newDate = formatDate.format(createTime);
                String title = "新增系统消息";
                String content = "您在" + newDate + "提交的账号申诉已处理，您可点此查看处理结果";
                Message message = new Message();
                message.setType(11);
                message.setCode(UUID.randomUUID().toString());
                message.setCzrq(new Date());
                message.setCreateTime(new Date());
                message.setContent(content);
                message.setRead(1);//设置未读
                message.setReceiver(creater);//设置接受医生的code
                message.setSender("system");//设置发送的用户
                message.setTitle(title);
                message.setReadonly(1);//是否只读消息
                message.setRelationCode(appeal.getCode());

                //wlyy给医生发送系统消息
                Doctor doctor = doctorDao.findByCode(creater);
                String doctorOpenID = doctor.getOpenid();
                String url = wlyyService + "/third/template/sendMessage";
                List<NameValuePair> params = new ArrayList<>();
                params.add(new BasicNameValuePair("receiver", creater));
                params.add(new BasicNameValuePair("title", title));
                params.add(new BasicNameValuePair("type", "11"));
                params.add(new BasicNameValuePair("content", content));

                HttpManageClientUtil.post(url, params, "UTF-8");
                messageDao.save(message);
            }else if(appeal.getIdentity() == 1){
                //wlyy给患者发送模板消息
                String creater = appeal.getCreater();

                Patient patient = patientDao.findByCode(creater);
                String patientName = patient.getName();
                String patientOpendID = patient.getOpenid();
                if (StringUtils.isNotEmpty(patientOpendID)) {
                    String url = wlyyService + "/third/template/sendTemplate";
                    List<NameValuePair> params = new ArrayList<>();
                    params.add(new BasicNameValuePair("type", "1"));
                    params.add(new BasicNameValuePair("toUser", patientOpendID));
                    params.add(new BasicNameValuePair("url", null));
                    params.add(new BasicNameValuePair("first", patientName+"您好！您提交的账号申诉已处理"));
                    params.add(new BasicNameValuePair("remark", "感谢你的意见"));
                    SimpleDateFormat formatDate = new SimpleDateFormat("yyyy年MM月dd日 HH:mm");
                    String newDate = formatDate.format(new Date());
                    String keywords = result + "," + newDate;
                    params.add(new BasicNameValuePair("keywords", keywords));

                    HttpManageClientUtil.post(url, params, "UTF-8");
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}