package com.yihu.customer.service.manager.feedback;

import com.yihu.customer.service.BaseService;
import com.yihu.customer.service.manager.message.ManageMessageService;
import com.yihu.customer.util.DateUtil;
import  com.yihu.customer.entity.Doctor;
import  com.yihu.customer.entity.Message;
import  com.yihu.customer.entity.MessageNoticeSetting;
import  com.yihu.customer.entity.Patient;
import  com.yihu.customer.entity.feedback.Feedback;
import  com.yihu.customer.repository.DoctorDao;
import  com.yihu.customer.repository.MessageDao;
import  com.yihu.customer.repository.PatientDao;
import  com.yihu.customer.repository.feedback.FeedbackDao;
import com.yihu.customer.util.HttpManageClientUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springside.modules.persistence.DynamicSpecifications;
import org.springside.modules.persistence.SearchFilter;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by Reece on 2017/5/6.
 */
@Service
@Transactional
public class FeedbackService extends BaseService {
    @Autowired
    private FeedbackDao feedbackDao;
    @Autowired
    private PatientDao patientDao;
    @Autowired
    private DoctorDao doctorDao;
    @Value("${wlyyService.api}")
    private String wlyyService;
    @Value("${image.imgUrlDomain}")
    private String imgUrlDomain;
    @Autowired
    private MessageDao messageDao;
    @Autowired
    private ManageMessageService manageMessageService;

    /**
     * 分页查找问题
     *
     * @param page
     * @param rows
     * @return
     */
    public Page<Feedback> findFeedback(int page, int rows, int type, int identity, int status) {
        if (page <= 0) {
            page = 1;
        }
        if (rows <= 0) {
            rows = 15;
        }
        // 排序
        Sort sort = Sort.by(Sort.Direction.DESC, "id");
        // 分页信息
        PageRequest pageRequest = PageRequest.of(page - 1, rows, sort);
        // 设置查询条件
        Map<String, SearchFilter> filters = new HashMap<>();
        if (type != -1) {
            filters.put("type", new SearchFilter("type", SearchFilter.Operator.EQ, type));
        }
        if (identity != -1) {
            filters.put("identity", new SearchFilter("identity", SearchFilter.Operator.EQ, identity));
        }
        if (status != -1) {
            filters.put("status", new SearchFilter("status", SearchFilter.Operator.EQ, status));
        }
        // 未删除
        filters.put("del", new SearchFilter("del", SearchFilter.Operator.EQ, "1"));
        Specification<Feedback> spec = DynamicSpecifications.bySearchFilter(filters.values(), Feedback.class);
        return feedbackDao.findAll(spec, pageRequest);
    }

    /**
     * 根据类型、身份、状态查询反馈列表
     *
     * @param type
     * @param identity
     * @param status
     * @param pageNo
     * @param pageSize
     * @return
     */
    public List list(int type, int identity, int status, int pageNo, int pageSize) throws Exception {
        PageRequest request = PageRequest.of(pageNo, pageSize);
        List<Feedback> feedbackList = feedbackDao.findFeedbackList(type, identity, status, request);
        List list = new ArrayList();
        for (Feedback feedback : feedbackList) {
            Map<String, Object> map = new HashMap();
            long id = feedback.getId();
            int feedbackTypee = feedback.getType();
            String creater = feedback.getCreater();
            int idetity = feedback.getIdentity();
            String contact = feedback.getContact();
            String name = null;
            if (identity == 1) {
                name = patientDao.findNameByCode(creater);
            }
            Doctor doctor = doctorDao.findByCode(creater);
            name = doctor.getName();
            map.put("id", id);
            map.put("name", name);
            map.put("type", feedbackTypee);
            map.put("creater", creater);
            map.put("identity", idetity);
            map.put("contact", contact);
            list.add(map);
        }
        return list;
    }

    /**
     * 根据反馈ID查询反馈详情，并更改状态
     *
     * @param id
     * @return
     */
    public Map getFeedbackById(long id) throws Exception {
        Map<String, Object> map = new HashMap<>();
        Feedback feedback = feedbackDao.findFeedbackById(id);
        int identity = feedback.getIdentity();
        int type = feedback.getType();
        String creater = feedback.getCreater();
        String contact = feedback.getContact();
        String description = feedback.getDescription();
        String images = feedback.getImages();
        int status = feedback.getStatus();
//        identity 1、患者 2、医生
        String name = null;
        if (StringUtils.isNotBlank(creater)) {
            if (identity == 1) {
                name = patientDao.findNameByCode(creater);
            } else {
                Doctor doctor = doctorDao.findByCode(creater);
                name = doctor.getName();
            }
        }
        map.put("type", type);
        map.put("name", name);
        map.put("identity", identity);
        map.put("contact", contact);
        map.put("phone", feedback.getPhone());
        map.put("idcard", feedback.getIdcard());
        map.put("result", feedback.getResult());
        map.put("createTime", DateUtil.dateToStrLong(feedback.getCreateTime()));
        map.put("description", description);
        map.put("images", images);
        map.put("imgUrlDomain", imgUrlDomain);
        if (status == 0) {
            feedbackDao.modifyFeedbackStatus(id, 1);
        }
        return map;
    }


    public void delfeedback(String ids) {
        if (ids != null) {
            String[] _ids = ids.split(",");
            for (String id : _ids) {
                Long _id = Long.valueOf(id);
                feedbackDao.delFeedback(_id);
            }
        }
    }

    /***
     * 根据反馈id保存处理结果
     * @param id
     * @param result
     * @throws Exception
     */
    public Feedback dealFeedback(Long id, String result) throws Exception {
        Feedback feedback = feedbackDao.findFeedbackById(id);
        feedback.setResult(result);
        feedback.setStatus(2);
        feedbackDao.save(feedback);
        return feedback;
    }

    public void sendMsg(Feedback feedback){
        String result = feedback.getResult();
        int identity = feedback.getIdentity();
        int type = feedback.getType();
        String typeName = null;
        String creater = feedback.getCreater();
        switch (type) {
            case 1:
                typeName = "优化建议";
                break;
            case 2:
                typeName = "提交bug";
                break;
            case 3:
                typeName = "新功能建议";
                break;
            case 4:
                typeName = "其他";
                break;
        }
        try{
            if (identity == 1) {
                //wlyy给患者发送模板消息
                SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

                Patient patient = patientDao.findByCode(creater);
                String patientName = patient.getName();
                String patientOpendID = patient.getOpenid();
                if (StringUtils.isNotEmpty(patientOpendID)) {
                    String url = wlyyService + "/third/template/sendTemplate";
                    List<NameValuePair> params = new ArrayList<>();
                    params.add(new BasicNameValuePair("type", "2"));
                    params.add(new BasicNameValuePair("toUser", patientOpendID));
                    params.add(new BasicNameValuePair("url", null));
                    params.add(new BasicNameValuePair("first", "您好！您提交的意见反馈已处理"));
                    params.add(new BasicNameValuePair("remark", "感谢你的意见"));
                    SimpleDateFormat formatDate = new SimpleDateFormat("yyyy年MM月dd日 HH:mm");
                    String newDate = formatDate.format(new Date());
                    String keywords = newDate + "," + result;
                    params.add(new BasicNameValuePair("keywords", keywords));

                    HttpManageClientUtil.post(url, params, "UTF-8");
                }
            } else if (identity == 2) {
                SimpleDateFormat formatDate = new SimpleDateFormat("yyyy年MM月dd日 HH:mm");
                Date createTime = feedback.getCreateTime();
                String newDate = formatDate.format(createTime);
                String title = "新增系统消息";
                String content = "您在" + newDate + "提交的意见反馈已处理，您可点此查看处理结果";
                Message message = new Message();
                message.setType(10);
                message.setCode(UUID.randomUUID().toString());
                message.setCzrq(new Date());
                message.setCreateTime(new Date());
                message.setContent(content);
                message.setRead(1);//设置未读
                message.setReceiver(creater);//设置接受医生的code
                message.setSender("system");//设置发送的用户
                message.setTitle(title);
                message.setReadonly(1);//是否只读消息
                message.setRelationCode(feedback.getCode());

                if(manageMessageService.getMessageNoticeSettingByMessageType(creater,"1", MessageNoticeSetting.MessageTypeEnum.systemSwitch.getValue())){
                    //wlyy给医生发送系统消息
                    Doctor doctor = doctorDao.findByCode(creater);
                    String doctorOpenID = doctor.getOpenid();
                    String url = wlyyService + "/third/template/sendMessage";
                    List<NameValuePair> params = new ArrayList<>();
                    params.add(new BasicNameValuePair("receiver", creater));
                    params.add(new BasicNameValuePair("title", title));
                    params.add(new BasicNameValuePair("type", "10"));
                    params.add(new BasicNameValuePair("content",content));

                    HttpManageClientUtil.post(url, params, "UTF-8");
                }
                messageDao.save(message);
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }
    
    /**
     * 保存意见反馈
     * @param description
     * @param type
     * @param patientcode
     */
    public void saveFeedback(String description, int type, String patientcode) {
        Patient patient = patientDao.findByCode(patientcode);
        Feedback feedback = new Feedback();
        feedback.setCode(getCode());
        feedback.setCreater(patientcode);
        feedback.setIdentity(1);
        feedback.setIdcard(patient.getIdcard());
        feedback.setDescription(description);
        feedback.setType(type);
        feedback.setContact(patient.getMobile());
        feedback.setPhone(patient.getMobile());
        feedback.setStatus(0);
        feedback.setDel(1);
        feedback.setCreateTime(new Date());
        feedback.setCreaterName(patient.getName());
        feedbackDao.save(feedback);
    }
}
