package com.yihu.customer.service.manager.healthbank;

import com.alibaba.fastjson.JSONObject;
import  com.yihu.customer.repository.CityDao;
import  com.yihu.customer.repository.HospitalDao;
import  com.yihu.customer.repository.ProvinceDao;
import com.yihu.customer.util.HttpManageClientUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * Created by humingfen on 2018/6/19.
 * wlyy_health_bank_activity数据库
 */
@Service
public class ActivityService {
    @Value("${healthbank.url}")
    private String baseUrl;
    @Autowired
    private HttpManageClientUtil httpManageClientUtil;
    @Autowired
    private HospitalDao hospitalDao;
    @Autowired
    private ProvinceDao provinceDao;
    @Autowired
    private CityDao cityDao;

    /**
     * 查看活动列表
     * @param activity
     * @param page
     * @param pageSize
     * @return
     */
    public JSONObject searchList(String activity, Integer page, Integer pageSize) {
        String url = baseUrl + "findActivity";
        String response = "";
        Map<String,String> params = new HashMap<>();
        params.put("activity",activity);
        params.put("page",page.toString());
        params.put("size",pageSize.toString());
        try {
            response = httpManageClientUtil.httpPost(url,params);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return JSONObject.parseObject(response);
    }

    public JSONObject update(String jsonData,String value1,String value2,String value3,String registrationNumber) {
        String url = baseUrl + "updateActivity";
        String response = "";
        Map<String,String> params = new HashMap<>();
        params.put("activity",jsonData);
        params.put("value1",value1);
        params.put("value2",value2);
        params.put("value3",value3);
        params.put("registrationNumber",registrationNumber);
        try {
            response = httpManageClientUtil.httpPost(url,params);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return JSONObject.parseObject(response);
    }

    public JSONObject create(String jsonData,String value1,String value2,String value3,String registrationNumber)  {
        String url = baseUrl + "createActivity";
        String response = "";
        Map<String,String> params = new HashMap<>();
        params.put("activity",jsonData);
        params.put("value1",value1);
        params.put("value2",value2);
        params.put("value3",value3);
        params.put("registrationNumber",registrationNumber);
        try {
            response = httpManageClientUtil.httpPost(url,params);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return JSONObject.parseObject(response);
    }

    public JSONObject delete(String ids) {
        String url = baseUrl + "batchActivity";
        String response = "";
        Map<String,String> params = new HashMap<>();
        params.put("ids",ids);
        try {
            response = httpManageClientUtil.httpPost(url,params);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return JSONObject.parseObject(response);
    }

    public JSONObject deleteById(String id) {
        String url = baseUrl + "daleteActivity";
        String response = "";
        Map<String,String> params = new HashMap<>();
        params.put("id",id);
        try {
            response = httpManageClientUtil.httpPost(url,params);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return JSONObject.parseObject(response);
    }

    public JSONObject editActivity(String id,String type) {
        String url = baseUrl + "editActivity";
        String response = "";
        Map<String,String> params = new HashMap<>();
        params.put("id",id);
        params.put("type",type);
        try {
            response = httpManageClientUtil.httpPost(url,params);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return JSONObject.parseObject(response);
    }

    public JSONObject findActivityById(String id) {
        String url = baseUrl + "findActivityById";
        String response = "";
        Map<String,String> params = new HashMap<>();
        params.put("id",id);
        try {
            response = httpManageClientUtil.httpPost(url,params);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return JSONObject.parseObject(response);
    }

    /**
     * 分页获取活动
     * @param status
     * @param crowdType
     * @param releaseTime
     * @param activityOfflineTime
     * @param filter
     * @param page
     * @param size
     * @return
     */
    public JSONObject pageActivity(Integer status,Integer crowdType,String releaseTime,String activityOfflineTime,String filter,Integer page,Integer size) {
        String url = baseUrl + "pageActivity";
        String response = "";
        Map<String,String> params = new HashMap<>();
        params.put("status",null==status?"":status.toString());
        params.put("crowdType",null==crowdType?"":crowdType.toString());
        params.put("releaseTime",releaseTime);
        params.put("activityOfflineTime",activityOfflineTime);
        params.put("filter",filter);
        params.put("page",null==page?"1":page.toString());
        params.put("size",null==size?"10":size.toString());
        try {
            response = httpManageClientUtil.httpPost(url,params);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return JSONObject.parseObject(response);
    }


}
