package com.yihu.customer.service.manager.healthchannel;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.customer.util.Constant;
import com.yihu.customer.util.DateUtil;
import com.yihu.customer.util.query.BaseJpaService;
import  com.yihu.customer.entity.Doctor;
import  com.yihu.customer.entity.healthchannel.HealthChannelRecommendArticleDo;
import  com.yihu.customer.repository.healthchannel.ChannelRecommendArticleDao;
import com.yihu.customer.util.HttpManageClientUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by zdm on 2019/2/15.
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class ChannelRecommendArticleService extends BaseJpaService<HealthChannelRecommendArticleDo,ChannelRecommendArticleDao> {
    @Value("${jkEdu.web.articleBaseUrl}")
    private String articleBaseUrl;

    @Autowired
    private HttpManageClientUtil httpManageClientUtil;
    @Autowired
    private ChannelRecommendArticleDao channelRecommendArticleDao;
    @Autowired
   private JdbcTemplate jdbcTemplate;

    /**
     * 新增健康频道
     * @param jsonData 推荐/置顶文章
     * @param operationType 操作类型 1推荐、2置顶
     * @return
     * @throws Exception
     */
    public List<HealthChannelRecommendArticleDo> create(Integer operationType,String jsonData,Doctor d)throws Exception{
        ObjectMapper objectMapper = new ObjectMapper();
        List<HealthChannelRecommendArticleDo> healthChannelRecommendArticleDos = objectMapper.readValue(jsonData, new TypeReference<List<HealthChannelRecommendArticleDo>>(){});
        List<HealthChannelRecommendArticleDo> healthChannelRecommendArticleDoBack=new ArrayList<>();
        for(HealthChannelRecommendArticleDo harticleDo:healthChannelRecommendArticleDos){
            if(1==operationType) {
                //判断该频道是否已存在该篇文章：条件：频道+操作条件+位置
                // 推荐10条
                if (StringUtils.isBlank(harticleDo.getArticleType())) {
                    throw new Exception("文章类型不能为空！");
                }
                //新增推荐/置顶文章
                harticleDo.setAuthor(null != d ? d.getCode() : "");
                harticleDo.setAuthorName(null != d ? d.getName() : "");
                HealthChannelRecommendArticleDo harticleDo1 = createHealthChannelRecommendArticle(harticleDo);
                healthChannelRecommendArticleDoBack.add(harticleDo1);
            }else if (2 == operationType) {
                //置顶 同一时间只允许置顶两条
                    //新增推荐/置顶文章
                    harticleDo.setAuthor(null!=d?d.getCode():"");
                    harticleDo.setAuthorName(null!=d?d.getName():"");
                    HealthChannelRecommendArticleDo  healthChaDo1 = stick(harticleDo);
                    healthChannelRecommendArticleDoBack.add(healthChaDo1);
            }
        }
        return healthChannelRecommendArticleDoBack;
    }

    /**
     * 置顶文章
     * @param healthChannelRecommendArticleDo
     * @return
     * @throws Exception
     */
    public HealthChannelRecommendArticleDo stick(HealthChannelRecommendArticleDo healthChannelRecommendArticleDo)throws Exception{
        if(null==healthChannelRecommendArticleDo.getStartTime()||null==healthChannelRecommendArticleDo.getEndTime()){
            throw new Exception("置顶时间不能为空！");
        }else {
            String dateStart=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(healthChannelRecommendArticleDo.getStartTime());
            String dateEnd=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(healthChannelRecommendArticleDo.getStartTime());
            //获取起始时间s2  判断是否存在  s1(库中存在起始时间)<s2<e1(库中存在结束时间) 或者 结束时间e2  判断是否存在  s1(库中存在起始时间)<e2<e1(库中存在结束时间)
            StringBuilder stub=new StringBuilder();
            //s1<e2<e1
            stub.append(" (start_time<'"+dateEnd+"' AND end_time>='"+dateEnd+"')");
            //s1<s2<e1
            stub.append(" OR (start_time<='"+dateStart+"' AND end_time>'"+dateStart+"') ");
            //s1<=s2 e2<=e1
//            stub.append(" OR  (start_time<='"+dateStart+"' AND end_time>='"+dateEnd+"')");
            //s2<=s1 e1<=e2
            stub.append(" OR  (start_time>='"+dateStart+"' AND end_time<='"+dateEnd+"')");

            String countStr="SELECT COUNT(1),article_id FROM wlyy_channel_recommend_article WHERE channel_id='"+healthChannelRecommendArticleDo.getChannelId()+"' and type=2 AND del_flag>0 AND ("+stub.toString()+")  GROUP BY article_id";
            List<Map<String,Object>> mapList=jdbcTemplate.queryForList(countStr);
            if (null != mapList && mapList.size() > 1) {
                throw new Exception("同一时间段内，只允许置顶两条！");
            }else {
                String dateString = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date());
                Date now = DateUtil.strToDate(dateString,"yyyy-MM-dd HH:mm:ss");
                Date dateSt=DateUtil.strToDate(dateStart,"yyyy-MM-dd HH:mm:ss");
                int startEqual = now.compareTo(dateSt);
                //当前时间小于开始时间为 未生效，否则为生效
                if(startEqual<0){
                    healthChannelRecommendArticleDo.setDelFlag(2);
                }else {
                    healthChannelRecommendArticleDo.setDelFlag(1);
                }
                healthChannelRecommendArticleDo=channelRecommendArticleDao.save(healthChannelRecommendArticleDo);
            }
        }
        return healthChannelRecommendArticleDo;
    }

    /**
     *
     * @param harticleDo 待保存的置顶/推荐文章
     * @return
     */
    public HealthChannelRecommendArticleDo createHealthChannelRecommendArticle(HealthChannelRecommendArticleDo harticleDo) throws Exception{
        //判断1库里是否已存在10条推荐文章，若是判断2是否传sort，若sort>10 则提示只允许推荐10个；若sort<10则直接替换原有sort上的推荐文章。判断2为否,提示只允许推荐10个,判断1为否
        if(null==harticleDo.getSort()){
                harticleDo.setSort(getMaxSort(harticleDo.getChannelId(),harticleDo.getArticleType()));
        }else{
            //根据频道、文章类型、文章排序查找文章
           updateByChannelIdAndArticleTypeAndSort(harticleDo.getChannelId(),harticleDo.getArticleType(),harticleDo.getSort());
            harticleDo.setDelFlag(1);
        }
        harticleDo=channelRecommendArticleDao.save(harticleDo);
        return  harticleDo;
    }

    /**
     * 变更频道+文章类型+排序
     */
    public void updateByChannelIdAndArticleTypeAndSort(Integer channelId, String articleType, Integer sort) throws Exception {
        String countSql = "";
        if (Constant.health_column.equals(articleType) || Constant.selected_questions_and_answers.equals(articleType)) {
            //健康专栏/精选问答
            countSql = "update wlyy_channel_recommend_article set del_flag=-1 where channel_id='" + channelId + "' and article_type='" + articleType + "' and sort='" + sort + "'";
        } else {
            String filter = " and (article_type='" + Constant.health_article + "' or article_type='" + Constant.platform_information + "')";
            //平台资讯+健康文章
            countSql = "update wlyy_channel_recommend_article set del_flag=-1 where channel_id='" + channelId + "' and sort='" + sort + "'" + filter;
        }
        jdbcTemplate.execute(countSql);
    }

    /**
     * 变更频道+文章类型+排序
     */
    public void updateByChannelIdAndArticleTypeAndSort(Long id,Integer channelId, String articleType, Integer sort) throws Exception {
        String countSql = "";
        if (Constant.health_column.equals(articleType) || Constant.selected_questions_and_answers.equals(articleType)) {
            //健康专栏/精选问答
            countSql = "update wlyy_channel_recommend_article set del_flag=-1 where channel_id='" + channelId + "' and article_type='" + articleType + "' and sort='" + sort + "' and id!="+id;
        } else {
            String filter = " and (article_type='" + Constant.health_article + "' or article_type='" + Constant.platform_information + "')";
            //平台资讯+健康文章
            countSql = "update wlyy_channel_recommend_article set del_flag=-1 where channel_id='" + channelId + "' and sort='" + sort + "'" + filter+"  and id!="+id;
        }
        jdbcTemplate.execute(countSql);
    }

    /**
     * 修改置顶时间/推荐位置
     * @param operationType 操作类型:1推荐、2置顶
     * @param healthChannelRecommendArticleId 推荐/置顶记录id
     * @param startTime 置顶开始时间
     * @param endTime 置顶结束时间
     * @param sort 推荐排序位置
     * @return
     */
    public HealthChannelRecommendArticleDo update(Integer operationType,Long healthChannelRecommendArticleId,String startTime,String endTime,Integer sort)throws Exception{
        HealthChannelRecommendArticleDo healthChannelRecommendArticleDo=channelRecommendArticleDao.findById(healthChannelRecommendArticleId).get();
        if(null==healthChannelRecommendArticleDo){
            throw new Exception("该推荐/置顶记录不存在！");
        }
        if(StringUtils.isBlank(healthChannelRecommendArticleDo.getArticleType())){
            throw new Exception("文章类型不能为空！");
        }
        if(1==operationType){
            //同一位置的需要删除
            updateByChannelIdAndArticleTypeAndSort(healthChannelRecommendArticleDo.getId(),healthChannelRecommendArticleDo.getChannelId(),healthChannelRecommendArticleDo.getArticleType(),sort);
            healthChannelRecommendArticleDo.setSort(sort);
            healthChannelRecommendArticleDo.setDelFlag(1);
            healthChannelRecommendArticleDo=channelRecommendArticleDao.save(healthChannelRecommendArticleDo);
        }else {
            //置顶
            healthChannelRecommendArticleDo.setStartTime(DateUtil.strToDate(startTime,DateUtil.YYYY_MM_DD_HH_MM_SS));
            healthChannelRecommendArticleDo.setEndTime(DateUtil.strToDate(endTime,DateUtil.YYYY_MM_DD_HH_MM_SS));
            healthChannelRecommendArticleDo = stick(healthChannelRecommendArticleDo);
        }
        return healthChannelRecommendArticleDo;
    }

    /**
     * 获取当前推荐文章的最大排序号+1
     *
     */
    public Integer getMaxSort(Integer channelId,String articlType) throws Exception{
        String countSql ="";
        if(StringUtils.isBlank(articlType)){
            throw new Exception("文章类型不能为空！");
        }
        if (Constant.health_column.equals(articlType)||Constant.selected_questions_and_answers.equals(articlType)) {
            //健康专栏/精选问答
            countSql = "SELECT MAX(wp.sort) FROM wlyy.wlyy_channel_recommend_article wp WHERE wp.del_flag>0  and type=1  and channel_id='"+channelId+"' and article_type='"+articlType+"'";
        } else {
            //平台资讯+健康文章
            countSql = "SELECT MAX(wp.sort) FROM wlyy.wlyy_channel_recommend_article wp WHERE wp.del_flag>0  and type=1  and channel_id='"+channelId+"' and article_type!='"+Constant.health_column+"' and article_type!='"+Constant.selected_questions_and_answers+"'";
        }
        Integer count = jdbcTemplate.queryForObject(countSql, Integer.class);
        return null==count?1:count + 1;
    }

    /**
     * 根据推荐的文章类型获取推荐数量
     *
     */
    public Map<String,Integer> getCountByArtileType(Integer channelId) throws Exception{
        Map<String,Integer> map=new HashMap<>();
        String countSql = "SELECT article_type,COUNT(1) as co FROM wlyy.wlyy_channel_recommend_article wp WHERE wp.del_flag>0  and type=1 and channel_id='"+channelId+"' GROUP BY article_type ";
        List<Map<String,Object>> count = jdbcTemplate.queryForList(countSql);
        map.put(Constant.health_article,0);
        for (Map<String, Object> m : count) {
            if (null != m.get("article_type") && Constant.health_column.equals(m.get("article_type").toString())) {
                //健康专栏
                map.put(Constant.health_column, null == m.get("co") ? 0 : Integer.valueOf(m.get("co").toString()));
            } else if (null != m.get("article_type") && Constant.selected_questions_and_answers.equals(m.get("article_type").toString())) {
                //精选问答
                map.put(Constant.selected_questions_and_answers, null == m.get("co") ? 0 : Integer.valueOf(m.get("co").toString()));
            } else {
                //平台资讯+健康文章
                Integer hac = Integer.valueOf(map.get(Constant.health_article).toString());
                map.put(Constant.health_article, null == m.get("co") ? Integer.valueOf(map.get(Constant.health_article).toString()) : hac + Integer.valueOf(m.get("co").toString()));
            }
        }
        return map;
    }

    /**
     * 获取置顶列表
     * @param channelId 频道id
     * @param title 标题
     * @param articleAuthor 作者
     * @param categoryId 内容类型
     * @param classification 分类
     * @param startTime 置顶开始时间
     * @param endTime 置顶结束时间
     * @param delFlag 状态
     * @param page 当前页
     * @param pageSize 分页大小
     * @return
     */
    public Page<HealthChannelRecommendArticleDo> searchStickList(Integer channelId, String title, String articleAuthor, String categoryId, String classification, String startTime, String endTime, String delFlag, Integer page, Integer pageSize) {
        List<HealthChannelRecommendArticleDo> healthChannelRecommendArticleDos = new ArrayList<>();
        if (page == null) {
            page = 1;
        }
        if (pageSize == null) {
            pageSize = 15;
        }
        PageRequest pageRequest = PageRequest.of(page - 1, pageSize);
        StringBuilder filters = new StringBuilder();
        if(null!=channelId){
            filters.append(" and  p.channel_id =" +channelId);
        }
        if(StringUtils.isNotBlank(title)){
            filters.append(" and p.article_title like '%" +title+"%'");
        }
        if(StringUtils.isNotBlank(articleAuthor)){
            filters.append(" and p.article_author like '%" +articleAuthor+"%'");
        }
        if(StringUtils.isNotBlank(categoryId)){
            filters.append(" and p.category_id ='" +categoryId+"'");
        }
        if(StringUtils.isNotBlank(classification)){
            filters.append(" and p.classification ='" +classification+"'");
        }
        if(StringUtils.isNotBlank(startTime)){
            filters.append(" and  p.start_time >='" +startTime+"'");
        }
        if(StringUtils.isNotBlank(endTime)){
            filters.append(" and  p.end_time <='" +endTime+"'");
        }
        if(StringUtils.isNotBlank(delFlag)){
            filters.append(" and  p.del_flag =" +delFlag);
        }else {
            filters.append(" and  p.del_flag >0");
        }
        String sql = "SELECT p.* FROM wlyy_channel_recommend_article p WHERE 1=1 and p.type=2 ";
        String countSql ="SELECT count(1) FROM wlyy_channel_recommend_article p WHERE 1=1  and p.type=2 ";
        healthChannelRecommendArticleDos = jdbcTemplate.query(sql+filters.toString() + " order by p.create_time desc" + " limit " + (page - 1) * pageSize + "," + pageSize, new BeanPropertyRowMapper<>(HealthChannelRecommendArticleDo.class));
        if (healthChannelRecommendArticleDos.size() <= 0) {
            return new PageImpl(new ArrayList<>(), pageRequest, 0);
        }
        Long count = jdbcTemplate.queryForObject(countSql + filters.toString(), Long.class);
        return new PageImpl(healthChannelRecommendArticleDos, pageRequest, count);
    }

    /**
     * 根据记录id删除数据
     * @param healthChannelRecommendArticleId
     * @return
     */
    public void deleteById(String healthChannelRecommendArticleId)throws Exception{
        String[]  stringList=healthChannelRecommendArticleId.split(",");
        for (String str:stringList){
            channelRecommendArticleDao.updateById(Long.parseLong(str));
        }
    }

    public org.json.JSONObject queryArticleList(String title, String operatorName, String firstLevelCategoryId, String secondLevelCategoryId, Integer scopeOfApplication, Integer page, Integer pageSize) throws Exception {
        String response = "";
        org.json.JSONObject json = null;
        try {
            com.alibaba.fastjson.JSONObject param = new com.alibaba.fastjson.JSONObject();
            param.put("firstLevelCategoryId", firstLevelCategoryId);
            param.put("secondLevelCategoryId", secondLevelCategoryId);
            param.put("articleTitle", title);
            param.put("operatorName", operatorName);
            param.put("userScope", scopeOfApplication);
            param.put("isAuthentication", "1");
            param.put("pageIndex", (page - 1) + "");
            param.put("pageSize", pageSize + "");
            response = httpManageClientUtil.httpPost(articleBaseUrl + "/WsPlatform/rest", getParamsMap("JkEdu.Article.getArticalList", param.toString(), "1"));
            json = new org.json.JSONObject(response);
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (json.getInt("Code") == 10000) {
            Integer total = json.getInt("Count");
            json.put("totalPage", total % pageSize > 0 ? (total / pageSize) + 1 : (total / pageSize));
            return json;
        } else {
            throw new Exception(json.getString("Message"));
        }
    }

    /**
     * 获取文章类别
     *
     * @param categoryLevel 是    获取类别：1、一级类别 2、二级类别
     * @return
     */
    public JSONArray getCategoryList(Integer categoryLevel, String firstlevelId, String secondLevelId) throws Exception {
        JSONArray re = new JSONArray();
        JSONObject json = null;
        try {
            JSONObject param = new JSONObject();
            param.put("categoryLevel", categoryLevel);
            param.put("firstlevelId", firstlevelId);
            param.put("secondLevelId", secondLevelId);
//            param.put("userId", userId);
            String response = httpManageClientUtil.httpPost(articleBaseUrl+"/WsPlatform/rest", getParamsMap("JkEdu.Category.getCategoryList", param.toString(), "1"));
            json = JSON.parseObject(response);
        } catch (Exception e) {
            e.printStackTrace();
            return re;
        }

        if ("10000".equals(json.getString("Code"))) {
            return json.getJSONObject("Result").getJSONArray("result");
        } else {
            throw new Exception(json.getString("Message"));
        }
    }

    /**
     * 封装入参
     *
     * @param api
     * @param param
     * @param version
     * @return
     */
    public Map<String, String> getParamsMap(String api, String param, String version) {
        Map<String, String> params = new HashMap<>();
        com.alibaba.fastjson.JSONObject authInfo = new com.alibaba.fastjson.JSONObject();
        authInfo.put("clientId", "9000276");//填渠道ID（必填，固定为9000276）
        authInfo.put("clientVersion", "1.0");//接入方系统版本号（非必须，默认 1.0）
        authInfo.put("sign", "");//（保留，非必填）
        authInfo.put("sessionKey", "");//（保留，非必填）
        params.put("AuthInfo", authInfo.toString());
        params.put("SequenceNoString", DateUtil.getUserDate("yyyyMMddHHmmssSSS"));//请求序列号，yyyyMMddHHmmssSSS 格式的时间戳
        params.put("Api", api);//Api 名称    需要调用的 API 名称
        params.put("Param", param);//本次请求的方法参数   Json结构的字符串
        params.put("ParamType", "0");//指定 param 参数的类型，固定传0
        params.put("OutType", "0");//指定返回 JSON/Xml 结构的字符串，固定传0
        params.put("V", version);//API 版本号。版本号为整型，从数字 1 开始递增.

        return params;
    }

    /**
     * 获取健康文章一二级分类
     * @return
     * @throws Exception
     */
    public JSONArray getJkeduCategoryList() throws Exception{
        JSONArray resArray = new JSONArray();
        JSONArray firstlevelArray = getCategoryList(1, null, null);
        for(Object o : firstlevelArray) {
            JSONObject jsonObject = new JSONObject();
            JSONObject object = (JSONObject)o;
            JSONArray secondLevel = getCategoryList(2, object.getString("categoryid"), null);
            jsonObject.put("FirstLevelId", object.getString("categoryid"));
            jsonObject.put("FirstLevelName", object.getString("categoryname"));
            jsonObject.put("SecondLevel", secondLevel);
            resArray.add(jsonObject);
        }
        return resArray;
    }
}
