package com.yihu.customer.service.manager.healthchannel;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.customer.util.DateUtil;
import com.yihu.customer.util.query.BaseJpaService;
import  com.yihu.customer.entity.healthchannel.HealthChannelDo;
import  com.yihu.customer.entity.healthchannel.PatientNewChannelDO;
import  com.yihu.customer.repository.UserDao;
import  com.yihu.customer.repository.healthchannel.ChannelDao;
import  com.yihu.customer.repository.healthchannel.PatientNewChannelDao;
import  com.yihu.customer.repository.healthchannel.PatientSubscribeJkeduDao;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.util.*;

/**
 * Created by zdm on 2019/2/15.
 */
@Service
@Transactional
public class ChannelService  extends BaseJpaService<HealthChannelDo,ChannelDao> {
    @Autowired
    private ChannelDao channelDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private UserDao userDao;
    @Autowired
    private PatientSubscribeJkeduDao patientSubscribeJkeduDao;
    @Autowired
    private PatientNewChannelDao patientNewChannelDao;

    /**
     * 新增健康频道
     * @param jsonData
     * @param opeator
     * @return
     * @throws Exception
     */
    public HealthChannelDo create(String jsonData,String opeator)throws Exception{
        ObjectMapper objectMapper = new ObjectMapper();
        HealthChannelDo healthChannelDo=objectMapper.readValue(jsonData,HealthChannelDo.class);
        healthChannelDo.setAuthor(opeator);
        healthChannelDo.setDelFlag(1);
        healthChannelDo= channelDao.save(healthChannelDo);
        return healthChannelDo;
    }

    /**
     * 频道列表查询
     * @return
     */
    public List<Map<String, Object>> searchList(String level,String classification) {
        StringBuilder filters = new StringBuilder();
        if(StringUtils.isNotBlank(level)){
            filters.append(" and  level ='" +level+"'");
        }
        if(StringUtils.isNotBlank(classification)){
            filters.append(" and  classification ='" +classification+"'");
        }
        String sql = "select id,code,name,level,classification,classification_name from wlyy_channel where del_flag=1 "+filters;
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql + " group by code");
        return list;
    }

    /**
     * 新增频道配置
     * @param jsonData
     * @param userCode
     * @return
     */
    public List<HealthChannelDo> createChannel(String jsonData, String userCode) throws IOException {
        List<HealthChannelDo> result = new ArrayList<>();
        String userName = userDao.findByCode(userCode).getName();
        JSONObject jsonObject = JSONObject.parseObject(jsonData);
        //频道名称
        String channelName = jsonObject.getString("name");
        //推荐推荐栏目信息
        JSONArray recommendArray = jsonObject.getJSONArray("recommend");
        //健康文章置顶内容
        JSONArray listArray = jsonObject.getJSONArray("list");
        Date date = new Date();
        //判断是新增还是编辑，如果是编辑只更新频道名称和更新信息，并删除栏目信息；
        // 若是新增添加居民需重新打开订阅频道标识
        HealthChannelDo healthChannelDo = null;
        if(jsonObject.containsKey("id")) {
            Long channelId = jsonObject.getLong("id");
            healthChannelDo = channelDao.findById(channelId).get();
            healthChannelDo.setName(channelName);
            healthChannelDo.setLastUpdateTime(date);
            healthChannelDo.setUpdater(userCode);
            healthChannelDo.setUpdaterName(userName);
            healthChannelDo = channelDao.save(healthChannelDo);
            channelDao.deleteByClassification(channelId + "");
        }else {
            healthChannelDo = new HealthChannelDo();
            healthChannelDo.setCode(getCode());
            healthChannelDo.setLevel("0");
            healthChannelDo.setDefaultFlag(0);
            healthChannelDo.setCreateTime(date);
            healthChannelDo.setAuthor(userCode);
            healthChannelDo.setAuthorName(userName);
            healthChannelDo.setDelFlag(1);
            healthChannelDo.setName(channelName);
            healthChannelDo.setLastUpdateTime(date);
            healthChannelDo.setUpdater(userCode);
            healthChannelDo.setUpdaterName(userName);
            healthChannelDo = channelDao.save(healthChannelDo);

            //获取订阅频道居民
            List<String> patientList = patientSubscribeJkeduDao.getPatient();
            List<PatientNewChannelDO> patientNewChannelDOList = new ArrayList<>();
            for(String patient : patientList){
                PatientNewChannelDO patientNewChannelDO = new PatientNewChannelDO();
                patientNewChannelDO.setCode(getCode());
                patientNewChannelDO.setChannelId(healthChannelDo.getId());
                patientNewChannelDO.setChannelName(healthChannelDo.getName());
                patientNewChannelDO.setPatient(patient);
                patientNewChannelDO.setCreateTime(date);
                patientNewChannelDOList.add(patientNewChannelDO);
            }
            patientNewChannelDao.saveAll(patientNewChannelDOList);
        }

        //频道id
        String channelId = healthChannelDo.getId().toString();

        //创建频道推荐栏目信息
        for (Object object : recommendArray) {
            JSONObject json = (JSONObject) object;
            //获取多个分类
            JSONArray categoryList = json.getJSONArray("categoryList");
            //如果同一频道有多个内容分类，就存多条数据
            for (Object o : categoryList) {
                JSONObject category = (JSONObject) o;
                HealthChannelDo healthChannel = objectMapper.readValue(object.toString(), HealthChannelDo.class);
                healthChannel.setFirstLevelCategoryId(category.getString("firstLevelCategoryId"));
                healthChannel.setFirstLevelCategoryName(category.getString("firstLevelCategoryName"));
                healthChannel.setCategoryId(category.getString("categoryId"));
                healthChannel.setCategoryName(category.getString("categoryName"));
                healthChannel.setLevel("1");
                healthChannel.setClassification(channelId);
                healthChannel.setClassificationName(channelName);
                healthChannel.setCreateTime(date);
                healthChannel.setLastUpdateTime(date);
                healthChannel.setAuthor(userCode);
                healthChannel.setAuthorName(userName);
                healthChannel.setUpdater(userCode);
                healthChannel.setUpdaterName(userName);
                healthChannel.setDelFlag(1);
                healthChannel.setDefaultFlag(0);
                result.add(healthChannel);
            }
        }

        //创建文章置顶栏目信息
        for (Object object : listArray) {
            JSONObject json = (JSONObject) object;
            //获取多个分类
            JSONArray categoryList = json.getJSONArray("categoryList");
            //如果同一频道有多个内容分类，就存多条数据
            for (Object o : categoryList) {
                JSONObject category = (JSONObject) o;
                HealthChannelDo healthChannel = objectMapper.readValue(object.toString(), HealthChannelDo.class);
                healthChannel.setFirstLevelCategoryId(category.getString("firstLevelCategoryId"));
                healthChannel.setFirstLevelCategoryName(category.getString("firstLevelCategoryName"));
                healthChannel.setCategoryId(category.getString("categoryId"));
                healthChannel.setCategoryName(category.getString("categoryName"));
                healthChannel.setLevel("2");
                healthChannel.setClassification(channelId);
                healthChannel.setClassificationName(channelName);
                healthChannel.setCreateTime(date);
                healthChannel.setLastUpdateTime(date);
                healthChannel.setAuthor(userCode);
                healthChannel.setAuthorName(userName);
                healthChannel.setUpdater(userCode);
                healthChannel.setUpdaterName(userName);
                healthChannel.setDelFlag(1);
                healthChannel.setDefaultFlag(0);
                result.add(healthChannel);
            }
        }
        channelDao.saveAll(result);
        return result;
    }

    public void updateDelFlagById(Long channelId, Integer delFlag) {
        //如果将状态改为失效，则居民端不再显示
        if (delFlag == 1) {
            channelDao.updateDelFlagById(channelId, 0);
        } else if (delFlag == 0) {
            //如果将状态改为生效，则之前选该频道的居民仍可以看到
            channelDao.updateDelFlagById(channelId, 1);
        }
    }

    /**
     * 根据频道id和分类等级获取频道配置的专栏及分类信息
     * @param channelId
     * @param level
     * @return
     */
    public List<Map<String, Object>> getRecommendCategoryByChannelId(String channelId, String level) {
        List<Map<String, Object>> res = new ArrayList<>();
        List<HealthChannelDo> recommendlist = channelDao.getRecommendCategoryByChannelId(channelId, level);
        for (HealthChannelDo healthChannelDo : recommendlist) {
            Map<String, Object> map = new HashMap<>();
            map.put("recommend", healthChannelDo.getCode());
            List<HealthChannelDo> categoryList = channelDao.getCategoryByCode(channelId, level, healthChannelDo.getCode());
            map.put("categoryList", categoryList);
            res.add(map);
        }
        return res;
    }

    /**
     * 获取频道列表
     * @param page
     * @param pageSize
     * @return
     */
    public Map<String, Object> getChannelList(Integer page, Integer pageSize) {
        Map<String, Object> resMap = new HashMap<>();
        List<Map<String, Object>> result = new ArrayList<>();
        //获取所有频道
        Pageable pageRequest = PageRequest.of(page - 1, pageSize);
        List<HealthChannelDo> channelList = channelDao.findByLevel("0", pageRequest);
        Integer count = channelDao.findByLevel("0");
        resMap.put("count", count);
        resMap.put("total", (count % pageSize == 0 ? count/pageSize : count/pageSize+1));
        for (HealthChannelDo channel : channelList) {
            Map<String, Object> map = new HashMap<>();
            String column = "";
            //获取推荐栏目
            List<HealthChannelDo> recommendList = channelDao.getRecommendCategoryByChannelId(channel.getId() + "", "1");
            for (HealthChannelDo healthChannel : recommendList) {
                String recommend = healthChannel.getCode();
                if (!column.contains("内容列表栏目") && (recommend.equals("health_article") || recommend.equals("platform_information"))) {
                    column += "内容列表栏目,";
                } else if (recommend.equals("health_column")) {
                    column += "健康专栏栏目,";
                } else if (recommend.equals("selected_questions_and_answers")) {
                    column += "精选问答栏目,";
                }
            }
            //获取文章置顶栏目
            List<HealthChannelDo> list = channelDao.getRecommendCategoryByChannelId(channel.getId() + "", "2");
            if (list.size() > 0) {
                column += "文章置顶栏目,";
            }
            if(StringUtils.isNotBlank(column) && column.length() > 0) {
                column = column.substring(0, column.length() - 1);
            }
            //组装数据
            map.put("channelId", channel.getId());
            map.put("name", channel.getName());
            map.put("column", column);
            map.put("delFlag", channel.getDelFlag());
            String updateTime = DateUtil.dateToStr(channel.getLastUpdateTime(), "yyyy-MM-dd HH:mm:ss");
            map.put("updateTime", updateTime);
            map.put("updaterName", channel.getUpdaterName());
            result.add(map);
        }
        resMap.put("list", result);
        return resMap;
    }

    /**
     * 根据id获取频道管理配置信息
     * @param channelId
     * @return
     */
    public Map<String, Object> getChannelById(Long channelId) {
        Map<String, Object> map = new HashMap<>();
        HealthChannelDo healthChannelDo = channelDao.findById(channelId).get();
        map.put("channelId", healthChannelDo.getId());
        map.put("name", healthChannelDo.getName());
        //获取推荐栏目
        List<Map<String, Object>> recommendList = getRecommendCategoryByChannelId(channelId + "", "1");

        map.put("recommend", recommendList);
        //获取健康文章置顶栏目
        List<Map<String, Object>> list = getRecommendCategoryByChannelId(channelId + "", "2");
        map.put("list", list);
        return map;
    }

    /**
     * 获取一二级分类
     * @param channelId
     * @param level
     * @param code
     * @return
     */
    public List<Map<String, Object>> getCategoryByCode(String channelId, String level, String code) {
        List<Map<String, Object>> resList = new ArrayList<>();
        List<HealthChannelDo> secondLevelCategoryIds = null;
        List<HealthChannelDo> firstLevelCategoryIds = null;
        if(level.equals("1")) {
            //根据分类类型获取推荐内容分类
            firstLevelCategoryIds = channelDao.findFirstLevelCategoryIdByLevelAndCode(channelId, level, code);
            for(HealthChannelDo h : firstLevelCategoryIds) {
                Map<String, Object> map = new HashMap<>();
                if(StringUtils.isNotBlank(h.getFirstLevelCategoryId())) {
                    map.put("firstLevelCategoryId", h.getFirstLevelCategoryId());
                    map.put("firstLevelCategoryName", h.getFirstLevelCategoryName());
                    secondLevelCategoryIds = channelDao.getSecondLevelCategoryId(channelId, level, code, h.getFirstLevelCategoryId());
                    map.put("secondLevelCategoryIds", secondLevelCategoryIds);
                    resList.add(map);
                }
            }
        }else if(level.equals("2")){
            //获取置顶内容类型和分类
            firstLevelCategoryIds = channelDao.findByLevelAndClassification(channelId, level);
            for(HealthChannelDo h : firstLevelCategoryIds) {
                Map<String, Object> map = new HashMap<>();
                if(StringUtils.isNotBlank(h.getFirstLevelCategoryId())) {
                    map.put("firstLevelCategoryId", h.getFirstLevelCategoryId());
                    map.put("firstLevelCategoryName", h.getFirstLevelCategoryName());
                    secondLevelCategoryIds = channelDao.getSecondLevelCategoryId(channelId, level, code, h.getFirstLevelCategoryId());
                    map.put("secondLevelCategoryIds", secondLevelCategoryIds);
                    resList.add(map);
                }
            }
        }
        return resList;
    }
}
