package com.yihu.customer.service.manager.healthcolumn;

import com.yihu.customer.service.BaseService;
import  com.yihu.customer.entity.healthcolumn.PeriodicalDo;
import  com.yihu.customer.repository.healthColumn.PeriodicalDao;
import com.yihu.customer.util.HttpManageClientUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Created by zdm on 2018/12/25.
 */
@Service
@Transactional
public class PeriodicalService extends BaseService {

    @Value("${jkEdu.web.articleBaseUrl}")
    private String articleBaseUrl;

    @Autowired
    private PeriodicalDao periodicalDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private HttpManageClientUtil httpManageClientUtil;

    /**
     * 分页查询 期刊
     * @param periodicalName 健康专栏名称
     * @param authorName 专栏作者
     * @param classification 分类
     * @param page 当前页
     * @param pageSize 分页大小
     * @return
     */
    public Page<List<Map<String, Object>>> searchList(String periodicalName, String authorName,String classification, Integer page, Integer pageSize) throws Exception{
        if (page == null) {
            page = 1;
        }
        if (pageSize == null) {
            pageSize = 15;
        }
        PageRequest pageRequest = PageRequest.of(page - 1, pageSize);
        StringBuilder filters = new StringBuilder();
        if(StringUtils.isNotBlank(periodicalName)){
            filters.append(" and temp2.name like '%" +periodicalName+"%'");
        }
        if(StringUtils.isNotBlank(authorName)){
            filters.append(" and  wwp.author_name like '%" +authorName+"%'");
        }
        if(StringUtils.isNotBlank(classification)){
            filters.append(" and  temp2.classification ='" +classification+"'");
        }
        filters.append(" AND wwp.id IS NOT null ");
        String sql = "SELECT  temp2.*, wwp.author_name, wwp.author_org_name, wwp.id wwp_id, wwp.code wwp_code, wwp.image wwp_image, wwp.release_time from ";
        String centerSql="(SELECT p.id, p. code, " +
                " p. name, " +
                " p.last_update_time, " +
                " p.classification, " +
                "p.del_flag, " +
                "p.sort, " +
                "temp1.maxwp_releatime, " +
                " p.classification_name " +
                "FROM " +
                " wlyy.wlyy_periodical p " +
                "LEFT JOIN ( " +
                " SELECT " +
                " wp.periodical_code periodical_code, " +
                " COUNT(1) wpCount, " +
                " MAX(wp.release_time) maxwp_releatime " +
                " FROM " +
                "  wlyy.wlyy_periodical pe " +
                " LEFT JOIN wlyy.wlyy_publication wp ON wp.periodical_code = pe.id " +
                " WHERE " +
                " wp. STATUS = '2' gROUP BY pe.id " +
                ") AS temp1 ON p.id = temp1.periodical_code  ) temp2 " +
                "LEFT JOIN wlyy.wlyy_publication wwp ON temp2.maxwp_releatime = wwp.release_time " +
                " AND wwp.periodical_code = temp2.id " +
                "WHERE temp2.del_flag != 0 ";
        String countSql = "SELECT count(1) FROM ";
        List<Map<String, Object>> resultList = jdbcTemplate.queryForList(sql +centerSql+ filters.toString() + " ORDER BY temp2.sort DESC" + " limit " + (page - 1) * pageSize + "," + pageSize);
        for(Map<String, Object> m:resultList){
            if(null != m.get("last_update_time")){
                m.put("last_update_time", m.get("last_update_time").toString());
                m.put("author_org_name",null!= m.get("author_org_name")? m.get("author_org_name").toString():"");
                m.put("author_name",null!= m.get("author_name")? m.get("author_name").toString():"");
            }
        }
        if (resultList.size() <= 0) {
            return new PageImpl(new ArrayList<>(), pageRequest, 0);
        }
        Long count = jdbcTemplate.queryForObject(countSql+centerSql + filters.toString(), Long.class);
        return new PageImpl(resultList, pageRequest, count);
    }

    /**
     * 获取当前最大排序号+1
     *
     */
    public Integer getMaxSort() throws Exception{
        String countSql = "SELECT MAX(wp.sort) FROM wlyy.wlyy_periodical wp WHERE wp.del_flag>0  ";
        Integer count = jdbcTemplate.queryForObject(countSql, Integer.class);
        return null==count?1:count + 1;
    }

    /**
     * 置顶
     */
    public Integer stick(Long pubid) {
        //当前最大sort
        String countSql = "SELECT MAX(wp.sort) FROM wlyy.wlyy_periodical wp WHERE wp.del_flag>0  ";
        Integer maxSort = jdbcTemplate.queryForObject(countSql, Integer.class);
        //当前刊物的专栏sort
        String sql = "SELECT p.sort FROM wlyy.wlyy_periodical p LEFT JOIN wlyy.wlyy_publication wp ON p.id=wp.periodical_code WHERE  wp.id="+pubid;
        Integer sort = jdbcTemplate.queryForObject(sql, Integer.class);
        if(sort>=maxSort){
            return sort;
        }else {
            return maxSort + 1;
        }

    }


    /**
     * 删除专栏
     * @param id
     * @Param status 0删除，1上线，2下线，3编辑，4上移，5下移
     * @return
     */
    public boolean updateOrMovePeriodical(Long id,Integer status)throws Exception{
        PeriodicalDo periodicalDo = periodicalDao.findById(id).get();
        if(status<3){
            if (status == 0 && periodicalDo.getDelFlag() == 1) {
                throw new Exception("该专栏处于上线状态，不能被删除！");
            }
            periodicalDo.setDelFlag(status);
            periodicalDao.save(periodicalDo);
        }else {
            Integer temp = periodicalDo.getSort();
            PeriodicalDo periodicalDo1 =null;
            if(status==4){
                //4上移
                periodicalDo1 =periodicalDao.findUpBySortAndDelFlag(temp);
                if(null== periodicalDo1){
                    throw new Exception("已是置顶专栏！");
                }else {
                    periodicalDo.setSort(periodicalDo1.getSort());
                    periodicalDo1.setSort(temp);
                    periodicalDao.save(periodicalDo1);
                    periodicalDao.save(periodicalDo);
                }

            }else {
                periodicalDo1 =periodicalDao.findDownBySortAndDelFlag(temp);
                if(null== periodicalDo1){
                    throw new Exception("已是置底专栏！");
                }else {
                    periodicalDo.setSort(periodicalDo1.getSort());
                    periodicalDo1.setSort(temp);
                    periodicalDao.save(periodicalDo1);
                    periodicalDao.save(periodicalDo);
                }

            }
        }
        return  true;
    }

}
