package com.yihu.customer.service.manager.hos;

import  com.yihu.customer.entity.SpecialDisease;
import  com.yihu.customer.entity.SystemDict;
import  com.yihu.customer.entity.organization.HlwyyHospital;
import  com.yihu.customer.repository.SpecialDiseaseDao;
import  com.yihu.customer.repository.SystemDictDao;
import  com.yihu.customer.repository.organization.HlwyyHospitalDao;
import com.yihu.customer.util.query.BaseJpaService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Created by yeshijie on 2023/2/1.
 */
@Service
public class HlwyyHospitalService  extends BaseJpaService<HlwyyHospital, Long> {

    @Autowired
    private HlwyyHospitalDao hlwyyHospitalDao;
    @Autowired
    private SystemDictDao systemDictDao;
    @Autowired
    private SpecialDiseaseDao specialDiseaseDao;

    @Transactional
    public HlwyyHospital createHospital(HlwyyHospital hospital) {
        if(hospital.getId()==null){
            //新增
            if(StringUtils.isEmpty(hospital.getCode())){
                hospital.setCode(getCode());
            }
            hospital.setCreateTime(new Date());
        }else {
            //修改
            HlwyyHospital old = hlwyyHospitalDao.findById(hospital.getId()).get();
            old.setRecommend(hospital.getRecommend());
            old.setDel(hospital.getDel());
            old.setType(hospital.getType());
            old.setCode(hospital.getCode());
            old.setPhoto(hospital.getPhoto());
            old.setName(hospital.getName());
            old.setLabelA(hospital.getLabelA());
            old.setLabelB(hospital.getLabelB());
            old.setLabelC(hospital.getLabelC());
            old.setIntro(hospital.getIntro());
            old.setAppid(hospital.getAppid());
            old.setHospitalLevel(hospital.getHospitalLevel());
            old.setLinkUrl(hospital.getLinkUrl());
            old.setSort(hospital.getSort());
        }

        hlwyyHospitalDao.save(hospital);

        return hospital;
    }


    //根据id获取单个
    public HlwyyHospital getHospital(long id) {
        HlwyyHospital hospital = hlwyyHospitalDao.findById(id).get();
        return hospital;
    }

    //获取列表(分页)
    public Page<HlwyyHospital> searchHospitals(String name, int page, int pageSize) throws Exception {
        if (page <= 0) {
            page = 1;
        }
        if (pageSize <= 0) {
            pageSize = 15;
        }
        // 分页信息
        PageRequest pageRequest = PageRequest.of(page - 1, pageSize);
        StringBuffer filters = new StringBuffer();
        if (!StringUtils.isEmpty(name)) {
            filters.append("name?" + name + ";");
        }
        StringBuffer doctorFilters = new StringBuffer();
        if (!StringUtils.isEmpty(name)) {
            doctorFilters.append("name?" + name + ";");
        }
        List<HlwyyHospital> list = search(null, filters.toString(), "-recommend,+sort", page, pageSize);


        long count = 0;
        if (list.size() > 0) {

             for (HlwyyHospital hospital:list){
                hospital.setTypeName(systemDictDao.findByDictNameAndCode2("hlw_org_type",hospital.getType()+""));
                hospital.setHospitalLevelName(systemDictDao.findByDictNameAndCode2("hlw_org_level",hospital.getHospitalLevel()));
                labelBName(hospital);
                labelCName(hospital);
            }
            count = getCount(filters.toString());
        }
        return new PageImpl<HlwyyHospital>(list, pageRequest, count);

    }

    private void labelBName(HlwyyHospital hospital){
        String labelB = hospital.getLabelB();
        if(!StringUtils.isEmpty(labelB)){
            List<SpecialDisease> diseaseList = specialDiseaseDao.findDisByDel("1");
            Map<String,String> diseaseMap = diseaseList.stream().collect(Collectors.toMap(SpecialDisease::getCode,SpecialDisease::getName));
            String labelb[] = labelB.split(",");
            String labelName = "";
            for (String label:labelb){
                labelName += diseaseMap.get(label)+",";
            }
            labelName = labelName.substring(0,labelName.length()-1);
            hospital.setLabelBName(labelName);
        }
    }

    private void labelCName(HlwyyHospital hospital){
        String labelC = hospital.getLabelC();
        if(!StringUtils.isEmpty(labelC)){
            List<SystemDict> dictList = systemDictDao.findByDictName("hlw_service_type");
            Map<String,String> dictMap = dictList.stream().collect(Collectors.toMap(SystemDict::getCode,SystemDict::getValue));
            String labelc[] = labelC.split(",");
            String labelName = "";
            for (String label:labelc){
                labelName += dictMap.get(label)+",";
            }
            labelName = labelName.substring(0,labelName.length()-1);
            hospital.setLabelCName(labelName);
        }
    }

}
