package com.yihu.customer.service.manager.hos;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.customer.service.common.excel.ExcelData;
import  com.yihu.customer.entity.specialist.DoctorSpecialist;
import  com.yihu.customer.entity.twoCancers.BreastCervicalFormPermissions;
 
 
import  com.yihu.customer.repository.*;
import  com.yihu.customer.repository.specialist.DoctorSpecialistDao;
import  com.yihu.customer.repository.twoCancers.BreastCervicalFormPermissionsDao;
import  com.yihu.customer.repository.wechat.DoctorMappingDao;
import com.yihu.customer.util.AesEncryptUtils;
import com.yihu.customer.util.HttpManageClientUtil;
import com.yihu.customer.util.MD5;
import com.yihu.customer.util.ReadExcelUtil;
import com.yihu.customer.util.query.BaseJpaService;
import  com.yihu.customer.entity.*;
import jxl.Sheet;
import jxl.Workbook;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.text.ParseException;
import java.util.*;

/**
 * Created by Administrator on 2016/10/18.
 */
@Service
public class HosDoctorService extends BaseJpaService<Doctor, Long> {
    @Autowired
    private DoctorDao doctorDao;
    @Autowired
    private HospitalDao hospitalDao;

    @Autowired
    private WlyyAdminTeamMemberDao teamMemberDao;

    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private DmJobDao jobDao;
    @Autowired
    private HospitalDeptDao hospitalDeptDao;
    @Autowired
    private UserRoleDao userRoleDao;
    @Autowired
    private DoctorSpecialistDao doctorSpecialistDao;
    @Autowired
    private WlyyhospitalspecialistDao wlyyhospitalspecialistDao;
    @Autowired
    private DoctorMappingDao doctorMappingDao;
    @Autowired
    private OrderDoctorMappingDao orderDoctorMappingDao;
    @Autowired
    private HttpManageClientUtil httpManageClientUtil;
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private PharmacyRoleDictDao pharmacyRoleDictDao;
    @Autowired
    private BreastCervicalFormPermissionsDao formPermissionsDao;

    /**
     * 获取两癌筛查表单权限
     * @param doctor
     * @return
     */
    public List<BreastCervicalFormPermissions> findformPermissionsByDoctor(String doctor){
        List<BreastCervicalFormPermissions> list = formPermissionsDao.findByDoctor(doctor);
        return list;
    }

    /**
     * 保存两癌筛查表单权限
     */
    @Transactional(rollbackFor = Exception.class)
    public void addFormPermissions(String doctor,List<BreastCervicalFormPermissions> list){
        formPermissionsDao.deleteByDoctor(doctor);
        if(list!=null&&list.size()>0){
            for (BreastCervicalFormPermissions permissions:list){
                permissions.setDoctor(doctor);
            }
            formPermissionsDao.saveAll(list);
        }
    }

    /**
     * 获取两癌筛查医生列步兵
     * @param page
     * @param rows
     * @param mobile
     * @param name
     * @return
     * @throws Exception
     */
    public Page<Doctor> getBreastCervicalDoctorList(
            int page, int rows,String mobile, String name) throws Exception {
        if (page <= 0) {
            page = 1;
        }
        if (rows <= 0) {
            rows = 10;
        }
        PageRequest pageRequest = PageRequest.of(page - 1, rows);
        // 设置查询条件//
        StringBuffer filters = new StringBuffer();
        // 所属机构
        filters.append("expandLevel=18,19,20;");
        //名字模糊查询
        if (StringUtils.isNotBlank(name)) {
            filters.append("name?" + name + ";");
        }
        //状态码查询
        filters.append("status=1;");
        //手机号码查询
        if (StringUtils.isNotBlank(mobile)) {
            filters.append("mobile?" + mobile + ";");
        }

        List<Doctor> doctorList = search("", filters.toString(), "", page, rows);
        long count = 0;
        if (doctorList != null && doctorList.size() > 0) {
            count = getCount(filters.toString());
        }
        return new PageImpl<Doctor>(doctorList, pageRequest, count);
    }

    //根据id获取医生信息
    public Doctor getDoctor(Long id) throws Exception {
        Doctor doctor = doctorDao.findById(id).get();
        if (doctor != null) {
            List<WlyyAdminTeamMember> memberList = teamMemberDao.findByDoctorCode(doctor.getCode());
            if (memberList != null && memberList.size() > 0) {
                String teamId = "";
                for (WlyyAdminTeamMember m : memberList) {
                    teamId += "," + m.getTeamId();
                }
                doctor.setAdminTeamId(teamId.substring(1));
            }
        }

        List<DoctorSpecialist> doctorSpecialistList = doctorSpecialistDao.findByDoctor(doctor.getCode());
        if(doctorSpecialistList==null||doctorSpecialistList.size()==0){
            //专病为空时，默认填充医院服务专病
            doctorSpecialistList = doctorSpecialistList(doctor);
        }
        doctor.setSpecialist(doctorSpecialistList);

        doctor.setJwDoctor(getJwDoctor(doctor.getCode()));

        return doctor;
    }

    /**
     * 获取基卫code
     * @param doctor
     * @return
     */
    public String getJwDoctor(String doctor){
        String sql = "SELECT IFNULL(GROUP_CONCAT(jw_doctor),'') jwDoctor from wlyy_order_doctor_mapping where doctor = '"+doctor+"'";
        String jwDoctor =  jdbcTemplate.queryForObject(sql,String.class);
        return jwDoctor;
    }

    public void addJwDoctor(String doctor,String jwDoctor){
        if(StringUtils.isBlank(jwDoctor)) {
            return;
        }
        String jwd[] = jwDoctor.split(",");
        List<OrderDoctorMapping> list = new ArrayList<>();
        for(String tmp:jwd){
            OrderDoctorMapping mapping = new OrderDoctorMapping();
            mapping.setJwDoctor(tmp);
            mapping.setDoctor(doctor);
            list.add(mapping);
        }
        orderDoctorMappingDao.saveAll(list);
    }

    public void updJwDoctor(String doctor,String jwDoctor){
        if(StringUtils.isBlank(jwDoctor)){
            orderDoctorMappingDao.deleteByDoctor(doctor);
        }else if(!jwDoctor.equals(getJwDoctor(doctor))){
            orderDoctorMappingDao.deleteByDoctor(doctor);
            addJwDoctor(doctor, jwDoctor);
        }
    }

    //根据编码获取医生信息
    public Doctor getDoctorByCode(String code) throws Exception {
        Doctor doctor =  doctorDao.findByCode(code);
        if (null!=doctor){
            if (StringUtils.isNotBlank(doctor.getExpandLevel())) {
                String sql = " select value  from system_dict where dict_name='doctorExpandLevel' and code='"+doctor.getExpandLevel()+"' ";
                List<String> ExpandLevelNames = jdbcTemplate.queryForList(sql,String.class);
                if (ExpandLevelNames.size()>0){
                    doctor.setExpandLevelName(ExpandLevelNames.get(0));
                    return doctor;
                }
            }
            doctor.setExpandLevelName("无");
        }
        return doctorDao.findByCode(code);
    }

    //根据手机号查询
    public Doctor getDoctorByMobile(String mobile) throws Exception {
        return doctorDao.findByMobile(mobile);
    }

    // 手机号查询医生信息(用于其它功能，防多记录对后续操作影响）
    public boolean isExistMobile(String mobile) {
        int num = doctorDao.isExistMobile(mobile);
        return num > 0 ? true : false;
    }

    //批量更新状态
    @Transactional
    public void updateStatus(String ids, String status) throws Exception {
        int flag = Integer.parseInt(status);
        String[] idArray = ids.split(";");
        for (String id : idArray) {
            Doctor doctor = doctorDao.findById(Long.valueOf(id)).get();
            doctor.setStatus(flag);
            doctorDao.save(doctor);
        }
    }

    //新增医生信息
    @Transactional
    public void saveDoctor(Doctor doctor) throws Exception {
        // 判断手机是否注册过
        Doctor doc = doctorDao.findByMobile(doctor.getMobile());
        if (doc != null) {
            throw new Exception("手机已经注册过");
        }
        //判断编码是否重复
        Doctor docByCode = doctorDao.findByCode(doctor.getCode());
        if (docByCode != null) {
            throw new Exception("该编码已被使用");
        }
        //出生日期
        if (doctor.getBirthday() == null && !StringUtils.isEmpty(doctor.getIdCard())) {
            doctor.setBirthday(DateUtils.parseDate(getBirthdayByIdCard(doctor.getIdCard()), "yyyy-MM-dd"));
        }
        //获取所属机构信息
        Hospital hos = hospitalDao.findByCode(doctor.getHospital());
        //添加医生地址信息
        doctor.setProvince(hos.getProvince());
        doctor.setProvinceName(hos.getProvinceName());
        doctor.setCity(hos.getCity());
        doctor.setCityName(hos.getCityName());
        doctor.setTown(hos.getTown());
        doctor.setTownName(hos.getTownName());
        doctor.setHospitalName(hos.getName());
        //当为管理员时添加到wlyy_user_role
//        int level = doctor.getLevel();
//        if(level == 10){
//            UserRole userRole = new UserRole();
//            userRole.setCzrq(new Date());
//            userRole.setRole(hos.getCode());
//            userRole.setUser(doctor.getCode());
//            userRoleDao.save(userRole);
//        }
        // 保存用户
        String password = doctor.getMobile().substring(5);
        String salt = UUID.randomUUID().toString().replace("-", "");
        doctor.setSalt(salt);
        doctor.setPassword(MD5.GetMD5Code("a"+password+"a" + salt));
        doctor.setCzrq(new Date());
        doctor.setPhoto("../../../images/d-male.png");
        //默认为1
        doctor.setStatus(1);
        doctorDao.save(doctor);

        addJwDoctor(doctor.getCode(),doctor.getJwDoctor());

        doctorSpecialistList(doctor);
    }

    /**
     * 保存医生mapping表
     * @param jwDoctorCode
     * @param doctor
     */
    public void addJwDoctorCode(String jwDoctorCode,Doctor doctor){
        DoctorMapping doctorMapping = new DoctorMapping();
        doctorMapping.setDoctorCode(doctor.getCode());
        doctorMapping.setJwDocotrIdcard(doctor.getIdCard());
        doctorMapping.setJwDoctorName(doctor.getName());
        doctorMapping.setJwDoctor(jwDoctorCode);
        doctorMappingDao.save(doctorMapping);
    }

    public List<DoctorSpecialist> doctorSpecialistList(Doctor doctor){
        //默认填充医院服务专病
        List<Wlyyhospitalspecia> wlyyhospitalspeciaList = wlyyhospitalspecialistDao.findByCode(doctor.getHospital());
        if(wlyyhospitalspeciaList!=null){
            List<DoctorSpecialist> doctorSpecialistList = new ArrayList<>();
            for (Wlyyhospitalspecia one:wlyyhospitalspeciaList){
                DoctorSpecialist specialist = new DoctorSpecialist();
                specialist.setCreateTime(new Date());
                specialist.setDoctor(doctor.getCode());
                specialist.setDoctorName(doctor.getName());
                specialist.setHospital(one.getHospital());
                specialist.setHospitalName(one.getHospital_name());
                specialist.setCode(one.getCode());
                specialist.setName(one.getName());
                doctorSpecialistList.add(specialist);
            }
            if (doctorSpecialistList.size()>0){
                doctorSpecialistDao.saveAll(doctorSpecialistList);
            }
            return doctorSpecialistList;
        }
        return null;
    }

    //修改医生信息
    @Transactional(rollbackFor = Exception.class)
    public void updateDoctor(Doctor doctor) throws Exception {
        // 判断手机是否注册过
        List<Doctor> docs = doctorDao.findByMobileNotId(doctor.getMobile(), doctor.getId());
        if (docs != null && docs.size() > 0) {
            throw new Exception("手机已经注册过");
        }

        //判断编码是否重复
        Doctor docByCode = doctorDao.findByCode(doctor.getCode());
        if (docByCode != null && !StringUtils.equals(docByCode.getCode(), doctor.getCode())) {
            throw new Exception("该编码已被使用");
        }
        // 更新用户
        Doctor doctorNew = doctorDao.findById(doctor.getId()).get();
        doctorNew.setName(doctor.getName());
        doctorNew.setSex(doctor.getSex());
        doctorNew.setDeptName(doctor.getDeptName());
        doctorNew.setJobName(doctor.getJobName());
        doctorNew.setJob(doctor.getJob());
        doctorNew.setMobile(doctor.getMobile());
        doctorNew.setHospital(doctor.getHospital());
        doctorNew.setStatus(doctor.getStatus());
        if (StringUtils.isEmpty(doctorNew.getPhoto())) {
            doctorNew.setPhoto("../../../images/d-male.png");
        } else {
            //doctor.setPhoto(doctor.getPhoto());
        }
//        doctorNew.setCode(doctor.getCode());
        doctorNew.setIntroduce(doctor.getIntroduce());
        doctorNew.setExpertise(doctor.getExpertise());
        String idCardNew = doctor.getIdCard();
        //身份证变化对应出生日期变化
        if (!StringUtils.isEmpty(idCardNew) && !StringUtils.equals(doctorNew.getIdCard(), idCardNew)) {
            doctorNew.setBirthday(DateUtils.parseDate(getBirthdayByIdCard(idCardNew), "yyyy-MM-dd"));
        }
        doctorNew.setIdCard(idCardNew);
        doctorNew.setShequType(doctor.getShequType());
        doctorNew.setCzrq(new Date());
        doctorNew.setIscertified(doctor.getIscertified());
        doctorNew.setIsFamous(doctor.getIsFamous());
//        if(doctorNew.getLevel() == 10 && doctor.getLevel() !=10){
//            userRoleDao.deleteByUserRole(doctorNew.getCode());
//        }
//        if(doctor.getLevel() == 10 && doctorNew.getLevel() !=10){
//            UserRole userRole = new UserRole();
//            userRole.setCzrq(new Date());
//            userRole.setRole(doctor.getHospital());
//            userRole.setUser(doctor.getCode());
//            userRoleDao.save(userRole);
//        }
        doctorNew.setLevel(doctor.getLevel());
        doctorNew.setExpandLevel(doctor.getExpandLevel());
        doctorDao.save(doctorNew);

        if("18".equals(doctor.getExpandLevel())||"19".equals(doctor.getExpandLevel())||"20".equals(doctor.getExpandLevel())){
            List<BreastCervicalFormPermissions> breastCervicalFormPermissionsList = new ArrayList<>();
            String sql = " select count(id) from breast_cervical_form_permissions where doctor='"+doctor.getCode()+"' ";
            Integer roleCount = jdbcTemplate.queryForObject(sql,Integer.class);
            if (0==roleCount){
                sql = " SELECT code,`value` FROM `wlyy`.`system_dict` WHERE `dict_name` = 'breast_cervical_form_permissions'; ";
                List<Map<String,Object>> roleList = jdbcTemplate.queryForList(sql);
                for (Map<String,Object>tmp:roleList){
                    BreastCervicalFormPermissions breastCervicalFormPermissions = new BreastCervicalFormPermissions();
                    breastCervicalFormPermissions.setDoctor(doctor.getCode());
                    breastCervicalFormPermissions.setItem(tmp.get("code").toString());
                    breastCervicalFormPermissions.setItemName(tmp.get("value").toString());
                    breastCervicalFormPermissionsList.add(breastCervicalFormPermissions);
                }
            }
            if (breastCervicalFormPermissionsList.size()>0){
                formPermissionsDao.deleteByDoctor(doctor.getCode());
                formPermissionsDao.saveAll(breastCervicalFormPermissionsList);
            }
        }

        updJwDoctor(doctor.getCode(),doctor.getJwDoctor());

        doctorSpecialistDao.deleteByDoctor(doctor.getCode());
        if(doctor.getSpecialist()!=null&&doctor.getSpecialist().size()>0){
            List<DoctorSpecialist> doctorSpecialistList = doctor.getSpecialist();
            for(DoctorSpecialist specialist: doctorSpecialistList){
                specialist.setCreateTime(new Date());
                specialist.setDoctor(doctor.getCode());
                specialist.setDoctorName(doctor.getName());
                specialist.setHospital(doctor.getHospital());
                specialist.setHospitalName(doctor.getHospital());
            }
            doctorSpecialistDao.saveAll(doctorSpecialistList);
        }

        if (StringUtils.isNotBlank(doctorNew.getExpandLevel())){
            String  sql = " select id,name from wlyy_pharmacy_role_dict where id='"+doctorNew.getExpandLevel()+"' ";
            List<Map<String,Object>> pharmacyDict = jdbcTemplate.queryForList(sql);
            if (pharmacyDict.size()>0){
                doctorNew.setExpandLevelName(pharmacyDict.get(0).get("name").toString());
                sql = " select code from system_dict where dict_name ='hlwJumiaiUrl'  ";
                List<String> urls = jdbcTemplate.queryForList(sql,String.class);
                if (urls.size()>0) {
                    String url = urls.get(0) + "/base/open/noLogin/addUser";
                    Map<String,Object> param = new HashMap<>();
                    JSONObject jsonObject =JSONObject.parseObject(JSON.toJSONString(doctorNew, SerializerFeature.WriteMapNullValue));
                    jsonObject.put("idcard",doctorNew.getIdCard());
                    param.put("jsonData",jsonObject.toJSONString(jsonObject,SerializerFeature.WriteMapNullValue));
                    String content = com.alibaba.fastjson.JSONObject.toJSONString(param);
                    String params = AesEncryptUtils.encrypt(content,"46A61629A19AE04C");
                    String response = httpManageClientUtil.postBodyRawForm(url,params);
                    com.alibaba.fastjson.JSONObject resultObject = com.alibaba.fastjson.JSONObject.parseObject(response);
                    if (resultObject.get("status").equals("200")) {
                        com.alibaba.fastjson.JSONObject jsonObjectData = com.alibaba.fastjson.JSONObject.parseObject(AesEncryptUtils.decrypt(resultObject.get("data").toString(),"46A61629A19AE04C"));
                      if(jsonObjectData.getInteger("status")==200){
                          return;
                      }
                    }
                    throw new Exception("添加药柜角色信息失败");
                }
            }//同步角色信息给hlw
        }
    }

    //删除医生信息
    @Transactional
    public void deleteDoctorByid(Long id, String code) throws Exception {
        doctorDao.deleteById(id);
    }
    //查询机构医生列表（分页）

    public Page<Doctor> getDoctorList(
            int page, int rows, String hospital, String deptName, String jobName,
            String status, String mobile, String name) throws Exception {
        if (page <= 0) {
            page = 1;
        }
        if (rows <= 0) {
            rows = 10;
        }
        PageRequest pageRequest = PageRequest.of(page - 1, rows);
        // 设置查询条件//
        StringBuffer filters = new StringBuffer();
        // 所属机构
        filters.append("hospital=" + hospital + ";");
        //名字模糊查询
        if (StringUtils.isNotBlank(name)) {
            filters.append("name?" + name + ";");
        }
        //职称代码查询？名字 job_name
        if (StringUtils.isNotBlank(jobName)) {
            filters.append("jobName?" + jobName + ";");
        }
        //状态码查询
        if (StringUtils.isNotBlank(status)) {
            filters.append("status=" + status + ";");
        }
        //手机号码查询
        if (StringUtils.isNotBlank(mobile)) {
            filters.append("mobile?" + mobile + ";");
        }
        //所属部门码查询？名字 dept_name
        if (StringUtils.isNotBlank(deptName)) {
            filters.append("deptName?" + deptName + ";");
        }
        List<Doctor> doctorList = search("", filters.toString(), "", page, rows);
        long count = 0;
        if (doctorList != null && doctorList.size() > 0) {
            count = getCount(filters.toString());
            for (Doctor doc : doctorList) {
                List<WlyyAdminTeamMember> memberList = teamMemberDao.findByDoctorCode(doc.getCode());
                if (memberList != null && memberList.size() > 0) {
                    String teamId = "";
                    for (WlyyAdminTeamMember m : memberList) {
                        teamId += "," + m.getTeamId();
                    }
                    doc.setAdminTeamId(teamId.substring(1));
                }
            }
        }
        return new PageImpl<Doctor>(doctorList, pageRequest, count);


        // 设置查询条件//
//        Map<String, SearchFilter> filters = new HashMap<String, SearchFilter>();
//        // 所属机构
//        filters.put("hospital", new SearchFilter("hospital",
//                SearchFilter.Operator.EQ, hospital));
//        //名字模糊查询
//        if (StringUtils.isNotBlank(name)) {
//            filters.put("name", new SearchFilter("name", SearchFilter.Operator.LIKE, name));
//        }
//        //职称代码查询？名字 job_name
//        if (StringUtils.isNotBlank(jobName)) {
//            filters.put("jobName", new SearchFilter("jobName", SearchFilter.Operator.LIKE, jobName));
//        }
//        //签约状态码查询
//        if (StringUtils.isNotBlank(status)) {
//            filters.put("status", new SearchFilter("status", SearchFilter.Operator.EQ, status));
//        }
//        //手机号码查询
//        if (StringUtils.isNotBlank(mobile)) {
//            filters.put("mobile", new SearchFilter("mobile", SearchFilter.Operator.LIKE, mobile));
//        }
//        //所属部门码查询？名字 dept_name
//        if (StringUtils.isNotBlank(deptName)) {
//            filters.put("deptName", new SearchFilter("deptName", SearchFilter.Operator.LIKE, deptName));
//        }
//        // 分页信息
//        PageRequest pageRequest = PageRequest.of(page - 1, rows);
//
//        Specification<Doctor> spec = DynamicSpecifications.bySearchFilter(
//                filters.values(), Doctor.class);
//        Page<Doctor> doctors = doctorDao.findAll(spec, pageRequest);
//        return doctors;
//          return null;
    }

    public List<Doctor> doctorListNoPage(String hospital, String doctorName) throws Exception {

        // 设置查询条件//
        StringBuffer filters = new StringBuffer();
        // 所属机构
        filters.append("hospital=" + hospital + ";");
        //名字模糊查询
        if (StringUtils.isNotBlank(doctorName)) {
            filters.append("name?" + doctorName + ";");
        }
        List<Doctor> doctorList = search(filters.toString());
        long count = 0;
        if (doctorList != null && doctorList.size() > 0) {
            count = getCount(filters.toString());
            for (Doctor doc : doctorList) {
                List<WlyyAdminTeamMember> memberList = teamMemberDao.findByDoctorCode(doc.getCode());
                if (memberList != null && memberList.size() > 0) {
                    String teamId = "";
                    for (WlyyAdminTeamMember m : memberList) {
                        teamId += "," + m.getTeamId();
                    }
                    doc.setAdminTeamId(teamId.substring(1));
                }
            }
        }
       return doctorList;
    }

    @Transactional
    public boolean updateDel(Integer status, Long doctorId) {
        int sum = doctorDao.updateStatus(status, doctorId);
        return sum > 0 ? true : false;
    }

    @Transactional
    public boolean saveDoctorList(List<Doctor> doctorList) {
        doctorDao.saveAll(doctorList);
        return true;
    }

    //根据姓名、机构名判断是否重复
    public boolean findByNameHospital(String name, String hospital) {
        List<Doctor> doctorList = doctorDao.findByNameHospital(name, hospital);
        if (doctorList != null && doctorList.size() > 0) {
            return true;
        }
        return false;
    }

    //根据姓名、身份证号判断是否重复
    public boolean findByNameIdcard(String name, String idcard) {
        List<Doctor> doctorList = doctorDao.findByNameIdcard(name, idcard);
        if (doctorList != null && doctorList.size() > 0) {
            return true;
        }
        return false;
    }

    //初始密码
    @Transactional
    public boolean initPassword(Long id, String mobile) {
        String password = mobile.substring(5);
        String salt = UUID.randomUUID().toString().replace("-", "");
        password = MD5.GetMD5Code("a"+password+"*" + salt);
        int sum = doctorDao.initPassword(password, salt, id);
        return sum > 0 ? true : false;
    }

    //获取性别
    public Integer getGenderByIdCard(String idCard) {
        if (idCard.length() != 15 && idCard.length() != 18) {
            return null;
        }
        int gender = 0;
        if (idCard.length() == 15) {
            gender = idCard.charAt(14) % 2 == 0 ? 2 : 1;
        } else if (idCard.length() == 18) {
            gender = idCard.charAt(16) % 2 == 0 ? 2 : 1;
        }
        return gender;
    }

    //获取生日
    public String getBirthdayByIdCard(String idCard) {
        if (idCard.length() != 15 && idCard.length() != 18) {
            return null;
        }
        String birthday = null;
        if (idCard.length() == 15) {
            birthday = "19" + idCard.substring(6, 8) + "-" + idCard.substring(8, 10) + "-" + idCard.substring(10, 12);
        } else if (idCard.length() == 18) {
            birthday = idCard.substring(6, 10) + "-" + idCard.substring(10, 12) + "-" + idCard.substring(12, 14);
        }
        return birthday;
    }

    //根据身份证查询
    public boolean isExistIdCard(String idCard) {
        int num = doctorDao.isExistIdCard(idCard);
        return num > 0;
    }

    public boolean importData(Workbook workbook) throws ParseException {
        Sheet[] sheets = workbook.getSheets();
        Sheet sheet = sheets[0];
        int rows = ReadExcelUtil.getRightRows(sheet);

        List<Doctor> doctorList = new ArrayList<>();
        for (int row = 1; row < rows; row++) {  //索引从0开始，第一行为标题
            Doctor doctor = new Doctor();
            Map<Integer, ExcelData> mapping = mapping(doctor);
            int finalRow = row;
            mapping.forEach((index, excelData) -> {
                String value = sheet.getCell(index, finalRow).getContents().trim();
                excelData.transform(value);
            });

            //Additional Handel
            if (StringUtils.isEmpty(doctor.getHospitalName())) {
                continue;
            }

            if (StringUtils.isEmpty(doctor.getMobile())) {
                continue;
            }

            if (StringUtils.isEmpty(doctor.getIdCard())) {
                continue;
            }

            boolean existIdCard = this.isExistIdCard(doctor.getIdCard());
            if (existIdCard) {
                continue;
            }

            doctor.setSex(this.getGenderByIdCard(doctor.getIdCard()));
            String birthday = this.getBirthdayByIdCard(doctor.getIdCard());
            Date date = StringUtils.isEmpty(birthday) ? null : DateUtils.parseDate(birthday, "yyyy-MM-dd");
            doctor.setBirthday(date);

            doctor.setCode(UUID.randomUUID().toString().replace("-", ""));
            String subMobile = doctor.getMobile().substring(5); //初始密码、salt值
            String salt = UUID.randomUUID().toString().replace("-", "");
            doctor.setSalt(salt);
            doctor.setPassword(MD5.GetMD5Code(subMobile + salt));
            doctor.setIscertified(0);
            doctor.setCzrq(new Date());
            doctor.setPhoto("../../../images/d-male.png");
            doctor.setStatus(1);
            doctorList.add(doctor);
        }

        doctorDao.saveAll(doctorList);
        return true;
    }

    private Map<Integer, ExcelData> mapping(Doctor doctor) {
        Map<Integer, ExcelData> dataMap = new HashMap<>();

        //姓名
        dataMap.put(0, new ExcelData() {
            @Override
            public void transform(String data) {
                doctor.setName(data);
            }
        });

        //性别
        dataMap.put(1, new ExcelData() {
            @Override
            public void transform(String data) {
            }
        });

        //身份证号
        dataMap.put(2, new ExcelData() {
            @Override
            public void transform(String data) {
                doctor.setIdCard(data);
            }
        });

        //人员类型
        dataMap.put(3, new ExcelData() {
            @Override
            public void transform(String data) {
                doctor.setShequType(data);
            }
        });

        //技术职称   临床职称
        dataMap.put(4, new ExcelData() {
            @Override
            public void transform(String data) {
                doctor.setJobName(data);
            }
        });

        //行政职务
        dataMap.put(5, new ExcelData() {
            @Override
            public void transform(String data) {
            }
        });

        //人员专长
        dataMap.put(6, new ExcelData() {
            @Override
            public void transform(String data) {
                doctor.setExpertise(data);
            }
        });

        //机构名称
        dataMap.put(8, new ExcelData() {
            @Override
            public void transform(String data) {
                doctor.setHospitalName(data);
                Hospital hospital = hospitalDao.findByName(data);
                if (hospital != null) {
                    doctor.setHospital(hospital.getCode());
                }
            }
        });

        //科室
        dataMap.put(9, new ExcelData() {
            @Override
            public void transform(String data) {
                doctor.setDeptName(data);
//                doctor.setDept(data);   //TODO:
                String hospital = doctor.getHospital();
                //findByHospitalCodeAndDeptName
                HospitalDept hospitalDept = hospitalDeptDao.findByHospitalCodeAndDeptName(hospital,data);
                if(hospitalDept==null){
                    HospitalDept hospitalDept2 = new HospitalDept();
                    hospitalDept2.setHospital(hospital);
                    hospitalDept2.setDel("1");
                    hospitalDept2.setName(data);
                    hospitalDept2.setCode(UUID.randomUUID().toString());
                    hospitalDept = hospitalDeptDao.save(hospitalDept2);
                }
                doctor.setDept(hospitalDept.getCode());
            }
        });

        //简介
        dataMap.put(10, new ExcelData() {
            @Override
            public void transform(String data) {
                doctor.setIntroduce(data);
            }
        });

        //手机
        dataMap.put(11, new ExcelData() {
            @Override
            public void transform(String data) {
                doctor.setMobile(data);
            }
        });

        //角色
        dataMap.put(12, new ExcelData() {
            @Override
            public void transform(String data) {
                Map<String, Integer> map = new HashMap<>();
                map.put("专科医生", 1);
                map.put("全科医生", 2);
                map.put("健康管理师", 3);
                map.put("同安计生协会工作人员", 11);
                doctor.setLevel(map.get(data));
            }
        });

        return dataMap;
    }

    public List<Doctor> exportDoctorList(String hospital, String deptName, String jobName, String status, String mobile, String name) throws Exception {
        // 设置查询条件//
        StringBuffer filters = new StringBuffer();
        // 所属机构
        if (StringUtils.isNotBlank(hospital)) {
            filters.append("hospital=" + hospital + ";");
        }
        //名字模糊查询
        if (StringUtils.isNotBlank(name)) {
            filters.append("name?" + name + ";");
        }
        //职称代码查询？名字 job_name
        if (StringUtils.isNotBlank(jobName)) {
            filters.append("jobName?" + jobName + ";");
        }
        //状态码查询
        filters.append("status=1;");
        //手机号码查询
        if (StringUtils.isNotBlank(mobile)) {
            filters.append("mobile?" + mobile + ";");
        }
        //所属部门码查询？名字 dept_name
        if (StringUtils.isNotBlank(deptName)) {
            filters.append("deptName?" + deptName + ";");
        }
        List<Doctor> doctorList = search(filters.toString());
        if (doctorList != null && doctorList.size() > 0) {
            for (Doctor doc : doctorList) {
                List<WlyyAdminTeamMember> memberList = teamMemberDao.findByDoctorCode(doc.getCode());
                if (memberList != null && memberList.size() > 0) {
                    String teamId = "";
                    for (WlyyAdminTeamMember m : memberList) {
                        teamId += "," + m.getTeamId();
                    }
                    doc.setAdminTeamId(teamId.substring(1));
                }
            }
        }
        return doctorList;
    }

    public List<Map<String,Object>> getDoctorExpandLevelDict() throws IOException {
        //实时去获取hlw药柜角色
        String sql = " select code from system_dict where dict_name ='hlwJumiaiUrl'  ";

        List<String> urls = jdbcTemplate.queryForList(sql,String.class);
        if (urls.size()>0){
            String url = urls.get(0)+"/base/open/noLogin/roleDictList";
//                url  = "http://ehr.yihu.com/hlwyy/cityihealth/base/open/noLogin/roleDictList";
            Map map = new HashMap<String, String>();
            String content = com.alibaba.fastjson.JSONObject.toJSONString(map);
            String params = AesEncryptUtils.encrypt(content,"46A61629A19AE04C");
            List<PharmacyRoleDictDO> details = new ArrayList<>();
            String response = httpManageClientUtil.getBodyRawForm(url,params);
            com.alibaba.fastjson.JSONObject resultObject = com.alibaba.fastjson.JSONObject.parseObject(response);
            if (resultObject.get("status").equals("200")) {
                com.alibaba.fastjson.JSONObject jsonObjectData = com.alibaba.fastjson.JSONObject.parseObject(AesEncryptUtils.decrypt(resultObject.get("data").toString(),"46A61629A19AE04C"));
                if (jsonObjectData.getJSONArray("detailModelList")!=null){
                    JSONArray roleList = jsonObjectData.getJSONArray("detailModelList");
                    details = objectMapper.readValue(roleList.toString(), new TypeReference<List<PharmacyRoleDictDO>>(){});
                    pharmacyRoleDictDao.deleteAll();
                }
            }
            pharmacyRoleDictDao.saveAll(details);
        }

        List<Map<String,Object>> list = new ArrayList<>();
        sql = " select id as code,name from wlyy_pharmacy_role_dict where status=1 ";
        list = jdbcTemplate.queryForList(sql);
        sql = " select code,`value` as name from system_dict where dict_name='doctorExpandLevel' order by sort asc ";
        List<Map<String,Object>> doctorExpandLevels = jdbcTemplate.queryForList(sql);
        list.addAll(doctorExpandLevels);
        return list;
     }

    public List<Map<String,Object>> getDictByName(String dictName) throws IOException {
        String sql = " select code,`value` as name from system_dict where dict_name='"+dictName+"' order by sort asc ";
        List<Map<String,Object>> list = jdbcTemplate.queryForList(sql);
        return list;
    }

}
