package com.yihu.customer.service.manager.hos;

 
import com.yihu.customer.service.common.district.DistrictService;
import com.yihu.customer.service.common.excel.ExcelData;
import  com.yihu.customer.entity.*;
import  com.yihu.customer.repository.HospitalDao;
import  com.yihu.customer.repository.WlyyRoleDao;
import  com.yihu.customer.repository.WlyyhospitalspecialistDao;
import com.yihu.customer.util.ReadExcelUtil;
import com.yihu.customer.util.query.BaseJpaService;
import jxl.Sheet;
import jxl.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.NumberUtils;
import org.springframework.util.StringUtils;
import org.springside.modules.persistence.DynamicSpecifications;
import org.springside.modules.persistence.SearchFilter;

import java.util.*;

/**
 * Created by yww on 2016/9/27.
 */
@Service
public class HosService extends BaseJpaService<Hospital, HospitalDao> {
    @Autowired
    private HospitalDao hospitalDao;

    @Autowired
    private HosDoctorService doctorService;

    @Autowired
    private DistrictService districtService;

    @Autowired
    private WlyyRoleDao wlyyRoleDao;

    @Autowired
    private WlyyhospitalspecialistDao wlyyhospitalspeciaDao;
    //新增
    @Transactional
    public boolean createHospital(Hospital hospital) {
        hospital.setDel("1");
        hospitalDao.save(hospital);
        String name="";
        List<Wlyyhospitalspecia> li=new ArrayList<>();
        wlyyhospitalspeciaDao.deleteName(hospital.getCode(), hospital.getName());
        for(Wlyyhospitalspecia cia: hospital.getSpecialist()){
            List<Wlyyhospitalspecia> specia=wlyyhospitalspeciaDao.findByName(hospital.getCode(),hospital.getName());
            name=name +cia.getName()+",";
            cia.setHospital_name(hospital.getName());
            cia.setHospital(hospital.getCode());
            cia.setCreate_time(new Date());
            wlyyhospitalspeciaDao.save(cia);
        }
        return true;
    }

    //修改
    @Transactional
    public boolean updateHospital(Hospital hospital) {
        Hospital hospitalNew = hospitalDao.findById(hospital.getId()).get();
        hospitalNew.setCode(hospital.getCode());
        hospitalNew.setName(hospital.getName());
        hospitalNew.setLevel(hospital.getLevel());
        //TODO 照片上传
        hospitalNew.setPhoto(hospital.getPhoto());
        //街道编码和中心/站点码根据code获取

        String code = hospital.getCode();
        String roadCode = code.substring(0, code.length() - 2);
        String centerSite = code.substring(code.length() - 2, code.length());
        hospitalNew.setRoadCode(roadCode);
        hospitalNew.setCenterSite(centerSite);

        hospitalNew.setProvince(hospital.getProvince());
        hospitalNew.setProvinceName(hospital.getProvinceName());
        hospitalNew.setCity(hospital.getCity());
        hospitalNew.setCityName(hospital.getCityName());
        hospitalNew.setTown(hospital.getTown());
        hospitalNew.setTownName(hospital.getTownName());
        hospitalNew.setAddress(hospital.getAddress());
        hospitalNew.setIntro(hospital.getIntro());
        hospitalDao.save(hospital);
        return true;

    }

    //修改状态（正常/删除）
    @Transactional
    public boolean updateDel(Long id, String del) {
        int num = hospitalDao.updateDel(del, id);
        return num > 0 ? true : false;
    }

    //根据id删除
    @Transactional
    public boolean deleteHospital(long id) {
        hospitalDao.deleteById(id);
        return true;
    }

    //根据id获取单个
    public Hospital getHospital(long id) {
        Hospital hospital = hospitalDao.findById(id).get();
        List<Wlyyhospitalspecia> specia=wlyyhospitalspeciaDao.findByName(hospital.getCode(),hospital.getName());
        hospital.setSpecialist(specia);
        return hospital;
    }

    public List<Hospital> getHospitalByType(Integer type,String code){
        switch (type){
            case 1:return hospitalDao.findByCity(code);
            case 2:return hospitalDao.findByTownCode(code);
            default:return new ArrayList<>();
        }
    }

    //获取列表(不分页)
    public List<Hospital> searchHospitalNoPage(String name, String level) {
        Map<String, SearchFilter> filters = new HashMap<String, SearchFilter>();
        if (!StringUtils.isEmpty(name)) {
            filters.put("name", new SearchFilter("name", SearchFilter.Operator.LIKE, name));
        }
        if (org.apache.commons.lang3.StringUtils.isNotBlank(level)){
            filters.put("level", new SearchFilter("level", SearchFilter.Operator.EQ, Integer.parseInt(level)));
            if ("2".equals(level)){
                filters.put("centerSite", new SearchFilter("centerSite", SearchFilter.Operator.EQ, "00"));
            }
        }
        Specification<Hospital> spec = DynamicSpecifications.bySearchFilter(
                filters.values(), Hospital.class);
        List<Hospital> hospitalList = hospitalDao.findAll(spec);
        return hospitalList;

    }

    //获取列表(分页)
    public Page<Hospital> searchHospitals(String name, String doctorName, String doctorMobile, int page, int pageSize) throws Exception {
        if (page <= 0) {
            page = 1;
        }
        if (pageSize <= 0) {
            pageSize = 15;
        }
        // 分页信息
        PageRequest pageRequest = PageRequest.of(page - 1, pageSize);
        StringBuffer filters = new StringBuffer();
        if (!StringUtils.isEmpty(name)) {
            filters.append("name?" + name + ";");
        }
        StringBuffer doctorFilters = new StringBuffer();
        if (!StringUtils.isEmpty(doctorName)) {
            doctorFilters.append("name?" + doctorName + ";");
        }
        if (!StringUtils.isEmpty(doctorMobile)) {
            doctorFilters.append("mobile?" + doctorMobile + ";");
        }
        String hosCodes = "";
        if (doctorFilters.length() > 0) {
            List<Doctor> doctors = doctorService.search(doctorFilters.toString());
            for (Doctor doc : doctors) {
                hosCodes += "," + doc.getHospital();
            }
        }
        if (hosCodes.length() > 0) {
            hosCodes = hosCodes.substring(1);
            filters.append("code=" + hosCodes);
        }
        List<Hospital> list = search("code,name,level", filters.toString(), "+code,+name", page, pageSize);
        List<Hospital> lists=new ArrayList<>();
        for(Hospital ital:list){
            List<Wlyyhospitalspecia> specia=wlyyhospitalspeciaDao.findByName(ital.getCode(),ital.getName());
            ital.setSpecialist(specia);
            lists.add(ital);
        }
        long count = 0;
        if (lists.size() > 0) {
            count = getCount(filters.toString());
        }
        return new PageImpl<Hospital>(lists, pageRequest, count);

    }

    //医院名字是否存在
    public String isNameExist(String name) {
        Hospital hospital = hospitalDao.findByName(name);
        return hospital == null ? "false" : "true";
    }

    //医院名字是否存在
    public String isCodeExist(String code) {
        Hospital hospital = hospitalDao.findByCode(code);
        return hospital == null ? "false" : "true";
    }

    public boolean importData(Workbook workbook) {
        Sheet[] sheets = workbook.getSheets();
        Sheet sheet = sheets[0];
        int rows = ReadExcelUtil.getRightRows(sheet);
        for (int row = 1; row < rows; row++) {  //索引从0开始，第一行为标题
            Hospital hospital = new Hospital();
            Map<Integer, ExcelData> mapping = mapping(hospital);
            int finalRow = row;
            mapping.forEach((index, excelData) -> {
                String value = sheet.getCell(index, finalRow).getContents().trim();
                excelData.transform(value);
            });

            //Additional Handel
            String hospitalExist = this.isNameExist(hospital.getName());
            if (hospitalExist.equals("true")) {//机构存在,但是未判断wlyyRole是否存在
                hospital = hospitalDao.findByName(hospital.getName());
                WlyyRole wlyyRole = new WlyyRole();
                WlyyRole wlyyRole1 = wlyyRoleDao.findByName(hospital.getName());
                if(wlyyRole1==null){//不存在则保存
                    wlyyRole.setName(hospital.getName());
                    wlyyRole.setCode(hospital.getCode());
                    wlyyRole.setCzrq(new Date());
                    wlyyRoleDao.save(wlyyRole);
                }
                continue;
            }
            //机构不存在,先保存机构,在判断wlyyRole是否存在
            hospital.setDel("1");
            hospital = hospitalDao.save(hospital);
            WlyyRole wlyyRole = new WlyyRole();
            WlyyRole wlyyRole1 = wlyyRoleDao.findByName(hospital.getName());
            if(wlyyRole1==null){//不存在则保存
                wlyyRole.setName(hospital.getName());
                wlyyRole.setCode(hospital.getCode());
                wlyyRole.setCzrq(new Date());
                wlyyRoleDao.save(wlyyRole);
            }
        }
        return true;
    }

    private Map<Integer, ExcelData> mapping(Hospital hospital) {
        Map<Integer, ExcelData> dataMap = new HashMap<>();
        //机构名称
        dataMap.put(1, new ExcelData() {
            @Override
            public void transform(String data) {
                hospital.setName(data);
            }
        });
        //组织机构代码
        dataMap.put(2, new ExcelData() {
            @Override
            public void transform(String data) {
//                hospital.setName(data);
            }
        });
        //机构类型
        dataMap.put(3, new ExcelData() {
            @Override
            public void transform(String data) {
                Map<String, Integer> level = new HashMap<>();
                level.put("医院",1);
                level.put("社区服务中心", 2);
                level.put("行政机构", 3);
                if(org.apache.commons.lang3.StringUtils.isNotBlank(data)){
                    hospital.setLevel(level.get(data));
                }
            }
        });
        //是否为服务站
        dataMap.put(4, new ExcelData() {
            @Override
            public void transform(String data) {
                Map<String, String> centerSite = new HashMap<>();
                centerSite.put("是", "01");
                centerSite.put("否", "00");
                hospital.setCenterSite(centerSite.get(data));
            }
        });
        //所在省
        dataMap.put(6, new ExcelData() {
            @Override
            public void transform(String data) {
                hospital.setProvinceName(data);

                Province province = districtService.findProvinceByName(data);
                if (province != null) {
                    hospital.setProvince(province.getCode());
                }
            }
        });
        //所在市
        dataMap.put(7, new ExcelData() {
            @Override
            public void transform(String data) {
                hospital.setCityName(data);

                City city = districtService.findCityByName(data);
                if (city != null) {
                    hospital.setCity(city.getCode());
                }
            }
        });
        //所在区
        dataMap.put(8, new ExcelData() {
            @Override
            public void transform(String data) {
                hospital.setTownName(data);
                String city = hospital.getCity();
                Town town = districtService.finTownByNameAndCity(data,city);
                if (town != null) {
                    hospital.setTown(town.getCode());
                }

            }
        });
        //所在街道/乡镇
        dataMap.put(9, new ExcelData() {
            @Override
            public void transform(String data) {
                Street street = districtService.findStreetByName(data);
                if (street != null) {
                    hospital.setRoadCode(street.getCode());
                }
            }
        });
        //详细地址
        dataMap.put(10, new ExcelData() {
            @Override
            public void transform(String data) {
                hospital.setAddress(data);
            }
        });
        //机构简介
        dataMap.put(11, new ExcelData() {
            @Override
            public void transform(String data) {
                hospital.setIntro(data);
            }
        });

        hospital.setCode(UUID.randomUUID().toString());
        return dataMap;
    }

    private String genCode(String streetCode) {
        List<Hospital> hospitals = hospitalDao.findByCodePrefix(streetCode);

        if (hospitals == null || hospitals.size() == 0) {
            return streetCode + "00";
        }

        String code = hospitals.get(0).getCode();
        String serail = code.substring(streetCode.length(), streetCode.length() + 2);
        Integer searilNo = NumberUtils.parseNumber(serail, Integer.class);

        return streetCode + String.format("%1$02d", searilNo + 1);
    }

}
