package com.yihu.customer.service.manager.patient;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;

import com.yihu.customer.service.BaseService;
import com.yihu.customer.util.DateUtil;
import  com.yihu.customer.entity.*;
import  com.yihu.customer.entity.copd.CopdPatient;
import  com.yihu.customer.entity.organization.HospitalMapping;
import  com.yihu.customer.repository.DoctorDao;
import  com.yihu.customer.repository.FollowupDao;
import  com.yihu.customer.repository.HospitalDao;
import  com.yihu.customer.repository.PatientDao;
import  com.yihu.customer.repository.copd.CopdPatientDao;
import  com.yihu.customer.repository.copd.CopdPatientGroupDao;
import  com.yihu.customer.repository.organization.HospitalMappingDao;
import  com.yihu.customer.repository.wechat.DoctorMappingDao;
import com.yihu.customer.util.HttpManageClientUtil;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.util.*;


/**
 * Created by Bing on 2022/1/16.
 */
@Service
public class CopdPatientService extends BaseService {

    @Autowired
    private CopdPatientDao copdPatientDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private HospitalMappingDao hospitalMappingDao;
    @Autowired
    private DoctorMappingDao doctorMappingDao;
    @Autowired
    private FollowupDao followupDao;
    @Autowired
    private PatientDao patientDao;
    @Value("${server.server_url}")
    private String wlyyUrl;//
    @Autowired
    private HttpManageClientUtil httpManageClientUtil;
    @Autowired
    private HospitalDao hospitalDao;
    @Autowired
    private DoctorDao doctorDao;
    @Autowired
    private CopdPatientGroupDao copdPatientGroupDao;


    /**
     * 获取慢阻肺居民列表
     * @param name
     * @param idcard
     * @param mobile
     * @param hospitalName
     * @param doctorName
     * @param page
     * @param rows
     * @return
     */
    public Page<CopdPatient> getPatientList(String name,String idcard,String mobile,String hospitalName,String doctorName,Integer page,Integer rows){
        page=page>0?page-1:0;
        rows = rows<0?0:rows;
        String sql = " select cp.*,p.ssc from wlyy_copd_patient cp inner join wlyy_patient p on p.code= cp.patient where 1=1 ";
        String sqlCount = " select count(cp.id) from wlyy_copd_patient cp inner join wlyy_patient p on p.code= cp.patient where 1=1 ";
        if (StringUtils.isNotBlank(name)){
            sql += " and cp.name like '%"+name+"%' ";
            sqlCount += " and cp.name like '%"+name+"%' ";
        }
        if (StringUtils.isNotBlank(idcard)){
            sql += " and cp.idcard = '"+idcard+"' ";
            sqlCount += " and cp.idcard = '"+idcard+"' ";
        }
        if (StringUtils.isNotBlank(mobile)){
            sql += " and cp.phone = '"+mobile+"' ";
            sqlCount += " and cp.phone = '"+mobile+"' ";
        }
        if (StringUtils.isNotBlank(hospitalName)){
            sql += " and cp.hospital_name like '%"+hospitalName+"%' ";
            sqlCount += " and cp.hospital_name like '%"+hospitalName+"%' ";
        }
        if (StringUtils.isNotBlank(doctorName)){
            sql += " and cp.doctor_name like '%"+doctorName+"%' ";
            sqlCount += " and cp.doctor_name like '%"+doctorName+"%' ";
        }
        sql +=" order by cp.id asc limit "+(page*rows)+","+rows;
        List<CopdPatient> list = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(CopdPatient.class));
        Map<String,String> statusNameMap = new HashMap<>();
        statusNameMap.put("0","未分组");//0 未分组 1 随访观察组(旧) 2 肺康复锻炼组(旧)  3 肺康复锻炼组 4 常规治疗组 5待分配
        statusNameMap.put("1","随访观察组(旧)");
        statusNameMap.put("2","肺康复锻炼组(旧)");
        statusNameMap.put("3","肺康复锻炼组");
        statusNameMap.put("4","常规治疗组");
        statusNameMap.put("5","待分配");
        for (CopdPatient tmp:list){
            String statusName="";
            switch (tmp.getStatus()){
                case "0": statusName="未管理";break;
                case "1": statusName="已管理";break;
                case "2": statusName="失访";break;
                case "4": statusName="不纳入管理";break;
                case "8": statusName="死亡";break;
            }
            tmp.setStatusName(statusName);
            tmp.setGroupScreeningStatusName(statusNameMap.get(tmp.getGroupScreeningStatus()));
        }
        Long count = jdbcTemplate.queryForObject(sqlCount,Long.class);
        PageRequest pageRequest = PageRequest.of(page , rows);

        return new PageImpl(list, pageRequest, count);
    }

    public CopdPatient getCopdPatientInfo(String patient){
        CopdPatient copdPatient = copdPatientDao.findByPatient(patient);
        Patient patientDO = patientDao.findByCode(patient);
        copdPatient.setSsc(patientDO.getSsc());
        if (null==copdPatient){

        }else {
            String statusName="";
            switch (copdPatient.getStatus()){
                case "0": statusName="未管理";break;
                case "1": statusName="已管理";break;
                case "2": statusName="失访";break;
                case "4": statusName="不纳入管理";break;
                case "8": statusName="死亡";break;
            }
            copdPatient.setStatusName(statusName);
            copdPatient.setGroupScreeningStatusName(null==copdPatient.getGroupScreeningStatus()||"0".equals(copdPatient.getGroupScreeningStatus())?"未分组":"1".equals(copdPatient.getGroupScreeningStatus())?"随访观察组":"肺康复锻炼组");
        }
        return copdPatient;
    }

    /**
     * 导出列表
     * @param name
     * @param idcard
     * @param mobile
     * @param hospitalName
     * @param doctorName
     * @return
     */
    public List<Map<String,Object>> exportList(String name,String idcard,String mobile,String hospitalName,String doctorName){
        List<Map<String,Object>> list = new ArrayList<>();
        String sql = " select cp.name,cp.sex_name,cp.idcard,cp.phone,p.ssc,cp.hospital_name,cp.doctor_name,group_screening_status,cp.status from wlyy_copd_patient cp inner join wlyy_patient p on p.code= cp.patient where 1=1 ";
        if (StringUtils.isNotBlank(name)){
            sql += " and cp.name like '%"+name+"%' ";
        }
        if (StringUtils.isNotBlank(idcard)){
            sql += " and cp.idcard = '"+idcard+"' ";
        }
        if (StringUtils.isNotBlank(mobile)){
            sql += " and cp.phone = '"+mobile+"' ";
        }
        if (StringUtils.isNotBlank(hospitalName)){
            sql += " and cp.hospital_name like '%"+hospitalName+"%' ";
        }
        if (StringUtils.isNotBlank(doctorName)){
            sql += " and cp.doctor_name like '%"+doctorName+"%' ";
        }
        sql += " order by cp.id asc";
        list = jdbcTemplate.queryForList(sql);
        return list;
    }

    /**
     * 保存慢阻肺居民修改
     * @param json
     * @return
     * @throws IOException
     */
    @Transactional(rollbackFor = Exception.class)
    public String updateCopdPatient(String json) throws Exception {
        CopdPatient copdPatientTmp = objectMapper.readValue(json,CopdPatient.class);
        CopdPatient copdPatient = copdPatientDao.findByPatient(copdPatientTmp.getPatient());
        String sql =" select count(id) from wlyy_copd_patient_comanagement where patient='"+copdPatient.getPatient()+"' ";
        Integer count = jdbcTemplate.queryForObject(sql,Integer.class);
        if (count>0){
            throw new Exception("该患者为共管患者，不支持修改医生");
        }
        BeanUtils.copyProperties(copdPatientTmp,copdPatient,getNullPropertyNames(copdPatientTmp));
        HospitalMapping hospitalMapping = hospitalMappingDao.findByCode(copdPatient.getHospital());
        if (null!=hospitalMapping){
            copdPatient.setJwOrgCode(hospitalMapping.getMappingId());
            copdPatient.setJwOrgName(hospitalMapping.getName());
        }
        DoctorMapping doctorMapping = doctorMappingDao.findByJwDoctorHospitalAndDoctorCode(copdPatient.getJwOrgCode(),copdPatient.getDoctor());
        if (null!=doctorMapping){
            copdPatient.setJwDoctorCode(doctorMapping.getJwDoctor());
            copdPatient.setJwDoctorName(doctorMapping.getJwDoctorName());
        }
        String code=null;
        sql ="";
        Integer conut=1;
        switch (copdPatient.getGroupScreeningStatus()) {
            case "1":
                code = "GCZ" + code;
                sql = "select COUNT(id) from wlyy_copd_patient WHERE group_screening_code =";
                conut = jdbcTemplate.queryForObject(sql+"'"+code+"'", Integer.class);
                while (conut != 0) {
                    code ="GCZ"+ String.valueOf(new Random().nextInt(999999));
                    conut = jdbcTemplate.queryForObject(sql+"'"+code+"'", Integer.class);
                }
                break;
            case "2":
                code = "DLZ" + code;
                sql = "select COUNT(id) from wlyy_copd_patient WHERE group_screening_code =";
                conut = jdbcTemplate.queryForObject(sql+"'"+code+"'", Integer.class);
                while (conut != 0) {
                    code ="DLZ"+String.valueOf(new Random().nextInt(999999));
                    conut = jdbcTemplate.queryForObject(sql+"'"+code+"'", Integer.class);
                }
                break;
        }
        copdPatient.setGroupScreeningCode(code);
        Patient patient = patientDao.findByIdcard(copdPatient.getIdcard());
        boolean updatePatient=false;
        if (StringUtils.isNotBlank(copdPatient.getSsc())){
            patient.setSsc(copdPatient.getSsc());
            updatePatient = true;
        }
        if (StringUtils.isNotBlank(copdPatient.getName())){
            patient.setName(copdPatient.getName());
            updatePatient = true;
        }
        if (updatePatient){
            patientDao.save(patient);
        }
        copdPatientDao.save(copdPatient);
        followupDao.updateCoPDPatientPlan(copdPatient.getPatient(),copdPatient.getDoctor(),copdPatient.getDoctorName(),copdPatient.getHospital(),copdPatient.getHospitalName());
        return "success";
    }

    /**
     * 慢阻肺居民还原
     * 分组不还原-编号不还原
     * @param patient
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public String initCopdPatient(String patient){
        CopdPatient copdPatient = copdPatientDao.findByPatient(patient);
        copdPatient.setStatus("0");
        copdPatient.setInclusionConditions(null);
        copdPatient.setExclusionConditions(null);
        copdPatient.setSecondInclusionConditions(null);
        copdPatient.setSecondExclusionConditions(null);
//        copdPatient.setGroupScreeningStatus("0");
        copdPatient.setFollowupTwoStatus(null);
        copdPatient.setGroupWatch(null);
        copdPatient.setWatchSn(null);
        copdPatientDao.save(copdPatient);
        followupDao.delCopdPatientPlan(patient);
//        CopdPatientGroup copdPatientGroup = copdPatientGroupDao.findByPatient(patient);
//        if(copdPatientGroup!=null){
//            copdPatientGroup.setPatient(null);
//            copdPatientGroupDao.save(copdPatientGroup);
//        }
        return "success";
    }

    /**
     * 慢阻肺居民二次排纳还原
     * 二次排纳失访后续随访计划都会被修改为失访。
     */
    @Transactional(rollbackFor = Exception.class)
    public String initSecondScreening(String patient){
        CopdPatient copdPatient = copdPatientDao.findByPatient(patient);
        copdPatient.setStatus("1");
        copdPatient.setSecondInclusionConditions(null);
        copdPatient.setSecondExclusionConditions(null);
        copdPatient.setFollowupTwoStatus(null);
        copdPatientDao.save(copdPatient);
        String updStatusSql = "update wlyy_followup set status = 2 where type=4 and visit_count >= 2 and patient_code ='" + copdPatient.getPatient() + "'";
        jdbcTemplate.execute(updStatusSql);
        return "success";
    }

    /**
     * 慢阻肺居民删除
     * @param patient
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public String delCopdPatient(String patient){
        copdPatientDao.deleteCopdPatient(patient);
        followupDao.delCopdPatientPlan(patient);
        return "success";
    }

    /**
     * 添加慢阻肺居民
     * @param idcard
     * @param hospital
     * @param doctor
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public CopdPatient addCopdPatient(String idcard,String hospital,String doctor)throws Exception{
        Patient patient = patientDao.findByIdcard(idcard);
        CopdPatient copdPatient = null;
        if (null==patient){
            Map<String, Object> params = new HashMap<>();
            params.put("idcard",idcard);
            String response = httpManageClientUtil.post(wlyyUrl+"third/syncDisPatient/common/addPaitentByIdcard",params);
            com.alibaba.fastjson.JSONObject jsonObject = com.alibaba.fastjson.JSONObject.parseObject(response);
            if (jsonObject != null){
                if (jsonObject.getInteger("status") == 200) {
                    patient = patientDao.findByIdcard(idcard);
                    if (null != patient) {
                        addPatient(patient, hospital, doctor);
                    }
                }
            }
        }else {
            addPatient(patient,hospital,doctor);
        }
        return copdPatient;
    }

    public CopdPatient addPatient(Patient patient,String hospital,String doctor)throws Exception{
        CopdPatient copdPatient= copdPatientDao.findByPatient(patient.getCode());
        if (copdPatient!=null){
            if (!"0".equals(copdPatient.getStatus())){
                throw new Exception("该患者已在"+copdPatient.getDoctorName()+"医生管理，无法添加");
            }
        }else {
            copdPatient = new CopdPatient();
        }
        //慢阻肺居民信息赋值
        copdPatient.setInclusionConditions(null);
        copdPatient.setExclusionConditions(null);
        copdPatient.setSecondInclusionConditions(null);
        copdPatient.setSecondExclusionConditions(null);
        copdPatient.setGroupScreeningStatus("0");
        copdPatient.setGroupScreeningCode(null);
        copdPatient.setFollowupTwoStatus(null);
        copdPatient.setPatient(patient.getCode());
        copdPatient.setStatus("0");
        Hospital hospitalDO = hospitalDao.findByCode(hospital);
        if (null!=hospitalDO){
            copdPatient.setHospital(hospital);
            copdPatient.setHospitalName(hospitalDO.getName());
        }else {
            throw new Exception("机构不存在");
        }
        Doctor doctorDo = doctorDao.findByCode(doctor);
        if (null!=doctorDo){
            copdPatient.setDoctor(doctor);
            copdPatient.setDoctorName(doctorDo.getName());
        }else {
            throw new Exception("医生不存在");
        }
        copdPatient.setSort(3);
        copdPatient.setCreateTime(new Date());
        copdPatient.setUpdateTime(new Date());
        copdPatient.setDel(1);
        copdPatient.setName(patient.getName());
        copdPatient.setSexName(patient.getSex()+"");
        copdPatient.setBirthday(DateUtil.dateToStr(patient.getBirthday(),DateUtil.YYYY_MM_DD));
        copdPatient.setIdcard(patient.getIdcard());
        copdPatient.setLiveAddress(patient.getAddress());
        copdPatient.setPhone(patient.getMobile());
        HospitalMapping hospitalMapping = hospitalMappingDao.findByCode(copdPatient.getHospital());
        if (null!=hospitalMapping){
            copdPatient.setJwOrgCode(hospitalMapping.getMappingId());
            copdPatient.setJwOrgName(hospitalMapping.getName());
        }
        DoctorMapping doctorMapping = doctorMappingDao.findByJwDoctorHospitalAndDoctorCode(copdPatient.getJwOrgCode(),copdPatient.getDoctor());
        if (null!=doctorMapping){
            copdPatient.setJwDoctorCode(doctorMapping.getJwDoctor());
            copdPatient.setJwDoctorName(doctorMapping.getJwDoctorName());
        }
        copdPatientDao.save(copdPatient);
        return copdPatient;
    }

    /**
     * 获取慢阻肺居民计划列表
     * @param patient
     * @return
     */
    public List<Map<String,Object>> getFollowupList(String patient){
        String sql = " select id, DATE_FORMAT(followup_plan_date,'%Y-%m-%d') followup_plan_date,`status`,visit_count from wlyy_followup where patient_code='"+patient+"' " +
                " and type=4 ORDER BY visit_count asc " ;
        List<Map<String,Object>> list = jdbcTemplate.queryForList(sql);
        return list;
    }

    @Transactional(rollbackFor = Exception.class)
    public String modifyFollow(String json)throws Exception{
        JSONArray array = JSONArray.parseArray(json);
        List<Followup> followups = new ArrayList<>();
        for (int i=0;i<array.size();i++){
            JSONObject tmp = array.getJSONObject(i);
            String id = tmp.getString("id");
            String followup_plan_date = tmp.getString("followup_plan_date");
            Date followupPlanDate = DateUtil.strToDateLong(followup_plan_date+" 12:00:00");

            Followup followup = followupDao.findById(Long.parseLong(id));
            if (null != followup){
                String status = followup.getStatus();
                String sqlDate = DateUtil.dateToStrShort(followup.getFollowupPlanDate());
                String sqlVisitCount = followup.getVisitCount();
                if (sqlDate.equals(followup_plan_date)){//时间相等不做任何处理
                    continue;
                }
                if ("1".equals(status)){
                    continue;
                }else if ("5".equals(status)){//该计划未失访
                    Date date = DateUtil.getNowDateShort();
                    Date prDate = DateUtil.getPreDays(date,-7);
                    Date nextDate = DateUtil.getPreDays(date,7);
                    if (!DateUtil.isInArea(followupPlanDate,prDate,nextDate)){
                        throw new Exception("居民已失访的第"+sqlVisitCount+"次随访时间不在今天前后七天内，无法修改");
                    }

                    followup.setFollowupPlanDate(followupPlanDate);
                    followup.setStatus("2");
                    followups.add(followup);
                }else {
                    followup.setFollowupPlanDate(followupPlanDate);
                    followups.add(followup);
                }
            }
        }
        if (followups.size()>0){
            followupDao.saveAll(followups);
        }
        return "success";
    }

}
