/*
package com.yihu.jw.service.manager.platformInformation;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.es.util.ElasticsearchUtil;
import com.yihu.jw.entity.SignDict;
import com.yihu.jw.entity.platformInformation.*;
import com.yihu.jw.repository.SignDictDao;
import com.yihu.jw.repository.platformInformation.*;
import com.yihu.jw.service.manager.message.MessageService;
import com.yihu.jw.util.*;
import com.yihu.jw.util.fastdfs.FastDFSUtil;
import com.yihu.jw.util.query.BaseJpaService;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
@Transactional
public class PlatformInformationService extends BaseJpaService<PlatformInformationDO, PlatformInformationDao> {
    @Value("${es.type.PlatformInformationLog}")
    private String esType;
    @Value("${es.index.PlatformInformationLog}")
    private String esIndex;
    @Value("${wlyy.url}")
    private String wlyyUrl;
    @Value("${jkEdu.web.articleBaseUrl}")
    private String articleBaseUrl;
    @Autowired
    private DoctorAssistantUtil doctorAssistantUtil;
    @Autowired
    private MessageService messageService;

    @Autowired
    private PlatformInformationDao platformInformationDao;
    @Autowired
    private PlatformInformationLabelRelationDao labelRelationDao;
    @Autowired
    private PlatformInformationLabelDao labelDao;
    @Autowired
    private PlatformInformationVideoClasssifyDao platformInformationVideoClasssifyDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @ApiParam
    FastDFSUtil fastDFSUtil;
    @Autowired
    private ElasticsearchUtil elasticsearchUtil;
    @Autowired
    private SignDictDao signDictDao;
    @Autowired
    private FastDFSUtil fastDFSHelper;
    @Autowired
    private PlatformInformationOperationLogDao logDao;
    @Autowired
    private HttpClientUtil httpClientUtil;
    @Autowired
    private PlatformInformationVideoClasssifyDao classsifyDao;

    public Page<PlatformInformationDO> searchList(Integer sendingObjects, String title, Integer type, Integer isNotice, Integer sendType, String startDate, String endDate, Integer page, Integer pageSize) {
        List<PlatformInformationDO> platformInformationList = new ArrayList<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        if (page == null) {
            page = 1;
        }
        if (pageSize == null) {
            pageSize = 15;
        }
        PageRequest pageRequest = PageRequest.of(page - 1, pageSize);
        String sql = "select * from wlyy_platform_information p where 1=1  ";
        String countSql = "select count(1) from wlyy_platform_information p where 1=1  ";
        StringBuilder filters = new StringBuilder();
        if (sendingObjects != null) {
            filters.append("and p.sending_objects = '" + sendingObjects + "' ");
        }else {
            filters.append("and p.sending_objects in (2,3,4) ");
        }
        if (!StringUtils.isEmpty(title)) {
            filters.append("and p.title like '%" + title + "%' ");
        }
        if (type != null) {
            filters.append("and p.type = '" + type + "' ");
        }
        if (isNotice != null) {
            filters.append("and p.is_notice = '" + isNotice + "' ");
        }
        if (sendType != null) {
            filters.append("and p.send_type = '" + sendType + "' ");
        }
        if (!StringUtils.isEmpty(startDate)) {
            filters.append("and p.create_time > '" + startDate + "' ");
        }
        if (!StringUtils.isEmpty(endDate)) {
            endDate += " 23:59:59";
            filters.append("and p.create_time <= '" + endDate + "' ");
        }
        platformInformationList = jdbcTemplate.query(sql + filters.toString() + " order by p.create_time desc" + " limit " + (page - 1) * pageSize + "," + pageSize, new BeanPropertyRowMapper<>(PlatformInformationDO.class));
        if (platformInformationList.size() <= 0) {
            return new PageImpl<PlatformInformationDO>(new ArrayList<>(), pageRequest, 0);
        }
        Long count = jdbcTemplate.queryForObject(countSql + filters.toString(), Long.class);
        return new PageImpl<PlatformInformationDO>(platformInformationList, pageRequest, count);
    }

    public Map<String, Object> uploadImg(HttpServletRequest request) throws Exception {
        Map<String, Object> map = new HashMap<>();
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        MultipartFile multipartFile = multipartRequest.getFile("file");
        long size = multipartFile.getSize();
        if(size<=0){
            map.put("uploadStatus",1);//size小于0
            map.put("accessoryUrl",null);//
            return map;
        }
        String fileName = multipartFile.getOriginalFilename();
        String[] fs = fileName.split("\\.");
        String type = fs[1];
        //图片常见格式：bmp,jpg,png,tif,gif,pcx,tga,exif,fpx,svg,psd,cdr,pcd,dxf,ufo,eps,ai,raw,WMF,webp
        List img = new ArrayList(Arrays.asList("bmp", "jpg", "png", "tif", "gif", "pcx", "tga", "exif", "fpx", "svg", "psd", "cdr", "pcd", "dxf", "ufo", "eps", "ai", "raw", "WMF", "webp"));
        if (!img.contains(type)) {
            map.put("uploadStatus",2);//文件类型不对
            map.put("accessoryUrl",null);//
            return map;
        }
        String response = request(wlyyUrl + "/upload/chat", multipartFile, null);
        org.json.JSONObject rs = new org.json.JSONObject(response);
        Integer status = (Integer) rs.get("status");
        if (status == 200) {
            String url = rs.get("urls") + "";
            map.put("uploadStatus", 0);//文件类型正确
            map.put("accessory", url);//
            return map;
        }
        throw new Exception();
    }

    */
/**
     * 创建平台咨讯
     *
     * @param jsonData
     * @param token
     *//*

    public void create(String jsonData, String token, String patient) throws Exception {
        ObjectMapper objectMapper = new ObjectMapper();
        //设置时间格式
        objectMapper.setDateFormat(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"));
        JSONObject json = JSONObject.parseObject(jsonData);
        String labels = null;
        Integer sendObjects = json.getInteger("sendingObjects");
        //判断是否有标签条件
        if (json.get("label") != null) {
            labels = json.getString("label");
            json.remove("label");
            jsonData = json.toString();
        }
        //创建平台咨讯表
        PlatformInformationDO platformInformationDO = objectMapper.readValue(jsonData, PlatformInformationDO.class);
        String code = getCode();
        platformInformationDO.setCode(code);
        platformInformationDO.setProvince("350000");
        platformInformationDO.setProvinceName("福建省");
        platformInformationDO.setCity("350200");
        platformInformationDO.setCityName("厦门市");
        platformInformationDO.setAuthor("厦门i健康");
        platformInformationDO.setAuthorOrgName("厦门i健康");
        platformInformationDO.setSendStatus(0);
        platformInformationDO.setDel(1);
        platformInformationDO.setCreateTime(new Date());
        String qrCodeUrl = getQrcode("platformInformation_"+code, token);
        if(StringUtils.isNotBlank(qrCodeUrl)) {
            platformInformationDO.setQrCode(qrCodeUrl);
        }else {
            throw new Exception("二维码生成失败,请重新创建！");
        }
        platformInformationDO = platformInformationDao.save(platformInformationDO);

        //判断是否选了多个标签，如{"label":[[{"labelName":"低血糖"},{"labelCode":"2"}],{"labelName":"低血糖"},{"labelCode":"2"}]}
        if (labels != null) {
            List<PlatformInformationLabelRelationDO> list = new ArrayList<>();
            String[] labelCodes = new String[]{};
            if (labels.contains(",")) {
                labelCodes = labels.split(",");
            } else {
                labelCodes = new String[]{labels};
            }
            //创建平台咨讯标签关联表
            for (String label : labelCodes) {
                PlatformInformationLabelRelationDO labelRelationDO = new PlatformInformationLabelRelationDO();
                String labelName = null;
                if (sendObjects != 1) {
                    labelName = labelDao.findNameByCode(label);
                } else {
                    labelName = signDictDao.findNameByCodeAndYear(label, String.valueOf(DateUtil.getSignYear()));
                }
                labelRelationDO.setCode(getCode());
                labelRelationDO.setInformationCode(platformInformationDO.getCode());
                labelRelationDO.setInformationTitle(platformInformationDO.getTitle());
                labelRelationDO.setLabelCode(label);
                labelRelationDO.setLabelName(labelName);
                labelRelationDO.setDel(1);
                labelRelationDO.setCreateTime(new Date());
                list.add(labelRelationDO);
            }
            labelRelationDao.saveAll(list);
        }

        // 将操作记录到日志表中
        PlatformInformationOperationLogDO operationLogDO = new PlatformInformationOperationLogDO();
        operationLogDO.setInformationId(platformInformationDO.getId());
        operationLogDO.setOperator(patient);
        operationLogDO.setName("创建平台资讯");
        operationLogDO.setOperatorTime(new Date());
        logDao.save(operationLogDO);
    }

    */
/**
     * 更新平台咨讯表格
     *
     * @param jsonData
     *//*

    public void update(String jsonData, String patient) throws IOException {
        ObjectMapper objectMapper = new ObjectMapper();
        //设置时间格式
        objectMapper.setDateFormat(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"));
        JSONObject json = JSONObject.parseObject(jsonData);
        String labels = null;
        Integer sendObjects = json.getInteger("sendingObjects");
        //判断是否有标签条件
        if (json.get("label") != null) {
            labels = json.getString("label");
            json.remove("label");
            jsonData = json.toString();
        }
        PlatformInformationDO platformInformationDO = objectMapper.readValue(jsonData, PlatformInformationDO.class);
        //根据id查出原来数据
        PlatformInformationDO result = platformInformationDao.findById(json.getLong("id")).get();
        //更新相关字段
        result.setTitle(platformInformationDO.getTitle());
        result.setContent(platformInformationDO.getContent());
        result.setType(platformInformationDO.getType());
        result.setOrganizer(platformInformationDO.getOrganizer());
        result.setImage(platformInformationDO.getImage());
        result.setTown(platformInformationDO.getTown());
        result.setTownName(platformInformationDO.getTownName());
        result.setHospital(platformInformationDO.getHospital());
        result.setHospitalName(platformInformationDO.getHospitalName());
        result.setSendingObjects(platformInformationDO.getSendingObjects());
        result.setIsNotice(platformInformationDO.getIsNotice());
        result.setSendType(platformInformationDO.getSendType());
        result.setSendTime(platformInformationDO.getSendTime());
        result.setVideoTypeClassify(platformInformationDO.getVideoTypeClassify());
        result.setClassifyName(platformInformationDO.getClassifyName());
        result.setUpdateTime(new Date());
//        result.setAuthor(platformInformationDO.getAuthor());
//        result.setAuthorOrgName(platformInformationDO.getAuthorOrgName());
        platformInformationDao.save(result);

        //逻辑删除原有标签，
        labelRelationDao.updateDelByInformationCode(result.getCode());

        if (StringUtils.isNotBlank(labels)) {
            List<PlatformInformationLabelRelationDO> list = new ArrayList<>();
            String[] labelCodes = new String[]{};
            if (labels.contains(",")) {
                labelCodes = labels.split(",");
            } else {
                labelCodes = new String[]{labels};
            }
            //创建平台咨讯标签关联表
            for (String label : labelCodes) {
                PlatformInformationLabelRelationDO labelRelationDO = new PlatformInformationLabelRelationDO();
                String labelName = null;
                if (sendObjects != 1) {
                    labelName = labelDao.findNameByCode(label);
                } else {
                    labelName = signDictDao.findNameByCodeAndYear(label, String.valueOf(DateUtil.getSignYear()));
                }
                labelRelationDO.setCode(getCode());
                labelRelationDO.setInformationCode(result.getCode());
                labelRelationDO.setInformationTitle(result.getTitle());
                labelRelationDO.setLabelCode(label);
                labelRelationDO.setLabelName(labelName);
                labelRelationDO.setDel(1);
                labelRelationDO.setCreateTime(new Date());
                list.add(labelRelationDO);
            }
            labelRelationDao.saveAll(list);
        }
        // 将操作记录到日志表中
        PlatformInformationOperationLogDO operationLogDO = new PlatformInformationOperationLogDO();
        operationLogDO.setInformationId(platformInformationDO.getId());
        operationLogDO.setOperator(patient);
        operationLogDO.setName("编辑平台资讯");
        operationLogDO.setOperatorTime(new Date());
        logDao.save(operationLogDO);
    }

    */
/**
     * 根据id查看平台咨讯相关信息
     *
     * @param id
     * @return
     *//*

    public Map<String, Object> preview(Long id) {
        Map<String, Object> map = new HashMap<>();
        //根据id查出平台咨讯
        PlatformInformationDO platformInformationDO = platformInformationDao.findById(id).get();
        map.put("title", platformInformationDO.getTitle());
        map.put("content", platformInformationDO.getContent());
        //如果已发送，则查出浏览数、点赞数、转发数
        if (platformInformationDO.getSendStatus() == 1) {
            Long browses = findNumByInfomationIdAndType(id, 1);
            Long likes = findNumByInfomationIdAndType(id, 2);
            Long retweets = findNumByInfomationIdAndType(id, 3);
            map.put("browses", browses);
            map.put("likes", likes);
            map.put("retweets", retweets);
        }
        return map;
    }

    */
/**
     * 根据informationId和type查出相关数量
     *
     * @param informationId
     * @param type
     * @return
     *//*

    public Long findNumByInfomationIdAndType(Long informationId, Integer type) {
        StringBuffer sql = new StringBuffer("SELECT count(*) num FROM " + esType + " where informationId= " + informationId + " and type = " + type);
        Long num = elasticsearchUtil.excuteForLong(sql.toString(), esIndex, esType);
        return num;
    }

    */
/**
     * 手动发送时更新发送状态和时间
     *发送状态（0待发送、1已发送/上架，2下架，3撤回）
     * @param id
     *//*

    public void send(Long id,Integer sendStatus, String patient) {
        PlatformInformationDO one = platformInformationDao.findById(id).get();
        // 如果sendStatus=3 则是已撤回过的，需要清下缓存
        if (null != one && one.getSendStatus() == 3) {
            // 清除缓存
            HttpClientUtil.get(wlyyUrl + "/platformInfomation/cleanPreviewById?id=" + id, "UTF-8");
        }

        platformInformationDao.updateSendInformationById(id, new Date(),sendStatus);

        // 将操作记录到日志表中
        String name = one.getSendStatus() == 3 ? "重新发送" : "";
        Integer type = one.getType();
        if (StringUtils.isEmpty(name)) {
            if (type == 5) {
                name = sendStatus == 0 ? "下架" : "上架";
            } else {
                name = sendStatus == 0 ? "待发送" : "发送";
            }
        }
        PlatformInformationOperationLogDO operationLogDO = new PlatformInformationOperationLogDO();
        operationLogDO.setInformationId(id);
        operationLogDO.setOperator(patient);
        operationLogDO.setName(name);
        operationLogDO.setOperatorTime(new Date());
        logDao.save(operationLogDO);

        //批量发送要属于同一批次
        */
/*String batchNo = UUID.randomUUID().toString();
        //判断发送对象是居民还是医生
        if (platformInformationDO.getSendingObjects() == 1) {
            List<Map<String, Object>> sendPatientList = getSendPatient(platformInformationDO.getLabel(), platformInformationDO.getProvince(), platformInformationDO.getCity(),
                    platformInformationDO.getTown(),platformInformationDO.getHospital());
            for(Map<String, Object> map : sendPatientList){
                PlatformInformationPushRecordES es = new PlatformInformationPushRecordES();
                es.setBatchNo(batchNo);
                es.setType(platformInformationDO.getType());
                es.setReceiverCode(map.get("patient") + "");
                es.setReceiverName(map.get("name") + "");
                es.setInformationId(id);
            }
        }*//*

        try {
            //发送功能状态时，给医生助手发送模板消息
            if(null != one &&3==one.getType()){
            sendWeixinMessage(one);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    */
/**
     *分别给家庭医生和专科医生发送微信模板消息
     * 资讯发送对象（1居民、2家医、3专科医生、4家医和专科）
     * 医生类型：1专科医生，2全科医生，3健康管理师 status=1;del=1
     * @param platformInformationDO 待发送的模板消息
     *//*

    public void sendWeixinMessage( PlatformInformationDO platformInformationDO) throws Exception{
        try {
            String doctorSql ="";
            String town=platformInformationDO.getTown();
            String hospital=platformInformationDO.getHospital();
            if (2==platformInformationDO.getSendingObjects()) {
               doctorSql= "SELECT d.`code`,d.`name`, d.openid,d.cid ,d.hospital_name FROM wlyy_doctor d WHERE d.`status`=1 AND  d.del=1 AND d.level=1  and d.openid is not NULL ";
            } else if (3==platformInformationDO.getSendingObjects() ) {
                doctorSql="SELECT d.`code`,d.`name`, d.openid,d.cid ,d.hospital_name FROM wlyy_doctor d WHERE d.`status`=1 AND  d.del=1 AND d.level=2  and d.openid is not NULL ";
            }if (4==platformInformationDO.getSendingObjects() ) {
               doctorSql="SELECT d.`code`,d.`name`, d.openid,d.cid ,d.hospital_name FROM wlyy_doctor d WHERE d.`status`=1 AND  d.del=1 AND (d.level=1 or d.level=2)  and d.openid is not NULL ";
            }
            if(StringUtils.isNotBlank(town)){
                doctorSql=doctorSql+" AND d.town='"+town+"'";
            }
            if(StringUtils.isNotBlank(hospital)){
                doctorSql=doctorSql+" AND d.hospital='"+hospital+"'";
            }
            System.out.println("doctorSql==="+doctorSql);
            if(StringUtils.isNotBlank(doctorSql)){
//                doctorSql=doctorSql+" and d.hospital='ac02a9b87a3611e69f7c005056850d66'";
                //获取所有有openid的医生
                List<Map<String, Object>> doctorList = jdbcTemplate.queryForList(doctorSql);
                //给家庭医生发送模板消息
                sendWechatMessage(platformInformationDO, doctorList);
            }

            System.out.println("platformInformationSql===="+doctorSql);

        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    */
/**
     * 给医生发送平台资讯
     * @param platformInformationDO 平台资讯
     * @param doctorList 待发送的医生列表
     * @throws Exception
     *//*

    public void sendWechatMessage(PlatformInformationDO platformInformationDO, List<Map<String, Object>> doctorList) throws Exception{
        //获取当天
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date today = new Date();
        String todayString = simpleDateFormat.format(today);
        if (doctorList != null) {
            for (Map<String, Object> map : doctorList) {
                String doctor = String.valueOf(map.get("code"));
                String doctorName = String.valueOf(map.get("name"));
                String hospitalName=map.get("hospital_name")+"";
                System.out.println("doctorCode==="+doctor);
                //新增发送医生助手模板消息

                    if (org.apache.commons.lang.StringUtils.isNotBlank(map.get("openid") + "") && !(map.get("openid") + "").equals("null")) {
                        String keywords = "帮助手册" + "," +hospitalName + ","+doctorName;
                        String first = doctorName + " ，您好！ 厦门i健康帮助手册更新啦，《"+platformInformationDO.getTitle()+"》，为了您更好的使用快登录厦门i健康APP：我的—>“帮助手册”查看吧";
                        String remark = "";
                        doctorAssistantUtil.sendWXTemplate(15,map.get("openid") + "",map.get("cid") + "",first,remark,keywords);
                    }

            }
        }
    }



    public List<Map<String, Object>> getSendPatient(String serverType, String province, String city, String town, String hospital) {
        List<Map<String, Object>> list = new ArrayList<>();
        String sql = "SELECT s.patient, s.`name` from wlyy_patient p ";
        String whereSql = "where p.`status` = 1 ";
        if (StringUtils.isNotBlank(serverType)) {
            sql += "LEFT JOIN wlyy_sign_family s on s.patient = p.`code` ";
            whereSql += "and s.server_type = '" + serverType + "' ";
        }
        if (StringUtils.isNotBlank(hospital) || StringUtils.isNotBlank(town)) {
            sql += "LEFT JOIN wlyy_sign_family s on s.patient = p.`code` ";
            whereSql += "and s.status > 0 ";
            if (StringUtils.isNotBlank(hospital)) {
                whereSql += "and s.hospital = '" + hospital + "' ";
            }
            if (StringUtils.isNotBlank(town)) {
                whereSql += "and p.town = '" + town + "' ";
            }
            list = jdbcTemplate.queryForList(sql + whereSql);
        }
        return list;
    }

    */
/**
     * 获取居民标签
     *//*

    public List<SignDict> findPatientLabel() {
        int year = DateUtil.getSignYear();
        List<SignDict> list = signDictDao.findByYear(String.valueOf(year));
        return list;
    }

    */
/**
     * 获取咨讯标签
     *//*

    public List<PlatformInformationLabelDO> findInformationLabel() {
        return labelDao.findAllLabel();
    }

    public JSONObject findById(Long id) {
        JSONObject object = new JSONObject();
        PlatformInformationDO platformInformationDO = platformInformationDao.findById(id).get();
        if(StringUtils.isBlank(platformInformationDO.getClassifyName()) && StringUtils.isNotBlank(platformInformationDO.getVideoTypeClassify())){
            String classifyName = classsifyDao.findNameByCode(platformInformationDO.getVideoTypeClassify());
            platformInformationDO.setClassifyName(classifyName);
            platformInformationDao.save(platformInformationDO);
        }
        List<PlatformInformationLabelRelationDO> relationDOS = labelRelationDao.findByInformationCode(platformInformationDO.getCode());
        String labelCode = "";
        for(PlatformInformationLabelRelationDO relationDO : relationDOS){
            labelCode += relationDO.getLabelCode() + ",";
        }
        labelCode = labelCode.substring(0, labelCode.length()-1);
        object.put("platformInformationDO",platformInformationDO);
        object.put("labelCode", labelCode);
        return object;
    }

    public String getQrcode(String scene, String token)  {
        String token_url = "https://api.weixin.qq.com/cgi-bin/qrcode/create?access_token=" + token;
        String params = "{\"action_name\": \"QR_LIMIT_STR_SCENE\", \"action_info\": {\"scene\": {\"scene_str\": \"" + scene + "\"}}}";
        String result = HttpUtil.sendPost(token_url, params);
        System.out.println("result : " + result);
        try {
            if (!StringUtils.isEmpty(result)) {
                org.json.JSONObject json = new org.json.JSONObject(result);
                if (json.has("ticket")) {
                    // 请求输入流
                    InputStream inputStream = null;
                    // 下载二维码图片
                    URL urlGet = null;

                        urlGet = new URL("https://mp.weixin.qq.com/cgi-bin/showqrcode?ticket="
                                + URLEncoder.encode(json.get("ticket").toString(), "UTF-8"));
                    HttpURLConnection connection = (HttpURLConnection) urlGet.openConnection();
                    connection.connect();
                    //下载图片到内存
                    inputStream = connection.getInputStream();
                    //======start===
                    //fastDFS方法问题，直接将inputStream文件流上传文件服务器存数据缺失，
                    // 故将文件输出本地，在上传文件服务器，之后删除本地文件
                    String path = PlatformInformationService.class.getResource("/").getPath().replace("/WEB-INF/classes/", "")
                            + File.separator + "qrcode";

                    //判断目录是否存在，创建目录
                    File dir = new File(path);
                    if (!dir.exists()) {
                        dir.mkdir();
                    }
                    path += File.separator + scene + ".png";
                    File file = new File(path);
                    // 保存文件
                    FileOutputStream outputStream = new FileOutputStream(file);

                    byte[] data = new byte[1024];
                    int len = 0;

                    while ((len = inputStream.read(data)) != -1) {
                        outputStream.write(data, 0, len);
                    }

                    InputStream input = new FileInputStream(file);

                    //=======end=========
    //                ObjectNode objectNode = fastDFSHelper.upload(input, "png", "");
                    MultipartFile multipartFile = new MockMultipartFile(file.getName(), file.getName(), null, input);
                    String response = request(wlyyUrl + "/upload/chat", multipartFile, null);
                    System.out.println("response : " + response);

                    if (outputStream != null) {
                        outputStream.close();
                    }

                    if (inputStream != null) {
                        inputStream.close();
                    }

                    //流关闭，删除文件
                    if (file.exists() && file.isFile()) {
                        file.delete();
                    }
                    org.json.JSONObject rs = new org.json.JSONObject(response);
                    Integer status = (Integer) rs.get("status");
                    if (status == 200) {
                        return rs.get("urls") + "";
                    }
                    throw new Exception("二维码生成失败");
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }

    public String request(String remote_url, MultipartFile file, String type) {
        CloseableHttpClient httpClient = HttpClientBuilder.create().build();
        String result = "";
        try {
            String fileName = file.getOriginalFilename();
            HttpPost httpPost = new HttpPost(remote_url);
            MultipartEntityBuilder builder = MultipartEntityBuilder.create();
            builder.addBinaryBody("file", file.getInputStream(), ContentType.MULTIPART_FORM_DATA, fileName);// 文件流
            builder.addTextBody("filename", fileName);// 类似浏览器表单提交，对应input的name和value
            if (!org.springframework.util.StringUtils.isEmpty(type)) {
                builder.addTextBody("type", type); //发送类型
            }
            HttpEntity entity = builder.build();
            httpPost.setEntity(entity);
            HttpResponse response = httpClient.execute(httpPost);// 执行提交
            HttpEntity responseEntity = response.getEntity();
            if (responseEntity != null) {
                // 将响应内容转换为字符串
                result = EntityUtils.toString(responseEntity, Charset.forName("UTF-8"));
            }
        } catch (IOException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                httpClient.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return result;
    }

    */
/**
     * 获取咨讯视频二级分类
     * @param type
     *//*

    public List<PlatformInformationVideoClassifyDo> findInformationVideoClassify(Integer type) {
        return platformInformationVideoClasssifyDao.findAllClassify(type);
    }

    */
/**
     * 撤回文章操作
     * @param id
     * @param patient
     * @param reason
     * @return
     * @throws Exception
     *//*

    public PlatformInformationDO undoOperation(Long id, String patient, String reason) throws Exception {
        PlatformInformationDO platformInformationDO = platformInformationDao.findById(id).get();
        platformInformationDO.setSendStatus(3);
        platformInformationDO.setUndoReason(reason);
        platformInformationDO = platformInformationDao.save(platformInformationDO);
        // 将操作记录到日志表中
        PlatformInformationOperationLogDO operationLogDO = new PlatformInformationOperationLogDO();
        operationLogDO.setInformationId(platformInformationDO.getId());
        operationLogDO.setOperator(patient);
        operationLogDO.setReason(reason);
        operationLogDO.setName("撤回");
        operationLogDO.setOperatorTime(new Date());
        logDao.save(operationLogDO);
        // 清除缓存
        HttpClientUtil.get(wlyyUrl + "/platformInfomation/cleanPreviewById?id=" + id, "UTF-8");

        return platformInformationDO;
    }

    */
/**
     * 健康频道 -平台咨询查询
     * @param title 标题
     * @param type 一级分类
     * @param classify 二级分类
     * @param town 行政区划
     * @param hospital 机构
     * @param page 当前页
     * @param pageSize 分页大小
     * @return
     *//*

    public Page<List<Map<String, Object>>> channelSearchList(String title,Integer type,String classify,String town,String hospital, Integer page, Integer pageSize) {
        if (page == null) {
            page = 1;
        }
        if (pageSize == null) {
            pageSize = 15;
        }
        PageRequest pageRequest = PageRequest.of(page - 1, pageSize);
        String sql = "select p.title,p.type,p.update_time,p.id,p.video_type_classify ,c.name as video_type_classify_name,p.image,p.author,p.author_org_name  from wlyy_platform_information p LEFT JOIN wlyy_platform_information_video_classify c on c.id=p.video_type_classify where p.sending_objects =1 and p.del=1 AND p.send_status=1 ";
        String countSql = "select count(1) from wlyy_platform_information p where p.sending_objects =1 and p.del=1 AND p.send_status=1 ";
        StringBuilder filters = new StringBuilder();

        if (StringUtils.isNotBlank(title)) {
            filters.append("and p.title like '%" + title + "%' ");
        }
        if (null != type) {
            filters.append("and p.type = '" + type + "' ");
        }
        if (StringUtils.isNotBlank(classify)) {
            filters.append("and p.video_type_classify = '" + classify + "' ");
        }
        if (StringUtils.isNotBlank(town)) {
            filters.append("and p.town = '" + town + "' ");
        }
        if (StringUtils.isNotBlank(hospital)) {
            filters.append("and p.hospital = '" + hospital + "' ");
        }
        List<Map<String, Object>> resultList = jdbcTemplate.queryForList(sql + filters.toString() + " order by p.create_time desc" + " limit " + (page - 1) * pageSize + "," + pageSize);
        resultList.stream().forEach(m -> {
            //咨讯分类（1健康知识、2活动通知、3功能介绍、4平台宣传、5健康视频
            if (null != m.get("type")) {
                String s = m.get("type").toString();
                m.put("update_time",null!=m.get("update_time")?m.get("update_time").toString():"");
                //1健康知识、2活动通知、3功能介绍、4平台宣传、5健康视频
                switch (s) {
                    case "1":
                        m.put("type_name", "健康知识");
                        break;
                    case "2":
                        m.put("type_name", "活动通知");
                        break;
                    case "3":
                        m.put("type_name", "功能介绍");
                        break;
                    case "4":
                        m.put("type_name", "平台宣传");
                        break;
                    case "5":
                        m.put("type_name", "健康视频");
                        break;
                    default:
                        m.put("type_name", "");
                        break;
                }
            }
        });
        if (resultList.size() <= 0) {
            return new PageImpl(new ArrayList<>(), pageRequest, 0);
        }
        Long count = jdbcTemplate.queryForObject(countSql + filters.toString(), Long.class);
        return  new PageImpl(resultList, pageRequest, count);
    }

    */
/**
     * 获取平台咨讯全部分类
     * @return
     *//*

    public List<Map<String, Object>> findPlatformInformationClassify() {
        List<Map<String, Object>> resList = new ArrayList<>();
        for(int type = 1 ; type <= 5; type++) {
            Map<String, Object> map = new LinkedHashMap<>();
            if(type == 1){
                map.put("id",1);
                map.put("name", "健康知识");
            }else if(type == 2){
                map.put("id",2);
                map.put("name", "活动通知");
            }else if(type == 3){
                map.put("id",3);
                map.put("name", "功能介绍");
            }else if(type == 4){
                map.put("id",4);
                map.put("name", "平台宣传");
            }else if(type == 5){
                map.put("id",5);
                map.put("name", "健康视频");
            }
            List<PlatformInformationVideoClassifyDo> classifyList = platformInformationVideoClasssifyDao.findAllClassify(type);
            if(classifyList.size() > 0){
                map.put("classifyList", classifyList);
            }
            resList.add(map);
        }
        return resList;
    }
}
*/
