package com.yihu.customer.service.manager.sign;

 
 
import  com.yihu.customer.entity.*;
import  com.yihu.customer.repository.*;
import com.yihu.customer.service.manager.consult.WlyyConsultTeamService;
import com.yihu.customer.service.manager.patient.AdminPatientService;
import com.yihu.customer.service.manager.team.ManageWlyyAdminTeamService;
import com.yihu.customer.util.HttpManageClientUtil;
import com.yihu.customer.util.query.BaseJpaService;
import com.yihu.customer.vo.SignPayVO;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONException;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * Created by Administrator on 2016/10/26.
 * 签约管理服务类
 */
@Service
public class SignFamilyService extends BaseJpaService<SignFamily,Long> {
    @Autowired
    private PatientDao patientDao;

    @Autowired
    private FollowupDao fvPlanDao;

    @Autowired
    private SignFamilyDao signFamilyDao;

    @Autowired
    private WlyyDoctorTeamDao doctorTeamDao;

    @Autowired
    private WlyyDoctorTeamMemberDao doctorTeamMemberDao;

    @Autowired
    private DoctorDao doctorDao;

    @Autowired
    private ManageWlyyAdminTeamService teamService;

    @Autowired
    private WlyyAdminTeamDao adminTeamDao;

    @Autowired
    private WlyyConsultTeamService consultTeamService;

    @Autowired
    private AdminPatientService adminPatientService;

    @Value("${im-service.im_service_url}")
    private String imUrl;
    
    @Autowired
    JdbcTemplate jdbcTemplate;

    @Autowired
    private TrackPatientDao trackPatientDao;


    //签约居民管理**********************
    @Transactional
    public boolean updateStatus(Integer status,Long id){
        int sum = signFamilyDao.updateStatus(status,id);
        return sum>0?true:false;
    }
    public Page<SignFamily> searchList(String name,String idcard,String address,String hospitalName,String signDoctorName,Integer page,Integer pageSize) throws Exception{
        //默认分页，页码从0开始
        if(page <= 0){
            page = 1;
        }
        if(pageSize <= 0){
            pageSize = 15;
        }
        PageRequest pageRequest = PageRequest.of(page - 1, pageSize);
        StringBuilder filters = new StringBuilder();
        if(!StringUtils.isEmpty(name)){
            filters.append("name?"+name+";");
        }
        if(!StringUtils.isEmpty(idcard)){
            filters.append("idcard?"+idcard+";");
        }
        if(!StringUtils.isEmpty(hospitalName)){
            filters.append("hospitalName?"+hospitalName+";");
        }
        if(!StringUtils.isEmpty(signDoctorName)){
            filters.append("doctorName?"+signDoctorName+";");
        }
        if(!StringUtils.isEmpty(address)){
            Long[] ids = signFamilyDao.findByPatientAddressLike("%"+address+"%");
            if(ids.length>0){
                filters.append("id="+StringUtils.join(ids,",")+";");
            }else {
                return new PageImpl<SignFamily>(new ArrayList<>(),pageRequest,0);
            }
        }
        List<SignFamily> list = search("", filters.toString(), "", page, pageSize);
        long count = 0;
        if(list != null && list.size()>0){
            count = getCount(filters.toString());
            for(SignFamily signFamily :list){
                Long teamId = signFamily.getAdminTeamId();
                if(teamId != null && teamId >0){
                    WlyyAdminTeam team = teamService.findById(teamId);
                    if(team != null){
                        signFamily.setAdminTeamName(team.getName());
                        signFamily.setLeaderCode(team.getLeaderCode());
                        signFamily.setLeaderName(team.getLeaderName());
                    }
                }
                Patient patient = patientDao.findByCodeAndIdcard(signFamily.getPatient(),signFamily.getIdcard());
                if(patient != null){
                    signFamily.setPatientAddress(patient.getAddress());
                }
            }
        }
        return new PageImpl<SignFamily>(list,pageRequest,count);
    }
    public SignFamily findById(Long id){
        SignFamily signFamily = signFamilyDao.findById(id).get();
        return signFamily;
    }
    public SignFamily findByCode(String code){
        SignFamily signFamily = signFamilyDao.findByCode(code);
        return signFamily;
    }

    //签约居民分配******************
    public Page<SignFamily> searchListForAllocate(String name,String idcard,String address,String hospitalName,String signDoctorName,Integer adminTeamId,String teamName,String leaderName,String doctorHealthName, Integer page,Integer pageSize) throws Exception{
        //默认分页，页码从0开始
        if(page <= 0){
            page = 1;
        }
        if(pageSize <= 0){
            pageSize = 15;
        }
        PageRequest pageRequest = PageRequest.of(page - 1, pageSize);
        StringBuilder filters = new StringBuilder();
        filters.append("type=2;");//家庭签约有行政团队
        //签约状态(-1患者已取消，-2已拒绝，-3已解约，-4已到期，0待签约，1已签约，2患者申请取消签约，3医生申请取消签约',
        filters.append("status>0;");
        if(!StringUtils.isEmpty(name)){
            filters.append("name?"+name+";");
        }
        if(!StringUtils.isEmpty(idcard)){
            filters.append("idcard?"+idcard+";");
        }
        if(!StringUtils.isEmpty(hospitalName)){
            filters.append("hospitalName?"+hospitalName+";");
        }
        if(!StringUtils.isEmpty(signDoctorName)){
            filters.append("doctorName?"+signDoctorName+";");
        }
        if(!StringUtils.isEmpty(doctorHealthName)){
            filters.append("doctorHealthName?"+doctorHealthName+";");
        }
        if(adminTeamId != null){
            filters.append("adminTeamId="+adminTeamId+";");
        }
        if(!StringUtils.isEmpty(address)){
            Long[] ids = signFamilyDao.findByPatientAddressLike("%"+address+"%");
            if(ids.length>0){
                filters.append("id="+StringUtils.join(ids,",")+";");
            }else {
                return new PageImpl<SignFamily>(new ArrayList<>(),pageRequest,0);
            }
        }
        //团队名称模糊查询]
        if(!StringUtils.isEmpty(teamName) || !StringUtils.isEmpty(leaderName)){
            if(StringUtils.isEmpty(teamName)){teamName = "";}
            if(StringUtils.isEmpty(leaderName)){leaderName = "";}
            Long[] teamIds = adminTeamDao.findByNameAndLeaderNameLike("%" + teamName + "%", "%"+leaderName+"%");
            if(teamIds.length>0){
                filters.append("adminTeamId="+StringUtils.join(teamIds,",")+";");
            }else {
                return new PageImpl<SignFamily>(new ArrayList<>(),pageRequest,0);
            }
        }
        List<SignFamily> list = search("", filters.toString(), "", page, pageSize);
        long count = 0;
        if(list != null && list.size()>0){
            count = getCount(filters.toString());
            for(SignFamily signFamily :list){
                Long teamId = signFamily.getAdminTeamId();
                if(teamId != null && teamId >0){
                    WlyyAdminTeam team = teamService.findById(teamId);
                    if(team != null){
                        signFamily.setAdminTeamName(team.getName());
                        signFamily.setLeaderCode(team.getLeaderCode());
                        signFamily.setLeaderName(team.getLeaderName());
                    }
                }
                Patient patient = patientDao.findByCodeAndIdcard(signFamily.getPatient(),signFamily.getIdcard());
                if(patient != null){
                    signFamily.setPatientAddress(patient.getAddress());
                }
            }
        }
        return new PageImpl<SignFamily>(list,pageRequest,count);
    }



    /**
     * 批量分配全科医生或健管师 或全科医生+健管师（全科医生只能是全科医生担任，健管师可有全科医生、健管师担任，可以都为某个全科医生担任）
     * 批量分配只针对同个团队，
     * 批量处理符合条件的保存，不符条件的返回页面并可导出；
     * 存在未完成随访计划或咨询的签约居民不能进行分配操作
     * 签约居民分配 涉及表，wlyy_sign_family（全科医生，健康管理师），
     *     wlyy_doctor_team_member：出签约居民外，旧的医生失效，新增
     *
     */
    @Transactional
    public Map<String,Object> batchAllocate(Long[] ids,String doctor,String doctorHealth) throws JSONException {
        Map<String,Object> res = new HashMap<>();
        Integer memberCount = 1;//用于统计签约居民服务团队成员数的变量 用于wlyyDoctorTeam
        String doctorName = "";
        String doctorHealthName = "";
        if(!StringUtils.isEmpty(doctor)){
            Doctor doc = doctorDao.findByCode(doctor);
            doctorName = doc.getName();
            memberCount++;
        }
        if(!StringUtils.isEmpty(doctorHealth)){
            Doctor doc = doctorDao.findByCode(doctorHealth);
            doctorHealthName = doc.getName();
            memberCount++;
        }
        List<SignFamily> signFamilies = (ArrayList<SignFamily>)signFamilyDao.findAllById(Arrays.asList(ids));
        //验证批量分配签约居民属于同个行政团队
        long adminTeamId = signFamilies.get(0).getAdminTeamId();
        List<WlyyDoctorTeam> doctorTeams = new ArrayList<>();
        List<WlyyDoctorTeamMember> doctorTeamMembers = new ArrayList<>();
        StringBuffer teamCodes = new StringBuffer();
        List<SignFamily> errorList = new ArrayList<>();
        List<SignFamily> successList = new ArrayList<>();
        String oldHealthyDoctor = "";
        String oldDoctor = "";
        for(SignFamily signFamily : signFamilies){
            oldDoctor  = signFamily.getDoctor();
            oldHealthyDoctor = signFamily.getDoctorHealth();
            if(signFamily.getAdminTeamId() != adminTeamId){
                res.put("errorMsg","签约居民'"+signFamily.getName()+"'不属于同个团队！");
                res.put("successNum",0);
                res.put("failNum",signFamilies.size());
                return res;
            }
            //验证是否存在未完成的咨询和随访计划
            int fvPlanNum = fvPlanDao.searchUnfinished(signFamily.getPatient());
            int consultTeamNum = consultTeamService.isExistUnfinished(signFamily.getPatient());
            String msg = "";
            if(fvPlanNum >0){
                msg += "签约居民存在未结束的随访计划;";
            }
            if(consultTeamNum >0){
                msg += "签约居民存在未结束的咨询";
            }
            if(!StringUtils.isEmpty(msg)){
                signFamily.setMsg(msg);
                errorList.add(signFamily);
                continue;

            }

            if(!StringUtils.isEmpty(doctor)){

                //1.4.2更新重点关注列表,取消关注
                updateTrackPatient(signFamily.getPatient(),signFamily.getDoctor(),signFamily.getDoctorHealth(),doctor,doctorHealth);

                signFamily.setDoctor(doctor);
                signFamily.setDoctorName(doctorName);
            }
            if(!StringUtils.isEmpty(doctorHealth)){

                //1.4.2更新重点关注列表,取消关注
                updateTrackPatient(signFamily.getPatient(),signFamily.getDoctorHealth(),signFamily.getDoctor(),doctor,doctorHealth);

                signFamily.setDoctorHealth(doctorHealth);
                signFamily.setDoctorHealthName(doctorHealthName);
            }
            String teamCode = signFamily.getTeamCode();
            WlyyDoctorTeam doctorTeam = doctorTeamDao.findByCode(teamCode);
            WlyyDoctorTeamMember member;
            if(doctorTeam != null){
                teamCodes.append(","+teamCode);
                String name = signFamily.getName();
                if(!StringUtils.isEmpty(doctorName)){
                    name += ","+doctorName;
                    Doctor doc = doctorDao.findByCode(doctor);
                    //判断新设置的全科医生是否是原有的(不的话新增）
                    int memberNum = doctorTeamMemberDao.isMemberExist(teamCode, doctor,2,"1");
                    if(memberNum <= 0 && doc != null){
                        //判断是否存在原有全科医生（已失效的）改为生效就行
                        List<WlyyDoctorTeamMember> mList = doctorTeamMemberDao.findByTeamAndMemberCodeAndTypeAndDel(teamCode, doctor, 2, "0");
                        if(mList == null || mList.size()<=0){
                            //不存在，新增记录
                            member = new WlyyDoctorTeamMember();
                            member.setTeam(teamCode);
                            member.setMemberCode(doc.getCode());
                            member.setName(doc.getName());
                            member.setType(2);
                            member.setCzrq(new Date());
                            member.setDel("1");
                            member.setCode(UUID.randomUUID().toString());
                            member.setSignType(signFamily.getType()+"");
                            doctorTeamMemberDao.updateDelByTeamAndType(teamCode,2);
                            doctorTeamMemberDao.save(member);
                        }else {
                            doctorTeamMemberDao.updateDelByTeamAndType(teamCode,2);
                            //存在，修改del
                            member = mList.get(0);
                            member.setDel("1");
                            doctorTeamMemberDao.save(member);
                        }
                    }
                }
                if(!StringUtils.isEmpty(doctorHealthName)){
                    name += ","+doctorHealthName;
                    Doctor doc = doctorDao.findByCode(doctorHealth);
                    //判断新设置的健康管理师是否是原有的,原有的就不去做失效，新增操作
                    int memberNum = doctorTeamMemberDao.isMemberExist(teamCode,doctorHealth,3,"1");
                    if(memberNum <= 0 && doc != null){
                        //判断是否存在原有全科医生（已失效的）改为生效就行
                        List<WlyyDoctorTeamMember> mList = doctorTeamMemberDao.findByTeamAndMemberCodeAndTypeAndDel(teamCode, doctorHealth, 3, "0");
                        if(mList == null || mList.size()<=0){
                            //不存在，新增记录
                            member = new WlyyDoctorTeamMember();
                            member.setTeam(teamCode);
                            member.setMemberCode(doc.getCode());
                            member.setName(doc.getName());
                            member.setType(3);
                            member.setCzrq(new Date());
                            member.setDel("1");
                            member.setCode(UUID.randomUUID().toString());
                            member.setSignType(signFamily.getType()+"");
                            doctorTeamMemberDao.updateDelByTeamAndType(teamCode,3);
                            doctorTeamMemberDao.save(member);
                        }else {
                            doctorTeamMemberDao.updateDelByTeamAndType(teamCode,3);
                            //存在，修改del
                            member = mList.get(0);
                            member.setDel("1");
                            doctorTeamMemberDao.save(member);
                        }
                    }
                }
                doctorTeam.setName(name);
                doctorTeam.setMembers(memberCount);
                doctorTeamDao.save(doctorTeam);
            }
            //更新IM内医生信息
            String sessionId = signFamily.getPatient()+"_consult_"+signFamily.getType();
            String imAddr = imUrl+"/sessions/"+sessionId+"/participant/update";
            JSONObject healthyparams = new JSONObject();
            if(!StringUtils.isEmpty(doctorHealth)) {
                healthyparams.put("user_id", signFamily.getDoctorHealth());
                //当前旧人员在替换的列表中就不做移除
                if(!StringUtils.equals(oldHealthyDoctor,signFamily.getDoctor())&&!StringUtils.equals(oldHealthyDoctor,signFamily.getDoctorHealth())){
                    healthyparams.put("old_user_id", oldHealthyDoctor);
                }
                healthyparams.put("session_id", sessionId);
                String result = HttpManageClientUtil.postBody(imAddr, healthyparams);
            }
            if(!StringUtils.isEmpty(doctor)&&!StringUtils.equals(signFamily.getDoctorHealth(),signFamily.getDoctor())){
                JSONObject docparams = new JSONObject();
                docparams.put("user_id", signFamily.getDoctor());
                docparams.put("session_id", sessionId);
                //全科建管同一个，新增全科的时候不带入旧人员，//当前旧人员在替换的列表中就不做移除
                if(!StringUtils.equals(oldDoctor,signFamily.getDoctor())
                        &&!StringUtils.equals(oldDoctor,signFamily.getDoctorHealth())
                        &&!StringUtils.equals(oldDoctor,oldHealthyDoctor)
                        ){
                    docparams.put("old_user_id", oldDoctor);
                }
                String result =  HttpManageClientUtil.postBody(imAddr,docparams);
            }
            //im信息更新完毕
            successList.add(signFamily);
        }
        String team = teamCodes.toString();
        if(!StringUtils.isEmpty(team)){
            team = team.substring(1);
        }
        res.put("errorList",errorList);
        res.put("successNum",successList.size());
        res.put("failNum",errorList.size());
        signFamilyDao.saveAll(successList);
        return res;
    }

    public SignFamily findByPatientCode(String code) {
        return signFamilyDao.findSignByPatient(code);
    }
    
    /**
     * 根据时间获取签约支付明细
     * @param beginDate
     * @param endDate
     * @return
     */
    public List<SignPayVO> geSignPayDataList(String beginDate, String endDate) {
        List<SignPayVO> list = new ArrayList<>();
        String sql  = "SELECT DISTINCT " +
                "b.`code`," +
                "b.`charge_relation`," +
                "b.`create_time`," +
                "b.`name` ," +
                "b.`id_no` ," +
                "b.`patient` ," +
                "b.`openid` ," +
                "b.`ssc` ," +
                "b.`principal_ssc` ," +
                "b.`hospital_name` , " +
                "f.doctor_name,t.id as teamId, t.`name` as teamName," +
                "b.`total_amount` ," +
                "b.`selfpay_amount` ," +
                "b.`insurance_amount` ," +
                "b.`trade_status` ," +
                "b.`civil_pay` ," +
                "b.`self_pay` ," +
                "b.`account_pay` ," +
                "b.`heath_pay` ," +
                "b.`health_care_pay` ," +
                "b.`mi_total_fee` ," +
                "b.`cadres_pay`  " +
                "FROM wlyy_charge b LEFT JOIN wlyy_sign_family f on b.patient=f.patient and f.`status`>0 and f.expenses_status=1 " +
                "LEFT JOIN wlyy_admin_team t on f.admin_team_code=t.id and t.available=true WHERE b.trade_status='0' ";
        if(StringUtils.isNotBlank(beginDate)){
            sql = sql + "AND b.create_time>='"+beginDate+"' ";
        }
    
        if(StringUtils.isNotBlank(endDate)){
            sql = sql + "AND b.create_time<='"+endDate+"' ";
        }

        sql += " order by create_time ";
        list =  jdbcTemplate.query(sql,new BeanPropertyRowMapper(SignPayVO.class));
        return list ;
    }

    /**
     * 设置取消关注
     * @param patient
     * @param oldDoctor 患者签约旧的要取消关注的医生
     * @param oldDoctor2 患者旧的签约的另一个医生
     * @param newDoctor 患者新签约的全科医生
     * @param newHDoctor 患者新签约的健康管理师
     */
    public void updateTrackPatient(String patient,String oldDoctor,String oldDoctor2,String newDoctor,String newHDoctor){

        //是否需要修改
        boolean flag = false;

        if(StringUtils.isBlank(oldDoctor)){
            return ;
        }
        //1.两个都修改
        if(StringUtils.isNotBlank(newDoctor)&&StringUtils.isNotBlank(newHDoctor)){
            if((!oldDoctor.equals(newDoctor))&&(!oldDoctor.equals(newHDoctor))){
                flag = true;
            }
            //只改全科
        }else if(StringUtils.isNotBlank(newDoctor)&&StringUtils.isBlank(newHDoctor)){
            if(!oldDoctor.equals(newDoctor)){
                if(StringUtils.isBlank(oldDoctor2)){
                    flag = true;
                }else{
                    if(!oldDoctor.equals(oldDoctor2)){
                        flag = true;
                    }
                }
            }
            //只改健管
        }else if(StringUtils.isBlank(newDoctor)&&StringUtils.isNotBlank(newHDoctor)){
            if(!oldDoctor.equals(newHDoctor)){
                if(StringUtils.isBlank(oldDoctor2)){
                    flag = true;
                }else{
                    if(!oldDoctor.equals(oldDoctor2)){
                        flag = true;
                    }
                }
            }
        }else{
            return ;
        }
        if(flag){
            TrackPatient trackPatient = trackPatientDao.findByDoctorCodeAndPatientCode(oldDoctor,patient);
            if(trackPatient!=null){
                trackPatient.setDel("0");
                trackPatientDao.save(trackPatient);
            }
        }
    }
}
