package com.yihu.customer.service.manager.specialist;/**
 * Created by nature of king on 2018/8/17.
 */


import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.customer.service.BaseService;
import com.yihu.customer.service.common.excel.ExcelData;
import  com.yihu.customer.entity.specialist.SpecialistServiceItemDO;
import  com.yihu.customer.repository.SystemDictDao;
import com.yihu.customer.util.HttpManageClientUtil;
import com.yihu.customer.util.ReadExcelUtil;
import jxl.Sheet;
import jxl.Workbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * @author wangzhinan
 * @create 2018-08-17 15:27
 * @desc 服务项目
 **/
@Service
public class ServiceItemService extends BaseService {
    private Logger logger = LoggerFactory.getLogger(ServiceItemService.class);

    private static String URL = "specialist:url";
    @Autowired
    private RedisTemplate redisTemplate;
    @Autowired
    private SystemDictDao systemDictDao;
    @Autowired
    private HttpManageClientUtil httpManageClientUtil;

    /**
     * 获取url
     *
     * @return
     */
    public  String getBaseUrl(){
        String url = (String) redisTemplate.opsForValue().get(URL);
        String url1 = systemDictDao.findByDictNameAndCode3("SPECIALIST_URL","SPECIALIST_URL");
        if (url != null && url1.equalsIgnoreCase(url)){
            return url;
        }else {
            redisTemplate.opsForValue().set(URL,url1);
            return url1;
        }
    }


    /**
     * 添加服务项目
     *
     * @param serviceItem
     * @return
     * @throws Exception
     */
    public JSONObject createServiceItem(String serviceItem) throws Exception {
        String response = null;
        String url =getBaseUrl() + "createServiceItem";
        Map<String,String> params = new HashMap<>();
        params.put("serviceItem",serviceItem);
        try {
            response = httpManageClientUtil.httpPost(url,params);
        }catch (Exception e){
            e.printStackTrace();
            logger.error(e.getMessage());
        }
        return JSONObject.parseObject(response);
    }


    /**
     * 查询服务项目
     *
     * @param serviceItem
     * @param page
     * @param pageSize
     * @return
     * @throws Exception
     */
    public JSONObject selectServiceItem(String serviceItem,Integer page,Integer pageSize) throws Exception {
        String response = null;
        String url =getBaseUrl() + "getServiceItem";
        Map<String,String> params = new HashMap<>();
        params.put("serviceItem",serviceItem);
        params.put("page",page.toString());
        params.put("pageSize",pageSize.toString());
        try {
            response = httpManageClientUtil.httpPost(url,params);
        }catch (Exception e){
            e.printStackTrace();
            logger.error(e.getMessage());
        }
        return JSONObject.parseObject(response);
    }


    /**
     * 批量删除服务项目
     *
     * @param ids
     * @return
     * @throws Exception
     */
    public JSONObject delete(String ids) throws Exception {
        String response = null;
        String url =getBaseUrl() + "batchDelete";
        Map<String,String> params = new HashMap<>();
        params.put("ids",ids);
        try {
            response = httpManageClientUtil.httpPost(url,params);
        }catch (Exception e){
            e.printStackTrace();
            logger.error(e.getMessage());
        }
        return JSONObject.parseObject(response);
    }


    /**
     * 更新服务项目
     *
     * @param serviceItem
     * @return
     * @throws Exception
     */
    public JSONObject updateServiceItem(String serviceItem) throws Exception {
        String response = null;
        String url =getBaseUrl() + "updateServiceItem";
        Map<String,String> params = new HashMap<>();
        params.put("serviceItem",serviceItem);
        try {
            response = httpManageClientUtil.httpPost(url,params);
        }catch (Exception e){
            e.printStackTrace();
            logger.error(e.getMessage());
        }
        return JSONObject.parseObject(response);
    }

    /**
     * 查询历史记录
     *
     * @param operateLog
     * @param page
     * @param size
     * @return
     * @throws Exception
     */
    public JSONObject selectOperate(String operateLog,Integer page,Integer size) throws Exception {
        String response = null;
        String url =getBaseUrl() + "selectByOperate";
        Map<String,String> params = new HashMap<>();
        params.put("operateLog",operateLog);
        params.put("page",page.toString());
        params.put("size",size.toString());
        try {
            response = httpManageClientUtil.httpPost(url,params);
        }catch (Exception e){
            e.printStackTrace();
            logger.error(e.getMessage());
        }
        return JSONObject.parseObject(response);
    }


    /**
     * 导数据
     *
     * @param workbook
     * @return
     */
    public JSONObject importData(Workbook workbook) {
        Sheet[] sheets = workbook.getSheets();
        Sheet sheet = sheets[0];
        int rows = ReadExcelUtil.getRightRows(sheet);
        JSONArray array = new  JSONArray();
        for (int row = 1; row < rows; row++) {  //索引从0开始，第一行为标题
            SpecialistServiceItemDO itemDO = new SpecialistServiceItemDO();
            Map<Integer, ExcelData> mapping = mapping(itemDO);
            int finalRow = row;
            mapping.forEach((index, excelData) -> {
                String value = sheet.getCell(index-1, finalRow).getContents().trim();
                excelData.transform(value);
            });
            array.add(itemDO);
        }
        String response = null;
        String url =/*"http://localhost:10051/svr-specialist/"*/getBaseUrl() + "importData1";
        Map<String,String> params = new HashMap<>();
        params.put("serviceItems",array.toJSONString());
        try {
            response = httpManageClientUtil.httpPost(url,params);
        }catch (Exception e){
            e.printStackTrace();
            logger.error(e.getMessage());
        }
        return JSONObject.parseObject(response);
    }

    /**
     * 表格数据转为对象
     *
     * @param specialistServiceItemDO
     * @return
     */
    private Map<Integer, ExcelData> mapping(SpecialistServiceItemDO specialistServiceItemDO) {
        Map<Integer, ExcelData> dataMap = new HashMap<>();
        //项目名称
        dataMap.put(1, new ExcelData() {
            @Override
            public void transform(String data) {
                specialistServiceItemDO.setTitle(data);
            }
        });
        //项目内涵
        dataMap.put(2, new ExcelData() {
            @Override
            public void transform(String data) {
                specialistServiceItemDO.setContent(data);
            }
        });
        //除去内容
        dataMap.put(3, new ExcelData() {
            @Override
            public void transform(String data) {
                specialistServiceItemDO.setExcludeContent(data);
            }
        });
        //项目类型
        dataMap.put(4, new ExcelData() {
            @Override
            public void transform(String data) {
                Map<String, String> centerSite = new HashMap<>();
                centerSite.put("康复服务", "1");
                centerSite.put("健康服务", "2");
                System.out.print(data);
                String data1 = data;
                specialistServiceItemDO.setItemType(Integer.parseInt(centerSite.get(data)));
            }
        });
        //医院等级
        dataMap.put(5, new ExcelData() {
            @Override
            public void transform(String data) {
                Map<String,Integer> grade = new HashMap<>();
                grade.put("所有",0);
                grade.put("一级及一级以下医疗机构",1);
                grade.put("二级医院",2);
                grade.put("三级医院",3);
                specialistServiceItemDO.setHospitalGrade(grade.get(data));
            }
        });
        //三级医院收费
        dataMap.put(6, new ExcelData() {
            @Override
            public void transform(String data) {
                specialistServiceItemDO.setThreeHospitals(Double.parseDouble(data));
            }
        });
        //二级医院收费
        dataMap.put(7, new ExcelData() {
            @Override
            public void transform(String data) {
                specialistServiceItemDO.setTwoHospitals(Double.parseDouble(data));

            }
        });
        //一级及下收费
        dataMap.put(8, new ExcelData() {
            @Override
            public void transform(String data) {
                specialistServiceItemDO.setOneHospitals(Double.parseDouble(data));
            }
        });
        //计价单位
        dataMap.put(9, new ExcelData() {
            @Override
            public void transform(String data) {
                specialistServiceItemDO.setUnit((data));
            }
        });
        //加收项目
        dataMap.put(10, new ExcelData() {
            @Override
            public void transform(String data) {
                specialistServiceItemDO.setAddItem(data);
            }
        });

        //病案项目
        dataMap.put(11, new ExcelData() {
            @Override
            public void transform(String data) {
                specialistServiceItemDO.setDiseaseItem(data);
            }
        });


        //是否预约
        dataMap.put(12, new ExcelData() {
            @Override
            public void transform(String data) {
                Map<String,Integer> reserve = new HashMap<>();
                reserve.put("是",1);
                reserve.put("否",0);
                specialistServiceItemDO.setReserve(reserve.get(data));
            }
        });

        //完成方式
        dataMap.put(13, new ExcelData() {
            @Override
            public void transform(String data) {
                Map<String,Integer> type = new HashMap<>();
                type.put("扫码",1);
                type.put("上传附件",0);
                type.put("健康教育",2);
                type.put("健康指导",3);
                type.put("随访",4);
                specialistServiceItemDO.setType(type.get(data));
            }
        });

        //是否评价
        dataMap.put(14, new ExcelData() {
            @Override
            public void transform(String data) {
                Map<String,Integer> evaluate = new HashMap<>();
                evaluate.put("是",1);
                evaluate.put("否",0);
                specialistServiceItemDO.setEvaluation(evaluate.get(data));
            }
        });

        //是否生效
        dataMap.put(15, new ExcelData() {
            @Override
            public void transform(String data) {
                Map<String,Integer> imediate = new HashMap<>();
                imediate.put("是",1);
                imediate.put("否",0);
                specialistServiceItemDO.setEvaluation(imediate.get(data));
            }
        });
        return dataMap;
    }
}
