package com.yihu.customer.service.manager.specialist;/**
 * Created by nature of king on 2018/8/28.
 */

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.customer.service.BaseService;
import com.yihu.customer.service.common.excel.ExcelData;
import  com.yihu.customer.entity.Hospital;
import  com.yihu.customer.entity.specialist.HospitalServiceItemDO;
import  com.yihu.customer.repository.HospitalDao;
import  com.yihu.customer.repository.SystemDictDao;
import com.yihu.customer.util.HttpManageClientUtil;
import com.yihu.customer.util.ReadExcelUtil;
import jxl.Sheet;
import jxl.Workbook;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author wangzhinan
 * @create 2018-08-28 20:40
 * @desc 机构服务项目
 **/
@Service
@Transactional
public class SpecialistHospitalItemService extends BaseService {

    private static org.slf4j.Logger logger = LoggerFactory.getLogger(SpecialistHospitalItemService.class);
    @Autowired
    private HospitalDao hospitalDao;
    private static String URL = "specialist:url";
    @Autowired
    private RedisTemplate redisTemplate;
    @Autowired
    private SystemDictDao systemDictDao;
    @Autowired
    private HttpManageClientUtil httpManageClientUtil;
    @Autowired
    private JdbcTemplate jdbcTemplate;

    /**
     * 获取url
     *
     * @return
     */
    public  String getBaseUrl(){
        String url = (String) redisTemplate.opsForValue().get(URL);
        String url1 = systemDictDao.findByDictNameAndCode3("SPECIALIST_URL","SPECIALIST_URL");
        if (url != null && url1.equalsIgnoreCase(url)){
            return url;
        }else {
            redisTemplate.opsForValue().set(URL,url1);
            return url1;
        }
    }

    /**
     * 获取医院
     *
     * @return
     */
    public List<Hospital> selectHospital(String hospitalName,Integer grade){
        List<Hospital> hospitals = new ArrayList<>();
        if (StringUtils.isNoneBlank(hospitalName) &&  grade != null){
            String sql = "select * from dm_hospital where name like '%"+hospitalName+"%' AND grade = "+grade;
            hospitals = jdbcTemplate.query(sql,new BeanPropertyRowMapper(Hospital.class));
        } else if (StringUtils.isNoneBlank(hospitalName)){
            String sql = "select * from dm_hospital where name like '%"+hospitalName+"%'";
            hospitals = jdbcTemplate.query(sql,new BeanPropertyRowMapper(Hospital.class));
        }else if (grade != null){
            String sql = "select * from dm_hospital where grade = "+grade+"";
            hospitals = jdbcTemplate.query(sql,new BeanPropertyRowMapper(Hospital.class));
        }else {
            hospitals = hospitalDao.findAllHospital();
        }
        return hospitals;
    }

    /**
     * 添加机构服务项目
     *
     * @param hospitalServiceItem
     * @return
     */
    public JSONObject insert(String hospitalServiceItem){
        String response = null;
        String url =getBaseUrl() + "createHospitalServiceItem";
        Map<String,String> params = new HashMap<>();
        params.put("hospitalServiceItem",hospitalServiceItem);
        try {
            response = httpManageClientUtil.httpPost(url,params);
        }catch (Exception e){
            e.printStackTrace();
            logger.error(e.getMessage());
        }
        return JSONObject.parseObject(response);
    }


    /**
     * 根据医院code获取服务项目
     *
     * @param hospital
     * @return
     */
    public JSONObject selectItemByHospital(String hospital){
        String response = null;
        String url =getBaseUrl() + "selectItemByHospital";
        Map<String,String> params = new HashMap<>();
        params.put("hospital",hospital);
        try {
            response = httpManageClientUtil.httpPost(url,params);
        }catch (Exception e){
            e.printStackTrace();
            logger.error(e.getMessage());
        }
        return JSONObject.parseObject(response);
    }


    /**
     * 根据条件查找
     *
     * @param serviceItemName
     * @param hospital
     * @return
     */
    public JSONObject selectCondition(String serviceItemName,String hospital){
        JSONArray array = new JSONArray();
        if (StringUtils.isNoneBlank(hospital)){
            array.add(hospital);
        }
        String response = null;
        String url =getBaseUrl() + "selectByCondition";
        Map<String,String> params = new HashMap<>();
        params.put("serviceItemName",serviceItemName);
        params.put("hospitals",array.toJSONString());
        try {
            response = httpManageClientUtil.httpPost(url,params);
        }catch (Exception e){
            e.printStackTrace();
            logger.error(e.getMessage());
        }
        return JSONObject.parseObject(response);
    }


    /**
     * 删除机构服务项目
     */
    public JSONObject delete(String serviceItem,String hospital,String id){
        String response = null;
        String url =getBaseUrl() + "deleteHospitalItem";
        Map<String,String> params = new HashMap<>();
        params.put("hospital",hospital);
        params.put("serviceItem",serviceItem);
        params.put("id",id);
        try {
            response = httpManageClientUtil.httpPost(url,params);
        }catch (Exception e){
            e.printStackTrace();
            logger.error(e.getMessage());
        }
        return JSONObject.parseObject(response);
    }


    /**
     * 机构服务项目导数据
     *
     * @param workbook
     * @return
     */
    public JSONObject importData(Workbook workbook) {
        Sheet[] sheets = workbook.getSheets();
        Sheet sheet = sheets[0];
        int rows = ReadExcelUtil.getRightRows(sheet);
        JSONArray array = new  JSONArray();
        for (int row = 1; row < rows; row++) {  //索引从0开始，第一行为标题
            HospitalServiceItemDO hospitalServiceItemDO = new HospitalServiceItemDO();
            Map<Integer, ExcelData> mapping = mapping1(hospitalServiceItemDO);
            int finalRow = row;
            mapping.forEach((index, excelData) -> {
                String value = sheet.getCell(index-1, finalRow).getContents().trim();
                excelData.transform(value);
            });
            Hospital hospital = hospitalDao.findByName(hospitalServiceItemDO.getHospitalName());
            hospitalServiceItemDO.setHospital(hospital.getCode());
            array.add(hospitalServiceItemDO);
        }
        String response = null;
        String url =/*"http://localhost:10051/svr-specialist/"*/getBaseUrl()+ "importData2";
        Map<String,String> params = new HashMap<>();
        params.put("hospitalItems",array.toJSONString());
        try {
            response = httpManageClientUtil.httpPost(url,params);
        }catch (Exception e){
            e.printStackTrace();
            logger.error(e.getMessage());
        }
        return JSONObject.parseObject(response);
    }


    /**
     * 导入机构服务项目表
     *
     * @param hospitalServiceItemDO
     * @return
     */
    private Map<Integer, ExcelData> mapping1(HospitalServiceItemDO hospitalServiceItemDO) {
        Map<Integer, ExcelData> dataMap = new HashMap<>();
        //医院名称
        dataMap.put(1, new ExcelData() {
            @Override
            public void transform(String data) {
                hospitalServiceItemDO.setHospitalName(data);
            }
        });
        //服务项目标题
        dataMap.put(2, new ExcelData() {
            @Override
            public void transform(String data) {
                hospitalServiceItemDO.setTitle(data);
            }
        });
        //服务项目内容
        dataMap.put(3, new ExcelData() {
            @Override
            public void transform(String data) {
                hospitalServiceItemDO.setContent(data);
            }
        });
        dataMap.put(4, new ExcelData() {
            @Override
            public void transform(String data) {
                hospitalServiceItemDO.setExpense(Integer.parseInt(data));
            }
        });
        hospitalServiceItemDO.setSaasId("dev");
        hospitalServiceItemDO.setStatus(1);
        hospitalServiceItemDO.setImediate(1);
        return dataMap;
    }
}
