package com.yihu.customer.service.manager.survey;

import com.alibaba.fastjson.JSON;
import com.yihu.customer.service.manager.user.UserService;
import com.yihu.customer.util.query.BaseJpaService;
import  com.yihu.customer.entity.SystemDict;
import  com.yihu.customer.entity.User;
import  com.yihu.customer.entity.survey.SurveyLabelInfo;
import  com.yihu.customer.entity.survey.SurveyTemplate;
import  com.yihu.customer.entity.survey.SurveyTemplateOption;
import  com.yihu.customer.entity.survey.SurveyTemplateQuestions;
import  com.yihu.customer.repository.SystemDictDao;
import  com.yihu.customer.repository.survey.SurveyTemplateDao;
import com.yihu.customer.util.ClazzReflect;
import org.apache.commons.lang3.StringUtils;
import org.hibernate.Query;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Created by ysj on 2017/3/3.
 */
@Service
public class SurveyTemplateService extends BaseJpaService<SurveyTemplate,Long> {

    private static final Integer USE_BY_TEMPLATE = 0;

    private static final String  SURVEY_LABEL="SURVEY_LABEL";

    @Autowired
    SurveyLabelInfoService surveyLabelInfoService;
    @Autowired
    SurveyTemplateQuestionService surveyTemplateQuestionService;

    @Autowired
    SurveyTemplateOptionService surveyTemplateOptionService;

    @Autowired
    SystemDictDao systemDictDao;

    @Autowired
    SurveyTemplateDao surveyTemplateDao;

    @Autowired
    UserService userService;

    @Transactional
    public SurveyTemplate saveOrUpdate(String jsonData,String loginUserId) throws JSONException {
        JSONArray array = new JSONArray(jsonData);
        List<SurveyTemplate> surveyTemplates = new ArrayList<>();
        List<SurveyTemplateQuestions> surveyTemplateQuestions = new ArrayList<>();
        List<SurveyTemplateOption> surveyTemplateOptions = new ArrayList<>();
        List<SurveyLabelInfo> surveyLabelInfos = new ArrayList<>();
        SurveyTemplate surveyTemplate = null;
        for(int i=0;i<array.length()-1;i++){
            JSONObject jsonObject = array.getJSONObject(i);
            ClazzReflect clazzReflect = new ClazzReflect();
            surveyTemplate = new SurveyTemplate();
            if(!jsonObject.isNull("id")&&StringUtils.isNotBlank(jsonObject.getString("id"))){
                surveyTemplate = this.findById(jsonObject.getLong("id"));
            }else{
                surveyTemplate.setCode(getCode());
                surveyTemplate.setCreater(loginUserId+"");
                surveyTemplate.setCreateTime(new Date());
                surveyTemplate.setDel("1");
            }
            surveyTemplate.setUpdateTime(new Date());
            if(StringUtils.isBlank(jsonObject.getString("diseaseType"))){
                jsonObject.remove("diseaseType");
                jsonObject.remove("isThird");
            }
            clazzReflect.formatToClazz(surveyTemplate,jsonObject);
            surveyTemplates.add(surveyTemplate);

            surveyTemplateQuestionService.deleteByTemplateCode(surveyTemplate.getCode(),true);
            surveyTemplateOptionService.deleteByTemplateCode(surveyTemplate.getCode(),true);
            surveyLabelInfoService.deleteByUseTypeAndRelationCode(USE_BY_TEMPLATE,surveyTemplate.getCode());
            //问题遍历
            if(!jsonObject.isNull("questions")){
                JSONArray questionsArray = (JSONArray)jsonObject.get("questions");
                for(int j=0;j<questionsArray.length()-1;j++){
                    SurveyTemplateQuestions surveyTemplateQuestion = new SurveyTemplateQuestions();
                    JSONObject questionJson =questionsArray.getJSONObject(j);
                    clazzReflect.formatToClazz(surveyTemplateQuestion,questionJson);
                    surveyTemplateQuestion.setCreateTime(new Date());
                    surveyTemplateQuestion.setUpdateTime(new Date());
                    surveyTemplateQuestion.setDel("1");
                    surveyTemplateQuestion.setTemplateCode(surveyTemplate.getCode());
                    surveyTemplateQuestions.add(surveyTemplateQuestion);
                    //遍历选项
                    if(!questionJson.isNull("options")){
                        JSONArray optionsArray = (JSONArray)questionJson.get("options");
                        for(int z=0;z<optionsArray.length()-1;z++){
                            JSONObject optionsJson = optionsArray.getJSONObject(z);
                            SurveyTemplateOption surveyTemplateOption = new SurveyTemplateOption();
                            clazzReflect.formatToClazz(surveyTemplateOption,optionsJson);
                            surveyTemplateOption.setTemplateCode(surveyTemplate.getCode());
                            surveyTemplateOption.setQuestionCode(surveyTemplateQuestion.getCode());
                            surveyTemplateOption.setDel("1");
                            surveyTemplateOptions.add(surveyTemplateOption);
                        }
                    }
                }
            }
            //遍历labels
            if(!jsonObject.isNull("labels")){
                JSONArray labelsArray = (JSONArray)jsonObject.get("labels");
                for(int k=0;k<labelsArray.length()-1;k++){
                    JSONObject labelJson = labelsArray.getJSONObject(k);
                    SurveyLabelInfo surveyLabelInfo = new SurveyLabelInfo();
                    clazzReflect.formatToClazz(surveyLabelInfo,labelJson);
                    surveyLabelInfo.setRelationCode(surveyTemplate.getCode());
                    surveyLabelInfo.setCode(getCode());
                    surveyLabelInfos.add(surveyLabelInfo);
                }
            }
        }
        surveyTemplateDao.saveAll(surveyTemplates);
        surveyTemplateQuestionService.save(surveyTemplateQuestions);
        surveyTemplateOptionService.save(surveyTemplateOptions);
        surveyLabelInfoService.save(surveyLabelInfos);
        return surveyTemplate;
    }

    public SurveyTemplate findById(Long id){
         return  surveyTemplateDao.findById(id).get();
    }


    public SurveyTemplate findByTitle(String title){
        return  surveyTemplateDao.findByTitle(title);
    }


    public Page<JSONObject> queryList(int page, int pageSize, String title, Integer labelCode) throws JSONException {
        if (page <= 0) {
            page = 1;
        }
        if (pageSize <= 0) {
            pageSize = 15;
        }
        StringBuffer filters = new StringBuffer();
//        filters.append(" and t.del =1 ");
        if(StringUtils.isNotBlank(title)){
            filters.append(" and t.title like :title ");
        }
        if(labelCode!=null&&labelCode!=0){
            filters.append(" and t.code in(select relation_code from  wlyy_survey_label_info w where w.use_type=0 and w.label=:labelCode)");
        }
        Query queryCount = currentSession().createSQLQuery("select count(1) from wlyy_survey_templates t where 1=1 " +filters.toString());
        Query queryList = currentSession().createSQLQuery("select t.* from wlyy_survey_templates t where 1=1  " +filters.toString()).addEntity(SurveyTemplate.class);
        PageRequest pageRequest = PageRequest.of(page - 1, pageSize);
        if (!StringUtils.isEmpty(title)) {
            queryCount.setString("title","%"+title+"%");
            queryList.setString("title", "%"+title+"%");
        }
        if (labelCode!=null&&labelCode!=0) {
            queryCount.setInteger("labelCode",labelCode);
            queryList.setInteger("labelCode", labelCode);
        }
        int count = Integer.parseInt(queryCount.list().get(0).toString());
        queryList.setFirstResult(pageSize * (page - 1));
        queryList.setMaxResults(pageSize);
        List<SurveyTemplate> list = queryList.list();
        List<com.alibaba.fastjson.JSONObject> array = new ArrayList<>();

        for(SurveyTemplate surveyTemplate:list){
            com.alibaba.fastjson.JSONObject obj = com.alibaba.fastjson.JSONObject.parseObject(JSON.toJSONString(surveyTemplate));
            List<SurveyLabelInfo> surveyLabelInfos =  surveyLabelInfoService.findByUseTypeAndRelationCode(USE_BY_TEMPLATE,surveyTemplate.getCode());
            List<Integer> label = new ArrayList<>();
            List<String> labelName = new ArrayList<>();
            User user = userService.getUserByCode(surveyTemplate.getCreater());
            obj.put("createName",user==null?"":user.getName());
            for(SurveyLabelInfo surveyLabelInfo:surveyLabelInfos){
                List<SystemDict> systemDicts = systemDictDao.findByDictNameAndCode(SURVEY_LABEL,""+surveyLabelInfo.getLabel());
                label.add(surveyLabelInfo.getLabel());
                if(systemDicts!=null&&systemDicts.size()==1){
                    labelName.add(systemDicts.get(0).getValue());
                }else{
                    labelName.add("格式化字典异常");
                }
            }
            obj.put("label",StringUtils.join(label.toArray(),","));
            obj.put("labelName",StringUtils.join(labelName.toArray(),","));
            array.add(obj);
        }
        return new PageImpl(array,pageRequest,count);
    }

    /**
     * 获取模板信息
     * @param id 模板id
     * @param loadQuestion 是否加载问题 是1 否0
     * @return
     */
    public  com.alibaba.fastjson.JSONObject getTemplate(Long id,Long loadQuestion) throws JSONException {
        SurveyTemplate surveyTemplate = this.surveyTemplateDao.findById(id).get();
        if(surveyTemplate==null) throw new  RuntimeException("模板不存在！");
        com.alibaba.fastjson.JSONObject surveyTemplateJson = com.alibaba.fastjson.JSONObject.parseObject(JSON.toJSONString(surveyTemplate));
        if(loadQuestion==1l){
            List<SurveyTemplateQuestions> surveyTemplateQuestions = this.surveyTemplateQuestionService.findByTemplateCode(surveyTemplate.getCode());
            com.alibaba.fastjson.JSONArray questions = com.alibaba.fastjson.JSONArray.parseArray(JSON.toJSONString(surveyTemplateQuestions));
            for(Object question : questions){
                JSONObject questionObj = (JSONObject)question;
                List<SurveyTemplateOption> surveyTemplateOptions = this.surveyTemplateOptionService.findByTemplateCodeAndDelAndQuestionCode(surveyTemplate.getCode(),questionObj.getString("code"));
                JSONArray options = new JSONArray(surveyTemplateOptions);
                questionObj.put("options",options);
            }
            surveyTemplateJson.put("questions",questions);
        }
        List<SurveyLabelInfo> surveyLabelInfos = surveyLabelInfoService.findByUseTypeAndRelationCode(USE_BY_TEMPLATE,surveyTemplate.getCode());
        JSONArray labels = new JSONArray(surveyLabelInfos);
        surveyTemplateJson.put("labels",labels);
        return surveyTemplateJson;
    }

    @Transactional
    public void saveLabel(Long templateId,String labelJson){
        SurveyTemplate surveyTemplate = this.surveyTemplateDao.findById(templateId).get();
        if(surveyTemplate==null)throw new RuntimeException("模板不存在！");
        surveyLabelInfoService.deleteByUseTypeAndRelationCode(USE_BY_TEMPLATE,surveyTemplate.getCode());
        com.alibaba.fastjson.JSONArray labelsArray = com.alibaba.fastjson.JSONArray.parseArray(labelJson);
        ClazzReflect clazzReflect = new ClazzReflect();
        for(Object labelObj:labelsArray){
            JSONObject label = (JSONObject)labelObj;
            SurveyLabelInfo surveyLabelInfo = new SurveyLabelInfo();
            clazzReflect.formatToClazz(surveyLabelInfo,label);
            surveyLabelInfo.setRelationCode(surveyTemplate.getCode());
            surveyLabelInfo.setCode(getCode());
            surveyLabelInfoService.save(surveyLabelInfo);
        }
    }

    @Transactional
    public void deleteTemplate(Long templateId){
        SurveyTemplate surveyTemplate = this.surveyTemplateDao.findById(templateId).get();
        if(surveyTemplate==null)throw new RuntimeException("模板不存在！");
        this.surveyTemplateQuestionService.deleteByTemplateCode(surveyTemplate.getCode(),false);
        this.surveyTemplateOptionService.deleteByTemplateCode(surveyTemplate.getCode(),false);
        this.surveyLabelInfoService.deleteByUseTypeAndRelationCode(USE_BY_TEMPLATE,surveyTemplate.getCode());
        this.surveyTemplateDao.deleteById(templateId);
    }
    
    @Transactional
    public void deleteTemplateNew(Long templateId){
        SurveyTemplate surveyTemplate = this.surveyTemplateDao.findById(templateId).get();
        if(surveyTemplate==null)throw new RuntimeException("模板不存在！");
//        this.surveyTemplateQuestionService.deleteByTemplateCode(surveyTemplate.getCode(),false);
//        this.surveyTemplateOptionService.deleteByTemplateCode(surveyTemplate.getCode(),false);
//        this.surveyLabelInfoService.deleteByUseTypeAndRelationCode(USE_BY_TEMPLATE,surveyTemplate.getCode());
        this.surveyTemplateDao.deleteById(templateId);
    }
    
    @Transactional
    public void online(Long templateId) {
        SurveyTemplate surveyTemplate = this.surveyTemplateDao.findById(templateId).get();
        if(surveyTemplate==null)throw new RuntimeException("模板不存在！");
        if("0".equals(surveyTemplate.getDel())){
            this.surveyTemplateDao.onlineById(templateId);
        }else{
            this.surveyTemplateDao.deleteById(templateId);
        }
//        this.surveyTemplateQuestionService.deleteByTemplateCode(surveyTemplate.getCode(),true);
//        this.surveyTemplateOptionService.deleteByTemplateCode(surveyTemplate.getCode(),true);
//        this.surveyLabelInfoService.deleteByUseTypeAndRelationCode(USE_BY_TEMPLATE,surveyTemplate.getCode());

    }
}
