package com.yihu.customer.service.manager.twoCancers;

import com.alibaba.fastjson.JSONObject;
import  com.yihu.customer.entity.Hospital;
import  com.yihu.customer.entity.twoCancers.BreastCervicalAddress;
import  com.yihu.customer.repository.twoCancers.BreastCervicalAddressDao;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

/**
 * Created by wsl on 2022/9/21
 */

@Service
public class TwoCancersService {


    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private BreastCervicalAddressDao addressDao;

    //保存两癌筛查地址
    @Transactional(rollbackFor = Exception.class)
    public Integer saveTwoCancersAddress(String jsonData) {
        JSONObject jsonObject = JSONObject.parseObject(jsonData);
        BreastCervicalAddress breastCervicalAddress = jsonObject.toJavaObject(jsonObject,BreastCervicalAddress.class);
        addressDao.save(breastCervicalAddress);
        return 1;
    }

    //删除两癌地址
    @Transactional(rollbackFor = Exception.class)
    public void delTwoCancersAddress(Integer id) {
      addressDao.deleteById(Long.parseLong(id.toString()));
    }

    //查找
    public PageImpl<BreastCervicalAddress> findTwoCancersAddressList(Integer page, Integer rows, String hospitalName) {

        String sqlCount = " select count(id) from breast_cervical_address where 1=1 ";
        String sql = "select * from breast_cervical_address where 1=1 ";
        if (StringUtils.isNotBlank(hospitalName)) {
            sql += " and hospital_name like '%" + hospitalName + "%'";
            sqlCount += " and hospital_name like '%" + hospitalName + "%'";
        }

        sql += " order by id ASC limit "+(page-1)*rows+","+rows;
        List<BreastCervicalAddress> list = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(BreastCervicalAddress.class));

        if (page <= 0) {
            page = 1;
        }
        rows = rows<0?0:rows;
        PageRequest pageRequest = PageRequest.of(page-1 , rows);

        long count = jdbcTemplate.queryForObject(sqlCount,Long.class);
        return new PageImpl<BreastCervicalAddress>(list, pageRequest, count);
    }


    //获取所有医院
    public List<Map<String, Object>>  findHospitalList(){
        String sql = "SELECT code,`name` from dm_hospital WHERE `level` = 1 AND del = 1 ";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        return list;
    }

    public List<Hospital> getTwoCancerHospitalList(){
        String sql = " select p.* from dm_hospital p where code in (select code from system_dict where dict_name='BREAST_CERVICAL_HOSPITAL')  ";
        List<Hospital> hospitalList = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(Hospital.class));
        return hospitalList;
    }

    public List<BreastCervicalAddress> getTwoCancerAddressOne(String id) {

        String sql = "select * from breast_cervical_address where id='"+id+"' ";
        List<BreastCervicalAddress> result = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(BreastCervicalAddress.class));
        return result;
    }



    
    public JSONObject findTwoCancerCouCount(String startTime,String endTime){

        JSONObject result = new JSONObject();

        String cduSql="SELECT breast_ultrasound_inspect_org as org, count(id) count FROM breast_cdu_inspect WHERE " +
                "breast_ultrasound_inspect_time >= '"+startTime+"' " +
                "AND breast_ultrasound_inspect_time <= '"+endTime+" 23:59:59' " +
                "GROUP BY  breast_ultrasound_inspect_org ";

        String exSql = "SELECT cervical_cancer_inspect_org as org, count(id) count FROM cervical_gynecological_examination " +
                "WHERE cervical_cancer_inspect_time >= '"+startTime+"' " +
                "AND cervical_cancer_inspect_time <= '"+endTime+" 23:59:59' GROUP BY  cervical_cancer_inspect_org ";

        List<Map<String, Object>> list = jdbcTemplate.queryForList(cduSql);
        result.put("cdu",list);
        List<Map<String, Object>> list1 = jdbcTemplate.queryForList(exSql);
        result.put("ex",list1);

        return result;
    }


}