package com.yihu.customer.service.manager.user;

import  com.yihu.customer.entity.ManageFeature;
import  com.yihu.customer.entity.ManageRoleFeature;
import  com.yihu.customer.filter.RoleCache;
import  com.yihu.customer.repository.ManageFeatureDao;
import  com.yihu.customer.repository.ManageRoleFeatureDao;
import com.yihu.customer.util.Envelop;
import com.yihu.customer.util.query.BaseJpaService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * Created by yww on 2016/12/28.
 */
@Service
public class ManageFeatureService extends BaseJpaService<ManageFeature, ManageFeatureDao> {
    @Autowired
    private ManageFeatureDao featureDao;

    @Autowired
    private ManageRoleFeatureDao roleFeatureDao;

    @Autowired
    RoleCache roleCache;

    //新增（id自增长)
    @Transactional
    public boolean createFeature(ManageFeature feature) {
        ManageFeature newFeature = featureDao.save(feature);
        ManageRoleFeature roleFeature = new ManageRoleFeature();
        roleFeature.setFeatureId(newFeature.getId());
        roleFeature.setRoleId(new Long(1));//system的id固定1
        roleFeatureDao.save(roleFeature);
        roleCache.addRes(feature.getUrl());//更新缓存的整个系统的授权url
        return true;
    }

    //修改
    @Transactional
    public boolean updateFeature(ManageFeature feature) throws Exception {
        ManageFeature featureNew = featureDao.findById(feature.getId()).get();
        String oleUrl = featureNew.getUrl();
        String newUrl = feature.getUrl();
        featureNew.setCode(feature.getCode());
        featureNew.setName(feature.getName());
        featureNew.setUrl(newUrl == null ? "" : newUrl.trim());
        featureNew.setDescription(feature.getDescription());
        featureNew.setType(feature.getType());
        featureDao.save(featureNew);
        if (!StringUtils.isEmpty(oleUrl) && !StringUtils.isEmpty(newUrl) && !StringUtils.equals(oleUrl.trim(), newUrl.trim())) {
            roleCache.removeRes(oleUrl.trim());
            roleCache.addRes(newUrl.trim());
        }
        return true;
    }

    //删除（允许存在成员删除情况：同时删除成员关系，成员权限关系）
    @Transactional
    public boolean deleteUnique(Long id) {
        Envelop envelop = new Envelop();
        //删除角色权限关系
        List<Long> roleFeatureIds = roleFeatureDao.findByFeatureId(id);
        if (roleFeatureIds != null && roleFeatureIds.size() > 0) {
            for (Long roleFeatureId : roleFeatureIds) {
                roleFeatureDao.deleteById(roleFeatureId);
            }
            ;
        }
        ManageFeature feature = featureDao.findById(id).get();
        //删除角色
        featureDao.deleteById(id);
        String url = feature.getUrl();
        if (StringUtils.isEmpty(url)) {
            roleCache.removeRes(url.trim());
        }
        return true;
    }

    public ManageFeature getFeature(long id) {
        return featureDao.findById(id).get();
    }

    //查询列表（分页）
    public Page<ManageFeature> searchFeatures(String name, Integer page, Integer pageSize) throws Exception {
        if (page == null || page <= 0) {
            page = 1;
        }
        if (pageSize == null || pageSize <= 0) {
            pageSize = 15;
        }
        PageRequest pageRequest = PageRequest.of(page - 1, pageSize);
        String filters = "";
        if (!StringUtils.isEmpty(name)) {
            filters = "name?" + name + ";";
        }
        List list = search("", filters, "", page, pageSize);
        long count = 0;
        if (list != null && list.size() > 0) {
            count = getCount(filters);
        }
        return new PageImpl<ManageFeature>(list, pageRequest, count);

    }

    //查询列表（不分页）
    public List<ManageFeature> searchFeaturesNoPage(String name) throws Exception {
        String filters = "";
        if (!StringUtils.isEmpty(name)) {
            filters = "name?" + name + ";";
        }
        return search(filters);
    }

    //根据角色id联表查询权限列表
    public List<ManageFeature> findByRoleId(long roleId) {
        return featureDao.findByRoleId(roleId);
    }


    //编码是否存在查询
    public boolean isCodeExist(String code) {
        int num = featureDao.isCodeExist(code);
        return num > 0 ? true : false;
    }

    //名称是否存在查询
    public boolean isNameExist(String name, long parentId) {
        int num = featureDao.isNameExist(name, parentId);
        return num > 0 ? true : false;
    }

    public boolean isUrlExist(String url) {
        int num = featureDao.isUrlExist(url);
        return num > 0 ? true : false;
    }
}
