package com.yihu.customer.service.manager.user;

import  com.yihu.customer.entity.ManageFeature;
import  com.yihu.customer.entity.ManageRoleFeature;
import  com.yihu.customer.repository.ManageFeatureDao;
import  com.yihu.customer.repository.ManageRoleFeatureDao;
import com.yihu.customer.util.query.BaseJpaService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * Created by yww on 2016/12/28.
 */
@Service
public class ManageRoleFeatureService extends BaseJpaService<ManageRoleFeature, ManageRoleFeatureDao> {
    @Autowired
    private ManageRoleFeatureDao roleFeatureDao;

    @Autowired
    private ManageFeatureDao featureDao;


    public List<ManageRoleFeature> findByRoleId(long roleId) {
        List<ManageRoleFeature> roleFeatures = roleFeatureDao.findByRoleId(roleId);
        for (ManageRoleFeature m : roleFeatures) {
            ManageFeature feature = featureDao.findById(m.getFeatureId()).get();
            m.setFeatureName(feature.getName());
        }
        return roleFeatures;

    }

//    public List<RoleFeatureModel> findByRoleId(long roleId){
//        return roleFeatureDao.findByRoleId(roleId);
//    }

    public Page<ManageRoleFeature> getRoleUserList(String sorts, int page, int size) {
        Pageable pageable = PageRequest.of(page, size, parseSorts(sorts));
        return roleFeatureDao.findAll(pageable);
    }

    public ManageRoleFeature findRelation(long featureId, long roleId) {
        return roleFeatureDao.findRelation(featureId, roleId);
    }

    @Transactional
    public boolean deleteRoleFeatureRelationByRoleId(Long roleId) {
        Collection<ManageRoleFeature> relations = findByField("roleId", roleId);
        if (relations.size() > 0) {
            List<Long> deleteIds = new ArrayList<>();
            for (ManageRoleFeature relation : relations) {
                deleteIds.add(relation.getId());
            }
            delete(deleteIds);
        }
        return true;
    }

    @Transactional
    public boolean batchUpdateRoleFeatureRelation(Long roleId, Long[] addFeatureIds, String deleteFeatureIds) throws Exception {
        if (!StringUtils.isEmpty(deleteFeatureIds)) {
            List<ManageRoleFeature> deleteList = search("roleId=" + roleId + ";featureId=" + deleteFeatureIds);
            for (ManageRoleFeature m : deleteList) {
                delete(m.getId());
            }
        }
        if (addFeatureIds == null || addFeatureIds.length == 0) {
            return true;
        }
        List<ManageRoleFeature> relationList = findByField("roleId", roleId);
        List<Long> oldFeatureIds = new ArrayList<>();
        if (relationList != null && relationList.size() > 0) {
            for (int i = 0; i < relationList.size(); i++) {
                oldFeatureIds.add(relationList.get(i).getFeatureId());
            }
        }
        for (long featureId : addFeatureIds) {
            if (oldFeatureIds.contains(featureId)) {
                continue;
            }
            ManageRoleFeature roleFeatureRelation = new ManageRoleFeature();
            roleFeatureRelation.setRoleId(roleId);
            roleFeatureRelation.setFeatureId(featureId);
            save(roleFeatureRelation);
        }
        return true;
    }

}
