package com.yihu.customer.service.manager.user;

import  com.yihu.customer.entity.ManageRoles;
import  com.yihu.customer.repository.ManageRoleFeatureDao;
import  com.yihu.customer.repository.ManageRolesDao;
import com.yihu.customer.util.Envelop;
import com.yihu.customer.util.query.BaseJpaService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * Created by yww on 2016/12/28.
 */
@Service
public class ManageRolesService extends BaseJpaService<ManageRoles, ManageRolesDao> {
    @Autowired
    private ManageRolesDao rolesDao;

    @Autowired
    private ManageRoleFeatureDao roleFeatureDao;

    //新增（id自增长)
    @Transactional
    public Envelop createRole(ManageRoles role) {
        Envelop envelop = new Envelop();
        rolesDao.save(role);
        envelop.setSuccessFlg(true);
        return envelop;
    }

    //修改
    @Transactional
    public Envelop updateRole(ManageRoles role) throws Exception {
        Envelop envelop = new Envelop();
        ManageRoles roleNew = rolesDao.findById(role.getId()).get();
        if (roleNew == null) {
            envelop.setSuccessFlg(false);
            envelop.setErrorMsg("根据id未查询到记录！");
        }
        roleNew.setCode(role.getCode());
        roleNew.setName(role.getName());
        roleNew.setDescription(role.getDescription());
        roleNew.setType(role.getType());
        rolesDao.save(roleNew);
        envelop.setSuccessFlg(true);
        return envelop;
    }

    //删除不存在已配置人员的角色，并删除对应角色-权限数据
    @Transactional
    public boolean deleteRoleNoUser(long id, String code) {
        rolesDao.deleteById(id);
        //删除角色权限关系
        roleFeatureDao.deleteByRoleId(id);
        return true;
    }

    public ManageRoles getRole(long id) {
        return rolesDao.findById(id).get();
    }

    //查询列表（分页）
    public Page<ManageRoles> searchRoles(String searchNm, Integer page, Integer pageSize) throws Exception {
        if (page == null || page <= 0) {
            page = 1;
        }
        if (pageSize == null || pageSize <= 0) {
            pageSize = 15;
        }
        PageRequest pageRequest = PageRequest.of(page - 1, pageSize);
        String filters = "";
        if (!StringUtils.isEmpty(searchNm)) {
            filters = "name?" + searchNm + " g1;code?" + searchNm + " g1;";
        }
        List list = search("", filters, "", page, pageSize);
        long count = 0;
        if (list != null && list.size() > 0) {
            count = getCount(filters);
        }
        return new PageImpl<ManageRoles>(list, pageRequest, count);

    }

    //查询列表（不分页）
    public List<ManageRoles> searchRolesNoPage(String name) throws Exception {
        String filters = "";
        if (!StringUtils.isEmpty(name)) {
            filters = "name?" + name + ";";
        }
        return search(filters);
    }

    //编码是否存在查询
    public boolean isCodeExist(String code) {
        int num = rolesDao.isCodeExist(code);
        return num > 0 ? true : false;
    }

    //名称是否存在查询
    public boolean isNameExist(String name) {
        int num = rolesDao.isNameExist(name);
        return num > 0 ? true : false;
    }

}
