package com.yihu.customer.service.manager.user;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.code.kaptcha.impl.DefaultKaptcha;
import com.google.code.kaptcha.util.Config;
 
import  com.yihu.customer.entity.*;
import  com.yihu.customer.entity.login.UserLoginLog;
 
import  com.yihu.customer.repository.*;
import  com.yihu.customer.repository.login.UserLoginLogDao;
import com.yihu.customer.util.DateUtil;
import com.yihu.customer.util.Envelop;
import com.yihu.customer.util.MD5;
import com.yihu.customer.util.query.BaseJpaService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import sun.misc.BASE64Encoder;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;

/**
 * Created by yww on 2016/10/8.
 */
@Service
public class UserService extends BaseJpaService<User,UserDao> {
    @Autowired
    private UserDao userDao;

    @Autowired
    private ManageRoleUserDao roleUserDao;

    @Autowired
    private ManageFeatureDao featureDao;

    @Autowired
    private HospitalDao hospitalDao;

    @Autowired
    private ManageRoleFeatureDao roleFeatureDao;

    @Autowired
    private ManageDictDao manageDictDao;
    @Autowired
    SystemDictDao systemDictDao;
    @Autowired
    private ManageDictEntryDao manageDictEntryDao;
    @Autowired
    private UserLoginLogDao userLoginLogDao;

    @Autowired
    private StringRedisTemplate redisTemplate;

    private Map<String, String> captchaCache = new ConcurrentHashMap<>();
    private DefaultKaptcha captchaProducer = new DefaultKaptcha();

    public UserService() {
        Properties properties = new Properties();
        properties.put("kaptcha.textproducer.font.color", "blue");
        properties.put("kaptcha.textproducer.font.size", "45");
        properties.put("kaptcha.textproducer.char.length", "4");
        properties.put("kaptcha.textproducer.font.names", "宋体,楷体,微软雅黑");

        Config config = new Config(properties);
        captchaProducer.setConfig(config);
    }

    public User userValidation(String mobile , String pwd){
        User user = userDao.findByMobile(mobile);
        if(user == null || "1".equals(user.getStatus())){
            //手机号错误 或者账户被锁定
            return null;
        }

        String salt = user.getSalt().toString();
        String pwdMd5 = MD5.GetMD5Code(pwd + salt);
        if(StringUtils.equals(pwdMd5,user.getPassword().toString())){
            return user;
        }else{
            //密码错误 添加登录失败日志 60分钟内超过5次登录失败 就锁定账户
            Date end = new Date();
            Date begin = DateUtil.getNextMin(end,-60);
            List<UserLoginLog> loginLogs = userLoginLogDao.findByUserCode(user.getCode(),begin,end);
            if(loginLogs.size()>=5){
                user.setStatus("1");
                userDao.save(user);
            }

            UserLoginLog log = new UserLoginLog();
            log.setUserType(user.getType()+"");
            log.setUserCode(user.getCode());
            log.setType("2");
            log.setCreateTime(new Date());
            userLoginLogDao.save(log);
            return null;
        }
    }

    public JSONObject saveUserPwlimitDate(String user){
        String key = "base:authPw:user:" + user;
        JSONObject valueJson = new JSONObject();
        valueJson.put("updateTime",new Date().getTime());
        valueJson.put("isChanged",true);
        redisTemplate.opsForValue().set(key,valueJson.toJSONString());
        return valueJson;
    }

    /**
     * 判断密码是否修改
     * @param user
     * @return
     */
    public JSONObject findDoctorPwlimitDate(String user){
        String key = "base:authPw:user:" + user;
        String authPw = redisTemplate.opsForValue().get(key);
        JSONObject valueJson = null;
        if(StringUtils.isNotBlank(authPw)){

            String dictValue =  systemDictDao.findByDictNameAndCode2("pwOverDue","pwMaxOverDue");
            String code[] = dictValue.split(",");
            Integer maxDateInt = Integer.parseInt(code[0]);
            Integer remainDateInt = Integer.parseInt(code[1]);

            valueJson = JSON.parseObject(authPw);
            Long updateTimeLong = valueJson.getLong("updateTime");
            Date updateTime = new Date();
            updateTime.setTime(updateTimeLong);
            Date maxDate = DateUtil.getPreDays(updateTime,maxDateInt);

            Long betweenDate = (maxDate.getTime() - new Date().getTime())/(60*60*24*1000);
            valueJson.put("betweenDate",betweenDate);
            if(betweenDate<remainDateInt){
                valueJson.put("remaind",true);
            }else {
                valueJson.put("remaind",false);
            }
        }else {
            valueJson = new JSONObject();
            valueJson.put("isChanged",false);
        }
        return valueJson;
    }

    /**
     * 验证密码
     * @param pwd
     * @return
     */
    public boolean testPwd(String pwd){
        //4种
        String PW_PATTERN = "^(?![A-Za-z0-9]+$)(?![a-z0-9\\W]+$)(?![A-Za-z\\W]+$)(?![A-Z0-9\\W]+$)[a-zA-Z0-9\\W]{8,}$";
        //3种
        String PW_PATTERN2 = "^(?![A-Za-z0-9]+$)(?![A-Za-z\\W]+$)[a-zA-Z0-9_\\W]{8,}$";
        if(StringUtils.isBlank(pwd)){
            return false;
        }
        if(pwd.length()<8||pwd.length()>20){
            return false;
        }
        return pwd.matches(PW_PATTERN2);
    }

    public Page<User> searchList(String code,String name,String mobile,Integer page,Integer pageSize) throws Exception{
        if(page == null){
            page = 1;
        }
        if (page == null){
            pageSize = 15;
        }
        PageRequest pageRequest = PageRequest.of(page-1,pageSize);
        String filter = "";
        if(!StringUtils.isEmpty(code)){
            filter += filter + "cod?" + code + ";";
        }
        if(!StringUtils.isEmpty(name)){
            filter += filter + "name?" + name + ";";
        }
        if(!StringUtils.isEmpty(mobile)){
            filter += filter + "mobile?" + mobile + ";";
        }
        List<User> userList = search("id,code,name,mobile,type,roles,czrq,organizationId", filter, "-czrq", page, pageSize);
        long count = 0;
        if(userList !=null && userList.size()>0){
            count = getCount(filter.toString());
            for (User user : userList){
                Hospital hos = hospitalDao.findById(user.getOrganizationId()).get();
                user.setOrganizationName(hos== null?"":hos.getName());
            }
        }
        return new PageImpl<User>(userList,pageRequest,count);
    }

    public Page<User> searchUsers(String searchNm, int page, int pageSize) throws Exception {
        Envelop envelop = new Envelop();
        if (page < 0) {
            page = 1;
        }
        if (pageSize < 0) {
            pageSize = 15;
        }
        PageRequest pageRequest = PageRequest.of(page - 1, pageSize);
        Page<User> users = userDao.searchUsersByName("%" + searchNm + "%", pageRequest);
        return users;
    }

    @Transactional
    public boolean initPwd(String mobile){
        User user = userDao.findByMobile(mobile);
        if(user == null){
            //手机号错误
            return false;
        }
        String password=user.getMobile().substring(5);
        String salt= UUID.randomUUID().toString().replace("-", "");
        password = MD5.GetMD5Code("a"+password+"*"+salt);
        user.setSalt(salt);
        user.setPassword(password);
        User userNew = save(user);
        if(userNew == null){
            return false;
        }
        return true;
    }

    @Transactional
    public boolean changePwd(String mobile,String oldPwd,String newPwd){
        User user = userDao.findByMobile(mobile);
        if(user ==null){
            return false;
        }
        String oldSalt = user.getSalt().toString();
        String pwdMd5 = MD5.GetMD5Code(oldPwd + oldSalt);
        if (!StringUtils.equals(pwdMd5, user.getPassword().toString())) {
            return false;
        }
        String newSalt= UUID.randomUUID().toString().replace("-", "");
        newPwd = MD5.GetMD5Code(newPwd+newSalt);
        user.setSalt(newSalt);
        user.setPassword(newPwd);
        User userNew = save(user);
        if(userNew == null){
            return false;
        }
        return true;
    }

    //用户删除，同时删除对应角色组-人员关系
    @Transactional
    public boolean deleteUser(long id) {
        User user = userDao.findById(id).get();
        delete(id);
        roleUserDao.deleteByUserCode(user.getCode());
        return true;
    }


    @Transactional
    public User getUser(long id) {
        User user = userDao.findById(id).get();
        return user;
    }

    @Transactional
    public User getUserByCode(String code) {
        User user = userDao.findByCode(code);
        return user;
    }




    //权限相关
    //根据userCode 获取用户所有权限（所属所有角色对应的权限，去重复）
    public List<ManageFeature> getUserFeatures(String userCode) {
        List<ManageFeature> features = new ArrayList<>();
        long[] roleIds = roleUserDao.findByUserCode(userCode);
        if (roleIds.length <= 0) {
            return features;
        }
        features = featureDao.findByRoleIds(roleIds);
        return features;
    }

    //获取USER_TYPE的字典id
    public long getDictIdByName() {
        ManageDict manageDict = manageDictDao.findByName("USER_TYPE");
        if (null != manageDict) {
            return manageDict.getId();
        }
        return 0;
    }

    //获取USER_TYPE的value
    public String getValueByDictId(String code) {
        long dictId = getDictIdByName();
        if (dictId != 0) {
            String[] codes = code.split(",");
            List<ManageDictEntry> manageDictEntries = manageDictEntryDao.findByDictIdAndCodes(dictId, codes);
            if (null != manageDictEntries && manageDictEntries.size() > 0) {
                return manageDictEntries.get(0).getValue();
//                return "http://localhost:8180/admin/main";
            }
        }
        return null;
    }

    public Map<String, String> generateCaptcha() throws IOException {
        String captchaText = captchaProducer.createText();
        BufferedImage image = captchaProducer.createImage(captchaText);
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        ImageIO.write(image, "png", outputStream);
        String base64Img = new BASE64Encoder().encode(outputStream.toByteArray());
        base64Img = "data:image/jpeg;base64,"+base64Img.replaceAll("\n", "").replaceAll("\r", "");//格式化处理
        String uuid = UUID.randomUUID().toString();
        String key = "hwlyy:captcha:" + uuid;
        Map<String, String> data = new HashMap<>();
        data.put("key", uuid);
        data.put("image", base64Img);
        data.put("format", "png");

        redisTemplate.opsForValue().set(key,captchaText.toLowerCase(),5, TimeUnit.MINUTES);
        return data;
    }

    public boolean testCaptcha(String key,String imgCaptcha,boolean delflag){
        if(StringUtils.isBlank(key)||StringUtils.isBlank(imgCaptcha)){
            return false;
        }
        key = "hwlyy:captcha:" + key;
        String captchaImg = redisTemplate.opsForValue().get(key);
        if(!imgCaptcha.toLowerCase().equals(captchaImg)){
            return false;
        }
        if(delflag){
            redisTemplate.delete(key);
        }
        return true;
    }
}
