package com.yihu.customer.service.manager.wechat;

import com.yihu.customer.util.query.BaseJpaService;
import  com.yihu.customer.entity.wechat.WechatTemplateConfig;
import  com.yihu.customer.entity.wechat.WeixinTemplate;
import  com.yihu.customer.entity.WlyyAuditNotice;
import  com.yihu.customer.repository.WlyyAuditNoticeDao;
 
import  com.yihu.customer.repository.wechat.WechatTemplateConfigDao;
import  com.yihu.customer.repository.wechat.WeixinTemplateDao;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 微信模板管理业务逻辑
 * @Author: WuJunjie
 * @Date: Created in 2018/3/20 9:30
 */
@Service
public class WechatTemplateService extends BaseJpaService<WlyyAuditNotice, WlyyAuditNoticeDao> {

    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private WechatTemplateConfigDao wechatTemplateConfigDao;
    @Autowired
    private WeixinTemplateDao weixinTemplateDao;
    @Value("${wechat.accId}")
    private String accId;


    /**
     * 根据模板自定义名称和场景值查询模板消息
     *
     * @param templateName
     * @param scene
     * @param page
     * @param pageSize
     * @return
     * @throws Exception
     */
    public List<WechatTemplateConfig> searchList(String templateName, String scene, int page, int pageSize) throws Exception {
        if (page <= 0) {
            page = 1;
        }
        if (pageSize <= 0) {
            pageSize = 15;
        }
        String filters = "";
        if (!StringUtils.isEmpty(templateName)) {
            filters = "and t.template_name= '" + templateName + "' ";
        }
        if (!StringUtils.isEmpty(scene)) {
            filters += " and t.scene= '" + scene + "' ";
        }
        filters += " order by t.id asc limit " + (page-1)*pageSize +"," +pageSize;
        String sql = "SELECT t.* FROM `weixin_template_config` t WHERE t.`status` = 1 "
                + filters ;
        List<WechatTemplateConfig> WechatTemplateConfigs = jdbcTemplate.query(sql,new BeanPropertyRowMapper(WechatTemplateConfig.class));
        return WechatTemplateConfigs;
    }

    /**
     * 根据模板自定义名称和场景值查询模板消息
     *
     * @param templateName
     * @param scene
     * @return
     * @throws Exception
     */
    public int getListCount(String templateName, String scene) throws Exception {
        String filters = "";
        if (!StringUtils.isEmpty(templateName)) {
            filters = "and t.template_name= '" + templateName + "' ";
        }
        if (!StringUtils.isEmpty(scene)) {
            filters += " and t.scene= '" + scene + "' ";
        }
        filters += " order by t.id asc ";
        String sql = "SELECT t.* FROM `weixin_template_config` t WHERE t.`status` = 1 "
                + filters ;
        List<WechatTemplateConfig> WechatTemplateConfigs = jdbcTemplate.query(sql,new BeanPropertyRowMapper(WechatTemplateConfig.class));
        return WechatTemplateConfigs.size();
    }

    /**
     * 根据模板自定义名称和场景值查询模板消息详情
     *
     * @param templateName
     * @param scene
     * @return
     * @throws Exception
     */
    public List<Map<String,Object>> getTemplate(String templateName, String scene) throws Exception {
        String filters = "";
        if (!StringUtils.isEmpty(templateName)) {
            filters = " AND t.template_name= '" + templateName + "' ";
        }
        if (!StringUtils.isEmpty(scene)) {
            filters += " AND c.scene= '" + scene + "' ";
        }
        String sql = "SELECT " +
                " t.template_name templateName, " +
                " c.scene, " +
                " c.scene_description sceneDescription, " +
                " t.format, " +
                " c.`first`, " +
                " c.keyword1 , " +
                " c.keyword2, " +
                " c.keyword3, " +
                " c.keyword4, " +
                " c.keyword5, " +
                " c.keyword6 , " +
                " c.keyword7, " +
                " c.remark " +
                " FROM weixin_template t, weixin_template_config c " +
                " WHERE t.template_name = c.template_name and t.acc_id = '" + accId + "' " +
                filters;
        List<Map<String,Object>> WechatTemplateConfig = jdbcTemplate.queryForList(sql);
        return WechatTemplateConfig;
    }

    /**
     * 修改模板详细信息
     *
     * @param templateName
     * @param scene
     * @param sceneDescription
     * @param first
     * @param remark
     * @param value1
     * @param value2
     * @param value3
     * @param value4
     * @param value5
     * @param value6
     * @param value7
     * @return
     * @throws Exception
     */
    public void updateTemplate(String templateName, String scene, String sceneDescription, String first, String remark,
                               String value1, String value2, String value3, String value4, String value5, String value6, String value7) throws Exception {
//      保持数据唯一性
        WechatTemplateConfig wechatTemplateConfig = wechatTemplateConfigDao.findByScene(templateName, scene);
        if (wechatTemplateConfig != null) {
            wechatTemplateConfig.setTemplateName(templateName);
            wechatTemplateConfig.setScene(scene);
            wechatTemplateConfig.setSceneDescription(sceneDescription);
            wechatTemplateConfig.setFirst(first);
            wechatTemplateConfig.setRemark(remark);
            wechatTemplateConfig.setKeyword1(value1);
            wechatTemplateConfig.setKeyword2(value2);
            wechatTemplateConfig.setKeyword3(value3);
            wechatTemplateConfig.setKeyword4(value4);
            wechatTemplateConfig.setKeyword5(value5);
            wechatTemplateConfig.setKeyword6(value6);
            wechatTemplateConfig.setKeyword7(value7);
            wechatTemplateConfig.setStatus(1);
            if (StringUtils.isEmpty(first)) {
                wechatTemplateConfig.setFirst("");
            }

            wechatTemplateConfigDao.save(wechatTemplateConfig);
        }
    }


    /**
     * 保存模板详细信息
     *
     * @param templateName
     * @param scene
     * @param sceneDescription
     * @param first
     * @param remark
     * @param value1
     * @param value2
     * @param value3
     * @param value4
     * @param value5
     * @param value6
     * @param value7
     * @return
     * @throws Exception
     */
    public Map<String,Object> saveTemplate(String templateName, String scene, String sceneDescription, String first, String remark,
                               String value1, String value2, String value3, String value4, String value5, String value6, String value7) throws Exception {
        Map<String,Object> result = new HashMap<>();
        result.put("code",200);
        result.put("msg","保持成功！");
        WeixinTemplate template = weixinTemplateDao.findByName(accId,templateName);
        if (template == null){
            result.put("code",-1);
            result.put("msg","该名称的模板消息不存在！");
            return result;
        }
        //      保持数据唯一性
        WechatTemplateConfig wechatTemplateConfig = wechatTemplateConfigDao.findByScene(templateName, scene);
        if (wechatTemplateConfig != null) {
            result.put("code",-1);
            result.put("msg","该模板消息场景已存在！");
            return result;
        }else {
            wechatTemplateConfig = new WechatTemplateConfig();
        }
//        规范数据
        wechatTemplateConfig.setTemplateName(templateName);
        wechatTemplateConfig.setScene(scene);
        wechatTemplateConfig.setSceneDescription(sceneDescription);
        wechatTemplateConfig.setFirst(first);
        wechatTemplateConfig.setRemark(StringUtils.isNotEmpty(remark)?remark:null);
        wechatTemplateConfig.setKeyword1(StringUtils.isNotEmpty(value1)?value1:null);
        wechatTemplateConfig.setKeyword2(StringUtils.isNotEmpty(value2)?value2:null);
        wechatTemplateConfig.setKeyword3(StringUtils.isNotEmpty(value3)?value3:null);
        wechatTemplateConfig.setKeyword4(StringUtils.isNotEmpty(value4)?value4:null);
        wechatTemplateConfig.setKeyword5(StringUtils.isNotEmpty(value5)?value5:null);
        wechatTemplateConfig.setKeyword6(StringUtils.isNotEmpty(value6)?value6:null);
        wechatTemplateConfig.setKeyword7(StringUtils.isNotEmpty(value7)?value7:null);
        wechatTemplateConfig.setStatus(1);
        if (StringUtils.isEmpty(first)) {
            wechatTemplateConfig.setFirst("");
        }
        wechatTemplateConfigDao.save(wechatTemplateConfig);
        return result;
    }

}
