package com.yihu.customer.service.manager.wlyyrole;

import com.yihu.customer.util.query.BaseJpaService;
import  com.yihu.customer.entity.WlyyRole;
import  com.yihu.customer.entity.WlyyRoleArea;
import  com.yihu.customer.entity.WlyyUserRole;
import  com.yihu.customer.repository.WlyyRoleAreaDao;
import  com.yihu.customer.repository.WlyyRoleDao;
import  com.yihu.customer.repository.WlyyUserRoleDao;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by yww on 2016/12/13.
 */
@Service
public class WlyyRoleService extends BaseJpaService<WlyyRole, WlyyRoleDao> {
    @Autowired
    private WlyyRoleAreaDao roleAreaDao;
    @Autowired
    private WlyyUserRoleDao userRoleDao;
    @Autowired
    private WlyyRoleDao roleDao;
    public Page<WlyyRole> searchList(String name,Integer page,Integer pageSize) throws Exception{
        if(page == null){
            page = 1;
        }
        if (page == null){
            pageSize = 15;
        }
        PageRequest pageRequest = PageRequest.of(page-1,pageSize);
        String filter = "";
        if(!StringUtils.isEmpty(name)){
            filter += filter+"name?"+name+";";
        }
        List<WlyyRole> userList = search("", filter, "", page, pageSize);
        long count = 0;
        if(userList !=null && userList.size()>0){
            count = getCount(filter.toString());
//            for (User user : userList){
//                Hospital hos = hospitalDao.findOne(user.getOrganizationId());
//                user.setOrganizationName(hos== null?"":hos.getName());
//            }
        }
        return new PageImpl<WlyyRole>(userList,pageRequest,count);

    }

    /**
     * 获取角色所在/管辖机构
     */
    public List<String> getUserRoleOrgs(List<WlyyUserRole> userRoles) {
        if (userRoles == null || userRoles.size() == 0) {
            return new ArrayList<String>();
        }
        //遍历角色
        for (WlyyUserRole userRole : userRoles) {
            //获取区域权限
            List<WlyyRoleArea> areaList = roleAreaDao.findByRoleCode(userRole.getRole());
            if (areaList != null && areaList.size() > 0) {
                //获取最大权限区域
                List<String> areas = new ArrayList<>();
                List<String> orgs = new ArrayList<>();
                for (WlyyRoleArea areaItem : areaList) {
                    String currentArea = areaItem.getAreaCode();
                    int currentLength = currentArea.length();

                    if (currentLength == 6) //区域
                    {
                        //获取区域下的社区/医院（机构表的town为区域）


                        areas.add(currentArea);
                    } else {               //机构
                        orgs.add(currentArea);
                    }
                }

                //机构权限重复校验
                for (int m = orgs.size() - 1; m >= 0; m--) {
                    String currentOrg = orgs.get(m);
                    for (int n = 0; n < areas.size() - m - 1; n++) {
                        String shortArea = getShortArea(areas.get(n));
                        if (currentOrg.startsWith(shortArea))   //包含则移除
                        {
                            orgs.remove(m);
                        }
                    }
                }
            }
        }


        return null;
    }


    /**
     * 获取角色区域
     */
    public List<Map<String, String>> getUserRoleAndArea(String user) {
        List<Map<String, String>> re = new ArrayList<>();
        List<WlyyRole> roleList = roleDao.findUserRole(user);//用户所属角色（多个）

        if (roleList != null && roleList.size() > 0) {
            //遍历角色
            for (WlyyRole role : roleList) {
                Map<String, String> map = new HashMap<>();
                map.put("code", role.getCode());
                map.put("name", role.getName());


                //获取区域权限
                String areaString = "";
                List<WlyyRoleArea> areaList = roleAreaDao.findByRoleCode(role.getCode());
                if (areaList != null && areaList.size() > 0) {
                    //获取最大权限区域
                    List<String> areas = new ArrayList<>();
                    List<String> orgs = new ArrayList<>();
                    for (WlyyRoleArea areaItem : areaList) {
                        String currentArea = areaItem.getAreaCode();
                        int currentLength = currentArea.length();

                        if (currentLength == 6) //区域
                        {
                            areas.add(currentArea);
                        } else {               //机构
                            orgs.add(currentArea);
                        }
                    }

                    //区域权限重复校验
                    for (int i = areas.size() - 1; i >= 0; i--) {
                        String currentArea = areas.get(i);
                        for (int j = 0; j < areas.size() - i - 1; j++) {
                            String shortArea = getShortArea(areas.get(j));
                            if (currentArea.startsWith(shortArea))   //包含则移除
                            {
                                areas.remove(i);
                            }
                        }
                    }
                    //机构权限重复校验
                    for (int m = orgs.size() - 1; m >= 0; m--) {
                        String currentOrg = orgs.get(m);
                        for (int n = 0; n < areas.size() - m - 1; n++) {
                            String shortArea = getShortArea(areas.get(n));
                            if (currentOrg.startsWith(shortArea))   //包含则移除
                            {
                                orgs.remove(m);
                            }
                        }
                    }

                    for (String item : areas) {
                        areaString += item + ",";
                    }
                    for (String item : orgs) {
                        areaString += item + ",";
                    }
                    areaString = areaString.substring(0, areaString.length() - 1);
                }
                map.put("areas", areaString);
                re.add(map);
            }
        }

        return re;
    }

    /**
     * 区域去末尾零
     */
    private String getShortArea(String area) {
        char strs[] = area.toCharArray();// 将字符串转化成字符数组
        String last = "";
        for (int i = area.length() - 1; i >= 0; i--) {
            if ('0' == strs[i]) {
                last += "0";
            } else {
                break;
            }
        }
        if (last.length() > 0) {
            return area.substring(0, area.lastIndexOf(last));
        } else {
            return area;
        }
    }
}
