package com.yihu.customer.service.manager.wlyyrole;

 
 
import com.yihu.customer.service.common.excel.ExcelData;
import com.yihu.customer.service.manager.hos.HosDoctorService;
import com.yihu.customer.util.MD5;
import com.yihu.customer.util.query.BaseJpaService;
import  com.yihu.customer.entity.*;
import  com.yihu.customer.repository.*;

import com.yihu.customer.util.ReadExcelUtil;
import jxl.Sheet;
import jxl.Workbook;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.util.*;

/**
 * Created by yww on 2016/12/13.
 */
@Service
public class WlyyUserRoleService extends BaseJpaService<WlyyUserRole, WlyyUserRoleDao> {
    @Autowired
    private WlyyUserRoleDao wlyyUserRoleDao;
    @Autowired
    private WlyyRoleAreaDao roleAreaDao;
    @Autowired
    private HospitalDao hospitalDao;
    @Autowired
    private DoctorDao doctorDao;
    @Autowired
    private WlyyRoleDao roleDao;
    @Autowired
    private HospitalDeptDao hospitalDeptDao;
    @Autowired
    private HosDoctorService hosDoctorService;

    public Page<WlyyUserRole> searchList(String roleCode,Integer page,Integer pageSize) throws Exception{
        if(page <= 0){
            page = 1;
        }
        if(pageSize <= 0){
            pageSize = 15;
        }
        String filters = "";
        if(StringUtils.isEmpty(roleCode)){
            filters += "role="+roleCode+" ;";
        }
        PageRequest pageRequest = PageRequest.of(page-1,pageSize);
        List<WlyyUserRole> list = search("", filters, "", page, pageSize);
        long count = 0;
        if(list !=null && list.size()>0){
            count = getCount(list.toString());
            for (WlyyUserRole userRole : list){
                Doctor doc = doctorDao.findByCode(userRole.getUser());
                if(doc != null){
                    userRole.setUserName(doc.getName());
                }
            }
        }
        return new PageImpl<WlyyUserRole>(list,pageRequest,count);
    }

    /**
     * 获取角色区域
     */
    public List<Map<String, String>> getUserRoleAndArea(String user) {
        List<Map<String, String>> re = new ArrayList<>();
        List<WlyyRole> roleList = roleDao.findUserRole(user);

        if (roleList != null && roleList.size() > 0) {
            //遍历角色
            for (WlyyRole role : roleList) {
                Map<String, String> map = new HashMap<>();
                map.put("code", role.getCode());
                map.put("name", role.getName());
                //获取区域权限
                String areaString = "";
                List<WlyyRoleArea> areaList = roleAreaDao.findByRoleCode(role.getCode());
                if (areaList != null && areaList.size() > 0) {
                    //获取最大权限区域
                    List<String> areas = new ArrayList<>();
                    List<String> orgs = new ArrayList<>();
                    for (WlyyRoleArea areaItem : areaList) {
                        String currentArea = areaItem.getAreaCode();
                        int currentLength = currentArea.length();

                        if (currentLength == 6) //区域
                        {
                            areas.add(currentArea);
                        } else {               //机构
                            orgs.add(currentArea);
                        }
                    }

                    //区域权限重复校验
                    for (int i = areas.size() - 1; i >= 0; i--) {
                        String currentArea = areas.get(i);
                        for (int j = 0; j < areas.size() - i - 1; j++) {
                            String shortArea = getShortArea(areas.get(j));
                            if (currentArea.startsWith(shortArea))   //包含则移除
                            {
                                areas.remove(i);
                            }
                        }
                    }
                    //机构权限重复校验
                    for (int m = orgs.size() - 1; m >= 0; m--) {
                        String currentOrg = orgs.get(m);
                        for (int n = 0; n < areas.size() - m - 1; n++) {
                            String shortArea = getShortArea(areas.get(n));
                            if (currentOrg.startsWith(shortArea))   //包含则移除
                            {
                                orgs.remove(m);
                            }
                        }
                    }

                    for (String item : areas) {
                        areaString += item + ",";
                    }
                    for (String item : orgs) {
                        areaString += item + ",";
                    }
                    areaString = areaString.substring(0, areaString.length() - 1);
                }
                map.put("areas", areaString);
                re.add(map);
            }
        }

        return re;
    }

    //获取角色下辖机构集合
    public List<String> getRoleOrgCodes(String[] roleCodes) {
        List<String> orgCodes = new ArrayList<>();
        //获取区域权限
        List<WlyyRoleArea> areaList = roleAreaDao.findByRoleCodes(roleCodes);
        if (areaList != null && areaList.size() > 0) {
            //获取最大权限区域
            List<String> areas = new ArrayList<>();
            for (WlyyRoleArea areaItem : areaList) {
                String currentArea = areaItem.getAreaCode();
                if (StringUtils.equals(currentArea, "350200")) {//厦门全市
                    //获取厦门全市说有机构
                    List<String> hosOrgCodes = hospitalDao.findAllOrgCodesByCity(currentArea);
                    orgCodes.addAll(hosOrgCodes);
                    continue;
                }
                int currentLength = currentArea.length();
                if (!StringUtils.equals(currentArea, "350200") && currentLength == 6) {//区域
                    //获取区域下的社区/医院（机构表的town为区域）
                    List<String> hosOrgCodes = hospitalDao.findByOrgCodesTown(currentArea);
                    orgCodes.addAll(hosOrgCodes);
                } else {               //机构
                    orgCodes.add(currentArea);
                }
            }

            //机构权限重复校验
            for (int m = orgCodes.size() - 1; m >= 0; m--) {
                String currentOrg = orgCodes.get(m);
                for (int n = 0; n < areas.size() - m - 1; n++) {
                    String shortArea = getShortArea(areas.get(n));
                    if (currentOrg.startsWith(shortArea))   //包含则移除
                    {
                        orgCodes.remove(m);
                    }
                }
            }
        }
        return orgCodes;
    }

    /**
     * 区域去末尾零
     */
    private String getShortArea(String area) {
        char strs[] = area.toCharArray();// 将字符串转化成字符数组
        String last = "";
        for (int i = area.length() - 1; i >= 0; i--) {
            if ('0' == strs[i]) {
                last += "0";
            } else {
                break;
            }
        }
        if (last.length() > 0) {
            return area.substring(0, area.lastIndexOf(last));
        } else {
            return area;
        }
    }

    //根据user编码获取WlyyUserRole
    public List<WlyyUserRole> findByUserCode(String userCode){
        return wlyyUserRoleDao.findByUser(userCode);
    }

    public boolean importData(Workbook workbook) throws ParseException {
        Sheet[] sheets = workbook.getSheets();
        Sheet sheet = sheets[0];
        int rows = ReadExcelUtil.getRightRows(sheet);

        List<WlyyUserRole> wlyyUserRoleList = new ArrayList<>();
        for (int row = 1; row < rows; row++) {  //索引从0开始，第一行为标题

            HospitalDept hospitalDept = new HospitalDept();
            Map<Integer, ExcelData> mappingHospitalDept = mappingHospitalDept(hospitalDept);
            int finalRow = row;
            mappingHospitalDept.forEach((index, excelData) -> {
                String value = sheet.getCell(index, finalRow).getContents().trim();
                excelData.transform(value);
            });

            String hospital = hospitalDept.getHospital();

            //判断是否存在医院科室名,不存在则保存
            if(StringUtils.isNotBlank(hospital)){//存在医院,但是不存在科室名
                HospitalDept hospitalDept1 = hospitalDeptDao.findByHospitalCodeAndDeptName(hospital, hospitalDept.getName());
                if(hospitalDept1==null){
                    hospitalDept.setCode(UUID.randomUUID().toString());
                    hospitalDept = hospitalDeptDao.save(hospitalDept);
                }
            }else{//不存在医院
                hospitalDept.setCode(UUID.randomUUID().toString());
                hospitalDept = hospitalDeptDao.save(hospitalDept);
            }

            Doctor doctor = new Doctor();
            Map<Integer, ExcelData> mappingDoctor = mappingDoctor(doctor);
            mappingDoctor.forEach((index, excelData) -> {
                String value = sheet.getCell(index, finalRow).getContents().trim();
                excelData.transform(value);
            });
            Doctor doctor2 = doctorDao.findByIdCard(doctor.getIdCard());
            //判断导入的管理员是否存在doctor表中,不存在则保存
            if(doctor2==null){
                String idCard = doctor.getIdCard();
                doctor.setSex(hosDoctorService.getGenderByIdCard(idCard));
                String birthdayString = hosDoctorService.getBirthdayByIdCard(idCard);
                DateUtils.parseDate(birthdayString, "yyyy-MM-dd");
                doctor.setBirthday(DateUtils.parseDate(birthdayString, "yyyy-MM-dd"));
                String password = doctor.getMobile().substring(5);
                String salt = UUID.randomUUID().toString().replace("-", "");
                doctor.setSalt(salt);
                doctor.setPassword(MD5.GetMD5Code(password + salt));
                doctor.setCzrq(new Date());
                doctor.setPhoto("../../../images/d-male.png");
                doctor.setStatus(1);
                doctor.setIscertified(1);
                doctor.setCode(UUID.randomUUID().toString());
                doctor = doctorDao.save(doctor);
            }else{
                doctor = doctor2;
            }
            WlyyUserRole wlyyUserRole = new WlyyUserRole();
            String user = doctor.getCode();
            String role = doctor.getHospital();
            wlyyUserRole.setUser(user);
            wlyyUserRole.setRole(role);
            WlyyUserRole wlyyUserRole1 = wlyyUserRoleDao.findByUserAndRole(user,role);
            //判断权限是否存在,不存在则保存
            if(wlyyUserRole1==null){
                wlyyUserRole.setCzrq(new Date());
                wlyyUserRoleDao.save(wlyyUserRole);
            }
        }
        return true;
    }

    private Map<Integer, ExcelData> mappingHospitalDept(HospitalDept hospitalDept) {
        Map<Integer, ExcelData> dataMap = new HashMap<>();
        //所在机构
        dataMap.put(4, new ExcelData() {
            @Override
            public void transform(String data) {
                Hospital hospital = hospitalDao.findByName(data);
                if(hospital!=null){
                    hospitalDept.setHospital(hospital.getCode());
                }
            }
        });

        //所在部门
        dataMap.put(5, new ExcelData() {
            @Override
            public void transform(String data) {
                hospitalDept.setName(data);
            }
        });
        return dataMap;
    }

    private Map<Integer, ExcelData> mappingDoctor(Doctor doctor) {
        Map<Integer, ExcelData> dataMap = new HashMap<>();
        //姓名
        dataMap.put(0, new ExcelData() {
            @Override
            public void transform(String data) {
                doctor.setName(data);
            }
        });


        //手机号码
        dataMap.put(2, new ExcelData() {
            @Override
            public void transform(String data) {
                doctor.setMobile(data);
            }
        });

        //身份证号
        dataMap.put(3, new ExcelData() {
            @Override
            public void transform(String data){
                doctor.setIdCard(data);

            }
        });

        //所在机构
        dataMap.put(4, new ExcelData() {
            @Override
            public void transform(String data) {
                doctor.setHospitalName(data);
                Hospital hospital = hospitalDao.findByName(data);
                if (hospital != null) {
                    doctor.setHospital(hospital.getCode());
                }
            }
        });

        //所在部门
        dataMap.put(5, new ExcelData() {
            @Override
            public void transform(String data) {
                doctor.setDeptName(data);
                String hospital = doctor.getHospital();
                HospitalDept hospitalDept = hospitalDeptDao.findByHospitalCodeAndDeptName(hospital,data);
                if(hospitalDept==null){
                    HospitalDept hospitalDept2 = new HospitalDept();
                    hospitalDept2.setHospital(hospital);
                    hospitalDept2.setDel("1");
                    hospitalDept2.setName(data);
                    hospitalDept2.setCode(UUID.randomUUID().toString());
                    hospitalDept = hospitalDeptDao.save(hospitalDept2);
                }
                doctor.setDept(hospitalDept.getCode());
            }
        });

        //职称
        dataMap.put(6, new ExcelData() {
            @Override
            public void transform(String data) {
                doctor.setJobName(data);
            }
        });
        //管理级别
        dataMap.put(7, new ExcelData() {
            @Override
            public void transform(String data) {
                doctor.setLevel(10);
            }
        });
        return dataMap;
    }
}
