package com.yihu.customer.service.satisfaction;

import com.alibaba.fastjson.JSONObject;
import  com.yihu.customer.entity.Patient;
import  com.yihu.customer.entity.satisfaction.SatisfactionSurveyDetail;
import  com.yihu.customer.entity.satisfaction.SatisfactionSurveyPlan;
import  com.yihu.customer.repository.PatientDao;
import  com.yihu.customer.repository.UserDao;
import  com.yihu.customer.repository.satisfaction.SatisfactionSurveyDetailDao;
import  com.yihu.customer.repository.satisfaction.SatisfactionSurveyPlanDao;
import com.yihu.customer.util.DateUtil;
import com.yihu.customer.util.IdCardUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * @author xyq
 * @create 2022-07-18 10:22
 * @slogan 他化自在，我自逍遥
 */
@Service
@Transactional
public class SatisfactionService {

    private static Logger logger = LoggerFactory.getLogger(SatisfactionService.class);

    @Autowired
    private SatisfactionSurveyPlanDao planDao;
    @Autowired
    private SatisfactionSurveyDetailDao detailDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private PatientDao patientDao;
    @Autowired
    private UserDao userDao;

    /**
     *
     * @param id
     * @param name
     * @param rangType 调查范围，0集美区，1社区
     * @param rangCode
     * @param filterType 筛选类型，0随机筛选，2按团队平均筛选(每个团队都需要有人)
     * @param filterNumber
     * @param userCode
     * @param beginTime
     * @param endTime
     * @param remark
     * @return
     */
    //创建或编辑调查计划
    @Transactional(rollbackFor = Exception.class)
    public List<SatisfactionSurveyPlan> addOrUpdatePlan(Long id,String name,Integer rangType,String rangCode,Integer filterType,Integer filterNumber,String userCode,String beginTime,String endTime,String remark){
        List<SatisfactionSurveyPlan> plans = new ArrayList<>();
        String setRangCode = "";
        String setRangName = "";

        List<String> hospitalArray = new ArrayList<>();
        List<String> teamArray = new ArrayList<>();
        List<String> patientList = new ArrayList<>();
        Integer searchTotal = 0;
        if(rangType == 0){//集美区
            setRangCode = "350211";
            setRangName = "集美区";
            if (filterType==0){//随机筛选
                String sql = "select f.* from ( SELECT patient from wlyy_sign_family where status = 1 and expenses_status = '1' and hospital like '350211%' ) f inner join wlyy_patient p on f.patient = p.code" +
                        " where p.status = 1 and p.mobile is not null and p.mobile != '' GROUP BY p.mobile order by RAND() limit "+filterNumber;
                patientList = jdbcTemplate.queryForList(sql,String.class);
            }
            if(filterType == 2){//团队
                String sql = "SELECT CAST(id as char) id from wlyy_admin_team where org_code like '350211%' and available = 1 ";
                teamArray = jdbcTemplate.queryForList(sql, String.class);
            }
        }else {//指定社区
            hospitalArray = Arrays.asList(rangCode.split(","));
            String hospitalSql = "SELECT GROUP_CONCAT(name) FROM dm_hospital where code in ('"+String.join("','", hospitalArray)+"')";
            setRangName = jdbcTemplate.queryForObject(hospitalSql, String.class);
            setRangCode = rangCode;
            if (filterType==0){//随机筛选
               String sql = "select f.* from ( SELECT patient from wlyy_sign_family where status = 1 and expenses_status = '1' and hospital in ('"+String.join("','",hospitalArray)+"') ) f inner join wlyy_patient p on f.patient = p.code" +
                        " where p.status = 1 and p.mobile is not null and p.mobile != '' GROUP BY p.mobile order by RAND() limit "+filterNumber;
                patientList = jdbcTemplate.queryForList(sql,String.class);
            }
            if(filterType == 2){ //按团队平均筛选
                String sql = "SELECT CAST(id as char) id from wlyy_admin_team where org_code in ('"+String.join("','",hospitalArray)+"' )  and available = 1 ";
                teamArray = jdbcTemplate.queryForList(sql, String.class);
            }
        }
        if(filterType == 2){//按团队平均筛选
            for (String teamId : teamArray) {
                String sql = "select f.* from ( SELECT patient from wlyy_sign_family where status = 1 and expenses_status = '1' and admin_team_code = '"+teamId+"' ) f inner join wlyy_patient p on f.patient = p.code" +
                        " where p.status = 1 and p.mobile is not null and p.mobile != '' GROUP BY p.mobile order by RAND() limit "+filterNumber;
                patientList.addAll(jdbcTemplate.queryForList(sql,String.class));
            }
        }
        searchTotal = patientList.size();
        if(id != null){
            planDao.deleteById(Long.parseLong(id.toString()));
            detailDao.deleteByPlanId(Integer.parseInt(id.toString()));
        }
        String[] users = userCode.split(",");
        Integer averageSize = searchTotal/users.length;
        if (averageSize==0){
            averageSize = searchTotal%users.length;
        }
        Integer start = 0;
        Integer end = averageSize;
        for (String tmp:users){//平均分配给客服
            if (start>=patientList.size()){
                break;
            }
            SatisfactionSurveyPlan plan = new SatisfactionSurveyPlan();
            plan.setName(name);
            plan.setRangType(rangType);
            plan.setRangName(setRangName);
            plan.setRangCode(setRangCode);
            plan.setFilterType(filterType);
            plan.setFilterNumber(patientList.subList(start,end).size());
            plan.setUserCode(tmp);
            plan.setUserName(userDao.findNameByCode(tmp));
            plan.setBeginTime(StringUtils.isBlank(beginTime)?null:DateUtil.strToDateLong(beginTime));
            plan.setEndTime(StringUtils.isBlank(endTime)?null:DateUtil.strToDateLong(endTime));
            plan.setRemark(remark);
            plan.setCreateTime(new Date());
            planDao.save(plan);
            plans.add(plan);
            this.createPlanDetail(plan,patientList.subList(start,end));
            start +=averageSize;
            end +=averageSize;
            if (end>patientList.size()||end+averageSize>patientList.size()){
                end=patientList.size();
            }

        }
        return plans;
    }

    /**
     * 获取筛查数量
     * @return
     */
    public Integer getSearchTotal(Integer rangType,String  rangCode,Integer filterType,Integer filterNumber){
        List<String> hospitalArray = new ArrayList<>();
        List<String> teamArray = new ArrayList<>();
        List<String> patientList = new ArrayList<>();
        Integer searchTotal = 0;
        if(rangType == 0){//集美区
            if (filterType==0){//随机筛选
                String sql = "select f.* from ( SELECT patient from wlyy_sign_family where status = 1 and expenses_status = '1' and hospital like '350211%' ) f inner join wlyy_patient p on f.patient = p.code" +
                        " where p.status = 1 and p.mobile is not null and p.mobile != '' GROUP BY p.mobile order by RAND() limit "+filterNumber;
                patientList = jdbcTemplate.queryForList(sql,String.class);
            }
            if(filterType == 2){//团队
                String sql = "SELECT CAST(id as char) id from wlyy_admin_team where org_code like '350211%' and available = 1 ";
                teamArray = jdbcTemplate.queryForList(sql, String.class);
            }
        }else {//指定社区
            hospitalArray = Arrays.asList(rangCode.split(","));
            String hospitalSql = "SELECT GROUP_CONCAT(name) FROM dm_hospital where code in ('"+String.join("','", hospitalArray)+"')";
            if (filterType==0){//随机筛选
                String sql = "select f.* from ( SELECT patient from wlyy_sign_family where status = 1 and expenses_status = '1' and hospital in ('"+String.join("','",hospitalArray)+"') ) f inner join wlyy_patient p on f.patient = p.code" +
                        " where p.status = 1 and p.mobile is not null and p.mobile != '' GROUP BY p.mobile order by RAND() limit "+filterNumber;
                patientList = jdbcTemplate.queryForList(sql,String.class);
            }
            if(filterType == 2){ //按团队平均筛选
                String sql = "SELECT CAST(id as char) id from wlyy_admin_team where org_code in ('"+String.join("','",hospitalArray)+"' )  and available = 1 ";
                teamArray = jdbcTemplate.queryForList(sql, String.class);
            }
        }
        if(filterType == 2){//按团队平均筛选
            for (String teamId : teamArray) {
                String sql = "select f.* from ( SELECT patient from wlyy_sign_family where status = 1 and expenses_status = '1' and admin_team_code = '"+teamId+"' ) f inner join wlyy_patient p on f.patient = p.code" +
                        " where p.status = 1 and p.mobile is not null and p.mobile != '' GROUP BY p.mobile order by RAND() limit "+filterNumber;
                patientList.addAll(jdbcTemplate.queryForList(sql,String.class));
            }
        }
        return patientList.size();
    }


    /**
     * 创建满意度计划详情
     * @param plan
     * @param patientCodeList
     * @return
     */
    public void createPlanDetail(SatisfactionSurveyPlan plan,List<String> patientCodeList){
        if(CollectionUtils.isNotEmpty(patientCodeList)){
            String patientSql = "SELECT p.*,f.doctor,f.doctor_name,f.hospital,f.hospital_name from  " +
                    " (SELECT code,name,mobile,address,sex from wlyy_patient where code in ('"+String.join("','",patientCodeList)+"') and mobile is not null and mobile != '' ) p " +
                    " left join wlyy_sign_family f on p.code = f.patient and f.status =1 and f.expenses_status = '1' group by p.code ";
            List<Map<String, Object>> patientList = jdbcTemplate.queryForList(patientSql);
            List<SatisfactionSurveyDetail> detailsList = new ArrayList<>();
            for (Map<String, Object> map : patientList) {
                SatisfactionSurveyDetail d = new SatisfactionSurveyDetail();
                d.setPlanId(Integer.parseInt(plan.getId().toString()));
                d.setPatientCode(map.get("code").toString());
                d.setPatientName(map.get("name")!=null?map.get("name").toString():null);
                d.setSex(map.get("sex")!=null?Integer.parseInt(map.get("sex").toString()):0);
                d.setMobile(map.get("mobile").toString());
                d.setSignOrgCode(map.get("hospital")!=null?map.get("hospital").toString():null);
                d.setSignOrgName(map.get("hospital_name")!=null?map.get("hospital_name").toString():null);
                d.setDoctor(map.get("doctor")!=null?map.get("doctor").toString():null);
                d.setDoctorName(map.get("doctor_name")!=null?map.get("doctor_name").toString():null);
                d.setAddress(map.get("address")!=null?map.get("address").toString():null);
                d.setReturnUserCode(plan.getUserCode());
                d.setReturnUserName(plan.getUserName());
                d.setCreateTime(new Date());
                detailsList.add(d);
            }
            if(CollectionUtils.isNotEmpty(detailsList)){
                detailDao.saveAll(detailsList);
            }
        }
    }

    /**
     * 获取社区列表
     * @return
     */
    public List<Map<String, Object>> getHospitalList(){
        String sql = "select '350211' as code,'集美区' as name union all " +
                " select code,name from dm_hospital where town = '350211' and del = '1' and level = 2 ";
        return jdbcTemplate.queryForList(sql);
    }

    /**
     * 获取客服列表
     * @return
     */
    public List<Map<String, Object>> getSysUserList(){
        String sql = "SELECT code,name from wlyy_user where type = 4 ";
        return jdbcTemplate.queryForList(sql);
    }

    /**
     * 获取调查计划列表
     * @param name
     * @param userName
     * @param rangCode
     * @param beginTime
     * @param endTime
     * @param page
     * @param pageSize
     * @return
     */
    public JSONObject getPlanList(String name, String userName, String rangCode, String beginTime, String endTime, Integer page, Integer pageSize,String userCode){
        JSONObject jsonObject = new JSONObject();
        if(page != null && pageSize != null){
            page = page>0?page-1:0;
        }
        String colSql = "SELECT p.id,p.name,p.rang_type rangType,p.rang_code rangCode,p.rang_name rangName,p.filter_type filterType, " +
                " p.filter_number filterNumber,p.user_code userCode,p.user_name userName,CAST(DATE_FORMAT(p.begin_time,'%Y-%m-%d %H:%i:%S') as char) beginTime,CAST(DATE_FORMAT(p.end_time,'%Y-%m-%d %H:%i:%S') as char) endTime,p.remark,CAST(DATE_FORMAT(p.create_time,'%Y-%m-%d %H:%i:%S') as char) createTime, " +
                " count(d.id) overCount from wlyy_satisfaction_survey_plan p " +
                " left join wlyy_satisfaction_survey_detail d on p.id = d.plan_id  " +
                " and d.is_on is not null and d.is_know_doctor is not null " +
                " and d.is_introduce_doctor is not null and d.is_satisfy_doctor is not null ";
        String countSql = "SELECT count(p.id) from wlyy_satisfaction_survey_plan p ";
        String filterSql = " where 1=1 ";
        if(StringUtils.isNotBlank(name)){
            filterSql += " and p.name like '%"+name+"%' ";
        }
        if(StringUtils.isNotBlank(userName)){
            filterSql += " and p.user_name like '%"+userName+"%' ";
        }
        if(StringUtils.isNotBlank(rangCode)){
            filterSql += " and (p.rang_code = '"+rangCode+"' or p.rang_code like '"+rangCode+",%' or p.rang_code like '%,"+rangCode+"' or p.rang_code like '%,"+rangCode+",%') ";
        }
        if(StringUtils.isNotBlank(beginTime) ){
            filterSql += " and p.begin_time >= '"+beginTime+"' ";
        }
        if(StringUtils.isNotBlank(endTime)){
            filterSql += " and p.begin_time <= '"+endTime+"' ";
        }
        if(StringUtils.isNotBlank(userCode)){
            filterSql += " and p.user_code = '"+userCode+"' ";
        }
        Long total = jdbcTemplate.queryForObject(countSql + filterSql, Long.class);
        if(page != null && pageSize != null){
            filterSql += " GROUP BY p.id limit " + page * pageSize + "," + pageSize;
        }
        List<Map<String, Object>> list = jdbcTemplate.queryForList(colSql + filterSql);
        jsonObject.put("list",list);
        jsonObject.put("page",page);
        jsonObject.put("total",total);
        jsonObject.put("pageSize",page);
        return jsonObject;
    }

    /**
     * 根据计划id获取详情列表，分页
     * @param planId
     * @return
     */
    public JSONObject getPlanDetailListByPlanId(Integer planId,String hospital,String mobile,String name,Integer mobileRepeat,Integer page,Integer pageSize){
        JSONObject jsonObject = new JSONObject();
        page = page>0?page-1:0;

        String sqlCount = " select count(id) from wlyy_satisfaction_survey_detail  ";
        String sql = "SELECT id, plan_id planId, patient_code patientCode, patient_name patientName, sex, mobile, sign_org_code signOrgCode, sign_org_name signOrgName,  " +
                " doctor, doctor_name doctorName, address, is_on isOn, is_know_doctor isKnowDoctor, is_introduce_doctor isIntroduceDoctor, is_satisfy_doctor isSatisfyDoctor, return_user_code returnUserCode, return_user_name returnUserName, remark, CAST(DATE_FORMAT(create_time,'%Y-%m-%d %H:%i:%S') as char) createTime " +
                " from wlyy_satisfaction_survey_detail sd ";

        String condition = " where plan_id = "+planId+" " ;

        if (StringUtils.isNotBlank(hospital)){
            condition += " and sign_org_code='"+hospital+"' ";
        }
        if (StringUtils.isNotBlank(mobile)){
            condition += " and mobile like '%"+mobile+"%' ";
        }
        if (StringUtils.isNotBlank(name)){
            condition += " and patient_name like '%"+name+"%' ";
        }
        if (1==mobileRepeat){//之查询出手机重复的数据
            condition += " and mobile in ( select DISTINCT mobile from wlyy_satisfaction_survey_detail dd " +condition+
                    " GROUP BY mobile HAVING count(mobile)>1  ) ";
        }

        sql =sql+condition+" order by is_on asc limit "+page*pageSize+","+pageSize;
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        Long count = jdbcTemplate.queryForObject(sqlCount+condition,Long.class);
        for (Map<String, Object> tmp:list){
            try {
                String patient = tmp.get("patientCode").toString();
                Patient patientDO = patientDao.findByCode(patient);
                if (null!=patientDO){
                    if (StringUtils.isNotBlank(patientDO.getIdcard())){
                        tmp.put("age",IdCardUtil.getAgeForIdcard(patientDO.getIdcard())+"");
                    }
                }else {
                    tmp.put("age","");
                }
            }catch (Exception e){
            }

        }

        jsonObject.put("list",list);
        jsonObject.put("page",page);
        jsonObject.put("total", count);
        jsonObject.put("pageSize",pageSize);
        return jsonObject;
    }

    public String getPlanName(Long id){
        return planDao.findNameById(id);
    }

    /**
     * 导出调查列表
     * @param planId
     * @return
     */
    public List<Map<String, Object>> getPlanDetailListByPlanIdAndOver(Long planId){
        String sql = "SELECT *,IF(sex is null,'未知',IF(sex = 1,'男','女')) sexStr,CASE is_know_doctor WHEN 0 THEN '否' WHEN 1 THEN '是' WHEN 2 THEN '已告知 'ELSE is_know_doctor END as 'isKnowDoctorStr' ,if(is_introduce_doctor = 0,'否','是') isIntroduceDoctorStr from wlyy_satisfaction_survey_detail  " +
                " where plan_id = " + planId +
                " and is_on is not null and is_know_doctor is not null and is_introduce_doctor is not null and is_satisfy_doctor is not null ";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        return list;
    }


    /**
     * 编辑一条调查记录
     * @param id
     * @param mobile
     * @param address
     * @param isOn
     * @param isKnowDoctor
     * @param isIntroduceDoctor
     * @param isSatisfyDoctor
     * @param remark
     * @return
     */
    public SatisfactionSurveyDetail updatePlanDetailById(Long id,String mobile,String address,Integer isOn,Integer isKnowDoctor,Integer isIntroduceDoctor,Integer isSatisfyDoctor,String remark){
        SatisfactionSurveyDetail one = detailDao.findById(id).get();
        if(one != null){
            if(StringUtils.isNotBlank(mobile) && !mobile.equals(one.getMobile())){
                //电话修改，更新用户表
                patientDao.updateMobile(mobile,one.getPatientCode());
                one.setMobile(mobile);
            }
            one.setAddress(address);
            one.setIsOn(isOn);
            one.setIsKnowDoctor(isKnowDoctor);
            one.setIsIntroduceDoctor(isIntroduceDoctor);
            one.setIsSatisfyDoctor(isSatisfyDoctor);
            one.setRemark(remark);
            detailDao.save(one);
            return one;
        }
        return null;
    }

}
