package com.yihu.customer.service.synergy;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.customer.util.query.BaseJpaService;
import  com.yihu.customer.entity.User;
import  com.yihu.customer.entity.synergy.KnowledgeArticleDO;
import  com.yihu.customer.repository.UserDao;
import  com.yihu.customer.repository.synergy.KnowledgeArticleDao;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * Created by wxw on 2018/12/26.
 *
 * @author wxw.
 */
@Service
@Transactional
public class KnowledgeArticleService extends BaseJpaService<KnowledgeArticleDO, KnowledgeArticleDao> {

    @Autowired
    private KnowledgeArticleDao articleDao;
    @Autowired
    private ObjectMapper mapper;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private UserDao userDao;

    public KnowledgeArticleDO findById(Long id) {
        KnowledgeArticleDO one = articleDao.findById(id).get();

        if (null != one) {
            // 获取机构数列表 按市-区-机构展示
            String applyOrg = one.getApplyOrg();
            List<Map<String, Object>> list = hospitalsTree(applyOrg);
            one.setOrgTree(list);
            if (StringUtils.isNotEmpty(applyOrg)) {
                one.setSelected(selectedOrgs(applyOrg));
            }
        }
        return one;
    }

    public KnowledgeArticleDO queryOneById(Long id) {
        KnowledgeArticleDO one = articleDao.findById(id).get();
        User user = userDao.findByCode(one.getPatient());
        one.setName(user.getName());
        return one;
    }

    private List<Map<String, Object>> selectedOrgs(String applyOrg) {
        List<Map<String, Object>> listMap = new ArrayList<>();
        String sql = "SELECT code as id,`name` from dm_hospital WHERE city='350200' and `level`=2 and del='1' and center_site='00' ORDER BY town";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        List<String> orgList = Arrays.asList(applyOrg.split(","));
        list.stream().forEach(item -> {
            if (orgList.contains(String.valueOf(item.get("id")))) {
                listMap.add(item);
            }
        });
        return listMap;
    }

    /**
     * 组装机构树（市-区-机构）
     * @param applyOrg
     * @return
     */
    public List<Map<String, Object>> hospitalsTree(String applyOrg) {
        // 查询厦门市全部机构
        String sql = "SELECT town,code,`name` from dm_hospital WHERE city='350200' and `level`=2 and del='1' and center_site='00' GROUP BY town,code ORDER BY town";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        List<Map<String, Object>> listMapParent = new LinkedList<>();
        List<Map<String, Object>> listMap = new LinkedList<>();
        List<String> orgList = new ArrayList<>();
        if (StringUtils.isNotEmpty(applyOrg)) {
            orgList = Arrays.asList(applyOrg.split(","));
        }
        Map<String, Object> siMing = new HashMap<>();
        siMing.put("id", "350203");
        siMing.put("level", "1");
        siMing.put("name", "思明区");
        siMing.put("parentId", "350200");
        siMing.put("children", getChildInfo(list, "350203", orgList));
        listMap.add(siMing);

        Map<String, Object> huLi = new HashMap<>();
        huLi.put("id", "350206");
        huLi.put("level", "1");
        huLi.put("name", "湖里区");
        huLi.put("parentId", "350200");
        huLi.put("children", getChildInfo(list, "350206", orgList));
        listMap.add(huLi);

        Map<String, Object> haiCang = new HashMap<>();
        haiCang.put("id", "350205");
        haiCang.put("level", "1");
        haiCang.put("name", "海沧区");
        haiCang.put("parentId", "350200");
        haiCang.put("children", getChildInfo(list, "350205", orgList));
        listMap.add(haiCang);

        Map<String, Object> jiMei = new HashMap<>();
        jiMei.put("id", "350211");
        jiMei.put("level", "1");
        jiMei.put("name", "集美区");
        jiMei.put("parentId", "350200");
        jiMei.put("children", getChildInfo(list, "350211", orgList));
        listMap.add(jiMei);

        Map<String, Object> tongAn = new HashMap<>();
        tongAn.put("id", "350212");
        tongAn.put("level", "1");
        tongAn.put("name", "同安区");
        tongAn.put("parentId", "350200");
        tongAn.put("children", getChildInfo(list, "350212", orgList));
        listMap.add(tongAn);

        Map<String, Object> xiangAn = new HashMap<>();
        xiangAn.put("id", "350213");
        xiangAn.put("level", "1");
        xiangAn.put("name", "翔安区");
        xiangAn.put("parentId", "350200");
        xiangAn.put("children", getChildInfo(list, "350213", orgList));
        listMap.add(xiangAn);

        Map<String, Object> parent = new HashMap<>();
        parent.put("id", "350200");
        parent.put("level", "0");
        parent.put("name", "厦门市");
        parent.put("children", listMap);
        listMapParent.add(parent);

        return listMapParent;
    }

    public List<Map<String, Object>> getChildInfo(List<Map<String, Object>> mapList, String code, List<String> orgList) {
        List<Map<String, Object>> child = new ArrayList<>();
        mapList.stream().forEach(item -> {
            if (code.equals(item.get("town"))) {
                item.put("id", item.get("code"));
                item.put("level", "2");
                item.put("name", item.get("name"));
                item.put("parentId", code);
                item.put("flag", orgList.contains(String.valueOf(item.get("code"))) ? true : false);
                child.add(item);
            }
        });
        return child;
    }

    /**
     * 删除，支持批量
     * @param id
     */
    public boolean delArticle(String id){
        String[] split = id.split(",|，");
        String ids = StringUtils.join(split, ",");
        String sql = "update wlyy_knowledge_article a set a.del = 0 where a.id in(" + ids + ")";
        int update = jdbcTemplate.update(sql);
        if (update > 0) {
            return true;
        }
        return false;
    }

    /**
     * 新增、保存客服文章
     * @param json
     */
    public void updateArticle(String json) throws Exception {
        KnowledgeArticleDO knowledgeArticleDO = mapper.readValue(json, KnowledgeArticleDO.class);
        if (null == knowledgeArticleDO.getId()) {
            // 新增
            knowledgeArticleDO.setCode(UUID.randomUUID().toString().replace("-", ""));
            knowledgeArticleDO.setCzrq(new Date());
            knowledgeArticleDO.setSource("客服");
            knowledgeArticleDO.setDel(1);
            articleDao.save(knowledgeArticleDO);
        } else {
            KnowledgeArticleDO one = articleDao.findById(knowledgeArticleDO.getId()).get();
            one.setTitle(knowledgeArticleDO.getTitle());
            one.setCategory(knowledgeArticleDO.getCategory());
            one.setContent(knowledgeArticleDO.getContent());
            one.setImage(knowledgeArticleDO.getImage());
            one.setUpdateTime(new Date());
            one.setApplyOrg(knowledgeArticleDO.getApplyOrg());
            one.setApplyOrgName(knowledgeArticleDO.getApplyOrgName());
            articleDao.save(one);
        }
    }
}
