package com.yihu.customer.service.wechat;

import com.yihu.customer.util.HttpUtil;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Trick on 2018/6/28.
 */
@Service
public class WechatApiService {

    private static Logger logger = LoggerFactory.getLogger(WechatApiService.class);

    @Value("${wechat.appId}")
    private String appId;
    @Value("${wechat.appSecret}")
    private String appSecret;


    public List<String> getOpenIdList(String access_token) throws JSONException {
        //微信开发者文档地址：https://mp.weixin.qq.com/wiki?t=resource/res_main&id=mp1421140840
        /**
         * 返回成功格式：
         * {"total":2,
               "count":2,
               "data":{
               "openid":["OPENID1","OPENID2"]},
               "next_openid":"NEXT_OPENID"
            }
         */
        /**
         * 返回失败格式：
         *
         * {"errcode":40013,"errmsg":"invalid appid"}
         */
        List<String> list = new ArrayList<>();
        String openidList_url = "https://api.weixin.qq.com/cgi-bin/user/get";
        String params ="access_token="+ access_token +"&next_openid=";
        String result = HttpUtil.sendGet(openidList_url, params);
        JSONObject json = new JSONObject(result);
        if (json.has("data")) {
            JSONObject jsonObject = json.getJSONObject("data");
            JSONArray array = jsonObject.getJSONArray("openid");

            for(int i=0;i< array.length();i++){
                String openid = (String) array.get(i);
                list.add(openid);
            }
            String nextOpenid = json.getString("next_openid");
            //递归调用获取openid并存储
            if(StringUtils.isNotBlank(nextOpenid)){
                nextOpenIDList(nextOpenid,access_token,list);
            }
        }
        return list;
    }

    public void nextOpenIDList(String nextOpenid ,String accessToken,List<String> list) throws JSONException {
        String openidList_url = "https://api.weixin.qq.com/cgi-bin/user/get";
        String params ="access_token="+ accessToken +"&next_openid="+nextOpenid;
        String result = HttpUtil.sendGet(openidList_url, params);
        JSONObject json = new JSONObject(result);
        if (json.has("data")) {
            JSONObject jsonObject = json.getJSONObject("data");
            JSONArray array = jsonObject.getJSONArray("openid");
            for(int i=0;i< array.length();i++){
                String openid = (String) array.get(i);
                list.add(openid);
            }
            String next = json.getString("next_openid");
            //递归调用获取openid并存储
            if(StringUtils.isNotBlank(next)){
                nextOpenIDList(next,accessToken,list);
            }
        }
    }


    /**
     *
     * @param accesstoken
     * @param openidList 每次传入的openid列表个数不能超过50个
     * @param tagId
     */
    public String batchtagging(String accesstoken ,List<String> openidList,String tagId) throws JSONException {
        Map<String,Object> map = new HashMap<>();
        map.put("openid_list",openidList);
        map.put("tagid",tagId);
        String params = com.alibaba.fastjson.JSONObject.toJSONString(map);
        String batchTagging_url = "https://api.weixin.qq.com/cgi-bin/tags/members/batchtagging?access_token=" + accesstoken + "&lang=zh_CN";
        String result = HttpUtil.sendPost(batchTagging_url, params);
        org.json.JSONObject json = new org.json.JSONObject(result);
        String code = json.get("errcode").toString();
        String errmsg = json.get("errmsg").toString();
        if (code.equals("0")) {
            return "success";
        } else {
            return errmsg;
        }
    }

    /**
     * 最多可以创建100个标签。
     * 一次一个
     * @param accesstoken
     * @param tag
     * @return
     */
    public String createTagging(String accesstoken ,String tag) throws JSONException {
        Map<String,Object> map = new HashMap<>();
        Map<String,Object> tagMap = new HashMap<>();
        tagMap.put("name",tag);
        map.put("tag",tagMap);
        String params = com.alibaba.fastjson.JSONObject.toJSONString(map);
        String batchTagging_url = "https://api.weixin.qq.com/cgi-bin/tags/create?access_token=" + accesstoken + "&lang=zh_CN";
        String result = HttpUtil.sendPost(batchTagging_url, params);
        org.json.JSONObject json = new org.json.JSONObject(result);//{   "tag":{ "id":134,//标签id "name":"广东"   } }
        if (json.has("tag")) {
            org.json.JSONObject returnTag = (org.json.JSONObject) json.get("tag");
            return  returnTag.get("id").toString();
        } else {
            return null;
        }
    }

    /**
     * 获取微信标签
     * @param accesstoken
     * @return
     */
    public Map<String,String> getWeiXinTaggs(String accesstoken) throws UnsupportedEncodingException, JSONException {
        Map<String,String> tagMap = new HashMap<>();
        String batchTagging_url = "https://api.weixin.qq.com/cgi-bin/tags/get?access_token=" + accesstoken + "&lang=zh_CN";
        String result = HttpUtil.sendGet(batchTagging_url, "");
        org.json.JSONObject json = new org.json.JSONObject(result);
        if (json.has("tags")) {
            org.json.JSONArray arry = json.getJSONArray("tags");
            if(arry != null && arry.length() > 0){
                for(int i = 0 ; i < arry.length() ; i++){
                    org.json.JSONObject jsonMap = (org.json.JSONObject) arry.get(i);
                    tagMap.put(jsonMap.get("name").toString(), jsonMap.get("id").toString());
                }
            }
        }
        return  tagMap;
    }
}
