package com.yihu.customer.util;

import io.swagger.annotations.Api;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.concurrent.LinkedBlockingQueue;

@Controller
@RequestMapping(value = "/pushMsgTask", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(description = "消息推送")
public class PushMsgTask {

    private static Logger logger = LoggerFactory.getLogger(PushMsgTask.class);
    // 最大容量为50的数组堵塞队列
    private static LinkedBlockingQueue<JSONObject> queue = new LinkedBlockingQueue<JSONObject>();

    private static PushMsgTask instance;

    private static Object lock = new Object();

    public static PushMsgTask getInstance() {
        synchronized (lock) {
            if (instance == null) {
                instance = new PushMsgTask();
                instance.run();
            }
        }
        return instance;
    }

    @RequestMapping(value = "aa")
    @ResponseBody
    public String test() {
        return "1111";
    }

    /**
     * 添加一条推送消息
     *
     * @param receiver 接收人
     * @param type     消息类型
     * @param title    消息标题
     * @param msg      消息内容
     * @param data     消息数据
     */
    public void put(String receiver, String type, String title, String msg, String data) {
        try {
            JSONObject json = new JSONObject();
            json.put("receiver", receiver);
            json.put("type", type);
            json.put("title", title);
            json.put("msg", msg);
            json.put("data", data);
            queue.put(json);
        } catch (Exception e) {
            logger.error("添加到消息队列失败！", e);
            e.printStackTrace();
        }
    }

    /**
     * 添加微信消息
     *
     * @param type
     * @param data
     */
    public void putWxMsg(String access_token, int type, String openid, String name, JSONObject data) {
        try {
            JSONObject json = new JSONObject();
            json.put("wx", true);
            json.put("access_token", access_token);
            json.put("type", type);
            json.put("openid", openid);
            json.put("name", name);
            json.put("data", data);
            queue.put(json);
            String str = "";
        } catch (Exception e) {
            logger.error("添加到微信消息列队列失败！", e);
            e.printStackTrace();
        }
    }

    public void put(JSONArray array) throws JSONException {
        if (array == null || array.length() == 0) {
            return;
        }
        for (int i = 0; i < array.length(); i++) {
            JSONObject json = array.getJSONObject(i);
            if (json == null) {
                continue;
            }
            try {
                queue.put(json);
            } catch (Exception e) {
                logger.error("批量添加到消息队列失败！", e);
            }
        }
    }

    private void run() {
        new Thread(new ConsumerTask()).start();
    }

    // 消费者
    class ConsumerTask implements Runnable {
        @Override
        public void run() {
            try {
                while (true) {
                    // 如果queue为空，则当前线程会堵塞，直到有新数据加入
                    JSONObject json = queue.take();
                    // 推送平台消息
                    String receiver = json.has("receiver") ? json.getString("receiver") : "";
                    String type = json.has("type") ? json.getString("type") : "";
                    String title = json.has("title") ? json.getString("title") : "";
                    String msg = json.has("msg") ? json.getString("msg") : "";
                    String data = json.has("data") ? json.getString("data") : "";
                    HttpUtil httpUtil = new HttpUtil();
                    boolean res = httpUtil.pushMessage(receiver, type, title, msg, data);
                    if (res) {
                        logger.info("消息推送成功！");
                    } else {
                        logger.error("消息推送失败！");
                    }
                }
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }


}