package com.yihu.jw.quota.etl.save;

import com.yihu.jw.quota.etl.model.EsConfig;
import com.yihu.jw.quota.vo.SaveModel;
import net.sf.json.JSONObject;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.bulk.BulkResponse;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * Created by janseny on 2018/5/23.
 */
public class LargDataWithRunnable implements Runnable {

    private Logger logger = LoggerFactory.getLogger(LargDataWithRunnable.class);
    @Resource(name="restHighLevelClient")
    private RestHighLevelClient restHighLevelClient;
    private String jsonConfig;
    private List<SaveModel> list;//待处理数据
    private int threadCount = 0;//初始化线程数
    private int flag = 1;       //这是第几个线程
    private int perCount = 10000;//每个线程处理的数据量
    private int totalCount = 0;//待处理数据总数量
    private int havedCount = 0;//已经处理的数据量

    public LargDataWithRunnable(List<SaveModel> saveModels, String jsonConfig){
        this.list = saveModels;
        int count  = saveModels.size()/perCount;
        int remainder = saveModels.size()%perCount;
        if(remainder != 0){
            count ++;
        }
        this.threadCount = count;
        this.totalCount = list.size();
        this.jsonConfig = jsonConfig;
    }

    @Override
    public void run() {
        List<SaveModel> sublist = null;
        while(totalCount - havedCount > 0){//线程会循环执行，直到所有数据都处理完
            synchronized(this){//在分包时需要线程同步，避免线程间处理重复的数据
                if(totalCount-havedCount != 0) {
                    sublist = list.subList(perCount*(flag-1), totalCount - havedCount > perCount ? perCount*flag : perCount*(flag-1) + (totalCount - havedCount));
                    flag = flag+1;
                    havedCount = sublist.size() + havedCount;
                    logger.debug("这是第" + (flag-1) +"个线程；数据 = "+ sublist.size());
                    System.out.println(Thread.currentThread().getName()+"这是第" + (flag-1) +"个线程；数据 = "+ sublist.size());
                }
                if(sublist != null) {
                    //此处为数据处理（简单打印 ）
                    BulkResponse br = null;
                    boolean isSuccessed = false;
                    try {
                        //得到链接
                        EsConfig esConfig = (EsConfig) JSONObject.toBean(JSONObject.fromObject(jsonConfig), EsConfig.class);
                        BulkRequest bulkRequest = new BulkRequest();
                        for (SaveModel obj : sublist) {
                            obj.setCreateTime( new Date());
                            IndexRequest indexRequest = new IndexRequest(esConfig.getIndex());
                            indexRequest.source(obj);
                            bulkRequest.add(indexRequest);
                        }
                        br = restHighLevelClient.bulk(bulkRequest, RequestOptions.DEFAULT);
                        //关闭链接
                        restHighLevelClient.close();
                        isSuccessed = br.hasFailures();
                    }catch (Exception e){
                        throw new RuntimeException("ES 保存数据异常");
                    }
                }
            }
        }

    }


    public List<SaveModel> getList() {
        return list;
    }

    public void setList(List<SaveModel> list) {
        this.list = list;
    }

    public int getThreadCount() {
        return threadCount;
    }

    public void setThreadCount(int threadCount) {
        this.threadCount = threadCount;
    }

    public int getFlag() {
        return flag;
    }

    public void setFlag(int flag) {
        this.flag = flag;
    }

    public int getPerCount() {
        return perCount;
    }

    public void setPerCount(int perCount) {
        this.perCount = perCount;
    }

    public int getTotalCount() {
        return totalCount;
    }

    public void setTotalCount(int totalCount) {
        this.totalCount = totalCount;
    }

    public int getHavedCount() {
        return havedCount;
    }

    public void setHavedCount(int havedCount) {
        this.havedCount = havedCount;
    }

    public String getJsonConfig() {
        return jsonConfig;
    }

    public void setJsonConfig(String jsonConfig) {
        this.jsonConfig = jsonConfig;
    }

}
