package com.yihu.jw.quota.etl.save.es;

import com.alibaba.fastjson.JSON;
import com.yihu.jw.quota.etl.Contant;
import com.yihu.jw.quota.etl.model.EsConfig;
import com.yihu.jw.quota.etl.save.LargDataWithRunnable;
import com.yihu.jw.quota.vo.SaveModel;
import net.sf.json.JSONObject;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.bulk.BulkResponse;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.common.xcontent.XContentType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

;

/**
 * Created by chenweida on 2017/6/2.
 */
@Component
@Scope("prototype")
public class ElastricSearchSave {

    private Logger logger = LoggerFactory.getLogger(ElastricSearchSave.class);
    private EsConfig esConfig;
    @Resource(name="restHighLevelClient")
    private RestHighLevelClient restHighLevelClient;


    public Boolean saveByMoreThred(List<SaveModel> saveModels, String jsonConfig) {
        boolean isSuccessed = true;
        try {
            int perCount = Contant.compute.perCount;
            if(saveModels.size() > perCount){
                int count  = saveModels.size()/perCount;
                int remainder = saveModels.size()%perCount;
                if(remainder != 0){
                    count ++;
                }
                for(int i=0; i< count; i++){
                    int totalCount = saveModels.size();
                    int start = 0;
                    int end = perCount - 1;
                    if( i!=0 ){
                        start = i * perCount;
                        if((i + 1) * perCount >= totalCount){
                            end = totalCount-1;
                        }else {
                            end = (i + 1) * perCount-1;
                        }
                    }
                    logger.info("data save 这是第" + (i+1) + "个线程；数据 = " + start+ " - " + end);
                    List<SaveModel> list = saveModels.subList(start, end);
                    LargDataWithRunnable dataWithRunnable = new LargDataWithRunnable(list,jsonConfig);
                    Thread thread = new Thread(dataWithRunnable);
                    thread.start();
                }
            }else {
                save(saveModels,jsonConfig);
            }
        } catch (Exception e) {
            throw new RuntimeException("ES 保存数据异常");
        }
        return  isSuccessed;
    }

    public Boolean save(List<SaveModel> smss, String jsonConfig) {
        BulkResponse br = null;
        boolean isSuccessed = false;
        try {
            int perCount = Contant.compute.perCount;
            //初始化参数
            esConfig = (EsConfig) JSONObject.toBean(JSONObject.fromObject(jsonConfig), EsConfig.class);
            if(smss.size() > perCount){
                int count  = smss.size()/perCount;
                int remainder = smss.size()%perCount;
                if(remainder != 0){
                    count ++;
                }
                for(int i = 1;i<= count ;i++){
                    List<SaveModel> newList = null;
                    if(i == count){
                        newList = smss.subList(perCount*(i-1),smss.size() -1);
                    }else {
                        newList = smss.subList(perCount*(i-1),perCount*i);
                    }
                    //得到链接
                    BulkRequest bulkRequest = new BulkRequest();
                    for (SaveModel obj : newList) {
                        obj.setCreateTime( new Date());
                        IndexRequest indexRequest = new IndexRequest(esConfig.getIndex());
                        indexRequest.source(JSON.toJSONString(obj), XContentType.JSON);
                        bulkRequest.add(indexRequest);
                    }
                    br = restHighLevelClient.bulk(bulkRequest, RequestOptions.DEFAULT) ;
                    //关闭链接
//                    restHighLevelClient.close();
                    isSuccessed = br.hasFailures();
                }
            }else{
                //得到链接
                BulkRequest request = new BulkRequest();
                request.timeout(TimeValue.timeValueSeconds(10));
                for (SaveModel obj : smss) {
                    obj.setCreateTime( new Date());
                    IndexRequest indexRequest = new IndexRequest(esConfig.getIndex());
                    indexRequest.source(JSON.toJSONString(obj), XContentType.JSON);
                    request.add(indexRequest);
                }
                br = restHighLevelClient.bulk(request,RequestOptions.DEFAULT);
                //关闭链接
//                restHighLevelClient.close();
                isSuccessed = br.hasFailures();
                if(!isSuccessed){
                    logger.info("es保存失败日志"+br.buildFailureMessage());
                }
            }
            return isSuccessed;
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException("ES 保存数据异常"+ e.getMessage());
        }
    }



}
