package com.yihu.jw.quota.job;

import com.yihu.jw.entity.ehr.quota.TjQuotaDataSource;
import com.yihu.jw.entity.ehr.quota.TjQuotaLog;
import com.yihu.jw.quota.dao.jpa.TjQuotaLogDao;
import com.yihu.jw.quota.etl.Contant;
import com.yihu.jw.quota.etl.extract.ExtractHelper;
import com.yihu.jw.quota.etl.model.EsConfig;
import com.yihu.jw.quota.etl.save.SaveHelper;
import com.yihu.jw.quota.etl.util.ElasticsearchUtil;
import com.yihu.jw.quota.service.source.TjDataSourceService;
import com.yihu.jw.quota.util.SpringUtil;
import com.yihu.jw.quota.vo.QuotaVo;
import com.yihu.jw.quota.vo.SaveModel;
import com.yihu.jw.util.date.DateUtil;
import net.sf.json.JSONObject;
import org.elasticsearch.index.query.*;
import org.joda.time.DateTime;
import org.joda.time.LocalDate;
import org.quartz.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.web.context.support.SpringBeanAutowiringSupport;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * Created by chenweida on 2017/6/6.
 */
@Component
@Scope("prototype")
@DisallowConcurrentExecution//防止到了执行时间点前一任务还在执行中，但是这时有空闲的线程，那么马上又会执行，这样一来就会存在同一job被并行执行
public class EsQuotaJob implements Job {
    private Logger logger = LoggerFactory.getLogger(EsQuotaJob.class);

    private String saasid; // saasid
    private QuotaVo quotaVo = new QuotaVo(); // 指标对象
    private String endTime; // 结束时间
    private String startTime; //开始时间
    private String timeLevel; //时间
    private String dataLevel;//1 数据增量 2数据到达量
    private String year;//要统计的年份
    private Date quotaDate;//统计的时间
    private String executeFlag; // 执行动作 1 手动执行 2 周期执行
    private int haveThreadCount = 0;//已完成线程数
    private int threadCount = 1;//总线程数

    @Autowired
    private TjQuotaLogDao tjQuotaLogDao;
    @Autowired
    private ExtractHelper extractHelper;
    @Autowired
    ElasticsearchUtil elasticsearchUtil;
    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Autowired
    private TjDataSourceService dataSourceService;

    @Override
    public void execute(JobExecutionContext context) throws JobExecutionException {
        TjQuotaLog tjQuotaLog = new TjQuotaLog();
        String time = "";
        try {
            //springz注入
            SpringBeanAutowiringSupport.processInjectionBasedOnCurrentContext(this);
            //初始化参数
            initParams(context);
            quotaVo.setExecuteFlag(executeFlag);
            logger.warn("开始执行指标" + quotaVo.getCode());
            tjQuotaLog.setQuotaCode(quotaVo.getCode());
            tjQuotaLog.setSaasId(saasid);
            tjQuotaLog.setStartTime(new Date());
            tjQuotaLog.setStatus( Contant.save_status.executing);  //指标执行中
            tjQuotaLog.setContent( "时间：" + startTime + "到"+ endTime +" , " + "任务执行中。");
            tjQuotaLog = saveLog(tjQuotaLog);
            time = "时间：" + startTime + "到"+ endTime +" , ";
            TjQuotaDataSource quotaDataSource = dataSourceService.findSourceByQuotaCode(quotaVo.getCode());
            if (quotaDataSource == null) {
                throw new Exception("数据源配置错误");
            }
            JSONObject obj = new JSONObject().fromObject(quotaDataSource.getConfigJson());
            EsConfig esConfig = (EsConfig) JSONObject.toBean(obj,EsConfig.class);
            //查询是否已经统计过,如果已统计 先删除后保存
            deleteRecord(quotaVo,dataLevel);
            if(quotaDataSource.getSourceCode().equals("2") && esConfig.getAggregation()!= null && esConfig.getAggregation().equals("list")) {//来源solr
               /* moreThredQuota(tjQuotaLog,esConfig);*/
            }else{
                //统计并保存
                quota(tjQuotaLog, quotaVo,dataLevel);
            }
        } catch (Exception e) {
            //如果出錯立即重新執行
            tjQuotaLog.setStatus(Contant.save_status.fail);
            tjQuotaLog.setEndTime(new Date());
            tjQuotaLog.setContent( time+"统计异常," + e.getMessage());
            saveLog(tjQuotaLog);
            JobExecutionException e2 = new JobExecutionException(e);
            e2.setRefireImmediately(true);
            e.printStackTrace();
        }
    }

    /*
     * solr list 方式 多线程执行指标
     *//*
    public void moreThredQuota(TjQuotaLog tjQuotaLog, EsConfig esConfig){
        try {
            int perCount = Contant.compute.perCount;
            quotaVo.setStart(0);
            quotaVo.setRows(perCount);
            int rows = solrExtract.getExtractTotal(startTime,endTime, esConfig);
            if(rows > perCount*50){
                throw new Exception("数据量过大请缩小抽取时间范围");
            }
            if (rows > perCount) {
                int count  = rows/perCount;
                int remainder = rows % perCount;
                if(remainder != 0){
                    count ++;
                }else {
                    remainder = perCount;
                }
                threadCount = count;
                for (int i = 0; i < count; i++) {
                    //防止过快执行导致参数被覆盖
                    Thread.sleep(1000);
                    final int f = i;//传值用。
                    final TjQuotaLog quotaLogf = tjQuotaLog;//传值用。
                    final QuotaVo quotaVof = quotaVo;//传值用。
                    if (f != 0){
                        quotaVof.setStart(f*perCount);
                    }else {
                        quotaVof.setStart(0);
                    }
                    if(i+1 == count){
                        quotaVof.setRows(remainder);
                    }else {
                        quotaVof.setRows(perCount);
                    }
                    Thread th = new Thread(new Thread(){
                        public synchronized void run(){
                            logger.warn("启动第 "+ (f+1) + " 个线程。 ");//只能访问外部的final变量。
                            quota(quotaLogf, quotaVof);
                        }
                    });
                    Thread.sleep(10000);//延迟10 秒 Es 保存2万条 有时超时，延迟执行减缓个线程同时执行的压力
                    th.start();
                }
            }else {
                //统计并保存
                quota(tjQuotaLog, quotaVo);
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }*/

    /**
     * 统计过程
     */
    public void quota(TjQuotaLog tjQuotaLog,QuotaVo quotaVo,String dataLevel) {
        String time = "1".equals(dataLevel)? ("时间：" + startTime + "到"+ endTime +" , "):"时间："+ endTime +" , ";
        Integer status = 0;
        String content = "";
        try {
            //抽取数据
            List<SaveModel> dataModels = extract(quotaVo);
            if (dataModels != null && dataModels.size() > 0) {
                //保存数据
                Boolean success = saveData(dataModels,quotaVo);
                status = success ? Contant.save_status.success : Contant.save_status.fail;
                content = success ? time+"统计保存成功" : time+"统计保存失败";
                logger.info(content + dataModels.size());
                haveThreadCount++;
            } else {
                status = Contant.save_status.success;
                content = "没有抽取到数据" + time ;
                haveThreadCount++;
            }
            // 初始执行时，更新该指标为已初始执行过
            if (quotaVo.getExecuteFlag().equals("1")) {
                String sql = "UPDATE tj_quota SET is_init_exec = '1' WHERE id = " + quotaVo.getId();
                jdbcTemplate.update(sql);
            }
        } catch (Exception e) {
            haveThreadCount++;
            tjQuotaLog.setStatus(Contant.save_status.fail);
            tjQuotaLog.setContent(e.getMessage());
            tjQuotaLog = saveLog(tjQuotaLog);
            e.printStackTrace();
        }
        if(threadCount > 1){
            if(haveThreadCount  == threadCount){
                tjQuotaLog.setStatus(Contant.save_status.success);
                tjQuotaLog.setContent(time+"统计保存成功");
                logger.warn("指标" + tjQuotaLog.getQuotaCode() + "统计成功 结束！");
            }else {
                tjQuotaLog.setStatus(Contant.save_status.fail);
                tjQuotaLog.setContent( time+"统计保存失败");
            }
            tjQuotaLog.setEndTime(new Date());
            saveLog(tjQuotaLog);
        }else {
            tjQuotaLog.setStatus(status);
            tjQuotaLog.setContent(content);
            tjQuotaLog.setEndTime(new Date());
            saveLog(tjQuotaLog);
            logger.warn("结束！" + content);
        }
    }

    private void deleteRecord(QuotaVo quotaVo,String dataLevel) throws Exception {
        EsConfig esConfig = extractHelper.getEsConfig(quotaVo.getCode());
        EsConfig sourceEsConfig = extractHelper.getDataSourceEsConfig(quotaVo.getCode());

        String start = "";
        String end = "";
        if(sourceEsConfig.getFullQuery() !=null && sourceEsConfig.getFullQuery().equals("true")){
            start = LocalDate.now().toString();
            end = start;
        }else {
            if (!StringUtils.isEmpty(startTime)) {
                start = startTime;
            }
            if (!StringUtils.isEmpty(endTime)) {
                end = endTime;
            }
        }
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        RangeQueryBuilder rangeQuotaTime = QueryBuilders.rangeQuery("quotaDate").gte(start.substring(0, 10)).lte(end.substring(0, 10));
        RangeQueryBuilder rangeCreateTime = QueryBuilders.rangeQuery("createDate").gte(start.substring(0, 10) + "T00:00:00+0800").lte(end.substring(0, 10) + "T23:59:59+0800");
        String quotaCodeTerm = "quotaCode:" + quotaVo.getCode().replaceAll("_", "");
        QueryStringQueryBuilder termQueryQuotaCode = QueryBuilders.queryStringQuery(quotaCodeTerm);
        QueryBuilder qb = QueryBuilders.boolQuery()
                .must(termQueryQuotaCode)
                .must(QueryBuilders.matchQuery("timeLevel", dataLevel))
                .must(QueryBuilders.boolQuery()
                        .should(rangeQuotaTime)
                        .should(rangeCreateTime)
                );
        boolQueryBuilder.filter(qb);

        boolean flag = true ;
        try {
            while (flag){
                long count = elasticsearchUtil.getTotalCount(esConfig.getIndex() , boolQueryBuilder);
                if(count != 0){
                    boolean successFlag = elasticsearchUtil.queryDelete(esConfig.getIndex() ,boolQueryBuilder);
                    if(!successFlag){
                        throw  new Exception("Elasticsearch 指标统计时原始数据删除失败");
                    }
                }else {
                    flag = false ;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw  new Exception("Elasticsearch 指标统计时原始数据删除异常");
        } finally {
            logger.debug(quotaVo.getCode()+" delete success");
        }
    }

    /**
     * 抽取数据
     * @return
     */
    private List<SaveModel> extract(QuotaVo quotaVo) throws Exception {
        return SpringUtil.getBean(ExtractHelper.class).extractData(quotaVo, startTime, endTime, timeLevel,dataLevel, saasid);
    }

    /**
     * 初始化参数
     * @param context
     */
    private void initParams(JobExecutionContext context) throws Exception{
        JobDataMap map = context.getJobDetail().getJobDataMap();
        Map<String, Object> params = context.getJobDetail().getJobDataMap();
        Object object = map.get("quota");
        if (object != null) {
            BeanUtils.copyProperties(object, this.quotaVo);
        }

        this.saasid = map.getString("saasid");
        this.dataLevel = map.getString("dataLevel");
        // 默认按天，如果指标有配置时间维度，ES抽取过程中维度字典项转换为 SaveModel 时再覆盖。
        this.timeLevel = map.get("timeLevel")!=null?String.valueOf(map.get("timeLevel")):Contant.main_dimension_timeLevel.day;

        this.executeFlag = map.getString("executeFlag");
        if ("2".equals(executeFlag)) {
            if (StringUtils.isEmpty(map.getString("startTime"))) {
                startTime = Contant.main_dimension_timeLevel.getStartTime(timeLevel);
            } else {
                this.startTime = map.getString("startTime").split("T")[0] + " 00:00:00";
            }
            if (StringUtils.isEmpty(map.getString("endTime"))) {
                endTime = LocalDate.now().toString("yyyy-MM-dd 00:00:00");
            } else {
                this.endTime = map.getString("endTime").split("T")[0] + " 23:59:59";
            }
        }else{
            if (StringUtils.isEmpty(endTime)) {
                endTime = new LocalDate(new DateTime().minusDays(1)).toString("yyyy-MM-dd") + " 23:59:59"; //2017-06-01 17:00:00
            } else if (endTime.length()==10){
                endTime = endTime + " 23:59:59";
            }
            if("2".equals(timeLevel)){
                endTime =  DateUtil.getSundayOfThisWeek(DateUtil.strToDateLong(endTime));
            }else if("3".equals(timeLevel)){
                endTime=  DateUtil.getLastDayOfMonthThisDate((DateUtil.strToDateLong(endTime)));
            }else if("4".equals(timeLevel)){
                endTime=  DateUtil.getLastDayOfYearThisDate((DateUtil.strToDateLong(endTime)));
            }
            //初始化统计年份
            this.year = getNowYearByDate(endTime);
            //初始化开始时间
            if ("2".equals(dataLevel)) {
                //按年度到达量
                startTime = this.year + "-01-01 23:59:59";
            } else {
                //增量
                if (StringUtils.isEmpty(startTime)) {
                    getStartTime();
                } else {
                    startTime = startTime + " 23:59:59";
                }
            }
        }
        this.quotaDate = DateUtil.strToDate(endTime);
    }

    @Transactional
    TjQuotaLog saveLog(TjQuotaLog tjQuotaLog) {
        TjQuotaLog log =  tjQuotaLogDao.save(tjQuotaLog);
        return  log;
    }

    /**
     * 获取现在时间属于那个年度
     *
     * @param date
     * @return
     */
    public static String getNowYearByDate(String date) throws Exception {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date today = simpleDateFormat.parse(date);
        String todayString = simpleDateFormat.format(today);


        String startDateString = (1900 + today.getYear()) + "-01-01";
        Date startDate = simpleDateFormat.parse(startDateString);
        if (simpleDateFormat.parse(todayString).after(startDate)) {
            return (1900 + today.getYear()) + "";
        } else {
            return (1900 + today.getYear() - 1) + "";
        }
    }

    public void getStartTime(){
        if("1".equals(this.timeLevel)){//日
            startTime = new LocalDate(new DateTime().minusDays(1)).toString("yyyy-MM-dd") + " 00:00:00";
        }else if("2".equals(this.timeLevel)){//周
            startTime = DateUtil.getMondayOfThisWeek(DateUtil.strToDateLong(endTime))+ " 00:00:00";
        }else if("3".equals(this.timeLevel)){//月
            startTime =DateUtil.getFristDayOfMonthThisDate(DateUtil.strToDateLong(endTime))+ " 00:00:00";
        }else if("4".equals(this.timeLevel)){//年
            startTime = DateUtil.getFirstDayOfYear(DateUtil.strToDateLong(endTime))+ " 00:00:00";
        }
    }

    /**
     * 保存数据
     *
     * @param dataModels
     */
    private Boolean saveData(List<SaveModel> dataModels,QuotaVo quotaVo) {
        try {
            return SpringUtil.getBean(SaveHelper.class).save(dataModels, quotaVo);
        } catch (Exception e) {
            logger.error("save error:" + e.getMessage());
        }
        return false;
    }
}
