package com.yihu.rehabilitation.controller.consult;

import com.yihu.jw.entity.base.im.ConsultTeamDo;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.rehabilitation.dao.SignFamilyDao;
import com.yihu.rehabilitation.service.consult.ConsultTeamService;
import com.yihu.rehabilitation.util.BusinessLogs;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.http.MediaType;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 患者端：三师咨询控制类
 *
 * @author George
 */
@Controller
@RequestMapping(value = "/patient/consult", produces = MediaType.APPLICATION_JSON_UTF8_VALUE,method = {RequestMethod.GET,RequestMethod.POST})
@Api(description = "患者端-患者咨询")
public class ConsultController extends EnvelopRestEndpoint {

    @Autowired
    private ConsultTeamService consultTeamService;

    /**
     * 三师咨询添加接口
     *
     * @param type     咨询类型：1三师咨询，2家庭医生咨询,18康复咨询
     * @param when     发病时间
     * @param symptoms 主要症状
     * @param images   图片URL地址，多图以逗号分隔
     * @param voice    语音URL地址
     * @return
     */
    @RequestMapping(value = "add")
    @ResponseBody
    @ApiOperation("三师咨询添加接口")
    public Envelop add(@RequestParam(required = false) Integer type,
                       @RequestParam(required = false) String when,
                       @RequestParam String symptoms,
                       @RequestParam(required = false) String images,
                       @RequestParam(required = false) String voice,
                       @RequestParam(required = false) Long guidance,
                       @RequestParam(required = false) String doctor) {
        try {
            System.out.println("guidance1="+guidance);
            System.out.println("symptoms="+symptoms);
            if (type == null) {
                type = 1;
            }
            if (type != 1 && type != 2 && type !=18&&(StringUtils.isNoneBlank(doctor)&&type==18)) {
                return failed( "无效请求！");
            }

            ConsultTeamDo consult = new ConsultTeamDo();
            // 设置咨询类型：1三师咨询，2家庭医生咨询  6.名医咨询  18,康复咨询
            consult.setType(type);
            // 设置发病时间
            consult.setWhen(when);
            // 设置主要症状
            consult.setSymptoms(symptoms);
            // 设置咨询图片URL
            consult.setImages(images);
            // 设置咨询语音URL
            consult.setVoice(voice);
            consult.setDoctor(doctor);
            consult.setIsAuthentication(0);
            consult.setIsRefinement(0);
            // 设置关联咨询
            if (guidance != null && guidance > 0) {
                consult.setGuidance(guidance);
            }

            System.out.println("guidance2="+guidance);
            // 保存到数据库
            int res = 0;
            JSONArray dts = null;
            synchronized (getUID().intern()){//新增同步方法。设备保存写在service层但是不生效，写在controller层才生效
                JSONObject re;
                if(type==18){//康复咨询
                    re = consultTeamService.addRecoverConsult(consult,getRepUID(),getUID());
                }
                else{
                    re = consultTeamService.addTeamConsult(consult,getRepUID(), getUID());
                }
                res = re.getInt("status");
                dts = re.has("doctor")?re.getJSONArray("doctor"):null;
            }
            if (res == -1) {
                return failed( "家庭签约信息不存在或已过期，无法进行家庭医生咨询！");
            } else if (res == -2) {
                return failed("家庭签约信息不存在或已过期，无法进行三师医生咨询！");
            }else if (res == -3) {
                return failed( "还有咨询未结束，不允许再次提交咨询！");
            }

//            Doctor doctor = doctorService.findDoctorByCode(consult.getDoctor());
            //创建咨询讨论组
            // JSONObject jo= talkGroupService.createConsultTalkGroup(consult.getDoctor(), doctor.getName(), doctor.getHospital(), doctor.getHospitalName(),
            //        consult.getPatient(), consult.getName(), consult.getConsult());
            // consult.setGroupCode(jo.getString("groupCode"));
            //更新讨论组
            //consultTeamService.save(consult);

            BusinessLogs.info(BusinessLogs.BusinessType.consult, getRepUID(), getUID(), new JSONObject(consult));
            return success("提交成功",  consult);
        } catch (Exception ex) {
            ex.printStackTrace();
            return failedException(ex);
        }
    }


    @RequestMapping(value = "intoTopic",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation("进入咨询")
    public Envelop intoTopic(@RequestParam(required = true) String consult){
        try{
            int result = consultTeamService.intoTopic(consult,getRepUID(),getUID());
            if(result==-1){
                return failed("该咨询不是进行中");
            }
            JSONObject json = new JSONObject();
            json.put("consult",consult);
            json.put("content","进入咨询");
            BusinessLogs.info(BusinessLogs.BusinessType.consult, getRepUID(), getUID(), json);
            return success("进入成功");
        }catch (Exception e){
            e.printStackTrace();
            return failed("请求失败");
        }
    }

    /**
     * 查询与某个医生是否存在未结束的咨询
     *
     * @param doctor 医生
     * @return
     */
    @ApiOperation("查询与某个医生是否存在未结束的咨询")
    @RequestMapping(value = "/is_consult_unfinished",method = RequestMethod.POST)
    @ResponseBody
    public Envelop isExistsUnfinishedConsult(@RequestParam(required = false) String patient,
                                             @RequestParam(required = true) String doctor) {
        try {
            JSONObject result = new JSONObject();
            List<Map<String,Object>> consults = consultTeamService.getUnfinishedConsult(StringUtils.isEmpty(patient) ? getUID() : patient, doctor);

            if (consults != null && consults.size() > 0) {
                return success("查询成功",  consults.get(0).get("consult"));
            } else {
                return failed( "查询成功" );
            }
        } catch (Exception e) {
            return failedException(e);
        }
    }
    /**
     * 获取患者发起咨询时可选择的专科医生团队列表，
     * @param patientCode 患者code
     * @return
     */
    @RequestMapping(value = "getConsultDoctorForConsulting",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation("获取咨询医生团队列表列表")
    public Envelop getConsultDoctorForConsulting(@RequestParam("patientCode")String patientCode){
        try {
            return success("查询成功",consultTeamService.getConsultDoctorForConsulting(patientCode));
        }catch (Exception e){
            e.printStackTrace();
            return failedException(e);
        }
    }

    /**
     * 查询患者三师咨询咨询列表
     *
     * @param status   咨询状态（0未结束，1已结束，-1 已取消）
     * @param pagesize 页数
     * @return 查询结果
     */
    @RequestMapping(value = "list")
    @ResponseBody
    @ApiOperation("查询患者三师咨询咨询列表")
    public Envelop list(int status, String id, int pagesize) {
        try {
            Page<ConsultTeamDo> consults = consultTeamService.findByPatient(getRepUID(), status, id, pagesize);
            JSONArray jsonArray = new JSONArray();
            if (consults != null) {
                for (ConsultTeamDo consult : consults) {
                    if (consult == null) {
                        continue;
                    }
                    JSONObject json = new JSONObject();
                    json.put("id", consult.getId());
                    // 设置咨询标志
                    json.put("code", consult.getConsult());
                    // 设置咨询类型：0公共咨询，1指定医生，2三师咨询
                    json.put("type", consult.getType());
                    // 设置标题
                    json.put("title", consult.getSymptoms());
                    // 设置发病时间
                    json.put("when", consult.getWhen());
                    // 设置患者未读数量
                    json.put("patientRead", consult.getPatientRead());
                    // 设置状态
                    json.put("status", consult.getStatus());
                    // 设置患者评价标识
                    json.put("comment", consult.getComment());
                    // 设置关联指导
                    json.put("guidance", consult.getGuidance());
                    // 设置咨询或回复时间
                    json.put("time", DateUtil.dateToStr(consult.getCzrq(), DateUtil.YYYY_MM_DD_HH_MM_SS));
                    jsonArray.put(json);
                }
            }
            return success( "查询成功",  jsonArray);
        } catch (Exception ex) {
            ex.printStackTrace();
            return failedException(ex);
        }
    }

    /**
     * 患者取消三师咨询
     *
     * @param consult
     * @return
     */
    @RequestMapping(value = "cancel")
    @ResponseBody
    @ApiOperation("患者取消三师咨询")
    public Envelop cancel(String consult) {
        try {
            int row = consultTeamService.cancel(consult);
            if (row > 0) {
                return success("咨询已取消！");
            } else {
                return failed( "咨询不能取消！");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }


    /**
     * 修改状态为1的咨询记录为结束
     *
     * @param code 咨询标识
     * @return
     */
    @RequestMapping(value = "finish")
    @ResponseBody
    @ApiOperation("修改状态为1的咨询记录为结束")
    public Envelop finish(@RequestParam(required = true) String code) {
        try {
            int row = consultTeamService.finishConsult(code, getRepUID(), 1);
            if (row > 0) {
                return success("操作成功！");
            }else if(row == -2) {
                return failed( "续方未审核，不能结束续方咨询！");
            }  else {
                return failed( "操作失败！");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }

}
