package com.yihu.rehabilitation.controller.consult;

import com.yihu.jw.entity.base.im.ConsultTeamDo;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.hospital.message.service.SystemMessageService;
import com.yihu.jw.im.dao.ConsultTeamDao;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.rehabilitation.service.consult.ConsultTeamService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 医生端：三师咨询控制类
 *
 * @author George
 */
@RestController
@RequestMapping(value = "/doctor/consult", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(description = "三师咨询")
public class DoctorConsultController extends EnvelopRestEndpoint {

    @Autowired
    private ConsultTeamService consultTeamService;
    @Autowired
    private BasePatientDao basePatientDao;
    @Autowired
    private ConsultTeamDao consultTeamDao;

    @Autowired
    private SystemMessageService messageService;
    @Autowired
    protected HttpServletRequest request;

    @RequestMapping(value = "getKangFuConsultList", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation("获取续方咨询列表")
    public Envelop getKangFuConsultList(@ApiParam(name = "title", value = "咨询标题") @RequestParam(required = false) String title,
                                        @ApiParam(name = "patient", value = "居民CODE") @RequestParam(required = false) String patient,
                                        @ApiParam(name = "id", value = "第几页") @RequestParam(required = true) Integer id,
                                        @ApiParam(name = "pagesize", value = "页面大小") @RequestParam(required = true) int pagesize) {
        try {
            JSONArray array = new JSONArray();
//            Page<Object> data = consultTeamService.findConsultRecordByType("a663d0cf7f8c4d38a8327cedc921e65f", id, pagesize,8, title);//8表示续方咨询
            List<Map<String, Object>> data = consultTeamService.findConsultRecordByType(patient, id, pagesize, 18, title);//18表示康复咨询
            BasePatientDO patientobj = basePatientDao.findById(patient).orElse(null);
            if (data != null) {
                for (Map<String, Object> consult : data) {
                    if (consult == null) {
                        continue;
                    }
                    JSONObject json = new JSONObject();
                    json.put("id", consult.get("id"));
                    // 设置咨询类型：1三师咨询，2视频咨询，3图文咨询，4公共咨询，5病友圈，8 续方咨询  18康复咨询
                    json.put("type", consult.get("type"));
                    // 设置咨询标识
                    json.put("code", consult.get("relation_code"));
                    // 设置显示标题
                    json.put("title", consult.get("title"));
                    json.put("patientName", patientobj.getName());
                    json.put("patientPhoto", patientobj.getPhoto());
                    // 设置主诉
                    json.put("symptoms", consult.get("symptoms"));
                    // 咨询状态
                    json.put("status", consult.get("status"));
                    // 设置咨询日期
                    json.put("czrq", DateUtil.dateToStrLong((Date) consult.get("czrq")));
                    // 咨询状态
                    json.put("doctorCode", consult.get("doctor"));
                    json.put("evaluate", consult.get("evaluate"));
                    array.put(json);
                }
            }
            return success("查询成功！", array);
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }

    @RequestMapping(value = "evaluateStatus")
    @ResponseBody
    public Envelop statuss(String consult) {
        try {
            ConsultTeamDo ct = consultTeamService.findByCode(consult);
            if (ct == null) {
                return failed("获取状态失败！");
            } else {
                return success("获取状态成功！", ct.getEvaluate());
            }
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }

    @RequestMapping(value = "/isConsultFinished", method = RequestMethod.POST)
    public String isConsultFinished(@RequestParam String consult) {
        try {

            ConsultTeamDo ct = consultTeamDao.findByConsult(consult);
            return write(200, "查询咨询状态成功", "data", ct.getStatus());
        } catch (Exception e) {
            return e.getMessage();
        }
    }

}