package com.yihu.rehabilitation.controller.doctor;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.entity.specialist.rehabilitation.PatientMedicalRecordsDO;
import com.yihu.jw.entity.specialist.rehabilitation.RehabilitationAdviceDO;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.rehabilitation.RehabilitationRequestMapping;
import com.yihu.rehabilitation.aop.ObserverRequired;
import com.yihu.rehabilitation.service.RehabilitationInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.text.SimpleDateFormat;
import java.util.List;

@RestController
@RequestMapping(value = RehabilitationRequestMapping.DoctorRehabilitationInfo.Perfix, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(description = "医生端-康复下转管理")
public class DoctorRehabilitaionInfoController extends EnvelopRestEndpoint {

    @Autowired
    private RehabilitationInfoService rehabilitationInfoService;
    @Autowired
    private ObjectMapper objectMapper;

    @RequestMapping(value = RehabilitationRequestMapping.DoctorRehabilitationInfo.createAdvice, method = RequestMethod.POST)
    @ApiOperation("创建/编辑康复建议模板")
    @ObserverRequired
    public Envelop createAdvice(@ApiParam(name = "doctorCode", value = "医生code", required = false)
                               @RequestParam(value = "doctorCode", required = false) String doctorCode,
                                @ApiParam(name = "code", value = "模板code")
                               @RequestParam(value = "code", required = false) String code,
                                @ApiParam(name = "name", value = "模板名称", required = true)
                               @RequestParam(value = "name", required = true) String name,
                                @ApiParam(name = "content", value = "模板内容", required = true)
                               @RequestParam(value = "content", required = true) String content) {

        try {
            if (StringUtils.isBlank(doctorCode)) {
                doctorCode = getUID();
            }
            rehabilitationInfoService.createAdvice(doctorCode, code, name, content);
            return success("操作成功");
        } catch (Exception e) {
            e.printStackTrace();
            return failed(e.getMessage());
        }
    }

    @RequestMapping(value = RehabilitationRequestMapping.DoctorRehabilitationInfo.deleteAdvice, method = RequestMethod.POST)
    @ApiOperation("删除康复建议模板")
    @ObserverRequired
    public Envelop deleteAdvice(@ApiParam(name = "id", value = "模板id")
                               @RequestParam(value = "id", required = false) Long id) {

        try {
            rehabilitationInfoService.deleteAdvice(id);
            return success("操作成功");
        } catch (Exception e) {
            e.printStackTrace();
            return failed(e.getMessage());
        }
    }

    @RequestMapping(value =  RehabilitationRequestMapping.DoctorRehabilitationInfo.getAdviceListByDoctor, method = RequestMethod.GET)
    @ApiOperation("获取医生创建康复建议模板及系统模板")
    public Envelop getAdviceListByDoctor(@ApiParam(name = "doctorCode", value = "医生code", required = false)
                                        @RequestParam(value = "doctorCode", required = false) String doctorCode) {

        try {
            if (StringUtils.isBlank(doctorCode)) {
                doctorCode = getUID();
            }
            List<RehabilitationAdviceDO> list = rehabilitationInfoService.getAdviceListByDoctor(doctorCode);
            return success("操作成功",list);
        } catch (Exception e) {
           e.printStackTrace();
            return failed(e.getMessage());
        }
    }

    @RequestMapping(value = RehabilitationRequestMapping.DoctorRehabilitationInfo.sendWxMsgRemindSign, method = RequestMethod.POST)
    @ApiOperation("发送微信模板提醒签约消息")
    public Envelop sendWxTemplate(@ApiParam(name = "patient", value = "居民code", required = false)
                                 @RequestParam(value = "patient", required = false)String patient,
                                 @ApiParam(name = "wxId", value = "微信id", required = false)
                                 @RequestParam(value = "wxId", required = false)String wxId,
                                  @ApiParam(name = "scene", value = "模板code", required = false)
                                      @RequestParam(value = "scene", required = false)String scene){

        try {
            rehabilitationInfoService.sendSFWxTemplate(patient,wxId,scene);
            return success("发送成功");
        } catch (Exception e) {
           e.printStackTrace();
            return failed("发送失败");
        }
    }

    @RequestMapping(value = "createMedicalRecords", method = RequestMethod.POST)
    @ApiOperation("添加编辑住院病历")
    @ObserverRequired
    public Envelop createMedicalRecords(@ApiParam(name = "doctorCode", value = "医生code", required = false)
                                       @RequestParam(value = "doctorCode", required = false) String doctorCode,
                                       @ApiParam(name = "patient", value = "居民code")
                                       @RequestParam String patient,
                                       @ApiParam(name = "patientName", value = "居民姓名")
                                       @RequestParam String patientName,
                                       @ApiParam(name = "jsonData", value = "住院病历相关数据")
                                       @RequestParam(value = "jsonData", required = true) String jsonData,
                                       @ApiParam(name = "id", value = "模板id，编辑时传入")
                                       @RequestParam(value = "id", required = false) String id) {

        try {
            if (StringUtils.isBlank(doctorCode)) {
                doctorCode = getUID();
            }
            System.out.println("111doctorid"+doctorCode);
            objectMapper.setDateFormat(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"));
            PatientMedicalRecordsDO patientMedicalRecordsDO = objectMapper.readValue(jsonData, PatientMedicalRecordsDO.class);
            System.out.println("实体转换完成");
            return success( "添加成功", rehabilitationInfoService.createMedicalRecords(doctorCode, patient, patientName, patientMedicalRecordsDO, id));
        } catch (Exception e) {
            e.printStackTrace();
            return failed(e.getMessage());
        }
    }

    @RequestMapping(value = "getRehabilitationPatientList", method = RequestMethod.GET)
    @ApiOperation("获取康复下转居民列表")
    public Envelop getRehabilitationPatientList(@ApiParam(name = "idcard", value = "身份证", required = false)
                                               @RequestParam(value = "idcard", required = false) String idcard,
                                               @ApiParam(name = "isTurnDown", value = "是否已下转 -2:未下转未家签，-1未下转 0未下转全部，;1已下转(未接收);2已下转(已接收);3已下转全部;03已下转全部+未下转全部", defaultValue = "0",required = false)
                                               @RequestParam(value = "isTurnDown", required = false,defaultValue = "3") String isTurnDown,
                                                @ApiParam(name = "doctor", value = "",required = false)
                                                    @RequestParam(value = "doctor", required = false,defaultValue = "3") String doctor) {

        try {
            JSONObject result = rehabilitationInfoService.getRehabilitationPatientList(idcard, doctor,isTurnDown);
            return success( "请求成功", result);
        } catch (Exception e) {
            return failed("请求失败");
        }
    }
    @RequestMapping(value = "/getIcd10Info", method = RequestMethod.GET)
    @ApiOperation(value = "获取诊断结果")
    public Envelop getIcd10Info(@RequestParam(required = false)
                                @ApiParam(value = "诊断结果关键字", name = "nameKey") String nameKey,
                                @ApiParam(value = "页码", name = "page",defaultValue = "1",required = true)
                                @RequestParam(value = "page") Integer page,
                                @ApiParam(value = "每页数目", name = "pageSize",defaultValue = "15",required = true)
                                @RequestParam(value = "pageSize") Integer pageSize) {
        try {
            return success("操作成功！", rehabilitationInfoService.getIcd10Info(nameKey, page, pageSize).toString());
        } catch (Exception e) {
            return failed( "查询失败！");
        }
    }
}
