package com.yihu.rehabilitation.controller.doctor;

import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.rehabilitation.SpecialDiseaseMessagesDO;
import com.yihu.jw.entity.specialist.rehabilitation.RehabilitationOperateRecordsDO;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.rehabilitation.service.DoctorPatientRelationService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.rehabilitation.RehabilitationRequestMapping;
import com.yihu.jw.rm.specialist.SpecialistMapping;
import com.yihu.jw.wechat.service.WxAccessTokenService;
import com.yihu.rehabilitation.aop.ObserverRequired;
import com.yihu.rehabilitation.dao.SpecialDiseaseMessagesDao;
import com.yihu.rehabilitation.service.PatientRecordService;
import com.yihu.rehabilitation.service.RehabilitationPlanService;
import com.yihu.rehabilitation.service.ZsRehabilitationInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by humingfen on 2018/8/22.
 */
@RestController
@RequestMapping(value = RehabilitationRequestMapping.DoctorRehabilitationPlan.Perfix,produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(description = "医生端-康复服务套餐管理")
public class DoctorRehabilitationPlanController extends EnvelopRestEndpoint {
    @Autowired
    private RehabilitationPlanService rehabilitationPlanService;
    @Autowired
    private BaseDoctorDao baseDoctorDao;
    @Autowired
    private BasePatientDao basePatientDao;
    @Autowired
    private PatientRecordService patientRecordService;
    @Autowired
    private ZsRehabilitationInfoService zsRehabilitationInfoService;
    @Autowired
    private DoctorPatientRelationService doctorPatientRelationService;
    @Autowired
    private SpecialDiseaseMessagesDao specialDiseaseMessagesDao;
    @Autowired
    private WxAccessTokenService wxAccessTokenService;
    @Value("${demo.flag}")
    private Boolean demoFlag;
    @Value("${wechat.id}")
    private String  wxId;


    @RequestMapping(value = RehabilitationRequestMapping.DoctorRehabilitationPlan.createRehabilitationPlan, method = RequestMethod.POST)
    @ApiOperation(value = "创建居民康复计划")
    @ObserverRequired
    public ObjEnvelop createRehabilitationPlan(@ApiParam(name = "doctor", value = "医生标识")
                                           @RequestParam(required = false)String doctor,
                                               @ApiParam(name = "json", value = "康复计划json")
                                           @RequestParam(required = true) String json){
        try {
            if(!StringUtils.isNotBlank(doctor)){
                doctor = getUID();
            }
            BaseDoctorDO baseDoctorDO = baseDoctorDao.findById(doctor).orElse(null);
            JSONObject object = new JSONObject(json);
            JSONArray array = object.getJSONArray("detail");
            Set<String> list = new HashSet<>();
            for(int i=0;i<array.length();i++){
                JSONObject jsonObject = array.getJSONObject(i);
                String hospitalServiceItemId = jsonObject.getString("hospitalServiceItemId");
                String frequencyCode = jsonObject.getString("frequencyCode");
                Long timeType = jsonObject.getLong("timeType");
                String executeTimes =jsonObject.getString("executeTimes");
                String str[] = executeTimes.split(",");
                for (int j=i+1;j<array.length();j++){
                    JSONObject jsonObject1 = array.getJSONObject(j);
                    String hospitalServiceItemId1 = jsonObject1.getString("hospitalServiceItemId");
                    String frequencyCode1 = jsonObject1.getString("frequencyCode");
                    Long timeType1 = jsonObject1.getLong("timeType");
                    String executeTimes1 =jsonObject1.getString("executeTimes");
                    if (hospitalServiceItemId.equalsIgnoreCase(hospitalServiceItemId1)&&frequencyCode.equalsIgnoreCase(frequencyCode1)&&timeType.equals(timeType1)){
                        for (int z=0;z<str.length;z++){
                            if (executeTimes1.contains(str[z])){
                                list.add(hospitalServiceItemId);
                            }
                        }
                    }
                }
            }
            if (list != null && list.size()!=0){
                return ObjEnvelop.getError("计划创建失败！您设置了重复项目时间，请修改后生成!",-1);
            }
            return  ObjEnvelop.getSuccess("获取成功",rehabilitationPlanService.createRehabilitationPlan(json,baseDoctorDO),200);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError(e.getMessage());
        }
    }

    /*@RequestMapping(value = "/findTemplateList", method = RequestMethod.GET)
    @ApiOperation(value = "获取康复服务套餐模板列表")
    public String templateList(@ApiParam(name = "doctor", value = "医生标识")
                               @RequestParam(required = false)String doctor,
                               @ApiParam(name = "patient", value = "居民标识")
                               @RequestParam(required = false)String patient){
        try {
            if(!StringUtils.isNotBlank(doctor)){
                doctor = getUID();
            }
            return write(200, "获取成功", "data", rehabilitationPlanService.findTemplateInfo(doctor, patient));
        } catch (Exception e) {
            error(e);
            return error(-1, "请求失败");
        }
    }*/
    @PostMapping(value = RehabilitationRequestMapping.DoctorRehabilitationPlan.createServiceQrCode)
    @ApiOperation(value = "根据康复计划明细id和医生code生成服务码")
    public MixEnvelop<String,String> createServiceQrCode(@ApiParam(name = "planDetailId", value = "康复计划项目明细ID")@RequestParam(value = "planDetailId", required = true)String planDetailId,
                                                         @ApiParam(name = "doctorCode", value = "医生code")@RequestParam(value = "doctorCode", required = true)String doctorCode){
        try {
            return rehabilitationPlanService.createServiceQrCode(planDetailId,doctorCode);
        }catch (Exception e){
            e.printStackTrace();
            return MixEnvelop.getError(e.getMessage());
        }
    }

    @PostMapping(value =RehabilitationRequestMapping.DoctorRehabilitationPlan.checkAfterQrCode)
    @ApiOperation(value = "居民扫码后验证是否是关联的居民扫码")
    public MixEnvelop<Map<String,Object>,Map<String,Object>> checkAfterQrCode(@ApiParam(name = "planDetailId", value = "康复计划项目明细ID")@RequestParam(value = "planDetailId", required = true)String planDetailId,
                                                                              @ApiParam(name = "patientCode", value = "居民端登录的居民code")@RequestParam(value = "patientCode", required = true)String patientCode){
        try {
            String message="";
            Boolean flag = true;
            Map<String,Object> map = rehabilitationPlanService.checkAfterQrCode(planDetailId,patientCode);
            int result = Integer.valueOf(String.valueOf(map.get("code")));
            if (result==200){
                message = "验证成功！";
            }
            if (result==-1){
                message = "请相关居民扫描二维码";
                flag=false;
            }
            if (result==-10000){
                message = "相关康复管理数据错误,请联系工作人员！";
                flag=false;
            }
            map.put("flag",flag);
            map.remove("code");
            return MixEnvelop.getSuccess(message,map);
        }catch (Exception e){
            e.printStackTrace();
            return MixEnvelop.getError(e.getMessage());
        }
    }
    @RequestMapping(value = RehabilitationRequestMapping.DoctorRehabilitationPlan.saveRehabilitationOperateRecord, method = RequestMethod.POST)
    @ApiOperation("康复管理-新增operateRecord")
    public MixEnvelop saveRehabilitationOperateRecord(@ApiParam(name = "dataJson", value = "实体json",defaultValue = "{\"rehabilitationDetailId\":\"402803f6657f195301657f4fbd3c0001\",\"patientCode\":\"00\",\"patientName\":\"11\",\"doctorCode\":\"22\",\"doctorName\":\"33\",\"node\":\"jjjjjj\",\"relationRecordType\":\"4\",\"relation_record_code\":\"55\",\"relationRecordImg\":\"666666\",\"status\":\"0\"}")@RequestParam(value = "dataJson", required = true)String dataJson){
        try {
            JSONObject json = new JSONObject(dataJson);
            if(json.has("patientCode")){
                BasePatientDO patient = basePatientDao.findById(json.get("patientCode")+"").orElse(null);
                json.put("patientName",patient!=null?patient.getName():"");
                json.put("patientName",patient!=null?patient.getName():"");
                try {
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    SpecialDiseaseMessagesDO diseaseMessages = new SpecialDiseaseMessagesDO();
                    diseaseMessages.setCode(patient.getId());
                    diseaseMessages.setName(patient.getName());
                    diseaseMessages.setAddress(patient.getAddress());
                    diseaseMessages.setCodeType("2");
                    diseaseMessages.setCreateTime(sdf.format(new Date()));
                    diseaseMessages.setResult("完成电话短信关怀");
                    specialDiseaseMessagesDao.save(diseaseMessages);
                }catch (Exception e){
                    e.printStackTrace();
                }
            }
            if(json.has("doctorCode")){
                BaseDoctorDO doctor = baseDoctorDao.findById(json.get("doctorCode")+"").orElse(null);
                json.put("doctorName",doctor!=null?doctor.getName():"");
            }
            RehabilitationOperateRecordsDO rehabilitationOperateRecordsDO= toEntity(json.toString(), RehabilitationOperateRecordsDO.class);
            return MixEnvelop.getSuccess(SpecialistMapping.api_success,rehabilitationPlanService.saveRehabilitationRecord(rehabilitationOperateRecordsDO));
        }catch (Exception e){
            e.printStackTrace();
            return MixEnvelop.getError(e.getMessage());
        }
    }
    @PostMapping(value = RehabilitationRequestMapping.DoctorRehabilitationPlan.updatePlanStatusById)
    @ApiOperation(value = "康复管理-更新康复计划状态")
    public MixEnvelop updatePlanStatusById(@ApiParam(name = "planId", value = "康复计划id", required = true)
                                           @RequestParam(value = "planId", required = true)String planId,
                                           @ApiParam(name = "status", value = "状态", required = true)
                                           @RequestParam(value = "status", required = true)Integer status,
                                           @ApiParam(name = "abortReason", value = "中止原因", required = true)
                                           @RequestParam(value = "abortReason", required = false)String abortReason){
        try {
            if(status!=0){
                return rehabilitationPlanService.updatePlanStatusById(status,planId);
            }else {
                return rehabilitationPlanService.abortPlanById(planId,abortReason);
            }
        }catch (Exception e){
            e.printStackTrace();
            return MixEnvelop.getError(e.getMessage());
        }
    }
    @PostMapping(value = RehabilitationRequestMapping.DoctorRehabilitationPlan.updatePatientImgById)
    @ApiOperation(value = "更新康复计划居民签名照/证件照")
    public Envelop updatePatientImgById(@ApiParam(name = "planId", value = "康复计划id", required = true)
                                        @RequestParam(value = "planId")String planId,
                                        @ApiParam(name = "patientImg", value = "居民签名/证件照地址",required = true)
                                        @RequestParam(value = "patientImg")String patientImg){
        try {
            return rehabilitationPlanService.updatePatientImgById(planId, patientImg);
        }catch (Exception e){
            e.printStackTrace();
            return Envelop.getError(e.getMessage());
        }
    }
    @PostMapping(value = RehabilitationRequestMapping.DoctorRehabilitationPlan.findServicePackageItem)
    @ApiOperation(value = "查询康复计划项目")
    public ObjEnvelop findServicePackageItem(@ApiParam(name = "doctorId", value = "医生id", required = true)
                                        @RequestParam(value = "doctorId")String doctorId){
        try {
            return rehabilitationPlanService.findServiceItemByDoctor(doctorId);
        }catch (Exception e){
            e.printStackTrace();
            return ObjEnvelop.getError(e.getMessage());
        }
    }
    @PostMapping(value = RehabilitationRequestMapping.DoctorRehabilitationPlan.findDoctorTeams)
    @ApiOperation(value = "查询医生团队")
    public ObjEnvelop findDoctorTeams(@ApiParam(name = "doctor", value = "医生id", required = false)
                                             @RequestParam(value = "doctor", required = false)String doctor){
        try {
            return ObjEnvelop.getSuccess("操作成功",rehabilitationPlanService.findDoctorTeams(doctor));
        }catch (Exception e){
            e.printStackTrace();
            return ObjEnvelop.getError(e.getMessage());
        }
    }
    @PostMapping(value = RehabilitationRequestMapping.DoctorRehabilitationPlan.findDoctorByTeams)
    @ApiOperation(value = "根据团队查询医生列表")
    public ObjEnvelop findDoctorByTeams(@ApiParam(name = "teamCode", value = "团队id", required = false)
                                            @RequestParam(value = "teamCode", required = false)String teamCode){
        try {
            return ObjEnvelop.getSuccess("操作成功",rehabilitationPlanService.findDoctorByTeams( teamCode));
        }catch (Exception e){
            e.printStackTrace();
            return ObjEnvelop.getError(e.getMessage());
        }
    }
    @PostMapping(value = RehabilitationRequestMapping.DoctorRehabilitationPlan.findDoctorPatients)
    @ApiOperation(value = "根据团队查询医生列表")
    public MixEnvelop findDoctorPatients(@ApiParam(name = "doctor", value = "医生id", required = false)
                                        @RequestParam(value = "doctor", required = false)String doctor,
                                         @ApiParam(name = "patient", value = "患者id", required = false)
                                         @RequestParam(value = "patient", required = false)String patient,
                                         @ApiParam(name = "patientName", value = "患者姓名", required = false)
                                             @RequestParam(value = "patientName", required = false)String patientName,
                                         @ApiParam(name = "isDeal", value = "是否下转1是0否", required = false)
                                             @RequestParam(value = "isDeal", required = false)String isDeal,
                                         @ApiParam(name = "idcard", value = "是否下转1是0否", required = false)
                                             @RequestParam(value = "idcard", required = false)String idcard,
                                         @ApiParam(name = "page", value = "page", required = false)
                                             @RequestParam(value = "page", required = false)Integer page,
                                         @ApiParam(name = "pageSize", value = "pageSize", required = false)
                                             @RequestParam(value = "pageSize", required = false) Integer pageSize){
        try {
            return doctorPatientRelationService.findDoctorPatients(doctor,patient,patientName,isDeal,idcard,page,pageSize);
        }catch (Exception e){
            e.printStackTrace();
            return MixEnvelop.getError(e.getMessage());
        }
    }

    @RequestMapping(value = "getPatientByIdcardOrSsc", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation("根据患者身份证或社保卡号进行搜索添加出院记录")
    public Envelop getPatientByIdcardOrSsc(@ApiParam(name = "patientInfo", value = "居民身份证或者社保卡", required = true)
                                          @RequestParam(value = "patientInfo")String patientInfo) {
        try {

            com.alibaba.fastjson.JSONObject object = doctorPatientRelationService.getPatientByIdcardOrSsc(patientInfo, getUID());
            if(object.getInteger("status") == 200) {
                return success("添加成功！", object.get("data"));
            }else {
                return failed( object.getString("msg"));
            }
        } catch (Exception e) {
            e.printStackTrace();
            return failed("添加失败！");
        }
    }
    @RequestMapping(value = "/updateRehabilitationPlan", method = RequestMethod.POST)
    @ApiOperation(value = "编辑居民康复计划")
    public Envelop updateRehabilitationPlan(@ApiParam(name = "doctor", value = "医生标识")
                                           @RequestParam(required = false)String doctor,
                                           @ApiParam(name = "json", value = "康复计划json")
                                           @RequestParam(required = true) String json,
                                           @ApiParam(name = "updateStatus", value = "修改类型：0单个服务的修改(默认)，1整个计划修改")
                                           @RequestParam(required = false,defaultValue = "0") String updateStatus){
        try {
            if(!StringUtils.isNotBlank(doctor)){
                doctor = getUID();
            }
            BaseDoctorDO d = baseDoctorDao.findById(doctor).orElse(null);
            JSONObject object = new JSONObject(json);
            JSONArray array = object.getJSONArray("detail");//获取服务内容
            Set<String> list = new HashSet<>();
            for(int i=0;i<array.length();i++){
                JSONObject jsonObject = array.getJSONObject(i);
                String hospitalServiceItemId = jsonObject.getString("hospitalServiceItemId");//服务id
                String frequencyCode = jsonObject.getString("frequencyCode");
                Long timeType = jsonObject.getLong("timeType");
                String executeTimes =jsonObject.getString("executeTimes");
                String str[] = executeTimes.split(",");
                for (int j=i+1;j<array.length();j++){
                    JSONObject jsonObject1 = array.getJSONObject(j);
                    String hospitalServiceItemId1 = jsonObject1.getString("hospitalServiceItemId");
                    String frequencyCode1 = jsonObject1.getString("frequencyCode");
                    Long timeType1 = jsonObject1.getLong("timeType");
                    String executeTimes1 =jsonObject1.getString("executeTimes");
                    if (hospitalServiceItemId.equalsIgnoreCase(hospitalServiceItemId1)&&frequencyCode.equalsIgnoreCase(frequencyCode1)&&timeType.equals(timeType1)){
                        for (int z=0;z<str.length;z++){
                            if (executeTimes1.contains(str[z])){//时间与之前项目时间相同
                                list.add(hospitalServiceItemId);
                            }
                        }
                    }
                }
            }
            if (list != null && list.size()!=0){
                return failed( "计划创建失败！您设置了重复项目时间，请修改后生成!");
            }
            return success("获取成功", rehabilitationPlanService.updateRehabilitationPlan(json,d,updateStatus));
        } catch (Exception e) {
            e.printStackTrace();
            return failed("请求失败");
        }
    }

    @RequestMapping(value = "getRehabilitationPlanLog",method = RequestMethod.GET)
    @ApiOperation(value = "获取康复计划修改记录")
    public Envelop getRehabilitationPlanLog(@ApiParam(name = "planId",value = "康复计划Id",required = true)
                                           @RequestParam(value = "planId",required = true) String planId){

        try {
            return success("获取成功",rehabilitationPlanService.getRehabilitationPlanLog(planId));
        }catch (Exception e){
            return failed("请求失败");
        }
    }


    /**
     * 查询居民信息
     *
     * @param patient 患者
     * @return
     */
    @GetMapping(value = "/patient")
    @ApiOperation("查询居民信息")
    public Envelop getPatient(String patient) {
        try {
            if (StringUtils.isEmpty(patient)) {
                return failed( "居民不能为空");
            }
            JSONObject p = rehabilitationPlanService.getPatient(patient,getUID());
            return success( "查询成功",  p);
        } catch (Exception e) {
            e.printStackTrace();
            return failed( "居民不能为空");
        }
    }

    @RequestMapping(value = "/findTemplateDetail", method = RequestMethod.GET)
    @ApiOperation(value = "获取康复服务套餐模板明细")
    public Envelop findTemplateDetail(@ApiParam(name = "templateId", value = "模板id")
                                     @RequestParam(required = true)String templateId){
        try {
            return success( "获取成功", rehabilitationPlanService.findTemplateDetailInfo(templateId));
        } catch (Exception e) {
            e.printStackTrace();
            return failed( "请求失败");
        }
    }


    @ApiOperation("获取门诊记录/住院记录(基卫+APP)")
    @RequestMapping(value = "/event", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ResponseBody
    public Envelop getAllEvent(@ApiParam(name = "patient", value = "患者代码", defaultValue = "")
                              @RequestParam(value = "patient", required = false) String patient,
                              @ApiParam(name = "type", value = "类型", defaultValue = "")
                              @RequestParam(value = "type", required = false) String type,
                              @ApiParam(name = "page", value = "第几页", defaultValue = "1")
                              @RequestParam(value = "page", required = true) String page,
                              @ApiParam(name = "pageSize", value = "每页几行", defaultValue = "10")
                              @RequestParam(value = "pageSize", required = true) String pageSize,
                              @ApiParam(name = "lastTime", value = "最后一条时间", defaultValue = "2010-10-10 00:00:00")
                              @RequestParam(value = "lastTime", required = false) String lastTime) {
        try {
            if(demoFlag){
                String res;
                switch (type){
                    case "1":
                    case "2":
                        res = patientRecordService.getJosnFileResullt("event"+type);
                        break;
                    default:
                        return success("获取成功");
                }

                return success(res);
            }else {
                List<Map<String, String>> result = patientRecordService.getAllEvent(patient, type, page, pageSize, lastTime);

                return success("获取就诊记录成功！", result);
            }
        } catch (Exception e) {
            e.printStackTrace();
            if (e.getMessage().indexOf("卡号尚未在医疗机构注册过") > 0) {
                return failed("对不起，您的社保卡有变更，请改天再试");
            } else if (e.getMessage().startsWith("error")) {
                return failed( e.getMessage());
            } else {
                return failed("获取门/急诊数据失败！");
            }
        }
    }


    @RequestMapping(value = "sendWxMsg", method = RequestMethod.POST)
    @ApiOperation("发送微信模板消息")
    public Envelop sendWxTemplate(@ApiParam(name = "planId", value = "康复计划id", required = true)
                                 @RequestParam(value = "planId")String planId){

        try {
            JSONObject jsonObject = rehabilitationPlanService.getServiceItemsAfterStop(planId);
            rehabilitationPlanService.sendWxTemplate(wxAccessTokenService.getWxAccessTokenById(wxId).getAccessToken(), planId, getUID(), jsonObject);
            return success( "发送成功！");
        } catch (Exception e) {
           e.printStackTrace();
            return failed("发送失败");
        }
    }
}
