package com.yihu.rehabilitation.controller.followUp;

import com.alibaba.fastjson.JSON;
import com.yihu.jw.entity.followup.Followup;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.rehabilitation.aop.ObserverRequired;
import com.yihu.rehabilitation.service.followUp.FollowUpService;
import com.yihu.rehabilitation.service.followUp.FollowupDrugsService;
import com.yihu.rehabilitation.util.BusinessLogs;
import com.yihu.rehabilitation.vo.FollowupContentESDO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.models.auth.In;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.List;
import java.util.Map;

/**
 * 医生端：随访接口
 *
 * @author hzp add 2016-12-07
 */
@Controller
@RequestMapping(value = "/doctor/followup", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(description = "医生端-随访接口")
public class DoctorFollowUpController  extends EnvelopRestEndpoint {

   @Autowired
    private FollowUpService followUpService;
   @Autowired
   private JdbcTemplate jdbcTemplate;
   @Autowired
   private FollowupDrugsService followupDrugsService;
    @ApiOperation("新增随访计划（批量）")
    @RequestMapping(value = "/addFollowupPlan", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ResponseBody
    @ObserverRequired
    public Envelop addFollowupPlan(@ApiParam(name = "patient", value = "患者代码", defaultValue = "443a196ef8744536a531260eb26c05d7")
                                  @RequestParam(value = "patient", required = true) String patient,
                                   @ApiParam(name = "data", value = "随访计划列表json", defaultValue = "[{\"date\":\"2016-12-16 20:00:00\",\"type\":\"10\",\"doctor\":\"64de9952-5b15-11e6-8344-fa163e8aee56\"},{\"date\":\"2016-12-17 15:00:00\",\"type\":\"3\",\"doctor\":\"64de9952-5b15-11e6-8344-fa163e8aee56\"}]")
                                  @RequestParam(value = "data", required = true) String data) {
        try {
            Iterable<Followup> followups = followUpService.addFollowupPlan(getUID(), patient, data);

            if (followups != null) {
                for (Followup followup : followups) {
                    BusinessLogs.info(BusinessLogs.BusinessType.followup, getUID(), followup.getPatientCode(), new JSONObject(followup));
                }
            }

            return success( "新增随访计划成功！",followups);
        } catch (Exception e) {
            e.printStackTrace();
            return failed("新增随访计划失败！" );
        }
    }

    @ApiOperation("获取随访信息")
    @RequestMapping(value = "/getFollowup", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ResponseBody
    public Envelop getFollowup(@ApiParam(name = "id", value = "随访记录ID", defaultValue = "4")
                              @RequestParam(value = "id", required = true) String id) {
        try {
            Map<String, String> response = followUpService.getFollowup(id);

            try {
                String sql = "UPDATE `wlyy_followup` SET `doctor_read_status`='1' WHERE (`id`="+id+") ";
                jdbcTemplate.update(sql);
            }catch (Exception e){
                e.printStackTrace();
            }

            return success( "获取随访信息成功！", response);
        } catch (Exception e) {
            e.printStackTrace();
            return failed("获取随访信息失败！" + e.getMessage());
        }
    }

    @RequestMapping(value = "/isFirstTimeFollowup",produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ApiOperation("判断居民该类别随访是否为第一次")
    @ResponseBody
    public Envelop isFirstTimeFollowup(@ApiParam(name = "patient", value = "居民code", defaultValue = "")
                                      @RequestParam(value = "patient", required = true)String patient,
                                      @ApiParam(name = "followupClass", value = "随访类别", defaultValue = "")
                                      @RequestParam(value = "followupClass", required = true)String followupClass,
                                      @ApiParam(name = "followupId", value = "随访类别", defaultValue = "")
                                      @RequestParam(value = "followupId", required = true)Integer followupId) {
        try {
            return success( "操作成功！",followUpService.isFirstTimeFollowup(getUID(),patient,followupClass,followupId));
        }catch (Exception e){
            //日志文件中记录异常信息
          e.printStackTrace();
            //返回接口异常信息处理结果
            return failed( "操作失败！");
        }
    }

    @ApiOperation("开始随访记录")
    @RequestMapping(value = "/startFollowup", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ResponseBody
    @ObserverRequired
    public Envelop startFollowup(@ApiParam(name = "id", value = "随访记录ID", defaultValue = "")
                                @RequestParam(value = "id", required = true) String id,
                                @ApiParam(name = "followupNo", value = "随访记录编码")
                                @RequestParam(value = "followupNo", required = false) String followupNo,
                                @ApiParam(name = "date", value = "随访时间", defaultValue = "2016-12-14 20:00:00")
                                @RequestParam(value = "date", required = true) String date,
                                @ApiParam(name = "followupType", value = "随访方式【字典FOLLOWUP_WAY_DICT】", defaultValue = "12")
                                @RequestParam(value = "followupType", required = true) String followupType,
                                @ApiParam(name = "followupClass", value = "随访类别【1.高血压 2.糖尿病，3高糖】", defaultValue = "1")
                                @RequestParam(value = "followupClass", required = true) String followupClass,
                                @ApiParam(name = "followupManagerStatus", value = "随访管理状态【字典FOLLOWUP_MANAGER_STATUS】", defaultValue = "1")
                                @RequestParam(value = "followupManagerStatus", required = false) String followupManagerStatus,
                                @ApiParam(name = "plandate", value = "下次随访时间", defaultValue = "2016-12-14 20:00:00")
                                @RequestParam(value = "plandate", required = false) String plandate,
                                @ApiParam(name = "prescriptioncode", value = "续方CODE", defaultValue = "")
                                @RequestParam(value = "prescriptioncode", required = false) String prescriptioncode) {
        try {
            Followup followup = followUpService.startFollowup( id, followupNo, date, followupType, followupClass, followupManagerStatus,plandate,prescriptioncode);
            followUpService.getNotStartFollowup(getUID(), followup.getFollowupPlanDate());
            return success( "开始随访记录成功！");
        } catch (Exception e) {
            e.printStackTrace();
            return failed("开始随访记录失败！" + e.getMessage());
        }
    }

    @ApiOperation("获取上次随访")
    @RequestMapping(value = "/getLastFollowup", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ResponseBody
    public Envelop getLastFollowup(@ApiParam(name = "doctor", value = "医生代码", defaultValue = "443a196ef8744536a531260eb26c05d7")
                                  @RequestParam(value = "doctor", required = true) String doctor,
                                  @ApiParam(name = "patient", value = "患者代码", defaultValue = "P20161008001")
                                  @RequestParam(value = "patient", required = true) String patient,
                                  @ApiParam(name = "followClass", value = "随访类别", defaultValue = "1")
                                  @RequestParam(value = "followClass", required = true) String followClass) {
        try {
            Map<String, String> response = followUpService.getLastFollowup(doctor, patient, followClass);

            return success( "获取上次随访成功！",response);
        } catch (Exception e) {
          e.printStackTrace();
            return failed("获取上次随访失败！" + e.getMessage());
        }
    }

    @ApiOperation("复制上次随访数据")
    @RequestMapping(value = "/copyFollowup", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ResponseBody
    public Envelop copyFollowup(@ApiParam(name = "id", value = "本地随访ID", defaultValue = "")
                               @RequestParam(value = "id", required = true) Long id,
                               @ApiParam(name = "fromId", value = "拷贝随访记录ID", defaultValue = "")
                               @RequestParam(value = "fromId", required = true) Long fromId) {
        try {
            followUpService.copyFollowup(id, fromId);

            return success( "复制上次随访成功！");
        } catch (Exception e) {
            e.printStackTrace();
            return failed("复制上次随访失败！" + e.getMessage());
        }
    }

    @ApiOperation("保存全部随访表单数据")
    @RequestMapping(value = "/saveAllFollowupProjectData", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ResponseBody
    @ObserverRequired
    public Envelop saveAllFollowupProjectData(@ApiParam(name = "id", value = "随访记录ID")
                                             @RequestParam(value = "id", required = true) String id,
                                             @ApiParam(name = "followupNo", value = "随访记录编码")
                                             @RequestParam(value = "followupNo", required = false) String followupNo,
                                             @ApiParam(name = "date", value = "随访时间", defaultValue = "2016-12-14 20:00:00")
                                             @RequestParam(value = "date", required = false) String date,
                                             @ApiParam(name = "followupType", value = "随访方式【字典FOLLOWUP_WAY_DICT】", defaultValue = "12")
                                             @RequestParam(value = "followupType", required = false) String followupType,
                                             @ApiParam(name = "followupClass", value = "随访类别【1.高血压 2.糖尿病，3高糖】", defaultValue = "1")
                                             @RequestParam(value = "followupClass", required = false) String followupClass,
                                             @ApiParam(name = "followupManagerStatus", value = "随访管理状态【字典FOLLOWUP_MANAGER_STATUS】", defaultValue = "1")
                                             @RequestParam(value = "followupManagerStatus", required = false) String followupManagerStatus,
                                             @ApiParam(name = "plandate", value = "下次随访时间", defaultValue = "2016-12-14 20:00:00")
                                             @RequestParam(value = "plandate", required = false) String plandate,
                                             @ApiParam(name = "prescriptioncode", value = "续方CODE")
                                             @RequestParam(value = "prescriptioncode", required = false) String prescriptioncode,
                                             @ApiParam(name = "followupProjectData", value = "随访项目数据", defaultValue = "{\"WEIGHT\":\"76\",\"WEIGHT_EXP\":\"60\",\"BMI\":\"11\"}")
                                             @RequestParam(value = "followupProjectData", required = true) String followupProjectData) {
        try {
            JSONObject jsonObject = new JSONObject(followupProjectData);
            Followup followup = followUpService.startFollowup(id, followupNo, date, followupType, followupClass, followupManagerStatus, plandate, prescriptioncode);
            followUpService.finishFollowup(id);
            followUpService.saveAllFollowupProjectData(id, jsonObject.getJSONObject("followupProjectData").toString());
            followupDrugsService.saveFollowupDrugs(id,jsonObject.getJSONArray("drugsData").toString());
            return success( "保存随访项目数据成功！");
        } catch (Exception e) {
            e.printStackTrace();
            return failed("保存随访项目数据失败！" + e.getMessage());
        }
    }

    @RequestMapping(value = "/updateFollowStatus",produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ApiOperation("完成随访")
    @ResponseBody
    public Envelop  updateFollowStatus(@ApiParam(name = "followupId", value = "随访ID", defaultValue = "")
                                      @RequestParam(value = "followupId", required = true) Integer followupId,
                                      @ApiParam(name = "isFlag", value = "标识第一次还是N次肺结核(第一次0，n次为1)", defaultValue = "")
                                      @RequestParam(value = "isFlag", required = false)Integer isFlag)throws Exception {
        try {
            return success( "操作成功！",followUpService.updateFollowStatus(followupId,isFlag));
        }catch (Exception e){
            //日志文件中记录异常信息
            e.printStackTrace();
            //返回接口异常信息处理结果
            return failed( "操作失败！");
        }
    }

    @RequestMapping(value = "/urlAnalysis",produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ApiOperation("门牌解析上门地址")
    @ResponseBody
    public Envelop urlAnalysis(@ApiParam(name = "url", value = "地址解析", defaultValue = "")
                              @RequestParam(value = "url", required = true)String url)throws Exception {
        try {
            return success( "操作成功！",followUpService.urlAnalysis(url));
        }catch (Exception e){
            //日志文件中记录异常信息
            e.printStackTrace();
            //返回接口异常信息处理结果
            return failed( "操作失败！");
        }
    }

    @RequestMapping(value = "/saveFollowupSign",produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ApiOperation("签到")
    @ResponseBody
    public Envelop saveFollowupSign(@ApiParam(name = "jsonSign", value = "签到", defaultValue = "")
                                   @RequestParam(value = "jsonSign", required = true)String jsonSign)throws Exception {
        try {
            return success("操作成功！",followUpService.saveFollowupSign(jsonSign));
        }catch (Exception e){
            //日志文件中记录异常信息
            e.printStackTrace();
            //返回接口异常信息处理结果
            return failed( "操作失败！");
        }
    }

    @RequestMapping(value = "/updateFollowupSign",produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ApiOperation("更新签到")
    @ResponseBody
    public Envelop updateFollowupSign(@ApiParam(name = "jsonSign", value = "签到", defaultValue = "")
                                     @RequestParam(value = "jsonSign", required = true)String jsonSign)throws Exception {
        try {
            return success( "操作成功！",followUpService.updateFollowupSign(jsonSign));
        }catch (Exception e){
            //日志文件中记录异常信息
            e.printStackTrace();
            //返回接口异常信息处理结果
            return failed( "操作失败！");
        }
    }


    @RequestMapping(value = "/findByFollowupId",produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ApiOperation("签约详情")
    @ResponseBody
    public Envelop findByFollowupId(@ApiParam(name = "followupId", value = "签到", defaultValue = "")
                                   @RequestParam(value = "followupId", required = true)Integer followupId) {
        try {
            return success("操作成功！",followUpService.findByFollowupId(followupId));
        }catch (Exception e){
            //日志文件中记录异常信息
            e.printStackTrace();
            //返回接口异常信息处理结果
            return failed( "操作失败！");
        }
    }


    @RequestMapping(value = "/findFollowupInfo",produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ApiOperation("获取随访详情")
    @ResponseBody
    public Envelop findFollowupInfo(@ApiParam(name = "followupId", value = "随访ID", defaultValue = "")
                                   @RequestParam(value = "followupId", required = true)Integer followupId) {
        try {
            return success( "操作成功！",followUpService.findFollowupInfo(followupId));
        }catch (Exception e){
            //日志文件中记录异常信息
            e.printStackTrace();
            //返回接口异常信息处理结果
            return failed( "操作失败！");
        }
    }
    @RequestMapping(value = "/delFollowup",produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ApiOperation("删除随访访视")
    @ResponseBody
    public Envelop delFollowup(@ApiParam(name = "followupId", value = "随访ID", defaultValue = "")
                              @RequestParam(value = "followupId", required = true)Integer followupId)throws Exception {
        try {
            Followup followup = followUpService.delFollowup(followupId);
            followUpService.getNotStartFollowup(getUID(), followup.getFollowupPlanDate());
            return success("操作成功！", true);
        }catch (Exception e){
            //日志文件中记录异常信息
            e.printStackTrace();
            //返回接口异常信息处理结果
            return failed( "操作失败！"+e.getMessage());
        }
    }


    @RequestMapping(value = "/saveFollowup",produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ApiOperation("保存随访访视")
    @ResponseBody
    public Envelop saveFollowup(@ApiParam(name = "jsonFollowup", value = "创建访视json", defaultValue = "")
                               @RequestParam(value = "jsonFollowup", required = true)String jsonFollowup){
        try {
            return success( "操作成功！",followUpService.saveFollowup(jsonFollowup));
        }catch (Exception e){
            //日志文件中记录异常信息
            e.printStackTrace();
            //返回接口异常信息处理结果
            return failed( "操作失败！"+e.getMessage());
        }
    }


    @RequestMapping(value = "/findPatientFollowList",produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ApiOperation("获取居民随访列表")
    @ResponseBody
    public Envelop findPatientFollowList(@ApiParam(name = "patient", value = "居民code", defaultValue = "")
                                        @RequestParam(value = "patient", required = true)String patient,
                                        @ApiParam(name = "doctor", value = "医生code", defaultValue = "")
                                        @RequestParam(value = "doctor", required = true)String doctor){
        try {
            return success("操作成功！",followUpService.findPatientFollowList(patient,doctor));
        }catch (Exception e){
            //日志文件中记录异常信息
            e.printStackTrace();
            //返回接口异常信息处理结果
            return failed( "操作失败！");
        }
    }


    @RequestMapping(value = "/findPatientInfo",produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ApiOperation("获取居民单条信息")
    @ResponseBody
    public Envelop findPatientInfo(@ApiParam(name = "patient", value = "居民code", defaultValue = "")
                                  @RequestParam(value = "patient", required = true)String patient) {
        try {
            return success( "操作成功！",followUpService.findPatientInfo(patient));
        }catch (Exception e){
            //日志文件中记录异常信息
            e.printStackTrace();
            //返回接口异常信息处理结果
            return failed("操作失败！");
        }
    }


    @RequestMapping(value = "/findFollowUpPatient",produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ApiOperation("获取可随访居民列表")
    @ResponseBody
    public Envelop findFollowUpPatient(@ApiParam(name = "namekey", value = "居民姓名模糊", defaultValue = "")
                                      @RequestParam(value = "namekey", required = false)String namekey,
                                      @ApiParam(name = "labelCode", value = "标签code", defaultValue = "1")
                                      @RequestParam(value = "labelCode", required = false)String labelCode,
                                      @ApiParam(name = "doctor", value = "医生", defaultValue = "zjm20190214")
                                      @RequestParam(value = "doctor", required = true)String doctor,
                                      @ApiParam(name = "page", value = "页数，1开始", defaultValue = "1")
                                      @RequestParam(value = "page", required = true)Integer page,
                                      @ApiParam(name = "size", value = "每页大小", defaultValue = "10")
                                      @RequestParam(value = "size", required = true)Integer size)throws Exception {
        try {
            return success( "操作成功！",followUpService.findFollowUpPatient(namekey,doctor,labelCode,page,size));
        }catch (Exception e){
            //日志文件中记录异常信息
            e.printStackTrace();
            //返回接口异常信息处理结果
            return failed("操作失败！");
        }
    }

    @RequestMapping(value = "/findLabelListWithCount",produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ApiOperation("获取随访标签列表带居民数")
    @ResponseBody
    public Envelop findLabelListWithCount(){
        try {
            return success("操作成功！",followUpService.findLabelListWithCount());
        }catch (Exception e){
            //日志文件中记录异常信息
            e.printStackTrace();
            //返回接口异常信息处理结果
            return failed( "操作失败！");
        }
    }


    @RequestMapping(value = "/findFollowupList",produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ApiOperation("获取访视列表形式")
    @ResponseBody
    public Envelop findFollowupList(@ApiParam(name = "doctor", value = "医生", defaultValue = "zjm20190214")
                                   @RequestParam(value = "doctor", required = true) String doctor,
                                   @ApiParam(name = "patient", value = "居民")
                                   @RequestParam(value = "patient", required = false) String patient,
                                   @ApiParam(name = "followupClass", value = "随访类别，多类别“,”分割")
                                   @RequestParam(value = "followupClass", required = false) String followupClass,
                                   @ApiParam(name = "patientName", value = "居民姓名")
                                   @RequestParam(value = "patientName", required = false) String patientName,
                                   @ApiParam(value = "开始时间，格式（yyyy-MM-dd）", name = "startDate",required = false)
                                   @RequestParam(value = "startDate", required = false) String startDate,
                                   @ApiParam(value = "结束时间，格式（yyyy-MM-dd）", name = "endDate",required = false)
                                   @RequestParam(value = "endDate", required = false) String endDate,
                                   @ApiParam(name = "flag", value = "区分上门访视(1-是)")
                                   @RequestParam(value = "flag", required = false) String flag,
                                   @ApiParam(name = "type", value = "")
                                   @RequestParam(value = "type", required = false) Integer type){
        try {

            return success( "操作成功！",followUpService.findFollowupList(doctor,patient,followupClass,patientName,startDate,endDate,flag,type));
        }catch (Exception e){
            //日志文件中记录异常信息
            e.printStackTrace();
            //返回接口异常信息处理结果
            return failed("操作失败！");
        }
    }


    @RequestMapping(value = "/findFollowupByMonth",produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ApiOperation("获取访视计划按月日历形式")
    @ResponseBody
    public Envelop findFollowupByMonth(@ApiParam(name = "doctor", value = "医生", defaultValue = "zjm20190214")
                                      @RequestParam(value = "doctor", required = true) String doctor,
                                      @ApiParam(name = "patient", value = "居民")
                                      @RequestParam(value = "patient", required = false) String patient,
                                      @ApiParam(name = "followupClass", value = "随访类别，多类别“,”分割")
                                      @RequestParam(value = "followupClass", required = false) String followupClass,
                                      @ApiParam(name = "patientName", value = "居民姓名")
                                      @RequestParam(value = "patientName", required = false) String patientName,
                                      @ApiParam(value = "开始时间，格式（yyyy-MM-dd）", name = "startDate",required = false)
                                      @RequestParam(value = "startDate", required = false) String startDate,
                                      @ApiParam(value = "结束时间，格式（yyyy-MM-dd）", name = "endDate",required = false)
                                      @RequestParam(value = "endDate", required = false) String endDate,
                                      @ApiParam(name = "flag", value = "区分上门访视(1-是)")
                                      @RequestParam(value = "flag", required = false) String flag,
                                      @ApiParam(name = "type", value = "区分随访计划1、新增随访2、临时随访3、入户随访")
                                      @RequestParam(value = "type", required = false) Integer type){
        try {
            return success("操作成功！",followUpService.findFollowupByMonth(doctor,patient,followupClass,patientName,startDate,endDate,flag,type));
        }catch (Exception e){
            //日志文件中记录异常信息
            e.printStackTrace();
            //返回接口异常信息处理结果
            return failed( "操作失败！");
        }
    }

    @RequestMapping(value = "/getContinuedPrescriptionAsDoctor", method = RequestMethod.GET)
    @ApiOperation(value = "续方详情")
    public Envelop getContinuedPrescriptionAsDoctor(@RequestParam(required = true) @ApiParam(value = "续方CODE", name = "code") String code,
                                                   @RequestParam(required = true) @ApiParam(value = "团队长标识:1:是;2否", name = "type") String type) {
        try {
            return success( "查询成功！", followUpService.getContinuedPrescriptionAsDoctor(code));
        } catch (Exception e) {
            e.printStackTrace();
            return failed( "查询失败！");
        }
    }

    @ApiOperation("获取随访项目列表")
    @RequestMapping(value = "/getFollowupProject", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ResponseBody
    public Envelop getFollowupProject(@ApiParam(name = "id", value = "随访记录ID", defaultValue = "4")
                                     @RequestParam(value = "id", required = true) String id) {
        try {
            List<Map<String, String>> list = followUpService.getFollowupProject(id);

            return success( "获取随访项目列表成功！",  list);
        } catch (Exception e) {
            e.printStackTrace();
            return failed("获取随访项目列表失败！" + e.getMessage());
        }
    }

    @ApiOperation("完成随访记录")
    @RequestMapping(value = "/finishFollowup", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ResponseBody
    @ObserverRequired
    public Envelop finishFollowup(@ApiParam(name = "id", value = "随访记录ID", defaultValue = "")
                                 @RequestParam(value = "id", required = true) String id) {
        try {
            followUpService.finishFollowup(id);

            return success("完成随访记录成功！");
        } catch (Exception e) {
            e.printStackTrace();
            return failed("完成随访记录失败！" + e.getMessage());
        }
    }

    @ApiOperation("获取随访项目数据")
    @RequestMapping(value = "/getFollowupProjectData", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ResponseBody
    public Envelop getFollowupProjectData(@ApiParam(name = "id", value = "随访记录ID", defaultValue = "4")
                                         @RequestParam(value = "id", required = true) String id,
                                         @ApiParam(name = "followupProject", value = "随访项目")
                                         @RequestParam(value = "followupProject", required = false) String followupProject) {
        try {
//            Map<String, String> response = followUpService.getFollowupProjectData(id, followupProject);
            //修改为通过ES查询---2017.11.01--huangwenjie
            FollowupContentESDO followupContentESDO = followUpService.esGetFollowupProjectDataList(id, followupProject);
            String jsonString = JSON.toJSONString(followupContentESDO);
            Map maps = (Map)JSON.parse(jsonString);
            return success( "获取随访项目数据成功！", maps);
        } catch (Exception e) {
            e.printStackTrace();
            return failed( "获取随访项目数据失败！" + e.getMessage());
        }
    }
}
