package com.yihu.rehabilitation.controller.followUp;

import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.rehabilitation.aop.ObserverRequired;
import com.yihu.rehabilitation.service.followUp.VisitDetailService;
import com.yihu.rehabilitation.vo.entity.FollowupSevereMentalDisorderES;
import com.yihu.rehabilitation.vo.entity.NewbornFamilyVisitRecordES;
import com.yihu.rehabilitation.vo.entity.OlderPeopleHealthCheckRecordES;
import com.yihu.rehabilitation.vo.entity.PostpartumVisitRecordES;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

/**
 * 医生端：访视详情表单接口
 *
 * @author hmf on 2019-04-23
 */
@Controller
@RequestMapping(value = "/doctor/visitDetail", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(description = "医生端-访视详情表单接口")
public class DoctorVisitDetailController extends EnvelopRestEndpoint {
    @Autowired
    private VisitDetailService visitDetailService;

    @ApiOperation("保存孕产妇访视详情")
    @RequestMapping(value = "/savePostpartumVisitDetails", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ResponseBody
    public Envelop savePostpartumVisitDetails(@ApiParam(name = "followupId", value = "随访记录ID")
                                             @RequestParam(value = "followupId", required = true) String followupId,
                                              @ApiParam(name = "followupProject", value = "随访项目")
                                             @RequestParam(value = "followupProject", required = false) String followupProject,
                                              @ApiParam(name = "postpartumVisitDetails", value = "孕产妇访视详情数据")
                                             @RequestParam(value = "postpartumVisitDetails", required = true) String postpartumVisitDetails) {
        try {
            visitDetailService.savePostpartumVisitDetails(followupId, followupProject, postpartumVisitDetails, getUID());
            return success("保存孕产妇访视详情数据成功！");
        } catch (Exception e) {
            e.printStackTrace();
            return failed("保存孕产妇访视详情数据失败！" + e.getMessage());
        }
    }

    @ApiOperation("获取孕产妇访视详情")
    @RequestMapping(value = "/getPostpartumVisitDetails", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ResponseBody
    @ObserverRequired
    public Envelop getFollowupProjectData(@ApiParam(name = "followupId", value = "随访记录ID")
                                         @RequestParam(value = "followupId", required = true) String followupId,
                                         @ApiParam(name = "followupProject", value = "随访项目")
                                         @RequestParam(value = "followupProject", required = false) String followupProject) {
        try {
            PostpartumVisitRecordES postpartumVisitRecord = visitDetailService.getPostpartumVisitRecord(followupId, followupProject);
            return success("获取孕产妇访视详情成功！", postpartumVisitRecord);
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }

    @ApiOperation("保存严重精神障碍患者随访记录")
    @RequestMapping(value = "/saveFollowupSevereMentalDisorder", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ResponseBody
    public Envelop saveFollowupSevereMentalDisorder(@ApiParam(name = "followupId", value = "随访记录ID")
                                             @RequestParam(value = "followupId", required = true) String followupId,
                                             @ApiParam(name = "followupProject", value = "随访项目")
                                             @RequestParam(value = "followupProject", required = false) String followupProject,
                                             @ApiParam(name = "severeMentalDisorderData", value = "严重精神障碍患者详情数据")
                                             @RequestParam(value = "severeMentalDisorderData", required = true) String severeMentalDisorderData) {
        try {
            visitDetailService.saveFollowupSevereMentalDisorder(followupId, followupProject, severeMentalDisorderData, getUID());
            return success( "保存严重精神障碍患者随访记录成功！");
        } catch (Exception e) {
           e.printStackTrace();
            return failedException(e);
        }
    }

    @ApiOperation("获取严重精神障碍患者随访记录")
    @RequestMapping(value = "/getFollowupSevereMentalDisorder", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ResponseBody
    @ObserverRequired
    public Envelop getFollowupSevereMentalDisorder(@ApiParam(name = "followupId", value = "随访记录ID")
                                         @RequestParam(value = "followupId", required = true) String followupId,
                                         @ApiParam(name = "followupProject", value = "随访项目")
                                         @RequestParam(value = "followupProject", required = false) String followupProject) {
        try {
            FollowupSevereMentalDisorderES severeMentalDisorderES = visitDetailService.getFollowupSevereMentalDisorder(followupId, followupProject);
            return success( "获取严重精神障碍患者随访记录成功！", severeMentalDisorderES);
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }

    @ApiOperation("保存老人健康体检随访记录")
    @RequestMapping(value = "/saveOlderPeopleHealthCheckRecord", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ResponseBody
    public Envelop saveOlderPeopleHealthCheckRecord(@ApiParam(name = "followupId", value = "随访记录ID")
                                             @RequestParam(value = "followupId", required = true) String followupId,
                                             @ApiParam(name = "followupProject", value = "随访项目")
                                             @RequestParam(value = "followupProject", required = false) String followupProject,
                                             @ApiParam(name = "olderPeopleHealthCheckData", value = "老人健康体检随访记录")
                                             @RequestParam(value = "olderPeopleHealthCheckData", required = true) String olderPeopleHealthCheckData,
                                             @ApiParam(name = "bdStatus", value = "登记健康体检表状态（0-可编辑  1-不可编辑）")
                                             @RequestParam(value = "bdStatus", required = false) Integer bdStatus,
                                             @ApiParam(name = "pgStatus", value = "老年人生活自理能力评估表状态（0-可编辑  1-不可编辑）")
                                             @RequestParam(value = "pgStatus", required = false) Integer pgStatus ) {
        try {
            visitDetailService.saveOlderPeopleHealthCheckRecord(followupId, followupProject, olderPeopleHealthCheckData, getUID(),bdStatus,pgStatus);
            return success( "保存老人健康体检随访记录成功！");
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }

    @ApiOperation("获取老人健康体检随访记录")
    @RequestMapping(value = "/getOlderPeopleHealthCheckRecord", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ResponseBody
    @ObserverRequired
    public Envelop getOlderPeopleHealthCheckRecord(@ApiParam(name = "followupId", value = "随访记录ID")
                                         @RequestParam(value = "followupId", required = true) String followupId,
                                         @ApiParam(name = "followupProject", value = "随访项目")
                                         @RequestParam(value = "followupProject", required = false) String followupProject) {
        try {
            OlderPeopleHealthCheckRecordES olderPeopleHealthCheckRecord = visitDetailService.getOlderPeopleHealthCheckRecord(followupId, followupProject);
            return success( "获取老人随访服务随访成功！",  olderPeopleHealthCheckRecord);
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }

    @ApiOperation("保存新生儿家庭访视记录")
    @RequestMapping(value = "/saveNewbornFamilyVisitRecord", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ResponseBody
    public Envelop saveNewbornFamilyVisitRecord(@ApiParam(name = "followupId", value = "随访记录ID")
                                                   @RequestParam(value = "followupId", required = true) String followupId,
                                                   @ApiParam(name = "followupProject", value = "随访项目")
                                                   @RequestParam(value = "followupProject", required = false) String followupProject,
                                                   @ApiParam(name = "newbornFamilyVisitRecordData", value = "新生儿家庭访视记录")
                                                   @RequestParam(value = "newbornFamilyVisitRecordData", required = true) String newbornFamilyVisitRecordData) {
        try {
            visitDetailService.saveNewbornFamilyVisitRecord(followupId, followupProject, newbornFamilyVisitRecordData, getUID());
            return success( "保存新生儿家庭访视记录成功！");
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }

    @ApiOperation("获取新生儿家庭访视记录")
    @RequestMapping(value = "/getNewbornFamilyVisitRecord", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ResponseBody
    @ObserverRequired
    public Envelop getNewbornFamilyVisitRecord(@ApiParam(name = "followupId", value = "随访记录ID")
                                                  @RequestParam(value = "followupId", required = true) String followupId,
                                                  @ApiParam(name = "followupProject", value = "随访项目")
                                                  @RequestParam(value = "followupProject", required = false) String followupProject) {
        try {
            NewbornFamilyVisitRecordES newbornFamilyVisitRecordES = visitDetailService.getNewbornFamilyVisitRecord(followupId, followupProject);
            return success( "获取新生儿家庭访视记录成功！", newbornFamilyVisitRecordES);
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }

    @ApiOperation("获取上次填写内容")
    @RequestMapping(value = "/getLastVisitRecord", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ResponseBody
    public Envelop getLastVisitRecord(@ApiParam(name = "patientCode", value = "居民code")
                                         @RequestParam(value = "patientCode", required = true) String patientCode,
                                         @ApiParam(name = "followupClass", value = "随访类别(5.产后 6.新生儿 7.严重精神病 8.80岁以上老人 9.肺结核)")
                                         @RequestParam(value = "followupClass") String followupClass) {
        try {
            JSONObject jsonObject = visitDetailService.getLastVisitRecord(patientCode, followupClass);
            return success("获取上次随访记录成功！", jsonObject);
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }

    @ApiOperation("保存慢性妇科炎症日常随访")
    @RequestMapping(value = "/saveGynecologicalInflammationDaily", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ResponseBody
    public Envelop saveGynecologicalInflammationDaily(@ApiParam(name = "followupId", value = "随访记录ID")
                                                         @RequestParam(value = "followupId", required = true) String followupId,
                                                     @ApiParam(name = "jsonData", value = "随访表单数据", defaultValue = "{\"followup\":{\"followupDate\":\"2020-03-17\"},\"content\":{\"referralHospital\":\"2\",\"hospitalOrg\":\"\",\"hospitalOrgName\":\"医院\",\"illnessBetter\":\"1\",\"currentStatus\":\"2\",\"followClass\":\"1\"}}")
                                                   @RequestParam(value = "jsonData", required = true) String jsonData) {
        try {
            return success("保存慢性妇科炎症日常随访成功！",  visitDetailService.saveGynecologicalInflammationDaily(followupId, jsonData, getUID()));
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }

    @ApiOperation("获取慢性妇科炎症日常随访")
    @RequestMapping(value = "/getGynecologicalInflammationDaily", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ResponseBody
    @ObserverRequired
    public Envelop getGynecologicalInflammationDaily(@ApiParam(name = "followupId", value = "随访记录ID")
                                              @RequestParam(value = "followupId", required = true) String followupId) {
        try {
            JSONObject jsonObject = visitDetailService.getGynecologicalInflammationDaily(followupId);
            return success("获取慢性妇科炎症日常随访成功！",  jsonObject);
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }


    /**
     * type:
     * BreastFollowup //4.慢性乳腺疾病日常随访
     * CerebralApoplexyFollowup //7.脑卒中日常随访
     * CoronaryFollowup//1.冠心病日常随访
     * FattyLiverFollowUp//8.脂肪肝日常随访
     * NephropathyFollowup//5.慢性肾病日常随访
     * ProstateFollowup//3.慢性前列腺疾病日常随访
     * @param followupId
     * @param type
     * @return
     */
    @ApiOperation("获取随访病种日常随访")
    @RequestMapping(value = "/selectFollowupDiseaseById", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ResponseBody
    @ObserverRequired
    public Envelop selectFollowupDiseaseById(@ApiParam(name = "followupId", value = "随访记录ID")
                                                    @RequestParam(value = "followupId", required = true) String followupId,
                                                    @ApiParam(name = "type", value = "病种类型")
                                                    @RequestParam(value = "type", required = true) String type) {
        try {
            JSONObject jsonObject = visitDetailService.selectFollowupDiseaseById(followupId,type);
            return success( "获取随访病种日常随访成功！", jsonObject);
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }


    /**
     *
     * type:
     * BreastFollowup //4.慢性乳腺疾病日常随访
     * CerebralApoplexyFollowup //7.脑卒中日常随访
     * CoronaryFollowup//1.冠心病日常随访
     * FattyLiverFollowUp//8.脂肪肝日常随访
     * NephropathyFollowup//5.慢性肾病日常随访
     * ProstateFollowup//3.慢性前列腺疾病日常随访
     *
     * @param followupId
     * @param jsonData
     * @param type
     * @return
     */
    @ApiOperation("保存各类病种日常随访")
    @RequestMapping(value = "/saveFollowupDiseaseArchives", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ResponseBody
    public Envelop saveFollowupDiseaseArchives(@ApiParam(name = "followupId", value = "随访记录ID")
                                                     @RequestParam(value = "followupId", required = true) String followupId,
                                                     @ApiParam(name = "jsonData", value = "随访表单数据", defaultValue = "{\"followup\":{\"followupDate\":\"2020-03-17\"},\"content\":{\"referralHospital\":\"2\",\"hospitalOrg\":\"\",\"hospitalOrgName\":\"医院\",\"illnessBetter\":\"1\",\"currentStatus\":\"2\",\"followClass\":\"1\"}}")
                                                     @RequestParam(value = "jsonData", required = true) String jsonData,
                                                     @ApiParam(name = "type", value = "随访记录类型")
                                                         @RequestParam(value = "type", required = true) String type) {
        try {
            visitDetailService.saveFollowupDiseaseArchives(followupId, jsonData, getUID(),type);
            return success( "保存日常随访成功！");
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }

    @ApiOperation("保存慢性妇科炎症随访档案")
    @RequestMapping(value = "/saveGynecologicalInflammationFollowupArchives", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ResponseBody
    public Envelop saveGynecologicalInflammationFollowupArchives(@ApiParam(name = "followupId", value = "随访记录ID")
                                                     @RequestParam(value = "followupId", required = true) String followupId,
                                                     @ApiParam(name = "jsonData", value = "随访表单数据", defaultValue = "{\"followup\":{\"followupDate\":\"2020-03-17\"},\"content\":{\"referralHospital\":\"2\",\"hospitalOrg\":\"\",\"hospitalOrgName\":\"医院\",\"illnessBetter\":\"1\",\"currentStatus\":\"2\",\"followClass\":\"1\"}}")
                                                     @RequestParam(value = "jsonData", required = true) String jsonData) {
        try {
            return success( "保存慢性妇科炎症随访档案成功！", visitDetailService.saveGynecologicalInflammationFollowupArchives(followupId, jsonData, getUID()));
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }

    @ApiOperation("获取慢性妇科炎症随访档案")
    @RequestMapping(value = "/getGynecologicalInflammationFollowupArchives", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ResponseBody
    @ObserverRequired
    public Envelop getGynecologicalInflammationFollowupArchives(@ApiParam(name = "followupId", value = "随访记录ID")
                                                    @RequestParam(value = "followupId", required = true) String followupId) {
        try {
            JSONObject jsonObject = visitDetailService.getGynecologicalInflammationFollowupArchives(followupId);
            return success("获取慢性妇科炎症随访档案成功！",jsonObject);
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }

    @ApiOperation("保存慢性阻塞性肺病--首次问卷")
    @RequestMapping(value = "/saveCOPDFirstTimeQuestionnaire", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ResponseBody
    public Envelop saveCOPDFirstTimeQuestionnaire(@ApiParam(name = "followupId", value = "随访记录ID")
                                                     @RequestParam(value = "followupId", required = true) String followupId,
                                                     @ApiParam(name = "jsonData", value = "随访表单数据")
                                                     @RequestParam(value = "jsonData", required = true) String jsonData) {
        try {
            return success( "保存慢性阻塞性肺病--首次问卷成功！", visitDetailService.saveCOPDFirstTimeQuestionnaire(followupId, jsonData, getUID()));
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }

    @ApiOperation("获取慢性阻塞性肺病--首次问卷")
    @RequestMapping(value = "/getCOPDFirstTimeQuestionnaire", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ResponseBody
    @ObserverRequired
    public Envelop getCOPDFirstTimeQuestionnaire(@ApiParam(name = "followupId", value = "随访记录ID")
                                                    @RequestParam(value = "followupId", required = true) String followupId) {
        try {
            JSONObject jsonObject = visitDetailService.getCOPDFirstTimeQuestionnaire(followupId);
            return success("获取慢性阻塞性肺病--首次问卷成功！",  jsonObject);
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }

    @ApiOperation("保存慢性阻塞性肺病--日常随访")
    @RequestMapping(value = "/saveCOPDDailyFollowup", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ResponseBody
    public Envelop saveCOPDDailyFollowup(@ApiParam(name = "followupId", value = "随访记录ID")
                                                 @RequestParam(value = "followupId", required = true) String followupId,
                                                 @ApiParam(name = "jsonData", value = "随访表单数据")
                                                 @RequestParam(value = "jsonData", required = true) String jsonData) {
        try {
            return success( "保存慢性阻塞性肺病--日常随访成功！", visitDetailService.saveCOPDDailyFollowup(followupId, jsonData, getUID()));
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }

    @ApiOperation("获取慢性阻塞性肺病--日常随访")
    @RequestMapping(value = "/getCOPDDailyFollowup", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ResponseBody
    @ObserverRequired
    public Envelop getCOPDDailyFollowup(@ApiParam(name = "followupId", value = "随访记录ID")
                                                @RequestParam(value = "followupId", required = true) String followupId) {
        try {
            JSONObject jsonObject = visitDetailService.getCOPDDailyFollowup(followupId);
            return success( "获取慢性阻塞性肺病--日常随访成功！",  jsonObject);
        } catch (Exception e) {
            e.printStackTrace();
            return failedException(e);
        }
    }
}
