package com.yihu.rehabilitation.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.entity.base.org.BaseOrgDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.patient.PatientEvent;
import com.yihu.jw.entity.util.SystemConfEntity;
import com.yihu.jw.org.dao.BaseOrgDao;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.http.HttpClientUtil;
import com.yihu.rehabilitation.dao.PatientEventDao;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Created by hzp on 2016/11/24.
 * 市民健康档案服务
 */
@Service
public class PatientRecordService {


    @Autowired
    private BasePatientDao patientDao;


    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private BaseOrgDao hospitalDao;
    
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private RehabilitationInfoService rehabilitationInfoService;
    @Autowired
    private ZsRehabilitationInfoService zsRehabilitationInfoService;
    @Autowired
    private HttpClientUtil httpClientUtil;

    @Autowired
    private PatientEventDao patientEventDao;
    @Value("${sign.check_upload}")
    private String jwUrl;

    /**
     * 获取本地示例返参
     *
     * @param fid 服务id
     * @return
     * @throws Exception
     */
    public String getJosnFileResullt(String fid) throws Exception {
        String contentStr = null;
        Resource resource = new ClassPathResource("mqConfig/mqdata/" + fid + ".json");
        if (resource.exists()) {
            InputStream is = resource.getInputStream();
            contentStr = new BufferedReader(new InputStreamReader(is, "utf-8"))
                    .lines().collect(Collectors.joining(System.lineSeparator()));
//            contentStr =  replaceSpecialStr(contentStr);
        } else {
            throw new Exception("文件不存在");
        }
        return contentStr;
    }

    /******************************************************************************************************************/
    /**
     * 新的住院记录接口
     * @param patientCode
     * @param type
     * @param page
     * @param pageSize
     * @param lastTime
     * @param hospital
     * @return
     * @throws Exception
     */
    @Transactional(propagation= Propagation.NOT_SUPPORTED)
    public List<Map<String, String>> getAllEvent_new(String patientCode, String type, String page, String pageSize, String lastTime,String hospital) throws Exception {
        //获取患者
        BasePatientDO patient = patientDao.findById(patientCode).orElse(null);
        BaseOrgDO hos = hospitalDao.findByCode(hospital);

        //如果是中山医院，优先查找中山医院的接口
        if("922b8fa9-5aff-11e6-8344-fa163e8aee56".equals(hospital)){
            List<Map<String, String>> map = zsRehabilitationInfoService.zsHospitalRecord(patient,hos,page,pageSize);
            if(map.size()>0){
                return map;
            }
        }

        //获取基卫数据--
        List hlsit = new ArrayList();
        hlsit.add("922b91b8-5aff-11e6-8344-fa163e8aee56");//厦门市中医院
        hlsit.add("0a0b4ec0-5b04-11e6-8344-fa163e8aee56");//厦门大学附属第一医院
        hlsit.add("97fbc7707643471b8458-6def1ff92496");//厦门大学附属心血管病医院
        hlsit.add("0a11148d-5b04-11e6-8344-fa163e8aee56");//厦门市海沧医院
        hlsit.add("8d94ce907e4d4356b68f6c5d4d0d6bc9");//厦门市妇幼保健院
        hlsit.add("922b8fa9-5aff-11e6-8344-fa163e8aee56");//厦门大学附属中山医院
        if(hlsit.contains(hospital)){
            List<Map<String, String>> map = rehabilitationInfoService.getEmrList(patient.getIdcard(),null,null,Integer.parseInt(page),Integer.parseInt(pageSize));
            if(map.size()>0){
                return map;
            }
        }

        return new ArrayList<>();
    }

    /**
     * 获取门/急诊记录 + 住院记录
     */
    public List<Map<String, String>> getAllEvent(String patientCode, String type, String page, String pageSize, String lastTime) throws Exception {
        List<Map<String, String>> re = new ArrayList<>();

        //获取患者
        BasePatientDO patient = patientDao.findById(patientCode).orElse(null);
        //获取基卫数据
        String response = getResidentEventListJson(patient.getSsc(), type, page, pageSize);

        //获取app数据
        List<PatientEvent> eventList = new ArrayList<>();

        if (StringUtils.isEmpty(type)) {
            eventList = patientEventDao.findByPatient(patientCode);
        } else {
            eventList = patientEventDao.findByPatientAndEventType(patientCode, type);
        }

        if (!StringUtils.isEmpty(response)) {
            JSONArray array = new JSONArray(response);
            String max = "";
            String min = "";
            if (!"[{}]".equals(response)) {
                for (int i = 0; i < array.length(); i++) {
                    JSONObject item = array.getJSONObject(i);
                    if (i == 0) //最大值
                    {
                        max = item.optString("END_TIME");
                    } else if (i == array.length() - 1) //最小值
                    {
                        min = item.optString("END_TIME");
                    }

                    Map<String, String> map = new HashMap<>();
                    map.put("id", item.optString("EVENT"));
                    map.put("patient", patientCode);
                    map.put("eventDate", item.optString("END_TIME"));
                    map.put("eventType", item.optString("TYPE"));
                    map.put("orgName", item.optString("ORG_NAME"));
                    map.put("orgCode", item.optString("ORG_ID"));
                    map.put("dianosis", item.optString("DIAGNOSIS"));
                    map.put("createTime", item.optString("END_TIME"));
                    map.put("dataFrom", "1");//基卫数据
                    re.add(map);
                }
            }

            //过滤
            for (PatientEvent item : eventList) {
                String eventDate = DateUtil.dateToStrLong(item.getEventDate());
                int maxCompare = eventDate.compareTo(max);
                int minCompare = eventDate.compareTo(min);
                Boolean contain = false;
                if (maxCompare < 0 && minCompare >= 0)       //时间范围内
                {
                    contain = true;
                }

                //第一页特殊处理
                if ("1".equals(page) && maxCompare >= 0) {
                    contain = true;
                }

                //最后一页特殊处理
                if (!"1".equals(page) && (array.length() == 0 || "[{}]".equals(response)) && !StringUtils.isEmpty(lastTime)) {
                    int lastTimeCompare = eventDate.compareTo(lastTime);
                    if (lastTimeCompare < 0) {
                        contain = true;
                    }
                }

                if (contain) {
                    Map<String, String> map = new HashMap<>();
                    map.put("id", item.getId().toString());
                    map.put("patient", item.getPatient());
                    map.put("eventDate", DateUtil.dateToStrLong(item.getEventDate()));
                    map.put("eventType", item.getEventType());
                    map.put("orgName", item.getOrgName());
                    map.put("orgCode", item.getOrgCode());
                    map.put("dianosis", item.getDianosis());
                    map.put("createTime", DateUtil.dateToStrLong(item.getCreateTime()));
                    map.put("dataFrom", "2");   //APP数据
                    re.add(map);
                }
            }

            //排序
            re = sortMapList(re, "eventDate", "DESC");
        } else {
            if ("1".equals(page)) {
                for (PatientEvent item : eventList) {
                    Map<String, String> map = new HashMap<>();
                    map.put("id", item.getId().toString());
                    map.put("patient", item.getPatient());
                    map.put("eventDate", DateUtil.dateToStrLong(item.getEventDate()));
                    map.put("eventType", item.getEventType());
                    map.put("orgName", item.getOrgName());
                    map.put("orgCode", item.getOrgCode());
                    map.put("dianosis", item.getDianosis());
                    map.put("createTime", DateUtil.dateToStrLong(item.getCreateTime()));
                    map.put("dataFrom", "2");   //APP数据
                    re.add(map);
                }
            }
        }

        return re;
    }
    /**
     * 获取门/急诊记录 + 住院记录
     */
    public String getResidentEventListJson(String strSSID,String type,String page,String pageSize)  throws Exception
    {
        if(SystemConfEntity.getInstance().getEhrUsed())     //演示环境
        {
            return "";
        }
        else {
            String re = "";
            String url = jwUrl + "/third/smjk/ResidentEventList";

            List<NameValuePair> params = new ArrayList<>();
            params.add(new BasicNameValuePair("strSSID", strSSID));
            if (!StringUtils.isEmpty(type)) {
                params.add(new BasicNameValuePair("type", type));
            }
            params.add(new BasicNameValuePair("page", page));
            params.add(new BasicNameValuePair("pageSize", pageSize));

            String response = httpClientUtil.post(url, params, "UTF-8");

            if (!StringUtils.isEmpty(response)) {
                JSONObject responseObject = new JSONObject(response);
                int status = responseObject.getInt("status");
                if (status == 200) {
                    String data = responseObject.getString("data");
                    if (!StringUtils.isEmpty(data) && data.startsWith("error")) {
                        throw new Exception(data);
                    } else {
                        JSONObject jsonData = new JSONObject(data);
                        JSONArray jsonArray = jsonData.getJSONArray("EventList");
                        re = jsonArray.toString();
                    }
                } else {
                    throw new Exception(responseObject.getString("msg"));
                }
            } else {
                throw new Exception("null response.");
            }

            if(re.contains("[{}]"))
            {
                re = re.replace("[{}]","[]");
            }

            return re;
        }
    }
    /**
     * 字符串List排序
     */
    public List<Map<String, String>> sortMapList(List<Map<String, String>> mapList, final String sort, final String order) {
        Collections.sort(mapList, new Comparator<Map<String, String>>() {
            public int compare(Map<String, String> o1, Map<String, String> o2) {
                String map1value = o1.get(sort);
                String map2value = o2.get(sort);
                if ("DESC".equals(order.toUpperCase())) {
                    return map2value.compareTo(map1value);
                } else {
                    return map1value.compareTo(map2value);
                }
            }
        });
        return mapList;
    }


}
