package com.yihu.rehabilitation.service;


import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.rehabilitation.RehabilitationguidanceDO;
import com.yihu.jw.entity.rehabilitation.RehabilitationguidancePatientDO;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.rehabilitation.dao.RehabilitationGuidanceDao;
import com.yihu.rehabilitation.dao.RehabilitationGuidancePatientDao;
import com.yihu.rehabilitation.service.message.RehabilitationMessageService;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springside.modules.persistence.DynamicSpecifications;
import org.springside.modules.persistence.SearchFilter;

import java.util.*;


/**
 * Created by wangzhinan on 2018/8/29.
 */
@Service
@Transactional
public class RehabilitationGuidanceService {
    private Logger logger = LoggerFactory.getLogger(RehabilitationGuidanceService.class);

    @Autowired
    private RehabilitationGuidanceDao rehabilitationGuidanceDao;
    @Autowired
    private RehabilitationGuidancePatientDao rehabilitationGuidancePatientDao;
    @Autowired
    private RehabilitationMessageService rehabilitationMessageService;
    @Autowired
    private BaseDoctorDao doctorDao;
    @Autowired
    private BasePatientDao patientDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    /**
     * 康复指导类型标题
     * @return
     */
    public List<Map<String,Object>> selectType(String planId){
        String disease="";
        if (StringUtils.isNoneBlank(planId)){
            String sql ="select * from wlyy_patient_rehabilitation_plan where id='"+planId+"' ";
            Map<String,Object> map = jdbcTemplate.queryForMap(sql);
            if (map.get("disease")!=null&&StringUtils.isNoneBlank(map.get("disease").toString())){
                disease = map.get("disease").toString();
            }
        }
        StringBuffer sql = new StringBuffer("select g.type,g.type_name as typeName,COUNT(g.id) as number from wlyy_rehabilitation_guidance g where g.del=1 ");
        if (StringUtils.isNoneBlank(disease)){
            sql.append("and g.type in ( '").append(disease.replace(",","','")).append("') ");
        }
        sql.append("group by g.type");
        List<Map<String,Object>> mapList = jdbcTemplate.queryForList(sql.toString());
        return mapList;
    }


    /**
     * 查询康复指导列表
     *
     * @param type
     * @param code
     * @param page
     * @param pagesize
     * @return
     */
    public Page<RehabilitationguidanceDO> select(String title, Integer type, String code, String planId, Integer page, Integer pagesize){
        String disease="";
        if (StringUtils.isNoneBlank(planId)){
            String sql ="select * from wlyy_patient_rehabilitation_plan where id='"+planId+"' ";
            Map<String,Object> map = jdbcTemplate.queryForMap(sql);
            if (map.get("disease")!=null&&StringUtils.isNoneBlank(map.get("disease").toString())){
                disease = map.get("disease").toString();
            }
        }
        if(page >=1){
            page --;
        }

        if (pagesize <= 0) {
            pagesize = 10;
        }
        StringBuffer sql =  new StringBuffer("select * from wlyy_rehabilitation_guidance where 1=1 and del =1 ");
        StringBuffer sqlCounts =  new StringBuffer("select count(1) from wlyy_rehabilitation_guidance where 1=1 and del =1 ");
        if (StringUtils.isNoneBlank(code)){
            sql.append("and code='"+code+"' ");
            sqlCounts.append("and code='"+code+"' ");
        }
        if (type!=null){
            sql.append("and type='"+type+"' ");
            sqlCounts.append("and type='"+type+"' ");
        }
        if (StringUtils.isNoneBlank(title)){
            sql.append("and title like '%"+title+"%' ");
            sqlCounts.append("and title like '%"+title+"%' ");
        }
        if (StringUtils.isNoneBlank(disease)){
            sql.append("and type in ( '").append(disease.replace(",","','")).append("') ");
            sqlCounts.append("and type in ( '").append(disease.replace(",","','")).append("') ");
        }

        sql.append("order by czrq desc limit "+page*pagesize+","+(page+1)*pagesize);
        List<RehabilitationguidanceDO> list = jdbcTemplate.query(sql.toString(),new BeanPropertyRowMapper<>(RehabilitationguidanceDO.class));
        Long totalCount = jdbcTemplate.queryForObject(sqlCounts.toString(),Long.class);
        for (RehabilitationguidanceDO temp:list){
            String contents = temp.getContent();
//            System.out.println(contents);
            temp.setContent(StringEscapeUtils.unescapeJava(contents));
//            System.out.println(StringEscapeUtils.unescapeJava(contents));
        }
        PageRequest pageRequest = PageRequest.of(page, pagesize);
        if (list.size() <= 0) {
            return new PageImpl(new ArrayList<>(), pageRequest, 0);
        }
        return new PageImpl(list, pageRequest, totalCount);
    }




    /**
     * 发送康复指导
     *
     * @param rehabilitationguidancePatientDOS
     * @return
     */
    public List<RehabilitationguidancePatientDO> sendGuidanceToPatient(List<RehabilitationguidancePatientDO> rehabilitationguidancePatientDOS,String planId) throws Exception {
        for (RehabilitationguidancePatientDO rehabilitationguidancePatientDO:rehabilitationguidancePatientDOS){
            RehabilitationguidanceDO rehabilitationguidanceDO = rehabilitationGuidanceDao.findByCode(rehabilitationguidancePatientDO.getArticle());
            rehabilitationguidanceDO.setUseCount(rehabilitationguidanceDO.getUseCount()+1);
            rehabilitationGuidanceDao.save(rehabilitationguidanceDO);
            BasePatientDO patient = patientDao.findById(rehabilitationguidancePatientDO.getPatient()).orElse(null);
            BaseDoctorDO doctor = doctorDao.findById(rehabilitationguidancePatientDO.getDoctor()).orElse(null);
            rehabilitationguidancePatientDO.setAttachedTitle(rehabilitationguidanceDO.getTitle());
            if (patient!=null){
                rehabilitationguidancePatientDO.setName(patient.getName());
            }
            if (doctor!=null){
                rehabilitationguidancePatientDO.setDoctorName(doctor.getName());
            }
            rehabilitationguidancePatientDO.setAttachedContent(rehabilitationguidanceDO.getContent());
            rehabilitationguidancePatientDO.setCzrq(new Date());
            rehabilitationguidancePatientDO.setIsRead(0);
            rehabilitationguidancePatientDO.setSendStatus(0);
            rehabilitationguidancePatientDO = rehabilitationGuidancePatientDao.save(rehabilitationguidancePatientDO);
            rehabilitationMessageService.sendSpecialistWeixinMessage(rehabilitationguidancePatientDO.getPatient(),rehabilitationguidancePatientDO.getDoctor(),1,rehabilitationguidancePatientDO.getArticle(),planId,null);
        }
        return rehabilitationguidancePatientDOS;

    }


    /**
     * 查看发送的康复指导
     * @param patient
     * @param doctor
     * @param relationCode
     * @return
     */
    public List<RehabilitationguidancePatientDO> selectByRelationCode(String patient,String doctor,String relationCode){
        if (StringUtils.isNoneBlank(relationCode)){
            return rehabilitationGuidancePatientDao.findByRelationCode(relationCode);
        }else {
            return rehabilitationGuidancePatientDao.findByPatientAndDoctor(patient,doctor);
        }
    }

    public Page<RehabilitationguidancePatientDO> selectPatientGuidances(String patient,String doctor,String code,Integer page,Integer pagesize){
        if(page >=1){
            page --;
        }

        if (pagesize <= 0) {
            pagesize = 10;
        }
        // 排序
        Sort sort = Sort.by(Sort.Direction.DESC, "czrq");
        // 分页信息
        PageRequest pageRequest = PageRequest.of(page, pagesize, sort);
        // 设置查询条件
        Map<String, SearchFilter> filters = new HashMap<String, SearchFilter>();
        // 指定患者
        if (StringUtils.isNoneBlank(code)) {
            filters.put("code", new SearchFilter("code", SearchFilter.Operator.EQ, code));
        }
        if (StringUtils.isNoneBlank(patient)) {
            filters.put("patient", new SearchFilter("patient", SearchFilter.Operator.EQ, patient));
        }

        if (StringUtils.isNoneBlank(doctor)) {
            filters.put("doctor", new SearchFilter("doctor", SearchFilter.Operator.EQ, doctor));
        }
        Specification<RehabilitationguidancePatientDO> spec = DynamicSpecifications.bySearchFilter(filters.values(), RehabilitationguidancePatientDO.class);
        Page<RehabilitationguidancePatientDO> rehabilitationguidancePatientDOS = rehabilitationGuidancePatientDao.findAll(spec,pageRequest);
        for (RehabilitationguidancePatientDO rehabilitationguidancePatientDO:rehabilitationguidancePatientDOS){
            BasePatientDO patient1 = patientDao.findById(rehabilitationguidancePatientDO.getPatient()).orElse(null);
            rehabilitationguidancePatientDO.setPhoto(patient1.getPhoto());
        }
        return rehabilitationguidancePatientDOS;
    }
}
