package com.yihu.rehabilitation.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

import com.yihu.jw.entity.base.org.BaseOrgDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.http.HttpClientUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.core.PreparedStatementCreatorFactory;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;
import org.springframework.stereotype.Service;

import java.sql.Types;
import java.util.*;

/**
 * Created by yeshijie on 2020/7/30.
 */
@Service
public class ZsRehabilitationInfoService {

    //基卫服务地址
    @Value("${sign.check_upload}")
    private String jwUrl;
    @Autowired
    private HttpClientUtil httpClientUtil;
    @Autowired
    private JdbcTemplate jdbcTemplate;

    /**
     * 获取中山医院住院记录
     * @param patient
     * @param hos
     * @param page
     * @param size
     * @return
     */
    public List<Map<String, String>> zsHospitalRecord(BasePatientDO patient, BaseOrgDO hos, String page, String size){
        List<Map<String, String>> res = new ArrayList<>();
        String sql = "select DISTINCT a.PATIENT_ID,a.VISIT_ID,a.CREATE_DATE_TIME,a.FILE_INDEX_ID_EMT " +
                "from sde_jhemr_emr.mr_file_index a,sde_jhemr_medrec.pat_master_index b,sde_jhemr_emr.mr_file_xmlnodecontent c " +
                "where a.patient_id = b.patient_id and b.id_no = '"+patient.getIdcard()+"' and a.mr_class = 'A2' and a.file_index_id_emt = c.file_index_id_node " +
                " order by a.CREATE_DATE_TIME desc";
        //网闸会拦截sql语句，通过把sql语句存表
        String addSql = "INSERT INTO system_properties (`value`) VALUES (?) ";
        KeyHolder keyHolder = new GeneratedKeyHolder();
        PreparedStatementCreatorFactory pscf = new PreparedStatementCreatorFactory(    // 创建语句
                addSql, Types.VARCHAR
        );
        // 关键方法
        pscf.setReturnGeneratedKeys(true);
        PreparedStatementCreator psc = pscf.newPreparedStatementCreator(Arrays.asList(sql));
        jdbcTemplate.update(psc, keyHolder);

        String url = jwUrl +"/third/zsHospital/createSQLQuery?id="+keyHolder.getKey()+"&page="+page+"&size="+size;
        String re = httpClientUtil.get(url,"UTF-8");
//        String re = "{\"msg\":\"success\",\"data\":[{\"PATIENT_ID\":\"413352\",\"FILE_INDEX_ID_EMT\":\"4743AE3083CC2BFC20200404080036\",\"VISIT_ID\":3,\"CREATE_DATE_TIME\":1585874950000},{\"PATIENT_ID\":\"413352\",\"FILE_INDEX_ID_EMT\":\"E2C7A1C3D15B992A20191012083841\",\"VISIT_ID\":2,\"CREATE_DATE_TIME\":1570696957000}],\"status\":200}";
        System.out.println("re:"+re);
        JSONObject jsonObject = JSONObject.parseObject(re);
        if(jsonObject.getInteger("status")==200){
            JSONArray data = jsonObject.getJSONArray("data");
            for (int i=0;i<data.size();i++){
                Map<String, String> map = new HashMap<>();
                JSONObject item = data.getJSONObject(i);
                map.put("id", item.getString("FILE_INDEX_ID_EMT"));
                map.put("patient", item.getString("PATIENT_ID"));
                map.put("eventDate", DateUtil.dateToStrLong(new Date(item.getLong("CREATE_DATE_TIME"))));
                map.put("eventType", "2");//中山医院的都是住院记录
                map.put("orgName", hos.getName());
                map.put("orgCode", hos.getCode());
                map.put("dianosis", "");//诊断
                map.put("createTime", DateUtil.dateToStrLong(new Date(item.getLong("CREATE_DATE_TIME"))));
                map.put("dataFrom", "3");//中山医院
                res.add(map);
            }
        }
        return res;
    }

    /**
     * 中山医院病历明细
     * @param patient
     * @param id
     * @return
     */
    public JSONObject zsHospitalRecordDetail(String patient,String id){
        JSONObject res = new JSONObject();
        String sql = "select a.PATIENT_ID,b.NAME,b.SEX,a.DEPT_NAME,a.BED_LABEL,a.VISIT_ID,a.CREATE_DATE_TIME,a.FILE_INDEX_ID_EMT,c.MR_LAYER_CONTENT,c.FIELD_NAME " +
                "from sde_jhemr_emr.mr_file_index a,sde_jhemr_medrec.pat_master_index b,sde_jhemr_emr.mr_file_xmlnodecontent c " +
                "where a.patient_id = b.patient_id and b.patient_id = '"+patient+"' and a.mr_class = 'A2' and a.file_index_id_emt = c.file_index_id_node " +
                "and a.FILE_INDEX_ID_EMT = '"+id+"' and c.FIELD_NAME in ('出院医嘱','实验室检查及器械检查结果','住院经过','出院诊断','入院诊断') order by a.CREATE_DATE_TIME desc";
        //网闸会拦截sql语句，通过把sql语句存表
        String addSql = "INSERT INTO system_properties (`value`) VALUES (?) ";
        KeyHolder keyHolder = new GeneratedKeyHolder();
        PreparedStatementCreatorFactory pscf = new PreparedStatementCreatorFactory(    // 创建语句
                addSql,Types.VARCHAR
        );
        // 关键方法
        pscf.setReturnGeneratedKeys(true);
        PreparedStatementCreator psc = pscf.newPreparedStatementCreator(Arrays.asList(sql));
        jdbcTemplate.update(psc, keyHolder);

        String url = jwUrl +"/third/zsHospital/createSQLQuery?id="+keyHolder.getKey()+"&page=1&size=20";
        String re = httpClientUtil.get(url,"UTF-8");
//        String re = "{\"msg\":\"success\",\"data\":[{\"BED_LABEL\":\"821床\",\"DEPT_NAME\":\"眼科病区\",\"MR_LAYER_CONTENT\":\".1、门诊随诊：长期定期复诊。院外继续控制好血糖。2、可乐必妥眼水、典必殊眼水 2h/次，点右眼。3、出院1周陈跃主任门诊复查（周五上午）；出院1月、3月、6月、1年、2年谢婷玉副主任门诊复查（周一、四上午/周三下午）不适随诊。\",\"VISIT_ID\":3,\"SEX\":\"男\",\"PATIENT_ID\":\"413352\",\"FILE_INDEX_ID_EMT\":\"4743AE3083CC2BFC20200404080036\",\"FIELD_NAME\":\"出院医嘱\",\"CREATE_DATE_TIME\":1585874950000,\"NAME\":\"陈水合\"},{\"BED_LABEL\":\"821床\",\"DEPT_NAME\":\"眼科病区\",\"MR_LAYER_CONTENT\":\" 1.右眼代谢性白内障 2.左眼人工晶体眼 3.2型糖尿病 4.高血压病 \",\"VISIT_ID\":3,\"SEX\":\"男\",\"PATIENT_ID\":\"413352\",\"FILE_INDEX_ID_EMT\":\"4743AE3083CC2BFC20200404080036\",\"FIELD_NAME\":\"出院诊断\",\"CREATE_DATE_TIME\":1585874950000,\"NAME\":\"陈水合\"},{\"BED_LABEL\":\"821床\",\"DEPT_NAME\":\"眼科病区\",\"MR_LAYER_CONTENT\":\"1.右眼代谢性白内障 2.左眼人工晶体眼 3.2型糖尿病 4.高血压病 \",\"VISIT_ID\":3,\"SEX\":\"男\",\"PATIENT_ID\":\"413352\",\"FILE_INDEX_ID_EMT\":\"4743AE3083CC2BFC20200404080036\",\"FIELD_NAME\":\"入院诊断\",\"CREATE_DATE_TIME\":1585874950000,\"NAME\":\"陈水合\"},{\"BED_LABEL\":\"821床\",\"DEPT_NAME\":\"眼科病区\",\"MR_LAYER_CONTENT\":\"2020-04-01丙型肝炎抗体测定(Anti-HCV)(化学发光)|术前感染筛查八项：乙肝表面抗原 阳性[1207.420]IU/mL阳性、乙肝e抗体 阳性[0.010]S/CO阳性、乙肝核心抗体 阳性[0.020]S/CO阳性2020-04-01常规生化全套检查：γ-谷氨酰基转移酶 83.6U/L↑、总胆固醇 6.27mmol/L↑、载脂蛋白-A1 2.10g/L↑、糖 10.16mmol/L↑、碳酸氢盐 31.0mmol/L↑2020-04-01糖化血红蛋白测定：糖化血红蛋白 7.9%↑2020-04-01凝血筛查：凝血酶原时间 11.5sec2020-04-01血常规检查(进口五分类)|血常规(成人)：*白细胞计数 6.04×10^9/L。心电图示1.窦性心律2.正常心电图。胸部CT示双肺上叶轻度肺气肿。双肺上叶、左肺下叶背段小结节影，大致同前建议随访复查。右肺中叶慢性炎症或纤维灶，伴局限性不张，大致同前。纵隔内多发小淋巴结影。\",\"VISIT_ID\":3,\"SEX\":\"男\",\"PATIENT_ID\":\"413352\",\"FILE_INDEX_ID_EMT\":\"4743AE3083CC2BFC20200404080036\",\"FIELD_NAME\":\"实验室检查及器械检查结果\",\"CREATE_DATE_TIME\":1585874950000,\"NAME\":\"陈水合\"},{\"BED_LABEL\":\"821床\",\"DEPT_NAME\":\"眼科病区\",\"MR_LAYER_CONTENT\":\"入院后完善检查，于2020-04-02局麻下行\\\"右眼白内障超声乳化联合人工晶体植入术\\\"。植入人工晶体（RAYNER  A值118.0、+22.0D）。术顺，术后局部抗生素、激素应用，视力提高。\",\"VISIT_ID\":3,\"SEX\":\"男\",\"PATIENT_ID\":\"413352\",\"FILE_INDEX_ID_EMT\":\"4743AE3083CC2BFC20200404080036\",\"FIELD_NAME\":\"住院经过\",\"CREATE_DATE_TIME\":1585874950000,\"NAME\":\"陈水合\"}],\"status\":200}";
        System.out.println("re:"+re);
        JSONObject jsonObject = JSONObject.parseObject(re);
        if(jsonObject.getInteger("status")==200){
            JSONArray data = jsonObject.getJSONArray("data");
            for (int i=0;i<data.size();i++){
                JSONObject item = data.getJSONObject(i);
                String fileName = item.getString("FIELD_NAME");
                String content = item.getString("MR_LAYER_CONTENT").replaceAll("\\u0000","");
                if(i == 0){
                    res.put("name",item.getString("NAME"));
                    res.put("sex",item.getString("SEX"));
                    res.put("deptName",item.getString("DEPT_NAME"));
                    res.put("bedLabel",item.getString("BED_LABEL"));
                }
                res.put(fileName,content);
            }
        }
        return res;
    }

    private String getFileName(String fileName){
        String res;
        switch (fileName){
            case "出院医嘱":
                res = "dischargeOrder";
                break;
            case "实验室检查及器械检查结果":
                res = "inspectionResults";
                break;
            case "住院经过":
                res = "courseInHospital";
                break;
            case "出院诊断":
                res = "dischargeDiagnosis";
                break;
            case "入院诊断":
                res = "admissionDiagnosis";
                break;
            default:
                res = fileName;
                break;
        }
        return res;
    }
}
