package com.yihu.rehabilitation.service.consult;

import com.yihu.jw.doctor.dao.BaseDoctorHospitalDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.doctor.BaseDoctorHospitalDO;
import com.yihu.jw.entity.base.im.ConsultDo;
import com.yihu.jw.entity.base.im.ConsultTeamDo;
import com.yihu.jw.entity.base.im.ConsultTeamDoctorDo;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.hospital.prescription.WlyyPrescriptionDO;
import com.yihu.jw.entity.rehabilitation.SpecialDiseaseMessagesDO;
import com.yihu.jw.entity.util.TransforSqlUtl;
import com.yihu.jw.hospital.HospitalDao;
import com.yihu.jw.hospital.dict.WlyyHospitalSysDictDao;
import com.yihu.jw.hospital.message.dao.SystemMessageDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionDiagnosisDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionExpressageDao;
import com.yihu.jw.hospital.prescription.dao.PrescriptionInfoDao;
import com.yihu.jw.hospital.prescription.service.PrescriptionLogService;
import com.yihu.jw.im.dao.ConsultTeamDao;
import com.yihu.jw.im.dao.ConsultTeamDoctorDao;
import com.yihu.jw.im.dao.ConsultTeamLogDao;
import com.yihu.jw.im.util.ImUtil;
import com.yihu.jw.message.service.MessageService;
import com.yihu.jw.util.http.HttpClientUtil;
import com.yihu.jw.utils.hibernate.HibenateUtils;
import com.yihu.rehabilitation.dao.SignFamilyDao;
import com.yihu.rehabilitation.dao.SpecialDiseaseMessagesDao;
import com.yihu.rehabilitation.dao.followUp.FollowUpDao;
import org.json.JSONArray;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springside.modules.utils.Clock;

import javax.annotation.PostConstruct;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * 網絡諮詢类.
 * test
 *
 * @author George
 */
// Spring Service Bean的标识.
@Component
@Transactional(rollbackFor = Exception.class)
public class ConsultTeamService extends ConsultService {
    
    private Logger logger = LoggerFactory.getLogger(ConsultTeamService.class);

    private Clock clock = Clock.DEFAULT;

    // 咨询详细记录
    @Autowired
    private ConsultTeamLogDao consultTeamLogDao;
    @Autowired
    private ConsultTeamDoctorDao consultTeamDoctorDao;
    @Autowired
    private SignFamilyDao signFamilyDao;
    @Autowired
    private ConsultTeamDao consultTeamDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private SystemMessageDao messageDao;
    @Autowired
    private PrescriptionDao prescriptionDao;
    @Autowired
    private PrescriptionLogService prescriptionLogService;
    @Autowired
    private PrescriptionInfoDao prescriptionInfoDao;
    @Autowired
    private PrescriptionDiagnosisDao prescriptionDiagnosisDao;
    @Autowired
    private ImUtil ImUtill;
    @Autowired
    private HttpClientUtil httpClientUtil;
    @Autowired
    private FollowUpDao followupDao;
    @Autowired
    private MessageService messageService;
    @Autowired
    private WlyyHospitalSysDictDao systemDictDao;
    @Autowired
    private PrescriptionExpressageDao expressageDao;
    @Autowired
    private HospitalDao hospitalDao;

    @Autowired
    private SpecialDiseaseMessagesDao specialDiseaseMessagesDao;
    @Autowired
    private HibenateUtils hibenateUtils;
    @Autowired
    private BaseDoctorHospitalDao baseDoctorHospitalDao;

    //健康问题 高血压
    private static final String gxy = "HP0093";
    //健康问题 糖尿病
    private static final String tnb = "HP0047";


    Map<Integer, String> relations = new HashMap<>();

    @Value("${im.im_list_get}")
    private String im_list_get;
    @Value("${im.data_base_name}")
    private String imdb;

    @PostConstruct
    public void init() {
        relations.put(0, "其他");
        relations.put(1, "父亲");
        relations.put(2, "母亲");
        relations.put(3, "老公");
        relations.put(4, "老婆");
        relations.put(5, "儿子");
        relations.put(6, "女儿");
        relations.put(7, "未知");
    }
    /**
     * 按类型查找居民的咨询记录
     * @return
     */
    public List<Map<String,Object>> findConsultRecordByType(String patient,Integer page,int pagesize,Integer type,String title){
        if(!StringUtils.isEmpty(title)){
            title="%"+title+"%";
            return findConsultNew(patient,page,pagesize,type, title);
        }else{
            return findConsultNew(patient,page,pagesize,type,"");
        }

    }
    public List<Map<String,Object>> findConsultNew(String patient, Integer page, Integer pagesize, Integer type, String title){
        String sql = "SELECT\n" +
                "\ta.id,\n" +
                "\ta.type,\n" +
                "\ta.title,\n" +
                "\ta.symptoms,\n" +
                "\ta.czrq,\n" +
                "\tb.STATUS as \"status\",\n" +
                "\tb.doctor,\n" +
                "\tb.evaluate,\n" +
                "\ta.relation_code \n" +
                "FROM\n" +
                "\twlyy_consult a,\n" +
                "\twlyy_consult_team b \n" +
                "WHERE\n" +
                "\ta.CODE = b.consult \n" ;

        // 排序
        if (org.apache.commons.lang3.StringUtils.isNoneBlank(patient)){
            sql+=" AND a.patient = '"+patient+"'";
        }
        if (type!=null){
            sql+=" AND a.type = "+type+"";
        }
        if (org.apache.commons.lang3.StringUtils.isNoneBlank(title)){
            sql+=" AND a.symptoms like '%"+title+"%'";
        }
        sql+="ORDER BY\n" +
                "\ta.czrq DESC";
        return hibenateUtils.createSQLQuery(sql,page,pagesize);
    }
    public ConsultTeamDo findByCode(String code) {
        return consultTeamDao.findByConsult(code);
    }

    /**
     * 查询居民与某个医生未结束的咨询
     *
     * @param patient 居民
     * @param doctor  医生
     * @return
     */
    public List<Map<String,Object>> getUnfinishedConsult(String patient, String doctor) {
        return findUnfinishedConsultType(patient, doctor);
    }
    public List<Map<String,Object>> findUnfinishedConsultType(String patient,String doctor){
        String sql = "SELECT\n" +
                "\ta \n" +
                "FROM\n" +
                "\twlyy_consult_team a,\n" +
                "\twlyy_consult_team_doctor b \n" +
                "WHERE\n" +
                "\ta.consult = b.consult \n" ;

        if (!StringUtils.isEmpty(patient)){
            sql+="   AND a.patient = '"+patient+"'";
        }
        if (!StringUtils.isEmpty(doctor)){
            sql+="   AND b.to_doctor = '"+doctor+"'";
        }
        sql+= "\tAND a.del = '1' \n" +
                "\tAND a.type <> 8 \n" +
                "\tAND a.STATUS = 0";
        return  hibenateUtils.createSQLQuery(sql);
    }

    public JSONObject getConsultDoctorForConsulting(String patientCode){
        JSONObject result = new JSONObject();
        String sql="SELECT DISTINCT p.patient, p.name AS patient_name, d.`id` as special_code, d.`name` as special_name \n" +
                "FROM wlyy_patient_rehabilitation_plan p, base_doctor d \n" +
                "WHERE p.patient = '"+patientCode+"' AND d.`code`= p.create_user  GROUP BY p.patient, p.create_user,p.team_code";
        sql = TransforSqlUtl.wlyy_patient_rehabilitation_planName2(sql);
        List<Map<String,Object>> list = jdbcTemplate.queryForList(sql);
        result.put("ConsultTeamList",list);
        for (Map<String,Object> map:list){
            String doctor=map.get("special_code").toString();
            List<BaseDoctorHospitalDO> depts=baseDoctorHospitalDao.findByDoctorCode(doctor);
            if (depts!=null&&depts.size()>0){
                map.put("dept",depts.get(0).getDeptCode());
                map.put("deptName",depts.get(0).getDeptName());
                map.put("org",depts.get(0).getOrgCode());
                map.put("orgName",depts.get(0).getOrgName());
            }
        }
        return result;
    }


    /**
     *   患者直接发起康复咨询
     * @param ct
     * @param patient
     * @return
     * @throws Exception
     */
    public JSONObject addRecoverConsult(ConsultTeamDo ct,String patient,String agent) throws Exception {
        JSONObject re = new JSONObject();
        if (exist(patient, ct.getType(),ct.getDoctor())) {//判断是否有未结束的咨询移到同步方法中
            re.put("status", -3);
            return re;
        }
        if (patient.equals(agent)) {
            agent = null;
        }

        //议题添加团队内所有成员
        com.alibaba.fastjson.JSONObject users = new com.alibaba.fastjson.JSONObject();
        String specialDoctorCode="";
        users.put(ct.getDoctor(), 0);
        //ct.setAdminTeamId(teamId);

        // 设置患者信息
        ct.setPatient(patient);
        // 查询患者信息
        BasePatientDO tempPatient = patientDao.findById(patient).orElse(null);
        // 设置患者姓名
        ct.setName(tempPatient.getName());
        // 设置患者生日
        ct.setBirthday(tempPatient.getBirthday());
        //新增性别
        ct.setSex(tempPatient.getSex());
        // 设置患者头像
        ct.setPhoto(tempPatient.getPhoto());
        // 设置操作日期
        ct.setCzrq(new Date());
        ct.setDel("1");
        ct.setStatus(0);
        ct.setEvaluate(0);
        // 医生未读数量为1
        ct.setDoctorRead(1);
        // 添加咨询记录
        ConsultDo consult = addConsult(ct.getPatient(), tempPatient.getName()+"康复咨询", ct.getSymptoms(), ct.getImages(), ct.getType());
        // 设置关联指导
        consult.setGuidance(ct.getGuidance());
        // 设置咨询标识
        ct.setConsult(consult.getId());

        //推送给IM去创建议题，取得成员消息
//        JSONObject messages = ImUtill.getCreateTopicMessage(patient, tempPatient.getName(), consult.getTitle(), "咨询问题："+consult.getSymptoms(), consult.getImages());
        com.alibaba.fastjson.JSONObject messages = ImUtill.getCreateTopicMessage(patient, tempPatient.getName(), consult.getTitle(), consult.getSymptoms(), consult.getImages(), agent);
        users.put(patient, 0);//+ " "+(tempPatient.getSex()==1?"(男 ":"(女 ") + IdCardUtil.getAgeForIdcard(tempPatient.getIdcard())+")"
        com.alibaba.fastjson.JSONObject obj = ImUtill.createTopics(patient + "_" + ct.getDoctor() + "_" +specialDoctorCode+"_"+ ct.getType(), consult.getId(), tempPatient.getName(), users, messages, ImUtil.SESSION_TYPE_KANGFU);
        //specialDoctorCode
        if (obj == null) {
            throw new RuntimeException("IM消息发送异常！");
        }
        if (obj.getInteger("status") == -1) {//im议题创建失败
            throw new RuntimeException(obj.getString("message"));
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try {
            SpecialDiseaseMessagesDO diseaseMessages = new SpecialDiseaseMessagesDO();
            diseaseMessages.setCode(patient);
            diseaseMessages.setName(tempPatient.getName());
            diseaseMessages.setAddress(tempPatient.getAddress());
            diseaseMessages.setCodeType("2");
            diseaseMessages.setCreateTime(sdf.format(new Date()));
            diseaseMessages.setResult("发起康复咨询");
            specialDiseaseMessagesDao.save(diseaseMessages);
        }catch (Exception e){
            logger.info("ConsultTeamService中 addRecoverConsult 方法保存出错");
            e.printStackTrace();
        }
        ct.setStartMsgId(obj.get("start_msg_id").toString());
        consultTeamDao.save(ct);
        consultDao.save(consult);

        JSONArray doctor = new JSONArray();
        for (String key : users.keySet()) {
            if (patient.equals(key)) {
                continue;
            }
            doctor.put(key);
            //记录咨询的医生详情误删
            ConsultTeamDoctorDo cd = new ConsultTeamDoctorDo();
            cd.setConsult(consult.getId());
            cd.setDel("1");
            cd.setCzrq(new Date());
            cd.setTo(key);
            consultTeamDoctorDao.save(cd);
        }

        // 保存医生咨询信息
        // 添加咨询转发记录
        // 添加医生咨询日志
        //String content = addLogs(ct);
        re.put("doctor", doctor);
        re.put("status", 1);
        return re;

    }

    /**
     * 查询患者是否还有未结束的三师咨询
     *
     * @param patient
     * @return
     */
    public boolean exist(String patient, Integer type,String doctor) throws Exception {
        String sql ="SELECT\n" +
                "\tcount( 1 ) as \"count\"\n" +
                "FROM\n" +
                "\tConsultTeam a,\n" +
                "\tConsultTeamDoctor b \n" +
                "WHERE\n" +
                "\ta.consult = b.consult \n" +
                "\tAND a.STATUS = 0 \n" +
                "\tAND a.del = '1' \n" ;
        if (StringUtils.isEmpty(patient)){
            sql+=" and a.patient = '"+patient+"'";
        }
        if (StringUtils.isEmpty(doctor)){
            sql+=" and b.to_doctor = '"+doctor+"'";
        }
        if (type!=null){
            sql+=" and a.type = "+type+"";
        }
        List<Map<String,Object>> list = hibenateUtils.createSQLQuery(sql);
        int count = list==null?0:list.size();
        return count > 0;
    }


    /**
     * 患者端
     * 添加三师咨询
     *
     * @param ct      三师咨询对象
     * @param patient 患者标识
     * @return
     * @throws Exception
     */
    public JSONObject addTeamConsult(ConsultTeamDo ct, String patient,String agent) throws Exception {
        JSONObject re = new JSONObject();
        if (exist(patient, ct.getType(),ct.getDoctor())) {//判断是否有未结束的咨询移到同步方法中
            re.put("status", -3);
            return re;
        }

        com.alibaba.fastjson.JSONObject users = new  com.alibaba.fastjson.JSONObject ();

        if (patient.equals(agent)) {
            agent = null;
        }
      /*  // 咨询三师
        if (ct.getType() == 1) {
            // 查询三师签约信息
            SignFamily sc = signFamilyDao.findBySanshiPatientYes(patient);
            if (sc == null) {
                // 不存在三师签约
                re.put("status", -2);
                return re;
            }
            ct.setTeam(sc.getTeamCode());
            // 设置健康管理师，三师咨询默认给健康管理师处理
            //查找病人所在的团队
            DoctorTeam doctorTeam = doctorTeamDao.findBySanshiParientCode(patient);
            //得到团队的健康管理师
            DoctorTeamMember doctorTeamMemberHealthy = doctorTeamDoctor.findDoctorSanshi2ByTeam(doctorTeam.getCode(), 3);
            DoctorTeamMember doctorTeamMember = doctorTeamDoctor.findDoctorSanshi2ByTeam(doctorTeam.getCode(), 2);
            if (doctorTeamMemberHealthy == null) {
                ct.setDoctor(doctorTeamMember.getMemberCode());
                users.put(doctorTeamMember.getMemberCode(), 0);
            } else {
                users.put(doctorTeamMemberHealthy.getMemberCode(), 0);
                if (doctorTeamMember != null) {
                    if (users.isNull(doctorTeamMember.getMemberCode())) {
                        users.put(doctorTeamMember.getMemberCode(), 1);
                    }
                }
                ct.setDoctor(doctorTeamMemberHealthy.getMemberCode());
            }
            // 设置家庭医生

            ct.setAdminTeamId(sc.getAdminTeamId());
        } else if (ct.getType() == 2) {
            // 咨询家庭医生
            SignFamily sf = signFamilyDao.findByjiatingPatient(patient);

            if (sf == null) {
                // 不存在家庭签约
                re.put("status", -1);
                return re;
            }
            ct.setTeam(sf.getTeamCode());
            // 设置健康管理师，家庭医生咨询默认给健康管理师处理
            //查找病人所在的团队
            //1.3.3.2 更改从签约记录判断团队成员,分配建管师
            if (org.apache.commons.lang3.StringUtils.isNotBlank(sf.getDoctorHealth())) {
                users.put(sf.getDoctorHealth(), 0);
                ct.setDoctor(sf.getDoctorHealth());
                if (org.apache.commons.lang3.StringUtils.isNotBlank(sf.getDoctor())) {
                    if (users.isNull(sf.getDoctor())) {
                        //全科需要判断是否开启健管师邀请后在接收消息
                        int isGetMessage = 1;
                        if (!messageService.getMessageNoticeSettingByMessageType(sf.getDoctor(), "1", MessageNoticeSetting.MessageTypeEnum.familyTopicSwitch.getValue())) {
                            isGetMessage = 0;
                        }
                        users.put(sf.getDoctor(), isGetMessage);
                    }
                }
            } else {
                ct.setDoctor(sf.getDoctor());
                users.put(sf.getDoctor(), 0);
            }

//            DoctorTeam doctorTeam = doctorTeamDao.findByParientCode(patient);
//            //得到团队的健康管理师
//            DoctorTeamMember doctorTeamMemberHealthy = doctorTeamDoctor.findDoctorJiating2ByTeam(doctorTeam.getCode(), 3);
//            DoctorTeamMember doctorTeamMember = doctorTeamDoctor.findDoctorJiating2ByTeam(doctorTeam.getCode(), 2);
//            if (doctorTeamMemberHealthy == null) {
//                ct.setDoctor(doctorTeamMember.getMemberCode());
//                users.put(doctorTeamMember.getMemberCode(), 0);
//            } else {
//                users.put(doctorTeamMemberHealthy.getMemberCode(), 0);
//                if (doctorTeamMember != null) {
//                    //全健同一个人导致会话成员状态问题处理
//                    if (users.isNull(doctorTeamMember.getMemberCode())) {
//                        users.put(doctorTeamMember.getMemberCode(), 1);
//                    }
//                }
//                ct.setDoctor(doctorTeamMemberHealthy.getMemberCode());
//            }

            // 设置家庭医生
            ct.setAdminTeamId(sf.getAdminTeamId());
        }*/
        // 设置患者信息
        ct.setPatient(patient);
        // 查询患者信息
        BasePatientDO tempPatient = patientDao.findById(patient).orElse(null);
        // 设置患者姓名
        ct.setName(tempPatient.getName());
        // 设置患者生日
        ct.setBirthday(tempPatient.getBirthday());
        //新增性别
        ct.setSex(tempPatient.getSex());
        // 设置患者头像
        ct.setPhoto(tempPatient.getPhoto());
        // 设置操作日期
        ct.setCzrq(new Date());
        ct.setDel("1");
        ct.setStatus(0);
        ct.setEvaluate(0);
        // 医生未读数量为1
        ct.setDoctorRead(1);
        // 添加咨询记录
        ConsultDo consult = addConsult(ct.getPatient(), null, ct.getSymptoms(), ct.getImages(), ct.getType());
        // 设置关联指导
        consult.setGuidance(ct.getGuidance());
        // 设置咨询标识
        ct.setConsult(consult.getId());

        //推送给IM去创建议题，取得成员消息
//        JSONObject messages = ImUtill.getCreateTopicMessage(patient, tempPatient.getName(), consult.getTitle(), "咨询问题："+consult.getSymptoms(), consult.getImages());
        com.alibaba.fastjson.JSONObject messages = ImUtill.getCreateTopicMessage(patient, tempPatient.getName(), consult.getTitle(), consult.getSymptoms(), consult.getImages(), agent);
        users.put(patient, 0);//+ " "+(tempPatient.getSex()==1?"(男 ":"(女 ") + IdCardUtil.getAgeForIdcard(tempPatient.getIdcard())+")"
        com.alibaba.fastjson.JSONObject  obj = ImUtill.createTopics(patient + "_" + ct.getTeam() + "_" + ct.getType(), consult.getId(), tempPatient.getName(), users, messages, ImUtill.SESSION_TYPE_MUC);
        if (obj == null) {
            throw new RuntimeException("IM消息发送异常！");
        }
        if (obj.getInteger("status") == -1) {//im议题创建失败
            throw new RuntimeException(obj.getString("message"));
        }
        ct.setStartMsgId(obj.get("start_msg_id").toString());
        consultTeamDao.save(ct);
        consultDao.save(consult);

        JSONArray doctor = new JSONArray();
        for (String key : users.keySet()) {
            if (patient.equals(key)) {
                continue;
            }
            doctor.put(key);
            //记录咨询的医生详情误删
            ConsultTeamDoctorDo cd = new ConsultTeamDoctorDo();
            cd.setConsult(consult.getId());
            cd.setDel("1");
            cd.setCzrq(new Date());
            cd.setTo(key);
            consultTeamDoctorDao.save(cd);
        }

        // 保存医生咨询信息
        // 添加咨询转发记录
        // 添加医生咨询日志
        //String content = addLogs(ct);
        re.put("doctor", doctor);
        re.put("status", 1);
        return re;

    }

    /**
     * 进入咨询
     *
     * @param consult
     * @param patient
     * @param agent
     * @return
     */
    public int intoTopic(String consult, String patient, String agent) throws Exception {
        ConsultTeamDo ct = consultTeamDao.findByConsult(consult);
        if (ct.getStatus() != 0) {
            return -1;
        }

        String content = "进入了咨询";
        BasePatientDO p = patientDao.findById(patient).orElse(null);
        String intoUserName = p.getName();
        if (patient.equals(agent)) {
            content = intoUserName + content;
        } /*else {
            PatientFamilyMember familyMember = familyMemberDao.findByPatientAndFamilyMember(patient, agent);
            Patient member = patientDao.findByCode(agent);
            content = member.getName() + "(" + relations.get(familyMember.getFamilyRelation()) + ")" + content;
        }*/

        ImUtill.sendIntoTopicIM(ct.getPatient(), ct.getPatient(), ct.getConsult(), content, agent, intoUserName);

        return 0;
    }


    /**
     * 查询咨询列表
     *
     * @param patientCode 患者标志
     * @param status      咨询状态（0未结束，1已结束，-1 已取消）
     * @param pageSize    页数
     * @return
     */
    public Page<ConsultTeamDo> findByPatient(String patientCode, int status, String id, int pageSize) {
        if (pageSize <= 0) {
            pageSize = 10;
        }
        // 排序
        Sort sort = Sort.by(Direction.DESC, "id");
        // 分页信息
        PageRequest pageRequest = PageRequest.of(0, pageSize, sort);

        Page<ConsultTeamDo> list = null;
        switch (status) {
            case 0:
                // 未结束
                if (!StringUtils.isEmpty(id)) {
                    list = consultTeamDao.findNotFinishedBypatient(patientCode, id, pageRequest);
                } else {
                    list = consultTeamDao.findNotFinishedBypatient(patientCode, pageRequest);
                }
                break;
            case 1:
                // 已结束
                if (!StringUtils.isEmpty(id)) {
                    list = consultTeamDao.findFinishedBypatient(patientCode, id, pageRequest);
                } else {
                    list = consultTeamDao.findFinishedBypatient(patientCode, pageRequest);
                }
                break;
            case -1:
                // 已取消
                if (!StringUtils.isEmpty(id)) {
                    list = consultTeamDao.findCancelBypatient(patientCode, id, pageRequest);
                } else {
                    list = consultTeamDao.findCancelBypatient(patientCode, pageRequest);
                }
                break;
        }
        return list;
    }

    /**
     * 取消三师咨询
     *
     * @param consult
     * @return
     */
    public int cancel(String consult) {
        return consultTeamDao.cancel(consult);
    }

    public int finishConsult(String consult, String endOperator, int endType) throws Exception {
        ConsultTeamDo consultTeam = consultTeamDao.findByConsult(consult);
        ConsultDo cons = consultDao.findById(consult).orElse(null);

        if (consultTeam.getStatus() == 1) {
            return -1;
        }

        //新增续方咨询结束判断 （续方未审核时不可由医生或居民关闭）
        if (consultTeam.getType() == 8) {
            WlyyPrescriptionDO prescription = prescriptionDao.findById(consultTeam.getRelationCode()).orElse(null);
            if (prescription.getStatus() == 0) {
                return -2;
            }
        }

        String endName = "";
        String endId = "";
        //结束咨询才发送推送给IM文字消息
        if (endType == 1) {
            BasePatientDO p = patientDao.findById(endOperator).orElse(null);
            endName = p.getName();
            endId = p.getId();
        } else {
            if (endOperator.equals("admin")) {
                endId = "system";
                endName = "咨询超时未回复，系统自动";
            } else {
                BaseDoctorDO d = doctorDao.findById(endOperator).orElse(null);
                endId = d.getId();
                endName = d.getName();
            }
        }

        com.alibaba.fastjson.JSONObject obj = ImUtill.endTopics(consultTeam.getPatient(), endId, endName, consultTeam.getConsult());
        if (obj == null) {
            throw new RuntimeException("IM消息结束异常！");
        }
        if (obj.getInteger("status") == -1) {
            throw new RuntimeException(String.valueOf(obj.get("message")));
        }

        //结束咨询才发送推送给IM文字消息
        if (endType == 1) {
            BasePatientDO p = patientDao.findById(endOperator).orElse(null);
            endName = p.getName();
            if (consultTeam.getType() == 2 || consultTeam.getType() == 8) {
                /*String openId = p.getOpenid();

                WechatTemplateConfig templateConfig = templateConfigDao.findByScene("template_doctor_survey", "zxpj");
                String first = templateConfig.getFirst();
                first = first.replace("key1", (endName == null ? "" : endName));
                String keyword1 = templateConfig.getKeyword1();

                JSONObject json = new JSONObject();
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                json.put("keyword1", keyword1);
                json.put("keyword2", sdf.format(new Date()));
                json.put("toUser", p.getCode());
                json.put("represented", p.getCode());//被代理人
                json.put("remark", "");
                json.put("consult", consult);
//                String first = endName + "，您好！您有1条";;
                if (consultTeam.getType() == 8) {
//                    first += "续方";
                    first = first.replace("key2", "续方");
                } else {
                    first = first.replace("key2", "");
                }
//                first += "咨询已结束，请及时对咨询医生进行评价。";
                json.put("first", first);
                if (org.apache.commons.lang3.StringUtils.isNotBlank(p.getOpenid())) {
                    pushMsgTask.putWxMsg(accessTokenUtils.getAccessToken(), 17, openId, p.getName(), json);
                } else {
                    //发送代理人
                    JSONArray jsonArray = weiXinOpenIdUtils.getAgentOpenId(p.getCode(), p.getOpenid());
                    if (jsonArray != null && jsonArray.length() > 0) {
                        for (int i = 0; i < jsonArray.length(); i++) {
                            JSONObject j = jsonArray.getJSONObject(i);
                            Patient member = (Patient) j.get("member");
                            JSONObject data = json;
                            data.remove("toUser");
                            data.put("toUser", member.getCode());
                            data.remove("first");
                            data.put("first", weiXinOpenIdUtils.getTitleMes(p, j.getInt("relation"), p.getName()) + first);
                            pushMsgTask.putWxMsg(accessTokenUtils.getAccessToken(), 17, member.getOpenid(), p.getName(), data);
                        }
                    }
                }*/
            }
        } else {
            /*if (consultTeam.getType() == 2 || consultTeam.getType() == 8) {
                BasePatientDO p = patientDao.findById(consultTeam.getPatient());
                String name = p.getName();
                String openId = p.getOpenid();

                WechatTemplateConfig templateConfig = templateConfigDao.findByScene("template_doctor_survey", "zxpj");

                if (endOperator.equals("admin")) {
                    templateConfig = templateConfigDao.findByScene("template_doctor_survey", "zxcspj");
                }
                String first = templateConfig.getFirst();
                first = first.replace("key1", (name == null ? "" : name));
                String keyword1 = templateConfig.getKeyword1();

                JSONObject json = new JSONObject();
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                json.put("keyword1", keyword1);
                json.put("keyword2", sdf.format(new Date()));
                json.put("toUser", p.getCode());
                json.put("represented", p.getCode());//被代理人
                json.put("remark", "");
                json.put("consult", consult);
//                String first = name + "，您好！您有1条";;
                if (consultTeam.getType() == 8) {
//                    first += "续方";
                    first = first.replace("key2", "续方");
                } else {
                    first = first.replace("key2", "");
                }
                *//*if (endOperator.equals("admin")) {
                    first += "咨询超时未回复，系统自动关闭，请及时对咨询医生进行评价。";
                }else {
                    first += "咨询已结束，请及时对咨询医生进行评价。";
                }*//*
                json.put("first", first);
                if (org.apache.commons.lang3.StringUtils.isNotBlank(p.getOpenid())) {
                    pushMsgTask.putWxMsg(accessTokenUtils.getAccessToken(), 17, openId, p.getName(), json);
                } else {
                    //发送代理人
                    JSONArray jsonArray = weiXinOpenIdUtils.getAgentOpenId(p.getCode(), p.getOpenid());
                    if (jsonArray != null && jsonArray.length() > 0) {
                        for (int i = 0; i < jsonArray.length(); i++) {
                            JSONObject j = jsonArray.getJSONObject(i);
                            Patient member = (Patient) j.get("member");
                            JSONObject data = json;
                            data.remove("toUser");
                            data.put("toUser", member.getCode());
                            data.remove("first");
                            data.put("first", weiXinOpenIdUtils.getTitleMes(p, j.getInt("relation"), p.getName()) + first);
                            pushMsgTask.putWxMsg(accessTokenUtils.getAccessToken(), 17, member.getOpenid(), p.getName(), data);
                        }
                    }
                }

            }*/
        }

        consultTeam.setEndMsgId(obj.getString("id"));
        cons.setEndTime(new Date());
        consultTeam.setEndTime(new Date());
        consultTeam.setStatus(1);
        consultDao.save(cons);
        consultTeamDao.save(consultTeam);
        return 1;
    }

}
