package com.yihu.rehabilitation.service.followUp;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.entity.followup.Followup;
import com.yihu.jw.util.entity.EntityUtils;
import com.yihu.rehabilitation.dao.followUp.FollowUpDao;
import com.yihu.rehabilitation.es.ElasticSearchHelper;
import com.yihu.rehabilitation.vo.entity.phthisis_followup.PhthisisFirstFollowupES;
import com.yihu.rehabilitation.vo.entity.phthisis_followup.PhthisisFollowupManagementES;
import com.yihu.rehabilitation.vo.entity.phthisis_followup.PhthisisFollowupPatientInfoES;
import com.yihu.rehabilitation.vo.entity.phthisis_followup.PhthisisNTimesFollowupES;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.transaction.Transactional;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 肺结核随访服务
 */
@Component
@Transactional(rollbackOn = Exception.class)
public class PhthisisFollowupService {

    private Logger logger = LoggerFactory.getLogger(PhthisisFollowupService.class);

    @Value("${es.index.phthisis_first_followup_index}")
    private String firstIndex;

    @Value("${es.type.phthisis_first_followup_type}")
    private String firstType;

    @Value("${es.index.phthisis_n_times_followup_index}")
    private String nTimesIndex;

    @Value("${es.type.phthisis_n_times_followup_type}")
    private String nTimesType;

    @Autowired
    private ElasticSearchHelper elasticSearchHelper;

    @Autowired
    private FollowUpService followUpService;
    @Autowired
    private FollowUpDao followUpDao;

    @Autowired
    ObjectMapper objectMapper;

    /**
     * 保存第一次肺结核随访
     *
     * @param jsonData
     */
    public boolean saveFirstFollowup(String jsonData,StringBuilder errMsg) throws IOException {

        try {
            JSONObject.parseObject(jsonData);
        } catch (Exception e) {
            errMsg.append("参数转换成JSON对象异常：").append(e.getMessage());
            logger.error(errMsg.toString());
            return false;
        }
        PhthisisFirstFollowupES firstFollowupES = null;
        try {
            firstFollowupES = EntityUtils.jsonToEntity(jsonData, PhthisisFirstFollowupES.class);
        } catch (Exception e) {
            errMsg.append("第一次肺结核随访json转换成ES实体类异常：").append(e.getMessage());
            logger.error(errMsg.toString());
            return false;
        }
        if (StringUtils.isEmpty(firstFollowupES.getFollowup_id())) {
            errMsg.append("随访id为空！");
            logger.error(errMsg.toString());
            return false;
        }
        Followup followupMysqlEntity = followUpService.findFollowupInfo(Integer.parseInt(firstFollowupES.getFollowup_id()));
        if(null == followupMysqlEntity || null == followupMysqlEntity.getId()){
            errMsg.append("该随访不存在，随访id-->").append(firstFollowupES.getFollowup_id());
            logger.error(errMsg.toString());
            return false;
        }
        //设置当前随访状态
        if(followupMysqlEntity.getStatus().equals("2")){
            followupMysqlEntity.setStatus("3");
            followUpDao.save(followupMysqlEntity);
        }
        // 设置居民基本信息
        if(null == firstFollowupES.getPatient()){
            firstFollowupES.setPatient(new PhthisisFollowupPatientInfoES());
        }
        firstFollowupES.getPatient().setName(followupMysqlEntity.getPatientName());
        firstFollowupES.getPatient().setCode(followupMysqlEntity.getPatientCode());

        //设置医生基本信息
        firstFollowupES.setSignature_doctor(followupMysqlEntity.getDoctorCode());
        firstFollowupES.setSignature_doctor_name(followupMysqlEntity.getDoctorName());

        // 当前日期为随访日期
        firstFollowupES.setCreate_time(new Date());

        if (!elasticSearchHelper.saveWithCustomId(firstIndex, firstType, objectMapper.writeValueAsString(firstFollowupES), "followup_id", errMsg)) {
            return false;
        }
        return true;
    }


    /**
     * 保存第n次肺结核随访
     *
     * @param jsonData
     */
    public boolean saveNTimesFollowup(String jsonData,StringBuilder errMsg) throws IOException {

        try {
            JSONObject.parseObject(jsonData);
        } catch (Exception e) {
            errMsg.append("参数转换成JSON对象异常：").append(e.getMessage());
            logger.error(errMsg.toString());
            return false;
        }
        PhthisisNTimesFollowupES nTimesFollowupES = null;
        try {
            nTimesFollowupES = EntityUtils.jsonToEntity(jsonData, PhthisisNTimesFollowupES.class);
        } catch (Exception e) {
            errMsg.append("第n次肺结核随访json转换成ES实体类异常：").append(e.getMessage());
            logger.error(errMsg.toString());
            return false;
        }
        if (StringUtils.isEmpty(nTimesFollowupES.getFollowup_id())) {
            errMsg.append("随访id为空！");
            logger.error(errMsg.toString());
            return false;
        }
        Followup followupMysqlEntity = followUpService.findFollowupInfo(Integer.parseInt(nTimesFollowupES.getFollowup_id()));

        if(null == followupMysqlEntity || null == followupMysqlEntity.getId()){
            errMsg.append("该随访不存在，随访id-->").append(nTimesFollowupES.getFollowup_id());
            logger.error(errMsg.toString());
            return false;
        }

        //设置当前随访状态
        if(followupMysqlEntity.getStatus().equals("2")){
            followupMysqlEntity.setStatus("3");
            followUpDao.save(followupMysqlEntity);
        }
        // 设置居民基本信息
        if(null == nTimesFollowupES.getPatient()){
            nTimesFollowupES.setPatient(new PhthisisFollowupPatientInfoES());
        }
        nTimesFollowupES.getPatient().setName(followupMysqlEntity.getPatientName());
        nTimesFollowupES.getPatient().setCode(followupMysqlEntity.getPatientCode());

        //设置管理情况的医生
        if(null == nTimesFollowupES.getManagement()){
            nTimesFollowupES.setManagement(new PhthisisFollowupManagementES());
        }
        nTimesFollowupES.getManagement().setSignature_doctor(followupMysqlEntity.getDoctorCode());
        nTimesFollowupES.getManagement().setSignature_doctor_name(followupMysqlEntity.getDoctorName());

        nTimesFollowupES.getData().forEach(one -> {
            one.setFollowup_doctor(followupMysqlEntity.getDoctorCode());
            one.setFollowup_doctor_name(followupMysqlEntity.getDoctorName());
            one.setCreate_time(new Date());
        });
        if (!elasticSearchHelper.saveWithCustomId(nTimesIndex, nTimesType, objectMapper.writeValueAsString(nTimesFollowupES), "followup_id",errMsg)) {
            return false;
        }
        return true;
    }

    /**
     * 根据随访id查询第一次随访记录详情
     *
     * @param followupId
     */
    public JSONObject queryFirstByFollowupId(String followupId) {
        List<JSONObject> result = new ArrayList<>();
        String sql = "select * from " + firstIndex + " where followup_id = '" + followupId + "'";
        try {
            result = elasticSearchHelper.execCurl(sql);
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (!CollectionUtils.isEmpty(result)) {
            return result.get(0);
        }
        return new JSONObject();
    }

    /**
     * 根据随访id查询第n次随访记录详情
     *
     * @param followupId
     */
    public List<JSONObject>  queryNTimesByFollowupId(String followupId) {
        List<JSONObject> result = new ArrayList<>();
        String sql = "select * from " + nTimesIndex + " where followup_id = '" + followupId + "'";
        try {
            result = elasticSearchHelper.execCurl(sql);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

}
