package com.yihu.rehabilitation.service.followUp;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.org.BaseOrgDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.followup.Followup;
import com.yihu.jw.entity.followup.FollowupMapping;
import com.yihu.jw.entity.hospital.mapping.DoctorMappingDO;
import com.yihu.jw.es.es.ElasticFactory;
import com.yihu.jw.es.es.ElastricSearchSave;
import com.yihu.jw.es.util.ElasticsearchUtil;
import com.yihu.jw.hospital.mapping.dao.DoctorMappingDao;
import com.yihu.jw.org.dao.BaseOrgDao;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.rehabilitation.dao.followUp.FollowUpDao;
import com.yihu.rehabilitation.dao.followUp.FollowUpMappingDao;
import com.yihu.rehabilitation.vo.entity.*;
import com.yihu.rehabilitation.vo.entity.followupDisease.*;
import io.searchbox.client.JestClient;
import io.searchbox.core.Search;
import io.searchbox.core.SearchResult;
import org.apache.commons.lang3.StringUtils;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.transaction.Transactional;
import java.beans.PropertyDescriptor;
import java.io.IOException;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Component
@Transactional(rollbackOn = Exception.class)
public class VisitDetailService  {
    @Value("${es.type.PostpartumVisitRecord}")
    private String esTypePostpartum;
    @Value("${es.index.PostpartumVisitRecord}")
    private String esIndexPostpartum;
    @Value("${es.type.FollowupSevereMentalDisorder}")
    private String esTypeSevereMentalDisorder;
    @Value("${es.index.FollowupSevereMentalDisorder}")
    private String esIndexSevereMentalDisorder;
    @Value("${es.type.OlderPeopleHealthCheckRecord}")
    private String esTypeOlderPeopleHealthCheck;
    @Value("${es.index.OlderPeopleHealthCheckRecord}")
    private String esIndexOlderPeopleHealthCheck;
    @Value("${es.type.NewbornFamilyVisitRecord}")
    private String esTypeNewbornFamilyVisit;
    @Value("${es.index.NewbornFamilyVisitRecord}")
    private String esIndexNewbornFamilyVisit;
    @Value("${es.index.phthisis_n_times_followup_index}")
    private String esIndexPhthisisNTimes;
    @Value("${es.type.phthisis_n_times_followup_type}")
    private String esTypePhthisisNTimes;

    @Value("${es.type.GynecologicalInflammationDaily}")
    private String esTypeGynecologicalInflammationDaily;
    @Value("${es.index.GynecologicalInflammationDaily}")
    private String esIndexGynecologicalInflammationDaily;
    @Value("${es.type.GynecologicalInflammationFollowupArchives}")
    private String esTypeGynecologicalInflammationFollowupArchives;
    @Value("${es.index.GynecologicalInflammationFollowupArchives}")
    private String esIndexGynecologicalInflammationFollowupArchives;
    @Value("${es.type.COPDFirstTimeQuestionnaire}")
    private String esTypeCOPDFirstTimeQuestionnaire;
    @Value("${es.index.COPDFirstTimeQuestionnaire}")
    private String esIndexCOPDFirstTimeQuestionnaire;
    @Value("${es.type.COPDDailyFollowup}")
    private String esTypeCOPDDailyFollowup;
    @Value("${es.index.COPDDailyFollowup}")
    private String esIndexCOPDDailyFollowup;
    @Value("${es.type.BreastFollowup}")
    private String esTypeBreastFollowup;//慢性乳腺疾病日常随访
    @Value("${es.index.BreastFollowup}")
    private String esIndexBreastFollowup;//慢性乳腺疾病日常随访
    @Value("${es.type.CerebralApoplexyFollowup}")
    private String esTypeCerebralApoplexyFollowup;
    @Value("${es.index.CerebralApoplexyFollowup}")
    private String esIndexCerebralApoplexyFollowup;
    @Value("${es.type.CoronaryFollowup}")
    private String esTypeCoronaryFollowup;
    @Value("${es.index.CoronaryFollowup}")
    private String esIndexCoronaryFollowup;
    @Value("${es.type.FattyLiverFollowUp}")
    private String esTypeFattyLiverFollowUp;
    @Value("${es.index.FattyLiverFollowUp}")
    private String esIndexFattyLiverFollowUp;
    @Value("${es.type.NephropathyFollowup}")
    private String esTypeNephropathyFollowup;
    @Value("${es.index.NephropathyFollowup}")
    private String esIndexNephropathyFollowup;
    @Value("${es.type.ProstateFollowup}")
    private String esTypeProstateFollowup;
    @Value("${es.index.ProstateFollowup}")
    private String esIndexProstateFollowup;
    @Autowired
    private ElasticFactory elasticFactory;
    @Autowired
    private ElastricSearchSave elastricSearchSave;
    @Autowired
    private ElasticsearchUtil elasticsearchUtil;
    @Autowired
    private FollowUpDao followupDao;
    @Autowired
    private FollowUpMappingDao followUpMappingDao;
    @Autowired
    private BaseDoctorDao doctorDao;
    @Autowired
    private PhthisisFollowupService phthisisFollowupService;
    @Autowired
    private DoctorMappingDao doctorMappingDao;
    @Autowired
    private BasePatientDao patientDao;
    @Autowired
    private BaseOrgDao hospitalMappingDao;

    /**
     * @Title: getNullPropertyNames
     * @Description: 获取值为空的属性名称
     * @createdBy:byrc
     */
    public static String[] getNullPropertyNames(Object source) {
        BeanWrapper src = new BeanWrapperImpl(source);
        PropertyDescriptor[] pds = src.getPropertyDescriptors();

        Set<String> emptyNames = new HashSet<String>();
        for (PropertyDescriptor pd : pds) {
            Object srcValue = src.getPropertyValue(pd.getName());
            if (srcValue == null)
                emptyNames.add(pd.getName());
        }
        String[] result = new String[emptyNames.size()];
        return emptyNames.toArray(result);
    }

    /**
     * 保存孕产妇访视详情记录
     * @param followupId
     * @param followupProject
     * @param postpartumVisitDetails
     * @param doctorCode
     * @throws Exception
     */
    public void savePostpartumVisitDetails(String followupId, String followupProject, String postpartumVisitDetails, String doctorCode) throws Exception {
        BaseDoctorDO d = null;
        if(StringUtils.isNotBlank(doctorCode)){
            d = doctorDao.findById(doctorCode);
        }else {
            throw new Exception("该医生不存在！");
        }
        JestClient jestClient = null;
        jestClient = elasticFactory.getJestClient();
        //先根据条件查找出来
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.query(
                new BoolQueryBuilder()
                        .must(QueryBuilders.matchQuery("followup_id", followupId))
//                        .must(QueryBuilders.matchQuery("followup_project", followupProject))
        );
        Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexPostpartum).addType(esTypePostpartum)
                .build();
        SearchResult result = jestClient.execute(search);
        PostpartumVisitRecordES postpartumVisitRecordES = result.getSourceAsObject(PostpartumVisitRecordES.class);

        Followup followup = followupDao.findOne(Integer.parseInt(followupId));
        //设置当前随访状态
        if(followup.getStatus().equals("2")){
            followup.setStatus("3");
            followupDao.save(followup);
        }

        //保存新的随访详情信息,如果是编辑的id不变
        String id = null;
        if(postpartumVisitRecordES != null && StringUtils.isNotBlank(postpartumVisitRecordES.getId())) {
            id = postpartumVisitRecordES.getId();
        }
        postpartumVisitRecordES = JSON.parseObject(postpartumVisitDetails,PostpartumVisitRecordES.class);
        postpartumVisitRecordES.setId(id);
        postpartumVisitRecordES.setFollowup_id(followupId);
        postpartumVisitRecordES.setFollowup_project(followupProject);
        postpartumVisitRecordES.setCreate_time(new Date());
        postpartumVisitRecordES.setCreate_user(doctorCode);
        postpartumVisitRecordES.setCreate_user_name(d.getName());
        postpartumVisitRecordES.setVisiting_doctor_code(doctorCode);
        elastricSearchSave.save(postpartumVisitRecordES, esIndexPostpartum, esTypePostpartum);
    }

    /**
     * 根据随访id获取孕产妇访视详情
     * @param followupId
     * @param followupProject
     * @return
     * @throws Exception
     */
    public PostpartumVisitRecordES getPostpartumVisitRecord(String followupId, String followupProject) throws Exception{
        JestClient jestClient = null;
        PostpartumVisitRecordES postpartumVisitRecordES = null;
        try {
            //根据随访ID、分类ID获取随访记录详情
            jestClient = elasticFactory.getJestClient();
            //先根据条件查找出来
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            searchSourceBuilder.query(
                    new BoolQueryBuilder()
                            .must(QueryBuilders.matchQuery("followup_id", followupId))
//                            .must(QueryBuilders.matchQuery("followup_project", followupProject))
            );

            Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexPostpartum).addType(esTypePostpartum)
                    .build();
            SearchResult result = jestClient.execute(search);
            postpartumVisitRecordES = result.getSourceAsObject(PostpartumVisitRecordES.class);
        } finally {
            if (jestClient != null) {
                jestClient.shutdownClient();
            }
        }
        return postpartumVisitRecordES;
    }

    /**
     * 保存严重精神障碍患者随访记录
     * @param followupId
     * @param followupProject
     * @param severeMentalDisorderData
     * @param doctorCode
     * @throws Exception
     */
    public void saveFollowupSevereMentalDisorder(String followupId, String followupProject, String severeMentalDisorderData, String doctorCode) throws Exception{
        BaseDoctorDO d = null;
        if(StringUtils.isNotBlank(doctorCode)){
            d = doctorDao.findById(doctorCode);
        }else {
            throw new Exception("该医生不存在！");
        }
        JestClient jestClient = elasticFactory.getJestClient();
        //先根据条件查找出来
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.query(
                new BoolQueryBuilder()
                        .must(QueryBuilders.matchQuery("followup_id", followupId))
//                        .must(QueryBuilders.matchQuery("followup_project", followupProject))
        );
        Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexSevereMentalDisorder).addType(esTypeSevereMentalDisorder)
                .build();
        SearchResult result = jestClient.execute(search);
        FollowupSevereMentalDisorderES severeMentalDisorderES = result.getSourceAsObject(FollowupSevereMentalDisorderES.class);
        Followup followup = followupDao.findOne(Integer.parseInt(followupId));
        //设置当前随访状态
        if(followup.getStatus().equals("2")){
            followup.setStatus("3");
            followupDao.save(followup);
        }
        //保存新的随访详情信息
        String id = null;
        if(severeMentalDisorderES != null && StringUtils.isNotBlank(severeMentalDisorderES.getId())) {
            id = severeMentalDisorderES.getId();
        }
        severeMentalDisorderES = JSON.parseObject(severeMentalDisorderData,FollowupSevereMentalDisorderES.class);
        severeMentalDisorderES.setId(id);
        severeMentalDisorderES.setFollowup_id(followupId);
        severeMentalDisorderES.setFollowup_project(followupProject);
        severeMentalDisorderES.setCreate_time(new Date());
        severeMentalDisorderES.setCreate_user(doctorCode);
        severeMentalDisorderES.setCreate_user_name(d.getName());
        elastricSearchSave.save(severeMentalDisorderES, esIndexSevereMentalDisorder, esTypeSevereMentalDisorder);
    }

    /**
     * 获取严重精神障碍患者随访记录
     * @param followupId
     * @param followupProject
     * @return
     * @throws IOException
     */
    public FollowupSevereMentalDisorderES getFollowupSevereMentalDisorder(String followupId, String followupProject) throws IOException {
        JestClient jestClient = null;
        FollowupSevereMentalDisorderES severeMentalDisorderES = null;
        try {
            //根据随访ID获取随访记录详情
            jestClient = elasticFactory.getJestClient();
            //先根据条件查找出来
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            searchSourceBuilder.query(
                    new BoolQueryBuilder()
                            .must(QueryBuilders.matchQuery("followup_id", followupId))
//                            .must(QueryBuilders.matchQuery("followup_project", followupProject))
            );
            Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexSevereMentalDisorder).addType(esTypeSevereMentalDisorder).build();
            SearchResult result = jestClient.execute(search);
            severeMentalDisorderES = result.getSourceAsObject(FollowupSevereMentalDisorderES.class);
        } finally {
            if (jestClient != null) {
                jestClient.shutdownClient();
            }
        }
        return severeMentalDisorderES;
    }

    /**
     * 保存老人健康体检随访记录
     * @param followupId
     * @param followupProject
     * @param olderPeopleHealthCheckData
     * @param doctorCode
     * @throws Exception
     */
    public void saveOlderPeopleHealthCheckRecord(String followupId, String followupProject, String olderPeopleHealthCheckData, String doctorCode,Integer bdStatus,Integer pgStatus) throws Exception{
        BaseDoctorDO d = null;
        if(StringUtils.isNotBlank(doctorCode)){
            d = doctorDao.findById(doctorCode);
        }else {
            throw new Exception("该医生不存在！");
        }
        JestClient jestClient = elasticFactory.getJestClient();
        //先根据条件查找出来
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.query(
                new BoolQueryBuilder()
                        .must(QueryBuilders.matchQuery("followup_id", followupId))
//                        .must(QueryBuilders.matchQuery("followup_project", followupProject))
        );
        Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexOlderPeopleHealthCheck).addType(esTypeOlderPeopleHealthCheck)
                .build();
        SearchResult result = jestClient.execute(search);
        OlderPeopleHealthCheckRecordES olderPeopleHealthCheckRecord = result.getSourceAsObject(OlderPeopleHealthCheckRecordES.class);
        Followup followup = followupDao.findOne(Integer.parseInt(followupId));
        if (followup.getFollowupClass().equals("8")){
            /*if (bdStatus != null && bdStatus == 1){
                followup.setBdStatus(1);
            }else {
                followup.setPgStatus(1);
            }*/
        }
        followupDao.save(followup);
        //设置当前随访状态
        if(followup.getStatus().equals("2")){
            followup.setStatus("3");
            followupDao.save(followup);
        }
        //保存新的随访详情信息
        String id = null;
        if(olderPeopleHealthCheckRecord != null && StringUtils.isNotBlank(olderPeopleHealthCheckRecord.getId())) {
            id = olderPeopleHealthCheckRecord.getId();
        }
        olderPeopleHealthCheckRecord = JSON.parseObject(olderPeopleHealthCheckData,OlderPeopleHealthCheckRecordES.class);
        olderPeopleHealthCheckRecord.setId(id);
        olderPeopleHealthCheckRecord.setFollowup_id(followupId);
        olderPeopleHealthCheckRecord.setFollowup_project(followupProject);
        olderPeopleHealthCheckRecord.setCreate_time(new Date());
        olderPeopleHealthCheckRecord.setCreate_user(doctorCode);
        olderPeopleHealthCheckRecord.setCreate_user_name(d.getName());
        elastricSearchSave.save(olderPeopleHealthCheckRecord, esIndexOlderPeopleHealthCheck, esTypeOlderPeopleHealthCheck);
    }

    /**
     * 获取老人健康体检随访记录
     * @param followupId
     * @param followupProject
     * @return
     * @throws Exception
     */
    public OlderPeopleHealthCheckRecordES getOlderPeopleHealthCheckRecord(String followupId, String followupProject) throws Exception{
        JestClient jestClient = null;
        OlderPeopleHealthCheckRecordES olderPeopleHealthCheckRecord = null;
        try {
            //根据随访ID获取随访记录详情
            jestClient = elasticFactory.getJestClient();
            //先根据条件查找出来
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            searchSourceBuilder.query(
                    new BoolQueryBuilder()
                            .must(QueryBuilders.matchQuery("followup_id", followupId))
//                            .must(QueryBuilders.matchQuery("followup_project", followupProject))
            );
            Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexOlderPeopleHealthCheck).addType(esTypeOlderPeopleHealthCheck).build();
            SearchResult result = jestClient.execute(search);
            olderPeopleHealthCheckRecord = result.getSourceAsObject(OlderPeopleHealthCheckRecordES.class);
        } finally {
            if (jestClient != null) {
                jestClient.shutdownClient();
            }
        }
        return olderPeopleHealthCheckRecord;
    }

    /**
     * 保存新生儿家庭访视记录
     * @param followupId
     * @param followupProject
     * @param newbornFamilyVisitRecordData
     * @param doctorCode
     * @throws Exception
     */
    public void saveNewbornFamilyVisitRecord(String followupId, String followupProject, String newbornFamilyVisitRecordData, String doctorCode) throws Exception{
        BaseDoctorDO d = null;
        if(StringUtils.isNotBlank(doctorCode)){
            d = doctorDao.findById(doctorCode);
        }else {
            throw new Exception("该医生不存在！");
        }
        JestClient jestClient = elasticFactory.getJestClient();
        //先根据条件查找出来
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.query(
                new BoolQueryBuilder()
                        .must(QueryBuilders.matchQuery("followup_id", followupId))
//                        .must(QueryBuilders.matchQuery("followup_project", followupProject))
        );
        Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexNewbornFamilyVisit).addType(esTypeNewbornFamilyVisit)
                .build();
        SearchResult result = jestClient.execute(search);
        NewbornFamilyVisitRecordES newbornFamilyVisitRecordES = result.getSourceAsObject(NewbornFamilyVisitRecordES.class);
        Followup followup = followupDao.findOne(Integer.parseInt(followupId));
        //设置当前随访状态
        if(followup.getStatus().equals("2")){
            followup.setStatus("3");
            followupDao.save(followup);
        }
        //保存新的随访详情信息
        String id = null;
        if(newbornFamilyVisitRecordES != null && StringUtils.isNotBlank(newbornFamilyVisitRecordES.getId())) {
            id = newbornFamilyVisitRecordES.getId();
        }
        newbornFamilyVisitRecordES = JSON.parseObject(newbornFamilyVisitRecordData,NewbornFamilyVisitRecordES.class);
        newbornFamilyVisitRecordES.setId(id);
        newbornFamilyVisitRecordES.setFollowup_id(followupId);
        newbornFamilyVisitRecordES.setFollowup_project(followupProject);
        newbornFamilyVisitRecordES.setCreate_time(new Date());
        newbornFamilyVisitRecordES.setCreate_user(doctorCode);
        newbornFamilyVisitRecordES.setCreate_user_name(d.getName());
        elastricSearchSave.save(newbornFamilyVisitRecordES, esIndexNewbornFamilyVisit, esTypeNewbornFamilyVisit);
    }

    /**
     * 获取新生儿家庭访视记录
     * @param followupId
     * @param followupProject
     * @return
     * @throws Exception
     */
    public NewbornFamilyVisitRecordES getNewbornFamilyVisitRecord(String followupId, String followupProject) throws Exception{
        JestClient jestClient = null;
        NewbornFamilyVisitRecordES newbornFamilyVisitRecordES = null;
        try {
            //根据随访ID获取随访记录详情
            jestClient = elasticFactory.getJestClient();
            //先根据条件查找出来
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            searchSourceBuilder.query(
                    new BoolQueryBuilder()
                            .must(QueryBuilders.matchQuery("followup_id", followupId))
//                            .must(QueryBuilders.matchQuery("followup_project", followupProject))
            );
            Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexNewbornFamilyVisit).addType(esTypeNewbornFamilyVisit).build();
            SearchResult result = jestClient.execute(search);
            newbornFamilyVisitRecordES = result.getSourceAsObject(NewbornFamilyVisitRecordES.class);
        } finally {
            if (jestClient != null) {
                jestClient.shutdownClient();
            }
        }
        return newbornFamilyVisitRecordES;
    }

    /**
     * 根据居民code和访视类别获取上次随访记录
     * @param patientCode
     * @param followupClass 5.产后 6.新生儿 7.严重精神病 8.80岁以上老人 9.肺结核
     * @return
     */
    public JSONObject getLastVisitRecord(String patientCode, String followupClass) throws Exception {
        JSONObject res = new JSONObject();
        List<Followup> followupList = followupDao.findAllByPatientCodeAndClass(patientCode, followupClass);
        if (followupList.size() > 0) {
            String sql = "";
            res.put("followup", followupList.get(0));
            //产后记录
            if(followupClass.equals("5")) {
                PostpartumVisitRecordES es = this.getPostpartumVisitRecord(followupList.get(0).getId() + "", null);
//                sql = "select * from " + esIndexPostpartum + " where followup_id ='" + followupList.get(0).getId() + "' order by create_time desc ";
//                List<PostpartumVisitRecordES> returnList = elasticsearchUtil.excute(sql, PostpartumVisitRecordES.class, esTypePostpartum, esIndexPostpartum);
                res.put("detail", es);
            }else  if(followupClass.equals("6")) {
                NewbornFamilyVisitRecordES es = this.getNewbornFamilyVisitRecord(followupList.get(0).getId() + "", null);
//                sql = "select * from " + esIndexNewbornFamilyVisit + " where followup_id ='" + followupList.get(0).getId() + "' order by create_time desc ";
//                List<NewbornFamilyVisitRecordES> returnList = elasticsearchUtil.excute(sql, NewbornFamilyVisitRecordES.class, esTypeNewbornFamilyVisit, esIndexNewbornFamilyVisit);
                res.put("detail", es);
            }else  if(followupClass.equals("7")) {
                FollowupSevereMentalDisorderES es = this.getFollowupSevereMentalDisorder(followupList.get(0).getId() + "", null);
                res.put("detail", es);
//                sql = "select * from " + esIndexSevereMentalDisorder + " where followup_id ='" + followupList.get(0).getId() + "'order by create_time desc ";
//                List<FollowupSevereMentalDisorderES> returnList = elasticsearchUtil.excute(sql, FollowupSevereMentalDisorderES.class, esTypeSevereMentalDisorder, esIndexSevereMentalDisorder);
            }else  if(followupClass.equals("8")) {
                OlderPeopleHealthCheckRecordES es = this.getOlderPeopleHealthCheckRecord(followupList.get(0).getId() + "", null);
                res.put("detail", es);
//                sql = "select * from " + esIndexOlderPeopleHealthCheck + " where followup_id ='" + followupList.get(0).getId() + "' order by create_time desc ";
//                List<OlderPeopleHealthCheckRecordES> returnList = elasticsearchUtil.excute(sql, OlderPeopleHealthCheckRecordES.class, esTypeOlderPeopleHealthCheck, esIndexOlderPeopleHealthCheck);
            }else  if(followupClass.equals("9") && followupList.size() >= 2) {  //非第一次肺结核访视
                List<JSONObject> returnList = phthisisFollowupService.queryNTimesByFollowupId(followupList.get(0).getId() + "");
//                sql = "select * from " + esIndexPhthisisNTimes + " where followup_id ='" + followupList.get(0).getId() + "' ";
//                List<PhthisisNTimesFollowupDataES> returnList = elasticsearchUtil.excute(sql, PhthisisNTimesFollowupDataES.class, esTypePhthisisNTimes, esIndexPhthisisNTimes);
                res.put("detail", returnList);
            }
        }
        return res;
    }

    /**
     * 保存慢性妇科炎症日常随访
     * @param followupId
     * @param jsonData
     * @param doctorCode
     * @return
     * @throws Exception
     */
    public GynecologicalInflammationDailyFollowupES saveGynecologicalInflammationDaily(String followupId, String jsonData, String doctorCode) throws Exception {
        JSONObject jsonObject = JSONObject.parseObject(jsonData);
        Followup followup = followupDao.findOne(Integer.parseInt(followupId));
        Followup followupTemp = JSON.parseObject(jsonObject.getString("followup"), Followup.class);
        BeanUtils.copyProperties(followupTemp, followup, getNullPropertyNames(followupTemp));
        //设置当前随访状态
        if(followup.getStatus().equals("2")){
            followup.setStatus("3");
            followupDao.save(followup);
        }
        //判断是否已同步给智业，同步完再更新表单就要修改状态重新同步
        FollowupMapping followupMapping = followUpMappingDao.findByFollowupId(Integer.parseInt(followupId));
        if(followupMapping != null){
            followUpMappingDao.updateNeedUploadByFollowupId(1, Integer.parseInt(followupId));
        }
        BaseDoctorDO d = null;
        DoctorMappingDO doctorMapping = null;
        if(StringUtils.isNotBlank(doctorCode)){
            d = doctorDao.findById(doctorCode);
            doctorMapping = doctorMappingDao.findByDoctor(doctorCode);
        }else {
            throw new Exception("该医生不存在！");
        }
        JestClient jestClient = elasticFactory.getJestClient();
        //先根据条件查找出来
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.query(
                new BoolQueryBuilder()
                        .must(QueryBuilders.matchQuery("followup_id", followupId))
//                        .must(QueryBuilders.matchQuery("followup_project", followupProject))
        );
        Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexGynecologicalInflammationDaily).addType(esTypeGynecologicalInflammationDaily)
                .build();
        SearchResult result = jestClient.execute(search);
        GynecologicalInflammationDailyFollowupES dailyFollowupES = result.getSourceAsObject(GynecologicalInflammationDailyFollowupES.class);
        GynecologicalInflammationDailyFollowupES dailyFollowupNew = JSON.parseObject(jsonObject.getString("content"), GynecologicalInflammationDailyFollowupES.class);
        if(dailyFollowupES != null) {
            BeanUtils.copyProperties(dailyFollowupNew, dailyFollowupES, getNullPropertyNames(dailyFollowupNew));
        }else {
            dailyFollowupES = dailyFollowupNew;
        }
        //保存新的随访详情信息
        if(StringUtils.isBlank(dailyFollowupES.getId())) {
            dailyFollowupES.setFollowup_id(followupId);
            dailyFollowupES.setCreateTime(new Date());
            dailyFollowupES.setCreateOperator(doctorMapping.getMappingCode());
            dailyFollowupES.setCreateOperatorName(d.getName());
            dailyFollowupES.setValidFlag("1");
        }
        dailyFollowupES.setModifyTime(new Date());
        dailyFollowupES.setModifyOperator(doctorMapping.getMappingCode());
        dailyFollowupES.setModifyOperatorName(d.getName());
        elastricSearchSave.save(dailyFollowupES, esIndexGynecologicalInflammationDaily, esTypeGynecologicalInflammationDaily);
        return dailyFollowupES;
    }

    /**
     * 获取慢性妇科炎症日常随访
     * @param followupId
     * @return
     * @throws Exception
     */
    public JSONObject getGynecologicalInflammationDaily(String followupId) throws Exception {
        JSONObject jsonObject = new JSONObject();
        Followup followup = followupDao.findOne(Integer.parseInt(followupId));
        jsonObject.put("followup", followup);
        JestClient jestClient = null;
        GynecologicalInflammationDailyFollowupES dailyFollowupES = null;
        try {
            //根据随访ID获取随访记录详情
            jestClient = elasticFactory.getJestClient();
            //先根据条件查找出来
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            searchSourceBuilder.query(
                    new BoolQueryBuilder()
                            .must(QueryBuilders.matchQuery("followup_id", followupId))
            );
            Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexGynecologicalInflammationDaily).addType(esTypeGynecologicalInflammationDaily).build();
            SearchResult result = jestClient.execute(search);
            dailyFollowupES = result.getSourceAsObject(GynecologicalInflammationDailyFollowupES.class);
            jsonObject.put("content", dailyFollowupES);
        } finally {
            if (jestClient != null) {
                jestClient.shutdownClient();
            }
        }
        return jsonObject;
    }


    /**
     * 保存妇科炎症随访档案
     * @param followupId
     * @param jsonData
     * @param doctorCode
     * @return
     * @throws Exception
     */
    public GynecologicalInflammationFollowupArchivesES saveGynecologicalInflammationFollowupArchives(String followupId, String jsonData, String doctorCode) throws Exception {
        JSONObject jsonObject = JSONObject.parseObject(jsonData);
        Followup followup = followupDao.findOne(Integer.parseInt(followupId));
        Followup followupTemp = JSON.parseObject(jsonObject.getString("followup"), Followup.class);
        BeanUtils.copyProperties(followupTemp, followup, getNullPropertyNames(followupTemp));
        //设置当前随访状态
        if(followup.getStatus().equals("2")){
            followup.setStatus("3");
            followupDao.save(followup);
        }
        //判断是否已同步给智业，同步完再更新表单就要修改状态重新同步
        FollowupMapping followupMapping = followUpMappingDao.findByFollowupId(Integer.parseInt(followupId));
        if(followupMapping != null){
            followUpMappingDao.updateNeedUploadByFollowupId(1, Integer.parseInt(followupId));
        }
        BaseDoctorDO d = null;
        DoctorMappingDO doctorMapping = null;
        if(StringUtils.isNotBlank(doctorCode)){
            d = doctorDao.findById(doctorCode);
            doctorMapping = doctorMappingDao.findByDoctor(doctorCode);
        }else {
            throw new Exception("该医生不存在！");
        }
        JestClient jestClient = elasticFactory.getJestClient();
        //先根据条件查找出来
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.query(
                new BoolQueryBuilder()
                        .must(QueryBuilders.matchQuery("followup_id", followupId))
//                        .must(QueryBuilders.matchQuery("followup_project", followupProject))
        );
        Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexGynecologicalInflammationFollowupArchives).addType(esTypeGynecologicalInflammationFollowupArchives)
                .build();
        SearchResult result = jestClient.execute(search);
        GynecologicalInflammationFollowupArchivesES followupArchives = result.getSourceAsObject(GynecologicalInflammationFollowupArchivesES.class);
        GynecologicalInflammationFollowupArchivesES followupArchivesNew = JSON.parseObject(jsonObject.getString("content"), GynecologicalInflammationFollowupArchivesES.class);
        if(followupArchives != null) {
            BeanUtils.copyProperties(followupArchivesNew, followupArchives, getNullPropertyNames(followupArchivesNew));
        }else {
            followupArchives = followupArchivesNew;
        }
        //保存新的随访详情信息
        if(StringUtils.isBlank(followupArchives.getId())) {
            followupArchives.setFollowup_id(followupId);
            followupArchives.setCreateTime(new Date());
            followupArchives.setCreateOperator(doctorMapping.getMappingCode());
            followupArchives.setCreateOperatorName(d.getName());
            followupArchives.setValidFlag("1");
        }
        followupArchives.setModifyTime(new Date());
        followupArchives.setModifyOperator(doctorMapping.getMappingCode());
        followupArchives.setModifyOperatorName(d.getName());
        elastricSearchSave.save(followupArchives, esIndexGynecologicalInflammationFollowupArchives, esTypeGynecologicalInflammationFollowupArchives);
        return followupArchives;
    }


    /**
     * 获取各类病种单日常随访
     * type:
     * BreastFollowup //4.慢性乳腺疾病日常随访
     * CerebralApoplexyFollowup //7.脑卒中日常随访
     * CoronaryFollowup//1.冠心病日常随访
     * FattyLiverFollowUp//8.脂肪肝日常随访
     * NephropathyFollowup//5.慢性肾病日常随访
     * ProstateFollowup//3.慢性前列腺疾病日常随访
     * @param followupId
     * @return
     * @throws Exception
     */
    public JSONObject selectFollowupDiseaseById(String followupId,String type) throws Exception {
        JSONObject jsonObject = new JSONObject();
        Followup followup = followupDao.findOne(Integer.parseInt(followupId));
        jsonObject.put("followup", followup);
        JestClient jestClient = null;
        BreastFollowupES breastFollowupES = null;
        CerebralApoplexyFollowupES cerebralApoplexyFollowupES = null;
        CoronaryFollowupES coronaryFollowupES = null;
        FattyLiverFollowUpES fattyLiverFollowUpES = null;
        NephropathyFollowupES nephropathyFollowupES = null;
        ProstateFollowupES prostateFollowupES = null;
        try {
            //根据随访ID获取随访记录详情
            jestClient = elasticFactory.getJestClient();
            //先根据条件查找出来
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            searchSourceBuilder.query(
                    new BoolQueryBuilder()
                            .must(QueryBuilders.matchQuery("followup_id", followupId))
            );
            if (type.equalsIgnoreCase("BreastFollowup")){
                Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexBreastFollowup).addType(esTypeBreastFollowup).build();
                SearchResult result = jestClient.execute(search);
                breastFollowupES = result.getSourceAsObject(BreastFollowupES.class);
                jsonObject.put("content", breastFollowupES);
            }else if (type.equalsIgnoreCase("CerebralApoplexyFollowup")){
                Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexCerebralApoplexyFollowup).addType(esTypeCerebralApoplexyFollowup).build();
                SearchResult result = jestClient.execute(search);
                cerebralApoplexyFollowupES = result.getSourceAsObject(CerebralApoplexyFollowupES.class);
                jsonObject.put("content", cerebralApoplexyFollowupES);
            }else if (type.equalsIgnoreCase("CoronaryFollowup")){
                Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexCoronaryFollowup).addType(esTypeCoronaryFollowup).build();
                SearchResult result = jestClient.execute(search);
                coronaryFollowupES = result.getSourceAsObject(CoronaryFollowupES.class);
                jsonObject.put("content", coronaryFollowupES);
            }else if (type.equalsIgnoreCase("FattyLiverFollowUp")){
                Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexFattyLiverFollowUp).addType(esTypeFattyLiverFollowUp).build();
                SearchResult result = jestClient.execute(search);
                fattyLiverFollowUpES = result.getSourceAsObject(FattyLiverFollowUpES.class);
                jsonObject.put("content", fattyLiverFollowUpES);
            }else if (type.equalsIgnoreCase("NephropathyFollowup")){
                Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexNephropathyFollowup).addType(esTypeNephropathyFollowup).build();
                SearchResult result = jestClient.execute(search);
                nephropathyFollowupES = result.getSourceAsObject(NephropathyFollowupES.class);
                jsonObject.put("content", nephropathyFollowupES);
            }else if (type.equalsIgnoreCase("ProstateFollowup")){
                Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexProstateFollowup).addType(esTypeProstateFollowup).build();
                SearchResult result = jestClient.execute(search);
                prostateFollowupES = result.getSourceAsObject(ProstateFollowupES.class);
                jsonObject.put("content", prostateFollowupES);
            }


        } finally {
            if (jestClient != null) {
                jestClient.shutdownClient();
            }
        }
        return jsonObject;
    }

    /**
     *
     * @param followupId
     * @param jsonData
     * @param doctorCode
     * @param type
     * @return
     * @throws Exception
     * BreastFollowup: breast_followup
    CerebralApoplexyFollowup: cerebral_apoplexy_followup
    CoronaryFollowup: coronary_followup
    FattyLiverFollowUp: fatty_liverFollowUp
    NephropathyFollowup: nephropathy_followup
    ProstateFollowup: prostate_followup
     */
    public void saveFollowupDiseaseArchives(String followupId, String jsonData, String doctorCode,String type) throws Exception {
        JSONObject jsonObject = JSONObject.parseObject(jsonData);
        Followup followup = followupDao.findOne(Integer.parseInt(followupId));
        Followup followupTemp = JSON.parseObject(jsonObject.getString("followup"), Followup.class);
        BeanUtils.copyProperties(followupTemp, followup, getNullPropertyNames(followupTemp));
        //设置当前随访状态
        if(followup.getStatus().equals("2")){
            followup.setStatus("3");
            followupDao.save(followup);
        }
        //判断是否已同步给智业，同步完再更新表单就要修改状态重新同步
        FollowupMapping followupMapping = followUpMappingDao.findByFollowupId(Integer.parseInt(followupId));
        if(followupMapping != null){
            followUpMappingDao.updateNeedUploadByFollowupId(1, Integer.parseInt(followupId));
        }
        BaseDoctorDO d = null;
        DoctorMappingDO doctorMapping = null;
        if(StringUtils.isNotBlank(doctorCode)){
            d = doctorDao.findById(doctorCode);
            doctorMapping = doctorMappingDao.findByDoctor(doctorCode);
        }else {
            throw new Exception("该医生不存在！");
        }
        JestClient jestClient = elasticFactory.getJestClient();
        //先根据条件查找出来
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.query(
                new BoolQueryBuilder()
                        .must(QueryBuilders.matchQuery("followup_id", followupId))
//                        .must(QueryBuilders.matchQuery("followup_project", followupProject))
        );
        BasePatientDO patient1 = patientDao.findById(followup.getPatientCode());
        String idcard = "";
        if (patient1!=null){
            idcard = patient1.getIdcard();
        }
        //获取随访机构id
        BaseOrgDO hospitalMapping = hospitalMappingDao.findByCode(followup.getOrgCode());
        followup.setJworgCode(hospitalMapping.getCode());
        String orgId= hospitalMapping.getCode();
        String followOrgId=hospitalMapping.getCode();

        String followDoctor = followup.getJwdoctorCode();//随访医生
        if (StringUtils.isBlank(followDoctor)) {
            followDoctor = doctorMappingDao.findByDoctor(followup.getDoctorCode()).getMappingCode();
            followup.setJwdoctorCode(followDoctor);
        }
        String followTime= DateUtil.dateToStrLong(followup.getFollowupDate());//随访时间
        //判断随访方式【1.门诊 2.电话 3.入户】,待确定
        String followupType = followup.getFollowupType();
        String followType = "";
        if(followupType.equals("1") || followupType.equals("22")) {
            followType="1";
        }else if(followupType.equals("3")) {
            followType="2";
        }
        BaseDoctorDO doctor = doctorDao.findById(followup.getDoctorCode());
        String followDoctorType=doctor.getLevel()+"";
        String nextFollowTime=DateUtil.dateToStrLong(followup.getFollowupNextDate());//下次随访时间
        String assessSource="4";//评估来源【1.前台三合一 2. 前台独立 3.后台自动 4.i健康】
        //判断是新增还是编辑，如果是编辑传回随访ID,新增则不传
        if (type.equalsIgnoreCase("BreastFollowup")){
            //4.慢性乳腺疾病日常随访
            Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexBreastFollowup).addType(esTypeBreastFollowup)
                    .build();
            SearchResult result = jestClient.execute(search);
            BreastFollowupES breastFollowup = result.getSourceAsObject(BreastFollowupES.class);
            BreastFollowupES breastFollowupNew = JSON.parseObject(jsonObject.getString("content"), BreastFollowupES.class);
            if(breastFollowup != null) {
                BeanUtils.copyProperties(breastFollowupNew, breastFollowup, getNullPropertyNames(breastFollowupNew));
            }else {
                breastFollowup = breastFollowupNew;
            }
            //保存新的随访详情信息
            if(StringUtils.isBlank(breastFollowup.getId())) {
                breastFollowup.setFollowup_id(followupId);
                breastFollowup.setCreateTime(DateUtil.getStringDate());
                breastFollowup.setCreateOperator(doctorMapping.getMappingCode());
                breastFollowup.setCreateOperatorName(d.getName());
                breastFollowup.setValidFlag("1");
                breastFollowup.setIdentityCardNo(idcard);
                breastFollowup.setOrgId(orgId);
                breastFollowup.setFollowOrgId(followOrgId);
                breastFollowup.setFollowDoctor(followDoctor);
                breastFollowup.setFollowTime(followTime);
                breastFollowup.setFollowType(followType);
                breastFollowup.setFollowDoctorType(followDoctorType);
                breastFollowup.setAssessSource(assessSource);
                breastFollowup.setNextFollowTime(nextFollowTime);
            }
            breastFollowup.setModifyTime(DateUtil.getStringDate());
            breastFollowup.setModifyOperator(doctorMapping.getMappingCode());
            breastFollowup.setModifyOperatorName(d.getName());
            elastricSearchSave.save(breastFollowup, esIndexBreastFollowup, esTypeBreastFollowup);
        }else if (type.equalsIgnoreCase("CoronaryFollowup")){
            //1.冠心病日常随访
            Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexCoronaryFollowup).addType(esTypeCoronaryFollowup)
                    .build();
            SearchResult result = jestClient.execute(search);
            CoronaryFollowupES coronaryFollowup = result.getSourceAsObject(CoronaryFollowupES.class);
            CoronaryFollowupES coronaryFollowupNew = JSON.parseObject(jsonObject.getString("content"), CoronaryFollowupES.class);
            if(coronaryFollowup != null) {
                BeanUtils.copyProperties(coronaryFollowupNew, coronaryFollowup, getNullPropertyNames(coronaryFollowupNew));
            }else {
                coronaryFollowup = coronaryFollowupNew;
            }
            //保存新的随访详情信息
            if(StringUtils.isBlank(coronaryFollowup.getId())) {
                coronaryFollowup.setFollowup_id(followupId);
                coronaryFollowup.setCreateTime(DateUtil.getStringDate());
                coronaryFollowup.setCreateOperator(doctorMapping.getMappingCode());
                coronaryFollowup.setCreateOperatorName(d.getName());
                coronaryFollowup.setValidFlag("1");
                coronaryFollowup.setIdentityCardNo(idcard);
                coronaryFollowup.setOrgId(orgId);
                coronaryFollowup.setFollowOrgId(followOrgId);
                coronaryFollowup.setFollowDoctor(followDoctor);
                coronaryFollowup.setFollowTime(followTime);
                coronaryFollowup.setFollowType(followType);
                coronaryFollowup.setFollowDoctorType(followDoctorType);
                coronaryFollowup.setAssessSource(assessSource);
                coronaryFollowup.setNextFollowTime(nextFollowTime);
            }
            coronaryFollowup.setModifyTime(DateUtil.getStringDate());
            coronaryFollowup.setModifyOperator(doctorMapping.getMappingCode());
            coronaryFollowup.setModifyOperatorName(d.getName());
            elastricSearchSave.save(coronaryFollowup, esIndexCoronaryFollowup, esTypeCoronaryFollowup);
        }else if (type.equalsIgnoreCase("CerebralApoplexyFollowup")){
            //7.脑卒中日常随访
            Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexCerebralApoplexyFollowup).addType(esTypeCerebralApoplexyFollowup)
                    .build();
            SearchResult result = jestClient.execute(search);
            CerebralApoplexyFollowupES cerebralApoplexyFollowup = result.getSourceAsObject(CerebralApoplexyFollowupES.class);
            CerebralApoplexyFollowupES cerebralApoplexyFollowupNew = JSON.parseObject(jsonObject.getString("content"), CerebralApoplexyFollowupES.class);
            if(cerebralApoplexyFollowup != null) {
                BeanUtils.copyProperties(cerebralApoplexyFollowupNew, cerebralApoplexyFollowup, getNullPropertyNames(cerebralApoplexyFollowupNew));
            }else {
                cerebralApoplexyFollowup = cerebralApoplexyFollowupNew;
            }
            //保存新的随访详情信息
            if(StringUtils.isBlank(cerebralApoplexyFollowup.getId())) {
                cerebralApoplexyFollowup.setFollowup_id(followupId);
                cerebralApoplexyFollowup.setCreateTime(DateUtil.getStringDate());
                cerebralApoplexyFollowup.setCreateOperator(doctorMapping.getMappingCode());
                cerebralApoplexyFollowup.setCreateOperatorName(d.getName());
                cerebralApoplexyFollowup.setValidFlag("1");
                cerebralApoplexyFollowup.setIdentityCardNo(idcard);
                cerebralApoplexyFollowup.setOrgId(orgId);
                cerebralApoplexyFollowup.setFollowOrgId(followOrgId);
                cerebralApoplexyFollowup.setFollowDoctor(followDoctor);
                cerebralApoplexyFollowup.setFollowTime(followTime);
                cerebralApoplexyFollowup.setFollowType(followType);
                cerebralApoplexyFollowup.setFollowDoctorType(followDoctorType);
                cerebralApoplexyFollowup.setAssessSource(assessSource);
                cerebralApoplexyFollowup.setNextFollowTime(nextFollowTime);
            }
            cerebralApoplexyFollowup.setModifyTime(DateUtil.getStringDate());
            cerebralApoplexyFollowup.setModifyOperator(doctorMapping.getMappingCode());
            cerebralApoplexyFollowup.setModifyOperatorName(d.getName());
            elastricSearchSave.save(cerebralApoplexyFollowup,esIndexCerebralApoplexyFollowup , esTypeCerebralApoplexyFollowup);
        }else if (type.equalsIgnoreCase("FattyLiverFollowUp")){
            //8.脂肪肝日常随访
            Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexFattyLiverFollowUp).addType(esTypeFattyLiverFollowUp)
                    .build();
            SearchResult result = jestClient.execute(search);
            FattyLiverFollowUpES fattyLiverFollowUp = result.getSourceAsObject(FattyLiverFollowUpES.class);
            FattyLiverFollowUpES fattyLiverFollowUpNew = JSON.parseObject(jsonObject.getString("content"), FattyLiverFollowUpES.class);
            if(fattyLiverFollowUp != null) {
                BeanUtils.copyProperties(fattyLiverFollowUpNew, fattyLiverFollowUp, getNullPropertyNames(fattyLiverFollowUpNew));
            }else {
                fattyLiverFollowUp = fattyLiverFollowUpNew;
            }
            //保存新的随访详情信息
            if(StringUtils.isBlank(fattyLiverFollowUp.getId())) {
                fattyLiverFollowUp.setFollowup_id(followupId);
                fattyLiverFollowUp.setCreateTime(DateUtil.getStringDate());
                fattyLiverFollowUp.setCreateOperator(doctorMapping.getMappingCode());
                fattyLiverFollowUp.setCreateOperatorName(d.getName());
                fattyLiverFollowUp.setValidFlag("1");
                fattyLiverFollowUp.setIdentityCardNo(idcard);
                fattyLiverFollowUp.setOrgId(orgId);
                fattyLiverFollowUp.setFollowOrgId(followOrgId);
                fattyLiverFollowUp.setFollowDoctor(followDoctor);
                fattyLiverFollowUp.setFollowTime(followTime);
                fattyLiverFollowUp.setFollowType(followType);
                fattyLiverFollowUp.setFollowDoctorType(followDoctorType);
                fattyLiverFollowUp.setAssessSource(assessSource);
                fattyLiverFollowUp.setNextFollowTime(nextFollowTime);
            }
            fattyLiverFollowUp.setModifyTime(DateUtil.getStringDate());
            fattyLiverFollowUp.setModifyOperator(doctorMapping.getMappingCode());
            fattyLiverFollowUp.setModifyOperatorName(d.getName());
            elastricSearchSave.save(fattyLiverFollowUp,esIndexFattyLiverFollowUp , esTypeFattyLiverFollowUp);
        }else if (type.equalsIgnoreCase("NephropathyFollowup")){
            //8.脂肪肝日常随访
            Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexNephropathyFollowup).addType(esTypeNephropathyFollowup)
                    .build();
            SearchResult result = jestClient.execute(search);
            NephropathyFollowupES nephropathyFollowup = result.getSourceAsObject(NephropathyFollowupES.class);
            NephropathyFollowupES nephropathyFollowupNew = JSON.parseObject(jsonObject.getString("content"), NephropathyFollowupES.class);
            if(nephropathyFollowup != null) {
                BeanUtils.copyProperties(nephropathyFollowupNew, nephropathyFollowup, getNullPropertyNames(nephropathyFollowupNew));
            }else {
                nephropathyFollowup = nephropathyFollowupNew;
            }
            //保存新的随访详情信息
            if(StringUtils.isBlank(nephropathyFollowup.getId())) {
                nephropathyFollowup.setFollowup_id(followupId);
                nephropathyFollowup.setCreateTime(DateUtil.getStringDate());
                nephropathyFollowup.setCreateOperator(doctorMapping.getMappingCode());
                nephropathyFollowup.setCreateOperatorName(d.getName());
                nephropathyFollowup.setValidFlag("1");
                nephropathyFollowup.setIdentityCardNo(idcard);
                nephropathyFollowup.setOrgId(orgId);
                nephropathyFollowup.setFollowOrgId(followOrgId);
                nephropathyFollowup.setFollowDoctor(followDoctor);
                nephropathyFollowup.setFollowTime(followTime);
                nephropathyFollowup.setFollowType(followType);
                nephropathyFollowup.setFollowDoctorType(followDoctorType);
                nephropathyFollowup.setAssessSource(assessSource);
                nephropathyFollowup.setNextFollowTime(nextFollowTime);
            }
            nephropathyFollowup.setModifyTime(DateUtil.getStringDate());
            nephropathyFollowup.setModifyOperator(doctorMapping.getMappingCode());
            nephropathyFollowup.setModifyOperatorName(d.getName());
            elastricSearchSave.save(nephropathyFollowup,esIndexNephropathyFollowup , esTypeNephropathyFollowup);
        }else if (type.equalsIgnoreCase("ProstateFollowup")){
            //8.脂肪肝日常随访
            Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexProstateFollowup).addType(esTypeProstateFollowup)
                    .build();
            SearchResult result = jestClient.execute(search);
            ProstateFollowupES prostateFollowup = result.getSourceAsObject(ProstateFollowupES.class);
            ProstateFollowupES prostateFollowupNew = JSON.parseObject(jsonObject.getString("content"), ProstateFollowupES.class);
            if(prostateFollowup != null) {
                BeanUtils.copyProperties(prostateFollowupNew, prostateFollowup, getNullPropertyNames(prostateFollowupNew));
            }else {
                prostateFollowup = prostateFollowupNew;
            }
            //保存新的随访详情信息
            if(StringUtils.isBlank(prostateFollowup.getId())) {
                prostateFollowup.setFollowup_id(followupId);
                prostateFollowup.setCreateTime(DateUtil.getStringDate());
                prostateFollowup.setCreateOperator(doctorMapping.getMappingCode());
                prostateFollowup.setCreateOperatorName(d.getName());
                prostateFollowup.setValidFlag("1");
                prostateFollowup.setIdentityCardNo(idcard);
                prostateFollowup.setOrgId(orgId);
                prostateFollowup.setFollowOrgId(followOrgId);
                prostateFollowup.setFollowDoctor(followDoctor);
                prostateFollowup.setFollowTime(followTime);
                prostateFollowup.setFollowType(followType);
                prostateFollowup.setFollowDoctorType(followDoctorType);
                prostateFollowup.setAssessSource(assessSource);
                prostateFollowup.setNextFollowTime(nextFollowTime);

            }
            prostateFollowup.setModifyTime(DateUtil.getStringDate());
            prostateFollowup.setModifyOperator(doctorMapping.getMappingCode());
            prostateFollowup.setModifyOperatorName(d.getName());
            elastricSearchSave.save(prostateFollowup,esIndexProstateFollowup , esTypeProstateFollowup);
        }

    }

    /**
     * 获取妇科炎症随访档案
     * @param followupId
     * @return
     * @throws Exception
     */
    public JSONObject getGynecologicalInflammationFollowupArchives(String followupId) throws Exception {
        JSONObject jsonObject = new JSONObject();
        Followup followup = followupDao.findOne(Integer.parseInt(followupId));
        jsonObject.put("followup", followup);
        JestClient jestClient = null;
        GynecologicalInflammationFollowupArchivesES followupArchivesES = null;
        try {
            //根据随访ID获取随访记录详情
            jestClient = elasticFactory.getJestClient();
            //先根据条件查找出来
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            searchSourceBuilder.query(
                    new BoolQueryBuilder()
                            .must(QueryBuilders.matchQuery("followup_id", followupId))
            );
            Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexGynecologicalInflammationFollowupArchives).addType(esTypeGynecologicalInflammationFollowupArchives).build();
            SearchResult result = jestClient.execute(search);
            followupArchivesES = result.getSourceAsObject(GynecologicalInflammationFollowupArchivesES.class);
            jsonObject.put("content", followupArchivesES);
        } finally {
            if (jestClient != null) {
                jestClient.shutdownClient();
            }
        }
        return jsonObject;
    }

    /**
     * 保存慢性阻塞性肺病--首次问卷
     * @param followupId
     * @param jsonData
     * @param doctorCode
     * @return
     * @throws Exception
     */
    public COPDFirstTimeQuestionnaireES saveCOPDFirstTimeQuestionnaire(String followupId, String jsonData, String doctorCode) throws Exception {
        JSONObject jsonObject = JSONObject.parseObject(jsonData);
        Followup followup = followupDao.findOne(Integer.parseInt(followupId));
        Followup followupTemp = JSON.parseObject(jsonObject.getString("followup"), Followup.class);
        BeanUtils.copyProperties(followupTemp, followup, getNullPropertyNames(followupTemp));
        //设置当前随访状态
        if(followup.getStatus().equals("2")){
            followup.setStatus("3");
            followupDao.save(followup);
        }
        //判断是否已同步给智业，同步完再更新表单就要修改状态重新同步
        FollowupMapping followupMapping = followUpMappingDao.findByFollowupId(Integer.parseInt(followupId));
        if(followupMapping != null){
            followUpMappingDao.updateNeedUploadByFollowupId(1, Integer.parseInt(followupId));
        }
        BaseDoctorDO d = null;
        DoctorMappingDO doctorMapping = null;
        if(StringUtils.isNotBlank(doctorCode)){
            d = doctorDao.findById(doctorCode);
            doctorMapping = doctorMappingDao.findByDoctor(doctorCode);
        }else {
            throw new Exception("该医生不存在！");
        }
        JestClient jestClient = elasticFactory.getJestClient();
        //先根据条件查找出来
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.query(
                new BoolQueryBuilder()
                        .must(QueryBuilders.matchQuery("followup_id", followupId))
        );
        Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexCOPDFirstTimeQuestionnaire).addType(esTypeCOPDFirstTimeQuestionnaire)
                .build();
        SearchResult result = jestClient.execute(search);
        COPDFirstTimeQuestionnaireES COPD = result.getSourceAsObject(COPDFirstTimeQuestionnaireES.class);
        COPDFirstTimeQuestionnaireES COPDNew = JSON.parseObject(jsonObject.getString("content"), COPDFirstTimeQuestionnaireES.class);
        if(COPD != null) {
            BeanUtils.copyProperties(COPDNew, COPD, getNullPropertyNames(COPDNew));
        }else {
            COPD = COPDNew;
        }
        //保存新的随访详情信息
        if(StringUtils.isBlank(COPD.getId())) {
            COPD.setFollowup_id(followupId);
            COPD.setValidFlag("1");
            COPD.setCreateTime(new Date());
            COPD.setCreateOperator(doctorMapping.getMappingCode());
            COPD.setCreateOperatorName(d.getName());
        }
        COPD.setModifyTime(new Date());
        COPD.setModifyOperator(doctorMapping.getMappingCode());
        COPD.setModifyOperatorName(d.getName());
        elastricSearchSave.save(COPD, esIndexCOPDFirstTimeQuestionnaire, esTypeCOPDFirstTimeQuestionnaire);
        return COPD;
    }

    /**
     * 获取慢性阻塞性肺病--首次问卷
     * @param followupId
     * @return
     * @throws IOException
     */
    public JSONObject getCOPDFirstTimeQuestionnaire(String followupId) throws IOException {
        JSONObject jsonObject = new JSONObject();
        Followup followup = followupDao.findOne(Integer.parseInt(followupId));
        jsonObject.put("followup", followup);
        JestClient jestClient = null;
        COPDFirstTimeQuestionnaireES COPD = null;
        try {
            //根据随访ID获取随访记录详情
            jestClient = elasticFactory.getJestClient();
            //先根据条件查找出来
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            searchSourceBuilder.query(
                    new BoolQueryBuilder()
                            .must(QueryBuilders.matchQuery("followup_id", followupId))
            );
            Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexCOPDFirstTimeQuestionnaire).addType(esTypeCOPDFirstTimeQuestionnaire).build();
            SearchResult result = jestClient.execute(search);
            COPD = result.getSourceAsObject(COPDFirstTimeQuestionnaireES.class);
            jsonObject.put("content", COPD);
        } finally {
            if (jestClient != null) {
                jestClient.shutdownClient();
            }
        }
        return jsonObject;
    }

    /**
     * 保存慢性阻塞性肺病--日常随访
     * @param followupId
     * @param jsonData
     * @param doctorCode
     * @return
     * @throws Exception
     */
    public COPDDailyFollowupES saveCOPDDailyFollowup(String followupId, String jsonData, String doctorCode) throws Exception {
        JSONObject jsonObject = JSONObject.parseObject(jsonData);
        Followup followup = followupDao.findOne(Integer.parseInt(followupId));
        Followup followupTemp = JSON.parseObject(jsonObject.getString("followup"), Followup.class);
        BeanUtils.copyProperties(followupTemp, followup, getNullPropertyNames(followupTemp));
        //设置当前随访状态
        if(followup.getStatus().equals("2")){
            followup.setStatus("3");
            followupDao.save(followup);
        }
        //判断是否已同步给智业，同步完再更新表单就要修改状态重新同步
        FollowupMapping followupMapping = followUpMappingDao.findByFollowupId(Integer.parseInt(followupId));
        if(followupMapping != null){
            followUpMappingDao.updateNeedUploadByFollowupId(1, Integer.parseInt(followupId));
        }
        BaseDoctorDO d = null;
        DoctorMappingDO doctorMapping = null;
        if(StringUtils.isNotBlank(doctorCode)){
            d = doctorDao.findById(doctorCode);
            doctorMapping = doctorMappingDao.findByDoctor(doctorCode);
        }else {
            throw new Exception("该医生不存在！");
        }
        JestClient jestClient = elasticFactory.getJestClient();
        //先根据条件查找出来
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.query(
                new BoolQueryBuilder()
                        .must(QueryBuilders.matchQuery("followup_id", followupId))
        );
        Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexCOPDDailyFollowup).addType(esTypeCOPDDailyFollowup)
                .build();
        SearchResult result = jestClient.execute(search);
        COPDDailyFollowupES COPD = result.getSourceAsObject(COPDDailyFollowupES.class);
        COPDDailyFollowupES COPDNew = JSON.parseObject(jsonObject.getString("content"), COPDDailyFollowupES.class);
        if(COPD != null) {
            BeanUtils.copyProperties(COPDNew, COPD, getNullPropertyNames(COPDNew));
        }else {
            COPD = COPDNew;
        }
        //保存新的随访详情信息
        if(StringUtils.isBlank(COPD.getId())) {
            COPD.setFollowup_id(followupId);
            COPD.setValidFlag("1");
            COPD.setCreateTime(new Date());
            COPD.setCreateOperator(doctorMapping.getMappingCode());
            COPD.setCreateOperatorName(d.getName());
        }
        COPD.setModifyTime(new Date());
        COPD.setModifyOperator(doctorMapping.getMappingCode());
        COPD.setModifyOperatorName(d.getName());
        elastricSearchSave.save(COPD, esIndexCOPDDailyFollowup, esTypeCOPDDailyFollowup);
        return COPD;
    }

    /**
     * 获取慢性阻塞性肺病--日常随访
     * @param followupId
     * @return
     * @throws IOException
     */
    public JSONObject getCOPDDailyFollowup(String followupId) throws IOException {
        JSONObject jsonObject = new JSONObject();
        Followup followup = followupDao.findOne(Integer.parseInt(followupId));
        jsonObject.put("followup", followup);
        JestClient jestClient = null;
        COPDDailyFollowupES COPD = null;
        try {
            //根据随访ID获取随访记录详情
            jestClient = elasticFactory.getJestClient();
            //先根据条件查找出来
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            searchSourceBuilder.query(
                    new BoolQueryBuilder()
                            .must(QueryBuilders.matchQuery("followup_id", followupId))
            );
            Search search = new Search.Builder(searchSourceBuilder.toString()).addIndex(esIndexCOPDDailyFollowup).addType(esTypeCOPDDailyFollowup).build();
            SearchResult result = jestClient.execute(search);
            COPD = result.getSourceAsObject(COPDDailyFollowupES.class);
            jsonObject.put("content", COPD);
        } finally {
            if (jestClient != null) {
                jestClient.shutdownClient();
            }
        }
        return jsonObject;
    }
}
