package com.yihu.jw.adaption.controller;

import com.yihu.jw.adaption.common.ExtendEndPoint;
import com.yihu.jw.adaption.model.AdapterDataSet;
import com.yihu.jw.adaption.model.OrgAdapterPlan;
import com.yihu.jw.adaption.service.AdapterDataSetService;
import com.yihu.jw.adaption.service.OrgAdapterPlanService;
import com.yihu.jw.constants.ApiVersion;
import com.yihu.jw.ehr.model.adaption.MAdapterDataSet;
import com.yihu.jw.ehr.model.adaption.MAdapterDataVo;
import com.yihu.jw.ehr.model.adaption.MAdapterRelationship;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Collection;
import java.util.List;

/**
 * 适配管理方案适配管理
 *
 * @author lincl
 * @version 1.0
 * @created 2016.2.3
 */
@RestController
@RequestMapping(ApiVersion.Version1_0 + "/adapter")
@Api(value = "AdapterDataSetEndPoint", description = "数据集", tags = {"适配服务-数据集"})
public class AdapterDataSetEndPoint extends ExtendEndPoint<MAdapterDataSet> {

    @Autowired
    private AdapterDataSetService adapterDataSetService;
    @Autowired
    private OrgAdapterPlanService orgAdapterPlanService;

    @RequestMapping(value = "/plan/{planId}/datasets", method = RequestMethod.GET)
    @ApiOperation(value = "查询定制数据集")
    public Collection<MAdapterRelationship> searchAdapterDataSet(
            @ApiParam(name = "planId", value = "适配方案id", defaultValue = "")
            @PathVariable(value = "planId") Long planId,
            @ApiParam(name = "code", value = "代码查询值", defaultValue = "")
            @RequestParam(value = "code", required = false) String code,
            @ApiParam(name = "name", value = "名称查询值", defaultValue = "")
            @RequestParam(value = "name", required = false) String name,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档", defaultValue = "+name,+createTime")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size", required = false) int size,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception{

        OrgAdapterPlan orgAdapterPlan = orgAdapterPlanService.retrieve(planId);
        if (orgAdapterPlan==null)
            throw errNotFound();
        List ls = adapterDataSetService.searchAdapterDataSet(orgAdapterPlan, code, name, sorts, page, size);
        pagedResponse(request, response,(long) adapterDataSetService.searchDataSetInt(orgAdapterPlan, code, name), page, size);
        return ls;
    }

    @RequestMapping("/plan/{planId}/datasets/{dataSetId}/datametas")
    @ApiOperation(value = "搜索定制数据元")
    public Collection<MAdapterDataVo> searchAdapterMetaData(
            @ApiParam(name = "planId", value = "适配方案id", defaultValue = "")
            @PathVariable(value = "planId") Long planId,
            @ApiParam(name = "dataSetId", value = "数据集id", defaultValue = "")
            @PathVariable(value = "dataSetId") Long dataSetId,
            @ApiParam(name = "code", value = "代码查询值", defaultValue = "")
            @RequestParam(value = "code", required = false) String code,
            @ApiParam(name = "name", value = "名称查询值", defaultValue = "")
            @RequestParam(value = "name", required = false) String name,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档", defaultValue = "+name,+createTime")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size", required = false) int size,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception {

        OrgAdapterPlan orgAdapterPlan = orgAdapterPlanService.retrieve(planId);
        if (orgAdapterPlan==null)
            throw errNotFound();
        List ls = adapterDataSetService.searchAdapterMetaData(orgAdapterPlan, dataSetId, code, name, sorts, page, size);
        pagedResponse(request, response, (long) adapterDataSetService.searchMetaDataInt(orgAdapterPlan, dataSetId, code, name), page, size);
        return ls;
    }


    @RequestMapping(value = "/datameta/{id}", method = RequestMethod.GET)
    @ApiOperation(value = "根据数据集ID获取数据元适配关系明细")
    public MAdapterDataSet getAdapterMetaData(
            @ApiParam(name = "id", value = "id", defaultValue = "")
            @PathVariable(value = "id") Long id) throws Exception{

        return  getModel(adapterDataSetService.retrieve(id));
    }


    @RequestMapping(value = "/datameta/{id}", method = RequestMethod.PUT, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "修改数据元映射关系")
    public MAdapterDataSet updateAdapterMetaData(
            @ApiParam(name = "id", value = "编号", defaultValue = "")
            @PathVariable(value = "id") Long id,
            @ApiParam(name = "jsonModel", value = "数据元模型", defaultValue = "")
            @RequestBody String jsonModel) throws Exception{

        AdapterDataSet adapterDataSet = adapterDataSetService.retrieve(id);
        if(adapterDataSet==null)
            throw errNotFound();
        return getModel(saveAdapterMetaData(adapterDataSet, jsonModel));
    }


    @RequestMapping(value = "/datameta", method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "新增数据元映射关系")
    public MAdapterDataSet createAdapterMetaData(
            @ApiParam(name = "jsonModel", value = "数据元模型", defaultValue = "")
            @RequestBody String jsonModel) throws Exception{

        return getModel(saveAdapterMetaData(new AdapterDataSet(), jsonModel));
    }

    @RequestMapping(value = "/datametas", method = RequestMethod.DELETE)
    @ApiOperation(value = "批量删除数据元映射关系")
    public boolean delMetaData(
            @ApiParam(name = "ids", value = "数据元编号集")
            @RequestParam("ids") String ids) throws Exception{

        if (StringUtils.isEmpty(ids))
            throw errMissId();
        adapterDataSetService.deleteAdapterDataSet(strToLongArr(ids));
        return true;
    }

    @RequestMapping(value = "/data_set/{data_set_id}/is_left/meta", method = RequestMethod.GET)
    @ApiOperation(value = "判断除了metaIds之外是否还存在其他的数据元")
    public boolean isLeftMeta(
            @ApiParam(name = "plan_id", value = "数据集编号")
            @RequestParam("plan_id") long planId,
            @ApiParam(name = "data_set_id", value = "数据集编号")
            @PathVariable("data_set_id") long dataSetId,
            @ApiParam(name = "meta_ids", value = "数据元编号集")
            @RequestParam("meta_ids") String metaIds) {

        return adapterDataSetService.isLeftMeta(planId, dataSetId, strToLongArr(metaIds));
    }

    @RequestMapping(value = "/plan/{planId}/data_set/{data_set_id}/std_meta", method = RequestMethod.GET)
    @ApiOperation(value = "过滤后的标准数据元分页查询")
    public Collection<MAdapterRelationship> searchStdMeta(
            @ApiParam(name = "planId", value = "适配方案id", defaultValue = "")
            @PathVariable(value = "planId") Long planId,
            @ApiParam(name = "data_set_id", value = "字典编号", defaultValue = "")
            @PathVariable(value = "data_set_id") Long dataSetId,
            @ApiParam(name = "seach_name", value = "代码查询值", defaultValue = "")
            @RequestParam(value = "seach_name", required = false) String searchName,
            @ApiParam(name = "mode", value = "编辑模式（new/modify）", defaultValue = "")
            @RequestParam(value = "mode", required = false) String mode,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档", defaultValue = "+name,+createTime")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size", required = false) int size,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception {

        OrgAdapterPlan orgAdapterPlan = orgAdapterPlanService.retrieve(planId);
        if (orgAdapterPlan == null)
            throw errNotFound();

        List ls = adapterDataSetService.searchStdMeta(orgAdapterPlan, dataSetId, searchName, mode, sorts, page, size);
        pagedResponse(request, response, (long) adapterDataSetService.countStdMeta(orgAdapterPlan, dataSetId, searchName, mode), page, size);
        return ls;
    }

    private AdapterDataSet saveAdapterMetaData(AdapterDataSet adapterDataSet, String jsonModel)  {
        AdapterDataSet adapterDataSetModel = null;
        try {
            adapterDataSetModel = toDecodeObj(jsonModel, AdapterDataSet.class);
        } catch (IOException e) {
            throw errParm();
        }

        adapterDataSet.setAdapterPlanId(adapterDataSetModel.getAdapterPlanId());
        adapterDataSet.setMetaDataId(adapterDataSetModel.getMetaDataId());
        adapterDataSet.setDataSetId(adapterDataSetModel.getDataSetId());
        adapterDataSet.setOrgDataSetSeq(adapterDataSetModel.getOrgDataSetSeq());
        adapterDataSet.setOrgMetaDataSeq(adapterDataSetModel.getOrgMetaDataSeq());
        adapterDataSet.setDataType(adapterDataSetModel.getDataType());
        adapterDataSet.setDescription(adapterDataSetModel.getDescription());
        if (adapterDataSetModel.getId() == null) {
            OrgAdapterPlan orgAdapterPlan = orgAdapterPlanService.retrieve(adapterDataSetModel.getAdapterPlanId());
            return adapterDataSetService.addAdapterDataSet(adapterDataSet, orgAdapterPlan);
        } else {
            return adapterDataSetService.save(adapterDataSet);
        }
    }
}
