package com.yihu.jw.standard.controller;

import com.yihu.jw.ehr.model.standard.MStdSource;
import com.yihu.jw.adaption.common.ExtendEndPoint;
import com.yihu.jw.constants.ApiVersion;
import com.yihu.jw.constants.ServiceApi;
import com.yihu.jw.standard.model.StandardSource;
import com.yihu.jw.standard.service.StdSourceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

/**
 * @author lincl
 * @version 1.0
 * @created 2016.1.23
 */
@RestController
@RequestMapping(ApiVersion.Version1_0)
@Api(value = "StdSourceEndPoint", description = "标准来源服务", tags = {"标准服务-标准来源"})
public class StdSourceEndPoint extends ExtendEndPoint<MStdSource> {

    @Autowired
    private StdSourceService stdSourceService;

    @RequestMapping(value = ServiceApi.Standards.Sources, method = RequestMethod.GET)
    @ApiOperation(value = "标准来源分页搜索")
    public Collection<MStdSource> searchSources(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段", defaultValue = "id,name,secret,url,createTime")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件", defaultValue = "")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档", defaultValue = "+name,+createTime")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size", required = false) int size,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception {

        List appList = stdSourceService.search(fields, filters, sorts, page, size);
        pagedResponse(request, response, stdSourceService.getCount(filters), page, size);
        return convertToModels(appList, new ArrayList<>(appList.size()), MStdSource.class, fields);
    }

    @RequestMapping(value = ServiceApi.Standards.NoPageSources, method = RequestMethod.GET)
    @ApiOperation(value = "标准来源分页搜索(不分页)")
    public Collection<MStdSource> searchSourcesWithoutPaging(
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件", defaultValue = "")
            @RequestParam(value = "filters", required = false) String filters) throws Exception {
        List appList = stdSourceService.search(filters);
        return convertToModels(appList, new ArrayList<>(appList.size()), MStdSource.class, "");
    }




    @RequestMapping(value = ServiceApi.Standards.Source, method = RequestMethod.GET)
    @ApiOperation(value = "获取标准来源信息")
    public MStdSource getStdSource(
            @ApiParam(name = "id", value = "标准来源编号", defaultValue = "")
            @PathVariable(value = "id") String id) {

        return getModel(stdSourceService.retrieve(id));
    }


    @RequestMapping(value = ServiceApi.Standards.Source, method = RequestMethod.PUT, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "修改标准来源")
    public MStdSource updateStdSource(
            @ApiParam(name = "id", value = "标准来源编号", defaultValue = "")
            @PathVariable(value = "id") String id,
            @ApiParam(name = "model", value = "json数据模型", defaultValue = "")
            @RequestBody String model) throws Exception {

        StandardSource standardSourceModel = toEntity(model, StandardSource.class);
        StandardSource standardSource = stdSourceService.retrieve(id);
        if (standardSource == null)
            throw errNotFound();

        standardSourceModel.setId(id);
        standardSourceModel.setUpdateDate(new Date());
        return getModel(stdSourceService.save(standardSourceModel));
    }


    @RequestMapping(value = ServiceApi.Standards.Sources, method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "新增标准来源")
    public MStdSource addStdSource(
            @ApiParam(name = "model", value = "json数据模型", defaultValue = "")
            @RequestBody String model) throws Exception {

        StandardSource standardSource = toEntity(model, StandardSource.class);
        if (stdSourceService.isSourceCodeExist(standardSource.getCode()))
            throw errRepeatCode();
        standardSource.setCreateDate(new Date());
        return getModel(stdSourceService.save(standardSource));
    }


    @RequestMapping(value = ServiceApi.Standards.Sources, method = RequestMethod.DELETE)
    @ApiOperation(value = "通过id组删除标准来源，多个id以,分隔")
    public boolean delStdSources(
            @ApiParam(name = "ids", value = "标准来源编号", defaultValue = "")
            @RequestParam(value = "ids") String ids) throws Exception {

        stdSourceService.deleteSource(ids.split(","));
        return true;
    }

    @RequestMapping(value = ServiceApi.Standards.Source, method = RequestMethod.DELETE)
    @ApiOperation(value = "通过id删除标准来源")
    public boolean delStdSource(
            @ApiParam(name = "id", value = "标准来源编号", defaultValue = "")
            @PathVariable(value = "id") String id) throws Exception {

        stdSourceService.delete(id);
        return true;
    }

    @RequestMapping(value = ServiceApi.Standards.IsSourceCodeExist,method = RequestMethod.GET)
    public boolean isCodeExist(@RequestParam(value="code")String code) throws ParseException {

        return stdSourceService.isSourceCodeExist(code);
    }
}
