package com.yihu.jw.wlyy.statistics.etl.compute;


import com.yihu.jw.wlyy.statistics.util.DateUtil;
import com.yihu.jw.wlyy.statistics.vo.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * Created by chenweida on 2017/6/1.
 */
@Component
@Scope("prototype")
public class ComputeHelper {
    @Autowired
    private JdbcTemplate jdbcTemplate;

    public String findOrgType(String wlyyJobCongId){
        String orgType="";
        String sql = "SELECT IFNULL(dict_value,'') from wlyy_hospital_sys_dict WHERE dict_name = 'quota_org_type' and dict_code = '"+wlyyJobCongId+"'";
        List<String> orgTypes = jdbcTemplate.queryForList(sql,String.class);
        if (orgTypes.size()>0){
            return orgTypes.get(0);
        }
        return orgType;
    }

    /**
     * @param dataModels      过滤之后的数据
     * @param dimensionQuotas 指标的维度
     * @param wlyyJobCongId
     * @param endTime
     * @return
     */
    public List<SaveModel> compute(List<DataModel> dataModels, List<BaseDimensionQuota> dimensionQuotas, String wlyyJobCongId, String endTime, String timeLevel,String areaLevel) {
        try {
            //得到机构类型
            String orgType = findOrgType(wlyyJobCongId);
            if (StringUtils.isBlank(areaLevel)){
                areaLevel="6";
            }

            //计算数据 返回map key 根据团队和维度拼凑在map中的key   1-2-1  如果维度是长度是0返回团队id 1
            Map<String, List<DataModel>> computeMap = computeDataModel(dimensionQuotas, dataModels,wlyyJobCongId,areaLevel);
            List<SaveModel> savemodels = null;
            switch (areaLevel){
                case "2":
                    savemodels = initAndSetResultCity( dimensionQuotas, computeMap, wlyyJobCongId, endTime, timeLevel);
                    break;
                case "4":
                    List<BaseOrgDO> orgDOs = findAllOrg(orgType);
                    savemodels = initAndSetResultOrg(orgDOs, dimensionQuotas, computeMap, wlyyJobCongId, endTime, timeLevel);
                    break;
                case "5": //医养科室字段实际存储为团队
                    //得到全部团队
                    List<Map<String,Object>> teams = findAllTeam(orgType);
                    //初始化map并且设置统计出来的数目
                    savemodels = initAndSetResultTeam(teams, dimensionQuotas, computeMap, wlyyJobCongId, endTime, timeLevel);
                    break;
                default:
                    //得到全部医生
                    List<BaseDoctorDO> doctors = findAllDoctor(orgType);
                    //初始化map并且设置统计出来的数目
                    savemodels = initAndSetResult(doctors, dimensionQuotas, computeMap, wlyyJobCongId, endTime, timeLevel);
                    break;
            }
            return savemodels;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * //计算数据 返回map
     *
     * @param dataModels      过滤过后的元数据
     * @param dimensionQuotas 该指标的维度
     * @return
     */
    private Map<String, List<DataModel>> computeDataModel(List<BaseDimensionQuota> dimensionQuotas, List<DataModel> dataModels,String wlyyJobCongId,String areaLevel) {
        Map<String, List<DataModel>> returnMap = new HashMap<>();
        dataModels.stream().forEach(one -> {
            String key;
            switch (areaLevel){
                case "2":
                    key = getCityKey(one,dimensionQuotas.size());
                    break;
                case "4":
                    key = getOrgKey(one, dimensionQuotas.size());
                    break;
                case "5":
                    key = getTeamKey(one, dimensionQuotas.size());
                    break;
                default:
                    key = getKey(one, dimensionQuotas.size());
                    break;
            }
//            if ("45".equals(wlyyJobCongId)||"47".equals(wlyyJobCongId)||"50".equals(wlyyJobCongId)){
//                key = getCityKey(one,dimensionQuotas.size());
//            }
//            else if("42".equals(wlyyJobCongId)||"41".equals(wlyyJobCongId)||"43".equals(wlyyJobCongId)||"46".equals(wlyyJobCongId)||"48".equals(wlyyJobCongId)
//            ||"49".equals(wlyyJobCongId)){
//                key = getOrgKey(one, dimensionQuotas.size());
//            }else{
//                key = getKey(one, dimensionQuotas.size());
//            }
            List<DataModel> sms = new ArrayList<DataModel>();
            if (returnMap.containsKey(key)) {
                sms = returnMap.get(key);
            }
            sms.add(one);
            returnMap.put(key, sms);
        });

        return returnMap;
    }

    /**
     * 获取key  根据团队和维度拼凑在map中的key   1-2-1  如果维度是长度是0返回团队id 1
     *
     * @param one
     * @param size
     * @return
     */
    private String getKey(DataModel one, int size) {
        try {
            if (size > 0) {
                StringBuffer str = new StringBuffer(one.getDoctor());
                for (int i = 1; i <= size; i++) {
                    String invokeKey = (String) DataModel.class.getMethod("getSlaveKey" + i).invoke(one);
                    str.append("-" + invokeKey);
                }
                return str.toString();
            } else {
                return one.getDoctor();
            }
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    private String getOrgKey(DataModel one, int size) {
        try {
            if (size > 0) {
                StringBuffer str = new StringBuffer(one.getHospital());
                for (int i = 1; i <= size; i++) {
                    String invokeKey = (String) DataModel.class.getMethod("getSlaveKey" + i).invoke(one);
                    str.append("-" + invokeKey);
                }
                return str.toString();
            } else {
                return one.getHospital();
            }
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    private String getTeamKey(DataModel one, int size) {
        try {
            if (size > 0) {
                StringBuffer str = new StringBuffer(one.getDept());
                for (int i = 1; i <= size; i++) {
                    String invokeKey = (String) DataModel.class.getMethod("getSlaveKey" + i).invoke(one);
                    str.append("-" + invokeKey);
                }
                return str.toString();
            } else {
                return one.getDept();
            }
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    private String getCityKey(DataModel one, int size) {
        try {
             StringBuffer str = new StringBuffer("330100");
             for (int i = 1; i <= size; i++) {
                 String invokeKey = (String) DataModel.class.getMethod("getSlaveKey" + i).invoke(one);
                 str.append("-" + invokeKey);
             }
             return str.toString();
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 得到全部机构
     *
     * @return
     */
    private List<BaseOrgDO> findAllOrg(String orgType) {
        String sql = "SELECT " +
                "o.code," +
                "o.name," +
                "o.town_code," +
                "o.town_name" +
                " FROM " +
                "base_org o" +
                " WHERE "+
                " o.del='1' and o.code not in(SELECT dict_code FROM `base`.`wlyy_hospital_sys_dict` WHERE `dict_name` = 'jkzl_org')";
        if(StringUtils.isNotBlank(orgType)){
            sql += " and o.type = '"+orgType+"'";
        }
        List<BaseOrgDO> orgDOs = jdbcTemplate.query(sql, new BeanPropertyRowMapper(BaseOrgDO.class));
        return orgDOs;
    }

    /**
     * 得到全部医生
     *
     * @returncount(distinct
     */
    private List<Map<String,Object>> findAllTeam(String orgType) {
        String sql = " SELECT " +
                " bt.id AS code, " +
                " bt.name, " +
                " bt.org_code, " +
                " bt.org_name, " +
                " o.town_code, " +
                " o.town_name  " +
                " FROM " +
                " base_team bt, " +
                " base_org o  " +
                " WHERE " +
                " bt.org_code = o.CODE  " +
                " AND bt.del = 1  " +
                " AND o.del = 1  " +
                " AND o.CODE NOT IN ( SELECT dict_code FROM `base`.`wlyy_hospital_sys_dict` WHERE `dict_name` = 'jkzl_org' ) ";
        if(StringUtils.isNotBlank(orgType)){
            sql += " and o.type = '"+orgType+"'";
        }
        List<Map<String,Object>> teams = jdbcTemplate.queryForList(sql);
        return teams;
    }

    /**
     * 得到全部医生
     *
     * @return
     */
    private List<BaseDoctorDO> findAllDoctor(String orgType) {
        String sql = " SELECT" +
                " bd.id AS CODE," +
                " bd.NAME, " +
                " bt.id as dept_code, " +
                " bt.name AS dept_name, " +
                " o.code as org_code, " +
                " o.name as org_name, " +
                " o.town_code, " +
                " o.town_name  " +
                " FROM " +
                " base_doctor bd, " +
                " base_org o, " +
                " base_team bt, " +
                " base_team_member btm  " +
                " WHERE " +
                " btm.doctor_code = bd.id  " +
                " AND o.CODE = bt.org_code " +
                " and bt.id = btm.team_code " +
                " AND btm.del = 1  " +
                " AND o.del = 1  " +
                " AND o.CODE NOT IN ( SELECT dict_code  FROM `base`.`wlyy_hospital_sys_dict`  WHERE `dict_name` = 'jkzl_org') ";
        if(StringUtils.isNotBlank(orgType)){
            sql += " and o.type = '"+orgType+"'";
        }
        List<BaseDoctorDO> doctorDOS = jdbcTemplate.query(sql, new BeanPropertyRowMapper(BaseDoctorDO.class));
        return doctorDOS;
    }

    /**
     * 初始化map
     */
    private List<SaveModel> initAndSetResultTeam(List<Map<String,Object>> teams, List<BaseDimensionQuota> dimensionQuotas, Map<String, List<DataModel>> countMap, String wlyyJobCongId, String endTime, String timeLevel) throws Exception {
        List<SaveModel> returnList = new ArrayList<>();
        Map<String, SaveModel> lastMaps = new HashMap<>();

        //初始化医生的数据
        for (int i = 0; i < teams.size(); i++) {
                Map<String,Object> teamDO = teams.get(i);
            //排除测试机构
            SaveModel saveModel = new SaveModel();
            saveModel.setCity("330100");
            saveModel.setCityName("杭州市");
            saveModel.setTown(teamDO.get("town_code").toString());
            saveModel.setTownName(teamDO.get("town_name").toString());
            saveModel.setHospital(teamDO.get("org_code").toString());
            saveModel.setHospitalName(teamDO.get("org_name").toString());
            saveModel.setDept(teamDO.get("code").toString());
            String deptName = teamDO.get("name").toString();
            deptName = deptName.replace("团队","社区");
            deptName = deptName.replace("服务","");
            if(!deptName.contains("社区")){
                deptName = deptName+"社区";
            }
            saveModel.setDeptName(deptName);
            saveModel.setQuotaCode(wlyyJobCongId);
            saveModel.setCreateTime(new Date());
            saveModel.setTimeLevel(timeLevel);
            saveModel.setAreaLevel("5");//团队
            saveModel.setQuotaDate(DateUtil.strToDate(endTime, "yyyy-MM-dd"));
            lastMaps.put(teamDO.get("code").toString(), saveModel);
        }

        //根据维度初始化数据
        for (int i = 0; i < dimensionQuotas.size(); i++) {
            Map<String, SaveModel> mapTemp = new HashMap<>();
            BaseDimensionQuota dimensionQuota = dimensionQuotas.get(i);
            List<Map<String, Object>> listMap = jdbcTemplate.queryForList(dimensionQuota.getDictSql());
            for (Map.Entry<String, SaveModel> oneSaveModel : lastMaps.entrySet()) {
                for (int j = 0; j < listMap.size(); j++) {
                    SaveModel saveModelTemp = new SaveModel();
                    Map<String, Object> dictModel = listMap.get(j);

                    BeanUtils.copyProperties(oneSaveModel.getValue(), saveModelTemp);
                    SaveModel.class.getMethod("setSlaveKey" + (i + 1) + "Name", String.class).invoke(saveModelTemp, dictModel.get("name"));
                    SaveModel.class.getMethod("setSlaveKey" + (i + 1), String.class).invoke(saveModelTemp, dictModel.get("code"));
                    //根据维度生成新的key
                    StringBuffer strKey = new StringBuffer(oneSaveModel.getKey() + "-" + dictModel.get("code"));
                    mapTemp.put(strKey.toString(), saveModelTemp);
                }
            }
            //如果字典长度是0 抛出异常
            if (listMap.size() == 0) {
                throw new Exception("dict size:0,sql:" + dimensionQuota.getDictSql());
            }
            lastMaps = mapTemp;
        }

        //设置值
        for (Map.Entry<String, SaveModel> one : lastMaps.entrySet()) {
            if (countMap.containsKey(one.getKey())) {
                List<DataModel> dataModels = countMap.get(one.getKey());
                Double result1 = 0.0;
                Double result2 = 0.0;
                for (int j = 0; j < dataModels.size(); j++) {
                    DataModel dataModel = dataModels.get(j);
                    Double num = 0.0;
                    if (dataModel.getNum()!=null){
                        num=dataModel.getNum();
                    }else {
                        num=1.0;
                    }
                    result1 += num;
                    result2++;
                }
                SaveModel saveModel = one.getValue();
                saveModel.setResult1(result1);
                saveModel.setResult2(result2);
                if(dataModels.size()>0&& StringUtils.isNotEmpty(dataModels.get(0).getSlaveKey1())&&!StringUtils.isNotEmpty(saveModel.getSlaveKey1())){
                    saveModel.setSlaveKey1(dataModels.get(0).getSlaveKey1());
                }
                if(dataModels.size()>0&& StringUtils.isNotEmpty(dataModels.get(0).getSlaveKey2())&&!StringUtils.isNotEmpty(saveModel.getSlaveKey2())){
                    saveModel.setSlaveKey2(dataModels.get(0).getSlaveKey2());
                }
                returnList.add(saveModel);
            } else {
                returnList.add(one.getValue());
            }
        }
        return returnList;
    }


    /**
     * 初始化map
     */
    private List<SaveModel> initAndSetResult(List<BaseDoctorDO> doctors, List<BaseDimensionQuota> dimensionQuotas, Map<String, List<DataModel>> countMap, String wlyyJobCongId, String endTime, String timeLevel) throws Exception {
        List<SaveModel> returnList = new ArrayList<>();
        Map<String, SaveModel> lastMaps = new HashMap<>();

        //初始化医生的数据
        for (int i = 0; i < doctors.size(); i++) {
            BaseDoctorDO one = doctors.get(i);
            //排除测试机构
            SaveModel saveModel = new SaveModel();
            saveModel.setCity("330100");
            saveModel.setCityName("杭州市");
            saveModel.setTown(one.getTownCode());
            saveModel.setTownName(one.getTownName());
            String orgCode = one.getOrgCode();
            saveModel.setHospital(orgCode);
            saveModel.setHospitalName(one.getOrgName());
            saveModel.setDoctor(one.getCode());
            saveModel.setDoctorName(one.getName());
            saveModel.setDept(one.getDeptCode());
            String deptName = one.getDeptName();
            deptName = deptName.replace("团队","社区");
            deptName = deptName.replace("服务","");
            if(!deptName.contains("社区")){
                deptName = deptName+"社区";
            }
            saveModel.setDeptName(deptName);
            saveModel.setQuotaCode(wlyyJobCongId);
            saveModel.setCreateTime(new Date());
            saveModel.setTimeLevel(timeLevel);
            saveModel.setAreaLevel("6");//医生
            saveModel.setQuotaDate(DateUtil.strToDate(endTime, "yyyy-MM-dd"));
            lastMaps.put(one.getCode(), saveModel);
        }

        //根据维度初始化数据
            for (int i = 0; i < dimensionQuotas.size(); i++) {
                Map<String, SaveModel> mapTemp = new HashMap<>();
                BaseDimensionQuota dimensionQuota = dimensionQuotas.get(i);
                List<Map<String, Object>> listMap = jdbcTemplate.queryForList(dimensionQuota.getDictSql());
                for (Map.Entry<String, SaveModel> oneSaveModel : lastMaps.entrySet()) {
                    for (int j = 0; j < listMap.size(); j++) {
                        SaveModel saveModelTemp = new SaveModel();
                        Map<String, Object> dictModel = listMap.get(j);

                        BeanUtils.copyProperties(oneSaveModel.getValue(), saveModelTemp);
                        SaveModel.class.getMethod("setSlaveKey" + (i + 1) + "Name", String.class).invoke(saveModelTemp, dictModel.get("name"));
                        SaveModel.class.getMethod("setSlaveKey" + (i + 1), String.class).invoke(saveModelTemp, dictModel.get("code"));
                        //根据维度生成新的key
                        StringBuffer strKey = new StringBuffer(oneSaveModel.getKey() + "-" + dictModel.get("code"));
                        mapTemp.put(strKey.toString(), saveModelTemp);
                    }
                }
            //如果字典长度是0 抛出异常
            if (listMap.size() == 0) {
                throw new Exception("dict size:0,sql:" + dimensionQuota.getDictSql());
            }
            lastMaps = mapTemp;
        }

        //设置值
        for (Map.Entry<String, SaveModel> one : lastMaps.entrySet()) {
            if (countMap.containsKey(one.getKey())) {
                List<DataModel> dataModels = countMap.get(one.getKey());
                Double result1 = 0.0;
                Double result2 = 0.0;
                for (int j = 0; j < dataModels.size(); j++) {
                    DataModel dataModel = dataModels.get(j);
                    Double num = 0.0;
                    if (dataModel.getNum()!=null){
                        num=dataModel.getNum();
                    }else {
                        num=1.0;
                    }
                    result1 += num;
                    result2++;
                }
                SaveModel saveModel = one.getValue();
                saveModel.setResult1(result1);
                saveModel.setResult2(result2);
                if(dataModels.size()>0&& StringUtils.isNotEmpty(dataModels.get(0).getSlaveKey1())&&!StringUtils.isNotEmpty(saveModel.getSlaveKey1())){
                    saveModel.setSlaveKey1(dataModels.get(0).getSlaveKey1());
                }
                if(dataModels.size()>0&& StringUtils.isNotEmpty(dataModels.get(0).getSlaveKey2())&&!StringUtils.isNotEmpty(saveModel.getSlaveKey2())){
                    saveModel.setSlaveKey2(dataModels.get(0).getSlaveKey2());
                }
                returnList.add(saveModel);
            } else {
                returnList.add(one.getValue());
            }
        }
        return returnList;
    }

    /**
     * 初始化map
     */
    private List<SaveModel> initAndSetResultOrg(List<BaseOrgDO> orgDOs, List<BaseDimensionQuota> dimensionQuotas, Map<String, List<DataModel>> countMap, String wlyyJobCongId, String endTime, String timeLevel) throws Exception {
        List<SaveModel> returnList = new ArrayList<>();
        Map<String, SaveModel> lastMaps = new HashMap<>();

        //初始化医生的数据
        for (int i = 0; i < orgDOs.size(); i++) {
            BaseOrgDO one = orgDOs.get(i);
            SaveModel saveModel = new SaveModel();
            saveModel.setCity("330100");
            saveModel.setCityName("杭州市");
            saveModel.setTown(one.getTownCode());
            saveModel.setTownName(one.getTownName());
            saveModel.setHospitalName(one.getName());
            saveModel.setHospital(one.getCode());
            saveModel.setQuotaCode(wlyyJobCongId);
            saveModel.setCreateTime(new Date());
            saveModel.setTimeLevel(timeLevel);
            saveModel.setAreaLevel("4");//机构
            saveModel.setQuotaDate(DateUtil.strToDate(endTime, "yyyy-MM-dd"));
            lastMaps.put(one.getCode(), saveModel);
        }

        //根据维度初始化数据
        for (int i = 0; i < dimensionQuotas.size(); i++) {
            Map<String, SaveModel> mapTemp = new HashMap<>();
            BaseDimensionQuota dimensionQuota = dimensionQuotas.get(i);
            List<Map<String, Object>> listMap = jdbcTemplate.queryForList(dimensionQuota.getDictSql());
            for (Map.Entry<String, SaveModel> oneSaveModel : lastMaps.entrySet()) {
                for (int j = 0; j < listMap.size(); j++) {
                    SaveModel saveModelTemp = new SaveModel();
                    Map<String, Object> dictModel = listMap.get(j);

                    BeanUtils.copyProperties(oneSaveModel.getValue(), saveModelTemp);
                    SaveModel.class.getMethod("setSlaveKey" + (i + 1) + "Name", String.class).invoke(saveModelTemp, dictModel.get("name"));
                    SaveModel.class.getMethod("setSlaveKey" + (i + 1), String.class).invoke(saveModelTemp, dictModel.get("code"));
                    //根据维度生成新的key
                    StringBuffer strKey = new StringBuffer(oneSaveModel.getKey() + "-" + dictModel.get("code"));
                    mapTemp.put(strKey.toString(), saveModelTemp);
                }
            }
            //如果字典长度是0 抛出异常
            if (listMap.size() == 0) {
                throw new Exception("dict size:0,sql:" + dimensionQuota.getDictSql());
            }
            lastMaps = mapTemp;
        }

        //设置值
        for (Map.Entry<String, SaveModel> one : lastMaps.entrySet()) {
            if (countMap.containsKey(one.getKey())) {
                List<DataModel> dataModels = countMap.get(one.getKey());
                Double result1 = 0.0;
                Double result2 = 0.0;
                for (int j = 0; j < dataModels.size(); j++) {
                    DataModel dataModel = dataModels.get(j);
                    Double num = 0.0;
                    if (dataModel.getNum()!=null){
                        num=dataModel.getNum();
                    }else {
                        num=1.0;
                    }
                    result1 += num;
                    result2++;
                }
                SaveModel saveModel = one.getValue();
                saveModel.setResult1(result1);
                saveModel.setResult2(result2);
                if(dataModels.size()>0&& StringUtils.isNotEmpty(dataModels.get(0).getSlaveKey1())&&!StringUtils.isNotEmpty(saveModel.getSlaveKey1())){
                    saveModel.setSlaveKey1(dataModels.get(0).getSlaveKey1());
                }
                if(dataModels.size()>0&& StringUtils.isNotEmpty(dataModels.get(0).getSlaveKey2())&&!StringUtils.isNotEmpty(saveModel.getSlaveKey2())){
                    saveModel.setSlaveKey2(dataModels.get(0).getSlaveKey2());
                }
                returnList.add(saveModel);
            } else {
                returnList.add(one.getValue());
            }
        }
        return returnList;
    }

    private List<SaveModel> initAndSetResultCity(List<BaseDimensionQuota> dimensionQuotas, Map<String, List<DataModel>> countMap, String wlyyJobCongId, String endTime, String timeLevel) throws Exception {
        List<SaveModel> returnList = new ArrayList<>();
        Map<String, SaveModel> lastMaps = new HashMap<>();

        //初始化医生的数据
        SaveModel saveModel = new SaveModel();
        saveModel.setCity("330100");
        saveModel.setCityName("杭州市");
//        saveModel.setTown(one.getTownCode());
//        saveModel.setTownName(one.getTownName());
//        saveModel.setHospitalName(one.getName());
//        saveModel.setHospital(one.getCode());
        saveModel.setQuotaCode(wlyyJobCongId);
        saveModel.setCreateTime(new Date());
        saveModel.setTimeLevel(timeLevel);
        saveModel.setAreaLevel("2");//市
        saveModel.setQuotaDate(DateUtil.strToDate(endTime, "yyyy-MM-dd"));
        lastMaps.put("330100", saveModel);

        //根据维度初始化数据
        for (int i = 0; i < dimensionQuotas.size(); i++) {
            Map<String, SaveModel> mapTemp = new HashMap<>();
            BaseDimensionQuota dimensionQuota = dimensionQuotas.get(i);
            List<Map<String, Object>> listMap = jdbcTemplate.queryForList(dimensionQuota.getDictSql());
            for (Map.Entry<String, SaveModel> oneSaveModel : lastMaps.entrySet()) {
                for (int j = 0; j < listMap.size(); j++) {
                    SaveModel saveModelTemp = new SaveModel();
                    Map<String, Object> dictModel = listMap.get(j);

                    BeanUtils.copyProperties(oneSaveModel.getValue(), saveModelTemp);
                    SaveModel.class.getMethod("setSlaveKey" + (i + 1) + "Name", String.class).invoke(saveModelTemp, dictModel.get("name"));
                    SaveModel.class.getMethod("setSlaveKey" + (i + 1), String.class).invoke(saveModelTemp, dictModel.get("code"));
                    //根据维度生成新的key
                    StringBuffer strKey = new StringBuffer(oneSaveModel.getKey() + "-" + dictModel.get("code"));
                    mapTemp.put(strKey.toString(), saveModelTemp);
                }
            }
            //如果字典长度是0 抛出异常
            if (listMap.size() == 0) {
                throw new Exception("dict size:0,sql:" + dimensionQuota.getDictSql());
            }
            lastMaps = mapTemp;
        }

        //设置值
        for (Map.Entry<String, SaveModel> one : lastMaps.entrySet()) {
            if (countMap.containsKey(one.getKey())) {
                List<DataModel> dataModels = countMap.get(one.getKey());
                Double result1 = 0.0;
                Double result2 = 0.0;
                for (int j = 0; j < dataModels.size(); j++) {
                    DataModel dataModel = dataModels.get(j);
                    Double num = 0.0;
                    if (dataModel.getNum() != null) {
                        num = dataModel.getNum();
                    } else {
                        num = 1.0;
                    }
                    result1 += num;
                    result2++;
                }
                saveModel = one.getValue();
                saveModel.setResult1(result1);
                saveModel.setResult2(result2);
                if (dataModels.size() > 0 && StringUtils.isNotEmpty(dataModels.get(0).getSlaveKey1()) && !StringUtils.isNotEmpty(saveModel.getSlaveKey1())) {
                    saveModel.setSlaveKey1(dataModels.get(0).getSlaveKey1());
                }
                if (dataModels.size() > 0 && StringUtils.isNotEmpty(dataModels.get(0).getSlaveKey2()) && !StringUtils.isNotEmpty(saveModel.getSlaveKey2())) {
                    saveModel.setSlaveKey2(dataModels.get(0).getSlaveKey2());
                }
                returnList.add(saveModel);
            } else {
                returnList.add(one.getValue());
            }
        }
        return returnList;
    }
}
