package com.yihu.jw.hospital.async;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.door.SignFamily;
import com.yihu.jw.entity.specialist.rehabilitation.PatientMedicalRecordsDO;
import com.yihu.jw.entity.specialist.rehabilitation.RehabilitationPatientInfoDO;
import com.yihu.jw.hospital.module.followup.service.FollowUpService;
import com.yihu.jw.hospital.module.rehabilitation.service.RehabilitationManageService;
import com.yihu.jw.im.util.ImUtil;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.http.HttpClientUtil;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * 异步方法
 * Created by yeshijie on 2022/8/8.
 */
@Service
public class AsynService {

    private Logger logger = LoggerFactory.getLogger(AsynService.class);

    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private FollowUpService followUpService;
    @Autowired
    private BaseDoctorDao doctorDao;
    @Autowired
    private RehabilitationManageService rehabilitationManageService;
    @Autowired
    private ImUtil imUtill;
    //基卫服务地址
    @Value("${sign.check_upload}")
    private String jwUrl;
    @Autowired
    private HttpClientUtil httpClientUtil;

    /**
     * 创建随访计划job
     * @param jsonArray
     */
    @Async
    public void createFollowJob(JSONArray jsonArray, String planId, SignFamily signFamily){
        //创建过期job
        StringBuilder insertSql = new StringBuilder("INSERT into wlyy_followup_job(followup_id,expired_time,status,type)VALUES ");
        boolean bl = false;
        for (int i = 0; i < jsonArray.size(); i++) {
            com.alibaba.fastjson.JSONObject jsonObject = jsonArray.getJSONObject(i);
            String time = "";
            if("6".equals(jsonObject.get("hospitalServiceItemId").toString()) && "1".equals(jsonObject.get("followupDetailType")!=null?jsonObject.get("followupDetailType").toString():"0")){
                if(!bl){
                    bl = true;
                    time=DateUtil.dateToStrLong(DateUtil.getPreDays(DateUtil.strToDate(jsonObject.get("executeTime").toString()), 5));
                }else {
                    time=DateUtil.dateToStrLong(DateUtil.getPreDays(DateUtil.strToDate(jsonObject.get("executeTime").toString()), 7));
                }
                insertSql.append("('"+jsonObject.get("id")+"','"+time+"',1,0),");
            }
            //厦心自动创建新增社区随访表单
            if("6".equals(jsonObject.get("hospitalServiceItemId").toString())){
                //创建康复随访
                JSONObject data = new JSONObject();
                data.put("date",jsonObject.get("executeTime").toString());
                data.put("type","2");
                data.put("doctor",signFamily.getDoctor());
                data.put("followupClass","21");
                data.put("type1","1");
                data.put("id",jsonObject.get("id")+"");

                JSONObject dataJson = new JSONObject();
                //dataJson: {"rehabilitationDetailId":"808080eb827bb19601827bfb355b0039","patientCode":"6ce60381c5b14ea0884ead2af1c2063a","patientName":"黄金坤"
                // ,"doctorCode":"1b9fa6e86c5e468cbd0f2aabade30c35","doctorName":"王清秀","relationRecordType":5,"relationRecordCode":"","status":0}
                dataJson.put("rehabilitationDetailId",jsonObject.get("id")+"");
                dataJson.put("patientCode",signFamily.getPatient());
                dataJson.put("patientName",signFamily.getName());
                dataJson.put("doctorCode",signFamily.getDoctor());
                dataJson.put("doctorName",signFamily.getDoctorName());
                dataJson.put("relationRecordType",5);
                dataJson.put("relationRecordCode","");
                dataJson.put("status",0);
                try {
                    followUpService.addFollowupPlanNew(signFamily.getDoctor(),signFamily.getPatient(),data,dataJson.toString());
                }catch (Exception e){
                    e.printStackTrace();
                }
            }
        }
        if(bl){
            jdbcTemplate.execute(insertSql.substring(0,insertSql.length()-1)+";");
        }

        //创建提醒job
        //创建一条随访计划提醒
        String preDayStr = "";
        Date afterDate =DateUtil.strToDate(DateUtil.getStringDateShort()+" 10:00:00");
        if(afterDate.before(afterDate)){
            //早于上午十点
            preDayStr = DateUtil.dateToStrLong(afterDate);
        }else {
            //晚于上午十点
            preDayStr = DateUtil.dateToStrLong(DateUtil.getPreDays(afterDate,1));
        }
        String sql = "INSERT into wlyy_followup_job(followup_id,expired_time,status,type) VALUES ('"+planId+"','"+preDayStr+"',1,1)";
        jdbcTemplate.execute(sql);
    }

    /**
     * 发送消息
     */
    @Async
    public void sendMsg(String planId, SignFamily signFamily, BasePatientDO patient, BaseDoctorDO doctor, PatientMedicalRecordsDO patientMedicalRecordsDO1, RehabilitationPatientInfoDO re){
        try {
            org.json.JSONArray serverDoctorList = rehabilitationManageService.selectPlanServerDoctor(planId,signFamily.getDoctor());
            int length = serverDoctorList.length();
            BaseDoctorDO signDoctor = doctorDao.findById(signFamily.getDoctor()).orElse(null);
            for (int i=0;i<length;i++){
                JSONObject serverJson = serverDoctorList.getJSONObject(i);
                if (serverJson.getString("type").equals("4")){
                    String sessionId = serverJson.getString("sessionId");
                    JSONObject messageJson = new JSONObject();
                    String message= signDoctor.getName()+"医生您好，住院患者"+patient.getName()+"已出院，请您及时接收管理，保持联系，谢谢。";
                    messageJson.put("message",message);
                    messageJson.put("patientCode",patient.getId());
                    messageJson.put("patientName",patient.getName());
                    messageJson.put("medicalRecordDo", JSON.toJSONString(patientMedicalRecordsDO1));

                    ImUtil.sendImMsg(doctor.getId(),doctor.getName(),sessionId,"3101",messageJson.toString(),"1");
                }
            }
        } catch (RuntimeException se) {
            se.printStackTrace();
        }catch(Exception e){
            e.printStackTrace();
        }

        //厦心下转的居民给厦心推送居民签约信息
        if("12350200568420163H".equals(re.getHospitalCode())){
            xxSendPatientBaseInfo(re.getIdcard());
        }
    }

    /**
     * i健康推送患者信息给厦心集成平台
     * @param idCard
     */
    public void xxSendPatientBaseInfo(String idCard){
        try {
            String url = jwUrl +"/third/emr/xxSendPatientBaseInfo?idCard="+idCard;
            String re = httpClientUtil.get(url,"UTF-8");
            logger.info(idCard+":xxSendPatientBaseInfo:"+re);
        }catch (Exception e){
            e.printStackTrace();
        }
    }
}
