package com.yihu.jw.hospital.module.district.controller;

import com.yihu.jw.hospital.module.district.service.DistrictService;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;

/**
 * 省市区三级地址控制类
 *
 * @author George
 */
@Controller
@RequestMapping(value = "common", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
public class DistrictController extends EnvelopRestEndpoint {

    @Autowired
    private DistrictService districtService;
    @Autowired
    HttpServletRequest request;


    /**
     * 获取人群分类
     */
    @RequestMapping(value = "getProfessionalDict", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation("职业状态字典")
    public String getProfessionalDict() {
        try {
            // 获取医生下的患者
            return write(200, "获取成功！", "data", districtService.getProfessionalDict());
        } catch (Exception e) {
            return error(-1, "获取字典信息失败！");
        }
    }

    @RequestMapping(value = "getNationDict",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation("获取民族信息列表")
    public String getNationDict(){
        try {
            // 获取医生下的患者
            return write(200, "获取成功！", "data",districtService.getNationDict());
        } catch (Exception e) {
            return error(-1, "获取字典信息失败！");
        }
    }


    /**
     * 省市一二三级查询接口
     *
     * @param type 1一级目录，2二级目录，3三级目录，4街道目录
     * @param code 省或市标识
     * @return
     */
    @RequestMapping(value = "district")
    @ResponseBody
    public String district(int type, String code) {
        try {
            List<?> list = districtService.findByType(type, code);
            return write(200, "查询成功！", "list", list);
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败！");
        }
    }

    /**
     * 省市一二三级查询接口(包含权限控制)
     *
     * @param type 1一级目录，2二级目录，3三级目录，4街道目录
     * @param code 省或市标识
     * @return
     */
    @RequestMapping(value = "districtAuthority")
    @ResponseBody
    public String districtAuthority(int type, String code) {
        try {
            List<Map<String, String>> roleMap = (List<Map<String, String>>) request.getSession().getAttribute("roleMap");
            List<?> list = districtService.findByType(type, code, roleMap);
            return write(200, "查询成功！", "list", list);
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败！");
        }
    }


}
