package com.yihu.jw.hospital.module.district.service;


import com.yihu.jw.area.dao.BaseCityDao;
import com.yihu.jw.area.dao.BaseProvinceDao;
import com.yihu.jw.area.dao.BaseStreetDao;
import com.yihu.jw.area.dao.BaseTownDao;
import com.yihu.jw.entity.base.area.BaseCityDO;
import com.yihu.jw.entity.base.area.BaseProvinceDO;
import com.yihu.jw.entity.base.area.BaseStreetDO;
import com.yihu.jw.entity.base.area.BaseTownDO;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * 省市区三级业务处理类
 *
 * @author George
 */
@Service
public class DistrictService extends EnvelopRestEndpoint {

    @Autowired
    private BaseProvinceDao provinceDao;
    @Autowired
    private BaseCityDao cityDao;
    @Autowired
    private BaseTownDao townDao;
    @Autowired
    private BaseStreetDao streetDao;
    @Autowired
    JdbcTemplate jdbcTemplate;


    /**
     * 查询省市区三级目录
     *
     * @param type 1一级目录，2二级目录，3三级目录，4街道目录
     * @param code 目录标识
     * @return
     */
    public List<?> findByType(int type, String code) {
        switch (type) {
            case 1:
                return findProvince();
            case 2:
                return findCity(code);
            case 3:
                return findTown(code);
            case 4:
                return findStreet(code);
        }
        return null;
    }

    /**
     * 查询省市区三级目录
     *
     * @param type 1一级目录，2二级目录，3三级目录，4街道目录
     * @param code 目录标识
     * @return
     */
    public List<?> findByType(int type, String code, List<Map<String, String>> roleMap) {
        switch (type) {
            case 1:
                return findProvince();
            case 2:
                return findCity(code);
            case 3:
                return findTown(code, roleMap);
            case 4:
                return findStreet(code);
        }
        return null;
    }


    /**
     * 查询所有的省份信息
     *
     * @return
     */
    public List<BaseProvinceDO> findProvince() {
        List<BaseProvinceDO> list = new ArrayList<BaseProvinceDO>();
        Iterable<BaseProvinceDO> iterable = provinceDao.findAll(Sort.by(Direction.ASC, "id"));
        if (iterable != null) {
            Iterator<BaseProvinceDO> it = iterable.iterator();
            while (it != null && it.hasNext()) {
                list.add(it.next());
            }
        }
        return list;
    }

    /**
     * 查询省份下的城市信息
     *
     * @param province 省编码
     * @return
     */
    public List<BaseCityDO> findCity(String province) {
        List<BaseCityDO> list = new ArrayList<BaseCityDO>();
        Iterable<BaseCityDO> iterable = cityDao.findByProvince(province);
        if (iterable != null) {
            Iterator<BaseCityDO> it = iterable.iterator();
            while (it != null && it.hasNext()) {
                list.add(it.next());
            }
        }
        return list;
    }

    /**
     * 查询城市下的区县信息
     *
     * @param city 城市编码
     * @return
     */
    public List<BaseTownDO> findTown(String city) {
        List<BaseTownDO> list = new ArrayList<BaseTownDO>();
        Iterable<BaseTownDO> iterable = townDao.findByCity(city);
        if (iterable != null) {
            Iterator<BaseTownDO> it = iterable.iterator();
            while (it != null && it.hasNext()) {
                list.add(it.next());
            }
        }
        return list;
    }


    /**
     * 查询城市下的区县信息(包含权限控制)
     *
     * @param city 城市编码
     * @return
     */
    public List<BaseTownDO> findTown(String city, List<Map<String, String>> roleMap) {
        List<BaseTownDO> list = new ArrayList<BaseTownDO>();
        Iterable<BaseTownDO> iterable = townDao.findByCity(city);
        if (iterable != null) {
            String areaString = "";
            Boolean cityFlag = false;
            for (Map<String, String> map : roleMap) {
                String code = map.get("code");
                if ("350200".equals(code)) {
                    cityFlag = true;
                    break;
                } else if (code.length() == 6) {
                    areaString += map.get("areas") + ",";
                }
            }

            Iterator<BaseTownDO> it = iterable.iterator();
            while (it != null && it.hasNext()) {
                BaseTownDO town = it.next();
                String townCode = town.getCode();
                if (cityFlag) {
                    list.add(town);
                } else {
                    if (areaString.indexOf(townCode) >= 0) {
                        list.add(town);
                    }
                }
            }
        }
        return list;
    }


    /**
     * 查询城市下的街道信息
     *
     * @param town 区县编码
     * @return
     */
    public List<BaseStreetDO> findStreet(String town) {
        List<BaseStreetDO> list = new ArrayList<BaseStreetDO>();
        Iterable<BaseStreetDO> iterable = streetDao.findByTown(town);
        if (iterable != null) {
            Iterator<BaseStreetDO> it = iterable.iterator();
            while (it != null && it.hasNext()) {
                list.add(it.next());
            }
        }
        return list;
    }


//    /**
//     * 查询城市下的街道信息
//     *
//     * @param street 街道编码
//     * @return
//     */
//    public List<Country> findCountry(String street) {
//        List<Country> list = new ArrayList<Country>();
//        Iterable<Country> iterable = countryDao.findByStreet(street);
//        if (iterable != null) {
//            Iterator<Country> it = iterable.iterator();
//            while (it != null && it.hasNext()) {
//                list.add(it.next());
//            }
//        }
//        return list;
//    }

    public BaseProvinceDO findProvinceByName(String name) {
        return provinceDao.findByName(name);
    }

    public BaseCityDO findCityByName(String name) {
        return cityDao.findByName(name);
    }

    public BaseTownDO finTownByName(String name) {
        return townDao.findByName(name);
    }

    public BaseTownDO finTownByNameAndCity(String name, String city) {
        return townDao.findByNameAndCity(name, city);
    }

    public BaseStreetDO findStreetByName(String name) {
        return streetDao.findByName(name);
    }


    public List<Map<String, Object>> getProfessionalDict() {
        String sql = "SELECT t.dict_code 'code',t.dict_value `value` FROM `base`.`wlyy_hospital_sys_dict`  t WHERE t.dict_name='PROFESSIONAL_STATE'";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        return list;
    }

    public List<Map<String, Object>> getNationDict() {
        String sql = "SELECT t.NATION_CODE,t.NATION_NAME FROM zy_nation_dict t ";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        return list;
    }
}
