package com.yihu.jw.hospital.module.door.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.entity.door.WlyyDoorConclusionDO;
import com.yihu.jw.entity.door.WlyyDoorServiceOrderDO;
import com.yihu.jw.hospital.module.door.dao.WlyyDoorDoctorDao;
import com.yihu.jw.hospital.module.door.dao.WlyyDoorServiceOrderDao;
import com.yihu.jw.hospital.module.door.service.DoorOrderService;
import com.yihu.jw.hospital.module.door.service.WlyyDoorPrescriptionService;
import com.yihu.jw.hospital.module.door.service.WlyyDoorServiceOrderService;
import com.yihu.jw.restmodel.ResponseContant;
import com.yihu.jw.restmodel.qvo.ParamQvo;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.poifs.filesystem.DirectoryEntry;
import org.apache.poi.poifs.filesystem.DocumentEntry;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Created by wxw on 2019/3/15.
 *
 * @author wxw.
 */
@RestController
@RequestMapping(value = "/doctor/serviceOrder", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(value = "医生端-上门服务")
public class DoorOrderController extends EnvelopRestEndpoint {

    @Autowired
    private DoorOrderService doorOrderService;

    @Autowired
    private WlyyDoorDoctorDao doorDoctorDao;
    @Autowired
    private WlyyDoorServiceOrderDao doorServiceOrderDao;
    @Autowired
    private WlyyDoorServiceOrderService wlyyDoorServiceOrderService;
    @Autowired
    private WlyyDoorPrescriptionService doorPrescriptionService;
    @Autowired
    JdbcTemplate jdbcTemplate;

//    @Autowired
//    private ZyDictService zyDictService;
//    @Autowired
//    private JwDoorPrescriptionService jwDoorPrescriptionService;
//    @Autowired
//    private FamilyContractService familyContractService;
//    @Autowired
//    private ServerPackageService serverPackageService;
//    @Autowired
//    private ServiceItemService serviceItemService;
//    @Autowired
//    private HospitalService hospitalService;


    /**
     * 获取登录医生的信息
     */
    @GetMapping(value = "getDoctorInfo")
    @ApiOperation(value = "获取登录医生的信息")
    public Envelop getDoctorInfo(
            @ApiParam(name = "jsonStr", value = "jsonStr", required = true) @RequestParam String jsonStr
    ) {
        try {
            jsonStr = URLDecoder.decode(jsonStr, "UTF-8");
            ParamQvo qvo = JSON.parseObject(jsonStr, ParamQvo.class);
            System.out.println("参数:" + JSON.toJSONString(qvo));

            Map<String, Object> result = wlyyDoorServiceOrderService.getDoctorInfo(qvo);
            return success("查询成功！", result);
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }


    @GetMapping(value = "queryDoctorList")
    @ApiOperation(value = "服务人员列表（医生列表）")
    public Envelop queryDoctorList(
            @ApiParam(name = "town", value = "调度员所在的机构code") @RequestParam(value = "town", required = true) String town,
            @ApiParam(name = "doctorName", value = "医生姓名") @RequestParam(value = "doctorName", required = false) String doctorName,
            @ApiParam(name = "status", value = "医生接单状态，状态为全部时不传") @RequestParam(value = "status", required = false) String status,
            @ApiParam(name = "patient", value = "居民code") @RequestParam(value = "patient", required = false) String patient,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1") @RequestParam(value = "page") int page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15") @RequestParam(value = "size") int size) {
        try {
            JSONObject result = wlyyDoorServiceOrderService.queryDoctorList(patient, town, doctorName, status, page, size);
            if (result.getIntValue(ResponseContant.resultFlag) == ResponseContant.fail) {
                return failed(result.getString(ResponseContant.resultMsg));
            }
            int count = result.getIntValue(ResponseContant.count);
            JSONObject object = new JSONObject();
            object.put("total", count);
            if (count > 0) {
                object.put("detailModelList", result.get(ResponseContant.resultMsg));
            } else {
                List list = new ArrayList();
                object.put("detailModelList", list);
            }
            object.put("currPage", page);
            object.put("pageSize", size);
            return success("查询成功", object);
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }


    @GetMapping(value = "queryBriefList")
    @ApiOperation(value = "调度员查询工单列表")
    public Envelop queryBriefList(
            @ApiParam(name = "dispatcher", value = "调度员code") @RequestParam(value = "dispatcher", required = true) String dispatcher,
            @ApiParam(name = "hospital", value = "调度员所在机构code") @RequestParam(value = "hospital", required = true) String hospital,
            @ApiParam(name = "orderNumber", value = "工单号") @RequestParam(value = "orderNumber", required = false) String orderNumber,
            @ApiParam(name = "patientName", value = "工单服务对象姓名") @RequestParam(value = "patientName", required = false) String patientName,
            @ApiParam(name = "phone", value = "发起工单的居民的联系方式") @RequestParam(value = "phone", required = false) String phone,
            @ApiParam(name = "status", value = "工单状态") @RequestParam(value = "status", required = false) Integer status,
            @ApiParam(name = "patientType", value = "居民类型") @RequestParam(value = "patientType", required = false) String patientType,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1") @RequestParam(value = "page") int page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15") @RequestParam(value = "size") int size) {
        try {
            JSONObject result = wlyyDoorServiceOrderService.queryBriefList(dispatcher, hospital, orderNumber, patientName, phone, status, patientType, page, size);

            int count = result.getIntValue(ResponseContant.count);
            JSONObject object = new JSONObject();
            object.put("total", count);
            object.put("detailModelList", result.get(ResponseContant.resultMsg));
            object.put("currPage", page);
            object.put("pageSize", size);
            return success("查询成功", object);
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }


    @PostMapping(value = "proxyCreate")
    @ApiOperation(value = "创建上门服务咨询--医生代预约")
    public Envelop create(
            @ApiParam(name = "jsonData", value = "Json数据", required = true) @RequestParam String jsonData,
            @ApiParam(name = "doctorId", value = "doctorId", required = true) @RequestParam String doctorId
    ) {
        try {
            JSONObject result = wlyyDoorServiceOrderService.proxyCreate(jsonData, doctorId);
            if (result.getIntValue(ResponseContant.resultFlag) == ResponseContant.fail) {
                return failed(result.getString(ResponseContant.resultMsg));
            }
            return success("提交成功！", result);
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }


    /**
     * 调度平台修改预约信息
     * 参数：{"orderId":"4028f8f68b216198018b22bcfcbb018f","patientName":"黄小蕾","patientCode":"0fab4dd67e074e16ac86db6b6c15233e","number":"26669809","serveAddress":"明丰财富中心","serveTown":"思明区","phone":"15200000001","serviceStatus":"1","patientExpectedServeTime":"2023-10-12 15:00-15:30","sex":"男","age":43,"doctor":null,"status":1,"expectedDoctor":null,"serveDesc":"工程师测试1"}
     */
    @PostMapping(value = "updateOrderCardInfo")
    @ApiOperation(value = "更新预约简要信息")
    public Envelop updateOrderCardInfo(
            @ApiParam(name = "jsonData", value = "json数据") @RequestParam(value = "jsonData", required = true) String jsonData) {
        try {
            JSONObject result = wlyyDoorServiceOrderService.updateOrderCardInfo(jsonData);
            if (result.getIntValue(ResponseContant.resultFlag) == ResponseContant.fail) {
                return failed(result.getString(ResponseContant.resultMsg));
            }
            return success("保存成功", result.get(ResponseContant.resultMsg));
        } catch (Exception e) {
            return failed(e.getMessage());
        }

    }


    @GetMapping("/getDispatchOrderSwitch")
    @ApiOperation(value = "获取医生分派订单开关状态")
    public Envelop getDispatchOrderSwitch(
            @ApiParam(name = "doctor", value = "医生doctor") @RequestParam(value = "doctor", required = true) String doctor
    ) {
        try {
            String status = doorOrderService.findDispatchStatusByDoctor(doctor);
            return success("成功", status);
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    @PostMapping("/dispatchOrderSwitch")
    @ApiOperation(value = "分派订单开关修改")
    public Envelop dispatchOrderSwitch(
            @ApiParam(name = "doctor", value = "医生code") @RequestParam(value = "doctor", required = true) String doctor,
            @ApiParam(value = "开关值，5关闭 1开启", name = "value") @RequestParam(value = "value", required = true) Integer value
    ) {
        try {
            doorOrderService.updateDispatchStatusByDoctor(doctor, value);
            return success("成功");
        } catch (Exception e) {
            return failed(e.getMessage());
        }

    }

    @GetMapping("/topStatusBarNumTeam")
    @ApiOperation(value = "顶部状态栏订单分类tab(团队长)")
    public Envelop topStatusBarNumTeam(
            @ApiParam(name = "doctor", value = "医生code") @RequestParam(value = "doctor", required = true) String doctor,
            @ApiParam(value = "发起类型（1本人发起 2家人待预约 3医生代预约）", name = "type") @RequestParam(value = "type", required = false) Integer type
    ) {
        try {
            Map<String, String> map = doorOrderService.getNumGroupByStatusTeam(doctor, type);
            return success("获取成功", map);
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }


    @GetMapping("/getDoorOrderList")
    @ApiOperation(value = "服务工单列表")
    public Envelop getDoorOrderList(
            @ApiParam(value = "工单id", name = "orderId", required = false) @RequestParam(value = "orderId", required = false) String orderId,
            @ApiParam(value = "居民姓名", name = "patientName", required = false) @RequestParam(value = "patientName", required = false) String patientName,
            @ApiParam(name = "name", value = "姓名/身份证号", required = false) @RequestParam(value = "name", required = false) String name,
            @ApiParam(value = "联系方式", name = "patientPhone", required = false) @RequestParam(value = "patientPhone", required = false) String patientPhone,
            @ApiParam(value = "服务机构", name = "hospitalCode", required = false) @RequestParam(value = "hospitalCode", required = false) String hospitalCode,
            @ApiParam(value = "工单状态", name = "status", required = false) @RequestParam(value = "status", required = false) Integer[] status,
            //1.6.8.6版本，创建时间修改为服务时间
            @ApiParam(value = "创建时间开始，格式（yyyy-MM-dd mm:dd:ss）", name = "createTimeStart", required = false) @RequestParam(value = "createTimeStart", required = false) String createTimeStart,
            @ApiParam(value = "创建时间结束，格式（yyyy-MM-dd mm:dd:ss）", name = "createTimeEnd", required = false) @RequestParam(value = "createTimeEnd", required = false) String createTimeEnd,
            @ApiParam(value = "服务医生的名称", name = "serverDoctorName", required = false)
            @RequestParam(value = "serverDoctorName", required = false) String serverDoctorName, @ApiParam(value = "医生的code", name = "doctorCode", required = false)
            @RequestParam(value = "doctorCode", required = false) String doctorCode,
            @ApiParam(value = "角色0、医生，1、管理员", name = "isManage", required = false) @RequestParam(value = "isManage", required = false) String isManage,
            @ApiParam(value = "0-不需要，1-需要，待补录；2-需要，已补录", name = "examPaperStatus", required = false) @RequestParam(value = "examPaperStatus", required = false) Integer examPaperStatus,
            @ApiParam(value = "发起类型（1本人发起 2家人待预约 3医生代预约）", name = "type") @RequestParam(value = "type", required = false) Integer type,
            @ApiParam(value = "服务类型类型 ", name = "serverType") @RequestParam(value = "serverType", required = false) String serverType,
            @ApiParam(value = "页码", name = "page", defaultValue = "1", required = true) @RequestParam(value = "page", required = true) Integer page,
            @ApiParam(value = "每页数目", name = "pageSize", defaultValue = "10", required = true) @RequestParam(value = "pageSize", required = true) Integer pageSize
    ) {
        try {
            if (StringUtils.isEmpty(isManage)) {
                isManage = getCurrentRoleIsManange(doctorCode);
            }
            if ("0".equals(isManage)) {
                if (StringUtils.isEmpty(doctorCode)) {
                    doctorCode = getUID();
                }
            } else if ("1".equals(isManage) && StringUtils.isBlank(hospitalCode)) {
                //如果是管理员并且未筛选机构，就默认展示其管理下所有机构
                //先注释
//                String level = getCurrentRoleLevel();
//                String currentRoleCode = getCurrentRoleCode();
//                if (level.equals("2")) {
//                    //市管理员
//                    hospitalCode = currentRoleCode.substring(0, currentRoleCode.length() - 2) + "%";
//                } else if (level.equals("3")) {
//                    //区管理员
//                    hospitalCode = currentRoleCode + "%";
//                } else if (level.equals("4")) {
//                    //机构管理员
//                    hospitalCode = currentRoleCode;
//                }
            } else if (StringUtils.isNotBlank(hospitalCode) && hospitalCode.length() < 10) {
                hospitalCode += "%";
            }
            StringBuffer ss = new StringBuffer();
            if (status != null && status.length > 0) {
                int statusLength = status.length;
                for (int i = 0; i < statusLength; i++) {
                    ss.append(status[i]);
                    if (i < statusLength - 1) {
                        ss.append(",");
                    }
                }
            } else {
                ss = null;
            }
            JSONObject result = doorOrderService.getDoorOrderList(
                    orderId, patientName, patientPhone, hospitalCode,
                    ss, createTimeStart, createTimeEnd, serverDoctorName,
                    doctorCode, examPaperStatus, page, pageSize, type, name, serverType);
            return success("获取成功", result);
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }


    /**
     * 参数
     * orderId: 4028f8f68b216198018b22bcfcbb018f
     * dispatcher: 361aef4891de492cb0e6a47cf9fe31f3
     * dispatcherName: 黄琴
     */
    @PostMapping(value = "saveLastDispatcher")
    @ApiOperation(value = "保存最后一个回复内容的调度员到工单")
    public Envelop saveLastDispatcher(
            @ApiParam(name = "orderId", value = "工单id", required = true) @RequestParam String orderId,
            @ApiParam(name = "dispatcher", value = "调度员code", required = true) @RequestParam String dispatcher,
            @ApiParam(name = "dispatcherName", value = "调度员姓名", required = true) @RequestParam String dispatcherName
    ) {
        try {
            JSONObject result = wlyyDoorServiceOrderService.saveLastDispatcher(orderId, dispatcher, dispatcherName);
            if (result.getIntValue(ResponseContant.resultFlag) == ResponseContant.fail) {
                return failed(result.getString(ResponseContant.resultMsg));
            }
            return success("保存成功", result.get(ResponseContant.resultMsg));
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }


    /**
     * 参数：
     * orderId: 4028f8f68b216198018b22bcfcbb018f
     * remark: 111111
     * dispatcher: 361aef4891de492cb0e6a47cf9fe31f3
     * dispathcherName: 黄琴
     * doctor: 215d03f7a6b84361851f6af8a6cd9121
     * doctorName: 周丹琪
     * doctorJobName: 医师
     */
    @PostMapping(value = "sendOrderToDoctor")
    @ApiOperation(value = "调度员给医生派单")
    public Envelop sendOrderToDoctor(
            @ApiParam(name = "orderId", value = "工单id") @RequestParam(value = "orderId", required = true) String orderId,
            @ApiParam(name = "remark", value = "调度员备注") @RequestParam(value = "remark", required = false) String remark,
            @ApiParam(name = "dispatcher", value = "调度员code") @RequestParam(value = "dispatcher", required = true) String dispatcher,
            @ApiParam(name = "dispathcherName", value = "调度员姓名") @RequestParam(value = "dispathcherName", required = true) String dispathcherName,
            @ApiParam(name = "doctor", value = "医生code") @RequestParam(value = "doctor", required = true) String doctor,
            @ApiParam(name = "doctorName", value = "医生姓名") @RequestParam(value = "doctorName", required = true) String doctorName,
            @ApiParam(name = "doctorJobName", value = "医生职称") @RequestParam(value = "doctorJobName", required = true) String doctorJobName) {
        try {
            JSONObject result = wlyyDoorServiceOrderService.sendOrderToDoctor(orderId, remark, dispatcher, dispathcherName, doctor, doctorName, doctorJobName);
            if (result.getIntValue(ResponseContant.resultFlag) == ResponseContant.fail) {
                return failed(result.getString(ResponseContant.resultMsg));
            }
            return success("派单成功", result.get(ResponseContant.resultMsg));
        } catch (Exception e) {
            return failed(e.getMessage());
        }

    }


    @GetMapping("/topStatusBarNum")
    @ApiOperation(value = "顶部状态栏订单分类tab")
    public Envelop topStatusBarNum(
            @ApiParam(name = "doctor", value = "医生code") @RequestParam(value = "doctor", required = true) String doctor,
            @ApiParam(name = "name", value = "姓名/身份证号", required = false) @RequestParam(value = "name", required = false) String name,
            @ApiParam(value = "发起类型（1本人发起 2家人待预约 3医生代预约）", name = "type") @RequestParam(value = "type", required = false) Integer type
    ) {
        try {
            Map<String, String> map = doorOrderService.getNumGroupByStatus(doctor, type, name);
            return success("获取成功", map);
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    /**
     * 参数：
     * orderId: 4028f8f68b216198018b22bcfcbb018f
     * doctor: 215d03f7a6b84361851f6af8a6cd9121
     * level: 3
     */
    @GetMapping("getByOrderId")
    @ApiOperation(value = "根据工单id获取相应的工单，如果为空，则获取该医生当前最新一条的工单")
    public Envelop getByOrderId(
            @ApiParam(value = "医生code", name = "doctor") @RequestParam(value = "doctor", required = true) String doctor,
            @ApiParam(value = "工单id", name = "orderId") @RequestParam(value = "orderId", required = false) String orderId,
            @ApiParam(name = "level", value = "医院等级，1.三级医院，2.二级医院，3.一级及以下") @RequestParam(value = "level", required = true) Integer level
    ) {
        try {
            // 没有提供工单id的情况下，则获取该医生当前最新一条的工单
            if (StringUtils.isEmpty(orderId)) {
                // 根据接单医生code获取最近一次服务orderId
                orderId = doorOrderService.getOrderIdByDoctor(doctor);
                if (StringUtils.isEmpty(orderId)) {
                    return failed("获取失败, 该医生暂无工单");
                }
            }

            // 根据orderId获取工单信息
            WlyyDoorServiceOrderDO doorServiceOrderDO = doorOrderService.getDoorServiceOrderById(orderId, level);
            return success("获取成功", doorServiceOrderDO);
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    /**
     * 参数：
     * orderId: 4028f8f68b216198018b22bcfcbb018f
     * jobCode: 95b0d29e5f0a11e68344fa163e8aee56
     * jobCodeName: 医师
     * hospitalLevel: 3
     */
    @PostMapping("acceptOrder")
    @ApiOperation(value = "接单")
    public Envelop acceptOrder(
            @ApiParam(value = "工单id", name = "orderId", required = true) @RequestParam(value = "orderId", required = true) String orderId,
            @ApiParam(value = "医生职称code", name = "jobCode", required = true) @RequestParam(value = "jobCode", required = true) String jobCode,
            @ApiParam(value = "医生职称", name = "jobCodeName", required = true) @RequestParam(value = "jobCodeName", required = true) String jobCodeName,
            @ApiParam(value = "医院级别", name = "hospitalLevel", required = true) @RequestParam(value = "hospitalLevel", required = true) int hospitalLevel
    ) {
        try {
            doorOrderService.acceptOrder(orderId, jobCode, jobCodeName, hospitalLevel);
            return success("操作成功");
        } catch (RuntimeException se) {
            return failed(se.getMessage());
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }


    @PostMapping(value = "transferOrder")
    @ApiOperation(value = "医生转派单")
    public Envelop transferOrder(
            @ApiParam(name = "orderId", value = "工单id") @RequestParam(value = "orderId", required = true) String orderId,
            @ApiParam(name = "remark", value = "当前医生备注") @RequestParam(value = "remark", required = false) String remark,
            @ApiParam(name = "dispatcher", value = "当前医生code") @RequestParam(value = "dispatcher", required = true) String dispatcher,
            @ApiParam(name = "dispathcherName", value = "当前医生姓名") @RequestParam(value = "dispathcherName", required = true) String dispathcherName,
            @ApiParam(name = "doctor", value = "医生code") @RequestParam(value = "doctor", required = true) String doctor,
            @ApiParam(name = "doctorName", value = "医生姓名") @RequestParam(value = "doctorName", required = true) String doctorName,
            @ApiParam(name = "doctorJobName", value = "医生职称") @RequestParam(value = "doctorJobName", required = true) String doctorJobName
    ) {
        try {
            JSONObject result = wlyyDoorServiceOrderService.transferOrder(orderId, remark, dispatcher, dispathcherName, doctor, doctorName, doctorJobName);
            if (result.getIntValue(ResponseContant.resultFlag) == ResponseContant.fail) {
                return failed(result.getString(ResponseContant.resultMsg));
            }
            return success("派单成功", result.get(ResponseContant.resultMsg));
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    @PostMapping("refuse")
    @ApiOperation(value = "拒单")
    public Envelop refuseOrder(
            @ApiParam(value = "工单id", name = "orderId", required = true) @RequestParam(value = "orderId", required = true) String orderId,
            @ApiParam(value = "拒绝原因", name = "reason", required = false) @RequestParam(value = "reason", required = false) String reason
    ) {
        try {
            doorOrderService.refuseOrder(getUID(), orderId, reason);
            return success("操作成功");
        } catch (Exception e) {
            return failed(e.getMessage());
        }

    }

    @PostMapping("updateArrivingTime")
    @ApiOperation(value = "修改预计到达时间")
    public Envelop updateArrivingTime(
            @ApiParam(value = "工单id", name = "orderId") @RequestParam(value = "orderId", required = true) String orderId,
            @ApiParam(value = "医生预计到达时间", name = "arrivingTime") @RequestParam(value = "arrivingTime", required = true) String arrivingTime
    ) {
        try {
            WlyyDoorServiceOrderDO wlyyDoorServiceOrderDO = doorOrderService.updateArrivingTime(orderId, arrivingTime);
            return success("修改成功", wlyyDoorServiceOrderDO);
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    @RequestMapping(value = "/urlAnalysis", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ApiOperation("门牌解析上门地址")
    @ResponseBody
    public Envelop urlAnalysis(@ApiParam(name = "url", value = "地址解析", defaultValue = "")
                               @RequestParam(value = "url", required = true) String url) throws Exception {
        try {
            return success("操作成功！", doorOrderService.urlAnalysis(url));
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    @PostMapping("signIn")
    @ApiOperation(value = "上门服务签到")
    public Envelop signIn(
            @ApiParam(value = "工单id", name = "orderId") @RequestParam(value = "orderId", required = true) String orderId,
            @ApiParam(value = "签到时间", name = "signTime") @RequestParam(value = "signTime", required = false) String signTime,
            @ApiParam(value = "签到方式：1-定位，2-扫码，3-拍照，4-二维码", name = "signWay") @RequestParam(value = "signWay", required = false) Integer signWay,
            @ApiParam(value = "签到地址", name = "signLocation") @RequestParam(value = "signLocation", required = false) String signLocation,
            @ApiParam(value = "签到照片", name = "signImg") @RequestParam(value = "signImg", required = false) String signImg,
            @ApiParam(value = "二维码内容", name = "twoDimensionalCode") @RequestParam(value = "twoDimensionalCode", required = false) String twoDimensionalCode
    ) {
        try {
            WlyyDoorServiceOrderDO wlyyDoorServiceOrderDO = doorOrderService.signIn(orderId, signTime, signWay, signLocation, signImg, twoDimensionalCode);
            if (wlyyDoorServiceOrderDO != null) {
                return success("操作成功", wlyyDoorServiceOrderDO);
            } else {
                return failed("扫码签到失败");
            }
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    /**
     * 参数
     * doctor: 361aef4891de492cb0e6a47cf9fe31f3
     * orderId: 4028f8e07692c6e50176934c1fd70000
     */
    @GetMapping("getDoorConclusion")
    @ApiOperation("获取服务工单小结")
    public Envelop getDoorConclusion(
            @ApiParam(value = "医生code", name = "doctor") @RequestParam(value = "doctor", required = false) String doctor,
            @ApiParam(value = "工单id", name = "orderId") @RequestParam(value = "orderId", required = false) String orderId
    ) {
        try {
            // 没有提供工单id的情况下，默认代入上一次服务信息记录
            if (StringUtils.isEmpty(orderId)) {
                orderId = doorOrderService.getOrderIdByDoctor(doctor);
                if (StringUtils.isEmpty(orderId)) {
                    return failed("获取失败，该医生暂无工单");
                }
            }
            // 根据orderId查询工单小结表
            WlyyDoorConclusionDO doorConclusion = doorOrderService.getDoorConclusion(orderId, true);
            return success("获取成功", doorConclusion);
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    /**
     * 参数：
     * examPapeStatus:
     * model: {"id":null,"createTime":null,"createUser":null,"createUserName":null,"updateTime":null,"updateUser":null,"updateUserName":null,"orderId":"4028f8e07692c6e50176934c1fd70000","patient":"0fab4dd67e074e16ac86db6b6c15233e","patientName":"黄小蕾","idcard":"650101199912271633","sex":"男","age":19,"marriage":"未婚","profession":"国家公务员","nationa":"汉族","birthPlace":"北京市市辖区","company":"测试","address":"厦门市思明区软件园二期望海路55号","patientPhone":"15200000001","proxyPatientName":"黄小蕾","proxyPatientPhone":"15200000001","proxyPatientRelation":"兄弟","beforeRecord":"去去去","presenIllness":"拒绝斤斤计较","familyIllHistory":"无","physicalRecord":"正常","assistantExamRecord":"正常","diagnosticRecord":"111","examResult":"正常","imgs":"","serviceCount":32,"serviceTime":"2020-12-24","patientLastConclusion":null,"presenIllness2":"头痛,咳痰,体重下降,手脚麻木","familyIllHistory2":"职业病,糖尿病","temperature":"36.5","respiratoryRate":"63","pulse":"66","heartRate":"66","systolic":"111","diastolic":"92","bloodSugar":"8","bloodSugarType":"2","template":null}
     */
    @PostMapping("updateDoorConclusion")
    @ApiOperation("编辑保存服务工单小结")
    public Envelop updateDoorConclusion(
            @RequestParam String model,
            @ApiParam(value = "补录状态", name = "examPapeStatus") @RequestParam(value = "examPapeStatus", required = false) Integer examPapeStatus
    ) {
        try {
            WlyyDoorConclusionDO doorConclusion = doorOrderService.updateDoorConclusion(model, examPapeStatus);
            return success("保存成功", doorConclusion);
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    @RequestMapping(value = "deleteById", method = RequestMethod.POST)
    @ApiOperation("删除上门服务记录")
    public Envelop deleteById(
            @RequestParam(value = "id") @ApiParam(value = "上门服务id") String id,
            @RequestParam(value = "doctorId") @ApiParam(value = "医生id") String doctorId
    ) {
        try {
            String isManage = getCurrentRoleIsManange(doctorId);
            if (StringUtils.isNotBlank(isManage) && "1".equals(isManage)) {
                doorOrderService.deleteById(id);
            } else {
                return failed("仅管理员可操作！");
            }
            return success("操作成功");
        } catch (Exception ex) {
            error(ex);
            return failed("操作失败！");
        }
    }

    @PostMapping("savePatientSignOrCertificate")
    @ApiOperation(value = "居民确认记录")
    public Envelop savePatientSignOrCertificate(
            @ApiParam(value = "工单id", name = "orderId") @RequestParam(value = "orderId", required = true) String orderId,
            @ApiParam(value = "居民确认结束服务方式", name = "finishWay") @RequestParam(value = "finishWay", required = false) Integer finishWay,
            @ApiParam(value = "居民确认结束服务照片", name = "finishImg") @RequestParam(value = "finishImg", required = false) String finishImg,
            @ApiParam(name = "level", value = "医院等级，1.三级医院，2.二级医院，3.一级及以下") @RequestParam(value = "level", required = true) Integer level
    ) {
        try {
            WlyyDoorServiceOrderDO orderDO = doorOrderService.savePatientSignOrCertificate(orderId, finishWay, finishImg, level);
            return success("获取成功", orderDO);
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    @PostMapping("saveExamPaperInfo")
    @ApiOperation(value = "医生补录报告")
    public Envelop saveExamPaperInfo(
            @ApiParam(value = "工单id", name = "orderId") @RequestParam(value = "orderId", required = true) String orderId,
            @ApiParam(value = "化验检查报告补录方式，1-拍照补录，2-接口数据", name = "examPaperWay") @RequestParam(value = "examPaperWay", required = false) Integer examPaperWay,
            @ApiParam(value = "化验检查报告补录时间", name = "examPaperTime") @RequestParam(value = "examPaperTime", required = false) String examPaperTime,
            @ApiParam(value = "居民的化验检查报告照片", name = "examPaperImg") @RequestParam(value = "examPaperImg", required = false) String examPaperImg,
            @ApiParam(name = "level", value = "医院等级，1.三级医院，2.二级医院，3.一级及以下") @RequestParam(value = "level", required = true) Integer level
    ) {
        try {
            return success("获取成功", doorOrderService.saveExamPaperInfo(orderId, examPaperWay, examPaperTime, examPaperImg, level));
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

//    @PostMapping("confirmReceipt")
//    @ApiOperation(value = "医生确认线下收款")
//    public Envelop confirmReceipt(
//            @ApiParam(value = "工单id", name = "orderId") @RequestParam(value = "orderId", required = true) String orderId,
//            @ApiParam(name = "level", value = "医院等级，1.三级医院，2.二级医院，3.一级及以下") @RequestParam(value = "level", required = true) Integer level
//    ) {
//        try {
//            return success("获取成功", doorOrderService.confirmReceipt(orderId, level));
//        } catch (Exception e) {
//            return failed(e.getMessage());
//        }
//    }

    @GetMapping("/adminTeamLeader")
    @ApiOperation(value = "判断医生是不是团队长")
    public Envelop adminTeamLeader(@ApiParam(value = "医生id", name = "doctorId") @RequestParam(value = "doctorId", required = true) String doctorId) {
        //返回结果等于1是团队长，等于0不是
        return success("查询成功", doorOrderService.adminTeamLeader(doctorId));
    }

    @GetMapping("/getTeamLeaderAllOrder")
    @ApiOperation(value = "查询团队下面的所有工单")
    public Envelop getTeamLeaderAllOrder(@ApiParam(value = "医生id", name = "doctorId") @RequestParam(value = "doctorId", required = true) String doctorId,
                                         @ApiParam(value = "页码", name = "page", defaultValue = "1", required = true) @RequestParam(value = "page", required = true) Integer page,
                                         @ApiParam(value = "工单状态", name = "status", required = false) @RequestParam(value = "status", required = false) Integer[] status,
                                         @ApiParam(value = "每页数目", name = "pageSize", defaultValue = "10", required = true) @RequestParam(value = "pageSize", required = true) Integer pageSize) {
        try {
            StringBuffer ss = new StringBuffer();
            if (status != null && status.length > 0) {
                int statusLength = status.length;
                for (int i = 0; i < statusLength; i++) {
                    ss.append(status[i]);
                    if (i < statusLength - 1) {
                        ss.append(",");
                    }
                }
            } else {
                ss = null;
            }
            return success("查询成功", doorOrderService.getTeamLeaderAllOrder(doctorId, ss, page, pageSize));
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }


    @ApiOperation("导出工单到excel")
    @GetMapping(value = "/doorOrderOutExcel")
    public void doorOrderOutExcel(
            @ApiParam(value = "工单id,多个id逗号隔开", name = "orderId", required = false) @RequestParam(value = "orderId", required = false) String orderId,
            @ApiParam(value = "居民姓名", name = "patientName", required = false) @RequestParam(value = "patientName", required = false) String patientName,
            @ApiParam(value = "联系方式", name = "patientPhone", required = false) @RequestParam(value = "patientPhone", required = false) String patientPhone,
            @ApiParam(value = "服务机构", name = "hospitalCode", required = false) @RequestParam(value = "hospitalCode", required = false) String hospitalCode,
            @ApiParam(value = "工单状态", name = "status", required = false) @RequestParam(value = "status", required = false) Integer[] status,
            @ApiParam(value = "创建时间开始，格式（yyyy-MM-dd mm:dd:ss）", name = "createTimeStart", required = false) @RequestParam(value = "createTimeStart", required = false) String createTimeStart,
            @ApiParam(value = "创建时间结束，格式（yyyy-MM-dd mm:dd:ss）", name = "createTimeEnd", required = false) @RequestParam(value = "createTimeEnd", required = false) String createTimeEnd,
            @ApiParam(value = "服务医生的名称", name = "serverDoctorName", required = false) @RequestParam(value = "serverDoctorName", required = false) String serverDoctorName,
            @ApiParam(value = "医生的code", name = "doctorCode", required = false) @RequestParam(value = "doctorCode", required = false) String doctorCode,
            @ApiParam(value = "角色0、医生，1、管理员", name = "isManage", required = false) @RequestParam(value = "isManage", required = false) String isManage,
            @ApiParam(value = "0-不需要，1-需要，待补录；2-需要，已补录", name = "examPaperStatus", required = false) @RequestParam(value = "examPaperStatus", required = false) Integer examPaperStatus,
            @ApiParam(value = "角色等级", name = "level", required = false) @RequestParam(value = "level", required = false) String level,
            @ApiParam(value = "服务类型类型 ", name = "serverType") @RequestParam(value = "serverType", required = false) String serverType,
            HttpServletResponse response
    ) {
        try {
            if (StringUtils.isEmpty(isManage)) {
                isManage = getCurrentRoleIsManange(doctorCode);
            }
            if ("0".equals(isManage)) {
                if (StringUtils.isEmpty(doctorCode)) {
                    doctorCode = getUID();
                }
            } else if ("1".equals(isManage)) {
                //如果是管理员并且未筛选机构，就默认展示其管理下所有机构
//                String level = getCurrentRoleLevel();
//                String currentRoleCode = getCurrentRoleCode();
                if (level.equals("2")) {
                    //市管理员
                    if (StringUtils.isNotBlank(hospitalCode)) {
                        hospitalCode = hospitalCode.substring(0, hospitalCode.length() - 2) + "%";
                    }
                } else if (level.equals("3")) {
                    //区管理员
                    if (StringUtils.isNotBlank(hospitalCode)) {
                        hospitalCode = hospitalCode + "%";
                    }
                }
            }
            doorOrderService.doorOrderOutExcel(orderId, patientName, patientPhone, hospitalCode,
                    status, createTimeStart, createTimeEnd, serverDoctorName, doctorCode, examPaperStatus, serverType, response);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


//    /**
//     * 获取服务项目
//     */
//    @GetMapping("selectByHospital")
//    @ApiOperation(value = "获取服务项目")
//    public Envelop selectByHospital(
//            @ApiParam(value = "机构code", name = "hospital") @RequestParam(value = "hospital", required = false) String hospital,
//            @ApiParam(value = "服务项目名称", name = "serverItemName") @RequestParam(value = "serverItemName", required = false) String serverItemName) {
//        try {
//            return success("获取成功", "dara", serviceItemService.selectByHospital(hospital, serverItemName));
//        } catch (Exception e) {
//            return failed(e.getMessage());
//        }
//    }
//
//    /**
//     * 获取服务项目-上门服务
//     */
//    @GetMapping("selectServiceByHospital")
//    @ApiOperation(value = "获取服务项目-上门服务")
//    public Envelop selectServiceByHospital(
//            @ApiParam(value = "机构code", name = "hospital") @RequestParam(value = "hospital", required = false) String hospital,
//            @ApiParam(value = "服务项目名称", name = "serverItemName") @RequestParam(value = "serverItemName", required = false) String serverItemName,
//            @ApiParam(value = "类型", name = "type") @RequestParam(value = "type", required = false) String type,
//            @ApiParam(value = "居民", name = "patient") @RequestParam(value = "patient", required = false) String patient,
//            @ApiParam(value = "loginUserId", name = "loginUserId") @RequestParam(value = "loginUserId", required = false) String loginUserId
//    ) {
//        try {
//            return success("获取成功", serverPackageService.selectServiceByHospital(hospital, serverItemName, type, patient, loginUserId));
//        } catch (Exception e) {
//            return failed(e.getMessage());
//        }
//    }
//
//
//    @RequestMapping(value = "selectServiceTypes", method = RequestMethod.POST)
//    @ResponseBody
//    @ApiOperation(value = "获取服务项目类型-上门服务")
//    public Envelop selectServiceTypes() {
//        try {
//            return success("查询成功", serverPackageService.selectTypes());
//        } catch (Exception ex) {
//            return failed(ex.getMessage());
//        }
//    }

    @PostMapping("updateDoctorInfo")
    @ApiOperation(value = "修改保存服务医生")
    public Envelop updateDoctorInfo(
            @ApiParam(value = "jsonData", name = "jsonData")
            @RequestParam(value = "jsonData", required = false) String jsonData) {
        try {
            // 医生更新上门信息
            wlyyDoorServiceOrderService.updateDoctorInfo(jsonData);
            return success("修改成功");
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    @PostMapping("updateDoctorInfoNoPatient")
    @ApiOperation(value = "修改保存服务医生(不用经过居民同意)")
    public Envelop updateDoctorInfoNoPatient(
            @ApiParam(value = "jsonData", name = "jsonData")
            @RequestParam(value = "jsonData", required = false) String jsonData) {
        try {
            // 医生更新上门信息
            wlyyDoorServiceOrderService.updateDoctorInfoNoPatient(jsonData);
            return success("修改成功");
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

//    @PostMapping("updatePackageItemInfo")
//    @ApiOperation(value = "修改保存服务项")
//
//    public Envelop updatePackageItemInfo(
//            @ApiParam(value = "jsonData", name = "jsonData")
//            @RequestParam(value = "jsonData", required = false) String jsonData) {
//        try {
//            doorOrderService.updatePackageItemInfo(jsonData);
//            return success("修改成功");
//        } catch (Exception e) {
//            return failed(e.getMessage());
//        }
//    }


//    @ApiOperation("社区医院下医生列表查询接口 没分页")
//    @RequestMapping(value = "getDoctorListByHospitalNoPage", method = RequestMethod.POST)
//    @ResponseBody
//    public Envelop getDoctorListByHospitalNoPage(
//            @RequestParam(required = true) String orderId,
//            @RequestParam(required = false) String hospital,
//            @RequestParam(required = false) String name,
//            @RequestParam(required = false) Integer type
//    ) {
//        try {
//            List<JSONObject> array = new ArrayList<>();
//            WlyyDoorServiceOrderDO one = doorServiceOrderDao.findById(orderId).orElse(null);
//            // 被服务的居民的次数（非代理居民）
//            Integer times = wlyyDoorServiceOrderService.countPatientDoorTimes(one.getPatient());
//            List<String> doctors = doorDoctorDao.findDoctors(orderId);
//            List<Map<String, Object>> list = familyContractService.getDoctorsByhospitalNoPage(hospital, name, type, times);
//            if (list != null) {
//                for (Map<String, Object> doctor : list) {
//                    if (doctor == null) {
//                        continue;
//                    }
//                    JSONObject json = new JSONObject();
//                    json.put("id", doctor.get("id"));
//                    // 医生标识
//                    json.put("code", doctor.get("code"));
//                    // 医生姓名
//                    json.put("name", doctor.get("name"));
//                    // 所在医院名称
//                    json.put("hospital", doctor.get("hospital"));
//                    // 所在医院名称
//                    json.put("hospital_name", doctor.get("hospital_name"));
//                    // 科室名称
//                    json.put("dept_name", (doctor.get("dept_name") == null ||
//                            org.apache.commons.lang3.StringUtils.isEmpty(doctor.get("dept_name").toString())) ? " " : doctor.get("dept_name"));
//
//                    json.put("level", doctor.get("level"));
//                    json.put("job", doctor.get("job"));
//                    // 职称名称
//                    json.put("job_name", (doctor.get("job_name") == null ||
//                            org.apache.commons.lang3.StringUtils.isEmpty(doctor.get("job_name").toString())) ? " " : doctor.get("job_name"));
//                    // 头像
//                    json.put("photo", doctor.get("photo"));
//                    // 简介
//                    json.put("introduce", doctor.get("introduce"));
//                    // 专长
//                    json.put("expertise", doctor.get("expertise"));
//                    json.put("flag", doctors.contains(String.valueOf(doctor.get("code"))) ? true : false);
//                    json.put("fixed", one != null && String.valueOf(doctor.get("code")).equals(one.getDoctor()) ? true : false);
//                    //出诊费用
//                    json.put("fee", doctor.get("fee"));
//                    array.add(json);
//                }
//            }
//            // 排序
//            Collections.sort(array, new Comparator<JSONObject>() {
//                @Override
//                public int compare(JSONObject o1, JSONObject o2) {
//                    Integer flag1 = (Boolean) o1.get("flag") ? 2 : 1;
//                    Integer flag2 = (Boolean) o2.get("flag") ? 2 : 1;
//                    if (flag1 - flag2 > 0) {
//                        return -1;
//                    } else if (flag1 - flag2 < 0) {
//                        return 1;
//                    } else {
//                        return 0;
//                    }
//                }
//            });
//            Collections.sort(array, new Comparator<JSONObject>() {
//                @Override
//                public int compare(JSONObject o1, JSONObject o2) {
//                    Integer fixed = (Boolean) o1.get("fixed") ? 2 : 1;
//                    Integer fixed2 = (Boolean) o2.get("fixed") ? 2 : 1;
//                    int sort = fixed - fixed2;
//                    if (sort > 0) {
//                        return -1;
//                    } else if (sort < 0) {
//                        return 1;
//                    } else {
//                        return 0;
//                    }
//                }
//            });
//            return success("获取医院医生列表成功！", array);
//        } catch (RuntimeException se) {
//            return failed(se.getMessage());
//        } catch (Exception e) {
//            return failed(e.getMessage());
//        }
//    }


    @GetMapping(value = "queryDoctorListNotStopped")
    @ApiOperation(value = "服务人员列表（可接单状态的医生列表）")
    public Envelop queryDoctorListWithNotStopped(
            @ApiParam(name = "patient", value = "服务对象code", required = true) @RequestParam(value = "patient") String patient,
            @ApiParam(name = "hospital", value = "机构code", required = true) @RequestParam(value = "hospital") String hospital,
            @ApiParam(name = "page", value = "分页大小", required = true, defaultValue = "1") @RequestParam(value = "page") int page,
            @ApiParam(name = "size", value = "页码", required = true, defaultValue = "15") @RequestParam(value = "size") int size) {
        try {
            JSONObject result = wlyyDoorServiceOrderService.queryDoctorListWithNotStopped(patient, hospital, page, size);
            if (result.getIntValue(ResponseContant.resultFlag) == ResponseContant.fail) {
                return failed(result.getString(ResponseContant.resultMsg));
            }
            int count = result.getIntValue(ResponseContant.count);
            JSONObject object = new JSONObject();
            object.put("total", count);
            object.put("detailModelList", result.get(ResponseContant.resultMsg));
            object.put("currPage", page);
            object.put("pageSize", size);
            return success("查询成功", object);
        } catch (Exception e) {
            return failed(e.getMessage());
        }

    }


    @PostMapping(value = "cancelOrder")
    @ApiOperation(value = "取消工单")
    public Envelop cancelOrder(
            @ApiParam(name = "orderId", value = "工单id") @RequestParam(value = "orderId", required = true) String orderId,
            @ApiParam(name = "type", value = "取消类型：1-调度员主动取消，2-居民取消, 3-医生取消") @RequestParam(value = "type", required = true) int type,
            @ApiParam(name = "reason", value = "取消理由") @RequestParam(value = "reason", required = false) String reason,
            @ApiParam(name = "dispatcher", value = "取消工单的调度员（只允许调度员来操作）") @RequestParam(value = "dispatcher", required = false) String dispatcher,
            @ApiParam(name = "dispathcherName", value = "调度员姓名") @RequestParam(value = "dispathcherName", required = false) String dispatcherName) {
        try {
            JSONObject result = wlyyDoorServiceOrderService.cancelOrder(orderId, type, reason, dispatcher, dispatcherName);
            if (result.getIntValue(ResponseContant.resultFlag) == ResponseContant.fail) {
                return failed(result.getString(ResponseContant.resultMsg));
            }
            return success("取消成功", result.get(ResponseContant.resultMsg));
        } catch (Exception e) {
            return failed(e.getMessage());
        }

    }


    @PostMapping(value = "transOrderToOtherOrg")
    @ApiOperation(value = "调度员将工单转接给其他机构")

    public Envelop transOrderToOtherOrg(
            @ApiParam(name = "orderId", value = "工单id") @RequestParam(value = "orderId", required = true) String orderId,
            @ApiParam(name = "hospital", value = "机构code") @RequestParam(value = "hospital", required = true) String hospital) {
        try {
            JSONObject result = wlyyDoorServiceOrderService.transOrderToOtherOrg(orderId, hospital);
            if (result.getIntValue(ResponseContant.resultFlag) == ResponseContant.fail) {
                return failed(result.getString(ResponseContant.resultMsg));
            }
            return success("转接成功", result.get(ResponseContant.resultMsg));
        } catch (Exception e) {
            return failed(e.getMessage());
        }

    }


    @PostMapping(value = "dispatcherIntoTopic")
    @ApiOperation(value = "调度员进入会话")
    public Envelop dispatcherIntoTopic(
            @ApiParam(name = "orderId", value = "工单id", required = true) @RequestParam String orderId,
            @ApiParam(name = "hospitalName", value = "机构名称", required = true) @RequestParam String hospitalName,
            @ApiParam(name = "dispatcher", value = "调度员code", required = true) @RequestParam String dispatcher,
            @ApiParam(name = "dispatcherName", value = "调度员姓名", required = true) @RequestParam String dispatcherName
    ) {
        try {
            JSONObject result = wlyyDoorServiceOrderService.dispatcherIntoTopic(orderId, hospitalName, dispatcher, dispatcherName);
            if (result.getIntValue(ResponseContant.resultFlag) == ResponseContant.fail) {
                return failed(result.getString(ResponseContant.resultMsg));
            }
            return success("转接成功", result.get(ResponseContant.resultMsg));
        } catch (Exception e) {
            return failed(e.getMessage());
        }

    }


    @PostMapping(value = "updateServiceStatus")
    @ApiOperation(value = "更新预约服务项目类型")

    public Envelop updateServiceStatus(
            @ApiParam(name = "id", value = "工单id") @RequestParam(value = "id", required = true) String id) {
        try {
            Integer result = doorOrderService.updateServiceStatus(id);
            if (result < 1) {
                return failed("修改服务类型失败");
            }
            return success("修改成功", result);
        } catch (Exception e) {
            return failed(e.getMessage());
        }

    }

    @GetMapping(value = "orderIsCancel")
    @ApiOperation(value = "查看消息时判断工单是否取消")

    public Envelop orderIsCancel(
            @ApiParam(name = "messageId", value = "消息id", required = true) @RequestParam String messageId) {
        try {
            JSONObject result = wlyyDoorServiceOrderService.orderIsCancel(messageId);
            if (result.getIntValue(ResponseContant.resultFlag) == ResponseContant.fail) {
                return failed(result.getString(ResponseContant.resultMsg));
            }
            return success("查询成功", result.get(ResponseContant.resultMsg));
        } catch (Exception e) {
            return failed(e.getMessage());
        }

    }

//    @GetMapping(value = "getWorkingTimeByDoctor")
//    @ResponseBody
//    @ApiOperation("获取居民家签机构的上班时间")
//    public Envelop getWorkingTimeByPatient(@RequestParam String patient) {
//        try {
//            return success("查询成功", hospitalService.getWorkingTimeByPatient(patient));
//        } catch (RuntimeException se) {
//            return failed(se.getMessage());
//        } catch (Exception e) {
//            return failed(e.getMessage());
//        }
//    }

    @PostMapping(value = "cancelConclusion")
    @ResponseBody
    @ApiOperation("取消登记服务小结")
    public Envelop cancelConclusion(@ApiParam(name = "orderId", value = "工单id", required = true) @RequestParam String orderId) {
        try {
            wlyyDoorServiceOrderService.cancelConclusion(orderId);
            return success("取消成功");
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    @PostMapping(value = "createPrescription")
    @ResponseBody

    @ApiOperation("上门服务前手工开方接口")
    public Envelop createPrescription(@ApiParam(name = "orderId", value = "工单id", required = true) @RequestParam String orderId,
                                      @ApiParam(name = "prescriptionCode", value = "处方单号，多个用逗号隔开", required = true) @RequestParam String prescriptionCode,
                                      @ApiParam(name = "commitPrescriptionCode", value = "已提交处方表code") @RequestParam(required = false) String commitPrescriptionCode) {
        try {
            JSONObject json = wlyyDoorServiceOrderService.createPrescription(orderId, prescriptionCode, commitPrescriptionCode);
            if (json.getInteger("status") == -1) {
                return failed(json.getString("msg"));
            }
            return success("开方完成");
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }


    @PostMapping(value = "updateDoctorStatus")
    @ResponseBody
    @ApiOperation("更新医生地理位置状态")
    public Envelop updateDoctorStatus(@ApiParam(name = "positionStatus", value = "医生地理位置状态", required = true) @RequestParam Integer positionStatus,
                                      @ApiParam(name = "position", value = "地理位置，用逗号隔开", required = false) @RequestParam String position) {
        try {
            wlyyDoorServiceOrderService.updateDoctorStatus(getUID(), positionStatus, position);
            return success("更新完成");
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

//    @GetMapping(value = "getPrescription")
//    @ResponseBody
//    @ApiOperation("获取长处方信息")
//    public Envelop getPrescription(@ApiParam(name = "patient", value = "居民code") @RequestParam(required = true) String patient,
//                                   @ApiParam(name = "visitDept", value = "挂号科室编码") @RequestParam(required = false) String visitDept,
//                                   @ApiParam(name = "visitTimeFrom", value = "开始时间") @RequestParam(required = false) String visitTimeFrom,
//                                   @ApiParam(name = "visitTimeEnd", value = "结束时间") @RequestParam(required = false) String visitTimeEnd) {
//        try {
//            JSONArray json = doorPrescriptionService.getPrescription(getUID(), patient, visitDept, visitTimeFrom, visitTimeEnd);
//            return success("获取成功", json);
//        } catch (RuntimeException se) {
//            return failed(se.getMessage());
//        } catch (Exception e) {
//            return failed(e.getMessage());
//        }
//    }

//    @PostMapping(value = "createDoorPrescription")
//    @ResponseBody
//    @ApiOperation("开方接口")
//    public Envelop createDoorPrescription(@ApiParam(name = "orderId", value = "订单id", required = true) @RequestParam String orderId,
//                                          @ApiParam(name = "jsonData", value = "有关处方相关数据", required = true) @RequestParam String jsonData) {
//        try {
//            JSONObject jsonObject = doorPrescriptionService.createDoorPrescription(orderId, jsonData, getUID());
//            if (jsonObject.getInteger("status") == 200) {
//                return success("开方成功", jsonObject.get("data"));
//            } else {
//                return failed(jsonObject.getString("msg"));
//            }
//        } catch (RuntimeException se) {
//            return failed(se.getMessage());
//        } catch (Exception e) {
//            return failed(e.getMessage());
//        }
//    }

//    @GetMapping(value = "getPrescriptionByCode")
//    @ResponseBody
//    @ApiOperation("获取长处方详情")
//    public Envelop getPrescriptionByCode(@ApiParam(name = "recipeNo", value = "医嘱号", required = true)
//                                         @RequestParam String recipeNo,
//                                         @ApiParam(name = "patientCode", value = "居民code", required = true)
//                                         @RequestParam String patientCode) {
//        try {
//            JSONObject json = doorPrescriptionService.getPrescriptionByCode(recipeNo, patientCode);
//            return success("获取成功", json);
//        } catch (RuntimeException se) {
//            return success(se.getMessage());
//        } catch (Exception e) {
//            return failed(e.getMessage());
//        }
//    }

//    @PostMapping(value = "cancelPrescriptionByCode")
//    @ResponseBody
//    @ApiOperation("取消开方")
//    public Envelop cancelPrescriptionByCode(@ApiParam(name = "prescriptionCode", value = "处方表code", required = true)
//                                            @RequestParam String prescriptionCode) {
//        try {
//            JSONObject result = jwDoorPrescriptionService.fadeRecipeUnion(prescriptionCode, getUID());
//            if (result.getInteger("status") == 200) {
//                return success("取消成功", result.getString("msg"));
//            } else {
//                return failed(result.getString("msg"));
//            }
//        } catch (Exception e) {
//            return failed(e.getMessage());
//        }
//    }

//    @PostMapping(value = "skipPrescription")
//    @ResponseBody
//    @ApiOperation("跳过开方")
//    public Envelop skipPrescription(@ApiParam(name = "orderId", value = "订单id", required = true) @RequestParam String orderId) {
//        try {
//            doorPrescriptionService.skipPrescriptionByOrderId(orderId);
//            return success("成功跳过");
//        } catch (Exception e) {
//            return failed(e.getMessage());
//        }
//    }

//    @RequestMapping(value = "/findDictByName", method = RequestMethod.GET)
//    @ApiOperation(value = "药品列表")
//    public Envelop findDictByName(@ApiParam(name = "name", value = "药品名称（中文或拼音首字母查询）", defaultValue = "胰岛素")
//                                  @RequestParam(value = "name", required = false) String name,
//                                  @ApiParam(name = "subjectClass", value = "药品科目, 0101西成药，0103中草药", required = true)
//                                  @RequestParam(value = "subjectClass") String subjectClass) {
//        try {
//            return success("操作成功！", zyDictService.findDictByDoctorAndName(getUID(), name, null, subjectClass));
//        } catch (Exception e) {
//            return failed(e.getMessage());
//        }
//    }


    @PostMapping(value = "proxyUpdate")

    @ApiOperation(value = "上门服务-工单编辑")
    public Envelop proxyUpdate(@ApiParam(name = "jsonData", value = "Json数据", required = true) @RequestParam String jsonData) {
        try {
            JSONObject result = wlyyDoorServiceOrderService.proxyUpdate(jsonData, getUID());
            if (result.getIntValue(ResponseContant.resultFlag) == ResponseContant.fail) {
                return failed(result.getString(ResponseContant.resultMsg));
            }
            return success("修改成功！");
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    @GetMapping(value = "getDoorPatientList")
    @ResponseBody
    @ApiOperation("获取代预约服务对象列表")
    public Envelop getDoorPatientList(@ApiParam(name = "teamCode", value = "医生所在团队code", required = true) @RequestParam String teamCode,
                                      @ApiParam(name = "patientName", value = "居民姓名") @RequestParam(required = false) String patientName,
                                      @ApiParam(value = "页码", name = "page", defaultValue = "1", required = true)
                                      @RequestParam(value = "page") Integer page,
                                      @ApiParam(value = "每页数目", name = "pageSize", defaultValue = "10", required = true)
                                      @RequestParam(value = "pageSize") Integer pageSize) {
        try {
            List<Map<String, Object>> list = wlyyDoorServiceOrderService.getDoorPatientList(teamCode, getUID(), patientName, page, pageSize);
            return success("获取成功", list);
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

//    @PostMapping(value = "saveAuthorizeImage")
//    @ResponseBody
//    @ApiOperation("保存电子健康卡图片")
//    public Envelop saveAuthorizeImage(@ApiParam(name = "orderId", value = "订单id", required = true) @RequestParam String orderId,
//                                      @ApiParam(name = "authorizeImage", value = "电子健康卡授权图片", required = true) @RequestParam String authorizeImage) {
//        try {
//            doorPrescriptionService.saveAuthorizeImage(orderId, authorizeImage);
//            return success("保存成功");
//        } catch (Exception e) {
//            return failed(e.getMessage());
//        }
//    }

//    //ca认证
//    @ApiOperation("请求实名软证书并进行数字签名")
//    @ResponseBody
//    @RequestMapping(value = "/requestRealNameSoftCertAndSign", method = RequestMethod.POST)
//    public Envelop requestRealNameSoftCertAndSign(
//            @ApiParam(required = true, name = "strRealNameSoftCertCalledPasswd", value = "证书被调用保护口令")
//            @RequestParam(value = "strRealNameSoftCertCalledPasswd", required = true) String strRealNameSoftCertCalledPasswd,
//            @ApiParam(name = "strOriginalData", value = "原文")
//            @RequestParam(value = "strOriginalData", required = false) String strOriginalData,
//            @ApiParam(required = true, name = "prescriptionCode", value = "处方code")
//            @RequestParam(value = "prescriptionCode", required = true) String prescriptionCode) {
//
//        try {
//            org.json.JSONObject jsonObject = jwDoorPrescriptionService.requestRealNameSoftCertAndSign(getUID(), strRealNameSoftCertCalledPasswd, strOriginalData, prescriptionCode);
//            return success("获取信息成功！", jsonObject);
//        } catch (RuntimeException se) {
//            return failed(se.getMessage());
//        } catch (Exception e) {
//            return failed(e.getMessage());
//        }
//    }

//    @RequestMapping(value = "uploadCaDigital", method = RequestMethod.POST)
//    @ApiOperation("上传基位ca认证信息")
//    public Envelop uploadCaDigital(
//            @ApiParam(name = "prescriptionCode", value = "处方code") @RequestParam(value = "prescriptionCode", required = true) String prescriptionCode
//    ) {
//        try {
//            JSONObject json = jwDoorPrescriptionService.uploadCaDigital(prescriptionCode);
//            if (json.getInteger("status") == -1) {
//                return failed(json.getString("msg"));
//            }
//            return success("上传ca认证信息成功");
//        } catch (Exception e) {
//            return failed(e.getMessage());
//        }
//    }

    @RequestMapping(value = "/getIcd10Info", method = RequestMethod.GET)
    @ApiOperation(value = "获取诊断结果")
    public Envelop getIcd10Info(
            @RequestParam(required = false) @ApiParam(value = "诊断结果关键字", name = "nameKey") String nameKey,
            @ApiParam(value = "页码", name = "page", defaultValue = "1", required = true) @RequestParam(value = "page") Integer page,
            @ApiParam(value = "每页数目", name = "pageSize", defaultValue = "15", required = true) @RequestParam(value = "pageSize") Integer pageSize
    ) {
        try {
            return success("操作成功！", doorPrescriptionService.getIcd10Info(nameKey, page, pageSize));
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    @GetMapping("/getAdministratorStatusBarNum")
    @ApiOperation(value = "获取管理员顶部状态栏订单分类tab")
    public Envelop getAdministratorStatusBarNum(
            @ApiParam(value = "服务机构", name = "hospital", required = false) @RequestParam(value = "hospital", required = false) String hospital,
            @ApiParam(value = "角色0、医生，1、管理员", name = "isManage", required = false) @RequestParam(value = "isManage", required = false) String isManage,
            @ApiParam(value = "医生id", name = "doctorId", required = false) @RequestParam(value = "doctorId", required = false) String doctorId
    ) {
        try {
            if (StringUtils.isEmpty(isManage)) {
                isManage = getCurrentRoleIsManange(doctorId);
            }
            if ("1".equals(isManage) && StringUtils.isBlank(hospital)) {
                //如果是管理员并且未筛选机构，就默认展示其管理下所有机构
                //这边先注释-用到的时候在改20231011
//                String level = getCurrentRoleLevel();
//                String currentRoleCode = getCurrentRoleCode();
//                if (level.equals("2")) {
//                    //市管理员
//                    hospital = currentRoleCode.substring(0, currentRoleCode.length() - 2) + "%";
//                } else if (level.equals("3")) {
//                    //区管理员
//                    hospital = currentRoleCode + "%";
//                } else if (level.equals("4")) {
//                    //机构管理员
//                    hospital = currentRoleCode;
//                }
            } else if (StringUtils.isNotBlank(hospital) && hospital.length() < 10) {
                hospital += "%";
            }
            Map<String, String> map = doorOrderService.getAdministratorStatusBarNum(hospital);
            return success("获取成功", map);
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    @ApiOperation("导出服务小结到word")
    @RequestMapping(value = "exportConclusion", method = RequestMethod.GET)
    public void writeWordFile(@ApiParam(name = "urlString", value = "链接地址") @RequestParam(required = false) String urlString,
                              @ApiParam(name = "orderId", value = "订单id", required = true) @RequestParam String orderId,
                              HttpServletResponse response) {
        try {
            //导出路径
            WlyyDoorServiceOrderDO orderDO = doorOrderService.findById(orderId);
            response.setContentType("application/msword");
            response.setCharacterEncoding("utf-8");
            // 生成临时文件名称
            String fileName = orderDO.getPatientName() + "服务小结.doc";
            fileName = URLEncoder.encode(fileName, "utf-8");
            response.setHeader("Content-Disposition", "attachment; filename=" + fileName);
            OutputStream os = response.getOutputStream();
            //获取相对路径
            String pathName = DoorOrderController.class.getResource("/").getPath().replaceFirst("/", "");
            pathName = pathName.replace("target/classes/", "src/main/resources/conclusion.mht");
            Document doc = Jsoup.parse(new File(pathName), "UTF-8");
            String html = doorOrderService.handleData(doc, orderId);
            byte b[] = html.getBytes();
            ByteArrayInputStream bais = new ByteArrayInputStream(b);
            POIFSFileSystem poifs = new POIFSFileSystem();
            DirectoryEntry directory = poifs.getRoot();
            DocumentEntry documentEntry = directory.createDocument("WordDocument", bais);
            poifs.writeFilesystem(os);
            os.flush();
            os.close();
            bais.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 参数
     * orderId: 4028f8f676980a1e0176982702a20017
     * doctor: 361aef4891de492cb0e6a47cf9fe31f3
     * level: 3
     */
    @ApiOperation("推送知情同意書")
    @RequestMapping(value = "/sendInformedConsent", method = RequestMethod.GET)
    public Envelop sendInformedConsent(
            @ApiParam(value = "医生code", name = "doctor") @RequestParam(value = "doctor", required = true) String doctor,
            @ApiParam(value = "工单id", name = "orderId") @RequestParam(value = "orderId", required = false) String orderId,
            @ApiParam(name = "level", value = "医院等级，1.三级医院，2.二级医院，3.一级及以下") @RequestParam(value = "level", required = true) Integer level
    ) {
        try {
            // 没有提供工单id的情况下，则获取该医生当前最新一条的工单
            if (StringUtils.isEmpty(orderId)) {
                // 根据接单医生code获取最近一次服务orderId
                orderId = doorOrderService.getOrderIdByDoctor(doctor);
                if (StringUtils.isEmpty(orderId)) {
                    return failed("获取失败, 该医生暂无工单");
                }
            }
            // 根据orderId获取工单信息
            WlyyDoorServiceOrderDO doorServiceOrderDO = doorOrderService.getDoorServiceOrderById(orderId, level);
            if (doorServiceOrderDO != null) {
                doorServiceOrderDO.getServeDesc();//服务描述
                doorServiceOrderDO.getServeAddress();//服务地址
                doorServiceOrderDO.getDoctorArrivingTime();//服务时间
                doorServiceOrderDO.getPatientName();//服务对象
                doorServiceOrderDO.getPatientPhone();//服务电话
                doorServiceOrderDO.getIsPatientConfirm();//当前状态

                Integer success = doorOrderService.sendInformedConsent(orderId, doorServiceOrderDO.getServeDesc(), doctor);
                //微信模板信息
                //wlyyDoorServiceOrderService.orderMsgTask(orderId);
                if (success == 1) {
                    return success("知情同意书推送成功", doorServiceOrderDO);
                } else {
                    return failed("知情同意书推送失败");
                }
            } else {
                return failed("请检查工单");
            }
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }


    /**
     * 获取当前用户的角色是否是管理员
     */
    public String getCurrentRoleIsManange(String doctorId) {
        String sql = "SELECT\n" +
                "	r.doctor_code,\n" +
                "	t.CODE AS roleCode,\n" +
                "	t.NAME AS roleName \n" +
                "FROM\n" +
                "	base_doctor_role r\n" +
                "	INNER JOIN base_doctor_role_dict t ON t.CODE = r.role_code\n" +
                "	INNER JOIN base_doctor_hospital c ON r.doctor_code = c.doctor_code \n" +
                "WHERE\n" +
                "	1 = 1 \n" +
                "	AND t.CODE = 'admin' \n" +
                "	AND r.doctor_code = '" + doctorId + "'";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        if (list.isEmpty()) {
            return "0";
        } else {
            return "1";
        }
    }


}
