package com.yihu.jw.hospital.module.door.service;

import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.door.WlyyDoorServiceApplicationDo;
import com.yihu.jw.entity.door.WlyyDoorServiceVoucherDo;
import com.yihu.jw.entity.hospital.message.SystemMessageDO;
import com.yihu.jw.entity.wechat.WechatTemplateConfig;
import com.yihu.jw.hospital.message.dao.SystemMessageDao;
import com.yihu.jw.hospital.module.door.dao.DoorServiceApplicationDao;
import com.yihu.jw.hospital.module.door.dao.DoorServiceVoucherDao;
import com.yihu.jw.hospital.module.wx.dao.WechatTemplateConfigDao;
import com.yihu.jw.hospital.task.PushMsgTask;
import com.yihu.jw.hospital.utils.WeiXinAccessTokenUtils;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.util.common.IdCardUtil;
import com.yihu.jw.util.date.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.transaction.Transactional;
import java.util.*;

/**
 * Created by zdm on 2019/3/13.
 * 上门服务
 */
@Service
@Transactional
public class DoorServiceApplicationService {
    Map<Integer, String> relations = new HashMap<>();
    @Autowired
    private DoorServiceApplicationDao doorServiceApplicationDao;
    @Autowired
    private DoorServiceVoucherDao doorServiceVoucherDao;
    @Autowired
    JdbcTemplate jdbcTemplate;
    //    @Autowired
//    private MessageDao messageDao;
    @Autowired
    SystemMessageDao systemMessageDao;


    @Autowired
    private WlyyDoorServiceOrderService doorServiceOrderService;
    @Autowired
    private WlyyDoorServiceOrderService wlyyDoorServiceOrderService;
    @Autowired
    private BaseDoctorDao doctorDao;
    @Autowired
    BasePatientDao patientDao;

//    @Autowired
//    FamilyContractService contractService;

//    @Autowired
//    private BaseDoctorInfoService doctorInfoService;

//    @Autowired
//    private SignFamilyDao signFamilyDao;

//    @Autowired
//    private PatientFamilyMemberDao patientFamilyMemberDao;
    @Autowired
    private PushMsgTask pushMsgTask;
    @Autowired
    private WeiXinAccessTokenUtils weiXinAccessTokenUtils;
    @Autowired
    private WechatTemplateConfigDao templateConfigDao;


    @PostConstruct
    public void init() {
        relations.put(0, "其他");
        relations.put(1, "父亲");
        relations.put(2, "母亲");
        relations.put(3, "老公");
        relations.put(4, "老婆");
        relations.put(5, "儿子");
        relations.put(6, "女儿");
        relations.put(7, "未知");
        relations.put(8, "免疫关联");
    }

    /**
     * 获取当前登录居民开通服务情况及取消订单情况
     *
     * @param patientCode
     * @return
     */
    public JSONObject findPatientOnsiteService(String patientCode) throws Exception {
        JSONObject jsonObject = new JSONObject();
        ObjectMapper objectMapper = new ObjectMapper();
        WlyyDoorServiceApplicationDo wlyyDoorServiceApplicationDo = findOnsiteServiceByPatient(patientCode);
        jsonObject.put("onsiteFlag", false);
        jsonObject.put("wlyyDoorServiceApplicationDo", "");
        if (null != wlyyDoorServiceApplicationDo.getId()) {
            jsonObject.put("wlyyDoorServiceApplicationDo", objectMapper.writeValueAsString(wlyyDoorServiceApplicationDo));
            jsonObject.put("onsiteFlag", true);
        }
        int cancellation_times = wlyyDoorServiceOrderService.countPtientCancelTimesOneMonth(patientCode);
        jsonObject.put("cancellation_times", cancellation_times);
        return jsonObject;
    }

    /**
     * 获取居民服务申请信息
     *
     * @param patientCode 居民code
     * @return
     */
    public WlyyDoorServiceApplicationDo findOnsiteServiceByPatient(String patientCode) throws Exception {
        //根据居民获取所有申请信息
        List<WlyyDoorServiceApplicationDo> wlyyDoorServiceApplicationDos = doorServiceApplicationDao.findByPatientOrderByCreateTimeDesc(patientCode);
        WlyyDoorServiceApplicationDo wlyyDoorServiceApplicationDo = new WlyyDoorServiceApplicationDo();
        if (null != wlyyDoorServiceApplicationDos && wlyyDoorServiceApplicationDos.size() > 0) {
            //获取最新的记录展示
            wlyyDoorServiceApplicationDo = wlyyDoorServiceApplicationDos.get(0);
            List<WlyyDoorServiceVoucherDo> wlyyDoorServiceVoucherDos = doorServiceVoucherDao.findByServiceIdAndStatusOrderBySortAsc(wlyyDoorServiceApplicationDo.getId(), 1);
            wlyyDoorServiceApplicationDo.setWlyyDoorServiceVoucherDos(wlyyDoorServiceVoucherDos);
        }
        return wlyyDoorServiceApplicationDo;
    }

    /**
     * 获取居民服务申请状态： -1删除，0审核中，1未通过，2审核通过，3已过期
     *
     * @param patientCode 居民code
     * @return
     */
    public Integer findOnsiteServiceStatusByPatient(String patientCode) throws Exception {
        Integer status = -1;
        //根据居民获取所有申请信息
        List<WlyyDoorServiceApplicationDo> wlyyDoorServiceApplicationDos = doorServiceApplicationDao.findByPatientOrderByCreateTimeDesc(patientCode);
        if (null != wlyyDoorServiceApplicationDos && wlyyDoorServiceApplicationDos.size() > 0) {
            //获取最新的记录展示
            status = wlyyDoorServiceApplicationDos.get(0).getStatus();
        }
        return status;
    }

    /**
     * 获取居民的家庭成员及授权信息
     *
     * @param patient 居民
     * @return
     */
//    public JSONArray getPatientFamilyMembers(String patient) throws Exception {
//        JSONArray resultArray = new JSONArray();
//        //获取当前居民自己的信息
//        JSONObject objPatient = findPatientByCode(patient);
//        resultArray.put(objPatient);
//        //根据居民获取家庭成员基本信息，不包含自己
//        String sql = "select t1.family_relation,t1.is_authorize,t2.code,t2.name,t2.sex,t2.birthday,t2.idcard,t2.photo,t2.mobile,t2.address   " +
//                " from " +
//                "    wlyy_patient_family_member t1, " +
//                "    wlyy_patient t2 " +
//                " where " +
//                "    t2.code in (select family_member from wlyy_patient_family_member where patient = ? ) " +
//                "    and t1.patient = ? " +
//                "    and t1.family_member = t2.code ";
//        List<Map<String, Object>> result = jdbcTemplate.queryForList(sql, new Object[]{patient, patient});
//        if (result != null && result.size() > 0) {
//            for (Map<String, Object> map : result) {
//                JSONObject obj = new JSONObject();
//                //获取家签信息
//                obj = updatePatientInfo(obj, map.get("code").toString());
//                //根据家庭成员及当前登录人获取授权信息
//                PatientFamilyMember patientFamilyMember = patientFamilyMemberDao.findByPatientAndFamilyMember(map.get("code").toString(), patient);
//                Integer isAuthorize = null == patientFamilyMember ? 0 : patientFamilyMember.getIsAuthorize();
//                obj.put("code", map.get("code"));
//                obj.put("name", map.get("name"));
//                obj.put("sex", map.get("sex"));
//                //0:未授权,1:已授权
//                obj.put("isAuthorize", map.get("is_authorize"));
//                obj.put("birthday", map.get("birthday"));
//                obj.put("idcard", StringUtils.isEmpty(String.valueOf(map.get("idcard"))) ? "" : map.get("idcard").toString());
//                obj.put("photo", map.get("photo"));
//                obj.put("mobile", map.get("mobile"));
//                obj.put("address", StringUtils.isEmpty(String.valueOf(map.get("address"))) ? "" : map.get("address"));
//                obj.put("familyRelation", map.get("family_relation"));
//                obj.put("familyRelationName", relations.get(map.get("family_relation")));
//                resultArray.put(obj);
//            }
//        }
//        return resultArray;
//    }

    /**
     * 根据居民的code获取居民的基本信息
     *
     * @param patient
     * @return
     * @throws Exception
     */
    public JSONObject findPatientByCode(String patient) throws Exception {
        //获取当前居民自己的信息
        BasePatientDO p = patientDao.findById(patient).orElse(null);
        JSONObject obj = new JSONObject();
        //获取家签信息--先注释
//        obj = updatePatientInfo(obj, p.getCode());
        obj.put("code", p.getId());
        obj.put("name", p.getName());
        obj.put("age", IdCardUtil.getAgeByIdcardOrBirthday(p.getIdcard(), p.getBirthday()));
        obj.put("sex", p.getSex());
        //0:未授权,1:已授权
        obj.put("isAuthorize", 1);
        obj.put("birthday", p.getBirthday());
        obj.put("idcard", p.getIdcard());
        obj.put("photo", p.getPhoto());
        obj.put("mobile", p.getMobile());
        obj.put("address", p.getAddress());
        obj.put("familyRelation", -1);
        obj.put("familyRelationName", "本人");
        return obj;
    }

    /**
     * 获取上门服务的机构或区县
     *
     * @param type 类型：1区、2机构
     * @return
     */
    public Set<String> getConfigure(String type) throws Exception {
        String sql = "SELECT c.hospital from wlyy_door_service_configure c WHERE c.content_type='1' AND c.type='" + type + "'  and c.status=1 ";
        List<String> stringSet = jdbcTemplate.queryForList(sql, String.class);
        Set<String> simpleSet = new HashSet<>(stringSet);
        return simpleSet;
    }

//      /**
     //     * 根据居民code获取家签信息，封装家签、授权及上门服务资质信息
     //     *
     //     * @param obj     返参
     //     * @param patient 居民code
     //     * @return
     //     * @throws Exception
     //     */
//    public JSONObject updatePatientInfo(JSONObject obj, String patient) throws Exception {
//        //获取家签信息
//        List<SignFamily> signs = contractService.findAllSignByPatient(patient);
//        //待签约
//        int status = 0;
//        String hospital = "";
//        String hospitalName = "";
//        for (SignFamily sign : signs) {
//            if (sign.getStatus() > 0 && sign.getExpensesStatus().equals("1")) {
//                //已签约
//                status = 1;
//                hospital = sign.getHospital();
//                hospitalName = sign.getHospitalName();
//            }
//        }
//        obj.put("signStatus", status);
//        obj.put("hospital", hospital);
//        obj.put("hospitalName", hospitalName);
//        //获取授权医院,判断该居民签约医院是否开通上门服务
//        obj.put("aptitude", false);
//        if (null != getConfigure("1") && StringUtils.isNotBlank(hospital) && getConfigure("1").contains(hospital.substring(0, 6))) {
//            obj.put("aptitude", true);
//        } else if (null != getConfigure("2") && StringUtils.isNotBlank(hospital) && getConfigure("2").contains(hospital)) {
//            obj.put("aptitude", true);
//        }
//        return obj;
//    }

    /**
     * 开通上门服务资质评审
     *
     * @param jsonData   资质相关信息
     * @param userCode   当前操作的人，可能是医生，可能是居民
     * @param doctorFlag 居民端创建1，医生端创建2
     * @return
     * @throws Exception
     */
    public WlyyDoorServiceApplicationDo create(String doctorFlag, String jsonData, String userCode) throws Exception {
        WlyyDoorServiceApplicationDo wlyyDoorServiceApplicationDo = JSON.parseObject(jsonData, WlyyDoorServiceApplicationDo.class);
        //查询居民信息 base_patient
        String sql = "select * from base_patient where 1=1 AND id='" + wlyyDoorServiceApplicationDo.getPatient() + "'";
        List<BasePatientDO> list = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(BasePatientDO.class));
        BasePatientDO p = null;
        if (!list.isEmpty()) {
            p = list.get(0);
        }
        //判断该居民是否存在未审核的资质申请，若存在则不允许再申请
        List<WlyyDoorServiceApplicationDo> wlyyDoorServiceApplicationDos = doorServiceApplicationDao.findByPatientAndStatus(wlyyDoorServiceApplicationDo.getPatient(), 0);

        if (null != wlyyDoorServiceApplicationDos && wlyyDoorServiceApplicationDos.size() > 0) {
            throw new RuntimeException("该居民尚有未审核的上门服务资质申请!");
        }


        //默认先给调度员评审
        wlyyDoorServiceApplicationDo.setCode(userCode);
        if ("2".equals(doctorFlag)) {
            if (StringUtils.isNotBlank(wlyyDoorServiceApplicationDo.getStatus().toString()) && 2 == (wlyyDoorServiceApplicationDo.getStatus())) {
                if (null == wlyyDoorServiceApplicationDo.getStartTime() || null == wlyyDoorServiceApplicationDo.getEndTime()) {
                    throw new RuntimeException("资质有效开始和结束时间不能为空!");
                }
            }
            //查询待预约医生
            BaseDoctorDO doctor = doctorDao.findById(userCode).orElse(null);
            wlyyDoorServiceApplicationDo.setAuditorType(2);
            wlyyDoorServiceApplicationDo.setAuditTime(new Date());
            wlyyDoorServiceApplicationDo.setProxyPatient(userCode);
            wlyyDoorServiceApplicationDo.setProxyPatientName(doctor.getName());
        } else {
            //居民自己创建
            wlyyDoorServiceApplicationDo.setAuditorType(1);
            wlyyDoorServiceApplicationDo.setStatus(0);
            wlyyDoorServiceApplicationDo.setProxyPatient(userCode);
            wlyyDoorServiceApplicationDo.setProxyPatientName(p.getName());
        }
        WlyyDoorServiceApplicationDo wlyyDoorServiceApplicationDo1 = doorServiceApplicationDao.save(wlyyDoorServiceApplicationDo);
        if (!"2".equals(doctorFlag)) {
            //居民申请给调度员发系统消息
            List<Map<String, Object>> mapList = findDispatcherByPatient(userCode);
            SystemMessageDO message = null;

            for (Map<String, Object> map : mapList) {
                if (null != map.get("code")) {
                    message = new SystemMessageDO();
                    message.setCreateTime(new Date());
                    message.setCreateTime(new Date());
                    message.setIsRead("1");
                    message.setState(1);
                    message.setRelationCode(wlyyDoorServiceApplicationDo1.getId().toString());
                    message.setOver("1");
                    //调度员
                    message.setReceiver(map.get("code").toString());
                    message.setSender(wlyyDoorServiceApplicationDo1.getPatient());
                    message.setCode(userCode);
                    message.setTitle("居民资质审核");
                    message.setContent(p.getName() + "提交了资质申请，请及时处理");
                    //消息类型：401为资质申请审核-调度员
                    message.setType("401");
                    message.setDel("1");
                    systemMessageDao.save(message);
                    //发送医生助手模板消息
                    doorServiceOrderService.sendWeixinMessage(1, map.get("code").toString(), wlyyDoorServiceApplicationDo1.getPatient());
                }
            }
        }
        wlyyDoorServiceApplicationDo1.setWlyyDoorServiceVoucherDos(wlyyDoorServiceApplicationDo.getWlyyDoorServiceVoucherDos());
        wlyyDoorServiceApplicationDo1 = updateDtoAndSaveVoucher(wlyyDoorServiceApplicationDo1, p);
        return wlyyDoorServiceApplicationDo1;

    }

    /**
     * 根据居民获取签约机构的调度员
     *
     * @param patient 当前资质申请居民  roleCode=dispatcher为调度员的角色
     * @return
     */
    public List<Map<String, Object>> findDispatcherByPatient(String patient) throws Exception {
//        String sql = "SELECT d.code,d.name FROM " +
//                " manage_role_user ru LEFT JOIN " +
//                " manage_roles r  ON ru.role_id=r.id " +
//                " LEFT JOIN (select d.code,d.name from wlyy.wlyy_doctor d LEFT JOIN wlyy.wlyy_sign_family a ON d.hospital=a.hospital  " +
//                "                 where a.patient = '" + patient + "' and a.status > 0  " +
//                "                 AND d.status=1 and d.del=1) d ON ru.user_code=d.code " +
//                " WHERE r.code='dispatcher' AND d.`code` IS NOT NULL";
//
//        List<Map<String, Object>> mapList = jdbcTemplate.queryForList(sql);

        //第三医院是没有签约的这个动作的

        String sql = "SELECT\n" +
                "	a.doctor_code 'code',\n" +
                "	b.`name` 'name',\n" +
                "	a.org_code,\n" +
                "	a.org_name 'hospitalName',\n" +
                "	t.CODE AS roleCode,\n" +
                "	t.NAME AS roleName \n" +
                "FROM\n" +
                "	base_doctor_role r\n" +
                "	INNER JOIN base_doctor_role_dict t ON t.CODE = r.role_code\n" +
                "	INNER JOIN base_doctor_hospital a ON r.doctor_code = a.doctor_code\n" +
                "	INNER JOIN base_doctor b ON a.doctor_code = b.id \n" +
                "WHERE\n" +
                "	1 = 1 \n" +
                "	AND t.`code` = 'dispatcher' \n";
        List<Map<String, Object>> mapList = jdbcTemplate.queryForList(sql);
        return mapList;
    }

    /**
     * 将申请信息中的资质信息保存
     *
     * @param wlyyDoorServiceApplicationDo 资质信息
     * @param p                            居民信息
     * @return
     */
    public WlyyDoorServiceApplicationDo updateDtoAndSaveVoucher(WlyyDoorServiceApplicationDo wlyyDoorServiceApplicationDo, BasePatientDO p) throws Exception {
        List<WlyyDoorServiceVoucherDo> wlyyDoorServiceVoucherDos = new ArrayList<>();
        if (null != wlyyDoorServiceApplicationDo) {
            if (StringUtils.isNotBlank(wlyyDoorServiceApplicationDo.getPatientAddr())) {
                p.setAddress(wlyyDoorServiceApplicationDo.getPatientAddr());
                patientDao.save(p);
            }
            //将旧的人群类型失效
            if (null != wlyyDoorServiceApplicationDo.getId()) {
                doorServiceVoucherDao.deleteByPatientCode(wlyyDoorServiceApplicationDo.getId());
            }
            //保存新的凭证
            for (WlyyDoorServiceVoucherDo vo : wlyyDoorServiceApplicationDo.getWlyyDoorServiceVoucherDos()) {
                vo.setStatus(1);
                vo.setServiceId(wlyyDoorServiceApplicationDo.getId());
                vo.setPatientCode(wlyyDoorServiceApplicationDo.getPatient());
                vo.setPatientName(wlyyDoorServiceApplicationDo.getPatientName());
                vo = doorServiceVoucherDao.save(vo);
                wlyyDoorServiceVoucherDos.add(vo);
            }
            wlyyDoorServiceApplicationDo.setWlyyDoorServiceVoucherDos(wlyyDoorServiceVoucherDos);
        }
        return wlyyDoorServiceApplicationDo;
    }

    /**
     * 开通上门服务资质评审
     *
     * @param jsonData 资质相关信息
     * @param patient  居民
     * @return
     * @throws Exception
     */
    public WlyyDoorServiceApplicationDo update(String jsonData, String patient) throws Exception {
        ObjectMapper objectMapper = new ObjectMapper();
        WlyyDoorServiceApplicationDo wlyyDoorServiceApplicationDo = objectMapper.readValue(jsonData, WlyyDoorServiceApplicationDo.class);
        if (null == wlyyDoorServiceApplicationDo.getId()) {
            throw new RuntimeException("资质申请id不能为空！");
        }
        BasePatientDO p = patientDao.findById(patient).orElse(null);
        wlyyDoorServiceApplicationDo.setProxyPatient(patient);
        wlyyDoorServiceApplicationDo.setProxyPatientName(p.getName());
        //默认先给调度员评审
        wlyyDoorServiceApplicationDo.setAuditorType(1);
        wlyyDoorServiceApplicationDo.setStatus(0);
        WlyyDoorServiceApplicationDo wlyyDoorServiceApplicationDo1 = doorServiceApplicationDao.save(wlyyDoorServiceApplicationDo);
        wlyyDoorServiceApplicationDo1.setWlyyDoorServiceVoucherDos(wlyyDoorServiceApplicationDo.getWlyyDoorServiceVoucherDos());
        wlyyDoorServiceApplicationDo1 = updateDtoAndSaveVoucher(wlyyDoorServiceApplicationDo1, p);
        return wlyyDoorServiceApplicationDo1;

    }


    //-------------------------------------------------------------------医生端--------------------------------------------------------------------------------

    /**
     * 按照医生类型及查询条件获取资质审核信息
     *
     * @param doctorCode
     * @param auditorType 医生角色：1调度员、2家医
     * @param patientName 居民姓名
     * @param mobile      居民电话
     * @param crowdType   人群类型：1失能老人，2半失能老人，3残疾人，4计生特殊家庭，5其他
     * @param status      资质状态：为空默认查找审核过的申请，0审核中，1未通过，2审核通过，3已过期
     * @param page
     * @param pageSize
     * @return
     */
    public JSONObject getPOnsiteServiceByDoctor(String doctorCode, String auditorType, String patientName, String mobile, String crowdType, String status, Integer page, Integer pageSize) throws Exception {
        JSONObject jsonObject = new JSONObject();
        if (page == null) {
            page = 1;
        }
        if (pageSize == null) {
            pageSize = 15;
        }

        StringBuilder filters = new StringBuilder();
        StringBuilder crowdTypeStr = new StringBuilder();
        //根据医生角色获取可查看居民范围
        StringBuilder doctorShowStr = new StringBuilder();
        //居民姓名
        if (StringUtils.isNotBlank(patientName)) {
            filters.append(" AND temp2.patient_name LIKE '%" + patientName + "%' ");
        }
        //居民电话
        if (StringUtils.isNotBlank(mobile)) {
            filters.append(" AND temp2.patient_phone LIKE '%" + mobile + "%' ");
        }
        //审核人类型
        if (StringUtils.isNotBlank(auditorType)) {
//            filters.append(" AND temp2.auditor_type ='" + auditorType + "' ");
            if ("1".equals(auditorType)) {
                //1调度员：可查看调度员所在机构的居民
                doctorShowStr.append(" and sf.hospital=(SELECT d.hospital from wlyy_doctor d WHERE d.code='" + doctorCode + "' and d.status=1 AND d.del=1) ");
            } else if ("2".equals(auditorType)) {
                //2家医：可查看已签约居民
                doctorShowStr.append(" and (sf.doctor='" + doctorCode + "' or sf.doctor_health='" + doctorCode + "' )");
            } else {
                //不为调度员，不为家医，不可看
                doctorShowStr.append(" and sf.patient='" + doctorCode + "'  ");
            }
        }
        //审核状态
        if (StringUtils.isNotBlank(status)) {
            filters.append(" AND temp2.status='" + status + "' ");
        } else {
            filters.append(" AND temp2.status>0");
        }
        //人群类型
        if (StringUtils.isNotBlank(crowdType)) {
            crowdTypeStr.append(" AND type='" + crowdType + "' ");
        }
        StringBuilder sql = new StringBuilder();
        StringBuilder sqlCenter = new StringBuilder();
        sql.append(" SELECT temp2.*,GROUP_CONCAT(osv.type) as type,GROUP_CONCAT(osv.type_value) as type_value,GROUP_CONCAT(osv.image) as image,COUNT(1) as ImageCount FROM ");
        //查询居民申请+人群分类+统计人群分类凭证
        sqlCenter.append(" ( SELECT temp1.*,sf.hospital_name  FROM  " +
                //根据人群类型，获取居民申请
                " (SELECT wos.code id,wos. id oid,wos.code,wos.patient,wos.patient_name,wos.patient_idcard,wos.patient_phone,wos.patient_addr,wos.application_describe,wos.start_time,wos.end_time,wos.status,wos.create_time,wos.auditor_type " +
                " FROM  (SELECT DISTINCT(patient_code) code,service_id FROM wlyy_door_service_voucher  WHERE status=1" + crowdTypeStr.toString() + ") pc LEFT JOIN wlyy_door_service_application wos  ON wos.patient=pc.code AND wos.id=pc.service_id ) temp1 " +
                //根据当前医生所在医院再次筛选申请（属于他们机构可看的申请）
                " LEFT JOIN wlyy_sign_family sf ON temp1.patient=sf.patient AND sf.status>0 " +
                " WHERE 1=1" + doctorShowStr.toString() + " ) temp2 " +
                " LEFT JOIN wlyy_door_service_voucher osv ON temp2.oid=osv.service_id  AND osv.status=1 " +
                " WHERE 1=1 " + filters.toString() +
                "  GROUP BY temp2.id ORDER BY temp2.create_time desc, osv.sort ASC ");
        String countSql = "SELECT COUNT(1)  FROM " +
                " ( SELECT  temp1.*, sf.hospital_name  FROM " +
                //根据人群类型筛选申请
                " (  SELECT wos.id, wos.patient, wos.patient_name, wos.patient_phone, wos.STATUS, wos.create_time, wos.auditor_type " +
                " FROM " +
                " ( SELECT DISTINCT (patient_code) CODE, service_id FROM  wlyy_door_service_voucher WHERE STATUS = 1 " + crowdTypeStr.toString() + ") pc LEFT JOIN wlyy_door_service_application wos ON wos.patient = pc. CODE AND wos.id = pc.service_id ) temp1 " +
                //根据当前登录人筛选居民
                " LEFT JOIN wlyy_sign_family sf ON temp1.patient = sf.patient  AND sf.status > 0 " +
                " WHERE 1=1" + doctorShowStr.toString() + ") temp2 " +
                " WHERE 1 = 1 " + filters.toString();
        Long count = jdbcTemplate.queryForObject(countSql, Long.class);
        List<Map<String, Object>> mapList = jdbcTemplate.queryForList(sql.toString() + sqlCenter.toString() + " limit " + (page - 1) * pageSize + "," + pageSize);
        for (Map<String, Object> m : mapList) {
            m.put("age", IdCardUtil.getAgeForIdcard(m.get("patient_idcard") + ""));
            m.put("sex", IdCardUtil.getSexForIdcard_new(m.get("patient_idcard") + ""));
            //图片使用逗号隔开，用逗号分离计算图片数量
            if (m.get("image") != null) {
                String arry[] = m.get("image").toString().split(",");
                m.put("ImageCount", null == arry ? 0 : arry.length);
            } else {
                m.put("ImageCount", 0);
            }

        }
        jsonObject.put("list", mapList);
        jsonObject.put("count", count);
        return jsonObject;
    }

    /**
     * 调度或审核
     *
     * @param id             申请id
     * @param doctorCode     当前医生
     * @param status         审核状态：1未通过，2审核通过
     * @param startDate      开始时间
     * @param endDate        结束时间
     * @param operationType  操作类型：1审核，2调度
     * @param failureReasons 拒绝的原因
     * @return
     */
    public WlyyDoorServiceApplicationDo auditing(String id, String doctorCode, String status, String startDate, String endDate, String operationType, String failureReasons) throws Exception {
        BaseDoctorDO doctor = doctorDao.findById(doctorCode).orElse(null);
        WlyyDoorServiceApplicationDo wlyyDoorServiceApplicationDo = doorServiceApplicationDao.findByCode(id);
        String scene = "zzbtg";
        if ("2".equals(operationType)) {
            //转给家医审核：1调度员、2家医
            wlyyDoorServiceApplicationDo.setAuditorType(2);
            wlyyDoorServiceApplicationDo.setUpdateUser(doctor.getId());
            wlyyDoorServiceApplicationDo.setUpdateUserName(doctor.getName());
            //将所有调度员的消息处理为结束
            List<SystemMessageDO> messages = systemMessageDao.findMessageByRelationCode(String.valueOf(id));
            messages.forEach(item -> {
                item.setOver("0");
            });
            systemMessageDao.saveAll(messages);
            /**
             *  在转给家医的时候需要给家医发消息
             *  todo 先注释20231011
             *  第三医院上门服务是独立的没有签约折说法
             */
//            List<SignFamily> signFamilies = signFamilyDao.getSignFamilyByPatient(wlyyDoorServiceApplicationDo.getPatient());
//            if (null != signFamilies && signFamilies.size() > 0) {
//                SignFamily signFamily = signFamilies.get(0);
//                Message message = new Message();
//                message.setCzrq(new Date());
//                message.setCreateTime(new Date());
//                message.setRead(1);
//                message.setState(1);
//                message.setRelationCode(wlyyDoorServiceApplicationDo.getCode());
//                message.setOver("1");
//                //家医
//                message.setReceiver(signFamily.getDoctor());
//                //调度员
//                message.setSender(doctor.getCode());
//                message.setCode(UUID.randomUUID().toString().replaceAll("-", ""));
//                message.setTitle("居民资质审核");
//                message.setContent(signFamily.getName() + "提交了资质申请，请及时处理");
//                //消息类型：408为资质申请审核-家医
//                message.setType(408);
//                message.setDel("1");
//                systemMessageDao.saveAll(message);
//                //调度员转接给家庭医生审核
//                doorServiceOrderService.sendWeixinMessage(2, signFamily.getDoctor(), wlyyDoorServiceApplicationDo.getPatient());
//            }

        } else {
            //处理消息
            List<SystemMessageDO> messages = systemMessageDao.findMessageByRelationCode(wlyyDoorServiceApplicationDo.getCode());
            messages.forEach(item -> {
                item.setOver("0");
            });
            systemMessageDao.saveAll(messages);
            wlyyDoorServiceApplicationDo.setStatus(Integer.valueOf(status));
            //审核状态：1未通过，2审核通过
            if ("2".equals(status)) {
                scene = "zztg";
                Date start = DateUtil.strToDate(startDate + "", "yyyy-MM-dd HH:mm:ss");
                if (null == start) {
                    throw new RuntimeException("有效起始日期格式不正确");
                }
                Date end = DateUtil.strToDate(endDate + "", "yyyy-MM-dd HH:mm:ss");
                if (null == end) {
                    throw new RuntimeException("有效截止日期格式不正确");
                }
                wlyyDoorServiceApplicationDo.setStartTime(start);
                wlyyDoorServiceApplicationDo.setEndTime(end);
            } else {
                if (StringUtils.isNotBlank(failureReasons) && failureReasons.length() > 150) {
                    throw new RuntimeException("审核不通过原因不能超过150字！");
                } else if (StringUtils.isBlank(failureReasons)) {
                    throw new RuntimeException("审核不通过原因不能为空！");
                }
            }
            wlyyDoorServiceApplicationDo.setFailureReasons(failureReasons);
            wlyyDoorServiceApplicationDo.setAuditTime(new Date());
            wlyyDoorServiceApplicationDo.setAuditorCode(doctor.getId());
            wlyyDoorServiceApplicationDo.setAuditorName(doctor.getName());
        }
        WlyyDoorServiceApplicationDo wlyyDoorServiceApplicationDo1 = doorServiceApplicationDao.save(wlyyDoorServiceApplicationDo);
        if (null != wlyyDoorServiceApplicationDo1) {
            //给代理申请资质的居民发送微信模板消息-审核结果

            
            BasePatientDO patient = patientDao.findById(wlyyDoorServiceApplicationDo1.getProxyPatient()).orElse(null);
            WechatTemplateConfig templateConfig = templateConfigDao.findByScene("template_doctor_audit", scene);
            if (null == templateConfig) {
                throw new RuntimeException("微信模板不存在！请确认");
            }
            String first = templateConfig.getFirst();
            first = first.replace("key1", (patient.getName() == null ? "" : patient.getName()));
            first = first.replace("key2", (wlyyDoorServiceApplicationDo1.getPatientName() == null ? "" : wlyyDoorServiceApplicationDo1.getPatientName()));
            String remark = templateConfig.getRemark();
            String keyword1 = templateConfig.getKeyword1();
            JSONObject sendJson = new JSONObject();
            sendJson.put("keyword1", keyword1);
            sendJson.put("keyword2", wlyyDoorServiceApplicationDo1.getAuditorName());
            sendJson.put("first", first);
            sendJson.put("remark", remark);
            //带参数的模板跳转链接
            sendJson.put("url", templateConfig.getUrl());
            pushMsgTask.sendWeixinMessage(weiXinAccessTokenUtils.getAccessToken(), 19, patient.getOpenid(), wlyyDoorServiceApplicationDo1.getProxyPatientName(), sendJson);
        }

        return wlyyDoorServiceApplicationDo1;
    }

    /**
     * 获取资质审核状态
     *
     * @param patient
     */
    public Map<String, Object> getDoorSericeApplicationStatus(String patient) {
        String sql = "SELECT p.`name`,p.code,p.address,p.mobile,h.town_name townName, " +
                "(SELECT COUNT(*) from wlyy_door_service_application a WHERE a.patient = '" + patient + "' and a.`status` = 2 and " +
                "NOW()>=a.start_time and NOW()<a.end_time ) status " +
                " from wlyy_patient p,wlyy_sign_family s,dm_hospital h " +
                "WHERE p.`code` = s.patient and s.`status`>=0 and s.hospital = h.`code` " +
                "and s.patient = '" + patient + "'";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        if (list.size() > 0) {
            return list.get(0);
        }
        return null;
    }

    public void addDoorServiceApplicationCode() {
        String sql = "select * from wlyy_door_service_application a where a.code is null order by id desc ";
        List<WlyyDoorServiceApplicationDo> list = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(WlyyDoorServiceApplicationDo.class));
        List<SystemMessageDO> messages = new ArrayList<>();
        List<SystemMessageDO> messages1 = systemMessageDao.findMessageByRelationCode2();
        for (WlyyDoorServiceApplicationDo applicationDo : list) {
            System.out.println("id====" + applicationDo.getId());
            applicationDo.setCode(UUID.randomUUID().toString().replaceAll("-", ""));

            if (messages1.size() > 0) {
                for (SystemMessageDO m : messages1) {
                    if (m.getRelationCode().equals(applicationDo.getId() + "")) {
                        m.setRelationCode(applicationDo.getCode());
                        messages.add(m);
                    }
                }
            }
        }
        doorServiceApplicationDao.saveAll(list);
        if (messages.size() > 0) {
            systemMessageDao.saveAll(messages);
        }
    }

    /**
     * 根据资质申请id获取详情
     *
     * @param code 资质申请code
     * @return
     */
    public JSONObject getPOnsiteServiceById(String code) throws Exception {
        JSONObject jsonObject = new JSONObject();
        StringBuilder sql = new StringBuilder();
        sql.append(" SELECT temp2.*, GROUP_CONCAT(osv.type) AS type, GROUP_CONCAT(osv.type_value) AS type_value, GROUP_CONCAT(osv.image) AS image, COUNT(1) AS ImageCount " +
                " FROM ( SELECT wos.id, wos.patient, wos.patient_name, wos.patient_idcard, wos.patient_phone, wos.patient_addr," +
                " wos.application_describe, wos.start_time,wos.end_time, wos.status, wos.create_time, wos.failure_reasons, wos.auditor_type, sf.hospital_name FROM wlyy_door_service_application wos " +
                " LEFT JOIN wlyy_sign_family sf ON wos.patient = sf.patient AND sf. STATUS > 0 WHERE wos.code='" + code + "' " +
                " ) temp2  LEFT JOIN wlyy_door_service_voucher osv ON temp2.id = osv.service_id AND osv. STATUS = 1 ");
        List<Map<String, Object>> mapList = jdbcTemplate.queryForList(sql.toString());
        for (Map<String, Object> m : mapList) {
            m.put("age", IdCardUtil.getAgeForIdcard(m.get("patient_idcard") + ""));
            m.put("sex", IdCardUtil.getSexForIdcard_new(m.get("patient_idcard") + ""));
            m.put("application_describe", null == m.get("application_describe") ? "" : m.get("application_describe").toString());
            m.put("patient_addr", null == m.get("patient_addr") ? "" : m.get("patient_addr").toString());
            m.put("patient_phone", null == m.get("patient_phone") ? "" : m.get("patient_phone").toString());
            m.put("failure_reasons", null == m.get("failure_reasons") ? "" : m.get("failure_reasons").toString());
            //图片使用逗号隔开，用逗号分离计算图片数量
            if (m.get("image") != null) {
                String arry[] = m.get("image").toString().split(",");
                m.put("ImageCount", null == arry ? 0 : arry.length);
            } else {
                m.put("ImageCount", 0);
            }
            BasePatientDO patient = patientDao.findById(m.get("patient").toString()).orElse(null);
            m.put("photo", null == patient ? "" : patient.getPhoto());
        }
        jsonObject.put("list", mapList);
        return jsonObject;
    }


}
