package com.yihu.jw.hospital.module.door.service;

import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.entity.door.WlyyDoorDoctorFeeDictDO;
import com.yihu.jw.hospital.module.door.dao.WlyyDoorDoctorFeeDictDao;
import com.yihu.jw.hospital.utils.EntityUtils;
import com.yihu.jw.restmodel.ResponseContant;
import com.yihu.mysql.query.BaseJpaService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * 
 * 医生出诊费用字典服务service
 * 
 * @version 
 * <pre>
 * Author	Version		Date		Changes
 * Administrator    1.0  2019年03月20日 Created
 *
 * </pre>
 * @since 1.
 */
@Service
public class WlyyDoorDoctorFeeDictService extends BaseJpaService<WlyyDoorDoctorFeeDictDO, WlyyDoorDoctorFeeDictDao> {

    private Logger logger = LoggerFactory.getLogger(WlyyDoorDoctorFeeDictService.class);

    @Autowired
    private JdbcTemplate jdbcTemplate;


    //医生出诊费用字典服务基本信息
    private WlyyDoorDoctorFeeDictDO jsonToWlyyDoorDoctorFee(JSONObject result, JSONObject jsonObjectParam, String jsonData) {
        WlyyDoorDoctorFeeDictDO wlyyDoorDoctorFee;
        try {
            wlyyDoorDoctorFee = EntityUtils.jsonToEntity(jsonObjectParam.get("wlyyDoorDoctorFee").toString(), WlyyDoorDoctorFeeDictDO.class);
        } catch (Exception e) {
            result.put(ResponseContant.resultFlag,ResponseContant.fail);
            String failMsg = "医生出诊费用字典服务基本信息："  + e.getMessage();
            result.put(ResponseContant.resultMsg,failMsg);
            logger.error(failMsg);
            return new WlyyDoorDoctorFeeDictDO();
        }

        this.save(wlyyDoorDoctorFee);
        return wlyyDoorDoctorFee;
    }

     /**
     * 创建医生出诊费用字典
     * @param jsonData
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public JSONObject create(String jsonData){
        JSONObject result = new JSONObject();
        JSONObject jsonObjectParam ;

        try {
            jsonObjectParam = JSONObject.parseObject(jsonData);
        } catch (Exception e) {
            result.put(ResponseContant.resultFlag,ResponseContant.fail);
            String failMsg = "参数转换成JSON对象异常：" + e.getMessage();
            result.put(ResponseContant.resultMsg,failMsg);
            logger.error(failMsg);
            return result;
        }

        WlyyDoorDoctorFeeDictDO wlyyDoorDoctorFee = new WlyyDoorDoctorFeeDictDO();
        wlyyDoorDoctorFee = jsonToWlyyDoorDoctorFee(result, jsonObjectParam,jsonData);

        result.put(ResponseContant.resultFlag,ResponseContant.success);
        result.put(ResponseContant.resultMsg,wlyyDoorDoctorFee);
        return result;
    }

    /**
     * 医生出诊费用字典（包括角色分配的菜单，模块等）
     * @param jsonData
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public JSONObject update(String jsonData){
        JSONObject result = new JSONObject();
        JSONObject jsonObjectParam ;

        try {
            jsonObjectParam = JSONObject.parseObject(jsonData);
        } catch (Exception e) {
            result.put(ResponseContant.resultFlag,ResponseContant.fail);
            String failMsg = "参数转换成JSON对象异常：" + e.getMessage();
            result.put(ResponseContant.resultMsg,failMsg);
            logger.error(failMsg);
            return result;
        }

        WlyyDoorDoctorFeeDictDO wlyyDoorDoctorFee = new WlyyDoorDoctorFeeDictDO();
        wlyyDoorDoctorFee = jsonToWlyyDoorDoctorFee(result, jsonObjectParam,jsonData);

        result.put(ResponseContant.resultFlag,ResponseContant.success);
        result.put(ResponseContant.resultMsg,wlyyDoorDoctorFee);
        return result;
    }


   /**
     * 查询医生出诊费用字典列表
     *
     * @return
     */
    public JSONObject queryInfoList(String fields, String filters, String sorts, int page, int size) {
        JSONObject result = new JSONObject();

        int start = 0 == page ? page++ : (page - 1) * size;
        int end = 0 == size ? 15 : page * size;

        String sql = "";

        String finalSql = sql.replace("{fields}", fields)
                .replace("{start}", String.valueOf(start))
                .replace("{end}", String.valueOf(end));

        String countSql = "";

        String finqlCountSql = countSql.replace("{filters}", filters)
                .replace("{start}", String.valueOf(start));

        List<WlyyDoorDoctorFeeDictDO> sqlResultlist = new ArrayList<>();
    try {
        sqlResultlist = jdbcTemplate.query(finalSql, new BeanPropertyRowMapper<>(WlyyDoorDoctorFeeDictDO.class));
    } catch (Exception e) {
        result.put(ResponseContant.resultFlag, ResponseContant.fail);
        result.put(ResponseContant.resultMsg, "从数据库医生出诊费用字典列表信息失败：" + e.getMessage());
        return result;
    }

    Integer count = 0;
    try {
        count = jdbcTemplate.queryForObject(finqlCountSql, Integer.class);
    } catch (Exception e) {
        result.put(ResponseContant.resultFlag, ResponseContant.fail);
        result.put(ResponseContant.resultMsg, "从数据库统计医生出诊费用字典数量失败：" + e.getMessage());
        return result;
    }

    result.put(ResponseContant.resultFlag, ResponseContant.success);
    result.put(ResponseContant.resultMsg, sqlResultlist);
    JSONObject countItem = new JSONObject();
    countItem.put("count", count);
    result.putAll(countItem);

    return result;
    }
}
