package com.yihu.jw.hospital.module.door.service;

import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.entity.door.WlyyDoorTransRecordDO;
import com.yihu.jw.hospital.module.door.dao.WlyyDoorTransRecordDao;
import com.yihu.jw.hospital.utils.EntityUtils;
import com.yihu.jw.restmodel.ResponseContant;
import com.yihu.mysql.query.BaseJpaService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * 服务工单转接给其他结构的转接记录服务service
 *
 * @version <pre>
 * Author	Version		Date		Changes
 * Administrator    1.0  2019年03月20日 Created
 *
 * </pre>
 * @since 1.
 */
@Service
public class WlyyDoorTransRecordService extends BaseJpaService<WlyyDoorTransRecordDO, WlyyDoorTransRecordDao> {

    private Logger logger = LoggerFactory.getLogger(WlyyDoorTransRecordService.class);

    @Autowired
    private JdbcTemplate jdbcTemplate;


    //服务工单转接给其他结构的转接记录服务基本信息
    private WlyyDoorTransRecordDO jsonToWlyyDoorTransRecord(JSONObject result, JSONObject jsonObjectParam, String jsonData) {
        WlyyDoorTransRecordDO wlyyDoorTransRecord;
        try {
            wlyyDoorTransRecord = EntityUtils.jsonToEntity(jsonObjectParam.get("wlyyDoorTransRecord").toString(), WlyyDoorTransRecordDO.class);
        } catch (Exception e) {
            result.put(ResponseContant.resultFlag, ResponseContant.fail);
            String failMsg = "服务工单转接给其他结构的转接记录服务基本信息：" + e.getMessage();
            result.put(ResponseContant.resultMsg, failMsg);
            logger.error(failMsg);
            return new WlyyDoorTransRecordDO();
        }

        this.save(wlyyDoorTransRecord);
        return wlyyDoorTransRecord;
    }

    /**
     * 创建服务工单转接给其他结构的转接记录
     *
     * @param jsonData
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public JSONObject create(String jsonData) {
        JSONObject result = new JSONObject();
        JSONObject jsonObjectParam;

        try {
            jsonObjectParam = JSONObject.parseObject(jsonData);
        } catch (Exception e) {
            result.put(ResponseContant.resultFlag, ResponseContant.fail);
            String failMsg = "参数转换成JSON对象异常：" + e.getMessage();
            result.put(ResponseContant.resultMsg, failMsg);
            logger.error(failMsg);
            return result;
        }

        WlyyDoorTransRecordDO wlyyDoorTransRecord = new WlyyDoorTransRecordDO();
        wlyyDoorTransRecord = jsonToWlyyDoorTransRecord(result, jsonObjectParam, jsonData);

         /*
        //新增#######关联关系
        if (roleWithMenu(result, jsonObjectParam, wlyyDoorTransRecord)) return result;

        //新增#######关联关系
        if (roleWithFunction(result, jsonObjectParam, wlyyDoorTransRecord)) return result;
        */
        result.put(ResponseContant.resultFlag, ResponseContant.success);
        result.put(ResponseContant.resultMsg, wlyyDoorTransRecord);
        return result;
    }

    /**
     * 服务工单转接给其他结构的转接记录（包括角色分配的菜单，模块等）
     *
     * @param jsonData
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public JSONObject update(String jsonData) {
        JSONObject result = new JSONObject();
        JSONObject jsonObjectParam;

        try {
            jsonObjectParam = JSONObject.parseObject(jsonData);
        } catch (Exception e) {
            result.put(ResponseContant.resultFlag, ResponseContant.fail);
            String failMsg = "参数转换成JSON对象异常：" + e.getMessage();
            result.put(ResponseContant.resultMsg, failMsg);
            logger.error(failMsg);
            return result;
        }

        WlyyDoorTransRecordDO wlyyDoorTransRecord = new WlyyDoorTransRecordDO();
        wlyyDoorTransRecord = jsonToWlyyDoorTransRecord(result, jsonObjectParam, jsonData);

        /*
        //修改时有更改到关联关系，则先删除原有关联关系，再新增关联关系
        //删除#######关联关系
        List relatedList1 = new ArrayList();
        roleMenuList = baseRoleMenuService.findByField("roleId",wlyyDoorTransRecord.getId());
        Set roleMenuIdSet = new HashSet();
        if(!CollectionUtils.isEmpty(roleMenuList)){
            roleMenuList.forEach(one ->{roleMenuIdSet.add(one);});
        }
        baseRoleMenuService.delete(roleMenuIdSet);
        //新增#######关联关系
        if (roleWithMenu(result, jsonObjectParam, wlyyDoorTransRecord)) return result;

        //删除#######关联关系
        List relatedList2 = new ArrayList();
        roleFunctionList = baseRoleMenuFunctionService.findByField("roleId",wlyyDoorTransRecord.getId());
        Set roleFunctionIdSet = new HashSet();
        if(!CollectionUtils.isEmpty(roleFunctionList)){
            roleFunctionList.forEach(one ->{roleFunctionIdSet.add(one);});
        }
        baseRoleMenuFunctionService.delete(roleFunctionIdSet);
        //新增#######关联关系
        if (roleWithFunction(result, jsonObjectParam, wlyyDoorTransRecord)) return result;
        */

        result.put(ResponseContant.resultFlag, ResponseContant.success);
        result.put(ResponseContant.resultMsg, wlyyDoorTransRecord);
        return result;
    }


    /**
     * 查询服务工单转接给其他结构的转接记录列表
     *
     * @return
     */
    public JSONObject queryInfoList(String fields, String filters, String sorts, Integer status, int page, int size) {
        JSONObject result = new JSONObject();

        status = null == status ? -1 : status;
        int start = 0 == page ? page++ : (page - 1) * size;
        int end = 0 == size ? 15 : page * size;

        String sql = "";

        String finalSql = sql.replace("{fields}", fields)
                .replace("{status}", status.toString())
                .replace("{start}", String.valueOf(start))
                .replace("{end}", String.valueOf(end));

        String countSql = "";

        String finqlCountSql = countSql.replace("{filters}", filters)
                .replace("{status}", status.toString())
                .replace("{start}", String.valueOf(start));

        List<WlyyDoorTransRecordDO> sqlResultlist = new ArrayList<>();
        try {
            sqlResultlist = jdbcTemplate.query(finalSql, new BeanPropertyRowMapper<>(WlyyDoorTransRecordDO.class));
        } catch (Exception e) {
            result.put(ResponseContant.resultFlag, ResponseContant.fail);
            result.put(ResponseContant.resultMsg, "从数据库服务工单转接给其他结构的转接记录列表信息失败：" + e.getMessage());
            return result;
        }

        Integer count = 0;
        try {
            count = jdbcTemplate.queryForObject(finqlCountSql, Integer.class);
        } catch (Exception e) {
            result.put(ResponseContant.resultFlag, ResponseContant.fail);
            result.put(ResponseContant.resultMsg, "从数据库统计服务工单转接给其他结构的转接记录数量失败：" + e.getMessage());
            return result;
        }

        result.put(ResponseContant.resultFlag, ResponseContant.success);
        result.put(ResponseContant.resultMsg, sqlResultlist);
        JSONObject countItem = new JSONObject();
        countItem.put("count", count);
        result.putAll(countItem);

        return result;
    }
}
