package com.yihu.jw.hospital.module.rehabilitation.controller;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.entity.specialist.rehabilitation.PatientDiagnosisInformationDO;
import com.yihu.jw.hospital.module.rehabilitation.service.DiagnosisInformationService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.specialist.SpecialistMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.text.SimpleDateFormat;

/**
 * Created by humingfen on 2019/2/11.
 */
@RestController
@RequestMapping(SpecialistMapping.api_specialist_common)
@Api(tags = "诊疗信息管理相关操作", description = "诊疗信息相关操作")
public class DiagnosisInformationController extends EnvelopRestEndpoint {

    @Autowired
    private DiagnosisInformationService diagnosisInformationService;


    @PostMapping(value = SpecialistMapping.diagnosisInformation.createDiagnosisInformation)
    @ApiOperation(value = "诊疗信息创建")
    public Envelop createDiagnosisInformation(@ApiParam(name = "diagnosisInformationJson", value = "实体JSON")
                                              @RequestParam(value = "diagnosisInformationJson") String diagnosisInformationJson) {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            objectMapper.setDateFormat(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"));
            PatientDiagnosisInformationDO diagnosisInformationDO = objectMapper.readValue(diagnosisInformationJson, PatientDiagnosisInformationDO.class);
            return diagnosisInformationService.createDiagnosisInformation(diagnosisInformationDO);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError(e.getMessage());
        }
    }

    @GetMapping(value = SpecialistMapping.diagnosisInformation.findDiagnosisInformationByPlanId)
    @ApiOperation(value = "根据康复计划id查找诊疗信息")
    public Envelop findDiagnosisInformationByPlanId(@ApiParam(name = "planId", value = "康复计划id")
                                                    @RequestParam(value = "planId") String planId) {
        try {
            return diagnosisInformationService.findDiagnosisInformationByPlanId(planId);
        } catch (Exception e) {
            e.printStackTrace();
           
            return ObjEnvelop.getError(e.getMessage());
        }
    }

    @GetMapping(value = SpecialistMapping.diagnosisInformation.findDiagnosisInformationByPatient)
    @ApiOperation(value = "查找居民诊疗信息记录")
    public Envelop findDiagnosisInformationByPatient(@ApiParam(name = "patient", value = "居民code")
                                                     @RequestParam(value = "patient") String patient,
                                                     @ApiParam(name = "page", value = "第几页")
                                                     @RequestParam(value = "page") Integer page,
                                                     @ApiParam(name = "size", value = "行数")
                                                     @RequestParam(value = "size") Integer size) {
        try {
            return diagnosisInformationService.findDiagnosisInformationByPatient(patient, page, size);
        } catch (Exception e) {
            e.printStackTrace();
           
            return ObjEnvelop.getError(e.getMessage());
        }
    }
}
