package com.yihu.jw.hospital.module.rehabilitation.controller;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.entity.specialist.rehabilitation.PatientMedicalRecordsDO;
import com.yihu.jw.entity.specialist.rehabilitation.RehabilitationAdviceDO;
import com.yihu.jw.hospital.module.rehabilitation.dao.RehabilitationPatientInfoDao;
import com.yihu.jw.hospital.module.rehabilitation.service.RehabilitationInfoService;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.specialist.SpecialistMapping;
import com.yihu.jw.util.entity.ServiceException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping(value = "/doctor/rehabilitation", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(description = "医生端-康复下转管理")
public class DoctorRehabilitaionInfoController extends EnvelopRestEndpoint {

    @Autowired
    private RehabilitationInfoService rehabilitationInfoService;
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private RehabilitationPatientInfoDao rehabilitationPatientInfoDao;

    /**
     * 查询患者基本信息
     */
    @GetMapping(value = "getPatientInfo")
    @ApiOperation(value = "查询患者基本信息")
    public MixEnvelop getPatientInfo(
            @ApiParam(name = "patientId", value = "居民id") @RequestParam(value = "patientId", required = false) String patientId,
            @ApiParam(name = "idcard", value = "身份证") @RequestParam(value = "idcard", required = false) String idcard
    ) {
        try {
            List<Map<String, Object>> list = rehabilitationInfoService.getPatientInfo(patientId, idcard);
            return MixEnvelop.getSuccess(SpecialistMapping.api_success, list);
        } catch (Exception e) {
            e.printStackTrace();
            return MixEnvelop.getError("查询失败");
        }
    }


    /**
     * status
     * 是否分配下转：0未分配，1已分配(已下转列表)，2创建计划过程失败，3患者无住院病历信息(等待重新下转) 5同步居民失败
     */
    @RequestMapping(value = "getRehabilitationPatientListNew", method = RequestMethod.GET)
    @ApiOperation("获取康复下转居民列表")
    public String getRehabilitationPatientListNew(
            @ApiParam(name = "downPatientId", value = "下转表id", required = false) @RequestParam(value = "downPatientId", required = false) String downPatientId,
            @ApiParam(name = "doctorId", value = "医生id", required = false) @RequestParam(value = "doctorId", required = false) String doctorId,
            @ApiParam(name = "idcard", value = "身份证", required = false) @RequestParam(value = "idcard", required = false) String idcard,
            @ApiParam(name = "status", value = "", defaultValue = "0", required = false) @RequestParam(value = "status", required = false) String status,
            @ApiParam(name = "page", value = "起始页", required = false) @RequestParam(value = "page", required = false, defaultValue = "1") Integer page,
            @ApiParam(name = "pagesize", value = "每页显示数据条数", required = false) @RequestParam(value = "pagesize", required = false, defaultValue = "10") Integer pagesize
    ) {
        try {
            if (page == null) {
                page = 1;
            }
            if (pagesize == null) {
                pagesize = 10;
            }
            if (StringUtils.isBlank(doctorId)) {
                doctorId = getUID();
                System.out.println("医生id==>getUID()==>" + getUID());
            }
            JSONObject result = rehabilitationInfoService.getRehabilitationPatientListNew( downPatientId,idcard, doctorId, status, page, pagesize);
            return write(200, "请求成功", "data", result);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "操作失败");
        }
    }


    @RequestMapping(value = "getRehabilitationPatientCountNew", method = RequestMethod.GET)
    @ApiOperation("康复下转状态数量")
    public String getRehabilitationPatientListNew(
            @ApiParam(name = "doctorId", value = "医生id", required = false) @RequestParam(value = "doctorId", required = false) String doctorId
    ) {
        try {
            if (StringUtils.isBlank(doctorId)) {
                doctorId = getUID();
                System.out.println("医生id==>getUID()==>" + getUID());
            }
            List<Map<String, Object>> list = rehabilitationInfoService.getRehabilitationPatientCountNew(doctorId);
            return write(200, "请求成功", "data", list);
        } catch (Exception e) {
            return error(-1, "查询失败");
        }
    }


    /**
     * 查询科室和医生
     */
    @RequestMapping(value = "getDetpAndDoctor", method = RequestMethod.GET)
    @ApiOperation("获取康复下转居民列表")
    public String getDetpAndDoctor(
            @ApiParam(name = "type", value = "类型1科室2医生", required = false) @RequestParam(value = "type", required = false) String type,
            @ApiParam(name = "disease", value = "疾病", required = false) @RequestParam(value = "disease", required = false) String disease,
            @ApiParam(name = "doctorName", value = "医生名字", required = false) @RequestParam(value = "doctorName", required = false) String doctorName,
            @ApiParam(name = "deptCode", value = "科室code", required = false) @RequestParam(value = "deptCode", required = false) String deptCode,
            @ApiParam(name = "deptName", value = "科室", required = false) @RequestParam(value = "deptName", required = false) String deptName) {
        try {
            List<Map<String, Object>> list = rehabilitationInfoService.getDetpAndDoctor(doctorName, deptCode, deptName, type, disease);
            return write(200, "请求成功", "data", list);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "操作失败");
        }
    }


    @RequestMapping(value = "createAdvice", method = RequestMethod.POST)
    @ApiOperation("创建/编辑康复建议模板")
    public String createAdvice(
            @ApiParam(name = "doctorCode", value = "医生code", required = false) @RequestParam(value = "doctorCode", required = false) String doctorCode,
            @ApiParam(name = "code", value = "模板code") @RequestParam(value = "code", required = false) String code,
            @ApiParam(name = "name", value = "模板名称", required = true) @RequestParam(value = "name", required = true) String name,
            @ApiParam(name = "content", value = "模板内容", required = true) @RequestParam(value = "content", required = true) String content
    ) {
        try {
            if (StringUtils.isBlank(doctorCode)) {
                doctorCode = getUID();
            }
            rehabilitationInfoService.createAdvice(doctorCode, code, name, content);
            return write(200, "更新成功");
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "操作失败");
        }
    }

    @RequestMapping(value = "deleteAdvice", method = RequestMethod.POST)
    @ApiOperation("删除康复建议模板")
    public String deleteAdvice(
            @ApiParam(name = "id", value = "模板id") @RequestParam(value = "id", required = false) Long id) {
        try {
            rehabilitationInfoService.deleteAdvice(id);
            return write(200, "删除成功");
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "操作失败");
        }
    }

    @RequestMapping(value = "getAdviceListByDoctor", method = RequestMethod.GET)
    @ApiOperation("获取医生创建康复建议模板及系统模板")
    public String getAdviceListByDoctor(
            @ApiParam(name = "doctorCode", value = "医生code", required = false) @RequestParam(value = "doctorCode", required = false) String doctorCode) {
        try {
            if (StringUtils.isBlank(doctorCode)) {
                doctorCode = getUID();
            }
            List<RehabilitationAdviceDO> list = rehabilitationInfoService.getAdviceListByDoctor(doctorCode);
            return write(200, "请求成功", "data", list);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "操作失败");
        }
    }


    /**
     * 康复管理
     * 获取所有居民的康复计列表
     * todo 还需要代办项目还需要重新join一张新表
     */
    @RequestMapping(value = "getRehabilitationPatientPlan", method = RequestMethod.GET)
    @ApiOperation("获取所有居民的康复计列表")
    public String getRehabilitationPatientPlan(
            @ApiParam(name = "idcard", value = "身份证", required = false) @RequestParam(value = "idcard", required = false) String idcard,
            @ApiParam(name = "doctorId", value = "医生id", required = false) @RequestParam(value = "doctorId", required = false) String doctorId,
            @ApiParam(name = "patientId", value = "居民id", required = false) @RequestParam(value = "patientId", required = false) String patientId,
            @ApiParam(name = "finishStatus", value = "空全部 1未完成 2已完成", required = false) @RequestParam(value = "finishStatus", required = false) String finishStatus,
            @ApiParam(name = "page", value = "起始页", required = false) @RequestParam(value = "page", required = false, defaultValue = "1") Integer page,
            @ApiParam(name = "pagesize", value = "每页显示数据条数", required = false) @RequestParam(value = "pagesize", required = false, defaultValue = "10") Integer pagesize
    ) {
        try {
            if (StringUtils.isBlank(doctorId)) {
                doctorId = getUID();
                System.out.println("医生id==>getUID()==>" + getUID());
            }
            System.out.println("参数idcard：" + idcard);
            System.out.println("参数patientId：" + patientId);
            System.out.println("参数finishStatus：" + finishStatus);
            System.out.println("参数page：" + page);
            System.out.println("参数pagesize：" + pagesize);
            List<Map<String, Object>> result = rehabilitationInfoService.getRehabilitationPatientPlan(idcard, doctorId, patientId, finishStatus, page, pagesize);
            return write(200, "请求成功", "data", result);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "操作失败");
        }
    }


    /**
     * 康复管理
     * 获取所有居民的康复计列表数量统计
     * todo 还需要代办项目还需要重新join一张新表
     */
    @RequestMapping(value = "getRehabilitationPatientPlanCount", method = RequestMethod.GET)
    @ApiOperation("获取所有居民的康复计列表")
    public String getRehabilitationPatientPlanCount(
            @ApiParam(name = "doctorId", value = "医生id", required = false) @RequestParam(value = "doctorId", required = false) String doctorId
    ) {
        try {
            if (StringUtils.isBlank(doctorId)) {
                doctorId = getUID();
                System.out.println("医生id==>getUID()==>" + getUID());
            }
            HashMap<String, Object> result = rehabilitationInfoService.getRehabilitationPatientPlanCount(doctorId);
            return write(200, "请求成功", "data", result);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "操作失败");
        }
    }

    @RequestMapping(value = "findRehabilitationPatientInfo", method = RequestMethod.GET)
    @ApiOperation("查询下转居民信息")
    public String findRehabilitationPatientInfo(
            @ApiParam(name = "patientInfoCode", value = "下转居民信息code", required = true)
            @RequestParam(value = "patientInfoCode", required = true) String patientInfoCode
    ) {
        try {
            return write(200, "请求成功", "data", rehabilitationPatientInfoDao.findByCode(patientInfoCode));
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "请求失败");
        }
    }

    /**
     * 查询居民的康复计划
     * todo 还需要代办项目还需要重新join一张新表 todayTaskCount 今日任务项，等后面在整
     */
    @RequestMapping(value = "getPatientRehabilitationByPatientId", method = RequestMethod.GET)
    @ApiOperation("查询居民的康复计划")
    public String getPatientRehabilitationByPatientId(
            @ApiParam(name = "patientId", value = "居民", required = false) @RequestParam(value = "patientId", required = false) String patientId,
            @ApiParam(name = "planId", value = "计划id", required = false) @RequestParam(value = "planId", required = false) String planId
    ) {
        try {
            List<Map<String, Object>> result = rehabilitationInfoService.getPatientRehabilitationByPatientId(patientId,planId);
            return write(200, "请求成功", "data", result);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "操作失败");
        }
    }

    /**
     * 查询病例信息
     */
    @RequestMapping(value = "getPatientMedicalRecordsByPlanId", method = RequestMethod.GET)
    @ApiOperation("查询居民的康复计划")
    public String getPatientMedicalRecordsByPlanId(
            @ApiParam(name = "planId", value = "计划id", required = false) @RequestParam(value = "planId", required = false) String planId
    ) {
        try {
            List<PatientMedicalRecordsDO> result = rehabilitationInfoService.getPatientMedicalRecordsByPlanId(planId);
            return write(200, "请求成功", "data", result);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "操作失败");
        }
    }


    /**
     * 查询康复计划服务项
     */
    @RequestMapping(value = "getPatientPlanItemList", method = RequestMethod.GET)
    @ApiOperation("查询居民的康复计划")
    public String getPatientPlanItemList(
            @ApiParam(name = "patientId", value = "居民", required = false) @RequestParam(value = "patientId", required = false) String patientId,
            @ApiParam(name = "planId", value = "计划id", required = false) @RequestParam(value = "planId", required = false) String planId,
            @ApiParam(name = "itemId", value = "服务项id", required = false) @RequestParam(value = "itemId", required = false) String itemId,
            @ApiParam(name = "status", value = "状态  0未完成，1已完成，2已预约，3待补录", required = false) @RequestParam(value = "status", required = false) String status
    ) {
        try {
            List<Map<String, Object>> result = rehabilitationInfoService.getPatientPlanItemList(patientId, planId, itemId, status);
            return write(200, "请求成功", "data", result);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "查询失败");
        }
    }


    /**
     * 获取频次服务的信息
     * 这个服务项次数还没有-暂时没有
     */
    @RequestMapping(value = "getPatientPlanFrequencyItemList", method = RequestMethod.GET)
    @ApiOperation("获取频次服务的信息")
    public String getPatientPlanFrequencyItemList(
            @ApiParam(name = "patientId", value = "居民", required = false) @RequestParam(value = "patientId", required = false) String patientId,
            @ApiParam(name = "planId", value = "计划id", required = false) @RequestParam(value = "planId", required = false) String planId,
            @ApiParam(name = "startTime", value = "开始时间", required = false) @RequestParam(value = "startTime", required = false) String startTime,
            @ApiParam(name = "endTime", value = "结束时间", required = false) @RequestParam(value = "endTime", required = false) String endTime,
            @ApiParam(name = "doctorId", value = "医生id", required = false) @RequestParam(value = "doctorId", required = false) String doctorId,
            @ApiParam(name = "type", value = "1全部 2奔人 3 他人", required = false) @RequestParam(value = "type", required = false) String type
    ) {
        try {
            List<Map<String, Object>> result = rehabilitationInfoService.getPatientPlanFrequencyItemList(patientId, planId, startTime, endTime, doctorId, type);
            return write(200, "请求成功", "data", result);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "查询失败");
        }
    }


    /**
     * 获取康复项发服务医生
     */
    @RequestMapping(value = "getPlanDoctorList", method = RequestMethod.GET)
    @ApiOperation("获取频次服务的信息")
    public String getPlanDoctorList(
            @ApiParam(name = "planId", value = "计划id", required = false) @RequestParam(value = "planId", required = false) String planId

    ) {
        try {
            List<Map<String, Object>> result = rehabilitationInfoService.getPlanDoctorList(planId);
            return write(200, "请求成功", "data", result);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "查询失败");
        }
    }


    //=================================================没有用到的======================================================================
    @RequestMapping(value = "getRehabilitationPatientList", method = RequestMethod.GET)
    @ApiOperation("获取康复管理居民列表")
    public String getRehabilitationPatientList(
            @ApiParam(name = "doctorId", value = "医生id", required = false) @RequestParam(value = "doctorId", required = false) String doctorId,
            @ApiParam(name = "idcard", value = "身份证", required = false) @RequestParam(value = "idcard", required = false) String idcard,
            @ApiParam(name = "isTurnDown", value = "是否已下转 -2:未下转未家签，-1未下转 0未下转全部，;1已下转(未接收);2已下转(已接收);3已下转全部;03已下转全部+未下转全部", defaultValue = "0", required = false) @RequestParam(value = "isTurnDown", required = false, defaultValue = "3") String isTurnDown,
            @ApiParam(name = "page", value = "起始页", required = false) @RequestParam(value = "page", required = false, defaultValue = "1") Integer page,
            @ApiParam(name = "pagesize", value = "每页显示数据条数", required = false) @RequestParam(value = "pagesize", required = false, defaultValue = "10") Integer pagesize
    ) {
        try {
            if (page == null) {
                page = 1;
            }
            if (pagesize == null) {
                pagesize = 10;
            }
            if (StringUtils.isBlank(doctorId)) {
                doctorId = getUID();
                System.out.println("获取康复下转居民列表--医生id==>getUID()==>" + getUID());
            }
            JSONObject result = rehabilitationInfoService.getRehabilitationPatientList(idcard, doctorId, isTurnDown, page, pagesize);
            return write(200, "请求成功", "data", result);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "操作失败");
        }
    }


    @RequestMapping(value = "getDeorsumvergenceCount", method = RequestMethod.GET)
    @ApiOperation("康复下转已下转顶头数据")
    public String getDeorsumvergenceCount() {
        try {
            JSONObject result = rehabilitationInfoService.getDeorsumvergenceCount(getUID());
            return write(200, "请求成功", "data", result);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "操作失败");
        }
    }


    @RequestMapping(value = "ttt", method = RequestMethod.GET)
    @ApiOperation("添加编辑住院病历")
    public String ttt() {
        try {
            return write(200, "添加成功", "data", rehabilitationInfoService.findByCode("2c53ebdec84747d48526abc40702c395"));
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "添加失败");
        }
    }

    @RequestMapping(value = "createMedicalRecords", method = RequestMethod.POST)
    @ApiOperation("添加编辑住院病历")
    public String createMedicalRecords(
            @ApiParam(name = "doctorCode", value = "医生code", required = false) @RequestParam(value = "doctorCode", required = false) String doctorCode,
            @ApiParam(name = "patient", value = "居民code") @RequestParam String patient,
            @ApiParam(name = "patientName", value = "居民姓名") @RequestParam String patientName,
            @ApiParam(name = "jsonData", value = "住院病历相关数据") @RequestParam(value = "jsonData", required = true) String jsonData,
            @ApiParam(name = "id", value = "模板id，编辑时传入") @RequestParam(value = "id", required = false) Long id
    ) {
        try {
            if (StringUtils.isBlank(doctorCode)) {
                doctorCode = getUID();
            }
            objectMapper.setDateFormat(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"));
            PatientMedicalRecordsDO patientMedicalRecordsDO = objectMapper.readValue(jsonData, PatientMedicalRecordsDO.class);
            return write(200, "添加成功", "data", rehabilitationInfoService.createMedicalRecords(doctorCode, patient, patientName, patientMedicalRecordsDO, id));
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "添加失败");
        }
    }


    @RequestMapping(value = "getMedicalHistoryList", method = RequestMethod.GET)
    @ApiOperation("获取厦心出院就诊记录信息")
    public String getMedicalHistoryList(
            @ApiParam(name = "code", value = "居民code", required = true) @RequestParam(value = "code", required = true) String code,
            @ApiParam(name = "page", value = "page", required = true) @RequestParam(value = "page", required = true) Integer page,
            @ApiParam(name = "size", value = "size", required = true) @RequestParam(value = "size", required = true) Integer size) {
        try {
            List<Map<String, Object>> list = rehabilitationInfoService.getMedicalHistoryList(code, page, size);
            return write(200, "请求成功", "data", list);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "操作失败");
        }
    }


    @RequestMapping(value = "getDischargePatientList", method = RequestMethod.GET)
    @ApiOperation("获取即将出院居民列表，区分家签非家签")
    public String getDischargePatientList(@ApiParam(name = "hospital", value = "机构code", required = false)
                                          @RequestParam(value = "hospital", required = true) String hospital) {

        try {
            JSONObject result = rehabilitationInfoService.getDischargePatientList(hospital, getUID());
            return write(200, "请求成功", "data", result);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "操作失败");
        }
    }

    @RequestMapping(value = "getPatientByIdcardOrSsc", method = RequestMethod.POST)
    @ApiOperation("根据患者身份证或社保卡号进行搜索添加出院记录")
    public String getPatientByIdcardOrSsc(@ApiParam(name = "patientInfo", value = "居民身份证或者社保卡", required = true)
                                          @RequestParam(value = "patientInfo") String patientInfo) {
        try {

            JSONObject object = rehabilitationInfoService.getPatientByIdcardOrSsc(patientInfo, getUID(), 1);
            if (object.getInteger("status") == 200) {
                return write(200, "添加成功！", "data", object.get("data"));
            } else {
                return write(-1, "添加失败！", "data", object.getString("msg"));
            }
        } catch (ServiceException se) {
            return error(-1, se.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "操作失败");
        }
    }

}
