package com.yihu.jw.hospital.module.rehabilitation.controller;


import com.yihu.jw.entity.specialist.rehabilitation.PatientRehabilitationPlanDO;
import com.yihu.jw.entity.specialist.rehabilitation.RehabilitationOperateRecordsDO;
import com.yihu.jw.hospital.module.rehabilitation.service.RehabilitationManageService;
import com.yihu.jw.hospital.module.rehabilitation.service.SynchronizePatientService;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.specialist.SpecialistMapping;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.entity.ServiceException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.Map;


/**
 * Created by 刘文彬 on 2018/8/16.
 */
@RestController
@RequestMapping(SpecialistMapping.api_specialist_common)
@Api(tags = "康复计划管理相关操作", description = "康复计划管理相关操作")
public class RehabilitationManageController extends EnvelopRestEndpoint {

    @Autowired
    private RehabilitationManageService rehabilitationManageService;
    @Autowired
    private SynchronizePatientService synchronizePatientService;

    @GetMapping(value = "rehabilitationDetailInfo")
    @ApiOperation("康复计划服务项列表")
    public Envelop rehabilitationDetailInfo(
            @ApiParam(name = "planId", value = "康复计划id", required = true)
            @RequestParam(value = "planId", required = true) String planId) {
        try {
            return ListEnvelop.getSuccess("查询成功",rehabilitationManageService.rehabilitationDetailInfo(planId));
        }catch (ServiceException se){
            return Envelop.getError(se.getMessage());
        }catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("查询失败");
        }
    }

    @GetMapping(value = "findServiceItemPlanDetail")
    @ApiOperation("执行计划详情")
    public Envelop findServiceItemPlanDetail(
            @ApiParam(name = "itemPlanId", value = "执行计划id", required = true)
            @RequestParam(value = "itemPlanId", required = true) String itemPlanId) {
        try {
            return ObjEnvelop.getSuccess("查询成功",rehabilitationManageService.findServiceItemPlanDetail(itemPlanId));
        }catch (ServiceException se){
            return Envelop.getError(se.getMessage());
        }catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("查询失败");
        }
    }

    @GetMapping(value = "findGuidanceMessageDetail")
    @ApiOperation("查询指导留言详情")
    public Envelop findGuidanceMessageDetail(
            @ApiParam(name = "itemPlanId", value = "执行计划id", required = true)
            @RequestParam(value = "itemPlanId", required = true) String itemPlanId) {
        try {
            return ObjEnvelop.getSuccess("查询成功",rehabilitationManageService.findGuidanceMessageDetail(itemPlanId));
        }catch (ServiceException se){
            return Envelop.getError(se.getMessage());
        }catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("查询失败");
        }
    }

    @PostMapping(value = "assignExecutor")
    @ApiOperation("计划负责人分配执行人")
    public Envelop assignExecutor(
            @ApiParam(name = "jsonData", value = "jsonData", required = true)
            @RequestParam(value = "jsonData", required = true) String jsonData) {
        try {
            PatientRehabilitationPlanDO planDO = toEntity(jsonData, PatientRehabilitationPlanDO.class);
            rehabilitationManageService.assignExecutor(planDO,getUID());
            return Envelop.getSuccess("分配成功");
        }catch (ServiceException se){
            return Envelop.getError(se.getMessage());
        }catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("分配失败");
        }
    }

    @PostMapping(value = "synchronizePationSingle")
    @ApiOperation("手动重新下转")
    public MixEnvelop synchronizePationSingle(
            @ApiParam(name = "id", value = "id", required = true) @RequestParam(value = "id", required = true) Long id) {
        try {
            String res = synchronizePatientService.synchronizePationSingle(id);
            if (StringUtils.isBlank(res)) {
                return MixEnvelop.getSuccess("操作成功，康复计划生成");
            } else {
                return MixEnvelop.getError(res);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return MixEnvelop.getError("操作成功");
        }
    }


    @GetMapping(value = SpecialistMapping.rehabilitation.findRehabilitationPlanList)
    @ApiOperation(value = "康复管理-康复计划列表")
    public MixEnvelop findRehabilitationPlan(
            @ApiParam(name = "doctorType", value = "医生类型（1、专科医生，2、家庭医生）", required = true) @RequestParam(value = "doctorType", required = true) Integer doctorType,
            @ApiParam(name = "doctorCode", value = "医生code", required = true) @RequestParam(value = "doctorCode", required = true) String doctorCode,
            @ApiParam(name = "patientCondition", value = "居民条件，可以按身份证或者居民名称模糊匹配", required = false) @RequestParam(value = "patientCondition", required = false) String patientCondition,
            @ApiParam(name = "diseaseCode", value = "疾病类型code", required = false) @RequestParam(value = "diseaseCode", required = false) String diseaseCode,
            @ApiParam(name = "planType", value = "安排类型（1康复计划，2转社区医院，3转家庭病床）", required = false) @RequestParam(value = "planType", required = false) Integer planType,
            @ApiParam(name = "todaybacklog", value = "今日待办（1、今日待办，2、全部）", required = false) @RequestParam(value = "todaybacklog", required = false, defaultValue = "1") Integer todaybacklog,
            @ApiParam(name = "page", value = "第几页，从1开始", required = false) @RequestParam(value = "page", required = false, defaultValue = "1") Integer page,
            @ApiParam(name = "pageSize", value = "每页分页大小", required = false) @RequestParam(value = "pageSize", required = false, defaultValue = "10") Integer pageSize
    ) {
        try {
            return rehabilitationManageService.findRehabilitationPlan(doctorType, doctorCode, diseaseCode, planType, todaybacklog, patientCondition, page, pageSize);
        } catch (Exception e) {
            e.printStackTrace();

            return MixEnvelop.getError("获取失败");
        }
    }

    @GetMapping(value = SpecialistMapping.rehabilitation.findRehabilitationPlanDetailList)
    @ApiOperation(value = "康复管理-康复计划详情列表")
    public ObjEnvelop findRehabilitationPlanDetailList(
            @ApiParam(name = "doctorCode", value = "医生code", required = true) @RequestParam(value = "doctorCode", required = true) String doctorCode,
            @ApiParam(name = "patientCode", value = "居民code", required = true) @RequestParam(value = "patientCode", required = true) String patientCode
    ) {
        try {
            return rehabilitationManageService.findRehabilitationPlanDetailList(doctorCode, patientCode);
        } catch (Exception e) {
            e.printStackTrace();

            return ObjEnvelop.getError("获取失败");
        }
    }

    @GetMapping(value = SpecialistMapping.rehabilitation.calendarPlanDetail)
    @ApiOperation(value = "康复管理-康复计划按日历展示")
    public ObjEnvelop calendarPlanDetail(
            @ApiParam(name = "executeStartTime", value = "日历开始时间（格式：yyyy-MM-dd）") @RequestParam(value = "executeStartTime", required = false) String executeStartTime,
            @ApiParam(name = "executeEndTime", value = "日历结束时间（格式：yyyy-MM-dd）") @RequestParam(value = "executeEndTime", required = false) String executeEndTime,
            @ApiParam(name = "planId", value = "计划id,多个计划逗号隔开", required = true) @RequestParam(value = "planId", required = true) String planId,
            @ApiParam(name = "searchTask", value = "快速查找任务：（1电话/短信关怀,2康复咨询,3健康监测,4上门服务,5康复指导,6康复随访,7康复复诊）", required = false) @RequestParam(value = "searchTask", required = false) Integer searchTask,
            @ApiParam(name = "status", value = "任务状态（0未完成，1已完成，2已预约）", required = false) @RequestParam(value = "status", required = false) Integer status,
            @ApiParam(name = "doctorCode", value = "医生code（专科医生、家庭医生）", required = false) @RequestParam(value = "doctorCode", required = false) String doctorCode,
            @ApiParam(name = "taskExecutor", value = "任务执行人：0全部；1我的任务：2他人任务", required = false) @RequestParam(value = "taskExecutor", required = false, defaultValue = "0") String taskExecutor
    ) {
        try {
            return rehabilitationManageService.calendarPlanDetail(executeStartTime, executeEndTime, planId, searchTask, status, doctorCode, taskExecutor);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError("获取失败");
        }
    }


    @GetMapping(value = SpecialistMapping.rehabilitation.calendarPlanDetailList)
    @ApiOperation(value = "康复管理-康复计划按列表展示")
    public ObjEnvelop calendarPlanDetailList(
            @ApiParam(name = "executeStartTime", value = "日历开始时间（格式：yyyy-MM-dd）") @RequestParam(value = "executeStartTime", required = false) String executeStartTime,
            @ApiParam(name = "executeEndTime", value = "日历结束时间（格式：yyyy-MM-dd）") @RequestParam(value = "executeEndTime", required = false) String executeEndTime,
            @ApiParam(name = "planId", value = "计划id,多个计划逗号隔开", required = true) @RequestParam(value = "planId", required = true) String planId,
            @ApiParam(name = "searchTask", value = "快速查找任务：（1电话/短信关怀,2康复咨询,3健康监测,4上门服务,5康复指导,6康复随访,7康复复诊 8所有）") @RequestParam(value = "searchTask", required = false) Integer searchTask,
            @ApiParam(name = "status", value = "任务状态（0未完成，1已完成，2已预约）") @RequestParam(value = "status", required = false) Integer status,
            @ApiParam(name = "doctorCode", value = "医生code（专科医生、家庭医生）") @RequestParam(value = "doctorCode", required = false) String doctorCode,
            @ApiParam(name = "taskExecutor", value = "任务执行人：0全部；1我的任务：2他人任务", required = false) @RequestParam(value = "taskExecutor", required = false, defaultValue = "0") String taskExecutor
    ) {
        try {
            return rehabilitationManageService.calendarPlanDetailList(planId, searchTask, status, doctorCode, executeStartTime, executeEndTime, taskExecutor);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError(e.getMessage());
        }
    }

    @GetMapping(value = SpecialistMapping.rehabilitation.serviceItemList)
    @ApiOperation(value = "康复管理-多个康复计划服务项目内容信息列表")
    public ObjEnvelop serviceItemList(
            @ApiParam(name = "planDetailIds", value = "多个服务项目id用‘，’分隔", required = true) @RequestParam(value = "planDetailIds", required = true) String planDetailIds,
            @ApiParam(name = "doctorCode", value = "医生code", required = false) @RequestParam(value = "doctorCode", required = false) String doctorCode
    ) {
        try {
            return rehabilitationManageService.serviceItemList(planDetailIds, doctorCode);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError(e.getMessage());
        }
    }

    @GetMapping(value = SpecialistMapping.rehabilitation.serviceItem)
    @ApiOperation(value = "康复管理-康复计划服务项目确认详情页")
    public ObjEnvelop serviceItem(
            @ApiParam(name = "planDetailId", value = "服务项目id", required = true) @RequestParam(value = "planDetailId", required = true) String planDetailId,
            @ApiParam(name = "doctorCode", value = "医生code", required = false) @RequestParam(value = "doctorCode", required = false) String doctorCode,
            @ApiParam(name = "recordId", value = "记录id", required = false) @RequestParam(value = "recordId", required = false) String recordId
    ) {
        try {
            return rehabilitationManageService.serviceItem(planDetailId, doctorCode, recordId);
        } catch (Exception e) {
            e.printStackTrace();

            return ObjEnvelop.getError(e.getMessage());
        }
    }

    @PostMapping(value = "appointmentConsultation")
    @ApiOperation(value = "康复管理-预约协诊")
    public ObjEnvelop appointmentConsultation(
            @ApiParam(name = "patient", value = "居民姓名", required = true) @RequestParam(value = "patient", required = true) String patient,
            @ApiParam(name = "doctor", value = "医生code-家医", required = true) @RequestParam(value = "doctor", required = true) String doctor,
            @ApiParam(name = "doctorName", value = "医生姓名", required = true) @RequestParam(value = "doctorName", required = true) String doctorName,
            @ApiParam(name = "appointmentDoctor", value = "医生code-专科", required = true) @RequestParam(value = "appointmentDoctor", required = true) String appointmentDoctor,
            @ApiParam(name = "appointmentDoctorName", value = "医生姓名", required = true) @RequestParam(value = "appointmentDoctorName", required = true) String appointmentDoctorName,
            @ApiParam(name = "sendTime", value = "发送时间", required = true) @RequestParam(value = "sendTime", required = true) String sendTime,
            @ApiParam(name = "planDetailId", value = "计划明细id", required = true) @RequestParam(value = "planDetailId", required = true) String planDetailId
    ) {
        try {
            return rehabilitationManageService.appointmentConsultation(patient, doctor, doctorName, appointmentDoctor, appointmentDoctorName, sendTime, planDetailId);
        } catch (Exception e) {
            e.printStackTrace();

            return ObjEnvelop.getError(e.getMessage());
        }
    }

    @PostMapping(value = SpecialistMapping.rehabilitation.saveGuidanceMessage)
    @ApiOperation(value = "康复管理-保存指导留言")
    public Envelop saveGuidanceMessage(
            @ApiParam(name = "messageId", value = "消息id", required = true) @RequestParam(value = "messageId", required = true) String messageId,
            @ApiParam(name = "imgs", value = "附件", required = false) @RequestParam(value = "imgs", required = false) String imgs,
            @ApiParam(name = "doctorCode", value = "医生code", required = true) @RequestParam(value = "doctorCode", required = true) String doctorCode,
            @ApiParam(name = "doctorType", value = "医生类型（1、专科医生）", required = true) @RequestParam(value = "doctorType", required = true) Integer doctorType,
            @ApiParam(name = "content", value = "聊天内容", required = true) @RequestParam(value = "content", required = true) String content,
            @ApiParam(name = "itemPlanId", value = "服务项目执行计划id", required = true) @RequestParam(value = "itemPlanId", required = true) String itemPlanId,
            @ApiParam(name = "contentType", value = "内容类型（1,6,8 - 文本， 2，9- 图片， 3 - 语音， 4-文章， 5,7系统消息。12-语音 18-居民名片， 19-聊天记录）", required = true)
            @RequestParam(value = "contentType", required = true) Integer contentType) {
        try {
            return rehabilitationManageService.saveGuidanceMessage(messageId, doctorCode, doctorType, content,imgs, itemPlanId, contentType);
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError("操作失败");
        }
    }

    @PostMapping(value = SpecialistMapping.rehabilitation.updateStatusRehabilitationOperate)
    @ApiOperation(value = "康复管理-更新康复计划操作完成日志状态")
    public Envelop updateStatusRehabilitationOperate(@ApiParam(name = "planDetailId", value = "服务项目id", required = true)
                                                     @RequestParam(value = "planDetailId", required = true) String planDetailId,
                                                     @ApiParam(name = "status", value = "状态", required = true)
                                                     @RequestParam(value = "status", required = true) Integer status) {
        try {
            return rehabilitationManageService.updateStatusRehabilitationOperate(status, planDetailId);
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError(e.getMessage());
        }
    }


    @GetMapping(value = SpecialistMapping.rehabilitation.patientRehabilitationDetail)
    @ApiOperation(value = "康复管理-居民详情页")
    public Envelop patientRehabilitationDetail(
            @ApiParam(name = "patientCode", value = "居民code", required = true) @RequestParam(value = "patientCode", required = true) String patientCode,
            @ApiParam(name = "healthDoctor", value = "健管师医生code", required = false) @RequestParam(value = "healthDoctor", required = false) String healthDoctor,
            @ApiParam(name = "healthDoctorName", value = "健管师医生名称", required = false) @RequestParam(value = "healthDoctorName", required = false) String healthDoctorName,
            @ApiParam(name = "generalDoctor", value = "全科医生code", required = false) @RequestParam(value = "generalDoctor", required = false) String generalDoctor,
            @ApiParam(name = "generalDoctorName", value = "全科医生名称", required = false) @RequestParam(value = "generalDoctorName", required = false) String generalDoctorName
    ) {
        try {
            return rehabilitationManageService.patientRehabilitationDetail(patientCode, healthDoctor, healthDoctorName, generalDoctor, generalDoctorName);
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError(e.getMessage());
        }
    }

    @GetMapping(value = SpecialistMapping.rehabilitation.recentPlanDetailRecord)
    @ApiOperation(value = "居民康复计划详情页-近期康复相关记录")
    public Envelop recentPlanDetailRecord(
            @ApiParam(name = "patientCode", value = "居民code", required = true) @RequestParam(value = "patientCode", required = true) String patientCode,
            @ApiParam(name = "startTime", value = "开始时间（格式：yyyy-MM-dd HH:mm:ss）", required = false) @RequestParam(value = "startTime", required = false) String startTime,
            @ApiParam(name = "endTime", value = "结束时间（格式：yyyy-MM-dd HH:mm:ss）", required = false) @RequestParam(value = "endTime", required = false) String endTime,
            @ApiParam(name = "page", value = "第几页，从1开始", required = true) @RequestParam(value = "page", required = false, defaultValue = "1") Integer page,
            @ApiParam(name = "pageSize", value = "每页分页大小", required = true) @RequestParam(value = "pageSize", required = false, defaultValue = "10") Integer pageSize) {
        try {
            return rehabilitationManageService.recentPlanDetailRecord(patientCode, startTime, endTime, page, pageSize);
        } catch (Exception e) {
            e.printStackTrace();

            return Envelop.getError(e.getMessage());
        }
    }

    /**
     * 2.0方法
     */
//    @GetMapping(value = SpecialistMapping.rehabilitation.serviceDoctorList)
//    @ApiOperation(value = "康复管理-医生端居民详情服务医生列表")
//    public Envelop serviceDoctorList(
//            @ApiParam(name = "patientCode", value = "居民code", required = true) @RequestParam(value = "patientCode", required = true) String patientCode,
//            @ApiParam(name = "healthDoctor", value = "健管师医生code", required = false) @RequestParam(value = "healthDoctor", required = false) String healthDoctor,
//            @ApiParam(name = "healthDoctorName", value = "健管师医生名称", required = false) @RequestParam(value = "healthDoctorName", required = false) String healthDoctorName,
//            @ApiParam(name = "generalDoctor", value = "全科医生code", required = false) @RequestParam(value = "generalDoctor", required = false) String generalDoctor,
//            @ApiParam(name = "generalDoctorName", value = "全科医生名称", required = false) @RequestParam(value = "generalDoctorName", required = false) String generalDoctorName
//    ) {
//        try {
//            return rehabilitationManageService.serviceDoctorList(patientCode, healthDoctor, healthDoctorName, generalDoctor, generalDoctorName);
//        } catch (Exception e) {
//            e.printStackTrace();
//
//            return Envelop.getError(e.getMessage());
//        }
//    }
    @GetMapping(value = SpecialistMapping.rehabilitation.appCalendarPlanDetailList)
    @ApiOperation(value = "康复管理-app端、微信端计划的服务项目列表")
    public ObjEnvelop appCalendarPlanDetailList(
            @ApiParam(name = "executeStartTime", value = "日历开始时间（格式：yyyy-MM-dd HH:mm:ss）", required = true) @RequestParam(value = "executeStartTime", required = false) String executeStartTime,
            @ApiParam(name = "executeEndTime", value = "日历结束时间（格式：yyyy-MM-dd HH:mm:ss）", required = true) @RequestParam(value = "executeEndTime", required = false) String executeEndTime,
            @ApiParam(name = "planId", value = "计划id", required = true) @RequestParam(value = "planId", required = true) String planId,
            @ApiParam(name = "searchTask", value = "快速查找任务：（1、我的任务，2、随访，3、复诊，4、健康教育）", required = false) @RequestParam(value = "searchTask", required = false) Integer searchTask,
            @ApiParam(name = "status", value = "任务状态（0未完成，1已完成，2已预约）", required = false) @RequestParam(value = "status", required = false) Integer status
    ) {
        try {
            return rehabilitationManageService.appCalendarPlanDetailList(planId, searchTask, status, executeStartTime, executeEndTime);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError(e.getMessage());
        }
    }

    @GetMapping(value = SpecialistMapping.rehabilitation.dailyJob)
    @ApiOperation(value = "每日康复服务通知")
    public ObjEnvelop dailyJob(
            @ApiParam(name = "startTime", value = "开始时间（格式：yyyy-MM-dd HH:mm:ss）", required = true) @RequestParam(value = "startTime", required = true) String startTime,
            @ApiParam(name = "endTime", value = "结束时间（格式：yyyy-MM-dd HH:mm:ss）", required = true) @RequestParam(value = "endTime", required = true) String endTime
    ) {
        try {
            return rehabilitationManageService.dailyJob(startTime, endTime);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError(e.getMessage());
        }
    }

    @GetMapping(value = SpecialistMapping.rehabilitation.tomorrowJob)
    @ApiOperation(value = "明日康复服务通知")
    public ObjEnvelop tomorrowJob(
            @ApiParam(name = "startTime", value = "开始时间（格式：yyyy-MM-dd HH:mm:ss）", required = true) @RequestParam(value = "startTime", required = true) String startTime,
            @ApiParam(name = "endTime", value = "结束时间（格式：yyyy-MM-dd HH:mm:ss）", required = true) @RequestParam(value = "endTime", required = true) String endTime
    ) {
        try {
            return rehabilitationManageService.tomorrowJob(startTime, endTime);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError(e.getMessage());
        }
    }


    @GetMapping(value = SpecialistMapping.rehabilitation.dailyByJob)
    @ApiOperation(value = "每日康复计划提醒")
    public ObjEnvelop dailyByJob(
            @ApiParam(name = "startTime", value = "开始时间（格式：yyyy-MM-dd HH:mm:ss）", required = true) @RequestParam(value = "startTime", required = true) String startTime,
            @ApiParam(name = "endTime", value = "结束时间（格式：yyyy-MM-dd HH:mm:ss）", required = true) @RequestParam(value = "endTime", required = true) String endTime
    ) {
        try {
            return rehabilitationManageService.dailyByJob(startTime, endTime);
        } catch (Exception e) {
            e.printStackTrace();

            return ObjEnvelop.getError(e.getMessage());
        }
    }

    @PostMapping(value = SpecialistMapping.rehabilitation.updateNoteAndImageRehabilitationOperate)
    @ApiOperation(value = "康复计划完成时更新服务完成笔记和图片接口并且确认完成")
    public ObjEnvelop updateNoteAndImageRehabilitationOperate(
            @ApiParam(name = "planDetailId", value = "服务项目id", required = true) @RequestParam(value = "planDetailId", required = true) String planDetailId,
            @ApiParam(name = "node", value = "服务完成笔记", required = true) @RequestParam(value = "node", required = false) String node,
            @ApiParam(name = "image", value = "相关记录图片，json格式", required = true) @RequestParam(value = "image", required = false) String image,
            @ApiParam(name = "relationCode", value = "业务code", required = true) @RequestParam(value = "relationCode", required = false) String relationCode,
            @ApiParam(name = "status", value = "是否确认完成 0未确认 1确认", required = true) @RequestParam(value = "status", required = false) String status
    ) {
        try {
            Map<String, Object> map = rehabilitationManageService.updateNodeAndRelationRecordImg(node, image, status, planDetailId, relationCode);
            return ObjEnvelop.getSuccess(SpecialistMapping.api_success, map);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError(e.getMessage());
        }
    }


    @PostMapping(value = "planDetailAfterCall")
    @ApiOperation(value = "康复管理-电话关怀拨打后生成记录对应怪坏记录")
    public MixEnvelop<RehabilitationOperateRecordsDO, RehabilitationOperateRecordsDO> planDetailAfterCall(
            @ApiParam(name = "dataJson", value = "实体json", defaultValue = "{\"rehabilitationDetailId\":\"808080eb764ba46b017669712c640012\",\"doctorCode\":\"7e06886207e843948ae65d8f91d6ca97\",\"doctorName\":\"兰冠勇\"}")
            @RequestParam(value = "dataJson", required = true) String dataJson) {
        try {
            return MixEnvelop.getSuccess(SpecialistMapping.api_success, rehabilitationManageService.planDetailAfterCall(dataJson));
        } catch (Exception e) {
            e.printStackTrace();

            return MixEnvelop.getError(e.getMessage());
        }
    }

    @PostMapping(value = SpecialistMapping.rehabilitation.updatePlanDetailStatusById)
    @ApiOperation(value = "康复管理-更新康复计划项目状态")
    public Envelop updatePlanDetailStatusById(
            @ApiParam(name = "planDetailId", value = "服务项目id", required = true) @RequestParam(value = "planDetailId", required = true) String planDetailId,
            @ApiParam(name = "status", value = "状态", required = true) @RequestParam(value = "status", required = true) Integer status
    ) {
        try {
            return rehabilitationManageService.updatePlanDetailStatusById(status, planDetailId);
        } catch (Exception e) {
            e.printStackTrace();

            return Envelop.getError(e.getMessage());
        }
    }


    @PostMapping(value = SpecialistMapping.rehabilitation.updatePlanDetailExecuteTimeById)
    @ApiOperation(value = "康复管理-修改康复计划项目执行时间")
    public Envelop updatePlanDetailExecuteTimeById(
            @ApiParam(name = "planDetailId", value = "服务项目id", required = true) @RequestParam(value = "planDetailId", required = true) String planDetailId,
            @ApiParam(name = "executeTime", value = "状态", required = true) @RequestParam(value = "executeTime", required = true) String executeTime
    ) {
        try {
            Date date = DateUtil.strToDateLong(executeTime);
            return rehabilitationManageService.updatePlanDetailExecuteTimeById(date, planDetailId);
        } catch (Exception e) {
            e.printStackTrace();
            return Envelop.getError(e.getMessage());
        }
    }


    @GetMapping(value = "planSchedule")
    @ApiOperation(value = "康复管理-计划总进度")
    public ObjEnvelop planSchedule(
            @ApiParam(name = "planId", value = "计划id", required = true) @RequestParam(value = "planId", required = true) String planId
    ) {
        try {
            return rehabilitationManageService.planSchedule(planId);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError("查询失败");
        }
    }

    @GetMapping(value = SpecialistMapping.rehabilitation.planListByPatient)
    @ApiOperation(value = "康复管理-根据居民获取康复计划")
    public ObjEnvelop planListByPatient(@ApiParam(name = "patientCode", value = "居民code", required = true)
                                        @RequestParam(value = "patientCode", required = true) String patientCode) {
        try {
            return rehabilitationManageService.planListByPatient(patientCode);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError("查询失败");
        }
    }

    @GetMapping(value = SpecialistMapping.rehabilitation.patientCount)
    @ApiOperation(value = "医生已计划数，已完成计划数（居民数）")
    public ObjEnvelop patientCount(@ApiParam(name = "doctorCode", value = "医生code", required = true)
                                   @RequestParam(value = "doctorCode", required = true) String doctorCode) {
        try {
            return rehabilitationManageService.patientCount(doctorCode);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError(e.getMessage());
        }
    }

    @GetMapping(value = SpecialistMapping.rehabilitation.dailyJobReserve)
    @ApiOperation(value = "10天、7天、5天、当天康复服务预约复诊通知")
    public ObjEnvelop dailyJobReserve(
            @ApiParam(name = "startTime", value = "开始时间（格式：yyyy-MM-dd HH:mm:ss）", required = true) @RequestParam(value = "startTime", required = true) String startTime,
            @ApiParam(name = "endTime", value = "结束时间（格式：yyyy-MM-dd HH:mm:ss）", required = true) @RequestParam(value = "endTime", required = true) String endTime
    ) {
        try {
            return rehabilitationManageService.dailyJobReserve(startTime, endTime);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError(e.getMessage());
        }
    }


    /**
     * 查询康复服务项目
     *
     * @param ids
     * @return
     */
    @PostMapping(value = SpecialistMapping.rehabilitation.selectByIds)
    @ApiOperation(value = "查询康复服务项目")
    public ObjEnvelop selectByIds(
            @ApiParam(name = "ids", value = "康复服务套餐明细表ids") @RequestParam(value = "ids", required = true) String ids
    ) {
        try {
            return rehabilitationManageService.selectByIds(ids);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError(e.getMessage());
        }
    }

    @GetMapping(value = SpecialistMapping.serviceItem.getServiceItemsAfterStop)
    @ApiOperation(value = "康复管理-中止康复计划结算项目清单")
    public ObjEnvelop getServiceItemsAfterStop(@ApiParam(name = "planId", value = "康复计划id", required = true)
                                               @RequestParam(value = "planId", required = true) String planId) {
        try {
            return rehabilitationManageService.getServiceItemsAfterStop(planId);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError(e.getMessage());
        }
    }
}
