package com.yihu.jw.hospital.module.rehabilitation.controller;

import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.jw.entity.specialist.rehabilitation.*;
import com.yihu.jw.hospital.module.rehabilitation.dao.RehabilitationOperateRecordsDao;
import com.yihu.jw.hospital.module.rehabilitation.service.RehabilitationPlanService;
import com.yihu.jw.im.util.ImUtil;
import com.yihu.jw.restmodel.qvo.ParamQvo;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.specialist.SpecialistMapping;
import com.yihu.jw.util.date.DateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.net.URLDecoder;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by humingfen on 2018/8/17.
 */
@RestController
@RequestMapping(SpecialistMapping.api_specialist_common)
@Api(tags = "康复服务套餐管理相关操作", description = "康复服务套餐管理相关操作")
public class RehabilitationPlanController extends EnvelopRestEndpoint {

    @Autowired
    private RehabilitationPlanService rehabilitationPlanService;

    @Autowired
    private RehabilitationOperateRecordsDao rehabilitationOperateRecordsDao;
    @Autowired
    private ImUtil imUtill;

    @RequestMapping(value = "getRehabilitationPlanLog", method = RequestMethod.GET)
    @ApiOperation(value = "获取康复计划修改记录")
    public String getRehabilitationPlanLog(
            @ApiParam(name = "planId", value = "康复计划Id", required = true) @RequestParam(value = "planId", required = true) String planId
    ) {

        try {
            return write(200, "获取成功", "data", rehabilitationPlanService.getRehabilitationPlanLog(planId));
        } catch (Exception e) {
            return error(-1, e.getMessage());
        }
    }


    /**
     * 新增整个模板 /svr-specialist/saveRehabilitationPlanTemplate
     * 表：wlyy_rehabilitation_plan_template 、wlyy_rehabilitation_template_detail
     * 参数：
     * rehabilitationTemplate: {"id":"","title":"测试模板","hospital":"808080eb71672567017172672f3f0001","hospital_name":"中山医院","del":1,"status":"1","diseaseId":"ff808081882c93cf01882cdbbed60000","diseaseName":"冠心病"}
     * rehabilitationTemplateDetail:
     */
    @PostMapping(value = SpecialistMapping.rehabilitation.saveRehabilitationPlanTemplate)
    @ApiOperation(value = "康复服务套餐模板创建")
    public ObjEnvelop saveRehabilitationPlanTemplate(
            @ApiParam(name = "rehabilitationTemplate", value = "实体JSON") @RequestParam(value = "rehabilitationTemplate", required = false) String rehabilitationTemplate,
            @ApiParam(name = "rehabilitationTemplateDetail", value = "实体JSON") @RequestParam(value = "rehabilitationTemplateDetail", required = false) String rehabilitationTemplateDetail
    ) {
        try {
            rehabilitationTemplate = URLDecoder.decode(rehabilitationTemplate, "UTF-8");
            rehabilitationTemplateDetail = URLDecoder.decode(rehabilitationTemplateDetail, "UTF-8");
            //映射成实体
            RehabilitationPlanTemplateDO templateDO = toEntity(rehabilitationTemplate, RehabilitationPlanTemplateDO.class);
//            List<RehabilitationTemplateDetailDO> details =
//                    new ObjectMapper().readValue(rehabilitationTemplateDetail, new TypeReference<List<RehabilitationTemplateDetailDO>>() {
//                    });
            List<RehabilitationTemplateDetailDO> details = JSON.parseArray(rehabilitationTemplateDetail, RehabilitationTemplateDetailDO.class);
            System.out.println(details);
            //数据保存
            return rehabilitationPlanService.saveRehabilitationPlanTemplateData(templateDO, details);
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError(e.getMessage());
        }
    }

    /**
     * 计划名称，状态，病种id
     */
    @GetMapping(value = SpecialistMapping.rehabilitation.findRehabilitationPlanTemplatenew)
    @ApiOperation(value = "获取康复服务套餐模板列表")
    public MixEnvelop findRehabilitationPlanTemplatenew(
            @ApiParam(name = "templateId", value = "主键") @RequestParam(value = "templateId", required = false) String templateId,
            @ApiParam(name = "templateName", value = "templateName") @RequestParam(value = "templateName", required = false) String templateName,
            @ApiParam(name = "status", value = "状态") @RequestParam(value = "status", required = false) String status,
            @ApiParam(name = "diseaseId", value = "病种Id") @RequestParam(value = "diseaseId", required = false) String diseaseId,
            @ApiParam(name = "type", value = "类型1模板2计划") @RequestParam(value = "type", required = false) String type,
            @ApiParam(name = "pageIf", value = "是否分页") @RequestParam(value = "pageIf", required = false) String pageIf,
            @ApiParam(name = "page", value = "页") @RequestParam(value = "page", required = false) int page,
            @ApiParam(name = "pageSize", value = "条数") @RequestParam(value = "pageSize", required = false) int pageSize
    ) {
        try {
            HashMap<String, Object> map = rehabilitationPlanService.findRehabilitationPlanTemplatenew(templateId, templateName, status, diseaseId, type, pageIf, page, pageSize);
            return MixEnvelop.getSuccess(SpecialistMapping.api_success, map);
        } catch (Exception e) {
            e.printStackTrace();
            return MixEnvelop.getError(e.getMessage());
        }
    }


    /**
     * 查询某个人的康复计划的服务项目
     */
    @GetMapping(value = "getRehabilitationDetailItemByPatient")
    @ApiOperation(value = "查询某个人的康复计划的服务项目")
    public MixEnvelop getRehabilitationDetailItemByPatient(
            @ApiParam(name = "patientId", value = "居民id") @RequestParam(value = "patientId", required = false) String patientId,
            @ApiParam(name = "planId", value = "计划id") @RequestParam(value = "planId", required = false) String planId
    ) {
        try {
            List<Map<String, Object>> list = rehabilitationPlanService.getRehabilitationDetailItemByPatient(patientId,planId);
            return MixEnvelop.getSuccess(SpecialistMapping.api_success, list);
        } catch (Exception e) {
            e.printStackTrace();
            return MixEnvelop.getError(e.getMessage());
        }
    }






    /**
     * 查询服务项目
     * wlyy_rehabilitation_service_item
     */
    @GetMapping(value = SpecialistMapping.rehabilitation.findRehabilitationServiceItem)
    @ApiOperation(value = "查询服务项目")
    public MixEnvelop findRehabilitationServiceItem(
            @ApiParam(name = "itemName", value = "itemName") @RequestParam(value = "itemName", required = false) String itemName
    ) {
        try {
            List<Map<String, Object>> list = rehabilitationPlanService.findRehabilitationServiceItem(itemName);
            return MixEnvelop.getSuccess(SpecialistMapping.api_success, list);
        } catch (Exception e) {
            e.printStackTrace();
            return MixEnvelop.getError(e.getMessage());
        }
    }

    /**
     * 查询专病列表
     */
    @GetMapping(value = SpecialistMapping.rehabilitation.findDiseaseList)
    @ApiOperation(value = "查询专病列表")
    public MixEnvelop findDiseaseList() {
        try {
            List<Map<String, Object>> list = rehabilitationPlanService.findDiseaseList();
            return MixEnvelop.getSuccess(SpecialistMapping.api_success, list);
        } catch (Exception e) {
            e.printStackTrace();
            return MixEnvelop.getError(e.getMessage());
        }
    }


    /**
     * 删除康复模板和模型详情
     */
    @PostMapping(value = SpecialistMapping.rehabilitation.deleteRehabilitationPlanTemplate)
    @ApiOperation(value = "删除康复模板和模型详情")
    public ObjEnvelop deleteRehabilitationPlanTemplate(
            @ApiParam(name = "id", value = "康复模板id") @RequestParam(value = "id", required = true) String id) {
        try {
            if("system".equals(id)){
                return ObjEnvelop.getError("系统模板不允许删除");
            }
            return rehabilitationPlanService.deleteRehabilitationPlanTemplate(id);
        } catch (Exception e) {
            e.printStackTrace();
            // tracer.getCurrentSpan().logEvent(e.getMessage())
            return ObjEnvelop.getError(e.getMessage());
        }
    }

    /**
     * 改变状态 del改成0
     */
    @PostMapping(value = "changeState")
    @ApiOperation(value = "改变状态")
    public ObjEnvelop changeState(
            @ApiParam(name = "jsonStr", value = "jsonStr", required = true) @RequestParam String jsonStr) throws Exception {
        jsonStr = URLDecoder.decode(jsonStr, "UTF-8");
        ParamQvo qvo = JSON.parseObject(jsonStr, ParamQvo.class);
        System.out.println("参数:" + JSON.toJSONString(qvo));
        RehabilitationPlanTemplateDO entity = rehabilitationPlanService.changeState(qvo);
        return success(entity);
    }

    @GetMapping(value = "findByTemplateId")
    @ApiOperation(value = "获取康复服务套餐模板明细")
    public ObjEnvelop findByTemplateId(@ApiParam(name = "templateId", value = "模板id")
                                                      @RequestParam(value = "templateId", required = true) String templateId) {
        try {
            return ObjEnvelop.getSuccess("获取成功",rehabilitationPlanService.findByTemplateId(templateId));
        } catch (Exception e) {
            e.printStackTrace();
            return ObjEnvelop.getError("获取失败");
        }
    }

    @GetMapping(value = SpecialistMapping.rehabilitation.findTemplateDetailByTemplateId)
    @ApiOperation(value = "获取康复服务套餐模板明细")
    public ListEnvelop findTemplateDetailByTemplateId(@ApiParam(name = "templateId", value = "模板id")
                                                      @RequestParam(value = "templateId", required = true) String templateId) {

        try {
            ListEnvelop envelop = rehabilitationPlanService.findTemplateDetailByTemplateId(templateId);
            return envelop;
        } catch (Exception e) {
            e.printStackTrace();
            // tracer.getCurrentSpan().logEvent(e.getMessage())
            return ListEnvelop.getError(e.getMessage());
        }
    }


    @GetMapping(value = "findFrequencyList")
    @ApiOperation(value = "获取频次列表")
    public ListEnvelop findFrequencyList() {
        try {
            ListEnvelop envelop = rehabilitationPlanService.findFrequencyList();
            return envelop;
        } catch (Exception e) {
            e.printStackTrace();
            return ListEnvelop.getError(e.getMessage());
        }
    }


    /**
     * 另存为模板
     * 存成json串
     * --废弃，改用type区分模板合计划
     */
    @GetMapping(value = SpecialistMapping.rehabilitation.saveTemplate)
    @ApiOperation(value = "另存为模板")
    public MixEnvelop saveTemplate(
            @ApiParam(name = "templateName", value = "templateName") @RequestParam(value = "templateName", required = false) String templateName,
            @ApiParam(name = "templateJson", value = "templateJson") @RequestParam(value = "templateJson", required = false) String templateJson
    ) {
        try {
            templateJson = URLDecoder.decode(templateJson, "UTF-8");
            TemplateJson entity = rehabilitationPlanService.saveTemplate(templateName, templateJson);
            return MixEnvelop.getSuccess(SpecialistMapping.api_success, entity);
        } catch (Exception e) {
            e.printStackTrace();
            return MixEnvelop.getError(e.getMessage());
        }
    }


    /**
     * 查询另存为的模板
     * -- 废弃-采用type区分
     */
    @GetMapping(value = SpecialistMapping.rehabilitation.findSaveTemplate)
    @ApiOperation(value = "查询另存为的模板")
    public MixEnvelop saveTemplate(
            @ApiParam(name = "templateName", value = "templateName") @RequestParam(value = "templateName", required = false) String templateName
    ) {
        try {
            List<Map<String, Object>> list = rehabilitationPlanService.findSaveTemplate(templateName);
            return MixEnvelop.getSuccess(SpecialistMapping.api_success, list);
        } catch (Exception e) {
            e.printStackTrace();
            return MixEnvelop.getError(e.getMessage());
        }
    }


    //--------------------------------------------------------------------------------------
    //                    下面的没有用到
    //--------------------------------------------------------------------------------------
    @GetMapping(value = SpecialistMapping.rehabilitation.findRehabilitationPlanTemplate)
    @ApiOperation(value = "获取康复服务套餐模板列表")
    public MixEnvelop findRehabilitationPlanTemplate(
            @ApiParam(name = "adminTeamCode", value = "行政团队id") @RequestParam(value = "adminTeamCode", required = false) Long adminTeamCode,
            @ApiParam(name = "doctor", value = "专科医生") @RequestParam(value = "doctor", required = false) String doctor,
            @ApiParam(name = "patient", value = "患者") @RequestParam(value = "patient", required = false) String patient) {
        try {
            return rehabilitationPlanService.findRehabilitationPlanTemplate(adminTeamCode, doctor, patient);
        } catch (Exception e) {
            e.printStackTrace();
            // tracer.getCurrentSpan().logEvent(e.getMessage())
            return MixEnvelop.getError(e.getMessage());
        }
    }


    @PostMapping(value = SpecialistMapping.rehabilitation.createRehabilitationPlanTemplate)
    @ApiOperation(value = "康复服务套餐模板创建")
    public ObjEnvelop createRehabilitationPlanTemplate(@ApiParam(name = "rehabilitationTemplate", value = "实体JSON")
                                                       @RequestParam(value = "rehabilitationTemplate", required = true) String rehabilitationTemplate) {
        try {
            RehabilitationPlanTemplateDO templateDO = toEntity(rehabilitationTemplate, RehabilitationPlanTemplateDO.class);
            return rehabilitationPlanService.createRehabilitationTemplate(templateDO);
        } catch (Exception e) {
            e.printStackTrace();
            // tracer.getCurrentSpan().logEvent(e.getMessage())
            return ObjEnvelop.getError(e.getMessage());
        }
    }

    @PostMapping(value = SpecialistMapping.rehabilitation.createRehabilitationTemplateDetail)
    @ApiOperation(value = "康复服务套餐模板明细创建")
    public ObjEnvelop createRehabilitationTemplateDetail(@ApiParam(name = "rehabilitationTemplateDetail", value = "实体JSON")
                                                         @RequestParam(value = "rehabilitationTemplateDetail", required = true) String rehabilitationTemplateDetail) {
        try {
            List<RehabilitationTemplateDetailDO> details = new ObjectMapper().readValue(rehabilitationTemplateDetail, new TypeReference<List<RehabilitationTemplateDetailDO>>() {
            });
            return rehabilitationPlanService.createRehabilitationTemplateDetail(details);
        } catch (Exception e) {
            e.printStackTrace();
            // tracer.getCurrentSpan().logEvent(e.getMessage())
            return ObjEnvelop.getError(e.getMessage());
        }
    }


    @PostMapping(value = SpecialistMapping.rehabilitation.updateRehabilitationTemplateDetail)
    @ApiOperation(value = "编辑康复服务套餐模板明细")
    public ObjEnvelop updateRehabilitationTemplateDetail(@ApiParam(name = "rehabilitationTemplateDetail", value = "实体JSON")
                                                         @RequestParam(value = "rehabilitationTemplateDetail", required = true) String rehabilitationTemplateDetail) {
        try {
            List<RehabilitationTemplateDetailDO> details = new ObjectMapper().readValue(rehabilitationTemplateDetail, new TypeReference<List<RehabilitationTemplateDetailDO>>() {
            });
            return rehabilitationPlanService.updateRehabilitationTemplateDetail(details);
        } catch (Exception e) {
            e.printStackTrace();
            // tracer.getCurrentSpan().logEvent(e.getMessage())
            return ObjEnvelop.getError(e.getMessage());
        }
    }


    /******************************** 居民康复计划 ***************************************/

    @GetMapping(value = "findDisease")
    @ApiOperation(value = "查找专病")
    public MixEnvelop findDisease(
            @ApiParam(name = "name", value = "专病名称", required = false)
            @RequestParam(value = "name", required = false)String name,
            @ApiParam(name = "page", value = "页码", required = false)
            @RequestParam(value = "page", required = false)Integer page,
            @ApiParam(name = "size", value = "大小", required = false)
            @RequestParam(value = "size", required = false)Integer size) {
        try {
            return rehabilitationPlanService.selectByCondition(name, page, size);
        } catch (Exception e) {
            e.printStackTrace();
            return MixEnvelop.getError("创建失败");
        }
    }

    @PostMapping(value = SpecialistMapping.rehabilitation.createPatientRehabilitationPlan)
    @ApiOperation(value = "居民康复服务套餐创建")
    public MixEnvelop createPatientRehabilitationPlan(
            @ApiParam(name = "rehabilitationPlan", value = "实体JSON") @RequestParam(value = "rehabilitationPlan", required = true) String rehabilitationPlan) {
        try {
            PatientRehabilitationPlanDO planDO = toEntity(rehabilitationPlan, PatientRehabilitationPlanDO.class);
            List<RehabilitationDetailDO> details = planDO.getDetailDOList();
            planDO = rehabilitationPlanService.createPatientRehabilitationPlan(planDO,details);
            return MixEnvelop.getSuccess(SpecialistMapping.api_success, planDO);
        } catch (Exception e) {
            e.printStackTrace();
            return MixEnvelop.getError("创建失败");
        }
    }

    @PostMapping(value = "createRehabilitationDetail")
    @ApiOperation(value = "创建康复服务明细")
    public MixEnvelop createRehabilitationDetail(
            @ApiParam(name = "rehabilitationPlan", value = "实体JSON") @RequestParam(value = "rehabilitationPlan", required = true) String rehabilitationPlan) {
        try {
            JSONObject json = new JSONObject(rehabilitationPlan);
            String planId = json.getString("id");
            JSONArray array = new JSONArray();
            for (Object planDetail : json.getJSONArray("detail")) {
                JSONObject j = (JSONObject) planDetail;
                if (j.has("executeTime")) {
                    String executeTime = j.get("executeTime").toString();
                    String[] result = null;
                    if (executeTime.contains(",")) {
                        result = executeTime.split(",");
                    } else {
                        result = new String[1];
                        result[0] = executeTime;
                    }
                    int len = result.length;
                    while (len > 0) {
                        len--;
                        JSONObject temp = new JSONObject(j.toString());
                        temp.put("executeTime", result[len]);
                        temp.put("createUser", json.get("createUser"));
                        temp.put("createUserName", json.get("createUserName"));
                        array.put(temp);
                    }
                } else {
                    j.put("createUser", json.get("createUser"));
                    j.put("createUserName", json.get("createUserName"));
                    array.put(j);
                }
            }
            String planDetails = array.toString();
            ObjectMapper object = new ObjectMapper();
            object.setDateFormat(new SimpleDateFormat("yyyy-MM-dd HH:mm"));
            List<RehabilitationDetailDO> details = object.readValue(planDetails, new TypeReference<List<RehabilitationDetailDO>>() {
            });
            details = rehabilitationPlanService.createRehabilitationDetail(details, planId);
            return MixEnvelop.getSuccessList(SpecialistMapping.api_success, details);
        } catch (Exception e) {
            e.printStackTrace();
            // tracer.getCurrentSpan().logEvent(e.getMessage())
            return MixEnvelop.getError(e.getMessage());
        }
    }

    @GetMapping(value = SpecialistMapping.rehabilitation.findServiceItemsByHospital)
    @ApiOperation(value = "获取机构服务项目列表")
    public MixEnvelop findServiceItemsByHospital(@ApiParam(name = "doctorHospital", value = "医生所在机构")
                                                 @RequestParam(required = true) String doctorHospital,
                                                 @ApiParam(name = "signHospital", value = "居民签约机构")
                                                 @RequestParam(required = false) String signHospital) {
        try {
            return rehabilitationPlanService.findServiceItemsByHospital(doctorHospital, signHospital);
        } catch (Exception e) {
            e.printStackTrace();
            // tracer.getCurrentSpan().logEvent(e.getMessage())
            return MixEnvelop.getError(e.getMessage());
        }
    }

    @PostMapping(value = SpecialistMapping.rehabilitation.createServiceQrCode)
    @ApiOperation(value = "根据康复计划明细id和医生code生成服务码")
    public MixEnvelop<String, String> createServiceQrCode(@ApiParam(name = "planDetailId", value = "康复计划项目明细ID") @RequestParam(value = "planDetailId", required = true) String planDetailId,
                                                          @ApiParam(name = "doctorCode", value = "医生code") @RequestParam(value = "doctorCode", required = true) String doctorCode,
                                                          @ApiParam(name = "imageUrl", value = "二维码地址") @RequestParam(value = "imageUrl", required = true) String imageUrl) {
        try {
            return rehabilitationPlanService.createServiceQrCode(planDetailId, doctorCode, imageUrl);
        } catch (Exception e) {
            e.printStackTrace();
            // tracer.getCurrentSpan().logEvent(e.getMessage())
            return MixEnvelop.getError(e.getMessage());
        }
    }

    @PostMapping(value = SpecialistMapping.rehabilitation.checkAfterQrCode)
    @ApiOperation(value = "居民扫码后验证是否是关联的居民扫码")
    public MixEnvelop<Map<String, Object>, Map<String, Object>> checkAfterQrCode(@ApiParam(name = "planDetailId", value = "康复计划项目明细ID") @RequestParam(value = "planDetailId", required = true) String planDetailId,
                                                                                 @ApiParam(name = "patientCode", value = "居民端登录的居民code") @RequestParam(value = "patientCode", required = true) String patientCode) {
        try {
            String message = "";
            Boolean flag = true;
            Map<String, Object> map = rehabilitationPlanService.checkAfterQrCode(planDetailId, patientCode);
            int result = Integer.valueOf(String.valueOf(map.get("code")));
            if (result == 200) {
                message = "验证成功！";
            }
            if (result == -1) {
                message = "请相关居民扫描二维码";
                flag = false;
            }
            if (result == -10000) {
                message = "相关康复管理数据错误,请联系工作人员！";
                flag = false;
            }
            map.put("flag", flag);
            map.remove("code");
            return MixEnvelop.getSuccess(message, map);
        } catch (Exception e) {
            e.printStackTrace();
            // tracer.getCurrentSpan().logEvent(e.getMessage())
            return MixEnvelop.getError(e.getMessage());
        }
    }

    @PostMapping(value = SpecialistMapping.rehabilitation.saveRehabilitationOperateRecord)
    @ApiOperation(value = "新增operateRecord")
    public MixEnvelop<RehabilitationOperateRecordsDO, RehabilitationOperateRecordsDO> saveRehabilitationOperateRecord(@ApiParam(name = "dataJson", value = "实体json", defaultValue = "{\"rehabilitationDetailId\":\"402803f6657f195301657f4fbd3c0001\",\"patientCode\":\"00\",\"patientName\":\"11\",\"doctorCode\":\"22\",\"doctorName\":\"33\",\"node\":\"jjjjjj\",\"relationRecordType\":\"4\",\"relation_record_code\":\"55\",\"relationRecordImg\":\"666666\",\"status\":\"0\"}") @RequestParam(value = "dataJson", required = true) String dataJson) {
        try {
            RehabilitationOperateRecordsDO rehabilitationOperateRecordsDO = toEntity(dataJson, RehabilitationOperateRecordsDO.class);
            List<RehabilitationOperateRecordsDO> list = rehabilitationOperateRecordsDao.findByRehabilitationDetailId(rehabilitationOperateRecordsDO.getRehabilitationDetailId());
           /* if (list!=null && list.size()>0){
                return MixEnvelop.getError("该明细日志已存在！");
            }*/
            return MixEnvelop.getSuccess(SpecialistMapping.api_success, rehabilitationPlanService.saveRehabilitationRecord(rehabilitationOperateRecordsDO));
        } catch (Exception e) {
            e.printStackTrace();
            // tracer.getCurrentSpan().logEvent(e.getMessage())
            return MixEnvelop.getError(e.getMessage());
        }
    }

    @PostMapping(value = SpecialistMapping.rehabilitation.updatePlanStatusById)
    @ApiOperation(value = "康复管理-更新康复计划状态")
    public MixEnvelop updatePlanStatusById(@ApiParam(name = "planId", value = "康复计划id", required = true)
                                           @RequestParam(value = "planId", required = true) String planId,
                                           @ApiParam(name = "status", value = "状态", required = true)
                                           @RequestParam(value = "status", required = true) Integer status,
                                           @ApiParam(name = "abortReason", value = "中止原因", required = true)
                                           @RequestParam(value = "abortReason", required = false) String abortReason) {
        try {
            if (status != 0) {
                return rehabilitationPlanService.updatePlanStatusById(status, planId);
            } else {
                return rehabilitationPlanService.abortPlanById(planId, abortReason);
            }
        } catch (Exception e) {
            e.printStackTrace();
            // tracer.getCurrentSpan().logEvent(e.getMessage())
            return MixEnvelop.getError(e.getMessage());
        }
    }

    @PostMapping(value = SpecialistMapping.rehabilitation.sendToFamilyDoctor)
    @ApiOperation(value = "给家医发送居民的康复计划")
    public Envelop sendBusinessCard(@ApiParam(name = "doctorCode", value = "专医code")
                                    @RequestParam(value = "doctorCode", required = true) String doctorCode,
                                    @ApiParam(name = "doctorName", value = "专医名字")
                                    @RequestParam(value = "doctorName", required = true) String doctorName,
                                    @ApiParam(name = "sessionId", value = "会话id", defaultValue = "")
                                    @RequestParam(value = "sessionId", required = true) String sessionId,
                                    @ApiParam(name = "businessType", value = "businessType", defaultValue = "1")
                                    @RequestParam(value = "businessType", required = true) String businessType,
                                    @ApiParam(name = "contentJsonStr", value = "contentJsonStr")
                                    @RequestParam(value = "contentJsonStr", required = true) String contentJsonStr) {
        try {
            ImUtil.sendImMsg(doctorCode, doctorName, sessionId, ImUtil.ContentType.Rehabilitation.getValue(), contentJsonStr, businessType);
            return Envelop.getSuccess(SpecialistMapping.api_success);
        } catch (Exception e) {
            e.printStackTrace();
            // tracer.getCurrentSpan().logEvent(e.getMessage())
            return Envelop.getError(e.getMessage());
        }
    }

    /*@PostMapping(value = SpecialistMapping.rehabilitation.abortPlanById)
    @ApiOperation(value = "康复管理-中止康复计划填写中止原因")
    public Envelop abortPlanById(@ApiParam(name = "planId", value = "康复计划id", required = true)
                                           @RequestParam(value = "planId", required = true)String planId,
                                           @ApiParam(name = "abortReason", value = "中止原因", required = true)
                                           @RequestParam(value = "abortReason", required = true)String abortReason){
        try {
            return rehabilitationPlanService.abortPlanById(planId,abortReason);
        }catch (Exception e){
            e.printStackTrace();
            // tracer.getCurrentSpan().logEvent(e.getMessage())
            return Envelop.getError(e.getMessage());
        }
    }
    }*/


    /**
     * 根据居民code和标签code获取康复计划
     *
     * @param patient
     * @param label
     * @return
     */
    @GetMapping(value = SpecialistMapping.rehabilitation.selectByLabelAndPatient)
    @ApiOperation(value = "根据标签获取康复计划")
    public MixEnvelop selectByLabelAndPatient(@ApiParam(name = "patient", value = "居民code")
                                              @RequestParam(required = true) String patient,
                                              @ApiParam(name = "label", value = "标签code")
                                              @RequestParam(required = false) String label) {
        try {
            MixEnvelop envelop = new MixEnvelop();
            List<PatientRehabilitationPlanDO> planDOS = rehabilitationPlanService.selectByLabelAndPatient(patient, label);
            envelop.setDetailModelList(planDOS);
            return envelop;
        } catch (Exception e) {
            e.printStackTrace();
            // tracer.getCurrentSpan().logEvent(e.getMessage())
            return MixEnvelop.getError(e.getMessage());
        }
    }

    @PostMapping(value = SpecialistMapping.rehabilitation.updatePatientImgById)
    @ApiOperation(value = "更新康复计划居民签名照/证件照")
    public Envelop updatePatientImgById(@ApiParam(name = "planId", value = "康复计划id", required = true)
                                        @RequestParam(value = "planId") String planId,
                                        @ApiParam(name = "patientImg", value = "居民签名/证件照地址", required = true)
                                        @RequestParam(value = "patientImg") String patientImg) {
        try {
            return rehabilitationPlanService.updatePatientImgById(planId, patientImg);
        } catch (Exception e) {
            e.printStackTrace();
            // tracer.getCurrentSpan().logEvent(e.getMessage())
            return Envelop.getError(e.getMessage());
        }
    }

    @PostMapping(value = SpecialistMapping.rehabilitation.updateRehabilitationPlan)
    @ApiOperation(value = "居民康复下转套餐编辑")
    public MixEnvelop updateRehabilitationPlan(@ApiParam(name = "rehabilitationPlan", value = "实体JSON")
                                               @RequestParam(value = "rehabilitationPlan", required = true) String rehabilitationPlan,
                                               @ApiParam(name = "updateStatus", value = "修改类型：0单个服务的修改(默认)，1整个计划修改")
                                               @RequestParam(required = false, defaultValue = "0") String updateStatus) {
        try {
            JSONObject json = new JSONObject(rehabilitationPlan);
            String planId = json.getString("planId");
            String hospitalServiceItemId = json.getString("hospitalServiceItemId");
            //删除该服务项目今日之后未完成的计划
            if (StringUtils.isBlank(updateStatus) || updateStatus.equals("0")) {//单个服务的修改
                rehabilitationPlanService.deleteDetailByPlanIdAndHospitalServiceItemId(planId, hospitalServiceItemId, DateUtil.getDateShort(new Date()));
            } else {
                rehabilitationPlanService.deleteDetailByPlanId(planId, DateUtil.getDateShort(new Date()));
            }
            JSONArray array = new JSONArray();
            for (Object planDetail : json.getJSONArray("detail")) {
                String expense = "";
                JSONObject j = (JSONObject) planDetail;
                if (j.get("expense") instanceof Integer) {
                    expense = String.valueOf(j.getInt("expense"));
                } else {
                    expense = j.getString("expense");
                    if (expense.contains(".")) {
                        int pointIndex = expense.indexOf(".");
                        expense = expense.substring(0, pointIndex);
                    }
                }
                j.put("expense", expense);
                if (j.has("executeTime")) {
                    String executeTime = j.get("executeTime").toString();
                    String[] result = null;
                    if (executeTime.contains(",")) {
                        result = executeTime.split(",");
                    } else {
                        result = new String[1];
                        result[0] = executeTime;
                    }
                    int len = result.length;
                    while (len > 0) {
                        len--;
                        JSONObject temp = new JSONObject(j.toString());
                        temp.put("executeTime", result[len]);
                        temp.put("createUser", json.get("createUser"));
                        temp.put("createUserName", json.get("createUserName"));
                        array.put(temp);
                    }
                } else {
                    j.put("createUser", json.get("createUser"));
                    j.put("createUserName", json.get("createUserName"));
                    array.put(j);
                }
            }
            String planDetails = array.toString();
            ObjectMapper object = new ObjectMapper();
            object.setDateFormat(new SimpleDateFormat("yyyy-MM-dd HH:mm"));
            List<RehabilitationDetailDO> details = object.readValue(planDetails, new TypeReference<List<RehabilitationDetailDO>>() {
            });
            details = rehabilitationPlanService.createRehabilitationDetail(details, planId);
            //调用服务包接口
            return MixEnvelop.getSuccessList(SpecialistMapping.api_success, details);
        } catch (Exception e) {
            e.printStackTrace();
            // tracer.getCurrentSpan().logEvent(e.getMessage())
            return MixEnvelop.getError(e.getMessage());
        }
    }
}
