package com.yihu.jw.hospital.module.rehabilitation.service;


import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.rehabilitation.RehabilitationguidanceDO;
import com.yihu.jw.entity.rehabilitation.RehabilitationguidancePatientDO;
import com.yihu.jw.entity.util.TransforSqlUtl;
import com.yihu.jw.hospital.module.rehabilitation.dao.RehabilitationGuidanceDao;
import com.yihu.jw.hospital.module.rehabilitation.dao.RehabilitationGuidancePatientDao;
import com.yihu.jw.hospital.task.PushMsgTask;
import com.yihu.jw.hospital.utils.WeiXinAccessTokenUtils;
import com.yihu.jw.hospital.utils.WeiXinOpenIdUtils;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.util.date.DateUtil;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springside.modules.persistence.DynamicSpecifications;
import org.springside.modules.persistence.SearchFilter;

import java.util.*;


/**
 * Created by wangzhinan on 2018/8/29.
 */
@Service
@Transactional
public class RehabilitationGuidanceService {
    private Logger logger = LoggerFactory.getLogger(RehabilitationGuidanceService.class);

    @Autowired
    private RehabilitationGuidanceDao rehabilitationGuidanceDao;
    @Autowired
    private RehabilitationGuidancePatientDao rehabilitationGuidancePatientDao;
    @Autowired
    private BaseDoctorDao doctorDao;
    @Autowired
    private BasePatientDao patientDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private PushMsgTask pushMsgTask;

    @Autowired
    private WeiXinAccessTokenUtils weiXinAccessTokenUtils;
    @Autowired
    private WeiXinOpenIdUtils weiXinOpenIdUtils;

    /**
     * 康复指导类型标题
     *
     * @return
     */
    public List<Map<String, Object>> selectType(String planId) {
        String disease = "";
        if (StringUtils.isNoneBlank(planId)) {
            String sql = "select * from wlyy_patient_rehabilitation_plan where id='" + planId + "' ";
            sql = TransforSqlUtl.wlyy_patient_rehabilitation_planAll(sql);
            Map<String, Object> map = jdbcTemplate.queryForMap(sql);
            if (map.get("disease") != null && StringUtils.isNoneBlank(map.get("disease").toString())) {
                disease = map.get("disease").toString();
            }
        }
        StringBuffer sql = new StringBuffer("select g.type,g.type_name as typeName,COUNT(g.id) as number from wlyy_rehabilitation_guidance g where g.del=1 ");
        if (StringUtils.isNoneBlank(disease)) {
            sql.append("and g.type in ( '").append(disease.replace(",", "','")).append("') ");
        }
        sql.append("group by g.type");
        List<Map<String, Object>> mapList = jdbcTemplate.queryForList(sql.toString());
        return mapList;
    }


    /**
     * 查询康复指导列表
     *
     * @param type
     * @param code
     * @param page
     * @param pagesize
     * @return
     */
    public Page<RehabilitationguidanceDO> select(String title, Integer type, String code, String planId, Integer page, Integer pagesize) {
        String disease = "";
        if (StringUtils.isNoneBlank(planId)) {
            String sql = "select * from wlyy_patient_rehabilitation_plan where id='" + planId + "' ";
            sql = TransforSqlUtl.wlyy_patient_rehabilitation_planAll(sql);
            Map<String, Object> map = jdbcTemplate.queryForMap(sql);
            if (map.get("disease") != null && StringUtils.isNoneBlank(map.get("disease").toString())) {
                disease = map.get("disease").toString();
            }
        }
        if (page >= 1) {
            page--;
        }

        if (pagesize <= 0) {
            pagesize = 10;
        }
        StringBuffer sql = new StringBuffer("select * from wlyy_rehabilitation_guidance where 1=1 and del =1 ");
        StringBuffer sqlCounts = new StringBuffer("select count(1) from wlyy_rehabilitation_guidance where 1=1 and del =1 ");
        if (StringUtils.isNoneBlank(code)) {
            sql.append("and code='" + code + "' ");
            sqlCounts.append("and code='" + code + "' ");
        }
        if (type != null) {
            sql.append("and type='" + type + "' ");
            sqlCounts.append("and type='" + type + "' ");
        }
        if (StringUtils.isNoneBlank(title)) {
            sql.append("and title like '%" + title + "%' ");
            sqlCounts.append("and title like '%" + title + "%' ");
        }
        if (StringUtils.isNoneBlank(disease)) {
            sql.append("and type in ( '").append(disease.replace(",", "','")).append("') ");
            sqlCounts.append("and type in ( '").append(disease.replace(",", "','")).append("') ");
        }

        sql.append("order by czrq desc limit " + page * pagesize + "," + (page + 1) * pagesize);
        List<RehabilitationguidanceDO> list = jdbcTemplate.query(sql.toString(), new BeanPropertyRowMapper<>(RehabilitationguidanceDO.class));
        Long totalCount = jdbcTemplate.queryForObject(sqlCounts.toString(), Long.class);
        for (RehabilitationguidanceDO temp : list) {
            String contents = temp.getContent();
            temp.setContent(StringEscapeUtils.unescapeJava(contents));
        }

        PageRequest pageRequest = PageRequest.of(page, pagesize);
        if (list.size() <= 0) {
            return new PageImpl(new ArrayList<>(), pageRequest, 0);
        }
        return new PageImpl(list, pageRequest, totalCount);
    }


    public Page<RehabilitationguidancePatientDO> selectPatientGuidances(String patient, String doctor, String code, Integer page, Integer pagesize) {
        if (page >= 1) {
            page--;
        }

        if (pagesize <= 0) {
            pagesize = 10;
        }

        // 排序
        Sort sort = Sort.by(Sort.Direction.DESC, "czrq");
        // 分页信息

        PageRequest pageRequest = PageRequest.of(page, pagesize, sort);
        // 设置查询条件
        Map<String, SearchFilter> filters = new HashMap<String, SearchFilter>();
        // 指定患者
        if (StringUtils.isNoneBlank(code)) {
            filters.put("code", new SearchFilter("code", SearchFilter.Operator.EQ, code));
        }
        if (StringUtils.isNoneBlank(patient)) {
            filters.put("patient", new SearchFilter("patient", SearchFilter.Operator.EQ, patient));
        }

        if (StringUtils.isNoneBlank(doctor)) {
            filters.put("doctor", new SearchFilter("doctor", SearchFilter.Operator.EQ, doctor));
        }
        Specification<RehabilitationguidancePatientDO> spec = DynamicSpecifications.bySearchFilter(filters.values(), RehabilitationguidancePatientDO.class);
        Page<RehabilitationguidancePatientDO> rehabilitationguidancePatientDOS = rehabilitationGuidancePatientDao.findAll(spec, pageRequest);
        for (RehabilitationguidancePatientDO rehabilitationguidancePatientDO : rehabilitationguidancePatientDOS) {
            BasePatientDO patient1 = patientDao.findById(rehabilitationguidancePatientDO.getPatient()).orElse(null);
            rehabilitationguidancePatientDO.setPhoto(patient1.getPhoto());
        }
        return rehabilitationguidancePatientDOS;
    }

    /**
     * 发送康复指导
     *
     * @param rehabilitationguidancePatientDOS
     * @return
     */
    public List<RehabilitationguidancePatientDO> sendGuidanceToPatient(List<RehabilitationguidancePatientDO> rehabilitationguidancePatientDOS, String planId) throws Exception {
        for (RehabilitationguidancePatientDO rehabilitationguidancePatientDO : rehabilitationguidancePatientDOS) {
            RehabilitationguidanceDO rehabilitationguidanceDO = rehabilitationGuidanceDao.findByCode(rehabilitationguidancePatientDO.getArticle());
            rehabilitationguidanceDO.setUseCount(rehabilitationguidanceDO.getUseCount() + 1);
            rehabilitationGuidanceDao.save(rehabilitationguidanceDO);
            BasePatientDO patient = patientDao.findById(rehabilitationguidancePatientDO.getPatient()).orElse(null);
            BaseDoctorDO doctor = doctorDao.findById(rehabilitationguidancePatientDO.getDoctor()).orElse(null);
            rehabilitationguidancePatientDO.setAttachedTitle(rehabilitationguidanceDO.getTitle());
            if (patient != null) {
                rehabilitationguidancePatientDO.setName(patient.getName());
            }
            if (doctor != null) {
                rehabilitationguidancePatientDO.setDoctorName(doctor.getName());
            }
            rehabilitationguidancePatientDO.setAttachedContent(rehabilitationguidanceDO.getContent());
            rehabilitationguidancePatientDO.setCzrq(new Date());
            rehabilitationguidancePatientDO.setIsRead(0);
            rehabilitationguidancePatientDO.setSendStatus(0);
            rehabilitationguidancePatientDO = rehabilitationGuidancePatientDao.save(rehabilitationguidancePatientDO);
            this.sendSpecialistWeixinMessage(rehabilitationguidancePatientDO.getPatient(), rehabilitationguidancePatientDO.getDoctor(), 1, rehabilitationguidancePatientDO.getArticle(), planId, null);
        }
        return rehabilitationguidancePatientDOS;

    }


    /**
     * 查看发送的康复指导
     *
     * @param patient
     * @param doctor
     * @param relationCode
     * @return
     */
    public List<RehabilitationguidancePatientDO> selectByRelationCode(String patient, String doctor, String relationCode) {
        if (StringUtils.isNoneBlank(relationCode)) {
            return rehabilitationGuidancePatientDao.findByRelationCode(relationCode);
        } else {
            return rehabilitationGuidancePatientDao.findByPatientAndDoctor(patient, doctor);
        }
    }


    /**
     * 康复计划-微信模板发送
     *
     * @param patientCode  居民code
     * @param doctorCode   专科医生code
     * @param type         1康复指导2康复下转3复诊提醒4上门服务提醒5家签提醒
     * @param relationCode 业务关联code
     * @param planId       计划id
     * @throws Exception
     */
    public void sendSpecialistWeixinMessage(String patientCode, String doctorCode, Integer type, String relationCode, String planId, String reservationType) throws Exception {
        //康复指导
        BasePatientDO patient = patientDao.findById(patientCode).orElse(null);
        BaseDoctorDO doctor = doctorDao.findById(doctorCode).orElse(null);
//        SignFamily signFamily = signFamilyDao.findByPatient(patientCode);

        String name = "";
        String doctorName = "";
        String familyDoctorName = "";
        org.json.JSONObject json = new org.json.JSONObject();
        if (patient != null) {
            name = patient.getName();
        }
        if (doctor != null) {
            doctorName = doctor.getName();
        }
//        if (signFamily != null) {
//            familyDoctorName = signFamily.getDoctorName();
//        }
        Integer sendType = null;
        if (type == 1) {
            RehabilitationguidanceDO rehabilitationguidanceDO = rehabilitationGuidanceDao.findByCode(relationCode);
            json.put("toUser", patientCode);
            json.put("first", patient.getName() + "，您好！您的家庭医生发来一篇健康文章！");
            json.put("represented", patientCode);//被代理人
            json.put("id", planId);
            json.put("date", DateUtil.getStringDate());
//            json.put("orgName", doctor.getHospitalName());
            json.put("doctorName", doctor.getName());
            json.put("content", "《" + rehabilitationguidanceDO.getTitle() + "》");
            json.put("remark", "请及时阅读");
            json.put("url", "kfgl/html/plan-item-detail.html?planIds=" + planId + "");
            sendType = 36;
        } else if (type == 2) {
            //反馈处理30
            sendType = 30;
            String sql = "SELECT pr.manage_time as manageTime,pr.is_manage as isManage " +
                    "FROM `wlyy_specialist_patient_relation` pr " +
                    "where pr.doctor = '" + doctorCode + "' and pr.patient = '" + patientCode + "' and pr.sign_status=1 and pr.`status`=1";
            List<Map<String, Object>> mapList = jdbcTemplate.queryForList(sql);
            String manageTime = "";
            for (Map<String, Object> map : mapList) {
                manageTime = DateUtil.dateToStrLong((Date) map.get("manageTime"));
            }
            json.put("first", patient.getName() + "，您好！您已成功签约专科医生“" + doctorName + "”，" + doctorName + "医生（专科医生）和" + familyDoctorName + "（全科医生）将为您提供康复管理服务");
            json.put("toUser", patientCode);
            json.put("represented", patientCode);//被代理人
            json.put("keyword1", manageTime);
            json.put("keyword2", "康复下转");
            json.put("remark", "请及时阅读");
            json.put("relationCode", 1);
            json.put("url", "kfgl/html/recover_manage.html");
        } else if (type == 3) {
            //复诊提醒3
            //更新复诊类型
            if (StringUtils.isNoneBlank(reservationType)) {
                String updSql = "update wlyy_rehabilitation_plan_detail set reservation_type = " + reservationType + " where id = '" + relationCode + "'";
                jdbcTemplate.execute(updSql);
            }
            sendType = 11;//待办事项
            String sql = "select pd.id,pd.execute_time as executeTime from wlyy_rehabilitation_plan_detail pd " +
                    "where pd.id ='" + relationCode + "' and pd.hospital_service_item_id='7' and pd.status <>1 order by pd.execute_time desc ";
            List<Map<String, Object>> mapList = jdbcTemplate.queryForList(sql);
            String excuteTime = "";
            if (mapList != null && mapList.size() != 0) {
                excuteTime = DateUtil.dateToStrLong((Date) mapList.get(0).get("executeTime"));
            }
            if (StringUtils.isNoneBlank(excuteTime)) {
                json.put("first", patient.getName() + "，您好！您的下次复诊时间是[" + excuteTime + "]，请按时安排就诊。您可点击“查看详情”，提前预约就诊");
            } else {
                json.put("first", patient.getName() + "，您好！" + doctor.getName() + "医生提醒您预约复诊。您可点击“查看详情”，提前预约就诊");
            }
            json.put("toUser", patientCode);
            json.put("represented", patientCode);//被代理人
            json.put("keyword1", "复诊预约");
            json.put("keyword2", "" + DateUtil.getStringDate() + "");
            json.put("remark", "请及时阅读");
            json.put("url", "wdyy/html/appointment-register.html");
        } else if (type == 4) {
            //上门服务提醒 4
            sendType = 11;//待办事项
            String sql = "select pd.id,pd.execute_time as executeTime from wlyy_rehabilitation_plan_detail pd " +
                    "where pd.id ='" + relationCode + "' and pd.hospital_service_item_id='4' and pd.status <>1 order by pd.execute_time desc ";
            List<Map<String, Object>> mapList = jdbcTemplate.queryForList(sql);
            String excuteTime = "";
            String detailId = "";
            if (mapList != null && mapList.size() != 0) {
                excuteTime = DateUtil.dateToStrLong((Date) mapList.get(0).get("executeTime"));

            }
            if (StringUtils.isNoneBlank(excuteTime)) {
                json.put("first", patient.getName() + "，您好！您的下次上门诊疗服务时间是[" + excuteTime + "]，请按时预约上门诊疗服务。您可点击“查看详情”，提前预约就诊");
            } else {
                json.put("first", patient.getName() + "，您好！" + doctor.getName() + "医生提醒您预约上门服务。您可点击“查看详情”，提前预约就诊");
            }

            json.put("toUser", patientCode);
            json.put("represented", patientCode);//被代理人
            json.put("keyword1", "上门诊疗服务预约");
            json.put("keyword2", DateUtil.getStringDate());
            json.put("remark", "请及时阅读");
            json.put("url", "smfw/html/toAppointment.html?relationCode=" + relationCode + "");
        } else if (type == 5) {
            //家签提醒 5
            sendType = 11;//待办事项
            String sql = "select pd.id,pd.execute_time as executeTime from wlyy_rehabilitation_plan_detail pd " +
                    "where pd.plan_id ='" + planId + "' and pd.hospital_service_item_id='4' and pd.status <>1 order by pd.execute_time desc ";
            List<Map<String, Object>> mapList = jdbcTemplate.queryForList(sql);
            String excuteTime = "";
            if (mapList != null && mapList.size() != 0) {
                excuteTime = DateUtil.dateToStrLong((Date) mapList.get(0).get("executeTime"));
            }
            json.put("first", patient.getName() + "，您好！为方便为您提供康复管理服务，请先选择合适的社区和家庭医生进行签约");
            json.put("toUser", patientCode);
            json.put("represented", patientCode);//被代理人
            json.put("keyword1", "家庭医生签约");
            json.put("keyword2", DateUtil.getStringDate());
            json.put("remark", "请及时阅读");
            json.put("url", "qygl/html/signing-welcome.html");
        }
        logger.info("微信模板入参:" + sendType + "==" + json.toString());
        if (StringUtils.isNotBlank(patient.getOpenid())) {
            pushMsgTask.putWxMsg(weiXinAccessTokenUtils.getAccessToken(), sendType, patient.getOpenid(), patient.getName(), json);
        } else {
            //发送代理人
            org.json.JSONArray jsonArray = weiXinOpenIdUtils.getAgentOpenId(patient.getId(), patient.getOpenid());
            if (jsonArray != null && jsonArray.length() > 0) {
                for (int i = 0; i < jsonArray.length(); i++) {
                    org.json.JSONObject j = jsonArray.getJSONObject(i);
                    BasePatientDO member = (BasePatientDO) j.get("member");
                    org.json.JSONObject data = json;
                    data.remove("toUser");
                    data.put("toUser", member.getId());
                    data.remove("first");
                    data.put("first", weiXinOpenIdUtils.getTitleMes(patient, j.getInt("relation"), patient.getName()));
                    pushMsgTask.putWxMsg(weiXinAccessTokenUtils.getAccessToken(), sendType, member.getOpenid(), patient.getName(), data);
                }
            }
        }

    }


}
