package com.yihu.jw.hospital.module.rehabilitation.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.doctor.dao.BaseDoctorDao;
import com.yihu.jw.entity.base.doctor.BaseDoctorDO;
import com.yihu.jw.entity.base.patient.BasePatientDO;
import com.yihu.jw.entity.door.SignFamily;
import com.yihu.jw.entity.rehabilitation.PatientDischargeDO;
import com.yihu.jw.entity.specialist.rehabilitation.PatientMedicalRecordsDO;
import com.yihu.jw.entity.specialist.rehabilitation.RehabilitationAdviceDO;
import com.yihu.jw.entity.specialist.rehabilitation.RehabilitationPatientInfoDO;
import com.yihu.jw.entity.util.TransforSqlUtl;
import com.yihu.jw.hospital.HospitalDao;
import com.yihu.jw.hospital.module.rehabilitation.dao.PatientDischargeDao;
import com.yihu.jw.hospital.module.rehabilitation.dao.RehabilitationAdviceDao;
import com.yihu.jw.hospital.module.rehabilitation.dao.RehabilitationPatientInfoDao;
import com.yihu.jw.hospital.module.specialist.dao.PatientMedicalRecordsRehabilitationDao;
import com.yihu.jw.hospital.module.system.dao.SystemDictDao;
import com.yihu.jw.hospital.task.PushMsgTask;
import com.yihu.jw.hospital.team.dao.WlyySignFamilyDao;
import com.yihu.jw.hospital.utils.WeiXinAccessTokenUtils;
import com.yihu.jw.hospital.utils.WeiXinOpenIdUtils;
import com.yihu.jw.patient.dao.BasePatientDao;
import com.yihu.jw.patient.service.BasePatientService;
import com.yihu.jw.util.common.IdCardUtil;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.http.HttpClientUtil;
import com.yihu.jw.wlyy.service.WlyyBusinessService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by humingfen on 2020/2/26.
 */
@Service
@Transactional
public class RehabilitationInfoService {
    @Autowired
    private HospitalDao hospitalDao;
    @Autowired
    private WeiXinAccessTokenUtils weiXinAccessTokenUtils;
    @Autowired
    private PushMsgTask pushMsgTask;
    @Autowired
    private WeiXinOpenIdUtils weiXinOpenIdUtils;
    @Autowired
    private PatientDischargeDao patientDischargeDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private HttpClientUtil httpClientUtil;
    @Autowired
    private BasePatientService patientService;
    @Autowired
    private SystemDictDao systemDictDao;
    @Autowired
    private BaseDoctorDao doctorDao;
    @Autowired
    private BasePatientDao patientDao;
    @Autowired
    private WlyySignFamilyDao signFamilyDao;
    @Autowired
    private RehabilitationAdviceDao adviceDao;
    @Autowired
    private PatientMedicalRecordsRehabilitationDao patientMedicalRecordsDao;
    @Autowired
    private WlyyBusinessService businessService;
    @Autowired
    private RehabilitationPatientInfoDao rehabilitationPatientInfoDao;


    public PatientMedicalRecordsDO findByCode(String code) {
        return patientMedicalRecordsDao.findByCode(code);
    }

    /**
     * 提醒家医接收
     */
    public void remindDoctorReceive(String diseaseName, String specialistCode, String doctorCode) {
        BaseDoctorDO doctor = doctorDao.findById(doctorCode).orElse(null);
        if (StringUtils.isNoneBlank(doctor.getOpenid())) {
            BaseDoctorDO specialist = doctorDao.findById(specialistCode).orElse(null);
            //这边先注释掉
            //[医生姓名]医生您好，您收到来自[医院名称]的[医生名称]下转转的[疾病名称]疑似病历患病居民，请访问厦门i健康APP疾病筛查菜单查看。
//            String first = doctor.getName()+"医生您好，您收到来自"+specialist.getHospitalName()+"的"+specialist.getName()+"下转的"+diseaseName.replace(","," ")+"疑似病历患病居民，请访问厦门i健康APP疾病筛查菜单查看。";
//            doctorAssistantUtil.sendWXTemplate(9,specialist.getOpenid(),specialist.getCid(),first,"请及时处理","筛查转诊",specialist.getHospitalName(),specialist.getName());
        }

    }

    /**
     * 获取出院小结列表
     *
     * @param idCard
     * @param startDate
     * @param endDate
     * @return
     */
    public List<Map<String, String>> getEmrList(String idCard, String startDate, String endDate, Integer page, Integer size) {
        List<Map<String, String>> res = new ArrayList<>();
        if (StringUtils.isBlank(startDate) && StringUtils.isBlank(endDate)) {
            //如果没有时间，默认查下一年的数据
            endDate = DateUtil.getStringDateShort();
            startDate = DateUtil.getNextYear(new Date(), -1);
        }
        String re = "{\"msg\":\"查询成功\",\"data\":\"{\\\"code\\\":\\\"0\\\",\\\"message\\\":\\\"null\\\",\\\"data\\\":[{\\\"diagDept\\\":\\\"消化内科\\\",\\\"diagNote\\\":\\\"\\\",\\\"diagTime\\\":1551775118000,\\\"diagType\\\":\\\"1\\\",\\\"diagnosisName\\\":\\\"测试全院共享诊断,过敏性紫癜\\\",\\\"eventNo\\\":\\\"0626463001\\\",\\\"idNo\\\":\\\"352625196212160030\\\",\\\"name\\\":\\\"刘新树\\\",\\\"orgCode\\\":\\\"350211A2001\\\",\\\"orgName\\\":\\\"厦门市中医院\\\",\\\"patientId\\\":\\\"3037361346\\\",\\\"residenceFlag\\\":\\\"1\\\",\\\"residenceInTime\\\":1551775118000,\\\"rowId\\\":\\\"352625196212160030_350211A4444_1_3037361346_0626463001_24F430C3-E0BD-49C6-AFF9-D200FC85C85C\\\"}]}\",\"status\":200}";
        JSONObject json = JSONObject.parseObject(re);
        if (json.getInteger("status") == 200) {
            JSONObject dataTmp = JSONObject.parseObject(json.getString("data"));
            if ("0".equals(dataTmp.getString("code"))) {
                JSONArray data = dataTmp.getJSONArray("data");
                for (int i = (page - 1) * size; i < data.size(); i++) {
                    Map<String, String> map = new HashMap<>();
                    JSONObject item = data.getJSONObject(i);
                    map.put("id", item.getString("eventNo"));
                    map.put("patient", item.getString("patientId"));
                    map.put("eventDate", DateUtil.dateToStrLong(new Date(item.getLong("diagTime"))));
                    map.put("eventType", "2");//都是住院记录
                    map.put("orgName", item.getString("orgName"));
                    map.put("orgCode", item.getString("orgCode"));
                    map.put("dianosis", item.getString("diagnosisName"));//诊断
                    map.put("createTime", DateUtil.dateToStrLong(new Date(item.getLong("diagTime"))));
                    map.put("dataFrom", "4");//6家医院出院小结
                    res.add(map);
                    if (res.size() >= size) {
                        return res;
                    }
                }
            }
        }
        return res;
    }

    /**
     * 获取厦心出院就诊记录信息
     *
     * @param patientCode
     */
    public List<Map<String, Object>> getMedicalHistoryList(String patientCode, Integer page, Integer size) {
        BasePatientDO patient = patientDao.findById(patientCode).orElse(null);
        if (page <= 0) {
            page = 1;
        }
        if (size <= 0) {
            size = 10;
        }
        String sexName = "";
        if (patient.getSex() != null) {
            sexName = IdCardUtil.getSexNameBySexOrIdcard(patient.getSex() + "", patient.getIdcard());
        } else {
            sexName = IdCardUtil.getSexNameBySexOrIdcard(null, patient.getIdcard());
        }
        Integer age = IdCardUtil.getAgeByIdcardOrBirthday(patient.getIdcard(), patient.getBirthday());
        String sql = "select code,patient_name name,visit_time visitTime from wlyy_xxzx_medical_history " +
                "where id_card_no = '" + patient.getIdcard() + "' order by visit_time desc limit " + (page - 1) * size + "," + size;
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        for (Map<String, Object> map : list) {
            map.put("hospitalName", "厦心");
            map.put("sex", sexName);
            map.put("age", age);
        }

        return list;
    }


    /**
     * 新增或编辑康复建议模板
     * doctorCode
     * code 模板code
     * name
     * content
     */
//    public void createAdvice(String doctorCode, String code, String name, String content) {
//        RehabilitationAdviceDO adviceDO = null;
//        //判断是新增还是编辑
//        if (StringUtils.isNotBlank(code)) {
//            adviceDO = adviceDao.findByCode(code);
//        } else {
//            adviceDO = new RehabilitationAdviceDO();
//            adviceDO.setCode(getCode());
//            BaseDoctorDO doctor = doctorDao.findByCode(doctorCode);
//            adviceDO.setCreateUser(doctorCode);
//            adviceDO.setCreateUserName(doctor.getName());
//        }
//        adviceDO.setName(name);
//        adviceDO.setContent(content);
//        adviceDO.setCreateTime(new Date());
//        adviceDao.save(adviceDO);
//    }

//    public void deleteAdvice(Long id) {
//        adviceDao.delete(id);
//    }

    /**
     * 获取医生创建康复建议模板及系统模板
     * @param doctorCode
     * @return
     */
//    public List<RehabilitationAdviceDO> getAdviceListByDoctor(String doctorCode) {
//        List<RehabilitationAdviceDO> list = adviceDao.findByDoctor(doctorCode);
//        return list;
//    }

    /**
     * 新增住院病历
     *
     * @param doctorCode
     * @param patient
     * @param patientName
     * @param patientMedicalRecordsDO
     * @param id
     * @return
     */
    public PatientMedicalRecordsDO createMedicalRecords(String doctorCode, String patient, String patientName, PatientMedicalRecordsDO patientMedicalRecordsDO, Long id) {
        BaseDoctorDO doctor = doctorDao.findById(doctorCode).orElse(null);
        if (id != null) {
            PatientMedicalRecordsDO oldRecord = patientMedicalRecordsDao.findById(id).orElse(null);
            patientMedicalRecordsDO.setId(id.toString());
            patientMedicalRecordsDO.setCode(oldRecord.getCode());
        } else {
            patientMedicalRecordsDO.setCode(UUID.randomUUID().toString().replaceAll("-", ""));
        }
        patientMedicalRecordsDO.setPatient(patient);
        patientMedicalRecordsDO.setPatientName(patientName);
        patientMedicalRecordsDO.setCreateUser(doctorCode);
        patientMedicalRecordsDO.setCreateUserName(doctor.getName());
        patientMedicalRecordsDO.setCreateTime(new Date());
        patientMedicalRecordsDao.save(patientMedicalRecordsDO);
        return patientMedicalRecordsDO;
    }

    /**
     * 获取出院居民列表
     *
     * @param hospital
     * @return
     * @throws Exception
     */
    public JSONObject getDischargePatientList(String hospital, String doctorCode) throws Exception {
        JSONObject result = new JSONObject();
        List<Map<String, Object>> signList = new ArrayList<>();
        List<Map<String, Object>> nosignList = new ArrayList<>();
        //获取当前医生映射机构
//        HospitalMapping hospitalMapping = hospitalMappingDao.findByCode(hospital);
        String value = systemDictDao.findByDictNameAndCode("kfxz", hospital);
        if (value != null) {
            //查询已签约居民
            String signSql = "SELECT  i.idcard,s.patient,i.`name` patientName, i.sex,i.age,i.address,CONCAT(s.doctor_name ,'(全科医生)') as doctorName,IFNULL(CONCAT(s.doctor_health_name ,'(健管师)'),s.doctor_health_name) doctorHealthName, s.hospital_name hospitalName,s.admin_team_code teamCode " +
                    "FROM `wlyy_rehabilitation_patient_info` i , wlyy_sign_family s where i.jw_hospital_code='" + value + "' and  i.idcard = s.idcard and s.`status` = 1 and s.expenses_status = 1 group by i.idcard ORDER BY i.crzq desc ";
            signList = jdbcTemplate.queryForList(signSql);
            //查询代签约居民
            String nosignSql = "SELECT  i.idcard,p.`code` patient,i.`name` patientName, i.sex,i.age,i.address " +
                    "FROM `wlyy_rehabilitation_patient_info` i JOIN wlyy_patient p on i.idcard =  p.idcard and p.`status` = 1  LEFT JOIN wlyy_sign_family s on i.idcard = s.idcard " +
                    "where i.jw_hospital_code='" + value + "' and ( s.sign_year != '" + DateUtil.getSignYear() + "' or ( s.sign_year = '" + DateUtil.getSignYear() + "' and ( s.expenses_status != 1 or s.`status` != 1))) group by i.idcard ORDER BY i.crzq desc ";
            nosignList = jdbcTemplate.queryForList(nosignSql);
        }
        //获取自助添加出院居民列表
        List<PatientDischargeDO> patientDischargeDOS = patientDischargeDao.findAllByCreateUser(doctorCode);
        for (PatientDischargeDO patientDischargeDO : patientDischargeDOS) {
            Map jsonObject = JSONObject.parseObject(JSONObject.toJSONString(patientDischargeDO), Map.class);
//            JSONObject jsonObject = JSONObject.parseObject(JSONObject.toJSONString(patientDischargeDO));
            jsonObject.put("patientName", patientDischargeDO.getName());
            jsonObject.put("patient", patientDischargeDO.getPatient());
            String idcard = patientDischargeDO.getIdcard();
            //根据身份证获取居民性别跟年龄
            String sex = IdCardUtil.getSexForIdcard(idcard);
            Integer age = IdCardUtil.getAgeForIdcard(idcard);
            jsonObject.put("sex", sex);
            jsonObject.put("age", age);
            SignFamily signFamily = signFamilyDao.getExpensesSignByIdcard(idcard);
            if (signFamily != null) {
                jsonObject.put("doctorName", signFamily.getDoctorName() + "(全科医生)");
                if (StringUtils.isNotBlank(signFamily.getDoctorHealthName())) {
                    jsonObject.put("doctorHealthName", signFamily.getDoctorHealthName() + "(健管师)");
                }
                jsonObject.put("hospitalName", signFamily.getHospitalName());
                signList.add(jsonObject);
            } else {
                nosignList.add(jsonObject);
            }
        }
        result.put("noSignArray", nosignList);
        result.put("signArray", signList);
        return result;
    }


    /**
     * 未下转未家签
     */
    public JSONObject noDownAndNoSign(String idcard, String doctorCode, Integer page, Integer pagesize, JSONObject object) {
        String filterSql = " and a.create_user ='" + doctorCode + "'";
        if (StringUtils.isNotBlank(idcard)) {
            filterSql += " and  b.idcard like '%" + idcard + "%' ";
        }
        String sql = "SELECT\n" +
                "	a.idcard,a.id 'patientId',a.`name` 'patientName',a.sex,NULL age,a.address \n" +
                "FROM\n" +
                "	wlyy_rehabilitation_patient_info a\n" +
                "	INNER JOIN base_patient b ON a.patient = b.id \n" +
                "WHERE\n" +
                "	1 = 1 \n" + filterSql +
                "	AND (a.type = 2 OR ISNULL( a.type )) \n" +
                "	AND a.sign_status='0'\n" +
                "	AND a.`status` = '6'\n" +
                "	AND a.patient NOT IN (\n" +
                "	  SELECT i.id \n" +
                "	  FROM wlyy_patient_rehabilitation_plan i INNER JOIN base_patient p ON p.id = i.patient \n" +
                "	  WHERE	1=1 AND	i.create_user = '" + doctorCode + "' AND i.STATUS != 0 \n" +
                "	  GROUP BY	i.patient\n" +
                "	) \n" +
                "ORDER BY	a.create_time DESC ";

        String pageSql = " LIMIT " + (page - 1) * pagesize + "," + pagesize;
        sql += pageSql;

        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        list.stream().forEach(one -> {
            one.put("doctorName", null);
            one.put("doctorHealthName", null);
            one.put("hospitalName", null);
            one.put("teamCode", null);
            //isTurnDown 是否已经下转 0未下转，1以下转
            one.put("isTurnDown", -2);
            String idcardNo = one.get("idcard") + "";
            Integer age = IdCardUtil.getAgeForIdcard(idcardNo);
            one.put("age", age);
        });
        object.put("res", list);
        return noDownCount(doctorCode, object);
    }

    /**
     * 未下转统计
     */
    public JSONObject noDownCount(String doctorCode, JSONObject object) {
        //未下转所有
        String notTurnedDownCountSql = "select count(*) \n" +
                "from wlyy_rehabilitation_patient_info d INNER JOIN base_patient p ON p.id = d.patient \n" +
                "  where d.create_user ='" + doctorCode + "' and ( ISNULL(d.type) or d.type=2)  AND d.patient NOT IN (   \n" +
                "		SELECT i.patient FROM wlyy_patient_rehabilitation_plan i,base_patient p \n" +
                "		WHERE i.create_user = '" + doctorCode + "' AND i. STATUS != 0 AND p.id = i.patient ) ";

        //未家签
        String notSignedAtHomeSql = "SELECT count(*) FROM (\n" +
                "select '' id, p.idcard,p.id 'patientId',p.`name` patientName,p.sex,NULL age,p.address  \n" +
                " from wlyy_rehabilitation_patient_info d INNER JOIN base_patient p ON  p.id = d.patient\n" +
                " where d.create_user ='" + doctorCode + "' and ( ISNULL(d.type) or d.type=2) AND d.sign_status='0' \n" +
                " AND d.patient NOT IN ( \n" +
                "	SELECT i.patient FROM wlyy_patient_rehabilitation_plan i INNER JOIN base_patient p  ON p.id = i.patient \n" +
                "	WHERE i.create_user = '" + doctorCode + "' AND i. STATUS != 0 \n" +
                "	GROUP BY i.patient \n" +
                "  )GROUP BY d.patient \n" +
                ")aa";

        String notSignedAtHome1Sql = "SELECT count(*) FROM (\n" +
                "	select '' id, p.idcard,p.id 'patientId',p.`name` patientName,p.sex,NULL age,p.address\n" +
                "	from wlyy_rehabilitation_patient_info d \n" +
                "	INNER JOIN base_patient p ON  p.id= d.patient \n" +
                "	where d.create_user = '" + doctorCode + "' and ( ISNULL(d.type) or d.type=2)  AND d.sign_status='1'\n" +
                "	AND d.patient NOT IN (  \n" +
                "  	SELECT DISTINCT i.patient FROM wlyy_patient_rehabilitation_plan i,base_patient p   \n" +
                "	  WHERE i.create_user = '" + doctorCode + "'  AND i. STATUS != 0 AND p.id = i.patient  \n" +
                "	)\n" +
                "	GROUP BY d.patient\n" +
                ")aa";

        Integer allCount = jdbcTemplate.queryForObject(notTurnedDownCountSql, Integer.class);
        Integer notSignedAtHome = jdbcTemplate.queryForObject(notSignedAtHomeSql, Integer.class);
        Integer notSignedAtHome1 = jdbcTemplate.queryForObject(notSignedAtHome1Sql, Integer.class);
        object.put("allCount", allCount);//全部
        object.put("notTurnedDownCount", notSignedAtHome1);//签约的
        object.put("notSignedAtHome", notSignedAtHome);//未家签
        return object;
    }

    /**
     * 未下转已家签
     */
    public JSONObject noDownAndSign(String idcard, String doctorCode, Integer page, Integer pagesize, JSONObject object) {
        String filterSql = " and  d.create_user ='" + doctorCode + "'";
        if (StringUtils.isNotBlank(idcard)) {
            filterSql += " and  p.idcard like '%" + idcard + "%' ";
        }
//        String sql = "select '' id, p.idcard,p.code patient,p.NAME patientName,p.sex,NULL age,p.address,f.status signStatus," +
//                "f.czrq,f.doctor_name doctorName,f.doctor_health_name doctorHealthName,f.hospital_name hospitalName,f.admin_team_code teamCode " +
//                " from wlyy_patient_discharge d,wlyy_patient p,wlyy_sign_family f\n" +
//                "where 1=1 " + filterSql + " and ( ISNULL(d.type) or d.type=2) AND p. CODE = d.patient AND d.patient=f.patient AND f.`status`=1 AND f.sign_year='" + DateUtil.getSignYear() + "' AND d.patient NOT IN \n" +
//                "(   SELECT i.patient FROM wlyy_patient_rehabilitation_plan i,wlyy_patient p   WHERE i.create_user = '" + doctorCode + "' \n" +
//                "AND i. STATUS != 0 AND p. CODE = i.patient GROUP BY i.patient ORDER BY i.create_time desc)GROUP BY d.patient ORDER BY d.create_time DESC ";

        String sql = "SELECT  \n" +
                " p.idcard,p.id patient,p.`name` 'patientName',p.sex,NULL age,p.address,\n" +
                " d.`dept`,d.`dept_name`\n" +
                " FROM wlyy_rehabilitation_patient_info d INNER JOIN base_patient p ON  p.id = d.patient \n" +
                " WHERE  1=1 \n" + filterSql +
                " AND ( ISNULL(d.type) or d.type=2)  AND d.sign_status='1' \n" +
                " AND d.patient NOT IN (\n" +
                "		SELECT DISTINCT i.patient FROM wlyy_patient_rehabilitation_plan i,base_patient p  \n" +
                "		WHERE i.create_user = '" + doctorCode + "' AND i. STATUS != 0 AND p.id = i.patient\n" +
                ")\n" +
                "GROUP BY d.patient ORDER BY d.create_time DESC ";

        sql += " LIMIT " + (page - 1) * pagesize + "," + pagesize;
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        list.stream().forEach(one -> {
            String patient = one.get("patient").toString();
            //isTurnDown 是否已经下转 0未下转已家签
            one.put("isTurnDown", -1);
            String idcardNo = one.get("idcard") + "";
            Integer age = IdCardUtil.getAgeForIdcard(idcardNo);
            one.put("age", age);
        });

        object.put("res", list);
        return noDownCount(doctorCode, object);
    }

    /**
     * 未下转全部
     */
    public JSONObject noDown(String idcard, String doctorCode, Integer page, Integer pagesize, JSONObject object) {

        String filterSql = " and  d.create_user ='" + doctorCode + "'";
        if (StringUtils.isNotBlank(idcard)) {
            filterSql += " and  p.idcard like '%" + idcard + "%' ";
        }
//        String sql = "select '' id, p.idcard,p.code patient,p.NAME patientName,p.sex,NULL age,p.address," +
//                "f.status signStatus,f.czrq,f.doctor_name doctorName,f.doctor_health_name doctorHealthName,f.hospital_name hospitalName,f.admin_team_code teamCode," +
//                " if(f.id is null,-2,-1) as isTurnDown " +
//                " from wlyy_patient p,wlyy_patient_discharge d " +
//                " left JOIN wlyy_sign_family f on d.patient=f.patient AND f.`status`=1 AND f.sign_year='" + DateUtil.getSignYear() + "'" +
//                "where 1=1 " + filterSql + " and ( ISNULL(d.type) or d.type=2) AND p. CODE = d.patient  AND d.patient NOT IN \n" +
//                "(   SELECT DISTINCT i.patient FROM wlyy_patient_rehabilitation_plan i,wlyy_patient p   WHERE i.create_user = '" + doctorCode + "' \n" +
//                "AND i. STATUS != 0 AND p. CODE = i.patient )GROUP BY d.patient ORDER BY d.create_time DESC ";

        String sql = "SELECT '' id, p.idcard,p.id 'patient',p.`name` patientName,p.sex,d.age,p.address,\n" +
                " if(p.sign_status=0,-2,-1) as isTurnDown  \n" +
                " FROM base_patient p \n" +
                " INNER JOIN wlyy_rehabilitation_patient_info d  ON p.id = d.patient  \n" +
                " WHERE 1=1  " + filterSql +
                "	 AND d.create_user ='" + doctorCode + "'  AND ( ISNULL(d.type) or d.type=2) \n" +
                "    AND d.`status`='0'\n" +
                "GROUP BY d.patient ORDER BY d.create_time DESC ";
        sql += " LIMIT " + (page - 1) * pagesize + "," + pagesize;
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        list.stream().forEach(one -> {
            String idcardNo = one.get("idcard") + "";
            Integer age = IdCardUtil.getAgeForIdcard(idcardNo);
            one.put("age", age);
        });
        object.put("res", list);
        return noDownCount(doctorCode, object);
    }

    /**
     * 已下转未接收
     */
    public JSONObject downAndNoSign(String idcard, String doctorCode, Integer page, Integer pagesize, JSONObject object) {
//        String sql2 = "SELECT i.disease_name diseaseName,s.doctor,s.idcard,p.sex, s.patient,CAST(IFNULL(p.birthday,'') AS char ) birthday," +
//                " i. STATUS, p. NAME patientName, p.address, CONCAT(s.doctor_name,'(全科医生)' ) AS doctorName, \n" +
//                "IFNULL(CONCAT(s.doctor_health_name,'(健管师)'), s.doctor_health_name ) doctorHealthName, s.hospital_name hospitalName, \n" +
//                "s.admin_team_code teamCode ,pr.is_manage isManage \n" +
//                "FROM wlyy_patient_rehabilitation_plan i, wlyy_sign_family s, wlyy_patient p ,\n" +
//                "wlyy_specialist_patient_relation pr \n" +
//                "WHERE i.create_user = '" + doctorCode + "' AND i.status != 0 AND i.patient = s.patient AND p.code = i.patient AND s.status = 1 \n" +
//                "AND s.expenses_status = 1 AND pr.patient=p.code and pr.team_code = i.team_code and pr.`status`>=0 and pr.sign_status=1 " +
//                " and  (pr.is_manage != 1 OR isnull(pr.is_manage)) ";


        String sql2 = "SELECT \n" +
                " i.disease_name diseaseName,i.create_user,p.idcard,p.sex,p.id 'patient',\n" +
                " i.`status`, p.`name` 'patientName', p.address, \n" +
                " CAST(IFNULL(p.birthday,'') AS char ) birthday,\n" +
                " pr.is_manage isManage \n" +
                "FROM wlyy_patient_rehabilitation_plan i \n" +
                "INNER JOIN wlyy_rehabilitation_patient_info q ON q.patient=i.id AND q.sign_status='1' AND i.patient_info_code=q.`code` \n" +
                "INNER JOIN base_patient p ON  p.id = i.patient\n" +
                "INNER JOIN  wlyy_specialist_patient_relation pr ON  pr.patient=p.id\n" +
                "WHERE i.create_user = '" + doctorCode + "' AND i.status != 0 \n" +
                "	 and pr.`status`>=0 and pr.sign_status=1  \n" +
                "	AND  (pr.is_manage != 1 OR isnull(pr.is_manage))  \n";


        if (StringUtils.isNotBlank(idcard)) {
            sql2 += " and  p.idcard like '%" + idcard + "%' ";
        }
        sql2 += " GROUP BY i.patient ORDER BY i.create_time DESC LIMIT " + (page - 1) * pagesize + "," + pagesize;

        List<Map<String, Object>> list2 = jdbcTemplate.queryForList(sql2);
        for (Map<String, Object> map : list2) {
            String idcardNo = map.get("idcard") + "";
            Date birthday = DateUtil.strToDate(map.get("birthday").toString());
            JSONObject oo = this.getUnm(doctorCode, map.get("patient").toString());
            Integer age = IdCardUtil.getAgeByIdcardOrBirthday(idcardNo, birthday);
            String sex = map.get("sex") + "";
            map.put("countMe", oo.get("countMe"));
            map.put("countElse", oo.get("countElse"));
            if (oo.containsKey("serviceName")) {
                map.put("serviceName", oo.get("serviceName"));
                map.put("serviceExecutiveName", oo.get("serviceExecutiveName"));
                map.put("serviceExecutiveTime", oo.get("serviceExecutiveTime"));
            }
            map.put("sex", sex);
            map.put("age", age);
            map.put("isTurnDown", 1);
        }
        object.put("res", list2);
        return downCount(doctorCode, object);
    }

    /**
     * 已经下转统计
     */
    public JSONObject downCount(String doctorCode, JSONObject object) {
//        String sql1 = "SELECT COUNT(distinct i.patient) \n" +
//                "FROM wlyy_patient_rehabilitation_plan i, wlyy_sign_family s, wlyy_patient p ,\n" +
//                "wlyy_specialist_patient_relation pr \n" +
//                "WHERE i.create_user = '" + doctorCode + "' AND i.status != 0 AND i.patient = s.patient AND p.code = i.patient AND s.status = 1 \n" +
//                "AND s.expenses_status = 1 AND pr.patient=p.code and pr.team_code = i.team_code and pr.`status`>=0 and pr.sign_status=1 ";
        String sql1 = "SELECT \n" +
                " COUNT(distinct i.patient) \n" +
                "FROM wlyy_patient_rehabilitation_plan i \n" +
                "INNER JOIN  base_patient p ON  p.id = i.patient  \n" +
                "INNER JOIN wlyy_rehabilitation_patient_info q ON q.patient=i.id  AND i.patient_info_code=q.`code`  AND q.sign_status='1'\n" +
                "INNER JOIN wlyy_specialist_patient_relation pr ON  pr.patient=p.id\n" +
                "WHERE i.create_user = '" + doctorCode + "' AND i.status != 0   and pr.`status`>=0 and pr.sign_status=1 ";
        Integer list1 = jdbcTemplate.queryForObject(sql1, Integer.class);

        //康复计划数量
//        String sql2 = "SELECT COUNT(distinct i.id) \n" +
//                "FROM wlyy_patient_rehabilitation_plan i, wlyy_sign_family s, wlyy_patient p\n" +
//                "WHERE i.create_user = '" + doctorCode + "' AND i.status != 0 AND i.patient = s.patient AND p.code = i.patient \n" +
//                "AND s.status = 1 AND s.expenses_status = 1 ";
        String sql2 = "SELECT COUNT(distinct i.id) \n" +
                " FROM wlyy_patient_rehabilitation_plan i " +
                " inner join  base_patient p on  p.id = i.patient \n" +
                " inner join  wlyy_rehabilitation_patient_info q on  q.patient = i.patient AND i.patient_info_code=q.`code` AND q.sign_status='1' \n" +
                " WHERE i.create_user = '" + doctorCode + "' AND i.status != 0    \n";
        Integer list2 = jdbcTemplate.queryForObject(sql2, Integer.class);


        //家医接收
//        String sql3 = "SELECT COUNT(distinct i.patient) " +
//                " FROM wlyy_patient_rehabilitation_plan i, wlyy_sign_family s, wlyy_patient p ,\n" +
//                " wlyy_specialist_patient_relation pr \n" +
//                " WHERE i.create_user = '" + doctorCode + "' AND i.status != 0 AND i.patient = s.patient AND p.code = i.patient AND s.status = 1 \n" +
//                " AND s.expenses_status = 1 AND pr.patient=p.code and pr.team_code = i.team_code and pr.`status`>=0 and pr.sign_status=1 and pr.is_manage=1";
//        Integer list3 = jdbcTemplate.queryForObject(sql3, Integer.class);

        object.put("deorsumvergenceCount", list1);//下转人数
        object.put("rehabilitationPlan", list2);//康复计划
//        object.put("patientReceiveCount", list3);//家医接收人数
//        object.put("unPatientReceiveCount", list1 - list3);//家医未接收人数
        return object;
    }

    /**
     * 已下转已接收
     */
    public JSONObject downAndSign(String idcard, String doctorCode, Integer page, Integer pagesize, JSONObject object) {
//        String sql2 = "SELECT i.disease_name diseaseName,s.doctor,s.idcard,p.sex, s.patient,CAST(IFNULL(p.birthday,'') AS char ) birthday," +
//                " i. STATUS, p. NAME patientName, p.address, CONCAT(s.doctor_name,'(全科医生)' ) AS doctorName, \n" +
//                "IFNULL(CONCAT(s.doctor_health_name,'(健管师)'), s.doctor_health_name ) doctorHealthName, s.hospital_name hospitalName, \n" +
//                "s.admin_team_code teamCode ,pr.is_manage isManage \n" +
//                "FROM wlyy_patient_rehabilitation_plan i, wlyy_sign_family s, wlyy_patient p ,\n" +
//                "wlyy_specialist_patient_relation pr \n" +
//                "WHERE i.create_user = '" + doctorCode + "' AND i.status != 0 AND i.patient = s.patient AND p.code = i.patient AND s.status = 1 \n" +
//                "AND s.expenses_status = 1 AND pr.patient=p.code and pr.team_code = i.team_code and pr.`status`>=0 and pr.sign_status=1 and pr.is_manage=1";

        String sql2 = "SELECT i.disease_name diseaseName,p.idcard,p.sex, p.id 'patient',\n" +
                " CAST(IFNULL(p.birthday,'') AS char ) birthday, \n" +
                " i.`status`, p.`name` patientName, p.address,pr.is_manage isManage \n" +
                "FROM wlyy_patient_rehabilitation_plan i \n" +
                " INNER JOIN wlyy_rehabilitation_patient_info q ON q.patient=i.id AND q.sign_status='1' AND i.patient_info_code=q.`code` \n" +
                " INNER JOIN base_patient p ON p.id = i.patient \n" +
                " INNER JOIN wlyy_specialist_patient_relation pr ON pr.patient=p.id \n" +
                "WHERE i.create_user = '" + doctorCode + "' AND i.status != 0  AND pr.`status`>=0 and pr.sign_status=1 and pr.is_manage=1 \n";

        if (StringUtils.isNotBlank(idcard)) {
            sql2 += " and  p.idcard like '%" + idcard + "%' ";
        }
        sql2 += " GROUP BY i.patient ORDER BY i.create_time DESC LIMIT " + (page - 1) * pagesize + "," + pagesize;
        List<Map<String, Object>> list2 = jdbcTemplate.queryForList(sql2);
        for (Map<String, Object> map : list2) {
            String idcardNo = map.get("idcard") + "";
            Date birthday = DateUtil.strToDate(map.get("birthday").toString());
            JSONObject oo = this.getUnm(doctorCode, map.get("patient").toString());
            Integer age = IdCardUtil.getAgeByIdcardOrBirthday(idcardNo, birthday);
            String sex = map.get("sex") + "";
            map.put("countMe", oo.get("countMe"));
            map.put("countElse", oo.get("countElse"));
            if (oo.containsKey("serviceName")) {
                map.put("serviceName", oo.get("serviceName"));
                map.put("serviceExecutiveName", oo.get("serviceExecutiveName"));
                map.put("serviceExecutiveTime", oo.get("serviceExecutiveTime"));
            }
            map.put("sex", sex);
            map.put("age", age);
            map.put("isTurnDown", 2);
        }
        object.put("res", list2);
        return downCount(doctorCode, object);
    }

    /**
     * 已下转已接收
     */
    public JSONObject down(String idcard, String doctorCode, Integer page, Integer pagesize, JSONObject object) {
//        String sql2 = "SELECT i.disease_name diseaseName,s.doctor,s.idcard,p.sex, s.patient,CAST(IFNULL(p.birthday,'') AS char ) birthday," +
//                " i. STATUS, p. NAME patientName, p.address, CONCAT(s.doctor_name,'(全科医生)' ) AS doctorName, \n" +
//                "IFNULL(CONCAT(s.doctor_health_name,'(健管师)'), s.doctor_health_name ) doctorHealthName, s.hospital_name hospitalName, \n" +
//                "s.admin_team_code teamCode ,pr.is_manage isManage \n" +
//                "FROM wlyy_patient_rehabilitation_plan i, wlyy_sign_family s, wlyy_patient p ,\n" +
//                "wlyy_specialist_patient_relation pr \n" +
//                "WHERE i.create_user = '" + doctorCode + "' AND i.status != 0 AND i.patient = s.patient AND p.code = i.patient AND s.status = 1 \n" +
//                "AND s.expenses_status = 1 AND pr.patient=p.code and pr.team_code = i.team_code and pr.`status`>=0 and pr.sign_status=1 ";

        String sql2 = "SELECT i.disease_name diseaseName,p.idcard,p.sex, p.id 'patient',\n" +
                " CAST(IFNULL(p.birthday,'') AS char ) birthday, i.`status`, p.`name` patientName, p.address, \n" +
                " pr.is_manage isManage \n" +
                "FROM wlyy_patient_rehabilitation_plan i \n" +
                " INNER JOIN base_patient p ON p.id = i.patient\n" +
                " INNER JOIN wlyy_rehabilitation_patient_info q ON q.patient=i.id AND q.sign_status='1' AND i.patient_info_code=q.`code`\n" +
                " INNER JOIN wlyy_specialist_patient_relation pr ON pr.patient=p.id\n" +
                "WHERE i.create_user = '" + doctorCode + "' AND i.status != 0 \n" +
                " and pr.`status`>=0 and pr.sign_status=1  ";

        if (StringUtils.isNotBlank(idcard)) {
            sql2 += " and  p.idcard like '%" + idcard + "%' ";
        }
        sql2 += " GROUP BY i.patient ORDER BY i.create_time DESC LIMIT " + (page - 1) * pagesize + "," + pagesize;

        List<Map<String, Object>> list2 = jdbcTemplate.queryForList(sql2);

        for (Map<String, Object> map : list2) {
            String idcardNo = map.get("idcard") + "";
            JSONObject oo = this.getUnm(doctorCode, map.get("patient").toString());
            Date birthday = DateUtil.strToDate(map.get("birthday").toString());
            Integer age = IdCardUtil.getAgeByIdcardOrBirthday(idcardNo, birthday);
            String sex = map.get("sex") + "";
            map.put("countMe", oo.get("countMe"));
            map.put("countElse", oo.get("countElse"));
            if (oo.containsKey("serviceName")) {
                map.put("serviceName", oo.get("serviceName"));
                map.put("serviceExecutiveName", oo.get("serviceExecutiveName"));
                map.put("serviceExecutiveTime", oo.get("serviceExecutiveTime"));
            }
            map.put("sex", sex);
            map.put("age", age);
            Integer isMange = 0;
            if (map.get("isManage") != null) {
                isMange = new Integer((Integer) map.get("isManage"));
            }
            if (isMange == 1) {
                map.put("isTurnDown", 2);//已下转已接收
            } else {
                map.put("isTurnDown", 1);//已下转未接收
            }
        }
        object.put("res", list2);
        return downCount(doctorCode, object);
    }

    public JSONObject getRehabilitationPatientList(String idcard, String doctorCode, String isTurnDown, Integer page, Integer pagesize) throws Exception {
        JSONObject object = new JSONObject();
        List<Map<String, Object>> res = new ArrayList<>();

        if ("0".equals(isTurnDown)) {
            //未下转居民
            return noDown(idcard, doctorCode, page, pagesize, object);
        }

        if ("-2".equals(isTurnDown)) {
            //未下转未加签
            return noDownAndNoSign(idcard, doctorCode, page, pagesize, object);
        }

        if ("-1".equals(isTurnDown)) {
            //未下转已家签
            return noDownAndSign(idcard, doctorCode, page, pagesize, object);
        }

        //用户同时被接收未接收，先取最近时间的，在查询
        if ("1".equals(isTurnDown)) {
            //未接收 1 已下转
            return downAndNoSign(idcard, doctorCode, page, pagesize, object);
        }

        if ("2".equals(isTurnDown)) {
            //已被加签医生管理   已接收
            return downAndSign(idcard, doctorCode, page, pagesize, object);
        }

        if ("3".equals(isTurnDown)) {
            //已下转所有的-3全部
            return down(idcard, doctorCode, page, pagesize, object);
        }

        if ("03".equals(isTurnDown)) {
            noDown(idcard, doctorCode, page, pagesize, object);
            return down(idcard, doctorCode, page, pagesize, object);
        }

        return object;
    }

    /**
     * 开始分页
     *
     * @param list
     * @param pageNum  页码
     * @param pageSize 每页多少条数据
     * @return
     */
    public static List startPage(List list, Integer pageNum,
                                 Integer pageSize) {
        if (list == null) {
            return null;
        }
        if (list.size() == 0) {
            return null;
        }
        Integer count = list.size(); // 记录总数
        Integer pageCount = 0; // 页数
        if (count % pageSize == 0) {
            pageCount = count / pageSize;
        } else {
            pageCount = count / pageSize + 1;
        }
        int fromIndex = 0; // 开始索引
        int toIndex = 0; // 结束索引
        if (pageNum != pageCount) {
            fromIndex = (pageNum - 1) * pageSize;
            toIndex = fromIndex + pageSize;
        } else {
            fromIndex = (pageNum - 1) * pageSize;
            toIndex = count;
        }
        List pageList = list.subList(fromIndex, toIndex);
        return pageList;
    }

    public JSONObject getDeorsumvergenceCount(String doctorCode) {
        JSONObject object = new JSONObject();
        //已下转
        String sql1 = "SELECT s.idcard, s.patient, i. STATUS, p. NAME patientName, p.address, CONCAT(s.doctor_name,'(全科医生)' ) AS doctorName, \n" +
                "IFNULL(CONCAT(s.doctor_health_name,'(健管师)'), s.doctor_health_name ) doctorHealthName, s.hospital_name hospitalName, \n" +
                "s.admin_team_code teamCode ,pr.is_manage isManage \n" +
                "FROM wlyy_patient_rehabilitation_plan i, wlyy_sign_family s, wlyy_patient p ,\n" +
                "wlyy_specialist_patient_relation pr \n" +
                "WHERE i.create_user = '" + doctorCode + "' AND i.status != 0 AND i.patient = s.patient AND p.code = i.patient AND s.status = 1 \n" +
                "AND s.expenses_status = 1 AND pr.patient=p.code  GROUP BY i.patient ORDER BY i.create_time DESC ";
        List<Map<String, Object>> list1 = jdbcTemplate.queryForList(sql1);
        //康复计划数量
        String sql2 = "SELECT s.idcard, s.patient, i. STATUS, p. NAME patientName, p.address, CONCAT(s.doctor_name,'(全科医生)' ) AS doctorName, \n" +
                "IFNULL(CONCAT(s.doctor_health_name,'(健管师)'), s.doctor_health_name ) doctorHealthName, s.hospital_name hospitalName, s.admin_team_code teamCode \n" +
                "FROM wlyy_patient_rehabilitation_plan i, wlyy_sign_family s, wlyy_patient p\n" +
                "WHERE i.create_user = '" + doctorCode + "' AND i.status != 0 AND i.patient = s.patient AND p.code = i.patient \n" +
                "AND s.status = 1 AND s.expenses_status = 1 GROUP BY i.id";
        List<Map<String, Object>> list2 = jdbcTemplate.queryForList(sql2);
        //家医接收
        String sql3 = "SELECT * \n" +
                "FROM (\n" +
                "SELECT s.idcard, s.patient, i. STATUS, p. NAME patientName, p.address, CONCAT(s.doctor_name,'(全科医生)' ) AS doctorName, \n" +
                "IFNULL(CONCAT(s.doctor_health_name,'(健管师)'), s.doctor_health_name ) doctorHealthName, s.hospital_name hospitalName, \n" +
                "s.admin_team_code teamCode ,pr.is_manage isManage \n" +
                "FROM wlyy_patient_rehabilitation_plan i, wlyy_sign_family s, wlyy_patient p ,\n" +
                "wlyy_specialist_patient_relation pr \n" +
                "WHERE i.create_user = '" + doctorCode + "' AND i.status != 0 AND i.patient = s.patient AND p.code = i.patient AND s.status = 1 \n" +
                "AND s.expenses_status = 1 AND pr.patient=p.code  GROUP BY i.patient ORDER BY i.create_time DESC\n" +
                ") aa\n" +
                "WHERE aa.isManage = 1 ";
        List<Map<String, Object>> list3 = jdbcTemplate.queryForList(sql3);
        object.put("deorsumvergenceCount", list1.size());//下转人数
        object.put("rehabilitationPlan", list2.size());//康复计划
        object.put("patientReceiveCount", list3.size());//家医接收人数
        object.put("unPatientReceiveCount", list1.size() - list3.size());//家医未接收人数
        return object;
    }

    /**
     * 提醒居民家签
     * @param patient
     */
//    public void sendWxMsgRemindSign(String patient) throws Exception {
//        org.json.JSONObject json = new org.json.JSONObject();
//        if(StringUtils.isNotBlank(patient)) {
//            Patient p = patientDao.findByCode(patient);
//            json.put("first", p.getName() + "您好！为方便为您提供康复管理服务，请先选择合适的社区和家庭医生进行签约。");
//            json.put("toUser", patient);
//            json.put("represented", patient);//被代理人
//            json.put("keyword1", "家庭医生签约");
//            json.put("keyword2", DateUtil.dateToStrShort(new Date()));
//            json.put("remark", "");
//            if (StringUtils.isNotBlank(p.getOpenid())) {
//                json.put("url",  "/qygl/html/signing-welcome.html?openid=" + p.getOpenid() + "&toUser=" + patient + "&toName=" + p.getName());
//                pushMsgTask.putWxMsg(weiXinAccessTokenUtils.getAccessToken(), 35, p.getOpenid(), p.getName(), json);
//            }else {
//                //发送代理人
//                org.json.JSONArray jsonArray = weiXinOpenIdUtils.getAgentOpenId(p.getCode(), p.getOpenid());
//                if (jsonArray != null && jsonArray.length() > 0) {
//                    for (int i = 0; i < jsonArray.length(); i++) {
//                        org.json.JSONObject j = jsonArray.getJSONObject(i);
//                        Patient member = (Patient) j.get("member");
//                        org.json.JSONObject data = json;
//                        data.remove("toUser");
//                        data.put("toUser", member.getCode());
//                        data.remove("first");
//                        data.put("first", weiXinOpenIdUtils.getTitleMes(p, j.getInt("relation"), p.getName()));
//                        pushMsgTask.putWxMsg(weiXinAccessTokenUtils.getAccessToken(), 35, member.getOpenid(), p.getName(), data);
//                    }
//                }
//            }
//        }
//    }

    /**
     * patientInfo
     * type 1患者家签列表 2康复下转列表
     * doctorCode
     */
    public JSONObject getPatientByIdcardOrSsc(String patientInfo, String doctorCode, Integer fromSource) {
        //加签列表查询wlyy_patient_discharge表,与type无关。当家签处添加时（即type=1），统一保存type=2的记录
        JSONObject data = new JSONObject();
        List<BasePatientDO> patientList = patientDao.getPatientByIdcardOrSsc(patientInfo);
        BasePatientDO patient = null;
        if (patientList.size() > 0) {
            patient = patientList.get(0);
        }
        if (patient == null) {
            data.put("status", -1);
            data.put("msg", "未查询到患者");
            return data;
        }
        RehabilitationPatientInfoDO infoDO = new RehabilitationPatientInfoDO();
        infoDO.setPatient(patient.getId());
        infoDO.setStatus(0);
        infoDO.setAge(IdCardUtil.getAgeByBirthdayOrIdcard(patient.getIdcard(), patient.getBirthday()));
        infoDO.setSsc(patient.getSsc());
        infoDO.setAddress(patient.getAddress());
        infoDO.setCode(UUID.randomUUID().toString().replaceAll("-", ""));
        infoDO.setCreateTime(new Date());
        if (StringUtils.isNotBlank(doctorCode)) {
            BaseDoctorDO doctor = doctorDao.findById(doctorCode).orElse(null);
            infoDO.setCreateUser(doctor.getId());
            infoDO.setCreateUserName(doctor.getName());
        }
        infoDO.setName(patient.getName());
        infoDO.setSex(patient.getSex());
        infoDO.setMobile(patient.getMobile());
        infoDO.setFromSource(fromSource);
        infoDO.setSignStatus(0);
        rehabilitationPatientInfoDao.save(infoDO);
        //判断是否家签  修改
        JSONObject isSign = businessService.isSign(patient.getIdcard());
        if (isSign != null && isSign.getJSONObject("obj") != null) {
            infoDO.setSignStatus(1);
            rehabilitationPatientInfoDao.save(infoDO);
        }
        data.put("status", 200);
        data.put("data", infoDO);
        return data;
    }

    public JSONObject getUnm(String doctor, String patient) {
        String sqlMe = "select d.id,d.plan_id as planId,i.code,i.name,p.patient,d.frequency_code from wlyy_rehabilitation_plan_detail d \n" +
                "LEFT JOIN wlyy_patient_rehabilitation_plan p ON p.id=d.plan_id\n" +
                "LEFT JOIN wlyy_rehabilitation_service_item i on i.code = d.hospital_service_item_id  \n" +
                "where d.plan_id IN (SELECT id FROM wlyy_patient_rehabilitation_plan WHERE patient ='" + patient + "') \n" +
                "AND d.doctor = '" + doctor + "'  ";//我的任务
        List<Map<String, Object>> listMe = jdbcTemplate.queryForList(sqlMe);
        String sqlAll = "select d.id,d.plan_id as planId,i.code,i.name,p.patient,d.frequency_code from wlyy_rehabilitation_plan_detail d \n" +
                "LEFT JOIN wlyy_patient_rehabilitation_plan p ON p.id=d.plan_id\n" +
                "LEFT JOIN wlyy_rehabilitation_service_item i on i.code = d.hospital_service_item_id  \n" +
                "where d.plan_id IN (SELECT id FROM wlyy_patient_rehabilitation_plan WHERE patient ='" + patient + "') ";//全部任务
        List<Map<String, Object>> listAll = jdbcTemplate.queryForList(sqlAll);
        String newDetailSql = "select d.id,d.plan_id as planId,i.code,i.name,p.patient,d.frequency_code,r.complete_time createTime, r.doctor_name doctorName \n" +
                "from wlyy_rehabilitation_plan_detail d \n" +
                "INNER JOIN wlyy_rehabilitation_operate_records r ON d.id = r.rehabilitation_detail_id\n" +
                "LEFT JOIN wlyy_patient_rehabilitation_plan p ON p.id=d.plan_id\n" +
                "LEFT JOIN wlyy_rehabilitation_service_item i on i.code = d.hospital_service_item_id  \n" +
                "where d.plan_id IN (SELECT id FROM wlyy_patient_rehabilitation_plan WHERE patient ='" + patient + "')\n" +
                " and r.`status`=1 GROUP BY r.complete_time DESC LIMIT 0,1 ";
        newDetailSql = TransforSqlUtl.wlyy_rehabilitation_operate_recordsDoctorName(newDetailSql);
        List<Map<String, Object>> newDetail = jdbcTemplate.queryForList(newDetailSql);
        JSONObject object = new JSONObject();
        object.put("countMe", listMe.size());
        object.put("countElse", listAll.size() - listMe.size());
        if (newDetail.size() > 0) {
            object.put("serviceName", newDetail.get(0).get("name"));
            object.put("serviceExecutiveName", newDetail.get(0).get("doctorName"));
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            object.put("serviceExecutiveTime", sdf.format(newDetail.get(0).get("createTime")));
        }
        return object;
    }

    /**
     * 新增或编辑康复建议模板
     *
     * @param doctorCode
     * @param code       模板code
     * @param name
     * @param content
     */
    public void createAdvice(String doctorCode, String code, String name, String content) {
        RehabilitationAdviceDO adviceDO = null;
        //判断是新增还是编辑
        if (StringUtils.isNotBlank(code)) {
            adviceDO = adviceDao.findByCode(code);
        } else {
            adviceDO = new RehabilitationAdviceDO();
            adviceDO.setCode(UUID.randomUUID().toString().replaceAll("-", ""));
            BaseDoctorDO doctor = doctorDao.findById(doctorCode).orElse(null);
            adviceDO.setCreateUser(doctorCode);
            adviceDO.setCreateUserName(doctor.getName());
        }
        adviceDO.setName(name);
        adviceDO.setContent(content);
        adviceDO.setCreateTime(new Date());
        adviceDao.save(adviceDO);
    }

    public void deleteAdvice(Long id) {
        adviceDao.deleteById(id);
    }


    /**
     * 获取医生创建康复建议模板及系统模板
     *
     * @param doctorCode
     * @return
     */
    public List<RehabilitationAdviceDO> getAdviceListByDoctor(String doctorCode) {
        List<RehabilitationAdviceDO> list = adviceDao.findByDoctor(doctorCode);
        return list;
    }

    public JSONObject getRehabilitationPatientListNew(String downPatientId, String idcard, String doctorId, String status, Integer page, Integer pagesize) {
        JSONObject object = new JSONObject();
        List<Map<String, Object>> list = null;
        String detailSql = "SELECT \n" +
                " 	p.idcard, q.id 'planId',	p.id 'patientId', p.phone,	p.`name` 'patientName',	p.sex,	p.address, \n" +
                "   d.code 'patientInfoCode',	d.ssc , d.mobile, d.age,	d.diagnosis_name 'diagnosisName',	d.sign_status,	d.dept 'deptCode', \n" +
                " 	d.dept_name 'deptName',	d.create_user 'createUserId', \n" +
                " 	d.create_user_name 'createUserName',DATE_FORMAT(d.create_time, '%Y-%m-%d %H:%i:%s') 'createTime',\n" +
                "	q.`status` 'planStaus',q.plan_doctor 'planDoctor', q.plan_doctor_name 'planDoctorName' , q.advice_content 'adviceContent', \n" +
                "	DATE_FORMAT(q.create_time, '%Y-%m-%d %H:%i:%s') 'planCreateTime'," +
                "   d.from_source," +
                "       CASE  \n" +
                "        WHEN d.from_source='1' THEN '手动添加'\n" +
                "        WHEN d.from_source='2' THEN '自动下转'\n" +
                "        WHEN d.from_source='3' THEN '扫码添加'\n" +
                "      END 'fromSourceName',\n" +
                "   d.ijk_id, d.status 'patientStatus', \n" +
                "   d.problem_code," +
                "   q.disease 'diseaseId',q.disease_name 'diseaseName' \n" +
                " FROM base_patient p \n" +
                " INNER JOIN wlyy_rehabilitation_patient_info d ON p.id = d.patient \n" +
                " LEFT JOIN wlyy_patient_rehabilitation_plan q ON d.`code`=q.patient_info_code  " +
                " WHERE  1 = 1 ";
        String countSql = " select count(1) from base_patient p " +
                " INNER JOIN wlyy_rehabilitation_patient_info d ON p.id = d.patient " +
                " LEFT JOIN wlyy_patient_rehabilitation_plan q ON d.`code`=q.patient_info_code" +
                " WHERE  1 = 1   \n";
        String limitSql = " LIMIT " + (page - 1) * pagesize + "," + pagesize;
        String filterSql = "";
        if (StringUtils.isNotBlank(idcard)) {
            filterSql += " and d.idcard='" + idcard + "' ";
        }
        if (StringUtils.isNotBlank(downPatientId)) {
            //wlyy_rehabilitation_patient_info的id
            filterSql += " and d.id='" + downPatientId + "' ";
        }
        if (StringUtils.isBlank(status)) {
            //看全部数据
            filterSql += "	AND (d.create_user = '" + doctorId + "'  OR ISNULL(  d.create_user  ) )\n" +
                    "	    AND ( ISNULL( d.type ) OR d.type = 2 ) ORDER BY d.create_time DESC ";
            detailSql += filterSql + limitSql;
            countSql += filterSql;
            list = jdbcTemplate.queryForList(detailSql);
            Integer allCount = jdbcTemplate.queryForObject(countSql, Integer.class);
            object.put("allCount", allCount);
        }
        if (StringUtils.isNotBlank(status)) {
            if ("0".equals(status)) {
                //未下转的人
                filterSql += "	AND (d.create_user = '" + doctorId + "'  OR ISNULL(d.create_user)) AND ( ISNULL( d.type ) OR d.type = 2 ) " +
                        " and d.status='0' ";
                filterSql += " ORDER BY d.create_time DESC ";
                detailSql += filterSql + limitSql;
                countSql += filterSql;
                list = jdbcTemplate.queryForList(detailSql);
                Integer unDownCount = jdbcTemplate.queryForObject(countSql, Integer.class);
                object.put("unDownCount", unDownCount);//未下转的人
            }
            if ("1".equals(status)) {
                //待分配的人
                filterSql += "	AND (d.create_user = '" + doctorId + "' OR ISNULL( d.create_user) )AND ( ISNULL( d.type ) OR d.type = 2 ) " +
                        "   and d.status='1' ";
                filterSql += " ORDER BY d.create_time DESC ";
                detailSql += filterSql + limitSql;
                countSql += filterSql;
                list = jdbcTemplate.queryForList(detailSql);
                Integer unAllocationCount = jdbcTemplate.queryForObject(countSql, Integer.class);
                object.put("unAllocationCount", unAllocationCount);//待分配的人
            }

            if ("7".equals(status)) {
                //已经分配医生
                filterSql += "	AND ( d.create_user = '" + doctorId + "'  OR ISNULL(  d.create_user  ) )  \n" +
                        "	AND ( ISNULL( d.type ) OR d.type = 2 ) and d.status='7' ";
                filterSql += " ORDER BY d.create_time DESC ";
                detailSql += filterSql + limitSql;
                countSql += filterSql;
                list = jdbcTemplate.queryForList(detailSql);
                Integer allocation = jdbcTemplate.queryForObject(countSql, Integer.class);
                object.put("allocation", allocation);//已经分配
            }
        }
        object.put("data", list);
        return object;
    }

    /**
     * 根据疾病id查询科室
     * 根据科室查询医生
     */
    public List<Map<String, Object>> getDetpAndDoctor(String doctorName, String deptCode, String deptName, String type, String disease) {
        List<Map<String, Object>> list = null;
        if ("1".equals(type)) {
            String sql = "SELECT  DISTINCT b.id 'diseaseId' ,b.`name` 'diseaseName',a.* \n" +
                    "FROM\n" +
                    "	dict_hospital_dept a \n" +
                    "	INNER JOIN base_disease_hospital b ON FIND_IN_SET(a.`code`, b.dept) > 0 \n" +
                    "WHERE 1 =1\n";
            if (StringUtils.isNotBlank(deptName)) {
                sql += " and `name` LIKE '%" + deptName + "%'";
            }
            if (StringUtils.isNotBlank(disease)) {
                sql += "	AND b.id='" + disease + "' ";
            }
            list = jdbcTemplate.queryForList(sql);
        }
        if ("2".equals(type)) {
            String sql = "SELECT b.id 'doctorId',b.`name` 'doctorName' FROM base_doctor_hospital a \n" +
                    "INNER JOIN base_doctor  b ON a.doctor_code=b.id \n" +
                    "WHERE  1=1 ";
            if (StringUtils.isNotBlank(deptCode)) {
                sql += " and dept_code = '" + deptCode + "' ";
            }
            if (StringUtils.isNotBlank(doctorName)) {
                sql += " and b.`name` LIKE '%" + deptName + "%'";
            }
            list = jdbcTemplate.queryForList(sql);
        }
        return list;
    }

    public List<Map<String, Object>> getPatientInfo(String patientId, String idcard) {
        String sql = "SELECT\n" +
                "	b.sign_status 'signStatus',\n" +
                "	CASE \n" +
                "	 WHEN b.sign_status='1' THEN '已家签'\n" +
                "	 WHEN b.sign_status='0' THEN '未家签'\n" +
                "	END 'signStatusName',\n" +
                "	a.* \n" +
                "FROM\n" +
                "	base_patient a\n" +
                "	LEFT JOIN wlyy_rehabilitation_patient_info b ON a.id = b.patient \n" +
                "WHERE\n" +
                "	1 = 1 \n" +
                "	AND a.del = 1\n";
        if (StringUtils.isNotBlank(patientId)) {
            sql += "	AND a.id='" + patientId + "'\n";
        }
        if (StringUtils.isNotBlank(idcard)) {
            sql += "	AND a.idcard='" + idcard + "'\n";
        }
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        return list;
    }

    /**
     * 获取所有居民的康复计列表
     */
    public List<Map<String, Object>> getRehabilitationPatientPlan(String idcard, String doctorId, String patientId, String finishStatus, Integer page, Integer pagesize) {
        String limitSql = " LIMIT " + (page - 1) * pagesize + "," + pagesize;
        String sql = "SELECT * FROM ( " +
                "SELECT DISTINCT \n" +
                "	 a.idcard,a.id,a.`name`,a.birthday,a.phone,a.sex,a.mobile,TIMESTAMPDIFF(YEAR,a.birthday,NOW()) 'age',\n" +
                "	   (\n" +
                "		SELECT GROUP_CONCAT(q.`name`) FROM base_disease_hospital q \n" +
                "        INNER JOIN wlyy_patient_rehabilitation_plan w ON q.id = w.disease \n" +
                "		 WHERE 1 = 1 AND w.patient=a.id  GROUP BY w.patient\n" +
                "	   ) 'diseaseListName',\n" +
                "      (\n" +
                "        SELECT GROUP_CONCAT(DISTINCT w.plan_doctor_name) FROM wlyy_patient_rehabilitation_plan w WHERE a.id = w.patient \n" +
                "      ) 'planDoctorListName',\n" +
                "	   (\n" +
                "			SELECT count(1) FROM wlyy_patient_rehabilitation_plan w \n" +
                "			INNER JOIN wlyy_rehabilitation_plan_detail q ON w.id = q.plan_id \n" +
                "           INNER JOIN base_service_item_plan e ON e.plan_detail_id = q.id \n" +
                "			WHERE 1 = 1 AND w.patient = a.id \n" +
                "		)'itemAllCount',\n" +
                "		(\n" +
                "			SELECT count(1) FROM wlyy_patient_rehabilitation_plan w \n" +
                "			INNER JOIN wlyy_rehabilitation_plan_detail q ON w.id = q.plan_id \n" +
                "           INNER JOIN base_service_item_plan e ON e.plan_detail_id = q.id \n" +
                "			WHERE	1 = 1 	AND e.`status`='0'	AND w.patient = a.id \n" +
                "		)'unFinishCount',\n" +
                "		(\n" +
                "			SELECT count(1) 	FROM wlyy_patient_rehabilitation_plan w \n" +
                "			INNER JOIN wlyy_rehabilitation_plan_detail q ON w.id = q.plan_id \n" +
                "           INNER JOIN base_service_item_plan e ON e.plan_detail_id = q.id \n" +
                "			WHERE	1 = 1 	AND e.`status`='1'	AND w.patient = a.id \n" +
                "		)'finishCount',\n" +
                "		(\n" +
                "			SELECT count(1) FROM wlyy_patient_rehabilitation_plan w \n" +
                "			INNER JOIN wlyy_rehabilitation_plan_detail q ON w.id = q.plan_id \n" +
                "           INNER JOIN base_service_item_plan e ON e.plan_detail_id = q.id \n" +
                "			WHERE	1 = 1 	AND e.plan_time=DATE_FORMAT(NOW(),'%Y-%m-%d') AND e.`status`='0'	AND w.patient = a.id \n" +
                "		)'totayTask'\n" +
                "FROM\n" +
                "	base_patient a\n" +
                "	left JOIN wlyy_patient_rehabilitation_plan b ON a.id = b.patient\n" +
                "	INNER JOIN wlyy_rehabilitation_patient_info c ON c.patient = a.id \n" +
                "WHERE\n" +
                "	1 = 1 \n" +
                "	AND a.del = '1'\n";

        if (StringUtils.isNotBlank(doctorId) && StringUtils.isBlank(patientId)) {
            //计划负责人或者任务执行人-执行人不为空
            sql += "	AND (b.plan_doctor='" + doctorId + "' OR c.doctor='" + doctorId + "' )";
            sql += " and  c.doctor is not null  ";
        }


        if (StringUtils.isNotBlank(patientId)) {
            //康复计划状态 [0已中止，1进行中，2已完成]
            sql += "	AND a.id ='" + patientId + "' ";
        }
        if (StringUtils.isNotBlank(idcard) && !"null".equals(idcard)) {
            //康复计划状态 [0已中止，1进行中，2已完成]
            sql += " AND a.idcard like'%" + idcard + "%'";
        }

        sql += " )q WHERE 1=1  ";
        if (StringUtils.isNotBlank(finishStatus)) {
            //康复计划状态 [0已中止，1进行中，2已完成]
            if ("1".equals(finishStatus)) {
                //未完成
                sql += "	AND q.unFinishCount<>0 ";
            }
            if ("2".equals(finishStatus)) {
                //已完成
                sql += "	AND q.unFinishCount=0 ";
            }
        }
        sql += limitSql;
        System.out.println("查询sql==>" + sql);
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        return list;
    }

    /**
     * 还需要代办项目还需要重新join一张新表
     */
    public HashMap<String, Object> getRehabilitationPatientPlanCount(String doctorId) {
        String sql =
                "SELECT DISTINCT \n" +
                        "	 a.id,a.`name`,a.birthday,a.phone,a.sex,a.mobile,TIMESTAMPDIFF(YEAR,a.birthday,NOW()) 'age',\n" +
                        "	  (\n" +
                        "		SELECT GROUP_CONCAT(q.`name`) FROM base_disease_hospital q  INNER JOIN wlyy_patient_rehabilitation_plan w ON q.id = w.disease \n" +
                        "		 WHERE 1 = 1 AND w.patient=a.id  GROUP BY w.patient\n" +
                        "	  ) 'diseaseListName',\n" +
                        "	  (\n" +
                        "	  	SELECT count(1)  FROM wlyy_patient_rehabilitation_plan w \n" +
                        "	  	INNER JOIN wlyy_rehabilitation_plan_detail q ON w.id = q.plan_id \n" +
                        "       INNER JOIN base_service_item_plan e ON e.plan_detail_id = q.id \n" +
                        "	  	WHERE 1 = 1  AND e.`status`='0'	 AND w.patient = a.id \n" +
                        "	  )'unFinishCount'\n" +
                        "FROM\n" +
                        "	base_patient a\n" +
                        "	INNER JOIN wlyy_patient_rehabilitation_plan b ON a.id = b.patient\n" +
                        "	INNER JOIN wlyy_rehabilitation_patient_info c ON c.patient = b.patient \n" +
                        "WHERE\n" +
                        "	1 = 1 \n" +
                        "	AND a.del = '1'\n";

        if (StringUtils.isNotBlank(doctorId)) {
            //计划负责人或者任务执行人-执行人不为空
            sql += "	AND (b.plan_doctor='" + doctorId + "' OR c.doctor='" + doctorId + "' )";
            sql += " and  c.doctor is not null  ";
        }
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        int allCount = 0;
        int unFinishCount = 0;
        int finishCount = 0;

        for (Map<String, Object> map : list) {
            if (0 == Integer.parseInt(map.get("unFinishCount").toString())) {
                finishCount += 1;
            } else {
                unFinishCount += 1;
            }
            allCount += 1;
        }
        HashMap<String, Object> map = new HashMap<>();
        map.put("allCount", allCount);
        map.put("unFinishCount", unFinishCount);
        map.put("finishCount", finishCount);
        return map;
    }

    /**
     * todo 还需要代办项目还需要重新join一张新表 todayTaskCount 今日任务项，等后面在整
     */
    public List<Map<String, Object>> getPatientRehabilitationByPatientId(String patientId, String planId) {
        String sql = "SELECT DISTINCT \n" +
                "	 a.id,a.`name`,a.birthday,a.phone,a.sex,a.mobile,c.age,a.idcard,\n" +
                "	 b.id 'planId',b.title 'planTitle',b.plan_doctor_name 'planDoctorName',d. `name` 'diseaseName',c.dept_name 'deptName',\n" +
                "	 b.`status` 'planStatus', c.hospital_name 'hospitalName',b.medical_records_code medicalRecordsCode," +
                "    c.from_source, " +
                "       CASE  \n" +
                "        WHEN c.from_source='1' THEN '手动添加'\n" +
                "        WHEN c.from_source='2' THEN '自动下转'\n" +
                "        WHEN c.from_source='3' THEN '扫码添加'\n" +
                "      END 'fromSourceName',\n" +
                "     c.ijk_id, c.status,'patientStatus'," +
                "     b.advice_content 'adviceContent',\n" +
                "	 (SELECT count(1) FROM wlyy_rehabilitation_plan_detail q INNER JOIN base_service_item_plan w ON w.plan_detail_id = q.id WHERE 1 = 1 AND q.plan_id = b.id )'itemAllCount',\n" +
                "	 (SELECT count(1) FROM wlyy_rehabilitation_plan_detail q INNER JOIN base_service_item_plan w ON w.plan_detail_id = q.id WHERE 1 = 1 AND w.`status`='1' AND q.plan_id = b.id )'finishCount',\n" +
                "	 (SELECT count(1) FROM wlyy_rehabilitation_plan_detail q INNER JOIN base_service_item_plan w ON w.plan_detail_id = q.id WHERE 1 = 1 AND w.`status`='0' AND q.plan_id = b.id )'unFinishCount',\n" +
                "	 (SELECT count(1) FROM wlyy_rehabilitation_plan_detail q INNER JOIN base_service_item_plan w ON w.plan_detail_id = q.id WHERE 1 = 1 AND w.`status`='0' AND w.plan_time=DATE_FORMAT(NOW(),'%Y-%m-%d') AND q.plan_id = b.id )'todayTaskCount' \n" +
                "FROM\n" +
                "	base_patient a\n" +
                "	LEFT JOIN wlyy_patient_rehabilitation_plan b ON a.id = b.patient\n" +
                "	INNER JOIN wlyy_rehabilitation_patient_info c ON c.patient = b.patient \n" +
                "	LEFT JOIN base_disease_hospital d ON  d.id=b.disease\n" +
                "WHERE\n" +
                "	1 = 1 \n" +
                "	AND a.del = '1' ";
        if (StringUtils.isNotBlank(patientId)) {
            sql += "	AND a.id='" + patientId + "' ";
        }
        if (StringUtils.isNotBlank(planId)) {
            sql += "	AND b.id='" + planId + "' ";
        }
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        System.out.println("数据：" + JSON.toJSONString(list));
        return list;
    }

    public List<Map<String, Object>> getRehabilitationPatientCountNew(String doctorId) {
        String sql = "SELECT\n" +
                "	CASE \n" +
                "		WHEN d.`status`='0' THEN '未下转'\n" +
                "		WHEN d.`status`='1' THEN '待分配'\n" +
//                "		WHEN d.`status`='2' THEN '创建计划过程失败'\n" +
//                "		WHEN d.`status`='3' THEN '患者无住院病历信息'\n" +
//                "		WHEN d.`status`='5' THEN '同步居民失败'\n" +
                "		WHEN d.`status`='7' THEN '已管理'\n" +
                "  END 'statusName',\n" +
                "  d.`status`,count(1) 'count' \n" +
                "FROM\n" +
                "	base_patient p\n" +
                "	INNER JOIN wlyy_rehabilitation_patient_info d ON p.id = d.patient\n" +
                "	LEFT JOIN wlyy_patient_rehabilitation_plan q ON d.`code` = q.patient_info_code \n" +
                "WHERE\n" +
                "	1 = 1 \n" +
                " 	AND ( d.create_user = '" + doctorId + "'  OR q.plan_doctor='" + doctorId + "' OR ISNULL( d.create_user ) ) \n" +
                "	AND ( ISNULL( d.type ) OR d.type = 2 )\n" +
                "	GROUP BY d.`status`";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        return list;
    }


    /**
     * 查询病例信息
     */
    public List<PatientMedicalRecordsDO> getPatientMedicalRecordsByPlanId(String planId) {
        String sql = "SELECT\n" +
                " a.* \n" +
                "FROM\n" +
                "	wlyy_patient_medical_records_rehabilitation a\n" +
                "	INNER JOIN wlyy_patient_rehabilitation_plan b ON a.`CODE` = b.medical_records_code \n" +
                "WHERE\n" +
                "	b.id = '" + planId + "' \n" +
                "ORDER BY\n" +
                "	create_time DESC ";
        List<PatientMedicalRecordsDO> list = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(PatientMedicalRecordsDO.class));
        return list;
    }


    /**
     * 查询康复计划服务项
     */
    public List<Map<String, Object>> getPatientPlanItemList(String patientId, String planId, String itemId, String status) {
        String sql = "SELECT\n" +
                "	b.`name`,\n" +
                "	a.plan_id," +
                "   a.id, a.doctor,a.doctor_name,a.dept,a.dept_name \n" +
                "FROM\n" +
                "	wlyy_rehabilitation_plan_detail a\n" +
                "	INNER JOIN wlyy_rehabilitation_service_item b ON a.hospital_service_item_id = b.id \n" +
                "WHERE\n" +
                "	1 = 1 \n" +
                "	AND a.plan_id = '" + planId + "'";
        if (StringUtils.isNotBlank(itemId)) {
            sql += " and b.id= '" + itemId + "'";
        }
        if (StringUtils.isNotBlank(status)) {
            sql += " and a.status =" + status;
        }
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        return list;
    }

    /**
     * 获取频次服务的信息
     * 这个服务项次数还没有-暂时没有
     */
    public List<Map<String, Object>> getPatientPlanFrequencyItemList(String patientId, String planId, String startTime, String endTime, String doctorId, String type) {
        return null;
    }


    public List<Map<String, Object>> getPlanDoctorList(String planId) {
        String sql = "SELECT DISTINCT q.* FROM (\n" +
                "	SELECT \n" +
                "		a.id,a.`name`,a.mobile,a.photo,a.sex\n" +
                "	FROM\n" +
                "		base_doctor a\n" +
                "		INNER JOIN wlyy_rehabilitation_plan_detail b ON a.id = b.doctor\n" +
                "		WHERE 1=1 AND b.plan_id='" + planId + "'\n" +
                "	UNION ALL\n" +
                "	SELECT \n" +
                "		a.id,a.`name`,a.mobile,a.photo,a.sex\n" +
                "	FROM\n" +
                "		base_doctor a\n" +
                "		INNER JOIN wlyy_patient_rehabilitation_plan b ON a.id = b.plan_doctor\n" +
                "		WHERE 1=1 AND b.id='" + planId + "'\n" +
                ")q";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        return list;
    }
}
