package com.yihu.jw.hospital.module.rehabilitation.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yihu.fastdfs.FastDFSUtil;
import com.yihu.jw.entity.rehabilitation.RehabilitationPlanningDO;
import com.yihu.jw.entity.specialist.HospitalServiceItemDO;
import com.yihu.jw.entity.specialist.RehabilitationServiceItemDO;
import com.yihu.jw.entity.specialist.rehabilitation.*;
import com.yihu.jw.entity.util.TransforSqlUtl;
import com.yihu.jw.hospital.disease.dao.RehabilitationPlanTemplateDao;
import com.yihu.jw.hospital.disease.dao.RehabilitationTemplateDetailDao;
import com.yihu.jw.hospital.module.rehabilitation.dao.*;
import com.yihu.jw.hospital.module.rehabilitation.vo.TemplateDetailVo;
import com.yihu.jw.hospital.module.specialist.service.SpecialistHospitalServiceItemService;
import com.yihu.jw.hospital.module.specialist.service.SpecialistService;
import com.yihu.jw.mysql.query.BaseJpaService;
import com.yihu.jw.rehabilitation.PatientMedicalRecordsDao;
import com.yihu.jw.restmodel.qvo.ParamQvo;
import com.yihu.jw.restmodel.specialist.PatientSignInfoVO;
import com.yihu.jw.restmodel.web.Envelop;
import com.yihu.jw.restmodel.web.ListEnvelop;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.rm.specialist.SpecialistMapping;
import com.yihu.jw.util.date.DateUtil;
import com.yihu.jw.util.http.HttpClientUtil;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;


/**
 * Created by humingfen on 2018/8/17.
 */
@Service
@Transactional
public class RehabilitationPlanService extends BaseJpaService<RehabilitationPlanningDO, RehabilitationPlanTemplateDao> {


    @Value("${base.url}")
    private String baseUrl;
    @Value("${basedb.name}")
    private String basedb;


    @Autowired
    private RehabilitationPlanTemplateDao templateDao;
    @Autowired
    private RehabilitationTemplateDetailDao templateDetailDao;
    @Autowired
    private PatientRehabilitationPlanDao patientRehabilitationPlanDao;

    @Autowired
    RehabilitationPlanService planService;

    @Autowired
    private RehabilitationDetailDao rehabilitationDetailDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private FastDFSUtil fastDFSHelper;

    @Autowired
    protected HttpServletRequest request;
    @Autowired
    private RehabilitationOperateRecordsDao rehabilitationOperateRecordsDao;
    @Autowired
    private SpecialistHospitalServiceItemService hospitalServiceItemService;
    @Autowired
    private SpecialistService specialistService;
    @Autowired
    private HttpClientUtil httpClientUtil;
    @Autowired
    private RehabilitationServiceItemDao serviceItemDao;
    @Autowired
    TemplateJsonDao templateJsonDao;
    @Autowired
    private RehabilitationManageService rehabilitationManageService;
    @Autowired
    private RehabilitationPatientInfoDao rehabilitationPatientInfoDao;
    @Autowired
    private PatientMedicalRecordsDao patientMedicalRecordsDao;

    /**
     * 没有默认创建服务项
     *
     * @param templateDO
     * @return
     */
    public RehabilitationPlanTemplateDO createRehabilitationTemplateNew(RehabilitationPlanTemplateDO templateDO) {
        templateDO.setCreateTime(new Date());
        templateDO.setDel(1);
        templateDO.setUpdateTime(new Date());
        templateDO = templateDao.save(templateDO);
        return templateDO;
    }


    public ObjEnvelop createRehabilitationTemplate(RehabilitationPlanTemplateDO templateDO) {
        templateDO.setCreateTime(new Date());
        templateDO.setDel(1);
        templateDO.setUpdateTime(new Date());
        templateDO = templateDao.save(templateDO);
        String templateId = templateDO.getId();
        String user = templateDO.getCreateUser();
        String userName = templateDO.getCreateUserName();
        //默认新增服务项
        String sql = "SELECT code from wlyy.system_dict WHERE `dict_name` = 'default_service_item_id'";
        List<String> list = jdbcTemplate.queryForList(sql, String.class);
        if (list != null && list.size() > 0) {
            list.stream().forEach(code -> {
                RehabilitationTemplateDetailDO detailDO = new RehabilitationTemplateDetailDO();
                detailDO.setTemplateId(templateId);
                detailDO.setHospitalServiceItemId(code);
                detailDO.setCreateUser(user);
                detailDO.setCreateUserName(userName);
                detailDO.setCreateTime(new Date());
                detailDO.setUpdateTime(new Date());
                templateDetailDao.save(detailDO);
            });
        }
        return ObjEnvelop.getSuccess(SpecialistMapping.api_success, templateDO.getId());
    }


    public ObjEnvelop createRehabilitationTemplateDetail(List<RehabilitationTemplateDetailDO> details) {
        for (RehabilitationTemplateDetailDO detail : details) {
            detail.setCreateTime(new Date());
            detail.setUpdateTime(new Date());
            templateDetailDao.save(detail);
        }
        return ObjEnvelop.getSuccess(SpecialistMapping.api_success, true);
    }

    public ObjEnvelop createRehabilitationTemplateDetail(List<RehabilitationTemplateDetailDO> details, String templateId) {
        List<RehabilitationTemplateDetailDO> detailDOList = templateDetailDao.findTemplateDetailByTemplateId(templateId);
        List<RehabilitationTemplateDetailDO> delList = new ArrayList<>();
        Map<String, String> map = new HashMap<>();
        for (RehabilitationTemplateDetailDO detail : details) {
            if (StringUtils.isNotBlank(detail.getId())) {
                map.put(detail.getId(), "");
            }
            detail.setTemplateId(templateId);
            detail.setCreateTime(new Date());
            detail.setUpdateTime(new Date());
            try {
                templateDetailDao.save(detail);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        for (RehabilitationTemplateDetailDO detailDO : detailDOList) {
            if (!map.containsKey(detailDO.getId())) {
                delList.add(detailDO);
            }
        }
        if (delList.size() > 0) {
            templateDetailDao.deleteAll(delList);
        }
        return ObjEnvelop.getSuccess(SpecialistMapping.api_success, true);
    }


    public ObjEnvelop updateRehabilitationTemplateDetail(List<RehabilitationTemplateDetailDO> details) {
        String templateId = details.get(0).getTemplateId();
        if (templateId != null && templateId.length() > 0) {
            templateDetailDao.deleteByTemplateId(templateId);
        }
        for (RehabilitationTemplateDetailDO detail : details) {

            detail.setCreateTime(new Date());
            detail.setUpdateTime(new Date());

            templateDetailDao.save(detail);
        }
        return ObjEnvelop.getSuccess(SpecialistMapping.api_success, true);
    }

    public MixEnvelop findRehabilitationPlanTemplate(Long adminTeamCode, String doctor, String patient) {
        if (adminTeamCode == null && StringUtils.isNotBlank(doctor) && StringUtils.isNotBlank(patient)) {
            PatientSignInfoVO patientSignSpecialistInfo = (PatientSignInfoVO) specialistService.findPatientSignSpecialistInfo(patient, doctor).getObj();
            adminTeamCode = patientSignSpecialistInfo.getTeamCode();
        }
        List<RehabilitationPlanTemplateDO> list = templateDao.findByAdminTeamCode(adminTeamCode);
        return MixEnvelop.getSuccessList(SpecialistMapping.api_success, list);
    }


    /**
     * 根据模板id修改康复模板删除状态
     *
     * @param id
     * @return
     */
    public ObjEnvelop deleteRehabilitationPlanTemplate(String id) {
        //删除主表
        templateDao.delete(id);
        //detail也删除了
        templateDetailDao.deleteByTemplateId(id);
        return ObjEnvelop.getSuccess(SpecialistMapping.api_success, true);
    }

    public RehabilitationPlanTemplateDO findByTemplateId(String templateId) {
        RehabilitationPlanTemplateDO templateDO = templateDao.findById(templateId).orElse(null);
        List<RehabilitationTemplateDetailDO> list = templateDetailDao.findTemplateDetailByTemplateId(templateId);
        List<RehabilitationServiceItemDO> itemDOList = serviceItemDao.findList();
        if (itemDOList.size() > 0) {
            Map<String, String> map = itemDOList.stream().collect(Collectors.toMap(RehabilitationServiceItemDO::getCode, RehabilitationServiceItemDO::getName));
            for (RehabilitationTemplateDetailDO detailDO : list) {
                detailDO.setName(map.get(detailDO.getHospitalServiceItemId()));
            }
        }
        templateDO.setDetailDOList(list);
        return templateDO;
    }

    /**
     * 根据模板id获取机构服务项目id，然后找出具体服务项目内容
     *
     * @param templateId
     * @return
     */
    public ListEnvelop findTemplateDetailByTemplateId(String templateId) {
        List<TemplateDetailVo> resultList = new ArrayList<>();
        List<RehabilitationTemplateDetailDO> list = templateDetailDao.findTemplateDetailByTemplateId(templateId);
        List<RehabilitationServiceItemDO> itemDOList = serviceItemDao.findList();
        Map<String, List<RehabilitationTemplateDetailDO>> map = list.stream().collect(Collectors.groupingBy(RehabilitationTemplateDetailDO::getHospitalServiceItemId));
        for (String code : map.keySet()) {
            TemplateDetailVo detailVo = new TemplateDetailVo();
            detailVo.setCode(code);
            detailVo.setId(code);
            detailVo.setList(map.get(code));
            for (RehabilitationServiceItemDO item : itemDOList) {
                if (code.equals(item.getCode())) {
                    detailVo.setName(item.getName());
                    for (RehabilitationTemplateDetailDO detailDO : detailVo.getList()) {
                        detailDO.setName(item.getName());
                    }
                    break;
                }
            }
            resultList.add(detailVo);
        }

        return ListEnvelop.getSuccess(SpecialistMapping.api_success, resultList);
    }

    /**
     * 查询专病中心列表
     */
    public MixEnvelop selectByCondition(String name, Integer page, Integer size) {
        String orderBy = "  order by sort asc,create_time desc limit " + (page - 1) * size + "," + size;
        String condition = " ";
        String sql = "SELECT\n" +
                "\tid,\n" +
                "\tname,\n" +
                "\tphoto,\n" +
                "\tbanner,\n" +
                "\tdescription,\n" +
                "\tstatus,\n" +
                "\tdept,\n" +
                "\tsort,\n" +
                "\tdept_name AS deptName,\n" +
                "\tdiagnosis,\n" +
                "\tdiagnosis_name AS diagnosisName,\n" +
                "\tis_disease AS isDisease,\n" +
                "\tplan_doctor AS planDoctor,\n" +
                "\tplan_doctor_name AS planDoctorName,\n" +
                "\tarticle_relation as articleRelation,\n" +
                "\tarticle_relation_name as articleRelationName,\n" +
                "\tis_family as isFamily,\n" +
                "\tfamily_relation as familyRelation,\n" +
                "\tfamily_relation_name as familyRelationName,\n" +
                "\trehabilitation_type 'rehabilitationType'," +
                "\trehabilitation_down_type 'rehabilitationDownType'," +
                "\trehabilitation_template_id 'rehabilitationTemplateId'\n" +
                "FROM\n" +
                "\tbase_disease_hospital dh where dh.status='1' and rehabilitation_type='1' ";
        if (StringUtils.isNoneBlank(name)) {
            condition += " and name like '%" + name + "%' ";
        }
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql + condition + orderBy);
        String sqlCount = "select COUNT(1) as total from base_disease_hospital where 1=1 ";
        Long count = jdbcTemplate.queryForObject(sqlCount + condition, Long.class);
        return MixEnvelop.getSuccessListWithPage("success", list, page, size, count);
    }

    //创建康复计划
    public PatientRehabilitationPlanDO createPatientRehabilitationPlan(PatientRehabilitationPlanDO planDO, List<RehabilitationDetailDO> details) {

        String sql = "SELECT t.plan_doctor,t.plan_doctor_name from wlyy_rehabilitation_plan_template t,base_disease_hospital d " +
                "WHERE d.id='" + planDO.getDisease() + "' and t.id =d.rehabilitation_template_id ";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        if (list.size() > 0) {
            planDO.setPlanDoctor(list.get(0).get("plan_doctor") + "");
            planDO.setPlanDoctorName(list.get(0).get("plan_doctor_name") + "");
        }

        PatientMedicalRecordsDO recordsDO = planDO.getMedicalRecordsDO();
        String medicalRecordsCode = UUID.randomUUID().toString().replaceAll("-", "");
        recordsDO.setCode(medicalRecordsCode);
        recordsDO.setPatient(planDO.getPatient());
        recordsDO.setPatientName(planDO.getName());
        recordsDO.setCreateUser(planDO.getPlanDoctor());
        recordsDO.setCreateUserName(planDO.getPlanDoctorName());
        recordsDO.setDiseaseCode(planDO.getDisease());
        recordsDO.setDiseaseName(planDO.getDiseaseName());
        recordsDO.setCreateTime(new Date());
        patientMedicalRecordsDao.save(recordsDO);

        planDO.setMedicalRecordsCode(medicalRecordsCode);
        String patientInfoCode = planDO.getPatientInfoCode();
        planDO.setStatus(3);//待分配 执行医生和科室
        planDO.setTitle(planDO.getName() + "的康复计划");
        planDO.setHealthStatusCode("2");
        planDO.setPlanType(1);
        planDO.setPayment(2);
        planDO.setCreateTime(new Date());
        planDO.setUpdateTime(new Date());
        planDO = patientRehabilitationPlanDao.save(planDO);

        List<RehabilitationDetailDO> detailDOList = new ArrayList<>();

        for (RehabilitationDetailDO detailDO : details) {
            RehabilitationTemplateDetailDO templateDetailDO = templateDetailDao.findById(detailDO.getId()).orElse(null);
            String frequency_code = templateDetailDO.getFrequencyCode();//频次
            String serverCount = templateDetailDO.getServerCount();

            RehabilitationDetailDO detail = new RehabilitationDetailDO();
            detail.setHospitalServiceItemId(templateDetailDO.getHospitalServiceItemId());
            detail.setCreateTime(new Date());
            detail.setStatus(0);
            detail.setType(1);
            detail.setUpdateTime(new Date());
            detail.setPlanId(planDO.getId());
            detail.setFrequencyCode(frequency_code);
            detail.setServerCount(serverCount);
            if(StringUtils.isNotBlank(templateDetailDO.getPlanStartTimeType())){
                detail.setTimeType(Integer.parseInt(templateDetailDO.getPlanStartTimeType()));
            }
            detail.setServerTimeType(templateDetailDO.getServerTimeType());
//            //补充的科室和医生--有单独的分配接口
//            detail.setDoctor(detailDO.getDoctor());
//            detail.setDoctorName(detailDO.getDoctorName());
//            detail.setDept(detailDO.getDept());
//            detail.setDeptName(detailDO.getDeptName());
            detailDOList.add(detail);
        }
        rehabilitationDetailDao.saveAll(detailDOList);
        rehabilitationManageService.addPlanLog(planDO.getId(), planDO.getPlanDoctor(), planDO.getPlanDoctorName());
        if (StringUtils.isNotBlank(patientInfoCode)) {
            RehabilitationPatientInfoDO infoDO = rehabilitationPatientInfoDao.findByCode(patientInfoCode);
            if (StringUtils.isBlank(infoDO.getDiagnosis())) {
                infoDO.setDept(recordsDO.getDept());
                infoDO.setDeptName(recordsDO.getDeptName());
                infoDO.setDiagnosis(recordsDO.getAdmittingDiagnosis());
                infoDO.setDiagnosisName(recordsDO.getAdmittingDiagnosisName());
            }
            infoDO.setStatus(1);
            infoDO.setDoctor(planDO.getPlanDoctor());
            infoDO.setDoctorName(planDO.getPlanDoctorName());
            infoDO.setReceiveTime(new Date());
            rehabilitationPatientInfoDao.save(infoDO);
        }
        planDO.setDetailDOList(detailDOList);
        return planDO;
    }

    public List<RehabilitationDetailDO> createRehabilitationDetail(List<RehabilitationDetailDO> details, String planId) {
        for (RehabilitationDetailDO detail : details) {
            detail.setPlanId(planId);
            detail.setCreateTime(new Date());
            detail.setStatus(0);
            detail.setUpdateTime(new Date());
        }
        return (List<RehabilitationDetailDO>) rehabilitationDetailDao.saveAll(details);
    }

    public MixEnvelop findServiceItemsByHospital(String doctorHospital, String signHospital) {
        JSONArray jsonArray = new JSONArray();
        List<String> list = new ArrayList<>();
        list.add(doctorHospital);
        List<HospitalServiceItemDO> docHospitalServiceItemDO = hospitalServiceItemService.selectByHospital(list).getDetailModelList();
        if (StringUtils.isNotBlank(signHospital)) {
            if (signHospital.equals(doctorHospital)) {
                for (Object object : docHospitalServiceItemDO) {
                    JSONObject json = (JSONObject) JSONObject.toJSON(object);
                    json.put("type", "社区");
                    jsonArray.add(json);
                }
            } else if (!signHospital.equals(doctorHospital)) {
                JSONArray array = new JSONArray();
                List<String> itemIds = new ArrayList<>();
                list.remove(0);
                list.add(signHospital);
                List<HospitalServiceItemDO> signHospitalServiceItemDO = hospitalServiceItemService.selectByHospital(list).getDetailModelList();
                HashSet signSet = new HashSet();
                for (HospitalServiceItemDO h : signHospitalServiceItemDO) {
                    signSet.add(h.getServiceItemId());
                }
                //将服务项目重复的都放入itemIds集合里,并在列表中删除该条数据
                List<HospitalServiceItemDO> temp = new ArrayList<>();
                temp.addAll(docHospitalServiceItemDO);
                for (HospitalServiceItemDO h : docHospitalServiceItemDO) {
                    int size = signSet.size();
                    String itemId = h.getServiceItemId();
                    signSet.add(itemId);
                    if (signSet.size() == size) {
                        itemIds.add(itemId);
                        temp.remove(h);
                    }
                }
                array.addAll(signHospitalServiceItemDO);
                array.addAll(temp);
                for (Object obj : array) {
                    //往实体类插入数据
                    JSONObject json = (JSONObject) JSONObject.toJSON(obj);
                    if (json.get("hospital").equals(signHospital)) {
                        json.put("type", "社区");
                    } else if (json.get("hospital").equals(doctorHospital)) {
                        json.put("type", "医院");
                    }
                    if (itemIds.contains(json.get("serviceItemId"))) {
                        json.put("type", "社区，医院");
                    }
                    jsonArray.add(json);
                }
            }
        } else {
            jsonArray.addAll(docHospitalServiceItemDO);
        }
        return MixEnvelop.getSuccessList(SpecialistMapping.api_success, jsonArray);
    }

    /**
     * 调用服务包创建接口
     *
     * @param planDO
     * @param details
     */
    public String addServicePackage(PatientRehabilitationPlanDO planDO, List<RehabilitationDetailDO> details) {
        JSONObject jsonData = new JSONObject();
        JSONObject servicePackageVO = new JSONObject();
        JSONObject signRecordVO = new JSONObject();
        JSONArray detailsVOList = new JSONArray();
        //服务项目数据
        for (RehabilitationDetailDO detail : details) {
            JSONObject detailsVO = new JSONObject();
            detailsVO.put("executionType", "1");
            detailsVO.put("executionTime", DateUtil.dateToStr(detail.getExecuteTime(), "yyyy-MM-dd hh:mm:ss"));
            List<String> hospitalServiceIds = new ArrayList<>();
            hospitalServiceIds.add(detail.getHospitalServiceItemId());
            HospitalServiceItemDO signHospitalServiceItemDO = hospitalServiceItemService.selectById(hospitalServiceIds).getDetailModelList().get(0);
            detailsVO.put("code", signHospitalServiceItemDO.getServiceItemId());
            detailsVO.put("name", signHospitalServiceItemDO.getSpecialistServiceItemDO().getTitle());
            detailsVOList.add(detailsVO);
        }
        //服务包数据
        servicePackageVO.put("detailsVOList", detailsVOList);
        servicePackageVO.put("num", detailsVOList.size());
        servicePackageVO.put("saasId", "xmihealth");
        servicePackageVO.put("name", planDO.getTitle());
        servicePackageVO.put("introduce", planDO.getTitle());
        servicePackageVO.put("creater", planDO.getCreateUser());
        servicePackageVO.put("type", "1");
        servicePackageVO.put("price", planDO.getTotalExpense());
        //签约记录数据
        signRecordVO.put("servicePackageName", planDO.getTitle());
        signRecordVO.put("patient", planDO.getPatient());
        signRecordVO.put("name", planDO.getName());
        PatientSignInfoVO patientSignInfoVO = (PatientSignInfoVO) specialistService.findPatientSignSpecialistInfo(planDO.getPatient(), planDO.getCreateUser()).getObj();
        signRecordVO.put("idcard", patientSignInfoVO.getIdcard());
        signRecordVO.put("ssc", patientSignInfoVO.getSsc());
        signRecordVO.put("signDoctor", patientSignInfoVO.getDoctor());
        signRecordVO.put("signDoctorName", patientSignInfoVO.getDoctorName());
        signRecordVO.put("hospital", patientSignInfoVO.getHospital());
        signRecordVO.put("hospitalName", patientSignInfoVO.getHospitalName());
        signRecordVO.put("adminTeamCode", patientSignInfoVO.getTeamCode());
        signRecordVO.put("price", planDO.getTotalExpense());
        signRecordVO.put("saasId", "xmihealth");

        jsonData.put("servicePackageVO", servicePackageVO);
        jsonData.put("signRecordVO", signRecordVO);

        String response = null;
        try {
            response = httpClientUtil.postBody(baseUrl + "base_rehabilitation/create", jsonData);
        } catch (Exception e) {
            e.printStackTrace();
        }
        JSONObject rs = JSONObject.parseObject(response);
        if ("success".equals(rs.getString("message"))) {
            return rs.getJSONObject("obj").getString("id");
        }
        return null;
    }

    public void updateServicePackageId(String planId, String servicePackageId) {
        patientRehabilitationPlanDao.updateServicePackageId(planId, servicePackageId);
    }

    public MixEnvelop<String, String> createServiceQrCode(String planDetailId, String doctorCode, String imageUrl) {
        //更新到康复计划居民关系表中
        String sql = "update wlyy_rehabilitation_plan_detail set service_qr_code='" + imageUrl + "' where id='" + planDetailId + "'";
        jdbcTemplate.update(sql);
        return MixEnvelop.getSuccess("获取二维码成功！", imageUrl);
    }

    public Map<String, Object> checkAfterQrCode(String planDetailId, String patietCode) throws Exception {
        Map<String, Object> map = new HashedMap();
        int result = 0;
        String name = "";
        String sql = "SELECT p.patient,p.name FROM `wlyy_rehabilitation_plan_detail` pd LEFT JOIN wlyy_patient_rehabilitation_plan p ON pd.plan_id = p.id WHERE pd.id='" + planDetailId + "'";
        sql = TransforSqlUtl.wlyy_patient_rehabilitation_planName(sql);
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        if (list != null && list.size() > 0) {
            if (String.valueOf(list.get(0).get("patient")).equals(patietCode)) {
                result = 200;
            } else {
                result = -1;
                name = String.valueOf(list.get(0).get("name"));
            }
        } else {
            result = -10000;
        }
        map.put("code", result);
        if (StringUtils.isNotBlank(name)) {
            map.put("name", name);
        }
        return map;
    }

    public RehabilitationOperateRecordsDO saveRehabilitationRecord(RehabilitationOperateRecordsDO rehabilitationOperateRecordsDO) {
        RehabilitationDetailDO rehabilitationDetailDO = rehabilitationDetailDao.findById(rehabilitationOperateRecordsDO.getRehabilitationDetailId());
        if (!StringUtils.isNoneBlank(rehabilitationOperateRecordsDO.getId())) {
            rehabilitationOperateRecordsDO.setId(getCode());
        }
        if (rehabilitationDetailDO.getExecuteTime() != null) {
            rehabilitationOperateRecordsDO.setReserveTime(rehabilitationDetailDO.getExecuteTime());
        }
        rehabilitationOperateRecordsDO.setCompleteTime(new Date());
        rehabilitationOperateRecordsDO.setCreateTime(new Date());
        rehabilitationOperateRecordsDO.setUpdateTime(new Date());
        rehabilitationOperateRecordsDO.setStatus(rehabilitationOperateRecordsDO.getStatus());
        if (StringUtils.isEmpty(rehabilitationOperateRecordsDO.getPatientName())) {
            String sql = "select name from " + basedb + ".wlyy_patient where code='" + rehabilitationOperateRecordsDO.getPatientCode() + "'";
            List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
            if (list != null && list.size() > 0) {
                rehabilitationOperateRecordsDO.setPatientName(String.valueOf(list.get(0).get("name")));
            }
        }
        return rehabilitationOperateRecordsDao.save(rehabilitationOperateRecordsDO);
    }

    /**
     * 更新康复计划项目状态
     *
     * @param status
     * @param planId
     * @return
     */
    public MixEnvelop updatePlanStatusById(Integer status, String planId) throws Exception {
        if (status == 1) {
            if (patientRehabilitationPlanDao.updateStatusById(status, planId) > 0) {

                return MixEnvelop.getSuccess(SpecialistMapping.api_success, patientRehabilitationPlanDao.findById(planId));
            }
        } else if (status == 2) {
            if (patientRehabilitationPlanDao.updateStatusAndTimeById(2, planId, new Date()) > 0) {

                return MixEnvelop.getSuccess(SpecialistMapping.api_success, patientRehabilitationPlanDao.findById(planId));
            }
        }
        return MixEnvelop.getError("更新失败！");
    }

    public MixEnvelop abortPlanById(String planId, String abortReason) {
        PatientRehabilitationPlanDO patientRehabilitationPlanDO = patientRehabilitationPlanDao.findById(planId).orElse(null);
        List<RehabilitationDetailDO> detailDOList = rehabilitationDetailDao.findByPlanIdAndStatus(planId, 0);
        if (null != patientRehabilitationPlanDO) {
            patientRehabilitationPlanDO.setAbortReason(abortReason);
            patientRehabilitationPlanDO.setAbortTime(new Date());
            patientRehabilitationPlanDO.setStatus(0);
            patientRehabilitationPlanDO.setUpdateTime(new Date());
            patientRehabilitationPlanDao.save(patientRehabilitationPlanDO);
            return MixEnvelop.getSuccess(SpecialistMapping.api_success, patientRehabilitationPlanDO, detailDOList);
        }
        return MixEnvelop.getError("更新失败！");
    }


    /**
     * 根据标签和居民code查找康复计划
     *
     * @param patient
     * @param label
     * @return
     */
    public List<PatientRehabilitationPlanDO> selectByLabelAndPatient(String patient, String label) {
        return patientRehabilitationPlanDao.selectByDiseaseAndPatient(label, patient);
    }

    public Envelop updatePatientImgById(String planId, String patientImg) {
        if (patientRehabilitationPlanDao.updatePatientImgById(planId, patientImg) > 0) {
            return Envelop.getSuccess(SpecialistMapping.api_success);
        }
        return Envelop.getError("更新失败！");
    }

    public void deleteDetailByPlanIdAndHospitalServiceItemId(String planId, String hospitalServiceItemId, Date dateShort) {
        rehabilitationDetailDao.deleteDetailByPlanIdAndHospitalServiceItemId(planId, hospitalServiceItemId, dateShort);
    }

    public void deleteDetailByPlanId(String planId, Date dateShort) {
        rehabilitationDetailDao.deleteDetailByPlanId(planId, dateShort);
    }

    public HashMap<String, Object> findRehabilitationPlanTemplatenew(
            String templateId, String templateName, String status, String diseaseId, String type, String pageIf, int page, int pageSize) {
        String countSql = "select count(1) ";
        String detailSql = "SELECT \n" +
                "  (SELECT GROUP_CONCAT(b.`name`) FROM 	wlyy_rehabilitation_template_detail q \n" +
                "	 INNER JOIN wlyy_rehabilitation_service_item b ON q.hospital_service_item_id=b.`code` WHERE q.template_id=a.id) 'itemName',\n" +
                "	DATE_FORMAT(a.create_time, '%Y-%m-%d %H:%i:%s') 'createTime'," +
                "   DATE_FORMAT(a.update_time, '%Y-%m-%d %H:%i:%s') 'updateTime', a.* \n";
        String sql = " FROM	wlyy_rehabilitation_plan_template a WHERE 1=1\n";
        String pageSql = "";
        if (StringUtils.isNotBlank(templateId)) {
            sql += " AND a.id = '" + templateName + "' ";
        }
        if (StringUtils.isNotBlank(templateName)) {
            sql += " AND a.title LIKE '%" + templateName + "%' ";
        }
        if (StringUtils.isNotBlank(status)) {
            sql += "  AND a.status='" + status + "' ";
        }
        if (StringUtils.isNotBlank(diseaseId)) {
            sql += "  AND a.disease_id='" + diseaseId + "' ";
        }
        if (StringUtils.isNotBlank(type)) {
            sql += "  AND a.type='" + type + "' ";
        }
        if (StringUtils.isNotBlank(pageIf)) {
            pageSql += " LIMIT " + (page - 1) * pageSize + "," + pageSize;
        }
        detailSql += sql + pageSql;
        countSql += sql;
        List<Map<String, Object>> list = jdbcTemplate.queryForList(detailSql);
        Integer count = jdbcTemplate.queryForObject(countSql, Integer.class);
        HashMap<String, Object> map = new HashMap<>();
        map.put("count", count);
        map.put("list", list);
        return map;
    }

    public List<Map<String, Object>> findRehabilitationServiceItem(String itemName) {
        String sql = " SELECT * FROM base.wlyy_rehabilitation_service_item WHERE 1=1 ";
        if (StringUtils.isNotBlank(itemName)) {
            sql += " and `name` LIKE '%" + itemName + "%'";
        }
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        return list;
    }

    public TemplateJson saveTemplate(String templateName, String templateJson) {
        TemplateJson entity = new TemplateJson();
        entity.setCreateTime(new Date());
        entity.setTemplateName(templateName);
        entity.setTemplateJson(templateJson);
        templateJsonDao.save(entity);
        return entity;
    }

    public List<Map<String, Object>> findDiseaseList() {
        String sql = "SELECT * FROM base_disease_hospital WHERE `status`='1' ORDER BY sort ";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        return list;
    }

    public List<Map<String, Object>> findSaveTemplate(String templateName) {
        String sql = "select * FROM template_json WHERE 1=1 ";
        if (StringUtils.isNotBlank(templateName)) {
            sql += " and template_name LIKE '%" + templateName + "%'  ";
        }
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        return list;
    }

    /**
     * 新增整个模板
     *
     * @param templateDO
     * @param details
     * @return
     */
    public ObjEnvelop saveRehabilitationPlanTemplateData(RehabilitationPlanTemplateDO templateDO, List<RehabilitationTemplateDetailDO> details) {
        RehabilitationPlanTemplateDO planTemplateDO = this.createRehabilitationTemplateNew(templateDO);
        this.createRehabilitationTemplateDetail(details, planTemplateDO.getId());
        return ObjEnvelop.getSuccess(SpecialistMapping.api_success, true);
    }

    public RehabilitationPlanTemplateDO changeState(ParamQvo qvo) {
        RehabilitationPlanTemplateDO entity = templateDao.findById(qvo.getId()).orElse(null);
        if (entity != null) {
            entity.setStatus(qvo.getStatus());
            templateDao.save(entity);
            return entity;
        }
        return null;
    }

    public ListEnvelop findFrequencyList() {

        //        String sql = " SELECT * FROM `base`.`wlyy_frequency` WHERE del='1' ";
//        List<Frequency> list = jdbcTemplate.query(sql, new BeanPropertyRowMapper(Frequency.class));
//        return ListEnvelop.getSuccess(SpecialistMapping.api_success, list);
        String sql = " SELECT dict_code 'code',dict_value 'name' from wlyy_hospital_sys_dict WHERE dict_name = 'plan_frequency' ";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        return ListEnvelop.getSuccess(SpecialistMapping.api_success, list);
    }

    public List<String> getFrequencyDate(String code, String date, String unit, String time, Long timeType) throws ParseException {
        List<String> list = new ArrayList<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String nowDate = date + ":00";
        Date date6 = DateUtil.strToDateLong(nowDate);
        String date9 = sdf.format(date6) + " 09:00:00";//白天
        String date21 = sdf.format(date6) + " 21:00:00";//白天
        String date9s = DateUtil.getNextDay(date6, 1) + " 09:00:00";//晚上
        Calendar calendar = Calendar.getInstance();
        if (unit.equalsIgnoreCase("D") && !code.equalsIgnoreCase("qod")) {
            String[] str = time.split(",");
            for (int i = 0; i < str.length; i++) {
                Date date1 = sdf.parse(date);
                calendar.setTime(date1);
                String lastDay = sdf.format(calendar.getTime()) + " " + str[i];
                list.add(lastDay);
            }
        } else if (unit.equalsIgnoreCase("D") && code.equalsIgnoreCase("qod")) {
            Date date1 = sdf.parse(date);
            calendar.setTime(date1);
            calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) + 1);
            String lastDay = sdf.format(calendar.getTime()) + " " + time;
            list.add(lastDay);
        } else if (unit.equalsIgnoreCase("H") && code.equalsIgnoreCase("q0.5h")) {
            Date date1 = sdf.parse(date);
            calendar.setTime(date1);
            String endTime = null;
            String lastDay = null;
            if (timeType == 0) {
                lastDay = sdf.format(calendar.getTime()) + " 00:00:00";
                endTime = sdf.format(calendar.getTime()) + " 23:59:59";
            } else if (timeType == 1) {
                lastDay = date9;
                endTime = date21;
            } else if (timeType == 2) {
                lastDay = date21;
                endTime = date9s;
            }
            Date date2 = sdf1.parse(lastDay);
            Date date3 = sdf1.parse(endTime);
            Long time1 = date2.getTime();
            for (int i = 0; i < 47; i++) {
                time1 += 30 * 60 * 1000;
                String day = sdf1.format(time1);
                Date date4 = sdf1.parse(day);
                if (date4.compareTo(date3) == -1 || date4.compareTo(date3) == 0) {
                    list.add(day);
                }
            }
        } else if (unit.equalsIgnoreCase("H") && code.equalsIgnoreCase("q2h")) {
            Date date1 = sdf.parse(date);
            calendar.setTime(date1);
            /*      String lastDay = sdf.format(calendar.getTime()) + " 00:00:00";*/
            String endTime = null;
            String lastDay = null;
            if (timeType == 0) {
                lastDay = sdf.format(calendar.getTime()) + " 00:00:00";
                endTime = sdf.format(calendar.getTime()) + " 23:59:59";
            } else if (timeType == 1) {
                lastDay = date9;
                endTime = date21;
            } else if (timeType == 2) {
                lastDay = date21;
                endTime = date9s;
            }
            Date date2 = sdf1.parse(lastDay);
            Date date3 = sdf1.parse(endTime);
            calendar.setTime(date2);
            for (int i = 0; i < 11; i++) {
                calendar.set(Calendar.HOUR, calendar.get(Calendar.HOUR) + 2);
                Date date4 = sdf1.parse(sdf1.format(calendar.getTime()));
                System.out.print(sdf1.format(calendar.getTime()));
                if (date4.compareTo(date3) == -1 || date4.compareTo(date3) == 0) {
                    list.add(sdf1.format(calendar.getTime()));
                }
            }

        } else if (unit.equalsIgnoreCase("H") && code.equalsIgnoreCase("q3h")) {
            Date date1 = sdf.parse(date);
            calendar.setTime(date1);
            //String lastDay = sdf.format(calendar.getTime()) + " 00:00:00";
            String endTime = null;
            String lastDay = null;
            if (timeType == 0) {
                lastDay = sdf.format(calendar.getTime()) + " 00:00:00";
                endTime = sdf.format(calendar.getTime()) + " 23:59:59";
            } else if (timeType == 1) {
                lastDay = date9;
                endTime = date21;
            } else if (timeType == 2) {
                lastDay = date21;
                endTime = date9s;
            }
            Date date2 = sdf1.parse(lastDay);
            Date date3 = sdf1.parse(endTime);
            calendar.setTime(date2);
            for (int i = 0; i < 7; i++) {
                calendar.set(Calendar.HOUR, calendar.get(Calendar.HOUR) + 3);
                Date date4 = sdf1.parse(sdf1.format(calendar.getTime()));
                if (date4.compareTo(date3) == -1 || date4.compareTo(date3) == 0) {
                    list.add(sdf1.format(calendar.getTime()));
                }

            }
        } else if (unit.equalsIgnoreCase("H") && code.equalsIgnoreCase("q4h")) {
            Date date1 = sdf.parse(date);
            calendar.setTime(date1);
            //String lastDay = sdf.format(calendar.getTime()) + " 00:00:00";
            String endTime = null;
            String lastDay = null;
            if (timeType == 0) {
                lastDay = sdf.format(calendar.getTime()) + " 00:00:00";
                endTime = sdf.format(calendar.getTime()) + " 23:59:59";
            } else if (timeType == 1) {
                lastDay = date9;
                endTime = date21;
            } else if (timeType == 2) {
                lastDay = date21;
                endTime = date9s;
            }
            Date date2 = sdf1.parse(lastDay);
            Date date3 = sdf1.parse(endTime);
            calendar.setTime(date2);
            for (int i = 0; i < 5; i++) {
                calendar.set(Calendar.HOUR, calendar.get(Calendar.HOUR) + 4);
                Date date4 = sdf1.parse(sdf1.format(calendar.getTime()));
                if (date4.compareTo(date3) == -1 || date4.compareTo(date3) == 0) {
                    list.add(sdf1.format(calendar.getTime()));
                }

            }
        } else if (unit.equalsIgnoreCase("H") && code.equalsIgnoreCase("qh")) {
            Date date1 = sdf.parse(date);
            calendar.setTime(date1);
            //String lastDay = sdf.format(calendar.getTime()) + " 00:00:00";
            String endTime = null;
            String lastDay = null;
            if (timeType == 0) {
                lastDay = sdf.format(calendar.getTime()) + " 00:00:00";
                endTime = sdf.format(calendar.getTime()) + " 23:59:59";
            } else if (timeType == 1) {
                lastDay = date9;
                endTime = date21;
            } else if (timeType == 2) {
                lastDay = date21;
                endTime = date9s;
            }
            Date date2 = sdf1.parse(lastDay);
            Date date3 = sdf1.parse(endTime);
            calendar.setTime(date2);
            for (int i = 0; i < 23; i++) {
                calendar.set(Calendar.HOUR, calendar.get(Calendar.HOUR) + 1);
                Date date4 = sdf1.parse(sdf1.format(calendar.getTime()));
                if (date4.compareTo(date3) == -1 || date4.compareTo(date3) == 0) {
                    list.add(sdf1.format(calendar.getTime()));
                }
            }
        } else if (unit.equalsIgnoreCase("W") && code.equalsIgnoreCase("qw_1")) {
            String date2 = DateUtil.getWeek(date, "1") + " 08:00:00";
            Date date1 = new Date();
            String date3 = sdf1.format(date1);
            if (date2.compareTo(date3) == 1) {
                list.add(date2);
            }
        } else if (unit.equalsIgnoreCase("W") && code.equalsIgnoreCase("qw_2")) {
            String date2 = DateUtil.getWeek(date, "2") + " 08:00:00";
            Date date1 = new Date();
            String date3 = sdf1.format(date1);
            if (date2.compareTo(date3) == 1) {
                list.add(date2);
            }
        } else if (unit.equalsIgnoreCase("W") && code.equalsIgnoreCase("qw_3")) {
            String date2 = DateUtil.getWeek(date, "3") + " 08:00:00";
            Date date1 = new Date();
            String date3 = sdf1.format(date1);
            if (date2.compareTo(date3) == 1) {
                list.add(date2);
            }
        } else if (unit.equalsIgnoreCase("W") && code.equalsIgnoreCase("qw_4")) {
            String date2 = DateUtil.getWeek(date, "4") + " 08:00:00";
            Date date1 = new Date();
            String date3 = sdf1.format(date1);
            if (date2.compareTo(date3) == 1) {
                list.add(date2);
            }
        } else if (unit.equalsIgnoreCase("W") && code.equalsIgnoreCase("qw_5")) {
            String date2 = DateUtil.getWeek(date, "5") + " 08:00:00";
            Date date1 = new Date();
            String date3 = sdf1.format(date1);
            if (date2.compareTo(date3) == 1) {
                list.add(date2);
            }
        } else if (unit.equalsIgnoreCase("W") && code.equalsIgnoreCase("qw_6")) {
            String date2 = DateUtil.getWeek(date, "6") + " 08:00:00";
            Date date1 = new Date();
            String date3 = sdf1.format(date1);
            if (date2.compareTo(date3) == 1) {
                list.add(date2);
            }
        } else if (unit.equalsIgnoreCase("W") && code.equalsIgnoreCase("qw_7")) {
            String date2 = DateUtil.getWeek(date, "7") + " 08:00:00";
            Date date1 = new Date();
            String date3 = sdf1.format(date1);
            if (date2.compareTo(date3) == 1) {
                list.add(date2);
            }
        } else if (unit.equalsIgnoreCase("W") && code.equalsIgnoreCase("tiw135")) {
            String date2 = DateUtil.getWeek(date, "1") + " 08:00:00";
            String date4 = DateUtil.getWeek(date, "3") + " 08:00:00";
            String date5 = DateUtil.getWeek(date, "5") + " 08:00:00";
            Date date1 = new Date();
            String date3 = sdf1.format(date1);
            if (date2.compareTo(date3) == 1) {
                list.add(date2);
            }
            if (date4.compareTo(date3) == 1) {
                list.add(date2);
            }
            if (date5.compareTo(date3) == 1) {
                list.add(date2);
            }
        } else if (unit.equalsIgnoreCase("W") && code.equalsIgnoreCase("tiw246")) {
            String date2 = DateUtil.getWeek(date, "2") + " 08:00:00";
            String date4 = DateUtil.getWeek(date, "4") + " 08:00:00";
            String date5 = DateUtil.getWeek(date, "6") + " 08:00:00";
            Date date1 = new Date();
            String date3 = sdf1.format(date1);
            if (date2.compareTo(date3) == 1) {
                list.add(date2);
            }
            if (date4.compareTo(date3) == 1) {
                list.add(date2);
            }
            if (date5.compareTo(date3) == 1) {
                list.add(date2);
            }
        }
        return list;
    }

    /**
     * 获取康复计划修改记录
     *
     * @param planId
     * @return
     */
    public List<Map<String, Object>> getRehabilitationPlanLog(String planId) {
        String sql = "select date_format(a.create_time, '%Y-%m-%d %H:%i:%s') 'createTime', a.*  " +
                "from wlyy_patient_rehabilitation_plan_log a " +
                "where a.plan_id = '" + planId + "' ORDER BY a.create_time DESC";
        List<Map<String, Object>> planLogList = jdbcTemplate.queryForList(sql);
        return planLogList;
    }

    public List<Map<String, Object>> getRehabilitationDetailItemByPatient(String patientId, String planId) {
        String sql = "SELECT\n" +
                "	b.title 'planTitle', c.`name` 'itemName' ,b.`status` 'planStatus',a.* \n" +
                "FROM\n" +
                "	wlyy_rehabilitation_plan_detail a\n" +
                "	INNER JOIN wlyy_patient_rehabilitation_plan b ON a.plan_id=b.id\n" +
                "	INNER JOIN wlyy_rehabilitation_service_item c ON a.hospital_service_item_id=c.`id`\n" +
                "	WHERE 1=1 \n" +
                "	AND a.plan_id='" + planId + "'\n" +
                "	AND b.patient='" + patientId + "' ";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        return list;
    }


}
