package com.yihu.jw.hospital.module.specialist.service;

import com.yihu.jw.restmodel.specialist.SurveyScreenResultVo;
import com.yihu.jw.restmodel.specialist.SurveyTemplateQuestionsVo;
import com.yihu.jw.restmodel.web.MixEnvelop;
import com.yihu.jw.rm.specialist.SpecialistMapping;
import com.yihu.jw.util.common.IdCardUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * Created by zhangdan on 2018/7/6.
 */
@Service
@Transactional
public class SpecialistScreenResultService {
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Value("${basedb.name}")
    private String basedb;



    public MixEnvelop<SurveyScreenResultVo, SurveyScreenResultVo> getScreenList(String doctor, Integer type, Integer page, Integer size, String patientName, String isDanger, String diseaseType, Integer filterType, String patientCode) throws Exception {
        StringBuilder sql  =new StringBuilder("SELECT DISTINCT ssr.id,ssr.`code`,ssr.template_code templateCode,ssr.template_title templateTitle,ssr.disease,ssr.doctor ,ssr.patient_code patientCode,ssr.patient_name patientName,p.idcard,p.photo,ssr.screen_result_code screenResultCode,ssr.screen_result_score screenResultScore,ssr.screen_result screenResult,ssr.is_danger isDanger,ssr.is_order isOrder,ssr.following,ssr.is_educate isEducate,ssr.over,ssr.reservation_code reservationCode,ssr.czrq,ssr.is_again isAgain,ssr.parent_code parentCode,ssr.origin_code originCode,ssr.source, ssr.third_id thirdId, ssr.third_source thirdSource, ssr.third_advice thirdAdvice" +
                " FROM  "+basedb+".wlyy_survey_screen_result ssr " +
                "LEFT JOIN wlyy_specialist_patient_relation spr ON ssr.patient_code= spr.patient " +
                "LEFT JOIN "+basedb+".wlyy_patient p on p.`status` = 1 and ssr.patient_code = p.`code` " +
                "LEFT JOIN "+basedb+".wlyy_patient_reservation pr on pr.code = ssr.reservation_code " +
                "LEFT JOIN "+basedb+".wlyy_order_doctor_mapping d on d.jw_doctor = pr.doctor_code " +
        "WHERE 1=1 " +
                " AND ssr.over=1 and ssr.reservation_code is not null ");
        if (StringUtils.isNotBlank(isDanger)) {
            sql.append(" AND ssr.is_danger= " + isDanger) ;
        }
        if (StringUtils.isNotBlank(diseaseType)) {
            sql.append(" AND ssr.disease in (" + diseaseType + ") ");
        }
        if (StringUtils.isNotBlank(patientName)) {
            sql.append(" AND (ssr.patient_name like '%" + patientName + "%' or p.idcard like '%" + patientName + "%') ");
        }
        if(type != null) {
            String dealTypeSql = "";
            if (type == 1){
                //已预约
                dealTypeSql += "ssr.is_order = 1 ";
            }
            if (type == 2) {
                //已跟踪
                dealTypeSql += "or ssr.following= 1 ";
            }
            if (type == 3) {
                //已接诊
                dealTypeSql += "or ssr.is_order = 2 ";
            }
            if (type == 4) {
                //待处理
                dealTypeSql += "or (ssr.is_order = 0 AND ssr.following= 0) ";
            }
            if(dealTypeSql.startsWith("or")){
                dealTypeSql = dealTypeSql.replaceFirst("or", "");
            }
            sql.append(" AND (" + dealTypeSql +") ") ;
        }
        StringBuilder sqlUnion = new StringBuilder(sql);

        if(filterType == 0){
            sql.append(" AND ((spr.`status`>=0 AND spr.sign_status>0 AND spr.doctor='"+doctor+"')) ") ;
            sql.append(" UNION "+ sqlUnion );
            sql.append(" AND (d.doctor = '"+doctor+"') ");
        }else {
            sql.append(" AND ((spr.`status`>=0 AND spr.sign_status>0 AND ssr.patient_code = '"+patientCode+"' )) ") ;
            sql.append(" UNION "+ sqlUnion );
            sql.append(" AND ssr.patient_code = '"+patientCode+"' ");
        }

        int count = 0;

        List<SurveyScreenResultVo> surveyScreenResultVoList = new ArrayList<>();

        if(filterType == 0){
            List<Map<String,Object>> rstotal = jdbcTemplate.queryForList(sql.toString());
            if(rstotal!=null&&rstotal.size()>0){
                count = rstotal.size();
            }
            sql.append(" ORDER BY czrq DESC LIMIT "+(page-1)*size+","+page*size) ;
            surveyScreenResultVoList = jdbcTemplate.query(sql.toString(),new BeanPropertyRowMapper(SurveyScreenResultVo.class));
        }else{
            //每种疾病类型取最新的一条
            sql.insert(0,"SELECT * from ( ");
            sql.append(" ORDER BY czrq DESC ) t  GROUP BY t.disease ");
            surveyScreenResultVoList = jdbcTemplate.query(sql.toString(),new BeanPropertyRowMapper(SurveyScreenResultVo.class));
            if(CollectionUtils.isNotEmpty(surveyScreenResultVoList)){
                count = surveyScreenResultVoList.size();
                surveyScreenResultVoList.sort(new Comparator<SurveyScreenResultVo>() {
                    @Override
                    public int compare(SurveyScreenResultVo o1, SurveyScreenResultVo o2) {
                        return o1.getCzrq().before(o2.getCzrq())?1:-1;
                    }
                });
            }
        }
        for(SurveyScreenResultVo vo : surveyScreenResultVoList){
            String idcard = vo.getIdcard();
            String sex = IdCardUtil.getSexForIdcard_new(idcard);
            Integer age = IdCardUtil.getAgeForIdcard(idcard);
            vo.setAge(age);
            vo.setSex(sex);
        }
        return MixEnvelop.getSuccessListWithPage(SpecialistMapping.api_success,surveyScreenResultVoList,page,size,Long.valueOf(count));
    }

    public MixEnvelop<Map<String,Object>, Map<String,Object>> getResultCount(String doctor){
        String followSql = "SELECT count(*) followNumber" +
                " FROM  "+basedb+".wlyy_survey_screen_result ssr LEFT JOIN wlyy_specialist_patient_relation spr ON ssr.patient_code= spr.patient WHERE spr.`status`>=0 AND spr.sign_status>0 AND spr.doctor='"+doctor+"' AND ssr.over=1 AND ssr.following =1";

        String orderSwl = "SELECT count(*) orderNumber" +
                " FROM  "+basedb+".wlyy_survey_screen_result ssr LEFT JOIN  "+basedb+".wlyy_patient_reservation pr ON ssr.reservation_code = pr.`code` WHERE pr.status =1 " +
                " AND pr.doctor_code = (SELECT jw_doctor from wlyy.wlyy_doctor_mapping WHERE doctor_code='"+doctor+"') AND ssr.over = 1 AND ssr.is_order>0";

        Map<String,Object> followMap = jdbcTemplate.queryForMap(followSql);
        Map<String,Object> orderMap = jdbcTemplate.queryForMap(orderSwl);
        orderMap.putAll(followMap);
        return MixEnvelop.getSuccess(SpecialistMapping.api_success,orderMap);
    }

    public MixEnvelop<Map<String,Object>, Map<String,Object>> getScreenResultDetail(String code)throws Exception{
        Map<String,Object> map = new HashMap<>();
        //登记信息
        String infoSql = "SELECT ssr.id,ssr.`code`,p.photo,ssr.template_code templateCode,ssr.template_title templateTitle,ssr.disease,ssr.doctor,ssr.patient_code patientCode,ssr.patient_name patientName,ssr.screen_result_code screenResultCode,ssr.screen_result_score screenResultScore,ssr.screen_result screenResult,ssr.is_danger isDanger,ssr.is_order isOrder,ssr.following,ssr.is_educate isEducate,ssr.over,ssr.reservation_code reservationCode,ssr.czrq,ssr.is_again isAgain,ssr.parent_code parentCode,ssr.origin_code originCode,ssr.advice_code adviceCode,ssr.other_advice otherAdvice,ssr.source,p.idcard, ssr.third_id thirdId, ssr.third_source thirdSource, ssr.third_advice thirdAdvice " +
                " FROM "+basedb+".wlyy_survey_screen_result ssr LEFT JOIN " +basedb+
                ".wlyy_patient p ON ssr.patient_code=p.code where ssr.code ='"+code+"'";
        List<SurveyScreenResultVo> surveyScreenResultVoList = jdbcTemplate.query(infoSql,new BeanPropertyRowMapper<>(SurveyScreenResultVo.class));
        if (surveyScreenResultVoList==null || surveyScreenResultVoList.size()==0){
            return MixEnvelop.getError("没有改筛查结果",-1);
        }
        SurveyScreenResultVo surveyScreenResultVo = surveyScreenResultVoList.get(0);
        String templateCode = surveyScreenResultVo.getTemplateCode();
        String patientCode = surveyScreenResultVo.getPatientCode();
        String idcard = surveyScreenResultVo.getIdcard();
        surveyScreenResultVo.setSex(IdCardUtil.getSexForIdcard_new(idcard));
        surveyScreenResultVo.setAge(IdCardUtil.getAgeForIdcard(idcard));
        String doctorSql="select * from "+basedb+".wlyy_doctor where code ='"+surveyScreenResultVo.getDoctor()+"'";
        List<Map<String,Object>> doctorList = jdbcTemplate.queryForList(doctorSql);
        if (doctorList!=null && doctorList.size()>0){
            surveyScreenResultVo.setLevel(Integer.valueOf(String.valueOf(doctorList.get(0).get("level"))));
            surveyScreenResultVo.setDoctorName(String.valueOf(doctorList.get(0).get("name")));
            surveyScreenResultVo.setDoctorMobile(String.valueOf(doctorList.get(0).get("mobile")));
        }
        //获取问卷详情
        String templateSql = "select t.* from "+basedb+".wlyy_survey_templates t where t.code = '" + templateCode + "' and t.del = 1 ";
        List<Map<String, Object>> templateSqlList = jdbcTemplate.queryForList(templateSql);
        if (templateSqlList.size()>0){
            surveyScreenResultVo.setIsThird(templateSqlList.get(0).get("is_third").toString());
        }else {
            surveyScreenResultVo.setIsThird(null);
        }
//        String templateSql = "select t.is_third from "+basedb+".wlyy_survey_templates t where t.code = '" + templateCode + "' and t.del = 1 ";
//        String thirdId = jdbcTemplate.queryForObject(templateSql, String.class);
//        surveyScreenResultVo.setIsThird(thirdId);
        //获取转诊信息
        String reservationSql = "SELECT * FROM " + basedb + ".`wlyy_patient_reservation` r where r.relation_code = '" + code + "' and r.`status` = 1 ";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(reservationSql);
        if (list.size() > 0) {
            Map<String, Object> m = list.get(0);
            surveyScreenResultVo.setDoctorNameReservation(m.get("doctor_name").toString());
            String czrq = m.get("czrq").toString();
            String startTime = m.get("start_time").toString();
            surveyScreenResultVo.setCzrqReservation(czrq.substring(0, czrq.length() - 2));
            surveyScreenResultVo.setStartTimeReservation(startTime.substring(0, startTime.length() - 2));
        }
        map.put("info",surveyScreenResultVo);
        //json.put("info",surveyScreenResultVo);

        //题目和答案
        String questionSql = "select code,title,question_comment questionComment,question_type questionType,template_code templateCode,sort,del from "+basedb+".wlyy_survey_template_questions where template_code='"+templateCode+"' and del=1";
        Map<String,Object> answerMap = new HashMap<>();
        List<SurveyTemplateQuestionsVo> questionList = jdbcTemplate.query(questionSql,new BeanPropertyRowMapper<>(SurveyTemplateQuestionsVo.class));
        String sql = "SELECT soa.*,sto.score,sto.content FROM "+basedb+".wlyy_survey_option_answers soa LEFT JOIN "+basedb+".wlyy_survey_template_options sto ON soa.options_code= sto.code WHERE soa.screen_result_code=? AND soa.patient=? AND soa.survey_code=?";
        List<Map<String,Object>> optionAnswersList = jdbcTemplate.queryForList(sql,new Object[]{code,patientCode,templateCode});
        //问答题答案
        String sqlQuestion = "SELECT sa.* FROM "+basedb+".wlyy_survey_answers sa LEFT JOIN "+basedb+".wlyy_survey_template_questions stq ON sa.question_code = stq.`code` WHERE sa.question_result_code=? AND sa.patient=? AND sa.survey_code=? ";
        List<Map<String, Object>> questionAnswersList = jdbcTemplate.queryForList(sqlQuestion, new Object[]{code, patientCode, templateCode});

        for (SurveyTemplateQuestionsVo surveyTemplateQuestionsVo : questionList){
            Map<String,Object> Qusmap = new HashMap<>();
            Qusmap.put("question",surveyTemplateQuestionsVo);
            String qusCode = surveyTemplateQuestionsVo.getCode();
            /*for (Map<String,Object> option : optionAnswersList){
                if (option.get("question_code").equals(qusCode)){
                    Qusmap.put("option",option);
                }
            }*/
            if(surveyTemplateQuestionsVo.getQuestionType() == 2){//问答题答案放在wlyy_survey_answers
                for (Map<String, Object> question : questionAnswersList) {
                    if (question.get("question_code").equals(qusCode)) {
                        Qusmap.put("option", question);
                    }
                }
            }else if(surveyTemplateQuestionsVo.getQuestionType() == 0){
                for (Map<String, Object> option : optionAnswersList) {
                    if (option.get("question_code").equals(qusCode)) {
                        Qusmap.put("option", option);
                    }
                }
            }else {
                List<Map<String, Object>> options = new ArrayList<>();
                for (Map<String, Object> option : optionAnswersList) {
                    if (option.get("question_code").equals(qusCode)) {
                        options.add(option);
                    }
                }
                Qusmap.put("option", options);
            }
            //map.put(surveyTemplateQuestionsVo.getSort()+"",Qusmap);
            answerMap.put(surveyTemplateQuestionsVo.getSort()+"",Qusmap);
        }
        map.put("answer",answerMap);
        //结果
        String reultSql = "SELECT ssr.screen_result_score,ssr.screen_result,str.advice,ssr.other_advice,ssr.third_advice FROM " + basedb + ".wlyy_survey_screen_result ssr LEFT JOIN " + basedb + ".wlyy_survey_template_result str ON ssr.screen_result_code = str.code WHERE ssr.code='" + code + "'";
        Map<String, Object> resultMap = jdbcTemplate.queryForMap(reultSql);
        if(surveyScreenResultVo.getFollowing() != null) {
            int following = surveyScreenResultVo.getFollowing();
        /*int following = surveyScreenResultVo.getFollowing();
        int order = surveyScreenResultVo.getOrder();
        if (following==1){
            String adviceCodes = surveyScreenResultVo.getAdviceCode();
            List<SurveyAdvice> surveyAdviceList = new ArrayList<>();
            if (StringUtils.isNotEmpty(adviceCodes)){
                String[] advicesStr = adviceCodes.split(",");
                for (String adviceCode : advicesStr){
                    surveyAdviceList.add(surveyAdviceDao.getByCode(adviceCode));
                }
            }
            resultMap.put("advice",surveyAdviceList);
        }*/
            if (following == 1) {
                String adviceCodes = surveyScreenResultVo.getAdviceCode();
                List<Map<String, Object>> surveyAdviceList = new ArrayList<>();
                if (StringUtils.isNotEmpty(adviceCodes)) {
                    String[] advicesStr = adviceCodes.split(",");
                    for (String adviceCode : advicesStr) {
                        String advice = "SELECT * FROM wlyy.wlyy_survey_advice where code='" + adviceCode + "'";
                        surveyAdviceList.addAll(jdbcTemplate.queryForList(advice));
                    }
                }
                resultMap.put("doctorAdvice", surveyAdviceList);
                resultMap.put("doctorOtherAdvice", surveyScreenResultVo.getOtherAdvice());
            }
        }
        map.put("result", resultMap);
        return MixEnvelop.getSuccess(SpecialistMapping.api_success,map);
    }

}
