/*******************************************************************************
 * Copyright (c) 2005, 2014 springside.github.io
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 *******************************************************************************/
package com.yihu.jw.hospital.module.wx.service;

import com.yihu.jw.entity.base.wx.JsApiTicket;
import com.yihu.jw.entity.base.wx.WxAccessTokenDO;
import com.yihu.jw.entity.wechat.WxAccessTokenLog;
import com.yihu.jw.hospital.module.wx.dao.AccessTokenDao;
import com.yihu.jw.hospital.module.wx.dao.WxAccessTokenLogDao;
import com.yihu.jw.wechat.dao.JsapiTicketDao;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springside.modules.utils.Clock;

import java.util.Date;

/**
 * 患者基本信息类.
 *
 * @author George
 */
@Component
@Transactional(rollbackFor = Exception.class)
public class AccessTokenService {

    private AccessTokenDao accessTokenDao;
    private JsapiTicketDao jsapiTicketDao;
    @Autowired
    private WxAccessTokenLogDao wxAccessTokenLogDao;

//    @Value("${wechat.accId}")
    private String accId;

    @Autowired
    public void setJsapiTicketDao(JsapiTicketDao jsapiTicketDao) {

        this.jsapiTicketDao = jsapiTicketDao;
    }

    private Clock clock = Clock.DEFAULT;

	public void setClock(Clock clock) {
		this.clock = clock;
	}

    @Autowired
    public void setAccessTokenDao(AccessTokenDao accessTokenDao) {
        this.accessTokenDao = accessTokenDao;
    }



    public Iterable<WxAccessTokenDO> findAccessToken() {
        return accessTokenDao.findAccessToken(accId);
    }

    public Iterable<JsApiTicket> findJsapiTicket() {
        return jsapiTicketDao.findJsapiTicket(accId);
    }

    /**
     * 添加access_token
     *
     * @param accessToken
     */
    public void addAccessToken(WxAccessTokenDO accessToken) {
        accessToken.setAddTimestamp(System.currentTimeMillis());
        accessToken.setCzrq(clock.getCurrentDate());
        accessToken.setAcc_id(accId);
        accessTokenDao.save(accessToken);
        try {
            //新增获取微信accesstoken日志，排查次数耗尽问题，如果这边次数小于10000 还出现次数耗尽，可以把im的获取改成调i健康的 统一入口
            WxAccessTokenLog log = new WxAccessTokenLog();
            log.setAccid(accessToken.getAcc_id());
            log.setAccessToken(accessToken.getAccessToken());
            log.setCreateTime(new Date());
            log.setExpiresIn(accessToken.getExpiresIn() + "");
            wxAccessTokenLogDao.save(log);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void delAccessToken(WxAccessTokenDO accessToken) {
        accessTokenDao.delete(accessToken);
    }

    public void delAccId(String accId) {
        accessTokenDao.deleteByAccId(accId);
    }

    /**
     * 添加JsapiTicket
     *
     * @param jsApiTicket
     */
    public void addJsapiTicket(JsApiTicket jsApiTicket) {
        jsApiTicket.setAdd_timestamp(System.currentTimeMillis());
        jsApiTicket.setCzrq(clock.getCurrentDate());
        jsApiTicket.setAcc_id(accId);
        jsapiTicketDao.save(jsApiTicket);
    }

    public void delJsapiTicket(JsApiTicket jsApiTicket) {
        jsapiTicketDao.delete(jsApiTicket);
    }

}
