package com.yihu.jw.hospital.utils;


import com.yihu.jw.entity.base.wx.WxAccessTokenDO;
import com.yihu.jw.hospital.module.wx.service.AccessTokenService;
import com.yihu.jw.util.wechat.wxhttp.HttpUtil;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

/**
 * Created by lyr on 2016/08/22.
 */
@Component
public class WeiXinAccessTokenUtils {
    private static Logger logger = LoggerFactory.getLogger(WeiXinAccessTokenUtils.class);
    @Autowired
    private AccessTokenService accessTokenService;
    @Autowired
    StringRedisTemplate redisTemplate;
    @Autowired
    private HttpUtil httpUtil;

    @Value("${wechat.appId}")
    private String appId;
    @Value("${wechat.appSecret}")
    private String appSecret;
    @Value("${wechat.accId}")
    private String accId;
    /**
     * 获取微信AccessToken
     *
     * @return
     */
    public synchronized String getAccessToken(){
        try {
            if (redisTemplate.hasKey("AccessToken:"+accId)){
                String jsonStr = redisTemplate.opsForValue().get("AccessToken:"+accId);
                JSONObject jsonObject = new JSONObject(jsonStr);
                long add_timestamp = jsonObject.getLong("add_timestamp");
                long expires_in = jsonObject.getLong("expires_in");
                String token = jsonObject.getString("access_token");
                if ((System.currentTimeMillis() - add_timestamp) < (expires_in * 1000)) {
                    return token;
                } else {
                    redisTemplate.delete("AccessToken:"+accId);
                }
            }
            String token_url = "https://api.weixin.qq.com/cgi-bin/token";
            String params = "grant_type=client_credential&appid=" +appId + "&secret=" + appSecret;
            String result = httpUtil.sendGet(token_url, params);
            JSONObject json = new JSONObject(result);
            if (json.has("access_token")) {
                String token = json.get("access_token").toString();
                String expires_in = json.get("expires_in").toString();
                WxAccessTokenDO newaccessToken = new WxAccessTokenDO();
                newaccessToken.setAccessToken(token);
                newaccessToken.setExpiresIn(Long.parseLong(expires_in));
                newaccessToken.setAcc_id(accId);
                accessTokenService.addAccessToken(newaccessToken);
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("access_token",token);
                jsonObject.put("add_timestamp",System.currentTimeMillis());
                jsonObject.put("expires_in",Long.parseLong(expires_in));
                redisTemplate.opsForValue().set("AccessToken:"+accId,jsonObject.toString());
                return token;
            } else {
                return null;
            }
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public synchronized String setAccessToken(){
        accessTokenService.delAccId(accId);
        String token_url = "https://api.weixin.qq.com/cgi-bin/token";
        String params = "grant_type=client_credential&appid=" +appId + "&secret=" + appSecret;
        String result = httpUtil.sendGet(token_url, params);
        JSONObject json = new JSONObject(result);
        if (json.has("access_token")) {
            String token = json.get("access_token").toString();
            String expires_in = json.get("expires_in").toString();
            WxAccessTokenDO newaccessToken = new WxAccessTokenDO();
            newaccessToken.setAccessToken(token);
            newaccessToken.setExpiresIn(Long.parseLong(expires_in));
            newaccessToken.setAcc_id(accId);
            accessTokenService.addAccessToken(newaccessToken);
            if (redisTemplate.hasKey("AccessToken:"+accId)){
                String jsonStr = redisTemplate.opsForValue().get("AccessToken:"+accId);
                JSONObject jsonObject = new JSONObject(jsonStr);
                jsonObject.put("access_token",token);
                jsonObject.put("add_timestamp",System.currentTimeMillis());
                jsonObject.put("expires_in",Long.parseLong(expires_in));
                redisTemplate.opsForValue().set("AccessToken:"+accId,jsonObject.toString());
            }else {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("access_token",token);
                jsonObject.put("add_timestamp",System.currentTimeMillis());
                jsonObject.put("expires_in",Long.parseLong(expires_in));
                redisTemplate.opsForValue().set("AccessToken:"+accId,jsonObject.toString());
            }
            return token;
        } else {
            //token失败原因
            logger.info(result);
            return null;
        }
    }

    /**
     * 获取微信AccessToken
     *
     * @return
     */
    public synchronized JSONObject getAccessTokenAndExpire(){
        try {
            if (redisTemplate.hasKey("AccessToken:"+accId)){
                String jsonStr = redisTemplate.opsForValue().get("AccessToken:"+accId);
                JSONObject jsonObject = new JSONObject(jsonStr);
                long add_timestamp = jsonObject.getLong("add_timestamp");
                long expires_in = jsonObject.getLong("expires_in");
                String token = jsonObject.getString("access_token");
                if ((System.currentTimeMillis() - add_timestamp) < (expires_in * 1000)) {
                    return jsonObject;
                } else {
                    redisTemplate.delete("AccessToken:"+accId);
                }
            }
            String token_url = "https://api.weixin.qq.com/cgi-bin/token";
            String params = "grant_type=client_credential&appid=" +appId + "&secret=" + appSecret;
            String result = httpUtil.sendGet(token_url, params);
            JSONObject json = new JSONObject(result);
            if (json.has("access_token")) {
                String token = json.get("access_token").toString();
                String expires_in = json.get("expires_in").toString();
                WxAccessTokenDO newaccessToken = new WxAccessTokenDO();
                newaccessToken.setAccessToken(token);
                newaccessToken.setExpiresIn(Long.parseLong(expires_in));
                newaccessToken.setAcc_id(accId);
                accessTokenService.addAccessToken(newaccessToken);
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("access_token",token);
                jsonObject.put("add_timestamp",System.currentTimeMillis());
                jsonObject.put("expires_in",Long.parseLong(expires_in));
                redisTemplate.opsForValue().set("AccessToken:"+accId,jsonObject.toString());
                return jsonObject;
            } else {
                return null;
            }
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }
}
